// Created on: 1991-05-28
// Created by: Arnaud BOUZY
// Copyright (c) 1991-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#include <Expr.hxx>
#include <Expr_Division.hxx>
#include <Expr_Exponential.hxx>
#include <Expr_GeneralExpression.hxx>
#include <Expr_LogOfe.hxx>
#include <Expr_NamedUnknown.hxx>
#include <Expr_Operators.hxx>
#include <Standard_Type.hxx>
#include <TCollection_AsciiString.hxx>

IMPLEMENT_STANDARD_RTTIEXT(Expr_LogOfe, Expr_UnaryExpression)

Expr_LogOfe::Expr_LogOfe(const Handle(Expr_GeneralExpression)& exp)
{
  CreateOperand(exp);
}

Handle(Expr_GeneralExpression) Expr_LogOfe::ShallowSimplified() const
{
  Handle(Expr_GeneralExpression) myexp = Operand();
  if (myexp->IsKind(STANDARD_TYPE(Expr_NumericValue)))
  {
    Handle(Expr_NumericValue) myNVexp = Handle(Expr_NumericValue)::DownCast(myexp);
    return new Expr_NumericValue(Log(myNVexp->GetValue()));
  }
  if (myexp->IsKind(STANDARD_TYPE(Expr_Exponential)))
  {
    return myexp->SubExpression(1);
  }
  Handle(Expr_LogOfe) me = this;
  return me;
}

Handle(Expr_GeneralExpression) Expr_LogOfe::Copy() const
{
  return new Expr_LogOfe(Expr::CopyShare(Operand()));
}

Standard_Boolean Expr_LogOfe::IsIdentical(const Handle(Expr_GeneralExpression)& Other) const
{
  if (Other->IsKind(STANDARD_TYPE(Expr_LogOfe)))
  {
    Handle(Expr_GeneralExpression) myexp = Operand();
    return myexp->IsIdentical(Other->SubExpression(1));
  }
  return Standard_False;
}

Standard_Boolean Expr_LogOfe::IsLinear() const
{
  return !ContainsUnknowns();
}

Handle(Expr_GeneralExpression) Expr_LogOfe::Derivative(const Handle(Expr_NamedUnknown)& X) const
{
  if (!Contains(X))
  {
    return new Expr_NumericValue(0.0);
  }
  Handle(Expr_GeneralExpression) myexp  = Operand();
  Handle(Expr_GeneralExpression) myder  = myexp->Derivative(X);
  Handle(Expr_Division)          thediv = myder / Expr::CopyShare(myexp);
  return thediv->ShallowSimplified();
}

Standard_Real Expr_LogOfe::Evaluate(const Expr_Array1OfNamedUnknown& vars,
                                    const TColStd_Array1OfReal&      vals) const
{
  return ::Log(Operand()->Evaluate(vars, vals));
}

TCollection_AsciiString Expr_LogOfe::String() const
{
  TCollection_AsciiString str("Ln(");
  str += Operand()->String();
  str += ")";
  return str;
}
