// Created on: 1996-12-02
// Created by: Laurent PAINNOT
// Copyright (c) 1996-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef OCCT_DEBUG
  #define No_Standard_OutOfRange
  #define No_Standard_RangeError
#endif

#include <AppCont_ContMatrices.hxx>
#include <math_Matrix.hxx>
#include <Standard_DimensionError.hxx>

static const Standard_Real VBMatrix[] = {0.5,
                                         0.5,

                                         0.788675134594813,
                                         0.211324865405187,
                                         0.211324865405187,
                                         0.788675134594813,

                                         0.887298334620741,
                                         0.5,
                                         0.112701665379259,
                                         0.112701665379259,
                                         0.5,
                                         0.887298334620741,

                                         0.930568155797026,
                                         0.669990521792428,
                                         0.330009478207572,
                                         0.0694318442029735,
                                         0.0694318442029735,
                                         0.330009478207572,
                                         0.669990521792428,
                                         0.930568155797026,

                                         0.953089922969332,
                                         0.769234655052841,
                                         0.5,
                                         0.230765344947159,
                                         0.0469100770306681,
                                         0.046910077030668,
                                         0.230765344947159,
                                         0.5,
                                         0.769234655052841,
                                         0.953089922969332,

                                         0.966234757101576,
                                         0.830604693233133,
                                         0.619309593041598,
                                         0.380690406958402,
                                         0.169395306766867,
                                         0.033765242898424,
                                         0.033765242898424,
                                         0.169395306766867,
                                         0.380690406958402,
                                         0.619309593041598,
                                         0.830604693233133,
                                         0.966234757101576,

                                         0.974553956171379,
                                         0.870765592799697,
                                         0.702922575688699,
                                         0.5,
                                         0.297077424311301,
                                         0.129234407200303,
                                         0.025446043828621,
                                         0.025446043828621,
                                         0.129234407200303,
                                         0.297077424311301,
                                         0.5,
                                         0.702922575688699,
                                         0.870765592799697,
                                         0.974553956171379,

                                         0.980144928248768,
                                         0.898333238706813,
                                         0.762766204958164,
                                         0.591717321247825,
                                         0.408282678752175,
                                         0.237233795041836,
                                         0.101666761293187,
                                         0.019855071751232,
                                         0.019855071751232,
                                         0.101666761293187,
                                         0.237233795041836,
                                         0.408282678752175,
                                         0.591717321247825,
                                         0.762766204958164,
                                         0.898333238706813,
                                         0.980144928248768,

                                         0.984080119753813,
                                         0.918015553663318,
                                         0.806685716350295,
                                         0.662126711701905,
                                         0.5,
                                         0.337873288298095,
                                         0.193314283649705,
                                         0.081984446336682,
                                         0.0159198802461871,
                                         0.015919880246187,
                                         0.081984446336682,
                                         0.193314283649705,
                                         0.337873288298095,
                                         0.5,
                                         0.662126711701905,
                                         0.806685716350295,
                                         0.918015553663318,
                                         0.984080119753813,

                                         0.986953264258586,
                                         0.932531683344493,
                                         0.839704784149512,
                                         0.716697697064623,
                                         0.574437169490815,
                                         0.425562830509185,
                                         0.283302302935377,
                                         0.160295215850488,
                                         0.0674683166555075,
                                         0.013046735741414,
                                         0.013046735741414,
                                         0.0674683166555075,
                                         0.160295215850488,
                                         0.283302302935377,
                                         0.425562830509185,
                                         0.574437169490815,
                                         0.716697697064623,
                                         0.839704784149512,
                                         0.932531683344493,
                                         0.986953264258586,

                                         0.989114329073028,
                                         0.943531299884047,
                                         0.865076002787025,
                                         0.759548064603406,
                                         0.634771577976172,
                                         0.5,
                                         0.365228422023828,
                                         0.240451935396594,
                                         0.134923997212975,
                                         0.0564687001159525,
                                         0.0108856709269716,
                                         0.0108856709269715,
                                         0.0564687001159525,
                                         0.134923997212975,
                                         0.240451935396594,
                                         0.365228422023827,
                                         0.5,
                                         0.634771577976172,
                                         0.759548064603406,
                                         0.865076002787025,
                                         0.943531299884047,
                                         0.989114329073028,

                                         0.99078031712336,
                                         0.952058628185237,
                                         0.884951337097152,
                                         0.793658977143308,
                                         0.68391574949909,
                                         0.562616704255735,
                                         0.437383295744265,
                                         0.31608425050091,
                                         0.206341022856692,
                                         0.115048662902848,
                                         0.0479413718147625,
                                         0.00921968287664043,
                                         0.00921968287664049,
                                         0.0479413718147625,
                                         0.115048662902847,
                                         0.206341022856691,
                                         0.31608425050091,
                                         0.437383295744265,
                                         0.562616704255735,
                                         0.68391574949909,
                                         0.793658977143308,
                                         0.884951337097152,
                                         0.952058628185237,
                                         0.99078031712336,

                                         0.992091527359294,
                                         0.958799199611489,
                                         0.900789045366655,
                                         0.82117466972017,
                                         0.724246375518223,
                                         0.615229157977567,
                                         0.5,
                                         0.384770842022433,
                                         0.275753624481777,
                                         0.17882533027983,
                                         0.0992109546333451,
                                         0.0412008003885109,
                                         0.00790847264070593,
                                         0.00790847264070599,
                                         0.041200800388511,
                                         0.099210954633345,
                                         0.17882533027983,
                                         0.275753624481777,
                                         0.384770842022433,
                                         0.5,
                                         0.615229157977567,
                                         0.724246375518223,
                                         0.82117466972017,
                                         0.900789045366655,
                                         0.958799199611489,
                                         0.992091527359294,

                                         0.993141904348406,
                                         0.964217441831787,
                                         0.913600657534883,
                                         0.843646452405842,
                                         0.757624318179077,
                                         0.659556184463945,
                                         0.554027474353672,
                                         0.445972525646328,
                                         0.340443815536055,
                                         0.242375681820923,
                                         0.156353547594158,
                                         0.0863993424651175,
                                         0.035782558168213,
                                         0.00685809565159401,
                                         0.00685809565159401,
                                         0.035782558168213,
                                         0.0863993424651175,
                                         0.156353547594157,
                                         0.242375681820923,
                                         0.340443815536055,
                                         0.445972525646328,
                                         0.554027474353672,
                                         0.659556184463945,
                                         0.757624318179077,
                                         0.843646452405842,
                                         0.913600657534883,
                                         0.964217441831787,
                                         0.993141904348406,

                                         0.993996259010242,
                                         0.968636696200353,
                                         0.924103291705213,
                                         0.862208865680085,
                                         0.78548608630427,
                                         0.697075673538782,
                                         0.600597046998718,
                                         0.5,
                                         0.399402953001282,
                                         0.302924326461218,
                                         0.21451391369573,
                                         0.137791134319915,
                                         0.0758967082947866,
                                         0.031363303799647,
                                         0.00600374098975753,
                                         0.00600374098975748,
                                         0.031363303799647,
                                         0.0758967082947865,
                                         0.137791134319915,
                                         0.21451391369573,
                                         0.302924326461218,
                                         0.399402953001282,
                                         0.5,
                                         0.600597046998718,
                                         0.697075673538782,
                                         0.78548608630427,
                                         0.862208865680085,
                                         0.924103291705213,
                                         0.968636696200353,
                                         0.993996259010242,

                                         0.994700467495825,
                                         0.972287511536617,
                                         0.932815601193916,
                                         0.877702204177502,
                                         0.808938122201322,
                                         0.729008388828614,
                                         0.64080177538963,
                                         0.547506254918819,
                                         0.452493745081181,
                                         0.35919822461037,
                                         0.270991611171386,
                                         0.191061877798678,
                                         0.122297795822498,
                                         0.0671843988060841,
                                         0.0277124884633835,
                                         0.00529953250417492,
                                         0.00529953250417498,
                                         0.0277124884633835,
                                         0.067184398806084,
                                         0.122297795822499,
                                         0.191061877798678,
                                         0.270991611171386,
                                         0.35919822461037,
                                         0.452493745081181,
                                         0.547506254918819,
                                         0.64080177538963,
                                         0.729008388828614,
                                         0.808938122201322,
                                         0.877702204177502,
                                         0.932815601193916,
                                         0.972287511536617,
                                         0.994700467495825,

                                         0.995287737657208,
                                         0.975337760884384,
                                         0.940119576863493,
                                         0.890757001948401,
                                         0.828835579608346,
                                         0.756345268543239,
                                         0.675615881726938,
                                         0.589242090747924,
                                         0.5,
                                         0.410757909252076,
                                         0.324384118273062,
                                         0.243654731456761,
                                         0.171164420391654,
                                         0.109242998051599,
                                         0.059880423136507,
                                         0.0246622391156159,
                                         0.00471226234279154,
                                         0.00471226234279148,
                                         0.024662239115616,
                                         0.059880423136507,
                                         0.109242998051599,
                                         0.171164420391655,
                                         0.243654731456761,
                                         0.324384118273062,
                                         0.410757909252076,
                                         0.5,
                                         0.589242090747924,
                                         0.675615881726938,
                                         0.756345268543239,
                                         0.828835579608346,
                                         0.890757001948401,
                                         0.940119576863493,
                                         0.975337760884384,
                                         0.995287737657208,

                                         0.995782584210466,
                                         0.977911974785699,
                                         0.946301233248778,
                                         0.901852479486261,
                                         0.845843521530177,
                                         0.779885415536974,
                                         0.705875580731422,
                                         0.625943112845753,
                                         0.542387506520868,
                                         0.457612493479132,
                                         0.374056887154247,
                                         0.294124419268578,
                                         0.220114584463026,
                                         0.154156478469823,
                                         0.0981475205137385,
                                         0.053698766751222,
                                         0.022088025214301,
                                         0.0042174157895345,
                                         0.0042174157895345,
                                         0.022088025214301,
                                         0.053698766751222,
                                         0.0981475205137385,
                                         0.154156478469823,
                                         0.220114584463026,
                                         0.294124419268578,
                                         0.374056887154247,
                                         0.457612493479132,
                                         0.542387506520868,
                                         0.625943112845753,
                                         0.705875580731422,
                                         0.779885415536974,
                                         0.845843521530177,
                                         0.901852479486261,
                                         0.946301233248778,
                                         0.977911974785699,
                                         0.995782584210466,

                                         0.996203421921792,
                                         0.980104076067415,
                                         0.951577951807409,
                                         0.911357328268572,
                                         0.860483088667614,
                                         0.800272652330841,
                                         0.73228537068798,
                                         0.658282049981815,
                                         0.580179322820113,
                                         0.5,
                                         0.419820677179887,
                                         0.341717950018185,
                                         0.26771462931202,
                                         0.199727347669159,
                                         0.139516911332386,
                                         0.0886426717314285,
                                         0.048422048192591,
                                         0.019895923932585,
                                         0.00379657807820799,
                                         0.00379657807820799,
                                         0.019895923932585,
                                         0.048422048192591,
                                         0.0886426717314285,
                                         0.139516911332385,
                                         0.19972734766916,
                                         0.26771462931202,
                                         0.341717950018185,
                                         0.419820677179887,
                                         0.5,
                                         0.580179322820113,
                                         0.658282049981815,
                                         0.73228537068798,
                                         0.800272652330841,
                                         0.860483088667614,
                                         0.911357328268572,
                                         0.951577951807409,
                                         0.980104076067415,
                                         0.996203421921792,

                                         0.996564299592547,
                                         0.981985963638957,
                                         0.956117214125663,
                                         0.919558485911109,
                                         0.873165953230076,
                                         0.818026840363258,
                                         0.755433500975414,
                                         0.68685304435771,
                                         0.613892925570823,
                                         0.538263260566749,
                                         0.461736739433251,
                                         0.386107074429177,
                                         0.31314695564229,
                                         0.244566499024586,
                                         0.181973159636742,
                                         0.126834046769924,
                                         0.0804415140888906,
                                         0.043882785874337,
                                         0.018014036361043,
                                         0.00343570040745256,
                                         0.0034357004074525,
                                         0.018014036361043,
                                         0.043882785874337,
                                         0.0804415140888905,
                                         0.126834046769924,
                                         0.181973159636742,
                                         0.244566499024586,
                                         0.31314695564229,
                                         0.386107074429177,
                                         0.461736739433251,
                                         0.538263260566749,
                                         0.613892925570823,
                                         0.68685304435771,
                                         0.755433500975414,
                                         0.818026840363258,
                                         0.873165953230076,
                                         0.919558485911109,
                                         0.956117214125663,
                                         0.981985963638957,
                                         0.996564299592547,

                                         0.996876085310195,
                                         0.983613419283153,
                                         0.960049667075201,
                                         0.926681682291658,
                                         0.884219981737839,
                                         0.833569402098706,
                                         0.77580941794361,
                                         0.712171060103719,
                                         0.644010658401201,
                                         0.572780927080447,
                                         0.5,
                                         0.427219072919553,
                                         0.355989341598799,
                                         0.287828939896281,
                                         0.22419058205639,
                                         0.166430597901294,
                                         0.115780018262161,
                                         0.0733183177083415,
                                         0.0399503329247994,
                                         0.0163865807168471,
                                         0.00312391468980544,
                                         0.0031239146898055,
                                         0.016386580716847,
                                         0.0399503329247995,
                                         0.0733183177083415,
                                         0.115780018262161,
                                         0.166430597901294,
                                         0.22419058205639,
                                         0.287828939896281,
                                         0.355989341598799,
                                         0.427219072919553,
                                         0.5,
                                         0.572780927080447,
                                         0.644010658401201,
                                         0.712171060103719,
                                         0.77580941794361,
                                         0.833569402098706,
                                         0.884219981737839,
                                         0.926681682291658,
                                         0.960049667075201,
                                         0.983613419283153,
                                         0.996876085310195,

                                         0.9971472927412,
                                         0.985030248917715,
                                         0.963478386093587,
                                         0.93290628886015,
                                         0.893908402989604,
                                         0.847243631593341,
                                         0.793820201753456,
                                         0.734677918993379,
                                         0.670967910446042,
                                         0.60393021334411,
                                         0.534869636659861,
                                         0.465130363340139,
                                         0.39606978665589,
                                         0.329032089553958,
                                         0.265322081006621,
                                         0.206179798246544,
                                         0.152756368406659,
                                         0.106091597010396,
                                         0.06709371113985,
                                         0.0365216139064131,
                                         0.0149697510822855,
                                         0.00285270725880049,
                                         0.00285270725880049,
                                         0.0149697510822855,
                                         0.036521613906413,
                                         0.06709371113985,
                                         0.106091597010396,
                                         0.152756368406659,
                                         0.206179798246544,
                                         0.265322081006621,
                                         0.329032089553958,
                                         0.396069786655889,
                                         0.465130363340139,
                                         0.534869636659861,
                                         0.60393021334411,
                                         0.670967910446042,
                                         0.734677918993379,
                                         0.793820201753456,
                                         0.847243631593341,
                                         0.893908402989604,
                                         0.93290628886015,
                                         0.963478386093587,
                                         0.985030248917715,
                                         0.9971472927412,

                                         0.997384667498776,
                                         0.986271235609057,
                                         0.966485543413008,
                                         0.938376179135221,
                                         0.90244420080942,
                                         0.859330681565975,
                                         0.809804937881823,
                                         0.754750738923004,
                                         0.695150519015145,
                                         0.632067840485173,
                                         0.566628412149233,
                                         0.5,
                                         0.433371587850767,
                                         0.367932159514827,
                                         0.304849480984855,
                                         0.245249261076996,
                                         0.190195062118177,
                                         0.140669318434025,
                                         0.09755579919058,
                                         0.061623820864779,
                                         0.033514456586992,
                                         0.0137287643909425,
                                         0.00261533250122392,
                                         0.00261533250122398,
                                         0.0137287643909425,
                                         0.033514456586992,
                                         0.061623820864779,
                                         0.09755579919058,
                                         0.140669318434025,
                                         0.190195062118177,
                                         0.245249261076996,
                                         0.304849480984855,
                                         0.367932159514827,
                                         0.433371587850767,
                                         0.5,
                                         0.566628412149233,
                                         0.632067840485173,
                                         0.695150519015145,
                                         0.754750738923004,
                                         0.809804937881823,
                                         0.859330681565975,
                                         0.90244420080942,
                                         0.938376179135221,
                                         0.966485543413008,
                                         0.986271235609057,
                                         0.997384667498776,

                                         0.997593609998511,
                                         0.987364277985655,
                                         0.969137276001367,
                                         0.9432077635022,
                                         0.910000992986951,
                                         0.870062095789277,
                                         0.824046825968488,
                                         0.77271073569442,
                                         0.716896753813023,
                                         0.657521339848081,
                                         0.595559433736808,
                                         0.532028446431303,
                                         0.467971553568697,
                                         0.404440566263192,
                                         0.342478660151919,
                                         0.283103246186977,
                                         0.22728926430558,
                                         0.175953174031512,
                                         0.129937904210723,
                                         0.0899990070130485,
                                         0.0567922364977995,
                                         0.0308627239986334,
                                         0.0126357220143455,
                                         0.00240639000148946,
                                         0.00240639000148951,
                                         0.0126357220143455,
                                         0.0308627239986335,
                                         0.0567922364977995,
                                         0.0899990070130485,
                                         0.129937904210723,
                                         0.175953174031512,
                                         0.22728926430558,
                                         0.283103246186977,
                                         0.342478660151919,
                                         0.404440566263192,
                                         0.467971553568697,
                                         0.532028446431303,
                                         0.595559433736808,
                                         0.657521339848081,
                                         0.716896753813023,
                                         0.77271073569442,
                                         0.824046825968488,
                                         0.870062095789277,
                                         0.910000992986951,
                                         0.9432077635022,
                                         0.969137276001367,
                                         0.987364277985655,
                                         0.997593609998511,

                                         0.25,
                                         0.5,
                                         0.25,

                                         0.622008467928146,
                                         0.0446581987385204,
                                         0.333333333333333,
                                         0.333333333333333,
                                         0.0446581987385204,
                                         0.622008467928146,

                                         0.787298334620741,
                                         0.25,
                                         0.0127016653792584,
                                         0.2,
                                         0.5,
                                         0.2,
                                         0.0127016653792584,
                                         0.25,
                                         0.787298334620741,

                                         0.865957092583479,
                                         0.44888729929169,
                                         0.108906255706834,
                                         0.00482078098942599,
                                         0.129222126427095,
                                         0.442206445001476,
                                         0.442206445001476,
                                         0.129222126427095,
                                         0.00482078098942598,
                                         0.108906255706834,
                                         0.44888729929169,
                                         0.865957092583479,

                                         0.908380401265687,
                                         0.591721954534264,
                                         0.25,
                                         0.0532526444285811,
                                         0.00220055532702321,
                                         0.0894190434072896,
                                         0.355025401037155,
                                         0.5,
                                         0.355025401037155,
                                         0.0894190434072897,
                                         0.00220055532702321,
                                         0.0532526444285811,
                                         0.25,
                                         0.591721954534264,
                                         0.908380401265687,

                                         0.933609605831142,
                                         0.689904156420906,
                                         0.38354437203335,
                                         0.144925185950153,
                                         0.0286947699546411,
                                         0.00114009162798957,
                                         0.0652503025408688,
                                         0.281401073624453,
                                         0.471530442016496,
                                         0.471530442016496,
                                         0.281401073624453,
                                         0.0652503025408688,
                                         0.00114009162798957,
                                         0.0286947699546411,
                                         0.144925185950153,
                                         0.38354437203335,
                                         0.689904156420906,
                                         0.933609605831142,

                                         0.949755413489286,
                                         0.758232717603808,
                                         0.494100147412834,
                                         0.25,
                                         0.088254996035437,
                                         0.0167015320044137,
                                         0.000647501146528103,
                                         0.0495970853641858,
                                         0.225065750391779,
                                         0.417644856551729,
                                         0.5,
                                         0.417644856551729,
                                         0.225065750391779,
                                         0.0495970853641859,
                                         0.0006475011465281,
                                         0.0167015320044137,
                                         0.088254996035437,
                                         0.25,
                                         0.494100147412834,
                                         0.758232717603808,
                                         0.949755413489286,

                                         0.960684080371783,
                                         0.807002607765473,
                                         0.581812283426281,
                                         0.350129388264702,
                                         0.166694745769052,
                                         0.0562798735099516,
                                         0.0103361303518458,
                                         0.000394223874246572,
                                         0.0389216957539709,
                                         0.182661261882682,
                                         0.361907843063768,
                                         0.483175865966247,
                                         0.483175865966247,
                                         0.361907843063768,
                                         0.182661261882682,
                                         0.0389216957539709,
                                         0.000394223874246572,
                                         0.0103361303518458,
                                         0.0562798735099516,
                                         0.166694745769052,
                                         0.350129388264702,
                                         0.581812283426281,
                                         0.807002607765473,
                                         0.960684080371783,

                                         0.968413682094679,
                                         0.842752556767768,
                                         0.650741844963589,
                                         0.438411782349177,
                                         0.25,
                                         0.114158358945368,
                                         0.0373704122629986,
                                         0.00672144944113229,
                                         0.000253442587052937,
                                         0.0313328753182682,
                                         0.150525993791099,
                                         0.311887742773413,
                                         0.447429858705455,
                                         0.5,
                                         0.447429858705455,
                                         0.311887742773413,
                                         0.150525993791099,
                                         0.0313328753182683,
                                         0.000253442587052935,
                                         0.00672144944113229,
                                         0.0373704122629986,
                                         0.114158358945368,
                                         0.25,
                                         0.438411782349177,
                                         0.650741844963589,
                                         0.842752556767768,
                                         0.968413682094679,

                                         0.974076745830678,
                                         0.869615340441313,
                                         0.705104124523579,
                                         0.513655588977735,
                                         0.32997806169262,
                                         0.181103722710989,
                                         0.0802601948484878,
                                         0.0256945562245545,
                                         0.00455197375232782,
                                         0.00017021731350629,
                                         0.0257530368558155,
                                         0.125832685806359,
                                         0.269201319251867,
                                         0.406084216173777,
                                         0.488918215596391,
                                         0.488918215596391,
                                         0.406084216173777,
                                         0.269201319251867,
                                         0.125832685806359,
                                         0.0257530368558155,
                                         0.00017021731350629,
                                         0.00455197375232783,
                                         0.0256945562245545,
                                         0.0802601948484878,
                                         0.181103722710989,
                                         0.32997806169262,
                                         0.513655588977735,
                                         0.705104124523579,
                                         0.869615340441313,
                                         0.974076745830678,

                                         0.978347155977587,
                                         0.89025131386088,
                                         0.748356490597976,
                                         0.57691326244278,
                                         0.40293495620636,
                                         0.25,
                                         0.133391800254015,
                                         0.0578171332359678,
                                         0.018204485023927,
                                         0.00318871409278537,
                                         0.000118497831530314,
                                         0.0215343461908824,
                                         0.106559972046334,
                                         0.233439024378097,
                                         0.365269604321252,
                                         0.463673243539625,
                                         0.5,
                                         0.463673243539625,
                                         0.365269604321252,
                                         0.233439024378097,
                                         0.106559972046334,
                                         0.0215343461908825,
                                         0.000118497831530313,
                                         0.00318871409278537,
                                         0.018204485023927,
                                         0.0578171332359678,
                                         0.133391800254015,
                                         0.25,
                                         0.40293495620636,
                                         0.57691326244278,
                                         0.748356490597976,
                                         0.89025131386088,
                                         0.978347155977587,

                                         0.981645636799065,
                                         0.906415631501956,
                                         0.783138869030038,
                                         0.629894572000163,
                                         0.467740752412902,
                                         0.316537555907585,
                                         0.191304147396116,
                                         0.0999092534147221,
                                         0.0425766177135457,
                                         0.013236194835733,
                                         0.00229837513148131,
                                         8.50025523458168e-05,
                                         0.0182693606485893,
                                         0.0912859933665624,
                                         0.203624936134229,
                                         0.327528810286292,
                                         0.432349994172376,
                                         0.4921582966963,
                                         0.4921582966963,
                                         0.432349994172376,
                                         0.327528810286292,
                                         0.203624936134229,
                                         0.0912859933665625,
                                         0.0182693606485892,
                                         8.50025523458178e-05,
                                         0.0022983751314813,
                                         0.013236194835733,
                                         0.0425766177135457,
                                         0.099909253414722,
                                         0.191304147396116,
                                         0.316537555907585,
                                         0.467740752412902,
                                         0.629894572000163,
                                         0.783138869030038,
                                         0.906415631501956,
                                         0.981645636799065,

                                         0.984245598658097,
                                         0.919295905175632,
                                         0.81142090425257,
                                         0.67432783819003,
                                         0.524532812451284,
                                         0.378506916825787,
                                         0.25,
                                         0.148048600870652,
                                         0.0760400614148366,
                                         0.0319784987496903,
                                         0.00984281351925965,
                                         0.00169750595265392,
                                         6.25439395087943e-05,
                                         0.0156918574023944,
                                         0.0790065888717141,
                                         0.178736282228171,
                                         0.293693663060279,
                                         0.39942712613388,
                                         0.473444482303562,
                                         0.5,
                                         0.473444482303562,
                                         0.39942712613388,
                                         0.293693663060279,
                                         0.178736282228171,
                                         0.079006588871714,
                                         0.0156918574023943,
                                         6.25439395087951e-05,
                                         0.00169750595265393,
                                         0.00984281351925964,
                                         0.0319784987496903,
                                         0.0760400614148366,
                                         0.148048600870652,
                                         0.25,
                                         0.378506916825787,
                                         0.524532812451284,
                                         0.67432783819003,
                                         0.81142090425257,
                                         0.919295905175632,
                                         0.984245598658097,

                                         0.986330842172778,
                                         0.929715275132636,
                                         0.83466616144817,
                                         0.711739336656963,
                                         0.573994607496311,
                                         0.435014360464637,
                                         0.306946442338709,
                                         0.198891493631365,
                                         0.115901991536747,
                                         0.0587459711381573,
                                         0.0244464318452785,
                                         0.00746484637840465,
                                         0.00128039146906154,
                                         4.70334759664127e-05,
                                         0.0136221243512552,
                                         0.0690043333983029,
                                         0.157868992173426,
                                         0.263814231497758,
                                         0.367259421365531,
                                         0.449083647998615,
                                         0.494162064029927,
                                         0.494162064029927,
                                         0.449083647998615,
                                         0.367259421365531,
                                         0.263814231497758,
                                         0.157868992173426,
                                         0.0690043333983028,
                                         0.0136221243512552,
                                         4.70334759664127e-05,
                                         0.00128039146906155,
                                         0.00746484637840465,
                                         0.0244464318452785,
                                         0.0587459711381573,
                                         0.115901991536747,
                                         0.198891493631365,
                                         0.306946442338709,
                                         0.435014360464637,
                                         0.573994607496311,
                                         0.711739336656963,
                                         0.83466616144817,
                                         0.929715275132636,
                                         0.986330842172778,

                                         0.988028562926357,
                                         0.938257049225935,
                                         0.853966893740411,
                                         0.743404128057339,
                                         0.616988391777598,
                                         0.485914494639546,
                                         0.36071681286358,
                                         0.25,
                                         0.159522718866145,
                                         0.0917631475619829,
                                         0.0460162191690593,
                                         0.0189863966971688,
                                         0.00576031032998392,
                                         0.000983656825228953,
                                         3.60449058720948e-05,
                                         0.0119353921677708,
                                         0.0607592939488361,
                                         0.140272795929605,
                                         0.237609475245492,
                                         0.336995389053342,
                                         0.422322357798471,
                                         0.479760468270276,
                                         0.5,
                                         0.479760468270276,
                                         0.422322357798471,
                                         0.336995389053342,
                                         0.237609475245492,
                                         0.140272795929605,
                                         0.0607592939488361,
                                         0.0119353921677709,
                                         3.60449058720941e-05,
                                         0.000983656825228953,
                                         0.00576031032998392,
                                         0.0189863966971689,
                                         0.0460162191690593,
                                         0.0917631475619829,
                                         0.159522718866145,
                                         0.25,
                                         0.36071681286358,
                                         0.485914494639546,
                                         0.616988391777598,
                                         0.743404128057339,
                                         0.853966893740411,
                                         0.938257049225935,
                                         0.988028562926357,

                                         0.989429020036413,
                                         0.945343005090066,
                                         0.870144945830767,
                                         0.770361159218045,
                                         0.654380885550601,
                                         0.531453230982491,
                                         0.410626915342501,
                                         0.299763099175231,
                                         0.204750589337593,
                                         0.129023364563242,
                                         0.0734364533252639,
                                         0.036504641147957,
                                         0.0149567508630415,
                                         0.00451374344293495,
                                         0.000767982016833162,
                                         2.80850447628065e-05,
                                         0.0105428949188243,
                                         0.0538890128931006,
                                         0.125341310726298,
                                         0.214682089918914,
                                         0.309114473301442,
                                         0.395110315692245,
                                         0.460349720094257,
                                         0.495486311487176,
                                         0.495486311487176,
                                         0.460349720094257,
                                         0.395110315692245,
                                         0.309114473301442,
                                         0.214682089918914,
                                         0.125341310726298,
                                         0.0538890128931006,
                                         0.0105428949188242,
                                         2.80850447628071e-05,
                                         0.000767982016833162,
                                         0.00451374344293494,
                                         0.0149567508630415,
                                         0.036504641147957,
                                         0.0734364533252639,
                                         0.129023364563242,
                                         0.204750589337593,
                                         0.29976309917523,
                                         0.410626915342501,
                                         0.531453230982491,
                                         0.654380885550601,
                                         0.770361159218045,
                                         0.870144945830767,
                                         0.945343005090066,
                                         0.989429020036413,

                                         0.990597680730804,
                                         0.951283747806964,
                                         0.883824818801993,
                                         0.793448036520103,
                                         0.686968418024702,
                                         0.572058165247744,
                                         0.456456819641668,
                                         0.347206241508985,
                                         0.25,
                                         0.168722060013137,
                                         0.105225056187792,
                                         0.0593676281612665,
                                         0.029297258808011,
                                         0.0119340326233018,
                                         0.00358566507500712,
                                         0.000608226038195817,
                                         2.22054163872912e-05,
                                         0.00938011385280839,
                                         0.0481080261548403,
                                         0.112589516123,
                                         0.194617930856595,
                                         0.283734323167287,
                                         0.36857420659099,
                                         0.43831812417054,
                                         0.484071698477879,
                                         0.5,
                                         0.484071698477879,
                                         0.43831812417054,
                                         0.36857420659099,
                                         0.283734323167287,
                                         0.194617930856595,
                                         0.112589516123,
                                         0.0481080261548402,
                                         0.0093801138528085,
                                         2.22054163872907e-05,
                                         0.000608226038195819,
                                         0.00358566507500712,
                                         0.0119340326233018,
                                         0.029297258808011,
                                         0.0593676281612665,
                                         0.105225056187792,
                                         0.168722060013137,
                                         0.25,
                                         0.347206241508985,
                                         0.456456819641668,
                                         0.572058165247744,
                                         0.686968418024702,
                                         0.793448036520103,
                                         0.883824818801993,
                                         0.951283747806964,
                                         0.990597680730804,

                                         0.991582955016873,
                                         0.956311830429266,
                                         0.895486024048158,
                                         0.813337894755518,
                                         0.71545126291457,
                                         0.608221261367279,
                                         0.498260335472922,
                                         0.391804780519031,
                                         0.294184207229924,
                                         0.209409194188189,
                                         0.139918554827525,
                                         0.0865091740100785,
                                         0.0484504302933306,
                                         0.0237642198542172,
                                         0.00963293578299473,
                                         0.00288355755060214,
                                         0.000487880857867596,
                                         1.77865959418149e-05,
                                         0.00839925838718536,
                                         0.0432002887128668,
                                         0.10163041840124,
                                         0.177029169461488,
                                         0.260784517231213,
                                         0.343328308339391,
                                         0.415230490517,
                                         0.468276664653444,
                                         0.496406598581887,
                                         0.496406598581887,
                                         0.468276664653444,
                                         0.415230490517,
                                         0.343328308339391,
                                         0.260784517231213,
                                         0.177029169461488,
                                         0.10163041840124,
                                         0.0432002887128668,
                                         0.00839925838718536,
                                         1.77865959418149e-05,
                                         0.000487880857867596,
                                         0.00288355755060214,
                                         0.00963293578299472,
                                         0.0237642198542172,
                                         0.0484504302933306,
                                         0.0865091740100785,
                                         0.139918554827525,
                                         0.209409194188189,
                                         0.294184207229924,
                                         0.391804780519031,
                                         0.498260335472922,
                                         0.608221261367279,
                                         0.71545126291457,
                                         0.813337894755518,
                                         0.895486024048158,
                                         0.956311830429266,
                                         0.991582955016873,

                                         0.992421257848688,
                                         0.960603999923961,
                                         0.905500598365984,
                                         0.830572179788829,
                                         0.740431145882958,
                                         0.640436318068638,
                                         0.536241864123633,
                                         0.433335257328261,
                                         0.336608046628004,
                                         0.25,
                                         0.176249400987779,
                                         0.116771157364631,
                                         0.071671122747672,
                                         0.0398910134069573,
                                         0.0194649685477287,
                                         0.00785752325168579,
                                         0.0023446947511656,
                                         0.000395847789131208,
                                         1.44140051039295e-05,
                                         0.00756432814620812,
                                         0.0390001522869076,
                                         0.0921547068828508,
                                         0.161570296959485,
                                         0.240103885569314,
                                         0.319672668524404,
                                         0.392087013128695,
                                         0.449893585307108,
                                         0.487142552384216,
                                         0.5,
                                         0.487142552384216,
                                         0.449893585307108,
                                         0.392087013128695,
                                         0.319672668524404,
                                         0.240103885569314,
                                         0.161570296959485,
                                         0.0921547068828508,
                                         0.0390001522869076,
                                         0.00756432814620812,
                                         1.44140051039295e-05,
                                         0.000395847789131208,
                                         0.0023446947511656,
                                         0.00785752325168579,
                                         0.0194649685477287,
                                         0.0398910134069573,
                                         0.071671122747672,
                                         0.116771157364631,
                                         0.176249400987779,
                                         0.25,
                                         0.336608046628004,
                                         0.433335257328261,
                                         0.536241864123633,
                                         0.640436318068638,
                                         0.740431145882958,
                                         0.830572179788829,
                                         0.905500598365984,
                                         0.960603999923961,
                                         0.992421257848688,

                                         0.993140403222385,
                                         0.964296432783931,
                                         0.914160127147419,
                                         0.845587809011132,
                                         0.762418781880186,
                                         0.669167911554694,
                                         0.57067977439597,
                                         0.471767104543454,
                                         0.376864524065903,
                                         0.289727337675948,
                                         0.21320081654245,
                                         0.149078672924258,
                                         0.0980610158280343,
                                         0.0598127724451431,
                                         0.0331142308281793,
                                         0.0160868754200354,
                                         0.00647083718891318,
                                         0.00192569889609291,
                                         0.000324505506016981,
                                         1.18040372897697e-05,
                                         0.00684779274032547,
                                         0.035379061710052,
                                         0.0839141739564882,
                                         0.147941353799955,
                                         0.221494342699778,
                                         0.297717857617126,
                                         0.369507453158887,
                                         0.430171879628511,
                                         0.474056803009838,
                                         0.497071845781602,
                                         0.497071845781602,
                                         0.474056803009838,
                                         0.430171879628511,
                                         0.369507453158887,
                                         0.297717857617126,
                                         0.221494342699778,
                                         0.147941353799955,
                                         0.0839141739564882,
                                         0.0353790617100521,
                                         0.00684779274032558,
                                         1.18040372897693e-05,
                                         0.000324505506016979,
                                         0.00192569889609291,
                                         0.00647083718891317,
                                         0.0160868754200354,
                                         0.0331142308281794,
                                         0.0598127724451431,
                                         0.0980610158280343,
                                         0.149078672924258,
                                         0.21320081654245,
                                         0.289727337675948,
                                         0.376864524065903,
                                         0.471767104543454,
                                         0.57067977439597,
                                         0.669167911554694,
                                         0.762418781880186,
                                         0.845587809011132,
                                         0.914160127147419,
                                         0.964296432783931,
                                         0.993140403222385,

                                         0.993761929463378,
                                         0.967495358593896,
                                         0.921695363251203,
                                         0.858738940294898,
                                         0.781844976104464,
                                         0.694837948115194,
                                         0.601880252970003,
                                         0.507187618849256,
                                         0.414749728134348,
                                         0.328077990427137,
                                         0.25,
                                         0.182516136266242,
                                         0.126728411331947,
                                         0.0828454986418167,
                                         0.0502614170827829,
                                         0.0276991439177822,
                                         0.0134050126287863,
                                         0.00537557571158131,
                                         0.00159602910080231,
                                         0.000268520027589744,
                                         9.75884298918222e-06,
                                         0.00622831169363263,
                                         0.0322361213785145,
                                         0.0767086076479944,
                                         0.13588548399352,
                                         0.204750011266749,
                                         0.277462907967024,
                                         0.347858329947214,
                                         0.409966882508928,
                                         0.458521860533705,
                                         0.489405873306621,
                                         0.5,
                                         0.489405873306621,
                                         0.458521860533705,
                                         0.409966882508928,
                                         0.347858329947214,
                                         0.277462907967024,
                                         0.204750011266749,
                                         0.13588548399352,
                                         0.0767086076479942,
                                         0.0322361213785146,
                                         0.00622831169363252,
                                         9.75884298918257e-06,
                                         0.000268520027589742,
                                         0.00159602910080232,
                                         0.00537557571158131,
                                         0.0134050126287863,
                                         0.0276991439177822,
                                         0.0502614170827829,
                                         0.0828454986418167,
                                         0.126728411331947,
                                         0.182516136266242,
                                         0.25,
                                         0.328077990427137,
                                         0.414749728134348,
                                         0.507187618849256,
                                         0.601880252970003,
                                         0.694837948115194,
                                         0.781844976104464,
                                         0.858738940294898,
                                         0.921695363251203,
                                         0.967495358593896,
                                         0.993761929463378,

                                         0.994302723421103,
                                         0.970284591282895,
                                         0.928290600469503,
                                         0.870314143794818,
                                         0.799072232935424,
                                         0.717821771275474,
                                         0.630150512711898,
                                         0.539751644656441,
                                         0.450197936848328,
                                         0.364731702589863,
                                         0.286085528220652,
                                         0.21634625490093,
                                         0.156871275901642,
                                         0.108262115956244,
                                         0.0703958066696842,
                                         0.0425101092049856,
                                         0.0233345080887908,
                                         0.0112554269562163,
                                         0.00450156607451763,
                                         0.0013338282823291,
                                         0.000224093447465588,
                                         8.13793870441301e-06,
                                         0.00568913864019216,
                                         0.0294913152696398,
                                         0.0703755712481678,
                                         0.125184290130665,
                                         0.189672340108359,
                                         0.258843720635735,
                                         0.327339378083117,
                                         0.389852548673875,
                                         0.441539947195428,
                                         0.478397021508495,
                                         0.497568216878419,
                                         0.497568216878419,
                                         0.478397021508495,
                                         0.441539947195428,
                                         0.389852548673875,
                                         0.327339378083117,
                                         0.258843720635735,
                                         0.189672340108359,
                                         0.125184290130665,
                                         0.0703755712481679,
                                         0.0294913152696398,
                                         0.00568913864019216,
                                         8.13793870441301e-06,
                                         0.000224093447465588,
                                         0.0013338282823291,
                                         0.00450156607451763,
                                         0.0112554269562163,
                                         0.0233345080887908,
                                         0.0425101092049856,
                                         0.0703958066696842,
                                         0.108262115956244,
                                         0.156871275901642,
                                         0.21634625490093,
                                         0.286085528220652,
                                         0.364731702589863,
                                         0.450197936848328,
                                         0.539751644656441,
                                         0.630150512711898,
                                         0.717821771275474,
                                         0.799072232935424,
                                         0.870314143794818,
                                         0.928290600469503,
                                         0.970284591282895,
                                         0.994302723421103,

                                         0.994776174961644,
                                         0.972730950189817,
                                         0.934094305626337,
                                         0.880549853568416,
                                         0.814405535574553,
                                         0.738449220280643,
                                         0.655784037417783,
                                         0.56964867790482,
                                         0.483234244087026,
                                         0.399509754975589,
                                         0.321067757454761,
                                         0.25,
                                         0.187810933156295,
                                         0.135374074005244,
                                         0.0929332060567352,
                                         0.0601472000588126,
                                         0.0361741616541372,
                                         0.0197878571486931,
                                         0.00951713395571277,
                                         0.00379749529797437,
                                         0.00112321880032137,
                                         0.000188478971702011,
                                         6.83996409195817e-06,
                                         0.00521698507426404,
                                         0.027080570838481,
                                         0.0647824755733412,
                                         0.115652651133609,
                                         0.176077330469734,
                                         0.241762922570664,
                                         0.30804180092808,
                                         0.370204122036367,
                                         0.423832549856239,
                                         0.465116171019166,
                                         0.491121309388944,
                                         0.5,
                                         0.491121309388944,
                                         0.465116171019166,
                                         0.423832549856239,
                                         0.370204122036367,
                                         0.30804180092808,
                                         0.241762922570664,
                                         0.176077330469734,
                                         0.115652651133609,
                                         0.0647824755733413,
                                         0.027080570838481,
                                         0.00521698507426393,
                                         6.83996409195846e-06,
                                         0.000188478971702011,
                                         0.00112321880032137,
                                         0.00379749529797437,
                                         0.00951713395571277,
                                         0.0197878571486931,
                                         0.0361741616541372,
                                         0.0601472000588126,
                                         0.0929332060567352,
                                         0.135374074005244,
                                         0.187810933156295,
                                         0.25,
                                         0.321067757454761,
                                         0.399509754975589,
                                         0.483234244087026,
                                         0.56964867790482,
                                         0.655784037417783,
                                         0.738449220280643,
                                         0.814405535574553,
                                         0.880549853568416,
                                         0.934094305626337,
                                         0.972730950189817,
                                         0.994776174961644,

                                         0.99519301070986,
                                         0.974888217442133,
                                         0.939227059735349,
                                         0.889640885130823,
                                         0.828101807237238,
                                         0.757008050529229,
                                         0.67905317138874,
                                         0.597081881057412,
                                         0.513940955627649,
                                         0.432334312355616,
                                         0.354691039112907,
                                         0.283054267812106,
                                         0.2189973749495,
                                         0.163572171639291,
                                         0.117291632659453,
                                         0.0801474480016044,
                                         0.0516604096685718,
                                         0.0309595194517636,
                                         0.016883858950675,
                                         0.00809982126333476,
                                         0.00322535812642199,
                                         0.000952507732615824,
                                         0.000159661470823815,
                                         5.79071283926842e-06,
                                         0.00480119857730049,
                                         0.0249521210870433,
                                         0.0598204325320353,
                                         0.107133756742755,
                                         0.163798371499428,
                                         0.226108090520096,
                                         0.289987309159497,
                                         0.351257709274016,
                                         0.405911596370746,
                                         0.45037405498493,
                                         0.481736789247801,
                                         0.497948357238394,
                                         0.497948357238394,
                                         0.481736789247801,
                                         0.45037405498493,
                                         0.405911596370746,
                                         0.351257709274016,
                                         0.289987309159497,
                                         0.226108090520096,
                                         0.163798371499428,
                                         0.107133756742755,
                                         0.0598204325320352,
                                         0.0249521210870433,
                                         0.00480119857730037,
                                         5.79071283926869e-06,
                                         0.000159661470823815,
                                         0.000952507732615828,
                                         0.00322535812642199,
                                         0.00809982126333476,
                                         0.016883858950675,
                                         0.0309595194517636,
                                         0.0516604096685718,
                                         0.0801474480016044,
                                         0.117291632659453,
                                         0.163572171639291,
                                         0.2189973749495,
                                         0.283054267812106,
                                         0.354691039112907,
                                         0.432334312355616,
                                         0.513940955627649,
                                         0.597081881057412,
                                         0.67905317138874,
                                         0.757008050529229,
                                         0.828101807237238,
                                         0.889640885130823,
                                         0.939227059735349,
                                         0.974888217442133,
                                         0.99519301070986,

                                         0.125,
                                         0.375,
                                         0.375,
                                         0.125,

                                         0.490562612162344,
                                         0.00943738783765592,
                                         0.394337567297406,
                                         0.105662432702593,
                                         0.105662432702593,
                                         0.394337567297406,
                                         0.00943738783765592,
                                         0.490562612162344,

                                         0.698568501158667,
                                         0.125,
                                         0.00143149884133249,
                                         0.266189500386223,
                                         0.375,
                                         0.0338104996137776,
                                         0.0338104996137776,
                                         0.375,
                                         0.266189500386223,
                                         0.00143149884133249,
                                         0.125,
                                         0.698568501158667,

                                         0.805832094644763,
                                         0.300750235878433,
                                         0.0359400966193527,
                                         0.000334715714594482,
                                         0.180374993816148,
                                         0.44441119023977,
                                         0.218898477262444,
                                         0.0134581958244945,
                                         0.0134581958244945,
                                         0.218898477262444,
                                         0.44441119023977,
                                         0.180374993816148,
                                         0.000334715714594481,
                                         0.0359400966193527,
                                         0.300750235878433,
                                         0.805832094644763,

                                         0.865768206669165,
                                         0.455173033583358,
                                         0.125,
                                         0.0122888648609099,
                                         0.000103228219900906,
                                         0.127836583789568,
                                         0.409646762852719,
                                         0.375,
                                         0.122891338703014,
                                         0.00629198132136692,
                                         0.0062919813213669,
                                         0.122891338703014,
                                         0.375,
                                         0.409646762852719,
                                         0.127836583789568,
                                         0.000103228219900906,
                                         0.0122888648609099,
                                         0.125,
                                         0.455173033583358,
                                         0.865768206669165,

                                         0.902086050717951,
                                         0.57303763020425,
                                         0.23753270895737,
                                         0.0551716280178859,
                                         0.00486075935907112,
                                         3.84954707455275e-05,
                                         0.0945706653395711,
                                         0.350599578649969,
                                         0.438034989227942,
                                         0.269260673796802,
                                         0.07150203178671,
                                         0.00330478847173213,
                                         0.00330478847173213,
                                         0.07150203178671,
                                         0.269260673796802,
                                         0.438034989227942,
                                         0.350599578649969,
                                         0.0945706653395711,
                                         3.84954707455275e-05,
                                         0.00486075935907113,
                                         0.0551716280178859,
                                         0.23753270895737,
                                         0.57303763020425,
                                         0.902086050717951,

                                         0.925587895611168,
                                         0.660242961824405,
                                         0.347314148267595,
                                         0.125,
                                         0.0262185669048118,
                                         0.0021584125879273,
                                         1.64763425536365e-05,
                                         0.0725025536343552,
                                         0.293969267338208,
                                         0.440357997435717,
                                         0.375,
                                         0.186109287391876,
                                         0.0436293582494593,
                                         0.0018930744119234,
                                         0.00189307441192339,
                                         0.0436293582494593,
                                         0.186109287391876,
                                         0.375,
                                         0.440357997435717,
                                         0.293969267338208,
                                         0.0725025536343554,
                                         1.64763425536364e-05,
                                         0.0021584125879273,
                                         0.0262185669048118,
                                         0.125,
                                         0.347314148267595,
                                         0.660242961824405,
                                         0.925587895611168,

                                         0.941609629025734,
                                         0.724957266278801,
                                         0.443786747427108,
                                         0.207177623714129,
                                         0.0680585773365012,
                                         0.0133514879772403,
                                         0.00105084089717636,
                                         7.82734330921435e-06,
                                         0.0572233540381443,
                                         0.246136024460014,
                                         0.414076607997518,
                                         0.428855293651718,
                                         0.295908505297651,
                                         0.128785156598134,
                                         0.0278558683640082,
                                         0.00115918959281207,
                                         0.00115918959281207,
                                         0.0278558683640082,
                                         0.128785156598134,
                                         0.295908505297652,
                                         0.428855293651718,
                                         0.414076607997518,
                                         0.246136024460014,
                                         0.0572233540381443,
                                         7.82734330921435e-06,
                                         0.00105084089717636,
                                         0.0133514879772403,
                                         0.0680585773365012,
                                         0.207177623714129,
                                         0.443786747427108,
                                         0.724957266278801,
                                         0.941609629025734,

                                         0.952996652246962,
                                         0.77365995500234,
                                         0.524944151363565,
                                         0.290284151818232,
                                         0.125,
                                         0.0385710601235858,
                                         0.00722423447631573,
                                         0.000551054311011232,
                                         4.0347756351666e-06,
                                         0.0462510895431489,
                                         0.207277805296286,
                                         0.377393080800071,
                                         0.444382891592836,
                                         0.375,
                                         0.226761896465347,
                                         0.0904385333600487,
                                         0.0185111853903632,
                                         0.000748223434253312,
                                         0.000748223434253307,
                                         0.0185111853903632,
                                         0.0904385333600486,
                                         0.226761896465347,
                                         0.375,
                                         0.444382891592836,
                                         0.377393080800071,
                                         0.207277805296286,
                                         0.0462510895431491,
                                         4.03477563516656e-06,
                                         0.000551054311011232,
                                         0.00722423447631572,
                                         0.0385710601235858,
                                         0.125,
                                         0.290284151818232,
                                         0.524944151363565,
                                         0.77365995500234,
                                         0.952996652246962,

                                         0.961368223935969,
                                         0.810943857283931,
                                         0.592079306686002,
                                         0.368135777704715,
                                         0.189551663752774,
                                         0.0770710128526389,
                                         0.0227378980346186,
                                         0.00411871443619747,
                                         0.000307114006529612,
                                         2.22078030792999e-06,
                                         0.0381255656841281,
                                         0.176014449472144,
                                         0.339074453512729,
                                         0.436559433819058,
                                         0.421279193819537,
                                         0.31209812957505,
                                         0.172566890441608,
                                         0.0647275253650712,
                                         0.0127345792373946,
                                         0.00050398959959508,
                                         0.00050398959959508,
                                         0.0127345792373947,
                                         0.0647275253650712,
                                         0.172566890441608,
                                         0.31209812957505,
                                         0.421279193819537,
                                         0.436559433819058,
                                         0.339074453512729,
                                         0.176014449472144,
                                         0.0381255656841281,
                                         2.22078030792999e-06,
                                         0.000307114006529613,
                                         0.00411871443619747,
                                         0.0227378980346186,
                                         0.0770710128526389,
                                         0.189551663752774,
                                         0.368135777704715,
                                         0.592079306686002,
                                         0.810943857283931,
                                         0.961368223935969,

                                         0.967697190785277,
                                         0.839979979390638,
                                         0.647385241546223,
                                         0.43819335193245,
                                         0.255771657972871,
                                         0.125,
                                         0.0487184767176915,
                                         0.0139022415856712,
                                         0.00245622188663198,
                                         0.000180062539861009,
                                         1.28992839959871e-06,
                                         0.0319498955769315,
                                         0.150814003410728,
                                         0.30291374715526,
                                         0.416159731530989,
                                         0.441489894700467,
                                         0.375,
                                         0.254019970608971,
                                         0.13174467495089,
                                         0.0472447894118851,
                                         0.0090259546587731,
                                         0.000351623709392146,
                                         0.000351623709392142,
                                         0.0090259546587731,
                                         0.0472447894118851,
                                         0.13174467495089,
                                         0.254019970608971,
                                         0.375,
                                         0.441489894700467,
                                         0.416159731530989,
                                         0.30291374715526,
                                         0.150814003410728,
                                         0.0319498955769316,
                                         1.28992839959869e-06,
                                         0.000180062539861009,
                                         0.00245622188663198,
                                         0.0139022415856712,
                                         0.0487184767176915,
                                         0.125,
                                         0.255771657972871,
                                         0.43819335193245,
                                         0.647385241546223,
                                         0.839979979390638,
                                         0.967697190785277,

                                         0.97259517533054,
                                         0.862960822693408,
                                         0.693039789280884,
                                         0.499921481721771,
                                         0.319895267257738,
                                         0.178089316477891,
                                         0.0836732384776598,
                                         0.0315797414836979,
                                         0.00878530284879135,
                                         0.00152280651777266,
                                         0.000110187256748149,
                                         7.83696576333459e-07,
                                         0.0271513844055756,
                                         0.130364426425644,
                                         0.270297239247462,
                                         0.389919270835174,
                                         0.443536455465491,
                                         0.415344718289082,
                                         0.322892726755367,
                                         0.204988535793072,
                                         0.101373944594263,
                                         0.0351401649538812,
                                         0.00656456362419948,
                                         0.00025265656730845,
                                         0.000252656567308453,
                                         0.00656456362419946,
                                         0.0351401649538811,
                                         0.101373944594263,
                                         0.204988535793072,
                                         0.322892726755368,
                                         0.415344718289082,
                                         0.443536455465492,
                                         0.389919270835174,
                                         0.270297239247462,
                                         0.130364426425644,
                                         0.0271513844055754,
                                         7.83696576333474e-07,
                                         0.000110187256748149,
                                         0.00152280651777266,
                                         0.00878530284879134,
                                         0.0315797414836979,
                                         0.0836732384776598,
                                         0.178089316477891,
                                         0.319895267257738,
                                         0.499921481721771,
                                         0.693039789280884,
                                         0.862960822693408,
                                         0.97259517533054,

                                         0.976461719269374,
                                         0.881420178088515,
                                         0.73091906173222,
                                         0.553740939808814,
                                         0.379890988258222,
                                         0.232868491727414,
                                         0.125,
                                         0.0569647848172437,
                                         0.0209683225409581,
                                         0.00571856560076649,
                                         0.000976514925523745,
                                         6.99386039136033e-05,
                                         4.94627034447266e-07,
                                         0.0233516381661685,
                                         0.11362718126135,
                                         0.241505527561048,
                                         0.361760695143648,
                                         0.433925472579184,
                                         0.436915275295118,
                                         0.375,
                                         0.273251448160224,
                                         0.165215216621636,
                                         0.0787797994467713,
                                         0.0265988957812077,
                                         0.00488270204622096,
                                         0.000186147937423041,
                                         0.000186147937423044,
                                         0.00488270204622097,
                                         0.0265988957812077,
                                         0.0787797994467714,
                                         0.165215216621636,
                                         0.273251448160224,
                                         0.375,
                                         0.436915275295118,
                                         0.433925472579184,
                                         0.361760695143648,
                                         0.241505527561049,
                                         0.11362718126135,
                                         0.0233516381661684,
                                         4.94627034447277e-07,
                                         6.99386039136036e-05,
                                         0.000976514925523743,
                                         0.0057185656007665,
                                         0.0209683225409581,
                                         0.0569647848172437,
                                         0.125,
                                         0.232868491727414,
                                         0.379890988258222,
                                         0.553740939808814,
                                         0.73091906173222,
                                         0.881420178088515,
                                         0.976461719269374,

                                         0.97956649091304,
                                         0.896447684220326,
                                         0.762551553921164,
                                         0.600456366408335,
                                         0.43487227314286,
                                         0.28691641177508,
                                         0.17005676221076,
                                         0.0887001417443503,
                                         0.0394581162269978,
                                         0.0142385948088432,
                                         0.00382228634502808,
                                         0.000644957818697276,
                                         4.58156822197784e-05,
                                         3.22560077004606e-07,
                                         0.0202930537792146,
                                         0.0998027727369291,
                                         0.216343822581016,
                                         0.333848910745886,
                                         0.417367003060355,
                                         0.444293846068674,
                                         0.410669040383847,
                                         0.330574055661043,
                                         0.229331625929249,
                                         0.133522128987943,
                                         0.0618724365007512,
                                         0.0204596656791221,
                                         0.0037037273605253,
                                         0.000140132747668224,
                                         0.000140132747668224,
                                         0.00370372736052531,
                                         0.0204596656791221,
                                         0.0618724365007512,
                                         0.133522128987942,
                                         0.229331625929249,
                                         0.330574055661043,
                                         0.410669040383847,
                                         0.444293846068674,
                                         0.417367003060355,
                                         0.333848910745886,
                                         0.216343822581016,
                                         0.099802772736929,
                                         0.0202930537792146,
                                         3.22560077004606e-07,
                                         4.58156822197786e-05,
                                         0.000644957818697276,
                                         0.00382228634502807,
                                         0.0142385948088431,
                                         0.0394581162269978,
                                         0.0887001417443503,
                                         0.17005676221076,
                                         0.28691641177508,
                                         0.43487227314286,
                                         0.600456366408335,
                                         0.762551553921164,
                                         0.896447684220326,
                                         0.97956649091304,

                                         0.982096695344065,
                                         0.908830208348901,
                                         0.78915361751279,
                                         0.640969629994211,
                                         0.484635797152551,
                                         0.338719173633118,
                                         0.216645452608655,
                                         0.125,
                                         0.0637138449859316,
                                         0.0277972896691751,
                                         0.0098711192674354,
                                         0.00261615713755078,
                                         0.000437188592802236,
                                         3.0850727844252e-05,
                                         2.16404278856247e-07,
                                         0.0177956027468764,
                                         0.0882805226311001,
                                         0.194439828682863,
                                         0.307303494189384,
                                         0.397057783875142,
                                         0.441585963019283,
                                         0.432214080764774,
                                         0.375,
                                         0.287426621640639,
                                         0.191897573678423,
                                         0.108435299704872,
                                         0.0491107186788542,
                                         0.0159693652115451,
                                         0.0028584182921541,
                                         0.000107485504779716,
                                         0.000107485504779714,
                                         0.0028584182921541,
                                         0.015969365211545,
                                         0.0491107186788542,
                                         0.108435299704872,
                                         0.191897573678423,
                                         0.287426621640639,
                                         0.375,
                                         0.432214080764774,
                                         0.441585963019283,
                                         0.397057783875142,
                                         0.307303494189384,
                                         0.194439828682863,
                                         0.0882805226311001,
                                         0.0177956027468766,
                                         2.16404278856241e-07,
                                         3.0850727844252e-05,
                                         0.000437188592802235,
                                         0.00261615713755079,
                                         0.0098711192674354,
                                         0.0277972896691751,
                                         0.0637138449859316,
                                         0.125,
                                         0.216645452608655,
                                         0.338719173633118,
                                         0.484635797152551,
                                         0.640969629994211,
                                         0.78915361751279,
                                         0.908830208348901,
                                         0.982096695344065,

                                         0.984185508784156,
                                         0.919145197967567,
                                         0.811684780770974,
                                         0.676147687458413,
                                         0.529353644761741,
                                         0.387433863656307,
                                         0.263130456374242,
                                         0.164122171792289,
                                         0.0926483609769465,
                                         0.0463449634843732,
                                         0.0199006628053256,
                                         0.00697464528609555,
                                         0.00182917766321623,
                                         0.000303253139578489,
                                         2.1282692781575e-05,
                                         1.48837607601701e-07,
                                         0.0157305337567709,
                                         0.0785934213674962,
                                         0.175380495179378,
                                         0.282640415278895,
                                         0.375081722366579,
                                         0.432058101978553,
                                         0.442489376904778,
                                         0.406922782148825,
                                         0.33630668508194,
                                         0.248035203236607,
                                         0.160607371559815,
                                         0.0885899875855843,
                                         0.0393827195994759,
                                         0.0126314709100694,
                                         0.00224009797215476,
                                         8.38086214656144e-05,
                                         8.38086214656161e-05,
                                         0.00224009797215476,
                                         0.0126314709100694,
                                         0.0393827195994759,
                                         0.0885899875855843,
                                         0.160607371559815,
                                         0.248035203236607,
                                         0.33630668508194,
                                         0.406922782148825,
                                         0.442489376904778,
                                         0.432058101978553,
                                         0.375081722366579,
                                         0.282640415278895,
                                         0.175380495179378,
                                         0.0785934213674962,
                                         0.0157305337567707,
                                         1.48837607601705e-07,
                                         2.1282692781575e-05,
                                         0.000303253139578488,
                                         0.00182917766321623,
                                         0.00697464528609555,
                                         0.0199006628053256,
                                         0.0463449634843732,
                                         0.0926483609769465,
                                         0.164122171792289,
                                         0.263130456374242,
                                         0.387433863656307,
                                         0.529353644761741,
                                         0.676147687458413,
                                         0.811684780770974,
                                         0.919145197967567,
                                         0.984185508784156,

                                         0.98592972458304,
                                         0.927822960551749,
                                         0.830901014673583,
                                         0.706769394212492,
                                         0.569383866926132,
                                         0.432673486616657,
                                         0.308389476672479,
                                         0.204588531667483,
                                         0.125,
                                         0.0693039206156993,
                                         0.0341333370717103,
                                         0.0144652034968583,
                                         0.0050146483229375,
                                         0.00130370950261508,
                                         0.000214711141917221,
                                         1.5000215990329e-05,
                                         1.04637747447839e-07,
                                         0.0140038684432931,
                                         0.0703823617656441,
                                         0.15877141238523,
                                         0.260035926922833,
                                         0.35275365329571,
                                         0.41815403589326,
                                         0.444202028907566,
                                         0.427853129524506,
                                         0.375,
                                         0.298254418192312,
                                         0.213275157348245,
                                         0.134707273993225,
                                         0.0728478314552206,
                                         0.03189096936206,
                                         0.0101128617992697,
                                         0.00177967746661646,
                                         6.63023359195301e-05,
                                         6.63023359195286e-05,
                                         0.00177967746661647,
                                         0.0101128617992697,
                                         0.0318909693620601,
                                         0.0728478314552206,
                                         0.134707273993225,
                                         0.213275157348245,
                                         0.298254418192312,
                                         0.375,
                                         0.427853129524506,
                                         0.444202028907566,
                                         0.41815403589326,
                                         0.35275365329571,
                                         0.260035926922833,
                                         0.15877141238523,
                                         0.0703823617656439,
                                         0.0140038684432932,
                                         1.04637747447835e-07,
                                         1.50002159903291e-05,
                                         0.000214711141917221,
                                         0.00130370950261508,
                                         0.0050146483229375,
                                         0.0144652034968583,
                                         0.0341333370717103,
                                         0.0693039206156993,
                                         0.125,
                                         0.204588531667483,
                                         0.308389476672479,
                                         0.432673486616657,
                                         0.569383866926132,
                                         0.706769394212492,
                                         0.830901014673583,
                                         0.927822960551749,
                                         0.98592972458304,

                                         0.987401037405751,
                                         0.93518879060601,
                                         0.847399528913817,
                                         0.7335107970454,
                                         0.605159815706872,
                                         0.474342891159843,
                                         0.351709803657381,
                                         0.245247503945929,
                                         0.159561838617257,
                                         0.095828263509913,
                                         0.0523374990739049,
                                         0.0254444605671187,
                                         0.0106646463310713,
                                         0.00366340844630878,
                                         0.000945448762369001,
                                         0.000154843484323509,
                                         1.07763246901542e-05,
                                         7.50134705670802e-08,
                                         0.0125457528333643,
                                         0.0633691194697678,
                                         0.144259485403024,
                                         0.239481293130354,
                                         0.330874341623094,
                                         0.401635110622308,
                                         0.43965159544662,
                                         0.439671829719305,
                                         0.403867105838002,
                                         0.340742792034828,
                                         0.262743167260861,
                                         0.183194140328879,
                                         0.113357351886778,
                                         0.0603024342237251,
                                         0.0260624610618772,
                                         0.0081861421988359,
                                         0.00143131359953232,
                                         5.31347474137434e-05,
                                         5.31347474137434e-05,
                                         0.00143131359953232,
                                         0.0081861421988359,
                                         0.0260624610618772,
                                         0.0603024342237251,
                                         0.113357351886778,
                                         0.183194140328879,
                                         0.262743167260861,
                                         0.340742792034828,
                                         0.403867105838002,
                                         0.439671829719305,
                                         0.43965159544662,
                                         0.401635110622308,
                                         0.330874341623094,
                                         0.239481293130354,
                                         0.144259485403024,
                                         0.0633691194697678,
                                         0.0125457528333643,
                                         7.50134705670802e-08,
                                         1.07763246901542e-05,
                                         0.000154843484323509,
                                         0.000945448762368999,
                                         0.00366340844630878,
                                         0.0106646463310713,
                                         0.0254444605671187,
                                         0.0523374990739049,
                                         0.095828263509913,
                                         0.159561838617257,
                                         0.245247503945929,
                                         0.351709803657381,
                                         0.474342891159843,
                                         0.605159815706872,
                                         0.7335107970454,
                                         0.847399528913817,
                                         0.93518879060601,
                                         0.987401037405751,

                                         0.988653453056792,
                                         0.941491895812137,
                                         0.861654404753486,
                                         0.756948042706551,
                                         0.637128479355068,
                                         0.512523670909787,
                                         0.392682072248188,
                                         0.285256821523445,
                                         0.195293028548436,
                                         0.125,
                                         0.073993142875239,
                                         0.0399028005158925,
                                         0.0191874080587693,
                                         0.00796732630360646,
                                         0.00271569229096114,
                                         0.00069651185422125,
                                         0.000113534922237856,
                                         7.87575750143646e-06,
                                         5.47238957967567e-08,
                                         0.0113034143756878,
                                         0.057336312335472,
                                         0.131538580837493,
                                         0.220872411246834,
                                         0.309907999583667,
                                         0.383737941476554,
                                         0.430679375626334,
                                         0.444235307414448,
                                         0.423945054238704,
                                         0.375,
                                         0.306768774337621,
                                         0.230605070546215,
                                         0.157451144066708,
                                         0.0957710613100525,
                                         0.0502478287703028,
                                         0.0214830341923936,
                                         0.00669347948678325,
                                         0.00116391609488932,
                                         4.30778436243982e-05,
                                         4.30778436243982e-05,
                                         0.00116391609488932,
                                         0.00669347948678325,
                                         0.0214830341923936,
                                         0.0502478287703027,
                                         0.0957710613100526,
                                         0.157451144066708,
                                         0.230605070546215,
                                         0.306768774337621,
                                         0.375,
                                         0.423945054238704,
                                         0.444235307414448,
                                         0.430679375626334,
                                         0.383737941476554,
                                         0.309907999583668,
                                         0.220872411246834,
                                         0.131538580837493,
                                         0.057336312335472,
                                         0.0113034143756878,
                                         5.47238957967567e-08,
                                         7.87575750143646e-06,
                                         0.000113534922237856,
                                         0.00069651185422125,
                                         0.00271569229096114,
                                         0.00796732630360647,
                                         0.0191874080587693,
                                         0.0399028005158925,
                                         0.073993142875239,
                                         0.125,
                                         0.195293028548436,
                                         0.285256821523445,
                                         0.392682072248188,
                                         0.512523670909787,
                                         0.637128479355068,
                                         0.756948042706551,
                                         0.861654404753486,
                                         0.941491895812137,
                                         0.988653453056792,

                                         0.989728270334376,
                                         0.946925561780937,
                                         0.874044234032952,
                                         0.777567445359169,
                                         0.665718122440926,
                                         0.547397312361567,
                                         0.431110619907807,
                                         0.324034671983494,
                                         0.231354465222673,
                                         0.155949581452779,
                                         0.0984426498748178,
                                         0.0575603302625696,
                                         0.0307075085737394,
                                         0.0146282003538629,
                                         0.00602590121274422,
                                         0.00204036350940672,
                                         0.000520523940898876,
                                         8.45050323156925e-05,
                                         5.84565398474856e-06,
                                         4.05551357260468e-08,
                                         0.0102363986640261,
                                         0.0521126130089813,
                                         0.120347679343401,
                                         0.204061090955889,
                                         0.290101978317781,
                                         0.365311797579384,
                                         0.41870746346449,
                                         0.443197297679882,
                                         0.436530176529691,
                                         0.401333268669506,
                                         0.344274500002897,
                                         0.274555027985066,
                                         0.202060521762885,
                                         0.135553716273841,
                                         0.0812649888463054,
                                         0.042139535731886,
                                         0.0178509397440429,
                                         0.00552358159133166,
                                         0.000955979556096696,
                                         3.52904464621309e-05,
                                         3.52904464621297e-05,
                                         0.000955979556096691,
                                         0.00552358159133166,
                                         0.0178509397440429,
                                         0.042139535731886,
                                         0.0812649888463054,
                                         0.13555371627384,
                                         0.202060521762885,
                                         0.274555027985066,
                                         0.344274500002897,
                                         0.401333268669506,
                                         0.436530176529691,
                                         0.443197297679882,
                                         0.41870746346449,
                                         0.365311797579384,
                                         0.290101978317781,
                                         0.204061090955889,
                                         0.120347679343401,
                                         0.0521126130089815,
                                         0.0102363986640262,
                                         4.05551357260449e-08,
                                         5.84565398474851e-06,
                                         8.45050323156925e-05,
                                         0.000520523940898875,
                                         0.00204036350940672,
                                         0.00602590121274422,
                                         0.0146282003538629,
                                         0.0307075085737394,
                                         0.0575603302625696,
                                         0.0984426498748178,
                                         0.155949581452779,
                                         0.231354465222673,
                                         0.324034671983494,
                                         0.431110619907807,
                                         0.547397312361567,
                                         0.665718122440926,
                                         0.777567445359169,
                                         0.874044234032952,
                                         0.946925561780937,
                                         0.989728270334376,

                                         0.990657501973758,
                                         0.951641417807122,
                                         0.884873326634074,
                                         0.795777645841832,
                                         0.691322950492911,
                                         0.579195652965874,
                                         0.466944368728411,
                                         0.361204344187356,
                                         0.26710324548752,
                                         0.187916815511546,
                                         0.125,
                                         0.0779743745285226,
                                         0.0451139637119216,
                                         0.0238453320492528,
                                         0.0112681363507681,
                                         0.00460998508359048,
                                         0.00155203260696538,
                                         0.000394128167886962,
                                         6.37618939347207e-05,
                                         4.40012510618934e-06,
                                         3.04857929694112e-08,
                                         0.0093132824688603,
                                         0.0475618223603211,
                                         0.110466109851389,
                                         0.188883883359198,
                                         0.271566076834661,
                                         0.34692688544796,
                                         0.404807652724777,
                                         0.4379498239857,
                                         0.442939447940483,
                                         0.420483524746774,
                                         0.375,
                                         0.313625285213158,
                                         0.244843342860076,
                                         0.177000499777692,
                                         0.116979842196045,
                                         0.0692674765025752,
                                         0.0355589400654629,
                                         0.014944342631083,
                                         0.00459680162060278,
                                         0.000792359707450664,
                                         2.91850715886384e-05,
                                         2.91850715886395e-05,
                                         0.000792359707450659,
                                         0.00459680162060279,
                                         0.014944342631083,
                                         0.0355589400654629,
                                         0.0692674765025752,
                                         0.116979842196045,
                                         0.177000499777691,
                                         0.244843342860076,
                                         0.313625285213158,
                                         0.375,
                                         0.420483524746774,
                                         0.442939447940483,
                                         0.4379498239857,
                                         0.404807652724777,
                                         0.34692688544796,
                                         0.271566076834661,
                                         0.188883883359198,
                                         0.110466109851389,
                                         0.0475618223603213,
                                         0.00931328246886014,
                                         3.04857929694128e-08,
                                         4.4001251061893e-06,
                                         6.3761893934721e-05,
                                         0.000394128167886962,
                                         0.00155203260696538,
                                         0.00460998508359048,
                                         0.0112681363507681,
                                         0.0238453320492528,
                                         0.0451139637119216,
                                         0.0779743745285226,
                                         0.125,
                                         0.187916815511546,
                                         0.26710324548752,
                                         0.361204344187356,
                                         0.466944368728411,
                                         0.579195652965874,
                                         0.691322950492911,
                                         0.795777645841832,
                                         0.884873326634074,
                                         0.951641417807122,
                                         0.990657501973758,

                                         0.991466268824555,
                                         0.955759672472413,
                                         0.894387929566203,
                                         0.811921538030122,
                                         0.714297383616642,
                                         0.608169924332197,
                                         0.500226207136002,
                                         0.396543615069448,
                                         0.302068368974242,
                                         0.220272494958456,
                                         0.153018462533024,
                                         0.100629212149348,
                                         0.0621319727788005,
                                         0.0356217102326158,
                                         0.0186775619197404,
                                         0.00876472573932248,
                                         0.00356449471419948,
                                         0.00119410622081884,
                                         0.000302026773880634,
                                         4.87135615446777e-05,
                                         3.35462312773106e-06,
                                         2.32151568137525e-08,
                                         0.00850936378964544,
                                         0.0435747564314461,
                                         0.101708012709898,
                                         0.175177817294086,
                                         0.254324547956347,
                                         0.328955540829829,
                                         0.389772916727686,
                                         0.42962408876098,
                                         0.444388703622258,
                                         0.433377622894219,
                                         0.399201197062882,
                                         0.347151128254746,
                                         0.284217909368524,
                                         0.217921217170884,
                                         0.155154734249831,
                                         0.101236150396989,
                                         0.0593100401237739,
                                         0.0301839622061922,
                                         0.012598617901911,
                                         0.00385534416235328,
                                         0.000662216473013569,
                                         2.43441706427978e-05,
                                         2.43441706427978e-05,
                                         0.000662216473013569,
                                         0.00385534416235327,
                                         0.012598617901911,
                                         0.0301839622061923,
                                         0.0593100401237739,
                                         0.101236150396989,
                                         0.155154734249831,
                                         0.217921217170884,
                                         0.284217909368524,
                                         0.347151128254746,
                                         0.399201197062882,
                                         0.433377622894219,
                                         0.444388703622258,
                                         0.42962408876098,
                                         0.389772916727686,
                                         0.328955540829829,
                                         0.254324547956347,
                                         0.175177817294086,
                                         0.101708012709899,
                                         0.0435747564314461,
                                         0.00850936378964544,
                                         2.32151568137525e-08,
                                         3.35462312773106e-06,
                                         4.87135615446774e-05,
                                         0.000302026773880635,
                                         0.00119410622081884,
                                         0.00356449471419948,
                                         0.00876472573932248,
                                         0.0186775619197404,
                                         0.0356217102326158,
                                         0.0621319727788004,
                                         0.100629212149348,
                                         0.153018462533024,
                                         0.220272494958456,
                                         0.302068368974242,
                                         0.396543615069448,
                                         0.500226207136002,
                                         0.608169924332197,
                                         0.714297383616642,
                                         0.811921538030122,
                                         0.894387929566203,
                                         0.955759672472413,
                                         0.991466268824555,

                                         0.992174504499824,
                                         0.959376556158883,
                                         0.902788642572267,
                                         0.826287007129609,
                                         0.734955552686345,
                                         0.634572071765628,
                                         0.531057151684999,
                                         0.429942760575175,
                                         0.335920535582988,
                                         0.252517268080181,
                                         0.181926113598906,
                                         0.125,
                                         0.0813919223176779,
                                         0.0498084753910697,
                                         0.0283306396326543,
                                         0.014751056370274,
                                         0.0068801469228816,
                                         0.00278354437837652,
                                         0.000928451609053365,
                                         0.000234016169977213,
                                         3.7644067721064e-05,
                                         2.58758339514403e-06,
                                         1.78887803969028e-08,
                                         0.00780501138546137,
                                         0.0400631820928009,
                                         0.0939169891622109,
                                         0.162788539316422,
                                         0.238349948664623,
                                         0.311631445545046,
                                         0.374180657198353,
                                         0.419117751988935,
                                         0.441941125512115,
                                         0.440977460686225,
                                         0.417424931567564,
                                         0.375,
                                         0.319257032515852,
                                         0.256696795842524,
                                         0.193807699272243,
                                         0.136188431065616,
                                         0.0878820441937668,
                                         0.0510129383109499,
                                         0.0257660470399782,
                                         0.0106904373839915,
                                         0.00325672419780093,
                                         0.000557674164920601,
                                         2.04662259346838e-05,
                                         2.04662259346847e-05,
                                         0.000557674164920601,
                                         0.00325672419780092,
                                         0.0106904373839915,
                                         0.0257660470399782,
                                         0.0510129383109498,
                                         0.0878820441937668,
                                         0.136188431065616,
                                         0.193807699272243,
                                         0.256696795842524,
                                         0.319257032515852,
                                         0.375,
                                         0.417424931567564,
                                         0.440977460686225,
                                         0.441941125512115,
                                         0.419117751988935,
                                         0.374180657198353,
                                         0.311631445545046,
                                         0.238349948664623,
                                         0.162788539316422,
                                         0.0939169891622111,
                                         0.0400631820928009,
                                         0.00780501138546121,
                                         1.78887803969039e-08,
                                         2.58758339514403e-06,
                                         3.76440677210638e-05,
                                         0.000234016169977213,
                                         0.000928451609053365,
                                         0.00278354437837651,
                                         0.0068801469228816,
                                         0.014751056370274,
                                         0.0283306396326543,
                                         0.0498084753910697,
                                         0.0813919223176779,
                                         0.125,
                                         0.181926113598906,
                                         0.252517268080181,
                                         0.335920535582988,
                                         0.429942760575175,
                                         0.531057151684999,
                                         0.634572071765628,
                                         0.734955552686345,
                                         0.826287007129609,
                                         0.902788642572267,
                                         0.959376556158883,
                                         0.992174504499824,

                                         0.992798188199336,
                                         0.962569800931473,
                                         0.910239954218689,
                                         0.839116189584362,
                                         0.753573466880175,
                                         0.658644010972816,
                                         0.559571610546727,
                                         0.461371579581681,
                                         0.368442602741025,
                                         0.284269036322364,
                                         0.211239594405603,
                                         0.150592922359824,
                                         0.102484541782584,
                                         0.0661552217226951,
                                         0.0401698812002406,
                                         0.0226900027028562,
                                         0.0117418565072945,
                                         0.00544742571402813,
                                         0.00219385324704017,
                                         0.000728975870683305,
                                         0.000183175301505857,
                                         2.93969832582863e-05,
                                         2.01743796173126e-06,
                                         1.39347134779122e-08,
                                         0.00718446753157336,
                                         0.0369552495319788,
                                         0.0869613165499804,
                                         0.151574086639384,
                                         0.223585021071187,
                                         0.295092118669239,
                                         0.358444682526039,
                                         0.407130904427193,
                                         0.436495058659875,
                                         0.444195828099757,
                                         0.430354334121913,
                                         0.397384036356843,
                                         0.349538499500748,
                                         0.292250849749789,
                                         0.231365254377638,
                                         0.172372335896245,
                                         0.119755659483832,
                                         0.0765362812132063,
                                         0.0440700171109045,
                                         0.0221125361779544,
                                         0.0091265484747484,
                                         0.00276933224807261,
                                         0.000472932098586252,
                                         1.73303343773715e-05,
                                         1.73303343773723e-05,
                                         0.000472932098586252,
                                         0.00276933224807262,
                                         0.00912654847474841,
                                         0.0221125361779544,
                                         0.0440700171109046,
                                         0.0765362812132063,
                                         0.119755659483832,
                                         0.172372335896245,
                                         0.231365254377638,
                                         0.292250849749789,
                                         0.349538499500748,
                                         0.397384036356843,
                                         0.430354334121913,
                                         0.444195828099757,
                                         0.436495058659875,
                                         0.407130904427193,
                                         0.358444682526039,
                                         0.295092118669239,
                                         0.223585021071187,
                                         0.151574086639384,
                                         0.0869613165499802,
                                         0.0369552495319788,
                                         0.00718446753157319,
                                         1.39347134779131e-08,
                                         2.01743796173126e-06,
                                         2.93969832582865e-05,
                                         0.000183175301505857,
                                         0.000728975870683305,
                                         0.00219385324704017,
                                         0.00544742571402813,
                                         0.0117418565072946,
                                         0.0226900027028562,
                                         0.0401698812002406,
                                         0.0661552217226951,
                                         0.102484541782584,
                                         0.150592922359824,
                                         0.211239594405603,
                                         0.284269036322364,
                                         0.368442602741025,
                                         0.461371579581681,
                                         0.559571610546727,
                                         0.658644010972816,
                                         0.753573466880175,
                                         0.839116189584362,
                                         0.910239954218689,
                                         0.962569800931473,
                                         0.992798188199336,

                                         0.0625,
                                         0.25,
                                         0.375,
                                         0.25,
                                         0.0625,

                                         0.38689453417432,
                                         0.00199435471456919,
                                         0.414672311952097,
                                         0.0297721324923469,
                                         0.166666666666667,
                                         0.166666666666667,
                                         0.0297721324923469,
                                         0.414672311952097,
                                         0.00199435471456919,
                                         0.38689453417432,

                                         0.619838667696593,
                                         0.0625,
                                         0.00016133230340665,
                                         0.314919333848297,
                                         0.25,
                                         0.00508066615170335,
                                         0.0600000000000002,
                                         0.375,
                                         0.0600000000000002,
                                         0.00508066615170335,
                                         0.25,
                                         0.314919333848297,
                                         0.00016133230340665,
                                         0.0625,
                                         0.619838667696593,

                                         0.749881686195632,
                                         0.201499807465387,
                                         0.0118605725320823,
                                         2.3239929348011e-05,
                                         0.223801633796524,
                                         0.397001713652184,
                                         0.0963180963490815,
                                         0.00124590314098588,
                                         0.0250475369375102,
                                         0.293319810001265,
                                         0.293319810001265,
                                         0.0250475369375102,
                                         0.00124590314098588,
                                         0.0963180963490815,
                                         0.397001713652184,
                                         0.223801633796524,
                                         2.3239929348011e-05,
                                         0.0118605725320823,
                                         0.201499807465387,
                                         0.749881686195632,

                                         0.825154953403611,
                                         0.35013487147785,
                                         0.0625,
                                         0.00283584413863688,
                                         4.84244374729024e-06,
                                         0.162453013062215,
                                         0.420152648422032,
                                         0.25,
                                         0.037812082889092,
                                         0.000393543104614463,
                                         0.0119936479858121,
                                         0.189064553072389,
                                         0.375,
                                         0.189064553072389,
                                         0.0119936479858121,
                                         0.000393543104614461,
                                         0.037812082889092,
                                         0.25,
                                         0.420152648422032,
                                         0.162453013062215,
                                         4.84244374729021e-06,
                                         0.00283584413863688,
                                         0.0625,
                                         0.35013487147785,
                                         0.825154953403611,

                                         0.87162689610018,
                                         0.475967745046842,
                                         0.147106285318457,
                                         0.0210033095226865,
                                         0.000823389822749775,
                                         1.29980892021191e-06,
                                         0.121836618471087,
                                         0.388279540629631,
                                         0.36170569455565,
                                         0.136673273980797,
                                         0.0161494781452854,
                                         0.000148782647301262,
                                         0.00638640297251237,
                                         0.118779846355492,
                                         0.333511436622409,
                                         0.333511436622409,
                                         0.118779846355492,
                                         0.00638640297251237,
                                         0.000148782647301262,
                                         0.0161494781452854,
                                         0.136673273980797,
                                         0.36170569455565,
                                         0.388279540629631,
                                         0.121836618471087,
                                         1.29980892021191e-06,
                                         0.000823389822749776,
                                         0.0210033095226865,
                                         0.147106285318457,
                                         0.475967745046842,
                                         0.87162689610018,

                                         0.902035345452205,
                                         0.574916854044856,
                                         0.244134955673384,
                                         0.0625,
                                         0.00778894432521501,
                                         0.000278941171294456,
                                         4.19257734755207e-07,
                                         0.0942102006358513,
                                         0.341304431118197,
                                         0.412716770376842,
                                         0.25,
                                         0.073718490318387,
                                         0.00751788566653136,
                                         6.42283392755251e-05,
                                         0.00368980631493349,
                                         0.0759818879991215,
                                         0.261640839306171,
                                         0.375,
                                         0.261640839306171,
                                         0.0759818879991215,
                                         0.00368980631493351,
                                         6.42283392755246e-05,
                                         0.00751788566653136,
                                         0.073718490318387,
                                         0.25,
                                         0.412716770376842,
                                         0.341304431118197,
                                         0.0942102006358515,
                                         4.19257734755204e-07,
                                         0.000278941171294456,
                                         0.00778894432521501,
                                         0.0625,
                                         0.244134955673384,
                                         0.574916854044856,
                                         0.902035345452205,

                                         0.922913902279778,
                                         0.651253208940273,
                                         0.338505533145703,
                                         0.122590588526614,
                                         0.0277871382670088,
                                         0.00316742416229615,
                                         0.000106835590650347,
                                         1.55412463025977e-07,
                                         0.0747829069838277,
                                         0.294816229354112,
                                         0.421124857125621,
                                         0.338348140750059,
                                         0.16108575627797,
                                         0.0407362552597765,
                                         0.00377602122610407,
                                         3.06877233847535e-05,
                                         0.00227234760054701,
                                         0.0500477048888603,
                                         0.196465930306603,
                                         0.350188376178348,
                                         0.350188376178348,
                                         0.196465930306603,
                                         0.0500477048888603,
                                         0.00227234760054701,
                                         3.06877233847535e-05,
                                         0.00377602122610407,
                                         0.0407362552597765,
                                         0.16108575627797,
                                         0.338348140750059,
                                         0.421124857125621,
                                         0.294816229354112,
                                         0.0747829069838277,
                                         1.55412463025977e-07,
                                         0.000106835590650347,
                                         0.00316742416229615,
                                         0.0277871382670088,
                                         0.122590588526614,
                                         0.338505533145703,
                                         0.651253208940273,
                                         0.922913902279778,

                                         0.937825059668174,
                                         0.71023187193861,
                                         0.423464948786615,
                                         0.192204890902582,
                                         0.0625,
                                         0.0130321309170995,
                                         0.00139654771270648,
                                         4.51778825896976e-05,
                                         6.42331449320857e-08,
                                         0.0606863703151551,
                                         0.253712332254917,
                                         0.405916810307799,
                                         0.392317043662598,
                                         0.25,
                                         0.102155716825945,
                                         0.023310747054437,
                                         0.00202350571368614,
                                         1.58821699609381e-05,
                                         0.00147262361356521,
                                         0.0339871122101972,
                                         0.145910946138442,
                                         0.300290217691775,
                                         0.375,
                                         0.300290217691775,
                                         0.145910946138442,
                                         0.0339871122101972,
                                         0.00147262361356522,
                                         1.58821699609379e-05,
                                         0.00202350571368614,
                                         0.023310747054437,
                                         0.102155716825945,
                                         0.25,
                                         0.392317043662598,
                                         0.4059168103078,
                                         0.253712332254917,
                                         0.0606863703151553,
                                         6.42331449320848e-08,
                                         4.51778825896976e-05,
                                         0.00139654771270648,
                                         0.0130321309170995,
                                         0.0625,
                                         0.192204890902582,
                                         0.423464948786615,
                                         0.71023187193861,
                                         0.937825059668174,

                                         0.948825506768084,
                                         0.756230840330861,
                                         0.497171826420162,
                                         0.263842064088064,
                                         0.108885521198418,
                                         0.0327985583797788,
                                         0.00644169887711723,
                                         0.000660210219576795,
                                         2.07204650418815e-05,
                                         2.89739338172986e-08,
                                         0.0501708686715405,
                                         0.218852067812284,
                                         0.37962992106336,
                                         0.417174854466607,
                                         0.322664570217423,
                                         0.177089817891441,
                                         0.0651847966300056,
                                         0.0138340168664827,
                                         0.00114557416595092,
                                         8.76722549645076e-06,
                                         0.000994828360945484,
                                         0.0237507972258629,
                                         0.108704025430418,
                                         0.247356585938207,
                                         0.358561532312939,
                                         0.358561532312939,
                                         0.247356585938207,
                                         0.108704025430418,
                                         0.0237507972258629,
                                         0.000994828360945484,
                                         8.76722549645076e-06,
                                         0.00114557416595093,
                                         0.0138340168664827,
                                         0.0651847966300056,
                                         0.177089817891441,
                                         0.322664570217423,
                                         0.417174854466607,
                                         0.37962992106336,
                                         0.218852067812284,
                                         0.0501708686715405,
                                         2.89739338172986e-08,
                                         2.07204650418815e-05,
                                         0.000660210219576795,
                                         0.00644169887711723,
                                         0.0327985583797788,
                                         0.108885521198418,
                                         0.263842064088064,
                                         0.497171826420162,
                                         0.756230840330861,
                                         0.948825506768084,

                                         0.957163157609433,
                                         0.792547401831024,
                                         0.560037437020119,
                                         0.332828912382372,
                                         0.162356578933021,
                                         0.0625,
                                         0.0177933723750071,
                                         0.00334282089562565,
                                         0.000331403274986382,
                                         1.01678975655281e-05,
                                         1.40417360773867e-08,
                                         0.0421361327033732,
                                         0.189730310238456,
                                         0.349391218104416,
                                         0.421457758200314,
                                         0.373660316159399,
                                         0.25,
                                         0.123700417370738,
                                         0.0422376827601822,
                                         0.00849927444658239,
                                         0.000679578569181923,
                                         5.10354665408531e-06,
                                         0.000695592098803157,
                                         0.0170325414637733,
                                         0.0817406671538966,
                                         0.200132825761506,
                                         0.322489315161834,
                                         0.375,
                                         0.322489315161835,
                                         0.200132825761506,
                                         0.0817406671538966,
                                         0.0170325414637733,
                                         0.000695592098803164,
                                         5.10354665408523e-06,
                                         0.000679578569181923,
                                         0.0084992744465824,
                                         0.0422376827601822,
                                         0.123700417370738,
                                         0.25,
                                         0.373660316159399,
                                         0.421457758200314,
                                         0.349391218104416,
                                         0.189730310238456,
                                         0.0421361327033734,
                                         1.40417360773864e-08,
                                         1.01678975655281e-05,
                                         0.000331403274986383,
                                         0.00334282089562565,
                                         0.017793372375007,
                                         0.0625,
                                         0.162356578933021,
                                         0.332828912382372,
                                         0.560037437020119,
                                         0.792547401831024,
                                         0.957163157609433,

                                         0.963628156246642,
                                         0.82158929703109,
                                         0.613306488185647,
                                         0.396767171835268,
                                         0.218781411467788,
                                         0.100196024299947,
                                         0.0365972768109547,
                                         0.00998185891788715,
                                         0.00181276837592541,
                                         0.000175196853729486,
                                         5.28252824501172e-06,
                                         7.22543390530333e-09,
                                         0.0358680763355925,
                                         0.165486102649272,
                                         0.318933204380947,
                                         0.412617239546012,
                                         0.404455423159802,
                                         0.311573168711773,
                                         0.18830384666682,
                                         0.086391530263243,
                                         0.0278901378914638,
                                         0.00539043865617271,
                                         0.00041961891401255,
                                         3.10588456971262e-06,
                                         0.000500654307762337,
                                         0.0124996988773801,
                                         0.0621946719235032,
                                         0.16091268235133,
                                         0.28038977619128,
                                         0.363329683510505,
                                         0.363329683510505,
                                         0.28038977619128,
                                         0.16091268235133,
                                         0.0621946719235033,
                                         0.0124996988773801,
                                         0.000500654307762331,
                                         3.10588456971268e-06,
                                         0.000419618914012548,
                                         0.0053904386561727,
                                         0.0278901378914637,
                                         0.086391530263243,
                                         0.18830384666682,
                                         0.311573168711773,
                                         0.404455423159802,
                                         0.412617239546012,
                                         0.318933204380948,
                                         0.165486102649272,
                                         0.0358680763355923,
                                         7.2254339053035e-09,
                                         5.2825282450117e-06,
                                         0.000175196853729486,
                                         0.00181276837592541,
                                         0.00998185891788714,
                                         0.0365972768109547,
                                         0.100196024299947,
                                         0.218781411467788,
                                         0.396767171835268,
                                         0.613306488185647,
                                         0.82158929703109,
                                         0.963628156246642,

                                         0.968739398477836,
                                         0.845104961272685,
                                         0.658403883858058,
                                         0.45471803335804,
                                         0.275134671338053,
                                         0.143267486084963,
                                         0.0625,
                                         0.0219183882197575,
                                         0.00578209093997213,
                                         0.00102262438228394,
                                         9.68809779749206e-05,
                                         2.8815264592955e-06,
                                         3.91174436927972e-09,
                                         0.0308892831661543,
                                         0.145260867263323,
                                         0.29006071149665,
                                         0.396091625803099,
                                         0.419025267680675,
                                         0.358404022569804,
                                         0.25,
                                         0.140185586389945,
                                         0.0607449264039438,
                                         0.0187837648739302,
                                         0.0035185357901953,
                                         0.000268228309817231,
                                         1.96286116031195e-06,
                                         0.000369351583105619,
                                         0.00936306162771609,
                                         0.0479199878771224,
                                         0.129383951582647,
                                         0.239313043637356,
                                         0.336224516735531,
                                         0.375,
                                         0.336224516735531,
                                         0.239313043637356,
                                         0.129383951582647,
                                         0.0479199878771225,
                                         0.00936306162771607,
                                         0.000369351583105614,
                                         1.96286116031199e-06,
                                         0.000268228309817232,
                                         0.00351853579019529,
                                         0.0187837648739302,
                                         0.0607449264039438,
                                         0.140185586389945,
                                         0.25,
                                         0.358404022569804,
                                         0.419025267680675,
                                         0.396091625803099,
                                         0.29006071149665,
                                         0.145260867263323,
                                         0.0308892831661541,
                                         3.91174436927983e-09,
                                         2.88152645929552e-06,
                                         9.68809779749204e-05,
                                         0.00102262438228394,
                                         0.00578209093997213,
                                         0.0219183882197575,
                                         0.0625,
                                         0.143267486084963,
                                         0.275134671338053,
                                         0.45471803335804,
                                         0.658403883858058,
                                         0.845104961272685,
                                         0.968739398477836,

                                         0.972848530221262,
                                         0.864370492814952,
                                         0.696667601066622,
                                         0.506572883344894,
                                         0.329469809434844,
                                         0.189237493810458,
                                         0.0942161184643902,
                                         0.0395578262389152,
                                         0.0134332716421843,
                                         0.00345108912496521,
                                         0.000597628029965846,
                                         5.57239314531811e-05,
                                         1.63940231404558e-06,
                                         2.21214786148312e-09,
                                         0.0268718427671117,
                                         0.128308765621495,
                                         0.263535811418169,
                                         0.375533932253762,
                                         0.421609854832061,
                                         0.390715671858488,
                                         0.303362574985478,
                                         0.19656926202174,
                                         0.104099378339254,
                                         0.0431500227355117,
                                         0.0128986332602489,
                                         0.00235693554897638,
                                         0.000176705119622931,
                                         1.28139171657249e-06,
                                         0.00027834340776159,
                                         0.00714239704161622,
                                         0.0373839280347797,
                                         0.104396923111129,
                                         0.202319223872617,
                                         0.302514184349616,
                                         0.366294218289476,
                                         0.366294218289476,
                                         0.302514184349616,
                                         0.202319223872617,
                                         0.104396923111129,
                                         0.0373839280347797,
                                         0.0071423970416162,
                                         0.00027834340776159,
                                         1.28139171657249e-06,
                                         0.000176705119622932,
                                         0.00235693554897638,
                                         0.0128986332602489,
                                         0.0431500227355117,
                                         0.104099378339254,
                                         0.19656926202174,
                                         0.303362574985478,
                                         0.390715671858488,
                                         0.421609854832061,
                                         0.375533932253762,
                                         0.263535811418169,
                                         0.128308765621494,
                                         0.0268718427671117,
                                         2.21214786148312e-09,
                                         1.63940231404559e-06,
                                         5.57239314531811e-05,
                                         0.000597628029965845,
                                         0.00345108912496521,
                                         0.0134332716421843,
                                         0.0395578262389152,
                                         0.0942161184643902,
                                         0.189237493810458,
                                         0.329469809434844,
                                         0.506572883344894,
                                         0.696667601066622,
                                         0.864370492814952,
                                         0.972848530221262,

                                         0.976200441158322,
                                         0.880326290422158,
                                         0.729259455604646,
                                         0.552649697612692,
                                         0.380674675588307,
                                         0.236112896100805,
                                         0.130116619082459,
                                         0.0625,
                                         0.025447497834447,
                                         0.00842047525048224,
                                         0.0021174924266149,
                                         0.000360483259542264,
                                         3.31811750977195e-05,
                                         9.67580749819504e-07,
                                         1.29923523932817e-09,
                                         0.0235850167429701,
                                         0.114015671706972,
                                         0.239576647632575,
                                         0.353279729526074,
                                         0.415844486256976,
                                         0.410425110129251,
                                         0.346115334104785,
                                         0.25,
                                         0.153065388605938,
                                         0.0775072576747713,
                                         0.031014507363282,
                                         0.00902269551203407,
                                         0.00161602967081806,
                                         0.00011953258837773,
                                         8.60420174467677e-07,
                                         0.000213680379297726,
                                         0.00553753770174161,
                                         0.029514685916863,
                                         0.0846873940896574,
                                         0.17034883836482,
                                         0.26753426084469,
                                         0.345255160372371,
                                         0.375,
                                         0.345255160372371,
                                         0.26753426084469,
                                         0.17034883836482,
                                         0.0846873940896573,
                                         0.029514685916863,
                                         0.00553753770174161,
                                         0.00021368037929773,
                                         8.60420174467653e-07,
                                         0.00011953258837773,
                                         0.00161602967081806,
                                         0.00902269551203409,
                                         0.031014507363282,
                                         0.0775072576747713,
                                         0.153065388605938,
                                         0.25,
                                         0.346115334104785,
                                         0.410425110129251,
                                         0.415844486256976,
                                         0.353279729526074,
                                         0.239576647632575,
                                         0.114015671706972,
                                         0.0235850167429703,
                                         1.29923523932812e-09,
                                         9.67580749819504e-07,
                                         3.31811750977194e-05,
                                         0.000360483259542265,
                                         0.0021174924266149,
                                         0.00842047525048224,
                                         0.025447497834447,
                                         0.0625,
                                         0.130116619082459,
                                         0.236112896100805,
                                         0.380674675588307,
                                         0.552649697612692,
                                         0.729259455604646,
                                         0.880326290422158,
                                         0.976200441158322,

                                         0.978969785690217,
                                         0.893673397272717,
                                         0.757152226754828,
                                         0.593456315631769,
                                         0.428214343373989,
                                         0.282442536721729,
                                         0.168614463603698,
                                         0.0898579156271391,
                                         0.0419228038340917,
                                         0.0166470286032193,
                                         0.00539291267699367,
                                         0.00133258882534111,
                                         0.000223704396379093,
                                         2.03738798686383e-05,
                                         5.89796378179132e-07,
                                         7.88769739328845e-10,
                                         0.0208628923757585,
                                         0.101887202779402,
                                         0.218130216064584,
                                         0.330765487306574,
                                         0.40455720555101,
                                         0.419965307738311,
                                         0.378063971082177,
                                         0.297057024660599,
                                         0.202902228571419,
                                         0.118791739524615,
                                         0.0580310005133277,
                                         0.0225682258430177,
                                         0.00642189306734854,
                                         0.0011315170388394,
                                         8.27715856135834e-05,
                                         5.92195351449487e-07,
                                         0.000166728949904058,
                                         0.00435603856588915,
                                         0.0235656662618796,
                                         0.069132599597929,
                                         0.143327636406642,
                                         0.234168242349638,
                                         0.317882797186291,
                                         0.368260027306751,
                                         0.368260027306751,
                                         0.317882797186291,
                                         0.234168242349638,
                                         0.143327636406642,
                                         0.0691325995979289,
                                         0.0235656662618797,
                                         0.00435603856588915,
                                         0.000166728949904055,
                                         5.92195351449506e-07,
                                         8.27715856135834e-05,
                                         0.0011315170388394,
                                         0.00642189306734855,
                                         0.0225682258430177,
                                         0.0580310005133277,
                                         0.118791739524616,
                                         0.202902228571419,
                                         0.297057024660599,
                                         0.378063971082177,
                                         0.419965307738311,
                                         0.40455720555101,
                                         0.330765487306574,
                                         0.218130216064584,
                                         0.101887202779402,
                                         0.0208628923757583,
                                         7.88769739328878e-10,
                                         5.89796378179132e-07,
                                         2.03738798686382e-05,
                                         0.000223704396379094,
                                         0.00133258882534111,
                                         0.00539291267699367,
                                         0.0166470286032193,
                                         0.0419228038340917,
                                         0.089857915627139,
                                         0.168614463603698,
                                         0.282442536721729,
                                         0.428214343373989,
                                         0.593456315631769,
                                         0.757152226754828,
                                         0.893673397272717,
                                         0.978969785690217,

                                         0.981283765069248,
                                         0.904940768841663,
                                         0.781146310330376,
                                         0.629559786657607,
                                         0.471925607363362,
                                         0.327250544426615,
                                         0.208352828197386,
                                         0.120552174142795,
                                         0.0625,
                                         0.0284671335350765,
                                         0.011072312449724,
                                         0.00352451527349441,
                                         0.000858329373663579,
                                         0.000142421134654031,
                                         1.28569940301258e-05,
                                         3.69938913539379e-07,
                                         4.93080516932981e-10,
                                         0.0185838380551658,
                                         0.0915287668403439,
                                         0.199018817372829,
                                         0.308838430219542,
                                         0.389833038251081,
                                         0.421691768760169,
                                         0.400146593900373,
                                         0.336145430098749,
                                         0.25,
                                         0.163347148322491,
                                         0.0922440984879453,
                                         0.0437627528934554,
                                         0.0166252757970957,
                                         0.00464515347184419,
                                         0.000807416591548383,
                                         5.85211083071584e-05,
                                         4.16578667723622e-07,
                                         0.000131979803837472,
                                         0.0034715732707722,
                                         0.0190145987112168,
                                         0.0568142085163538,
                                         0.120757749214798,
                                         0.203770418646267,
                                         0.288184166964572,
                                         0.351488113900887,
                                         0.375,
                                         0.351488113900887,
                                         0.288184166964572,
                                         0.203770418646266,
                                         0.120757749214798,
                                         0.0568142085163538,
                                         0.0190145987112168,
                                         0.00347157327077219,
                                         0.000131979803837475,
                                         4.16578667723608e-07,
                                         5.85211083071588e-05,
                                         0.000807416591548383,
                                         0.0046451534718442,
                                         0.0166252757970957,
                                         0.0437627528934555,
                                         0.0922440984879453,
                                         0.163347148322491,
                                         0.25,
                                         0.336145430098749,
                                         0.400146593900373,
                                         0.421691768760169,
                                         0.389833038251081,
                                         0.308838430219541,
                                         0.199018817372829,
                                         0.0915287668403437,
                                         0.018583838055166,
                                         4.93080516932958e-10,
                                         3.69938913539382e-07,
                                         1.28569940301258e-05,
                                         0.000142421134654031,
                                         0.00085832937366358,
                                         0.00352451527349441,
                                         0.011072312449724,
                                         0.0284671335350765,
                                         0.0625,
                                         0.120552174142795,
                                         0.208352828197386,
                                         0.327250544426615,
                                         0.471925607363362,
                                         0.629559786657607,
                                         0.781146310330376,
                                         0.904940768841663,
                                         0.981283765069248,

                                         0.983236756679994,
                                         0.914532317018973,
                                         0.801895219265578,
                                         0.661518531045338,
                                         0.511870509606053,
                                         0.369933102779203,
                                         0.248263361905588,
                                         0.153510986037566,
                                         0.086544347783499,
                                         0.0438522106105466,
                                         0.0195772019850231,
                                         0.00748383718790605,
                                         0.00234744419560889,
                                         0.000564738145279569,
                                         9.27934517993002e-05,
                                         8.31490414763463e-06,
                                         2.38027731473621e-07,
                                         3.16362995197385e-10,
                                         0.016657122903031,
                                         0.0826258943481487,
                                         0.182017238592954,
                                         0.287969064000248,
                                         0.373157224403275,
                                         0.417639153522556,
                                         0.413785767007172,
                                         0.366946071633453,
                                         0.292069963335031,
                                         0.207904211597465,
                                         0.131041188355527,
                                         0.0718424935168508,
                                         0.0332688085418495,
                                         0.0123946812041168,
                                         0.0034106212422788,
                                         0.000586114320703499,
                                         4.21531878347225e-05,
                                         2.98788430287531e-07,
                                         0.000105821312182055,
                                         0.00279939741731256,
                                         0.0154931129166166,
                                         0.0470089902603361,
                                         0.102012846641275,
                                         0.176811490960782,
                                         0.258624540382482,
                                         0.328924551988431,
                                         0.369629266673458,
                                         0.369629266673458,
                                         0.328924551988431,
                                         0.258624540382483,
                                         0.176811490960782,
                                         0.102012846641275,
                                         0.0470089902603361,
                                         0.0154931129166166,
                                         0.00279939741731256,
                                         0.000105821312182055,
                                         2.98788430287531e-07,
                                         4.21531878347225e-05,
                                         0.000586114320703499,
                                         0.0034106212422788,
                                         0.0123946812041168,
                                         0.0332688085418495,
                                         0.0718424935168508,
                                         0.131041188355527,
                                         0.207904211597465,
                                         0.292069963335031,
                                         0.366946071633453,
                                         0.413785767007172,
                                         0.417639153522556,
                                         0.373157224403275,
                                         0.287969064000248,
                                         0.182017238592954,
                                         0.0826258943481487,
                                         0.016657122903031,
                                         3.16362995197385e-10,
                                         2.38027731473621e-07,
                                         8.31490414763463e-06,
                                         9.27934517993e-05,
                                         0.000564738145279569,
                                         0.00234744419560889,
                                         0.00748383718790605,
                                         0.0195772019850231,
                                         0.0438522106105466,
                                         0.086544347783499,
                                         0.153510986037566,
                                         0.248263361905588,
                                         0.369933102779203,
                                         0.511870509606053,
                                         0.661518531045338,
                                         0.801895219265578,
                                         0.914532317018973,
                                         0.983236756679994,

                                         0.984899953029972,
                                         0.922760044669914,
                                         0.819931333641154,
                                         0.689850145839167,
                                         0.54823828179355,
                                         0.410158677501314,
                                         0.287555336838789,
                                         0.18777944524375,
                                         0.113304977054721,
                                         0.0625,
                                         0.031063851348551,
                                         0.0136355031922754,
                                         0.00513674983591187,
                                         0.00159129295063405,
                                         0.000378885000564069,
                                         6.17406716507828e-05,
                                         5.49759347614354e-06,
                                         1.56695472160065e-07,
                                         2.07763543136105e-10,
                                         0.0150140001072802,
                                         0.0749274045688941,
                                         0.166892284449326,
                                         0.268391587469536,
                                         0.355560790246075,
                                         0.409459973633892,
                                         0.420506941637598,
                                         0.389909505118779,
                                         0.327952205974863,
                                         0.25,
                                         0.171717166106752,
                                         0.105069189294469,
                                         0.0562026328914296,
                                         0.0255041334118897,
                                         0.00934722916158829,
                                         0.00253908473028187,
                                         0.00043214931504685,
                                         3.08762481171056e-05,
                                         2.18064529014483e-07,
                                         8.58285904552746e-05,
                                         0.00228151781760297,
                                         0.0127387350009962,
                                         0.0391574412893644,
                                         0.086474813798223,
                                         0.153285922502271,
                                         0.230598338796272,
                                         0.303606357150727,
                                         0.355961799515113,
                                         0.375,
                                         0.355961799515113,
                                         0.303606357150727,
                                         0.230598338796272,
                                         0.153285922502271,
                                         0.0864748137982231,
                                         0.0391574412893644,
                                         0.0127387350009962,
                                         0.00228151781760297,
                                         8.58285904552746e-05,
                                         2.18064529014483e-07,
                                         3.08762481171056e-05,
                                         0.00043214931504685,
                                         0.00253908473028187,
                                         0.00934722916158828,
                                         0.0255041334118897,
                                         0.0562026328914296,
                                         0.105069189294469,
                                         0.171717166106752,
                                         0.25,
                                         0.327952205974863,
                                         0.389909505118779,
                                         0.420506941637598,
                                         0.409459973633892,
                                         0.355560790246075,
                                         0.268391587469536,
                                         0.166892284449326,
                                         0.0749274045688941,
                                         0.0150140001072802,
                                         2.07763543136105e-10,
                                         1.56695472160065e-07,
                                         5.49759347614354e-06,
                                         6.17406716507828e-05,
                                         0.000378885000564068,
                                         0.00159129295063405,
                                         0.00513674983591187,
                                         0.0136355031922754,
                                         0.031063851348551,
                                         0.0625,
                                         0.113304977054721,
                                         0.18777944524375,
                                         0.287555336838789,
                                         0.410158677501314,
                                         0.54823828179355,
                                         0.689850145839167,
                                         0.819931333641154,
                                         0.922760044669914,
                                         0.984899953029972,

                                         0.986327860512721,
                                         0.929867610279814,
                                         0.835688738066185,
                                         0.715018742748247,
                                         0.581282398963667,
                                         0.447785693854471,
                                         0.325675404904635,
                                         0.222564200929315,
                                         0.14202686949942,
                                         0.0839419301967926,
                                         0.0454545881743675,
                                         0.022224450720858,
                                         0.009615962825226,
                                         0.00357756774757447,
                                         0.00109655228334194,
                                         0.000258787560779739,
                                         4.18717339254218e-05,
                                         3.70831623841347e-06,
                                         1.05303823435337e-07,
                                         1.39335296338273e-10,
                                         0.0136016392866203,
                                         0.0682318060044914,
                                         0.153421983867068,
                                         0.250194810443689,
                                         0.337742893909035,
                                         0.398446474028381,
                                         0.421740860012686,
                                         0.405881884216716,
                                         0.357310382893013,
                                         0.288030605023946,
                                         0.211952246801801,
                                         0.141343518166847,
                                         0.0843661829940535,
                                         0.0442025304251537,
                                         0.0197173957176091,
                                         0.00712630379450791,
                                         0.00191460882789382,
                                         0.000323186864309116,
                                         2.29614006452529e-05,
                                         1.61663201718834e-07,
                                         7.03383981216812e-05,
                                         0.0018775170112255,
                                         0.0105623828861997,
                                         0.032829966246245,
                                         0.0735896157720102,
                                         0.132953884116197,
                                         0.20480363690995,
                                         0.27757176903469,
                                         0.337094778719863,
                                         0.370620629803093,
                                         0.370620629803093,
                                         0.337094778719863,
                                         0.27757176903469,
                                         0.20480363690995,
                                         0.132953884116197,
                                         0.0735896157720102,
                                         0.0328299662462451,
                                         0.0105623828861997,
                                         0.00187751701122551,
                                         7.03383981216835e-05,
                                         1.61663201718826e-07,
                                         2.29614006452527e-05,
                                         0.000323186864309116,
                                         0.00191460882789381,
                                         0.00712630379450791,
                                         0.0197173957176091,
                                         0.0442025304251537,
                                         0.0843661829940535,
                                         0.141343518166847,
                                         0.211952246801801,
                                         0.288030605023946,
                                         0.357310382893013,
                                         0.405881884216716,
                                         0.421740860012686,
                                         0.39844647402838,
                                         0.337742893909035,
                                         0.250194810443689,
                                         0.153421983867068,
                                         0.0682318060044916,
                                         0.0136016392866205,
                                         1.39335296338264e-10,
                                         1.05303823435335e-07,
                                         3.70831623841347e-06,
                                         4.18717339254217e-05,
                                         0.000258787560779739,
                                         0.00109655228334194,
                                         0.00357756774757447,
                                         0.009615962825226,
                                         0.022224450720858,
                                         0.0454545881743675,
                                         0.0839419301967926,
                                         0.14202686949942,
                                         0.222564200929315,
                                         0.325675404904635,
                                         0.447785693854471,
                                         0.581282398963667,
                                         0.715018742748247,
                                         0.835688738066185,
                                         0.929867610279814,
                                         0.986327860512721,

                                         0.987562772450777,
                                         0.936047268900731,
                                         0.849522342638768,
                                         0.737432567578805,
                                         0.61128156665979,
                                         0.482799774140933,
                                         0.362259838915235,
                                         0.257239280713978,
                                         0.172017336987515,
                                         0.107635167802709,
                                         0.0625,
                                         0.0333121399975574,
                                         0.0160600902387191,
                                         0.00686337664521125,
                                         0.00252621004716946,
                                         0.000767242573778011,
                                         0.000179694363577921,
                                         2.88968142309429e-05,
                                         2.54730889060784e-06,
                                         7.21030052167969e-08,
                                         9.5235016487511e-11,
                                         0.0123789180919274,
                                         0.0623765956255645,
                                         0.141403935981224,
                                         0.23338031305211,
                                         0.320165535332481,
                                         0.385583515299763,
                                         0.418738119252704,
                                         0.415860253893511,
                                         0.380343634000019,
                                         0.321126590835349,
                                         0.25,
                                         0.178648938123861,
                                         0.11621549389281,
                                         0.0679278216161664,
                                         0.0349677052143945,
                                         0.0153709700392499,
                                         0.00548935297354984,
                                         0.00146092541462408,
                                         0.000244858340176451,
                                         1.73120884038902e-05,
                                         1.21562231811695e-07,
                                         5.81877998295614e-05,
                                         0.00155875128229548,
                                         0.00882631573094091,
                                         0.0276972971402299,
                                         0.062883850670601,
                                         0.115478497946276,
                                         0.181508126570497,
                                         0.252109267131133,
                                         0.315363444880936,
                                         0.359277163240525,
                                         0.375,
                                         0.359277163240525,
                                         0.315363444880936,
                                         0.252109267131133,
                                         0.181508126570497,
                                         0.115478497946276,
                                         0.062883850670601,
                                         0.0276972971402299,
                                         0.00882631573094088,
                                         0.00155875128229549,
                                         5.81877998295593e-05,
                                         1.21562231811701e-07,
                                         1.731208840389e-05,
                                         0.000244858340176453,
                                         0.00146092541462408,
                                         0.00548935297354984,
                                         0.0153709700392499,
                                         0.0349677052143945,
                                         0.0679278216161664,
                                         0.11621549389281,
                                         0.178648938123861,
                                         0.25,
                                         0.321126590835349,
                                         0.380343634000019,
                                         0.415860253893511,
                                         0.418738119252704,
                                         0.385583515299763,
                                         0.320165535332481,
                                         0.23338031305211,
                                         0.141403935981224,
                                         0.0623765956255647,
                                         0.0123789180919271,
                                         9.52350164875178e-11,
                                         7.2103005216796e-08,
                                         2.54730889060786e-06,
                                         2.88968142309429e-05,
                                         0.000179694363577921,
                                         0.000767242573778011,
                                         0.00252621004716946,
                                         0.00686337664521124,
                                         0.0160600902387191,
                                         0.0333121399975574,
                                         0.0625,
                                         0.107635167802709,
                                         0.172017336987515,
                                         0.257239280713978,
                                         0.362259838915235,
                                         0.482799774140933,
                                         0.61128156665979,
                                         0.737432567578805,
                                         0.849522342638768,
                                         0.936047268900731,
                                         0.987562772450777,

                                         0.988637905802623,
                                         0.941452188081014,
                                         0.86172343892003,
                                         0.757446708889307,
                                         0.638516433448405,
                                         0.515268095317059,
                                         0.397089668671067,
                                         0.291331837909333,
                                         0.202678182342491,
                                         0.1330292148741,
                                         0.0818449294572894,
                                         0.046805702009658,
                                         0.024608597203009,
                                         0.0117206857513232,
                                         0.00495556959667556,
                                         0.0018071093846198,
                                         0.000544499267745842,
                                         0.000126684635966719,
                                         2.0264097123248e-05,
                                         1.77909788674101e-06,
                                         5.0217873197012e-08,
                                         6.62260463567833e-11,
                                         0.0113134520877266,
                                         0.0572299375655949,
                                         0.130657962584692,
                                         0.217899316563263,
                                         0.303123800672948,
                                         0.371607316060555,
                                         0.412546153859739,
                                         0.420847108640458,
                                         0.397560746527003,
                                         0.348973120337425,
                                         0.28469413230294,
                                         0.215294040558759,
                                         0.150093502303166,
                                         0.0956040979251705,
                                         0.0548879692922594,
                                         0.0278304654188107,
                                         0.0120799817858145,
                                         0.00426968633940849,
                                         0.00112705070702954,
                                         0.000187737854631747,
                                         1.32176210181362e-05,
                                         9.25957230695827e-08,
                                         4.85494477009912e-05,
                                         0.00130460651449993,
                                         0.00742908154275892,
                                         0.0235066597432777,
                                         0.0539633949032718,
                                         0.100500107568826,
                                         0.160726602665763,
                                         0.227977514561274,
                                         0.292436287454013,
                                         0.3432955652823,
                                         0.371361195671353,
                                         0.371361195671353,
                                         0.3432955652823,
                                         0.292436287454013,
                                         0.227977514561274,
                                         0.160726602665763,
                                         0.100500107568826,
                                         0.0539633949032717,
                                         0.0235066597432776,
                                         0.00742908154275894,
                                         0.00130460651449993,
                                         4.85494477009912e-05,
                                         9.25957230695827e-08,
                                         1.32176210181362e-05,
                                         0.000187737854631746,
                                         0.00112705070702955,
                                         0.0042696863394085,
                                         0.0120799817858145,
                                         0.0278304654188107,
                                         0.0548879692922594,
                                         0.0956040979251705,
                                         0.150093502303166,
                                         0.215294040558759,
                                         0.28469413230294,
                                         0.348973120337425,
                                         0.397560746527003,
                                         0.420847108640458,
                                         0.412546153859739,
                                         0.371607316060555,
                                         0.303123800672948,
                                         0.217899316563263,
                                         0.130657962584692,
                                         0.0572299375655949,
                                         0.0113134520877266,
                                         6.62260463567833e-11,
                                         5.0217873197012e-08,
                                         1.779097886741e-06,
                                         2.02640971232481e-05,
                                         0.00012668463596672,
                                         0.000544499267745842,
                                         0.0018071093846198,
                                         0.00495556959667556,
                                         0.0117206857513232,
                                         0.024608597203009,
                                         0.046805702009658,
                                         0.0818449294572894,
                                         0.1330292148741,
                                         0.202678182342491,
                                         0.291331837909333,
                                         0.397089668671067,
                                         0.515268095317059,
                                         0.638516433448405,
                                         0.757446708889307,
                                         0.86172343892003,
                                         0.941452188081014,
                                         0.988637905802623,

                                         0.98957963827132,
                                         0.946205501457184,
                                         0.872532171803549,
                                         0.775368044619359,
                                         0.663256376374474,
                                         0.54530725093309,
                                         0.430052703731969,
                                         0.32449961623871,
                                         0.23351533465836,
                                         0.159608044320656,
                                         0.103084504877029,
                                         0.0625,
                                         0.0352729466130383,
                                         0.0183261399127774,
                                         0.0086365807879836,
                                         0.00361768567491482,
                                         0.00130856997137965,
                                         0.000391559290537087,
                                         9.0575838730981e-05,
                                         1.44209705381375e-05,
                                         1.26162047339539e-06,
                                         3.55243227738474e-08,
                                         4.67851087792772e-11,
                                         0.0103794649140167,
                                         0.0526842188067965,
                                         0.121025883074871,
                                         0.203675850040998,
                                         0.286796705247483,
                                         0.357059283330152,
                                         0.404017791812122,
                                         0.421772577345862,
                                         0.409620803698513,
                                         0.371636895038103,
                                         0.315366434887508,
                                         0.25,
                                         0.184475902818558,
                                         0.125929341913169,
                                         0.0787762353786826,
                                         0.0445334827814369,
                                         0.0222863078060078,
                                         0.00956794035135772,
                                         0.00335150308128954,
                                         0.000878380797756304,
                                         0.000145529788990674,
                                         1.02082362894807e-05,
                                         7.1367981152494e-08,
                                         4.08253998976406e-05,
                                         0.00110003597540698,
                                         0.00629515371211583,
                                         0.0200633035713485,
                                         0.0465048394580221,
                                         0.0876739660948632,
                                         0.142334626678522,
                                         0.205576637959076,
                                         0.269451045476462,
                                         0.324499578815295,
                                         0.361800210803866,
                                         0.375,
                                         0.361800210803866,
                                         0.324499578815295,
                                         0.269451045476462,
                                         0.205576637959076,
                                         0.142334626678522,
                                         0.0876739660948632,
                                         0.0465048394580221,
                                         0.0200633035713485,
                                         0.00629515371211585,
                                         0.00110003597540698,
                                         4.08253998976389e-05,
                                         7.13679811524985e-08,
                                         1.02082362894807e-05,
                                         0.000145529788990674,
                                         0.000878380797756304,
                                         0.00335150308128954,
                                         0.00956794035135771,
                                         0.0222863078060078,
                                         0.0445334827814368,
                                         0.0787762353786826,
                                         0.125929341913169,
                                         0.184475902818558,
                                         0.25,
                                         0.315366434887508,
                                         0.371636895038103,
                                         0.409620803698513,
                                         0.421772577345863,
                                         0.404017791812122,
                                         0.357059283330152,
                                         0.286796705247483,
                                         0.203675850040998,
                                         0.121025883074871,
                                         0.0526842188067965,
                                         0.0103794649140165,
                                         4.67851087792812e-11,
                                         3.55243227738474e-08,
                                         1.26162047339538e-06,
                                         1.44209705381375e-05,
                                         9.0575838730981e-05,
                                         0.000391559290537086,
                                         0.00130856997137965,
                                         0.00361768567491482,
                                         0.0086365807879836,
                                         0.0183261399127774,
                                         0.0352729466130383,
                                         0.0625,
                                         0.103084504877029,
                                         0.159608044320656,
                                         0.23351533465836,
                                         0.32449961623871,
                                         0.430052703731969,
                                         0.54530725093309,
                                         0.663256376374474,
                                         0.775368044619359,
                                         0.872532171803549,
                                         0.946205501457184,
                                         0.98957963827132,

                                         0.990409128565756,
                                         0.950407036507499,
                                         0.882147469739109,
                                         0.791460904496354,
                                         0.685752603149579,
                                         0.573061188566064,
                                         0.461113209573105,
                                         0.356506772687057,
                                         0.264135305871462,
                                         0.186912957640004,
                                         0.125805733226994,
                                         0.0801197185266472,
                                         0.0479598502347719,
                                         0.0267558553347938,
                                         0.0137573270919201,
                                         0.00642361342116988,
                                         0.00266879792712466,
                                         0.000958491844684126,
                                         0.000285064693066289,
                                         6.56071044979699e-05,
                                         1.04029350436764e-05,
                                         9.07270980692938e-07,
                                         2.5491785265624e-08,
                                         3.35323551868682e-11,
                                         0.00955623853431914,
                                         0.0486510576958959,
                                         0.112369937918321,
                                         0.19062114035203,
                                         0.271283454922385,
                                         0.342331289627009,
                                         0.393833603894487,
                                         0.419459227578494,
                                         0.417229187478252,
                                         0.389424314729441,
                                         0.341735444714437,
                                         0.281892815332709,
                                         0.218098766191249,
                                         0.157597465551605,
                                         0.105650216433282,
                                         0.0650655571267452,
                                         0.0362922343206795,
                                         0.017955735477376,
                                         0.00763515421589551,
                                         0.00265347506474134,
                                         0.000691089465848724,
                                         0.000113958849110374,
                                         7.96778470586254e-06,
                                         5.56047244909011e-08,
                                         3.45772616680083e-05,
                                         0.000933912520113709,
                                         0.00536772622247969,
                                         0.0172164627507237,
                                         0.0402448597587967,
                                         0.0766873028979659,
                                         0.126138959210348,
                                         0.185072967486644,
                                         0.247146336102371,
                                         0.304255184105354,
                                         0.348105501172171,
                                         0.371928849714623,
                                         0.371928849714623,
                                         0.348105501172171,
                                         0.304255184105354,
                                         0.247146336102371,
                                         0.185072967486644,
                                         0.126138959210348,
                                         0.0766873028979658,
                                         0.0402448597587967,
                                         0.0172164627507237,
                                         0.00536772622247967,
                                         0.000933912520113709,
                                         3.45772616680067e-05,
                                         5.5604724490905e-08,
                                         7.96778470586254e-06,
                                         0.000113958849110374,
                                         0.000691089465848724,
                                         0.00265347506474134,
                                         0.00763515421589552,
                                         0.017955735477376,
                                         0.0362922343206796,
                                         0.0650655571267452,
                                         0.105650216433282,
                                         0.157597465551605,
                                         0.218098766191248,
                                         0.281892815332709,
                                         0.341735444714437,
                                         0.389424314729441,
                                         0.417229187478252,
                                         0.419459227578494,
                                         0.393833603894487,
                                         0.342331289627008,
                                         0.271283454922385,
                                         0.19062114035203,
                                         0.112369937918321,
                                         0.0486510576958959,
                                         0.00955623853431892,
                                         3.35323551868712e-11,
                                         2.5491785265624e-08,
                                         9.07270980692945e-07,
                                         1.04029350436764e-05,
                                         6.56071044979699e-05,
                                         0.000285064693066289,
                                         0.000958491844684126,
                                         0.00266879792712467,
                                         0.00642361342116988,
                                         0.0137573270919201,
                                         0.0267558553347938,
                                         0.0479598502347719,
                                         0.0801197185266472,
                                         0.125805733226994,
                                         0.186912957640004,
                                         0.264135305871462,
                                         0.356506772687057,
                                         0.461113209573105,
                                         0.573061188566064,
                                         0.685752603149579,
                                         0.791460904496354,
                                         0.882147469739109,
                                         0.950407036507499,
                                         0.990409128565756,

                                         0.03125,
                                         0.15625,
                                         0.3125,
                                         0.3125,
                                         0.15625,
                                         0.03125,

                                         0.305134098813929,
                                         0.000421456741626533,
                                         0.408802176801953,
                                         0.00786448986471326,
                                         0.219076426276337,
                                         0.0587013515014408,
                                         0.0587013515014408,
                                         0.219076426276337,
                                         0.00786448986471326,
                                         0.408802176801953,
                                         0.000421456741626533,
                                         0.305134098813929,

                                         0.549981817580726,
                                         0.03125,
                                         1.81824192734013e-05,
                                         0.349284250579334,
                                         0.15625,
                                         0.000715749420666245,
                                         0.0887298334620744,
                                         0.3125,
                                         0.0112701665379259,
                                         0.0112701665379259,
                                         0.3125,
                                         0.0887298334620744,
                                         0.000715749420666245,
                                         0.15625,
                                         0.349284250579334,
                                         1.81824192734013e-05,
                                         0.03125,
                                         0.549981817580726,

                                         0.697816017789034,
                                         0.135002961144809,
                                         0.00391410135255554,
                                         1.61359115377922e-06,
                                         0.260328342032991,
                                         0.332484231602893,
                                         0.0397323558976338,
                                         0.000108131690971159,
                                         0.0388474004253279,
                                         0.327535820924673,
                                         0.161330529077436,
                                         0.00289849447052239,
                                         0.00289849447052239,
                                         0.161330529077436,
                                         0.327535820924673,
                                         0.038847400425328,
                                         0.000108131690971159,
                                         0.0397323558976338,
                                         0.332484231602893,
                                         0.260328342032991,
                                         1.61359115377921e-06,
                                         0.00391410135255554,
                                         0.135002961144809,
                                         0.697816017789034,

                                         0.78644687097721,
                                         0.269335877083235,
                                         0.03125,
                                         0.000654414550868918,
                                         2.27159409202062e-07,
                                         0.193540412132003,
                                         0.403994971973075,
                                         0.15625,
                                         0.0109071479388398,
                                         2.30764216904409e-05,
                                         0.0190517083915316,
                                         0.242391677108931,
                                         0.3125,
                                         0.0727159113450503,
                                         0.000937704918155274,
                                         0.000937704918155271,
                                         0.0727159113450503,
                                         0.3125,
                                         0.242391677108931,
                                         0.0190517083915316,
                                         2.30764216904408e-05,
                                         0.0109071479388398,
                                         0.15625,
                                         0.403994971973075,
                                         0.193540412132004,
                                         2.27159409202061e-07,
                                         0.000654414550868918,
                                         0.03125,
                                         0.269335877083235,
                                         0.78644687097721,

                                         0.842196202236558,
                                         0.395341042863498,
                                         0.0911043336944349,
                                         0.0079957584496648,
                                         0.000139478371613415,
                                         4.38883639124933e-08,
                                         0.14715346931811,
                                         0.40313351091672,
                                         0.280009758120111,
                                         0.0650377553651086,
                                         0.0034195572556818,
                                         6.27960278149708e-06,
                                         0.0102846075414971,
                                         0.164431829740637,
                                         0.344244720148905,
                                         0.211607674221776,
                                         0.0335345808518499,
                                         0.000359397412690162,
                                         0.000359397412690162,
                                         0.0335345808518499,
                                         0.211607674221776,
                                         0.344244720148905,
                                         0.164431829740637,
                                         0.0102846075414971,
                                         6.27960278149708e-06,
                                         0.0034195572556818,
                                         0.0650377553651086,
                                         0.280009758120111,
                                         0.40313351091672,
                                         0.14715346931811,
                                         4.38883639124933e-08,
                                         0.000139478371613415,
                                         0.0079957584496648,
                                         0.0911043336944349,
                                         0.395341042863498,
                                         0.842196202236558,

                                         0.879082114516863,
                                         0.500617815222906,
                                         0.171607971857582,
                                         0.03125,
                                         0.002313919518239,
                                         3.60487969159972e-05,
                                         1.06684506940694e-08,
                                         0.11476615467671,
                                         0.37149519410975,
                                         0.362634919079014,
                                         0.15625,
                                         0.02737512403488,
                                         0.00121446187189229,
                                         2.04294642030569e-06,
                                         0.00599319223620762,
                                         0.110270689575992,
                                         0.306522087784078,
                                         0.3125,
                                         0.129545977726207,
                                         0.0163657904225438,
                                         0.000156484955348201,
                                         0.0001564849553482,
                                         0.0163657904225438,
                                         0.129545977726207,
                                         0.3125,
                                         0.306522087784078,
                                         0.110270689575992,
                                         0.00599319223620765,
                                         2.04294642030567e-06,
                                         0.00121446187189229,
                                         0.02737512403488,
                                         0.15625,
                                         0.362634919079014,
                                         0.37149519410975,
                                         0.114766154676711,
                                         1.06684506940693e-08,
                                         3.60487969159972e-05,
                                         0.002313919518239,
                                         0.03125,
                                         0.171607971857582,
                                         0.500617815222906,
                                         0.879082114516863,

                                         0.904589380529803,
                                         0.585042404405521,
                                         0.258200580874888,
                                         0.0725389746531626,
                                         0.0113450072465114,
                                         0.000751420054528723,
                                         1.08616284922655e-05,
                                         3.08572560441647e-09,
                                         0.0916226087498726,
                                         0.331054022673763,
                                         0.401524761354074,
                                         0.250258069367259,
                                         0.0822106551024868,
                                         0.0120800205388372,
                                         0.00047986981079041,
                                         7.61633687107802e-07,
                                         0.00371204995982402,
                                         0.0749325280377545,
                                         0.249762620105905,
                                         0.34535421314063,
                                         0.238293080489951,
                                         0.077680597071767,
                                         0.00848031344367935,
                                         7.51960410876682e-05,
                                         7.51960410876682e-05,
                                         0.00848031344367935,
                                         0.077680597071767,
                                         0.238293080489951,
                                         0.34535421314063,
                                         0.249762620105905,
                                         0.0749325280377545,
                                         0.00371204995982402,
                                         7.61633687107802e-07,
                                         0.00047986981079041,
                                         0.0120800205388372,
                                         0.0822106551024869,
                                         0.250258069367259,
                                         0.401524761354074,
                                         0.331054022673762,
                                         0.0916226087498726,
                                         3.08572560441647e-09,
                                         1.08616284922655e-05,
                                         0.000751420054528723,
                                         0.0113450072465114,
                                         0.0725389746531626,
                                         0.258200580874888,
                                         0.585042404405521,
                                         0.904589380529803,

                                         0.922894997026383,
                                         0.652003905147058,
                                         0.341603125561172,
                                         0.12726399238635,
                                         0.03125,
                                         0.00440320892649167,
                                         0.000269972620664487,
                                         3.70388369077998e-06,
                                         1.02258397515478e-09,
                                         0.0746503132089526,
                                         0.291139833957761,
                                         0.409309116127218,
                                         0.324704492581161,
                                         0.15625,
                                         0.043144609953039,
                                         0.00563287546020995,
                                         0.000207369994494588,
                                         3.16052804784654e-07,
                                         0.00241529936998257,
                                         0.0520011627217691,
                                         0.196173793515064,
                                         0.331383623994174,
                                         0.3125,
                                         0.169100072158785,
                                         0.0470111167156726,
                                         0.00464402429522616,
                                         3.90733192927759e-05,
                                         3.90733192927754e-05,
                                         0.00464402429522616,
                                         0.0470111167156726,
                                         0.169100072158785,
                                         0.3125,
                                         0.331383623994174,
                                         0.196173793515064,
                                         0.0520011627217691,
                                         0.00241529936998259,
                                         3.1605280478465e-07,
                                         0.000207369994494588,
                                         0.00563287546020995,
                                         0.043144609953039,
                                         0.15625,
                                         0.324704492581161,
                                         0.409309116127218,
                                         0.291139833957761,
                                         0.0746503132089528,
                                         1.02258397515476e-09,
                                         3.70388369077998e-06,
                                         0.000269972620664486,
                                         0.00440320892649167,
                                         0.03125,
                                         0.12726399238635,
                                         0.341603125561172,
                                         0.652003905147058,
                                         0.922894997026383,

                                         0.936446431116567,
                                         0.705209218530758,
                                         0.417477561189361,
                                         0.189094999720692,
                                         0.0625478905957517,
                                         0.0139578473407194,
                                         0.00182494812670354,
                                         0.00010582853965376,
                                         1.39797489669503e-06,
                                         3.78015257903514e-10,
                                         0.0618953782575821,
                                         0.255108109000515,
                                         0.398471326154006,
                                         0.373735321836858,
                                         0.231688153013334,
                                         0.0942035551952969,
                                         0.0230837537520685,
                                         0.00277190839961517,
                                         9.66124507259321e-05,
                                         1.42979592796975e-07,
                                         0.00163641516368694,
                                         0.0369139515296794,
                                         0.152132150350388,
                                         0.295466492492801,
                                         0.34328511951689,
                                         0.254317434338008,
                                         0.116794484070877,
                                         0.0290412253669764,
                                         0.00267071051342544,
                                         2.16321045555329e-05,
                                         2.16321045555329e-05,
                                         0.00267071051342545,
                                         0.0290412253669764,
                                         0.116794484070877,
                                         0.254317434338008,
                                         0.34328511951689,
                                         0.295466492492801,
                                         0.152132150350388,
                                         0.0369139515296794,
                                         0.00163641516368694,
                                         1.42979592796975e-07,
                                         9.66124507259325e-05,
                                         0.00277190839961517,
                                         0.0230837537520685,
                                         0.0942035551952969,
                                         0.231688153013334,
                                         0.373735321836858,
                                         0.398471326154006,
                                         0.255108109000515,
                                         0.0618953782575821,
                                         3.78015257903514e-10,
                                         1.39797489669504e-06,
                                         0.00010582853965376,
                                         0.00182494812670354,
                                         0.0139578473407194,
                                         0.0625478905957517,
                                         0.189094999720692,
                                         0.417477561189361,
                                         0.705209218530758,
                                         0.936446431116567,

                                         0.946743794452276,
                                         0.74779328026935,
                                         0.484474947428454,
                                         0.252799556244087,
                                         0.103059341804127,
                                         0.03125,
                                         0.00649864531500619,
                                         0.000803787754037363,
                                         4.47142545506336e-05,
                                         5.74167958437527e-07,
                                         1.52853718181816e-10,
                                         0.0520968157857863,
                                         0.223770607808367,
                                         0.377812447958322,
                                         0.400146780691424,
                                         0.296486185644472,
                                         0.15625,
                                         0.0564736353000043,
                                         0.0126951657079414,
                                         0.00143344510217874,
                                         4.79686480354526e-05,
                                         6.94444117960244e-08,
                                         0.00114670018686031,
                                         0.026784559979405,
                                         0.117853149344396,
                                         0.253350834117938,
                                         0.341178419109555,
                                         0.3125,
                                         0.196303772826836,
                                         0.0802038754845725,
                                         0.0183812959120985,
                                         0.0016030091268839,
                                         1.26199778116212e-05,
                                         1.2619977811621e-05,
                                         0.0016030091268839,
                                         0.0183812959120985,
                                         0.0802038754845726,
                                         0.196303772826836,
                                         0.3125,
                                         0.341178419109555,
                                         0.253350834117938,
                                         0.117853149344396,
                                         0.0267845599794049,
                                         0.00114670018686032,
                                         6.9444411796023e-08,
                                         4.79686480354526e-05,
                                         0.00143344510217875,
                                         0.0126951657079414,
                                         0.0564736353000043,
                                         0.15625,
                                         0.296486185644472,
                                         0.400146780691424,
                                         0.377812447958322,
                                         0.223770607808367,
                                         0.0520968157857866,
                                         1.52853718181812e-10,
                                         5.74167958437527e-07,
                                         4.47142545506337e-05,
                                         0.000803787754037364,
                                         0.00649864531500619,
                                         0.03125,
                                         0.103059341804127,
                                         0.252799556244087,
                                         0.484474947428454,
                                         0.74779328026935,
                                         0.946743794452276,

                                         0.954743810235046,
                                         0.782201179063093,
                                         0.542746396770247,
                                         0.314897827762822,
                                         0.149628053000461,
                                         0.0563719569711638,
                                         0.0160070375468406,
                                         0.00315510839466618,
                                         0.000374048480890713,
                                         2.01561637663631e-05,
                                         2.53251650716092e-07,
                                         6.66162092530223e-11,
                                         0.0444217300579791,
                                         0.196940589839984,
                                         0.352800457076999,
                                         0.409346720362229,
                                         0.345766792336634,
                                         0.219120336643917,
                                         0.102951196320571,
                                         0.0341337526161048,
                                         0.0071935994751735,
                                         0.000775203449815615,
                                         2.51463829714782e-05,
                                         3.57940884802515e-08,
                                         0.000826730723023241,
                                         0.0198340769432118,
                                         0.0917320967983715,
                                         0.212849658140572,
                                         0.319604973226236,
                                         0.340692248491599,
                                         0.264857224025909,
                                         0.147711320425898,
                                         0.0553381457783124,
                                         0.0119256897408005,
                                         0.000998754519088418,
                                         7.69312324732106e-06,
                                         7.6931232473212e-06,
                                         0.000998754519088415,
                                         0.0119256897408005,
                                         0.0553381457783123,
                                         0.147711320425898,
                                         0.264857224025909,
                                         0.340692248491599,
                                         0.319604973226236,
                                         0.212849658140572,
                                         0.0917320967983715,
                                         0.0198340769432118,
                                         0.000826730723023231,
                                         3.57940884802524e-08,
                                         2.5146382971478e-05,
                                         0.000775203449815614,
                                         0.00719359947517349,
                                         0.0341337526161048,
                                         0.102951196320571,
                                         0.219120336643917,
                                         0.345766792336634,
                                         0.40934672036223,
                                         0.352800457076999,
                                         0.196940589839984,
                                         0.0444217300579788,
                                         6.66162092530243e-11,
                                         2.53251650716091e-07,
                                         2.01561637663631e-05,
                                         0.000374048480890713,
                                         0.00315510839466618,
                                         0.0160070375468406,
                                         0.0563719569711638,
                                         0.149628053000461,
                                         0.314897827762822,
                                         0.542746396770247,
                                         0.782201179063093,
                                         0.954743810235046,

                                         0.961078149449,
                                         0.810285960455949,
                                         0.593083006006198,
                                         0.373402930858594,
                                         0.199265288495983,
                                         0.0881423348296147,
                                         0.03125,
                                         0.00843355669109067,
                                         0.00159443253378056,
                                         0.000182871142914133,
                                         9.61165431070395e-06,
                                         1.18721196463647e-07,
                                         3.09359233218841e-11,
                                         0.0383062451441797,
                                         0.174095004083681,
                                         0.3266043892593,
                                         0.406575512497231,
                                         0.379346914210353,
                                         0.275625756276742,
                                         0.15625,
                                         0.0674241576433343,
                                         0.0209382920309579,
                                         0.00419876619684904,
                                         0.000436346618321083,
                                         1.38140263141593e-05,
                                         1.94040422297892e-08,
                                         0.000610717627026378,
                                         0.0149621599909454,
                                         0.0719430002230247,
                                         0.177078039513285,
                                         0.288869340780982,
                                         0.344758543871026,
                                         0.3125,
                                         0.215615650688193,
                                         0.109985731947944,
                                         0.0385618797911274,
                                         0.00792364623884607,
                                         0.00064294272191476,
                                         4.86834481632029e-06,
                                         4.86834481632039e-06,
                                         0.000642942721914762,
                                         0.00792364623884606,
                                         0.0385618797911274,
                                         0.109985731947944,
                                         0.215615650688193,
                                         0.3125,
                                         0.344758543871026,
                                         0.288869340780982,
                                         0.177078039513285,
                                         0.0719430002230248,
                                         0.0149621599909454,
                                         0.00061071762702637,
                                         1.94040422297897e-08,
                                         1.38140263141593e-05,
                                         0.000436346618321082,
                                         0.00419876619684905,
                                         0.0209382920309579,
                                         0.0674241576433343,
                                         0.15625,
                                         0.275625756276742,
                                         0.379346914210353,
                                         0.406575512497231,
                                         0.3266043892593,
                                         0.174095004083681,
                                         0.0383062451441794,
                                         3.09359233218852e-11,
                                         1.18721196463647e-07,
                                         9.61165431070392e-06,
                                         0.000182871142914133,
                                         0.00159443253378056,
                                         0.00843355669109067,
                                         0.03125,
                                         0.0881423348296147,
                                         0.199265288495983,
                                         0.373402930858594,
                                         0.593083006006198,
                                         0.810285960455949,
                                         0.961078149449,

                                         0.966176641946492,
                                         0.833441105376914,
                                         0.636475978417715,
                                         0.427368415918919,
                                         0.249614339733664,
                                         0.124812759375145,
                                         0.0521983181562325,
                                         0.0176417036768476,
                                         0.0045732742529975,
                                         0.000836460079688217,
                                         9.34412626268675e-05,
                                         4.81451103712612e-06,
                                         5.86620086634389e-08,
                                         1.51711216295204e-11,
                                         0.0333594413738503,
                                         0.15464693719019,
                                         0.300958113244535,
                                         0.396022337129878,
                                         0.3992773485059,
                                         0.322123672176564,
                                         0.210089001540789,
                                         0.109580612810338,
                                         0.0442999869459339,
                                         0.013073145226385,
                                         0.00252093383669489,
                                         0.000254547102080275,
                                         7.9037015269107e-06,
                                         1.0984883699268e-08,
                                         0.000460724170078617,
                                         0.0114780396733569,
                                         0.0569233020563524,
                                         0.146790156374649,
                                         0.255469940068353,
                                         0.332541835293092,
                                         0.338228434382118,
                                         0.272261929433675,
                                         0.171648471956268,
                                         0.0817287663860094,
                                         0.0272047154772325,
                                         0.0053832446682804,
                                         0.000425955396003506,
                                         3.18150952403269e-06,
                                         3.18150952403269e-06,
                                         0.000425955396003508,
                                         0.0053832446682804,
                                         0.0272047154772325,
                                         0.0817287663860093,
                                         0.171648471956268,
                                         0.272261929433675,
                                         0.338228434382118,
                                         0.332541835293092,
                                         0.255469940068353,
                                         0.146790156374649,
                                         0.0569233020563524,
                                         0.0114780396733568,
                                         0.000460724170078617,
                                         1.0984883699268e-08,
                                         7.90370152691075e-06,
                                         0.000254547102080275,
                                         0.00252093383669489,
                                         0.013073145226385,
                                         0.0442999869459338,
                                         0.109580612810338,
                                         0.210089001540789,
                                         0.322123672176564,
                                         0.3992773485059,
                                         0.396022337129878,
                                         0.300958113244535,
                                         0.15464693719019,
                                         0.0333594413738503,
                                         1.51711216295204e-11,
                                         5.86620086634393e-08,
                                         4.81451103712612e-06,
                                         9.34412626268674e-05,
                                         0.000836460079688216,
                                         0.0045732742529975,
                                         0.0176417036768476,
                                         0.0521983181562325,
                                         0.124812759375145,
                                         0.249614339733664,
                                         0.427368415918919,
                                         0.636475978417715,
                                         0.833441105376914,
                                         0.966176641946492,

                                         0.970339586555521,
                                         0.852716349532832,
                                         0.673911063431405,
                                         0.476499468897082,
                                         0.299014661083007,
                                         0.164588556080661,
                                         0.0781476571863817,
                                         0.03125,
                                         0.0101638057815719,
                                         0.00255076679373569,
                                         0.000454231587654232,
                                         4.96713972356689e-05,
                                         2.51834196726985e-06,
                                         3.03465290072794e-08,
                                         7.80027186169198e-12,
                                         0.0293042730140078,
                                         0.138049704446632,
                                         0.276741960866203,
                                         0.380751143578055,
                                         0.408300072526502,
                                         0.357621700100721,
                                         0.259844809480385,
                                         0.15625,
                                         0.0764184602643757,
                                         0.0293485422837327,
                                         0.00831630419480335,
                                         0.00155405931153298,
                                         0.000153314165652248,
                                         4.68617110406112e-06,
                                         6.4571748373324e-09,
                                         0.000353995829409715,
                                         0.00893977037416649,
                                         0.045457697349031,
                                         0.121697036659076,
                                         0.223011070589436,
                                         0.310819375121687,
                                         0.345598716301191,
                                         0.3125,
                                         0.229826550986094,
                                         0.135071059619463,
                                         0.0609036600185984,
                                         0.0194486201570192,
                                         0.00373344584574066,
                                         0.000289459128736202,
                                         2.13813608649453e-06,
                                         2.13813608649447e-06,
                                         0.000289459128736202,
                                         0.00373344584574066,
                                         0.0194486201570193,
                                         0.0609036600185984,
                                         0.135071059619463,
                                         0.229826550986094,
                                         0.3125,
                                         0.345598716301191,
                                         0.310819375121687,
                                         0.223011070589436,
                                         0.121697036659076,
                                         0.045457697349031,
                                         0.00893977037416649,
                                         0.000353995829409722,
                                         6.45717483733216e-09,
                                         4.68617110406112e-06,
                                         0.000153314165652248,
                                         0.00155405931153298,
                                         0.00831630419480335,
                                         0.0293485422837327,
                                         0.0764184602643757,
                                         0.15625,
                                         0.259844809480385,
                                         0.357621700100721,
                                         0.408300072526502,
                                         0.380751143578054,
                                         0.276741960866203,
                                         0.138049704446632,
                                         0.029304273014008,
                                         7.80027186169162e-12,
                                         3.03465290072794e-08,
                                         2.51834196726984e-06,
                                         4.9671397235669e-05,
                                         0.000454231587654232,
                                         0.00255076679373569,
                                         0.0101638057815719,
                                         0.03125,
                                         0.0781476571863817,
                                         0.164588556080661,
                                         0.299014661083007,
                                         0.476499468897082,
                                         0.673911063431405,
                                         0.852716349532832,
                                         0.970339586555521,

                                         0.973781703490346,
                                         0.868907483560764,
                                         0.706283409595617,
                                         0.520877916313063,
                                         0.346398906828627,
                                         0.205902978632174,
                                         0.10804844763362,
                                         0.0491977708598261,
                                         0.0189698065111919,
                                         0.00597958311931442,
                                         0.00146143409524511,
                                         0.000254606923303208,
                                         2.73585545929656e-05,
                                         1.36880687032184e-06,
                                         1.63447253260345e-08,
                                         4.18011087188279e-12,
                                         0.0259404109993526,
                                         0.123829568559764,
                                         0.254344085796055,
                                         0.362891996593532,
                                         0.409077182726811,
                                         0.382697790447774,
                                         0.302830079850391,
                                         0.203300723836565,
                                         0.114764986614499,
                                         0.0533372274195243,
                                         0.0196573929087428,
                                         0.00538990951018953,
                                         0.000981729208930638,
                                         9.50253649915821e-05,
                                         2.86725826426549e-06,
                                         3.92294814228481e-09,
                                         0.000276408940691091,
                                         0.00705886982897649,
                                         0.0366373685693507,
                                         0.101129725079372,
                                         0.193238648423905,
                                         0.28451768845023,
                                         0.339499768004661,
                                         0.336041113978369,
                                         0.27772559819955,
                                         0.19030489397249,
                                         0.105762715465834,
                                         0.0456407455871653,
                                         0.0140912742505101,
                                         0.00263874186711534,
                                         0.000201194447505428,
                                         1.47264248233915e-06,
                                         1.47264248233919e-06,
                                         0.000201194447505428,
                                         0.00263874186711533,
                                         0.0140912742505101,
                                         0.0456407455871653,
                                         0.105762715465834,
                                         0.19030489397249,
                                         0.27772559819955,
                                         0.336041113978368,
                                         0.339499768004661,
                                         0.28451768845023,
                                         0.193238648423905,
                                         0.101129725079371,
                                         0.0366373685693508,
                                         0.00705886982897649,
                                         0.000276408940691085,
                                         3.92294814228497e-09,
                                         2.86725826426549e-06,
                                         9.50253649915818e-05,
                                         0.00098172920893064,
                                         0.00538990951018953,
                                         0.0196573929087428,
                                         0.0533372274195243,
                                         0.114764986614499,
                                         0.203300723836565,
                                         0.302830079850391,
                                         0.382697790447774,
                                         0.409077182726811,
                                         0.362891996593532,
                                         0.254344085796055,
                                         0.123829568559764,
                                         0.0259404109993523,
                                         4.18011087188301e-12,
                                         1.63447253260345e-08,
                                         1.36880687032183e-06,
                                         2.73585545929657e-05,
                                         0.000254606923303208,
                                         0.00146143409524511,
                                         0.00597958311931443,
                                         0.0189698065111919,
                                         0.0491977708598261,
                                         0.10804844763362,
                                         0.205902978632174,
                                         0.346398906828627,
                                         0.520877916313063,
                                         0.706283409595617,
                                         0.868907483560764,
                                         0.973781703490346,

                                         0.97665969853552,
                                         0.882622903215021,
                                         0.734370938736272,
                                         0.560784788110404,
                                         0.391148734311032,
                                         0.247514400905269,
                                         0.140766479732878,
                                         0.0710344151361086,
                                         0.03125,
                                         0.0116931002532677,
                                         0.00359168231124755,
                                         0.000858764822478534,
                                         0.000146915449748258,
                                         1.55585117355168e-05,
                                         7.69882242787479e-07,
                                         9.12352194387934e-09,
                                         2.32352475190747e-12,
                                         0.0231203326686423,
                                         0.111589328133212,
                                         0.233876857970521,
                                         0.343874992736012,
                                         0.403884365261647,
                                         0.398680717606729,
                                         0.337931742322539,
                                         0.247588795033434,
                                         0.15625,
                                         0.0838701664090441,
                                         0.037403150692382,
                                         0.0133287522550794,
                                         0.0035570696195766,
                                         0.000634313114592569,
                                         6.04355589366917e-05,
                                         1.8040769579775e-06,
                                         2.45378496090537e-09,
                                         0.000218929800629732,
                                         0.00564326083443507,
                                         0.0297933274910305,
                                         0.0843460900768311,
                                         0.16681386510441,
                                         0.256867986686964,
                                         0.324503000105854,
                                         0.345185985180005,
                                         0.3125,
                                         0.24062753798814,
                                         0.155803944835099,
                                         0.0827493777234798,
                                         0.034449050253586,
                                         0.0103442574504253,
                                         0.00189767036099757,
                                         0.000142694616851941,
                                         1.03653909938724e-06,
                                         1.03653909938721e-06,
                                         0.000142694616851942,
                                         0.00189767036099757,
                                         0.0103442574504253,
                                         0.034449050253586,
                                         0.0827493777234799,
                                         0.155803944835099,
                                         0.24062753798814,
                                         0.3125,
                                         0.345185985180005,
                                         0.324503000105854,
                                         0.256867986686964,
                                         0.16681386510441,
                                         0.084346090076831,
                                         0.0297933274910305,
                                         0.00564326083443504,
                                         0.000218929800629737,
                                         2.45378496090525e-09,
                                         1.80407695797751e-06,
                                         6.04355589366917e-05,
                                         0.00063431311459257,
                                         0.00355706961957661,
                                         0.0133287522550794,
                                         0.037403150692382,
                                         0.0838701664090441,
                                         0.15625,
                                         0.247588795033434,
                                         0.337931742322539,
                                         0.398680717606729,
                                         0.403884365261647,
                                         0.343874992736011,
                                         0.233876857970521,
                                         0.111589328133212,
                                         0.0231203326686426,
                                         2.32352475190734e-12,
                                         9.12352194387944e-09,
                                         7.69882242787479e-07,
                                         1.55585117355169e-05,
                                         0.000146915449748259,
                                         0.000858764822478534,
                                         0.00359168231124755,
                                         0.0116931002532677,
                                         0.03125,
                                         0.0710344151361086,
                                         0.140766479732878,
                                         0.247514400905269,
                                         0.391148734311032,
                                         0.560784788110404,
                                         0.734370938736272,
                                         0.882622903215021,
                                         0.97665969853552,

                                         0.979090038457521,
                                         0.894332104141364,
                                         0.758834434927316,
                                         0.596592127449347,
                                         0.43296235441263,
                                         0.288505431581841,
                                         0.175243044759442,
                                         0.0960891444563751,
                                         0.0469405729977668,
                                         0.0200673194420643,
                                         0.0073229872337077,
                                         0.00220117926679346,
                                         0.000516706703666593,
                                         8.70580437338779e-05,
                                         9.10744721401243e-06,
                                         4.465000983826e-07,
                                         5.25756253449221e-09,
                                         1.33423429116988e-12,
                                         0.0207335911123645,
                                         0.101001064388041,
                                         0.215303921691312,
                                         0.324632017979952,
                                         0.394540775967116,
                                         0.407138355986811,
                                         0.365101585730731,
                                         0.287109207905955,
                                         0.198018873928661,
                                         0.118924455842412,
                                         0.0612710737565772,
                                         0.0264132896055629,
                                         0.00915368745971148,
                                         0.00238840050772845,
                                         0.000418430022926439,
                                         3.93420202462602e-05,
                                         1.16385084469564e-06,
                                         1.57514380453108e-09,
                                         0.000175625032848649,
                                         0.00456260709429019,
                                         0.0244352530997614,
                                         0.070658624040716,
                                         0.143811509073956,
                                         0.229821171832769,
                                         0.30426124605647,
                                         0.343146763271722,
                                         0.334137160480256,
                                         0.28191161730884,
                                         0.205060823375663,
                                         0.126779654581001,
                                         0.0648646464352009,
                                         0.0262099019948352,
                                         0.00768969305984413,
                                         0.00138660176126623,
                                         0.000103055267897415,
                                         7.43820788109766e-07,
                                         7.43820788109766e-07,
                                         0.000103055267897415,
                                         0.00138660176126623,
                                         0.00768969305984412,
                                         0.0262099019948352,
                                         0.0648646464352009,
                                         0.126779654581001,
                                         0.205060823375663,
                                         0.28191161730884,
                                         0.334137160480256,
                                         0.343146763271722,
                                         0.30426124605647,
                                         0.229821171832769,
                                         0.143811509073956,
                                         0.0706586240407161,
                                         0.0244352530997614,
                                         0.00456260709429019,
                                         0.000175625032848649,
                                         1.57514380453108e-09,
                                         1.16385084469564e-06,
                                         3.93420202462602e-05,
                                         0.000418430022926438,
                                         0.00238840050772845,
                                         0.00915368745971148,
                                         0.0264132896055629,
                                         0.0612710737565772,
                                         0.118924455842412,
                                         0.198018873928661,
                                         0.287109207905955,
                                         0.365101585730731,
                                         0.407138355986811,
                                         0.394540775967116,
                                         0.324632017979952,
                                         0.215303921691312,
                                         0.101001064388041,
                                         0.0207335911123645,
                                         1.33423429116988e-12,
                                         5.25756253449221e-09,
                                         4.465000983826e-07,
                                         9.1074472140124e-06,
                                         8.70580437338779e-05,
                                         0.000516706703666593,
                                         0.00220117926679346,
                                         0.0073229872337077,
                                         0.0200673194420643,
                                         0.0469405729977668,
                                         0.0960891444563751,
                                         0.175243044759442,
                                         0.288505431581841,
                                         0.43296235441263,
                                         0.596592127449347,
                                         0.758834434927316,
                                         0.894332104141364,
                                         0.979090038457521,

                                         0.98116070345907,
                                         0.904400881013132,
                                         0.780228579088967,
                                         0.628699985817667,
                                         0.47174977004354,
                                         0.328238772720487,
                                         0.2105725664303,
                                         0.123611838159504,
                                         0.0657372048597563,
                                         0.03125,
                                         0.013041247108964,
                                         0.00465949619833076,
                                         0.00137518307818972,
                                         0.000317824720394769,
                                         5.2860865028868e-05,
                                         5.47285808961825e-06,
                                         2.66204736245096e-07,
                                         3.11760119467715e-09,
                                         7.88790513321357e-13,
                                         0.0186962478545084,
                                         0.0917958182839067,
                                         0.198513772760937,
                                         0.305750800107497,
                                         0.38244255875005,
                                         0.409599523904138,
                                         0.384913852042446,
                                         0.320838035421231,
                                         0.237838860974822,
                                         0.15625,
                                         0.0901130211979348,
                                         0.0448800349697231,
                                         0.0188078337886107,
                                         0.00636734115119639,
                                         0.001630120677676,
                                         0.000281339067805823,
                                         2.61569436994922e-05,
                                         7.67889354826942e-07,
                                         1.03487376311392e-09,
                                         0.000142504559183781,
                                         0.00372687485442184,
                                         0.0202031656014422,
                                         0.0594773684588477,
                                         0.124016858115086,
                                         0.204450886276453,
                                         0.281439650009101,
                                         0.333097691954486,
                                         0.344202792987512,
                                         0.3125,
                                         0.249066872871011,
                                         0.172912903296725,
                                         0.102890914651352,
                                         0.0510256512273314,
                                         0.0201078315486187,
                                         0.00578503369009302,
                                         0.00102805940021814,
                                         7.56548415831101e-05,
                                         5.43091575009979e-07,
                                         5.43091575009979e-07,
                                         7.56548415831101e-05,
                                         0.00102805940021814,
                                         0.00578503369009303,
                                         0.0201078315486187,
                                         0.0510256512273314,
                                         0.102890914651352,
                                         0.172912903296725,
                                         0.24906687287101,
                                         0.3125,
                                         0.344202792987512,
                                         0.333097691954486,
                                         0.281439650009101,
                                         0.204450886276453,
                                         0.124016858115086,
                                         0.0594773684588477,
                                         0.0202031656014422,
                                         0.00372687485442184,
                                         0.000142504559183781,
                                         1.03487376311392e-09,
                                         7.67889354826942e-07,
                                         2.61569436994922e-05,
                                         0.000281339067805823,
                                         0.001630120677676,
                                         0.0063673411511964,
                                         0.0188078337886107,
                                         0.0448800349697231,
                                         0.0901130211979348,
                                         0.15625,
                                         0.237838860974822,
                                         0.320838035421231,
                                         0.384913852042446,
                                         0.409599523904138,
                                         0.38244255875005,
                                         0.305750800107497,
                                         0.198513772760937,
                                         0.0917958182839066,
                                         0.0186962478545084,
                                         7.88790513321357e-13,
                                         3.11760119467715e-09,
                                         2.66204736245096e-07,
                                         5.47285808961825e-06,
                                         5.28608650288679e-05,
                                         0.000317824720394769,
                                         0.00137518307818972,
                                         0.00465949619833076,
                                         0.013041247108964,
                                         0.03125,
                                         0.0657372048597563,
                                         0.123611838159504,
                                         0.2105725664303,
                                         0.328238772720487,
                                         0.47174977004354,
                                         0.628699985817667,
                                         0.780228579088967,
                                         0.904400881013132,
                                         0.98116070345907,

                                         0.982939133480475,
                                         0.913116941337277,
                                         0.799016388116032,
                                         0.657501552479643,
                                         0.507555999986976,
                                         0.366300716303642,
                                         0.246026111308694,
                                         0.152868898973341,
                                         0.0871892904266643,
                                         0.045182857045992,
                                         0.0209880533359137,
                                         0.00858101764862591,
                                         0.00301120948428896,
                                         0.000874953219047563,
                                         0.000199543083706618,
                                         3.28230735874121e-05,
                                         3.36822567448809e-06,
                                         1.62731247444625e-07,
                                         1.89694690432101e-09,
                                         4.78714334401927e-13,
                                         0.0169436351612265,
                                         0.0837533447126836,
                                         0.183361749750767,
                                         0.287585951343019,
                                         0.368631994883458,
                                         0.407424887754146,
                                         0.398246467979707,
                                         0.348476509779869,
                                         0.274187895363778,
                                         0.193795365754003,
                                         0.122332674192269,
                                         0.0682171653611605,
                                         0.0330237667046852,
                                         0.0135130726426345,
                                         0.00448504599817663,
                                         0.00112982243596163,
                                         0.000192517541254668,
                                         1.77279249548442e-05,
                                         5.17034382655079e-07,
                                         6.94282910019355e-10,
                                         0.000116827894097658,
                                         0.00307282558586135,
                                         0.0168314601661363,
                                         0.0503151234231832,
                                         0.10709324500567,
                                         0.181266409562659,
                                         0.257859214072302,
                                         0.317751690982053,
                                         0.344900166504976,
                                         0.332485781051858,
                                         0.285215268619964,
                                         0.216924464694795,
                                         0.144867924075763,
                                         0.0834801807776153,
                                         0.0403233972976695,
                                         0.0155561146143465,
                                         0.00440148698722521,
                                         0.000772511310863102,
                                         5.63694328478221e-05,
                                         4.02769438477047e-07,
                                         4.02769438477027e-07,
                                         5.63694328478216e-05,
                                         0.000772511310863102,
                                         0.0044014869872252,
                                         0.0155561146143465,
                                         0.0403233972976695,
                                         0.0834801807776153,
                                         0.144867924075763,
                                         0.216924464694795,
                                         0.285215268619964,
                                         0.332485781051858,
                                         0.344900166504976,
                                         0.317751690982053,
                                         0.257859214072302,
                                         0.181266409562659,
                                         0.10709324500567,
                                         0.0503151234231833,
                                         0.0168314601661363,
                                         0.00307282558586137,
                                         0.000116827894097662,
                                         6.9428291001931e-10,
                                         5.17034382655072e-07,
                                         1.77279249548442e-05,
                                         0.000192517541254668,
                                         0.00112982243596163,
                                         0.00448504599817663,
                                         0.0135130726426345,
                                         0.0330237667046852,
                                         0.0682171653611605,
                                         0.122332674192269,
                                         0.193795365754003,
                                         0.274187895363778,
                                         0.348476509779869,
                                         0.398246467979706,
                                         0.407424887754146,
                                         0.368631994883458,
                                         0.28758595134302,
                                         0.183361749750767,
                                         0.0837533447126838,
                                         0.0169436351612268,
                                         4.78714334401888e-13,
                                         1.89694690432098e-09,
                                         1.62731247444625e-07,
                                         3.36822567448808e-06,
                                         3.28230735874121e-05,
                                         0.000199543083706618,
                                         0.000874953219047563,
                                         0.00301120948428896,
                                         0.00858101764862591,
                                         0.0209880533359137,
                                         0.045182857045992,
                                         0.0871892904266643,
                                         0.152868898973341,
                                         0.246026111308694,
                                         0.366300716303642,
                                         0.507555999986976,
                                         0.657501552479643,
                                         0.799016388116032,
                                         0.913116941337277,
                                         0.982939133480475,

                                         0.984477710598813,
                                         0.920708654774105,
                                         0.815583642223294,
                                         0.683365252300584,
                                         0.540507375708598,
                                         0.402447119064048,
                                         0.281044594773174,
                                         0.183198371246392,
                                         0.110780998449751,
                                         0.0616513712004949,
                                         0.03125,
                                         0.0142315815667228,
                                         0.00571722095009891,
                                         0.00197547842390004,
                                         0.000566352500871622,
                                         0.000127692640289202,
                                         2.08050166966591e-05,
                                         2.11866580654319e-06,
                                         1.01765838242085e-07,
                                         1.18152171491229e-09,
                                         2.97506066989199e-13,
                                         0.0154253092598201,
                                         0.0766930706331301,
                                         0.169693502077372,
                                         0.270336576391104,
                                         0.353870954755965,
                                         0.401763275384426,
                                         0.406076220710303,
                                         0.37020454733793,
                                         0.306181692688822,
                                         0.229918983011068,
                                         0.15625,
                                         0.0954027921541729,
                                         0.0517143464431009,
                                         0.024439491106556,
                                         0.0097992877314892,
                                         0.00319774966744404,
                                         0.00079444673440631,
                                         0.000133890742121998,
                                         1.22277152618288e-05,
                                         3.54607417509423e-07,
                                         4.74687552102609e-10,
                                         9.66767101781773e-05,
                                         0.0025553477976511,
                                         0.0141228357983174,
                                         0.042777629848067,
                                         0.092671928819273,
                                         0.160432237480556,
                                         0.234692856711154,
                                         0.299241540057918,
                                         0.338495699622404,
                                         0.342978511066235,
                                         0.3125,
                                         0.255816761001306,
                                         0.187110041845823,
                                         0.120940571827304,
                                         0.0678206875730078,
                                         0.0320319257632366,
                                         0.012134488965062,
                                         0.0033845320523162,
                                         0.000587690419917471,
                                         4.25710061747066e-05,
                                         3.02956204425031e-07,
                                         3.02956204425048e-07,
                                         4.25710061747062e-05,
                                         0.000587690419917474,
                                         0.0033845320523162,
                                         0.012134488965062,
                                         0.0320319257632366,
                                         0.0678206875730078,
                                         0.120940571827304,
                                         0.187110041845823,
                                         0.255816761001306,
                                         0.3125,
                                         0.342978511066235,
                                         0.338495699622404,
                                         0.299241540057918,
                                         0.234692856711154,
                                         0.160432237480556,
                                         0.092671928819273,
                                         0.042777629848067,
                                         0.0141228357983173,
                                         0.00255534779765112,
                                         9.66767101781738e-05,
                                         4.74687552102643e-10,
                                         3.54607417509418e-07,
                                         1.22277152618289e-05,
                                         0.000133890742121998,
                                         0.00079444673440631,
                                         0.00319774966744404,
                                         0.0097992877314892,
                                         0.024439491106556,
                                         0.0517143464431009,
                                         0.0954027921541729,
                                         0.15625,
                                         0.229918983011068,
                                         0.306181692688822,
                                         0.37020454733793,
                                         0.406076220710303,
                                         0.401763275384426,
                                         0.353870954755965,
                                         0.270336576391104,
                                         0.169693502077372,
                                         0.0766930706331304,
                                         0.0154253092598198,
                                         2.97506066989225e-13,
                                         1.18152171491227e-09,
                                         1.01765838242086e-07,
                                         2.11866580654319e-06,
                                         2.08050166966591e-05,
                                         0.000127692640289202,
                                         0.000566352500871622,
                                         0.00197547842390004,
                                         0.00571722095009891,
                                         0.0142315815667228,
                                         0.03125,
                                         0.0616513712004949,
                                         0.110780998449751,
                                         0.183198371246392,
                                         0.281044594773174,
                                         0.402447119064048,
                                         0.540507375708598,
                                         0.683365252300584,
                                         0.815583642223294,
                                         0.920708654774105,
                                         0.984477710598813,

                                         0.985817611272415,
                                         0.927358883169568,
                                         0.830251908189687,
                                         0.706626798199257,
                                         0.570775205306482,
                                         0.436557612320609,
                                         0.31521780089868,
                                         0.214035068411745,
                                         0.135990556499343,
                                         0.0803403621199148,
                                         0.0437763676812723,
                                         0.0217707531821425,
                                         0.00974672184409651,
                                         0.00385648172376317,
                                         0.0013148220379631,
                                         0.000372589448330347,
                                         8.31757307409397e-05,
                                         1.34401753463899e-05,
                                         1.35959347889707e-06,
                                         6.49755261212705e-08,
                                         7.51749061641046e-10,
                                         1.88923523163654e-13,
                                         0.0141014726510423,
                                         0.070466524557229,
                                         0.157357653651719,
                                         0.254099553450246,
                                         0.338706140709618,
                                         0.393552414982249,
                                         0.409359338861938,
                                         0.38648384748794,
                                         0.33343812921574,
                                         0.263444263770927,
                                         0.190342808880085,
                                         0.125174744137578,
                                         0.0743093767945626,
                                         0.0393210201378001,
                                         0.0182037377935623,
                                         0.00717259968144727,
                                         0.00230661768502451,
                                         0.000566222303101648,
                                         9.45225182217549e-05,
                                         8.57061180309869e-06,
                                         2.47330620676855e-07,
                                         3.30185614168098e-10,
                                         8.0684917231873e-05,
                                         0.00214179479952924,
                                         0.0119295991582917,
                                         0.0365491845076658,
                                         0.080397220263135,
                                         0.141913460186889,
                                         0.212646706925472,
                                         0.279150076625265,
                                         0.327025607886028,
                                         0.34554427330171,
                                         0.331049712997181,
                                         0.287885613121742,
                                         0.226615002168789,
                                         0.160368204537326,
                                         0.100812447643524,
                                         0.0552309641841323,
                                         0.0255867190944873,
                                         0.00954177124231793,
                                         0.00262858173113035,
                                         0.000452203412973169,
                                         3.25493913039868e-05,
                                         2.30828936445621e-07,
                                         2.30828936445621e-07,
                                         3.25493913039868e-05,
                                         0.000452203412973167,
                                         0.00262858173113036,
                                         0.00954177124231795,
                                         0.0255867190944873,
                                         0.0552309641841323,
                                         0.100812447643524,
                                         0.160368204537326,
                                         0.226615002168789,
                                         0.287885613121742,
                                         0.331049712997181,
                                         0.34554427330171,
                                         0.327025607886028,
                                         0.279150076625265,
                                         0.212646706925472,
                                         0.141913460186889,
                                         0.080397220263135,
                                         0.0365491845076657,
                                         0.0119295991582917,
                                         0.00214179479952924,
                                         8.0684917231873e-05,
                                         3.30185614168098e-10,
                                         2.47330620676855e-07,
                                         8.57061180309864e-06,
                                         9.45225182217552e-05,
                                         0.000566222303101649,
                                         0.00230661768502451,
                                         0.00717259968144727,
                                         0.0182037377935623,
                                         0.0393210201378001,
                                         0.0743093767945626,
                                         0.125174744137578,
                                         0.190342808880085,
                                         0.263444263770927,
                                         0.33343812921574,
                                         0.38648384748794,
                                         0.409359338861938,
                                         0.393552414982249,
                                         0.338706140709617,
                                         0.254099553450246,
                                         0.157357653651719,
                                         0.070466524557229,
                                         0.0141014726510423,
                                         1.88923523163654e-13,
                                         7.51749061641046e-10,
                                         6.497552612127e-08,
                                         1.35959347889707e-06,
                                         1.344017534639e-05,
                                         8.31757307409397e-05,
                                         0.000372589448330347,
                                         0.0013148220379631,
                                         0.00385648172376317,
                                         0.0097467218440965,
                                         0.0217707531821425,
                                         0.0437763676812723,
                                         0.0803403621199148,
                                         0.135990556499343,
                                         0.214035068411745,
                                         0.31521780089868,
                                         0.436557612320609,
                                         0.570775205306482,
                                         0.706626798199257,
                                         0.830251908189687,
                                         0.927358883169568,
                                         0.985817611272415,

                                         0.986991558480799,
                                         0.933215269062265,
                                         0.843289730210885,
                                         0.727586903133462,
                                         0.598551870509014,
                                         0.4685992516072,
                                         0.348258803031577,
                                         0.244916325136397,
                                         0.162328306085754,
                                         0.100883111897818,
                                         0.058410609315661,
                                         0.03125,
                                         0.0152862928818678,
                                         0.00674277623367906,
                                         0.00263285717070057,
                                         0.000887234738581694,
                                         0.000248883546992533,
                                         5.50803785263624e-05,
                                         8.83619833475794e-06,
                                         8.8867530513844e-07,
                                         4.228252458487e-08,
                                         4.87705057509945e-10,
                                         1.2235861556374e-13,
                                         0.0129403989526022,
                                         0.0649511619745965,
                                         0.146212207963319,
                                         0.238905707429487,
                                         0.3235225293273,
                                         0.383539996629448,
                                         0.408969503501959,
                                         0.397916455511562,
                                         0.355935142863028,
                                         0.293624662114185,
                                         0.223369477806842,
                                         0.15625,
                                         0.0999332686558529,
                                         0.0579168183954917,
                                         0.0300186180864152,
                                         0.0136522546816656,
                                         0.00529843212193558,
                                         0.00168239456005362,
                                         0.000408698201981115,
                                         6.76614761649952e-05,
                                         6.09668974405258e-06,
                                         1.75183088581687e-07,
                                         2.33313750818567e-10,
                                         6.78643798373548e-05,
                                         0.00180822306779843,
                                         0.0101402917605378,
                                         0.0313782102435201,
                                         0.0699467044641086,
                                         0.125568215066485,
                                         0.192105472526388,
                                         0.258598532341532,
                                         0.312181723520226,
                                         0.341842913366886,
                                         0.341677131605087,
                                         0.3125,
                                         0.261323219734689,
                                         0.198989717991939,
                                         0.136903352273876,
                                         0.0840291975902609,
                                         0.0451189052711483,
                                         0.0205550617582871,
                                         0.00756136129926161,
                                         0.00206062904206077,
                                         0.000351631092988581,
                                         2.51702245465384e-05,
                                         1.77953325379598e-07,
                                         1.77953325379609e-07,
                                         2.51702245465384e-05,
                                         0.000351631092988579,
                                         0.00206062904206077,
                                         0.00756136129926161,
                                         0.020555061758287,
                                         0.0451189052711484,
                                         0.0840291975902608,
                                         0.136903352273876,
                                         0.198989717991939,
                                         0.261323219734689,
                                         0.3125,
                                         0.341677131605087,
                                         0.341842913366886,
                                         0.312181723520226,
                                         0.258598532341532,
                                         0.192105472526388,
                                         0.125568215066485,
                                         0.0699467044641086,
                                         0.0313782102435201,
                                         0.0101402917605378,
                                         0.00180822306779843,
                                         6.78643798373519e-05,
                                         2.33313750818587e-10,
                                         1.75183088581687e-07,
                                         6.09668974405254e-06,
                                         6.76614761649952e-05,
                                         0.000408698201981115,
                                         0.00168239456005362,
                                         0.00529843212193559,
                                         0.0136522546816656,
                                         0.0300186180864152,
                                         0.0579168183954917,
                                         0.0999332686558529,
                                         0.15625,
                                         0.223369477806842,
                                         0.293624662114185,
                                         0.355935142863028,
                                         0.397916455511562,
                                         0.408969503501959,
                                         0.383539996629448,
                                         0.3235225293273,
                                         0.238905707429487,
                                         0.14621220796332,
                                         0.0649511619745964,
                                         0.012940398952602,
                                         1.22358615563753e-13,
                                         4.87705057509945e-10,
                                         4.22825245848697e-08,
                                         8.8867530513844e-07,
                                         8.83619833475794e-06,
                                         5.50803785263623e-05,
                                         0.000248883546992534,
                                         0.000887234738581693,
                                         0.00263285717070057,
                                         0.00674277623367906,
                                         0.0152862928818678,
                                         0.03125,
                                         0.058410609315661,
                                         0.100883111897818,
                                         0.162328306085754,
                                         0.244916325136397,
                                         0.348258803031577,
                                         0.4685992516072,
                                         0.598551870509014,
                                         0.727586903133462,
                                         0.843289730210885,
                                         0.933215269062265,
                                         0.986991558480799,

                                         0.988025817941392,
                                         0.938397957393713,
                                         0.854921995854458,
                                         0.746512069629435,
                                         0.624035549809504,
                                         0.498598818739284,
                                         0.379978876760859,
                                         0.27547661060306,
                                         0.189357743346661,
                                         0.122899258342423,
                                         0.0749247912415125,
                                         0.0426259693762454,
                                         0.0224438456232883,
                                         0.0108211532824601,
                                         0.0047115909497125,
                                         0.00181854581178343,
                                         0.000606589117436422,
                                         0.000168649682355491,
                                         3.70407087815065e-05,
                                         5.9045742578186e-06,
                                         5.90805947271715e-07,
                                         2.80008538690956e-08,
                                         3.22107112265813e-10,
                                         8.06919242480726e-14,
                                         0.0119165531218229,
                                         0.0600453955689333,
                                         0.136127369423255,
                                         0.224744174334596,
                                         0.308585266700376,
                                         0.372311849133901,
                                         0.405671664061228,
                                         0.40515081041999,
                                         0.373887812624005,
                                         0.320068496487904,
                                         0.254404709927408,
                                         0.187468745752009,
                                         0.127580023057418,
                                         0.0796735102616688,
                                         0.0452286807110382,
                                         0.0230253380469322,
                                         0.0103110440484412,
                                         0.00394921081164317,
                                         0.00124011992142391,
                                         0.000298512651200756,
                                         4.90606454820233e-05,
                                         4.39635063411921e-06,
                                         1.25848390766791e-07,
                                         1.672583163131e-10,
                                         5.74900921520859e-05,
                                         0.00153685310187306,
                                         0.00867010594929178,
                                         0.0270645022108818,
                                         0.0610381039052094,
                                         0.111204525799719,
                                         0.17324068161376,
                                         0.238346448106257,
                                         0.29529734344762,
                                         0.333423793847795,
                                         0.345529191931276,
                                         0.329794546827754,
                                         0.290086869363285,
                                         0.234646643355675,
                                         0.173668179661128,
                                         0.116613216722999,
                                         0.0701084977048164,
                                         0.0369909170701537,
                                         0.016607645696891,
                                         0.00603666235945183,
                                         0.00162960237365776,
                                         0.000276104421507695,
                                         1.96677649831228e-05,
                                         1.38677294594627e-07,
                                         1.38677294594636e-07,
                                         1.96677649831228e-05,
                                         0.000276104421507697,
                                         0.00162960237365776,
                                         0.00603666235945183,
                                         0.016607645696891,
                                         0.0369909170701537,
                                         0.0701084977048165,
                                         0.116613216722999,
                                         0.173668179661128,
                                         0.234646643355675,
                                         0.290086869363285,
                                         0.329794546827754,
                                         0.345529191931276,
                                         0.333423793847795,
                                         0.29529734344762,
                                         0.238346448106257,
                                         0.17324068161376,
                                         0.111204525799719,
                                         0.0610381039052094,
                                         0.0270645022108818,
                                         0.00867010594929175,
                                         0.00153685310187306,
                                         5.74900921520832e-05,
                                         1.67258316313116e-10,
                                         1.25848390766791e-07,
                                         4.39635063411925e-06,
                                         4.90606454820233e-05,
                                         0.000298512651200756,
                                         0.00124011992142391,
                                         0.00394921081164317,
                                         0.0103110440484412,
                                         0.0230253380469322,
                                         0.0452286807110382,
                                         0.0796735102616688,
                                         0.127580023057418,
                                         0.187468745752009,
                                         0.254404709927408,
                                         0.320068496487904,
                                         0.373887812624005,
                                         0.405150810419989,
                                         0.405671664061228,
                                         0.372311849133901,
                                         0.308585266700376,
                                         0.224744174334596,
                                         0.136127369423255,
                                         0.0600453955689333,
                                         0.0119165531218226,
                                         8.06919242480819e-14,
                                         3.22107112265813e-10,
                                         2.80008538690959e-08,
                                         5.90805947271715e-07,
                                         5.9045742578186e-06,
                                         3.70407087815066e-05,
                                         0.000168649682355491,
                                         0.000606589117436422,
                                         0.00181854581178343,
                                         0.0047115909497125,
                                         0.0108211532824601,
                                         0.0224438456232883,
                                         0.0426259693762454,
                                         0.0749247912415125,
                                         0.122899258342423,
                                         0.189357743346661,
                                         0.27547661060306,
                                         0.379978876760859,
                                         0.498598818739284,
                                         0.624035549809504,
                                         0.746512069629435,
                                         0.854921995854458,
                                         0.938397957393713,
                                         0.988025817941392,

                                         0.015625,
                                         0.09375,
                                         0.234375,
                                         0.3125,
                                         0.234375,
                                         0.09375,
                                         0.015625,

                                         0.240651676451543,
                                         8.90642891983359e-05,
                                         0.38689453417432,
                                         0.00199435471456919,
                                         0.259170194970061,
                                         0.0186075828077168,
                                         0.0925925925925925,
                                         0.0925925925925925,
                                         0.0186075828077168,
                                         0.259170194970061,
                                         0.00199435471456919,
                                         0.38689453417432,
                                         8.90642891983359e-05,
                                         0.240651676451543,

                                         0.487997950811067,
                                         0.015625,
                                         2.04918893273626e-06,
                                         0.371903200617956,
                                         0.09375,
                                         9.67993820439904e-05,
                                         0.118094750193112,
                                         0.234375,
                                         0.00190524980688876,
                                         0.0200000000000001,
                                         0.3125,
                                         0.0200000000000001,
                                         0.00190524980688876,
                                         0.234375,
                                         0.118094750193112,
                                         9.67993820439904e-05,
                                         0.09375,
                                         0.371903200617956,
                                         2.04918893273626e-06,
                                         0.015625,
                                         0.487997950811067,

                                         0.649365364759566,
                                         0.0904507043809332,
                                         0.00129169054500841,
                                         1.12034609596495e-07,
                                         0.290703918176805,
                                         0.267313540583252,
                                         0.0157344648452828,
                                         9.00933926509632e-06,
                                         0.0542252306569591,
                                         0.329168843350549,
                                         0.0798609055796943,
                                         0.000301871724750737,
                                         0.00539449330804379,
                                         0.21617985071528,
                                         0.21617985071528,
                                         0.0053944933080438,
                                         0.000301871724750736,
                                         0.0798609055796944,
                                         0.329168843350549,
                                         0.0542252306569591,
                                         9.00933926509629e-06,
                                         0.0157344648452828,
                                         0.267313540583252,
                                         0.290703918176806,
                                         1.12034609596494e-07,
                                         0.00129169054500841,
                                         0.0904507043809332,
                                         0.649365364759566,

                                         0.749554587679141,
                                         0.207182490501477,
                                         0.015625,
                                         0.000151016199569706,
                                         1.06560653839098e-08,
                                         0.221353699788412,
                                         0.372920319490549,
                                         0.09375,
                                         0.00302039010779528,
                                         1.29902006290891e-06,
                                         0.0272369869249785,
                                         0.279684117192853,
                                         0.234375,
                                         0.0251704685470313,
                                         6.59817149140504e-05,
                                         0.00178743421642514,
                                         0.111871197960726,
                                         0.3125,
                                         0.111871197960726,
                                         0.00178743421642515,
                                         6.598171491405e-05,
                                         0.0251704685470313,
                                         0.234375,
                                         0.279684117192853,
                                         0.0272369869249786,
                                         1.29902006290891e-06,
                                         0.00302039010779528,
                                         0.09375,
                                         0.372920319490549,
                                         0.221353699788413,
                                         1.06560653839097e-08,
                                         0.000151016199569706,
                                         0.015625,
                                         0.207182490501477,
                                         0.749554587679141,

                                         0.81375924289991,
                                         0.328372125630103,
                                         0.0564217878246265,
                                         0.00304390853814397,
                                         2.36269815467975e-05,
                                         1.48190126791976e-09,
                                         0.170621756019885,
                                         0.401813503400374,
                                         0.208095275218851,
                                         0.029711099469125,
                                         0.000695108340399703,
                                         2.54438775867441e-07,
                                         0.0149060179046153,
                                         0.204866774249226,
                                         0.319791086313206,
                                         0.120835517422513,
                                         0.00852090091604614,
                                         1.82027114048226e-05,
                                         0.000694524543507226,
                                         0.055707960482305,
                                         0.262101325213535,
                                         0.262101325213535,
                                         0.0557079604823049,
                                         0.000694524543507226,
                                         1.82027114048226e-05,
                                         0.00852090091604616,
                                         0.120835517422513,
                                         0.319791086313206,
                                         0.204866774249226,
                                         0.0149060179046153,
                                         2.54438775867441e-07,
                                         0.000695108340399704,
                                         0.029711099469125,
                                         0.208095275218851,
                                         0.401813503400373,
                                         0.170621756019885,
                                         1.48190126791976e-09,
                                         2.36269815467976e-05,
                                         0.00304390853814397,
                                         0.0564217878246265,
                                         0.328372125630103,
                                         0.81375924289991,

                                         0.85671295250191,
                                         0.435920768638663,
                                         0.120627117586845,
                                         0.015625,
                                         0.000687413250542091,
                                         4.65874489972301e-06,
                                         2.71469863944772e-10,
                                         0.134214972089717,
                                         0.388182279505458,
                                         0.30588512562442,
                                         0.09375,
                                         0.00975903760618147,
                                         0.000188340312097645,
                                         6.23818849807477e-08,
                                         0.0087610338058376,
                                         0.144029883565605,
                                         0.323191943175992,
                                         0.234375,
                                         0.0577277780891864,
                                         0.00317253483543277,
                                         5.9728845484652e-06,
                                         0.00030500606463178,
                                         0.0285015343978439,
                                         0.182121584666833,
                                         0.3125,
                                         0.182121584666833,
                                         0.0285015343978439,
                                         0.000305006064631782,
                                         5.97288454846515e-06,
                                         0.00317253483543277,
                                         0.0577277780891864,
                                         0.234375,
                                         0.323191943175992,
                                         0.144029883565605,
                                         0.00876103380583764,
                                         6.2381884980747e-08,
                                         0.000188340312097645,
                                         0.00975903760618147,
                                         0.09375,
                                         0.30588512562442,
                                         0.388182279505458,
                                         0.134214972089718,
                                         2.71469863944768e-10,
                                         4.65874489972301e-06,
                                         0.000687413250542091,
                                         0.015625,
                                         0.120627117586845,
                                         0.435920768638663,
                                         0.85671295250191,

                                         0.886628693473981,
                                         0.525563037930433,
                                         0.196946677191932,
                                         0.0429225677678332,
                                         0.00463196994906852,
                                         0.000178262231206392,
                                         1.10426659117843e-06,
                                         6.12673032803027e-11,
                                         0.107764122334931,
                                         0.356876198850528,
                                         0.367523422097736,
                                         0.177698441311976,
                                         0.0402782237846574,
                                         0.00343894693993399,
                                         5.85441714065222e-05,
                                         1.8146749806817e-08,
                                         0.00545752041229134,
                                         0.100971570894968,
                                         0.285765728817883,
                                         0.306528104821836,
                                         0.145936405845817,
                                         0.0276426942666765,
                                         0.00129324900385492,
                                         2.23953418680636e-06,
                                         0.000147406036592928,
                                         0.0152362948822188,
                                         0.118504268454632,
                                         0.282004286518812,
                                         0.282004286518812,
                                         0.118504268454632,
                                         0.0152362948822188,
                                         0.000147406036592928,
                                         2.23953418680636e-06,
                                         0.00129324900385492,
                                         0.0276426942666765,
                                         0.145936405845817,
                                         0.306528104821836,
                                         0.285765728817883,
                                         0.100971570894968,
                                         0.00545752041229134,
                                         1.8146749806817e-08,
                                         5.85441714065222e-05,
                                         0.00343894693993399,
                                         0.0402782237846574,
                                         0.177698441311976,
                                         0.367523422097736,
                                         0.356876198850528,
                                         0.107764122334931,
                                         6.12673032803027e-11,
                                         1.10426659117843e-06,
                                         0.000178262231206392,
                                         0.00463196994906852,
                                         0.0429225677678332,
                                         0.196946677191932,
                                         0.525563037930433,
                                         0.886628693473981,

                                         0.908202619193918,
                                         0.598549725974222,
                                         0.275566362050813,
                                         0.0842648887968301,
                                         0.015625,
                                         0.00148772667905727,
                                         5.21895637687888e-05,
                                         3.03660853684063e-07,
                                         1.62794144261341e-11,
                                         0.0881542669947908,
                                         0.320725075037017,
                                         0.396220581062149,
                                         0.257994621537119,
                                         0.09375,
                                         0.0174928934846064,
                                         0.00130669834137419,
                                         2.04013370225755e-05,
                                         6.03782736437188e-09,
                                         0.00356527213988063,
                                         0.0716068142807417,
                                         0.237375895726281,
                                         0.329126923900684,
                                         0.234375,
                                         0.0857015961476009,
                                         0.0136318805271944,
                                         0.000571106640927325,
                                         9.33063845943033e-07,
                                         7.69025534576269e-05,
                                         0.00852657306921589,
                                         0.0758463927284193,
                                         0.223931349454102,
                                         0.3125,
                                         0.223931349454102,
                                         0.0758463927284194,
                                         0.00852657306921589,
                                         7.69025534576277e-05,
                                         9.3306384594302e-07,
                                         0.000571106640927325,
                                         0.0136318805271944,
                                         0.0857015961476009,
                                         0.234375,
                                         0.329126923900684,
                                         0.237375895726281,
                                         0.0716068142807417,
                                         0.00356527213988066,
                                         6.03782736437178e-09,
                                         2.04013370225755e-05,
                                         0.00130669834137419,
                                         0.0174928934846064,
                                         0.09375,
                                         0.257994621537119,
                                         0.396220581062149,
                                         0.320725075037017,
                                         0.0881542669947911,
                                         1.62794144261337e-11,
                                         3.03660853684063e-07,
                                         5.21895637687887e-05,
                                         0.00148772667905727,
                                         0.015625,
                                         0.0842648887968301,
                                         0.275566362050813,
                                         0.598549725974222,
                                         0.908202619193918,

                                         0.924228861993799,
                                         0.657629939666541,
                                         0.350557905405777,
                                         0.135523950826256,
                                         0.0359298332314448,
                                         0.00593994102213164,
                                         0.000517012007032715,
                                         1.69638086069414e-05,
                                         9.43190130066706e-08,
                                         4.93186517608961e-12,
                                         0.0733054147366087,
                                         0.285475673185297,
                                         0.401517934701504,
                                         0.321426293366619,
                                         0.159708344185841,
                                         0.0481074379115265,
                                         0.00784761671802496,
                                         0.000533188386280913,
                                         7.82193530213016e-06,
                                         2.23850035636455e-09,
                                         0.00242259793122461,
                                         0.0516351440383035,
                                         0.19161914170826,
                                         0.317640232094028,
                                         0.295793598575398,
                                         0.162342070807074,
                                         0.049632219461143,
                                         0.00698275423314323,
                                         0.000270282513922471,
                                         4.23342527500015e-07,
                                         4.26997524077325e-05,
                                         0.00498104434162094,
                                         0.0487721117564285,
                                         0.167412675526897,
                                         0.292178774266583,
                                         0.292178774266583,
                                         0.167412675526897,
                                         0.0487721117564285,
                                         0.00498104434162092,
                                         4.26997524077325e-05,
                                         4.23342527500015e-07,
                                         0.000270282513922472,
                                         0.00698275423314323,
                                         0.049632219461143,
                                         0.162342070807074,
                                         0.295793598575398,
                                         0.317640232094028,
                                         0.19161914170826,
                                         0.0516351440383034,
                                         0.00242259793122461,
                                         2.23850035636455e-09,
                                         7.82193530213019e-06,
                                         0.000533188386280913,
                                         0.00784761671802496,
                                         0.0481074379115265,
                                         0.159708344185841,
                                         0.321426293366619,
                                         0.401517934701504,
                                         0.285475673185297,
                                         0.0733054147366087,
                                         4.93186517608961e-12,
                                         9.43190130066711e-08,
                                         1.69638086069414e-05,
                                         0.000517012007032715,
                                         0.00593994102213164,
                                         0.0359298332314448,
                                         0.135523950826256,
                                         0.350557905405777,
                                         0.657629939666541,
                                         0.924228861993799,

                                         0.936437853053716,
                                         0.705566365777096,
                                         0.419107650971861,
                                         0.192013413677796,
                                         0.0654191410221913,
                                         0.015625,
                                         0.00237348997369225,
                                         0.000193272321106365,
                                         6.03302595636996e-06,
                                         3.24225182611974e-08,
                                         1.6639152760913e-12,
                                         0.061835648391359,
                                         0.253361486953526,
                                         0.392203778739559,
                                         0.364716855397745,
                                         0.225841204691613,
                                         0.09375,
                                         0.0247509320478836,
                                         0.00366309259758599,
                                         0.000232087371565582,
                                         3.25047264105798e-06,
                                         9.07138817434349e-10,
                                         0.00170132637896137,
                                         0.0379081060412853,
                                         0.152927897026068,
                                         0.288648203579908,
                                         0.324855545204382,
                                         0.234375,
                                         0.107543575780304,
                                         0.0289277656298593,
                                         0.00372011687762228,
                                         0.000135779762503713,
                                         2.06065388344487e-07,
                                         2.49652017721161e-05,
                                         0.00302497857042952,
                                         0.0318024359873673,
                                         0.121837396795999,
                                         0.249216111279933,
                                         0.3125,
                                         0.249216111279934,
                                         0.121837396795999,
                                         0.0318024359873673,
                                         0.00302497857042952,
                                         2.49652017721165e-05,
                                         2.06065388344483e-07,
                                         0.000135779762503713,
                                         0.00372011687762229,
                                         0.0289277656298594,
                                         0.107543575780304,
                                         0.234375,
                                         0.324855545204382,
                                         0.288648203579908,
                                         0.152927897026068,
                                         0.0379081060412853,
                                         0.00170132637896139,
                                         9.07138817434326e-10,
                                         3.25047264105798e-06,
                                         0.000232087371565582,
                                         0.00366309259758599,
                                         0.0247509320478836,
                                         0.09375,
                                         0.225841204691613,
                                         0.364716855397745,
                                         0.392203778739559,
                                         0.253361486953526,
                                         0.0618356483913593,
                                         1.66391527609125e-12,
                                         3.24225182611974e-08,
                                         6.03302595636998e-06,
                                         0.000193272321106366,
                                         0.00237348997369225,
                                         0.015625,
                                         0.0654191410221913,
                                         0.192013413677796,
                                         0.419107650971861,
                                         0.705566365777096,
                                         0.936437853053716,

                                         0.945941375076244,
                                         0.744701381503684,
                                         0.480304149526492,
                                         0.249921487886891,
                                         0.1023329820139,
                                         0.0317158046435623,
                                         0.00700121083733933,
                                         0.000997280072177191,
                                         7.71815461449814e-05,
                                         2.3189396905709e-06,
                                         1.21412315496825e-08,
                                         6.14180323756786e-13,
                                         0.0528146109528153,
                                         0.224998785356456,
                                         0.374653483462532,
                                         0.389858039255586,
                                         0.283770425919367,
                                         0.147936913965609,
                                         0.0540349602570074,
                                         0.012946969934934,
                                         0.00178120160847439,
                                         0.000107023344454753,
                                         1.44666251499846e-06,
                                         3.96012173575593e-10,
                                         0.00122866279189889,
                                         0.0283248061288119,
                                         0.121767662574666,
                                         0.253395062947723,
                                         0.327874312211486,
                                         0.287518725017728,
                                         0.173766188319194,
                                         0.0700338330109796,
                                         0.0171277944043345,
                                         0.00205805198830996,
                                         7.18224926269383e-05,
                                         1.06392235006816e-07,
                                         1.52443901813e-05,
                                         0.00190174571467425,
                                         0.0211073101638544,
                                         0.0878392323508453,
                                         0.202044196837156,
                                         0.298026196959559,
                                         0.298026196959559,
                                         0.202044196837156,
                                         0.0878392323508454,
                                         0.0211073101638545,
                                         0.00190174571467425,
                                         1.52443901812997e-05,
                                         1.06392235006818e-07,
                                         7.1822492626938e-05,
                                         0.00205805198830996,
                                         0.0171277944043345,
                                         0.0700338330109795,
                                         0.173766188319194,
                                         0.287518725017728,
                                         0.327874312211487,
                                         0.253395062947724,
                                         0.121767662574666,
                                         0.028324806128812,
                                         0.00122866279189887,
                                         3.96012173575605e-10,
                                         1.44666251499845e-06,
                                         0.000107023344454753,
                                         0.00178120160847439,
                                         0.012946969934934,
                                         0.0540349602570074,
                                         0.147936913965609,
                                         0.283770425919367,
                                         0.389858039255586,
                                         0.374653483462532,
                                         0.224998785356456,
                                         0.052814610952815,
                                         6.14180323756808e-13,
                                         1.21412315496825e-08,
                                         2.3189396905709e-06,
                                         7.71815461449813e-05,
                                         0.000997280072177189,
                                         0.00700121083733933,
                                         0.0317158046435623,
                                         0.1023329820139,
                                         0.249921487886891,
                                         0.480304149526492,
                                         0.744701381503684,
                                         0.945941375076244,

                                         0.953477489198502,
                                         0.77690153034159,
                                         0.534242674803509,
                                         0.306629028420349,
                                         0.144317162959809,
                                         0.0542277344394006,
                                         0.015625,
                                         0.00324498670927488,
                                         0.000439670550181651,
                                         3.27019925302698e-05,
                                         9.53581399770645e-07,
                                         4.89140831738389e-09,
                                         2.44655903206097e-13,
                                         0.0456039615029865,
                                         0.200306580686151,
                                         0.353041987216133,
                                         0.400643414629467,
                                         0.329688753217045,
                                         0.203487602341284,
                                         0.09375,
                                         0.0311314198908948,
                                         0.00692857190159343,
                                         0.000901014902303179,
                                         5.19484374655998e-05,
                                         6.82978728877576e-07,
                                         1.84147604512068e-10,
                                         0.000908831675072765,
                                         0.0215185605356662,
                                         0.0972081997375672,
                                         0.218118000918025,
                                         0.313818859588447,
                                         0.318158262977015,
                                         0.234375,
                                         0.124443923202766,
                                         0.04549344633889,
                                         0.0103437613347892,
                                         0.00117916876129925,
                                         3.97346321202839e-05,
                                         5.77517576780873e-08,
                                         9.65968728906999e-06,
                                         0.00123290593433581,
                                         0.0142750674626265,
                                         0.0633320778025359,
                                         0.159313535444034,
                                         0.265306070439365,
                                         0.3125,
                                         0.265306070439365,
                                         0.159313535444034,
                                         0.0633320778025359,
                                         0.0142750674626265,
                                         0.00123290593433581,
                                         9.65968728906979e-06,
                                         5.77517576780889e-08,
                                         3.97346321202841e-05,
                                         0.00117916876129925,
                                         0.0103437613347892,
                                         0.04549344633889,
                                         0.124443923202766,
                                         0.234375,
                                         0.318158262977015,
                                         0.313818859588447,
                                         0.218118000918025,
                                         0.0972081997375673,
                                         0.0215185605356662,
                                         0.000908831675072752,
                                         1.84147604512075e-10,
                                         6.82978728877581e-07,
                                         5.19484374655997e-05,
                                         0.00090101490230318,
                                         0.00692857190159343,
                                         0.0311314198908948,
                                         0.09375,
                                         0.203487602341284,
                                         0.329688753217045,
                                         0.400643414629467,
                                         0.353041987216133,
                                         0.20030658068615,
                                         0.0456039615029862,
                                         2.44655903206107e-13,
                                         4.89140831738393e-09,
                                         9.53581399770642e-07,
                                         3.27019925302699e-05,
                                         0.000439670550181651,
                                         0.00324498670927488,
                                         0.015625,
                                         0.0542277344394006,
                                         0.144317162959809,
                                         0.306629028420349,
                                         0.534242674803509,
                                         0.77690153034159,
                                         0.953477489198502,

                                         0.959550510119687,
                                         0.803618450543985,
                                         0.581484872387582,
                                         0.3605478479603,
                                         0.189113893948438,
                                         0.082321027345887,
                                         0.0289193023736069,
                                         0.00786771514546783,
                                         0.00155694293618327,
                                         0.000202737582130415,
                                         1.46098729033881e-05,
                                         4.15970587898748e-07,
                                         2.09907673726371e-09,
                                         1.04045003277217e-13,
                                         0.0397567909608296,
                                         0.178935928997575,
                                         0.329946636180797,
                                         0.40092340775171,
                                         0.363002674711358,
                                         0.254950392175547,
                                         0.139674094695753,
                                         0.0586439311882786,
                                         0.0180979879008854,
                                         0.00380233498534681,
                                         0.000472988338340877,
                                         2.63912426953643e-05,
                                         3.39377591557051e-07,
                                         9.04024597574588e-11,
                                         0.000686346719476824,
                                         0.0166009890766319,
                                         0.0780077492816105,
                                         0.185758492010357,
                                         0.290325358739303,
                                         0.328995036090824,
                                         0.281081767882982,
                                         0.182132010460318,
                                         0.0876549910855881,
                                         0.0297135982157879,
                                         0.00638033066423249,
                                         0.000697663199502414,
                                         2.28626606018395e-05,
                                         3.27286449484103e-08,
                                         6.31938085480084e-06,
                                         0.000821427244537897,
                                         0.00983627173722424,
                                         0.045902323402155,
                                         0.123839401817635,
                                         0.226423622465085,
                                         0.301681178253593,
                                         0.301681178253593,
                                         0.226423622465085,
                                         0.123839401817635,
                                         0.0459023234021551,
                                         0.00983627173722424,
                                         0.000821427244537893,
                                         6.31938085480084e-06,
                                         3.27286449484103e-08,
                                         2.28626606018396e-05,
                                         0.000697663199502414,
                                         0.00638033066423248,
                                         0.0297135982157879,
                                         0.0876549910855881,
                                         0.182132010460317,
                                         0.281081767882983,
                                         0.328995036090824,
                                         0.290325358739303,
                                         0.185758492010357,
                                         0.0780077492816105,
                                         0.0166009890766318,
                                         0.000686346719476824,
                                         9.04024597574588e-11,
                                         3.39377591557054e-07,
                                         2.63912426953643e-05,
                                         0.000472988338340876,
                                         0.00380233498534681,
                                         0.0180979879008854,
                                         0.0586439311882785,
                                         0.139674094695753,
                                         0.254950392175547,
                                         0.363002674711359,
                                         0.40092340775171,
                                         0.329946636180797,
                                         0.178935928997575,
                                         0.0397567909608296,
                                         1.04045003277217e-13,
                                         2.09907673726373e-09,
                                         4.15970587898748e-07,
                                         1.46098729033881e-05,
                                         0.000202737582130415,
                                         0.00155694293618327,
                                         0.00786771514546783,
                                         0.0289193023736069,
                                         0.082321027345887,
                                         0.189113893948438,
                                         0.3605478479603,
                                         0.581484872387582,
                                         0.803618450543985,
                                         0.959550510119687,

                                         0.964513919005733,
                                         0.825972347607508,
                                         0.622763432033523,
                                         0.410842066574916,
                                         0.234871855881689,
                                         0.114730678586702,
                                         0.046935252136009,
                                         0.015625,
                                         0.00405945404289132,
                                         0.000772689312952027,
                                         9.74389955919345e-05,
                                         6.8442781683579e-06,
                                         1.91133865676399e-07,
                                         9.51767408520104e-10,
                                         4.68308119072925e-14,
                                         0.0349540052987262,
                                         0.160464011551945,
                                         0.306885788387296,
                                         0.393944413932996,
                                         0.384856831207909,
                                         0.299147264963753,
                                         0.187274430302237,
                                         0.09375,
                                         0.0366261104320834,
                                         0.010668464884702,
                                         0.00214075555237378,
                                         0.000256962714403866,
                                         1.39632486095607e-05,
                                         1.76368569592556e-07,
                                         4.65206462987081e-11,
                                         0.000527805795207727,
                                         0.0129890844600336,
                                         0.0630114116303683,
                                         0.157392395901675,
                                         0.262758139559731,
                                         0.32499693789278,
                                         0.311348352685565,
                                         0.234375,
                                         0.137690104712919,
                                         0.0613744646394433,
                                         0.0195970237034756,
                                         0.00401977114858926,
                                         0.000425034375432843,
                                         1.3617591888202e-05,
                                         1.92552228962504e-08,
                                         4.2505985424606e-06,
                                         0.000560761468288135,
                                         0.00690017919090419,
                                         0.0335375454492528,
                                         0.0956779550992293,
                                         0.188309499719668,
                                         0.276066295688297,
                                         0.3125,
                                         0.276066295688297,
                                         0.188309499719668,
                                         0.0956779550992293,
                                         0.0335375454492528,
                                         0.0069001791909042,
                                         0.000560761468288135,
                                         4.25059854246072e-06,
                                         1.92552228962497e-08,
                                         1.3617591888202e-05,
                                         0.000425034375432842,
                                         0.00401977114858927,
                                         0.0195970237034756,
                                         0.0613744646394432,
                                         0.137690104712919,
                                         0.234375,
                                         0.311348352685565,
                                         0.32499693789278,
                                         0.262758139559731,
                                         0.157392395901675,
                                         0.0630114116303684,
                                         0.0129890844600336,
                                         0.000527805795207737,
                                         4.6520646298706e-11,
                                         1.76368569592556e-07,
                                         1.39632486095607e-05,
                                         0.000256962714403866,
                                         0.00214075555237378,
                                         0.010668464884702,
                                         0.0366261104320834,
                                         0.09375,
                                         0.187274430302237,
                                         0.299147264963753,
                                         0.384856831207909,
                                         0.393944413932995,
                                         0.306885788387297,
                                         0.160464011551945,
                                         0.0349540052987265,
                                         4.68308119072899e-14,
                                         9.51767408520104e-10,
                                         1.91133865676398e-07,
                                         6.84427816835792e-06,
                                         9.74389955919345e-05,
                                         0.000772689312952027,
                                         0.00405945404289132,
                                         0.015625,
                                         0.046935252136009,
                                         0.114730678586702,
                                         0.234871855881689,
                                         0.410842066574916,
                                         0.622763432033523,
                                         0.825972347607508,
                                         0.964513919005733,

                                         0.968621115700728,
                                         0.844827894946839,
                                         0.658832183335224,
                                         0.45717569525536,
                                         0.28021528122254,
                                         0.150104998707654,
                                         0.0692376370717168,
                                         0.0269360872738176,
                                         0.00858371879171459,
                                         0.00214785564036788,
                                         0.000396036380091269,
                                         4.86456768668548e-05,
                                         3.34589092360918e-06,
                                         9.19624666642103e-08,
                                         4.52953012034904e-10,
                                         2.21526334365978e-14,
                                         0.0309635267377078,
                                         0.144477531683551,
                                         0.284707357562356,
                                         0.382213326346221,
                                         0.39710175363652,
                                         0.334787879547122,
                                         0.232864863371416,
                                         0.133570101516051,
                                         0.0623165263168637,
                                         0.0229903648736792,
                                         0.00639238629092303,
                                         0.00123576747861812,
                                         0.000144075982016138,
                                         7.66106642194578e-06,
                                         9.53506338839978e-08,
                                         2.49477494306772e-11,
                                         0.000412416153788181,
                                         0.0102948764704147,
                                         0.0512638634822729,
                                         0.133142673915044,
                                         0.234477164089132,
                                         0.311123672475516,
                                         0.326328081122631,
                                         0.275976917719567,
                                         0.188503644051338,
                                         0.102535770074375,
                                         0.0429912129989208,
                                         0.0130803098340233,
                                         0.00258499767175157,
                                         0.000265923428919882,
                                         8.36339820808646e-06,
                                         1.17064750532777e-08,
                                         2.92967633127402e-06,
                                         0.000391237697400073,
                                         0.00492291916233749,
                                         0.0247358849386848,
                                         0.0738410780622995,
                                         0.154203813599773,
                                         0.243895427845814,
                                         0.304113004330648,
                                         0.304113004330648,
                                         0.243895427845814,
                                         0.154203813599773,
                                         0.0738410780622995,
                                         0.0247358849386847,
                                         0.0049229191623375,
                                         0.000391237697400073,
                                         2.92967633127393e-06,
                                         1.17064750532782e-08,
                                         8.36339820808646e-06,
                                         0.000265923428919881,
                                         0.00258499767175157,
                                         0.0130803098340233,
                                         0.0429912129989208,
                                         0.102535770074375,
                                         0.188503644051338,
                                         0.275976917719567,
                                         0.326328081122631,
                                         0.311123672475516,
                                         0.234477164089132,
                                         0.133142673915044,
                                         0.051263863482273,
                                         0.0102948764704147,
                                         0.000412416153788172,
                                         2.49477494306785e-11,
                                         9.53506338839979e-08,
                                         7.66106642194575e-06,
                                         0.000144075982016139,
                                         0.00123576747861812,
                                         0.00639238629092303,
                                         0.0229903648736793,
                                         0.0623165263168636,
                                         0.133570101516051,
                                         0.232864863371416,
                                         0.334787879547123,
                                         0.39710175363652,
                                         0.382213326346221,
                                         0.284707357562357,
                                         0.144477531683551,
                                         0.0309635267377075,
                                         2.21526334365992e-14,
                                         4.52953012034904e-10,
                                         9.19624666642099e-08,
                                         3.34589092360919e-06,
                                         4.86456768668548e-05,
                                         0.000396036380091269,
                                         0.00214785564036788,
                                         0.00858371879171458,
                                         0.0269360872738176,
                                         0.0692376370717168,
                                         0.150104998707654,
                                         0.28021528122254,
                                         0.45717569525536,
                                         0.658832183335224,
                                         0.844827894946839,
                                         0.968621115700728,

                                         0.972057421816389,
                                         0.860855446127013,
                                         0.69039649618559,
                                         0.499522976595493,
                                         0.324197987915755,
                                         0.187206346021014,
                                         0.0951040693223257,
                                         0.0418564672898566,
                                         0.015625,
                                         0.00480303341270715,
                                         0.00116508469965099,
                                         0.00020924211220552,
                                         2.51466978027399e-05,
                                         1.69965846720885e-06,
                                         4.61008744633972e-08,
                                         2.25006479756521e-10,
                                         1.09490581909576e-14,
                                         0.0276136603147861,
                                         0.130604742528048,
                                         0.263846655304091,
                                         0.367570869089469,
                                         0.401704478371663,
                                         0.361848329305527,
                                         0.273974462463315,
                                         0.175067687077512,
                                         0.09375,
                                         0.0413404010433632,
                                         0.0145595856695794,
                                         0.00389713626163808,
                                         0.000730612511673111,
                                         8.3153119609848e-05,
                                         4.34268820994449e-06,
                                         5.33910927847369e-08,
                                         1.38754541622991e-11,
                                         0.000326847218961765,
                                         0.00825612807951666,
                                         0.0420139356513346,
                                         0.112697805484362,
                                         0.207391899855783,
                                         0.29142132955637,
                                         0.32885907080933,
                                         0.305097167406522,
                                         0.234375,
                                         0.148259496618724,
                                         0.0758104879031977,
                                         0.0302434161111429,
                                         0.00884467757954704,
                                         0.00169505654475309,
                                         0.000170449956285214,
                                         5.27875314197065e-06,
                                         7.32666624731036e-09,
                                         2.06330931044467e-06,
                                         0.000278350896181256,
                                         0.00356807411361486,
                                         0.0184284395078466,
                                         0.057105197067776,
                                         0.125174200632103,
                                         0.210527239132601,
                                         0.283575747151314,
                                         0.3125,
                                         0.283575747151314,
                                         0.210527239132601,
                                         0.125174200632102,
                                         0.0571051970677759,
                                         0.0184284395078466,
                                         0.00356807411361486,
                                         0.000278350896181254,
                                         2.06330931044474e-06,
                                         7.32666624731001e-09,
                                         5.2787531419707e-06,
                                         0.000170449956285214,
                                         0.00169505654475309,
                                         0.00884467757954705,
                                         0.0302434161111429,
                                         0.0758104879031977,
                                         0.148259496618724,
                                         0.234375,
                                         0.305097167406523,
                                         0.32885907080933,
                                         0.29142132955637,
                                         0.207391899855783,
                                         0.112697805484362,
                                         0.0420139356513346,
                                         0.00825612807951662,
                                         0.000326847218961773,
                                         1.38754541622983e-11,
                                         5.33910927847375e-08,
                                         4.34268820994449e-06,
                                         8.31531196098482e-05,
                                         0.000730612511673112,
                                         0.00389713626163808,
                                         0.0145595856695794,
                                         0.0413404010433632,
                                         0.09375,
                                         0.175067687077512,
                                         0.273974462463315,
                                         0.361848329305527,
                                         0.401704478371663,
                                         0.367570869089469,
                                         0.263846655304091,
                                         0.130604742528048,
                                         0.0276136603147864,
                                         1.09490581909569e-14,
                                         2.25006479756524e-10,
                                         4.61008744633972e-08,
                                         1.69965846720886e-06,
                                         2.51466978027399e-05,
                                         0.000209242112205521,
                                         0.00116508469965099,
                                         0.00480303341270715,
                                         0.015625,
                                         0.0418564672898566,
                                         0.0951040693223257,
                                         0.187206346021014,
                                         0.324197987915755,
                                         0.499522976595493,
                                         0.69039649618559,
                                         0.860855446127013,
                                         0.972057421816389,

                                         0.974960808669954,
                                         0.874578074075131,
                                         0.718085961603359,
                                         0.538038089382177,
                                         0.366218402546375,
                                         0.225001178393878,
                                         0.123699785988714,
                                         0.0601463381917086,
                                         0.0254599803429195,
                                         0.00918305608732532,
                                         0.00273921380931099,
                                         0.000647420573551661,
                                         0.000113734681366832,
                                         1.34205614444865e-05,
                                         8.93873362265076e-07,
                                         2.39765046374449e-08,
                                         1.16129173827628e-10,
                                         5.62702076651821e-15,
                                         0.0247753787254001,
                                         0.1185241803974,
                                         0.244490839943744,
                                         0.351324228403018,
                                         0.400463711197528,
                                         0.381025519127778,
                                         0.309259552624371,
                                         0.215656837587999,
                                         0.128883555929084,
                                         0.0653055801284339,
                                         0.0275026405463803,
                                         0.00932255215945078,
                                         0.00241783213379856,
                                         0.000441824893736348,
                                         4.92814431104841e-05,
                                         2.53514156247093e-06,
                                         3.08486001639875e-08,
                                         7.97164362242016e-12,
                                         0.000262326523593114,
                                         0.00669274217062284,
                                         0.0346846652145753,
                                         0.095585482932311,
                                         0.182463049907526,
                                         0.26885127014099,
                                         0.322155875631265,
                                         0.322185529747869,
                                         0.271847731963274,
                                         0.19350941720615,
                                         0.115056619903781,
                                         0.0559334884180619,
                                         0.021416482044638,
                                         0.00606063928884449,
                                         0.00113208646100311,
                                         0.000111688206832603,
                                         3.41443103367697e-06,
                                         4.70550230453718e-09,
                                         1.48136757314682e-06,
                                         0.000201557961083261,
                                         0.00262428591342232,
                                         0.0138699375050174,
                                         0.0443389516045444,
                                         0.101173983477551,
                                         0.178981324604586,
                                         0.256712820212951,
                                         0.305810678342814,
                                         0.305810678342814,
                                         0.256712820212951,
                                         0.178981324604586,
                                         0.101173983477551,
                                         0.0443389516045444,
                                         0.0138699375050175,
                                         0.00262428591342232,
                                         0.000201557961083261,
                                         1.48136757314682e-06,
                                         4.70550230453718e-09,
                                         3.41443103367696e-06,
                                         0.000111688206832603,
                                         0.0011320864610031,
                                         0.00606063928884449,
                                         0.021416482044638,
                                         0.0559334884180619,
                                         0.115056619903781,
                                         0.19350941720615,
                                         0.271847731963274,
                                         0.322185529747869,
                                         0.322155875631265,
                                         0.26885127014099,
                                         0.182463049907526,
                                         0.0955854829323111,
                                         0.0346846652145753,
                                         0.00669274217062284,
                                         0.000262326523593114,
                                         7.97164362242016e-12,
                                         3.08486001639875e-08,
                                         2.53514156247093e-06,
                                         4.9281443110484e-05,
                                         0.000441824893736348,
                                         0.00241783213379857,
                                         0.00932255215945078,
                                         0.0275026405463803,
                                         0.0653055801284339,
                                         0.128883555929084,
                                         0.215656837587999,
                                         0.309259552624371,
                                         0.381025519127778,
                                         0.400463711197528,
                                         0.351324228403018,
                                         0.244490839943744,
                                         0.1185241803974,
                                         0.0247753787254001,
                                         5.62702076651821e-15,
                                         1.16129173827628e-10,
                                         2.39765046374449e-08,
                                         8.93873362265073e-07,
                                         1.34205614444865e-05,
                                         0.000113734681366832,
                                         0.000647420573551661,
                                         0.00273921380931099,
                                         0.00918305608732532,
                                         0.0254599803429195,
                                         0.0601463381917086,
                                         0.123699785988714,
                                         0.225001178393878,
                                         0.366218402546375,
                                         0.538038089382177,
                                         0.718085961603359,
                                         0.874578074075131,
                                         0.974960808669954,

                                         0.977435650241118,
                                         0.886406989879932,
                                         0.742448313231084,
                                         0.572970339357278,
                                         0.405932699205302,
                                         0.262680513242844,
                                         0.154199209865131,
                                         0.0813714542256584,
                                         0.0381393669996204,
                                         0.015625,
                                         0.00547498519255554,
                                         0.00159223348901111,
                                         0.000368156628013724,
                                         6.34782884281394e-05,
                                         7.37498461918577e-06,
                                         4.85128763070724e-07,
                                         1.2890178567556e-08,
                                         6.20275562214327e-11,
                                         2.99470477117429e-15,
                                         0.022350319307711,
                                         0.107963346799201,
                                         0.226681595147296,
                                         0.334377878762335,
                                         0.394902425029427,
                                         0.393349556865857,
                                         0.33824013939101,
                                         0.253442303603072,
                                         0.165587027160815,
                                         0.09375,
                                         0.0453975714984511,
                                         0.0184035762559179,
                                         0.006042158701056,
                                         0.00152607859179978,
                                         0.000272915282458093,
                                         2.99263759592852e-05,
                                         1.51988734606524e-06,
                                         1.83334418307343e-08,
                                         4.71477485130109e-12,
                                         0.000212945294247509,
                                         0.005479087853718,
                                         0.0288373304145694,
                                         0.0813077034166513,
                                         0.160071613676584,
                                         0.245424679547772,
                                         0.309141207649815,
                                         0.328908347256015,
                                         0.299549015022429,
                                         0.234375,
                                         0.156845134847677,
                                         0.0886311642693745,
                                         0.0413181046131922,
                                         0.0152868269740897,
                                         0.00420807382688278,
                                         0.000769201263519256,
                                         7.46711127333135e-05,
                                         2.25783445990399e-06,
                                         3.0928343522135e-09,
                                         1.08205937088367e-06,
                                         0.000148299237219682,
                                         0.00195655731679186,
                                         0.0105444656954937,
                                         0.034604897994727,
                                         0.0816688664892098,
                                         0.150691023151782,
                                         0.227650920900951,
                                         0.289006899278452,
                                         0.3125,
                                         0.289006899278452,
                                         0.227650920900952,
                                         0.150691023151782,
                                         0.0816688664892097,
                                         0.0346048979947271,
                                         0.0105444656954937,
                                         0.00195655731679186,
                                         0.000148299237219682,
                                         1.08205937088367e-06,
                                         3.0928343522135e-09,
                                         2.25783445990399e-06,
                                         7.46711127333135e-05,
                                         0.000769201263519256,
                                         0.00420807382688277,
                                         0.0152868269740897,
                                         0.0413181046131922,
                                         0.0886311642693745,
                                         0.156845134847677,
                                         0.234375,
                                         0.299549015022429,
                                         0.328908347256015,
                                         0.309141207649815,
                                         0.245424679547772,
                                         0.160071613676584,
                                         0.0813077034166513,
                                         0.0288373304145694,
                                         0.005479087853718,
                                         0.000212945294247509,
                                         4.71477485130109e-12,
                                         1.83334418307343e-08,
                                         1.51988734606524e-06,
                                         2.99263759592852e-05,
                                         0.000272915282458093,
                                         0.00152607859179978,
                                         0.006042158701056,
                                         0.0184035762559179,
                                         0.0453975714984511,
                                         0.09375,
                                         0.165587027160815,
                                         0.253442303603072,
                                         0.33824013939101,
                                         0.393349556865857,
                                         0.394902425029427,
                                         0.334377878762335,
                                         0.226681595147296,
                                         0.107963346799201,
                                         0.022350319307711,
                                         2.99470477117429e-15,
                                         6.20275562214327e-11,
                                         1.2890178567556e-08,
                                         4.85128763070724e-07,
                                         7.37498461918576e-06,
                                         6.34782884281395e-05,
                                         0.000368156628013724,
                                         0.00159223348901111,
                                         0.00547498519255554,
                                         0.015625,
                                         0.0381393669996204,
                                         0.0813714542256584,
                                         0.154199209865131,
                                         0.262680513242844,
                                         0.405932699205302,
                                         0.572970339357278,
                                         0.742448313231084,
                                         0.886406989879932,
                                         0.977435650241118,

                                         0.979562049099076,
                                         0.896668019554143,
                                         0.76395332304625,
                                         0.604611132082384,
                                         0.443180618546272,
                                         0.299643817580666,
                                         0.185856366597293,
                                         0.10499846864745,
                                         0.0535248885784691,
                                         0.0243202719552969,
                                         0.00969095531437596,
                                         0.00331319161993609,
                                         0.000942951082806277,
                                         0.000213984245592755,
                                         3.63114854257522e-05,
                                         4.1630832505185e-06,
                                         2.70945173048897e-07,
                                         7.14110048667623e-09,
                                         3.41716705094067e-11,
                                         1.64471903375808e-15,
                                         0.0202625062883993,
                                         0.0986935306988044,
                                         0.210378390418692,
                                         0.317342522383551,
                                         0.386252288644223,
                                         0.399941392337855,
                                         0.361018468268403,
                                         0.287222581955343,
                                         0.201986411089171,
                                         0.12517551054417,
                                         0.0677825881292264,
                                         0.0316069561721389,
                                         0.0124095504088961,
                                         0.00396581384072885,
                                         0.000979389589685194,
                                         0.000171959942021362,
                                         1.85836830086352e-05,
                                         9.33540881747693e-07,
                                         1.11766514028696e-08,
                                         2.86241769220901e-12,
                                         0.000174639762681458,
                                         0.00452620739103975,
                                         0.02413927320557,
                                         0.0694015480701794,
                                         0.140265263039817,
                                         0.222421182417801,
                                         0.292193233268112,
                                         0.32737307445125,
                                         0.317597658368405,
                                         0.268447320901583,
                                         0.197541552253742,
                                         0.125634105653134,
                                         0.0680474240918155,
                                         0.0306246833260814,
                                         0.0110066640203169,
                                         0.0029595674528315,
                                         0.000531093416242418,
                                         5.08499226601634e-05,
                                         1.52316151945806e-06,
                                         2.07569268582754e-09,
                                         8.02771286706285e-07,
                                         0.000110707983669699,
                                         0.00147722272484599,
                                         0.00809484941946051,
                                         0.0271661392915844,
                                         0.0659712425682498,
                                         0.126127450453789,
                                         0.199005949362439,
                                         0.266336788518745,
                                         0.307041800901606,
                                         0.307041800901606,
                                         0.266336788518745,
                                         0.199005949362439,
                                         0.126127450453789,
                                         0.0659712425682498,
                                         0.0271661392915844,
                                         0.00809484941946053,
                                         0.00147722272484599,
                                         0.0001107079836697,
                                         8.02771286706324e-07,
                                         2.07569268582741e-09,
                                         1.52316151945804e-06,
                                         5.08499226601634e-05,
                                         0.000531093416242416,
                                         0.0029595674528315,
                                         0.0110066640203169,
                                         0.0306246833260814,
                                         0.0680474240918155,
                                         0.125634105653134,
                                         0.197541552253742,
                                         0.268447320901583,
                                         0.317597658368405,
                                         0.32737307445125,
                                         0.292193233268112,
                                         0.222421182417801,
                                         0.140265263039817,
                                         0.0694015480701795,
                                         0.02413927320557,
                                         0.00452620739103978,
                                         0.000174639762681463,
                                         2.86241769220878e-12,
                                         1.11766514028695e-08,
                                         9.33540881747693e-07,
                                         1.85836830086351e-05,
                                         0.000171959942021362,
                                         0.000979389589685195,
                                         0.00396581384072885,
                                         0.0124095504088961,
                                         0.0316069561721389,
                                         0.0677825881292264,
                                         0.12517551054417,
                                         0.201986411089172,
                                         0.287222581955343,
                                         0.361018468268403,
                                         0.399941392337855,
                                         0.386252288644223,
                                         0.317342522383552,
                                         0.210378390418692,
                                         0.0986935306988047,
                                         0.0202625062883996,
                                         1.64471903375792e-15,
                                         3.41716705094061e-11,
                                         7.14110048667623e-09,
                                         2.70945173048896e-07,
                                         4.1630832505185e-06,
                                         3.63114854257523e-05,
                                         0.000213984245592755,
                                         0.000942951082806277,
                                         0.00331319161993609,
                                         0.00969095531437596,
                                         0.0243202719552969,
                                         0.0535248885784691,
                                         0.10499846864745,
                                         0.185856366597293,
                                         0.299643817580666,
                                         0.443180618546272,
                                         0.604611132082384,
                                         0.76395332304625,
                                         0.896668019554143,
                                         0.979562049099076,

                                         0.981402286216887,
                                         0.905621388085949,
                                         0.783000804188452,
                                         0.633262061621569,
                                         0.477927421878223,
                                         0.335467604414565,
                                         0.218037043487174,
                                         0.130468578259818,
                                         0.0713441437499665,
                                         0.0353127295520003,
                                         0.015625,
                                         0.00608000308311431,
                                         0.00203526972180058,
                                         0.000568599860539125,
                                         0.000126970896819501,
                                         2.12519624709268e-05,
                                         2.40880521308376e-06,
                                         1.55337012721933e-07,
                                         4.06557911814258e-09,
                                         1.93611009501178e-11,
                                         9.293835729738e-16,
                                         0.0184525462915543,
                                         0.090523600128933,
                                         0.195497028209047,
                                         0.300619144074091,
                                         0.375479722982245,
                                         0.401877087896897,
                                         0.378045307716001,
                                         0.316378757919445,
                                         0.236621128198707,
                                         0.158031849890968,
                                         0.09375,
                                         0.048909470901651,
                                         0.02209170736979,
                                         0.00844127138016552,
                                         0.00263628962431273,
                                         0.000638644066909652,
                                         0.000110377268901452,
                                         1.17799727629276e-05,
                                         5.86201554743653e-07,
                                         6.97296368377302e-09,
                                         1.77946010049735e-12,
                                         0.000144562050574634,
                                         0.00377021157705791,
                                         0.0203379357094985,
                                         0.0594618689880849,
                                         0.122913556812282,
                                         0.200597106411037,
                                         0.273115392840906,
                                         0.319666747215176,
                                         0.326992257569699,
                                         0.294677324305785,
                                         0.234375,
                                         0.163934699208391,
                                         0.0999137709048275,
                                         0.0522152948692543,
                                         0.0228071391336858,
                                         0.00799663883505801,
                                         0.0021073970309653,
                                         0.000372222294458676,
                                         3.52176418986272e-05,
                                         1.04638984331884e-06,
                                         1.41961400605658e-09,
                                         6.04019590175353e-07,
                                         8.37468258916539e-05,
                                         0.00112842398397011,
                                         0.00627276771202083,
                                         0.0214591152221702,
                                         0.0534016664130626,
                                         0.105231856301101,
                                         0.172260750495602,
                                         0.241001722485209,
                                         0.293053923058091,
                                         0.3125,
                                         0.293053923058091,
                                         0.241001722485209,
                                         0.172260750495602,
                                         0.105231856301101,
                                         0.0534016664130626,
                                         0.0214591152221702,
                                         0.00627276771202082,
                                         0.00112842398397011,
                                         8.37468258916548e-05,
                                         6.04019590175321e-07,
                                         1.41961400605668e-09,
                                         1.04638984331882e-06,
                                         3.52176418986274e-05,
                                         0.000372222294458676,
                                         0.0021073970309653,
                                         0.007996638835058,
                                         0.0228071391336858,
                                         0.0522152948692542,
                                         0.0999137709048275,
                                         0.163934699208391,
                                         0.234375,
                                         0.294677324305785,
                                         0.326992257569699,
                                         0.319666747215176,
                                         0.273115392840906,
                                         0.200597106411037,
                                         0.122913556812282,
                                         0.059461868988085,
                                         0.0203379357094984,
                                         0.00377021157705793,
                                         0.000144562050574629,
                                         1.77946010049751e-12,
                                         6.9729636837729e-09,
                                         5.86201554743658e-07,
                                         1.17799727629276e-05,
                                         0.000110377268901452,
                                         0.000638644066909652,
                                         0.00263628962431273,
                                         0.00844127138016551,
                                         0.02209170736979,
                                         0.048909470901651,
                                         0.09375,
                                         0.158031849890968,
                                         0.236621128198707,
                                         0.316378757919445,
                                         0.378045307716001,
                                         0.401877087896897,
                                         0.375479722982245,
                                         0.300619144074091,
                                         0.195497028209047,
                                         0.0905236001289333,
                                         0.018452546291554,
                                         9.29383572973899e-16,
                                         1.93611009501174e-11,
                                         4.06557911814261e-09,
                                         1.55337012721933e-07,
                                         2.40880521308376e-06,
                                         2.12519624709268e-05,
                                         0.000126970896819501,
                                         0.000568599860539124,
                                         0.00203526972180058,
                                         0.00608000308311431,
                                         0.015625,
                                         0.0353127295520003,
                                         0.0713441437499665,
                                         0.130468578259818,
                                         0.218037043487174,
                                         0.335467604414565,
                                         0.477927421878223,
                                         0.633262061621569,
                                         0.783000804188452,
                                         0.905621388085949,
                                         0.981402286216887,

                                         0.983005362216885,
                                         0.913476551524573,
                                         0.79992976855372,
                                         0.659216583917199,
                                         0.51022075224158,
                                         0.369870656862231,
                                         0.250226258305671,
                                         0.157246838652346,
                                         0.0912452995347586,
                                         0.0485199720352232,
                                         0.0234146498759706,
                                         0.0101262383377984,
                                         0.0038603820413856,
                                         0.00126890623989645,
                                         0.000348851319265737,
                                         7.6820417285542e-05,
                                         1.2705622567556e-05,
                                         1.42588966659826e-06,
                                         9.12201721407436e-08,
                                         2.3730110783671e-09,
                                         1.12534963291082e-11,
                                         5.38943505887117e-16,
                                         0.0168734943331811,
                                         0.0832939898699682,
                                         0.181932837815798,
                                         0.284461285692348,
                                         0.363326718389408,
                                         0.400121732750269,
                                         0.389949255558054,
                                         0.340729378556393,
                                         0.268471541787507,
                                         0.190922340508149,
                                         0.12217030683181,
                                         0.0698670890660645,
                                         0.0353180388162654,
                                         0.0155254529032003,
                                         0.00579582431218419,
                                         0.00177461418626883,
                                         0.000422820649040302,
                                         7.20857140787499e-05,
                                         7.61023984053795e-06,
                                         3.7561509025742e-07,
                                         4.44297339187163e-09,
                                         1.1303074779466e-12,
                                         0.000120682120174215,
                                         0.00316459899676643,
                                         0.0172408664156614,
                                         0.0511454461198671,
                                         0.107801626155334,
                                         0.180352913071076,
                                         0.253204877690679,
                                         0.307628096072838,
                                         0.329135533178452,
                                         0.313026940042406,
                                         0.26560265956073,
                                         0.200856509747572,
                                         0.134633033343024,
                                         0.0791494281553994,
                                         0.0401216526002263,
                                         0.0170812635786697,
                                         0.00586280143247278,
                                         0.00151845262410807,
                                         0.00026454195506392,
                                         2.47727976836525e-05,
                                         7.30884428550886e-07,
                                         9.87731073809429e-10,
                                         4.60340898126162e-07,
                                         6.41242700365725e-05,
                                         0.000871376429034798,
                                         0.00490444085550882,
                                         0.0170589389858251,
                                         0.0433563696123443,
                                         0.0876869102633715,
                                         0.148129358486746,
                                         0.215203838200786,
                                         0.273719293213546,
                                         0.307962546580055,
                                         0.307962546580055,
                                         0.273719293213546,
                                         0.215203838200786,
                                         0.148129358486746,
                                         0.0876869102633716,
                                         0.0433563696123443,
                                         0.0170589389858251,
                                         0.00490444085550881,
                                         0.000871376429034801,
                                         6.41242700365724e-05,
                                         4.60340898126162e-07,
                                         9.87731073809429e-10,
                                         7.30884428550886e-07,
                                         2.47727976836524e-05,
                                         0.000264541955063921,
                                         0.00151845262410807,
                                         0.00586280143247278,
                                         0.0170812635786697,
                                         0.0401216526002263,
                                         0.0791494281553994,
                                         0.134633033343024,
                                         0.200856509747572,
                                         0.26560265956073,
                                         0.313026940042406,
                                         0.329135533178452,
                                         0.307628096072838,
                                         0.253204877690679,
                                         0.180352913071076,
                                         0.107801626155334,
                                         0.051145446119867,
                                         0.0172408664156615,
                                         0.00316459899676643,
                                         0.000120682120174215,
                                         1.1303074779466e-12,
                                         4.44297339187163e-09,
                                         3.75615090257418e-07,
                                         7.61023984053798e-06,
                                         7.20857140787501e-05,
                                         0.000422820649040302,
                                         0.00177461418626883,
                                         0.00579582431218419,
                                         0.0155254529032003,
                                         0.0353180388162654,
                                         0.0698670890660645,
                                         0.12217030683181,
                                         0.190922340508149,
                                         0.268471541787507,
                                         0.340729378556393,
                                         0.389949255558054,
                                         0.400121732750269,
                                         0.363326718389408,
                                         0.284461285692348,
                                         0.181932837815799,
                                         0.0832939898699682,
                                         0.0168734943331811,
                                         5.38943505887117e-16,
                                         1.12534963291082e-11,
                                         2.37301107836708e-09,
                                         9.12201721407441e-08,
                                         1.42588966659826e-06,
                                         1.2705622567556e-05,
                                         7.6820417285542e-05,
                                         0.000348851319265737,
                                         0.00126890623989645,
                                         0.0038603820413856,
                                         0.0101262383377984,
                                         0.0234146498759706,
                                         0.0485199720352232,
                                         0.0912452995347586,
                                         0.157246838652346,
                                         0.250226258305671,
                                         0.369870656862231,
                                         0.51022075224158,
                                         0.659216583917199,
                                         0.79992976855372,
                                         0.913476551524573,
                                         0.983005362216885,

                                         0.984410247379471,
                                         0.920403376507279,
                                         0.815027333157476,
                                         0.682750218151206,
                                         0.540159664424491,
                                         0.402681714264921,
                                         0.282021698355784,
                                         0.184850777371003,
                                         0.112842606226361,
                                         0.0637649706786781,
                                         0.0330971108092022,
                                         0.015625,
                                         0.00662464501856691,
                                         0.00248088422078279,
                                         0.00080262514199532,
                                         0.000217593664039002,
                                         4.73364216804371e-05,
                                         7.74811930639151e-06,
                                         8.62022390353783e-07,
                                         5.47635678108041e-08,
                                         1.41707583458805e-09,
                                         6.69558782682509e-12,
                                         3.20008464088613e-16,
                                         0.0154878666079712,
                                         0.0768713553299151,
                                         0.169574382320453,
                                         0.269020109893535,
                                         0.350353236507141,
                                         0.395505224053674,
                                         0.397422628054756,
                                         0.360393286592369,
                                         0.296914199156356,
                                         0.222708847314842,
                                         0.151880991038753,
                                         0.09375,
                                         0.0519698871798051,
                                         0.0255713520773776,
                                         0.0109813921722315,
                                         0.00401784644725615,
                                         0.00120928275187258,
                                         0.000283993555319825,
                                         4.78450556664249e-05,
                                         5.00347042396582e-06,
                                         2.45192692501692e-07,
                                         2.88605681809872e-09,
                                         7.3223164259791e-13,
                                         0.000101530337878636,
                                         0.00267509759900154,
                                         0.0147006680888247,
                                         0.044166847554624,
                                         0.0946844967140477,
                                         0.161856929754158,
                                         0.233351940368985,
                                         0.292766150053764,
                                         0.325519930698192,
                                         0.324101868055451,
                                         0.290405955823643,
                                         0.234375,
                                         0.169875088018046,
                                         0.109822074993031,
                                         0.0626023738286668,
                                         0.0309121479268566,
                                         0.0128720894861253,
                                         0.0043371997918613,
                                         0.00110648196677728,
                                         0.000190475752435071,
                                         1.76770875009035e-05,
                                         5.18334123699816e-07,
                                         6.98110673349207e-10,
                                         3.54975836528086e-07,
                                         4.96493369281438e-05,
                                         0.000679692735975953,
                                         0.00386729041420811,
                                         0.0136474133094868,
                                         0.035327190460759,
                                         0.0730750245607963,
                                         0.126842197944713,
                                         0.190336872776197,
                                         0.251550002659837,
                                         0.296146322111984,
                                         0.3125,
                                         0.296146322111984,
                                         0.251550002659837,
                                         0.190336872776197,
                                         0.126842197944713,
                                         0.0730750245607962,
                                         0.035327190460759,
                                         0.0136474133094868,
                                         0.00386729041420811,
                                         0.000679692735975956,
                                         4.96493369281438e-05,
                                         3.54975836528063e-07,
                                         6.98110673349266e-10,
                                         5.18334123699816e-07,
                                         1.76770875009034e-05,
                                         0.000190475752435071,
                                         0.00110648196677728,
                                         0.00433719979186129,
                                         0.0128720894861253,
                                         0.0309121479268565,
                                         0.0626023738286668,
                                         0.109822074993031,
                                         0.169875088018046,
                                         0.234375,
                                         0.290405955823643,
                                         0.324101868055451,
                                         0.325519930698192,
                                         0.292766150053764,
                                         0.233351940368985,
                                         0.161856929754158,
                                         0.0946844967140477,
                                         0.044166847554624,
                                         0.0147006680888248,
                                         0.00267509759900154,
                                         0.000101530337878632,
                                         7.32231642597988e-13,
                                         2.88605681809872e-09,
                                         2.4519269250169e-07,
                                         5.00347042396582e-06,
                                         4.78450556664249e-05,
                                         0.000283993555319825,
                                         0.00120928275187258,
                                         0.00401784644725615,
                                         0.0109813921722315,
                                         0.0255713520773776,
                                         0.0519698871798051,
                                         0.09375,
                                         0.151880991038753,
                                         0.222708847314842,
                                         0.296914199156356,
                                         0.360393286592369,
                                         0.397422628054756,
                                         0.395505224053674,
                                         0.350353236507141,
                                         0.269020109893535,
                                         0.169574382320454,
                                         0.0768713553299151,
                                         0.0154878666079709,
                                         3.20008464088654e-16,
                                         6.69558782682509e-12,
                                         1.41707583458804e-09,
                                         5.47635678108041e-08,
                                         8.62022390353783e-07,
                                         7.74811930639149e-06,
                                         4.73364216804372e-05,
                                         0.000217593664039002,
                                         0.00080262514199532,
                                         0.00248088422078279,
                                         0.00662464501856691,
                                         0.015625,
                                         0.0330971108092022,
                                         0.0637649706786781,
                                         0.112842606226361,
                                         0.184850777371003,
                                         0.282021698355784,
                                         0.402681714264921,
                                         0.540159664424491,
                                         0.682750218151206,
                                         0.815027333157476,
                                         0.920403376507279,
                                         0.984410247379471,

                                         0.985648242491884,
                                         0.926540621665256,
                                         0.828536774256041,
                                         0.704115979622578,
                                         0.567872969985807,
                                         0.433811933190359,
                                         0.313120387329857,
                                         0.212863734445695,
                                         0.13574995151458,
                                         0.0808088850116454,
                                         0.0446221662446437,
                                         0.0226782282648721,
                                         0.0105030813043862,
                                         0.00437651336117894,
                                         0.00161361935564144,
                                         0.000514836222655621,
                                         0.000137871194237895,
                                         2.96744469098549e-05,
                                         4.81299206954868e-06,
                                         5.31405820038482e-07,
                                         3.35531910817617e-08,
                                         8.64182624687966e-10,
                                         4.07005592943438e-12,
                                         1.94176239711507e-16,
                                         0.014265452697046,
                                         0.0711440143707394,
                                         0.158311329590502,
                                         0.25437654004114,
                                         0.336975478942183,
                                         0.388721313293548,
                                         0.401150936586012,
                                         0.375677256944185,
                                         0.321646750992481,
                                         0.252542239984665,
                                         0.181815749981213,
                                         0.11968644666824,
                                         0.0716445859134123,
                                         0.0386678395276867,
                                         0.0185878295644263,
                                         0.00782225753476684,
                                         0.00281230753919116,
                                         0.000833851412673816,
                                         0.000193366300271747,
                                         3.22390106266807e-05,
                                         3.34351653713972e-06,
                                         1.62820027466446e-07,
                                         1.90822233801827e-09,
                                         4.82986488050167e-13,
                                         8.60276228537196e-05,
                                         0.00227615077995136,
                                         0.0126037842935098,
                                         0.0382911729009392,
                                         0.0833171027456719,
                                         0.145132264167834,
                                         0.214137650718655,
                                         0.276259288899507,
                                         0.317546560390812,
                                         0.328849889502049,
                                         0.308674754829191,
                                         0.263190120585428,
                                         0.203628604388724,
                                         0.14235093196579,
                                         0.0892164682220487,
                                         0.0495203703038796,
                                         0.0239023632973457,
                                         0.00976300390324498,
                                         0.00323694401359236,
                                         0.000814940427035567,
                                         0.000138823145103221,
                                         1.27820018336915e-05,
                                         3.72774616455327e-07,
                                         5.00567482719176e-10,
                                         2.7668716587898e-07,
                                         3.88384971443051e-05,
                                         0.000535166173903805,
                                         0.00307410722051123,
                                         0.0109867374828562,
                                         0.0288993660423255,
                                         0.0609644956026475,
                                         0.108347177679839,
                                         0.167199273040824,
                                         0.228381068359525,
                                         0.279492044090297,
                                         0.308668932874937,
                                         0.308668932874937,
                                         0.279492044090297,
                                         0.228381068359525,
                                         0.167199273040824,
                                         0.108347177679839,
                                         0.0609644956026475,
                                         0.0288993660423254,
                                         0.0109867374828562,
                                         0.00307410722051123,
                                         0.000535166173903802,
                                         3.88384971443051e-05,
                                         2.76687165878961e-07,
                                         5.00567482719222e-10,
                                         3.72774616455327e-07,
                                         1.27820018336916e-05,
                                         0.000138823145103221,
                                         0.000814940427035567,
                                         0.00323694401359237,
                                         0.00976300390324498,
                                         0.0239023632973458,
                                         0.0495203703038796,
                                         0.0892164682220487,
                                         0.14235093196579,
                                         0.203628604388724,
                                         0.263190120585428,
                                         0.308674754829191,
                                         0.328849889502049,
                                         0.317546560390812,
                                         0.276259288899507,
                                         0.214137650718655,
                                         0.145132264167834,
                                         0.0833171027456719,
                                         0.0382911729009392,
                                         0.0126037842935098,
                                         0.00227615077995136,
                                         8.60276228537156e-05,
                                         4.82986488050222e-13,
                                         1.90822233801827e-09,
                                         1.62820027466448e-07,
                                         3.34351653713972e-06,
                                         3.22390106266807e-05,
                                         0.000193366300271748,
                                         0.000833851412673816,
                                         0.00281230753919116,
                                         0.00782225753476684,
                                         0.0185878295644263,
                                         0.0386678395276867,
                                         0.0716445859134123,
                                         0.11968644666824,
                                         0.181815749981213,
                                         0.252542239984665,
                                         0.321646750992481,
                                         0.375677256944185,
                                         0.401150936586012,
                                         0.388721313293548,
                                         0.336975478942183,
                                         0.25437654004114,
                                         0.158311329590502,
                                         0.0711440143707394,
                                         0.0142654526970456,
                                         1.94176239711533e-16,
                                         4.07005592943438e-12,
                                         8.64182624687975e-10,
                                         3.35531910817617e-08,
                                         5.31405820038482e-07,
                                         4.81299206954869e-06,
                                         2.96744469098549e-05,
                                         0.000137871194237896,
                                         0.000514836222655621,
                                         0.00161361935564144,
                                         0.00437651336117894,
                                         0.0105030813043862,
                                         0.0226782282648721,
                                         0.0446221662446437,
                                         0.0808088850116454,
                                         0.13574995151458,
                                         0.212863734445695,
                                         0.313120387329857,
                                         0.433811933190359,
                                         0.567872969985807,
                                         0.704115979622578,
                                         0.828536774256041,
                                         0.926540621665256,
                                         0.985648242491884,

                                         0.0078125,
                                         0.0546875,
                                         0.1640625,
                                         0.2734375,
                                         0.2734375,
                                         0.1640625,
                                         0.0546875,
                                         0.0078125,

                                         0.189795993315888,
                                         1.8821498927247e-05,
                                         0.355989781949584,
                                         0.000491699531897622,
                                         0.286161523761367,
                                         0.00550514290529928,
                                         0.12779458199453,
                                         0.0342424550425071,
                                         0.0342424550425071,
                                         0.12779458199453,
                                         0.00550514290529928,
                                         0.286161523761367,
                                         0.000491699531897622,
                                         0.355989781949584,
                                         1.8821498927247e-05,
                                         0.189795993315888,

                                         0.432999769052994,
                                         0.0078125,
                                         2.30947005396121e-07,
                                         0.384987272306509,
                                         0.0546875,
                                         1.27276934913809e-05,
                                         0.146699385243321,
                                         0.1640625,
                                         0.000300614756679824,
                                         0.0310554417117261,
                                         0.2734375,
                                         0.00394455828827407,
                                         0.00394455828827407,
                                         0.2734375,
                                         0.0310554417117261,
                                         0.000300614756679824,
                                         0.1640625,
                                         0.146699385243321,
                                         1.27276934913809e-05,
                                         0.0546875,
                                         0.384987272306509,
                                         2.30947005396121e-07,
                                         0.0078125,
                                         0.432999769052994,

                                         0.604278729922773,
                                         0.0606011146246741,
                                         0.000426270122763878,
                                         7.77876955884479e-09,
                                         0.315606443857553,
                                         0.208947128293814,
                                         0.00605794295571169,
                                         7.2979088026355e-07,
                                         0.0706443820461607,
                                         0.308756007159942,
                                         0.0368967980913548,
                                         2.93433147870465e-05,
                                         0.00878490145596999,
                                         0.253467289218044,
                                         0.124847449533696,
                                         0.000655461833106641,
                                         0.000655461833106639,
                                         0.124847449533696,
                                         0.253467289218044,
                                         0.00878490145597001,
                                         2.93433147870464e-05,
                                         0.0368967980913548,
                                         0.308756007159942,
                                         0.0706443820461608,
                                         7.29790880263547e-07,
                                         0.00605794295571169,
                                         0.208947128293814,
                                         0.315606443857553,
                                         7.77876955884475e-09,
                                         0.000426270122763878,
                                         0.0606011146246741,
                                         0.604278729922773,

                                         0.714392924232422,
                                         0.159371951613892,
                                         0.0078125,
                                         3.48493053863121e-05,
                                         4.99876848003043e-10,
                                         0.246131644127034,
                                         0.334673772213092,
                                         0.0546875,
                                         0.000813168259283756,
                                         7.10933197513471e-08,
                                         0.0363430168783423,
                                         0.301199801577644,
                                         0.1640625,
                                         0.0081318605994322,
                                         4.33329026092715e-06,
                                         0.00298127469437943,
                                         0.150596604153917,
                                         0.2734375,
                                         0.0451779922773527,
                                         0.000146735184364572,
                                         0.000146735184364572,
                                         0.0451779922773527,
                                         0.2734375,
                                         0.150596604153917,
                                         0.00298127469437944,
                                         4.33329026092713e-06,
                                         0.0081318605994322,
                                         0.1640625,
                                         0.301199801577644,
                                         0.0363430168783424,
                                         7.10933197513466e-08,
                                         0.000813168259283756,
                                         0.0546875,
                                         0.334673772213092,
                                         0.246131644127034,
                                         4.99876848003039e-10,
                                         3.48493053863121e-05,
                                         0.0078125,
                                         0.159371951613892,
                                         0.714392924232422,

                                         0.786282464402557,
                                         0.272747428675303,
                                         0.0349425544563488,
                                         0.00115878678013018,
                                         4.00229978709488e-06,
                                         5.00367562627932e-11,
                                         0.192337449481471,
                                         0.389372878683597,
                                         0.150354633577943,
                                         0.0131958523060965,
                                         0.000137372772317918,
                                         1.00230515815988e-08,
                                         0.0201637976251847,
                                         0.238228625850516,
                                         0.277269562532147,
                                         0.0644012912236479,
                                         0.00202076087444521,
                                         8.60466560791248e-07,
                                         0.00117437906879438,
                                         0.0809747634973347,
                                         0.284063263843902,
                                         0.174614055279785,
                                         0.016514167346699,
                                         4.10388823432674e-05,
                                         4.10388823432674e-05,
                                         0.016514167346699,
                                         0.174614055279785,
                                         0.284063263843902,
                                         0.0809747634973346,
                                         0.00117437906879438,
                                         8.60466560791248e-07,
                                         0.00202076087444521,
                                         0.0644012912236479,
                                         0.277269562532147,
                                         0.238228625850516,
                                         0.0201637976251847,
                                         1.00230515815988e-08,
                                         0.000137372772317919,
                                         0.0131958523060965,
                                         0.150354633577943,
                                         0.389372878683597,
                                         0.192337449481471,
                                         5.00367562627932e-11,
                                         4.00229978709489e-06,
                                         0.00115878678013018,
                                         0.0349425544563488,
                                         0.272747428675303,
                                         0.786282464402557,

                                         0.834912997163999,
                                         0.379584806517345,
                                         0.0847915241920486,
                                         0.0078125,
                                         0.000204214957908504,
                                         6.02070135413138e-07,
                                         6.90783405608846e-12,
                                         0.152599687365376,
                                         0.394351734849226,
                                         0.250849153763575,
                                         0.0546875,
                                         0.00338238804843511,
                                         2.83967233501691e-05,
                                         1.85193420922078e-09,
                                         0.0119533402178823,
                                         0.175582773721426,
                                         0.318050478394745,
                                         0.1640625,
                                         0.0240094674763298,
                                         0.000574000922291251,
                                         2.12780794804954e-07,
                                         0.000520178517150538,
                                         0.0434317721173692,
                                         0.224030403419406,
                                         0.2734375,
                                         0.0946823697475519,
                                         0.00644591307885771,
                                         1.35820959550772e-05,
                                         1.35820959550771e-05,
                                         0.00644591307885771,
                                         0.0946823697475519,
                                         0.2734375,
                                         0.224030403419406,
                                         0.0434317721173692,
                                         0.000520178517150542,
                                         2.12780794804952e-07,
                                         0.000574000922291251,
                                         0.0240094674763298,
                                         0.1640625,
                                         0.318050478394745,
                                         0.175582773721426,
                                         0.0119533402178824,
                                         1.85193420922076e-09,
                                         2.83967233501691e-05,
                                         0.00338238804843511,
                                         0.0546875,
                                         0.250849153763575,
                                         0.394351734849226,
                                         0.152599687365376,
                                         6.90783405608835e-12,
                                         6.02070135413138e-07,
                                         0.000204214957908504,
                                         0.0078125,
                                         0.0847915241920486,
                                         0.379584806517345,
                                         0.834912997163999,

                                         0.869024617148354,
                                         0.472130746008638,
                                         0.150224269540811,
                                         0.0253980268206605,
                                         0.00189115309870527,
                                         4.22898256217175e-05,
                                         1.12267207929378e-07,
                                         1.2164667026349e-12,
                                         0.123228534279389,
                                         0.374026043452567,
                                         0.327056853557849,
                                         0.122671786630209,
                                         0.0191857179525427,
                                         0.000951806839092721,
                                         6.94399568274334e-06,
                                         4.20355856043675e-10,
                                         0.00748882533409007,
                                         0.126988565619147,
                                         0.305161416668529,
                                         0.253929184701289,
                                         0.0834166293886726,
                                         0.00918089377249079,
                                         0.000184072612874598,
                                         6.22525567557284e-08,
                                         0.000252838738529668,
                                         0.0239527227230124,
                                         0.158184339460846,
                                         0.292016936748802,
                                         0.201490564659119,
                                         0.0491981303347605,
                                         0.00271079332087049,
                                         5.1218255079553e-06,
                                         5.1218255079553e-06,
                                         0.00271079332087049,
                                         0.0491981303347605,
                                         0.201490564659119,
                                         0.292016936748802,
                                         0.158184339460846,
                                         0.0239527227230124,
                                         0.000252838738529668,
                                         6.22525567557284e-08,
                                         0.000184072612874598,
                                         0.00918089377249079,
                                         0.0834166293886727,
                                         0.253929184701289,
                                         0.305161416668529,
                                         0.126988565619147,
                                         0.00748882533409007,
                                         4.20355856043675e-10,
                                         6.94399568274333e-06,
                                         0.000951806839092721,
                                         0.0191857179525427,
                                         0.122671786630209,
                                         0.327056853557849,
                                         0.374026043452567,
                                         0.123228534279389,
                                         1.2164667026349e-12,
                                         1.12267207929378e-07,
                                         4.22898256217175e-05,
                                         0.00189115309870527,
                                         0.0253980268206605,
                                         0.150224269540811,
                                         0.472130746008638,
                                         0.869024617148354,

                                         0.893744142257077,
                                         0.549477958085253,
                                         0.222295448173005,
                                         0.0557940337309718,
                                         0.0078125,
                                         0.000502663105141885,
                                         1.0088988133954e-05,
                                         2.48954669634121e-08,
                                         2.59166328142104e-13,
                                         0.101209338557884,
                                         0.343502375222785,
                                         0.372896397144658,
                                         0.199295985461008,
                                         0.0546875,
                                         0.00689544501740769,
                                         0.000294704029443844,
                                         1.95135770704456e-06,
                                         1.12141736685944e-10,
                                         0.00491191880811613,
                                         0.0920306369612021,
                                         0.268082842283547,
                                         0.305093218996892,
                                         0.1640625,
                                         0.0405387921438994,
                                         0.00368933210647813,
                                         6.55506064578806e-05,
                                         2.07959705652438e-08,
                                         0.000132436979527922,
                                         0.0136981717197272,
                                         0.107072352888744,
                                         0.259474124106776,
                                         0.2734375,
                                         0.132405737437903,
                                         0.02565883438599,
                                         0.00122333115140062,
                                         2.14248902292501e-06,
                                         2.14248902292498e-06,
                                         0.00122333115140062,
                                         0.02565883438599,
                                         0.132405737437903,
                                         0.2734375,
                                         0.259474124106776,
                                         0.107072352888744,
                                         0.0136981717197272,
                                         0.000132436979527923,
                                         2.07959705652434e-08,
                                         6.55506064578806e-05,
                                         0.00368933210647812,
                                         0.0405387921438994,
                                         0.1640625,
                                         0.305093218996892,
                                         0.268082842283547,
                                         0.0920306369612021,
                                         0.00491191880811617,
                                         1.12141736685941e-10,
                                         1.95135770704456e-06,
                                         0.000294704029443843,
                                         0.00689544501740769,
                                         0.0546875,
                                         0.199295985461008,
                                         0.372896397144658,
                                         0.343502375222785,
                                         0.101209338557884,
                                         2.59166328142098e-13,
                                         2.48954669634121e-08,
                                         1.0088988133954e-05,
                                         0.000502663105141885,
                                         0.0078125,
                                         0.0557940337309718,
                                         0.222295448173005,
                                         0.549477958085253,
                                         0.893744142257077,

                                         0.912170692266778,
                                         0.613260754654977,
                                         0.294365150290663,
                                         0.0971297034542767,
                                         0.0206394317017482,
                                         0.00252781811443596,
                                         0.000146470692237609,
                                         2.71921736229605e-06,
                                         6.36354503616898e-09,
                                         6.43447416647235e-14,
                                         0.0844071880891463,
                                         0.310584295080952,
                                         0.393349285805798,
                                         0.268759731603853,
                                         0.107032810707876,
                                         0.0238848603538697,
                                         0.00259378920356574,
                                         9.97121387125178e-05,
                                         6.15688275793512e-07,
                                         3.40726430409742e-11,
                                         0.00334738731069152,
                                         0.0674119709056842,
                                         0.225264914037869,
                                         0.318712831971607,
                                         0.237880772526816,
                                         0.0967214516287335,
                                         0.0196852909023901,
                                         0.00156702293584564,
                                         2.5529708730075e-05,
                                         7.73253331815299e-09,
                                         7.37496550382288e-05,
                                         0.00812871791323433,
                                         0.0716698072561585,
                                         0.20997248826672,
                                         0.293717109131236,
                                         0.217595745835285,
                                         0.0829996939053501,
                                         0.0136813883175915,
                                         0.000588109684602307,
                                         9.74911675303113e-07,
                                         9.74911675303113e-07,
                                         0.000588109684602309,
                                         0.0136813883175915,
                                         0.0829996939053501,
                                         0.217595745835285,
                                         0.293717109131236,
                                         0.20997248826672,
                                         0.0716698072561585,
                                         0.00812871791323431,
                                         7.37496550382288e-05,
                                         7.73253331815299e-09,
                                         2.55297087300751e-05,
                                         0.00156702293584564,
                                         0.0196852909023901,
                                         0.0967214516287335,
                                         0.237880772526816,
                                         0.318712831971607,
                                         0.225264914037869,
                                         0.0674119709056841,
                                         0.00334738731069152,
                                         3.40726430409742e-11,
                                         6.15688275793515e-07,
                                         9.97121387125178e-05,
                                         0.00259378920356574,
                                         0.0238848603538697,
                                         0.107032810707876,
                                         0.268759731603853,
                                         0.393349285805798,
                                         0.310584295080951,
                                         0.0844071880891463,
                                         6.43447416647235e-14,
                                         6.36354503616902e-09,
                                         2.71921736229605e-06,
                                         0.000146470692237609,
                                         0.00252781811443596,
                                         0.0206394317017482,
                                         0.0971297034542767,
                                         0.294365150290663,
                                         0.613260754654977,
                                         0.912170692266778,

                                         0.926244098741814,
                                         0.665723950256127,
                                         0.362559971440197,
                                         0.145843416736863,
                                         0.0415262113765022,
                                         0.0078125,
                                         0.000866865997780997,
                                         4.64727036686175e-05,
                                         8.13999977323069e-07,
                                         1.83085746069555e-09,
                                         1.8112834145891e-14,
                                         0.0713562801833172,
                                         0.278896908646785,
                                         0.395833756721649,
                                         0.323189978586531,
                                         0.167250507519824,
                                         0.0546875,
                                         0.0105463678313788,
                                         0.00102759732206423,
                                         3.65331818533283e-05,
                                         2.14141625603513e-07,
                                         1.15206170936179e-11,
                                         0.00235592881980487,
                                         0.0500744783969867,
                                         0.185211955423512,
                                         0.306939058132516,
                                         0.288692693861173,
                                         0.1640625,
                                         0.0549891586734564,
                                         0.00973803212535825,
                                         0.000702706254919552,
                                         1.07342293668924e-05,
                                         3.14042400973937e-09,
                                         4.32135179017489e-05,
                                         0.00499478343468782,
                                         0.0481451673549734,
                                         0.161947378132257,
                                         0.276841782374935,
                                         0.2734375,
                                         0.159286412364948,
                                         0.0512680662607414,
                                         0.0075090956229194,
                                         0.000298929063563843,
                                         4.75585199454238e-07,
                                         4.75585199454228e-07,
                                         0.000298929063563843,
                                         0.00750909562291941,
                                         0.0512680662607414,
                                         0.159286412364948,
                                         0.2734375,
                                         0.276841782374935,
                                         0.161947378132257,
                                         0.0481451673549733,
                                         0.00499478343468782,
                                         4.32135179017496e-05,
                                         3.14042400973929e-09,
                                         1.07342293668924e-05,
                                         0.000702706254919553,
                                         0.00973803212535826,
                                         0.0549891586734564,
                                         0.1640625,
                                         0.288692693861174,
                                         0.306939058132516,
                                         0.185211955423511,
                                         0.0500744783969867,
                                         0.0023559288198049,
                                         1.15206170936175e-11,
                                         2.14141625603513e-07,
                                         3.65331818533283e-05,
                                         0.00102759732206424,
                                         0.0105463678313788,
                                         0.0546875,
                                         0.167250507519824,
                                         0.323189978586531,
                                         0.395833756721648,
                                         0.278896908646785,
                                         0.0713562801833176,
                                         1.81128341458903e-14,
                                         1.83085746069555e-09,
                                         8.13999977323072e-07,
                                         4.64727036686176e-05,
                                         0.000866865997780996,
                                         0.0078125,
                                         0.0415262113765022,
                                         0.145843416736863,
                                         0.362559971440197,
                                         0.665723950256127,
                                         0.926244098741814,

                                         0.937220095578147,
                                         0.708999375682048,
                                         0.42504579933678,
                                         0.198352432442444,
                                         0.0699871380925131,
                                         0.0178438414813797,
                                         0.00306221267023594,
                                         0.000315224524153621,
                                         1.59257191772164e-05,
                                         2.66790910752525e-07,
                                         5.82067296012457e-10,
                                         5.66254781410992e-15,
                                         0.0610489564866734,
                                         0.249914040751448,
                                         0.386808451327986,
                                         0.360983388111131,
                                         0.226420907449706,
                                         0.0971037421352778,
                                         0.0275729871697237,
                                         0.00477438883616499,
                                         0.000428790788774355,
                                         1.43650414587286e-05,
                                         8.09141497756906e-08,
                                         4.25962443159873e-12,
                                         0.00170426887483335,
                                         0.0377536264932533,
                                         0.150861838134904,
                                         0.281552973061156,
                                         0.313933768368664,
                                         0.226467972473799,
                                         0.106403399390355,
                                         0.0309912282637739,
                                         0.0049478332633373,
                                         0.000331486581215451,
                                         4.82057635316753e-06,
                                         1.37326373421978e-09,
                                         2.64317230418161e-05,
                                         0.00316850347847235,
                                         0.0326881491160478,
                                         0.122000191776094,
                                         0.241817114545694,
                                         0.293430404251701,
                                         0.228115440427528,
                                         0.111760229919329,
                                         0.031718464837885,
                                         0.00424964367069749,
                                         0.000159551522207577,
                                         2.4595977545887e-07,
                                         2.45959775458876e-07,
                                         0.000159551522207576,
                                         0.00424964367069749,
                                         0.031718464837885,
                                         0.111760229919329,
                                         0.228115440427528,
                                         0.293430404251701,
                                         0.241817114545695,
                                         0.122000191776094,
                                         0.0326881491160478,
                                         0.00316850347847236,
                                         2.64317230418156e-05,
                                         1.37326373421982e-09,
                                         4.8205763531675e-06,
                                         0.00033148658121545,
                                         0.0049478332633373,
                                         0.0309912282637739,
                                         0.106403399390355,
                                         0.226467972473799,
                                         0.313933768368664,
                                         0.281552973061156,
                                         0.150861838134904,
                                         0.0377536264932534,
                                         0.00170426887483333,
                                         4.25962443159889e-12,
                                         8.09141497756901e-08,
                                         1.43650414587286e-05,
                                         0.000428790788774354,
                                         0.00477438883616498,
                                         0.0275729871697237,
                                         0.0971037421352778,
                                         0.226420907449706,
                                         0.360983388111132,
                                         0.386808451327986,
                                         0.249914040751448,
                                         0.061048956486673,
                                         5.66254781411016e-15,
                                         5.82067296012452e-10,
                                         2.66790910752525e-07,
                                         1.59257191772164e-05,
                                         0.00031522452415362,
                                         0.00306221267023594,
                                         0.0178438414813797,
                                         0.0699871380925131,
                                         0.198352432442444,
                                         0.42504579933678,
                                         0.708999375682048,
                                         0.937220095578147,

                                         0.945936938561647,
                                         0.744892565468458,
                                         0.481239949030381,
                                         0.251795991139697,
                                         0.104521182198714,
                                         0.0333624833981836,
                                         0.0078125,
                                         0.0012485762684793,
                                         0.000121240747790487,
                                         5.84794461503403e-06,
                                         9.46057209918471e-08,
                                         2.01529937703236e-10,
                                         1.93485451689262e-15,
                                         0.0527838544579877,
                                         0.224062754111928,
                                         0.371019080411895,
                                         0.383831260964566,
                                         0.278571865327661,
                                         0.146056757288519,
                                         0.0546875,
                                         0.0139748730855691,
                                         0.00222900861673815,
                                         0.00018797833540665,
                                         6.01282975145158e-06,
                                         3.28291486577646e-08,
                                         1.69904734082443e-12,
                                         0.00126230188648962,
                                         0.0288847700657434,
                                         0.12258971402078,
                                         0.250758168309437,
                                         0.318195040276673,
                                         0.274036336328936,
                                         0.1640625,
                                         0.0670353503614245,
                                         0.0175629758053626,
                                         0.00258961715184117,
                                         0.000163781041875245,
                                         2.29193810509822e-06,
                                         6.39419473769765e-10,
                                         1.67707643537466e-05,
                                         0.00206869114031551,
                                         0.0225029426863568,
                                         0.0910117216263317,
                                         0.201918938578588,
                                         0.285642053064808,
                                         0.2734375,
                                         0.17864357020408,
                                         0.0768797484484723,
                                         0.0198194145281061,
                                         0.00247842537323951,
                                         8.88942447721653e-05,
                                         1.33688402125921e-07,
                                         1.33688402125925e-07,
                                         8.88942447721658e-05,
                                         0.0024784253732395,
                                         0.0198194145281062,
                                         0.0768797484484723,
                                         0.17864357020408,
                                         0.2734375,
                                         0.285642053064808,
                                         0.201918938578588,
                                         0.0910117216263317,
                                         0.0225029426863568,
                                         0.0020686911403155,
                                         1.67707643537462e-05,
                                         6.39419473769788e-10,
                                         2.29193810509824e-06,
                                         0.000163781041875244,
                                         0.00258961715184118,
                                         0.0175629758053626,
                                         0.0670353503614245,
                                         0.1640625,
                                         0.274036336328936,
                                         0.318195040276673,
                                         0.250758168309436,
                                         0.12258971402078,
                                         0.0288847700657433,
                                         0.00126230188648961,
                                         1.6990473408245e-12,
                                         3.28291486577648e-08,
                                         6.01282975145156e-06,
                                         0.000187978335406651,
                                         0.00222900861673815,
                                         0.0139748730855691,
                                         0.0546875,
                                         0.146056757288519,
                                         0.278571865327661,
                                         0.383831260964566,
                                         0.371019080411895,
                                         0.224062754111928,
                                         0.0527838544579874,
                                         1.93485451689271e-15,
                                         2.01529937703238e-10,
                                         9.46057209918468e-08,
                                         5.84794461503404e-06,
                                         0.000121240747790487,
                                         0.0012485762684793,
                                         0.0078125,
                                         0.0333624833981836,
                                         0.104521182198714,
                                         0.251795991139697,
                                         0.481239949030381,
                                         0.744892565468458,
                                         0.945936938561647,

                                         0.952969820938751,
                                         0.774862926592346,
                                         0.531244961759882,
                                         0.304174912854268,
                                         0.143277284960876,
                                         0.0542953426974053,
                                         0.0160220880541196,
                                         0.00350878479449016,
                                         0.000530051593766141,
                                         4.91386596995848e-05,
                                         2.28430545834448e-06,
                                         3.59395852792802e-08,
                                         7.51103354506816e-11,
                                         7.13550584545569e-16,
                                         0.0460648242665565,
                                         0.201288667661476,
                                         0.351679374393899,
                                         0.394610545742223,
                                         0.320856262912937,
                                         0.196179792539372,
                                         0.0902805002364112,
                                         0.0305125124568437,
                                         0.00718823939691991,
                                         0.00107519245701581,
                                         8.62789721153054e-05,
                                         2.66021701833628e-06,
                                         1.41677648126912e-08,
                                         7.23320168848703e-13,
                                         0.000954295563234292,
                                         0.0224097485070864,
                                         0.0997751034510938,
                                         0.219400289904316,
                                         0.307940572750945,
                                         0.303786994996299,
                                         0.218017830725903,
                                         0.113716221788444,
                                         0.0417782394623391,
                                         0.0100825950776664,
                                         0.00139662226784715,
                                         8.43886983787661e-05,
                                         1.1453182760116e-06,
                                         3.1423864864456e-10,
                                         1.09830733887696e-05,
                                         0.00138606033366371,
                                         0.0157262425719347,
                                         0.0677693315169741,
                                         0.164191549140133,
                                         0.261343425884759,
                                         0.292494407183787,
                                         0.235447654760001,
                                         0.13489791342914,
                                         0.0525274040407279,
                                         0.0125597344367972,
                                         0.00148723296820769,
                                         5.14373442776061e-05,
                                         7.58431071318831e-08,
                                         7.58431071318831e-08,
                                         5.14373442776064e-05,
                                         0.00148723296820769,
                                         0.0125597344367972,
                                         0.0525274040407278,
                                         0.13489791342914,
                                         0.235447654760001,
                                         0.292494407183787,
                                         0.261343425884759,
                                         0.164191549140133,
                                         0.0677693315169742,
                                         0.0157262425719347,
                                         0.00138606033366371,
                                         1.09830733887696e-05,
                                         3.1423864864456e-10,
                                         1.14531827601161e-06,
                                         8.43886983787661e-05,
                                         0.00139662226784715,
                                         0.0100825950776664,
                                         0.0417782394623391,
                                         0.113716221788444,
                                         0.218017830725903,
                                         0.303786994996299,
                                         0.307940572750945,
                                         0.219400289904316,
                                         0.0997751034510938,
                                         0.0224097485070863,
                                         0.000954295563234292,
                                         7.23320168848703e-13,
                                         1.41677648126914e-08,
                                         2.66021701833628e-06,
                                         8.62789721153052e-05,
                                         0.00107519245701581,
                                         0.00718823939691991,
                                         0.0305125124568437,
                                         0.0902805002364112,
                                         0.196179792539372,
                                         0.320856262912937,
                                         0.394610545742223,
                                         0.351679374393899,
                                         0.201288667661475,
                                         0.0460648242665565,
                                         7.13550584545569e-16,
                                         7.51103354506824e-11,
                                         3.59395852792802e-08,
                                         2.28430545834448e-06,
                                         4.91386596995847e-05,
                                         0.000530051593766141,
                                         0.00350878479449015,
                                         0.0160220880541196,
                                         0.0542953426974053,
                                         0.143277284960876,
                                         0.304174912854268,
                                         0.531244961759882,
                                         0.774862926592346,
                                         0.952969820938751,

                                         0.958723227255006,
                                         0.800067125939386,
                                         0.575497737495814,
                                         0.35423167219522,
                                         0.184488574859528,
                                         0.079975965051387,
                                         0.0281891738330272,
                                         0.0078125,
                                         0.00162135793230379,
                                         0.000234066389689774,
                                         2.09020202910069e-05,
                                         9.43080852419066e-07,
                                         1.45064312484966e-08,
                                         2.98505703800188e-11,
                                         2.81160065031437e-16,
                                         0.0405348422550844,
                                         0.181336551676853,
                                         0.330859861763959,
                                         0.39627276065787,
                                         0.352682967155124,
                                         0.243282994747208,
                                         0.131222548120872,
                                         0.0546875,
                                         0.0170666727741127,
                                         0.00377036046283577,
                                         0.000535758827106493,
                                         4.13083812115719e-05,
                                         1.23639204099532e-06,
                                         6.45341786698059e-09,
                                         3.25847562895827e-13,
                                         0.00073449178028857,
                                         0.0176143854012481,
                                         0.0815206740636618,
                                         0.189987166791876,
                                         0.28895000776231,
                                         0.317166443131511,
                                         0.261792861695211,
                                         0.1640625,
                                         0.0769913681899537,
                                         0.0260285457079497,
                                         0.00588536795198876,
                                         0.000775444356778815,
                                         4.51621940104766e-05,
                                         5.97929739973003e-07,
                                         1.61844719356791e-10,
                                         7.3938883370804e-06,
                                         0.000950554737998388,
                                         0.0111588370314232,
                                         0.0506036457841155,
                                         0.131518979368856,
                                         0.229715449863967,
                                         0.290158053440965,
                                         0.2734375,
                                         0.192957964013554,
                                         0.0998261746454514,
                                         0.0359174420547951,
                                         0.00808705875207693,
                                         0.000916476552659172,
                                         3.07778315058496e-05,
                                         4.46591122256563e-08,
                                         4.46591122256547e-08,
                                         3.07778315058496e-05,
                                         0.00091647655265917,
                                         0.00808705875207694,
                                         0.0359174420547951,
                                         0.0998261746454514,
                                         0.192957964013554,
                                         0.2734375,
                                         0.290158053440965,
                                         0.229715449863967,
                                         0.131518979368856,
                                         0.0506036457841154,
                                         0.0111588370314232,
                                         0.000950554737998388,
                                         7.3938883370806e-06,
                                         1.61844719356784e-10,
                                         5.97929739973003e-07,
                                         4.51621940104764e-05,
                                         0.000775444356778817,
                                         0.00588536795198876,
                                         0.0260285457079497,
                                         0.0769913681899537,
                                         0.1640625,
                                         0.261792861695211,
                                         0.317166443131511,
                                         0.28895000776231,
                                         0.189987166791875,
                                         0.0815206740636619,
                                         0.017614385401248,
                                         0.000734491780288583,
                                         3.25847562895809e-13,
                                         6.45341786698059e-09,
                                         1.23639204099531e-06,
                                         4.1308381211572e-05,
                                         0.000535758827106493,
                                         0.00377036046283577,
                                         0.0170666727741127,
                                         0.0546875,
                                         0.131222548120872,
                                         0.243282994747208,
                                         0.352682967155124,
                                         0.396272760657869,
                                         0.330859861763959,
                                         0.181336551676853,
                                         0.0405348422550848,
                                         2.81160065031419e-16,
                                         2.98505703800188e-11,
                                         1.45064312484965e-08,
                                         9.43080852419068e-07,
                                         2.09020202910069e-05,
                                         0.000234066389689774,
                                         0.00162135793230379,
                                         0.0078125,
                                         0.0281891738330272,
                                         0.079975965051387,
                                         0.184488574859528,
                                         0.35423167219522,
                                         0.575497737495814,
                                         0.800067125939386,
                                         0.958723227255006,

                                         0.963487876613842,
                                         0.82141561165458,
                                         0.614568939183748,
                                         0.401264115422011,
                                         0.226676823404277,
                                         0.109427803262988,
                                         0.044367600759339,
                                         0.0147476762654543,
                                         0.00388407906278665,
                                         0.000771505932739513,
                                         0.000107322536723417,
                                         9.29433436896899e-06,
                                         4.09195085019907e-07,
                                         6.17844303555952e-09,
                                         1.25524551204721e-11,
                                         1.17398600950322e-16,
                                         0.0359326736082037,
                                         0.163885983045812,
                                         0.309842709060337,
                                         0.391381058833441,
                                         0.374769204727842,
                                         0.284740368112662,
                                         0.174090254186645,
                                         0.0853188770585429,
                                         0.0328974781024956,
                                         0.00963444795339857,
                                         0.00202099690357497,
                                         0.000275459397485201,
                                         2.05568708701249e-05,
                                         6.00488165400556e-07,
                                         3.08280389840102e-09,
                                         1.54246643849532e-13,
                                         0.000574322757366307,
                                         0.0140134117549949,
                                         0.066947624287235,
                                         0.163603465711448,
                                         0.265548523544295,
                                         0.317536474076944,
                                         0.292756259240022,
                                         0.211538724130551,
                                         0.119415407801536,
                                         0.0515629331976816,
                                         0.0163103613075057,
                                         0.00349880798270781,
                                         0.00044259532444611,
                                         2.50122679806086e-05,
                                         3.24478806898789e-07,
                                         8.68543830758214e-11,
                                         5.09976322857701e-06,
                                         0.000665692172642758,
                                         0.00803630764657857,
                                         0.0379937962826882,
                                         0.104532510300816,
                                         0.196727778981297,
                                         0.273505090552769,
                                         0.291381601128071,
                                         0.240816156450563,
                                         0.153311908177405,
                                         0.0731288948183056,
                                         0.024689376308208,
                                         0.00529400236001014,
                                         0.000578800887512043,
                                         1.89737978073704e-05,
                                         2.71703511525217e-08,
                                         2.71703511525228e-08,
                                         1.89737978073704e-05,
                                         0.000578800887512041,
                                         0.00529400236001015,
                                         0.024689376308208,
                                         0.0731288948183056,
                                         0.153311908177405,
                                         0.240816156450563,
                                         0.291381601128071,
                                         0.273505090552769,
                                         0.196727778981297,
                                         0.104532510300816,
                                         0.0379937962826881,
                                         0.00803630764657859,
                                         0.000665692172642758,
                                         5.09976322857685e-06,
                                         8.6854383075826e-11,
                                         3.24478806898789e-07,
                                         2.50122679806085e-05,
                                         0.000442595324446111,
                                         0.0034988079827078,
                                         0.0163103613075057,
                                         0.0515629331976817,
                                         0.119415407801536,
                                         0.21153872413055,
                                         0.292756259240022,
                                         0.317536474076944,
                                         0.265548523544295,
                                         0.163603465711448,
                                         0.0669476242872351,
                                         0.0140134117549949,
                                         0.000574322757366295,
                                         1.54246643849542e-13,
                                         3.08280389840102e-09,
                                         6.00488165400553e-07,
                                         2.0556870870125e-05,
                                         0.000275459397485201,
                                         0.00202099690357497,
                                         0.00963444795339858,
                                         0.0328974781024956,
                                         0.0853188770585428,
                                         0.174090254186645,
                                         0.284740368112662,
                                         0.374769204727842,
                                         0.391381058833441,
                                         0.309842709060338,
                                         0.163885983045812,
                                         0.0359326736082033,
                                         1.17398600950331e-16,
                                         1.25524551204721e-11,
                                         6.17844303555948e-09,
                                         4.09195085019908e-07,
                                         9.29433436896899e-06,
                                         0.000107322536723417,
                                         0.000771505932739514,
                                         0.00388407906278664,
                                         0.0147476762654543,
                                         0.044367600759339,
                                         0.109427803262988,
                                         0.226676823404277,
                                         0.401264115422011,
                                         0.614568939183748,
                                         0.82141561165458,
                                         0.963487876613842,

                                         0.967476832232532,
                                         0.839624823270648,
                                         0.649055261862035,
                                         0.444953589036542,
                                         0.268706827222014,
                                         0.141592634054263,
                                         0.0642538196510229,
                                         0.0246635922971972,
                                         0.0078125,
                                         0.00197288396267145,
                                         0.000377934973009722,
                                         5.09828306588816e-05,
                                         4.30421995417006e-06,
                                         1.85675786621681e-07,
                                         2.76053986983122e-09,
                                         5.54916360631833e-12,
                                         5.1594834602283e-17,
                                         0.0320641270869944,
                                         0.148614359994553,
                                         0.289388640264885,
                                         0.381985712912654,
                                         0.388438124856186,
                                         0.319295983767263,
                                         0.215951747699119,
                                         0.120350124948616,
                                         0.0546875,
                                         0.0198110461502499,
                                         0.00551004808648888,
                                         0.00110781497082647,
                                         0.000145897344939989,
                                         1.05978787641102e-05,
                                         3.03382342154962e-07,
                                         1.53620121305142e-09,
                                         7.62822434944875e-14,
                                         0.000455429840768007,
                                         0.0112735188645107,
                                         0.0552973727696637,
                                         0.14054090307518,
                                         0.240651299732262,
                                         0.308581201267556,
                                         0.311055375524245,
                                         0.251686529925444,
                                         0.1640625,
                                         0.0852582652010214,
                                         0.0344284055840612,
                                         0.0103165320032539,
                                         0.00211945175603592,
                                         0.000259242282342137,
                                         1.42892617083408e-05,
                                         1.82260221107425e-07,
                                         4.83352428375634e-11,
                                         3.59377629743918e-06,
                                         0.000475100744687779,
                                         0.00587022857034107,
                                         0.0287267076715445,
                                         0.0828289334430581,
                                         0.165681100185602,
                                         0.248912206014693,
                                         0.292415840739479,
                                         0.2734375,
                                         0.203841716775321,
                                         0.119510462467359,
                                         0.053373750920577,
                                         0.0171051614255499,
                                         0.00352306146718697,
                                         0.000373901128484931,
                                         1.20133236294192e-05,
                                         1.70149958389949e-08,
                                         1.70149958389941e-08,
                                         1.20133236294193e-05,
                                         0.000373901128484931,
                                         0.00352306146718698,
                                         0.0171051614255499,
                                         0.053373750920577,
                                         0.119510462467359,
                                         0.203841716775321,
                                         0.2734375,
                                         0.292415840739479,
                                         0.248912206014693,
                                         0.165681100185602,
                                         0.082828933443058,
                                         0.0287267076715445,
                                         0.00587022857034108,
                                         0.000475100744687776,
                                         3.5937762974393e-06,
                                         4.83352428375605e-11,
                                         1.82260221107427e-07,
                                         1.42892617083408e-05,
                                         0.000259242282342138,
                                         0.00211945175603593,
                                         0.0103165320032539,
                                         0.0344284055840612,
                                         0.0852582652010214,
                                         0.1640625,
                                         0.251686529925444,
                                         0.311055375524245,
                                         0.308581201267556,
                                         0.240651299732261,
                                         0.140540903075179,
                                         0.0552973727696637,
                                         0.0112735188645106,
                                         0.000455429840768018,
                                         7.62822434944821e-14,
                                         1.53620121305144e-09,
                                         3.03382342154962e-07,
                                         1.05978787641102e-05,
                                         0.000145897344939989,
                                         0.00110781497082647,
                                         0.00551004808648888,
                                         0.0198110461502499,
                                         0.0546875,
                                         0.120350124948616,
                                         0.215951747699119,
                                         0.319295983767263,
                                         0.388438124856186,
                                         0.381985712912654,
                                         0.289388640264885,
                                         0.148614359994552,
                                         0.0320641270869948,
                                         5.15948346022787e-17,
                                         5.54916360631841e-12,
                                         2.76053986983122e-09,
                                         1.85675786621682e-07,
                                         4.30421995417007e-06,
                                         5.09828306588817e-05,
                                         0.000377934973009722,
                                         0.00197288396267145,
                                         0.0078125,
                                         0.0246635922971972,
                                         0.0642538196510229,
                                         0.141592634054263,
                                         0.268706827222014,
                                         0.444953589036542,
                                         0.649055261862035,
                                         0.839624823270648,
                                         0.967476832232532,

                                         0.970848993561292,
                                         0.855260371523085,
                                         0.679525631043893,
                                         0.485230984967367,
                                         0.309763463258982,
                                         0.175475137508019,
                                         0.0873166582711359,
                                         0.0376481861539915,
                                         0.0138091752542664,
                                         0.00420228119387966,
                                         0.0010246217907608,
                                         0.000190422200218412,
                                         2.50346621280949e-05,
                                         2.06886649136993e-06,
                                         8.7731454159596e-08,
                                         1.28750873003575e-09,
                                         2.56506411962059e-12,
                                         2.37314862287524e-17,
                                         0.0287827057606339,
                                         0.135223917864324,
                                         0.26992231391626,
                                         0.369649730903669,
                                         0.395184575011754,
                                         0.346682286201018,
                                         0.254681894023045,
                                         0.15748706426402,
                                         0.0815556356205716,
                                         0.0348654242541196,
                                         0.0120021441298514,
                                         0.00319898861333274,
                                         0.00062090013467116,
                                         7.9461864671816e-05,
                                         5.64299335673836e-06,
                                         1.58822971351864e-07,
                                         7.94948767956052e-10,
                                         3.92230249620262e-14,
                                         0.000365708256998698,
                                         0.00916287779792744,
                                         0.045950998054323,
                                         0.120685606699555,
                                         0.216069264156086,
                                         0.293542458344168,
                                         0.318362752116162,
                                         0.282337738765935,
                                         0.206425538890078,
                                         0.12397325768716,
                                         0.0602528095227768,
                                         0.0230319667180795,
                                         0.0065997120642815,
                                         0.00130800153406177,
                                         0.000155556070816479,
                                         8.39652655459267e-06,
                                         1.05585254270088e-07,
                                         2.77830836035845e-11,
                                         2.58146005276901e-06,
                                         0.000344935401574243,
                                         0.00434588874347064,
                                         0.021890115676134,
                                         0.0656316761907518,
                                         0.138082199755362,
                                         0.221092456279348,
                                         0.281203338135137,
                                         0.290268809764174,
                                         0.24489987733575,
                                         0.168044097237528,
                                         0.0921248617786786,
                                         0.0389722713303529,
                                         0.0119614891172009,
                                         0.00238227495764645,
                                         0.00024661160501842,
                                         7.79103032146277e-06,
                                         1.09332002379141e-08,
                                         1.09332002379141e-08,
                                         7.79103032146277e-06,
                                         0.00024661160501842,
                                         0.00238227495764644,
                                         0.0119614891172009,
                                         0.0389722713303529,
                                         0.0921248617786786,
                                         0.168044097237528,
                                         0.24489987733575,
                                         0.290268809764174,
                                         0.281203338135137,
                                         0.221092456279348,
                                         0.138082199755362,
                                         0.0656316761907518,
                                         0.0218901156761341,
                                         0.00434588874347064,
                                         0.000344935401574243,
                                         2.58146005276901e-06,
                                         2.77830836035845e-11,
                                         1.05585254270088e-07,
                                         8.39652655459267e-06,
                                         0.000155556070816479,
                                         0.00130800153406177,
                                         0.0065997120642815,
                                         0.0230319667180795,
                                         0.0602528095227768,
                                         0.12397325768716,
                                         0.206425538890078,
                                         0.282337738765935,
                                         0.318362752116162,
                                         0.293542458344168,
                                         0.216069264156086,
                                         0.120685606699555,
                                         0.045950998054323,
                                         0.00916287779792744,
                                         0.000365708256998698,
                                         3.92230249620262e-14,
                                         7.94948767956052e-10,
                                         1.58822971351864e-07,
                                         5.64299335673834e-06,
                                         7.9461864671816e-05,
                                         0.00062090013467116,
                                         0.00319898861333274,
                                         0.0120021441298514,
                                         0.0348654242541196,
                                         0.0815556356205716,
                                         0.15748706426402,
                                         0.254681894023045,
                                         0.346682286201018,
                                         0.395184575011754,
                                         0.36964973090367,
                                         0.26992231391626,
                                         0.135223917864324,
                                         0.0287827057606339,
                                         2.37314862287524e-17,
                                         2.56506411962059e-12,
                                         1.28750873003575e-09,
                                         8.77314541595956e-08,
                                         2.06886649136993e-06,
                                         2.50346621280949e-05,
                                         0.000190422200218412,
                                         0.0010246217907608,
                                         0.00420228119387966,
                                         0.0138091752542664,
                                         0.0376481861539915,
                                         0.0873166582711359,
                                         0.175475137508019,
                                         0.309763463258982,
                                         0.485230984967367,
                                         0.679525631043893,
                                         0.855260371523085,
                                         0.970848993561292,

                                         0.973724739478554,
                                         0.868771103835969,
                                         0.706497445227301,
                                         0.522180717653786,
                                         0.34929822280336,
                                         0.210216031048477,
                                         0.112917825555881,
                                         0.0535653676976678,
                                         0.0221276721186275,
                                         0.0078125,
                                         0.00229851199108852,
                                         0.00054409476381518,
                                         9.85609151974571e-05,
                                         1.26783501823302e-05,
                                         1.02893507519265e-06,
                                         4.30031096923521e-08,
                                         6.24168847809301e-10,
                                         1.23409554030576e-12,
                                         1.13696304849452e-17,
                                         0.0259763753379508,
                                         0.123451202307739,
                                         0.251656076026484,
                                         0.355527351924447,
                                         0.396441334813594,
                                         0.367251375360567,
                                         0.28896969016475,
                                         0.194642605695934,
                                         0.11208186416695,
                                         0.0546875,
                                         0.0222353124102691,
                                         0.00733697107637154,
                                         0.00188716998971387,
                                         0.000355599567720664,
                                         4.44223468079519e-05,
                                         3.0948795736486e-06,
                                         8.5862068038227e-08,
                                         4.25554224767889e-10,
                                         2.08833459848254e-14,
                                         0.000296991563136116,
                                         0.00751810687398467,
                                         0.0384173549360853,
                                         0.103740519894834,
                                         0.192834483162211,
                                         0.274969322948799,
                                         0.316931417374285,
                                         0.303120245522948,
                                         0.243309002562003,
                                         0.1640625,
                                         0.0921855630137715,
                                         0.042401603666598,
                                         0.0154860454845544,
                                         0.00427447636813722,
                                         0.000821936448179471,
                                         9.54576771365523e-05,
                                         5.06201950711366e-06,
                                         6.28903837332664e-08,
                                         1.64390619415994e-11,
                                         1.8864146839945e-06,
                                         0.000254360202034222,
                                         0.00325817940718655,
                                         0.0168171081474633,
                                         0.0521096266416783,
                                         0.114375380696803,
                                         0.193110455559094,
                                         0.262252401059121,
                                         0.293432697448997,
                                         0.2734375,
                                         0.212329376288294,
                                         0.136136710517544,
                                         0.0705988349565245,
                                         0.028545135659314,
                                         0.00844894484909402,
                                         0.00163570681965068,
                                         0.000165795897199209,
                                         5.16346310022053e-06,
                                         7.18921505192884e-09,
                                         7.18921505192884e-09,
                                         5.16346310022053e-06,
                                         0.000165795897199209,
                                         0.00163570681965068,
                                         0.00844894484909401,
                                         0.028545135659314,
                                         0.0705988349565245,
                                         0.136136710517544,
                                         0.212329376288294,
                                         0.2734375,
                                         0.293432697448997,
                                         0.262252401059121,
                                         0.193110455559094,
                                         0.114375380696803,
                                         0.0521096266416784,
                                         0.0168171081474633,
                                         0.00325817940718655,
                                         0.000254360202034222,
                                         1.8864146839945e-06,
                                         1.64390619415994e-11,
                                         6.28903837332664e-08,
                                         5.06201950711366e-06,
                                         9.54576771365523e-05,
                                         0.000821936448179469,
                                         0.00427447636813723,
                                         0.0154860454845544,
                                         0.0424016036665979,
                                         0.0921855630137715,
                                         0.1640625,
                                         0.243309002562002,
                                         0.303120245522948,
                                         0.316931417374285,
                                         0.274969322948799,
                                         0.192834483162211,
                                         0.103740519894834,
                                         0.0384173549360853,
                                         0.00751810687398467,
                                         0.000296991563136116,
                                         2.08833459848254e-14,
                                         4.25554224767889e-10,
                                         8.5862068038227e-08,
                                         3.0948795736486e-06,
                                         4.44223468079518e-05,
                                         0.000355599567720665,
                                         0.00188716998971387,
                                         0.00733697107637153,
                                         0.0222353124102691,
                                         0.0546875,
                                         0.11208186416695,
                                         0.194642605695934,
                                         0.28896969016475,
                                         0.367251375360567,
                                         0.396441334813594,
                                         0.355527351924447,
                                         0.251656076026484,
                                         0.123451202307739,
                                         0.0259763753379508,
                                         1.13696304849452e-17,
                                         1.23409554030576e-12,
                                         6.24168847809301e-10,
                                         4.30031096923521e-08,
                                         1.02893507519265e-06,
                                         1.26783501823302e-05,
                                         9.85609151974571e-05,
                                         0.00054409476381518,
                                         0.00229851199108852,
                                         0.0078125,
                                         0.0221276721186275,
                                         0.0535653676976678,
                                         0.112917825555881,
                                         0.210216031048477,
                                         0.34929822280336,
                                         0.522180717653786,
                                         0.706497445227301,
                                         0.868771103835969,
                                         0.973724739478554,

                                         0.976196567367861,
                                         0.88051540924611,
                                         0.730428922953023,
                                         0.555975297182679,
                                         0.38697022724605,
                                         0.245116685329897,
                                         0.140402125697163,
                                         0.0721185178433988,
                                         0.0328585504402887,
                                         0.0130907088805282,
                                         0.00447467010885329,
                                         0.00127924672339679,
                                         0.000295282260900386,
                                         5.23333777910373e-05,
                                         6.60771573402766e-06,
                                         5.28020695703352e-07,
                                         2.17952399551297e-08,
                                         3.13371383563937e-10,
                                         6.15569715074035e-13,
                                         5.65076185442762e-18,
                                         0.0235583721185029,
                                         0.113068272156229,
                                         0.234670800652588,
                                         0.340450844297936,
                                         0.393472739101553,
                                         0.381689925755387,
                                         0.318179686300911,
                                         0.230159655628361,
                                         0.144664366967263,
                                         0.0786069415233813,
                                         0.0365139964386586,
                                         0.0142376142757751,
                                         0.00453368175334123,
                                         0.00113155607461202,
                                         0.000207926387842072,
                                         2.5445437883706e-05,
                                         1.74404953165637e-06,
                                         4.77941037217861e-08,
                                         2.34892705560329e-10,
                                         1.14734779033256e-14,
                                         0.000243655653888718,
                                         0.00622254097712791,
                                         0.032311964507659,
                                         0.0893462954486217,
                                         0.171464792950121,
                                         0.254725095916331,
                                         0.309025580036677,
                                         0.314800069958618,
                                         0.272959337910312,
                                         0.202293462334452,
                                         0.127697069136316,
                                         0.067911503775161,
                                         0.0298323811711126,
                                         0.0104856802187149,
                                         0.00280408440037196,
                                         0.000525523483423647,
                                         5.9810741935254e-05,
                                         3.12401077495157e-06,
                                         3.84136017933627e-08,
                                         9.98404148902157e-12,
                                         1.40002310887155e-06,
                                         0.000190248950546227,
                                         0.00247169663356501,
                                         0.0130264530827157,
                                         0.0415109588427052,
                                         0.0944409324476511,
                                         0.166741577564465,
                                         0.239203723788553,
                                         0.286128973009092,
                                         0.289221311546274,
                                         0.248101840031537,
                                         0.179960406898712,
                                         0.109056687595715,
                                         0.0539814607296651,
                                         0.0210087420467862,
                                         0.00602978491756742,
                                         0.00113953340134022,
                                         0.000113443134915462,
                                         3.49002087574654e-06,
                                         4.82664286444923e-09,
                                         4.82664286444892e-09,
                                         3.4900208757465e-06,
                                         0.000113443134915462,
                                         0.00113953340134021,
                                         0.00602978491756742,
                                         0.0210087420467862,
                                         0.0539814607296651,
                                         0.109056687595715,
                                         0.179960406898712,
                                         0.248101840031537,
                                         0.289221311546273,
                                         0.286128973009092,
                                         0.239203723788553,
                                         0.166741577564465,
                                         0.094440932447651,
                                         0.0415109588427052,
                                         0.0130264530827157,
                                         0.00247169663356501,
                                         0.000190248950546229,
                                         1.40002310887162e-06,
                                         9.98404148902077e-12,
                                         3.84136017933621e-08,
                                         3.12401077495157e-06,
                                         5.98107419352538e-05,
                                         0.000525523483423647,
                                         0.00280408440037197,
                                         0.0104856802187149,
                                         0.0298323811711126,
                                         0.067911503775161,
                                         0.127697069136316,
                                         0.202293462334452,
                                         0.272959337910312,
                                         0.314800069958618,
                                         0.309025580036677,
                                         0.254725095916331,
                                         0.171464792950121,
                                         0.0893462954486219,
                                         0.032311964507659,
                                         0.00622254097712795,
                                         0.000243655653888726,
                                         1.14734779033245e-14,
                                         2.34892705560325e-10,
                                         4.77941037217861e-08,
                                         1.74404953165636e-06,
                                         2.5445437883706e-05,
                                         0.000207926387842072,
                                         0.00113155607461202,
                                         0.00453368175334123,
                                         0.0142376142757751,
                                         0.0365139964386586,
                                         0.0786069415233813,
                                         0.144664366967263,
                                         0.230159655628361,
                                         0.318179686300911,
                                         0.381689925755387,
                                         0.393472739101553,
                                         0.340450844297936,
                                         0.234670800652588,
                                         0.11306827215623,
                                         0.0235583721185033,
                                         5.65076185442698e-18,
                                         6.15569715074022e-13,
                                         3.13371383563937e-10,
                                         2.17952399551296e-08,
                                         5.28020695703352e-07,
                                         6.60771573402767e-06,
                                         5.23333777910373e-05,
                                         0.000295282260900386,
                                         0.00127924672339679,
                                         0.00447467010885329,
                                         0.0130907088805282,
                                         0.0328585504402887,
                                         0.0721185178433988,
                                         0.140402125697163,
                                         0.245116685329897,
                                         0.38697022724605,
                                         0.555975297182679,
                                         0.730428922953023,
                                         0.88051540924611,
                                         0.976196567367861,

                                         0.978336469198365,
                                         0.890781350111176,
                                         0.751719661380738,
                                         0.586832352594959,
                                         0.422592976245175,
                                         0.279635530435334,
                                         0.16915519179793,
                                         0.0929159456895194,
                                         0.0459463889894858,
                                         0.0202264579705358,
                                         0.0078125,
                                         0.00259749328051612,
                                         0.000724534328239758,
                                         0.000163659495084149,
                                         2.84656792621858e-05,
                                         3.5369768206122e-06,
                                         2.78891511560826e-07,
                                         1.13890484506114e-08,
                                         1.62421239301908e-10,
                                         3.17262243486129e-13,
                                         2.90331499607672e-18,
                                         0.0214607191296514,
                                         0.103880265823414,
                                         0.218967999654001,
                                         0.325007963186267,
                                         0.387341119431338,
                                         0.390824517854616,
                                         0.342172961824708,
                                         0.262868427992087,
                                         0.177784283323365,
                                         0.105603901070251,
                                         0.0546875,
                                         0.0243775686181874,
                                         0.00917514775492572,
                                         0.00283458255818483,
                                         0.000689536522901207,
                                         0.000124004899552202,
                                         1.49093959106605e-05,
                                         1.00763574989925e-06,
                                         2.73221051518847e-08,
                                         1.33306870946422e-10,
                                         6.48536180584406e-15,
                                         0.000201754631485758,
                                         0.0051918029810232,
                                         0.0273355997696612,
                                         0.0771431147005187,
                                         0.152155672143844,
                                         0.23409625407529,
                                         0.296639691531881,
                                         0.318720368741796,
                                         0.29482109872538,
                                         0.236299771407634,
                                         0.1640625,
                                         0.0980504423012164,
                                         0.049795532529488,
                                         0.0210407021560248,
                                         0.00715840411639092,
                                         0.00186323953552717,
                                         0.000341592253423101,
                                         3.82069974205487e-05,
                                         1.96973912614713e-06,
                                         2.40054522803663e-08,
                                         6.20865426632319e-12,
                                         1.05373219788318e-06,
                                         0.000144155378096451,
                                         0.00189585037272781,
                                         0.0101725031380004,
                                         0.0332055123222507,
                                         0.0778994915002695,
                                         0.142869764075645,
                                         0.214688462265752,
                                         0.271613436453665,
                                         0.293747471034107,
                                         0.2734375,
                                         0.219096894317552,
                                         0.150139577895451,
                                         0.086767851101552,
                                         0.041285984451282,
                                         0.0155534247225901,
                                         0.0043479393165472,
                                         0.000804840358035996,
                                         7.8891599219885e-05,
                                         2.40156721394334e-06,
                                         3.30208492368816e-09,
                                         3.30208492368839e-09,
                                         2.40156721394331e-06,
                                         7.88915992198854e-05,
                                         0.000804840358035996,
                                         0.0043479393165472,
                                         0.0155534247225901,
                                         0.041285984451282,
                                         0.0867678511015519,
                                         0.150139577895451,
                                         0.219096894317552,
                                         0.2734375,
                                         0.293747471034107,
                                         0.271613436453665,
                                         0.214688462265752,
                                         0.142869764075645,
                                         0.0778994915002695,
                                         0.0332055123222507,
                                         0.0101725031380004,
                                         0.0018958503727278,
                                         0.000144155378096453,
                                         1.05373219788312e-06,
                                         6.20865426632375e-12,
                                         2.40054522803659e-08,
                                         1.96973912614715e-06,
                                         3.82069974205487e-05,
                                         0.000341592253423101,
                                         0.00186323953552717,
                                         0.00715840411639092,
                                         0.0210407021560248,
                                         0.049795532529488,
                                         0.0980504423012164,
                                         0.1640625,
                                         0.236299771407634,
                                         0.29482109872538,
                                         0.318720368741796,
                                         0.296639691531881,
                                         0.23409625407529,
                                         0.152155672143844,
                                         0.0771431147005187,
                                         0.0273355997696611,
                                         0.00519180298102323,
                                         0.000201754631485751,
                                         6.48536180584475e-15,
                                         1.33306870946419e-10,
                                         2.73221051518849e-08,
                                         1.00763574989925e-06,
                                         1.49093959106605e-05,
                                         0.000124004899552202,
                                         0.000689536522901207,
                                         0.00283458255818483,
                                         0.00917514775492572,
                                         0.0243775686181874,
                                         0.0546875,
                                         0.105603901070251,
                                         0.177784283323365,
                                         0.262868427992088,
                                         0.342172961824708,
                                         0.390824517854616,
                                         0.387341119431338,
                                         0.325007963186267,
                                         0.218967999654001,
                                         0.103880265823414,
                                         0.021460719129651,
                                         2.90331499607708e-18,
                                         3.17262243486122e-13,
                                         1.6242123930191e-10,
                                         1.13890484506114e-08,
                                         2.78891511560826e-07,
                                         3.5369768206122e-06,
                                         2.84656792621858e-05,
                                         0.000163659495084149,
                                         0.000724534328239758,
                                         0.00259749328051612,
                                         0.0078125,
                                         0.0202264579705358,
                                         0.0459463889894858,
                                         0.0929159456895194,
                                         0.16915519179793,
                                         0.279635530435334,
                                         0.422592976245175,
                                         0.586832352594959,
                                         0.751719661380738,
                                         0.890781350111176,
                                         0.978336469198365,

                                         0.980201135684649,
                                         0.899802034928746,
                                         0.770715042394355,
                                         0.61498729685726,
                                         0.456090617808426,
                                         0.313370558539771,
                                         0.19863465885222,
                                         0.115525780189393,
                                         0.0612226679668602,
                                         0.0293026770626826,
                                         0.0125237852716783,
                                         0.00471002091732903,
                                         0.00152898069154182,
                                         0.000417510871561184,
                                         9.25579579894907e-05,
                                         1.58388181371484e-05,
                                         1.94086476176554e-06,
                                         1.5127491189003e-07,
                                         6.12029987973844e-09,
                                         8.66661943997641e-11,
                                         1.68462038852163e-13,
                                         1.53744805132756e-18,
                                         0.0196295857256514,
                                         0.0957216161707918,
                                         0.204503083115557,
                                         0.309605009419575,
                                         0.378910941032084,
                                         0.395500688257218,
                                         0.361141196174155,
                                         0.292047409240671,
                                         0.210158420975289,
                                         0.134521064807784,
                                         0.0762360522300465,
                                         0.0379135219432858,
                                         0.0163198094489065,
                                         0.00595976757834684,
                                         0.00179405352893373,
                                         0.000426871194038755,
                                         7.53533046405333e-05,
                                         8.92230328295759e-06,
                                         5.95699105827036e-07,
                                         1.60044141877713e-08,
                                         7.75952400317919e-11,
                                         3.76184240485053e-15,
                                         0.000168472989179581,
                                         0.00436411603251342,
                                         0.0232556830086233,
                                         0.0667994716644947,
                                         0.134910691266675,
                                         0.213923999854287,
                                         0.281398805930724,
                                         0.316410597225363,
                                         0.309175133330406,
                                         0.26466499735517,
                                         0.198887917221197,
                                         0.130794245901368,
                                         0.0746537075102097,
                                         0.0364597824261607,
                                         0.0149032245058435,
                                         0.00493053606982463,
                                         0.00125381235771946,
                                         0.000225533089426752,
                                         2.48487421244017e-05,
                                         1.26663957333766e-06,
                                         1.53176211514553e-08,
                                         3.94479064559854e-12,
                                         8.03298440532971e-07,
                                         0.000110537604932635,
                                         0.00146921662217127,
                                         0.00800692150553196,
                                         0.0266859755846531,
                                         0.0642834640753651,
                                         0.121813371393712,
                                         0.190447895461017,
                                         0.252691021865712,
                                         0.289287864506997,
                                         0.288259676939709,
                                         0.250674779575387,
                                         0.189720898616707,
                                         0.123915694985664,
                                         0.0687784818907889,
                                         0.0316387215671883,
                                         0.0115901827462374,
                                         0.00316716764054091,
                                         0.000575849991608476,
                                         5.56921286396264e-05,
                                         1.67986763136631e-06,
                                         2.29813118781267e-09,
                                         2.29813118781267e-09,
                                         1.67986763136631e-06,
                                         5.56921286396261e-05,
                                         0.000575849991608478,
                                         0.00316716764054091,
                                         0.0115901827462374,
                                         0.0316387215671883,
                                         0.068778481890789,
                                         0.123915694985664,
                                         0.189720898616707,
                                         0.250674779575387,
                                         0.288259676939709,
                                         0.289287864506998,
                                         0.252691021865712,
                                         0.190447895461017,
                                         0.121813371393712,
                                         0.0642834640753651,
                                         0.026685975584653,
                                         0.00800692150553194,
                                         0.00146921662217128,
                                         0.000110537604932635,
                                         8.03298440532971e-07,
                                         3.94479064559854e-12,
                                         1.53176211514553e-08,
                                         1.26663957333765e-06,
                                         2.48487421244018e-05,
                                         0.000225533089426753,
                                         0.00125381235771946,
                                         0.00493053606982463,
                                         0.0149032245058435,
                                         0.0364597824261607,
                                         0.0746537075102096,
                                         0.130794245901368,
                                         0.198887917221197,
                                         0.26466499735517,
                                         0.309175133330406,
                                         0.316410597225363,
                                         0.281398805930724,
                                         0.213923999854287,
                                         0.134910691266675,
                                         0.0667994716644946,
                                         0.0232556830086233,
                                         0.00436411603251342,
                                         0.000168472989179581,
                                         3.76184240485053e-15,
                                         7.75952400317919e-11,
                                         1.60044141877712e-08,
                                         5.95699105827039e-07,
                                         8.92230328295762e-06,
                                         7.53533046405333e-05,
                                         0.000426871194038755,
                                         0.00179405352893373,
                                         0.00595976757834684,
                                         0.0163198094489064,
                                         0.0379135219432858,
                                         0.0762360522300465,
                                         0.134521064807784,
                                         0.210158420975289,
                                         0.292047409240671,
                                         0.361141196174155,
                                         0.395500688257218,
                                         0.378910941032084,
                                         0.309605009419575,
                                         0.204503083115557,
                                         0.0957216161707918,
                                         0.0196295857256514,
                                         1.53744805132756e-18,
                                         1.68462038852163e-13,
                                         8.66661943997631e-11,
                                         6.12029987973848e-09,
                                         1.51274911890031e-07,
                                         1.94086476176554e-06,
                                         1.58388181371484e-05,
                                         9.25579579894907e-05,
                                         0.000417510871561184,
                                         0.00152898069154182,
                                         0.00471002091732903,
                                         0.0125237852716783,
                                         0.0293026770626826,
                                         0.0612226679668602,
                                         0.115525780189393,
                                         0.19863465885222,
                                         0.313370558539771,
                                         0.456090617808426,
                                         0.61498729685726,
                                         0.770715042394355,
                                         0.899802034928746,
                                         0.980201135684649,

                                         0.981835687264961,
                                         0.907767375406583,
                                         0.787712134983158,
                                         0.640676541012468,
                                         0.487463956671044,
                                         0.34603675197343,
                                         0.228382563918332,
                                         0.139516260811256,
                                         0.0784425962852767,
                                         0.0403037873154724,
                                         0.0187537633445454,
                                         0.0078125,
                                         0.00287093293064401,
                                         0.000912797088858873,
                                         0.000244679857962669,
                                         5.33646853206015e-05,
                                         9.00315366196295e-06,
                                         1.0899226619756e-06,
                                         8.40952832111374e-08,
                                         3.37474029268917e-09,
                                         4.74925265387767e-11,
                                         9.19221477333445e-14,
                                         8.36928536797698e-19,
                                         0.0180219208015663,
                                         0.0884520077048747,
                                         0.191206387220227,
                                         0.294515739971171,
                                         0.368869954274127,
                                         0.396514736040439,
                                         0.375473941062165,
                                         0.317341615918227,
                                         0.240800069587592,
                                         0.16422828354244,
                                         0.100403432252597,
                                         0.0546875,
                                         0.0262759846154603,
                                         0.0109766099234674,
                                         0.00390561698822856,
                                         0.00114960285102881,
                                         0.000268332876129319,
                                         4.66073765109113e-05,
                                         5.44548974999852e-06,
                                         3.59721792626804e-07,
                                         9.58708315634491e-09,
                                         4.62256597536422e-11,
                                         2.23420074886271e-15,
                                         0.000141770723200571,
                                         0.00369372054007889,
                                         0.0198911764609069,
                                         0.0580231647138582,
                                         0.119626464952611,
                                         0.194724076066543,
                                         0.264557375005152,
                                         0.30935165531861,
                                         0.316799488284471,
                                         0.286796114974627,
                                         0.230373171877843,
                                         0.1640625,
                                         0.103066651282937,
                                         0.0565699025004194,
                                         0.0267180216381245,
                                         0.0106136540123101,
                                         0.00342749100316607,
                                         0.000854155314086654,
                                         0.000151121225582492,
                                         1.64329811059999e-05,
                                         8.29413174286887e-07,
                                         9.96252188408458e-09,
                                         2.5561081468461e-12,
                                         6.19583049199914e-07,
                                         8.56934975387588e-05,
                                         0.00114959810574629,
                                         0.00635070310435903,
                                         0.0215530507450925,
                                         0.0531260426487963,
                                         0.103558902519045,
                                         0.167534924594432,
                                         0.231547357821663,
                                         0.278244167171674,
                                         0.293658610458761,
                                         0.2734375,
                                         0.224597453237212,
                                         0.16196833748304,
                                         0.101542169536682,
                                         0.0544389218088151,
                                         0.0243223904623489,
                                         0.00869654065753194,
                                         0.00232992254650951,
                                         0.000417055120505166,
                                         3.98641822116301e-05,
                                         1.19284208549276e-06,
                                         1.62466472423674e-09,
                                         1.62466472423688e-09,
                                         1.19284208549276e-06,
                                         3.98641822116298e-05,
                                         0.000417055120505166,
                                         0.00232992254650951,
                                         0.00869654065753193,
                                         0.0243223904623489,
                                         0.0544389218088151,
                                         0.101542169536682,
                                         0.16196833748304,
                                         0.224597453237212,
                                         0.2734375,
                                         0.293658610458761,
                                         0.278244167171674,
                                         0.231547357821663,
                                         0.167534924594433,
                                         0.103558902519045,
                                         0.0531260426487964,
                                         0.0215530507450925,
                                         0.00635070310435903,
                                         0.00114959810574629,
                                         8.56934975387588e-05,
                                         6.19583049199874e-07,
                                         2.55610814684637e-12,
                                         9.96252188408458e-09,
                                         8.2941317428688e-07,
                                         1.64329811059999e-05,
                                         0.000151121225582492,
                                         0.000854155314086653,
                                         0.00342749100316607,
                                         0.0106136540123101,
                                         0.0267180216381245,
                                         0.0565699025004194,
                                         0.103066651282937,
                                         0.1640625,
                                         0.230373171877843,
                                         0.286796114974627,
                                         0.316799488284471,
                                         0.30935165531861,
                                         0.264557375005152,
                                         0.194724076066544,
                                         0.119626464952611,
                                         0.0580231647138582,
                                         0.0198911764609069,
                                         0.00369372054007889,
                                         0.000141770723200565,
                                         2.23420074886299e-15,
                                         4.62256597536422e-11,
                                         9.58708315634482e-09,
                                         3.59721792626804e-07,
                                         5.44548974999852e-06,
                                         4.66073765109112e-05,
                                         0.000268332876129319,
                                         0.0011496028510288,
                                         0.00390561698822856,
                                         0.0109766099234674,
                                         0.0262759846154603,
                                         0.0546875,
                                         0.100403432252597,
                                         0.16422828354244,
                                         0.240800069587592,
                                         0.317341615918227,
                                         0.375473941062165,
                                         0.396514736040439,
                                         0.368869954274127,
                                         0.294515739971171,
                                         0.191206387220227,
                                         0.0884520077048747,
                                         0.0180219208015659,
                                         8.36928536797822e-19,
                                         9.19221477333445e-14,
                                         4.74925265387761e-11,
                                         3.37474029268917e-09,
                                         8.40952832111374e-08,
                                         1.0899226619756e-06,
                                         9.00315366196297e-06,
                                         5.33646853206014e-05,
                                         0.000244679857962669,
                                         0.000912797088858873,
                                         0.00287093293064401,
                                         0.0078125,
                                         0.0187537633445454,
                                         0.0403037873154724,
                                         0.0784425962852767,
                                         0.139516260811256,
                                         0.228382563918332,
                                         0.34603675197343,
                                         0.487463956671044,
                                         0.640676541012468,
                                         0.787712134983158,
                                         0.907767375406583,
                                         0.981835687264961,

                                         0.983276388416166,
                                         0.914833111934895,
                                         0.802965872469459,
                                         0.664127658385973,
                                         0.516764966577533,
                                         0.377443319770002,
                                         0.258025861325193,
                                         0.164482092846195,
                                         0.0973186995710779,
                                         0.0531335663444866,
                                         0.0265751520607697,
                                         0.0120654625515744,
                                         0.00491514327527196,
                                         0.00177003954205364,
                                         0.000552630194915283,
                                         0.000145751805888448,
                                         3.13366423072629e-05,
                                         5.22131312141857e-06,
                                         6.25390102499986e-07,
                                         4.78259961244181e-08,
                                         1.90556076317127e-09,
                                         2.66710298301593e-11,
                                         5.14280953071714e-14,
                                         4.67263761768589e-19,
                                         0.0166029785300273,
                                         0.0819525681125272,
                                         0.178996312506076,
                                         0.279918248656237,
                                         0.357756023857913,
                                         0.394580293942502,
                                         0.385661682032654,
                                         0.338671491196503,
                                         0.269018763604518,
                                         0.193727230670111,
                                         0.126329099287118,
                                         0.0742893599930842,
                                         0.0391155662037998,
                                         0.0182453167338771,
                                         0.00742692412508311,
                                         0.00258359091737021,
                                         0.000745741863514427,
                                         0.000171171936519054,
                                         2.93132137693409e-05,
                                         3.38505876739845e-06,
                                         2.21533412230133e-07,
                                         5.86258116400465e-09,
                                         2.81303948388905e-11,
                                         1.35596283164817e-15,
                                         0.000120148849579125,
                                         0.00314634596000642,
                                         0.0171007160485293,
                                         0.0505631441752794,
                                         0.1061461047239,
                                         0.176783714699912,
                                         0.247043231953079,
                                         0.298855925715137,
                                         0.31870733766013,
                                         0.302716147935993,
                                         0.25736782707289,
                                         0.196034483359586,
                                         0.133409352085544,
                                         0.0806014881452719,
                                         0.0427766311002428,
                                         0.0196271286195733,
                                         0.00760585079662577,
                                         0.00240496413480119,
                                         0.000588842409643092,
                                         0.000102681360891187,
                                         1.10377076432986e-05,
                                         5.52282352640547e-07,
                                         6.59438699854727e-09,
                                         1.68638481968064e-12,
                                         4.8303736013657e-07,
                                         6.71085532091358e-05,
                                         0.000907636603974112,
                                         0.00507416314339264,
                                         0.0174963985334008,
                                         0.0440024252250925,
                                         0.0879157984217301,
                                         0.146511797906954,
                                         0.209763078145012,
                                         0.262789495611459,
                                         0.29129471614663,
                                         0.287386142433357,
                                         0.252784490097784,
                                         0.197816361011389,
                                         0.136877374017709,
                                         0.0828356496764302,
                                         0.0430957630327638,
                                         0.018772068882903,
                                         0.00657146534897703,
                                         0.00173039206159768,
                                         0.000305524492502017,
                                         2.8904200357546e-05,
                                         8.58816793398183e-07,
                                         1.16518015164528e-09,
                                         1.16518015164538e-09,
                                         8.58816793398183e-07,
                                         2.89042003575462e-05,
                                         0.000305524492502017,
                                         0.00173039206159768,
                                         0.00657146534897704,
                                         0.018772068882903,
                                         0.0430957630327638,
                                         0.0828356496764302,
                                         0.136877374017709,
                                         0.197816361011389,
                                         0.252784490097783,
                                         0.287386142433357,
                                         0.29129471614663,
                                         0.262789495611459,
                                         0.209763078145012,
                                         0.146511797906954,
                                         0.0879157984217301,
                                         0.0440024252250925,
                                         0.0174963985334007,
                                         0.00507416314339264,
                                         0.000907636603974107,
                                         6.71085532091358e-05,
                                         4.83037360136536e-07,
                                         1.68638481968083e-12,
                                         6.59438699854727e-09,
                                         5.52282352640552e-07,
                                         1.10377076432986e-05,
                                         0.000102681360891187,
                                         0.000588842409643094,
                                         0.00240496413480119,
                                         0.00760585079662578,
                                         0.0196271286195733,
                                         0.0427766311002428,
                                         0.0806014881452719,
                                         0.133409352085543,
                                         0.196034483359586,
                                         0.25736782707289,
                                         0.302716147935993,
                                         0.31870733766013,
                                         0.298855925715137,
                                         0.247043231953079,
                                         0.176783714699912,
                                         0.1061461047239,
                                         0.0505631441752794,
                                         0.0171007160485292,
                                         0.00314634596000642,
                                         0.00012014884957912,
                                         1.35596283164835e-15,
                                         2.81303948388905e-11,
                                         5.86258116400471e-09,
                                         2.21533412230133e-07,
                                         3.38505876739845e-06,
                                         2.93132137693409e-05,
                                         0.000171171936519054,
                                         0.000745741863514428,
                                         0.00258359091737021,
                                         0.00742692412508311,
                                         0.0182453167338771,
                                         0.0391155662037998,
                                         0.0742893599930842,
                                         0.126329099287118,
                                         0.193727230670111,
                                         0.269018763604518,
                                         0.338671491196503,
                                         0.385661682032654,
                                         0.394580293942502,
                                         0.357756023857913,
                                         0.279918248656237,
                                         0.178996312506076,
                                         0.0819525681125272,
                                         0.0166029785300269,
                                         4.67263761768665e-19,
                                         5.14280953071714e-14,
                                         2.66710298301596e-11,
                                         1.90556076317127e-09,
                                         4.78259961244181e-08,
                                         6.25390102499987e-07,
                                         5.22131312141857e-06,
                                         3.1336642307263e-05,
                                         0.000145751805888448,
                                         0.000552630194915283,
                                         0.00177003954205364,
                                         0.00491514327527195,
                                         0.0120654625515744,
                                         0.0265751520607697,
                                         0.0531335663444866,
                                         0.0973186995710779,
                                         0.164482092846195,
                                         0.258025861325193,
                                         0.377443319770002,
                                         0.516764966577533,
                                         0.664127658385973,
                                         0.802965872469459,
                                         0.914833111934895,
                                         0.983276388416166,

                                         0.00390625,
                                         0.03125,
                                         0.109375,
                                         0.21875,
                                         0.2734375,
                                         0.21875,
                                         0.109375,
                                         0.03125,
                                         0.00390625,

                                         0.149687380573964,
                                         3.97745072752434e-06,
                                         0.32086890193539,
                                         0.000118752385597781,
                                         0.300917971024471,
                                         0.00155116477799825,
                                         0.161261454648038,
                                         0.0115780515248016,
                                         0.0540123456790123,
                                         0.0540123456790123,
                                         0.0115780515248016,
                                         0.161261454648038,
                                         0.00155116477799825,
                                         0.300917971024471,
                                         0.000118752385597781,
                                         0.32086890193539,
                                         3.97745072752434e-06,
                                         0.149687380573964,

                                         0.384199973971887,
                                         0.00390625,
                                         2.60281121224955e-08,
                                         0.390398360648854,
                                         0.03125,
                                         1.63935114618901e-06,
                                         0.173554826955047,
                                         0.109375,
                                         4.51730449538622e-05,
                                         0.0440887067387617,
                                         0.21875,
                                         0.000711293261238472,
                                         0.00700000000000005,
                                         0.2734375,
                                         0.00700000000000005,
                                         0.000711293261238471,
                                         0.21875,
                                         0.0440887067387617,
                                         4.51730449538622e-05,
                                         0.109375,
                                         0.173554826955047,
                                         1.63935114618901e-06,
                                         0.03125,
                                         0.390398360648854,
                                         2.60281121224955e-08,
                                         0.00390625,
                                         0.384199973971887,

                                         0.562322543291604,
                                         0.0406021724085881,
                                         0.000140673180788785,
                                         5.40094316100545e-10,
                                         0.335649493049349,
                                         0.159991537728688,
                                         0.00228477553580074,
                                         5.7909401941954e-08,
                                         0.0876525497574884,
                                         0.275818131124848,
                                         0.0162350574475442,
                                         2.71648061425736e-06,
                                         0.013079919274785,
                                         0.271713090176816,
                                         0.0659213466818577,
                                         7.28158782036092e-05,
                                         0.00121990381845877,
                                         0.167293215715069,
                                         0.167293215715069,
                                         0.00121990381845877,
                                         7.28158782036089e-05,
                                         0.0659213466818577,
                                         0.271713090176816,
                                         0.013079919274785,
                                         2.71648061425735e-06,
                                         0.0162350574475442,
                                         0.275818131124848,
                                         0.0876525497574885,
                                         5.79094019419536e-08,
                                         0.00228477553580074,
                                         0.159991537728688,
                                         0.335649493049349,
                                         5.40094316100541e-10,
                                         0.000140673180788785,
                                         0.0406021724085881,
                                         0.562322543291604,

                                         0.680880697126515,
                                         0.12259442822481,
                                         0.00390625,
                                         8.04201197864117e-06,
                                         2.34492614456703e-11,
                                         0.268097816847257,
                                         0.294220187112653,
                                         0.03125,
                                         0.000214458347261367,
                                         3.81142069245898e-09,
                                         0.0461842175427365,
                                         0.308924433958084,
                                         0.109375,
                                         0.00250206882172024,
                                         2.71033306581782e-07,
                                         0.00454627659010641,
                                         0.185350602957548,
                                         0.21875,
                                         0.0166808239550454,
                                         1.10133740826422e-05,
                                         0.00027970365112584,
                                         0.0695049546108987,
                                         0.2734375,
                                         0.0695049546108987,
                                         0.000279703651125842,
                                         1.10133740826421e-05,
                                         0.0166808239550454,
                                         0.21875,
                                         0.185350602957548,
                                         0.00454627659010643,
                                         2.7103330658178e-07,
                                         0.00250206882172024,
                                         0.109375,
                                         0.308924433958084,
                                         0.0461842175427366,
                                         3.81142069245895e-09,
                                         0.000214458347261367,
                                         0.03125,
                                         0.294220187112653,
                                         0.268097816847257,
                                         2.34492614456701e-11,
                                         8.04201197864117e-06,
                                         0.00390625,
                                         0.12259442822481,
                                         0.680880697126515,

                                         0.759733446005233,
                                         0.226545294324976,
                                         0.0216402591801953,
                                         0.000441139010905775,
                                         6.77970800207906e-07,
                                         1.68950322906245e-12,
                                         0.21239214717859,
                                         0.369617074802618,
                                         0.106418362209228,
                                         0.00574118215379526,
                                         2.65946318950958e-05,
                                         3.86778024269846e-10,
                                         0.0259772828008209,
                                         0.263831752925225,
                                         0.228954266579475,
                                         0.0326892716861027,
                                         0.000456409877638838,
                                         3.87384832414507e-08,
                                         0.00181556139885074,
                                         0.107612829750927,
                                         0.281476966926776,
                                         0.106358233224189,
                                         0.00447587590990954,
                                         2.21710052896043e-06,
                                         7.93063890253343e-05,
                                         0.0274334898060111,
                                         0.216280319029333,
                                         0.216280319029333,
                                         0.0274334898060111,
                                         7.93063890253343e-05,
                                         2.21710052896043e-06,
                                         0.00447587590990955,
                                         0.106358233224189,
                                         0.281476966926776,
                                         0.107612829750926,
                                         0.00181556139885074,
                                         3.87384832414507e-08,
                                         0.000456409877638839,
                                         0.0326892716861027,
                                         0.228954266579475,
                                         0.263831752925225,
                                         0.0259772828008209,
                                         3.86778024269846e-10,
                                         2.65946318950959e-05,
                                         0.00574118215379526,
                                         0.106418362209228,
                                         0.369617074802618,
                                         0.21239214717859,
                                         1.68950322906245e-12,
                                         6.77970800207907e-07,
                                         0.000441139010905775,
                                         0.0216402591801953,
                                         0.226545294324976,
                                         0.759733446005233,

                                         0.813667764445078,
                                         0.330529389064834,
                                         0.0596018765816454,
                                         0.00390625,
                                         6.06676537012991e-05,
                                         7.7808177043123e-08,
                                         1.75777048152068e-13,
                                         0.169961861751363,
                                         0.392443339620086,
                                         0.201517180883226,
                                         0.03125,
                                         0.00114837843365764,
                                         4.19409566696012e-06,
                                         5.38564560634911e-11,
                                         0.0155322333317329,
                                         0.203855250726603,
                                         0.29808648196301,
                                         0.109375,
                                         0.00951022767593871,
                                         9.89075585663159e-05,
                                         7.21923924066092e-09,
                                         0.00081110725088707,
                                         0.0605102284705957,
                                         0.251961645126635,
                                         0.21875,
                                         0.0450047912516688,
                                         0.00133285400897737,
                                         5.5297697433189e-07,
                                         2.64729706922393e-05,
                                         0.0112257586464937,
                                         0.133108750430518,
                                         0.2734375,
                                         0.133108750430518,
                                         0.0112257586464937,
                                         2.64729706922396e-05,
                                         5.52976974331884e-07,
                                         0.00133285400897737,
                                         0.0450047912516688,
                                         0.21875,
                                         0.251961645126635,
                                         0.0605102284705957,
                                         0.000811107250887075,
                                         7.21923924066082e-09,
                                         9.89075585663159e-05,
                                         0.00951022767593871,
                                         0.109375,
                                         0.29808648196301,
                                         0.203855250726603,
                                         0.0155322333317329,
                                         5.38564560634903e-11,
                                         4.19409566696012e-06,
                                         0.00114837843365764,
                                         0.03125,
                                         0.201517180883226,
                                         0.392443339620086,
                                         0.169961861751364,
                                         1.75777048152065e-13,
                                         7.7808177043123e-08,
                                         6.06676537012991e-05,
                                         0.00390625,
                                         0.0596018765816454,
                                         0.330529389064834,
                                         0.813667764445078,

                                         0.851770071021287,
                                         0.424130742155003,
                                         0.114585995970256,
                                         0.0150284523953017,
                                         0.000772125053069863,
                                         1.00325758238975e-05,
                                         1.14138434296086e-08,
                                         2.41530336638006e-14,
                                         0.13803636901654,
                                         0.384000030829074,
                                         0.285106188564433,
                                         0.0829565954028709,
                                         0.00895222436508324,
                                         0.00025805799838256,
                                         8.06826915998154e-07,
                                         9.53850935176882e-12,
                                         0.00978684555966569,
                                         0.152104065908508,
                                         0.310355754255881,
                                         0.200339062610788,
                                         0.0454100865323796,
                                         0.00290402436203193,
                                         2.49520885249798e-05,
                                         1.64803864144351e-09,
                                         0.000396509771575473,
                                         0.034428043167376,
                                         0.193052269270981,
                                         0.276466367315195,
                                         0.131624171971701,
                                         0.0186743346692449,
                                         0.000440956123948967,
                                         1.62710740732389e-07,
                                         1.00402625899951e-05,
                                         0.00487039148680477,
                                         0.0750533423329648,
                                         0.238450914353611,
                                         0.238450914353611,
                                         0.0750533423329648,
                                         0.00487039148680477,
                                         1.00402625899951e-05,
                                         1.62710740732389e-07,
                                         0.000440956123948967,
                                         0.0186743346692449,
                                         0.131624171971701,
                                         0.276466367315195,
                                         0.193052269270981,
                                         0.034428043167376,
                                         0.000396509771575473,
                                         1.64803864144351e-09,
                                         2.49520885249798e-05,
                                         0.00290402436203193,
                                         0.0454100865323796,
                                         0.200339062610788,
                                         0.310355754255881,
                                         0.152104065908508,
                                         0.00978684555966569,
                                         9.53850935176882e-12,
                                         8.06826915998154e-07,
                                         0.00025805799838256,
                                         0.00895222436508325,
                                         0.0829565954028709,
                                         0.285106188564433,
                                         0.384000030829074,
                                         0.13803636901654,
                                         2.41530336638006e-14,
                                         1.14138434296086e-08,
                                         1.00325758238975e-05,
                                         0.000772125053069864,
                                         0.0150284523953017,
                                         0.114585995970256,
                                         0.424130742155003,
                                         0.851770071021287,

                                         0.879515842541613,
                                         0.504429311917423,
                                         0.179322562850851,
                                         0.0369427200868735,
                                         0.00390625,
                                         0.00016983643624042,
                                         1.95034551386569e-06,
                                         2.0410410752885e-09,
                                         4.12589690786632e-15,
                                         0.11382639772371,
                                         0.36038916934264,
                                         0.343783082577237,
                                         0.150810509152786,
                                         0.03125,
                                         0.00266261335121172,
                                         6.51091409607065e-05,
                                         1.82835407104989e-07,
                                         2.0403234498739e-12,
                                         0.00644496219854924,
                                         0.112647408191901,
                                         0.288344799558302,
                                         0.269347159809281,
                                         0.109375,
                                         0.0182626333403897,
                                         0.000950934124412902,
                                         7.16550690331077e-06,
                                         4.41425814669216e-10,
                                         0.000208525757877873,
                                         0.0201202155127369,
                                         0.138197980306189,
                                         0.274887597706482,
                                         0.21875,
                                         0.0715781790362857,
                                         0.00793635070178253,
                                         0.000160470603414393,
                                         5.45730698779783e-08,
                                         4.21676170850248e-06,
                                         0.00224607404853326,
                                         0.0413972303947519,
                                         0.175338751080449,
                                         0.2734375,
                                         0.175338751080449,
                                         0.0413972303947519,
                                         0.00224607404853326,
                                         4.21676170850254e-06,
                                         5.45730698779773e-08,
                                         0.000160470603414393,
                                         0.00793635070178252,
                                         0.0715781790362857,
                                         0.21875,
                                         0.274887597706482,
                                         0.138197980306189,
                                         0.0201202155127369,
                                         0.000208525757877875,
                                         4.41425814669207e-10,
                                         7.16550690331077e-06,
                                         0.0009509341244129,
                                         0.0182626333403897,
                                         0.109375,
                                         0.269347159809281,
                                         0.288344799558302,
                                         0.112647408191901,
                                         0.00644496219854928,
                                         2.04032344987385e-12,
                                         1.82835407104989e-07,
                                         6.51091409607064e-05,
                                         0.00266261335121172,
                                         0.03125,
                                         0.150810509152786,
                                         0.343783082577237,
                                         0.36038916934264,
                                         0.113826397723711,
                                         4.12589690786621e-15,
                                         2.0410410752885e-09,
                                         1.95034551386569e-06,
                                         0.00016983643624042,
                                         0.00390625,
                                         0.0369427200868735,
                                         0.179322562850851,
                                         0.504429311917423,
                                         0.879515842541613,

                                         0.900269842293711,
                                         0.57188508386752,
                                         0.24717982498596,
                                         0.0696126347822499,
                                         0.0118560567266512,
                                         0.00107574543179176,
                                         4.14954844234534e-05,
                                         4.35877534033639e-07,
                                         4.29337671551831e-10,
                                         8.394888408492e-16,
                                         0.0952067997845387,
                                         0.33100536629966,
                                         0.377482602437625,
                                         0.220136549376214,
                                         0.0702669998007756,
                                         0.0116165814611536,
                                         0.000839801662513245,
                                         1.82667186260992e-05,
                                         4.74736589169372e-08,
                                         5.08042022590994e-13,
                                         0.00440495311069968,
                                         0.0838183982749969,
                                         0.252208034691503,
                                         0.30456100359866,
                                         0.18219674352879,
                                         0.0548814063014413,
                                         0.00743585099546659,
                                         0.000334915039658724,
                                         2.29659529696477e-06,
                                         1.34512425084828e-10,
                                         0.000116459940444679,
                                         0.0121284591984975,
                                         0.0962903680513115,
                                         0.240778878060299,
                                         0.269955239680595,
                                         0.148161058407073,
                                         0.0376224070821072,
                                         0.00350889774960427,
                                         6.34860326862705e-05,
                                         2.03510639982383e-08,
                                         1.92438452060843e-06,
                                         0.00109686182834678,
                                         0.0229766544481776,
                                         0.118971378958066,
                                         0.249990168661728,
                                         0.249990168661728,
                                         0.118971378958066,
                                         0.0229766544481776,
                                         0.00109686182834678,
                                         1.92438452060843e-06,
                                         2.03510639982383e-08,
                                         6.34860326862708e-05,
                                         0.00350889774960427,
                                         0.0376224070821072,
                                         0.148161058407073,
                                         0.269955239680595,
                                         0.240778878060299,
                                         0.0962903680513115,
                                         0.0121284591984975,
                                         0.000116459940444679,
                                         1.34512425084828e-10,
                                         2.29659529696478e-06,
                                         0.000334915039658724,
                                         0.00743585099546659,
                                         0.0548814063014413,
                                         0.18219674352879,
                                         0.304561003598661,
                                         0.252208034691503,
                                         0.0838183982749968,
                                         0.00440495311069968,
                                         5.08042022590994e-13,
                                         4.74736589169375e-08,
                                         1.82667186260992e-05,
                                         0.000839801662513245,
                                         0.0116165814611536,
                                         0.0702669998007756,
                                         0.220136549376214,
                                         0.377482602437625,
                                         0.33100536629966,
                                         0.0952067997845387,
                                         8.394888408492e-16,
                                         4.29337671551834e-10,
                                         4.35877534033639e-07,
                                         4.14954844234534e-05,
                                         0.00107574543179176,
                                         0.0118560567266512,
                                         0.0696126347822499,
                                         0.24717982498596,
                                         0.57188508386752,
                                         0.900269842293711,

                                         0.916161310284861,
                                         0.628131384149106,
                                         0.313641930864063,
                                         0.110775084917633,
                                         0.0263596587228344,
                                         0.00390625,
                                         0.000316604100475664,
                                         1.11744515402315e-05,
                                         1.098281306717e-07,
                                         1.03386140903071e-10,
                                         1.97170352066983e-16,
                                         0.0806623076556215,
                                         0.300740528856164,
                                         0.391344324609069,
                                         0.280546654553846,
                                         0.121332421229342,
                                         0.03125,
                                         0.00440209517844266,
                                         0.000282386017027088,
                                         5.63337477321096e-06,
                                         1.38197705583398e-08,
                                         1.43325310350592e-13,
                                         0.00310704393859348,
                                         0.062995783590566,
                                         0.213629890754853,
                                         0.31084662340766,
                                         0.244338555776599,
                                         0.109375,
                                         0.0267781382009658,
                                         0.00312203822866213,
                                         0.000126415915707075,
                                         8.08197305459863e-07,
                                         4.55808297882444e-11,
                                         6.83889756260379e-05,
                                         0.00754037521083249,
                                         0.0666387662863243,
                                         0.196810908204723,
                                         0.281170072076598,
                                         0.21875,
                                         0.0930814800606188,
                                         0.0197240092102977,
                                         0.00162105151503799,
                                         2.70082170341266e-05,
                                         8.28330236639516e-09,
                                         9.40816270950463e-07,
                                         0.000564097855835028,
                                         0.0129918768520413,
                                         0.0778811210086107,
                                         0.202220974654123,
                                         0.2734375,
                                         0.202220974654123,
                                         0.0778811210086106,
                                         0.0129918768520413,
                                         0.000564097855835028,
                                         9.40816270950482e-07,
                                         8.28330236639495e-09,
                                         2.70082170341266e-05,
                                         0.00162105151503799,
                                         0.0197240092102978,
                                         0.0930814800606187,
                                         0.21875,
                                         0.281170072076598,
                                         0.196810908204723,
                                         0.0666387662863242,
                                         0.00754037521083249,
                                         6.8388975626039e-05,
                                         4.5580829788243e-11,
                                         8.08197305459863e-07,
                                         0.000126415915707075,
                                         0.00312203822866214,
                                         0.0267781382009658,
                                         0.109375,
                                         0.244338555776599,
                                         0.31084662340766,
                                         0.213629890754853,
                                         0.062995783590566,
                                         0.00310704393859351,
                                         1.43325310350587e-13,
                                         1.38197705583398e-08,
                                         5.63337477321097e-06,
                                         0.000282386017027089,
                                         0.00440209517844266,
                                         0.03125,
                                         0.121332421229342,
                                         0.280546654553846,
                                         0.391344324609069,
                                         0.300740528856164,
                                         0.0806623076556219,
                                         1.97170352066975e-16,
                                         1.03386140903071e-10,
                                         1.098281306717e-07,
                                         1.11744515402315e-05,
                                         0.000316604100475664,
                                         0.00390625,
                                         0.0263596587228344,
                                         0.110775084917633,
                                         0.313641930864063,
                                         0.628131384149106,
                                         0.916161310284861,

                                         0.928579223511302,
                                         0.675008972996041,
                                         0.376144848450611,
                                         0.157424188646157,
                                         0.0478653060038374,
                                         0.0100392432855156,
                                         0.00133936066997764,
                                         9.96375074566032e-05,
                                         3.28612918475526e-06,
                                         3.0693937556711e-08,
                                         2.79051046593466e-11,
                                         5.22068951199069e-17,
                                         0.0691269765347617,
                                         0.27192322148806,
                                         0.391207607089348,
                                         0.327425950370293,
                                         0.176974656709406,
                                         0.0624367855669128,
                                         0.0137828160020664,
                                         0.00172469613357614,
                                         0.000101116719939689,
                                         1.88877578556652e-06,
                                         4.43329753082488e-09,
                                         4.48827273519201e-14,
                                         0.00225140807502781,
                                         0.0479248877975795,
                                         0.178007180499224,
                                         0.2979427261485,
                                         0.286272331315905,
                                         0.169886219056916,
                                         0.0620520926716624,
                                         0.0130611188771435,
                                         0.00136125463530851,
                                         5.08494505854318e-05,
                                         3.08140057744875e-07,
                                         1.68814081806632e-11,
                                         4.19008494999797e-05,
                                         0.00482656172018327,
                                         0.0462838740279641,
                                         0.154922475866084,
                                         0.264612053017961,
                                         0.264142155149632,
                                         0.159638213030955,
                                         0.0565210376157768,
                                         0.0104717127649491,
                                         0.000782265315403673,
                                         1.22385901596237e-05,
                                         3.62827380822475e-09,
                                         4.87384208657471e-07,
                                         0.000303804806715623,
                                         0.00752145569714038,
                                         0.0503472887195836,
                                         0.152869162818937,
                                         0.256683114566362,
                                         0.256683114566362,
                                         0.152869162818937,
                                         0.0503472887195837,
                                         0.0075214556971404,
                                         0.000303804806715624,
                                         4.87384208657459e-07,
                                         3.62827380822486e-09,
                                         1.22385901596236e-05,
                                         0.000782265315403671,
                                         0.0104717127649491,
                                         0.0565210376157768,
                                         0.159638213030955,
                                         0.264142155149632,
                                         0.264612053017962,
                                         0.154922475866084,
                                         0.0462838740279642,
                                         0.00482656172018328,
                                         4.1900849499979e-05,
                                         1.68814081806638e-11,
                                         3.08140057744873e-07,
                                         5.08494505854316e-05,
                                         0.00136125463530851,
                                         0.0130611188771435,
                                         0.0620520926716624,
                                         0.169886219056916,
                                         0.286272331315905,
                                         0.2979427261485,
                                         0.178007180499224,
                                         0.0479248877975796,
                                         0.00225140807502779,
                                         4.4882727351922e-14,
                                         4.43329753082484e-09,
                                         1.88877578556651e-06,
                                         0.000101116719939689,
                                         0.00172469613357614,
                                         0.0137828160020664,
                                         0.0624367855669128,
                                         0.176974656709406,
                                         0.327425950370293,
                                         0.391207607089348,
                                         0.271923221488061,
                                         0.0691269765347612,
                                         5.22068951199094e-17,
                                         2.79051046593464e-11,
                                         3.06939375567109e-08,
                                         3.28612918475525e-06,
                                         9.96375074566031e-05,
                                         0.00133936066997764,
                                         0.0100392432855156,
                                         0.0478653060038374,
                                         0.157424188646157,
                                         0.376144848450611,
                                         0.675008972996041,
                                         0.928579223511302,

                                         0.938456022163199,
                                         0.714202395567706,
                                         0.433495674279375,
                                         0.206768489861003,
                                         0.0756990873722987,
                                         0.0205255725691051,
                                         0.00390625,
                                         0.000480415742152006,
                                         3.34325756381077e-05,
                                         1.04576062724161e-06,
                                         9.38592389337705e-09,
                                         8.30319473562006e-12,
                                         1.53017440105916e-17,
                                         0.0598473311875837,
                                         0.245521359206014,
                                         0.381954198008051,
                                         0.360220010229547,
                                         0.230576758611325,
                                         0.102695286632628,
                                         0.03125,
                                         0.00614528421061833,
                                         0.000702465377219034,
                                         3.84174719023393e-05,
                                         6.81758376787761e-07,
                                         1.54581394374093e-09,
                                         1.53564221830562e-14,
                                         0.00166975867540801,
                                         0.0369262592266623,
                                         0.147236628619399,
                                         0.274555008054849,
                                         0.307268806171007,
                                         0.224793525939879,
                                         0.109375,
                                         0.0343909976051121,
                                         0.00645740564668597,
                                         0.000617452189968414,
                                         2.16651646870492e-05,
                                         1.25906245827965e-07,
                                         6.74244188565703e-12,
                                         2.6621013156306e-05,
                                         0.00317353505532462,
                                         0.0324326468166142,
                                         0.119578432715467,
                                         0.233982495062447,
                                         0.281176511664073,
                                         0.21875,
                                         0.109978939086908,
                                         0.0339197908542616,
                                         0.0056707413583063,
                                         0.000393419115626554,
                                         5.86002245527266e-06,
                                         1.69163371294806e-09,
                                         2.65262262110664e-07,
                                         0.000170463461475241,
                                         0.00446507685194582,
                                         0.0325504023583294,
                                         0.111359758329118,
                                         0.219813466549525,
                                         0.2734375,
                                         0.219813466549525,
                                         0.111359758329118,
                                         0.0325504023583294,
                                         0.00446507685194582,
                                         0.00017046346147524,
                                         2.65262262110657e-07,
                                         1.69163371294812e-09,
                                         5.8600224552727e-06,
                                         0.000393419115626553,
                                         0.00567074135830631,
                                         0.0339197908542616,
                                         0.109978939086908,
                                         0.21875,
                                         0.281176511664073,
                                         0.233982495062447,
                                         0.119578432715467,
                                         0.0324326468166143,
                                         0.00317353505532461,
                                         2.66210131563054e-05,
                                         6.74244188565731e-12,
                                         1.25906245827966e-07,
                                         2.16651646870491e-05,
                                         0.000617452189968415,
                                         0.00645740564668597,
                                         0.0343909976051121,
                                         0.109375,
                                         0.224793525939879,
                                         0.307268806171007,
                                         0.274555008054848,
                                         0.1472366286194,
                                         0.0369262592266621,
                                         0.00166975867540799,
                                         1.5356422183057e-14,
                                         1.54581394374094e-09,
                                         6.81758376787758e-07,
                                         3.84174719023394e-05,
                                         0.000702465377219034,
                                         0.00614528421061833,
                                         0.03125,
                                         0.102695286632628,
                                         0.230576758611325,
                                         0.360220010229547,
                                         0.381954198008051,
                                         0.245521359206014,
                                         0.0598473311875833,
                                         1.53017440105924e-17,
                                         8.30319473562015e-12,
                                         9.385923893377e-09,
                                         1.04576062724162e-06,
                                         3.34325756381077e-05,
                                         0.000480415742152006,
                                         0.00390625,
                                         0.0205255725691051,
                                         0.0756990873722987,
                                         0.206768489861003,
                                         0.433495674279375,
                                         0.714202395567706,
                                         0.938456022163199,

                                         0.94643426275367,
                                         0.747136348849164,
                                         0.485345746375922,
                                         0.25661608614036,
                                         0.108550355329033,
                                         0.035810829063663,
                                         0.008876676978496,
                                         0.00156482161674821,
                                         0.000180452787012712,
                                         1.19100161484532e-05,
                                         3.57159262200858e-07,
                                         3.10515653659882e-09,
                                         2.687639947298e-12,
                                         4.89359816106433e-18,
                                         0.0522844654806429,
                                         0.221812621945459,
                                         0.367193723071683,
                                         0.380470613711269,
                                         0.277815437054743,
                                         0.147876109069939,
                                         0.0571632886049885,
                                         0.0155517054219356,
                                         0.00279679045402743,
                                         0.000297829148409053,
                                         1.5417169569149e-05,
                                         2.62675429941451e-07,
                                         5.79381564027068e-10,
                                         5.66925589107604e-15,
                                         0.00126366788397565,
                                         0.0288104938367954,
                                         0.121539466824707,
                                         0.246795034979453,
                                         0.311071021960145,
                                         0.267152788412702,
                                         0.161050490828185,
                                         0.0676190808506004,
                                         0.0189641909985836,
                                         0.00325836780863157,
                                         0.0002911557949692,
                                         9.72150406855002e-06,
                                         5.46432237766702e-08,
                                         2.87343827977604e-12,
                                         1.74524006734735e-05,
                                         0.00213834167863958,
                                         0.0229880088868362,
                                         0.0914773697859371,
                                         0.199032816748895,
                                         0.27579307649806,
                                         0.259279900279373,
                                         0.168005096401316,
                                         0.0734802565690704,
                                         0.0203701845898472,
                                         0.00314201445765401,
                                         0.000205593520872943,
                                         2.94489562181094e-06,
                                         8.32222853159274e-10,
                                         1.50645935697317e-07,
                                         9.91935690279492e-05,
                                         0.0027174740353242,
                                         0.0211919508015269,
                                         0.0795920773441467,
                                         0.177945506146943,
                                         0.260888939018358,
                                         0.260888939018358,
                                         0.177945506146943,
                                         0.0795920773441467,
                                         0.0211919508015269,
                                         0.0027174740353242,
                                         9.91935690279486e-05,
                                         1.50645935697317e-07,
                                         8.32222853159274e-10,
                                         2.94489562181096e-06,
                                         0.000205593520872943,
                                         0.003142014457654,
                                         0.0203701845898472,
                                         0.0734802565690704,
                                         0.168005096401316,
                                         0.259279900279373,
                                         0.27579307649806,
                                         0.199032816748896,
                                         0.0914773697859372,
                                         0.0229880088868362,
                                         0.00213834167863957,
                                         1.74524006734735e-05,
                                         2.87343827977604e-12,
                                         5.46432237766707e-08,
                                         9.72150406855002e-06,
                                         0.000291155794969199,
                                         0.00325836780863156,
                                         0.0189641909985836,
                                         0.0676190808506003,
                                         0.161050490828185,
                                         0.267152788412702,
                                         0.311071021960146,
                                         0.246795034979453,
                                         0.121539466824707,
                                         0.0288104938367953,
                                         0.00126366788397565,
                                         5.66925589107604e-15,
                                         5.79381564027075e-10,
                                         2.62675429941451e-07,
                                         1.5417169569149e-05,
                                         0.000297829148409052,
                                         0.00279679045402743,
                                         0.0155517054219356,
                                         0.0571632886049885,
                                         0.147876109069939,
                                         0.277815437054743,
                                         0.380470613711269,
                                         0.367193723071683,
                                         0.221812621945459,
                                         0.0522844654806429,
                                         4.89359816106433e-18,
                                         2.68763994729803e-12,
                                         3.10515653659882e-09,
                                         3.57159262200857e-07,
                                         1.19100161484531e-05,
                                         0.000180452787012712,
                                         0.00156482161674821,
                                         0.008876676978496,
                                         0.035810829063663,
                                         0.108550355329033,
                                         0.25661608614036,
                                         0.485345746375922,
                                         0.747136348849164,
                                         0.94643426275367,

                                         0.952967301317703,
                                         0.774974377608438,
                                         0.531819353588785,
                                         0.3054216882714,
                                         0.144913208634263,
                                         0.0557492997051097,
                                         0.0169303345614497,
                                         0.00390625,
                                         0.000647575146034186,
                                         7.0904403443984e-05,
                                         4.48377417677146e-06,
                                         1.29948180410215e-07,
                                         1.10099038086552e-09,
                                         9.36212507421273e-13,
                                         1.68801220711213e-18,
                                         0.0460474074984276,
                                         0.200741986647579,
                                         0.349427071256235,
                                         0.390479871390557,
                                         0.316602929802121,
                                         0.193813322770219,
                                         0.0900707141726205,
                                         0.03125,
                                         0.00779026229015681,
                                         0.00130529588996632,
                                         0.000131345968913884,
                                         6.5050613760708e-06,
                                         1.07243526941048e-07,
                                         2.3131486298078e-10,
                                         2.2357764225946e-15,
                                         0.000973442775840815,
                                         0.0227492534408862,
                                         0.10044469765901,
                                         0.218411492764528,
                                         0.302621614313074,
                                         0.294785349293064,
                                         0.209642692879317,
                                         0.109375,
                                         0.041000773080902,
                                         0.0105129062364609,
                                         0.00168332441722738,
                                         0.00014246581003004,
                                         4.5702158196876e-06,
                                         2.50040694474896e-08,
                                         1.29556503410423e-12,
                                         1.17591957545558e-05,
                                         0.00147318752155576,
                                         0.0164990688517443,
                                         0.0698094592492785,
                                         0.16529012540668,
                                         0.256206483097903,
                                         0.278828912095263,
                                         0.21875,
                                         0.123308769011406,
                                         0.0483836427482773,
                                         0.0123276657041819,
                                         0.00178291999801676,
                                         0.000111292085721896,
                                         1.54447116769969e-06,
                                         4.28994788216568e-10,
                                         8.87819809660387e-08,
                                         5.96250740520746e-05,
                                         0.00169383799816597,
                                         0.0139454675066329,
                                         0.0564253019793627,
                                         0.139172795855556,
                                         0.231779966762851,
                                         0.2734375,
                                         0.231779966762851,
                                         0.139172795855556,
                                         0.0564253019793627,
                                         0.0139454675066329,
                                         0.00169383799816598,
                                         5.96250740520746e-05,
                                         8.8781980966042e-08,
                                         4.28994788216548e-10,
                                         1.54447116769969e-06,
                                         0.000111292085721895,
                                         0.00178291999801676,
                                         0.0123276657041819,
                                         0.0483836427482773,
                                         0.123308769011406,
                                         0.21875,
                                         0.278828912095263,
                                         0.256206483097903,
                                         0.16529012540668,
                                         0.0698094592492784,
                                         0.0164990688517443,
                                         0.00147318752155576,
                                         1.17591957545561e-05,
                                         1.29556503410416e-12,
                                         2.50040694474896e-08,
                                         4.57021581968758e-06,
                                         0.00014246581003004,
                                         0.00168332441722738,
                                         0.0105129062364609,
                                         0.041000773080902,
                                         0.109375,
                                         0.209642692879317,
                                         0.294785349293064,
                                         0.302621614313074,
                                         0.218411492764528,
                                         0.10044469765901,
                                         0.0227492534408862,
                                         0.000973442775840833,
                                         2.23577642259445e-15,
                                         2.3131486298078e-10,
                                         1.07243526941048e-07,
                                         6.50506137607082e-06,
                                         0.000131345968913884,
                                         0.00130529588996632,
                                         0.00779026229015681,
                                         0.03125,
                                         0.0900707141726205,
                                         0.193813322770219,
                                         0.316602929802121,
                                         0.390479871390557,
                                         0.349427071256236,
                                         0.200741986647579,
                                         0.0460474074984281,
                                         1.68801220711201e-18,
                                         9.36212507421273e-13,
                                         1.10099038086551e-09,
                                         1.29948180410216e-07,
                                         4.48377417677146e-06,
                                         7.0904403443984e-05,
                                         0.000647575146034186,
                                         0.00390625,
                                         0.0169303345614497,
                                         0.0557492997051097,
                                         0.144913208634263,
                                         0.3054216882714,
                                         0.531819353588785,
                                         0.774974377608438,
                                         0.952967301317703,

                                         0.958381841294348,
                                         0.798652140992959,
                                         0.573279494479795,
                                         0.352190398563234,
                                         0.183367523871216,
                                         0.0797737865498052,
                                         0.0284308373363627,
                                         0.00807444500085405,
                                         0.00175752148131173,
                                         0.000277123561316401,
                                         2.9083507141679e-05,
                                         1.77579297742401e-06,
                                         5.00436569593344e-08,
                                         4.15094980901703e-10,
                                         3.47859767713221e-13,
                                         6.22157701680893e-19,
                                         0.0408482825559487,
                                         0.182107765292965,
                                         0.330315557631623,
                                         0.392589734870213,
                                         0.346474396264484,
                                         0.23723213370546,
                                         0.12749410738381,
                                         0.0533858501168022,
                                         0.0170124606517993,
                                         0.0039550589713849,
                                         0.000625912236653902,
                                         6.01483311323599e-05,
                                         2.87321142448458e-06,
                                         4.61067844372625e-08,
                                         9.76367628220708e-11,
                                         9.34211545989131e-16,
                                         0.000761705486994343,
                                         0.0181667536578693,
                                         0.0832663845306687,
                                         0.191460163288022,
                                         0.286416431985675,
                                         0.308649004481535,
                                         0.250131640903244,
                                         0.154425032825364,
                                         0.0720463001286848,
                                         0.0246950854137472,
                                         0.00589329478601122,
                                         0.000891318430977544,
                                         7.21712434948037e-05,
                                         2.2405789160718e-06,
                                         1.19894869237268e-08,
                                         6.13716834987168e-13,
                                         8.11637898813371e-06,
                                         0.00103559069758117,
                                         0.0119942290379559,
                                         0.0533555819878182,
                                         0.135296532146772,
                                         0.229465921908779,
                                         0.280420076166905,
                                         0.255253198697407,
                                         0.174348487213393,
                                         0.0881109843663233,
                                         0.0317077072479928,
                                         0.00754751775859906,
                                         0.00103591171153335,
                                         6.22182234494792e-05,
                                         8.41297844549318e-07,
                                         2.30384254532216e-10,
                                         5.40527219868804e-08,
                                         3.68959733090542e-05,
                                         0.00107982899571223,
                                         0.00929311508060361,
                                         0.0399443554181672,
                                         0.10662315557662,
                                         0.196485085896906,
                                         0.263696703884384,
                                         0.263696703884384,
                                         0.196485085896906,
                                         0.10662315557662,
                                         0.0399443554181672,
                                         0.00929311508060359,
                                         0.00107982899571224,
                                         3.68959733090542e-05,
                                         5.40527219868781e-08,
                                         2.30384254532228e-10,
                                         8.41297844549318e-07,
                                         6.2218223449479e-05,
                                         0.00103591171153335,
                                         0.00754751775859906,
                                         0.0317077072479928,
                                         0.0881109843663234,
                                         0.174348487213393,
                                         0.255253198697407,
                                         0.280420076166905,
                                         0.229465921908779,
                                         0.135296532146772,
                                         0.0533555819878182,
                                         0.011994229037956,
                                         0.00103559069758117,
                                         8.11637898813346e-06,
                                         6.13716834987206e-13,
                                         1.19894869237268e-08,
                                         2.24057891607179e-06,
                                         7.21712434948039e-05,
                                         0.000891318430977544,
                                         0.00589329478601122,
                                         0.0246950854137472,
                                         0.0720463001286848,
                                         0.154425032825364,
                                         0.250131640903244,
                                         0.308649004481535,
                                         0.286416431985674,
                                         0.191460163288022,
                                         0.0832663845306688,
                                         0.0181667536578693,
                                         0.000761705486994326,
                                         9.34211545989199e-16,
                                         9.76367628220708e-11,
                                         4.61067844372622e-08,
                                         2.87321142448459e-06,
                                         6.01483311323599e-05,
                                         0.000625912236653902,
                                         0.0039550589713849,
                                         0.0170124606517993,
                                         0.0533858501168022,
                                         0.12749410738381,
                                         0.23723213370546,
                                         0.346474396264484,
                                         0.392589734870212,
                                         0.330315557631623,
                                         0.182107765292965,
                                         0.0408482825559482,
                                         6.22157701680946e-19,
                                         3.47859767713221e-13,
                                         4.15094980901701e-10,
                                         5.00436569593346e-08,
                                         1.77579297742401e-06,
                                         2.9083507141679e-05,
                                         0.000277123561316401,
                                         0.00175752148131173,
                                         0.00807444500085404,
                                         0.0284308373363627,
                                         0.0797737865498052,
                                         0.183367523871216,
                                         0.352190398563234,
                                         0.573279494479795,
                                         0.798652140992959,
                                         0.958381841294348,

                                         0.96291782758848,
                                         0.818917795111741,
                                         0.61018955814276,
                                         0.396345524976371,
                                         0.222713778885278,
                                         0.107092918827516,
                                         0.0434109010178495,
                                         0.0145328266905549,
                                         0.00390625,
                                         0.000810377691703877,
                                         0.000122596102984312,
                                         1.24222079130953e-05,
                                         7.36729313693712e-07,
                                         2.02837795961415e-08,
                                         1.65302295490691e-10,
                                         1.36854799750696e-13,
                                         2.43128396178896e-19,
                                         0.0364720371524205,
                                         0.16565622527126,
                                         0.3109256297542,
                                         0.388864512481369,
                                         0.367944386693893,
                                         0.275997721813975,
                                         0.166743349065387,
                                         0.0810461248531386,
                                         0.03125,
                                         0.00930005016774062,
                                         0.00204271096020328,
                                         0.00030848498196629,
                                         2.85399251238108e-05,
                                         1.32313605620432e-06,
                                         2.07619005947242e-08,
                                         4.3298470452541e-11,
                                         4.10813649648833e-16,
                                         0.000604378314506097,
                                         0.0146606515287996,
                                         0.0693148569198388,
                                         0.166917057965824,
                                         0.265947145996121,
                                         0.31119190872014,
                                         0.280205269067622,
                                         0.197739062808478,
                                         0.109375,
                                         0.0466940090139075,
                                         0.0148907039852441,
                                         0.00335156244642387,
                                         0.000483699641826617,
                                         3.77605388597257e-05,
                                         1.14086271653831e-06,
                                         5.99326020562179e-09,
                                         3.03691126204179e-13,
                                         5.72294636915894e-06,
                                         0.000741413914429252,
                                         0.00882994687942566,
                                         0.040941625602165,
                                         0.109842507293789,
                                         0.20049938593987,
                                         0.269070463262743,
                                         0.275685954184228,
                                         0.21875,
                                         0.133967355841576,
                                         0.0620276735870083,
                                         0.0208076271158292,
                                         0.00468447206577589,
                                         0.000615791675192914,
                                         3.58229724558322e-05,
                                         4.74040735875223e-07,
                                         1.28286598647761e-10,
                                         3.38696334296785e-08,
                                         2.34340963389945e-05,
                                         0.000703023541400072,
                                         0.00627638334038282,
                                         0.0283547327688799,
                                         0.0807379679463675,
                                         0.161486332950958,
                                         0.240224238748673,
                                         0.2734375,
                                         0.240224238748673,
                                         0.161486332950958,
                                         0.0807379679463675,
                                         0.0283547327688799,
                                         0.0062763833403828,
                                         0.000703023541400072,
                                         2.34340963389943e-05,
                                         3.38696334296801e-08,
                                         1.28286598647753e-10,
                                         4.74040735875228e-07,
                                         3.58229724558322e-05,
                                         0.000615791675192916,
                                         0.0046844720657759,
                                         0.0208076271158292,
                                         0.0620276735870083,
                                         0.133967355841576,
                                         0.21875,
                                         0.275685954184228,
                                         0.269070463262743,
                                         0.20049938593987,
                                         0.109842507293789,
                                         0.040941625602165,
                                         0.00882994687942566,
                                         0.000741413914429247,
                                         5.72294636915914e-06,
                                         3.03691126204158e-13,
                                         5.99326020562187e-09,
                                         1.14086271653831e-06,
                                         3.77605388597258e-05,
                                         0.000483699641826618,
                                         0.00335156244642388,
                                         0.0148907039852441,
                                         0.0466940090139074,
                                         0.109375,
                                         0.197739062808478,
                                         0.280205269067622,
                                         0.31119190872014,
                                         0.265947145996121,
                                         0.166917057965823,
                                         0.0693148569198388,
                                         0.0146606515287995,
                                         0.000604378314506111,
                                         4.10813649648799e-16,
                                         4.32984704525417e-11,
                                         2.07619005947242e-08,
                                         1.32313605620432e-06,
                                         2.85399251238109e-05,
                                         0.000308484981966291,
                                         0.00204271096020328,
                                         0.00930005016774062,
                                         0.03125,
                                         0.0810461248531386,
                                         0.166743349065387,
                                         0.275997721813975,
                                         0.367944386693892,
                                         0.388864512481369,
                                         0.3109256297542,
                                         0.16565622527126,
                                         0.0364720371524209,
                                         2.43128396178873e-19,
                                         1.36854799750699e-13,
                                         1.65302295490691e-10,
                                         2.02837795961416e-08,
                                         7.36729313693714e-07,
                                         1.24222079130954e-05,
                                         0.000122596102984312,
                                         0.000810377691703877,
                                         0.00390625,
                                         0.0145328266905549,
                                         0.0434109010178495,
                                         0.107092918827516,
                                         0.222713778885278,
                                         0.396345524976371,
                                         0.61018955814276,
                                         0.818917795111741,
                                         0.96291782758848,

                                         0.966754519686593,
                                         0.83636935887209,
                                         0.64303594268099,
                                         0.437606766916381,
                                         0.262011418604361,
                                         0.136850500531849,
                                         0.0616346968646651,
                                         0.0235656228342258,
                                         0.00748992413327123,
                                         0.00192301637543174,
                                         0.000383266837562394,
                                         5.60078190550855e-05,
                                         5.51049425149786e-06,
                                         3.18929172733807e-07,
                                         8.61062469682906e-09,
                                         6.91376309843516e-11,
                                         5.66572009504783e-14,
                                         1.00085544730261e-19,
                                         0.0327557909975925,
                                         0.151128101207955,
                                         0.291917506903224,
                                         0.38099374440789,
                                         0.38201635723697,
                                         0.308997095809359,
                                         0.205455691251766,
                                         0.112660506558125,
                                         0.0505540089679615,
                                         0.0182341185475834,
                                         0.00513083962558723,
                                         0.00107531504930661,
                                         0.000156193343012776,
                                         1.3999498549089e-05,
                                         6.32966635702135e-07,
                                         9.74696879241116e-09,
                                         2.00672553493609e-11,
                                         1.89051205472177e-16,
                                         0.000485554550961691,
                                         0.011947317229455,
                                         0.0579779815037575,
                                         0.145120818187063,
                                         0.243681049717623,
                                         0.305239309471316,
                                         0.299632656710999,
                                         0.235636484102641,
                                         0.149283511094421,
                                         0.0756422820999364,
                                         0.0300506378298502,
                                         0.00903235178075781,
                                         0.00193692383813992,
                                         0.000268849213765453,
                                         2.0356590201996e-05,
                                         6.01177494634018e-07,
                                         3.10955967810144e-09,
                                         1.56230420628952e-13,
                                         4.11291673981186e-06,
                                         0.000539706335563146,
                                         0.0065800318040132,
                                         0.0315866481580201,
                                         0.088822604980982,
                                         0.172301269975151,
                                         0.249702025554433,
                                         0.281627668503879,
                                         0.251901081518033,
                                         0.179310789632554,
                                         0.100572883067704,
                                         0.043353874353363,
                                         0.0137253844951375,
                                         0.00295030566330049,
                                         0.000374103008439953,
                                         2.11883824896458e-05,
                                         2.75341558697365e-07,
                                         7.37757621016341e-11,
                                         2.17741807732012e-08,
                                         1.52378836945538e-05,
                                         0.000466737731924782,
                                         0.00429692115474295,
                                         0.0202350961552761,
                                         0.0607878120417839,
                                         0.130057380615653,
                                         0.210372090640425,
                                         0.265661267630807,
                                         0.265661267630807,
                                         0.210372090640425,
                                         0.130057380615653,
                                         0.0607878120417839,
                                         0.0202350961552761,
                                         0.00429692115474296,
                                         0.000466737731924782,
                                         1.52378836945538e-05,
                                         2.17741807732012e-08,
                                         7.37757621016341e-11,
                                         2.75341558697365e-07,
                                         2.11883824896458e-05,
                                         0.000374103008439952,
                                         0.00295030566330049,
                                         0.0137253844951375,
                                         0.043353874353363,
                                         0.100572883067704,
                                         0.179310789632554,
                                         0.251901081518033,
                                         0.281627668503879,
                                         0.249702025554433,
                                         0.172301269975151,
                                         0.088822604980982,
                                         0.0315866481580202,
                                         0.0065800318040132,
                                         0.000539706335563146,
                                         4.11291673981186e-06,
                                         1.56230420628952e-13,
                                         3.10955967810144e-09,
                                         6.01177494634018e-07,
                                         2.03565902019959e-05,
                                         0.000268849213765453,
                                         0.00193692383813992,
                                         0.00903235178075781,
                                         0.0300506378298502,
                                         0.0756422820999364,
                                         0.149283511094421,
                                         0.235636484102641,
                                         0.299632656710999,
                                         0.305239309471316,
                                         0.243681049717623,
                                         0.145120818187063,
                                         0.0579779815037575,
                                         0.011947317229455,
                                         0.000485554550961691,
                                         1.89051205472177e-16,
                                         2.00672553493609e-11,
                                         9.74696879241116e-09,
                                         6.32966635702133e-07,
                                         1.3999498549089e-05,
                                         0.000156193343012776,
                                         0.00107531504930661,
                                         0.00513083962558723,
                                         0.0182341185475834,
                                         0.0505540089679615,
                                         0.112660506558125,
                                         0.205455691251766,
                                         0.308997095809359,
                                         0.38201635723697,
                                         0.38099374440789,
                                         0.291917506903224,
                                         0.151128101207955,
                                         0.0327557909975925,
                                         1.00085544730261e-19,
                                         5.66572009504783e-14,
                                         6.91376309843516e-11,
                                         8.61062469682902e-09,
                                         3.18929172733807e-07,
                                         5.51049425149786e-06,
                                         5.60078190550855e-05,
                                         0.000383266837562394,
                                         0.00192301637543174,
                                         0.00748992413327123,
                                         0.0235656228342258,
                                         0.0616346968646651,
                                         0.136850500531849,
                                         0.262011418604361,
                                         0.437606766916381,
                                         0.64303594268099,
                                         0.83636935887209,
                                         0.966754519686593,

                                         0.970027917478441,
                                         0.851486100039221,
                                         0.672287391886562,
                                         0.475893223714319,
                                         0.300565213623944,
                                         0.168230140729627,
                                         0.0826880717444693,
                                         0.0352611200560505,
                                         0.0128380178253708,
                                         0.00390625,
                                         0.000964962860604875,
                                         0.000185926947306552,
                                         2.63861988767406e-05,
                                         2.53221325473761e-06,
                                         1.43553843652434e-07,
                                         3.81191053588978e-09,
                                         3.0223534028936e-11,
                                         2.45534709954659e-14,
                                         4.31656898564682e-20,
                                         0.0295745760009045,
                                         0.138280030373987,
                                         0.27368042672591,
                                         0.37029995151573,
                                         0.389864073435329,
                                         0.3358871225508,
                                         0.241838030491297,
                                         0.146433981132939,
                                         0.0743172343460537,
                                         0.03125,
                                         0.0106683930438692,
                                         0.00286534253206903,
                                         0.000577397730565732,
                                         8.11690954207405e-05,
                                         7.08304985232172e-06,
                                         3.13529593251699e-07,
                                         4.75156251024292e-09,
                                         9.67633655448239e-12,
                                         9.06117183607098e-17,
                                         0.000394485348637467,
                                         0.00982470292200377,
                                         0.048742810565251,
                                         0.126059577392731,
                                         0.221241082230723,
                                         0.293400572514468,
                                         0.30944565393946,
                                         0.26605148881845,
                                         0.188217136456613,
                                         0.109375,
                                         0.0516018739875343,
                                         0.0193191854432446,
                                         0.0055277879018754,
                                         0.00113830643691007,
                                         0.000152898712748682,
                                         1.12821647182135e-05,
                                         3.26817803367058e-07,
                                         1.66834972113087e-09,
                                         8.32162429250392e-14,
                                         3.00680442137414e-06,
                                         0.000398879153284916,
                                         0.00496065869905865,
                                         0.0245222316007613,
                                         0.0717431239711161,
                                         0.146450382834528,
                                         0.226259138452506,
                                         0.276217677090962,
                                         0.272389733918781,
                                         0.21875,
                                         0.142624420061655,
                                         0.0744325722244388,
                                         0.0302405454883942,
                                         0.0091219907745458,
                                         0.00188603310298123,
                                         0.000231989476261046,
                                         1.28450830789023e-05,
                                         1.64370990513115e-07,
                                         4.36710660250815e-11,
                                         1.43238412713263e-08,
                                         1.01214624622997e-05,
                                         0.00031553544054979,
                                         0.00298142679397504,
                                         0.0145403483194615,
                                         0.0456877828504458,
                                         0.103396988052556,
                                         0.179232705754539,
                                         0.246378227499518,
                                         0.2734375,
                                         0.246378227499518,
                                         0.17923270575454,
                                         0.103396988052556,
                                         0.0456877828504457,
                                         0.0145403483194615,
                                         0.00298142679397504,
                                         0.00031553544054979,
                                         1.01214624622997e-05,
                                         1.43238412713263e-08,
                                         4.36710660250815e-11,
                                         1.64370990513115e-07,
                                         1.28450830789023e-05,
                                         0.000231989476261046,
                                         0.00188603310298122,
                                         0.00912199077454581,
                                         0.0302405454883942,
                                         0.0744325722244387,
                                         0.142624420061655,
                                         0.21875,
                                         0.272389733918781,
                                         0.276217677090962,
                                         0.226259138452506,
                                         0.146450382834528,
                                         0.0717431239711162,
                                         0.0245222316007613,
                                         0.00496065869905865,
                                         0.000398879153284916,
                                         3.00680442137414e-06,
                                         8.32162429250392e-14,
                                         1.66834972113087e-09,
                                         3.26817803367058e-07,
                                         1.12821647182135e-05,
                                         0.000152898712748681,
                                         0.00113830643691007,
                                         0.0055277879018754,
                                         0.0193191854432445,
                                         0.0516018739875343,
                                         0.109375,
                                         0.188217136456613,
                                         0.26605148881845,
                                         0.30944565393946,
                                         0.293400572514468,
                                         0.221241082230723,
                                         0.126059577392731,
                                         0.048742810565251,
                                         0.00982470292200377,
                                         0.000394485348637467,
                                         9.06117183607098e-17,
                                         9.67633655448239e-12,
                                         4.75156251024292e-09,
                                         3.13529593251699e-07,
                                         7.0830498523217e-06,
                                         8.11690954207407e-05,
                                         0.000577397730565732,
                                         0.00286534253206902,
                                         0.0106683930438692,
                                         0.03125,
                                         0.0743172343460537,
                                         0.146433981132939,
                                         0.241838030491297,
                                         0.3358871225508,
                                         0.38986407343533,
                                         0.37029995151573,
                                         0.27368042672591,
                                         0.138280030373987,
                                         0.0295745760009045,
                                         4.31656898564682e-20,
                                         2.45534709954659e-14,
                                         3.0223534028936e-11,
                                         3.81191053588978e-09,
                                         1.43553843652434e-07,
                                         2.53221325473762e-06,
                                         2.63861988767406e-05,
                                         0.000185926947306552,
                                         0.000964962860604875,
                                         0.00390625,
                                         0.0128380178253708,
                                         0.0352611200560505,
                                         0.0826880717444693,
                                         0.168230140729627,
                                         0.300565213623944,
                                         0.475893223714319,
                                         0.672287391886562,
                                         0.851486100039221,
                                         0.970027917478441,

                                         0.972842648423601,
                                         0.864653772647492,
                                         0.698375666930653,
                                         0.511251802481283,
                                         0.337889227344956,
                                         0.20051202762073,
                                         0.106064469359798,
                                         0.0495348235353043,
                                         0.0201716316598053,
                                         0.00704624764516319,
                                         0.00206611958610135,
                                         0.000493926209843846,
                                         9.24667410561284e-05,
                                         1.2798990988485e-05,
                                         1.20242691010243e-06,
                                         6.69710016143271e-08,
                                         1.75324210192132e-09,
                                         1.3751609324081e-11,
                                         1.10888952301006e-14,
                                         1.94143248056743e-20,
                                         0.0268313515540759,
                                         0.126893092788945,
                                         0.256426048178961,
                                         0.357787957611164,
                                         0.39264799920875,
                                         0.356837261673332,
                                         0.274701250698921,
                                         0.180669554464756,
                                         0.101495350243867,
                                         0.0483556898829199,
                                         0.0192684041820155,
                                         0.00628256410842356,
                                         0.00162252415875406,
                                         0.000316275094420418,
                                         4.32423105914018e-05,
                                         3.6883975527122e-06,
                                         1.60335982825667e-07,
                                         2.39695819391885e-09,
                                         4.83584655875148e-12,
                                         4.50507802369755e-17,
                                         0.000323758034745832,
                                         0.00814726386361046,
                                         0.0411920339839869,
                                         0.109545525552668,
                                         0.199622959175587,
                                         0.277829287164886,
                                         0.31126436775742,
                                         0.288295181886796,
                                         0.223423742015515,
                                         0.145182851503305,
                                         0.0786165711175802,
                                         0.0349614827236179,
                                         0.0124558924577257,
                                         0.00341926146797662,
                                         0.000680357464298382,
                                         8.88723601003313e-05,
                                         6.41502218673564e-06,
                                         1.82787061208428e-07,
                                         9.22645359285686e-10,
                                         4.57362338824729e-14,
                                         2.23234087824953e-06,
                                         0.000298914878453499,
                                         0.00378117071911674,
                                         0.0191657367576546,
                                         0.0579935295158162,
                                         0.123608348113779,
                                         0.2015394779163,
                                         0.26287648944939,
                                         0.281044083729802,
                                         0.249083529885262,
                                         0.183292375461683,
                                         0.111174377953193,
                                         0.0546412315408488,
                                         0.0211232909806198,
                                         0.00611684347239514,
                                         0.00122365123559573,
                                         0.000146665267453873,
                                         7.96512127745399e-06,
                                         1.00590980730396e-07,
                                         2.65326381696259e-11,
                                         9.62011993118581e-09,
                                         6.85430302558e-06,
                                         0.000216929868234106,
                                         0.00209573521836309,
                                         0.0105300057906402,
                                         0.0343714297530784,
                                         0.0815588077335555,
                                         0.149811835765369,
                                         0.220952841355931,
                                         0.26708821073597,
                                         0.26708821073597,
                                         0.220952841355931,
                                         0.149811835765369,
                                         0.0815588077335555,
                                         0.0343714297530784,
                                         0.0105300057906402,
                                         0.00209573521836309,
                                         0.000216929868234106,
                                         6.85430302558008e-06,
                                         9.62011993118643e-09,
                                         2.65326381696238e-11,
                                         1.00590980730394e-07,
                                         7.96512127745399e-06,
                                         0.000146665267453872,
                                         0.00122365123559573,
                                         0.00611684347239515,
                                         0.0211232909806198,
                                         0.0546412315408488,
                                         0.111174377953193,
                                         0.183292375461683,
                                         0.249083529885262,
                                         0.281044083729802,
                                         0.26287648944939,
                                         0.2015394779163,
                                         0.123608348113779,
                                         0.0579935295158162,
                                         0.0191657367576547,
                                         0.00378117071911674,
                                         0.000298914878453502,
                                         2.23234087824964e-06,
                                         4.57362338824684e-14,
                                         9.22645359285669e-10,
                                         1.82787061208428e-07,
                                         6.41502218673562e-06,
                                         8.88723601003313e-05,
                                         0.000680357464298383,
                                         0.00341926146797662,
                                         0.0124558924577257,
                                         0.0349614827236179,
                                         0.0786165711175802,
                                         0.145182851503305,
                                         0.223423742015515,
                                         0.288295181886796,
                                         0.31126436775742,
                                         0.277829287164885,
                                         0.199622959175587,
                                         0.109545525552668,
                                         0.0411920339839869,
                                         0.00814726386361051,
                                         0.000323758034745843,
                                         4.50507802369704e-17,
                                         4.83584655875137e-12,
                                         2.39695819391885e-09,
                                         1.60335982825666e-07,
                                         3.6883975527122e-06,
                                         4.32423105914019e-05,
                                         0.000316275094420418,
                                         0.00162252415875406,
                                         0.00628256410842356,
                                         0.0192684041820155,
                                         0.0483556898829199,
                                         0.101495350243867,
                                         0.180669554464756,
                                         0.274701250698921,
                                         0.356837261673332,
                                         0.39264799920875,
                                         0.357787957611165,
                                         0.256426048178961,
                                         0.126893092788945,
                                         0.0268313515540764,
                                         1.94143248056718e-20,
                                         1.10888952301003e-14,
                                         1.3751609324081e-11,
                                         1.75324210192131e-09,
                                         6.69710016143271e-08,
                                         1.20242691010243e-06,
                                         1.2798990988485e-05,
                                         9.24667410561284e-05,
                                         0.000493926209843846,
                                         0.00206611958610135,
                                         0.00704624764516319,
                                         0.0201716316598053,
                                         0.0495348235353043,
                                         0.106064469359798,
                                         0.20051202762073,
                                         0.337889227344956,
                                         0.511251802481283,
                                         0.698375666930653,
                                         0.864653772647492,
                                         0.972842648423601,

                                         0.975280229530664,
                                         0.876184489616517,
                                         0.72168821064246,
                                         0.543806791725869,
                                         0.373665153738048,
                                         0.233095621910536,
                                         0.131232190890892,
                                         0.0661720475422446,
                                         0.0295899642242764,
                                         0.0115853293479172,
                                         0.00390625,
                                         0.00110969867121686,
                                         0.0002579264984758,
                                         4.71059389740312e-05,
                                         6.38173720241994e-06,
                                         5.88661167017507e-07,
                                         3.22900643016741e-08,
                                         8.35025872697622e-10,
                                         6.48878258416976e-12,
                                         5.19884336129344e-15,
                                         9.06970836537649e-21,
                                         0.0244499173416097,
                                         0.11677488395727,
                                         0.240251605906224,
                                         0.344204486952726,
                                         0.391422580057019,
                                         0.372319268198386,
                                         0.303384007256306,
                                         0.213951185178198,
                                         0.130851398121675,
                                         0.0691290289809489,
                                         0.03125,
                                         0.011902356874394,
                                         0.00373286263811167,
                                         0.000932428448880945,
                                         0.000176671536478127,
                                         2.35865252287576e-05,
                                         1.97281157807321e-06,
                                         8.44321806233103e-08,
                                         1.24745965374191e-09,
                                         2.49650720099869e-12,
                                         2.31539623016907e-17,
                                         0.000268165822971631,
                                         0.00680896944321159,
                                         0.0349913779442189,
                                         0.0953161484105267,
                                         0.179385447525785,
                                         0.260180632724115,
                                         0.306847821901761,
                                         0.302644563844657,
                                         0.253157239867596,
                                         0.180464002847683,
                                         0.109375,
                                         0.0558520254123703,
                                         0.023635571786312,
                                         0.00807483066165601,
                                         0.00213979571393139,
                                         0.000413466759908156,
                                         5.27327431193858e-05,
                                         3.73503036741543e-06,
                                         1.04922311819442e-07,
                                         5.24489708582191e-10,
                                         2.58604083553203e-14,
                                         1.68070468542575e-06,
                                         0.00022686906297201,
                                         0.00291217683065876,
                                         0.0150826757136631,
                                         0.0469775639986817,
                                         0.103895412085877,
                                         0.177343533614827,
                                         0.24463185562214,
                                         0.27987511686582,
                                         0.269204718058324,
                                         0.21875,
                                         0.149763795311836,
                                         0.0855169431726772,
                                         0.0399588777594208,
                                         0.0148094862158464,
                                         0.00414170524158949,
                                         0.000805447189556164,
                                         9.44152657199656e-05,
                                         5.0427930460868e-06,
                                         6.29655599971458e-08,
                                         1.65046905601512e-11,
                                         6.58353898417659e-09,
                                         4.72442747789019e-06,
                                         0.000151479707132162,
                                         0.00149166163392204,
                                         0.00768906964614919,
                                         0.0259297178931932,
                                         0.064060111132756,
                                         0.123587105003828,
                                         0.193382976825055,
                                         0.250989044495309,
                                         0.2734375,
                                         0.250989044495309,
                                         0.193382976825055,
                                         0.123587105003828,
                                         0.064060111132756,
                                         0.0259297178931932,
                                         0.00768906964614919,
                                         0.00149166163392204,
                                         0.000151479707132161,
                                         4.72442747789025e-06,
                                         6.58353898417612e-09,
                                         1.65046905601527e-11,
                                         6.29655599971447e-08,
                                         5.04279304608684e-06,
                                         9.44152657199656e-05,
                                         0.000805447189556164,
                                         0.00414170524158949,
                                         0.0148094862158464,
                                         0.0399588777594208,
                                         0.0855169431726772,
                                         0.149763795311836,
                                         0.21875,
                                         0.269204718058324,
                                         0.27987511686582,
                                         0.24463185562214,
                                         0.177343533614827,
                                         0.103895412085877,
                                         0.0469775639986817,
                                         0.0150826757136631,
                                         0.00291217683065875,
                                         0.000226869062972012,
                                         1.68070468542566e-06,
                                         2.58604083553231e-14,
                                         5.2448970858218e-10,
                                         1.04922311819443e-07,
                                         3.73503036741543e-06,
                                         5.27327431193858e-05,
                                         0.000413466759908156,
                                         0.00213979571393139,
                                         0.008074830661656,
                                         0.023635571786312,
                                         0.0558520254123703,
                                         0.109375,
                                         0.180464002847683,
                                         0.253157239867596,
                                         0.302644563844657,
                                         0.306847821901761,
                                         0.260180632724115,
                                         0.179385447525785,
                                         0.0953161484105267,
                                         0.0349913779442188,
                                         0.00680896944321164,
                                         0.000268165822971622,
                                         2.31539623016936e-17,
                                         2.49650720099863e-12,
                                         1.24745965374192e-09,
                                         8.44321806233103e-08,
                                         1.97281157807321e-06,
                                         2.35865252287576e-05,
                                         0.000176671536478127,
                                         0.000932428448880943,
                                         0.00373286263811167,
                                         0.011902356874394,
                                         0.03125,
                                         0.0691290289809489,
                                         0.130851398121675,
                                         0.213951185178198,
                                         0.303384007256306,
                                         0.372319268198386,
                                         0.391422580057019,
                                         0.344204486952726,
                                         0.240251605906224,
                                         0.11677488395727,
                                         0.0244499173416093,
                                         9.06970836537778e-21,
                                         5.1988433612933e-15,
                                         6.48878258416983e-12,
                                         8.35025872697622e-10,
                                         3.22900643016741e-08,
                                         5.88661167017507e-07,
                                         6.38173720241994e-06,
                                         4.71059389740312e-05,
                                         0.0002579264984758,
                                         0.00110969867121686,
                                         0.00390625,
                                         0.0115853293479172,
                                         0.0295899642242764,
                                         0.0661720475422446,
                                         0.131232190890892,
                                         0.233095621910536,
                                         0.373665153738048,
                                         0.543806791725869,
                                         0.72168821064246,
                                         0.876184489616517,
                                         0.975280229530664,

                                         0.977404908789797,
                                         0.886332222442529,
                                         0.742567285184163,
                                         0.573725516807242,
                                         0.407703235783672,
                                         0.265501210051669,
                                         0.157680204965298,
                                         0.08487423977963,
                                         0.041078445597656,
                                         0.0176967720100195,
                                         0.00669859247786867,
                                         0.00219077374061691,
                                         0.000605583056299944,
                                         0.00013737447448127,
                                         2.45576700274951e-05,
                                         3.26564432798095e-06,
                                         2.96479452575759e-07,
                                         1.60489969900202e-08,
                                         4.1063363222043e-10,
                                         3.16518929060632e-12,
                                         2.52183478843118e-15,
                                         4.38588921605081e-21,
                                         0.0223698151588167,
                                         0.107758499889738,
                                         0.225182057681531,
                                         0.330094240400146,
                                         0.387099056198032,
                                         0.382954787904813,
                                         0.327635631095374,
                                         0.245212323278107,
                                         0.161153778953633,
                                         0.0928472404213051,
                                         0.0466015423504767,
                                         0.020153977413697,
                                         0.00738718108193503,
                                         0.00224109117663931,
                                         0.000544002303695965,
                                         0.000100585390473339,
                                         1.31550824735183e-05,
                                         1.08180731920008e-06,
                                         4.56773299801441e-08,
                                         6.68008040873262e-10,
                                         1.32752163250985e-12,
                                         1.22644972968921e-17,
                                         0.000223989846747249,
                                         0.00573171506908331,
                                         0.0298751305768698,
                                         0.08309019627779,
                                         0.160797067435223,
                                         0.241660995362026,
                                         0.297840075862812,
                                         0.309946505489309,
                                         0.276595457563439,
                                         0.213118917756569,
                                         0.141838810693517,
                                         0.0811151668252038,
                                         0.0394241040088532,
                                         0.0159952511951498,
                                         0.00527220605279903,
                                         0.00135543590949833,
                                         0.000255370429904819,
                                         3.19028875146301e-05,
                                         2.22292576837764e-06,
                                         6.16796286080289e-08,
                                         3.05734634413383e-10,
                                         1.5004443878863e-14,
                                         1.28161098438509e-06,
                                         0.000174212615202499,
                                         0.00226489353592231,
                                         0.0119515318830725,
                                         0.0381677085073549,
                                         0.0871420088873798,
                                         0.15471666408964,
                                         0.22386858162235,
                                         0.271276107087539,
                                         0.279535490767317,
                                         0.24669015786949,
                                         0.186554322086574,
                                         0.120228345342853,
                                         0.0652355840794623,
                                         0.0291975199099846,
                                         0.0104372243672023,
                                         0.00283275876077558,
                                         0.000537615796775412,
                                         6.18174607949826e-05,
                                         3.25434627168436e-06,
                                         4.02355204683874e-08,
                                         1.04894328338384e-11,
                                         4.58315058458304e-09,
                                         3.30944086214713e-06,
                                         0.000107316324439647,
                                         0.00107442815722323,
                                         0.00566231553511256,
                                         0.0196394170415053,
                                         0.0502309126753737,
                                         0.101060063894097,
                                         0.166286909872,
                                         0.229156365554849,
                                         0.268156656542556,
                                         0.268156656542556,
                                         0.229156365554849,
                                         0.166286909872,
                                         0.101060063894097,
                                         0.0502309126753737,
                                         0.0196394170415053,
                                         0.00566231553511255,
                                         0.00107442815722322,
                                         0.000107316324439647,
                                         3.30944086214713e-06,
                                         4.58315058458304e-09,
                                         1.04894328338384e-11,
                                         4.02355204683874e-08,
                                         3.25434627168434e-06,
                                         6.18174607949829e-05,
                                         0.000537615796775413,
                                         0.00283275876077558,
                                         0.0104372243672023,
                                         0.0291975199099846,
                                         0.0652355840794623,
                                         0.120228345342853,
                                         0.186554322086574,
                                         0.24669015786949,
                                         0.279535490767317,
                                         0.271276107087539,
                                         0.22386858162235,
                                         0.15471666408964,
                                         0.0871420088873798,
                                         0.0381677085073548,
                                         0.0119515318830725,
                                         0.00226489353592232,
                                         0.000174212615202499,
                                         1.28161098438509e-06,
                                         1.5004443878863e-14,
                                         3.05734634413383e-10,
                                         6.16796286080284e-08,
                                         2.22292576837765e-06,
                                         3.19028875146302e-05,
                                         0.000255370429904819,
                                         0.00135543590949833,
                                         0.00527220605279903,
                                         0.0159952511951498,
                                         0.0394241040088532,
                                         0.0811151668252038,
                                         0.141838810693517,
                                         0.213118917756569,
                                         0.276595457563439,
                                         0.309946505489309,
                                         0.297840075862812,
                                         0.241660995362026,
                                         0.160797067435223,
                                         0.0830901962777899,
                                         0.0298751305768699,
                                         0.00573171506908331,
                                         0.000223989846747249,
                                         1.22644972968921e-17,
                                         1.32752163250985e-12,
                                         6.68008040873255e-10,
                                         4.56773299801444e-08,
                                         1.08180731920008e-06,
                                         1.31550824735183e-05,
                                         0.000100585390473339,
                                         0.000544002303695965,
                                         0.00224109117663931,
                                         0.00738718108193502,
                                         0.020153977413697,
                                         0.0466015423504767,
                                         0.0928472404213051,
                                         0.161153778953633,
                                         0.245212323278107,
                                         0.327635631095374,
                                         0.382954787904813,
                                         0.387099056198032,
                                         0.330094240400145,
                                         0.225182057681531,
                                         0.107758499889738,
                                         0.0223698151588167,
                                         4.38588921605081e-21,
                                         2.52183478843118e-15,
                                         3.16518929060628e-12,
                                         4.10633632220432e-10,
                                         1.60489969900203e-08,
                                         2.96479452575759e-07,
                                         3.26564432798095e-06,
                                         2.45576700274951e-05,
                                         0.00013737447448127,
                                         0.000605583056299944,
                                         0.00219077374061691,
                                         0.00669859247786867,
                                         0.0176967720100195,
                                         0.041078445597656,
                                         0.08487423977963,
                                         0.157680204965298,
                                         0.265501210051669,
                                         0.407703235783672,
                                         0.573725516807242,
                                         0.742567285184163,
                                         0.886332222442529,
                                         0.977404908789797,

                                         0.979267860481196,
                                         0.895304850987842,
                                         0.761312390832218,
                                         0.601195604616849,
                                         0.439909020801398,
                                         0.297359997920204,
                                         0.184945327987176,
                                         0.10530000093907,
                                         0.0545294115206056,
                                         0.0254747278118643,
                                         0.0106264151457423,
                                         0.00390625,
                                         0.00124418076276625,
                                         0.000335847404102693,
                                         7.45905277073675e-05,
                                         1.30876496424839e-05,
                                         1.71235536999654e-06,
                                         1.53318678005907e-07,
                                         8.20398256182067e-09,
                                         2.07964391261829e-10,
                                         1.5916862188904e-12,
                                         1.2619775085405e-15,
                                         2.1888464034888e-21,
                                         0.0205426142701251,
                                         0.0997001953499298,
                                         0.211197953207519,
                                         0.315847491164948,
                                         0.380439486957168,
                                         0.389414032425809,
                                         0.347497887449246,
                                         0.273730078977488,
                                         0.191305478117369,
                                         0.118632476028865,
                                         0.0650187855904253,
                                         0.03125,
                                         0.0130140173430221,
                                         0.00461559747804944,
                                         0.00136071464204241,
                                         0.00032221628542494,
                                         5.83263863357313e-05,
                                         7.49283187175758e-06,
                                         6.07130405194534e-07,
                                         2.53342072114187e-08,
                                         3.6720672255909e-10,
                                         7.25281361798432e-13,
                                         6.67791752315367e-18,
                                         0.000188533260827283,
                                         0.00485734709474422,
                                         0.0256327126545915,
                                         0.0725967408073652,
                                         0.143941612746421,
                                         0.223109830671423,
                                         0.285653158176301,
                                         0.3113111872517,
                                         0.293631104939577,
                                         0.241699468068733,
                                         0.1740479794439,
                                         0.109375,
                                         0.0595548777612637,
                                         0.0277518485206967,
                                         0.0108599667057658,
                                         0.00347065440512793,
                                         0.000869189152342216,
                                         0.000160204594492494,
                                         1.96570025818132e-05,
                                         1.35021744526725e-06,
                                         3.70631090964228e-08,
                                         1.82364154248274e-10,
                                         8.91343028411979e-15,
                                         9.88740213622614e-07,
                                         0.000135227250721941,
                                         0.00177771191990203,
                                         0.00953495762222498,
                                         0.0311206810474558,
                                         0.0730445415012691,
                                         0.134180016994472,
                                         0.202315373012892,
                                         0.257536425546103,
                                         0.281390703794873,
                                         0.266232499453116,
                                         0.21875,
                                         0.155734647898638,
                                         0.0953493762930582,
                                         0.0495281242901337,
                                         0.0213617685559044,
                                         0.00740159770375841,
                                         0.00195733831524609,
                                         0.000363675929723018,
                                         4.11208480587987e-05,
                                         2.13764224657219e-06,
                                         2.6201996715729e-08,
                                         6.79846153102135e-12,
                                         3.24083137155998e-09,
                                         2.35293167509086e-06,
                                         7.70563116150444e-05,
                                         0.000782709180936833,
                                         0.00420525018086524,
                                         0.0149464084210062,
                                         0.0393927837949999,
                                         0.0821756329227496,
                                         0.141174183710696,
                                         0.204749954599758,
                                         0.254526596601126,
                                         0.2734375,
                                         0.254526596601126,
                                         0.204749954599758,
                                         0.141174183710696,
                                         0.0821756329227497,
                                         0.0393927837949998,
                                         0.0149464084210063,
                                         0.00420525018086524,
                                         0.000782709180936833,
                                         7.70563116150449e-05,
                                         2.35293167509086e-06,
                                         3.2408313715597e-09,
                                         6.79846153102207e-12,
                                         2.6201996715729e-08,
                                         2.13764224657217e-06,
                                         4.11208480587987e-05,
                                         0.000363675929723018,
                                         0.00195733831524609,
                                         0.00740159770375842,
                                         0.0213617685559044,
                                         0.0495281242901337,
                                         0.0953493762930582,
                                         0.155734647898638,
                                         0.21875,
                                         0.266232499453116,
                                         0.281390703794873,
                                         0.257536425546103,
                                         0.202315373012892,
                                         0.134180016994471,
                                         0.0730445415012692,
                                         0.0311206810474558,
                                         0.00953495762222498,
                                         0.00177771191990203,
                                         0.000135227250721941,
                                         9.88740213622551e-07,
                                         8.91343028412093e-15,
                                         1.82364154248274e-10,
                                         3.70631090964225e-08,
                                         1.35021744526725e-06,
                                         1.96570025818132e-05,
                                         0.000160204594492493,
                                         0.000869189152342218,
                                         0.00347065440512793,
                                         0.0108599667057658,
                                         0.0277518485206967,
                                         0.0595548777612637,
                                         0.109375,
                                         0.1740479794439,
                                         0.241699468068733,
                                         0.293631104939577,
                                         0.3113111872517,
                                         0.285653158176301,
                                         0.223109830671424,
                                         0.143941612746421,
                                         0.0725967408073652,
                                         0.0256327126545915,
                                         0.00485734709474422,
                                         0.000188533260827275,
                                         6.67791752315466e-18,
                                         7.25281361798432e-13,
                                         3.67206722559086e-10,
                                         2.53342072114187e-08,
                                         6.07130405194534e-07,
                                         7.49283187175756e-06,
                                         5.83263863357314e-05,
                                         0.00032221628542494,
                                         0.00136071464204241,
                                         0.00461559747804944,
                                         0.0130140173430221,
                                         0.03125,
                                         0.0650187855904253,
                                         0.118632476028865,
                                         0.191305478117369,
                                         0.273730078977488,
                                         0.347497887449245,
                                         0.389414032425809,
                                         0.380439486957168,
                                         0.315847491164948,
                                         0.211197953207519,
                                         0.0997001953499298,
                                         0.0205426142701246,
                                         2.18884640348917e-21,
                                         1.2619775085405e-15,
                                         1.59168621889038e-12,
                                         2.07964391261829e-10,
                                         8.20398256182067e-09,
                                         1.53318678005906e-07,
                                         1.71235536999654e-06,
                                         1.30876496424839e-05,
                                         7.45905277073675e-05,
                                         0.000335847404102693,
                                         0.00124418076276625,
                                         0.00390625,
                                         0.0106264151457423,
                                         0.0254747278118643,
                                         0.0545294115206056,
                                         0.10530000093907,
                                         0.184945327987176,
                                         0.297359997920204,
                                         0.439909020801398,
                                         0.601195604616849,
                                         0.761312390832218,
                                         0.895304850987842,
                                         0.979267860481196,

                                         0.980910241946381,
                                         0.903273535042967,
                                         0.778184158367112,
                                         0.626410363346187,
                                         0.470256632726424,
                                         0.32839912584075,
                                         0.21262539204281,
                                         0.127097078971741,
                                         0.0697674598078105,
                                         0.0349364537337338,
                                         0.0158270825127816,
                                         0.00641916929678917,
                                         0.00230014723454175,
                                         0.000715875794696415,
                                         0.00018926404871408,
                                         4.12628093846338e-05,
                                         7.1224823758249e-06,
                                         9.18706616325978e-07,
                                         8.12618792329773e-08,
                                         4.30429216060754e-09,
                                         1.0822105752295e-10,
                                         8.23140632407526e-13,
                                         6.49831116028683e-16,
                                         1.12441884437828e-21,
                                         0.018929171758283,
                                         0.0924766151354235,
                                         0.198253712818775,
                                         0.301738360318288,
                                         0.372066670808874,
                                         0.392353551434014,
                                         0.363203754259059,
                                         0.29908011099563,
                                         0.220409918106139,
                                         0.145576900886023,
                                         0.0859845563839051,
                                         0.0451703460382816,
                                         0.0209199683258416,
                                         0.00843330997885779,
                                         0.00290692916960963,
                                         0.000835911972030511,
                                         0.000193713279451504,
                                         3.44208520407407e-05,
                                         4.35302578613607e-06,
                                         3.48173631710485e-07,
                                         1.43787176451866e-08,
                                         2.06783113582014e-10,
                                         4.06226113529142e-13,
                                         3.72911474339369e-18,
                                         0.00015981296611841,
                                         0.00414211947612643,
                                         0.0220972551585927,
                                         0.0635887335109394,
                                         0.128790747600595,
                                         0.205083745750959,
                                         0.271433588223909,
                                         0.307905576301308,
                                         0.304640341046584,
                                         0.265389769579365,
                                         0.204370449804804,
                                         0.139061228838351,
                                         0.0832423756747535,
                                         0.0434646820095063,
                                         0.0195334444066987,
                                         0.00740867176737405,
                                         0.00230497097597744,
                                         0.000564214763933625,
                                         0.000102017264825887,
                                         1.23216273586071e-05,
                                         8.3580813716238e-07,
                                         2.27265837584815e-08,
                                         1.1109978795821e-10,
                                         5.41079942499078e-15,
                                         7.70999974180466e-07,
                                         0.0001060169410976,
                                         0.00140739914555936,
                                         0.00765758411219946,
                                         0.0254747840625442,
                                         0.0612557476978479,
                                         0.115914775427059,
                                         0.181137982637751,
                                         0.240605551667179,
                                         0.276463522003917,
                                         0.277573305918098,
                                         0.244636164615526,
                                         0.189273520878609,
                                         0.128007937701713,
                                         0.0750041274539168,
                                         0.0375216661174474,
                                         0.0156723268390472,
                                         0.00528480816493593,
                                         0.00136621189606314,
                                         0.000249173707659285,
                                         2.77622707744716e-05,
                                         1.42729977285783e-06,
                                         1.73628324202096e-08,
                                         4.48620458696506e-12,
                                         2.32475254755706e-09,
                                         1.6959300462711e-06,
                                         5.60242759990201e-05,
                                         0.000576346146535945,
                                         0.00314931698862125,
                                         0.0114351658278751,
                                         0.030938127559636,
                                         0.0666011175167186,
                                         0.118769216706051,
                                         0.179999588718022,
                                         0.235622799895638,
                                         0.268977079097306,
                                         0.268977079097306,
                                         0.235622799895638,
                                         0.179999588718022,
                                         0.118769216706051,
                                         0.0666011175167186,
                                         0.030938127559636,
                                         0.0114351658278751,
                                         0.00314931698862125,
                                         0.000576346146535945,
                                         5.60242759990197e-05,
                                         1.6959300462711e-06,
                                         2.32475254755685e-09,
                                         4.48620458696557e-12,
                                         1.73628324202096e-08,
                                         1.42729977285784e-06,
                                         2.77622707744716e-05,
                                         0.000249173707659285,
                                         0.00136621189606314,
                                         0.00528480816493593,
                                         0.0156723268390472,
                                         0.0375216661174474,
                                         0.0750041274539168,
                                         0.128007937701713,
                                         0.189273520878609,
                                         0.244636164615526,
                                         0.277573305918098,
                                         0.276463522003917,
                                         0.240605551667179,
                                         0.181137982637751,
                                         0.115914775427059,
                                         0.0612557476978478,
                                         0.0254747840625442,
                                         0.00765758411219946,
                                         0.00140739914555936,
                                         0.0001060169410976,
                                         7.70999974180413e-07,
                                         5.41079942499153e-15,
                                         1.1109978795821e-10,
                                         2.27265837584818e-08,
                                         8.3580813716238e-07,
                                         1.23216273586071e-05,
                                         0.000102017264825887,
                                         0.000564214763933625,
                                         0.00230497097597744,
                                         0.00740867176737405,
                                         0.0195334444066987,
                                         0.0434646820095063,
                                         0.0832423756747535,
                                         0.139061228838351,
                                         0.204370449804804,
                                         0.265389769579365,
                                         0.304640341046584,
                                         0.307905576301307,
                                         0.271433588223909,
                                         0.205083745750959,
                                         0.128790747600595,
                                         0.0635887335109394,
                                         0.0220972551585926,
                                         0.00414211947612643,
                                         0.000159812966118403,
                                         3.72911474339429e-18,
                                         4.06226113529142e-13,
                                         2.06783113582017e-10,
                                         1.43787176451866e-08,
                                         3.48173631710485e-07,
                                         4.35302578613608e-06,
                                         3.44208520407407e-05,
                                         0.000193713279451505,
                                         0.000835911972030511,
                                         0.00290692916960963,
                                         0.00843330997885779,
                                         0.0209199683258416,
                                         0.0451703460382816,
                                         0.0859845563839051,
                                         0.145576900886023,
                                         0.220409918106139,
                                         0.29908011099563,
                                         0.363203754259059,
                                         0.392353551434014,
                                         0.372066670808874,
                                         0.301738360318288,
                                         0.198253712818774,
                                         0.0924766151354235,
                                         0.0189291717582826,
                                         1.12441884437849e-21,
                                         6.49831116028683e-16,
                                         8.23140632407538e-13,
                                         1.0822105752295e-10,
                                         4.30429216060754e-09,
                                         8.12618792329776e-08,
                                         9.18706616325978e-07,
                                         7.12248237582492e-06,
                                         4.12628093846338e-05,
                                         0.00018926404871408,
                                         0.000715875794696415,
                                         0.00230014723454175,
                                         0.00641916929678917,
                                         0.0158270825127816,
                                         0.0349364537337338,
                                         0.0697674598078105,
                                         0.127097078971741,
                                         0.21262539204281,
                                         0.32839912584075,
                                         0.470256632726424,
                                         0.626410363346187,
                                         0.778184158367112,
                                         0.903273535042967,
                                         0.980910241946381,

                                         0.001953125,
                                         0.017578125,
                                         0.0703125,
                                         0.1640625,
                                         0.24609375,
                                         0.24609375,
                                         0.1640625,
                                         0.0703125,
                                         0.017578125,
                                         0.001953125,

                                         0.118054715021316,
                                         8.40534239649844e-07,
                                         0.284693989973831,
                                         2.82322483908705e-05,
                                         0.305134098813929,
                                         0.000421456741626533,
                                         0.190774349174245,
                                         0.00367009527019952,
                                         0.0766767491967178,
                                         0.0205454730255043,
                                         0.0205454730255043,
                                         0.0766767491967178,
                                         0.00367009527019952,
                                         0.190774349174245,
                                         0.000421456741626533,
                                         0.305134098813929,
                                         2.82322483908705e-05,
                                         0.284693989973831,
                                         8.40534239649844e-07,
                                         0.118054715021316,

                                         0.340899997066588,
                                         0.001953125,
                                         2.93341158288331e-09,
                                         0.389699792147695,
                                         0.017578125,
                                         2.0785230485651e-07,
                                         0.197993454329062,
                                         0.0703125,
                                         6.5456709384245e-06,
                                         0.0586797540973283,
                                         0.1640625,
                                         0.00012024590267193,
                                         0.0111799590162214,
                                         0.24609375,
                                         0.00142004098377867,
                                         0.00142004098377867,
                                         0.24609375,
                                         0.0111799590162214,
                                         0.00012024590267193,
                                         0.1640625,
                                         0.0586797540973283,
                                         6.5456709384245e-06,
                                         0.0703125,
                                         0.197993454329062,
                                         2.0785230485651e-07,
                                         0.017578125,
                                         0.389699792147695,
                                         2.93341158288331e-09,
                                         0.001953125,
                                         0.340899997066588,

                                         0.523279452073962,
                                         0.0272030706779361,
                                         4.64234829899064e-05,
                                         3.74997444104046e-11,
                                         0.351387820958782,
                                         0.120591915575868,
                                         0.000848247280189908,
                                         4.52335114521126e-09,
                                         0.104871434886943,
                                         0.237594257475621,
                                         0.00688850079034372,
                                         2.42498904157948e-07,
                                         0.018257634536266,
                                         0.273067792598097,
                                         0.0326320038318305,
                                         7.58361106640354e-06,
                                         0.00204336656386727,
                                         0.20175276400069,
                                         0.0993750242864341,
                                         0.000152460309358515,
                                         0.000152460309358515,
                                         0.0993750242864341,
                                         0.20175276400069,
                                         0.00204336656386727,
                                         7.5836110664035e-06,
                                         0.0326320038318305,
                                         0.273067792598097,
                                         0.018257634536266,
                                         2.42498904157946e-07,
                                         0.00688850079034372,
                                         0.237594257475621,
                                         0.104871434886943,
                                         4.52335114521123e-09,
                                         0.000848247280189908,
                                         0.120591915575868,
                                         0.351387820958782,
                                         3.74997444104043e-11,
                                         4.64234829899064e-05,
                                         0.0272030706779361,
                                         0.523279452073962,

                                         0.648940531175615,
                                         0.0943038827069123,
                                         0.001953125,
                                         1.85581766832031e-06,
                                         1.10000666072867e-12,
                                         0.287461493558098,
                                         0.254614909661082,
                                         0.017578125,
                                         5.56757487928878e-05,
                                         2.01143293064475e-10,
                                         0.0565942015802643,
                                         0.30553120336261,
                                         0.0703125,
                                         0.000742359567504601,
                                         1.63468199438075e-08,
                                         0.00649951560759269,
                                         0.213867160694829,
                                         0.1640625,
                                         0.00577403414098331,
                                         7.74957339876462e-07,
                                         0.000479848916350383,
                                         0.0962381156122405,
                                         0.24609375,
                                         0.0288708026873771,
                                         2.36176556761302e-05,
                                         2.36176556761301e-05,
                                         0.0288708026873771,
                                         0.24609375,
                                         0.0962381156122405,
                                         0.000479848916350385,
                                         7.74957339876457e-07,
                                         0.00577403414098331,
                                         0.1640625,
                                         0.213867160694829,
                                         0.00649951560759272,
                                         1.63468199438074e-08,
                                         0.000742359567504601,
                                         0.0703125,
                                         0.30553120336261,
                                         0.0565942015802645,
                                         2.01143293064473e-10,
                                         5.56757487928878e-05,
                                         0.017578125,
                                         0.254614909661082,
                                         0.287461493558098,
                                         1.10000666072866e-12,
                                         1.85581766832031e-06,
                                         0.001953125,
                                         0.0943038827069123,
                                         0.648940531175615,

                                         0.73408086166281,
                                         0.188169584696206,
                                         0.0134020201062015,
                                         0.000167937389586946,
                                         1.14845071680197e-07,
                                         5.70464869069653e-14,
                                         0.23087325908181,
                                         0.345381386658926,
                                         0.0741441516659445,
                                         0.00245881459186946,
                                         5.06813155674938e-06,
                                         1.46921106793994e-11,
                                         0.0322716259764131,
                                         0.281751289976078,
                                         0.18230602327775,
                                         0.0160000613246008,
                                         9.94033173009336e-05,
                                         1.68173266649671e-09,
                                         0.00263138779083231,
                                         0.134075582164827,
                                         0.261482078757008,
                                         0.0607343386342396,
                                         0.00113728855921434,
                                         1.1229140683586e-07,
                                         0.000137931461165695,
                                         0.041015493692345,
                                         0.241100057449734,
                                         0.148204516803066,
                                         0.00836478795847495,
                                         4.82003907990717e-06,
                                         4.82003907990717e-06,
                                         0.00836478795847497,
                                         0.148204516803066,
                                         0.241100057449734,
                                         0.041015493692345,
                                         0.000137931461165695,
                                         1.1229140683586e-07,
                                         0.00113728855921434,
                                         0.0607343386342396,
                                         0.261482078757008,
                                         0.134075582164826,
                                         0.00263138779083231,
                                         1.68173266649671e-09,
                                         9.94033173009338e-05,
                                         0.0160000613246008,
                                         0.18230602327775,
                                         0.281751289976078,
                                         0.0322716259764131,
                                         1.46921106793994e-11,
                                         5.06813155674939e-06,
                                         0.00245881459186946,
                                         0.0741441516659445,
                                         0.345381386658926,
                                         0.23087325908181,
                                         5.70464869069653e-14,
                                         1.14845071680197e-07,
                                         0.000167937389586946,
                                         0.0134020201062015,
                                         0.188169584696206,
                                         0.73408086166281,

                                         0.792963138849073,
                                         0.287813619406762,
                                         0.0418955046026501,
                                         0.001953125,
                                         1.80229903005919e-05,
                                         1.00554936355042e-08,
                                         4.47283047134315e-15,
                                         0.18634163036405,
                                         0.384441926922649,
                                         0.159357347810958,
                                         0.017578125,
                                         0.000383801970606365,
                                         6.09774150668569e-07,
                                         1.54173795912652e-12,
                                         0.0194618564249364,
                                         0.22822732059979,
                                         0.269397922730685,
                                         0.0703125,
                                         0.0036324950690339,
                                         1.64343338986463e-05,
                                         2.36187100449204e-10,
                                         0.00118570167034693,
                                         0.0790353374469651,
                                         0.265664292850764,
                                         0.1640625,
                                         0.0200548612000703,
                                         0.000258375896602107,
                                         2.11066144876013e-08,
                                         4.64388089755155e-05,
                                         0.0175950078883927,
                                         0.168417262258783,
                                         0.24609375,
                                         0.0711784885161493,
                                         0.00261135767529592,
                                         1.21253827051535e-06,
                                         1.21253827051534e-06,
                                         0.00261135767529592,
                                         0.0711784885161493,
                                         0.24609375,
                                         0.168417262258783,
                                         0.0175950078883928,
                                         4.64388089755159e-05,
                                         2.1106614487601e-08,
                                         0.000258375896602107,
                                         0.0200548612000703,
                                         0.1640625,
                                         0.265664292850764,
                                         0.0790353374469651,
                                         0.00118570167034694,
                                         2.361871004492e-10,
                                         1.64343338986463e-05,
                                         0.0036324950690339,
                                         0.0703125,
                                         0.269397922730685,
                                         0.22822732059979,
                                         0.0194618564249365,
                                         1.5417379591265e-12,
                                         6.09774150668569e-07,
                                         0.000383801970606365,
                                         0.017578125,
                                         0.159357347810958,
                                         0.384441926922649,
                                         0.18634163036405,
                                         4.47283047134306e-15,
                                         1.00554936355042e-08,
                                         1.80229903005919e-05,
                                         0.001953125,
                                         0.0418955046026501,
                                         0.287813619406762,
                                         0.792963138849073,

                                         0.834858115145607,
                                         0.381010743235229,
                                         0.087402325287584,
                                         0.00889259559384836,
                                         0.000315245284999029,
                                         2.38006603674817e-06,
                                         1.16040849539582e-09,
                                         4.79560216404684e-16,
                                         0.152207602881117,
                                         0.388079990277973,
                                         0.244653036144052,
                                         0.0552227112130797,
                                         0.00411191791263751,
                                         6.88725880843439e-05,
                                         9.22809144079149e-08,
                                         2.13061261026564e-13,
                                         0.0123332490499632,
                                         0.175680177618941,
                                         0.304365703963741,
                                         0.1524138344606,
                                         0.0238373379923246,
                                         0.000885770640384145,
                                         3.26159746436003e-06,
                                         4.20710470388535e-11,
                                         0.000582955562416167,
                                         0.0463917832813303,
                                         0.220880620185582,
                                         0.245384907424297,
                                         0.0806098042817147,
                                         0.00664527492519944,
                                         6.72458714914327e-05,
                                         4.84595014790653e-09,
                                         1.77136424205652e-05,
                                         0.00787542220460056,
                                         0.103046675581335,
                                         0.253971965322744,
                                         0.175239680513756,
                                         0.0320493406180019,
                                         0.000891282471648028,
                                         3.58830241426015e-07,
                                         3.58830241426015e-07,
                                         0.000891282471648028,
                                         0.0320493406180019,
                                         0.175239680513756,
                                         0.253971965322744,
                                         0.103046675581335,
                                         0.00787542220460056,
                                         1.77136424205652e-05,
                                         4.84595014790653e-09,
                                         6.72458714914327e-05,
                                         0.00664527492519944,
                                         0.0806098042817148,
                                         0.245384907424297,
                                         0.220880620185582,
                                         0.0463917832813303,
                                         0.000582955562416167,
                                         4.20710470388534e-11,
                                         3.26159746436003e-06,
                                         0.000885770640384145,
                                         0.0238373379923246,
                                         0.1524138344606,
                                         0.304365703963741,
                                         0.175680177618941,
                                         0.0123332490499632,
                                         2.13061261026564e-13,
                                         9.22809144079149e-08,
                                         6.88725880843439e-05,
                                         0.00411191791263751,
                                         0.0552227112130797,
                                         0.244653036144052,
                                         0.388079990277973,
                                         0.152207602881117,
                                         4.79560216404684e-16,
                                         1.16040849539582e-09,
                                         2.38006603674817e-06,
                                         0.000315245284999029,
                                         0.00889259559384836,
                                         0.087402325287584,
                                         0.381010743235229,
                                         0.834858115145607,

                                         0.865514055653727,
                                         0.463073954063879,
                                         0.144656950071109,
                                         0.0244607617724454,
                                         0.001953125,
                                         5.73831951853805e-05,
                                         3.77029645882362e-07,
                                         1.67333622507954e-10,
                                         6.56837846813454e-17,
                                         0.126016081990981,
                                         0.37219822068189,
                                         0.311990515017672,
                                         0.112337624829852,
                                         0.017578125,
                                         0.00101207916949535,
                                         1.415984281185e-05,
                                         1.68633670750249e-08,
                                         3.65419181086648e-14,
                                         0.00815446179277346,
                                         0.132958379314319,
                                         0.299061811526877,
                                         0.229296791868129,
                                         0.0703125,
                                         0.00793344340247131,
                                         0.00023635176307578,
                                         7.55305863672351e-07,
                                         9.03528785199791e-12,
                                         0.000307809079176318,
                                         0.0277060061756257,
                                         0.167223505112193,
                                         0.273015631735544,
                                         0.1640625,
                                         0.0362765320820695,
                                         0.00230131492606189,
                                         1.97341403613635e-05,
                                         1.30319510568632e-09,
                                         7.46933646073754e-06,
                                         0.00371147564021952,
                                         0.0601101980206355,
                                         0.20897364723627,
                                         0.24609375,
                                         0.106636104708539,
                                         0.0144048166899179,
                                         0.00033145764714034,
                                         1.20834614566922e-07,
                                         1.2083461456692e-07,
                                         0.00033145764714034,
                                         0.0144048166899178,
                                         0.106636104708539,
                                         0.24609375,
                                         0.20897364723627,
                                         0.0601101980206356,
                                         0.00371147564021952,
                                         7.46933646073765e-06,
                                         1.30319510568629e-09,
                                         1.97341403613635e-05,
                                         0.00230131492606188,
                                         0.0362765320820695,
                                         0.1640625,
                                         0.273015631735544,
                                         0.167223505112193,
                                         0.0277060061756257,
                                         0.000307809079176321,
                                         9.03528785199769e-12,
                                         7.55305863672351e-07,
                                         0.000236351763075779,
                                         0.00793344340247132,
                                         0.0703125,
                                         0.229296791868129,
                                         0.299061811526877,
                                         0.132958379314319,
                                         0.00815446179277351,
                                         3.65419181086638e-14,
                                         1.68633670750249e-08,
                                         1.415984281185e-05,
                                         0.00101207916949535,
                                         0.017578125,
                                         0.112337624829852,
                                         0.311990515017672,
                                         0.37219822068189,
                                         0.126016081990981,
                                         6.56837846813433e-17,
                                         1.67333622507954e-10,
                                         3.77029645882362e-07,
                                         5.73831951853805e-05,
                                         0.001953125,
                                         0.0244607617724454,
                                         0.144656950071109,
                                         0.463073954063879,
                                         0.865514055653727,

                                         0.88852425956534,
                                         0.533300959938584,
                                         0.20755808158595,
                                         0.0498912150350392,
                                         0.00681055966738008,
                                         0.000457797270860624,
                                         1.17557662985834e-05,
                                         6.98690834023006e-08,
                                         2.89666899763972e-11,
                                         1.09525890644255e-17,
                                         0.105710244555335,
                                         0.347257115360417,
                                         0.356595690600092,
                                         0.177492777724896,
                                         0.0454094735334404,
                                         0.00556153344838018,
                                         0.00026765746312383,
                                         3.29407605568205e-06,
                                         3.60333883417891e-09,
                                         7.45682626606297e-15,
                                         0.00558962080908566,
                                         0.1004956869068,
                                         0.272288948568944,
                                         0.280643361293379,
                                         0.134563604969729,
                                         0.0300284827816705,
                                         0.00270847762881428,
                                         6.90239295947184e-05,
                                         1.99218109789502e-07,
                                         2.25636179659522e-12,
                                         0.000172410777565855,
                                         0.0169652587091248,
                                         0.121283224080305,
                                         0.25884850111143,
                                         0.232608485657004,
                                         0.0945777590804261,
                                         0.0159877718524998,
                                         0.000843689283255162,
                                         6.42494363471879e-06,
                                         3.9827243106243e-10,
                                         3.41869965174573e-06,
                                         0.00184114513293217,
                                         0.0347284919949928,
                                         0.153479723968527,
                                         0.258486560795834,
                                         0.191495742795276,
                                         0.0606687581559915,
                                         0.00662948601172686,
                                         0.000133206158092031,
                                         4.51924853494426e-08,
                                         4.51924853494426e-08,
                                         0.000133206158092031,
                                         0.00662948601172686,
                                         0.0606687581559915,
                                         0.191495742795276,
                                         0.258486560795834,
                                         0.153479723968527,
                                         0.0347284919949928,
                                         0.00184114513293217,
                                         3.41869965174573e-06,
                                         3.9827243106243e-10,
                                         6.42494363471883e-06,
                                         0.000843689283255162,
                                         0.0159877718524998,
                                         0.0945777590804261,
                                         0.232608485657004,
                                         0.25884850111143,
                                         0.121283224080305,
                                         0.0169652587091248,
                                         0.000172410777565855,
                                         2.25636179659522e-12,
                                         1.99218109789503e-07,
                                         6.90239295947184e-05,
                                         0.00270847762881428,
                                         0.0300284827816705,
                                         0.134563604969729,
                                         0.280643361293379,
                                         0.272288948568944,
                                         0.1004956869068,
                                         0.00558962080908566,
                                         7.45682626606297e-15,
                                         3.60333883417893e-09,
                                         3.29407605568205e-06,
                                         0.00026765746312383,
                                         0.00556153344838018,
                                         0.0454094735334404,
                                         0.177492777724896,
                                         0.356595690600092,
                                         0.347257115360417,
                                         0.105710244555335,
                                         1.09525890644255e-17,
                                         2.89666899763974e-11,
                                         6.98690834023006e-08,
                                         1.17557662985834e-05,
                                         0.000457797270860624,
                                         0.00681055966738008,
                                         0.0498912150350392,
                                         0.20755808158595,
                                         0.533300959938584,
                                         0.88852425956534,

                                         0.906188279745077,
                                         0.592661621384172,
                                         0.271324107858288,
                                         0.0841390013554658,
                                         0.0167323621624069,
                                         0.001953125,
                                         0.000115632816023,
                                         2.68691849984411e-06,
                                         1.48184503966547e-08,
                                         5.83808098680115e-12,
                                         2.14633156915631e-18,
                                         0.0897572748580563,
                                         0.319227864884406,
                                         0.380860407051976,
                                         0.239724752059501,
                                         0.0866456690438467,
                                         0.017578125,
                                         0.00180874156007398,
                                         7.63877973634863e-05,
                                         8.55087122475406e-07,
                                         8.77932539246432e-10,
                                         1.75521618448044e-15,
                                         0.00395128501807154,
                                         0.0764209203151126,
                                         0.237607832532905,
                                         0.303560937254304,
                                         0.199413219356654,
                                         0.0703125,
                                         0.0125744620626961,
                                         0.000965185887167953,
                                         2.19298379895477e-05,
                                         5.86772373555435e-08,
                                         6.37943031839741e-13,
                                         0.00010146677361351,
                                         0.0106718700364353,
                                         0.0864713963544483,
                                         0.224231016629604,
                                         0.267718155497604,
                                         0.1640625,
                                         0.0509940031232733,
                                         0.0071140142825945,
                                         0.000328078125145613,
                                         2.28767836254999e-06,
                                         1.35253955623774e-10,
                                         1.67503473831988e-06,
                                         0.00095803916972009,
                                         0.0202301296125573,
                                         0.106478018516221,
                                         0.231055428925939,
                                         0.24609375,
                                         0.132942325451482,
                                         0.0337079992992782,
                                         0.00315524872111706,
                                         5.733697078296e-05,
                                         1.84345493909534e-08,
                                         1.8434549390953e-08,
                                         5.733697078296e-05,
                                         0.00315524872111706,
                                         0.0337079992992782,
                                         0.132942325451482,
                                         0.24609375,
                                         0.231055428925939,
                                         0.106478018516221,
                                         0.0202301296125573,
                                         0.00095803916972009,
                                         1.67503473831991e-06,
                                         1.3525395562377e-10,
                                         2.28767836254999e-06,
                                         0.000328078125145614,
                                         0.00711401428259451,
                                         0.0509940031232732,
                                         0.1640625,
                                         0.267718155497605,
                                         0.224231016629604,
                                         0.0864713963544482,
                                         0.0106718700364353,
                                         0.000101466773613512,
                                         6.37943031839718e-13,
                                         5.86772373555435e-08,
                                         2.19298379895477e-05,
                                         0.000965185887167954,
                                         0.0125744620626961,
                                         0.0703125,
                                         0.199413219356654,
                                         0.303560937254304,
                                         0.237607832532905,
                                         0.0764209203151126,
                                         0.00395128501807158,
                                         1.75521618448037e-15,
                                         8.77932539246432e-10,
                                         8.55087122475409e-07,
                                         7.63877973634864e-05,
                                         0.00180874156007397,
                                         0.017578125,
                                         0.0866456690438467,
                                         0.239724752059501,
                                         0.380860407051976,
                                         0.319227864884406,
                                         0.0897572748580568,
                                         2.14633156915621e-18,
                                         5.83808098680115e-12,
                                         1.48184503966548e-08,
                                         2.68691849984411e-06,
                                         0.000115632816023,
                                         0.001953125,
                                         0.0167323621624069,
                                         0.0841390013554658,
                                         0.271324107858288,
                                         0.592661621384172,
                                         0.906188279745077,

                                         0.920018017544691,
                                         0.642648116843337,
                                         0.332869886578574,
                                         0.124941120538524,
                                         0.0327358366306177,
                                         0.00564824597051831,
                                         0.00058581398402507,
                                         3.14938468661993e-05,
                                         6.78063257221626e-07,
                                         3.53129647512309e-09,
                                         1.3378089980036e-12,
                                         4.81331016979569e-19,
                                         0.0770508536995015,
                                         0.291247705374337,
                                         0.389474656848333,
                                         0.292347612968695,
                                         0.136165224358977,
                                         0.0395189758349758,
                                         0.00678192017357317,
                                         0.000613292945313636,
                                         2.34725933478027e-05,
                                         2.44463769734291e-07,
                                         2.39105660952087e-10,
                                         4.65530076926346e-16,
                                         0.00286797960842162,
                                         0.058663675198922,
                                         0.202535604508734,
                                         0.304026324791538,
                                         0.251725057756417,
                                         0.122889631711204,
                                         0.0348949913150061,
                                         0.00530796081983809,
                                         0.00036113486633739,
                                         7.52163595611216e-06,
                                         1.89934162449036e-08,
                                         2.00110152775935e-13,
                                         6.22718054329922e-05,
                                         0.00689275459525359,
                                         0.0614384643106245,
                                         0.184433420598578,
                                         0.271458525849408,
                                         0.222916183177939,
                                         0.104734631613306,
                                         0.0267981147184749,
                                         0.00324111588447162,
                                         0.0001349978678587,
                                         8.80102201996518e-07,
                                         5.01773008521791e-11,
                                         8.6920322546622e-07,
                                         0.000520631977531963,
                                         0.0119810200969825,
                                         0.0719254398008224,
                                         0.188189330516301,
                                         0.259945574319763,
                                         0.202084031899689,
                                         0.0869751625577856,
                                         0.0186996798944281,
                                         0.00155760015787021,
                                         2.62166745561584e-05,
                                         8.08835011722743e-09,
                                         8.08835011722767e-09,
                                         2.62166745561583e-05,
                                         0.00155760015787021,
                                         0.0186996798944281,
                                         0.0869751625577855,
                                         0.202084031899689,
                                         0.259945574319763,
                                         0.188189330516301,
                                         0.0719254398008225,
                                         0.0119810200969825,
                                         0.000520631977531965,
                                         8.69203225466199e-07,
                                         5.01773008521809e-11,
                                         8.80102201996511e-07,
                                         0.0001349978678587,
                                         0.00324111588447161,
                                         0.0267981147184748,
                                         0.104734631613306,
                                         0.222916183177939,
                                         0.271458525849408,
                                         0.184433420598578,
                                         0.0614384643106246,
                                         0.00689275459525361,
                                         6.2271805432991e-05,
                                         2.00110152775943e-13,
                                         1.89934162449035e-08,
                                         7.52163595611213e-06,
                                         0.00036113486633739,
                                         0.00530796081983808,
                                         0.0348949913150061,
                                         0.122889631711204,
                                         0.251725057756417,
                                         0.304026324791538,
                                         0.202535604508734,
                                         0.0586636751989222,
                                         0.00286797960842158,
                                         4.65530076926369e-16,
                                         2.39105660952085e-10,
                                         2.4446376973429e-07,
                                         2.34725933478027e-05,
                                         0.000613292945313635,
                                         0.00678192017357317,
                                         0.0395189758349758,
                                         0.136165224358977,
                                         0.292347612968695,
                                         0.389474656848333,
                                         0.291247705374338,
                                         0.077050853699501,
                                         4.81331016979595e-19,
                                         1.33780899800359e-12,
                                         3.53129647512308e-09,
                                         6.78063257221625e-07,
                                         3.14938468661992e-05,
                                         0.00058581398402507,
                                         0.00564824597051831,
                                         0.0327358366306177,
                                         0.124941120538524,
                                         0.332869886578574,
                                         0.642648116843337,
                                         0.920018017544691,

                                         0.931034268387415,
                                         0.684776685230924,
                                         0.390488154604692,
                                         0.169793046370148,
                                         0.0548247896594246,
                                         0.012627930728698,
                                         0.001953125,
                                         0.000184849969628659,
                                         9.21915390796935e-06,
                                         1.87008489560124e-07,
                                         9.31186469577859e-10,
                                         3.42098268889217e-13,
                                         1.21013423862849e-19,
                                         0.0667957839820509,
                                         0.264831393031033,
                                         0.387067677072141,
                                         0.332778991417701,
                                         0.187868679415866,
                                         0.0710787765636639,
                                         0.017578125,
                                         0.00266009195271012,
                                         0.000217920795571245,
                                         7.72876923913341e-06,
                                         7.60926368141927e-08,
                                         7.16498682005776e-11,
                                         1.36626575280558e-16,
                                         0.00212985441592324,
                                         0.045520544302933,
                                         0.170523182747666,
                                         0.289874080362162,
                                         0.286120696087496,
                                         0.177813683592171,
                                         0.0703125,
                                         0.017013411136942,
                                         0.00228941101520067,
                                         0.000141963546603993,
                                         2.76354214828815e-06,
                                         6.66956327403186e-09,
                                         6.85573935226307e-14,
                                         3.96157224031372e-05,
                                         0.00456417430647637,
                                         0.0438224594469777,
                                         0.147292169986169,
                                         0.254191460975531,
                                         0.259481982771236,
                                         0.1640625,
                                         0.063475033495805,
                                         0.0140302579045897,
                                         0.00152110829449592,
                                         5.85472290484752e-05,
                                         3.62156423177821e-07,
                                         2.00673584054183e-11,
                                         4.7369599698267e-07,
                                         0.000294192414765836,
                                         0.00723976616691544,
                                         0.0481132186305477,
                                         0.145173422427208,
                                         0.24342417708731,
                                         0.24609375,
                                         0.152240062701835,
                                         0.055274142565204,
                                         0.0104775056144453,
                                         0.000797372166587034,
                                         1.26418158895968e-05,
                                         3.77607481652501e-09,
                                         3.77607481652514e-09,
                                         1.26418158895968e-05,
                                         0.000797372166587031,
                                         0.0104775056144453,
                                         0.055274142565204,
                                         0.152240062701835,
                                         0.24609375,
                                         0.24342417708731,
                                         0.145173422427208,
                                         0.0481132186305476,
                                         0.00723976616691545,
                                         0.000294192414765835,
                                         4.73695996982657e-07,
                                         2.00673584054191e-11,
                                         3.62156423177824e-07,
                                         5.8547229048475e-05,
                                         0.00152110829449593,
                                         0.0140302579045897,
                                         0.063475033495805,
                                         0.1640625,
                                         0.259481982771236,
                                         0.254191460975531,
                                         0.147292169986169,
                                         0.0438224594469778,
                                         0.00456417430647635,
                                         3.96157224031364e-05,
                                         6.85573935226341e-14,
                                         6.66956327403192e-09,
                                         2.76354214828814e-06,
                                         0.000141963546603994,
                                         0.00228941101520067,
                                         0.017013411136942,
                                         0.0703125,
                                         0.177813683592171,
                                         0.286120696087496,
                                         0.289874080362161,
                                         0.170523182747666,
                                         0.0455205443029329,
                                         0.00212985441592321,
                                         1.36626575280566e-16,
                                         7.16498682005784e-11,
                                         7.60926368141923e-08,
                                         7.72876923913343e-06,
                                         0.000217920795571246,
                                         0.00266009195271012,
                                         0.017578125,
                                         0.0710787765636639,
                                         0.187868679415866,
                                         0.3327789914177,
                                         0.387067677072141,
                                         0.264831393031032,
                                         0.0667957839820504,
                                         1.21013423862857e-19,
                                         3.42098268889221e-13,
                                         9.31186469577855e-10,
                                         1.87008489560124e-07,
                                         9.21915390796935e-06,
                                         0.000184849969628659,
                                         0.001953125,
                                         0.012627930728698,
                                         0.0548247896594246,
                                         0.169793046370148,
                                         0.390488154604692,
                                         0.684776685230924,
                                         0.931034268387415,

                                         0.93994352605176,
                                         0.720401898986882,
                                         0.443412193020801,
                                         0.216493250702587,
                                         0.0822403889442549,
                                         0.0236192537797201,
                                         0.00491792292704952,
                                         0.000697867448607169,
                                         6.14340353347228e-05,
                                         2.88669828447954e-06,
                                         5.58431177012161e-08,
                                         2.682834830134e-10,
                                         9.61706327494035e-14,
                                         3.35607642690437e-20,
                                         0.0584166303171948,
                                         0.240610048760533,
                                         0.377401980196092,
                                         0.361105518939959,
                                         0.236789697463,
                                         0.109724177555486,
                                         0.0356287864630183,
                                         0.00780258751326935,
                                         0.0010711687651019,
                                         8.12098607757627e-05,
                                         2.71184530049678e-06,
                                         2.55318574822688e-08,
                                         2.33232238309374e-11,
                                         4.37403365711576e-17,
                                         0.00161357339411421,
                                         0.0357166037124326,
                                         0.142763833038203,
                                         0.267695685940874,
                                         0.303010676894345,
                                         0.226545780592781,
                                         0.114719652870375,
                                         0.0387723243456328,
                                         0.00830088198271583,
                                         0.00101539172473769,
                                         5.85298818591834e-05,
                                         1.07991200480745e-06,
                                         2.51392414279806e-09,
                                         2.53366901635575e-14,
                                         2.59990656604573e-05,
                                         0.00309273951471022,
                                         0.0315027900516469,
                                         0.115761837742985,
                                         0.226188153126965,
                                         0.27285154385495,
                                         0.215472282453679,
                                         0.112388485745325,
                                         0.0375238483694139,
                                         0.00740585606817343,
                                         0.000736897660569505,
                                         2.6644717527766e-05,
                                         1.58063848330148e-07,
                                         8.56119589561316e-12,
                                         2.69303024629406e-07,
                                         0.000172159504873729,
                                         0.00446883491791109,
                                         0.0321813254038812,
                                         0.108541607994568,
                                         0.21125710633821,
                                         0.26017135194807,
                                         0.209428738284974,
                                         0.109044804726288,
                                         0.0347241312248962,
                                         0.00596418603886725,
                                         0.000422618345672472,
                                         6.38891937657939e-06,
                                         1.85965962576495e-09,
                                         1.85965962576495e-09,
                                         6.38891937657944e-06,
                                         0.000422618345672472,
                                         0.00596418603886724,
                                         0.0347241312248961,
                                         0.109044804726288,
                                         0.209428738284974,
                                         0.26017135194807,
                                         0.21125710633821,
                                         0.108541607994568,
                                         0.0321813254038812,
                                         0.00446883491791109,
                                         0.000172159504873728,
                                         2.69303024629406e-07,
                                         8.56119589561317e-12,
                                         1.5806384833015e-07,
                                         2.6644717527766e-05,
                                         0.000736897660569503,
                                         0.00740585606817342,
                                         0.0375238483694139,
                                         0.112388485745325,
                                         0.215472282453679,
                                         0.27285154385495,
                                         0.226188153126965,
                                         0.115761837742985,
                                         0.0315027900516469,
                                         0.00309273951471021,
                                         2.59990656604573e-05,
                                         2.53366901635575e-14,
                                         2.51392414279809e-09,
                                         1.07991200480745e-06,
                                         5.85298818591833e-05,
                                         0.00101539172473768,
                                         0.00830088198271583,
                                         0.0387723243456328,
                                         0.114719652870375,
                                         0.226545780592781,
                                         0.303010676894345,
                                         0.267695685940874,
                                         0.142763833038203,
                                         0.0357166037124324,
                                         0.00161357339411421,
                                         4.37403365711576e-17,
                                         2.33232238309377e-11,
                                         2.55318574822688e-08,
                                         2.71184530049677e-06,
                                         8.12098607757625e-05,
                                         0.0010711687651019,
                                         0.00780258751326935,
                                         0.0356287864630183,
                                         0.109724177555486,
                                         0.236789697463,
                                         0.361105518939959,
                                         0.377401980196092,
                                         0.240610048760533,
                                         0.0584166303171948,
                                         3.35607642690437e-20,
                                         9.61706327494049e-14,
                                         2.682834830134e-10,
                                         5.58431177012159e-08,
                                         2.88669828447953e-06,
                                         6.14340353347228e-05,
                                         0.000697867448607168,
                                         0.00491792292704952,
                                         0.0236192537797201,
                                         0.0822403889442549,
                                         0.216493250702587,
                                         0.443412193020801,
                                         0.720401898986882,
                                         0.93994352605176,

                                         0.947245932468883,
                                         0.750668620766563,
                                         0.491456015243935,
                                         0.263337287398581,
                                         0.113827309103921,
                                         0.0388614806412547,
                                         0.010168308942307,
                                         0.001953125,
                                         0.000258643425616291,
                                         2.14786686564033e-05,
                                         9.61831946787098e-07,
                                         1.79057071815325e-08,
                                         8.35615457719164e-11,
                                         2.93627172912827e-14,
                                         1.01343880790502e-20,
                                         0.0514923196393769,
                                         0.218751811576882,
                                         0.363270045103649,
                                         0.378759607855378,
                                         0.279773095773075,
                                         0.151990371574695,
                                         0.0608582305722841,
                                         0.017578125,
                                         0.00350038548376106,
                                         0.000444831613088226,
                                         3.16974800698593e-05,
                                         1.00838225905815e-06,
                                         9.15685951584244e-09,
                                         8.16164811116992e-12,
                                         1.51009003712977e-17,
                                         0.00124405518541669,
                                         0.0283316936065772,
                                         0.119341640238462,
                                         0.242120989835997,
                                         0.305620801017244,
                                         0.264198466167207,
                                         0.161885291487656,
                                         0.0703125,
                                         0.0210546383706614,
                                         0.00409450505249555,
                                         0.000464266939833039,
                                         2.5239247156086e-05,
                                         4.45968433171348e-07,
                                         1.00827029096883e-09,
                                         1.00005903001905e-14,
                                         1.75328948834964e-05,
                                         0.00214047524064504,
                                         0.0228702657539518,
                                         0.0902855020095908,
                                         0.194749640565652,
                                         0.267892960155709,
                                         0.25119573183342,
                                         0.1640625,
                                         0.0738748297111628,
                                         0.0219848735868932,
                                         0.00396668372540505,
                                         0.000368505853392585,
                                         1.26700544483296e-05,
                                         7.26595776635417e-08,
                                         3.86336039161224e-12,
                                         1.5884812250597e-07,
                                         0.000103959062532905,
                                         0.00281750628549698,
                                         0.0216430302964904,
                                         0.0797783213165511,
                                         0.174625146736717,
                                         0.250571454464212,
                                         0.24609375,
                                         0.166632485708919,
                                         0.0758858858032842,
                                         0.0217872222463018,
                                         0.00345881121544883,
                                         0.000231402111201771,
                                         3.366070760829e-06,
                                         9.59443232899859e-10,
                                         9.59443232899814e-10,
                                         3.366070760829e-06,
                                         0.00023140211120177,
                                         0.00345881121544884,
                                         0.0217872222463018,
                                         0.0758858858032842,
                                         0.166632485708919,
                                         0.24609375,
                                         0.250571454464212,
                                         0.174625146736717,
                                         0.0797783213165511,
                                         0.0216430302964904,
                                         0.00281750628549698,
                                         0.000103959062532905,
                                         1.58848122505976e-07,
                                         3.86336039161202e-12,
                                         7.26595776635416e-08,
                                         1.26700544483296e-05,
                                         0.000368505853392586,
                                         0.00396668372540505,
                                         0.0219848735868932,
                                         0.0738748297111628,
                                         0.1640625,
                                         0.25119573183342,
                                         0.267892960155709,
                                         0.194749640565652,
                                         0.0902855020095907,
                                         0.0228702657539518,
                                         0.00214047524064504,
                                         1.75328948834969e-05,
                                         1.00005903001899e-14,
                                         1.00827029096883e-09,
                                         4.45968433171346e-07,
                                         2.52392471560861e-05,
                                         0.000464266939833039,
                                         0.00409450505249555,
                                         0.0210546383706614,
                                         0.0703125,
                                         0.161885291487656,
                                         0.264198466167207,
                                         0.305620801017244,
                                         0.242120989835997,
                                         0.119341640238463,
                                         0.0283316936065772,
                                         0.00124405518541671,
                                         1.51009003712966e-17,
                                         8.16164811116991e-12,
                                         9.15685951584239e-09,
                                         1.00838225905815e-06,
                                         3.16974800698593e-05,
                                         0.000444831613088226,
                                         0.00350038548376106,
                                         0.017578125,
                                         0.0608582305722841,
                                         0.151990371574695,
                                         0.279773095773075,
                                         0.378759607855377,
                                         0.36327004510365,
                                         0.218751811576882,
                                         0.0514923196393774,
                                         1.01343880790493e-20,
                                         2.93627172912827e-14,
                                         8.35615457719158e-11,
                                         1.79057071815326e-08,
                                         9.61831946787098e-07,
                                         2.14786686564033e-05,
                                         0.000258643425616291,
                                         0.001953125,
                                         0.010168308942307,
                                         0.0388614806412547,
                                         0.113827309103921,
                                         0.263337287398581,
                                         0.491456015243935,
                                         0.750668620766563,
                                         0.947245932468883,

                                         0.953302865574998,
                                         0.776519502749435,
                                         0.534764056295314,
                                         0.309118289109104,
                                         0.148332980433088,
                                         0.0581557596034312,
                                         0.018218531040955,
                                         0.00442080914296558,
                                         0.000795267477139372,
                                         9.95422912225544e-05,
                                         7.88138645883811e-06,
                                         3.39286340848336e-07,
                                         6.12022894102384e-09,
                                         2.78879067393039e-11,
                                         9.6400597996279e-15,
                                         3.29714496278066e-21,
                                         0.0457107814741542,
                                         0.199193744191715,
                                         0.346638943660327,
                                         0.387648985087177,
                                         0.315310890943157,
                                         0.194562242517366,
                                         0.0919107566586694,
                                         0.0328827227209963,
                                         0.00866028603755126,
                                         0.00159823143084462,
                                         0.000190819086145568,
                                         1.2928559729181e-05,
                                         3.95310852164795e-07,
                                         3.48486366746159e-09,
                                         3.04397737122234e-12,
                                         5.56974501046301e-18,
                                         0.000974145605152381,
                                         0.0227099670514823,
                                         0.0998642347009965,
                                         0.216057866567247,
                                         0.29789121941755,
                                         0.289295631605108,
                                         0.206080456592469,
                                         0.108705434641625,
                                         0.0419149287828918,
                                         0.0114048396478536,
                                         0.00205332872036029,
                                         0.000218953251178895,
                                         1.13482080015214e-05,
                                         1.93541075297835e-07,
                                         4.27189523214429e-10,
                                         4.18167297690924e-15,
                                         1.21100489608048e-05,
                                         0.0015103378534825,
                                         0.0167826059563476,
                                         0.0702454678738222,
                                         0.164169783982741,
                                         0.250923873032687,
                                         0.269540523993972,
                                         0.2096290843123,
                                         0.11833739989264,
                                         0.0474739637295832,
                                         0.0128887840105263,
                                         0.00216306437351521,
                                         0.000190034578480861,
                                         6.2701409058538e-06,
                                         3.49716852170135e-08,
                                         1.83139326801538e-12,
                                         9.67792820935891e-08,
                                         6.45722893338871e-05,
                                         0.00181310640087947,
                                         0.0146818576618577,
                                         0.0581625213561258,
                                         0.139912514745722,
                                         0.226634385384579,
                                         0.259876070600716,
                                         0.214777996391175,
                                         0.127038769229853,
                                         0.0520091652921943,
                                         0.0137373183965751,
                                         0.00204574948322875,
                                         0.000130585791402548,
                                         1.84046262241045e-06,
                                         5.15617482795463e-10,
                                         5.1561748279549e-10,
                                         1.84046262241045e-06,
                                         0.000130585791402547,
                                         0.00204574948322876,
                                         0.0137373183965751,
                                         0.0520091652921943,
                                         0.127038769229853,
                                         0.214777996391175,
                                         0.259876070600716,
                                         0.226634385384579,
                                         0.139912514745722,
                                         0.0581625213561258,
                                         0.0146818576618577,
                                         0.00181310640087948,
                                         6.45722893338871e-05,
                                         9.67792820935851e-08,
                                         1.8313932680155e-12,
                                         3.49716852170135e-08,
                                         6.27014090585377e-06,
                                         0.000190034578480862,
                                         0.00216306437351521,
                                         0.0128887840105263,
                                         0.0474739637295832,
                                         0.11833739989264,
                                         0.2096290843123,
                                         0.269540523993972,
                                         0.250923873032687,
                                         0.164169783982741,
                                         0.0702454678738221,
                                         0.0167826059563476,
                                         0.0015103378534825,
                                         1.21100489608045e-05,
                                         4.18167297690954e-15,
                                         4.27189523214429e-10,
                                         1.93541075297834e-07,
                                         1.13482080015215e-05,
                                         0.000218953251178895,
                                         0.00205332872036029,
                                         0.0114048396478536,
                                         0.0419149287828918,
                                         0.108705434641625,
                                         0.206080456592469,
                                         0.289295631605108,
                                         0.29789121941755,
                                         0.216057866567247,
                                         0.0998642347009966,
                                         0.0227099670514823,
                                         0.000974145605152361,
                                         5.56974501046348e-18,
                                         3.04397737122234e-12,
                                         3.48486366746157e-09,
                                         3.95310852164797e-07,
                                         1.2928559729181e-05,
                                         0.000190819086145568,
                                         0.00159823143084462,
                                         0.00866028603755125,
                                         0.0328827227209962,
                                         0.0919107566586694,
                                         0.194562242517366,
                                         0.315310890943157,
                                         0.387648985087177,
                                         0.346638943660327,
                                         0.199193744191715,
                                         0.0457107814741537,
                                         3.29714496278097e-21,
                                         9.6400597996279e-15,
                                         2.78879067393037e-11,
                                         6.12022894102386e-09,
                                         3.39286340848336e-07,
                                         7.88138645883811e-06,
                                         9.95422912225545e-05,
                                         0.000795267477139371,
                                         0.00442080914296557,
                                         0.018218531040955,
                                         0.0581557596034312,
                                         0.148332980433088,
                                         0.309118289109104,
                                         0.534764056295314,
                                         0.776519502749435,
                                         0.953302865574998,

                                         0.958380306170332,
                                         0.798721448632662,
                                         0.573651149207693,
                                         0.353047551563617,
                                         0.184593104009144,
                                         0.0809992224496767,
                                         0.0293290941677352,
                                         0.00856335318361979,
                                         0.001953125,
                                         0.000332869046348808,
                                         3.97682287702796e-05,
                                         3.0267297331653e-06,
                                         1.26101845963926e-07,
                                         2.21586089490036e-09,
                                         9.89837139941849e-12,
                                         3.37514579557141e-15,
                                         1.14568478577711e-21,
                                         0.04083769276333,
                                         0.181767118311709,
                                         0.3288456804156,
                                         0.389681760714785,
                                         0.343086073885203,
                                         0.234843267400552,
                                         0.126736261651028,
                                         0.0537252615624158,
                                         0.017578125,
                                         0.00429757780819562,
                                         0.000745450867926293,
                                         8.45593036193704e-05,
                                         5.49564720956808e-06,
                                         1.6261126831117e-07,
                                         1.39863531682145e-09,
                                         1.20131688559612e-12,
                                         2.17784440253807e-18,
                                         0.000773396132572093,
                                         0.0183845404738358,
                                         0.0837826122314988,
                                         0.191163263307023,
                                         0.283405444581703,
                                         0.302616678560679,
                                         0.243400024190129,
                                         0.149806515589461,
                                         0.0703125,
                                         0.0246599145220503,
                                         0.00621039584920958,
                                         0.00104994520437082,
                                         0.000106447074218876,
                                         5.30366717967475e-06,
                                         8.78340114089731e-08,
                                         1.9003784949405e-10,
                                         1.83995004580959e-15,
                                         8.54396751674061e-06,
                                         0.00108469348078193,
                                         0.0124518088860192,
                                         0.0547035595144176,
                                         0.136562067297722,
                                         0.227470142852168,
                                         0.2726824174259,
                                         0.243668652050026,
                                         0.1640625,
                                         0.0825422264902717,
                                         0.0301811883075765,
                                         0.00760481519573969,
                                         0.00120272241896914,
                                         0.000100906393159936,
                                         3.217642122994e-06,
                                         1.75363589680459e-08,
                                         9.06780161838982e-13,
                                         6.06780554967066e-08,
                                         4.11410862929229e-05,
                                         0.001189667149679,
                                         0.0100633183332449,
                                         0.0423025404644418,
                                         0.109918404086455,
                                         0.196384916202321,
                                         0.254790418839474,
                                         0.24609375,
                                         0.177613210908137,
                                         0.094290483109404,
                                         0.0354099382170062,
                                         0.00873597851954204,
                                         0.00123417167944415,
                                         7.57752248411315e-05,
                                         1.04028711726718e-06,
                                         2.87284676714703e-10,
                                         2.87284676714686e-10,
                                         1.04028711726719e-06,
                                         7.57752248411315e-05,
                                         0.00123417167944416,
                                         0.00873597851954206,
                                         0.0354099382170062,
                                         0.094290483109404,
                                         0.177613210908137,
                                         0.24609375,
                                         0.254790418839474,
                                         0.196384916202321,
                                         0.109918404086455,
                                         0.0423025404644418,
                                         0.0100633183332449,
                                         0.001189667149679,
                                         4.11410862929225e-05,
                                         6.06780554967095e-08,
                                         9.06780161838918e-13,
                                         1.75363589680462e-08,
                                         3.217642122994e-06,
                                         0.000100906393159936,
                                         0.00120272241896914,
                                         0.0076048151957397,
                                         0.0301811883075765,
                                         0.0825422264902717,
                                         0.1640625,
                                         0.243668652050026,
                                         0.2726824174259,
                                         0.227470142852168,
                                         0.136562067297722,
                                         0.0547035595144175,
                                         0.0124518088860192,
                                         0.00108469348078192,
                                         8.54396751674091e-06,
                                         1.83995004580944e-15,
                                         1.90037849494053e-10,
                                         8.78340114089731e-08,
                                         5.30366717967477e-06,
                                         0.000106447074218877,
                                         0.00104994520437083,
                                         0.00621039584920958,
                                         0.0246599145220503,
                                         0.0703125,
                                         0.149806515589461,
                                         0.243400024190129,
                                         0.302616678560679,
                                         0.283405444581703,
                                         0.191163263307023,
                                         0.0837826122314988,
                                         0.0183845404738357,
                                         0.000773396132572111,
                                         2.17784440253786e-18,
                                         1.20131688559614e-12,
                                         1.39863531682145e-09,
                                         1.62611268311171e-07,
                                         5.49564720956809e-06,
                                         8.45593036193705e-05,
                                         0.000745450867926293,
                                         0.00429757780819562,
                                         0.017578125,
                                         0.0537252615624158,
                                         0.126736261651028,
                                         0.234843267400552,
                                         0.343086073885203,
                                         0.389681760714785,
                                         0.3288456804156,
                                         0.181767118311709,
                                         0.0408376927633305,
                                         1.14568478577699e-21,
                                         3.37514579557147e-15,
                                         9.89837139941849e-12,
                                         2.21586089490037e-09,
                                         1.26101845963926e-07,
                                         3.02672973316531e-06,
                                         3.97682287702796e-05,
                                         0.000332869046348808,
                                         0.001953125,
                                         0.00856335318361979,
                                         0.0293290941677352,
                                         0.0809992224496767,
                                         0.184593104009144,
                                         0.353047551563617,
                                         0.573651149207693,
                                         0.798721448632662,
                                         0.958380306170332,

                                         0.962677313910663,
                                         0.817895611384855,
                                         0.608505705582312,
                                         0.394656747783505,
                                         0.22162066099343,
                                         0.106727709473724,
                                         0.0435064274425506,
                                         0.0147507393130043,
                                         0.00406244127467545,
                                         0.000879996318562521,
                                         0.000143363600208042,
                                         1.64732672540766e-05,
                                         1.21294015235435e-06,
                                         4.91649981499378e-08,
                                         8.45111464068134e-10,
                                         3.71260551996076e-12,
                                         1.25144568316588e-15,
                                         4.22102356649563e-22,
                                         0.0366948519833707,
                                         0.166263727385119,
                                         0.310772133888108,
                                         0.386550172195887,
                                         0.363516818498379,
                                         0.271105119523125,
                                         0.163154424799031,
                                         0.079333951690994,
                                         0.030847345727362,
                                         0.00938718051182295,
                                         0.00215912913618917,
                                         0.00035581096620908,
                                         3.86779868922917e-05,
                                         2.42787757125483e-06,
                                         6.98896190948483e-08,
                                         5.88825229179518e-10,
                                         4.98651797405812e-13,
                                         8.96970981362501e-19,
                                         0.000621651555683724,
                                         0.0150215458953183,
                                         0.070540245512075,
                                         0.168271161051957,
                                         0.265006333572846,
                                         0.306066453049615,
                                         0.271932911436825,
                                         0.189636472747589,
                                         0.104103657446379,
                                         0.0445048114168335,
                                         0.0144522617703858,
                                         0.00341567385704345,
                                         0.000548158095988327,
                                         5.3286233185881e-05,
                                         2.56879138428022e-06,
                                         4.15060586491322e-08,
                                         8.83080418825007e-11,
                                         8.47142540699347e-16,
                                         6.14335628971851e-06,
                                         0.000791677932622364,
                                         0.0093400383164308,
                                         0.0427297454399559,
                                         0.112695037482896,
                                         0.201562871298179,
                                         0.264387843447072,
                                         0.264424349230215,
                                         0.204941999241714,
                                         0.123082286327198,
                                         0.0564299693586503,
                                         0.0191271496758387,
                                         0.00453173595711367,
                                         0.000682213097195969,
                                         5.50759240426674e-05,
                                         1.70668501372075e-06,
                                         9.1226269365785e-09,
                                         4.66714595958558e-13,
                                         3.90282299989274e-08,
                                         2.68223560835327e-05,
                                         0.000795014084383486,
                                         0.00697534019561142,
                                         0.0308083049828649,
                                         0.0853335504968222,
                                         0.165247792326867,
                                         0.237025730288406,
                                         0.259364434553004,
                                         0.21882584718245,
                                         0.141644032864359,
                                         0.0688554927813087,
                                         0.0240845111783888,
                                         0.00561486809663214,
                                         0.000759117882925893,
                                         4.51138330811218e-05,
                                         6.05834566664204e-07,
                                         1.65295392834739e-10,
                                         1.65295392834739e-10,
                                         6.05834566664204e-07,
                                         4.51138330811218e-05,
                                         0.000759117882925891,
                                         0.00561486809663214,
                                         0.0240845111783888,
                                         0.0688554927813087,
                                         0.141644032864359,
                                         0.21882584718245,
                                         0.259364434553004,
                                         0.237025730288406,
                                         0.165247792326867,
                                         0.0853335504968222,
                                         0.0308083049828649,
                                         0.00697534019561144,
                                         0.000795014084383486,
                                         2.68223560835327e-05,
                                         3.90282299989274e-08,
                                         4.66714595958558e-13,
                                         9.1226269365785e-09,
                                         1.70668501372075e-06,
                                         5.50759240426672e-05,
                                         0.000682213097195969,
                                         0.00453173595711367,
                                         0.0191271496758387,
                                         0.0564299693586503,
                                         0.123082286327198,
                                         0.204941999241714,
                                         0.264424349230215,
                                         0.264387843447072,
                                         0.201562871298179,
                                         0.112695037482896,
                                         0.042729745439956,
                                         0.0093400383164308,
                                         0.000791677932622364,
                                         6.14335628971851e-06,
                                         8.47142540699347e-16,
                                         8.83080418825007e-11,
                                         4.15060586491322e-08,
                                         2.56879138428021e-06,
                                         5.3286233185881e-05,
                                         0.000548158095988326,
                                         0.00341567385704345,
                                         0.0144522617703858,
                                         0.0445048114168335,
                                         0.104103657446379,
                                         0.189636472747589,
                                         0.271932911436825,
                                         0.306066453049615,
                                         0.265006333572846,
                                         0.168271161051957,
                                         0.070540245512075,
                                         0.0150215458953183,
                                         0.000621651555683724,
                                         8.96970981362501e-19,
                                         4.98651797405812e-13,
                                         5.88825229179518e-10,
                                         6.9889619094848e-08,
                                         2.42787757125483e-06,
                                         3.86779868922916e-05,
                                         0.00035581096620908,
                                         0.00215912913618917,
                                         0.00938718051182295,
                                         0.030847345727362,
                                         0.079333951690994,
                                         0.163154424799031,
                                         0.271105119523125,
                                         0.363516818498379,
                                         0.386550172195887,
                                         0.310772133888108,
                                         0.166263727385119,
                                         0.0366948519833707,
                                         4.22102356649563e-22,
                                         1.25144568316588e-15,
                                         3.71260551996076e-12,
                                         8.45111464068129e-10,
                                         4.91649981499378e-08,
                                         1.21294015235435e-06,
                                         1.64732672540766e-05,
                                         0.000143363600208042,
                                         0.000879996318562521,
                                         0.00406244127467545,
                                         0.0147507393130043,
                                         0.0435064274425506,
                                         0.106727709473724,
                                         0.22162066099343,
                                         0.394656747783505,
                                         0.608505705582312,
                                         0.817895611384855,
                                         0.962677313910663,

                                         0.966345130751693,
                                         0.834544997363187,
                                         0.63973385939736,
                                         0.4337087769054,
                                         0.258631283365172,
                                         0.134629980923689,
                                         0.0605512652688731,
                                         0.0232117623951518,
                                         0.00744835248827616,
                                         0.001953125,
                                         0.00040511136159258,
                                         6.3534575286734e-05,
                                         7.06397145123984e-06,
                                         5.05752237101432e-07,
                                         2.00281888762798e-08,
                                         3.37897934302451e-10,
                                         1.46348542129955e-12,
                                         4.88513991206721e-16,
                                         1.63881911839792e-22,
                                         0.0331450805407356,
                                         0.152469924084304,
                                         0.292981792402822,
                                         0.379660021280277,
                                         0.377405372328941,
                                         0.302401438253441,
                                         0.199231258280367,
                                         0.108444218948088,
                                         0.0485069880338517,
                                         0.017578125,
                                         0.00503866349111065,
                                         0.00110153134817836,
                                         0.000173900046829507,
                                         1.82381491587256e-05,
                                         1.11173089298539e-06,
                                         3.12661134142859e-08,
                                         2.58840437468728e-10,
                                         2.16584613038332e-13,
                                         3.87016271501656e-19,
                                         0.000505269841127986,
                                         0.0123804403457237,
                                         0.059634750655309,
                                         0.147709696699677,
                                         0.244766841143218,
                                         0.301886298464837,
                                         0.291346104089369,
                                         0.225176039305873,
                                         0.140399602421835,
                                         0.0703125,
                                         0.0278531147329687,
                                         0.00848791600159717,
                                         0.00190268960022777,
                                         0.00029230833275843,
                                         2.74268007635062e-05,
                                         1.2858187159755e-06,
                                         2.03466695462182e-08,
                                         4.26771760430174e-11,
                                         4.06204667537187e-16,
                                         4.49308328043373e-06,
                                         0.000586414625989298,
                                         0.00708068016669876,
                                         0.0335227732122794,
                                         0.0926006173579943,
                                         0.175800354458783,
                                         0.248529385609854,
                                         0.27274370807498,
                                         0.237052337052224,
                                         0.1640625,
                                         0.0898150209190093,
                                         0.0381524189926736,
                                         0.0121437546384281,
                                         0.00273286653429386,
                                         0.000394700269797864,
                                         3.08462504840309e-05,
                                         9.32977847826665e-07,
                                         4.90546908595889e-09,
                                         2.48700917884197e-13,
                                         2.56850274412078e-08,
                                         1.78561559071138e-05,
                                         0.000540461822833831,
                                         0.00489086128329377,
                                         0.0225211028980197,
                                         0.0658128296895151,
                                         0.136288983103356,
                                         0.212374211342194,
                                         0.257298395738922,
                                         0.24609375,
                                         0.18618241376021,
                                         0.110244659015977,
                                         0.0498255953912449,
                                         0.0164251794412873,
                                         0.00365152407701096,
                                         0.000475706945861307,
                                         2.75019701557902e-05,
                                         3.62476525025571e-07,
                                         9.78868471796072e-11,
                                         9.78868471796072e-11,
                                         3.62476525025571e-07,
                                         2.75019701557902e-05,
                                         0.000475706945861307,
                                         0.00365152407701096,
                                         0.0164251794412873,
                                         0.0498255953912449,
                                         0.110244659015977,
                                         0.18618241376021,
                                         0.24609375,
                                         0.257298395738922,
                                         0.212374211342194,
                                         0.136288983103356,
                                         0.065812829689515,
                                         0.0225211028980198,
                                         0.00489086128329377,
                                         0.000540461822833831,
                                         1.78561559071138e-05,
                                         2.56850274412078e-08,
                                         2.48700917884197e-13,
                                         4.90546908595889e-09,
                                         9.32977847826665e-07,
                                         3.08462504840309e-05,
                                         0.000394700269797863,
                                         0.00273286653429387,
                                         0.0121437546384281,
                                         0.0381524189926735,
                                         0.0898150209190093,
                                         0.1640625,
                                         0.237052337052224,
                                         0.27274370807498,
                                         0.248529385609854,
                                         0.175800354458783,
                                         0.0926006173579944,
                                         0.0335227732122794,
                                         0.00708068016669876,
                                         0.000586414625989298,
                                         4.49308328043373e-06,
                                         4.06204667537187e-16,
                                         4.26771760430174e-11,
                                         2.03466695462182e-08,
                                         1.2858187159755e-06,
                                         2.74268007635061e-05,
                                         0.00029230833275843,
                                         0.00190268960022777,
                                         0.00848791600159716,
                                         0.0278531147329687,
                                         0.0703125,
                                         0.140399602421835,
                                         0.225176039305873,
                                         0.291346104089369,
                                         0.301886298464837,
                                         0.244766841143218,
                                         0.147709696699677,
                                         0.059634750655309,
                                         0.0123804403457237,
                                         0.000505269841127986,
                                         3.87016271501656e-19,
                                         2.16584613038332e-13,
                                         2.58840437468728e-10,
                                         3.12661134142859e-08,
                                         1.11173089298539e-06,
                                         1.82381491587256e-05,
                                         0.000173900046829507,
                                         0.00110153134817836,
                                         0.00503866349111065,
                                         0.017578125,
                                         0.0485069880338517,
                                         0.108444218948088,
                                         0.199231258280367,
                                         0.302401438253441,
                                         0.377405372328941,
                                         0.379660021280277,
                                         0.292981792402822,
                                         0.152469924084304,
                                         0.0331450805407356,
                                         1.63881911839792e-22,
                                         4.88513991206721e-16,
                                         1.46348542129955e-12,
                                         3.37897934302451e-10,
                                         2.00281888762797e-08,
                                         5.05752237101434e-07,
                                         7.06397145123984e-06,
                                         6.35345752867339e-05,
                                         0.00040511136159258,
                                         0.001953125,
                                         0.00744835248827616,
                                         0.0232117623951518,
                                         0.0605512652688731,
                                         0.134629980923689,
                                         0.258631283365172,
                                         0.4337087769054,
                                         0.63973385939736,
                                         0.834544997363187,
                                         0.966345130751693,

                                         0.969500252540025,
                                         0.849077868147307,
                                         0.667728997078888,
                                         0.470125933409015,
                                         0.295033369280832,
                                         0.164024220409416,
                                         0.0801246534175718,
                                         0.0340231443469457,
                                         0.0123832219731749,
                                         0.00379273623224631,
                                         0.000954003320965617,
                                         0.000190708403866699,
                                         2.89556784598906e-05,
                                         3.13020441710102e-06,
                                         2.18809424063584e-07,
                                         8.49420315098025e-09,
                                         1.4103344924294e-10,
                                         6.03458927396183e-13,
                                         1.99755761878828e-16,
                                         6.67018036452716e-23,
                                         0.0300815629521852,
                                         0.140183140501664,
                                         0.275820028665886,
                                         0.370132821650419,
                                         0.385702722577114,
                                         0.328390264901827,
                                         0.233458343480037,
                                         0.139605112695227,
                                         0.0700956871796734,
                                         0.0292816027162519,
                                         0.0100090463862216,
                                         0.00272896025379432,
                                         0.000571599563366141,
                                         8.70190791424562e-05,
                                         8.85255737434961e-06,
                                         5.26291186170121e-07,
                                         1.45098778741054e-08,
                                         1.18333353570163e-10,
                                         9.80022552139957e-14,
                                         1.74128607018262e-19,
                                         0.00041483018460078,
                                         0.0102863555435936,
                                         0.050637102141779,
                                         0.129514522648562,
                                         0.22410510613092,
                                         0.292206617922687,
                                         0.302322254224997,
                                         0.254592544310493,
                                         0.17634632737871,
                                         0.100474193608132,
                                         0.0466716332741835,
                                         0.0173556974727287,
                                         0.00501496046092873,
                                         0.00107516160832206,
                                         0.00015918016816391,
                                         1.44926242425244e-05,
                                         6.63472411219081e-07,
                                         1.03129784583542e-08,
                                         2.13693004935257e-11,
                                         2.02031996638317e-16,
                                         3.33700683567683e-06,
                                         0.000440295322446271,
                                         0.00542286362114314,
                                         0.0264360238113597,
                                         0.0759569632212813,
                                         0.151672419675054,
                                         0.228374510080601,
                                         0.270836275602571,
                                         0.258796462162889,
                                         0.201108769424274,
                                         0.126949235712979,
                                         0.0643878207344867,
                                         0.0256661029643434,
                                         0.00774907398451173,
                                         0.00166965200051269,
                                         0.000232800957068437,
                                         1.76969642673624e-05,
                                         5.24297567222459e-07,
                                         2.7180743767055e-09,
                                         1.36737293655263e-13,
                                         1.72567225461933e-08,
                                         1.21154928509657e-05,
                                         0.000373338686297954,
                                         0.00346887198768339,
                                         0.0165499965786646,
                                         0.0506101537434212,
                                         0.110902060190773,
                                         0.18521768785727,
                                         0.244154495147721,
                                         0.258774788105428,
                                         0.221983991219318,
                                         0.153560619292955,
                                         0.0844436165203947,
                                         0.0359037937296271,
                                         0.01125841981212,
                                         0.00240401384448774,
                                         0.00030345140537017,
                                         1.71350765234378e-05,
                                         2.22252595078333e-07,
                                         5.94933299411754e-11,
                                         5.94933299411706e-11,
                                         2.22252595078329e-07,
                                         1.71350765234378e-05,
                                         0.000303451405370169,
                                         0.00240401384448774,
                                         0.01125841981212,
                                         0.0359037937296271,
                                         0.0844436165203947,
                                         0.153560619292955,
                                         0.221983991219318,
                                         0.258774788105428,
                                         0.244154495147721,
                                         0.18521768785727,
                                         0.110902060190773,
                                         0.0506101537434211,
                                         0.0165499965786646,
                                         0.0034688719876834,
                                         0.000373338686297954,
                                         1.21154928509658e-05,
                                         1.72567225461944e-08,
                                         1.36737293655249e-13,
                                         2.71807437670545e-09,
                                         5.24297567222459e-07,
                                         1.76969642673623e-05,
                                         0.000232800957068437,
                                         0.00166965200051269,
                                         0.00774907398451173,
                                         0.0256661029643434,
                                         0.0643878207344867,
                                         0.126949235712979,
                                         0.201108769424274,
                                         0.258796462162889,
                                         0.270836275602571,
                                         0.228374510080601,
                                         0.151672419675054,
                                         0.0759569632212813,
                                         0.0264360238113597,
                                         0.00542286362114314,
                                         0.000440295322446275,
                                         3.337006835677e-06,
                                         2.02031996638294e-16,
                                         2.13693004935252e-11,
                                         1.03129784583542e-08,
                                         6.63472411219078e-07,
                                         1.44926242425244e-05,
                                         0.00015918016816391,
                                         0.00107516160832206,
                                         0.00501496046092873,
                                         0.0173556974727287,
                                         0.0466716332741835,
                                         0.100474193608132,
                                         0.17634632737871,
                                         0.254592544310493,
                                         0.302322254224997,
                                         0.292206617922687,
                                         0.22410510613092,
                                         0.129514522648563,
                                         0.050637102141779,
                                         0.0102863555435937,
                                         0.000414830184600794,
                                         1.74128607018239e-19,
                                         9.80022552139933e-14,
                                         1.18333353570163e-10,
                                         1.45098778741053e-08,
                                         5.26291186170121e-07,
                                         8.85255737434963e-06,
                                         8.70190791424562e-05,
                                         0.000571599563366141,
                                         0.00272896025379432,
                                         0.0100090463862216,
                                         0.0292816027162519,
                                         0.0700956871796734,
                                         0.139605112695227,
                                         0.233458343480037,
                                         0.328390264901827,
                                         0.385702722577114,
                                         0.37013282165042,
                                         0.275820028665886,
                                         0.140183140501665,
                                         0.0300815629521857,
                                         6.67018036452619e-23,
                                         1.99755761878823e-16,
                                         6.03458927396183e-13,
                                         1.41033449242939e-10,
                                         8.49420315098025e-09,
                                         2.18809424063585e-07,
                                         3.13020441710102e-06,
                                         2.89556784598906e-05,
                                         0.000190708403866699,
                                         0.000954003320965617,
                                         0.00379273623224631,
                                         0.0123832219731749,
                                         0.0340231443469457,
                                         0.0801246534175718,
                                         0.164024220409416,
                                         0.295033369280832,
                                         0.470125933409015,
                                         0.667728997078888,
                                         0.849077868147307,
                                         0.969500252540025,

                                         0.972233537294956,
                                         0.861826821754566,
                                         0.692856526359391,
                                         0.503935792598157,
                                         0.330402195414323,
                                         0.194301378187792,
                                         0.101811169630528,
                                         0.0471258172473941,
                                         0.0190562523421442,
                                         0.00663585568443234,
                                         0.001953125,
                                         0.000474084437537328,
                                         9.18190843732838e-05,
                                         1.35584524777143e-05,
                                         1.43072537794144e-06,
                                         9.79712299879971e-08,
                                         3.73854423453418e-09,
                                         6.12226922291294e-11,
                                         2.59229024514222e-13,
                                         8.51912663740795e-17,
                                         2.83329951948509e-23,
                                         0.0274202301213695,
                                         0.129219010757554,
                                         0.259485158547622,
                                         0.3588389921494,
                                         0.389366624913519,
                                         0.349148193504701,
                                         0.264789191343279,
                                         0.171416072653655,
                                         0.0948034069391898,
                                         0.0445452629713639,
                                         0.017578125,
                                         0.00572052810311582,
                                         0.00149496672692265,
                                         0.000301927378466852,
                                         4.45591064203065e-05,
                                         4.41620943326559e-06,
                                         2.56963680604259e-07,
                                         6.96422862421643e-09,
                                         5.60659820368998e-11,
                                         4.60228688542743e-14,
                                         8.13723783316347e-20,
                                         0.000343707551765931,
                                         0.00861093477749811,
                                         0.0431915923875212,
                                         0.113564222689665,
                                         0.203935110602507,
                                         0.278843932873933,
                                         0.306071267280261,
                                         0.277116042687271,
                                         0.209617663790779,
                                         0.132899578528814,
                                         0.0703125,
                                         0.0306784935223099,
                                         0.0108180149638119,
                                         0.00298821850609684,
                                         0.000616785488470345,
                                         8.84745257963467e-05,
                                         7.84979737891243e-06,
                                         3.5208789830803e-07,
                                         5.38930451369099e-09,
                                         1.1050190929077e-11,
                                         1.03867340844282e-16,
                                         2.51318146105458e-06,
                                         0.000334727182139195,
                                         0.00419375159510709,
                                         0.0209652589556953,
                                         0.0623077511715038,
                                         0.129906054799834,
                                         0.206377175384674,
                                         0.26132959193034,
                                         0.270363837424305,
                                         0.231292991975816,
                                         0.1640625,
                                         0.0959729246850546,
                                         0.0456646804433749,
                                         0.0172519821374087,
                                         0.00498022100203002,
                                         0.00103395971953299,
                                         0.000139882035474028,
                                         1.03835526728609e-05,
                                         3.02191891593047e-07,
                                         1.54768534691206e-09,
                                         7.73388679373243e-14,
                                         1.18133506260594e-08,
                                         8.36461847822913e-06,
                                         0.000261770476321586,
                                         0.00248813192219903,
                                         0.0122378922397782,
                                         0.038905594993471,
                                         0.0894571875563516,
                                         0.158427287254306,
                                         0.224173256811638,
                                         0.258771127667505,
                                         0.24609375,
                                         0.19300915242405,
                                         0.123916101473461,
                                         0.0640295017525838,
                                         0.0258510124826093,
                                         0.00776789721427686,
                                         0.00160243312329033,
                                         0.000196859018860631,
                                         1.08929965163057e-05,
                                         1.3935098197321e-07,
                                         3.70195454584342e-11,
                                         3.70195454584375e-11,
                                         1.39350981973208e-07,
                                         1.08929965163058e-05,
                                         0.000196859018860631,
                                         0.00160243312329033,
                                         0.00776789721427686,
                                         0.0258510124826093,
                                         0.0640295017525837,
                                         0.123916101473461,
                                         0.19300915242405,
                                         0.24609375,
                                         0.258771127667505,
                                         0.224173256811638,
                                         0.158427287254306,
                                         0.0894571875563516,
                                         0.038905594993471,
                                         0.0122378922397782,
                                         0.00248813192219903,
                                         0.000261770476321585,
                                         8.36461847822925e-06,
                                         1.18133506260586e-08,
                                         7.73388679373326e-14,
                                         1.54768534691203e-09,
                                         3.0219189159305e-07,
                                         1.03835526728609e-05,
                                         0.000139882035474028,
                                         0.00103395971953299,
                                         0.00498022100203002,
                                         0.0172519821374087,
                                         0.0456646804433749,
                                         0.0959729246850546,
                                         0.1640625,
                                         0.231292991975816,
                                         0.270363837424305,
                                         0.26132959193034,
                                         0.206377175384674,
                                         0.129906054799834,
                                         0.0623077511715038,
                                         0.0209652589556953,
                                         0.00419375159510707,
                                         0.000334727182139198,
                                         2.51318146105445e-06,
                                         1.03867340844295e-16,
                                         1.10501909290767e-11,
                                         5.38930451369104e-09,
                                         3.5208789830803e-07,
                                         7.84979737891243e-06,
                                         8.84745257963467e-05,
                                         0.000616785488470345,
                                         0.00298821850609684,
                                         0.0108180149638119,
                                         0.0306784935223099,
                                         0.0703125,
                                         0.132899578528814,
                                         0.209617663790779,
                                         0.277116042687271,
                                         0.306071267280261,
                                         0.278843932873933,
                                         0.203935110602507,
                                         0.113564222689665,
                                         0.0431915923875211,
                                         0.00861093477749817,
                                         0.000343707551765919,
                                         8.13723783316463e-20,
                                         4.6022868854273e-14,
                                         5.60659820369004e-11,
                                         6.96422862421643e-09,
                                         2.56963680604259e-07,
                                         4.41620943326559e-06,
                                         4.45591064203065e-05,
                                         0.000301927378466852,
                                         0.00149496672692265,
                                         0.00572052810311582,
                                         0.017578125,
                                         0.0445452629713639,
                                         0.0948034069391898,
                                         0.171416072653655,
                                         0.264789191343279,
                                         0.349148193504701,
                                         0.389366624913519,
                                         0.3588389921494,
                                         0.259485158547622,
                                         0.129219010757555,
                                         0.027420230121369,
                                         2.83329951948554e-23,
                                         8.5191266374077e-17,
                                         2.59229024514226e-13,
                                         6.12226922291294e-11,
                                         3.73854423453418e-09,
                                         9.79712299879971e-08,
                                         1.43072537794144e-06,
                                         1.35584524777143e-05,
                                         9.18190843732838e-05,
                                         0.000474084437537328,
                                         0.001953125,
                                         0.00663585568443234,
                                         0.0190562523421442,
                                         0.0471258172473941,
                                         0.101811169630528,
                                         0.194301378187792,
                                         0.330402195414323,
                                         0.503935792598157,
                                         0.692856526359391,
                                         0.861826821754566,
                                         0.972233537294956,

                                         0.974616658711705,
                                         0.873064049696355,
                                         0.715447529495134,
                                         0.535232142709016,
                                         0.364449348393076,
                                         0.224944209396603,
                                         0.125169732118079,
                                         0.0623552298574436,
                                         0.0275623188070307,
                                         0.0106876152955132,
                                         0.0035828737247701,
                                         0.00101899538596918,
                                         0.000239853151911141,
                                         4.52006103899493e-05,
                                         6.51569211636894e-06,
                                         6.73309888688083e-07,
                                         4.5289124482667e-08,
                                         1.70266372108628e-09,
                                         2.75509343045049e-11,
                                         1.15597821212238e-13,
                                         3.77512390534628e-17,
                                         1.25116580029229e-23,
                                         0.025094250702827,
                                         0.119413554715562,
                                         0.244077801201264,
                                         0.346440366884036,
                                         0.389284986515362,
                                         0.365013005895598,
                                         0.29259425562497,
                                         0.202671089299678,
                                         0.121645141115628,
                                         0.0630824104305571,
                                         0.0280414687778872,
                                         0.0105460051918296,
                                         0.00329156913949924,
                                         0.00082956477682189,
                                         0.000162377801200136,
                                         2.33310099536358e-05,
                                         2.26071295283782e-06,
                                         1.29116999420406e-07,
                                         3.44774428124332e-09,
                                         2.74463232245468e-11,
                                         2.23567519443995e-14,
                                         3.9360398022431e-20,
                                         0.000287165403367117,
                                         0.00725903064157441,
                                         0.0370080547618342,
                                         0.0996626142645101,
                                         0.184805806829698,
                                         0.263244521989266,
                                         0.303983317429305,
                                         0.292771097552771,
                                         0.238611440828838,
                                         0.165482940173645,
                                         0.0975410654655965,
                                         0.0485088775943182,
                                         0.0200760383107107,
                                         0.00676665118758931,
                                         0.0017984991618313,
                                         0.000359310217315484,
                                         5.01550193194809e-05,
                                         4.35166493871874e-06,
                                         1.91757007785675e-07,
                                         2.89625089103149e-09,
                                         5.88442033851673e-12,
                                         5.50327962439248e-17,
                                         1.91693238514046e-06,
                                         0.000257407043576286,
                                         0.00327326395299634,
                                         0.0167244888828465,
                                         0.0511776530363735,
                                         0.110745768111124,
                                         0.18422582025339,
                                         0.246706955511461,
                                         0.273026344089694,
                                         0.253229892864535,
                                         0.197920612660827,
                                         0.130158119422202,
                                         0.0714282226349014,
                                         0.0321969008144076,
                                         0.0116201201141241,
                                         0.00322791722142554,
                                         0.000649082911302336,
                                         8.55547776868799e-05,
                                         6.22134428696635e-06,
                                         1.78280967078347e-07,
                                         9.03473589116944e-10,
                                         4.48849217786865e-14,
                                         8.226137155756e-09,
                                         5.86781884119464e-06,
                                         0.000186114526330692,
                                         0.00180421341264355,
                                         0.00911086458698825,
                                         0.029950867829918,
                                         0.0717737638216057,
                                         0.133643875383096,
                                         0.200831724812422,
                                         0.249110014929661,
                                         0.258171936215112,
                                         0.224510045561488,
                                         0.163371443069066,
                                         0.0984847129571788,
                                         0.0482642396262792,
                                         0.018641878994067,
                                         0.00540008284479155,
                                         0.00108130337621436,
                                         0.000129757270358261,
                                         7.05485766067229e-06,
                                         8.91747106701963e-08,
                                         2.35338964934684e-11,
                                         2.35338964934684e-11,
                                         8.91747106701963e-08,
                                         7.05485766067224e-06,
                                         0.000129757270358262,
                                         0.00108130337621436,
                                         0.00540008284479155,
                                         0.018641878994067,
                                         0.0482642396262792,
                                         0.0984847129571788,
                                         0.163371443069066,
                                         0.224510045561488,
                                         0.258171936215112,
                                         0.249110014929662,
                                         0.200831724812422,
                                         0.133643875383096,
                                         0.0717737638216057,
                                         0.029950867829918,
                                         0.00911086458698823,
                                         0.00180421341264354,
                                         0.000186114526330693,
                                         5.86781884119464e-06,
                                         8.226137155756e-09,
                                         4.48849217786865e-14,
                                         9.03473589116944e-10,
                                         1.78280967078345e-07,
                                         6.22134428696638e-06,
                                         8.55547776868801e-05,
                                         0.000649082911302336,
                                         0.00322791722142554,
                                         0.0116201201141241,
                                         0.0321969008144076,
                                         0.0714282226349013,
                                         0.130158119422202,
                                         0.197920612660827,
                                         0.253229892864535,
                                         0.273026344089694,
                                         0.246706955511461,
                                         0.18422582025339,
                                         0.110745768111124,
                                         0.0511776530363734,
                                         0.0167244888828464,
                                         0.00327326395299636,
                                         0.000257407043576285,
                                         1.91693238514046e-06,
                                         5.50327962439248e-17,
                                         5.88442033851673e-12,
                                         2.89625089103146e-09,
                                         1.91757007785676e-07,
                                         4.35166493871875e-06,
                                         5.01550193194809e-05,
                                         0.000359310217315484,
                                         0.0017984991618313,
                                         0.00676665118758931,
                                         0.0200760383107107,
                                         0.0485088775943182,
                                         0.0975410654655965,
                                         0.165482940173645,
                                         0.238611440828838,
                                         0.292771097552771,
                                         0.303983317429305,
                                         0.263244521989266,
                                         0.184805806829698,
                                         0.09966261426451,
                                         0.0370080547618343,
                                         0.00725903064157441,
                                         0.000287165403367117,
                                         3.9360398022431e-20,
                                         2.23567519443995e-14,
                                         2.74463232245464e-11,
                                         3.44774428124334e-09,
                                         1.29116999420406e-07,
                                         2.26071295283782e-06,
                                         2.33310099536358e-05,
                                         0.000162377801200136,
                                         0.00082956477682189,
                                         0.00329156913949923,
                                         0.0105460051918296,
                                         0.0280414687778872,
                                         0.0630824104305571,
                                         0.121645141115628,
                                         0.202671089299678,
                                         0.29259425562497,
                                         0.365013005895598,
                                         0.389284986515361,
                                         0.346440366884036,
                                         0.244077801201264,
                                         0.119413554715562,
                                         0.025094250702827,
                                         1.25116580029229e-23,
                                         3.77512390534628e-17,
                                         1.15597821212236e-13,
                                         2.75509343045051e-11,
                                         1.70266372108629e-09,
                                         4.5289124482667e-08,
                                         6.73309888688083e-07,
                                         6.51569211636894e-06,
                                         4.52006103899493e-05,
                                         0.00023985315191114,
                                         0.00101899538596918,
                                         0.0035828737247701,
                                         0.0106876152955132,
                                         0.0275623188070307,
                                         0.0623552298574436,
                                         0.125169732118079,
                                         0.224944209396603,
                                         0.364449348393076,
                                         0.535232142709016,
                                         0.715447529495134,
                                         0.873064049696355,
                                         0.974616658711705,

                                         0.976706749418275,
                                         0.883013421630562,
                                         0.735797419760533,
                                         0.564147634373248,
                                         0.396993344705972,
                                         0.255530569683225,
                                         0.149769639842189,
                                         0.0794752535173561,
                                         0.0379061487201395,
                                         0.0161017561949927,
                                         0.0060212287408705,
                                         0.001953125,
                                         0.000539192592733389,
                                         0.000123569060658953,
                                         2.27388836579774e-05,
                                         3.20973640405379e-06,
                                         3.25681535964885e-07,
                                         2.15672339382966e-08,
                                         8.00346075363997e-10,
                                         1.28155603933718e-11,
                                         5.33444986831158e-14,
                                         1.73253918814211e-17,
                                         5.72456113923135e-24,
                                         0.0230499995662848,
                                         0.11062286421552,
                                         0.229634739645169,
                                         0.333431732192411,
                                         0.386241084858833,
                                         0.376464854132803,
                                         0.316581193304889,
                                         0.232422726795425,
                                         0.149609365204196,
                                         0.0843567445518452,
                                         0.0414466776438459,
                                         0.017578125,
                                         0.00634489353029577,
                                         0.00191050509099366,
                                         0.000466664796444511,
                                         8.89012191458712e-05,
                                         1.24800645062849e-05,
                                         1.18576299660849e-06,
                                         6.66327283781101e-08,
                                         1.75633947781612e-09,
                                         1.38450754818655e-11,
                                         1.12018690499317e-14,
                                         1.96480965811461e-20,
                                         0.000241765950423445,
                                         0.00615942221260558,
                                         0.0318518308531573,
                                         0.087586781472624,
                                         0.167013351871923,
                                         0.246503729384927,
                                         0.29741572030205,
                                         0.302094448216995,
                                         0.262437190711377,
                                         0.19641916392251,
                                         0.12679782458153,
                                         0.0703125,
                                         0.0331835039224164,
                                         0.0131281682872478,
                                         0.0042565567034128,
                                         0.00109436840782875,
                                         0.000212548480485652,
                                         2.89746914364752e-05,
                                         2.46555590986296e-06,
                                         1.0697857454012e-07,
                                         1.59704994958844e-09,
                                         3.21895865189322e-12,
                                         2.9972036467867e-17,
                                         1.47923149380999e-06,
                                         0.000200056121486318,
                                         0.00257719930640744,
                                         0.0134210656526396,
                                         0.0421270172047741,
                                         0.0941541234494396,
                                         0.162989460490785,
                                         0.229046515915445,
                                         0.268539869825518,
                                         0.266787021720342,
                                         0.226282347641461,
                                         0.1640625,
                                         0.101236457464819,
                                         0.0526231528918452,
                                         0.0226479344760009,
                                         0.00785843693045006,
                                         0.00211162100256013,
                                         0.000413006170125706,
                                         5.32180439557594e-05,
                                         3.80103566187481e-06,
                                         1.07462877406895e-07,
                                         5.39581559223739e-10,
                                         2.6670356100601e-14,
                                         5.81823993589731e-09,
                                         4.177131894892e-06,
                                         0.000134052860168403,
                                         0.00132205617104682,
                                         0.00683100654961429,
                                         0.0231190332036961,
                                         0.0574208475013835,
                                         0.111639815405839,
                                         0.176647152740454,
                                         0.232948550957951,
                                         0.25959960230732,
                                         0.24609375,
                                         0.198548271574707,
                                         0.135601367321613,
                                         0.0774663779387994,
                                         0.0362763238551099,
                                         0.0134861633296162,
                                         0.00378450195411515,
                                         0.000738443775943152,
                                         8.68203546394848e-05,
                                         4.64850073867707e-06,
                                         5.81451202715547e-08,
                                         1.52565329106471e-11,
                                         1.52565329106488e-11,
                                         5.81451202715547e-08,
                                         4.64850073867704e-06,
                                         8.68203546394848e-05,
                                         0.000738443775943152,
                                         0.00378450195411514,
                                         0.0134861633296162,
                                         0.0362763238551099,
                                         0.0774663779387994,
                                         0.135601367321613,
                                         0.198548271574707,
                                         0.24609375,
                                         0.25959960230732,
                                         0.232948550957951,
                                         0.176647152740454,
                                         0.11163981540584,
                                         0.0574208475013834,
                                         0.0231190332036961,
                                         0.00683100654961429,
                                         0.00132205617104682,
                                         0.000134052860168404,
                                         4.177131894892e-06,
                                         5.81823993589682e-09,
                                         2.66703561006044e-14,
                                         5.39581559223739e-10,
                                         1.07462877406894e-07,
                                         3.80103566187481e-06,
                                         5.32180439557594e-05,
                                         0.000413006170125705,
                                         0.00211162100256013,
                                         0.00785843693045005,
                                         0.0226479344760009,
                                         0.0526231528918452,
                                         0.101236457464819,
                                         0.1640625,
                                         0.226282347641461,
                                         0.266787021720342,
                                         0.268539869825518,
                                         0.229046515915446,
                                         0.162989460490785,
                                         0.0941541234494397,
                                         0.0421270172047741,
                                         0.0134210656526396,
                                         0.00257719930640745,
                                         0.000200056121486318,
                                         1.4792314938099e-06,
                                         2.99720364678714e-17,
                                         3.21895865189322e-12,
                                         1.59704994958843e-09,
                                         1.0697857454012e-07,
                                         2.46555590986296e-06,
                                         2.89746914364751e-05,
                                         0.000212548480485652,
                                         0.00109436840782874,
                                         0.0042565567034128,
                                         0.0131281682872478,
                                         0.0331835039224164,
                                         0.0703125,
                                         0.12679782458153,
                                         0.19641916392251,
                                         0.262437190711377,
                                         0.302094448216995,
                                         0.29741572030205,
                                         0.246503729384928,
                                         0.167013351871923,
                                         0.087586781472624,
                                         0.0318518308531574,
                                         0.00615942221260558,
                                         0.000241765950423435,
                                         1.96480965811495e-20,
                                         1.12018690499317e-14,
                                         1.38450754818654e-11,
                                         1.75633947781612e-09,
                                         6.66327283781101e-08,
                                         1.18576299660849e-06,
                                         1.24800645062849e-05,
                                         8.8901219145871e-05,
                                         0.000466664796444511,
                                         0.00191050509099366,
                                         0.00634489353029577,
                                         0.017578125,
                                         0.0414466776438459,
                                         0.0843567445518452,
                                         0.149609365204196,
                                         0.232422726795425,
                                         0.316581193304889,
                                         0.376464854132803,
                                         0.386241084858833,
                                         0.333431732192411,
                                         0.229634739645169,
                                         0.11062286421552,
                                         0.0230499995662844,
                                         5.72456113923244e-24,
                                         1.73253918814211e-17,
                                         5.3344498683115e-14,
                                         1.28155603933718e-11,
                                         8.00346075363997e-10,
                                         2.15672339382966e-08,
                                         3.25681535964886e-07,
                                         3.20973640405379e-06,
                                         2.27388836579774e-05,
                                         0.000123569060658953,
                                         0.000539192592733389,
                                         0.001953125,
                                         0.0060212287408705,
                                         0.0161017561949927,
                                         0.0379061487201395,
                                         0.0794752535173561,
                                         0.149769639842189,
                                         0.255530569683225,
                                         0.396993344705972,
                                         0.564147634373248,
                                         0.735797419760533,
                                         0.883013421630562,
                                         0.976706749418275,

                                         0.978549789347802,
                                         0.891860021751249,
                                         0.754167275467319,
                                         0.590835117846358,
                                         0.427934002739746,
                                         0.285727631684369,
                                         0.175213279433183,
                                         0.0982092773968659,
                                         0.0500160654579999,
                                         0.0229714638685451,
                                         0.00942596829901793,
                                         0.00341518066835026,
                                         0.00107640347478525,
                                         0.00028952921178113,
                                         6.48188978185254e-05,
                                         1.16816352835843e-05,
                                         1.6188637792307e-06,
                                         1.61649345146306e-07,
                                         1.05589982797579e-08,
                                         3.87382020348728e-10,
                                         6.14611589288535e-12,
                                         2.54043621500541e-14,
                                         8.21108533841033e-18,
                                         2.70579026459823e-24,
                                         0.0212440733872058,
                                         0.102721619625463,
                                         0.216151946098138,
                                         0.320177209498463,
                                         0.380903669880102,
                                         0.384043447407425,
                                         0.336709013486643,
                                         0.259990214173877,
                                         0.177762549148296,
                                         0.107684908786698,
                                         0.0576100279238728,
                                         0.0270358976559502,
                                         0.0110136938378086,
                                         0.00383711924623756,
                                         0.00112000635805999,
                                         0.000266230566909445,
                                         4.95325673693478e-05,
                                         6.81351544061705e-06,
                                         6.36325928578974e-07,
                                         3.52521912623293e-08,
                                         9.18674474670583e-10,
                                         7.17962643231725e-12,
                                         5.77458027621246e-15,
                                         1.00954174870232e-20,
                                         0.000204979363450245,
                                         0.00525828960755472,
                                         0.0275339232919359,
                                         0.077113783438445,
                                         0.150685339119523,
                                         0.229417191823365,
                                         0.287580840219192,
                                         0.305899642784826,
                                         0.280794434884442,
                                         0.224356418840312,
                                         0.156490392032082,
                                         0.0951229665484662,
                                         0.0500850821150531,
                                         0.0226014179199098,
                                         0.00860115583100336,
                                         0.00269668160649952,
                                         0.000673579488054378,
                                         0.000127639772420865,
                                         1.70433123232964e-05,
                                         1.42577257764786e-06,
                                         6.10295315046572e-08,
                                         9.01805505389795e-10,
                                         1.80491918977629e-12,
                                         1.67406346753235e-17,
                                         1.15371697132717e-06,
                                         0.00015701601075162,
                                         0.00204594446126108,
                                         0.0108340391764465,
                                         0.0347731181895651,
                                         0.0799444563316931,
                                         0.143278804160278,
                                         0.20995089573933,
                                         0.258734008409388,
                                         0.272670998110703,
                                         0.247967101339555,
                                         0.195230097901966,
                                         0.13286193542247,
                                         0.077657404215396,
                                         0.0385309696144217,
                                         0.0159337582202899,
                                         0.00534322745580544,
                                         0.00139481815615219,
                                         0.000266284065723303,
                                         3.36380793946429e-05,
                                         2.36502217130961e-06,
                                         6.60755384295351e-08,
                                         3.29087885765152e-10,
                                         1.61933367940633e-14,
                                         4.17448491528723e-09,
                                         3.01410134217117e-06,
                                         9.77313856169503e-05,
                                         0.000978505487779104,
                                         0.00515858685637683,
                                         0.0179087478226182,
                                         0.0458900384704673,
                                         0.0926341173259455,
                                         0.15326147863707,
                                         0.213036427342759,
                                         0.252589286306389,
                                         0.257586223531984,
                                         0.226572518843166,
                                         0.171531753505759,
                                         0.11096283234968,
                                         0.0605231114338218,
                                         0.027247894204148,
                                         0.00979859113687756,
                                         0.00267455066755711,
                                         0.000510183723141426,
                                         5.89175759855966e-05,
                                         3.11231118128581e-06,
                                         3.8572741116824e-08,
                                         1.00696703154803e-11,
                                         1.00696703154814e-11,
                                         3.8572741116824e-08,
                                         3.11231118128584e-06,
                                         5.89175759855966e-05,
                                         0.000510183723141426,
                                         0.00267455066755711,
                                         0.00979859113687756,
                                         0.027247894204148,
                                         0.0605231114338218,
                                         0.11096283234968,
                                         0.171531753505759,
                                         0.226572518843166,
                                         0.257586223531985,
                                         0.252589286306389,
                                         0.213036427342759,
                                         0.15326147863707,
                                         0.0926341173259455,
                                         0.0458900384704673,
                                         0.0179087478226181,
                                         0.00515858685637683,
                                         0.000978505487779104,
                                         9.77313856169496e-05,
                                         3.01410134217116e-06,
                                         4.17448491528684e-09,
                                         1.61933367940655e-14,
                                         3.29087885765152e-10,
                                         6.60755384295358e-08,
                                         2.36502217130961e-06,
                                         3.36380793946429e-05,
                                         0.000266284065723304,
                                         0.00139481815615219,
                                         0.00534322745580545,
                                         0.0159337582202899,
                                         0.0385309696144217,
                                         0.077657404215396,
                                         0.13286193542247,
                                         0.195230097901966,
                                         0.247967101339555,
                                         0.272670998110703,
                                         0.258734008409388,
                                         0.209950895739329,
                                         0.143278804160278,
                                         0.079944456331693,
                                         0.0347731181895651,
                                         0.0108340391764465,
                                         0.00204594446126107,
                                         0.00015701601075162,
                                         1.15371697132709e-06,
                                         1.67406346753262e-17,
                                         1.80491918977629e-12,
                                         9.01805505389807e-10,
                                         6.10295315046572e-08,
                                         1.42577257764786e-06,
                                         1.70433123232964e-05,
                                         0.000127639772420865,
                                         0.000673579488054379,
                                         0.00269668160649952,
                                         0.00860115583100336,
                                         0.0226014179199098,
                                         0.0500850821150531,
                                         0.0951229665484662,
                                         0.156490392032082,
                                         0.224356418840312,
                                         0.280794434884442,
                                         0.305899642784826,
                                         0.287580840219192,
                                         0.229417191823364,
                                         0.150685339119523,
                                         0.077113783438445,
                                         0.0275339232919358,
                                         0.00525828960755472,
                                         0.000204979363450235,
                                         1.0095417487025e-20,
                                         5.77458027621246e-15,
                                         7.17962643231735e-12,
                                         9.18674474670583e-10,
                                         3.52521912623293e-08,
                                         6.36325928578977e-07,
                                         6.81351544061705e-06,
                                         4.95325673693479e-05,
                                         0.000266230566909445,
                                         0.00112000635805999,
                                         0.00383711924623756,
                                         0.0110136938378085,
                                         0.0270358976559502,
                                         0.0576100279238728,
                                         0.107684908786698,
                                         0.177762549148296,
                                         0.259990214173877,
                                         0.336709013486643,
                                         0.384043447407425,
                                         0.380903669880102,
                                         0.320177209498463,
                                         0.216151946098137,
                                         0.102721619625463,
                                         0.0212440733872054,
                                         2.70579026459879e-24,
                                         8.21108533841033e-18,
                                         2.54043621500545e-14,
                                         6.14611589288535e-12,
                                         3.87382020348728e-10,
                                         1.0558998279758e-08,
                                         1.61649345146306e-07,
                                         1.61886377923071e-06,
                                         1.16816352835843e-05,
                                         6.48188978185254e-05,
                                         0.00028952921178113,
                                         0.00107640347478525,
                                         0.00341518066835026,
                                         0.00942596829901793,
                                         0.0229714638685451,
                                         0.0500160654579999,
                                         0.0982092773968659,
                                         0.175213279433183,
                                         0.285727631684369,
                                         0.427934002739746,
                                         0.590835117846358,
                                         0.754167275467319,
                                         0.891860021751249,
                                         0.978549789347802,

                                         0.0009765625,
                                         0.009765625,
                                         0.0439453125,
                                         0.1171875,
                                         0.205078125,
                                         0.24609375,
                                         0.205078125,
                                         0.1171875,
                                         0.0439453125,
                                         0.009765625,
                                         0.0009765625,

                                         0.0931068182589887,
                                         1.77625785062455e-07,
                                         0.249478967623273,
                                         6.6290845458739e-06,
                                         0.300814595564428,
                                         0.00011133036149792,
                                         0.214941407874622,
                                         0.00110797484142732,
                                         0.100788409155024,
                                         0.00723628220300098,
                                         0.0324074074074074,
                                         0.0324074074074074,
                                         0.00723628220300098,
                                         0.100788409155024,
                                         0.00110797484142732,
                                         0.214941407874622,
                                         0.00011133036149792,
                                         0.300814595564428,
                                         6.6290845458739e-06,
                                         0.249478967623273,
                                         1.77625785062455e-07,
                                         0.0931068182589887,

                                         0.302479999669399,
                                         0.0009765625,
                                         3.30600370633756e-10,
                                         0.384199973971888,
                                         0.009765625,
                                         2.60281121224955e-08,
                                         0.219599077864981,
                                         0.0439453125,
                                         9.22135019731318e-07,
                                         0.0743806401235915,
                                         0.1171875,
                                         1.93598764087981e-05,
                                         0.0165332650270357,
                                         0.205078125,
                                         0.000266734972964427,
                                         0.00252000000000002,
                                         0.24609375,
                                         0.00252000000000002,
                                         0.000266734972964427,
                                         0.205078125,
                                         0.0165332650270357,
                                         1.93598764087981e-05,
                                         0.1171875,
                                         0.0743806401235915,
                                         9.22135019731318e-07,
                                         0.0439453125,
                                         0.219599077864981,
                                         2.60281121224955e-08,
                                         0.009765625,
                                         0.384199973971888,
                                         3.30600370633756e-10,
                                         0.0009765625,
                                         0.302479999669399,

                                         0.486947194682945,
                                         0.0182257995178667,
                                         1.53201893980771e-05,
                                         2.60367641155454e-12,
                                         0.363322573910166,
                                         0.0897727116006939,
                                         0.000311032935918293,
                                         3.489606799885e-10,
                                         0.121987522198163,
                                         0.19898237567622,
                                         0.00284158818931722,
                                         2.10464326661081e-08,
                                         0.0242713904280416,
                                         0.26136118978215,
                                         0.0153841007962998,
                                         7.52205860083538e-07,
                                         0.00316915309159211,
                                         0.225287399376478,
                                         0.0546578331860515,
                                         1.76426674108627e-05,
                                         0.000283749417823995,
                                         0.133160648749606,
                                         0.133160648749606,
                                         0.000283749417823996,
                                         1.76426674108626e-05,
                                         0.0546578331860515,
                                         0.225287399376478,
                                         0.00316915309159212,
                                         7.52205860083534e-07,
                                         0.0153841007962998,
                                         0.26136118978215,
                                         0.0242713904280416,
                                         2.10464326661079e-08,
                                         0.00284158818931722,
                                         0.19898237567622,
                                         0.121987522198163,
                                         3.48960679988498e-10,
                                         0.000311032935918293,
                                         0.0897727116006939,
                                         0.363322573910167,
                                         2.60367641155452e-12,
                                         1.53201893980771e-05,
                                         0.0182257995178667,
                                         0.486947194682945,

                                         0.618498680869844,
                                         0.0725418146841953,
                                         0.0009765625,
                                         4.28258404388968e-07,
                                         5.16013971890298e-14,
                                         0.304418503057707,
                                         0.21762068022717,
                                         0.009765625,
                                         1.42755926393134e-05,
                                         1.04840526353964e-11,
                                         0.0674242040308062,
                                         0.293781487283146,
                                         0.0439453125,
                                         0.000214138577087528,
                                         9.5853822846309e-10,
                                         0.00884946118539785,
                                         0.23502004512031,
                                         0.1171875,
                                         0.00190349568611526,
                                         5.19332978701235e-08,
                                         0.000762231944535506,
                                         0.123382822776529,
                                         0.205078125,
                                         0.0111039679017566,
                                         1.84651007841844e-06,
                                         4.5019499258158e-05,
                                         0.0444168438926464,
                                         0.24609375,
                                         0.0444168438926464,
                                         4.50194992581583e-05,
                                         1.84651007841843e-06,
                                         0.0111039679017566,
                                         0.205078125,
                                         0.123382822776529,
                                         0.00076223194453551,
                                         5.19332978701231e-08,
                                         0.00190349568611526,
                                         0.1171875,
                                         0.23502004512031,
                                         0.00884946118539788,
                                         9.58538228463081e-10,
                                         0.000214138577087528,
                                         0.0439453125,
                                         0.293781487283146,
                                         0.0674242040308064,
                                         1.04840526353963e-11,
                                         1.42755926393134e-05,
                                         0.009765625,
                                         0.21762068022717,
                                         0.304418503057708,
                                         5.16013971890292e-14,
                                         4.28258404388968e-07,
                                         0.0009765625,
                                         0.0725418146841953,
                                         0.618498680869844,

                                         0.709294443061681,
                                         0.156294540172398,
                                         0.00829999961790695,
                                         6.39321531853861e-05,
                                         1.94542161479298e-08,
                                         1.92618848691545e-15,
                                         0.247864186011291,
                                         0.318750445238079,
                                         0.0510202048829451,
                                         0.0010400523640156,
                                         9.53908555322669e-07,
                                         5.51202984200498e-13,
                                         0.0389774583582405,
                                         0.292529929723274,
                                         0.141129836356469,
                                         0.00761383732127709,
                                         2.10480692847949e-05,
                                         7.09801399680946e-11,
                                         0.00363219763273559,
                                         0.159091153991528,
                                         0.231340513975248,
                                         0.0330299715585972,
                                         0.000275216206243659,
                                         5.41649518174078e-09,
                                         0.000222123619793501,
                                         0.0567794359268929,
                                         0.248859297435835,
                                         0.0940333963580539,
                                         0.00236159303710944,
                                         2.71249650521603e-07,
                                         9.31457857918842e-06,
                                         0.0138956642724186,
                                         0.183568957976467,
                                         0.183568957976467,
                                         0.0138956642724186,
                                         9.31457857918842e-06,
                                         2.71249650521603e-07,
                                         0.00236159303710944,
                                         0.0940333963580539,
                                         0.248859297435835,
                                         0.0567794359268928,
                                         0.000222123619793501,
                                         5.41649518174078e-09,
                                         0.00027521620624366,
                                         0.0330299715585972,
                                         0.231340513975248,
                                         0.159091153991528,
                                         0.00363219763273559,
                                         7.09801399680946e-11,
                                         2.10480692847949e-05,
                                         0.00761383732127709,
                                         0.141129836356469,
                                         0.292529929723274,
                                         0.0389774583582405,
                                         5.51202984200498e-13,
                                         9.53908555322672e-07,
                                         0.0010400523640156,
                                         0.0510202048829451,
                                         0.318750445238079,
                                         0.247864186011291,
                                         1.92618848691545e-15,
                                         1.94542161479299e-08,
                                         6.39321531853861e-05,
                                         0.00829999961790695,
                                         0.156294540172398,
                                         0.709294443061681,

                                         0.772785364063438,
                                         0.250618196918555,
                                         0.0294492960050725,
                                         0.0009765625,
                                         5.35422353688742e-06,
                                         1.29951575909081e-09,
                                         1.13815840211789e-16,
                                         0.201777747856344,
                                         0.371954224882065,
                                         0.124462085975776,
                                         0.009765625,
                                         0.000126687667637045,
                                         8.75597787641342e-08,
                                         4.35901463113136e-14,
                                         0.0237082864667015,
                                         0.248415622643954,
                                         0.236707352163797,
                                         0.0439453125,
                                         0.00134891534866512,
                                         2.65485174890424e-06,
                                         7.5125341372317e-12,
                                         0.00165075750525087,
                                         0.098316074948758,
                                         0.266773469998823,
                                         0.1171875,
                                         0.00851120930033936,
                                         4.77015083317429e-05,
                                         7.67256910464728e-10,
                                         7.54285416782925e-05,
                                         0.0255352124570861,
                                         0.197307159628971,
                                         0.205078125,
                                         0.035242536724582,
                                         0.000562462101924716,
                                         5.14238366256899e-08,
                                         2.36336793707985e-06,
                                         0.00454776082828218,
                                         0.1000659329628,
                                         0.24609375,
                                         0.1000659329628,
                                         0.00454776082828218,
                                         2.36336793707987e-06,
                                         5.14238366256893e-08,
                                         0.000562462101924716,
                                         0.035242536724582,
                                         0.205078125,
                                         0.197307159628971,
                                         0.0255352124570861,
                                         7.54285416782932e-05,
                                         7.67256910464717e-10,
                                         4.77015083317429e-05,
                                         0.00851120930033936,
                                         0.1171875,
                                         0.266773469998823,
                                         0.098316074948758,
                                         0.00165075750525088,
                                         7.51253413723157e-12,
                                         2.65485174890424e-06,
                                         0.00134891534866512,
                                         0.0439453125,
                                         0.236707352163797,
                                         0.248415622643954,
                                         0.0237082864667016,
                                         4.35901463113127e-14,
                                         8.75597787641342e-08,
                                         0.000126687667637045,
                                         0.009765625,
                                         0.124462085975776,
                                         0.371954224882065,
                                         0.201777747856344,
                                         1.13815840211787e-16,
                                         1.29951575909081e-09,
                                         5.35422353688742e-06,
                                         0.0009765625,
                                         0.0294492960050725,
                                         0.250618196918555,
                                         0.772785364063438,

                                         0.818281947367293,
                                         0.342274614952593,
                                         0.0666675399641295,
                                         0.00526190284373216,
                                         0.000128709189423396,
                                         5.64632098347949e-07,
                                         1.17974973503993e-10,
                                         9.52170250575136e-18,
                                         0.165761677783144,
                                         0.387361282826356,
                                         0.207347853234545,
                                         0.036306927501162,
                                         0.00186536095575633,
                                         1.81543393840022e-05,
                                         1.04243352189183e-08,
                                         4.70038513898933e-15,
                                         0.015110464381438,
                                         0.197274178671264,
                                         0.290199841164804,
                                         0.11273238231017,
                                         0.0121654652622841,
                                         0.000262668413193709,
                                         4.14495063554442e-07,
                                         1.04415457200737e-12,
                                         0.000816258482709448,
                                         0.0595361156064315,
                                         0.24068610343966,
                                         0.207426428708215,
                                         0.0470165526083243,
                                         0.00225211969943059,
                                         9.76667137838826e-06,
                                         1.37452411270825e-10,
                                         2.8936561298882e-05,
                                         0.0117912558920706,
                                         0.13100086944455,
                                         0.250466018321366,
                                         0.119245543639719,
                                         0.0126719778389951,
                                         0.000151023003816402,
                                         1.18743336500424e-08,
                                         7.03411282471979e-07,
                                         0.00160133733871637,
                                         0.0488923078292098,
                                         0.207384708659849,
                                         0.207384708659849,
                                         0.0488923078292098,
                                         0.00160133733871637,
                                         7.03411282471979e-07,
                                         1.18743336500424e-08,
                                         0.000151023003816402,
                                         0.0126719778389951,
                                         0.119245543639719,
                                         0.250466018321366,
                                         0.13100086944455,
                                         0.0117912558920706,
                                         2.8936561298882e-05,
                                         1.37452411270825e-10,
                                         9.76667137838826e-06,
                                         0.00225211969943059,
                                         0.0470165526083244,
                                         0.207426428708215,
                                         0.24068610343966,
                                         0.0595361156064315,
                                         0.000816258482709448,
                                         1.04415457200737e-12,
                                         4.14495063554442e-07,
                                         0.000262668413193709,
                                         0.0121654652622841,
                                         0.11273238231017,
                                         0.290199841164804,
                                         0.197274178671264,
                                         0.015110464381438,
                                         4.70038513898933e-15,
                                         1.04243352189183e-08,
                                         1.81543393840022e-05,
                                         0.00186536095575633,
                                         0.036306927501162,
                                         0.207347853234545,
                                         0.387361282826356,
                                         0.165761677783144,
                                         9.52170250575136e-18,
                                         1.17974973503993e-10,
                                         5.64632098347949e-07,
                                         0.000128709189423397,
                                         0.00526190284373216,
                                         0.0666675399641295,
                                         0.342274614952593,
                                         0.818281947367293,

                                         0.851735175536328,
                                         0.425109092327014,
                                         0.116692695393162,
                                         0.016196123758113,
                                         0.0009765625,
                                         1.9388248850336e-05,
                                         7.28852159084509e-08,
                                         1.37187543948259e-11,
                                         1.04567798624335e-18,
                                         0.13778880117399,
                                         0.379648617368653,
                                         0.279642546779476,
                                         0.0826463801433249,
                                         0.009765625,
                                         0.000379949463350445,
                                         3.04144429973912e-06,
                                         1.53614868113128e-09,
                                         6.4638106695102e-16,
                                         0.0100308046719505,
                                         0.152572325250515,
                                         0.301561114580719,
                                         0.1897794135043,
                                         0.0439453125,
                                         0.00335062326239977,
                                         5.71127147104239e-05,
                                         7.74041663100339e-08,
                                         1.79800875742044e-13,
                                         0.000432726850710203,
                                         0.0363350637130233,
                                         0.192709732874338,
                                         0.258244203548962,
                                         0.1171875,
                                         0.0175098159751717,
                                         0.000635538637691468,
                                         2.31127510208108e-06,
                                         2.96381571713476e-11,
                                         1.22507091979402e-05,
                                         0.00567865394127341,
                                         0.0808167302503907,
                                         0.230611723128175,
                                         0.205078125,
                                         0.0600491522486233,
                                         0.00464109469919465,
                                         4.52906194747668e-05,
                                         3.20612098916594e-09,
                                         2.3782188394684e-07,
                                         0.00060856655091096,
                                         0.0232403197408021,
                                         0.141213226718729,
                                         0.24609375,
                                         0.141213226718729,
                                         0.0232403197408022,
                                         0.00060856655091096,
                                         2.37821883946844e-07,
                                         3.20612098916587e-09,
                                         4.52906194747668e-05,
                                         0.00464109469919464,
                                         0.0600491522486233,
                                         0.205078125,
                                         0.230611723128175,
                                         0.0808167302503908,
                                         0.00567865394127341,
                                         1.22507091979404e-05,
                                         2.96381571713469e-11,
                                         2.31127510208108e-06,
                                         0.000635538637691467,
                                         0.0175098159751717,
                                         0.1171875,
                                         0.258244203548962,
                                         0.192709732874338,
                                         0.0363350637130233,
                                         0.000432726850710207,
                                         1.79800875742039e-13,
                                         7.74041663100339e-08,
                                         5.71127147104238e-05,
                                         0.00335062326239977,
                                         0.0439453125,
                                         0.1897794135043,
                                         0.301561114580719,
                                         0.152572325250515,
                                         0.0100308046719506,
                                         6.46381066951e-16,
                                         1.53614868113128e-09,
                                         3.04144429973911e-06,
                                         0.000379949463350445,
                                         0.009765625,
                                         0.0826463801433249,
                                         0.279642546779476,
                                         0.379648617368653,
                                         0.13778880117399,
                                         1.04567798624332e-18,
                                         1.37187543948259e-11,
                                         7.28852159084507e-08,
                                         1.9388248850336e-05,
                                         0.0009765625,
                                         0.016196123758113,
                                         0.116692695393162,
                                         0.425109092327014,
                                         0.851735175536328,

                                         0.876931918350956,
                                         0.497320041900762,
                                         0.174287514096617,
                                         0.0357569189193685,
                                         0.00391223861797812,
                                         0.000194821502386827,
                                         3.33043566515876e-06,
                                         1.11996798052475e-08,
                                         1.95433381178948e-12,
                                         1.4289553520786e-19,
                                         0.115923412143846,
                                         0.359809180378225,
                                         0.33270567489333,
                                         0.141342961156707,
                                         0.0289832104940196,
                                         0.00262975768473797,
                                         8.42533063342464e-05,
                                         5.86694035970531e-07,
                                         2.70123561646077e-10,
                                         1.08096935292176e-16,
                                         0.00689586812936851,
                                         0.117144265100074,
                                         0.285802915980478,
                                         0.2514205634193,
                                         0.096622920444114,
                                         0.01597375766058,
                                         0.000959147437115043,
                                         1.38302571165428e-05,
                                         1.68011381434872e-08,
                                         3.67976951215e-14,
                                         0.000243087685302831,
                                         0.0226009160891357,
                                         0.145488719281871,
                                         0.265023035193131,
                                         0.190884228714792,
                                         0.0574982555106884,
                                         0.00647053226374083,
                                         0.000193199079671614,
                                         6.19257330915707e-07,
                                         7.42307880166008e-12,
                                         5.62349463468357e-06,
                                         0.0028615436168246,
                                         0.0486028014574891,
                                         0.183330941190596,
                                         0.247473813891624,
                                         0.135822450557361,
                                         0.028645998169703,
                                         0.00177112481871258,
                                         1.49786587576945e-05,
                                         9.82690689753169e-10,
                                         8.92057418711813e-08,
                                         0.000248437925674829,
                                         0.0111336222409986,
                                         0.0869623185083395,
                                         0.220004544921719,
                                         0.220004544921719,
                                         0.0869623185083395,
                                         0.0111336222409986,
                                         0.000248437925674828,
                                         8.92057418711813e-08,
                                         9.82690689753169e-10,
                                         1.49786587576946e-05,
                                         0.00177112481871258,
                                         0.028645998169703,
                                         0.135822450557361,
                                         0.247473813891624,
                                         0.183330941190596,
                                         0.0486028014574891,
                                         0.00286154361682459,
                                         5.62349463468356e-06,
                                         7.42307880166008e-12,
                                         6.1925733091571e-07,
                                         0.000193199079671614,
                                         0.00647053226374083,
                                         0.0574982555106884,
                                         0.190884228714792,
                                         0.265023035193131,
                                         0.145488719281871,
                                         0.0226009160891356,
                                         0.000243087685302831,
                                         3.67976951215e-14,
                                         1.68011381434873e-08,
                                         1.38302571165428e-05,
                                         0.000959147437115043,
                                         0.01597375766058,
                                         0.096622920444114,
                                         0.2514205634193,
                                         0.285802915980478,
                                         0.117144265100074,
                                         0.00689586812936851,
                                         1.08096935292176e-16,
                                         2.70123561646079e-10,
                                         5.86694035970531e-07,
                                         8.42533063342464e-05,
                                         0.00262975768473797,
                                         0.0289832104940195,
                                         0.141342961156707,
                                         0.33270567489333,
                                         0.359809180378225,
                                         0.115923412143846,
                                         1.4289553520786e-19,
                                         1.9543338117895e-12,
                                         1.11996798052475e-08,
                                         3.33043566515876e-06,
                                         0.000194821502386827,
                                         0.00391223861797812,
                                         0.0357569189193685,
                                         0.174287514096617,
                                         0.497320041900762,
                                         0.876931918350956,

                                         0.896323812333893,
                                         0.559194790015995,
                                         0.234715974685804,
                                         0.0639076156372074,
                                         0.0106212279330999,
                                         0.0009765625,
                                         4.22323909302519e-05,
                                         6.46074753540428e-07,
                                         1.99936456001886e-09,
                                         3.29668844496318e-13,
                                         2.33642591620061e-20,
                                         0.0986446741118331,
                                         0.33466831368177,
                                         0.366081331724848,
                                         0.202313857182584,
                                         0.0611113422930709,
                                         0.009765625,
                                         0.000734004250927483,
                                         2.04084374630368e-05,
                                         1.28190858366359e-07,
                                         5.50841214230483e-11,
                                         2.1229673099943e-17,
                                         0.00488534078703268,
                                         0.0901319128540668,
                                         0.256936042498068,
                                         0.288211402975878,
                                         0.158227304900415,
                                         0.0439453125,
                                         0.00574068867119621,
                                         0.000290101018233766,
                                         3.69857674972842e-06,
                                         4.14178414982844e-09,
                                         8.68054739345246e-15,
                                         0.000143374628151332,
                                         0.0143846334395307,
                                         0.106863328448169,
                                         0.243306049578672,
                                         0.242771251454407,
                                         0.1171875,
                                         0.0266063704191303,
                                         0.0024436835752698,
                                         6.32365886325498e-05,
                                         1.84546033452269e-07,
                                         2.1033286464166e-12,
                                         2.76133476894547e-06,
                                         0.00150656657190972,
                                         0.0291676661018242,
                                         0.134791954811336,
                                         0.244445698698799,
                                         0.205078125,
                                         0.0809238595747051,
                                         0.0135085894497641,
                                         0.00070953128275707,
                                         5.3962403478335e-06,
                                         3.34454063928206e-10,
                                         3.64677539051921e-08,
                                         0.00010819845314852,
                                         0.00545905990292564,
                                         0.0512056912588394,
                                         0.168776019413319,
                                         0.24609375,
                                         0.168776019413319,
                                         0.0512056912588394,
                                         0.00545905990292563,
                                         0.00010819845314852,
                                         3.6467753905193e-08,
                                         3.34454063928195e-10,
                                         5.3962403478335e-06,
                                         0.000709531282757072,
                                         0.0135085894497641,
                                         0.080923859574705,
                                         0.205078125,
                                         0.2444456986988,
                                         0.134791954811335,
                                         0.0291676661018241,
                                         0.00150656657190972,
                                         2.76133476894553e-06,
                                         2.10332864641652e-12,
                                         1.84546033452269e-07,
                                         6.323658863255e-05,
                                         0.0024436835752698,
                                         0.0266063704191303,
                                         0.1171875,
                                         0.242771251454407,
                                         0.243306049578671,
                                         0.106863328448169,
                                         0.0143846334395307,
                                         0.000143374628151334,
                                         8.6805473934521e-15,
                                         4.14178414982844e-09,
                                         3.69857674972843e-06,
                                         0.000290101018233766,
                                         0.0057406886711962,
                                         0.0439453125,
                                         0.158227304900415,
                                         0.288211402975878,
                                         0.256936042498068,
                                         0.0901319128540668,
                                         0.00488534078703272,
                                         2.1229673099942e-17,
                                         5.50841214230483e-11,
                                         1.28190858366359e-07,
                                         2.04084374630368e-05,
                                         0.000734004250927482,
                                         0.009765625,
                                         0.0611113422930709,
                                         0.202313857182584,
                                         0.366081331724847,
                                         0.33466831368177,
                                         0.0986446741118336,
                                         2.33642591620049e-20,
                                         3.29668844496318e-13,
                                         1.99936456001887e-09,
                                         6.4607475354043e-07,
                                         4.22323909302519e-05,
                                         0.0009765625,
                                         0.0106212279330999,
                                         0.0639076156372074,
                                         0.234715974685804,
                                         0.559194790015995,
                                         0.896323812333893,

                                         0.911535743182133,
                                         0.611838684527693,
                                         0.294573651207087,
                                         0.099160641929744,
                                         0.0223885542447087,
                                         0.00317779753275875,
                                         0.000256225251025963,
                                         9.95470898209303e-06,
                                         1.3991226605665e-07,
                                         4.06270937776451e-10,
                                         6.41363985904255e-14,
                                         4.43771933524246e-21,
                                         0.0848227436255752,
                                         0.308094323156433,
                                         0.382962353714874,
                                         0.257804786087803,
                                         0.10347282385909,
                                         0.0247044843775957,
                                         0.00329588732999106,
                                         0.000215391378841062,
                                         5.38150991164976e-06,
                                         3.12502553734664e-08,
                                         1.27367259941317e-11,
                                         4.76893297644326e-18,
                                         0.00355192218241913,
                                         0.0698140726677376,
                                         0.22404269252473,
                                         0.301616527448364,
                                         0.215198414428977,
                                         0.0864246994756985,
                                         0.0190781078829061,
                                         0.0020972035217834,
                                         9.31461721365896e-05,
                                         1.08169269949086e-06,
                                         1.13821303778684e-09,
                                         2.30619018623774e-15,
                                         8.8139541621063e-05,
                                         0.00937472354910661,
                                         0.0776715016298326,
                                         0.209110342776155,
                                         0.265221087377451,
                                         0.179166240435485,
                                         0.0654416833622706,
                                         0.0121006600080379,
                                         0.000955393095427062,
                                         2.21876059883982e-05,
                                         6.02761527155804e-08,
                                         6.60884002089816e-13,
                                         1.43531574562011e-06,
                                         0.000826120277197413,
                                         0.0176710329243542,
                                         0.0951404516381722,
                                         0.21450941171298,
                                         0.243749537182748,
                                         0.147313633149292,
                                         0.0458191317821208,
                                         0.00643085179418168,
                                         0.000298666359167054,
                                         2.09477223773903e-06,
                                         1.24286705126791e-10,
                                         1.60275561883032e-08,
                                         4.99196224270298e-05,
                                         0.00275680068473996,
                                         0.0296823376358382,
                                         0.118967366977026,
                                         0.227391704020228,
                                         0.227391704020228,
                                         0.118967366977026,
                                         0.0296823376358382,
                                         0.00275680068473996,
                                         4.99196224270301e-05,
                                         1.60275561883027e-08,
                                         1.24286705126795e-10,
                                         2.09477223773901e-06,
                                         0.000298666359167053,
                                         0.00643085179418167,
                                         0.0458191317821208,
                                         0.147313633149292,
                                         0.243749537182748,
                                         0.21450941171298,
                                         0.0951404516381723,
                                         0.0176710329243542,
                                         0.000826120277197417,
                                         1.43531574562008e-06,
                                         6.60884002089844e-13,
                                         6.02761527155799e-08,
                                         2.21876059883982e-05,
                                         0.00095539309542706,
                                         0.0121006600080379,
                                         0.0654416833622706,
                                         0.179166240435485,
                                         0.265221087377452,
                                         0.209110342776155,
                                         0.0776715016298327,
                                         0.00937472354910664,
                                         8.81395416210614e-05,
                                         2.30619018623785e-15,
                                         1.13821303778683e-09,
                                         1.08169269949085e-06,
                                         9.31461721365894e-05,
                                         0.00209720352178339,
                                         0.0190781078829061,
                                         0.0864246994756985,
                                         0.215198414428977,
                                         0.301616527448364,
                                         0.224042692524731,
                                         0.0698140726677377,
                                         0.00355192218241908,
                                         4.76893297644352e-18,
                                         1.27367259941316e-11,
                                         3.12502553734663e-08,
                                         5.38150991164975e-06,
                                         0.000215391378841062,
                                         0.00329588732999106,
                                         0.0247044843775957,
                                         0.10347282385909,
                                         0.257804786087803,
                                         0.382962353714875,
                                         0.308094323156434,
                                         0.0848227436255747,
                                         4.43771933524272e-21,
                                         6.41363985904247e-14,
                                         4.06270937776449e-10,
                                         1.3991226605665e-07,
                                         9.95470898209301e-06,
                                         0.000256225251025963,
                                         0.00317779753275875,
                                         0.0223885542447087,
                                         0.099160641929744,
                                         0.294573651207087,
                                         0.611838684527693,
                                         0.911535743182133,

                                         0.923671209348314,
                                         0.656563337712019,
                                         0.351747452013348,
                                         0.139429748773788,
                                         0.0397066551993873,
                                         0.0077690711892159,
                                         0.0009765625,
                                         7.11248784618402e-05,
                                         2.54221510477788e-06,
                                         3.34418549107212e-08,
                                         9.23838985884738e-11,
                                         1.40947224897598e-14,
                                         9.57031351777494e-22,
                                         0.0736305903910159,
                                         0.282133475189055,
                                         0.387407025913447,
                                         0.303632975963602,
                                         0.151181344600374,
                                         0.0485885953948207,
                                         0.009765625,
                                         0.00113725091166819,
                                         6.67693880319147e-05,
                                         1.53566634649402e-06,
                                         8.38802570989386e-09,
                                         3.28003546399457e-12,
                                         1.20056392511072e-18,
                                         0.00264126315068278,
                                         0.0545563268044144,
                                         0.192006768750192,
                                         0.297546565252295,
                                         0.25902734637765,
                                         0.136745203541627,
                                         0.0439453125,
                                         0.00818283066104375,
                                         0.000789141731712612,
                                         3.17333476364439e-05,
                                         3.42717068376441e-07,
                                         3.43489181414912e-10,
                                         6.77730338739794e-16,
                                         5.6146317923386e-05,
                                         0.00625160953133838,
                                         0.0563925591583743,
                                         0.172789427201085,
                                         0.262996063284587,
                                         0.228058402529567,
                                         0.1171875,
                                         0.03489048869369,
                                         0.00552699209943528,
                                         0.000388589561649461,
                                         8.29789497862333e-06,
                                         2.13159064296664e-08,
                                         2.2671736417213e-13,
                                         7.83249641917534e-07,
                                         0.000470119086348759,
                                         0.0108691700902893,
                                         0.0658489273635239,
                                         0.175235541690802,
                                         0.249602752501347,
                                         0.205078125,
                                         0.097629228525555,
                                         0.0254034085874624,
                                         0.00312273900335325,
                                         0.000131846756408597,
                                         8.68088221692636e-07,
                                         4.97716406262445e-11,
                                         7.49242366429878e-09,
                                         2.42419259131625e-05,
                                         0.00143652822548375,
                                         0.0172077244248667,
                                         0.0800641948254532,
                                         0.187325051173004,
                                         0.24609375,
                                         0.187325051173004,
                                         0.0800641948254532,
                                         0.0172077244248667,
                                         0.00143652822548375,
                                         2.42419259131624e-05,
                                         7.49242366429852e-09,
                                         4.97716406262466e-11,
                                         8.68088221692643e-07,
                                         0.000131846756408597,
                                         0.00312273900335326,
                                         0.0254034085874624,
                                         0.097629228525555,
                                         0.205078125,
                                         0.249602752501347,
                                         0.175235541690802,
                                         0.0658489273635238,
                                         0.0108691700902893,
                                         0.000470119086348756,
                                         7.83249641917512e-07,
                                         2.26717364172141e-13,
                                         2.13159064296666e-08,
                                         8.2978949786233e-06,
                                         0.000388589561649462,
                                         0.00552699209943528,
                                         0.03489048869369,
                                         0.1171875,
                                         0.228058402529567,
                                         0.262996063284586,
                                         0.172789427201085,
                                         0.0563925591583744,
                                         0.00625160953133836,
                                         5.61463179233848e-05,
                                         6.77730338739832e-16,
                                         3.43489181414916e-10,
                                         3.42717068376439e-07,
                                         3.1733347636444e-05,
                                         0.000789141731712611,
                                         0.00818283066104375,
                                         0.0439453125,
                                         0.136745203541627,
                                         0.25902734637765,
                                         0.297546565252295,
                                         0.192006768750192,
                                         0.0545563268044142,
                                         0.00264126315068275,
                                         1.20056392511079e-18,
                                         3.28003546399461e-12,
                                         8.38802570989381e-09,
                                         1.53566634649403e-06,
                                         6.67693880319147e-05,
                                         0.00113725091166819,
                                         0.009765625,
                                         0.0485885953948207,
                                         0.151181344600374,
                                         0.303632975963602,
                                         0.387407025913448,
                                         0.282133475189055,
                                         0.0736305903910154,
                                         9.57031351777561e-22,
                                         1.409472248976e-14,
                                         9.23838985884733e-11,
                                         3.34418549107213e-08,
                                         2.54221510477788e-06,
                                         7.11248784618402e-05,
                                         0.0009765625,
                                         0.0077690711892159,
                                         0.0397066551993873,
                                         0.139429748773788,
                                         0.351747452013348,
                                         0.656563337712019,
                                         0.923671209348314,

                                         0.933497303443,
                                         0.694624076131893,
                                         0.405101671102788,
                                         0.182643762925046,
                                         0.0623073186006732,
                                         0.0155782249028378,
                                         0.00272466441833927,
                                         0.000311229708621698,
                                         2.09148373931298e-05,
                                         6.99665464912018e-07,
                                         8.73126956130323e-09,
                                         2.31795165266093e-11,
                                         3.44123126042938e-15,
                                         2.301629314977e-22,
                                         0.0644622260875951,
                                         0.257778228549892,
                                         0.383105219180129,
                                         0.338494877775408,
                                         0.199330703435817,
                                         0.080410288768823,
                                         0.0219325850871026,
                                         0.00386637739985471,
                                         0.000405191979415929,
                                         2.18703281956752e-05,
                                         4.71118481399128e-07,
                                         2.45103966486791e-09,
                                         9.27294014889742e-13,
                                         3.3330601337546e-19,
                                         0.00200313419179564,
                                         0.0430482153281518,
                                         0.163036414669878,
                                         0.282300644710459,
                                         0.286960321853823,
                                         0.186774588317726,
                                         0.0794472994231299,
                                         0.0216142392670006,
                                         0.00353247991813784,
                                         0.000307632826998275,
                                         1.14391933361878e-05,
                                         1.16629608919439e-07,
                                         1.12443296087683e-10,
                                         2.17201805795598e-16,
                                         3.68868022590083e-05,
                                         0.0042601048330371,
                                         0.0411156710076686,
                                         0.139517233908357,
                                         0.244808064704287,
                                         0.257087033128668,
                                         0.170539377772904,
                                         0.0716031097734411,
                                         0.0182496601606852,
                                         0.00256428487713022,
                                         0.000164595090634111,
                                         3.28869439223968e-06,
                                         8.07989835309304e-09,
                                         8.38764290630702e-14,
                                         4.45760197878723e-07,
                                         0.000276665328960624,
                                         0.00680455086569735,
                                         0.0452494350178373,
                                         0.137056269584908,
                                         0.232226551662206,
                                         0.240236795031616,
                                         0.15566577225979,
                                         0.0618727156423356,
                                         0.0140271416354557,
                                         0.00155420274281407,
                                         6.08565786329956e-05,
                                         3.81019798707458e-07,
                                         2.12562059881725e-11,
                                         3.69381180434409e-09,
                                         1.23206149947099e-05,
                                         0.00077220879698535,
                                         0.0100633287863576,
                                         0.0526156924872454,
                                         0.143842350681772,
                                         0.232058549858201,
                                         0.232058549858201,
                                         0.143842350681772,
                                         0.0526156924872455,
                                         0.0100633287863576,
                                         0.00077220879698535,
                                         1.23206149947098e-05,
                                         3.69381180434409e-09,
                                         2.12562059881725e-11,
                                         3.81019798707462e-07,
                                         6.08565786329956e-05,
                                         0.00155420274281407,
                                         0.0140271416354557,
                                         0.0618727156423356,
                                         0.15566577225979,
                                         0.240236795031616,
                                         0.232226551662206,
                                         0.137056269584909,
                                         0.0452494350178373,
                                         0.00680455086569735,
                                         0.000276665328960622,
                                         4.45760197878723e-07,
                                         8.38764290630702e-14,
                                         8.07989835309313e-09,
                                         3.28869439223968e-06,
                                         0.00016459509063411,
                                         0.00256428487713022,
                                         0.0182496601606852,
                                         0.071603109773441,
                                         0.170539377772904,
                                         0.257087033128668,
                                         0.244808064704288,
                                         0.139517233908357,
                                         0.0411156710076686,
                                         0.00426010483303708,
                                         3.68868022590083e-05,
                                         2.17201805795598e-16,
                                         1.12443296087684e-10,
                                         1.16629608919439e-07,
                                         1.14391933361878e-05,
                                         0.000307632826998274,
                                         0.00353247991813784,
                                         0.0216142392670006,
                                         0.0794472994231299,
                                         0.186774588317726,
                                         0.286960321853823,
                                         0.282300644710459,
                                         0.163036414669878,
                                         0.0430482153281517,
                                         0.00200313419179564,
                                         3.3330601337546e-19,
                                         9.27294014889755e-13,
                                         2.45103966486791e-09,
                                         4.71118481399127e-07,
                                         2.18703281956752e-05,
                                         0.000405191979415929,
                                         0.0038663773998547,
                                         0.0219325850871026,
                                         0.080410288768823,
                                         0.199330703435817,
                                         0.338494877775408,
                                         0.383105219180129,
                                         0.257778228549892,
                                         0.0644622260875951,
                                         2.301629314977e-22,
                                         3.44123126042943e-15,
                                         2.31795165266093e-11,
                                         8.7312695613032e-09,
                                         6.99665464912016e-07,
                                         2.09148373931298e-05,
                                         0.000311229708621698,
                                         0.00272466441833927,
                                         0.0155782249028378,
                                         0.0623073186006732,
                                         0.182643762925046,
                                         0.405101671102788,
                                         0.694624076131893,
                                         0.933497303443,

                                         0.941558913236739,
                                         0.727125172760599,
                                         0.454156121415248,
                                         0.227051743859201,
                                         0.0894097675425855,
                                         0.027089392792717,
                                         0.00610705632372024,
                                         0.0009765625,
                                         0.000103302947965514,
                                         6.50641123602467e-06,
                                         2.06326335222884e-07,
                                         2.46724770334361e-09,
                                         6.34204626411259e-12,
                                         9.20911822789648e-16,
                                         6.08442411163037e-23,
                                         0.0568701923214448,
                                         0.235434480059637,
                                         0.372998938286871,
                                         0.362855435393799,
                                         0.244175415613358,
                                         0.117720878485378,
                                         0.0406125261858676,
                                         0.009765625,
                                         0.00155340477650777,
                                         0.000149722574203787,
                                         7.55505611564214e-06,
                                         1.54384594781889e-07,
                                         7.72194995078038e-10,
                                         2.8441805468493e-13,
                                         1.00735438379339e-19,
                                         0.00154573275038311,
                                         0.0343038976160445,
                                         0.137855003227328,
                                         0.260948580004793,
                                         0.300076108605264,
                                         0.230207904689273,
                                         0.121534785025016,
                                         0.0439453125,
                                         0.0105116059245203,
                                         0.00155040648152409,
                                         0.000124489647828907,
                                         4.34718061877223e-06,
                                         4.2309420101361e-08,
                                         3.95283593097674e-11,
                                         7.50511923837816e-17,
                                         2.48966170340218e-05,
                                         0.00296191837913866,
                                         0.0301921255219998,
                                         0.111207086107209,
                                         0.21853304711011,
                                         0.266773808052627,
                                         0.215524878225477,
                                         0.1171875,
                                         0.0421511787701504,
                                         0.00951393289092093,
                                         0.00121558407189971,
                                         7.25383422035608e-05,
                                         1.3737363236342e-06,
                                         3.25549201173672e-09,
                                         3.31351644876116e-14,
                                         2.63157399202891e-07,
                                         0.000167830938119932,
                                         0.00433944472137981,
                                         0.0311013543363612,
                                         0.104441268971437,
                                         0.202878236297176,
                                         0.250820792688966,
                                         0.205078125,
                                         0.110922511430144,
                                         0.0383128014081213,
                                         0.00778943718768813,
                                         0.000794322534625231,
                                         2.92710975544643e-05,
                                         1.75951833138315e-07,
                                         9.60041444117728e-12,
                                         1.90736596847022e-09,
                                         6.52099932189202e-06,
                                         0.000427678905338183,
                                         0.0059644353893474,
                                         0.0342271198673778,
                                         0.105796409916823,
                                         0.200157957701666,
                                         0.24609375,
                                         0.200157957701666,
                                         0.105796409916823,
                                         0.0342271198673778,
                                         0.00596443538934739,
                                         0.000427678905338185,
                                         6.52099932189202e-06,
                                         1.90736596847031e-09,
                                         9.60041444117674e-12,
                                         1.75951833138315e-07,
                                         2.92710975544641e-05,
                                         0.000794322534625233,
                                         0.00778943718768813,
                                         0.0383128014081213,
                                         0.110922511430144,
                                         0.205078125,
                                         0.250820792688966,
                                         0.202878236297176,
                                         0.104441268971437,
                                         0.0311013543363611,
                                         0.00433944472137982,
                                         0.000167830938119932,
                                         2.63157399202901e-07,
                                         3.31351644876094e-14,
                                         3.25549201173672e-09,
                                         1.37373632363419e-06,
                                         7.2538342203561e-05,
                                         0.00121558407189971,
                                         0.00951393289092093,
                                         0.0421511787701504,
                                         0.1171875,
                                         0.215524878225477,
                                         0.266773808052627,
                                         0.21853304711011,
                                         0.111207086107209,
                                         0.0301921255219998,
                                         0.00296191837913866,
                                         2.48966170340225e-05,
                                         7.50511923837761e-17,
                                         3.95283593097674e-11,
                                         4.23094201013608e-08,
                                         4.34718061877224e-06,
                                         0.000124489647828907,
                                         0.00155040648152409,
                                         0.0105116059245203,
                                         0.0439453125,
                                         0.121534785025016,
                                         0.230207904689273,
                                         0.300076108605264,
                                         0.260948580004793,
                                         0.137855003227328,
                                         0.0343038976160445,
                                         0.00154573275038313,
                                         1.0073543837933e-19,
                                         2.8441805468493e-13,
                                         7.72194995078033e-10,
                                         1.5438459478189e-07,
                                         7.55505611564214e-06,
                                         0.000149722574203787,
                                         0.00155340477650777,
                                         0.009765625,
                                         0.0406125261858676,
                                         0.117720878485378,
                                         0.244175415613358,
                                         0.362855435393799,
                                         0.372998938286871,
                                         0.235434480059637,
                                         0.0568701923214453,
                                         6.08442411162981e-23,
                                         9.20911822789648e-16,
                                         6.34204626411255e-12,
                                         2.46724770334362e-09,
                                         2.06326335222884e-07,
                                         6.50641123602467e-06,
                                         0.000103302947965514,
                                         0.0009765625,
                                         0.00610705632372024,
                                         0.027089392792717,
                                         0.0894097675425855,
                                         0.227051743859201,
                                         0.454156121415248,
                                         0.727125172760599,
                                         0.941558913236739,

                                         0.94825080605256,
                                         0.755000214987899,
                                         0.49883625467001,
                                         0.271313803702638,
                                         0.119992202652067,
                                         0.0423960366096015,
                                         0.011674467036035,
                                         0.00242042065757596,
                                         0.000359853559072057,
                                         3.575541428079e-05,
                                         2.13578961474489e-06,
                                         6.48246853939254e-08,
                                         7.48490509416279e-10,
                                         1.87363224824027e-12,
                                         2.67150045983515e-16,
                                         1.74733269012327e-23,
                                         0.0505205952243785,
                                         0.21519287761536,
                                         0.359278016253035,
                                         0.378044854064652,
                                         0.283407777810203,
                                         0.157597229938297,
                                         0.0654406400491995,
                                         0.0200038848538962,
                                         0.00435413918067315,
                                         0.000637868769417644,
                                         5.74559684409323e-05,
                                         2.74461655454411e-06,
                                         5.37173843160756e-08,
                                         2.60142744910636e-10,
                                         9.37290975364439e-14,
                                         3.27967163587943e-20,
                                         0.0012112288610676,
                                         0.0276007716894553,
                                         0.116443645162975,
                                         0.237043082144952,
                                         0.301219454569868,
                                         0.263623677864495,
                                         0.165070903071949,
                                         0.0743961317624482,
                                         0.0237078038747271,
                                         0.0051207476918437,
                                         0.000695543572743644,
                                         5.22920241504567e-05,
                                         1.73482603140164e-06,
                                         1.62536759852101e-08,
                                         1.47981059171977e-11,
                                         2.77011398287005e-17,
                                         1.72083876610142e-05,
                                         0.00209783233306007,
                                         0.0223643952353871,
                                         0.0880780028376195,
                                         0.18971885253885,
                                         0.261322297711707,
                                         0.246745780449698,
                                         0.163961764105555,
                                         0.0764956189437002,
                                         0.024360804981262,
                                         0.0049896462072179,
                                         0.000590398772861767,
                                         3.3201157254667e-05,
                                         6.0179378169889e-07,
                                         1.38450346160224e-09,
                                         1.38650402168209e-14,
                                         1.60443995237339e-07,
                                         0.000104638050851113,
                                         0.00281882322894154,
                                         0.0214771647187215,
                                         0.0784164680138645,
                                         0.169995661586231,
                                         0.242046194197959,
                                         0.237139623596028,
                                         0.161976166580125,
                                         0.0760535006067494,
                                         0.0234900791636845,
                                         0.00437446308127993,
                                         0.000416984421006486,
                                         1.46222131466614e-05,
                                         8.50063319847299e-08,
                                         4.55421934965902e-12,
                                         1.02576990237139e-09,
                                         3.57891764643921e-06,
                                         0.0002436249270291,
                                         0.00359111766124973,
                                         0.0222252810956142,
                                         0.0758302355879672,
                                         0.162813337731606,
                                         0.235184592886199,
                                         0.235184592886199,
                                         0.162813337731606,
                                         0.0758302355879672,
                                         0.0222252810956142,
                                         0.00359111766124972,
                                         0.000243624927029101,
                                         3.57891764643921e-06,
                                         1.02576990237134e-09,
                                         4.5542193496593e-12,
                                         8.50063319847299e-08,
                                         1.46222131466614e-05,
                                         0.000416984421006487,
                                         0.00437446308127993,
                                         0.0234900791636845,
                                         0.0760535006067495,
                                         0.161976166580125,
                                         0.237139623596028,
                                         0.242046194197959,
                                         0.169995661586231,
                                         0.0784164680138646,
                                         0.0214771647187214,
                                         0.00281882322894155,
                                         0.000104638050851113,
                                         1.60443995237332e-07,
                                         1.38650402168219e-14,
                                         1.38450346160224e-09,
                                         6.01793781698886e-07,
                                         3.32011572546671e-05,
                                         0.000590398772861767,
                                         0.0049896462072179,
                                         0.024360804981262,
                                         0.0764956189437002,
                                         0.163961764105555,
                                         0.246745780449697,
                                         0.261322297711707,
                                         0.18971885253885,
                                         0.0880780028376194,
                                         0.0223643952353871,
                                         0.00209783233306007,
                                         1.72083876610136e-05,
                                         2.77011398287028e-17,
                                         1.47981059171977e-11,
                                         1.625367598521e-08,
                                         1.73482603140164e-06,
                                         5.22920241504567e-05,
                                         0.000695543572743644,
                                         0.00512074769184371,
                                         0.0237078038747271,
                                         0.0743961317624481,
                                         0.165070903071949,
                                         0.263623677864495,
                                         0.301219454569868,
                                         0.237043082144952,
                                         0.116443645162976,
                                         0.0276007716894553,
                                         0.00121122886106757,
                                         3.27967163587973e-20,
                                         9.37290975364439e-14,
                                         2.60142744910634e-10,
                                         5.37173843160758e-08,
                                         2.74461655454411e-06,
                                         5.74559684409323e-05,
                                         0.000637868769417645,
                                         0.00435413918067314,
                                         0.0200038848538962,
                                         0.0654406400491995,
                                         0.157597229938296,
                                         0.283407777810203,
                                         0.378044854064652,
                                         0.359278016253036,
                                         0.21519287761536,
                                         0.050520595224378,
                                         1.74733269012345e-23,
                                         2.67150045983515e-16,
                                         1.87363224824026e-12,
                                         7.48490509416283e-10,
                                         6.48246853939254e-08,
                                         2.13578961474489e-06,
                                         3.575541428079e-05,
                                         0.000359853559072057,
                                         0.00242042065757595,
                                         0.011674467036035,
                                         0.0423960366096015,
                                         0.119992202652067,
                                         0.271313803702638,
                                         0.49883625467001,
                                         0.755000214987899,
                                         0.94825080605256,

                                         0.953864166743492,
                                         0.779023189279712,
                                         0.539300675660393,
                                         0.314479578576031,
                                         0.152997332353123,
                                         0.0612633786554942,
                                         0.0198152018163868,
                                         0.00504588813372901,
                                         0.0009765625,
                                         0.000136728593532969,
                                         1.29001818249286e-05,
                                         7.37477020326587e-07,
                                         2.1584149374733e-08,
                                         2.42067287424215e-10,
                                         5.92718667759478e-13,
                                         8.32386526604478e-17,
                                         5.39876727272669e-24,
                                         0.0451613942683943,
                                         0.196982593529499,
                                         0.343504735473002,
                                         0.385679729875862,
                                         0.315957716560216,
                                         0.197358437941825,
                                         0.0951389235134839,
                                         0.0351746504989077,
                                         0.009765625,
                                         0.00196140452815839,
                                         0.00026868046945351,
                                         2.28925271283872e-05,
                                         1.04517696589193e-06,
                                         1.97379360747614e-08,
                                         9.30565273165902e-11,
                                         3.29190714291096e-14,
                                         1.14028601850439e-20,
                                         0.000962189608875641,
                                         0.0224139206757991,
                                         0.0984570924494935,
                                         0.212850019132546,
                                         0.293620644905046,
                                         0.28610336626455,
                                         0.205556152444465,
                                         0.110340380566994,
                                         0.0439453125,
                                         0.0126615686642653,
                                         0.00251819222709067,
                                         0.00031978014601911,
                                         2.27749397013267e-05,
                                         7.24235629219425e-07,
                                         6.57442221118261e-09,
                                         5.85844860654962e-12,
                                         1.08379091418576e-17,
                                         1.21481515719335e-05,
                                         0.00151134644398819,
                                         0.0167231275730135,
                                         0.069610826669955,
                                         0.161696428858888,
                                         0.245779951830131,
                                         0.26318367411519,
                                         0.205114037119551,
                                         0.1171875,
                                         0.0484355319687933,
                                         0.0139861402251235,
                                         0.00264707029185179,
                                         0.000294090408192716,
                                         1.5747595587664e-05,
                                         2.75248245466757e-07,
                                         6.17836968696035e-10,
                                         6.10426572832036e-15,
                                         1.00653540967926e-07,
                                         6.68774249754844e-05,
                                         0.00186404896227436,
                                         0.0149399521136227,
                                         0.0584364177412519,
                                         0.13856044142769,
                                         0.221134613863169,
                                         0.250222065165851,
                                         0.205078125,
                                         0.121593385280291,
                                         0.0509772253749751,
                                         0.0143796649786086,
                                         0.0024921478330856,
                                         0.000224707690621428,
                                         7.56242087791819e-06,
                                         4.27596827248967e-08,
                                         2.25626793957289e-12,
                                         5.71861831901885e-10,
                                         2.02926261526451e-06,
                                         0.000142475544628761,
                                         0.0021986941301426,
                                         0.0144813796393814,
                                         0.0535642784596821,
                                         0.127408295768838,
                                         0.209314359479926,
                                         0.24609375,
                                         0.209314359479926,
                                         0.127408295768838,
                                         0.0535642784596821,
                                         0.0144813796393814,
                                         0.00219869413014259,
                                         0.000142475544628761,
                                         2.02926261526449e-06,
                                         5.71861831901918e-10,
                                         2.25626793957273e-12,
                                         4.27596827248973e-08,
                                         7.56242087791819e-06,
                                         0.000224707690621429,
                                         0.0024921478330856,
                                         0.0143796649786086,
                                         0.0509772253749751,
                                         0.121593385280291,
                                         0.205078125,
                                         0.250222065165851,
                                         0.221134613863169,
                                         0.13856044142769,
                                         0.0584364177412518,
                                         0.0149399521136227,
                                         0.00186404896227436,
                                         6.68774249754838e-05,
                                         1.00653540967931e-07,
                                         6.10426572831986e-15,
                                         6.17836968696045e-10,
                                         2.75248245466757e-07,
                                         1.57475955876641e-05,
                                         0.000294090408192717,
                                         0.00264707029185179,
                                         0.0139861402251235,
                                         0.0484355319687933,
                                         0.1171875,
                                         0.205114037119551,
                                         0.26318367411519,
                                         0.245779951830131,
                                         0.161696428858888,
                                         0.0696108266699549,
                                         0.0167231275730135,
                                         0.00151134644398818,
                                         1.21481515719339e-05,
                                         1.08379091418566e-17,
                                         5.85844860654973e-12,
                                         6.57442221118261e-09,
                                         7.24235629219428e-07,
                                         2.27749397013268e-05,
                                         0.00031978014601911,
                                         0.00251819222709067,
                                         0.0126615686642653,
                                         0.0439453125,
                                         0.110340380566994,
                                         0.205556152444465,
                                         0.28610336626455,
                                         0.293620644905046,
                                         0.212850019132545,
                                         0.0984570924494935,
                                         0.022413920675799,
                                         0.000962189608875664,
                                         1.14028601850427e-20,
                                         3.29190714291103e-14,
                                         9.30565273165902e-11,
                                         1.97379360747615e-08,
                                         1.04517696589193e-06,
                                         2.28925271283872e-05,
                                         0.00026868046945351,
                                         0.00196140452815839,
                                         0.009765625,
                                         0.0351746504989077,
                                         0.0951389235134839,
                                         0.197358437941825,
                                         0.315957716560216,
                                         0.385679729875862,
                                         0.343504735473002,
                                         0.196982593529499,
                                         0.0451613942683948,
                                         5.39876727272605e-24,
                                         8.32386526604497e-17,
                                         5.92718667759478e-13,
                                         2.42067287424217e-10,
                                         2.15841493747331e-08,
                                         7.37477020326589e-07,
                                         1.29001818249286e-05,
                                         0.000136728593532969,
                                         0.0009765625,
                                         0.00504588813372901,
                                         0.0198152018163868,
                                         0.0612633786554942,
                                         0.152997332353123,
                                         0.314479578576031,
                                         0.539300675660393,
                                         0.779023189279712,
                                         0.953864166743492,

                                         0.958617303406749,
                                         0.79982991249792,
                                         0.575829699631459,
                                         0.355922166534538,
                                         0.187456400338528,
                                         0.0832353840522244,
                                         0.0307101247365599,
                                         0.00923312368235811,
                                         0.00220341739335867,
                                         0.000402697309589852,
                                         5.3626142025046e-05,
                                         4.84519016456138e-06,
                                         2.66985817613996e-07,
                                         7.5791029787698e-09,
                                         8.29455947560227e-11,
                                         1.99362337855672e-13,
                                         2.76419638040961e-17,
                                         1.78018114373359e-24,
                                         0.0406001050391349,
                                         0.180656988869348,
                                         0.326760059508523,
                                         0.387345812489669,
                                         0.341642606549017,
                                         0.234923254214993,
                                         0.127963027059908,
                                         0.0551761563064615,
                                         0.0185902388131678,
                                         0.00477299008972669,
                                         0.000897374581829956,
                                         0.000116280770895153,
                                         9.45954334740349e-06,
                                         4.1585895171168e-07,
                                         7.62165869312111e-09,
                                         3.51324318210508e-11,
                                         1.22380371936179e-14,
                                         4.20322175505829e-21,
                                         0.000773787240746493,
                                         0.0183621870135309,
                                         0.0834404016521852,
                                         0.189694704775925,
                                         0.280192363021321,
                                         0.298370953648158,
                                         0.239938502225569,
                                         0.148377055075893,
                                         0.0705806539775549,
                                         0.0254574471553447,
                                         0.00675746006271106,
                                         0.00125579136201721,
                                         0.000150821989398143,
                                         1.02680225735716e-05,
                                         3.15150631355197e-07,
                                         2.78603020270286e-09,
                                         2.43818781965778e-12,
                                         4.46594040891474e-18,
                                         8.73921028843073e-06,
                                         0.00110598761497857,
                                         0.0126264139677562,
                                         0.0550513241040586,
                                         0.136174810519298,
                                         0.224565633770294,
                                         0.2666070321879,
                                         0.236449428956247,
                                         0.158797114214449,
                                         0.0804628456418592,
                                         0.0301543124007233,
                                         0.00803680255809892,
                                         0.00142500168156604,
                                         0.000150239383756746,
                                         7.72223626398686e-06,
                                         1.30924114956566e-07,
                                         2.87858305422582e-10,
                                         2.81189929457405e-15,
                                         6.47727195424872e-08,
                                         4.37165053434211e-05,
                                         0.00125387134750373,
                                         0.0104845464177872,
                                         0.0434316752984702,
                                         0.110917319147433,
                                         0.194407302288855,
                                         0.247274372402104,
                                         0.234460048228998,
                                         0.166895735944741,
                                         0.0883048766953601,
                                         0.0337534697129237,
                                         0.00883558695004361,
                                         0.00144261382141562,
                                         0.000124175896644691,
                                         4.03759533312787e-06,
                                         2.23028153069567e-08,
                                         1.16186566613089e-12,
                                         3.29196546870121e-10,
                                         1.18490575496006e-06,
                                         8.53825517624902e-05,
                                         0.00136922468987815,
                                         0.00949859960756553,
                                         0.0375663180167253,
                                         0.097206821907109,
                                         0.177322213694286,
                                         0.237376811231211,
                                         0.237376811231211,
                                         0.177322213694286,
                                         0.097206821907109,
                                         0.0375663180167253,
                                         0.00949859960756553,
                                         0.00136922468987816,
                                         8.53825517624902e-05,
                                         1.18490575496006e-06,
                                         3.29196546870121e-10,
                                         1.16186566613089e-12,
                                         2.23028153069567e-08,
                                         4.03759533312787e-06,
                                         0.000124175896644691,
                                         0.00144261382141562,
                                         0.00883558695004361,
                                         0.0337534697129237,
                                         0.0883048766953601,
                                         0.166895735944741,
                                         0.234460048228998,
                                         0.247274372402104,
                                         0.194407302288855,
                                         0.110917319147433,
                                         0.0434316752984702,
                                         0.0104845464177873,
                                         0.00125387134750373,
                                         4.37165053434211e-05,
                                         6.47727195424872e-08,
                                         2.81189929457405e-15,
                                         2.87858305422582e-10,
                                         1.30924114956566e-07,
                                         7.72223626398683e-06,
                                         0.000150239383756746,
                                         0.00142500168156604,
                                         0.00803680255809892,
                                         0.0301543124007233,
                                         0.0804628456418593,
                                         0.158797114214449,
                                         0.236449428956247,
                                         0.2666070321879,
                                         0.224565633770294,
                                         0.136174810519298,
                                         0.0550513241040587,
                                         0.0126264139677562,
                                         0.00110598761497857,
                                         8.73921028843073e-06,
                                         4.46594040891474e-18,
                                         2.43818781965778e-12,
                                         2.78603020270286e-09,
                                         3.15150631355195e-07,
                                         1.02680225735716e-05,
                                         0.000150821989398143,
                                         0.00125579136201721,
                                         0.00675746006271106,
                                         0.0254574471553447,
                                         0.0705806539775549,
                                         0.148377055075893,
                                         0.239938502225569,
                                         0.298370953648158,
                                         0.280192363021321,
                                         0.189694704775925,
                                         0.0834404016521852,
                                         0.0183621870135309,
                                         0.000773787240746493,
                                         4.20322175505829e-21,
                                         1.22380371936179e-14,
                                         3.51324318210508e-11,
                                         7.62165869312107e-09,
                                         4.1585895171168e-07,
                                         9.45954334740349e-06,
                                         0.000116280770895153,
                                         0.000897374581829956,
                                         0.00477299008972669,
                                         0.0185902388131678,
                                         0.0551761563064615,
                                         0.127963027059908,
                                         0.234923254214993,
                                         0.341642606549017,
                                         0.387345812489669,
                                         0.326760059508523,
                                         0.180656988869348,
                                         0.0406001050391349,
                                         1.78018114373359e-24,
                                         2.76419638040961e-17,
                                         1.99362337855672e-13,
                                         8.29455947560223e-11,
                                         7.5791029787698e-09,
                                         2.66985817613996e-07,
                                         4.84519016456138e-06,
                                         5.3626142025046e-05,
                                         0.000402697309589852,
                                         0.00220341739335867,
                                         0.00923312368235811,
                                         0.0307101247365599,
                                         0.0832353840522244,
                                         0.187456400338528,
                                         0.355922166534538,
                                         0.575829699631459,
                                         0.79982991249792,
                                         0.958617303406749,

                                         0.962676326012298,
                                         0.81794095357733,
                                         0.608756635627188,
                                         0.395263672167135,
                                         0.222547845536133,
                                         0.107740691917051,
                                         0.0443408057330429,
                                         0.0152798865331713,
                                         0.00432138010277356,
                                         0.0009765625,
                                         0.000170074126157063,
                                         2.17109048222588e-05,
                                         1.89112849853936e-06,
                                         1.01012552894013e-07,
                                         2.7942710516002e-09,
                                         2.99521756691e-11,
                                         7.08649615993212e-14,
                                         9.71943720905241e-18,
                                         6.22190473905769e-25,
                                         0.0366880473939491,
                                         0.166040437858573,
                                         0.309772237701712,
                                         0.384451047382647,
                                         0.360834378290398,
                                         0.26889289006638,
                                         0.162104595358301,
                                         0.0793187586198047,
                                         0.031269723855026,
                                         0.009765625,
                                         0.00235037235435517,
                                         0.000418236704644752,
                                         5.17284295270048e-05,
                                         4.04739684207419e-06,
                                         1.72339178246796e-07,
                                         3.07945758633351e-09,
                                         1.39262045970023e-11,
                                         4.78794553997668e-15,
                                         1.63259721365886e-21,
                                         0.000629189430906975,
                                         0.0151676500579416,
                                         0.0709338923564056,
                                         0.168270393179474,
                                         0.263272159337914,
                                         0.301989185968494,
                                         0.266685612289478,
                                         0.185286680951319,
                                         0.101821182821642,
                                         0.0439453125,
                                         0.014616641860955,
                                         0.00362559156999042,
                                         0.000636722301276013,
                                         7.29774600042941e-05,
                                         4.78312816281637e-06,
                                         1.42473007932929e-07,
                                         1.23153426665713e-09,
                                         1.06137731026177e-12,
                                         1.92773467004681e-18,
                                         6.39432134135382e-06,
                                         0.000821067664568085,
                                         0.00962545590061507,
                                         0.0436446071869945,
                                         0.113830378909621,
                                         0.200983165633474,
                                         0.259992047525955,
                                         0.256488981816057,
                                         0.196475520548406,
                                         0.1171875,
                                         0.0538660041473489,
                                         0.0186248091520161,
                                         0.00464437253068985,
                                         0.000779754549183316,
                                         7.86676607775102e-05,
                                         3.90613436543052e-06,
                                         6.45381404429751e-08,
                                         1.39426913982693e-10,
                                         1.3488749326705e-15,
                                         4.26458537151439e-08,
                                         2.91681519790959e-05,
                                         0.000857152590670526,
                                         0.00742887045345802,
                                         0.0322983803031486,
                                         0.0877803462883769,
                                         0.166337380854215,
                                         0.233003552009351,
                                         0.248798681670849,
                                         0.205078125,
                                         0.130272045039663,
                                         0.0627876314656557,
                                         0.022231734667363,
                                         0.00546759587466385,
                                         0.000849082268134016,
                                         7.02798910705738e-05,
                                         2.21950287379146e-06,
                                         1.20196756154275e-08,
                                         6.1939176357832e-13,
                                         1.9503042424292e-10,
                                         7.1052943931263e-07,
                                         5.23405368630307e-05,
                                         0.000867078022437926,
                                         0.0062841494322611,
                                         0.0262892438329779,
                                         0.0729731091816542,
                                         0.145144160273167,
                                         0.216038373472826,
                                         0.24609375,
                                         0.216038373472826,
                                         0.145144160273167,
                                         0.0729731091816542,
                                         0.0262892438329779,
                                         0.00628414943226111,
                                         0.000867078022437926,
                                         5.23405368630307e-05,
                                         7.1052943931263e-07,
                                         1.9503042424292e-10,
                                         6.1939176357832e-13,
                                         1.20196756154275e-08,
                                         2.21950287379146e-06,
                                         7.02798910705738e-05,
                                         0.000849082268134014,
                                         0.00546759587466386,
                                         0.022231734667363,
                                         0.0627876314656557,
                                         0.130272045039663,
                                         0.205078125,
                                         0.248798681670848,
                                         0.233003552009352,
                                         0.166337380854215,
                                         0.0877803462883768,
                                         0.0322983803031487,
                                         0.00742887045345802,
                                         0.000857152590670526,
                                         2.91681519790959e-05,
                                         4.26458537151439e-08,
                                         1.3488749326705e-15,
                                         1.39426913982693e-10,
                                         6.45381404429751e-08,
                                         3.90613436543052e-06,
                                         7.866766077751e-05,
                                         0.000779754549183317,
                                         0.00464437253068985,
                                         0.0186248091520161,
                                         0.0538660041473489,
                                         0.1171875,
                                         0.196475520548406,
                                         0.256488981816057,
                                         0.259992047525955,
                                         0.200983165633474,
                                         0.113830378909621,
                                         0.0436446071869945,
                                         0.00962545590061507,
                                         0.000821067664568085,
                                         6.39432134135382e-06,
                                         1.92773467004681e-18,
                                         1.06137731026177e-12,
                                         1.23153426665713e-09,
                                         1.42473007932929e-07,
                                         4.78312816281635e-06,
                                         7.29774600042943e-05,
                                         0.000636722301276013,
                                         0.00362559156999041,
                                         0.014616641860955,
                                         0.0439453125,
                                         0.101821182821642,
                                         0.185286680951319,
                                         0.266685612289478,
                                         0.301989185968494,
                                         0.263272159337914,
                                         0.168270393179474,
                                         0.0709338923564056,
                                         0.0151676500579416,
                                         0.000629189430906975,
                                         1.63259721365886e-21,
                                         4.78794553997668e-15,
                                         1.39262045970023e-11,
                                         3.07945758633351e-09,
                                         1.72339178246795e-07,
                                         4.0473968420742e-06,
                                         5.17284295270048e-05,
                                         0.000418236704644752,
                                         0.00235037235435517,
                                         0.009765625,
                                         0.031269723855026,
                                         0.0793187586198047,
                                         0.162104595358301,
                                         0.26889289006638,
                                         0.360834378290398,
                                         0.384451047382647,
                                         0.309772237701712,
                                         0.166040437858573,
                                         0.0366880473939491,
                                         6.22190473905769e-25,
                                         9.71943720905241e-18,
                                         7.08649615993212e-14,
                                         2.99521756691e-11,
                                         2.79427105160019e-09,
                                         1.01012552894013e-07,
                                         1.89112849853936e-06,
                                         2.17109048222588e-05,
                                         0.000170074126157063,
                                         0.0009765625,
                                         0.00432138010277356,
                                         0.0152798865331713,
                                         0.0443408057330429,
                                         0.107740691917051,
                                         0.222547845536133,
                                         0.395263672167135,
                                         0.608756635627188,
                                         0.81794095357733,
                                         0.962676326012298,

                                         0.966169340127348,
                                         0.833782548557145,
                                         0.638427188477989,
                                         0.432308291513141,
                                         0.257613093122779,
                                         0.134176214764561,
                                         0.0605288474456779,
                                         0.0233689002733215,
                                         0.00760197236510522,
                                         0.00204149057083855,
                                         0.000440498382831158,
                                         7.36338638860294e-05,
                                         9.06738255827176e-06,
                                         7.65543135521692e-07,
                                         3.98174422551463e-08,
                                         1.07735415972467e-09,
                                         1.13449441942808e-11,
                                         2.64814588948838e-14,
                                         3.59840755781307e-18,
                                         2.2916741396188e-25,
                                         0.0333091241267707,
                                         0.152953195901625,
                                         0.293018086008986,
                                         0.37817641895874,
                                         0.374202761580535,
                                         0.29848005644855,
                                         0.195958059718939,
                                         0.106542440736242,
                                         0.0478124960806966,
                                         0.0175124566140777,
                                         0.00513504938134459,
                                         0.0011707453998067,
                                         0.000198882959016188,
                                         2.36466128157932e-05,
                                         1.78991981808438e-06,
                                         7.41684899125558e-08,
                                         1.29688505048659e-09,
                                         5.76977468501299e-12,
                                         1.96157354321015e-15,
                                         6.64726362313097e-22,
                                         0.000516756190458155,
                                         0.0126263209510109,
                                         0.060518756288993,
                                         0.148870222937765,
                                         0.244601185773165,
                                         0.298791070490661,
                                         0.285480448664961,
                                         0.218584580163046,
                                         0.135322203535233,
                                         0.0676019588179099,
                                         0.0269375097150574,
                                         0.00837644696984145,
                                         0.00196302450125786,
                                         0.000328685638041211,
                                         3.62081476903683e-05,
                                         2.29769772624411e-06,
                                         6.67134066433374e-08,
                                         5.65702781768258e-10,
                                         4.81184195125533e-13,
                                         8.67651766460899e-19,
                                         4.75077411418832e-06,
                                         0.000617662609283039,
                                         0.0074069903686154,
                                         0.0347278143278342,
                                         0.0947471917079598,
                                         0.177245871767192,
                                         0.246459650976762,
                                         0.265749600600185,
                                         0.226961881835078,
                                         0.15464208851268,
                                         0.0837387516737918,
                                         0.0355151329895186,
                                         0.0114818028664081,
                                         0.00270737699296362,
                                         0.000434045500038717,
                                         4.21815535384305e-05,
                                         2.03367228634804e-06,
                                         3.28680541097985e-08,
                                         6.99478437914175e-11,
                                         6.71126250750494e-16,
                                         2.86623893625167e-08,
                                         1.98287398703607e-05,
                                         0.00059492590778089,
                                         0.00531638445468933,
                                         0.0240848225642736,
                                         0.0690007735950492,
                                         0.139631885992169,
                                         0.212028887956103,
                                         0.249807862195836,
                                         0.232148268663495,
                                         0.170830273853312,
                                         0.0988180691045593,
                                         0.0440721023946443,
                                         0.014634775223593,
                                         0.00341455037621397,
                                         0.000508184673978839,
                                         4.0683484167297e-05,
                                         1.253224823364e-06,
                                         6.67277721512876e-09,
                                         3.40668763199343e-13,
                                         1.18577857366502e-10,
                                         4.36497857498504e-07,
                                         3.27662832588388e-05,
                                         0.000558082629739582,
                                         0.00419820608020087,
                                         0.0184193791727833,
                                         0.0542458571909425,
                                         0.116000710167216,
                                         0.188539555660439,
                                         0.238971653814661,
                                         0.238971653814661,
                                         0.188539555660439,
                                         0.116000710167216,
                                         0.0542458571909425,
                                         0.0184193791727833,
                                         0.00419820608020087,
                                         0.000558082629739584,
                                         3.27662832588388e-05,
                                         4.36497857498511e-07,
                                         1.18577857366512e-10,
                                         3.4066876319931e-13,
                                         6.67277721512864e-09,
                                         1.253224823364e-06,
                                         4.06834841672968e-05,
                                         0.000508184673978839,
                                         0.00341455037621397,
                                         0.014634775223593,
                                         0.0440721023946443,
                                         0.0988180691045593,
                                         0.170830273853312,
                                         0.232148268663495,
                                         0.249807862195836,
                                         0.212028887956103,
                                         0.139631885992169,
                                         0.0690007735950491,
                                         0.0240848225642736,
                                         0.00531638445468934,
                                         0.00059492590778089,
                                         1.98287398703609e-05,
                                         2.86623893625186e-08,
                                         6.71126250750418e-16,
                                         6.9947843791416e-11,
                                         3.28680541097985e-08,
                                         2.03367228634803e-06,
                                         4.21815535384305e-05,
                                         0.000434045500038718,
                                         0.00270737699296362,
                                         0.0114818028664081,
                                         0.0355151329895186,
                                         0.0837387516737918,
                                         0.15464208851268,
                                         0.226961881835078,
                                         0.265749600600185,
                                         0.246459650976762,
                                         0.177245871767192,
                                         0.0947471917079598,
                                         0.0347278143278342,
                                         0.0074069903686154,
                                         0.000617662609283044,
                                         4.75077411418855e-06,
                                         8.67651766460786e-19,
                                         4.81184195125521e-13,
                                         5.65702781768258e-10,
                                         6.6713406643337e-08,
                                         2.29769772624411e-06,
                                         3.62081476903684e-05,
                                         0.000328685638041211,
                                         0.00196302450125786,
                                         0.00837644696984145,
                                         0.0269375097150574,
                                         0.06760195881791,
                                         0.135322203535233,
                                         0.218584580163046,
                                         0.285480448664961,
                                         0.298791070490661,
                                         0.244601185773165,
                                         0.148870222937765,
                                         0.060518756288993,
                                         0.012626320951011,
                                         0.000516756190458171,
                                         6.64726362313001e-22,
                                         1.9615735432101e-15,
                                         5.76977468501299e-12,
                                         1.29688505048658e-09,
                                         7.41684899125558e-08,
                                         1.78991981808439e-06,
                                         2.36466128157932e-05,
                                         0.000198882959016188,
                                         0.0011707453998067,
                                         0.00513504938134459,
                                         0.0175124566140777,
                                         0.0478124960806966,
                                         0.106542440736242,
                                         0.195958059718939,
                                         0.298480056448549,
                                         0.374202761580535,
                                         0.378176418958741,
                                         0.293018086008986,
                                         0.152953195901625,
                                         0.0333091241267712,
                                         2.29167413961843e-25,
                                         3.59840755781296e-18,
                                         2.64814588948838e-14,
                                         1.13449441942807e-11,
                                         1.07735415972467e-09,
                                         3.98174422551464e-08,
                                         7.65543135521692e-07,
                                         9.06738255827176e-06,
                                         7.36338638860294e-05,
                                         0.000440498382831158,
                                         0.00204149057083855,
                                         0.00760197236510522,
                                         0.0233689002733215,
                                         0.0605288474456779,
                                         0.134176214764561,
                                         0.257613093122779,
                                         0.432308291513141,
                                         0.638427188477989,
                                         0.833782548557145,
                                         0.966169340127348,

                                         0.969196362665879,
                                         0.847704426975941,
                                         0.665176677462213,
                                         0.466988068051841,
                                         0.292148223195395,
                                         0.161963683642952,
                                         0.0789860642512177,
                                         0.0335616432273308,
                                         0.0122724296175237,
                                         0.00380089157090121,
                                         0.0009765625,
                                         0.000202537913890285,
                                         3.26866153922499e-05,
                                         3.9025150032946e-06,
                                         3.20755155243541e-07,
                                         1.63054103840275e-08,
                                         4.32848719748264e-10,
                                         4.48874479981532e-12,
                                         1.03562858331142e-14,
                                         1.39599356280927e-18,
                                         8.85098598953817e-26,
                                         0.0303717462907727,
                                         0.141223947786249,
                                         0.276798488971778,
                                         0.369477245463166,
                                         0.382539722189285,
                                         0.323376945448396,
                                         0.228251053793098,
                                         0.135641740200633,
                                         0.0678382272462051,
                                         0.0283496411353113,
                                         0.009765625,
                                         0.00271546523647043,
                                         0.000591324689810339,
                                         9.65593747441969e-05,
                                         1.1099702226979e-05,
                                         8.16658196039696e-07,
                                         3.30569551478592e-08,
                                         5.6733947429314e-10,
                                         2.48872738681108e-12,
                                         8.37952728112703e-16,
                                         2.82444853349555e-22,
                                         0.000428292298369966,
                                         0.0105872887496489,
                                         0.0518325923651094,
                                         0.131547356162754,
                                         0.225404374715816,
                                         0.29054471300572,
                                         0.296816214647453,
                                         0.246692532365427,
                                         0.168745012088026,
                                         0.0951529297479188,
                                         0.0439453125,
                                         0.0163830469514622,
                                         0.00481387253046671,
                                         0.00107511970598538,
                                         0.000172846872080127,
                                         1.84060852841493e-05,
                                         1.13606210485593e-06,
                                         3.2268115486763e-08,
                                         2.69130636943849e-10,
                                         2.26343556994864e-13,
                                         4.05590889818101e-19,
                                         3.57904356652892e-06,
                                         0.000470345925929926,
                                         0.00575172831811236,
                                         0.0277544591982057,
                                         0.078705369432847,
                                         0.154693874897857,
                                         0.228727651874328,
                                         0.265873389316432,
                                         0.248738847067859,
                                         0.189257449101598,
                                         0.1171875,
                                         0.0585735198704672,
                                         0.0232230564647952,
                                         0.00709374247102847,
                                         0.00159502663602081,
                                         0.000245832191896757,
                                         2.31364923167589e-05,
                                         1.08757801972873e-06,
                                         1.7246666680453e-08,
                                         3.6230386944937e-11,
                                         3.45142893774758e-16,
                                         1.96274112108381e-08,
                                         1.37125849706167e-05,
                                         0.000418854431071093,
                                         0.0038428437923783,
                                         0.0180349814212772,
                                         0.0540508559283367,
                                         0.115669547681609,
                                         0.188045548522093,
                                         0.240616611192008,
                                         0.247031944011744,
                                         0.205078125,
                                         0.137428651939319,
                                         0.0735213522950458,
                                         0.030715906019222,
                                         0.00965925589203863,
                                         0.00215469296301316,
                                         0.000309216227130743,
                                         2.40556201476269e-05,
                                         7.25298062291825e-07,
                                         3.80581019012651e-09,
                                         1.92743169779205e-13,
                                         7.380779911314e-11,
                                         2.74134991718263e-07,
                                         2.09156353578614e-05,
                                         0.00036485129354411,
                                         0.00283380677402376,
                                         0.0129501628729379,
                                         0.0401109178947722,
                                         0.091199916282101,
                                         0.159606580192867,
                                         0.221103922520918,
                                         0.24609375,
                                         0.221103922520918,
                                         0.159606580192867,
                                         0.0911999162821011,
                                         0.0401109178947722,
                                         0.0129501628729379,
                                         0.00283380677402376,
                                         0.00036485129354411,
                                         2.09156353578613e-05,
                                         2.74134991718268e-07,
                                         7.38077991131334e-11,
                                         1.92743169779226e-13,
                                         3.80581019012643e-09,
                                         7.25298062291831e-07,
                                         2.40556201476269e-05,
                                         0.000309216227130743,
                                         0.00215469296301316,
                                         0.00965925589203863,
                                         0.030715906019222,
                                         0.0735213522950458,
                                         0.137428651939319,
                                         0.205078125,
                                         0.247031944011744,
                                         0.240616611192008,
                                         0.188045548522093,
                                         0.115669547681609,
                                         0.0540508559283367,
                                         0.0180349814212772,
                                         0.0038428437923783,
                                         0.00041885443107109,
                                         1.37125849706169e-05,
                                         1.96274112108367e-08,
                                         3.45142893774801e-16,
                                         3.62303869449361e-11,
                                         1.72466666804532e-08,
                                         1.08757801972873e-06,
                                         2.31364923167589e-05,
                                         0.000245832191896758,
                                         0.00159502663602081,
                                         0.00709374247102846,
                                         0.0232230564647952,
                                         0.0585735198704672,
                                         0.1171875,
                                         0.189257449101598,
                                         0.248738847067859,
                                         0.265873389316432,
                                         0.228727651874328,
                                         0.154693874897857,
                                         0.078705369432847,
                                         0.0277544591982057,
                                         0.00575172831811234,
                                         0.000470345925929931,
                                         3.57904356652873e-06,
                                         4.05590889818158e-19,
                                         2.26343556994858e-13,
                                         2.69130636943852e-10,
                                         3.2268115486763e-08,
                                         1.13606210485593e-06,
                                         1.84060852841493e-05,
                                         0.000172846872080127,
                                         0.00107511970598537,
                                         0.00481387253046671,
                                         0.0163830469514622,
                                         0.0439453125,
                                         0.0951529297479188,
                                         0.168745012088026,
                                         0.246692532365427,
                                         0.296816214647453,
                                         0.29054471300572,
                                         0.225404374715816,
                                         0.131547356162754,
                                         0.0518325923651092,
                                         0.010587288749649,
                                         0.000428292298369951,
                                         2.824448533496e-22,
                                         8.37952728112677e-16,
                                         2.48872738681111e-12,
                                         5.67339474293141e-10,
                                         3.30569551478592e-08,
                                         8.16658196039696e-07,
                                         1.1099702226979e-05,
                                         9.65593747441968e-05,
                                         0.000591324689810339,
                                         0.00271546523647043,
                                         0.009765625,
                                         0.0283496411353113,
                                         0.0678382272462051,
                                         0.135641740200633,
                                         0.228251053793098,
                                         0.323376945448396,
                                         0.382539722189285,
                                         0.369477245463166,
                                         0.276798488971778,
                                         0.14122394778625,
                                         0.0303717462907722,
                                         8.85098598953974e-26,
                                         1.39599356280923e-18,
                                         1.03562858331143e-14,
                                         4.48874479981532e-12,
                                         4.32848719748264e-10,
                                         1.63054103840275e-08,
                                         3.20755155243541e-07,
                                         3.9025150032946e-06,
                                         3.26866153922499e-05,
                                         0.000202537913890285,
                                         0.0009765625,
                                         0.00380089157090121,
                                         0.0122724296175237,
                                         0.0335616432273308,
                                         0.0789860642512177,
                                         0.161963683642952,
                                         0.292148223195395,
                                         0.466988068051841,
                                         0.665176677462213,
                                         0.847704426975941,
                                         0.969196362665879,

                                         0.97183636269485,
                                         0.859994498193509,
                                         0.689318231052616,
                                         0.499321431933334,
                                         0.325784334992656,
                                         0.190582548875071,
                                         0.0993622620033997,
                                         0.0458110105100205,
                                         0.018493431457001,
                                         0.00645457378555904,
                                         0.00191637036736594,
                                         0.000473965694117768,
                                         9.49985867061881e-05,
                                         1.48724512857194e-05,
                                         1.72875699151345e-06,
                                         1.38822897007112e-07,
                                         6.9182021842893e-09,
                                         1.80638313341707e-10,
                                         1.84849442785943e-12,
                                         4.2218189947359e-15,
                                         5.65126651678194e-19,
                                         3.56920976045675e-26,
                                         0.0278029601685476,
                                         0.130695515028466,
                                         0.261292984425183,
                                         0.359107107756817,
                                         0.386650134004196,
                                         0.34361660521532,
                                         0.258074701146795,
                                         0.165442193474232,
                                         0.0906888735002966,
                                         0.0423304150995412,
                                         0.0166650335740415,
                                         0.00545029691851408,
                                         0.00144854565204952,
                                         0.000303281591042299,
                                         4.7869351248555e-05,
                                         5.34486991680971e-06,
                                         3.83709222983777e-07,
                                         1.52202540774458e-08,
                                         2.57024398766454e-10,
                                         1.11376002217502e-12,
                                         3.71861124017847e-16,
                                         1.24759659053184e-22,
                                         0.000357932755670569,
                                         0.0089379559497142,
                                         0.0445705760929939,
                                         0.116219849514506,
                                         0.206499329557926,
                                         0.278790306009049,
                                         0.30163512296427,
                                         0.268865575864346,
                                         0.200125774826806,
                                         0.12492518420485,
                                         0.0652146928062492,
                                         0.0282036898258344,
                                         0.00993939026327332,
                                         0.0027830567244191,
                                         0.000596476925382181,
                                         9.26031351425354e-05,
                                         9.57687326076212e-06,
                                         5.77093853753522e-07,
                                         1.60821116117676e-08,
                                         1.32219696022946e-10,
                                         1.10110384663917e-13,
                                         1.96240571646416e-19,
                                         2.73066276887248e-06,
                                         0.000362219606010174,
                                         0.00450531295813033,
                                         0.0222891155096845,
                                         0.0653544772778581,
                                         0.13404092394009,
                                         0.208917396859631,
                                         0.258928789537646,
                                         0.261702736557977,
                                         0.218475976032549,
                                         0.151231037401991,
                                         0.0864864191121688,
                                         0.0404150870003068,
                                         0.0151340193601801,
                                         0.00440439207175185,
                                         0.000950759030671518,
                                         0.000141645069036237,
                                         1.29666328523864e-05,
                                         5.96304394987537e-07,
                                         9.30158378071045e-09,
                                         1.93211067692853e-11,
                                         1.82919345955359e-16,
                                         1.36711173242998e-08,
                                         9.63329842291002e-06,
                                         0.000298862205762779,
                                         0.00280527006516833,
                                         0.013573797354682,
                                         0.0422928033826532,
                                         0.0949591061291199,
                                         0.163642007087772,
                                         0.224586071247775,
                                         0.250741774104375,
                                         0.230147216198583,
                                         0.17404406510921,
                                         0.107844154336717,
                                         0.0540077181557039,
                                         0.0213426141597444,
                                         0.00640596474988869,
                                         0.00137482840744242,
                                         0.000191195336725523,
                                         1.45098280261877e-05,
                                         4.29424646079623e-07,
                                         2.22487203594611e-09,
                                         1.11892195591294e-13,
                                         4.69335223522271e-11,
                                         1.75679574897257e-07,
                                         1.35944057460489e-05,
                                         0.000242102747085099,
                                         0.00193317234835809,
                                         0.00915037160065219,
                                         0.0295966002882675,
                                         0.0709173422608651,
                                         0.132160164127513,
                                         0.197329900934073,
                                         0.240167212991924,
                                         0.240167212991924,
                                         0.197329900934073,
                                         0.132160164127513,
                                         0.0709173422608651,
                                         0.0295966002882675,
                                         0.0091503716006522,
                                         0.00193317234835809,
                                         0.000242102747085098,
                                         1.3594405746049e-05,
                                         1.75679574897257e-07,
                                         4.69335223522271e-11,
                                         1.11892195591294e-13,
                                         2.22487203594611e-09,
                                         4.29424646079619e-07,
                                         1.45098280261878e-05,
                                         0.000191195336725524,
                                         0.00137482840744242,
                                         0.00640596474988869,
                                         0.0213426141597444,
                                         0.0540077181557039,
                                         0.107844154336716,
                                         0.17404406510921,
                                         0.230147216198583,
                                         0.250741774104375,
                                         0.224586071247775,
                                         0.163642007087772,
                                         0.0949591061291199,
                                         0.0422928033826532,
                                         0.013573797354682,
                                         0.00280527006516832,
                                         0.000298862205762781,
                                         9.63329842291002e-06,
                                         1.36711173242998e-08,
                                         1.82919345955359e-16,
                                         1.93211067692853e-11,
                                         9.30158378071035e-09,
                                         5.96304394987541e-07,
                                         1.29666328523864e-05,
                                         0.000141645069036237,
                                         0.000950759030671518,
                                         0.00440439207175185,
                                         0.0151340193601801,
                                         0.0404150870003068,
                                         0.0864864191121688,
                                         0.151231037401991,
                                         0.218475976032549,
                                         0.261702736557977,
                                         0.258928789537646,
                                         0.208917396859631,
                                         0.13404092394009,
                                         0.065354477277858,
                                         0.0222891155096844,
                                         0.00450531295813035,
                                         0.000362219606010174,
                                         2.73066276887248e-06,
                                         1.96240571646416e-19,
                                         1.10110384663917e-13,
                                         1.32219696022945e-10,
                                         1.60821116117677e-08,
                                         5.77093853753524e-07,
                                         9.57687326076212e-06,
                                         9.26031351425353e-05,
                                         0.000596476925382181,
                                         0.0027830567244191,
                                         0.00993939026327331,
                                         0.0282036898258344,
                                         0.0652146928062492,
                                         0.12492518420485,
                                         0.200125774826806,
                                         0.268865575864346,
                                         0.30163512296427,
                                         0.278790306009049,
                                         0.206499329557926,
                                         0.116219849514506,
                                         0.044570576092994,
                                         0.0089379559497142,
                                         0.000357932755670569,
                                         1.24759659053184e-22,
                                         3.71861124017847e-16,
                                         1.113760022175e-12,
                                         2.57024398766456e-10,
                                         1.52202540774458e-08,
                                         3.83709222983777e-07,
                                         5.34486991680971e-06,
                                         4.7869351248555e-05,
                                         0.000303281591042299,
                                         0.00144854565204952,
                                         0.00545029691851408,
                                         0.0166650335740415,
                                         0.0423304150995412,
                                         0.0906888735002966,
                                         0.165442193474232,
                                         0.258074701146795,
                                         0.34361660521532,
                                         0.386650134004196,
                                         0.359107107756817,
                                         0.261292984425184,
                                         0.130695515028466,
                                         0.0278029601685476,
                                         3.56920976045675e-26,
                                         5.65126651678194e-19,
                                         4.22181899473584e-15,
                                         1.84849442785945e-12,
                                         1.80638313341708e-10,
                                         6.9182021842893e-09,
                                         1.38822897007112e-07,
                                         1.72875699151345e-06,
                                         1.48724512857194e-05,
                                         9.4998586706188e-05,
                                         0.000473965694117768,
                                         0.00191637036736594,
                                         0.00645457378555904,
                                         0.018493431457001,
                                         0.0458110105100205,
                                         0.0993622620033997,
                                         0.190582548875071,
                                         0.325784334992656,
                                         0.499321431933334,
                                         0.689318231052616,
                                         0.859994498193509,
                                         0.97183636269485,

                                         0.974152336512357,
                                         0.870890738410956,
                                         0.711137569079148,
                                         0.529382701611342,
                                         0.35826434168984,
                                         0.219585258606828,
                                         0.121284193888987,
                                         0.0599840063183176,
                                         0.0263504789566702,
                                         0.0101774022661878,
                                         0.00341179928062678,
                                         0.0009765625,
                                         0.000233670750070241,
                                         4.54650313374672e-05,
                                         6.93193688130939e-06,
                                         7.87185481346128e-07,
                                         6.19430199635846e-08,
                                         3.03384809860736e-09,
                                         7.80784010111789e-11,
                                         7.897437979629e-13,
                                         1.78781188527014e-15,
                                         2.37856223120778e-19,
                                         1.49716308026752e-26,
                                         0.0255441290591844,
                                         0.12122683219606,
                                         0.246598506813851,
                                         0.347649327619059,
                                         0.387290030161325,
                                         0.359453110763975,
                                         0.284854459532021,
                                         0.194912471990385,
                                         0.115556697634692,
                                         0.059243539288049,
                                         0.0260942946024372,
                                         0.009765625,
                                         0.00305521842663148,
                                         0.000781040293214856,
                                         0.00015806946776668,
                                         2.42255092270767e-05,
                                         2.637385160013e-06,
                                         1.85333858396893e-07,
                                         7.22267674352819e-09,
                                         1.20258165954089e-10,
                                         5.15566867978456e-13,
                                         1.70875356583003e-16,
                                         5.70958950842867e-23,
                                         0.000301417065094517,
                                         0.00759357619533047,
                                         0.0384804175635161,
                                         0.102736686676291,
                                         0.1884002885682,
                                         0.26478527222613,
                                         0.30106089863035,
                                         0.285007510020393,
                                         0.228041686664863,
                                         0.155187795963005,
                                         0.0898090625082619,
                                         0.0439453125,
                                         0.0179759847316372,
                                         0.00603784413550146,
                                         0.00162201137727249,
                                         0.000335491304207511,
                                         5.05320893113658e-05,
                                         5.09481262025643e-06,
                                         3.00661596544673e-07,
                                         8.24053564228719e-09,
                                         6.69053265034247e-11,
                                         5.52404061450349e-14,
                                         9.79835513778513e-20,
                                         2.10766115943913e-06,
                                         0.000281870854470666,
                                         0.00355832267448117,
                                         0.0179914404386384,
                                         0.0543104033912101,
                                         0.115585038680077,
                                         0.188556671325903,
                                         0.246961467335603,
                                         0.266679471264955,
                                         0.240896423840354,
                                         0.183168581916403,
                                         0.1171875,
                                         0.0626757204570221,
                                         0.0276596432628222,
                                         0.00986315867198268,
                                         0.00275325121487579,
                                         0.00057374269678853,
                                         8.29961378009e-05,
                                         7.41675544209074e-06,
                                         3.346204867543e-07,
                                         5.14508562795235e-09,
                                         1.05825544232573e-11,
                                         9.96454987558822e-17,
                                         9.6717055064884e-09,
                                         6.86630839212856e-06,
                                         0.000215933585676545,
                                         0.00206764336398175,
                                         0.0102743370773176,
                                         0.0331114909334643,
                                         0.0774994764066325,
                                         0.140433721951308,
                                         0.204660599850124,
                                         0.245398812580237,
                                         0.245160850749948,
                                         0.205078125,
                                         0.14340863286226,
                                         0.0831535065196741,
                                         0.0393593085140326,
                                         0.0148279027000925,
                                         0.0042750027870204,
                                         0.000887272184162689,
                                         0.00012006578786574,
                                         8.917003302867e-06,
                                         2.59653293668322e-07,
                                         1.33043442781865e-09,
                                         6.65015106286798e-14,
                                         3.04332640085431e-11,
                                         1.14693719229727e-07,
                                         8.98541752495209e-06,
                                         0.000162940305315529,
                                         0.00133280860644742,
                                         0.00650427728723505,
                                         0.021842323314808,
                                         0.0547591644701087,
                                         0.10770158566076,
                                         0.171418526819617,
                                         0.225006183714702,
                                         0.24609375,
                                         0.225006183714702,
                                         0.171418526819617,
                                         0.10770158566076,
                                         0.0547591644701088,
                                         0.021842323314808,
                                         0.00650427728723506,
                                         0.00133280860644742,
                                         0.000162940305315529,
                                         8.98541752495216e-06,
                                         1.14693719229727e-07,
                                         3.04332640085399e-11,
                                         6.65015106286882e-14,
                                         1.33043442781865e-09,
                                         2.59653293668319e-07,
                                         8.917003302867e-06,
                                         0.00012006578786574,
                                         0.000887272184162687,
                                         0.0042750027870204,
                                         0.0148279027000925,
                                         0.0393593085140326,
                                         0.0831535065196741,
                                         0.14340863286226,
                                         0.205078125,
                                         0.245160850749948,
                                         0.245398812580237,
                                         0.204660599850124,
                                         0.140433721951309,
                                         0.0774994764066324,
                                         0.0331114909334644,
                                         0.0102743370773176,
                                         0.00206764336398175,
                                         0.000215933585676546,
                                         6.86630839212856e-06,
                                         9.67170550648758e-09,
                                         9.9645498755897e-17,
                                         1.05825544232573e-11,
                                         5.14508562795229e-09,
                                         3.346204867543e-07,
                                         7.41675544209074e-06,
                                         8.29961378008998e-05,
                                         0.000573742696788531,
                                         0.00275325121487579,
                                         0.00986315867198268,
                                         0.0276596432628222,
                                         0.0626757204570221,
                                         0.1171875,
                                         0.183168581916403,
                                         0.240896423840354,
                                         0.266679471264955,
                                         0.246961467335603,
                                         0.188556671325903,
                                         0.115585038680077,
                                         0.0543104033912101,
                                         0.0179914404386384,
                                         0.00355832267448119,
                                         0.000281870854470666,
                                         2.107661159439e-06,
                                         9.79835513778679e-20,
                                         5.52404061450348e-14,
                                         6.69053265034238e-11,
                                         8.24053564228719e-09,
                                         3.00661596544673e-07,
                                         5.09481262025641e-06,
                                         5.05320893113659e-05,
                                         0.00033549130420751,
                                         0.00162201137727249,
                                         0.00603784413550146,
                                         0.0179759847316372,
                                         0.0439453125,
                                         0.0898090625082619,
                                         0.155187795963005,
                                         0.228041686664863,
                                         0.285007510020393,
                                         0.301060898630349,
                                         0.264785272226131,
                                         0.1884002885682,
                                         0.102736686676291,
                                         0.0384804175635163,
                                         0.00759357619533047,
                                         0.000301417065094504,
                                         5.70958950842977e-23,
                                         1.70875356583003e-16,
                                         5.15566867978449e-13,
                                         1.20258165954089e-10,
                                         7.22267674352818e-09,
                                         1.85333858396892e-07,
                                         2.63738516001301e-06,
                                         2.42255092270766e-05,
                                         0.00015806946776668,
                                         0.000781040293214856,
                                         0.00305521842663148,
                                         0.009765625,
                                         0.0260942946024372,
                                         0.059243539288049,
                                         0.115556697634692,
                                         0.194912471990385,
                                         0.284854459532021,
                                         0.359453110763975,
                                         0.387290030161325,
                                         0.347649327619059,
                                         0.246598506813851,
                                         0.12122683219606,
                                         0.0255441290591838,
                                         1.49716308026784e-26,
                                         2.37856223120778e-19,
                                         1.78781188527011e-15,
                                         7.897437979629e-13,
                                         7.80784010111789e-11,
                                         3.03384809860735e-09,
                                         6.19430199635848e-08,
                                         7.87185481346126e-07,
                                         6.93193688130939e-06,
                                         4.54650313374672e-05,
                                         0.000233670750070241,
                                         0.0009765625,
                                         0.00341179928062678,
                                         0.0101774022661878,
                                         0.0263504789566702,
                                         0.0599840063183176,
                                         0.121284193888987,
                                         0.219585258606828,
                                         0.35826434168984,
                                         0.529382701611342,
                                         0.711137569079148,
                                         0.870890738410956,
                                         0.974152336512357,

                                         0.976195016918756,
                                         0.880590726440692,
                                         0.730891618995769,
                                         0.557280270102422,
                                         0.38942036742605,
                                         0.248600782048209,
                                         0.144383946784444,
                                         0.0758873629893497,
                                         0.0358563549653398,
                                         0.0151042277011176,
                                         0.00561372434258422,
                                         0.00181697326526461,
                                         0.000503726206361996,
                                         0.000117097358362496,
                                         2.21990892774127e-05,
                                         3.30710886955505e-06,
                                         3.67950357392297e-07,
                                         2.8442715358608e-08,
                                         1.37201410703638e-09,
                                         3.48639971660941e-11,
                                         3.49051667331629e-13,
                                         7.84047817398448e-16,
                                         1.03752991772221e-19,
                                         6.51118663885669e-27,
                                         0.0235477242904622,
                                         0.112692953105569,
                                         0.232756564715493,
                                         0.335548477439356,
                                         0.385136353136963,
                                         0.371268496361603,
                                         0.308293326487388,
                                         0.223219144075163,
                                         0.141597104926601,
                                         0.0786723616742755,
                                         0.0381224395643371,
                                         0.0159820740308565,
                                         0.00572677268423251,
                                         0.00172431853418634,
                                         0.000426198085411127,
                                         8.37452641402925e-05,
                                         1.2509134218384e-05,
                                         1.33206629787698e-06,
                                         9.18698417272157e-08,
                                         3.52518023182634e-09,
                                         5.79706422555372e-11,
                                         2.46203143326556e-13,
                                         8.10733234663811e-17,
                                         2.69927907795937e-23,
                                         0.000255607628949408,
                                         0.00648980915225342,
                                         0.0333551892709716,
                                         0.090917899015211,
                                         0.171404760284176,
                                         0.249509003409909,
                                         0.29622509823997,
                                         0.295464922531154,
                                         0.251625773571773,
                                         0.184398916399249,
                                         0.116499161579847,
                                         0.0632601551408966,
                                         0.0292979921099965,
                                         0.0114261628273492,
                                         0.00368214040594987,
                                         0.00095429914591591,
                                         0.000191371732864011,
                                         2.80732788626388e-05,
                                         2.7682153551224e-06,
                                         1.60397645268428e-07,
                                         4.332504483203e-09,
                                         3.47902180166167e-11,
                                         2.85080714254636e-14,
                                         5.03556198766077e-20,
                                         1.64420096906118e-06,
                                         0.000221474285839944,
                                         0.00283257291719521,
                                         0.0145982140875876,
                                         0.0452051029739383,
                                         0.0993666303181247,
                                         0.168669205424053,
                                         0.231759015866343,
                                         0.264979386756745,
                                         0.256124285736375,
                                         0.210970209227347,
                                         0.148382808119163,
                                         0.0888222947568532,
                                         0.0448682921934348,
                                         0.0188514783541449,
                                         0.00644414096588931,
                                         0.00173494033921056,
                                         0.000350603831102513,
                                         4.94291334639949e-05,
                                         4.32484820477707e-06,
                                         1.91878426393649e-07,
                                         2.91324443658834e-09,
                                         5.94037577545306e-12,
                                         5.56678339314074e-17,
                                         6.94073246087618e-09,
                                         4.96002665914737e-06,
                                         0.000157858548061084,
                                         0.00153822328783793,
                                         0.00782386526952058,
                                         0.0259695377725147,
                                         0.0630259009086006,
                                         0.119298961582224,
                                         0.183121094199166,
                                         0.233459995238099,
                                         0.250719887201029,
                                         0.22840533054638,
                                         0.176715822731682,
                                         0.115623999199979,
                                         0.0633373369163007,
                                         0.0285571488604185,
                                         0.0103219230458951,
                                         0.00287348868595108,
                                         0.000579209180746267,
                                         7.65267141282474e-05,
                                         5.57676818208513e-06,
                                         1.60090668309808e-07,
                                         8.12324056805836e-10,
                                         4.03859232757782e-14,
                                         2.00908775230319e-11,
                                         7.61706933654809e-08,
                                         6.03251356059991e-06,
                                         0.000111143030152691,
                                         0.000928535389328956,
                                         0.0046540503182187,
                                         0.0161489958506138,
                                         0.0421094807532219,
                                         0.0867776442351414,
                                         0.145920860399799,
                                         0.204314707971543,
                                         0.241086050408313,
                                         0.241086050408313,
                                         0.204314707971543,
                                         0.145920860399799,
                                         0.0867776442351414,
                                         0.0421094807532218,
                                         0.0161489958506138,
                                         0.00465405031821869,
                                         0.000928535389328956,
                                         0.000111143030152691,
                                         6.03251356059986e-06,
                                         7.61706933654809e-08,
                                         2.00908775230296e-11,
                                         4.03859232757838e-14,
                                         8.12324056805836e-10,
                                         1.6009066830981e-07,
                                         5.57676818208513e-06,
                                         7.65267141282474e-05,
                                         0.000579209180746269,
                                         0.00287348868595108,
                                         0.0103219230458951,
                                         0.0285571488604185,
                                         0.0633373369163007,
                                         0.115623999199979,
                                         0.176715822731682,
                                         0.22840533054638,
                                         0.250719887201029,
                                         0.233459995238099,
                                         0.183121094199166,
                                         0.119298961582224,
                                         0.0630259009086006,
                                         0.0259695377725146,
                                         0.00782386526952058,
                                         0.00153822328783793,
                                         0.000157858548061083,
                                         4.96002665914737e-06,
                                         6.94073246087554e-09,
                                         5.56678339314164e-17,
                                         5.94037577545306e-12,
                                         2.91324443658838e-09,
                                         1.91878426393649e-07,
                                         4.32484820477707e-06,
                                         4.94291334639951e-05,
                                         0.000350603831102513,
                                         0.00173494033921057,
                                         0.00644414096588931,
                                         0.0188514783541449,
                                         0.0448682921934348,
                                         0.0888222947568531,
                                         0.148382808119163,
                                         0.210970209227347,
                                         0.256124285736375,
                                         0.264979386756745,
                                         0.231759015866342,
                                         0.168669205424053,
                                         0.0993666303181245,
                                         0.0452051029739383,
                                         0.0145982140875876,
                                         0.00283257291719519,
                                         0.000221474285839944,
                                         1.64420096906106e-06,
                                         5.0355619876617e-20,
                                         2.85080714254636e-14,
                                         3.47902180166172e-11,
                                         4.332504483203e-09,
                                         1.60397645268428e-07,
                                         2.76821535512241e-06,
                                         2.80732788626388e-05,
                                         0.000191371732864011,
                                         0.00095429914591591,
                                         0.00368214040594987,
                                         0.0114261628273492,
                                         0.0292979921099965,
                                         0.0632601551408966,
                                         0.116499161579847,
                                         0.184398916399249,
                                         0.251625773571773,
                                         0.295464922531154,
                                         0.29622509823997,
                                         0.249509003409909,
                                         0.171404760284176,
                                         0.090917899015211,
                                         0.0333551892709715,
                                         0.00648980915225342,
                                         0.000255607628949396,
                                         2.69927907795993e-23,
                                         8.10733234663811e-17,
                                         2.4620314332656e-13,
                                         5.79706422555372e-11,
                                         3.52518023182634e-09,
                                         9.1869841727216e-08,
                                         1.33206629787698e-06,
                                         1.25091342183841e-05,
                                         8.37452641402925e-05,
                                         0.000426198085411127,
                                         0.00172431853418634,
                                         0.0057267726842325,
                                         0.0159820740308565,
                                         0.0381224395643371,
                                         0.0786723616742755,
                                         0.141597104926601,
                                         0.223219144075163,
                                         0.308293326487388,
                                         0.371268496361603,
                                         0.385136353136963,
                                         0.335548477439356,
                                         0.232756564715492,
                                         0.112692953105569,
                                         0.0235477242904616,
                                         6.51118663885819e-27,
                                         1.03752991772221e-19,
                                         7.84047817398462e-16,
                                         3.49051667331629e-13,
                                         3.48639971660941e-11,
                                         1.37201410703638e-09,
                                         2.8442715358608e-08,
                                         3.67950357392298e-07,
                                         3.30710886955505e-06,
                                         2.21990892774127e-05,
                                         0.000117097358362496,
                                         0.000503726206361995,
                                         0.00181697326526461,
                                         0.00561372434258422,
                                         0.0151042277011176,
                                         0.0358563549653398,
                                         0.0758873629893497,
                                         0.144383946784444,
                                         0.248600782048209,
                                         0.38942036742605,
                                         0.557280270102422,
                                         0.730891618995769,
                                         0.880590726440692,
                                         0.976195016918756,

                                         0.00048828125,
                                         0.00537109375,
                                         0.02685546875,
                                         0.08056640625,
                                         0.1611328125,
                                         0.2255859375,
                                         0.2255859375,
                                         0.1611328125,
                                         0.08056640625,
                                         0.02685546875,
                                         0.00537109375,
                                         0.00048828125,

                                         0.0734310324221027,
                                         3.75367451208139e-08,
                                         0.216433644205746,
                                         1.54097943935805e-06,
                                         0.289966100899273,
                                         2.87550678055162e-05,
                                         0.233088547705085,
                                         0.000321946122075824,
                                         0.124911776245041,
                                         0.00240303856977349,
                                         0.0468580133979942,
                                         0.0125555668489193,
                                         0.0125555668489193,
                                         0.0468580133979942,
                                         0.00240303856977349,
                                         0.124911776245041,
                                         0.000321946122075824,
                                         0.233088547705085,
                                         2.87550678055162e-05,
                                         0.289966100899273,
                                         1.54097943935805e-06,
                                         0.216433644205746,
                                         3.75367451208139e-08,
                                         0.0734310324221027,

                                         0.26838999996274,
                                         0.00048828125,
                                         3.72592123454244e-11,
                                         0.374989996773247,
                                         0.00537109375,
                                         3.22675274117165e-09,
                                         0.238149872979147,
                                         0.02685546875,
                                         1.27020852967867e-07,
                                         0.0907469999008202,
                                         0.08056640625,
                                         3.00009918011123e-06,
                                         0.0230527605382362,
                                         0.1611328125,
                                         4.72394617639724e-05,
                                         0.00409931830594786,
                                         0.2255859375,
                                         0.000520681694052179,
                                         0.000520681694052179,
                                         0.2255859375,
                                         0.00409931830594786,
                                         4.72394617639724e-05,
                                         0.1611328125,
                                         0.0230527605382362,
                                         3.00009918011123e-06,
                                         0.08056640625,
                                         0.0907469999008202,
                                         1.27020852967867e-07,
                                         0.02685546875,
                                         0.238149872979147,
                                         3.22675274117165e-09,
                                         0.00537109375,
                                         0.374989996773247,
                                         3.72592123454244e-11,
                                         0.00048828125,
                                         0.26838999996274,

                                         0.453137552926644,
                                         0.0122111129290597,
                                         5.0558077093006e-06,
                                         1.80778054962012e-13,
                                         0.371906059319314,
                                         0.0661615524768771,
                                         0.000112908198576542,
                                         2.66518819225178e-11,
                                         0.138743859909347,
                                         0.162942151419431,
                                         0.0011461401546679,
                                         1.78602433032416e-09,
                                         0.0310560016652218,
                                         0.240775589887845,
                                         0.00698073623015944,
                                         7.18121801180904e-08,
                                         0.00463432034667064,
                                         0.237192092125223,
                                         0.0283448047295056,
                                         1.92494175499355e-06,
                                         0.00048408831616375,
                                         0.16356334965294,
                                         0.0805645063880055,
                                         3.61189498469069e-05,
                                         3.61189498469067e-05,
                                         0.0805645063880055,
                                         0.16356334965294,
                                         0.000484088316163752,
                                         1.92494175499354e-06,
                                         0.0283448047295056,
                                         0.237192092125223,
                                         0.00463432034667066,
                                         7.18121801180899e-08,
                                         0.00698073623015944,
                                         0.240775589887845,
                                         0.0310560016652219,
                                         1.78602433032415e-09,
                                         0.0011461401546679,
                                         0.162942151419431,
                                         0.138743859909347,
                                         2.66518819225176e-11,
                                         0.000112908198576542,
                                         0.0661615524768771,
                                         0.371906059319314,
                                         1.8077805496201e-13,
                                         5.0558077093006e-06,
                                         0.0122111129290597,
                                         0.453137552926644,

                                         0.589484860106873,
                                         0.0558016777955041,
                                         0.00048828125,
                                         9.88271984153399e-08,
                                         2.42062551702749e-15,
                                         0.31915202839268,
                                         0.184141505775603,
                                         0.00537109375,
                                         3.62374326570991e-06,
                                         5.40988488392026e-13,
                                         0.0785416248539873,
                                         0.27620621237142,
                                         0.02685546875,
                                         6.03970431876743e-05,
                                         5.495734705272e-11,
                                         0.0115972068843275,
                                         0.24858014959061,
                                         0.08056640625,
                                         0.000603983653091248,
                                         3.34976812987317e-09,
                                         0.00114160449118903,
                                         0.149144824899634,
                                         0.1611328125,
                                         0.00402664583063447,
                                         1.36117032883093e-07,
                                         7.86639903134722e-05,
                                         0.0626394552488756,
                                         0.2255859375,
                                         0.0187914252209762,
                                         3.87175832648424e-06,
                                         3.87175832648421e-06,
                                         0.0187914252209762,
                                         0.2255859375,
                                         0.0626394552488756,
                                         7.86639903134727e-05,
                                         1.36117032883092e-07,
                                         0.00402664583063447,
                                         0.1611328125,
                                         0.149144824899634,
                                         0.00114160449118903,
                                         3.34976812987314e-09,
                                         0.000603983653091248,
                                         0.08056640625,
                                         0.24858014959061,
                                         0.0115972068843276,
                                         5.49573470527195e-11,
                                         6.03970431876743e-05,
                                         0.02685546875,
                                         0.27620621237142,
                                         0.0785416248539875,
                                         5.4098848839202e-13,
                                         3.62374326570991e-06,
                                         0.00537109375,
                                         0.184141505775603,
                                         0.319152028392681,
                                         2.42062551702746e-15,
                                         9.88271984153399e-08,
                                         0.00048828125,
                                         0.0558016777955041,
                                         0.589484860106873,

                                         0.685344943905201,
                                         0.129818978593909,
                                         0.00514026938561138,
                                         2.43383574138715e-05,
                                         3.29545291228752e-09,
                                         6.50382221288478e-17,
                                         0.26344449072128,
                                         0.291231177363389,
                                         0.0347570325552513,
                                         0.000435531753486661,
                                         1.77746395592065e-07,
                                         2.04726529126526e-14,
                                         0.0460305694557027,
                                         0.29697156199249,
                                         0.106825964079942,
                                         0.00354262923465249,
                                         4.35776507631436e-06,
                                         2.92925314853948e-12,
                                         0.00482563894644039,
                                         0.181695056341202,
                                         0.196998174400562,
                                         0.0172895158073918,
                                         6.41029588153048e-05,
                                         2.51472753770728e-10,
                                         0.000337265597142093,
                                         0.074110560794297,
                                         0.242190064630808,
                                         0.0562533901713586,
                                         0.000628638649539453,
                                         1.43924162422457e-08,
                                         1.65001275467704e-05,
                                         0.0211599739271486,
                                         0.208424363875706,
                                         0.128118725747816,
                                         0.00431541057228553,
                                         5.76599848408382e-07,
                                         5.76599848408382e-07,
                                         0.00431541057228554,
                                         0.128118725747816,
                                         0.208424363875706,
                                         0.0211599739271485,
                                         1.65001275467704e-05,
                                         1.43924162422457e-08,
                                         0.000628638649539454,
                                         0.0562533901713586,
                                         0.242190064630808,
                                         0.0741105607942969,
                                         0.000337265597142093,
                                         2.51472753770728e-10,
                                         6.4102958815305e-05,
                                         0.0172895158073918,
                                         0.196998174400562,
                                         0.181695056341202,
                                         0.00482563894644039,
                                         2.92925314853948e-12,
                                         4.35776507631437e-06,
                                         0.00354262923465249,
                                         0.106825964079942,
                                         0.29697156199249,
                                         0.0460305694557027,
                                         2.04726529126526e-14,
                                         1.77746395592066e-07,
                                         0.000435531753486661,
                                         0.0347570325552513,
                                         0.291231177363389,
                                         0.26344449072128,
                                         6.50382221288478e-17,
                                         3.29545291228753e-09,
                                         2.43383574138715e-05,
                                         0.00514026938561138,
                                         0.129818978593909,
                                         0.685344943905201,

                                         0.753121033819363,
                                         0.218229702806177,
                                         0.0207005750001045,
                                         0.00048828125,
                                         1.59061893752546e-06,
                                         1.67942148773552e-10,
                                         2.89616285842052e-18,
                                         0.216307632684826,
                                         0.356273435236161,
                                         0.0962359310546485,
                                         0.00537109375,
                                         4.13996505929815e-05,
                                         1.24473097134898e-08,
                                         1.22011645088706e-15,
                                         0.0282394497857612,
                                         0.26438106067055,
                                         0.203361817593523,
                                         0.02685546875,
                                         0.00048978391903884,
                                         4.19342234635289e-07,
                                         2.3364522245779e-13,
                                         0.00221203435395502,
                                         0.11771410101618,
                                         0.257841505153353,
                                         0.08056640625,
                                         0.00347667118798892,
                                         8.47642970874302e-06,
                                         2.68450228358095e-11,
                                         0.000115514431529856,
                                         0.0349410040767241,
                                         0.217944032190056,
                                         0.1611328125,
                                         0.0164524831999554,
                                         0.000114226288494807,
                                         2.05626645810638e-09,
                                         4.22258755044455e-06,
                                         0.00726006169817569,
                                         0.128954106117658,
                                         0.2255859375,
                                         0.0545001043141428,
                                         0.00107749982034165,
                                         1.10253667535169e-07,
                                         1.10253667535167e-07,
                                         0.00107749982034165,
                                         0.0545001043141428,
                                         0.2255859375,
                                         0.128954106117658,
                                         0.00726006169817569,
                                         4.2225875504446e-06,
                                         2.05626645810635e-09,
                                         0.000114226288494807,
                                         0.0164524831999554,
                                         0.1611328125,
                                         0.217944032190056,
                                         0.0349410040767241,
                                         0.000115514431529857,
                                         2.68450228358091e-11,
                                         8.47642970874302e-06,
                                         0.00347667118798892,
                                         0.08056640625,
                                         0.257841505153353,
                                         0.11771410101618,
                                         0.00221203435395504,
                                         2.33645222457785e-13,
                                         4.19342234635289e-07,
                                         0.00048978391903884,
                                         0.02685546875,
                                         0.203361817593523,
                                         0.26438106067055,
                                         0.0282394497857614,
                                         1.22011645088703e-15,
                                         1.24473097134898e-08,
                                         4.13996505929815e-05,
                                         0.00537109375,
                                         0.0962359310546485,
                                         0.356273435236161,
                                         0.216307632684826,
                                         2.89616285842045e-18,
                                         1.67942148773552e-10,
                                         1.59061893752546e-06,
                                         0.00048828125,
                                         0.0207005750001045,
                                         0.218229702806177,
                                         0.753121033819363,

                                         0.802034900589577,
                                         0.30747666337749,
                                         0.0508517464523358,
                                         0.00311355905535951,
                                         5.25497326378054e-05,
                                         1.33949815493519e-07,
                                         1.19941334698004e-11,
                                         1.89054086445579e-19,
                                         0.17871751455487,
                                         0.382777467326129,
                                         0.17397372862973,
                                         0.0236317816720992,
                                         0.000837754024641501,
                                         4.73750511139874e-06,
                                         1.16578924037612e-09,
                                         1.02659132612364e-16,
                                         0.018101655032939,
                                         0.216599718914313,
                                         0.270544549641348,
                                         0.0815291928958948,
                                         0.00607071513345229,
                                         7.61613410550185e-05,
                                         5.150489750217e-08,
                                         2.53388226013795e-14,
                                         0.00110007096645559,
                                         0.0735394983850297,
                                         0.252432435346902,
                                         0.168764489782938,
                                         0.0263945605613514,
                                         0.000734633491878545,
                                         1.36530054052644e-06,
                                         3.75255029622287e-12,
                                         4.45688945398067e-05,
                                         0.0166453211476272,
                                         0.157021913765262,
                                         0.232893699381714,
                                         0.0765063985501891,
                                         0.00472406218967703,
                                         2.41277452095148e-05,
                                         3.70489030402324e-10,
                                         1.2639825018111e-06,
                                         0.00263731335587725,
                                         0.0683712335066433,
                                         0.224974061172605,
                                         0.155231238037118,
                                         0.0212646641802413,
                                         0.000298471765102764,
                                         2.560484938753e-08,
                                         2.560484938753e-08,
                                         0.000298471765102764,
                                         0.0212646641802413,
                                         0.155231238037118,
                                         0.224974061172605,
                                         0.0683712335066433,
                                         0.00263731335587725,
                                         1.2639825018111e-06,
                                         3.70489030402324e-10,
                                         2.41277452095148e-05,
                                         0.00472406218967703,
                                         0.0765063985501891,
                                         0.232893699381714,
                                         0.157021913765262,
                                         0.0166453211476272,
                                         4.45688945398067e-05,
                                         3.75255029622287e-12,
                                         1.36530054052644e-06,
                                         0.000734633491878545,
                                         0.0263945605613514,
                                         0.168764489782938,
                                         0.252432435346902,
                                         0.0735394983850298,
                                         0.00110007096645559,
                                         2.53388226013795e-14,
                                         5.150489750217e-08,
                                         7.61613410550185e-05,
                                         0.0060707151334523,
                                         0.0815291928958948,
                                         0.270544549641348,
                                         0.216599718914313,
                                         0.018101655032939,
                                         1.02659132612364e-16,
                                         1.16578924037612e-09,
                                         4.73750511139874e-06,
                                         0.000837754024641503,
                                         0.0236317816720992,
                                         0.17397372862973,
                                         0.382777467326129,
                                         0.17871751455487,
                                         1.89054086445579e-19,
                                         1.19941334698004e-11,
                                         1.33949815493519e-07,
                                         5.25497326378055e-05,
                                         0.00311355905535951,
                                         0.0508517464523358,
                                         0.30747666337749,
                                         0.802034900589577,

                                         0.838175653540324,
                                         0.390256758759895,
                                         0.0941343305760793,
                                         0.0107238861662764,
                                         0.00048828125,
                                         6.55077139340478e-06,
                                         1.40897533019963e-08,
                                         1.12472448348873e-12,
                                         1.66470683170683e-20,
                                         0.149154741956038,
                                         0.383375669238317,
                                         0.248142012987905,
                                         0.0601946135102018,
                                         0.00537109375,
                                         0.000141212252026243,
                                         6.46750088671001e-07,
                                         1.38534329024709e-10,
                                         1.13193400971892e-17,
                                         0.0120646966767557,
                                         0.171189049336007,
                                         0.297323942347594,
                                         0.153582023237278,
                                         0.02685546875,
                                         0.00138366078829624,
                                         1.34941932052101e-05,
                                         7.75614610109849e-09,
                                         3.49849916774466e-15,
                                         0.000585527100217932,
                                         0.0458647112438666,
                                         0.213752259753187,
                                         0.235111779803934,
                                         0.08056640625,
                                         0.00813463626391044,
                                         0.000168930707655924,
                                         2.60546838166829e-07,
                                         6.48774380217595e-13,
                                         1.89446390171942e-05,
                                         0.00819200272308077,
                                         0.102447245898054,
                                         0.239948000151779,
                                         0.1611328125,
                                         0.0318827214039012,
                                         0.00140986983833969,
                                         5.83491285438931e-06,
                                         8.02073834607706e-11,
                                         4.29065611396553e-07,
                                         0.00102423485848842,
                                         0.0343706622935846,
                                         0.171418590669494,
                                         0.2255859375,
                                         0.0874723249815095,
                                         0.00823659056455266,
                                         9.1470484848342e-05,
                                         6.94117583931999e-09,
                                         6.94117583931984e-09,
                                         9.1470484848342e-05,
                                         0.00823659056455265,
                                         0.0874723249815095,
                                         0.2255859375,
                                         0.171418590669494,
                                         0.0343706622935846,
                                         0.00102423485848842,
                                         4.29065611396561e-07,
                                         8.02073834607687e-11,
                                         5.83491285438931e-06,
                                         0.00140986983833969,
                                         0.0318827214039012,
                                         0.1611328125,
                                         0.239948000151779,
                                         0.102447245898054,
                                         0.00819200272308077,
                                         1.89446390171945e-05,
                                         6.48774380217577e-13,
                                         2.60546838166829e-07,
                                         0.000168930707655923,
                                         0.00813463626391044,
                                         0.08056640625,
                                         0.235111779803934,
                                         0.213752259753188,
                                         0.0458647112438666,
                                         0.000585527100217938,
                                         3.49849916774456e-15,
                                         7.75614610109849e-09,
                                         1.34941932052101e-05,
                                         0.00138366078829624,
                                         0.02685546875,
                                         0.153582023237278,
                                         0.297323942347595,
                                         0.171189049336007,
                                         0.0120646966767558,
                                         1.13193400971888e-17,
                                         1.38534329024709e-10,
                                         6.46750088670999e-07,
                                         0.000141212252026243,
                                         0.00537109375,
                                         0.0601946135102018,
                                         0.248142012987905,
                                         0.383375669238317,
                                         0.149154741956038,
                                         1.66470683170676e-20,
                                         1.12472448348873e-12,
                                         1.40897533019962e-08,
                                         6.55077139340478e-06,
                                         0.00048828125,
                                         0.0107238861662764,
                                         0.0941343305760793,
                                         0.390256758759895,
                                         0.838175653540324,

                                         0.86549081934902,
                                         0.463766695834671,
                                         0.146350059404454,
                                         0.0256269014436379,
                                         0.00224733527808401,
                                         8.290878999979e-05,
                                         9.4352009371759e-07,
                                         1.7952550918385e-09,
                                         1.31855612464378e-13,
                                         1.86432028648487e-21,
                                         0.125852089021298,
                                         0.369086806726999,
                                         0.307312001613783,
                                         0.111430192233037,
                                         0.0183139367388352,
                                         0.00123103983625741,
                                         2.62560712858529e-05,
                                         1.03448671847499e-07,
                                         2.00472601925761e-11,
                                         1.55134336413513e-18,
                                         0.00831832168466077,
                                         0.133516458445242,
                                         0.293321203844397,
                                         0.220235325196703,
                                         0.0678379740229315,
                                         0.00830846808477181,
                                         0.00033211282840909,
                                         2.70957383860042e-06,
                                         1.38544328809055e-09,
                                         5.86776427286293e-16,
                                         0.000329884753702203,
                                         0.0289795966978778,
                                         0.167980413728562,
                                         0.261169423613991,
                                         0.150770119559623,
                                         0.0336450405011447,
                                         0.00252053545086122,
                                         4.25822212448558e-05,
                                         5.74478433285147e-08,
                                         1.33164552830308e-13,
                                         8.72162717837964e-06,
                                         0.00419332584936749,
                                         0.0641331505680211,
                                         0.206474499553127,
                                         0.223391389846431,
                                         0.0908301216521036,
                                         0.0127528928235648,
                                         0.000446133026607226,
                                         1.58806197336116e-06,
                                         2.01471375989046e-11,
                                         1.61410146572347e-07,
                                         0.00042473976789963,
                                         0.0171397524112466,
                                         0.114263771244933,
                                         0.23169444477657,
                                         0.171647220913248,
                                         0.0451671460203559,
                                         0.00327188836391756,
                                         3.07297625042223e-05,
                                         2.13371352481851e-09,
                                         2.13371352481851e-09,
                                         3.07297625042224e-05,
                                         0.00327188836391756,
                                         0.0451671460203559,
                                         0.171647220913248,
                                         0.23169444477657,
                                         0.114263771244933,
                                         0.0171397524112466,
                                         0.000424739767899628,
                                         1.61410146572347e-07,
                                         2.01471375989046e-11,
                                         1.58806197336117e-06,
                                         0.000446133026607226,
                                         0.0127528928235648,
                                         0.0908301216521036,
                                         0.223391389846431,
                                         0.206474499553127,
                                         0.0641331505680211,
                                         0.00419332584936748,
                                         8.72162717837964e-06,
                                         1.33164552830308e-13,
                                         5.74478433285151e-08,
                                         4.25822212448558e-05,
                                         0.00252053545086122,
                                         0.0336450405011447,
                                         0.150770119559623,
                                         0.261169423613991,
                                         0.167980413728562,
                                         0.0289795966978777,
                                         0.000329884753702203,
                                         5.86776427286293e-16,
                                         1.38544328809055e-09,
                                         2.70957383860042e-06,
                                         0.00033211282840909,
                                         0.00830846808477181,
                                         0.0678379740229315,
                                         0.220235325196703,
                                         0.293321203844397,
                                         0.133516458445242,
                                         0.00831832168466078,
                                         1.55134336413513e-18,
                                         2.00472601925763e-11,
                                         1.03448671847499e-07,
                                         2.62560712858529e-05,
                                         0.00123103983625741,
                                         0.0183139367388352,
                                         0.111430192233037,
                                         0.307312001613783,
                                         0.369086806726999,
                                         0.125852089021298,
                                         1.86432028648487e-21,
                                         1.31855612464379e-13,
                                         1.7952550918385e-09,
                                         9.4352009371759e-07,
                                         8.290878999979e-05,
                                         0.00224733527808401,
                                         0.0256269014436379,
                                         0.146350059404454,
                                         0.463766695834671,
                                         0.86549081934902,

                                         0.886566726268818,
                                         0.527617787112179,
                                         0.203047157171455,
                                         0.0485409057706592,
                                         0.0067420536151384,
                                         0.00048828125,
                                         1.54244694977493e-05,
                                         1.55349924899673e-07,
                                         2.69762258323706e-10,
                                         1.86159711174352e-14,
                                         2.54335636690079e-22,
                                         0.107327946715829,
                                         0.347347031941979,
                                         0.34835699265783,
                                         0.16903380853203,
                                         0.0426709174975761,
                                         0.00537109375,
                                         0.000294887135757529,
                                         5.39797311504831e-06,
                                         1.90256253186467e-08,
                                         3.42158160716771e-12,
                                         2.54209158778476e-19,
                                         0.00590597403593869,
                                         0.103940565539841,
                                         0.271662361197511,
                                         0.267557171844063,
                                         0.122757795124009,
                                         0.02685546875,
                                         0.00256258770131351,
                                         8.52565404714609e-05,
                                         6.0992159442174e-07,
                                         2.85854759790927e-10,
                                         1.15492156255794e-16,
                                         0.000194994111303749,
                                         0.0186619838453894,
                                         0.127111738900382,
                                         0.254103628712698,
                                         0.211893399262825,
                                         0.08056640625,
                                         0.0133614286904456,
                                         0.000807934112109426,
                                         1.17316832990723e-05,
                                         1.43289776033315e-08,
                                         3.1482197307225e-14,
                                         4.29200480866383e-06,
                                         0.00223377426793059,
                                         0.0396506754317028,
                                         0.160884378915951,
                                         0.243834142973968,
                                         0.1611328125,
                                         0.0464446612717173,
                                         0.0051042616077731,
                                         0.000150437252141367,
                                         4.78843636787077e-07,
                                         5.72118938303119e-12,
                                         6.61297595506798e-08,
                                         0.000187162483098554,
                                         0.00865791981962936,
                                         0.0713041701026069,
                                         0.196412736973804,
                                         0.2255859375,
                                         0.113009965283947,
                                         0.0225729305385987,
                                         0.00135035666906764,
                                         1.12013476737318e-05,
                                         7.27789275505809e-10,
                                         7.27789275505786e-10,
                                         1.12013476737318e-05,
                                         0.00135035666906764,
                                         0.0225729305385987,
                                         0.113009965283947,
                                         0.2255859375,
                                         0.196412736973804,
                                         0.0713041701026068,
                                         0.00865791981962934,
                                         0.000187162483098554,
                                         6.61297595506814e-08,
                                         5.72118938303099e-12,
                                         4.78843636787077e-07,
                                         0.000150437252141368,
                                         0.0051042616077731,
                                         0.0464446612717172,
                                         0.1611328125,
                                         0.243834142973968,
                                         0.160884378915951,
                                         0.0396506754317027,
                                         0.00223377426793059,
                                         4.29200480866392e-06,
                                         3.14821973072237e-14,
                                         1.43289776033315e-08,
                                         1.17316832990723e-05,
                                         0.000807934112109427,
                                         0.0133614286904455,
                                         0.08056640625,
                                         0.211893399262826,
                                         0.254103628712698,
                                         0.127111738900381,
                                         0.0186619838453894,
                                         0.000194994111303752,
                                         1.15492156255789e-16,
                                         2.85854759790927e-10,
                                         6.09921594421742e-07,
                                         8.5256540471461e-05,
                                         0.00256258770131351,
                                         0.02685546875,
                                         0.122757795124009,
                                         0.267557171844063,
                                         0.271662361197511,
                                         0.103940565539841,
                                         0.00590597403593875,
                                         2.54209158778463e-19,
                                         3.42158160716771e-12,
                                         1.90256253186468e-08,
                                         5.39797311504832e-06,
                                         0.000294887135757528,
                                         0.00537109375,
                                         0.0426709174975761,
                                         0.16903380853203,
                                         0.34835699265783,
                                         0.347347031941979,
                                         0.107327946715829,
                                         2.54335636690064e-22,
                                         1.86159711174352e-14,
                                         2.69762258323707e-10,
                                         1.55349924899674e-07,
                                         1.54244694977493e-05,
                                         0.00048828125,
                                         0.0067420536151384,
                                         0.0485409057706592,
                                         0.203047157171455,
                                         0.527617787112179,
                                         0.886566726268818,

                                         0.903131672699272,
                                         0.582506298662096,
                                         0.260683346509302,
                                         0.0786997336468345,
                                         0.015311884856471,
                                         0.00178788197467273,
                                         0.000112068644746638,
                                         3.14652672755955e-06,
                                         2.88696400883268e-08,
                                         4.67409281674666e-11,
                                         3.0747869316834e-15,
                                         4.0914364966471e-23,
                                         0.0924447753114829,
                                         0.32265624452157,
                                         0.372793351675636,
                                         0.225069991112005,
                                         0.0778433632706149,
                                         0.0152890711389462,
                                         0.00158572266907258,
                                         7.48900047998682e-05,
                                         1.22146888565156e-06,
                                         3.95483010569882e-09,
                                         6.71677728246163e-13,
                                         4.83648546730358e-20,
                                         0.0043012133832491,
                                         0.0812375547525309,
                                         0.24232618705363,
                                         0.292576367922893,
                                         0.179883714871923,
                                         0.0594293083820453,
                                         0.0101987669695879,
                                         0.000810202559626502,
                                         2.34909600858159e-05,
                                         1.52102254025571e-07,
                                         6.66936043264938e-11,
                                         2.59873070970728e-17,
                                         0.000120074519122833,
                                         0.0122722688574451,
                                         0.0945113114297881,
                                         0.228198163541989,
                                         0.249409708290481,
                                         0.138602639598092,
                                         0.0393567613285586,
                                         0.00525913856765962,
                                         0.000271063084243381,
                                         3.50989980272309e-06,
                                         3.97336699223894e-09,
                                         8.37806876158048e-15,
                                         2.2347012122563e-06,
                                         0.00123595204515291,
                                         0.0245740066224636,
                                         0.118657115550449,
                                         0.23053857370703,
                                         0.215501882001401,
                                         0.101251106589127,
                                         0.022758537886785,
                                         0.00208520484393766,
                                         5.3996116862649e-05,
                                         1.57812685983368e-07,
                                         1.80067486822383e-12,
                                         2.91129432054257e-08,
                                         8.71317466202388e-05,
                                         0.00447266316213011,
                                         0.0431890318333497,
                                         0.149166702578715,
                                         0.234546347000086,
                                         0.182338443703665,
                                         0.0689401368791668,
                                         0.011228587165687,
                                         0.00058147142655981,
                                         4.38756116264915e-06,
                                         2.70909806463426e-10,
                                         2.70909806463436e-10,
                                         4.38756116264912e-06,
                                         0.000581471426559808,
                                         0.011228587165687,
                                         0.0689401368791667,
                                         0.182338443703665,
                                         0.234546347000086,
                                         0.149166702578715,
                                         0.0431890318333498,
                                         0.00447266316213012,
                                         8.71317466202393e-05,
                                         2.91129432054249e-08,
                                         1.80067486822391e-12,
                                         1.57812685983367e-07,
                                         5.39961168626488e-05,
                                         0.00208520484393765,
                                         0.022758537886785,
                                         0.101251106589127,
                                         0.215501882001401,
                                         0.23053857370703,
                                         0.11865711555045,
                                         0.0245740066224637,
                                         0.00123595204515291,
                                         2.23470121225625e-06,
                                         8.37806876158088e-15,
                                         3.97336699223891e-09,
                                         3.50989980272308e-06,
                                         0.00027106308424338,
                                         0.00525913856765961,
                                         0.0393567613285586,
                                         0.138602639598092,
                                         0.249409708290481,
                                         0.228198163541989,
                                         0.0945113114297882,
                                         0.0122722688574452,
                                         0.000120074519122831,
                                         2.59873070970742e-17,
                                         6.66936043264931e-11,
                                         1.52102254025571e-07,
                                         2.34909600858158e-05,
                                         0.0008102025596265,
                                         0.0101987669695879,
                                         0.0594293083820453,
                                         0.179883714871923,
                                         0.292576367922893,
                                         0.24232618705363,
                                         0.0812375547525311,
                                         0.00430121338324905,
                                         4.83648546730388e-20,
                                         6.71677728246155e-13,
                                         3.95483010569881e-09,
                                         1.22146888565156e-06,
                                         7.48900047998681e-05,
                                         0.00158572266907258,
                                         0.0152890711389462,
                                         0.0778433632706149,
                                         0.225069991112005,
                                         0.372793351675636,
                                         0.322656244521571,
                                         0.0924447753114823,
                                         4.09143649664738e-23,
                                         3.07478693168336e-15,
                                         4.67409281674664e-11,
                                         2.88696400883267e-08,
                                         3.14652672755955e-06,
                                         0.000112068644746638,
                                         0.00178788197467273,
                                         0.015311884856471,
                                         0.0786997336468345,
                                         0.260683346509302,
                                         0.582506298662096,
                                         0.903131672699272,

                                         0.916366380860175,
                                         0.629512402692532,
                                         0.316850251509257,
                                         0.114496177898481,
                                         0.028757401112108,
                                         0.00477975912600908,
                                         0.00048828125,
                                         2.73667793745054e-05,
                                         7.01025029354821e-07,
                                         5.98025074957988e-09,
                                         9.16549477171262e-12,
                                         5.80713847832048e-16,
                                         7.56865626183012e-24,
                                         0.0803531133695285,
                                         0.297560285214362,
                                         0.383869205545,
                                         0.274269279628369,
                                         0.120441794960071,
                                         0.0328824326952751,
                                         0.00537109375,
                                         0.000481339089960683,
                                         2.02530908296537e-05,
                                         3.02077645772555e-07,
                                         9.15402441984373e-10,
                                         1.48654095061205e-13,
                                         1.04440896506723e-20,
                                         0.00320268030294508,
                                         0.0639326874679965,
                                         0.21139261479896,
                                         0.298634969657963,
                                         0.229288420501698,
                                         0.102825111195138,
                                         0.02685546875,
                                         0.00384818456437162,
                                         0.000265966180027262,
                                         6.93577667685436e-06,
                                         4.15571291944944e-08,
                                         1.72969245766641e-11,
                                         6.55088113985559e-18,
                                         7.65906436682922e-05,
                                         0.00824180254552984,
                                         0.0698469743538251,
                                         0.195099163617859,
                                         0.261901675212957,
                                         0.192923746067215,
                                         0.08056640625,
                                         0.0184591587307122,
                                         0.00209562114286446,
                                         9.55482779697313e-05,
                                         1.13195786313013e-06,
                                         1.20756955812469e-09,
                                         2.46535859862634e-15,
                                         1.22108695272707e-06,
                                         0.00070832112012086,
                                         0.0153855889778792,
                                         0.0849725975672659,
                                         0.199435823606699,
                                         0.241313114821879,
                                         0.1611328125,
                                         0.059030526446223,
                                         0.0110079859877181,
                                         0.000877524738596554,
                                         2.05552954649539e-05,
                                         5.62036035653333e-08,
                                         6.18542034276104e-13,
                                         1.36274784006847e-08,
                                         4.26124217980656e-05,
                                         0.00237234962960556,
                                         0.0259060036055803,
                                         0.106308038670385,
                                         0.211287694752332,
                                         0.2255859375,
                                         0.132141565731509,
                                         0.040476318509612,
                                         0.00564149117334199,
                                         0.000261285450447978,
                                         1.83110904273233e-06,
                                         1.08631650529751e-10,
                                         1.08631650529756e-10,
                                         1.83110904273235e-06,
                                         0.000261285450447977,
                                         0.005641491173342,
                                         0.040476318509612,
                                         0.132141565731509,
                                         0.2255859375,
                                         0.211287694752332,
                                         0.106308038670385,
                                         0.0259060036055803,
                                         0.00237234962960556,
                                         4.26124217980653e-05,
                                         1.36274784006842e-08,
                                         6.18542034276134e-13,
                                         5.62036035653338e-08,
                                         2.05552954649538e-05,
                                         0.000877524738596556,
                                         0.0110079859877181,
                                         0.059030526446223,
                                         0.1611328125,
                                         0.241313114821879,
                                         0.199435823606699,
                                         0.0849725975672657,
                                         0.0153855889778792,
                                         0.000708321120120856,
                                         1.22108695272703e-06,
                                         2.46535859862648e-15,
                                         1.2075695581247e-09,
                                         1.13195786313013e-06,
                                         9.55482779697316e-05,
                                         0.00209562114286445,
                                         0.0184591587307122,
                                         0.08056640625,
                                         0.192923746067215,
                                         0.261901675212957,
                                         0.195099163617859,
                                         0.0698469743538252,
                                         0.00824180254552981,
                                         7.65906436682906e-05,
                                         6.550881139856e-18,
                                         1.72969245766643e-11,
                                         4.15571291944941e-08,
                                         6.93577667685438e-06,
                                         0.000265966180027262,
                                         0.00384818456437162,
                                         0.02685546875,
                                         0.102825111195138,
                                         0.229288420501698,
                                         0.298634969657963,
                                         0.21139261479896,
                                         0.0639326874679964,
                                         0.00320268030294504,
                                         1.0444089650673e-20,
                                         1.48654095061207e-13,
                                         9.15402441984368e-10,
                                         3.02077645772556e-07,
                                         2.02530908296537e-05,
                                         0.000481339089960683,
                                         0.00537109375,
                                         0.0328824326952751,
                                         0.120441794960071,
                                         0.274269279628369,
                                         0.383869205545,
                                         0.297560285214361,
                                         0.0803531133695279,
                                         7.56865626183071e-24,
                                         5.80713847832056e-16,
                                         9.16549477171256e-12,
                                         5.9802507495799e-09,
                                         7.01025029354821e-07,
                                         2.73667793745054e-05,
                                         0.00048828125,
                                         0.00477975912600908,
                                         0.028757401112108,
                                         0.114496177898481,
                                         0.316850251509257,
                                         0.629512402692532,
                                         0.916366380860175,

                                         0.927095289645483,
                                         0.669768649722662,
                                         0.370101153087987,
                                         0.154086762645769,
                                         0.0472055397724016,
                                         0.0102747145776369,
                                         0.00150953894615382,
                                         0.000138799899210189,
                                         7.12032704343328e-06,
                                         1.69581894104603e-07,
                                         1.36516497091064e-09,
                                         2.00269498655837e-12,
                                         1.23136057746587e-16,
                                         1.57847939966251e-24,
                                         0.0704221517726889,
                                         0.273409690501538,
                                         0.385005698162813,
                                         0.314127003072049,
                                         0.166119567110988,
                                         0.0583386135772098,
                                         0.0133663801940399,
                                         0.00189672790352659,
                                         0.000151739613846662,
                                         5.83091927888156e-06,
                                         8.10271504943185e-08,
                                         2.3294503694056e-10,
                                         3.64990472295107e-14,
                                         2.51442897307842e-21,
                                         0.00243148461832873,
                                         0.0507318045167199,
                                         0.182050214676648,
                                         0.291086812404496,
                                         0.265721033342052,
                                         0.150563520342477,
                                         0.0537973170088647,
                                         0.0117814361815679,
                                         0.0014698578175543,
                                         9.1132208681806e-05,
                                         2.18601589522361e-06,
                                         1.23159929720707e-08,
                                         4.91762184574603e-12,
                                         1.82061092869964e-18,
                                         5.03715149311475e-05,
                                         0.00564804265306362,
                                         0.0516495430929416,
                                         0.16184192672486,
                                         0.255024746771197,
                                         0.233149596137562,
                                         0.129914813524882,
                                         0.0439079021009651,
                                         0.0085428529138425,
                                         0.000854590406281589,
                                         3.53856612136845e-05,
                                         3.90693920455063e-07,
                                         3.975392201176e-10,
                                         7.90944788464428e-16,
                                         6.9567634997771e-07,
                                         0.000419202984724798,
                                         0.00976900908520531,
                                         0.0599885397982626,
                                         0.163172684394396,
                                         0.240690148828711,
                                         0.209153661825722,
                                         0.109092747675033,
                                         0.0331008596141993,
                                         0.00534260259954493,
                                         0.000381865176816435,
                                         8.26252173774899e-06,
                                         2.14246420307707e-08,
                                         2.29078290346514e-13,
                                         6.72554536439666e-09,
                                         2.17795450986549e-05,
                                         0.00129339918524673,
                                         0.0155648013216743,
                                         0.0730820349346437,
                                         0.173932205296657,
                                         0.235705822513543,
                                         0.189734852226491,
                                         0.0897787709532594,
                                         0.0233800679586396,
                                         0.00288463478664794,
                                         0.000122316942559742,
                                         8.08249058313329e-07,
                                         4.64429435674945e-11,
                                         4.64429435674945e-11,
                                         8.08249058313337e-07,
                                         0.000122316942559742,
                                         0.00288463478664794,
                                         0.0233800679586396,
                                         0.0897787709532594,
                                         0.189734852226491,
                                         0.235705822513544,
                                         0.173932205296657,
                                         0.0730820349346438,
                                         0.0155648013216744,
                                         0.00129339918524673,
                                         2.17795450986547e-05,
                                         6.72554536439666e-09,
                                         2.29078290346514e-13,
                                         2.14246420307709e-08,
                                         8.26252173774899e-06,
                                         0.000381865176816434,
                                         0.00534260259954492,
                                         0.0331008596141993,
                                         0.109092747675033,
                                         0.209153661825722,
                                         0.240690148828711,
                                         0.163172684394396,
                                         0.0599885397982627,
                                         0.00976900908520531,
                                         0.000419202984724795,
                                         6.9567634997771e-07,
                                         7.90944788464428e-16,
                                         3.97539220117605e-10,
                                         3.90693920455063e-07,
                                         3.53856612136844e-05,
                                         0.000854590406281588,
                                         0.0085428529138425,
                                         0.043907902100965,
                                         0.129914813524882,
                                         0.233149596137562,
                                         0.255024746771197,
                                         0.16184192672486,
                                         0.0516495430929416,
                                         0.00564804265306359,
                                         5.03715149311475e-05,
                                         1.82061092869964e-18,
                                         4.91762184574609e-12,
                                         1.23159929720707e-08,
                                         2.18601589522361e-06,
                                         9.11322086818058e-05,
                                         0.0014698578175543,
                                         0.0117814361815679,
                                         0.0537973170088647,
                                         0.150563520342477,
                                         0.265721033342052,
                                         0.291086812404497,
                                         0.182050214676648,
                                         0.0507318045167197,
                                         0.00243148461832873,
                                         2.51442897307842e-21,
                                         3.64990472295113e-14,
                                         2.3294503694056e-10,
                                         8.10271504943182e-08,
                                         5.83091927888154e-06,
                                         0.000151739613846662,
                                         0.00189672790352659,
                                         0.0133663801940399,
                                         0.0583386135772098,
                                         0.166119567110988,
                                         0.314127003072049,
                                         0.385005698162813,
                                         0.273409690501537,
                                         0.0704221517726889,
                                         1.57847939966251e-24,
                                         1.23136057746589e-16,
                                         2.00269498655837e-12,
                                         1.36516497091064e-09,
                                         1.69581894104603e-07,
                                         7.12032704343328e-06,
                                         0.000138799899210189,
                                         0.00150953894615382,
                                         0.0102747145776369,
                                         0.0472055397724016,
                                         0.154086762645769,
                                         0.370101153087987,
                                         0.669768649722662,
                                         0.927095289645483,

                                         0.935906037395068,
                                         0.704320125066937,
                                         0.419687166747903,
                                         0.195766026523527,
                                         0.0702301283844,
                                         0.0188833567267398,
                                         0.00366787999388122,
                                         0.00048828125,
                                         4.12595024711641e-05,
                                         1.97095024135248e-06,
                                         4.42598696671581e-08,
                                         3.39964859691922e-10,
                                         4.81340435299394e-13,
                                         2.88828372708384e-17,
                                         3.65293064380643e-25,
                                         0.0621816342583801,
                                         0.250855524630276,
                                         0.379158501340792,
                                         0.344142890692415,
                                         0.210976030740041,
                                         0.0902663967257489,
                                         0.0268309396282292,
                                         0.00537109375,
                                         0.000682477900437849,
                                         4.98900709413941e-05,
                                         1.78273112111298e-06,
                                         2.34001112801686e-08,
                                         6.44677641169452e-11,
                                         9.8123188407069e-15,
                                         6.65268428571154e-22,
                                         0.00187788647604607,
                                         0.0406120171766229,
                                         0.15570165387383,
                                         0.27499044120382,
                                         0.288084632173267,
                                         0.196132848040833,
                                         0.089214195881126,
                                         0.02685546875,
                                         0.00513133676860347,
                                         0.000574023803413857,
                                         3.26391530304668e-05,
                                         7.32114714899547e-07,
                                         3.92473365234448e-09,
                                         1.51523770656358e-12,
                                         5.50718568943508e-19,
                                         3.40273232665139e-05,
                                         0.00394490639562797,
                                         0.0383633835453784,
                                         0.131840136406116,
                                         0.236025168366167,
                                         0.255697106404837,
                                         0.177984957448347,
                                         0.08056640625,
                                         0.0231485447507642,
                                         0.00396275235534675,
                                         0.000358544582947924,
                                         1.37433181241329e-05,
                                         1.43360339414624e-07,
                                         1.40391604349456e-10,
                                         2.73535549700369e-16,
                                         4.1105031053199e-07,
                                         0.000255462751375381,
                                         0.00630157809474269,
                                         0.0421392139825932,
                                         0.128915542820469,
                                         0.22223375933505,
                                         0.236723500223368,
                                         0.1611328125,
                                         0.0696186521163853,
                                         0.0182378107393391,
                                         0.00262576703182836,
                                         0.000171993804811527,
                                         3.4910542111648e-06,
                                         8.67181982357707e-09,
                                         9.05746312415311e-14,
                                         3.47584350157521e-09,
                                         1.15802119383174e-05,
                                         0.000724569054395812,
                                         0.0094280799643642,
                                         0.0492890317885051,
                                         0.135204856784716,
                                         0.22039284360301,
                                         0.2255859375,
                                         0.146563412183377,
                                         0.0587552280627923,
                                         0.0134606879683542,
                                         0.00150671824943937,
                                         5.95089387241907e-05,
                                         3.74953485151285e-07,
                                         2.09941072868519e-11,
                                         2.09941072868507e-11,
                                         3.74953485151285e-07,
                                         5.95089387241904e-05,
                                         0.00150671824943937,
                                         0.0134606879683542,
                                         0.0587552280627923,
                                         0.146563412183377,
                                         0.2255859375,
                                         0.22039284360301,
                                         0.135204856784716,
                                         0.0492890317885051,
                                         0.00942807996436418,
                                         0.000724569054395814,
                                         1.15802119383174e-05,
                                         3.47584350157537e-09,
                                         9.05746312415253e-14,
                                         8.67181982357707e-09,
                                         3.49105421116478e-06,
                                         0.000171993804811527,
                                         0.00262576703182836,
                                         0.0182378107393391,
                                         0.0696186521163853,
                                         0.1611328125,
                                         0.236723500223368,
                                         0.22223375933505,
                                         0.128915542820469,
                                         0.0421392139825931,
                                         0.00630157809474271,
                                         0.000255462751375381,
                                         4.11050310532005e-07,
                                         2.73535549700349e-16,
                                         1.40391604349456e-10,
                                         1.43360339414623e-07,
                                         1.37433181241329e-05,
                                         0.000358544582947924,
                                         0.00396275235534675,
                                         0.0231485447507642,
                                         0.08056640625,
                                         0.177984957448347,
                                         0.255697106404837,
                                         0.236025168366167,
                                         0.131840136406116,
                                         0.0383633835453784,
                                         0.00394490639562797,
                                         3.40273232665149e-05,
                                         5.50718568943462e-19,
                                         1.51523770656358e-12,
                                         3.92473365234446e-09,
                                         7.3211471489955e-07,
                                         3.26391530304668e-05,
                                         0.000574023803413857,
                                         0.00513133676860347,
                                         0.02685546875,
                                         0.089214195881126,
                                         0.196132848040833,
                                         0.288084632173267,
                                         0.27499044120382,
                                         0.155701653873831,
                                         0.0406120171766229,
                                         0.0018778864760461,
                                         6.65268428571092e-22,
                                         9.8123188407069e-15,
                                         6.44677641169447e-11,
                                         2.34001112801687e-08,
                                         1.78273112111298e-06,
                                         4.98900709413941e-05,
                                         0.000682477900437849,
                                         0.00537109375,
                                         0.0268309396282292,
                                         0.0902663967257489,
                                         0.210976030740041,
                                         0.344142890692415,
                                         0.379158501340792,
                                         0.250855524630276,
                                         0.0621816342583806,
                                         3.65293064380606e-25,
                                         2.88828372708384e-17,
                                         4.8134043529939e-13,
                                         3.39964859691923e-10,
                                         4.42598696671581e-08,
                                         1.97095024135248e-06,
                                         4.12595024711641e-05,
                                         0.00048828125,
                                         0.00366787999388122,
                                         0.0188833567267398,
                                         0.0702301283844,
                                         0.195766026523527,
                                         0.419687166747903,
                                         0.704320125066937,
                                         0.935906037395068,

                                         0.943225520083775,
                                         0.734077280240195,
                                         0.465322240797327,
                                         0.238132723533588,
                                         0.097066267092164,
                                         0.0309070663414845,
                                         0.00748101920341895,
                                         0.00132519544955756,
                                         0.000162831484625307,
                                         1.2843281329868e-05,
                                         5.78781068822832e-07,
                                         1.23855261190719e-08,
                                         9.153873949567e-11,
                                         1.25878856181714e-13,
                                         7.40339256731053e-18,
                                         9.26004638691567e-26,
                                         0.0552781456566423,
                                         0.230152282224746,
                                         0.368654152599516,
                                         0.364991881859558,
                                         0.252185291158939,
                                         0.126378672949287,
                                         0.0461279261587768,
                                         0.0120474772882024,
                                         0.00216724281891425,
                                         0.000252033462460141,
                                         1.71270940051426e-05,
                                         5.76830752023388e-07,
                                         7.2264694691267e-09,
                                         1.92252873126441e-11,
                                         2.85721318757825e-15,
                                         1.91187990810999e-22,
                                         0.00147254545087023,
                                         0.0327994157607488,
                                         0.132758326394115,
                                         0.254287288057798,
                                         0.297816322161424,
                                         0.234891399914195,
                                         0.129283889476713,
                                         0.0497839802554172,
                                         0.0131115513991311,
                                         0.00224811091949633,
                                         0.000230372356399414,
                                         1.22112372898756e-05,
                                         2.59313266392782e-07,
                                         1.33465866044528e-09,
                                         5.0122397051255e-13,
                                         1.79426000019313e-19,
                                         2.35361379704827e-05,
                                         0.00280458224575654,
                                         0.0286850530818994,
                                         0.10629610369143,
                                         0.21102236693858,
                                         0.261945952427229,
                                         0.217408309500341,
                                         0.123433875696058,
                                         0.0475939600099396,
                                         0.0120317421116046,
                                         0.00185920936419512,
                                         0.000155103710015381,
                                         5.58308898262766e-06,
                                         5.55928359644346e-08,
                                         5.27560497848539e-11,
                                         1.01032568038511e-16,
                                         2.50790126823467e-07,
                                         0.00015987442440211,
                                         0.00413198073351561,
                                         0.0296223004205937,
                                         0.0996821106046788,
                                         0.194744413852737,
                                         0.243734277235987,
                                         0.204027099898159,
                                         0.115175032075297,
                                         0.0429287294752613,
                                         0.010003108341459,
                                         0.0013133892053391,
                                         8.0137004485079e-05,
                                         1.54374722774308e-06,
                                         3.70187241983645e-09,
                                         3.79267954601805e-14,
                                         1.87061196929194e-09,
                                         6.37951770949326e-06,
                                         0.000416638076749536,
                                         0.00577854179235601,
                                         0.0329612747839516,
                                         0.101348276095877,
                                         0.191273639105129,
                                         0.236069232053838,
                                         0.19510252157086,
                                         0.107217480069483,
                                         0.0376738224820633,
                                         0.00778507389134111,
                                         0.000805173919935158,
                                         3.00076228038148e-05,
                                         1.81831308978635e-07,
                                         9.96618505560559e-12,
                                         9.96618505560621e-12,
                                         1.81831308978635e-07,
                                         3.00076228038147e-05,
                                         0.00080517391993516,
                                         0.00778507389134111,
                                         0.0376738224820633,
                                         0.107217480069483,
                                         0.19510252157086,
                                         0.236069232053838,
                                         0.191273639105128,
                                         0.101348276095877,
                                         0.0329612747839516,
                                         0.00577854179235599,
                                         0.000416638076749538,
                                         6.37951770949326e-06,
                                         1.87061196929184e-09,
                                         3.79267954601833e-14,
                                         3.70187241983645e-09,
                                         1.54374722774307e-06,
                                         8.01370044850793e-05,
                                         0.0013133892053391,
                                         0.010003108341459,
                                         0.0429287294752614,
                                         0.115175032075296,
                                         0.204027099898159,
                                         0.243734277235987,
                                         0.194744413852737,
                                         0.0996821106046788,
                                         0.0296223004205937,
                                         0.00413198073351562,
                                         0.00015987442440211,
                                         2.50790126823457e-07,
                                         1.01032568038519e-16,
                                         5.27560497848539e-11,
                                         5.55928359644342e-08,
                                         5.58308898262768e-06,
                                         0.000155103710015381,
                                         0.00185920936419512,
                                         0.0120317421116046,
                                         0.0475939600099396,
                                         0.123433875696058,
                                         0.217408309500341,
                                         0.261945952427229,
                                         0.21102236693858,
                                         0.10629610369143,
                                         0.0286850530818995,
                                         0.00280458224575654,
                                         2.3536137970482e-05,
                                         1.7942600001933e-19,
                                         5.0122397051255e-13,
                                         1.33465866044527e-09,
                                         2.59313266392783e-07,
                                         1.22112372898756e-05,
                                         0.000230372356399414,
                                         0.00224811091949634,
                                         0.0131115513991311,
                                         0.0497839802554172,
                                         0.129283889476713,
                                         0.234891399914195,
                                         0.297816322161424,
                                         0.254287288057798,
                                         0.132758326394115,
                                         0.0327994157607488,
                                         0.0014725454508702,
                                         1.91187990811019e-22,
                                         2.85721318757825e-15,
                                         1.9225287312644e-11,
                                         7.22646946912673e-09,
                                         5.76830752023388e-07,
                                         1.71270940051426e-05,
                                         0.000252033462460142,
                                         0.00216724281891425,
                                         0.0120474772882024,
                                         0.0461279261587768,
                                         0.126378672949287,
                                         0.252185291158939,
                                         0.364991881859558,
                                         0.368654152599516,
                                         0.230152282224746,
                                         0.0552781456566417,
                                         9.26004638691674e-26,
                                         7.40339256731053e-18,
                                         1.25878856181713e-13,
                                         9.15387394956704e-11,
                                         1.23855261190719e-08,
                                         5.78781068822832e-07,
                                         1.28432813298681e-05,
                                         0.000162831484625307,
                                         0.00132519544955755,
                                         0.00748101920341895,
                                         0.0309070663414845,
                                         0.097066267092164,
                                         0.238132723533588,
                                         0.465322240797327,
                                         0.734077280240195,
                                         0.943225520083775,

                                         0.949369308550409,
                                         0.759810733109086,
                                         0.507007123004045,
                                         0.280124886586382,
                                         0.126809632639431,
                                         0.0463362665810559,
                                         0.0133874650467754,
                                         0.00297324967359862,
                                         0.00048828125,
                                         5.61623512145792e-05,
                                         4.18461410684163e-06,
                                         1.79689765343207e-07,
                                         3.69443841737307e-09,
                                         2.64441562084395e-11,
                                         3.54922446263443e-14,
                                         2.05285155557366e-18,
                                         2.54404077167653e-26,
                                         0.0494434401239189,
                                         0.211337017876886,
                                         0.355229079219833,
                                         0.377901611886141,
                                         0.288064696850607,
                                         0.164198232818822,
                                         0.0707051044657256,
                                         0.0227990230614343,
                                         0.00537109375,
                                         0.000886228665502286,
                                         9.58712448989563e-05,
                                         6.13565980481718e-06,
                                         1.9678682053096e-07,
                                         2.37185444337353e-09,
                                         6.12949065446448e-12,
                                         8.93043812153615e-16,
                                         5.91065955151091e-23,
                                         0.00117046785657402,
                                         0.0267191750278128,
                                         0.113130649002345,
                                         0.231730454886536,
                                         0.29744395682815,
                                         0.264480244585927,
                                         0.169738556995534,
                                         0.079465462436821,
                                         0.02685546875,
                                         0.00635658157735973,
                                         0.000998386357499526,
                                         9.52306001820435e-05,
                                         4.7645392097508e-06,
                                         9.66993761943203e-08,
                                         4.81163446968923e-10,
                                         1.76589673799335e-13,
                                         6.24201980401657e-20,
                                         1.66249961552765e-05,
                                         0.00202685072782503,
                                         0.0216173919744415,
                                         0.0852587054930585,
                                         0.184277160834052,
                                         0.255604942545569,
                                         0.24449022130977,
                                         0.166185008101849,
                                         0.08056640625,
                                         0.0273560070368938,
                                         0.00623821242683387,
                                         0.000886835401523939,
                                         6.92144946089456e-05,
                                         2.36543251188827e-06,
                                         2.26627244334295e-08,
                                         2.09512092026173e-11,
                                         3.95517395926909e-17,
                                         1.574245122641e-07,
                                         0.000102501265317475,
                                         0.00275381687690404,
                                         0.0209123623562593,
                                         0.0761108576938379,
                                         0.164684982441723,
                                         0.234774661197232,
                                         0.231693585875069,
                                         0.1611328125,
                                         0.0784856988403942,
                                         0.0259854607654219,
                                         0.00550577249954455,
                                         0.000670319633312079,
                                         3.85750228422995e-05,
                                         7.11607226166722e-07,
                                         1.65714924550886e-09,
                                         1.67076272736956e-14,
                                         1.04347295969791e-09,
                                         3.62856350160054e-06,
                                         0.000245564089337942,
                                         0.00359058735120684,
                                         0.0220049182593812,
                                         0.0742739957225073,
                                         0.157811624822848,
                                         0.226117523139776,
                                         0.2255859375,
                                         0.157625469240088,
                                         0.0757702507533546,
                                         0.0239271814535765,
                                         0.0045442777461514,
                                         0.000440351887387922,
                                         1.56410758147866e-05,
                                         9.17512930510604e-08,
                                         4.94039878887719e-12,
                                         4.94039878887684e-12,
                                         9.17512930510617e-08,
                                         1.56410758147866e-05,
                                         0.000440351887387924,
                                         0.00454427774615141,
                                         0.0239271814535766,
                                         0.0757702507533546,
                                         0.157625469240088,
                                         0.2255859375,
                                         0.226117523139776,
                                         0.157811624822848,
                                         0.0742739957225072,
                                         0.0220049182593811,
                                         0.00359058735120683,
                                         0.000245564089337942,
                                         3.6285635016005e-06,
                                         1.04347295969797e-09,
                                         1.67076272736942e-14,
                                         1.65714924550889e-09,
                                         7.11607226166722e-07,
                                         3.85750228422997e-05,
                                         0.00067031963331208,
                                         0.00550577249954455,
                                         0.0259854607654219,
                                         0.0784856988403941,
                                         0.1611328125,
                                         0.231693585875069,
                                         0.234774661197232,
                                         0.164684982441722,
                                         0.0761108576938378,
                                         0.0209123623562593,
                                         0.00275381687690404,
                                         0.000102501265317474,
                                         1.57424512264107e-07,
                                         3.95517395926872e-17,
                                         2.09512092026177e-11,
                                         2.26627244334295e-08,
                                         2.36543251188828e-06,
                                         6.92144946089458e-05,
                                         0.00088683540152394,
                                         0.00623821242683387,
                                         0.0273560070368938,
                                         0.08056640625,
                                         0.166185008101849,
                                         0.24449022130977,
                                         0.255604942545569,
                                         0.184277160834052,
                                         0.0852587054930583,
                                         0.0216173919744415,
                                         0.00202685072782501,
                                         1.66249961552771e-05,
                                         6.24201980401591e-20,
                                         1.76589673799338e-13,
                                         4.81163446968923e-10,
                                         9.66993761943207e-08,
                                         4.76453920975081e-06,
                                         9.52306001820437e-05,
                                         0.000998386357499526,
                                         0.00635658157735973,
                                         0.02685546875,
                                         0.079465462436821,
                                         0.169738556995534,
                                         0.264480244585926,
                                         0.29744395682815,
                                         0.231730454886536,
                                         0.113130649002345,
                                         0.0267191750278127,
                                         0.00117046785657405,
                                         5.91065955151022e-23,
                                         8.93043812153636e-16,
                                         6.12949065446447e-12,
                                         2.37185444337355e-09,
                                         1.9678682053096e-07,
                                         6.13565980481719e-06,
                                         9.58712448989563e-05,
                                         0.000886228665502286,
                                         0.00537109375,
                                         0.0227990230614343,
                                         0.0707051044657256,
                                         0.164198232818822,
                                         0.288064696850607,
                                         0.377901611886141,
                                         0.355229079219833,
                                         0.211337017876886,
                                         0.0494434401239195,
                                         2.5440407716762e-26,
                                         2.05285155557371e-18,
                                         3.54922446263443e-14,
                                         2.64441562084397e-11,
                                         3.69443841737308e-09,
                                         1.79689765343208e-07,
                                         4.18461410684163e-06,
                                         5.61623512145792e-05,
                                         0.00048828125,
                                         0.00297324967359862,
                                         0.0133874650467754,
                                         0.0463362665810559,
                                         0.126809632639431,
                                         0.280124886586382,
                                         0.507007123004045,
                                         0.759810733109086,
                                         0.949369308550409,

                                         0.954574415655241,
                                         0.782163249223514,
                                         0.544908354902523,
                                         0.320989288393295,
                                         0.158558781795711,
                                         0.0649140620789487,
                                         0.0216775271327536,
                                         0.00577941017902508,
                                         0.00119510606580852,
                                         0.00018427931995875,
                                         2.00592277559802e-05,
                                         1.42508874339744e-06,
                                         5.8767472301626e-08,
                                         1.1683678251673e-09,
                                         8.14090446284099e-12,
                                         1.070551167949e-14,
                                         6.1055639347767e-19,
                                         7.50776406381361e-27,
                                         0.0444717652665947,
                                         0.194333296018468,
                                         0.340134792018295,
                                         0.384261659553671,
                                         0.317873803970986,
                                         0.201534541706033,
                                         0.0993585736418691,
                                         0.0379908485366634,
                                         0.0110914246030517,
                                         0.00240259788594212,
                                         0.000369236056959723,
                                         3.76211156328033e-05,
                                         2.29040179843607e-06,
                                         7.05180866896275e-08,
                                         8.22851593224999e-10,
                                         2.075225087938e-12,
                                         2.97345481516803e-16,
                                         1.94994071763675e-23,
                                         0.000941751382268435,
                                         0.0219469586890722,
                                         0.0965063672053998,
                                         0.209093670924822,
                                         0.289665316164664,
                                         0.284405189652292,
                                         0.207003780620146,
                                         0.113514617002221,
                                         0.0467891904571644,
                                         0.0142384560637862,
                                         0.00308937991526614,
                                         0.000451438661759323,
                                         4.05754794185388e-05,
                                         1.9346338009661e-06,
                                         3.78048648460411e-08,
                                         1.8285224957609e-10,
                                         6.58224771573142e-14,
                                         2.30202226169388e-20,
                                         1.19657359318368e-05,
                                         0.00148714298239679,
                                         0.0164290377751462,
                                         0.0682662380705897,
                                         0.158376049250852,
                                         0.240811261086371,
                                         0.258763166299983,
                                         0.203505350938277,
                                         0.118428159879541,
                                         0.0506286047115719,
                                         0.0155092138174755,
                                         0.00325025234211847,
                                         0.000431287522870906,
                                         3.18455147001975e-05,
                                         1.04213772043093e-06,
                                         9.66688732784889e-09,
                                         8.74255457393991e-12,
                                         1.63060541648366e-17,
                                         1.01356429510848e-07,
                                         6.71799763974657e-05,
                                         0.001864562861037,
                                         0.0148586651449818,
                                         0.0577286304263636,
                                         0.135932970695567,
                                         0.21564300591676,
                                         0.243225227753126,
                                         0.199835744330653,
                                         0.120015616091969,
                                         0.051905931467038,
                                         0.0156007023505351,
                                         0.0030561795785648,
                                         0.000349467275930656,
                                         1.9151874285102e-05,
                                         3.40707541474859e-07,
                                         7.7412523076422e-10,
                                         7.70011095174897e-15,
                                         6.00981678285065e-10,
                                         2.12434479907454e-06,
                                         0.000148128959056416,
                                         0.00226387091615744,
                                         0.0147296030597254,
                                         0.0537119431505579,
                                         0.125795856752017,
                                         0.203488300430253,
                                         0.23604206404088,
                                         0.199148756549673,
                                         0.121602424675939,
                                         0.052416650077683,
                                         0.0151596398801052,
                                         0.00268449622081144,
                                         0.000246374348619178,
                                         8.40571917481651e-06,
                                         4.79824183522349e-08,
                                         2.54532431015551e-12,
                                         2.54532431015551e-12,
                                         4.79824183522349e-08,
                                         8.40571917481651e-06,
                                         0.000246374348619178,
                                         0.00268449622081144,
                                         0.0151596398801052,
                                         0.052416650077683,
                                         0.121602424675939,
                                         0.199148756549673,
                                         0.23604206404088,
                                         0.203488300430252,
                                         0.125795856752017,
                                         0.0537119431505579,
                                         0.0147296030597254,
                                         0.00226387091615744,
                                         0.000148128959056416,
                                         2.12434479907454e-06,
                                         6.00981678285065e-10,
                                         7.70011095174897e-15,
                                         7.7412523076422e-10,
                                         3.40707541474859e-07,
                                         1.91518742851019e-05,
                                         0.000349467275930656,
                                         0.0030561795785648,
                                         0.0156007023505351,
                                         0.051905931467038,
                                         0.120015616091969,
                                         0.199835744330653,
                                         0.243225227753126,
                                         0.21564300591676,
                                         0.135932970695567,
                                         0.0577286304263636,
                                         0.0148586651449818,
                                         0.001864562861037,
                                         6.71799763974657e-05,
                                         1.01356429510848e-07,
                                         1.63060541648366e-17,
                                         8.74255457393991e-12,
                                         9.66688732784888e-09,
                                         1.04213772043093e-06,
                                         3.18455147001975e-05,
                                         0.000431287522870906,
                                         0.00325025234211847,
                                         0.0155092138174755,
                                         0.0506286047115719,
                                         0.118428159879541,
                                         0.203505350938277,
                                         0.258763166299983,
                                         0.240811261086371,
                                         0.158376049250852,
                                         0.0682662380705898,
                                         0.0164290377751462,
                                         0.00148714298239679,
                                         1.19657359318368e-05,
                                         2.30202226169388e-20,
                                         6.58224771573142e-14,
                                         1.8285224957609e-10,
                                         3.78048648460409e-08,
                                         1.9346338009661e-06,
                                         4.05754794185388e-05,
                                         0.000451438661759323,
                                         0.00308937991526614,
                                         0.0142384560637862,
                                         0.0467891904571644,
                                         0.113514617002221,
                                         0.207003780620146,
                                         0.284405189652292,
                                         0.289665316164664,
                                         0.209093670924823,
                                         0.0965063672053998,
                                         0.0219469586890722,
                                         0.000941751382268435,
                                         1.94994071763675e-23,
                                         2.97345481516803e-16,
                                         2.075225087938e-12,
                                         8.22851593224995e-10,
                                         7.05180866896275e-08,
                                         2.29040179843607e-06,
                                         3.76211156328033e-05,
                                         0.000369236056959723,
                                         0.00240259788594212,
                                         0.0110914246030517,
                                         0.0379908485366634,
                                         0.0993585736418691,
                                         0.201534541706033,
                                         0.317873803970986,
                                         0.384261659553671,
                                         0.340134792018295,
                                         0.194333296018468,
                                         0.0444717652665947,
                                         7.50776406381361e-27,
                                         6.1055639347767e-19,
                                         1.070551167949e-14,
                                         8.14090446284094e-12,
                                         1.1683678251673e-09,
                                         5.8767472301626e-08,
                                         1.42508874339744e-06,
                                         2.00592277559802e-05,
                                         0.00018427931995875,
                                         0.00119510606580852,
                                         0.00577941017902508,
                                         0.0216775271327536,
                                         0.0649140620789487,
                                         0.158558781795711,
                                         0.320989288393295,
                                         0.544908354902523,
                                         0.782163249223514,
                                         0.954574415655241,

                                         0.95902145017655,
                                         0.801667262583609,
                                         0.579279392479289,
                                         0.360226444227865,
                                         0.191498657503255,
                                         0.0862219292844185,
                                         0.0324701233628251,
                                         0.0100584750305455,
                                         0.00250717538167547,
                                         0.00048828125,
                                         7.14006348140359e-05,
                                         7.41900588890221e-06,
                                         5.06282764967861e-07,
                                         2.01749692708119e-08,
                                         3.89848066544757e-10,
                                         2.65504087547811e-12,
                                         3.43142658572844e-15,
                                         1.93377183378843e-19,
                                         2.36219471370049e-27,
                                         0.0402036341932291,
                                         0.179010600930926,
                                         0.324249674626892,
                                         0.385409507331974,
                                         0.341541068361658,
                                         0.23670638895896,
                                         0.130577506072396,
                                         0.0574355265288836,
                                         0.019956251932079,
                                         0.00537109375,
                                         0.0010854084047733,
                                         0.000157210888266923,
                                         1.52333030692865e-05,
                                         8.89213419855212e-07,
                                         2.64486528356099e-08,
                                         3.00268482729841e-10,
                                         7.41768885149521e-13,
                                         1.04786660282409e-16,
                                         6.81811107111276e-24,
                                         0.000766089700574676,
                                         0.01816940356752,
                                         0.0824989342249547,
                                         0.187433223944691,
                                         0.276883738788899,
                                         0.295378950570291,
                                         0.238687744108674,
                                         0.149075539764508,
                                         0.072202221542248,
                                         0.02685546875,
                                         0.00750000592508692,
                                         0.00151424743421153,
                                         0.000208339847052094,
                                         1.7814615532132e-05,
                                         8.15622216179329e-07,
                                         1.54356743111851e-08,
                                         7.28852808577651e-11,
                                         2.58097671684597e-14,
                                         8.94519411976819e-21,
                                         8.7588116015458e-06,
                                         0.00110650617655921,
                                         0.0125941359652898,
                                         0.0546917698239971,
                                         0.134680034538989,
                                         0.221156830173605,
                                         0.261784012735397,
                                         0.232157877527981,
                                         0.156737672385942,
                                         0.08056640625,
                                         0.0310943357149076,
                                         0.00875109345399696,
                                         0.00170962889685577,
                                         0.000214140173419349,
                                         1.50912699484554e-05,
                                         4.76094006153851e-07,
                                         4.29696980183618e-09,
                                         3.81428750278776e-12,
                                         7.04152487447901e-18,
                                         6.67604856314039e-08,
                                         4.49237244438175e-05,
                                         0.00128173179611192,
                                         0.0106391301162407,
                                         0.04367347292348,
                                         0.110390045144844,
                                         0.191410105225581,
                                         0.241028944938195,
                                         0.226832336736232,
                                         0.1611328125,
                                         0.0859428399753942,
                                         0.0337160382600504,
                                         0.00935276666568555,
                                         0.00171604466341542,
                                         0.000186153527241242,
                                         9.78968149262624e-06,
                                         1.68885946614509e-07,
                                         3.75795438446831e-10,
                                         3.69532301509492e-15,
                                         3.56198289351036e-10,
                                         1.27672013266637e-06,
                                         9.13111849184754e-05,
                                         0.00144873283487061,
                                         0.00991357457405492,
                                         0.0385706986316473,
                                         0.0979680905634605,
                                         0.1751672915071,
                                         0.229791828245142,
                                         0.2255859375,
                                         0.166278523121706,
                                         0.090930335660372,
                                         0.0358159429362388,
                                         0.00962624839547888,
                                         0.0016073660517571,
                                         0.000140910206265586,
                                         4.64646599708089e-06,
                                         2.5917172740115e-08,
                                         1.35748842765117e-12,
                                         1.35748842765117e-12,
                                         2.5917172740115e-08,
                                         4.64646599708089e-06,
                                         0.000140910206265586,
                                         0.00160736605175709,
                                         0.0096262483954789,
                                         0.0358159429362388,
                                         0.090930335660372,
                                         0.166278523121706,
                                         0.2255859375,
                                         0.229791828245142,
                                         0.175167291507101,
                                         0.0979680905634605,
                                         0.0385706986316473,
                                         0.00991357457405494,
                                         0.00144873283487061,
                                         9.13111849184754e-05,
                                         1.27672013266637e-06,
                                         3.56198289351036e-10,
                                         3.69532301509492e-15,
                                         3.75795438446831e-10,
                                         1.68885946614509e-07,
                                         9.78968149262624e-06,
                                         0.000186153527241242,
                                         0.00171604466341542,
                                         0.00935276666568554,
                                         0.0337160382600504,
                                         0.0859428399753942,
                                         0.1611328125,
                                         0.226832336736232,
                                         0.241028944938195,
                                         0.191410105225581,
                                         0.110390045144844,
                                         0.0436734729234801,
                                         0.0106391301162407,
                                         0.00128173179611192,
                                         4.49237244438175e-05,
                                         6.67604856314039e-08,
                                         7.04152487447901e-18,
                                         3.81428750278776e-12,
                                         4.29696980183618e-09,
                                         4.76094006153851e-07,
                                         1.50912699484553e-05,
                                         0.000214140173419349,
                                         0.00170962889685577,
                                         0.00875109345399695,
                                         0.0310943357149076,
                                         0.08056640625,
                                         0.156737672385942,
                                         0.232157877527981,
                                         0.261784012735397,
                                         0.221156830173605,
                                         0.134680034538989,
                                         0.0546917698239971,
                                         0.0125941359652898,
                                         0.00110650617655921,
                                         8.7588116015458e-06,
                                         8.94519411976819e-21,
                                         2.58097671684597e-14,
                                         7.28852808577651e-11,
                                         1.54356743111851e-08,
                                         8.15622216179326e-07,
                                         1.78146155321321e-05,
                                         0.000208339847052094,
                                         0.00151424743421152,
                                         0.00750000592508692,
                                         0.02685546875,
                                         0.072202221542248,
                                         0.149075539764508,
                                         0.238687744108674,
                                         0.295378950570291,
                                         0.276883738788899,
                                         0.187433223944691,
                                         0.0824989342249547,
                                         0.01816940356752,
                                         0.000766089700574676,
                                         6.81811107111276e-24,
                                         1.04786660282409e-16,
                                         7.41768885149521e-13,
                                         3.00268482729841e-10,
                                         2.64486528356098e-08,
                                         8.89213419855214e-07,
                                         1.52333030692865e-05,
                                         0.000157210888266922,
                                         0.0010854084047733,
                                         0.00537109375,
                                         0.019956251932079,
                                         0.0574355265288836,
                                         0.130577506072396,
                                         0.23670638895896,
                                         0.341541068361658,
                                         0.385409507331974,
                                         0.324249674626892,
                                         0.179010600930926,
                                         0.0402036341932291,
                                         2.36219471370049e-27,
                                         1.93377183378843e-19,
                                         3.43142658572844e-15,
                                         2.65504087547811e-12,
                                         3.89848066544755e-10,
                                         2.01749692708119e-08,
                                         5.06282764967861e-07,
                                         7.41900588890219e-06,
                                         7.14006348140359e-05,
                                         0.00048828125,
                                         0.00250717538167547,
                                         0.0100584750305455,
                                         0.0324701233628251,
                                         0.0862219292844185,
                                         0.191498657503255,
                                         0.360226444227865,
                                         0.579279392479289,
                                         0.801667262583609,
                                         0.95902145017655,

                                         0.962849871731804,
                                         0.818762759410233,
                                         0.610411224869654,
                                         0.397532757990642,
                                         0.224938982021099,
                                         0.109759745015756,
                                         0.0457255191358952,
                                         0.0160510002960226,
                                         0.00466679705532299,
                                         0.00109885937107583,
                                         0.000203394287014079,
                                         2.84305557639511e-05,
                                         2.8394232437668e-06,
                                         1.87226204506845e-07,
                                         7.24570577582251e-09,
                                         1.36645187882291e-10,
                                         9.12604488241914e-13,
                                         1.16208019032424e-15,
                                         6.48218445882968e-20,
                                         7.8735057752368e-28,
                                         0.0365141523509801,
                                         0.165217680616026,
                                         0.308175599691681,
                                         0.382530868747483,
                                         0.359415222118474,
                                         0.268581167236859,
                                         0.16283661140761,
                                         0.0804968997502879,
                                         0.0322869284076045,
                                         0.0103689431973899,
                                         0.00260814505398787,
                                         0.000497236389342861,
                                         6.85075524595546e-05,
                                         6.36148624116332e-06,
                                         3.58289101272562e-07,
                                         1.03477986902661e-08,
                                         1.14755736766427e-10,
                                         2.78513165750155e-13,
                                         3.88694428454725e-17,
                                         2.51218069722792e-24,
                                         0.000629420942338276,
                                         0.0151541743788028,
                                         0.0707214745910223,
                                         0.167315960535656,
                                         0.261039078100572,
                                         0.298734474282727,
                                         0.263586271416115,
                                         0.183498925297894,
                                         0.101534086405808,
                                         0.0444737953904778,
                                         0.0152020463274559,
                                         0.00395291775222255,
                                         0.000751318512291261,
                                         9.82489392810462e-05,
                                         8.05311349310128e-06,
                                         3.56187701067726e-07,
                                         6.55908909384411e-09,
                                         3.03411949388207e-11,
                                         1.05943072734285e-14,
                                         3.64343408923589e-21,
                                         6.50987133173918e-06,
                                         0.000833987017298266,
                                         0.00973768261990724,
                                         0.0439096024981698,
                                         0.113753780199886,
                                         0.199363835617578,
                                         0.256002830856511,
                                         0.250980018037489,
                                         0.191579153745094,
                                         0.11445246282757,
                                         0.0531647299728426,
                                         0.018854885632751,
                                         0.00494380096773836,
                                         0.000910433854974636,
                                         0.000108603867718713,
                                         7.35632855969188e-06,
                                         2.24938557077371e-07,
                                         1.98321994833382e-09,
                                         1.73255912697334e-12,
                                         3.17045950808892e-18,
                                         4.4886150539537e-08,
                                         3.05981409318237e-05,
                                         0.000893858273877872,
                                         0.00768228440520433,
                                         0.0330472167971175,
                                         0.0886984761246234,
                                         0.165758378473072,
                                         0.228851365575532,
                                         0.240986867556278,
                                         0.196360817754732,
                                         0.123952107157242,
                                         0.0599568444556741,
                                         0.0216873559471456,
                                         0.00562441902070069,
                                         0.000976417389669045,
                                         0.0001012866151113,
                                         5.14272167330236e-06,
                                         8.64207089052784e-08,
                                         1.88891452172451e-10,
                                         1.83925627054768e-15,
                                         2.16645842184994e-10,
                                         7.85830410240349e-07,
                                         5.74354136889377e-05,
                                         0.00094084763303047,
                                         0.0067205061254373,
                                         0.0276238353346353,
                                         0.0751284193204711,
                                         0.146071641697683,
                                         0.212195682252051,
                                         0.235821046203064,
                                         0.202293652457148,
                                         0.133460169792087,
                                         0.0665963269422135,
                                         0.0243223188629236,
                                         0.00614502648213407,
                                         0.000976205021597626,
                                         8.230385482543e-05,
                                         2.63610561893341e-06,
                                         1.44156618402418e-08,
                                         7.46896320259788e-13,
                                         7.46896320259716e-13,
                                         1.44156618402416e-08,
                                         2.63610561893341e-06,
                                         8.23038548254297e-05,
                                         0.000976205021597626,
                                         0.00614502648213408,
                                         0.0243223188629236,
                                         0.0665963269422135,
                                         0.133460169792087,
                                         0.202293652457148,
                                         0.235821046203064,
                                         0.212195682252051,
                                         0.146071641697683,
                                         0.0751284193204711,
                                         0.0276238353346353,
                                         0.0067205061254373,
                                         0.000940847633030474,
                                         5.74354136889377e-05,
                                         7.85830410240361e-07,
                                         2.16645842185012e-10,
                                         1.83925627054747e-15,
                                         1.88891452172447e-10,
                                         8.64207089052783e-08,
                                         5.14272167330234e-06,
                                         0.0001012866151113,
                                         0.000976417389669047,
                                         0.00562441902070069,
                                         0.0216873559471456,
                                         0.0599568444556741,
                                         0.123952107157242,
                                         0.196360817754732,
                                         0.240986867556278,
                                         0.228851365575532,
                                         0.165758378473072,
                                         0.0886984761246233,
                                         0.0330472167971175,
                                         0.00768228440520435,
                                         0.000893858273877873,
                                         3.05981409318241e-05,
                                         4.48861505395399e-08,
                                         3.17045950808851e-18,
                                         1.73255912697329e-12,
                                         1.98321994833382e-09,
                                         2.2493855707737e-07,
                                         7.35632855969188e-06,
                                         0.000108603867718714,
                                         0.000910433854974636,
                                         0.00494380096773836,
                                         0.018854885632751,
                                         0.0531647299728426,
                                         0.11445246282757,
                                         0.191579153745094,
                                         0.250980018037489,
                                         0.256002830856511,
                                         0.199363835617578,
                                         0.113753780199886,
                                         0.0439096024981699,
                                         0.00973768261990724,
                                         0.000833987017298274,
                                         6.50987133173949e-06,
                                         3.64343408923536e-21,
                                         1.05943072734282e-14,
                                         3.03411949388207e-11,
                                         6.55908909384407e-09,
                                         3.56187701067726e-07,
                                         8.05311349310131e-06,
                                         9.82489392810462e-05,
                                         0.000751318512291261,
                                         0.00395291775222255,
                                         0.0152020463274559,
                                         0.0444737953904778,
                                         0.101534086405808,
                                         0.183498925297894,
                                         0.263586271416115,
                                         0.298734474282727,
                                         0.261039078100572,
                                         0.167315960535656,
                                         0.0707214745910223,
                                         0.0151541743788029,
                                         0.000629420942338296,
                                         2.51218069722751e-24,
                                         3.88694428454713e-17,
                                         2.78513165750155e-13,
                                         1.14755736766427e-10,
                                         1.03477986902662e-08,
                                         3.58289101272563e-07,
                                         6.36148624116332e-06,
                                         6.85075524595546e-05,
                                         0.000497236389342861,
                                         0.00260814505398787,
                                         0.0103689431973899,
                                         0.0322869284076045,
                                         0.0804968997502879,
                                         0.16283661140761,
                                         0.268581167236859,
                                         0.359415222118474,
                                         0.382530868747483,
                                         0.308175599691681,
                                         0.165217680616027,
                                         0.0365141523509807,
                                         7.8735057752354e-28,
                                         6.48218445882946e-20,
                                         1.16208019032424e-15,
                                         9.12604488241907e-13,
                                         1.36645187882291e-10,
                                         7.24570577582254e-09,
                                         1.87226204506845e-07,
                                         2.8394232437668e-06,
                                         2.84305557639511e-05,
                                         0.000203394287014079,
                                         0.00109885937107583,
                                         0.00466679705532299,
                                         0.0160510002960226,
                                         0.0457255191358952,
                                         0.109759745015756,
                                         0.224938982021099,
                                         0.397532757990642,
                                         0.610411224869654,
                                         0.818762759410233,
                                         0.962849871731804,

                                         0.966168675911241,
                                         0.833813449959271,
                                         0.638602647743786,
                                         0.432749288512411,
                                         0.258323296578574,
                                         0.13500797093596,
                                         0.0612781325323938,
                                         0.023901631036031,
                                         0.00790357547816385,
                                         0.00217707819771306,
                                         0.00048828125,
                                         8.65280598032676e-05,
                                         1.16360866925802e-05,
                                         1.12325675632762e-06,
                                         7.19102849516372e-08,
                                         2.71371919923967e-09,
                                         5.0115232677207e-11,
                                         3.29107217344525e-13,
                                         4.13737066897295e-16,
                                         2.28755611971731e-20,
                                         2.76497251519804e-28,
                                         0.0333045543010179,
                                         0.152800747183368,
                                         0.292314326902701,
                                         0.376626574933724,
                                         0.372074192785028,
                                         0.296512839776918,
                                         0.194787248907063,
                                         0.106260134104298,
                                         0.0480573955329587,
                                         0.0178619471050697,
                                         0.00537109375,
                                         0.00127610839495719,
                                         0.000231555815696367,
                                         3.05718407166369e-05,
                                         2.73729357321094e-06,
                                         1.49508603032667e-07,
                                         4.21006835778163e-09,
                                         4.57560134071787e-11,
                                         1.09368036428386e-13,
                                         1.51042980177331e-17,
                                         9.70566989082481e-25,
                                         0.000521833094160348,
                                         0.0127279769075314,
                                         0.0608200548312718,
                                         0.148991975378792,
                                         0.243597508115925,
                                         0.296009001081589,
                                         0.281444551326725,
                                         0.214728900581992,
                                         0.132823272189335,
                                         0.0666132907188633,
                                         0.02685546875,
                                         0.00855451682580143,
                                         0.00209450671547503,
                                         0.000378217357509899,
                                         4.736189438233e-05,
                                         3.74407706305499e-06,
                                         1.60762911524317e-07,
                                         2.89158704157638e-09,
                                         1.3141160445319e-11,
                                         4.5332185145312e-15,
                                         1.54859385847714e-21,
                                         4.90581154216448e-06,
                                         0.000636128026118523,
                                         0.00759267417825231,
                                         0.0353643797937216,
                                         0.09569018294355,
                                         0.17730361110954,
                                         0.243992466393818,
                                         0.260352583593922,
                                         0.220261894421426,
                                         0.149054203585779,
                                         0.08056640625,
                                         0.034407621677957,
                                         0.0113673457986195,
                                         0.00280745351608335,
                                         0.000491686181146808,
                                         5.62567481860491e-05,
                                         3.68327231656546e-06,
                                         1.09641662326735e-07,
                                         9.47388854124823e-10,
                                         8.16326720104242e-13,
                                         1.48252081442427e-18,
                                         3.07467236255673e-08,
                                         2.11952440702499e-05,
                                         0.000631904518304361,
                                         0.00559600320762242,
                                         0.0250594000532292,
                                         0.0708009337500252,
                                         0.141016109866767,
                                         0.210446653432344,
                                         0.243508040593762,
                                         0.222349577857836,
                                         0.1611328125,
                                         0.0922619362878709,
                                         0.0411287136809477,
                                         0.0138928847631616,
                                         0.00340295088676361,
                                         0.000563525031343985,
                                         5.62588092379562e-05,
                                         2.77155622960055e-06,
                                         4.55335556629962e-08,
                                         9.80009106583683e-11,
                                         9.46177916251735e-16,
                                         1.34891588049658e-10,
                                         4.94345237006793e-07,
                                         3.68134227302988e-05,
                                         0.000619851852560864,
                                         0.00459379905228907,
                                         0.0197905757923055,
                                         0.0570504510860664,
                                         0.119074891943323,
                                         0.18844528779115,
                                         0.232180867824867,
                                         0.2255859375,
                                         0.173176323463482,
                                         0.104166775895774,
                                         0.0481249545738623,
                                         0.0164862317210159,
                                         0.00395138947474737,
                                         0.000601513366754496,
                                         4.90421856033385e-05,
                                         1.53190875911382e-06,
                                         8.2355811433566e-09,
                                         4.22710324431499e-13,
                                         4.22710324431544e-13,
                                         8.23558114335643e-09,
                                         1.53190875911383e-06,
                                         4.90421856033385e-05,
                                         0.000601513366754496,
                                         0.00395138947474737,
                                         0.0164862317210159,
                                         0.0481249545738622,
                                         0.104166775895774,
                                         0.173176323463482,
                                         0.2255859375,
                                         0.232180867824867,
                                         0.18844528779115,
                                         0.119074891943323,
                                         0.0570504510860665,
                                         0.0197905757923055,
                                         0.00459379905228907,
                                         0.000619851852560864,
                                         3.68134227302985e-05,
                                         4.94345237006801e-07,
                                         1.34891588049646e-10,
                                         9.46177916251853e-16,
                                         9.8000910658366e-11,
                                         4.55335556629967e-08,
                                         2.77155622960055e-06,
                                         5.62588092379562e-05,
                                         0.000563525031343985,
                                         0.00340295088676361,
                                         0.0138928847631616,
                                         0.0411287136809477,
                                         0.0922619362878709,
                                         0.1611328125,
                                         0.222349577857836,
                                         0.243508040593762,
                                         0.210446653432344,
                                         0.141016109866767,
                                         0.0708009337500251,
                                         0.0250594000532292,
                                         0.00559600320762242,
                                         0.000631904518304357,
                                         2.11952440702502e-05,
                                         3.07467236255651e-08,
                                         1.48252081442448e-18,
                                         8.1632672010422e-13,
                                         9.47388854124834e-10,
                                         1.09641662326735e-07,
                                         3.68327231656546e-06,
                                         5.62567481860491e-05,
                                         0.000491686181146808,
                                         0.00280745351608335,
                                         0.0113673457986195,
                                         0.034407621677957,
                                         0.08056640625,
                                         0.149054203585779,
                                         0.220261894421426,
                                         0.260352583593922,
                                         0.243992466393818,
                                         0.17730361110954,
                                         0.09569018294355,
                                         0.0353643797937216,
                                         0.00759267417825228,
                                         0.00063612802611853,
                                         4.90581154216422e-06,
                                         1.54859385847739e-21,
                                         4.53321851453106e-15,
                                         1.31411604453192e-11,
                                         2.89158704157638e-09,
                                         1.60762911524317e-07,
                                         3.74407706305499e-06,
                                         4.736189438233e-05,
                                         0.000378217357509898,
                                         0.00209450671547503,
                                         0.00855451682580143,
                                         0.02685546875,
                                         0.0666132907188633,
                                         0.132823272189335,
                                         0.214728900581992,
                                         0.281444551326725,
                                         0.296009001081589,
                                         0.243597508115925,
                                         0.148991975378792,
                                         0.0608200548312716,
                                         0.0127279769075315,
                                         0.000521833094160329,
                                         9.70566989082653e-25,
                                         1.51042980177326e-17,
                                         1.09368036428387e-13,
                                         4.57560134071787e-11,
                                         4.21006835778163e-09,
                                         1.49508603032667e-07,
                                         2.73729357321094e-06,
                                         3.05718407166368e-05,
                                         0.000231555815696367,
                                         0.00127610839495719,
                                         0.00537109375,
                                         0.0178619471050697,
                                         0.0480573955329587,
                                         0.106260134104298,
                                         0.194787248907063,
                                         0.296512839776918,
                                         0.372074192785028,
                                         0.376626574933724,
                                         0.292314326902701,
                                         0.152800747183368,
                                         0.0333045543010174,
                                         2.76497251519858e-28,
                                         2.28755611971722e-20,
                                         4.13737066897301e-16,
                                         3.29107217344525e-13,
                                         5.0115232677207e-11,
                                         2.71371919923967e-09,
                                         7.19102849516372e-08,
                                         1.12325675632761e-06,
                                         1.16360866925802e-05,
                                         8.65280598032676e-05,
                                         0.00048828125,
                                         0.00217707819771306,
                                         0.00790357547816385,
                                         0.023901631036031,
                                         0.0612781325323938,
                                         0.13500797093596,
                                         0.258323296578574,
                                         0.432749288512411,
                                         0.638602647743786,
                                         0.833813449959271,
                                         0.966168675911241,

                                         0.969063998048624,
                                         0.847120594623417,
                                         0.66414321675946,
                                         0.465820104013263,
                                         0.291221354612316,
                                         0.161469850827231,
                                         0.0788757708702185,
                                         0.0336563378684856,
                                         0.0124084990616811,
                                         0.00389811212335797,
                                         0.00102500832209875,
                                         0.000220455835515759,
                                         3.76260699693309e-05,
                                         4.89351372332969e-06,
                                         4.58677402543094e-07,
                                         2.86224768969271e-08,
                                         1.05679944157505e-09,
                                         1.9164207143686e-11,
                                         1.24022351186423e-13,
                                         1.54187643308506e-16,
                                         8.45980530558801e-21,
                                         1.01819105918365e-28,
                                         0.0304960111084845,
                                         0.14161293927101,
                                         0.276925157224707,
                                         0.368514607120785,
                                         0.380192784183748,
                                         0.320239678526244,
                                         0.225351402464993,
                                         0.133701399056883,
                                         0.0669342563485193,
                                         0.0281210782842117,
                                         0.00980498249793917,
                                         0.0027886084446221,
                                         0.000631097684105429,
                                         0.000109768313186286,
                                         1.39708754786739e-05,
                                         1.21220462121203e-06,
                                         6.44754301698568e-08,
                                         1.77621516817823e-09,
                                         1.89691928434031e-11,
                                         4.47439448657014e-14,
                                         6.12333531009866e-18,
                                         3.91493063485141e-25,
                                         0.000436225384589262,
                                         0.0107606363015098,
                                         0.0524856282149701,
                                         0.132516057058566,
                                         0.225611816104338,
                                         0.288692936053042,
                                         0.292653843982409,
                                         0.241425068823858,
                                         0.164117522509034,
                                         0.0922118916264178,
                                         0.0426327721675325,
                                         0.0160335908287169,
                                         0.00481151266574524,
                                         0.00111920718480121,
                                         0.000193427054473683,
                                         2.33357614363932e-05,
                                         1.78802357556149e-06,
                                         7.48303215850606e-08,
                                         1.31878822899848e-09,
                                         5.90196039763408e-12,
                                         2.01461950554766e-15,
                                         6.84220659475085e-22,
                                         3.74395035763428e-06,
                                         0.000490596244422732,
                                         0.00596856102940066,
                                         0.0285912770441581,
                                         0.0803287600660469,
                                         0.156152313874052,
                                         0.228033918921764,
                                         0.261565238364364,
                                         0.241441940171186,
                                         0.181423333871863,
                                         0.111222223786983,
                                         0.0553127568752421,
                                         0.0220098929680998,
                                         0.00684691976846641,
                                         0.00160680089631361,
                                         0.000269537544546783,
                                         2.97511312294433e-05,
                                         1.89151983234106e-06,
                                         5.50113778894856e-08,
                                         4.67099670891234e-10,
                                         3.97694218584386e-13,
                                         7.17496100725099e-19,
                                         2.14218991307717e-08,
                                         1.49114276825135e-05,
                                         0.000452488576057087,
                                         0.00411251356331618,
                                         0.0190672923820159,
                                         0.0563079130871436,
                                         0.118455003520458,
                                         0.188923694499798,
                                         0.236798645192066,
                                         0.237962266345785,
                                         0.193440905281509,
                                         0.12721213880798,
                                         0.0671217033146442,
                                         0.0279247137035625,
                                         0.00889847640469035,
                                         0.00207551224525311,
                                         0.000330021677390766,
                                         3.18752006793804e-05,
                                         1.52981433043676e-06,
                                         2.46451560551713e-08,
                                         5.23376551783659e-11,
                                         5.01593209175786e-16,
                                         8.57993303792751e-11,
                                         3.1725777488314e-07,
                                         2.40128461981918e-05,
                                         0.000414075154727679,
                                         0.00316814484547815,
                                         0.0142130891198361,
                                         0.043073028555423,
                                         0.0955192432933815,
                                         0.162571253476213,
                                         0.218484730145526,
                                         0.23550660824277,
                                         0.204799948909091,
                                         0.143286754900275,
                                         0.0797223807575611,
                                         0.0344958841848712,
                                         0.0111874053064008,
                                         0.00256259214802626,
                                         0.000376004459845019,
                                         2.97798815950014e-05,
                                         9.10231002897932e-07,
                                         4.82144576183173e-09,
                                         2.45460599808001e-13,
                                         2.45460599808001e-13,
                                         4.82144576183173e-09,
                                         9.10231002897923e-07,
                                         2.97798815950016e-05,
                                         0.00037600445984502,
                                         0.00256259214802626,
                                         0.0111874053064008,
                                         0.0344958841848712,
                                         0.0797223807575611,
                                         0.143286754900274,
                                         0.204799948909091,
                                         0.23550660824277,
                                         0.218484730145526,
                                         0.162571253476213,
                                         0.0955192432933815,
                                         0.043073028555423,
                                         0.0142130891198361,
                                         0.00316814484547814,
                                         0.000414075154727678,
                                         2.4012846198192e-05,
                                         3.1725777488314e-07,
                                         8.57993303792751e-11,
                                         5.01593209175786e-16,
                                         5.23376551783659e-11,
                                         2.4645156055171e-08,
                                         1.52981433043676e-06,
                                         3.18752006793806e-05,
                                         0.000330021677390766,
                                         0.00207551224525311,
                                         0.00889847640469035,
                                         0.0279247137035625,
                                         0.0671217033146442,
                                         0.12721213880798,
                                         0.193440905281509,
                                         0.237962266345785,
                                         0.236798645192066,
                                         0.188923694499798,
                                         0.118455003520458,
                                         0.0563079130871436,
                                         0.0190672923820159,
                                         0.00411251356331617,
                                         0.00045248857605709,
                                         1.49114276825135e-05,
                                         2.14218991307717e-08,
                                         7.17496100725099e-19,
                                         3.97694218584386e-13,
                                         4.67099670891228e-10,
                                         5.5011377889486e-08,
                                         1.89151983234107e-06,
                                         2.97511312294433e-05,
                                         0.000269537544546783,
                                         0.00160680089631361,
                                         0.00684691976846641,
                                         0.0220098929680997,
                                         0.0553127568752421,
                                         0.111222223786983,
                                         0.181423333871863,
                                         0.241441940171186,
                                         0.261565238364364,
                                         0.228033918921764,
                                         0.156152313874052,
                                         0.0803287600660468,
                                         0.028591277044158,
                                         0.00596856102940069,
                                         0.000490596244422732,
                                         3.74395035763428e-06,
                                         6.84220659475085e-22,
                                         2.01461950554766e-15,
                                         5.901960397634e-12,
                                         1.31878822899849e-09,
                                         7.48303215850609e-08,
                                         1.78802357556149e-06,
                                         2.33357614363932e-05,
                                         0.000193427054473683,
                                         0.00111920718480121,
                                         0.00481151266574523,
                                         0.0160335908287169,
                                         0.0426327721675325,
                                         0.0922118916264178,
                                         0.164117522509034,
                                         0.241425068823858,
                                         0.292653843982409,
                                         0.288692936053042,
                                         0.225611816104338,
                                         0.132516057058566,
                                         0.0524856282149703,
                                         0.0107606363015098,
                                         0.000436225384589262,
                                         3.91493063485141e-25,
                                         6.12333531009866e-18,
                                         4.47439448657007e-14,
                                         1.89691928434033e-11,
                                         1.77621516817824e-09,
                                         6.44754301698568e-08,
                                         1.21220462121203e-06,
                                         1.39708754786739e-05,
                                         0.000109768313186286,
                                         0.000631097684105428,
                                         0.0027886084446221,
                                         0.00980498249793917,
                                         0.0281210782842117,
                                         0.0669342563485193,
                                         0.133701399056883,
                                         0.225351402464993,
                                         0.320239678526244,
                                         0.380192784183748,
                                         0.368514607120785,
                                         0.276925157224708,
                                         0.14161293927101,
                                         0.0304960111084845,
                                         1.01819105918365e-28,
                                         8.45980530558801e-21,
                                         1.54187643308503e-16,
                                         1.24022351186424e-13,
                                         1.91642071436861e-11,
                                         1.05679944157505e-09,
                                         2.86224768969271e-08,
                                         4.58677402543094e-07,
                                         4.89351372332969e-06,
                                         3.76260699693309e-05,
                                         0.000220455835515759,
                                         0.00102500832209875,
                                         0.00389811212335797,
                                         0.0124084990616811,
                                         0.0336563378684856,
                                         0.0788757708702185,
                                         0.161469850827231,
                                         0.291221354612316,
                                         0.465820104013263,
                                         0.66414321675946,
                                         0.847120594623417,
                                         0.969063998048624,

                                         0.971604604245533,
                                         0.858934484653058,
                                         0.687304179892866,
                                         0.496760116838332,
                                         0.3233135775148,
                                         0.188696349940446,
                                         0.0982165390983178,
                                         0.0452729730923123,
                                         0.018317549123027,
                                         0.0064328086721382,
                                         0.00193322240895345,
                                         0.00048828125,
                                         0.00010126626399222,
                                         1.67280471624036e-05,
                                         2.11319736048694e-06,
                                         1.93056657630677e-07,
                                         1.17812565297614e-08,
                                         4.2676934426346e-10,
                                         7.61700081016815e-12,
                                         4.8667030334736e-14,
                                         5.99175438145944e-17,
                                         3.26547204614462e-21,
                                         3.91557926345616e-29,
                                         0.0280250549350645,
                                         0.131518791336877,
                                         0.262167281049103,
                                         0.358848432503111,
                                         0.384458405925433,
                                         0.339777995330198,
                                         0.253744202697357,
                                         0.161821365486058,
                                         0.0883622281700758,
                                         0.0411905295345452,
                                         0.0162643455884066,
                                         0.00537109375,
                                         0.00145644934685823,
                                         0.000316106825925699,
                                         5.3006134729047e-05,
                                         6.53541706086995e-06,
                                         5.51779397772054e-07,
                                         2.86778662977829e-08,
                                         7.75075402211119e-10,
                                         8.1518444439098e-12,
                                         1.9006837756011e-14,
                                         2.58049826182097e-18,
                                         1.64257225110447e-25,
                                         0.000367435150191696,
                                         0.00915362039394014,
                                         0.0454553822306642,
                                         0.117829139389268,
                                         0.207803136260126,
                                         0.278102132550871,
                                         0.297978513939328,
                                         0.262911768516829,
                                         0.193750696140428,
                                         0.119886818411544,
                                         0.0621968923713716,
                                         0.02685546875,
                                         0.00952145461218202,
                                         0.00271518748305321,
                                         0.000604351399071504,
                                         0.000100563215444572,
                                         1.17467213912112e-05,
                                         8.75946889693995e-07,
                                         3.58493450783494e-08,
                                         6.2066069052794e-10,
                                         2.74058358510145e-12,
                                         9.26911969897413e-16,
                                         3.13206136838025e-22,
                                         2.89045477147252e-06,
                                         0.000382251534391326,
                                         0.00472871770756655,
                                         0.0232137663119291,
                                         0.0673916493030245,
                                         0.136572933843198,
                                         0.20995441982663,
                                         0.256292231190951,
                                         0.254900762683212,
                                         0.209361463296388,
                                         0.142709218749512,
                                         0.08056640625,
                                         0.0373475801794569,
                                         0.0139931993810124,
                                         0.00413432085278463,
                                         0.000928445135760491,
                                         0.000150044163301374,
                                         1.60531389385249e-05,
                                         9.94877818762088e-07,
                                         2.83533152834692e-08,
                                         2.37097779757253e-10,
                                         1.99767419955436e-13,
                                         3.58333403308274e-19,
                                         1.51586455125836e-08,
                                         1.06417810116789e-05,
                                         0.000327951939690123,
                                         0.00304892858239749,
                                         0.0145703107197788,
                                         0.0447129886838163,
                                         0.0986220064929736,
                                         0.166559572791005,
                                         0.223567020612202,
                                         0.243742238968196,
                                         0.218295162771083,
                                         0.1611328125,
                                         0.097663070897897,
                                         0.0480776202107363,
                                         0.0188550446423831,
                                         0.00571455056938745,
                                         0.00127770408956571,
                                         0.000196133101075425,
                                         1.84063218282254e-05,
                                         8.63499708007387e-07,
                                         1.36747899173545e-08,
                                         2.87024898240467e-11,
                                         2.7330845477206e-16,
                                         5.56483966574493e-11,
                                         2.07385046332385e-07,
                                         1.59211729222262e-05,
                                         0.000280315385403373,
                                         0.00220510656240848,
                                         0.0102470958962786,
                                         0.0324280390044284,
                                         0.0757707863854716,
                                         0.137259490813191,
                                         0.198638253121046,
                                         0.23374064377037,
                                         0.2255859375,
                                         0.178770693039916,
                                         0.115629046048252,
                                         0.0601934162315354,
                                         0.0246210151430611,
                                         0.00761622040605288,
                                         0.00167741246365232,
                                         0.000238375882745112,
                                         1.84085076750971e-05,
                                         5.52092540186012e-07,
                                         2.88677225544736e-09,
                                         1.45920691546415e-13,
                                         1.45920691546433e-13,
                                         2.88677225544736e-09,
                                         5.52092540186007e-07,
                                         1.84085076750971e-05,
                                         0.000238375882745112,
                                         0.00167741246365232,
                                         0.00761622040605289,
                                         0.0246210151430611,
                                         0.0601934162315354,
                                         0.115629046048252,
                                         0.178770693039916,
                                         0.2255859375,
                                         0.23374064377037,
                                         0.198638253121046,
                                         0.13725949081319,
                                         0.0757707863854717,
                                         0.0324280390044284,
                                         0.0102470958962786,
                                         0.00220510656240848,
                                         0.000280315385403373,
                                         1.59211729222263e-05,
                                         2.07385046332385e-07,
                                         5.56483966574433e-11,
                                         2.733084547721e-16,
                                         2.87024898240467e-11,
                                         1.36747899173543e-08,
                                         8.63499708007387e-07,
                                         1.84063218282254e-05,
                                         0.000196133101075425,
                                         0.00127770408956571,
                                         0.00571455056938744,
                                         0.0188550446423831,
                                         0.0480776202107363,
                                         0.097663070897897,
                                         0.1611328125,
                                         0.218295162771083,
                                         0.243742238968196,
                                         0.223567020612202,
                                         0.166559572791005,
                                         0.0986220064929735,
                                         0.0447129886838164,
                                         0.0145703107197788,
                                         0.00304892858239749,
                                         0.000327951939690125,
                                         1.06417810116789e-05,
                                         1.51586455125823e-08,
                                         3.58333403308335e-19,
                                         1.99767419955436e-13,
                                         2.3709777975725e-10,
                                         2.83533152834692e-08,
                                         9.94877818762088e-07,
                                         1.60531389385249e-05,
                                         0.000150044163301374,
                                         0.000928445135760489,
                                         0.00413432085278463,
                                         0.0139931993810124,
                                         0.0373475801794569,
                                         0.08056640625,
                                         0.142709218749512,
                                         0.209361463296388,
                                         0.254900762683212,
                                         0.256292231190952,
                                         0.209954419826629,
                                         0.136572933843198,
                                         0.0673916493030245,
                                         0.0232137663119291,
                                         0.00472871770756657,
                                         0.000382251534391326,
                                         2.89045477147233e-06,
                                         3.13206136838085e-22,
                                         9.26911969897413e-16,
                                         2.74058358510141e-12,
                                         6.2066069052794e-10,
                                         3.58493450783494e-08,
                                         8.75946889693992e-07,
                                         1.17467213912113e-05,
                                         0.000100563215444572,
                                         0.000604351399071504,
                                         0.00271518748305321,
                                         0.00952145461218202,
                                         0.02685546875,
                                         0.0621968923713716,
                                         0.119886818411544,
                                         0.193750696140428,
                                         0.262911768516829,
                                         0.297978513939328,
                                         0.278102132550871,
                                         0.207803136260126,
                                         0.117829139389268,
                                         0.0454553822306643,
                                         0.00915362039394014,
                                         0.000367435150191681,
                                         1.64257225110482e-25,
                                         2.58049826182097e-18,
                                         1.90068377560107e-14,
                                         8.1518444439098e-12,
                                         7.75075402211119e-10,
                                         2.86778662977828e-08,
                                         5.51779397772056e-07,
                                         6.53541706086994e-06,
                                         5.3006134729047e-05,
                                         0.000316106825925699,
                                         0.00145644934685823,
                                         0.00537109375,
                                         0.0162643455884066,
                                         0.0411905295345452,
                                         0.0883622281700758,
                                         0.161821365486058,
                                         0.253744202697357,
                                         0.339777995330198,
                                         0.384458405925433,
                                         0.358848432503111,
                                         0.262167281049104,
                                         0.131518791336877,
                                         0.0280250549350639,
                                         3.91557926345707e-29,
                                         3.26547204614462e-21,
                                         5.99175438145933e-17,
                                         4.8667030334736e-14,
                                         7.61700081016815e-12,
                                         4.26769344263459e-10,
                                         1.17812565297615e-08,
                                         1.93056657630677e-07,
                                         2.11319736048694e-06,
                                         1.67280471624036e-05,
                                         0.00010126626399222,
                                         0.00048828125,
                                         0.00193322240895345,
                                         0.0064328086721382,
                                         0.018317549123027,
                                         0.0452729730923123,
                                         0.0982165390983178,
                                         0.188696349940446,
                                         0.3233135775148,
                                         0.496760116838332,
                                         0.687304179892866,
                                         0.858934484653058,
                                         0.971604604245533,

                                         0.973845910990539,
                                         0.869463826812977,
                                         0.708334312685789,
                                         0.525631077207208,
                                         0.354372921047049,
                                         0.216298117443718,
                                         0.118979133068524,
                                         0.0586389800854099,
                                         0.0257053044782195,
                                         0.00993135203540935,
                                         0.00334330649062399,
                                         0.000966681463525941,
                                         0.000235729535364489,
                                         4.73589219240518e-05,
                                         7.60271435232113e-06,
                                         9.36253256464779e-07,
                                         8.36311660326704e-08,
                                         5.00458604542191e-09,
                                         1.78276637615853e-10,
                                         3.1377251254542e-12,
                                         1.98234248410491e-14,
                                         2.41978513900993e-17,
                                         1.31099396219046e-21,
                                         1.56684544255765e-29,
                                         0.025840165210389,
                                         0.122395895904866,
                                         0.248130369409789,
                                         0.348141121847358,
                                         0.385521910169011,
                                         0.355329310649401,
                                         0.279452950875119,
                                         0.189732211943338,
                                         0.111661555358323,
                                         0.0569016323227907,
                                         0.0249745963715625,
                                         0.00935320981912534,
                                         0.00294796338097257,
                                         0.000767122800822886,
                                         0.000160560124176007,
                                         2.60794117439929e-05,
                                         3.12751110495589e-06,
                                         2.57819422445047e-07,
                                         1.31311121636257e-08,
                                         3.48988992447038e-10,
                                         3.62151066739638e-12,
                                         8.35834962609184e-15,
                                         1.12686197591033e-18,
                                         7.14507000287423e-26,
                                         0.000311657545597199,
                                         0.00783176255629703,
                                         0.0395092588862638,
                                         0.104811016679668,
                                         0.190640391408244,
                                         0.265330176741812,
                                         0.29834854130504,
                                         0.279044232696708,
                                         0.220476300304694,
                                         0.148189827594562,
                                         0.0848004357460414,
                                         0.0411353580740843,
                                         0.0167574328584159,
                                         0.00564813793391046,
                                         0.00154128884888116,
                                         0.000330201894051644,
                                         5.31626826763328e-05,
                                         6.03726752609817e-06,
                                         4.39628568681557e-07,
                                         1.76435463128097e-08,
                                         3.00730979068473e-10,
                                         1.3123255401656e-12,
                                         4.40268969185544e-16,
                                         1.48103095787622e-22,
                                         2.25533602289709e-06,
                                         0.000300679222704774,
                                         0.00377458400143799,
                                         0.0189325796834347,
                                         0.0565629468172468,
                                         0.118875815610896,
                                         0.191113069631437,
                                         0.246238684524107,
                                         0.261198935515754,
                                         0.231559877346895,
                                         0.172762285221315,
                                         0.108547827961035,
                                         0.0571536724947393,
                                         0.0249515165652158,
                                         0.00887731494183936,
                                         0.0025084911862034,
                                         0.000542208305805708,
                                         8.48235532513811e-05,
                                         8.83123726273747e-06,
                                         5.3519406037914e-07,
                                         1.49836568345389e-08,
                                         1.23627156107098e-10,
                                         1.03208788319143e-13,
                                         1.84192963593532e-19,
                                         1.08806191240675e-08,
                                         7.69584065029787e-06,
                                         0.000240407519410849,
                                         0.0022799293739964,
                                         0.0111881395438369,
                                         0.0355066021530517,
                                         0.0816141756532721,
                                         0.144859924584227,
                                         0.20629544254954,
                                         0.241222031081243,
                                         0.234643504932574,
                                         0.190957066405629,
                                         0.129953965591868,
                                         0.0734847704015141,
                                         0.0340869355902198,
                                         0.0127044052837888,
                                         0.00368666932121763,
                                         0.000794513429029149,
                                         0.000118267642500511,
                                         1.08229444423787e-05,
                                         4.97698358913458e-07,
                                         7.76416788840374e-09,
                                         1.61296158058576e-11,
                                         1.52718157384183e-16,
                                         3.67446402331055e-11,
                                         1.37881739707204e-07,
                                         1.07182785591954e-05,
                                         0.000192190109648492,
                                         0.00154910823157361,
                                         0.00742374008525988,
                                         0.0243971360843404,
                                         0.0596538210629755,
                                         0.114052787668809,
                                         0.175901146010079,
                                         0.223082844936662,
                                         0.235151834234155,
                                         0.206839258181086,
                                         0.151494119677834,
                                         0.0916204313895538,
                                         0.0450395600956165,
                                         0.0175468936512646,
                                         0.00520935630845157,
                                         0.00110868549814107,
                                         0.000153206648862814,
                                         1.15721122981084e-05,
                                         3.41329635237813e-07,
                                         1.76453146284464e-09,
                                         8.86352257863742e-14,
                                         8.86352257863864e-14,
                                         1.76453146284464e-09,
                                         3.41329635237816e-07,
                                         1.15721122981084e-05,
                                         0.000153206648862814,
                                         0.00110868549814108,
                                         0.00520935630845157,
                                         0.0175468936512646,
                                         0.0450395600956165,
                                         0.0916204313895539,
                                         0.151494119677834,
                                         0.206839258181086,
                                         0.235151834234155,
                                         0.223082844936662,
                                         0.175901146010079,
                                         0.114052787668809,
                                         0.0596538210629755,
                                         0.0243971360843404,
                                         0.00742374008525986,
                                         0.0015491082315736,
                                         0.000192190109648492,
                                         1.07182785591953e-05,
                                         1.37881739707204e-07,
                                         3.67446402331013e-11,
                                         1.52718157384208e-16,
                                         1.61296158058576e-11,
                                         7.76416788840384e-09,
                                         4.97698358913458e-07,
                                         1.08229444423787e-05,
                                         0.000118267642500511,
                                         0.000794513429029149,
                                         0.00368666932121764,
                                         0.0127044052837888,
                                         0.0340869355902198,
                                         0.0734847704015141,
                                         0.129953965591867,
                                         0.190957066405629,
                                         0.234643504932574,
                                         0.241222031081243,
                                         0.20629544254954,
                                         0.144859924584227,
                                         0.0816141756532721,
                                         0.0355066021530517,
                                         0.0111881395438369,
                                         0.0022799293739964,
                                         0.000240407519410848,
                                         7.69584065029787e-06,
                                         1.08806191240665e-08,
                                         1.84192963593566e-19,
                                         1.03208788319143e-13,
                                         1.236271561071e-10,
                                         1.49836568345389e-08,
                                         5.3519406037914e-07,
                                         8.8312372627375e-06,
                                         8.48235532513811e-05,
                                         0.000542208305805709,
                                         0.0025084911862034,
                                         0.00887731494183936,
                                         0.0249515165652158,
                                         0.0571536724947393,
                                         0.108547827961035,
                                         0.172762285221315,
                                         0.231559877346895,
                                         0.261198935515754,
                                         0.246238684524107,
                                         0.191113069631437,
                                         0.118875815610895,
                                         0.0565629468172468,
                                         0.0189325796834347,
                                         0.00377458400143797,
                                         0.000300679222704774,
                                         2.25533602289693e-06,
                                         1.48103095787653e-22,
                                         4.40268969185544e-16,
                                         1.31232554016562e-12,
                                         3.00730979068473e-10,
                                         1.76435463128097e-08,
                                         4.39628568681559e-07,
                                         6.03726752609817e-06,
                                         5.31626826763329e-05,
                                         0.000330201894051644,
                                         0.00154128884888116,
                                         0.00564813793391046,
                                         0.0167574328584159,
                                         0.0411353580740843,
                                         0.0848004357460414,
                                         0.148189827594562,
                                         0.220476300304694,
                                         0.279044232696708,
                                         0.29834854130504,
                                         0.265330176741812,
                                         0.190640391408244,
                                         0.104811016679668,
                                         0.0395092588862636,
                                         0.00783176255629703,
                                         0.000311657545597185,
                                         7.14507000287587e-26,
                                         1.12686197591033e-18,
                                         8.35834962609199e-15,
                                         3.62151066739638e-12,
                                         3.48988992447038e-10,
                                         1.31311121636258e-08,
                                         2.57819422445047e-07,
                                         3.1275111049559e-06,
                                         2.60794117439929e-05,
                                         0.000160560124176007,
                                         0.000767122800822886,
                                         0.00294796338097257,
                                         0.00935320981912534,
                                         0.0249745963715625,
                                         0.0569016323227907,
                                         0.111661555358323,
                                         0.189732211943338,
                                         0.279452950875119,
                                         0.355329310649401,
                                         0.385521910169011,
                                         0.348141121847358,
                                         0.248130369409789,
                                         0.122395895904867,
                                         0.0258401652103884,
                                         1.56684544255805e-29,
                                         1.31099396219046e-21,
                                         2.41978513900997e-17,
                                         1.98234248410491e-14,
                                         3.1377251254542e-12,
                                         1.78276637615854e-10,
                                         5.00458604542191e-09,
                                         8.36311660326706e-08,
                                         9.36253256464779e-07,
                                         7.60271435232113e-06,
                                         4.73589219240518e-05,
                                         0.000235729535364489,
                                         0.000966681463525941,
                                         0.00334330649062399,
                                         0.00993135203540935,
                                         0.0257053044782195,
                                         0.0586389800854099,
                                         0.118979133068524,
                                         0.216298117443718,
                                         0.354372921047049,
                                         0.525631077207208,
                                         0.708334312685789,
                                         0.869463826812977,
                                         0.973845910990539,

                                         0.000244140625,
                                         0.0029296875,
                                         0.01611328125,
                                         0.0537109375,
                                         0.120849609375,
                                         0.193359375,
                                         0.2255859375,
                                         0.193359375,
                                         0.120849609375,
                                         0.0537109375,
                                         0.01611328125,
                                         0.0029296875,
                                         0.000244140625,

                                         0.0579132293789379,
                                         7.93244761040481e-09,
                                         0.186213636517977,
                                         3.55251570124909e-07,
                                         0.2744268643856,
                                         7.29199300046128e-06,
                                         0.245108188978423,
                                         9.07136278871939e-05,
                                         0.147772217913803,
                                         0.000761732703481285,
                                         0.0633527143260148,
                                         0.00454852024188633,
                                         0.0198045267489712,
                                         0.0198045267489712,
                                         0.00454852024188633,
                                         0.0633527143260148,
                                         0.000761732703481285,
                                         0.147772217913803,
                                         9.07136278871938e-05,
                                         0.245108188978423,
                                         7.29199300046128e-06,
                                         0.2744268643856,
                                         3.55251570124909e-07,
                                         0.186213636517977,
                                         7.93244761040481e-09,
                                         0.0579132293789379,

                                         0.2381419999958,
                                         0.000244140625,
                                         4.19917528204876e-12,
                                         0.362975999603279,
                                         0.0029296875,
                                         3.96720444760508e-10,
                                         0.253571982821446,
                                         0.01611328125,
                                         1.71785540008471e-08,
                                         0.107359549178435,
                                         0.0537109375,
                                         4.50821565201979e-07,
                                         0.0306820140509815,
                                         0.120849609375,
                                         7.98594901862925e-06,
                                         0.00623540281019632,
                                         0.193359375,
                                         0.000100597189803727,
                                         0.000924000000000009,
                                         0.2255859375,
                                         0.000924000000000009,
                                         0.000100597189803727,
                                         0.193359375,
                                         0.00623540281019632,
                                         7.98594901862924e-06,
                                         0.120849609375,
                                         0.0306820140509815,
                                         4.50821565201979e-07,
                                         0.0537109375,
                                         0.107359549178435,
                                         1.71785540008471e-08,
                                         0.01611328125,
                                         0.253571982821446,
                                         3.96720444760508e-10,
                                         0.0029296875,
                                         0.362975999603279,
                                         4.19917528204876e-12,
                                         0.000244140625,
                                         0.2381419999958,

                                         0.421675376949324,
                                         0.00818132992300697,
                                         1.66846446406411e-06,
                                         1.2551753747439e-14,
                                         0.377546111727832,
                                         0.0483573960726327,
                                         4.06481189428379e-05,
                                         2.01871561457488e-12,
                                         0.154932741412802,
                                         0.131003636461783,
                                         0.000453884537273641,
                                         1.48808355654945e-10,
                                         0.0385329682613798,
                                         0.215089817471783,
                                         0.00307161216109281,
                                         6.6480694691135e-09,
                                         0.0064688264075609,
                                         0.238374680352024,
                                         0.0140310813280195,
                                         2.00478384048766e-07,
                                         0.000772246579912095,
                                         0.187861532537298,
                                         0.0455778012259822,
                                         4.29909479750649e-06,
                                         6.72222890967225e-05,
                                         0.107954911345698,
                                         0.107954911345698,
                                         6.72222890967229e-05,
                                         4.29909479750647e-06,
                                         0.0455778012259822,
                                         0.187861532537298,
                                         0.000772246579912098,
                                         2.00478384048765e-07,
                                         0.0140310813280195,
                                         0.238374680352024,
                                         0.00646882640756092,
                                         6.64806946911345e-09,
                                         0.00307161216109281,
                                         0.215089817471783,
                                         0.0385329682613799,
                                         1.48808355654944e-10,
                                         0.000453884537273641,
                                         0.131003636461783,
                                         0.154932741412803,
                                         2.01871561457486e-12,
                                         4.06481189428379e-05,
                                         0.0483573960726327,
                                         0.377546111727833,
                                         1.25517537474389e-14,
                                         1.66846446406411e-06,
                                         0.00818132992300697,
                                         0.421675376949324,

                                         0.561832079910847,
                                         0.0429245843703944,
                                         0.000244140625,
                                         2.28058925324772e-08,
                                         1.1355172946616e-16,
                                         0.331833362352312,
                                         0.154525121101316,
                                         0.0029296875,
                                         9.12255670594352e-07,
                                         2.7684885450736e-14,
                                         0.0898286774183675,
                                         0.254960868596377,
                                         0.01611328125,
                                         1.67250534059905e-05,
                                         3.09366406037311e-12,
                                         0.0147375746880575,
                                         0.254955287497755,
                                         0.0537109375,
                                         0.000185837994730729,
                                         2.09517174676303e-10,
                                         0.00163207760485319,
                                         0.17209105190188,
                                         0.120849609375,
                                         0.0013938154711296,
                                         9.57789074659783e-09,
                                         0.000128526611088561,
                                         0.0826018967161144,
                                         0.193359375,
                                         0.00743384523971537,
                                         3.11356253724867e-07,
                                         7.38026769028941e-06,
                                         0.0289100309956177,
                                         0.2255859375,
                                         0.0289100309956177,
                                         7.38026769028946e-06,
                                         3.11356253724865e-07,
                                         0.00743384523971537,
                                         0.193359375,
                                         0.0826018967161144,
                                         0.000128526611088562,
                                         9.57789074659774e-09,
                                         0.0013938154711296,
                                         0.120849609375,
                                         0.17209105190188,
                                         0.0016320776048532,
                                         2.09517174676301e-10,
                                         0.000185837994730729,
                                         0.0537109375,
                                         0.254955287497755,
                                         0.0147375746880576,
                                         3.09366406037307e-12,
                                         1.67250534059905e-05,
                                         0.01611328125,
                                         0.254960868596377,
                                         0.0898286774183677,
                                         2.76848854507356e-14,
                                         9.12255670594353e-07,
                                         0.0029296875,
                                         0.154525121101316,
                                         0.331833362352312,
                                         1.13551729466159e-16,
                                         2.28058925324772e-08,
                                         0.000244140625,
                                         0.0429245843703944,
                                         0.561832079910847,

                                         0.662204105405035,
                                         0.107828252890832,
                                         0.00318341814132717,
                                         9.26537918858577e-06,
                                         5.5823425701271e-10,
                                         2.1960313678622e-18,
                                         0.27769006200199,
                                         0.263888708436918,
                                         0.0234822149314105,
                                         0.000180875738703429,
                                         3.28466238632977e-08,
                                         7.54106289131827e-16,
                                         0.0533716033167337,
                                         0.295999167777284,
                                         0.0793900132087502,
                                         0.00161837395805111,
                                         8.85821942304254e-07,
                                         1.1868833288569e-13,
                                         0.00621693343369831,
                                         0.201222355379015,
                                         0.162670478957266,
                                         0.00877593707843962,
                                         1.44783204975766e-05,
                                         1.13213848178723e-11,
                                         0.000488816613499963,
                                         0.0923348694208237,
                                         0.224985945547836,
                                         0.0321226889956864,
                                         0.000159732655326367,
                                         7.28945145471973e-10,
                                         2.73308515410835e-05,
                                         0.0301295548329948,
                                         0.221278642237401,
                                         0.0836118340181624,
                                         0.0012531605103725,
                                         3.33754867486345e-08,
                                         1.11426162894336e-06,
                                         0.0071688005491365,
                                         0.158690311807777,
                                         0.158690311807777,
                                         0.00716880054913648,
                                         1.11426162894336e-06,
                                         3.33754867486345e-08,
                                         0.0012531605103725,
                                         0.0836118340181624,
                                         0.221278642237401,
                                         0.0301295548329948,
                                         2.73308515410835e-05,
                                         7.28945145471973e-10,
                                         0.000159732655326368,
                                         0.0321226889956864,
                                         0.224985945547836,
                                         0.0923348694208236,
                                         0.000488816613499963,
                                         1.13213848178723e-11,
                                         1.44783204975766e-05,
                                         0.00877593707843962,
                                         0.162670478957266,
                                         0.201222355379014,
                                         0.00621693343369831,
                                         1.1868833288569e-13,
                                         8.85821942304257e-07,
                                         0.00161837395805111,
                                         0.0793900132087502,
                                         0.295999167777284,
                                         0.0533716033167337,
                                         7.54106289131827e-16,
                                         3.28466238632978e-08,
                                         0.000180875738703429,
                                         0.0234822149314105,
                                         0.263888708436918,
                                         0.27769006200199,
                                         2.1960313678622e-18,
                                         5.58234257012713e-10,
                                         9.26537918858577e-06,
                                         0.00318341814132717,
                                         0.107828252890832,
                                         0.662204105405035,

                                         0.733957082984539,
                                         0.190026916530522,
                                         0.0145509014973105,
                                         0.000244140625,
                                         4.72536977020843e-07,
                                         2.17039040406951e-11,
                                         7.3695887030193e-20,
                                         0.229967410017886,
                                         0.338433435307855,
                                         0.0737960820335275,
                                         0.0029296875,
                                         1.34169835260554e-05,
                                         1.75485893679429e-09,
                                         3.3869603656684e-17,
                                         0.03302504101058,
                                         0.276256717223765,
                                         0.17153713514349,
                                         0.01611328125,
                                         0.000174604494164584,
                                         6.50321341285703e-08,
                                         7.13441588521059e-15,
                                         0.00287432910777849,
                                         0.136668518602983,
                                         0.241656819895795,
                                         0.0537109375,
                                         0.00137712069560675,
                                         1.46059515811259e-06,
                                         9.10799503547123e-13,
                                         0.000168862569363464,
                                         0.0456381361918279,
                                         0.229796670694522,
                                         0.120849609375,
                                         0.00733149199885157,
                                         2.21429500204757e-05,
                                         7.84857696244476e-11,
                                         7.05452469011272e-06,
                                         0.0108373918772131,
                                         0.155391004144899,
                                         0.193359375,
                                         0.0277555724817305,
                                         0.000238714372354775,
                                         4.80946226805621e-09,
                                         2.14896295757603e-07,
                                         0.00187649953960272,
                                         0.0766187073996,
                                         0.2255859375,
                                         0.0766187073996,
                                         0.00187649953960272,
                                         2.14896295757606e-07,
                                         4.80946226805613e-09,
                                         0.000238714372354775,
                                         0.0277555724817305,
                                         0.193359375,
                                         0.155391004144899,
                                         0.0108373918772131,
                                         7.05452469011279e-06,
                                         7.84857696244462e-11,
                                         2.21429500204757e-05,
                                         0.00733149199885157,
                                         0.120849609375,
                                         0.229796670694522,
                                         0.0456381361918279,
                                         0.000168862569363466,
                                         9.10799503547105e-13,
                                         1.46059515811259e-06,
                                         0.00137712069560675,
                                         0.0537109375,
                                         0.241656819895795,
                                         0.136668518602983,
                                         0.00287432910777851,
                                         7.13441588521043e-15,
                                         6.50321341285703e-08,
                                         0.000174604494164584,
                                         0.01611328125,
                                         0.17153713514349,
                                         0.276256717223765,
                                         0.0330250410105801,
                                         3.38696036566831e-17,
                                         1.75485893679429e-09,
                                         1.34169835260554e-05,
                                         0.0029296875,
                                         0.0737960820335275,
                                         0.338433435307855,
                                         0.229967410017887,
                                         7.36958870301911e-20,
                                         2.17039040406951e-11,
                                         4.72536977020843e-07,
                                         0.000244140625,
                                         0.0145509014973105,
                                         0.190026916530522,
                                         0.733957082984539,

                                         0.786110440091379,
                                         0.276216506838666,
                                         0.038787993656943,
                                         0.00184234682378424,
                                         2.14551456090738e-05,
                                         3.17774230746812e-08,
                                         1.21940470439282e-12,
                                         3.75368245124059e-21,
                                         0.19109352597838,
                                         0.375121878465898,
                                         0.144765033544714,
                                         0.0152545467789033,
                                         0.000373135044344779,
                                         1.22606870902605e-06,
                                         1.29296745184891e-10,
                                         2.22360484793206e-18,
                                         0.0212906944481328,
                                         0.233494472394336,
                                         0.247634687282456,
                                         0.0578906827486273,
                                         0.00297428140395272,
                                         2.16816527687491e-05,
                                         6.28360334373979e-09,
                                         6.03724969010555e-16,
                                         0.00143763863798022,
                                         0.088083967676131,
                                         0.256729240957205,
                                         0.133147829088155,
                                         0.0143685891873001,
                                         0.000232373188324244,
                                         1.85074245529547e-07,
                                         9.93428738421494e-14,
                                         6.55259639112686e-05,
                                         0.0224295678837945,
                                         0.179656513886995,
                                         0.206710853900466,
                                         0.046854356012629,
                                         0.00168106080190609,
                                         3.67948456913785e-06,
                                         1.10341294225238e-11,
                                         2.12380463750629e-06,
                                         0.00406146214023419,
                                         0.0894021708174356,
                                         0.228207512275368,
                                         0.108648386900247,
                                         0.00864805197217513,
                                         5.2019413192215e-05,
                                         8.71519065889538e-10,
                                         5.01929265315222e-08,
                                         0.000536254214814611,
                                         0.032439934392945,
                                         0.183706024690614,
                                         0.183706024690613,
                                         0.032439934392945,
                                         0.000536254214814611,
                                         5.01929265315222e-08,
                                         8.71519065889539e-10,
                                         5.2019413192215e-05,
                                         0.00864805197217513,
                                         0.108648386900247,
                                         0.228207512275368,
                                         0.0894021708174356,
                                         0.00406146214023419,
                                         2.12380463750629e-06,
                                         1.10341294225238e-11,
                                         3.67948456913785e-06,
                                         0.00168106080190609,
                                         0.0468543560126291,
                                         0.206710853900466,
                                         0.179656513886995,
                                         0.0224295678837945,
                                         6.55259639112686e-05,
                                         9.93428738421494e-14,
                                         1.85074245529547e-07,
                                         0.000232373188324244,
                                         0.0143685891873001,
                                         0.133147829088155,
                                         0.256729240957205,
                                         0.088083967676131,
                                         0.00143763863798022,
                                         6.03724969010555e-16,
                                         6.28360334373979e-09,
                                         2.16816527687491e-05,
                                         0.00297428140395273,
                                         0.0578906827486273,
                                         0.247634687282456,
                                         0.233494472394336,
                                         0.0212906944481328,
                                         2.22360484793206e-18,
                                         1.29296745184891e-10,
                                         1.22606870902605e-06,
                                         0.00037313504434478,
                                         0.0152545467789033,
                                         0.144765033544714,
                                         0.375121878465898,
                                         0.19109352597838,
                                         3.75368245124059e-21,
                                         1.21940470439282e-12,
                                         3.17774230746812e-08,
                                         2.14551456090738e-05,
                                         0.00184234682378424,
                                         0.038787993656943,
                                         0.276216506838666,
                                         0.786110440091379,

                                         0.824831997510693,
                                         0.358261774463816,
                                         0.07593681989392,
                                         0.00710057148394215,
                                         0.000244140625,
                                         2.21333067157877e-06,
                                         2.72375056637647e-09,
                                         9.2209914060134e-14,
                                         2.65019334057822e-22,
                                         0.160123872355578,
                                         0.383939811552936,
                                         0.218370128185911,
                                         0.0434797761880113,
                                         0.0029296875,
                                         5.20492886619121e-05,
                                         1.36392032827438e-07,
                                         1.23901748331431e-11,
                                         1.96584587796125e-19,
                                         0.0142471537805463,
                                         0.18858505188875,
                                         0.287816372904916,
                                         0.122028912027149,
                                         0.01611328125,
                                         0.000561002424516941,
                                         3.1303443514751e-06,
                                         7.63060012565969e-10,
                                         6.68348253502562e-17,
                                         0.000768274105202087,
                                         0.0561393577148619,
                                         0.229907859707326,
                                         0.207565052858614,
                                         0.0537109375,
                                         0.00366463507146181,
                                         4.35422916492569e-05,
                                         2.8481051029174e-08,
                                         1.37712139198111e-14,
                                         2.79645639490999e-05,
                                         0.0112805788731606,
                                         0.123964094918079,
                                         0.238313970479918,
                                         0.120849609375,
                                         0.0161584798809423,
                                         0.000408821966756944,
                                         7.17558149684844e-07,
                                         1.91533790933321e-12,
                                         7.23831322706337e-07,
                                         0.00161188033833688,
                                         0.0475308382864017,
                                         0.1945728475964,
                                         0.193359375,
                                         0.0506649635598553,
                                         0.00272957246520414,
                                         1.28556978110386e-05,
                                         1.89433179650916e-10,
                                         1.36613463023803e-08,
                                         0.000167942655583806,
                                         0.0132886799197005,
                                         0.115835525809854,
                                         0.2255859375,
                                         0.115835525809854,
                                         0.0132886799197005,
                                         0.000167942655583806,
                                         1.36613463023806e-08,
                                         1.89433179650912e-10,
                                         1.28556978110386e-05,
                                         0.00272957246520414,
                                         0.0506649635598553,
                                         0.193359375,
                                         0.1945728475964,
                                         0.0475308382864018,
                                         0.00161188033833688,
                                         7.2383132270635e-07,
                                         1.91533790933316e-12,
                                         7.17558149684844e-07,
                                         0.000408821966756943,
                                         0.0161584798809423,
                                         0.120849609375,
                                         0.238313970479918,
                                         0.123964094918079,
                                         0.0112805788731606,
                                         2.79645639491002e-05,
                                         1.37712139198107e-14,
                                         2.8481051029174e-08,
                                         4.35422916492568e-05,
                                         0.00366463507146181,
                                         0.0537109375,
                                         0.207565052858614,
                                         0.229907859707326,
                                         0.0561393577148619,
                                         0.000768274105202095,
                                         6.68348253502539e-17,
                                         7.63060012565969e-10,
                                         3.13034435147509e-06,
                                         0.000561002424516941,
                                         0.01611328125,
                                         0.122028912027149,
                                         0.287816372904916,
                                         0.18858505188875,
                                         0.0142471537805464,
                                         1.96584587796118e-19,
                                         1.23901748331431e-11,
                                         1.36392032827437e-07,
                                         5.20492886619121e-05,
                                         0.0029296875,
                                         0.0434797761880113,
                                         0.218370128185912,
                                         0.383939811552936,
                                         0.160123872355578,
                                         2.6501933405781e-22,
                                         9.2209914060134e-14,
                                         2.72375056637647e-09,
                                         2.21333067157877e-06,
                                         0.000244140625,
                                         0.00710057148394215,
                                         0.07593681989392,
                                         0.358261774463816,
                                         0.824831997510693,

                                         0.854198989342353,
                                         0.432477137545819,
                                         0.122890845042486,
                                         0.0183667412475573,
                                         0.00129095291603943,
                                         3.52828993464022e-05,
                                         2.67301415415996e-07,
                                         2.87770802452941e-10,
                                         8.89607621455251e-15,
                                         2.43232941151254e-23,
                                         0.135501960079999,
                                         0.375474699466224,
                                         0.281510572343625,
                                         0.0871219223529665,
                                         0.0114765883445349,
                                         0.000571510687840654,
                                         8.11462413961914e-06,
                                         1.80898114726268e-08,
                                         1.4755144349979e-12,
                                         2.2079963908437e-20,
                                         0.00985175368779472,
                                         0.149409993297764,
                                         0.295563861792762,
                                         0.189410580456907,
                                         0.0467623845380691,
                                         0.00424293023442086,
                                         0.000112905994947212,
                                         5.21198067985548e-07,
                                         1.12168231762968e-10,
                                         9.18662038331437e-18,
                                         0.000434107779327372,
                                         0.036032522788422,
                                         0.188071942735051,
                                         0.24957269926379,
                                         0.115477280964829,
                                         0.0190907715576844,
                                         0.000952097997145654,
                                         9.10096846111653e-06,
                                         5.16787904648562e-09,
                                         2.31648364120079e-15,
                                         1.29117576200223e-05,
                                         0.00586561381968396,
                                         0.0807793700318224,
                                         0.221969697498446,
                                         0.192486476508004,
                                         0.0579808854986442,
                                         0.00541938585900593,
                                         0.000107269484697055,
                                         1.60715802130952e-07,
                                         3.94281570298222e-13,
                                         2.73093036075488e-07,
                                         0.000678999926987642,
                                         0.0246725693123347,
                                         0.140387282929993,
                                         0.228160973218627,
                                         0.125222875167218,
                                         0.0219359254021461,
                                         0.000899088088342055,
                                         3.55420345265727e-06,
                                         4.77222797248939e-11,
                                         4.21175105662464e-09,
                                         5.7312954313678e-05,
                                         0.00549484062476939,
                                         0.0647423790715412,
                                         0.197201087464742,
                                         0.197201087464742,
                                         0.0647423790715412,
                                         0.00549484062476939,
                                         5.73129543136777e-05,
                                         4.21175105662464e-09,
                                         4.77222797248939e-11,
                                         3.55420345265729e-06,
                                         0.000899088088342055,
                                         0.0219359254021461,
                                         0.125222875167218,
                                         0.228160973218627,
                                         0.140387282929993,
                                         0.0246725693123347,
                                         0.000678999926987639,
                                         2.73093036075488e-07,
                                         3.94281570298222e-13,
                                         1.60715802130953e-07,
                                         0.000107269484697055,
                                         0.00541938585900593,
                                         0.0579808854986442,
                                         0.192486476508004,
                                         0.221969697498446,
                                         0.0807793700318224,
                                         0.00586561381968394,
                                         1.29117576200223e-05,
                                         2.31648364120079e-15,
                                         5.16787904648566e-09,
                                         9.10096846111654e-06,
                                         0.000952097997145654,
                                         0.0190907715576844,
                                         0.115477280964829,
                                         0.24957269926379,
                                         0.188071942735051,
                                         0.0360325227884219,
                                         0.000434107779327372,
                                         9.18662038331437e-18,
                                         1.12168231762969e-10,
                                         5.21198067985548e-07,
                                         0.000112905994947212,
                                         0.00424293023442086,
                                         0.0467623845380691,
                                         0.189410580456907,
                                         0.295563861792762,
                                         0.149409993297764,
                                         0.00985175368779472,
                                         2.2079963908437e-20,
                                         1.47551443499791e-12,
                                         1.80898114726268e-08,
                                         8.11462413961914e-06,
                                         0.000571510687840654,
                                         0.0114765883445349,
                                         0.0871219223529665,
                                         0.281510572343625,
                                         0.375474699466224,
                                         0.135501960079999,
                                         2.43232941151254e-23,
                                         8.89607621455259e-15,
                                         2.87770802452941e-10,
                                         2.67301415415996e-07,
                                         3.52828993464022e-05,
                                         0.00129095291603943,
                                         0.0183667412475573,
                                         0.122890845042486,
                                         0.432477137545819,
                                         0.854198989342353,

                                         0.876915852631853,
                                         0.497823896515899,
                                         0.175651223103151,
                                         0.0368691510322005,
                                         0.00427966401208136,
                                         0.000244140625,
                                         5.63345465521764e-06,
                                         3.7354190105842e-08,
                                         3.63974021902337e-11,
                                         1.05121969039768e-15,
                                         2.76861404600999e-24,
                                         0.115810483643577,
                                         0.357526687155361,
                                         0.328751208819649,
                                         0.140061056861505,
                                         0.0295486752366845,
                                         0.0029296875,
                                         0.00011749217811038,
                                         1.41594881752598e-06,
                                         2.80037827360167e-09,
                                         2.1077701712445e-13,
                                         3.01880427172882e-21,
                                         0.00701002025529626,
                                         0.117685412297387,
                                         0.282010307438914,
                                         0.243867038453903,
                                         0.0935077911836918,
                                         0.01611328125,
                                         0.00112311583493808,
                                         2.4600120193897e-05,
                                         9.87516714070708e-08,
                                         1.93702160488218e-11,
                                         1.50865152917635e-18,
                                         0.000257161959433865,
                                         0.0234775545014071,
                                         0.146615086660333,
                                         0.257338559196576,
                                         0.179338543217065,
                                         0.0537109375,
                                         0.00650663135546044,
                                         0.00025902576123952,
                                         2.11051413966339e-06,
                                         1.0788516523343e-09,
                                         4.56939786592589e-16,
                                         6.3679251850496e-06,
                                         0.00316145390800213,
                                         0.0514512717153947,
                                         0.183299127945797,
                                         0.23216847555008,
                                         0.120849609375,
                                         0.0254443655215507,
                                         0.00184099437353936,
                                         3.04463930829743e-05,
                                         4.05595165922423e-08,
                                         9.34184774018416e-14,
                                         1.12131244713825e-07,
                                         0.000302731990230009,
                                         0.0128395862914552,
                                         0.0928439046850065,
                                         0.213732382257395,
                                         0.193359375,
                                         0.0707562022176403,
                                         0.00930463686099246,
                                         0.000312335176206523,
                                         1.0843295017414e-06,
                                         1.35813849554319e-11,
                                         1.4397336018969e-09,
                                         2.11376442620986e-05,
                                         0.00233632229922767,
                                         0.0342904514060396,
                                         0.143471027980647,
                                         0.2255859375,
                                         0.143471027980647,
                                         0.0342904514060395,
                                         0.00233632229922767,
                                         2.11376442620986e-05,
                                         1.43973360189695e-09,
                                         1.35813849554314e-11,
                                         1.0843295017414e-06,
                                         0.000312335176206524,
                                         0.00930463686099248,
                                         0.0707562022176402,
                                         0.193359375,
                                         0.213732382257395,
                                         0.0928439046850064,
                                         0.0128395862914552,
                                         0.000302731990230009,
                                         1.12131244713828e-07,
                                         9.34184774018378e-14,
                                         4.05595165922423e-08,
                                         3.04463930829744e-05,
                                         0.00184099437353936,
                                         0.0254443655215507,
                                         0.120849609375,
                                         0.23216847555008,
                                         0.183299127945797,
                                         0.0514512717153946,
                                         0.00316145390800213,
                                         6.36792518504973e-06,
                                         4.56939786592568e-16,
                                         1.0788516523343e-09,
                                         2.1105141396634e-06,
                                         0.000259025761239521,
                                         0.00650663135546043,
                                         0.0537109375,
                                         0.179338543217065,
                                         0.257338559196576,
                                         0.146615086660333,
                                         0.0234775545014071,
                                         0.000257161959433869,
                                         1.50865152917627e-18,
                                         1.93702160488218e-11,
                                         9.87516714070712e-08,
                                         2.4600120193897e-05,
                                         0.00112311583493808,
                                         0.01611328125,
                                         0.0935077911836918,
                                         0.243867038453903,
                                         0.282010307438913,
                                         0.117685412297387,
                                         0.00701002025529633,
                                         3.01880427172865e-21,
                                         2.1077701712445e-13,
                                         2.80037827360168e-09,
                                         1.41594881752598e-06,
                                         0.00011749217811038,
                                         0.0029296875,
                                         0.0295486752366845,
                                         0.140061056861504,
                                         0.328751208819648,
                                         0.357526687155361,
                                         0.115810483643578,
                                         2.76861404600982e-24,
                                         1.05121969039768e-15,
                                         3.63974021902339e-11,
                                         3.73541901058421e-08,
                                         5.63345465521763e-06,
                                         0.000244140625,
                                         0.00427966401208136,
                                         0.0368691510322005,
                                         0.175651223103151,
                                         0.497823896515899,
                                         0.876915852631853,

                                         0.894805085081134,
                                         0.554580147613495,
                                         0.230692076052367,
                                         0.0624607501075975,
                                         0.0104720392078571,
                                         0.00100589226418861,
                                         4.90169531888776e-05,
                                         9.94567542361742e-07,
                                         5.9569910653299e-09,
                                         5.37748128850508e-12,
                                         1.47409503543007e-16,
                                         3.7721747008999e-25,
                                         0.0999190514176459,
                                         0.335113812583207,
                                         0.359895245483218,
                                         0.194867802470844,
                                         0.0580781477833664,
                                         0.00938387652580949,
                                         0.00075662029869312,
                                         2.58235102223737e-05,
                                         2.74951788275963e-07,
                                         4.96361362547539e-10,
                                         3.51285291376847e-14,
                                         4.86445769956573e-22,
                                         0.00511386907184494,
                                         0.0928114979217812,
                                         0.257336259896117,
                                         0.278647033082385,
                                         0.147630366815174,
                                         0.0401231059417248,
                                         0.00535292437162334,
                                         0.000307310722576154,
                                         5.81657847839156e-06,
                                         2.09989931401815e-08,
                                         3.83685945921971e-12,
                                         2.87513676303454e-19,
                                         0.000158623293513274,
                                         0.0155785592708528,
                                         0.111517215227462,
                                         0.241482028083622,
                                         0.227433636770445,
                                         0.103973547055765,
                                         0.0229519866396072,
                                         0.00221644116325216,
                                         7.45752454152917e-05,
                                         5.3841237230168e-07,
                                         2.53984885775243e-10,
                                         1.0299084946728e-16,
                                         3.32114696363289e-06,
                                         0.00176504821291652,
                                         0.0326202000275752,
                                         0.141259927437818,
                                         0.236503442137942,
                                         0.181867437918804,
                                         0.0664283140465595,
                                         0.0107904230856615,
                                         0.000645394950545737,
                                         9.3182715704905e-06,
                                         1.13486349837225e-08,
                                         2.49024768734401e-14,
                                         4.94477676025133e-08,
                                         0.00014220776770054,
                                         0.00678529584979238,
                                         0.0587611934205705,
                                         0.174887069476165,
                                         0.226216616133275,
                                         0.13671735333941,
                                         0.0373558139912256,
                                         0.0039718597045772,
                                         0.000114681445957573,
                                         3.60592630386128e-07,
                                         4.28177572073969e-12,
                                         5.36824207919342e-10,
                                         8.35443092318109e-06,
                                         0.00102914783283578,
                                         0.0178233380093672,
                                         0.0942984907685703,
                                         0.205173308511351,
                                         0.205173308511351,
                                         0.0942984907685704,
                                         0.0178233380093672,
                                         0.00102914783283579,
                                         8.35443092318115e-06,
                                         5.36824207919322e-10,
                                         4.28177572073987e-12,
                                         3.60592630386125e-07,
                                         0.000114681445957572,
                                         0.0039718597045772,
                                         0.0373558139912255,
                                         0.13671735333941,
                                         0.226216616133275,
                                         0.174887069476165,
                                         0.0587611934205706,
                                         0.00678529584979239,
                                         0.000142207767700541,
                                         4.94477676025118e-08,
                                         2.49024768734413e-14,
                                         1.13486349837224e-08,
                                         9.31827157049046e-06,
                                         0.000645394950545735,
                                         0.0107904230856615,
                                         0.0664283140465595,
                                         0.181867437918804,
                                         0.236503442137942,
                                         0.141259927437818,
                                         0.0326202000275752,
                                         0.00176504821291653,
                                         3.32114696363281e-06,
                                         1.02990849467285e-16,
                                         2.5398488577524e-10,
                                         5.38412372301678e-07,
                                         7.45752454152915e-05,
                                         0.00221644116325216,
                                         0.0229519866396072,
                                         0.103973547055765,
                                         0.227433636770445,
                                         0.241482028083622,
                                         0.111517215227462,
                                         0.0155785592708529,
                                         0.000158623293513272,
                                         2.87513676303471e-19,
                                         3.83685945921967e-12,
                                         2.09989931401814e-08,
                                         5.81657847839154e-06,
                                         0.000307310722576154,
                                         0.00535292437162334,
                                         0.0401231059417248,
                                         0.147630366815174,
                                         0.278647033082385,
                                         0.257336259896118,
                                         0.0928114979217814,
                                         0.00511386907184487,
                                         4.86445769956605e-22,
                                         3.51285291376843e-14,
                                         4.96361362547536e-10,
                                         2.74951788275962e-07,
                                         2.58235102223737e-05,
                                         0.00075662029869312,
                                         0.00938387652580949,
                                         0.0580781477833664,
                                         0.194867802470844,
                                         0.359895245483218,
                                         0.335113812583208,
                                         0.0999190514176453,
                                         3.77217470090018e-25,
                                         1.47409503543005e-16,
                                         5.37748128850505e-12,
                                         5.95699106532988e-09,
                                         9.9456754236174e-07,
                                         4.90169531888776e-05,
                                         0.00100589226418861,
                                         0.0104720392078571,
                                         0.0624607501075975,
                                         0.230692076052367,
                                         0.554580147613495,
                                         0.894805085081134,

                                         0.90911932240828,
                                         0.603575987847105,
                                         0.285415235581208,
                                         0.0940213610700072,
                                         0.020827443524768,
                                         0.00294064718243016,
                                         0.000244140625,
                                         1.05299387433706e-05,
                                         1.93310192697036e-07,
                                         1.06942031544982e-09,
                                         9.09317485988542e-13,
                                         2.39258753273723e-17,
                                         5.98565109735912e-26,
                                         0.0869647014227455,
                                         0.311236978145123,
                                         0.377220191136586,
                                         0.245697801941689,
                                         0.0951594910480808,
                                         0.022069343322947,
                                         0.0029296875,
                                         0.000202042087573618,
                                         6.09257803989342e-06,
                                         5.89299652095607e-08,
                                         9.90741274286889e-11,
                                         6.68145567005611e-15,
                                         9.01055970102784e-23,
                                         0.00381282239207078,
                                         0.0735583314879919,
                                         0.228504182018779,
                                         0.294277767090925,
                                         0.199273568995984,
                                         0.0759132078954417,
                                         0.01611328125,
                                         0.0017768030581092,
                                         8.80093657585083e-05,
                                         1.48835106598275e-06,
                                         4.94750695104845e-09,
                                         8.55176564181921e-13,
                                         6.21689571204773e-20,
                                         0.000101313238211077,
                                         0.0105363115786799,
                                         0.0838898524665751,
                                         0.213613988328769,
                                         0.25290845202018,
                                         0.158256418462415,
                                         0.0537109375,
                                         0.00947006139712251,
                                         0.000770500167580688,
                                         2.27819364874749e-05,
                                         1.49736826941149e-07,
                                         6.63371097593834e-11,
                                         2.59962947023541e-17,
                                         1.81714502995425e-06,
                                         0.00101870658455969,
                                         0.0207887550116813,
                                         0.104666017113847,
                                         0.216661008593466,
                                         0.222694296661213,
                                         0.120849609375,
                                         0.034069838048611,
                                         0.00455323805153682,
                                         0.000235385476812375,
                                         3.05896572877282e-06,
                                         3.47345017741545e-09,
                                         7.33758413279874e-15,
                                         2.31766386181561e-08,
                                         7.00401529945585e-05,
                                         0.00366340552821999,
                                         0.0364686067792834,
                                         0.131988362906533,
                                         0.222840600914569,
                                         0.193359375,
                                         0.0871615225931575,
                                         0.0191339854880646,
                                         0.00172944260973193,
                                         4.44383639498528e-05,
                                         1.29331128272935e-07,
                                         1.47276074765017e-12,
                                         2.15545080187253e-10,
                                         3.51133176914627e-06,
                                         0.00047072614295446,
                                         0.00926529930199674,
                                         0.0586296539498153,
                                         0.162594688437667,
                                         0.2255859375,
                                         0.162594688437667,
                                         0.0586296539498153,
                                         0.00926529930199672,
                                         0.000470726142954461,
                                         3.51133176914624e-06,
                                         2.15545080187244e-10,
                                         1.47276074765024e-12,
                                         1.29331128272936e-07,
                                         4.44383639498527e-05,
                                         0.00172944260973193,
                                         0.0191339854880646,
                                         0.0871615225931575,
                                         0.193359375,
                                         0.222840600914569,
                                         0.131988362906533,
                                         0.0364686067792833,
                                         0.00366340552822,
                                         7.0040152994558e-05,
                                         2.31766386181553e-08,
                                         7.33758413279915e-15,
                                         3.47345017741548e-09,
                                         3.0589657287728e-06,
                                         0.000235385476812376,
                                         0.00455323805153682,
                                         0.034069838048611,
                                         0.120849609375,
                                         0.222694296661213,
                                         0.216661008593466,
                                         0.104666017113847,
                                         0.0207887550116813,
                                         0.00101870658455969,
                                         1.8171450299542e-06,
                                         2.59962947023557e-17,
                                         6.63371097593842e-11,
                                         1.49736826941149e-07,
                                         2.2781936487475e-05,
                                         0.000770500167580688,
                                         0.00947006139712251,
                                         0.0537109375,
                                         0.158256418462415,
                                         0.25290845202018,
                                         0.213613988328769,
                                         0.0838898524665752,
                                         0.0105363115786799,
                                         0.000101313238211074,
                                         6.21689571204816e-20,
                                         8.55176564181932e-13,
                                         4.94750695104842e-09,
                                         1.48835106598275e-06,
                                         8.80093657585084e-05,
                                         0.0017768030581092,
                                         0.01611328125,
                                         0.0759132078954417,
                                         0.199273568995984,
                                         0.294277767090925,
                                         0.228504182018779,
                                         0.0735583314879917,
                                         0.00381282239207072,
                                         9.01055970102854e-23,
                                         6.6814556700562e-15,
                                         9.90741274286883e-11,
                                         5.89299652095609e-08,
                                         6.09257803989342e-06,
                                         0.000202042087573618,
                                         0.0029296875,
                                         0.022069343322947,
                                         0.0951594910480808,
                                         0.245697801941689,
                                         0.377220191136586,
                                         0.311236978145123,
                                         0.0869647014227449,
                                         5.98565109735962e-26,
                                         2.39258753273727e-17,
                                         9.09317485988536e-13,
                                         1.06942031544983e-09,
                                         1.93310192697036e-07,
                                         1.05299387433706e-05,
                                         0.000244140625,
                                         0.00294064718243016,
                                         0.020827443524768,
                                         0.0940213610700072,
                                         0.285415235581208,
                                         0.603575987847105,
                                         0.90911932240828,

                                         0.920737181470952,
                                         0.645802614054716,
                                         0.338124656815603,
                                         0.129994750668804,
                                         0.035764064884341,
                                         0.00677675154328227,
                                         0.000836326049776105,
                                         6.1900941610224e-05,
                                         2.42407130653098e-06,
                                         4.11025272080868e-08,
                                         2.13448386253154e-10,
                                         1.7303152999683e-13,
                                         4.40612314892168e-18,
                                         1.08253627069562e-26,
                                         0.0762972980943717,
                                         0.28759242801536,
                                         0.383717955268606,
                                         0.289104143723578,
                                         0.137297698656726,
                                         0.0419755564122556,
                                         0.00807855475653259,
                                         0.000922787491199586,
                                         5.63550688428276e-05,
                                         1.5417524027582e-06,
                                         1.38205990158899e-08,
                                         2.19559614787384e-11,
                                         1.42475921517198e-15,
                                         1.88118484434666e-23,
                                         0.00289777111708903,
                                         0.0586997889247463,
                                         0.199585434999543,
                                         0.294689227952614,
                                         0.241580060053934,
                                         0.119166121195853,
                                         0.0357662300033102,
                                         0.00630503621956185,
                                         0.000600484804444421,
                                         2.65058774581193e-05,
                                         4.10149608378516e-07,
                                         1.2769124335103e-09,
                                         2.11158107693618e-13,
                                         1.49831086720314e-20,
                                         6.6701416351512e-05,
                                         0.00726125498439176,
                                         0.0629160753747836,
                                         0.182049823109272,
                                         0.25761726652176,
                                         0.205033677383734,
                                         0.0959685013577582,
                                         0.0261089573277321,
                                         0.00387781525540246,
                                         0.000276175909866826,
                                         7.37689821963274e-06,
                                         4.50075971099151e-08,
                                         1.89666270240054e-11,
                                         7.23250001922512e-18,
                                         1.03635800254052e-06,
                                         0.000606304244309391,
                                         0.0133874596855617,
                                         0.0759136781787164,
                                         0.185435390639631,
                                         0.238123014299286,
                                         0.17381531251969,
                                         0.0729785523154979,
                                         0.016903474416872,
                                         0.00194237542164441,
                                         8.95589626468798e-05,
                                         1.07081466786788e-06,
                                         1.14994274954879e-09,
                                         2.35656124035003e-15,
                                         1.14504358816774e-08,
                                         3.60003724444886e-05,
                                         0.00202568630759399,
                                         0.022510610429884,
                                         0.094917817523141,
                                         0.19665953431005,
                                         0.223864288350229,
                                         0.145056910715282,
                                         0.0523965040070831,
                                         0.00971444556427678,
                                         0.000773182084124437,
                                         1.8116748702009e-05,
                                         4.95792324745715e-08,
                                         5.46017398847074e-13,
                                         9.2248866836334e-11,
                                         1.55865567873967e-06,
                                         0.000223497678300473,
                                         0.00486722380848403,
                                         0.0354266160922896,
                                         0.118428287231588,
                                         0.21023664195182,
                                         0.21023664195182,
                                         0.118428287231588,
                                         0.0354266160922896,
                                         0.00486722380848404,
                                         0.000223497678300473,
                                         1.55865567873966e-06,
                                         9.2248866836334e-11,
                                         5.46017398847074e-13,
                                         4.95792324745721e-08,
                                         1.8116748702009e-05,
                                         0.000773182084124435,
                                         0.00971444556427677,
                                         0.0523965040070831,
                                         0.145056910715282,
                                         0.223864288350229,
                                         0.19665953431005,
                                         0.0949178175231411,
                                         0.022510610429884,
                                         0.00202568630759399,
                                         3.60003724444883e-05,
                                         1.14504358816774e-08,
                                         2.35656124035003e-15,
                                         1.1499427495488e-09,
                                         1.07081466786788e-06,
                                         8.95589626468796e-05,
                                         0.00194237542164441,
                                         0.016903474416872,
                                         0.0729785523154979,
                                         0.17381531251969,
                                         0.238123014299286,
                                         0.185435390639632,
                                         0.0759136781787165,
                                         0.0133874596855617,
                                         0.000606304244309388,
                                         1.03635800254052e-06,
                                         7.23250001922512e-18,
                                         1.89666270240056e-11,
                                         4.50075971099151e-08,
                                         7.37689821963271e-06,
                                         0.000276175909866826,
                                         0.00387781525540246,
                                         0.0261089573277321,
                                         0.0959685013577583,
                                         0.205033677383734,
                                         0.25761726652176,
                                         0.182049823109272,
                                         0.0629160753747836,
                                         0.00726125498439173,
                                         6.6701416351512e-05,
                                         1.49831086720314e-20,
                                         2.11158107693622e-13,
                                         1.2769124335103e-09,
                                         4.10149608378515e-07,
                                         2.65058774581192e-05,
                                         0.000600484804444421,
                                         0.00630503621956184,
                                         0.0357662300033102,
                                         0.119166121195853,
                                         0.241580060053934,
                                         0.294689227952614,
                                         0.199585434999543,
                                         0.0586997889247461,
                                         0.00289777111708903,
                                         1.88118484434666e-23,
                                         1.42475921517201e-15,
                                         2.19559614787384e-11,
                                         1.38205990158898e-08,
                                         1.54175240275819e-06,
                                         5.63550688428276e-05,
                                         0.000922787491199584,
                                         0.00807855475653259,
                                         0.0419755564122556,
                                         0.137297698656726,
                                         0.289104143723578,
                                         0.383717955268606,
                                         0.287592428015359,
                                         0.0762972980943717,
                                         1.08253627069562e-26,
                                         4.40612314892176e-18,
                                         1.7303152999683e-13,
                                         2.13448386253153e-10,
                                         4.11025272080867e-08,
                                         2.42407130653098e-06,
                                         6.19009416102239e-05,
                                         0.000836326049776105,
                                         0.00677675154328227,
                                         0.035764064884341,
                                         0.129994750668804,
                                         0.338124656815603,
                                         0.645802614054716,
                                         0.920737181470952,

                                         0.930287099955798,
                                         0.682230319012258,
                                         0.387834292278172,
                                         0.168791203667547,
                                         0.0551647886853087,
                                         0.0131631286089652,
                                         0.00220291789307073,
                                         0.000244140625,
                                         1.64791671263467e-05,
                                         5.97048774350275e-07,
                                         9.49435786196503e-09,
                                         4.68441436458606e-11,
                                         3.65321546084037e-14,
                                         9.05861199921074e-19,
                                         2.19312494389621e-27,
                                         0.0674272492712432,
                                         0.265077672656157,
                                         0.382234493636772,
                                         0.323697874271752,
                                         0.180784076389095,
                                         0.0686427374132948,
                                         0.0175795452097258,
                                         0.0029296875,
                                         0.00029736402413781,
                                         1.64868176040264e-05,
                                         4.17186141662317e-07,
                                         3.51744859255273e-09,
                                         5.33769936829189e-12,
                                         3.35723712851008e-16,
                                         4.35719927324097e-24,
                                         0.00223993455844287,
                                         0.0472059481727952,
                                         0.172661293042503,
                                         0.284519035659854,
                                         0.271543764300221,
                                         0.164063324581372,
                                         0.0642981391158833,
                                         0.01611328125,
                                         0.00245936526986914,
                                         0.000208662928826219,
                                         8.40186294753517e-06,
                                         1.21054700421972e-07,
                                         3.57449238176066e-10,
                                         5.70274326235609e-14,
                                         3.9676459754241e-21,
                                         4.5097376041396e-05,
                                         0.00509490813050763,
                                         0.0472689720203117,
                                         0.151564979282433,
                                         0.247192647692329,
                                         0.237653643558759,
                                         0.142529653138226,
                                         0.0537109375,
                                         0.0123274628415167,
                                         0.00160055211756803,
                                         0.00010255040229675,
                                         2.52494319152495e-06,
                                         1.4507437148791e-08,
                                         5.87085938417579e-12,
                                         2.18964878918928e-18,
                                         6.12873706400694e-07,
                                         0.000371175893241752,
                                         0.00873496359043378,
                                         0.0544992058328731,
                                         0.151892047790761,
                                         0.232370621207304,
                                         0.213263152784032,
                                         0.120849609375,
                                         0.0417088428588799,
                                         0.00828701480151219,
                                         0.000844895343676084,
                                         3.55488321914674e-05,
                                         3.97439284659091e-07,
                                         4.07965379433974e-10,
                                         8.15679939325431e-16,
                                         5.92281503566427e-09,
                                         1.92291741141114e-05,
                                         0.00114784568268841,
                                         0.0139353842256267,
                                         0.0663700263039079,
                                         0.161568028472435,
                                         0.226915356081631,
                                         0.193359375,
                                         0.100350616505117,
                                         0.0305115220919942,
                                         0.00495000832650632,
                                         0.000355907000041316,
                                         7.74262725134096e-06,
                                         2.01596229694906e-08,
                                         2.16074027076754e-13,
                                         4.17361282087785e-11,
                                         7.26387410171497e-07,
                                         0.000109984812321816,
                                         0.00259821166549721,
                                         0.021146366222451,
                                         0.0819136803515913,
                                         0.176051105110784,
                                         0.2255859375,
                                         0.176051105110784,
                                         0.0819136803515913,
                                         0.021146366222451,
                                         0.00259821166549721,
                                         0.000109984812321817,
                                         7.26387410171497e-07,
                                         4.17361282087808e-11,
                                         2.1607402707674e-13,
                                         2.01596229694906e-08,
                                         7.74262725134092e-06,
                                         0.000355907000041317,
                                         0.00495000832650632,
                                         0.0305115220919942,
                                         0.100350616505117,
                                         0.193359375,
                                         0.226915356081631,
                                         0.161568028472435,
                                         0.0663700263039079,
                                         0.0139353842256267,
                                         0.00114784568268841,
                                         1.92291741141114e-05,
                                         5.92281503566454e-09,
                                         8.1567993932537e-16,
                                         4.07965379433974e-10,
                                         3.97439284659089e-07,
                                         3.55488321914676e-05,
                                         0.000844895343676084,
                                         0.00828701480151219,
                                         0.0417088428588799,
                                         0.120849609375,
                                         0.213263152784032,
                                         0.232370621207304,
                                         0.151892047790761,
                                         0.054499205832873,
                                         0.0087349635904338,
                                         0.000371175893241752,
                                         6.12873706400717e-07,
                                         2.1896487891891e-18,
                                         5.87085938417579e-12,
                                         1.45074371487909e-08,
                                         2.52494319152496e-06,
                                         0.00010255040229675,
                                         0.00160055211756803,
                                         0.0123274628415167,
                                         0.0537109375,
                                         0.142529653138226,
                                         0.237653643558759,
                                         0.247192647692329,
                                         0.151564979282433,
                                         0.0472689720203118,
                                         0.00509490813050763,
                                         4.50973760413973e-05,
                                         3.96764597542373e-21,
                                         5.70274326235609e-14,
                                         3.57449238176063e-10,
                                         1.21054700421972e-07,
                                         8.40186294753517e-06,
                                         0.000208662928826219,
                                         0.00245936526986914,
                                         0.01611328125,
                                         0.0642981391158833,
                                         0.164063324581372,
                                         0.271543764300221,
                                         0.284519035659854,
                                         0.172661293042503,
                                         0.0472059481727952,
                                         0.00223993455844291,
                                         4.35719927324052e-24,
                                         3.35723712851008e-16,
                                         5.33769936829184e-12,
                                         3.51744859255275e-09,
                                         4.17186141662317e-07,
                                         1.64868176040264e-05,
                                         0.00029736402413781,
                                         0.0029296875,
                                         0.0175795452097258,
                                         0.0686427374132948,
                                         0.180784076389095,
                                         0.323697874271752,
                                         0.382234493636772,
                                         0.265077672656157,
                                         0.0674272492712438,
                                         2.19312494389596e-27,
                                         9.05861199921074e-19,
                                         3.65321546084034e-14,
                                         4.68441436458609e-11,
                                         9.49435786196503e-09,
                                         5.97048774350275e-07,
                                         1.64791671263467e-05,
                                         0.000244140625,
                                         0.00220291789307073,
                                         0.0131631286089652,
                                         0.0551647886853087,
                                         0.168791203667547,
                                         0.387834292278172,
                                         0.682230319012258,
                                         0.930287099955798,

                                         0.938226865781323,
                                         0.713734172080307,
                                         0.434059845798259,
                                         0.209009616332222,
                                         0.0785206038306271,
                                         0.0225315106370247,
                                         0.00479385038727478,
                                         0.000725552797622719,
                                         7.36802282952341e-05,
                                         4.61328385186012e-06,
                                         1.56844814355796e-07,
                                         2.36640187783445e-09,
                                         1.11949860726903e-11,
                                         8.45709527496599e-15,
                                         2.05166431111492e-19,
                                         4.90739168176271e-28,
                                         0.0599838516294157,
                                         0.24411729791866,
                                         0.375148739988819,
                                         0.349477286416394,
                                         0.222547959138442,
                                         0.100506668453518,
                                         0.0322460257937301,
                                         0.00719571182321807,
                                         0.00106981507596088,
                                         9.87599697360951e-05,
                                         5.06323505360443e-06,
                                         1.2022949089485e-07,
                                         9.64125041075756e-10,
                                         1.40906113088098e-12,
                                         8.63787136343884e-17,
                                         1.10531669641177e-24,
                                         0.00175768997806737,
                                         0.038268554795848,
                                         0.148606845658589,
                                         0.267826215867178,
                                         0.289097971692199,
                                         0.205485361201374,
                                         0.0994144150871455,
                                         0.0327084487015151,
                                         0.00711947399570066,
                                         0.000969020941212325,
                                         7.49147712360314e-05,
                                         2.79972231221866e-06,
                                         3.80561290888436e-08,
                                         1.07601887656019e-10,
                                         1.66681962004804e-14,
                                         1.14104870303573e-21,
                                         3.12152099230472e-05,
                                         0.00363581372350187,
                                         0.0356771533811619,
                                         0.124395099340598,
                                         0.227605383005031,
                                         0.254614395652201,
                                         0.185754174283035,
                                         0.0901077586832853,
                                         0.0287146256108554,
                                         0.00576237387394425,
                                         0.000671773521477553,
                                         3.95125414521071e-05,
                                         9.1039930194165e-07,
                                         4.97996168292771e-09,
                                         1.9493352280486e-12,
                                         7.13900504400468e-19,
                                         3.7419158459196e-07,
                                         0.000233165859390415,
                                         0.00578156413808406,
                                         0.0389993375579449,
                                         0.12095498905442,
                                         0.212955467064234,
                                         0.234278036364193,
                                         0.167559170050783,
                                         0.0781739724053941,
                                         0.0231298851184391,
                                         0.00406613766926086,
                                         0.000376407911778902,
                                         1.47008685185143e-05,
                                         1.55573594106716e-07,
                                         1.53882145091452e-10,
                                         3.01491427980631e-16,
                                         3.18976902918511e-09,
                                         1.06332435404005e-05,
                                         0.000666251139502963,
                                         0.00869458091671316,
                                         0.0457090829641571,
                                         0.126657845943793,
                                         0.21011740718366,
                                         0.221570367674329,
                                         0.151341721132081,
                                         0.0660211345511246,
                                         0.0175016397486842,
                                         0.00254988143396759,
                                         0.000168807421134567,
                                         3.45607559601264e-06,
                                         8.63828237546115e-09,
                                         9.05419228196642e-14,
                                         1.98267378679228e-11,
                                         3.53584621852565e-07,
                                         5.59831574122813e-05,
                                         0.00141340584854666,
                                         0.01259528610972,
                                         0.0549290652573283,
                                         0.137410303162654,
                                         0.21363970182096,
                                         0.21363970182096,
                                         0.137410303162654,
                                         0.0549290652573283,
                                         0.01259528610972,
                                         0.00141340584854665,
                                         5.59831574122816e-05,
                                         3.53584621852565e-07,
                                         1.98267378679216e-11,
                                         9.05419228196709e-14,
                                         8.63828237546114e-09,
                                         3.45607559601262e-06,
                                         0.000168807421134567,
                                         0.00254988143396759,
                                         0.0175016397486842,
                                         0.0660211345511247,
                                         0.151341721132081,
                                         0.221570367674329,
                                         0.21011740718366,
                                         0.126657845943793,
                                         0.0457090829641571,
                                         0.00869458091671314,
                                         0.000666251139502966,
                                         1.06332435404005e-05,
                                         3.18976902918494e-09,
                                         3.01491427980656e-16,
                                         1.53882145091452e-10,
                                         1.55573594106715e-07,
                                         1.47008685185143e-05,
                                         0.000376407911778902,
                                         0.00406613766926086,
                                         0.0231298851184392,
                                         0.0781739724053941,
                                         0.167559170050783,
                                         0.234278036364193,
                                         0.212955467064234,
                                         0.12095498905442,
                                         0.0389993375579448,
                                         0.00578156413808408,
                                         0.000233165859390415,
                                         3.74191584591944e-07,
                                         7.13900504400535e-19,
                                         1.9493352280486e-12,
                                         4.97996168292768e-09,
                                         9.10399301941654e-07,
                                         3.95125414521071e-05,
                                         0.000671773521477553,
                                         0.00576237387394426,
                                         0.0287146256108554,
                                         0.0901077586832852,
                                         0.185754174283035,
                                         0.254614395652201,
                                         0.227605383005031,
                                         0.124395099340598,
                                         0.0356771533811619,
                                         0.00363581372350187,
                                         3.12152099230462e-05,
                                         1.14104870303585e-21,
                                         1.66681962004804e-14,
                                         1.07601887656019e-10,
                                         3.80561290888437e-08,
                                         2.79972231221866e-06,
                                         7.49147712360314e-05,
                                         0.000969020941212326,
                                         0.00711947399570066,
                                         0.032708448701515,
                                         0.0994144150871454,
                                         0.205485361201374,
                                         0.289097971692199,
                                         0.267826215867178,
                                         0.148606845658589,
                                         0.038268554795848,
                                         0.00175768997806733,
                                         1.10531669641189e-24,
                                         8.63787136343884e-17,
                                         1.40906113088097e-12,
                                         9.6412504107576e-10,
                                         1.2022949089485e-07,
                                         5.06323505360443e-06,
                                         9.87599697360953e-05,
                                         0.00106981507596088,
                                         0.00719571182321805,
                                         0.0322460257937301,
                                         0.100506668453518,
                                         0.222547959138442,
                                         0.349477286416394,
                                         0.37514873998882,
                                         0.24411729791866,
                                         0.0599838516294151,
                                         4.90739168176333e-28,
                                         2.05166431111492e-19,
                                         8.45709527496591e-15,
                                         1.11949860726904e-11,
                                         2.36640187783445e-09,
                                         1.56844814355796e-07,
                                         4.61328385186013e-06,
                                         7.3680228295234e-05,
                                         0.000725552797622717,
                                         0.00479385038727478,
                                         0.0225315106370247,
                                         0.0785206038306271,
                                         0.209009616332222,
                                         0.434059845798259,
                                         0.713734172080307,
                                         0.938226865781323,

                                         0.944895631308325,
                                         0.741072099126538,
                                         0.476647321945339,
                                         0.249523204146821,
                                         0.105104335368624,
                                         0.0350462159905398,
                                         0.00904478400166573,
                                         0.00175196385398684,
                                         0.000244140625,
                                         2.30691299635813e-05,
                                         1.35742235736084e-06,
                                         4.37822615202276e-08,
                                         6.32356410382323e-10,
                                         2.88883890515475e-12,
                                         2.12529062628991e-15,
                                         5.06279159324218e-20,
                                         1.19881875268977e-28,
                                         0.053684126905009,
                                         0.224863607790573,
                                         0.364317612704463,
                                         0.367220189274728,
                                         0.260463567249682,
                                         0.135480607086193,
                                         0.0521121725413161,
                                         0.0146554298353415,
                                         0.0029296875,
                                         0.000397118655011974,
                                         3.39263009937695e-05,
                                         1.63089004587575e-06,
                                         3.6744984083889e-08,
                                         2.82663807639417e-10,
                                         4.00403448000652e-13,
                                         2.40266836756949e-17,
                                         3.03846310097956e-25,
                                         0.00139794276596405,
                                         0.0312722644131659,
                                         0.127627605444452,
                                         0.247698630305844,
                                         0.295838561230391,
                                         0.240046057938869,
                                         0.137613677817115,
                                         0.0561892742742277,
                                         0.01611328125,
                                         0.00313321939044786,
                                         0.000388632813928005,
                                         2.78440635765864e-05,
                                         9.78623510724368e-07,
                                         1.26764757182244e-08,
                                         3.45747249627833e-11,
                                         5.22611611270537e-15,
                                         3.52968418385116e-22,
                                         2.20622064159266e-05,
                                         0.00263581873403166,
                                         0.0270972445278723,
                                         0.101259718526665,
                                         0.203647289877962,
                                         0.25776745188081,
                                         0.220241968591752,
                                         0.130564268833192,
                                         0.0537109375,
                                         0.0149822616746127,
                                         0.00269810271690475,
                                         0.000288108855472886,
                                         1.5796078469922e-05,
                                         3.44542585716533e-07,
                                         1.80940470466642e-09,
                                         6.88938308154987e-13,
                                         2.48504230766046e-19,
                                         2.35024029994688e-07,
                                         0.000149960031903843,
                                         0.00388337573561186,
                                         0.0279417497941802,
                                         0.0946250802767406,
                                         0.186838060904885,
                                         0.237926234597866,
                                         0.204785419430864,
                                         0.120849609375,
                                         0.0483579323428026,
                                         0.0126439061674659,
                                         0.00201226127975782,
                                         0.000172102307269512,
                                         6.3210767178026e-06,
                                         6.3917012714789e-08,
                                         6.13035164145031e-11,
                                         1.18096084258849e-16,
                                         1.78038144233864e-09,
                                         6.06698571579064e-06,
                                         0.000395759327590723,
                                         0.00548286998433404,
                                         0.0312659300224259,
                                         0.0963030604123185,
                                         0.18277721151677,
                                         0.228407935010783,
                                         0.193359375,
                                         0.110992985468462,
                                         0.042134855969067,
                                         0.00999423595129439,
                                         0.00133340342831777,
                                         8.24663320730347e-05,
                                         1.60559012245647e-06,
                                         3.87907256295806e-09,
                                         3.99093517220553e-14,
                                         9.83423666741129e-12,
                                         1.78977001445339e-07,
                                         2.9408963153374e-05,
                                         0.000784493054023973,
                                         0.00753291815926542,
                                         0.0361944209639763,
                                         0.102383169542798,
                                         0.185759122100304,
                                         0.2255859375,
                                         0.185759122100304,
                                         0.102383169542798,
                                         0.0361944209639763,
                                         0.00753291815926541,
                                         0.000784493054023971,
                                         2.9408963153374e-05,
                                         1.78977001445336e-07,
                                         9.83423666741199e-12,
                                         3.9909351722052e-14,
                                         3.87907256295812e-09,
                                         1.60559012245647e-06,
                                         8.24663320730351e-05,
                                         0.00133340342831777,
                                         0.00999423595129441,
                                         0.042134855969067,
                                         0.110992985468462,
                                         0.193359375,
                                         0.228407935010783,
                                         0.18277721151677,
                                         0.0963030604123185,
                                         0.0312659300224259,
                                         0.00548286998433402,
                                         0.000395759327590723,
                                         6.06698571579057e-06,
                                         1.78038144233874e-09,
                                         1.18096084258838e-16,
                                         6.13035164145042e-11,
                                         6.3917012714789e-08,
                                         6.32107671780262e-06,
                                         0.000172102307269513,
                                         0.00201226127975783,
                                         0.0126439061674659,
                                         0.0483579323428026,
                                         0.120849609375,
                                         0.204785419430864,
                                         0.237926234597866,
                                         0.186838060904885,
                                         0.0946250802767405,
                                         0.0279417497941802,
                                         0.00388337573561186,
                                         0.000149960031903842,
                                         2.35024029994699e-07,
                                         2.4850423076602e-19,
                                         6.88938308155001e-13,
                                         1.80940470466642e-09,
                                         3.44542585716535e-07,
                                         1.5796078469922e-05,
                                         0.000288108855472887,
                                         0.00269810271690475,
                                         0.0149822616746127,
                                         0.0537109375,
                                         0.130564268833192,
                                         0.220241968591752,
                                         0.25776745188081,
                                         0.203647289877962,
                                         0.101259718526664,
                                         0.0270972445278723,
                                         0.00263581873403165,
                                         2.20622064159273e-05,
                                         3.52968418385074e-22,
                                         5.22611611270549e-15,
                                         3.45747249627833e-11,
                                         1.26764757182245e-08,
                                         9.78623510724371e-07,
                                         2.78440635765865e-05,
                                         0.000388632813928005,
                                         0.00313321939044786,
                                         0.01611328125,
                                         0.0561892742742277,
                                         0.137613677817115,
                                         0.240046057938869,
                                         0.295838561230391,
                                         0.247698630305844,
                                         0.127627605444452,
                                         0.0312722644131658,
                                         0.00139794276596408,
                                         3.03846310097917e-25,
                                         2.40266836756955e-17,
                                         4.00403448000652e-13,
                                         2.82663807639419e-10,
                                         3.67449840838891e-08,
                                         1.63089004587576e-06,
                                         3.39263009937695e-05,
                                         0.000397118655011974,
                                         0.0029296875,
                                         0.0146554298353415,
                                         0.0521121725413161,
                                         0.135480607086193,
                                         0.260463567249682,
                                         0.367220189274727,
                                         0.364317612704463,
                                         0.224863607790573,
                                         0.0536841269050096,
                                         1.1988187526896e-28,
                                         5.06279159324232e-20,
                                         2.12529062628991e-15,
                                         2.88883890515477e-12,
                                         6.32356410382325e-10,
                                         4.37822615202277e-08,
                                         1.35742235736084e-06,
                                         2.30691299635813e-05,
                                         0.000244140625,
                                         0.00175196385398684,
                                         0.00904478400166573,
                                         0.0350462159905398,
                                         0.105104335368624,
                                         0.249523204146821,
                                         0.476647321945339,
                                         0.741072099126538,
                                         0.944895631308325,

                                         0.950548578442371,
                                         0.764886807652965,
                                         0.515647448251821,
                                         0.289484985626024,
                                         0.134115918363619,
                                         0.0506255302786338,
                                         0.0153016370536536,
                                         0.00361758199787139,
                                         0.000648210599061848,
                                         8.43285191029626e-05,
                                         7.50329229312004e-06,
                                         4.19153399057962e-07,
                                         1.29355777456148e-08,
                                         1.80111469485238e-10,
                                         7.99009587767071e-13,
                                         5.74872774629418e-16,
                                         1.34859850138874e-20,
                                         3.16633627068272e-29,
                                         0.0483100465544409,
                                         0.207317298846583,
                                         0.351130879808432,
                                         0.378051633207255,
                                         0.293314361185104,
                                         0.171462381603778,
                                         0.0765106809492001,
                                         0.0259419381738443,
                                         0.00656274560096008,
                                         0.00119940961026945,
                                         0.000150671225554322,
                                         1.20712241320738e-05,
                                         5.49982734672135e-07,
                                         1.18590762681848e-08,
                                         8.8102738500887e-11,
                                         1.21567666858327e-13,
                                         7.16484490156539e-18,
                                         8.97132084132814e-26,
                                         0.00112533555014285,
                                         0.0257546324546049,
                                         0.109588913163395,
                                         0.226285974682123,
                                         0.294013837307845,
                                         0.26616415141542,
                                         0.175342696630614,
                                         0.0852644312638367,
                                         0.0304534468130297,
                                         0.00781883445917073,
                                         0.00138672460120956,
                                         0.000159334961070414,
                                         1.07175057499197e-05,
                                         3.57883600662749e-07,
                                         4.45254449759512e-09,
                                         1.17827283599072e-11,
                                         1.74466624214221e-15,
                                         1.16503020411932e-22,
                                         1.58870285975793e-05,
                                         0.00193905990760579,
                                         0.0207290916102822,
                                         0.0820881014255476,
                                         0.17861514029917,
                                         0.250406920557769,
                                         0.243539467045202,
                                         0.16984369712943,
                                         0.0856452724518921,
                                         0.0308910427245757,
                                         0.00773510432369935,
                                         0.00127463811013591,
                                         0.000126576898507756,
                                         6.54558986832192e-06,
                                         1.36377644392181e-07,
                                         6.92133237104668e-10,
                                         2.57474354488783e-13,
                                         9.1692547066382e-20,
                                         1.51393450956987e-07,
                                         9.85441550773534e-05,
                                         0.00264665720230377,
                                         0.0201004860042869,
                                         0.0732440820794242,
                                         0.159018212004131,
                                         0.228325698048243,
                                         0.228367734273614,
                                         0.162582616621867,
                                         0.0823809680044204,
                                         0.0291236567241029,
                                         0.00688282127854962,
                                         0.00100906454697027,
                                         8.08089668968681e-05,
                                         2.81956346141039e-06,
                                         2.74433622000611e-08,
                                         2.564834642422e-11,
                                         4.87118542636104e-17,
                                         1.02590929485573e-09,
                                         3.56129523015208e-06,
                                         0.000240299342802761,
                                         0.00350001874109729,
                                         0.0213581816961939,
                                         0.0718099904627506,
                                         0.152222097323034,
                                         0.218352171769721,
                                         0.219473599798581,
                                         0.156227929813653,
                                         0.0779763847623267,
                                         0.0264291715956048,
                                         0.0057203277134031,
                                         0.000709427115198586,
                                         4.14531967459882e-05,
                                         7.73788720019565e-07,
                                         1.81686319955538e-09,
                                         1.84023273173757e-14,
                                         5.06917923844075e-12,
                                         9.38451629716552e-08,
                                         1.59086848429435e-05,
                                         0.000444386634368036,
                                         0.0045413274738912,
                                         0.0236455640945735,
                                         0.0739992666271605,
                                         0.152232400462497,
                                         0.216031594983416,
                                         0.216031594983417,
                                         0.152232400462497,
                                         0.0739992666271605,
                                         0.0236455640945735,
                                         0.0045413274738912,
                                         0.000444386634368037,
                                         1.59086848429435e-05,
                                         9.38451629716552e-08,
                                         5.06917923844075e-12,
                                         1.84023273173757e-14,
                                         1.81686319955538e-09,
                                         7.73788720019565e-07,
                                         4.1453196745988e-05,
                                         0.000709427115198586,
                                         0.0057203277134031,
                                         0.0264291715956048,
                                         0.0779763847623266,
                                         0.156227929813653,
                                         0.219473599798581,
                                         0.218352171769721,
                                         0.152222097323034,
                                         0.0718099904627506,
                                         0.0213581816961939,
                                         0.0035000187410973,
                                         0.000240299342802761,
                                         3.56129523015208e-06,
                                         1.02590929485573e-09,
                                         4.87118542636104e-17,
                                         2.564834642422e-11,
                                         2.74433622000612e-08,
                                         2.81956346141037e-06,
                                         8.08089668968681e-05,
                                         0.00100906454697027,
                                         0.00688282127854962,
                                         0.0291236567241029,
                                         0.0823809680044204,
                                         0.162582616621867,
                                         0.228367734273614,
                                         0.228325698048243,
                                         0.159018212004131,
                                         0.0732440820794242,
                                         0.0201004860042869,
                                         0.00264665720230377,
                                         9.85441550773535e-05,
                                         1.51393450956987e-07,
                                         9.1692547066382e-20,
                                         2.57474354488783e-13,
                                         6.92133237104668e-10,
                                         1.3637764439218e-07,
                                         6.54558986832192e-06,
                                         0.000126576898507756,
                                         0.00127463811013591,
                                         0.00773510432369935,
                                         0.0308910427245757,
                                         0.0856452724518921,
                                         0.16984369712943,
                                         0.243539467045202,
                                         0.250406920557769,
                                         0.17861514029917,
                                         0.0820881014255477,
                                         0.0207290916102822,
                                         0.00193905990760579,
                                         1.58870285975793e-05,
                                         1.16503020411932e-22,
                                         1.74466624214221e-15,
                                         1.17827283599072e-11,
                                         4.45254449759509e-09,
                                         3.57883600662749e-07,
                                         1.07175057499197e-05,
                                         0.000159334961070414,
                                         0.00138672460120956,
                                         0.00781883445917072,
                                         0.0304534468130297,
                                         0.0852644312638367,
                                         0.175342696630614,
                                         0.26616415141542,
                                         0.294013837307845,
                                         0.226285974682123,
                                         0.109588913163395,
                                         0.0257546324546049,
                                         0.00112533555014285,
                                         8.97132084132814e-26,
                                         7.16484490156539e-18,
                                         1.21567666858327e-13,
                                         8.81027385008864e-11,
                                         1.18590762681848e-08,
                                         5.49982734672135e-07,
                                         1.20712241320738e-05,
                                         0.000150671225554322,
                                         0.00119940961026945,
                                         0.00656274560096008,
                                         0.0259419381738443,
                                         0.0765106809492001,
                                         0.171462381603778,
                                         0.293314361185104,
                                         0.378051633207255,
                                         0.351130879808432,
                                         0.207317298846583,
                                         0.0483100465544409,
                                         3.16633627068272e-29,
                                         1.34859850138874e-20,
                                         5.74872774629418e-16,
                                         7.99009587767066e-13,
                                         1.80111469485238e-10,
                                         1.29355777456148e-08,
                                         4.19153399057962e-07,
                                         7.50329229312004e-06,
                                         8.43285191029626e-05,
                                         0.000648210599061848,
                                         0.00361758199787139,
                                         0.0153016370536536,
                                         0.0506255302786338,
                                         0.134115918363619,
                                         0.289484985626024,
                                         0.515647448251821,
                                         0.764886807652965,
                                         0.950548578442371,

                                         0.955380450362278,
                                         0.785717351708002,
                                         0.551229497819682,
                                         0.328295009783194,
                                         0.164781356284102,
                                         0.0690010520375238,
                                         0.0237773963230308,
                                         0.00662131356279842,
                                         0.00145461131513173,
                                         0.000244140625,
                                         2.99754628587024e-05,
                                         2.5352074835285e-06,
                                         1.35539302750435e-07,
                                         4.02949310176605e-09,
                                         5.43903981332267e-11,
                                         2.3534991675853e-13,
                                         1.66156703503481e-16,
                                         3.847417730803e-21,
                                         8.96825666649406e-30,
                                         0.0436919977712583,
                                         0.191398930507286,
                                         0.336598735915284,
                                         0.383177213336044,
                                         0.320607614629829,
                                         0.206650526962737,
                                         0.104312724477531,
                                         0.0412459376129655,
                                         0.0126307687985249,
                                         0.0029296875,
                                         0.000497102063464002,
                                         5.86055808644844e-05,
                                         4.44892154660911e-06,
                                         1.9374571402855e-07,
                                         4.02549202093836e-09,
                                         2.90362915046349e-11,
                                         3.91832385866996e-14,
                                         2.27435718777648e-18,
                                         2.82387174844079e-26,
                                         0.00091581741745384,
                                         0.0213694877954883,
                                         0.0942050002272919,
                                         0.204982370643598,
                                         0.28590452970589,
                                         0.283660435458705,
                                         0.209745051807958,
                                         0.117760502301991,
                                         0.050268283200587,
                                         0.01611328125,
                                         0.00377838907958779,
                                         0.000620934634846871,
                                         6.69307499093689e-05,
                                         4.26967909197425e-06,
                                         1.36551710898498e-07,
                                         1.64191129310355e-09,
                                         4.23510549867028e-12,
                                         6.16210997161685e-16,
                                         4.07533534805123e-23,
                                         1.1634077452571e-05,
                                         0.00144598828511927,
                                         0.0159790694755126,
                                         0.0664583269667703,
                                         0.154519856135961,
                                         0.235981017418816,
                                         0.255600803741503,
                                         0.203767151384728,
                                         0.121247942167035,
                                         0.0537109375,
                                         0.017405393435055,
                                         0.00398720762068987,
                                         0.000610256888510478,
                                         5.70261984886139e-05,
                                         2.80731649505565e-06,
                                         5.6269659601062e-08,
                                         2.77424105102159e-10,
                                         1.01185032016633e-13,
                                         3.5644931967471e-20,
                                         9.97605363527478e-08,
                                         6.60448881592724e-05,
                                         0.00182950157596591,
                                         0.0145440737967583,
                                         0.0563704273110563,
                                         0.132513201328479,
                                         0.210250229787809,
                                         0.237997541968303,
                                         0.197405147281996,
                                         0.120849609375,
                                         0.0541208719158491,
                                         0.0172820632154387,
                                         0.00375580869141873,
                                         0.000514111573658666,
                                         3.89573477314909e-05,
                                         1.30167528435916e-06,
                                         1.22667051690287e-08,
                                         1.12151961863259e-11,
                                         2.10443735265102e-17,
                                         6.08307350972899e-10,
                                         2.14511761032625e-06,
                                         0.000148953789123138,
                                         0.00226339420416452,
                                         0.014623651318662,
                                         0.0529149862220604,
                                         0.122983884880898,
                                         0.197673400702382,
                                         0.228549372515763,
                                         0.193359375,
                                         0.119669420875588,
                                         0.0532671906794191,
                                         0.0164373460913753,
                                         0.00329592867434314,
                                         0.000384436709008597,
                                         2.14125551273283e-05,
                                         3.85699543604376e-07,
                                         8.83964738374273e-10,
                                         8.83510423858538e-15,
                                         2.70466923369066e-12,
                                         5.080305328546e-08,
                                         8.84294919329001e-06,
                                         0.000256839098215955,
                                         0.00276622260967082,
                                         0.0154072466708908,
                                         0.0524549820992064,
                                         0.119715615528088,
                                         0.192942721888559,
                                         0.2255859375,
                                         0.192942721888559,
                                         0.119715615528088,
                                         0.0524549820992064,
                                         0.0154072466708908,
                                         0.00276622260967083,
                                         0.000256839098215955,
                                         8.84294919329001e-06,
                                         5.080305328546e-08,
                                         2.70466923369066e-12,
                                         8.83510423858538e-15,
                                         8.83964738374273e-10,
                                         3.85699543604376e-07,
                                         2.14125551273283e-05,
                                         0.000384436709008595,
                                         0.00329592867434315,
                                         0.0164373460913753,
                                         0.053267190679419,
                                         0.119669420875588,
                                         0.193359375,
                                         0.228549372515763,
                                         0.197673400702382,
                                         0.122983884880898,
                                         0.0529149862220603,
                                         0.014623651318662,
                                         0.00226339420416452,
                                         0.000148953789123138,
                                         2.14511761032625e-06,
                                         6.08307350972899e-10,
                                         2.10443735265102e-17,
                                         1.12151961863259e-11,
                                         1.22667051690287e-08,
                                         1.30167528435916e-06,
                                         3.89573477314907e-05,
                                         0.000514111573658667,
                                         0.00375580869141873,
                                         0.0172820632154387,
                                         0.0541208719158491,
                                         0.120849609375,
                                         0.197405147281996,
                                         0.237997541968303,
                                         0.210250229787809,
                                         0.132513201328478,
                                         0.0563704273110564,
                                         0.0145440737967583,
                                         0.00182950157596591,
                                         6.60448881592724e-05,
                                         9.97605363527478e-08,
                                         3.5644931967471e-20,
                                         1.01185032016633e-13,
                                         2.77424105102159e-10,
                                         5.6269659601062e-08,
                                         2.80731649505564e-06,
                                         5.7026198488614e-05,
                                         0.000610256888510478,
                                         0.00398720762068986,
                                         0.017405393435055,
                                         0.0537109375,
                                         0.121247942167035,
                                         0.203767151384728,
                                         0.255600803741503,
                                         0.235981017418816,
                                         0.154519856135961,
                                         0.0664583269667703,
                                         0.0159790694755126,
                                         0.00144598828511927,
                                         1.1634077452571e-05,
                                         4.07533534805123e-23,
                                         6.16210997161685e-16,
                                         4.23510549867028e-12,
                                         1.64191129310355e-09,
                                         1.36551710898498e-07,
                                         4.26967909197426e-06,
                                         6.69307499093689e-05,
                                         0.00062093463484687,
                                         0.00377838907958779,
                                         0.01611328125,
                                         0.050268283200587,
                                         0.117760502301991,
                                         0.209745051807958,
                                         0.283660435458704,
                                         0.28590452970589,
                                         0.204982370643598,
                                         0.0942050002272919,
                                         0.0213694877954882,
                                         0.00091581741745384,
                                         2.82387174844079e-26,
                                         2.27435718777648e-18,
                                         3.91832385866995e-14,
                                         2.90362915046349e-11,
                                         4.02549202093834e-09,
                                         1.93745714028551e-07,
                                         4.44892154660911e-06,
                                         5.86055808644843e-05,
                                         0.000497102063464002,
                                         0.0029296875,
                                         0.0126307687985249,
                                         0.0412459376129655,
                                         0.104312724477531,
                                         0.206650526962737,
                                         0.320607614629829,
                                         0.383177213336044,
                                         0.336598735915284,
                                         0.191398930507286,
                                         0.0436919977712583,
                                         8.96825666649406e-30,
                                         3.847417730803e-21,
                                         1.66156703503481e-16,
                                         2.3534991675853e-13,
                                         5.43903981332265e-11,
                                         4.02949310176607e-09,
                                         1.35539302750435e-07,
                                         2.5352074835285e-06,
                                         2.99754628587024e-05,
                                         0.000244140625,
                                         0.00145461131513173,
                                         0.00662131356279842,
                                         0.0237773963230308,
                                         0.0690010520375238,
                                         0.164781356284102,
                                         0.328295009783194,
                                         0.551229497819682,
                                         0.785717351708002,
                                         0.955380450362278,

                                         0.95954180803518,
                                         0.804013537291149,
                                         0.583624679793408,
                                         0.365554621037942,
                                         0.196409060655056,
                                         0.0897864174143157,
                                         0.0345425890047475,
                                         0.0110246784183096,
                                         0.00286491369733753,
                                         0.000591475627979603,
                                         9.39146149052316e-05,
                                         1.09772387104147e-05,
                                         8.8915674456553e-07,
                                         4.57892573619003e-08,
                                         1.31852397382462e-09,
                                         1.73312621507477e-11,
                                         7.34112867984967e-14,
                                         5.09953161608075e-17,
                                         1.16770306540346e-21,
                                         2.70510070000612e-30,
                                         0.0396967643594946,
                                         0.176990665429006,
                                         0.321438540914962,
                                         0.383737643432397,
                                         0.34235905639252,
                                         0.239679931217284,
                                         0.134195161573771,
                                         0.0603158625325556,
                                         0.0216226002958255,
                                         0.00608860491715472,
                                         0.00131375606530617,
                                         0.000209439804642436,
                                         2.34031979904153e-05,
                                         1.69724336573933e-06,
                                         7.11261816239747e-08,
                                         1.43176710877852e-09,
                                         1.0070318417321e-11,
                                         1.33330184899612e-14,
                                         7.6384969827472e-19,
                                         9.41574572188409e-27,
                                         0.00075271012866027,
                                         0.0178574238366257,
                                         0.0811416231177925,
                                         0.184628173606717,
                                         0.273516522551984,
                                         0.293247381726094,
                                         0.238946279789917,
                                         0.151244154572672,
                                         0.0747972688185869,
                                         0.0287263321399884,
                                         0.00842321196474328,
                                         0.00183149941052377,
                                         0.000282327725810043,
                                         2.88340789354136e-05,
                                         1.75854060870351e-06,
                                         5.4212073043315e-08,
                                         6.33147669303299e-10,
                                         1.59774739280614e-12,
                                         2.29015483732324e-16,
                                         1.50212975818972e-23,
                                         8.65000715220301e-06,
                                         0.00109195139312536,
                                         0.0124138213047808,
                                         0.053836596786901,
                                         0.132434570562344,
                                         0.217446624710596,
                                         0.257857486364734,
                                         0.22984851928267,
                                         0.156812116227944,
                                         0.0821407410952831,
                                         0.0327308120940128,
                                         0.00970667297381096,
                                         0.00206418162979823,
                                         0.000296882160672806,
                                         2.63506519433934e-05,
                                         1.24404389412652e-06,
                                         2.41258641443654e-08,
                                         1.16038955112595e-10,
                                         4.16138441479395e-14,
                                         1.45236653650039e-20,
                                         6.7097902760759e-08,
                                         4.50704173627463e-05,
                                         0.00128194992396493,
                                         0.0105964647239823,
                                         0.0432835568343842,
                                         0.108836601253892,
                                         0.187829148248881,
                                         0.23578088572646,
                                         0.221910199722407,
                                         0.158540721018322,
                                         0.0858498627069991,
                                         0.0347246427071783,
                                         0.0101869942361691,
                                         0.00206331670341009,
                                         0.000266522636283505,
                                         1.9269886917291e-05,
                                         6.20532476907292e-07,
                                         5.68857219599811e-09,
                                         5.10404623158714e-12,
                                         9.4787002771955e-18,
                                         3.70116877674388e-10,
                                         1.32287045598286e-06,
                                         9.41399789630092e-05,
                                         0.00148313901411866,
                                         0.0100596293780675,
                                         0.0387377806928685,
                                         0.097293471137164,
                                         0.171993860218941,
                                         0.223312162579214,
                                         0.21760080898204,
                                         0.160125276846183,
                                         0.0883369983621324,
                                         0.0357504634952791,
                                         0.0101972989242255,
                                         0.0019169655171521,
                                         0.000212256057199455,
                                         1.1349680052874e-05,
                                         1.98307985859071e-07,
                                         4.45173011243344e-10,
                                         4.39904912887092e-15,
                                         1.48866041653575e-12,
                                         2.83119551673659e-08,
                                         5.04085192103124e-06,
                                         0.00015136641625584,
                                         0.00170477797646256,
                                         0.0100535931942574,
                                         0.0367477889809174,
                                         0.0914837798066015,
                                         0.163860508161686,
                                         0.217774481927082,
                                         0.217774481927082,
                                         0.163860508161686,
                                         0.0914837798066015,
                                         0.0367477889809174,
                                         0.0100535931942573,
                                         0.00170477797646256,
                                         0.00015136641625584,
                                         5.04085192103123e-06,
                                         2.83119551673665e-08,
                                         1.48866041653589e-12,
                                         4.39904912887042e-15,
                                         4.45173011243334e-10,
                                         1.98307985859071e-07,
                                         1.13496800528739e-05,
                                         0.000212256057199455,
                                         0.0019169655171521,
                                         0.0101972989242255,
                                         0.0357504634952791,
                                         0.0883369983621324,
                                         0.160125276846183,
                                         0.21760080898204,
                                         0.223312162579214,
                                         0.171993860218941,
                                         0.097293471137164,
                                         0.0387377806928685,
                                         0.0100596293780675,
                                         0.00148313901411866,
                                         9.41399789630092e-05,
                                         1.32287045598288e-06,
                                         3.70116877674418e-10,
                                         9.47870027719428e-18,
                                         5.10404623158702e-12,
                                         5.68857219599811e-09,
                                         6.20532476907289e-07,
                                         1.9269886917291e-05,
                                         0.000266522636283505,
                                         0.00206331670341009,
                                         0.0101869942361691,
                                         0.0347246427071783,
                                         0.0858498627069991,
                                         0.158540721018322,
                                         0.221910199722407,
                                         0.23578088572646,
                                         0.187829148248881,
                                         0.108836601253892,
                                         0.0432835568343842,
                                         0.0105964647239824,
                                         0.00128194992396493,
                                         4.50704173627468e-05,
                                         6.70979027607633e-08,
                                         1.45236653650018e-20,
                                         4.16138441479384e-14,
                                         1.16038955112596e-10,
                                         2.41258641443653e-08,
                                         1.24404389412652e-06,
                                         2.63506519433935e-05,
                                         0.000296882160672806,
                                         0.00206418162979823,
                                         0.00970667297381096,
                                         0.0327308120940128,
                                         0.0821407410952831,
                                         0.156812116227944,
                                         0.22984851928267,
                                         0.257857486364734,
                                         0.217446624710596,
                                         0.132434570562344,
                                         0.0538365967869011,
                                         0.0124138213047808,
                                         0.00109195139312537,
                                         8.65000715220343e-06,
                                         1.50212975818947e-23,
                                         2.29015483732317e-16,
                                         1.59774739280614e-12,
                                         6.33147669303295e-10,
                                         5.4212073043315e-08,
                                         1.75854060870351e-06,
                                         2.88340789354136e-05,
                                         0.000282327725810043,
                                         0.00183149941052377,
                                         0.00842321196474328,
                                         0.0287263321399884,
                                         0.0747972688185869,
                                         0.151244154572672,
                                         0.238946279789917,
                                         0.293247381726093,
                                         0.273516522551984,
                                         0.184628173606717,
                                         0.0811416231177925,
                                         0.0178574238366258,
                                         0.000752710128660294,
                                         9.41574572188242e-27,
                                         7.63849698274694e-19,
                                         1.33330184899612e-14,
                                         1.00703184173209e-11,
                                         1.43176710877852e-09,
                                         7.1126181623975e-08,
                                         1.69724336573933e-06,
                                         2.34031979904153e-05,
                                         0.000209439804642436,
                                         0.00131375606530617,
                                         0.00608860491715472,
                                         0.0216226002958255,
                                         0.0603158625325556,
                                         0.134195161573771,
                                         0.239679931217284,
                                         0.34235905639252,
                                         0.383737643432397,
                                         0.321438540914962,
                                         0.176990665429007,
                                         0.0396967643594952,
                                         2.70510070000559e-30,
                                         1.16770306540342e-21,
                                         5.09953161608075e-17,
                                         7.34112867984961e-14,
                                         1.73312621507477e-11,
                                         1.31852397382462e-09,
                                         4.57892573619003e-08,
                                         8.8915674456553e-07,
                                         1.09772387104147e-05,
                                         9.39146149052316e-05,
                                         0.000591475627979603,
                                         0.00286491369733753,
                                         0.0110246784183096,
                                         0.0345425890047475,
                                         0.0897864174143157,
                                         0.196409060655056,
                                         0.365554621037942,
                                         0.583624679793408,
                                         0.804013537291149,
                                         0.95954180803518,

                                         0.963150447391732,
                                         0.820150098558721,
                                         0.613090259359763,
                                         0.4010208386892,
                                         0.228414620583165,
                                         0.112538513611647,
                                         0.0475401523326278,
                                         0.0170220499131382,
                                         0.00508998684741589,
                                         0.00124698886841271,
                                         0.000244140625,
                                         3.69664374906796e-05,
                                         4.14232284047819e-06,
                                         3.23305801405112e-07,
                                         1.61216086391484e-08,
                                         4.51645908865679e-10,
                                         5.80234255457948e-12,
                                         2.41295875213741e-14,
                                         1.6528933565877e-17,
                                         3.74852230000651e-22,
                                         8.63753825713546e-31,
                                         0.0362187422341086,
                                         0.163960216806603,
                                         0.306148660608272,
                                         0.380741397878541,
                                         0.358904111944907,
                                         0.269633487891747,
                                         0.164855762397192,
                                         0.082554973474714,
                                         0.0337630635689756,
                                         0.0111610719516041,
                                         0.0029296875,
                                         0.000594739467751056,
                                         8.99251662252245e-05,
                                         9.59941145907004e-06,
                                         6.69464115749865e-07,
                                         2.71448794844879e-08,
                                         5.31754681471531e-10,
                                         3.65973155787781e-12,
                                         4.76649759997702e-15,
                                         2.70008507606069e-19,
                                         3.30760197232909e-27,
                                         0.000624243518510249,
                                         0.0150232906638887,
                                         0.0700683280707095,
                                         0.165681761270372,
                                         0.258472541013176,
                                         0.296092855256898,
                                         0.262016800257824,
                                         0.18350845051486,
                                         0.102647523568386,
                                         0.0457857868965959,
                                         0.01611328125,
                                         0.00438558329711232,
                                         0.000894746479939465,
                                         0.000130634281274936,
                                         1.27417088026414e-05,
                                         7.47754781031316e-07,
                                         2.23357593985964e-08,
                                         2.54407556874744e-10,
                                         6.2999248177044e-13,
                                         8.91407413145653e-17,
                                         5.80521012364707e-24,
                                         6.52064827389666e-06,
                                         0.000834272083829711,
                                         0.00971912575605548,
                                         0.0436953639472627,
                                         0.112814895753115,
                                         0.197059820136695,
                                         0.252388871114152,
                                         0.247220767278434,
                                         0.189134676862717,
                                         0.113833873220134,
                                         0.0537109375,
                                         0.0195994563128313,
                                         0.00539553859543524,
                                         0.00107742182578981,
                                         0.000146975214853849,
                                         1.24837923154489e-05,
                                         5.68599114768615e-07,
                                         1.0718322976723e-08,
                                         5.04646668420413e-11,
                                         1.78357382537429e-14,
                                         6.17502473349641e-21,
                                         4.59760102259549e-08,
                                         3.12718897387192e-05,
                                         0.000909989583632125,
                                         0.00777857049982359,
                                         0.0332370333861413,
                                         0.0885262380210579,
                                         0.164102439174613,
                                         0.224811024405289,
                                         0.235232660323163,
                                         0.191036396012036,
                                         0.120849609375,
                                         0.0591240883300005,
                                         0.0219620755561292,
                                         0.00599816144022298,
                                         0.00114436431001926,
                                         0.000140681711848387,
                                         9.770468941467e-06,
                                         3.04808760282579e-07,
                                         2.72862106197988e-09,
                                         2.4088497492418e-12,
                                         4.43366863762245e-18,
                                         2.30520339833518e-10,
                                         8.33562186649015e-07,
                                         6.05875101190663e-05,
                                         0.000984694898576061,
                                         0.00696330670992401,
                                         0.0282802601663678,
                                         0.0758747610008599,
                                         0.145374329189164,
                                         0.208047040907969,
                                         0.227980753239549,
                                         0.193359375,
                                         0.126830105762889,
                                         0.0635695919444679,
                                         0.023745865127231,
                                         0.00633609923220185,
                                         0.00112736933626814,
                                         0.000119388391864748,
                                         6.16404093458919e-06,
                                         1.04914739892023e-07,
                                         2.31348025981297e-10,
                                         2.26373312918397e-15,
                                         8.42779626879039e-13,
                                         1.62012562564014e-08,
                                         2.94141698835364e-06,
                                         9.0892990116323e-05,
                                         0.00106374027633345,
                                         0.00658751472384856,
                                         0.0255803476711297,
                                         0.0685463998326216,
                                         0.134169027856335,
                                         0.198384190203594,
                                         0.2255859375,
                                         0.198384190203594,
                                         0.134169027856335,
                                         0.0685463998326217,
                                         0.0255803476711297,
                                         0.00658751472384856,
                                         0.00106374027633345,
                                         9.0892990116323e-05,
                                         2.94141698835361e-06,
                                         1.62012562564017e-08,
                                         8.42779626878949e-13,
                                         2.26373312918425e-15,
                                         2.31348025981292e-10,
                                         1.04914739892024e-07,
                                         6.16404093458919e-06,
                                         0.000119388391864748,
                                         0.00112736933626814,
                                         0.00633609923220185,
                                         0.023745865127231,
                                         0.0635695919444679,
                                         0.126830105762889,
                                         0.193359375,
                                         0.227980753239549,
                                         0.208047040907969,
                                         0.145374329189164,
                                         0.0758747610008599,
                                         0.0282802601663678,
                                         0.00696330670992401,
                                         0.000984694898576061,
                                         6.05875101190658e-05,
                                         8.3356218664903e-07,
                                         2.30520339833498e-10,
                                         4.43366863762308e-18,
                                         2.40884974924174e-12,
                                         2.72862106197991e-09,
                                         3.04808760282579e-07,
                                         9.770468941467e-06,
                                         0.000140681711848387,
                                         0.00114436431001926,
                                         0.00599816144022297,
                                         0.0219620755561292,
                                         0.0591240883300005,
                                         0.120849609375,
                                         0.191036396012036,
                                         0.235232660323163,
                                         0.224811024405289,
                                         0.164102439174613,
                                         0.0885262380210579,
                                         0.0332370333861413,
                                         0.00777857049982359,
                                         0.00090998958363212,
                                         3.12718897387197e-05,
                                         4.59760102259517e-08,
                                         6.1750247334974e-21,
                                         1.78357382537424e-14,
                                         5.04646668420419e-11,
                                         1.0718322976723e-08,
                                         5.68599114768615e-07,
                                         1.24837923154489e-05,
                                         0.000146975214853849,
                                         0.00107742182578981,
                                         0.00539553859543524,
                                         0.0195994563128313,
                                         0.0537109375,
                                         0.113833873220134,
                                         0.189134676862717,
                                         0.247220767278434,
                                         0.252388871114152,
                                         0.197059820136695,
                                         0.112814895753115,
                                         0.0436953639472627,
                                         0.00971912575605544,
                                         0.00083427208382972,
                                         6.52064827389631e-06,
                                         5.80521012364811e-24,
                                         8.91407413145622e-17,
                                         6.29992481770449e-13,
                                         2.54407556874744e-10,
                                         2.23357593985964e-08,
                                         7.47754781031316e-07,
                                         1.27417088026414e-05,
                                         0.000130634281274936,
                                         0.000894746479939465,
                                         0.00438558329711232,
                                         0.01611328125,
                                         0.0457857868965959,
                                         0.102647523568386,
                                         0.18350845051486,
                                         0.262016800257824,
                                         0.296092855256898,
                                         0.258472541013176,
                                         0.165681761270372,
                                         0.0700683280707094,
                                         0.0150232906638888,
                                         0.000624243518510226,
                                         3.30760197232974e-27,
                                         2.70008507606059e-19,
                                         4.76649759997709e-15,
                                         3.65973155787781e-12,
                                         5.31754681471531e-10,
                                         2.71448794844879e-08,
                                         6.69464115749865e-07,
                                         9.59941145907002e-06,
                                         8.99251662252245e-05,
                                         0.000594739467751056,
                                         0.0029296875,
                                         0.0111610719516041,
                                         0.0337630635689756,
                                         0.082554973474714,
                                         0.164855762397192,
                                         0.269633487891747,
                                         0.358904111944907,
                                         0.380741397878541,
                                         0.306148660608272,
                                         0.163960216806604,
                                         0.036218742234108,
                                         8.6375382571373e-31,
                                         3.74852230000636e-22,
                                         1.65289335658772e-17,
                                         2.41295875213741e-14,
                                         5.80234255457948e-12,
                                         4.51645908865679e-10,
                                         1.61216086391484e-08,
                                         3.23305801405112e-07,
                                         4.14232284047819e-06,
                                         3.69664374906796e-05,
                                         0.000244140625,
                                         0.00124698886841271,
                                         0.00508998684741589,
                                         0.0170220499131382,
                                         0.0475401523326278,
                                         0.112538513611647,
                                         0.228414620583165,
                                         0.4010208386892,
                                         0.613090259359763,
                                         0.820150098558721,
                                         0.963150447391732,

                                         0.966299542147149,
                                         0.834439410185226,
                                         0.639887634618408,
                                         0.434566504511462,
                                         0.260325216017964,
                                         0.136804302807698,
                                         0.0626131803456562,
                                         0.0247265682661571,
                                         0.00832570468718782,
                                         0.00235418768629884,
                                         0.000548245828814291,
                                         0.000102540702873899,
                                         1.49025495054525e-05,
                                         1.61012304564814e-06,
                                         1.21697242953445e-07,
                                         5.9013765119248e-09,
                                         1.61432844829189e-10,
                                         2.03316134131169e-12,
                                         8.3211198053869e-15,
                                         5.63118157805298e-18,
                                         1.26641179629251e-22,
                                         2.90460102537897e-31,
                                         0.0331734708177064,
                                         0.152174213258283,
                                         0.291066985692626,
                                         0.375043194021609,
                                         0.370753663132217,
                                         0.295986576234392,
                                         0.195151086294747,
                                         0.107157235227943,
                                         0.048993532493919,
                                         0.0185270932447095,
                                         0.00572114991941348,
                                         0.00141498159170232,
                                         0.000272682245566542,
                                         3.94006881321786e-05,
                                         4.04376191507578e-06,
                                         2.72653204620028e-07,
                                         1.07443991609503e-08,
                                         2.05572549628492e-10,
                                         1.38841477657243e-12,
                                         1.78267754076543e-15,
                                         9.99979695115051e-20,
                                         1.21834374978993e-27,
                                         0.000521977153521805,
                                         0.012719462705507,
                                         0.0606825220388009,
                                         0.148350075605864,
                                         0.242011557875296,
                                         0.293511901868309,
                                         0.278777440168848,
                                         0.212843600587614,
                                         0.132141109374562,
                                         0.0668274568593678,
                                         0.0273635704308609,
                                         0.00894925191336987,
                                         0.00228683375401659,
                                         0.000441906094390736,
                                         6.15845623391264e-05,
                                         5.77363510186205e-06,
                                         3.27758385633914e-07,
                                         9.52664198611269e-09,
                                         1.0617887578927e-10,
                                         2.58658942719998e-13,
                                         3.61900230282787e-17,
                                         2.342257490287e-24,
                                         4.97769328436329e-06,
                                         0.000644336187682426,
                                         0.0076674393972107,
                                         0.0355639762147172,
                                         0.0957420714997001,
                                         0.176398737984474,
                                         0.241357242033474,
                                         0.256221606670051,
                                         0.215999725454265,
                                         0.146089376974445,
                                         0.079319187233927,
                                         0.0343035236036348,
                                         0.0116232714829257,
                                         0.00300380842456907,
                                         0.000568426343430977,
                                         7.40975954060328e-05,
                                         6.05956635013291e-06,
                                         2.675658130532e-07,
                                         4.9212233300297e-09,
                                         2.2745645114803e-11,
                                         7.93784461209639e-15,
                                         2.72907511293272e-21,
                                         3.2041183085436e-08,
                                         2.20323109827372e-05,
                                         0.000653944444477902,
                                         0.0057548846493605,
                                         0.0255666193238155,
                                         0.0715597811570907,
                                         0.141047962189974,
                                         0.208197100085478,
                                         0.238326438241461,
                                         0.215568903423087,
                                         0.155198500084613,
                                         0.0887553425175479,
                                         0.0398773180677166,
                                         0.0137821903501188,
                                         0.00354144341622115,
                                         0.000641893043976776,
                                         7.56193694005309e-05,
                                         5.07253641765349e-06,
                                         1.5396138117589e-07,
                                         1.35012131116539e-09,
                                         1.17522250537594e-12,
                                         2.1463478831712e-18,
                                         1.46664977154346e-10,
                                         5.35728865652948e-07,
                                         3.96614713723669e-05,
                                         0.000662217112982043,
                                         0.00485491079873334,
                                         0.0206433415577996,
                                         0.0586152689451399,
                                         0.120301506662749,
                                         0.18699444727462,
                                         0.226199193752945,
                                         0.21594057254024,
                                         0.163300585111075,
                                         0.0972883790467997,
                                         0.0449678083283598,
                                         0.015690033905303,
                                         0.00395420051824462,
                                         0.000671061034696989,
                                         6.83844233622675e-05,
                                         3.42521618316679e-06,
                                         5.69881804345464e-08,
                                         1.23730016419477e-10,
                                         1.20038954540881e-15,
                                         4.89520745146359e-13,
                                         9.49853983784074e-09,
                                         1.75397579528888e-06,
                                         5.55636776429752e-05,
                                         0.000672227092434062,
                                         0.00434227975557271,
                                         0.0177615766748495,
                                         0.0506867288135555,
                                         0.106982318465369,
                                         0.173070400912616,
                                         0.219082548521927,
                                         0.219082548521927,
                                         0.173070400912616,
                                         0.106982318465369,
                                         0.0506867288135555,
                                         0.0177615766748495,
                                         0.0043422797555727,
                                         0.000672227092434059,
                                         5.55636776429749e-05,
                                         1.75397579528889e-06,
                                         9.49853983784074e-09,
                                         4.89520745146359e-13,
                                         1.20038954540881e-15,
                                         1.23730016419477e-10,
                                         5.69881804345458e-08,
                                         3.42521618316681e-06,
                                         6.83844233622677e-05,
                                         0.000671061034696989,
                                         0.00395420051824462,
                                         0.015690033905303,
                                         0.0449678083283598,
                                         0.0972883790467995,
                                         0.163300585111075,
                                         0.21594057254024,
                                         0.226199193752945,
                                         0.18699444727462,
                                         0.120301506662749,
                                         0.0586152689451399,
                                         0.0206433415577996,
                                         0.00485491079873333,
                                         0.00066221711298204,
                                         3.96614713723672e-05,
                                         5.35728865652948e-07,
                                         1.46664977154346e-10,
                                         2.1463478831712e-18,
                                         1.17522250537594e-12,
                                         1.35012131116538e-09,
                                         1.53961381175891e-07,
                                         5.07253641765352e-06,
                                         7.56193694005309e-05,
                                         0.000641893043976776,
                                         0.00354144341622114,
                                         0.0137821903501188,
                                         0.0398773180677165,
                                         0.0887553425175479,
                                         0.155198500084613,
                                         0.215568903423087,
                                         0.238326438241461,
                                         0.208197100085478,
                                         0.141047962189974,
                                         0.0715597811570907,
                                         0.0255666193238155,
                                         0.00575488464936048,
                                         0.000653944444477906,
                                         2.20323109827372e-05,
                                         3.2041183085436e-08,
                                         2.72907511293272e-21,
                                         7.93784461209639e-15,
                                         2.27456451148027e-11,
                                         4.92122333002974e-09,
                                         2.67565813053201e-07,
                                         6.05956635013291e-06,
                                         7.40975954060328e-05,
                                         0.000568426343430977,
                                         0.00300380842456907,
                                         0.0116232714829256,
                                         0.0343035236036348,
                                         0.079319187233927,
                                         0.146089376974445,
                                         0.215999725454265,
                                         0.256221606670051,
                                         0.241357242033474,
                                         0.176398737984474,
                                         0.0957420714997,
                                         0.0355639762147171,
                                         0.00766743939721074,
                                         0.000644336187682426,
                                         4.97769328436329e-06,
                                         2.342257490287e-24,
                                         3.61900230282787e-17,
                                         2.58658942719995e-13,
                                         1.06178875789271e-10,
                                         9.52664198611274e-09,
                                         3.27758385633914e-07,
                                         5.77363510186205e-06,
                                         6.15845623391264e-05,
                                         0.000441906094390736,
                                         0.00228683375401659,
                                         0.00894925191336987,
                                         0.0273635704308609,
                                         0.0668274568593678,
                                         0.132141109374562,
                                         0.212843600587614,
                                         0.278777440168848,
                                         0.293511901868309,
                                         0.242011557875296,
                                         0.148350075605864,
                                         0.0606825220388011,
                                         0.012719462705507,
                                         0.000521977153521805,
                                         1.21834374978993e-27,
                                         9.99979695115051e-20,
                                         1.7826775407654e-15,
                                         1.38841477657245e-12,
                                         2.05572549628493e-10,
                                         1.07443991609503e-08,
                                         2.72653204620028e-07,
                                         4.04376191507578e-06,
                                         3.94006881321786e-05,
                                         0.000272682245566541,
                                         0.00141498159170232,
                                         0.00572114991941348,
                                         0.0185270932447095,
                                         0.048993532493919,
                                         0.107157235227943,
                                         0.195151086294747,
                                         0.295986576234392,
                                         0.370753663132217,
                                         0.375043194021609,
                                         0.291066985692627,
                                         0.152174213258283,
                                         0.0331734708177064,
                                         2.90460102537897e-31,
                                         1.26641179629251e-22,
                                         5.63118157805287e-18,
                                         8.32111980538698e-15,
                                         2.0331613413117e-12,
                                         1.61432844829189e-10,
                                         5.9013765119248e-09,
                                         1.21697242953445e-07,
                                         1.61012304564814e-06,
                                         1.49025495054525e-05,
                                         0.000102540702873899,
                                         0.000548245828814291,
                                         0.00235418768629884,
                                         0.00832570468718782,
                                         0.0247265682661571,
                                         0.0626131803456562,
                                         0.136804302807698,
                                         0.260325216017964,
                                         0.434566504511462,
                                         0.639887634618408,
                                         0.834439410185226,
                                         0.966299542147149,

                                         0.969063535145711,
                                         0.847142375486,
                                         0.664269553793788,
                                         0.46614786038552,
                                         0.291772463071178,
                                         0.162152563003335,
                                         0.0795362383434809,
                                         0.034169809894664,
                                         0.0127334537799576,
                                         0.00406597148565268,
                                         0.00109541874391661,
                                         0.000244140625,
                                         4.38859216220233e-05,
                                         6.15478651692904e-06,
                                         6.44207118563008e-07,
                                         4.73470026299182e-08,
                                         2.24073681750815e-09,
                                         6.00333527860768e-11,
                                         7.43082601471249e-13,
                                         2.99904835936854e-15,
                                         2.00810392097342e-18,
                                         4.48308963467284e-23,
                                         1.02405417088353e-31,
                                         0.0304928291978664,
                                         0.141505310004689,
                                         0.276415513188931,
                                         0.367347077433745,
                                         0.378493373323463,
                                         0.31852544324533,
                                         0.224163609058043,
                                         0.13323795837178,
                                         0.0670091441168323,
                                         0.0284020462378262,
                                         0.0100536439804421,
                                         0.0029296875,
                                         0.00068856410844236,
                                         0.000126879127745695,
                                         1.76278829030872e-05,
                                         1.74851586000911e-06,
                                         1.14486236547039e-07,
                                         4.4008318977286e-09,
                                         8.24870185043628e-11,
                                         5.48015783704409e-13,
                                         6.94913278723452e-16,
                                         3.86476937975747e-20,
                                         4.68640646609679e-28,
                                         0.000439769022121569,
                                         0.0108335429954731,
                                         0.0527183637554994,
                                         0.132681669133071,
                                         0.225036882273552,
                                         0.28677803413187,
                                         0.289565366364907,
                                         0.238119421871557,
                                         0.161623076377877,
                                         0.0909319228992269,
                                         0.0422910316380084,
                                         0.01611328125,
                                         0.0049515934847164,
                                         0.00119880575295288,
                                         0.000221083452407302,
                                         2.95956651351696e-05,
                                         2.68100208562361e-06,
                                         1.4786262234919e-07,
                                         4.19677381149272e-09,
                                         4.58969798530846e-11,
                                         1.10219003503087e-13,
                                         1.52704272550392e-17,
                                         9.82965827106328e-25,
                                         3.84386036155383e-06,
                                         0.000502671590850122,
                                         0.00609364973765889,
                                         0.0290443271135012,
                                         0.0810896041286622,
                                         0.15648174976392,
                                         0.226696167874291,
                                         0.257915667828795,
                                         0.236259196635455,
                                         0.176440863982085,
                                         0.107817464025458,
                                         0.0537109375,
                                         0.0215805068330068,
                                         0.00686473075570325,
                                         0.00168046075492834,
                                         0.000303600644661056,
                                         3.80502119460996e-05,
                                         3.01091215094535e-06,
                                         1.29408134275089e-07,
                                         2.32965282926814e-09,
                                         1.05949376620622e-11,
                                         3.65674645540619e-15,
                                         1.24954799459508e-21,
                                         2.267850092145e-08,
                                         1.57435237612043e-05,
                                         0.000475441212967138,
                                         0.00429156293040948,
                                         0.0197233386195835,
                                         0.0576348645607734,
                                         0.119796881762735,
                                         0.188566440958066,
                                         0.233119095619863,
                                         0.231092445929473,
                                         0.185538362191256,
                                         0.120849609375,
                                         0.0634866001641055,
                                         0.0265339539427049,
                                         0.00862192585976513,
                                         0.0021022339567941,
                                         0.000364519513025398,
                                         4.13848644759477e-05,
                                         2.69346515416731e-06,
                                         7.98182269845543e-08,
                                         6.87454729532119e-10,
                                         5.91074580341643e-13,
                                         1.07218872693698e-18,
                                         9.51477558806229e-11,
                                         3.50636410102429e-07,
                                         2.63787145088746e-05,
                                         0.000450927909098802,
                                         0.00341140393613543,
                                         0.0150953895439217,
                                         0.0450178047627609,
                                         0.0980366691655059,
                                         0.163570296477504,
                                         0.215233460036514,
                                         0.22704701114459,
                                         0.193359375,
                                         0.132812809892384,
                                         0.0729319621974393,
                                         0.0314570257660952,
                                         0.0103513470356593,
                                         0.00248325859411706,
                                         0.000404503659419504,
                                         3.98656281410732e-05,
                                         1.94469013604244e-06,
                                         3.17195682343993e-08,
                                         6.79402562491654e-11,
                                         6.54224789489844e-16,
                                         2.91078120838461e-13,
                                         5.69428087860403e-09,
                                         1.06717891743188e-06,
                                         3.45482101914779e-05,
                                         0.000430241865992304,
                                         0.00288290399131521,
                                         0.0123353057856359,
                                         0.0371654587445196,
                                         0.0836869690692931,
                                         0.146170802866159,
                                         0.202593107872052,
                                         0.2255859375,
                                         0.202593107872052,
                                         0.146170802866159,
                                         0.0836869690692931,
                                         0.0371654587445197,
                                         0.0123353057856358,
                                         0.00288290399131522,
                                         0.000430241865992305,
                                         3.45482101914779e-05,
                                         1.06717891743189e-06,
                                         5.69428087860403e-09,
                                         2.91078120838424e-13,
                                         6.54224789489941e-16,
                                         6.79402562491654e-11,
                                         3.17195682343989e-08,
                                         1.94469013604244e-06,
                                         3.98656281410732e-05,
                                         0.000404503659419503,
                                         0.00248325859411707,
                                         0.0103513470356593,
                                         0.0314570257660952,
                                         0.0729319621974393,
                                         0.132812809892384,
                                         0.193359375,
                                         0.22704701114459,
                                         0.215233460036514,
                                         0.163570296477504,
                                         0.098036669165506,
                                         0.0450178047627608,
                                         0.0150953895439217,
                                         0.00341140393613543,
                                         0.000450927909098802,
                                         2.63787145088749e-05,
                                         3.50636410102429e-07,
                                         9.51477558806128e-11,
                                         1.07218872693717e-18,
                                         5.91074580341643e-13,
                                         6.87454729532109e-10,
                                         7.98182269845543e-08,
                                         2.69346515416731e-06,
                                         4.13848644759476e-05,
                                         0.000364519513025399,
                                         0.00210223395679409,
                                         0.00862192585976513,
                                         0.0265339539427049,
                                         0.0634866001641055,
                                         0.120849609375,
                                         0.185538362191256,
                                         0.231092445929473,
                                         0.233119095619863,
                                         0.188566440958066,
                                         0.119796881762735,
                                         0.0576348645607735,
                                         0.0197233386195835,
                                         0.00429156293040948,
                                         0.000475441212967141,
                                         1.57435237612043e-05,
                                         2.26785009214481e-08,
                                         1.24954799459532e-21,
                                         3.65674645540619e-15,
                                         1.0594937662062e-11,
                                         2.32965282926814e-09,
                                         1.29408134275089e-07,
                                         3.01091215094534e-06,
                                         3.80502119460997e-05,
                                         0.000303600644661055,
                                         0.00168046075492834,
                                         0.00686473075570325,
                                         0.0215805068330067,
                                         0.0537109375,
                                         0.107817464025458,
                                         0.176440863982085,
                                         0.236259196635455,
                                         0.257915667828795,
                                         0.22669616787429,
                                         0.15648174976392,
                                         0.0810896041286622,
                                         0.0290443271135012,
                                         0.00609364973765892,
                                         0.000502671590850122,
                                         3.84386036155358e-06,
                                         9.82965827106537e-25,
                                         1.52704272550392e-17,
                                         1.10219003503085e-13,
                                         4.58969798530846e-11,
                                         4.19677381149272e-09,
                                         1.4786262234919e-07,
                                         2.68100208562362e-06,
                                         2.95956651351696e-05,
                                         0.000221083452407302,
                                         0.00119880575295288,
                                         0.0049515934847164,
                                         0.01611328125,
                                         0.0422910316380084,
                                         0.0909319228992269,
                                         0.161623076377877,
                                         0.238119421871557,
                                         0.289565366364907,
                                         0.28677803413187,
                                         0.225036882273552,
                                         0.132681669133071,
                                         0.0527183637554995,
                                         0.0108335429954731,
                                         0.000439769022121551,
                                         4.68640646609788e-28,
                                         3.86476937975747e-20,
                                         6.94913278723439e-16,
                                         5.48015783704409e-13,
                                         8.24870185043628e-11,
                                         4.40083189772858e-09,
                                         1.1448623654704e-07,
                                         1.7485158600091e-06,
                                         1.76278829030872e-05,
                                         0.000126879127745695,
                                         0.00068856410844236,
                                         0.0029296875,
                                         0.0100536439804421,
                                         0.0284020462378262,
                                         0.0670091441168323,
                                         0.13323795837178,
                                         0.224163609058043,
                                         0.31852544324533,
                                         0.378493373323463,
                                         0.367347077433745,
                                         0.276415513188931,
                                         0.141505310004689,
                                         0.0304928291978658,
                                         1.02405417088379e-31,
                                         4.48308963467284e-23,
                                         2.00810392097338e-18,
                                         2.99904835936854e-15,
                                         7.43082601471249e-13,
                                         6.00333527860765e-11,
                                         2.24073681750816e-09,
                                         4.73470026299181e-08,
                                         6.44207118563008e-07,
                                         6.15478651692904e-06,
                                         4.38859216220233e-05,
                                         0.000244140625,
                                         0.00109541874391661,
                                         0.00406597148565268,
                                         0.0127334537799576,
                                         0.034169809894664,
                                         0.0795362383434809,
                                         0.162152563003335,
                                         0.291772463071178,
                                         0.46614786038552,
                                         0.664269553793788,
                                         0.847142375486,
                                         0.969063535145711,

                                         0.97150245792734,
                                         0.858477523595839,
                                         0.686473186294606,
                                         0.495779312759863,
                                         0.322479710040501,
                                         0.188192793378357,
                                         0.0980443769615999,
                                         0.0453109694421675,
                                         0.0184280493362109,
                                         0.00653007589682532,
                                         0.00199113772036462,
                                         0.000514302037233644,
                                         0.000110314716886547,
                                         1.91538692005778e-05,
                                         2.6037674249007e-06,
                                         2.65056336158308e-07,
                                         1.90084662005835e-08,
                                         8.80572799405798e-10,
                                         2.31648926615385e-11,
                                         2.82392145570771e-13,
                                         1.12581663176922e-15,
                                         7.46811608812582e-19,
                                         1.6565355268724e-23,
                                         3.77044120685004e-32,
                                         0.0281214367583889,
                                         0.131835638605654,
                                         0.262333516694199,
                                         0.358221173368138,
                                         0.382718532078574,
                                         0.337263888784339,
                                         0.251217073283094,
                                         0.159936127718908,
                                         0.0873270617041032,
                                         0.0408153136630083,
                                         0.0162260252431125,
                                         0.00542855311550756,
                                         0.00150497782173531,
                                         0.000338460632681688,
                                         5.99873631290451e-05,
                                         8.05436304367766e-06,
                                         7.75472397985043e-07,
                                         4.94881589521934e-08,
                                         1.86134093945178e-09,
                                         3.42639957586012e-11,
                                         2.24371298511359e-13,
                                         2.8141247737544e-16,
                                         1.55331432830608e-20,
                                         1.87569000162096e-28,
                                         0.000373089091194703,
                                         0.00927936309810396,
                                         0.0459478746406397,
                                         0.118630277559386,
                                         0.208179534581908,
                                         0.277024475582545,
                                         0.295023802193696,
                                         0.25874456920603,
                                         0.189670492777368,
                                         0.116925568790199,
                                         0.0606044393922564,
                                         0.0262622167794605,
                                         0.00941040226629124,
                                         0.00274120332518803,
                                         0.000633430247846296,
                                         0.000112177473723731,
                                         1.44999684408176e-05,
                                         1.27473166043322e-06,
                                         6.85492978147697e-08,
                                         1.90548197800992e-09,
                                         2.04950218625658e-11,
                                         4.86023291309861e-14,
                                         6.67573956740517e-18,
                                         4.27672570671562e-25,
                                         2.99987840645545e-06,
                                         0.000395839898174916,
                                         0.00487745340958926,
                                         0.0238098081873862,
                                         0.0686297836932836,
                                         0.137905788358767,
                                         0.209981491241174,
                                         0.253695033433399,
                                         0.249670225294215,
                                         0.203007414410917,
                                         0.137186945009978,
                                         0.0770007096981354,
                                         0.0356617238793596,
                                         0.0134552073183484,
                                         0.004053721236037,
                                         0.00094688266379414,
                                         0.000164317502569272,
                                         1.98999645696153e-05,
                                         1.53001661534366e-06,
                                         6.42225786578722e-08,
                                         1.134607176732e-09,
                                         5.08729439689244e-12,
                                         1.73882341151749e-15,
                                         5.90986807914915e-22,
                                         1.6281654166496e-08,
                                         1.1397897220761e-05,
                                         0.000349481832738115,
                                         0.00322567062868517,
                                         0.0152718271418522,
                                         0.0463394230254604,
                                         0.10088085360167,
                                         0.167902228347175,
                                         0.221838799635279,
                                         0.237912949616121,
                                         0.209616229391495,
                                         0.152391887062299,
                                         0.0912221387556589,
                                         0.0445803332293635,
                                         0.0175110720444348,
                                         0.0053949875650734,
                                         0.00125691053663626,
                                         0.000209695739472509,
                                         2.30511744036892e-05,
                                         1.46108137915721e-06,
                                         4.23981043559697e-08,
                                         3.59435055928286e-10,
                                         3.05714012281515e-13,
                                         5.51249170462787e-19,
                                         6.28392313684118e-11,
                                         2.33382007497273e-07,
                                         1.78071142050556e-05,
                                         0.00031075749169508,
                                         0.00241661147824498,
                                         0.0110727683265876,
                                         0.0344646558051806,
                                         0.0790202536466659,
                                         0.140166982702449,
                                         0.198272155209188,
                                         0.227758444811785,
                                         0.214469940073832,
                                         0.165934095411428,
                                         0.105034915796652,
                                         0.0537909301454317,
                                         0.0218585925769757,
                                         0.0068369495123043,
                                         0.00157131904651394,
                                         0.000246960462955324,
                                         2.36373364550573e-05,
                                         1.12663909442275e-06,
                                         1.80589068426837e-08,
                                         3.82219355144079e-11,
                                         3.65641579049299e-16,
                                         1.76843869730549e-13,
                                         3.48447066758914e-09,
                                         6.61590545825835e-07,
                                         2.18298123193904e-05,
                                         0.000278836405194728,
                                         0.00192925045616761,
                                         0.00858550706263687,
                                         0.0271173462048408,
                                         0.064577428851428,
                                         0.120484777609437,
                                         0.180447504259575,
                                         0.220088738382172,
                                         0.220088738382172,
                                         0.180447504259575,
                                         0.120484777609437,
                                         0.064577428851428,
                                         0.0271173462048408,
                                         0.00858550706263687,
                                         0.0019292504561676,
                                         0.000278836405194728,
                                         2.18298123193904e-05,
                                         6.61590545825827e-07,
                                         3.48447066758914e-09,
                                         1.76843869730524e-13,
                                         3.65641579049358e-16,
                                         3.82219355144079e-11,
                                         1.80589068426839e-08,
                                         1.12663909442275e-06,
                                         2.36373364550573e-05,
                                         0.000246960462955325,
                                         0.00157131904651394,
                                         0.00683694951230432,
                                         0.0218585925769757,
                                         0.0537909301454317,
                                         0.105034915796652,
                                         0.165934095411428,
                                         0.214469940073832,
                                         0.227758444811785,
                                         0.198272155209188,
                                         0.140166982702449,
                                         0.0790202536466658,
                                         0.0344646558051806,
                                         0.0110727683265875,
                                         0.00241661147824498,
                                         0.00031075749169508,
                                         1.78071142050555e-05,
                                         2.33382007497273e-07,
                                         6.28392313684046e-11,
                                         5.51249170462889e-19,
                                         3.05714012281515e-13,
                                         3.59435055928291e-10,
                                         4.23981043559697e-08,
                                         1.46108137915721e-06,
                                         2.30511744036892e-05,
                                         0.000209695739472509,
                                         0.00125691053663626,
                                         0.0053949875650734,
                                         0.0175110720444348,
                                         0.0445803332293635,
                                         0.0912221387556588,
                                         0.152391887062299,
                                         0.209616229391495,
                                         0.237912949616121,
                                         0.221838799635279,
                                         0.167902228347174,
                                         0.10088085360167,
                                         0.0463394230254603,
                                         0.0152718271418522,
                                         0.00322567062868517,
                                         0.000349481832738112,
                                         1.1397897220761e-05,
                                         1.62816541664945e-08,
                                         5.90986807915038e-22,
                                         1.73882341151749e-15,
                                         5.08729439689253e-12,
                                         1.134607176732e-09,
                                         6.42225786578722e-08,
                                         1.53001661534367e-06,
                                         1.98999645696153e-05,
                                         0.000164317502569273,
                                         0.000946882663794141,
                                         0.004053721236037,
                                         0.0134552073183484,
                                         0.0356617238793595,
                                         0.0770007096981355,
                                         0.137186945009978,
                                         0.203007414410917,
                                         0.249670225294215,
                                         0.253695033433399,
                                         0.209981491241174,
                                         0.137905788358767,
                                         0.0686297836932836,
                                         0.0238098081873862,
                                         0.00487745340958924,
                                         0.000395839898174916,
                                         2.99987840645524e-06,
                                         4.27672570671661e-25,
                                         6.67573956740517e-18,
                                         4.8602329130987e-14,
                                         2.04950218625658e-11,
                                         1.90548197800992e-09,
                                         6.854929781477e-08,
                                         1.27473166043322e-06,
                                         1.44999684408177e-05,
                                         0.000112177473723731,
                                         0.000633430247846296,
                                         0.00274120332518803,
                                         0.00941040226629123,
                                         0.0262622167794605,
                                         0.0606044393922564,
                                         0.116925568790199,
                                         0.189670492777368,
                                         0.258744569206029,
                                         0.295023802193696,
                                         0.277024475582545,
                                         0.208179534581908,
                                         0.118630277559386,
                                         0.0459478746406396,
                                         0.00927936309810396,
                                         0.000373089091194685,
                                         1.87569000162143e-28,
                                         1.55331432830608e-20,
                                         2.81412477375446e-16,
                                         2.24371298511359e-13,
                                         3.42639957586012e-11,
                                         1.86134093945178e-09,
                                         4.94881589521934e-08,
                                         7.75472397985045e-07,
                                         8.05436304367766e-06,
                                         5.99873631290451e-05,
                                         0.000338460632681688,
                                         0.0015049778217353,
                                         0.00542855311550756,
                                         0.0162260252431125,
                                         0.0408153136630083,
                                         0.0873270617041032,
                                         0.159936127718908,
                                         0.251217073283094,
                                         0.337263888784338,
                                         0.382718532078574,
                                         0.358221173368138,
                                         0.262333516694199,
                                         0.131835638605654,
                                         0.0281214367583883,
                                         3.77044120685109e-32,
                                         1.6565355268724e-23,
                                         7.46811608812598e-19,
                                         1.12581663176922e-15,
                                         2.82392145570771e-13,
                                         2.31648926615386e-11,
                                         8.80572799405798e-10,
                                         1.90084662005835e-08,
                                         2.65056336158308e-07,
                                         2.6037674249007e-06,
                                         1.91538692005778e-05,
                                         0.000110314716886547,
                                         0.000514302037233644,
                                         0.00199113772036462,
                                         0.00653007589682532,
                                         0.0184280493362109,
                                         0.0453109694421675,
                                         0.0980443769615999,
                                         0.188192793378357,
                                         0.322479710040501,
                                         0.495779312759863,
                                         0.686473186294606,
                                         0.858477523595839,
                                         0.97150245792734,

                                         0.0001220703125,
                                         0.0015869140625,
                                         0.009521484375,
                                         0.034912109375,
                                         0.0872802734375,
                                         0.1571044921875,
                                         0.20947265625,
                                         0.20947265625,
                                         0.1571044921875,
                                         0.0872802734375,
                                         0.034912109375,
                                         0.009521484375,
                                         0.0015869140625,
                                         0.0001220703125,

                                         0.0456747239752542,
                                         1.67632342360249e-09,
                                         0.159100570247889,
                                         8.13296144284301e-08,
                                         0.255785215879518,
                                         1.82115751924133e-06,
                                         0.251303954112703,
                                         2.49210587647807e-05,
                                         0.168341728898117,
                                         0.000232516643721428,
                                         0.0811926545592767,
                                         0.00156197507035277,
                                         0.0290073416273297,
                                         0.00777249376361669,
                                         0.00777249376361669,
                                         0.0290073416273297,
                                         0.00156197507035277,
                                         0.0811926545592767,
                                         0.000232516643721428,
                                         0.168341728898117,
                                         2.49210587647807e-05,
                                         0.251303954112703,
                                         1.82115751924133e-06,
                                         0.255785215879518,
                                         8.13296144284301e-08,
                                         0.159100570247889,
                                         1.67632342360249e-09,
                                         0.0456747239752542,

                                         0.211302999999526,
                                         0.0001220703125,
                                         4.73254047506313e-13,
                                         0.348906999951563,
                                         0.0015869140625,
                                         4.84369760490518e-11,
                                         0.265901997711939,
                                         0.009521484375,
                                         2.28806103464899e-09,
                                         0.123837933949157,
                                         0.034912109375,
                                         6.60508435432911e-08,
                                         0.0393236999570222,
                                         0.0872802734375,
                                         1.3000429780482e-06,
                                         0.00899057660991212,
                                         0.1571044921875,
                                         1.84233900879493e-05,
                                         0.00152260394220921,
                                         0.20947265625,
                                         0.000193396057790809,
                                         0.000193396057790809,
                                         0.20947265625,
                                         0.00152260394220921,
                                         1.84233900879493e-05,
                                         0.1571044921875,
                                         0.00899057660991212,
                                         1.3000429780482e-06,
                                         0.0872802734375,
                                         0.0393236999570222,
                                         6.60508435432911e-08,
                                         0.034912109375,
                                         0.123837933949157,
                                         2.28806103464899e-09,
                                         0.009521484375,
                                         0.265901997711939,
                                         4.84369760490518e-11,
                                         0.0015869140625,
                                         0.348906999951563,
                                         4.73254047506313e-13,
                                         0.0001220703125,
                                         0.211302999999526,

                                         0.392397677872749,
                                         0.00548141350407144,
                                         5.50609087193674e-07,
                                         8.71491410666274e-16,
                                         0.380610087995483,
                                         0.0350989134461618,
                                         1.45321198993157e-05,
                                         1.51843410378045e-13,
                                         0.170389198258014,
                                         0.103729593795142,
                                         0.000177020053732552,
                                         1.22105910324684e-11,
                                         0.0466148191760909,
                                         0.187340580752205,
                                         0.00131775946449975,
                                         6.00064040719044e-10,
                                         0.0086950989092573,
                                         0.230690454902781,
                                         0.00668834086230226,
                                         2.01060656728213e-08,
                                         0.00116777062299521,
                                         0.204531350090257,
                                         0.0244417979007066,
                                         4.85052880315713e-07,
                                         0.000116173425815927,
                                         0.134324853710468,
                                         0.066162838788686,
                                         8.66796822084312e-06,
                                         8.66796822084306e-06,
                                         0.066162838788686,
                                         0.134324853710468,
                                         0.000116173425815928,
                                         4.8505288031571e-07,
                                         0.0244417979007066,
                                         0.204531350090257,
                                         0.00116777062299521,
                                         2.01060656728211e-08,
                                         0.00668834086230226,
                                         0.230690454902781,
                                         0.00869509890925733,
                                         6.00064040719039e-10,
                                         0.00131775946449975,
                                         0.187340580752205,
                                         0.046614819176091,
                                         1.22105910324683e-11,
                                         0.000177020053732552,
                                         0.103729593795142,
                                         0.170389198258015,
                                         1.51843410378044e-13,
                                         1.45321198993157e-05,
                                         0.0350989134461618,
                                         0.380610087995483,
                                         8.71491410666265e-16,
                                         5.50609087193674e-07,
                                         0.00548141350407144,
                                         0.392397677872749,

                                         0.535476493763929,
                                         0.0330190778514469,
                                         0.0001220703125,
                                         5.26280965708492e-09,
                                         5.32672037622316e-18,
                                         0.342622619909938,
                                         0.128771584746317,
                                         0.0015869140625,
                                         2.28060077380099e-07,
                                         1.40692511816918e-15,
                                         0.101181135830401,
                                         0.231783778680654,
                                         0.009521484375,
                                         4.56130139458269e-06,
                                         1.71510204720769e-13,
                                         0.0182601041014541,
                                         0.254956575421904,
                                         0.034912109375,
                                         5.57504596458223e-05,
                                         1.27770068443073e-11,
                                         0.0022468574825515,
                                         0.191213245812945,
                                         0.0872802734375,
                                         0.000464597333760313,
                                         6.48988300587217e-10,
                                         0.000199058304025598,
                                         0.103252892481587,
                                         0.1571044921875,
                                         0.0027876450241684,
                                         2.37343370000974e-08,
                                         1.30632519910861e-05,
                                         0.0413002529094647,
                                         0.20947265625,
                                         0.0123898046538254,
                                         6.42959433737083e-07,
                                         6.42959433737078e-07,
                                         0.0123898046538254,
                                         0.20947265625,
                                         0.0413002529094647,
                                         1.30632519910862e-05,
                                         2.37343370000971e-08,
                                         0.0027876450241684,
                                         0.1571044921875,
                                         0.103252892481587,
                                         0.000199058304025599,
                                         6.4898830058721e-10,
                                         0.000464597333760313,
                                         0.0872802734375,
                                         0.191213245812945,
                                         0.00224685748255151,
                                         1.27770068443072e-11,
                                         5.57504596458224e-05,
                                         0.034912109375,
                                         0.254956575421904,
                                         0.0182601041014542,
                                         1.71510204720766e-13,
                                         4.56130139458269e-06,
                                         0.009521484375,
                                         0.231783778680654,
                                         0.101181135830402,
                                         1.40692511816916e-15,
                                         2.28060077380099e-07,
                                         0.0015869140625,
                                         0.128771584746317,
                                         0.342622619909938,
                                         5.32672037622308e-18,
                                         5.26280965708492e-09,
                                         0.0001220703125,
                                         0.0330190778514469,
                                         0.535476493763929,

                                         0.6398446229377,
                                         0.0895626529142541,
                                         0.00197152139358657,
                                         3.52724097392662e-06,
                                         9.45622632144424e-11,
                                         7.41495325484254e-20,
                                         0.290673272075349,
                                         0.237452799695512,
                                         0.0157546577206278,
                                         7.45957967905689e-05,
                                         6.02773591937748e-09,
                                         2.7584463859079e-17,
                                         0.0609457705608415,
                                         0.290559806666898,
                                         0.0581064507304015,
                                         0.000728117520828873,
                                         1.7733663959517e-07,
                                         4.7361840962024e-15,
                                         0.007809122316094,
                                         0.217277102589605,
                                         0.130966404559779,
                                         0.00434318957518226,
                                         3.18832740480281e-06,
                                         4.96950100818583e-13,
                                         0.000682227869284508,
                                         0.110779898507784,
                                         0.201263045211669,
                                         0.0176638215669731,
                                         3.90837231051168e-05,
                                         3.55521254060384e-11,
                                         4.29130303877893e-05,
                                         0.0406668431801296,
                                         0.222689977043369,
                                         0.0517241125682329,
                                         0.000344954202259344,
                                         1.83126355249626e-09,
                                         1.99947115529514e-06,
                                         0.0110582445646493,
                                         0.182517088789877,
                                         0.112193490281708,
                                         0.00225524216946129,
                                         6.98718698853798e-08,
                                         6.98718698853798e-08,
                                         0.00225524216946129,
                                         0.112193490281708,
                                         0.182517088789877,
                                         0.0110582445646493,
                                         1.99947115529514e-06,
                                         1.83126355249626e-09,
                                         0.000344954202259345,
                                         0.0517241125682329,
                                         0.222689977043369,
                                         0.0406668431801295,
                                         4.29130303877893e-05,
                                         3.55521254060384e-11,
                                         3.9083723105117e-05,
                                         0.0176638215669731,
                                         0.20126304521167,
                                         0.110779898507784,
                                         0.000682227869284508,
                                         4.96950100818583e-13,
                                         3.18832740480282e-06,
                                         0.00434318957518226,
                                         0.130966404559779,
                                         0.217277102589605,
                                         0.007809122316094,
                                         4.7361840962024e-15,
                                         1.77336639595171e-07,
                                         0.000728117520828873,
                                         0.0581064507304015,
                                         0.290559806666897,
                                         0.0609457705608415,
                                         2.7584463859079e-17,
                                         6.02773591937751e-09,
                                         7.45957967905689e-05,
                                         0.0157546577206278,
                                         0.237452799695512,
                                         0.290673272075349,
                                         7.41495325484254e-20,
                                         9.45622632144428e-11,
                                         3.52724097392662e-06,
                                         0.00197152139358657,
                                         0.0895626529142541,
                                         0.6398446229377,

                                         0.715280778882588,
                                         0.165468900620599,
                                         0.0102281571590821,
                                         0.0001220703125,
                                         1.40380068025201e-07,
                                         2.8048911726315e-12,
                                         1.8752687713594e-21,
                                         0.242791953325368,
                                         0.319254206829006,
                                         0.0561956763969701,
                                         0.0015869140625,
                                         4.31803981694335e-06,
                                         2.45687167284827e-10,
                                         9.33668037364837e-19,
                                         0.0380364451640523,
                                         0.284292088527023,
                                         0.142500474836108,
                                         0.009521484375,
                                         6.13021540177001e-05,
                                         9.9324600854539e-09,
                                         2.14550415544257e-16,
                                         0.0036415454443213,
                                         0.154708116703898,
                                         0.220825844556059,
                                         0.034912109375,
                                         0.000531844909981632,
                                         2.45386894243807e-07,
                                         3.01291173122503e-14,
                                         0.00023770598947684,
                                         0.0574023936999505,
                                         0.233320053271184,
                                         0.0872802734375,
                                         0.00314602998576785,
                                         4.13346702825639e-06,
                                         2.88477559324752e-12,
                                         1.11718992866943e-05,
                                         0.0153348454388417,
                                         0.177495247917627,
                                         0.1571044921875,
                                         0.013399025222632,
                                         5.01314294023754e-05,
                                         1.98870404955718e-10,
                                         3.88937779651843e-07,
                                         0.00303455514883954,
                                         0.100020178423779,
                                         0.20947265625,
                                         0.0422717067469068,
                                         0.000450372567565513,
                                         1.01553410408475e-08,
                                         1.01553410408473e-08,
                                         0.000450372567565513,
                                         0.0422717067469068,
                                         0.20947265625,
                                         0.100020178423779,
                                         0.00303455514883954,
                                         3.88937779651849e-07,
                                         1.98870404955715e-10,
                                         5.01314294023754e-05,
                                         0.013399025222632,
                                         0.1571044921875,
                                         0.177495247917627,
                                         0.0153348454388417,
                                         1.11718992866944e-05,
                                         2.88477559324747e-12,
                                         4.13346702825639e-06,
                                         0.00314602998576785,
                                         0.0872802734375,
                                         0.233320053271184,
                                         0.0574023936999505,
                                         0.000237705989476842,
                                         3.01291173122496e-14,
                                         2.45386894243807e-07,
                                         0.000531844909981632,
                                         0.034912109375,
                                         0.220825844556059,
                                         0.154708116703898,
                                         0.00364154544432132,
                                         2.14550415544252e-16,
                                         9.9324600854539e-09,
                                         6.13021540177001e-05,
                                         0.009521484375,
                                         0.142500474836108,
                                         0.284292088527023,
                                         0.0380364451640525,
                                         9.33668037364812e-19,
                                         2.45687167284827e-10,
                                         4.31803981694335e-06,
                                         0.0015869140625,
                                         0.0561956763969701,
                                         0.319254206829006,
                                         0.242791953325369,
                                         1.87526877135934e-21,
                                         2.8048911726315e-12,
                                         1.40380068025201e-07,
                                         0.0001220703125,
                                         0.0102281571590821,
                                         0.165468900620599,
                                         0.715280778882588,

                                         0.770502160898972,
                                         0.248134469172661,
                                         0.0295861707196478,
                                         0.00109014852737905,
                                         8.75976432229061e-06,
                                         7.5386786726566e-09,
                                         1.23972927001293e-13,
                                         7.45296344007225e-23,
                                         0.20290762950129,
                                         0.365066489658058,
                                         0.119623698184838,
                                         0.00977857785326746,
                                         0.000165039956728181,
                                         3.15103677226319e-07,
                                         1.42406131060898e-11,
                                         4.78289866189183e-20,
                                         0.0246621418517282,
                                         0.247893272079987,
                                         0.223230528931612,
                                         0.0404830869432664,
                                         0.00143513804787198,
                                         6.07882454531143e-06,
                                         7.54985165065255e-10,
                                         1.41664575918449e-17,
                                         0.00183182248557201,
                                         0.102867382748839,
                                         0.254571705474733,
                                         0.102421639785408,
                                         0.00762637990826454,
                                         7.16648053317709e-05,
                                         2.44606688842998e-08,
                                         2.56419785454231e-15,
                                         9.27693595056825e-05,
                                         0.0291044380753294,
                                         0.197940769424085,
                                         0.176676345072983,
                                         0.0276319650880639,
                                         0.000576050848724365,
                                         5.4033962576028e-07,
                                         3.1645384535063e-13,
                                         3.38265905906988e-06,
                                         0.00592888796227275,
                                         0.110813551142835,
                                         0.219430799009843,
                                         0.0720837884743204,
                                         0.00333386655725197,
                                         8.59404855338991e-06,
                                         2.81191195769307e-11,
                                         9.13646358371213e-08,
                                         0.000894650687477093,
                                         0.0459533019139972,
                                         0.201875211250174,
                                         0.139293120318108,
                                         0.0142922905300435,
                                         0.000101249997178613,
                                         1.85079914999143e-09,
                                         1.85079914999143e-09,
                                         0.000101249997178613,
                                         0.0142922905300435,
                                         0.139293120318109,
                                         0.201875211250174,
                                         0.0459533019139971,
                                         0.000894650687477093,
                                         9.13646358371212e-08,
                                         2.81191195769307e-11,
                                         8.59404855338991e-06,
                                         0.00333386655725197,
                                         0.0720837884743205,
                                         0.219430799009843,
                                         0.110813551142835,
                                         0.00592888796227275,
                                         3.38265905906988e-06,
                                         3.1645384535063e-13,
                                         5.4033962576028e-07,
                                         0.000576050848724365,
                                         0.027631965088064,
                                         0.176676345072983,
                                         0.197940769424085,
                                         0.0291044380753294,
                                         9.27693595056825e-05,
                                         2.56419785454231e-15,
                                         2.44606688842998e-08,
                                         7.16648053317709e-05,
                                         0.00762637990826455,
                                         0.102421639785408,
                                         0.254571705474733,
                                         0.102867382748839,
                                         0.00183182248557201,
                                         1.41664575918449e-17,
                                         7.54985165065255e-10,
                                         6.07882454531143e-06,
                                         0.00143513804787198,
                                         0.0404830869432664,
                                         0.223230528931612,
                                         0.247893272079987,
                                         0.0246621418517282,
                                         4.78289866189183e-20,
                                         1.42406131060898e-11,
                                         3.15103677226319e-07,
                                         0.000165039956728182,
                                         0.00977857785326746,
                                         0.119623698184838,
                                         0.365066489658058,
                                         0.20290762950129,
                                         7.45296344007225e-23,
                                         1.23972927001293e-13,
                                         7.5386786726566e-09,
                                         8.75976432229062e-06,
                                         0.00109014852737905,
                                         0.0295861707196478,
                                         0.248134469172661,
                                         0.770502160898972,

                                         0.811700770887099,
                                         0.328889881240803,
                                         0.0612571479534902,
                                         0.00470147804786693,
                                         0.0001220703125,
                                         7.47825312097351e-07,
                                         5.26539889579547e-10,
                                         7.55977875097312e-15,
                                         4.21907606112476e-24,
                                         0.170705946106715,
                                         0.381834611899172,
                                         0.190835735225588,
                                         0.0311882146689779,
                                         0.0015869140625,
                                         1.90515696732584e-05,
                                         2.85637387983601e-08,
                                         1.10045175901909e-12,
                                         3.39040335395706e-21,
                                         0.0165694936709675,
                                         0.204601103699054,
                                         0.274391421854899,
                                         0.0954892572082059,
                                         0.009521484375,
                                         0.000224010958262878,
                                         7.15165780588183e-07,
                                         7.39334258613158e-11,
                                         1.25745740055107e-18,
                                         0.00098285625548633,
                                         0.0669978446213856,
                                         0.241102402453337,
                                         0.178664675687558,
                                         0.034912109375,
                                         0.00160963699260952,
                                         1.09425509942354e-05,
                                         3.03550415962771e-09,
                                         2.85006899382423e-16,
                                         3.9750203190957e-05,
                                         0.0149583010198371,
                                         0.144444537915465,
                                         0.227924732571601,
                                         0.0872802734375,
                                         0.00788597150072706,
                                         0.000114156070374496,
                                         8.49746554457464e-08,
                                         4.40439279909301e-14,
                                         1.15750052393692e-06,
                                         0.00240456323451069,
                                         0.0623064785391685,
                                         0.209351804618906,
                                         0.1571044921875,
                                         0.0278172989891412,
                                         0.00085745618689396,
                                         1.71269680937825e-06,
                                         4.90059949388267e-12,
                                         2.49671672811687e-08,
                                         0.000286323087048988,
                                         0.0199081782349791,
                                         0.14243876363366,
                                         0.20947265625,
                                         0.0726843557274982,
                                         0.00477079875875037,
                                         2.55704161780793e-05,
                                         4.03904423251809e-10,
                                         4.03904423251799e-10,
                                         2.55704161780793e-05,
                                         0.00477079875875035,
                                         0.0726843557274982,
                                         0.20947265625,
                                         0.14243876363366,
                                         0.0199081782349791,
                                         0.000286323087048988,
                                         2.49671672811693e-08,
                                         4.90059949388254e-12,
                                         1.71269680937825e-06,
                                         0.000857456186893958,
                                         0.0278172989891412,
                                         0.1571044921875,
                                         0.209351804618906,
                                         0.0623064785391686,
                                         0.00240456323451069,
                                         1.15750052393694e-06,
                                         4.40439279909288e-14,
                                         8.49746554457464e-08,
                                         0.000114156070374496,
                                         0.00788597150072706,
                                         0.0872802734375,
                                         0.227924732571601,
                                         0.144444537915465,
                                         0.0149583010198371,
                                         3.97502031909576e-05,
                                         2.85006899382413e-16,
                                         3.03550415962771e-09,
                                         1.09425509942354e-05,
                                         0.00160963699260952,
                                         0.034912109375,
                                         0.178664675687558,
                                         0.241102402453337,
                                         0.0669978446213856,
                                         0.00098285625548634,
                                         1.25745740055102e-18,
                                         7.39334258613158e-11,
                                         7.15165780588181e-07,
                                         0.000224010958262878,
                                         0.009521484375,
                                         0.0954892572082059,
                                         0.274391421854899,
                                         0.204601103699054,
                                         0.0165694936709676,
                                         3.39040335395692e-21,
                                         1.10045175901909e-12,
                                         2.856373879836e-08,
                                         1.90515696732584e-05,
                                         0.0015869140625,
                                         0.0311882146689779,
                                         0.190835735225588,
                                         0.381834611899172,
                                         0.170705946106715,
                                         4.21907606112457e-24,
                                         7.55977875097312e-15,
                                         5.26539889579545e-10,
                                         7.47825312097351e-07,
                                         0.0001220703125,
                                         0.00470147804786693,
                                         0.0612571479534902,
                                         0.328889881240803,
                                         0.811700770887099,

                                         0.843054480857821,
                                         0.40329863308361,
                                         0.103192030510352,
                                         0.0131634011547062,
                                         0.000741571339035607,
                                         1.50150905144256e-05,
                                         7.57271065652373e-08,
                                         4.61282828946623e-11,
                                         6.00203287034957e-16,
                                         3.17339590680732e-25,
                                         0.144878610298924,
                                         0.379320558008714,
                                         0.256084588917743,
                                         0.0676434212070652,
                                         0.00714196050104975,
                                         0.000263481514815696,
                                         2.49046601505986e-06,
                                         3.14135275425762e-09,
                                         1.07846348057728e-13,
                                         3.12077408817781e-22,
                                         0.0114910787264479,
                                         0.164662198478172,
                                         0.293311186727103,
                                         0.160431968051891,
                                         0.0317460612331785,
                                         0.00213393038207007,
                                         3.78022608171652e-05,
                                         9.87356580465282e-08,
                                         8.94376573913999e-12,
                                         1.41647300951934e-19,
                                         0.000556977316801354,
                                         0.0436819098703467,
                                         0.20530238310259,
                                         0.232528632456328,
                                         0.0862347751433117,
                                         0.0105616196149001,
                                         0.00035065102177498,
                                         1.89649421510011e-06,
                                         4.5326852992935e-10,
                                         3.92893148908718e-17,
                                         1.8406990810573e-05,
                                         0.00790092438650464,
                                         0.0979778561324406,
                                         0.229789691466497,
                                         0.159714225277415,
                                         0.0356409585252239,
                                         0.00221769093628593,
                                         2.48369119608785e-05,
                                         1.56624355762549e-08,
                                         7.43034854667539e-15,
                                         4.37986353026611e-07,
                                         0.00102893203546993,
                                         0.0336662210443453,
                                         0.163499768856264,
                                         0.212979233421464,
                                         0.0865965769510718,
                                         0.0100985595481008,
                                         0.000234194218682762,
                                         3.89668701503215e-07,
                                         1.01175745539136e-12,
                                         7.71977412807758e-09,
                                         9.92571278466489e-05,
                                         0.00856893878426476,
                                         0.0861727545399658,
                                         0.210376464078406,
                                         0.155854126927543,
                                         0.0340630923071823,
                                         0.0016357652331641,
                                         7.18121587875313e-06,
                                         1.02049262796748e-10,
                                         1.02049262796748e-10,
                                         7.18121587875318e-06,
                                         0.0016357652331641,
                                         0.0340630923071823,
                                         0.155854126927543,
                                         0.210376464078406,
                                         0.0861727545399658,
                                         0.00856893878426476,
                                         9.92571278466484e-05,
                                         7.71977412807758e-09,
                                         1.01175745539136e-12,
                                         3.89668701503218e-07,
                                         0.000234194218682762,
                                         0.0100985595481008,
                                         0.0865965769510718,
                                         0.212979233421464,
                                         0.163499768856264,
                                         0.0336662210443453,
                                         0.00102893203546993,
                                         4.37986353026611e-07,
                                         7.43034854667539e-15,
                                         1.5662435576255e-08,
                                         2.48369119608785e-05,
                                         0.00221769093628593,
                                         0.0356409585252239,
                                         0.159714225277415,
                                         0.229789691466497,
                                         0.0979778561324406,
                                         0.00790092438650462,
                                         1.8406990810573e-05,
                                         3.92893148908718e-17,
                                         4.53268529929353e-10,
                                         1.89649421510011e-06,
                                         0.00035065102177498,
                                         0.0105616196149001,
                                         0.0862347751433117,
                                         0.232528632456328,
                                         0.20530238310259,
                                         0.0436819098703466,
                                         0.000556977316801354,
                                         1.41647300951934e-19,
                                         8.94376573914007e-12,
                                         9.87356580465282e-08,
                                         3.78022608171652e-05,
                                         0.00213393038207007,
                                         0.0317460612331785,
                                         0.160431968051891,
                                         0.293311186727103,
                                         0.164662198478171,
                                         0.0114910787264479,
                                         3.12077408817781e-22,
                                         1.07846348057729e-13,
                                         3.14135275425762e-09,
                                         2.49046601505986e-06,
                                         0.000263481514815696,
                                         0.00714196050104975,
                                         0.0676434212070652,
                                         0.256084588917744,
                                         0.379320558008714,
                                         0.144878610298924,
                                         3.17339590680732e-25,
                                         6.00203287034963e-16,
                                         4.61282828946623e-11,
                                         7.57271065652373e-08,
                                         1.50150905144256e-05,
                                         0.000741571339035607,
                                         0.0131634011547062,
                                         0.103192030510352,
                                         0.40329863308361,
                                         0.843054480857821,

                                         0.867370035229458,
                                         0.469712428192987,
                                         0.151951657966726,
                                         0.0280038923100786,
                                         0.00271660907815672,
                                         0.0001220703125,
                                         2.05749775426793e-06,
                                         8.98188730612201e-09,
                                         4.91088299167464e-12,
                                         5.9361009453051e-17,
                                         3.01382214286561e-26,
                                         0.124095626231135,
                                         0.365449088197846,
                                         0.308094346773529,
                                         0.115248363387585,
                                         0.0203197141410203,
                                         0.0015869140625,
                                         4.64874397123463e-05,
                                         3.6883993639636e-07,
                                         4.09324749581268e-10,
                                         1.28941628522802e-14,
                                         3.56001857195574e-23,
                                         0.0081943862964431,
                                         0.131228937322769,
                                         0.28831677668654,
                                         0.218906689274268,
                                         0.0701481041923273,
                                         0.009521484375,
                                         0.000484774519443393,
                                         6.99062769554069e-06,
                                         1.57465102809233e-08,
                                         1.29268563419524e-12,
                                         1.940862665192e-20,
                                         0.000330671352659093,
                                         0.0287973497718546,
                                         0.164883151051313,
                                         0.254099305961265,
                                         0.147990713090798,
                                         0.034912109375,
                                         0.00308932871343925,
                                         8.09682192899043e-05,
                                         3.70186705067255e-07,
                                         7.91977555528453e-11,
                                         6.46632499537379e-18,
                                         9.09798651232984e-06,
                                         0.00430867769993663,
                                         0.0642911540178017,
                                         0.20110205248571,
                                         0.212873482728467,
                                         0.0872802734375,
                                         0.0134232301216483,
                                         0.00063941317580368,
                                         5.93370419123788e-06,
                                         3.30827348120437e-09,
                                         1.46888849393748e-15,
                                         1.80229758935263e-07,
                                         0.000464160300919583,
                                         0.0180492292279831,
                                         0.114594038184795,
                                         0.220465767518966,
                                         0.1571044921875,
                                         0.0419935361370649,
                                         0.0036356416547557,
                                         6.8479954471505e-05,
                                         9.94998508736622e-08,
                                         2.40244041955701e-13,
                                         2.64468496627056e-09,
                                         3.70389109389083e-05,
                                         0.00375346466084213,
                                         0.0483697425711212,
                                         0.169132471532401,
                                         0.20947265625,
                                         0.0973137232888008,
                                         0.0153125243258094,
                                         0.000585419609152126,
                                         2.21671411927481e-06,
                                         2.91060086808282e-11,
                                         2.91060086808272e-11,
                                         2.21671411927481e-06,
                                         0.000585419609152128,
                                         0.0153125243258094,
                                         0.0973137232888007,
                                         0.20947265625,
                                         0.169132471532401,
                                         0.0483697425711211,
                                         0.00375346466084212,
                                         3.70389109389083e-05,
                                         2.64468496627064e-09,
                                         2.40244041955691e-13,
                                         9.94998508736622e-08,
                                         6.84799544715052e-05,
                                         0.00363564165475571,
                                         0.0419935361370648,
                                         0.1571044921875,
                                         0.220465767518966,
                                         0.114594038184795,
                                         0.018049229227983,
                                         0.000464160300919583,
                                         1.80229758935267e-07,
                                         1.46888849393741e-15,
                                         3.30827348120437e-09,
                                         5.9337041912379e-06,
                                         0.000639413175803681,
                                         0.0134232301216483,
                                         0.0872802734375,
                                         0.212873482728467,
                                         0.20110205248571,
                                         0.0642911540178016,
                                         0.00430867769993663,
                                         9.09798651233002e-06,
                                         6.46632499537347e-18,
                                         7.91977555528453e-11,
                                         3.70186705067257e-07,
                                         8.09682192899045e-05,
                                         0.00308932871343924,
                                         0.034912109375,
                                         0.147990713090798,
                                         0.254099305961265,
                                         0.164883151051313,
                                         0.0287973497718546,
                                         0.000330671352659098,
                                         1.94086266519189e-20,
                                         1.29268563419524e-12,
                                         1.57465102809233e-08,
                                         6.99062769554071e-06,
                                         0.000484774519443392,
                                         0.009521484375,
                                         0.0701481041923273,
                                         0.218906689274267,
                                         0.28831677668654,
                                         0.131228937322769,
                                         0.00819438629644319,
                                         3.56001857195552e-23,
                                         1.28941628522802e-14,
                                         4.0932474958127e-10,
                                         3.68839936396361e-07,
                                         4.64874397123463e-05,
                                         0.0015869140625,
                                         0.0203197141410203,
                                         0.115248363387585,
                                         0.308094346773529,
                                         0.365449088197846,
                                         0.124095626231136,
                                         3.01382214286541e-26,
                                         5.9361009453051e-17,
                                         4.91088299167466e-12,
                                         8.98188730612204e-09,
                                         2.05749775426792e-06,
                                         0.0001220703125,
                                         0.00271660907815672,
                                         0.0280038923100786,
                                         0.151951657966726,
                                         0.469712428192987,
                                         0.867370035229458,

                                         0.886555265960281,
                                         0.527992814555671,
                                         0.20415126116026,
                                         0.0495725350419996,
                                         0.00716199254362546,
                                         0.000565931790514132,
                                         2.14391965330937e-05,
                                         3.14367136199943e-07,
                                         1.22917162956834e-09,
                                         6.18672032027591e-13,
                                         7.06701381838485e-18,
                                         3.47782544975831e-27,
                                         0.107247648571094,
                                         0.345635329751718,
                                         0.345030593597388,
                                         0.167546795852772,
                                         0.0430306066350116,
                                         0.00571948615776816,
                                         0.000358510836525191,
                                         8.84260528010338e-06,
                                         6.14616526649002e-08,
                                         6.18645203342073e-11,
                                         1.82445236642009e-15,
                                         4.85861538032302e-24,
                                         0.0059879427881353,
                                         0.104427803280542,
                                         0.269135534056586,
                                         0.261359940943853,
                                         0.119324320781812,
                                         0.0266782804711527,
                                         0.00276696692235413,
                                         0.000114797184764809,
                                         1.41841670780436e-06,
                                         2.85516173455376e-09,
                                         2.1738872519643e-13,
                                         3.13274581243578e-21,
                                         0.000204309088165304,
                                         0.0192812122990637,
                                         0.128293501342361,
                                         0.249150693229541,
                                         0.202209079999111,
                                         0.076046430686581,
                                         0.0130504602284027,
                                         0.000910756787025702,
                                         2.00043121444141e-05,
                                         8.0526710580756e-08,
                                         1.58292989975652e-11,
                                         1.23440586266937e-18,
                                         4.75298350488213e-06,
                                         0.00242728688261238,
                                         0.0416971961333478,
                                         0.161939858197919,
                                         0.233636619506167,
                                         0.147797951214784,
                                         0.0419678060077167,
                                         0.00492654181300527,
                                         0.000192358767238663,
                                         1.54852343352857e-06,
                                         7.85877631275626e-10,
                                         3.31634246111985e-16,
                                         7.9611796657673e-08,
                                         0.000220008964880674,
                                         0.00975756703166947,
                                         0.0757840665820714,
                                         0.194363034447547,
                                         0.20681902640228,
                                         0.0971715657071646,
                                         0.0191873247593104,
                                         0.00133178109038932,
                                         2.14401639029239e-05,
                                         2.80918712213825e-08,
                                         6.41494982279426e-14,
                                         9.87767595214812e-10,
                                         1.47715435102711e-05,
                                         0.00169138496565752,
                                         0.0262704969684742,
                                         0.119771271268294,
                                         0.214377299752389,
                                         0.166658844625835,
                                         0.0553544973019082,
                                         0.00682998790606484,
                                         0.000219889581037509,
                                         7.43828204208101e-07,
                                         9.19164806396542e-12,
                                         9.1916480639658e-12,
                                         7.43828204208095e-07,
                                         0.000219889581037508,
                                         0.00682998790606483,
                                         0.0553544973019081,
                                         0.166658844625835,
                                         0.214377299752389,
                                         0.119771271268294,
                                         0.0262704969684742,
                                         0.00169138496565752,
                                         1.47715435102712e-05,
                                         9.87767595214776e-10,
                                         6.41494982279457e-14,
                                         2.80918712213822e-08,
                                         2.14401639029238e-05,
                                         0.00133178109038932,
                                         0.0191873247593104,
                                         0.0971715657071646,
                                         0.20681902640228,
                                         0.194363034447547,
                                         0.0757840665820715,
                                         0.0097575670316695,
                                         0.000220008964880675,
                                         7.96117966576706e-08,
                                         3.31634246112003e-16,
                                         7.85877631275617e-10,
                                         1.54852343352856e-06,
                                         0.000192358767238662,
                                         0.00492654181300526,
                                         0.0419678060077167,
                                         0.147797951214784,
                                         0.233636619506167,
                                         0.161939858197919,
                                         0.0416971961333478,
                                         0.00242728688261239,
                                         4.75298350488201e-06,
                                         1.23440586266944e-18,
                                         1.5829298997565e-11,
                                         8.05267105807556e-08,
                                         2.00043121444141e-05,
                                         0.0009107567870257,
                                         0.0130504602284027,
                                         0.076046430686581,
                                         0.202209079999111,
                                         0.249150693229541,
                                         0.128293501342361,
                                         0.0192812122990638,
                                         0.0002043090881653,
                                         3.13274581243599e-21,
                                         2.17388725196427e-13,
                                         2.85516173455374e-09,
                                         1.41841670780435e-06,
                                         0.000114797184764809,
                                         0.00276696692235413,
                                         0.0266782804711527,
                                         0.119324320781812,
                                         0.261359940943853,
                                         0.269135534056586,
                                         0.104427803280542,
                                         0.00598794278813523,
                                         4.85861538032337e-24,
                                         1.82445236642006e-15,
                                         6.1864520334207e-11,
                                         6.14616526649e-08,
                                         8.84260528010337e-06,
                                         0.000358510836525191,
                                         0.00571948615776816,
                                         0.0430306066350116,
                                         0.167546795852772,
                                         0.345030593597388,
                                         0.345635329751718,
                                         0.107247648571093,
                                         3.47782544975858e-27,
                                         7.06701381838474e-18,
                                         6.18672032027587e-13,
                                         1.22917162956834e-09,
                                         3.14367136199943e-07,
                                         2.14391965330937e-05,
                                         0.000565931790514132,
                                         0.00716199254362546,
                                         0.0495725350419996,
                                         0.20415126116026,
                                         0.527992814555671,
                                         0.886555265960281,

                                         0.901929577119877,
                                         0.578708174052518,
                                         0.257098917592295,
                                         0.0772079601233041,
                                         0.0150842004841237,
                                         0.00180917188995561,
                                         0.0001220703125,
                                         4.05161339673134e-06,
                                         5.33059862854782e-08,
                                         1.91239441118274e-10,
                                         9.02142558497167e-14,
                                         9.85765213483465e-19,
                                         4.7337357940276e-28,
                                         0.0934666887492396,
                                         0.32328157932963,
                                         0.368112133855865,
                                         0.218574212307142,
                                         0.0746621595283756,
                                         0.0147091788021691,
                                         0.0015869140625,
                                         8.42182295063104e-05,
                                         1.82005468335025e-06,
                                         1.14163513663101e-08,
                                         1.06483419918047e-11,
                                         2.98221431480555e-16,
                                         7.71980786124449e-25,
                                         0.00447042675240816,
                                         0.0833508819655238,
                                         0.243258439252618,
                                         0.28559043877813,
                                         0.170563734642271,
                                         0.0551956587861411,
                                         0.009521484375,
                                         0.000807964192190656,
                                         2.86814291592058e-05,
                                         3.14546665664284e-07,
                                         5.8009177633565e-10,
                                         4.16401332664814e-14,
                                         5.81054495850063e-22,
                                         0.000130665606810106,
                                         0.0131328692410443,
                                         0.0982371781551104,
                                         0.228038715207531,
                                         0.238118438627602,
                                         0.12657315199773,
                                         0.034912109375,
                                         0.00473694454710745,
                                         0.000276208678036448,
                                         5.29618351182286e-06,
                                         1.93121936079792e-08,
                                         3.55308462281122e-12,
                                         2.67268281037285e-19,
                                         2.60400716073429e-06,
                                         0.00141083952809913,
                                         0.027049075128593,
                                         0.124148674049673,
                                         0.22665637249658,
                                         0.197900480008523,
                                         0.0872802734375,
                                         0.0189353381728795,
                                         0.00181360384954612,
                                         6.08008348047364e-05,
                                         4.38364203538787e-07,
                                         2.06712895160968e-10,
                                         8.38197870800576e-17,
                                         3.73641885593289e-08,
                                         0.000109125969276755,
                                         0.00536242779890384,
                                         0.0486640312065907,
                                         0.155337151849166,
                                         0.222784307303814,
                                         0.1571044921875,
                                         0.0544979702152056,
                                         0.00857393200481271,
                                         0.000502560737063651,
                                         7.16425532843951e-06,
                                         8.65888724999042e-09,
                                         1.89268431285326e-14,
                                         3.97132260232971e-10,
                                         6.25237245253741e-06,
                                         0.000787474912604854,
                                         0.0141299397463263,
                                         0.0788585838319341,
                                         0.185775558909814,
                                         0.20947265625,
                                         0.116186005331567,
                                         0.0300250592177229,
                                         0.00307704467166765,
                                         8.67307814534285e-05,
                                         2.68672261591372e-07,
                                         3.16574582907066e-12,
                                         3.16574582907082e-12,
                                         2.68672261591375e-07,
                                         8.67307814534281e-05,
                                         0.00307704467166765,
                                         0.0300250592177229,
                                         0.116186005331567,
                                         0.20947265625,
                                         0.185775558909814,
                                         0.0788585838319341,
                                         0.0141299397463263,
                                         0.000787474912604857,
                                         6.25237245253736e-06,
                                         3.97132260232954e-10,
                                         1.89268431285337e-14,
                                         8.65888724999051e-09,
                                         7.16425532843947e-06,
                                         0.000502560737063652,
                                         0.00857393200481271,
                                         0.0544979702152056,
                                         0.1571044921875,
                                         0.222784307303814,
                                         0.155337151849166,
                                         0.0486640312065907,
                                         0.00536242779890386,
                                         0.000109125969276754,
                                         3.73641885593276e-08,
                                         8.38197870800629e-17,
                                         2.0671289516097e-10,
                                         4.38364203538785e-07,
                                         6.08008348047366e-05,
                                         0.00181360384954612,
                                         0.0189353381728795,
                                         0.0872802734375,
                                         0.197900480008523,
                                         0.22665637249658,
                                         0.124148674049672,
                                         0.0270490751285931,
                                         0.00141083952809913,
                                         2.60400716073422e-06,
                                         2.67268281037303e-19,
                                         3.55308462281127e-12,
                                         1.93121936079791e-08,
                                         5.29618351182288e-06,
                                         0.000276208678036448,
                                         0.00473694454710745,
                                         0.034912109375,
                                         0.12657315199773,
                                         0.238118438627602,
                                         0.228038715207531,
                                         0.0982371781551106,
                                         0.0131328692410442,
                                         0.000130665606810103,
                                         5.81054495850108e-22,
                                         4.1640133266482e-14,
                                         5.80091776335646e-10,
                                         3.14546665664285e-07,
                                         2.86814291592058e-05,
                                         0.000807964192190656,
                                         0.009521484375,
                                         0.0551956587861411,
                                         0.170563734642271,
                                         0.28559043877813,
                                         0.243258439252618,
                                         0.0833508819655235,
                                         0.0044704267524081,
                                         7.71980786124514e-25,
                                         2.9822143148056e-16,
                                         1.06483419918047e-11,
                                         1.14163513663102e-08,
                                         1.82005468335025e-06,
                                         8.42182295063104e-05,
                                         0.0015869140625,
                                         0.0147091788021691,
                                         0.0746621595283756,
                                         0.218574212307141,
                                         0.368112133855865,
                                         0.323281579329629,
                                         0.0934666887492389,
                                         4.73373579402803e-28,
                                         9.85765213483482e-19,
                                         9.0214255849716e-14,
                                         1.91239441118275e-10,
                                         5.33059862854782e-08,
                                         4.05161339673134e-06,
                                         0.0001220703125,
                                         0.00180917188995561,
                                         0.0150842004841237,
                                         0.0772079601233041,
                                         0.257098917592295,
                                         0.578708174052518,
                                         0.901929577119877,

                                         0.914422677810445,
                                         0.622694144452119,
                                         0.308910908795491,
                                         0.109669610233118,
                                         0.0270957252733111,
                                         0.00446964839094741,
                                         0.000463347609093639,
                                         2.76061192697975e-05,
                                         8.25260084726878e-07,
                                         9.96225305662309e-09,
                                         3.33734124189286e-11,
                                         1.49498104174594e-14,
                                         1.576623578726e-19,
                                         7.42413729075041e-29,
                                         0.0820885475865892,
                                         0.30041010483376,
                                         0.379778724261451,
                                         0.264226825663911,
                                         0.112688414943389,
                                         0.0299923409803533,
                                         0.00484871972887206,
                                         0.000445832690425544,
                                         2.07845458834534e-05,
                                         4.04823563969029e-07,
                                         2.34097465984493e-09,
                                         2.05506235453182e-12,
                                         5.52299902836381e-17,
                                         1.39764577342633e-25,
                                         0.00340115209388099,
                                         0.066890153097282,
                                         0.215494363677235,
                                         0.293815980219792,
                                         0.21630455160839,
                                         0.0928870707975419,
                                         0.0234182875442295,
                                         0.00332312255024404,
                                         0.000241600672177659,
                                         7.59244923411412e-06,
                                         7.57880456442146e-08,
                                         1.30383375484609e-10,
                                         8.92955495691607e-15,
                                         1.21438427418477e-22,
                                         8.61171631554995e-05,
                                         0.0091018573172001,
                                         0.0747242181814915,
                                         0.199661393655423,
                                         0.253730237669618,
                                         0.175800598924375,
                                         0.0691199423521695,
                                         0.0151370409338732,
                                         0.00171623168794108,
                                         8.70198217934316e-05,
                                         1.49942546894478e-06,
                                         5.05521483510106e-09,
                                         8.82276765163654e-13,
                                         6.44815300116018e-20,
                                         1.48669525366527e-06,
                                         0.000844435406272981,
                                         0.017666699514318,
                                         0.0925084409665764,
                                         0.202930522021675,
                                         0.226857954186197,
                                         0.139097773532291,
                                         0.0470115089804465,
                                         0.00831232036020531,
                                         0.000680022152583607,
                                         2.02263555414444e-05,
                                         1.33636653519471e-07,
                                         5.94358459151083e-11,
                                         2.33444212374526e-17,
                                         1.84793500078662e-08,
                                         5.64073039130516e-05,
                                         0.00300733605668808,
                                         0.0308603695248251,
                                         0.116857076024026,
                                         0.210775519642988,
                                         0.201543820193071,
                                         0.105123519855491,
                                         0.0289868568354976,
                                         0.00382613622162497,
                                         0.000196445862907288,
                                         2.54357216012144e-06,
                                         2.88286662617965e-09,
                                         6.08503926668266e-15,
                                         1.70144399812807e-10,
                                         2.7910684123232e-06,
                                         0.000379205590869545,
                                         0.00762582989831521,
                                         0.0498458366014381,
                                         0.145061631481124,
                                         0.216314197834733,
                                         0.17412528007579,
                                         0.0748766162346834,
                                         0.0159464504270997,
                                         0.00141330003172656,
                                         3.58615259741908e-05,
                                         1.03577848193332e-07,
                                         1.17492431181308e-12,
                                         1.17492431181308e-12,
                                         1.03577848193333e-07,
                                         3.58615259741908e-05,
                                         0.00141330003172656,
                                         0.0159464504270997,
                                         0.0748766162346834,
                                         0.17412528007579,
                                         0.216314197834734,
                                         0.145061631481124,
                                         0.0498458366014381,
                                         0.00762582989831523,
                                         0.000379205590869545,
                                         2.79106841232318e-06,
                                         1.70144399812807e-10,
                                         6.08503926668266e-15,
                                         2.88286662617969e-09,
                                         2.54357216012144e-06,
                                         0.000196445862907287,
                                         0.00382613622162496,
                                         0.0289868568354976,
                                         0.105123519855491,
                                         0.201543820193071,
                                         0.210775519642988,
                                         0.116857076024026,
                                         0.0308603695248251,
                                         0.00300733605668808,
                                         5.64073039130512e-05,
                                         1.84793500078662e-08,
                                         2.33444212374526e-17,
                                         5.94358459151092e-11,
                                         1.33636653519471e-07,
                                         2.02263555414444e-05,
                                         0.000680022152583606,
                                         0.00831232036020531,
                                         0.0470115089804465,
                                         0.139097773532291,
                                         0.226857954186197,
                                         0.202930522021676,
                                         0.0925084409665766,
                                         0.017666699514318,
                                         0.000844435406272976,
                                         1.48669525366527e-06,
                                         6.44815300116018e-20,
                                         8.82276765163668e-13,
                                         5.05521483510106e-09,
                                         1.49942546894478e-06,
                                         8.70198217934314e-05,
                                         0.00171623168794108,
                                         0.0151370409338732,
                                         0.0691199423521695,
                                         0.175800598924375,
                                         0.253730237669618,
                                         0.199661393655423,
                                         0.0747242181814915,
                                         0.00910185731720006,
                                         8.61171631554995e-05,
                                         1.21438427418477e-22,
                                         8.92955495691622e-15,
                                         1.30383375484609e-10,
                                         7.57880456442143e-08,
                                         7.5924492341141e-06,
                                         0.000241600672177659,
                                         0.00332312255024404,
                                         0.0234182875442295,
                                         0.0928870707975419,
                                         0.21630455160839,
                                         0.293815980219792,
                                         0.215494363677235,
                                         0.0668901530972818,
                                         0.00340115209388099,
                                         1.39764577342633e-25,
                                         5.52299902836391e-17,
                                         2.05506235453182e-12,
                                         2.34097465984492e-09,
                                         4.04823563969028e-07,
                                         2.07845458834534e-05,
                                         0.000445832690425544,
                                         0.00484871972887206,
                                         0.0299923409803533,
                                         0.112688414943389,
                                         0.264226825663911,
                                         0.379778724261451,
                                         0.300410104833759,
                                         0.0820885475865892,
                                         7.42413729075041e-29,
                                         1.57662357872603e-19,
                                         1.49498104174594e-14,
                                         3.33734124189285e-11,
                                         9.96225305662306e-09,
                                         8.25260084726878e-07,
                                         2.76061192697974e-05,
                                         0.000463347609093639,
                                         0.00446964839094741,
                                         0.0270957252733111,
                                         0.109669610233118,
                                         0.308910908795491,
                                         0.622694144452119,
                                         0.914422677810445,

                                         0.92470189716155,
                                         0.660833322255746,
                                         0.358398946130421,
                                         0.145533272250972,
                                         0.0433311739662252,
                                         0.00917569674097203,
                                         0.00132306598135892,
                                         0.0001220703125,
                                         6.58182801326452e-06,
                                         1.80860597834553e-07,
                                         2.03667186299795e-09,
                                         6.45470768920817e-12,
                                         2.77267028169406e-15,
                                         2.84108000134374e-20,
                                         1.31669541213293e-29,
                                         0.072607636325214,
                                         0.27816095783465,
                                         0.382659499920768,
                                         0.302353108415477,
                                         0.153836991348086,
                                         0.0518366142839114,
                                         0.0114380748522536,
                                         0.0015869140625,
                                         0.000128665408470068,
                                         5.41044629470438e-06,
                                         9.69499179865721e-08,
                                         5.25062667436482e-10,
                                         4.38873296247226e-13,
                                         1.14068551987993e-17,
                                         2.83394538670734e-26,
                                         0.00263130231179632,
                                         0.0540391252571198,
                                         0.188567209114411,
                                         0.289917532273524,
                                         0.252074548440602,
                                         0.135158107482948,
                                         0.0456385947496963,
                                         0.009521484375,
                                         0.00116087370607535,
                                         7.47016366579454e-05,
                                         2.13001041288563e-06,
                                         1.97130398469739e-08,
                                         3.20618061164139e-11,
                                         2.11376300233876e-15,
                                         2.81517585528638e-23,
                                         5.82746099992236e-05,
                                         0.00641564947267299,
                                         0.0567858364313392,
                                         0.169884869523112,
                                         0.252416301018749,
                                         0.215361345748468,
                                         0.111283755419941,
                                         0.034912109375,
                                         0.00640071258048998,
                                         0.000630300023834484,
                                         2.85980345920718e-05,
                                         4.5228922238964e-07,
                                         1.43138674300277e-09,
                                         2.39368410360188e-13,
                                         1.70899094454773e-20,
                                         8.79947136478268e-07,
                                         0.000519327742467308,
                                         0.0116595679876651,
                                         0.0678740088601278,
                                         0.172335352453197,
                                         0.233970977194798,
                                         0.185011984149382,
                                         0.0872802734375,
                                         0.0240624727837045,
                                         0.00362604432250989,
                                         0.000261793720984259,
                                         7.07534231648199e-06,
                                         4.3570703876064e-08,
                                         1.84818719726709e-11,
                                         7.07363379125148e-18,
                                         9.56679098092234e-09,
                                         3.02673859874007e-05,
                                         0.00172368295733071,
                                         0.0195247192172399,
                                         0.0847156898402231,
                                         0.183015856188353,
                                         0.221462625769596,
                                         0.1571044921875,
                                         0.0651305404224197,
                                         0.0150193587034139,
                                         0.00172549919578615,
                                         7.96913475281477e-05,
                                         9.54914873136722e-07,
                                         1.02744261697752e-09,
                                         2.10803530142187e-15,
                                         7.70446027094672e-11,
                                         1.30669593070724e-06,
                                         0.000188755036050603,
                                         0.00416037353253807,
                                         0.0308474705381697,
                                         0.106042920105805,
                                         0.196366437150739,
                                         0.20947265625,
                                         0.13058561519786,
                                         0.0460824862614356,
                                         0.00842435244638216,
                                         0.00066487670338532,
                                         1.55024725470565e-05,
                                         4.23092596112596e-08,
                                         4.65349678264405e-13,
                                         4.65349678264375e-13,
                                         4.23092596112596e-08,
                                         1.55024725470564e-05,
                                         0.000664876703385322,
                                         0.00842435244638217,
                                         0.0460824862614356,
                                         0.13058561519786,
                                         0.20947265625,
                                         0.196366437150739,
                                         0.106042920105805,
                                         0.0308474705381697,
                                         0.00416037353253806,
                                         0.000188755036050604,
                                         1.30669593070724e-06,
                                         7.70446027094714e-11,
                                         2.10803530142171e-15,
                                         1.02744261697752e-09,
                                         9.54914873136716e-07,
                                         7.9691347528148e-05,
                                         0.00172549919578615,
                                         0.0150193587034139,
                                         0.0651305404224198,
                                         0.1571044921875,
                                         0.221462625769596,
                                         0.183015856188353,
                                         0.0847156898402231,
                                         0.0195247192172398,
                                         0.00172368295733071,
                                         3.02673859874007e-05,
                                         9.56679098092278e-09,
                                         7.07363379125089e-18,
                                         1.84818719726709e-11,
                                         4.35707038760637e-08,
                                         7.07534231648201e-06,
                                         0.000261793720984259,
                                         0.00362604432250989,
                                         0.0240624727837045,
                                         0.0872802734375,
                                         0.185011984149382,
                                         0.233970977194798,
                                         0.172335352453197,
                                         0.0678740088601277,
                                         0.0116595679876651,
                                         0.000519327742467308,
                                         8.799471364783e-07,
                                         1.70899094454757e-20,
                                         2.39368410360188e-13,
                                         1.43138674300276e-09,
                                         4.52289222389642e-07,
                                         2.85980345920718e-05,
                                         0.000630300023834484,
                                         0.00640071258048998,
                                         0.034912109375,
                                         0.111283755419941,
                                         0.215361345748468,
                                         0.252416301018749,
                                         0.169884869523112,
                                         0.0567858364313393,
                                         0.00641564947267299,
                                         5.82746099992252e-05,
                                         2.8151758552861e-23,
                                         2.11376300233876e-15,
                                         3.20618061164137e-11,
                                         1.9713039846974e-08,
                                         2.13001041288563e-06,
                                         7.47016366579454e-05,
                                         0.00116087370607535,
                                         0.009521484375,
                                         0.0456385947496963,
                                         0.135158107482948,
                                         0.252074548440602,
                                         0.289917532273524,
                                         0.188567209114411,
                                         0.0540391252571198,
                                         0.00263130231179637,
                                         2.83394538670703e-26,
                                         1.14068551987993e-17,
                                         4.38873296247222e-13,
                                         5.25062667436485e-10,
                                         9.69499179865721e-08,
                                         5.41044629470439e-06,
                                         0.000128665408470068,
                                         0.0015869140625,
                                         0.0114380748522536,
                                         0.0518366142839114,
                                         0.153836991348086,
                                         0.302353108415477,
                                         0.382659499920768,
                                         0.27816095783465,
                                         0.0726076363252147,
                                         1.31669541213278e-29,
                                         2.84108000134374e-20,
                                         2.77267028169403e-15,
                                         6.45470768920821e-12,
                                         2.03667186299795e-09,
                                         1.80860597834553e-07,
                                         6.58182801326452e-06,
                                         0.0001220703125,
                                         0.00132306598135892,
                                         0.00917569674097203,
                                         0.0433311739662252,
                                         0.145533272250972,
                                         0.358398946130421,
                                         0.660833322255746,
                                         0.92470189716155,

                                         0.933254702009825,
                                         0.693954822070609,
                                         0.404897796012441,
                                         0.183448200949085,
                                         0.0635183098168614,
                                         0.0164256602673722,
                                         0.00307190783911794,
                                         0.000397244694972286,
                                         3.33398424397469e-05,
                                         1.65708336921185e-06,
                                         4.25036289461543e-08,
                                         4.52129186405369e-10,
                                         1.36912212095359e-12,
                                         5.68184861694365e-16,
                                         5.68567235525078e-21,
                                         2.60068817282191e-30,
                                         0.0646381290294784,
                                         0.257131550126075,
                                         0.379106647215628,
                                         0.332298399980778,
                                         0.195029822178954,
                                         0.0793760548054833,
                                         0.0223852531260389,
                                         0.00426800533445562,
                                         0.000524425016121333,
                                         3.84306062744276e-05,
                                         1.48643541032535e-06,
                                         2.48855449885781e-08,
                                         1.27736231372577e-10,
                                         1.02555835372531e-13,
                                         2.59324986383114e-18,
                                         6.34580024004484e-27,
                                         0.00206626141433203,
                                         0.0439731357148409,
                                         0.163826926633557,
                                         0.277811961821895,
                                         0.276382801326153,
                                         0.177037016114967,
                                         0.0752876489030125,
                                         0.0211640948441837,
                                         0.00380724789701771,
                                         0.000411356165638053,
                                         2.39924153864767e-05,
                                         6.32178420885054e-07,
                                         5.50039537875695e-09,
                                         8.54356233849118e-12,
                                         5.45902139440538e-16,
                                         7.14648372523621e-24,
                                         4.0364719074466e-05,
                                         0.00459557316092461,
                                         0.0432642668641769,
                                         0.141936408744158,
                                         0.239354272470303,
                                         0.241300839451077,
                                         0.154741086066585,
                                         0.0641349299445585,
                                         0.0168911450256379,
                                         0.00269078480458055,
                                         0.000236658485605721,
                                         9.81414247636898e-06,
                                         1.44741776329547e-07,
                                         4.34948451268283e-10,
                                         7.02272090241329e-14,
                                         4.91834060616229e-21,
                                         5.3763456373846e-07,
                                         0.00032746169906956,
                                         0.00779008132833387,
                                         0.0494430509965465,
                                         0.141331813590592,
                                         0.224244687241961,
                                         0.216848351253403,
                                         0.132512890859281,
                                         0.0510946706711851,
                                         0.0120007530788675,
                                         0.00159161773078774,
                                         0.000103880403528426,
                                         2.5969432904865e-06,
                                         1.50975043413444e-08,
                                         6.15977146859761e-12,
                                         2.30788078778611e-18,
                                         5.15590520986796e-09,
                                         1.68001760898706e-05,
                                         0.00100992036801758,
                                         0.012400785856873,
                                         0.0600857070784257,
                                         0.150043777331479,
                                         0.218795862290777,
                                         0.197130638585331,
                                         0.111281921045891,
                                         0.0385363457659803,
                                         0.00770704602466031,
                                         0.000791675844273979,
                                         3.35477602252614e-05,
                                         3.77315836863043e-07,
                                         3.89005988594301e-10,
                                         7.79723527331624e-16,
                                         3.66259500771171e-11,
                                         6.38459551040343e-07,
                                         9.69836448996459e-05,
                                         0.00230387751038115,
                                         0.0189220703177727,
                                         0.0743669631029123,
                                         0.163526565843561,
                                         0.217228278513938,
                                         0.179531167724987,
                                         0.0916639971727963,
                                         0.0276441580892687,
                                         0.00446917531456447,
                                         0.000321019065491213,
                                         6.98507600887666e-06,
                                         1.81976038287067e-08,
                                         1.95134534739497e-13,
                                         1.95134534739511e-13,
                                         1.81976038287067e-08,
                                         6.98507600887662e-06,
                                         0.000321019065491214,
                                         0.00446917531456447,
                                         0.0276441580892687,
                                         0.0916639971727964,
                                         0.179531167724987,
                                         0.217228278513938,
                                         0.163526565843561,
                                         0.0743669631029123,
                                         0.0189220703177727,
                                         0.00230387751038115,
                                         9.69836448996463e-05,
                                         6.38459551040343e-07,
                                         3.66259500771148e-11,
                                         7.7972352733169e-16,
                                         3.89005988594301e-10,
                                         3.7731583686304e-07,
                                         3.35477602252615e-05,
                                         0.000791675844273979,
                                         0.00770704602466031,
                                         0.0385363457659804,
                                         0.111281921045891,
                                         0.197130638585331,
                                         0.218795862290777,
                                         0.150043777331479,
                                         0.0600857070784257,
                                         0.012400785856873,
                                         0.00100992036801759,
                                         1.68001760898706e-05,
                                         5.15590520986769e-09,
                                         2.30788078778633e-18,
                                         6.15977146859761e-12,
                                         1.50975043413443e-08,
                                         2.59694329048651e-06,
                                         0.000103880403528426,
                                         0.00159161773078774,
                                         0.0120007530788675,
                                         0.0510946706711851,
                                         0.132512890859281,
                                         0.216848351253402,
                                         0.224244687241961,
                                         0.141331813590592,
                                         0.0494430509965464,
                                         0.0077900813283339,
                                         0.00032746169906956,
                                         5.37634563738438e-07,
                                         4.91834060616281e-21,
                                         7.02272090241329e-14,
                                         4.34948451268279e-10,
                                         1.44741776329547e-07,
                                         9.81414247636898e-06,
                                         0.000236658485605721,
                                         0.00269078480458055,
                                         0.0168911450256379,
                                         0.0641349299445584,
                                         0.154741086066585,
                                         0.241300839451077,
                                         0.239354272470303,
                                         0.141936408744158,
                                         0.043264266864177,
                                         0.00459557316092461,
                                         4.03647190744647e-05,
                                         7.14648372523703e-24,
                                         5.45902139440538e-16,
                                         8.5435623384911e-12,
                                         5.50039537875697e-09,
                                         6.32178420885054e-07,
                                         2.39924153864767e-05,
                                         0.000411356165638054,
                                         0.0038072478970177,
                                         0.0211640948441837,
                                         0.0752876489030124,
                                         0.177037016114967,
                                         0.276382801326153,
                                         0.277811961821894,
                                         0.163826926633557,
                                         0.0439731357148409,
                                         0.00206626141433199,
                                         6.34580024004564e-27,
                                         2.59324986383114e-18,
                                         1.0255583537253e-13,
                                         1.27736231372578e-10,
                                         2.48855449885781e-08,
                                         1.48643541032535e-06,
                                         3.84306062744276e-05,
                                         0.000524425016121333,
                                         0.00426800533445561,
                                         0.0223852531260389,
                                         0.0793760548054833,
                                         0.195029822178954,
                                         0.332298399980778,
                                         0.379106647215628,
                                         0.257131550126075,
                                         0.0646381290294777,
                                         2.60068817282227e-30,
                                         5.68567235525078e-21,
                                         5.68184861694358e-16,
                                         1.3691221209536e-12,
                                         4.52129186405369e-10,
                                         4.25036289461543e-08,
                                         1.65708336921185e-06,
                                         3.33398424397468e-05,
                                         0.000397244694972285,
                                         0.00307190783911794,
                                         0.0164256602673722,
                                         0.0635183098168614,
                                         0.183448200949085,
                                         0.404897796012441,
                                         0.693954822070609,
                                         0.933254702009825,

                                         0.940443035207042,
                                         0.722795601815968,
                                         0.44810547862037,
                                         0.222264541242381,
                                         0.0871142127246036,
                                         0.0265070396447892,
                                         0.00611079971831509,
                                         0.00103233084423799,
                                         0.0001220703125,
                                         9.47582759210509e-06,
                                         4.40326254516638e-07,
                                         1.06677551732808e-08,
                                         1.08236918464037e-10,
                                         3.15585422887205e-13,
                                         1.27263301990292e-16,
                                         1.24859776865069e-21,
                                         5.64914846413231e-31,
                                         0.0578837493166713,
                                         0.237594465037412,
                                         0.371043963224607,
                                         0.354362617757721,
                                         0.233871594372268,
                                         0.111009292494758,
                                         0.0381417956835582,
                                         0.00935522912673495,
                                         0.0015869140625,
                                         0.000176712930829191,
                                         1.19222493369746e-05,
                                         4.30488582510309e-07,
                                         6.81355339493771e-09,
                                         3.34522952694781e-11,
                                         2.5974355215895e-14,
                                         6.41931136129025e-19,
                                         1.55112048549332e-27,
                                         0.00164432898253073,
                                         0.0360466604142529,
                                         0.141800703231365,
                                         0.260755523739406,
                                         0.289783620889325,
                                         0.214568191091705,
                                         0.109878347417205,
                                         0.0391289191693098,
                                         0.009521484375,
                                         0.00152099367260268,
                                         0.000148987460437654,
                                         8.01785380313055e-06,
                                         1.97961076175652e-07,
                                         1.63660097803934e-09,
                                         2.44677628070887e-12,
                                         1.52321857075243e-16,
                                         1.96569429272376e-24,
                                         2.85457165648641e-05,
                                         0.00334205760479182,
                                         0.0331170450776228,
                                         0.117257144280836,
                                         0.219427155404171,
                                         0.253449550398847,
                                         0.193438663344411,
                                         0.100014151567517,
                                         0.034912109375,
                                         0.00800030722573087,
                                         0.00113778817208329,
                                         9.12588115537292e-05,
                                         3.51484460049487e-06,
                                         4.89305245261615e-08,
                                         1.40852295142795e-10,
                                         2.2087989679114e-14,
                                         1.52232226726218e-21,
                                         3.37879439601156e-07,
                                         0.000211266873623355,
                                         0.00527343202152777,
                                         0.0359512245095703,
                                         0.113285803592951,
                                         0.204120342615516,
                                         0.232189739562167,
                                         0.174298494789081,
                                         0.0872802734375,
                                         0.0286915823781642,
                                         0.00592436339973222,
                                         0.000708206751402556,
                                         4.25501435260092e-05,
                                         9.97437092263329e-07,
                                         5.52843455230887e-09,
                                         2.18382952730592e-12,
                                         8.03832944321494e-19,
                                         2.87948670410717e-09,
                                         9.61571042795477e-06,
                                         0.000604599273840854,
                                         0.00793634534764217,
                                         0.0421107622522134,
                                         0.118362341644773,
                                         0.200666678742552,
                                         0.218704799899901,
                                         0.1571044921875,
                                         0.0740857758105911,
                                         0.0222103019158934,
                                         0.00395710717484574,
                                         0.000370875740553916,
                                         1.46394127002128e-05,
                                         1.56233050864295e-07,
                                         1.55458249528557e-10,
                                         3.05602919773229e-16,
                                         1.81775195908305e-11,
                                         3.24188480273353e-07,
                                         5.1346177992096e-05,
                                         0.00129775783586752,
                                         0.0115951653789715,
                                         0.0508401753669935,
                                         0.128461719962932,
                                         0.203277459323496,
                                         0.20947265625,
                                         0.141703767434213,
                                         0.0616784520451206,
                                         0.0163780349946769,
                                         0.00239453977394994,
                                         0.000159157835891292,
                                         3.27046786416996e-06,
                                         8.19737955370439e-09,
                                         8.60627915047555e-14,
                                         8.60627915047484e-14,
                                         8.19737955370452e-09,
                                         3.27046786416996e-06,
                                         0.000159157835891292,
                                         0.00239453977394995,
                                         0.0163780349946769,
                                         0.0616784520451206,
                                         0.141703767434213,
                                         0.20947265625,
                                         0.203277459323496,
                                         0.128461719962932,
                                         0.0508401753669934,
                                         0.0115951653789715,
                                         0.00129775783586751,
                                         5.1346177992096e-05,
                                         3.24188480273349e-07,
                                         1.81775195908318e-11,
                                         3.056029197732e-16,
                                         1.5545824952856e-10,
                                         1.56233050864295e-07,
                                         1.46394127002128e-05,
                                         0.000370875740553917,
                                         0.00395710717484574,
                                         0.0222103019158934,
                                         0.0740857758105911,
                                         0.1571044921875,
                                         0.218704799899901,
                                         0.200666678742552,
                                         0.118362341644772,
                                         0.0421107622522134,
                                         0.00793634534764215,
                                         0.000604599273840854,
                                         9.61571042795467e-06,
                                         2.87948670410733e-09,
                                         8.03832944321409e-19,
                                         2.18382952730597e-12,
                                         5.52843455230887e-09,
                                         9.97437092263333e-07,
                                         4.25501435260093e-05,
                                         0.000708206751402558,
                                         0.00592436339973222,
                                         0.0286915823781642,
                                         0.0872802734375,
                                         0.174298494789081,
                                         0.232189739562167,
                                         0.204120342615515,
                                         0.113285803592951,
                                         0.0359512245095702,
                                         0.00527343202152777,
                                         0.000211266873623353,
                                         3.37879439601172e-07,
                                         1.522322267262e-21,
                                         2.20879896791145e-14,
                                         1.40852295142795e-10,
                                         4.89305245261618e-08,
                                         3.51484460049488e-06,
                                         9.12588115537294e-05,
                                         0.00113778817208329,
                                         0.00800030722573087,
                                         0.034912109375,
                                         0.100014151567517,
                                         0.193438663344411,
                                         0.253449550398847,
                                         0.21942715540417,
                                         0.117257144280836,
                                         0.0331170450776228,
                                         0.0033420576047918,
                                         2.85457165648651e-05,
                                         1.9656942927235e-24,
                                         1.52321857075246e-16,
                                         2.44677628070887e-12,
                                         1.63660097803935e-09,
                                         1.97961076175653e-07,
                                         8.01785380313057e-06,
                                         0.000148987460437654,
                                         0.00152099367260268,
                                         0.009521484375,
                                         0.0391289191693098,
                                         0.109878347417205,
                                         0.214568191091705,
                                         0.289783620889325,
                                         0.260755523739406,
                                         0.141800703231365,
                                         0.0360466604142527,
                                         0.00164432898253077,
                                         1.5511204854931e-27,
                                         6.41931136129042e-19,
                                         2.5974355215895e-14,
                                         3.34522952694783e-11,
                                         6.81355339493774e-09,
                                         4.3048858251031e-07,
                                         1.19222493369746e-05,
                                         0.000176712930829191,
                                         0.0015869140625,
                                         0.00935522912673495,
                                         0.0381417956835582,
                                         0.111009292494758,
                                         0.233871594372268,
                                         0.35436261775772,
                                         0.371043963224607,
                                         0.237594465037412,
                                         0.0578837493166719,
                                         5.64914846413145e-31,
                                         1.24859776865073e-21,
                                         1.27263301990292e-16,
                                         3.15585422887207e-13,
                                         1.08236918464038e-10,
                                         1.06677551732808e-08,
                                         4.40326254516638e-07,
                                         9.47582759210509e-06,
                                         0.0001220703125,
                                         0.00103233084423799,
                                         0.00611079971831509,
                                         0.0265070396447892,
                                         0.0871142127246036,
                                         0.222264541242381,
                                         0.44810547862037,
                                         0.722795601815968,
                                         0.940443035207042,

                                         0.946539719858928,
                                         0.747991968559441,
                                         0.487957816202283,
                                         0.261072752060874,
                                         0.113441080681937,
                                         0.039482112718132,
                                         0.0108010519413892,
                                         0.00226440053672237,
                                         0.000351581330525553,
                                         3.85897838981094e-05,
                                         2.80665815857294e-06,
                                         1.23283250082374e-07,
                                         2.84730932026517e-09,
                                         2.77653498678693e-11,
                                         7.84208099060424e-14,
                                         3.08699590364529e-17,
                                         2.97878777026431e-22,
                                         1.33537565829531e-31,
                                         0.0521151615847515,
                                         0.219632908215823,
                                         0.359965216643985,
                                         0.369359036346943,
                                         0.268772889861864,
                                         0.144864428286524,
                                         0.0585076064594375,
                                         0.0175913589949372,
                                         0.00385618049097183,
                                         0.000594603557663092,
                                         6.10562437491124e-05,
                                         3.84631193668264e-06,
                                         1.31147489529545e-07,
                                         1.98049955502579e-09,
                                         9.36765411219338e-12,
                                         7.07203660270855e-15,
                                         1.71445381079193e-19,
                                         4.0988772683297e-28,
                                         0.001324333095357,
                                         0.0297649932078466,
                                         0.122559418890898,
                                         0.241181397765499,
                                         0.29390600853199,
                                         0.245318910705417,
                                         0.146273787413175,
                                         0.0630744441603648,
                                         0.0195207634604096,
                                         0.00422854112077289,
                                         0.000613025503608422,
                                         5.53850852383838e-05,
                                         2.7880028381916e-06,
                                         6.52009984130463e-08,
                                         5.16461875582605e-10,
                                         7.47757614962877e-13,
                                         4.55428195736999e-17,
                                         5.80676528325331e-25,
                                         2.05660343100201e-05,
                                         0.00246509887451692,
                                         0.025500754441418,
                                         0.0962407651490347,
                                         0.196404597050034,
                                         0.253875316880289,
                                         0.223481131551019,
                                         0.138206240221955,
                                         0.0603888035016268,
                                         0.0183769652135725,
                                         0.00376137975867724,
                                         0.00048737285209772,
                                         3.62198478429494e-05,
                                         1.31175860869074e-06,
                                         1.7400665945776e-08,
                                         4.83167116380673e-11,
                                         7.39323004417933e-15,
                                         5.02717274514513e-22,
                                         2.17757167082706e-07,
                                         0.00013919751342652,
                                         0.00361766162987208,
                                         0.0261844167604428,
                                         0.089487713347219,
                                         0.179134199612028,
                                         0.23280043901936,
                                         0.206476415115761,
                                         0.127375126714583,
                                         0.0544534758209396,
                                         0.0157356396553298,
                                         0.00292414172769741,
                                         0.000320825300542835,
                                         1.79937705503194e-05,
                                         3.99725679410147e-07,
                                         2.128641241495e-09,
                                         8.18308576978096e-13,
                                         2.96743984779455e-19,
                                         1.66007173929509e-09,
                                         5.65927903338348e-06,
                                         0.000369517792220076,
                                         0.00512931344234883,
                                         0.0293567293815087,
                                         0.0910057919090912,
                                         0.174606024690584,
                                         0.222098561903026,
                                         0.193439575130605,
                                         0.116174260333802,
                                         0.0473973561030739,
                                         0.0126318802143737,
                                         0.00204608228114559,
                                         0.000177714526941282,
                                         6.61135877672874e-06,
                                         6.7521187485468e-08,
                                         6.52127452644113e-11,
                                         1.26116681912784e-16,
                                         9.37448646063534e-12,
                                         1.70434287484866e-07,
                                         2.7958186445881e-05,
                                         0.000744289349245683,
                                         0.00713375449974177,
                                         0.0342472567905048,
                                         0.0970065112791287,
                                         0.176964956297028,
                                         0.217606699389451,
                                         0.18359483415118,
                                         0.105752358847609,
                                         0.0404206981713122,
                                         0.00966593367084593,
                                         0.00130013938034189,
                                         8.10001145806694e-05,
                                         1.58651397672843e-06,
                                         3.84958660535076e-09,
                                         3.97035536117637e-14,
                                         3.97035536117637e-14,
                                         3.84958660535076e-09,
                                         1.58651397672843e-06,
                                         8.10001145806691e-05,
                                         0.00130013938034189,
                                         0.00966593367084593,
                                         0.0404206981713122,
                                         0.105752358847609,
                                         0.18359483415118,
                                         0.217606699389451,
                                         0.176964956297028,
                                         0.0970065112791286,
                                         0.0342472567905048,
                                         0.00713375449974177,
                                         0.000744289349245685,
                                         2.7958186445881e-05,
                                         1.70434287484866e-07,
                                         9.37448646063534e-12,
                                         1.26116681912784e-16,
                                         6.52127452644113e-11,
                                         6.7521187485468e-08,
                                         6.61135877672871e-06,
                                         0.000177714526941282,
                                         0.00204608228114559,
                                         0.0126318802143737,
                                         0.0473973561030739,
                                         0.116174260333802,
                                         0.193439575130605,
                                         0.222098561903026,
                                         0.174606024690584,
                                         0.0910057919090911,
                                         0.0293567293815087,
                                         0.00512931344234884,
                                         0.000369517792220076,
                                         5.65927903338348e-06,
                                         1.66007173929509e-09,
                                         2.96743984779455e-19,
                                         8.18308576978096e-13,
                                         2.128641241495e-09,
                                         3.99725679410145e-07,
                                         1.79937705503194e-05,
                                         0.000320825300542835,
                                         0.00292414172769741,
                                         0.0157356396553298,
                                         0.0544534758209396,
                                         0.127375126714583,
                                         0.206476415115761,
                                         0.23280043901936,
                                         0.179134199612028,
                                         0.0894877133472191,
                                         0.0261844167604428,
                                         0.00361766162987208,
                                         0.00013919751342652,
                                         2.17757167082706e-07,
                                         5.02717274514513e-22,
                                         7.39323004417933e-15,
                                         4.83167116380673e-11,
                                         1.74006659457759e-08,
                                         1.31175860869074e-06,
                                         3.62198478429494e-05,
                                         0.00048737285209772,
                                         0.00376137975867724,
                                         0.0183769652135725,
                                         0.0603888035016268,
                                         0.138206240221955,
                                         0.223481131551019,
                                         0.253875316880289,
                                         0.196404597050034,
                                         0.0962407651490349,
                                         0.025500754441418,
                                         0.00246509887451692,
                                         2.05660343100201e-05,
                                         5.80676528325331e-25,
                                         4.55428195736999e-17,
                                         7.47757614962877e-13,
                                         5.16461875582602e-10,
                                         6.52009984130463e-08,
                                         2.7880028381916e-06,
                                         5.53850852383838e-05,
                                         0.000613025503608421,
                                         0.00422854112077289,
                                         0.0195207634604096,
                                         0.0630744441603648,
                                         0.146273787413175,
                                         0.245318910705417,
                                         0.29390600853199,
                                         0.241181397765499,
                                         0.122559418890898,
                                         0.0297649932078466,
                                         0.001324333095357,
                                         4.09887726832969e-28,
                                         1.71445381079193e-19,
                                         7.07203660270855e-15,
                                         9.36765411219331e-12,
                                         1.98049955502579e-09,
                                         1.31147489529545e-07,
                                         3.84631193668264e-06,
                                         6.10562437491124e-05,
                                         0.000594603557663092,
                                         0.00385618049097183,
                                         0.0175913589949372,
                                         0.0585076064594375,
                                         0.144864428286524,
                                         0.268772889861864,
                                         0.369359036346943,
                                         0.359965216643985,
                                         0.219632908215823,
                                         0.0521151615847515,
                                         1.33537565829531e-31,
                                         2.97878777026431e-22,
                                         3.08699590364529e-17,
                                         7.84208099060418e-14,
                                         2.77653498678693e-11,
                                         2.84730932026517e-09,
                                         1.23283250082374e-07,
                                         2.80665815857294e-06,
                                         3.85897838981094e-05,
                                         0.000351581330525553,
                                         0.00226440053672237,
                                         0.0108010519413892,
                                         0.039482112718132,
                                         0.113441080681937,
                                         0.261072752060874,
                                         0.487957816202283,
                                         0.747991968559441,
                                         0.946539719858928,

                                         0.951753273888084,
                                         0.770084779045908,
                                         0.52453783651108,
                                         0.299194062999917,
                                         0.141791570410183,
                                         0.0552196549276875,
                                         0.0174118394804056,
                                         0.00435869186569134,
                                         0.000843935407779603,
                                         0.0001220703125,
                                         1.2584319116121e-05,
                                         8.66325904142122e-07,
                                         3.62858541930424e-08,
                                         8.04799969666908e-10,
                                         7.58838035368654e-12,
                                         2.08620454132454e-14,
                                         8.04564790456759e-18,
                                         7.65479305089352e-23,
                                         3.4048686659754e-32,
                                         0.0471532941645207,
                                         0.203223444607229,
                                         0.346991597011831,
                                         0.378312308182611,
                                         0.298867216360951,
                                         0.179158162427872,
                                         0.0827522389541272,
                                         0.0294140820623921,
                                         0.00793878679557771,
                                         0.0015869140625,
                                         0.000226084868653558,
                                         2.1695460532023e-05,
                                         1.29029483124611e-06,
                                         4.19210107172889e-08,
                                         6.08426231134023e-10,
                                         2.7883423274887e-12,
                                         2.05544372278587e-15,
                                         4.90213074038228e-20,
                                         1.16144703737846e-28,
                                         0.00107822052605457,
                                         0.024752380653982,
                                         0.105942201378358,
                                         0.220778037588618,
                                         0.290746196928183,
                                         0.26827945069056,
                                         0.181519275379189,
                                         0.0916141021099234,
                                         0.0344672764169455,
                                         0.009521484375,
                                         0.00187465420059466,
                                         0.000250763512427011,
                                         2.11762210654826e-05,
                                         1.00782107688184e-06,
                                         2.25151407492952e-08,
                                         1.72005840815195e-10,
                                         2.42358388476832e-13,
                                         1.44891938761242e-17,
                                         1.82854795426224e-25,
                                         1.50668800999007e-05,
                                         0.00184238471584846,
                                         0.0197669292642856,
                                         0.0787374682973254,
                                         0.172850239988851,
                                         0.245503901122332,
                                         0.24332454811079,
                                         0.174377135572242,
                                         0.0914492136737435,
                                         0.034912109375,
                                         0.00949928727603333,
                                         0.00177125053877073,
                                         0.000212387105700496,
                                         1.48065987833216e-05,
                                         5.0916856447941e-07,
                                         6.48426085379301e-09,
                                         1.74634764031562e-11,
                                         2.61712061015484e-15,
                                         1.75927397498991e-22,
                                         1.43551471103965e-07,
                                         9.35001370164614e-05,
                                         0.0025146526347021,
                                         0.0191458918986899,
                                         0.0700639324697447,
                                         0.153178553805322,
                                         0.22239124188291,
                                         0.226300403069762,
                                         0.165432777858506,
                                         0.0872802734375,
                                         0.0328193104738456,
                                         0.00853029842031523,
                                         0.00145236712340781,
                                         0.000148318648129691,
                                         7.85085719773235e-06,
                                         1.66665741568969e-07,
                                         8.57969650574128e-10,
                                         3.22308552528671e-13,
                                         1.15406210400533e-19,
                                         9.84746530006496e-10,
                                         3.41646258447771e-06,
                                         0.000230329355047591,
                                         0.00335198645392966,
                                         0.0204480325632058,
                                         0.0688129266044645,
                                         0.146346362059067,
                                         0.211452883592016,
                                         0.21547438278788,
                                         0.1571044921875,
                                         0.0816395081282855,
                                         0.0295788272035732,
                                         0.00715084177555462,
                                         0.00106971652487909,
                                         8.7157561145958e-05,
                                         3.08435740450968e-06,
                                         3.03490880684411e-08,
                                         2.85793546898956e-11,
                                         5.45076399902056e-17,
                                         5.00388709928571e-12,
                                         9.24713764032488e-08,
                                         1.56274030366775e-05,
                                         0.000434705503783575,
                                         0.00442053443949323,
                                         0.0228985680085115,
                                         0.0713366011631905,
                                         0.146355190065806,
                                         0.20789113006698,
                                         0.20947265625,
                                         0.150431067726059,
                                         0.0759738102006438,
                                         0.0260797941639071,
                                         0.00571489009457136,
                                         0.000716736121324015,
                                         4.22813929031984e-05,
                                         7.95216893718227e-07,
                                         1.87715112689106e-09,
                                         1.90700489969481e-14,
                                         1.90700489969481e-14,
                                         1.87715112689106e-09,
                                         7.95216893718227e-07,
                                         4.22813929031984e-05,
                                         0.000716736121324013,
                                         0.00571489009457137,
                                         0.0260797941639071,
                                         0.0759738102006437,
                                         0.150431067726059,
                                         0.20947265625,
                                         0.20789113006698,
                                         0.146355190065806,
                                         0.0713366011631905,
                                         0.0228985680085115,
                                         0.00442053443949324,
                                         0.000434705503783575,
                                         1.56274030366775e-05,
                                         9.24713764032488e-08,
                                         5.00388709928571e-12,
                                         5.45076399902056e-17,
                                         2.85793546898956e-11,
                                         3.03490880684411e-08,
                                         3.08435740450968e-06,
                                         8.71575611459577e-05,
                                         0.00106971652487909,
                                         0.00715084177555462,
                                         0.0295788272035731,
                                         0.0816395081282855,
                                         0.1571044921875,
                                         0.21547438278788,
                                         0.211452883592016,
                                         0.146346362059067,
                                         0.0688129266044644,
                                         0.0204480325632059,
                                         0.00335198645392966,
                                         0.000230329355047591,
                                         3.41646258447771e-06,
                                         9.84746530006496e-10,
                                         1.15406210400533e-19,
                                         3.22308552528671e-13,
                                         8.57969650574128e-10,
                                         1.66665741568969e-07,
                                         7.85085719773232e-06,
                                         0.000148318648129691,
                                         0.00145236712340781,
                                         0.00853029842031522,
                                         0.0328193104738456,
                                         0.0872802734375,
                                         0.165432777858506,
                                         0.226300403069762,
                                         0.22239124188291,
                                         0.153178553805322,
                                         0.0700639324697449,
                                         0.0191458918986899,
                                         0.0025146526347021,
                                         9.35001370164614e-05,
                                         1.43551471103965e-07,
                                         1.75927397498991e-22,
                                         2.61712061015484e-15,
                                         1.74634764031562e-11,
                                         6.48426085379301e-09,
                                         5.09168564479408e-07,
                                         1.48065987833217e-05,
                                         0.000212387105700496,
                                         0.00177125053877073,
                                         0.00949928727603333,
                                         0.034912109375,
                                         0.0914492136737435,
                                         0.174377135572242,
                                         0.24332454811079,
                                         0.245503901122332,
                                         0.172850239988851,
                                         0.0787374682973254,
                                         0.0197669292642856,
                                         0.00184238471584846,
                                         1.50668800999007e-05,
                                         1.82854795426224e-25,
                                         1.44891938761242e-17,
                                         2.42358388476832e-13,
                                         1.72005840815195e-10,
                                         2.25151407492951e-08,
                                         1.00782107688184e-06,
                                         2.11762210654826e-05,
                                         0.00025076351242701,
                                         0.00187465420059466,
                                         0.009521484375,
                                         0.0344672764169455,
                                         0.0916141021099234,
                                         0.181519275379189,
                                         0.26827945069056,
                                         0.290746196928184,
                                         0.220778037588618,
                                         0.105942201378358,
                                         0.024752380653982,
                                         0.00107822052605457,
                                         1.16144703737846e-28,
                                         4.90213074038228e-20,
                                         2.05544372278587e-15,
                                         2.7883423274887e-12,
                                         6.0842623113402e-10,
                                         4.1921010717289e-08,
                                         1.29029483124611e-06,
                                         2.16954605320229e-05,
                                         0.000226084868653558,
                                         0.0015869140625,
                                         0.00793878679557771,
                                         0.0294140820623921,
                                         0.0827522389541272,
                                         0.179158162427872,
                                         0.298867216360951,
                                         0.378312308182611,
                                         0.346991597011831,
                                         0.203223444607229,
                                         0.0471532941645207,
                                         3.4048686659754e-32,
                                         7.65479305089352e-23,
                                         8.04564790456759e-18,
                                         2.08620454132454e-14,
                                         7.5883803536865e-12,
                                         8.04799969666911e-10,
                                         3.62858541930424e-08,
                                         8.6632590414212e-07,
                                         1.2584319116121e-05,
                                         0.0001220703125,
                                         0.000843935407779603,
                                         0.00435869186569134,
                                         0.0174118394804056,
                                         0.0552196549276875,
                                         0.141791570410183,
                                         0.299194062999917,
                                         0.52453783651108,
                                         0.770084779045908,
                                         0.951753273888084,

                                         0.956245109854345,
                                         0.789530008195616,
                                         0.558013602939055,
                                         0.33614885383946,
                                         0.171497704669896,
                                         0.0734476993449692,
                                         0.0260946289446113,
                                         0.0075723339346807,
                                         0.00175875025116646,
                                         0.000318369600062066,
                                         4.33638280714711e-05,
                                         4.23838952378895e-06,
                                         2.78436727649505e-07,
                                         1.11985183659357e-08,
                                         2.39935973573659e-10,
                                         2.19819411420975e-12,
                                         5.90531506128486e-15,
                                         2.23781653967884e-18,
                                         2.10350454790793e-23,
                                         9.29391557721121e-33,
                                         0.0428570763508443,
                                         0.188285878241936,
                                         0.332943999106584,
                                         0.382274973580275,
                                         0.323847627807084,
                                         0.212403334901504,
                                         0.109823480781772,
                                         0.0448804782871758,
                                         0.0143801248002239,
                                         0.00355037836292797,
                                         0.000657160228838887,
                                         8.76050394261352e-05,
                                         7.93936021990832e-06,
                                         4.49679606947539e-07,
                                         1.40216440032624e-08,
                                         1.96729884474993e-10,
                                         8.77577632583754e-13,
                                         6.33847495074673e-16,
                                         1.49066842590169e-20,
                                         3.50454881975758e-29,
                                         0.000886510231649,
                                         0.0207240558369215,
                                         0.0916865213077489,
                                         0.200644840828931,
                                         0.282248099708874,
                                         0.283499543503321,
                                         0.213327665538884,
                                         0.122770236738556,
                                         0.0542661531215223,
                                         0.0182736617839378,
                                         0.00459645305145675,
                                         0.000835728493619022,
                                         0.000104484625618249,
                                         8.33400423562044e-06,
                                         3.78190316536261e-07,
                                         8.12610690021045e-09,
                                         6.0191603917084e-11,
                                         8.28615352142999e-14,
                                         4.87558293323158e-18,
                                         6.09920742630611e-26,
                                         1.12063748148374e-05,
                                         0.00139396522333264,
                                         0.0154297887153551,
                                         0.0643576692563604,
                                         0.15032856545939,
                                         0.231240327967563,
                                         0.253232438780401,
                                         0.205233801773537,
                                         0.125145603434962,
                                         0.0574773460655111,
                                         0.0196469239918795,
                                         0.00487215963566659,
                                         0.000840309851243275,
                                         9.43896598561839e-05,
                                         6.23364481041558e-06,
                                         2.05123257886927e-07,
                                         2.52293735261832e-09,
                                         6.61974640637419e-12,
                                         9.74523292084889e-16,
                                         6.48686519159231e-23,
                                         9.65862075661956e-08,
                                         6.39289693258181e-05,
                                         0.00177044745214993,
                                         0.0140747664165271,
                                         0.0545909406791424,
                                         0.128600710390529,
                                         0.204955733734385,
                                         0.233923183234833,
                                         0.196775369153414,
                                         0.123264043395892,
                                         0.0572578293258429,
                                         0.0193662880757191,
                                         0.00460781566873607,
                                         0.000728892872546161,
                                         7.00555067899897e-05,
                                         3.53033451119388e-06,
                                         7.21017150876419e-08,
                                         3.605772381e-10,
                                         1.32808685250591e-13,
                                         4.70397408064729e-20,
                                         5.99373558821305e-10,
                                         2.11094035666773e-06,
                                         0.000146264388438936,
                                         0.0022162287326053,
                                         0.0142735545469425,
                                         0.0514938845571671,
                                         0.119435464725197,
                                         0.191968573066097,
                                         0.222770854802114,
                                         0.190330596535032,
                                         0.12014555025168,
                                         0.0554247525023692,
                                         0.0181921168103146,
                                         0.00405261625828315,
                                         0.000566858942115131,
                                         4.37471038648075e-05,
                                         1.4836013528012e-06,
                                         1.41412486810151e-08,
                                         1.30314645686755e-11,
                                         2.45599491872567e-17,
                                         2.75515653276704e-12,
                                         5.16321790725535e-08,
                                         8.95076983460471e-06,
                                         0.00025849622045002,
                                         0.00276445758988948,
                                         0.0152733454249924,
                                         0.051555234496926,
                                         0.116695266386241,
                                         0.186815212518812,
                                         0.217694290747711,
                                         0.186744032831156,
                                         0.117497159781461,
                                         0.0532031818260185,
                                         0.0166906593247777,
                                         0.00339761336434271,
                                         0.000401558652112408,
                                         2.26128383108254e-05,
                                         4.10812304453301e-07,
                                         9.47166238269011e-10,
                                         9.49852651363732e-15,
                                         9.49852651363624e-15,
                                         9.47166238268991e-10,
                                         4.10812304453301e-07,
                                         2.26128383108253e-05,
                                         0.000401558652112408,
                                         0.00339761336434272,
                                         0.0166906593247777,
                                         0.0532031818260185,
                                         0.117497159781461,
                                         0.186744032831156,
                                         0.217694290747711,
                                         0.186815212518812,
                                         0.116695266386241,
                                         0.051555234496926,
                                         0.0152733454249924,
                                         0.00276445758988948,
                                         0.000258496220450021,
                                         8.95076983460471e-06,
                                         5.16321790725544e-08,
                                         2.75515653276731e-12,
                                         2.45599491872535e-17,
                                         1.30314645686752e-11,
                                         1.41412486810151e-08,
                                         1.4836013528012e-06,
                                         4.37471038648075e-05,
                                         0.000566858942115132,
                                         0.00405261625828315,
                                         0.0181921168103146,
                                         0.0554247525023692,
                                         0.12014555025168,
                                         0.190330596535032,
                                         0.222770854802114,
                                         0.191968573066097,
                                         0.119435464725197,
                                         0.051493884557167,
                                         0.0142735545469425,
                                         0.00221622873260531,
                                         0.000146264388438936,
                                         2.11094035666776e-06,
                                         5.99373558821353e-10,
                                         4.70397408064661e-20,
                                         1.32808685250588e-13,
                                         3.605772381e-10,
                                         7.21017150876414e-08,
                                         3.53033451119388e-06,
                                         7.00555067899899e-05,
                                         0.000728892872546161,
                                         0.00460781566873607,
                                         0.0193662880757191,
                                         0.0572578293258429,
                                         0.123264043395892,
                                         0.196775369153414,
                                         0.233923183234833,
                                         0.204955733734385,
                                         0.128600710390529,
                                         0.0545909406791424,
                                         0.0140747664165271,
                                         0.00177044745214993,
                                         6.39289693258189e-05,
                                         9.65862075662018e-08,
                                         6.48686519159127e-23,
                                         9.74523292084859e-16,
                                         6.61974640637418e-12,
                                         2.5229373526183e-09,
                                         2.05123257886927e-07,
                                         6.2336448104156e-06,
                                         9.43896598561839e-05,
                                         0.000840309851243275,
                                         0.00487215963566659,
                                         0.0196469239918795,
                                         0.0574773460655111,
                                         0.125145603434962,
                                         0.205233801773538,
                                         0.253232438780401,
                                         0.231240327967563,
                                         0.15032856545939,
                                         0.0643576692563605,
                                         0.0154297887153551,
                                         0.00139396522333265,
                                         1.1206374814838e-05,
                                         6.09920742630503e-26,
                                         4.87558293323141e-18,
                                         8.28615352142999e-14,
                                         6.01916039170836e-11,
                                         8.12610690021045e-09,
                                         3.78190316536262e-07,
                                         8.33400423562044e-06,
                                         0.000104484625618249,
                                         0.000835728493619022,
                                         0.00459645305145675,
                                         0.0182736617839378,
                                         0.0542661531215223,
                                         0.122770236738556,
                                         0.213327665538884,
                                         0.28349954350332,
                                         0.282248099708874,
                                         0.200644840828931,
                                         0.0916865213077489,
                                         0.0207240558369216,
                                         0.000886510231649028,
                                         3.5045488197569e-29,
                                         1.49066842590164e-20,
                                         6.33847495074673e-16,
                                         8.77577632583746e-13,
                                         1.96729884474993e-10,
                                         1.40216440032625e-08,
                                         4.49679606947539e-07,
                                         7.93936021990832e-06,
                                         8.76050394261352e-05,
                                         0.000657160228838887,
                                         0.00355037836292797,
                                         0.0143801248002239,
                                         0.0448804782871758,
                                         0.109823480781772,
                                         0.212403334901504,
                                         0.323847627807084,
                                         0.382274973580275,
                                         0.332943999106584,
                                         0.188285878241937,
                                         0.042857076350845,
                                         9.29391557720926e-33,
                                         2.10350454790785e-23,
                                         2.23781653967884e-18,
                                         5.9053150612848e-15,
                                         2.19819411420975e-12,
                                         2.3993597357366e-10,
                                         1.11985183659357e-08,
                                         2.78436727649505e-07,
                                         4.23838952378895e-06,
                                         4.33638280714711e-05,
                                         0.000318369600062066,
                                         0.00175875025116646,
                                         0.0075723339346807,
                                         0.0260946289446113,
                                         0.0734476993449692,
                                         0.171497704669896,
                                         0.33614885383946,
                                         0.558013602939055,
                                         0.789530008195616,
                                         0.956245109854345,

                                         0.960141647560633,
                                         0.806710642768759,
                                         0.588597099385389,
                                         0.371618665430519,
                                         0.201968771640702,
                                         0.0938086615043379,
                                         0.0368820979101265,
                                         0.012122611331778,
                                         0.00327800578085776,
                                         0.000714251440108432,
                                         0.0001220703125,
                                         1.57927671539067e-05,
                                         1.4746227806715e-06,
                                         9.30567660807509e-08,
                                         3.61431282449601e-09,
                                         7.51676986521884e-11,
                                         6.71795326932526e-13,
                                         1.76914076406334e-15,
                                         6.60336398848677e-19,
                                         6.14254632379579e-24,
                                         2.6982932645222e-33,
                                         0.039114397804297,
                                         0.174712925269513,
                                         0.318411079666865,
                                         0.382228252362844,
                                         0.343796036252025,
                                         0.243488077395022,
                                         0.138554707492517,
                                         0.0636927015576817,
                                         0.0235557538652557,
                                         0.00692558656795566,
                                         0.0015869140625,
                                         0.000275257714378047,
                                         3.4680100777487e-05,
                                         2.9932374592167e-06,
                                         1.62594845590481e-07,
                                         4.89421673277538e-09,
                                         6.66971139594104e-11,
                                         2.9068580784504e-13,
                                         2.06291763171368e-16,
                                         4.79322588779912e-21,
                                         1.11937219218373e-29,
                                         0.000735437695924169,
                                         0.0174638576258454,
                                         0.0794998159525826,
                                         0.18144957203345,
                                         0.270100510129749,
                                         0.291689206926226,
                                         0.240234210626646,
                                         0.154451118239551,
                                         0.0781253900068072,
                                         0.0309934482776927,
                                         0.009521484375,
                                         0.00221426025411358,
                                         0.000376432975799038,
                                         4.44367497286551e-05,
                                         3.37594767883124e-06,
                                         1.47076416252519e-07,
                                         3.05622274580849e-09,
                                         2.20441402791356e-11,
                                         2.97444838208224e-14,
                                         1.72629594411265e-18,
                                         2.14322504886081e-26,
                                         8.45036182245682e-06,
                                         0.00106678158208469,
                                         0.012130096480272,
                                         0.0526392013822944,
                                         0.129679140581348,
                                         0.21354194738373,
                                         0.254547362152869,
                                         0.228882518686038,
                                         0.158346172104714,
                                         0.0847624328670422,
                                         0.034912109375,
                                         0.0108852400224036,
                                         0.00249698050180787,
                                         0.000403147136519654,
                                         4.28355966557315e-05,
                                         2.70099052487647e-06,
                                         8.55821406592864e-08,
                                         1.0216042321004e-09,
                                         2.62090431366223e-12,
                                         3.7994679390137e-16,
                                         2.50773256173458e-23,
                                         6.62023340221121e-08,
                                         4.44303172348351e-05,
                                         0.00126191750650033,
                                         0.0104119293728678,
                                         0.0424505597442532,
                                         0.106589546984932,
                                         0.18389542573882,
                                         0.231261196939814,
                                         0.218822269542046,
                                         0.158054005797829,
                                         0.0872802734375,
                                         0.0364851329606928,
                                         0.0112930491806449,
                                         0.00249375309251774,
                                         0.000370580456635679,
                                         3.38198487130178e-05,
                                         1.63399177134978e-06,
                                         3.22805390940989e-08,
                                         1.57457906452479e-10,
                                         5.7016282339911e-14,
                                         2.000613706982e-20,
                                         3.73425347681122e-10,
                                         1.33234229796684e-06,
                                         9.4521405742928e-05,
                                         0.00148281042837928,
                                         0.0100052792643115,
                                         0.038307034281872,
                                         0.0956545755241174,
                                         0.168238508962085,
                                         0.21772483166454,
                                         0.212197419195202,
                                         0.1571044921875,
                                         0.0880493903287543,
                                         0.0367739079207751,
                                         0.0111064641780478,
                                         0.00230830238410587,
                                         0.000304896723122375,
                                         2.24620340593846e-05,
                                         7.34397806365327e-07,
                                         6.81099052953322e-09,
                                         6.16038003981685e-12,
                                         1.14915274110927e-17,
                                         1.56027273112724e-12,
                                         2.95950071170771e-08,
                                         5.2443976007396e-06,
                                         0.000156425042409096,
                                         0.0017467921857534,
                                         0.0101978513179676,
                                         0.0368558814697066,
                                         0.0906597012937414,
                                         0.160468813081214,
                                         0.211028406425419,
                                         0.20947265625,
                                         0.157399375381254,
                                         0.0887022386519802,
                                         0.0366407555382701,
                                         0.0106504784909629,
                                         0.00203610459775115,
                                         0.00022872547029444,
                                         1.23762249497895e-05,
                                         2.18233949060006e-07,
                                         4.93040216239721e-10,
                                         4.88943307668381e-15,
                                         4.88943307668442e-15,
                                         4.93040216239709e-10,
                                         2.18233949060008e-07,
                                         1.23762249497895e-05,
                                         0.00022872547029444,
                                         0.00203610459775115,
                                         0.0106504784909629,
                                         0.0366407555382701,
                                         0.0887022386519802,
                                         0.157399375381254,
                                         0.20947265625,
                                         0.211028406425419,
                                         0.160468813081214,
                                         0.0906597012937415,
                                         0.0368558814697066,
                                         0.0101978513179676,
                                         0.0017467921857534,
                                         0.000156425042409096,
                                         5.24439760073956e-06,
                                         2.95950071170777e-08,
                                         1.56027273112708e-12,
                                         1.14915274110943e-17,
                                         6.16038003981669e-12,
                                         6.8109905295333e-09,
                                         7.34397806365328e-07,
                                         2.24620340593846e-05,
                                         0.000304896723122375,
                                         0.00230830238410587,
                                         0.0111064641780478,
                                         0.0367739079207751,
                                         0.0880493903287543,
                                         0.1571044921875,
                                         0.212197419195202,
                                         0.21772483166454,
                                         0.168238508962085,
                                         0.0956545755241174,
                                         0.038307034281872,
                                         0.0100052792643115,
                                         0.00148281042837928,
                                         9.45214057429273e-05,
                                         1.33234229796686e-06,
                                         3.73425347681088e-10,
                                         2.00061370698232e-20,
                                         5.70162823399093e-14,
                                         1.57457906452481e-10,
                                         3.22805390940989e-08,
                                         1.63399177134978e-06,
                                         3.38198487130178e-05,
                                         0.000370580456635679,
                                         0.00249375309251774,
                                         0.0112930491806449,
                                         0.0364851329606928,
                                         0.0872802734375,
                                         0.158054005797829,
                                         0.218822269542046,
                                         0.231261196939815,
                                         0.18389542573882,
                                         0.106589546984932,
                                         0.0424505597442532,
                                         0.0104119293728678,
                                         0.00126191750650032,
                                         4.44303172348357e-05,
                                         6.62023340221074e-08,
                                         2.50773256173502e-23,
                                         3.79946793901357e-16,
                                         2.62090431366226e-12,
                                         1.0216042321004e-09,
                                         8.55821406592864e-08,
                                         2.70099052487647e-06,
                                         4.28355966557315e-05,
                                         0.000403147136519653,
                                         0.00249698050180787,
                                         0.0108852400224036,
                                         0.034912109375,
                                         0.0847624328670422,
                                         0.158346172104714,
                                         0.228882518686039,
                                         0.254547362152869,
                                         0.21354194738373,
                                         0.129679140581348,
                                         0.0526392013822944,
                                         0.0121300964802719,
                                         0.0010667815820847,
                                         8.45036182245637e-06,
                                         2.14322504886122e-26,
                                         1.72629594411259e-18,
                                         2.97444838208229e-14,
                                         2.20441402791356e-11,
                                         3.05622274580849e-09,
                                         1.47076416252519e-07,
                                         3.37594767883124e-06,
                                         4.4436749728655e-05,
                                         0.000376432975799038,
                                         0.00221426025411358,
                                         0.009521484375,
                                         0.0309934482776927,
                                         0.0781253900068072,
                                         0.154451118239551,
                                         0.240234210626646,
                                         0.291689206926226,
                                         0.270100510129749,
                                         0.18144957203345,
                                         0.0794998159525823,
                                         0.0174638576258455,
                                         0.000735437695924143,
                                         1.11937219218397e-29,
                                         4.79322588779893e-21,
                                         2.06291763171371e-16,
                                         2.9068580784504e-13,
                                         6.66971139594104e-11,
                                         4.89421673277538e-09,
                                         1.62594845590481e-07,
                                         2.99323745921669e-06,
                                         3.4680100777487e-05,
                                         0.000275257714378047,
                                         0.0015869140625,
                                         0.00692558656795566,
                                         0.0235557538652557,
                                         0.0636927015576817,
                                         0.138554707492517,
                                         0.243488077395022,
                                         0.343796036252025,
                                         0.382228252362844,
                                         0.318411079666864,
                                         0.174712925269513,
                                         0.0391143978042963,
                                         2.69829326452282e-33,
                                         6.14254632379552e-24,
                                         6.60336398848689e-19,
                                         1.76914076406334e-15,
                                         6.71795326932526e-13,
                                         7.51676986521884e-11,
                                         3.61431282449601e-09,
                                         9.30567660807507e-08,
                                         1.4746227806715e-06,
                                         1.57927671539067e-05,
                                         0.0001220703125,
                                         0.000714251440108432,
                                         0.00327800578085776,
                                         0.012122611331778,
                                         0.0368820979101265,
                                         0.0938086615043379,
                                         0.201968771640702,
                                         0.371618665430519,
                                         0.588597099385389,
                                         0.806710642768759,
                                         0.960141647560633,

                                         0.96354297242909,
                                         0.821948059921505,
                                         0.616517905483387,
                                         0.405409824986716,
                                         0.232706898108542,
                                         0.115906574328389,
                                         0.0497036074544144,
                                         0.018166063717628,
                                         0.00558628067695323,
                                         0.00142176507163854,
                                         0.000293240047258184,
                                         4.76947943848897e-05,
                                         5.9024496032534e-06,
                                         5.2978215014859e-07,
                                         3.22889657531765e-08,
                                         1.21674461860555e-09,
                                         2.46598951176625e-11,
                                         2.15701333679556e-13,
                                         5.58294808582713e-16,
                                         2.05659839430557e-19,
                                         1.89578693581689e-24,
                                         8.28597642901794e-34,
                                         0.0358354063347639,
                                         0.162387553428385,
                                         0.303806478755278,
                                         0.379036833821703,
                                         0.359038132822488,
                                         0.271670470231014,
                                         0.167824447586144,
                                         0.085286559130878,
                                         0.0356125121330497,
                                         0.012121493990584,
                                         0.00331507516022939,
                                         0.000712996810357117,
                                         0.000117001298728588,
                                         1.40444316414941e-05,
                                         1.1623076036035e-06,
                                         6.09002146131502e-08,
                                         1.77804834624984e-09,
                                         2.36269800992177e-11,
                                         1.00916724958454e-13,
                                         7.05317826020915e-17,
                                         1.62169010501464e-21,
                                         3.76520956363494e-30,
                                         0.000615122306508303,
                                         0.0148070656085243,
                                         0.069096534470402,
                                         0.163559758210242,
                                         0.255670013424483,
                                         0.29388992413746,
                                         0.261535375398992,
                                         0.18480267419636,
                                         0.104782888412175,
                                         0.0476971421471082,
                                         0.0172970235148113,
                                         0.00491939948392237,
                                         0.00107042680381099,
                                         0.000171837883010196,
                                         1.93106068263716e-05,
                                         1.40684454235128e-06,
                                         5.91703044686778e-08,
                                         1.19445969198989e-09,
                                         8.4191956979701e-12,
                                         1.11642133193628e-14,
                                         6.40256661194695e-19,
                                         7.89664311625274e-27,
                                         6.45254139737858e-06,
                                         0.000825097825941365,
                                         0.00960363577666344,
                                         0.0431312141878572,
                                         0.111260034903177,
                                         0.194288519591982,
                                         0.249072530935371,
                                         0.244712463826341,
                                         0.188407549778457,
                                         0.11469612518453,
                                         0.0551530523127558,
                                         0.0207422935168874,
                                         0.0059847146534316,
                                         0.00128485417132247,
                                         0.000196060878439519,
                                         1.98606554527809e-05,
                                         1.20332855469514e-06,
                                         3.69024297358587e-08,
                                         4.29238077527614e-10,
                                         1.07991996961566e-12,
                                         1.54475825364827e-16,
                                         1.01208280998174e-23,
                                         4.61496807342533e-08,
                                         3.13480451144728e-05,
                                         0.000910088599276185,
                                         0.00775488722818788,
                                         0.0330116451160818,
                                         0.0875745994695842,
                                         0.161729709270363,
                                         0.220939062111813,
                                         0.230980233280219,
                                         0.188050162205621,
                                         0.119904727728373,
                                         0.0596307179195947,
                                         0.0228138456859294,
                                         0.00655024195154332,
                                         0.00135723342005188,
                                         0.000191165546437655,
                                         1.66852692511941e-05,
                                         7.77332818083253e-07,
                                         1.49208806287775e-08,
                                         7.12235466990706e-11,
                                         2.54118054259012e-14,
                                         8.84419204678179e-21,
                                         2.37650700477785e-10,
                                         8.57527349065608e-07,
                                         6.20960769454107e-05,
                                         0.00100390307759911,
                                         0.00705224903297314,
                                         0.0284211519631842,
                                         0.075611225007279,
                                         0.14362214842098,
                                         0.203884319523405,
                                         0.221988856929908,
                                         0.187687590308921,
                                         0.123428598290354,
                                         0.0626161047413903,
                                         0.0240432593975309,
                                         0.0067647327260816,
                                         0.00132482393075184,
                                         0.000166576875789231,
                                         1.17893956133492e-05,
                                         3.73442005925514e-07,
                                         3.3821130249717e-09,
                                         3.00983726421082e-12,
                                         5.56458495056091e-18,
                                         9.06516530703358e-13,
                                         1.73760768272429e-08,
                                         3.13841871291398e-06,
                                         9.62663079956123e-05,
                                         0.00111597468662471,
                                         0.00683237075765338,
                                         0.0261847827047644,
                                         0.0691570665413173,
                                         0.133308876397137,
                                         0.19411311055817,
                                         0.217621120091959,
                                         0.189246470020191,
                                         0.127303348279546,
                                         0.065372572181405,
                                         0.0249754298267144,
                                         0.00680100254852755,
                                         0.0012318630741245,
                                         0.000132447056467114,
                                         6.92337905562736e-06,
                                         1.18964906908222e-07,
                                         2.64068585889918e-10,
                                         2.59342456845168e-15,
                                         2.59342456845168e-15,
                                         2.64068585889918e-10,
                                         1.1896490690822e-07,
                                         6.9233790556274e-06,
                                         0.000132447056467114,
                                         0.0012318630741245,
                                         0.00680100254852755,
                                         0.0249754298267144,
                                         0.0653725721814051,
                                         0.127303348279545,
                                         0.189246470020191,
                                         0.217621120091959,
                                         0.19411311055817,
                                         0.133308876397137,
                                         0.0691570665413173,
                                         0.0261847827047643,
                                         0.00683237075765338,
                                         0.00111597468662471,
                                         9.62663079956118e-05,
                                         3.13841871291401e-06,
                                         1.73760768272429e-08,
                                         9.06516530703358e-13,
                                         5.56458495056091e-18,
                                         3.00983726421082e-12,
                                         3.38211302497166e-09,
                                         3.73442005925517e-07,
                                         1.17893956133493e-05,
                                         0.000166576875789231,
                                         0.00132482393075184,
                                         0.0067647327260816,
                                         0.0240432593975309,
                                         0.0626161047413902,
                                         0.123428598290354,
                                         0.187687590308921,
                                         0.221988856929908,
                                         0.203884319523405,
                                         0.14362214842098,
                                         0.075611225007279,
                                         0.0284211519631842,
                                         0.00705224903297313,
                                         0.00100390307759911,
                                         6.20960769454112e-05,
                                         8.57527349065608e-07,
                                         2.37650700477785e-10,
                                         8.84419204678179e-21,
                                         2.54118054259012e-14,
                                         7.12235466990696e-11,
                                         1.49208806287776e-08,
                                         7.77332818083257e-07,
                                         1.66852692511941e-05,
                                         0.000191165546437655,
                                         0.00135723342005188,
                                         0.00655024195154331,
                                         0.0228138456859293,
                                         0.0596307179195947,
                                         0.119904727728373,
                                         0.188050162205621,
                                         0.230980233280219,
                                         0.220939062111813,
                                         0.161729709270363,
                                         0.0875745994695842,
                                         0.0330116451160817,
                                         0.00775488722818785,
                                         0.000910088599276191,
                                         3.13480451144728e-05,
                                         4.61496807342533e-08,
                                         1.01208280998174e-23,
                                         1.54475825364827e-16,
                                         1.07991996961565e-12,
                                         4.29238077527618e-10,
                                         3.69024297358589e-08,
                                         1.20332855469514e-06,
                                         1.98606554527809e-05,
                                         0.000196060878439519,
                                         0.00128485417132247,
                                         0.00598471465343159,
                                         0.0207422935168874,
                                         0.0551530523127558,
                                         0.11469612518453,
                                         0.188407549778457,
                                         0.244712463826341,
                                         0.249072530935371,
                                         0.194288519591982,
                                         0.111260034903177,
                                         0.0431312141878571,
                                         0.00960363577666348,
                                         0.000825097825941365,
                                         6.45254139737858e-06,
                                         7.89664311625274e-27,
                                         6.40256661194696e-19,
                                         1.11642133193626e-14,
                                         8.41919569797017e-12,
                                         1.1944596919899e-09,
                                         5.91703044686778e-08,
                                         1.40684454235128e-06,
                                         1.93106068263716e-05,
                                         0.000171837883010196,
                                         0.00107042680381099,
                                         0.00491939948392237,
                                         0.0172970235148113,
                                         0.0476971421471082,
                                         0.104782888412175,
                                         0.18480267419636,
                                         0.261535375398992,
                                         0.29388992413746,
                                         0.255670013424483,
                                         0.163559758210241,
                                         0.0690965344704022,
                                         0.0148070656085243,
                                         0.000615122306508303,
                                         3.76520956363494e-30,
                                         1.62169010501464e-21,
                                         7.05317826020902e-17,
                                         1.00916724958455e-13,
                                         2.36269800992179e-11,
                                         1.77804834624984e-09,
                                         6.09002146131502e-08,
                                         1.1623076036035e-06,
                                         1.40444316414941e-05,
                                         0.000117001298728588,
                                         0.000712996810357117,
                                         0.00331507516022939,
                                         0.012121493990584,
                                         0.0356125121330497,
                                         0.085286559130878,
                                         0.167824447586144,
                                         0.271670470231014,
                                         0.359038132822488,
                                         0.379036833821702,
                                         0.303806478755278,
                                         0.162387553428385,
                                         0.0358354063347639,
                                         8.28597642901794e-34,
                                         1.89578693581689e-24,
                                         2.05659839430553e-19,
                                         5.58294808582719e-16,
                                         2.15701333679557e-13,
                                         2.46598951176625e-11,
                                         1.21674461860555e-09,
                                         3.22889657531765e-08,
                                         5.2978215014859e-07,
                                         5.90244960325339e-06,
                                         4.76947943848897e-05,
                                         0.000293240047258184,
                                         0.00142176507163854,
                                         0.00558628067695323,
                                         0.018166063717628,
                                         0.0497036074544144,
                                         0.115906574328389,
                                         0.232706898108542,
                                         0.405409824986716,
                                         0.616517905483387,
                                         0.821948059921505,
                                         0.96354297242909,

                                         0.966529111786493,
                                         0.83551215740737,
                                         0.642006920671105,
                                         0.437422048140622,
                                         0.263308367254466,
                                         0.139342672483326,
                                         0.0644088385510964,
                                         0.0257896892668562,
                                         0.00885166700399291,
                                         0.00256996981641078,
                                         0.000620695383503974,
                                         0.0001220703125,
                                         1.90189115376306e-05,
                                         2.26454389452645e-06,
                                         1.96386205740682e-07,
                                         1.1611817409198e-08,
                                         4.2617707819645e-10,
                                         8.4448508197268e-12,
                                         7.2492017051143e-14,
                                         1.84812818862536e-16,
                                         6.73005116816321e-20,
                                         6.15472813378999e-25,
                                         2.67824215612562e-34,
                                         0.0329475036698286,
                                         0.151192835022198,
                                         0.289414230594872,
                                         0.373435559183665,
                                         0.370033245617266,
                                         0.296528576760123,
                                         0.196656197300998,
                                         0.108941568161501,
                                         0.0504632280875413,
                                         0.0194480217001447,
                                         0.00617140368536422,
                                         0.0015869140625,
                                         0.000323271131097106,
                                         5.05731540912338e-05,
                                         5.82167186669025e-06,
                                         4.64557407869362e-07,
                                         2.35892766110522e-08,
                                         6.7065052556255e-10,
                                         8.71767759746138e-12,
                                         3.6585062026578e-14,
                                         2.52304443207932e-17,
                                         5.74800505933542e-22,
                                         1.32778870734563e-30,
                                         0.000518367767160435,
                                         0.0126275048972908,
                                         0.06021545215882,
                                         0.147142648217352,
                                         0.240007452898908,
                                         0.291243920533906,
                                         0.27712627507129,
                                         0.212397320447563,
                                         0.132780068234162,
                                         0.067925170345002,
                                         0.0283202637606882,
                                         0.009521484375,
                                         0.0025360399182927,
                                         0.000521275405799614,
                                         7.96512076699252e-05,
                                         8.57800864284303e-06,
                                         6.02624877889443e-07,
                                         2.45815041818606e-08,
                                         4.8385955469359e-10,
                                         3.34259222191919e-12,
                                         4.36555364577768e-15,
                                         2.47761206648634e-19,
                                         3.03819747071884e-27,
                                         4.98394960520611e-06,
                                         0.000644501690317316,
                                         0.00765625169149072,
                                         0.035430856113017,
                                         0.0951324958893956,
                                         0.174810439280445,
                                         0.238653578986533,
                                         0.253060653135683,
                                         0.21350641411221,
                                         0.144979374631643,
                                         0.0794201699755132,
                                         0.034912109375,
                                         0.0121580920666978,
                                         0.00328348177486388,
                                         0.000665973865641918,
                                         9.67951838953106e-05,
                                         9.40805115210768e-06,
                                         5.50605848179668e-07,
                                         1.64118681492586e-08,
                                         1.86630741216329e-10,
                                         4.61608651812192e-13,
                                         6.52633936807914e-17,
                                         4.24837852673478e-24,
                                         3.26721620346494e-08,
                                         2.24284444696051e-05,
                                         0.000663732418664588,
                                         0.00581692283633644,
                                         0.025709973694663,
                                         0.0715395885316263,
                                         0.140128598125111,
                                         0.205574287604373,
                                         0.234076353784706,
                                         0.210984371362669,
                                         0.151856333143956,
                                         0.0872802734375,
                                         0.0397414170405274,
                                         0.0141016705188759,
                                         0.00379656278941231,
                                         0.000744714135410154,
                                         0.000100143060944555,
                                         8.40894987012321e-06,
                                         3.79546766020892e-07,
                                         7.10479484208064e-09,
                                         3.32795257721717e-11,
                                         1.17212674958681e-14,
                                         4.05041003611717e-21,
                                         1.54210733401156e-10,
                                         5.61961733842025e-07,
                                         4.14288001183003e-05,
                                         0.000687602513659047,
                                         0.00500272776052375,
                                         0.0210793885010835,
                                         0.0592404159579109,
                                         0.1202390288031,
                                         0.184772211799172,
                                         0.221068490963826,
                                         0.209058342038144,
                                         0.1571044921875,
                                         0.0935306097537248,
                                         0.0436052733170561,
                                         0.0155831942144472,
                                         0.00412532284392638,
                                         0.000767493224165837,
                                         9.24645378712424e-05,
                                         6.31982522199741e-06,
                                         1.94738759444096e-07,
                                         1.7274791503936e-09,
                                         1.51569562739571e-12,
                                         2.78039995197115e-18,
                                         5.39159873141755e-13,
                                         1.0429910099228e-08,
                                         1.91548127816139e-06,
                                         6.02071181686735e-05,
                                         0.000721071514261761,
                                         0.00460082601038575,
                                         0.0185513557087674,
                                         0.0520940781211288,
                                         0.108039359969029,
                                         0.17158117546068,
                                         0.213190734762421,
                                         0.20947265625,
                                         0.163053608428532,
                                         0.0998788870050436,
                                         0.0473792968739435,
                                         0.0169274881186933,
                                         0.00435706932169919,
                                         0.000753138544913936,
                                         7.7949094009662e-05,
                                         3.95384361549983e-06,
                                         6.64224256406737e-08,
                                         1.45182961036664e-10,
                                         1.4137798439587e-15,
                                         1.41377984395891e-15,
                                         1.45182961036664e-10,
                                         6.64224256406729e-08,
                                         3.95384361549983e-06,
                                         7.7949094009662e-05,
                                         0.000753138544913934,
                                         0.0043570693216992,
                                         0.0169274881186933,
                                         0.0473792968739435,
                                         0.0998788870050436,
                                         0.163053608428532,
                                         0.20947265625,
                                         0.213190734762421,
                                         0.17158117546068,
                                         0.108039359969029,
                                         0.0520940781211289,
                                         0.0185513557087674,
                                         0.00460082601038576,
                                         0.000721071514261761,
                                         6.02071181686735e-05,
                                         1.91548127816141e-06,
                                         1.0429910099228e-08,
                                         5.39159873141687e-13,
                                         2.78039995197162e-18,
                                         1.51569562739571e-12,
                                         1.72747915039358e-09,
                                         1.94738759444096e-07,
                                         6.31982522199741e-06,
                                         9.24645378712421e-05,
                                         0.000767493224165838,
                                         0.00412532284392637,
                                         0.0155831942144472,
                                         0.0436052733170561,
                                         0.0935306097537248,
                                         0.1571044921875,
                                         0.209058342038144,
                                         0.221068490963826,
                                         0.184772211799172,
                                         0.120239028803101,
                                         0.0592404159579108,
                                         0.0210793885010835,
                                         0.00500272776052375,
                                         0.000687602513659047,
                                         4.14288001183006e-05,
                                         5.61961733842025e-07,
                                         1.5421073340114e-10,
                                         4.05041003611794e-21,
                                         1.17212674958681e-14,
                                         3.32795257721712e-11,
                                         7.10479484208064e-09,
                                         3.79546766020892e-07,
                                         8.40894987012318e-06,
                                         0.000100143060944555,
                                         0.000744714135410153,
                                         0.00379656278941231,
                                         0.0141016705188759,
                                         0.0397414170405274,
                                         0.0872802734375,
                                         0.151856333143956,
                                         0.210984371362669,
                                         0.234076353784706,
                                         0.205574287604374,
                                         0.140128598125111,
                                         0.0715395885316264,
                                         0.025709973694663,
                                         0.00581692283633644,
                                         0.000663732418664592,
                                         2.24284444696051e-05,
                                         3.26721620346466e-08,
                                         4.24837852673569e-24,
                                         6.52633936807914e-17,
                                         4.61608651812184e-13,
                                         1.86630741216329e-10,
                                         1.64118681492586e-08,
                                         5.50605848179666e-07,
                                         9.40805115210771e-06,
                                         9.67951838953104e-05,
                                         0.000665973865641918,
                                         0.00328348177486388,
                                         0.0121580920666978,
                                         0.034912109375,
                                         0.0794201699755132,
                                         0.144979374631643,
                                         0.21350641411221,
                                         0.253060653135684,
                                         0.238653578986533,
                                         0.174810439280445,
                                         0.0951324958893956,
                                         0.035430856113017,
                                         0.00765625169149076,
                                         0.000644501690317316,
                                         4.9839496052058e-06,
                                         3.03819747071955e-27,
                                         2.47761206648634e-19,
                                         4.3655536457776e-15,
                                         3.34259222191919e-12,
                                         4.8385955469359e-10,
                                         2.45815041818605e-08,
                                         6.02624877889445e-07,
                                         8.57800864284301e-06,
                                         7.96512076699252e-05,
                                         0.000521275405799614,
                                         0.0025360399182927,
                                         0.009521484375,
                                         0.0283202637606882,
                                         0.067925170345002,
                                         0.132780068234162,
                                         0.212397320447563,
                                         0.27712627507129,
                                         0.291243920533906,
                                         0.240007452898908,
                                         0.147142648217352,
                                         0.0602154521588202,
                                         0.0126275048972908,
                                         0.000518367767160413,
                                         1.32778870734597e-30,
                                         5.74800505933542e-22,
                                         2.52304443207927e-17,
                                         3.6585062026578e-14,
                                         8.71767759746138e-12,
                                         6.70650525562547e-10,
                                         2.35892766110523e-08,
                                         4.64557407869361e-07,
                                         5.82167186669025e-06,
                                         5.05731540912338e-05,
                                         0.000323271131097106,
                                         0.0015869140625,
                                         0.00617140368536422,
                                         0.0194480217001447,
                                         0.0504632280875413,
                                         0.108941568161501,
                                         0.196656197300998,
                                         0.296528576760124,
                                         0.370033245617266,
                                         0.373435559183665,
                                         0.289414230594872,
                                         0.151192835022198,
                                         0.0329475036698279,
                                         2.67824215612636e-34,
                                         6.15472813378999e-25,
                                         6.73005116816306e-20,
                                         1.84812818862536e-16,
                                         7.2492017051143e-14,
                                         8.44485081972676e-12,
                                         4.26177078196451e-10,
                                         1.1611817409198e-08,
                                         1.96386205740682e-07,
                                         2.26454389452645e-06,
                                         1.90189115376306e-05,
                                         0.0001220703125,
                                         0.000620695383503974,
                                         0.00256996981641078,
                                         0.00885166700399291,
                                         0.0257896892668562,
                                         0.0644088385510964,
                                         0.139342672483326,
                                         0.263308367254466,
                                         0.437422048140622,
                                         0.642006920671105,
                                         0.83551215740737,
                                         0.966529111786493,

                                         0.969164644126161,
                                         0.847630040252118,
                                         0.665286753813533,
                                         0.467622896778888,
                                         0.293456856355,
                                         0.163739416219211,
                                         0.0807931576392643,
                                         0.0350122725326846,
                                         0.0132110087482358,
                                         0.00429366425299025,
                                         0.00118584085323235,
                                         0.00027362331386587,
                                         5.16241494428885e-05,
                                         7.74660170561281e-06,
                                         8.91734779027203e-07,
                                         7.50383091888437e-08,
                                         4.3204202983081e-09,
                                         1.54939579021264e-10,
                                         3.00999760370667e-12,
                                         2.54150126896537e-14,
                                         6.39376444045934e-17,
                                         2.30486405617581e-20,
                                         2.09315224244469e-25,
                                         9.07315202136778e-35,
                                         0.0303915794153249,
                                         0.14101728346837,
                                         0.275423622253948,
                                         0.36603340775267,
                                         0.377297097911512,
                                         0.317893903068889,
                                         0.224265851190362,
                                         0.133883059823277,
                                         0.0678215276436762,
                                         0.029073351369856,
                                         0.0104688592727195,
                                         0.00312882340378107,
                                         0.000762977376767564,
                                         0.000148294477434545,
                                         2.22564243963555e-05,
                                         2.47023435060303e-06,
                                         1.9094459672958e-07,
                                         9.43323186499894e-09,
                                         2.62013635751814e-10,
                                         3.34070272745453e-12,
                                         1.38044268357401e-14,
                                         9.40891858726071e-18,
                                         2.12628520578233e-22,
                                         4.88977847127728e-31,
                                         0.000439862437578894,
                                         0.0108279501265309,
                                         0.0526261249886059,
                                         0.132237180376877,
                                         0.223887871041661,
                                         0.28485185868487,
                                         0.287315869197938,
                                         0.236286471233241,
                                         0.160696735214614,
                                         0.0908594305904179,
                                         0.0426560084439142,
                                         0.0165126548281127,
                                         0.0052044915806741,
                                         0.0013102272478237,
                                         0.00025637931396066,
                                         3.75319536802866e-05,
                                         3.8949030065253e-06,
                                         2.65073641999263e-07,
                                         1.05266342919257e-08,
                                         2.0267175606618e-10,
                                         1.37558687930939e-12,
                                         1.7727275914168e-15,
                                         9.96896602164258e-20,
                                         1.21626463397086e-27,
                                         3.89045738776821e-06,
                                         0.000508089627837208,
                                         0.00614499848455079,
                                         0.0291948747087932,
                                         0.0811891227021781,
                                         0.155982579013172,
                                         0.224944955780245,
                                         0.254842738704245,
                                         0.232684106248345,
                                         0.173526219259329,
                                         0.106213873072092,
                                         0.0532565383412488,
                                         0.0216952740247903,
                                         0.00707438116712791,
                                         0.00180480692281153,
                                         0.000348485222641782,
                                         4.85518855529502e-05,
                                         4.55190050787998e-06,
                                         2.58449298126941e-07,
                                         7.51395879913367e-09,
                                         8.3767942846984e-11,
                                         2.04110091732412e-13,
                                         2.85626760379622e-17,
                                         1.84878816925221e-24,
                                         2.34613515596807e-08,
                                         1.62555994754565e-05,
                                         0.000489227369788129,
                                         0.00439468983702216,
                                         0.0200739902477265,
                                         0.0582373646330636,
                                         0.120077457083203,
                                         0.187402011897935,
                                         0.229717966585335,
                                         0.225958548687157,
                                         0.180322888602197,
                                         0.117110960665818,
                                         0.0616624175459427,
                                         0.0260434708668125,
                                         0.00866257671009142,
                                         0.0022061556007265,
                                         0.00041265216946776,
                                         5.32951335815498e-05,
                                         4.32643075454955e-06,
                                         1.89938483640251e-07,
                                         3.47805346726156e-09,
                                         1.60234315605694e-11,
                                         5.57976939733695e-15,
                                         1.91608515530034e-21,
                                         1.01868025464312e-10,
                                         3.74453718157004e-07,
                                         2.80434995004659e-05,
                                         0.000476301927941564,
                                         0.00357356812290823,
                                         0.0156552435266823,
                                         0.0461507966145777,
                                         0.0992221722863717,
                                         0.163288539198122,
                                         0.211848281365032,
                                         0.220420996933934,
                                         0.185419177163505,
                                         0.126185209182016,
                                         0.0690306188360827,
                                         0.029936149237366,
                                         0.0100558875878831,
                                         0.00252519349021231,
                                         0.000449277682181733,
                                         5.21454780914026e-05,
                                         3.45692231525629e-06,
                                         1.0397457508445e-07,
                                         9.05688968604518e-10,
                                         7.84812847016732e-13,
                                         1.42979888992371e-18,
                                         3.27634012476837e-13,
                                         6.38939203475177e-09,
                                         1.19074811033268e-06,
                                         3.82386614172549e-05,
                                         0.000471234038986498,
                                         0.00311734000536327,
                                         0.013139025425121,
                                         0.0389143198526139,
                                         0.0859770769244775,
                                         0.147125294466531,
                                         0.199581967846955,
                                         0.217459300608629,
                                         0.191276927815405,
                                         0.13553482577797,
                                         0.0766321496652808,
                                         0.0339524337996031,
                                         0.0114464659563762,
                                         0.00280548769120457,
                                         0.000465553698947999,
                                         4.66049992322824e-05,
                                         2.30241860447002e-06,
                                         3.79200462010087e-08,
                                         8.17677764851947e-11,
                                         7.90317022753583e-16,
                                         7.90317022753711e-16,
                                         8.17677764851947e-11,
                                         3.79200462010092e-08,
                                         2.30241860447002e-06,
                                         4.66049992322824e-05,
                                         0.000465553698948,
                                         0.00280548769120457,
                                         0.0114464659563763,
                                         0.0339524337996031,
                                         0.0766321496652808,
                                         0.13553482577797,
                                         0.191276927815405,
                                         0.217459300608629,
                                         0.199581967846955,
                                         0.147125294466531,
                                         0.0859770769244775,
                                         0.0389143198526138,
                                         0.013139025425121,
                                         0.00311734000536326,
                                         0.000471234038986498,
                                         3.82386614172549e-05,
                                         1.19074811033267e-06,
                                         6.38939203475177e-09,
                                         3.27634012476791e-13,
                                         1.42979888992397e-18,
                                         7.84812847016733e-13,
                                         9.05688968604531e-10,
                                         1.0397457508445e-07,
                                         3.45692231525629e-06,
                                         5.21454780914028e-05,
                                         0.000449277682181733,
                                         0.00252519349021232,
                                         0.0100558875878831,
                                         0.029936149237366,
                                         0.0690306188360826,
                                         0.126185209182016,
                                         0.185419177163505,
                                         0.220420996933934,
                                         0.211848281365032,
                                         0.163288539198122,
                                         0.0992221722863716,
                                         0.0461507966145777,
                                         0.0156552435266823,
                                         0.00357356812290823,
                                         0.000476301927941564,
                                         2.80434995004656e-05,
                                         3.74453718157005e-07,
                                         1.018680254643e-10,
                                         1.91608515530074e-21,
                                         5.57976939733695e-15,
                                         1.60234315605697e-11,
                                         3.47805346726156e-09,
                                         1.89938483640251e-07,
                                         4.32643075454957e-06,
                                         5.32951335815498e-05,
                                         0.000412652169467761,
                                         0.0022061556007265,
                                         0.00866257671009142,
                                         0.0260434708668125,
                                         0.0616624175459427,
                                         0.117110960665818,
                                         0.180322888602197,
                                         0.225958548687157,
                                         0.229717966585335,
                                         0.187402011897934,
                                         0.120077457083203,
                                         0.0582373646330635,
                                         0.0200739902477265,
                                         0.00439468983702216,
                                         0.000489227369788125,
                                         1.62555994754565e-05,
                                         2.34613515596785e-08,
                                         1.84878816925264e-24,
                                         2.85626760379622e-17,
                                         2.04110091732415e-13,
                                         8.3767942846984e-11,
                                         7.51395879913367e-09,
                                         2.58449298126942e-07,
                                         4.55190050787998e-06,
                                         4.85518855529503e-05,
                                         0.000348485222641782,
                                         0.00180480692281153,
                                         0.00707438116712791,
                                         0.0216952740247903,
                                         0.0532565383412488,
                                         0.106213873072092,
                                         0.173526219259329,
                                         0.232684106248345,
                                         0.254842738704245,
                                         0.224944955780245,
                                         0.155982579013172,
                                         0.0811891227021781,
                                         0.0291948747087932,
                                         0.00614499848455076,
                                         0.000508089627837208,
                                         3.89045738776794e-06,
                                         1.21626463397116e-27,
                                         9.96896602164258e-20,
                                         1.77272759141683e-15,
                                         1.37558687930939e-12,
                                         2.0267175606618e-10,
                                         1.05266342919257e-08,
                                         2.65073641999263e-07,
                                         3.89490300652531e-06,
                                         3.75319536802866e-05,
                                         0.00025637931396066,
                                         0.0013102272478237,
                                         0.0052044915806741,
                                         0.0165126548281127,
                                         0.0426560084439142,
                                         0.0908594305904179,
                                         0.160696735214614,
                                         0.236286471233241,
                                         0.287315869197938,
                                         0.284851858684869,
                                         0.223887871041661,
                                         0.132237180376877,
                                         0.0526261249886057,
                                         0.0108279501265309,
                                         0.000439862437578873,
                                         4.88977847127863e-31,
                                         2.12628520578233e-22,
                                         9.40891858726091e-18,
                                         1.38044268357401e-14,
                                         3.34070272745453e-12,
                                         2.62013635751815e-10,
                                         9.43323186499894e-09,
                                         1.9094459672958e-07,
                                         2.47023435060303e-06,
                                         2.22564243963555e-05,
                                         0.000148294477434545,
                                         0.000762977376767562,
                                         0.00312882340378107,
                                         0.0104688592727195,
                                         0.029073351369856,
                                         0.0678215276436762,
                                         0.133883059823277,
                                         0.224265851190362,
                                         0.317893903068888,
                                         0.377297097911512,
                                         0.366033407752671,
                                         0.275423622253948,
                                         0.14101728346837,
                                         0.0303915794153242,
                                         9.0731520213705e-35,
                                         2.09315224244469e-25,
                                         2.30486405617586e-20,
                                         6.39376444045934e-17,
                                         2.54150126896537e-14,
                                         3.00999760370668e-12,
                                         1.54939579021264e-10,
                                         4.32042029830811e-09,
                                         7.50383091888437e-08,
                                         8.91734779027203e-07,
                                         7.74660170561281e-06,
                                         5.16241494428884e-05,
                                         0.00027362331386587,
                                         0.00118584085323235,
                                         0.00429366425299025,
                                         0.0132110087482358,
                                         0.0350122725326846,
                                         0.0807931576392643,
                                         0.163739416219211,
                                         0.293456856355,
                                         0.467622896778888,
                                         0.665286753813533,
                                         0.847630040252118,
                                         0.969164644126161,

                                         6.103515625e-05,
                                         0.0008544921875,
                                         0.00555419921875,
                                         0.022216796875,
                                         0.06109619140625,
                                         0.1221923828125,
                                         0.18328857421875,
                                         0.20947265625,
                                         0.18328857421875,
                                         0.1221923828125,
                                         0.06109619140625,
                                         0.022216796875,
                                         0.00555419921875,
                                         0.0008544921875,
                                         6.103515625e-05,

                                         0.0360225190787645,
                                         3.54248821868359e-10,
                                         0.135130868550855,
                                         1.85090444242779e-08,
                                         0.235353346154666,
                                         4.48998512241204e-07,
                                         0.25225095615242,
                                         6.70274104082805e-06,
                                         0.185873709975304,
                                         6.87911678144553e-05,
                                         0.0996094209641187,
                                         0.000513464266791089,
                                         0.040035395858801,
                                         0.00287441209730317,
                                         0.0122599451303155,
                                         0.0122599451303155,
                                         0.00287441209730317,
                                         0.040035395858801,
                                         0.000513464266791089,
                                         0.0996094209641187,
                                         6.87911678144553e-05,
                                         0.185873709975304,
                                         6.70274104082805e-06,
                                         0.252250956152421,
                                         4.48998512241204e-07,
                                         0.235353346154666,
                                         1.85090444242779e-08,
                                         0.135130868550855,
                                         3.54248821868359e-10,
                                         0.0360225190787645,

                                         0.187488799999946,
                                         6.103515625e-05,
                                         5.33365193014362e-14,
                                         0.333398799994121,
                                         0.0008544921875,
                                         5.87884539486827e-12,
                                         0.275256799699154,
                                         0.00555419921875,
                                         3.00846337276719e-10,
                                         0.139848790525768,
                                         0.022216796875,
                                         9.47423281258841e-09,
                                         0.0488485948761881,
                                         0.06109619140625,
                                         2.05123812166901e-07,
                                         0.0124091701272859,
                                         0.1221923828125,
                                         3.22987271420117e-06,
                                         0.00236425689886611,
                                         0.18328857421875,
                                         3.81431011339132e-05,
                                         0.000343200000000004,
                                         0.20947265625,
                                         0.000343200000000004,
                                         3.81431011339132e-05,
                                         0.18328857421875,
                                         0.00236425689886611,
                                         3.22987271420117e-06,
                                         0.1221923828125,
                                         0.0124091701272859,
                                         2.05123812166901e-07,
                                         0.06109619140625,
                                         0.0488485948761881,
                                         9.47423281258841e-09,
                                         0.022216796875,
                                         0.139848790525768,
                                         3.00846337276719e-10,
                                         0.00555419921875,
                                         0.275256799699154,
                                         5.87884539486827e-12,
                                         0.0008544921875,
                                         0.333398799994121,
                                         5.33365193014362e-14,
                                         6.103515625e-05,
                                         0.187488799999946,

                                         0.365152783437079,
                                         0.00367249509375289,
                                         1.81706217561132e-07,
                                         6.05092558496104e-17,
                                         0.38142852209937,
                                         0.0253248577444598,
                                         5.16464017485559e-06,
                                         1.13537501674333e-14,
                                         0.184985222322477,
                                         0.0810808187841441,
                                         6.81546781586485e-05,
                                         9.89104496558002e-13,
                                         0.0552087025808268,
                                         0.159748162574084,
                                         0.000553474871450649,
                                         5.30263401652873e-11,
                                         0.0113279350190444,
                                         0.216384585553985,
                                         0.00309010222925985,
                                         1.95440170706211e-09,
                                         0.00169040690783159,
                                         0.213164102619815,
                                         0.0125471499559266,
                                         5.23881804697753e-08,
                                         0.000189187758574763,
                                         0.157493662947544,
                                         0.0382101368344255,
                                         1.05320778336533e-06,
                                         1.61322704035543e-05,
                                         0.0886569497666001,
                                         0.0886569497666001,
                                         1.61322704035544e-05,
                                         1.05320778336533e-06,
                                         0.0382101368344255,
                                         0.157493662947544,
                                         0.000189187758574764,
                                         5.2388180469775e-08,
                                         0.0125471499559266,
                                         0.213164102619815,
                                         0.0016904069078316,
                                         1.9544017070621e-09,
                                         0.00309010222925985,
                                         0.216384585553985,
                                         0.0113279350190445,
                                         5.30263401652868e-11,
                                         0.000553474871450649,
                                         0.159748162574084,
                                         0.0552087025808269,
                                         9.89104496557992e-13,
                                         6.81546781586485e-05,
                                         0.0810808187841441,
                                         0.184985222322477,
                                         1.13537501674332e-14,
                                         5.16464017485559e-06,
                                         0.0253248577444598,
                                         0.38142852209937,
                                         6.05092558496097e-17,
                                         1.81706217561132e-07,
                                         0.00367249509375289,
                                         0.365152783437079,

                                         0.510357250193351,
                                         0.0253994189612207,
                                         6.103515625e-05,
                                         1.21447408590844e-09,
                                         2.49876863169458e-19,
                                         0.351669409988091,
                                         0.106675224463167,
                                         0.0008544921875,
                                         5.66766979964708e-08,
                                         7.10758091827518e-17,
                                         0.112507174446971,
                                         0.208012134213633,
                                         0.00555419921875,
                                         1.22802200468364e-06,
                                         9.38648306749639e-15,
                                         0.0221499360873215,
                                         0.249609096988519,
                                         0.022216796875,
                                         1.63739851559847e-05,
                                         7.62835023093601e-13,
                                         0.00299804011495531,
                                         0.205922997258237,
                                         0.06109619140625,
                                         0.00015009814958142,
                                         4.26217276415682e-11,
                                         0.000295120721233587,
                                         0.123551093759772,
                                         0.1221923828125,
                                         0.00100067623536604,
                                         1.73192378636107e-09,
                                         2.17882942093482e-05,
                                         0.0555967751507125,
                                         0.18328857421875,
                                         0.00500349070334389,
                                         5.27822339873523e-08,
                                         1.22559631434575e-06,
                                         0.019061334218115,
                                         0.20947265625,
                                         0.019061334218115,
                                         1.22559631434576e-06,
                                         5.27822339873518e-08,
                                         0.00500349070334389,
                                         0.18328857421875,
                                         0.0555967751507125,
                                         2.17882942093483e-05,
                                         1.73192378636105e-09,
                                         0.00100067623536604,
                                         0.1221923828125,
                                         0.123551093759772,
                                         0.000295120721233588,
                                         4.26217276415677e-11,
                                         0.00015009814958142,
                                         0.06109619140625,
                                         0.205922997258237,
                                         0.00299804011495532,
                                         7.62835023093591e-13,
                                         1.63739851559847e-05,
                                         0.022216796875,
                                         0.249609096988519,
                                         0.0221499360873216,
                                         9.38648306749625e-15,
                                         1.22802200468364e-06,
                                         0.00555419921875,
                                         0.208012134213633,
                                         0.112507174446971,
                                         7.10758091827507e-17,
                                         5.66766979964708e-08,
                                         0.0008544921875,
                                         0.106675224463167,
                                         0.351669409988091,
                                         2.49876863169453e-19,
                                         1.21447408590844e-09,
                                         6.103515625e-05,
                                         0.0253994189612207,
                                         0.510357250193351,

                                         0.618240113826958,
                                         0.0743911598489896,
                                         0.0012209821119349,
                                         1.34278680180447e-06,
                                         1.60184035857797e-11,
                                         2.50367697730218e-21,
                                         0.302463127550387,
                                         0.212400902913704,
                                         0.0105075499431233,
                                         3.05823584097101e-05,
                                         1.09961403480128e-09,
                                         1.00304197799572e-18,
                                         0.0687025754499269,
                                         0.28156372892951,
                                         0.0419835294140928,
                                         0.000323385247870867,
                                         3.50466602094341e-08,
                                         1.86571474156581e-16,
                                         0.00960329415088601,
                                         0.229690848727484,
                                         0.103229319085503,
                                         0.00210434077238461,
                                         6.87384343939723e-07,
                                         2.13559065523182e-14,
                                         0.000922869191392459,
                                         0.128820025063038,
                                         0.174501788474093,
                                         0.00941422638908022,
                                         9.2688389709756e-06,
                                         1.68059660984617e-12,
                                         6.44996512117038e-05,
                                         0.05254366569572,
                                         0.214532949644489,
                                         0.0306302476093643,
                                         9.0896746752376e-05,
                                         9.61847579172153e-11,
                                         3.38092742061936e-06,
                                         0.0160738022100557,
                                         0.197810521967794,
                                         0.0747442245784971,
                                         0.000668548018476573,
                                         4.12867115228212e-09,
                                         1.35025258453866e-07,
                                         0.00374642946066364,
                                         0.138965009616562,
                                         0.138965009616562,
                                         0.00374642946066363,
                                         1.35025258453866e-07,
                                         4.12867115228212e-09,
                                         0.000668548018476575,
                                         0.0747442245784971,
                                         0.197810521967794,
                                         0.0160738022100556,
                                         3.38092742061937e-06,
                                         9.61847579172153e-11,
                                         9.08967467523762e-05,
                                         0.0306302476093643,
                                         0.214532949644489,
                                         0.0525436656957199,
                                         6.44996512117038e-05,
                                         1.68059660984617e-12,
                                         9.26883897097563e-06,
                                         0.00941422638908022,
                                         0.174501788474093,
                                         0.128820025063038,
                                         0.000922869191392459,
                                         2.13559065523182e-14,
                                         6.87384343939726e-07,
                                         0.00210434077238461,
                                         0.103229319085503,
                                         0.229690848727484,
                                         0.00960329415088601,
                                         1.86571474156581e-16,
                                         3.50466602094342e-08,
                                         0.000323385247870867,
                                         0.0419835294140928,
                                         0.281563728929509,
                                         0.0687025754499269,
                                         1.00304197799572e-18,
                                         1.09961403480128e-09,
                                         3.05823584097101e-05,
                                         0.0105075499431233,
                                         0.212400902913704,
                                         0.302463127550387,
                                         2.50367697730218e-21,
                                         1.60184035857798e-11,
                                         1.34278680180447e-06,
                                         0.0012209821119349,
                                         0.0743911598489896,
                                         0.618240113826958,

                                         0.697079712833371,
                                         0.14408462533881,
                                         0.00718960257481076,
                                         6.103515625e-05,
                                         4.17037490335719e-08,
                                         3.62488447956394e-13,
                                         4.77181713464555e-23,
                                         0.254814924689031,
                                         0.299379853945045,
                                         0.0425397641797981,
                                         0.0008544921875,
                                         1.3814684658828e-06,
                                         3.41936381454514e-11,
                                         2.55857084001812e-20,
                                         0.0432466627988767,
                                         0.288810397160246,
                                         0.116861267610103,
                                         0.00555419921875,
                                         2.12467336903652e-05,
                                         1.49755152304836e-09,
                                         6.36936915695268e-18,
                                         0.0045167595700706,
                                         0.171454824485121,
                                         0.197557145461426,
                                         0.022216796875,
                                         0.000201089783987806,
                                         4.03612743065914e-08,
                                         9.75757795912054e-16,
                                         0.000324320237430402,
                                         0.0699776411295626,
                                         0.229608305927287,
                                         0.06109619140625,
                                         0.00130846027896925,
                                         7.47860625510198e-07,
                                         1.02768576654118e-13,
                                         1.69362956743495e-05,
                                         0.0207714201007362,
                                         0.194079537304741,
                                         0.1221923828125,
                                         0.00619196340221148,
                                         1.00779864280975e-05,
                                         7.87183450778483e-12,
                                         6.63321490762378e-07,
                                         0.00462417587285961,
                                         0.12303627251735,
                                         0.18328857421875,
                                         0.021976447082824,
                                         0.000101856355630063,
                                         4.52223193134998e-10,
                                         1.97938555752547e-08,
                                         0.000784337871553811,
                                         0.0594274739705861,
                                         0.20947265625,
                                         0.0594274739705861,
                                         0.000784337871553811,
                                         1.9793855575255e-08,
                                         4.52223193134991e-10,
                                         0.000101856355630063,
                                         0.021976447082824,
                                         0.18328857421875,
                                         0.12303627251735,
                                         0.00462417587285961,
                                         6.63321490762387e-07,
                                         7.87183450778467e-12,
                                         1.00779864280975e-05,
                                         0.00619196340221148,
                                         0.1221923828125,
                                         0.194079537304741,
                                         0.0207714201007362,
                                         1.69362956743497e-05,
                                         1.02768576654116e-13,
                                         7.47860625510198e-07,
                                         0.00130846027896925,
                                         0.06109619140625,
                                         0.229608305927287,
                                         0.0699776411295626,
                                         0.000324320237430405,
                                         9.75757795912031e-16,
                                         4.03612743065914e-08,
                                         0.000201089783987806,
                                         0.022216796875,
                                         0.197557145461426,
                                         0.171454824485121,
                                         0.00451675957007063,
                                         6.36936915695251e-18,
                                         1.49755152304836e-09,
                                         2.12467336903652e-05,
                                         0.00555419921875,
                                         0.116861267610103,
                                         0.288810397160246,
                                         0.0432466627988769,
                                         2.55857084001804e-20,
                                         3.41936381454514e-11,
                                         1.3814684658828e-06,
                                         0.0008544921875,
                                         0.0425397641797981,
                                         0.299379853945045,
                                         0.254814924689031,
                                         4.7718171346454e-23,
                                         3.62488447956394e-13,
                                         4.17037490335719e-08,
                                         6.103515625e-05,
                                         0.00718960257481076,
                                         0.14408462533881,
                                         0.697079712833371,

                                         0.755203785209844,
                                         0.222907441326673,
                                         0.0225673311590701,
                                         0.000645059766382991,
                                         3.57646004274254e-06,
                                         1.78842935111527e-09,
                                         1.26039259762581e-14,
                                         1.47979123861944e-24,
                                         0.214177259647798,
                                         0.353178389843839,
                                         0.0982637538480874,
                                         0.00623124265394479,
                                         7.2566259913673e-05,
                                         8.05034905015786e-08,
                                         1.55916601435049e-12,
                                         1.02269780426944e-21,
                                         0.0282012187983436,
                                         0.259805893621451,
                                         0.198651487281298,
                                         0.0279469677222311,
                                         0.000683599007658395,
                                         1.68245305232397e-06,
                                         8.95497126493507e-11,
                                         3.28155370604677e-19,
                                         0.00228512011469059,
                                         0.1176116952208,
                                         0.247136519222332,
                                         0.0771332015129853,
                                         0.00396291485943565,
                                         2.16380356688241e-05,
                                         3.16506525304045e-09,
                                         6.47975139461908e-17,
                                         0.000127298384102934,
                                         0.0366036777637336,
                                         0.211375541300151,
                                         0.146359435088219,
                                         0.0157943138154778,
                                         0.000191322220571932,
                                         7.69084116491881e-08,
                                         8.79649932754605e-15,
                                         5.15743841004326e-06,
                                         0.00828507108345516,
                                         0.131483071787134,
                                         0.201974896027915,
                                         0.0457808746156233,
                                         0.00123029793528436,
                                         1.35913412883041e-06,
                                         8.68477768326672e-13,
                                         1.56713522764838e-07,
                                         0.001406465286787,
                                         0.0613403449859136,
                                         0.20904285364776,
                                         0.099524194516646,
                                         0.00593357506399473,
                                         1.80140787646642e-05,
                                         6.43085623603483e-11,
                                         3.62810280014245e-09,
                                         0.000181912475769038,
                                         0.0218033524155216,
                                         0.164844304045764,
                                         0.164844304045764,
                                         0.0218033524155216,
                                         0.000181912475769038,
                                         3.62810280014245e-09,
                                         6.43085623603483e-11,
                                         1.80140787646642e-05,
                                         0.00593357506399473,
                                         0.0995241945166462,
                                         0.20904285364776,
                                         0.0613403449859136,
                                         0.001406465286787,
                                         1.56713522764838e-07,
                                         8.68477768326672e-13,
                                         1.35913412883041e-06,
                                         0.00123029793528436,
                                         0.0457808746156234,
                                         0.201974896027915,
                                         0.131483071787134,
                                         0.00828507108345516,
                                         5.15743841004326e-06,
                                         8.79649932754605e-15,
                                         7.69084116491881e-08,
                                         0.000191322220571932,
                                         0.0157943138154779,
                                         0.146359435088219,
                                         0.211375541300151,
                                         0.0366036777637336,
                                         0.000127298384102934,
                                         6.47975139461908e-17,
                                         3.16506525304045e-09,
                                         2.16380356688241e-05,
                                         0.00396291485943566,
                                         0.0771332015129853,
                                         0.247136519222332,
                                         0.1176116952208,
                                         0.00228512011469059,
                                         3.28155370604677e-19,
                                         8.95497126493507e-11,
                                         1.68245305232397e-06,
                                         0.000683599007658396,
                                         0.0279469677222311,
                                         0.198651487281298,
                                         0.259805893621451,
                                         0.0282012187983436,
                                         1.02269780426944e-21,
                                         1.55916601435049e-12,
                                         8.05034905015786e-08,
                                         7.25662599136731e-05,
                                         0.00623124265394479,
                                         0.0982637538480874,
                                         0.353178389843839,
                                         0.214177259647798,
                                         1.47979123861944e-24,
                                         1.26039259762581e-14,
                                         1.78842935111527e-09,
                                         3.57646004274255e-06,
                                         0.000645059766382991,
                                         0.0225673311590701,
                                         0.222907441326673,
                                         0.755203785209844,

                                         0.798778591818839,
                                         0.301926026421539,
                                         0.0494152662784372,
                                         0.00311297419997282,
                                         6.103515625e-05,
                                         2.52670197270881e-07,
                                         1.01787681567065e-10,
                                         6.19784275326344e-16,
                                         6.71671856426609e-26,
                                         0.180910506955644,
                                         0.377493967469701,
                                         0.165786343450741,
                                         0.0222390538705175,
                                         0.0008544921875,
                                         6.93217160757057e-06,
                                         5.94653091217474e-09,
                                         9.71599226590548e-14,
                                         5.81267242567495e-23,
                                         0.0190233275353167,
                                         0.219131494741148,
                                         0.258238914149295,
                                         0.0737636525244812,
                                         0.00555419921875,
                                         8.83018722636004e-05,
                                         1.61293720659385e-07,
                                         7.07162281584978e-12,
                                         2.33549997700305e-20,
                                         0.00123099365658153,
                                         0.0782791716309913,
                                         0.247537645392351,
                                         0.150561923540369,
                                         0.022216796875,
                                         0.000692176982839029,
                                         2.69226542677398e-06,
                                         3.16736162756075e-10,
                                         5.77471453682488e-18,
                                         5.47643386029536e-05,
                                         0.0192247341896237,
                                         0.163129883757714,
                                         0.211281075170438,
                                         0.06109619140625,
                                         0.00373024277132601,
                                         3.08951985561955e-05,
                                         9.75324011111777e-09,
                                         9.81643682862212e-16,
                                         1.77189172877258e-06,
                                         0.00343377447629657,
                                         0.0781849386478743,
                                         0.215627100859607,
                                         0.1221923828125,
                                         0.0146202346593838,
                                         0.000257846599936199,
                                         2.18422555025854e-07,
                                         1.2135971100888e-13,
                                         4.29969626939405e-08,
                                         0.000459985832746768,
                                         0.0281043752862485,
                                         0.16504689282137,
                                         0.18328857421875,
                                         0.0429766789855871,
                                         0.00161396120284827,
                                         3.66865872267714e-06,
                                         1.12526925858796e-11,
                                         7.9494862640545e-10,
                                         4.69480795302418e-05,
                                         0.00769707042853126,
                                         0.0962525069000398,
                                         0.20947265625,
                                         0.0962525069000398,
                                         0.00769707042853127,
                                         4.69480795302418e-05,
                                         7.94948626405469e-10,
                                         1.12526925858793e-11,
                                         3.66865872267714e-06,
                                         0.00161396120284827,
                                         0.0429766789855871,
                                         0.18328857421875,
                                         0.16504689282137,
                                         0.0281043752862486,
                                         0.000459985832746768,
                                         4.29969626939414e-08,
                                         1.21359711008876e-13,
                                         2.18422555025854e-07,
                                         0.000257846599936198,
                                         0.0146202346593838,
                                         0.1221923828125,
                                         0.215627100859607,
                                         0.0781849386478744,
                                         0.00343377447629657,
                                         1.77189172877261e-06,
                                         9.81643682862178e-16,
                                         9.75324011111777e-09,
                                         3.08951985561954e-05,
                                         0.00373024277132601,
                                         0.06109619140625,
                                         0.211281075170438,
                                         0.163129883757714,
                                         0.0192247341896237,
                                         5.47643386029543e-05,
                                         5.77471453682466e-18,
                                         3.16736162756075e-10,
                                         2.69226542677398e-06,
                                         0.000692176982839029,
                                         0.022216796875,
                                         0.150561923540369,
                                         0.247537645392351,
                                         0.0782791716309913,
                                         0.00123099365658154,
                                         2.33549997700296e-20,
                                         7.07162281584978e-12,
                                         1.61293720659384e-07,
                                         8.83018722636003e-05,
                                         0.00555419921875,
                                         0.0737636525244812,
                                         0.258238914149295,
                                         0.219131494741148,
                                         0.0190233275353168,
                                         5.81267242567468e-23,
                                         9.71599226590548e-14,
                                         5.94653091217472e-09,
                                         6.93217160757057e-06,
                                         0.0008544921875,
                                         0.0222390538705175,
                                         0.165786343450741,
                                         0.377493967469701,
                                         0.180910506955645,
                                         6.71671856426576e-26,
                                         6.19784275326344e-16,
                                         1.01787681567064e-10,
                                         2.52670197270881e-07,
                                         6.103515625e-05,
                                         0.00311297419997282,
                                         0.0494152662784372,
                                         0.301926026421539,
                                         0.798778591818839,

                                         0.832055371830454,
                                         0.376088753199992,
                                         0.0866508417056446,
                                         0.00943417929311573,
                                         0.000425986140971128,
                                         6.38986441967056e-06,
                                         2.14536636845644e-08,
                                         7.39414306341226e-12,
                                         4.04947054273509e-17,
                                         4.1402457799e-27,
                                         0.153987526383138,
                                         0.380938318370657,
                                         0.231576643265897,
                                         0.0522091060622664,
                                         0.00441819277290271,
                                         0.00012075316532657,
                                         7.59828200329421e-07,
                                         5.422779576375e-10,
                                         7.83592014250648e-15,
                                         4.38479082861165e-24,
                                         0.0132313506020735,
                                         0.17914483665173,
                                         0.287343941195872,
                                         0.134144759044725,
                                         0.0212754704834807,
                                         0.00105947502908717,
                                         1.24943788032778e-05,
                                         1.84646625551596e-08,
                                         7.03990955477805e-13,
                                         2.15604072133849e-21,
                                         0.000699631648462836,
                                         0.0518442462912141,
                                         0.219409773276324,
                                         0.212103481396592,
                                         0.0630464038209102,
                                         0.005720441666645,
                                         0.000126433035266993,
                                         3.86907753996493e-07,
                                         3.89216096274087e-11,
                                         6.52396574890337e-19,
                                         2.54335755319419e-05,
                                         0.0103150072453746,
                                         0.115181464349172,
                                         0.23056563975652,
                                         0.128444102494088,
                                         0.0212344540688766,
                                         0.0008795877292282,
                                         5.57373342936002e-06,
                                         1.47940542827735e-09,
                                         1.35718511537103e-16,
                                         6.72423205720508e-07,
                                         0.00149257379146377,
                                         0.0439750684724893,
                                         0.182279856593151,
                                         0.190311625788588,
                                         0.0573257757328737,
                                         0.00445035916314419,
                                         5.83958866317397e-05,
                                         4.0896008756959e-08,
                                         2.05335389076169e-14,
                                         1.3333348481329e-08,
                                         0.000161980728900856,
                                         0.012591913061405,
                                         0.10807967577489,
                                         0.2114841059672,
                                         0.11607001595319,
                                         0.0168877668675188,
                                         0.000458859346978836,
                                         8.47882957038731e-07,
                                         2.32996708755175e-12,
                                         2.01435706064866e-10,
                                         1.33934226637478e-05,
                                         0.00274711978406668,
                                         0.0488258796229144,
                                         0.179056807051441,
                                         0.179056807051441,
                                         0.0488258796229144,
                                         0.00274711978406668,
                                         1.33934226637477e-05,
                                         2.01435706064866e-10,
                                         2.32996708755175e-12,
                                         8.47882957038736e-07,
                                         0.000458859346978836,
                                         0.0168877668675188,
                                         0.11607001595319,
                                         0.2114841059672,
                                         0.10807967577489,
                                         0.012591913061405,
                                         0.000161980728900855,
                                         1.3333348481329e-08,
                                         2.05335389076169e-14,
                                         4.08960087569593e-08,
                                         5.83958866317397e-05,
                                         0.00445035916314419,
                                         0.0573257757328737,
                                         0.190311625788588,
                                         0.182279856593151,
                                         0.0439750684724893,
                                         0.00149257379146376,
                                         6.72423205720508e-07,
                                         1.35718511537103e-16,
                                         1.47940542827736e-09,
                                         5.57373342936001e-06,
                                         0.0008795877292282,
                                         0.0212344540688766,
                                         0.128444102494088,
                                         0.23056563975652,
                                         0.115181464349172,
                                         0.0103150072453746,
                                         2.54335755319419e-05,
                                         6.52396574890337e-19,
                                         3.89216096274091e-11,
                                         3.86907753996493e-07,
                                         0.000126433035266993,
                                         0.005720441666645,
                                         0.0630464038209102,
                                         0.212103481396592,
                                         0.219409773276324,
                                         0.051844246291214,
                                         0.000699631648462836,
                                         2.15604072133849e-21,
                                         7.03990955477812e-13,
                                         1.84646625551596e-08,
                                         1.24943788032778e-05,
                                         0.00105947502908717,
                                         0.0212754704834807,
                                         0.134144759044725,
                                         0.287343941195872,
                                         0.17914483665173,
                                         0.0132313506020735,
                                         4.38479082861165e-24,
                                         7.83592014250656e-15,
                                         5.422779576375e-10,
                                         7.59828200329421e-07,
                                         0.00012075316532657,
                                         0.00441819277290271,
                                         0.0522091060622664,
                                         0.231576643265897,
                                         0.380938318370657,
                                         0.153987526383138,
                                         4.1402457799e-27,
                                         4.04947054273513e-17,
                                         7.39414306341226e-12,
                                         2.14536636845644e-08,
                                         6.38986441967056e-06,
                                         0.000425986140971128,
                                         0.00943417929311573,
                                         0.0866508417056446,
                                         0.376088753199992,
                                         0.832055371830454,

                                         0.857928130454034,
                                         0.443188377944622,
                                         0.131449732890717,
                                         0.0212703022054824,
                                         0.00172442623128594,
                                         6.103515625e-05,
                                         7.51456658108843e-07,
                                         2.15971218627114e-09,
                                         6.62595963081957e-13,
                                         3.35203904138456e-18,
                                         3.28074760796553e-28,
                                         0.13218666685593,
                                         0.37133670347712,
                                         0.287026951064134,
                                         0.0942702614643467,
                                         0.013890559856191,
                                         0.0008544921875,
                                         1.82845753462271e-05,
                                         9.55104516779122e-08,
                                         5.94760184002975e-11,
                                         7.8412558576333e-16,
                                         4.17342053350033e-25,
                                         0.00945604905440013,
                                         0.144455044783647,
                                         0.290985245497836,
                                         0.193981844194844,
                                         0.0519493599219006,
                                         0.00555419921875,
                                         0.000206562338235948,
                                         1.96106161886809e-06,
                                         2.47867912746694e-09,
                                         8.51623236584996e-14,
                                         2.46488576690126e-22,
                                         0.000416273165800634,
                                         0.0345815283716681,
                                         0.181537309212508,
                                         0.245637173167206,
                                         0.119560379876591,
                                         0.022216796875,
                                         0.00143603173779204,
                                         2.47786827703842e-05,
                                         6.35689981344407e-08,
                                         5.6918836649438e-12,
                                         8.95876367355328e-20,
                                         1.2598528355082e-05,
                                         0.00569152117940367,
                                         0.0778634283451988,
                                         0.213845344654611,
                                         0.189176451157166,
                                         0.06109619140625,
                                         0.00686356321810926,
                                         0.000215247389896108,
                                         1.12083872286568e-06,
                                         2.61539464356363e-10,
                                         2.23857714827856e-17,
                                         2.77305524359494e-07,
                                         0.000681255201015239,
                                         0.0242883745594472,
                                         0.135395057650766,
                                         0.217692849325377,
                                         0.1221923828125,
                                         0.0238579179043968,
                                         0.00135986211245809,
                                         1.43726942897347e-05,
                                         8.74008681865951e-09,
                                         4.06811624005937e-15,
                                         4.57781764303891e-09,
                                         6.11579006228358e-05,
                                         0.0056823063594564,
                                         0.0642935026217052,
                                         0.187880850222856,
                                         0.18328857421875,
                                         0.0621980407965562,
                                         0.00644337069240951,
                                         0.00013822751899891,
                                         2.19056188477223e-07,
                                         5.54467256869879e-13,
                                         5.75783404966602e-11,
                                         4.18307830886137e-06,
                                         0.00101286491087693,
                                         0.0232611964317222,
                                         0.123543971381537,
                                         0.20947265625,
                                         0.123543971381537,
                                         0.0232611964317222,
                                         0.00101286491087693,
                                         4.18307830886137e-06,
                                         5.75783404966623e-11,
                                         5.54467256869856e-13,
                                         2.19056188477223e-07,
                                         0.00013822751899891,
                                         0.00644337069240952,
                                         0.0621980407965561,
                                         0.18328857421875,
                                         0.187880850222856,
                                         0.0642935026217051,
                                         0.00568230635945639,
                                         6.11579006228358e-05,
                                         4.57781764303905e-09,
                                         4.06811624005918e-15,
                                         8.74008681865951e-09,
                                         1.43726942897348e-05,
                                         0.00135986211245809,
                                         0.0238579179043967,
                                         0.1221923828125,
                                         0.217692849325377,
                                         0.135395057650766,
                                         0.0242883745594471,
                                         0.000681255201015239,
                                         2.77305524359501e-07,
                                         2.23857714827844e-17,
                                         2.61539464356363e-10,
                                         1.12083872286569e-06,
                                         0.000215247389896109,
                                         0.00686356321810924,
                                         0.06109619140625,
                                         0.189176451157166,
                                         0.213845344654611,
                                         0.0778634283451987,
                                         0.00569152117940367,
                                         1.25985283550823e-05,
                                         8.95876367355278e-20,
                                         5.6918836649438e-12,
                                         6.3568998134441e-08,
                                         2.47786827703843e-05,
                                         0.00143603173779204,
                                         0.022216796875,
                                         0.119560379876591,
                                         0.245637173167206,
                                         0.181537309212507,
                                         0.0345815283716681,
                                         0.000416273165800641,
                                         2.46488576690111e-22,
                                         8.51623236584996e-14,
                                         2.47867912746695e-09,
                                         1.9610616188681e-06,
                                         0.000206562338235948,
                                         0.00555419921875,
                                         0.0519493599219006,
                                         0.193981844194844,
                                         0.290985245497836,
                                         0.144455044783647,
                                         0.00945604905440023,
                                         4.17342053350006e-25,
                                         7.8412558576333e-16,
                                         5.94760184002978e-11,
                                         9.55104516779125e-08,
                                         1.82845753462271e-05,
                                         0.0008544921875,
                                         0.013890559856191,
                                         0.0942702614643467,
                                         0.287026951064134,
                                         0.37133670347712,
                                         0.132186666855931,
                                         3.2807476079653e-28,
                                         3.35203904138456e-18,
                                         6.62595963081961e-13,
                                         2.15971218627114e-09,
                                         7.51456658108842e-07,
                                         6.103515625e-05,
                                         0.00172442623128594,
                                         0.0212703022054824,
                                         0.131449732890717,
                                         0.443188377944622,
                                         0.857928130454034,

                                         0.878381507555512,
                                         0.502680114717534,
                                         0.180663931533842,
                                         0.0393436874558342,
                                         0.0048981994983805,
                                         0.000318402678812608,
                                         9.37714643775355e-06,
                                         9.93665006278766e-08,
                                         2.53628531311558e-10,
                                         7.1177390060162e-14,
                                         3.38802337087253e-19,
                                         3.2064447747081e-29,
                                         0.114432617666772,
                                         0.354377797733909,
                                         0.328822614769851,
                                         0.143203866206315,
                                         0.0316931026334294,
                                         0.00346540756382134,
                                         0.000168868701334762,
                                         3.01000889800893e-06,
                                         1.3657603375595e-08,
                                         7.66492498754401e-12,
                                         9.41949607381663e-17,
                                         4.82406540281572e-26,
                                         0.00692152516363609,
                                         0.115991622991616,
                                         0.277867159177687,
                                         0.242002440628356,
                                         0.0952090793277899,
                                         0.0175112539395384,
                                         0.00141192929700039,
                                         4.23331791236656e-05,
                                         3.41457146712934e-07,
                                         3.83229629920415e-10,
                                         1.21588993201425e-14,
                                         3.36967434110781e-23,
                                         0.000257632356753678,
                                         0.0233632566760663,
                                         0.144497188886652,
                                         0.251669961891222,
                                         0.176010513003963,
                                         0.054453626440559,
                                         0.00726479511631774,
                                         0.000366387479074446,
                                         5.25344938290193e-06,
                                         1.17911695749025e-08,
                                         9.65845120302771e-13,
                                         1.44846934843893e-20,
                                         6.5928275059489e-06,
                                         0.00323528718754075,
                                         0.0516599852599716,
                                         0.17993503110253,
                                         0.223702869235992,
                                         0.116415034691496,
                                         0.0256984242295356,
                                         0.00218008318713523,
                                         5.55681067024691e-05,
                                         2.49417770701664e-07,
                                         5.27464724106455e-11,
                                         4.28058761226072e-18,
                                         1.22698801772166e-07,
                                         0.000325828896233164,
                                         0.0134321786534304,
                                         0.0935615407491133,
                                         0.206776796145285,
                                         0.181004194018403,
                                         0.0661130083625357,
                                         0.00943415070214429,
                                         0.000427468334863318,
                                         3.83703007247667e-06,
                                         2.09496442275046e-09,
                                         9.20014713889037e-16,
                                         1.71265620965444e-09,
                                         2.46109070384926e-05,
                                         0.00261938842708312,
                                         0.0364871775678298,
                                         0.14334855282635,
                                         0.211071437244382,
                                         0.127564140772914,
                                         0.0306191983851745,
                                         0.00246628670861344,
                                         4.42715039981073e-05,
                                         6.24052528824334e-08,
                                         1.48302140461032e-13,
                                         1.82138079674047e-11,
                                         1.41633611940769e-06,
                                         0.000389183157505752,
                                         0.0108413624308572,
                                         0.0757156250207597,
                                         0.187530099796912,
                                         0.187530099796912,
                                         0.0757156250207598,
                                         0.0108413624308572,
                                         0.000389183157505753,
                                         1.41633611940771e-06,
                                         1.82138079674039e-11,
                                         1.4830214046104e-13,
                                         6.24052528824328e-08,
                                         4.42715039981072e-05,
                                         0.00246628670861344,
                                         0.0306191983851745,
                                         0.127564140772914,
                                         0.211071437244382,
                                         0.14334855282635,
                                         0.0364871775678299,
                                         0.00261938842708313,
                                         2.46109070384928e-05,
                                         1.71265620965438e-09,
                                         9.20014713889087e-16,
                                         2.09496442275044e-09,
                                         3.83703007247665e-06,
                                         0.000427468334863317,
                                         0.00943415070214427,
                                         0.0661130083625357,
                                         0.181004194018403,
                                         0.206776796145285,
                                         0.0935615407491135,
                                         0.0134321786534305,
                                         0.000325828896233166,
                                         1.22698801772162e-07,
                                         4.28058761226098e-18,
                                         5.27464724106449e-11,
                                         2.49417770701663e-07,
                                         5.55681067024689e-05,
                                         0.00218008318713523,
                                         0.0256984242295356,
                                         0.116415034691496,
                                         0.223702869235992,
                                         0.17993503110253,
                                         0.0516599852599717,
                                         0.00323528718754076,
                                         6.59282750594874e-06,
                                         1.44846934843903e-20,
                                         9.65845120302759e-13,
                                         1.17911695749025e-08,
                                         5.25344938290191e-06,
                                         0.000366387479074445,
                                         0.00726479511631773,
                                         0.054453626440559,
                                         0.176010513003963,
                                         0.251669961891223,
                                         0.144497188886652,
                                         0.0233632566760664,
                                         0.000257632356753674,
                                         3.36967434110805e-23,
                                         1.21588993201423e-14,
                                         3.83229629920413e-10,
                                         3.41457146712933e-07,
                                         4.23331791236655e-05,
                                         0.00141192929700039,
                                         0.0175112539395384,
                                         0.0952090793277899,
                                         0.242002440628356,
                                         0.277867159177687,
                                         0.115991622991616,
                                         0.00692152516363601,
                                         4.8240654028161e-26,
                                         9.41949607381649e-17,
                                         7.66492498754396e-12,
                                         1.36576033755949e-08,
                                         3.01000889800893e-06,
                                         0.000168868701334762,
                                         0.00346540756382134,
                                         0.0316931026334294,
                                         0.143203866206315,
                                         0.328822614769851,
                                         0.354377797733909,
                                         0.114432617666772,
                                         3.20644477470837e-29,
                                         3.38802337087247e-19,
                                         7.11773900601615e-14,
                                         2.53628531311557e-10,
                                         9.93665006278764e-08,
                                         9.37714643775355e-06,
                                         0.000318402678812608,
                                         0.0048981994983805,
                                         0.0393436874558342,
                                         0.180663931533842,
                                         0.502680114717534,
                                         0.878381507555512,

                                         0.894796691735381,
                                         0.55486493409018,
                                         0.231591888542764,
                                         0.0634012211540223,
                                         0.0109246775282168,
                                         0.00111305529849408,
                                         6.103515625e-05,
                                         1.55894269820969e-06,
                                         1.46993189247965e-08,
                                         3.41984562205055e-11,
                                         8.95024244438722e-15,
                                         4.06143157906701e-20,
                                         3.74366200153977e-30,
                                         0.0998603953829449,
                                         0.333805359472724,
                                         0.357098406693438,
                                         0.193294345569945,
                                         0.0582333213826963,
                                         0.00974563228046152,
                                         0.0008544921875,
                                         3.48973897793032e-05,
                                         5.40493343049544e-07,
                                         2.19857378856876e-09,
                                         1.13769618767461e-12,
                                         1.32321125676991e-17,
                                         6.57481884361709e-27,
                                         0.0051742512555352,
                                         0.0932362187346985,
                                         0.255645293483706,
                                         0.273606239945348,
                                         0.144118527711103,
                                         0.0396176417921838,
                                         0.00555419921875,
                                         0.000362694572978702,
                                         9.22717605362969e-06,
                                         6.56237299384739e-08,
                                         6.71433687227481e-11,
                                         2.00154128867384e-15,
                                         5.36112918038763e-24,
                                         0.000164986489097267,
                                         0.0160259075669835,
                                         0.112624875910726,
                                         0.238330421183217,
                                         0.219489984152853,
                                         0.0991091738332565,
                                         0.022216796875,
                                         0.00231972127164159,
                                         9.69379651951082e-05,
                                         1.20538952001276e-06,
                                         2.43852148134204e-09,
                                         1.86314456755551e-13,
                                         2.69014313057874e-21,
                                         3.61677881788639e-06,
                                         0.00189379653445038,
                                         0.0341117147883907,
                                         0.142726844972512,
                                         0.229817078776259,
                                         0.1704558039506,
                                         0.06109619140625,
                                         0.0102000724178617,
                                         0.000700150968841022,
                                         1.52218211113449e-05,
                                         6.08867435542366e-08,
                                         1.19234314237615e-11,
                                         9.28041090021405e-19,
                                         5.76624142832349e-08,
                                         0.000162757609776814,
                                         0.00751398078327899,
                                         0.0621625973940599,
                                         0.175003685437906,
                                         0.213209736122664,
                                         0.1221923828125,
                                         0.0326188020483393,
                                         0.00367778884104708,
                                         0.000139798695230572,
                                         1.10564628280013e-06,
                                         5.54949243603187e-10,
                                         2.32839321644118e-16,
                                         6.89485213581809e-10,
                                         1.04908469805401e-05,
                                         0.00124136035585715,
                                         0.0203055100576219,
                                         0.0999478261578619,
                                         0.20001544619157,
                                         0.18328857421875,
                                         0.0782337274296374,
                                         0.0144891580829924,
                                         0.000962943676969326,
                                         1.50581263421586e-05,
                                         1.93716463845729e-08,
                                         4.38133749841099e-14,
                                         6.28141922958836e-12,
                                         5.15205498743238e-07,
                                         0.000156252275658675,
                                         0.00505358228394179,
                                         0.0434910806263117,
                                         0.142962036457835,
                                         0.20947265625,
                                         0.142962036457835,
                                         0.0434910806263117,
                                         0.00505358228394178,
                                         0.000156252275658676,
                                         5.15205498743233e-07,
                                         6.28141922958805e-12,
                                         4.38133749841124e-14,
                                         1.93716463845731e-08,
                                         1.50581263421586e-05,
                                         0.000962943676969329,
                                         0.0144891580829924,
                                         0.0782337274296374,
                                         0.18328857421875,
                                         0.20001544619157,
                                         0.0999478261578619,
                                         0.0203055100576219,
                                         0.00124136035585716,
                                         1.04908469805401e-05,
                                         6.8948521358178e-10,
                                         2.32839321644133e-16,
                                         5.54949243603194e-10,
                                         1.10564628280012e-06,
                                         0.000139798695230572,
                                         0.00367778884104708,
                                         0.0326188020483393,
                                         0.1221923828125,
                                         0.213209736122664,
                                         0.175003685437906,
                                         0.0621625973940598,
                                         0.00751398078327901,
                                         0.000162757609776813,
                                         5.76624142832329e-08,
                                         9.2804109002147e-19,
                                         1.19234314237617e-11,
                                         6.08867435542363e-08,
                                         1.5221821111345e-05,
                                         0.000700150968841022,
                                         0.0102000724178617,
                                         0.06109619140625,
                                         0.1704558039506,
                                         0.229817078776259,
                                         0.142726844972512,
                                         0.0341117147883908,
                                         0.00189379653445037,
                                         3.61677881788629e-06,
                                         2.69014313057895e-21,
                                         1.86314456755554e-13,
                                         2.43852148134203e-09,
                                         1.20538952001277e-06,
                                         9.69379651951082e-05,
                                         0.00231972127164159,
                                         0.022216796875,
                                         0.0991091738332565,
                                         0.219489984152853,
                                         0.238330421183217,
                                         0.112624875910726,
                                         0.0160259075669834,
                                         0.000164986489097263,
                                         5.36112918038809e-24,
                                         2.00154128867387e-15,
                                         6.71433687227477e-11,
                                         6.56237299384741e-08,
                                         9.22717605362969e-06,
                                         0.000362694572978702,
                                         0.00555419921875,
                                         0.0396176417921838,
                                         0.144118527711103,
                                         0.273606239945348,
                                         0.255645293483707,
                                         0.0932362187346983,
                                         0.00517425125553513,
                                         6.57481884361769e-27,
                                         1.32321125676994e-17,
                                         1.1376961876746e-12,
                                         2.19857378856877e-09,
                                         5.40493343049544e-07,
                                         3.48973897793032e-05,
                                         0.0008544921875,
                                         0.00974563228046152,
                                         0.0582333213826963,
                                         0.193294345569945,
                                         0.357098406693438,
                                         0.333805359472724,
                                         0.0998603953829442,
                                         3.74366200154013e-30,
                                         4.06143157906709e-20,
                                         8.95024244438715e-15,
                                         3.41984562205057e-11,
                                         1.46993189247965e-08,
                                         1.55894269820969e-06,
                                         6.103515625e-05,
                                         0.00111305529849408,
                                         0.0109246775282168,
                                         0.0634012211540223,
                                         0.231591888542764,
                                         0.55486493409018,
                                         0.894796691735381,

                                         0.908151479620035,
                                         0.600412555007255,
                                         0.282221209395259,
                                         0.0925223776099018,
                                         0.0205283803857599,
                                         0.00294798423862868,
                                         0.000256707305613961,
                                         1.23115707340453e-05,
                                         2.80954692054026e-07,
                                         2.4146078770716e-09,
                                         5.21805142702241e-12,
                                         1.29165379004665e-15,
                                         5.64156249151391e-21,
                                         5.09154436705323e-31,
                                         0.0877967746657507,
                                         0.31194225222809,
                                         0.373655791603252,
                                         0.240061256725032,
                                         0.0919428284257169,
                                         0.0213032981324621,
                                         0.00289296424871548,
                                         0.00021412367950053,
                                         7.62027549741992e-06,
                                         1.05667032513721e-07,
                                         3.94175053886687e-10,
                                         1.91214192783778e-13,
                                         2.1282911353352e-18,
                                         1.03225105859118e-27,
                                         0.00394079777874479,
                                         0.0752460943537308,
                                         0.229688424409013,
                                         0.289189610934668,
                                         0.19119051983656,
                                         0.0714749490014689,
                                         0.0151367704854538,
                                         0.00172902491622537,
                                         9.59600304509441e-05,
                                         2.14692923644401e-06,
                                         1.3824684768651e-08,
                                         1.31425442286282e-11,
                                         3.72776039605788e-16,
                                         9.71642409517586e-25,
                                         0.00010885198979879,
                                         0.0111696703723928,
                                         0.0868866661910427,
                                         0.214382797287023,
                                         0.244659161492912,
                                         0.14757320104932,
                                         0.0487382599312558,
                                         0.00859180556957072,
                                         0.000743629681691965,
                                         2.68437128134232e-05,
                                         2.98378807265064e-07,
                                         5.55885575346995e-10,
                                         4.01801523071852e-14,
                                         5.62826091648157e-22,
                                         2.0670990975752e-06,
                                         0.00113990708612021,
                                         0.022596431609853,
                                         0.109262185254667,
                                         0.215243137738155,
                                         0.209475793349683,
                                         0.10788958342164,
                                         0.0293521779522367,
                                         0.00396182928314086,
                                         0.000230749166040097,
                                         4.42744742132782e-06,
                                         1.61645665906495e-08,
                                         2.97747325922803e-12,
                                         2.24137583288574e-19,
                                         2.85485151123402e-08,
                                         8.46049653239323e-05,
                                         0.00427389542038448,
                                         0.040499264197081,
                                         0.137719186184382,
                                         0.216250685021987,
                                         0.173694599047134,
                                         0.0729278692407769,
                                         0.0153508384422931,
                                         0.00144256369515391,
                                         4.77789006733888e-05,
                                         3.41853496673221e-07,
                                         1.60465422043847e-10,
                                         6.49161042982902e-17,
                                         2.95710683177539e-10,
                                         4.70959447788862e-06,
                                         0.000606274335028809,
                                         0.0112586325956371,
                                         0.0660877314461528,
                                         0.167433518300658,
                                         0.209727015213131,
                                         0.13589640913498,
                                         0.0446097416193879,
                                         0.00676380564106074,
                                         0.000386705329106921,
                                         5.42222146194018e-06,
                                         6.48599066135342e-09,
                                         1.41010507991454e-14,
                                         2.33373313699857e-12,
                                         1.99743135630834e-07,
                                         6.552622742045e-05,
                                         0.00238465111590235,
                                         0.0241628372644157,
                                         0.0987706706186377,
                                         0.192940378283031,
                                         0.192940378283031,
                                         0.0987706706186377,
                                         0.0241628372644158,
                                         0.00238465111590236,
                                         6.552622742045e-05,
                                         1.99743135630832e-07,
                                         2.33373313699857e-12,
                                         1.41010507991454e-14,
                                         6.4859906613535e-09,
                                         5.42222146194018e-06,
                                         0.00038670532910692,
                                         0.00676380564106072,
                                         0.0446097416193879,
                                         0.13589640913498,
                                         0.209727015213131,
                                         0.167433518300658,
                                         0.0660877314461529,
                                         0.0112586325956371,
                                         0.000606274335028809,
                                         4.70959447788858e-06,
                                         2.95710683177539e-10,
                                         6.49161042982902e-17,
                                         1.60465422043849e-10,
                                         3.41853496673221e-07,
                                         4.77789006733886e-05,
                                         0.0014425636951539,
                                         0.0153508384422931,
                                         0.0729278692407768,
                                         0.173694599047135,
                                         0.216250685021987,
                                         0.137719186184383,
                                         0.040499264197081,
                                         0.00427389542038448,
                                         8.46049653239316e-05,
                                         2.85485151123402e-08,
                                         2.24137583288574e-19,
                                         2.97747325922808e-12,
                                         1.61645665906495e-08,
                                         4.4274474213278e-06,
                                         0.000230749166040096,
                                         0.00396182928314086,
                                         0.0293521779522366,
                                         0.10788958342164,
                                         0.209475793349683,
                                         0.215243137738155,
                                         0.109262185254667,
                                         0.022596431609853,
                                         0.0011399070861202,
                                         2.0670990975752e-06,
                                         5.62826091648157e-22,
                                         4.01801523071859e-14,
                                         5.55885575346995e-10,
                                         2.98378807265063e-07,
                                         2.68437128134231e-05,
                                         0.000743629681691965,
                                         0.0085918055695707,
                                         0.0487382599312558,
                                         0.14757320104932,
                                         0.244659161492912,
                                         0.214382797287024,
                                         0.0868866661910427,
                                         0.0111696703723927,
                                         0.00010885198979879,
                                         9.71642409517586e-25,
                                         3.72776039605795e-16,
                                         1.31425442286282e-11,
                                         1.3824684768651e-08,
                                         2.14692923644401e-06,
                                         9.59600304509441e-05,
                                         0.00172902491622537,
                                         0.0151367704854538,
                                         0.0714749490014689,
                                         0.19119051983656,
                                         0.289189610934668,
                                         0.229688424409013,
                                         0.0752460943537306,
                                         0.00394079777874479,
                                         1.03225105859118e-27,
                                         2.12829113533524e-18,
                                         1.91214192783778e-13,
                                         3.94175053886685e-10,
                                         1.05667032513721e-07,
                                         7.62027549741992e-06,
                                         0.000214123679500529,
                                         0.00289296424871549,
                                         0.0213032981324621,
                                         0.0919428284257169,
                                         0.240061256725032,
                                         0.373655791603252,
                                         0.31194225222809,
                                         0.0877967746657507,
                                         5.09154436705323e-31,
                                         5.64156249151403e-21,
                                         1.29165379004665e-15,
                                         5.21805142702238e-12,
                                         2.41460787707159e-09,
                                         2.80954692054026e-07,
                                         1.23115707340453e-05,
                                         0.000256707305613961,
                                         0.00294798423862868,
                                         0.0205283803857599,
                                         0.0925223776099018,
                                         0.282221209395259,
                                         0.600412555007255,
                                         0.908151479620035,

                                         0.919150226478255,
                                         0.640107406008909,
                                         0.331197645862801,
                                         0.125480077586222,
                                         0.0340360342536997,
                                         0.00639615498590068,
                                         0.000794629521388627,
                                         6.103515625e-05,
                                         2.62880154464441e-06,
                                         5.47870747824053e-08,
                                         4.36894452245664e-10,
                                         8.89401494199471e-13,
                                         2.10436547567358e-16,
                                         8.91056552012454e-22,
                                         7.90509821684819e-32,
                                         0.0777233895661342,
                                         0.290162827455718,
                                         0.380818203746672,
                                         0.280744725306507,
                                         0.130131955975357,
                                         0.0389135845709989,
                                         0.00739811043958409,
                                         0.0008544921875,
                                         5.53423705606815e-05,
                                         1.76502932273007e-06,
                                         2.2396883750532e-08,
                                         7.79142867301218e-11,
                                         3.58712722777738e-14,
                                         3.8527640845995e-19,
                                         1.83230643948252e-28,
                                         0.00305142209662564,
                                         0.0610683263803825,
                                         0.203298175092008,
                                         0.291631044416046,
                                         0.231001225596779,
                                         0.109918000275887,
                                         0.0319788061084785,
                                         0.00555419921875,
                                         0.000540932450646046,
                                         2.64004334651852e-05,
                                         5.33069681527547e-07,
                                         3.16899580830958e-09,
                                         2.83894980392505e-12,
                                         7.73436897366052e-17,
                                         1.9718517788385e-25,
                                         7.37224018802843e-05,
                                         0.00790927901169573,
                                         0.0667876088325528,
                                         0.186424306278931,
                                         0.252342990335695,
                                         0.191065833816877,
                                         0.0850648843980017,
                                         0.022216796875,
                                         0.00325368082576747,
                                         0.000243005903876338,
                                         7.80776986735607e-06,
                                         7.93182027193064e-08,
                                         1.38265962660898e-10,
                                         9.55485258530112e-15,
                                         1.30586132535162e-22,
                                         1.22452982650064e-06,
                                         0.000704255872192209,
                                         0.0150845032201668,
                                         0.0819302010638312,
                                         0.18951383014246,
                                         0.228333667123227,
                                         0.15556471187529,
                                         0.06109619140625,
                                         0.0134548717608544,
                                         0.00153778384776077,
                                         7.8621753937022e-05,
                                         1.36488722088565e-06,
                                         4.62962220319222e-09,
                                         8.11513591651081e-13,
                                         5.94555711946987e-20,
                                         1.4792329137864e-08,
                                         4.56059345240437e-05,
                                         0.00247778392512853,
                                         0.0261868226806955,
                                         0.103511326584032,
                                         0.19845140189898,
                                         0.206904131867691,
                                         0.1221923828125,
                                         0.0404651802726637,
                                         0.00707735640750615,
                                         0.00057577891088188,
                                         1.70811840443783e-05,
                                         1.12738726446595e-07,
                                         5.01262143401763e-11,
                                         1.96872634731147e-17,
                                         1.34018582022746e-10,
                                         2.21499885120273e-06,
                                         0.000305251012745522,
                                         0.00627744415251643,
                                         0.0424029530844731,
                                         0.129359894924354,
                                         0.206389928994187,
                                         0.18328857421875,
                                         0.0912734905766504,
                                         0.0244291356967065,
                                         0.00316249642384487,
                                         0.000160324701499111,
                                         2.05902661431579e-06,
                                         2.32217678477062e-09,
                                         4.88921814144135e-15,
                                         9.25111678652816e-13,
                                         8.19646028970671e-08,
                                         2.8651771820609e-05,
                                         0.00114652517648595,
                                         0.013234423265513,
                                         0.0642459602980638,
                                         0.156858669736691,
                                         0.20947265625,
                                         0.156858669736691,
                                         0.0642459602980638,
                                         0.013234423265513,
                                         0.00114652517648594,
                                         2.86517718206092e-05,
                                         8.19646028970671e-08,
                                         9.25111678652876e-13,
                                         4.88921814144099e-15,
                                         2.32217678477062e-09,
                                         2.05902661431578e-06,
                                         0.000160324701499111,
                                         0.00316249642384487,
                                         0.0244291356967065,
                                         0.0912734905766504,
                                         0.18328857421875,
                                         0.206389928994187,
                                         0.129359894924354,
                                         0.0424029530844731,
                                         0.00627744415251641,
                                         0.000305251012745523,
                                         2.21499885120273e-06,
                                         1.34018582022754e-10,
                                         1.96872634731131e-17,
                                         5.01262143401763e-11,
                                         1.12738726446594e-07,
                                         1.70811840443783e-05,
                                         0.00057577891088188,
                                         0.00707735640750615,
                                         0.0404651802726638,
                                         0.1221923828125,
                                         0.206904131867691,
                                         0.19845140189898,
                                         0.103511326584032,
                                         0.0261868226806955,
                                         0.00247778392512854,
                                         4.56059345240437e-05,
                                         1.47923291378647e-08,
                                         5.94555711946932e-20,
                                         8.11513591651081e-13,
                                         4.62962220319218e-09,
                                         1.36488722088565e-06,
                                         7.8621753937022e-05,
                                         0.00153778384776077,
                                         0.0134548717608544,
                                         0.06109619140625,
                                         0.15556471187529,
                                         0.228333667123227,
                                         0.18951383014246,
                                         0.081930201063831,
                                         0.0150845032201669,
                                         0.000704255872192209,
                                         1.22452982650069e-06,
                                         1.30586132535149e-22,
                                         9.55485258530112e-15,
                                         1.38265962660897e-10,
                                         7.93182027193068e-08,
                                         7.80776986735607e-06,
                                         0.000243005903876338,
                                         0.00325368082576747,
                                         0.022216796875,
                                         0.0850648843980017,
                                         0.191065833816877,
                                         0.252342990335695,
                                         0.186424306278931,
                                         0.0667876088325529,
                                         0.00790927901169573,
                                         7.37224018802863e-05,
                                         1.97185177883828e-25,
                                         7.73436897366052e-17,
                                         2.83894980392502e-12,
                                         3.1689958083096e-09,
                                         5.33069681527547e-07,
                                         2.64004334651852e-05,
                                         0.000540932450646046,
                                         0.00555419921875,
                                         0.0319788061084785,
                                         0.109918000275887,
                                         0.231001225596779,
                                         0.291631044416045,
                                         0.203298175092008,
                                         0.0610683263803825,
                                         0.0030514220966257,
                                         1.8323064394823e-28,
                                         3.8527640845995e-19,
                                         3.58712722777735e-14,
                                         7.79142867301222e-11,
                                         2.2396883750532e-08,
                                         1.76502932273007e-06,
                                         5.53423705606815e-05,
                                         0.0008544921875,
                                         0.00739811043958409,
                                         0.0389135845709989,
                                         0.130131955975357,
                                         0.280744725306507,
                                         0.380818203746672,
                                         0.290162827455718,
                                         0.077723389566135,
                                         7.90509821684716e-32,
                                         8.91056552012454e-22,
                                         2.10436547567356e-16,
                                         8.89401494199476e-13,
                                         4.36894452245664e-10,
                                         5.47870747824053e-08,
                                         2.62880154464441e-06,
                                         6.103515625e-05,
                                         0.000794629521388627,
                                         0.00639615498590068,
                                         0.0340360342536997,
                                         0.125480077586222,
                                         0.331197645862801,
                                         0.640107406008909,
                                         0.919150226478255,

                                         0.92830888838185,
                                         0.674723607069868,
                                         0.377694981009437,
                                         0.161012890325409,
                                         0.0513823822686537,
                                         0.0119744441269631,
                                         0.0019684839971401,
                                         0.000217493955230645,
                                         1.50860701659776e-05,
                                         5.95221404252266e-07,
                                         1.15181268887491e-08,
                                         8.63846513621983e-11,
                                         1.67440617604449e-13,
                                         3.81731583436539e-17,
                                         1.57564129551466e-22,
                                         1.3782431505093e-32,
                                         0.0692413907916528,
                                         0.269237010010375,
                                         0.38083941004206,
                                         0.314094348731462,
                                         0.169902985674908,
                                         0.0623170259657261,
                                         0.0154479337876898,
                                         0.00251651035638298,
                                         0.00025555281183277,
                                         1.48660675094341e-05,
                                         4.33797028803672e-07,
                                         5.12042349060439e-09,
                                         1.6823541046888e-11,
                                         7.42016384690995e-15,
                                         7.73935151597904e-20,
                                         3.62166803784355e-29,
                                         0.00239786306060544,
                                         0.0498802858150853,
                                         0.178290365236005,
                                         0.284475533110945,
                                         0.260839348365772,
                                         0.150571714861163,
                                         0.0562852022622884,
                                         0.0135187200247,
                                         0.00200988183593633,
                                         0.000172384805109671,
                                         7.58536718505357e-06,
                                         1.40916062231118e-07,
                                         7.8480060280327e-10,
                                         6.69659782602804e-13,
                                         1.76496911982793e-17,
                                         4.41851932578541e-26,
                                         5.11010244610841e-05,
                                         0.00568682340891618,
                                         0.0513641966792451,
                                         0.158553689391729,
                                         0.246429012725624,
                                         0.223885882425191,
                                         0.126201552498238,
                                         0.044690895840724,
                                         0.00972762950900399,
                                         0.00123012288587223,
                                         8.1623136396677e-05,
                                         2.38650171520578e-06,
                                         2.25293093563193e-08,
                                         3.71913184492143e-11,
                                         2.47694455259606e-15,
                                         3.31735166114042e-23,
                                         7.48699492649584e-07,
                                         0.000445741688716653,
                                         0.010173393156695,
                                         0.0607547847773012,
                                         0.160060168650594,
                                         0.228866761409494,
                                         0.194539531862892,
                                         0.101572291243964,
                                         0.0323680264399718,
                                         0.00603490887988328,
                                         0.000603841074529163,
                                         2.77866189504756e-05,
                                         4.44640616423535e-07,
                                         1.42004345370365e-09,
                                         2.38983634064826e-13,
                                         1.71229649508867e-20,
                                         7.97779316851973e-09,
                                         2.54093799614624e-05,
                                         0.00146544140594491,
                                         0.0169309732357277,
                                         0.0756087407524717,
                                         0.170151601458504,
                                         0.218096319783743,
                                         0.16789151191806,
                                         0.0783290249993748,
                                         0.0215322908610623,
                                         0.00324884749714735,
                                         0.00023529189197864,
                                         6.38215998051512e-06,
                                         3.94329252483569e-08,
                                         1.67693928439437e-11,
                                         6.42782027589935e-18,
                                         6.37557369123275e-11,
                                         1.08634093417115e-06,
                                         0.000158318749790321,
                                         0.0035387071457789,
                                         0.0267868720542858,
                                         0.0948747449190294,
                                         0.183379199002865,
                                         0.20813422215535,
                                         0.142164278274685,
                                         0.0576197038290273,
                                         0.013109836145153,
                                         0.00149430579867132,
                                         6.87048672215038e-05,
                                         8.21254231474564e-07,
                                         8.82526550787453e-10,
                                         1.80971316669327e-15,
                                         3.88200825859944e-13,
                                         3.5386605885085e-08,
                                         1.30315757532109e-05,
                                         0.000563518282729281,
                                         0.00723057257350457,
                                         0.0403056462983625,
                                         0.117476904255276,
                                         0.19658887456462,
                                         0.19658887456462,
                                         0.117476904255276,
                                         0.0403056462983625,
                                         0.00723057257350457,
                                         0.000563518282729279,
                                         1.3031575753211e-05,
                                         3.5386605885085e-08,
                                         3.88200825859916e-13,
                                         1.80971316669343e-15,
                                         8.82526550787453e-10,
                                         8.21254231474559e-07,
                                         6.87048672215041e-05,
                                         0.00149430579867132,
                                         0.013109836145153,
                                         0.0576197038290273,
                                         0.142164278274685,
                                         0.20813422215535,
                                         0.183379199002865,
                                         0.0948747449190294,
                                         0.0267868720542858,
                                         0.00353870714577889,
                                         0.000158318749790322,
                                         1.08634093417115e-06,
                                         6.37557369123235e-11,
                                         6.42782027589995e-18,
                                         1.67693928439437e-11,
                                         3.94329252483566e-08,
                                         6.38215998051515e-06,
                                         0.00023529189197864,
                                         0.00324884749714735,
                                         0.0215322908610624,
                                         0.0783290249993747,
                                         0.16789151191806,
                                         0.218096319783743,
                                         0.170151601458504,
                                         0.0756087407524717,
                                         0.0169309732357276,
                                         0.00146544140594492,
                                         2.54093799614624e-05,
                                         7.97779316851931e-09,
                                         1.71229649508885e-20,
                                         2.38983634064826e-13,
                                         1.42004345370364e-09,
                                         4.44640616423537e-07,
                                         2.77866189504756e-05,
                                         0.000603841074529163,
                                         0.00603490887988329,
                                         0.0323680264399718,
                                         0.101572291243963,
                                         0.194539531862892,
                                         0.228866761409494,
                                         0.160060168650594,
                                         0.0607547847773011,
                                         0.010173393156695,
                                         0.000445741688716653,
                                         7.48699492649553e-07,
                                         3.3173516611408e-23,
                                         2.47694455259606e-15,
                                         3.7191318449214e-11,
                                         2.25293093563195e-08,
                                         2.38650171520578e-06,
                                         8.1623136396677e-05,
                                         0.00123012288587223,
                                         0.00972762950900399,
                                         0.0446908958407239,
                                         0.126201552498238,
                                         0.223885882425191,
                                         0.246429012725624,
                                         0.158553689391728,
                                         0.0513641966792452,
                                         0.00568682340891618,
                                         5.11010244610825e-05,
                                         4.41851932578596e-26,
                                         1.76496911982793e-17,
                                         6.69659782602797e-13,
                                         7.84800602803274e-10,
                                         1.40916062231118e-07,
                                         7.58536718505357e-06,
                                         0.000172384805109671,
                                         0.00200988183593632,
                                         0.0135187200247,
                                         0.0562852022622884,
                                         0.150571714861163,
                                         0.260839348365772,
                                         0.284475533110945,
                                         0.178290365236005,
                                         0.0498802858150853,
                                         0.00239786306060539,
                                         3.62166803784404e-29,
                                         7.73935151597904e-20,
                                         7.42016384690987e-15,
                                         1.68235410468881e-11,
                                         5.12042349060439e-09,
                                         4.33797028803672e-07,
                                         1.48660675094342e-05,
                                         0.00025555281183277,
                                         0.00251651035638297,
                                         0.0154479337876898,
                                         0.0623170259657261,
                                         0.169902985674908,
                                         0.314094348731462,
                                         0.380839410042061,
                                         0.269237010010375,
                                         0.069241390791652,
                                         1.3782431505095e-32,
                                         1.57564129551466e-22,
                                         3.81731583436535e-17,
                                         1.6744061760445e-13,
                                         8.63846513621983e-11,
                                         1.15181268887491e-08,
                                         5.95221404252267e-07,
                                         1.50860701659776e-05,
                                         0.000217493955230644,
                                         0.0019684839971401,
                                         0.0119744441269631,
                                         0.0513823822686537,
                                         0.161012890325409,
                                         0.377694981009437,
                                         0.674723607069868,
                                         0.92830888838185,

                                         0.936011420906696,
                                         0.704969843852267,
                                         0.421272732950795,
                                         0.1979836963965,
                                         0.0722033589957216,
                                         0.0200484740184243,
                                         0.00412855333974618,
                                         0.000608292785002365,
                                         6.103515625e-05,
                                         3.89227113016622e-06,
                                         1.42834843823859e-07,
                                         2.5992490219922e-09,
                                         1.85263094138757e-11,
                                         3.44754977375801e-14,
                                         7.62058037292775e-18,
                                         3.07932167296878e-23,
                                         2.66202695763694e-33,
                                         0.0620426002048534,
                                         0.249560611491814,
                                         0.375658439374046,
                                         0.339931827842335,
                                         0.208751952204349,
                                         0.0904199187691077,
                                         0.0277514492999648,
                                         0.0059365328292988,
                                         0.0008544921875,
                                         7.81697904671442e-05,
                                         4.1648797496989e-06,
                                         1.1295908611804e-07,
                                         1.25594852670226e-09,
                                         3.93553895209475e-12,
                                         1.6749981026431e-15,
                                         1.7049263726894e-20,
                                         7.87153947237832e-30,
                                         0.00190934388515173,
                                         0.0410172805650967,
                                         0.155527886640952,
                                         0.270981443328865,
                                         0.28021347127761,
                                         0.189335575464108,
                                         0.0866081493351363,
                                         0.0268991404967024,
                                         0.00555419921875,
                                         0.000728886877767901,
                                         5.63840269857797e-05,
                                         2.27918601780491e-06,
                                         3.95312083409993e-08,
                                         2.08585063697731e-10,
                                         1.70932998844085e-13,
                                         4.38269773867836e-18,
                                         1.08066783918828e-26,
                                         3.61597112031877e-05,
                                         0.00414862633945999,
                                         0.0396250685158924,
                                         0.132933337468436,
                                         0.231469679039743,
                                         0.243975923238191,
                                         0.166333023939746,
                                         0.0750050608033028,
                                         0.022216796875,
                                         0.00418242296107426,
                                         0.0004697387074326,
                                         2.82999070100563e-05,
                                         7.65693522122379e-07,
                                         6.80313097605935e-09,
                                         1.07345573145984e-11,
                                         6.93304542063086e-16,
                                         9.13001331914333e-24,
                                         4.70802168258128e-07,
                                         0.000288479183256386,
                                         0.00694071935297591,
                                         0.0448333269447261,
                                         0.131453426555305,
                                         0.216139637490938,
                                         0.219619505871138,
                                         0.143785613277228,
                                         0.06109619140625,
                                         0.0164994121471038,
                                         0.00269047715685187,
                                         0.000241581096160397,
                                         1.01962989158955e-05,
                                         1.52548225657402e-07,
                                         4.63463000384637e-10,
                                         7.54013763862257e-14,
                                         5.30302039878999e-21,
                                         4.45809436698085e-09,
                                         1.45888796326218e-05,
                                         0.000884170954325927,
                                         0.0109967747373445,
                                         0.0542933969496529,
                                         0.139257684341568,
                                         0.210892259031791,
                                         0.200464558854971,
                                         0.1221923828125,
                                         0.0473376063646522,
                                         0.0112072632055465,
                                         0.00149981671160636,
                                         9.87478040410348e-05,
                                         2.48772740702252e-06,
                                         1.45526907456956e-08,
                                         5.96391992368413e-12,
                                         2.2401262033026e-18,
                                         3.16607591121102e-11,
                                         5.53338216295067e-07,
                                         8.44752074731021e-05,
                                         0.00202297703848169,
                                         0.0168183498306854,
                                         0.0672922706587845,
                                         0.151883861851602,
                                         0.20961436148398,
                                         0.18328857421875,
                                         0.101860400677734,
                                         0.0350131429954316,
                                         0.00698352500723051,
                                         0.000717255021897585,
                                         3.0427038523296e-05,
                                         3.42714749231479e-07,
                                         3.53790035681108e-10,
                                         7.09713290165913e-16,
                                         1.7131448210645e-13,
                                         1.59904276380592e-08,
                                         6.14926172921822e-06,
                                         0.000283541913470246,
                                         0.00396935952287409,
                                         0.0247748985525189,
                                         0.0833418835240336,
                                         0.166995648379586,
                                         0.20947265625,
                                         0.166995648379586,
                                         0.0833418835240336,
                                         0.0247748985525189,
                                         0.00396935952287408,
                                         0.000283541913470245,
                                         6.14926172921822e-06,
                                         1.59904276380589e-08,
                                         1.71314482106464e-13,
                                         7.09713290165846e-16,
                                         3.53790035681114e-10,
                                         3.42714749231479e-07,
                                         3.04270385232961e-05,
                                         0.000717255021897587,
                                         0.00698352500723052,
                                         0.0350131429954316,
                                         0.101860400677734,
                                         0.18328857421875,
                                         0.20961436148398,
                                         0.151883861851602,
                                         0.0672922706587844,
                                         0.0168183498306854,
                                         0.00202297703848168,
                                         8.44752074731021e-05,
                                         5.53338216295059e-07,
                                         3.16607591121125e-11,
                                         2.24012620330236e-18,
                                         5.96391992368425e-12,
                                         1.45526907456956e-08,
                                         2.48772740702253e-06,
                                         9.8747804041035e-05,
                                         0.00149981671160637,
                                         0.0112072632055465,
                                         0.0473376063646522,
                                         0.1221923828125,
                                         0.200464558854971,
                                         0.210892259031791,
                                         0.139257684341567,
                                         0.0542933969496528,
                                         0.0109967747373445,
                                         0.000884170954325926,
                                         1.45888796326217e-05,
                                         4.45809436698111e-09,
                                         5.30302039878937e-21,
                                         7.54013763862274e-14,
                                         4.63463000384637e-10,
                                         1.52548225657403e-07,
                                         1.01962989158955e-05,
                                         0.000241581096160398,
                                         0.00269047715685187,
                                         0.0164994121471038,
                                         0.06109619140625,
                                         0.143785613277228,
                                         0.219619505871138,
                                         0.216139637490938,
                                         0.131453426555305,
                                         0.044833326944726,
                                         0.00694071935297591,
                                         0.000288479183256383,
                                         4.7080216825815e-07,
                                         9.13001331914215e-24,
                                         6.93304542063103e-16,
                                         1.07345573145984e-11,
                                         6.80313097605939e-09,
                                         7.65693522122382e-07,
                                         2.82999070100564e-05,
                                         0.000469738707432601,
                                         0.00418242296107426,
                                         0.022216796875,
                                         0.0750050608033028,
                                         0.166333023939746,
                                         0.243975923238191,
                                         0.231469679039742,
                                         0.132933337468436,
                                         0.0396250685158924,
                                         0.00414862633945997,
                                         3.61597112031889e-05,
                                         1.08066783918813e-26,
                                         4.38269773867848e-18,
                                         1.70932998844085e-13,
                                         2.08585063697732e-10,
                                         3.95312083409994e-08,
                                         2.27918601780491e-06,
                                         5.63840269857797e-05,
                                         0.000728886877767901,
                                         0.00555419921875,
                                         0.0268991404967024,
                                         0.0866081493351363,
                                         0.189335575464108,
                                         0.28021347127761,
                                         0.270981443328864,
                                         0.155527886640952,
                                         0.0410172805650965,
                                         0.00190934388515177,
                                         7.87153947237711e-30,
                                         1.70492637268945e-20,
                                         1.6749981026431e-15,
                                         3.93553895209477e-12,
                                         1.25594852670227e-09,
                                         1.1295908611804e-07,
                                         4.16487974969889e-06,
                                         7.81697904671442e-05,
                                         0.0008544921875,
                                         0.0059365328292988,
                                         0.0277514492999648,
                                         0.0904199187691077,
                                         0.208751952204349,
                                         0.339931827842335,
                                         0.375658439374046,
                                         0.249560611491813,
                                         0.0620426002048541,
                                         2.6620269576365e-33,
                                         3.07932167296888e-23,
                                         7.62058037292775e-18,
                                         3.44754977375804e-14,
                                         1.85263094138758e-11,
                                         2.59924902199221e-09,
                                         1.42834843823859e-07,
                                         3.89227113016622e-06,
                                         6.103515625e-05,
                                         0.000608292785002365,
                                         0.00412855333974618,
                                         0.0200484740184243,
                                         0.0722033589957216,
                                         0.1979836963965,
                                         0.421272732950795,
                                         0.704969843852267,
                                         0.936011420906696,

                                         0.942547768298974,
                                         0.731470303097805,
                                         0.461755083245601,
                                         0.235449108772402,
                                         0.0959534031701987,
                                         0.030791523883458,
                                         0.00762419881163833,
                                         0.0014173859206856,
                                         0.000190693321203044,
                                         1.76591672324347e-05,
                                         1.04984981410186e-06,
                                         3.62606143360211e-08,
                                         6.26734307867869e-10,
                                         4.28020855911331e-12,
                                         7.69680804895729e-15,
                                         1.65767872991826e-18,
                                         6.57955393776494e-24,
                                         5.63183438625465e-34,
                                         0.0558873218393644,
                                         0.231303316462898,
                                         0.366838261393549,
                                         0.35873100603862,
                                         0.24482748516434,
                                         0.121668243685436,
                                         0.0444759438165118,
                                         0.0118582046245149,
                                         0.00225243213051513,
                                         0.000293028633319445,
                                         2.4595316822595e-05,
                                         1.21831690044894e-06,
                                         3.10880501735622e-08,
                                         3.28791978322584e-10,
                                         9.90136025999192e-13,
                                         4.08971924291485e-16,
                                         4.07818912324132e-21,
                                         1.86164135347267e-30,
                                         0.00153853913739171,
                                         0.0339588005019295,
                                         0.135307817449829,
                                         0.253761715177574,
                                         0.290031575464839,
                                         0.223207414050334,
                                         0.120459610408736,
                                         0.0460611829052134,
                                         0.0123524545884544,
                                         0.00225753878706525,
                                         0.000267524146896919,
                                         1.90051237038604e-05,
                                         7.15960100578662e-07,
                                         1.17263490260837e-08,
                                         5.91376946163589e-11,
                                         4.68459387110652e-14,
                                         1.17360943825328e-18,
                                         2.85711341903321e-27,
                                         2.60645620991546e-05,
                                         0.00306809962889926,
                                         0.0307126850248758,
                                         0.110466328862033,
                                         0.211435071289929,
                                         0.25199192709061,
                                         0.200772566293207,
                                         0.110102674460849,
                                         0.0416870777947603,
                                         0.0107030367486792,
                                         0.00179068909591829,
                                         0.000182443236297016,
                                         1.01468395092462e-05,
                                         2.57365929823214e-07,
                                         2.17360464092006e-09,
                                         3.30215178164916e-12,
                                         2.07838720257586e-16,
                                         2.69839534517541e-24,
                                         3.03574312395154e-07,
                                         0.000190572081336262,
                                         0.00479275672655462,
                                         0.0330602736510297,
                                         0.105969643627813,
                                         0.195585809581834,
                                         0.230059403122249,
                                         0.180939486009508,
                                         0.0967213483201027,
                                         0.0348860271886362,
                                         0.00824043414159503,
                                         0.00120408608252523,
                                         9.88656587998961e-05,
                                         3.88339882340375e-06,
                                         5.49249180476857e-08,
                                         1.600275733337e-10,
                                         2.53047486739193e-14,
                                         1.75208987360156e-21,
                                         2.57144304126893e-09,
                                         8.60887492173231e-06,
                                         0.000543939110532569,
                                         0.00719581962718034,
                                         0.0386263101165881,
                                         0.110404139750011,
                                         0.19172242300971,
                                         0.216254990305115,
                                         0.163207658160626,
                                         0.0826976779213586,
                                         0.0275789227517334,
                                         0.0057794246725023,
                                         0.000700579523920145,
                                         4.26157598940867e-05,
                                         1.00938206625304e-06,
                                         5.64014032951861e-09,
                                         2.24065451819083e-12,
                                         8.27378977635272e-19,
                                         1.63361631184806e-11,
                                         2.91672028629658e-07,
                                         4.62995160479888e-05,
                                         0.00117466859137676,
                                         0.0105595700486383,
                                         0.0467406381628625,
                                         0.119830423096797,
                                         0.193847492316055,
                                         0.20654752139714,
                                         0.147026757230166,
                                         0.0692254467795723,
                                         0.0208052501581185,
                                         0.00372332270345949,
                                         0.000350743589688528,
                                         1.39124307129875e-05,
                                         1.49089226971814e-07,
                                         1.4880209050634e-10,
                                         2.93031189330043e-16,
                                         7.90722144357217e-14,
                                         7.52911367869428e-09,
                                         3.00264026548908e-06,
                                         0.000146100308346495,
                                         0.0021994289438969,
                                         0.015076641394881,
                                         0.057063967590489,
                                         0.132389921375707,
                                         0.199159088610741,
                                         0.199159088610741,
                                         0.132389921375707,
                                         0.057063967590489,
                                         0.015076641394881,
                                         0.0021994289438969,
                                         0.000146100308346496,
                                         3.00264026548908e-06,
                                         7.52911367869428e-09,
                                         7.90722144357217e-14,
                                         2.93031189330043e-16,
                                         1.4880209050634e-10,
                                         1.49089226971814e-07,
                                         1.39124307129874e-05,
                                         0.000350743589688528,
                                         0.00372332270345949,
                                         0.0208052501581185,
                                         0.0692254467795723,
                                         0.147026757230166,
                                         0.20654752139714,
                                         0.193847492316055,
                                         0.119830423096797,
                                         0.0467406381628625,
                                         0.0105595700486383,
                                         0.00117466859137677,
                                         4.62995160479888e-05,
                                         2.91672028629658e-07,
                                         1.63361631184806e-11,
                                         8.27378977635272e-19,
                                         2.24065451819083e-12,
                                         5.64014032951861e-09,
                                         1.00938206625304e-06,
                                         4.26157598940867e-05,
                                         0.000700579523920145,
                                         0.0057794246725023,
                                         0.0275789227517334,
                                         0.0826976779213587,
                                         0.163207658160626,
                                         0.216254990305115,
                                         0.19172242300971,
                                         0.110404139750011,
                                         0.0386263101165881,
                                         0.00719581962718036,
                                         0.000543939110532569,
                                         8.60887492173231e-06,
                                         2.57144304126893e-09,
                                         1.75208987360156e-21,
                                         2.53047486739193e-14,
                                         1.600275733337e-10,
                                         5.49249180476854e-08,
                                         3.88339882340375e-06,
                                         9.88656587998961e-05,
                                         0.00120408608252523,
                                         0.00824043414159503,
                                         0.0348860271886362,
                                         0.0967213483201027,
                                         0.180939486009508,
                                         0.230059403122249,
                                         0.195585809581834,
                                         0.105969643627813,
                                         0.0330602736510298,
                                         0.00479275672655462,
                                         0.000190572081336261,
                                         3.03574312395154e-07,
                                         2.69839534517541e-24,
                                         2.07838720257586e-16,
                                         3.30215178164916e-12,
                                         2.17360464092004e-09,
                                         2.57365929823214e-07,
                                         1.01468395092462e-05,
                                         0.000182443236297016,
                                         0.00179068909591829,
                                         0.0107030367486792,
                                         0.0416870777947603,
                                         0.110102674460849,
                                         0.200772566293207,
                                         0.25199192709061,
                                         0.211435071289929,
                                         0.110466328862034,
                                         0.0307126850248757,
                                         0.00306809962889926,
                                         2.60645620991546e-05,
                                         2.85711341903321e-27,
                                         1.17360943825328e-18,
                                         4.68459387110652e-14,
                                         5.91376946163585e-11,
                                         1.17263490260837e-08,
                                         7.15960100578662e-07,
                                         1.90051237038604e-05,
                                         0.000267524146896919,
                                         0.00225753878706525,
                                         0.0123524545884544,
                                         0.0460611829052134,
                                         0.120459610408736,
                                         0.223207414050334,
                                         0.290031575464839,
                                         0.253761715177574,
                                         0.135307817449829,
                                         0.0339588005019295,
                                         0.00153853913739171,
                                         1.86164135347267e-30,
                                         4.07818912324132e-21,
                                         4.08971924291485e-16,
                                         9.90136025999185e-13,
                                         3.28791978322584e-10,
                                         3.10880501735622e-08,
                                         1.21831690044894e-06,
                                         2.4595316822595e-05,
                                         0.000293028633319445,
                                         0.00225243213051513,
                                         0.0118582046245149,
                                         0.0444759438165118,
                                         0.121668243685436,
                                         0.24482748516434,
                                         0.35873100603862,
                                         0.366838261393549,
                                         0.231303316462898,
                                         0.0558873218393644,
                                         5.63183438625465e-34,
                                         6.57955393776494e-24,
                                         1.65767872991826e-18,
                                         7.69680804895723e-15,
                                         4.28020855911331e-12,
                                         6.26734307867869e-10,
                                         3.62606143360211e-08,
                                         1.04984981410186e-06,
                                         1.76591672324347e-05,
                                         0.000190693321203044,
                                         0.0014173859206856,
                                         0.00762419881163833,
                                         0.030791523883458,
                                         0.0959534031701987,
                                         0.235449108772402,
                                         0.461755083245601,
                                         0.731470303097805,
                                         0.942547768298974,

                                         0.948139868272578,
                                         0.754763230860369,
                                         0.499138640112703,
                                         0.272672701889423,
                                         0.122009248453586,
                                         0.0441907797097743,
                                         0.0127504353282685,
                                         0.00286924861658636,
                                         0.000489633873389485,
                                         6.103515625e-05,
                                         5.28315737317772e-06,
                                         2.96039112011097e-07,
                                         9.71425400456032e-09,
                                         1.60740563345792e-10,
                                         1.0587073889617e-12,
                                         1.84926744321247e-15,
                                         3.89586750575591e-19,
                                         1.52299180260258e-24,
                                         1.29268497364195e-34,
                                         0.0505876786170866,
                                         0.214501674597545,
                                         0.355588749577277,
                                         0.371299055546915,
                                         0.27695250739236,
                                         0.154404253050786,
                                         0.0652596581299206,
                                         0.0208522054874698,
                                         0.00496022148146164,
                                         0.0008544921875,
                                         0.000102216264401206,
                                         7.98401508983436e-06,
                                         3.72002402638748e-07,
                                         9.01683168849563e-09,
                                         9.14154215061478e-11,
                                         2.66178891580461e-13,
                                         1.07184856155888e-16,
                                         1.05034914188866e-21,
                                         4.74871854273458e-31,
                                         0.00125314814058219,
                                         0.0283032273665633,
                                         0.117614306830517,
                                         0.234742296223332,
                                         0.291879216476313,
                                         0.250479492164995,
                                         0.155077894834407,
                                         0.070359238768191,
                                         0.0233300679395433,
                                         0.00555419921875,
                                         0.000918188361967067,
                                         9.99721256402375e-05,
                                         6.61404820157089e-06,
                                         2.34837669045801e-07,
                                         3.6647833781482e-09,
                                         1.77882334971479e-11,
                                         1.36914044944878e-14,
                                         3.36321882404483e-19,
                                         8.09926259112144e-28,
                                         1.9103225925899e-05,
                                         0.0022982002523294,
                                         0.0239397124436028,
                                         0.0913283238535587,
                                         0.189298719759602,
                                         0.250052801229302,
                                         0.226778372431921,
                                         0.146095521440212,
                                         0.0675270181875727,
                                         0.022216796875,
                                         0.00507563282157349,
                                         0.000770341222098434,
                                         7.23661721659685e-05,
                                         3.76381434926539e-06,
                                         9.04115233174515e-08,
                                         7.31540989815651e-10,
                                         1.07624019491393e-12,
                                         6.62709505589615e-17,
                                         8.50082673619264e-25,
                                         2.00209053429949e-07,
                                         0.000128295811563774,
                                         0.0033500432050917,
                                         0.0244282484433525,
                                         0.0844043606220712,
                                         0.171618450547582,
                                         0.227995394199982,
                                         0.208557290542264,
                                         0.134372947842277,
                                         0.06109619140625,
                                         0.0192895152067896,
                                         0.00408093852492688,
                                         0.000544347896495322,
                                         4.14726062811459e-05,
                                         1.53345828655494e-06,
                                         2.06831752662825e-08,
                                         5.81625068750332e-11,
                                         8.97767702150481e-15,
                                         6.13408163894014e-22,
                                         1.5260122312059e-09,
                                         5.20876051854421e-06,
                                         0.000340940966982479,
                                         0.00475200042962676,
                                         0.0273702896711429,
                                         0.0856630495597369,
                                         0.166704688872184,
                                         0.216526547510807,
                                         0.194465882319262,
                                         0.1221923828125,
                                         0.0533150389131885,
                                         0.0157229585270289,
                                         0.00297793215255124,
                                         0.000332347002200843,
                                         1.89154835805407e-05,
                                         4.25297725860548e-07,
                                         2.28599000785749e-09,
                                         8.84508593037269e-13,
                                         3.21910572793705e-19,
                                         8.72355653963278e-12,
                                         1.5860525263168e-07,
                                         2.60237113039933e-05,
                                         0.000693301081395724,
                                         0.00665664147409921,
                                         0.0320689210708119,
                                         0.0914178114962133,
                                         0.168600240448495,
                                         0.211074736359494,
                                         0.18328857421875,
                                         0.110519627262883,
                                         0.0454328256844607,
                                         0.0122183992474673,
                                         0.00199748472141661,
                                         0.000174994417303091,
                                         6.55888735506509e-06,
                                         6.73855538145764e-08,
                                         6.53583980535339e-11,
                                         1.26701627451289e-16,
                                         3.79952961339518e-14,
                                         3.67960694172095e-09,
                                         1.51342172593408e-06,
                                         7.70669145434653e-05,
                                         0.00123347862287307,
                                         0.00914694050752375,
                                         0.0381957034735659,
                                         0.100024391047618,
                                         0.174553990008823,
                                         0.20947265625,
                                         0.174553990008823,
                                         0.100024391047618,
                                         0.0381957034735659,
                                         0.00914694050752374,
                                         0.00123347862287307,
                                         7.70669145434653e-05,
                                         1.51342172593408e-06,
                                         3.67960694172095e-09,
                                         3.79952961339518e-14,
                                         1.26701627451289e-16,
                                         6.53583980535339e-11,
                                         6.73855538145764e-08,
                                         6.55888735506509e-06,
                                         0.00017499441730309,
                                         0.00199748472141661,
                                         0.0122183992474673,
                                         0.0454328256844606,
                                         0.110519627262883,
                                         0.18328857421875,
                                         0.211074736359494,
                                         0.168600240448495,
                                         0.0914178114962133,
                                         0.0320689210708119,
                                         0.00665664147409923,
                                         0.000693301081395724,
                                         2.60237113039933e-05,
                                         1.5860525263168e-07,
                                         8.72355653963278e-12,
                                         3.21910572793705e-19,
                                         8.84508593037269e-13,
                                         2.28599000785749e-09,
                                         4.25297725860548e-07,
                                         1.89154835805406e-05,
                                         0.000332347002200844,
                                         0.00297793215255124,
                                         0.0157229585270289,
                                         0.0533150389131885,
                                         0.1221923828125,
                                         0.194465882319262,
                                         0.216526547510807,
                                         0.166704688872184,
                                         0.0856630495597368,
                                         0.027370289671143,
                                         0.00475200042962676,
                                         0.000340940966982479,
                                         5.20876051854421e-06,
                                         1.5260122312059e-09,
                                         6.13408163894014e-22,
                                         8.97767702150481e-15,
                                         5.81625068750332e-11,
                                         2.06831752662825e-08,
                                         1.53345828655494e-06,
                                         4.14726062811461e-05,
                                         0.000544347896495322,
                                         0.00408093852492687,
                                         0.0192895152067895,
                                         0.06109619140625,
                                         0.134372947842277,
                                         0.208557290542264,
                                         0.227995394199982,
                                         0.171618450547582,
                                         0.0844043606220713,
                                         0.0244282484433525,
                                         0.0033500432050917,
                                         0.000128295811563774,
                                         2.00209053429949e-07,
                                         8.50082673619264e-25,
                                         6.62709505589615e-17,
                                         1.07624019491393e-12,
                                         7.31540989815651e-10,
                                         9.04115233174511e-08,
                                         3.7638143492654e-06,
                                         7.23661721659685e-05,
                                         0.000770341222098433,
                                         0.00507563282157349,
                                         0.022216796875,
                                         0.0675270181875727,
                                         0.146095521440212,
                                         0.226778372431921,
                                         0.250052801229302,
                                         0.189298719759603,
                                         0.0913283238535587,
                                         0.0239397124436028,
                                         0.0022982002523294,
                                         1.9103225925899e-05,
                                         8.09926259112145e-28,
                                         3.36321882404483e-19,
                                         1.36914044944878e-14,
                                         1.77882334971479e-11,
                                         3.66478337814818e-09,
                                         2.34837669045801e-07,
                                         6.61404820157089e-06,
                                         9.99721256402373e-05,
                                         0.000918188361967067,
                                         0.00555419921875,
                                         0.0233300679395433,
                                         0.070359238768191,
                                         0.155077894834407,
                                         0.250479492164995,
                                         0.291879216476314,
                                         0.234742296223332,
                                         0.117614306830517,
                                         0.0283032273665633,
                                         0.00125314814058218,
                                         4.74871854273458e-31,
                                         1.05034914188866e-21,
                                         1.07184856155888e-16,
                                         2.66178891580461e-13,
                                         9.14154215061473e-11,
                                         9.01683168849567e-09,
                                         3.72002402638748e-07,
                                         7.98401508983434e-06,
                                         0.000102216264401206,
                                         0.0008544921875,
                                         0.00496022148146164,
                                         0.0208522054874698,
                                         0.0652596581299206,
                                         0.154404253050786,
                                         0.27695250739236,
                                         0.371299055546915,
                                         0.355588749577277,
                                         0.214501674597545,
                                         0.0505876786170866,
                                         1.29268497364195e-34,
                                         1.52299180260258e-24,
                                         3.89586750575591e-19,
                                         1.84926744321247e-15,
                                         1.0587073889617e-12,
                                         1.60740563345792e-10,
                                         9.71425400456032e-09,
                                         2.96039112011096e-07,
                                         5.28315737317772e-06,
                                         6.103515625e-05,
                                         0.000489633873389485,
                                         0.00286924861658636,
                                         0.0127504353282685,
                                         0.0441907797097743,
                                         0.122009248453586,
                                         0.272672701889423,
                                         0.499138640112703,
                                         0.754763230860369,
                                         0.948139868272578,

                                         0.952959738140794,
                                         0.775307385919845,
                                         0.533526411486313,
                                         0.309108531077368,
                                         0.149745956774859,
                                         0.0600821894271157,
                                         0.019712756900282,
                                         0.00520108061592863,
                                         0.001079684337037,
                                         0.000171366658994739,
                                         2.00226725830651e-05,
                                         1.63647217932143e-06,
                                         8.71916136024439e-08,
                                         2.73878243101943e-09,
                                         4.36619072217167e-11,
                                         2.78805855091052e-13,
                                         4.75032484701683e-16,
                                         9.82016240367762e-20,
                                         3.78926074116329e-25,
                                         3.19311095354542e-35,
                                         0.0459952039897145,
                                         0.199116711860785,
                                         0.342820680338386,
                                         0.378564518669278,
                                         0.304524470530507,
                                         0.18711713884995,
                                         0.089346208620609,
                                         0.0331975464625289,
                                         0.00950692279781247,
                                         0.00205804117494257,
                                         0.000326776176837683,
                                         3.64268428225453e-05,
                                         2.67743159665885e-06,
                                         1.18436303088828e-07,
                                         2.74783692892719e-09,
                                         2.68714356276618e-11,
                                         7.60239560721644e-14,
                                         2.99546088189888e-17,
                                         2.89185671669482e-22,
                                         1.29667782547461e-31,
                                         0.00103070852276573,
                                         0.0237425205984481,
                                         0.102273571546578,
                                         0.215255443711615,
                                         0.28752433620129,
                                         0.270561941785855,
                                         0.188014009438443,
                                         0.0983792960037931,
                                         0.0388658754157864,
                                         0.0114753809033691,
                                         0.00247607645242727,
                                         0.000376460797636402,
                                         3.81722161610757e-05,
                                         2.37792127855539e-06,
                                         8.02905679848104e-08,
                                         1.20244485974515e-09,
                                         5.64888771361721e-12,
                                         4.24222750819929e-15,
                                         1.02467082947267e-19,
                                         2.44475576796472e-28,
                                         1.42136566324073e-05,
                                         0.00174217817850808,
                                         0.0187761465831837,
                                         0.0753208156885504,
                                         0.167060453836256,
                                         0.240750102538743,
                                         0.243473068094353,
                                         0.179410587431423,
                                         0.0977785462372914,
                                         0.0393755647115668,
                                         0.0115458084304224,
                                         0.00239422311300983,
                                         0.000334906054907527,
                                         2.93803346520071e-05,
                                         1.44372587189664e-06,
                                         3.31120527620015e-08,
                                         2.5829860075859e-10,
                                         3.69718254300602e-13,
                                         2.23428520232916e-17,
                                         2.83651777587099e-25,
                                         1.34756112810909e-07,
                                         8.78882907670023e-05,
                                         0.00236985739998766,
                                         0.0181195992537478,
                                         0.0667337310581618,
                                         0.147278365904929,
                                         0.216762598471932,
                                         0.224939190770968,
                                         0.169118609869815,
                                         0.0928879082724799,
                                         0.0370132607879164,
                                         0.010468445164056,
                                         0.00202009282835576,
                                         0.000249567889203624,
                                         1.78475106887388e-05,
                                         6.2687325750874e-07,
                                         8.11995958207801e-09,
                                         2.2152387222983e-11,
                                         3.34938867923306e-15,
                                         2.26260239317367e-22,
                                         9.29155563530516e-10,
                                         3.22453257828601e-06,
                                         0.000217538066044491,
                                         0.00317014745878014,
                                         0.0193871717852751,
                                         0.065525257283624,
                                         0.140350857512415,
                                         0.205106531515596,
                                         0.212733813889929,
                                         0.159363504963539,
                                         0.0862954005365273,
                                         0.0332887162839016,
                                         0.00886169821574948,
                                         0.001541764264722,
                                         0.000160460397634494,
                                         8.63119011632537e-06,
                                         1.85644883081241e-07,
                                         9.65311492234034e-10,
                                         3.65165541343189e-13,
                                         1.31258753779489e-19,
                                         4.80495862060343e-12,
                                         8.8728631462346e-08,
                                         1.4976473957449e-05,
                                         0.000415979187908838,
                                         0.00422420293161994,
                                         0.0218645113746205,
                                         0.0681564647568372,
                                         0.140266873214167,
                                         0.20069794037004,
                                         0.2050594678031,
                                         0.15089651644913,
                                         0.0793913480796758,
                                         0.0291557252337766,
                                         0.00714345820557769,
                                         0.00108198026848356,
                                         8.91297905100628e-05,
                                         3.18326916524761e-06,
                                         3.15482796840175e-08,
                                         2.98590023482281e-11,
                                         5.71096599729296e-17,
                                         1.89317848444463e-14,
                                         1.86020790242572e-09,
                                         7.85569432124867e-07,
                                         4.15876547185104e-05,
                                         0.00070125468649903,
                                         0.0055586778504185,
                                         0.0252173664146096,
                                         0.0730855348134351,
                                         0.144261350329007,
                                         0.201034904006164,
                                         0.201034904006164,
                                         0.144261350329007,
                                         0.0730855348134351,
                                         0.0252173664146096,
                                         0.00555867785041849,
                                         0.00070125468649903,
                                         4.15876547185106e-05,
                                         7.85569432124867e-07,
                                         1.86020790242576e-09,
                                         1.89317848444484e-14,
                                         5.71096599729222e-17,
                                         2.98590023482273e-11,
                                         3.15482796840175e-08,
                                         3.1832691652476e-06,
                                         8.91297905100628e-05,
                                         0.00108198026848357,
                                         0.00714345820557769,
                                         0.0291557252337766,
                                         0.0793913480796758,
                                         0.15089651644913,
                                         0.2050594678031,
                                         0.20069794037004,
                                         0.140266873214167,
                                         0.0681564647568372,
                                         0.0218645113746205,
                                         0.00422420293161994,
                                         0.00041597918790884,
                                         1.4976473957449e-05,
                                         8.87286314623477e-08,
                                         4.80495862060389e-12,
                                         1.3125875377947e-19,
                                         3.65165541343179e-13,
                                         9.65311492234034e-10,
                                         1.8564488308124e-07,
                                         8.63119011632537e-06,
                                         0.000160460397634494,
                                         0.001541764264722,
                                         0.00886169821574948,
                                         0.0332887162839016,
                                         0.0862954005365273,
                                         0.159363504963539,
                                         0.212733813889929,
                                         0.205106531515596,
                                         0.140350857512415,
                                         0.0655252572836239,
                                         0.0193871717852751,
                                         0.00317014745878015,
                                         0.000217538066044491,
                                         3.22453257828606e-06,
                                         9.29155563530591e-10,
                                         2.2626023931733e-22,
                                         3.34938867923296e-15,
                                         2.2152387222983e-11,
                                         8.11995958207795e-09,
                                         6.26873257508739e-07,
                                         1.78475106887388e-05,
                                         0.000249567889203624,
                                         0.00202009282835576,
                                         0.010468445164056,
                                         0.0370132607879164,
                                         0.0928879082724799,
                                         0.169118609869815,
                                         0.224939190770968,
                                         0.216762598471932,
                                         0.147278365904929,
                                         0.0667337310581618,
                                         0.0181195992537479,
                                         0.00236985739998766,
                                         8.78882907670034e-05,
                                         1.34756112810917e-07,
                                         2.83651777587049e-25,
                                         2.23428520232909e-17,
                                         3.69718254300602e-13,
                                         2.58298600758588e-10,
                                         3.31120527620015e-08,
                                         1.44372587189665e-06,
                                         2.93803346520071e-05,
                                         0.000334906054907527,
                                         0.00239422311300983,
                                         0.0115458084304224,
                                         0.0393755647115668,
                                         0.0977785462372914,
                                         0.179410587431423,
                                         0.243473068094353,
                                         0.240750102538742,
                                         0.167060453836256,
                                         0.0753208156885505,
                                         0.0187761465831837,
                                         0.0017421781785081,
                                         1.4213656632408e-05,
                                         2.44475576796424e-28,
                                         1.02467082947263e-19,
                                         4.24222750819929e-15,
                                         5.64888771361716e-12,
                                         1.20244485974515e-09,
                                         8.02905679848107e-08,
                                         2.37792127855539e-06,
                                         3.81722161610757e-05,
                                         0.000376460797636402,
                                         0.00247607645242727,
                                         0.0114753809033691,
                                         0.0388658754157864,
                                         0.0983792960037931,
                                         0.188014009438443,
                                         0.270561941785855,
                                         0.28752433620129,
                                         0.215255443711616,
                                         0.102273571546578,
                                         0.0237425205984482,
                                         0.00103070852276576,
                                         1.29667782547433e-31,
                                         2.8918567166947e-22,
                                         2.99546088189888e-17,
                                         7.60239560721637e-14,
                                         2.68714356276618e-11,
                                         2.7478369289272e-09,
                                         1.18436303088828e-07,
                                         2.67743159665885e-06,
                                         3.64268428225453e-05,
                                         0.000326776176837683,
                                         0.00205804117494257,
                                         0.00950692279781247,
                                         0.0331975464625289,
                                         0.089346208620609,
                                         0.18711713884995,
                                         0.304524470530507,
                                         0.378564518669278,
                                         0.342820680338386,
                                         0.199116711860786,
                                         0.0459952039897153,
                                         3.1931109535447e-35,
                                         3.78926074116313e-25,
                                         9.82016240367762e-20,
                                         4.75032484701679e-16,
                                         2.78805855091052e-13,
                                         4.36619072217168e-11,
                                         2.73878243101943e-09,
                                         8.71916136024439e-08,
                                         1.63647217932143e-06,
                                         2.00226725830651e-05,
                                         0.000171366658994739,
                                         0.001079684337037,
                                         0.00520108061592863,
                                         0.019712756900282,
                                         0.0600821894271157,
                                         0.149745956774859,
                                         0.309108531077368,
                                         0.533526411486313,
                                         0.775307385919845,
                                         0.952959738140794,

                                         0.957142246963524,
                                         0.793491413705889,
                                         0.565082449306372,
                                         0.344372210052135,
                                         0.178584823571755,
                                         0.0781960298818508,
                                         0.0286134789121945,
                                         0.00863337296337772,
                                         0.00211107066117314,
                                         0.000409109602033853,
                                         6.103515625e-05,
                                         6.74697134232639e-06,
                                         5.24949992797838e-07,
                                         2.67844303311987e-08,
                                         8.10294895857634e-10,
                                         1.2510205029548e-11,
                                         7.77804752206823e-14,
                                         1.29710424610374e-16,
                                         2.63806589763678e-20,
                                         1.00655331141852e-25,
                                         8.42923796644396e-36,
                                         0.041991608359522,
                                         0.185069206880177,
                                         0.329205101106246,
                                         0.381450375297386,
                                         0.327375272965254,
                                         0.218576842714819,
                                         0.115760665971048,
                                         0.0488493371576043,
                                         0.0163370916755846,
                                         0.00427198573304412,
                                         0.0008544921875,
                                         0.000126641141362125,
                                         1.32954190302313e-05,
                                         9.27812700493731e-07,
                                         3.92562510009372e-08,
                                         8.77204910716965e-10,
                                         8.31620792396581e-12,
                                         2.29520247523415e-14,
                                         8.87538035821232e-18,
                                         8.45864738971552e-23,
                                         3.76580963717805e-32,
                                         0.000855330293186174,
                                         0.0200406321654373,
                                         0.0890444004774527,
                                         0.196170327106902,
                                         0.278632979490026,
                                         0.28366706411883,
                                         0.217438623635801,
                                         0.128328219379344,
                                         0.0586991811654902,
                                         0.0207111987109029,
                                         0.00555419921875,
                                         0.00110363658179252,
                                         0.000156340481745906,
                                         1.49218796613076e-05,
                                         8.82998287627276e-07,
                                         2.85576852097674e-08,
                                         4.12824446210095e-10,
                                         1.88561249402506e-12,
                                         1.38635236987119e-15,
                                         3.30027691342623e-20,
                                         7.81112758264446e-29,
                                         1.07214082347597e-05,
                                         0.00133547359219605,
                                         0.0148215392022411,
                                         0.062083361061825,
                                         0.14593712931206,
                                         0.226548042513735,
                                         0.251338488381144,
                                         0.20745900760053,
                                         0.129788428703139,
                                         0.0617912971189546,
                                         0.022216796875,
                                         0.00591866819202663,
                                         0.00113132529341189,
                                         0.000147683980088493,
                                         1.22224293507033e-05,
                                         5.72125868339353e-07,
                                         1.26110750289326e-08,
                                         9.53302046598657e-11,
                                         1.33262181684353e-13,
                                         7.92403666265534e-18,
                                         9.97047238435318e-26,
                                         9.23937330097054e-08,
                                         6.11831587572679e-05,
                                         0.00169610487478882,
                                         0.0135079619180115,
                                         0.0525498864265545,
                                         0.124389698930283,
                                         0.199734924486897,
                                         0.230576544499676,
                                         0.197293423432866,
                                         0.126742447957523,
                                         0.06109619140625,
                                         0.0218220025503393,
                                         0.00562828719944487,
                                         0.00100488403257543,
                                         0.000116312907580626,
                                         7.88012069913441e-06,
                                         2.64857035977938e-07,
                                         3.31345674953675e-09,
                                         8.80669409818582e-12,
                                         1.30802267783249e-15,
                                         8.74964516701405e-23,
                                         5.79069242503041e-10,
                                         2.03857074300248e-06,
                                         0.000141159268623299,
                                         0.00213747840800686,
                                         0.0137617944308001,
                                         0.049671333697245,
                                         0.115437343094902,
                                         0.186378262432129,
                                         0.218115507851999,
                                         0.189066320318876,
                                         0.1221923828125,
                                         0.0585143671892613,
                                         0.0203639633069158,
                                         0.00497274059389881,
                                         0.00080499946341865,
                                         7.89353349981811e-05,
                                         4.04546288782351e-06,
                                         8.37585959644034e-08,
                                         4.23268749870568e-10,
                                         1.57029545196086e-13,
                                         5.58421908921556e-20,
                                         2.72194750138915e-12,
                                         5.09425807522405e-08,
                                         8.81104379855006e-06,
                                         0.000253673387541359,
                                         0.00270295997052667,
                                         0.0148760794451671,
                                         0.0500379949139205,
                                         0.112989127263338,
                                         0.180851345439261,
                                         0.211527830977157,
                                         0.18328857421875,
                                         0.117677026649868,
                                         0.0552598401881015,
                                         0.0184559721912634,
                                         0.00417853970111904,
                                         0.00059302268478827,
                                         4.63432184734954e-05,
                                         1.58795698757249e-06,
                                         1.52574081107717e-08,
                                         1.41386757751119e-11,
                                         2.6729800672878e-17,
                                         9.74831780974268e-15,
                                         9.69921945879291e-10,
                                         4.19030860279134e-07,
                                         2.29376419137818e-05,
                                         0.000404487262333457,
                                         0.0033944689843157,
                                         0.0165254830377897,
                                         0.0521889714293821,
                                         0.114250374231844,
                                         0.180310720305516,
                                         0.20947265625,
                                         0.180310720305516,
                                         0.114250374231844,
                                         0.0521889714293822,
                                         0.0165254830377897,
                                         0.0033944689843157,
                                         0.000404487262333457,
                                         2.29376419137818e-05,
                                         4.1903086027913e-07,
                                         9.69921945879314e-10,
                                         9.74831780974147e-15,
                                         2.67298006728818e-17,
                                         1.41386757751115e-11,
                                         1.52574081107718e-08,
                                         1.58795698757249e-06,
                                         4.63432184734954e-05,
                                         0.00059302268478827,
                                         0.00417853970111904,
                                         0.0184559721912633,
                                         0.0552598401881015,
                                         0.117677026649868,
                                         0.18328857421875,
                                         0.211527830977157,
                                         0.180851345439261,
                                         0.112989127263338,
                                         0.0500379949139205,
                                         0.0148760794451671,
                                         0.00270295997052667,
                                         0.000253673387541359,
                                         8.81104379854999e-06,
                                         5.09425807522416e-08,
                                         2.72194750138886e-12,
                                         5.58421908921645e-20,
                                         1.57029545196081e-13,
                                         4.23268749870574e-10,
                                         8.37585959644033e-08,
                                         4.04546288782351e-06,
                                         7.89353349981811e-05,
                                         0.00080499946341865,
                                         0.0049727405938988,
                                         0.0203639633069158,
                                         0.0585143671892613,
                                         0.1221923828125,
                                         0.189066320318876,
                                         0.218115507851999,
                                         0.186378262432129,
                                         0.115437343094902,
                                         0.049671333697245,
                                         0.0137617944308001,
                                         0.00213747840800686,
                                         0.000141159268623298,
                                         2.03857074300252e-06,
                                         5.79069242502989e-10,
                                         8.7496451670156e-23,
                                         1.30802267783245e-15,
                                         8.80669409818594e-12,
                                         3.31345674953675e-09,
                                         2.64857035977938e-07,
                                         7.88012069913441e-06,
                                         0.000116312907580626,
                                         0.00100488403257543,
                                         0.00562828719944487,
                                         0.0218220025503393,
                                         0.06109619140625,
                                         0.126742447957522,
                                         0.197293423432866,
                                         0.230576544499676,
                                         0.199734924486897,
                                         0.124389698930283,
                                         0.0525498864265545,
                                         0.0135079619180115,
                                         0.00169610487478881,
                                         6.11831587572687e-05,
                                         9.23937330096988e-08,
                                         9.97047238435513e-26,
                                         7.92403666265504e-18,
                                         1.33262181684355e-13,
                                         9.53302046598657e-11,
                                         1.26110750289326e-08,
                                         5.72125868339353e-07,
                                         1.22224293507033e-05,
                                         0.000147683980088493,
                                         0.00113132529341189,
                                         0.00591866819202663,
                                         0.022216796875,
                                         0.0617912971189546,
                                         0.129788428703139,
                                         0.207459007600531,
                                         0.251338488381144,
                                         0.226548042513735,
                                         0.14593712931206,
                                         0.062083361061825,
                                         0.0148215392022411,
                                         0.00133547359219607,
                                         1.07214082347591e-05,
                                         7.81112758264613e-29,
                                         3.3002769134261e-20,
                                         1.38635236987122e-15,
                                         1.88561249402506e-12,
                                         4.12824446210095e-10,
                                         2.85576852097674e-08,
                                         8.82998287627276e-07,
                                         1.49218796613076e-05,
                                         0.000156340481745906,
                                         0.00110363658179252,
                                         0.00555419921875,
                                         0.0207111987109029,
                                         0.0586991811654902,
                                         0.128328219379344,
                                         0.217438623635801,
                                         0.28366706411883,
                                         0.278632979490026,
                                         0.196170327106902,
                                         0.0890444004774525,
                                         0.0200406321654374,
                                         0.000855330293186144,
                                         3.76580963717892e-32,
                                         8.45864738971515e-23,
                                         8.87538035821249e-18,
                                         2.29520247523415e-14,
                                         8.31620792396581e-12,
                                         8.77204910716965e-10,
                                         3.92562510009372e-08,
                                         9.27812700493729e-07,
                                         1.32954190302313e-05,
                                         0.000126641141362125,
                                         0.0008544921875,
                                         0.00427198573304412,
                                         0.0163370916755846,
                                         0.0488493371576043,
                                         0.115760665971048,
                                         0.218576842714819,
                                         0.327375272965254,
                                         0.381450375297386,
                                         0.329205101106246,
                                         0.185069206880178,
                                         0.0419916083595212,
                                         8.42923796644606e-36,
                                         1.00655331141847e-25,
                                         2.63806589763683e-20,
                                         1.29710424610374e-16,
                                         7.77804752206823e-14,
                                         1.2510205029548e-11,
                                         8.10294895857634e-10,
                                         2.67844303311986e-08,
                                         5.24949992797838e-07,
                                         6.74697134232639e-06,
                                         6.103515625e-05,
                                         0.000409109602033853,
                                         0.00211107066117314,
                                         0.00863337296337772,
                                         0.0286134789121945,
                                         0.0781960298818508,
                                         0.178584823571755,
                                         0.344372210052135,
                                         0.565082449306372,
                                         0.793491413705889,
                                         0.957142246963524,

                                         0.960794266397475,
                                         0.809643702061912,
                                         0.594001676572932,
                                         0.3782093752958,
                                         0.208018651652871,
                                         0.098201106959528,
                                         0.0394557276973378,
                                         0.013346205888368,
                                         0.00374821507298041,
                                         0.000858646883039867,
                                         0.000156845197531105,
                                         2.2184297041677e-05,
                                         2.33778195510771e-06,
                                         1.74315327871779e-07,
                                         8.56697558718433e-09,
                                         2.5086815998166e-10,
                                         3.76695602346321e-12,
                                         2.28840989673364e-14,
                                         3.74580706179264e-17,
                                         7.5110292517377e-21,
                                         2.83794585342275e-26,
                                         2.36374651053093e-36,
                                         0.0384818844426053,
                                         0.172261010034294,
                                         0.315227204746366,
                                         0.380806295672822,
                                         0.34563545037939,
                                         0.247876543164056,
                                         0.143470316599072,
                                         0.0674780096096391,
                                         0.0257329184556195,
                                         0.0078836546403814,
                                         0.0019095278961791,
                                         0.000357146962804978,
                                         4.99053470740396e-05,
                                         4.97653551187535e-06,
                                         3.32107862323891e-07,
                                         1.35222704207344e-08,
                                         2.9250114731879e-10,
                                         2.69944128597107e-12,
                                         7.29171433150701e-15,
                                         2.77408334250347e-18,
                                         2.61437046819572e-23,
                                         1.15672745494777e-32,
                                         0.000715595466412723,
                                         0.0170163087757899,
                                         0.0776685204355699,
                                         0.178016914878577,
                                         0.266636502291379,
                                         0.290495761050733,
                                         0.242214075209043,
                                         0.158398851453492,
                                         0.0820236149698211,
                                         0.0336067027716087,
                                         0.0107935947964415,
                                         0.00266952241426746,
                                         0.000494624335118859,
                                         6.59635406632691e-05,
                                         5.97745212011918e-06,
                                         3.38406744557278e-07,
                                         1.05450809661768e-08,
                                         1.47842492335712e-10,
                                         6.59020931554385e-13,
                                         4.75690936488368e-16,
                                         1.11818966546698e-20,
                                         2.6281279660873e-29,
                                         8.1888980545229e-06,
                                         0.00103440440328717,
                                         0.0117764124529295,
                                         0.0512112121334878,
                                         0.12658072014874,
                                         0.209503268438552,
                                         0.251642117692459,
                                         0.228817073769045,
                                         0.160892352710867,
                                         0.088159852266737,
                                         0.0375450638833531,
                                         0.0122791079345679,
                                         0.00301682774397586,
                                         0.000538055958061174,
                                         6.62063567092574e-05,
                                         5.21164755274352e-06,
                                         2.33947763655789e-07,
                                         4.98277525994332e-09,
                                         3.66534961976429e-11,
                                         5.01968984110727e-14,
                                         2.9431368322899e-18,
                                         3.6745876090536e-26,
                                         6.44252408870439e-08,
                                         4.323028175507e-05,
                                         0.00122759097276591,
                                         0.0101284162904088,
                                         0.0413131417520867,
                                         0.10387583036592,
                                         0.179738034619538,
                                         0.227246670527318,
                                         0.216972454269715,
                                         0.15899684441233,
                                         0.0897867574154243,
                                         0.0388304804890442,
                                         0.012650224991077,
                                         0.00301733571496041,
                                         0.000504145593587858,
                                         5.51802633146883e-05,
                                         3.56829359137958e-06,
                                         1.15455872110661e-07,
                                         1.40153615680313e-09,
                                         3.64167842302437e-12,
                                         5.32571762488098e-16,
                                         3.5321847190112e-23,
                                         3.68624281821402e-10,
                                         1.31396281038376e-06,
                                         9.30661324415006e-05,
                                         0.00145685165810842,
                                         0.00980632282085556,
                                         0.0374572177829958,
                                         0.0933671167179399,
                                         0.164136032858441,
                                         0.212799744645183,
                                         0.20854676535108,
                                         0.156159722808596,
                                         0.0893050491967767,
                                         0.0385783179384482,
                                         0.0123060060344005,
                                         0.00279196238896954,
                                         0.000424903003396056,
                                         3.95821667205844e-05,
                                         1.94562014641179e-06,
                                         3.89753934469707e-08,
                                         1.92142573911349e-10,
                                         7.00879116675471e-14,
                                         2.46930940366088e-20,
                                         1.581878382728e-12,
                                         2.99529322441167e-08,
                                         5.29164754370748e-06,
                                         0.000157163027235296,
                                         0.00174576351232066,
                                         0.0101301945729362,
                                         0.0363755166067413,
                                         0.0889142970279578,
                                         0.15653046192017,
                                         0.205153851476498,
                                         0.203698126507922,
                                         0.154042488882328,
                                         0.088236767488905,
                                         0.0376419295426381,
                                         0.0115964327774027,
                                         0.00245390133332687,
                                         0.000329306126760662,
                                         2.45901595448639e-05,
                                         8.12901590322411e-07,
                                         7.60338319740027e-09,
                                         6.9178217489906e-12,
                                         1.29469919102539e-17,
                                         5.17205257472022e-15,
                                         5.2023108978099e-10,
                                         2.29240233019412e-07,
                                         1.29177277223149e-05,
                                         0.000236791073454384,
                                         0.00208737628909396,
                                         0.0107975464303958,
                                         0.0366977936221114,
                                         0.0877257963140808,
                                         0.153764676571771,
                                         0.202444381317722,
                                         0.202444381317722,
                                         0.153764676571771,
                                         0.0877257963140808,
                                         0.0366977936221114,
                                         0.0107975464303958,
                                         0.00208737628909396,
                                         0.000236791073454383,
                                         1.29177277223148e-05,
                                         2.29240233019414e-07,
                                         5.2023108978099e-10,
                                         5.17205257472022e-15,
                                         1.29469919102539e-17,
                                         6.9178217489906e-12,
                                         7.60338319740018e-09,
                                         8.12901590322416e-07,
                                         2.4590159544864e-05,
                                         0.000329306126760662,
                                         0.00245390133332687,
                                         0.0115964327774027,
                                         0.0376419295426381,
                                         0.0882367674889049,
                                         0.154042488882328,
                                         0.203698126507922,
                                         0.205153851476498,
                                         0.15653046192017,
                                         0.0889142970279578,
                                         0.0363755166067413,
                                         0.0101301945729362,
                                         0.00174576351232066,
                                         0.000157163027235295,
                                         5.29164754370752e-06,
                                         2.99529322441167e-08,
                                         1.581878382728e-12,
                                         2.46930940366088e-20,
                                         7.00879116675471e-14,
                                         1.92142573911346e-10,
                                         3.8975393446971e-08,
                                         1.94562014641179e-06,
                                         3.95821667205844e-05,
                                         0.000424903003396056,
                                         0.00279196238896954,
                                         0.0123060060344005,
                                         0.0385783179384482,
                                         0.0893050491967768,
                                         0.156159722808596,
                                         0.20854676535108,
                                         0.212799744645183,
                                         0.164136032858441,
                                         0.0933671167179399,
                                         0.0374572177829958,
                                         0.00980632282085553,
                                         0.00145685165810841,
                                         9.30661324415013e-05,
                                         1.31396281038376e-06,
                                         3.68624281821402e-10,
                                         3.5321847190112e-23,
                                         5.32571762488098e-16,
                                         3.64167842302432e-12,
                                         1.40153615680314e-09,
                                         1.15455872110662e-07,
                                         3.56829359137958e-06,
                                         5.51802633146883e-05,
                                         0.000504145593587858,
                                         0.00301733571496041,
                                         0.012650224991077,
                                         0.0388304804890442,
                                         0.0897867574154243,
                                         0.15899684441233,
                                         0.216972454269715,
                                         0.227246670527318,
                                         0.179738034619538,
                                         0.10387583036592,
                                         0.0413131417520867,
                                         0.0101284162904088,
                                         0.00122759097276592,
                                         4.323028175507e-05,
                                         6.44252408870439e-08,
                                         3.6745876090536e-26,
                                         2.9431368322899e-18,
                                         5.01968984110719e-14,
                                         3.66534961976432e-11,
                                         4.98277525994334e-09,
                                         2.33947763655789e-07,
                                         5.21164755274352e-06,
                                         6.62063567092574e-05,
                                         0.000538055958061174,
                                         0.00301682774397585,
                                         0.0122791079345679,
                                         0.0375450638833531,
                                         0.088159852266737,
                                         0.160892352710867,
                                         0.228817073769045,
                                         0.251642117692459,
                                         0.209503268438552,
                                         0.12658072014874,
                                         0.0512112121334876,
                                         0.0117764124529296,
                                         0.00103440440328717,
                                         8.1888980545229e-06,
                                         2.6281279660873e-29,
                                         1.11818966546698e-20,
                                         4.75690936488359e-16,
                                         6.59020931554392e-13,
                                         1.47842492335713e-10,
                                         1.05450809661768e-08,
                                         3.38406744557278e-07,
                                         5.97745212011918e-06,
                                         6.59635406632691e-05,
                                         0.000494624335118859,
                                         0.00266952241426746,
                                         0.0107935947964415,
                                         0.0336067027716087,
                                         0.0820236149698211,
                                         0.158398851453492,
                                         0.242214075209043,
                                         0.290495761050733,
                                         0.266636502291379,
                                         0.178016914878576,
                                         0.0776685204355701,
                                         0.0170163087757899,
                                         0.000715595466412723,
                                         1.15672745494777e-32,
                                         2.61437046819572e-23,
                                         2.77408334250341e-18,
                                         7.29171433150709e-15,
                                         2.69944128597109e-12,
                                         2.9250114731879e-10,
                                         1.35222704207344e-08,
                                         3.32107862323891e-07,
                                         4.97653551187535e-06,
                                         4.99053470740395e-05,
                                         0.000357146962804978,
                                         0.0019095278961791,
                                         0.0078836546403814,
                                         0.0257329184556195,
                                         0.0674780096096391,
                                         0.143470316599072,
                                         0.247876543164056,
                                         0.34563545037939,
                                         0.380806295672822,
                                         0.315227204746366,
                                         0.172261010034294,
                                         0.0384818844426053,
                                         2.36374651053093e-36,
                                         2.83794585342275e-26,
                                         7.51102925173754e-21,
                                         3.74580706179268e-17,
                                         2.28840989673366e-14,
                                         3.76695602346321e-12,
                                         2.5086815998166e-10,
                                         8.56697558718433e-09,
                                         1.74315327871779e-07,
                                         2.33778195510771e-06,
                                         2.2184297041677e-05,
                                         0.000156845197531105,
                                         0.000858646883039867,
                                         0.00374821507298041,
                                         0.013346205888368,
                                         0.0394557276973378,
                                         0.098201106959528,
                                         0.208018651652871,
                                         0.3782093752958,
                                         0.594001676572932,
                                         0.809643702061912,
                                         0.960794266397475,

                                         0.964001316787059,
                                         0.824041607852556,
                                         0.620490407599725,
                                         0.4104664302037,
                                         0.23762110905339,
                                         0.119741433716321,
                                         0.052158595501911,
                                         0.0194647870307544,
                                         0.00615324091197491,
                                         0.00162439527197084,
                                         0.000351703639583216,
                                         6.103515625e-05,
                                         8.24225589225623e-06,
                                         8.33198525429233e-07,
                                         5.98682328926317e-08,
                                         2.84778963936682e-09,
                                         8.10567758609169e-11,
                                         1.18793140908799e-12,
                                         7.0720166583614e-15,
                                         1.13888720430998e-17,
                                         2.25554007703641e-21,
                                         8.4496812439108e-27,
                                         7.00449375706336e-37,
                                         0.035389129992081,
                                         0.160587693767394,
                                         0.301231182999322,
                                         0.377378651116915,
                                         0.359621614815066,
                                         0.27441734273807,
                                         0.171503402688596,
                                         0.0885486313054255,
                                         0.037777965288252,
                                         0.0132380436221592,
                                         0.00376588441489061,
                                         0.0008544921875,
                                         0.000150873179035241,
                                         2.0038835167361e-05,
                                         1.9112516198727e-06,
                                         1.22696388777637e-07,
                                         4.83168423269746e-09,
                                         1.01596871748943e-10,
                                         9.15880005498942e-13,
                                         2.42793525547211e-15,
                                         9.10629602464339e-19,
                                         8.49832384989123e-24,
                                         3.73973272731597e-33,
                                         0.000603180740273292,
                                         0.0145298356673288,
                                         0.0678969246685066,
                                         0.161087682025709,
                                         0.252692223022936,
                                         0.291987309523406,
                                         0.261821263631115,
                                         0.187024873645241,
                                         0.107685822239151,
                                         0.0500888683569783,
                                         0.0187215771007606,
                                         0.00555419921875,
                                         0.00128222225395068,
                                         0.00022375965005079,
                                         2.83285675376592e-05,
                                         2.4543753280309e-06,
                                         1.33718988764832e-07,
                                         4.03417401256972e-09,
                                         5.50705231464865e-11,
                                         2.40313855025477e-13,
                                         1.70694017829534e-16,
                                         3.9683644365105e-21,
                                         9.27021268869184e-30,
                                         6.32661898886074e-06,
                                         0.000809013518041763,
                                         0.00941774473380003,
                                         0.042314963578138,
                                         0.10926588810316,
                                         0.19118905773127,
                                         0.245970895808227,
                                         0.243088000840994,
                                         0.188897029469483,
                                         0.116628654848762,
                                         0.0572749224801692,
                                         0.022216796875,
                                         0.0067059639362299,
                                         0.00153757996019504,
                                         0.000258391365642348,
                                         3.02132056878106e-05,
                                         2.27737347509141e-06,
                                         9.85769901317374e-08,
                                         2.03772916265081e-09,
                                         1.46375082821877e-11,
                                         1.96898076089777e-14,
                                         1.14034550661425e-18,
                                         1.41411740125998e-26,
                                         4.56213988543529e-08,
                                         3.09687414957572e-05,
                                         0.000898082902267424,
                                         0.00764184655568013,
                                         0.0324825433291959,
                                         0.0860666287205646,
                                         0.158867562980241,
                                         0.217220283662159,
                                         0.227805618351656,
                                         0.186699010376653,
                                         0.120464558093831,
                                         0.06109619140625,
                                         0.0241119214088101,
                                         0.00726384132148721,
                                         0.00162033227423026,
                                         0.000255696827992108,
                                         2.66654019758755e-05,
                                         1.65603374576656e-06,
                                         5.18377833251154e-08,
                                         6.12954446481137e-10,
                                         1.56148331041798e-12,
                                         2.25285927739581e-16,
                                         1.48304366150371e-23,
                                         2.39255988312529e-10,
                                         8.62161523379929e-07,
                                         6.22849677259978e-05,
                                         0.00100369083038179,
                                         0.00702283968666459,
                                         0.0281775904474245,
                                         0.0746249487898279,
                                         0.141167437967927,
                                         0.199802553893866,
                                         0.217358219062166,
                                         0.184268616615416,
                                         0.1221923828125,
                                         0.0630521248958567,
                                         0.024956994809878,
                                         0.00738971126189397,
                                         0.00157380592316422,
                                         0.000227069766693002,
                                         2.02329921448119e-05,
                                         9.59055378208266e-07,
                                         1.86675166648635e-08,
                                         9.00597055412447e-11,
                                         3.23689771329516e-14,
                                         1.1311487227898e-20,
                                         9.41062133903801e-13,
                                         1.80017605614997e-08,
                                         3.2397486870372e-06,
                                         9.88696196317595e-05,
                                         0.00113877191122519,
                                         0.00691885416472469,
                                         0.0262902140503835,
                                         0.0688065769220107,
                                         0.131431330023936,
                                         0.189789150322344,
                                         0.211399873165878,
                                         0.18328857421875,
                                         0.123659902081573,
                                         0.0643101455249805,
                                         0.0252762196075358,
                                         0.00726504441774855,
                                         0.00145021287301412,
                                         0.000185401100149014,
                                         1.33076757840149e-05,
                                         4.26389163705595e-07,
                                         3.89569512593986e-09,
                                         3.48806966489056e-12,
                                         6.47063265709083e-18,
                                         2.82016467916686e-15,
                                         2.86379556742025e-10,
                                         1.28392628280272e-07,
                                         7.42038532962183e-06,
                                         0.000140689415694736,
                                         0.001294390118229,
                                         0.00705675250281083,
                                         0.0255520683313883,
                                         0.0658714856249889,
                                         0.126260464838681,
                                         0.184781614478124,
                                         0.20947265625,
                                         0.184781614478124,
                                         0.126260464838681,
                                         0.0658714856249889,
                                         0.0255520683313883,
                                         0.00705675250281082,
                                         0.001294390118229,
                                         0.000140689415694736,
                                         7.42038532962183e-06,
                                         1.28392628280273e-07,
                                         2.86379556742025e-10,
                                         2.82016467916644e-15,
                                         6.47063265709192e-18,
                                         3.48806966489056e-12,
                                         3.89569512593981e-09,
                                         4.26389163705595e-07,
                                         1.33076757840149e-05,
                                         0.000185401100149014,
                                         0.00145021287301412,
                                         0.00726504441774854,
                                         0.0252762196075358,
                                         0.0643101455249805,
                                         0.123659902081573,
                                         0.18328857421875,
                                         0.211399873165878,
                                         0.189789150322344,
                                         0.131431330023936,
                                         0.0688065769220108,
                                         0.0262902140503835,
                                         0.0069188541647247,
                                         0.00113877191122519,
                                         9.88696196317595e-05,
                                         3.23974868703723e-06,
                                         1.80017605614997e-08,
                                         9.41062133903681e-13,
                                         1.13114872279002e-20,
                                         3.23689771329516e-14,
                                         9.00597055412433e-11,
                                         1.86675166648635e-08,
                                         9.59055378208266e-07,
                                         2.02329921448118e-05,
                                         0.000227069766693002,
                                         0.00157380592316421,
                                         0.00738971126189396,
                                         0.024956994809878,
                                         0.0630521248958567,
                                         0.1221923828125,
                                         0.184268616615416,
                                         0.217358219062166,
                                         0.199802553893866,
                                         0.141167437967927,
                                         0.0746249487898278,
                                         0.0281775904474246,
                                         0.00702283968666459,
                                         0.00100369083038179,
                                         6.22849677259983e-05,
                                         8.62161523379929e-07,
                                         2.39255988312503e-10,
                                         1.48304366150402e-23,
                                         2.25285927739581e-16,
                                         1.56148331041796e-12,
                                         6.12954446481137e-10,
                                         5.18377833251154e-08,
                                         1.65603374576655e-06,
                                         2.66654019758756e-05,
                                         0.000255696827992107,
                                         0.00162033227423026,
                                         0.00726384132148721,
                                         0.0241119214088101,
                                         0.06109619140625,
                                         0.120464558093831,
                                         0.186699010376653,
                                         0.227805618351656,
                                         0.21722028366216,
                                         0.158867562980241,
                                         0.0860666287205647,
                                         0.0324825433291959,
                                         0.00764184655568012,
                                         0.00089808290226743,
                                         3.09687414957572e-05,
                                         4.5621398854349e-08,
                                         1.41411740126031e-26,
                                         1.14034550661425e-18,
                                         1.96898076089773e-14,
                                         1.46375082821877e-11,
                                         2.03772916265081e-09,
                                         9.85769901317369e-08,
                                         2.27737347509142e-06,
                                         3.02132056878105e-05,
                                         0.000258391365642348,
                                         0.00153757996019504,
                                         0.0067059639362299,
                                         0.022216796875,
                                         0.0572749224801692,
                                         0.116628654848762,
                                         0.188897029469483,
                                         0.243088000840994,
                                         0.245970895808227,
                                         0.191189057731271,
                                         0.10926588810316,
                                         0.042314963578138,
                                         0.00941774473380008,
                                         0.000809013518041763,
                                         6.32661898886034e-06,
                                         9.2702126886942e-30,
                                         3.9683644365105e-21,
                                         1.70694017829531e-16,
                                         2.40313855025477e-13,
                                         5.50705231464865e-11,
                                         4.0341740125697e-09,
                                         1.33718988764832e-07,
                                         2.45437532803089e-06,
                                         2.83285675376592e-05,
                                         0.00022375965005079,
                                         0.00128222225395068,
                                         0.00555419921875,
                                         0.0187215771007606,
                                         0.0500888683569783,
                                         0.107685822239151,
                                         0.187024873645241,
                                         0.261821263631115,
                                         0.291987309523406,
                                         0.252692223022936,
                                         0.161087682025709,
                                         0.0678969246685068,
                                         0.0145298356673288,
                                         0.000603180740273266,
                                         3.73973272731701e-33,
                                         8.49832384989123e-24,
                                         9.1062960246432e-19,
                                         2.42793525547211e-15,
                                         9.15880005498942e-13,
                                         1.01596871748943e-10,
                                         4.83168423269748e-09,
                                         1.22696388777637e-07,
                                         1.9112516198727e-06,
                                         2.0038835167361e-05,
                                         0.000150873179035241,
                                         0.0008544921875,
                                         0.00376588441489061,
                                         0.0132380436221592,
                                         0.037777965288252,
                                         0.0885486313054255,
                                         0.171503402688596,
                                         0.274417342738071,
                                         0.359621614815066,
                                         0.377378651116914,
                                         0.301231182999323,
                                         0.160587693767394,
                                         0.0353891299920803,
                                         7.00449375706544e-37,
                                         8.4496812439108e-27,
                                         2.25554007703636e-21,
                                         1.13888720430998e-17,
                                         7.0720166583614e-15,
                                         1.18793140908798e-12,
                                         8.10567758609172e-11,
                                         2.84778963936681e-09,
                                         5.98682328926317e-08,
                                         8.33198525429233e-07,
                                         8.24225589225623e-06,
                                         6.103515625e-05,
                                         0.000351703639583216,
                                         0.00162439527197084,
                                         0.00615324091197491,
                                         0.0194647870307544,
                                         0.052158595501911,
                                         0.119741433716321,
                                         0.23762110905339,
                                         0.4104664302037,
                                         0.620490407599725,
                                         0.824041607852556,
                                         0.964001316787059,

                                         0.966832456016739,
                                         0.836919622692484,
                                         0.644754192350639,
                                         0.441065546633236,
                                         0.267046030681879,
                                         0.142463459639,
                                         0.0665773451126075,
                                         0.0270543588670643,
                                         0.00947092928620572,
                                         0.00282317587248396,
                                         0.000706238707053033,
                                         0.000145575386583443,
                                         2.41586334164511e-05,
                                         3.13303998043345e-06,
                                         3.05400132332104e-07,
                                         2.12435889197437e-08,
                                         9.81985151093342e-10,
                                         2.72621107118977e-11,
                                         3.91112780304942e-13,
                                         2.28732590529286e-15,
                                         3.63116182213787e-18,
                                         7.11343832201248e-22,
                                         2.6448489869235e-27,
                                         2.18335423062133e-37,
                                         0.0326506335319127,
                                         0.149945845834879,
                                         0.287455860480514,
                                         0.371802902039138,
                                         0.369751559423692,
                                         0.297863392122962,
                                         0.199021375373196,
                                         0.111410791318685,
                                         0.0523611124684218,
                                         0.020586837327088,
                                         0.00671443004651047,
                                         0.00179267098195397,
                                         0.000384517224370123,
                                         6.4589864152511e-05,
                                         8.20868505373139e-06,
                                         7.53126083767399e-07,
                                         4.67380920610066e-08,
                                         1.78748455633113e-09,
                                         3.66643875276241e-11,
                                         3.23787614981051e-13,
                                         8.44290756154377e-16,
                                         3.12722154213796e-19,
                                         2.89338525360564e-24,
                                         1.26718458706862e-33,
                                         0.000511937949841585,
                                         0.0124729863518713,
                                         0.0595022626542924,
                                         0.145514991014296,
                                         0.237694549126587,
                                         0.289145272682967,
                                         0.276222018406761,
                                         0.213011275191489,
                                         0.134403462460992,
                                         0.0696990169629195,
                                         0.0296382196067184,
                                         0.0102494024437667,
                                         0.00284147967896714,
                                         0.000618227225050495,
                                         0.000102438517925234,
                                         1.23963209097331e-05,
                                         1.03281457871052e-06,
                                         5.44139734388402e-08,
                                         1.59577693133314e-09,
                                         2.12802995948049e-11,
                                         9.11430979351771e-14,
                                         6.38297361084353e-17,
                                         1.4695926398992e-21,
                                         3.41460823007815e-30,
                                         4.93957600182972e-06,
                                         0.000638488516325527,
                                         0.00757953266299101,
                                         0.0350468777015735,
                                         0.0940318683547364,
                                         0.172727583130858,
                                         0.235919315963333,
                                         0.250625098322503,
                                         0.212304247824229,
                                         0.145214608236939,
                                         0.0805084943113927,
                                         0.0360614460894592,
                                         0.0129217086606106,
                                         0.00364148492297529,
                                         0.000786682726782145,
                                         0.000125563833535738,
                                         1.40449557156094e-05,
                                         1.01935443557453e-06,
                                         4.27411856369872e-08,
                                         8.60680329929623e-10,
                                         6.05483304503645e-12,
                                         8.01740981551131e-15,
                                         4.59340043783723e-19,
                                         5.66224319227701e-27,
                                         3.27668521570125e-08,
                                         2.24702775350313e-05,
                                         0.000663779872702493,
                                         0.00580314780144893,
                                         0.025574291482098,
                                         0.0709381729362428,
                                         0.138529226331693,
                                         0.202730565077974,
                                         0.230557690352577,
                                         0.208001594756071,
                                         0.150350196395994,
                                         0.0872289074483578,
                                         0.0403987602700871,
                                         0.0147462505467656,
                                         0.00415344673118945,
                                         0.000874397737022956,
                                         0.0001313079712174,
                                         1.312842707975e-05,
                                         7.8703428294258e-07,
                                         2.39319848896614e-08,
                                         2.76537009090594e-10,
                                         6.92337444070784e-13,
                                         9.87061810124624e-17,
                                         6.45518742360397e-24,
                                         1.58080053081009e-10,
                                         5.75123461215543e-07,
                                         4.22768900017756e-05,
                                         0.000698835940764187,
                                         0.005058589729438,
                                         0.0211882751000927,
                                         0.0591584271695822,
                                         0.119264503158268,
                                         0.182094925733784,
                                         0.216680746811899,
                                         0.204203695294163,
                                         0.153452874967576,
                                         0.0918572485884654,
                                         0.0434292173335437,
                                         0.0159483213258771,
                                         0.0044284402079883,
                                         0.000892810132074928,
                                         0.00012296951986884,
                                         1.05399375468536e-05,
                                         4.83963784413379e-07,
                                         9.18547569015117e-09,
                                         4.34809334814528e-11,
                                         1.54259419505185e-14,
                                         5.35212805999376e-21,
                                         5.71979795213863e-13,
                                         1.104015054303e-08,
                                         2.01949716509029e-06,
                                         6.31172540507021e-05,
                                         0.00075044102596219,
                                         0.00474648891211971,
                                         0.018947551346308,
                                         0.0526216472641322,
                                         0.107864202861608,
                                         0.16929153630056,
                                         0.208010116571268,
                                         0.202465434263481,
                                         0.15664628187534,
                                         0.0959276179505439,
                                         0.0459285328983717,
                                         0.0168210773930782,
                                         0.00455290294571634,
                                         0.000863860311954118,
                                         0.000105862875892992,
                                         7.34020639231128e-06,
                                         2.28829128328491e-07,
                                         2.04805285985503e-09,
                                         1.8080910634466e-12,
                                         3.32816921773199e-18,
                                         1.5768304235443e-15,
                                         1.61469163183593e-10,
                                         7.3499460562184e-08,
                                         4.34331820513605e-06,
                                         8.48211911590661e-05,
                                         0.000810121254018294,
                                         0.00462370645446157,
                                         0.0176896142605053,
                                         0.0486807791499741,
                                         0.100774547446708,
                                         0.161438088183889,
                                         0.203529533487565,
                                         0.203529533487565,
                                         0.161438088183889,
                                         0.100774547446708,
                                         0.0486807791499741,
                                         0.0176896142605052,
                                         0.00462370645446157,
                                         0.000810121254018292,
                                         8.48211911590661e-05,
                                         4.34331820513605e-06,
                                         7.34994605621831e-08,
                                         1.61469163183593e-10,
                                         1.57683042354404e-15,
                                         3.3281692177326e-18,
                                         1.8080910634466e-12,
                                         2.04805285985506e-09,
                                         2.28829128328491e-07,
                                         7.34020639231128e-06,
                                         0.000105862875892993,
                                         0.000863860311954118,
                                         0.00455290294571635,
                                         0.0168210773930782,
                                         0.0459285328983717,
                                         0.0959276179505439,
                                         0.15664628187534,
                                         0.202465434263482,
                                         0.208010116571268,
                                         0.16929153630056,
                                         0.107864202861608,
                                         0.0526216472641321,
                                         0.018947551346308,
                                         0.0047464889121197,
                                         0.00075044102596219,
                                         6.31172540507021e-05,
                                         2.01949716509026e-06,
                                         1.104015054303e-08,
                                         5.71979795213784e-13,
                                         5.35212805999487e-21,
                                         1.54259419505185e-14,
                                         4.34809334814535e-11,
                                         9.18547569015117e-09,
                                         4.83963784413379e-07,
                                         1.05399375468536e-05,
                                         0.00012296951986884,
                                         0.00089281013207493,
                                         0.00442844020798829,
                                         0.0159483213258771,
                                         0.0434292173335437,
                                         0.0918572485884653,
                                         0.153452874967576,
                                         0.204203695294163,
                                         0.216680746811899,
                                         0.182094925733784,
                                         0.119264503158268,
                                         0.0591584271695821,
                                         0.0211882751000927,
                                         0.005058589729438,
                                         0.000698835940764187,
                                         4.22768900017752e-05,
                                         5.75123461215543e-07,
                                         1.58080053080991e-10,
                                         6.45518742360546e-24,
                                         9.87061810124624e-17,
                                         6.92337444070797e-13,
                                         2.76537009090594e-10,
                                         2.39319848896614e-08,
                                         7.87034282942583e-07,
                                         1.312842707975e-05,
                                         0.0001313079712174,
                                         0.000874397737022956,
                                         0.00415344673118945,
                                         0.0147462505467656,
                                         0.0403987602700871,
                                         0.0872289074483578,
                                         0.150350196395994,
                                         0.208001594756071,
                                         0.230557690352577,
                                         0.202730565077974,
                                         0.138529226331693,
                                         0.0709381729362426,
                                         0.025574291482098,
                                         0.00580314780144893,
                                         0.000663779872702488,
                                         2.24702775350313e-05,
                                         3.27668521570095e-08,
                                         5.66224319227845e-27,
                                         4.59340043783723e-19,
                                         8.01740981551147e-15,
                                         6.05483304503645e-12,
                                         8.60680329929623e-10,
                                         4.27411856369874e-08,
                                         1.01935443557453e-06,
                                         1.40449557156094e-05,
                                         0.000125563833535738,
                                         0.000786682726782145,
                                         0.00364148492297529,
                                         0.0129217086606106,
                                         0.0360614460894593,
                                         0.0805084943113927,
                                         0.145214608236939,
                                         0.212304247824229,
                                         0.250625098322503,
                                         0.235919315963333,
                                         0.172727583130858,
                                         0.0940318683547364,
                                         0.0350468777015735,
                                         0.00757953266299097,
                                         0.000638488516325527,
                                         4.93957600182938e-06,
                                         3.41460823007909e-30,
                                         1.4695926398992e-21,
                                         6.38297361084366e-17,
                                         9.11430979351772e-14,
                                         2.12802995948049e-11,
                                         1.59577693133315e-09,
                                         5.44139734388402e-08,
                                         1.03281457871052e-06,
                                         1.23963209097331e-05,
                                         0.000102438517925234,
                                         0.000618227225050495,
                                         0.00284147967896714,
                                         0.0102494024437667,
                                         0.0296382196067184,
                                         0.0696990169629195,
                                         0.134403462460992,
                                         0.213011275191489,
                                         0.276222018406761,
                                         0.289145272682967,
                                         0.237694549126587,
                                         0.145514991014296,
                                         0.0595022626542922,
                                         0.0124729863518713,
                                         0.000511937949841562,
                                         1.267184587069e-33,
                                         2.89338525360564e-24,
                                         3.12722154213803e-19,
                                         8.44290756154377e-16,
                                         3.23787614981051e-13,
                                         3.66643875276244e-11,
                                         1.78748455633113e-09,
                                         4.67380920610067e-08,
                                         7.53126083767399e-07,
                                         8.20868505373139e-06,
                                         6.4589864152511e-05,
                                         0.000384517224370123,
                                         0.00179267098195397,
                                         0.00671443004651047,
                                         0.020586837327088,
                                         0.0523611124684218,
                                         0.111410791318685,
                                         0.199021375373196,
                                         0.297863392122962,
                                         0.369751559423692,
                                         0.371802902039138,
                                         0.287455860480514,
                                         0.149945845834879,
                                         0.0326506335319119,
                                         2.18335423062203e-37,
                                         2.6448489869235e-27,
                                         7.11343832201266e-22,
                                         3.63116182213787e-18,
                                         2.28732590529286e-15,
                                         3.91112780304945e-13,
                                         2.72621107118977e-11,
                                         9.81985151093345e-10,
                                         2.12435889197437e-08,
                                         3.05400132332104e-07,
                                         3.13303998043345e-06,
                                         2.41586334164511e-05,
                                         0.000145575386583443,
                                         0.000706238707053033,
                                         0.00282317587248396,
                                         0.00947092928620572,
                                         0.0270543588670643,
                                         0.0665773451126075,
                                         0.142463459639,
                                         0.267046030681879,
                                         0.441065546633236,
                                         0.644754192350639,
                                         0.836919622692484,
                                         0.966832456016739,

                                         3.0517578125e-05,
                                         0.000457763671875,
                                         0.003204345703125,
                                         0.013885498046875,
                                         0.041656494140625,
                                         0.091644287109375,
                                         0.152740478515625,
                                         0.196380615234375,
                                         0.196380615234375,
                                         0.152740478515625,
                                         0.091644287109375,
                                         0.041656494140625,
                                         0.013885498046875,
                                         0.003204345703125,
                                         0.000457763671875,
                                         3.0517578125e-05,

                                         0.0284100650828888,
                                         7.48615846012771e-11,
                                         0.114186809938135,
                                         4.19080855900623e-09,
                                         0.214173844564466,
                                         1.0948217326904e-07,
                                         0.248680070993976,
                                         1.77056981037351e-06,
                                         0.19990087258965,
                                         1.98235694719846e-05,
                                         0.117839210228682,
                                         0.000162761650605,
                                         0.0526248686958275,
                                         0.00101239124930272,
                                         0.0181295885170811,
                                         0.00485780860226043,
                                         0.00485780860226043,
                                         0.0181295885170811,
                                         0.00101239124930272,
                                         0.0526248686958275,
                                         0.000162761650605,
                                         0.117839210228682,
                                         1.98235694719846e-05,
                                         0.19990087258965,
                                         1.77056981037351e-06,
                                         0.248680070993976,
                                         1.0948217326904e-07,
                                         0.214173844564466,
                                         4.19080855900623e-09,
                                         0.114186809938135,
                                         7.48615846012771e-11,
                                         0.0284100650828888,

                                         0.166358499999993,
                                         3.0517578125e-05,
                                         6.01111455080482e-15,
                                         0.31695449999929,
                                         0.000457763671875,
                                         7.0988107125947e-13,
                                         0.281809499960878,
                                         0.003204345703125,
                                         3.91221729626957e-11,
                                         0.155109498665298,
                                         0.013885498046875,
                                         1.33470227021191e-09,
                                         0.059104468475734,
                                         0.041656494140625,
                                         3.15242662365708e-08,
                                         0.0165159539819493,
                                         0.091644287109375,
                                         5.46018050780246e-07,
                                         0.00349633534829916,
                                         0.152740478515625,
                                         7.16465170086917e-06,
                                         0.000570976478328454,
                                         0.196380615234375,
                                         7.25235216715537e-05,
                                         7.25235216715537e-05,
                                         0.196380615234375,
                                         0.000570976478328454,
                                         7.16465170086917e-06,
                                         0.152740478515625,
                                         0.00349633534829916,
                                         5.46018050780246e-07,
                                         0.091644287109375,
                                         0.0165159539819493,
                                         3.15242662365708e-08,
                                         0.041656494140625,
                                         0.059104468475734,
                                         1.33470227021191e-09,
                                         0.013885498046875,
                                         0.155109498665298,
                                         3.91221729626957e-11,
                                         0.003204345703125,
                                         0.281809499960878,
                                         7.0988107125947e-13,
                                         0.000457763671875,
                                         0.31695449999929,
                                         6.01111455080482e-15,
                                         3.0517578125e-05,
                                         0.166358499999993,

                                         0.339799552267194,
                                         0.00246053690414363,
                                         5.99647740444206e-08,
                                         4.20126922498801e-18,
                                         0.380298467548281,
                                         0.0181793728441389,
                                         1.82612165275067e-06,
                                         8.44619799369336e-16,
                                         0.198624642907304,
                                         0.0626808231744762,
                                         2.59519497421622e-05,
                                         7.92407876601644e-14,
                                         0.0642193256807331,
                                         0.133787193497991,
                                         0.00022831494191054,
                                         4.60214573626263e-12,
                                         0.014374657635901,
                                         0.197694029158845,
                                         0.00139058594220832,
                                         1.85042338411039e-10,
                                         0.0023595582581511,
                                         0.214226892512497,
                                         0.00621101761492126,
                                         5.45611197668205e-09,
                                         0.000293420172660483,
                                         0.175865435695377,
                                         0.0210161788649478,
                                         1.21876931213302e-07,
                                         2.81478320967272e-05,
                                         0.111373717564968,
                                         0.0548580632474074,
                                         2.10017490993718e-06,
                                         2.10017490993717e-06,
                                         0.0548580632474074,
                                         0.111373717564968,
                                         2.81478320967274e-05,
                                         1.21876931213301e-07,
                                         0.0210161788649478,
                                         0.175865435695377,
                                         0.000293420172660485,
                                         5.45611197668201e-09,
                                         0.00621101761492125,
                                         0.214226892512497,
                                         0.00235955825815111,
                                         1.85042338411038e-10,
                                         0.00139058594220832,
                                         0.197694029158845,
                                         0.014374657635901,
                                         4.60214573626259e-12,
                                         0.00022831494191054,
                                         0.133787193497991,
                                         0.0642193256807332,
                                         7.92407876601635e-14,
                                         2.59519497421622e-05,
                                         0.0626808231744762,
                                         0.198624642907304,
                                         8.44619799369326e-16,
                                         1.82612165275067e-06,
                                         0.0181793728441389,
                                         0.380298467548282,
                                         4.20126922498796e-18,
                                         5.99647740444206e-08,
                                         0.00246053690414363,
                                         0.339799552267194,

                                         0.486416352273621,
                                         0.0195381132831772,
                                         3.0517578125e-05,
                                         2.80258531364046e-10,
                                         1.1721742899461e-20,
                                         0.35911346879595,
                                         0.0879195851706524,
                                         0.000457763671875,
                                         1.40132333181659e-08,
                                         3.57232680404995e-18,
                                         0.123726293339032,
                                         0.184627087279188,
                                         0.003204345703125,
                                         3.2698260174637e-07,
                                         5.08062281242289e-16,
                                         0.0263886010990511,
                                         0.240009959525018,
                                         0.013885498046875,
                                         4.72318541585058e-06,
                                         4.47308121187654e-14,
                                         0.00389645703030235,
                                         0.216004235131891,
                                         0.041656494140625,
                                         4.72328880873909e-05,
                                         2.72643890024471e-12,
                                         0.000421914878200759,
                                         0.142559674484549,
                                         0.091644287109375,
                                         0.000346382094952001,
                                         1.21867017344166e-10,
                                         3.46103394160346e-05,
                                         0.0712782769251813,
                                         0.152740478515625,
                                         0.00192438709682842,
                                         4.12669777032907e-09,
                                         2.19019405655867e-06,
                                         0.0274924478477224,
                                         0.196380615234375,
                                         0.00824755381124322,
                                         1.07799032839617e-07,
                                         1.07799032839616e-07,
                                         0.00824755381124322,
                                         0.196380615234375,
                                         0.0274924478477224,
                                         2.19019405655869e-06,
                                         4.12669777032902e-09,
                                         0.00192438709682842,
                                         0.152740478515625,
                                         0.0712782769251813,
                                         3.46103394160349e-05,
                                         1.21867017344165e-10,
                                         0.000346382094952001,
                                         0.091644287109375,
                                         0.142559674484549,
                                         0.000421914878200761,
                                         2.72643890024467e-12,
                                         4.72328880873909e-05,
                                         0.041656494140625,
                                         0.216004235131891,
                                         0.00389645703030237,
                                         4.47308121187647e-14,
                                         4.72318541585058e-06,
                                         0.013885498046875,
                                         0.240009959525018,
                                         0.0263886010990512,
                                         5.08062281242281e-16,
                                         3.2698260174637e-07,
                                         0.003204345703125,
                                         0.184627087279188,
                                         0.123726293339032,
                                         3.57232680404989e-18,
                                         1.40132333181659e-08,
                                         0.000457763671875,
                                         0.0879195851706524,
                                         0.359113468795951,
                                         1.17217428994608e-20,
                                         2.80258531364046e-10,
                                         3.0517578125e-05,
                                         0.0195381132831772,
                                         0.486416352273621,

                                         0.597365086214042,
                                         0.0617896465056269,
                                         0.000756165934853477,
                                         5.11186054037316e-07,
                                         2.71344238932865e-12,
                                         8.45372612777999e-23,
                                         0.313125414193748,
                                         0.18902270015044,
                                         0.00697224265622141,
                                         1.24740112165074e-05,
                                         1.99574417946766e-10,
                                         3.62870957403657e-20,
                                         0.0765955572716687,
                                         0.269847870799699,
                                         0.0300009259798751,
                                         0.000142049609557274,
                                         6.85008433538222e-09,
                                         7.26880516478538e-18,
                                         0.0115987957390704,
                                         0.238477871182187,
                                         0.079913634491005,
                                         0.00100137793127282,
                                         1.45549602260514e-07,
                                         9.01359215068836e-16,
                                         0.0012159658486113,
                                         0.145907069181504,
                                         0.14736904309762,
                                         0.00488714410262383,
                                         2.14104248299242e-06,
                                         7.73805719259868e-14,
                                         9.34827072325155e-05,
                                         0.0654645229898067,
                                         0.199293470607514,
                                         0.0174909621414682,
                                         2.30962234503435e-05,
                                         4.87155257130132e-12,
                                         5.44461597506702e-06,
                                         0.0222516259229556,
                                         0.204176589765367,
                                         0.047424015454297,
                                         0.000188748161130367,
                                         2.32342640507536e-10,
                                         2.44623933383903e-07,
                                         0.00583462854917643,
                                         0.161366931661228,
                                         0.0991924613698261,
                                         0.0011899266895679,
                                         8.54842621709484e-09,
                                         8.54842621709484e-09,
                                         0.0011899266895679,
                                         0.0991924613698261,
                                         0.161366931661228,
                                         0.00583462854917642,
                                         2.44623933383903e-07,
                                         2.32342640507536e-10,
                                         0.000188748161130368,
                                         0.047424015454297,
                                         0.204176589765367,
                                         0.0222516259229555,
                                         5.44461597506702e-06,
                                         4.87155257130132e-12,
                                         2.30962234503436e-05,
                                         0.0174909621414683,
                                         0.199293470607514,
                                         0.0654645229898066,
                                         9.34827072325155e-05,
                                         7.73805719259868e-14,
                                         2.14104248299243e-06,
                                         0.00488714410262383,
                                         0.14736904309762,
                                         0.145907069181503,
                                         0.0012159658486113,
                                         9.01359215068836e-16,
                                         1.45549602260515e-07,
                                         0.00100137793127282,
                                         0.079913634491005,
                                         0.238477871182187,
                                         0.0115987957390704,
                                         7.26880516478537e-18,
                                         6.85008433538225e-09,
                                         0.000142049609557274,
                                         0.0300009259798751,
                                         0.269847870799699,
                                         0.0765955572716687,
                                         3.62870957403657e-20,
                                         1.99574417946767e-10,
                                         1.24740112165074e-05,
                                         0.00697224265622141,
                                         0.18902270015044,
                                         0.313125414193748,
                                         8.45372612777999e-23,
                                         2.71344238932866e-12,
                                         5.11186054037316e-07,
                                         0.000756165934853477,
                                         0.0617896465056269,
                                         0.597365086214042,

                                         0.679341791908571,
                                         0.125463934196471,
                                         0.00505373396006408,
                                         3.0517578125e-05,
                                         1.23892423470185e-08,
                                         4.68459796886025e-14,
                                         1.21423867950355e-24,
                                         0.266068813872007,
                                         0.279310367135083,
                                         0.0320380292212002,
                                         0.000457763671875,
                                         4.39717600298301e-07,
                                         4.73463702401688e-12,
                                         6.97558990004279e-22,
                                         0.0486302380636787,
                                         0.290176334642259,
                                         0.0947820268000844,
                                         0.003204345703125,
                                         7.28299029203291e-06,
                                         2.23309826922768e-10,
                                         1.87009900071329e-19,
                                         0.00550228238510904,
                                         0.186621202351443,
                                         0.173584221916815,
                                         0.013885498046875,
                                         7.46740438530169e-05,
                                         6.52008169857646e-09,
                                         3.1036469551121e-17,
                                         0.000431001232437599,
                                         0.0830919847648746,
                                         0.220086629729901,
                                         0.041656494140625,
                                         0.000530064558395223,
                                         1.31794533553989e-07,
                                         3.56598232601684e-15,
                                         2.47580009284883e-05,
                                         0.0271305569059637,
                                         0.204634332376079,
                                         0.091644287109375,
                                         0.00275923880843824,
                                         1.95363390271182e-06,
                                         3.00460568845116e-13,
                                         1.07740430505995e-06,
                                         0.00671095540856776,
                                         0.144141622635054,
                                         0.152740478515625,
                                         0.0108811771581316,
                                         2.19389095657241e-05,
                                         1.91788186547202e-11,
                                         3.6169087985155e-08,
                                         0.00128057705939776,
                                         0.0783242120063952,
                                         0.196380615234375,
                                         0.0331023016884537,
                                         0.000190056449765633,
                                         9.44391218447499e-10,
                                         9.44391218447483e-10,
                                         0.000190056449765633,
                                         0.0331023016884537,
                                         0.196380615234375,
                                         0.0783242120063952,
                                         0.00128057705939776,
                                         3.61690879851556e-08,
                                         1.91788186547198e-11,
                                         2.19389095657241e-05,
                                         0.0108811771581316,
                                         0.152740478515625,
                                         0.144141622635054,
                                         0.00671095540856776,
                                         1.07740430505997e-06,
                                         3.0046056884511e-13,
                                         1.95363390271182e-06,
                                         0.00275923880843824,
                                         0.091644287109375,
                                         0.204634332376079,
                                         0.0271305569059637,
                                         2.47580009284886e-05,
                                         3.56598232601675e-15,
                                         1.31794533553989e-07,
                                         0.000530064558395223,
                                         0.041656494140625,
                                         0.220086629729901,
                                         0.0830919847648746,
                                         0.000431001232437602,
                                         3.10364695511202e-17,
                                         6.52008169857646e-09,
                                         7.46740438530169e-05,
                                         0.013885498046875,
                                         0.173584221916815,
                                         0.186621202351443,
                                         0.00550228238510908,
                                         1.87009900071323e-19,
                                         2.23309826922768e-10,
                                         7.28299029203291e-06,
                                         0.003204345703125,
                                         0.0947820268000844,
                                         0.290176334642259,
                                         0.0486302380636789,
                                         6.97558990004258e-22,
                                         4.73463702401687e-12,
                                         4.39717600298301e-07,
                                         0.000457763671875,
                                         0.0320380292212002,
                                         0.279310367135083,
                                         0.266068813872008,
                                         1.21423867950352e-24,
                                         4.68459796886025e-14,
                                         1.23892423470185e-08,
                                         3.0517578125e-05,
                                         0.00505373396006408,
                                         0.125463934196471,
                                         0.679341791908571,

                                         0.7402091598677,
                                         0.200245163698839,
                                         0.017213597544238,
                                         0.000381693037008891,
                                         1.46020668670104e-06,
                                         4.24275882129284e-10,
                                         1.28140033358523e-15,
                                         2.93813612196334e-26,
                                         0.224919380132152,
                                         0.339934164417507,
                                         0.0803060042224809,
                                         0.0039505009406115,
                                         3.17438003406225e-05,
                                         2.04623020347898e-08,
                                         1.69837884640094e-13,
                                         2.1756148160997e-23,
                                         0.0318937864334196,
                                         0.269298772906242,
                                         0.174836124303289,
                                         0.0190808133203054,
                                         0.00032204034696819,
                                         4.60540064518311e-07,
                                         1.05048799151481e-11,
                                         7.51794049489384e-21,
                                         0.00279968611356653,
                                         0.132068118846875,
                                         0.235634231092236,
                                         0.0570513142231655,
                                         0.00202248686809648,
                                         6.41659164870719e-06,
                                         4.02227416947779e-10,
                                         1.60819911087884e-18,
                                         0.000170142089390119,
                                         0.0448395005373755,
                                         0.219859253807038,
                                         0.118095563004199,
                                         0.00879347011859424,
                                         6.18928588119689e-05,
                                         1.06623124480584e-08,
                                         2.38166079965579e-16,
                                         7.58255565053893e-06,
                                         0.0111641321089747,
                                         0.150436265524971,
                                         0.179268066655419,
                                         0.0280373071855262,
                                         0.000437802372329464,
                                         2.07268147561836e-07,
                                         2.58655326067139e-14,
                                         2.56003274209922e-07,
                                         0.0021057908603468,
                                         0.0779805702595499,
                                         0.206157128977423,
                                         0.0677233412298479,
                                         0.00234607421766179,
                                         3.05238840947296e-06,
                                         2.12808519980549e-12,
                                         6.66762479760896e-09,
                                         0.000306408794097702,
                                         0.0311828632075365,
                                         0.182889806274228,
                                         0.12619326381158,
                                         0.00969842257156642,
                                         3.46770979692437e-05,
                                         1.35067952658139e-10,
                                         1.35067952658139e-10,
                                         3.46770979692437e-05,
                                         0.00969842257156642,
                                         0.12619326381158,
                                         0.182889806274228,
                                         0.0311828632075365,
                                         0.000306408794097702,
                                         6.66762479760896e-09,
                                         2.12808519980549e-12,
                                         3.05238840947296e-06,
                                         0.00234607421766179,
                                         0.067723341229848,
                                         0.206157128977423,
                                         0.0779805702595499,
                                         0.00210579086034679,
                                         2.56003274209922e-07,
                                         2.58655326067139e-14,
                                         2.07268147561836e-07,
                                         0.000437802372329464,
                                         0.0280373071855262,
                                         0.179268066655419,
                                         0.150436265524971,
                                         0.0111641321089747,
                                         7.58255565053893e-06,
                                         2.38166079965579e-16,
                                         1.06623124480584e-08,
                                         6.18928588119689e-05,
                                         0.00879347011859426,
                                         0.118095563004199,
                                         0.219859253807038,
                                         0.0448395005373755,
                                         0.000170142089390119,
                                         1.60819911087884e-18,
                                         4.02227416947779e-10,
                                         6.41659164870719e-06,
                                         0.00202248686809649,
                                         0.0570513142231655,
                                         0.235634231092236,
                                         0.132068118846875,
                                         0.00279968611356653,
                                         7.51794049489384e-21,
                                         1.05048799151481e-11,
                                         4.60540064518311e-07,
                                         0.000322040346968191,
                                         0.0190808133203054,
                                         0.174836124303289,
                                         0.269298772906242,
                                         0.0318937864334196,
                                         2.1756148160997e-23,
                                         1.69837884640094e-13,
                                         2.04623020347898e-08,
                                         3.17438003406225e-05,
                                         0.0039505009406115,
                                         0.0803060042224809,
                                         0.339934164417507,
                                         0.224919380132152,
                                         2.93813612196334e-26,
                                         1.28140033358523e-15,
                                         4.24275882129284e-10,
                                         1.46020668670105e-06,
                                         0.000381693037008891,
                                         0.017213597544238,
                                         0.200245163698839,
                                         0.7402091598677,

                                         0.786062132293865,
                                         0.277172788310735,
                                         0.0398625894764617,
                                         0.00206118337064087,
                                         3.0517578125e-05,
                                         8.53705104068412e-08,
                                         1.96770127465014e-11,
                                         5.0812670660812e-17,
                                         1.06929355190458e-27,
                                         0.190746892874607,
                                         0.371298571662064,
                                         0.143290152029633,
                                         0.0157768624399792,
                                         0.000457763671875,
                                         2.5094953029606e-06,
                                         1.23166003230845e-09,
                                         8.53457406998298e-15,
                                         9.91468381361345e-25,
                                         0.0216005520450813,
                                         0.232114754388307,
                                         0.24036651167313,
                                         0.0563548669490269,
                                         0.003204345703125,
                                         3.44248199360551e-05,
                                         3.59773616151514e-08,
                                         6.6895740145303e-13,
                                         4.29010153256092e-22,
                                         0.00151424548123117,
                                         0.0898268713564129,
                                         0.249606353496243,
                                         0.124613839176623,
                                         0.013885498046875,
                                         0.000292335141595097,
                                         6.505667029646e-07,
                                         3.24592986729525e-11,
                                         1.1491595485271e-19,
                                         7.34897684872267e-05,
                                         0.0240662795469785,
                                         0.179447109732589,
                                         0.190765695746516,
                                         0.041656494140625,
                                         0.00171865826086107,
                                         8.14429524150864e-06,
                                         1.09038271431642e-09,
                                         2.13104316485352e-17,
                                         2.61552513696197e-06,
                                         0.00472838756551855,
                                         0.0946060098614465,
                                         0.214158694868979,
                                         0.091644287109375,
                                         0.00740968014008366,
                                         7.47681461372675e-05,
                                         2.68608783618572e-08,
                                         2.89804809895986e-15,
                                         7.0520760328172e-08,
                                         0.000703790248210511,
                                         0.0377856635172749,
                                         0.182136594034051,
                                         0.152740478515625,
                                         0.02420111974832,
                                         0.000520002922945051,
                                         5.01288256961541e-07,
                                         2.98569197357267e-13,
                                         1.46679963650851e-09,
                                         8.08107510438458e-05,
                                         0.011642093948322,
                                         0.119496292287728,
                                         0.196380615234375,
                                         0.0609771581498467,
                                         0.0027899131051741,
                                         7.2168980753576e-06,
                                         2.37290380017112e-11,
                                         2.37290380017105e-11,
                                         7.2168980753576e-06,
                                         0.00278991310517409,
                                         0.0609771581498467,
                                         0.196380615234375,
                                         0.119496292287728,
                                         0.011642093948322,
                                         8.08107510438458e-05,
                                         1.46679963650854e-09,
                                         2.98569197357258e-13,
                                         5.01288256961541e-07,
                                         0.00052000292294505,
                                         0.0242011197483199,
                                         0.152740478515625,
                                         0.182136594034051,
                                         0.0377856635172749,
                                         0.000703790248210511,
                                         7.05207603281735e-08,
                                         2.89804809895976e-15,
                                         2.68608783618572e-08,
                                         7.47681461372673e-05,
                                         0.00740968014008366,
                                         0.091644287109375,
                                         0.214158694868979,
                                         0.0946060098614466,
                                         0.00472838756551854,
                                         2.61552513696202e-06,
                                         2.13104316485343e-17,
                                         1.09038271431642e-09,
                                         8.14429524150861e-06,
                                         0.00171865826086107,
                                         0.041656494140625,
                                         0.190765695746516,
                                         0.179447109732589,
                                         0.0240662795469785,
                                         7.34897684872277e-05,
                                         1.14915954852705e-19,
                                         3.24592986729525e-11,
                                         6.50566702964598e-07,
                                         0.000292335141595097,
                                         0.013885498046875,
                                         0.124613839176623,
                                         0.249606353496243,
                                         0.0898268713564129,
                                         0.00151424548123118,
                                         4.29010153256072e-22,
                                         6.6895740145303e-13,
                                         3.59773616151513e-08,
                                         3.44248199360551e-05,
                                         0.003204345703125,
                                         0.0563548669490269,
                                         0.24036651167313,
                                         0.232114754388307,
                                         0.0216005520450814,
                                         9.91468381361296e-25,
                                         8.53457406998298e-15,
                                         1.23166003230845e-09,
                                         2.5094953029606e-06,
                                         0.000457763671875,
                                         0.0157768624399792,
                                         0.143290152029633,
                                         0.371298571662064,
                                         0.190746892874608,
                                         1.06929355190452e-27,
                                         5.0812670660812e-17,
                                         1.96770127465013e-11,
                                         8.53705104068412e-08,
                                         3.0517578125e-05,
                                         0.00206118337064087,
                                         0.0398625894764617,
                                         0.277172788310735,
                                         0.786062132293865,

                                         0.821199765271958,
                                         0.35071467810852,
                                         0.0727611263308118,
                                         0.0067614545730708,
                                         0.00024470227306177,
                                         2.71928878900493e-06,
                                         6.07787232823815e-09,
                                         1.18524575837906e-12,
                                         2.73210960864401e-18,
                                         5.40166925948599e-29,
                                         0.162834098377439,
                                         0.380611126372081,
                                         0.208345730622492,
                                         0.0400908708006739,
                                         0.00271925801864037,
                                         5.50586344599844e-05,
                                         2.30636870344894e-07,
                                         9.31334595754981e-11,
                                         5.66438937280603e-16,
                                         6.12934363095771e-26,
                                         0.0150677592314611,
                                         0.19275950317536,
                                         0.278404710136788,
                                         0.11093219986228,
                                         0.0141016396662877,
                                         0.000520238298729387,
                                         4.0842534100564e-06,
                                         3.41515046525276e-09,
                                         5.48043285078344e-14,
                                         3.24568771604203e-23,
                                         0.000863129674036264,
                                         0.0604330028320905,
                                         0.230299295386611,
                                         0.190017595820408,
                                         0.0452702471968233,
                                         0.00304300918427516,
                                         4.47734625728111e-05,
                                         7.75243274263693e-08,
                                         3.28246935385508e-12,
                                         1.06395571389973e-20,
                                         3.42296596268418e-05,
                                         0.0131169150957816,
                                         0.131888763626381,
                                         0.225335267775995,
                                         0.100613272737944,
                                         0.0123226286970933,
                                         0.000339803494532792,
                                         1.21833109520774e-06,
                                         1.36108628041218e-10,
                                         2.41456848442177e-18,
                                         9.95475416773617e-07,
                                         0.00208780852540437,
                                         0.0553891130694775,
                                         0.195959330162372,
                                         0.163983107458787,
                                         0.0365935790730247,
                                         0.00189119549971246,
                                         1.4040871878623e-05,
                                         4.13877730314137e-09,
                                         4.01843483945581e-16,
                                         2.19323196785749e-08,
                                         0.000251753602985472,
                                         0.0176224827320942,
                                         0.129100757878925,
                                         0.20247388537349,
                                         0.0823251408771431,
                                         0.00797390540833972,
                                         0.000122588263448311,
                                         9.53420597204952e-08,
                                         5.06641081291329e-14,
                                         3.72764301823036e-10,
                                         2.34183580919419e-05,
                                         0.00432519304746106,
                                         0.0656126165304324,
                                         0.192856660163799,
                                         0.142874853057652,
                                         0.0259359077625321,
                                         0.000825656547663964,
                                         1.69430940258521e-06,
                                         4.92764704858773e-12,
                                         4.92764704858773e-12,
                                         1.69430940258523e-06,
                                         0.000825656547663964,
                                         0.0259359077625321,
                                         0.142874853057652,
                                         0.192856660163799,
                                         0.0656126165304324,
                                         0.00432519304746106,
                                         2.34183580919418e-05,
                                         3.72764301823036e-10,
                                         5.06641081291329e-14,
                                         9.53420597204959e-08,
                                         0.000122588263448311,
                                         0.00797390540833972,
                                         0.0823251408771432,
                                         0.20247388537349,
                                         0.129100757878925,
                                         0.0176224827320942,
                                         0.000251753602985471,
                                         2.19323196785749e-08,
                                         4.01843483945581e-16,
                                         4.13877730314141e-09,
                                         1.4040871878623e-05,
                                         0.00189119549971246,
                                         0.0365935790730247,
                                         0.163983107458787,
                                         0.195959330162372,
                                         0.0553891130694775,
                                         0.00208780852540436,
                                         9.95475416773617e-07,
                                         2.41456848442177e-18,
                                         1.36108628041219e-10,
                                         1.21833109520774e-06,
                                         0.000339803494532792,
                                         0.0123226286970933,
                                         0.100613272737944,
                                         0.225335267775995,
                                         0.131888763626381,
                                         0.0131169150957815,
                                         3.42296596268418e-05,
                                         1.06395571389973e-20,
                                         3.28246935385511e-12,
                                         7.75243274263693e-08,
                                         4.47734625728111e-05,
                                         0.00304300918427516,
                                         0.0452702471968232,
                                         0.190017595820408,
                                         0.230299295386611,
                                         0.0604330028320903,
                                         0.000863129674036264,
                                         3.24568771604203e-23,
                                         5.4804328507835e-14,
                                         3.41515046525276e-09,
                                         4.0842534100564e-06,
                                         0.000520238298729387,
                                         0.0141016396662877,
                                         0.11093219986228,
                                         0.278404710136788,
                                         0.19275950317536,
                                         0.0150677592314611,
                                         6.12934363095772e-26,
                                         5.6643893728061e-16,
                                         9.31334595754981e-11,
                                         2.30636870344894e-07,
                                         5.50586344599844e-05,
                                         0.00271925801864037,
                                         0.0400908708006739,
                                         0.208345730622492,
                                         0.380611126372081,
                                         0.162834098377439,
                                         5.40166925948599e-29,
                                         2.73210960864404e-18,
                                         1.18524575837906e-12,
                                         6.07787232823815e-09,
                                         2.71928878900493e-06,
                                         0.00024470227306177,
                                         0.0067614545730708,
                                         0.0727611263308118,
                                         0.35071467810852,
                                         0.821199765271958,

                                         0.84858900714692,
                                         0.418162106335592,
                                         0.113714009496523,
                                         0.0161558168737037,
                                         0.00109461675993688,
                                         3.0517578125e-05,
                                         2.74453329460392e-07,
                                         5.19306975088504e-10,
                                         8.94000958761988e-14,
                                         1.8928528740491e-19,
                                         3.57131388547619e-30,
                                         0.140086849606718,
                                         0.375394074135454,
                                         0.266035850912901,
                                         0.0767172799766803,
                                         0.00944714207023588,
                                         0.000457763671875,
                                         7.15504992972677e-06,
                                         2.46060781677395e-08,
                                         8.59793800808638e-12,
                                         4.74413063096948e-17,
                                         4.86755170366615e-27,
                                         0.0107920541724515,
                                         0.157266757130219,
                                         0.290451176590695,
                                         0.170006001145838,
                                         0.0380492044297812,
                                         0.003204345703125,
                                         8.70489655886162e-05,
                                         5.44085840410165e-07,
                                         3.85884571945627e-10,
                                         5.54885274905712e-15,
                                         3.09599253819959e-24,
                                         0.0005146771913775,
                                         0.0407859430206213,
                                         0.196304462262835,
                                         0.233216549342254,
                                         0.0948669137471182,
                                         0.013885498046875,
                                         0.000655599506962402,
                                         7.4476027858964e-06,
                                         1.07212291589037e-08,
                                         4.0176658971325e-13,
                                         1.2190269157851e-21,
                                         1.69927976198788e-05,
                                         0.00732290233189136,
                                         0.0918515227583979,
                                         0.221489751350261,
                                         0.163750683295863,
                                         0.041656494140625,
                                         0.00341832049583295,
                                         7.05772520312516e-05,
                                         2.06220055527442e-07,
                                         2.01392639743995e-11,
                                         3.32296557010893e-19,
                                         4.11430301512628e-07,
                                         0.000964178408050014,
                                         0.031516934967121,
                                         0.154258580993259,
                                         0.207277850220598,
                                         0.091644287109375,
                                         0.0130703845634953,
                                         0.000490472215219571,
                                         2.90883204643667e-06,
                                         7.4031201232541e-10,
                                         6.64262620229327e-17,
                                         7.54664171102184e-09,
                                         9.61739891214061e-05,
                                         0.00819271145341639,
                                         0.0813900091381503,
                                         0.198769039612445,
                                         0.152740478515625,
                                         0.0378608204884998,
                                         0.0025822015891126,
                                         3.1083682313609e-05,
                                         2.06163636927731e-08,
                                         1.00595801634435e-14,
                                         1.06784178055298e-10,
                                         7.40037246426887e-06,
                                         0.00164288461587122,
                                         0.0331274938688893,
                                         0.14704162811869,
                                         0.196380615234375,
                                         0.0846033182216922,
                                         0.0104872494405899,
                                         0.000256237092023023,
                                         4.42899364846197e-07,
                                         1.1752103759396e-12,
                                         1.17521037593955e-12,
                                         4.42899364846197e-07,
                                         0.000256237092023024,
                                         0.0104872494405899,
                                         0.0846033182216921,
                                         0.196380615234375,
                                         0.147041628118691,
                                         0.0331274938688892,
                                         0.00164288461587121,
                                         7.40037246426887e-06,
                                         1.06784178055302e-10,
                                         1.00595801634431e-14,
                                         2.06163636927731e-08,
                                         3.10836823136091e-05,
                                         0.0025822015891126,
                                         0.0378608204884998,
                                         0.152740478515625,
                                         0.198769039612446,
                                         0.0813900091381502,
                                         0.00819271145341637,
                                         9.61739891214061e-05,
                                         7.54664171102207e-09,
                                         6.64262620229294e-17,
                                         7.4031201232541e-10,
                                         2.90883204643668e-06,
                                         0.000490472215219573,
                                         0.0130703845634953,
                                         0.091644287109375,
                                         0.207277850220598,
                                         0.154258580993259,
                                         0.0315169349671209,
                                         0.000964178408050014,
                                         4.11430301512638e-07,
                                         3.32296557010874e-19,
                                         2.01392639743995e-11,
                                         2.06220055527443e-07,
                                         7.05772520312518e-05,
                                         0.00341832049583294,
                                         0.041656494140625,
                                         0.163750683295863,
                                         0.221489751350261,
                                         0.0918515227583978,
                                         0.00732290233189136,
                                         1.69927976198792e-05,
                                         1.21902691578503e-21,
                                         4.0176658971325e-13,
                                         1.07212291589037e-08,
                                         7.44760278589642e-06,
                                         0.000655599506962401,
                                         0.013885498046875,
                                         0.0948669137471182,
                                         0.233216549342254,
                                         0.196304462262835,
                                         0.0407859430206213,
                                         0.000514677191377508,
                                         3.09599253819938e-24,
                                         5.54885274905712e-15,
                                         3.85884571945629e-10,
                                         5.44085840410167e-07,
                                         8.7048965588616e-05,
                                         0.003204345703125,
                                         0.0380492044297812,
                                         0.170006001145838,
                                         0.290451176590695,
                                         0.157266757130219,
                                         0.0107920541724516,
                                         4.86755170366581e-27,
                                         4.74413063096948e-17,
                                         8.59793800808643e-12,
                                         2.46060781677396e-08,
                                         7.15504992972676e-06,
                                         0.000457763671875,
                                         0.00944714207023588,
                                         0.0767172799766802,
                                         0.266035850912901,
                                         0.375394074135454,
                                         0.140086849606718,
                                         3.57131388547592e-30,
                                         1.8928528740491e-19,
                                         8.94000958761993e-14,
                                         5.19306975088506e-10,
                                         2.74453329460391e-07,
                                         3.0517578125e-05,
                                         0.00109461675993688,
                                         0.0161558168737037,
                                         0.113714009496523,
                                         0.418162106335592,
                                         0.84858900714692,

                                         0.870283108611144,
                                         0.478580940433974,
                                         0.159878787776102,
                                         0.0312254707432434,
                                         0.00334995578113096,
                                         0.000179138665779747,
                                         4.10140721362125e-06,
                                         3.14081858758606e-08,
                                         5.23339705764674e-11,
                                         8.18886355533607e-15,
                                         1.62426488140105e-20,
                                         2.95624039842695e-31,
                                         0.121475984165508,
                                         0.361487614253412,
                                         0.311777156366101,
                                         0.121773250688862,
                                         0.023223655758743,
                                         0.00208896019549292,
                                         7.91360883619845e-05,
                                         1.01937472128024e-06,
                                         3.01941841102636e-09,
                                         9.44827897572389e-13,
                                         4.83839532409863e-18,
                                         4.76532355608575e-28,
                                         0.00791274334223617,
                                         0.127420183230435,
                                         0.283729516211171,
                                         0.221616241725329,
                                         0.0751326794395197,
                                         0.0113678353602096,
                                         0.000712562641476823,
                                         1.54394436861053e-05,
                                         8.12960964397779e-08,
                                         5.08731421235733e-11,
                                         6.72593438267557e-16,
                                         3.58469178721919e-25,
                                         0.00031907133515706,
                                         0.0278039876261941,
                                         0.159841225640028,
                                         0.249675155665354,
                                         0.150470452401031,
                                         0.0382956498034503,
                                         0.00397187503860237,
                                         0.000144761639645205,
                                         1.3550026489923e-06,
                                         1.69569786706659e-09,
                                         5.78799250348865e-14,
                                         1.66930350614262e-22,
                                         8.90733235511408e-06,
                                         0.00420024965666635,
                                         0.0623407814048606,
                                         0.194736890096023,
                                         0.208628066561768,
                                         0.0893141497417454,
                                         0.0153273565703844,
                                         0.000939668127593556,
                                         1.56354272389053e-05,
                                         3.91297923046847e-08,
                                         3.44827942603073e-12,
                                         5.38168095146173e-20,
                                         1.82351336595724e-07,
                                         0.000465312317956285,
                                         0.0178302366892216,
                                         0.11138393509634,
                                         0.212126861272085,
                                         0.152753974642649,
                                         0.0433750882337611,
                                         0.00447297968069987,
                                         0.000132306380181816,
                                         6.62167769034686e-07,
                                         1.50653202494669e-10,
                                         1.272336585585e-17,
                                         2.82811010420786e-09,
                                         3.90517106557695e-05,
                                         0.00386338548487336,
                                         0.0482639600455502,
                                         0.163397221576405,
                                         0.197920527308259,
                                         0.0929907071834041,
                                         0.0161304106208609,
                                         0.000848160203521935,
                                         8.48896223280053e-06,
                                         4.98632238605171e-09,
                                         2.27883117496284e-15,
                                         3.38360295636923e-11,
                                         2.52831566792342e-06,
                                         0.000645765291769512,
                                         0.0161331461582137,
                                         0.0970933283153717,
                                         0.197826687555914,
                                         0.153792249563296,
                                         0.0448734685985528,
                                         0.00419440839964346,
                                         8.39531285537722e-05,
                                         1.27314555966005e-07,
                                         3.14860375191546e-13,
                                         3.14860375191561e-13,
                                         1.27314555966004e-07,
                                         8.39531285537719e-05,
                                         0.00419440839964345,
                                         0.0448734685985527,
                                         0.153792249563296,
                                         0.197826687555914,
                                         0.0970933283153718,
                                         0.0161331461582138,
                                         0.000645765291769514,
                                         2.52831566792344e-06,
                                         3.38360295636908e-11,
                                         2.27883117496297e-15,
                                         4.98632238605165e-09,
                                         8.4889622328005e-06,
                                         0.000848160203521933,
                                         0.0161304106208609,
                                         0.0929907071834041,
                                         0.197920527308259,
                                         0.163397221576405,
                                         0.0482639600455503,
                                         0.00386338548487337,
                                         3.90517106557698e-05,
                                         2.82811010420776e-09,
                                         1.27233658558508e-17,
                                         1.50653202494667e-10,
                                         6.62167769034682e-07,
                                         0.000132306380181815,
                                         0.00447297968069986,
                                         0.0433750882337611,
                                         0.152753974642649,
                                         0.212126861272085,
                                         0.11138393509634,
                                         0.0178302366892217,
                                         0.000465312317956288,
                                         1.82351336595719e-07,
                                         5.38168095146208e-20,
                                         3.44827942603069e-12,
                                         3.91297923046845e-08,
                                         1.56354272389053e-05,
                                         0.000939668127593554,
                                         0.0153273565703844,
                                         0.0893141497417454,
                                         0.208628066561769,
                                         0.194736890096023,
                                         0.0623407814048607,
                                         0.00420024965666637,
                                         8.90733235511386e-06,
                                         1.66930350614274e-22,
                                         5.78799250348857e-14,
                                         1.69569786706658e-09,
                                         1.35500264899229e-06,
                                         0.000144761639645205,
                                         0.00397187503860237,
                                         0.0382956498034503,
                                         0.150470452401031,
                                         0.249675155665354,
                                         0.159841225640028,
                                         0.0278039876261942,
                                         0.000319071335157054,
                                         3.58469178721947e-25,
                                         6.72593438267547e-16,
                                         5.0873142123573e-11,
                                         8.12960964397776e-08,
                                         1.54394436861053e-05,
                                         0.000712562641476823,
                                         0.0113678353602096,
                                         0.0751326794395197,
                                         0.221616241725329,
                                         0.283729516211171,
                                         0.127420183230435,
                                         0.00791274334223608,
                                         4.76532355608615e-28,
                                         4.83839532409855e-18,
                                         9.44827897572383e-13,
                                         3.01941841102635e-09,
                                         1.01937472128024e-06,
                                         7.91360883619845e-05,
                                         0.00208896019549292,
                                         0.023223655758743,
                                         0.121773250688862,
                                         0.311777156366102,
                                         0.361487614253412,
                                         0.121475984165507,
                                         2.95624039842721e-31,
                                         1.62426488140102e-20,
                                         8.18886355533601e-15,
                                         5.23339705764672e-11,
                                         3.14081858758605e-08,
                                         4.10140721362125e-06,
                                         0.000179138665779747,
                                         0.00334995578113096,
                                         0.0312254707432434,
                                         0.159878787776102,
                                         0.478580940433974,
                                         0.870283108611144,

                                         0.887720216579797,
                                         0.532004054698147,
                                         0.208615436195097,
                                         0.0520634768410097,
                                         0.00791215810351642,
                                         0.000684784074074981,
                                         3.0517578125e-05,
                                         5.99835694654864e-07,
                                         4.0533904709262e-09,
                                         6.1155502286922e-12,
                                         8.8796209710754e-16,
                                         1.67334231780735e-21,
                                         2.96066485152277e-32,
                                         0.106147127333752,
                                         0.342913190880507,
                                         0.344646785215002,
                                         0.170066164695189,
                                         0.0451877913705061,
                                         0.00642406836628644,
                                         0.000457763671875,
                                         1.43866050533223e-05,
                                         1.59688926808054e-07,
                                         4.212435898772e-10,
                                         1.20934205209195e-13,
                                         5.84114602092941e-19,
                                         5.57108302953681e-29,
                                         0.00592307403582077,
                                         0.103147859881882,
                                         0.265710553695768,
                                         0.259244438908268,
                                         0.12043537077668,
                                         0.0281237635394563,
                                         0.003204345703125,
                                         0.000161024187971517,
                                         2.9358775852152e-06,
                                         1.35405982851253e-08,
                                         7.68618197109522e-12,
                                         9.51520420430929e-17,
                                         4.89211655150606e-26,
                                         0.000204602122452694,
                                         0.0192070341853393,
                                         0.126814068070201,
                                         0.244638631124246,
                                         0.19870603185657,
                                         0.0762185669566084,
                                         0.013885498046875,
                                         0.0011157013836836,
                                         3.34137440655492e-05,
                                         2.6944272379016e-07,
                                         3.02410055072328e-10,
                                         9.59538092784913e-15,
                                         2.65936541847062e-23,
                                         4.89296675666952e-06,
                                         0.00247605082017005,
                                         0.0419010804546192,
                                         0.159823186064324,
                                         0.226969345003489,
                                         0.143003701004886,
                                         0.041656494140625,
                                         0.00535185061760514,
                                         0.000263276137285008,
                                         3.71188252867738e-06,
                                         8.23722538981566e-09,
                                         6.69893070049769e-13,
                                         1.00082557771162e-20,
                                         8.58095889862183e-08,
                                         0.000234077798977032,
                                         0.0101527673650099,
                                         0.0765695255760225,
                                         0.190118677321102,
                                         0.196759269641049,
                                         0.091644287109375,
                                         0.0188261458948538,
                                         0.00152124540449605,
                                         3.74993217709445e-05,
                                         1.64538334805116e-07,
                                         3.42965295171751e-11,
                                         2.76210510735456e-18,
                                         1.14005406439004e-09,
                                         1.67643594803142e-05,
                                         0.00186367301651429,
                                         0.0277906175251121,
                                         0.120644751392928,
                                         0.205092224238245,
                                         0.152740478515625,
                                         0.0501700952960918,
                                         0.00665906309512429,
                                         0.000286997868458189,
                                         2.4898851489918e-06,
                                         1.33021222647934e-09,
                                         5.77494795598038e-16,
                                         1.16845177452494e-11,
                                         9.26209912182102e-07,
                                         0.000263906884175522,
                                         0.00778101330547425,
                                         0.059059232833082,
                                         0.164914524961335,
                                         0.196380615234375,
                                         0.103139293397105,
                                         0.0224865433412524,
                                         0.0016944534769166,
                                         2.90661326844934e-05,
                                         3.98003979614684e-08,
                                         9.31433102287523e-14,
                                         9.31433102287576e-14,
                                         3.98003979614688e-08,
                                         2.90661326844932e-05,
                                         0.0016944534769166,
                                         0.0224865433412524,
                                         0.103139293397105,
                                         0.196380615234375,
                                         0.164914524961335,
                                         0.059059232833082,
                                         0.00778101330547424,
                                         0.000263906884175523,
                                         9.26209912182093e-07,
                                         1.16845177452488e-11,
                                         5.77494795598075e-16,
                                         1.33021222647936e-09,
                                         2.48988514899179e-06,
                                         0.00028699786845819,
                                         0.00665906309512429,
                                         0.0501700952960918,
                                         0.152740478515625,
                                         0.205092224238245,
                                         0.120644751392928,
                                         0.0277906175251121,
                                         0.0018636730165143,
                                         1.6764359480314e-05,
                                         1.14005406439e-09,
                                         2.76210510735475e-18,
                                         3.42965295171755e-11,
                                         1.64538334805115e-07,
                                         3.74993217709446e-05,
                                         0.00152124540449605,
                                         0.0188261458948538,
                                         0.091644287109375,
                                         0.196759269641049,
                                         0.190118677321102,
                                         0.0765695255760224,
                                         0.0101527673650099,
                                         0.00023407779897703,
                                         8.58095889862153e-08,
                                         1.00082557771169e-20,
                                         6.69893070049779e-13,
                                         8.23722538981561e-09,
                                         3.71188252867739e-06,
                                         0.000263276137285008,
                                         0.00535185061760514,
                                         0.041656494140625,
                                         0.143003701004886,
                                         0.226969345003489,
                                         0.159823186064324,
                                         0.0419010804546193,
                                         0.00247605082017004,
                                         4.89296675666939e-06,
                                         2.65936541847085e-23,
                                         9.59538092784929e-15,
                                         3.02410055072326e-10,
                                         2.69442723790161e-07,
                                         3.34137440655492e-05,
                                         0.0011157013836836,
                                         0.013885498046875,
                                         0.0762185669566084,
                                         0.19870603185657,
                                         0.244638631124246,
                                         0.126814068070201,
                                         0.0192070341853392,
                                         0.000204602122452689,
                                         4.8921165515065e-26,
                                         9.51520420430945e-17,
                                         7.68618197109517e-12,
                                         1.35405982851254e-08,
                                         2.9358775852152e-06,
                                         0.000161024187971517,
                                         0.003204345703125,
                                         0.0281237635394563,
                                         0.12043537077668,
                                         0.259244438908267,
                                         0.265710553695769,
                                         0.103147859881881,
                                         0.00592307403582068,
                                         5.57108302953735e-29,
                                         5.84114602092952e-19,
                                         1.20934205209194e-13,
                                         4.21243589877201e-10,
                                         1.59688926808054e-07,
                                         1.43866050533223e-05,
                                         0.000457763671875,
                                         0.00642406836628644,
                                         0.0451877913705061,
                                         0.170066164695189,
                                         0.344646785215003,
                                         0.342913190880507,
                                         0.106147127333752,
                                         2.96066485152308e-32,
                                         1.67334231780738e-21,
                                         8.87962097107533e-16,
                                         6.11555022869223e-12,
                                         4.0533904709262e-09,
                                         5.99835694654864e-07,
                                         3.0517578125e-05,
                                         0.000684784074074981,
                                         0.00791215810351642,
                                         0.0520634768410097,
                                         0.208615436195097,
                                         0.532004054698147,
                                         0.887720216579797,

                                         0.901923289906664,
                                         0.578928257832783,
                                         0.257837482473798,
                                         0.0780561756387474,
                                         0.0155528001930821,
                                         0.00194436123628978,
                                         0.000142222900177439,
                                         5.49062229493562e-06,
                                         9.564928735563e-08,
                                         5.85242230535399e-10,
                                         8.1586085214371e-13,
                                         1.11598038152608e-16,
                                         2.01869538012205e-22,
                                         3.49182982835857e-33,
                                         0.0934228457005624,
                                         0.322264457617088,
                                         0.365755903821909,
                                         0.216993029567316,
                                         0.0746337028901674,
                                         0.0150543450350835,
                                         0.00171726608154783,
                                         0.000102314226586646,
                                         2.77958107047594e-06,
                                         2.74404846980429e-08,
                                         6.60328586231805e-11,
                                         1.77008362784107e-14,
                                         8.15953943025256e-20,
                                         7.58493910315447e-30,
                                         0.00451589008919373,
                                         0.0837156883910626,
                                         0.24212711027103,
                                         0.281508218466529,
                                         0.167135292961704,
                                         0.0543943207478814,
                                         0.00967636929453132,
                                         0.000889728010147452,
                                         3.76949987373178e-05,
                                         6.00419350966606e-07,
                                         2.49408289378789e-09,
                                         1.31020059192946e-12,
                                         1.53910157548963e-17,
                                         7.68878836571179e-27,
                                         0.000135131840551105,
                                         0.0134624887407161,
                                         0.099224644203938,
                                         0.226079107985048,
                                         0.231699663015415,
                                         0.121666021766525,
                                         0.033752918817633,
                                         0.00478963653715455,
                                         0.000316455157726343,
                                         8.13282899469811e-06,
                                         5.83157313035077e-08,
                                         6.00351852447799e-11,
                                         1.79718579642439e-15,
                                         4.82489396466985e-24,
                                         2.79944009214915e-06,
                                         0.00149879767432466,
                                         0.0281510656045959,
                                         0.125698165871195,
                                         0.222372866552174,
                                         0.188401438635374,
                                         0.0815097182893102,
                                         0.0178503612744265,
                                         0.00183924583316584,
                                         7.62654360662426e-05,
                                         9.43973333333467e-07,
                                         1.90446535181689e-09,
                                         1.45284013762671e-13,
                                         2.09612316178658e-21,
                                         4.25290899974832e-08,
                                         0.000122366374846364,
                                         0.00585695049944788,
                                         0.0512505908473715,
                                         0.15650910679968,
                                         0.213944215051224,
                                         0.144347370028436,
                                         0.0487857390529717,
                                         0.00783914701645774,
                                         0.000524463538774557,
                                         1.12056009306498e-05,
                                         4.43038759979513e-08,
                                         8.61279494740621e-12,
                                         6.67801278909791e-19,
                                         4.89471118378515e-10,
                                         7.56845523255813e-06,
                                         0.00092315438521474,
                                         0.0158305090804166,
                                         0.0834494541281558,
                                         0.184053020802927,
                                         0.193657547570442,
                                         0.101010108013656,
                                         0.0253118510783033,
                                         0.00273230333992717,
                                         0.000100771250132389,
                                         7.80793948353133e-07,
                                         3.86808896863941e-10,
                                         1.61177258614209e-16,
                                         4.34574032234951e-12,
                                         3.61117153615167e-07,
                                         0.000112246508357069,
                                         0.00377212960154372,
                                         0.0343244120769843,
                                         0.122146512469076,
                                         0.200426757151856,
                                         0.161336452128828,
                                         0.0630484949408699,
                                         0.0109809077937952,
                                         0.000699091240773188,
                                         1.06151680562749e-05,
                                         1.34012256926465e-08,
                                         3.00093095228077e-14,
                                         3.00093095228077e-14,
                                         1.34012256926467e-08,
                                         1.06151680562749e-05,
                                         0.000699091240773186,
                                         0.0109809077937952,
                                         0.0630484949408699,
                                         0.161336452128827,
                                         0.200426757151856,
                                         0.122146512469076,
                                         0.0343244120769844,
                                         0.00377212960154373,
                                         0.000112246508357069,
                                         3.61117153615163e-07,
                                         4.34574032234952e-12,
                                         1.61177258614209e-16,
                                         3.86808896863947e-10,
                                         7.80793948353134e-07,
                                         0.000100771250132389,
                                         0.00273230333992716,
                                         0.0253118510783033,
                                         0.101010108013656,
                                         0.193657547570442,
                                         0.184053020802927,
                                         0.0834494541281559,
                                         0.0158305090804167,
                                         0.00092315438521474,
                                         7.56845523255806e-06,
                                         4.89471118378515e-10,
                                         6.67801278909791e-19,
                                         8.61279494740635e-12,
                                         4.43038759979513e-08,
                                         1.12056009306498e-05,
                                         0.000524463538774556,
                                         0.00783914701645774,
                                         0.0487857390529716,
                                         0.144347370028437,
                                         0.213944215051224,
                                         0.15650910679968,
                                         0.0512505908473716,
                                         0.00585695049944788,
                                         0.000122366374846363,
                                         4.25290899974832e-08,
                                         2.09612316178658e-21,
                                         1.45284013762674e-13,
                                         1.90446535181689e-09,
                                         9.43973333333463e-07,
                                         7.62654360662424e-05,
                                         0.00183924583316584,
                                         0.0178503612744265,
                                         0.0815097182893102,
                                         0.188401438635374,
                                         0.222372866552174,
                                         0.125698165871195,
                                         0.0281510656045959,
                                         0.00149879767432465,
                                         2.79944009214915e-06,
                                         4.82489396466985e-24,
                                         1.79718579642442e-15,
                                         6.00351852447799e-11,
                                         5.83157313035074e-08,
                                         8.13282899469809e-06,
                                         0.000316455157726343,
                                         0.00478963653715454,
                                         0.033752918817633,
                                         0.121666021766525,
                                         0.231699663015415,
                                         0.226079107985048,
                                         0.099224644203938,
                                         0.013462488740716,
                                         0.000135131840551105,
                                         7.68878836571179e-27,
                                         1.53910157548966e-17,
                                         1.31020059192946e-12,
                                         2.49408289378788e-09,
                                         6.00419350966604e-07,
                                         3.76949987373178e-05,
                                         0.00088972801014745,
                                         0.00967636929453133,
                                         0.0543943207478814,
                                         0.167135292961705,
                                         0.281508218466529,
                                         0.24212711027103,
                                         0.0837156883910623,
                                         0.00451589008919373,
                                         7.58493910315447e-30,
                                         8.15953943025273e-20,
                                         1.77008362784107e-14,
                                         6.60328586231801e-11,
                                         2.74404846980428e-08,
                                         2.77958107047594e-06,
                                         0.000102314226586646,
                                         0.00171726608154783,
                                         0.0150543450350835,
                                         0.0746337028901674,
                                         0.216993029567316,
                                         0.365755903821909,
                                         0.322264457617087,
                                         0.0934228457005624,
                                         3.49182982835857e-33,
                                         2.0186953801221e-22,
                                         1.11598038152608e-16,
                                         8.15860852143705e-13,
                                         5.85242230535397e-10,
                                         9.564928735563e-08,
                                         5.49062229493561e-06,
                                         0.000142222900177439,
                                         0.00194436123628978,
                                         0.0155528001930821,
                                         0.0780561756387474,
                                         0.257837482473798,
                                         0.578928257832783,
                                         0.901923289906664,

                                         0.913631886587802,
                                         0.620031522969848,
                                         0.306060834746832,
                                         0.108190035361065,
                                         0.0267348313392566,
                                         0.00445860404485515,
                                         0.000477252144004014,
                                         3.0517578125e-05,
                                         1.04995109978531e-06,
                                         1.65963377272405e-08,
                                         9.37199388231698e-11,
                                         1.22551640751572e-13,
                                         1.59714412652817e-17,
                                         2.79464773434326e-23,
                                         4.74601621925506e-34,
                                         0.0827750983567855,
                                         0.301138245585921,
                                         0.377052166739535,
                                         0.259350633377347,
                                         0.109518043716646,
                                         0.0290632641156829,
                                         0.0047606606607692,
                                         0.000457763671875,
                                         2.36827566728865e-05,
                                         5.72861055827471e-07,
                                         5.14761770133741e-09,
                                         1.15027478017185e-11,
                                         2.91697659453114e-15,
                                         1.29466511200353e-20,
                                         1.17864570819835e-30,
                                         0.00349973324850817,
                                         0.0682534868164369,
                                         0.216771360923293,
                                         0.290131006157374,
                                         0.209363363798658,
                                         0.0884090354727113,
                                         0.0221612036714963,
                                         0.003204345703125,
                                         0.000249288482494905,
                                         9.22769252968322e-06,
                                         1.31943304219628e-07,
                                         5.03837915863884e-10,
                                         2.48615705921586e-13,
                                         2.79894650560937e-18,
                                         1.3659793096545e-27,
                                         9.15997395928153e-05,
                                         0.00957652236401968,
                                         0.0771483114591028,
                                         0.200920862064942,
                                         0.247764884856377,
                                         0.166484180997685,
                                         0.0638621479659089,
                                         0.013885498046875,
                                         0.00162441216241897,
                                         9.2015499697299e-05,
                                         2.09359408935268e-06,
                                         1.36616814061377e-08,
                                         1.3117414293965e-11,
                                         3.74589680492052e-16,
                                         9.80006645744081e-25,
                                         1.65978827262014e-06,
                                         0.000930229201799945,
                                         0.0190085987447646,
                                         0.0963285623511643,
                                         0.202991559189727,
                                         0.217044333820232,
                                         0.12740687259478,
                                         0.041656494140625,
                                         0.00732806660937109,
                                         0.000635225640444369,
                                         2.29983547345272e-05,
                                         2.56458215978986e-07,
                                         4.79145117096473e-10,
                                         3.4706928153403e-14,
                                         4.86757977069627e-22,
                                         2.20552797376276e-08,
                                         6.62633726167489e-05,
                                         0.00343459242201831,
                                         0.033867766018932,
                                         0.121960060209981,
                                         0.207503466965171,
                                         0.186399015917353,
                                         0.091644287109375,
                                         0.0242428687419467,
                                         0.00321585513430468,
                                         0.000185268881395106,
                                         3.53045358750317e-06,
                                         1.28347473519644e-08,
                                         2.35818553301576e-12,
                                         1.77295846034543e-19,
                                         2.22023281947445e-10,
                                         3.57588194886098e-06,
                                         0.000470139109457478,
                                         0.00902078000351879,
                                         0.0555115494434438,
                                         0.150289393138833,
                                         0.206595303140307,
                                         0.152740478515625,
                                         0.0607581694450815,
                                         0.0123336324615909,
                                         0.00113066580821286,
                                         3.68188707984404e-05,
                                         2.60455570529879e-07,
                                         1.21385226628748e-10,
                                         4.89226656060626e-17,
                                         1.72417040202672e-12,
                                         1.48863604041729e-07,
                                         4.96447437236465e-05,
                                         0.00185352032229676,
                                         0.0194914787537288,
                                         0.0839705550879724,
                                         0.176641600950007,
                                         0.196380615234375,
                                         0.117468404806289,
                                         0.0364906204708972,
                                         0.00532306486910819,
                                         0.000296214383614385,
                                         4.07732843999542e-06,
                                         4.82002639027152e-09,
                                         1.04139954473094e-14,
                                         1.04139954473086e-14,
                                         4.82002639027152e-09,
                                         4.0773284399954e-06,
                                         0.000296214383614386,
                                         0.00532306486910819,
                                         0.0364906204708971,
                                         0.117468404806289,
                                         0.196380615234375,
                                         0.176641600950007,
                                         0.0839705550879724,
                                         0.0194914787537288,
                                         0.00185352032229676,
                                         4.96447437236467e-05,
                                         1.48863604041729e-07,
                                         1.72417040202683e-12,
                                         4.89226656060586e-17,
                                         1.21385226628748e-10,
                                         2.60455570529878e-07,
                                         3.68188707984405e-05,
                                         0.00113066580821286,
                                         0.0123336324615909,
                                         0.0607581694450815,
                                         0.152740478515625,
                                         0.206595303140307,
                                         0.150289393138833,
                                         0.0555115494434438,
                                         0.00902078000351876,
                                         0.00047013910945748,
                                         3.57588194886098e-06,
                                         2.22023281947458e-10,
                                         1.77295846034527e-19,
                                         2.35818553301576e-12,
                                         1.28347473519643e-08,
                                         3.53045358750319e-06,
                                         0.000185268881395106,
                                         0.00321585513430468,
                                         0.0242428687419467,
                                         0.091644287109375,
                                         0.186399015917353,
                                         0.207503466965171,
                                         0.121960060209981,
                                         0.0338677660189319,
                                         0.00343459242201832,
                                         6.62633726167489e-05,
                                         2.20552797376286e-08,
                                         4.86757977069578e-22,
                                         3.4706928153403e-14,
                                         4.79145117096469e-10,
                                         2.56458215978987e-07,
                                         2.29983547345272e-05,
                                         0.000635225640444369,
                                         0.00732806660937109,
                                         0.041656494140625,
                                         0.12740687259478,
                                         0.217044333820232,
                                         0.202991559189727,
                                         0.0963285623511642,
                                         0.0190085987447647,
                                         0.000930229201799945,
                                         1.6597882726202e-06,
                                         9.80006645743973e-25,
                                         3.74589680492052e-16,
                                         1.31174142939649e-11,
                                         1.36616814061378e-08,
                                         2.09359408935268e-06,
                                         9.2015499697299e-05,
                                         0.00162441216241897,
                                         0.013885498046875,
                                         0.0638621479659089,
                                         0.166484180997685,
                                         0.247764884856377,
                                         0.200920862064942,
                                         0.0771483114591029,
                                         0.00957652236401968,
                                         9.15997395928179e-05,
                                         1.36597930965434e-27,
                                         2.79894650560937e-18,
                                         2.48615705921583e-13,
                                         5.03837915863887e-10,
                                         1.31943304219628e-07,
                                         9.22769252968322e-06,
                                         0.000249288482494905,
                                         0.003204345703125,
                                         0.0221612036714963,
                                         0.0884090354727113,
                                         0.209363363798658,
                                         0.290131006157374,
                                         0.216771360923293,
                                         0.0682534868164369,
                                         0.00349973324850823,
                                         1.17864570819819e-30,
                                         1.29466511200353e-20,
                                         2.91697659453111e-15,
                                         1.15027478017185e-11,
                                         5.14761770133741e-09,
                                         5.72861055827471e-07,
                                         2.36827566728865e-05,
                                         0.000457763671875,
                                         0.0047606606607692,
                                         0.0290632641156829,
                                         0.109518043716646,
                                         0.259350633377346,
                                         0.377052166739536,
                                         0.301138245585921,
                                         0.0827750983567863,
                                         4.74601621925441e-34,
                                         2.79464773434326e-23,
                                         1.59714412652816e-17,
                                         1.22551640751573e-13,
                                         9.37199388231698e-11,
                                         1.65963377272405e-08,
                                         1.04995109978531e-06,
                                         3.0517578125e-05,
                                         0.000477252144004014,
                                         0.00445860404485515,
                                         0.0267348313392566,
                                         0.108190035361065,
                                         0.306060834746832,
                                         0.620031522969848,
                                         0.913631886587802,

                                         0.923389285253956,
                                         0.656025336892972,
                                         0.352319770778243,
                                         0.141321368739602,
                                         0.0415651678266352,
                                         0.00872947022011565,
                                         0.00126140804019345,
                                         0.000119079300895812,
                                         6.82635238796068e-06,
                                         2.13802471657506e-07,
                                         3.1213157632586e-09,
                                         1.65048137022457e-11,
                                         2.04776184641819e-14,
                                         2.56464069384784e-18,
                                         4.36649412243805e-24,
                                         7.30404437478048e-35,
                                         0.0737940469184123,
                                         0.280474052653443,
                                         0.380628153467915,
                                         0.295372823787108,
                                         0.147258216630277,
                                         0.0486746086027123,
                                         0.0106061393541997,
                                         0.0014762198150225,
                                         0.000123895766670254,
                                         5.72128398892141e-06,
                                         1.25952166882358e-07,
                                         1.04819756489929e-09,
                                         2.204444987104e-12,
                                         5.34127764747091e-16,
                                         2.29796453143541e-21,
                                         2.05640865920178e-31,
                                         0.00275210250850984,
                                         0.0559592065037129,
                                         0.191898501040046,
                                         0.288097848976207,
                                         0.243464876149874,
                                         0.126655434523959,
                                         0.0416165279282755,
                                         0.00854028896771484,
                                         0.001049375722054,
                                         7.14465183983061e-05,
                                         2.37181254785104e-06,
                                         3.10657932252379e-08,
                                         1.10745442941932e-10,
                                         5.1912334498595e-14,
                                         5.6436561197838e-19,
                                         2.70185616776825e-28,
                                         6.3537766151195e-05,
                                         0.00691153422600411,
                                         0.0598916550064907,
                                         0.173953653324494,
                                         0.24918227851274,
                                         0.204018358035326,
                                         0.101087723622248,
                                         0.0305856812634023,
                                         0.00550211438411429,
                                         0.000552322445822363,
                                         2.76489815512467e-05,
                                         5.69961873846227e-07,
                                         3.44410609460131e-09,
                                         3.12334546352011e-12,
                                         8.58028716728236e-17,
                                         2.19755161949904e-25,
                                         1.01554327548058e-06,
                                         0.000590985105423333,
                                         0.012940772527697,
                                         0.0727153752455005,
                                         0.17656196218287,
                                         0.227516984988488,
                                         0.169992651001648,
                                         0.0758338156125082,
                                         0.0199722675064221,
                                         0.00295599348455377,
                                         0.000223139816833799,
                                         7.23949581048299e-06,
                                         7.41525918023936e-08,
                                         1.30097407793993e-10,
                                         9.03113345721675e-15,
                                         1.23741421806916e-22,
                                         1.19032718914674e-08,
                                         3.7057834218628e-05,
                                         0.00205047990915144,
                                         0.0222905287918027,
                                         0.0917441891494665,
                                         0.186062917253808,
                                         0.20963476338505,
                                         0.137882479384373,
                                         0.0531650908057868,
                                         0.0116015409736315,
                                         0.00132061562655313,
                                         6.74329660683641e-05,
                                         1.17078614730534e-06,
                                         3.97391606396417e-09,
                                         6.97082408588602e-13,
                                         5.10966637246848e-20,
                                         1.05696435520388e-10,
                                         1.76039287260939e-06,
                                         0.000246136999609879,
                                         0.00517655176964802,
                                         0.0361148699654017,
                                         0.115274141556579,
                                         0.19584952715094,
                                         0.189924647487862,
                                         0.107214077822125,
                                         0.0344948255299366,
                                         0.00592109269861316,
                                         0.000475841453165994,
                                         1.40040897057789e-05,
                                         9.1959119680952e-08,
                                         4.07616780955448e-11,
                                         1.59843895835406e-17,
                                         7.2401914305966e-13,
                                         6.45111655832555e-08,
                                         2.2792607195129e-05,
                                         0.000927377322835894,
                                         0.0109670358750959,
                                         0.0550934142537473,
                                         0.141148891526359,
                                         0.201813072134213,
                                         0.166791067674449,
                                         0.0791203039522828,
                                         0.0204796725556824,
                                         0.00259028770022513,
                                         0.000129219457281507,
                                         1.64159734214141e-06,
                                         1.83872045127884e-09,
                                         3.85740542773532e-15,
                                         3.85740542773565e-15,
                                         1.83872045127884e-09,
                                         1.6415973421414e-06,
                                         0.000129219457281507,
                                         0.00259028770022513,
                                         0.0204796725556824,
                                         0.0791203039522829,
                                         0.166791067674449,
                                         0.201813072134213,
                                         0.141148891526359,
                                         0.0550934142537473,
                                         0.0109670358750959,
                                         0.000927377322835891,
                                         2.27926071951291e-05,
                                         6.45111655832555e-08,
                                         7.24019143059607e-13,
                                         1.59843895835421e-17,
                                         4.07616780955448e-11,
                                         9.19591196809513e-08,
                                         1.4004089705779e-05,
                                         0.000475841453165994,
                                         0.00592109269861316,
                                         0.0344948255299366,
                                         0.107214077822126,
                                         0.189924647487861,
                                         0.19584952715094,
                                         0.115274141556579,
                                         0.0361148699654017,
                                         0.00517655176964801,
                                         0.00024613699960988,
                                         1.76039287260939e-06,
                                         1.05696435520382e-10,
                                         5.10966637246901e-20,
                                         6.97082408588601e-13,
                                         3.97391606396414e-09,
                                         1.17078614730534e-06,
                                         6.74329660683641e-05,
                                         0.00132061562655313,
                                         0.0116015409736316,
                                         0.0531650908057868,
                                         0.137882479384373,
                                         0.20963476338505,
                                         0.186062917253808,
                                         0.0917441891494665,
                                         0.0222905287918026,
                                         0.00205047990915145,
                                         3.7057834218628e-05,
                                         1.19032718914667e-08,
                                         1.2374142180693e-22,
                                         9.03113345721675e-15,
                                         1.30097407793992e-10,
                                         7.4152591802394e-08,
                                         7.23949581048299e-06,
                                         0.000223139816833799,
                                         0.00295599348455378,
                                         0.0199722675064221,
                                         0.0758338156125081,
                                         0.169992651001648,
                                         0.227516984988488,
                                         0.17656196218287,
                                         0.0727153752455003,
                                         0.0129407725276971,
                                         0.000590985105423333,
                                         1.01554327548054e-06,
                                         2.19755161949932e-25,
                                         8.58028716728236e-17,
                                         3.12334546352008e-12,
                                         3.44410609460133e-09,
                                         5.69961873846227e-07,
                                         2.76489815512467e-05,
                                         0.000552322445822364,
                                         0.00550211438411428,
                                         0.0305856812634022,
                                         0.101087723622248,
                                         0.204018358035326,
                                         0.24918227851274,
                                         0.173953653324494,
                                         0.0598916550064909,
                                         0.00691153422600411,
                                         6.3537766151193e-05,
                                         2.70185616776862e-28,
                                         5.6436561197838e-19,
                                         5.19123344985944e-14,
                                         1.10745442941933e-10,
                                         3.10657932252379e-08,
                                         2.37181254785104e-06,
                                         7.14465183983062e-05,
                                         0.001049375722054,
                                         0.00854028896771482,
                                         0.0416165279282755,
                                         0.126655434523959,
                                         0.243464876149874,
                                         0.288097848976207,
                                         0.191898501040047,
                                         0.0559592065037129,
                                         0.00275210250850979,
                                         2.05640865920208e-31,
                                         2.29796453143541e-21,
                                         5.34127764747085e-16,
                                         2.20444498710402e-12,
                                         1.04819756489929e-09,
                                         1.25952166882358e-07,
                                         5.72128398892142e-06,
                                         0.000123895766670253,
                                         0.0014762198150225,
                                         0.0106061393541997,
                                         0.0486746086027123,
                                         0.147258216630277,
                                         0.295372823787108,
                                         0.380628153467915,
                                         0.280474052653443,
                                         0.0737940469184115,
                                         7.30404437478163e-35,
                                         4.36649412243805e-24,
                                         2.56464069384781e-18,
                                         2.0477618464182e-14,
                                         1.65048137022457e-11,
                                         3.1213157632586e-09,
                                         2.13802471657506e-07,
                                         6.82635238796066e-06,
                                         0.000119079300895811,
                                         0.00126140804019345,
                                         0.00872947022011565,
                                         0.0415651678266352,
                                         0.141321368739602,
                                         0.352319770778243,
                                         0.656025336892972,
                                         0.923389285253956,

                                         0.931600689535534,
                                         0.687583708993884,
                                         0.396046743445829,
                                         0.176355363836809,
                                         0.0598447129028883,
                                         0.0151635684653473,
                                         0.00278931620488931,
                                         0.000358431712421671,
                                         3.0517578125e-05,
                                         1.59878115166929e-06,
                                         4.63333548724731e-08,
                                         6.33319322442759e-10,
                                         3.17104501282249e-12,
                                         3.76620673217438e-15,
                                         4.56323577276674e-19,
                                         7.59429674126545e-25,
                                         1.25441693879685e-35,
                                         0.0661609705674206,
                                         0.260792022875746,
                                         0.378389842574495,
                                         0.32442498839537,
                                         0.185379691392499,
                                         0.0732735832961556,
                                         0.020088557022853,
                                         0.00374791608871041,
                                         0.000457763671875,
                                         3.44023496774539e-05,
                                         1.44752233427079e-06,
                                         2.94889454932416e-08,
                                         2.30328966015799e-10,
                                         4.60639365081086e-13,
                                         1.07463851934766e-16,
                                         4.50506805833419e-22,
                                         3.97422418237345e-32,
                                         0.00219270756445648,
                                         0.0461604260583784,
                                         0.168709397283883,
                                         0.278513790049927,
                                         0.267981801785128,
                                         0.165234307695218,
                                         0.0675159705897651,
                                         0.0182885835987682,
                                         0.003204345703125,
                                         0.000345456980761404,
                                         2.11039417828462e-05,
                                         6.40770527432607e-07,
                                         7.80731118815639e-09,
                                         2.6292066585143e-11,
                                         1.18102388062799e-14,
                                         1.24715930310871e-19,
                                         5.87583503500712e-29,
                                         4.49866464471983e-05,
                                         0.00505788990584361,
                                         0.0465653783079346,
                                         0.148014126427974,
                                         0.239812881985828,
                                         0.230662543974593,
                                         0.140471540786699,
                                         0.0552451735555166,
                                         0.013885498046875,
                                         0.00214745413887342,
                                         0.000190469720536565,
                                         8.61925780348323e-06,
                                         1.63824359889652e-07,
                                         9.28993029953034e-10,
                                         8.03487292726546e-13,
                                         2.1373052995378e-17,
                                         5.37787724412304e-26,
                                         6.38977670358737e-07,
                                         0.000383679055444144,
                                         0.00889787201079268,
                                         0.0544576362227137,
                                         0.148572650441549,
                                         0.222922080219439,
                                         0.20233421741395,
                                         0.115533457345836,
                                         0.041656494140625,
                                         0.00924172368740819,
                                         0.00119011099126256,
                                         8.02668778772614e-05,
                                         2.37987762828997e-06,
                                         2.27247620703635e-08,
                                         3.78441280515643e-11,
                                         2.53577287375044e-15,
                                         3.40762136294638e-23,
                                         6.65562998516207e-09,
                                         2.13436277920398e-05,
                                         0.00124683963518382,
                                         0.0146931811442083,
                                         0.0675004486945073,
                                         0.157990335990049,
                                         0.213723239302724,
                                         0.177183233670845,
                                         0.091644287109375,
                                         0.0291664443290135,
                                         0.00545318728977798,
                                         0.000548156157151217,
                                         2.53531659654486e-05,
                                         4.07650200417407e-07,
                                         1.30713191944047e-09,
                                         2.20625428836426e-13,
                                         1.58340935263852e-20,
                                         5.25192755153367e-11,
                                         8.99486094821649e-07,
                                         0.000132361327175114,
                                         0.003003301603014,
                                         0.0232327445499534,
                                         0.0848269842038378,
                                         0.171025248741604,
                                         0.205856007686019,
                                         0.152740478515625,
                                         0.0697332753632748,
                                         0.0189295125309029,
                                         0.00283594818376388,
                                         0.000204614233493506,
                                         5.53990151686061e-06,
                                         3.42031736651714e-08,
                                         1.45420907611496e-11,
                                         5.5739253523792e-18,
                                         3.19701006231835e-13,
                                         2.92426272901658e-08,
                                         1.08394525029291e-05,
                                         0.000473563021442755,
                                         0.00616864950152859,
                                         0.0351344574353183,
                                         0.105575812728513,
                                         0.184501621869361,
                                         0.196380615234375,
                                         0.128615218842363,
                                         0.0506902188790498,
                                         0.0113184773506547,
                                         0.00127389960386032,
                                         5.80780663139564e-05,
                                         6.90413239355092e-07,
                                         7.39424531195182e-10,
                                         1.51364771775961e-15,
                                         1.51364771775947e-15,
                                         7.39424531195195e-10,
                                         6.90413239355092e-07,
                                         5.80780663139566e-05,
                                         0.00127389960386032,
                                         0.0113184773506547,
                                         0.0506902188790498,
                                         0.128615218842363,
                                         0.196380615234375,
                                         0.184501621869361,
                                         0.105575812728513,
                                         0.0351344574353182,
                                         0.00616864950152858,
                                         0.000473563021442753,
                                         1.08394525029291e-05,
                                         2.92426272901653e-08,
                                         3.19701006231861e-13,
                                         5.57392535237861e-18,
                                         1.45420907611499e-11,
                                         3.42031736651714e-08,
                                         5.53990151686064e-06,
                                         0.000204614233493507,
                                         0.00283594818376388,
                                         0.0189295125309029,
                                         0.0697332753632748,
                                         0.152740478515625,
                                         0.205856007686019,
                                         0.171025248741604,
                                         0.0848269842038377,
                                         0.0232327445499533,
                                         0.00300330160301399,
                                         0.000132361327175114,
                                         8.99486094821637e-07,
                                         5.25192755153405e-11,
                                         1.58340935263833e-20,
                                         2.20625428836431e-13,
                                         1.30713191944047e-09,
                                         4.07650200417409e-07,
                                         2.53531659654487e-05,
                                         0.000548156157151218,
                                         0.00545318728977798,
                                         0.0291664443290135,
                                         0.091644287109375,
                                         0.177183233670845,
                                         0.213723239302724,
                                         0.157990335990049,
                                         0.0675004486945072,
                                         0.0146931811442083,
                                         0.00124683963518382,
                                         2.13436277920395e-05,
                                         6.65562998516246e-09,
                                         3.40762136294594e-23,
                                         2.5357728737505e-15,
                                         3.78441280515643e-11,
                                         2.27247620703636e-08,
                                         2.37987762828997e-06,
                                         8.02668778772616e-05,
                                         0.00119011099126256,
                                         0.00924172368740819,
                                         0.041656494140625,
                                         0.115533457345836,
                                         0.20233421741395,
                                         0.222922080219438,
                                         0.148572650441549,
                                         0.0544576362227136,
                                         0.00889787201079268,
                                         0.000383679055444141,
                                         6.38977670358767e-07,
                                         5.37787724412228e-26,
                                         2.13730529953786e-17,
                                         8.03487292726547e-13,
                                         9.28993029953039e-10,
                                         1.63824359889652e-07,
                                         8.61925780348325e-06,
                                         0.000190469720536565,
                                         0.00214745413887342,
                                         0.013885498046875,
                                         0.0552451735555166,
                                         0.140471540786699,
                                         0.230662543974593,
                                         0.239812881985828,
                                         0.148014126427974,
                                         0.0465653783079346,
                                         0.00505788990584358,
                                         4.49866464471999e-05,
                                         5.87583503500622e-29,
                                         1.24715930310875e-19,
                                         1.18102388062799e-14,
                                         2.62920665851432e-11,
                                         7.80731118815642e-09,
                                         6.40770527432609e-07,
                                         2.11039417828462e-05,
                                         0.000345456980761404,
                                         0.003204345703125,
                                         0.0182885835987682,
                                         0.0675159705897651,
                                         0.165234307695218,
                                         0.267981801785128,
                                         0.278513790049926,
                                         0.168709397283883,
                                         0.0461604260583782,
                                         0.00219270756445654,
                                         3.97422418237279e-32,
                                         4.50506805833433e-22,
                                         1.07463851934766e-16,
                                         4.60639365081089e-13,
                                         2.303289660158e-10,
                                         2.94889454932417e-08,
                                         1.44752233427079e-06,
                                         3.44023496774539e-05,
                                         0.000457763671875,
                                         0.00374791608871041,
                                         0.020088557022853,
                                         0.0732735832961556,
                                         0.185379691392498,
                                         0.32442498839537,
                                         0.378389842574495,
                                         0.260792022875746,
                                         0.0661609705674213,
                                         1.25441693879662e-35,
                                         7.59429674126571e-25,
                                         4.56323577276674e-19,
                                         3.76620673217441e-15,
                                         3.17104501282251e-12,
                                         6.33319322442762e-10,
                                         4.63333548724731e-08,
                                         1.59878115166929e-06,
                                         3.0517578125e-05,
                                         0.000358431712421671,
                                         0.00278931620488931,
                                         0.0151635684653473,
                                         0.0598447129028883,
                                         0.176355363836809,
                                         0.396046743445829,
                                         0.687583708993884,
                                         0.931600689535534,

                                         0.938572652458559,
                                         0.715313568599468,
                                         0.436959404734205,
                                         0.212340362539221,
                                         0.0811615644402857,
                                         0.0240138603988673,
                                         0.00538173576377702,
                                         0.000887202955297686,
                                         0.000103429674997502,
                                         8.08105554999943e-06,
                                         3.92703553442408e-07,
                                         1.06651321339041e-08,
                                         1.37953361745058e-10,
                                         6.59821878589306e-13,
                                         7.55422625875343e-16,
                                         8.90153034663427e-20,
                                         1.45329353276205e-25,
                                         2.37517872646337e-36,
                                         0.0596267376062189,
                                         0.24235101747505,
                                         0.371935177670949,
                                         0.346631193497711,
                                         0.221877580948695,
                                         0.101664952268861,
                                         0.0336369457179196,
                                         0.00795274448081866,
                                         0.00130895469308313,
                                         0.000143671675236529,
                                         9.85719390989183e-06,
                                         3.83932233031755e-07,
                                         7.33171419184216e-09,
                                         5.43058002078601e-11,
                                         1.04120781346229e-13,
                                         2.35299513967788e-17,
                                         9.65133687673311e-23,
                                         8.41212389848502e-33,
                                         0.00176775055170099,
                                         0.0383177511463794,
                                         0.14774071675497,
                                         0.264064190805671,
                                         0.283063072091682,
                                         0.200857161758744,
                                         0.0981109585984009,
                                         0.033267323318131,
                                         0.00773055812728158,
                                         0.00119201302324014,
                                         0.00011546451880022,
                                         6.44985112214474e-06,
                                         1.81838376958821e-07,
                                         2.08579923596436e-09,
                                         6.69717472557033e-12,
                                         2.90257977240869e-15,
                                         2.99108248429386e-20,
                                         1.39034252837557e-29,
                                         3.24432962542629e-05,
                                         0.00375041420867018,
                                         0.0363293146442765,
                                         0.124530415729962,
                                         0.223551231593572,
                                         0.245656035963779,
                                         0.177150564783944,
                                         0.0861475134808328,
                                         0.0282631877240519,
                                         0.00612230416795233,
                                         0.000837274486350311,
                                         6.70762636566749e-05,
                                         2.79183420273842e-06,
                                         4.95932817745731e-08,
                                         2.66667382604323e-10,
                                         2.21651847874888e-13,
                                         5.73843361694702e-18,
                                         1.42253189189365e-26,
                                         4.12219109041038e-07,
                                         0.000254130982361679,
                                         0.00618462491045457,
                                         0.0406574860427393,
                                         0.122227822556518,
                                         0.208001618698451,
                                         0.221445429247693,
                                         0.154442488785684,
                                         0.0715369785581957,
                                         0.0217694753036899,
                                         0.00420326065064266,
                                         0.000482933345143786,
                                         2.96751455514579e-05,
                                         8.16342391513335e-07,
                                         7.35101525563724e-09,
                                         1.17181136375597e-11,
                                         7.62179900115107e-16,
                                         1.0076306587651e-23,
                                         3.8408972951772e-09,
                                         1.26280828130915e-05,
                                         0.000772095376663821,
                                         0.00973435165906274,
                                         0.0490077212590973,
                                         0.129153867608909,
                                         0.202998265021822,
                                         0.203044982700018,
                                         0.132782692132278,
                                         0.0567652358977907,
                                         0.0154741289933712,
                                         0.00254980488825935,
                                         0.000231311656186481,
                                         9.85424320888193e-06,
                                         1.48602520580655e-07,
                                         4.5430286999847e-10,
                                         7.42374502415047e-14,
                                         5.23410174631185e-21,
                                         2.71121112103405e-11,
                                         4.75382615844966e-07,
                                         7.3022148558389e-05,
                                         0.00176562963617961,
                                         0.0148862398596413,
                                         0.0607539033601789,
                                         0.140975615821239,
                                         0.202229171262758,
                                         0.186714658514434,
                                         0.112135468307079,
                                         0.0431570918903814,
                                         0.0101988868674902,
                                         0.00136592938282289,
                                         9.0115661053747e-05,
                                         2.27578429799818e-06,
                                         1.33431793737991e-08,
                                         5.47790721174123e-12,
                                         2.05972394117766e-18,
                                         1.47635126306338e-13,
                                         1.38052695485961e-08,
                                         5.32762909919016e-06,
                                         0.000247051735004989,
                                         0.00348819885611465,
                                         0.0220463488859039,
                                         0.075525114865829,
                                         0.155378549053715,
                                         0.202540127761028,
                                         0.170883163384111,
                                         0.0928525535257361,
                                         0.0314698243663379,
                                         0.00622235372949805,
                                         0.000635730413692028,
                                         2.68863431446898e-05,
                                         3.0232139860186e-07,
                                         3.11818598955633e-10,
                                         6.25275760640007e-16,
                                         6.25275760640007e-16,
                                         3.11818598955633e-10,
                                         3.0232139860186e-07,
                                         2.68863431446897e-05,
                                         0.000635730413692028,
                                         0.00622235372949805,
                                         0.0314698243663379,
                                         0.0928525535257361,
                                         0.170883163384111,
                                         0.202540127761028,
                                         0.155378549053715,
                                         0.075525114865829,
                                         0.0220463488859039,
                                         0.00348819885611465,
                                         0.00024705173500499,
                                         5.32762909919016e-06,
                                         1.38052695485961e-08,
                                         1.47635126306338e-13,
                                         2.05972394117766e-18,
                                         5.47790721174123e-12,
                                         1.33431793737991e-08,
                                         2.27578429799817e-06,
                                         9.0115661053747e-05,
                                         0.00136592938282289,
                                         0.0101988868674902,
                                         0.0431570918903814,
                                         0.112135468307079,
                                         0.186714658514434,
                                         0.202229171262758,
                                         0.140975615821239,
                                         0.0607539033601789,
                                         0.0148862398596413,
                                         0.00176562963617962,
                                         7.3022148558389e-05,
                                         4.75382615844966e-07,
                                         2.71121112103405e-11,
                                         5.23410174631185e-21,
                                         7.42374502415047e-14,
                                         4.5430286999847e-10,
                                         1.48602520580654e-07,
                                         9.85424320888193e-06,
                                         0.000231311656186481,
                                         0.00254980488825935,
                                         0.0154741289933712,
                                         0.0567652358977907,
                                         0.132782692132278,
                                         0.203044982700018,
                                         0.202998265021822,
                                         0.129153867608909,
                                         0.0490077212590973,
                                         0.00973435165906277,
                                         0.000772095376663821,
                                         1.26280828130915e-05,
                                         3.8408972951772e-09,
                                         1.0076306587651e-23,
                                         7.62179900115107e-16,
                                         1.17181136375597e-11,
                                         7.35101525563719e-09,
                                         8.16342391513335e-07,
                                         2.96751455514579e-05,
                                         0.000482933345143786,
                                         0.00420326065064266,
                                         0.0217694753036899,
                                         0.0715369785581957,
                                         0.154442488785684,
                                         0.221445429247693,
                                         0.208001618698451,
                                         0.122227822556518,
                                         0.0406574860427394,
                                         0.00618462491045457,
                                         0.000254130982361679,
                                         4.12219109041038e-07,
                                         1.42253189189365e-26,
                                         5.73843361694702e-18,
                                         2.21651847874888e-13,
                                         2.66667382604321e-10,
                                         4.95932817745731e-08,
                                         2.79183420273842e-06,
                                         6.70762636566749e-05,
                                         0.000837274486350311,
                                         0.00612230416795233,
                                         0.0282631877240519,
                                         0.0861475134808328,
                                         0.177150564783944,
                                         0.245656035963779,
                                         0.223551231593572,
                                         0.124530415729962,
                                         0.0363293146442765,
                                         0.00375041420867018,
                                         3.24432962542629e-05,
                                         1.39034252837557e-29,
                                         2.99108248429386e-20,
                                         2.90257977240869e-15,
                                         6.69717472557029e-12,
                                         2.08579923596436e-09,
                                         1.81838376958821e-07,
                                         6.44985112214474e-06,
                                         0.00011546451880022,
                                         0.00119201302324014,
                                         0.00773055812728158,
                                         0.033267323318131,
                                         0.0981109585984009,
                                         0.200857161758744,
                                         0.283063072091682,
                                         0.264064190805671,
                                         0.14774071675497,
                                         0.0383177511463794,
                                         0.00176775055170099,
                                         8.41212389848502e-33,
                                         9.65133687673311e-23,
                                         2.35299513967788e-17,
                                         1.04120781346228e-13,
                                         5.43058002078601e-11,
                                         7.33171419184216e-09,
                                         3.83932233031755e-07,
                                         9.85719390989183e-06,
                                         0.000143671675236529,
                                         0.00130895469308313,
                                         0.00795274448081866,
                                         0.0336369457179196,
                                         0.101664952268861,
                                         0.221877580948696,
                                         0.346631193497711,
                                         0.371935177670949,
                                         0.24235101747505,
                                         0.0596267376062189,
                                         2.37517872646337e-36,
                                         1.45329353276205e-25,
                                         8.90153034663427e-20,
                                         7.55422625875337e-16,
                                         6.59821878589306e-13,
                                         1.37953361745058e-10,
                                         1.06651321339041e-08,
                                         3.92703553442408e-07,
                                         8.08105554999943e-06,
                                         0.000103429674997502,
                                         0.000887202955297686,
                                         0.00538173576377702,
                                         0.0240138603988673,
                                         0.0811615644402857,
                                         0.212340362539221,
                                         0.436959404734205,
                                         0.715313568599468,
                                         0.938572652458559,

                                         0.944540181233619,
                                         0.739746519032059,
                                         0.474969324826381,
                                         0.248502265085717,
                                         0.104986894955356,
                                         0.0353646724869089,
                                         0.0093369572607942,
                                         0.00188877486123395,
                                         0.0002840754490929,
                                         3.0517578125e-05,
                                         2.21797870605539e-06,
                                         1.01161878481636e-07,
                                         2.60064790987367e-09,
                                         3.21042863799015e-11,
                                         1.47707584912711e-13,
                                         1.63924006912301e-16,
                                         1.88645884115662e-20,
                                         3.03013290545314e-26,
                                         4.9077794329579e-37,
                                         0.0539953055843802,
                                         0.22525067742465,
                                         0.362539729294824,
                                         0.362556552055586,
                                         0.255335302473448,
                                         0.13239160834298,
                                         0.0512021710121141,
                                         0.014707106330286,
                                         0.00308337636444878,
                                         0.000457763671875,
                                         4.5977680006835e-05,
                                         2.92315850294191e-06,
                                         1.067040914203e-07,
                                         1.92954415448835e-09,
                                         1.36649970607349e-11,
                                         2.52801515445025e-14,
                                         5.56083243246037e-18,
                                         2.23903571032207e-23,
                                         1.93166579131349e-33,
                                         0.00144045053748797,
                                         0.0320078175090363,
                                         0.129137516765806,
                                         0.24684705221276,
                                         0.289796688128563,
                                         0.231290639480034,
                                         0.131032238889606,
                                         0.0534417968440209,
                                         0.0156180265598209,
                                         0.003204345703125,
                                         0.000444778222961199,
                                         3.94180036531643e-05,
                                         2.04308937984851e-06,
                                         5.41194285822988e-08,
                                         5.89960681870964e-10,
                                         1.81938062604194e-12,
                                         7.64960594141938e-16,
                                         7.72088606444238e-21,
                                         3.54801724651174e-30,
                                         2.37883737964071e-05,
                                         0.00281559429365901,
                                         0.0284756281674274,
                                         0.104040921528033,
                                         0.20361043382446,
                                         0.250138023078162,
                                         0.207583105650408,
                                         0.120215074183531,
                                         0.0489722246051592,
                                         0.013885498046875,
                                         0.00266356951033681,
                                         0.000329049279037475,
                                         2.42168536951775e-05,
                                         9.39670821372375e-07,
                                         1.57674206026335e-08,
                                         8.10571847728912e-11,
                                         6.51421932311574e-14,
                                         1.6481522390765e-18,
                                         4.03425655415917e-27,
                                         2.71975832899961e-07,
                                         0.000171468065258225,
                                         0.0043470371612284,
                                         0.0303584498667472,
                                         0.0990388976251295,
                                         0.187283935375395,
                                         0.227669579668481,
                                         0.187212982850201,
                                         0.10630964438792,
                                         0.041656494140625,
                                         0.0110429145498902,
                                         0.0019016317457567,
                                         0.000198722584536849,
                                         1.12952913456281e-05,
                                         2.91740950632542e-07,
                                         2.50010715749002e-09,
                                         3.84047415123377e-12,
                                         2.43571607878876e-16,
                                         3.17570725640976e-24,
                                         2.28032790993275e-09,
                                         7.6576911232263e-06,
                                         0.000486647860572637,
                                         0.00649615562321363,
                                         0.0353275070909286,
                                         0.102830693816878,
                                         0.183113107329288,
                                         0.213803309356348,
                                         0.169237625873408,
                                         0.091644287109375,
                                         0.0335741336106103,
                                         0.00805922573411588,
                                         0.0011958540035049,
                                         9.9568177883056e-05,
                                         3.95854476827324e-06,
                                         5.65492900523694e-08,
                                         1.66038477492385e-10,
                                         2.63971735271809e-14,
                                         1.83323793571794e-21,
                                         1.4484061460184e-11,
                                         2.59082757650019e-07,
                                         4.12726498351354e-05,
                                         0.00105307503537753,
                                         0.00954654569297639,
                                         0.042773134204545,
                                         0.111573209964981,
                                         0.18497751984977,
                                         0.204101996009142,
                                         0.152740478515625,
                                         0.0773307079319541,
                                         0.0258753534273791,
                                         0.0054517403755366,
                                         0.000664920542363681,
                                         4.0691134337563e-05,
                                         9.68995497897421e-07,
                                         5.43824422382309e-09,
                                         2.1672761933812e-12,
                                         8.01721035424734e-19,
                                         7.09707075479524e-14,
                                         6.76199580357498e-09,
                                         2.70026014909722e-06,
                                         0.00013169155750544,
                                         0.00199010155352867,
                                         0.0137250868887533,
                                         0.0524440403940532,
                                         0.123457989725641,
                                         0.189886154472877,
                                         0.196380615234375,
                                         0.137402576793666,
                                         0.0640877434886427,
                                         0.0191729036188828,
                                         0.00342542656285371,
                                         0.000322670864358328,
                                         1.28089072635571e-05,
                                         1.37405587029177e-07,
                                         1.37267212151797e-10,
                                         2.70472703207217e-16,
                                         2.70472703207216e-16,
                                         1.37267212151797e-10,
                                         1.37405587029177e-07,
                                         1.28089072635571e-05,
                                         0.000322670864358327,
                                         0.00342542656285372,
                                         0.0191729036188828,
                                         0.0640877434886426,
                                         0.137402576793666,
                                         0.196380615234375,
                                         0.189886154472877,
                                         0.123457989725642,
                                         0.0524440403940532,
                                         0.0137250868887533,
                                         0.00199010155352868,
                                         0.00013169155750544,
                                         2.70026014909723e-06,
                                         6.76199580357498e-09,
                                         7.09707075479524e-14,
                                         8.01721035424734e-19,
                                         2.1672761933812e-12,
                                         5.43824422382309e-09,
                                         9.68995497897421e-07,
                                         4.06911343375629e-05,
                                         0.000664920542363682,
                                         0.0054517403755366,
                                         0.0258753534273791,
                                         0.0773307079319541,
                                         0.152740478515625,
                                         0.204101996009142,
                                         0.18497751984977,
                                         0.111573209964981,
                                         0.042773134204545,
                                         0.00954654569297641,
                                         0.00105307503537753,
                                         4.12726498351354e-05,
                                         2.59082757650019e-07,
                                         1.4484061460184e-11,
                                         1.83323793571794e-21,
                                         2.63971735271809e-14,
                                         1.66038477492385e-10,
                                         5.65492900523694e-08,
                                         3.95854476827322e-06,
                                         9.95681778830563e-05,
                                         0.0011958540035049,
                                         0.00805922573411587,
                                         0.0335741336106103,
                                         0.091644287109375,
                                         0.169237625873408,
                                         0.213803309356348,
                                         0.183113107329288,
                                         0.102830693816878,
                                         0.0353275070909286,
                                         0.00649615562321363,
                                         0.000486647860572637,
                                         7.6576911232263e-06,
                                         2.28032790993275e-09,
                                         3.17570725640976e-24,
                                         2.43571607878876e-16,
                                         3.84047415123377e-12,
                                         2.50010715749002e-09,
                                         2.91740950632541e-07,
                                         1.12952913456281e-05,
                                         0.000198722584536849,
                                         0.0019016317457567,
                                         0.0110429145498902,
                                         0.041656494140625,
                                         0.10630964438792,
                                         0.187212982850201,
                                         0.227669579668481,
                                         0.187283935375395,
                                         0.0990388976251297,
                                         0.0303584498667472,
                                         0.0043470371612284,
                                         0.000171468065258225,
                                         2.71975832899961e-07,
                                         4.03425655415917e-27,
                                         1.6481522390765e-18,
                                         6.51421932311573e-14,
                                         8.10571847728912e-11,
                                         1.57674206026334e-08,
                                         9.39670821372378e-07,
                                         2.42168536951775e-05,
                                         0.000329049279037475,
                                         0.00266356951033681,
                                         0.013885498046875,
                                         0.0489722246051592,
                                         0.120215074183531,
                                         0.207583105650408,
                                         0.250138023078162,
                                         0.20361043382446,
                                         0.104040921528033,
                                         0.0284756281674274,
                                         0.00281559429365901,
                                         2.37883737964071e-05,
                                         3.54801724651174e-30,
                                         7.72088606444238e-21,
                                         7.64960594141938e-16,
                                         1.81938062604194e-12,
                                         5.89960681870961e-10,
                                         5.4119428582299e-08,
                                         2.04308937984852e-06,
                                         3.94180036531642e-05,
                                         0.000444778222961199,
                                         0.003204345703125,
                                         0.0156180265598209,
                                         0.0534417968440209,
                                         0.131032238889606,
                                         0.231290639480033,
                                         0.289796688128563,
                                         0.24684705221276,
                                         0.129137516765806,
                                         0.0320078175090363,
                                         0.00144045053748797,
                                         1.93166579131349e-33,
                                         2.23903571032207e-23,
                                         5.56083243246037e-18,
                                         2.52801515445025e-14,
                                         1.36649970607348e-11,
                                         1.92954415448836e-09,
                                         1.067040914203e-07,
                                         2.92315850294191e-06,
                                         4.5977680006835e-05,
                                         0.000457763671875,
                                         0.00308337636444878,
                                         0.0147071063302861,
                                         0.0512021710121141,
                                         0.13239160834298,
                                         0.255335302473449,
                                         0.362556552055586,
                                         0.362539729294824,
                                         0.22525067742465,
                                         0.0539953055843802,
                                         4.9077794329579e-37,
                                         3.03013290545314e-26,
                                         1.88645884115662e-20,
                                         1.63924006912301e-16,
                                         1.4770758491271e-13,
                                         3.21042863799016e-11,
                                         2.60064790987367e-09,
                                         1.01161878481636e-07,
                                         2.21797870605539e-06,
                                         3.0517578125e-05,
                                         0.0002840754490929,
                                         0.00188877486123395,
                                         0.0093369572607942,
                                         0.0353646724869089,
                                         0.104986894955356,
                                         0.248502265085717,
                                         0.474969324826381,
                                         0.739746519032059,
                                         0.944540181233619,

                                         0.949685653980178,
                                         0.7613409704789,
                                         0.510113786212756,
                                         0.284243372819712,
                                         0.13075307108967,
                                         0.0491488435791702,
                                         0.0148916769590573,
                                         0.00357237805500046,
                                         0.000662810576356635,
                                         9.22403766229386e-05,
                                         9.24520355324406e-06,
                                         6.31853485542536e-07,
                                         2.73037883571442e-08,
                                         6.69814430744469e-10,
                                         7.94529521290208e-12,
                                         3.53620748643473e-14,
                                         3.82123323108111e-17,
                                         4.309360840118e-21,
                                         6.82598807727884e-27,
                                         1.09705726041372e-37,
                                         0.0491112624092424,
                                         0.209496231614179,
                                         0.35118937910336,
                                         0.372977373864846,
                                         0.284893285277844,
                                         0.164000187719183,
                                         0.0723161991183711,
                                         0.0244305384139227,
                                         0.00625310641020541,
                                         0.00118689423557701,
                                         0.000161662035447316,
                                         1.50692804066833e-05,
                                         8.98317378679496e-07,
                                         3.10345200041245e-08,
                                         5.35749180132219e-10,
                                         3.65165670340057e-12,
                                         6.55230228586308e-15,
                                         1.40838394794987e-18,
                                         5.58150129058575e-24,
                                         4.77321057141193e-34,
                                         0.00118519305816242,
                                         0.0269017176566363,
                                         0.11282944881438,
                                         0.228392272965666,
                                         0.289680532033897,
                                         0.255377227340343,
                                         0.16388317082597,
                                         0.077967829571508,
                                         0.0275301761121557,
                                         0.00712704916303021,
                                         0.00131918707815141,
                                         0.000167716358322306,
                                         1.37925153241849e-05,
                                         6.71030633137339e-07,
                                         1.68585327060284e-08,
                                         1.75974170283659e-10,
                                         5.24313554540191e-13,
                                         2.14800878506767e-16,
                                         2.12982202848701e-21,
                                         9.69167121705966e-31,
                                         1.77060284581549e-05,
                                         0.00213849314681628,
                                         0.0224402462039108,
                                         0.0865773690401914,
                                         0.182339375526229,
                                         0.246175057121122,
                                         0.229909640279678,
                                         0.154035885209098,
                                         0.075031947259591,
                                         0.0264930248103812,
                                         0.00666390492348025,
                                         0.00115553310211868,
                                         0.000131093514400577,
                                         8.98180698251516e-06,
                                         3.28399198197929e-07,
                                         5.24966956082988e-09,
                                         2.59724131650785e-11,
                                         2.02803337341338e-14,
                                         5.03106185946224e-19,
                                         1.2181781597883e-27,
                                         1.83126997062569e-07,
                                         0.000117688728956476,
                                         0.0030898110752063,
                                         0.0227209517114897,
                                         0.0794585753072719,
                                         0.164287713156918,
                                         0.22329508451479,
                                         0.210682047240544,
                                         0.14157370661107,
                                         0.068179293237232,
                                         0.0233050668436434,
                                         0.00551173736743083,
                                         0.000862617162701495,
                                         8.32308339974813e-05,
                                         4.42877442501863e-06,
                                         1.08421189175016e-07,
                                         8.90702513349477e-10,
                                         1.32560245242486e-12,
                                         8.22763765295049e-17,
                                         1.06003963147226e-24,
                                         1.38894489507346e-09,
                                         4.74966859676009e-06,
                                         0.000311987834509114,
                                         0.00437270399596622,
                                         0.0253923274984874,
                                         0.0804021287695688,
                                         0.159038609483258,
                                         0.211317068383707,
                                         0.195893675065091,
                                         0.128669279695529,
                                         0.0597686353077339,
                                         0.0192795132838202,
                                         0.00416252072712399,
                                         0.000565595832816414,
                                         4.37992283311753e-05,
                                         1.64209315634118e-06,
                                         2.24003332168652e-08,
                                         6.35408362736143e-11,
                                         9.86715800933427e-15,
                                         6.7644863076286e-22,
                                         7.98075037052138e-12,
                                         1.4521711778153e-07,
                                         2.38654409293694e-05,
                                         0.000637528653673307,
                                         0.00614738363237534,
                                         0.0298095952597789,
                                         0.0858127946422748,
                                         0.160571214816146,
                                         0.205345076283004,
                                         0.183959962916298,
                                         0.116124109161762,
                                         0.0510892685700535,
                                         0.015216710994167,
                                         0.00291175094044432,
                                         0.000328152280200942,
                                         1.88411533635781e-05,
                                         4.26778319008328e-07,
                                         2.30737733679573e-09,
                                         8.96468590009083e-13,
                                         3.27019470064119e-19,
                                         3.5375139194142e-14,
                                         3.42505884306061e-09,
                                         1.40830585677282e-06,
                                         7.17042765104015e-05,
                                         0.00114808446898872,
                                         0.00852590189732826,
                                         0.0357188313674406,
                                         0.0941231664095987,
                                         0.166051917000509,
                                         0.202893192971402,
                                         0.174047252040156,
                                         0.104438114866379,
                                         0.0429122113655922,
                                         0.0115637306599523,
                                         0.00189661907220643,
                                         0.000166768068196963,
                                         6.27257608680561e-06,
                                         6.46368284613001e-08,
                                         6.28309739876199e-11,
                                         1.21957389194767e-16,
                                         1.21957389194751e-16,
                                         6.28309739876183e-11,
                                         6.46368284613001e-08,
                                         6.27257608680557e-06,
                                         0.000166768068196963,
                                         0.00189661907220643,
                                         0.0115637306599523,
                                         0.0429122113655922,
                                         0.104438114866379,
                                         0.174047252040156,
                                         0.202893192971402,
                                         0.166051917000509,
                                         0.0941231664095987,
                                         0.0357188313674406,
                                         0.00852590189732824,
                                         0.00114808446898872,
                                         7.17042765104019e-05,
                                         1.40830585677282e-06,
                                         3.42505884306068e-09,
                                         3.5375139194146e-14,
                                         3.27019470064071e-19,
                                         8.96468590009058e-13,
                                         2.30737733679573e-09,
                                         4.26778319008325e-07,
                                         1.88411533635781e-05,
                                         0.000328152280200943,
                                         0.00291175094044432,
                                         0.015216710994167,
                                         0.0510892685700536,
                                         0.116124109161762,
                                         0.183959962916298,
                                         0.205345076283004,
                                         0.160571214816146,
                                         0.0858127946422748,
                                         0.0298095952597788,
                                         0.00614738363237534,
                                         0.00063752865367331,
                                         2.38654409293694e-05,
                                         1.45217117781532e-07,
                                         7.98075037052216e-12,
                                         6.76448630762751e-22,
                                         9.86715800933396e-15,
                                         6.35408362736143e-11,
                                         2.2400333216865e-08,
                                         1.64209315634118e-06,
                                         4.37992283311755e-05,
                                         0.000565595832816414,
                                         0.00416252072712399,
                                         0.0192795132838202,
                                         0.0597686353077339,
                                         0.128669279695529,
                                         0.195893675065091,
                                         0.211317068383707,
                                         0.159038609483258,
                                         0.0804021287695686,
                                         0.0253923274984874,
                                         0.00437270399596624,
                                         0.000311987834509114,
                                         4.74966859676017e-06,
                                         1.38894489507357e-09,
                                         1.06003963147207e-24,
                                         8.22763765295021e-17,
                                         1.32560245242486e-12,
                                         8.9070251334947e-10,
                                         1.08421189175016e-07,
                                         4.42877442501864e-06,
                                         8.32308339974813e-05,
                                         0.000862617162701495,
                                         0.00551173736743083,
                                         0.0233050668436434,
                                         0.068179293237232,
                                         0.14157370661107,
                                         0.210682047240544,
                                         0.22329508451479,
                                         0.164287713156917,
                                         0.0794585753072719,
                                         0.0227209517114897,
                                         0.0030898110752063,
                                         0.000117688728956477,
                                         1.83126997062581e-07,
                                         1.21817815978806e-27,
                                         5.03106185946205e-19,
                                         2.02803337341338e-14,
                                         2.59724131650783e-11,
                                         5.24966956082988e-09,
                                         3.2839919819793e-07,
                                         8.98180698251516e-06,
                                         0.000131093514400577,
                                         0.00115553310211868,
                                         0.00666390492348025,
                                         0.0264930248103812,
                                         0.075031947259591,
                                         0.154035885209098,
                                         0.229909640279678,
                                         0.246175057121122,
                                         0.182339375526229,
                                         0.0865773690401916,
                                         0.0224402462039108,
                                         0.0021384931468163,
                                         1.77060284581558e-05,
                                         9.69167121705763e-31,
                                         2.12982202848693e-21,
                                         2.14800878506767e-16,
                                         5.24313554540187e-13,
                                         1.75974170283659e-10,
                                         1.68585327060285e-08,
                                         6.71030633137339e-07,
                                         1.37925153241849e-05,
                                         0.000167716358322306,
                                         0.00131918707815141,
                                         0.00712704916303021,
                                         0.0275301761121557,
                                         0.077967829571508,
                                         0.16388317082597,
                                         0.255377227340343,
                                         0.289680532033897,
                                         0.228392272965666,
                                         0.11282944881438,
                                         0.0269017176566365,
                                         0.00118519305816246,
                                         4.77321057141085e-34,
                                         5.58150129058551e-24,
                                         1.40838394794987e-18,
                                         6.55230228586302e-15,
                                         3.65165670340057e-12,
                                         5.35749180132221e-10,
                                         3.10345200041245e-08,
                                         8.98317378679496e-07,
                                         1.50692804066833e-05,
                                         0.000161662035447316,
                                         0.00118689423557701,
                                         0.00625310641020541,
                                         0.0244305384139227,
                                         0.0723161991183711,
                                         0.164000187719183,
                                         0.284893285277844,
                                         0.372977373864846,
                                         0.35118937910336,
                                         0.20949623161418,
                                         0.0491112624092431,
                                         1.09705726041346e-37,
                                         6.82598807727852e-27,
                                         4.309360840118e-21,
                                         3.82123323108107e-17,
                                         3.53620748643473e-14,
                                         7.94529521290212e-12,
                                         6.69814430744469e-10,
                                         2.73037883571442e-08,
                                         6.31853485542536e-07,
                                         9.24520355324406e-06,
                                         9.22403766229386e-05,
                                         0.000662810576356635,
                                         0.00357237805500046,
                                         0.0148916769590573,
                                         0.0491488435791702,
                                         0.13075307108967,
                                         0.284243372819712,
                                         0.510113786212756,
                                         0.7613409704789,
                                         0.949685653980178,

                                         0.954152216238001,
                                         0.780488802607072,
                                         0.542507217326621,
                                         0.319123418945609,
                                         0.157908269437273,
                                         0.065181817875107,
                                         0.0221986064202114,
                                         0.0061484383755995,
                                         0.00135955200643357,
                                         0.000234330177130463,
                                         3.0517578125e-05,
                                         2.88243484188347e-06,
                                         1.86876602308397e-07,
                                         7.7093341879547e-09,
                                         1.81660484339645e-10,
                                         2.08208090293545e-12,
                                         9.00542484149015e-15,
                                         9.51015012166728e-18,
                                         1.05391610888149e-21,
                                         1.64939670833693e-27,
                                         2.633222030724e-38,
                                         0.0448504608828419,
                                         0.195039166482249,
                                         0.33862847969626,
                                         0.378731866597892,
                                         0.310148312017239,
                                         0.195213180101158,
                                         0.0962230873797469,
                                         0.0372740188166733,
                                         0.0112727798210936,
                                         0.00262169137355084,
                                         0.000457763671875,
                                         5.79680475066438e-05,
                                         5.07110085734162e-06,
                                         2.8612644214866e-07,
                                         9.42951617276984e-09,
                                         1.56421861899188e-10,
                                         1.03162575568788e-12,
                                         1.8030041173306e-15,
                                         3.79901143012294e-19,
                                         1.48508901650273e-24,
                                         1.26043586192051e-34,
                                         0.000983836516521524,
                                         0.0227448862255866,
                                         0.0986389004230261,
                                         0.209754748545149,
                                         0.284276363118728,
                                         0.272834059653033,
                                         0.194643383124635,
                                         0.105451896965319,
                                         0.0436187288192293,
                                         0.013688053382975,
                                         0.003204345703125,
                                         0.000544032227669428,
                                         6.42179367253432e-05,
                                         4.95571015866236e-06,
                                         2.2841526929764e-07,
                                         5.48408379708292e-09,
                                         5.51501791399284e-11,
                                         1.59519156821247e-13,
                                         6.39060446855064e-17,
                                         6.24002931113145e-22,
                                         2.8155341768501e-31,
                                         1.33598943375997e-05,
                                         0.00164198718297789,
                                         0.0177867672208171,
                                         0.0719143918388656,
                                         0.161300657268978,
                                         0.236054395431008,
                                         0.243738457972253,
                                         0.184682876713669,
                                         0.104481414277458,
                                         0.0442410955616227,
                                         0.013885498046875,
                                         0.00316070992239508,
                                         0.000503424682919708,
                                         5.3134654285668e-05,
                                         3.42519193784661e-06,
                                         1.19024062928144e-07,
                                         1.82513812144412e-09,
                                         8.73681279056656e-12,
                                         6.65483565571877e-15,
                                         1.62309832969821e-19,
                                         3.89336314322255e-28,
                                         1.25597867549878e-07,
                                         8.20644218015185e-05,
                                         0.00222047034595313,
                                         0.0170694284652471,
                                         0.0633622631132911,
                                         0.141391973133483,
                                         0.211303957512532,
                                         0.223922648360982,
                                         0.1732623648044,
                                         0.0989940775112115,
                                         0.041656494140625,
                                         0.0127128759529146,
                                         0.00273219580153545,
                                         0.000394410962474845,
                                         3.55585342515977e-05,
                                         1.78839981748814e-06,
                                         4.18161169941647e-08,
                                         3.31277829102797e-10,
                                         4.79768674349168e-13,
                                         2.92282079860481e-17,
                                         3.72724705470278e-25,
                                         8.65890419384957e-10,
                                         3.0077483084629e-06,
                                         0.000203279863269566,
                                         0.00297114313049084,
                                         0.0182526804304229,
                                         0.0621067558971868,
                                         0.134336066933119,
                                         0.199099807104868,
                                         0.210703067728917,
                                         0.162440373347902,
                                         0.091644287109375,
                                         0.0374976805546057,
                                         0.0108740308349566,
                                         0.00214694798028164,
                                         0.000270709947388363,
                                         1.97058824989293e-05,
                                         7.02575650546651e-07,
                                         9.2115590245841e-09,
                                         2.53645912109893e-11,
                                         3.85976597592817e-15,
                                         2.61669360658387e-22,
                                         4.52240728267448e-12,
                                         8.35130100680324e-08,
                                         1.4098399442305e-05,
                                         0.000391790802532417,
                                         0.00398335202629534,
                                         0.020667074414468,
                                         0.0646999128487233,
                                         0.134112644238876,
                                         0.194116990081802,
                                         0.201931845217354,
                                         0.152740478515625,
                                         0.0837897837154769,
                                         0.0327865235423619,
                                         0.00885360485094428,
                                         0.00156131541289935,
                                         0.000164495199997237,
                                         8.94269780198103e-06,
                                         1.94043891536702e-07,
                                         1.01589755599144e-09,
                                         3.86140919697001e-13,
                                         1.39169361629292e-19,
                                         1.82209966810049e-14,
                                         1.78880295304519e-09,
                                         7.54294571072282e-07,
                                         3.98547986183756e-05,
                                         0.000670604474463147,
                                         0.00530536027818495,
                                         0.0240386725814711,
                                         0.0696890158285983,
                                         0.137959610121813,
                                         0.193647265385882,
                                         0.196380615234375,
                                         0.144435335186961,
                                         0.0762598415628951,
                                         0.0281653056014931,
                                         0.00694660811438449,
                                         0.00105926906740699,
                                         8.7809142412086e-05,
                                         3.1532799699653e-06,
                                         3.13882919510931e-08,
                                         2.98006954784864e-11,
                                         5.70992122626438e-17,
                                         5.7099212262652e-17,
                                         2.98006954784856e-11,
                                         3.13882919510935e-08,
                                         3.1532799699653e-06,
                                         8.7809142412086e-05,
                                         0.00105926906740699,
                                         0.00694660811438449,
                                         0.0281653056014931,
                                         0.0762598415628951,
                                         0.144435335186961,
                                         0.196380615234375,
                                         0.193647265385882,
                                         0.137959610121813,
                                         0.0696890158285984,
                                         0.0240386725814711,
                                         0.00530536027818495,
                                         0.000670604474463146,
                                         3.98547986183756e-05,
                                         7.54294571072275e-07,
                                         1.78880295304523e-09,
                                         1.82209966810026e-14,
                                         1.39169361629314e-19,
                                         3.8614091969699e-13,
                                         1.01589755599145e-09,
                                         1.94043891536702e-07,
                                         8.94269780198103e-06,
                                         0.000164495199997237,
                                         0.00156131541289935,
                                         0.00885360485094427,
                                         0.0327865235423619,
                                         0.0837897837154769,
                                         0.152740478515625,
                                         0.201931845217354,
                                         0.194116990081802,
                                         0.134112644238876,
                                         0.0646999128487233,
                                         0.020667074414468,
                                         0.00398335202629534,
                                         0.000391790802532417,
                                         1.40983994423049e-05,
                                         8.35130100680341e-08,
                                         4.52240728267399e-12,
                                         2.61669360658433e-22,
                                         3.85976597592804e-15,
                                         2.53645912109896e-11,
                                         9.21155902458411e-09,
                                         7.02575650546651e-07,
                                         1.97058824989293e-05,
                                         0.000270709947388363,
                                         0.00214694798028164,
                                         0.0108740308349566,
                                         0.0374976805546057,
                                         0.091644287109375,
                                         0.162440373347902,
                                         0.210703067728917,
                                         0.199099807104868,
                                         0.134336066933119,
                                         0.0621067558971868,
                                         0.0182526804304229,
                                         0.00297114313049084,
                                         0.000203279863269564,
                                         3.00774830846296e-06,
                                         8.6589041938488e-10,
                                         3.7272470547035e-25,
                                         2.9228207986047e-17,
                                         4.79768674349175e-13,
                                         3.31277829102797e-10,
                                         4.18161169941647e-08,
                                         1.78839981748814e-06,
                                         3.55585342515977e-05,
                                         0.000394410962474844,
                                         0.00273219580153545,
                                         0.0127128759529146,
                                         0.041656494140625,
                                         0.0989940775112115,
                                         0.1732623648044,
                                         0.223922648360982,
                                         0.211303957512532,
                                         0.141391973133483,
                                         0.0633622631132911,
                                         0.0170694284652471,
                                         0.00222047034595312,
                                         8.20644218015196e-05,
                                         1.25597867549869e-07,
                                         3.89336314322338e-28,
                                         1.62309832969815e-19,
                                         6.65483565571888e-15,
                                         8.73681279056657e-12,
                                         1.82513812144412e-09,
                                         1.19024062928144e-07,
                                         3.42519193784661e-06,
                                         5.31346542856678e-05,
                                         0.000503424682919708,
                                         0.00316070992239508,
                                         0.013885498046875,
                                         0.0442410955616227,
                                         0.104481414277458,
                                         0.184682876713669,
                                         0.243738457972253,
                                         0.236054395431008,
                                         0.161300657268978,
                                         0.0719143918388656,
                                         0.017786767220817,
                                         0.00164198718297791,
                                         1.33598943375989e-05,
                                         2.81553417685075e-31,
                                         6.24002931113117e-22,
                                         6.39060446855075e-17,
                                         1.59519156821247e-13,
                                         5.51501791399284e-11,
                                         5.48408379708292e-09,
                                         2.2841526929764e-07,
                                         4.95571015866235e-06,
                                         6.42179367253432e-05,
                                         0.000544032227669428,
                                         0.003204345703125,
                                         0.013688053382975,
                                         0.0436187288192293,
                                         0.105451896965319,
                                         0.194643383124635,
                                         0.272834059653033,
                                         0.284276363118728,
                                         0.209754748545149,
                                         0.0986389004230258,
                                         0.0227448862255868,
                                         0.000983836516521489,
                                         1.26043586192082e-34,
                                         1.48508901650266e-24,
                                         3.79901143012302e-19,
                                         1.8030041173306e-15,
                                         1.03162575568788e-12,
                                         1.56421861899188e-10,
                                         9.42951617276984e-09,
                                         2.86126442148659e-07,
                                         5.07110085734162e-06,
                                         5.79680475066438e-05,
                                         0.000457763671875,
                                         0.00262169137355084,
                                         0.0112727798210936,
                                         0.0372740188166733,
                                         0.0962230873797469,
                                         0.195213180101158,
                                         0.310148312017239,
                                         0.378731866597892,
                                         0.33862847969626,
                                         0.19503916648225,
                                         0.0448504608828411,
                                         2.63322203072471e-38,
                                         1.64939670833684e-27,
                                         1.05391610888151e-21,
                                         9.51015012166728e-18,
                                         9.00542484149015e-15,
                                         2.08208090293545e-12,
                                         1.81660484339645e-10,
                                         7.70933418795468e-09,
                                         1.86876602308397e-07,
                                         2.88243484188347e-06,
                                         3.0517578125e-05,
                                         0.000234330177130463,
                                         0.00135955200643357,
                                         0.0061484383755995,
                                         0.0221986064202114,
                                         0.065181817875107,
                                         0.157908269437273,
                                         0.319123418945609,
                                         0.542507217326621,
                                         0.780488802607072,
                                         0.954152216238001,

                                         0.958053401619509,
                                         0.797523537376705,
                                         0.572307776681373,
                                         0.35283390471932,
                                         0.185949620691069,
                                         0.0832002624868767,
                                         0.0313207537210301,
                                         0.00980516276852341,
                                         0.00251493203542002,
                                         0.000518562795261523,
                                         8.38917338153065e-05,
                                         1.03185901434408e-05,
                                         9.259248002075e-07,
                                         5.73553365709348e-08,
                                         2.27300779072467e-09,
                                         5.17239466115004e-11,
                                         5.75426522091828e-13,
                                         2.42781060558867e-15,
                                         2.51320096989525e-18,
                                         2.74314910371739e-22,
                                         4.24833430107428e-28,
                                         6.7430768285559e-39,
                                         0.0411129716694896,
                                         0.181802470278104,
                                         0.32540849837338,
                                         0.380632058647193,
                                         0.331035464427035,
                                         0.22501266708977,
                                         0.122024609644615,
                                         0.0531156467976695,
                                         0.0184992455634058,
                                         0.00510126131667517,
                                         0.00109430195573698,
                                         0.000177985603473543,
                                         2.11778573235032e-05,
                                         1.75439986951267e-06,
                                         9.44095169468949e-08,
                                         2.98716320055239e-09,
                                         4.78729425205707e-11,
                                         3.06844325426216e-13,
                                         5.24173044720467e-16,
                                         1.08550715120489e-19,
                                         4.19319376561801e-25,
                                         3.53550515055355e-35,
                                         0.000823331633113064,
                                         0.0193402833104734,
                                         0.0863445469840842,
                                         0.191622807015815,
                                         0.275017626856177,
                                         0.283985404102032,
                                         0.221855106980733,
                                         0.134275544488607,
                                         0.0635021694733057,
                                         0.0234185805861343,
                                         0.00666134553118437,
                                         0.00143270299672254,
                                         0.000226045101790774,
                                         2.50432172524764e-05,
                                         1.82994234880092e-06,
                                         8.05068857516415e-08,
                                         1.85864800724693e-09,
                                         1.80978993667995e-11,
                                         5.10186461732594e-14,
                                         2.00457700629326e-17,
                                         1.93142549478746e-22,
                                         8.65070737605439e-32,
                                         1.02069219070015e-05,
                                         0.00127364953356442,
                                         0.0141828985801512,
                                         0.0597190773243524,
                                         0.141439461746792,
                                         0.221875387478192,
                                         0.249698245795368,
                                         0.210133564483496,
                                         0.134942007131448,
                                         0.0665529979847947,
                                         0.0251021433470754,
                                         0.00713923241887295,
                                         0.00149359290116761,
                                         0.000221297095222281,
                                         2.19575104224586e-05,
                                         1.34317055119594e-06,
                                         4.46712634661471e-08,
                                         6.60788231089097e-10,
                                         3.0740238576878e-12,
                                         2.2915896788358e-15,
                                         5.50725322256075e-20,
                                         1.31031534318069e-28,
                                         8.76019834564301e-08,
                                         5.80679116336085e-05,
                                         0.00161285095803212,
                                         0.0128848135275218,
                                         0.0503593153173968,
                                         0.120011094209994,
                                         0.194563203960616,
                                         0.227663333183433,
                                         0.198520301271409,
                                         0.13094045204588,
                                         0.0654875595213479,
                                         0.0246289574980108,
                                         0.00683232533640664,
                                         0.00135381855706256,
                                         0.000182401306392339,
                                         1.55141666692004e-05,
                                         7.43290323310769e-07,
                                         1.67030425315201e-08,
                                         1.28228539168098e-10,
                                         1.81363600005015e-13,
                                         1.08715455244103e-17,
                                         1.37403940736556e-25,
                                         5.51359056985321e-10,
                                         1.9414396712714e-06,
                                         0.000134500810627063,
                                         0.00203865911067852,
                                         0.0131489315579873,
                                         0.0476030838357721,
                                         0.11117505514526,
                                         0.180880678578402,
                                         0.214172700012045,
                                         0.188921538736053,
                                         0.125287641299308,
                                         0.0623077349715086,
                                         0.0229195592331364,
                                         0.0060736063193436,
                                         0.00111115390670043,
                                         0.000131409623271824,
                                         9.06964206285504e-06,
                                         3.0962092276264e-07,
                                         3.92250568423958e-09,
                                         1.05260353490621e-11,
                                         1.57379788731059e-15,
                                         1.05663252900715e-22,
                                         2.62894291130508e-12,
                                         4.91742405070628e-08,
                                         8.49731339197895e-06,
                                         0.000244363960806845,
                                         0.00260092112216014,
                                         0.0143045713978694,
                                         0.0481260332194159,
                                         0.108872284515432,
                                         0.175044861592883,
                                         0.206497682150945,
                                         0.181586571522644,
                                         0.119416398039427,
                                         0.0582465294575599,
                                         0.0206423378870952,
                                         0.00512798279458982,
                                         0.000843241469703766,
                                         8.38393466967025e-05,
                                         4.3480154947584e-06,
                                         9.09009741436942e-08,
                                         4.62813042529935e-10,
                                         1.72596782690017e-13,
                                         6.15566296700208e-20,
                                         9.66993416774515e-15,
                                         9.60831299740692e-10,
                                         4.14127518257364e-07,
                                         2.259568018112e-05,
                                         0.00039688036933838,
                                         0.00331596800188837,
                                         0.0160712071609108,
                                         0.0505519849686399,
                                         0.110364739209514,
                                         0.174118376112708,
                                         0.203027536273576,
                                         0.176555678697153,
                                         0.114190392459362,
                                         0.054121128879388,
                                         0.018256378072819,
                                         0.00417419239608131,
                                         0.000597862540162801,
                                         4.71028933885903e-05,
                                         1.62505929822042e-06,
                                         1.56979186540335e-08,
                                         1.46019935986641e-11,
                                         2.7664409855254e-17,
                                         2.7664409855254e-17,
                                         1.46019935986641e-11,
                                         1.56979186540333e-08,
                                         1.62505929822043e-06,
                                         4.71028933885905e-05,
                                         0.000597862540162801,
                                         0.00417419239608131,
                                         0.018256378072819,
                                         0.054121128879388,
                                         0.114190392459362,
                                         0.176555678697153,
                                         0.203027536273576,
                                         0.174118376112708,
                                         0.110364739209514,
                                         0.0505519849686399,
                                         0.0160712071609108,
                                         0.00331596800188837,
                                         0.000396880369338379,
                                         2.25956801811199e-05,
                                         4.14127518257368e-07,
                                         9.60831299740692e-10,
                                         9.66993416774515e-15,
                                         6.15566296700208e-20,
                                         1.72596782690017e-13,
                                         4.62813042529929e-10,
                                         9.09009741436949e-08,
                                         4.34801549475842e-06,
                                         8.38393466967025e-05,
                                         0.000843241469703767,
                                         0.00512798279458982,
                                         0.0206423378870952,
                                         0.0582465294575598,
                                         0.119416398039427,
                                         0.181586571522644,
                                         0.206497682150946,
                                         0.175044861592883,
                                         0.108872284515432,
                                         0.0481260332194159,
                                         0.0143045713978694,
                                         0.00260092112216014,
                                         0.000244363960806843,
                                         8.49731339197903e-06,
                                         4.91742405070628e-08,
                                         2.62894291130508e-12,
                                         1.05663252900715e-22,
                                         1.57379788731059e-15,
                                         1.05260353490619e-11,
                                         3.92250568423961e-09,
                                         3.09620922762642e-07,
                                         9.06964206285504e-06,
                                         0.000131409623271824,
                                         0.00111115390670043,
                                         0.0060736063193436,
                                         0.0229195592331364,
                                         0.0623077349715086,
                                         0.125287641299308,
                                         0.188921538736053,
                                         0.214172700012045,
                                         0.180880678578402,
                                         0.11117505514526,
                                         0.0476030838357721,
                                         0.0131489315579872,
                                         0.00203865911067851,
                                         0.000134500810627064,
                                         1.9414396712714e-06,
                                         5.51359056985321e-10,
                                         1.37403940736556e-25,
                                         1.08715455244103e-17,
                                         1.81363600005012e-13,
                                         1.28228539168099e-10,
                                         1.67030425315202e-08,
                                         7.43290323310769e-07,
                                         1.55141666692004e-05,
                                         0.000182401306392339,
                                         0.00135381855706256,
                                         0.00683232533640663,
                                         0.0246289574980108,
                                         0.0654875595213479,
                                         0.13094045204588,
                                         0.198520301271409,
                                         0.227663333183433,
                                         0.194563203960616,
                                         0.120011094209994,
                                         0.0503593153173967,
                                         0.0128848135275217,
                                         0.00161285095803213,
                                         5.80679116336085e-05,
                                         8.76019834564301e-08,
                                         1.31031534318069e-28,
                                         5.50725322256075e-20,
                                         2.29158967883576e-15,
                                         3.07402385768783e-12,
                                         6.60788231089101e-10,
                                         4.46712634661471e-08,
                                         1.34317055119594e-06,
                                         2.19575104224586e-05,
                                         0.000221297095222281,
                                         0.00149359290116761,
                                         0.00713923241887295,
                                         0.0251021433470754,
                                         0.0665529979847947,
                                         0.134942007131448,
                                         0.210133564483496,
                                         0.249698245795368,
                                         0.221875387478192,
                                         0.141439461746792,
                                         0.0597190773243523,
                                         0.0141828985801512,
                                         0.00127364953356442,
                                         1.02069219070015e-05,
                                         8.65070737605439e-32,
                                         1.93142549478746e-22,
                                         2.00457700629322e-17,
                                         5.10186461732599e-14,
                                         1.80978993667997e-11,
                                         1.85864800724693e-09,
                                         8.05068857516415e-08,
                                         1.82994234880092e-06,
                                         2.50432172524764e-05,
                                         0.000226045101790774,
                                         0.00143270299672254,
                                         0.00666134553118437,
                                         0.0234185805861343,
                                         0.0635021694733057,
                                         0.134275544488607,
                                         0.221855106980733,
                                         0.283985404102032,
                                         0.275017626856177,
                                         0.191622807015814,
                                         0.0863445469840845,
                                         0.0193402833104734,
                                         0.000823331633113064,
                                         3.53550515055355e-35,
                                         4.19319376561801e-25,
                                         1.08550715120487e-19,
                                         5.24173044720473e-16,
                                         3.06844325426218e-13,
                                         4.78729425205707e-11,
                                         2.98716320055239e-09,
                                         9.44095169468949e-08,
                                         1.75439986951267e-06,
                                         2.11778573235031e-05,
                                         0.000177985603473543,
                                         0.00109430195573698,
                                         0.00510126131667517,
                                         0.0184992455634058,
                                         0.0531156467976695,
                                         0.122024609644615,
                                         0.22501266708977,
                                         0.331035464427035,
                                         0.380632058647193,
                                         0.32540849837338,
                                         0.181802470278104,
                                         0.0411129716694896,
                                         6.7430768285559e-39,
                                         4.24833430107428e-28,
                                         2.74314910371733e-22,
                                         2.51320096989529e-18,
                                         2.42781060558869e-15,
                                         5.75426522091828e-13,
                                         5.17239466115004e-11,
                                         2.27300779072467e-09,
                                         5.73553365709348e-08,
                                         9.25924800207498e-07,
                                         1.03185901434408e-05,
                                         8.38917338153065e-05,
                                         0.000518562795261523,
                                         0.00251493203542002,
                                         0.00980516276852341,
                                         0.0313207537210301,
                                         0.0832002624868767,
                                         0.185949620691069,
                                         0.35283390471932,
                                         0.572307776681373,
                                         0.797523537376705,
                                         0.958053401619509,

                                         0.961480132812043,
                                         0.812728534770014,
                                         0.599695008771579,
                                         0.385171920437822,
                                         0.214439791855134,
                                         0.102897487847133,
                                         0.0422382881904282,
                                         0.0146910623944408,
                                         0.00427742861358459,
                                         0.00102672801164893,
                                         0.000199285274844144,
                                         3.0517578125e-05,
                                         3.57195952349942e-06,
                                         3.06560532765748e-07,
                                         1.82507997247992e-08,
                                         6.98418304757437e-10,
                                         1.54165985199662e-11,
                                         1.67105501662778e-13,
                                         6.89916236995542e-16,
                                         7.0182581063587e-19,
                                         7.55931999920574e-23,
                                         1.16003682976217e-28,
                                         1.83190801774679e-39,
                                         0.0378177596252386,
                                         0.169696096238118,
                                         0.311930982422189,
                                         0.379417646488172,
                                         0.347719757973831,
                                         0.252659188037818,
                                         0.148804609672243,
                                         0.0716058695447042,
                                         0.0281371844758549,
                                         0.00896500890482858,
                                         0.00228627547108608,
                                         0.000457763671875,
                                         7.00544455313521e-05,
                                         7.89956988995228e-06,
                                         6.24261497517488e-07,
                                         3.22405700191407e-08,
                                         9.8460266011426e-10,
                                         1.53123886113781e-11,
                                         9.5731506320488e-14,
                                         1.60305693486959e-16,
                                         3.26992031556653e-20,
                                         1.25005163414019e-25,
                                         1.04792620153288e-35,
                                         0.000694157563937473,
                                         0.0165350295886303,
                                         0.0757169955395927,
                                         0.174416357959655,
                                         0.26312380529618,
                                         0.289515754270805,
                                         0.244643252458772,
                                         0.162873647977762,
                                         0.0863744483309061,
                                         0.0365302648323939,
                                         0.012240204814077,
                                         0.003204345703125,
                                         0.00064116772404484,
                                         9.49942745255414e-05,
                                         9.96455666642284e-06,
                                         6.94538925698293e-07,
                                         2.93454131244311e-08,
                                         6.54789817837429e-10,
                                         6.19897949699865e-12,
                                         1.70873745616321e-14,
                                         6.60082759639289e-18,
                                         6.28623927302861e-23,
                                         2.79746406207625e-32,
                                         7.8875909707454e-06,
                                         0.000997383452579351,
                                         0.0113776426709646,
                                         0.0496341923033739,
                                         0.123257958831234,
                                         0.205368279110211,
                                         0.248985557500897,
                                         0.22933856032257,
                                         0.164139835095163,
                                         0.0921465275111847,
                                         0.0405669979761359,
                                         0.013885498046875,
                                         0.00363271779889242,
                                         0.000707156393976604,
                                         9.84630921337968e-05,
                                         9.26220796212855e-06,
                                         5.41431486951624e-07,
                                         1.73334475188864e-08,
                                         2.48490371245439e-10,
                                         1.12752398536031e-12,
                                         8.24866502896636e-16,
                                         1.95694184807213e-20,
                                         4.62298400007692e-29,
                                         6.2048295991606e-08,
                                         4.16503349185608e-05,
                                         0.00118361473885618,
                                         0.00977853650789558,
                                         0.0399732038931458,
                                         0.10085412916163,
                                         0.17543418677816,
                                         0.223564322186017,
                                         0.215944355225072,
                                         0.160917873149305,
                                         0.093079965372227,
                                         0.041656494140625,
                                         0.0142492113641849,
                                         0.00364445566880158,
                                         0.000673578344757413,
                                         8.5512897442904e-05,
                                         6.91585607073791e-06,
                                         3.17663370437356e-07,
                                         6.89601324620421e-09,
                                         5.15080841021227e-11,
                                         7.13621790249765e-14,
                                         4.21758739436127e-18,
                                         5.2890713027247e-26,
                                         3.57945381525274e-10,
                                         1.27548766643776e-06,
                                         9.0296281318684e-05,
                                         0.00141275934967008,
                                         0.00950658142266706,
                                         0.0363208020061083,
                                         0.0906474780287712,
                                         0.15981934217724,
                                         0.208339273559808,
                                         0.20607771020149,
                                         0.156617750462593,
                                         0.091644287109375,
                                         0.0409874992252234,
                                         0.0137737214930981,
                                         0.00337912446422446,
                                         0.000578962109601935,
                                         6.47813225720031e-05,
                                         4.2692418223375e-06,
                                         1.40342120833697e-07,
                                         1.72554555441874e-09,
                                         4.527453137399e-12,
                                         6.66579090951147e-16,
                                         4.43749502764514e-23,
                                         1.56433490586555e-12,
                                         2.95910310535478e-08,
                                         5.21861711718787e-06,
                                         0.000154628159837672,
                                         0.00171279684554972,
                                         0.00990930610838075,
                                         0.0354832419266177,
                                         0.0865530246244168,
                                         0.152274262134983,
                                         0.1999326973196,
                                         0.199641957434218,
                                         0.152740478515625,
                                         0.089317813530936,
                                         0.0394362845361982,
                                         0.0128424040476941,
                                         0.00296957795857398,
                                         0.000459705545779166,
                                         4.34670773248005e-05,
                                         2.16373491079784e-06,
                                         4.37928824047942e-08,
                                         2.17603508624113e-10,
                                         7.98114776807937e-14,
                                         2.82047598192842e-20,
                                         5.27397939529243e-15,
                                         5.29589848652121e-10,
                                         2.32668035838155e-07,
                                         1.3055836562478e-05,
                                         0.000238058151204337,
                                         0.00208557964220622,
                                         0.010714861916599,
                                         0.036160204601487,
                                         0.0858573701634558,
                                         0.149634711279823,
                                         0.196317211511458,
                                         0.196380615234375,
                                         0.150148315635024,
                                         0.0871036602927034,
                                         0.0376516653833985,
                                         0.0117499235198661,
                                         0.00251654902617132,
                                         0.000341401829473144,
                                         2.57345032232918e-05,
                                         8.57385930562969e-07,
                                         8.06814218735442e-09,
                                         7.37182023917589e-12,
                                         1.3829378145427e-17,
                                         1.38293781454293e-17,
                                         7.37182023917589e-12,
                                         8.06814218735431e-09,
                                         8.57385930562969e-07,
                                         2.57345032232918e-05,
                                         0.000341401829473143,
                                         0.00251654902617133,
                                         0.0117499235198661,
                                         0.0376516653833985,
                                         0.0871036602927034,
                                         0.150148315635024,
                                         0.196380615234375,
                                         0.196317211511458,
                                         0.149634711279823,
                                         0.0858573701634558,
                                         0.0361602046014871,
                                         0.010714861916599,
                                         0.00208557964220623,
                                         0.000238058151204337,
                                         1.3055836562478e-05,
                                         2.32668035838158e-07,
                                         5.29589848652121e-10,
                                         5.27397939529165e-15,
                                         2.82047598192896e-20,
                                         7.98114776807937e-14,
                                         2.1760350862411e-10,
                                         4.37928824047942e-08,
                                         2.16373491079784e-06,
                                         4.34670773248003e-05,
                                         0.000459705545779167,
                                         0.00296957795857397,
                                         0.0128424040476941,
                                         0.0394362845361982,
                                         0.089317813530936,
                                         0.152740478515625,
                                         0.199641957434218,
                                         0.1999326973196,
                                         0.152274262134983,
                                         0.0865530246244169,
                                         0.0354832419266177,
                                         0.00990930610838077,
                                         0.00171279684554972,
                                         0.000154628159837672,
                                         5.21861711718793e-06,
                                         2.95910310535478e-08,
                                         1.56433490586535e-12,
                                         4.43749502764608e-23,
                                         6.66579090951147e-16,
                                         4.52745313739892e-12,
                                         1.72554555441874e-09,
                                         1.40342120833697e-07,
                                         4.26924182233748e-06,
                                         6.47813225720033e-05,
                                         0.000578962109601934,
                                         0.00337912446422446,
                                         0.0137737214930981,
                                         0.0409874992252234,
                                         0.091644287109375,
                                         0.156617750462593,
                                         0.20607771020149,
                                         0.208339273559808,
                                         0.15981934217724,
                                         0.0906474780287711,
                                         0.0363208020061084,
                                         0.00950658142266706,
                                         0.00141275934967008,
                                         9.02962813186847e-05,
                                         1.27548766643776e-06,
                                         3.57945381525236e-10,
                                         5.28907130272593e-26,
                                         4.21758739436127e-18,
                                         7.13621790249752e-14,
                                         5.15080841021227e-11,
                                         6.89601324620421e-09,
                                         3.17663370437355e-07,
                                         6.91585607073793e-06,
                                         8.55128974429038e-05,
                                         0.000673578344757413,
                                         0.00364445566880158,
                                         0.0142492113641849,
                                         0.041656494140625,
                                         0.093079965372227,
                                         0.160917873149305,
                                         0.215944355225072,
                                         0.223564322186017,
                                         0.17543418677816,
                                         0.10085412916163,
                                         0.0399732038931458,
                                         0.00977853650789559,
                                         0.00118361473885618,
                                         4.16503349185608e-05,
                                         6.20482959916008e-08,
                                         4.6229840000781e-29,
                                         1.95694184807213e-20,
                                         8.24866502896619e-16,
                                         1.12752398536031e-12,
                                         2.48490371245439e-10,
                                         1.73334475188863e-08,
                                         5.41431486951626e-07,
                                         9.26220796212852e-06,
                                         9.84630921337968e-05,
                                         0.000707156393976604,
                                         0.00363271779889242,
                                         0.013885498046875,
                                         0.0405669979761359,
                                         0.0921465275111847,
                                         0.164139835095163,
                                         0.22933856032257,
                                         0.248985557500897,
                                         0.205368279110212,
                                         0.123257958831234,
                                         0.0496341923033739,
                                         0.0113776426709647,
                                         0.000997383452579351,
                                         7.88759097074489e-06,
                                         2.79746406207702e-32,
                                         6.28623927302861e-23,
                                         6.60082759639275e-18,
                                         1.70873745616321e-14,
                                         6.19897949699865e-12,
                                         6.54789817837425e-10,
                                         2.93454131244312e-08,
                                         6.94538925698291e-07,
                                         9.96455666642284e-06,
                                         9.49942745255414e-05,
                                         0.00064116772404484,
                                         0.003204345703125,
                                         0.012240204814077,
                                         0.0365302648323939,
                                         0.0863744483309061,
                                         0.162873647977762,
                                         0.244643252458771,
                                         0.289515754270805,
                                         0.26312380529618,
                                         0.174416357959655,
                                         0.0757169955395929,
                                         0.0165350295886303,
                                         0.000694157563937443,
                                         1.04792620153319e-35,
                                         1.25005163414019e-25,
                                         3.26992031556645e-20,
                                         1.60305693486959e-16,
                                         9.5731506320488e-14,
                                         1.53123886113781e-11,
                                         9.84602660114264e-10,
                                         3.22405700191406e-08,
                                         6.24261497517488e-07,
                                         7.89956988995228e-06,
                                         7.00544455313521e-05,
                                         0.000457763671875,
                                         0.00228627547108608,
                                         0.00896500890482858,
                                         0.0281371844758549,
                                         0.0716058695447042,
                                         0.148804609672242,
                                         0.252659188037818,
                                         0.347719757973831,
                                         0.379417646488172,
                                         0.31193098242219,
                                         0.169696096238118,
                                         0.0378177596252378,
                                         1.83190801774737e-39,
                                         1.16003682976217e-28,
                                         7.55931999920555e-23,
                                         7.0182581063587e-19,
                                         6.89916236995542e-16,
                                         1.67105501662777e-13,
                                         1.54165985199663e-11,
                                         6.98418304757435e-10,
                                         1.82507997247992e-08,
                                         3.06560532765748e-07,
                                         3.57195952349942e-06,
                                         3.0517578125e-05,
                                         0.000199285274844144,
                                         0.00102672801164893,
                                         0.00427742861358459,
                                         0.0146910623944408,
                                         0.0422382881904282,
                                         0.102897487847133,
                                         0.214439791855134,
                                         0.385171920437822,
                                         0.599695008771579,
                                         0.812728534770014,
                                         0.961480132812043,

                                         0.964505880061465,
                                         0.826344538991791,
                                         0.624855321665159,
                                         0.41601644779781,
                                         0.243012153093734,
                                         0.123952056266899,
                                         0.0548628499214528,
                                         0.0209051935439101,
                                         0.00678967846087357,
                                         0.00185629838230243,
                                         0.00042060712445552,
                                         7.74502467626258e-05,
                                         1.13055532119933e-05,
                                         1.26712846381173e-06,
                                         1.04593028131317e-07,
                                         6.01412898384116e-09,
                                         2.23194682551009e-10,
                                         4.79685491055688e-12,
                                         5.08203749828531e-14,
                                         2.05857060191579e-16,
                                         2.06221800964635e-19,
                                         2.19540083613573e-23,
                                         3.34195765686887e-29,
                                         5.25400179027687e-40,
                                         0.0348986393291134,
                                         0.158626255510397,
                                         0.298483060282194,
                                         0.375736482531389,
                                         0.360508163822178,
                                         0.277671050581508,
                                         0.17571742786732,
                                         0.0922374798473128,
                                         0.0402187623799823,
                                         0.014503162352723,
                                         0.0042844737389627,
                                         0.00102187709731227,
                                         0.000192796203066868,
                                         2.79886727493259e-05,
                                         3.01210656301179e-06,
                                         2.28441899038539e-07,
                                         1.1381857028135e-08,
                                         3.36978837020112e-10,
                                         5.10438607983134e-12,
                                         3.12220326765192e-14,
                                         5.13741003175986e-17,
                                         1.03408473575984e-20,
                                         3.91714411553222e-26,
                                         3.26715034324658e-36,
                                         0.000589276185551196,
                                         0.0142100551888158,
                                         0.0665375316284986,
                                         0.158366387573812,
                                         0.249579548922438,
                                         0.290278086851659,
                                         0.262638320227731,
                                         0.189918575958948,
                                         0.111177006853287,
                                         0.0528791434840996,
                                         0.0203669091760896,
                                         0.00629189268346891,
                                         0.00153430576130785,
                                         0.000288503271898551,
                                         4.04803919619029e-05,
                                         4.04935233498572e-06,
                                         2.70862691260604e-07,
                                         1.10472823133427e-08,
                                         2.39252203898362e-10,
                                         2.20985288362225e-12,
                                         5.97256196893464e-15,
                                         2.27303022510028e-18,
                                         2.14261893139551e-23,
                                         9.48101435061192e-33,
                                         6.15961181940915e-06,
                                         0.000788025941154857,
                                         0.00918200954796812,
                                         0.0413206089182954,
                                         0.106961366969036,
                                         0.187854653724314,
                                         0.243010704380301,
                                         0.242075880135334,
                                         0.190250282607382,
                                         0.119352129716833,
                                         0.0599344916038701,
                                         0.0239821439238015,
                                         0.00755874009583503,
                                         0.00184095528035876,
                                         0.000336777557791259,
                                         4.44344110970609e-05,
                                         3.99033456475662e-06,
                                         2.2419831050305e-07,
                                         6.94212510643946e-09,
                                         9.68254688116613e-11,
                                         4.29834387810502e-13,
                                         3.09298882900075e-16,
                                         7.25511637913552e-21,
                                         1.70319567548714e-29,
                                         4.45745486340022e-08,
                                         3.02541127561547e-05,
                                         0.000877218842311947,
                                         0.00746396462601457,
                                         0.0317354054231549,
                                         0.0841644755677766,
                                         0.155665321721593,
                                         0.213616478303382,
                                         0.225390081518715,
                                         0.186498391738022,
                                         0.122103378856112,
                                         0.0632839910640677,
                                         0.0257801871897846,
                                         0.00813270262008109,
                                         0.00193972755205927,
                                         0.000337561142467835,
                                         4.06975802392652e-05,
                                         3.14998420189535e-06,
                                         1.39453070819384e-07,
                                         2.9370748308011e-09,
                                         2.14161207554552e-11,
                                         2.91373901594672e-14,
                                         1.70075981505156e-18,
                                         2.11823161007742e-26,
                                         2.3654947623276e-10,
                                         8.51784541553542e-07,
                                         6.14581650211953e-05,
                                         0.000988721227114718,
                                         0.00690498251535326,
                                         0.0276526725596196,
                                         0.0731239712075732,
                                         0.138235442966482,
                                         0.19581489171656,
                                         0.213708322444564,
                                         0.182423155704533,
                                         0.122461942004124,
                                         0.0644798689927353,
                                         0.0263468058761185,
                                         0.00819293957903797,
                                         0.00188055869763963,
                                         0.000304389237125817,
                                         3.24553159950801e-05,
                                         2.0543060930251e-06,
                                         6.53343900412218e-08,
                                         7.8249556160978e-10,
                                         2.01291007386153e-12,
                                         2.92376871444274e-16,
                                         1.93189611753902e-23,
                                         9.51005647922677e-13,
                                         1.81677504496195e-08,
                                         3.26194996911347e-06,
                                         9.92211400526049e-05,
                                         0.00113817013133957,
                                         0.00688290015086574,
                                         0.026022782578,
                                         0.0677688529515331,
                                         0.128879161473527,
                                         0.185521329622141,
                                         0.206470645394519,
                                         0.179528950745398,
                                         0.122176673149938,
                                         0.0646617002069952,
                                         0.0262159040162961,
                                         0.00793683602390469,
                                         0.00172470993497762,
                                         0.000253331606346965,
                                         2.29260003787588e-05,
                                         1.10101881096474e-06,
                                         2.16595299560283e-08,
                                         1.05347483580529e-10,
                                         3.80775600905559e-14,
                                         1.33481218813587e-20,
                                         2.9494424148357e-15,
                                         2.98929156981968e-10,
                                         1.33558250619009e-07,
                                         7.68122146958393e-06,
                                         0.000144726315339537,
                                         0.00132160461771486,
                                         0.007144032427517,
                                         0.0256292903425979,
                                         0.065435798523197,
                                         0.124240153987019,
                                         0.180273705716906,
                                         0.203031565320519,
                                         0.178586309968665,
                                         0.122422709627886,
                                         0.0647121224755586,
                                         0.0258406623830044,
                                         0.00753873639584949,
                                         0.00152541717459844,
                                         0.000197372733569437,
                                         1.43134180837124e-05,
                                         4.62500165046158e-07,
                                         4.25323793508581e-09,
                                         3.82552398726914e-12,
                                         7.11462930986395e-18,
                                         7.11462930986526e-18,
                                         3.82552398726914e-12,
                                         4.25323793508587e-09,
                                         4.62500165046158e-07,
                                         1.43134180837124e-05,
                                         0.000197372733569437,
                                         0.00152541717459844,
                                         0.00753873639584951,
                                         0.0258406623830044,
                                         0.0647121224755586,
                                         0.122422709627886,
                                         0.178586309968665,
                                         0.203031565320519,
                                         0.180273705716906,
                                         0.124240153987019,
                                         0.065435798523197,
                                         0.0256292903425978,
                                         0.007144032427517,
                                         0.00132160461771486,
                                         0.000144726315339537,
                                         7.68122146958393e-06,
                                         1.33558250619007e-07,
                                         2.98929156981968e-10,
                                         2.94944241483522e-15,
                                         1.33481218813615e-20,
                                         3.80775600905559e-14,
                                         1.05347483580531e-10,
                                         2.16595299560283e-08,
                                         1.10101881096474e-06,
                                         2.29260003787589e-05,
                                         0.000253331606346965,
                                         0.00172470993497763,
                                         0.00793683602390469,
                                         0.0262159040162961,
                                         0.0646617002069952,
                                         0.122176673149938,
                                         0.179528950745399,
                                         0.206470645394519,
                                         0.185521329622141,
                                         0.128879161473527,
                                         0.067768852951533,
                                         0.026022782578,
                                         0.00688290015086572,
                                         0.00113817013133957,
                                         9.92211400526049e-05,
                                         3.26194996911343e-06,
                                         1.81677504496195e-08,
                                         9.51005647922546e-13,
                                         1.93189611753947e-23,
                                         2.92376871444274e-16,
                                         2.01291007386156e-12,
                                         7.8249556160978e-10,
                                         6.53343900412218e-08,
                                         2.05430609302511e-06,
                                         3.24553159950801e-05,
                                         0.000304389237125818,
                                         0.00188055869763963,
                                         0.00819293957903797,
                                         0.0263468058761185,
                                         0.0644798689927352,
                                         0.122461942004124,
                                         0.182423155704533,
                                         0.213708322444564,
                                         0.19581489171656,
                                         0.138235442966482,
                                         0.0731239712075732,
                                         0.0276526725596196,
                                         0.00690498251535326,
                                         0.000988721227114718,
                                         6.14581650211947e-05,
                                         8.51784541553542e-07,
                                         2.36549476232732e-10,
                                         2.11823161007796e-26,
                                         1.70075981505156e-18,
                                         2.91373901594678e-14,
                                         2.14161207554552e-11,
                                         2.9370748308011e-09,
                                         1.39453070819384e-07,
                                         3.14998420189535e-06,
                                         4.06975802392653e-05,
                                         0.000337561142467835,
                                         0.00193972755205927,
                                         0.00813270262008109,
                                         0.0257801871897846,
                                         0.0632839910640678,
                                         0.122103378856112,
                                         0.186498391738022,
                                         0.225390081518715,
                                         0.213616478303382,
                                         0.155665321721593,
                                         0.0841644755677765,
                                         0.0317354054231549,
                                         0.00746396462601457,
                                         0.00087721884231194,
                                         3.02541127561547e-05,
                                         4.4574548633998e-08,
                                         1.70319567548761e-29,
                                         7.25511637913552e-21,
                                         3.09298882900082e-16,
                                         4.29834387810502e-13,
                                         9.68254688116613e-11,
                                         6.9421251064395e-09,
                                         2.2419831050305e-07,
                                         3.99033456475664e-06,
                                         4.44344110970609e-05,
                                         0.000336777557791259,
                                         0.00184095528035876,
                                         0.00755874009583502,
                                         0.0239821439238015,
                                         0.0599344916038701,
                                         0.119352129716833,
                                         0.190250282607382,
                                         0.242075880135334,
                                         0.243010704380302,
                                         0.187854653724313,
                                         0.106961366969036,
                                         0.0413206089182954,
                                         0.00918200954796807,
                                         0.000788025941154857,
                                         6.15961181940872e-06,
                                         9.48101435061476e-33,
                                         2.14261893139551e-23,
                                         2.27303022510034e-18,
                                         5.97256196893464e-15,
                                         2.20985288362225e-12,
                                         2.39252203898363e-10,
                                         1.10472823133427e-08,
                                         2.70862691260605e-07,
                                         4.04935233498572e-06,
                                         4.04803919619029e-05,
                                         0.000288503271898551,
                                         0.00153430576130785,
                                         0.00629189268346891,
                                         0.0203669091760896,
                                         0.0528791434840996,
                                         0.111177006853287,
                                         0.189918575958948,
                                         0.262638320227731,
                                         0.290278086851658,
                                         0.249579548922438,
                                         0.158366387573812,
                                         0.0665375316284984,
                                         0.0142100551888158,
                                         0.000589276185551169,
                                         3.26715034324763e-36,
                                         3.91714411553222e-26,
                                         1.03408473575986e-20,
                                         5.13741003175986e-17,
                                         3.12220326765192e-14,
                                         5.10438607983137e-12,
                                         3.36978837020112e-10,
                                         1.1381857028135e-08,
                                         2.28441899038539e-07,
                                         3.01210656301179e-06,
                                         2.79886727493259e-05,
                                         0.000192796203066867,
                                         0.00102187709731227,
                                         0.0042844737389627,
                                         0.014503162352723,
                                         0.0402187623799823,
                                         0.0922374798473128,
                                         0.17571742786732,
                                         0.277671050581508,
                                         0.360508163822178,
                                         0.375736482531389,
                                         0.298483060282194,
                                         0.158626255510397,
                                         0.0348986393291126,
                                         5.25400179027868e-40,
                                         3.34195765686887e-29,
                                         2.19540083613579e-23,
                                         2.06221800964635e-19,
                                         2.05857060191579e-16,
                                         5.08203749828535e-14,
                                         4.79685491055688e-12,
                                         2.2319468255101e-10,
                                         6.01412898384116e-09,
                                         1.04593028131317e-07,
                                         1.26712846381173e-06,
                                         1.13055532119933e-05,
                                         7.74502467626258e-05,
                                         0.00042060712445552,
                                         0.00185629838230243,
                                         0.00678967846087357,
                                         0.0209051935439101,
                                         0.0548628499214528,
                                         0.123952056266899,
                                         0.243012153093734,
                                         0.41601644779781,
                                         0.624855321665159,
                                         0.826344538991791,
                                         0.964505880061465,

                                         1.52587890625e-05,
                                         0.000244140625,
                                         0.0018310546875,
                                         0.008544921875,
                                         0.02777099609375,
                                         0.066650390625,
                                         0.1221923828125,
                                         0.174560546875,
                                         0.196380615234375,
                                         0.174560546875,
                                         0.1221923828125,
                                         0.066650390625,
                                         0.02777099609375,
                                         0.008544921875,
                                         0.0018310546875,
                                         0.000244140625,
                                         1.52587890625e-05,

                                         0.0224063119030947,
                                         1.58201142898839e-11,
                                         0.0960600508767053,
                                         9.44663524982291e-10,
                                         0.193044097929793,
                                         2.64414920346827e-08,
                                         0.241388047338119,
                                         4.60511294606363e-07,
                                         0.210209130127017,
                                         5.58561753402337e-06,
                                         0.135180879982039,
                                         5.00299402286948e-05,
                                         0.0664062806427458,
                                         0.000342309511194059,
                                         0.0254192989579689,
                                         0.00182502355384328,
                                         0.00766246570644717,
                                         0.00766246570644717,
                                         0.00182502355384328,
                                         0.0254192989579689,
                                         0.000342309511194059,
                                         0.0664062806427458,
                                         5.00299402286948e-05,
                                         0.135180879982039,
                                         5.58561753402337e-06,
                                         0.210209130127017,
                                         4.60511294606363e-07,
                                         0.241388047338119,
                                         2.64414920346827e-08,
                                         0.193044097929793,
                                         9.44663524982291e-10,
                                         0.0960600508767053,
                                         1.58201142898839e-11,
                                         0.0224063119030947,

                                         0.147609619999999,
                                         1.52587890625e-05,
                                         6.77462620661197e-16,
                                         0.299982079999914,
                                         0.000244140625,
                                         8.5338430882298e-14,
                                         0.285770399994961,
                                         0.0018310546875,
                                         5.03901033845853e-12,
                                         0.169388799814864,
                                         0.008544921875,
                                         1.85136207554904e-10,
                                         0.0699243952628839,
                                         0.02777099609375,
                                         4.73711640629421e-09,
                                         0.0213157504914276,
                                         0.066650390625,
                                         8.95085725819204e-08,
                                         0.00496366805091436,
                                         0.1221923828125,
                                         1.29194908568047e-06,
                                         0.000900669294806138,
                                         0.174560546875,
                                         1.45307051938717e-05,
                                         0.000128700000000002,
                                         0.196380615234375,
                                         0.000128700000000002,
                                         1.45307051938717e-05,
                                         0.174560546875,
                                         0.000900669294806138,
                                         1.29194908568047e-06,
                                         0.1221923828125,
                                         0.00496366805091436,
                                         8.95085725819204e-08,
                                         0.066650390625,
                                         0.0213157504914276,
                                         4.73711640629421e-09,
                                         0.02777099609375,
                                         0.0699243952628839,
                                         1.85136207554904e-10,
                                         0.008544921875,
                                         0.169388799814864,
                                         5.03901033845853e-12,
                                         0.0018310546875,
                                         0.285770399994961,
                                         8.5338430882298e-14,
                                         0.000244140625,
                                         0.299982079999914,
                                         6.77462620661197e-16,
                                         1.52587890625e-05,
                                         0.147609619999999,

                                         0.316206642693938,
                                         0.00164853640429671,
                                         1.97889437932342e-08,
                                         2.91701870284115e-19,
                                         0.377486553172095,
                                         0.0129920079975506,
                                         6.42813284018983e-07,
                                         6.25530776752624e-17,
                                         0.211238591595535,
                                         0.0479949227714814,
                                         9.78787359186296e-06,
                                         6.28781031239022e-15,
                                         0.0735513347264596,
                                         0.110321417330293,
                                         9.27336551961714e-05,
                                         3.93274419396389e-13,
                                         0.0178354648619387,
                                         0.176604167668509,
                                         0.000611875388254601,
                                         1.71304410820123e-11,
                                         0.00319378876623027,
                                         0.208770890903881,
                                         0.00298137408325557,
                                         5.51022424318497e-10,
                                         0.000436875950314098,
                                         0.188525080042877,
                                         0.0110968611538215,
                                         1.35394241599016e-08,
                                         4.6566179918108e-05,
                                         0.132656595813894,
                                         0.0321843214715643,
                                         2.59233808259116e-07,
                                         3.90871178558284e-06,
                                         0.073508764839305,
                                         0.073508764839305,
                                         3.90871178558286e-06,
                                         2.59233808259114e-07,
                                         0.0321843214715643,
                                         0.132656595813894,
                                         4.65661799181083e-05,
                                         1.35394241599015e-08,
                                         0.0110968611538215,
                                         0.188525080042877,
                                         0.0004368759503141,
                                         5.51022424318492e-10,
                                         0.00298137408325557,
                                         0.208770890903881,
                                         0.00319378876623029,
                                         1.71304410820121e-11,
                                         0.000611875388254601,
                                         0.176604167668509,
                                         0.0178354648619388,
                                         3.93274419396385e-13,
                                         9.27336551961714e-05,
                                         0.110321417330293,
                                         0.0735513347264598,
                                         6.28781031239015e-15,
                                         9.78787359186296e-06,
                                         0.0479949227714814,
                                         0.211238591595535,
                                         6.25530776752616e-17,
                                         6.42813284018983e-07,
                                         0.0129920079975506,
                                         0.377486553172096,
                                         2.91701870284112e-19,
                                         1.97889437932342e-08,
                                         0.00164853640429671,
                                         0.316206642693938,

                                         0.463598523719489,
                                         0.0150293938317682,
                                         1.52587890625e-05,
                                         6.46739566646081e-11,
                                         5.498678623474e-22,
                                         0.365085256866114,
                                         0.0721395112225448,
                                         0.000244140625,
                                         3.44935319519101e-09,
                                         1.78750000593817e-19,
                                         0.134768323871747,
                                         0.162310347196133,
                                         0.0018310546875,
                                         8.62357175813947e-08,
                                         2.7237989427946e-17,
                                         0.0309547197400161,
                                         0.227229511907045,
                                         0.008544921875,
                                         1.3414738606008e-06,
                                         2.58255488262846e-15,
                                         0.00495156524115224,
                                         0.221543924402178,
                                         0.02777099609375,
                                         1.45329516164497e-05,
                                         1.70529945106519e-13,
                                         0.000584905918202147,
                                         0.159508133856826,
                                         0.066650390625,
                                         0.000116266158000172,
                                         8.31533261252742e-12,
                                         5.27788251647534e-05,
                                         0.0877275532212843,
                                         0.1221923828125,
                                         0.000710530963538355,
                                         3.09733936461476e-10,
                                         3.71102557271714e-06,
                                         0.0375966997985795,
                                         0.174560546875,
                                         0.00338355484483874,
                                         8.98997499437862e-09,
                                         2.05484343810557e-07,
                                         0.0126886084220429,
                                         0.196380615234375,
                                         0.0126886084220429,
                                         2.05484343810559e-07,
                                         8.98997499437852e-09,
                                         0.00338355484483874,
                                         0.174560546875,
                                         0.0375966997985795,
                                         3.71102557271717e-06,
                                         3.09733936461473e-10,
                                         0.000710530963538355,
                                         0.1221923828125,
                                         0.0877275532212844,
                                         5.27788251647537e-05,
                                         8.31533261252731e-12,
                                         0.000116266158000172,
                                         0.066650390625,
                                         0.159508133856826,
                                         0.000584905918202151,
                                         1.70529945106516e-13,
                                         1.45329516164497e-05,
                                         0.02777099609375,
                                         0.221543924402178,
                                         0.00495156524115226,
                                         2.58255488262842e-15,
                                         1.3414738606008e-06,
                                         0.008544921875,
                                         0.227229511907045,
                                         0.0309547197400162,
                                         2.72379894279455e-17,
                                         8.62357175813947e-08,
                                         0.0018310546875,
                                         0.162310347196134,
                                         0.134768323871748,
                                         1.78750000593814e-19,
                                         3.44935319519101e-09,
                                         0.000244140625,
                                         0.0721395112225448,
                                         0.365085256866114,
                                         5.4986786234739e-22,
                                         6.46739566646081e-11,
                                         1.52587890625e-05,
                                         0.0150293938317682,
                                         0.463598523719489,

                                         0.577194908978987,
                                         0.0513227703807899,
                                         0.000468300817386027,
                                         1.94603626942925e-07,
                                         4.59644405934548e-13,
                                         2.85442116101245e-24,
                                         0.322722835760882,
                                         0.167470017997392,
                                         0.0046058418794792,
                                         5.06531883351025e-06,
                                         3.60607677343056e-11,
                                         1.3069254418686e-21,
                                         0.0845820453433721,
                                         0.256156466223083,
                                         0.0212341271536772,
                                         6.18021984807321e-05,
                                         1.32613958556684e-09,
                                         2.80494825108911e-19,
                                         0.0137934271798295,
                                         0.243791801890672,
                                         0.0609123451755069,
                                         0.000469187658062044,
                                         3.03451317227266e-08,
                                         3.74579850283471e-17,
                                         0.00156654462183567,
                                         0.161588128584064,
                                         0.121689416143623,
                                         0.00248065183638962,
                                         4.83576730943195e-07,
                                         3.48369840893321e-15,
                                         0.000131383623150556,
                                         0.0790911127790571,
                                         0.17952633916769,
                                         0.00968529672106115,
                                         5.69075179131207e-06,
                                         2.39256953981718e-13,
                                         8.41724351068838e-06,
                                         0.0295716878778799,
                                         0.202317633145938,
                                         0.0288861883886366,
                                         5.11568842501771e-05,
                                         1.2552169107837e-11,
                                         4.20202927741205e-07,
                                         0.0086155908554986,
                                         0.177664157826643,
                                         0.0671317661974838,
                                         0.000358343105083444,
                                         5.13137222434601e-10,
                                         1.65195730589508e-08,
                                         0.00197671738591693,
                                         0.122861685767483,
                                         0.122861685767483,
                                         0.00197671738591692,
                                         1.65195730589508e-08,
                                         5.13137222434601e-10,
                                         0.000358343105083445,
                                         0.0671317661974838,
                                         0.177664157826643,
                                         0.00861559085549858,
                                         4.20202927741205e-07,
                                         1.2552169107837e-11,
                                         5.11568842501773e-05,
                                         0.0288861883886366,
                                         0.202317633145938,
                                         0.0295716878778798,
                                         8.41724351068838e-06,
                                         2.39256953981718e-13,
                                         5.6907517913121e-06,
                                         0.00968529672106115,
                                         0.17952633916769,
                                         0.079091112779057,
                                         0.000131383623150556,
                                         3.48369840893321e-15,
                                         4.83576730943197e-07,
                                         0.00248065183638962,
                                         0.121689416143623,
                                         0.161588128584064,
                                         0.00156654462183567,
                                         3.74579850283471e-17,
                                         3.03451317227267e-08,
                                         0.000469187658062044,
                                         0.0609123451755069,
                                         0.243791801890672,
                                         0.0137934271798295,
                                         2.80494825108911e-19,
                                         1.32613958556684e-09,
                                         6.18021984807321e-05,
                                         0.0212341271536772,
                                         0.256156466223083,
                                         0.0845820453433721,
                                         1.3069254418686e-21,
                                         3.60607677343058e-11,
                                         5.06531883351025e-06,
                                         0.0046058418794792,
                                         0.167470017997392,
                                         0.322722835760882,
                                         2.85442116101245e-24,
                                         4.5964440593455e-13,
                                         1.94603626942925e-07,
                                         0.000468300817386027,
                                         0.0513227703807899,
                                         0.577194908978987,

                                         0.662055230897051,
                                         0.109249677035572,
                                         0.00355238369205369,
                                         1.52587890625e-05,
                                         3.68056420562075e-09,
                                         6.05411241477398e-15,
                                         3.08975706570544e-26,
                                         0.27658497618431,
                                         0.25942811457438,
                                         0.0240216042881662,
                                         0.000244140625,
                                         1.39338850262363e-07,
                                         6.52669876381256e-13,
                                         1.8933457741544e-23,
                                         0.0541631895937304,
                                         0.288772077772811,
                                         0.076142201608355,
                                         0.0018310546875,
                                         2.47269942541869e-06,
                                         3.29820721192756e-11,
                                         5.43847098697265e-21,
                                         0.00659971823554475,
                                         0.200004088485595,
                                         0.15017386876146,
                                         0.008544921875,
                                         2.7303350905555e-05,
                                         1.03706940703147e-09,
                                         9.72006602441214e-19,
                                         0.000560045274915736,
                                         0.0964715218275859,
                                         0.206271814192516,
                                         0.02777099609375,
                                         0.000209960284969014,
                                         2.27098512214535e-08,
                                         1.2098685674655e-16,
                                         3.50952840025497e-05,
                                         0.0343626988613926,
                                         0.209224861073645,
                                         0.066650390625,
                                         0.00119230190293908,
                                         3.67238864441275e-07,
                                         1.11207749870622e-14,
                                         1.67998180462761e-06,
                                         0.00934987050335014,
                                         0.162112641034529,
                                         0.1221923828125,
                                         0.00517208333378034,
                                         4.53641915575585e-06,
                                         7.80840096110696e-13,
                                         6.2664404954744e-08,
                                         0.00198236878622611,
                                         0.0978770788307965,
                                         0.174560546875,
                                         0.0174825715989004,
                                         4.36654802134324e-05,
                                         4.27218139306309e-11,
                                         1.84072039622301e-09,
                                         0.000330989234291154,
                                         0.0465367103281444,
                                         0.196380615234375,
                                         0.0465367103281444,
                                         0.000330989234291154,
                                         1.84072039622304e-09,
                                         4.27218139306301e-11,
                                         4.36654802134324e-05,
                                         0.0174825715989004,
                                         0.174560546875,
                                         0.0978770788307965,
                                         0.00198236878622611,
                                         6.2664404954745e-08,
                                         7.80840096110679e-13,
                                         4.53641915575585e-06,
                                         0.00517208333378034,
                                         0.1221923828125,
                                         0.162112641034529,
                                         0.00934987050335014,
                                         1.67998180462763e-06,
                                         1.11207749870619e-14,
                                         3.67238864441275e-07,
                                         0.00119230190293908,
                                         0.066650390625,
                                         0.209224861073645,
                                         0.0343626988613926,
                                         3.50952840025501e-05,
                                         1.20986856746547e-16,
                                         2.27098512214535e-08,
                                         0.000209960284969014,
                                         0.02777099609375,
                                         0.206271814192516,
                                         0.0964715218275859,
                                         0.000560045274915741,
                                         9.72006602441187e-19,
                                         1.03706940703147e-09,
                                         2.7303350905555e-05,
                                         0.008544921875,
                                         0.15017386876146,
                                         0.200004088485595,
                                         0.0065997182355448,
                                         5.43847098697249e-21,
                                         3.29820721192756e-11,
                                         2.47269942541869e-06,
                                         0.0018310546875,
                                         0.076142201608355,
                                         0.288772077772811,
                                         0.0541631895937306,
                                         1.89334577415434e-23,
                                         6.52669876381256e-13,
                                         1.39338850262363e-07,
                                         0.000244140625,
                                         0.0240216042881662,
                                         0.25942811457438,
                                         0.276584976184311,
                                         3.08975706570533e-26,
                                         6.05411241477398e-15,
                                         3.68056420562075e-09,
                                         1.52587890625e-05,
                                         0.00355238369205369,
                                         0.109249677035572,
                                         0.662055230897051,

                                         0.725512253887608,
                                         0.179886886440954,
                                         0.0131299504724956,
                                         0.000225854381397848,
                                         5.96177097578139e-07,
                                         1.00652577662252e-10,
                                         1.30275821835619e-16,
                                         5.83369035164687e-28,
                                         0.235150495681478,
                                         0.325732436126159,
                                         0.0653383531478785,
                                         0.00249341848977669,
                                         1.38244734259664e-05,
                                         5.1779728714725e-09,
                                         1.84179921879938e-14,
                                         4.60767874951499e-25,
                                         0.0357263234461321,
                                         0.276480044393868,
                                         0.152410385170759,
                                         0.0129033688515668,
                                         0.000150266852030231,
                                         1.24863619742275e-07,
                                         1.2205681357108e-12,
                                         1.7059342622584e-22,
                                         0.00337735156295475,
                                         0.146019914995237,
                                         0.221210865487335,
                                         0.0415486164009839,
                                         0.00101630320768927,
                                         1.87351678530037e-06,
                                         5.03300415808059e-11,
                                         3.92995799837132e-20,
                                         0.000222351874663165,
                                         0.0537077516529819,
                                         0.223601611535104,
                                         0.0931722535894643,
                                         0.00478696204739582,
                                         1.95774370426025e-05,
                                         1.44533703265349e-09,
                                         6.30507280856831e-18,
                                         1.08101868567851e-05,
                                         0.014587797752445,
                                         0.166905744498271,
                                         0.154292392998722,
                                         0.0166503954657516,
                                         0.000151071299296054,
                                         3.06505909554184e-08,
                                         7.46999281149288e-16,
                                         4.01472497331094e-07,
                                         0.00302672307778327,
                                         0.0951695098197606,
                                         0.195178790583461,
                                         0.0442404274741919,
                                         0.000890508944169138,
                                         4.96522310079962e-07,
                                         6.76052549357967e-14,
                                         1.16182020068305e-08,
                                         0.000489346141102289,
                                         0.0422848608507417,
                                         0.19238945111488,
                                         0.0915955978479497,
                                         0.00409029972012817,
                                         6.26757020725253e-06,
                                         4.76761580678998e-12,
                                         2.6477233753364e-10,
                                         6.23031794553284e-05,
                                         0.0147952579579886,
                                         0.149341480044216,
                                         0.149341480044216,
                                         0.0147952579579886,
                                         6.23031794553284e-05,
                                         2.6477233753364e-10,
                                         4.76761580678998e-12,
                                         6.26757020725253e-06,
                                         0.00409029972012817,
                                         0.0915955978479499,
                                         0.19238945111488,
                                         0.0422848608507417,
                                         0.000489346141102289,
                                         1.16182020068305e-08,
                                         6.76052549357967e-14,
                                         4.96522310079962e-07,
                                         0.000890508944169138,
                                         0.044240427474192,
                                         0.195178790583461,
                                         0.0951695098197606,
                                         0.00302672307778327,
                                         4.01472497331094e-07,
                                         7.46999281149288e-16,
                                         3.06505909554184e-08,
                                         0.000151071299296054,
                                         0.0166503954657516,
                                         0.154292392998721,
                                         0.166905744498271,
                                         0.014587797752445,
                                         1.08101868567851e-05,
                                         6.30507280856831e-18,
                                         1.44533703265349e-09,
                                         1.95774370426025e-05,
                                         0.00478696204739583,
                                         0.0931722535894642,
                                         0.223601611535104,
                                         0.0537077516529819,
                                         0.000222351874663165,
                                         3.92995799837132e-20,
                                         5.03300415808059e-11,
                                         1.87351678530037e-06,
                                         0.00101630320768927,
                                         0.0415486164009839,
                                         0.221210865487335,
                                         0.146019914995237,
                                         0.00337735156295475,
                                         1.7059342622584e-22,
                                         1.2205681357108e-12,
                                         1.24863619742275e-07,
                                         0.000150266852030232,
                                         0.0129033688515668,
                                         0.152410385170759,
                                         0.276480044393868,
                                         0.0357263234461321,
                                         4.60767874951499e-25,
                                         1.84179921879938e-14,
                                         5.1779728714725e-09,
                                         1.38244734259665e-05,
                                         0.00249341848977669,
                                         0.0653383531478785,
                                         0.325732436126159,
                                         0.235150495681478,
                                         5.83369035164687e-28,
                                         1.30275821835619e-16,
                                         1.00652577662252e-10,
                                         5.96177097578141e-07,
                                         0.000225854381397848,
                                         0.0131299504724956,
                                         0.179886886440954,
                                         0.725512253887608,

                                         0.773548117281684,
                                         0.254448930721485,
                                         0.0321565815473972,
                                         0.00136476456741709,
                                         1.52587890625e-05,
                                         2.88444150748462e-08,
                                         3.80384762345604e-12,
                                         4.16584867101484e-18,
                                         1.70230252943409e-29,
                                         0.200224240194892,
                                         0.363581721427999,
                                         0.123296126865031,
                                         0.0111427008515805,
                                         0.000244140625,
                                         9.04417525311919e-07,
                                         2.53970641968726e-10,
                                         7.46349151836755e-16,
                                         1.68363284257638e-26,
                                         0.0242933415351681,
                                         0.243525662586522,
                                         0.221600264749326,
                                         0.04264464313298,
                                         0.0018310546875,
                                         1.32928309838454e-05,
                                         7.94850044370216e-09,
                                         6.26789739210882e-14,
                                         7.80547458769753e-24,
                                         0.00183401707631586,
                                         0.101492264667201,
                                         0.247820160093171,
                                         0.101550955774237,
                                         0.008544921875,
                                         0.000121565828401015,
                                         1.54786259876864e-07,
                                         3.27527092945108e-12,
                                         2.25162860262323e-21,
                                         9.642642689812e-05,
                                         0.0294576252572475,
                                         0.193009893682165,
                                         0.168414750440222,
                                         0.02777099609375,
                                         0.000774251624077088,
                                         2.09921146725859e-06,
                                         1.19192564171094e-10,
                                         4.52346026452313e-19,
                                         3.7438346036373e-06,
                                         0.0063137939329371,
                                         0.111007006307089,
                                         0.206254825332318,
                                         0.066650390625,
                                         0.00364150253697041,
                                         2.1023637251407e-05,
                                         3.20316253180193e-09,
                                         6.7107750811827e-17,
                                         1.11036925230204e-07,
                                         0.00103374463099811,
                                         0.0487698480675265,
                                         0.192956006541361,
                                         0.1221923828125,
                                         0.0130830590557773,
                                         0.0001608383880718,
                                         6.57565443233156e-08,
                                         7.60509738740461e-15,
                                         2.5661304212447e-09,
                                         0.000131885380198159,
                                         0.0166960193715904,
                                         0.140660777018743,
                                         0.174560546875,
                                         0.0366267607736208,
                                         0.00095880898377183,
                                         1.05186380973593e-06,
                                         6.71579454834603e-13,
                                         4.67025491167322e-11,
                                         1.32504493647623e-05,
                                         0.00450116610360487,
                                         0.0807492104293699,
                                         0.196380615234375,
                                         0.0807492104293699,
                                         0.00450116610360488,
                                         1.32504493647623e-05,
                                         4.67025491167335e-11,
                                         6.71579454834582e-13,
                                         1.05186380973593e-06,
                                         0.000958808983771828,
                                         0.0366267607736208,
                                         0.174560546875,
                                         0.140660777018743,
                                         0.0166960193715904,
                                         0.000131885380198159,
                                         2.56613042124476e-09,
                                         7.60509738740434e-15,
                                         6.57565443233157e-08,
                                         0.0001608383880718,
                                         0.0130830590557773,
                                         0.1221923828125,
                                         0.192956006541361,
                                         0.0487698480675266,
                                         0.00103374463099811,
                                         1.11036925230206e-07,
                                         6.71077508118244e-17,
                                         3.20316253180193e-09,
                                         2.1023637251407e-05,
                                         0.00364150253697041,
                                         0.066650390625,
                                         0.206254825332318,
                                         0.11100700630709,
                                         0.0063137939329371,
                                         3.74383460363737e-06,
                                         4.52346026452294e-19,
                                         1.19192564171094e-10,
                                         2.09921146725859e-06,
                                         0.000774251624077088,
                                         0.02777099609375,
                                         0.168414750440222,
                                         0.193009893682165,
                                         0.0294576252572475,
                                         9.64264268981214e-05,
                                         2.25162860262313e-21,
                                         3.27527092945108e-12,
                                         1.54786259876864e-07,
                                         0.000121565828401015,
                                         0.008544921875,
                                         0.101550955774237,
                                         0.247820160093171,
                                         0.101492264667201,
                                         0.00183401707631588,
                                         7.80547458769715e-24,
                                         6.26789739210882e-14,
                                         7.94850044370213e-09,
                                         1.32928309838454e-05,
                                         0.0018310546875,
                                         0.04264464313298,
                                         0.221600264749326,
                                         0.243525662586522,
                                         0.0242933415351682,
                                         1.68363284257629e-26,
                                         7.46349151836755e-16,
                                         2.53970641968725e-10,
                                         9.04417525311919e-07,
                                         0.000244140625,
                                         0.0111427008515805,
                                         0.123296126865031,
                                         0.363581721427999,
                                         0.200224240194893,
                                         1.70230252943399e-29,
                                         4.16584867101484e-18,
                                         3.80384762345602e-12,
                                         2.88444150748462e-08,
                                         1.52587890625e-05,
                                         0.00136476456741709,
                                         0.0321565815473972,
                                         0.254448930721485,
                                         0.773548117281684,

                                         0.810485788943543,
                                         0.32705254915016,
                                         0.0610978658800897,
                                         0.00484591892132691,
                                         0.000140566081105572,
                                         1.15722823402083e-06,
                                         1.72187522753707e-09,
                                         1.89989224675246e-13,
                                         1.84330836213548e-19,
                                         7.04741513910333e-31,
                                         0.171423621254631,
                                         0.37859406333376,
                                         0.186612167211554,
                                         0.0306485704279022,
                                         0.00166617907129917,
                                         2.49929688797456e-05,
                                         6.96959536112173e-08,
                                         1.5924104539261e-11,
                                         4.07644603588873e-17,
                                         8.52991217295194e-28,
                                         0.0169956276097748,
                                         0.205433535973448,
                                         0.267174590893279,
                                         0.0908626881961245,
                                         0.00925772111437913,
                                         0.000253021809081783,
                                         1.32237531067502e-06,
                                         6.25636892559527e-10,
                                         4.22577804555317e-15,
                                         4.83950056346903e-25,
                                         0.00104845372217666,
                                         0.0693608490591653,
                                         0.238010363227568,
                                         0.167612521016915,
                                         0.0320060463530987,
                                         0.00159383581750841,
                                         1.5611600070484e-05,
                                         1.52944969827369e-08,
                                         2.72569454495869e-13,
                                         1.7084491125929e-22,
                                         4.50440990708958e-05,
                                         0.0163092518860747,
                                         0.147663501056849,
                                         0.215329689976661,
                                         0.0770613381397223,
                                         0.0069920703301983,
                                         0.000128356150062171,
                                         2.60390194511582e-07,
                                         1.22440266883087e-11,
                                         4.20029825943965e-20,
                                         1.42907303574385e-06,
                                         0.00283192377992192,
                                         0.0676516410679833,
                                         0.204281600939197,
                                         0.137015261226086,
                                         0.0226514430382226,
                                         0.000779316422355723,
                                         3.27372303783698e-06,
                                         4.06161945679955e-10,
                                         7.62581199192312e-18,
                                         3.46338791992517e-08,
                                         0.000375629137888134,
                                         0.0236762928939706,
                                         0.148041945377797,
                                         0.186093640975609,
                                         0.0560552319579911,
                                         0.00361444122491439,
                                         3.14404994402936e-05,
                                         1.02921092412971e-08,
                                         1.05760196853636e-15,
                                         6.54046123525899e-10,
                                         3.88237526980314e-05,
                                         0.00645668496768579,
                                         0.083598953183547,
                                         0.196949393745679,
                                         0.108092847779197,
                                         0.0130625820406131,
                                         0.000235286745824291,
                                         2.03221694730707e-07,
                                         1.1429281576868e-13,
                                         9.72671467943569e-12,
                                         3.1599943985984e-06,
                                         0.0013866155062756,
                                         0.0371764107293745,
                                         0.164145252363708,
                                         0.164145252363708,
                                         0.0371764107293745,
                                         0.0013866155062756,
                                         3.15999439859838e-06,
                                         9.72671467943569e-12,
                                         1.1429281576868e-13,
                                         2.03221694730709e-07,
                                         0.000235286745824291,
                                         0.0130625820406131,
                                         0.108092847779197,
                                         0.196949393745679,
                                         0.083598953183547,
                                         0.00645668496768579,
                                         3.88237526980312e-05,
                                         6.54046123525899e-10,
                                         1.05760196853636e-15,
                                         1.02921092412972e-08,
                                         3.14404994402936e-05,
                                         0.00361444122491439,
                                         0.0560552319579911,
                                         0.186093640975609,
                                         0.148041945377797,
                                         0.0236762928939706,
                                         0.000375629137888132,
                                         3.46338791992516e-08,
                                         7.62581199192312e-18,
                                         4.06161945679959e-10,
                                         3.27372303783698e-06,
                                         0.000779316422355723,
                                         0.0226514430382226,
                                         0.137015261226086,
                                         0.204281600939197,
                                         0.0676516410679833,
                                         0.0028319237799219,
                                         1.42907303574385e-06,
                                         4.20029825943965e-20,
                                         1.22440266883089e-11,
                                         2.60390194511582e-07,
                                         0.000128356150062171,
                                         0.0069920703301983,
                                         0.0770613381397223,
                                         0.215329689976661,
                                         0.147663501056849,
                                         0.0163092518860746,
                                         4.50440990708958e-05,
                                         1.7084491125929e-22,
                                         2.72569454495871e-13,
                                         1.52944969827369e-08,
                                         1.5611600070484e-05,
                                         0.00159383581750841,
                                         0.0320060463530987,
                                         0.167612521016915,
                                         0.238010363227568,
                                         0.0693608490591652,
                                         0.00104845372217666,
                                         4.83950056346903e-25,
                                         4.22577804555322e-15,
                                         6.25636892559527e-10,
                                         1.32237531067502e-06,
                                         0.000253021809081783,
                                         0.00925772111437913,
                                         0.0908626881961245,
                                         0.267174590893279,
                                         0.205433535973447,
                                         0.0169956276097748,
                                         8.52991217295194e-28,
                                         4.07644603588878e-17,
                                         1.5924104539261e-11,
                                         6.96959536112173e-08,
                                         2.49929688797456e-05,
                                         0.00166617907129917,
                                         0.0306485704279022,
                                         0.186612167211554,
                                         0.37859406333376,
                                         0.171423621254631,
                                         7.04741513910333e-31,
                                         1.84330836213551e-19,
                                         1.89989224675246e-13,
                                         1.72187522753707e-09,
                                         1.15722823402083e-06,
                                         0.000140566081105572,
                                         0.00484591892132691,
                                         0.0610978658800897,
                                         0.32705254915016,
                                         0.810485788943543,

                                         0.839351546462873,
                                         0.394549035753072,
                                         0.098371260796138,
                                         0.0122711194385087,
                                         0.000694831607984297,
                                         1.52587890625e-05,
                                         1.00238156438005e-07,
                                         1.24868367224982e-10,
                                         1.206221828684e-14,
                                         1.06886941308297e-20,
                                         3.8876147734218e-32,
                                         0.14779937094475,
                                         0.377809129320313,
                                         0.245483979206162,
                                         0.0621551589631201,
                                         0.00639656243124129,
                                         0.000244140625,
                                         2.78744276835819e-06,
                                         6.31101772581636e-09,
                                         1.23740604142974e-12,
                                         2.85754549238528e-18,
                                         5.65190038038716e-29,
                                         0.012199514768119,
                                         0.169584123181285,
                                         0.287156963256991,
                                         0.147574547590041,
                                         0.0276029183275773,
                                         0.0018310546875,
                                         3.63345786751277e-05,
                                         1.49515992398293e-07,
                                         5.9502958753968e-11,
                                         3.58098859284669e-16,
                                         3.85165211008002e-26,
                                         0.000626553335185903,
                                         0.0473634631818382,
                                         0.209007113284418,
                                         0.218017450690946,
                                         0.0741154714301386,
                                         0.008544921875,
                                         0.000294699782655357,
                                         2.20404985099551e-06,
                                         1.78037057619149e-09,
                                         2.79227533183095e-14,
                                         1.63322164385941e-23,
                                         2.24104261558136e-05,
                                         0.00921251674151117,
                                         0.105944730876982,
                                         0.224309482623441,
                                         0.138592372840522,
                                         0.02777099609375,
                                         0.0016646237342197,
                                         2.26272491278502e-05,
                                         3.70987122629923e-08,
                                         1.5163174105685e-12,
                                         4.82302795971497e-21,
                                         5.91929609659602e-07,
                                         0.00132324728242554,
                                         0.0396575187221172,
                                         0.170424446024578,
                                         0.191380491729509,
                                         0.066650390625,
                                         0.00694352862453815,
                                         0.000171541808593165,
                                         5.70867268256632e-07,
                                         6.0806482932714e-11,
                                         1.05177371533154e-18,
                                         1.1943186324403e-08,
                                         0.000145189070353207,
                                         0.0113397089217743,
                                         0.0989113982702969,
                                         0.201876699084161,
                                         0.1221923828125,
                                         0.0221245563576675,
                                         0.000993432591498055,
                                         6.71029546536063e-06,
                                         1.86268681415778e-09,
                                         1.75208446916379e-16,
                                         1.87772018902722e-10,
                                         1.24133032014893e-05,
                                         0.00252661343384564,
                                         0.0447323090724908,
                                         0.165933948993932,
                                         0.174560546875,
                                         0.054932509177046,
                                         0.004482985644403,
                                         6.14622803377339e-05,
                                         4.44621358489702e-08,
                                         2.27430283065618e-14,
                                         2.32483484503422e-12,
                                         8.35778826862303e-07,
                                         0.000443329118666097,
                                         0.0159311400312265,
                                         0.107407563619207,
                                         0.196380615234375,
                                         0.107407563619208,
                                         0.0159311400312264,
                                         0.000443329118666096,
                                         8.35778826862303e-07,
                                         2.32483484503432e-12,
                                         2.27430283065608e-14,
                                         4.44621358489702e-08,
                                         6.14622803377341e-05,
                                         0.00448298564440301,
                                         0.0549325091770459,
                                         0.174560546875,
                                         0.165933948993932,
                                         0.0447323090724907,
                                         0.00252661343384563,
                                         1.24133032014893e-05,
                                         1.87772018902729e-10,
                                         1.75208446916371e-16,
                                         1.86268681415778e-09,
                                         6.71029546536065e-06,
                                         0.000993432591498058,
                                         0.0221245563576675,
                                         0.1221923828125,
                                         0.201876699084161,
                                         0.0989113982702968,
                                         0.0113397089217743,
                                         0.000145189070353207,
                                         1.19431863244034e-08,
                                         1.05177371533148e-18,
                                         6.0806482932714e-11,
                                         5.70867268256635e-07,
                                         0.000171541808593165,
                                         0.00694352862453814,
                                         0.066650390625,
                                         0.191380491729509,
                                         0.170424446024578,
                                         0.0396575187221171,
                                         0.00132324728242554,
                                         5.91929609659618e-07,
                                         4.82302795971467e-21,
                                         1.5163174105685e-12,
                                         3.70987122629925e-08,
                                         2.26272491278503e-05,
                                         0.0016646237342197,
                                         0.02777099609375,
                                         0.138592372840522,
                                         0.224309482623441,
                                         0.105944730876982,
                                         0.00921251674151117,
                                         2.24104261558141e-05,
                                         1.6332216438593e-23,
                                         2.79227533183095e-14,
                                         1.7803705761915e-09,
                                         2.20404985099552e-06,
                                         0.000294699782655356,
                                         0.008544921875,
                                         0.0741154714301387,
                                         0.218017450690946,
                                         0.209007113284418,
                                         0.0473634631818382,
                                         0.000626553335185913,
                                         3.85165211007974e-26,
                                         3.58098859284669e-16,
                                         5.95029587539683e-11,
                                         1.49515992398294e-07,
                                         3.63345786751277e-05,
                                         0.0018310546875,
                                         0.0276029183275773,
                                         0.147574547590041,
                                         0.28715696325699,
                                         0.169584123181285,
                                         0.0121995147681191,
                                         5.65190038038672e-29,
                                         2.85754549238528e-18,
                                         1.23740604142975e-12,
                                         6.31101772581638e-09,
                                         2.78744276835819e-06,
                                         0.000244140625,
                                         0.00639656243124129,
                                         0.0621551589631201,
                                         0.245483979206162,
                                         0.377809129320313,
                                         0.14779937094475,
                                         3.88761477342149e-32,
                                         1.06886941308297e-20,
                                         1.20622182868401e-14,
                                         1.24868367224982e-10,
                                         1.00238156438004e-07,
                                         1.52587890625e-05,
                                         0.000694831607984297,
                                         0.0122711194385087,
                                         0.098371260796138,
                                         0.394549035753072,
                                         0.839351546462873,

                                         0.862259374336853,
                                         0.45563711362517,
                                         0.141484947015933,
                                         0.0247823751709009,
                                         0.00229108751884099,
                                         0.000100786405745771,
                                         1.79388700428296e-06,
                                         9.92763289216466e-09,
                                         1.07986450189003e-11,
                                         9.42117802735273e-16,
                                         7.78694866049089e-22,
                                         2.72555989806096e-33,
                                         0.128379748388666,
                                         0.367101228940861,
                                         0.294301452162698,
                                         0.103089529157481,
                                         0.0169418921966395,
                                         0.00125363616054362,
                                         3.69203233494125e-05,
                                         3.43688847739135e-07,
                                         6.64565208921073e-10,
                                         1.15947932041613e-13,
                                         2.47423263167382e-19,
                                         4.68637567911414e-30,
                                         0.00895976040907029,
                                         0.138641696970834,
                                         0.286956359708576,
                                         0.201014536829793,
                                         0.0587250545951474,
                                         0.00730941035986622,
                                         0.000356186281775283,
                                         5.57733141219841e-06,
                                         1.91711082213029e-08,
                                         6.68901369026702e-12,
                                         3.68514881190337e-17,
                                         3.77711102727524e-27,
                                         0.000389082582931551,
                                         0.0325797246984291,
                                         0.174094407819557,
                                         0.243885450662721,
                                         0.126657368900084,
                                         0.0265178735750757,
                                         0.00213813143959174,
                                         5.63161530689692e-05,
                                         3.44114009312735e-07,
                                         2.40108027437812e-10,
                                         3.41519139287148e-15,
                                         1.89420910172295e-24,
                                         1.17669461006974e-05,
                                         0.0053318452348817,
                                         0.0735580771465502,
                                         0.206072908007571,
                                         0.190245360678851,
                                         0.0669995100948053,
                                         0.00893857297573633,
                                         0.00039601906110667,
                                         4.3016400657028e-06,
                                         6.00244037909348e-09,
                                         2.20420328112714e-13,
                                         6.61565222876449e-22,
                                         2.62792894691171e-07,
                                         0.000644370337616223,
                                         0.0229511153438333,
                                         0.128583069089102,
                                         0.211020947368418,
                                         0.125006454946053,
                                         0.0275949658834629,
                                         0.00205649226164337,
                                         3.97094310068103e-05,
                                         1.10809478465167e-07,
                                         1.05054853758278e-11,
                                         1.70626033911872e-19,
                                         4.48325732145095e-09,
                                         5.948722892035e-05,
                                         0.00547025304089665,
                                         0.061288200260219,
                                         0.178799893216793,
                                         0.178165431645967,
                                         0.0650761311703476,
                                         0.0081577100021868,
                                         0.000280016390305689,
                                         1.56263000690636e-06,
                                         3.82481816796766e-10,
                                         3.36161612200949e-17,
                                         5.95983504023296e-11,
                                         4.27929732697318e-06,
                                         0.00101594819271531,
                                         0.0227630511609447,
                                         0.118050944722078,
                                         0.197867731496067,
                                         0.119584286175856,
                                         0.0252156385588439,
                                         0.00153862847904201,
                                         1.71710136651064e-05,
                                         1.08509057155514e-08,
                                         5.16073388388636e-15,
                                         6.2391492476375e-13,
                                         2.42421843002013e-07,
                                         0.000148588866734299,
                                         0.00665785976036465,
                                         0.0613793438184061,
                                         0.173052177178754,
                                         0.173052177178754,
                                         0.0613793438184062,
                                         0.00665785976036466,
                                         0.0001485888667343,
                                         2.42421843002015e-07,
                                         6.2391492476372e-13,
                                         5.16073388388664e-15,
                                         1.08509057155513e-08,
                                         1.71710136651063e-05,
                                         0.00153862847904201,
                                         0.0252156385588438,
                                         0.119584286175856,
                                         0.197867731496067,
                                         0.118050944722078,
                                         0.0227630511609448,
                                         0.00101594819271531,
                                         4.27929732697322e-06,
                                         5.95983504023271e-11,
                                         3.36161612200969e-17,
                                         3.82481816796761e-10,
                                         1.56263000690635e-06,
                                         0.000280016390305688,
                                         0.00815771000218678,
                                         0.0650761311703476,
                                         0.178165431645967,
                                         0.178799893216793,
                                         0.0612882002602191,
                                         0.00547025304089667,
                                         5.94872289203504e-05,
                                         4.48325732145079e-09,
                                         1.70626033911883e-19,
                                         1.05054853758277e-11,
                                         1.10809478465166e-07,
                                         3.97094310068102e-05,
                                         0.00205649226164337,
                                         0.0275949658834629,
                                         0.125006454946053,
                                         0.211020947368418,
                                         0.128583069089102,
                                         0.0229511153438334,
                                         0.000644370337616227,
                                         2.62792894691164e-07,
                                         6.61565222876497e-22,
                                         2.20420328112711e-13,
                                         6.00244037909345e-09,
                                         4.30164006570279e-06,
                                         0.000396019061106669,
                                         0.00893857297573633,
                                         0.0669995100948053,
                                         0.190245360678851,
                                         0.206072908007571,
                                         0.0735580771465504,
                                         0.00533184523488173,
                                         1.17669461006972e-05,
                                         1.8942091017231e-24,
                                         3.41519139287143e-15,
                                         2.4010802743781e-10,
                                         3.44114009312734e-07,
                                         5.6316153068969e-05,
                                         0.00213813143959174,
                                         0.0265178735750757,
                                         0.126657368900084,
                                         0.243885450662721,
                                         0.174094407819558,
                                         0.0325797246984292,
                                         0.000389082582931544,
                                         3.77711102727556e-27,
                                         3.68514881190331e-17,
                                         6.68901369026697e-12,
                                         1.91711082213028e-08,
                                         5.5773314121984e-06,
                                         0.000356186281775283,
                                         0.00730941035986622,
                                         0.0587250545951474,
                                         0.201014536829793,
                                         0.286956359708576,
                                         0.138641696970834,
                                         0.00895976040907018,
                                         4.68637567911456e-30,
                                         2.47423263167378e-19,
                                         1.15947932041612e-13,
                                         6.64565208921071e-10,
                                         3.43688847739134e-07,
                                         3.69203233494125e-05,
                                         0.00125363616054362,
                                         0.0169418921966395,
                                         0.103089529157481,
                                         0.294301452162698,
                                         0.367101228940862,
                                         0.128379748388665,
                                         2.72555989806122e-33,
                                         7.78694866049075e-22,
                                         9.42117802735266e-16,
                                         1.07986450189002e-11,
                                         9.92763289216464e-09,
                                         1.79388700428296e-06,
                                         0.000100786405745771,
                                         0.00229108751884099,
                                         0.0247823751709009,
                                         0.141484947015933,
                                         0.45563711362517,
                                         0.862259374336853,

                                         0.880699705534374,
                                         0.51008506183465,
                                         0.18791849961893,
                                         0.0427532083993999,
                                         0.00573035182899891,
                                         0.000421299129289599,
                                         1.52587890625e-05,
                                         2.30799285307463e-07,
                                         1.1177371137978e-09,
                                         1.09361528948877e-12,
                                         8.80955673322661e-17,
                                         6.89430428176287e-23,
                                         2.34143369765675e-34,
                                         0.112328176726767,
                                         0.350703885815949,
                                         0.331150985218676,
                                         0.148964295065757,
                                         0.0349089003922802,
                                         0.00421575911656611,
                                         0.000244140625,
                                         5.90458254955842e-06,
                                         4.69704537140545e-08,
                                         8.03509590272549e-11,
                                         1.27978644764044e-14,
                                         2.56703883998355e-20,
                                         4.69960082327392e-31,
                                         0.00671569321926812,
                                         0.113026383424441,
                                         0.27354189257995,
                                         0.243297104568329,
                                         0.0996855780219472,
                                         0.0197743535560456,
                                         0.0018310546875,
                                         7.08084713048905e-05,
                                         9.25233011609026e-07,
                                         2.76731652631319e-09,
                                         8.71489658100529e-13,
                                         4.48038890374476e-18,
                                         4.42161941745489e-28,
                                         0.000249826501126211,
                                         0.0226654633893093,
                                         0.14059412100433,
                                         0.247250519525228,
                                         0.177122613373206,
                                         0.0577130889488875,
                                         0.008544921875,
                                         0.00052835613642527,
                                         1.1340259733639e-05,
                                         5.93023803978736e-08,
                                         3.69260187747054e-11,
                                         4.86569076012353e-16,
                                         2.58849460352177e-25,
                                         6.47236115059002e-06,
                                         0.00316538072610189,
                                         0.0503253790167304,
                                         0.174990336039995,
                                         0.219175633963363,
                                         0.11730672874255,
                                         0.02777099609375,
                                         0.00274564809135228,
                                         9.67991321278703e-05,
                                         8.85038158867549e-07,
                                         1.08963065927152e-09,
                                         3.68001742143564e-14,
                                         1.0553335599268e-22,
                                         1.23826859926415e-07,
                                         0.000326448881899632,
                                         0.0133025478146284,
                                         0.0914573889098479,
                                         0.200280382499094,
                                         0.176075694233518,
                                         0.066650390625,
                                         0.010536366557091,
                                         0.000610165722205137,
                                         9.75393251048549e-06,
                                         2.37440076651585e-08,
                                         2.0553374060448e-12,
                                         3.17731384323893e-20,
                                         1.80966076482062e-09,
                                         2.57178471227592e-05,
                                         0.00268604197987441,
                                         0.0365135218680056,
                                         0.139802438274601,
                                         0.201885946281347,
                                         0.1221923828125,
                                         0.0308863836982767,
                                         0.00293801725461338,
                                         8.21159817866287e-05,
                                         3.95238212094184e-07,
                                         8.76892934680514e-11,
                                         7.30736286581944e-18,
                                         2.06081974334956e-11,
                                         1.57875435106209e-06,
                                         0.000422621209354946,
                                         0.0113592373888197,
                                         0.0760416627915494,
                                         0.180373732142636,
                                         0.174560546875,
                                         0.0705510982506717,
                                         0.0110235481394078,
                                         0.000538686582494963,
                                         5.12654003756099e-06,
                                         2.91521466985557e-09,
                                         1.30954901441577e-15,
                                         1.84813377816297e-13,
                                         7.63211794193501e-08,
                                         5.23649078267304e-05,
                                         0.00278288454852637,
                                         0.032571595025671,
                                         0.126908601262204,
                                         0.196380615234375,
                                         0.126908601262204,
                                         0.032571595025671,
                                         0.00278288454852636,
                                         5.23649078267306e-05,
                                         7.63211794193493e-08,
                                         1.84813377816287e-13,
                                         1.30954901441586e-15,
                                         2.9152146698556e-09,
                                         5.12654003756097e-06,
                                         0.000538686582494965,
                                         0.0110235481394078,
                                         0.0705510982506717,
                                         0.174560546875,
                                         0.180373732142636,
                                         0.0760416627915493,
                                         0.0113592373888197,
                                         0.000422621209354948,
                                         1.57875435106208e-06,
                                         2.06081974334946e-11,
                                         7.30736286581995e-18,
                                         8.76892934680526e-11,
                                         3.95238212094182e-07,
                                         8.21159817866289e-05,
                                         0.00293801725461338,
                                         0.0308863836982767,
                                         0.1221923828125,
                                         0.201885946281347,
                                         0.139802438274601,
                                         0.0365135218680056,
                                         0.00268604197987442,
                                         2.5717847122759e-05,
                                         1.80966076482055e-09,
                                         3.17731384323918e-20,
                                         2.05533740604483e-12,
                                         2.37440076651583e-08,
                                         9.75393251048552e-06,
                                         0.000610165722205137,
                                         0.010536366557091,
                                         0.066650390625,
                                         0.176075694233518,
                                         0.200280382499094,
                                         0.0914573889098477,
                                         0.0133025478146284,
                                         0.00032644888189963,
                                         1.23826859926411e-07,
                                         1.05533355992689e-22,
                                         3.6800174214357e-14,
                                         1.08963065927151e-09,
                                         8.85038158867553e-07,
                                         9.67991321278703e-05,
                                         0.00274564809135227,
                                         0.02777099609375,
                                         0.11730672874255,
                                         0.219175633963363,
                                         0.174990336039995,
                                         0.0503253790167305,
                                         0.00316538072610187,
                                         6.47236115058984e-06,
                                         2.58849460352201e-25,
                                         4.86569076012362e-16,
                                         3.69260187747051e-11,
                                         5.93023803978738e-08,
                                         1.1340259733639e-05,
                                         0.00052835613642527,
                                         0.008544921875,
                                         0.0577130889488875,
                                         0.177122613373206,
                                         0.247250519525227,
                                         0.14059412100433,
                                         0.0226654633893093,
                                         0.000249826501126206,
                                         4.42161941745533e-28,
                                         4.48038890374485e-18,
                                         8.71489658100522e-13,
                                         2.7673165263132e-09,
                                         9.25233011609026e-07,
                                         7.08084713048905e-05,
                                         0.0018310546875,
                                         0.0197743535560456,
                                         0.0996855780219472,
                                         0.243297104568328,
                                         0.273541892579951,
                                         0.113026383424441,
                                         0.00671569321926802,
                                         4.69960082327441e-31,
                                         2.5670388399836e-20,
                                         1.27978644764043e-14,
                                         8.03509590272553e-11,
                                         4.69704537140545e-08,
                                         5.90458254955842e-06,
                                         0.000244140625,
                                         0.00421575911656611,
                                         0.0349089003922802,
                                         0.148964295065757,
                                         0.331150985218676,
                                         0.350703885815949,
                                         0.112328176726766,
                                         2.34143369765701e-34,
                                         6.89430428176302e-23,
                                         8.80955673322653e-17,
                                         1.09361528948878e-12,
                                         1.1177371137978e-09,
                                         2.30799285307463e-07,
                                         1.52587890625e-05,
                                         0.000421299129289599,
                                         0.00573035182899891,
                                         0.0427532083993999,
                                         0.18791849961893,
                                         0.51008506183465,
                                         0.880699705534374,

                                         0.895737813714083,
                                         0.558212723771659,
                                         0.235560493525201,
                                         0.0658518156659966,
                                         0.0117831796420592,
                                         0.00128241547822689,
                                         7.87953941805609e-05,
                                         2.44866669224247e-06,
                                         3.25632083406552e-08,
                                         1.41848484656415e-10,
                                         1.27562738575861e-13,
                                         9.64199711678241e-18,
                                         7.223408486312e-24,
                                         2.39473029619722e-35,
                                         0.098967619081284,
                                         0.33144854497798,
                                         0.356431823177559,
                                         0.195269759564013,
                                         0.0603139288163657,
                                         0.0105911321290063,
                                         0.00101484009595005,
                                         4.86712896430903e-05,
                                         1.0093772642396e-06,
                                         7.09429993406375e-09,
                                         1.10127698170856e-11,
                                         1.63129665657321e-15,
                                         3.11433807241429e-21,
                                         5.54861204063456e-32,
                                         0.00512562249486852,
                                         0.0922515736018485,
                                         0.252808556743582,
                                         0.27142103980843,
                                         0.144715156998596,
                                         0.0410012693131208,
                                         0.00612682793275727,
                                         0.00045347914036999,
                                         1.46663190820106e-05,
                                         1.66316628078865e-07,
                                         4.45667095357302e-10,
                                         1.29371965302986e-13,
                                         6.29405618877097e-19,
                                         6.02633669221881e-29,
                                         0.000165175499646805,
                                         0.0159763279437077,
                                         0.111571323308779,
                                         0.234745646048816,
                                         0.216050829802308,
                                         0.0987637871941373,
                                         0.0230154392179665,
                                         0.00262898006572646,
                                         0.000132597170882979,
                                         2.42609227412053e-06,
                                         1.12219956552013e-08,
                                         6.38400065220985e-12,
                                         7.91481911380207e-17,
                                         4.07256422381593e-26,
                                         3.70698835230171e-06,
                                         0.00192688914581435,
                                         0.0342917760622219,
                                         0.141393082281539,
                                         0.224633455204157,
                                         0.165681778685154,
                                         0.0602114978121411,
                                         0.0106143609350072,
                                         0.000834879825535692,
                                         2.46465160879007e-05,
                                         1.96791439334626e-07,
                                         2.19392738859437e-10,
                                         6.93151156449898e-15,
                                         1.91672175214054e-23,
                                         6.14362493531712e-08,
                                         0.000171618607884495,
                                         0.00778314738537431,
                                         0.0628907333121284,
                                         0.17247288047698,
                                         0.205248334788828,
                                         0.116323503776654,
                                         0.0316466898341476,
                                         0.00388187508484503,
                                         0.000184897756801014,
                                         2.54841521226399e-06,
                                         5.56774655255141e-09,
                                         4.48273216285751e-13,
                                         6.66159279566568e-21,
                                         7.77782845807972e-10,
                                         1.1676218468729e-05,
                                         0.00134943112534143,
                                         0.0213685645207552,
                                         0.101157337258016,
                                         0.194229293023747,
                                         0.171666563151965,
                                         0.0720763727786538,
                                         0.0137876210550048,
                                         0.00105959021593029,
                                         2.52095079259156e-05,
                                         1.07936133981526e-07,
                                         2.2145618963226e-11,
                                         1.76859049030072e-18,
                                         7.6727765680744e-12,
                                         6.1901414766292e-07,
                                         0.000182308415717356,
                                         0.00565750001130209,
                                         0.0462311276386185,
                                         0.143222200372767,
                                         0.197407875658203,
                                         0.127914000061709,
                                         0.0381590581004007,
                                         0.00473156446850454,
                                         0.000194320703265581,
                                         1.63047740483355e-06,
                                         8.524980228844e-10,
                                         3.65878604703477e-16,
                                         5.96070056153239e-14,
                                         2.58433911095485e-08,
                                         1.93960490321121e-05,
                                         0.00117957169037259,
                                         0.0166388055605228,
                                         0.083168049518789,
                                         0.178769654188233,
                                         0.178769654188233,
                                         0.083168049518789,
                                         0.0166388055605228,
                                         0.0011795716903726,
                                         1.93960490321121e-05,
                                         2.58433911095481e-08,
                                         5.96070056153239e-14,
                                         3.65878604703477e-16,
                                         8.52498022884412e-10,
                                         1.63047740483355e-06,
                                         0.00019432070326558,
                                         0.00473156446850453,
                                         0.0381590581004007,
                                         0.127914000061709,
                                         0.197407875658204,
                                         0.143222200372767,
                                         0.0462311276386186,
                                         0.00565750001130211,
                                         0.000182308415717356,
                                         6.19014147662914e-07,
                                         7.6727765680744e-12,
                                         1.76859049030072e-18,
                                         2.21456189632264e-11,
                                         1.07936133981526e-07,
                                         2.52095079259155e-05,
                                         0.00105959021593029,
                                         0.0137876210550048,
                                         0.0720763727786538,
                                         0.171666563151965,
                                         0.194229293023747,
                                         0.101157337258016,
                                         0.0213685645207552,
                                         0.00134943112534143,
                                         1.16762184687289e-05,
                                         7.77782845807972e-10,
                                         6.66159279566568e-21,
                                         4.48273216285758e-13,
                                         5.56774655255141e-09,
                                         2.54841521226398e-06,
                                         0.000184897756801014,
                                         0.00388187508484503,
                                         0.0316466898341476,
                                         0.116323503776654,
                                         0.205248334788828,
                                         0.172472880476981,
                                         0.0628907333121285,
                                         0.00778314738537431,
                                         0.000171618607884494,
                                         6.14362493531712e-08,
                                         1.91672175214054e-23,
                                         6.93151156449911e-15,
                                         2.19392738859437e-10,
                                         1.96791439334626e-07,
                                         2.46465160879007e-05,
                                         0.000834879825535692,
                                         0.0106143609350072,
                                         0.0602114978121411,
                                         0.165681778685154,
                                         0.224633455204157,
                                         0.141393082281539,
                                         0.0342917760622219,
                                         0.00192688914581433,
                                         3.70698835230171e-06,
                                         4.07256422381593e-26,
                                         7.91481911380223e-17,
                                         6.38400065220985e-12,
                                         1.12219956552013e-08,
                                         2.42609227412052e-06,
                                         0.000132597170882979,
                                         0.00262898006572645,
                                         0.0230154392179665,
                                         0.0987637871941374,
                                         0.216050829802309,
                                         0.234745646048816,
                                         0.111571323308779,
                                         0.0159763279437076,
                                         0.000165175499646805,
                                         6.02633669221881e-29,
                                         6.29405618877111e-19,
                                         1.29371965302986e-13,
                                         4.456670953573e-10,
                                         1.66316628078865e-07,
                                         1.46663190820106e-05,
                                         0.000453479140369989,
                                         0.00612682793275727,
                                         0.0410012693131208,
                                         0.144715156998596,
                                         0.27142103980843,
                                         0.252808556743582,
                                         0.0922515736018482,
                                         0.00512562249486852,
                                         5.54861204063456e-32,
                                         3.11433807241436e-21,
                                         1.63129665657321e-15,
                                         1.10127698170855e-11,
                                         7.09429993406372e-09,
                                         1.0093772642396e-06,
                                         4.86712896430902e-05,
                                         0.00101484009595005,
                                         0.0105911321290063,
                                         0.0603139288163658,
                                         0.195269759564013,
                                         0.356431823177559,
                                         0.33144854497798,
                                         0.098967619081284,
                                         2.39473029619722e-35,
                                         7.22340848631218e-24,
                                         9.64199711678241e-18,
                                         1.27562738575861e-13,
                                         1.41848484656415e-10,
                                         3.25632083406552e-08,
                                         2.44866669224247e-06,
                                         7.87953941805609e-05,
                                         0.00128241547822689,
                                         0.0117831796420592,
                                         0.0658518156659966,
                                         0.235560493525201,
                                         0.558212723771659,
                                         0.895737813714083,

                                         0.908146677380746,
                                         0.600585285949586,
                                         0.282831824851593,
                                         0.0932824076665524,
                                         0.0209998380366774,
                                         0.00310798441761014,
                                         0.000286636228362617,
                                         1.52587890625e-05,
                                         4.19353569761197e-07,
                                         5.02743442774725e-09,
                                         2.01042308682826e-11,
                                         1.68865295919259e-14,
                                         1.2121798187584e-18,
                                         8.76493859052028e-25,
                                         2.84938521135957e-36,
                                         0.0877633473129063,
                                         0.311139792324178,
                                         0.371664158323827,
                                         0.238522043112207,
                                         0.0917598928412673,
                                         0.0216099140359202,
                                         0.00304985465026234,
                                         0.000244140625,
                                         1.00895604803858e-05,
                                         1.85102452791893e-07,
                                         1.1778513272782e-09,
                                         1.69064177855434e-12,
                                         2.36148183144373e-16,
                                         4.33119735750088e-22,
                                         7.54803578742635e-33,
                                         0.00397568200748672,
                                         0.0755575222560317,
                                         0.228936146487582,
                                         0.285889743677224,
                                         0.187945153423662,
                                         0.070431757656062,
                                         0.0152113754091861,
                                         0.0018310546875,
                                         0.000113790349780199,
                                         3.19462005068058e-06,
                                         3.23470566561128e-08,
                                         7.93421690745903e-11,
                                         2.15647013826663e-14,
                                         1.00324810942157e-19,
                                         9.37255539718106e-30,
                                         0.000112061290438856,
                                         0.0114168258261819,
                                         0.087745241315515,
                                         0.213213229012282,
                                         0.239527224282417,
                                         0.142833320126171,
                                         0.0472066676717785,
                                         0.008544921875,
                                         0.000798516940998569,
                                         3.43061332551345e-05,
                                         5.52744691442821e-07,
                                         2.31687209559263e-09,
                                         1.22531515846268e-12,
                                         1.44595322455199e-17,
                                         7.2414843929705e-27,
                                         2.19976444497798e-06,
                                         0.00120140552098745,
                                         0.0234212115609874,
                                         0.110740453969854,
                                         0.212596060507654,
                                         0.201728433580686,
                                         0.102026921930355,
                                         0.02777099609375,
                                         0.00390246859143055,
                                         0.000256567065710009,
                                         6.57795611022155e-06,
                                         4.71168913138749e-08,
                                         4.84873829108565e-11,
                                         1.45136524217027e-15,
                                         3.89649175869917e-24,
                                         3.18878044372745e-08,
                                         9.33601953899354e-05,
                                         0.00461660823687695,
                                         0.0424743099960755,
                                         0.139342444188758,
                                         0.210393627631096,
                                         0.162837379670442,
                                         0.066650390625,
                                         0.0140838885305542,
                                         0.00141696109171796,
                                         5.78076404859554e-05,
                                         7.07585751979455e-07,
                                         1.41689465572266e-09,
                                         1.07578893509681e-13,
                                         1.54827394640193e-21,
                                         3.53104498670299e-10,
                                         5.54196876311549e-06,
                                         0.000695131357774421,
                                         0.0124444743910136,
                                         0.0697656795472266,
                                         0.167620927916779,
                                         0.198528846383797,
                                         0.1221923828125,
                                         0.0388271876725086,
                                         0.00597785168999623,
                                         0.000388069676162698,
                                         8.11730235471279e-06,
                                         3.16283527364135e-08,
                                         6.09126678327427e-12,
                                         4.69950420523711e-19,
                                         3.04678920801911e-12,
                                         2.56346221517259e-07,
                                         8.15588819393484e-05,
                                         0.00284110516373774,
                                         0.0272182850889764,
                                         0.10406014443622,
                                         0.188605198058134,
                                         0.174560546875,
                                         0.0834084049137455,
                                         0.0196513717836417,
                                         0.00202999373853982,
                                         7.25612727468455e-05,
                                         5.50143657301991e-07,
                                         2.68750136889603e-10,
                                         1.11151877927395e-16,
                                         2.07029450319489e-14,
                                         9.33770887654224e-09,
                                         7.53574526552606e-06,
                                         0.000510797335388571,
                                         0.00836238678235908,
                                         0.0508734476851974,
                                         0.141102354084614,
                                         0.196380615234375,
                                         0.141102354084614,
                                         0.0508734476851974,
                                         0.00836238678235908,
                                         0.000510797335388569,
                                         7.53574526552611e-06,
                                         9.33770887654223e-09,
                                         2.07029450319505e-14,
                                         1.11151877927386e-16,
                                         2.68750136889603e-10,
                                         5.50143657301987e-07,
                                         7.25612727468457e-05,
                                         0.00202999373853982,
                                         0.0196513717836417,
                                         0.0834084049137455,
                                         0.174560546875,
                                         0.188605198058134,
                                         0.10406014443622,
                                         0.0272182850889764,
                                         0.00284110516373773,
                                         8.15588819393488e-05,
                                         2.56346221517259e-07,
                                         3.0467892080193e-12,
                                         4.69950420523668e-19,
                                         6.09126678327427e-12,
                                         3.16283527364133e-08,
                                         8.11730235471282e-06,
                                         0.000388069676162698,
                                         0.00597785168999623,
                                         0.0388271876725086,
                                         0.1221923828125,
                                         0.198528846383797,
                                         0.167620927916779,
                                         0.0697656795472266,
                                         0.0124444743910136,
                                         0.000695131357774424,
                                         5.54196876311549e-06,
                                         3.53104498670319e-10,
                                         1.54827394640177e-21,
                                         1.07578893509681e-13,
                                         1.41689465572265e-09,
                                         7.07585751979458e-07,
                                         5.78076404859554e-05,
                                         0.00141696109171796,
                                         0.0140838885305542,
                                         0.066650390625,
                                         0.162837379670442,
                                         0.210393627631096,
                                         0.139342444188758,
                                         0.0424743099960754,
                                         0.00461660823687696,
                                         9.33601953899354e-05,
                                         3.1887804437276e-08,
                                         3.89649175869874e-24,
                                         1.45136524217027e-15,
                                         4.8487382910856e-11,
                                         4.71168913138752e-08,
                                         6.57795611022155e-06,
                                         0.000256567065710009,
                                         0.00390246859143055,
                                         0.02777099609375,
                                         0.102026921930355,
                                         0.201728433580686,
                                         0.212596060507654,
                                         0.110740453969854,
                                         0.0234212115609875,
                                         0.00120140552098745,
                                         2.19976444497807e-06,
                                         7.24148439296963e-27,
                                         1.44595322455199e-17,
                                         1.22531515846267e-12,
                                         2.31687209559264e-09,
                                         5.52744691442821e-07,
                                         3.43061332551344e-05,
                                         0.000798516940998569,
                                         0.008544921875,
                                         0.0472066676717785,
                                         0.142833320126171,
                                         0.239527224282417,
                                         0.213213229012281,
                                         0.0877452413155152,
                                         0.0114168258261819,
                                         0.000112061290438859,
                                         9.37255539717985e-30,
                                         1.00324810942157e-19,
                                         2.15647013826661e-14,
                                         7.93421690745907e-11,
                                         3.23470566561128e-08,
                                         3.19462005068058e-06,
                                         0.000113790349780199,
                                         0.0018310546875,
                                         0.0152113754091861,
                                         0.070431757656062,
                                         0.187945153423663,
                                         0.285889743677224,
                                         0.228936146487582,
                                         0.0755575222560317,
                                         0.00397568200748679,
                                         7.5480357874253e-33,
                                         4.33119735750088e-22,
                                         2.36148183144371e-16,
                                         1.69064177855435e-12,
                                         1.1778513272782e-09,
                                         1.85102452791893e-07,
                                         1.00895604803858e-05,
                                         0.000244140625,
                                         0.00304985465026234,
                                         0.0216099140359202,
                                         0.0917598928412673,
                                         0.238522043112206,
                                         0.371664158323827,
                                         0.311139792324178,
                                         0.0877633473129071,
                                         2.84938521135915e-36,
                                         8.76493859052028e-25,
                                         1.21217981875839e-18,
                                         1.6886529591926e-14,
                                         2.01042308682826e-11,
                                         5.02743442774725e-09,
                                         4.19353569761197e-07,
                                         1.52587890625e-05,
                                         0.000286636228362617,
                                         0.00310798441761014,
                                         0.0209998380366774,
                                         0.0932824076665524,
                                         0.282831824851593,
                                         0.600585285949586,
                                         0.908146677380746,

                                         0.918495753722746,
                                         0.637845242312638,
                                         0.328649378791009,
                                         0.12403807684013,
                                         0.0336236488106611,
                                         0.00636385702049388,
                                         0.000808312511646715,
                                         6.5196662071817e-05,
                                         3.08888175727219e-06,
                                         7.67974682366851e-08,
                                         8.45850387660093e-10,
                                         3.15344069866842e-12,
                                         2.50436760186354e-15,
                                         1.72303843169785e-19,
                                         1.21006417993496e-25,
                                         3.87080205760852e-37,
                                         0.0782965044993638,
                                         0.29088151328534,
                                         0.378726271795737,
                                         0.276532670391549,
                                         0.127064304255586,
                                         0.0378498111939485,
                                         0.00724952845674774,
                                         0.000862122221183915,
                                         5.97995300910158e-05,
                                         2.19208005473313e-06,
                                         3.6407446009576e-08,
                                         2.13621968057237e-10,
                                         2.87572013797094e-13,
                                         3.82773896108489e-17,
                                         6.79278032711129e-23,
                                         1.1624538166727e-33,
                                         0.00312859160206991,
                                         0.0621810715874956,
                                         0.204578189275294,
                                         0.288987562360245,
                                         0.225083451125322,
                                         0.105523284867085,
                                         0.0304776514079898,
                                         0.00534384186130064,
                                         0.000542669657679411,
                                         2.93296715008728e-05,
                                         7.34561489987044e-07,
                                         6.78341575876503e-09,
                                         1.54787697933538e-11,
                                         3.98594169589536e-15,
                                         1.787425772695e-20,
                                         1.63640852373638e-30,
                                         7.77859023929035e-05,
                                         0.00827076727815593,
                                         0.0687604555955424,
                                         0.18791323685863,
                                         0.24808990088116,
                                         0.183053654748052,
                                         0.0797257757135172,
                                         0.0206102791417428,
                                         0.00306421211511743,
                                         0.000244176297786893,
                                         9.22171330193265e-06,
                                         1.34028290327032e-07,
                                         5.18408103321321e-10,
                                         2.58264722744995e-13,
                                         2.9265367278256e-18,
                                         1.4333533830323e-27,
                                         1.34688182784371e-06,
                                         0.000766143250009676,
                                         0.0160951393404499,
                                         0.0850965935074265,
                                         0.190436936187189,
                                         0.221149054210135,
                                         0.145242123420061,
                                         0.0553593178429448,
                                         0.0120497681623255,
                                         0.00141571681548205,
                                         8.06253579737056e-05,
                                         1.84425555182206e-06,
                                         1.20915980426109e-08,
                                         1.16540215051592e-11,
                                         3.33700242325861e-16,
                                         8.74362249304761e-25,
                                         1.72220947129782e-08,
                                         5.24085373314418e-05,
                                         0.00278213767155076,
                                         0.0284573763677779,
                                         0.107949632135931,
                                         0.197296621858837,
                                         0.195395386997128,
                                         0.109805847136959,
                                         0.0349918124309696,
                                         0.00606145879341515,
                                         0.000520546554731263,
                                         1.87401732691726e-05,
                                         2.08268458465393e-07,
                                         3.88342053328397e-10,
                                         2.80987464815115e-14,
                                         3.938740803838e-22,
                                         1.68218070119636e-10,
                                         2.73857280869788e-06,
                                         0.000367360693227449,
                                         0.00726955343721442,
                                         0.0467435121493711,
                                         0.134457305935621,
                                         0.200801159532067,
                                         0.16637589194057,
                                         0.0776219193586541,
                                         0.0198247681417563,
                                         0.0025673063204677,
                                         0.000145464258522154,
                                         2.74027088562768e-06,
                                         9.88514907280239e-09,
                                         1.8073720543535e-12,
                                         1.35535667451789e-19,
                                         1.2803238756931e-12,
                                         1.1150826782448e-07,
                                         3.77978659262241e-05,
                                         0.0014470419917463,
                                         0.0157718282789594,
                                         0.0714018865070075,
                                         0.160797262727768,
                                         0.19643363434287,
                                         0.134172293081067,
                                         0.0505242181516317,
                                         0.00986634571044766,
                                         0.00087983152363348,
                                         2.80946752051694e-05,
                                         1.96070632023887e-07,
                                         9.0587589855026e-11,
                                         3.63421252374473e-17,
                                         7.67392596457916e-15,
                                         3.57552986397078e-09,
                                         3.06261522325593e-06,
                                         0.000226832404957199,
                                         0.0041907649363626,
                                         0.0298597061871112,
                                         0.10140086248398,
                                         0.182638305632697,
                                         0.182638305632697,
                                         0.10140086248398,
                                         0.0298597061871112,
                                         0.0041907649363626,
                                         0.000226832404957198,
                                         3.06261522325595e-06,
                                         3.57552986397078e-09,
                                         7.67392596457852e-15,
                                         3.63421252374507e-17,
                                         9.05875898550261e-11,
                                         1.96070632023885e-07,
                                         2.80946752051695e-05,
                                         0.00087983152363348,
                                         0.00986634571044765,
                                         0.0505242181516318,
                                         0.134172293081067,
                                         0.19643363434287,
                                         0.160797262727767,
                                         0.0714018865070075,
                                         0.0157718282789594,
                                         0.0014470419917463,
                                         3.77978659262243e-05,
                                         1.1150826782448e-07,
                                         1.28032387569301e-12,
                                         1.35535667451803e-19,
                                         1.8073720543535e-12,
                                         9.88514907280231e-09,
                                         2.74027088562769e-06,
                                         0.000145464258522154,
                                         0.0025673063204677,
                                         0.0198247681417564,
                                         0.0776219193586541,
                                         0.166375891940569,
                                         0.200801159532067,
                                         0.134457305935621,
                                         0.0467435121493711,
                                         0.0072695534372144,
                                         0.000367360693227451,
                                         2.73857280869788e-06,
                                         1.68218070119626e-10,
                                         3.93874080383845e-22,
                                         2.80987464815115e-14,
                                         3.88342053328393e-10,
                                         2.08268458465394e-07,
                                         1.87401732691726e-05,
                                         0.000520546554731263,
                                         0.00606145879341516,
                                         0.0349918124309695,
                                         0.109805847136959,
                                         0.195395386997127,
                                         0.197296621858837,
                                         0.107949632135931,
                                         0.0284573763677778,
                                         0.00278213767155077,
                                         5.24085373314418e-05,
                                         1.72220947129773e-08,
                                         8.74362249304871e-25,
                                         3.33700242325861e-16,
                                         1.16540215051591e-11,
                                         1.2091598042611e-08,
                                         1.84425555182206e-06,
                                         8.06253579737056e-05,
                                         0.00141571681548205,
                                         0.0120497681623255,
                                         0.0553593178429447,
                                         0.145242123420061,
                                         0.221149054210135,
                                         0.190436936187189,
                                         0.0850965935074264,
                                         0.0160951393404499,
                                         0.000766143250009676,
                                         1.34688182784365e-06,
                                         1.43335338303249e-27,
                                         2.92653672782559e-18,
                                         2.58264722744992e-13,
                                         5.18408103321324e-10,
                                         1.34028290327032e-07,
                                         9.22171330193265e-06,
                                         0.000244176297786893,
                                         0.00306421211511743,
                                         0.0206102791417428,
                                         0.0797257757135171,
                                         0.183053654748052,
                                         0.24808990088116,
                                         0.18791323685863,
                                         0.0687604555955426,
                                         0.00827076727815593,
                                         7.7785902392901e-05,
                                         1.63640852373662e-30,
                                         1.787425772695e-20,
                                         3.98594169589532e-15,
                                         1.54787697933539e-11,
                                         6.78341575876503e-09,
                                         7.34561489987044e-07,
                                         2.93296715008729e-05,
                                         0.00054266965767941,
                                         0.00534384186130062,
                                         0.0304776514079898,
                                         0.105523284867085,
                                         0.225083451125322,
                                         0.288987562360244,
                                         0.204578189275294,
                                         0.0621810715874956,
                                         0.00312859160206985,
                                         1.16245381667289e-33,
                                         6.79278032711129e-23,
                                         3.82773896108484e-17,
                                         2.87572013797096e-13,
                                         2.13621968057237e-10,
                                         3.6407446009576e-08,
                                         2.19208005473313e-06,
                                         5.97995300910157e-05,
                                         0.000862122221183913,
                                         0.00724952845674774,
                                         0.0378498111939485,
                                         0.127064304255586,
                                         0.276532670391549,
                                         0.378726271795737,
                                         0.29088151328534,
                                         0.078296504499363,
                                         3.87080205760916e-37,
                                         1.21006417993496e-25,
                                         1.72303843169783e-19,
                                         2.50436760186356e-15,
                                         3.15344069866842e-12,
                                         8.45850387660093e-10,
                                         7.67974682366853e-08,
                                         3.08888175727219e-06,
                                         6.51966620718168e-05,
                                         0.000808312511646715,
                                         0.00636385702049388,
                                         0.0336236488106611,
                                         0.12403807684013,
                                         0.328649378791009,
                                         0.637845242312638,
                                         0.918495753722746,

                                         0.927210742687717,
                                         0.670626355150675,
                                         0.372331296866457,
                                         0.157089775168795,
                                         0.0496014273053605,
                                         0.0114688932629969,
                                         0.00188450632718153,
                                         0.000211203051617704,
                                         1.52587890625e-05,
                                         6.56712003211304e-07,
                                         1.50298044669401e-08,
                                         1.54311249436169e-10,
                                         5.42770081655608e-13,
                                         4.11431714704847e-16,
                                         2.73248488944918e-20,
                                         1.87292362148032e-26,
                                         5.91114170285222e-38,
                                         0.07023914956507,
                                         0.271317661491349,
                                         0.37944714526995,
                                         0.308249418688218,
                                         0.163892569560446,
                                         0.0591148032376066,
                                         0.0144769580433245,
                                         0.00235565857286347,
                                         0.000244140625,
                                         1.50731063753278e-05,
                                         5.00856806488529e-07,
                                         7.66412916810545e-09,
                                         4.20523988986702e-11,
                                         5.36764002795126e-14,
                                         6.86397965411491e-18,
                                         1.18512070065879e-23,
                                         1.99760927535039e-34,
                                         0.00249414280133913,
                                         0.0514537218208548,
                                         0.181265151071336,
                                         0.283529267001325,
                                         0.253843259436646,
                                         0.142827642087196,
                                         0.0521312708578905,
                                         0.0123158894132072,
                                         0.0018310546875,
                                         0.000162170499604673,
                                         7.82375262550238e-06,
                                         1.78430595185142e-07,
                                         1.52723873638636e-09,
                                         3.28254191855234e-12,
                                         8.08230968072267e-16,
                                         3.51517039411794e-21,
                                         3.16439727633363e-31,
                                         5.51072708519591e-05,
                                         0.00607157048069568,
                                         0.0538794138478079,
                                         0.162270300926761,
                                         0.244634398816335,
                                         0.214720644634253,
                                         0.116805912475258,
                                         0.0400649619317964,
                                         0.008544921875,
                                         0.00108564156590568,
                                         7.60435105895017e-05,
                                         2.58476670210991e-06,
                                         3.45118789003644e-08,
                                         1.24905826167518e-10,
                                         5.92161957824887e-14,
                                         6.48747499818763e-19,
                                         3.11901149804758e-28,
                                         8.47955519926428e-07,
                                         0.000498955561113502,
                                         0.0111534182263626,
                                         0.0646780276999214,
                                         0.164189731790226,
                                         0.224808080837052,
                                         0.182266947602208,
                                         0.0907695679437284,
                                         0.02777099609375,
                                         0.00506148146630023,
                                         0.000514737472730379,
                                         2.60765394320757e-05,
                                         5.43133833132423e-07,
                                         3.3100281847677e-09,
                                         3.0214965346131e-12,
                                         8.33837826071012e-17,
                                         2.14101411028056e-25,
                                         9.63529732453155e-09,
                                         3.02796307488549e-05,
                                         0.00170498669126639,
                                         0.0190371694328724,
                                         0.081377125116415,
                                         0.17381126269328,
                                         0.210028821479341,
                                         0.151860100441726,
                                         0.066650390625,
                                         0.0174259602805857,
                                         0.00257298523748727,
                                         0.000194270314570255,
                                         6.31208721101008e-06,
                                         6.47751709817206e-08,
                                         1.13849618081934e-10,
                                         7.91435211774436e-15,
                                         1.08530040227817e-22,
                                         8.36348654576882e-11,
                                         1.40368440587198e-06,
                                         0.000199096759835134,
                                         0.00428033909095613,
                                         0.0308098004719252,
                                         0.102653581035784,
                                         0.184875798761805,
                                         0.194078438979089,
                                         0.1221923828125,
                                         0.0458295910296289,
                                         0.00982469317068129,
                                         0.00110558750902444,
                                         5.60362826876778e-05,
                                         9.6831272097993e-07,
                                         3.27696081869104e-09,
                                         5.73824831347938e-13,
                                         4.20252776632761e-20,
                                         5.65679095500828e-13,
                                         5.070477977523e-08,
                                         1.81162337786411e-05,
                                         0.000749919248380398,
                                         0.00908941544000033,
                                         0.0472422767005098,
                                         0.126806570321089,
                                         0.193273104740773,
                                         0.174560546875,
                                         0.0939194993594439,
                                         0.0292321812553761,
                                         0.0049027565499968,
                                         0.000387637844145617,
                                         1.12793281514244e-05,
                                         7.34973100651189e-08,
                                         3.24193148378449e-11,
                                         1.26803646944906e-17,
                                         3.01303002523764e-15,
                                         1.44237733319781e-09,
                                         1.29814200488693e-06,
                                         0.000103466888457561,
                                         0.00211170663305682,
                                         0.017121353582563,
                                         0.0684942338458014,
                                         0.151571000905351,
                                         0.196380615234375,
                                         0.151571000905351,
                                         0.0684942338458014,
                                         0.0171213535825629,
                                         0.00211170663305682,
                                         0.00010346688845756,
                                         1.29814200488693e-06,
                                         1.44237733319779e-09,
                                         3.01303002523792e-15,
                                         1.26803646944893e-17,
                                         3.24193148378456e-11,
                                         7.34973100651189e-08,
                                         1.12793281514244e-05,
                                         0.000387637844145618,
                                         0.00490275654999681,
                                         0.0292321812553761,
                                         0.0939194993594439,
                                         0.174560546875,
                                         0.193273104740773,
                                         0.126806570321089,
                                         0.0472422767005098,
                                         0.00908941544000031,
                                         0.000749919248380396,
                                         1.81162337786411e-05,
                                         5.07047797752292e-08,
                                         5.65679095500874e-13,
                                         4.20252776632711e-20,
                                         5.73824831347951e-13,
                                         3.27696081869104e-09,
                                         9.68312720979935e-07,
                                         5.60362826876779e-05,
                                         0.00110558750902445,
                                         0.00982469317068129,
                                         0.0458295910296289,
                                         0.1221923828125,
                                         0.194078438979089,
                                         0.184875798761805,
                                         0.102653581035783,
                                         0.0308098004719251,
                                         0.00428033909095611,
                                         0.000199096759835134,
                                         1.40368440587196e-06,
                                         8.36348654576941e-11,
                                         1.08530040227803e-22,
                                         7.91435211774456e-15,
                                         1.13849618081934e-10,
                                         6.47751709817209e-08,
                                         6.3120872110101e-06,
                                         0.000194270314570255,
                                         0.00257298523748727,
                                         0.0174259602805857,
                                         0.066650390625,
                                         0.151860100441726,
                                         0.210028821479341,
                                         0.17381126269328,
                                         0.0813771251164149,
                                         0.0190371694328723,
                                         0.00170498669126639,
                                         3.02796307488546e-05,
                                         9.63529732453211e-09,
                                         2.14101411028026e-25,
                                         8.33837826071034e-17,
                                         3.0214965346131e-12,
                                         3.31002818476772e-09,
                                         5.43133833132425e-07,
                                         2.60765394320758e-05,
                                         0.000514737472730379,
                                         0.00506148146630023,
                                         0.02777099609375,
                                         0.0907695679437284,
                                         0.182266947602208,
                                         0.224808080837051,
                                         0.164189731790226,
                                         0.0646780276999213,
                                         0.0111534182263626,
                                         0.000498955561113498,
                                         8.47955519926468e-07,
                                         3.1190114980471e-28,
                                         6.48747499818782e-19,
                                         5.92161957824887e-14,
                                         1.24905826167519e-10,
                                         3.45118789003645e-08,
                                         2.58476670210992e-06,
                                         7.60435105895017e-05,
                                         0.00108564156590568,
                                         0.008544921875,
                                         0.0400649619317964,
                                         0.116805912475258,
                                         0.214720644634252,
                                         0.244634398816334,
                                         0.162270300926761,
                                         0.0538794138478079,
                                         0.00607157048069564,
                                         5.5107270851961e-05,
                                         3.16439727633311e-31,
                                         3.51517039411805e-21,
                                         8.08230968072267e-16,
                                         3.28254191855237e-12,
                                         1.52723873638637e-09,
                                         1.78430595185143e-07,
                                         7.82375262550238e-06,
                                         0.000162170499604673,
                                         0.0018310546875,
                                         0.0123158894132072,
                                         0.0521312708578905,
                                         0.142827642087196,
                                         0.253843259436646,
                                         0.283529267001324,
                                         0.181265151071336,
                                         0.0514537218208546,
                                         0.00249414280133919,
                                         1.99760927535004e-34,
                                         1.18512070065883e-23,
                                         6.86397965411491e-18,
                                         5.3676400279513e-14,
                                         4.20523988986704e-11,
                                         7.66412916810548e-09,
                                         5.00856806488529e-07,
                                         1.50731063753278e-05,
                                         0.000244140625,
                                         0.00235565857286347,
                                         0.0144769580433245,
                                         0.0591148032376066,
                                         0.163892569560446,
                                         0.308249418688218,
                                         0.37944714526995,
                                         0.271317661491348,
                                         0.0702391495650709,
                                         5.9111417028511e-38,
                                         1.87292362148038e-26,
                                         2.73248488944918e-20,
                                         4.1143171470485e-16,
                                         5.42770081655611e-13,
                                         1.54311249436169e-10,
                                         1.50298044669401e-08,
                                         6.56712003211304e-07,
                                         1.52587890625e-05,
                                         0.000211203051617704,
                                         0.00188450632718153,
                                         0.0114688932629969,
                                         0.0496014273053605,
                                         0.157089775168795,
                                         0.372331296866457,
                                         0.670626355150675,
                                         0.927210742687717,

                                         0.934614301334455,
                                         0.699513704460112,
                                         0.41349522357963,
                                         0.191499682451008,
                                         0.0686499834790696,
                                         0.0187280594958175,
                                         0.00379883585759916,
                                         0.000555338579564985,
                                         5.60989635221588e-05,
                                         3.69799198017862e-06,
                                         1.46893468775079e-07,
                                         3.1368757953072e-09,
                                         3.0365546895791e-11,
                                         1.01715817220671e-13,
                                         7.41428576696425e-17,
                                         4.78001201812838e-21,
                                         3.21003841954288e-27,
                                         1.0017116263953e-38,
                                         0.063333617985664,
                                         0.252797826229707,
                                         0.375426898473198,
                                         0.333450881411405,
                                         0.200185295379457,
                                         0.0845728144487967,
                                         0.0253263984988457,
                                         0.00530983001172322,
                                         0.00075729138360549,
                                         7.0129017117133e-05,
                                         3.93296135467727e-06,
                                         1.2045210141755e-07,
                                         1.72140503758827e-09,
                                         8.92969698189816e-12,
                                         1.09004762912912e-14,
                                         1.34776466317143e-18,
                                         2.2739090377066e-24,
                                         3.78425857631906e-35,
                                         0.00201176595727118,
                                         0.0428244430776033,
                                         0.159779682818613,
                                         0.272167937396146,
                                         0.273630932243633,
                                         0.179023509784909,
                                         0.0791475770020145,
                                         0.0237982307586252,
                                         0.00479195216762387,
                                         0.000623405773513735,
                                         4.93603411190551e-05,
                                         2.16806710362241e-06,
                                         4.57431757528252e-08,
                                         3.67473674298645e-10,
                                         7.51212678585149e-13,
                                         1.78131376200445e-16,
                                         7.55052632355849e-22,
                                         6.70131717946563e-32,
                                         3.97618084730776e-05,
                                         0.00451393841854154,
                                         0.0423119695396452,
                                         0.138225309781564,
                                         0.232725367352019,
                                         0.235795150383727,
                                         0.153903086515404,
                                         0.0663673141564479,
                                         0.0188671998965904,
                                         0.00344817584754997,
                                         0.000385462508378915,
                                         2.42815595012007e-05,
                                         7.5633652360053e-07,
                                         9.40938544508291e-09,
                                         3.22126060363111e-11,
                                         1.46491456972443e-14,
                                         1.56000820211345e-19,
                                         7.38388733783218e-29,
                                         5.47307479549094e-07,
                                         0.000331356974420718,
                                         0.00780335757325903,
                                         0.0488894061297633,
                                         0.137847482480228,
                                         0.216289905108006,
                                         0.208417227960712,
                                         0.128896282914875,
                                         0.0517343512322887,
                                         0.0132826451672719,
                                         0.00209634479316977,
                                         0.000189389986247797,
                                         8.70924310925195e-06,
                                         1.67792624401773e-07,
                                         9.61978560799281e-10,
                                         8.38997667983509e-13,
                                         2.24467318021012e-17,
                                         5.66612993372663e-26,
                                         5.56319801349605e-09,
                                         1.79624049476475e-05,
                                         0.00106274153763295,
                                         0.0127693806253337,
                                         0.0602950742283115,
                                         0.146509407575828,
                                         0.208424026486908,
                                         0.184864885118489,
                                         0.104755888428733,
                                         0.0377839725703551,
                                         0.00841920657844905,
                                         0.0010908507374654,
                                         7.40582823024605e-05,
                                         2.20959335427842e-06,
                                         2.12145002721209e-08,
                                         3.54843692370306e-11,
                                         2.3851035240433e-15,
                                         3.21081939620737e-23,
                                         4.3196429063095e-11,
                                         7.43811764223588e-07,
                                         0.000110561518776446,
                                         0.00254773994438881,
                                         0.0201462872723551,
                                         0.0758097330680741,
                                         0.15921799149886,
                                         0.202534331149484,
                                         0.162034716900063,
                                         0.0821033460151241,
                                         0.0258291319218334,
                                         0.00479958668333836,
                                         0.000481057565609438,
                                         2.22270607408414e-05,
                                         3.57380137716192e-07,
                                         1.14641964306136e-09,
                                         1.93593844183266e-13,
                                         1.38987396345678e-20,
                                         2.61355533499669e-13,
                                         2.40006016119405e-08,
                                         8.96274130996574e-06,
                                         0.000396096390712235,
                                         0.00524528071867298,
                                         0.0305664461545889,
                                         0.0947757054501766,
                                         0.172903346958469,
                                         0.195298195318617,
                                         0.139019147537431,
                                         0.0617460215753954,
                                         0.016455189006637,
                                         0.00243489920986742,
                                         0.000174225709921648,
                                         4.69124962725842e-06,
                                         2.88609533643102e-08,
                                         1.22443681351467e-11,
                                         4.68808509464241e-18,
                                         1.24527742554854e-15,
                                         6.09862283759226e-10,
                                         5.72174224668871e-07,
                                         4.84950304587137e-05,
                                         0.0010754569037222,
                                         0.00970544584789525,
                                         0.0444275611002093,
                                         0.116240832779152,
                                         0.185369785788612,
                                         0.185369785788612,
                                         0.116240832779152,
                                         0.0444275611002093,
                                         0.00970544584789525,
                                         0.0010754569037222,
                                         4.84950304587139e-05,
                                         5.72174224668871e-07,
                                         6.09862283759226e-10,
                                         1.24527742554854e-15,
                                         4.68808509464241e-18,
                                         1.22443681351467e-11,
                                         2.88609533643102e-08,
                                         4.69124962725839e-06,
                                         0.000174225709921648,
                                         0.00243489920986742,
                                         0.016455189006637,
                                         0.0617460215753954,
                                         0.139019147537431,
                                         0.195298195318617,
                                         0.172903346958469,
                                         0.0947757054501766,
                                         0.0305664461545889,
                                         0.00524528071867298,
                                         0.000396096390712237,
                                         8.96274130996574e-06,
                                         2.40006016119405e-08,
                                         2.61355533499669e-13,
                                         1.38987396345678e-20,
                                         1.93593844183266e-13,
                                         1.14641964306136e-09,
                                         3.5738013771619e-07,
                                         2.22270607408414e-05,
                                         0.000481057565609438,
                                         0.00479958668333836,
                                         0.0258291319218334,
                                         0.0821033460151241,
                                         0.162034716900063,
                                         0.202534331149484,
                                         0.15921799149886,
                                         0.0758097330680741,
                                         0.0201462872723551,
                                         0.00254773994438882,
                                         0.000110561518776446,
                                         7.43811764223588e-07,
                                         4.3196429063095e-11,
                                         3.21081939620737e-23,
                                         2.3851035240433e-15,
                                         3.54843692370306e-11,
                                         2.12145002721208e-08,
                                         2.20959335427842e-06,
                                         7.40582823024605e-05,
                                         0.0010908507374654,
                                         0.00841920657844905,
                                         0.0377839725703551,
                                         0.104755888428733,
                                         0.184864885118489,
                                         0.208424026486908,
                                         0.146509407575828,
                                         0.0602950742283115,
                                         0.0127693806253338,
                                         0.00106274153763295,
                                         1.79624049476475e-05,
                                         5.56319801349605e-09,
                                         5.66612993372663e-26,
                                         2.24467318021012e-17,
                                         8.38997667983509e-13,
                                         9.61978560799275e-10,
                                         1.67792624401773e-07,
                                         8.70924310925195e-06,
                                         0.000189389986247797,
                                         0.00209634479316977,
                                         0.0132826451672719,
                                         0.0517343512322887,
                                         0.128896282914875,
                                         0.208417227960713,
                                         0.216289905108006,
                                         0.137847482480228,
                                         0.0488894061297634,
                                         0.00780335757325903,
                                         0.000331356974420718,
                                         5.47307479549094e-07,
                                         7.38388733783218e-29,
                                         1.56000820211345e-19,
                                         1.46491456972443e-14,
                                         3.22126060363109e-11,
                                         9.40938544508291e-09,
                                         7.5633652360053e-07,
                                         2.42815595012007e-05,
                                         0.000385462508378915,
                                         0.00344817584754997,
                                         0.0188671998965904,
                                         0.0663673141564479,
                                         0.153903086515404,
                                         0.235795150383727,
                                         0.232725367352019,
                                         0.138225309781565,
                                         0.0423119695396452,
                                         0.00451393841854154,
                                         3.97618084730776e-05,
                                         6.70131717946563e-32,
                                         7.55052632355849e-22,
                                         1.78131376200445e-16,
                                         7.51212678585143e-13,
                                         3.67473674298645e-10,
                                         4.57431757528252e-08,
                                         2.16806710362241e-06,
                                         4.93603411190551e-05,
                                         0.000623405773513735,
                                         0.00479195216762387,
                                         0.0237982307586252,
                                         0.0791475770020145,
                                         0.179023509784909,
                                         0.273630932243633,
                                         0.272167937396146,
                                         0.159779682818613,
                                         0.0428244430776033,
                                         0.00201176595727118,
                                         3.78425857631906e-35,
                                         2.2739090377066e-24,
                                         1.34776466317143e-18,
                                         1.09004762912911e-14,
                                         8.92969698189816e-12,
                                         1.72140503758827e-09,
                                         1.2045210141755e-07,
                                         3.93296135467727e-06,
                                         7.0129017117133e-05,
                                         0.00075729138360549,
                                         0.00530983001172322,
                                         0.0253263984988457,
                                         0.0845728144487967,
                                         0.200185295379457,
                                         0.333450881411406,
                                         0.375426898473198,
                                         0.252797826229707,
                                         0.063333617985664,
                                         1.0017116263953e-38,
                                         3.21003841954288e-27,
                                         4.78001201812838e-21,
                                         7.41428576696418e-17,
                                         1.01715817220671e-13,
                                         3.0365546895791e-11,
                                         3.1368757953072e-09,
                                         1.46893468775079e-07,
                                         3.69799198017862e-06,
                                         5.60989635221588e-05,
                                         0.000555338579564985,
                                         0.00379883585759916,
                                         0.0187280594958175,
                                         0.0686499834790696,
                                         0.191499682451008,
                                         0.41349522357963,
                                         0.699513704460112,
                                         0.934614301334455,

                                         0.940954160687561,
                                         0.725028578560002,
                                         0.451970337289636,
                                         0.226474360377207,
                                         0.0903394476408069,
                                         0.0283013802499101,
                                         0.00683731720881851,
                                         0.00124334658760721,
                                         0.000164814701684538,
                                         1.52587890625e-05,
                                         9.31153322346743e-07,
                                         3.45688297347333e-08,
                                         6.96231491162907e-10,
                                         6.41210396746884e-12,
                                         2.06077060273875e-14,
                                         1.45306619336275e-17,
                                         9.13462009198251e-22,
                                         6.02872937925183e-28,
                                         1.8632767807848e-39,
                                         0.0573763287369306,
                                         0.235487047552902,
                                         0.367983800587927,
                                         0.352446475336154,
                                         0.234359157032781,
                                         0.113012675791981,
                                         0.039994240831611,
                                         0.010326852378028,
                                         0.00190817195853379,
                                         0.000244140625,
                                         2.05892061393383e-05,
                                         1.06548877995044e-06,
                                         3.04706626993722e-08,
                                         4.11074918598922e-10,
                                         2.03359806216518e-12,
                                         2.39029351965877e-15,
                                         2.87218022437887e-19,
                                         4.75175297865699e-25,
                                         7.82263466424008e-36,
                                         0.0016399791480624,
                                         0.0358525627504326,
                                         0.140439329949143,
                                         0.257103851423535,
                                         0.284988742041727,
                                         0.21153779830398,
                                         0.10966056185983,
                                         0.0402054578070786,
                                         0.0103557212265868,
                                         0.0018310546875,
                                         0.000213402394009643,
                                         1.5394102173907e-05,
                                         6.25102761117108e-07,
                                         1.23532913464149e-08,
                                         9.406799101964e-11,
                                         1.84314010958579e-13,
                                         4.23325242913988e-17,
                                         1.75559042091773e-22,
                                         1.53946565705261e-32,
                                         2.91668423113052e-05,
                                         0.0033964005461752,
                                         0.0333498829882953,
                                         0.116699638491554,
                                         0.215634873824276,
                                         0.246373685141605,
                                         0.187089318732026,
                                         0.097397446735078,
                                         0.0349694425949732,
                                         0.008544921875,
                                         0.00137627268374806,
                                         0.000138390209338644,
                                         7.97933047397891e-06,
                                         2.30988259488991e-07,
                                         2.70747301188682e-09,
                                         8.84323128775033e-12,
                                         3.88223805539714e-15,
                                         4.03587834805977e-20,
                                         1.88509169174001e-29,
                                         3.61257673886627e-07,
                                         0.00022407539956664,
                                         0.00551539295774988,
                                         0.0368898610145788,
                                         0.113628395368943,
                                         0.199837615602433,
                                         0.222292136722547,
                                         0.164318594845121,
                                         0.0822382099869739,
                                         0.02777099609375,
                                         0.00618139181916603,
                                         0.00086642893579931,
                                         7.09345907402787e-05,
                                         3.00797144215028e-06,
                                         5.42703951219017e-08,
                                         2.95488237406376e-10,
                                         2.47951499244589e-13,
                                         6.46144290999406e-18,
                                         1.60757607366551e-26,
                                         3.30424795196912e-09,
                                         1.09168498663845e-05,
                                         0.000673575817331153,
                                         0.00861137313860193,
                                         0.0442163235149517,
                                         0.119698315755424,
                                         0.195041526805026,
                                         0.204716917492353,
                                         0.142819158072607,
                                         0.066650390625,
                                         0.0205019861936501,
                                         0.0040057921405031,
                                         0.000465669252741248,
                                         2.89258008448492e-05,
                                         8.03322093731572e-07,
                                         7.29117113419275e-09,
                                         1.1694433685761e-11,
                                         7.63921682228418e-16,
                                         1.01236813947433e-23,
                                         2.30865145439374e-11,
                                         4.06284906898512e-07,
                                         6.28386297532521e-05,
                                         0.0015355642411328,
                                         0.0131434257983981,
                                         0.0547682712933988,
                                         0.130725487068886,
                                         0.194827809547614,
                                         0.189465212529308,
                                         0.1221923828125,
                                         0.0519440482732689,
                                         0.01414731636672,
                                         0.00233521704598744,
                                         0.000212484506139259,
                                         9.08336221022075e-06,
                                         1.37430959726965e-07,
                                         4.21329478222466e-10,
                                         6.89919396550637e-14,
                                         4.87007441269081e-21,
                                         1.25691131938633e-13,
                                         1.17821504878837e-08,
                                         4.56801426137797e-06,
                                         0.000213365450673218,
                                         0.00304435330052012,
                                         0.0195267763341046,
                                         0.0682737841072616,
                                         0.144480317445741,
                                         0.195854258693314,
                                         0.174560546875,
                                         0.10255012059694,
                                         0.0389332130244094,
                                         0.00912509650695873,
                                         0.0012162690880609,
                                         8.00320753284001e-05,
                                         2.01851690986987e-06,
                                         1.18283732569921e-08,
                                         4.85521424250695e-12,
                                         1.82554797466698e-18,
                                         5.38891664942933e-16,
                                         2.69072308280773e-10,
                                         2.61504254144238e-07,
                                         2.33469830035106e-05,
                                         0.000555305643972204,
                                         0.00548255040163892,
                                         0.028080073667437,
                                         0.0843756223248247,
                                         0.159436267915776,
                                         0.196380615234375,
                                         0.159436267915776,
                                         0.0843756223248248,
                                         0.028080073667437,
                                         0.0054825504016389,
                                         0.000555305643972206,
                                         2.33469830035106e-05,
                                         2.61504254144238e-07,
                                         2.69072308280773e-10,
                                         5.38891664942933e-16,
                                         1.82554797466698e-18,
                                         4.85521424250695e-12,
                                         1.18283732569921e-08,
                                         2.01851690986987e-06,
                                         8.00320753283998e-05,
                                         0.0012162690880609,
                                         0.00912509650695873,
                                         0.0389332130244093,
                                         0.10255012059694,
                                         0.174560546875,
                                         0.195854258693314,
                                         0.144480317445741,
                                         0.0682737841072616,
                                         0.0195267763341046,
                                         0.00304435330052013,
                                         0.000213365450673218,
                                         4.56801426137797e-06,
                                         1.17821504878837e-08,
                                         1.25691131938633e-13,
                                         4.87007441269081e-21,
                                         6.89919396550637e-14,
                                         4.21329478222466e-10,
                                         1.37430959726965e-07,
                                         9.08336221022071e-06,
                                         0.00021248450613926,
                                         0.00233521704598744,
                                         0.01414731636672,
                                         0.0519440482732689,
                                         0.1221923828125,
                                         0.189465212529308,
                                         0.194827809547614,
                                         0.130725487068886,
                                         0.0547682712933987,
                                         0.0131434257983982,
                                         0.0015355642411328,
                                         6.28386297532521e-05,
                                         4.06284906898512e-07,
                                         2.30865145439374e-11,
                                         1.01236813947433e-23,
                                         7.63921682228418e-16,
                                         1.1694433685761e-11,
                                         7.29117113419275e-09,
                                         8.03322093731568e-07,
                                         2.89258008448493e-05,
                                         0.000465669252741248,
                                         0.00400579214050309,
                                         0.0205019861936501,
                                         0.066650390625,
                                         0.142819158072607,
                                         0.204716917492354,
                                         0.195041526805026,
                                         0.119698315755424,
                                         0.0442163235149518,
                                         0.00861137313860193,
                                         0.000673575817331153,
                                         1.09168498663845e-05,
                                         3.30424795196912e-09,
                                         1.60757607366551e-26,
                                         6.46144290999406e-18,
                                         2.47951499244589e-13,
                                         2.95488237406376e-10,
                                         5.42703951219015e-08,
                                         3.00797144215029e-06,
                                         7.09345907402787e-05,
                                         0.000866428935799309,
                                         0.00618139181916603,
                                         0.02777099609375,
                                         0.0822382099869739,
                                         0.164318594845121,
                                         0.222292136722547,
                                         0.199837615602433,
                                         0.113628395368943,
                                         0.0368898610145788,
                                         0.00551539295774988,
                                         0.00022407539956664,
                                         3.61257673886627e-07,
                                         1.88509169174001e-29,
                                         4.03587834805977e-20,
                                         3.88223805539714e-15,
                                         8.84323128775033e-12,
                                         2.70747301188681e-09,
                                         2.30988259488992e-07,
                                         7.97933047397891e-06,
                                         0.000138390209338643,
                                         0.00137627268374806,
                                         0.008544921875,
                                         0.0349694425949732,
                                         0.097397446735078,
                                         0.187089318732026,
                                         0.246373685141605,
                                         0.215634873824276,
                                         0.116699638491554,
                                         0.0333498829882953,
                                         0.0033964005461752,
                                         2.91668423113052e-05,
                                         1.53946565705261e-32,
                                         1.75559042091773e-22,
                                         4.23325242913988e-17,
                                         1.84314010958579e-13,
                                         9.40679910196395e-11,
                                         1.23532913464149e-08,
                                         6.25102761117108e-07,
                                         1.5394102173907e-05,
                                         0.000213402394009643,
                                         0.0018310546875,
                                         0.0103557212265868,
                                         0.0402054578070786,
                                         0.10966056185983,
                                         0.21153779830398,
                                         0.284988742041728,
                                         0.257103851423535,
                                         0.140439329949143,
                                         0.0358525627504326,
                                         0.0016399791480624,
                                         7.82263466424008e-36,
                                         4.75175297865699e-25,
                                         2.87218022437887e-19,
                                         2.39029351965877e-15,
                                         2.03359806216517e-12,
                                         4.11074918598924e-10,
                                         3.04706626993722e-08,
                                         1.06548877995044e-06,
                                         2.05892061393383e-05,
                                         0.000244140625,
                                         0.00190817195853379,
                                         0.010326852378028,
                                         0.039994240831611,
                                         0.113012675791981,
                                         0.234359157032781,
                                         0.352446475336154,
                                         0.367983800587927,
                                         0.235487047552902,
                                         0.0573763287369306,
                                         1.8632767807848e-39,
                                         6.02872937925183e-28,
                                         9.13462009198251e-22,
                                         1.45306619336275e-17,
                                         2.06077060273874e-14,
                                         6.41210396746887e-12,
                                         6.96231491162907e-10,
                                         3.45688297347333e-08,
                                         9.31153322346743e-07,
                                         1.52587890625e-05,
                                         0.000164814701684538,
                                         0.00124334658760721,
                                         0.00683731720881851,
                                         0.0283013802499101,
                                         0.0903394476408069,
                                         0.226474360377207,
                                         0.451970337289636,
                                         0.725028578560002,
                                         0.940954160687561,

                                         0.946422818591847,
                                         0.747626146553541,
                                         0.487728572160834,
                                         0.261378405540361,
                                         0.114169129955771,
                                         0.0402050732205766,
                                         0.0112496716605756,
                                         0.00245369874267374,
                                         0.000406894723818858,
                                         4.96496058769678e-05,
                                         4.26885014407162e-06,
                                         2.43963100770707e-07,
                                         8.55009820154111e-09,
                                         1.63814170323321e-10,
                                         1.44583047413848e-12,
                                         4.4851150572262e-15,
                                         3.07385786794948e-18,
                                         1.89106759002151e-22,
                                         1.22963597424147e-28,
                                         3.76916007660222e-40,
                                         0.0522053662133039,
                                         0.219437182805739,
                                         0.358163424830747,
                                         0.36583947646961,
                                         0.265343058142375,
                                         0.143100325737498,
                                         0.0582720847757079,
                                         0.0178988689972275,
                                         0.00409465364060444,
                                         0.000681452331935532,
                                         7.9621654546759e-05,
                                         6.20624615634926e-06,
                                         3.00059042489649e-07,
                                         8.09600416673837e-09,
                                         1.03991435820218e-10,
                                         4.94031356913938e-13,
                                         5.62215591085786e-16,
                                         6.59240652978535e-20,
                                         1.07248391677675e-25,
                                         1.74926096053939e-36,
                                         0.00134985267415953,
                                         0.0301909818703945,
                                         0.123289346596273,
                                         0.240022917396691,
                                         0.28907334615494,
                                         0.238749058722229,
                                         0.141488957129159,
                                         0.0612027898321752,
                                         0.01931494897711,
                                         0.00438426139509961,
                                         0.000696133874477838,
                                         7.40073970808473e-05,
                                         4.9360962107636e-06,
                                         1.87556128782458e-07,
                                         3.50605767240612e-09,
                                         2.55080184503501e-11,
                                         4.82018013537613e-14,
                                         1.07726410938651e-17,
                                         4.38476473871034e-23,
                                         3.8054489999255e-33,
                                         2.17171641218054e-05,
                                         0.00258457877355285,
                                         0.0264067762274181,
                                         0.0979851746323271,
                                         0.195953888791061,
                                         0.247849605111429,
                                         0.213761777802431,
                                         0.130215405164558,
                                         0.0566911773716504,
                                         0.0175510423590296,
                                         0.00378703966924426,
                                         0.000549119391341681,
                                         5.05249660787562e-05,
                                         2.70356810908101e-06,
                                         7.3550571960923e-08,
                                         8.19491488744549e-10,
                                         2.5713972178968e-12,
                                         1.09533236026472e-15,
                                         1.11544284641243e-20,
                                         5.1511325537655e-30,
                                         2.43330436752194e-07,
                                         0.000154091573218359,
                                         0.00393896207653449,
                                         0.0278576586057025,
                                         0.0925073635339703,
                                         0.179189011872345,
                                         0.224912783260811,
                                         0.192943474051577,
                                         0.1158814625805,
                                         0.0489312115746786,
                                         0.0143477407688772,
                                         0.00283749438661426,
                                         0.000360167917846351,
                                         2.71406315755474e-05,
                                         1.07455743391872e-06,
                                         1.83353309048998e-08,
                                         9.55326117021495e-11,
                                         7.75615047656747e-14,
                                         1.97617285127649e-18,
                                         4.85597145835346e-27,
                                         2.01334239495495e-09,
                                         6.78415693666204e-06,
                                         0.000433886456977386,
                                         0.00584866482308095,
                                         0.0322497685017411,
                                         0.0956670536085092,
                                         0.17475359062138,
                                         0.211118213445957,
                                         0.174920350962224,
                                         0.100738830579914,
                                         0.0401416360543537,
                                         0.0108275730479044,
                                         0.00189597191781354,
                                         0.000201201153010626,
                                         1.15931403186547e-05,
                                         3.02943011188292e-07,
                                         2.62096977463317e-09,
                                         4.05578023632192e-12,
                                         2.58541590051352e-16,
                                         3.38047248921118e-24,
                                         1.27253294451546e-11,
                                         2.28161874146508e-07,
                                         3.65090542324308e-05,
                                         0.000937991813594839,
                                         0.00858829774277436,
                                         0.0390160784365831,
                                         0.103721375816157,
                                         0.176462124372297,
                                         0.201695823409499,
                                         0.158430223124904,
                                         0.0857900280543086,
                                         0.0315614848356958,
                                         0.00762410675633916,
                                         0.00113938677365762,
                                         9.55438516322673e-05,
                                         3.82351956306462e-06,
                                         5.49291106581463e-08,
                                         1.62006632963048e-10,
                                         2.58384284431306e-14,
                                         1.79766548247074e-21,
                                         6.26730681137761e-14,
                                         5.97930614848534e-09,
                                         2.39378750651465e-06,
                                         0.000117220046117789,
                                         0.00178216581289454,
                                         0.012398962827233,
                                         0.047970136587833,
                                         0.11493117047648,
                                         0.181223283776153,
                                         0.194151025058819,
                                         0.142869352292158,
                                         0.0716876355376998,
                                         0.0238894726204687,
                                         0.00502773533007612,
                                         0.000613571138127486,
                                         3.76033225980863e-05,
                                         8.97023142507398e-07,
                                         5.04256729415731e-09,
                                         2.01215902224486e-12,
                                         7.44904980885885e-19,
                                         2.43076760286006e-16,
                                         1.23398269075211e-10,
                                         1.23600768716673e-07,
                                         1.15360011382903e-05,
                                         0.000291232398471078,
                                         0.00310297061401944,
                                         0.0174712590735691,
                                         0.058948766033157,
                                         0.128227639752846,
                                         0.187366482751634,
                                         0.187366482751634,
                                         0.128227639752846,
                                         0.058948766033157,
                                         0.0174712590735691,
                                         0.00310297061401943,
                                         0.000291232398471078,
                                         1.15360011382904e-05,
                                         1.23600768716673e-07,
                                         1.23398269075214e-10,
                                         2.43076760286037e-16,
                                         7.44904980885777e-19,
                                         2.0121590222448e-12,
                                         5.04256729415731e-09,
                                         8.97023142507392e-07,
                                         3.76033225980863e-05,
                                         0.000613571138127487,
                                         0.00502773533007612,
                                         0.0238894726204687,
                                         0.0716876355376998,
                                         0.142869352292158,
                                         0.194151025058819,
                                         0.181223283776153,
                                         0.11493117047648,
                                         0.047970136587833,
                                         0.0123989628272329,
                                         0.00178216581289454,
                                         0.00011722004611779,
                                         2.39378750651465e-06,
                                         5.97930614848547e-09,
                                         6.26730681137831e-14,
                                         1.79766548247045e-21,
                                         2.58384284431298e-14,
                                         1.62006632963048e-10,
                                         5.49291106581459e-08,
                                         3.82351956306462e-06,
                                         9.55438516322676e-05,
                                         0.00113938677365762,
                                         0.00762410675633916,
                                         0.0315614848356958,
                                         0.0857900280543086,
                                         0.158430223124904,
                                         0.201695823409499,
                                         0.176462124372297,
                                         0.103721375816157,
                                         0.039016078436583,
                                         0.00858829774277436,
                                         0.000937991813594843,
                                         3.65090542324308e-05,
                                         2.28161874146512e-07,
                                         1.27253294451558e-11,
                                         3.38047248921058e-24,
                                         2.58541590051343e-16,
                                         4.05578023632192e-12,
                                         2.62096977463315e-09,
                                         3.02943011188292e-07,
                                         1.15931403186547e-05,
                                         0.000201201153010626,
                                         0.00189597191781354,
                                         0.0108275730479044,
                                         0.0401416360543537,
                                         0.100738830579914,
                                         0.174920350962224,
                                         0.211118213445957,
                                         0.17475359062138,
                                         0.0956670536085091,
                                         0.0322497685017411,
                                         0.00584866482308097,
                                         0.000433886456977386,
                                         6.78415693666214e-06,
                                         2.01334239495511e-09,
                                         4.85597145835252e-27,
                                         1.97617285127642e-18,
                                         7.75615047656747e-14,
                                         9.55326117021488e-11,
                                         1.83353309048997e-08,
                                         1.07455743391872e-06,
                                         2.71406315755474e-05,
                                         0.000360167917846351,
                                         0.00283749438661426,
                                         0.0143477407688772,
                                         0.0489312115746786,
                                         0.1158814625805,
                                         0.192943474051577,
                                         0.224912783260811,
                                         0.179189011872344,
                                         0.0925073635339703,
                                         0.0278576586057026,
                                         0.00393896207653449,
                                         0.00015409157321836,
                                         2.4333043675221e-07,
                                         5.15113255376442e-30,
                                         1.11544284641238e-20,
                                         1.09533236026472e-15,
                                         2.57139721789678e-12,
                                         8.19491488744549e-10,
                                         7.35505719609233e-08,
                                         2.70356810908101e-06,
                                         5.05249660787562e-05,
                                         0.000549119391341681,
                                         0.00378703966924426,
                                         0.0175510423590296,
                                         0.0566911773716504,
                                         0.130215405164558,
                                         0.213761777802431,
                                         0.247849605111429,
                                         0.195953888791061,
                                         0.0979851746323273,
                                         0.0264067762274181,
                                         0.00258457877355287,
                                         2.17171641218065e-05,
                                         3.80544899992464e-33,
                                         4.38476473871016e-23,
                                         1.07726410938651e-17,
                                         4.82018013537608e-14,
                                         2.55080184503501e-11,
                                         3.50605767240613e-09,
                                         1.87556128782458e-07,
                                         4.9360962107636e-06,
                                         7.40073970808473e-05,
                                         0.000696133874477838,
                                         0.00438426139509961,
                                         0.01931494897711,
                                         0.0612027898321752,
                                         0.141488957129159,
                                         0.238749058722229,
                                         0.28907334615494,
                                         0.240022917396691,
                                         0.123289346596273,
                                         0.0301909818703946,
                                         0.00134985267415957,
                                         1.74926096053897e-36,
                                         1.0724839167767e-25,
                                         6.59240652978535e-20,
                                         5.62215591085781e-16,
                                         4.94031356913938e-13,
                                         1.03991435820218e-10,
                                         8.09600416673837e-09,
                                         3.00059042489649e-07,
                                         6.20624615634926e-06,
                                         7.9621654546759e-05,
                                         0.000681452331935532,
                                         0.00409465364060444,
                                         0.0178988689972275,
                                         0.0582720847757079,
                                         0.143100325737498,
                                         0.265343058142375,
                                         0.36583947646961,
                                         0.358163424830747,
                                         0.219437182805739,
                                         0.0522053662133047,
                                         3.76916007660124e-40,
                                         1.22963597424141e-28,
                                         1.89106759002151e-22,
                                         3.07385786794945e-18,
                                         4.4851150572262e-15,
                                         1.44583047413848e-12,
                                         1.63814170323321e-10,
                                         8.55009820154111e-09,
                                         2.43963100770707e-07,
                                         4.26885014407162e-06,
                                         4.96496058769678e-05,
                                         0.000406894723818858,
                                         0.00245369874267374,
                                         0.0112496716605756,
                                         0.0402050732205766,
                                         0.114169129955771,
                                         0.261378405540361,
                                         0.487728572160834,
                                         0.747626146553541,
                                         0.946422818591847,

                                         0.951171526113385,
                                         0.767699259844556,
                                         0.520833873380316,
                                         0.295725826727182,
                                         0.139625647118079,
                                         0.0543335689538597,
                                         0.0172218879260235,
                                         0.00437873987593309,
                                         0.00087556598279396,
                                         0.000134219856099712,
                                         1.52587890625e-05,
                                         1.23143114090047e-06,
                                         6.65260786159869e-08,
                                         2.21896948662515e-09,
                                         4.07265697207507e-11,
                                         3.46521969554413e-13,
                                         1.04264825260625e-15,
                                         6.97268208074425e-19,
                                         4.21042994246248e-23,
                                         2.70279722952649e-29,
                                         8.22596098329803e-41,
                                         0.0476910419938618,
                                         0.204632684200257,
                                         0.346773503140881,
                                         0.374361475494821,
                                         0.292521957107098,
                                         0.173571982739957,
                                         0.0796274959070065,
                                         0.0283151759946626,
                                         0.00774377637823384,
                                         0.00160176513649202,
                                         0.000244140625,
                                         2.64160592157279e-05,
                                         1.92560837907856e-06,
                                         8.78458352212728e-08,
                                         2.25494263390231e-09,
                                         2.77689429340966e-11,
                                         1.27404425422142e-13,
                                         1.41006110617486e-16,
                                         1.61889889513099e-20,
                                         2.59578997766666e-26,
                                         4.19999371158513e-37,
                                         0.00112087210877165,
                                         0.0255682003560635,
                                         0.108226564013474,
                                         0.22214386657198,
                                         0.287271817834675,
                                         0.25991557025959,
                                         0.172578479735427,
                                         0.0858283305734166,
                                         0.0321039157319946,
                                         0.00896029246471664,
                                         0.0018310546875,
                                         0.000265623935935191,
                                         2.61267440156437e-05,
                                         1.63016777302973e-06,
                                         5.85240596278914e-08,
                                         1.04310782318778e-09,
                                         7.29747285483698e-12,
                                         1.33664871090137e-14,
                                         2.91779172696353e-18,
                                         1.16860278836968e-23,
                                         1.00519869425272e-33,
                                         1.63915805137528e-05,
                                         0.00198779154149908,
                                         0.0210168368599255,
                                         0.082020614904888,
                                         0.175538786738064,
                                         0.242175656938089,
                                         0.232731804566063,
                                         0.161877907805758,
                                         0.0828149469532481,
                                         0.0311882532071889,
                                         0.008544921875,
                                         0.00166192684653939,
                                         0.000220570857128826,
                                         1.88230045720605e-05,
                                         9.45102491323921e-07,
                                         2.43806104095659e-08,
                                         2.60079415423712e-10,
                                         7.88391896537922e-13,
                                         3.27215876930204e-16,
                                         3.27348083581285e-21,
                                         1.49692730041354e-30,
                                         1.66940680701907e-07,
                                         0.000107626222039582,
                                         0.00284234908851021,
                                         0.0210905689145764,
                                         0.0747015721772033,
                                         0.157146696675245,
                                         0.218575467049305,
                                         0.212628306485964,
                                         0.148777079511774,
                                         0.0756025593231272,
                                         0.02777099609375,
                                         0.00724157743832729,
                                         0.00129684344601015,
                                         0.000151363852283475,
                                         1.06291846545837e-05,
                                         3.96859267881488e-07,
                                         6.45529438564941e-09,
                                         3.2384977498518e-11,
                                         2.55558910228519e-14,
                                         6.38600519162894e-19,
                                         1.55248024356267e-27,
                                         1.25554247503139e-09,
                                         4.30321686986228e-06,
                                         0.000283867294625511,
                                         0.00400480569380735,
                                         0.0234754687372438,
                                         0.0753022420065575,
                                         0.151591543121771,
                                         0.206244539188829,
                                         0.197374576545822,
                                         0.135334905660372,
                                         0.066650390625,
                                         0.0233014171973414,
                                         0.00563060229448909,
                                         0.000898841834439164,
                                         8.82772664341118e-05,
                                         4.77041717304648e-06,
                                         1.18318867855769e-07,
                                         9.82365107132506e-10,
                                         1.47392561946249e-12,
                                         9.19976243093628e-17,
                                         1.18899310492034e-24,
                                         7.21324746900935e-12,
                                         1.31431227820233e-07,
                                         2.1656261905405e-05,
                                         0.000580904575995423,
                                         0.005635455129514,
                                         0.027563905380476,
                                         0.0803116827650726,
                                         0.152817830433461,
                                         0.200021456943105,
                                         0.185060335217058,
                                         0.1221923828125,
                                         0.0572745499504894,
                                         0.0186746446866543,
                                         0.00407731791761258,
                                         0.00056005153790643,
                                         4.3803255179893e-05,
                                         1.65661714372216e-06,
                                         2.27631547024814e-08,
                                         6.49367129269569e-11,
                                         1.0124046957908e-14,
                                         6.95605141063344e-22,
                                         3.22916903854403e-14,
                                         3.12798326945525e-09,
                                         1.28739600326147e-06,
                                         6.56581543663494e-05,
                                         0.00105415444651221,
                                         0.00786201954667529,
                                         0.0331545397041211,
                                         0.0882320004982004,
                                         0.157951038839683,
                                         0.197186595901011,
                                         0.174560546875,
                                         0.10969871999182,
                                         0.0482625614016068,
                                         0.014412071205569,
                                         0.00276864731818934,
                                         0.000313412949736695,
                                         1.80738561992107e-05,
                                         4.11010102508916e-07,
                                         2.22928482379906e-09,
                                         8.68144892224706e-13,
                                         3.17107676379719e-19,
                                         1.13841678389377e-16,
                                         5.86247279532184e-11,
                                         6.0268638475413e-08,
                                         5.84417357460806e-06,
                                         0.00015528519660006,
                                         0.00176594856636019,
                                         0.010778487995806,
                                         0.0401170310967211,
                                         0.0982243015489827,
                                         0.165459610383126,
                                         0.196380615234375,
                                         0.165459610383126,
                                         0.0982243015489827,
                                         0.0401170310967212,
                                         0.010778487995806,
                                         0.00176594856636019,
                                         0.00015528519660006,
                                         5.84417357460806e-06,
                                         6.02686384754123e-08,
                                         5.862472795322e-11,
                                         1.1384167838936e-16,
                                         3.1710767637977e-19,
                                         8.68144892224679e-13,
                                         2.22928482379909e-09,
                                         4.11010102508916e-07,
                                         1.80738561992107e-05,
                                         0.000313412949736695,
                                         0.00276864731818934,
                                         0.0144120712055689,
                                         0.0482625614016068,
                                         0.10969871999182,
                                         0.174560546875,
                                         0.197186595901011,
                                         0.157951038839683,
                                         0.0882320004982006,
                                         0.0331545397041211,
                                         0.00786201954667529,
                                         0.00105415444651221,
                                         6.56581543663494e-05,
                                         1.28739600326146e-06,
                                         3.12798326945532e-09,
                                         3.22916903854363e-14,
                                         6.95605141063468e-22,
                                         1.01240469579076e-14,
                                         6.49367129269578e-11,
                                         2.27631547024814e-08,
                                         1.65661714372216e-06,
                                         4.3803255179893e-05,
                                         0.00056005153790643,
                                         0.00407731791761257,
                                         0.0186746446866543,
                                         0.0572745499504894,
                                         0.1221923828125,
                                         0.185060335217058,
                                         0.200021456943105,
                                         0.152817830433461,
                                         0.0803116827650726,
                                         0.027563905380476,
                                         0.005635455129514,
                                         0.000580904575995423,
                                         2.16562619054048e-05,
                                         1.31431227820236e-07,
                                         7.21324746900859e-12,
                                         1.18899310492057e-24,
                                         9.19976243093594e-17,
                                         1.47392561946252e-12,
                                         9.82365107132506e-10,
                                         1.18318867855769e-07,
                                         4.77041717304648e-06,
                                         8.82772664341118e-05,
                                         0.000898841834439162,
                                         0.00563060229448909,
                                         0.0233014171973414,
                                         0.066650390625,
                                         0.135334905660372,
                                         0.197374576545822,
                                         0.206244539188829,
                                         0.151591543121771,
                                         0.0753022420065575,
                                         0.0234754687372438,
                                         0.00400480569380735,
                                         0.000283867294625509,
                                         4.30321686986235e-06,
                                         1.25554247503128e-09,
                                         1.55248024356301e-27,
                                         6.38600519162868e-19,
                                         2.55558910228524e-14,
                                         3.2384977498518e-11,
                                         6.45529438564941e-09,
                                         3.96859267881488e-07,
                                         1.06291846545837e-05,
                                         0.000151363852283475,
                                         0.00129684344601015,
                                         0.00724157743832729,
                                         0.02777099609375,
                                         0.0756025593231272,
                                         0.148777079511774,
                                         0.212628306485964,
                                         0.218575467049305,
                                         0.157146696675245,
                                         0.0747015721772033,
                                         0.0210905689145764,
                                         0.00284234908851019,
                                         0.000107626222039583,
                                         1.66940680701895e-07,
                                         1.49692730041388e-30,
                                         3.27348083581271e-21,
                                         3.2721587693021e-16,
                                         7.88391896537921e-13,
                                         2.60079415423712e-10,
                                         2.43806104095659e-08,
                                         9.45102491323921e-07,
                                         1.88230045720605e-05,
                                         0.000220570857128826,
                                         0.00166192684653939,
                                         0.008544921875,
                                         0.0311882532071889,
                                         0.0828149469532481,
                                         0.161877907805758,
                                         0.232731804566063,
                                         0.242175656938089,
                                         0.175538786738064,
                                         0.082020614904888,
                                         0.0210168368599255,
                                         0.0019877915414991,
                                         1.6391580513752e-05,
                                         1.00519869425297e-33,
                                         1.16860278836963e-23,
                                         2.91779172696359e-18,
                                         1.33664871090137e-14,
                                         7.29747285483698e-12,
                                         1.04310782318778e-09,
                                         5.85240596278914e-08,
                                         1.63016777302973e-06,
                                         2.61267440156437e-05,
                                         0.000265623935935191,
                                         0.0018310546875,
                                         0.00896029246471664,
                                         0.0321039157319946,
                                         0.0858283305734167,
                                         0.172578479735427,
                                         0.25991557025959,
                                         0.287271817834675,
                                         0.22214386657198,
                                         0.108226564013474,
                                         0.0255682003560636,
                                         0.00112087210877161,
                                         4.19999371158625e-37,
                                         2.59578997766653e-26,
                                         1.61889889513102e-20,
                                         1.41006110617486e-16,
                                         1.27404425422142e-13,
                                         2.77689429340966e-11,
                                         2.25494263390231e-09,
                                         8.78458352212725e-08,
                                         1.92560837907856e-06,
                                         2.64160592157279e-05,
                                         0.000244140625,
                                         0.00160176513649202,
                                         0.00774377637823384,
                                         0.0283151759946626,
                                         0.0796274959070065,
                                         0.173571982739957,
                                         0.292521957107098,
                                         0.374361475494821,
                                         0.34677350314088,
                                         0.204632684200258,
                                         0.0476910419938609,
                                         8.22596098330037e-41,
                                         2.70279722952635e-29,
                                         4.21042994246257e-23,
                                         6.97268208074425e-19,
                                         1.04264825260625e-15,
                                         3.46521969554413e-13,
                                         4.07265697207507e-11,
                                         2.21896948662515e-09,
                                         6.65260786159869e-08,
                                         1.23143114090047e-06,
                                         1.52587890625e-05,
                                         0.000134219856099712,
                                         0.00087556598279396,
                                         0.00437873987593309,
                                         0.0172218879260235,
                                         0.0543335689538597,
                                         0.139625647118079,
                                         0.295725826727182,
                                         0.520833873380316,
                                         0.767699259844556,
                                         0.951171526113385,

                                         0.955320355726391,
                                         0.785584808539912,
                                         0.551406173025779,
                                         0.329160968635737,
                                         0.166221928468476,
                                         0.0704908925389006,
                                         0.0248630470378984,
                                         0.00720363657817013,
                                         0.00168743869271958,
                                         0.00031317573957461,
                                         4.48711411845588e-05,
                                         4.79948958257659e-06,
                                         3.66730838077582e-07,
                                         1.88717462390052e-08,
                                         6.03079157179332e-10,
                                         1.06644328768742e-11,
                                         8.79000657996214e-14,
                                         2.57570304385679e-16,
                                         1.68619979910543e-19,
                                         1.00184232453689e-23,
                                         6.35965070014173e-30,
                                         1.92360242154708e-41,
                                         0.0437287342898956,
                                         0.191019661388689,
                                         0.334425658489516,
                                         0.378766977337335,
                                         0.315643075561486,
                                         0.203349919167616,
                                         0.103323306930107,
                                         0.0416244190456524,
                                         0.013239893483207,
                                         0.00328619289099061,
                                         0.000624329482091964,
                                         8.83056089738271e-05,
                                         8.94710339407869e-06,
                                         6.15737445310873e-07,
                                         2.67188581367254e-08,
                                         6.56952219754019e-10,
                                         7.8004233006753e-12,
                                         3.47238448192479e-14,
                                         3.75132958397554e-17,
                                         4.22874379402192e-21,
                                         6.69558047051659e-27,
                                         1.07581452869447e-37,
                                         0.000938266181699294,
                                         0.0217723018096626,
                                         0.0950755483156655,
                                         0.20430413914753,
                                         0.280960648705139,
                                         0.274976942961038,
                                         0.20127207518112,
                                         0.112742031538963,
                                         0.0486947633831935,
                                         0.0161636438509718,
                                         0.00407194453020614,
                                         0.00076159276048464,
                                         0.000102319583132435,
                                         9.41716811626978e-06,
                                         5.54884699549719e-07,
                                         1.8970163956264e-08,
                                         3.24480365409501e-10,
                                         2.19432576726537e-12,
                                         3.91203463896557e-15,
                                         8.36690142508751e-19,
                                         3.30433815896554e-24,
                                         2.82033551147764e-34,
                                         1.25265286729691e-05,
                                         0.00154410254409956,
                                         0.0168183584420099,
                                         0.0685689880625403,
                                         0.155610982608965,
                                         0.231363088059327,
                                         0.243957553052016,
                                         0.19000675675741,
                                         0.111436008069394,
                                         0.0494687584881812,
                                         0.016524768358688,
                                         0.00408698310025855,
                                         0.000728082488266099,
                                         8.96170408039487e-05,
                                         7.1702305957062e-06,
                                         3.40842625546189e-07,
                                         8.39854766673928e-09,
                                         8.62819430423591e-11,
                                         2.53843284608877e-13,
                                         1.030062196706e-16,
                                         1.01467335247814e-21,
                                         4.60054903484211e-31,
                                         1.16469440856452e-07,
                                         7.62648659341136e-05,
                                         0.00207192938407238,
                                         0.0160270980941539,
                                         0.0600221535080321,
                                         0.135571513719955,
                                         0.20593093576242,
                                         0.223012298472401,
                                         0.177601002300261,
                                         0.10543852685259,
                                         0.0467030762225658,
                                         0.0152742345996515,
                                         0.00360810351780561,
                                         0.000593932998276291,
                                         6.45267922537891e-05,
                                         4.26494367175866e-06,
                                         1.51389773947686e-07,
                                         2.36273660946872e-09,
                                         1.14711047557724e-11,
                                         8.83158850141374e-15,
                                         2.16992440506876e-19,
                                         5.22630958835953e-28,
                                         7.99689005092732e-10,
                                         2.78163898567446e-06,
                                         0.00018849254392908,
                                         0.00276636786210041,
                                         0.0170966405963927,
                                         0.0586638685440899,
                                         0.128368006844163,
                                         0.193293149853223,
                                         0.209022558547882,
                                         0.165956982100599,
                                         0.0974728075341552,
                                         0.0421545009544711,
                                         0.0132039926337678,
                                         0.00290678018673709,
                                         0.000428819879046392,
                                         3.94094685292204e-05,
                                         2.01519357712002e-06,
                                         4.77791682381395e-08,
                                         3.82800673924058e-10,
                                         5.59167707612656e-13,
                                         3.42681638208965e-17,
                                         4.38438296055772e-25,
                                         4.19430929084645e-12,
                                         7.75009829872186e-08,
                                         1.30991644688565e-05,
                                         0.000364749881291969,
                                         0.00371997639457937,
                                         0.0193911312312273,
                                         0.0611254678397284,
                                         0.127977701478164,
                                         0.18791917602767,
                                         0.199536302778376,
                                         0.155400229652203,
                                         0.0888707081741593,
                                         0.0369115047931228,
                                         0.0108671865092316,
                                         0.00217690730628275,
                                         0.000278174969754627,
                                         2.04911906095601e-05,
                                         7.38060652263784e-07,
                                         9.75821392244478e-09,
                                         2.7044286800209e-11,
                                         4.1339693991566e-15,
                                         2.80964870859139e-22,
                                         1.71419532024048e-14,
                                         1.68257403440268e-09,
                                         7.09338511871196e-07,
                                         3.74749371414036e-05,
                                         0.000630710572681237,
                                         0.00499454715051951,
                                         0.0226802647304815,
                                         0.0660256482093243,
                                         0.131646575029919,
                                         0.186942840947339,
                                         0.193054692548612,
                                         0.14599361241275,
                                         0.0804042033413901,
                                         0.0316579344430295,
                                         0.00861123595008708,
                                         0.00153001625967343,
                                         0.000162359663007377,
                                         8.8839487704995e-06,
                                         1.93833349582094e-07,
                                         1.0192236874967e-09,
                                         3.88601261292672e-13,
                                         1.40299489430249e-19,
                                         5.5170982784899e-17,
                                         2.8766810778374e-11,
                                         3.02492106596329e-08,
                                         3.032056078161e-06,
                                         8.42113444103691e-05,
                                         0.0010130704594423,
                                         0.00662711650003002,
                                         0.02682511570179,
                                         0.0726270815103678,
                                         0.137926056159661,
                                         0.188868543429963,
                                         0.188868543429963,
                                         0.137926056159661,
                                         0.0726270815103678,
                                         0.02682511570179,
                                         0.00662711650003002,
                                         0.0010130704594423,
                                         8.42113444103687e-05,
                                         3.03205607816098e-06,
                                         3.02492106596333e-08,
                                         2.8766810778374e-11,
                                         5.5170982784899e-17,
                                         1.40299489430249e-19,
                                         3.88601261292672e-13,
                                         1.01922368749668e-09,
                                         1.93833349582096e-07,
                                         8.88394877049954e-06,
                                         0.000162359663007377,
                                         0.00153001625967343,
                                         0.00861123595008708,
                                         0.0316579344430295,
                                         0.08040420334139,
                                         0.14599361241275,
                                         0.193054692548612,
                                         0.186942840947339,
                                         0.131646575029919,
                                         0.0660256482093243,
                                         0.0226802647304815,
                                         0.00499454715051951,
                                         0.000630710572681234,
                                         3.74749371414033e-05,
                                         7.09338511871203e-07,
                                         1.68257403440268e-09,
                                         1.71419532024048e-14,
                                         2.80964870859139e-22,
                                         4.1339693991566e-15,
                                         2.70442868002086e-11,
                                         9.75821392244486e-09,
                                         7.38060652263788e-07,
                                         2.04911906095601e-05,
                                         0.000278174969754627,
                                         0.00217690730628275,
                                         0.0108671865092316,
                                         0.0369115047931227,
                                         0.0888707081741593,
                                         0.155400229652203,
                                         0.199536302778376,
                                         0.18791917602767,
                                         0.127977701478164,
                                         0.0611254678397284,
                                         0.0193911312312273,
                                         0.00371997639457935,
                                         0.000364749881291967,
                                         1.30991644688566e-05,
                                         7.75009829872186e-08,
                                         4.19430929084645e-12,
                                         4.38438296055772e-25,
                                         3.42681638208965e-17,
                                         5.59167707612647e-13,
                                         3.82800673924062e-10,
                                         4.77791682381398e-08,
                                         2.01519357712002e-06,
                                         3.94094685292204e-05,
                                         0.000428819879046392,
                                         0.00290678018673709,
                                         0.0132039926337678,
                                         0.0421545009544711,
                                         0.0974728075341552,
                                         0.1659569821006,
                                         0.209022558547882,
                                         0.193293149853223,
                                         0.128368006844163,
                                         0.0586638685440899,
                                         0.0170966405963927,
                                         0.0027663678621004,
                                         0.000188492543929081,
                                         2.78163898567446e-06,
                                         7.99689005092732e-10,
                                         5.22630958835953e-28,
                                         2.16992440506876e-19,
                                         8.83158850141358e-15,
                                         1.14711047557725e-11,
                                         2.36273660946873e-09,
                                         1.51389773947686e-07,
                                         4.26494367175865e-06,
                                         6.45267922537891e-05,
                                         0.000593932998276291,
                                         0.0036081035178056,
                                         0.0152742345996515,
                                         0.0467030762225658,
                                         0.10543852685259,
                                         0.177601002300261,
                                         0.223012298472401,
                                         0.20593093576242,
                                         0.135571513719955,
                                         0.060022153508032,
                                         0.0160270980941538,
                                         0.00207192938407239,
                                         7.62648659341136e-05,
                                         1.16469440856452e-07,
                                         4.60054903484211e-31,
                                         1.01467335247814e-21,
                                         1.03006219670597e-16,
                                         2.5384328460888e-13,
                                         8.62819430423597e-11,
                                         8.39854766673928e-09,
                                         3.40842625546189e-07,
                                         7.1702305957062e-06,
                                         8.96170408039487e-05,
                                         0.000728082488266098,
                                         0.00408698310025855,
                                         0.016524768358688,
                                         0.0494687584881812,
                                         0.111436008069394,
                                         0.19000675675741,
                                         0.243957553052016,
                                         0.231363088059327,
                                         0.155610982608965,
                                         0.0685689880625401,
                                         0.01681835844201,
                                         0.00154410254409956,
                                         1.25265286729691e-05,
                                         2.82033551147764e-34,
                                         3.30433815896554e-24,
                                         8.36690142508733e-19,
                                         3.91203463896562e-15,
                                         2.19432576726538e-12,
                                         3.24480365409501e-10,
                                         1.8970163956264e-08,
                                         5.54884699549719e-07,
                                         9.41716811626977e-06,
                                         0.000102319583132435,
                                         0.00076159276048464,
                                         0.00407194453020614,
                                         0.0161636438509718,
                                         0.0486947633831935,
                                         0.112742031538963,
                                         0.20127207518112,
                                         0.274976942961038,
                                         0.280960648705138,
                                         0.204304139147529,
                                         0.0950755483156658,
                                         0.0217723018096626,
                                         0.000938266181699294,
                                         1.07581452869447e-37,
                                         6.69558047051659e-27,
                                         4.22874379402182e-21,
                                         3.75132958397559e-17,
                                         3.47238448192481e-14,
                                         7.8004233006753e-12,
                                         6.56952219754019e-10,
                                         2.67188581367254e-08,
                                         6.15737445310873e-07,
                                         8.94710339407867e-06,
                                         8.83056089738271e-05,
                                         0.000624329482091964,
                                         0.00328619289099061,
                                         0.013239893483207,
                                         0.0416244190456524,
                                         0.103323306930107,
                                         0.203349919167616,
                                         0.315643075561486,
                                         0.378766977337335,
                                         0.334425658489517,
                                         0.191019661388689,
                                         0.0437287342898956,
                                         1.92360242154708e-41,
                                         6.35965070014173e-30,
                                         1.00184232453687e-23,
                                         1.68619979910545e-19,
                                         2.57570304385681e-16,
                                         8.79000657996214e-14,
                                         1.06644328768742e-11,
                                         6.03079157179332e-10,
                                         1.88717462390052e-08,
                                         3.66730838077581e-07,
                                         4.79948958257659e-06,
                                         4.48711411845588e-05,
                                         0.00031317573957461,
                                         0.00168743869271958,
                                         0.00720363657817013,
                                         0.0248630470378984,
                                         0.0704908925389006,
                                         0.166221928468476,
                                         0.329160968635737,
                                         0.551406173025779,
                                         0.785584808539912,
                                         0.955320355726391,

                                         0.958965542571419,
                                         0.801570776202361,
                                         0.579596556434668,
                                         0.361436155010619,
                                         0.193519946582445,
                                         0.0884229683631035,
                                         0.0342047743442842,
                                         0.0110880901977681,
                                         0.00297345672078356,
                                         0.000648961757088575,
                                         0.000112920698849661,
                                         1.52587890625e-05,
                                         1.54798577043761e-06,
                                         1.12793478842518e-07,
                                         5.56374682366355e-09,
                                         1.7128657316441e-10,
                                         2.93216091315597e-12,
                                         2.35066170254789e-14,
                                         6.73053298746577e-17,
                                         4.32491880329032e-20,
                                         2.53346501940561e-24,
                                         1.59258723206208e-30,
                                         4.79104857806586e-42,
                                         0.0402334438499917,
                                         0.178524137082455,
                                         0.321575237390574,
                                         0.379772246835252,
                                         0.334717524363028,
                                         0.231592311744473,
                                         0.128536221538303,
                                         0.0576475551467623,
                                         0.0208635502848164,
                                         0.0060442600729657,
                                         0.00138183321591173,
                                         0.000244140625,
                                         3.23835800489889e-05,
                                         3.10027286277168e-06,
                                         2.0299284641817e-07,
                                         8.43410770548844e-09,
                                         1.99751001708964e-10,
                                         2.29758215419679e-12,
                                         9.96177451393415e-15,
                                         1.05372259616475e-17,
                                         1.16895575956243e-21,
                                         1.83057753190649e-27,
                                         2.92338715066995e-38,
                                         0.000791248126970899,
                                         0.0186377417865266,
                                         0.0836335789482098,
                                         0.187049320640985,
                                         0.271376631067937,
                                         0.284331166218992,
                                         0.226415215840666,
                                         0.140490292756916,
                                         0.0686208486707157,
                                         0.0263881206913859,
                                         0.00792645464935069,
                                         0.0018310546875,
                                         0.0003175587138834,
                                         3.99445126488307e-05,
                                         3.47164563200363e-06,
                                         1.94668752361962e-07,
                                         6.37868876809685e-09,
                                         1.05267242734549e-10,
                                         6.91138741709397e-13,
                                         1.20341635318331e-15,
                                         2.52826457048604e-19,
                                         9.86311975822852e-25,
                                         8.36148420863304e-35,
                                         9.68241513565938e-06,
                                         0.00121069613557103,
                                         0.0135339411195154,
                                         0.0573237461268958,
                                         0.136902683262589,
                                         0.217205247088996,
                                         0.248159672523674,
                                         0.213038089682458,
                                         0.140433097301493,
                                         0.0716835158796333,
                                         0.0282909706447742,
                                         0.008544921875,
                                         0.00193762053011662,
                                         0.000320228405108345,
                                         3.69432892715715e-05,
                                         2.79575342603507e-06,
                                         1.26741655745847e-07,
                                         3.00096522903839e-09,
                                         2.98359098560156e-11,
                                         8.55167213471824e-14,
                                         3.40245570478686e-17,
                                         3.30663305341019e-22,
                                         1.4880788071433e-31,
                                         8.25146920886049e-08,
                                         5.47713697115467e-05,
                                         0.00152526204543341,
                                         0.0122345943010844,
                                         0.0480981147215226,
                                         0.115556063401608,
                                         0.189423294301648,
                                         0.224980449822289,
                                         0.2001517741508,
                                         0.135614683435931,
                                         0.0703223373090274,
                                         0.02777099609375,
                                         0.00823360447269067,
                                         0.00178788325930429,
                                         0.00027378667840258,
                                         2.79626332139002e-05,
                                         1.75381556663249e-06,
                                         5.95806530811708e-08,
                                         8.96994777949704e-10,
                                         4.23216659706291e-12,
                                         3.18888620118097e-15,
                                         7.72030181805267e-20,
                                         1.84435734390755e-28,
                                         5.19286160487721e-10,
                                         1.82978443168245e-06,
                                         0.000126938255299581,
                                         0.00192829050266323,
                                         0.0124787771264123,
                                         0.0453986611533555,
                                         0.106773491366157,
                                         0.175452751421761,
                                         0.210657678758311,
                                         0.189461953831542,
                                         0.129082279649461,
                                         0.066650390625,
                                         0.0258368256309341,
                                         0.00737133831783476,
                                         0.00149836267505753,
                                         0.000206530952103932,
                                         1.79215820664433e-05,
                                         8.73529218004729e-07,
                                         1.99144549207742e-08,
                                         1.54668669293842e-10,
                                         2.2070564599709e-13,
                                         1.33109924183228e-17,
                                         1.68807635924653e-25,
                                         2.49638983990951e-12,
                                         4.66956524162057e-08,
                                         8.0699488005916e-06,
                                         0.000232159010904296,
                                         0.00247312572868962,
                                         0.0136245932351371,
                                         0.0459752072387686,
                                         0.104521534866079,
                                         0.169365651769251,
                                         0.20219364517948,
                                         0.180996488542905,
                                         0.1221923828125,
                                         0.0619324842772167,
                                         0.0232158037322313,
                                         0.00626400033365977,
                                         0.00116525888008128,
                                         0.000139893959736862,
                                         9.783174626558e-06,
                                         3.37735821535106e-07,
                                         4.3178955847446e-09,
                                         1.1668581348736e-11,
                                         1.75314075643613e-15,
                                         1.18023720071342e-22,
                                         9.35144210767846e-15,
                                         9.28567527815361e-10,
                                         3.99769409870003e-07,
                                         2.17800640513991e-05,
                                         0.000381927463128473,
                                         0.00318613791181724,
                                         0.0154256854911711,
                                         0.0485190064756966,
                                         0.106104525209603,
                                         0.168140957902685,
                                         0.197758061931205,
                                         0.174560546875,
                                         0.115680024817544,
                                         0.0569746450366939,
                                         0.0204054944895012,
                                         0.00512295121948155,
                                         0.000850907019299719,
                                         8.53773558473182e-05,
                                         4.46320005105919e-06,
                                         9.39295946613231e-08,
                                         4.80710046356922e-10,
                                         1.79921747904048e-13,
                                         6.42994171296906e-20,
                                         2.75864194465877e-17,
                                         1.45412285119597e-11,
                                         1.55954855725571e-08,
                                         1.60910106713195e-06,
                                         4.64479063891421e-05,
                                         0.000586754133618054,
                                         0.00407582765563047,
                                         0.0177365269178154,
                                         0.0523471494661081,
                                         0.110110844919126,
                                         0.170156603350311,
                                         0.196380615234375,
                                         0.170156603350311,
                                         0.110110844919126,
                                         0.0523471494661081,
                                         0.0177365269178154,
                                         0.00407582765563046,
                                         0.000586754133618056,
                                         4.64479063891421e-05,
                                         1.60910106713195e-06,
                                         1.55954855725573e-08,
                                         1.45412285119597e-11,
                                         2.7586419446583e-17,
                                         6.42994171297029e-20,
                                         1.79921747904048e-13,
                                         4.80710046356915e-10,
                                         9.39295946613231e-08,
                                         4.46320005105919e-06,
                                         8.53773558473179e-05,
                                         0.000850907019299721,
                                         0.00512295121948154,
                                         0.0204054944895012,
                                         0.0569746450366939,
                                         0.115680024817544,
                                         0.174560546875,
                                         0.197758061931205,
                                         0.168140957902685,
                                         0.106104525209603,
                                         0.0485190064756967,
                                         0.0154256854911711,
                                         0.00318613791181724,
                                         0.000381927463128473,
                                         2.17800640513991e-05,
                                         3.99769409870008e-07,
                                         9.28567527815361e-10,
                                         9.35144210767708e-15,
                                         1.18023720071367e-22,
                                         1.75314075643613e-15,
                                         1.16685813487358e-11,
                                         4.3178955847446e-09,
                                         3.37735821535106e-07,
                                         9.78317462655796e-06,
                                         0.000139893959736863,
                                         0.00116525888008128,
                                         0.00626400033365977,
                                         0.0232158037322313,
                                         0.0619324842772167,
                                         0.1221923828125,
                                         0.180996488542905,
                                         0.20219364517948,
                                         0.169365651769251,
                                         0.104521534866079,
                                         0.0459752072387685,
                                         0.0136245932351372,
                                         0.00247312572868962,
                                         0.000232159010904296,
                                         8.06994880059168e-06,
                                         4.66956524162057e-08,
                                         2.49638983990919e-12,
                                         1.68807635924692e-25,
                                         1.33109924183228e-17,
                                         2.20705645997086e-13,
                                         1.54668669293842e-10,
                                         1.99144549207742e-08,
                                         8.73529218004725e-07,
                                         1.79215820664434e-05,
                                         0.000206530952103932,
                                         0.00149836267505753,
                                         0.00737133831783476,
                                         0.0258368256309341,
                                         0.066650390625,
                                         0.129082279649461,
                                         0.189461953831542,
                                         0.210657678758311,
                                         0.175452751421761,
                                         0.106773491366157,
                                         0.0453986611533555,
                                         0.0124787771264123,
                                         0.00192829050266323,
                                         0.000126938255299582,
                                         1.82978443168245e-06,
                                         5.19286160487665e-10,
                                         1.84435734390802e-28,
                                         7.72030181805267e-20,
                                         3.18888620118091e-15,
                                         4.23216659706291e-12,
                                         8.96994777949704e-10,
                                         5.95806530811705e-08,
                                         1.7538155666325e-06,
                                         2.79626332139001e-05,
                                         0.00027378667840258,
                                         0.00178788325930429,
                                         0.00823360447269067,
                                         0.02777099609375,
                                         0.0703223373090274,
                                         0.135614683435931,
                                         0.2001517741508,
                                         0.224980449822289,
                                         0.189423294301648,
                                         0.115556063401608,
                                         0.0480981147215226,
                                         0.0122345943010844,
                                         0.00152526204543342,
                                         5.47713697115467e-05,
                                         8.25146920885978e-08,
                                         1.48807880714372e-31,
                                         3.30663305341019e-22,
                                         3.40245570478679e-17,
                                         8.55167213471824e-14,
                                         2.98359098560156e-11,
                                         3.00096522903837e-09,
                                         1.26741655745848e-07,
                                         2.79575342603506e-06,
                                         3.69432892715715e-05,
                                         0.000320228405108345,
                                         0.00193762053011662,
                                         0.008544921875,
                                         0.0282909706447742,
                                         0.0716835158796333,
                                         0.140433097301493,
                                         0.213038089682459,
                                         0.248159672523674,
                                         0.217205247088996,
                                         0.136902683262589,
                                         0.0573237461268958,
                                         0.0135339411195155,
                                         0.00121069613557103,
                                         9.68241513565876e-06,
                                         8.36148420863553e-35,
                                         9.86311975822852e-25,
                                         2.52826457048598e-19,
                                         1.20341635318331e-15,
                                         6.91138741709397e-13,
                                         1.05267242734549e-10,
                                         6.37868876809687e-09,
                                         1.94668752361962e-07,
                                         3.47164563200363e-06,
                                         3.99445126488307e-05,
                                         0.0003175587138834,
                                         0.0018310546875,
                                         0.00792645464935069,
                                         0.0263881206913859,
                                         0.0686208486707157,
                                         0.140490292756916,
                                         0.226415215840666,
                                         0.284331166218992,
                                         0.271376631067937,
                                         0.187049320640985,
                                         0.0836335789482101,
                                         0.0186377417865266,
                                         0.000791248126970866,
                                         2.92338715067088e-38,
                                         1.83057753190649e-27,
                                         1.1689557595624e-21,
                                         1.05372259616475e-17,
                                         9.96177451393415e-15,
                                         2.29758215419678e-12,
                                         1.99751001708965e-10,
                                         8.43410770548841e-09,
                                         2.0299284641817e-07,
                                         3.10027286277168e-06,
                                         3.23835800489889e-05,
                                         0.000244140625,
                                         0.00138183321591173,
                                         0.0060442600729657,
                                         0.0208635502848164,
                                         0.0576475551467623,
                                         0.128536221538303,
                                         0.231592311744473,
                                         0.334717524363028,
                                         0.379772246835252,
                                         0.321575237390574,
                                         0.178524137082455,
                                         0.0402334438499909,
                                         4.79104857806748e-42,
                                         1.59258723206208e-30,
                                         2.53346501940555e-24,
                                         4.32491880329032e-20,
                                         6.73053298746577e-17,
                                         2.35066170254787e-14,
                                         2.93216091315599e-12,
                                         1.71286573164409e-10,
                                         5.56374682366355e-09,
                                         1.12793478842518e-07,
                                         1.54798577043761e-06,
                                         1.52587890625e-05,
                                         0.000112920698849661,
                                         0.000648961757088575,
                                         0.00297345672078356,
                                         0.0110880901977681,
                                         0.0342047743442842,
                                         0.0884229683631035,
                                         0.193519946582445,
                                         0.361436155010619,
                                         0.579596556434668,
                                         0.801570776202361,
                                         0.958965542571419,

                                         0.962184902755307,
                                         0.815903079109018,
                                         0.60557058433353,
                                         0.392389943307502,
                                         0.221141300623195,
                                         0.107845985852969,
                                         0.0452095573413587,
                                         0.016153667483149,
                                         0.00486749844803446,
                                         0.00122055579948932,
                                         0.000250496540866396,
                                         4.12057344608408e-05,
                                         5.29067730057007e-06,
                                         5.12478153432223e-07,
                                         3.58208801356455e-08,
                                         1.70261943831262e-09,
                                         5.07297551939364e-11,
                                         8.44021846881128e-13,
                                         6.60349301647499e-15,
                                         1.85269310038675e-17,
                                         1.17117972913857e-20,
                                         6.77560500720262e-25,
                                         4.22280479359084e-31,
                                         1.264317737593e-42,
                                         0.037135636898524,
                                         0.167063358124364,
                                         0.308555797306067,
                                         0.378024071844923,
                                         0.349933639528624,
                                         0.257697126622888,
                                         0.154452681281506,
                                         0.0760244169721774,
                                         0.0307548802054257,
                                         0.0101718813250098,
                                         0.00272176933742597,
                                         0.000579912196828559,
                                         9.62380145827714e-05,
                                         1.2074404966072e-05,
                                         1.10035436793075e-06,
                                         6.89841527284566e-08,
                                         2.75943883771317e-09,
                                         6.32453290188121e-11,
                                         7.0747011146205e-13,
                                         2.99728206700339e-15,
                                         3.11216005877199e-18,
                                         3.40423165770193e-22,
                                         5.27956737429273e-28,
                                         8.3861737806415e-39,
                                         0.000671837893977337,
                                         0.0160348581504473,
                                         0.0736960024620547,
                                         0.170711321414192,
                                         0.259563014112748,
                                         0.288639954980407,
                                         0.247344313327266,
                                         0.167716711487172,
                                         0.0910884974991654,
                                         0.0397361888842104,
                                         0.0138625198810068,
                                         0.00382567530228393,
                                         0.000820584515164157,
                                         0.000133351344749067,
                                         1.58441947446137e-05,
                                         1.31015404684489e-06,
                                         7.03590649422311e-08,
                                         2.22149072851981e-09,
                                         3.55290628026853e-11,
                                         2.27296645909628e-13,
                                         3.87651602099999e-16,
                                         8.01736051175104e-20,
                                         3.09411853711858e-25,
                                         2.60743064426178e-35,
                                         7.56281771214014e-06,
                                         0.000957622971596715,
                                         0.0109521571957371,
                                         0.0479679004607685,
                                         0.119796861726848,
                                         0.201163339966948,
                                         0.246464245687328,
                                         0.230221084840921,
                                         0.16786438155476,
                                         0.0965865504555493,
                                         0.043931756161113,
                                         0.0157036095678425,
                                         0.00435356965620914,
                                         0.000916377841296625,
                                         0.000141955848321951,
                                         1.5482493567981e-05,
                                         1.11625871699281e-06,
                                         4.85518822714019e-08,
                                         1.1102094610454e-09,
                                         1.07251643650737e-11,
                                         3.00446230245181e-14,
                                         1.17486843766531e-17,
                                         1.12829087690438e-22,
                                         5.0443729773198e-32,
                                         5.92897131811312e-08,
                                         3.9829106930103e-05,
                                         0.00113352730572703,
                                         0.00938675917568366,
                                         0.038505667263886,
                                         0.0976377600046736,
                                         0.17103401903739,
                                         0.220084994808344,
                                         0.215441890376559,
                                         0.163502229886795,
                                         0.096959758891863,
                                         0.0448918445997179,
                                         0.0160858590006604,
                                         0.00438559313722099,
                                         0.000885753724118692,
                                         0.000127419540292305,
                                         1.23334974287999e-05,
                                         7.38999624630142e-07,
                                         2.41603196773603e-08,
                                         3.52445091060155e-10,
                                         1.62169252641233e-12,
                                         1.19901230737618e-15,
                                         2.86537709815481e-20,
                                         6.79638848977228e-29,
                                         3.43243994092066e-10,
                                         1.22330418744779e-06,
                                         8.66347616533581e-05,
                                         0.00135646478160585,
                                         0.00913969592076947,
                                         0.0349956978056688,
                                         0.0876473838193626,
                                         0.155368743030799,
                                         0.204187723956146,
                                         0.204389501833361,
                                         0.158027390999089,
                                         0.0947683443656938,
                                         0.0438905374057257,
                                         0.0154992248549291,
                                         0.0040813192287048,
                                         0.00077438875919554,
                                         0.000100631862936529,
                                         8.30635034695279e-06,
                                         3.88265059396363e-07,
                                         8.55277124001915e-09,
                                         6.4639524354067e-11,
                                         9.03620189725921e-14,
                                         5.37366235780926e-18,
                                         6.76202981987229e-26,
                                         1.51794745190425e-12,
                                         2.87011004884994e-08,
                                         5.05804369203099e-06,
                                         0.00014973783936185,
                                         0.00165717751953135,
                                         0.00958168084859292,
                                         0.0343103862180304,
                                         0.0837851523541537,
                                         0.147828883991226,
                                         0.195174773157676,
                                         0.196744865047093,
                                         0.152823214007226,
                                         0.0914803320701303,
                                         0.0418429034356126,
                                         0.0143654202914758,
                                         0.00359511046851386,
                                         0.000627212883618542,
                                         7.13192003508001e-05,
                                         4.76633030584026e-06,
                                         1.58544959503223e-07,
                                         1.96814903631029e-09,
                                         5.20209649551431e-12,
                                         7.6981994286208e-16,
                                         5.13932592543429e-23,
                                         5.23083538861985e-15,
                                         5.24714615559754e-10,
                                         2.30108940786514e-07,
                                         1.28799781747402e-05,
                                         0.000234135272302793,
                                         0.00204422770398894,
                                         0.0104658084382424,
                                         0.0352071605261417,
                                         0.0833968205234541,
                                         0.14522764891107,
                                         0.190868810834482,
                                         0.192033010264874,
                                         0.148574778528244,
                                         0.0880225955651138,
                                         0.0394000373351399,
                                         0.0130054673853338,
                                         0.00304617573192237,
                                         0.000477159099720492,
                                         4.55932432859626e-05,
                                         2.29012162577195e-06,
                                         4.66958555619071e-08,
                                         2.33362681761904e-10,
                                         8.59346802886104e-14,
                                         3.04365739298852e-20,
                                         1.41950174740594e-17,
                                         7.55437145921359e-12,
                                         8.2439428531896e-09,
                                         8.72467492585171e-07,
                                         2.60504493384313e-05,
                                         0.000343453068442166,
                                         0.00251403036201133,
                                         0.0116505250932864,
                                         0.0370501739575082,
                                         0.0850992029490849,
                                         0.145819999245019,
                                         0.190025994093056,
                                         0.190025994093056,
                                         0.145819999245019,
                                         0.0850992029490849,
                                         0.0370501739575082,
                                         0.0116505250932863,
                                         0.00251403036201133,
                                         0.000343453068442165,
                                         2.60504493384313e-05,
                                         8.72467492585171e-07,
                                         8.24394285318948e-09,
                                         7.55437145921359e-12,
                                         1.41950174740568e-17,
                                         3.04365739298915e-20,
                                         8.59346802886104e-14,
                                         2.33362681761908e-10,
                                         4.66958555619071e-08,
                                         2.29012162577195e-06,
                                         4.55932432859628e-05,
                                         0.000477159099720492,
                                         0.00304617573192237,
                                         0.0130054673853338,
                                         0.0394000373351399,
                                         0.0880225955651138,
                                         0.148574778528244,
                                         0.192033010264874,
                                         0.190868810834482,
                                         0.14522764891107,
                                         0.0833968205234541,
                                         0.0352071605261416,
                                         0.0104658084382424,
                                         0.00204422770398894,
                                         0.000234135272302793,
                                         1.28799781747402e-05,
                                         2.30108940786511e-07,
                                         5.24714615559754e-10,
                                         5.23083538861901e-15,
                                         5.13932592543547e-23,
                                         7.6981994286208e-16,
                                         5.20209649551441e-12,
                                         1.96814903631029e-09,
                                         1.58544959503223e-07,
                                         4.76633030584028e-06,
                                         7.13192003508001e-05,
                                         0.000627212883618544,
                                         0.00359511046851386,
                                         0.0143654202914758,
                                         0.0418429034356126,
                                         0.0914803320701302,
                                         0.152823214007226,
                                         0.196744865047093,
                                         0.195174773157676,
                                         0.147828883991225,
                                         0.0837851523541536,
                                         0.0343103862180304,
                                         0.00958168084859289,
                                         0.00165717751953135,
                                         0.00014973783936185,
                                         5.05804369203094e-06,
                                         2.87011004884994e-08,
                                         1.51794745190404e-12,
                                         6.762029819874e-26,
                                         5.37366235780926e-18,
                                         9.03620189725939e-14,
                                         6.4639524354067e-11,
                                         8.55277124001915e-09,
                                         3.88265059396365e-07,
                                         8.30635034695279e-06,
                                         0.000100631862936529,
                                         0.00077438875919554,
                                         0.0040813192287048,
                                         0.0154992248549291,
                                         0.0438905374057256,
                                         0.0947683443656939,
                                         0.158027390999089,
                                         0.204389501833361,
                                         0.204187723956146,
                                         0.155368743030799,
                                         0.0876473838193626,
                                         0.0349956978056687,
                                         0.00913969592076947,
                                         0.00135646478160585,
                                         8.66347616533573e-05,
                                         1.22330418744779e-06,
                                         3.43243994092027e-10,
                                         6.79638848977416e-29,
                                         2.86537709815481e-20,
                                         1.1990123073762e-15,
                                         1.62169252641233e-12,
                                         3.52445091060156e-10,
                                         2.41603196773604e-08,
                                         7.38999624630142e-07,
                                         1.23334974287999e-05,
                                         0.000127419540292305,
                                         0.000885753724118692,
                                         0.00438559313722099,
                                         0.0160858590006604,
                                         0.0448918445997179,
                                         0.096959758891863,
                                         0.163502229886795,
                                         0.215441890376559,
                                         0.220084994808344,
                                         0.17103401903739,
                                         0.0976377600046734,
                                         0.038505667263886,
                                         0.00938675917568366,
                                         0.00113352730572702,
                                         3.9829106930103e-05,
                                         5.92897131811257e-08,
                                         5.04437297732132e-32,
                                         1.12829087690438e-22,
                                         1.17486843766534e-17,
                                         3.00446230245181e-14,
                                         1.07251643650737e-11,
                                         1.1102094610454e-09,
                                         4.85518822714019e-08,
                                         1.11625871699282e-06,
                                         1.5482493567981e-05,
                                         0.000141955848321951,
                                         0.000916377841296626,
                                         0.00435356965620913,
                                         0.0157036095678425,
                                         0.043931756161113,
                                         0.0965865504555493,
                                         0.16786438155476,
                                         0.230221084840921,
                                         0.246464245687328,
                                         0.201163339966948,
                                         0.119796861726848,
                                         0.0479679004607685,
                                         0.010952157195737,
                                         0.000957622971596715,
                                         7.56281771213962e-06,
                                         2.60743064426262e-35,
                                         3.09411853711858e-25,
                                         8.01736051175125e-20,
                                         3.87651602099999e-16,
                                         2.27296645909628e-13,
                                         3.55290628026856e-11,
                                         2.22149072851981e-09,
                                         7.03590649422313e-08,
                                         1.31015404684489e-06,
                                         1.58441947446137e-05,
                                         0.000133351344749067,
                                         0.000820584515164156,
                                         0.00382567530228394,
                                         0.0138625198810068,
                                         0.0397361888842104,
                                         0.0910884974991654,
                                         0.167716711487172,
                                         0.247344313327266,
                                         0.288639954980406,
                                         0.259563014112748,
                                         0.170711321414192,
                                         0.0736960024620545,
                                         0.0160348581504473,
                                         0.000671837893977306,
                                         8.3861737806444e-39,
                                         5.27956737429273e-28,
                                         3.40423165770202e-22,
                                         3.11216005877199e-18,
                                         2.99728206700339e-15,
                                         7.07470111462055e-13,
                                         6.32453290188121e-11,
                                         2.75943883771318e-09,
                                         6.89841527284566e-08,
                                         1.10035436793075e-06,
                                         1.2074404966072e-05,
                                         9.62380145827712e-05,
                                         0.000579912196828559,
                                         0.00272176933742597,
                                         0.0101718813250098,
                                         0.0307548802054257,
                                         0.0760244169721774,
                                         0.154452681281506,
                                         0.257697126622888,
                                         0.349933639528624,
                                         0.378024071844923,
                                         0.308555797306066,
                                         0.167063358124364,
                                         0.0371356368985232,
                                         1.26431773759346e-42,
                                         4.22280479359084e-31,
                                         6.77560500720281e-25,
                                         1.17117972913857e-20,
                                         1.85269310038675e-17,
                                         6.60349301647504e-15,
                                         8.44021846881128e-13,
                                         5.07297551939366e-11,
                                         1.70261943831262e-09,
                                         3.58208801356455e-08,
                                         5.12478153432223e-07,
                                         5.29067730057006e-06,
                                         4.12057344608408e-05,
                                         0.000250496540866396,
                                         0.00122055579948932,
                                         0.00486749844803446,
                                         0.016153667483149,
                                         0.0452095573413587,
                                         0.107845985852969,
                                         0.221141300623195,
                                         0.392389943307502,
                                         0.60557058433353,
                                         0.815903079109018,
                                         0.962184902755307,

                                         7.62939453125e-06,
                                         0.00012969970703125,
                                         0.00103759765625,
                                         0.00518798828125,
                                         0.018157958984375,
                                         0.047210693359375,
                                         0.09442138671875,
                                         0.14837646484375,
                                         0.185470581054688,
                                         0.185470581054688,
                                         0.14837646484375,
                                         0.09442138671875,
                                         0.047210693359375,
                                         0.018157958984375,
                                         0.00518798828125,
                                         0.00103759765625,
                                         0.00012969970703125,
                                         7.62939453125e-06,

                                         0.0176713010559466,
                                         3.34318352300439e-12,
                                         0.0804951844015183,
                                         2.12107823036952e-10,
                                         0.172548957239849,
                                         6.33277737805386e-09,
                                         0.231171768736249,
                                         1.18171234639599e-07,
                                         0.216798010610133,
                                         1.54357368083845e-06,
                                         0.151036214845513,
                                         1.49778080454995e-05,
                                         0.0809400635914176,
                                         0.000111795881223636,
                                         0.0340808673458692,
                                         0.000655643856691286,
                                         0.0114149261033473,
                                         0.00305862023105286,
                                         0.00305862023105286,
                                         0.0114149261033473,
                                         0.000655643856691286,
                                         0.0340808673458692,
                                         0.000111795881223636,
                                         0.0809400635914176,
                                         1.49778080454995e-05,
                                         0.151036214845513,
                                         1.54357368083845e-06,
                                         0.216798010610133,
                                         1.18171234639599e-07,
                                         0.231171768736249,
                                         6.33277737805386e-09,
                                         0.172548957239849,
                                         2.12107823036952e-10,
                                         0.0804951844015183,
                                         3.34318352300439e-12,
                                         0.0176713010559466,

                                         0.130973769999999,
                                         7.62939453125e-06,
                                         7.63511655807138e-17,
                                         0.282809449999989,
                                         0.00012969970703125,
                                         1.02188947363682e-14,
                                         0.287372079999356,
                                         0.00103759765625,
                                         6.43625504608587e-13,
                                         0.182505199974664,
                                         0.00518798828125,
                                         2.53362643948887e-11,
                                         0.0811341993018484,
                                         0.018157958984375,
                                         6.98151956726232e-10,
                                         0.0267940257089995,
                                         0.047210693359375,
                                         1.42910006939121e-08,
                                         0.00680657497437913,
                                         0.09442138671875,
                                         2.25025620927617e-07,
                                         0.00135857602105339,
                                         0.14837646484375,
                                         2.78397894662345e-06,
                                         0.000215702225146305,
                                         0.185470581054688,
                                         2.73977748536981e-05,
                                         2.73977748536981e-05,
                                         0.185470581054688,
                                         0.000215702225146305,
                                         2.78397894662345e-06,
                                         0.14837646484375,
                                         0.00135857602105339,
                                         2.25025620927617e-07,
                                         0.09442138671875,
                                         0.00680657497437913,
                                         1.42910006939121e-08,
                                         0.047210693359375,
                                         0.0267940257089995,
                                         6.98151956726232e-10,
                                         0.018157958984375,
                                         0.0811341993018484,
                                         2.53362643948887e-11,
                                         0.00518798828125,
                                         0.182505199974664,
                                         6.43625504608587e-13,
                                         0.00103759765625,
                                         0.287372079999356,
                                         1.02188947363682e-14,
                                         0.00012969970703125,
                                         0.282809449999989,
                                         7.63511655807138e-17,
                                         7.62939453125e-06,
                                         0.130973769999999,

                                         0.294251832342467,
                                         0.00110450376570857,
                                         6.53053901548419e-09,
                                         2.02533988112827e-20,
                                         0.373231775975004,
                                         0.00924855485599848,
                                         2.2539288122175e-07,
                                         4.61462401503815e-18,
                                         0.22278149416278,
                                         0.0364436291311924,
                                         3.66076986438735e-06,
                                         4.94784168119425e-16,
                                         0.083111214894128,
                                         0.0897530833824326,
                                         3.716076769862e-05,
                                         3.31570042629474e-14,
                                         0.0217039204580053,
                                         0.154730231815232,
                                         0.000264055347638559,
                                         1.55536676751434e-12,
                                         0.0042103873397779,
                                         0.198155567353329,
                                         0.00139383241620539,
                                         5.41995460833044e-11,
                                         0.000628293491429971,
                                         0.195206389521006,
                                         0.00565956173681332,
                                         1.45283111006913e-09,
                                         7.36661070871407e-05,
                                         0.151093725142548,
                                         0.0180559229300027,
                                         3.05984383582237e-08,
                                         6.87049846705363e-06,
                                         0.0930281096763387,
                                         0.0458217794645707,
                                         5.12623794602841e-07,
                                         5.12623794602837e-07,
                                         0.0458217794645707,
                                         0.0930281096763387,
                                         6.87049846705368e-06,
                                         3.05984383582235e-08,
                                         0.0180559229300027,
                                         0.151093725142548,
                                         7.36661070871412e-05,
                                         1.45283111006912e-09,
                                         0.00565956173681332,
                                         0.195206389521006,
                                         0.000628293491429974,
                                         5.41995460833039e-11,
                                         0.00139383241620539,
                                         0.198155567353329,
                                         0.00421038733977792,
                                         1.55536676751432e-12,
                                         0.000264055347638558,
                                         0.154730231815232,
                                         0.0217039204580053,
                                         3.31570042629471e-14,
                                         3.716076769862e-05,
                                         0.0897530833824326,
                                         0.0831112148941282,
                                         4.94784168119419e-16,
                                         3.66076986438735e-06,
                                         0.0364436291311924,
                                         0.222781494162781,
                                         4.61462401503809e-18,
                                         2.2539288122175e-07,
                                         0.00924855485599848,
                                         0.373231775975004,
                                         2.02533988112824e-20,
                                         6.53053901548419e-09,
                                         0.00110450376570857,
                                         0.294251832342467,

                                         0.441851081260504,
                                         0.0115611305798335,
                                         7.62939453125e-06,
                                         1.49245079188059e-11,
                                         2.57943437794053e-23,
                                         0.369706521802748,
                                         0.0589604752828895,
                                         0.00012969970703125,
                                         8.45740628678638e-10,
                                         8.90924981565591e-21,
                                         0.14557250893998,
                                         0.141502043128515,
                                         0.00103759765625,
                                         2.25535771296945e-08,
                                         1.4481010065222e-18,
                                         0.0358246239066669,
                                         0.21224841846885,
                                         0.00518798828125,
                                         3.75901180646097e-07,
                                         1.4710810172575e-16,
                                         0.0061713752217343,
                                         0.222855960963965,
                                         0.018157958984375,
                                         4.38560977529204e-06,
                                         1.04609798951308e-14,
                                         0.000789746243408421,
                                         0.173823844461093,
                                         0.047210693359375,
                                         3.80094500801697e-05,
                                         5.52603265634824e-13,
                                         7.7740948089243e-05,
                                         0.104292023672152,
                                         0.09442138671875,
                                         0.000253401880840146,
                                         2.2454902537558e-11,
                                         6.01279983130494e-06,
                                         0.0491651634811654,
                                         0.14837646484375,
                                         0.00132737224155863,
                                         7.16924713133137e-10,
                                         3.69929552892748e-07,
                                         0.0184365327205247,
                                         0.185470581054688,
                                         0.00553083874333404,
                                         1.82075409716381e-08,
                                         1.82075409716379e-08,
                                         0.00553083874333404,
                                         0.185470581054688,
                                         0.0184365327205247,
                                         3.69929552892751e-07,
                                         7.16924713133129e-10,
                                         0.00132737224155863,
                                         0.14837646484375,
                                         0.0491651634811654,
                                         6.01279983130499e-06,
                                         2.24549025375578e-11,
                                         0.000253401880840146,
                                         0.09442138671875,
                                         0.104292023672152,
                                         7.77409480892435e-05,
                                         5.52603265634816e-13,
                                         3.80094500801697e-05,
                                         0.047210693359375,
                                         0.173823844461093,
                                         0.000789746243408425,
                                         1.04609798951307e-14,
                                         4.38560977529204e-06,
                                         0.018157958984375,
                                         0.222855960963965,
                                         0.00617137522173433,
                                         1.47108101725747e-16,
                                         3.75901180646097e-07,
                                         0.00518798828125,
                                         0.21224841846885,
                                         0.035824623906667,
                                         1.44810100652217e-18,
                                         2.25535771296945e-08,
                                         0.00103759765625,
                                         0.141502043128515,
                                         0.145572508939981,
                                         8.90924981565575e-21,
                                         8.45740628678638e-10,
                                         0.00012969970703125,
                                         0.0589604752828895,
                                         0.369706521802749,
                                         2.57943437794048e-23,
                                         1.49245079188059e-11,
                                         7.62939453125e-06,
                                         0.0115611305798335,
                                         0.441851081260504,

                                         0.557705782677578,
                                         0.0426289339480105,
                                         0.000290023188636388,
                                         7.40837339364831e-08,
                                         7.78616051469573e-14,
                                         9.63802238359866e-26,
                                         0.331315147123956,
                                         0.14779521935725,
                                         0.00303071968874386,
                                         2.04883818110951e-06,
                                         6.49030761338904e-12,
                                         4.68866959319998e-23,
                                         0.092622926975842,
                                         0.241133398119828,
                                         0.0149038984656224,
                                         2.6664504635961e-05,
                                         2.54594064834485e-10,
                                         1.07337726884271e-20,
                                         0.0161836220665653,
                                         0.24588631799833,
                                         0.0458072282093924,
                                         0.000216889934877677,
                                         6.24182066070632e-09,
                                         1.53580181217503e-18,
                                         0.00197938828129684,
                                         0.1755130450412,
                                         0.0985521682630309,
                                         0.00123493277469182,
                                         1.07120437509116e-07,
                                         1.53821130027863e-16,
                                         0.000179842182869504,
                                         0.0930657200786974,
                                         0.157508377404436,
                                         0.00522339102952599,
                                         1.36564774768316e-06,
                                         1.14446396523005e-14,
                                         1.25692331875683e-05,
                                         0.0379600460499337,
                                         0.19364120616625,
                                         0.0169948919839546,
                                         1.33925012466846e-05,
                                         6.550054236436e-13,
                                         6.90224945492971e-07,
                                         0.0121654553392411,
                                         0.187049499378884,
                                         0.0434459129690458,
                                         0.000103192788362783,
                                         2.94545450247357e-11,
                                         3.01500395838477e-08,
                                         0.00310131139388313,
                                         0.143724461158011,
                                         0.0883476119583458,
                                         0.00063248811284884,
                                         1.05359841639261e-09,
                                         1.05359841639261e-09,
                                         0.000632488112848841,
                                         0.0883476119583458,
                                         0.143724461158011,
                                         0.00310131139388313,
                                         3.01500395838477e-08,
                                         2.94545450247357e-11,
                                         0.000103192788362783,
                                         0.0434459129690458,
                                         0.187049499378884,
                                         0.0121654553392411,
                                         6.90224945492971e-07,
                                         6.550054236436e-13,
                                         1.33925012466846e-05,
                                         0.0169948919839546,
                                         0.19364120616625,
                                         0.0379600460499336,
                                         1.25692331875683e-05,
                                         1.14446396523005e-14,
                                         1.36564774768317e-06,
                                         0.00522339102952599,
                                         0.157508377404436,
                                         0.0930657200786973,
                                         0.000179842182869504,
                                         1.53821130027863e-16,
                                         1.07120437509116e-07,
                                         0.00123493277469182,
                                         0.0985521682630309,
                                         0.1755130450412,
                                         0.00197938828129684,
                                         1.53580181217503e-18,
                                         6.24182066070635e-09,
                                         0.000216889934877677,
                                         0.0458072282093924,
                                         0.24588631799833,
                                         0.0161836220665653,
                                         1.07337726884271e-20,
                                         2.54594064834486e-10,
                                         2.6664504635961e-05,
                                         0.0149038984656224,
                                         0.241133398119828,
                                         0.092622926975842,
                                         4.68866959319998e-23,
                                         6.49030761338907e-12,
                                         2.04883818110951e-06,
                                         0.00303071968874386,
                                         0.14779521935725,
                                         0.331315147123956,
                                         9.63802238359866e-26,
                                         7.78616051469577e-14,
                                         7.40837339364831e-08,
                                         0.000290023188636388,
                                         0.0426289339480105,
                                         0.557705782677578,

                                         0.645208544474677,
                                         0.0951308597870556,
                                         0.00249705069465291,
                                         7.62939453125e-06,
                                         1.09341253421818e-09,
                                         7.8239962904731e-16,
                                         7.86220937137321e-28,
                                         0.286393669180361,
                                         0.240019893224785,
                                         0.0179406609558133,
                                         0.00012969970703125,
                                         4.39815784138436e-08,
                                         8.96191173573533e-14,
                                         5.1189294523859e-25,
                                         0.0598229441237545,
                                         0.284979828083955,
                                         0.0606583488029068,
                                         0.00103759765625,
                                         8.32527599919341e-07,
                                         4.83074101038185e-12,
                                         1.56839247241215e-22,
                                         0.0078100204123665,
                                         0.211475966959491,
                                         0.128180731766144,
                                         0.00518798828125,
                                         9.84932541110918e-06,
                                         1.62744703623986e-10,
                                         3.00338060233056e-20,
                                         0.000713731057782439,
                                         0.10985149170556,
                                         0.189606381054702,
                                         0.018157958984375,
                                         8.15666024097266e-05,
                                         3.83793851719982e-09,
                                         4.02590973929359e-18,
                                         4.84531844791608e-05,
                                         0.0423892957793371,
                                         0.208347577512328,
                                         0.047210693359375,
                                         0.000501791802629358,
                                         6.72348540082225e-08,
                                         4.00887947616106e-16,
                                         2.53026904890255e-06,
                                         0.0125823885486047,
                                         0.176108618017337,
                                         0.09442138671875,
                                         0.00237460511973535,
                                         9.06040407900501e-07,
                                         3.07070865681106e-14,
                                         1.03818734391611e-07,
                                         0.00293450350318585,
                                         0.11696001419947,
                                         0.14837646484375,
                                         0.0088292514795967,
                                         9.59324016584914e-06,
                                         1.84807195451895e-12,
                                         3.38844253931735e-09,
                                         0.00054440429174817,
                                         0.0617887747661054,
                                         0.185470581054688,
                                         0.0261139002981674,
                                         8.07975952462324e-05,
                                         8.8473764659442e-11,
                                         8.84737646594403e-11,
                                         8.07975952462324e-05,
                                         0.0261139002981674,
                                         0.185470581054688,
                                         0.0617887747661054,
                                         0.00054440429174817,
                                         3.38844253931741e-09,
                                         1.84807195451891e-12,
                                         9.59324016584914e-06,
                                         0.0088292514795967,
                                         0.14837646484375,
                                         0.11696001419947,
                                         0.00293450350318585,
                                         1.03818734391613e-07,
                                         3.07070865681098e-14,
                                         9.06040407900501e-07,
                                         0.00237460511973535,
                                         0.09442138671875,
                                         0.176108618017337,
                                         0.0125823885486047,
                                         2.53026904890258e-06,
                                         4.00887947616096e-16,
                                         6.72348540082225e-08,
                                         0.000501791802629358,
                                         0.047210693359375,
                                         0.208347577512328,
                                         0.0423892957793371,
                                         4.84531844791614e-05,
                                         4.02590973929348e-18,
                                         3.83793851719982e-09,
                                         8.15666024097266e-05,
                                         0.018157958984375,
                                         0.189606381054702,
                                         0.10985149170556,
                                         0.000713731057782445,
                                         3.00338060233047e-20,
                                         1.62744703623986e-10,
                                         9.84932541110918e-06,
                                         0.00518798828125,
                                         0.128180731766144,
                                         0.211475966959491,
                                         0.00781002041236655,
                                         1.5683924724121e-22,
                                         4.83074101038185e-12,
                                         8.32527599919341e-07,
                                         0.00103759765625,
                                         0.0606583488029068,
                                         0.284979828083955,
                                         0.0598229441237547,
                                         5.11892945238572e-25,
                                         8.96191173573533e-14,
                                         4.39815784138436e-08,
                                         0.00012969970703125,
                                         0.0179406609558133,
                                         0.240019893224785,
                                         0.286393669180361,
                                         7.86220937137292e-28,
                                         7.8239962904731e-16,
                                         1.09341253421818e-09,
                                         7.62939453125e-06,
                                         0.00249705069465291,
                                         0.0951308597870556,
                                         0.645208544474677,

                                         0.711107156030271,
                                         0.161598369297387,
                                         0.0100150824931941,
                                         0.000133641949552819,
                                         2.434087824099e-07,
                                         2.38781929795592e-11,
                                         1.32447208808356e-17,
                                         1.15828340506419e-29,
                                         0.244886663574721,
                                         0.310904791440639,
                                         0.0529527556481251,
                                         0.00156761134136549,
                                         5.99706135786008e-06,
                                         1.30516453960578e-09,
                                         1.98952871623132e-15,
                                         9.72036541893877e-27,
                                         0.0396859046947949,
                                         0.281487375547234,
                                         0.131753956571966,
                                         0.00865316643217796,
                                         6.95315332578342e-05,
                                         3.357145309067e-08,
                                         1.40636703868096e-13,
                                         3.83876401373623e-24,
                                         0.00401964272077382,
                                         0.159283373829086,
                                         0.204889066441133,
                                         0.0298532579979887,
                                         0.000503854495215474,
                                         5.39703246419542e-07,
                                         6.21336924968736e-12,
                                         9.47502261877745e-22,
                                         0.000284994619849309,
                                         0.0630928303279566,
                                         0.22303444577721,
                                         0.0720952167112211,
                                         0.00255579789942523,
                                         6.07348497505819e-06,
                                         1.92155884344519e-10,
                                         1.63706957014209e-19,
                                         1.50103622465575e-05,
                                         0.0185649967674513,
                                         0.180355920198606,
                                         0.129338098755004,
                                         0.0096305964226402,
                                         5.07722250096973e-05,
                                         4.41454061172613e-09,
                                         2.10116094608953e-17,
                                         6.08138267776133e-07,
                                         0.00420210009702481,
                                         0.112187769014555,
                                         0.178485582653037,
                                         0.0279149276410159,
                                         0.000326490897986093,
                                         7.80142598168998e-08,
                                         2.07447474433453e-15,
                                         1.93587870130197e-08,
                                         0.000747313036434671,
                                         0.0548308868251174,
                                         0.193528290105061,
                                         0.0635747232871554,
                                         0.00164960745328976,
                                         1.08324605905335e-06,
                                         1.60924301674409e-13,
                                         4.90195498240095e-10,
                                         0.000105719254299025,
                                         0.0213167207764295,
                                         0.166917220987793,
                                         0.115172241317949,
                                         0.00662987758866017,
                                         1.19645291165949e-05,
                                         9.9300282123357e-12,
                                         9.9300282123357e-12,
                                         1.19645291165949e-05,
                                         0.00662987758866017,
                                         0.115172241317949,
                                         0.166917220987793,
                                         0.0213167207764295,
                                         0.000105719254299025,
                                         4.90195498240095e-10,
                                         1.60924301674409e-13,
                                         1.08324605905335e-06,
                                         0.00164960745328976,
                                         0.0635747232871555,
                                         0.193528290105061,
                                         0.0548308868251174,
                                         0.000747313036434671,
                                         1.93587870130197e-08,
                                         2.07447474433453e-15,
                                         7.80142598168998e-08,
                                         0.000326490897986093,
                                         0.0279149276410159,
                                         0.178485582653037,
                                         0.112187769014555,
                                         0.00420210009702481,
                                         6.08138267776133e-07,
                                         2.10116094608953e-17,
                                         4.41454061172613e-09,
                                         5.07722250096973e-05,
                                         0.00963059642264021,
                                         0.129338098755004,
                                         0.180355920198606,
                                         0.0185649967674513,
                                         1.50103622465575e-05,
                                         1.63706957014209e-19,
                                         1.92155884344519e-10,
                                         6.07348497505819e-06,
                                         0.00255579789942523,
                                         0.0720952167112211,
                                         0.22303444577721,
                                         0.0630928303279566,
                                         0.000284994619849309,
                                         9.47502261877745e-22,
                                         6.21336924968736e-12,
                                         5.39703246419542e-07,
                                         0.000503854495215475,
                                         0.0298532579979887,
                                         0.204889066441133,
                                         0.159283373829086,
                                         0.00401964272077382,
                                         3.83876401373623e-24,
                                         1.40636703868096e-13,
                                         3.357145309067e-08,
                                         6.95315332578343e-05,
                                         0.00865316643217796,
                                         0.131753956571966,
                                         0.281487375547234,
                                         0.0396859046947949,
                                         9.72036541893877e-27,
                                         1.98952871623132e-15,
                                         1.30516453960578e-09,
                                         5.99706135786009e-06,
                                         0.00156761134136549,
                                         0.0529527556481251,
                                         0.310904791440639,
                                         0.244886663574721,
                                         1.15828340506419e-29,
                                         1.32447208808356e-17,
                                         2.38781929795592e-11,
                                         2.434087824099e-07,
                                         0.000133641949552819,
                                         0.0100150824931941,
                                         0.161598369297387,
                                         0.711107156030271,

                                         0.761233323889896,
                                         0.233588076015323,
                                         0.0259402550209388,
                                         0.000903647075271148,
                                         7.62939453125e-06,
                                         9.74575737037344e-09,
                                         7.35338078441037e-13,
                                         3.41534796815554e-19,
                                         2.7100452411372e-31,
                                         0.209351487660393,
                                         0.354634530004749,
                                         0.105677550949793,
                                         0.00783899736648096,
                                         0.00012969970703125,
                                         3.24677180976037e-07,
                                         5.2164662265255e-11,
                                         6.50133358613879e-17,
                                         2.84784353093862e-28,
                                         0.0270941403734349,
                                         0.253368392100004,
                                         0.202596670754422,
                                         0.0320009783065867,
                                         0.00103759765625,
                                         5.09013151734302e-06,
                                         1.74143315861213e-09,
                                         5.82386110372131e-15,
                                         1.40830516794241e-25,
                                         0.00219156683211105,
                                         0.113136794156934,
                                         0.242751479807405,
                                         0.0816480862206196,
                                         0.00518798828125,
                                         4.98753846550757e-05,
                                         3.63343367212516e-08,
                                         3.26061546700893e-13,
                                         4.35268700796481e-23,
                                         0.000124088661953745,
                                         0.0353633452863351,
                                         0.203605501070058,
                                         0.145823260268337,
                                         0.018157958984375,
                                         0.000342090924411548,
                                         5.30671425952292e-07,
                                         1.2778686036714e-11,
                                         9.41707751586997e-21,
                                         5.21933037387164e-06,
                                         0.00821122813017025,
                                         0.12685933573721,
                                         0.19346967479908,
                                         0.047210693359375,
                                         0.00174301911839207,
                                         5.75757328120325e-06,
                                         3.72030134486263e-10,
                                         1.5134920883966e-18,
                                         1.68870629229071e-07,
                                         0.00146662654964796,
                                         0.060801179729001,
                                         0.197449322176742,
                                         0.09442138671875,
                                         0.00683155228463203,
                                         4.80518256499134e-05,
                                         8.33156690446626e-09,
                                         1.87111643123383e-16,
                                         4.29297248481205e-09,
                                         0.000205823811548608,
                                         0.0228963485898486,
                                         0.158329938179854,
                                         0.14837646484375,
                                         0.0210378469738946,
                                         0.00031509750215308,
                                         1.46602002506748e-07,
                                         1.8175489644503e-14,
                                         8.68115391299151e-11,
                                         2.29766684853289e-05,
                                         0.00685860542731875,
                                         0.100991728440012,
                                         0.185470581054688,
                                         0.0515345579265759,
                                         0.00164359721282379,
                                         2.05195809255541e-06,
                                         1.4043869794679e-12,
                                         1.40438697946786e-12,
                                         2.05195809255541e-06,
                                         0.00164359721282378,
                                         0.0515345579265759,
                                         0.185470581054688,
                                         0.100991728440012,
                                         0.00685860542731876,
                                         2.29766684853289e-05,
                                         8.68115391299175e-11,
                                         1.81754896445024e-14,
                                         1.46602002506748e-07,
                                         0.000315097502153079,
                                         0.0210378469738946,
                                         0.14837646484375,
                                         0.158329938179854,
                                         0.0228963485898487,
                                         0.000205823811548608,
                                         4.29297248481216e-09,
                                         1.87111643123376e-16,
                                         8.33156690446626e-09,
                                         4.80518256499133e-05,
                                         0.00683155228463203,
                                         0.09442138671875,
                                         0.197449322176742,
                                         0.0608011797290011,
                                         0.00146662654964796,
                                         1.68870629229075e-07,
                                         1.51349208839654e-18,
                                         3.72030134486263e-10,
                                         5.75757328120324e-06,
                                         0.00174301911839207,
                                         0.047210693359375,
                                         0.19346967479908,
                                         0.12685933573721,
                                         0.00821122813017025,
                                         5.21933037387173e-06,
                                         9.41707751586955e-21,
                                         1.2778686036714e-11,
                                         5.3067142595229e-07,
                                         0.000342090924411549,
                                         0.018157958984375,
                                         0.145823260268337,
                                         0.203605501070058,
                                         0.0353633452863351,
                                         0.000124088661953747,
                                         4.3526870079646e-23,
                                         3.26061546700893e-13,
                                         3.63343367212515e-08,
                                         4.98753846550757e-05,
                                         0.00518798828125,
                                         0.0816480862206196,
                                         0.242751479807406,
                                         0.113136794156934,
                                         0.00219156683211107,
                                         1.40830516794234e-25,
                                         5.82386110372131e-15,
                                         1.74143315861213e-09,
                                         5.09013151734302e-06,
                                         0.00103759765625,
                                         0.0320009783065867,
                                         0.202596670754422,
                                         0.253368392100004,
                                         0.0270941403734351,
                                         2.84784353093846e-28,
                                         6.50133358613879e-17,
                                         5.21646622652548e-11,
                                         3.24677180976037e-07,
                                         0.00012969970703125,
                                         0.00783899736648096,
                                         0.105677550949793,
                                         0.354634530004749,
                                         0.209351487660394,
                                         2.71004524113704e-31,
                                         3.41534796815554e-19,
                                         7.35338078441033e-13,
                                         9.74575737037344e-09,
                                         7.62939453125e-06,
                                         0.000903647075271148,
                                         0.0259402550209388,
                                         0.233588076015323,
                                         0.761233323889896,

                                         0.799911595033025,
                                         0.304986864201106,
                                         0.0513041702808366,
                                         0.00347305893107688,
                                         8.07463817567011e-05,
                                         4.9247332281505e-07,
                                         4.87811217328626e-10,
                                         3.04543637785855e-14,
                                         1.24364912270302e-20,
                                         9.19457629799216e-33,
                                         0.179761296478804,
                                         0.375116644133917,
                                         0.166492825187304,
                                         0.0233386198342505,
                                         0.0010169348889308,
                                         1.13008334904983e-05,
                                         2.09790881735435e-08,
                                         2.71209263524323e-12,
                                         2.92220386477081e-18,
                                         1.18242979394098e-29,
                                         0.019010408833936,
                                         0.217116385265628,
                                         0.254260819799779,
                                         0.0738038899631652,
                                         0.00602704299459654,
                                         0.000122033567553851,
                                         4.24582900306999e-07,
                                         1.13658147501772e-10,
                                         3.23120282132376e-16,
                                         7.15583096349387e-27,
                                         0.00125651228571054,
                                         0.0785414441880639,
                                         0.242685249396356,
                                         0.14586911662888,
                                         0.0223252046751657,
                                         0.000823622413694183,
                                         5.37054559229012e-06,
                                         2.97698498699513e-09,
                                         2.23304741808061e-14,
                                         2.70660449781498e-24,
                                         5.81353192639486e-05,
                                         0.0198885538432292,
                                         0.162145670829918,
                                         0.201811306120809,
                                         0.0578874806375895,
                                         0.00389112377648109,
                                         4.75523907265418e-05,
                                         5.45821647221491e-08,
                                         1.08026352197462e-12,
                                         7.16617757109628e-22,
                                         1.99810675477937e-06,
                                         0.003741216420258,
                                         0.0804771594354975,
                                         0.207411550006546,
                                         0.111501100017323,
                                         0.0136561173038234,
                                         0.000312774694322374,
                                         7.43413033061791e-07,
                                         3.88210055831157e-11,
                                         1.40946934652463e-19,
                                         5.28267583821693e-08,
                                         0.000541351202596095,
                                         0.0307253308216243,
                                         0.163974769314634,
                                         0.165207706772597,
                                         0.0368668540006506,
                                         0.0015825138080298,
                                         7.78872254108119e-06,
                                         1.07315016826031e-09,
                                         2.13245734411439e-17,
                                         1.09737202620225e-09,
                                         6.15474450773174e-05,
                                         0.00921690573709039,
                                         0.101855801280224,
                                         0.19232958886954,
                                         0.078200507039813,
                                         0.00629112127645956,
                                         6.41160775047282e-05,
                                         2.33087436073124e-08,
                                         2.53495187960936e-15,
                                         1.81329797397196e-11,
                                         5.56616813667004e-06,
                                         0.00219932338496931,
                                         0.0503280239147569,
                                         0.178105475622643,
                                         0.131946667731028,
                                         0.0198940852407283,
                                         0.000419839237995709,
                                         4.02710171793613e-07,
                                         2.39703543658934e-13,
                                         2.39703543658934e-13,
                                         4.02710171793616e-07,
                                         0.000419839237995709,
                                         0.0198940852407283,
                                         0.131946667731028,
                                         0.178105475622643,
                                         0.0503280239147569,
                                         0.00219932338496931,
                                         5.56616813667e-06,
                                         1.81329797397196e-11,
                                         2.53495187960936e-15,
                                         2.33087436073126e-08,
                                         6.41160775047282e-05,
                                         0.00629112127645956,
                                         0.078200507039813,
                                         0.19232958886954,
                                         0.101855801280224,
                                         0.00921690573709039,
                                         6.15474450773171e-05,
                                         1.09737202620225e-09,
                                         2.13245734411439e-17,
                                         1.07315016826032e-09,
                                         7.78872254108119e-06,
                                         0.0015825138080298,
                                         0.0368668540006506,
                                         0.165207706772597,
                                         0.163974769314634,
                                         0.0307253308216243,
                                         0.000541351202596092,
                                         5.28267583821693e-08,
                                         1.40946934652463e-19,
                                         3.8821005583116e-11,
                                         7.43413033061791e-07,
                                         0.000312774694322374,
                                         0.0136561173038234,
                                         0.111501100017323,
                                         0.207411550006546,
                                         0.0804771594354975,
                                         0.00374121642025798,
                                         1.99810675477937e-06,
                                         7.16617757109628e-22,
                                         1.08026352197463e-12,
                                         5.45821647221491e-08,
                                         4.75523907265417e-05,
                                         0.00389112377648109,
                                         0.0578874806375895,
                                         0.201811306120809,
                                         0.162145670829918,
                                         0.0198885538432291,
                                         5.81353192639486e-05,
                                         2.70660449781498e-24,
                                         2.23304741808063e-14,
                                         2.97698498699513e-09,
                                         5.37054559229012e-06,
                                         0.000823622413694183,
                                         0.0223252046751657,
                                         0.14586911662888,
                                         0.242685249396356,
                                         0.0785414441880637,
                                         0.00125651228571054,
                                         7.15583096349387e-27,
                                         3.2312028213238e-16,
                                         1.13658147501772e-10,
                                         4.24582900306999e-07,
                                         0.000122033567553851,
                                         0.00602704299459654,
                                         0.0738038899631651,
                                         0.254260819799779,
                                         0.217116385265627,
                                         0.019010408833936,
                                         1.18242979394098e-29,
                                         2.92220386477085e-18,
                                         2.71209263524323e-12,
                                         2.09790881735435e-08,
                                         1.13008334904983e-05,
                                         0.0010169348889308,
                                         0.0233386198342505,
                                         0.166492825187304,
                                         0.375116644133916,
                                         0.179761296478804,
                                         9.19457629799216e-33,
                                         1.24364912270303e-20,
                                         3.04543637785855e-14,
                                         4.87811217328626e-10,
                                         4.9247332281505e-07,
                                         8.07463817567011e-05,
                                         0.00347305893107688,
                                         0.0513041702808366,
                                         0.304986864201106,
                                         0.799911595033025,

                                         0.830214641736034,
                                         0.372269364572094,
                                         0.085098617078643,
                                         0.00932050502003652,
                                         0.000441059356227914,
                                         7.62939453125e-06,
                                         3.660982370243e-08,
                                         3.00248405690594e-11,
                                         1.6274827065159e-15,
                                         6.03576663504965e-22,
                                         4.23192951143029e-34,
                                         0.155327380356271,
                                         0.378754410076634,
                                         0.225634943197415,
                                         0.0501604451140271,
                                         0.00431412827985852,
                                         0.00012969970703125,
                                         1.08168165650477e-06,
                                         1.61233995315068e-09,
                                         1.77390504865509e-13,
                                         1.71446996944521e-19,
                                         6.53700231312275e-31,
                                         0.0136756101802025,
                                         0.181342318609592,
                                         0.28153427767306,
                                         0.127035290274304,
                                         0.0198577544266828,
                                         0.00103759765625,
                                         1.50398102790065e-05,
                                         4.07449310442336e-08,
                                         9.09882731322872e-12,
                                         2.29175607097187e-17,
                                         4.7518193048241e-28,
                                         0.000752532784995346,
                                         0.0542651049793208,
                                         0.219551403424312,
                                         0.201079318305379,
                                         0.0571277650561907,
                                         0.00518798828125,
                                         0.000130696894430691,
                                         6.43532635035827e-07,
                                         2.91689296373008e-10,
                                         1.91463906573119e-15,
                                         2.15884376585456e-25,
                                         2.89869270563785e-05,
                                         0.0113668510951898,
                                         0.119850319473685,
                                         0.222796551367695,
                                         0.115043575881422,
                                         0.018157958984375,
                                         0.000795034945777848,
                                         7.11484764410554e-06,
                                         6.54566241150831e-09,
                                         1.11970464872756e-13,
                                         6.86563245459757e-23,
                                         8.294385831823e-07,
                                         0.00176874407364459,
                                         0.0486012543501561,
                                         0.183381207363691,
                                         0.172100770366078,
                                         0.047210693359375,
                                         0.00359262983732457,
                                         5.84340431600162e-05,
                                         1.09116899424252e-07,
                                         4.86435598726372e-12,
                                         1.62197886192114e-20,
                                         1.82567276709412e-08,
                                         0.000211712486249855,
                                         0.0151604610123533,
                                         0.116106849008923,
                                         0.198043185834786,
                                         0.09442138671875,
                                         0.0124880714282089,
                                         0.000369167038027267,
                                         1.39922346121195e-06,
                                         1.62556323001496e-10,
                                         2.94758594953428e-18,
                                         3.15737590642612e-10,
                                         1.99109781794452e-05,
                                         0.00371571150489661,
                                         0.0577597759281279,
                                         0.179061262892583,
                                         0.14837646484375,
                                         0.0341069531957213,
                                         0.00183250237673814,
                                         1.40976521196856e-05,
                                         4.26822232680941e-09,
                                         4.20874307447653e-16,
                                         4.34355186501973e-12,
                                         1.48954657885824e-06,
                                         0.000724414165801217,
                                         0.0228564368688829,
                                         0.128783062996376,
                                         0.185470581054688,
                                         0.0740978905065711,
                                         0.0072357165234337,
                                         0.000112985280568078,
                                         8.914676077055e-08,
                                         4.78028422671343e-14,
                                         4.78028422671321e-14,
                                         8.914676077055e-08,
                                         0.000112985280568078,
                                         0.00723571652343372,
                                         0.074097890506571,
                                         0.185470581054688,
                                         0.128783062996377,
                                         0.0228564368688829,
                                         0.000724414165801215,
                                         1.48954657885824e-06,
                                         4.34355186501991e-12,
                                         4.20874307447632e-16,
                                         4.26822232680941e-09,
                                         1.40976521196856e-05,
                                         0.00183250237673815,
                                         0.0341069531957212,
                                         0.14837646484375,
                                         0.179061262892583,
                                         0.0577597759281279,
                                         0.0037157115048966,
                                         1.99109781794452e-05,
                                         3.15737590642623e-10,
                                         2.94758594953411e-18,
                                         1.62556323001496e-10,
                                         1.39922346121196e-06,
                                         0.000369167038027268,
                                         0.0124880714282089,
                                         0.09442138671875,
                                         0.198043185834787,
                                         0.116106849008923,
                                         0.0151604610123533,
                                         0.000211712486249855,
                                         1.82567276709418e-08,
                                         1.62197886192104e-20,
                                         4.86435598726372e-12,
                                         1.09116899424253e-07,
                                         5.84340431600164e-05,
                                         0.00359262983732457,
                                         0.047210693359375,
                                         0.172100770366078,
                                         0.183381207363691,
                                         0.048601254350156,
                                         0.00176874407364459,
                                         8.29438583182321e-07,
                                         6.86563245459711e-23,
                                         1.11970464872756e-13,
                                         6.54566241150835e-09,
                                         7.11484764410556e-06,
                                         0.000795034945777847,
                                         0.018157958984375,
                                         0.115043575881422,
                                         0.222796551367695,
                                         0.119850319473685,
                                         0.0113668510951898,
                                         2.8986927056379e-05,
                                         2.1588437658544e-25,
                                         1.91463906573119e-15,
                                         2.9168929637301e-10,
                                         6.43532635035829e-07,
                                         0.000130696894430691,
                                         0.00518798828125,
                                         0.0571277650561907,
                                         0.201079318305379,
                                         0.219551403424312,
                                         0.0542651049793208,
                                         0.000752532784995357,
                                         4.75181930482374e-28,
                                         2.29175607097187e-17,
                                         9.09882731322878e-12,
                                         4.07449310442338e-08,
                                         1.50398102790065e-05,
                                         0.00103759765625,
                                         0.0198577544266828,
                                         0.127035290274304,
                                         0.28153427767306,
                                         0.181342318609592,
                                         0.0136756101802026,
                                         6.53700231312222e-31,
                                         1.71446996944521e-19,
                                         1.77390504865511e-13,
                                         1.61233995315068e-09,
                                         1.08168165650477e-06,
                                         0.00012969970703125,
                                         0.00431412827985852,
                                         0.0501604451140271,
                                         0.225634943197415,
                                         0.378754410076634,
                                         0.155327380356272,
                                         4.23192951142992e-34,
                                         6.03576663504965e-22,
                                         1.62748270651591e-15,
                                         3.00248405690596e-11,
                                         3.66098237024299e-08,
                                         7.62939453125e-06,
                                         0.000441059356227914,
                                         0.00932050502003652,
                                         0.085098617078643,
                                         0.372269364572094,
                                         0.830214641736034,

                                         0.854309616348057,
                                         0.43379324534826,
                                         0.12520729304087,
                                         0.0196687545293189,
                                         0.00156691083761615,
                                         5.67041154344666e-05,
                                         7.8461621012609e-07,
                                         3.13796840196805e-09,
                                         2.2282034586662e-12,
                                         1.08389393501662e-16,
                                         3.73317001035061e-23,
                                         2.51287979214105e-35,
                                         0.135145885809536,
                                         0.371345760707461,
                                         0.276720117576077,
                                         0.0869315509068933,
                                         0.0123110035808224,
                                         0.000749398935292166,
                                         1.71576035006669e-05,
                                         1.15424296333342e-07,
                                         1.45697506523979e-10,
                                         1.41733829569714e-14,
                                         1.26031738210749e-20,
                                         4.59073287023724e-32,
                                         0.0100607748273744,
                                         0.149594360337636,
                                         0.28780140277432,
                                         0.180808590583439,
                                         0.0455180550248572,
                                         0.00466071588228341,
                                         0.000176561920464671,
                                         1.99796083511591e-06,
                                         4.48322422363729e-09,
                                         8.72170358697937e-13,
                                         2.00227234635415e-18,
                                         3.94669346428512e-29,
                                         0.000468101514526405,
                                         0.0376644811465475,
                                         0.187079024476994,
                                         0.235039422451634,
                                         0.105185034248216,
                                         0.0181164126278249,
                                         0.00113557932773658,
                                         2.16150738268781e-05,
                                         8.62201588025298e-08,
                                         3.35435591180234e-11,
                                         1.9881373760942e-16,
                                         2.12062944813454e-26,
                                         1.52456766166756e-05,
                                         0.00663814595540746,
                                         0.0851246475636382,
                                         0.213875186735847,
                                         0.170146197956601,
                                         0.0492935184966845,
                                         0.00511253097118686,
                                         0.000163690892149045,
                                         1.16071398377027e-06,
                                         9.03056659697618e-10,
                                         1.38187153380708e-14,
                                         7.97616665163783e-24,
                                         3.68857539014836e-07,
                                         0.000869094314538393,
                                         0.0287733786328114,
                                         0.144572401712539,
                                         0.204454111620929,
                                         0.0996351862309582,
                                         0.0170985675924177,
                                         0.00092086848817516,
                                         1.16077198655868e-05,
                                         1.8060349973704e-08,
                                         7.13500455704242e-13,
                                         2.22858372448567e-21,
                                         6.86479026197978e-09,
                                         8.7527327502513e-05,
                                         0.0074814028752383,
                                         0.0751738923273776,
                                         0.188984460968659,
                                         0.154914583218566,
                                         0.0439886014849761,
                                         0.0039849910983059,
                                         8.92946147881223e-05,
                                         2.77839489037231e-07,
                                         2.83385409867704e-11,
                                         4.78983261967999e-19,
                                         1.00383083269804e-10,
                                         6.92604130261523e-06,
                                         0.00152841000966025,
                                         0.0307123698317955,
                                         0.137252730575748,
                                         0.189250274653888,
                                         0.088917087651596,
                                         0.0135494525651158,
                                         0.000539719696075338,
                                         3.35835367685693e-06,
                                         8.84352419241507e-10,
                                         8.08865606942808e-17,
                                         1.1676405176958e-12,
                                         4.35955191548974e-07,
                                         0.00024837739744473,
                                         0.00998102142726313,
                                         0.0792923443147298,
                                         0.183906086111781,
                                         0.142970248559198,
                                         0.0366464162311484,
                                         0.0025949358978701,
                                         3.229046194228e-05,
                                         2.19527341214948e-08,
                                         1.08654513023907e-14,
                                         1.08654513023913e-14,
                                         2.19527341214946e-08,
                                         3.22904619422798e-05,
                                         0.00259493589787009,
                                         0.0366464162311484,
                                         0.142970248559198,
                                         0.183906086111781,
                                         0.0792923443147299,
                                         0.00998102142726315,
                                         0.00024837739744473,
                                         4.35955191548978e-07,
                                         1.16764051769575e-12,
                                         8.08865606942857e-17,
                                         8.84352419241496e-10,
                                         3.35835367685691e-06,
                                         0.000539719696075337,
                                         0.0135494525651158,
                                         0.088917087651596,
                                         0.189250274653888,
                                         0.137252730575748,
                                         0.0307123698317956,
                                         0.00152841000966025,
                                         6.92604130261529e-06,
                                         1.00383083269799e-10,
                                         4.7898326196803e-19,
                                         2.833854098677e-11,
                                         2.7783948903723e-07,
                                         8.92946147881221e-05,
                                         0.00398499109830589,
                                         0.0439886014849761,
                                         0.154914583218566,
                                         0.188984460968659,
                                         0.0751738923273778,
                                         0.00748140287523832,
                                         8.75273275025136e-05,
                                         6.86479026197953e-09,
                                         2.22858372448583e-21,
                                         7.13500455704233e-13,
                                         1.80603499737039e-08,
                                         1.16077198655868e-05,
                                         0.000920868488175159,
                                         0.0170985675924177,
                                         0.0996351862309583,
                                         0.204454111620929,
                                         0.14457240171254,
                                         0.0287733786328114,
                                         0.000869094314538398,
                                         3.68857539014825e-07,
                                         7.97616665163845e-24,
                                         1.38187153380706e-14,
                                         9.03056659697612e-10,
                                         1.16071398377027e-06,
                                         0.000163690892149045,
                                         0.00511253097118686,
                                         0.0492935184966845,
                                         0.170146197956601,
                                         0.213875186735847,
                                         0.0851246475636384,
                                         0.00663814595540749,
                                         1.52456766166752e-05,
                                         2.12062944813472e-26,
                                         1.98813737609417e-16,
                                         3.35435591180232e-11,
                                         8.62201588025295e-08,
                                         2.16150738268781e-05,
                                         0.00113557932773658,
                                         0.0181164126278249,
                                         0.105185034248216,
                                         0.235039422451634,
                                         0.187079024476995,
                                         0.0376644811465476,
                                         0.000468101514526396,
                                         3.94669346428548e-29,
                                         2.00227234635412e-18,
                                         8.72170358697931e-13,
                                         4.48322422363727e-09,
                                         1.9979608351159e-06,
                                         0.000176561920464671,
                                         0.00466071588228341,
                                         0.0455180550248572,
                                         0.180808590583439,
                                         0.287801402774321,
                                         0.149594360337636,
                                         0.0100607748273743,
                                         4.59073287023768e-32,
                                         1.26031738210747e-20,
                                         1.41733829569713e-14,
                                         1.45697506523979e-10,
                                         1.15424296333342e-07,
                                         1.71576035006669e-05,
                                         0.000749398935292166,
                                         0.0123110035808224,
                                         0.0869315509068933,
                                         0.276720117576077,
                                         0.371345760707461,
                                         0.135145885809535,
                                         2.51287979214131e-35,
                                         3.73317001035054e-23,
                                         1.08389393501661e-16,
                                         2.22820345866619e-12,
                                         3.13796840196804e-09,
                                         7.8461621012609e-07,
                                         5.67041154344666e-05,
                                         0.00156691083761615,
                                         0.0196687545293189,
                                         0.12520729304087,
                                         0.43379324534826,
                                         0.854309616348057,

                                         0.873734716008478,
                                         0.489069149020839,
                                         0.16927492587847,
                                         0.0351078517868548,
                                         0.00415018654259668,
                                         0.000259195508569522,
                                         7.62939453125e-06,
                                         8.8804835345928e-08,
                                         3.08220060347542e-10,
                                         1.95566115341902e-13,
                                         8.74004533400025e-18,
                                         2.84050854530568e-24,
                                         1.85171643379453e-36,
                                         0.118404821940239,
                                         0.357270517834782,
                                         0.316940753587818,
                                         0.129971062413266,
                                         0.0268628098688378,
                                         0.0027557615522413,
                                         0.00012969970703125,
                                         2.41390564934609e-06,
                                         1.37617899086543e-08,
                                         1.52668359604968e-11,
                                         1.34904387397052e-15,
                                         1.12374308262949e-21,
                                         3.94895810664196e-33,
                                         0.00755092665560418,
                                         0.122818886757314,
                                         0.279257345656199,
                                         0.226428008752807,
                                         0.0818231743836789,
                                         0.0137878600728816,
                                         0.00103759765625,
                                         3.08777064764778e-05,
                                         2.89154537300229e-07,
                                         5.60848464047692e-10,
                                         9.79894970576731e-14,
                                         2.09208356737566e-19,
                                         3.96306903492969e-30,
                                         0.000300961631165088,
                                         0.026388405618599,
                                         0.153783996338723,
                                         0.246543548789827,
                                         0.155769070205973,
                                         0.0431153697865174,
                                         0.00518798828125,
                                         0.000246859471678657,
                                         3.79721437928334e-06,
                                         1.28772179955363e-08,
                                         4.44849391061463e-12,
                                         2.43428286708658e-17,
                                         2.48576899138312e-27,
                                         8.39692070858975e-06,
                                         0.00396879973946816,
                                         0.0592810270828713,
                                         0.187912287217823,
                                         0.20757936111522,
                                         0.0943768337799606,
                                         0.018157958984375,
                                         0.0013815054289321,
                                         3.49058535404739e-05,
                                         2.06964853706586e-07,
                                         1.41365851105347e-10,
                                         1.98271867270447e-15,
                                         1.09141001502691e-24,
                                         1.74034069672712e-07,
                                         0.000443415146129197,
                                         0.0169756182414181,
                                         0.106395195769644,
                                         0.205490816575861,
                                         0.153463109896771,
                                         0.047210693359375,
                                         0.00574328939537428,
                                         0.000238361830029527,
                                         2.47102112051255e-06,
                                         3.33719302864927e-09,
                                         1.1996552377978e-13,
                                         3.55975744336043e-22,
                                         2.77463044608567e-09,
                                         3.81082064572315e-05,
                                         0.00373931565861083,
                                         0.0463388770386141,
                                         0.156479450595711,
                                         0.191954913868093,
                                         0.09442138671875,
                                         0.0183664597876759,
                                         0.00125207921952217,
                                         2.26940998720171e-05,
                                         6.06003023273172e-08,
                                         5.58352493623405e-12,
                                         8.9311940736431e-20,
                                         3.47568707154615e-11,
                                         2.57330429390853e-06,
                                         0.000647177344735123,
                                         0.0158574606187629,
                                         0.0936239277307696,
                                         0.188651004890554,
                                         0.14837646484375,
                                         0.0461482093151811,
                                         0.0051676317019548,
                                         0.000163762370254357,
                                         8.64635182857234e-07,
                                         2.04185602094043e-10,
                                         1.760610533869e-17,
                                         3.46331151851268e-13,
                                         1.3822262862128e-07,
                                         8.90983889603563e-05,
                                         0.00431655367778818,
                                         0.0445586237685804,
                                         0.147480424689907,
                                         0.185470581054688,
                                         0.0922358221387009,
                                         0.0169655001687982,
                                         0.000940006024983851,
                                         9.81310360124557e-06,
                                         5.93959917082553e-09,
                                         2.76078402396016e-15,
                                         2.76078402396033e-15,
                                         5.9395991708256e-09,
                                         9.81310360124552e-06,
                                         0.000940006024983854,
                                         0.0169655001687982,
                                         0.0922358221387009,
                                         0.185470581054688,
                                         0.147480424689907,
                                         0.0445586237685804,
                                         0.00431655367778817,
                                         8.90983889603567e-05,
                                         1.38222628621279e-07,
                                         3.46331151851248e-13,
                                         1.76061053386912e-17,
                                         2.04185602094045e-10,
                                         8.6463518285723e-07,
                                         0.000163762370254358,
                                         0.0051676317019548,
                                         0.0461482093151811,
                                         0.14837646484375,
                                         0.188651004890554,
                                         0.0936239277307696,
                                         0.0158574606187629,
                                         0.000647177344735126,
                                         2.57330429390851e-06,
                                         3.47568707154598e-11,
                                         8.93119407364378e-20,
                                         5.58352493623413e-12,
                                         6.06003023273168e-08,
                                         2.26940998720172e-05,
                                         0.00125207921952217,
                                         0.0183664597876759,
                                         0.09442138671875,
                                         0.191954913868093,
                                         0.156479450595711,
                                         0.0463388770386141,
                                         0.00373931565861084,
                                         3.81082064572312e-05,
                                         2.77463044608555e-09,
                                         3.55975744336073e-22,
                                         1.19965523779782e-13,
                                         3.33719302864924e-09,
                                         2.47102112051255e-06,
                                         0.000238361830029527,
                                         0.00574328939537428,
                                         0.047210693359375,
                                         0.153463109896771,
                                         0.205490816575861,
                                         0.106395195769644,
                                         0.0169756182414182,
                                         0.000443415146129194,
                                         1.74034069672706e-07,
                                         1.09141001502701e-24,
                                         1.9827186727045e-15,
                                         1.41365851105346e-10,
                                         2.06964853706587e-07,
                                         3.49058535404739e-05,
                                         0.0013815054289321,
                                         0.018157958984375,
                                         0.0943768337799606,
                                         0.20757936111522,
                                         0.187912287217823,
                                         0.0592810270828714,
                                         0.00396879973946814,
                                         8.39692070858952e-06,
                                         2.48576899138337e-27,
                                         2.43428286708663e-17,
                                         4.4484939106146e-12,
                                         1.28772179955363e-08,
                                         3.79721437928333e-06,
                                         0.000246859471678657,
                                         0.00518798828125,
                                         0.0431153697865174,
                                         0.155769070205973,
                                         0.246543548789827,
                                         0.153783996338723,
                                         0.0263884056185989,
                                         0.000300961631165081,
                                         3.96306903493011e-30,
                                         2.0920835673757e-19,
                                         9.79894970576723e-14,
                                         5.60848464047695e-10,
                                         2.89154537300229e-07,
                                         3.08777064764778e-05,
                                         0.00103759765625,
                                         0.0137878600728816,
                                         0.0818231743836789,
                                         0.226428008752807,
                                         0.279257345656199,
                                         0.122818886757313,
                                         0.00755092665560408,
                                         3.94895810664241e-33,
                                         1.12374308262952e-21,
                                         1.34904387397051e-15,
                                         1.52668359604969e-11,
                                         1.37617899086543e-08,
                                         2.41390564934609e-06,
                                         0.00012969970703125,
                                         0.0027557615522413,
                                         0.0268628098688378,
                                         0.129971062413266,
                                         0.316940753587818,
                                         0.357270517834781,
                                         0.118404821940238,
                                         1.85171643379475e-36,
                                         2.84050854530575e-24,
                                         8.74004533400016e-18,
                                         1.95566115341903e-13,
                                         3.08220060347542e-10,
                                         8.8804835345928e-08,
                                         7.62939453125e-06,
                                         0.000259195508569522,
                                         0.00415018654259668,
                                         0.0351078517868548,
                                         0.16927492587847,
                                         0.489069149020839,
                                         0.873734716008478,

                                         0.889594758108883,
                                         0.538238444513063,
                                         0.215208221773865,
                                         0.0555556506711015,
                                         0.00892722344229672,
                                         0.000845825059716832,
                                         4.36548132285582e-05,
                                         1.09203806920542e-06,
                                         1.10859428935882e-08,
                                         3.43806231838634e-11,
                                         1.9944886717162e-14,
                                         8.33062210940558e-19,
                                         2.58472034334222e-25,
                                         1.64232894310906e-37,
                                         0.104431945288415,
                                         0.339562747396132,
                                         0.345988619772711,
                                         0.175034804913216,
                                         0.048551255395963,
                                         0.00742203711467096,
                                         0.000597389876184047,
                                         2.306268659163e-05,
                                         3.6511351260014e-07,
                                         1.82695364503338e-09,
                                         1.82950348159789e-12,
                                         1.49751893399311e-16,
                                         1.18403919683622e-22,
                                         4.04312191150242e-34,
                                         0.00576919988359471,
                                         0.100810653143775,
                                         0.261761538827565,
                                         0.259514516988379,
                                         0.124258351771405,
                                         0.030648326179186,
                                         0.00384702180610304,
                                         0.000229204469233227,
                                         5.65879864523545e-06,
                                         4.56859202792748e-08,
                                         7.89725155924442e-11,
                                         1.26680064336778e-14,
                                         2.55246422580525e-20,
                                         4.68397525924736e-31,
                                         0.000199194719614698,
                                         0.0187056513582649,
                                         0.123774127409138,
                                         0.240479973972544,
                                         0.198760797468354,
                                         0.0790988952117545,
                                         0.0154835825884426,
                                         0.00142369278259714,
                                         5.48151482385493e-05,
                                         7.1403129105053e-07,
                                         2.13058429572916e-09,
                                         6.69767771215201e-13,
                                         3.43900862901329e-18,
                                         3.39150424929442e-28,
                                         4.8143548474799e-06,
                                         0.00242961400683071,
                                         0.0409686781303264,
                                         0.155989086803563,
                                         0.222553235520571,
                                         0.142899962333943,
                                         0.0436230776168082,
                                         0.00619024054024747,
                                         0.000371684957417738,
                                         7.81178264633538e-06,
                                         4.02364364995537e-08,
                                         2.47878155726387e-11,
                                         3.24343282135041e-16,
                                         1.71896953024924e-25,
                                         8.64377943780619e-08,
                                         0.00023442667800892,
                                         0.0100734754727059,
                                         0.0751648540679703,
                                         0.185115335340651,
                                         0.191778145461272,
                                         0.0912990907575783,
                                         0.0199942017743811,
                                         0.00187221051753523,
                                         6.34875198183905e-05,
                                         5.6447615883889e-07,
                                         6.81486991633227e-10,
                                         2.27238467857454e-14,
                                         6.47216074949135e-23,
                                         1.19378441119448e-09,
                                         1.73993663215616e-05,
                                         0.00190529997981541,
                                         0.0278607029150898,
                                         0.118442490670142,
                                         0.197980657645802,
                                         0.146985079185363,
                                         0.0496772176479893,
                                         0.00725422503865711,
                                         0.000396901937966093,
                                         6.09155745040353e-06,
                                         1.44123079156292e-08,
                                         1.22465975257147e-12,
                                         1.87450697060629e-20,
                                         1.29542650851773e-11,
                                         1.01466920445928e-06,
                                         0.000283147050404983,
                                         0.00811398068383574,
                                         0.059543905144959,
                                         0.160587249614268,
                                         0.185927957506345,
                                         0.0969784204441761,
                                         0.0220847260728361,
                                         0.00194958747902685,
                                         5.16506432551608e-05,
                                         2.39482698659145e-07,
                                         5.18577521566509e-11,
                                         4.26569179547794e-18,
                                         1.11818850686472e-13,
                                         4.70685582096024e-08,
                                         3.34715703931533e-05,
                                         0.00187971166922227,
                                         0.0238112648008656,
                                         0.103613113774295,
                                         0.187081788890751,
                                         0.150594224575911,
                                         0.0534820908723064,
                                         0.00761759014678858,
                                         0.000348368190370409,
                                         3.16541111194737e-06,
                                         1.74673610843354e-09,
                                         7.72159920250542e-16,
                                         7.72159920250542e-16,
                                         1.74673610843356e-09,
                                         3.16541111194737e-06,
                                         0.000348368190370408,
                                         0.00761759014678857,
                                         0.0534820908723064,
                                         0.150594224575911,
                                         0.187081788890751,
                                         0.103613113774295,
                                         0.0238112648008656,
                                         0.00187971166922228,
                                         3.34715703931533e-05,
                                         4.70685582096018e-08,
                                         1.11818850686472e-13,
                                         4.26569179547794e-18,
                                         5.18577521566517e-11,
                                         2.39482698659145e-07,
                                         5.16506432551606e-05,
                                         0.00194958747902685,
                                         0.0220847260728361,
                                         0.096978420444176,
                                         0.185927957506345,
                                         0.160587249614268,
                                         0.0595439051449591,
                                         0.00811398068383576,
                                         0.000283147050404983,
                                         1.01466920445927e-06,
                                         1.29542650851773e-11,
                                         1.87450697060629e-20,
                                         1.22465975257149e-12,
                                         1.44123079156292e-08,
                                         6.0915574504035e-06,
                                         0.000396901937966092,
                                         0.00725422503865711,
                                         0.0496772176479893,
                                         0.146985079185363,
                                         0.197980657645802,
                                         0.118442490670142,
                                         0.0278607029150898,
                                         0.00190529997981541,
                                         1.73993663215614e-05,
                                         1.19378441119448e-09,
                                         6.47216074949135e-23,
                                         2.27238467857459e-14,
                                         6.81486991633227e-10,
                                         5.64476158838888e-07,
                                         6.34875198183904e-05,
                                         0.00187221051753523,
                                         0.0199942017743811,
                                         0.0912990907575783,
                                         0.191778145461272,
                                         0.185115335340651,
                                         0.0751648540679705,
                                         0.0100734754727059,
                                         0.000234426678008919,
                                         8.64377943780619e-08,
                                         1.71896953024924e-25,
                                         3.24343282135048e-16,
                                         2.47878155726387e-11,
                                         4.02364364995535e-08,
                                         7.81178264633536e-06,
                                         0.000371684957417738,
                                         0.00619024054024746,
                                         0.0436230776168082,
                                         0.142899962333943,
                                         0.222553235520571,
                                         0.155989086803563,
                                         0.0409686781303264,
                                         0.0024296140068307,
                                         4.8143548474799e-06,
                                         3.39150424929442e-28,
                                         3.43900862901336e-18,
                                         6.69767771215201e-13,
                                         2.13058429572915e-09,
                                         7.14031291050528e-07,
                                         5.48151482385493e-05,
                                         0.00142369278259714,
                                         0.0154835825884426,
                                         0.0790988952117544,
                                         0.198760797468355,
                                         0.240479973972544,
                                         0.123774127409138,
                                         0.0187056513582648,
                                         0.000199194719614698,
                                         4.68397525924736e-31,
                                         2.55246422580531e-20,
                                         1.26680064336778e-14,
                                         7.89725155924438e-11,
                                         4.56859202792746e-08,
                                         5.65879864523545e-06,
                                         0.000229204469233227,
                                         0.00384702180610305,
                                         0.030648326179186,
                                         0.124258351771405,
                                         0.259514516988379,
                                         0.261761538827565,
                                         0.100810653143775,
                                         0.00576919988359471,
                                         4.04312191150242e-34,
                                         1.18403919683625e-22,
                                         1.49751893399311e-16,
                                         1.82950348159788e-12,
                                         1.82695364503338e-09,
                                         3.6511351260014e-07,
                                         2.30626865916299e-05,
                                         0.000597389876184047,
                                         0.00742203711467096,
                                         0.048551255395963,
                                         0.175034804913217,
                                         0.345988619772711,
                                         0.339562747396132,
                                         0.104431945288415,
                                         1.64232894310906e-37,
                                         2.58472034334229e-25,
                                         8.33062210940558e-19,
                                         1.99448867171619e-14,
                                         3.43806231838633e-11,
                                         1.10859428935882e-08,
                                         1.09203806920541e-06,
                                         4.36548132285582e-05,
                                         0.000845825059716832,
                                         0.00892722344229672,
                                         0.0555556506711015,
                                         0.215208221773865,
                                         0.538238444513063,
                                         0.889594758108883,

                                         0.902694399949043,
                                         0.581748947168752,
                                         0.261365820344349,
                                         0.0804289189020854,
                                         0.0164950805924533,
                                         0.00216650033125363,
                                         0.000172152872317438,
                                         7.62939453125e-06,
                                         1.67491054114252e-07,
                                         1.52293218785328e-09,
                                         4.31263724539781e-12,
                                         2.32681406719828e-15,
                                         9.20004581051338e-20,
                                         2.74897431799738e-26,
                                         1.71069707890484e-38,
                                         0.0926887163389483,
                                         0.320217759274191,
                                         0.364922076623139,
                                         0.218509308995939,
                                         0.0765808765518103,
                                         0.0160052294680607,
                                         0.00194621705276805,
                                         0.00012969970703125,
                                         4.28166276599808e-06,
                                         5.95765380781975e-08,
                                         2.68457091589041e-10,
                                         2.47515163920369e-13,
                                         1.90430491311056e-17,
                                         1.44330699698249e-23,
                                         4.81487300896989e-35,
                                         0.00447872144811695,
                                         0.0829461605619895,
                                         0.239768732767412,
                                         0.279362894486244,
                                         0.167312076736289,
                                         0.0556424335608356,
                                         0.0103540281050855,
                                         0.00103759765625,
                                         5.15079619552948e-05,
                                         1.09675854410551e-06,
                                         7.86407954915233e-09,
                                         1.2390333806349e-11,
                                         1.85491516367833e-15,
                                         3.56605319411715e-21,
                                         6.37731143524064e-32,
                                         0.000135257468506673,
                                         0.0134284899742322,
                                         0.0984611662592362,
                                         0.223227408406382,
                                         0.228462152385975,
                                         0.120901125580173,
                                         0.03442765345996,
                                         0.00518798828125,
                                         0.000387272172311318,
                                         1.26190542333291e-05,
                                         1.43979589972211e-07,
                                         3.87653955724267e-10,
                                         1.12925398683384e-13,
                                         5.5067699603497e-19,
                                         5.27922816789307e-29,
                                         2.85934459178474e-06,
                                         0.00152179485146035,
                                         0.028303193683612,
                                         0.12486029387986,
                                         0.218373169849359,
                                         0.18388767100562,
                                         0.0801315504951987,
                                         0.018157958984375,
                                         0.00203824439615431,
                                         0.00010163437651767,
                                         1.84523637372925e-06,
                                         8.48991756123374e-09,
                                         4.81235052807455e-12,
                                         5.95256425573373e-17,
                                         3.05915356842484e-26,
                                         4.49031742848265e-08,
                                         0.000128112157560429,
                                         0.00604381572996613,
                                         0.0518807794098691,
                                         0.15505636411769,
                                         0.20776872955972,
                                         0.138549503275692,
                                         0.047210693359375,
                                         0.00796895778086267,
                                         0.000608078644468089,
                                         1.75674362030572e-05,
                                         1.38123644807967e-07,
                                         1.52344990523918e-10,
                                         4.77987515272984e-15,
                                         1.31685339867981e-23,
                                         5.424306692914e-10,
                                         8.29623848395869e-06,
                                         0.000992758544552415,
                                         0.0165823195024757,
                                         0.0846908636327683,
                                         0.180577819168666,
                                         0.184273569181536,
                                         0.09442138671875,
                                         0.0239664352748449,
                                         0.00279856580426471,
                                         0.000128653442304092,
                                         1.72857900765919e-06,
                                         3.70984487683302e-09,
                                         2.95247114638637e-13,
                                         4.36043911349854e-21,
                                         5.14844502709841e-12,
                                         4.22120807059667e-07,
                                         0.000128127013155513,
                                         0.00416436430285705,
                                         0.0363452931917717,
                                         0.123314251961416,
                                         0.19256873250395,
                                         0.14837646484375,
                                         0.0566330574870503,
                                         0.0101198935547177,
                                         0.000740285232774407,
                                         1.69971101351238e-05,
                                         7.09819575534096e-08,
                                         1.43291167220911e-11,
                                         1.13445604552209e-18,
                                         3.88707831675869e-14,
                                         1.7084711899592e-08,
                                         1.31538576767243e-05,
                                         0.000831893094371381,
                                         0.0124072393243603,
                                         0.066984991975198,
                                         0.160075130243597,
                                         0.185470581054688,
                                         0.106451538582896,
                                         0.0291092980968415,
                                         0.00338838015342911,
                                         0.000132946316918142,
                                         1.08032782482497e-06,
                                         5.53182644987792e-10,
                                         2.34779670538812e-16,
                                         2.34779670538792e-16,
                                         5.53182644987792e-10,
                                         1.08032782482496e-06,
                                         0.000132946316918142,
                                         0.00338838015342911,
                                         0.0291092980968415,
                                         0.106451538582896,
                                         0.185470581054688,
                                         0.160075130243597,
                                         0.066984991975198,
                                         0.0124072393243603,
                                         0.000831893094371378,
                                         1.31538576767243e-05,
                                         1.7084711899592e-08,
                                         3.88707831675898e-14,
                                         1.13445604552198e-18,
                                         1.43291167220911e-11,
                                         7.0981957553409e-08,
                                         1.69971101351238e-05,
                                         0.000740285232774407,
                                         0.0101198935547177,
                                         0.0566330574870503,
                                         0.14837646484375,
                                         0.19256873250395,
                                         0.123314251961416,
                                         0.0363452931917717,
                                         0.00416436430285704,
                                         0.000128127013155513,
                                         4.22120807059667e-07,
                                         5.14844502709874e-12,
                                         4.36043911349809e-21,
                                         2.95247114638637e-13,
                                         3.709844876833e-09,
                                         1.7285790076592e-06,
                                         0.000128653442304092,
                                         0.00279856580426471,
                                         0.0239664352748449,
                                         0.09442138671875,
                                         0.184273569181536,
                                         0.180577819168666,
                                         0.0846908636327683,
                                         0.0165823195024756,
                                         0.000992758544552419,
                                         8.29623848395869e-06,
                                         5.4243066929143e-10,
                                         1.31685339867967e-23,
                                         4.77987515272984e-15,
                                         1.52344990523917e-10,
                                         1.38123644807968e-07,
                                         1.75674362030572e-05,
                                         0.000608078644468089,
                                         0.00796895778086267,
                                         0.047210693359375,
                                         0.138549503275692,
                                         0.20776872955972,
                                         0.15505636411769,
                                         0.051880779409869,
                                         0.00604381572996615,
                                         0.000128112157560429,
                                         4.49031742848286e-08,
                                         3.05915356842447e-26,
                                         5.95256425573373e-17,
                                         4.8123505280745e-12,
                                         8.48991756123378e-09,
                                         1.84523637372925e-06,
                                         0.00010163437651767,
                                         0.00203824439615431,
                                         0.018157958984375,
                                         0.0801315504951987,
                                         0.18388767100562,
                                         0.218373169849359,
                                         0.12486029387986,
                                         0.0283031936836121,
                                         0.00152179485146035,
                                         2.85934459178484e-06,
                                         5.27922816789238e-29,
                                         5.5067699603497e-19,
                                         1.12925398683383e-13,
                                         3.87653955724269e-10,
                                         1.43979589972211e-07,
                                         1.26190542333291e-05,
                                         0.000387272172311318,
                                         0.00518798828125,
                                         0.03442765345996,
                                         0.120901125580173,
                                         0.228462152385975,
                                         0.223227408406382,
                                         0.0984611662592364,
                                         0.0134284899742322,
                                         0.000135257468506677,
                                         6.37731143523975e-32,
                                         3.56605319411715e-21,
                                         1.85491516367831e-15,
                                         1.2390333806349e-11,
                                         7.86407954915233e-09,
                                         1.09675854410551e-06,
                                         5.15079619552948e-05,
                                         0.00103759765625,
                                         0.0103540281050855,
                                         0.0556424335608356,
                                         0.167312076736289,
                                         0.279362894486244,
                                         0.239768732767412,
                                         0.0829461605619895,
                                         0.00447872144811703,
                                         4.81487300896917e-35,
                                         1.44330699698249e-23,
                                         1.90430491311054e-17,
                                         2.47515163920371e-13,
                                         2.68457091589041e-10,
                                         5.95765380781975e-08,
                                         4.28166276599808e-06,
                                         0.00012969970703125,
                                         0.00194621705276805,
                                         0.0160052294680607,
                                         0.0765808765518103,
                                         0.218509308995939,
                                         0.36492207662314,
                                         0.320217759274191,
                                         0.0926887163389492,
                                         1.71069707890457e-38,
                                         2.74897431799738e-26,
                                         9.20004581051327e-20,
                                         2.32681406719829e-15,
                                         4.31263724539781e-12,
                                         1.52293218785328e-09,
                                         1.67491054114252e-07,
                                         7.62939453125e-06,
                                         0.000172152872317438,
                                         0.00216650033125363,
                                         0.0164950805924533,
                                         0.0804289189020854,
                                         0.261365820344349,
                                         0.581748947168752,
                                         0.902694399949043,

                                         0.913628155620945,
                                         0.620168963393625,
                                         0.306569267858942,
                                         0.10886849344452,
                                         0.0271994513304529,
                                         0.0046393051532459,
                                         0.000517968092532866,
                                         3.56955802841483e-05,
                                         1.39769967446103e-06,
                                         2.75855142451886e-08,
                                         2.2921835936195e-10,
                                         6.02502301414364e-13,
                                         3.06278637637188e-16,
                                         1.15761301153398e-20,
                                         3.35338896264012e-27,
                                         2.05134413215235e-39,
                                         0.0827491677306065,
                                         0.300496741623221,
                                         0.375361885845137,
                                         0.257882917725363,
                                         0.109211357163539,
                                         0.0293173817432156,
                                         0.00493585512493544,
                                         0.000501518390390368,
                                         2.87500954077897e-05,
                                         8.3660321785544e-07,
                                         1.04827444810684e-08,
                                         4.3365952753319e-11,
                                         3.7367512391848e-14,
                                         2.73237112192557e-18,
                                         2.00010149352455e-24,
                                         6.54549064768789e-36,
                                         0.00352694639974012,
                                         0.0685189299396178,
                                         0.216278223502662,
                                         0.287464356525266,
                                         0.206355728864166,
                                         0.0871843412028374,
                                         0.0221341508828722,
                                         0.00331589175694034,
                                         0.000278295242511317,
                                         1.19398547223881e-05,
                                         2.25601335232849e-07,
                                         1.46885910645996e-09,
                                         2.14542201814051e-12,
                                         3.03498842716811e-16,
                                         5.61385515856263e-22,
                                         9.82849351653648e-33,
                                         9.3953746362264e-05,
                                         0.009764755964386,
                                         0.0778852883800214,
                                         0.200274404081724,
                                         0.243694245389329,
                                         0.162043574899295,
                                         0.0620359368975809,
                                         0.0137023117626686,
                                         0.00168365184762675,
                                         0.000106502198226339,
                                         3.03450843376234e-06,
                                         3.10950604340354e-08,
                                         7.69859187383024e-11,
                                         2.1069508729823e-14,
                                         9.84805328734352e-20,
                                         9.22383916692346e-30,
                                         1.75197290191554e-06,
                                         0.00097411505681171,
                                         0.0196334269509806,
                                         0.0976708423631467,
                                         0.20145221988228,
                                         0.210825520531688,
                                         0.121708767640915,
                                         0.039635595183067,
                                         0.00713012002255547,
                                         0.000664991571802107,
                                         2.85715020150456e-05,
                                         4.60787522370762e-07,
                                         1.93378372353156e-09,
                                         1.02388179111185e-12,
                                         1.20930992282018e-17,
                                         6.05946844080303e-27,
                                         2.42686696881971e-08,
                                         7.21879023224537e-05,
                                         0.00367656368498014,
                                         0.0353842277810688,
                                         0.123709811342515,
                                         0.203760430932072,
                                         0.177380423761829,
                                         0.0851691331900382,
                                         0.0224308996932625,
                                         0.00308445908595349,
                                         0.000199840311871481,
                                         5.07242131803101e-06,
                                         3.60835956636951e-08,
                                         3.69615804606505e-11,
                                         1.1031387659146e-15,
                                         2.9570770296901e-24,
                                         2.58595643710655e-10,
                                         4.11505112751109e-06,
                                         0.000529596032766873,
                                         0.00986077747989972,
                                         0.0584376683667744,
                                         0.151486233402561,
                                         0.198859415634869,
                                         0.140778300507974,
                                         0.0542816691678887,
                                         0.0110052150761026,
                                         0.00107520128132895,
                                         4.29523149599271e-05,
                                         5.17926774324557e-07,
                                         1.02637932350916e-09,
                                         7.74068374991202e-14,
                                         1.11006240702805e-21,
                                         2.16501588808659e-12,
                                         1.84310763605998e-07,
                                         5.99393463472901e-05,
                                         0.00215912230767832,
                                         0.0216893363578273,
                                         0.088489376211055,
                                         0.175166591437368,
                                         0.182832693914568,
                                         0.103210609750049,
                                         0.0308519560818184,
                                         0.00454528476476178,
                                         0.000285773847188204,
                                         5.84105864830007e-06,
                                         2.23939088112914e-08,
                                         4.26769281593131e-12,
                                         3.27413665743301e-19,
                                         1.44183757396021e-14,
                                         6.56661461952279e-09,
                                         5.39628215911346e-06,
                                         0.000376061347862988,
                                         0.00640346464550457,
                                         0.0411172885635722,
                                         0.122735943999795,
                                         0.188880605585389,
                                         0.156102860609706,
                                         0.0687990184699444,
                                         0.015284378678749,
                                         0.00151242467873589,
                                         5.23998615006099e-05,
                                         3.88657707036645e-07,
                                         1.87164012422016e-10,
                                         7.68177490473964e-17,
                                         7.68177490474036e-17,
                                         1.87164012422016e-10,
                                         3.88657707036642e-07,
                                         5.23998615006101e-05,
                                         0.00151242467873589,
                                         0.015284378678749,
                                         0.0687990184699445,
                                         0.156102860609706,
                                         0.188880605585389,
                                         0.122735943999795,
                                         0.0411172885635722,
                                         0.00640346464550457,
                                         0.000376061347862986,
                                         5.39628215911349e-06,
                                         6.56661461952279e-09,
                                         1.44183757396009e-14,
                                         3.27413665743335e-19,
                                         4.26769281593131e-12,
                                         2.23939088112912e-08,
                                         5.84105864830009e-06,
                                         0.000285773847188204,
                                         0.00454528476476178,
                                         0.0308519560818184,
                                         0.103210609750049,
                                         0.182832693914568,
                                         0.175166591437368,
                                         0.088489376211055,
                                         0.0216893363578273,
                                         0.00215912230767832,
                                         5.99393463472905e-05,
                                         1.84310763605998e-07,
                                         2.16501588808643e-12,
                                         1.11006240702818e-21,
                                         7.74068374991202e-14,
                                         1.02637932350915e-09,
                                         5.1792677432456e-07,
                                         4.29523149599271e-05,
                                         0.00107520128132895,
                                         0.0110052150761026,
                                         0.0542816691678887,
                                         0.140778300507973,
                                         0.198859415634869,
                                         0.151486233402561,
                                         0.0584376683667744,
                                         0.00986077747989969,
                                         0.000529596032766875,
                                         4.11505112751109e-06,
                                         2.58595643710639e-10,
                                         2.95707702969047e-24,
                                         1.1031387659146e-15,
                                         3.69615804606501e-11,
                                         3.60835956636953e-08,
                                         5.07242131803101e-06,
                                         0.000199840311871481,
                                         0.0030844590859535,
                                         0.0224308996932625,
                                         0.085169133190038,
                                         0.177380423761829,
                                         0.203760430932072,
                                         0.123709811342515,
                                         0.0353842277810687,
                                         0.00367656368498016,
                                         7.21879023224537e-05,
                                         2.42686696881958e-08,
                                         6.05946844080385e-27,
                                         1.20930992282018e-17,
                                         1.02388179111184e-12,
                                         1.93378372353157e-09,
                                         4.60787522370762e-07,
                                         2.85715020150456e-05,
                                         0.000664991571802109,
                                         0.00713012002255546,
                                         0.039635595183067,
                                         0.121708767640915,
                                         0.210825520531688,
                                         0.20145221988228,
                                         0.0976708423631465,
                                         0.0196334269509806,
                                         0.000974115056811709,
                                         1.75197290191547e-06,
                                         9.22383916692482e-30,
                                         9.84805328734351e-20,
                                         2.10695087298228e-14,
                                         7.69859187383029e-11,
                                         3.10950604340354e-08,
                                         3.03450843376234e-06,
                                         0.000106502198226339,
                                         0.00168365184762674,
                                         0.0137023117626686,
                                         0.0620359368975809,
                                         0.162043574899295,
                                         0.243694245389329,
                                         0.200274404081723,
                                         0.0778852883800216,
                                         0.009764755964386,
                                         9.39537463622611e-05,
                                         9.82849351653802e-33,
                                         5.61385515856263e-22,
                                         3.03498842716807e-16,
                                         2.14542201814053e-12,
                                         1.46885910645996e-09,
                                         2.25601335232849e-07,
                                         1.19398547223881e-05,
                                         0.000278295242511316,
                                         0.00331589175694033,
                                         0.0221341508828722,
                                         0.0871843412028374,
                                         0.206355728864166,
                                         0.287464356525266,
                                         0.216278223502663,
                                         0.0685189299396178,
                                         0.00352694639974004,
                                         6.54549064768898e-36,
                                         2.00010149352455e-24,
                                         2.73237112192554e-18,
                                         3.73675123918483e-14,
                                         4.3365952753319e-11,
                                         1.04827444810684e-08,
                                         8.36603217855442e-07,
                                         2.87500954077896e-05,
                                         0.000501518390390366,
                                         0.00493585512493544,
                                         0.0293173817432156,
                                         0.109211357163539,
                                         0.257882917725363,
                                         0.375361885845138,
                                         0.300496741623221,
                                         0.0827491677306056,
                                         2.05134413215271e-39,
                                         3.35338896264012e-27,
                                         1.15761301153397e-20,
                                         3.0627863763719e-16,
                                         6.02502301414364e-13,
                                         2.2921835936195e-10,
                                         2.75855142451887e-08,
                                         1.39769967446103e-06,
                                         3.56955802841482e-05,
                                         0.000517968092532866,
                                         0.0046393051532459,
                                         0.0271994513304529,
                                         0.10886849344452,
                                         0.306569267858942,
                                         0.620168963393625,
                                         0.913628155620945,

                                         0.922841482421118,
                                         0.654087207622715,
                                         0.350035941263129,
                                         0.139928817166104,
                                         0.0411114277500397,
                                         0.00867444315489512,
                                         0.00127320240385874,
                                         0.000124449727707558,
                                         7.62939453125e-06,
                                         2.69749649419818e-07,
                                         4.87542986982489e-09,
                                         3.7598666042127e-11,
                                         9.29029264325132e-14,
                                         4.49460340078678e-17,
                                         1.63622351394328e-21,
                                         4.61904901982329e-28,
                                         2.78548504492552e-40,
                                         0.0742774245321839,
                                         0.281165507975319,
                                         0.379021045256573,
                                         0.291736286045745,
                                         0.144329992440454,
                                         0.0475056518377301,
                                         0.0103921666964874,
                                         0.00147480650647249,
                                         0.00012969970703125,
                                         6.57836001445527e-06,
                                         1.72624368150958e-07,
                                         1.98411391769871e-09,
                                         7.64774163879262e-12,
                                         6.23025657184864e-15,
                                         4.36706631469324e-19,
                                         3.10544632317954e-25,
                                         1.0001587649085e-36,
                                         0.00281337504562402,
                                         0.0568760588739099,
                                         0.193132372741986,
                                         0.286229770483891,
                                         0.238446901740157,
                                         0.122430612817816,
                                         0.0399168097963594,
                                         0.00822464581755959,
                                         0.00103759765625,
                                         7.54945240746441e-05,
                                         2.87628790994483e-06,
                                         4.92721865873067e-08,
                                         2.96263457528355e-10,
                                         4.06407349801068e-13,
                                         5.48501740082221e-17,
                                         9.82509024974533e-23,
                                         1.68996661392856e-33,
                                         6.66006461350042e-05,
                                         0.00719079594862775,
                                         0.0615073256943086,
                                         0.175516993921384,
                                         0.246210628106878,
                                         0.197203574405029,
                                         0.0958264858795822,
                                         0.0286668109203765,
                                         0.00518798828125,
                                         0.00054149354405326,
                                         2.9953158661456e-05,
                                         7.64745773112605e-07,
                                         7.17303555188095e-09,
                                         1.65690341227913e-11,
                                         4.30572461570173e-15,
                                         1.94280443875144e-20,
                                         1.78470862351942e-30,
                                         1.10363964831121e-06,
                                         0.000636388722759487,
                                         0.0137118689231033,
                                         0.0753393002138922,
                                         0.17795899659535,
                                         0.222350229277971,
                                         0.16103242744131,
                                         0.0699422499888169,
                                         0.018157958984375,
                                         0.00271874925091273,
                                         0.000218348864690286,
                                         8.308648278073e-06,
                                         1.21569860894965e-07,
                                         4.72858141782184e-10,
                                         2.36598795920621e-13,
                                         2.68917871887344e-18,
                                         1.3193334064879e-27,
                                         1.3585682140699e-08,
                                         4.18382286112417e-05,
                                         0.00227076276956427,
                                         0.024023113623613,
                                         0.09555169693886,
                                         0.18623687872325,
                                         0.201023310500099,
                                         0.126766681037752,
                                         0.047210693359375,
                                         0.0101402889330477,
                                         0.00118240035908855,
                                         6.70577485460676e-05,
                                         1.53057339432333e-06,
                                         1.00246646595765e-08,
                                         9.65793134829092e-12,
                                         2.76512996195073e-16,
                                         7.24514530638523e-25,
                                         1.28644804774637e-10,
                                         2.11582989927232e-06,
                                         0.000289270086126243,
                                         0.00589241947924587,
                                         0.0394651272854559,
                                         0.119991486851127,
                                         0.193035039857935,
                                         0.176736922509385,
                                         0.09442138671875,
                                         0.0290929762622306,
                                         0.00492532412137018,
                                         0.00041631706085692,
                                         1.48231003092152e-05,
                                         1.63480321795722e-07,
                                         3.03258055202232e-10,
                                         2.18709716745522e-14,
                                         3.06052751584204e-22,
                                         9.57122894241691e-13,
                                         8.40722868325847e-08,
                                         2.89534242583715e-05,
                                         0.00113559289636423,
                                         0.0128071725546733,
                                         0.0607435031779898,
                                         0.145643305787628,
                                         0.193603902148753,
                                         0.14837646484375,
                                         0.0655829012313077,
                                         0.01612017408093,
                                         0.00203078571199849,
                                         0.000112794671755593,
                                         2.09471895955798e-06,
                                         7.48178504436044e-09,
                                         1.35920592064213e-12,
                                         1.0158044857261e-19,
                                         5.66446013714491e-15,
                                         2.65729848163292e-09,
                                         2.30521645664793e-06,
                                         0.000174087282353063,
                                         0.00330604211665905,
                                         0.0244604590160961,
                                         0.0874098296977797,
                                         0.168700469888202,
                                         0.185470581054688,
                                         0.117600309599684,
                                         0.0419681675665118,
                                         0.00787987552874509,
                                         0.000682737079114944,
                                         2.13501736223293e-05,
                                         1.46829552582927e-07,
                                         6.71920366296178e-11,
                                         2.68187994150768e-17,
                                         2.6818799415074e-17,
                                         6.71920366296192e-11,
                                         1.46829552582927e-07,
                                         2.13501736223294e-05,
                                         0.000682737079114946,
                                         0.0078798755287451,
                                         0.0419681675665118,
                                         0.117600309599684,
                                         0.185470581054688,
                                         0.168700469888202,
                                         0.0874098296977797,
                                         0.024460459016096,
                                         0.00330604211665904,
                                         0.000174087282353062,
                                         2.30521645664793e-06,
                                         2.65729848163287e-09,
                                         5.66446013714544e-15,
                                         1.01580448572598e-19,
                                         1.35920592064216e-12,
                                         7.48178504436044e-09,
                                         2.09471895955799e-06,
                                         0.000112794671755594,
                                         0.0020307857119985,
                                         0.01612017408093,
                                         0.0655829012313077,
                                         0.14837646484375,
                                         0.193603902148753,
                                         0.145643305787628,
                                         0.0607435031779897,
                                         0.0128071725546733,
                                         0.00113559289636422,
                                         2.89534242583715e-05,
                                         8.40722868325834e-08,
                                         9.5712289424177e-13,
                                         3.06052751584164e-22,
                                         2.18709716745527e-14,
                                         3.03258055202232e-10,
                                         1.63480321795723e-07,
                                         1.48231003092153e-05,
                                         0.000416317060856921,
                                         0.00492532412137018,
                                         0.0290929762622306,
                                         0.09442138671875,
                                         0.176736922509385,
                                         0.193035039857935,
                                         0.119991486851126,
                                         0.0394651272854559,
                                         0.00589241947924585,
                                         0.000289270086126243,
                                         2.11582989927229e-06,
                                         1.28644804774646e-10,
                                         7.2451453063842e-25,
                                         2.76512996195081e-16,
                                         9.65793134829092e-12,
                                         1.00246646595766e-08,
                                         1.53057339432333e-06,
                                         6.70577485460678e-05,
                                         0.00118240035908854,
                                         0.0101402889330477,
                                         0.047210693359375,
                                         0.126766681037752,
                                         0.201023310500099,
                                         0.18623687872325,
                                         0.0955516969388598,
                                         0.0240231136236129,
                                         0.00227076276956427,
                                         4.18382286112412e-05,
                                         1.35856821406998e-08,
                                         1.3193334064877e-27,
                                         2.68917871887352e-18,
                                         2.36598795920621e-13,
                                         4.72858141782187e-10,
                                         1.21569860894966e-07,
                                         8.30864827807302e-06,
                                         0.000218348864690286,
                                         0.00271874925091273,
                                         0.018157958984375,
                                         0.0699422499888169,
                                         0.16103242744131,
                                         0.222350229277971,
                                         0.17795899659535,
                                         0.0753393002138921,
                                         0.0137118689231033,
                                         0.000636388722759482,
                                         1.10363964831127e-06,
                                         1.78470862351912e-30,
                                         1.9428044387515e-20,
                                         4.30572461570173e-15,
                                         1.65690341227914e-11,
                                         7.17303555188098e-09,
                                         7.64745773112608e-07,
                                         2.9953158661456e-05,
                                         0.00054149354405326,
                                         0.00518798828125,
                                         0.0286668109203765,
                                         0.0958264858795822,
                                         0.197203574405029,
                                         0.246210628106878,
                                         0.175516993921383,
                                         0.0615073256943087,
                                         0.00719079594862771,
                                         6.66006461350066e-05,
                                         1.68996661392826e-33,
                                         9.82509024974566e-23,
                                         5.48501740082221e-17,
                                         4.06407349801071e-13,
                                         2.96263457528357e-10,
                                         4.92721865873068e-08,
                                         2.87628790994483e-06,
                                         7.54945240746441e-05,
                                         0.00103759765625,
                                         0.00822464581755959,
                                         0.0399168097963594,
                                         0.122430612817816,
                                         0.238446901740157,
                                         0.286229770483891,
                                         0.193132372741986,
                                         0.0568760588739097,
                                         0.00281337504562408,
                                         1.00015876490831e-36,
                                         3.10544632317965e-25,
                                         4.36706631469324e-19,
                                         6.23025657184869e-15,
                                         7.64774163879266e-12,
                                         1.98411391769872e-09,
                                         1.72624368150958e-07,
                                         6.57836001445527e-06,
                                         0.00012969970703125,
                                         0.00147480650647249,
                                         0.0103921666964874,
                                         0.04750565183773,
                                         0.144329992440454,
                                         0.291736286045745,
                                         0.379021045256573,
                                         0.281165507975319,
                                         0.0742774245321848,
                                         2.78548504492496e-40,
                                         4.61904901982347e-28,
                                         1.63622351394328e-21,
                                         4.49460340078682e-17,
                                         9.29029264325137e-14,
                                         3.75986660421271e-11,
                                         4.87542986982489e-09,
                                         2.69749649419818e-07,
                                         7.62939453125e-06,
                                         0.000124449727707558,
                                         0.00127320240385874,
                                         0.00867444315489512,
                                         0.0411114277500397,
                                         0.139928817166104,
                                         0.350035941263129,
                                         0.654087207622715,
                                         0.922841482421118,

                                         0.930672644222882,
                                         0.684062828118248,
                                         0.391291040015883,
                                         0.172704463439273,
                                         0.0580671437789247,
                                         0.0146057404620968,
                                         0.00268150546708616,
                                         0.000347610359176246,
                                         3.04273769431888e-05,
                                         1.69224733091537e-06,
                                         5.49465136732955e-08,
                                         9.22631771612389e-10,
                                         6.68389973695956e-12,
                                         1.56801521874188e-14,
                                         7.27693764407843e-18,
                                         2.56680750429514e-22,
                                         7.0903409549738e-29,
                                         4.22463442971983e-41,
                                         0.0670081708967356,
                                         0.262664897811688,
                                         0.377471120583698,
                                         0.31951872319949,
                                         0.179908274902464,
                                         0.0700794235732519,
                                         0.0189946166387211,
                                         0.00353137974660857,
                                         0.00043641697184249,
                                         3.40976590374752e-05,
                                         1.56309823673031e-06,
                                         3.76421484028117e-08,
                                         4.02588001700134e-10,
                                         1.46260630556528e-12,
                                         1.13672064043459e-15,
                                         7.68966315508807e-20,
                                         5.33652951698834e-26,
                                         1.69572788634149e-37,
                                         0.00227038570425922,
                                         0.047462340458999,
                                         0.171359672352599,
                                         0.278182706401026,
                                         0.262308811505678,
                                         0.158233534228757,
                                         0.0633174541304195,
                                         0.0168825171267788,
                                         0.00294564098590675,
                                         0.000323315373195828,
                                         2.09253856209143e-05,
                                         7.22705674826684e-07,
                                         1.14112388058993e-08,
                                         6.42015739016121e-11,
                                         8.35602833524986e-14,
                                         1.08408265845501e-17,
                                         1.8901304459147e-23,
                                         3.20305396678047e-34,
                                         4.80785699072463e-05,
                                         0.00536014181142349,
                                         0.0486198408758119,
                                         0.151371446573031,
                                         0.239031225185529,
                                         0.223298884304034,
                                         0.131915665692651,
                                         0.0504440553316488,
                                         0.0124261906870015,
                                         0.00191605585059869,
                                         0.000175081671570074,
                                         8.67218521306025e-06,
                                         2.02155135236513e-07,
                                         1.76134295151759e-09,
                                         3.83907042855335e-12,
                                         9.55206998879769e-16,
                                         4.18412506105408e-21,
                                         3.78139779852995e-31,
                                         7.12691335217918e-07,
                                         0.000423741938819326,
                                         0.00965642747815043,
                                         0.0576575035660418,
                                         0.152473523096729,
                                         0.22058329406672,
                                         0.192383287766186,
                                         0.105506891504133,
                                         0.0366939321560038,
                                         0.00794855187499195,
                                         0.00102542981011513,
                                         7.28439796343921e-05,
                                         2.50688725197446e-06,
                                         3.38251878112908e-08,
                                         1.23466829154385e-10,
                                         5.89156447172089e-14,
                                         6.48359048184528e-19,
                                         3.12491722628382e-28,
                                         7.8479589003343e-09,
                                         2.48846721001937e-05,
                                         0.00142470430588958,
                                         0.0163144715694865,
                                         0.0722502803812811,
                                         0.161869112793746,
                                         0.208422026874339,
                                         0.163929443999831,
                                         0.0804925705841718,
                                         0.0244947586937454,
                                         0.00446145479047788,
                                         0.000454531606193092,
                                         2.30935197163231e-05,
                                         4.82549434656672e-07,
                                         2.94971335091615e-09,
                                         2.69941139487919e-12,
                                         7.46331546018643e-17,
                                         1.91835939267872e-25,
                                         6.6476570903538e-11,
                                         1.12413648461385e-06,
                                         0.000161692411514196,
                                         0.00355096863280592,
                                         0.0263354828843205,
                                         0.0913717625440195,
                                         0.173690687964229,
                                         0.19592495316939,
                                         0.135823209379734,
                                         0.0580650715618487,
                                         0.0149315090579832,
                                         0.00218168054376579,
                                         0.000163644760424325,
                                         5.2954156344755e-06,
                                         5.42083240691769e-08,
                                         9.51402233818284e-11,
                                         6.60852700891896e-15,
                                         9.05895443473398e-23,
                                         4.42430590528209e-13,
                                         3.98998087212419e-08,
                                         1.44184703633476e-05,
                                         0.00060727487053496,
                                         0.00754236741464437,
                                         0.0405251534532921,
                                         0.113729755410585,
                                         0.183987020668277,
                                         0.180076412017713,
                                         0.108148727868111,
                                         0.0392640918619074,
                                         0.00822778394790406,
                                         0.000911126607241839,
                                         4.56586372307247e-05,
                                         7.82738682344903e-07,
                                         2.63465592497758e-09,
                                         4.59771650573917e-13,
                                         3.36117269714046e-20,
                                         2.3422705265354e-15,
                                         1.12651752382124e-09,
                                         1.02273732949271e-06,
                                         8.26112420947997e-05,
                                         0.00171826225887462,
                                         0.0142972562618862,
                                         0.059236179818394,
                                         0.137435836451401,
                                         0.18991315002992,
                                         0.160229778681904,
                                         0.0821301810203311,
                                         0.0246825467042235,
                                         0.00403525256191596,
                                         0.000313156337045097,
                                         8.99048210499301e-06,
                                         5.80362059929372e-08,
                                         2.54445677239694e-11,
                                         9.92016616740086e-18,
                                         9.92016616740086e-18,
                                         2.54445677239694e-11,
                                         5.80362059929372e-08,
                                         8.99048210499296e-06,
                                         0.000313156337045097,
                                         0.00403525256191596,
                                         0.0246825467042235,
                                         0.0821301810203311,
                                         0.160229778681904,
                                         0.18991315002992,
                                         0.137435836451401,
                                         0.059236179818394,
                                         0.0142972562618862,
                                         0.00171826225887462,
                                         8.26112420948e-05,
                                         1.02273732949271e-06,
                                         1.12651752382124e-09,
                                         2.3422705265354e-15,
                                         3.36117269714046e-20,
                                         4.59771650573917e-13,
                                         2.63465592497758e-09,
                                         7.82738682344899e-07,
                                         4.56586372307247e-05,
                                         0.000911126607241839,
                                         0.00822778394790406,
                                         0.0392640918619074,
                                         0.108148727868111,
                                         0.180076412017713,
                                         0.183987020668277,
                                         0.113729755410585,
                                         0.0405251534532921,
                                         0.00754236741464437,
                                         0.000607274870534963,
                                         1.44184703633476e-05,
                                         3.98998087212419e-08,
                                         4.42430590528209e-13,
                                         9.05895443473398e-23,
                                         6.60852700891896e-15,
                                         9.51402233818284e-11,
                                         5.42083240691765e-08,
                                         5.2954156344755e-06,
                                         0.000163644760424325,
                                         0.00218168054376579,
                                         0.0149315090579832,
                                         0.0580650715618487,
                                         0.135823209379734,
                                         0.19592495316939,
                                         0.173690687964229,
                                         0.0913717625440195,
                                         0.0263354828843205,
                                         0.00355096863280593,
                                         0.000161692411514196,
                                         1.12413648461385e-06,
                                         6.6476570903538e-11,
                                         1.91835939267872e-25,
                                         7.46331546018643e-17,
                                         2.69941139487919e-12,
                                         2.94971335091613e-09,
                                         4.82549434656671e-07,
                                         2.30935197163231e-05,
                                         0.000454531606193092,
                                         0.00446145479047788,
                                         0.0244947586937454,
                                         0.0804925705841717,
                                         0.163929443999831,
                                         0.208422026874339,
                                         0.161869112793746,
                                         0.072250280381281,
                                         0.0163144715694865,
                                         0.00142470430588958,
                                         2.48846721001937e-05,
                                         7.8479589003343e-09,
                                         3.12491722628382e-28,
                                         6.48359048184528e-19,
                                         5.89156447172089e-14,
                                         1.23466829154384e-10,
                                         3.38251878112908e-08,
                                         2.50688725197446e-06,
                                         7.28439796343921e-05,
                                         0.00102542981011513,
                                         0.00794855187499195,
                                         0.0366939321560038,
                                         0.105506891504133,
                                         0.192383287766186,
                                         0.22058329406672,
                                         0.152473523096729,
                                         0.057657503566042,
                                         0.00965642747815043,
                                         0.000423741938819326,
                                         7.12691335217918e-07,
                                         3.78139779852995e-31,
                                         4.18412506105408e-21,
                                         9.5520699887977e-16,
                                         3.83907042855332e-12,
                                         1.76134295151759e-09,
                                         2.02155135236513e-07,
                                         8.67218521306025e-06,
                                         0.000175081671570074,
                                         0.00191605585059869,
                                         0.0124261906870015,
                                         0.0504440553316488,
                                         0.131915665692651,
                                         0.223298884304034,
                                         0.239031225185529,
                                         0.151371446573031,
                                         0.0486198408758119,
                                         0.00536014181142349,
                                         4.80785699072463e-05,
                                         3.20305396678047e-34,
                                         1.8901304459147e-23,
                                         1.08408265845501e-17,
                                         8.35602833524979e-14,
                                         6.42015739016121e-11,
                                         1.14112388058993e-08,
                                         7.22705674826684e-07,
                                         2.09253856209143e-05,
                                         0.000323315373195828,
                                         0.00294564098590675,
                                         0.0168825171267788,
                                         0.0633174541304195,
                                         0.158233534228757,
                                         0.262308811505678,
                                         0.278182706401026,
                                         0.171359672352599,
                                         0.047462340458999,
                                         0.00227038570425922,
                                         1.69572788634149e-37,
                                         5.33652951698834e-26,
                                         7.68966315508807e-20,
                                         1.13672064043458e-15,
                                         1.46260630556528e-12,
                                         4.02588001700134e-10,
                                         3.76421484028117e-08,
                                         1.56309823673031e-06,
                                         3.40976590374752e-05,
                                         0.00043641697184249,
                                         0.00353137974660857,
                                         0.0189946166387211,
                                         0.0700794235732519,
                                         0.179908274902464,
                                         0.31951872319949,
                                         0.377471120583698,
                                         0.262664897811688,
                                         0.0670081708967356,
                                         4.22463442971983e-41,
                                         7.0903409549738e-29,
                                         2.56680750429514e-22,
                                         7.27693764407836e-18,
                                         1.56801521874188e-14,
                                         6.68389973695956e-12,
                                         9.22631771612389e-10,
                                         5.49465136732955e-08,
                                         1.69224733091537e-06,
                                         3.04273769431888e-05,
                                         0.000347610359176246,
                                         0.00268150546708616,
                                         0.0146057404620968,
                                         0.0580671437789247,
                                         0.172704463439273,
                                         0.391291040015883,
                                         0.684062828118248,
                                         0.930672644222882,

                                         0.937381754748496,
                                         0.710603465112022,
                                         0.430085007835775,
                                         0.206399067994705,
                                         0.0777355669344877,
                                         0.0226488206372192,
                                         0.00500686736677097,
                                         0.000818472740527965,
                                         9.56220820141343e-05,
                                         7.62939453125e-06,
                                         3.90917418345912e-07,
                                         1.18127896314808e-08,
                                         1.86391355572032e-10,
                                         1.28067251840145e-12,
                                         2.87512349458689e-15,
                                         1.28803669582291e-18,
                                         4.42317014314987e-23,
                                         1.19947141139735e-29,
                                         7.07407577956153e-42,
                                         0.0607309009641049,
                                         0.245226928615661,
                                         0.372050600715626,
                                         0.341279970502534,
                                         0.214265972007425,
                                         0.096093513415745,
                                         0.0311176473148082,
                                         0.00722285540034708,
                                         0.00117627453439687,
                                         0.00012969970703125,
                                         9.18401036801413e-06,
                                         3.86852681755294e-07,
                                         8.66728230504488e-09,
                                         8.72343346341457e-11,
                                         3.01453903057611e-13,
                                         2.25124629042678e-16,
                                         1.47769152320348e-20,
                                         1.00449298047906e-26,
                                         3.15554459850891e-38,
                                         0.00185158655107083,
                                         0.0398244752743833,
                                         0.151457499272372,
                                         0.265555276337034,
                                         0.277925058719239,
                                         0.19185963690588,
                                         0.0910098685502282,
                                         0.0299954020104612,
                                         0.0068092653723623,
                                         0.00103759765625,
                                         0.000101536169240262,
                                         5.96183317553639e-06,
                                         1.89662374504304e-07,
                                         2.79626213101767e-09,
                                         1.48739523039431e-11,
                                         1.85164978847581e-14,
                                         2.32313786886576e-18,
                                         3.95863059342011e-24,
                                         6.623995107816e-35,
                                         3.52824169994275e-05,
                                         0.00404214588053474,
                                         0.0385353733499514,
                                         0.129145443048194,
                                         0.225310911306928,
                                         0.239416005859823,
                                         0.166360507787896,
                                         0.0778539175211393,
                                         0.0246360934221801,
                                         0.00518798828125,
                                         0.000701599386519998,
                                         5.74240797744578e-05,
                                         2.59393710714209e-06,
                                         5.60206736412627e-08,
                                         4.58682187591578e-10,
                                         9.51863572674825e-13,
                                         2.28268614973598e-16,
                                         9.75041418886278e-22,
                                         8.69051874775904e-32,
                                         4.70620325051015e-07,
                                         0.00028719173937299,
                                         0.00686319597542514,
                                         0.0439644129204287,
                                         0.127860024178924,
                                         0.20913214134244,
                                         0.212867827353476,
                                         0.141450437300094,
                                         0.0623938040510056,
                                         0.018157958984375,
                                         0.00339356105310927,
                                         0.000387174110478633,
                                         2.48333746394131e-05,
                                         7.85627745083791e-07,
                                         9.90137264394848e-09,
                                         3.4252210468577e-11,
                                         1.57005715830303e-14,
                                         1.68112184826438e-19,
                                         7.98122287427791e-29,
                                         4.66324608189209e-09,
                                         1.51578361568009e-05,
                                         0.000908026520244222,
                                         0.0111180538564533,
                                         0.0539004813892021,
                                         0.13570432555793,
                                         0.202336913737621,
                                         0.190912085493168,
                                         0.117386023423097,
                                         0.047210693359375,
                                         0.0121934734470804,
                                         0.00193920569447321,
                                         0.000176610834454474,
                                         8.1844707660931e-06,
                                         1.587757745402e-07,
                                         9.15604259963379e-10,
                                         8.02213611315723e-13,
                                         2.1531814213431e-17,
                                         5.44500747098962e-26,
                                         3.55437001283272e-11,
                                         6.15402307820939e-07,
                                         9.24117752831567e-05,
                                         0.00216278284646547,
                                         0.0174786205139588,
                                         0.0677365768578396,
                                         0.147943831805664,
                                         0.198207095241999,
                                         0.169882234359526,
                                         0.09442138671875,
                                         0.0337020139878477,
                                         0.0074713330091457,
                                         0.000966174547191257,
                                         6.55874941948866e-05,
                                         1.95852771649239e-06,
                                         1.88271096936194e-08,
                                         3.15298015536915e-11,
                                         2.12138113788699e-15,
                                         2.85748638023528e-23,
                                         2.12863690762232e-13,
                                         1.96311473206247e-08,
                                         7.38959681293744e-06,
                                         0.000330568684019618,
                                         0.0044533447027459,
                                         0.0265654666502205,
                                         0.0849930186155372,
                                         0.161684959235349,
                                         0.193172005006207,
                                         0.14837646484375,
                                         0.0731895238256067,
                                         0.0226171021619482,
                                         0.00415296710895466,
                                         0.000412967738317665,
                                         1.89819075274766e-05,
                                         3.04175444104085e-07,
                                         9.73681901813044e-10,
                                         1.64218254517046e-13,
                                         1.1782420216274e-20,
                                         1.01404191680464e-15,
                                         4.98135635972058e-10,
                                         4.70034289256385e-07,
                                         4.01907276560655e-05,
                                         0.000902569885172882,
                                         0.00828756639719672,
                                         0.0388405179585093,
                                         0.104914475528012,
                                         0.174725293465533,
                                         0.185470581054688,
                                         0.126432101486488,
                                         0.0544616999744346,
                                         0.014199621191094,
                                         0.00206836213923233,
                                         0.000146340775663504,
                                         3.90912912834337e-06,
                                         2.39181907938419e-08,
                                         1.0112057447181e-11,
                                         3.86456142089698e-18,
                                         3.86456142089698e-18,
                                         1.0112057447181e-11,
                                         2.39181907938419e-08,
                                         3.90912912834337e-06,
                                         0.000146340775663504,
                                         0.00206836213923234,
                                         0.014199621191094,
                                         0.0544616999744345,
                                         0.126432101486488,
                                         0.185470581054688,
                                         0.174725293465533,
                                         0.104914475528012,
                                         0.0388405179585093,
                                         0.00828756639719671,
                                         0.000902569885172885,
                                         4.01907276560655e-05,
                                         4.70034289256385e-07,
                                         4.98135635972058e-10,
                                         1.01404191680464e-15,
                                         1.1782420216274e-20,
                                         1.64218254517046e-13,
                                         9.73681901813044e-10,
                                         3.04175444104085e-07,
                                         1.89819075274766e-05,
                                         0.000412967738317666,
                                         0.00415296710895467,
                                         0.0226171021619482,
                                         0.0731895238256067,
                                         0.14837646484375,
                                         0.193172005006207,
                                         0.16168495923535,
                                         0.0849930186155372,
                                         0.0265654666502205,
                                         0.00445334470274591,
                                         0.000330568684019618,
                                         7.38959681293744e-06,
                                         1.96311473206247e-08,
                                         2.12863690762232e-13,
                                         2.85748638023528e-23,
                                         2.12138113788699e-15,
                                         3.15298015536915e-11,
                                         1.88271096936194e-08,
                                         1.95852771649238e-06,
                                         6.55874941948868e-05,
                                         0.000966174547191257,
                                         0.00747133300914569,
                                         0.0337020139878477,
                                         0.09442138671875,
                                         0.169882234359526,
                                         0.198207095241999,
                                         0.147943831805664,
                                         0.0677365768578395,
                                         0.0174786205139589,
                                         0.00216278284646547,
                                         9.24117752831567e-05,
                                         6.15402307820939e-07,
                                         3.55437001283272e-11,
                                         5.44500747098962e-26,
                                         2.1531814213431e-17,
                                         8.02213611315723e-13,
                                         9.15604259963379e-10,
                                         1.58775774540199e-07,
                                         8.18447076609312e-06,
                                         0.000176610834454474,
                                         0.00193920569447321,
                                         0.0121934734470804,
                                         0.047210693359375,
                                         0.117386023423097,
                                         0.190912085493168,
                                         0.202336913737621,
                                         0.13570432555793,
                                         0.0539004813892022,
                                         0.0111180538564533,
                                         0.000908026520244222,
                                         1.51578361568009e-05,
                                         4.66324608189209e-09,
                                         7.98122287427791e-29,
                                         1.68112184826438e-19,
                                         1.57005715830303e-14,
                                         3.4252210468577e-11,
                                         9.90137264394842e-09,
                                         7.85627745083794e-07,
                                         2.48333746394131e-05,
                                         0.000387174110478632,
                                         0.00339356105310927,
                                         0.018157958984375,
                                         0.0623938040510056,
                                         0.141450437300094,
                                         0.212867827353476,
                                         0.209132141342439,
                                         0.127860024178925,
                                         0.0439644129204287,
                                         0.00686319597542514,
                                         0.00028719173937299,
                                         4.70620325051015e-07,
                                         8.69051874775904e-32,
                                         9.75041418886278e-22,
                                         2.28268614973598e-16,
                                         9.51863572674825e-13,
                                         4.58682187591575e-10,
                                         5.6020673641263e-08,
                                         2.59393710714209e-06,
                                         5.74240797744576e-05,
                                         0.000701599386519998,
                                         0.00518798828125,
                                         0.0246360934221801,
                                         0.0778539175211393,
                                         0.166360507787896,
                                         0.239416005859823,
                                         0.225310911306928,
                                         0.129145443048194,
                                         0.0385353733499514,
                                         0.00404214588053474,
                                         3.52824169994275e-05,
                                         6.623995107816e-35,
                                         3.95863059342011e-24,
                                         2.32313786886576e-18,
                                         1.85164978847581e-14,
                                         1.4873952303943e-11,
                                         2.79626213101768e-09,
                                         1.89662374504304e-07,
                                         5.96183317553637e-06,
                                         0.000101536169240262,
                                         0.00103759765625,
                                         0.0068092653723623,
                                         0.0299954020104612,
                                         0.0910098685502282,
                                         0.19185963690588,
                                         0.277925058719239,
                                         0.265555276337034,
                                         0.151457499272372,
                                         0.0398244752743833,
                                         0.00185158655107083,
                                         3.15554459850891e-38,
                                         1.00449298047906e-26,
                                         1.47769152320348e-20,
                                         2.25124629042678e-16,
                                         3.01453903057609e-13,
                                         8.72343346341461e-11,
                                         8.66728230504488e-09,
                                         3.86852681755293e-07,
                                         9.18401036801413e-06,
                                         0.00012969970703125,
                                         0.00117627453439687,
                                         0.00722285540034708,
                                         0.0311176473148082,
                                         0.096093513415745,
                                         0.214265972007425,
                                         0.341279970502534,
                                         0.372050600715626,
                                         0.245226928615661,
                                         0.0607309009641049,
                                         7.07407577956153e-42,
                                         1.19947141139735e-29,
                                         4.42317014314987e-23,
                                         1.28803669582291e-18,
                                         2.87512349458687e-15,
                                         1.28067251840145e-12,
                                         1.86391355572032e-10,
                                         1.18127896314807e-08,
                                         3.90917418345912e-07,
                                         7.62939453125e-06,
                                         9.56220820141343e-05,
                                         0.000818472740527965,
                                         0.00500686736677097,
                                         0.0226488206372192,
                                         0.0777355669344877,
                                         0.206399067994705,
                                         0.430085007835775,
                                         0.710603465112022,
                                         0.937381754748496,

                                         0.943171193328388,
                                         0.734158381965059,
                                         0.466325683663904,
                                         0.240352730848555,
                                         0.0996885971872795,
                                         0.0328888290132017,
                                         0.00849837884737249,
                                         0.00168533045134214,
                                         0.00024978979240449,
                                         2.67245587451907e-05,
                                         1.9710849466528e-06,
                                         9.41958791072484e-08,
                                         2.67743722225522e-09,
                                         4.0063458126592e-11,
                                         2.63102339678068e-13,
                                         5.6886529293672e-16,
                                         2.47265780991905e-19,
                                         8.29853141268126e-24,
                                         2.21507071508325e-30,
                                         1.29497048109361e-42,
                                         0.055277629478792,
                                         0.228951998004193,
                                         0.36384910444781,
                                         0.357436469760714,
                                         0.246169057064362,
                                         0.124376151525373,
                                         0.0467719778244522,
                                         0.0130622609526371,
                                         0.00267078383404424,
                                         0.000389725801240211,
                                         3.906200835612e-05,
                                         2.5460427682788e-06,
                                         9.98352366478601e-08,
                                         2.1037621073444e-09,
                                         2.01063782858269e-11,
                                         6.65762459929211e-14,
                                         4.80520654782788e-17,
                                         3.07373986902099e-21,
                                         2.05272495405409e-27,
                                         6.38555763204517e-39,
                                         0.00152457698274717,
                                         0.033600069815233,
                                         0.133596275478869,
                                         0.250143791905971,
                                         0.286063537695291,
                                         0.221343556565746,
                                         0.1211368979979,
                                         0.0476422988553366,
                                         0.0134382852727838,
                                         0.00267453841153034,
                                         0.000364287996798491,
                                         3.23847501827383e-05,
                                         1.75181996797914e-06,
                                         5.19859385585209e-08,
                                         7.23076178185235e-10,
                                         3.6666565658251e-12,
                                         4.39441600040295e-15,
                                         5.35764636079587e-19,
                                         8.95189529627805e-25,
                                         1.48176337035285e-35,
                                         2.62802398347941e-05,
                                         0.00308188152273696,
                                         0.0306582533178699,
                                         0.109410905718061,
                                         0.207764606401541,
                                         0.246193549930566,
                                         0.196086267075737,
                                         0.10860431477231,
                                         0.0422599511730377,
                                         0.0114714558479128,
                                         0.00212331863804863,
                                         0.00025745149921111,
                                         1.9212112021098e-05,
                                         8.02888370307991e-07,
                                         1.62522792527085e-08,
                                         1.26212155056191e-10,
                                         2.51171461002632e-13,
                                         5.83661430181709e-17,
                                         2.43994054212114e-22,
                                         2.14901228310602e-32,
                                         3.17108095893499e-07,
                                         0.000197874458020234,
                                         0.00492491235398211,
                                         0.0334988221741771,
                                         0.105627904956614,
                                         0.191683396966594,
                                         0.222185620895251,
                                         0.173300370246287,
                                         0.0930276747238825,
                                         0.0344418345581812,
                                         0.00866330336111791,
                                         0.00143267716596326,
                                         0.000147488713760871,
                                         8.68005516751631e-06,
                                         2.55706953449443e-07,
                                         3.04109628446766e-09,
                                         1.00493380625522e-11,
                                         4.45088103056147e-15,
                                         4.65523417827857e-20,
                                         2.18170979235947e-29,
                                         2.84243563436337e-09,
                                         9.43775808981182e-06,
                                         0.000587698939863769,
                                         0.00761910160652801,
                                         0.039892483128302,
                                         0.110865808252828,
                                         0.187020848759106,
                                         0.205426849135015,
                                         0.152125018491606,
                                         0.0768173495026363,
                                         0.0262577298752758,
                                         0.005922520282984,
                                         0.000841100264899328,
                                         6.97100039213186e-05,
                                         2.98865719635508e-06,
                                         5.44332747370432e-08,
                                         2.98682600824673e-10,
                                         2.52136825523834e-13,
                                         6.59795160570483e-18,
                                         1.64535785038004e-26,
                                         1.9598850312283e-11,
                                         3.46261807585481e-07,
                                         5.39470817083879e-05,
                                         0.00133301378567335,
                                         0.0115893778316623,
                                         0.0493250353851212,
                                         0.121093475909033,
                                         0.187314573160181,
                                         0.191357624076423,
                                         0.131791987637816,
                                         0.0612191757367838,
                                         0.0188330830697612,
                                         0.00368971990400638,
                                         0.000430649925687471,
                                         2.68699165101448e-05,
                                         7.49471982226074e-07,
                                         6.82871582647796e-09,
                                         1.09871036851978e-11,
                                         7.19338601934088e-16,
                                         9.54509822909074e-24,
                                         1.06178161787806e-13,
                                         9.98171100719162e-09,
                                         3.89086045129392e-06,
                                         0.000183244169815058,
                                         0.0026454150684112,
                                         0.0172425634550829,
                                         0.0616050219821863,
                                         0.134199401366721,
                                         0.18912787616012,
                                         0.177657418443912,
                                         0.112145649116946,
                                         0.0470544754913509,
                                         0.0127175565590994,
                                         0.0020903465670882,
                                         0.00018981091373385,
                                         8.10794868108739e-06,
                                         1.22668429585319e-07,
                                         3.76179231404948e-10,
                                         6.16200798445635e-14,
                                         4.35075958878323e-21,
                                         4.57567507016447e-16,
                                         2.28886806541823e-10,
                                         2.23222887226221e-07,
                                         2.00373857314796e-05,
                                         0.000480333516886907,
                                         0.00479459168901646,
                                         0.0249302627714123,
                                         0.0764794855541191,
                                         0.148689632824175,
                                         0.1904991551951,
                                         0.163415312224652,
                                         0.0935181311534234,
                                         0.0348681836196219,
                                         0.0080710043675515,
                                         0.00106657502635359,
                                         6.97721246695741e-05,
                                         1.75283864084657e-06,
                                         1.02452314608281e-08,
                                         4.19881282246462e-12,
                                         1.57748463489653e-18,
                                         1.5774846348963e-18,
                                         4.19881282246451e-12,
                                         1.02452314608281e-08,
                                         1.75283864084656e-06,
                                         6.97721246695741e-05,
                                         0.0010665750263536,
                                         0.0080710043675515,
                                         0.0348681836196219,
                                         0.0935181311534234,
                                         0.163415312224652,
                                         0.1904991551951,
                                         0.148689632824175,
                                         0.0764794855541192,
                                         0.0249302627714123,
                                         0.00479459168901645,
                                         0.000480333516886907,
                                         2.00373857314797e-05,
                                         2.23222887226221e-07,
                                         2.28886806541829e-10,
                                         4.57567507016507e-16,
                                         4.35075958878253e-21,
                                         6.16200798445616e-14,
                                         3.76179231404948e-10,
                                         1.22668429585318e-07,
                                         8.10794868108739e-06,
                                         0.000189810913733851,
                                         0.0020903465670882,
                                         0.0127175565590994,
                                         0.0470544754913509,
                                         0.112145649116946,
                                         0.177657418443912,
                                         0.18912787616012,
                                         0.134199401366721,
                                         0.0616050219821863,
                                         0.0172425634550828,
                                         0.0026454150684112,
                                         0.000183244169815059,
                                         3.89086045129392e-06,
                                         9.98171100719183e-09,
                                         1.06178161787818e-13,
                                         9.54509822908904e-24,
                                         7.19338601934063e-16,
                                         1.09871036851978e-11,
                                         6.82871582647791e-09,
                                         7.49471982226074e-07,
                                         2.68699165101449e-05,
                                         0.000430649925687471,
                                         0.00368971990400638,
                                         0.0188330830697612,
                                         0.0612191757367838,
                                         0.131791987637816,
                                         0.191357624076423,
                                         0.187314573160181,
                                         0.121093475909033,
                                         0.0493250353851211,
                                         0.0115893778316623,
                                         0.00133301378567336,
                                         5.39470817083879e-05,
                                         3.46261807585487e-07,
                                         1.95988503122849e-11,
                                         1.64535785037972e-26,
                                         6.59795160570459e-18,
                                         2.52136825523834e-13,
                                         2.9868260082467e-10,
                                         5.44332747370432e-08,
                                         2.98865719635509e-06,
                                         6.97100039213186e-05,
                                         0.000841100264899328,
                                         0.005922520282984,
                                         0.0262577298752758,
                                         0.0768173495026363,
                                         0.152125018491606,
                                         0.205426849135015,
                                         0.187020848759106,
                                         0.110865808252827,
                                         0.039892483128302,
                                         0.00761910160652804,
                                         0.000587698939863769,
                                         9.43775808981196e-06,
                                         2.8424356343636e-09,
                                         2.18170979235901e-29,
                                         4.65523417827838e-20,
                                         4.45088103056147e-15,
                                         1.00493380625521e-11,
                                         3.04109628446766e-09,
                                         2.55706953449444e-07,
                                         8.68005516751631e-06,
                                         0.000147488713760871,
                                         0.00143267716596326,
                                         0.00866330336111791,
                                         0.0344418345581812,
                                         0.0930276747238825,
                                         0.173300370246287,
                                         0.222185620895251,
                                         0.191683396966594,
                                         0.105627904956614,
                                         0.0334988221741772,
                                         0.00492491235398211,
                                         0.000197874458020236,
                                         3.17108095893519e-07,
                                         2.14901228310553e-32,
                                         2.43994054212103e-22,
                                         5.83661430181709e-17,
                                         2.5117146100263e-13,
                                         1.26212155056191e-10,
                                         1.62522792527086e-08,
                                         8.02888370307991e-07,
                                         1.9212112021098e-05,
                                         0.00025745149921111,
                                         0.00212331863804863,
                                         0.0114714558479128,
                                         0.0422599511730376,
                                         0.10860431477231,
                                         0.196086267075737,
                                         0.246193549930565,
                                         0.207764606401541,
                                         0.109410905718061,
                                         0.0306582533178699,
                                         0.00308188152273699,
                                         2.62802398347954e-05,
                                         1.48176337035249e-35,
                                         8.95189529627764e-25,
                                         5.35764636079587e-19,
                                         4.39441600040291e-15,
                                         3.6666565658251e-12,
                                         7.23076178185238e-10,
                                         5.19859385585209e-08,
                                         1.75181996797914e-06,
                                         3.23847501827383e-05,
                                         0.000364287996798491,
                                         0.00267453841153034,
                                         0.0134382852727838,
                                         0.0476422988553366,
                                         0.1211368979979,
                                         0.221343556565746,
                                         0.286063537695291,
                                         0.250143791905971,
                                         0.133596275478869,
                                         0.0336000698152332,
                                         0.00152457698274722,
                                         6.38555763204353e-39,
                                         2.05272495405399e-27,
                                         3.07373986902099e-21,
                                         4.80520654782783e-17,
                                         6.65762459929211e-14,
                                         2.0106378285827e-11,
                                         2.1037621073444e-09,
                                         9.98352366478601e-08,
                                         2.5460427682788e-06,
                                         3.906200835612e-05,
                                         0.000389725801240211,
                                         0.00267078383404424,
                                         0.0130622609526371,
                                         0.0467719778244522,
                                         0.124376151525373,
                                         0.246169057064362,
                                         0.357436469760714,
                                         0.36384910444781,
                                         0.228951998004194,
                                         0.0552776294787929,
                                         1.29497048109326e-42,
                                         2.21507071508313e-30,
                                         8.29853141268126e-24,
                                         2.47265780991902e-19,
                                         5.6886529293672e-16,
                                         2.63102339678069e-13,
                                         4.0063458126592e-11,
                                         2.67743722225522e-09,
                                         9.41958791072484e-08,
                                         1.9710849466528e-06,
                                         2.67245587451907e-05,
                                         0.00024978979240449,
                                         0.00168533045134214,
                                         0.00849837884737249,
                                         0.0328888290132017,
                                         0.0996885971872795,
                                         0.240352730848555,
                                         0.466325683663904,
                                         0.734158381965059,
                                         0.943171193328388,

                                         0.948200147410435,
                                         0.755119293956849,
                                         0.500026386740259,
                                         0.274043706608637,
                                         0.123459787144882,
                                         0.0452908005867576,
                                         0.0133609028477784,
                                         0.00311841181936169,
                                         0.000563873825052832,
                                         7.68785736093973e-05,
                                         7.62939453125e-06,
                                         5.26090870379766e-07,
                                         2.36825749256552e-08,
                                         6.38683634997512e-10,
                                         9.13051337085525e-12,
                                         5.76718585788749e-14,
                                         1.20717833727762e-16,
                                         5.11225320075267e-20,
                                         1.6820807795792e-24,
                                         4.42896049629065e-31,
                                         2.56972003534911e-43,
                                         0.0505134379501554,
                                         0.213859420091011,
                                         0.35372727288096,
                                         0.368596042015273,
                                         0.274819619544353,
                                         0.153727062240735,
                                         0.065636746330167,
                                         0.0214255769617136,
                                         0.00529876668159917,
                                         0.000974801802335351,
                                         0.00012969970703125,
                                         1.1990784598852e-05,
                                         7.2833956273564e-07,
                                         2.68648594776699e-08,
                                         5.37132957948223e-10,
                                         4.91045188658414e-12,
                                         1.56728171209343e-14,
                                         1.09844764931373e-17,
                                         6.87177716965775e-22,
                                         4.5194629617581e-28,
                                         1.39404484310057e-39,
                                         0.00126635334658242,
                                         0.0285024549740969,
                                         0.117756593649806,
                                         0.233304205583789,
                                         0.287879679055515,
                                         0.245545355363755,
                                         0.151739744568219,
                                         0.0692743802609229,
                                         0.0234319657621943,
                                         0.00581658924149932,
                                         0.00103759765625,
                                         0.000128610226542871,
                                         1.05409547202826e-05,
                                         5.31770723559459e-07,
                                         1.48699487245838e-08,
                                         1.96752399847148e-10,
                                         9.57555079120737e-13,
                                         1.11067612830353e-15,
                                         1.32108984350408e-19,
                                         2.1702657773226e-25,
                                         3.55884229610255e-36,
                                         1.98418834605818e-05,
                                         0.00237419381387519,
                                         0.0245008944939907,
                                         0.0922942159889417,
                                         0.188475239118916,
                                         0.245128121318903,
                                         0.219245995659656,
                                         0.139988638611413,
                                         0.0647623603369979,
                                         0.0216920444258977,
                                         0.00518798828125,
                                         0.00086215117091839,
                                         9.5346775820568e-05,
                                         6.57876376270453e-06,
                                         2.57286594268466e-07,
                                         4.92718233216169e-09,
                                         3.65645707818059e-11,
                                         7.01900463095598e-14,
                                         1.5873608197708e-17,
                                         6.51356917856206e-23,
                                         5.67833172261822e-33,
                                         2.17625071413179e-07,
                                         0.000138435702807921,
                                         0.00356842592571595,
                                         0.025557857384478,
                                         0.0863765067205652,
                                         0.171298117370715,
                                         0.221749184596974,
                                         0.198020873034526,
                                         0.125295263371812,
                                         0.0566279206399108,
                                         0.018157958984375,
                                         0.00404565999957805,
                                         0.000603712427425523,
                                         5.69720962617914e-05,
                                         3.11618250818703e-06,
                                         8.63724560780893e-08,
                                         9.77361517814456e-10,
                                         3.10500039820271e-12,
                                         1.33510984826139e-15,
                                         1.3684318630955e-20,
                                         6.34206686572837e-30,
                                         1.77312871222078e-09,
                                         5.99632763398309e-06,
                                         0.000386079494073243,
                                         0.0052575051099172,
                                         0.0294064279290217,
                                         0.0889236635319724,
                                         0.166608708015505,
                                         0.208081972162507,
                                         0.180074385573322,
                                         0.109816108034864,
                                         0.047210693359375,
                                         0.0141026472914098,
                                         0.00283961540512814,
                                         0.000366509647483158,
                                         2.80371533042983e-05,
                                         1.12475312499403e-06,
                                         1.94068609648904e-08,
                                         1.02035922459634e-10,
                                         8.34187438722169e-14,
                                         2.13566253671336e-18,
                                         5.2619434542622e-27,
                                         1.1112921480705e-11,
                                         1.99792529976939e-07,
                                         3.21316799591287e-05,
                                         0.000831939245953104,
                                         0.00770097223081416,
                                         0.035509025287968,
                                         0.0962919561473389,
                                         0.168195583376666,
                                         0.199079195733185,
                                         0.163816683301324,
                                         0.09442138671875,
                                         0.0378153874763142,
                                         0.0102741423574628,
                                         0.00181369923594465,
                                         0.00019404461495472,
                                         1.12666757403103e-05,
                                         2.9642307032823e-07,
                                         2.57929595862283e-09,
                                         4.00928510073263e-12,
                                         2.56388610469768e-16,
                                         3.35828991036577e-24,
                                         5.47243836291273e-14,
                                         5.23043474251993e-09,
                                         2.1011389773527e-06,
                                         0.000103435155205436,
                                         0.00158457752325461,
                                         0.0111410161857778,
                                         0.0437267270550723,
                                         0.106821671460788,
                                         0.172927659281107,
                                         0.192006026053632,
                                         0.14837646484375,
                                         0.0796711552741235,
                                         0.0292076276772904,
                                         0.00705195900057466,
                                         0.00105519791141534,
                                         8.86745578449668e-05,
                                         3.55740538138089e-06,
                                         5.12287677710474e-08,
                                         1.51403140521458e-10,
                                         2.41840747956097e-14,
                                         1.68404945843755e-21,
                                         2.14362532651672e-16,
                                         1.08921019442485e-10,
                                         1.09292785239748e-07,
                                         1.02296340216976e-05,
                                         0.000259356294770183,
                                         0.00278052130446276,
                                         0.015794988052417,
                                         0.0539659117326909,
                                         0.119486383432944,
                                         0.179013983722609,
                                         0.185470581054688,
                                         0.133520835889962,
                                         0.0660484083818014,
                                         0.0218107025611157,
                                         0.00456437816188316,
                                         0.000555159320884268,
                                         3.39601876965966e-05,
                                         8.09360508117637e-07,
                                         4.54797632492119e-09,
                                         1.81457780248299e-12,
                                         6.71748750483856e-19,
                                         6.71748750483964e-19,
                                         1.81457780248293e-12,
                                         4.54797632492124e-09,
                                         8.09360508117637e-07,
                                         3.39601876965966e-05,
                                         0.000555159320884268,
                                         0.00456437816188316,
                                         0.0218107025611157,
                                         0.0660484083818014,
                                         0.133520835889962,
                                         0.185470581054688,
                                         0.179013983722609,
                                         0.119486383432944,
                                         0.053965911732691,
                                         0.015794988052417,
                                         0.00278052130446276,
                                         0.000259356294770183,
                                         1.02296340216976e-05,
                                         1.09292785239747e-07,
                                         1.08921019442488e-10,
                                         2.14362532651642e-16,
                                         1.68404945843785e-21,
                                         2.41840747956089e-14,
                                         1.5140314052146e-10,
                                         5.12287677710474e-08,
                                         3.55740538138089e-06,
                                         8.86745578449668e-05,
                                         0.00105519791141534,
                                         0.00705195900057465,
                                         0.0292076276772904,
                                         0.0796711552741235,
                                         0.14837646484375,
                                         0.192006026053632,
                                         0.172927659281107,
                                         0.106821671460788,
                                         0.0437267270550723,
                                         0.0111410161857778,
                                         0.00158457752325461,
                                         0.000103435155205436,
                                         2.10113897735268e-06,
                                         5.23043474252005e-09,
                                         5.47243836291205e-14,
                                         3.35828991036642e-24,
                                         2.56388610469758e-16,
                                         4.00928510073269e-12,
                                         2.57929595862283e-09,
                                         2.9642307032823e-07,
                                         1.12666757403103e-05,
                                         0.00019404461495472,
                                         0.00181369923594465,
                                         0.0102741423574628,
                                         0.0378153874763142,
                                         0.09442138671875,
                                         0.163816683301324,
                                         0.199079195733185,
                                         0.168195583376666,
                                         0.0962919561473389,
                                         0.035509025287968,
                                         0.00770097223081416,
                                         0.000831939245953104,
                                         3.21316799591284e-05,
                                         1.99792529976943e-07,
                                         1.11129214807038e-11,
                                         5.26194345426332e-27,
                                         2.13566253671327e-18,
                                         8.34187438722183e-14,
                                         1.02035922459634e-10,
                                         1.94068609648904e-08,
                                         1.12475312499403e-06,
                                         2.80371533042983e-05,
                                         0.000366509647483157,
                                         0.00283961540512814,
                                         0.0141026472914098,
                                         0.047210693359375,
                                         0.109816108034864,
                                         0.180074385573322,
                                         0.208081972162508,
                                         0.166608708015505,
                                         0.0889236635319724,
                                         0.0294064279290217,
                                         0.0052575051099172,
                                         0.00038607949407324,
                                         5.99632763398319e-06,
                                         1.77312871222062e-09,
                                         6.34206686572984e-30,
                                         1.36843186309544e-20,
                                         1.33510984826141e-15,
                                         3.10500039820271e-12,
                                         9.77361517814456e-10,
                                         8.63724560780893e-08,
                                         3.11618250818703e-06,
                                         5.69720962617912e-05,
                                         0.000603712427425523,
                                         0.00404565999957805,
                                         0.018157958984375,
                                         0.0566279206399108,
                                         0.125295263371812,
                                         0.198020873034527,
                                         0.221749184596974,
                                         0.171298117370715,
                                         0.0863765067205652,
                                         0.025557857384478,
                                         0.00356842592571593,
                                         0.000138435702807923,
                                         2.17625071413163e-07,
                                         5.67833172261963e-33,
                                         6.51356917856175e-23,
                                         1.58736081977083e-17,
                                         7.01900463095598e-14,
                                         3.65645707818059e-11,
                                         4.92718233216169e-09,
                                         2.57286594268466e-07,
                                         6.57876376270451e-06,
                                         9.5346775820568e-05,
                                         0.00086215117091839,
                                         0.00518798828125,
                                         0.0216920444258977,
                                         0.0647623603369979,
                                         0.139988638611413,
                                         0.219245995659656,
                                         0.245128121318903,
                                         0.188475239118916,
                                         0.0922942159889417,
                                         0.0245008944939906,
                                         0.00237419381387521,
                                         1.98418834605808e-05,
                                         3.5588422961035e-36,
                                         2.17026577732249e-25,
                                         1.3210898435041e-19,
                                         1.11067612830353e-15,
                                         9.57555079120737e-13,
                                         1.96752399847148e-10,
                                         1.48699487245838e-08,
                                         5.31770723559457e-07,
                                         1.05409547202826e-05,
                                         0.000128610226542871,
                                         0.00103759765625,
                                         0.00581658924149932,
                                         0.0234319657621943,
                                         0.069274380260923,
                                         0.151739744568219,
                                         0.245545355363755,
                                         0.287879679055515,
                                         0.233304205583789,
                                         0.117756593649806,
                                         0.0285024549740971,
                                         0.00126635334658238,
                                         1.39404484310097e-39,
                                         4.51946296175785e-28,
                                         6.8717771696579e-22,
                                         1.09844764931373e-17,
                                         1.56728171209343e-14,
                                         4.91045188658414e-12,
                                         5.37132957948223e-10,
                                         2.68648594776698e-08,
                                         7.2833956273564e-07,
                                         1.1990784598852e-05,
                                         0.00012969970703125,
                                         0.00097480180233535,
                                         0.00529876668159917,
                                         0.0214255769617136,
                                         0.065636746330167,
                                         0.153727062240735,
                                         0.274819619544353,
                                         0.368596042015273,
                                         0.35372727288096,
                                         0.213859420091012,
                                         0.0505134379501545,
                                         2.56972003534989e-43,
                                         4.42896049629039e-31,
                                         1.68208077957924e-24,
                                         5.11225320075267e-20,
                                         1.20717833727762e-16,
                                         5.76718585788749e-14,
                                         9.13051337085525e-12,
                                         6.3868363499751e-10,
                                         2.36825749256552e-08,
                                         5.26090870379766e-07,
                                         7.62939453125e-06,
                                         7.68785736093973e-05,
                                         0.000563873825052832,
                                         0.00311841181936169,
                                         0.0133609028477784,
                                         0.0452908005867576,
                                         0.123459787144882,
                                         0.274043706608637,
                                         0.500026386740259,
                                         0.755119293956849,
                                         0.948200147410435,

                                         0.95259510641313,
                                         0.773824799502045,
                                         0.531267929668918,
                                         0.307076337687578,
                                         0.148587178619108,
                                         0.0597229597889142,
                                         0.0197367890158302,
                                         0.00529235273043461,
                                         0.00113221721365986,
                                         0.000189136291215494,
                                         2.40002109818983e-05,
                                         2.23238833339106e-06,
                                         1.45251004797523e-07,
                                         6.20941009855194e-09,
                                         1.6001021699454e-10,
                                         2.19879061896772e-12,
                                         1.34272948342565e-14,
                                         2.73260449347304e-17,
                                         1.13133402245253e-20,
                                         3.65888985718399e-25,
                                         9.52023879514044e-32,
                                         5.48747459099356e-44,
                                         0.0463292383254292,
                                         0.199920153643746,
                                         0.342350137066624,
                                         0.375438726118709,
                                         0.299790747439263,
                                         0.18305485674977,
                                         0.0871463863751599,
                                         0.0324918254115038,
                                         0.00943876514401531,
                                         0.00210867062210497,
                                         0.000354805813445228,
                                         4.3640721236154e-05,
                                         3.76515716576099e-06,
                                         2.15259714387706e-07,
                                         7.53217198314147e-09,
                                         1.43915918384409e-10,
                                         1.2660371064112e-12,
                                         3.91415241066612e-15,
                                         2.6742128746623e-18,
                                         1.64093082414059e-22,
                                         1.06495621307236e-28,
                                         3.26079540982535e-40,
                                         0.00106033486067902,
                                         0.0243058926538947,
                                         0.103817000627892,
                                         0.216009498417675,
                                         0.284640162758525,
                                         0.264035458926577,
                                         0.181077017904629,
                                         0.0938729585960153,
                                         0.0370289734551372,
                                         0.0110632745965804,
                                         0.00246835409021986,
                                         0.000401471906388299,
                                         4.5929121523581e-05,
                                         3.51169057004077e-06,
                                         1.66852918297034e-07,
                                         4.43276652083389e-09,
                                         5.61753012044504e-11,
                                         2.63839461678278e-13,
                                         2.97469311640622e-16,
                                         3.46315775898738e-20,
                                         5.60604690289331e-26,
                                         9.11833713062438e-37,
                                         1.51673929009167e-05,
                                         0.00184691365194791,
                                         0.0196764373159764,
                                         0.0776759630809604,
                                         0.168909528869828,
                                         0.238025382146327,
                                         0.235156669836538,
                                         0.169506719074047,
                                         0.0907921252290771,
                                         0.0362776088392717,
                                         0.0107325818867355,
                                         0.00230833277747147,
                                         0.000350165363465895,
                                         3.58054998086582e-05,
                                         2.31008203929657e-06,
                                         8.53337631479932e-08,
                                         1.55784556463159e-09,
                                         1.1115315372779e-11,
                                         2.06808497292685e-14,
                                         4.56808625293355e-18,
                                         1.844428055566e-23,
                                         1.59363095460535e-33,
                                         1.51871706910043e-07,
                                         9.82380306054502e-05,
                                         0.00261049277262468,
                                         0.0195523284824346,
                                         0.0701633250437039,
                                         0.150204286739994,
                                         0.213771256043187,
                                         0.214255199459829,
                                         0.155830595993154,
                                         0.083270592637319,
                                         0.0326662289208497,
                                         0.0092905134549487,
                                         0.00186877180300029,
                                         0.000255553174086478,
                                         2.23881928942134e-05,
                                         1.14991298755333e-06,
                                         3.02413681074314e-08,
                                         3.277946541253e-10,
                                         1.00645098553529e-12,
                                         4.21788131714781e-16,
                                         4.24780676608729e-21,
                                         1.94965563146678e-30,
                                         1.12966094582275e-09,
                                         3.88166460181569e-06,
                                         0.000257278697021908,
                                         0.00365607946579328,
                                         0.0216506768136789,
                                         0.070412001123863,
                                         0.14435991587994,
                                         0.201178296210608,
                                         0.19868385823869,
                                         0.141987450441776,
                                         0.0738582639625144,
                                         0.0277770626559485,
                                         0.00740874527273832,
                                         0.00135493394151454,
                                         0.000161181792137928,
                                         1.15110347163408e-05,
                                         4.36097674342809e-07,
                                         7.18103837146786e-09,
                                         3.63849836072343e-11,
                                         2.89307517623483e-14,
                                         7.26730000131552e-19,
                                         1.77187615540043e-27,
                                         6.46362278389777e-12,
                                         1.17981255779595e-07,
                                         1.95048137552437e-05,
                                         0.0005258833443646,
                                         0.00513912806242154,
                                         0.0253902919605289,
                                         0.0749895209652481,
                                         0.145307332162915,
                                         0.194862866074951,
                                         0.186236548401479,
                                         0.128456426755078,
                                         0.0638836273924379,
                                         0.0225938219168654,
                                         0.005526009312726,
                                         0.000892626073022254,
                                         8.86380914001097e-05,
                                         4.83751978648776e-06,
                                         1.21012233265126e-07,
                                         1.01183194223703e-09,
                                         1.52644700137783e-12,
                                         9.56396708256102e-17,
                                         1.23869808651389e-24,
                                         2.90581887877263e-14,
                                         2.81755674388126e-09,
                                         1.16183495183987e-06,
                                         5.94330277075521e-05,
                                         0.000958455717316043,
                                         0.00719371705214955,
                                         0.0306068889510934,
                                         0.0824628959052459,
                                         0.150162066520845,
                                         0.191930730688016,
                                         0.175540458540227,
                                         0.115440305377699,
                                         0.0541376486282239,
                                         0.0177080097452788,
                                         0.00388407677193743,
                                         0.000536279354346779,
                                         4.21625032444509e-05,
                                         1.60227093179542e-06,
                                         2.2108497902422e-08,
                                         6.32802797983424e-11,
                                         9.88934905808292e-15,
                                         6.80396732236244e-22,
                                         1.03914570452352e-16,
                                         5.35238932541154e-11,
                                         5.50506479264574e-08,
                                         5.3429567910424e-06,
                                         0.000142190320302574,
                                         0.00162126637966702,
                                         0.00993695136340637,
                                         0.0372258825632567,
                                         0.0920463887875221,
                                         0.157340123653074,
                                         0.190815648490516,
                                         0.165936044654969,
                                         0.103186871315174,
                                         0.0451380985098392,
                                         0.0134437804290133,
                                         0.00258093535887256,
                                         0.000292311154835112,
                                         1.68755524725672e-05,
                                         3.84260245483923e-07,
                                         2.08674998618255e-09,
                                         8.13415993924324e-13,
                                         2.97285919123983e-19,
                                         2.97285919123983e-19,
                                         8.13415993924324e-13,
                                         2.08674998618252e-09,
                                         3.84260245483926e-07,
                                         1.68755524725672e-05,
                                         0.000292311154835113,
                                         0.00258093535887256,
                                         0.0134437804290133,
                                         0.0451380985098392,
                                         0.103186871315174,
                                         0.165936044654969,
                                         0.190815648490516,
                                         0.157340123653074,
                                         0.0920463887875221,
                                         0.0372258825632567,
                                         0.00993695136340637,
                                         0.00162126637966702,
                                         0.000142190320302574,
                                         5.34295679104237e-06,
                                         5.50506479264581e-08,
                                         5.35238932541154e-11,
                                         1.03914570452352e-16,
                                         6.80396732236244e-22,
                                         9.88934905808292e-15,
                                         6.32802797983415e-11,
                                         2.21084979024222e-08,
                                         1.60227093179543e-06,
                                         4.21625032444509e-05,
                                         0.000536279354346779,
                                         0.00388407677193743,
                                         0.0177080097452788,
                                         0.0541376486282238,
                                         0.115440305377699,
                                         0.175540458540227,
                                         0.191930730688017,
                                         0.150162066520845,
                                         0.0824628959052459,
                                         0.0306068889510934,
                                         0.00719371705214955,
                                         0.000958455717316039,
                                         5.94330277075518e-05,
                                         1.16183495183988e-06,
                                         2.81755674388126e-09,
                                         2.90581887877263e-14,
                                         1.23869808651389e-24,
                                         9.56396708256102e-17,
                                         1.5264470013778e-12,
                                         1.01183194223704e-09,
                                         1.21012233265127e-07,
                                         4.83751978648776e-06,
                                         8.86380914001097e-05,
                                         0.000892626073022254,
                                         0.005526009312726,
                                         0.0225938219168654,
                                         0.0638836273924379,
                                         0.128456426755078,
                                         0.186236548401479,
                                         0.194862866074951,
                                         0.145307332162915,
                                         0.0749895209652481,
                                         0.0253902919605289,
                                         0.00513912806242152,
                                         0.000525883344364598,
                                         1.95048137552439e-05,
                                         1.17981255779595e-07,
                                         6.46362278389777e-12,
                                         1.77187615540043e-27,
                                         7.26730000131552e-19,
                                         2.89307517623478e-14,
                                         3.63849836072346e-11,
                                         7.18103837146791e-09,
                                         4.36097674342809e-07,
                                         1.15110347163408e-05,
                                         0.000161181792137928,
                                         0.00135493394151454,
                                         0.00740874527273831,
                                         0.0277770626559485,
                                         0.0738582639625144,
                                         0.141987450441776,
                                         0.19868385823869,
                                         0.201178296210608,
                                         0.14435991587994,
                                         0.070412001123863,
                                         0.0216506768136789,
                                         0.00365607946579327,
                                         0.00025727869702191,
                                         3.88166460181569e-06,
                                         1.12966094582275e-09,
                                         1.94965563146678e-30,
                                         4.24780676608729e-21,
                                         4.21788131714772e-16,
                                         1.0064509855353e-12,
                                         3.27794654125302e-10,
                                         3.02413681074314e-08,
                                         1.14991298755333e-06,
                                         2.23881928942134e-05,
                                         0.000255553174086478,
                                         0.00186877180300028,
                                         0.0092905134549487,
                                         0.0326662289208497,
                                         0.083270592637319,
                                         0.155830595993153,
                                         0.214255199459828,
                                         0.213771256043187,
                                         0.150204286739994,
                                         0.0701633250437038,
                                         0.0195523284824345,
                                         0.0026104927726247,
                                         9.82380306054502e-05,
                                         1.51871706910043e-07,
                                         1.59363095460535e-33,
                                         1.844428055566e-23,
                                         4.56808625293345e-18,
                                         2.06808497292687e-14,
                                         1.11153153727791e-11,
                                         1.55784556463159e-09,
                                         8.53337631479932e-08,
                                         2.31008203929657e-06,
                                         3.58054998086582e-05,
                                         0.000350165363465894,
                                         0.00230833277747147,
                                         0.0107325818867355,
                                         0.0362776088392717,
                                         0.0907921252290771,
                                         0.169506719074047,
                                         0.235156669836538,
                                         0.238025382146327,
                                         0.168909528869828,
                                         0.0776759630809603,
                                         0.0196764373159765,
                                         0.00184691365194791,
                                         1.51673929009167e-05,
                                         9.11833713062438e-37,
                                         5.60604690289331e-26,
                                         3.4631577589873e-20,
                                         2.97469311640626e-16,
                                         2.6383946167828e-13,
                                         5.61753012044504e-11,
                                         4.43276652083389e-09,
                                         1.66852918297034e-07,
                                         3.51169057004078e-06,
                                         4.59291215235809e-05,
                                         0.000401471906388299,
                                         0.00246835409021986,
                                         0.0110632745965804,
                                         0.0370289734551372,
                                         0.0938729585960153,
                                         0.181077017904629,
                                         0.264035458926577,
                                         0.284640162758525,
                                         0.216009498417675,
                                         0.103817000627893,
                                         0.0243058926538947,
                                         0.00106033486067902,
                                         3.26079540982535e-40,
                                         1.06495621307236e-28,
                                         1.64093082414055e-22,
                                         2.67421287466234e-18,
                                         3.91415241066615e-15,
                                         1.2660371064112e-12,
                                         1.43915918384409e-10,
                                         7.53217198314147e-09,
                                         2.15259714387706e-07,
                                         3.76515716576098e-06,
                                         4.3640721236154e-05,
                                         0.000354805813445228,
                                         0.00210867062210497,
                                         0.00943876514401531,
                                         0.0324918254115038,
                                         0.0871463863751599,
                                         0.18305485674977,
                                         0.299790747439263,
                                         0.375438726118709,
                                         0.342350137066625,
                                         0.199920153643746,
                                         0.0463292383254292,
                                         5.48747459099356e-44,
                                         9.52023879514044e-32,
                                         3.65888985718389e-25,
                                         1.13133402245254e-20,
                                         2.73260449347307e-17,
                                         1.34272948342565e-14,
                                         2.19879061896772e-12,
                                         1.6001021699454e-10,
                                         6.20941009855194e-09,
                                         1.45251004797523e-07,
                                         2.23238833339106e-06,
                                         2.40002109818983e-05,
                                         0.000189136291215494,
                                         0.00113221721365986,
                                         0.00529235273043461,
                                         0.0197367890158302,
                                         0.0597229597889142,
                                         0.148587178619108,
                                         0.307076337687578,
                                         0.531267929668918,
                                         0.773824799502045,
                                         0.95259510641313,

                                         0.956457528820378,
                                         0.790566199873214,
                                         0.560171692806069,
                                         0.33916307814019,
                                         0.174640953534276,
                                         0.0759845696695523,
                                         0.0276991951631349,
                                         0.00836874427001042,
                                         0.00206699998272176,
                                         0.000410187856360438,
                                         6.3984076287965e-05,
                                         7.62939453125e-06,
                                         6.70853051304942e-07,
                                         4.15003482497175e-08,
                                         1.69610533152497e-09,
                                         4.20079055009823e-11,
                                         5.57682527018191e-13,
                                         3.30665979566376e-15,
                                         6.56602524570785e-18,
                                         2.66518021588677e-21,
                                         8.49077034075324e-26,
                                         2.18642548810035e-32,
                                         1.25301850611583e-44,
                                         0.0426362337676956,
                                         0.187077797595501,
                                         0.330222681686196,
                                         0.378642306797291,
                                         0.320942881818869,
                                         0.211452777790369,
                                         0.110594846079539,
                                         0.0462288807718812,
                                         0.0154097645470505,
                                         0.00405915631237831,
                                         0.000831922583548828,
                                         0.00012969970703125,
                                         1.49112562252554e-05,
                                         1.2119832200776e-06,
                                         6.57499053663559e-08,
                                         2.19773735027826e-09,
                                         4.03661325643423e-11,
                                         3.43399272906857e-13,
                                         1.03256817869215e-15,
                                         6.89928132889279e-19,
                                         4.16254743719674e-23,
                                         2.67022906120782e-29,
                                         8.12348126810799e-41,
                                         0.000894402583364953,
                                         0.0208327844368615,
                                         0.0916080643303133,
                                         0.198925643721317,
                                         0.277555902534785,
                                         0.27691242750507,
                                         0.207799114439263,
                                         0.12017317257243,
                                         0.0540620610445355,
                                         0.0189029601211819,
                                         0.00509020166685909,
                                         0.00103759765625,
                                         0.000155970380614363,
                                         1.66564535729385e-05,
                                         1.1994399516236e-06,
                                         5.41080166944256e-08,
                                         1.37495445401146e-09,
                                         1.67822541274179e-11,
                                         7.64145379389031e-14,
                                         8.40469956108893e-17,
                                         9.6031201613049e-21,
                                         1.53462906963085e-26,
                                         2.47838029305497e-37,
                                         1.17264693436643e-05,
                                         0.0014499479393432,
                                         0.0158832923882891,
                                         0.0653179316923326,
                                         0.150021396711052,
                                         0.22664780438227,
                                         0.244023984234124,
                                         0.195245796093705,
                                         0.118541170578045,
                                         0.0550178833119439,
                                         0.0194655680120252,
                                         0.00518798828125,
                                         0.00101964747560079,
                                         0.000143069970478681,
                                         1.36754588232359e-05,
                                         8.32582846578992e-07,
                                         2.92711307491582e-08,
                                         5.12603104858689e-10,
                                         3.53438017999207e-12,
                                         6.39912435665099e-15,
                                         1.38466765580223e-18,
                                         5.51236974284795e-24,
                                         4.72578248342678e-34,
                                         1.0762162372719e-07,
                                         7.0640788475687e-05,
                                         0.00192772639892885,
                                         0.0150131601161431,
                                         0.056761515377321,
                                         0.129854984790288,
                                         0.200594663406183,
                                         0.22205159482248,
                                         0.181946566508186,
                                         0.112092350896638,
                                         0.0521071371982022,
                                         0.018157958984375,
                                         0.00466612108839287,
                                         0.00086022582503508,
                                         0.000109144873522853,
                                         8.96791209762258e-06,
                                         4.36203079297798e-07,
                                         1.09599913564077e-08,
                                         1.14432286258094e-10,
                                         3.41049130477247e-13,
                                         1.39758030658134e-16,
                                         1.38602575359936e-21,
                                         6.30779469166705e-31,
                                         7.33731410562579e-10,
                                         2.5566069824727e-06,
                                         0.00017380231725858,
                                         0.00256340432171513,
                                         0.0159536500721424,
                                         0.0552676551107189,
                                         0.122493075769528,
                                         0.187599382857335,
                                         0.207454959191435,
                                         0.169649811350906,
                                         0.103617390135367,
                                         0.047210693359375,
                                         0.0158623403773268,
                                         0.0038422159365434,
                                         0.000647098035409353,
                                         7.1756381473442e-05,
                                         4.8288449203762e-06,
                                         1.74078242949321e-07,
                                         2.75225830075795e-09,
                                         1.35026386907731e-11,
                                         1.04788422043041e-14,
                                         2.58886594854433e-19,
                                         6.25441470308735e-28,
                                         3.84796692338481e-12,
                                         7.11752581548853e-08,
                                         1.20537554983212e-05,
                                         0.000336681114115558,
                                         0.0034492350472168,
                                         0.0180942297130693,
                                         0.0575387406650558,
                                         0.121917363313652,
                                         0.181953504765679,
                                         0.197509246487557,
                                         0.158498345402736,
                                         0.09442138671875,
                                         0.0414796585329928,
                                         0.0132010266941117,
                                         0.0029511648418443,
                                         0.000441658268014258,
                                         4.11201260141704e-05,
                                         2.12684296511476e-06,
                                         5.09197723406776e-08,
                                         4.11222618951006e-10,
                                         6.04374979249814e-13,
                                         3.71967053288724e-17,
                                         4.77037418845898e-25,
                                         1.58558744612416e-14,
                                         1.55689305310834e-09,
                                         6.56831304074782e-07,
                                         3.47444185859829e-05,
                                         0.000585935981191245,
                                         0.00465450830765267,
                                         0.021236053677636,
                                         0.0622535851818814,
                                         0.125389646807829,
                                         0.180670036669719,
                                         0.19049407225704,
                                         0.14837646484375,
                                         0.0852251412642519,
                                         0.0356367671161006,
                                         0.0105750274874184,
                                         0.00213588000188929,
                                         0.000275125132767255,
                                         2.04169565764605e-05,
                                         7.40198781478478e-07,
                                         9.84011087602816e-09,
                                         2.7388251165252e-11,
                                         4.1991755858281e-15,
                                         2.85879404229358e-22,
                                         5.19714022647426e-17,
                                         2.70896802222212e-11,
                                         2.84708658802782e-08,
                                         2.85211287674452e-06,
                                         7.91758826589491e-05,
                                         0.000952407678045794,
                                         0.0062345145716936,
                                         0.0252859072835035,
                                         0.0687350575606203,
                                         0.131461989706056,
                                         0.18211829128251,
                                         0.185470581054688,
                                         0.139288626156966,
                                         0.0765251618078488,
                                         0.0301428914309171,
                                         0.00821642133903668,
                                         0.00146427100005283,
                                         0.000155905685454976,
                                         8.55905163165268e-06,
                                         1.87300250060276e-07,
                                         9.87273534551895e-10,
                                         3.77084744813818e-13,
                                         1.36278912145306e-19,
                                         1.36278912145332e-19,
                                         3.77084744813818e-13,
                                         9.8727353455188e-10,
                                         1.87300250060276e-07,
                                         8.55905163165268e-06,
                                         0.000155905685454975,
                                         0.00146427100005284,
                                         0.00821642133903666,
                                         0.0301428914309171,
                                         0.0765251618078488,
                                         0.139288626156966,
                                         0.185470581054688,
                                         0.18211829128251,
                                         0.131461989706056,
                                         0.0687350575606203,
                                         0.0252859072835035,
                                         0.00623451457169358,
                                         0.000952407678045797,
                                         7.91758826589491e-05,
                                         2.85211287674452e-06,
                                         2.84708658802786e-08,
                                         2.70896802222212e-11,
                                         5.19714022647338e-17,
                                         2.85879404229419e-22,
                                         4.1991755858281e-15,
                                         2.73882511652515e-11,
                                         9.84011087602816e-09,
                                         7.40198781478478e-07,
                                         2.04169565764604e-05,
                                         0.000275125132767256,
                                         0.00213588000188928,
                                         0.0105750274874184,
                                         0.0356367671161006,
                                         0.0852251412642519,
                                         0.14837646484375,
                                         0.19049407225704,
                                         0.180670036669719,
                                         0.125389646807829,
                                         0.0622535851818815,
                                         0.021236053677636,
                                         0.00465450830765268,
                                         0.000585935981191245,
                                         3.47444185859829e-05,
                                         6.5683130407479e-07,
                                         1.55689305310834e-09,
                                         1.58558744612392e-14,
                                         4.7703741884601e-25,
                                         3.71967053288724e-17,
                                         6.04374979249803e-13,
                                         4.11222618951006e-10,
                                         5.09197723406776e-08,
                                         2.12684296511475e-06,
                                         4.11201260141705e-05,
                                         0.000441658268014256,
                                         0.0029511648418443,
                                         0.0132010266941117,
                                         0.0414796585329928,
                                         0.09442138671875,
                                         0.158498345402736,
                                         0.197509246487557,
                                         0.181953504765679,
                                         0.121917363313653,
                                         0.0575387406650557,
                                         0.0180942297130694,
                                         0.0034492350472168,
                                         0.000336681114115558,
                                         1.20537554983214e-05,
                                         7.11752581548853e-08,
                                         3.84796692338432e-12,
                                         6.25441470308894e-28,
                                         2.58886594854433e-19,
                                         1.04788422043039e-14,
                                         1.35026386907731e-11,
                                         2.75225830075795e-09,
                                         1.7407824294932e-07,
                                         4.82884492037621e-06,
                                         7.17563814734418e-05,
                                         0.000647098035409353,
                                         0.0038422159365434,
                                         0.0158623403773268,
                                         0.047210693359375,
                                         0.103617390135367,
                                         0.169649811350906,
                                         0.207454959191435,
                                         0.187599382857335,
                                         0.122493075769528,
                                         0.055267655110719,
                                         0.0159536500721424,
                                         0.00256340432171513,
                                         0.000173802317258581,
                                         2.5566069824727e-06,
                                         7.33731410562501e-10,
                                         6.30779469166879e-31,
                                         1.38602575359936e-21,
                                         1.39758030658131e-16,
                                         3.41049130477247e-13,
                                         1.14432286258094e-10,
                                         1.09599913564077e-08,
                                         4.36203079297799e-07,
                                         8.96791209762256e-06,
                                         0.000109144873522853,
                                         0.00086022582503508,
                                         0.00466612108839287,
                                         0.018157958984375,
                                         0.0521071371982022,
                                         0.112092350896638,
                                         0.181946566508186,
                                         0.222051594822481,
                                         0.200594663406183,
                                         0.129854984790288,
                                         0.056761515377321,
                                         0.0150131601161431,
                                         0.00192772639892886,
                                         7.0640788475687e-05,
                                         1.07621623727181e-07,
                                         4.72578248342819e-34,
                                         5.51236974284795e-24,
                                         1.3846676558022e-18,
                                         6.39912435665099e-15,
                                         3.53438017999207e-12,
                                         5.12603104858686e-10,
                                         2.92711307491583e-08,
                                         8.32582846578989e-07,
                                         1.36754588232359e-05,
                                         0.000143069970478681,
                                         0.00101964747560079,
                                         0.00518798828125,
                                         0.0194655680120252,
                                         0.0550178833119439,
                                         0.118541170578045,
                                         0.195245796093705,
                                         0.244023984234123,
                                         0.22664780438227,
                                         0.150021396711052,
                                         0.0653179316923326,
                                         0.0158832923882892,
                                         0.0014499479393432,
                                         1.17264693436636e-05,
                                         2.47838029305576e-37,
                                         1.53462906963085e-26,
                                         9.60312016130467e-21,
                                         8.40469956108893e-17,
                                         7.64145379389031e-14,
                                         1.67822541274178e-11,
                                         1.37495445401146e-09,
                                         5.41080166944254e-08,
                                         1.1994399516236e-06,
                                         1.66564535729385e-05,
                                         0.000155970380614363,
                                         0.00103759765625,
                                         0.00509020166685909,
                                         0.0189029601211819,
                                         0.0540620610445355,
                                         0.12017317257243,
                                         0.207799114439263,
                                         0.27691242750507,
                                         0.277555902534785,
                                         0.198925643721317,
                                         0.0916080643303136,
                                         0.0208327844368615,
                                         0.000894402583364915,
                                         8.12348126811074e-41,
                                         2.67022906120782e-29,
                                         4.16254743719663e-23,
                                         6.89928132889279e-19,
                                         1.03256817869215e-15,
                                         3.43399272906854e-13,
                                         4.03661325643425e-11,
                                         2.19773735027826e-09,
                                         6.57499053663559e-08,
                                         1.2119832200776e-06,
                                         1.49112562252554e-05,
                                         0.00012969970703125,
                                         0.000831922583548828,
                                         0.00405915631237832,
                                         0.0154097645470505,
                                         0.0462288807718812,
                                         0.110594846079539,
                                         0.211452777790369,
                                         0.320942881818869,
                                         0.378642306797291,
                                         0.330222681686196,
                                         0.187077797595501,
                                         0.0426362337676947,
                                         1.25301850611628e-44,
                                         2.18642548810035e-32,
                                         8.490770340753e-26,
                                         2.66518021588677e-21,
                                         6.56602524570785e-18,
                                         3.30665979566374e-15,
                                         5.57682527018193e-13,
                                         4.20079055009821e-11,
                                         1.69610533152497e-09,
                                         4.15003482497175e-08,
                                         6.70853051304942e-07,
                                         7.62939453125e-06,
                                         6.3984076287965e-05,
                                         0.000410187856360438,
                                         0.00206699998272176,
                                         0.00836874427001042,
                                         0.0276991951631349,
                                         0.0759845696695523,
                                         0.174640953534276,
                                         0.33916307814019,
                                         0.560171692806069,
                                         0.790566199873214,
                                         0.956457528820378,

                                         0.959869510625733,
                                         0.805593554610748,
                                         0.586881026527553,
                                         0.370105240847824,
                                         0.201238803157533,
                                         0.0938327044736947,
                                         0.037254792230587,
                                         0.0124821122850671,
                                         0.00348949383658583,
                                         0.000802541484639564,
                                         0.00014918557803142,
                                         2.19226228892619e-05,
                                         2.47588647577842e-06,
                                         2.07266954571643e-07,
                                         1.22678870343184e-08,
                                         4.82017090007351e-10,
                                         1.1530328736432e-11,
                                         1.48508322910673e-13,
                                         8.58044043030905e-16,
                                         1.66740539334734e-18,
                                         6.65139161586663e-22,
                                         2.09113627261053e-26,
                                         5.33581874926594e-33,
                                         3.04244156244955e-45,
                                         0.039361666202767,
                                         0.175261916470592,
                                         0.317722482701608,
                                         0.378839941814523,
                                         0.338342456916247,
                                         0.238225783447659,
                                         0.135231006883119,
                                         0.0624164383673925,
                                         0.0234260783946267,
                                         0.00710624335244585,
                                         0.0017222863681946,
                                         0.000327812896716841,
                                         4.78514440214581e-05,
                                         5.18859038062986e-06,
                                         4.00400882722562e-07,
                                         2.07502399211896e-08,
                                         6.66390249777575e-10,
                                         1.18237299074977e-11,
                                         9.76726325485495e-14,
                                         2.86611935378843e-16,
                                         1.87793188206583e-19,
                                         1.11630353459007e-23,
                                         7.08805923036691e-30,
                                         2.14416800325193e-41,
                                         0.000759584015318202,
                                         0.0179432122923613,
                                         0.0809444154887012,
                                         0.182485076165663,
                                         0.267696280663325,
                                         0.284619308713279,
                                         0.230999735827847,
                                         0.146876037324368,
                                         0.0740078545891049,
                                         0.0296110444430161,
                                         0.009356748422564,
                                         0.00230675049930802,
                                         0.000435211571781892,
                                         6.11237191665735e-05,
                                         6.14980506563089e-06,
                                         4.20363378822365e-07,
                                         1.81241081223413e-08,
                                         4.42995457399921e-10,
                                         5.23211488703903e-12,
                                         2.31840020864981e-14,
                                         2.49510149939092e-17,
                                         2.80429262627944e-21,
                                         4.43092779430589e-27,
                                         7.11109436951926e-38,
                                         9.1613226139034e-06,
                                         0.00114813472406158,
                                         0.0128886031748044,
                                         0.0549387738521043,
                                         0.132375676655614,
                                         0.212529867989245,
                                         0.246619096381937,
                                         0.216014511805466,
                                         0.146128879549746,
                                         0.0771165147954451,
                                         0.0317705372128852,
                                         0.0101450742164022,
                                         0.0024739210603541,
                                         0.000450039024411845,
                                         5.90347448913232e-05,
                                         5.32238937134253e-06,
                                         3.0808082739427e-07,
                                         1.03734701746126e-08,
                                         1.75170781446688e-10,
                                         1.17209431638873e-12,
                                         2.07193733693045e-15,
                                         4.40295632534495e-19,
                                         1.731179198729e-24,
                                         1.47398848456358e-34,
                                         7.73461279337228e-08,
                                         5.14260950705021e-05,
                                         0.00143655697006714,
                                         0.0115778684759012,
                                         0.0458217940444572,
                                         0.111089656897171,
                                         0.184306206834363,
                                         0.222388819254783,
                                         0.201972543183618,
                                         0.140585037652027,
                                         0.0755130834396324,
                                         0.0312325809059229,
                                         0.00984394732764948,
                                         0.0023194692400692,
                                         0.000396690748248662,
                                         4.71722348642206e-05,
                                         3.66581665133951e-06,
                                         1.70038354042314e-07,
                                         4.10529247437953e-09,
                                         4.14796184442028e-11,
                                         1.20437867174945e-13,
                                         4.8390873886905e-17,
                                         4.73464495488811e-22,
                                         2.13869955566494e-31,
                                         4.85092188166867e-10,
                                         1.71111637904464e-06,
                                         0.000118944717297331,
                                         0.00181252315998127,
                                         0.0117826041816236,
                                         0.0431352760832452,
                                         0.102319526957782,
                                         0.170078052285932,
                                         0.207373815002893,
                                         0.190386483719834,
                                         0.13332916328929,
                                         0.0714275612836413,
                                         0.0290976575503568,
                                         0.00888039664237143,
                                         0.00198016671655703,
                                         0.000310578626346864,
                                         3.24027679644368e-05,
                                         2.07049900323247e-06,
                                         7.14713264696382e-08,
                                         1.0904663016496e-09,
                                         5.20061613514705e-12,
                                         3.95082557297305e-15,
                                         9.61918136489928e-20,
                                         2.30521146767785e-28,
                                         2.34027359378767e-12,
                                         4.37957730127959e-08,
                                         7.5757234231858e-06,
                                         0.000218270561254038,
                                         0.00233059674559964,
                                         0.0128839249495711,
                                         0.0436952002392959,
                                         0.1000553340154,
                                         0.163784254536627,
                                         0.198330621088188,
                                         0.181085947918838,
                                         0.12565518646885,
                                         0.0661612075488424,
                                         0.0261536771375563,
                                         0.00760340438154953,
                                         0.00157294423169364,
                                         0.000220318075724625,
                                         1.93936613099013e-05,
                                         9.57141682017085e-07,
                                         2.20519192434217e-08,
                                         1.72744086732896e-10,
                                         2.48124069276398e-13,
                                         1.50329930531616e-17,
                                         1.91129802602846e-25,
                                         8.87102152969011e-15,
                                         8.8074359481915e-10,
                                         3.79112158497584e-07,
                                         2.06524421938615e-05,
                                         0.00036220766149098,
                                         0.00302362856868531,
                                         0.0146613375820373,
                                         0.046248416550173,
                                         0.101637746849706,
                                         0.162333473101489,
                                         0.193245325527624,
                                         0.17368394099507,
                                         0.118198908879278,
                                         0.060519844270328,
                                         0.0229392423940062,
                                         0.00625921305944365,
                                         0.00117701716979205,
                                         0.000142728018793527,
                                         1.00712938138709e-05,
                                         3.50384742839593e-07,
                                         4.50833552331616e-09,
                                         1.22437536659576e-11,
                                         1.84593944358151e-15,
                                         1.245118542136e-22,
                                         2.67482887045512e-17,
                                         1.40890645405391e-11,
                                         1.50913010493632e-08,
                                         1.55440087899597e-06,
                                         4.47778767797098e-05,
                                         0.000564448160119871,
                                         0.00391317095371837,
                                         0.0170046954303348,
                                         0.050171060050208,
                                         0.105691912559163,
                                         0.164039566113745,
                                         0.190965220569019,
                                         0.16797276827342,
                                         0.111398210237957,
                                         0.0550510263446644,
                                         0.0198126018695297,
                                         0.00500185196809497,
                                         0.000835553063414137,
                                         8.4296524715332e-05,
                                         4.42852752621591e-06,
                                         9.35932736649074e-08,
                                         4.80591006661599e-10,
                                         1.8030377130882e-13,
                                         6.45220797832391e-20,
                                         6.45220797832525e-20,
                                         1.8030377130882e-13,
                                         4.80591006661607e-10,
                                         9.35932736649074e-08,
                                         4.42852752621591e-06,
                                         8.42965247153323e-05,
                                         0.000835553063414137,
                                         0.00500185196809498,
                                         0.0198126018695297,
                                         0.0550510263446644,
                                         0.111398210237957,
                                         0.16797276827342,
                                         0.190965220569019,
                                         0.164039566113745,
                                         0.105691912559163,
                                         0.050171060050208,
                                         0.0170046954303348,
                                         0.00391317095371837,
                                         0.000564448160119869,
                                         4.47778767797098e-05,
                                         1.55440087899597e-06,
                                         1.5091301049363e-08,
                                         1.40890645405391e-11,
                                         2.67482887045462e-17,
                                         1.24511854213628e-22,
                                         1.84593944358151e-15,
                                         1.22437536659578e-11,
                                         4.50833552331616e-09,
                                         3.50384742839593e-07,
                                         1.0071293813871e-05,
                                         0.000142728018793527,
                                         0.00117701716979205,
                                         0.00625921305944365,
                                         0.0229392423940062,
                                         0.060519844270328,
                                         0.118198908879278,
                                         0.17368394099507,
                                         0.193245325527624,
                                         0.162333473101489,
                                         0.101637746849706,
                                         0.0462484165501729,
                                         0.0146613375820373,
                                         0.0030236285686853,
                                         0.00036220766149098,
                                         2.06524421938615e-05,
                                         3.79112158497579e-07,
                                         8.8074359481915e-10,
                                         8.87102152968867e-15,
                                         1.91129802602894e-25,
                                         1.50329930531616e-17,
                                         2.48124069276403e-13,
                                         1.72744086732896e-10,
                                         2.20519192434217e-08,
                                         9.57141682017089e-07,
                                         1.93936613099013e-05,
                                         0.000220318075724626,
                                         0.00157294423169364,
                                         0.00760340438154953,
                                         0.0261536771375563,
                                         0.0661612075488423,
                                         0.12565518646885,
                                         0.181085947918838,
                                         0.198330621088188,
                                         0.163784254536627,
                                         0.1000553340154,
                                         0.0436952002392959,
                                         0.0128839249495711,
                                         0.00233059674559964,
                                         0.000218270561254038,
                                         7.57572342318571e-06,
                                         4.3795773012796e-08,
                                         2.34027359378735e-12,
                                         2.30521146767848e-28,
                                         9.61918136489928e-20,
                                         3.95082557297314e-15,
                                         5.20061613514705e-12,
                                         1.0904663016496e-09,
                                         7.14713264696386e-08,
                                         2.07049900323247e-06,
                                         3.24027679644369e-05,
                                         0.000310578626346864,
                                         0.00198016671655703,
                                         0.00888039664237143,
                                         0.0290976575503568,
                                         0.0714275612836414,
                                         0.13332916328929,
                                         0.190386483719834,
                                         0.207373815002893,
                                         0.170078052285932,
                                         0.102319526957782,
                                         0.0431352760832451,
                                         0.0117826041816236,
                                         0.00181252315998127,
                                         0.00011894471729733,
                                         1.71111637904464e-06,
                                         4.85092188166811e-10,
                                         2.13869955566559e-31,
                                         4.73464495488811e-22,
                                         4.83908738869062e-17,
                                         1.20437867174945e-13,
                                         4.14796184442028e-11,
                                         4.10529247437955e-09,
                                         1.70038354042314e-07,
                                         3.66581665133952e-06,
                                         4.71722348642206e-05,
                                         0.000396690748248662,
                                         0.0023194692400692,
                                         0.00984394732764947,
                                         0.0312325809059229,
                                         0.0755130834396324,
                                         0.140585037652027,
                                         0.201972543183618,
                                         0.222388819254783,
                                         0.184306206834363,
                                         0.111089656897171,
                                         0.0458217940444572,
                                         0.0115778684759012,
                                         0.00143655697006713,
                                         5.14260950705021e-05,
                                         7.73461279337156e-08,
                                         1.47398848456406e-34,
                                         1.731179198729e-24,
                                         4.40295632534506e-19,
                                         2.07193733693045e-15,
                                         1.17209431638873e-12,
                                         1.75170781446689e-10,
                                         1.03734701746126e-08,
                                         3.08080827394271e-07,
                                         5.32238937134253e-06,
                                         5.90347448913232e-05,
                                         0.000450039024411845,
                                         0.00247392106035409,
                                         0.0101450742164022,
                                         0.0317705372128852,
                                         0.0771165147954451,
                                         0.146128879549746,
                                         0.216014511805466,
                                         0.246619096381937,
                                         0.212529867989245,
                                         0.132375676655614,
                                         0.0549387738521043,
                                         0.0128886031748043,
                                         0.00114813472406158,
                                         9.16132261390276e-06,
                                         7.11109436952172e-38,
                                         4.43092779430589e-27,
                                         2.80429262627951e-21,
                                         2.49510149939092e-17,
                                         2.31840020864981e-14,
                                         5.23211488703907e-12,
                                         4.42995457399921e-10,
                                         1.81241081223414e-08,
                                         4.20363378822365e-07,
                                         6.14980506563089e-06,
                                         6.11237191665735e-05,
                                         0.000435211571781891,
                                         0.00230675049930802,
                                         0.009356748422564,
                                         0.0296110444430162,
                                         0.0740078545891049,
                                         0.146876037324368,
                                         0.230999735827847,
                                         0.284619308713278,
                                         0.267696280663325,
                                         0.182485076165663,
                                         0.0809444154887009,
                                         0.0179432122923613,
                                         0.000759584015318167,
                                         2.14416800325272e-41,
                                         7.08805923036691e-30,
                                         1.1163035345901e-23,
                                         1.87793188206583e-19,
                                         2.86611935378843e-16,
                                         9.76726325485501e-14,
                                         1.18237299074977e-11,
                                         6.66390249777578e-10,
                                         2.07502399211896e-08,
                                         4.00400882722562e-07,
                                         5.18859038062986e-06,
                                         4.7851444021458e-05,
                                         0.000327812896716841,
                                         0.0017222863681946,
                                         0.00710624335244585,
                                         0.0234260783946267,
                                         0.0624164383673925,
                                         0.135231006883119,
                                         0.238225783447658,
                                         0.338342456916247,
                                         0.378839941814523,
                                         0.317722482701608,
                                         0.175261916470592,
                                         0.0393616662027661,
                                         3.04244156245075e-45,
                                         5.33581874926594e-33,
                                         2.0911362726106e-26,
                                         6.65139161586663e-22,
                                         1.66740539334734e-18,
                                         8.58044043030911e-16,
                                         1.48508322910673e-13,
                                         1.1530328736432e-11,
                                         4.82017090007351e-10,
                                         1.22678870343184e-08,
                                         2.07266954571643e-07,
                                         2.47588647577841e-06,
                                         2.19226228892619e-05,
                                         0.00014918557803142,
                                         0.000802541484639564,
                                         0.00348949383658583,
                                         0.0124821122850671,
                                         0.037254792230587,
                                         0.0938327044736947,
                                         0.201238803157533,
                                         0.370105240847824,
                                         0.586881026527553,
                                         0.805593554610748,
                                         0.959869510625733,

                                         3.814697265625e-06,
                                         6.866455078125e-05,
                                         0.000583648681640625,
                                         0.00311279296875,
                                         0.0116729736328125,
                                         0.032684326171875,
                                         0.0708160400390625,
                                         0.12139892578125,
                                         0.166923522949219,
                                         0.185470581054688,
                                         0.166923522949219,
                                         0.12139892578125,
                                         0.0708160400390625,
                                         0.032684326171875,
                                         0.0116729736328125,
                                         0.00311279296875,
                                         0.000583648681640625,
                                         6.866455078125e-05,
                                         3.814697265625e-06,

                                         0.0139369157387641,
                                         7.06497808023742e-13,
                                         0.0672189357092842,
                                         4.74603428696517e-11,
                                         0.153095706084749,
                                         1.50555749294053e-09,
                                         0.218783310987099,
                                         2.99670243059737e-08,
                                         0.219835543111501,
                                         4.19394214730795e-07,
                                         0.164933317484275,
                                         4.38256144977218e-06,
                                         0.095753123320611,
                                         3.54378743286588e-05,
                                         0.043983380685455,
                                         0.000226724481440221,
                                         0.0162048030857052,
                                         0.00116345251557509,
                                         0.00482451544480007,
                                         0.00482451544480007,
                                         0.00116345251557509,
                                         0.0162048030857052,
                                         0.000226724481440221,
                                         0.043983380685455,
                                         3.54378743286588e-05,
                                         0.095753123320611,
                                         4.38256144977218e-06,
                                         0.164933317484275,
                                         4.19394214730795e-07,
                                         0.219835543111501,
                                         2.99670243059737e-08,
                                         0.218783310987099,
                                         1.50555749294053e-09,
                                         0.153095706084749,
                                         4.74603428696517e-11,
                                         0.0672189357092842,
                                         7.06497808023742e-13,
                                         0.0139369157387641,

                                         0.116212808,
                                         3.814697265625e-06,
                                         8.60490351459396e-18,
                                         0.265697315999998,
                                         6.866455078125e-05,
                                         1.21943271719016e-15,
                                         0.286857863999918,
                                         0.000583648681640625,
                                         8.16048745311976e-14,
                                         0.194323871996574,
                                         0.00311279296875,
                                         3.4265270301518e-12,
                                         0.0925588798988367,
                                         0.0116729736328125,
                                         1.0116371341393e-10,
                                         0.0329182537699116,
                                         0.032684326171875,
                                         2.23008864665543e-09,
                                         0.00905919395885673,
                                         0.0708160400390625,
                                         3.80411433473162e-08,
                                         0.00197257457607766,
                                         0.12139892578125,
                                         5.1342392236133e-07,
                                         0.000344506005263348,
                                         0.166923522949219,
                                         5.55799473665594e-06,
                                         4.86200000000007e-05,
                                         0.185470581054688,
                                         4.86200000000007e-05,
                                         5.55799473665594e-06,
                                         0.166923522949219,
                                         0.000344506005263348,
                                         5.1342392236133e-07,
                                         0.12139892578125,
                                         0.00197257457607766,
                                         3.80411433473162e-08,
                                         0.0708160400390625,
                                         0.00905919395885673,
                                         2.23008864665543e-09,
                                         0.032684326171875,
                                         0.0329182537699116,
                                         1.0116371341393e-10,
                                         0.0116729736328125,
                                         0.0925588798988367,
                                         3.4265270301518e-12,
                                         0.00311279296875,
                                         0.194323871996574,
                                         8.16048745311976e-14,
                                         0.000583648681640625,
                                         0.286857863999918,
                                         1.21943271719016e-15,
                                         6.866455078125e-05,
                                         0.265697315999998,
                                         8.60490351459396e-18,
                                         3.814697265625e-06,
                                         0.116212808,

                                         0.273821384962825,
                                         0.000740007054308786,
                                         2.15513977291413e-09,
                                         1.40623083084567e-21,
                                         0.36774805283355,
                                         0.0065609408051961,
                                         7.87571863662611e-08,
                                         3.39249023647866e-19,
                                         0.23322753468986,
                                         0.0274689968598195,
                                         1.35909984688253e-06,
                                         3.86479994343365e-17,
                                         0.0928087799640949,
                                         0.0721604582014504,
                                         1.47160866696173e-05,
                                         2.76258234506676e-15,
                                         0.0259675421582203,
                                         0.133287156965508,
                                         0.000112038129632725,
                                         1.38846835389263e-13,
                                         0.00542499560580222,
                                         0.183824795031413,
                                         0.00063689248852718,
                                         5.21054922396168e-12,
                                         0.000877004873428894,
                                         0.196179646158592,
                                         0.00280156352347396,
                                         1.5130911493133e-10,
                                         0.000112174909227535,
                                         0.165651322496429,
                                         0.00975047842585032,
                                         3.47646437172406e-09,
                                         1.15082407582063e-05,
                                         0.112190313138144,
                                         0.0272189187569618,
                                         6.40663477948829e-08,
                                         9.54062758322472e-07,
                                         0.0614003158658506,
                                         0.0614003158658506,
                                         9.54062758322479e-07,
                                         6.40663477948823e-08,
                                         0.0272189187569618,
                                         0.112190313138144,
                                         1.15082407582064e-05,
                                         3.47646437172403e-09,
                                         0.00975047842585033,
                                         0.165651322496429,
                                         0.000112174909227535,
                                         1.51309114931328e-10,
                                         0.00280156352347396,
                                         0.196179646158592,
                                         0.000877004873428898,
                                         5.21054922396162e-12,
                                         0.00063689248852718,
                                         0.183824795031413,
                                         0.00542499560580224,
                                         1.38846835389262e-13,
                                         0.000112038129632725,
                                         0.133287156965508,
                                         0.0259675421582203,
                                         2.76258234506672e-15,
                                         1.47160866696173e-05,
                                         0.0721604582014504,
                                         0.0928087799640951,
                                         3.8647999434336e-17,
                                         1.35909984688253e-06,
                                         0.0274689968598195,
                                         0.23322753468986,
                                         3.39249023647862e-19,
                                         7.87571863662611e-08,
                                         0.0065609408051961,
                                         0.36774805283355,
                                         1.40623083084565e-21,
                                         2.15513977291413e-09,
                                         0.000740007054308786,
                                         0.273821384962825,

                                         0.421123813002489,
                                         0.00889322229359907,
                                         3.814697265625e-06,
                                         3.44405921804984e-12,
                                         1.21001465364744e-24,
                                         0.373090828644255,
                                         0.0480223491522195,
                                         6.866455078125e-05,
                                         2.06648076613609e-10,
                                         4.42517924263642e-22,
                                         0.156086652748565,
                                         0.12245430975214,
                                         0.000583648681640625,
                                         5.85515700689207e-09,
                                         7.64218459846623e-20,
                                         0.0409729056475325,
                                         0.195922606759676,
                                         0.00311279296875,
                                         1.04093958741409e-07,
                                         8.28102286054832e-18,
                                         0.00756241140175404,
                                         0.220408107761039,
                                         0.0116729736328125,
                                         1.30120296762716e-06,
                                         6.30932622038817e-16,
                                         0.00104219887333216,
                                         0.185138757739368,
                                         0.032684326171875,
                                         1.21448268816953e-05,
                                         3.58929162807623e-14,
                                         0.000111141171338909,
                                         0.120337558281316,
                                         0.0708160400390625,
                                         8.77145586635027e-05,
                                         1.58004181162949e-12,
                                         9.37757279135199e-06,
                                         0.0618865323889923,
                                         0.12139892578125,
                                         0.000501237021594372,
                                         5.50325348480701e-11,
                                         6.34637032327131e-07,
                                         0.0255276357977578,
                                         0.166923522949219,
                                         0.00229738663881466,
                                         1.53741086913346e-09,
                                         3.47068476442387e-08,
                                         0.0085090256657629,
                                         0.185470581054688,
                                         0.0085090256657629,
                                         3.47068476442391e-08,
                                         1.53741086913344e-09,
                                         0.00229738663881466,
                                         0.166923522949219,
                                         0.0255276357977578,
                                         6.34637032327137e-07,
                                         5.50325348480694e-11,
                                         0.000501237021594372,
                                         0.12139892578125,
                                         0.0618865323889923,
                                         9.37757279135207e-06,
                                         1.58004181162947e-12,
                                         8.77145586635027e-05,
                                         0.0708160400390625,
                                         0.120337558281316,
                                         0.00011114117133891,
                                         3.58929162807618e-14,
                                         1.21448268816953e-05,
                                         0.032684326171875,
                                         0.185138757739368,
                                         0.00104219887333217,
                                         6.30932622038806e-16,
                                         1.30120296762716e-06,
                                         0.0116729736328125,
                                         0.220408107761039,
                                         0.00756241140175408,
                                         8.28102286054817e-18,
                                         1.04093958741409e-07,
                                         0.00311279296875,
                                         0.195922606759676,
                                         0.0409729056475326,
                                         7.64218459846608e-20,
                                         5.85515700689207e-09,
                                         0.000583648681640625,
                                         0.12245430975214,
                                         0.156086652748566,
                                         4.42517924263633e-22,
                                         2.06648076613609e-10,
                                         6.866455078125e-05,
                                         0.0480223491522195,
                                         0.373090828644256,
                                         1.21001465364741e-24,
                                         3.44405921804984e-12,
                                         3.814697265625e-06,
                                         0.00889322229359907,
                                         0.421123813002489,

                                         0.538874711459614,
                                         0.0354077926047428,
                                         0.000179614142927028,
                                         2.82029668212777e-08,
                                         1.31893904892295e-14,
                                         3.25430166842656e-27,
                                         0.338959281923353,
                                         0.12998054417882,
                                         0.00198736282276848,
                                         8.25853808073697e-07,
                                         1.1640998638391e-12,
                                         1.67626659901608e-24,
                                         0.100682427767107,
                                         0.225322348695281,
                                         0.0103838932051626,
                                         1.14197862613592e-05,
                                         4.8517919677869e-11,
                                         4.07731997296362e-22,
                                         0.0187646137638138,
                                         0.245081195677473,
                                         0.0340426270328669,
                                         9.90815010885169e-05,
                                         1.26880215072494e-09,
                                         6.22280654783153e-20,
                                         0.00245899768524243,
                                         0.187433947201962,
                                         0.0784726755690148,
                                         0.000604449077867608,
                                         2.33301849079599e-08,
                                         6.67775290924393e-18,
                                         0.000240604293989814,
                                         0.107031909982828,
                                         0.13506431415367,
                                         0.00275330057086123,
                                         3.20309057290529e-07,
                                         5.35058359954422e-16,
                                         1.82172449639147e-05,
                                         0.0472946886066311,
                                         0.179885784880357,
                                         0.00970468851837864,
                                         3.40294028558668e-06,
                                         3.31746258436669e-14,
                                         1.09132254417903e-06,
                                         0.0165349379456047,
                                         0.189558898918317,
                                         0.0270645419272342,
                                         2.86042484304688e-05,
                                         1.62742887363726e-12,
                                         5.24376291130236e-08,
                                         0.0046367348380861,
                                         0.160217887589803,
                                         0.0605395590304061,
                                         0.000192852932224367,
                                         6.40350116044042e-11,
                                         2.03604681989143e-09,
                                         0.00105069518989283,
                                         0.10942904721624,
                                         0.10942904721624,
                                         0.00105069518989283,
                                         2.03604681989143e-09,
                                         6.40350116044042e-11,
                                         0.000192852932224368,
                                         0.0605395590304061,
                                         0.160217887589803,
                                         0.00463673483808608,
                                         5.24376291130236e-08,
                                         1.62742887363726e-12,
                                         2.86042484304689e-05,
                                         0.0270645419272342,
                                         0.189558898918317,
                                         0.0165349379456046,
                                         1.09132254417903e-06,
                                         3.31746258436669e-14,
                                         3.40294028558669e-06,
                                         0.00970468851837864,
                                         0.179885784880357,
                                         0.047294688606631,
                                         1.82172449639147e-05,
                                         5.35058359954422e-16,
                                         3.2030905729053e-07,
                                         0.00275330057086123,
                                         0.13506431415367,
                                         0.107031909982828,
                                         0.000240604293989814,
                                         6.67775290924393e-18,
                                         2.333018490796e-08,
                                         0.000604449077867608,
                                         0.0784726755690148,
                                         0.187433947201962,
                                         0.00245899768524243,
                                         6.22280654783153e-20,
                                         1.26880215072495e-09,
                                         9.90815010885169e-05,
                                         0.0340426270328669,
                                         0.245081195677473,
                                         0.0187646137638138,
                                         4.07731997296362e-22,
                                         4.85179196778693e-11,
                                         1.14197862613592e-05,
                                         0.0103838932051626,
                                         0.22532234869528,
                                         0.100682427767107,
                                         1.67626659901608e-24,
                                         1.16409986383911e-12,
                                         8.25853808073697e-07,
                                         0.00198736282276848,
                                         0.12998054417882,
                                         0.338959281923353,
                                         3.25430166842656e-27,
                                         1.31893904892296e-14,
                                         2.82029668212777e-08,
                                         0.000179614142927028,
                                         0.0354077926047428,
                                         0.538874711459614,

                                         0.628790539573374,
                                         0.0828366795160203,
                                         0.00175523330591067,
                                         3.814697265625e-06,
                                         3.24828179375231e-10,
                                         1.01112952253666e-16,
                                         2.00062124253758e-29,
                                         0.295524088223461,
                                         0.221295244878635,
                                         0.0133527129973602,
                                         6.866455078125e-05,
                                         1.38345183871732e-08,
                                         1.22631601822856e-14,
                                         1.3791865044815e-26,
                                         0.0655882727238273,
                                         0.279169457554666,
                                         0.0479678881247581,
                                         0.000583648681640625,
                                         2.78240799433621e-07,
                                         7.02335194666753e-13,
                                         4.48980565426638e-24,
                                         0.0091335435487812,
                                         0.220975194878845,
                                         0.108121356152064,
                                         0.00311279296875,
                                         3.5112146692034e-06,
                                         2.52386583574018e-11,
                                         9.17089853291205e-22,
                                         0.000894303547719761,
                                         0.122984870522043,
                                         0.171358207377408,
                                         0.0116729736328125,
                                         3.11549093404786e-05,
                                         6.37706197467679e-10,
                                         1.31713040155033e-19,
                                         6.53818744014478e-05,
                                         0.0511077326782932,
                                         0.202779991140185,
                                         0.032684326171875,
                                         0.000206406022521676,
                                         1.20310013090098e-08,
                                         1.41244785490228e-17,
                                         3.69882556767895e-06,
                                         0.0164344665350428,
                                         0.185686085066585,
                                         0.0708160400390625,
                                         0.00105816235909111,
                                         1.75637392521813e-07,
                                         1.1720608059921e-15,
                                         1.65562295442627e-07,
                                         0.00418134220776722,
                                         0.134531729073291,
                                         0.12139892578125,
                                         0.00429213483516328,
                                         2.02872551884961e-06,
                                         7.69518326505836e-14,
                                         5.94399614751201e-09,
                                         0.000853287346488229,
                                         0.0781789044730314,
                                         0.166923522949219,
                                         0.0139641304307431,
                                         1.87952927847423e-05,
                                         4.05235312777308e-12,
                                         1.72444914732466e-10,
                                         0.000140711531842751,
                                         0.0367121001177314,
                                         0.185470581054688,
                                         0.0367121001177314,
                                         0.000140711531842751,
                                         1.72444914732469e-10,
                                         4.05235312777299e-12,
                                         1.87952927847423e-05,
                                         0.0139641304307431,
                                         0.166923522949219,
                                         0.0781789044730314,
                                         0.000853287346488229,
                                         5.94399614751211e-09,
                                         7.69518326505818e-14,
                                         2.02872551884961e-06,
                                         0.00429213483516328,
                                         0.12139892578125,
                                         0.134531729073291,
                                         0.00418134220776722,
                                         1.6556229544263e-07,
                                         1.17206080599207e-15,
                                         1.75637392521813e-07,
                                         0.00105816235909111,
                                         0.0708160400390625,
                                         0.185686085066585,
                                         0.0164344665350428,
                                         3.698825567679e-06,
                                         1.41244785490224e-17,
                                         1.20310013090098e-08,
                                         0.000206406022521676,
                                         0.032684326171875,
                                         0.202779991140185,
                                         0.0511077326782932,
                                         6.53818744014485e-05,
                                         1.31713040155029e-19,
                                         6.37706197467679e-10,
                                         3.11549093404786e-05,
                                         0.0116729736328125,
                                         0.171358207377408,
                                         0.122984870522043,
                                         0.000894303547719769,
                                         9.17089853291175e-22,
                                         2.52386583574018e-11,
                                         3.5112146692034e-06,
                                         0.00311279296875,
                                         0.108121356152064,
                                         0.220975194878845,
                                         0.00913354354878126,
                                         4.48980565426623e-24,
                                         7.02335194666753e-13,
                                         2.78240799433621e-07,
                                         0.000583648681640625,
                                         0.0479678881247581,
                                         0.279169457554666,
                                         0.0655882727238276,
                                         1.37918650448145e-26,
                                         1.22631601822856e-14,
                                         1.38345183871732e-08,
                                         6.866455078125e-05,
                                         0.0133527129973602,
                                         0.221295244878635,
                                         0.295524088223462,
                                         2.0006212425375e-29,
                                         1.01112952253666e-16,
                                         3.24828179375231e-10,
                                         3.814697265625e-06,
                                         0.00175523330591067,
                                         0.0828366795160203,
                                         0.628790539573374,

                                         0.696988072424476,
                                         0.145169186460661,
                                         0.00763916646567665,
                                         7.90782563957312e-05,
                                         9.93795897141191e-08,
                                         5.66471433928215e-12,
                                         1.34654787618679e-18,
                                         2.29978001158108e-31,
                                         0.25414350490432,
                                         0.295725291061062,
                                         0.0427664884953149,
                                         0.000982146476827586,
                                         2.59252546852405e-06,
                                         3.27842615524987e-10,
                                         2.14167114083678e-16,
                                         2.04351408890707e-28,
                                         0.0437601804857631,
                                         0.284478148946731,
                                         0.113059648622949,
                                         0.00576025701925493,
                                         3.19370857382863e-05,
                                         8.95981862448967e-09,
                                         1.60853379763706e-14,
                                         8.57463017948779e-26,
                                         0.00472779891136625,
                                         0.171707458900837,
                                         0.187538946776069,
                                         0.0211976278237081,
                                         0.000246858075609478,
                                         1.53643019213408e-07,
                                         7.58031754001269e-13,
                                         2.25752704728447e-23,
                                         0.000359146325858768,
                                         0.0728722113527478,
                                         0.218729748574839,
                                         0.0548485566520438,
                                         0.00134162744493409,
                                         1.85250328683766e-06,
                                         2.51175425460884e-11,
                                         4.17910291417669e-21,
                                         2.03709190529633e-05,
                                         0.02309199739295,
                                         0.190480708788408,
                                         0.105966821534673,
                                         0.00544431559211537,
                                         1.66775367070641e-05,
                                         6.21432064511222e-10,
                                         5.77643557091458e-19,
                                         8.94095458252152e-07,
                                         0.00566232928429577,
                                         0.128359558220933,
                                         0.158419516273219,
                                         0.0170957721516706,
                                         0.000116182012163786,
                                         1.18971856954041e-08,
                                         6.17832673423211e-17,
                                         3.10490458492245e-08,
                                         0.00109855004784248,
                                         0.0684378776586852,
                                         0.187386613210864,
                                         0.0424742045311768,
                                         0.000640378859683462,
                                         1.80212921194192e-07,
                                         5.22844945570195e-15,
                                         8.64832736610732e-10,
                                         0.000170948016194603,
                                         0.0292674135758219,
                                         0.177782059571451,
                                         0.0846410961657314,
                                         0.0028310958378372,
                                         2.18951086622803e-06,
                                         3.5489056076583e-13,
                                         1.9465733579376e-11,
                                         2.14962683818254e-05,
                                         0.010114093135279,
                                         0.13629882022953,
                                         0.13629882022953,
                                         0.010114093135279,
                                         2.14962683818254e-05,
                                         1.9465733579376e-11,
                                         3.5489056076583e-13,
                                         2.18951086622803e-06,
                                         0.0028310958378372,
                                         0.0846410961657316,
                                         0.17778205957145,
                                         0.0292674135758219,
                                         0.000170948016194603,
                                         8.64832736610732e-10,
                                         5.22844945570195e-15,
                                         1.80212921194192e-07,
                                         0.000640378859683462,
                                         0.0424742045311769,
                                         0.187386613210864,
                                         0.0684378776586852,
                                         0.00109855004784248,
                                         3.10490458492245e-08,
                                         6.17832673423211e-17,
                                         1.18971856954041e-08,
                                         0.000116182012163786,
                                         0.0170957721516707,
                                         0.158419516273219,
                                         0.128359558220933,
                                         0.00566232928429577,
                                         8.94095458252152e-07,
                                         5.77643557091458e-19,
                                         6.21432064511222e-10,
                                         1.6677536707064e-05,
                                         0.00544431559211537,
                                         0.105966821534673,
                                         0.190480708788408,
                                         0.02309199739295,
                                         2.03709190529633e-05,
                                         4.17910291417669e-21,
                                         2.51175425460884e-11,
                                         1.85250328683766e-06,
                                         0.00134162744493409,
                                         0.0548485566520438,
                                         0.218729748574839,
                                         0.0728722113527478,
                                         0.000359146325858768,
                                         2.25752704728447e-23,
                                         7.58031754001269e-13,
                                         1.53643019213408e-07,
                                         0.000246858075609479,
                                         0.0211976278237081,
                                         0.187538946776069,
                                         0.171707458900837,
                                         0.00472779891136625,
                                         8.57463017948779e-26,
                                         1.60853379763706e-14,
                                         8.95981862448967e-09,
                                         3.19370857382864e-05,
                                         0.00576025701925493,
                                         0.113059648622949,
                                         0.284478148946731,
                                         0.0437601804857631,
                                         2.04351408890707e-28,
                                         2.14167114083678e-16,
                                         3.27842615524987e-10,
                                         2.59252546852405e-06,
                                         0.000982146476827586,
                                         0.0427664884953149,
                                         0.295725291061062,
                                         0.25414350490432,
                                         2.29978001158108e-31,
                                         1.34654787618679e-18,
                                         5.66471433928215e-12,
                                         9.93795897141193e-08,
                                         7.90782563957312e-05,
                                         0.00763916646567665,
                                         0.145169186460661,
                                         0.696988072424476,

                                         0.749114580534162,
                                         0.214437486932356,
                                         0.0209256332038754,
                                         0.000598328866488328,
                                         3.814697265625e-06,
                                         3.29283108968348e-09,
                                         1.4215135387418e-13,
                                         2.80005412216344e-20,
                                         4.31435957006534e-33,
                                         0.218137380403213,
                                         0.344710603493407,
                                         0.0902631927071421,
                                         0.00549572775809075,
                                         6.866455078125e-05,
                                         1.16152673052419e-07,
                                         1.06773610422034e-11,
                                         5.64361660069056e-18,
                                         4.80042296178579e-30,
                                         0.029995655516231,
                                         0.261670640348779,
                                         0.18386082053743,
                                         0.0238372903545573,
                                         0.000583648681640625,
                                         1.93479690783877e-06,
                                         3.78724391528566e-10,
                                         5.37149281646621e-16,
                                         2.52225558266958e-27,
                                         0.00258801282071079,
                                         0.124633604073205,
                                         0.234988981660237,
                                         0.0648736546152149,
                                         0.00311279296875,
                                         2.02218722622514e-05,
                                         8.42873553018711e-09,
                                         3.20783704535459e-14,
                                         8.31531070991209e-25,
                                         0.000157002666834255,
                                         0.0417395584316854,
                                         0.211172977907435,
                                         0.124140183185732,
                                         0.0116729736328125,
                                         0.000148607209964398,
                                         1.31896757007431e-07,
                                         1.34698307095322e-12,
                                         1.927526738422e-22,
                                         7.11171589756752e-06,
                                         0.0104372794220871,
                                         0.141695465711391,
                                         0.177371224045963,
                                         0.032684326171875,
                                         0.00081542713998126,
                                         1.54110621380745e-06,
                                         4.22317171335013e-11,
                                         3.33617715703737e-20,
                                         2.49273343551969e-07,
                                         0.00201957897598859,
                                         0.0735711648369483,
                                         0.196104705630985,
                                         0.0708160400390625,
                                         0.00346229855188349,
                                         1.39336563803603e-05,
                                         1.02458834966954e-09,
                                         4.46819242678733e-18,
                                         6.91302907137873e-09,
                                         0.000309190025971456,
                                         0.0302238938683769,
                                         0.171547333087078,
                                         0.12139892578125,
                                         0.011631479785825,
                                         9.9675569304874e-05,
                                         1.96675920120512e-08,
                                         4.73484466728493e-16,
                                         1.53773117681363e-10,
                                         3.79672902736159e-05,
                                         0.00995893025814117,
                                         0.120364777909938,
                                         0.166923522949219,
                                         0.0313418709857536,
                                         0.000571915472050229,
                                         3.02811566623613e-07,
                                         4.02438105583801e-14,
                                         2.76405861387085e-12,
                                         3.76745888886236e-06,
                                         0.00265173279003621,
                                         0.0682448147578701,
                                         0.185470581054688,
                                         0.0682448147578701,
                                         0.00265173279003621,
                                         3.76745888886236e-06,
                                         2.76405861387094e-12,
                                         4.02438105583787e-14,
                                         3.02811566623613e-07,
                                         0.000571915472050227,
                                         0.0313418709857536,
                                         0.166923522949219,
                                         0.120364777909938,
                                         0.00995893025814119,
                                         3.79672902736159e-05,
                                         1.53773117681368e-10,
                                         4.73484466728475e-16,
                                         1.96675920120512e-08,
                                         9.96755693048736e-05,
                                         0.011631479785825,
                                         0.12139892578125,
                                         0.171547333087078,
                                         0.0302238938683769,
                                         0.000309190025971456,
                                         6.9130290713789e-09,
                                         4.46819242678714e-18,
                                         1.02458834966954e-09,
                                         1.39336563803602e-05,
                                         0.00346229855188349,
                                         0.0708160400390625,
                                         0.196104705630985,
                                         0.0735711648369484,
                                         0.00201957897598859,
                                         2.49273343551974e-07,
                                         3.33617715703722e-20,
                                         4.22317171335013e-11,
                                         1.54110621380744e-06,
                                         0.00081542713998126,
                                         0.032684326171875,
                                         0.177371224045963,
                                         0.141695465711391,
                                         0.0104372794220871,
                                         7.11171589756764e-06,
                                         1.9275267384219e-22,
                                         1.34698307095322e-12,
                                         1.3189675700743e-07,
                                         0.000148607209964398,
                                         0.0116729736328125,
                                         0.124140183185732,
                                         0.211172977907435,
                                         0.0417395584316854,
                                         0.000157002666834257,
                                         8.31531070991166e-25,
                                         3.20783704535459e-14,
                                         8.42873553018707e-09,
                                         2.02218722622514e-05,
                                         0.00311279296875,
                                         0.0648736546152149,
                                         0.234988981660237,
                                         0.124633604073205,
                                         0.00258801282071082,
                                         2.52225558266944e-27,
                                         5.37149281646621e-16,
                                         3.78724391528564e-10,
                                         1.93479690783877e-06,
                                         0.000583648681640625,
                                         0.0238372903545573,
                                         0.18386082053743,
                                         0.261670640348779,
                                         0.0299956555162312,
                                         4.8004229617855e-30,
                                         5.64361660069056e-18,
                                         1.06773610422034e-11,
                                         1.16152673052419e-07,
                                         6.866455078125e-05,
                                         0.00549572775809075,
                                         0.0902631927071422,
                                         0.344710603493407,
                                         0.218137380403213,
                                         4.31435957006507e-33,
                                         2.80005412216344e-20,
                                         1.42151353874179e-13,
                                         3.29283108968348e-09,
                                         3.814697265625e-06,
                                         0.000598328866488328,
                                         0.0209256332038754,
                                         0.214437486932356,
                                         0.749114580534162,

                                         0.789475359836136,
                                         0.284409913871415,
                                         0.0430803572316397,
                                         0.00248913333767252,
                                         4.63837229829442e-05,
                                         2.09578341207436e-07,
                                         1.38198041266909e-10,
                                         4.88168881547764e-15,
                                         8.39069128188711e-22,
                                         1.19959207214173e-34,
                                         0.187852233544,
                                         0.370385105934429,
                                         0.148028634885544,
                                         0.0177106606812784,
                                         0.000618527857927624,
                                         5.09210966893705e-06,
                                         6.29303716911091e-09,
                                         4.60308149335941e-13,
                                         2.08753597779146e-19,
                                         1.63343107634004e-31,
                                         0.0211076831852364,
                                         0.227776396762606,
                                         0.240192030158608,
                                         0.0595069627173878,
                                         0.00389492720799242,
                                         5.84245692285196e-05,
                                         1.35320977624449e-07,
                                         2.04962144478336e-11,
                                         2.45254292018145e-17,
                                         1.05030265039799e-28,
                                         0.00148814268235564,
                                         0.0878908621932033,
                                         0.244540757952765,
                                         0.125452871952923,
                                         0.0153893128582863,
                                         0.000420603702771003,
                                         1.82578552117827e-06,
                                         5.72635741288855e-10,
                                         1.80791940371791e-15,
                                         4.23748243674176e-26,
                                         7.37702268637746e-05,
                                         0.0238457656217752,
                                         0.175055779960733,
                                         0.185962755006498,
                                         0.0427534978196717,
                                         0.00212903697623256,
                                         1.7320759460887e-05,
                                         1.12490484116449e-08,
                                         9.37074360496852e-14,
                                         1.20208146487896e-23,
                                         2.7305141316461e-06,
                                         0.00483065009465435,
                                         0.0935682310976542,
                                         0.205824988016717,
                                         0.0886851364002416,
                                         0.00804674206188075,
                                         0.000122690480125067,
                                         1.64998457447131e-07,
                                         3.62656785816951e-12,
                                         2.54616564457805e-21,
                                         7.82063124382224e-08,
                                         0.000757240722356253,
                                         0.0387003109282416,
                                         0.176280509316751,
                                         0.142352171184779,
                                         0.0235337441107286,
                                         0.000672494709362811,
                                         1.87274244138326e-06,
                                         1.08605453056646e-10,
                                         4.1732411172747e-19,
                                         1.77227166005406e-09,
                                         9.39189969220967e-05,
                                         0.0126646033786198,
                                         0.119454247980342,
                                         0.180787523955629,
                                         0.0544569203832811,
                                         0.00291647314745466,
                                         1.6817728063266e-05,
                                         2.5733482274294e-09,
                                         5.41192046571229e-17,
                                         3.22135263807319e-11,
                                         9.3431306560813e-06,
                                         0.00332420826285114,
                                         0.0649259619075352,
                                         0.184158729517476,
                                         0.101072875312568,
                                         0.0101448722942838,
                                         0.000121136798298648,
                                         4.89063193037375e-08,
                                         5.62922782271751e-15,
                                         4.73152389737072e-13,
                                         7.510799888053e-07,
                                         0.000705082033437364,
                                         0.0285160901544746,
                                         0.151590140670313,
                                         0.151590140670313,
                                         0.0285160901544746,
                                         0.000705082033437364,
                                         7.51079988805295e-07,
                                         4.73152389737071e-13,
                                         5.62922782271751e-15,
                                         4.89063193037379e-08,
                                         0.000121136798298648,
                                         0.0101448722942838,
                                         0.101072875312568,
                                         0.184158729517476,
                                         0.0649259619075352,
                                         0.00332420826285114,
                                         9.34313065608124e-06,
                                         3.22135263807319e-11,
                                         5.41192046571229e-17,
                                         2.57334822742942e-09,
                                         1.6817728063266e-05,
                                         0.00291647314745466,
                                         0.0544569203832811,
                                         0.180787523955629,
                                         0.119454247980342,
                                         0.0126646033786198,
                                         9.39189969220961e-05,
                                         1.77227166005406e-09,
                                         4.1732411172747e-19,
                                         1.08605453056647e-10,
                                         1.87274244138326e-06,
                                         0.000672494709362811,
                                         0.0235337441107286,
                                         0.142352171184779,
                                         0.176280509316751,
                                         0.0387003109282416,
                                         0.000757240722356249,
                                         7.82063124382224e-08,
                                         2.54616564457805e-21,
                                         3.62656785816955e-12,
                                         1.64998457447131e-07,
                                         0.000122690480125067,
                                         0.00804674206188075,
                                         0.0886851364002416,
                                         0.205824988016717,
                                         0.0935682310976542,
                                         0.00483065009465433,
                                         2.7305141316461e-06,
                                         1.20208146487896e-23,
                                         9.37074360496863e-14,
                                         1.12490484116449e-08,
                                         1.7320759460887e-05,
                                         0.00212903697623256,
                                         0.0427534978196717,
                                         0.185962755006498,
                                         0.175055779960733,
                                         0.0238457656217752,
                                         7.37702268637746e-05,
                                         4.23748243674176e-26,
                                         1.80791940371793e-15,
                                         5.72635741288855e-10,
                                         1.82578552117827e-06,
                                         0.000420603702771003,
                                         0.0153893128582863,
                                         0.125452871952923,
                                         0.244540757952765,
                                         0.087890862193203,
                                         0.00148814268235564,
                                         1.05030265039799e-28,
                                         2.45254292018149e-17,
                                         2.04962144478336e-11,
                                         1.35320977624449e-07,
                                         5.84245692285196e-05,
                                         0.00389492720799242,
                                         0.0595069627173878,
                                         0.240192030158607,
                                         0.227776396762605,
                                         0.0211076831852364,
                                         1.63343107634004e-31,
                                         2.08753597779149e-19,
                                         4.60308149335941e-13,
                                         6.29303716911091e-09,
                                         5.09210966893705e-06,
                                         0.000618527857927624,
                                         0.0177106606812784,
                                         0.148028634885544,
                                         0.370385105934428,
                                         0.187852233544,
                                         1.19959207214173e-34,
                                         8.39069128188724e-22,
                                         4.88168881547764e-15,
                                         1.38198041266909e-10,
                                         2.09578341207436e-07,
                                         4.63837229829442e-05,
                                         0.00248913333767252,
                                         0.0430803572316397,
                                         0.284409913871415,
                                         0.789475359836136,

                                         0.821177198347341,
                                         0.351247797461716,
                                         0.0736167715050961,
                                         0.00707937154909507,
                                         0.000279971943533948,
                                         3.814697265625e-06,
                                         1.3370948141409e-08,
                                         7.21953102480451e-12,
                                         2.19586472158117e-16,
                                         3.40831896084491e-23,
                                         4.60673920475697e-36,
                                         0.162673980996456,
                                         0.378388207986799,
                                         0.206673220323844,
                                         0.0403404024769461,
                                         0.00289957342849139,
                                         6.866455078125e-05,
                                         4.18299760098377e-07,
                                         4.10495571796589e-10,
                                         2.53421322184401e-14,
                                         1.02508825301373e-20,
                                         7.53455181488889e-33,
                                         0.0152175847365622,
                                         0.192489922801907,
                                         0.273992116024061,
                                         0.108550584972202,
                                         0.0141807803765499,
                                         0.000583648681640625,
                                         6.1795869477067e-06,
                                         1.10218472180851e-08,
                                         1.38110641993284e-12,
                                         1.45589047099452e-18,
                                         5.81925839138392e-30,
                                         0.000893209152883355,
                                         0.0614409900473794,
                                         0.227914380576702,
                                         0.183275288460747,
                                         0.0435156978851642,
                                         0.00311279296875,
                                         5.72810646196033e-05,
                                         1.85686401102281e-07,
                                         4.72270629730638e-11,
                                         1.29740615079675e-16,
                                         2.82005553814041e-27,
                                         3.68632091664753e-05,
                                         0.0137892597292708,
                                         0.13330238824677,
                                         0.217574600646402,
                                         0.0938910756834921,
                                         0.0116729736328125,
                                         0.000373332032614598,
                                         2.19957285352767e-06,
                                         1.13550034752955e-09,
                                         8.1293484892416e-15,
                                         9.60904486366524e-25,
                                         1.13595173683151e-06,
                                         0.00231073670072505,
                                         0.0582144630143102,
                                         0.192858703113776,
                                         0.151261861259341,
                                         0.032684326171875,
                                         0.00181679611347938,
                                         1.94546475289025e-05,
                                         2.03849837083472e-08,
                                         3.80331497772046e-13,
                                         2.44472235803686e-22,
                                         2.7086986411968e-08,
                                         0.000299636036029514,
                                         0.0196724265194627,
                                         0.132283098677891,
                                         0.188568278369664,
                                         0.0708160400390625,
                                         0.0068414979327684,
                                         0.000133150393167427,
                                         2.83183233571337e-07,
                                         1.37690163832851e-11,
                                         4.81296760133927e-20,
                                         5.11037304760803e-10,
                                         3.07417600204606e-05,
                                         0.0052598828555439,
                                         0.0717894425751316,
                                         0.185994000655742,
                                         0.12139892578125,
                                         0.0203839015020772,
                                         0.000721028852354528,
                                         3.11254621413702e-06,
                                         3.94397945346786e-10,
                                         7.4969987113509e-18,
                                         7.73328489977321e-12,
                                         2.52978087561832e-06,
                                         0.0011280119596445,
                                         0.0312490122974819,
                                         0.147146090606666,
                                         0.166923522949219,
                                         0.0487127801250168,
                                         0.00313171567567335,
                                         2.74399662248542e-05,
                                         9.06120306046934e-09,
                                         9.36658818529113e-16,
                                         9.45649525136764e-14,
                                         1.68225518140578e-07,
                                         0.000195481709775207,
                                         0.0109917489627859,
                                         0.0940704697631235,
                                         0.185470581054688,
                                         0.0940704697631236,
                                         0.0109917489627859,
                                         0.000195481709775206,
                                         1.68225518140578e-07,
                                         9.45649525136807e-14,
                                         9.36658818529065e-16,
                                         9.06120306046934e-09,
                                         2.74399662248543e-05,
                                         0.00313171567567335,
                                         0.0487127801250167,
                                         0.166923522949219,
                                         0.147146090606667,
                                         0.0312490122974819,
                                         0.0011280119596445,
                                         2.52978087561832e-06,
                                         7.73328489977352e-12,
                                         7.49699871135048e-18,
                                         3.94397945346786e-10,
                                         3.11254621413703e-06,
                                         0.00072102885235453,
                                         0.0203839015020771,
                                         0.12139892578125,
                                         0.185994000655742,
                                         0.0717894425751315,
                                         0.00525988285554389,
                                         3.07417600204606e-05,
                                         5.11037304760821e-10,
                                         4.81296760133898e-20,
                                         1.37690163832851e-11,
                                         2.83183233571339e-07,
                                         0.000133150393167427,
                                         0.00684149793276839,
                                         0.0708160400390625,
                                         0.188568278369664,
                                         0.132283098677891,
                                         0.0196724265194627,
                                         0.000299636036029514,
                                         2.70869864119688e-08,
                                         2.4447223580367e-22,
                                         3.80331497772046e-13,
                                         2.03849837083473e-08,
                                         1.94546475289025e-05,
                                         0.00181679611347938,
                                         0.032684326171875,
                                         0.151261861259341,
                                         0.192858703113776,
                                         0.0582144630143101,
                                         0.00231073670072505,
                                         1.13595173683154e-06,
                                         9.60904486366456e-25,
                                         8.1293484892416e-15,
                                         1.13550034752955e-09,
                                         2.19957285352767e-06,
                                         0.000373332032614597,
                                         0.0116729736328125,
                                         0.0938910756834922,
                                         0.217574600646402,
                                         0.13330238824677,
                                         0.0137892597292707,
                                         3.68632091664761e-05,
                                         2.8200555381402e-27,
                                         1.29740615079675e-16,
                                         4.72270629730641e-11,
                                         1.85686401102282e-07,
                                         5.72810646196032e-05,
                                         0.00311279296875,
                                         0.0435156978851643,
                                         0.183275288460747,
                                         0.227914380576702,
                                         0.0614409900473794,
                                         0.000893209152883368,
                                         5.81925839138345e-30,
                                         1.45589047099452e-18,
                                         1.38110641993285e-12,
                                         1.10218472180851e-08,
                                         6.17958694770669e-06,
                                         0.000583648681640625,
                                         0.0141807803765499,
                                         0.108550584972202,
                                         0.273992116024061,
                                         0.192489922801907,
                                         0.0152175847365623,
                                         7.53455181488824e-33,
                                         1.02508825301373e-20,
                                         2.53421322184403e-14,
                                         4.1049557179659e-10,
                                         4.18299760098376e-07,
                                         6.866455078125e-05,
                                         0.00289957342849139,
                                         0.0403404024769461,
                                         0.206673220323844,
                                         0.378388207986799,
                                         0.162673980996457,
                                         4.60673920475655e-36,
                                         3.40831896084491e-23,
                                         2.19586472158118e-16,
                                         7.21953102480454e-12,
                                         1.3370948141409e-08,
                                         3.814697265625e-06,
                                         0.000279971943533948,
                                         0.00707937154909507,
                                         0.0736167715050961,
                                         0.351247797461716,
                                         0.821177198347341,

                                         0.846433152606863,
                                         0.412996602082287,
                                         0.110802361390833,
                                         0.0156102836014221,
                                         0.00107163499990649,
                                         3.19026825434763e-05,
                                         3.43178023879325e-07,
                                         9.91862390431609e-10,
                                         4.59769780794001e-13,
                                         1.24700547952168e-17,
                                         1.7897329151394e-24,
                                         2.31679547906586e-37,
                                         0.141776347341483,
                                         0.374339578787522,
                                         0.259288769700667,
                                         0.0730524767021432,
                                         0.00891496507877376,
                                         0.000446425792037825,
                                         7.94588735244179e-06,
                                         3.86299082076559e-08,
                                         3.18318062016996e-11,
                                         1.72654809671601e-15,
                                         6.397554093906e-22,
                                         4.4814813072307e-34,
                                         0.0112140198832193,
                                         0.16022542667321,
                                         0.286526515729017,
                                         0.161437906193822,
                                         0.0350218290578243,
                                         0.00294997118530798,
                                         8.68783890102466e-05,
                                         7.10464447235007e-07,
                                         1.04070720600137e-09,
                                         1.12884787790656e-13,
                                         1.07990643409854e-19,
                                         4.09356699210205e-31,
                                         0.000556542920410076,
                                         0.0430305531020289,
                                         0.198666999424497,
                                         0.223849377133581,
                                         0.0863252418407471,
                                         0.0122311156387246,
                                         0.000596020114733379,
                                         8.19862129210874e-06,
                                         2.13489069098165e-08,
                                         4.63096995063746e-12,
                                         1.1437683979939e-17,
                                         2.34618372127986e-28,
                                         1.94208638310386e-05,
                                         0.00812559102685535,
                                         0.0968543623046128,
                                         0.218242236781423,
                                         0.149612997214169,
                                         0.0356571731799948,
                                         0.00287503154456443,
                                         6.65230023755439e-05,
                                         3.07932313699572e-07,
                                         1.33579878716215e-10,
                                         8.5177050431762e-16,
                                         9.45485062705745e-26,
                                         5.06017093124024e-07,
                                         0.00114567056427186,
                                         0.0352565167761818,
                                         0.158872409261066,
                                         0.19360992044409,
                                         0.0776165816840789,
                                         0.0103550231714923,
                                         0.000403022805085041,
                                         3.31635986321417e-06,
                                         2.87698031450602e-09,
                                         4.73624178049656e-14,
                                         2.84494641283386e-23,
                                         1.02022486091245e-08,
                                         0.000124996721026144,
                                         0.00993101621670692,
                                         0.0894936517386416,
                                         0.193874173900592,
                                         0.130736298377371,
                                         0.0288598192390199,
                                         0.00188938938684123,
                                         2.76377132264631e-05,
                                         4.79475925730157e-08,
                                         2.03788279520197e-12,
                                         6.62411066784561e-21,
                                         1.6274877230598e-10,
                                         1.07901775330726e-05,
                                         0.00221329387911519,
                                         0.0398866058612997,
                                         0.153604315804106,
                                         0.174232416772249,
                                         0.0636395708373329,
                                         0.00700816673248735,
                                         0.000182235786781235,
                                         6.32248527399139e-07,
                                         6.93770206027775e-11,
                                         1.22031562677283e-18,
                                         2.08237543633572e-12,
                                         7.47098822909427e-07,
                                         0.000395643437952164,
                                         0.0142587490622528,
                                         0.097612709564787,
                                         0.186243544909407,
                                         0.112559037314758,
                                         0.0208500390143405,
                                         0.000963795109345313,
                                         6.68695404775428e-06,
                                         1.89439953996457e-09,
                                         1.80316827575319e-16,
                                         2.15305505741433e-14,
                                         4.18005798652507e-08,
                                         5.71509749426105e-05,
                                         0.00411898834091206,
                                         0.0501261224463629,
                                         0.160874900101999,
                                         0.160874900101999,
                                         0.050126122446363,
                                         0.00411898834091207,
                                         5.71509749426108e-05,
                                         4.18005798652512e-08,
                                         2.15305505741421e-14,
                                         1.8031682757533e-16,
                                         1.89439953996455e-09,
                                         6.68695404775424e-06,
                                         0.000963795109345311,
                                         0.0208500390143405,
                                         0.112559037314758,
                                         0.186243544909407,
                                         0.0976127095647872,
                                         0.0142587490622528,
                                         0.000395643437952165,
                                         7.47098822909434e-07,
                                         2.08237543633562e-12,
                                         1.22031562677291e-18,
                                         6.93770206027767e-11,
                                         6.32248527399135e-07,
                                         0.000182235786781234,
                                         0.00700816673248733,
                                         0.0636395708373329,
                                         0.174232416772249,
                                         0.153604315804107,
                                         0.0398866058612998,
                                         0.0022132938791152,
                                         1.07901775330727e-05,
                                         1.62748772305973e-10,
                                         6.62411066784608e-21,
                                         2.03788279520194e-12,
                                         4.79475925730154e-08,
                                         2.7637713226463e-05,
                                         0.00188938938684122,
                                         0.0288598192390199,
                                         0.130736298377371,
                                         0.193874173900593,
                                         0.0894936517386417,
                                         0.00993101621670695,
                                         0.000124996721026145,
                                         1.02022486091241e-08,
                                         2.84494641283408e-23,
                                         4.73624178049649e-14,
                                         2.876980314506e-09,
                                         3.31635986321416e-06,
                                         0.00040302280508504,
                                         0.0103550231714923,
                                         0.0776165816840789,
                                         0.19360992044409,
                                         0.158872409261066,
                                         0.0352565167761818,
                                         0.00114567056427187,
                                         5.06017093124008e-07,
                                         9.45485062705825e-26,
                                         8.51770504317606e-16,
                                         1.33579878716214e-10,
                                         3.07932313699571e-07,
                                         6.65230023755437e-05,
                                         0.00287503154456443,
                                         0.0356571731799948,
                                         0.149612997214169,
                                         0.218242236781423,
                                         0.096854362304613,
                                         0.00812559102685539,
                                         1.94208638310381e-05,
                                         2.34618372128007e-28,
                                         1.14376839799388e-17,
                                         4.63096995063742e-12,
                                         2.13489069098164e-08,
                                         8.19862129210872e-06,
                                         0.000596020114733379,
                                         0.0122311156387246,
                                         0.0863252418407471,
                                         0.223849377133581,
                                         0.198666999424497,
                                         0.043030553102029,
                                         0.000556542920410066,
                                         4.09356699210245e-31,
                                         1.07990643409852e-19,
                                         1.12884787790656e-13,
                                         1.04070720600136e-09,
                                         7.10464447235005e-07,
                                         8.68783890102466e-05,
                                         0.00294997118530798,
                                         0.0350218290578243,
                                         0.161437906193822,
                                         0.286526515729018,
                                         0.16022542667321,
                                         0.0112140198832192,
                                         4.48148130723116e-34,
                                         6.39755409390588e-22,
                                         1.726548096716e-15,
                                         3.18318062016994e-11,
                                         3.86299082076558e-08,
                                         7.94588735244179e-06,
                                         0.000446425792037825,
                                         0.00891496507877376,
                                         0.0730524767021433,
                                         0.259288769700668,
                                         0.374339578787522,
                                         0.141776347341482,
                                         2.31679547906611e-37,
                                         1.78973291513936e-24,
                                         1.24700547952167e-17,
                                         4.59769780793999e-13,
                                         9.91862390431606e-10,
                                         3.43178023879325e-07,
                                         3.19026825434763e-05,
                                         0.00107163499990649,
                                         0.0156102836014221,
                                         0.110802361390833,
                                         0.412996602082287,
                                         0.846433152606863,

                                         0.86682480891169,
                                         0.468919108635853,
                                         0.152480998886578,
                                         0.0288296785956552,
                                         0.00300575756120015,
                                         0.000159464634488794,
                                         3.814697265625e-06,
                                         3.41695112717162e-08,
                                         8.49927987788266e-11,
                                         3.49721751675589e-14,
                                         8.67108241124878e-19,
                                         1.17031225576999e-25,
                                         1.46442487550096e-38,
                                         0.124378327742185,
                                         0.362700726929757,
                                         0.302290685854051,
                                         0.113007117441593,
                                         0.0205997216651375,
                                         0.0017951557334531,
                                         6.866455078125e-05,
                                         9.83435833335813e-07,
                                         4.01809070823688e-09,
                                         2.89069092313817e-12,
                                         1.41712867671757e-16,
                                         4.90225917551163e-23,
                                         3.30672993307114e-35,
                                         0.00842761165357841,
                                         0.132478481610097,
                                         0.282995952530933,
                                         0.209179063465855,
                                         0.0666676546658719,
                                         0.00954303023582038,
                                         0.000583648681640625,
                                         1.33659462607604e-05,
                                         8.97023381578753e-08,
                                         1.12830650797797e-10,
                                         1.09368354905247e-14,
                                         9.69699571374693e-21,
                                         3.52595509154666e-32,
                                         0.00035829778120694,
                                         0.0303614186233633,
                                         0.166232327105553,
                                         0.24294638069895,
                                         0.135378221417423,
                                         0.0318309991795807,
                                         0.00311279296875,
                                         0.000113981192134812,
                                         1.25651475362604e-06,
                                         2.76332731336457e-09,
                                         5.2960719306324e-13,
                                         1.20353282995208e-18,
                                         2.35903632713623e-29,
                                         1.0710660716871e-05,
                                         0.00489250544608332,
                                         0.06865675687843,
                                         0.19839704193316,
                                         0.19329248561154,
                                         0.0746529171159009,
                                         0.0116729736328125,
                                         0.000683438152048413,
                                         1.23755343806774e-05,
                                         4.7585003554796e-08,
                                         1.80321956237787e-11,
                                         1.05029480906576e-16,
                                         1.10974965989564e-26,
                                         2.39064543684215e-07,
                                         0.000588663813052087,
                                         0.0211727782387326,
                                         0.120972516571316,
                                         0.206066740302481,
                                         0.130728433683336,
                                         0.032684326171875,
                                         0.00305979271842,
                                         9.10095764798094e-05,
                                         6.1183546339028e-07,
                                         4.58426916232668e-10,
                                         6.84370467519768e-15,
                                         3.89800306361981e-24,
                                         4.1290310356721e-09,
                                         5.48071767747344e-05,
                                         0.00505250187366695,
                                         0.0570784680710813,
                                         0.16999451240554,
                                         0.177144390043085,
                                         0.0708160400390625,
                                         0.0106003172962128,
                                         0.000517898074382327,
                                         6.08741985752536e-06,
                                         9.01832076744369e-09,
                                         3.45068544543079e-13,
                                         1.05948155970362e-21,
                                         5.64250859252701e-11,
                                         4.03737070476487e-06,
                                         0.000953951338713074,
                                         0.0213083099774398,
                                         0.110956565979474,
                                         0.189922252729807,
                                         0.12139892578125,
                                         0.0290560669462304,
                                         0.00233180700840158,
                                         4.79206799151434e-05,
                                         1.40369370383198e-07,
                                         1.37660894739566e-11,
                                         2.27843022077045e-19,
                                         6.1846596254201e-13,
                                         2.38549942242503e-07,
                                         0.000144465934923551,
                                         0.00638036017323677,
                                         0.0580885591724541,
                                         0.163321663500263,
                                         0.166923522949219,
                                         0.0638813789080907,
                                         0.00842093669284612,
                                         0.000302574398188981,
                                         1.75242127715188e-06,
                                         4.40489231684906e-10,
                                         3.93004528566965e-17,
                                         5.47790087807989e-15,
                                         1.13897658620013e-08,
                                         1.76790724501001e-05,
                                         0.00154381827420217,
                                         0.0245744040122118,
                                         0.113492334379523,
                                         0.185470581054688,
                                         0.113492334379523,
                                         0.0245744040122118,
                                         0.00154381827420217,
                                         1.76790724501002e-05,
                                         1.13897658620012e-08,
                                         5.47790087807954e-15,
                                         3.93004528566992e-17,
                                         4.40489231684912e-10,
                                         1.75242127715187e-06,
                                         0.000302574398188982,
                                         0.00842093669284612,
                                         0.0638813789080907,
                                         0.166923522949219,
                                         0.163321663500263,
                                         0.0580885591724541,
                                         0.00638036017323675,
                                         0.000144465934923552,
                                         2.38549942242501e-07,
                                         6.18465962541976e-13,
                                         2.27843022077062e-19,
                                         1.37660894739568e-11,
                                         1.40369370383197e-07,
                                         4.79206799151436e-05,
                                         0.00233180700840158,
                                         0.0290560669462304,
                                         0.12139892578125,
                                         0.189922252729807,
                                         0.110956565979474,
                                         0.0213083099774398,
                                         0.000953951338713077,
                                         4.03737070476483e-06,
                                         5.64250859252673e-11,
                                         1.05948155970371e-21,
                                         3.45068544543084e-13,
                                         9.01832076744363e-09,
                                         6.08741985752538e-06,
                                         0.000517898074382327,
                                         0.0106003172962128,
                                         0.0708160400390625,
                                         0.177144390043085,
                                         0.16999451240554,
                                         0.0570784680710812,
                                         0.00505250187366696,
                                         5.4807176774734e-05,
                                         4.12903103567192e-09,
                                         3.89800306362017e-24,
                                         6.8437046751978e-15,
                                         4.58426916232664e-10,
                                         6.11835463390283e-07,
                                         9.10095764798094e-05,
                                         0.00305979271842,
                                         0.032684326171875,
                                         0.130728433683336,
                                         0.206066740302481,
                                         0.120972516571316,
                                         0.0211727782387327,
                                         0.000588663813052084,
                                         2.39064543684206e-07,
                                         1.10974965989575e-26,
                                         1.05029480906578e-16,
                                         1.80321956237785e-11,
                                         4.75850035547962e-08,
                                         1.23755343806774e-05,
                                         0.000683438152048413,
                                         0.0116729736328125,
                                         0.0746529171159009,
                                         0.19329248561154,
                                         0.19839704193316,
                                         0.0686567568784302,
                                         0.00489250544608329,
                                         1.07106607168707e-05,
                                         2.35903632713648e-29,
                                         1.2035328299521e-18,
                                         5.29607193063236e-13,
                                         2.76332731336458e-09,
                                         1.25651475362604e-06,
                                         0.000113981192134812,
                                         0.00311279296875,
                                         0.0318309991795807,
                                         0.135378221417423,
                                         0.242946380698949,
                                         0.166232327105553,
                                         0.0303614186233632,
                                         0.000358297781206932,
                                         3.52595509154706e-32,
                                         9.69699571374714e-21,
                                         1.09368354905246e-14,
                                         1.12830650797797e-10,
                                         8.97023381578753e-08,
                                         1.33659462607604e-05,
                                         0.000583648681640625,
                                         0.00954303023582038,
                                         0.0666676546658719,
                                         0.209179063465855,
                                         0.282995952530933,
                                         0.132478481610097,
                                         0.00842761165357829,
                                         3.30672993307153e-35,
                                         4.90225917551174e-23,
                                         1.41712867671755e-16,
                                         2.89069092313818e-12,
                                         4.01809070823688e-09,
                                         9.83435833335813e-07,
                                         6.866455078125e-05,
                                         0.0017951557334531,
                                         0.0205997216651375,
                                         0.113007117441593,
                                         0.302290685854051,
                                         0.362700726929757,
                                         0.124378327742184,
                                         1.46442487550115e-38,
                                         1.17031225577002e-25,
                                         8.67108241124869e-19,
                                         3.49721751675591e-14,
                                         8.49927987788266e-11,
                                         3.41695112717162e-08,
                                         3.814697265625e-06,
                                         0.000159464634488794,
                                         0.00300575756120015,
                                         0.0288296785956552,
                                         0.152480998886578,
                                         0.468919108635853,
                                         0.86682480891169,

                                         0.883493832166615,
                                         0.518978896063906,
                                         0.196614372919516,
                                         0.0468693275997731,
                                         0.00676348157370232,
                                         0.000557869149110822,
                                         2.41859659163994e-05,
                                         4.87018975825479e-07,
                                         3.77414069750797e-09,
                                         8.33302698561713e-12,
                                         3.11845379459187e-15,
                                         7.19760272578013e-20,
                                         9.24879060342065e-27,
                                         1.12632489832232e-39,
                                         0.109816666960812,
                                         0.346671872084829,
                                         0.334689279378284,
                                         0.156353815283912,
                                         0.0389473536346991,
                                         0.00518320639090817,
                                         0.000350439251618859,
                                         1.08903436808389e-05,
                                         1.31612439529443e-07,
                                         4.68856731568433e-10,
                                         3.02875792606263e-13,
                                         1.36995513062896e-17,
                                         4.48601838715443e-24,
                                         2.9359182494265e-36,
                                         0.00644583842882984,
                                         0.109353813844143,
                                         0.269038703238982,
                                         0.246305814305694,
                                         0.105908792668725,
                                         0.0227410797093191,
                                         0.00239777524689612,
                                         0.00011499625803754,
                                         2.16731587740099e-06,
                                         1.24573005869688e-08,
                                         1.38910870972278e-11,
                                         1.23132085449034e-15,
                                         1.02750412086179e-21,
                                         3.61385441523205e-33,
                                         0.000237394347809145,
                                         0.0216435783605561,
                                         0.135696149024154,
                                         0.243456092299903,
                                         0.18070321639523,
                                         0.0626041986254138,
                                         0.0102939961865056,
                                         0.000761913439199153,
                                         2.23937738586074e-05,
                                         2.07676585211815e-07,
                                         3.99749295702784e-10,
                                         6.94409940447847e-14,
                                         1.47667831570385e-19,
                                         2.79111126533384e-30,
                                         6.14743398184522e-06,
                                         0.00301201226010641,
                                         0.0481230145005443,
                                         0.169199536751811,
                                         0.216786527125483,
                                         0.121179283607593,
                                         0.0310736359485957,
                                         0.0035494421246903,
                                         0.000162691515103694,
                                         2.43435361518308e-06,
                                         8.08856947189576e-09,
                                         2.75355124280046e-12,
                                         1.49217844621709e-17,
                                         1.51571024493749e-27,
                                         1.18862301761037e-07,
                                         0.00031297609629261,
                                         0.0127428006676509,
                                         0.0878020095877561,
                                         0.194189371922703,
                                         0.175137870300934,
                                         0.0700368987644415,
                                         0.012346427995758,
                                         0.000882529604413514,
                                         2.13062274042948e-05,
                                         1.22203176877085e-07,
                                         8.15261925816579e-11,
                                         1.12585481919207e-15,
                                         6.14585042203902e-25,
                                         1.77839598527269e-09,
                                         2.51651587261054e-05,
                                         0.00261102497154083,
                                         0.0352568747637729,
                                         0.134602366856097,
                                         0.195869050731792,
                                         0.122150658283112,
                                         0.0332320113323342,
                                         0.00370448407637642,
                                         0.000144299066745866,
                                         1.42865492661632e-06,
                                         1.86782089097276e-09,
                                         6.57321882489868e-14,
                                         1.92833221559965e-22,
                                         2.10525111754806e-11,
                                         1.60095558212055e-06,
                                         0.000423299996883908,
                                         0.0112014507580487,
                                         0.0738194899699132,
                                         0.17331760412042,
                                         0.168560503705599,
                                         0.0707722545251139,
                                         0.0123031773970444,
                                         0.000773235154634183,
                                         1.32148821411239e-05,
                                         3.38582416842361e-08,
                                         3.03644132675694e-12,
                                         4.78710365786305e-20,
                                         1.99893575363064e-13,
                                         8.16917846176321e-08,
                                         5.5043367695838e-05,
                                         0.00285446174631342,
                                         0.0324719878675271,
                                         0.123009605966525,
                                         0.186567211794077,
                                         0.120889596027365,
                                         0.0327737647429452,
                                         0.00332337349018841,
                                         9.80434843800664e-05,
                                         4.92280989667253e-07,
                                         1.12504835528174e-10,
                                         9.53198386452975e-18,
                                         1.53372874731827e-15,
                                         3.36846684405804e-09,
                                         5.78384334648668e-06,
                                         0.000587799175874075,
                                         0.0115425430822567,
                                         0.0705488875857847,
                                         0.166866675788083,
                                         0.166866675788083,
                                         0.0705488875857847,
                                         0.0115425430822567,
                                         0.000587799175874077,
                                         5.78384334648668e-06,
                                         3.36846684405799e-09,
                                         1.53372874731827e-15,
                                         9.53198386452975e-18,
                                         1.12504835528175e-10,
                                         4.92280989667253e-07,
                                         9.80434843800661e-05,
                                         0.0033233734901884,
                                         0.0327737647429452,
                                         0.120889596027364,
                                         0.186567211794078,
                                         0.123009605966525,
                                         0.0324719878675271,
                                         0.00285446174631343,
                                         5.5043367695838e-05,
                                         8.16917846176311e-08,
                                         1.99893575363064e-13,
                                         4.78710365786305e-20,
                                         3.03644132675699e-12,
                                         3.38582416842361e-08,
                                         1.32148821411239e-05,
                                         0.000773235154634181,
                                         0.0123031773970444,
                                         0.0707722545251138,
                                         0.168560503705599,
                                         0.17331760412042,
                                         0.0738194899699133,
                                         0.0112014507580488,
                                         0.000423299996883908,
                                         1.60095558212053e-06,
                                         2.10525111754806e-11,
                                         1.92833221559965e-22,
                                         6.5732188248988e-14,
                                         1.86782089097276e-09,
                                         1.42865492661631e-06,
                                         0.000144299066745866,
                                         0.00370448407637642,
                                         0.0332320113323342,
                                         0.122150658283112,
                                         0.195869050731792,
                                         0.134602366856097,
                                         0.035256874763773,
                                         0.00261102497154083,
                                         2.51651587261052e-05,
                                         1.77839598527269e-09,
                                         6.14585042203902e-25,
                                         1.12585481919209e-15,
                                         8.15261925816579e-11,
                                         1.22203176877084e-07,
                                         2.13062274042947e-05,
                                         0.000882529604413514,
                                         0.012346427995758,
                                         0.0700368987644416,
                                         0.175137870300934,
                                         0.194189371922703,
                                         0.0878020095877562,
                                         0.0127428006676509,
                                         0.000312976096292608,
                                         1.18862301761037e-07,
                                         1.51571024493749e-27,
                                         1.49217844621712e-17,
                                         2.75355124280046e-12,
                                         8.08856947189572e-09,
                                         2.43435361518307e-06,
                                         0.000162691515103694,
                                         0.00354944212469029,
                                         0.0310736359485957,
                                         0.121179283607593,
                                         0.216786527125483,
                                         0.169199536751811,
                                         0.0481230145005443,
                                         0.00301201226010639,
                                         6.14743398184522e-06,
                                         2.79111126533384e-30,
                                         1.47667831570389e-19,
                                         6.94409940447847e-14,
                                         3.99749295702782e-10,
                                         2.07676585211815e-07,
                                         2.23937738586074e-05,
                                         0.000761913439199152,
                                         0.0102939961865056,
                                         0.0626041986254138,
                                         0.18070321639523,
                                         0.243456092299904,
                                         0.135696149024154,
                                         0.021643578360556,
                                         0.000237394347809145,
                                         3.61385441523205e-33,
                                         1.02750412086181e-21,
                                         1.23132085449034e-15,
                                         1.38910870972277e-11,
                                         1.24573005869687e-08,
                                         2.16731587740099e-06,
                                         0.000114996258037539,
                                         0.00239777524689612,
                                         0.0227410797093191,
                                         0.105908792668725,
                                         0.246305814305694,
                                         0.269038703238982,
                                         0.109353813844143,
                                         0.00644583842882984,
                                         2.9359182494265e-36,
                                         4.48601838715455e-24,
                                         1.36995513062896e-17,
                                         3.02875792606261e-13,
                                         4.68856731568431e-10,
                                         1.31612439529443e-07,
                                         1.08903436808388e-05,
                                         0.000350439251618859,
                                         0.00518320639090817,
                                         0.0389473536346991,
                                         0.156353815283912,
                                         0.334689279378284,
                                         0.346671872084829,
                                         0.109816666960812,
                                         1.12632489832232e-39,
                                         9.24879060342091e-27,
                                         7.19760272578013e-20,
                                         3.11845379459185e-15,
                                         8.33302698561709e-12,
                                         3.77414069750797e-09,
                                         4.87018975825478e-07,
                                         2.41859659163994e-05,
                                         0.000557869149110822,
                                         0.00676348157370232,
                                         0.0468693275997731,
                                         0.196614372919516,
                                         0.518978896063906,
                                         0.883493832166615,

                                         0.897274856578844,
                                         0.563503378203573,
                                         0.241529014919447,
                                         0.0693465269344426,
                                         0.0129566562978396,
                                         0.00151021467763062,
                                         0.000103394506746201,
                                         3.814697265625e-06,
                                         6.68964216145297e-08,
                                         4.61333207251564e-10,
                                         9.25120693860259e-13,
                                         3.20614349670786e-16,
                                         6.98253193179207e-21,
                                         8.62169166727792e-28,
                                         1.02705821736795e-40,
                                         0.0975517806635748,
                                         0.328420241373211,
                                         0.357062497648248,
                                         0.199483055417571,
                                         0.0636916373030455,
                                         0.0118131417652142,
                                         0.00123765058028227,
                                         6.866455078125e-05,
                                         1.81070338499499e-06,
                                         1.91087816508308e-08,
                                         6.0975297927676e-11,
                                         3.61115949154949e-14,
                                         1.53032267112015e-18,
                                         4.79296332238427e-25,
                                         3.06076769411609e-37,
                                         0.00500831141014933,
                                         0.0903877817954282,
                                         0.249267459598147,
                                         0.270977809914101,
                                         0.147848971890406,
                                         0.0436353602082259,
                                         0.0069959235425131,
                                         0.000583648681640625,
                                         2.31439861215253e-05,
                                         3.73764198671716e-07,
                                         1.89782379191613e-09,
                                         1.92068791850162e-12,
                                         1.58379699475429e-16,
                                         1.25823610904398e-22,
                                         4.30736918267291e-34,
                                         0.000161334501238594,
                                         0.0156087937963199,
                                         0.109185945414353,
                                         0.230962380708926,
                                         0.215344610335571,
                                         0.101132680254475,
                                         0.0248125764037766,
                                         0.00311279296875,
                                         0.000185613179083634,
                                         4.58714220505056e-06,
                                         3.70627504046947e-08,
                                         6.40983339394186e-11,
                                         1.02847992515343e-14,
                                         2.07252599065461e-20,
                                         3.80341422550346e-31,
                                         3.65422863529954e-06,
                                         0.00189522814784562,
                                         0.0336279528627378,
                                         0.138414410170246,
                                         0.220537396978496,
                                         0.164807514156497,
                                         0.0618772790556578,
                                         0.0116729736328125,
                                         0.0010466753538373,
                                         3.95839607810412e-05,
                                         5.08922840857345e-07,
                                         1.50407404848638e-09,
                                         4.69596296881975e-13,
                                         2.40032675750782e-18,
                                         2.36138987220624e-28,
                                         6.18003515863418e-08,
                                         0.000171822651289523,
                                         0.00773322924533739,
                                         0.0619367094908058,
                                         0.168638699917902,
                                         0.200534575982038,
                                         0.115217200426873,
                                         0.032684326171875,
                                         0.00440698883541106,
                                         0.000255048665276695,
                                         5.21786699102474e-06,
                                         2.63522958846796e-08,
                                         1.60095922697988e-11,
                                         2.07571398285392e-16,
                                         1.09468339299076e-25,
                                         8.0876108407211e-10,
                                         1.2054061553988e-05,
                                         0.00137611715833405,
                                         0.0214461343328615,
                                         0.0997852425309476,
                                         0.188814607384744,
                                         0.166011242235517,
                                         0.0708160400390625,
                                         0.0143583975325307,
                                         0.00127163049197143,
                                         4.13969301286181e-05,
                                         3.57274293340429e-07,
                                         4.22347521653857e-10,
                                         1.38898874285712e-14,
                                         3.92684205585797e-23,
                                         8.37414833998207e-12,
                                         6.69079151914422e-07,
                                         0.000193749700276418,
                                         0.00587544843574639,
                                         0.0467160907169225,
                                         0.140660779488439,
                                         0.18925561977735,
                                         0.12139892578125,
                                         0.037013580650977,
                                         0.00501637408187252,
                                         0.000259856971418607,
                                         3.83244723111146e-06,
                                         8.81557678902117e-09,
                                         7.35395630621801e-13,
                                         1.11452404280387e-20,
                                         6.95473434960835e-14,
                                         2.97879820019201e-08,
                                         2.1879941719829e-05,
                                         0.00129107808227708,
                                         0.0175422849457179,
                                         0.0840484951165818,
                                         0.173053170940031,
                                         0.166923522949219,
                                         0.0765307059507698,
                                         0.0158722461355402,
                                         0.00130833843804183,
                                         3.29738828612502e-05,
                                         1.47587986410267e-07,
                                         3.12293436326e-11,
                                         2.53720139683614e-18,
                                         4.66740228414433e-16,
                                         1.0716660192727e-09,
                                         1.99666899808296e-06,
                                         0.000229254986212673,
                                         0.00532305093125618,
                                         0.0405827671541939,
                                         0.127868959442715,
                                         0.185470581054688,
                                         0.127868959442715,
                                         0.0405827671541939,
                                         0.00532305093125618,
                                         0.000229254986212672,
                                         1.99666899808297e-06,
                                         1.0716660192727e-09,
                                         4.66740228414472e-16,
                                         2.53720139683591e-18,
                                         3.12293436326e-11,
                                         1.47587986410266e-07,
                                         3.29738828612504e-05,
                                         0.00130833843804183,
                                         0.0158722461355402,
                                         0.0765307059507698,
                                         0.166923522949219,
                                         0.173053170940031,
                                         0.0840484951165818,
                                         0.0175422849457179,
                                         0.00129107808227708,
                                         2.18799417198291e-05,
                                         2.97879820019201e-08,
                                         6.95473434960887e-14,
                                         1.11452404280375e-20,
                                         7.35395630621801e-13,
                                         8.8155767890211e-09,
                                         3.83244723111148e-06,
                                         0.000259856971418607,
                                         0.00501637408187252,
                                         0.037013580650977,
                                         0.12139892578125,
                                         0.18925561977735,
                                         0.140660779488439,
                                         0.0467160907169225,
                                         0.00587544843574637,
                                         0.000193749700276419,
                                         6.69079151914422e-07,
                                         8.37414833998261e-12,
                                         3.92684205585754e-23,
                                         1.38898874285712e-14,
                                         4.22347521653853e-10,
                                         3.5727429334043e-07,
                                         4.13969301286181e-05,
                                         0.00127163049197143,
                                         0.0143583975325307,
                                         0.0708160400390625,
                                         0.166011242235517,
                                         0.188814607384744,
                                         0.0997852425309476,
                                         0.0214461343328614,
                                         0.00137611715833405,
                                         1.2054061553988e-05,
                                         8.08761084072155e-10,
                                         1.09468339299063e-25,
                                         2.07571398285392e-16,
                                         1.60095922697987e-11,
                                         2.63522958846797e-08,
                                         5.21786699102474e-06,
                                         0.000255048665276695,
                                         0.00440698883541106,
                                         0.032684326171875,
                                         0.115217200426873,
                                         0.200534575982038,
                                         0.168638699917902,
                                         0.0619367094908057,
                                         0.00773322924533742,
                                         0.000171822651289523,
                                         6.18003515863447e-08,
                                         2.36138987220594e-28,
                                         2.40032675750782e-18,
                                         4.6959629688197e-13,
                                         1.50407404848639e-09,
                                         5.08922840857345e-07,
                                         3.95839607810412e-05,
                                         0.0010466753538373,
                                         0.0116729736328125,
                                         0.0618772790556578,
                                         0.164807514156497,
                                         0.220537396978496,
                                         0.138414410170246,
                                         0.0336279528627379,
                                         0.00189522814784562,
                                         3.65422863529968e-06,
                                         3.80341422550293e-31,
                                         2.07252599065461e-20,
                                         1.02847992515342e-14,
                                         6.4098333939419e-11,
                                         3.70627504046947e-08,
                                         4.58714220505056e-06,
                                         0.000185613179083634,
                                         0.00311279296875,
                                         0.0248125764037766,
                                         0.101132680254475,
                                         0.215344610335571,
                                         0.230962380708926,
                                         0.109185945414354,
                                         0.0156087937963199,
                                         0.000161334501238599,
                                         4.30736918267227e-34,
                                         1.25823610904398e-22,
                                         1.58379699475427e-16,
                                         1.92068791850163e-12,
                                         1.89782379191613e-09,
                                         3.73764198671716e-07,
                                         2.31439861215253e-05,
                                         0.000583648681640625,
                                         0.0069959235425131,
                                         0.0436353602082259,
                                         0.147848971890406,
                                         0.270977809914101,
                                         0.249267459598148,
                                         0.0903877817954282,
                                         0.00500831141014942,
                                         3.0607676941156e-37,
                                         4.79296332238427e-25,
                                         1.53032267112013e-18,
                                         3.61115949154951e-14,
                                         6.0975297927676e-11,
                                         1.91087816508308e-08,
                                         1.81070338499499e-06,
                                         6.866455078125e-05,
                                         0.00123765058028228,
                                         0.0118131417652142,
                                         0.0636916373030455,
                                         0.19948305541757,
                                         0.357062497648248,
                                         0.328420241373211,
                                         0.0975517806635757,
                                         1.02705821736777e-40,
                                         8.62169166727792e-28,
                                         6.98253193179198e-21,
                                         3.20614349670788e-16,
                                         9.25120693860259e-13,
                                         4.61333207251564e-10,
                                         6.68964216145297e-08,
                                         3.814697265625e-06,
                                         0.000103394506746201,
                                         0.00151021467763062,
                                         0.0129566562978396,
                                         0.0693465269344426,
                                         0.241529014919447,
                                         0.563503378203573,
                                         0.897274856578844,

                                         0.908786353513503,
                                         0.602982538150231,
                                         0.285972595905418,
                                         0.0955541166617394,
                                         0.0220026730841628,
                                         0.00338209237505208,
                                         0.00033191487329024,
                                         1.95435534785281e-05,
                                         6.32450360195622e-07,
                                         9.90866774183582e-09,
                                         6.21162525135568e-11,
                                         1.15115221086253e-13,
                                         3.74572022905458e-17,
                                         7.77735342300111e-22,
                                         9.29307529404019e-29,
                                         1.08711649055899e-41,
                                         0.0871524379339674,
                                         0.309355654381098,
                                         0.370740095163438,
                                         0.239658782090057,
                                         0.0935420084332214,
                                         0.0226298300074888,
                                         0.00334895794636726,
                                         0.000290736482501164,
                                         1.37744876567774e-05,
                                         3.1818323706035e-07,
                                         3.00783792327108e-09,
                                         8.77296744590598e-12,
                                         4.83878583623955e-15,
                                         1.94371105914715e-19,
                                         5.8688247774595e-26,
                                         3.67285134104417e-38,
                                         0.00394678713673599,
                                         0.0749476123696606,
                                         0.226966163717012,
                                         0.28384661176278,
                                         0.187795142789471,
                                         0.0715028806252852,
                                         0.0159565535802973,
                                         0.00204240541225341,
                                         0.000141667713587499,
                                         4.82487144568599e-06,
                                         6.87780779818116e-08,
                                         3.1572335148967e-10,
                                         2.95177931918596e-13,
                                         2.29391856970551e-17,
                                         1.75020633356369e-23,
                                         5.85972234652022e-35,
                                         0.000112147002515434,
                                         0.0113929803328502,
                                         0.0871831345252454,
                                         0.210937543083437,
                                         0.236560278307814,
                                         0.141757350548837,
                                         0.0477032862023146,
                                         0.00900252167629051,
                                         0.000914210315934572,
                                         4.59064806240032e-05,
                                         9.86791595494099e-07,
                                         7.1292967641482e-09,
                                         1.12982498052772e-11,
                                         1.69865073258324e-15,
                                         3.27496875734752e-21,
                                         5.86584425740711e-32,
                                         2.24059919731123e-06,
                                         0.00121772559154861,
                                         0.023547043240426,
                                         0.110219031804866,
                                         0.209523060597676,
                                         0.197606022488691,
                                         0.100274392780434,
                                         0.0279009466459192,
                                         0.00414814462956571,
                                         0.000307110589678513,
                                         9.95481946882764e-06,
                                         1.13192909087603e-07,
                                         3.04068197552571e-10,
                                         8.84428490370163e-14,
                                         4.30881265090405e-19,
                                         4.12873070915028e-29,
                                         3.34246944244866e-08,
                                         9.71825481860495e-05,
                                         0.00474861595033726,
                                         0.0430017434537024,
                                         0.138563421902714,
                                         0.205675010945742,
                                         0.157383263722078,
                                         0.0645654920504675,
                                         0.0140536271184157,
                                         0.00153406000738775,
                                         7.49839127414467e-05,
                                         1.34189493508945e-06,
                                         6.1102304515664e-09,
                                         3.43833447069901e-12,
                                         4.23286896792732e-17,
                                         2.16984819270347e-26,
                                         3.85837811537059e-10,
                                         6.00151923092056e-06,
                                         0.000741023162543015,
                                         0.0129822391935178,
                                         0.0709086865716631,
                                         0.165652102413774,
                                         0.191144199887653,
                                         0.115615500127435,
                                         0.0368431736565535,
                                         0.00592958057518701,
                                         0.000437055791341307,
                                         1.23098249280659e-05,
                                         9.50119543460249e-08,
                                         1.03435016695437e-10,
                                         3.21770413677205e-15,
                                         8.82421771156173e-24,
                                         3.52397833529829e-12,
                                         2.93241360593274e-07,
                                         9.14929484696473e-05,
                                         0.00310101775942594,
                                         0.028710541677426,
                                         0.105560996040115,
                                         0.183677011825116,
                                         0.163803343944901,
                                         0.076421708734765,
                                         0.0181341292096576,
                                         0.00201556479540363,
                                         8.93462528774139e-05,
                                         1.16893406938425e-06,
                                         2.46194394611708e-09,
                                         1.935300893347e-13,
                                         2.83931895913803e-21,
                                         2.58155371596829e-14,
                                         1.14923472977444e-08,
                                         9.06072513563776e-06,
                                         0.000594125773065552,
                                         0.00932401199865064,
                                         0.0539547269197157,
                                         0.141568939474544,
                                         0.186143963383539,
                                         0.127144022427308,
                                         0.0444824735208122,
                                         0.00745548912703402,
                                         0.000520140058467015,
                                         1.15350976132718e-05,
                                         4.70011218994947e-08,
                                         9.33620496301023e-12,
                                         7.32776684353612e-19,
                                         1.52821301778859e-16,
                                         3.63954463774021e-10,
                                         7.25091945296069e-07,
                                         9.19829478753625e-05,
                                         0.0024469159591751,
                                         0.0222848805496825,
                                         0.0881730663612088,
                                         0.170934585189069,
                                         0.170934585189069,
                                         0.0881730663612086,
                                         0.0222848805496825,
                                         0.0024469159591751,
                                         9.19829478753622e-05,
                                         7.25091945296074e-07,
                                         3.63954463774021e-10,
                                         1.52821301778844e-16,
                                         7.32776684353689e-19,
                                         9.33620496301023e-12,
                                         4.70011218994943e-08,
                                         1.15350976132719e-05,
                                         0.000520140058467015,
                                         0.00745548912703402,
                                         0.0444824735208123,
                                         0.127144022427308,
                                         0.186143963383539,
                                         0.141568939474544,
                                         0.0539547269197157,
                                         0.00932401199865064,
                                         0.00059412577306555,
                                         9.06072513563782e-06,
                                         1.14923472977444e-08,
                                         2.58155371596807e-14,
                                         2.83931895913836e-21,
                                         1.935300893347e-13,
                                         2.46194394611706e-09,
                                         1.16893406938425e-06,
                                         8.93462528774139e-05,
                                         0.00201556479540363,
                                         0.0181341292096576,
                                         0.076421708734765,
                                         0.1638033439449,
                                         0.183677011825116,
                                         0.105560996040115,
                                         0.028710541677426,
                                         0.00310101775942594,
                                         9.14929484696479e-05,
                                         2.93241360593274e-07,
                                         3.52397833529804e-12,
                                         8.82421771156283e-24,
                                         3.21770413677205e-15,
                                         1.03435016695436e-10,
                                         9.50119543460254e-08,
                                         1.23098249280659e-05,
                                         0.000437055791341307,
                                         0.00592958057518702,
                                         0.0368431736565535,
                                         0.115615500127435,
                                         0.191144199887653,
                                         0.165652102413774,
                                         0.0709086865716631,
                                         0.0129822391935177,
                                         0.000741023162543018,
                                         6.00151923092056e-06,
                                         3.85837811537035e-10,
                                         2.16984819270376e-26,
                                         4.23286896792732e-17,
                                         3.43833447069897e-12,
                                         6.11023045156644e-09,
                                         1.34189493508945e-06,
                                         7.49839127414467e-05,
                                         0.00153406000738775,
                                         0.0140536271184157,
                                         0.0645654920504674,
                                         0.157383263722078,
                                         0.205675010945742,
                                         0.138563421902714,
                                         0.0430017434537023,
                                         0.00474861595033728,
                                         9.71825481860495e-05,
                                         3.34246944244848e-08,
                                         4.12873070915089e-29,
                                         4.30881265090405e-19,
                                         8.84428490370153e-14,
                                         3.04068197552573e-10,
                                         1.13192909087603e-07,
                                         9.95481946882764e-06,
                                         0.000307110589678513,
                                         0.00414814462956571,
                                         0.0279009466459192,
                                         0.100274392780434,
                                         0.197606022488691,
                                         0.209523060597676,
                                         0.110219031804866,
                                         0.0235470432404261,
                                         0.00121772559154861,
                                         2.24059919731114e-06,
                                         5.86584425740803e-32,
                                         3.27496875734752e-21,
                                         1.69865073258322e-15,
                                         1.12982498052773e-11,
                                         7.1292967641482e-09,
                                         9.86791595494099e-07,
                                         4.59064806240033e-05,
                                         0.000914210315934571,
                                         0.00900252167629049,
                                         0.0477032862023146,
                                         0.141757350548837,
                                         0.236560278307814,
                                         0.210937543083437,
                                         0.0871831345252456,
                                         0.0113929803328502,
                                         0.00011214700251543,
                                         5.85972234652121e-35,
                                         1.75020633356369e-23,
                                         2.29391856970548e-17,
                                         2.95177931918598e-13,
                                         3.1572335148967e-10,
                                         6.87780779818116e-08,
                                         4.824871445686e-06,
                                         0.000141667713587499,
                                         0.0020424054122534,
                                         0.0159565535802973,
                                         0.0715028806252852,
                                         0.187795142789471,
                                         0.283846611762779,
                                         0.226966163717012,
                                         0.0749476123696606,
                                         0.0039467871367359,
                                         3.67285134104482e-38,
                                         5.8688247774595e-26,
                                         1.94371105914712e-19,
                                         4.83878583623959e-15,
                                         8.77296744590598e-12,
                                         3.00783792327108e-09,
                                         3.18183237060351e-07,
                                         1.37744876567774e-05,
                                         0.000290736482501163,
                                         0.00334895794636726,
                                         0.0226298300074888,
                                         0.0935420084332214,
                                         0.239658782090057,
                                         0.370740095163439,
                                         0.309355654381098,
                                         0.0871524379339665,
                                         1.08711649055919e-41,
                                         9.29307529404019e-29,
                                         7.77735342300099e-22,
                                         3.74572022905461e-17,
                                         1.15115221086253e-13,
                                         6.21162525135568e-11,
                                         9.90866774183585e-09,
                                         6.3245036019562e-07,
                                         1.9543553478528e-05,
                                         0.00033191487329024,
                                         0.00338209237505208,
                                         0.0220026730841628,
                                         0.0955541166617394,
                                         0.285972595905418,
                                         0.602982538150231,
                                         0.908786353513503,

                                         0.918492811255139,
                                         0.637955952505858,
                                         0.329075640987307,
                                         0.124642573665065,
                                         0.0340746140477307,
                                         0.00656087403745221,
                                         0.00086019576469988,
                                         7.33310177474111e-05,
                                         3.814697265625e-06,
                                         1.10801802017165e-07,
                                         1.5815120195253e-09,
                                         9.16109287762691e-12,
                                         1.59016755555096e-14,
                                         4.91003950554862e-18,
                                         9.79777563608261e-23,
                                         1.13916091413633e-29,
                                         1.31259362836115e-42,
                                         0.0782760809876203,
                                         0.290362592103425,
                                         0.37728540496479,
                                         0.275152383018708,
                                         0.126662646641561,
                                         0.0380442441139724,
                                         0.00743411950485949,
                                         0.000920136779282637,
                                         6.866455078125e-05,
                                         2.86106125324776e-06,
                                         5.92905213053926e-08,
                                         5.11876316961002e-10,
                                         1.38602251578606e-12,
                                         7.20647901041745e-16,
                                         2.76884236364842e-20,
                                         8.10923927113738e-27,
                                         4.99024639555543e-39,
                                         0.00315013239488292,
                                         0.0624075388987587,
                                         0.204263465108443,
                                         0.286831318752686,
                                         0.222337435510819,
                                         0.104174791570805,
                                         0.0303394844770806,
                                         0.00545209593434998,
                                         0.000583648681640625,
                                         3.48862194774915e-05,
                                         1.04964988226279e-06,
                                         1.35060765651199e-08,
                                         5.7048483364952e-11,
                                         4.9946801987783e-14,
                                         3.6950080823138e-18,
                                         2.72597315705692e-24,
                                         8.95901178981431e-36,
                                         7.954416770186e-05,
                                         0.00841614578341331,
                                         0.0693890892068862,
                                         0.187611589555689,
                                         0.244881754383239,
                                         0.17898478852927,
                                         0.0776902749003934,
                                         0.0202700299221577,
                                         0.00311279296875,
                                         0.000266907307234577,
                                         1.16595947542674e-05,
                                         2.23600711176534e-07,
                                         1.47332216722372e-09,
                                         2.17206115487156e-12,
                                         3.09394334276993e-16,
                                         5.74966891480416e-22,
                                         1.0092018287359e-32,
                                         1.41227872554403e-06,
                                         0.000798035081024972,
                                         0.0165738810985657,
                                         0.0862830118123935,
                                         0.189641247543805,
                                         0.216223127837869,
                                         0.139880655581644,
                                         0.0529880369336029,
                                         0.0116729736328125,
                                         0.00143581854611001,
                                         9.10657336480491e-05,
                                         2.60285331209472e-06,
                                         2.67537018563753e-08,
                                         6.64154242217923e-11,
                                         1.82155320171191e-14,
                                         8.52700739007632e-20,
                                         7.99334373725977e-30,
                                         1.87223023970949e-08,
                                         5.65011750642333e-05,
                                         0.00295586104718788,
                                         0.0296290476953103,
                                         0.109656894620604,
                                         0.195036067454664,
                                         0.188050903160113,
                                         0.103425596545653,
                                         0.032684326171875,
                                         0.00576720537417781,
                                         0.000531071858670493,
                                         2.26231445271976e-05,
                                         3.62741134024024e-07,
                                         1.51632612259484e-09,
                                         8.00752175666302e-13,
                                         9.44228718102225e-18,
                                         4.72721890089212e-27,
                                         1.92057895058253e-10,
                                         3.09547319455274e-06,
                                         0.000407922706452405,
                                         0.00787307086433311,
                                         0.049065152471937,
                                         0.136132490017977,
                                         0.19562630798672,
                                         0.156211250597676,
                                         0.0708160400390625,
                                         0.0179252551550912,
                                         0.00239654538347957,
                                         0.000152156432495941,
                                         3.80578105925127e-06,
                                         2.67886207131073e-08,
                                         2.72388309975958e-11,
                                         8.09080699693004e-16,
                                         2.16330128429697e-24,
                                         1.55882074920502e-12,
                                         1.34179978895547e-07,
                                         4.45412961205027e-05,
                                         0.00165524289348976,
                                         0.0173700659249947,
                                         0.0751795547292226,
                                         0.161016431657456,
                                         0.186675656856689,
                                         0.12139892578125,
                                         0.0440815015512938,
                                         0.00855675565470121,
                                         0.000809689986496179,
                                         3.15923475506941e-05,
                                         3.74454620537958e-07,
                                         7.33111288809861e-10,
                                         5.48526459636605e-14,
                                         7.83284273300587e-22,
                                         1.01479818869071e-14,
                                         4.66517439193918e-09,
                                         3.90092241564471e-06,
                                         0.000279125038271087,
                                         0.00493229760114727,
                                         0.0333009943977961,
                                         0.10629984449316,
                                         0.178929751656747,
                                         0.166923522949219,
                                         0.0869494631374133,
                                         0.0245048526568783,
                                         0.00345594412056436,
                                         0.000210348533567881,
                                         4.19824255576577e-06,
                                         1.58259883276974e-08,
                                         2.98279093324475e-12,
                                         2.27478993412583e-19,
                                         5.33848443930229e-17,
                                         1.31069861111189e-10,
                                         2.76074673690635e-07,
                                         3.80356332938079e-05,
                                         0.00113175356537669,
                                         0.011919813145752,
                                         0.0567087210698255,
                                         0.138590104602242,
                                         0.185470581054688,
                                         0.138590104602242,
                                         0.0567087210698255,
                                         0.011919813145752,
                                         0.00113175356537669,
                                         3.80356332938077e-05,
                                         2.76074673690635e-07,
                                         1.31069861111186e-10,
                                         5.33848443930286e-17,
                                         2.27478993412557e-19,
                                         2.98279093324482e-12,
                                         1.58259883276974e-08,
                                         4.19824255576579e-06,
                                         0.000210348533567882,
                                         0.00345594412056437,
                                         0.0245048526568783,
                                         0.0869494631374133,
                                         0.166923522949219,
                                         0.178929751656747,
                                         0.10629984449316,
                                         0.0333009943977961,
                                         0.00493229760114726,
                                         0.000279125038271086,
                                         3.90092241564471e-06,
                                         4.6651743919391e-09,
                                         1.01479818869081e-14,
                                         7.83284273300486e-22,
                                         5.48526459636619e-14,
                                         7.33111288809861e-10,
                                         3.7445462053796e-07,
                                         3.15923475506942e-05,
                                         0.000809689986496181,
                                         0.00855675565470121,
                                         0.0440815015512938,
                                         0.12139892578125,
                                         0.186675656856689,
                                         0.161016431657456,
                                         0.0751795547292225,
                                         0.0173700659249946,
                                         0.00165524289348976,
                                         4.45412961205027e-05,
                                         1.34179978895544e-07,
                                         1.55882074920515e-12,
                                         2.16330128429666e-24,
                                         8.09080699693026e-16,
                                         2.72388309975958e-11,
                                         2.67886207131075e-08,
                                         3.80578105925128e-06,
                                         0.000152156432495942,
                                         0.00239654538347957,
                                         0.0179252551550912,
                                         0.0708160400390625,
                                         0.156211250597676,
                                         0.19562630798672,
                                         0.136132490017977,
                                         0.0490651524719369,
                                         0.00787307086433309,
                                         0.000407922706452405,
                                         3.0954731945527e-06,
                                         1.92057895058266e-10,
                                         4.7272189008914e-27,
                                         9.44228718102253e-18,
                                         8.00752175666302e-13,
                                         1.51632612259485e-09,
                                         3.62741134024026e-07,
                                         2.26231445271976e-05,
                                         0.000531071858670493,
                                         0.00576720537417781,
                                         0.032684326171875,
                                         0.103425596545653,
                                         0.188050903160113,
                                         0.195036067454664,
                                         0.109656894620604,
                                         0.0296290476953103,
                                         0.00295586104718788,
                                         5.65011750642327e-05,
                                         1.8722302397096e-08,
                                         7.99334373725845e-30,
                                         8.52700739007659e-20,
                                         1.82155320171191e-14,
                                         6.64154242217928e-11,
                                         2.67537018563754e-08,
                                         2.60285331209473e-06,
                                         9.10657336480491e-05,
                                         0.00143581854611001,
                                         0.0116729736328125,
                                         0.0529880369336029,
                                         0.139880655581644,
                                         0.216223127837869,
                                         0.189641247543805,
                                         0.0862830118123933,
                                         0.0165738810985657,
                                         0.000798035081024965,
                                         1.41227872554409e-06,
                                         1.00920182873573e-32,
                                         5.74966891480436e-22,
                                         3.09394334276993e-16,
                                         2.17206115487158e-12,
                                         1.47332216722373e-09,
                                         2.23600711176535e-07,
                                         1.16595947542674e-05,
                                         0.000266907307234577,
                                         0.00311279296875,
                                         0.0202700299221577,
                                         0.0776902749003934,
                                         0.17898478852927,
                                         0.244881754383238,
                                         0.187611589555689,
                                         0.0693890892068862,
                                         0.00841614578341326,
                                         7.95441677018628e-05,
                                         8.95901178981262e-36,
                                         2.72597315705702e-24,
                                         3.6950080823138e-18,
                                         4.99468019877834e-14,
                                         5.70484833649523e-11,
                                         1.35060765651199e-08,
                                         1.04964988226279e-06,
                                         3.48862194774915e-05,
                                         0.000583648681640625,
                                         0.00545209593434998,
                                         0.0303394844770806,
                                         0.104174791570805,
                                         0.222337435510819,
                                         0.286831318752686,
                                         0.204263465108443,
                                         0.0624075388987584,
                                         0.00315013239488299,
                                         4.99024639555443e-39,
                                         8.10923927113769e-27,
                                         2.76884236364842e-20,
                                         7.20647901041751e-16,
                                         1.38602251578607e-12,
                                         5.11876316961004e-10,
                                         5.92905213053926e-08,
                                         2.86106125324776e-06,
                                         6.866455078125e-05,
                                         0.000920136779282637,
                                         0.00743411950485949,
                                         0.0380442441139724,
                                         0.126662646641561,
                                         0.275152383018708,
                                         0.37728540496479,
                                         0.290362592103425,
                                         0.0782760809876212,
                                         1.31259362836087e-42,
                                         1.13916091413638e-29,
                                         9.79777563608261e-23,
                                         4.91003950554867e-18,
                                         1.59016755555097e-14,
                                         9.16109287762695e-12,
                                         1.5815120195253e-09,
                                         1.10801802017165e-07,
                                         3.814697265625e-06,
                                         7.33310177474111e-05,
                                         0.00086019576469988,
                                         0.00656087403745221,
                                         0.0340746140477307,
                                         0.124642573665065,
                                         0.329075640987307,
                                         0.637955952505858,
                                         0.918492811255139,

                                         0.926747610718249,
                                         0.668953231122606,
                                         0.370279193726227,
                                         0.155753948571053,
                                         0.0491157173791647,
                                         0.0113908039695076,
                                         0.00189280922881392,
                                         0.000217584310280209,
                                         1.65034291101867e-05,
                                         7.7439352068359e-07,
                                         2.05531218646112e-08,
                                         2.71368534024234e-10,
                                         1.47122381319338e-12,
                                         2.41719704308339e-15,
                                         7.14213386699384e-19,
                                         1.37834397468431e-23,
                                         1.56611629791452e-30,
                                         1.78170399489115e-43,
                                         0.0706506030834013,
                                         0.271972745921554,
                                         0.378213233213807,
                                         0.305109267627965,
                                         0.161125675195679,
                                         0.0578688568666064,
                                         0.0141965322889002,
                                         0.00234046888012865,
                                         0.000250631060994038,
                                         1.65213685841721e-05,
                                         6.19081052556317e-07,
                                         1.17227382765868e-08,
                                         9.38281666277912e-11,
                                         2.38733192598038e-13,
                                         1.18129036632823e-16,
                                         4.37215159228807e-21,
                                         1.24807127853282e-27,
                                         7.57227130158766e-40,
                                         0.00254341186170947,
                                         0.0522157399719873,
                                         0.182427602935917,
                                         0.282239733957709,
                                         0.249606234958903,
                                         0.138829528793113,
                                         0.0502810252928381,
                                         0.0118884322383836,
                                         0.00179738872813309,
                                         0.000166447298371814,
                                         8.80569518384412e-06,
                                         2.39136060274318e-07,
                                         2.82575259896498e-09,
                                         1.11342246130042e-11,
                                         9.22638895253231e-15,
                                         6.54906395423478e-19,
                                         4.69679050661421e-25,
                                         1.51971866710099e-36,
                                         5.74509631048575e-05,
                                         0.00629009623672865,
                                         0.0552108184573682,
                                         0.163817657298372,
                                         0.24261961591992,
                                         0.208977051809273,
                                         0.111739256554047,
                                         0.0378901308226268,
                                         0.0080877726987307,
                                         0.00105217331452529,
                                         7.85886060783171e-05,
                                         3.06084172749708e-06,
                                         5.33967523075823e-08,
                                         3.25826912140317e-10,
                                         4.52154292368153e-13,
                                         6.15521253983754e-17,
                                         1.10902871818021e-22,
                                         1.91372721384374e-33,
                                         9.12452939392764e-07,
                                         0.000532777263673279,
                                         0.0117487147260229,
                                         0.0668552947097436,
                                         0.165816953635178,
                                         0.221181035083378,
                                         0.174598283539256,
                                         0.0849102584075692,
                                         0.0255887104712667,
                                         0.00467660139822424,
                                         0.000493160249271645,
                                         2.75466769806571e-05,
                                         7.09460287410876e-07,
                                         6.70419236130298e-09,
                                         1.55802383321095e-11,
                                         4.06761102471505e-15,
                                         1.84126770054258e-20,
                                         1.69445253228656e-30,
                                         1.08205764847649e-08,
                                         3.3694641464393e-05,
                                         0.00186673768847745,
                                         0.0203721876504685,
                                         0.0846172129697244,
                                         0.174792960406734,
                                         0.203704642048353,
                                         0.142076085873684,
                                         0.0604497664420669,
                                         0.0155203028349431,
                                         0.00231069861845385,
                                         0.000185107631137972,
                                         7.03830530235759e-06,
                                         1.02998937508998e-07,
                                         4.00855917625881e-10,
                                         2.0070701011275e-13,
                                         2.28253707784911e-18,
                                         1.12022573437177e-27,
                                         9.9294317345663e-11,
                                         1.64895979439607e-06,
                                         0.000229514592634279,
                                         0.00480367479911093,
                                         0.033413546376107,
                                         0.106889257499983,
                                         0.183906022851585,
                                         0.18395681260651,
                                         0.110503217794704,
                                         0.0398569532721928,
                                         0.00837785069811697,
                                         0.000962528284447003,
                                         5.40308976605277e-05,
                                         1.22448393936718e-06,
                                         7.98061889789239e-09,
                                         7.66336899605994e-12,
                                         2.18953966803586e-16,
                                         5.73080662045809e-25,
                                         7.20924016532633e-13,
                                         6.38484557566276e-08,
                                         2.23268993777402e-05,
                                         0.000896191114453613,
                                         0.0104394479149264,
                                         0.0517172336846515,
                                         0.131365729876728,
                                         0.188452486538701,
                                         0.15982559361411,
                                         0.0809839784067091,
                                         0.024033279238042,
                                         0.00395998719634573,
                                         0.000328176416530218,
                                         1.15176677354504e-05,
                                         1.25711772352925e-07,
                                         2.31509084702885e-10,
                                         1.6618005508414e-14,
                                         2.31962118615367e-22,
                                         4.19830595610017e-15,
                                         1.9829429574313e-09,
                                         1.74207167313932e-06,
                                         0.000134105676329943,
                                         0.00261608579992603,
                                         0.0200703989535115,
                                         0.0752640710933156,
                                         0.154848627512909,
                                         0.185411735820453,
                                         0.131981667394024,
                                         0.0552984477369838,
                                         0.0130675314878088,
                                         0.00159879229356509,
                                         8.68951406328863e-05,
                                         1.58830834829076e-06,
                                         5.60965083973309e-09,
                                         1.01163645621724e-12,
                                         7.53074377725642e-20,
                                         1.97566574039433e-17,
                                         4.97650949410307e-11,
                                         1.09839466608393e-07,
                                         1.62161771563295e-05,
                                         0.000529762517831431,
                                         0.00629406924209294,
                                         0.0348456139775484,
                                         0.102817642332902,
                                         0.173813260259336,
                                         0.173813260259336,
                                         0.102817642332902,
                                         0.0348456139775484,
                                         0.00629406924209294,
                                         0.000529762517831431,
                                         1.62161771563296e-05,
                                         1.09839466608393e-07,
                                         4.97650949410307e-11,
                                         1.97566574039433e-17,
                                         7.53074377725642e-20,
                                         1.01163645621724e-12,
                                         5.60965083973309e-09,
                                         1.58830834829075e-06,
                                         8.68951406328862e-05,
                                         0.00159879229356509,
                                         0.0130675314878088,
                                         0.0552984477369838,
                                         0.131981667394024,
                                         0.185411735820453,
                                         0.15484862751291,
                                         0.0752640710933156,
                                         0.0200703989535115,
                                         0.00261608579992603,
                                         0.000134105676329943,
                                         1.74207167313932e-06,
                                         1.9829429574313e-09,
                                         4.19830595610017e-15,
                                         2.31962118615367e-22,
                                         1.6618005508414e-14,
                                         2.31509084702885e-10,
                                         1.25711772352924e-07,
                                         1.15176677354504e-05,
                                         0.000328176416530218,
                                         0.00395998719634573,
                                         0.024033279238042,
                                         0.0809839784067091,
                                         0.15982559361411,
                                         0.188452486538701,
                                         0.131365729876728,
                                         0.0517172336846515,
                                         0.0104394479149264,
                                         0.000896191114453616,
                                         2.23268993777402e-05,
                                         6.38484557566276e-08,
                                         7.20924016532633e-13,
                                         5.7308066204581e-25,
                                         2.18953966803586e-16,
                                         7.66336899605994e-12,
                                         7.98061889789233e-09,
                                         1.22448393936718e-06,
                                         5.40308976605277e-05,
                                         0.000962528284447003,
                                         0.00837785069811697,
                                         0.0398569532721928,
                                         0.110503217794704,
                                         0.18395681260651,
                                         0.183906022851585,
                                         0.106889257499983,
                                         0.033413546376107,
                                         0.00480367479911095,
                                         0.000229514592634279,
                                         1.64895979439607e-06,
                                         9.9294317345663e-11,
                                         1.12022573437177e-27,
                                         2.28253707784911e-18,
                                         2.0070701011275e-13,
                                         4.00855917625878e-10,
                                         1.02998937508998e-07,
                                         7.03830530235759e-06,
                                         0.000185107631137972,
                                         0.00231069861845385,
                                         0.0155203028349431,
                                         0.0604497664420669,
                                         0.142076085873684,
                                         0.203704642048353,
                                         0.174792960406734,
                                         0.0846172129697244,
                                         0.0203721876504686,
                                         0.00186673768847745,
                                         3.3694641464393e-05,
                                         1.08205764847649e-08,
                                         1.69445253228656e-30,
                                         1.84126770054258e-20,
                                         4.06761102471506e-15,
                                         1.55802383321094e-11,
                                         6.70419236130298e-09,
                                         7.09460287410876e-07,
                                         2.75466769806571e-05,
                                         0.000493160249271645,
                                         0.00467660139822424,
                                         0.0255887104712667,
                                         0.0849102584075692,
                                         0.174598283539256,
                                         0.221181035083378,
                                         0.165816953635178,
                                         0.0668552947097437,
                                         0.0117487147260229,
                                         0.000532777263673279,
                                         9.12452939392764e-07,
                                         1.91372721384374e-33,
                                         1.10902871818021e-22,
                                         6.15521253983754e-17,
                                         4.52154292368149e-13,
                                         3.25826912140317e-10,
                                         5.33967523075823e-08,
                                         3.06084172749708e-06,
                                         7.85886060783171e-05,
                                         0.00105217331452529,
                                         0.0080877726987307,
                                         0.0378901308226268,
                                         0.111739256554047,
                                         0.208977051809273,
                                         0.24261961591992,
                                         0.163817657298372,
                                         0.0552108184573682,
                                         0.00629009623672865,
                                         5.74509631048575e-05,
                                         1.51971866710099e-36,
                                         4.69679050661421e-25,
                                         6.54906395423478e-19,
                                         9.22638895253223e-15,
                                         1.11342246130042e-11,
                                         2.82575259896498e-09,
                                         2.39136060274318e-07,
                                         8.80569518384411e-06,
                                         0.000166447298371814,
                                         0.00179738872813309,
                                         0.0118884322383836,
                                         0.0502810252928381,
                                         0.138829528793113,
                                         0.249606234958903,
                                         0.28223973395771,
                                         0.182427602935917,
                                         0.0522157399719873,
                                         0.00254341186170947,
                                         7.57227130158766e-40,
                                         1.24807127853282e-27,
                                         4.37215159228807e-21,
                                         1.18129036632822e-16,
                                         2.38733192598038e-13,
                                         9.38281666277912e-11,
                                         1.17227382765868e-08,
                                         6.19081052556317e-07,
                                         1.65213685841721e-05,
                                         0.000250631060994038,
                                         0.00234046888012865,
                                         0.0141965322889002,
                                         0.0578688568666064,
                                         0.161125675195679,
                                         0.305109267627965,
                                         0.378213233213807,
                                         0.271972745921554,
                                         0.0706506030834013,
                                         1.78170399489115e-43,
                                         1.56611629791452e-30,
                                         1.37834397468431e-23,
                                         7.14213386699377e-19,
                                         2.41719704308339e-15,
                                         1.47122381319338e-12,
                                         2.71368534024234e-10,
                                         2.05531218646112e-08,
                                         7.7439352068359e-07,
                                         1.65034291101867e-05,
                                         0.000217584310280209,
                                         0.00189280922881392,
                                         0.0113908039695076,
                                         0.0491157173791647,
                                         0.155753948571053,
                                         0.370279193726227,
                                         0.668953231122606,
                                         0.926747610718249,

                                         0.933822911727506,
                                         0.696465352623922,
                                         0.409259410859441,
                                         0.188103303164778,
                                         0.0668901407351161,
                                         0.0181252317635129,
                                         0.00366645572566143,
                                         0.000538785913488983,
                                         5.54779547896097e-05,
                                         3.814697265625e-06,
                                         1.64115215291394e-07,
                                         4.03664225686568e-09,
                                         4.98996926639315e-11,
                                         2.55785325333104e-13,
                                         4.01128349663938e-16,
                                         1.14175014005864e-19,
                                         2.14178957835633e-24,
                                         2.3864591960472e-31,
                                         2.68572810282654e-44,
                                         0.064059174377825,
                                         0.254486024785802,
                                         0.374860745574026,
                                         0.329323766938694,
                                         0.19521767158869,
                                         0.0814245997267138,
                                         0.0241274095399717,
                                         0.00503436288670168,
                                         0.000722594290041443,
                                         6.866455078125e-05,
                                         4.08243965498132e-06,
                                         1.39970652743071e-07,
                                         2.45684993234582e-09,
                                         1.84479694752302e-11,
                                         4.45319126086132e-14,
                                         2.11295102727068e-17,
                                         7.57618413356563e-22,
                                         2.11609227498638e-28,
                                         1.26849932973599e-40,
                                         0.00207512646543127,
                                         0.0439111468616277,
                                         0.16213906906141,
                                         0.27226771554391,
                                         0.269043539562966,
                                         0.172732523064638,
                                         0.0749758447435142,
                                         0.0222136140661594,
                                         0.00444441934421955,
                                         0.000583648681640625,
                                         4.7955356244786e-05,
                                         2.29192358748154e-06,
                                         5.71223163204645e-08,
                                         6.28301271167855e-10,
                                         2.33456387034528e-12,
                                         1.84652082406609e-15,
                                         1.26552480574782e-19,
                                         8.86056898093767e-26,
                                         2.82920789435526e-37,
                                         4.21781574582015e-05,
                                         0.00475406838428545,
                                         0.0440032939733766,
                                         0.141237175121355,
                                         0.23265147464628,
                                         0.229917698423876,
                                         0.14618803933596,
                                         0.0614998037099168,
                                         0.0171520223983362,
                                         0.00311279296875,
                                         0.000353455115469383,
                                         2.35474065866501e-05,
                                         8.33321893316683e-07,
                                         1.34266326732108e-08,
                                         7.67927065151506e-11,
                                         1.01250876246863e-13,
                                         1.32638806501291e-17,
                                         2.32792198815373e-23,
                                         3.95930795591971e-34,
                                         6.02786029167987e-07,
                                         0.000361900021335889,
                                         0.00839682767461874,
                                         0.0515150870117891,
                                         0.141456070957624,
                                         0.215180657279669,
                                         0.200417137535681,
                                         0.119718364932939,
                                         0.0465423364060499,
                                         0.0116729736328125,
                                         0.0018317405563298,
                                         0.000170105584285122,
                                         8.54775988220183e-06,
                                         2.01743158861142e-07,
                                         1.77609719473029e-09,
                                         3.90369529111098e-12,
                                         9.77469214943206e-16,
                                         4.30038931043248e-21,
                                         3.89588603165687e-31,
                                         6.43228851329209e-09,
                                         2.05702020022744e-05,
                                         0.00119638802259804,
                                         0.0140296428805339,
                                         0.0642190883627796,
                                         0.15036986844971,
                                         0.205156193372607,
                                         0.17401015246811,
                                         0.0942991526466805,
                                         0.032684326171875,
                                         0.00708794623346992,
                                         0.000917531161724736,
                                         6.54664210317219e-05,
                                         2.26337903749045e-06,
                                         3.06718693729697e-08,
                                         1.12377610871766e-10,
                                         5.37851438970679e-14,
                                         5.93162775305967e-19,
                                         2.86233175385141e-28,
                                         5.31131335434024e-11,
                                         9.04737465474894e-07,
                                         0.000131905511770249,
                                         0.0029566019948698,
                                         0.0225600860482506,
                                         0.0813115950327528,
                                         0.162505655572216,
                                         0.195714259465265,
                                         0.147843239534816,
                                         0.0708160400390625,
                                         0.0212232035020564,
                                         0.0038296328996827,
                                         0.000387988591128023,
                                         1.96494243837166e-05,
                                         4.09871606645832e-07,
                                         2.50332795633464e-09,
                                         2.29010635550352e-12,
                                         6.33102566271302e-17,
                                         1.62730052249688e-25,
                                         3.46999969865843e-13,
                                         3.14845650111668e-08,
                                         1.15065448362615e-05,
                                         0.00049298104256298,
                                         0.00627059095317876,
                                         0.0347884632925829,
                                         0.101845872233622,
                                         0.174165228681827,
                                         0.183394477721953,
                                         0.12139892578125,
                                         0.0502796871080832,
                                         0.01264691419549,
                                         0.00181932553655805,
                                         0.000134968828986194,
                                         4.333577088159e-06,
                                         4.41211484298761e-08,
                                         7.71507359572007e-11,
                                         5.34644820034861e-15,
                                         7.31992554021935e-23,
                                         1.81834564948725e-15,
                                         8.78804581051166e-10,
                                         8.05093679249715e-07,
                                         6.5930605520043e-05,
                                         0.00139796302055748,
                                         0.0119381629356946,
                                         0.0511962175637283,
                                         0.124313968842024,
                                         0.182469604396281,
                                         0.166923522949219,
                                         0.0955418588340007,
                                         0.0334989728455846,
                                         0.00684260338238068,
                                         0.000743595271358729,
                                         3.67506234406038e-05,
                                         6.23728170143111e-07,
                                         2.0847020171382e-09,
                                         3.62139706387875e-13,
                                         2.64097965104384e-20,
                                         7.69977862344903e-18,
                                         1.98217374428198e-11,
                                         4.55200460130858e-08,
                                         7.12522695652772e-06,
                                         0.000251847525281893,
                                         0.00331050731028831,
                                         0.0207963497350983,
                                         0.0717023190093307,
                                         0.146706582046309,
                                         0.185470581054688,
                                         0.146706582046309,
                                         0.0717023190093308,
                                         0.0207963497350983,
                                         0.0033105073102883,
                                         0.000251847525281894,
                                         7.12522695652772e-06,
                                         4.55200460130858e-08,
                                         1.98217374428198e-11,
                                         7.69977862344903e-18,
                                         2.64097965104385e-20,
                                         3.62139706387875e-13,
                                         2.0847020171382e-09,
                                         6.23728170143111e-07,
                                         3.67506234406037e-05,
                                         0.000743595271358731,
                                         0.00684260338238068,
                                         0.0334989728455846,
                                         0.0955418588340007,
                                         0.166923522949219,
                                         0.182469604396281,
                                         0.124313968842024,
                                         0.0511962175637283,
                                         0.0119381629356946,
                                         0.00139796302055749,
                                         6.5930605520043e-05,
                                         8.05093679249715e-07,
                                         8.78804581051166e-10,
                                         1.81834564948725e-15,
                                         7.31992554021935e-23,
                                         5.34644820034861e-15,
                                         7.71507359572007e-11,
                                         4.41211484298761e-08,
                                         4.33357708815898e-06,
                                         0.000134968828986195,
                                         0.00181932553655805,
                                         0.01264691419549,
                                         0.0502796871080832,
                                         0.12139892578125,
                                         0.183394477721953,
                                         0.174165228681827,
                                         0.101845872233622,
                                         0.0347884632925829,
                                         0.00627059095317878,
                                         0.00049298104256298,
                                         1.15065448362615e-05,
                                         3.14845650111668e-08,
                                         3.46999969865843e-13,
                                         1.62730052249688e-25,
                                         6.33102566271302e-17,
                                         2.29010635550352e-12,
                                         2.50332795633464e-09,
                                         4.0987160664583e-07,
                                         1.96494243837167e-05,
                                         0.000387988591128023,
                                         0.00382963289968269,
                                         0.0212232035020564,
                                         0.0708160400390625,
                                         0.147843239534816,
                                         0.195714259465266,
                                         0.162505655572216,
                                         0.0813115950327526,
                                         0.0225600860482507,
                                         0.0029566019948698,
                                         0.000131905511770249,
                                         9.04737465474894e-07,
                                         5.31131335434024e-11,
                                         2.86233175385141e-28,
                                         5.93162775305967e-19,
                                         5.37851438970679e-14,
                                         1.12377610871766e-10,
                                         3.06718693729696e-08,
                                         2.26337903749046e-06,
                                         6.5466421031722e-05,
                                         0.000917531161724734,
                                         0.00708794623346992,
                                         0.032684326171875,
                                         0.0942991526466805,
                                         0.17401015246811,
                                         0.205156193372607,
                                         0.15036986844971,
                                         0.0642190883627797,
                                         0.0140296428805339,
                                         0.00119638802259804,
                                         2.05702020022744e-05,
                                         6.43228851329209e-09,
                                         3.89588603165687e-31,
                                         4.30038931043248e-21,
                                         9.77469214943206e-16,
                                         3.90369529111098e-12,
                                         1.77609719473028e-09,
                                         2.01743158861143e-07,
                                         8.54775988220183e-06,
                                         0.000170105584285122,
                                         0.0018317405563298,
                                         0.0116729736328125,
                                         0.0465423364060499,
                                         0.119718364932939,
                                         0.200417137535681,
                                         0.215180657279669,
                                         0.141456070957625,
                                         0.0515150870117891,
                                         0.00839682767461874,
                                         0.000361900021335889,
                                         6.02786029167987e-07,
                                         3.95930795591971e-34,
                                         2.32792198815373e-23,
                                         1.32638806501291e-17,
                                         1.01250876246863e-13,
                                         7.67927065151501e-11,
                                         1.34266326732109e-08,
                                         8.33321893316683e-07,
                                         2.35474065866501e-05,
                                         0.000353455115469383,
                                         0.00311279296875,
                                         0.0171520223983362,
                                         0.0614998037099169,
                                         0.14618803933596,
                                         0.229917698423876,
                                         0.232651474646281,
                                         0.141237175121355,
                                         0.0440032939733766,
                                         0.00475406838428545,
                                         4.21781574582015e-05,
                                         2.82920789435526e-37,
                                         8.86056898093767e-26,
                                         1.26552480574782e-19,
                                         1.84652082406609e-15,
                                         2.33456387034527e-12,
                                         6.28301271167858e-10,
                                         5.71223163204645e-08,
                                         2.29192358748153e-06,
                                         4.7955356244786e-05,
                                         0.000583648681640625,
                                         0.00444441934421955,
                                         0.0222136140661594,
                                         0.0749758447435142,
                                         0.172732523064638,
                                         0.269043539562966,
                                         0.27226771554391,
                                         0.16213906906141,
                                         0.0439111468616277,
                                         0.00207512646543127,
                                         1.26849932973599e-40,
                                         2.11609227498638e-28,
                                         7.57618413356563e-22,
                                         2.11295102727068e-17,
                                         4.45319126086129e-14,
                                         1.84479694752302e-11,
                                         2.45684993234582e-09,
                                         1.39970652743071e-07,
                                         4.08243965498132e-06,
                                         6.866455078125e-05,
                                         0.000722594290041443,
                                         0.00503436288670168,
                                         0.0241274095399717,
                                         0.0814245997267138,
                                         0.19521767158869,
                                         0.329323766938694,
                                         0.374860745574026,
                                         0.254486024785802,
                                         0.064059174377825,
                                         2.68572810282654e-44,
                                         2.3864591960472e-31,
                                         2.14178957835633e-24,
                                         1.14175014005864e-19,
                                         4.01128349663935e-16,
                                         2.55785325333105e-13,
                                         4.98996926639315e-11,
                                         4.03664225686566e-09,
                                         1.64115215291394e-07,
                                         3.814697265625e-06,
                                         5.54779547896097e-05,
                                         0.000538785913488983,
                                         0.00366645572566143,
                                         0.0181252317635129,
                                         0.0668901407351161,
                                         0.188103303164778,
                                         0.409259410859441,
                                         0.696465352623922,
                                         0.933822911727506,

                                         0.939930739675172,
                                         0.720933226177576,
                                         0.445862013539977,
                                         0.221018393263697,
                                         0.0870446889891999,
                                         0.0269039448809168,
                                         0.006419960085286,
                                         0.0011575743512531,
                                         0.000153344186436921,
                                         1.4384848127394e-05,
                                         9.10122336413426e-07,
                                         3.63696953053841e-08,
                                         8.38431315072571e-10,
                                         9.79817969283871e-12,
                                         4.78775640590375e-14,
                                         7.21514871701228e-17,
                                         1.98904338053609e-20,
                                         3.64162677054151e-25,
                                         3.99023644037912e-32,
                                         4.44913060953237e-45,
                                         0.0583281657578845,
                                         0.238052804174695,
                                         0.368346062230685,
                                         0.348018076527432,
                                         0.227590347565432,
                                         0.107727914381128,
                                         0.0374115377175568,
                                         0.00949960980160269,
                                         0.00173602090741625,
                                         0.000222114791120341,
                                         1.90973269843087e-05,
                                         1.04087130843356e-06,
                                         3.31021063292876e-08,
                                         5.44775011807558e-10,
                                         3.87404596114255e-12,
                                         8.94084850379875e-15,
                                         4.0927562493578e-18,
                                         1.42818637241288e-22,
                                         3.91530303122302e-29,
                                         2.32293843087135e-41,
                                         0.00170925636710965,
                                         0.0371191465528288,
                                         0.143700411069473,
                                         0.258774577363258,
                                         0.281003559273086,
                                         0.203698091488774,
                                         0.102949729820837,
                                         0.0368136652601113,
                                         0.00928087679336009,
                                         0.001619556486639,
                                         0.000189230795838457,
                                         1.4066978792824e-05,
                                         6.17149226031796e-07,
                                         1.43032713657353e-08,
                                         1.48028013902731e-10,
                                         5.23189001654001e-13,
                                         3.97680161184968e-16,
                                         2.6449700404638e-20,
                                         1.81417238288328e-26,
                                         5.72725689217825e-38,
                                         3.14279385649029e-05,
                                         0.00363163727631114,
                                         0.0351754605026888,
                                         0.120731672165116,
                                         0.217695576715287,
                                         0.241671518121017,
                                         0.177756162276269,
                                         0.0895142450780929,
                                         0.0311317020721154,
                                         0.00740959587377403,
                                         0.00117649706965251,
                                         0.000119284614201369,
                                         7.21945726903858e-06,
                                         2.35631517400537e-07,
                                         3.54897432829685e-09,
                                         1.92095980527959e-11,
                                         2.42455351427646e-14,
                                         3.07352274765279e-18,
                                         5.27438131734639e-24,
                                         8.86003485202547e-35,
                                         4.06309638187583e-07,
                                         0.000249827066149546,
                                         0.00605416304533171,
                                         0.0396053051120895,
                                         0.118582316124606,
                                         0.20160278173373,
                                         0.215802593304809,
                                         0.153040997432546,
                                         0.0734258975082365,
                                         0.0238355667889549,
                                         0.00514306986002191,
                                         0.000711214443194861,
                                         5.93815393360464e-05,
                                         2.72937947613394e-06,
                                         5.9826602906075e-08,
                                         4.95918705054877e-10,
                                         1.03935081772648e-12,
                                         2.51121933278071e-16,
                                         1.07819431401446e-21,
                                         9.63733014328198e-32,
                                         3.92215829136401e-09,
                                         1.28322636541134e-05,
                                         0.000778027947406827,
                                         0.0097009055131871,
                                         0.0482299726949149,
                                         0.125572440225294,
                                         0.19562097396944,
                                         0.195366540075503,
                                         0.129307115982915,
                                         0.0572510174003786,
                                         0.0167872964919631,
                                         0.00316623735652212,
                                         0.000364691059398205,
                                         2.36059360698837e-05,
                                         7.53030544280985e-07,
                                         9.55937424992994e-09,
                                         3.32674347355538e-11,
                                         1.53200302968427e-14,
                                         1.64569486338788e-19,
                                         7.82717072809289e-29,
                                         2.92972718014192e-11,
                                         5.10036352189793e-07,
                                         7.736960020985e-05,
                                         0.00183867620767863,
                                         0.0151791752125904,
                                         0.0605238042703448,
                                         0.137217102676864,
                                         0.192986377241453,
                                         0.176209637511835,
                                         0.106408177473755,
                                         0.0424007138932406,
                                         0.0109073799098208,
                                         0.00173313683266854,
                                         0.000157983816945875,
                                         7.33440540978978e-06,
                                         1.42587846669615e-07,
                                         8.23968360546985e-10,
                                         7.2321707759501e-13,
                                         1.94372875967138e-17,
                                         4.91911468122926e-26,
                                         1.731491434376e-13,
                                         1.6039472894448e-08,
                                         6.08746689039741e-06,
                                         0.000275733378568101,
                                         0.00377981405983386,
                                         0.023080712241149,
                                         0.0761539048914605,
                                         0.150832255712261,
                                         0.189988797604798,
                                         0.156479663970804,
                                         0.0847337995061743,
                                         0.0297295623339789,
                                         0.00651675946858462,
                                         0.000836556122717711,
                                         5.65208046093041e-05,
                                         1.68277736000485e-06,
                                         1.61470377928628e-08,
                                         2.70127516289171e-11,
                                         1.81640676909329e-15,
                                         2.44602106231265e-23,
                                         8.20791795863077e-16,
                                         4.04574536315189e-10,
                                         3.84169041114879e-07,
                                         3.31659865527413e-05,
                                         0.00075493957165364,
                                         0.00705978844235652,
                                         0.033899680234161,
                                         0.0945543014707648,
                                         0.164303124653672,
                                         0.184569653538946,
                                         0.13581873619214,
                                         0.0649944216463184,
                                         0.0196539579886697,
                                         0.00355301510721159,
                                         0.00034935844956337,
                                         1.592906566244e-05,
                                         2.5380178960178e-07,
                                         8.09260737171373e-10,
                                         1.36147620342765e-13,
                                         9.75557628515546e-21,
                                         3.14412974058687e-18,
                                         8.24635051121484e-12,
                                         1.95912843247991e-08,
                                         3.2236752932467e-06,
                                         0.000121845287491993,
                                         0.00174497399763678,
                                         0.0121942141715346,
                                         0.0478986361407218,
                                         0.114820238255382,
                                         0.175920917569149,
                                         0.175920917569149,
                                         0.114820238255382,
                                         0.0478986361407218,
                                         0.0121942141715346,
                                         0.00174497399763678,
                                         0.000121845287491993,
                                         3.22367529324672e-06,
                                         1.95912843247991e-08,
                                         8.24635051121506e-12,
                                         3.14412974058733e-18,
                                         9.75557628515389e-21,
                                         1.3614762034276e-13,
                                         8.09260737171373e-10,
                                         2.53801789601778e-07,
                                         1.592906566244e-05,
                                         0.000349358449563371,
                                         0.00355301510721159,
                                         0.0196539579886697,
                                         0.0649944216463184,
                                         0.13581873619214,
                                         0.184569653538946,
                                         0.164303124653672,
                                         0.0945543014707648,
                                         0.033899680234161,
                                         0.00705978844235651,
                                         0.00075493957165364,
                                         3.31659865527414e-05,
                                         3.84169041114879e-07,
                                         4.04574536315199e-10,
                                         8.20791795863183e-16,
                                         2.44602106231222e-23,
                                         1.81640676909323e-15,
                                         2.70127516289171e-11,
                                         1.61470377928627e-08,
                                         1.68277736000485e-06,
                                         5.65208046093043e-05,
                                         0.000836556122717711,
                                         0.00651675946858462,
                                         0.0297295623339789,
                                         0.0847337995061743,
                                         0.156479663970804,
                                         0.189988797604798,
                                         0.150832255712261,
                                         0.0761539048914605,
                                         0.023080712241149,
                                         0.00377981405983386,
                                         0.000275733378568102,
                                         6.08746689039741e-06,
                                         1.60394728944483e-08,
                                         1.7314914343762e-13,
                                         4.91911468122831e-26,
                                         1.94372875967131e-17,
                                         7.2321707759501e-13,
                                         8.23968360546978e-10,
                                         1.42587846669615e-07,
                                         7.33440540978981e-06,
                                         0.000157983816945875,
                                         0.00173313683266854,
                                         0.0109073799098208,
                                         0.0424007138932406,
                                         0.106408177473755,
                                         0.176209637511835,
                                         0.192986377241453,
                                         0.137217102676864,
                                         0.0605238042703447,
                                         0.0151791752125904,
                                         0.00183867620767864,
                                         7.736960020985e-05,
                                         5.10036352189802e-07,
                                         2.92972718014221e-11,
                                         7.82717072809125e-29,
                                         1.64569486338781e-19,
                                         1.53200302968427e-14,
                                         3.32674347355535e-11,
                                         9.55937424992994e-09,
                                         7.53030544280988e-07,
                                         2.36059360698837e-05,
                                         0.000364691059398205,
                                         0.00316623735652212,
                                         0.0167872964919631,
                                         0.0572510174003786,
                                         0.129307115982915,
                                         0.195366540075503,
                                         0.19562097396944,
                                         0.125572440225294,
                                         0.0482299726949149,
                                         0.00970090551318714,
                                         0.000778027947406827,
                                         1.28322636541136e-05,
                                         3.92215829136433e-09,
                                         9.6373301432798e-32,
                                         1.07819431401442e-21,
                                         2.51121933278071e-16,
                                         1.03935081772647e-12,
                                         4.95918705054877e-10,
                                         5.98266029060753e-08,
                                         2.72937947613394e-06,
                                         5.93815393360464e-05,
                                         0.000711214443194861,
                                         0.00514306986002191,
                                         0.0238355667889549,
                                         0.0734258975082365,
                                         0.153040997432546,
                                         0.215802593304809,
                                         0.20160278173373,
                                         0.118582316124606,
                                         0.0396053051120896,
                                         0.00605416304533171,
                                         0.000249827066149549,
                                         4.06309638187609e-07,
                                         8.86003485202333e-35,
                                         5.27438131734615e-24,
                                         3.07352274765279e-18,
                                         2.42455351427643e-14,
                                         1.92095980527959e-11,
                                         3.54897432829686e-09,
                                         2.35631517400537e-07,
                                         7.21945726903859e-06,
                                         0.000119284614201369,
                                         0.00117649706965251,
                                         0.00740959587377403,
                                         0.0311317020721154,
                                         0.0895142450780929,
                                         0.177756162276269,
                                         0.241671518121017,
                                         0.217695576715287,
                                         0.120731672165116,
                                         0.0351754605026888,
                                         0.00363163727631117,
                                         3.14279385649045e-05,
                                         5.72725689217677e-38,
                                         1.81417238288319e-26,
                                         2.6449700404638e-20,
                                         3.97680161184964e-16,
                                         5.23189001654001e-13,
                                         1.48028013902732e-10,
                                         1.43032713657353e-08,
                                         6.17149226031796e-07,
                                         1.4066978792824e-05,
                                         0.000189230795838457,
                                         0.001619556486639,
                                         0.00928087679336009,
                                         0.0368136652601113,
                                         0.102949729820837,
                                         0.203698091488774,
                                         0.281003559273086,
                                         0.258774577363258,
                                         0.143700411069473,
                                         0.037119146552829,
                                         0.0017092563671097,
                                         2.32293843087071e-41,
                                         3.91530303122282e-29,
                                         1.42818637241288e-22,
                                         4.09275624935775e-18,
                                         8.94084850379875e-15,
                                         3.87404596114257e-12,
                                         5.44775011807558e-10,
                                         3.31021063292876e-08,
                                         1.04087130843356e-06,
                                         1.90973269843087e-05,
                                         0.000222114791120341,
                                         0.00173602090741625,
                                         0.00949960980160269,
                                         0.0374115377175568,
                                         0.107727914381128,
                                         0.227590347565432,
                                         0.348018076527432,
                                         0.368346062230685,
                                         0.238052804174696,
                                         0.0583281657578854,
                                         4.44913060953107e-45,
                                         3.9902364403789e-32,
                                         3.64162677054151e-25,
                                         1.98904338053606e-20,
                                         7.21514871701228e-17,
                                         4.78775640590377e-14,
                                         9.79817969283871e-12,
                                         8.38431315072571e-10,
                                         3.63696953053841e-08,
                                         9.10122336413426e-07,
                                         1.4384848127394e-05,
                                         0.000153344186436921,
                                         0.0011575743512531,
                                         0.006419960085286,
                                         0.0269039448809168,
                                         0.0870446889891999,
                                         0.221018393263697,
                                         0.445862013539977,
                                         0.720933226177576,
                                         0.939930739675172,

                                         0.945238051041064,
                                         0.742745470695577,
                                         0.480050166118802,
                                         0.253951283061533,
                                         0.109165610734605,
                                         0.0377530255656753,
                                         0.0103655142615361,
                                         0.00222084265123479,
                                         0.000363140753327478,
                                         4.4034580664613e-05,
                                         3.814697265625e-06,
                                         2.24756053915084e-07,
                                         8.43074425514821e-09,
                                         1.83831633590437e-10,
                                         2.04697510708569e-12,
                                         9.59836190536102e-15,
                                         1.39767129935688e-17,
                                         3.7482180437827e-21,
                                         6.71996871505952e-26,
                                         7.25755186641938e-33,
                                         8.02758616711444e-46,
                                         0.0533177346486815,
                                         0.222728818702904,
                                         0.359571971186242,
                                         0.361663623847864,
                                         0.257295175384984,
                                         0.135679950379482,
                                         0.0539169945523616,
                                         0.0161562450262843,
                                         0.00361319529105638,
                                         0.000591191873006118,
                                         6.866455078125e-05,
                                         5.42402669636428e-06,
                                         2.74532952069125e-07,
                                         8.18733602532736e-09,
                                         1.27503688747852e-10,
                                         8.6532294012325e-13,
                                         1.92134017321547e-15,
                                         8.52737651347391e-19,
                                         2.90678596637149e-23,
                                         7.84149295972762e-30,
                                         4.61104640852759e-42,
                                         0.00142019703760562,
                                         0.031539821844415,
                                         0.127183700845582,
                                         0.243223575430617,
                                         0.286367585126808,
                                         0.230263981941014,
                                         0.13243626327643,
                                         0.0555021099320059,
                                         0.0169767401604133,
                                         0.00374808530046616,
                                         0.000583648681640625,
                                         6.18128344705716e-05,
                                         4.2215259720332e-06,
                                         1.72191379083747e-07,
                                         3.75041526717726e-09,
                                         3.68388219882097e-11,
                                         1.24723962616077e-13,
                                         9.16120184017826e-17,
                                         5.9375226455504e-21,
                                         4.0008639754246e-27,
                                         1.25072096934327e-38,
                                         2.37358789312391e-05,
                                         0.00280234667436802,
                                         0.0282264907224005,
                                         0.102632831206107,
                                         0.199984286990115,
                                         0.245197561830453,
                                         0.204111729935022,
                                         0.11963502859484,
                                         0.050049180384295,
                                         0.0149097471798431,
                                         0.00311279296875,
                                         0.000441992908747514,
                                         4.07309231375188e-05,
                                         2.27227031957677e-06,
                                         6.92174775892243e-08,
                                         9.84040681812435e-10,
                                         5.08013600743868e-12,
                                         6.17545933834501e-15,
                                         7.60987118659511e-19,
                                         1.28082152519129e-24,
                                         2.12863486582504e-35,
                                         2.78929580471641e-07,
                                         0.000175072133558271,
                                         0.00440468501395643,
                                         0.0304508549273379,
                                         0.0981974998221897,
                                         0.183585689070867,
                                         0.221187993212119,
                                         0.181317516520709,
                                         0.103746195075384,
                                         0.0417026479921279,
                                         0.0116729736328125,
                                         0.00222220686132958,
                                         0.000276319529426861,
                                         2.10834232337575e-05,
                                         8.98224133248504e-07,
                                         1.8482167937931e-08,
                                         1.45490058490231e-10,
                                         2.92697235874225e-13,
                                         6.85775351947127e-17,
                                         2.88307532315825e-22,
                                         2.54726689443136e-32,
                                         2.44743176684735e-09,
                                         8.16655614535962e-06,
                                         0.000513215293499416,
                                         0.0067458927875747,
                                         0.0360024246919033,
                                         0.102633293136147,
                                         0.178970683555172,
                                         0.205186096666309,
                                         0.160573601927446,
                                         0.0870930999257209,
                                         0.032684326171875,
                                         0.00834219678675815,
                                         0.00139967005633667,
                                         0.000146065961487928,
                                         8.70322945637751e-06,
                                         2.59190771654915e-07,
                                         3.1111293003594e-09,
                                         1.03584491730819e-11,
                                         4.61437835519581e-15,
                                         4.84562859809535e-20,
                                         2.27601172435781e-29,
                                         1.66173084930672e-11,
                                         2.94777920336773e-07,
                                         4.62720129709926e-05,
                                         0.00115641429000641,
                                         0.0102140302879412,
                                         0.044398855467599,
                                         0.112056309676978,
                                         0.179676040377186,
                                         0.192313685877165,
                                         0.140746607598865,
                                         0.0708160400390625,
                                         0.0242331821695869,
                                         0.00548622775998829,
                                         0.000783052692558964,
                                         6.52544627574102e-05,
                                         2.81267936972977e-06,
                                         5.14798027438925e-08,
                                         2.83664460837225e-10,
                                         2.4025841184706e-13,
                                         6.30199899051467e-18,
                                         1.57365167754255e-26,
                                         8.92692479834471e-14,
                                         8.41864222051761e-09,
                                         3.30086908748648e-06,
                                         0.0001568478495827,
                                         0.00229273381419444,
                                         0.0151965985103195,
                                         0.055511356361195,
                                         0.124486859464823,
                                         0.182237327524479,
                                         0.179963001176779,
                                         0.12139892578125,
                                         0.0556969697912303,
                                         0.0170142613306388,
                                         0.00332142199089945,
                                         0.000387107073728006,
                                         2.41497158412612e-05,
                                         6.73979661854491e-07,
                                         6.14619338930918e-09,
                                         9.89771869586037e-12,
                                         6.4848157150995e-16,
                                         8.6086257407541e-24,
                                         3.8464718829656e-16,
                                         1.92845117458133e-10,
                                         1.88867703749654e-07,
                                         1.70633060360185e-05,
                                         0.000412790432805513,
                                         0.00417196346631073,
                                         0.0220570208772695,
                                         0.0691793290226416,
                                         0.138510908036331,
                                         0.1845644320026,
                                         0.166923522949219,
                                         0.102676765903836,
                                         0.042322552944275,
                                         0.0112999525138063,
                                         0.00184192307430851,
                                         0.000166311895869441,
                                         7.07744007305707e-06,
                                         1.06813711574557e-07,
                                         3.27047702966473e-10,
                                         5.35225061292956e-14,
                                         3.77727442109094e-21,
                                         1.33930052938894e-18,
                                         3.56968615371109e-12,
                                         8.73256631321307e-09,
                                         1.50003911448577e-06,
                                         6.00565530897965e-05,
                                         0.000925527646358047,
                                         0.0070821751300902,
                                         0.0310659023291133,
                                         0.0850717579366306,
                                         0.152956376331218,
                                         0.185470581054688,
                                         0.152956376331218,
                                         0.0850717579366306,
                                         0.0310659023291134,
                                         0.0070821751300902,
                                         0.000925527646358046,
                                         6.00565530897965e-05,
                                         1.50003911448577e-06,
                                         8.73256631321296e-09,
                                         3.5696861537112e-12,
                                         1.33930052938872e-18,
                                         3.77727442109161e-21,
                                         5.35225061292938e-14,
                                         3.27047702966478e-10,
                                         1.06813711574557e-07,
                                         7.07744007305707e-06,
                                         0.000166311895869441,
                                         0.00184192307430851,
                                         0.0112999525138062,
                                         0.042322552944275,
                                         0.102676765903836,
                                         0.166923522949219,
                                         0.1845644320026,
                                         0.138510908036331,
                                         0.0691793290226417,
                                         0.0220570208772695,
                                         0.00417196346631073,
                                         0.000412790432805513,
                                         1.70633060360185e-05,
                                         1.88867703749652e-07,
                                         1.92845117458139e-10,
                                         3.84647188296505e-16,
                                         8.60862574075578e-24,
                                         6.48481571509925e-16,
                                         9.89771869586053e-12,
                                         6.14619338930918e-09,
                                         6.73979661854491e-07,
                                         2.41497158412612e-05,
                                         0.000387107073728006,
                                         0.00332142199089944,
                                         0.0170142613306388,
                                         0.0556969697912303,
                                         0.12139892578125,
                                         0.179963001176779,
                                         0.182237327524479,
                                         0.124486859464823,
                                         0.055511356361195,
                                         0.0151965985103195,
                                         0.00229273381419444,
                                         0.0001568478495827,
                                         3.30086908748645e-06,
                                         8.41864222051781e-09,
                                         8.9269247983436e-14,
                                         1.57365167754289e-26,
                                         6.30199899051441e-18,
                                         2.40258411847064e-13,
                                         2.83664460837225e-10,
                                         5.14798027438925e-08,
                                         2.81267936972977e-06,
                                         6.52544627574102e-05,
                                         0.000783052692558962,
                                         0.00548622775998829,
                                         0.0242331821695869,
                                         0.0708160400390625,
                                         0.140746607598865,
                                         0.192313685877165,
                                         0.179676040377186,
                                         0.112056309676978,
                                         0.044398855467599,
                                         0.0102140302879412,
                                         0.00115641429000641,
                                         4.62720129709922e-05,
                                         2.94777920336779e-07,
                                         1.66173084930654e-11,
                                         2.27601172435833e-29,
                                         4.84562859809514e-20,
                                         4.61437835519589e-15,
                                         1.03584491730819e-11,
                                         3.1111293003594e-09,
                                         2.59190771654915e-07,
                                         8.70322945637751e-06,
                                         0.000146065961487928,
                                         0.00139967005633667,
                                         0.00834219678675815,
                                         0.032684326171875,
                                         0.0870930999257209,
                                         0.160573601927446,
                                         0.205186096666309,
                                         0.178970683555172,
                                         0.102633293136147,
                                         0.0360024246919033,
                                         0.0067458927875747,
                                         0.000513215293499413,
                                         8.16655614535976e-06,
                                         2.44743176684714e-09,
                                         2.54726689443199e-32,
                                         2.88307532315812e-22,
                                         6.8577535194714e-17,
                                         2.92697235874225e-13,
                                         1.45490058490231e-10,
                                         1.8482167937931e-08,
                                         8.98224133248504e-07,
                                         2.10834232337574e-05,
                                         0.000276319529426861,
                                         0.00222220686132958,
                                         0.0116729736328125,
                                         0.0417026479921279,
                                         0.103746195075384,
                                         0.181317516520709,
                                         0.221187993212119,
                                         0.183585689070867,
                                         0.0981974998221897,
                                         0.0304508549273379,
                                         0.0044046850139564,
                                         0.000175072133558273,
                                         2.78929580471621e-07,
                                         2.1286348658256e-35,
                                         1.28082152519123e-24,
                                         7.60987118659527e-19,
                                         6.17545933834501e-15,
                                         5.08013600743868e-12,
                                         9.84040681812435e-10,
                                         6.92174775892242e-08,
                                         2.27227031957676e-06,
                                         4.07309231375188e-05,
                                         0.000441992908747514,
                                         0.00311279296875,
                                         0.0149097471798431,
                                         0.050049180384295,
                                         0.11963502859484,
                                         0.204111729935022,
                                         0.245197561830453,
                                         0.199984286990115,
                                         0.102632831206107,
                                         0.0282264907224004,
                                         0.00280234667436805,
                                         2.37358789312378e-05,
                                         1.25072096934362e-38,
                                         4.00086397542439e-27,
                                         5.93752264555053e-21,
                                         9.16120184017826e-17,
                                         1.24723962616077e-13,
                                         3.68388219882097e-11,
                                         3.75041526717726e-09,
                                         1.72191379083746e-07,
                                         4.2215259720332e-06,
                                         6.18128344705716e-05,
                                         0.000583648681640625,
                                         0.00374808530046616,
                                         0.0169767401604133,
                                         0.0555021099320059,
                                         0.13243626327643,
                                         0.230263981941014,
                                         0.286367585126808,
                                         0.243223575430617,
                                         0.127183700845581,
                                         0.0315398218444152,
                                         0.00142019703760557,
                                         4.61104640852899e-42,
                                         7.84149295972717e-30,
                                         2.90678596637156e-23,
                                         8.52737651347391e-19,
                                         1.92134017321547e-15,
                                         8.6532294012325e-13,
                                         1.27503688747852e-10,
                                         8.18733602532733e-09,
                                         2.74532952069125e-07,
                                         5.42402669636428e-06,
                                         6.866455078125e-05,
                                         0.000591191873006118,
                                         0.00361319529105638,
                                         0.0161562450262843,
                                         0.0539169945523616,
                                         0.135679950379482,
                                         0.257295175384984,
                                         0.361663623847864,
                                         0.359571971186242,
                                         0.222728818702905,
                                         0.0533177346486805,
                                         8.02758616711701e-46,
                                         7.25755186641893e-33,
                                         6.71996871505969e-26,
                                         3.7482180437827e-21,
                                         1.39767129935688e-17,
                                         9.59836190536102e-15,
                                         2.04697510708569e-12,
                                         1.83831633590436e-10,
                                         8.43074425514821e-09,
                                         2.24756053915084e-07,
                                         3.814697265625e-06,
                                         4.4034580664613e-05,
                                         0.000363140753327478,
                                         0.00222084265123479,
                                         0.0103655142615361,
                                         0.0377530255656753,
                                         0.109165610734605,
                                         0.253951283061533,
                                         0.480050166118802,
                                         0.742745470695577,
                                         0.945238051041064,

                                         0.949877631438368,
                                         0.7622408348722,
                                         0.511865167460691,
                                         0.286473446588884,
                                         0.132823327544138,
                                         0.0505998973410627,
                                         0.0156674618385117,
                                         0.00388817469057463,
                                         0.000759681418020396,
                                         0.000114225120704887,
                                         1.28369841276479e-05,
                                         1.03835159662647e-06,
                                         5.75295344817087e-08,
                                         2.04309517962399e-09,
                                         4.24542437553124e-11,
                                         4.53346206205159e-13,
                                         2.05110479640651e-15,
                                         2.89906374710339e-18,
                                         7.59053981051144e-22,
                                         1.33628562690164e-26,
                                         1.4251560500717e-33,
                                         1.56541585982106e-46,
                                         0.0489145495457261,
                                         0.208511363337212,
                                         0.349249719748097,
                                         0.370852039776481,
                                         0.283749319349462,
                                         0.164215124061326,
                                         0.0732478891917326,
                                         0.0252752047174798,
                                         0.00670564432151035,
                                         0.00134840106919092,
                                         0.000200938083376506,
                                         2.14926612617626e-05,
                                         1.57898646568466e-06,
                                         7.4993668540703e-08,
                                         2.11600751830609e-09,
                                         3.14179994297262e-11,
                                         2.047714206813e-13,
                                         4.39685661377287e-16,
                                         1.89977152382534e-19,
                                         6.34547033008888e-24,
                                         1.68799017422399e-30,
                                         9.84927677831163e-43,
                                         0.00118947379019085,
                                         0.0269347944274123,
                                         0.112528615740793,
                                         0.226706196968297,
                                         0.286247512482946,
                                         0.251665156226664,
                                         0.161710419246713,
                                         0.0775871886049561,
                                         0.0279509095632998,
                                         0.00751662651082192,
                                         0.00148527861230675,
                                         0.000210078870400404,
                                         2.04650295335293e-05,
                                         1.29989124689338e-06,
                                         4.98034839426715e-08,
                                         1.02819027030701e-09,
                                         9.65377688190979e-12,
                                         3.14900435742881e-14,
                                         2.24531100767092e-17,
                                         1.42290124392078e-21,
                                         9.44112675284216e-28,
                                         2.92634398358125e-39,
                                         1.81489497229094e-05,
                                         0.00218311897716933,
                                         0.0227493864831223,
                                         0.0869572733417962,
                                         0.181187576642106,
                                         0.241998587017256,
                                         0.224006538111973,
                                         0.149439412349659,
                                         0.0731023230598911,
                                         0.0262909728550987,
                                         0.0068886386090165,
                                         0.00128841079619431,
                                         0.000166427904962663,
                                         1.413739010348e-05,
                                         7.35498928754623e-07,
                                         2.11129176833659e-08,
                                         2.8556499718985e-10,
                                         1.4150898710068e-12,
                                         1.66506594943462e-15,
                                         2.00200658904999e-19,
                                         3.31327546572083e-25,
                                         5.4553951104622e-36,
                                         1.94706593214859e-07,
                                         0.000124415269380608,
                                         0.00323376861018523,
                                         0.0234520588325863,
                                         0.0806394675063316,
                                         0.16361951834376,
                                         0.218180496344519,
                                         0.202382439521989,
                                         0.134430852056255,
                                         0.0646580915701027,
                                         0.0224642237064978,
                                         0.00555595701289295,
                                         0.000951639491986542,
                                         0.000108109536250912,
                                         7.63724819400473e-06,
                                         3.04828492853347e-07,
                                         5.93943630138021e-09,
                                         4.47123321612301e-11,
                                         8.68198264713282e-14,
                                         1.98056356673072e-17,
                                         8.17567842008249e-23,
                                         7.15088156405983e-33,
                                         1.55968387455132e-09,
                                         5.29415591391912e-06,
                                         0.000343221872930209,
                                         0.00472261780552294,
                                         0.0267974611396058,
                                         0.08260078090145,
                                         0.158671131990821,
                                         0.204647887393814,
                                         0.184583759817837,
                                         0.118731477098061,
                                         0.0546985977368652,
                                         0.0178891688017151,
                                         0.00406298791323872,
                                         0.000617284725208769,
                                         5.92131994759549e-05,
                                         3.28616697520261e-06,
                                         9.22385035428884e-08,
                                         1.05486622479964e-09,
                                         3.38012803380732e-12,
                                         1.46298149430475e-15,
                                         1.50631853621519e-20,
                                         6.99873780490104e-30,
                                         9.667775940396e-12,
                                         1.74322658622299e-07,
                                         2.81886997169374e-05,
                                         0.000735899818746815,
                                         0.00689086463855744,
                                         0.0322676447517806,
                                         0.0892922949930424,
                                         0.160131132611895,
                                         0.196120095110756,
                                         0.168710817612864,
                                         0.103061163457669,
                                         0.044571322230796,
                                         0.0134230953395311,
                                         0.0027273515865913,
                                         0.000355250110882549,
                                         2.741307570275e-05,
                                         1.10844293201884e-06,
                                         1.92575716273377e-08,
                                         1.01831340081787e-10,
                                         8.36224620493846e-14,
                                         2.14755309877666e-18,
                                         5.30046453429066e-27,
                                         4.74141179154174e-14,
                                         4.54152865216114e-09,
                                         1.8317501415913e-06,
                                         9.07289105144317e-05,
                                         0.00140198992298548,
                                         0.0099733597525933,
                                         0.0397576910654224,
                                         0.0991369125128187,
                                         0.164870064002865,
                                         0.189675437124608,
                                         0.153640245728482,
                                         0.0878642037563328,
                                         0.0350873786327435,
                                         0.00953427836999605,
                                         0.00168632399768714,
                                         0.000180932676681282,
                                         1.05391487103648e-05,
                                         2.78161334177746e-07,
                                         2.42728555418359e-09,
                                         3.78179664002976e-12,
                                         2.42249062525095e-16,
                                         3.17613714040549e-24,
                                         1.86512638685269e-16,
                                         9.49007770112637e-11,
                                         9.5472196951661e-08,
                                         8.97206038464871e-06,
                                         0.000228789219856058,
                                         0.00247249370752069,
                                         0.0141986749250043,
                                         0.0492282610816776,
                                         0.111168311667962,
                                         0.171040418001701,
                                         0.183710693838849,
                                         0.138927406934865,
                                         0.0735645637934815,
                                         0.0267333891681326,
                                         0.0064204772400394,
                                         0.000957846116843489,
                                         8.03743028232628e-05,
                                         3.22263776204529e-06,
                                         4.64066026434471e-08,
                                         1.3718065916623e-10,
                                         2.19179429197146e-14,
                                         1.52652545896349e-21,
                                         5.92875698849118e-19,
                                         1.60247871793785e-12,
                                         4.02107701773591e-09,
                                         7.16957599141798e-07,
                                         3.01703963206397e-05,
                                         0.000495317528755489,
                                         0.00409759725458569,
                                         0.0197536972579827,
                                         0.0605724312773088,
                                         0.124635338415368,
                                         0.177508303826756,
                                         0.177508303826756,
                                         0.124635338415369,
                                         0.0605724312773088,
                                         0.0197536972579827,
                                         0.00409759725458569,
                                         0.000495317528755489,
                                         3.01703963206396e-05,
                                         7.16957599141793e-07,
                                         4.02107701773596e-09,
                                         1.60247871793785e-12,
                                         5.92875698849118e-19,
                                         1.52652545896349e-21,
                                         2.19179429197146e-14,
                                         1.37180659166228e-10,
                                         4.64066026434474e-08,
                                         3.22263776204531e-06,
                                         8.03743028232628e-05,
                                         0.000957846116843489,
                                         0.0064204772400394,
                                         0.0267333891681326,
                                         0.0735645637934814,
                                         0.138927406934865,
                                         0.183710693838849,
                                         0.171040418001701,
                                         0.111168311667962,
                                         0.0492282610816776,
                                         0.0141986749250043,
                                         0.00247249370752069,
                                         0.000228789219856057,
                                         8.97206038464865e-06,
                                         9.54721969516622e-08,
                                         9.49007770112637e-11,
                                         1.86512638685269e-16,
                                         3.17613714040549e-24,
                                         2.42249062525095e-16,
                                         3.7817966400297e-12,
                                         2.42728555418361e-09,
                                         2.78161334177747e-07,
                                         1.05391487103648e-05,
                                         0.000180932676681282,
                                         0.00168632399768714,
                                         0.00953427836999605,
                                         0.0350873786327435,
                                         0.0878642037563328,
                                         0.153640245728482,
                                         0.189675437124608,
                                         0.164870064002865,
                                         0.0991369125128187,
                                         0.0397576910654224,
                                         0.0099733597525933,
                                         0.00140198992298548,
                                         9.07289105144312e-05,
                                         1.83175014159132e-06,
                                         4.54152865216114e-09,
                                         4.74141179154174e-14,
                                         5.30046453429066e-27,
                                         2.14755309877666e-18,
                                         8.36224620493831e-14,
                                         1.01831340081788e-10,
                                         1.92575716273378e-08,
                                         1.10844293201884e-06,
                                         2.741307570275e-05,
                                         0.000355250110882549,
                                         0.0027273515865913,
                                         0.0134230953395311,
                                         0.044571322230796,
                                         0.103061163457669,
                                         0.168710817612864,
                                         0.196120095110756,
                                         0.160131132611895,
                                         0.0892922949930424,
                                         0.0322676447517806,
                                         0.00689086463855742,
                                         0.000735899818746812,
                                         2.81886997169376e-05,
                                         1.74322658622299e-07,
                                         9.667775940396e-12,
                                         6.99873780490104e-30,
                                         1.5063185362152e-20,
                                         1.46298149430472e-15,
                                         3.38012803380736e-12,
                                         1.05486622479964e-09,
                                         9.22385035428884e-08,
                                         3.28616697520261e-06,
                                         5.92131994759549e-05,
                                         0.000617284725208769,
                                         0.00406298791323871,
                                         0.0178891688017151,
                                         0.0546985977368652,
                                         0.118731477098061,
                                         0.184583759817837,
                                         0.204647887393814,
                                         0.158671131990821,
                                         0.08260078090145,
                                         0.0267974611396058,
                                         0.00472261780552292,
                                         0.000343221872930211,
                                         5.29415591391912e-06,
                                         1.55968387455132e-09,
                                         7.15088156405983e-33,
                                         8.17567842008248e-23,
                                         1.98056356673068e-17,
                                         8.68198264713292e-14,
                                         4.47123321612304e-11,
                                         5.93943630138021e-09,
                                         3.04828492853347e-07,
                                         7.63724819400473e-06,
                                         0.000108109536250912,
                                         0.00095163949198654,
                                         0.00555595701289295,
                                         0.0224642237064978,
                                         0.0646580915701028,
                                         0.134430852056255,
                                         0.202382439521989,
                                         0.218180496344519,
                                         0.16361951834376,
                                         0.0806394675063314,
                                         0.0234520588325862,
                                         0.00323376861018525,
                                         0.000124415269380608,
                                         1.94706593214859e-07,
                                         5.4553951104622e-36,
                                         3.31327546572083e-25,
                                         2.00200658904994e-19,
                                         1.66506594943464e-15,
                                         1.41508987100681e-12,
                                         2.8556499718985e-10,
                                         2.11129176833659e-08,
                                         7.35498928754623e-07,
                                         1.413739010348e-05,
                                         0.000166427904962663,
                                         0.00128841079619431,
                                         0.0068886386090165,
                                         0.0262909728550987,
                                         0.0731023230598911,
                                         0.149439412349659,
                                         0.224006538111973,
                                         0.241998587017256,
                                         0.181187576642105,
                                         0.086957273341796,
                                         0.0227493864831224,
                                         0.00218311897716933,
                                         1.81489497229094e-05,
                                         2.92634398358125e-39,
                                         9.44112675284216e-28,
                                         1.42290124392074e-21,
                                         2.24531100767095e-17,
                                         3.14900435742884e-14,
                                         9.65377688190979e-12,
                                         1.02819027030701e-09,
                                         4.98034839426715e-08,
                                         1.29989124689338e-06,
                                         2.04650295335292e-05,
                                         0.000210078870400404,
                                         0.00148527861230675,
                                         0.00751662651082192,
                                         0.0279509095632998,
                                         0.0775871886049561,
                                         0.161710419246713,
                                         0.251665156226664,
                                         0.286247512482945,
                                         0.226706196968297,
                                         0.112528615740794,
                                         0.0269347944274123,
                                         0.00118947379019085,
                                         9.84927677831163e-43,
                                         1.68799017422399e-30,
                                         6.34547033008871e-24,
                                         1.89977152382537e-19,
                                         4.39685661377291e-16,
                                         2.047714206813e-13,
                                         3.14179994297262e-11,
                                         2.11600751830609e-09,
                                         7.4993668540703e-08,
                                         1.57898646568466e-06,
                                         2.14926612617626e-05,
                                         0.000200938083376506,
                                         0.00134840106919092,
                                         0.00670564432151035,
                                         0.0252752047174798,
                                         0.0732478891917326,
                                         0.164215124061326,
                                         0.283749319349462,
                                         0.37085203977648,
                                         0.349249719748098,
                                         0.208511363337212,
                                         0.0489145495457261,
                                         1.56541585982106e-46,
                                         1.4251560500717e-33,
                                         1.33628562690161e-26,
                                         7.59053981051155e-22,
                                         2.89906374710342e-18,
                                         2.05110479640651e-15,
                                         4.53346206205159e-13,
                                         4.24542437553124e-11,
                                         2.04309517962399e-09,
                                         5.75295344817085e-08,
                                         1.03835159662647e-06,
                                         1.28369841276479e-05,
                                         0.000114225120704887,
                                         0.000759681418020396,
                                         0.00388817469057463,
                                         0.0156674618385117,
                                         0.0505998973410627,
                                         0.132823327544138,
                                         0.286473446588884,
                                         0.511865167460691,
                                         0.7622408348722,
                                         0.949877631438368,

                                         0.953956074359213,
                                         0.779712702779712,
                                         0.541397842926258,
                                         0.318262553368932,
                                         0.157603715740835,
                                         0.0652958720426337,
                                         0.0224309450184589,
                                         0.00631631592164802,
                                         0.00143687611079333,
                                         0.000259266552562984,
                                         3.6255195549885e-05,
                                         3.814697265625e-06,
                                         2.90728652058555e-07,
                                         1.5269312752136e-08,
                                         5.17056830011031e-10,
                                         1.03024077835082e-11,
                                         1.06068462868447e-13,
                                         4.65145579749214e-16,
                                         6.40553840350554e-19,
                                         1.6423858819616e-22,
                                         2.84563553975294e-27,
                                         3.00169203844812e-34,
                                         3.27706002367977e-47,
                                         0.0450261803009574,
                                         0.195362947683038,
                                         0.337929297836594,
                                         0.376209445882646,
                                         0.306670280281943,
                                         0.192396557284535,
                                         0.094828502604167,
                                         0.0369437102705232,
                                         0.0113422296947118,
                                         0.00271658346835417,
                                         0.00049911985328544,
                                         6.866455078125e-05,
                                         6.84223918643497e-06,
                                         4.72158638956468e-07,
                                         2.12228730272508e-08,
                                         5.70698958914534e-10,
                                         8.12905315469539e-12,
                                         5.11472558864618e-14,
                                         1.06658485296431e-16,
                                         4.5016949298431e-20,
                                         1.47711722162003e-24,
                                         3.88153542188857e-31,
                                         2.24953460296587e-43,
                                         0.00100357135112194,
                                         0.0231151230536856,
                                         0.0996051035647087,
                                         0.210000471173126,
                                         0.281788553973309,
                                         0.267704263194773,
                                         0.189311342580434,
                                         0.102038389647484,
                                         0.0422789285184045,
                                         0.0134414473303944,
                                         0.00324478449901321,
                                         0.000583648681640625,
                                         7.60422729426017e-05,
                                         6.89450054956844e-06,
                                         4.11354727565571e-07,
                                         1.49286950017308e-08,
                                         2.9419824126417e-10,
                                         2.65584178112678e-12,
                                         8.38651648320966e-15,
                                         5.82670912696685e-18,
                                         3.62073772963936e-22,
                                         2.37021310400099e-28,
                                         7.29201209717198e-40,
                                         1.40349608727183e-05,
                                         0.00171605033484579,
                                         0.0184211669700999,
                                         0.0735513494045597,
                                         0.162463127351062,
                                         0.233718494658297,
                                         0.237134192873267,
                                         0.176834290648001,
                                         0.098884747502389,
                                         0.0417300416316547,
                                         0.0132356926730841,
                                         0.00311279296875,
                                         0.000530263494658967,
                                         6.31680518399326e-05,
                                         5.00274782939186e-06,
                                         2.45028393490656e-07,
                                         6.6806694373265e-09,
                                         8.65290352651643e-11,
                                         4.13759139722967e-13,
                                         4.73206191654846e-16,
                                         5.56876608453551e-20,
                                         9.08136305223838e-26,
                                         1.48313910271449e-36,
                                         1.38008773795914e-07,
                                         8.95769713726893e-05,
                                         0.00239543960942616,
                                         0.0181131323483978,
                                         0.0658595576332518,
                                         0.1434707647516,
                                         0.208854705778804,
                                         0.21547749249167,
                                         0.162617464467244,
                                         0.0910928187850424,
                                         0.037961208530048,
                                         0.0116729736328125,
                                         0.00259992553523241,
                                         0.000406934672754319,
                                         4.27792603443421e-05,
                                         2.8277663340155e-06,
                                         1.06667577981237e-07,
                                         1.98223008961974e-09,
                                         1.43531140360032e-11,
                                         2.70215363862238e-14,
                                         6.02217572293996e-18,
                                         2.44651127283568e-23,
                                         2.12104034590687e-33,
                                         1.01327878932558e-09,
                                         3.49131866894321e-06,
                                         0.000232584129750598,
                                         0.0033306058426012,
                                         0.0199346939852506,
                                         0.0657598039405578,
                                         0.13734761208161,
                                         0.196048762384255,
                                         0.199678738921189,
                                         0.148472570629778,
                                         0.0812943100293935,
                                         0.032684326171875,
                                         0.00951824441956356,
                                         0.0019573958864142,
                                         0.000273139615718114,
                                         2.43667378161979e-05,
                                         1.27166186712461e-06,
                                         3.39057246321326e-08,
                                         3.7176751122833e-10,
                                         1.15211656783666e-12,
                                         4.86266818345049e-16,
                                         4.92119039731831e-21,
                                         2.2648671760316e-30,
                                         5.75685481563968e-12,
                                         1.05297164707819e-07,
                                         1.74746706494438e-05,
                                         0.000473900306176121,
                                         0.0046691132483841,
                                         0.0233233901276148,
                                         0.0698927344650958,
                                         0.138025830072785,
                                         0.18972760991173,
                                         0.187258864354865,
                                         0.13471449867575,
                                         0.0708160400390625,
                                         0.0269641582229261,
                                         0.0072856233890661,
                                         0.00134949160550548,
                                         0.000162474545818564,
                                         1.17312673823586e-05,
                                         4.48772325478341e-07,
                                         7.4512786279458e-09,
                                         3.80116635086731e-11,
                                         3.03829485064982e-14,
                                         7.65997205369108e-19,
                                         1.87141719871147e-27,
                                         2.58781190357909e-14,
                                         2.51266718487325e-09,
                                         1.03879302520809e-06,
                                         5.33509114238007e-05,
                                         0.000865267409899009,
                                         0.0065450647576958,
                                         0.0281406454842651,
                                         0.0768860826817601,
                                         0.142633109548781,
                                         0.186865723502519,
                                         0.176628033305751,
                                         0.12139892578125,
                                         0.0604377078455369,
                                         0.0214558571178882,
                                         0.00527529541244738,
                                         0.000857164896347696,
                                         8.56267228095377e-05,
                                         4.69970078090365e-06,
                                         1.18162936942407e-07,
                                         9.92266739859146e-10,
                                         1.50202203491695e-12,
                                         9.4335532779325e-17,
                                         1.22345950469166e-24,
                                         9.33038635810778e-17,
                                         4.80919902930439e-11,
                                         4.95300245071312e-08,
                                         4.81743861073539e-06,
                                         0.000128613269069164,
                                         0.00147317965038677,
                                         0.00908773323381658,
                                         0.0343522029718131,
                                         0.086006179688042,
                                         0.149567212690904,
                                         0.185748116782933,
                                         0.166923522949219,
                                         0.108654719556954,
                                         0.0506809224741301,
                                         0.0165402806545762,
                                         0.00362712827177281,
                                         0.000501294804863209,
                                         3.94760637232936e-05,
                                         1.50297322003077e-06,
                                         2.0775882703757e-08,
                                         5.95582848238062e-11,
                                         9.31843371054165e-15,
                                         6.41546402697104e-22,
                                         2.71844994954334e-19,
                                         7.438156743937e-13,
                                         1.90837119707731e-09,
                                         3.51516186005267e-07,
                                         1.54481330188267e-05,
                                         0.000267949077884068,
                                         0.00237154777247989,
                                         0.0124027001538813,
                                         0.0419076932456384,
                                         0.0967381875333307,
                                         0.15784978613954,
                                         0.185470581054688,
                                         0.15784978613954,
                                         0.0967381875333307,
                                         0.0419076932456384,
                                         0.0124027001538813,
                                         0.00237154777247988,
                                         0.000267949077884069,
                                         1.54481330188267e-05,
                                         3.51516186005267e-07,
                                         1.90837119707734e-09,
                                         7.438156743937e-13,
                                         2.71844994954282e-19,
                                         6.4154640269724e-22,
                                         9.31843371054165e-15,
                                         5.95582848238052e-11,
                                         2.0775882703757e-08,
                                         1.50297322003077e-06,
                                         3.94760637232935e-05,
                                         0.000501294804863211,
                                         0.00362712827177281,
                                         0.0165402806545762,
                                         0.0506809224741301,
                                         0.108654719556954,
                                         0.166923522949219,
                                         0.185748116782933,
                                         0.149567212690904,
                                         0.086006179688042,
                                         0.0343522029718131,
                                         0.00908773323381656,
                                         0.00147317965038677,
                                         0.000128613269069164,
                                         4.81743861073539e-06,
                                         4.95300245071318e-08,
                                         4.80919902930439e-11,
                                         9.3303863581062e-17,
                                         1.22345950469195e-24,
                                         9.4335532779325e-17,
                                         1.50202203491693e-12,
                                         9.92266739859146e-10,
                                         1.18162936942407e-07,
                                         4.69970078090363e-06,
                                         8.5626722809538e-05,
                                         0.000857164896347694,
                                         0.00527529541244738,
                                         0.0214558571178882,
                                         0.0604377078455369,
                                         0.12139892578125,
                                         0.176628033305751,
                                         0.186865723502519,
                                         0.142633109548781,
                                         0.0768860826817602,
                                         0.028140645484265,
                                         0.00654506475769582,
                                         0.000865267409899009,
                                         5.33509114238007e-05,
                                         1.03879302520811e-06,
                                         2.51266718487325e-09,
                                         2.58781190357871e-14,
                                         1.87141719871195e-27,
                                         7.65997205369108e-19,
                                         3.03829485064976e-14,
                                         3.80116635086731e-11,
                                         7.4512786279458e-09,
                                         4.48772325478339e-07,
                                         1.17312673823586e-05,
                                         0.000162474545818563,
                                         0.00134949160550548,
                                         0.0072856233890661,
                                         0.0269641582229261,
                                         0.0708160400390625,
                                         0.13471449867575,
                                         0.187258864354865,
                                         0.18972760991173,
                                         0.138025830072785,
                                         0.0698927344650957,
                                         0.0233233901276148,
                                         0.0046691132483841,
                                         0.000473900306176121,
                                         1.7474670649444e-05,
                                         1.05297164707819e-07,
                                         5.75685481563894e-12,
                                         2.26486717603222e-30,
                                         4.92119039731831e-21,
                                         4.86266818345039e-16,
                                         1.15211656783666e-12,
                                         3.7176751122833e-10,
                                         3.39057246321324e-08,
                                         1.27166186712461e-06,
                                         2.43667378161978e-05,
                                         0.000273139615718114,
                                         0.0019573958864142,
                                         0.00951824441956356,
                                         0.032684326171875,
                                         0.0812943100293935,
                                         0.148472570629778,
                                         0.199678738921189,
                                         0.196048762384255,
                                         0.13734761208161,
                                         0.065759803940558,
                                         0.0199346939852506,
                                         0.0033306058426012,
                                         0.000232584129750599,
                                         3.49131866894321e-06,
                                         1.01327878932547e-09,
                                         2.1210403459075e-33,
                                         2.44651127283568e-23,
                                         6.02217572293982e-18,
                                         2.70215363862238e-14,
                                         1.43531140360032e-11,
                                         1.98223008961973e-09,
                                         1.06667577981238e-07,
                                         2.8277663340155e-06,
                                         4.27792603443421e-05,
                                         0.000406934672754319,
                                         0.00259992553523241,
                                         0.0116729736328125,
                                         0.037961208530048,
                                         0.0910928187850424,
                                         0.162617464467244,
                                         0.21547749249167,
                                         0.208854705778804,
                                         0.1434707647516,
                                         0.0658595576332518,
                                         0.0181131323483978,
                                         0.00239543960942618,
                                         8.95769713726893e-05,
                                         1.38008773795902e-07,
                                         1.48313910271496e-36,
                                         9.08136305223838e-26,
                                         5.56876608453537e-20,
                                         4.73206191654846e-16,
                                         4.13759139722967e-13,
                                         8.65290352651638e-11,
                                         6.68066943732653e-09,
                                         2.45028393490655e-07,
                                         5.00274782939186e-06,
                                         6.31680518399326e-05,
                                         0.000530263494658967,
                                         0.00311279296875,
                                         0.0132356926730841,
                                         0.0417300416316547,
                                         0.098884747502389,
                                         0.176834290648001,
                                         0.237134192873267,
                                         0.233718494658298,
                                         0.162463127351062,
                                         0.0735513494045597,
                                         0.0184211669701,
                                         0.00171605033484579,
                                         1.40349608727174e-05,
                                         7.29201209717445e-40,
                                         2.37021310400099e-28,
                                         3.62073772963927e-22,
                                         5.82670912696685e-18,
                                         8.38651648320966e-15,
                                         2.65584178112677e-12,
                                         2.94198241264171e-10,
                                         1.49286950017308e-08,
                                         4.11354727565571e-07,
                                         6.89450054956844e-06,
                                         7.60422729426017e-05,
                                         0.000583648681640625,
                                         0.00324478449901321,
                                         0.0134414473303944,
                                         0.0422789285184045,
                                         0.102038389647484,
                                         0.189311342580434,
                                         0.267704263194773,
                                         0.281788553973309,
                                         0.210000471173126,
                                         0.099605103564709,
                                         0.0231151230536856,
                                         0.00100357135112189,
                                         2.24953460296668e-43,
                                         3.88153542188857e-31,
                                         1.47711722161999e-24,
                                         4.5016949298431e-20,
                                         1.06658485296431e-16,
                                         5.11472558864615e-14,
                                         8.12905315469543e-12,
                                         5.70698958914532e-10,
                                         2.12228730272508e-08,
                                         4.72158638956468e-07,
                                         6.84223918643497e-06,
                                         6.866455078125e-05,
                                         0.00049911985328544,
                                         0.00271658346835417,
                                         0.0113422296947118,
                                         0.0369437102705232,
                                         0.094828502604167,
                                         0.192396557284535,
                                         0.306670280281943,
                                         0.376209445882646,
                                         0.337929297836595,
                                         0.195362947683038,
                                         0.0450261803009565,
                                         3.27706002368102e-47,
                                         3.00169203844812e-34,
                                         2.84563553975286e-27,
                                         1.6423858819616e-22,
                                         6.40553840350554e-19,
                                         4.6514557974921e-16,
                                         1.06068462868447e-13,
                                         1.03024077835081e-11,
                                         5.17056830011031e-10,
                                         1.5269312752136e-08,
                                         2.90728652058555e-07,
                                         3.814697265625e-06,
                                         3.6255195549885e-05,
                                         0.000259266552562984,
                                         0.00143687611079333,
                                         0.00631631592164802,
                                         0.0224309450184589,
                                         0.0652958720426337,
                                         0.157603715740835,
                                         0.318262553368932,
                                         0.541397842926258,
                                         0.779712702779712,
                                         0.953956074359213,

                                         0.957559690232628,
                                         0.795414298398138,
                                         0.568768279385799,
                                         0.34908613648052,
                                         0.183127510700861,
                                         0.0816402795079587,
                                         0.0306996932897307,
                                         0.00964506216681456,
                                         0.00250160680389893,
                                         0.000527688152263875,
                                         8.8848878374091e-05,
                                         1.16634589974733e-05,
                                         1.15864444052975e-06,
                                         8.38271644746027e-08,
                                         4.20148951440844e-09,
                                         1.36460602898682e-10,
                                         2.62071993570511e-12,
                                         2.61305107862297e-14,
                                         1.11492444671931e-16,
                                         1.50064829689462e-19,
                                         3.77747405687779e-23,
                                         6.45381616251099e-28,
                                         6.74219224346571e-35,
                                         7.32130095599456e-48,
                                         0.0415767670758791,
                                         0.183226611826978,
                                         0.326029448551582,
                                         0.378343878611484,
                                         0.326003264220102,
                                         0.219463649383248,
                                         0.117991780935413,
                                         0.0510669021285458,
                                         0.0177819665883642,
                                         0.00494735998276241,
                                         0.00108606059383193,
                                         0.000184664950052195,
                                         2.3710356634476e-05,
                                         2.22191622174673e-06,
                                         1.45195155358378e-07,
                                         6.22001676795605e-09,
                                         1.60372958413084e-10,
                                         2.20280061823998e-12,
                                         1.34379287704615e-14,
                                         2.73121301458418e-17,
                                         1.12925595783219e-20,
                                         3.64787659977376e-25,
                                         9.48311428829631e-32,
                                         5.4632164706884e-44,
                                         0.000852475679930755,
                                         0.0199310477060145,
                                         0.088252031626023,
                                         0.193636508133087,
                                         0.274054366375357,
                                         0.278591031271317,
                                         0.214148923997058,
                                         0.127679277213893,
                                         0.059687989550545,
                                         0.0219036303185322,
                                         0.00626906226617999,
                                         0.00138066399500791,
                                         0.000229124964800077,
                                         2.78110255408215e-05,
                                         2.36944912395684e-06,
                                         1.3388201676308e-07,
                                         4.63434210148696e-09,
                                         8.76897639124397e-11,
                                         7.64831298388022e-13,
                                         2.34735431216993e-15,
                                         1.59415193744351e-18,
                                         9.73666230032983e-23,
                                         6.2986468358797e-29,
                                         1.92510746892665e-40,
                                         1.09671322785179e-05,
                                         0.00136035265542399,
                                         0.014988990926751,
                                         0.0621824136175158,
                                         0.144554396644714,
                                         0.221897018832649,
                                         0.243870820316103,
                                         0.200300078805447,
                                         0.125711183265056,
                                         0.0608469049592585,
                                         0.0227054917824241,
                                         0.00647696168913926,
                                         0.00138926961842427,
                                         0.000218416845448393,
                                         2.42617683993488e-05,
                                         1.80814285019776e-06,
                                         8.40281575261175e-08,
                                         2.19029400353318e-09,
                                         2.73135890641647e-11,
                                         1.26584789520749e-13,
                                         1.4120394629709e-16,
                                         1.6306467101659e-20,
                                         2.62496389345884e-26,
                                         4.25638938187728e-37,
                                         9.9205718123326e-08,
                                         6.52838004371673e-05,
                                         0.00178999831130355,
                                         0.0140404312687852,
                                         0.0536115575325852,
                                         0.124270585329169,
                                         0.195270357533334,
                                         0.220940007604171,
                                         0.186163020729895,
                                         0.118848422982283,
                                         0.0578218232738929,
                                         0.0213642276395376,
                                         0.00592288370250242,
                                         0.00120611243942183,
                                         0.000174674720513396,
                                         1.71702164827998e-05,
                                         1.07125813255127e-06,
                                         3.84670132725074e-08,
                                         6.8584255751948e-10,
                                         4.7997314630465e-12,
                                         8.79420321757293e-15,
                                         1.92018109477401e-18,
                                         7.6918702482758e-24,
                                         6.61698672035408e-34,
                                         6.700498160891e-10,
                                         2.339301029739e-06,
                                         0.000159609820591783,
                                         0.00236711896064568,
                                         0.0148460974688072,
                                         0.0519651259081441,
                                         0.116745343510372,
                                         0.181967728025543,
                                         0.205844697417318,
                                         0.173330551196903,
                                         0.109945995215777,
                                         0.0526174538706171,
                                         0.0188541360125313,
                                         0.00497297443386801,
                                         0.000938997476257674,
                                         0.000121743439359355,
                                         1.01974171736787e-05,
                                         5.04430437389309e-07,
                                         1.28586937467118e-08,
                                         1.358873783026e-10,
                                         4.08952552820598e-13,
                                         1.68830638927491e-16,
                                         1.68293494706455e-21,
                                         7.68079083757681e-31,
                                         3.50196297421625e-12,
                                         6.48635726994043e-08,
                                         1.1012873943129e-05,
                                         0.000308811731878186,
                                         0.00318126802912175,
                                         0.0168147221154234,
                                         0.054010336600874,
                                         0.115970746135786,
                                         0.17612460060449,
                                         0.195609923566211,
                                         0.161771166900354,
                                         0.100278200464587,
                                         0.0464423446239192,
                                         0.0158664119870669,
                                         0.00390600561777947,
                                         0.000667958427095324,
                                         7.51139000170066e-05,
                                         5.1185643953539e-06,
                                         1.86553476291039e-07,
                                         2.97697625195984e-09,
                                         1.47157845409965e-11,
                                         1.14866770012429e-14,
                                         2.84929081895005e-19,
                                         6.89899268955212e-28,
                                         1.44812853690186e-14,
                                         1.42300597678216e-09,
                                         6.01219185685113e-07,
                                         3.18756171477773e-05,
                                         0.000539360724476084,
                                         0.00430485481531419,
                                         0.019769937870977,
                                         0.0584781512353956,
                                         0.119231624915057,
                                         0.174661728113264,
                                         0.188327579962118,
                                         0.151207850123465,
                                         0.0905133714845904,
                                         0.04005274923303,
                                         0.0128556016363626,
                                         0.00289964214933451,
                                         0.000437765508977024,
                                         4.10947329762824e-05,
                                         2.14141550868786e-06,
                                         5.16015473368676e-08,
                                         4.18972020957167e-10,
                                         6.18341874708608e-13,
                                         3.81678179963098e-17,
                                         4.90293950803395e-25,
                                         4.80310594021e-17,
                                         2.50398702676128e-11,
                                         2.63259763025335e-08,
                                         2.63902135799456e-06,
                                         7.33462422000899e-05,
                                         0.000883988908484933,
                                         0.00580434498699067,
                                         0.0236514792892204,
                                         0.0647414461536342,
                                         0.125090438322612,
                                         0.175851559989241,
                                         0.182878073319143,
                                         0.141491659187065,
                                         0.0810971194128217,
                                         0.0339368431365155,
                                         0.0100962214284133,
                                         0.0020463610571887,
                                         0.000264632784443047,
                                         1.97159647567638e-05,
                                         7.17413543800891e-07,
                                         9.56766839864524e-09,
                                         2.66982256706802e-11,
                                         4.10088299831347e-15,
                                         2.79477517798245e-22,
                                         1.28733628991173e-19,
                                         3.56051005952848e-13,
                                         9.31517318133568e-10,
                                         1.76555804664653e-07,
                                         8.05992889265305e-06,
                                         0.000146686421923149,
                                         0.00137706969966933,
                                         0.00772996942820251,
                                         0.0284071799297113,
                                         0.0723944492043115,
                                         0.132688510017223,
                                         0.178732581894545,
                                         0.178732581894545,
                                         0.132688510017223,
                                         0.0723944492043115,
                                         0.0284071799297113,
                                         0.00772996942820249,
                                         0.00137706969966933,
                                         0.000146686421923149,
                                         8.05992889265305e-06,
                                         1.76555804664653e-07,
                                         9.31517318133553e-10,
                                         3.56051005952848e-13,
                                         1.28733628991147e-19,
                                         2.79477517798309e-22,
                                         4.10088299831347e-15,
                                         2.66982256706807e-11,
                                         9.56766839864525e-09,
                                         7.17413543800891e-07,
                                         1.97159647567638e-05,
                                         0.000264632784443047,
                                         0.00204636105718871,
                                         0.0100962214284133,
                                         0.0339368431365155,
                                         0.0810971194128217,
                                         0.141491659187064,
                                         0.182878073319143,
                                         0.175851559989241,
                                         0.125090438322612,
                                         0.0647414461536342,
                                         0.0236514792892203,
                                         0.00580434498699067,
                                         0.00088398890848493,
                                         7.33462422000899e-05,
                                         2.63902135799456e-06,
                                         2.63259763025332e-08,
                                         2.50398702676128e-11,
                                         4.80310594020912e-17,
                                         4.90293950803519e-25,
                                         3.81678179963098e-17,
                                         6.1834187470862e-13,
                                         4.18972020957167e-10,
                                         5.16015473368676e-08,
                                         2.14141550868787e-06,
                                         4.10947329762824e-05,
                                         0.000437765508977026,
                                         0.00289964214933451,
                                         0.0128556016363626,
                                         0.04005274923303,
                                         0.0905133714845903,
                                         0.151207850123465,
                                         0.188327579962118,
                                         0.174661728113264,
                                         0.119231624915057,
                                         0.0584781512353955,
                                         0.019769937870977,
                                         0.00430485481531418,
                                         0.000539360724476084,
                                         3.18756171477773e-05,
                                         6.01219185685105e-07,
                                         1.42300597678216e-09,
                                         1.44812853690163e-14,
                                         6.89899268955403e-28,
                                         2.84929081895005e-19,
                                         1.14866770012432e-14,
                                         1.47157845409965e-11,
                                         2.97697625195984e-09,
                                         1.8655347629104e-07,
                                         5.1185643953539e-06,
                                         7.51139000170068e-05,
                                         0.000667958427095324,
                                         0.00390600561777947,
                                         0.0158664119870669,
                                         0.0464423446239191,
                                         0.100278200464587,
                                         0.161771166900354,
                                         0.195609923566211,
                                         0.17612460060449,
                                         0.115970746135786,
                                         0.054010336600874,
                                         0.0168147221154234,
                                         0.00318126802912175,
                                         0.000308811731878186,
                                         1.10128739431289e-05,
                                         6.48635726994043e-08,
                                         3.50196297421577e-12,
                                         7.68079083757911e-31,
                                         1.68293494706455e-21,
                                         1.68830638927495e-16,
                                         4.08952552820598e-13,
                                         1.358873783026e-10,
                                         1.28586937467118e-08,
                                         5.04430437389309e-07,
                                         1.01974171736787e-05,
                                         0.000121743439359355,
                                         0.000938997476257674,
                                         0.00497297443386801,
                                         0.0188541360125313,
                                         0.0526174538706171,
                                         0.109945995215777,
                                         0.173330551196903,
                                         0.205844697417318,
                                         0.181967728025543,
                                         0.116745343510372,
                                         0.051965125908144,
                                         0.0148460974688072,
                                         0.00236711896064568,
                                         0.000159609820591782,
                                         2.339301029739e-06,
                                         6.70049816089023e-10,
                                         6.61698672035622e-34,
                                         7.6918702482758e-24,
                                         1.92018109477406e-18,
                                         8.79420321757293e-15,
                                         4.7997314630465e-12,
                                         6.85842557519484e-10,
                                         3.84670132725074e-08,
                                         1.07125813255128e-06,
                                         1.71702164827998e-05,
                                         0.000174674720513396,
                                         0.00120611243942183,
                                         0.00592288370250242,
                                         0.0213642276395376,
                                         0.0578218232738929,
                                         0.118848422982283,
                                         0.186163020729895,
                                         0.22094000760417,
                                         0.195270357533334,
                                         0.124270585329168,
                                         0.0536115575325852,
                                         0.0140404312687852,
                                         0.00178999831130353,
                                         6.52838004371673e-05,
                                         9.92057181233168e-08,
                                         4.25638938187875e-37,
                                         2.62496389345884e-26,
                                         1.63064671016595e-20,
                                         1.4120394629709e-16,
                                         1.26584789520749e-13,
                                         2.73135890641649e-11,
                                         2.19029400353318e-09,
                                         8.40281575261178e-08,
                                         1.80814285019776e-06,
                                         2.42617683993488e-05,
                                         0.000218416845448393,
                                         0.00138926961842427,
                                         0.00647696168913926,
                                         0.0227054917824241,
                                         0.0608469049592585,
                                         0.125711183265056,
                                         0.200300078805447,
                                         0.243870820316103,
                                         0.221897018832649,
                                         0.144554396644713,
                                         0.0621824136175158,
                                         0.0149889909267509,
                                         0.00136035265542399,
                                         1.09671322785171e-05,
                                         1.92510746892736e-40,
                                         6.2986468358797e-29,
                                         9.73666230033011e-23,
                                         1.59415193744351e-18,
                                         2.34735431216993e-15,
                                         7.64831298388027e-13,
                                         8.76897639124397e-11,
                                         4.63434210148698e-09,
                                         1.3388201676308e-07,
                                         2.36944912395684e-06,
                                         2.78110255408215e-05,
                                         0.000229124964800077,
                                         0.00138066399500791,
                                         0.00626906226617999,
                                         0.0219036303185322,
                                         0.059687989550545,
                                         0.127679277213893,
                                         0.214148923997058,
                                         0.278591031271316,
                                         0.274054366375357,
                                         0.193636508133087,
                                         0.0882520316260227,
                                         0.0199310477060145,
                                         0.000852475679930715,
                                         5.46321647069055e-44,
                                         9.48311428829631e-32,
                                         3.64787659977387e-25,
                                         1.12925595783219e-20,
                                         2.73121301458418e-17,
                                         1.34379287704616e-14,
                                         2.20280061823999e-12,
                                         1.60372958413084e-10,
                                         6.22001676795605e-09,
                                         1.45195155358378e-07,
                                         2.22191622174673e-06,
                                         2.37103566344759e-05,
                                         0.000184664950052195,
                                         0.00108606059383193,
                                         0.00494735998276241,
                                         0.0177819665883642,
                                         0.0510669021285458,
                                         0.117991780935413,
                                         0.219463649383248,
                                         0.326003264220102,
                                         0.378343878611484,
                                         0.326029448551582,
                                         0.183226611826978,
                                         0.0415767670758781,
                                         7.3213009559976e-48,
                                         6.74219224346571e-35,
                                         6.4538161625112e-28,
                                         3.77747405687779e-23,
                                         1.50064829689462e-19,
                                         1.11492444671932e-16,
                                         2.61305107862297e-14,
                                         2.62071993570512e-12,
                                         1.36460602898682e-10,
                                         4.20148951440844e-09,
                                         8.38271644746027e-08,
                                         1.15864444052975e-06,
                                         1.16634589974733e-05,
                                         8.8848878374091e-05,
                                         0.000527688152263875,
                                         0.00250160680389893,
                                         0.00964506216681456,
                                         0.0306996932897307,
                                         0.0816402795079587,
                                         0.183127510700861,
                                         0.34908613648052,
                                         0.568768279385799,
                                         0.795414298398138,
                                         0.957559690232628,

                                         1.9073486328125e-06,
                                         3.62396240234375e-05,
                                         0.000326156616210938,
                                         0.00184822082519531,
                                         0.00739288330078125,
                                         0.0221786499023438,
                                         0.0517501831054688,
                                         0.0961074829101562,
                                         0.144161224365234,
                                         0.176197052001953,
                                         0.176197052001953,
                                         0.144161224365234,
                                         0.0961074829101562,
                                         0.0517501831054688,
                                         0.0221786499023438,
                                         0.00739288330078125,
                                         0.00184822082519531,
                                         0.000326156616210938,
                                         3.62396240234375e-05,
                                         1.9073486328125e-06,

                                         0.0109916988961064,
                                         1.49300554189677e-13,
                                         0.0559591200104975,
                                         1.05867478228472e-11,
                                         0.134947809143722,
                                         3.55592526856066e-10,
                                         0.20490188672232,
                                         7.52017313643895e-09,
                                         0.219613180299437,
                                         1.12262672907619e-07,
                                         0.176535522925394,
                                         1.25690999725416e-06,
                                         0.110372618540952,
                                         1.09453212640189e-05,
                                         0.0549236145798905,
                                         7.58614908303245e-05,
                                         0.0220751072581198,
                                         0.000424678407175037,
                                         0.00722945319878665,
                                         0.00193712614633348,
                                         0.00193712614633348,
                                         0.00722945319878665,
                                         0.000424678407175037,
                                         0.0220751072581198,
                                         7.58614908303245e-05,
                                         0.0549236145798905,
                                         1.09453212640189e-05,
                                         0.110372618540952,
                                         1.25690999725416e-06,
                                         0.176535522925394,
                                         1.12262672907619e-07,
                                         0.219613180299437,
                                         7.52017313643895e-09,
                                         0.204901886722321,
                                         3.55592526856066e-10,
                                         0.134947809143722,
                                         1.05867478228472e-11,
                                         0.0559591200104975,
                                         1.49300554189677e-13,
                                         0.0109916988961064,

                                         0.103115431,
                                         1.9073486328125e-06,
                                         9.69786956522574e-19,
                                         0.248850162999999,
                                         3.62396240234375e-05,
                                         1.45067214603356e-16,
                                         0.28447303499999,
                                         0.000326156616210938,
                                         1.02790058818763e-14,
                                         0.204752606999542,
                                         0.00184822082519531,
                                         4.58583172033619e-13,
                                         0.104027963985559,
                                         0.00739288330078125,
                                         1.44416707050866e-11,
                                         0.0396398516589031,
                                         0.0221786499023438,
                                         3.41097098857674e-10,
                                         0.0117481497339459,
                                         0.0517501831054688,
                                         6.26605415040763e-09,
                                         0.00277124838242579,
                                         0.0961074829101562,
                                         9.16175742348156e-08,
                                         0.00052799204454575,
                                         0.144161224365234,
                                         1.08195545425594e-06,
                                         8.1966845555596e-05,
                                         0.176197052001953,
                                         1.04111544444053e-05,
                                         1.04111544444053e-05,
                                         0.176197052001953,
                                         8.1966845555596e-05,
                                         1.08195545425594e-06,
                                         0.144161224365234,
                                         0.000527992044545751,
                                         9.16175742348156e-08,
                                         0.0961074829101562,
                                         0.00277124838242579,
                                         6.26605415040763e-09,
                                         0.0517501831054688,
                                         0.0117481497339459,
                                         3.41097098857674e-10,
                                         0.0221786499023438,
                                         0.0396398516589031,
                                         1.44416707050866e-11,
                                         0.00739288330078125,
                                         0.104027963985559,
                                         4.58583172033619e-13,
                                         0.00184822082519531,
                                         0.204752606999542,
                                         1.02790058818763e-14,
                                         0.000326156616210938,
                                         0.28447303499999,
                                         1.45067214603356e-16,
                                         3.62396240234375e-05,
                                         0.248850162999999,
                                         9.69786956522574e-19,
                                         1.9073486328125e-06,
                                         0.103115431,

                                         0.254809461222644,
                                         0.000495797712446421,
                                         7.11216551923777e-10,
                                         9.76371999606946e-23,
                                         0.361226551063445,
                                         0.00463997749538493,
                                         2.74345411988167e-08,
                                         2.48632789868145e-20,
                                         0.242567542347716,
                                         0.0205691401908986,
                                         5.01282399690131e-07,
                                         2.9990962137734e-18,
                                         0.10255831306539,
                                         0.0574118523637649,
                                         5.76703209867864e-06,
                                         2.27775784539415e-16,
                                         0.0306084525678916,
                                         0.11311476700183,
                                         4.68332832859675e-05,
                                         1.22111630009094e-14,
                                         0.00685130249756204,
                                         0.16714689546344,
                                         0.000285245042746454,
                                         4.90984485476472e-13,
                                         0.00119278025739561,
                                         0.19210242818478,
                                         0.00135125444726101,
                                         1.53544420764335e-11,
                                         0.000165278464143729,
                                         0.175725958658731,
                                         0.005094771304516,
                                         3.82180776671661e-10,
                                         1.84977131998011e-05,
                                         0.129832952940922,
                                         0.0155152293046205,
                                         7.68333171783715e-09,
                                         1.68685880090086e-06,
                                         0.0781614963638442,
                                         0.038499103781272,
                                         1.25860439911841e-07,
                                         1.2586043991184e-07,
                                         0.038499103781272,
                                         0.0781614963638442,
                                         1.68685880090087e-06,
                                         7.68333171783708e-09,
                                         0.0155152293046205,
                                         0.129832952940922,
                                         1.84977131998012e-05,
                                         3.82180776671657e-10,
                                         0.005094771304516,
                                         0.175725958658731,
                                         0.00016527846414373,
                                         1.53544420764334e-11,
                                         0.00135125444726101,
                                         0.19210242818478,
                                         0.00119278025739561,
                                         4.90984485476466e-13,
                                         0.000285245042746454,
                                         0.16714689546344,
                                         0.00685130249756207,
                                         1.22111630009093e-14,
                                         4.68332832859675e-05,
                                         0.11311476700183,
                                         0.0306084525678917,
                                         2.27775784539412e-16,
                                         5.76703209867864e-06,
                                         0.0574118523637649,
                                         0.10255831306539,
                                         2.99909621377336e-18,
                                         5.01282399690131e-07,
                                         0.0205691401908986,
                                         0.242567542347716,
                                         2.48632789868142e-20,
                                         2.74345411988167e-08,
                                         0.00463997749538493,
                                         0.361226551063445,
                                         9.76371999606931e-23,
                                         7.11216551923777e-10,
                                         0.000495797712446421,
                                         0.254809461222644,

                                         0.401368862495094,
                                         0.00684097478332492,
                                         1.9073486328125e-06,
                                         7.94769513471711e-13,
                                         5.67618806108385e-26,
                                         0.375344059640513,
                                         0.0389927026952088,
                                         3.62396240234375e-05,
                                         5.03365043869844e-11,
                                         2.19118026876954e-23,
                                         0.166266335355809,
                                         0.105277992689206,
                                         0.000326156616210938,
                                         1.51012818834642e-09,
                                         4.00671405631534e-21,
                                         0.0463729003913308,
                                         0.178968669858052,
                                         0.00184822082519531,
                                         2.85252679763534e-08,
                                         4.61300311583741e-19,
                                         0.0091297002604562,
                                         0.214757702676255,
                                         0.00739288330078125,
                                         3.80345232116279e-07,
                                         3.74896573412695e-17,
                                         0.00134806254529676,
                                         0.193277701463181,
                                         0.0221786499023438,
                                         3.80353558063435e-06,
                                         2.28507499172369e-15,
                                         0.000154817159859409,
                                         0.135291429427243,
                                         0.0517501831054688,
                                         2.95837021038883e-05,
                                         1.08329059908392e-13,
                                         1.41513110381359e-05,
                                         0.0753750035415497,
                                         0.0961074829101562,
                                         0.000183141212465143,
                                         4.08750237745017e-12,
                                         1.04476882225717e-06,
                                         0.0339180091115321,
                                         0.144161224365234,
                                         0.000915726107588918,
                                         1.24571016697991e-10,
                                         6.28496188207361e-08,
                                         0.0124363311033939,
                                         0.176197052001953,
                                         0.0037308176615556,
                                         3.09339170331754e-09,
                                         3.09339170331751e-09,
                                         0.0037308176615556,
                                         0.176197052001953,
                                         0.0124363311033939,
                                         6.28496188207367e-08,
                                         1.2457101669799e-10,
                                         0.000915726107588918,
                                         0.144161224365234,
                                         0.0339180091115321,
                                         1.04476882225718e-06,
                                         4.08750237745011e-12,
                                         0.000183141212465143,
                                         0.0961074829101562,
                                         0.0753750035415497,
                                         1.4151311038136e-05,
                                         1.0832905990839e-13,
                                         2.95837021038883e-05,
                                         0.0517501831054688,
                                         0.135291429427243,
                                         0.000154817159859411,
                                         2.28507499172366e-15,
                                         3.80353558063435e-06,
                                         0.0221786499023438,
                                         0.193277701463181,
                                         0.00134806254529677,
                                         3.74896573412689e-17,
                                         3.80345232116279e-07,
                                         0.00739288330078125,
                                         0.214757702676255,
                                         0.00912970026045624,
                                         4.61300311583732e-19,
                                         2.85252679763534e-08,
                                         0.00184822082519531,
                                         0.178968669858052,
                                         0.0463729003913309,
                                         4.00671405631526e-21,
                                         1.51012818834642e-09,
                                         0.000326156616210938,
                                         0.105277992689206,
                                         0.166266335355809,
                                         2.19118026876949e-23,
                                         5.03365043869844e-11,
                                         3.62396240234375e-05,
                                         0.0389927026952088,
                                         0.375344059640513,
                                         5.67618806108372e-26,
                                         7.94769513471711e-13,
                                         1.9073486328125e-06,
                                         0.00684097478332492,
                                         0.401368862495094,

                                         0.520679475935362,
                                         0.0294098787145247,
                                         0.000111236761760653,
                                         1.07365989166265e-08,
                                         2.23422084799104e-15,
                                         1.09882286299169e-28,
                                         0.345709474960788,
                                         0.113960363914142,
                                         0.00129917024216112,
                                         3.31860990188373e-07,
                                         2.08148223183531e-13,
                                         5.97439682604204e-26,
                                         0.108727903624765,
                                         0.209171894471508,
                                         0.00718741463685043,
                                         4.85886226500477e-06,
                                         9.18561469781966e-12,
                                         1.5386836976309e-23,
                                         0.0215305886513429,
                                         0.241734139731565,
                                         0.0250359740238777,
                                         4.47917601535812e-05,
                                         2.55228341338859e-10,
                                         2.49511057334454e-21,
                                         0.00300956077274384,
                                         0.197199120541737,
                                         0.0615585823106071,
                                         0.00029147008955613,
                                         5.00589685058804e-09,
                                         2.8560286872862e-19,
                                         0.000315508885689717,
                                         0.120651637742244,
                                         0.113520420230435,
                                         0.00142249622722852,
                                         7.36370120984836e-08,
                                         2.45186524489411e-17,
                                         2.57261976917379e-05,
                                         0.0574138935470523,
                                         0.16282268094894,
                                         0.00539962727752736,
                                         8.4249231985688e-07,
                                         1.63714128414147e-15,
                                         1.66958347454097e-06,
                                         0.0217454953449016,
                                         0.185876437198651,
                                         0.0163134181773341,
                                         7.67192360971534e-06,
                                         8.70050077622616e-14,
                                         8.75158306137358e-08,
                                         0.0066522346034587,
                                         0.171387729133026,
                                         0.0398081598111802,
                                         5.64272046077905e-05,
                                         3.73463606324509e-12,
                                         3.73786848848394e-09,
                                         0.00165815347617668,
                                         0.128763871526997,
                                         0.0791513181838593,
                                         0.000338167384619698,
                                         1.30620469309776e-10,
                                         1.30620469309776e-10,
                                         0.000338167384619699,
                                         0.0791513181838593,
                                         0.128763871526997,
                                         0.00165815347617667,
                                         3.73786848848394e-09,
                                         3.73463606324509e-12,
                                         5.64272046077907e-05,
                                         0.0398081598111802,
                                         0.171387729133026,
                                         0.00665223460345868,
                                         8.75158306137358e-08,
                                         8.70050077622616e-14,
                                         7.67192360971537e-06,
                                         0.0163134181773341,
                                         0.185876437198651,
                                         0.0217454953449016,
                                         1.66958347454097e-06,
                                         1.63714128414147e-15,
                                         8.42492319856884e-07,
                                         0.00539962727752736,
                                         0.16282268094894,
                                         0.0574138935470522,
                                         2.57261976917379e-05,
                                         2.45186524489411e-17,
                                         7.36370120984839e-08,
                                         0.00142249622722852,
                                         0.113520420230435,
                                         0.120651637742244,
                                         0.000315508885689717,
                                         2.8560286872862e-19,
                                         5.00589685058807e-09,
                                         0.00029147008955613,
                                         0.0615585823106071,
                                         0.197199120541737,
                                         0.00300956077274384,
                                         2.49511057334454e-21,
                                         2.5522834133886e-10,
                                         4.47917601535812e-05,
                                         0.0250359740238777,
                                         0.241734139731565,
                                         0.0215305886513429,
                                         1.5386836976309e-23,
                                         9.18561469781971e-12,
                                         4.85886226500477e-06,
                                         0.00718741463685043,
                                         0.209171894471508,
                                         0.108727903624765,
                                         5.97439682604204e-26,
                                         2.08148223183533e-13,
                                         3.31860990188373e-07,
                                         0.00129917024216112,
                                         0.113960363914142,
                                         0.345709474960788,
                                         1.09882286299169e-28,
                                         2.23422084799105e-15,
                                         1.07365989166265e-08,
                                         0.000111236761760653,
                                         0.0294098787145247,
                                         0.520679475935362,

                                         0.612790307944367,
                                         0.0721313303443259,
                                         0.00123379311632532,
                                         1.9073486328125e-06,
                                         9.6499118872523e-11,
                                         1.30672724447751e-17,
                                         5.09078958220815e-31,
                                         0.30400440095112,
                                         0.203401634262193,
                                         0.00990736360212172,
                                         3.62396240234375e-05,
                                         4.33825214955145e-09,
                                         1.67286791636893e-15,
                                         3.70445535875944e-28,
                                         0.0714392295628007,
                                         0.271690117987294,
                                         0.0376845010558261,
                                         0.000326156616210938,
                                         9.23836553321819e-08,
                                         1.01444210484393e-13,
                                         1.27688708102859e-25,
                                         0.0105700930617023,
                                         0.228495895918213,
                                         0.09025111880712,
                                         0.00184822082519531,
                                         1.23868434953057e-06,
                                         3.87327237347787e-12,
                                         2.77118664644376e-23,
                                         0.0011039596099013,
                                         0.13564859200166,
                                         0.152571966493826,
                                         0.00739288330078125,
                                         1.17235322805939e-05,
                                         1.04390537703747e-10,
                                         4.24532933727547e-21,
                                         8.64746516311995e-05,
                                         0.0603967319787847,
                                         0.193445288552672,
                                         0.0221786499023438,
                                         8.3218058652037e-05,
                                         2.11011193726594e-09,
                                         4.877735645773e-19,
                                         5.26841513178601e-06,
                                         0.0209154455307641,
                                         0.19076429865435,
                                         0.0517501831054688,
                                         0.000459443601130553,
                                         3.31745762915773e-08,
                                         4.35893770878919e-17,
                                         2.5546986752601e-07,
                                         0.00576486746655429,
                                         0.149728533394082,
                                         0.0961074829101562,
                                         0.00201890257257627,
                                         4.15120138017706e-07,
                                         3.10035620167248e-15,
                                         1.00056503872255e-08,
                                         0.00128338654361571,
                                         0.0949200564579431,
                                         0.144161224365234,
                                         0.00716546637464837,
                                         4.19554290024126e-06,
                                         1.78110068242627e-13,
                                         3.19308060360906e-10,
                                         0.000232800844833732,
                                         0.0490309515500246,
                                         0.176197052001953,
                                         0.0207220386736651,
                                         3.45510656674945e-05,
                                         8.33727763077995e-12,
                                         8.33727763077977e-12,
                                         3.45510656674945e-05,
                                         0.0207220386736651,
                                         0.176197052001953,
                                         0.0490309515500246,
                                         0.000232800844833732,
                                         3.19308060360912e-10,
                                         1.78110068242623e-13,
                                         4.19554290024126e-06,
                                         0.00716546637464837,
                                         0.144161224365234,
                                         0.0949200564579431,
                                         0.00128338654361571,
                                         1.00056503872256e-08,
                                         3.1003562016724e-15,
                                         4.15120138017706e-07,
                                         0.00201890257257627,
                                         0.0961074829101562,
                                         0.149728533394082,
                                         0.00576486746655429,
                                         2.55469867526014e-07,
                                         4.35893770878907e-17,
                                         3.31745762915773e-08,
                                         0.000459443601130553,
                                         0.0517501831054688,
                                         0.19076429865435,
                                         0.0209154455307641,
                                         5.26841513178607e-06,
                                         4.87773564577285e-19,
                                         2.11011193726594e-09,
                                         8.3218058652037e-05,
                                         0.0221786499023438,
                                         0.193445288552672,
                                         0.0603967319787847,
                                         8.64746516312004e-05,
                                         4.24532933727533e-21,
                                         1.04390537703747e-10,
                                         1.17235322805939e-05,
                                         0.00739288330078125,
                                         0.152571966493826,
                                         0.13564859200166,
                                         0.00110395960990131,
                                         2.77118664644366e-23,
                                         3.87327237347787e-12,
                                         1.23868434953057e-06,
                                         0.00184822082519531,
                                         0.09025111880712,
                                         0.228495895918213,
                                         0.0105700930617024,
                                         1.27688708102854e-25,
                                         1.01444210484393e-13,
                                         9.23836553321819e-08,
                                         0.000326156616210938,
                                         0.0376845010558261,
                                         0.271690117987294,
                                         0.071439229562801,
                                         3.7044553587593e-28,
                                         1.67286791636893e-15,
                                         4.33825214955145e-09,
                                         3.62396240234375e-05,
                                         0.00990736360212172,
                                         0.203401634262193,
                                         0.30400440095112,
                                         5.09078958220794e-31,
                                         1.30672724447751e-17,
                                         9.6499118872523e-11,
                                         1.9073486328125e-06,
                                         0.00123379311632532,
                                         0.0721313303443259,
                                         0.612790307944367,

                                         0.683149324236735,
                                         0.130410305433639,
                                         0.00582689801406785,
                                         4.67919740434308e-05,
                                         4.05749651017726e-08,
                                         1.34386168053581e-12,
                                         1.3689916149813e-19,
                                         4.56622971419915e-33,
                                         0.262936215567076,
                                         0.280418739513421,
                                         0.0344331005805671,
                                         0.000613439364693709,
                                         1.11728786763458e-06,
                                         8.20962005161804e-11,
                                         2.29833255790846e-17,
                                         4.28282365743426e-30,
                                         0.0479373564873563,
                                         0.285621609459299,
                                         0.0963837354803872,
                                         0.00380943724761869,
                                         1.45734011422672e-05,
                                         2.37563904284175e-09,
                                         1.82773765358318e-15,
                                         1.90279297154481e-27,
                                         0.00550278964814737,
                                         0.183172489726601,
                                         0.169869940223152,
                                         0.0148948167187753,
                                         0.000119685603202966,
                                         4.32835633789979e-08,
                                         9.15166271467092e-14,
                                         5.32277417862139e-25,
                                         0.000445886236400224,
                                         0.0829204708725723,
                                         0.211330236293719,
                                         0.0411094652875121,
                                         0.000693833446333159,
                                         5.56670087747697e-07,
                                         3.23458432291919e-12,
                                         1.05103425074564e-22,
                                         2.70973290626453e-05,
                                         0.0281529905227245,
                                         0.197182335703231,
                                         0.0850961194152301,
                                         0.00301668395175006,
                                         5.36950222674001e-06,
                                         8.57429087063783e-11,
                                         1.55652807986477e-20,
                                         1.28080918821123e-06,
                                         0.00743434719131778,
                                         0.143096794522421,
                                         0.137003989557595,
                                         0.0102014034876152,
                                         4.02833610433095e-05,
                                         1.76780141730399e-09,
                                         1.79288560892611e-18,
                                         4.8184894292128e-08,
                                         0.00156253470206978,
                                         0.0826533253437519,
                                         0.175559849277488,
                                         0.0274573465061064,
                                         0.000240539219649846,
                                         2.90093013982465e-08,
                                         1.64367795226866e-16,
                                         1.46414245371616e-09,
                                         0.000265254310521222,
                                         0.0385599714237495,
                                         0.181703432459569,
                                         0.0596902160023855,
                                         0.00116009096227346,
                                         3.84491735738837e-07,
                                         1.21710157644518e-14,
                                         3.62505560806876e-11,
                                         3.66905435515565e-05,
                                         0.0146579080310414,
                                         0.153235708311367,
                                         0.10573205012474,
                                         0.00455886892598875,
                                         4.1523663739117e-06,
                                         7.34337720126867e-13,
                                         7.34337720126867e-13,
                                         4.1523663739117e-06,
                                         0.00455886892598875,
                                         0.10573205012474,
                                         0.153235708311367,
                                         0.0146579080310414,
                                         3.66905435515565e-05,
                                         3.62505560806876e-11,
                                         1.21710157644518e-14,
                                         3.84491735738837e-07,
                                         0.00116009096227345,
                                         0.0596902160023856,
                                         0.181703432459569,
                                         0.0385599714237495,
                                         0.000265254310521222,
                                         1.46414245371616e-09,
                                         1.64367795226866e-16,
                                         2.90093013982465e-08,
                                         0.000240539219649846,
                                         0.0274573465061064,
                                         0.175559849277488,
                                         0.0826533253437519,
                                         0.00156253470206978,
                                         4.8184894292128e-08,
                                         1.79288560892611e-18,
                                         1.76780141730399e-09,
                                         4.02833610433095e-05,
                                         0.0102014034876152,
                                         0.137003989557595,
                                         0.143096794522421,
                                         0.00743434719131778,
                                         1.28080918821123e-06,
                                         1.55652807986477e-20,
                                         8.57429087063783e-11,
                                         5.36950222674001e-06,
                                         0.00301668395175006,
                                         0.0850961194152301,
                                         0.197182335703231,
                                         0.0281529905227245,
                                         2.70973290626453e-05,
                                         1.05103425074564e-22,
                                         3.23458432291919e-12,
                                         5.56670087747697e-07,
                                         0.00069383344633316,
                                         0.0411094652875121,
                                         0.211330236293719,
                                         0.0829204708725723,
                                         0.000445886236400224,
                                         5.32277417862139e-25,
                                         9.15166271467092e-14,
                                         4.32835633789979e-08,
                                         0.000119685603202966,
                                         0.0148948167187753,
                                         0.169869940223152,
                                         0.183172489726601,
                                         0.00550278964814737,
                                         1.90279297154481e-27,
                                         1.82773765358318e-15,
                                         2.37563904284175e-09,
                                         1.45734011422673e-05,
                                         0.00380943724761869,
                                         0.0963837354803872,
                                         0.285621609459299,
                                         0.0479373564873563,
                                         4.28282365743426e-30,
                                         2.29833255790846e-17,
                                         8.20962005161804e-11,
                                         1.11728786763459e-06,
                                         0.000613439364693709,
                                         0.0344331005805671,
                                         0.280418739513421,
                                         0.262936215567076,
                                         4.56622971419915e-33,
                                         1.3689916149813e-19,
                                         1.34386168053581e-12,
                                         4.05749651017727e-08,
                                         4.67919740434308e-05,
                                         0.00582689801406785,
                                         0.130410305433639,
                                         0.683149324236735,

                                         0.737188766121386,
                                         0.196856948292377,
                                         0.0168804094111517,
                                         0.000396169524884245,
                                         1.9073486328125e-06,
                                         1.11255966808156e-09,
                                         2.74798871440228e-14,
                                         2.29560886918314e-21,
                                         6.86840876944313e-35,
                                         0.226590473842753,
                                         0.334030234159595,
                                         0.0768592520617493,
                                         0.00384102749047759,
                                         3.62396240234375e-05,
                                         4.14251570104365e-08,
                                         2.17875786787298e-12,
                                         4.88393714696574e-19,
                                         8.06678341650473e-32,
                                         0.0329908492457428,
                                         0.26847862575101,
                                         0.165767062162106,
                                         0.0176401662875638,
                                         0.000326156616210938,
                                         7.30623980904056e-07,
                                         8.18261090900758e-11,
                                         4.92188142742911e-17,
                                         4.48780076294796e-29,
                                         0.00302433921025375,
                                         0.135868509619878,
                                         0.225105177818455,
                                         0.0510085632826681,
                                         0.00184822082519531,
                                         8.11351119118921e-06,
                                         1.93490652817049e-09,
                                         3.12303883620762e-15,
                                         1.57199766470069e-26,
                                         0.000195704057361234,
                                         0.0485355808682131,
                                         0.215776951612307,
                                         0.104115606291598,
                                         0.00739288330078125,
                                         6.35998484809371e-05,
                                         3.22968676557068e-08,
                                         1.39880104309513e-13,
                                         3.88689268062022e-24,
                                         9.49796188646667e-06,
                                         0.0130035794357654,
                                         0.155126461211333,
                                         0.159385877230987,
                                         0.0221786499023438,
                                         0.000373907852421901,
                                         4.04317073661116e-07,
                                         4.69889535669371e-12,
                                         7.20799482558498e-22,
                                         3.58522607208234e-07,
                                         0.00270969948649002,
                                         0.0867405652596281,
                                         0.189775162606582,
                                         0.0517501831054688,
                                         0.00170973428762289,
                                         3.93676317184764e-06,
                                         1.22769681757135e-10,
                                         1.03963744513547e-19,
                                         1.0771376254338e-08,
                                         0.000449415317058989,
                                         0.0386035405038359,
                                         0.179844613300449,
                                         0.0961074829101562,
                                         0.00622244667809838,
                                         3.0508792856118e-05,
                                         2.55302468298263e-09,
                                         1.19348753471833e-17,
                                         2.61379663017552e-10,
                                         6.02033360937236e-05,
                                         0.0138764371816412,
                                         0.137657996131137,
                                         0.144161224365234,
                                         0.0182910944741867,
                                         0.000190966287814899,
                                         4.28809940041476e-08,
                                         1.1066232954594e-15,
                                         5.16810475031472e-12,
                                         6.57131312974926e-06,
                                         0.00406431843406985,
                                         0.0858547580940349,
                                         0.176197052001953,
                                         0.0438103899459183,
                                         0.00097397386699894,
                                         5.86858759089225e-07,
                                         8.36066160399077e-14,
                                         8.36066160399047e-14,
                                         5.86858759089225e-07,
                                         0.000973973866998937,
                                         0.0438103899459183,
                                         0.176197052001953,
                                         0.0858547580940349,
                                         0.00406431843406986,
                                         6.57131312974926e-06,
                                         5.16810475031488e-12,
                                         1.10662329545935e-15,
                                         4.28809940041476e-08,
                                         0.000190966287814898,
                                         0.0182910944741867,
                                         0.144161224365234,
                                         0.137657996131137,
                                         0.0138764371816413,
                                         6.02033360937236e-05,
                                         2.61379663017559e-10,
                                         1.19348753471828e-17,
                                         2.55302468298263e-09,
                                         3.05087928561179e-05,
                                         0.00622244667809838,
                                         0.0961074829101562,
                                         0.179844613300449,
                                         0.038603540503836,
                                         0.000449415317058989,
                                         1.07713762543382e-08,
                                         1.03963744513542e-19,
                                         1.22769681757135e-10,
                                         3.93676317184763e-06,
                                         0.00170973428762289,
                                         0.0517501831054688,
                                         0.189775162606582,
                                         0.0867405652596283,
                                         0.00270969948649002,
                                         3.58522607208242e-07,
                                         7.20799482558462e-22,
                                         4.69889535669371e-12,
                                         4.04317073661114e-07,
                                         0.000373907852421901,
                                         0.0221786499023438,
                                         0.159385877230987,
                                         0.155126461211333,
                                         0.0130035794357654,
                                         9.49796188646684e-06,
                                         3.88689268062001e-24,
                                         1.39880104309513e-13,
                                         3.22968676557067e-08,
                                         6.35998484809371e-05,
                                         0.00739288330078125,
                                         0.104115606291598,
                                         0.215776951612307,
                                         0.0485355808682131,
                                         0.000195704057361237,
                                         1.57199766470061e-26,
                                         3.12303883620762e-15,
                                         1.93490652817048e-09,
                                         8.11351119118921e-06,
                                         0.00184822082519531,
                                         0.0510085632826681,
                                         0.225105177818455,
                                         0.135868509619878,
                                         0.00302433921025378,
                                         4.48780076294769e-29,
                                         4.92188142742911e-17,
                                         8.18261090900754e-11,
                                         7.30623980904056e-07,
                                         0.000326156616210938,
                                         0.0176401662875638,
                                         0.165767062162106,
                                         0.26847862575101,
                                         0.0329908492457431,
                                         8.06678341650422e-32,
                                         4.88393714696574e-19,
                                         2.17875786787297e-12,
                                         4.14251570104365e-08,
                                         3.62396240234375e-05,
                                         0.00384102749047759,
                                         0.0768592520617493,
                                         0.334030234159595,
                                         0.226590473842754,
                                         6.86840876944268e-35,
                                         2.29560886918314e-21,
                                         2.74798871440226e-14,
                                         1.11255966808156e-09,
                                         1.9073486328125e-06,
                                         0.000396169524884245,
                                         0.0168804094111517,
                                         0.196856948292377,
                                         0.737188766121386,

                                         0.779175283441997,
                                         0.265221255742373,
                                         0.0361747820702779,
                                         0.00178395613079668,
                                         2.66445345407686e-05,
                                         8.91887520976561e-08,
                                         3.91518233520736e-11,
                                         7.82511362391902e-16,
                                         5.66105816364965e-23,
                                         1.56507607627284e-36,
                                         0.195701451488656,
                                         0.364584504451803,
                                         0.131205928065874,
                                         0.0133983669306411,
                                         0.000375044580401337,
                                         2.28740219308582e-06,
                                         1.88187814038188e-09,
                                         7.78843716086291e-14,
                                         1.48667123844921e-20,
                                         2.24948849162009e-33,
                                         0.0232831552700881,
                                         0.237397966310843,
                                         0.225418678819798,
                                         0.0476659740963754,
                                         0.00250061342670039,
                                         2.77884221171296e-05,
                                         4.28469498431167e-08,
                                         3.67196807421378e-12,
                                         1.84935876744146e-18,
                                         1.53151412609846e-30,
                                         0.00174411364266469,
                                         0.0973287037350566,
                                         0.243843677692326,
                                         0.106770243997328,
                                         0.0104977295659831,
                                         0.000212554546450223,
                                         6.14233475843847e-07,
                                         1.09001539082401e-10,
                                         1.4484801859599e-16,
                                         6.56513098537503e-28,
                                         9.22231705305118e-05,
                                         0.0281667804774889,
                                         0.186193889506332,
                                         0.16882016578707,
                                         0.0311083178129277,
                                         0.00114764940236137,
                                         6.2155473222214e-06,
                                         2.28401361479246e-09,
                                         8.00822509327611e-15,
                                         1.98654363351084e-25,
                                         3.65735049080815e-06,
                                         0.0061135679302792,
                                         0.106630295331789,
                                         0.200197971663483,
                                         0.0691383382759694,
                                         0.0046473923025996,
                                         4.71722439847064e-05,
                                         3.58943431198731e-08,
                                         3.32063581708975e-13,
                                         4.50831325753472e-23,
                                         1.12810271660314e-07,
                                         0.00103206679575397,
                                         0.047495376035065,
                                         0.184650528171477,
                                         0.119513475956836,
                                         0.01463743448989,
                                         0.000278451284431729,
                                         4.38741647969545e-07,
                                         1.07093165268825e-11,
                                         7.95766389848802e-21,
                                         2.76948639173529e-09,
                                         0.000138672197138166,
                                         0.0168380027401066,
                                         0.135553258684154,
                                         0.164430866438847,
                                         0.0366934985336105,
                                         0.00130821896861156,
                                         4.2683521375254e-06,
                                         2.74897499032401e-10,
                                         1.11795835602023e-18,
                                         5.49156050254659e-11,
                                         1.50493219827307e-05,
                                         0.00482142891406212,
                                         0.0803739509270803,
                                         0.182724069736348,
                                         0.0742949381098769,
                                         0.00496429527228748,
                                         3.35395759439688e-05,
                                         5.69935579159622e-09,
                                         1.26856173526637e-16,
                                         8.87260661531231e-13,
                                         1.33077118394524e-06,
                                         0.00112491543772092,
                                         0.0388310906716984,
                                         0.165450121525802,
                                         0.122571145747793,
                                         0.0153494806218034,
                                         0.000214740425810072,
                                         9.62807947848282e-08,
                                         1.17288789692044e-14,
                                         1.17288789692044e-14,
                                         9.6280794784829e-08,
                                         0.000214740425810072,
                                         0.0153494806218034,
                                         0.122571145747793,
                                         0.165450121525802,
                                         0.0388310906716984,
                                         0.00112491543772092,
                                         1.33077118394523e-06,
                                         8.87260661531231e-13,
                                         1.26856173526637e-16,
                                         5.69935579159627e-09,
                                         3.35395759439688e-05,
                                         0.00496429527228748,
                                         0.0742949381098769,
                                         0.182724069736348,
                                         0.0803739509270803,
                                         0.00482142891406212,
                                         1.50493219827306e-05,
                                         5.49156050254659e-11,
                                         1.11795835602023e-18,
                                         2.74897499032404e-10,
                                         4.2683521375254e-06,
                                         0.00130821896861156,
                                         0.0366934985336105,
                                         0.164430866438847,
                                         0.135553258684154,
                                         0.0168380027401066,
                                         0.000138672197138165,
                                         2.76948639173529e-09,
                                         7.95766389848802e-21,
                                         1.07093165268826e-11,
                                         4.38741647969545e-07,
                                         0.000278451284431729,
                                         0.01463743448989,
                                         0.119513475956836,
                                         0.184650528171477,
                                         0.047495376035065,
                                         0.00103206679575397,
                                         1.12810271660314e-07,
                                         4.50831325753472e-23,
                                         3.32063581708979e-13,
                                         3.58943431198731e-08,
                                         4.71722439847064e-05,
                                         0.0046473923025996,
                                         0.0691383382759694,
                                         0.200197971663483,
                                         0.106630295331789,
                                         0.00611356793027917,
                                         3.65735049080815e-06,
                                         1.98654363351084e-25,
                                         8.00822509327621e-15,
                                         2.28401361479246e-09,
                                         6.2155473222214e-06,
                                         0.00114764940236137,
                                         0.0311083178129277,
                                         0.16882016578707,
                                         0.186193889506332,
                                         0.0281667804774888,
                                         9.22231705305118e-05,
                                         6.56513098537503e-28,
                                         1.44848018595992e-16,
                                         1.09001539082401e-10,
                                         6.14233475843847e-07,
                                         0.000212554546450223,
                                         0.0104977295659831,
                                         0.106770243997328,
                                         0.243843677692326,
                                         0.0973287037350564,
                                         0.00174411364266469,
                                         1.53151412609846e-30,
                                         1.84935876744148e-18,
                                         3.67196807421378e-12,
                                         4.28469498431167e-08,
                                         2.77884221171296e-05,
                                         0.00250061342670039,
                                         0.0476659740963754,
                                         0.225418678819798,
                                         0.237397966310843,
                                         0.0232831552700881,
                                         2.24948849162009e-33,
                                         1.48667123844923e-20,
                                         7.78843716086291e-14,
                                         1.88187814038188e-09,
                                         2.28740219308582e-06,
                                         0.000375044580401337,
                                         0.0133983669306411,
                                         0.131205928065874,
                                         0.364584504451803,
                                         0.195701451488656,
                                         1.56507607627284e-36,
                                         5.66105816364974e-23,
                                         7.82511362391902e-16,
                                         3.91518233520736e-11,
                                         8.91887520976561e-08,
                                         2.66445345407686e-05,
                                         0.00178395613079668,
                                         0.0361747820702779,
                                         0.265221255742373,
                                         0.779175283441997,

                                         0.8122381335934,
                                         0.331413290920461,
                                         0.0636841024317143,
                                         0.00537712295872358,
                                         0.0001777182323861,
                                         1.9073486328125e-06,
                                         4.88345029064924e-09,
                                         1.73595020757e-12,
                                         2.96274845574689e-17,
                                         1.92463341299466e-24,
                                         5.01474470293631e-38,
                                         0.16984223032489,
                                         0.376855624283835,
                                         0.188720712394255,
                                         0.0323427232170584,
                                         0.00194282051180911,
                                         3.62396240234375e-05,
                                         1.61262459164436e-07,
                                         1.04188035527456e-10,
                                         3.60922076441231e-15,
                                         6.11012567713634e-22,
                                         8.65752433968245e-35,
                                         0.0168227465423245,
                                         0.202987357320083,
                                         0.264909181528225,
                                         0.0921493145774629,
                                         0.0100605629643862,
                                         0.000326156616210938,
                                         2.52248558845466e-06,
                                         2.96201561232049e-09,
                                         2.0826726919547e-13,
                                         9.18842709268816e-20,
                                         7.07990650508731e-32,
                                         0.00104913959172163,
                                         0.0688411529316104,
                                         0.234131372825782,
                                         0.16530758888499,
                                         0.0328017522532939,
                                         0.00184822082519531,
                                         2.48432990008994e-05,
                                         5.30202772447227e-08,
                                         7.56682613413368e-12,
                                         8.69996211437964e-18,
                                         3.64541084434765e-29,
                                         4.61851093094241e-05,
                                         0.0164800909986103,
                                         0.146067816436208,
                                         0.20932726464859,
                                         0.0754925559413547,
                                         0.00739288330078125,
                                         0.000172711860939518,
                                         6.69929296256946e-07,
                                         1.94061244585518e-10,
                                         5.81468073170937e-16,
                                         1.32494473723931e-26,
                                         1.52486690433381e-06,
                                         0.00295891397539805,
                                         0.0683456260291028,
                                         0.19880168851056,
                                         0.130308419773206,
                                         0.0221786499023438,
                                         0.000900526141116917,
                                         6.34858895463429e-06,
                                         3.73271758685573e-09,
                                         2.91471200396053e-14,
                                         3.61168870607561e-24,
                                         3.91577231875535e-08,
                                         0.000413200276367199,
                                         0.0248726721440759,
                                         0.146848620002318,
                                         0.174942914517098,
                                         0.0517501831054688,
                                         0.00365196003007856,
                                         4.67930096140445e-05,
                                         5.58427740404361e-08,
                                         1.13637312951907e-12,
                                         7.65734806397497e-22,
                                         8.00334341313803e-10,
                                         4.5925870255312e-05,
                                         0.00720448085668637,
                                         0.0863352592642561,
                                         0.186934200043048,
                                         0.0961074829101562,
                                         0.0117875686187969,
                                         0.000274506906456933,
                                         6.64932196475677e-07,
                                         3.52626372283812e-11,
                                         1.29215613110185e-19,
                                         1.32120868362023e-11,
                                         4.12287466562593e-06,
                                         0.00168550049688721,
                                         0.0409970372195535,
                                         0.161334451492814,
                                         0.144161224365234,
                                         0.0307304131392379,
                                         0.0013006831646566,
                                         6.3948989638619e-06,
                                         8.83801164356045e-10,
                                         1.76115485197931e-17,
                                         1.77717544162837e-13,
                                         3.0157947942921e-07,
                                         0.000321302418609587,
                                         0.015862647137452,
                                         0.113455195011746,
                                         0.176197052001953,
                                         0.065278697538189,
                                         0.00502167589184121,
                                         5.01128299633054e-05,
                                         1.8049005037889e-08,
                                         1.95586561314838e-15,
                                         1.95586561314828e-15,
                                         1.8049005037889e-08,
                                         5.01128299633056e-05,
                                         0.00502167589184123,
                                         0.0652786975381889,
                                         0.176197052001953,
                                         0.113455195011746,
                                         0.015862647137452,
                                         0.000321302418609586,
                                         3.0157947942921e-07,
                                         1.77717544162845e-13,
                                         1.76115485197921e-17,
                                         8.83801164356045e-10,
                                         6.39489896386193e-06,
                                         0.00130068316465661,
                                         0.0307304131392379,
                                         0.144161224365234,
                                         0.161334451492814,
                                         0.0409970372195534,
                                         0.0016855004968872,
                                         4.12287466562593e-06,
                                         1.32120868362028e-11,
                                         1.29215613110177e-19,
                                         3.52626372283812e-11,
                                         6.64932196475681e-07,
                                         0.000274506906456933,
                                         0.0117875686187969,
                                         0.0961074829101562,
                                         0.186934200043049,
                                         0.0863352592642559,
                                         0.00720448085668635,
                                         4.5925870255312e-05,
                                         8.00334341313831e-10,
                                         7.65734806397446e-22,
                                         1.13637312951907e-12,
                                         5.58427740404364e-08,
                                         4.67930096140446e-05,
                                         0.00365196003007856,
                                         0.0517501831054688,
                                         0.174942914517098,
                                         0.146848620002318,
                                         0.0248726721440759,
                                         0.000413200276367199,
                                         3.91577231875547e-08,
                                         3.61168870607536e-24,
                                         2.91471200396053e-14,
                                         3.73271758685575e-09,
                                         6.34858895463431e-06,
                                         0.000900526141116915,
                                         0.0221786499023438,
                                         0.130308419773206,
                                         0.19880168851056,
                                         0.0683456260291027,
                                         0.00295891397539805,
                                         1.52486690433385e-06,
                                         1.3249447372392e-26,
                                         5.81468073170937e-16,
                                         1.9406124458552e-10,
                                         6.69929296256948e-07,
                                         0.000172711860939518,
                                         0.00739288330078125,
                                         0.0754925559413547,
                                         0.20932726464859,
                                         0.146067816436208,
                                         0.0164800909986103,
                                         4.61851093094251e-05,
                                         3.64541084434736e-29,
                                         8.69996211437964e-18,
                                         7.56682613413373e-12,
                                         5.30202772447229e-08,
                                         2.48432990008993e-05,
                                         0.00184822082519531,
                                         0.0328017522532939,
                                         0.16530758888499,
                                         0.234131372825782,
                                         0.0688411529316104,
                                         0.00104913959172164,
                                         7.07990650508669e-32,
                                         9.18842709268816e-20,
                                         2.08267269195471e-13,
                                         2.9620156123205e-09,
                                         2.52248558845466e-06,
                                         0.000326156616210938,
                                         0.0100605629643862,
                                         0.0921493145774629,
                                         0.264909181528224,
                                         0.202987357320083,
                                         0.0168227465423246,
                                         8.65752433968166e-35,
                                         6.11012567713634e-22,
                                         3.60922076441234e-15,
                                         1.04188035527456e-10,
                                         1.61262459164435e-07,
                                         3.62396240234375e-05,
                                         0.00194282051180911,
                                         0.0323427232170584,
                                         0.188720712394255,
                                         0.376855624283835,
                                         0.169842230324891,
                                         5.01474470293582e-38,
                                         1.92463341299466e-24,
                                         2.96274845574691e-17,
                                         1.73595020757001e-12,
                                         4.88345029064923e-09,
                                         1.9073486328125e-06,
                                         0.0001777182323861,
                                         0.00537712295872358,
                                         0.0636841024317143,
                                         0.331413290920461,
                                         0.8122381335934,

                                         0.838629307363553,
                                         0.393196978423626,
                                         0.0980546978663394,
                                         0.0123892417160216,
                                         0.000732908054150508,
                                         1.79489821095276e-05,
                                         1.50100335111343e-07,
                                         3.13512080279616e-10,
                                         9.48693668476311e-14,
                                         1.43466313051493e-18,
                                         8.58022511338166e-26,
                                         2.13601196070215e-39,
                                         0.148273059622896,
                                         0.376192849514548,
                                         0.242205606965375,
                                         0.0611997958226088,
                                         0.00643581196936376,
                                         0.000265120308245026,
                                         3.66847608659165e-06,
                                         1.28886558928879e-08,
                                         6.93310786498104e-12,
                                         2.09672441629335e-16,
                                         3.2374682616106e-23,
                                         4.36132718297179e-36,
                                         0.0124177631380206,
                                         0.170490352850525,
                                         0.283392849467968,
                                         0.143200366266881,
                                         0.0267698605240769,
                                         0.00185496225015411,
                                         4.24696450688722e-05,
                                         2.5098622493674e-07,
                                         2.40004188131317e-10,
                                         1.45151549441381e-14,
                                         5.78630424566575e-21,
                                         4.21815529722242e-33,
                                         0.000654801478272278,
                                         0.048649036110689,
                                         0.208775119298623,
                                         0.210971330381881,
                                         0.0701090410631181,
                                         0.0081716980894106,
                                         0.000309568475007953,
                                         3.07735289118018e-06,
                                         5.23112190526453e-09,
                                         6.32684444657375e-13,
                                         6.51151684203637e-19,
                                         2.56868954831371e-30,
                                         2.43729588587463e-05,
                                         0.00979898279188143,
                                         0.108567770071868,
                                         0.219399219856988,
                                         0.129608734491076,
                                         0.0254110069262992,
                                         0.00159282164495174,
                                         2.66340395727958e-05,
                                         8.048282020057e-08,
                                         1.94663694868984e-11,
                                         5.17243921678958e-17,
                                         1.10416250941468e-27,
                                         6.80405981707854e-07,
                                         0.00148029752640605,
                                         0.0423432665419239,
                                         0.171122840198442,
                                         0.179703185940614,
                                         0.0592642373050826,
                                         0.00614665493448964,
                                         0.000172885290308679,
                                         9.28694331456665e-07,
                                         4.49204430660921e-10,
                                         3.08155473990327e-15,
                                         3.55971836299939e-25,
                                         1.47735042410827e-08,
                                         0.000173929225246789,
                                         0.0128446811934198,
                                         0.103809335530345,
                                         0.193790647544852,
                                         0.107502621621057,
                                         0.018448711862583,
                                         0.000872839872049044,
                                         8.33485279344599e-06,
                                         8.06229399106025e-09,
                                         1.42790695322617e-13,
                                         8.92593687883723e-23,
                                         2.55309777041531e-10,
                                         1.62653958983528e-05,
                                         0.00310120751471056,
                                         0.0501225744485293,
                                         0.166332983718312,
                                         0.155207941156614,
                                         0.0440719016182569,
                                         0.00350735428762082,
                                         5.95376378554288e-05,
                                         1.15170633857645e-07,
                                         5.26621343852048e-12,
                                         1.78139615564025e-20,
                                         3.5636686644063e-12,
                                         1.22857779351829e-06,
                                         0.000604761690832741,
                                         0.0195468272477929,
                                         0.115310774457284,
                                         0.18099007809917,
                                         0.0850361283112803,
                                         0.0113833645582262,
                                         0.00034350353682229,
                                         1.32883430178649e-06,
                                         1.56871103773816e-10,
                                         2.87152867125087e-18,
                                         4.05308868788114e-14,
                                         7.56135451753303e-08,
                                         9.60940802145512e-05,
                                         0.00621123693967561,
                                         0.0651358847445362,
                                         0.171970721576428,
                                         0.133691588617369,
                                         0.0301037479035534,
                                         0.00161484090805731,
                                         1.24927721764089e-05,
                                         3.8075565686461e-09,
                                         3.77159212060812e-16,
                                         3.77159212060834e-16,
                                         3.80755656864605e-09,
                                         1.24927721764088e-05,
                                         0.0016148409080573,
                                         0.0301037479035534,
                                         0.133691588617369,
                                         0.171970721576428,
                                         0.0651358847445363,
                                         0.00621123693967562,
                                         9.60940802145516e-05,
                                         7.56135451753311e-08,
                                         4.05308868788092e-14,
                                         2.87152867125106e-18,
                                         1.56871103773814e-10,
                                         1.32883430178648e-06,
                                         0.000343503536822289,
                                         0.0113833645582262,
                                         0.0850361283112803,
                                         0.18099007809917,
                                         0.115310774457285,
                                         0.019546827247793,
                                         0.000604761690832743,
                                         1.2285777935183e-06,
                                         3.56366866440613e-12,
                                         1.78139615564038e-20,
                                         5.26621343852041e-12,
                                         1.15170633857644e-07,
                                         5.95376378554286e-05,
                                         0.00350735428762081,
                                         0.0440719016182569,
                                         0.155207941156614,
                                         0.166332983718312,
                                         0.0501225744485293,
                                         0.00310120751471057,
                                         1.62653958983529e-05,
                                         2.5530977704152e-10,
                                         8.92593687883793e-23,
                                         1.42790695322615e-13,
                                         8.0622939910602e-09,
                                         8.33485279344597e-06,
                                         0.000872839872049042,
                                         0.018448711862583,
                                         0.107502621621057,
                                         0.193790647544852,
                                         0.103809335530345,
                                         0.0128446811934199,
                                         0.000173929225246791,
                                         1.47735042410822e-08,
                                         3.55971836299969e-25,
                                         3.08155473990322e-15,
                                         4.49204430660918e-10,
                                         9.28694331456662e-07,
                                         0.000172885290308679,
                                         0.00614665493448964,
                                         0.0592642373050826,
                                         0.179703185940615,
                                         0.171122840198442,
                                         0.042343266541924,
                                         0.00148029752640606,
                                         6.80405981707833e-07,
                                         1.10416250941478e-27,
                                         5.17243921678949e-17,
                                         1.94663694868983e-11,
                                         8.04828202005697e-08,
                                         2.66340395727957e-05,
                                         0.00159282164495174,
                                         0.0254110069262992,
                                         0.129608734491076,
                                         0.219399219856988,
                                         0.108567770071868,
                                         0.00979898279188147,
                                         2.43729588587457e-05,
                                         2.56868954831395e-30,
                                         6.51151684203625e-19,
                                         6.3268444465737e-13,
                                         5.23112190526451e-09,
                                         3.07735289118017e-06,
                                         0.000309568475007953,
                                         0.00817169808941061,
                                         0.0701090410631181,
                                         0.210971330381881,
                                         0.208775119298623,
                                         0.0486490361106891,
                                         0.000654801478272267,
                                         4.21815529722285e-33,
                                         5.78630424566564e-21,
                                         1.4515154944138e-14,
                                         2.40004188131316e-10,
                                         2.5098622493674e-07,
                                         4.24696450688722e-05,
                                         0.00185496225015411,
                                         0.0267698605240769,
                                         0.143200366266881,
                                         0.283392849467969,
                                         0.170490352850525,
                                         0.0124177631380204,
                                         4.36132718297227e-36,
                                         3.23746826161053e-23,
                                         2.09672441629333e-16,
                                         6.933107864981e-12,
                                         1.28886558928878e-08,
                                         3.66847608659165e-06,
                                         0.000265120308245026,
                                         0.00643581196936376,
                                         0.0611997958226089,
                                         0.242205606965375,
                                         0.376192849514548,
                                         0.148273059622895,
                                         2.13601196070239e-39,
                                         8.58022511338147e-26,
                                         1.43466313051492e-18,
                                         9.48693668476306e-14,
                                         3.13512080279615e-10,
                                         1.50100335111343e-07,
                                         1.79489821095276e-05,
                                         0.000732908054150508,
                                         0.0123892417160216,
                                         0.0980546978663394,
                                         0.393196978423626,
                                         0.838629307363553,

                                         0.859969548626127,
                                         0.449599266042588,
                                         0.137353213423595,
                                         0.0236742017989258,
                                         0.00217690901938571,
                                         9.81072928037416e-05,
                                         1.9073486328125e-06,
                                         1.31474316235132e-08,
                                         2.34370723181118e-11,
                                         6.25391077494278e-15,
                                         8.60266363724399e-20,
                                         4.82178016422073e-27,
                                         1.15813640622685e-40,
                                         0.130249945425703,
                                         0.367077009272021,
                                         0.287427923796685,
                                         0.0979540591378582,
                                         0.0157481222944745,
                                         0.001165789492016,
                                         3.62396240234375e-05,
                                         3.99419513315856e-07,
                                         1.16955880275358e-09,
                                         5.45647023459706e-13,
                                         1.48405504902963e-17,
                                         2.13197946284279e-24,
                                         2.76040267173352e-37,
                                         0.00934460471943542,
                                         0.141963822384507,
                                         0.284910201443315,
                                         0.191981083454412,
                                         0.0539642551685357,
                                         0.00656187403966038,
                                         0.000326156616210938,
                                         5.74786479684751e-06,
                                         2.76458325034681e-08,
                                         2.25507405586753e-11,
                                         1.21270728846902e-15,
                                         4.4652680650802e-22,
                                         3.11654981237198e-34,
                                         0.000422113729195859,
                                         0.0345687233517407,
                                         0.177816557850455,
                                         0.236907929042078,
                                         0.116431033595486,
                                         0.0232552386021202,
                                         0.00184822082519531,
                                         5.20797591360132e-05,
                                         4.11455090813558e-07,
                                         5.86806591886887e-10,
                                         6.23946168053323e-14,
                                         5.88839876168518e-20,
                                         2.21544444237611e-31,
                                         1.34595439495285e-05,
                                         0.00594184505401275,
                                         0.0783373223495559,
                                         0.2063635921517,
                                         0.177322417350817,
                                         0.0581762916945275,
                                         0.00739288330078125,
                                         0.000333091626096303,
                                         4.32262471646945e-06,
                                         1.07785783709342e-08,
                                         2.26605569982906e-12,
                                         5.48124499180496e-18,
                                         1.11168047762021e-28,
                                         3.21878875524384e-07,
                                         0.00076598553307837,
                                         0.0258837090893664,
                                         0.134817982890907,
                                         0.2025441932714,
                                         0.109152209956841,
                                         0.0221786499023438,
                                         0.00159779009949506,
                                         3.40591564971656e-05,
                                         1.48487278395422e-07,
                                         6.17241762708719e-11,
                                         3.82668292469574e-16,
                                         4.18369829327484e-26,
                                         5.98701210978333e-09,
                                         7.68024974820795e-05,
                                         0.00665180988079846,
                                         0.0685043423970512,
                                         0.179941559991254,
                                         0.159284883431268,
                                         0.0517501831054688,
                                         0.00596116670950819,
                                         0.00020872562702601,
                                         1.59100865114657e-06,
                                         1.30766215677141e-09,
                                         2.07788387890303e-14,
                                         1.22460567412863e-23,
                                         8.86331786552193e-11,
                                         6.1291273504171e-06,
                                         0.00136057244989885,
                                         0.027704920312697,
                                         0.127236493688423,
                                         0.18500570374459,
                                         0.0961074829101562,
                                         0.0177015516292053,
                                         0.00101809003741801,
                                         1.35682664040109e-05,
                                         2.20497837919145e-08,
                                         8.98025348865871e-13,
                                         2.85299298524744e-21,
                                         1.05981108968752e-12,
                                         3.95064398190918e-07,
                                         0.000224775754595268,
                                         0.00904985572737411,
                                         0.0726671036686172,
                                         0.173556794616407,
                                         0.144161224365234,
                                         0.0424558315534893,
                                         0.00401090658882674,
                                         9.34592151924492e-05,
                                         3.00302578972223e-07,
                                         3.1347424477348e-11,
                                         5.3684768795511e-19,
                                         1.03257001928285e-14,
                                         2.07489469452733e-08,
                                         3.02577181110578e-05,
                                         0.00240871447670927,
                                         0.0338160537690932,
                                         0.132665207302723,
                                         0.176197052001953,
                                         0.0829702280183712,
                                         0.0128753138541091,
                                         0.000524540244274856,
                                         3.33251954413236e-06,
                                         8.91608192529153e-10,
                                         8.2311475523303e-17,
                                         8.23114755233088e-17,
                                         8.91608192529165e-10,
                                         3.33251954413235e-06,
                                         0.000524540244274857,
                                         0.0128753138541091,
                                         0.0829702280183712,
                                         0.176197052001953,
                                         0.132665207302723,
                                         0.0338160537690932,
                                         0.00240871447670926,
                                         3.02577181110579e-05,
                                         2.07489469452731e-08,
                                         1.03257001928278e-14,
                                         5.36847687955152e-19,
                                         3.13474244773485e-11,
                                         3.00302578972221e-07,
                                         9.34592151924495e-05,
                                         0.00401090658882674,
                                         0.0424558315534893,
                                         0.144161224365234,
                                         0.173556794616407,
                                         0.0726671036686172,
                                         0.00904985572737408,
                                         0.00022477575459527,
                                         3.95064398190914e-07,
                                         1.05981108968746e-12,
                                         2.85299298524768e-21,
                                         8.98025348865886e-13,
                                         2.20497837919144e-08,
                                         1.3568266404011e-05,
                                         0.00101809003741801,
                                         0.0177015516292053,
                                         0.0961074829101562,
                                         0.18500570374459,
                                         0.127236493688423,
                                         0.027704920312697,
                                         0.00136057244989886,
                                         6.12912735041704e-06,
                                         8.86331786552149e-11,
                                         1.22460567412874e-23,
                                         2.07788387890307e-14,
                                         1.3076621567714e-09,
                                         1.59100865114658e-06,
                                         0.00020872562702601,
                                         0.00596116670950819,
                                         0.0517501831054688,
                                         0.159284883431268,
                                         0.179941559991254,
                                         0.0685043423970511,
                                         0.00665180988079848,
                                         7.68024974820789e-05,
                                         5.98701210978308e-09,
                                         4.18369829327525e-26,
                                         3.82668292469581e-16,
                                         6.17241762708714e-11,
                                         1.48487278395423e-07,
                                         3.40591564971656e-05,
                                         0.00159779009949506,
                                         0.0221786499023438,
                                         0.109152209956841,
                                         0.2025441932714,
                                         0.134817982890906,
                                         0.0258837090893664,
                                         0.000765985533078365,
                                         3.21878875524373e-07,
                                         1.11168047762032e-28,
                                         5.48124499180507e-18,
                                         2.26605569982904e-12,
                                         1.07785783709342e-08,
                                         4.32262471646945e-06,
                                         0.000333091626096303,
                                         0.00739288330078125,
                                         0.0581762916945275,
                                         0.177322417350817,
                                         0.2063635921517,
                                         0.0783373223495561,
                                         0.00594184505401272,
                                         1.34595439495281e-05,
                                         2.21544444237636e-31,
                                         5.8883987616853e-20,
                                         6.23946168053317e-14,
                                         5.8680659188689e-10,
                                         4.11455090813558e-07,
                                         5.20797591360132e-05,
                                         0.00184822082519531,
                                         0.0232552386021202,
                                         0.116431033595486,
                                         0.236907929042078,
                                         0.177816557850456,
                                         0.0345687233517406,
                                         0.00042211372919585,
                                         3.11654981237235e-34,
                                         4.4652680650803e-22,
                                         1.21270728846901e-15,
                                         2.25507405586754e-11,
                                         2.76458325034681e-08,
                                         5.74786479684751e-06,
                                         0.000326156616210938,
                                         0.00656187403966038,
                                         0.0539642551685357,
                                         0.191981083454412,
                                         0.284910201443315,
                                         0.141963822384507,
                                         0.00934460471943529,
                                         2.76040267173386e-37,
                                         2.13197946284284e-24,
                                         1.48405504902962e-17,
                                         5.45647023459709e-13,
                                         1.16955880275358e-09,
                                         3.99419513315856e-07,
                                         3.62396240234375e-05,
                                         0.001165789492016,
                                         0.0157481222944745,
                                         0.0979540591378582,
                                         0.287427923796685,
                                         0.367077009272021,
                                         0.130249945425702,
                                         1.15813640622701e-40,
                                         4.82178016422085e-27,
                                         8.6026636372439e-20,
                                         6.25391077494282e-15,
                                         2.34370723181118e-11,
                                         1.31474316235132e-08,
                                         1.9073486328125e-06,
                                         9.81072928037416e-05,
                                         0.00217690901938571,
                                         0.0236742017989258,
                                         0.137353213423595,
                                         0.449599266042588,
                                         0.859969548626127,

                                         0.877434746958023,
                                         0.500408503527424,
                                         0.179627020380078,
                                         0.0395411419561958,
                                         0.00512417811579297,
                                         0.000367946047417681,
                                         1.33996896114667e-05,
                                         2.17197082686577e-07,
                                         1.28488285942952e-09,
                                         2.0197230972711e-12,
                                         4.87581313792902e-16,
                                         6.2186814283254e-21,
                                         3.30945387752521e-28,
                                         7.72444388746638e-42,
                                         0.115122618963248,
                                         0.352837458193152,
                                         0.322759698249314,
                                         0.139235527227968,
                                         0.0311467657002776,
                                         0.00360853893216967,
                                         0.00020493924979372,
                                         5.12661596963914e-06,
                                         4.72958989234905e-08,
                                         1.19952773878574e-10,
                                         4.99865771351805e-14,
                                         1.24938957076004e-18,
                                         1.69439059097694e-25,
                                         2.12534086342623e-38,
                                         0.0071547654584867,
                                         0.117845661067508,
                                         0.274710869849874,
                                         0.232241500145452,
                                         0.0896789682271426,
                                         0.0167636103241006,
                                         0.00148471964223568,
                                         5.7318721241217e-05,
                                         8.24655085218298e-07,
                                         3.37456398499294e-09,
                                         2.42744083554287e-12,
                                         1.18901231272911e-16,
                                         4.10922231460878e-23,
                                         2.76999426918434e-35,
                                         0.000279972451164358,
                                         0.0247820749636952,
                                         0.147216858030934,
                                         0.243901656445166,
                                         0.162574866948115,
                                         0.0490330463224511,
                                         0.00677249859100656,
                                         0.000403503547204187,
                                         9.0532884073026e-06,
                                         5.97737078025088e-08,
                                         7.4221386881033e-11,
                                         7.12459176789233e-15,
                                         6.27467016763016e-21,
                                         2.27307782878825e-32,
                                         7.73334743600876e-06,
                                         0.00367869735786066,
                                         0.0556892757409935,
                                         0.180809812643878,
                                         0.208040810084881,
                                         0.101237758180911,
                                         0.0218064875218755,
                                         0.00200507464737923,
                                         7.01572721991748e-05,
                                         7.47368948546088e-07,
                                         1.60192360706409e-09,
                                         3.01346354641158e-13,
                                         6.76323519288809e-19,
                                         1.31668553971842e-29,
                                         1.60206822985545e-07,
                                         0.000409554514822388,
                                         0.0157996278790881,
                                         0.100528801725248,
                                         0.19966637282219,
                                         0.15676800316612,
                                         0.0526603540390371,
                                         0.00747265613168545,
                                         0.000407755940796513,
                                         7.00843689205743e-06,
                                         2.59307931720116e-08,
                                         9.55945565871828e-12,
                                         5.46738103983828e-17,
                                         5.72019836457091e-27,
                                         2.5813686103451e-09,
                                         3.54637703410268e-05,
                                         0.0034864037296888,
                                         0.043472493002316,
                                         0.149044807836783,
                                         0.188811248565344,
                                         0.0989093533296449,
                                         0.0216608243459678,
                                         0.0018432465521176,
                                         5.11167006987994e-05,
                                         3.26471542709409e-07,
                                         2.35860879971574e-10,
                                         3.43763470317866e-15,
                                         1.93283883746169e-24,
                                         3.31045408135438e-11,
                                         2.44414305180777e-06,
                                         0.000612317996188782,
                                         0.0149626016402254,
                                         0.0885517811982393,
                                         0.180995104650653,
                                         0.147862987727676,
                                         0.0499739284023955,
                                         0.00663185603908902,
                                         0.000296737879869581,
                                         3.27147336435539e-06,
                                         4.63177221269313e-09,
                                         1.7203176079849e-13,
                                         5.19814900678905e-22,
                                         3.42902821450783e-13,
                                         1.3605492982464e-07,
                                         8.68604983161078e-05,
                                         0.00415954309002768,
                                         0.0424936168811849,
                                         0.140136652810018,
                                         0.178536714709282,
                                         0.0931232118935522,
                                         0.019272262259347,
                                         0.00139132667245181,
                                         2.64781350386363e-05,
                                         7.3465665681021e-08,
                                         6.95350050985e-12,
                                         1.12913989523229e-19,
                                         2.89409954884457e-15,
                                         6.17107551821838e-09,
                                         1.00398338604151e-05,
                                         0.000942199909961158,
                                         0.01661533153222,
                                         0.0884088147170863,
                                         0.175652573587263,
                                         0.141394110410095,
                                         0.0456340716959047,
                                         0.00531550032406776,
                                         0.000174618524199585,
                                         9.49468497909619e-07,
                                         2.29011485491883e-10,
                                         1.99850710601455e-17,
                                         1.99850710601455e-17,
                                         2.29011485491887e-10,
                                         9.49468497909619e-07,
                                         0.000174618524199584,
                                         0.00531550032406775,
                                         0.0456340716959047,
                                         0.141394110410095,
                                         0.175652573587263,
                                         0.0884088147170863,
                                         0.01661533153222,
                                         0.000942199909961161,
                                         1.00398338604151e-05,
                                         6.1710755182183e-09,
                                         2.89409954884457e-15,
                                         1.12913989523229e-19,
                                         6.95350050985012e-12,
                                         7.3465665681021e-08,
                                         2.64781350386362e-05,
                                         0.00139132667245181,
                                         0.019272262259347,
                                         0.0931232118935521,
                                         0.178536714709283,
                                         0.140136652810018,
                                         0.042493616881185,
                                         0.0041595430900277,
                                         8.68604983161078e-05,
                                         1.36054929824639e-07,
                                         3.42902821450783e-13,
                                         5.19814900678905e-22,
                                         1.72031760798493e-13,
                                         4.63177221269313e-09,
                                         3.27147336435538e-06,
                                         0.00029673787986958,
                                         0.00663185603908902,
                                         0.0499739284023955,
                                         0.147862987727677,
                                         0.180995104650653,
                                         0.0885517811982395,
                                         0.0149626016402254,
                                         0.000612317996188782,
                                         2.44414305180775e-06,
                                         3.31045408135438e-11,
                                         1.93283883746169e-24,
                                         3.43763470317873e-15,
                                         2.35860879971574e-10,
                                         3.26471542709407e-07,
                                         5.11167006987993e-05,
                                         0.0018432465521176,
                                         0.0216608243459677,
                                         0.098909353329645,
                                         0.188811248565344,
                                         0.149044807836783,
                                         0.0434724930023161,
                                         0.0034864037296888,
                                         3.54637703410265e-05,
                                         2.5813686103451e-09,
                                         5.72019836457091e-27,
                                         5.4673810398384e-17,
                                         9.55945565871828e-12,
                                         2.59307931720115e-08,
                                         7.00843689205741e-06,
                                         0.000407755940796513,
                                         0.00747265613168543,
                                         0.0526603540390372,
                                         0.15676800316612,
                                         0.19966637282219,
                                         0.100528801725248,
                                         0.0157996278790881,
                                         0.000409554514822385,
                                         1.60206822985545e-07,
                                         1.31668553971842e-29,
                                         6.76323519288825e-19,
                                         3.01346354641158e-13,
                                         1.60192360706408e-09,
                                         7.47368948546085e-07,
                                         7.01572721991748e-05,
                                         0.00200507464737922,
                                         0.0218064875218756,
                                         0.101237758180911,
                                         0.208040810084882,
                                         0.180809812643878,
                                         0.0556892757409935,
                                         0.00367869735786064,
                                         7.73334743600876e-06,
                                         2.27307782878825e-32,
                                         6.27467016763031e-21,
                                         7.12459176789233e-15,
                                         7.42213868810325e-11,
                                         5.97737078025086e-08,
                                         9.0532884073026e-06,
                                         0.000403503547204187,
                                         0.00677249859100656,
                                         0.0490330463224511,
                                         0.162574866948115,
                                         0.243901656445166,
                                         0.147216858030934,
                                         0.0247820749636951,
                                         0.000279972451164358,
                                         2.76999426918434e-35,
                                         4.10922231460889e-23,
                                         1.18901231272911e-16,
                                         2.42744083554286e-12,
                                         3.37456398499293e-09,
                                         8.24655085218298e-07,
                                         5.73187212412169e-05,
                                         0.00148471964223568,
                                         0.0167636103241006,
                                         0.0896789682271426,
                                         0.232241500145452,
                                         0.274710869849874,
                                         0.117845661067507,
                                         0.0071547654584867,
                                         2.12534086342622e-38,
                                         1.69439059097699e-25,
                                         1.24938957076004e-18,
                                         4.99865771351801e-14,
                                         1.19952773878574e-10,
                                         4.72958989234905e-08,
                                         5.12661596963913e-06,
                                         0.00020493924979372,
                                         0.00360853893216967,
                                         0.0311467657002777,
                                         0.139235527227968,
                                         0.322759698249314,
                                         0.352837458193152,
                                         0.115122618963248,
                                         7.72444388746638e-42,
                                         3.30945387752531e-28,
                                         6.2186814283254e-21,
                                         4.87581313792898e-16,
                                         2.01972309727109e-12,
                                         1.28488285942952e-09,
                                         2.17197082686576e-07,
                                         1.33996896114667e-05,
                                         0.000367946047417681,
                                         0.00512417811579297,
                                         0.0395411419561958,
                                         0.179627020380078,
                                         0.500408503527424,
                                         0.877434746958023,

                                         0.891887850743323,
                                         0.545830050560847,
                                         0.223197757729378,
                                         0.0597911903269992,
                                         0.0101772732469796,
                                         0.00105273391359752,
                                         6.2098435427657e-05,
                                         1.9073486328125e-06,
                                         2.6718628338062e-08,
                                         1.39749051080874e-10,
                                         1.98451260680874e-13,
                                         4.41778149203794e-17,
                                         5.29951189186256e-22,
                                         2.70404735027723e-29,
                                         6.16619151847924e-43,
                                         0.102353110874902,
                                         0.335793225211795,
                                         0.348293886611299,
                                         0.181551395541424,
                                         0.0528082779663403,
                                         0.00869213451662891,
                                         0.000784625355052327,
                                         3.62396240234375e-05,
                                         7.63378072252886e-07,
                                         6.11009896724311e-09,
                                         1.38067192304083e-11,
                                         5.25229416025772e-15,
                                         1.22599034109511e-19,
                                         1.58674451712754e-26,
                                         1.93969484911399e-39,
                                         0.00556391842984048,
                                         0.0978532661393434,
                                         0.257448748156661,
                                         0.2611264665941,
                                         0.12979605268187,
                                         0.0339956361198748,
                                         0.00469605231721067,
                                         0.000326156616210938,
                                         1.03312795071765e-05,
                                         1.26542534977705e-07,
                                         4.55004857239247e-10,
                                         2.95789504254882e-13,
                                         1.34346740686558e-17,
                                         4.41050814972358e-24,
                                         2.89027205576826e-36,
                                         0.000190434495183029,
                                         0.0179541099147661,
                                         0.119817671233853,
                                         0.236476152089405,
                                         0.200865856775306,
                                         0.0837153433061401,
                                         0.0176965526383892,
                                         0.00184822082519531,
                                         8.80346615623267e-05,
                                         1.65009889338054e-06,
                                         9.4411898244464e-09,
                                         1.04882162930659e-11,
                                         9.26941610288667e-16,
                                         7.71889989546085e-22,
                                         2.71162227404349e-33,
                                         4.60090015120407e-06,
                                         0.00232533087345512,
                                         0.0393625557827674,
                                         0.151166700009777,
                                         0.219423471992735,
                                         0.145518857984197,
                                         0.047073527364382,
                                         0.00739288330078125,
                                         0.000529523954397955,
                                         1.5188529900468e-05,
                                         1.38283305124514e-07,
                                         2.62514221039975e-10,
                                         4.51450300036335e-14,
                                         9.53574245979098e-20,
                                         1.79577526801797e-30,
                                         8.33683605260349e-08,
                                         0.000225874341447993,
                                         0.00969855353010135,
                                         0.0724746586176048,
                                         0.179771692540079,
                                         0.189711979842098,
                                         0.0939130783183537,
                                         0.0221786499023438,
                                         0.00238879448138788,
                                         0.000104853461266553,
                                         1.51905687988437e-06,
                                         4.92793872112832e-09,
                                         1.6490308381406e-12,
                                         8.83516940473597e-18,
                                         8.91940825634318e-28,
                                         1.17493879600109e-09,
                                         1.70649323715047e-05,
                                         0.0018586010399986,
                                         0.0270253766131404,
                                         0.114555267146505,
                                         0.192364870978226,
                                         0.145723951942274,
                                         0.0517501831054688,
                                         0.00838161085556329,
                                         0.000562996030420912,
                                         1.29787794185148e-05,
                                         7.19504132855193e-08,
                                         4.68493035653683e-11,
                                         6.36694032626026e-16,
                                         3.44568545853928e-25,
                                         1.31794641937034e-11,
                                         1.02614981374429e-06,
                                         0.000283487498337866,
                                         0.00802095090764898,
                                         0.0581001621690619,
                                         0.155247745370456,
                                         0.179971746746465,
                                         0.0961074829101562,
                                         0.023406944570823,
                                         0.00240600442171218,
                                         8.82596485718644e-05,
                                         8.36122314393771e-07,
                                         1.0593659950105e-09,
                                         3.65186912455305e-14,
                                         1.05945799930988e-22,
                                         1.19406016902267e-13,
                                         4.98382651803177e-08,
                                         3.49242906496938e-05,
                                         0.00192276367342421,
                                         0.0238004721990981,
                                         0.101197733229358,
                                         0.179524476851508,
                                         0.144161224365234,
                                         0.0527968371898359,
                                         0.00830488181187896,
                                         0.000484770834261394,
                                         7.84787870229907e-06,
                                         1.93479458814095e-08,
                                         1.69178658585848e-12,
                                         2.63110273166954e-20,
                                         8.81482277849713e-16,
                                         1.97230456144313e-09,
                                         3.50574394779101e-06,
                                         0.000375564795066472,
                                         0.00794424664206236,
                                         0.0537495935213468,
                                         0.145915666943778,
                                         0.176197052001953,
                                         0.0970353559973812,
                                         0.0233576640716216,
                                         0.00216955012732439,
                                         6.00196787376072e-05,
                                         2.8792714856662e-07,
                                         6.38608751749951e-11,
                                         5.32415613771145e-18,
                                         5.32415613771095e-18,
                                         6.38608751749951e-11,
                                         2.87927148566617e-07,
                                         6.00196787376074e-05,
                                         0.00216955012732439,
                                         0.0233576640716216,
                                         0.0970353559973812,
                                         0.176197052001953,
                                         0.145915666943778,
                                         0.0537495935213468,
                                         0.00794424664206236,
                                         0.000375564795066471,
                                         3.50574394779103e-06,
                                         1.97230456144313e-09,
                                         8.81482277849786e-16,
                                         2.63110273166927e-20,
                                         1.69178658585848e-12,
                                         1.93479458814094e-08,
                                         7.8478787022991e-06,
                                         0.000484770834261394,
                                         0.00830488181187896,
                                         0.0527968371898359,
                                         0.144161224365234,
                                         0.179524476851509,
                                         0.101197733229358,
                                         0.0238004721990981,
                                         0.0019227636734242,
                                         3.4924290649694e-05,
                                         4.98382651803177e-08,
                                         1.19406016902276e-13,
                                         1.05945799930976e-22,
                                         3.65186912455305e-14,
                                         1.05936599501049e-09,
                                         8.36122314393775e-07,
                                         8.82596485718644e-05,
                                         0.00240600442171218,
                                         0.023406944570823,
                                         0.0961074829101562,
                                         0.179971746746465,
                                         0.155247745370456,
                                         0.0581001621690619,
                                         0.00802095090764896,
                                         0.000283487498337867,
                                         1.02614981374429e-06,
                                         1.31794641937043e-11,
                                         3.44568545853887e-25,
                                         6.36694032626026e-16,
                                         4.68493035653678e-11,
                                         7.19504132855196e-08,
                                         1.29787794185148e-05,
                                         0.000562996030420912,
                                         0.00838161085556329,
                                         0.0517501831054688,
                                         0.145723951942274,
                                         0.192364870978226,
                                         0.114555267146505,
                                         0.0270253766131403,
                                         0.00185860103999861,
                                         1.70649323715047e-05,
                                         1.17493879600115e-09,
                                         8.91940825634203e-28,
                                         8.83516940473597e-18,
                                         1.64903083814059e-12,
                                         4.92793872112835e-09,
                                         1.51905687988437e-06,
                                         0.000104853461266553,
                                         0.00238879448138788,
                                         0.0221786499023438,
                                         0.0939130783183537,
                                         0.189711979842098,
                                         0.179771692540079,
                                         0.0724746586176047,
                                         0.00969855353010138,
                                         0.000225874341447993,
                                         8.33683605260387e-08,
                                         1.79577526801772e-30,
                                         9.53574245979098e-20,
                                         4.5145030003633e-14,
                                         2.62514221039976e-10,
                                         1.38283305124514e-07,
                                         1.5188529900468e-05,
                                         0.000529523954397955,
                                         0.00739288330078125,
                                         0.047073527364382,
                                         0.145518857984197,
                                         0.219423471992735,
                                         0.151166700009777,
                                         0.0393625557827675,
                                         0.00232533087345512,
                                         4.60090015120425e-06,
                                         2.71162227404309e-33,
                                         7.71889989546085e-22,
                                         9.26941610288657e-16,
                                         1.0488216293066e-11,
                                         9.4411898244464e-09,
                                         1.65009889338054e-06,
                                         8.80346615623267e-05,
                                         0.00184822082519531,
                                         0.0176965526383892,
                                         0.0837153433061401,
                                         0.200865856775306,
                                         0.236476152089405,
                                         0.119817671233853,
                                         0.0179541099147661,
                                         0.000190434495183035,
                                         2.8902720557678e-36,
                                         4.41050814972358e-24,
                                         1.34346740686557e-17,
                                         2.95789504254884e-13,
                                         4.55004857239247e-10,
                                         1.26542534977705e-07,
                                         1.03312795071765e-05,
                                         0.000326156616210938,
                                         0.00469605231721067,
                                         0.0339956361198748,
                                         0.12979605268187,
                                         0.2611264665941,
                                         0.257448748156662,
                                         0.0978532661393434,
                                         0.00556391842984058,
                                         1.93969484911366e-39,
                                         1.58674451712754e-26,
                                         1.22599034109509e-19,
                                         5.25229416025775e-15,
                                         1.38067192304083e-11,
                                         6.11009896724311e-09,
                                         7.63378072252886e-07,
                                         3.62396240234375e-05,
                                         0.000784625355052327,
                                         0.00869213451662891,
                                         0.0528082779663403,
                                         0.181551395541424,
                                         0.3482938866113,
                                         0.335793225211795,
                                         0.102353110874903,
                                         6.16619151847815e-43,
                                         2.70404735027723e-29,
                                         5.29951189186248e-22,
                                         4.41778149203797e-17,
                                         1.98451260680874e-13,
                                         1.39749051080874e-10,
                                         2.6718628338062e-08,
                                         1.9073486328125e-06,
                                         6.2098435427657e-05,
                                         0.00105273391359752,
                                         0.0101772732469796,
                                         0.0597911903269992,
                                         0.223197757729378,
                                         0.545830050560847,
                                         0.891887850743323,

                                         0.903970210693707,
                                         0.58627239151812,
                                         0.266759698974497,
                                         0.0838680588122428,
                                         0.0177988010481122,
                                         0.00246557371320625,
                                         0.00021269164008261,
                                         1.07002177728346e-05,
                                         2.86179832062859e-07,
                                         3.55917586112148e-09,
                                         1.68329833485774e-11,
                                         2.19941303039495e-14,
                                         4.58093327781119e-18,
                                         5.2251681402677e-23,
                                         2.57534241875443e-30,
                                         5.76120917754192e-44,
                                         0.0915067135761137,
                                         0.317492786010089,
                                         0.365045041687493,
                                         0.222035099140435,
                                         0.079873568684961,
                                         0.0174138545750707,
                                         0.00226524143094498,
                                         0.000168023378408177,
                                         6.57914003452249e-06,
                                         1.20640345733573e-07,
                                         8.60382114134608e-10,
                                         1.76930072486377e-12,
                                         6.24649111241957e-16,
                                         1.37841895570512e-20,
                                         1.7167527999113e-27,
                                         2.05457503462474e-40,
                                         0.00438773818766695,
                                         0.0814436425296269,
                                         0.236625528865227,
                                         0.278442537591626,
                                         0.169786961950955,
                                         0.0582586938955078,
                                         0.0114279276119842,
                                         0.00124978617808747,
                                         7.16453724286832e-05,
                                         1.93697764047117e-06,
                                         2.08310212438638e-08,
                                         6.74194842123329e-11,
                                         4.03466234430981e-14,
                                         1.72246780017633e-18,
                                         5.42087578659451e-25,
                                         3.47071759868033e-37,
                                         0.00013246880254855,
                                         0.0131542373399649,
                                         0.0965743733047875,
                                         0.219854161478392,
                                         0.227243119944573,
                                         0.122718978552262,
                                         0.0362999162073059,
                                         0.00585311260177592,
                                         0.000491238408958288,
                                         1.95813125266746e-05,
                                         3.17552040169578e-07,
                                         1.61753748223136e-09,
                                         1.64082936930689e-12,
                                         1.35520858547016e-16,
                                         1.07774571513297e-22,
                                         3.69149008640362e-34,
                                         2.82305175411259e-06,
                                         0.00149970722117892,
                                         0.0278223957757657,
                                         0.122536683732759,
                                         0.214688842183827,
                                         0.182471500897928,
                                         0.0813909446317711,
                                         0.0193495275552762,
                                         0.00237754536485607,
                                         0.000139730532857317,
                                         3.41705191791866e-06,
                                         2.73940097007785e-08,
                                         4.71033690978393e-11,
                                         7.52650754558231e-15,
                                         1.51250033113475e-20,
                                         2.77151006192276e-31,
                                         4.51216874449211e-08,
                                         0.000128235584347959,
                                         0.00601157698632176,
                                         0.0512222696602151,
                                         0.152121103719689,
                                         0.203488382763242,
                                         0.136869858670337,
                                         0.0479750145886645,
                                         0.0086303134977815,
                                         0.000747828642206399,
                                         2.75771582277891e-05,
                                         3.47951025430557e-07,
                                         1.01414904340625e-09,
                                         3.13503303703915e-13,
                                         1.5919737974095e-18,
                                         1.56060313929134e-28,
                                         5.60928306058161e-10,
                                         8.528372443919e-06,
                                         0.00101027087465057,
                                         0.0166535583962225,
                                         0.0838349273459863,
                                         0.176497974880617,
                                         0.179017331555794,
                                         0.0925156907862634,
                                         0.0243657543801595,
                                         0.00311292319157961,
                                         0.000173102354483074,
                                         3.43744823511197e-06,
                                         1.6982715328679e-08,
                                         1.01565514485711e-11,
                                         1.30326245123743e-16,
                                         6.83477187031085e-26,
                                         5.55006292114676e-12,
                                         4.51431945220506e-07,
                                         0.000135131245408554,
                                         0.0043094693608494,
                                         0.0367729984805396,
                                         0.121845181773384,
                                         0.186359212520011,
                                         0.141998646028549,
                                         0.0547522132189203,
                                         0.0103134327768598,
                                         0.000864818489600697,
                                         2.70285495109709e-05,
                                         2.26350261900235e-07,
                                         2.61890021197411e-10,
                                         8.49173391600862e-15,
                                         2.38245165966475e-23,
                                         4.43541646133141e-14,
                                         1.93003135782664e-08,
                                         1.4598884502582e-05,
                                         0.000900713137362513,
                                         0.0130280387630773,
                                         0.0679395929351968,
                                         0.156694084304635,
                                         0.176034972826316,
                                         0.0993732384166864,
                                         0.0275984077076471,
                                         0.00348973865468258,
                                         0.000171654526317402,
                                         2.43669302193724e-06,
                                         5.45428184023195e-09,
                                         4.46896361295899e-13,
                                         6.70764575305785e-21,
                                         2.88821698613228e-16,
                                         6.72349921801425e-10,
                                         1.28511644985722e-06,
                                         0.000153393908584205,
                                         0.00376086684227915,
                                         0.0308671432430481,
                                         0.107352769185039,
                                         0.177816733688641,
                                         0.14695897817059,
                                         0.0601760569012572,
                                         0.0114741298013487,
                                         0.00088827348015354,
                                         2.13736923789839e-05,
                                         9.25582462053164e-08,
                                         1.91635593692155e-11,
                                         1.53877476660389e-18,
                                         1.53877476660405e-18,
                                         1.91635593692155e-11,
                                         9.25582462053157e-08,
                                         2.1373692378984e-05,
                                         0.00088827348015354,
                                         0.0114741298013487,
                                         0.0601760569012573,
                                         0.14695897817059,
                                         0.177816733688641,
                                         0.107352769185039,
                                         0.0308671432430481,
                                         0.00376086684227915,
                                         0.000153393908584204,
                                         1.28511644985722e-06,
                                         6.72349921801424e-10,
                                         2.888216986132e-16,
                                         6.70764575305863e-21,
                                         4.46896361295899e-13,
                                         5.4542818402319e-09,
                                         2.43669302193725e-06,
                                         0.000171654526317402,
                                         0.00348973865468258,
                                         0.0275984077076471,
                                         0.0993732384166865,
                                         0.176034972826316,
                                         0.156694084304634,
                                         0.0679395929351968,
                                         0.0130280387630773,
                                         0.00090071313736251,
                                         1.45988845025821e-05,
                                         1.93003135782664e-08,
                                         4.43541646133104e-14,
                                         2.38245165966505e-23,
                                         8.49173391600862e-15,
                                         2.61890021197408e-10,
                                         2.26350261900236e-07,
                                         2.70285495109709e-05,
                                         0.000864818489600697,
                                         0.0103134327768598,
                                         0.0547522132189203,
                                         0.141998646028548,
                                         0.186359212520011,
                                         0.121845181773384,
                                         0.0367729984805396,
                                         0.00430946936084939,
                                         0.000135131245408554,
                                         4.51431945220506e-07,
                                         5.55006292114635e-12,
                                         6.83477187031178e-26,
                                         1.30326245123743e-16,
                                         1.01565514485709e-11,
                                         1.69827153286791e-08,
                                         3.43744823511197e-06,
                                         0.000173102354483074,
                                         0.00311292319157962,
                                         0.0243657543801595,
                                         0.0925156907862632,
                                         0.179017331555794,
                                         0.176497974880617,
                                         0.0838349273459863,
                                         0.0166535583962224,
                                         0.00101027087465057,
                                         8.528372443919e-06,
                                         5.60928306058126e-10,
                                         1.56060313929157e-28,
                                         1.5919737974095e-18,
                                         3.13503303703912e-13,
                                         1.01414904340626e-09,
                                         3.47951025430557e-07,
                                         2.75771582277891e-05,
                                         0.000747828642206401,
                                         0.0086303134977815,
                                         0.0479750145886643,
                                         0.136869858670337,
                                         0.203488382763242,
                                         0.152121103719689,
                                         0.0512222696602149,
                                         0.00601157698632178,
                                         0.000128235584347959,
                                         4.51216874449188e-08,
                                         2.7715100619232e-31,
                                         1.51250033113475e-20,
                                         7.52650754558222e-15,
                                         4.71033690978396e-11,
                                         2.73940097007785e-08,
                                         3.41705191791866e-06,
                                         0.000139730532857317,
                                         0.00237754536485606,
                                         0.0193495275552762,
                                         0.081390944631771,
                                         0.182471500897928,
                                         0.214688842183827,
                                         0.122536683732759,
                                         0.0278223957757657,
                                         0.00149970722117892,
                                         2.82305175411247e-06,
                                         3.69149008640424e-34,
                                         1.07774571513297e-22,
                                         1.35520858547015e-16,
                                         1.6408293693069e-12,
                                         1.61753748223136e-09,
                                         3.17552040169578e-07,
                                         1.95813125266747e-05,
                                         0.000491238408958287,
                                         0.00585311260177591,
                                         0.0362999162073059,
                                         0.122718978552262,
                                         0.227243119944573,
                                         0.219854161478392,
                                         0.0965743733047877,
                                         0.0131542373399649,
                                         0.000132468802548545,
                                         3.47071759868095e-37,
                                         5.42087578659451e-25,
                                         1.72246780017631e-18,
                                         4.03466234430985e-14,
                                         6.74194842123329e-11,
                                         2.08310212438638e-08,
                                         1.93697764047117e-06,
                                         7.16453724286831e-05,
                                         0.00124978617808747,
                                         0.0114279276119842,
                                         0.0582586938955078,
                                         0.169786961950955,
                                         0.278442537591625,
                                         0.236625528865227,
                                         0.0814436425296269,
                                         0.00438773818766685,
                                         2.05457503462513e-40,
                                         1.7167527999113e-27,
                                         1.3784189557051e-20,
                                         6.24649111241962e-16,
                                         1.76930072486377e-12,
                                         8.60382114134608e-10,
                                         1.20640345733573e-07,
                                         6.57914003452248e-06,
                                         0.000168023378408176,
                                         0.00226524143094498,
                                         0.0174138545750707,
                                         0.0798735686849609,
                                         0.222035099140435,
                                         0.365045041687493,
                                         0.317492786010089,
                                         0.0915067135761127,
                                         5.76120917754306e-44,
                                         2.57534241875443e-30,
                                         5.22516814026761e-23,
                                         4.58093327781123e-18,
                                         2.19941303039495e-14,
                                         1.68329833485774e-11,
                                         3.55917586112149e-09,
                                         2.86179832062858e-07,
                                         1.07002177728345e-05,
                                         0.00021269164008261,
                                         0.00246557371320625,
                                         0.0177988010481122,
                                         0.0838680588122428,
                                         0.266759698974497,
                                         0.58627239151812,
                                         0.903970210693707,

                                         0.914164632168537,
                                         0.622222530259928,
                                         0.30937045236107,
                                         0.111026245233026,
                                         0.0282422524841816,
                                         0.00496228603573515,
                                         0.000581161920025487,
                                         4.32097222141576e-05,
                                         1.9073486328125e-06,
                                         4.55127165379331e-08,
                                         5.13017381991963e-10,
                                         2.23214362494863e-12,
                                         2.72180107971495e-15,
                                         5.36387436137924e-19,
                                         5.86694950885185e-24,
                                         2.80942588555939e-31,
                                         6.18528552631437e-45,
                                         0.0822354026454433,
                                         0.298935022672671,
                                         0.374398583898507,
                                         0.258710240208742,
                                         0.110814869707434,
                                         0.0303731720326241,
                                         0.00530164304881346,
                                         0.000572304615131816,
                                         3.62396240234375e-05,
                                         1.2404926241054e-06,
                                         2.03013981131333e-08,
                                         1.31650035800887e-10,
                                         2.50417615040099e-13,
                                         8.30993893188033e-17,
                                         1.75010533018751e-21,
                                         2.11102664503341e-28,
                                         2.48217585138619e-41,
                                         0.00350414557340297,
                                         0.0680294209285051,
                                         0.21462409207894,
                                         0.285555476799054,
                                         0.205961315727919,
                                         0.0880617707891216,
                                         0.022909347856844,
                                         0.00359055786699871,
                                         0.000326156616210938,
                                         1.6015648288905e-05,
                                         3.8054736938303e-07,
                                         3.67797468892153e-09,
                                         1.09134553646067e-11,
                                         6.09826055602735e-15,
                                         2.47289076574912e-19,
                                         7.5137849095275e-26,
                                         4.71839449311518e-38,
                                         9.40135849750242e-05,
                                         0.00974769443060918,
                                         0.0774654239061437,
                                         0.198450650239037,
                                         0.241022969110316,
                                         0.160756978810074,
                                         0.0623304304993946,
                                         0.0141834463378905,
                                         0.00184822082519531,
                                         0.000130190716386984,
                                         4.49134749449381e-06,
                                         6.46966283418704e-08,
                                         2.99464147578592e-10,
                                         2.81772936105137e-13,
                                         2.20004130873962e-17,
                                         1.68387155164873e-23,
                                         5.64730323142141e-35,
                                         1.78045668373829e-06,
                                         0.000985914748776506,
                                         0.0197364781081346,
                                         0.0973524494333756,
                                         0.199096456879118,
                                         0.207149830273736,
                                         0.119707083779291,
                                         0.0395488567786869,
                                         0.00739288330078125,
                                         0.000747046843816302,
                                         3.74176851047951e-05,
                                         8.03316864721056e-07,
                                         5.80042370399831e-09,
                                         9.19019874121939e-12,
                                         1.38162143546613e-15,
                                         2.66373787246603e-21,
                                         4.77111947356986e-32,
                                         2.52891058524453e-08,
                                         7.47890615653738e-05,
                                         0.00377131385014582,
                                         0.0358180965869682,
                                         0.123347370029106,
                                         0.200198394962692,
                                         0.172425239872376,
                                         0.0827079700116303,
                                         0.0221786499023438,
                                         0.00321496994376912,
                                         0.000233796732548201,
                                         7.48089199179677e-06,
                                         8.42627959422313e-08,
                                         2.24808015819153e-10,
                                         6.50741573586541e-14,
                                         3.16035067205502e-19,
                                         3.023157261738e-29,
                                         2.79377268427919e-10,
                                         4.41257738420011e-06,
                                         0.000560494332421375,
                                         0.0102497589988902,
                                         0.0594363028973343,
                                         0.150484625360155,
                                         0.19316896697148,
                                         0.134529125700764,
                                         0.0517501831054688,
                                         0.0107612204827685,
                                         0.00113620184317743,
                                         5.41845430219332e-05,
                                         9.52067067210871e-07,
                                         4.27714735130565e-09,
                                         2.38387552243976e-12,
                                         2.91631609164243e-17,
                                         1.48989861833845e-26,
                                         2.45650236340818e-12,
                                         2.07212100271536e-07,
                                         6.63007287311177e-05,
                                         0.00233449706239378,
                                         0.0227951370430654,
                                         0.0900310258085053,
                                         0.172243325874064,
                                         0.174162161035131,
                                         0.0961074829101562,
                                         0.0286691402386775,
                                         0.00439481976068646,
                                         0.000312367594019679,
                                         8.56185260743325e-06,
                                         6.47686968545809e-08,
                                         6.95067724518004e-11,
                                         2.14191602889337e-15,
                                         5.84414822589406e-24,
                                         1.74457342496469e-14,
                                         7.85929946961941e-09,
                                         6.33448518951737e-06,
                                         0.000429456278447512,
                                         0.00706120100726429,
                                         0.0435049037691456,
                                         0.124049036375362,
                                         0.18211144348194,
                                         0.144161224365234,
                                         0.0616898558263066,
                                         0.0137300650388857,
                                         0.00145446232689891,
                                         6.21890465217487e-05,
                                         7.92176678495779e-07,
                                         1.63687915224572e-09,
                                         1.27062160120354e-13,
                                         1.85153392675005e-21,
                                         1.00953233902116e-16,
                                         2.42891031225666e-10,
                                         4.93132090284257e-07,
                                         6.4372962692002e-05,
                                         0.00178227148243232,
                                         0.0171294391219472,
                                         0.0727952939156746,
                                         0.155159933686317,
                                         0.176197052001953,
                                         0.108161265057942,
                                         0.0349512761169939,
                                         0.00551820585498165,
                                         0.000368060291783389,
                                         7.89474056623289e-06,
                                         3.14097897279498e-08,
                                         6.14170488559181e-12,
                                         4.77970444627204e-19,
                                         4.77970444627147e-19,
                                         6.14170488559195e-12,
                                         3.14097897279498e-08,
                                         7.89474056623293e-06,
                                         0.00036806029178339,
                                         0.00551820585498167,
                                         0.0349512761169939,
                                         0.108161265057942,
                                         0.176197052001953,
                                         0.155159933686317,
                                         0.0727952939156746,
                                         0.0171294391219471,
                                         0.00178227148243232,
                                         6.43729626920017e-05,
                                         4.93132090284257e-07,
                                         2.42891031225662e-10,
                                         1.00953233902127e-16,
                                         1.85153392674981e-21,
                                         1.27062160120357e-13,
                                         1.63687915224572e-09,
                                         7.92176678495784e-07,
                                         6.21890465217489e-05,
                                         0.00145446232689891,
                                         0.0137300650388857,
                                         0.0616898558263066,
                                         0.144161224365234,
                                         0.18211144348194,
                                         0.124049036375361,
                                         0.0435049037691455,
                                         0.00706120100726427,
                                         0.00042945627844751,
                                         6.33448518951737e-06,
                                         7.85929946961926e-09,
                                         1.74457342496486e-14,
                                         5.84414822589323e-24,
                                         2.14191602889342e-15,
                                         6.95067724518004e-11,
                                         6.47686968545813e-08,
                                         8.56185260743329e-06,
                                         0.00031236759401968,
                                         0.00439481976068646,
                                         0.0286691402386775,
                                         0.0961074829101562,
                                         0.174162161035131,
                                         0.172243325874064,
                                         0.0900310258085052,
                                         0.0227951370430653,
                                         0.00233449706239377,
                                         6.63007287311177e-05,
                                         2.07212100271532e-07,
                                         2.45650236340838e-12,
                                         1.48989861833822e-26,
                                         2.91631609164251e-17,
                                         2.38387552243976e-12,
                                         4.27714735130567e-09,
                                         9.52067067210875e-07,
                                         5.41845430219334e-05,
                                         0.00113620184317743,
                                         0.0107612204827684,
                                         0.0517501831054688,
                                         0.134529125700764,
                                         0.19316896697148,
                                         0.150484625360155,
                                         0.0594363028973341,
                                         0.0102497589988902,
                                         0.000560494332421375,
                                         4.41257738420005e-06,
                                         2.79377268427938e-10,
                                         3.02315726173751e-29,
                                         3.16035067205512e-19,
                                         6.50741573586541e-14,
                                         2.24808015819154e-10,
                                         8.42627959422317e-08,
                                         7.48089199179679e-06,
                                         0.000233796732548201,
                                         0.00321496994376912,
                                         0.0221786499023438,
                                         0.0827079700116303,
                                         0.172425239872376,
                                         0.200198394962692,
                                         0.123347370029105,
                                         0.0358180965869682,
                                         0.00377131385014582,
                                         7.47890615653729e-05,
                                         2.52891058524468e-08,
                                         4.77111947356901e-32,
                                         2.66373787246612e-21,
                                         1.38162143546613e-15,
                                         9.19019874121945e-12,
                                         5.80042370399834e-09,
                                         8.03316864721058e-07,
                                         3.74176851047951e-05,
                                         0.000747046843816302,
                                         0.00739288330078125,
                                         0.0395488567786869,
                                         0.119707083779291,
                                         0.207149830273736,
                                         0.199096456879118,
                                         0.0973524494333755,
                                         0.0197364781081346,
                                         0.000985914748776498,
                                         1.78045668373837e-06,
                                         5.64730323142035e-35,
                                         1.68387155164879e-23,
                                         2.20004130873962e-17,
                                         2.81772936105139e-13,
                                         2.99464147578593e-10,
                                         6.46966283418707e-08,
                                         4.49134749449381e-06,
                                         0.000130190716386984,
                                         0.00184822082519531,
                                         0.0141834463378905,
                                         0.0623304304993946,
                                         0.160756978810074,
                                         0.241022969110316,
                                         0.198450650239036,
                                         0.0774654239061437,
                                         0.00974769443060912,
                                         9.40135849750275e-05,
                                         4.71839449311424e-38,
                                         7.51378490952779e-26,
                                         2.47289076574912e-19,
                                         6.0982605560274e-15,
                                         1.09134553646068e-11,
                                         3.67797468892154e-09,
                                         3.8054736938303e-07,
                                         1.6015648288905e-05,
                                         0.000326156616210938,
                                         0.00359055786699871,
                                         0.022909347856844,
                                         0.0880617707891216,
                                         0.205961315727919,
                                         0.285555476799054,
                                         0.21462409207894,
                                         0.0680294209285048,
                                         0.00350414557340305,
                                         2.48217585138566e-41,
                                         2.11102664503349e-28,
                                         1.75010533018751e-21,
                                         8.3099389318804e-17,
                                         2.50417615040101e-13,
                                         1.31650035800888e-10,
                                         2.03013981131333e-08,
                                         1.2404926241054e-06,
                                         3.62396240234375e-05,
                                         0.000572304615131816,
                                         0.00530164304881346,
                                         0.0303731720326241,
                                         0.110814869707434,
                                         0.258710240208742,
                                         0.374398583898507,
                                         0.29893502267267,
                                         0.0822354026454443,
                                         6.18528552631298e-45,
                                         2.80942588555951e-31,
                                         5.86694950885185e-24,
                                         5.3638743613793e-19,
                                         2.72180107971496e-15,
                                         2.23214362494864e-12,
                                         5.13017381991963e-10,
                                         4.55127165379331e-08,
                                         1.9073486328125e-06,
                                         4.32097222141576e-05,
                                         0.000581161920025487,
                                         0.00496228603573515,
                                         0.0282422524841816,
                                         0.111026245233026,
                                         0.30937045236107,
                                         0.622222530259928,
                                         0.914164632168537,

                                         0.922839130711893,
                                         0.654177375286382,
                                         0.350395657669492,
                                         0.14046708470858,
                                         0.0415442113504736,
                                         0.00888352188705962,
                                         0.00133608781360282,
                                         0.00013619540048319,
                                         8.95125376411808e-06,
                                         3.54372149934102e-07,
                                         7.68803678597835e-09,
                                         7.98161124776432e-11,
                                         3.2383781829317e-13,
                                         3.72626583929405e-16,
                                         7.00982730222645e-20,
                                         7.40153715995248e-25,
                                         3.45924162768636e-32,
                                         7.51418656033062e-46,
                                         0.0742611201207737,
                                         0.28074126088826,
                                         0.377787185077969,
                                         0.290450413386991,
                                         0.143858614545132,
                                         0.047638359566511,
                                         0.0105777068890109,
                                         0.00154638928614336,
                                         0.000143491331575304,
                                         7.98040604424028e-06,
                                         2.44436616494024e-07,
                                         3.63949600938522e-09,
                                         2.1800333903104e-11,
                                         3.88468387239257e-14,
                                         1.22381871598653e-17,
                                         2.47822434586109e-22,
                                         2.90989537511155e-29,
                                         3.37096063582855e-42,
                                         0.00283064820534868,
                                         0.0570697382604206,
                                         0.192941049829451,
                                         0.28448432198275,
                                         0.235966383452765,
                                         0.121008904134162,
                                         0.0396676947434538,
                                         0.00831695078593194,
                                         0.00108957309526562,
                                         8.51293471514723e-05,
                                         3.6813404508388e-06,
                                         7.86105495431076e-08,
                                         6.9516457783608e-10,
                                         1.91834378116603e-12,
                                         1.01208216357303e-15,
                                         3.93050382154617e-20,
                                         1.15947865623018e-26,
                                         7.16331909078582e-39,
                                         6.79352938507979e-05,
                                         0.00730450301353604,
                                         0.0620422028939223,
                                         0.175440490496574,
                                         0.24369664850738,
                                         0.19353655892946,
                                         0.0936628899750699,
                                         0.0281640163894818,
                                         0.00520921440500436,
                                         0.000571766589163147,
                                         3.49084736095929e-05,
                                         1.06906860099307e-06,
                                         1.39571671857071e-08,
                                         5.96461411224192e-11,
                                         5.26986644391241e-14,
                                         3.92501195446108e-18,
                                         2.90893019700263e-24,
                                         9.58429275012518e-36,
                                         1.15089934488135e-06,
                                         0.0006599450703169,
                                         0.0140825760968098,
                                         0.0763719101809715,
                                         0.177656581590101,
                                         0.218494760376205,
                                         0.156109908731443,
                                         0.0673220558495501,
                                         0.0175800626989534,
                                         0.00271075688734256,
                                         0.000233661984433635,
                                         1.02627238016388e-05,
                                         1.97805904718188e-07,
                                         1.30909326817683e-09,
                                         1.93693823099223e-12,
                                         2.76672547824439e-16,
                                         5.1515292034759e-22,
                                         9.05186709475728e-33,
                                         1.46231350484484e-08,
                                         4.47183910077624e-05,
                                         0.00239738766845753,
                                         0.024934389354111,
                                         0.0971346790433737,
                                         0.185003652188223,
                                         0.195143751254842,
                                         0.120692814400113,
                                         0.0444969116939534,
                                         0.00963881465122443,
                                         0.00117302299393135,
                                         7.38892011215803e-05,
                                         2.10253137800677e-06,
                                         2.15486511684208e-08,
                                         5.33940909690395e-11,
                                         1.46269042504439e-14,
                                         6.84227138595751e-20,
                                         6.41176402140467e-30,
                                         1.44510422042591e-10,
                                         2.35678561913242e-06,
                                         0.000317431453777675,
                                         0.00633168573355796,
                                         0.041306923302922,
                                         0.121835852848803,
                                         0.189729280225154,
                                         0.168291038333071,
                                         0.0875981331139871,
                                         0.026657058124463,
                                         0.00458015863926405,
                                         0.000413766029319889,
                                         1.73820602421166e-05,
                                         2.7588311605218e-07,
                                         1.14479086022086e-09,
                                         6.01442755439339e-13,
                                         7.06838108084984e-18,
                                         3.53242070946067e-27,
                                         1.1366490019869e-12,
                                         9.88604349719699e-08,
                                         3.34526229916468e-05,
                                         0.00127970094955062,
                                         0.013981054029721,
                                         0.0638614007807473,
                                         0.146819113036159,
                                         0.186770848741965,
                                         0.137255058231077,
                                         0.0586771937933777,
                                         0.0142338715648271,
                                         0.00184415414619064,
                                         0.00011437432462893,
                                         2.81124490561402e-06,
                                         1.95356396967263e-08,
                                         1.96836078720611e-11,
                                         5.81176630745299e-16,
                                         1.54894444423534e-24,
                                         7.22103628465097e-15,
                                         3.3494299640356e-09,
                                         2.84745153466277e-06,
                                         0.000208902472501407,
                                         0.00382210775336871,
                                         0.0270363288299264,
                                         0.0917649389029907,
                                         0.167418382416467,
                                         0.173703197486897,
                                         0.104321158025867,
                                         0.0357282308431092,
                                         0.00663873837203515,
                                         0.000607857502315872,
                                         2.31375935132737e-05,
                                         2.69261999793108e-07,
                                         5.20308664361259e-10,
                                         3.85959981363654e-14,
                                         5.48586615045143e-22,
                                         3.73793371936131e-17,
                                         9.24651763114442e-11,
                                         1.9748832315073e-07,
                                         2.77867391948261e-05,
                                         0.000851382767948738,
                                         0.00932644033194747,
                                         0.0467336692044554,
                                         0.122280190684905,
                                         0.179120867581416,
                                         0.151124326911323,
                                         0.07307332974761,
                                         0.0194729973528866,
                                         0.00263229122802912,
                                         0.000155166015930981,
                                         3.02399740220003e-06,
                                         1.12066634052172e-08,
                                         2.08850407651418e-12,
                                         1.58311873879247e-19,
                                         1.58311873879247e-19,
                                         2.08850407651418e-12,
                                         1.12066634052172e-08,
                                         3.02399740220001e-06,
                                         0.000155166015930981,
                                         0.00263229122802912,
                                         0.0194729973528866,
                                         0.07307332974761,
                                         0.151124326911323,
                                         0.179120867581416,
                                         0.122280190684905,
                                         0.0467336692044554,
                                         0.00932644033194747,
                                         0.000851382767948738,
                                         2.77867391948262e-05,
                                         1.9748832315073e-07,
                                         9.24651763114442e-11,
                                         3.73793371936131e-17,
                                         5.48586615045143e-22,
                                         3.85959981363654e-14,
                                         5.20308664361259e-10,
                                         2.69261999793106e-07,
                                         2.31375935132737e-05,
                                         0.000607857502315872,
                                         0.00663873837203515,
                                         0.0357282308431092,
                                         0.104321158025867,
                                         0.173703197486897,
                                         0.167418382416467,
                                         0.0917649389029907,
                                         0.0270363288299264,
                                         0.00382210775336871,
                                         0.000208902472501408,
                                         2.84745153466277e-06,
                                         3.3494299640356e-09,
                                         7.22103628465097e-15,
                                         1.54894444423534e-24,
                                         5.81176630745299e-16,
                                         1.96836078720611e-11,
                                         1.95356396967262e-08,
                                         2.81124490561402e-06,
                                         0.00011437432462893,
                                         0.00184415414619064,
                                         0.0142338715648271,
                                         0.0586771937933777,
                                         0.137255058231077,
                                         0.186770848741965,
                                         0.146819113036159,
                                         0.0638614007807473,
                                         0.013981054029721,
                                         0.00127970094955062,
                                         3.34526229916468e-05,
                                         9.88604349719699e-08,
                                         1.1366490019869e-12,
                                         3.53242070946067e-27,
                                         7.06838108084984e-18,
                                         6.01442755439339e-13,
                                         1.14479086022085e-09,
                                         2.7588311605218e-07,
                                         1.73820602421166e-05,
                                         0.000413766029319889,
                                         0.00458015863926405,
                                         0.026657058124463,
                                         0.0875981331139871,
                                         0.168291038333071,
                                         0.189729280225154,
                                         0.121835852848803,
                                         0.041306923302922,
                                         0.00633168573355798,
                                         0.000317431453777675,
                                         2.35678561913242e-06,
                                         1.44510422042591e-10,
                                         6.41176402140467e-30,
                                         6.84227138595751e-20,
                                         1.46269042504439e-14,
                                         5.3394090969039e-11,
                                         2.15486511684208e-08,
                                         2.10253137800677e-06,
                                         7.38892011215803e-05,
                                         0.00117302299393135,
                                         0.00963881465122443,
                                         0.0444969116939534,
                                         0.120692814400113,
                                         0.195143751254842,
                                         0.185003652188223,
                                         0.0971346790433737,
                                         0.0249343893541111,
                                         0.00239738766845753,
                                         4.47183910077624e-05,
                                         1.46231350484484e-08,
                                         9.05186709475728e-33,
                                         5.1515292034759e-22,
                                         2.76672547824439e-16,
                                         1.93693823099221e-12,
                                         1.30909326817683e-09,
                                         1.97805904718188e-07,
                                         1.02627238016388e-05,
                                         0.000233661984433635,
                                         0.00271075688734256,
                                         0.0175800626989534,
                                         0.0673220558495501,
                                         0.156109908731443,
                                         0.218494760376205,
                                         0.177656581590101,
                                         0.0763719101809717,
                                         0.0140825760968098,
                                         0.0006599450703169,
                                         1.15089934488135e-06,
                                         9.58429275012518e-36,
                                         2.90893019700263e-24,
                                         3.92501195446108e-18,
                                         5.26986644391236e-14,
                                         5.96461411224192e-11,
                                         1.39571671857071e-08,
                                         1.06906860099307e-06,
                                         3.49084736095929e-05,
                                         0.000571766589163147,
                                         0.00520921440500436,
                                         0.0281640163894818,
                                         0.0936628899750699,
                                         0.19353655892946,
                                         0.24369664850738,
                                         0.175440490496574,
                                         0.0620422028939223,
                                         0.00730450301353604,
                                         6.79352938507979e-05,
                                         7.16331909078582e-39,
                                         1.15947865623018e-26,
                                         3.93050382154617e-20,
                                         1.01208216357302e-15,
                                         1.91834378116603e-12,
                                         6.9516457783608e-10,
                                         7.86105495431076e-08,
                                         3.6813404508388e-06,
                                         8.51293471514723e-05,
                                         0.00108957309526562,
                                         0.00831695078593194,
                                         0.0396676947434538,
                                         0.121008904134162,
                                         0.235966383452765,
                                         0.284484321982751,
                                         0.192941049829451,
                                         0.0570697382604206,
                                         0.00283064820534868,
                                         3.37096063582855e-42,
                                         2.90989537511155e-29,
                                         2.47822434586109e-22,
                                         1.22381871598652e-17,
                                         3.88468387239257e-14,
                                         2.1800333903104e-11,
                                         3.63949600938522e-09,
                                         2.44436616494024e-07,
                                         7.98040604424028e-06,
                                         0.000143491331575304,
                                         0.00154638928614336,
                                         0.0105777068890109,
                                         0.047638359566511,
                                         0.143858614545132,
                                         0.290450413386991,
                                         0.377787185077969,
                                         0.28074126088826,
                                         0.0742611201207737,
                                         7.51418656033062e-46,
                                         3.45924162768636e-32,
                                         7.40153715995248e-25,
                                         7.00982730222637e-20,
                                         3.72626583929405e-16,
                                         3.2383781829317e-13,
                                         7.98161124776432e-11,
                                         7.68803678597835e-09,
                                         3.54372149934102e-07,
                                         8.95125376411808e-06,
                                         0.00013619540048319,
                                         0.00133608781360282,
                                         0.00888352188705962,
                                         0.0415442113504736,
                                         0.14046708470858,
                                         0.350395657669492,
                                         0.654177375286382,
                                         0.922839130711893,

                                         0.930277580131913,
                                         0.682608530946436,
                                         0.389442231943533,
                                         0.171429323810745,
                                         0.0575578349011641,
                                         0.0145051272974977,
                                         0.00268489189017705,
                                         0.000354673095632853,
                                         3.21871622412806e-05,
                                         1.9073486328125e-06,
                                         6.8898960819156e-08,
                                         1.37939311697292e-09,
                                         1.33588777243081e-11,
                                         5.10873246014738e-14,
                                         5.59641883929698e-17,
                                         1.0120778286453e-20,
                                         1.03709838181559e-25,
                                         4.7480810632773e-33,
                                         1.01965764392184e-46,
                                         0.0673613003162671,
                                         0.263279611872244,
                                         0.376526399402237,
                                         0.316805607726623,
                                         0.177313810845088,
                                         0.0687819848542895,
                                         0.0186497128742033,
                                         0.00349814353926648,
                                         0.000442525058418253,
                                         3.62396240234375e-05,
                                         1.80910883497252e-06,
                                         5.04877336579624e-08,
                                         6.94275483852844e-10,
                                         3.88926201390096e-12,
                                         6.5581190641484e-15,
                                         1.9770304786688e-18,
                                         3.87235150633206e-23,
                                         4.44405893228741e-30,
                                         5.08350990013591e-43,
                                         0.00231045374293406,
                                         0.0481007286149204,
                                         0.172439488333113,
                                         0.277325316377989,
                                         0.258743582486762,
                                         0.154495833715175,
                                         0.0613629747619016,
                                         0.0163431555702676,
                                         0.00288192022962943,
                                         0.000326156616210938,
                                         2.25011972075698e-05,
                                         8.75331598137517e-07,
                                         1.70915950026097e-08,
                                         1.40252351889578e-10,
                                         3.6403009820449e-13,
                                         1.82937073282695e-16,
                                         6.84886329131746e-21,
                                         1.97029113084647e-27,
                                         1.20049920433907e-39,
                                         4.98964044383821e-05,
                                         0.00553313463909291,
                                         0.0497236701679585,
                                         0.152852072302823,
                                         0.237728783140464,
                                         0.21849625502338,
                                         0.127123493058148,
                                         0.0480750075208264,
                                         0.0118171078788237,
                                         0.00184822082519531,
                                         0.000176210472040749,
                                         9.55530366460381e-06,
                                         2.64922298348154e-07,
                                         3.18447805668881e-09,
                                         1.27227339556947e-11,
                                         1.06579884704833e-14,
                                         7.62688818322822e-19,
                                         5.50004385717922e-25,
                                         1.78502883760041e-36,
                                         7.6063017292881e-07,
                                         0.00044928626898424,
                                         0.0101209657017051,
                                         0.0594682926151437,
                                         0.154179372007976,
                                         0.218124047419893,
                                         0.185899214613218,
                                         0.0998240375387994,
                                         0.034203674876802,
                                         0.00739288330078125,
                                         0.000974069910316573,
                                         7.36289666281728e-05,
                                         2.89858979986163e-06,
                                         5.10385929709961e-08,
                                         3.13874420124186e-10,
                                         4.38309708290666e-13,
                                         5.99526778148217e-17,
                                         1.08376276894431e-22,
                                         1.87353116371146e-33,
                                         8.69639205191904e-09,
                                         2.7361274123659e-05,
                                         0.00154504805843603,
                                         0.0173524527993676,
                                         0.0749949536150455,
                                         0.163314355403064,
                                         0.203887478795935,
                                         0.155457674128768,
                                         0.0742498537125835,
                                         0.0221786499023438,
                                         0.00403840438310354,
                                         0.000425514320398945,
                                         2.37857181527821e-05,
                                         6.13508224759351e-07,
                                         5.80754607960253e-09,
                                         1.35191129813497e-11,
                                         3.53452498333972e-15,
                                         1.60163505489601e-20,
                                         1.47481609853848e-30,
                                         7.73321709472005e-11,
                                         1.29600005199801e-06,
                                         0.000183449935209723,
                                         0.00393814592316642,
                                         0.0283722213803624,
                                         0.0951044208169333,
                                         0.173923828489406,
                                         0.188297576515222,
                                         0.12536432471846,
                                         0.0517501831054688,
                                         0.0130222195120798,
                                         0.00191264859786413,
                                         0.000151810324243961,
                                         5.73584776094794e-06,
                                         8.35766454953315e-08,
                                         3.24317837470778e-10,
                                         1.62072397379589e-13,
                                         1.84097730385466e-18,
                                         9.02963817823022e-28,
                                         5.47330715862927e-13,
                                         4.88587382927023e-08,
                                         1.73365094154772e-05,
                                         0.000711362985908983,
                                         0.00854325099600723,
                                         0.04408040500031,
                                         0.118085383644261,
                                         0.181528919306022,
                                         0.168469332831646,
                                         0.0961074829101562,
                                         0.0334217161260048,
                                         0.00684265618881967,
                                         0.000771178430894421,
                                         4.26818631997561e-05,
                                         9.57294876404499e-07,
                                         6.1924427547483e-09,
                                         5.91501136980461e-12,
                                         1.68423167280766e-16,
                                         4.40018923301766e-25,
                                         3.1288546369873e-15,
                                         1.48773446246767e-09,
                                         1.3232798629052e-06,
                                         0.000103785497223603,
                                         0.00207777701978871,
                                         0.0165019928194196,
                                         0.0647558710884607,
                                         0.141349039160305,
                                         0.18285788534217,
                                         0.144161224365234,
                                         0.0692816826926902,
                                         0.0197724369310593,
                                         0.00316413050298373,
                                         0.000256528174042577,
                                         8.85630326977087e-06,
                                         9.54990633888333e-08,
                                         1.74360480843645e-10,
                                         1.24451797249068e-14,
                                         1.73188203195257e-22,
                                         1.45740370441684e-17,
                                         3.6911994758647e-11,
                                         8.23001570874913e-08,
                                         1.23378928245758e-05,
                                         0.00041175001921293,
                                         0.00503368608495362,
                                         0.0289348390819744,
                                         0.0896806641372333,
                                         0.161720638307273,
                                         0.176197052001953,
                                         0.117021867580713,
                                         0.0465537419804951,
                                         0.0105782254147124,
                                         0.00125627780459417,
                                         6.67602788226658e-05,
                                         1.20003839282688e-06,
                                         4.18793033737174e-09,
                                         7.49306382710628e-13,
                                         5.55423403847981e-20,
                                         5.55423403847981e-20,
                                         7.49306382710628e-13,
                                         4.18793033737174e-09,
                                         1.20003839282688e-06,
                                         6.67602788226656e-05,
                                         0.00125627780459417,
                                         0.0105782254147124,
                                         0.046553741980495,
                                         0.117021867580713,
                                         0.176197052001953,
                                         0.161720638307273,
                                         0.0896806641372334,
                                         0.0289348390819744,
                                         0.0050336860849536,
                                         0.000411750019212932,
                                         1.23378928245758e-05,
                                         8.23001570874913e-08,
                                         3.6911994758647e-11,
                                         1.45740370441684e-17,
                                         1.73188203195257e-22,
                                         1.24451797249068e-14,
                                         1.74360480843645e-10,
                                         9.54990633888333e-08,
                                         8.85630326977083e-06,
                                         0.000256528174042578,
                                         0.00316413050298373,
                                         0.0197724369310593,
                                         0.0692816826926902,
                                         0.144161224365234,
                                         0.18285788534217,
                                         0.141349039160306,
                                         0.0647558710884607,
                                         0.0165019928194195,
                                         0.00207777701978872,
                                         0.000103785497223603,
                                         1.3232798629052e-06,
                                         1.48773446246767e-09,
                                         3.1288546369873e-15,
                                         4.40018923301766e-25,
                                         1.68423167280766e-16,
                                         5.91501136980461e-12,
                                         6.1924427547483e-09,
                                         9.57294876404495e-07,
                                         4.26818631997563e-05,
                                         0.000771178430894421,
                                         0.00684265618881966,
                                         0.0334217161260048,
                                         0.0961074829101562,
                                         0.168469332831646,
                                         0.181528919306022,
                                         0.118085383644261,
                                         0.0440804050003099,
                                         0.00854325099600725,
                                         0.000711362985908983,
                                         1.73365094154772e-05,
                                         4.88587382927023e-08,
                                         5.47330715862927e-13,
                                         9.02963817823022e-28,
                                         1.84097730385466e-18,
                                         1.62072397379589e-13,
                                         3.24317837470778e-10,
                                         8.35766454953311e-08,
                                         5.73584776094796e-06,
                                         0.000151810324243961,
                                         0.00191264859786412,
                                         0.0130222195120798,
                                         0.0517501831054688,
                                         0.12536432471846,
                                         0.188297576515222,
                                         0.173923828489406,
                                         0.0951044208169331,
                                         0.0283722213803625,
                                         0.00393814592316642,
                                         0.000183449935209723,
                                         1.29600005199801e-06,
                                         7.73321709472005e-11,
                                         1.47481609853848e-30,
                                         1.60163505489601e-20,
                                         3.53452498333972e-15,
                                         1.35191129813497e-11,
                                         5.8075460796025e-09,
                                         6.13508224759353e-07,
                                         2.37857181527821e-05,
                                         0.000425514320398944,
                                         0.00403840438310354,
                                         0.0221786499023438,
                                         0.0742498537125835,
                                         0.155457674128768,
                                         0.203887478795935,
                                         0.163314355403064,
                                         0.0749949536150457,
                                         0.0173524527993676,
                                         0.00154504805843603,
                                         2.7361274123659e-05,
                                         8.69639205191904e-09,
                                         1.87353116371146e-33,
                                         1.08376276894431e-22,
                                         5.99526778148217e-17,
                                         4.38309708290666e-13,
                                         3.13874420124185e-10,
                                         5.10385929709963e-08,
                                         2.89858979986163e-06,
                                         7.36289666281726e-05,
                                         0.000974069910316573,
                                         0.00739288330078125,
                                         0.034203674876802,
                                         0.0998240375387994,
                                         0.185899214613218,
                                         0.218124047419892,
                                         0.154179372007976,
                                         0.0594682926151437,
                                         0.0101209657017051,
                                         0.00044928626898424,
                                         7.6063017292881e-07,
                                         1.78502883760041e-36,
                                         5.50004385717922e-25,
                                         7.62688818322822e-19,
                                         1.06579884704833e-14,
                                         1.27227339556946e-11,
                                         3.18447805668882e-09,
                                         2.64922298348154e-07,
                                         9.55530366460378e-06,
                                         0.000176210472040749,
                                         0.00184822082519531,
                                         0.0118171078788237,
                                         0.0480750075208264,
                                         0.127123493058148,
                                         0.218496255023379,
                                         0.237728783140464,
                                         0.152852072302823,
                                         0.0497236701679585,
                                         0.00553313463909291,
                                         4.98964044383821e-05,
                                         1.20049920433907e-39,
                                         1.97029113084647e-27,
                                         6.84886329131746e-21,
                                         1.82937073282695e-16,
                                         3.64030098204487e-13,
                                         1.40252351889578e-10,
                                         1.70915950026097e-08,
                                         8.75331598137515e-07,
                                         2.25011972075698e-05,
                                         0.000326156616210938,
                                         0.00288192022962943,
                                         0.0163431555702676,
                                         0.0613629747619016,
                                         0.154495833715175,
                                         0.258743582486763,
                                         0.277325316377989,
                                         0.172439488333113,
                                         0.0481007286149204,
                                         0.00231045374293406,
                                         5.08350990013591e-43,
                                         4.44405893228741e-30,
                                         3.87235150633206e-23,
                                         1.9770304786688e-18,
                                         6.55811906414835e-15,
                                         3.88926201390098e-12,
                                         6.94275483852844e-10,
                                         5.04877336579622e-08,
                                         1.80910883497252e-06,
                                         3.62396240234375e-05,
                                         0.000442525058418253,
                                         0.00349814353926648,
                                         0.0186497128742033,
                                         0.0687819848542895,
                                         0.177313810845088,
                                         0.316805607726623,
                                         0.376526399402237,
                                         0.263279611872244,
                                         0.0673613003162671,
                                         1.01965764392184e-46,
                                         4.7480810632773e-33,
                                         1.03709838181559e-25,
                                         1.0120778286453e-20,
                                         5.59641883929693e-17,
                                         5.10873246014741e-14,
                                         1.33588777243081e-11,
                                         1.37939311697291e-09,
                                         6.8898960819156e-08,
                                         1.9073486328125e-06,
                                         3.21871622412806e-05,
                                         0.000354673095632853,
                                         0.00268489189017705,
                                         0.0145051272974977,
                                         0.0575578349011641,
                                         0.171429323810745,
                                         0.389442231943533,
                                         0.682608530946436,
                                         0.930277580131913,

                                         0.936701419249893,
                                         0.707946308827329,
                                         0.426296346270302,
                                         0.203239339068072,
                                         0.0760044588348702,
                                         0.0220081490242436,
                                         0.00484985292335002,
                                         0.000795083467228595,
                                         9.41369112310391e-05,
                                         7.74283525580857e-06,
                                         4.20236920100908e-07,
                                         1.40425966522425e-08,
                                         2.62552213830137e-10,
                                         2.39630650429136e-12,
                                         8.71243160753359e-15,
                                         9.15126509825496e-18,
                                         1.60001661118808e-21,
                                         1.59804727805927e-26,
                                         7.18802643261485e-34,
                                         1.528587984798e-47,
                                         0.0613570880803033,
                                         0.246751429654691,
                                         0.371747678123836,
                                         0.337802029716887,
                                         0.209764372932265,
                                         0.0930201212767906,
                                         0.0298320360767837,
                                         0.00688732679646566,
                                         0.00112493822891176,
                                         0.000126198244560123,
                                         9.30782290993784e-06,
                                         4.24214874409692e-07,
                                         1.09417029236062e-08,
                                         1.406355905824e-10,
                                         7.44137516578574e-13,
                                         1.19700421936549e-15,
                                         3.47517926689283e-19,
                                         6.61546188119761e-24,
                                         7.44487673450065e-31,
                                         8.42430498640033e-44,
                                         0.00190378197717305,
                                         0.0407387727673864,
                                         0.153558488076981,
                                         0.265953459558615,
                                         0.274228945481225,
                                         0.186234095129598,
                                         0.0869212836257369,
                                         0.0282603519470346,
                                         0.00636775456024855,
                                         0.000974306314602139,
                                         9.76542001614916e-05,
                                         6.07034356043155e-06,
                                         2.15994683815776e-07,
                                         3.90964229693021e-09,
                                         3.0106198981647e-11,
                                         7.41650228117987e-14,
                                         3.57535230286956e-17,
                                         1.29723789686146e-21,
                                         3.65253398905136e-28,
                                         2.19920963484002e-40,
                                         3.71924543804895e-05,
                                         0.00423488248605909,
                                         0.039937837670434,
                                         0.131836052468484,
                                         0.225725184335933,
                                         0.234761373694514,
                                         0.159461014986125,
                                         0.0730112189475085,
                                         0.0226949438498721,
                                         0.0047361219659682,
                                         0.000645087906061774,
                                         5.4692252178773e-05,
                                         2.68464188991198e-06,
                                         6.84327456337192e-08,
                                         7.66908960487962e-10,
                                         2.89326188120848e-12,
                                         2.31603772367552e-15,
                                         1.60163754480492e-19,
                                         1.12827814988812e-25,
                                         3.6148005104488e-37,
                                         5.12890661330939e-07,
                                         0.000310747118241543,
                                         0.00733208670603578,
                                         0.046131232910366,
                                         0.131153252053884,
                                         0.208894216296776,
                                         0.206497710867778,
                                         0.133147788330908,
                                         0.05709580944306,
                                         0.0162510925365538,
                                         0.00300800945660233,
                                         0.000347832908741409,
                                         2.35538544682854e-05,
                                         8.45518725117676e-07,
                                         1.37899922174582e-08,
                                         7.96725432259468e-11,
                                         1.0590214103343e-13,
                                         1.39585780334288e-17,
                                         2.46019999974401e-23,
                                         4.19405735267031e-34,
                                         5.30464111999724e-09,
                                         1.71014966436443e-05,
                                         0.00100955945415314,
                                         0.012106460694842,
                                         0.0571530451118501,
                                         0.139407921697845,
                                         0.200556721954938,
                                         0.182112425250953,
                                         0.107730982202119,
                                         0.0418218761883669,
                                         0.0105196370982763,
                                         0.00165911615791624,
                                         0.00015498828607212,
                                         7.83508583395596e-06,
                                         1.85971114390711e-07,
                                         1.64547344953069e-09,
                                         3.63182668426033e-12,
                                         9.12387612356383e-16,
                                         4.02333239326264e-21,
                                         3.64960328238914e-31,
                                         4.26719759924864e-11,
                                         7.32009402855681e-07,
                                         0.000108116440430962,
                                         0.00247112583712778,
                                         0.0193711416062469,
                                         0.0723609100851272,
                                         0.151500733008372,
                                         0.193731717986869,
                                         0.158100242140952,
                                         0.0837105106616759,
                                         0.0286139523285719,
                                         0.00615514726051547,
                                         0.00079321568725937,
                                         5.64702639420677e-05,
                                         1.95066412331146e-06,
                                         2.64319137425398e-08,
                                         9.68726143993128e-11,
                                         4.63845248445036e-14,
                                         5.11748931155216e-19,
                                         2.47008832290384e-28,
                                         2.73210903520325e-13,
                                         2.49383506403161e-08,
                                         9.2155254835209e-06,
                                         0.000401458866176126,
                                         0.00522564116542984,
                                         0.0298943500042711,
                                         0.0910879173930824,
                                         0.164032690536902,
                                         0.184668566413213,
                                         0.133359819057081,
                                         0.0619474548143053,
                                         0.0181747576999848,
                                         0.00323111369804721,
                                         0.000323939870103534,
                                         1.62850098832795e-05,
                                         3.37936315438523e-07,
                                         2.05657926617166e-09,
                                         1.87687509304666e-12,
                                         5.18079611760777e-17,
                                         1.33060096373398e-25,
                                         1.41286038381421e-15,
                                         6.86222163839804e-10,
                                         6.34445639412511e-07,
                                         5.26784748350502e-05,
                                         0.00113859664396064,
                                         0.00997516656632228,
                                         0.0442336480275951,
                                         0.112177571511265,
                                         0.174220544691576,
                                         0.171599473345038,
                                         0.108321591605706,
                                         0.0433455739932229,
                                         0.0106306331908177,
                                         0.00150091098629926,
                                         0.000109809396122178,
                                         3.48969175685373e-06,
                                         3.52643458587916e-08,
                                         6.13399724791082e-11,
                                         4.23625413483244e-15,
                                         5.78934100853654e-23,
                                         5.9533221602365e-18,
                                         1.53858208611938e-11,
                                         3.55899719605643e-08,
                                         5.63228014628128e-06,
                                         0.000202143197539123,
                                         0.00271212757502543,
                                         0.0175026440161681,
                                         0.0625087157022903,
                                         0.133925930756937,
                                         0.179914356716062,
                                         0.154335386665321,
                                         0.0842325219282886,
                                         0.0284986929650811,
                                         0.0056663629097476,
                                         0.000603323020484452,
                                         2.93628486956631e-05,
                                         4.92702910887458e-07,
                                         1.63346825655399e-09,
                                         2.82245110114375e-13,
                                         2.05243468785572e-20,
                                         2.05243468785539e-20,
                                         2.82245110114366e-13,
                                         1.63346825655399e-09,
                                         4.92702910887454e-07,
                                         2.93628486956631e-05,
                                         0.000603323020484454,
                                         0.0056663629097476,
                                         0.0284986929650811,
                                         0.0842325219282886,
                                         0.154335386665321,
                                         0.179914356716062,
                                         0.133925930756937,
                                         0.0625087157022903,
                                         0.0175026440161681,
                                         0.00271212757502543,
                                         0.000202143197539123,
                                         5.63228014628131e-06,
                                         3.55899719605643e-08,
                                         1.53858208611942e-11,
                                         5.95332216023736e-18,
                                         5.78934100853551e-23,
                                         4.2362541348323e-15,
                                         6.13399724791082e-11,
                                         3.52643458587913e-08,
                                         3.48969175685373e-06,
                                         0.000109809396122178,
                                         0.00150091098629926,
                                         0.0106306331908177,
                                         0.0433455739932229,
                                         0.108321591605706,
                                         0.171599473345038,
                                         0.174220544691576,
                                         0.112177571511265,
                                         0.0442336480275951,
                                         0.00997516656632226,
                                         0.00113859664396064,
                                         5.26784748350505e-05,
                                         6.34445639412512e-07,
                                         6.86222163839821e-10,
                                         1.41286038381439e-15,
                                         1.33060096373372e-25,
                                         5.18079611760758e-17,
                                         1.87687509304666e-12,
                                         2.05657926617165e-09,
                                         3.37936315438523e-07,
                                         1.62850098832796e-05,
                                         0.000323939870103534,
                                         0.00323111369804721,
                                         0.0181747576999848,
                                         0.0619474548143053,
                                         0.133359819057081,
                                         0.184668566413213,
                                         0.164032690536902,
                                         0.0910879173930824,
                                         0.029894350004271,
                                         0.00522564116542985,
                                         0.000401458866176128,
                                         9.2155254835209e-06,
                                         2.49383506403166e-08,
                                         2.73210903520355e-13,
                                         2.47008832290332e-28,
                                         5.11748931155196e-19,
                                         4.63845248445036e-14,
                                         9.68726143993119e-11,
                                         2.64319137425398e-08,
                                         1.95066412331147e-06,
                                         5.64702639420677e-05,
                                         0.00079321568725937,
                                         0.00615514726051547,
                                         0.0286139523285719,
                                         0.0837105106616759,
                                         0.158100242140952,
                                         0.193731717986869,
                                         0.151500733008372,
                                         0.072360910085127,
                                         0.0193711416062469,
                                         0.00247112583712779,
                                         0.000108116440430962,
                                         7.32009402855694e-07,
                                         4.26719759924905e-11,
                                         3.64960328238832e-31,
                                         4.02333239326247e-21,
                                         9.12387612356383e-16,
                                         3.63182668426029e-12,
                                         1.64547344953069e-09,
                                         1.85971114390711e-07,
                                         7.83508583395596e-06,
                                         0.00015498828607212,
                                         0.00165911615791624,
                                         0.0105196370982763,
                                         0.0418218761883669,
                                         0.107730982202119,
                                         0.182112425250953,
                                         0.200556721954938,
                                         0.139407921697845,
                                         0.0571530451118501,
                                         0.0121064606948421,
                                         0.00100955945415314,
                                         1.71014966436446e-05,
                                         5.30464111999766e-09,
                                         4.19405735266929e-34,
                                         2.4601999997439e-23,
                                         1.39585780334288e-17,
                                         1.05902141033429e-13,
                                         7.96725432259468e-11,
                                         1.37899922174582e-08,
                                         8.45518725117676e-07,
                                         2.35538544682854e-05,
                                         0.000347832908741409,
                                         0.00300800945660233,
                                         0.0162510925365538,
                                         0.0570958094430599,
                                         0.133147788330908,
                                         0.206497710867778,
                                         0.208894216296776,
                                         0.131153252053884,
                                         0.0461312329103661,
                                         0.00733208670603578,
                                         0.000310747118241547,
                                         5.12890661330972e-07,
                                         3.61480051044786e-37,
                                         1.12827814988807e-25,
                                         1.60163754480492e-19,
                                         2.3160377236755e-15,
                                         2.89326188120848e-12,
                                         7.66908960487966e-10,
                                         6.84327456337192e-08,
                                         2.68464188991198e-06,
                                         5.4692252178773e-05,
                                         0.000645087906061774,
                                         0.0047361219659682,
                                         0.0226949438498721,
                                         0.0730112189475085,
                                         0.159461014986125,
                                         0.234761373694514,
                                         0.225725184335933,
                                         0.131836052468484,
                                         0.039937837670434,
                                         0.00423488248605913,
                                         3.71924543804913e-05,
                                         2.19920963483942e-40,
                                         3.65253398905117e-28,
                                         1.29723789686146e-21,
                                         3.57535230286951e-17,
                                         7.41650228117988e-14,
                                         3.01061989816472e-11,
                                         3.90964229693021e-09,
                                         2.15994683815776e-07,
                                         6.07034356043155e-06,
                                         9.76542001614916e-05,
                                         0.000974306314602139,
                                         0.00636775456024855,
                                         0.0282603519470346,
                                         0.0869212836257369,
                                         0.186234095129598,
                                         0.274228945481225,
                                         0.265953459558615,
                                         0.153558488076981,
                                         0.0407387727673867,
                                         0.00190378197717311,
                                         8.42430498639788e-44,
                                         7.44487673450024e-31,
                                         6.61546188119761e-24,
                                         3.47517926689279e-19,
                                         1.19700421936549e-15,
                                         7.44137516578578e-13,
                                         1.406355905824e-10,
                                         1.09417029236062e-08,
                                         4.24214874409692e-07,
                                         9.30782290993784e-06,
                                         0.000126198244560123,
                                         0.00112493822891176,
                                         0.00688732679646566,
                                         0.0298320360767837,
                                         0.0930201212767906,
                                         0.209764372932265,
                                         0.337802029716887,
                                         0.371747678123836,
                                         0.246751429654692,
                                         0.0613570880803043,
                                         1.52858798479753e-47,
                                         7.18802643261443e-34,
                                         1.59804727805927e-26,
                                         1.60001661118806e-21,
                                         9.15126509825496e-18,
                                         8.71243160753364e-15,
                                         2.39630650429136e-12,
                                         2.62552213830137e-10,
                                         1.40425966522425e-08,
                                         4.20236920100908e-07,
                                         7.74283525580857e-06,
                                         9.41369112310391e-05,
                                         0.000795083467228595,
                                         0.00484985292335002,
                                         0.0220081490242436,
                                         0.0760044588348702,
                                         0.203239339068072,
                                         0.426296346270302,
                                         0.707946308827329,
                                         0.936701419249893,

                                         0.942285208008053,
                                         0.730574412087951,
                                         0.46087200216175,
                                         0.235332002207587,
                                         0.0965264143301523,
                                         0.0314697669481971,
                                         0.00804166358592847,
                                         0.00158161986525343,
                                         0.000233866515642737,
                                         2.52221679366758e-05,
                                         1.9073486328125e-06,
                                         9.60200729866593e-08,
                                         3.00125509657807e-09,
                                         5.29120642157369e-11,
                                         4.58912540712482e-13,
                                         1.59746111078224e-15,
                                         1.61822408564037e-18,
                                         2.74813041374198e-22,
                                         2.68464987410865e-27,
                                         1.18926459465785e-34,
                                         2.50774943511278e-48,
                                         0.0561040176271956,
                                         0.231250113544886,
                                         0.364385115183977,
                                         0.353766336224983,
                                         0.240144731684597,
                                         0.119381913732085,
                                         0.044153162836544,
                                         0.0121452329336457,
                                         0.00245621051601008,
                                         0.000357435841830807,
                                         3.62396240234375e-05,
                                         2.44598363764008e-06,
                                         1.03160294012833e-07,
                                         2.4874718181193e-09,
                                         3.01731887610909e-11,
                                         1.52017115096997e-13,
                                         2.34811289250639e-16,
                                         6.59946950457615e-20,
                                         1.22578570825325e-24,
                                         1.35633882732118e-31,
                                         1.52047664782503e-44,
                                         0.00158232051771359,
                                         0.0346727557736096,
                                         0.13646768961351,
                                         0.251907400529864,
                                         0.283001580995974,
                                         0.214522305016314,
                                         0.114832982718539,
                                         0.0441772313468898,
                                         0.0122194606209449,
                                         0.00239940021708085,
                                         0.000326156616210938,
                                         2.95144008767e-05,
                                         1.67962039854119e-06,
                                         5.53924458775362e-08,
                                         9.39726344254777e-10,
                                         6.8524138952979e-12,
                                         1.61394300422912e-14,
                                         7.50705543238836e-18,
                                         2.65112595431012e-22,
                                         7.32734781728233e-29,
                                         4.36680979827079e-41,
                                         2.80983044585648e-05,
                                         0.00327325563084038,
                                         0.0321798642121309,
                                         0.112940708058011,
                                         0.209985746825927,
                                         0.242712157200647,
                                         0.188042765345669,
                                         0.101175718603662,
                                         0.0382757441639297,
                                         0.0101412723394942,
                                         0.00184822082519531,
                                         0.000224233013345653,
                                         1.72184822311435e-05,
                                         7.76654874224358e-07,
                                         1.84275140746789e-08,
                                         1.94482193851973e-10,
                                         6.9846165966217e-13,
                                         5.37669469094422e-16,
                                         3.61020053810434e-20,
                                         2.49235888013765e-26,
                                         7.89648755031715e-38,
                                         3.52207089126527e-07,
                                         0.000218124179886567,
                                         0.00535637408287866,
                                         0.0357431159969606,
                                         0.109982375899339,
                                         0.193839789892063,
                                         0.217359655808679,
                                         0.163563511410841,
                                         0.0846306301696824,
                                         0.030256209746278,
                                         0.00739288330078125,
                                         0.00120253426316808,
                                         0.00012459795597864,
                                         7.68666452109223e-06,
                                         2.55072962250099e-07,
                                         3.89626446320117e-09,
                                         2.1336799396685e-11,
                                         2.71827539807611e-14,
                                         3.4702807921085e-18,
                                         5.98420788945314e-24,
                                         1.00794296624677e-34,
                                         3.31113841265525e-09,
                                         1.09015678617282e-05,
                                         0.00066868080439845,
                                         0.00848390073300206,
                                         0.0432033716263044,
                                         0.116106248793105,
                                         0.188435406780018,
                                         0.198316028546172,
                                         0.140343650777413,
                                         0.0677014331312518,
                                         0.0221786499023438,
                                         0.00483678328354818,
                                         0.000676220343886149,
                                         5.70570147250017e-05,
                                         2.64803281959402e-06,
                                         5.85434447745342e-08,
                                         4.88851824072825e-10,
                                         1.03070123437977e-12,
                                         2.50183791363583e-16,
                                         1.07761599913178e-21,
                                         9.64937590985175e-32,
                                         2.42109654877117e-11,
                                         4.23769449606228e-07,
                                         6.49265524851012e-05,
                                         0.00156622545031507,
                                         0.0131998110629837,
                                         0.0540908477472211,
                                         0.127057882104669,
                                         0.187018572835578,
                                         0.181014554289614,
                                         0.117824805791862,
                                         0.0517501831054688,
                                         0.0151311288297884,
                                         0.00285444104266511,
                                         0.000329409177020101,
                                         2.13814833661427e-05,
                                         6.8416940576665e-07,
                                         8.71125519496816e-09,
                                         3.03967861678733e-11,
                                         1.40284359449384e-14,
                                         1.50930468275045e-19,
                                         7.18485525001007e-29,
                                         1.40901432574846e-13,
                                         1.31110916453877e-08,
                                         5.01758059179201e-06,
                                         0.000230134379432282,
                                         0.00320986166458482,
                                         0.0200567475957867,
                                         0.0681882023574118,
                                         0.140371902900572,
                                         0.185824403700165,
                                         0.163209009869174,
                                         0.0961074829101562,
                                         0.0376751123478431,
                                         0.00959008484073301,
                                         0.00151366883676557,
                                         0.000137410786947277,
                                         6.36381510712845e-06,
                                         1.2355284779991e-07,
                                         7.13495219389275e-10,
                                         6.2607716540142e-13,
                                         1.68251263903147e-17,
                                         4.25799696500834e-26,
                                         6.62315098418028e-16,
                                         3.27637805647824e-10,
                                         3.13193195092377e-07,
                                         2.73120736104896e-05,
                                         0.000630450311834551,
                                         0.00600680006832866,
                                         0.0295571678217205,
                                         0.0850984368780959,
                                         0.15407704732039,
                                         0.18259861299109,
                                         0.144161224365234,
                                         0.0757676347324074,
                                         0.0260237433991675,
                                         0.00561787397323783,
                                         0.000713263119683099,
                                         4.78098524623026e-05,
                                         1.41537242520455e-06,
                                         1.35269664705254e-08,
                                         2.25679661545663e-11,
                                         1.51490604275066e-15,
                                         2.03816161798159e-23,
                                         2.53672167070312e-18,
                                         6.67126328649704e-12,
                                         1.59290250252534e-08,
                                         2.64111166321842e-06,
                                         0.000100896088124965,
                                         0.00146583390092095,
                                         0.010439394514355,
                                         0.0420360495301262,
                                         0.104095525796222,
                                         0.166459940571935,
                                         0.176197052001953,
                                         0.124157174457378,
                                         0.0575408142833758,
                                         0.0169891648951892,
                                         0.0030167382328164,
                                         0.000292668627159339,
                                         1.32113627456482e-05,
                                         2.08962654304533e-07,
                                         6.62850969851418e-10,
                                         1.1114040555404e-13,
                                         7.9493351358611e-21,
                                         7.94933513586252e-21,
                                         1.11140405554036e-13,
                                         6.62850969851427e-10,
                                         2.08962654304533e-07,
                                         1.32113627456482e-05,
                                         0.000292668627159339,
                                         0.0030167382328164,
                                         0.0169891648951892,
                                         0.0575408142833758,
                                         0.124157174457378,
                                         0.176197052001953,
                                         0.166459940571935,
                                         0.104095525796222,
                                         0.0420360495301262,
                                         0.010439394514355,
                                         0.00146583390092095,
                                         0.000100896088124965,
                                         2.64111166321842e-06,
                                         1.59290250252532e-08,
                                         6.67126328649724e-12,
                                         2.53672167070271e-18,
                                         2.03816161798198e-23,
                                         1.5149060427506e-15,
                                         2.25679661545666e-11,
                                         1.35269664705254e-08,
                                         1.41537242520455e-06,
                                         4.78098524623026e-05,
                                         0.000713263119683099,
                                         0.00561787397323782,
                                         0.0260237433991675,
                                         0.0757676347324074,
                                         0.144161224365234,
                                         0.18259861299109,
                                         0.15407704732039,
                                         0.085098436878096,
                                         0.0295571678217205,
                                         0.00600680006832866,
                                         0.000630450311834551,
                                         2.73120736104896e-05,
                                         3.13193195092374e-07,
                                         3.27637805647833e-10,
                                         6.62315098417934e-16,
                                         4.25799696500924e-26,
                                         1.68251263903141e-17,
                                         6.2607716540143e-13,
                                         7.13495219389275e-10,
                                         1.2355284779991e-07,
                                         6.36381510712845e-06,
                                         0.000137410786947277,
                                         0.00151366883676557,
                                         0.00959008484073301,
                                         0.0376751123478431,
                                         0.0961074829101562,
                                         0.163209009869174,
                                         0.185824403700166,
                                         0.140371902900572,
                                         0.0681882023574118,
                                         0.0200567475957867,
                                         0.00320986166458482,
                                         0.000230134379432282,
                                         5.01758059179197e-06,
                                         1.3111091645388e-08,
                                         1.40901432574828e-13,
                                         7.18485525001173e-29,
                                         1.50930468275039e-19,
                                         1.40284359449386e-14,
                                         3.03967861678733e-11,
                                         8.71125519496816e-09,
                                         6.8416940576665e-07,
                                         2.13814833661427e-05,
                                         0.000329409177020101,
                                         0.00285444104266511,
                                         0.0151311288297884,
                                         0.0517501831054688,
                                         0.117824805791862,
                                         0.181014554289614,
                                         0.187018572835579,
                                         0.127057882104669,
                                         0.0540908477472211,
                                         0.0131998110629837,
                                         0.00156622545031507,
                                         6.49265524851006e-05,
                                         4.23769449606237e-07,
                                         2.42109654877091e-11,
                                         9.64937590985415e-32,
                                         1.07761599913173e-21,
                                         2.50183791363588e-16,
                                         1.03070123437977e-12,
                                         4.88851824072825e-10,
                                         5.85434447745342e-08,
                                         2.64803281959402e-06,
                                         5.70570147250015e-05,
                                         0.000676220343886149,
                                         0.00483678328354818,
                                         0.0221786499023438,
                                         0.0677014331312518,
                                         0.140343650777413,
                                         0.198316028546172,
                                         0.188435406780018,
                                         0.116106248793105,
                                         0.0432033716263044,
                                         0.00848390073300206,
                                         0.000668680804398446,
                                         1.09015678617284e-05,
                                         3.31113841265495e-09,
                                         1.00794296624703e-34,
                                         5.98420788945283e-24,
                                         3.47028079210858e-18,
                                         2.71827539807611e-14,
                                         2.1336799396685e-11,
                                         3.89626446320117e-09,
                                         2.55072962250099e-07,
                                         7.68666452109221e-06,
                                         0.00012459795597864,
                                         0.00120253426316808,
                                         0.00739288330078125,
                                         0.030256209746278,
                                         0.0846306301696824,
                                         0.163563511410841,
                                         0.217359655808679,
                                         0.193839789892063,
                                         0.109982375899339,
                                         0.0357431159969606,
                                         0.00535637408287863,
                                         0.00021812417988657,
                                         3.52207089126502e-07,
                                         7.89648755031939e-38,
                                         2.49235888013751e-26,
                                         3.61020053810442e-20,
                                         5.37669469094422e-16,
                                         6.9846165966217e-13,
                                         1.94482193851973e-10,
                                         1.84275140746789e-08,
                                         7.76654874224356e-07,
                                         1.72184822311435e-05,
                                         0.000224233013345653,
                                         0.00184822082519531,
                                         0.0101412723394942,
                                         0.0382757441639297,
                                         0.101175718603662,
                                         0.188042765345669,
                                         0.242712157200647,
                                         0.209985746825927,
                                         0.112940708058011,
                                         0.0321798642121308,
                                         0.00327325563084041,
                                         2.80983044585633e-05,
                                         4.36680979827211e-41,
                                         7.32734781728191e-29,
                                         2.65112595431019e-22,
                                         7.50705543238836e-18,
                                         1.61394300422912e-14,
                                         6.8524138952979e-12,
                                         9.39726344254777e-10,
                                         5.5392445877536e-08,
                                         1.67962039854119e-06,
                                         2.95144008767e-05,
                                         0.000326156616210938,
                                         0.00239940021708085,
                                         0.0122194606209449,
                                         0.0441772313468898,
                                         0.114832982718539,
                                         0.214522305016314,
                                         0.283001580995974,
                                         0.251907400529864,
                                         0.136467689613509,
                                         0.0346727557736099,
                                         0.00158232051771354,
                                         1.52047664782552e-44,
                                         1.3563388273211e-31,
                                         1.22578570825328e-24,
                                         6.59946950457615e-20,
                                         2.34811289250639e-16,
                                         1.52017115096997e-13,
                                         3.0173188761091e-11,
                                         2.48747181811929e-09,
                                         1.03160294012833e-07,
                                         2.44598363764008e-06,
                                         3.62396240234375e-05,
                                         0.000357435841830807,
                                         0.00245621051601008,
                                         0.0121452329336457,
                                         0.044153162836544,
                                         0.119381913732085,
                                         0.240144731684597,
                                         0.353766336224983,
                                         0.364385115183977,
                                         0.231250113544887,
                                         0.0561040176271946,
                                         2.50774943511362e-48,
                                         1.18926459465778e-34,
                                         2.68464987410872e-27,
                                         2.74813041374198e-22,
                                         1.61822408564037e-18,
                                         1.59746111078224e-15,
                                         4.58912540712482e-13,
                                         5.29120642157367e-11,
                                         3.00125509657807e-09,
                                         9.60200729866593e-08,
                                         1.9073486328125e-06,
                                         2.52221679366758e-05,
                                         0.000233866515642737,
                                         0.00158161986525343,
                                         0.00804166358592847,
                                         0.0314697669481971,
                                         0.0965264143301523,
                                         0.235332002207587,
                                         0.46087200216175,
                                         0.730574412087951,
                                         0.942285208008053,

                                         0.947167908624191,
                                         0.750830279309409,
                                         0.49317102544255,
                                         0.267252879914212,
                                         0.118731888604745,
                                         0.0428704407814923,
                                         0.0124371477176119,
                                         0.00285655609035409,
                                         0.000509721853653832,
                                         6.89840015165591e-05,
                                         6.86611303616346e-06,
                                         4.82968855413684e-07,
                                         2.2785710448583e-08,
                                         6.72243876109301e-10,
                                         1.12640483007218e-11,
                                         9.34708293312158e-14,
                                         3.13319319920537e-16,
                                         3.07566302765141e-19,
                                         5.09277485441986e-23,
                                         4.8803307734391e-28,
                                         2.13342313229865e-35,
                                         4.46567318635295e-49,
                                         0.0514847334693525,
                                         0.216800555693016,
                                         0.355188698344675,
                                         0.365190766818765,
                                         0.267737339848457,
                                         0.146859674631839,
                                         0.0613759682970957,
                                         0.0196007534041902,
                                         0.00474923172296473,
                                         0.000859581264578227,
                                         0.000113446550738205,
                                         1.05522720830429e-05,
                                         6.60132656629388e-07,
                                         2.60461747667791e-08,
                                         5.9261371363722e-10,
                                         6.83763216060492e-12,
                                         3.30179240532335e-14,
                                         4.92384514424268e-17,
                                         1.3454398417632e-20,
                                         2.44621640641778e-25,
                                         2.66726145562255e-32,
                                         2.96580535460594e-45,
                                         0.00132561946022547,
                                         0.0296529504663674,
                                         0.121174052504848,
                                         0.23637747650768,
                                         0.285982475183774,
                                         0.238306606896041,
                                         0.143471232647598,
                                         0.063707664178346,
                                         0.0209605355476658,
                                         0.0050735787761097,
                                         0.000887892835432961,
                                         0.000109209833239358,
                                         9.05917751433982e-06,
                                         4.78024278235666e-07,
                                         1.47685480011729e-08,
                                         2.36932305136954e-10,
                                         1.64816717999325e-12,
                                         3.73386772010238e-15,
                                         1.68369336187539e-18,
                                         5.80803733700683e-23,
                                         1.57958531241219e-29,
                                         9.3301206912046e-42,
                                         2.14903965977256e-05,
                                         0.00255364539752898,
                                         0.0260282688308863,
                                         0.0963335472556977,
                                         0.19233355301727,
                                         0.243475217024637,
                                         0.211162336892791,
                                         0.130375430820761,
                                         0.0582460591307929,
                                         0.0188550215039172,
                                         0.00437537181048932,
                                         0.000711643790846193,
                                         7.82765144644264e-05,
                                         5.52384032032261e-06,
                                         2.31733626296939e-07,
                                         5.16925531616833e-09,
                                         5.18009728988003e-11,
                                         1.78278358889911e-13,
                                         1.32662101435198e-16,
                                         8.68258576240121e-21,
                                         5.88987044243001e-27,
                                         1.84806412120978e-38,
                                         2.45924792917207e-07,
                                         0.000155233551438383,
                                         0.00394651047128573,
                                         0.0277128593507414,
                                         0.0913067769809214,
                                         0.175592420233419,
                                         0.219381708460709,
                                         0.188335485377836,
                                         0.114251798011311,
                                         0.0494620350460499,
                                         0.0152195461508711,
                                         0.00327337611853818,
                                         0.000477426490714943,
                                         4.50572417102394e-05,
                                         2.5666854063967e-06,
                                         7.96093377313149e-08,
                                         1.14922984505659e-09,
                                         6.00856345172264e-12,
                                         7.37841485407365e-15,
                                         9.16222786749139e-19,
                                         1.55024636444767e-24,
                                         2.5839204209847e-35,
                                         2.1106754648413e-09,
                                         7.07736933116013e-06,
                                         0.000448789304846678,
                                         0.00597924551160358,
                                         0.0325096455812957,
                                         0.0949769090060298,
                                         0.170940760727046,
                                         0.204046814050049,
                                         0.168081843779837,
                                         0.0973146428282407,
                                         0.0397054116320785,
                                         0.0112925082933787,
                                         0.00218395059740403,
                                         0.000275644512622747,
                                         2.13214869180279e-05,
                                         9.19520259648775e-07,
                                         1.9122168410075e-08,
                                         1.51881171857507e-10,
                                         3.07780096028826e-13,
                                         7.25127471755201e-17,
                                         3.06025040869791e-22,
                                         2.70958323307978e-32,
                                         1.4089518116161e-11,
                                         2.509652880369e-07,
                                         3.96942196367446e-05,
                                         0.00100338352374762,
                                         0.00900278725239523,
                                         0.039956351840522,
                                         0.103596809607824,
                                         0.171942410630295,
                                         0.192324270603532,
                                         0.148915510877964,
                                         0.08056626569189,
                                         0.0302998485208808,
                                         0.00777024366464654,
                                         0.00131156443370802,
                                         0.000137758328865125,
                                         8.26064814896111e-06,
                                         2.47470201595638e-07,
                                         2.98602031086466e-09,
                                         9.98558521632258e-12,
                                         4.46357832188883e-15,
                                         4.69860285514517e-20,
                                         2.20994461067481e-29,
                                         7.48581939186948e-14,
                                         7.08310990628326e-09,
                                         2.79434847773289e-06,
                                         0.00013401582106721,
                                         0.00198431340735049,
                                         0.0133789537652922,
                                         0.0499707257094416,
                                         0.115320025918882,
                                         0.175152327036921,
                                         0.18137198474727,
                                         0.130114378814448,
                                         0.0647081559448105,
                                         0.0220036634015265,
                                         0.00496704892957577,
                                         0.000708413404503116,
                                         5.90657160593937e-05,
                                         2.54904329823066e-06,
                                         4.67250852683109e-08,
                                         2.5785469339168e-10,
                                         2.1868575153625e-13,
                                         5.74181292936967e-18,
                                         1.43459333360193e-26,
                                         3.21239171073967e-16,
                                         1.61465689457809e-10,
                                         1.5888386968001e-07,
                                         1.44574308709604e-05,
                                         0.000353255956064778,
                                         0.0036182987644708,
                                         0.0194684277162159,
                                         0.062470128339622,
                                         0.128837911451424,
                                         0.178421186050039,
                                         0.169724015383473,
                                         0.111615250004075,
                                         0.0503270291504762,
                                         0.015193337734377,
                                         0.00294239938775229,
                                         0.000341116533028954,
                                         2.12069132397704e-05,
                                         5.9054554076968e-07,
                                         5.3780211510985e-09,
                                         8.6537383927663e-12,
                                         5.66728804103046e-16,
                                         7.52180680845902e-24,
                                         1.12325035627495e-18,
                                         2.99913101978976e-12,
                                         7.36101951127179e-09,
                                         1.27082308086237e-06,
                                         5.12421845058383e-05,
                                         0.000797343781523815,
                                         0.00618023541074528,
                                         0.0275739498684479,
                                         0.0772200995250675,
                                         0.143014988386877,
                                         0.180393223746658,
                                         0.156872553524178,
                                         0.0937921546023233,
                                         0.0378675199018192,
                                         0.00995807492171926,
                                         0.00160519937296753,
                                         0.000143759523111613,
                                         6.08156850337713e-06,
                                         9.13963575070456e-08,
                                         2.79026822426437e-10,
                                         4.55785442921604e-14,
                                         3.21347153837926e-21,
                                         3.21347153837926e-21,
                                         4.55785442921604e-14,
                                         2.79026822426433e-10,
                                         9.13963575070464e-08,
                                         6.08156850337716e-06,
                                         0.000143759523111613,
                                         0.00160519937296753,
                                         0.00995807492171926,
                                         0.0378675199018192,
                                         0.0937921546023232,
                                         0.156872553524178,
                                         0.180393223746658,
                                         0.143014988386877,
                                         0.0772200995250675,
                                         0.0275739498684479,
                                         0.00618023541074528,
                                         0.000797343781523815,
                                         5.1242184505838e-05,
                                         1.27082308086236e-06,
                                         7.36101951127189e-09,
                                         2.99913101978976e-12,
                                         1.12325035627495e-18,
                                         7.52180680845902e-24,
                                         5.66728804103046e-16,
                                         8.65373839276616e-12,
                                         5.37802115109854e-09,
                                         5.90545540769683e-07,
                                         2.12069132397704e-05,
                                         0.000341116533028954,
                                         0.00294239938775229,
                                         0.015193337734377,
                                         0.0503270291504761,
                                         0.111615250004075,
                                         0.169724015383473,
                                         0.17842118605004,
                                         0.128837911451424,
                                         0.062470128339622,
                                         0.0194684277162159,
                                         0.0036182987644708,
                                         0.000353255956064777,
                                         1.44574308709603e-05,
                                         1.58883869680012e-07,
                                         1.61465689457809e-10,
                                         3.21239171073967e-16,
                                         1.43459333360193e-26,
                                         5.74181292936967e-18,
                                         2.18685751536246e-13,
                                         2.57854693391682e-10,
                                         4.67250852683112e-08,
                                         2.54904329823066e-06,
                                         5.90657160593937e-05,
                                         0.000708413404503116,
                                         0.00496704892957577,
                                         0.0220036634015264,
                                         0.0647081559448105,
                                         0.130114378814448,
                                         0.18137198474727,
                                         0.175152327036921,
                                         0.115320025918882,
                                         0.0499707257094416,
                                         0.0133789537652922,
                                         0.00198431340735049,
                                         0.000134015821067209,
                                         2.79434847773292e-06,
                                         7.08310990628326e-09,
                                         7.48581939186948e-14,
                                         2.20994461067481e-29,
                                         4.69860285514517e-20,
                                         4.46357832188874e-15,
                                         9.98558521632269e-12,
                                         2.98602031086468e-09,
                                         2.47470201595638e-07,
                                         8.26064814896111e-06,
                                         0.000137758328865125,
                                         0.00131156443370802,
                                         0.00777024366464652,
                                         0.0302998485208808,
                                         0.08056626569189,
                                         0.148915510877964,
                                         0.192324270603532,
                                         0.171942410630295,
                                         0.103596809607824,
                                         0.039956351840522,
                                         0.0090027872523952,
                                         0.00100338352374761,
                                         3.9694219636745e-05,
                                         2.509652880369e-07,
                                         1.4089518116161e-11,
                                         2.70958323307978e-32,
                                         3.06025040869791e-22,
                                         7.25127471755185e-17,
                                         3.0778009602883e-13,
                                         1.51881171857508e-10,
                                         1.9122168410075e-08,
                                         9.19520259648775e-07,
                                         2.13214869180279e-05,
                                         0.000275644512622747,
                                         0.00218395059740403,
                                         0.0112925082933787,
                                         0.0397054116320785,
                                         0.0973146428282408,
                                         0.168081843779837,
                                         0.204046814050049,
                                         0.170940760727046,
                                         0.0949769090060298,
                                         0.0325096455812956,
                                         0.00597924551160356,
                                         0.000448789304846682,
                                         7.07736933116013e-06,
                                         2.1106754648413e-09,
                                         2.5839204209847e-35,
                                         1.55024636444767e-24,
                                         9.16222786749118e-19,
                                         7.37841485407374e-15,
                                         6.00856345172269e-12,
                                         1.14922984505659e-09,
                                         7.96093377313149e-08,
                                         2.5666854063967e-06,
                                         4.50572417102394e-05,
                                         0.000477426490714942,
                                         0.00327337611853818,
                                         0.0152195461508711,
                                         0.0494620350460499,
                                         0.114251798011311,
                                         0.188335485377836,
                                         0.219381708460709,
                                         0.175592420233419,
                                         0.0913067769809212,
                                         0.0277128593507413,
                                         0.00394651047128575,
                                         0.000155233551438383,
                                         2.45924792917207e-07,
                                         1.84806412120978e-38,
                                         5.88987044243001e-27,
                                         8.682585762401e-21,
                                         1.32662101435199e-16,
                                         1.78278358889913e-13,
                                         5.18009728988003e-11,
                                         5.16925531616833e-09,
                                         2.31733626296939e-07,
                                         5.52384032032261e-06,
                                         7.82765144644263e-05,
                                         0.000711643790846193,
                                         0.00437537181048932,
                                         0.0188550215039172,
                                         0.0582460591307929,
                                         0.130375430820761,
                                         0.211162336892791,
                                         0.243475217024637,
                                         0.19233355301727,
                                         0.0963335472556974,
                                         0.0260282688308864,
                                         0.00255364539752898,
                                         2.14903965977256e-05,
                                         9.3301206912046e-42,
                                         1.57958531241219e-29,
                                         5.80803733700668e-23,
                                         1.68369336187541e-18,
                                         3.73386772010242e-15,
                                         1.64816717999325e-12,
                                         2.36932305136954e-10,
                                         1.47685480011729e-08,
                                         4.78024278235666e-07,
                                         9.0591775143398e-06,
                                         0.000109209833239358,
                                         0.000887892835432961,
                                         0.0050735787761097,
                                         0.0209605355476658,
                                         0.063707664178346,
                                         0.143471232647598,
                                         0.238306606896041,
                                         0.285982475183774,
                                         0.23637747650768,
                                         0.121174052504848,
                                         0.0296529504663674,
                                         0.00132561946022547,
                                         2.96580535460594e-45,
                                         2.66726145562255e-32,
                                         2.44621640641771e-25,
                                         1.34543984176322e-20,
                                         4.92384514424272e-17,
                                         3.30179240532335e-14,
                                         6.83763216060492e-12,
                                         5.9261371363722e-10,
                                         2.60461747667791e-08,
                                         6.60132656629386e-07,
                                         1.0552272083043e-05,
                                         0.000113446550738205,
                                         0.000859581264578227,
                                         0.00474923172296473,
                                         0.0196007534041902,
                                         0.0613759682970957,
                                         0.146859674631839,
                                         0.267737339848457,
                                         0.365190766818765,
                                         0.355188698344675,
                                         0.216800555693016,
                                         0.0514847334693525,
                                         4.46567318635295e-49,
                                         2.13342313229865e-35,
                                         4.88033077343895e-28,
                                         5.09277485441994e-23,
                                         3.07566302765145e-19,
                                         3.13319319920537e-16,
                                         9.34708293312158e-14,
                                         1.12640483007218e-11,
                                         6.72243876109301e-10,
                                         2.27857104485829e-08,
                                         4.82968855413684e-07,
                                         6.86611303616346e-06,
                                         6.89840015165591e-05,
                                         0.000509721853653832,
                                         0.00285655609035409,
                                         0.0124371477176119,
                                         0.0428704407814923,
                                         0.118731888604745,
                                         0.267252879914212,
                                         0.49317102544255,
                                         0.750830279309409,
                                         0.947167908624191,

                                         0.951461162033202,
                                         0.769008210790624,
                                         0.523253188423215,
                                         0.298649998792157,
                                         0.142228559296333,
                                         0.0561107462258411,
                                         0.0181646900373037,
                                         0.00476724410913498,
                                         0.000998845174178448,
                                         0.000163874049988521,
                                         2.05432238865913e-05,
                                         1.9073486328125e-06,
                                         1.25993537576329e-07,
                                         5.61807121520068e-09,
                                         1.57624506268537e-10,
                                         2.52665789621928e-12,
                                         2.01736978840438e-14,
                                         6.54317116759214e-17,
                                         6.24897418199934e-20,
                                         1.01210093380843e-23,
                                         9.53699287594516e-29,
                                         4.12095227700222e-36,
                                         8.57060158839134e-50,
                                         0.0474033341942221,
                                         0.203385347792665,
                                         0.34474843555782,
                                         0.37263853695871,
                                         0.292127972445541,
                                         0.174517390519059,
                                         0.081058844641949,
                                         0.0294323644377478,
                                         0.00832258779568278,
                                         0.00181245754891481,
                                         0.000298527461602581,
                                         3.62396240234375e-05,
                                         3.12996717516229e-06,
                                         1.8337358920177e-07,
                                         6.82921415110738e-09,
                                         1.47739247858489e-10,
                                         1.63200053470365e-12,
                                         7.59456349339256e-15,
                                         1.09832178720806e-17,
                                         2.92823399830343e-21,
                                         5.22550466088763e-26,
                                         5.62491677978839e-33,
                                         6.21012990197362e-46,
                                         0.00111870511109712,
                                         0.0254798728548743,
                                         0.107592409427264,
                                         0.220242903256745,
                                         0.284215910668589,
                                         0.25711077953155,
                                         0.171341172962046,
                                         0.0860739676302402,
                                         0.032847891938617,
                                         0.00949542500913139,
                                         0.00205489145178845,
                                         0.000326156616210938,
                                         3.68315676946716e-05,
                                         2.83514476727051e-06,
                                         1.40154366398917e-07,
                                         4.09198687896167e-09,
                                         6.25380001572733e-11,
                                         4.17547859480854e-13,
                                         9.14406649467372e-16,
                                         4.01306912350364e-19,
                                         1.35623181859104e-23,
                                         3.63683439977604e-30,
                                         2.1314687559058e-42,
                                         1.66229275552809e-05,
                                         0.00200983316238739,
                                         0.021142002488661,
                                         0.0819598656415823,
                                         0.174104014708952,
                                         0.23849924955478,
                                         0.228038522891154,
                                         0.158490651196036,
                                         0.0816284929643987,
                                         0.0313217580407532,
                                         0.00890591693361577,
                                         0.00184822082519531,
                                         0.000272888845033338,
                                         2.75993497994005e-05,
                                         1.81103853165476e-06,
                                         7.13604760301792e-08,
                                         1.50887352711519e-09,
                                         1.44542267434115e-11,
                                         4.79329667133461e-14,
                                         3.4628418634138e-17,
                                         2.21628075905144e-21,
                                         1.4805262376019e-27,
                                         4.60619600525879e-39,
                                         1.74353924288178e-07,
                                         0.000111906440967955,
                                         0.00293253315333043,
                                         0.0215294471053294,
                                         0.0752837960817356,
                                         0.156165851407793,
                                         0.214233324583405,
                                         0.206000275793861,
                                         0.143188578778753,
                                         0.0729306655873473,
                                         0.0272458724626862,
                                         0.00739288330078125,
                                         0.00142719621949674,
                                         0.000189650847042078,
                                         1.65188980629923e-05,
                                         8.78442964959897e-07,
                                         2.56976857188401e-08,
                                         3.53196010535885e-10,
                                         1.77362404683071e-12,
                                         2.10921573582397e-15,
                                         2.55651265979231e-19,
                                         4.25442640030915e-25,
                                         7.0264859538728e-36,
                                         1.37156755993758e-09,
                                         4.67316831235344e-06,
                                         0.000305071055828122,
                                         0.00424156160792326,
                                         0.0244149307611498,
                                         0.0766913518327003,
                                         0.150947908209238,
                                         0.200813644086763,
                                         0.188380628639266,
                                         0.127360714621119,
                                         0.0625149750261237,
                                         0.0221786499023438,
                                         0.00559812837539295,
                                         0.000977399215340178,
                                         0.000113004495119523,
                                         8.11018317437922e-06,
                                         3.28243739172182e-07,
                                         6.47288630894734e-09,
                                         4.922096119632e-11,
                                         9.63541910601785e-14,
                                         2.21173139492342e-17,
                                         9.16911004476632e-23,
                                         8.0388738565845e-33,
                                         8.39185967665959e-12,
                                         1.51783056162131e-07,
                                         2.46839472779408e-05,
                                         0.000649941416416202,
                                         0.00615835917727744,
                                         0.0292928915354657,
                                         0.0827223191035427,
                                         0.152255911347694,
                                         0.192761206425477,
                                         0.172988139545018,
                                         0.111563706698791,
                                         0.0517501831054688,
                                         0.0170788077860344,
                                         0.00391782213785121,
                                         0.000601264961161807,
                                         5.82375756777418e-05,
                                         3.26102718782616e-06,
                                         9.22647266142095e-08,
                                         1.0624148760983e-09,
                                         3.42354268567568e-12,
                                         1.48823785887778e-15,
                                         1.53698236904634e-20,
                                         7.1533220184347e-30,
                                         4.08665286541763e-14,
                                         3.92377133440718e-09,
                                         1.58963253231466e-06,
                                         7.92667719908089e-05,
                                         0.00123635463067018,
                                         0.00890526035194661,
                                         0.0360816866415378,
                                         0.0918805605518432,
                                         0.156989843541667,
                                         0.187010372665317,
                                         0.158463898250709,
                                         0.0961074829101562,
                                         0.0414706435738784,
                                         0.0124993080857414,
                                         0.00254627085850008,
                                         0.000332846840963153,
                                         2.57858181175819e-05,
                                         1.04674753401482e-06,
                                         1.82518429345275e-08,
                                         9.68165073930007e-11,
                                         7.9704132779722e-14,
                                         2.05059131343386e-18,
                                         5.06627622703975e-27,
                                         1.60739728739011e-16,
                                         8.19275624631968e-11,
                                         8.2684636397241e-08,
                                         7.80825664531335e-06,
                                         0.00020047815250487,
                                         0.00218663827160761,
                                         0.0127115030628228,
                                         0.0447836055414154,
                                         0.103268869865854,
                                         0.163290534320509,
                                         0.181795731725095,
                                         0.144161224365234,
                                         0.0813335907723325,
                                         0.0322086985263722,
                                         0.00870942031681241,
                                         0.00153649636738105,
                                         0.000164684739496279,
                                         9.59166805362394e-06,
                                         2.53259210836425e-07,
                                         2.21140874607597e-09,
                                         3.44774613375818e-12,
                                         2.20971003380746e-16,
                                         2.89811690930214e-24,
                                         5.15154656817095e-19,
                                         1.39384760807435e-12,
                                         3.50438502953789e-09,
                                         6.26723389502836e-07,
                                         2.64880483067281e-05,
                                         0.000437488041073873,
                                         0.00364893308349602,
                                         0.0177857995009865,
                                         0.0553510939498228,
                                         0.11617568484462,
                                         0.169940129988853,
                                         0.176197052001953,
                                         0.129974463676273,
                                         0.0676268714687086,
                                         0.0242735232168902,
                                         0.005779330791388,
                                         0.000857007684215767,
                                         7.16152069058573e-05,
                                         2.86340442904265e-06,
                                         4.1157363907172e-08,
                                         1.21520244908997e-10,
                                         1.94021732736781e-14,
                                         1.35083359613807e-21,
                                         1.35083359613836e-21,
                                         1.94021732736781e-14,
                                         1.21520244908995e-10,
                                         4.1157363907172e-08,
                                         2.86340442904265e-06,
                                         7.1615206905857e-05,
                                         0.000857007684215769,
                                         0.00577933079138798,
                                         0.0242735232168902,
                                         0.0676268714687086,
                                         0.129974463676273,
                                         0.176197052001953,
                                         0.169940129988853,
                                         0.11617568484462,
                                         0.0553510939498228,
                                         0.0177857995009866,
                                         0.00364893308349601,
                                         0.000437488041073874,
                                         2.64880483067281e-05,
                                         6.26723389502836e-07,
                                         3.50438502953794e-09,
                                         1.39384760807435e-12,
                                         5.15154656816997e-19,
                                         2.89811690930281e-24,
                                         2.20971003380746e-16,
                                         3.44774613375812e-12,
                                         2.21140874607597e-09,
                                         2.53259210836425e-07,
                                         9.59166805362389e-06,
                                         0.00016468473949628,
                                         0.00153649636738104,
                                         0.0087094203168124,
                                         0.0322086985263722,
                                         0.0813335907723325,
                                         0.144161224365234,
                                         0.181795731725095,
                                         0.163290534320509,
                                         0.103268869865854,
                                         0.0447836055414155,
                                         0.0127115030628228,
                                         0.00218663827160762,
                                         0.00020047815250487,
                                         7.80825664531335e-06,
                                         8.26846363972421e-08,
                                         8.19275624631968e-11,
                                         1.60739728738984e-16,
                                         5.06627622704104e-27,
                                         2.05059131343386e-18,
                                         7.97041327797204e-14,
                                         9.68165073930007e-11,
                                         1.82518429345275e-08,
                                         1.04674753401482e-06,
                                         2.5785818117582e-05,
                                         0.000332846840963152,
                                         0.00254627085850008,
                                         0.0124993080857414,
                                         0.0414706435738784,
                                         0.0961074829101562,
                                         0.158463898250709,
                                         0.187010372665317,
                                         0.156989843541667,
                                         0.0918805605518434,
                                         0.0360816866415377,
                                         0.00890526035194664,
                                         0.00123635463067018,
                                         7.92667719908089e-05,
                                         1.58963253231467e-06,
                                         3.92377133440718e-09,
                                         4.08665286541702e-14,
                                         7.15332201843667e-30,
                                         1.53698236904634e-20,
                                         1.48823785887774e-15,
                                         3.42354268567568e-12,
                                         1.0624148760983e-09,
                                         9.2264726614209e-08,
                                         3.26102718782618e-06,
                                         5.82375756777416e-05,
                                         0.000601264961161807,
                                         0.00391782213785121,
                                         0.0170788077860344,
                                         0.0517501831054688,
                                         0.111563706698791,
                                         0.172988139545018,
                                         0.192761206425477,
                                         0.152255911347694,
                                         0.0827223191035426,
                                         0.0292928915354658,
                                         0.00615835917727744,
                                         0.000649941416416202,
                                         2.4683947277941e-05,
                                         1.51783056162131e-07,
                                         8.39185967665851e-12,
                                         8.03887385658689e-33,
                                         9.16911004476632e-23,
                                         2.21173139492336e-17,
                                         9.63541910601785e-14,
                                         4.922096119632e-11,
                                         6.47288630894731e-09,
                                         3.28243739172183e-07,
                                         8.1101831743792e-06,
                                         0.000113004495119523,
                                         0.000977399215340177,
                                         0.00559812837539295,
                                         0.0221786499023438,
                                         0.0625149750261237,
                                         0.127360714621119,
                                         0.188380628639266,
                                         0.200813644086763,
                                         0.150947908209238,
                                         0.0766913518327004,
                                         0.0244149307611498,
                                         0.00424156160792326,
                                         0.000305071055828125,
                                         4.67316831235344e-06,
                                         1.37156755993743e-09,
                                         7.02648595387504e-36,
                                         4.25442640030915e-25,
                                         2.55651265979225e-19,
                                         2.10921573582397e-15,
                                         1.77362404683071e-12,
                                         3.53196010535883e-10,
                                         2.56976857188402e-08,
                                         8.78442964959894e-07,
                                         1.65188980629923e-05,
                                         0.000189650847042078,
                                         0.00142719621949674,
                                         0.00739288330078125,
                                         0.0272458724626862,
                                         0.0729306655873473,
                                         0.143188578778753,
                                         0.206000275793861,
                                         0.214233324583405,
                                         0.156165851407793,
                                         0.0752837960817356,
                                         0.0215294471053294,
                                         0.00293253315333045,
                                         0.000111906440967955,
                                         1.74353924288163e-07,
                                         4.60619600526035e-39,
                                         1.4805262376019e-27,
                                         2.21628075905138e-21,
                                         3.4628418634138e-17,
                                         4.79329667133461e-14,
                                         1.44542267434114e-11,
                                         1.5088735271152e-09,
                                         7.1360476030179e-08,
                                         1.81103853165476e-06,
                                         2.75993497994005e-05,
                                         0.000272888845033338,
                                         0.00184822082519531,
                                         0.00890591693361577,
                                         0.0313217580407532,
                                         0.0816284929643987,
                                         0.158490651196036,
                                         0.228038522891153,
                                         0.23849924955478,
                                         0.174104014708952,
                                         0.0819598656415823,
                                         0.0211420024886611,
                                         0.00200983316238739,
                                         1.66229275552799e-05,
                                         2.13146875590657e-42,
                                         3.63683439977605e-30,
                                         1.356231818591e-23,
                                         4.01306912350364e-19,
                                         9.14406649467372e-16,
                                         4.17547859480852e-13,
                                         6.25380001572736e-11,
                                         4.09198687896165e-09,
                                         1.40154366398917e-07,
                                         2.83514476727051e-06,
                                         3.68315676946716e-05,
                                         0.000326156616210938,
                                         0.00205489145178845,
                                         0.00949542500913139,
                                         0.032847891938617,
                                         0.0860739676302402,
                                         0.171341172962046,
                                         0.25711077953155,
                                         0.284215910668589,
                                         0.220242903256744,
                                         0.107592409427264,
                                         0.0254798728548743,
                                         0.00111870511109707,
                                         6.210129901976e-46,
                                         5.62491677978839e-33,
                                         5.22550466088747e-26,
                                         2.92823399830343e-21,
                                         1.09832178720806e-17,
                                         7.5945634933925e-15,
                                         1.63200053470366e-12,
                                         1.47739247858489e-10,
                                         6.82921415110738e-09,
                                         1.8337358920177e-07,
                                         3.12996717516229e-06,
                                         3.62396240234375e-05,
                                         0.000298527461602581,
                                         0.0018124575489148,
                                         0.00832258779568278,
                                         0.0294323644377478,
                                         0.081058844641949,
                                         0.174517390519059,
                                         0.292127972445541,
                                         0.37263853695871,
                                         0.344748435557821,
                                         0.203385347792665,
                                         0.0474033341942211,
                                         8.5706015883948e-50,
                                         4.12095227700222e-36,
                                         9.53699287594486e-29,
                                         1.01210093380843e-23,
                                         6.24897418199934e-20,
                                         6.54317116759209e-17,
                                         2.01736978840439e-14,
                                         2.52665789621926e-12,
                                         1.57624506268537e-10,
                                         5.61807121520068e-09,
                                         1.25993537576329e-07,
                                         1.9073486328125e-06,
                                         2.05432238865913e-05,
                                         0.000163874049988521,
                                         0.000998845174178448,
                                         0.00476724410913498,
                                         0.0181646900373037,
                                         0.0561107462258411,
                                         0.142228559296333,
                                         0.298649998792157,
                                         0.523253188423215,
                                         0.769008210790624,
                                         0.951461162033202,

                                         0.955255428168223,
                                         0.785363664437344,
                                         0.551214540959937,
                                         0.329260754059415,
                                         0.166646216581012,
                                         0.0710321126895169,
                                         0.0252979848136087,
                                         0.0074528430827377,
                                         0.00179339379703171,
                                         0.000346966220898501,
                                         5.29147876926242e-05,
                                         6.20529197044094e-06,
                                         5.42212638868442e-07,
                                         3.3903105868346e-08,
                                         1.43892049953694e-09,
                                         3.86324396572488e-11,
                                         5.95661506137381e-13,
                                         4.59774631190177e-15,
                                         1.44870945960007e-17,
                                         1.35056856596338e-20,
                                         2.14531200002505e-24,
                                         1.99182346961496e-29,
                                         8.5192466955709e-37,
                                         1.76179054184005e-50,
                                         0.0437809792236986,
                                         0.190962045255096,
                                         0.33352103009137,
                                         0.376682266000991,
                                         0.313144588277129,
                                         0.201555169550394,
                                         0.102632461046319,
                                         0.0416521625974604,
                                         0.0134560471304771,
                                         0.00343371669594209,
                                         0.00068274772294787,
                                         0.000103705173513616,
                                         1.17122042315649e-05,
                                         9.48557113518876e-07,
                                         5.24888112825586e-08,
                                         1.85873510158722e-09,
                                         3.84761101617868e-11,
                                         4.0912252501223e-13,
                                         1.84310165144268e-15,
                                         2.59462373656674e-18,
                                         6.76959142806304e-22,
                                         1.1883804249544e-26,
                                         1.2648299575369e-33,
                                         1.38769977960947e-46,
                                         0.00095047420756371,
                                         0.0219944050604227,
                                         0.0955904904177046,
                                         0.204126452800183,
                                         0.278729715596806,
                                         0.270908143187314,
                                         0.19722976946956,
                                         0.110266106844041,
                                         0.0478242584151656,
                                         0.016096469572764,
                                         0.00417284613487247,
                                         0.000820970463873308,
                                         0.000119838549943438,
                                         1.25711900849241e-05,
                                         9.06954674361567e-07,
                                         4.23615433812975e-08,
                                         1.17725811346821e-09,
                                         1.72445031481698e-11,
                                         1.110724084564e-13,
                                         2.36113622756396e-16,
                                         1.01186683708802e-19,
                                         3.35852853153917e-24,
                                         8.89512387770318e-31,
                                         5.1775663491375e-43,
                                         1.29921300337562e-05,
                                         0.00159500679569643,
                                         0.0172500879311533,
                                         0.0696479856418482,
                                         0.156209265328691,
                                         0.229263719990226,
                                         0.238641158320531,
                                         0.183794150238421,
                                         0.107019802800847,
                                         0.0475096644383744,
                                         0.0160579329215293,
                                         0.00409203933976804,
                                         0.000772039660721009,
                                         0.0001048997512773,
                                         9.86710129701111e-06,
                                         6.07870693672153e-07,
                                         2.26797039997642e-08,
                                         4.57649653605823e-10,
                                         4.2145212418712e-12,
                                         1.352860011479e-14,
                                         9.5229043961257e-18,
                                         5.97623617342167e-22,
                                         3.93873729408349e-28,
                                         1.2163007760557e-39,
                                         1.25358187935182e-07,
                                         8.16479304782175e-05,
                                         0.0021973551774541,
                                         0.0167745221158072,
                                         0.0617963227476242,
                                         0.136955959494178,
                                         0.203821763219362,
                                         0.216248773372189,
                                         0.169048909305632,
                                         0.0989841408029804,
                                         0.0436193542803973,
                                         0.0143974106643393,
                                         0.00351087204463125,
                                         0.000617881010770669,
                                         7.57749947088625e-05,
                                         6.15719576375076e-06,
                                         3.08414932250001e-07,
                                         8.57329790235931e-09,
                                         1.12881463087298e-10,
                                         5.47163349764397e-13,
                                         6.32627127326674e-16,
                                         7.50652242635117e-20,
                                         1.23110290021662e-25,
                                         2.01691975328749e-36,
                                         9.07165063093722e-10,
                                         3.1346502265834e-06,
                                         0.000209928050591169,
                                         0.00303007247396221,
                                         0.0183349503809512,
                                         0.0613603457683055,
                                         0.130562068968581,
                                         0.19082570877928,
                                         0.200272750856706,
                                         0.154671584923735,
                                         0.0888648655996012,
                                         0.0379918330372249,
                                         0.0119743419356155,
                                         0.00272958423749094,
                                         0.000436438953824319,
                                         4.67752353433868e-05,
                                         3.14553610694483e-06,
                                         1.2045475672845e-07,
                                         2.26755732931213e-09,
                                         1.65974895102835e-11,
                                         3.15200908447971e-14,
                                         7.07149248735069e-18,
                                         2.8859776073383e-23,
                                         2.50840419447469e-33,
                                         5.10593706346621e-12,
                                         9.36027321455715e-08,
                                         1.55989804945861e-05,
                                         0.00042570760279948,
                                         0.00423109109567002,
                                         0.0213820919164102,
                                         0.065048760189489,
                                         0.130971151595898,
                                         0.184538456489193,
                                         0.187979713968145,
                                         0.140810965117556,
                                         0.0779743268367628,
                                         0.0317646328565798,
                                         0.0093787224864365,
                                         0.00195513444922589,
                                         0.000276378675503388,
                                         2.49522368004445e-05,
                                         1.31630195269976e-06,
                                         3.54282297628589e-08,
                                         3.91582555767572e-10,
                                         1.22146953862737e-12,
                                         5.1813020746657e-16,
                                         5.26195452153318e-21,
                                         2.42639748877888e-30,
                                         2.28735264354384e-14,
                                         2.22462334232175e-09,
                                         9.22551212833164e-07,
                                         4.76034384703285e-05,
                                         0.000777129758514713,
                                         0.00593036075424464,
                                         0.0257946447076069,
                                         0.0715456008333241,
                                         0.135338211017972,
                                         0.18183603802459,
                                         0.177587089225499,
                                         0.12737422284989,
                                         0.0670663315312753,
                                         0.0256483479186568,
                                         0.00697105127112475,
                                         0.00129975533332387,
                                         0.000157540717416764,
                                         1.1448685446661e-05,
                                         4.40564151801797e-07,
                                         7.35313936603692e-09,
                                         3.7674400325254e-11,
                                         3.02158814752215e-14,
                                         7.63606735317881e-19,
                                         1.86807756325642e-27,
                                         8.27630982617167e-17,
                                         4.27041813750075e-11,
                                         4.40687467523967e-08,
                                         4.29943302047847e-06,
                                         0.000115287082858629,
                                         0.00132848905500424,
                                         0.00826163538215996,
                                         0.0315672079340783,
                                         0.0801677926463017,
                                         0.142067547232117,
                                         0.180897368571782,
                                         0.168057309768394,
                                         0.114369759978989,
                                         0.056652757550461,
                                         0.020075476979674,
                                         0.00493701710468364,
                                         0.000803382007695287,
                                         8.04269626486791e-05,
                                         4.42501560543098e-06,
                                         1.11523965876005e-07,
                                         9.3854694928539e-10,
                                         1.42323813022011e-12,
                                         8.9503157638259e-17,
                                         1.16164601681318e-24,
                                         2.44005306779676e-19,
                                         6.67948884395529e-13,
                                         1.71525949686372e-09,
                                         3.1640473073705e-07,
                                         1.39356322618651e-05,
                                         0.000242489961796342,
                                         0.0021560628387848,
                                         0.0113487576911473,
                                         0.0386935286455258,
                                         0.0904417009529273,
                                         0.150145398389552,
                                         0.18067255395682,
                                         0.158919351642816,
                                         0.101962770643171,
                                         0.0471077525352646,
                                         0.0152801132209258,
                                         0.00333818422243746,
                                         0.000460369590586936,
                                         3.62142398576412e-05,
                                         1.37823263417565e-06,
                                         1.90512981257912e-08,
                                         5.46234075900558e-11,
                                         8.54802691488168e-15,
                                         5.88588303553522e-22,
                                         5.88588303553658e-22,
                                         8.54802691488168e-15,
                                         5.46234075900568e-11,
                                         1.90512981257912e-08,
                                         1.37823263417565e-06,
                                         3.62142398576413e-05,
                                         0.000460369590586936,
                                         0.00333818422243747,
                                         0.0152801132209258,
                                         0.0471077525352646,
                                         0.101962770643171,
                                         0.158919351642816,
                                         0.18067255395682,
                                         0.150145398389552,
                                         0.0904417009529273,
                                         0.0386935286455258,
                                         0.0113487576911473,
                                         0.0021560628387848,
                                         0.000242489961796341,
                                         1.39356322618651e-05,
                                         3.1640473073705e-07,
                                         1.71525949686369e-09,
                                         6.67948884395529e-13,
                                         2.44005306779625e-19,
                                         1.16164601681347e-24,
                                         8.9503157638259e-17,
                                         1.42323813022014e-12,
                                         9.3854694928539e-10,
                                         1.11523965876005e-07,
                                         4.425015605431e-06,
                                         8.04269626486791e-05,
                                         0.00080338200769529,
                                         0.00493701710468364,
                                         0.020075476979674,
                                         0.056652757550461,
                                         0.114369759978989,
                                         0.168057309768394,
                                         0.180897368571782,
                                         0.142067547232117,
                                         0.0801677926463017,
                                         0.0315672079340783,
                                         0.00826163538215996,
                                         0.00132848905500424,
                                         0.000115287082858629,
                                         4.29943302047847e-06,
                                         4.40687467523961e-08,
                                         4.27041813750075e-11,
                                         8.27630982617014e-17,
                                         1.86807756325693e-27,
                                         7.63606735317881e-19,
                                         3.02158814752221e-14,
                                         3.7674400325254e-11,
                                         7.35313936603692e-09,
                                         4.40564151801799e-07,
                                         1.1448685446661e-05,
                                         0.000157540717416764,
                                         0.00129975533332387,
                                         0.00697105127112475,
                                         0.0256483479186568,
                                         0.0670663315312752,
                                         0.127374222849891,
                                         0.177587089225499,
                                         0.18183603802459,
                                         0.135338211017972,
                                         0.071545600833324,
                                         0.0257946447076069,
                                         0.00593036075424462,
                                         0.000777129758514713,
                                         4.76034384703285e-05,
                                         9.22551212833153e-07,
                                         2.22462334232175e-09,
                                         2.28735264354347e-14,
                                         2.42639748877961e-30,
                                         5.26195452153318e-21,
                                         5.18130207466582e-16,
                                         1.22146953862737e-12,
                                         3.91582555767572e-10,
                                         3.54282297628591e-08,
                                         1.31630195269976e-06,
                                         2.49522368004446e-05,
                                         0.000276378675503388,
                                         0.00195513444922589,
                                         0.0093787224864365,
                                         0.0317646328565798,
                                         0.0779743268367629,
                                         0.140810965117556,
                                         0.187979713968145,
                                         0.184538456489193,
                                         0.130971151595898,
                                         0.0650487601894891,
                                         0.0213820919164101,
                                         0.00423109109567002,
                                         0.00042570760279948,
                                         1.5598980494586e-05,
                                         9.36027321455715e-08,
                                         5.1059370634655e-12,
                                         2.5084041944755e-33,
                                         2.8859776073383e-23,
                                         7.07149248735087e-18,
                                         3.15200908447971e-14,
                                         1.65974895102835e-11,
                                         2.26755732931215e-09,
                                         1.2045475672845e-07,
                                         3.14553610694484e-06,
                                         4.67752353433868e-05,
                                         0.000436438953824319,
                                         0.00272958423749094,
                                         0.0119743419356154,
                                         0.0379918330372249,
                                         0.0888648655996011,
                                         0.154671584923735,
                                         0.200272750856706,
                                         0.19082570877928,
                                         0.130562068968581,
                                         0.0613603457683054,
                                         0.0183349503809512,
                                         0.00303007247396221,
                                         0.000209928050591167,
                                         3.1346502265834e-06,
                                         9.07165063093618e-10,
                                         2.01691975328818e-36,
                                         1.23110290021662e-25,
                                         7.50652242635137e-20,
                                         6.32627127326674e-16,
                                         5.47163349764397e-13,
                                         1.12881463087298e-10,
                                         8.57329790235931e-09,
                                         3.08414932250002e-07,
                                         6.15719576375076e-06,
                                         7.57749947088625e-05,
                                         0.000617881010770669,
                                         0.00351087204463124,
                                         0.0143974106643393,
                                         0.0436193542803973,
                                         0.0989841408029804,
                                         0.169048909305632,
                                         0.216248773372189,
                                         0.203821763219362,
                                         0.136955959494178,
                                         0.0617963227476242,
                                         0.0167745221158072,
                                         0.00219735517745409,
                                         8.16479304782175e-05,
                                         1.2535818793517e-07,
                                         1.21630077605615e-39,
                                         3.93873729408349e-28,
                                         5.97623617342184e-22,
                                         9.52290439612571e-18,
                                         1.352860011479e-14,
                                         4.21452124187123e-12,
                                         4.57649653605823e-10,
                                         2.26797039997643e-08,
                                         6.07870693672153e-07,
                                         9.86710129701111e-06,
                                         0.0001048997512773,
                                         0.000772039660721008,
                                         0.00409203933976805,
                                         0.0160579329215293,
                                         0.0475096644383744,
                                         0.107019802800847,
                                         0.183794150238421,
                                         0.238641158320531,
                                         0.229263719990226,
                                         0.156209265328691,
                                         0.0696479856418482,
                                         0.0172500879311532,
                                         0.00159500679569643,
                                         1.29921300337553e-05,
                                         5.17756634913953e-43,
                                         8.89512387770318e-31,
                                         3.35852853153928e-24,
                                         1.01186683708802e-19,
                                         2.36113622756396e-16,
                                         1.11072408456401e-13,
                                         1.72445031481698e-11,
                                         1.17725811346822e-09,
                                         4.23615433812975e-08,
                                         9.06954674361567e-07,
                                         1.25711900849241e-05,
                                         0.000119838549943437,
                                         0.000820970463873308,
                                         0.00417284613487247,
                                         0.016096469572764,
                                         0.0478242584151656,
                                         0.110266106844041,
                                         0.19722976946956,
                                         0.270908143187314,
                                         0.278729715596806,
                                         0.204126452800183,
                                         0.0955904904177042,
                                         0.0219944050604227,
                                         0.000950474207563666,
                                         1.38769977961005e-46,
                                         1.2648299575369e-33,
                                         1.18838042495444e-26,
                                         6.76959142806304e-22,
                                         2.59462373656674e-18,
                                         1.84310165144269e-15,
                                         4.0912252501223e-13,
                                         3.8476110161787e-11,
                                         1.85873510158722e-09,
                                         5.24888112825586e-08,
                                         9.48557113518876e-07,
                                         1.17122042315649e-05,
                                         0.000103705173513616,
                                         0.00068274772294787,
                                         0.00343371669594209,
                                         0.0134560471304771,
                                         0.0416521625974604,
                                         0.102632461046319,
                                         0.201555169550394,
                                         0.313144588277129,
                                         0.376682266000991,
                                         0.333521030091369,
                                         0.190962045255096,
                                         0.0437809792236976,
                                         1.76179054184082e-50,
                                         8.5192466955709e-37,
                                         1.99182346961503e-29,
                                         2.14531200002505e-24,
                                         1.35056856596338e-20,
                                         1.44870945960008e-17,
                                         4.59774631190177e-15,
                                         5.95661506137384e-13,
                                         3.86324396572488e-11,
                                         1.43892049953694e-09,
                                         3.3903105868346e-08,
                                         5.42212638868441e-07,
                                         6.20529197044094e-06,
                                         5.29147876926242e-05,
                                         0.000346966220898501,
                                         0.00179339379703171,
                                         0.0074528430827377,
                                         0.0252979848136087,
                                         0.0710321126895169,
                                         0.166646216581012,
                                         0.329260754059415,
                                         0.551214540959937,
                                         0.785363664437344,
                                         0.955255428168223,

                                         9.5367431640625e-07,
                                         1.9073486328125e-05,
                                         0.000181198120117188,
                                         0.00108718872070312,
                                         0.00462055206298828,
                                         0.0147857666015625,
                                         0.0369644165039062,
                                         0.0739288330078125,
                                         0.120134353637695,
                                         0.160179138183594,
                                         0.176197052001953,
                                         0.160179138183594,
                                         0.120134353637695,
                                         0.0739288330078125,
                                         0.0369644165039062,
                                         0.0147857666015625,
                                         0.00462055206298828,
                                         0.00108718872070312,
                                         0.000181198120117188,
                                         1.9073486328125e-05,
                                         9.5367431640625e-07,

                                         0.00866887960631235,
                                         3.15509195190533e-14,
                                         0.0464563857958805,
                                         2.35499269341247e-12,
                                         0.118255535044111,
                                         8.34950476410538e-11,
                                         0.190118850693479,
                                         1.86964655986078e-09,
                                         0.216504318164316,
                                         2.96548678027864e-08,
                                         0.185638903071934,
                                         3.5415511466156e-07,
                                         0.12435448540481,
                                         3.30431220419331e-06,
                                         0.0666413678788135,
                                         2.46637220602387e-05,
                                         0.0290168136466543,
                                         0.000149575178727923,
                                         0.0103667090447272,
                                         0.000744296222085074,
                                         0.00305552644837338,
                                         0.00305552644837338,
                                         0.000744296222085074,
                                         0.0103667090447272,
                                         0.000149575178727923,
                                         0.0290168136466543,
                                         2.46637220602387e-05,
                                         0.0666413678788135,
                                         3.30431220419331e-06,
                                         0.12435448540481,
                                         3.5415511466156e-07,
                                         0.185638903071934,
                                         2.96548678027864e-08,
                                         0.216504318164316,
                                         1.86964655986078e-09,
                                         0.190118850693479,
                                         8.34950476410538e-11,
                                         0.118255535044111,
                                         2.35499269341247e-12,
                                         0.0464563857958804,
                                         3.15509195190533e-14,
                                         0.00866887960631235,

                                         0.0914941501999996,
                                         9.5367431640625e-07,
                                         1.09296605063177e-19,
                                         0.232425615999999,
                                         1.9073486328125e-05,
                                         1.7209807029188e-17,
                                         0.280458277999999,
                                         0.000181198120117188,
                                         1.28717897925628e-15,
                                         0.21373723199994,
                                         0.00108718872070312,
                                         6.08036320036375e-14,
                                         0.115379798997966,
                                         0.00462055206298828,
                                         2.03450042415264e-12,
                                         0.046896499148744,
                                         0.0147857666015625,
                                         5.12562814630579e-11,
                                         0.0148915909911505,
                                         0.0369644165039062,
                                         1.00884962586794e-09,
                                         0.00378296011468743,
                                         0.0739288330078125,
                                         1.58853126065716e-08,
                                         0.000780810769697408,
                                         0.120134353637695,
                                         2.0323030260136e-07,
                                         0.000132234628282902,
                                         0.160179138183594,
                                         2.13337171710026e-06,
                                         1.84756000000003e-05,
                                         0.176197052001953,
                                         1.84756000000003e-05,
                                         2.13337171710026e-06,
                                         0.160179138183594,
                                         0.000132234628282902,
                                         2.0323030260136e-07,
                                         0.120134353637695,
                                         0.000780810769697408,
                                         1.58853126065716e-08,
                                         0.0739288330078125,
                                         0.00378296011468743,
                                         1.00884962586794e-09,
                                         0.0369644165039062,
                                         0.0148915909911505,
                                         5.12562814630579e-11,
                                         0.0147857666015625,
                                         0.046896499148744,
                                         2.03450042415264e-12,
                                         0.00462055206298828,
                                         0.115379798997966,
                                         6.08036320036375e-14,
                                         0.00108718872070312,
                                         0.21373723199994,
                                         1.28717897925628e-15,
                                         0.000181198120117188,
                                         0.280458277999999,
                                         1.7209807029188e-17,
                                         1.9073486328125e-05,
                                         0.232425615999999,
                                         1.09296605063177e-19,
                                         9.5367431640625e-07,
                                         0.0914941501999996,

                                         0.23711757040959,
                                         0.00033217976806547,
                                         2.34708203192954e-10,
                                         6.77913085608552e-24,
                                         0.353837816261085,
                                         0.00327235888761902,
                                         9.53016697461646e-09,
                                         1.81716138209218e-21,
                                         0.250806256154146,
                                         0.0153123655214686,
                                         1.83808825729311e-07,
                                         2.3136975673827e-19,
                                         0.112279412059899,
                                         0.0452534081438455,
                                         2.23902971022501e-06,
                                         1.86057562180597e-17,
                                         0.0356040640723796,
                                         0.0947322772074811,
                                         1.93192842249369e-05,
                                         1.05980445877032e-15,
                                         0.00850080523995183,
                                         0.14931578094338,
                                         0.000125511423624072,
                                         4.54532777355727e-14,
                                         0.0015856618919939,
                                         0.183866865853018,
                                         0.000637038250094667,
                                         1.52298175724931e-12,
                                         0.00023661980857107,
                                         0.181130348821905,
                                         0.00258665049198498,
                                         4.08238709895972e-11,
                                         2.86889714313354e-05,
                                         0.144978079811233,
                                         0.00853362121181441,
                                         8.89113151321057e-10,
                                         2.85406742444417e-06,
                                         0.0952135667870708,
                                         0.0231001257278486,
                                         1.58885862823211e-08,
                                         2.34243434913126e-07,
                                         0.0515880692619105,
                                         0.0515880692619105,
                                         2.34243434913128e-07,
                                         1.5888586282321e-08,
                                         0.0231001257278486,
                                         0.0952135667870708,
                                         2.85406742444419e-06,
                                         8.89113151321048e-10,
                                         0.00853362121181441,
                                         0.144978079811233,
                                         2.86889714313356e-05,
                                         4.08238709895968e-11,
                                         0.00258665049198498,
                                         0.181130348821905,
                                         0.000236619808571071,
                                         1.52298175724929e-12,
                                         0.000637038250094667,
                                         0.183866865853018,
                                         0.00158566189199391,
                                         4.54532777355721e-14,
                                         0.000125511423624072,
                                         0.14931578094338,
                                         0.00850080523995186,
                                         1.05980445877031e-15,
                                         1.93192842249369e-05,
                                         0.0947322772074811,
                                         0.0356040640723797,
                                         1.86057562180595e-17,
                                         2.23902971022501e-06,
                                         0.0452534081438455,
                                         0.112279412059899,
                                         2.31369756738266e-19,
                                         1.83808825729311e-07,
                                         0.0153123655214686,
                                         0.250806256154146,
                                         1.81716138209215e-21,
                                         9.53016697461646e-09,
                                         0.00327235888761902,
                                         0.353837816261085,
                                         6.77913085608541e-24,
                                         2.34708203192954e-10,
                                         0.00033217976806547,
                                         0.23711757040959,

                                         0.382540618237738,
                                         0.00526231487767613,
                                         9.5367431640625e-07,
                                         1.83405260929785e-13,
                                         2.66270419186002e-27,
                                         0.376564885147129,
                                         0.0315731981129758,
                                         1.9073486328125e-05,
                                         1.22272850508385e-11,
                                         1.08198352837957e-24,
                                         0.176074187507401,
                                         0.0899816448788187,
                                         0.000181198120117188,
                                         3.87205835886878e-10,
                                         2.08839183357348e-22,
                                         0.0519971106609858,
                                         0.161963415321793,
                                         0.00108718872070312,
                                         7.74428624032155e-09,
                                         2.54583919419582e-20,
                                         0.0108767816474641,
                                         0.206498834172637,
                                         0.00462055206298828,
                                         1.097131233604e-07,
                                         2.19830339216538e-18,
                                         0.00171309976993964,
                                         0.198234541352581,
                                         0.0147857666015625,
                                         1.17029893371184e-06,
                                         1.42924058510149e-16,
                                         0.000210792392806765,
                                         0.148672651495816,
                                         0.0369644165039062,
                                         9.75270460116825e-06,
                                         7.25960649280361e-15,
                                         2.07499568419261e-05,
                                         0.0892016382290627,
                                         0.0739288330078125,
                                         6.50194539516302e-05,
                                         2.94992386752656e-13,
                                         1.65959772720974e-06,
                                         0.0434848467316476,
                                         0.120134353637695,
                                         0.000352196418491459,
                                         9.73939331515235e-12,
                                         1.08911524291831e-07,
                                         0.0173935579389857,
                                         0.160179138183594,
                                         0.00156535168109787,
                                         2.63838623797555e-10,
                                         5.8965609204577e-09,
                                         0.00573974847390355,
                                         0.176197052001953,
                                         0.00573974847390355,
                                         5.89656092045777e-09,
                                         2.63838623797552e-10,
                                         0.00156535168109787,
                                         0.160179138183594,
                                         0.0173935579389857,
                                         1.08911524291832e-07,
                                         9.73939331515221e-12,
                                         0.000352196418491459,
                                         0.120134353637695,
                                         0.0434848467316475,
                                         1.65959772720976e-06,
                                         2.94992386752651e-13,
                                         6.50194539516302e-05,
                                         0.0739288330078125,
                                         0.0892016382290627,
                                         2.07499568419263e-05,
                                         7.25960649280349e-15,
                                         9.75270460116825e-06,
                                         0.0369644165039062,
                                         0.148672651495816,
                                         0.000210792392806766,
                                         1.42924058510146e-16,
                                         1.17029893371184e-06,
                                         0.0147857666015625,
                                         0.198234541352581,
                                         0.00171309976993965,
                                         2.19830339216534e-18,
                                         1.097131233604e-07,
                                         0.00462055206298828,
                                         0.206498834172637,
                                         0.0108767816474642,
                                         2.54583919419577e-20,
                                         7.74428624032156e-09,
                                         0.00108718872070312,
                                         0.161963415321793,
                                         0.051997110660986,
                                         2.08839183357344e-22,
                                         3.87205835886878e-10,
                                         0.000181198120117188,
                                         0.0899816448788187,
                                         0.176074187507401,
                                         1.08198352837954e-24,
                                         1.22272850508385e-11,
                                         1.9073486328125e-05,
                                         0.0315731981129758,
                                         0.37656488514713,
                                         2.66270419185995e-27,
                                         1.83405260929785e-13,
                                         9.5367431640625e-07,
                                         0.00526231487767613,
                                         0.382540618237738,

                                         0.50309860695818,
                                         0.0244279832877015,
                                         6.88899936572555e-05,
                                         4.08732021091967e-09,
                                         3.78466525930373e-16,
                                         3.71020208712561e-30,
                                         0.351617379543632,
                                         0.0996379085364657,
                                         0.000846935362067958,
                                         1.32985574114136e-07,
                                         3.71150864412133e-14,
                                         2.12344168424087e-27,
                                         0.116729643943376,
                                         0.193043508045,
                                         0.00494581648196563,
                                         2.05524694779943e-06,
                                         1.72888891064379e-12,
                                         5.77266986603916e-25,
                                         0.0244748271715109,
                                         0.23621824820672,
                                         0.0182411986872091,
                                         2.00609334132352e-05,
                                         5.0864097854935e-11,
                                         9.91153112557697e-23,
                                         0.00363492777779321,
                                         0.204743143779266,
                                         0.0476547756987498,
                                         0.000138699833761656,
                                         1.05996929081082e-09,
                                         1.20543127938857e-20,
                                         0.000406474202037057,
                                         0.133618422073296,
                                         0.0937390470064291,
                                         0.000722040890187222,
                                         1.66316856717575e-08,
                                         1.10383767397404e-18,
                                         3.55107805397479e-05,
                                         0.0681260706242416,
                                         0.144053783252043,
                                         0.00293655186529368,
                                         2.03877492815551e-07,
                                         7.89692473115352e-17,
                                         2.4818608968983e-06,
                                         0.0277875546002377,
                                         0.177100093350029,
                                         0.00955440277663367,
                                         1.9993659282457e-06,
                                         4.51960803149543e-15,
                                         1.40934728892676e-07,
                                         0.00920896213686774,
                                         0.17690344130283,
                                         0.0252576409313054,
                                         1.59308393908891e-05,
                                         2.10168156354474e-13,
                                         6.56665172917804e-09,
                                         0.00250412738075124,
                                         0.144990365225174,
                                         0.0547857227831042,
                                         0.00010415266882946,
                                         8.01896704318313e-12,
                                         2.52420074872051e-10,
                                         0.000561766833526992,
                                         0.0980383413063039,
                                         0.0980383413063039,
                                         0.00056176683352699,
                                         2.52420074872051e-10,
                                         8.01896704318313e-12,
                                         0.00010415266882946,
                                         0.0547857227831042,
                                         0.144990365225174,
                                         0.00250412738075123,
                                         6.56665172917804e-09,
                                         2.10168156354474e-13,
                                         1.59308393908892e-05,
                                         0.0252576409313054,
                                         0.17690344130283,
                                         0.00920896213686772,
                                         1.40934728892676e-07,
                                         4.51960803149543e-15,
                                         1.99936592824571e-06,
                                         0.00955440277663367,
                                         0.177100093350029,
                                         0.0277875546002376,
                                         2.4818608968983e-06,
                                         7.89692473115352e-17,
                                         2.03877492815552e-07,
                                         0.00293655186529368,
                                         0.144053783252043,
                                         0.0681260706242414,
                                         3.55107805397479e-05,
                                         1.10383767397404e-18,
                                         1.66316856717576e-08,
                                         0.000722040890187222,
                                         0.0937390470064291,
                                         0.133618422073296,
                                         0.000406474202037057,
                                         1.20543127938857e-20,
                                         1.05996929081083e-09,
                                         0.000138699833761656,
                                         0.0476547756987498,
                                         0.204743143779266,
                                         0.00363492777779321,
                                         9.91153112557697e-23,
                                         5.08640978549353e-11,
                                         2.00609334132352e-05,
                                         0.0182411986872091,
                                         0.236218248206719,
                                         0.0244748271715109,
                                         5.77266986603916e-25,
                                         1.7288889106438e-12,
                                         2.05524694779943e-06,
                                         0.00494581648196563,
                                         0.193043508045,
                                         0.116729643943376,
                                         2.12344168424087e-27,
                                         3.71150864412136e-14,
                                         1.32985574114136e-07,
                                         0.000846935362067958,
                                         0.0996379085364657,
                                         0.351617379543632,
                                         3.71020208712561e-30,
                                         3.78466525930375e-16,
                                         4.08732021091967e-09,
                                         6.88899936572555e-05,
                                         0.0244279832877015,
                                         0.50309860695818,

                                         0.597197218910661,
                                         0.0628094806267077,
                                         0.000867261035194381,
                                         9.5367431640625e-07,
                                         2.86677096829592e-11,
                                         1.68874120812536e-18,
                                         1.29540454831156e-32,
                                         0.31186178067413,
                                         0.186436994352364,
                                         0.00733064162261881,
                                         1.9073486328125e-05,
                                         1.35662818379128e-09,
                                         2.27570624732994e-16,
                                         9.92249825475398e-30,
                                         0.0773570931069586,
                                         0.262864896274473,
                                         0.0294325406063386,
                                         0.000181198120117188,
                                         3.04945537494974e-08,
                                         1.45667582287258e-14,
                                         3.61019162533928e-27,
                                         0.0121189717769196,
                                         0.234078075601789,
                                         0.0746347634008089,
                                         0.00108718872070312,
                                         4.32923713050895e-07,
                                         5.88894187190263e-13,
                                         8.29596904267025e-25,
                                         0.0013448352566034,
                                         0.147647658283462,
                                         0.134057849582162,
                                         0.00462055206298828,
                                         4.35349596718653e-06,
                                         1.68635615718308e-11,
                                         1.35033545479053e-22,
                                         0.000112365618474303,
                                         0.0701218614995612,
                                         0.181302747312384,
                                         0.0147857666015625,
                                         3.29629420273786e-05,
                                         3.63598753785129e-10,
                                         1.65492100035689e-20,
                                         7.33479258490744e-06,
                                         0.0260177861803794,
                                         0.191560760227948,
                                         0.0369644165039062,
                                         0.000194986173771677,
                                         6.12470957309031e-09,
                                         1.58453885691541e-18,
                                         3.83029492430848e-07,
                                         0.00772284344142436,
                                         0.161919332842246,
                                         0.0739288330078125,
                                         0.000922723655686798,
                                         8.25350845440402e-08,
                                         1.21371999680146e-16,
                                         1.62517436148971e-08,
                                         0.00186254807407114,
                                         0.111202417616557,
                                         0.120134353637695,
                                         0.00354783049094963,
                                         9.03680832660199e-07,
                                         7.55366100470097e-15,
                                         5.65787151749328e-10,
                                         0.000368572664828937,
                                         0.0626635686400201,
                                         0.160179138183594,
                                         0.0111928179557302,
                                         8.11852089032254e-06,
                                         3.85728603644015e-13,
                                         1.62502537975511e-11,
                                         6.01717583556342e-05,
                                         0.029131977596027,
                                         0.176197052001953,
                                         0.029131977596027,
                                         6.01717583556342e-05,
                                         1.62502537975515e-11,
                                         3.85728603644006e-13,
                                         8.11852089032254e-06,
                                         0.0111928179557302,
                                         0.160179138183594,
                                         0.0626635686400201,
                                         0.000368572664828937,
                                         5.65787151749339e-10,
                                         7.55366100470077e-15,
                                         9.03680832660199e-07,
                                         0.00354783049094963,
                                         0.120134353637695,
                                         0.111202417616557,
                                         0.00186254807407114,
                                         1.62517436148974e-08,
                                         1.21371999680143e-16,
                                         8.25350845440402e-08,
                                         0.000922723655686798,
                                         0.0739288330078125,
                                         0.161919332842246,
                                         0.00772284344142436,
                                         3.83029492430853e-07,
                                         1.58453885691536e-18,
                                         6.12470957309031e-09,
                                         0.000194986173771677,
                                         0.0369644165039062,
                                         0.191560760227948,
                                         0.0260177861803794,
                                         7.33479258490754e-06,
                                         1.65492100035684e-20,
                                         3.63598753785129e-10,
                                         3.29629420273786e-05,
                                         0.0147857666015625,
                                         0.181302747312384,
                                         0.0701218614995611,
                                         0.000112365618474304,
                                         1.35033545479048e-22,
                                         1.68635615718308e-11,
                                         4.35349596718653e-06,
                                         0.00462055206298828,
                                         0.134057849582162,
                                         0.147647658283462,
                                         0.00134483525660341,
                                         8.29596904266995e-25,
                                         5.88894187190263e-13,
                                         4.32923713050895e-07,
                                         0.00108718872070312,
                                         0.0746347634008088,
                                         0.234078075601789,
                                         0.0121189717769197,
                                         3.61019162533914e-27,
                                         1.45667582287258e-14,
                                         3.04945537494974e-08,
                                         0.000181198120117188,
                                         0.0294325406063386,
                                         0.262864896274473,
                                         0.0773570931069589,
                                         9.92249825475357e-30,
                                         2.27570624732994e-16,
                                         1.35662818379128e-09,
                                         1.9073486328125e-05,
                                         0.00733064162261881,
                                         0.186436994352364,
                                         0.311861780674131,
                                         1.2954045483115e-32,
                                         1.68874120812536e-18,
                                         2.86677096829592e-11,
                                         9.5367431640625e-07,
                                         0.000867261035194381,
                                         0.0628094806267077,
                                         0.597197218910661,

                                         0.669585345387209,
                                         0.117151912040946,
                                         0.0044445608848688,
                                         2.76876215368766e-05,
                                         1.65660554420277e-08,
                                         3.18809406484808e-13,
                                         1.39180943732678e-20,
                                         9.06628186080317e-35,
                                         0.271279576990521,
                                         0.265167867853867,
                                         0.027646742583981,
                                         0.000382087050131083,
                                         4.80178193194898e-07,
                                         2.050104548102e-11,
                                         2.45962134249725e-18,
                                         8.95113379118223e-32,
                                         0.0522061742608171,
                                         0.285092650522472,
                                         0.0816869512578517,
                                         0.0025045666706918,
                                         6.6111858409943e-06,
                                         6.26202073092115e-10,
                                         2.06466853037122e-16,
                                         4.19778788641803e-29,
                                         0.00634533101747231,
                                         0.193588159927162,
                                         0.152436528988804,
                                         0.0103688482933071,
                                         5.7488892569149e-05,
                                         1.20803811803923e-08,
                                         1.09461165723318e-14,
                                         1.24334192037803e-26,
                                         0.000546291416479514,
                                         0.0931127689425649,
                                         0.20149445291334,
                                         0.0304064783473214,
                                         0.000354100222595944,
                                         1.65076196878322e-07,
                                         4.11062140301136e-13,
                                         2.60854505769463e-24,
                                         3.5412412866453e-05,
                                         0.0337210228740817,
                                         0.200538695852187,
                                         0.06713713043862,
                                         0.00164221307302561,
                                         1.69843652098016e-06,
                                         1.16229384427131e-11,
                                         4.12066356113633e-22,
                                         1.79339804268523e-06,
                                         0.00954074455721058,
                                         0.155927712713636,
                                         0.11581090528755,
                                         0.00595008048993616,
                                         1.36522494533496e-05,
                                         2.56752349581145e-10,
                                         5.08541034385416e-20,
                                         7.26587380902035e-08,
                                         0.0021595028607725,
                                         0.0969925589225017,
                                         0.159818159589457,
                                         0.0172467061275997,
                                         8.77908183598992e-05,
                                         4.53735649313482e-09,
                                         5.02082210434036e-18,
                                         2.39178633373933e-09,
                                         0.00039714460641912,
                                         0.0490204051103145,
                                         0.179195113860852,
                                         0.0406174688079165,
                                         0.000458687969289778,
                                         6.51500491942722e-08,
                                         4.02760652147612e-16,
                                         6.46014521612427e-11,
                                         5.9927881488333e-05,
                                         0.020328285239024,
                                         0.164858586984573,
                                         0.0784882993725235,
                                         0.0019663959571102,
                                         7.67559347160609e-07,
                                         2.65096874959183e-14,
                                         1.43951478400822e-12,
                                         7.46041746594672e-06,
                                         0.00695470229915629,
                                         0.125126970939704,
                                         0.125126970939703,
                                         0.00695470229915629,
                                         7.46041746594673e-06,
                                         1.43951478400822e-12,
                                         2.65096874959183e-14,
                                         7.67559347160609e-07,
                                         0.0019663959571102,
                                         0.0784882993725237,
                                         0.164858586984573,
                                         0.020328285239024,
                                         5.9927881488333e-05,
                                         6.46014521612427e-11,
                                         4.02760652147612e-16,
                                         6.51500491942722e-08,
                                         0.000458687969289778,
                                         0.0406174688079165,
                                         0.179195113860852,
                                         0.0490204051103145,
                                         0.000397144606419121,
                                         2.39178633373933e-09,
                                         5.02082210434036e-18,
                                         4.53735649313482e-09,
                                         8.77908183598993e-05,
                                         0.0172467061275997,
                                         0.159818159589457,
                                         0.0969925589225017,
                                         0.0021595028607725,
                                         7.26587380902035e-08,
                                         5.08541034385416e-20,
                                         2.56752349581145e-10,
                                         1.36522494533496e-05,
                                         0.00595008048993617,
                                         0.11581090528755,
                                         0.155927712713636,
                                         0.00954074455721058,
                                         1.79339804268523e-06,
                                         4.12066356113633e-22,
                                         1.16229384427131e-11,
                                         1.69843652098016e-06,
                                         0.00164221307302562,
                                         0.0671371304386199,
                                         0.200538695852187,
                                         0.0337210228740817,
                                         3.5412412866453e-05,
                                         2.60854505769463e-24,
                                         4.11062140301136e-13,
                                         1.65076196878322e-07,
                                         0.000354100222595945,
                                         0.0304064783473214,
                                         0.20149445291334,
                                         0.0931127689425649,
                                         0.000546291416479514,
                                         1.24334192037803e-26,
                                         1.09461165723318e-14,
                                         1.20803811803923e-08,
                                         5.74888925691492e-05,
                                         0.0103688482933071,
                                         0.152436528988804,
                                         0.193588159927162,
                                         0.00634533101747231,
                                         4.19778788641803e-29,
                                         2.06466853037122e-16,
                                         6.26202073092115e-10,
                                         6.61118584099431e-06,
                                         0.0025045666706918,
                                         0.0816869512578517,
                                         0.285092650522472,
                                         0.0522061742608171,
                                         8.95113379118223e-32,
                                         2.45962134249725e-18,
                                         2.050104548102e-11,
                                         4.80178193194899e-07,
                                         0.000382087050131083,
                                         0.027646742583981,
                                         0.265167867853867,
                                         0.271279576990521,
                                         9.06628186080317e-35,
                                         1.39180943732678e-20,
                                         3.18809406484808e-13,
                                         1.65660554420278e-08,
                                         2.76876215368766e-05,
                                         0.0044445608848688,
                                         0.117151912040946,
                                         0.669585345387209,

                                         0.725452809245899,
                                         0.180717740379098,
                                         0.0136171851581212,
                                         0.000262314424788111,
                                         9.5367431640625e-07,
                                         3.75904193482553e-10,
                                         5.3122546980215e-15,
                                         1.88204222145556e-22,
                                         1.09344245091396e-36,
                                         0.234719137509736,
                                         0.322784158265588,
                                         0.0652644850606107,
                                         0.00267710200192268,
                                         1.9073486328125e-05,
                                         1.47331094919801e-08,
                                         4.43352648920025e-13,
                                         4.21480929407516e-20,
                                         1.35181290487035e-33,
                                         0.036072932085034,
                                         0.273852838072866,
                                         0.148579912541692,
                                         0.0129778058865104,
                                         0.000181198120117188,
                                         2.74287029930554e-07,
                                         1.75757285139896e-11,
                                         4.4835302640286e-18,
                                         7.93836119054204e-31,
                                         0.0035014024614149,
                                         0.146740476569532,
                                         0.213634272497225,
                                         0.0397342732646959,
                                         0.00108718872070312,
                                         3.22510435977705e-06,
                                         4.40053022849901e-10,
                                         3.01224246911103e-16,
                                         2.94423700815538e-28,
                                         0.000240735590255438,
                                         0.0556955226788792,
                                         0.217580230979071,
                                         0.0861721550383828,
                                         0.00462055206298828,
                                         2.68608624268936e-05,
                                         7.80430729374037e-09,
                                         1.43349911316659e-14,
                                         7.73485825065686e-26,
                                         1.2462340627535e-05,
                                         0.0159166509004388,
                                         0.1668510673162,
                                         0.140711529043569,
                                         0.0147857666015625,
                                         0.000168444634157689,
                                         1.04213687282858e-07,
                                         5.13648445616719e-13,
                                         1.53000552584598e-23,
                                         5.04021386051327e-07,
                                         0.00355363753478763,
                                         0.0999605357472752,
                                         0.1795074348277,
                                         0.0369644165039062,
                                         0.000825247922678781,
                                         1.08718936066324e-06,
                                         1.43788634082706e-11,
                                         2.36441480371551e-21,
                                         1.63075342065881e-08,
                                         0.000634723463253359,
                                         0.04790911496153,
                                         0.183199880649119,
                                         0.0739288330078125,
                                         0.0032344592621364,
                                         9.07351605538107e-06,
                                         3.2201279248956e-10,
                                         2.92310440431275e-19,
                                         4.28697550139207e-10,
                                         9.21126648607641e-05,
                                         0.0186565394471035,
                                         0.151911727196306,
                                         0.120134353637695,
                                         0.0103001203942743,
                                         6.15275185503009e-05,
                                         5.85929091966104e-09,
                                         2.93621839022575e-17,
                                         9.2469620756181e-12,
                                         1.09683048383669e-05,
                                         0.00596114114084245,
                                         0.10335768847412,
                                         0.160179138183594,
                                         0.0269133827502714,
                                         0.000342332837077151,
                                         8.74787098941133e-08,
                                         2.42001330026231e-15,
                                         1.64551217449521e-13,
                                         1.07749093729492e-06,
                                         0.00157138161321301,
                                         0.0580160588665382,
                                         0.176197052001953,
                                         0.0580160588665382,
                                         0.00157138161321301,
                                         1.07749093729492e-06,
                                         1.64551217449527e-13,
                                         2.42001330026222e-15,
                                         8.74787098941133e-08,
                                         0.00034233283707715,
                                         0.0269133827502714,
                                         0.160179138183594,
                                         0.10335768847412,
                                         0.00596114114084247,
                                         1.09683048383669e-05,
                                         9.24696207561839e-12,
                                         2.93621839022562e-17,
                                         5.85929091966104e-09,
                                         6.15275185503006e-05,
                                         0.0103001203942743,
                                         0.120134353637695,
                                         0.151911727196306,
                                         0.0186565394471036,
                                         9.21126648607641e-05,
                                         4.28697550139219e-10,
                                         2.92310440431261e-19,
                                         3.2201279248956e-10,
                                         9.07351605538104e-06,
                                         0.0032344592621364,
                                         0.0739288330078125,
                                         0.183199880649119,
                                         0.0479091149615301,
                                         0.000634723463253359,
                                         1.63075342065885e-08,
                                         2.36441480371539e-21,
                                         1.43788634082706e-11,
                                         1.08718936066324e-06,
                                         0.000825247922678781,
                                         0.0369644165039062,
                                         0.1795074348277,
                                         0.0999605357472754,
                                         0.00355363753478763,
                                         5.04021386051337e-07,
                                         1.5300055258459e-23,
                                         5.1364844561672e-13,
                                         1.04213687282858e-07,
                                         0.000168444634157689,
                                         0.0147857666015625,
                                         0.140711529043569,
                                         0.166851067316201,
                                         0.0159166509004388,
                                         1.24623406275353e-05,
                                         7.73485825065643e-26,
                                         1.43349911316659e-14,
                                         7.80430729374033e-09,
                                         2.68608624268936e-05,
                                         0.00462055206298828,
                                         0.0861721550383828,
                                         0.217580230979071,
                                         0.0556955226788792,
                                         0.000240735590255441,
                                         2.94423700815521e-28,
                                         3.01224246911103e-16,
                                         4.40053022849899e-10,
                                         3.22510435977705e-06,
                                         0.00108718872070312,
                                         0.0397342732646959,
                                         0.213634272497225,
                                         0.146740476569532,
                                         0.00350140246141494,
                                         7.93836119054155e-31,
                                         4.4835302640286e-18,
                                         1.75757285139895e-11,
                                         2.74287029930554e-07,
                                         0.000181198120117188,
                                         0.0129778058865105,
                                         0.148579912541692,
                                         0.273852838072866,
                                         0.0360729320850342,
                                         1.35181290487026e-33,
                                         4.21480929407516e-20,
                                         4.43352648920023e-13,
                                         1.47331094919801e-08,
                                         1.9073486328125e-05,
                                         0.00267710200192268,
                                         0.0652644850606107,
                                         0.322784158265588,
                                         0.234719137509737,
                                         1.09344245091388e-36,
                                         1.88204222145556e-22,
                                         5.31225469802147e-15,
                                         3.75904193482553e-10,
                                         9.5367431640625e-07,
                                         0.000262314424788111,
                                         0.0136171851581212,
                                         0.180717740379098,
                                         0.725452809245899,

                                         0.769009589422688,
                                         0.247327224076175,
                                         0.0303761375699783,
                                         0.00127855725060629,
                                         1.53056110039994e-05,
                                         3.79554177922605e-08,
                                         1.10918017197615e-11,
                                         1.25432827740069e-16,
                                         3.8194206479036e-24,
                                         2.04191339823408e-38,
                                         0.203313880386182,
                                         0.357880633323955,
                                         0.115972890005991,
                                         0.0101079776038076,
                                         0.000226778470735384,
                                         1.02466668610791e-06,
                                         5.61200432646242e-10,
                                         1.31415706930366e-14,
                                         1.05582321977186e-21,
                                         3.08931388458099e-35,
                                         0.0255326512178366,
                                         0.245979027940455,
                                         0.21031682560183,
                                         0.0379578820702381,
                                         0.00159605033202723,
                                         1.3139688412833e-05,
                                         1.34873772936795e-08,
                                         6.53998794502442e-13,
                                         1.38636803257088e-19,
                                         2.22014000971658e-32,
                                         0.002025127826901,
                                         0.106778941096223,
                                         0.240890238521009,
                                         0.0900258682210744,
                                         0.00709445418583927,
                                         0.000106418016970533,
                                         2.04722068525368e-07,
                                         2.05557943944105e-11,
                                         1.14972376300672e-17,
                                         1.00768857734067e-29,
                                         0.000113774948994206,
                                         0.0328330190163344,
                                         0.195434874747341,
                                         0.151241280047076,
                                         0.0223372175400986,
                                         0.000610496160126351,
                                         2.20109858798642e-06,
                                         4.57645569235763e-10,
                                         6.75376833052167e-16,
                                         3.23973872815054e-27,
                                         4.81284534058779e-06,
                                         0.00760146105768555,
                                         0.119383958833838,
                                         0.191308566997636,
                                         0.0529541751233952,
                                         0.00263700989704118,
                                         1.78186738073291e-05,
                                         7.67158863761538e-09,
                                         2.98716945073375e-14,
                                         7.84250289474253e-25,
                                         1.59054951224349e-07,
                                         0.00137490712338344,
                                         0.0569744206880345,
                                         0.18905515471752,
                                         0.0980756897780768,
                                         0.00889878293272906,
                                         0.000112694128764032,
                                         1.00468838805539e-07,
                                         1.03219936942824e-12,
                                         1.48316482860805e-22,
                                         4.20515943791212e-09,
                                         0.000198948026815897,
                                         0.021752233009831,
                                         0.149462628197751,
                                         0.145315694606236,
                                         0.0240236755342275,
                                         0.000570186840848306,
                                         1.05260988801619e-06,
                                         2.85336484808078e-11,
                                         2.2439549601387e-20,
                                         9.03318927310313e-11,
                                         2.33899492695819e-05,
                                         0.00674762820929125,
                                         0.0960063758890391,
                                         0.174939162361983,
                                         0.0526952735909065,
                                         0.00234399380515041,
                                         8.96038927578718e-06,
                                         6.4087656879969e-10,
                                         2.75843162194832e-18,
                                         1.5921541929936e-12,
                                         2.25633871329236e-06,
                                         0.00171744886330527,
                                         0.0506002786525706,
                                         0.172801271820351,
                                         0.0948394976785178,
                                         0.00790644219821608,
                                         6.25852052855477e-05,
                                         1.18107330007031e-08,
                                         2.78224699007706e-16,
                                         2.31517107695003e-14,
                                         1.79569783268884e-07,
                                         0.000360637125806042,
                                         0.0220018748255691,
                                         0.140818844049217,
                                         0.140818844049217,
                                         0.0220018748255692,
                                         0.000360637125806042,
                                         1.79569783268883e-07,
                                         2.31517107695003e-14,
                                         2.78224699007706e-16,
                                         1.18107330007032e-08,
                                         6.25852052855477e-05,
                                         0.00790644219821608,
                                         0.0948394976785178,
                                         0.172801271820351,
                                         0.0506002786525706,
                                         0.00171744886330527,
                                         2.25633871329235e-06,
                                         1.5921541929936e-12,
                                         2.75843162194832e-18,
                                         6.40876568799697e-10,
                                         8.96038927578718e-06,
                                         0.00234399380515041,
                                         0.0526952735909065,
                                         0.174939162361983,
                                         0.0960063758890391,
                                         0.00674762820929125,
                                         2.33899492695817e-05,
                                         9.03318927310313e-11,
                                         2.2439549601387e-20,
                                         2.85336484808081e-11,
                                         1.05260988801619e-06,
                                         0.000570186840848306,
                                         0.0240236755342275,
                                         0.145315694606236,
                                         0.149462628197751,
                                         0.021752233009831,
                                         0.000198948026815896,
                                         4.20515943791212e-09,
                                         1.48316482860805e-22,
                                         1.03219936942825e-12,
                                         1.00468838805539e-07,
                                         0.000112694128764032,
                                         0.00889878293272906,
                                         0.0980756897780768,
                                         0.18905515471752,
                                         0.0569744206880345,
                                         0.00137490712338343,
                                         1.59054951224349e-07,
                                         7.84250289474253e-25,
                                         2.98716945073379e-14,
                                         7.67158863761538e-09,
                                         1.78186738073291e-05,
                                         0.00263700989704118,
                                         0.0529541751233952,
                                         0.191308566997636,
                                         0.119383958833838,
                                         0.00760146105768552,
                                         4.81284534058779e-06,
                                         3.23973872815054e-27,
                                         6.75376833052176e-16,
                                         4.57645569235763e-10,
                                         2.20109858798642e-06,
                                         0.000610496160126351,
                                         0.0223372175400986,
                                         0.151241280047076,
                                         0.195434874747341,
                                         0.0328330190163343,
                                         0.000113774948994206,
                                         1.00768857734067e-29,
                                         1.14972376300674e-17,
                                         2.05557943944105e-11,
                                         2.04722068525368e-07,
                                         0.000106418016970533,
                                         0.00709445418583927,
                                         0.0900258682210744,
                                         0.240890238521009,
                                         0.106778941096223,
                                         0.002025127826901,
                                         2.22014000971658e-32,
                                         1.3863680325709e-19,
                                         6.53998794502442e-13,
                                         1.34873772936795e-08,
                                         1.3139688412833e-05,
                                         0.00159605033202723,
                                         0.0379578820702381,
                                         0.21031682560183,
                                         0.245979027940455,
                                         0.0255326512178366,
                                         3.08931388458099e-35,
                                         1.05582321977188e-21,
                                         1.31415706930366e-14,
                                         5.61200432646242e-10,
                                         1.02466668610791e-06,
                                         0.000226778470735384,
                                         0.0101079776038076,
                                         0.115972890005991,
                                         0.357880633323955,
                                         0.203313880386182,
                                         2.04191339823408e-38,
                                         3.81942064790367e-24,
                                         1.25432827740069e-16,
                                         1.10918017197615e-11,
                                         3.79554177922605e-08,
                                         1.53056110039994e-05,
                                         0.00127855725060629,
                                         0.0303761375699783,
                                         0.247327224076175,
                                         0.769009589422688,

                                         0.803396376556765,
                                         0.312698813181033,
                                         0.0550915887727068,
                                         0.00408418333643303,
                                         0.000112810482806861,
                                         9.5367431640625e-07,
                                         1.78357484368563e-09,
                                         4.17412587162325e-13,
                                         3.9974586438594e-18,
                                         1.08681547031538e-25,
                                         5.45888606189384e-40,
                                         0.176835140732706,
                                         0.374289554788569,
                                         0.171850273180149,
                                         0.0258587924458109,
                                         0.00129815499158478,
                                         1.9073486328125e-05,
                                         6.19975089392724e-08,
                                         2.63707524081535e-11,
                                         5.12600518272189e-16,
                                         3.63190373192624e-23,
                                         9.92031168463474e-37,
                                         0.0184884662881965,
                                         0.212805472346939,
                                         0.254629528731132,
                                         0.0777687039353804,
                                         0.00709573269803565,
                                         0.000181198120117188,
                                         1.02364825672127e-06,
                                         7.91358207397099e-10,
                                         3.12225027205373e-14,
                                         5.76509482260335e-21,
                                         8.56328137867842e-34,
                                         0.00122084588631728,
                                         0.0764162147189164,
                                         0.238284037801372,
                                         0.14771654023747,
                                         0.024496023574361,
                                         0.00108718872070312,
                                         1.06746810493701e-05,
                                         1.49986215044207e-08,
                                         1.20111344497991e-12,
                                         5.7797123724359e-19,
                                         4.66855798178613e-31,
                                         5.71029417596944e-05,
                                         0.0194368521026577,
                                         0.15794970347308,
                                         0.198742648415702,
                                         0.0599006610754352,
                                         0.00462055206298828,
                                         7.88491005446738e-05,
                                         2.01357244829826e-07,
                                         3.27293985295038e-11,
                                         4.10434328136129e-17,
                                         1.80286405075124e-28,
                                         2.0110236066744e-06,
                                         0.00372243726593653,
                                         0.0788322146309774,
                                         0.201332583664114,
                                         0.110288108324026,
                                         0.0147857666015625,
                                         0.000438530322015118,
                                         2.03537400157234e-06,
                                         6.71510903047662e-10,
                                         2.19453330768019e-15,
                                         5.24208729933318e-26,
                                         5.53306644266919e-08,
                                         0.000556953419818844,
                                         0.0307382168528991,
                                         0.159340835874917,
                                         0.158641128433955,
                                         0.0369644165039062,
                                         0.00190542799868526,
                                         1.60735281781835e-05,
                                         1.07636146985666e-08,
                                         9.16707335294837e-14,
                                         1.1907910171102e-23,
                                         1.21788025391186e-09,
                                         6.66653785543087e-05,
                                         0.00958834384870444,
                                         0.100885813971075,
                                         0.182554641752368,
                                         0.0739288330078125,
                                         0.00662331551713966,
                                         0.000101547256826617,
                                         1.38023553575541e-07,
                                         3.06343890299551e-12,
                                         2.16399791222207e-21,
                                         2.17804406777414e-11,
                                         6.4834354875284e-06,
                                         0.00243014338757122,
                                         0.0518987004576427,
                                         0.170684207260023,
                                         0.120134353637695,
                                         0.0187060338316403,
                                         0.000521252973799079,
                                         1.43804221662894e-06,
                                         8.31785048535853e-11,
                                         3.19522536168102e-19,
                                         3.19605399016601e-13,
                                         5.17363051353085e-07,
                                         0.000505365476321064,
                                         0.0219062598777063,
                                         0.130942060304,
                                         0.160179138183594,
                                         0.0433484285338973,
                                         0.00219540306750479,
                                         1.22934969641879e-05,
                                         1.85309791431976e-09,
                                         3.8710744440205e-17,
                                         3.86914940741234e-15,
                                         3.40596023700263e-08,
                                         8.67028132660045e-05,
                                         0.00762840840942717,
                                         0.0828741236890909,
                                         0.176197052001953,
                                         0.0828741236890911,
                                         0.00762840840942715,
                                         8.67028132660041e-05,
                                         3.40596023700263e-08,
                                         3.86914940741253e-15,
                                         3.87107444402028e-17,
                                         1.85309791431976e-09,
                                         1.22934969641879e-05,
                                         0.0021954030675048,
                                         0.0433484285338972,
                                         0.160179138183594,
                                         0.130942060304,
                                         0.0219062598777062,
                                         0.000505365476321062,
                                         5.17363051353085e-07,
                                         3.19605399016616e-13,
                                         3.19522536168082e-19,
                                         8.31785048535853e-11,
                                         1.43804221662895e-06,
                                         0.00052125297379908,
                                         0.0187060338316403,
                                         0.120134353637695,
                                         0.170684207260023,
                                         0.0518987004576426,
                                         0.00243014338757121,
                                         6.4834354875284e-06,
                                         2.17804406777423e-11,
                                         2.16399791222193e-21,
                                         3.06343890299551e-12,
                                         1.38023553575542e-07,
                                         0.000101547256826618,
                                         0.00662331551713964,
                                         0.0739288330078125,
                                         0.182554641752368,
                                         0.100885813971075,
                                         0.00958834384870441,
                                         6.66653785543087e-05,
                                         1.2178802539119e-09,
                                         1.19079101711012e-23,
                                         9.16707335294837e-14,
                                         1.07636146985667e-08,
                                         1.60735281781835e-05,
                                         0.00190542799868526,
                                         0.0369644165039062,
                                         0.158641128433955,
                                         0.159340835874917,
                                         0.030738216852899,
                                         0.000556953419818844,
                                         5.53306644266936e-08,
                                         5.24208729933278e-26,
                                         2.19453330768019e-15,
                                         6.71510903047666e-10,
                                         2.03537400157235e-06,
                                         0.000438530322015116,
                                         0.0147857666015625,
                                         0.110288108324026,
                                         0.201332583664113,
                                         0.0788322146309773,
                                         0.00372243726593653,
                                         2.01102360667445e-06,
                                         1.80286405075109e-28,
                                         4.10434328136129e-17,
                                         3.2729398529504e-11,
                                         2.01357244829827e-07,
                                         7.88491005446736e-05,
                                         0.00462055206298828,
                                         0.0599006610754353,
                                         0.198742648415701,
                                         0.157949703473079,
                                         0.0194368521026577,
                                         5.71029417596956e-05,
                                         4.66855798178573e-31,
                                         5.7797123724359e-19,
                                         1.20111344497992e-12,
                                         1.49986215044207e-08,
                                         1.06746810493701e-05,
                                         0.00108718872070312,
                                         0.024496023574361,
                                         0.14771654023747,
                                         0.238284037801372,
                                         0.0764162147189164,
                                         0.0012208458863173,
                                         8.56328137867763e-34,
                                         5.76509482260335e-21,
                                         3.12225027205376e-14,
                                         7.91358207397102e-10,
                                         1.02364825672127e-06,
                                         0.000181198120117188,
                                         0.00709573269803566,
                                         0.0777687039353804,
                                         0.254629528731132,
                                         0.212805472346939,
                                         0.0184884662881967,
                                         9.92031168463378e-37,
                                         3.63190373192624e-23,
                                         5.12600518272193e-16,
                                         2.63707524081536e-11,
                                         6.19975089392722e-08,
                                         1.9073486328125e-05,
                                         0.00129815499158478,
                                         0.0258587924458109,
                                         0.171850273180149,
                                         0.374289554788569,
                                         0.176835140732706,
                                         5.45888606189329e-40,
                                         1.08681547031538e-25,
                                         3.99745864385943e-18,
                                         4.17412587162327e-13,
                                         1.78357484368562e-09,
                                         9.5367431640625e-07,
                                         0.000112810482806861,
                                         0.00408418333643303,
                                         0.0550915887727068,
                                         0.312698813181033,
                                         0.803396376556765,

                                         0.830897411098604,
                                         0.374346575884578,
                                         0.0867736359854743,
                                         0.00983283290791891,
                                         0.000501247361168264,
                                         1.00983971592076e-05,
                                         6.5651379263318e-08,
                                         9.90962309181637e-11,
                                         1.95754421931069e-14,
                                         1.65056074881757e-19,
                                         4.11347762414993e-27,
                                         1.96933528983848e-41,
                                         0.154637925298972,
                                         0.377008050780964,
                                         0.225621237617301,
                                         0.0511281761620537,
                                         0.00463321385964487,
                                         0.000157011699006401,
                                         1.6889791169605e-06,
                                         4.28831698722905e-09,
                                         1.50587849309048e-12,
                                         2.53921411126636e-17,
                                         1.63377547019333e-24,
                                         4.23263721560753e-38,
                                         0.0136703058887231,
                                         0.180352012726319,
                                         0.27865431228939,
                                         0.126280284686578,
                                         0.0203425880270113,
                                         0.00115959194188945,
                                         2.06394592547917e-05,
                                         8.81475475502027e-08,
                                         5.50252329654508e-11,
                                         1.85549907572305e-15,
                                         3.08225959194223e-22,
                                         4.32111712942352e-35,
                                         0.000763252254465238,
                                         0.0544902759789137,
                                         0.217359789383451,
                                         0.196987400326407,
                                         0.0564102086651116,
                                         0.00540886334969068,
                                         0.000159294211597064,
                                         1.14435621427705e-06,
                                         1.26987652308274e-09,
                                         8.5634705173249e-14,
                                         3.67260058826063e-20,
                                         2.78617682870507e-32,
                                         3.0185309884129e-05,
                                         0.0116615076430617,
                                         0.120096491613445,
                                         0.217660200522171,
                                         0.110801818488866,
                                         0.0178708212108676,
                                         0.000870841975752241,
                                         1.05232505452234e-05,
                                         2.0758634303221e-08,
                                         2.79947472630057e-12,
                                         3.09967289601711e-18,
                                         1.27250352263486e-29,
                                         8.98843805772184e-07,
                                         0.00187910670972819,
                                         0.0499623071244489,
                                         0.181084237757004,
                                         0.163869118799933,
                                         0.0444573998304204,
                                         0.00358459225739977,
                                         7.28617565464682e-05,
                                         2.55503651031973e-07,
                                         6.89071588234319e-11,
                                         1.96978615404328e-16,
                                         4.3759299249344e-27,
                                         2.09104245957189e-08,
                                         0.000236558313699685,
                                         0.0162384539952907,
                                         0.117698872935629,
                                         0.189337822802216,
                                         0.0864039581075576,
                                         0.0115273691381327,
                                         0.000394129909662761,
                                         2.45688864394228e-06,
                                         1.32508020514449e-09,
                                         9.77940259450007e-15,
                                         1.17563296285413e-24,
                                         3.89162925941388e-10,
                                         2.38240161628854e-05,
                                         0.00422218113347515,
                                         0.0612003556440139,
                                         0.17501191880943,
                                         0.134342431273619,
                                         0.0296558670454004,
                                         0.00170556838652889,
                                         1.89000878362468e-05,
                                         2.03849652784546e-08,
                                         3.88414610018476e-13,
                                         2.5267550204107e-22,
                                         5.88468794907176e-12,
                                         1.94946348119311e-06,
                                         0.000891974444879281,
                                         0.0258558581998005,
                                         0.131438091230457,
                                         0.169713402071905,
                                         0.0619889700968667,
                                         0.0059968370909426,
                                         0.000118131451904671,
                                         2.54801016066624e-07,
                                         1.25343598550212e-11,
                                         4.41243062001896e-20,
                                         7.30130999183268e-14,
                                         1.30888232894268e-07,
                                         0.00015461560867816,
                                         0.00896291628425032,
                                         0.080995377153356,
                                         0.175915637457838,
                                         0.106317213895912,
                                         0.0173005828970237,
                                         0.000605832590398861,
                                         2.61323042699225e-06,
                                         3.31889973024007e-10,
                                         6.32234241673502e-18,
                                         7.4736384746326e-16,
                                         7.2500341669657e-09,
                                         2.21109908831262e-05,
                                         0.00256326596667586,
                                         0.0411768546203807,
                                         0.150434241949235,
                                         0.150434241949235,
                                         0.0411768546203808,
                                         0.00256326596667587,
                                         2.21109908831263e-05,
                                         7.25003416696579e-09,
                                         7.47363847463215e-16,
                                         6.32234241673544e-18,
                                         3.31889973024003e-10,
                                         2.61323042699224e-06,
                                         0.000605832590398859,
                                         0.0173005828970237,
                                         0.106317213895912,
                                         0.175915637457838,
                                         0.0809953771533561,
                                         0.00896291628425034,
                                         0.000154615608678161,
                                         1.30888232894269e-07,
                                         7.30130999183228e-14,
                                         4.41243062001928e-20,
                                         1.2534359855021e-11,
                                         2.54801016066622e-07,
                                         0.00011813145190467,
                                         0.00599683709094258,
                                         0.0619889700968667,
                                         0.169713402071905,
                                         0.131438091230457,
                                         0.0258558581998005,
                                         0.000891974444879285,
                                         1.94946348119313e-06,
                                         5.88468794907147e-12,
                                         2.52675502041089e-22,
                                         3.8841461001847e-13,
                                         2.03849652784545e-08,
                                         1.89000878362468e-05,
                                         0.00170556838652889,
                                         0.0296558670454004,
                                         0.134342431273619,
                                         0.17501191880943,
                                         0.0612003556440141,
                                         0.00422218113347516,
                                         2.38240161628856e-05,
                                         3.89162925941371e-10,
                                         1.17563296285423e-24,
                                         9.77940259449991e-15,
                                         1.32508020514448e-09,
                                         2.45688864394228e-06,
                                         0.00039412990966276,
                                         0.0115273691381327,
                                         0.0864039581075576,
                                         0.189337822802216,
                                         0.117698872935629,
                                         0.0162384539952908,
                                         0.000236558313699687,
                                         2.09104245957181e-08,
                                         4.3759299249348e-27,
                                         1.96978615404325e-16,
                                         6.89071588234314e-11,
                                         2.55503651031972e-07,
                                         7.2861756546468e-05,
                                         0.00358459225739977,
                                         0.0444573998304204,
                                         0.163869118799933,
                                         0.181084237757004,
                                         0.049962307124449,
                                         0.0018791067097282,
                                         8.98843805772157e-07,
                                         1.27250352263498e-29,
                                         3.09967289601705e-18,
                                         2.79947472630055e-12,
                                         2.07586343032209e-08,
                                         1.05232505452234e-05,
                                         0.000870841975752241,
                                         0.0178708212108676,
                                         0.110801818488866,
                                         0.217660200522171,
                                         0.120096491613445,
                                         0.0116615076430618,
                                         3.01853098841283e-05,
                                         2.78617682870536e-32,
                                         3.67260058826056e-20,
                                         8.56347051732483e-14,
                                         1.26987652308273e-09,
                                         1.14435621427705e-06,
                                         0.000159294211597064,
                                         0.00540886334969068,
                                         0.0564102086651117,
                                         0.196987400326408,
                                         0.217359789383452,
                                         0.0544902759789138,
                                         0.000763252254465225,
                                         4.32111712942399e-35,
                                         3.08225959194217e-22,
                                         1.85549907572303e-15,
                                         5.50252329654505e-11,
                                         8.81475475502024e-08,
                                         2.06394592547917e-05,
                                         0.00115959194188945,
                                         0.0203425880270113,
                                         0.126280284686578,
                                         0.27865431228939,
                                         0.18035201272632,
                                         0.0136703058887229,
                                         4.23263721560801e-38,
                                         1.6337754701933e-24,
                                         2.53921411126633e-17,
                                         1.50587849309048e-12,
                                         4.28831698722904e-09,
                                         1.6889791169605e-06,
                                         0.000157011699006401,
                                         0.00463321385964487,
                                         0.0511281761620538,
                                         0.225621237617301,
                                         0.377008050780965,
                                         0.154637925298972,
                                         1.96933528983871e-41,
                                         4.11347762414984e-27,
                                         1.65056074881755e-19,
                                         1.95754421931068e-14,
                                         9.90962309181633e-11,
                                         6.5651379263318e-08,
                                         1.00983971592076e-05,
                                         0.000501247361168264,
                                         0.00983283290791891,
                                         0.0867736359854743,
                                         0.374346575884578,
                                         0.830897411098604,

                                         0.853168502978977,
                                         0.431075416427547,
                                         0.123726269997882,
                                         0.0194406548431215,
                                         0.00157661846712303,
                                         6.03584671431046e-05,
                                         9.5367431640625e-07,
                                         5.05874833621155e-09,
                                         6.46285763896083e-12,
                                         1.11835765986973e-15,
                                         8.5347847184054e-21,
                                         1.9866120206334e-28,
                                         9.15909008285057e-43,
                                         0.136020912943,
                                         0.370476992300835,
                                         0.272538868514248,
                                         0.084670939116085,
                                         0.0120058110452536,
                                         0.00075497651321274,
                                         1.9073486328125e-05,
                                         1.61773665746034e-07,
                                         3.39484293583018e-10,
                                         1.02711062301461e-13,
                                         1.54983703308069e-18,
                                         9.24623792431478e-26,
                                         2.29795463228801e-39,
                                         0.0103007812985262,
                                         0.151238665862281,
                                         0.285159987081495,
                                         0.175166669775775,
                                         0.0434260180148362,
                                         0.00448561804463902,
                                         0.000181198120117188,
                                         2.45734530857124e-06,
                                         8.47048723849715e-09,
                                         4.48071514273318e-12,
                                         1.33682053088697e-16,
                                         2.0441402025618e-23,
                                         2.73857210272678e-36,
                                         0.000492677005079114,
                                         0.0389934873896941,
                                         0.188441420466466,
                                         0.228873871041433,
                                         0.0992055930345544,
                                         0.0168321186632351,
                                         0.00108718872070312,
                                         2.35750267942227e-05,
                                         1.33482626592138e-07,
                                         1.23453979534769e-10,
                                         7.28262293792796e-15,
                                         2.85419696456642e-21,
                                         2.06126844229829e-33,
                                         1.66913743930591e-05,
                                         0.00712129535250363,
                                         0.0882067522697955,
                                         0.2118256932843,
                                         0.160531397580574,
                                         0.0447396886918519,
                                         0.00462055206298828,
                                         0.00016020493180462,
                                         1.4899742910512e-06,
                                         2.40935354641166e-09,
                                         2.81021936540467e-13,
                                         2.82289600984852e-19,
                                         1.09896183030829e-30,
                                         4.25777740324904e-07,
                                         0.000979235088039362,
                                         0.0310876821348781,
                                         0.14761215009704,
                                         0.195589197145432,
                                         0.0895381629641839,
                                         0.0147857666015625,
                                         0.000819710722610428,
                                         1.2522581134514e-05,
                                         3.54043821352193e-08,
                                         8.16495260238675e-12,
                                         2.10216532440683e-17,
                                         4.41155513191096e-28,
                                         8.48523426901934e-09,
                                         0.00010519739016283,
                                         0.00855982496069276,
                                         0.0803629010604228,
                                         0.186174318041087,
                                         0.139995292445677,
                                         0.0369644165039062,
                                         0.0032766901917908,
                                         8.22240688209338e-05,
                                         4.0644663931336e-07,
                                         1.85334872730273e-10,
                                         1.22300684178841e-15,
                                         1.3835372099285e-25,
                                         1.35280347056563e-10,
                                         9.04092676599508e-06,
                                         0.00188552116661009,
                                         0.0350008904421578,
                                         0.141770106749981,
                                         0.175109082055125,
                                         0.0739288330078125,
                                         0.0104785245007275,
                                         0.000431910796718161,
                                         3.73284572464919e-06,
                                         3.36550784531492e-09,
                                         5.69220971422241e-14,
                                         3.47120261045466e-23,
                                         1.75238267113402e-12,
                                         6.31312381300745e-07,
                                         0.000337459229005736,
                                         0.012385853813236,
                                         0.0877148107523384,
                                         0.177962001021896,
                                         0.120134353637695,
                                         0.0272262767593311,
                                         0.00184337004887802,
                                         2.78547917074724e-05,
                                         4.96555092311496e-08,
                                         2.15256496430856e-12,
                                         7.07607542069871e-21,
                                         1.86255266824679e-14,
                                         3.61710431343805e-08,
                                         4.95560382040602e-05,
                                         0.00359631875429441,
                                         0.0445293715938093,
                                         0.148399097785043,
                                         0.160179138183594,
                                         0.0580445899953126,
                                         0.00645529902111095,
                                         0.000170547422595479,
                                         6.01131718741185e-07,
                                         6.67908566639176e-11,
                                         1.18356009489487e-18,
                                         1.63321034942233e-16,
                                         1.70974644272802e-09,
                                         6.00379419764939e-06,
                                         0.000861478323694686,
                                         0.0186497987849962,
                                         0.102091407041899,
                                         0.176197052001953,
                                         0.102091407041899,
                                         0.0186497987849962,
                                         0.000861478323694684,
                                         6.00379419764943e-06,
                                         1.709746442728e-09,
                                         1.63321034942222e-16,
                                         1.18356009489496e-18,
                                         6.67908566639186e-11,
                                         6.01131718741181e-07,
                                         0.00017054742259548,
                                         0.00645529902111095,
                                         0.0580445899953126,
                                         0.160179138183594,
                                         0.148399097785043,
                                         0.0445293715938093,
                                         0.0035963187542944,
                                         4.95560382040604e-05,
                                         3.61710431343801e-08,
                                         1.86255266824668e-14,
                                         7.0760754206993e-21,
                                         2.1525649643086e-12,
                                         4.96555092311492e-08,
                                         2.78547917074725e-05,
                                         0.00184337004887802,
                                         0.0272262767593311,
                                         0.120134353637695,
                                         0.177962001021896,
                                         0.0877148107523384,
                                         0.012385853813236,
                                         0.000337459229005737,
                                         6.31312381300738e-07,
                                         1.75238267113392e-12,
                                         3.47120261045498e-23,
                                         5.69220971422251e-14,
                                         3.3655078453149e-09,
                                         3.73284572464921e-06,
                                         0.000431910796718161,
                                         0.0104785245007275,
                                         0.0739288330078125,
                                         0.175109082055125,
                                         0.141770106749981,
                                         0.0350008904421578,
                                         0.0018855211666101,
                                         9.04092676599499e-06,
                                         1.35280347056556e-10,
                                         1.38353720992864e-25,
                                         1.22300684178843e-15,
                                         1.85334872730271e-10,
                                         4.06446639313362e-07,
                                         8.22240688209338e-05,
                                         0.0032766901917908,
                                         0.0369644165039062,
                                         0.139995292445677,
                                         0.186174318041087,
                                         0.0803629010604227,
                                         0.00855982496069278,
                                         0.00010519739016283,
                                         8.48523426901898e-09,
                                         4.41155513191143e-28,
                                         2.10216532440687e-17,
                                         8.16495260238668e-12,
                                         3.54043821352195e-08,
                                         1.2522581134514e-05,
                                         0.000819710722610428,
                                         0.0147857666015625,
                                         0.0895381629641839,
                                         0.195589197145432,
                                         0.147612150097039,
                                         0.0310876821348781,
                                         0.000979235088039355,
                                         4.2577774032489e-07,
                                         1.09896183030841e-30,
                                         2.82289600984858e-19,
                                         2.81021936540465e-13,
                                         2.40935354641168e-09,
                                         1.4899742910512e-06,
                                         0.00016020493180462,
                                         0.00462055206298828,
                                         0.0447396886918519,
                                         0.160531397580574,
                                         0.2118256932843,
                                         0.0882067522697957,
                                         0.0071212953525036,
                                         1.66913743930587e-05,
                                         2.06126844229854e-33,
                                         2.85419696456649e-21,
                                         7.28262293792789e-15,
                                         1.2345397953477e-10,
                                         1.33482626592138e-07,
                                         2.35750267942227e-05,
                                         0.00108718872070312,
                                         0.0168321186632351,
                                         0.0992055930345544,
                                         0.228873871041432,
                                         0.188441420466466,
                                         0.0389934873896939,
                                         0.000492677005079103,
                                         2.73857210272713e-36,
                                         2.04414020256185e-23,
                                         1.33682053088695e-16,
                                         4.4807151427332e-12,
                                         8.47048723849715e-09,
                                         2.45734530857124e-06,
                                         0.000181198120117188,
                                         0.00448561804463902,
                                         0.0434260180148362,
                                         0.175166669775775,
                                         0.285159987081495,
                                         0.151238665862281,
                                         0.010300781298526,
                                         2.29795463228831e-39,
                                         9.24623792431501e-26,
                                         1.54983703308067e-18,
                                         1.02711062301462e-13,
                                         3.39484293583018e-10,
                                         1.61773665746034e-07,
                                         1.9073486328125e-05,
                                         0.00075497651321274,
                                         0.0120058110452536,
                                         0.0846709391160849,
                                         0.272538868514248,
                                         0.370476992300834,
                                         0.136020912942999,
                                         9.15909008285186e-43,
                                         1.98661202063345e-28,
                                         8.53478471840531e-21,
                                         1.11835765986974e-15,
                                         6.46285763896083e-12,
                                         5.05874833621155e-09,
                                         9.5367431640625e-07,
                                         6.03584671431046e-05,
                                         0.00157661846712303,
                                         0.0194406548431215,
                                         0.123726269997882,
                                         0.431075416427547,
                                         0.853168502978977,

                                         0.871417215535353,
                                         0.482502607142086,
                                         0.164107363930271,
                                         0.0333587441354204,
                                         0.0038822019512058,
                                         0.000242681091123396,
                                         7.42379619256405e-06,
                                         9.68639315287471e-08,
                                         4.37430423181062e-10,
                                         4.8953176279055e-13,
                                         7.62350681521403e-17,
                                         5.37289986407352e-22,
                                         1.18420725877564e-29,
                                         5.29749750356151e-44,
                                         0.120350628453406,
                                         0.35811792770677,
                                         0.31039312899614,
                                         0.123647956415508,
                                         0.0248395232917435,
                                         0.00250529912588571,
                                         0.000119517868378054,
                                         2.4066630231566e-06,
                                         1.69490487249691e-08,
                                         3.0603826689611e-11,
                                         8.22692491281523e-15,
                                         1.13627828838361e-19,
                                         6.38206630329529e-27,
                                         1.53429378248615e-40,
                                         0.00789521932511964,
                                         0.126254268717209,
                                         0.278862257029804,
                                         0.217699686332357,
                                         0.0754921857312133,
                                         0.0122850476257824,
                                         0.00091397274834569,
                                         2.84028609764037e-05,
                                         3.11943026347698e-07,
                                         9.0879138523444e-10,
                                         4.2170998468006e-13,
                                         1.1414431333636e-17,
                                         1.63376096109564e-24,
                                         2.11076507249261e-37,
                                         0.000327120439193484,
                                         0.0281121281467974,
                                         0.15823225682033,
                                         0.242078549642203,
                                         0.144906673793671,
                                         0.0380471164059761,
                                         0.00441429445816372,
                                         0.000211707642416357,
                                         3.6260424100358e-06,
                                         1.70443449218796e-08,
                                         1.36526789955388e-11,
                                         7.24188287150925e-16,
                                         2.64145588540678e-22,
                                         1.83399825568794e-34,
                                         9.600389249482e-06,
                                         0.0044338301945872,
                                         0.0635971986682662,
                                         0.190674446246465,
                                         0.197020971117476,
                                         0.0834649868868568,
                                         0.015101741507837,
                                         0.00111776025575142,
                                         2.98557617938608e-05,
                                         2.26430073094556e-07,
                                         3.13083048674125e-10,
                                         3.25451586190702e-14,
                                         3.02507320868529e-20,
                                         1.12874442180745e-31,
                                         2.12144145692629e-07,
                                         0.000526532808763602,
                                         0.0192460672255218,
                                         0.113081022586821,
                                         0.201696132763601,
                                         0.137863074685704,
                                         0.0389003772624239,
                                         0.00444346577111237,
                                         0.000185090651056345,
                                         2.26489956028177e-06,
                                         5.40582867249917e-09,
                                         1.10124091098361e-12,
                                         2.60849173447731e-18,
                                         5.23062233737585e-29,
                                         3.66237905357619e-09,
                                         4.88496941656212e-05,
                                         0.0045502581998225,
                                         0.0523934492837738,
                                         0.161314244164963,
                                         0.177902323839474,
                                         0.0782835703073973,
                                         0.0138001893445039,
                                         0.000896459841680847,
                                         1.76992074061537e-05,
                                         7.29214055587908e-08,
                                         2.91117499182686e-11,
                                         1.75724805325083e-16,
                                         1.89365623234686e-26,
                                         5.05807795479115e-11,
                                         3.62566978597697e-06,
                                         0.000860637113751558,
                                         0.0194202242962123,
                                         0.103213819775167,
                                         0.183656062507749,
                                         0.126031011755619,
                                         0.0342876908709002,
                                         0.0034734990369743,
                                         0.000110649301469977,
                                         7.86933025195015e-07,
                                         6.15664728653675e-10,
                                         9.47036382700316e-15,
                                         5.48452355381503e-24,
                                         5.67585268503453e-13,
                                         2.18644227306864e-07,
                                         0.000132259680625673,
                                         0.00584863961921849,
                                         0.0536569958609511,
                                         0.15404666005154,
                                         0.16485707521631,
                                         0.0692173233407761,
                                         0.0109352041626393,
                                         0.00056203958478524,
                                         6.89991724494659e-06,
                                         1.05789753476706e-08,
                                         4.14690060777344e-13,
                                         1.29062490092231e-21,
                                         5.22591188602429e-15,
                                         1.08186520900496e-08,
                                         1.66770887571204e-05,
                                         0.00144524292999243,
                                         0.0228875985880317,
                                         0.106019237286875,
                                         0.17693882128777,
                                         0.114650706420192,
                                         0.028246977248363,
                                         0.00234245093684593,
                                         4.96404104259297e-05,
                                         1.49151734383152e-07,
                                         1.48993032741858e-11,
                                         2.49199143517057e-19,
                                         3.96960230624223e-17,
                                         4.41633737382169e-10,
                                         1.73487008799777e-06,
                                         0.000294632596930646,
                                         0.00805430461760499,
                                         0.06019646841861,
                                         0.156672443757978,
                                         0.156672443757978,
                                         0.06019646841861,
                                         0.008054304617605,
                                         0.000294632596930647,
                                         1.73487008799777e-06,
                                         4.41633737382162e-10,
                                         3.96960230624223e-17,
                                         2.49199143517057e-19,
                                         1.4899303274186e-11,
                                         1.49151734383152e-07,
                                         4.96404104259295e-05,
                                         0.00234245093684592,
                                         0.028246977248363,
                                         0.114650706420192,
                                         0.17693882128777,
                                         0.106019237286875,
                                         0.0228875985880318,
                                         0.00144524292999243,
                                         1.66770887571204e-05,
                                         1.08186520900495e-08,
                                         5.22591188602429e-15,
                                         1.29062490092231e-21,
                                         4.14690060777352e-13,
                                         1.05789753476706e-08,
                                         6.89991724494656e-06,
                                         0.000562039584785239,
                                         0.0109352041626393,
                                         0.069217323340776,
                                         0.16485707521631,
                                         0.15404666005154,
                                         0.0536569958609512,
                                         0.0058486396192185,
                                         0.000132259680625673,
                                         2.18644227306861e-07,
                                         5.67585268503453e-13,
                                         5.48452355381503e-24,
                                         9.47036382700335e-15,
                                         6.15664728653675e-10,
                                         7.86933025195011e-07,
                                         0.000110649301469976,
                                         0.0034734990369743,
                                         0.0342876908709001,
                                         0.126031011755619,
                                         0.183656062507749,
                                         0.103213819775168,
                                         0.0194202242962123,
                                         0.000860637113751558,
                                         3.62566978597693e-06,
                                         5.05807795479115e-11,
                                         1.89365623234686e-26,
                                         1.75724805325087e-16,
                                         2.91117499182686e-11,
                                         7.29214055587905e-08,
                                         1.76992074061536e-05,
                                         0.000896459841680847,
                                         0.0138001893445039,
                                         0.0782835703073974,
                                         0.177902323839474,
                                         0.161314244164963,
                                         0.0523934492837739,
                                         0.0045502581998225,
                                         4.88496941656208e-05,
                                         3.66237905357619e-09,
                                         5.23062233737585e-29,
                                         2.60849173447737e-18,
                                         1.10124091098361e-12,
                                         5.40582867249914e-09,
                                         2.26489956028177e-06,
                                         0.000185090651056345,
                                         0.00444346577111236,
                                         0.0389003772624239,
                                         0.137863074685704,
                                         0.201696132763601,
                                         0.113081022586821,
                                         0.0192460672255218,
                                         0.000526532808763598,
                                         2.12144145692629e-07,
                                         1.12874442180745e-31,
                                         3.02507320868537e-20,
                                         3.25451586190702e-14,
                                         3.13083048674123e-10,
                                         2.26430073094555e-07,
                                         2.98557617938608e-05,
                                         0.00111776025575142,
                                         0.015101741507837,
                                         0.0834649868868568,
                                         0.197020971117476,
                                         0.190674446246466,
                                         0.0635971986682663,
                                         0.00443383019458717,
                                         9.600389249482e-06,
                                         1.83399825568794e-34,
                                         2.64145588540685e-22,
                                         7.24188287150925e-16,
                                         1.36526789955387e-11,
                                         1.70443449218796e-08,
                                         3.6260424100358e-06,
                                         0.000211707642416357,
                                         0.00441429445816373,
                                         0.0380471164059761,
                                         0.144906673793671,
                                         0.242078549642204,
                                         0.15823225682033,
                                         0.0281121281467973,
                                         0.000327120439193484,
                                         2.11076507249261e-37,
                                         1.63376096109568e-24,
                                         1.1414431333636e-17,
                                         4.21709984680057e-13,
                                         9.08791385234436e-10,
                                         3.11943026347698e-07,
                                         2.84028609764037e-05,
                                         0.000913972748345691,
                                         0.0122850476257824,
                                         0.0754921857312134,
                                         0.217699686332358,
                                         0.278862257029804,
                                         0.126254268717209,
                                         0.00789521932511964,
                                         1.53429378248615e-40,
                                         6.38206630329548e-27,
                                         1.13627828838361e-19,
                                         8.22692491281517e-15,
                                         3.06038266896109e-11,
                                         1.69490487249691e-08,
                                         2.40666302315659e-06,
                                         0.000119517868378054,
                                         0.00250529912588571,
                                         0.0248395232917435,
                                         0.123647956415508,
                                         0.31039312899614,
                                         0.358117927706769,
                                         0.120350628453406,
                                         5.29749750356151e-44,
                                         1.18420725877567e-29,
                                         5.37289986407352e-22,
                                         7.62350681521398e-17,
                                         4.89531762790548e-13,
                                         4.37430423181062e-10,
                                         9.68639315287468e-08,
                                         7.42379619256405e-06,
                                         0.000242681091123396,
                                         0.0038822019512058,
                                         0.0333587441354204,
                                         0.164107363930271,
                                         0.482502607142086,
                                         0.871417215535353,

                                         0.886533187095549,
                                         0.528711016862131,
                                         0.206257782618941,
                                         0.0515524943895041,
                                         0.00799410653201917,
                                         0.000733835201878105,
                                         3.72961369410914e-05,
                                         9.5367431640625e-07,
                                         1.06714990583657e-08,
                                         4.2333387172268e-11,
                                         4.25705566065059e-14,
                                         6.08731122965434e-18,
                                         4.02215508161445e-23,
                                         8.48078585353752e-31,
                                         3.7020216770189e-45,
                                         0.107093272955488,
                                         0.342380673974331,
                                         0.338799502208741,
                                         0.164773918749903,
                                         0.0436633342992091,
                                         0.0063779742343882,
                                         0.000496045969731314,
                                         1.9073486328125e-05,
                                         3.20942585593925e-07,
                                         1.94831327817211e-09,
                                         3.11761408148736e-12,
                                         7.61810073814501e-16,
                                         9.79459276740222e-21,
                                         5.2384789834837e-28,
                                         1.22583426034181e-41,
                                         0.00614501567188842,
                                         0.105315849361808,
                                         0.264343595129952,
                                         0.250161727290164,
                                         0.113281103820917,
                                         0.0263305899396012,
                                         0.00313381683807579,
                                         0.000181198120117188,
                                         4.58482615938657e-06,
                                         4.2592013529796e-08,
                                         1.08449858529953e-10,
                                         4.52857459013394e-14,
                                         1.13294170980478e-18,
                                         1.53697896678444e-25,
                                         1.92804942364074e-38,
                                         0.000222695500939336,
                                         0.0204600114247729,
                                         0.13026341693136,
                                         0.23987274688635,
                                         0.185620394953635,
                                         0.0686540344948917,
                                         0.0125041144196164,
                                         0.00108718872070312,
                                         4.13662397581903e-05,
                                         5.8806481867259e-07,
                                         2.38266656374859e-09,
                                         1.70021555295784e-12,
                                         8.27668435322103e-17,
                                         2.84812002847532e-23,
                                         1.91527974063699e-35,
                                         5.71659692297064e-06,
                                         0.00281550101854531,
                                         0.0454688342109818,
                                         0.162921586180039,
                                         0.215442605323585,
                                         0.126797069927411,
                                         0.0353402769085764,
                                         0.00462055206298828,
                                         0.000264366788839325,
                                         5.75121898754418e-06,
                                         3.70796162263005e-08,
                                         4.52151653652587e-11,
                                         4.28294896643144e-15,
                                         3.73840484652022e-21,
                                         1.34767119812404e-32,
                                         1.10490451310261e-07,
                                         0.000291720234475486,
                                         0.0119499536559278,
                                         0.083317724262982,
                                         0.188277550935469,
                                         0.176324808169075,
                                         0.0752052233500834,
                                         0.0147857666015625,
                                         0.00127212209330598,
                                         4.23502188417306e-05,
                                         4.34478448573895e-07,
                                         9.0536835499107e-10,
                                         1.66874683321953e-13,
                                         3.69466802882774e-19,
                                         7.13997559373191e-30,
                                         1.66840681113012e-09,
                                         2.36138853045954e-05,
                                         0.00245362762718491,
                                         0.0332878847345612,
                                         0.128545097794927,
                                         0.191561245717639,
                                         0.125030536019304,
                                         0.0369644165039062,
                                         0.00478234303802798,
                                         0.000243635990450849,
                                         3.97732681151316e-06,
                                         1.41630415162834e-08,
                                         5.0795827521638e-12,
                                         2.85268976752463e-17,
                                         2.9552861464634e-27,
                                         2.01543663143087e-11,
                                         1.52917902367981e-06,
                                         0.000403033431340459,
                                         0.0106395922827072,
                                         0.0702105676858766,
                                         0.166491425645366,
                                         0.166293076367889,
                                         0.0739288330078125,
                                         0.0143827735112677,
                                         0.00112128810601519,
                                         2.9127573287016e-05,
                                         1.77246526354631e-07,
                                         1.23695987539582e-10,
                                         1.76207201215244e-15,
                                         9.78570987289726e-25,
                                         1.97815223506946e-13,
                                         8.04586208810334e-08,
                                         5.37894199164195e-05,
                                         0.00276303980972326,
                                         0.0311582329331052,
                                         0.117570796752421,
                                         0.179703117768343,
                                         0.120134353637695,
                                         0.0351453544712473,
                                         0.00419291788200576,
                                         0.00017331681483826,
                                         1.80228018065815e-06,
                                         2.44740900777443e-09,
                                         8.84333610940786e-14,
                                         2.63274321973123e-22,
                                         1.59307288466626e-15,
                                         3.47353922599483e-09,
                                         5.89030822187145e-06,
                                         0.000588755083533884,
                                         0.0113456276427444,
                                         0.0681227892842338,
                                         0.159339124867784,
                                         0.160179138183594,
                                         0.0704658322379723,
                                         0.0128647357392789,
                                         0.000846179434129863,
                                         1.50366902086759e-05,
                                         3.97322232816553e-08,
                                         3.64161459822674e-12,
                                         5.81179171633588e-20,
                                         1.05843825665422e-17,
                                         1.23715974291941e-10,
                                         5.32148851523414e-07,
                                         0.000103498998245671,
                                         0.00340830287710592,
                                         0.0325641188300365,
                                         0.116558296532993,
                                         0.176197052001953,
                                         0.116558296532993,
                                         0.0325641188300365,
                                         0.00340830287710592,
                                         0.000103498998245671,
                                         5.32148851523418e-07,
                                         1.23715974291941e-10,
                                         1.05843825665432e-17,
                                         5.81179171633529e-20,
                                         3.64161459822674e-12,
                                         3.97322232816549e-08,
                                         1.5036690208676e-05,
                                         0.000846179434129862,
                                         0.0128647357392789,
                                         0.0704658322379723,
                                         0.160179138183594,
                                         0.159339124867784,
                                         0.0681227892842338,
                                         0.0113456276427444,
                                         0.000588755083533882,
                                         5.89030822187149e-06,
                                         3.47353922599483e-09,
                                         1.59307288466639e-15,
                                         2.63274321973094e-22,
                                         8.84333610940786e-14,
                                         2.4474090077744e-09,
                                         1.80228018065816e-06,
                                         0.00017331681483826,
                                         0.00419291788200576,
                                         0.0351453544712473,
                                         0.120134353637695,
                                         0.179703117768343,
                                         0.117570796752421,
                                         0.0311582329331052,
                                         0.00276303980972325,
                                         5.37894199164198e-05,
                                         8.04586208810334e-08,
                                         1.9781522350696e-13,
                                         9.78570987289608e-25,
                                         1.76207201215244e-15,
                                         1.23695987539581e-10,
                                         1.77246526354632e-07,
                                         2.9127573287016e-05,
                                         0.00112128810601519,
                                         0.0143827735112677,
                                         0.0739288330078125,
                                         0.166293076367889,
                                         0.166491425645366,
                                         0.0702105676858766,
                                         0.0106395922827072,
                                         0.000403033431340461,
                                         1.52917902367981e-06,
                                         2.015436631431e-11,
                                         2.95528614646301e-27,
                                         2.85268976752463e-17,
                                         5.07958275216375e-12,
                                         1.41630415162835e-08,
                                         3.97732681151316e-06,
                                         0.000243635990450849,
                                         0.00478234303802798,
                                         0.0369644165039062,
                                         0.125030536019304,
                                         0.191561245717639,
                                         0.128545097794927,
                                         0.0332878847345612,
                                         0.00245362762718492,
                                         2.36138853045954e-05,
                                         1.66840681113021e-09,
                                         7.13997559373092e-30,
                                         3.69466802882774e-19,
                                         1.66874683321951e-13,
                                         9.05368354991076e-10,
                                         4.34478448573895e-07,
                                         4.23502188417306e-05,
                                         0.00127212209330598,
                                         0.0147857666015625,
                                         0.0752052233500834,
                                         0.176324808169075,
                                         0.188277550935469,
                                         0.0833177242629818,
                                         0.0119499536559279,
                                         0.000291720234475486,
                                         1.10490451310266e-07,
                                         1.34767119812384e-32,
                                         3.73840484652022e-21,
                                         4.28294896643139e-15,
                                         4.5215165365259e-11,
                                         3.70796162263005e-08,
                                         5.75121898754418e-06,
                                         0.000264366788839325,
                                         0.00462055206298828,
                                         0.0353402769085764,
                                         0.126797069927411,
                                         0.215442605323585,
                                         0.162921586180039,
                                         0.0454688342109819,
                                         0.00281550101854531,
                                         5.71659692297085e-06,
                                         1.91527974063669e-35,
                                         2.84812002847532e-23,
                                         8.27668435322092e-17,
                                         1.70021555295785e-12,
                                         2.38266656374859e-09,
                                         5.8806481867259e-07,
                                         4.13662397581903e-05,
                                         0.00108718872070312,
                                         0.0125041144196164,
                                         0.0686540344948917,
                                         0.185620394953635,
                                         0.23987274688635,
                                         0.130263416931361,
                                         0.0204600114247729,
                                         0.000222695500939342,
                                         1.92804942364042e-38,
                                         1.53697896678444e-25,
                                         1.13294170980477e-18,
                                         4.52857459013397e-14,
                                         1.08449858529953e-10,
                                         4.2592013529796e-08,
                                         4.58482615938657e-06,
                                         0.000181198120117188,
                                         0.00313381683807579,
                                         0.0263305899396012,
                                         0.113281103820917,
                                         0.250161727290164,
                                         0.264343595129953,
                                         0.105315849361808,
                                         0.00614501567188854,
                                         1.22583426034159e-41,
                                         5.2384789834837e-28,
                                         9.79459276740209e-21,
                                         7.61810073814507e-16,
                                         3.11761408148736e-12,
                                         1.94831327817211e-09,
                                         3.20942585593925e-07,
                                         1.9073486328125e-05,
                                         0.000496045969731313,
                                         0.0063779742343882,
                                         0.0436633342992091,
                                         0.164773918749903,
                                         0.338799502208741,
                                         0.342380673974331,
                                         0.107093272955489,
                                         3.70202167701822e-45,
                                         8.48078585353752e-31,
                                         4.02215508161439e-23,
                                         6.08731122965439e-18,
                                         4.25705566065059e-14,
                                         4.2333387172268e-11,
                                         1.06714990583657e-08,
                                         9.5367431640625e-07,
                                         3.72961369410914e-05,
                                         0.000733835201878105,
                                         0.00799410653201917,
                                         0.0515524943895041,
                                         0.206257782618941,
                                         0.528711016862131,
                                         0.886533187095549,

                                         0.89917959117933,
                                         0.570025324631774,
                                         0.248837608973203,
                                         0.0736111800795938,
                                         0.0143981286972948,
                                         0.00179742392020267,
                                         0.000136293180575469,
                                         5.85843615962043e-06,
                                         1.29494583976827e-07,
                                         1.27844965039092e-09,
                                         4.56159727845212e-12,
                                         4.20223983642141e-15,
                                         5.60238042686241e-19,
                                         3.51049780164589e-24,
                                         7.13691470689942e-32,
                                         3.05317152997342e-46,
                                         0.0958123902875441,
                                         0.324941337726923,
                                         0.35844180002587,
                                         0.205137574652979,
                                         0.068013447016348,
                                         0.0133629958600716,
                                         0.00152796919014283,
                                         9.68356322642825e-05,
                                         3.13370496172065e-06,
                                         4.56145242146111e-08,
                                         2.45427721402506e-10,
                                         3.55837809350563e-13,
                                         8.04139047024989e-17,
                                         9.74823672020621e-22,
                                         5.00794654337087e-29,
                                         1.14613549244844e-42,
                                         0.00484942802946844,
                                         0.0879851516951207,
                                         0.245253316629161,
                                         0.271544032201052,
                                         0.152607940194303,
                                         0.0471900850800262,
                                         0.00813670700309291,
                                         0.000760295277571084,
                                         3.60212032088788e-05,
                                         7.7306547729692e-07,
                                         6.27225778802227e-09,
                                         1.43125480598074e-11,
                                         5.48255901774583e-15,
                                         1.28581070686317e-19,
                                         1.66917730774928e-26,
                                         2.04368674744648e-39,
                                         0.000155019740969005,
                                         0.0150467066934547,
                                         0.105983625993213,
                                         0.22701939073786,
                                         0.216265438507214,
                                         0.105250782156562,
                                         0.0273659420613369,
                                         0.00377013618848997,
                                         0.000261508596937949,
                                         8.27479140602627e-06,
                                         1.01239928230959e-07,
                                         3.63587939723375e-10,
                                         2.36082135514179e-13,
                                         1.07116322437243e-17,
                                         3.51376655259805e-24,
                                         2.30154961196888e-36,
                                         3.51011362447588e-06,
                                         0.00182268325264193,
                                         0.0324414560527824,
                                         0.134438396756055,
                                         0.217087486067201,
                                         0.166279068595921,
                                         0.0651943272758474,
                                         0.0132424842077972,
                                         0.00134478050780516,
                                         6.27386991577615e-05,
                                         1.15749050586632e-06,
                                         6.54243866733244e-09,
                                         7.20079777059917e-12,
                                         6.32079855699254e-16,
                                         5.23939349717943e-22,
                                         1.83596345735094e-33,
                                         5.98434181275514e-08,
                                         0.000166242476261495,
                                         0.00747692373415899,
                                         0.0599438653116601,
                                         0.164075413320264,
                                         0.197792984084762,
                                         0.116941931244373,
                                         0.0350221607561538,
                                         0.00520688383444776,
                                         0.00035815829412443,
                                         9.96423805290242e-06,
                                         8.86402350676504e-08,
                                         1.6537092352544e-10,
                                         2.80833746440916e-14,
                                         5.88234073262926e-20,
                                         1.10272894170558e-30,
                                         7.97079497525278e-10,
                                         1.18457571727935e-05,
                                         0.00134628061900839,
                                         0.0208812355882331,
                                         0.0968818124316377,
                                         0.183812148998903,
                                         0.163878034123524,
                                         0.0723613134051636,
                                         0.0157505020731523,
                                         0.00159736640537692,
                                         6.70132656270408e-05,
                                         9.38236163766064e-07,
                                         2.96706950254057e-09,
                                         9.74802575736155e-13,
                                         5.15952080638259e-18,
                                         5.17444224076182e-28,
                                         8.49330797276296e-12,
                                         6.75264065610123e-07,
                                         0.000193926975270558,
                                         0.0058191242413122,
                                         0.0457647389823997,
                                         0.13665563023253,
                                         0.183722021912364,
                                         0.119607918293282,
                                         0.0381154369398656,
                                         0.0056993334509022,
                                         0.000360551624411843,
                                         7.94480834421636e-06,
                                         4.25879047911446e-08,
                                         2.70691132730164e-11,
                                         3.62041658740786e-16,
                                         1.94244307846443e-25,
                                         7.35318471270839e-14,
                                         3.12757564348151e-08,
                                         2.26967787067277e-05,
                                         0.00131759650999118,
                                         0.0175647953549496,
                                         0.0825475553055995,
                                         0.167349745692437,
                                         0.160633747844787,
                                         0.0749429480200193,
                                         0.0165221650844333,
                                         0.0015761498343325,
                                         5.46610602180756e-05,
                                         4.96670309464978e-07,
                                         6.10737743924876e-10,
                                         2.06410170945678e-14,
                                         5.92456444913683e-23,
                                         5.22347415697301e-16,
                                         1.18857714972754e-09,
                                         2.17959395232318e-06,
                                         0.000244789403039561,
                                         0.00553147011135001,
                                         0.0409134661151936,
                                         0.125076081975939,
                                         0.177010498043208,
                                         0.120905488010389,
                                         0.0393002414599713,
                                         0.00565344167574018,
                                         0.000308573089303458,
                                         4.75264630279611e-06,
                                         1.13063093197267e-08,
                                         9.65581668975908e-13,
                                         1.48268852585845e-20,
                                         3.06123995942366e-18,
                                         3.72649789025575e-11,
                                         1.72679552158932e-07,
                                         3.75194738248922e-05,
                                         0.00143711656207328,
                                         0.0167294737593831,
                                         0.0771218481965461,
                                         0.160921919529752,
                                         0.160921919529752,
                                         0.077121848196546,
                                         0.0167294737593831,
                                         0.00143711656207328,
                                         3.7519473824892e-05,
                                         1.72679552158933e-07,
                                         3.72649789025575e-11,
                                         3.06123995942333e-18,
                                         1.48268852585862e-20,
                                         9.65581668975908e-13,
                                         1.13063093197266e-08,
                                         4.75264630279613e-06,
                                         0.000308573089303458,
                                         0.00565344167574018,
                                         0.0393002414599714,
                                         0.120905488010389,
                                         0.177010498043208,
                                         0.125076081975939,
                                         0.0409134661151936,
                                         0.00553147011135001,
                                         0.00024478940303956,
                                         2.1795939523232e-06,
                                         1.18857714972754e-09,
                                         5.22347415697252e-16,
                                         5.92456444913757e-23,
                                         2.06410170945678e-14,
                                         6.1073774392487e-10,
                                         4.96670309464981e-07,
                                         5.46610602180756e-05,
                                         0.0015761498343325,
                                         0.0165221650844334,
                                         0.0749429480200193,
                                         0.160633747844787,
                                         0.167349745692437,
                                         0.0825475553055995,
                                         0.0175647953549496,
                                         0.00131759650999118,
                                         2.26967787067278e-05,
                                         3.12757564348151e-08,
                                         7.35318471270777e-14,
                                         1.9424430784647e-25,
                                         3.62041658740786e-16,
                                         2.70691132730161e-11,
                                         4.25879047911449e-08,
                                         7.94480834421636e-06,
                                         0.000360551624411843,
                                         0.00569933345090221,
                                         0.0381154369398656,
                                         0.119607918293282,
                                         0.183722021912364,
                                         0.13665563023253,
                                         0.0457647389823997,
                                         0.00581912424131218,
                                         0.000193926975270559,
                                         6.75264065610122e-07,
                                         8.49330797276234e-12,
                                         5.17444224076258e-28,
                                         5.15952080638259e-18,
                                         9.74802575736144e-13,
                                         2.96706950254059e-09,
                                         9.38236163766064e-07,
                                         6.70132656270408e-05,
                                         0.00159736640537692,
                                         0.0157505020731523,
                                         0.0723613134051635,
                                         0.163878034123524,
                                         0.183812148998903,
                                         0.0968818124316377,
                                         0.0208812355882331,
                                         0.0013462806190084,
                                         1.18457571727935e-05,
                                         7.97079497525228e-10,
                                         1.10272894170576e-30,
                                         5.88234073262926e-20,
                                         2.80833746440913e-14,
                                         1.65370923525441e-10,
                                         8.86402350676504e-08,
                                         9.96423805290242e-06,
                                         0.000358158294124431,
                                         0.00520688383444776,
                                         0.0350221607561537,
                                         0.116941931244372,
                                         0.197792984084762,
                                         0.164075413320264,
                                         0.05994386531166,
                                         0.00747692373415902,
                                         0.000166242476261495,
                                         5.98434181275483e-08,
                                         1.83596345735125e-33,
                                         5.23939349717943e-22,
                                         6.32079855699245e-16,
                                         7.20079777059922e-12,
                                         6.54243866733244e-09,
                                         1.15749050586632e-06,
                                         6.27386991577616e-05,
                                         0.00134478050780515,
                                         0.0132424842077972,
                                         0.0651943272758474,
                                         0.166279068595921,
                                         0.217087486067201,
                                         0.134438396756055,
                                         0.0324414560527825,
                                         0.00182268325264193,
                                         3.51011362447573e-06,
                                         2.30154961196929e-36,
                                         3.51376655259805e-24,
                                         1.07116322437242e-17,
                                         2.36082135514181e-13,
                                         3.63587939723375e-10,
                                         1.01239928230959e-07,
                                         8.27479140602629e-06,
                                         0.000261508596937949,
                                         0.00377013618848996,
                                         0.0273659420613369,
                                         0.105250782156562,
                                         0.216265438507214,
                                         0.22701939073786,
                                         0.105983625993213,
                                         0.0150467066934548,
                                         0.000155019740969,
                                         2.04368674744687e-39,
                                         1.66917730774928e-26,
                                         1.28581070686315e-19,
                                         5.48255901774587e-15,
                                         1.43125480598074e-11,
                                         6.27225778802227e-09,
                                         7.73065477296923e-07,
                                         3.60212032088787e-05,
                                         0.000760295277571081,
                                         0.00813670700309291,
                                         0.0471900850800262,
                                         0.152607940194303,
                                         0.271544032201052,
                                         0.245253316629161,
                                         0.0879851516951207,
                                         0.00484942802946834,
                                         1.14613549244866e-42,
                                         5.00794654337087e-29,
                                         9.74823672020606e-22,
                                         8.04139047024996e-17,
                                         3.55837809350563e-13,
                                         2.45427721402506e-10,
                                         4.56145242146112e-08,
                                         3.13370496172064e-06,
                                         9.68356322642821e-05,
                                         0.00152796919014283,
                                         0.0133629958600716,
                                         0.068013447016348,
                                         0.205137574652979,
                                         0.358441800025871,
                                         0.324941337726923,
                                         0.0958123902875431,
                                         3.05317152997406e-46,
                                         7.13691470689942e-32,
                                         3.51049780164584e-24,
                                         5.60238042686246e-19,
                                         4.20223983642141e-15,
                                         4.56159727845212e-12,
                                         1.27844965039092e-09,
                                         1.29494583976826e-07,
                                         5.85843615962041e-06,
                                         0.000136293180575469,
                                         0.00179742392020267,
                                         0.0143981286972948,
                                         0.0736111800795938,
                                         0.248837608973203,
                                         0.570025324631774,
                                         0.89917959117933,

                                         0.909856848597257,
                                         0.606877129435534,
                                         0.290845218767757,
                                         0.0988974053413581,
                                         0.0234081837071719,
                                         0.00375320156428647,
                                         0.000392642223024139,
                                         2.54609870581073e-05,
                                         9.5367431640625e-07,
                                         1.86947082895038e-08,
                                         1.66414691116217e-10,
                                         5.43872355509781e-13,
                                         4.65875504230788e-16,
                                         5.85965716409177e-20,
                                         3.51315419110571e-25,
                                         6.92867329672667e-33,
                                         2.91466880650647e-47,
                                         0.0861556714255925,
                                         0.306908016487879,
                                         0.370504671866269,
                                         0.242576797833357,
                                         0.0966813755401941,
                                         0.0241816894289737,
                                         0.00377039394002695,
                                         0.000354974703121008,
                                         1.9073486328125e-05,
                                         5.36360164968587e-07,
                                         6.9320538175149e-09,
                                         3.37654253887771e-11,
                                         4.51185115096832e-14,
                                         9.55581728994013e-18,
                                         1.10312681794826e-22,
                                         5.48027830518424e-30,
                                         1.23122776764986e-43,
                                         0.00387514791130418,
                                         0.0737240700918549,
                                         0.224191456255516,
                                         0.282622822670524,
                                         0.189675629442497,
                                         0.0740056707509903,
                                         0.0171976880578786,
                                         0.00235078647166859,
                                         0.000181198120117188,
                                         7.30950467385247e-06,
                                         1.37159469864942e-07,
                                         9.9572881681549e-10,
                                         2.075550291059e-12,
                                         7.40213628933602e-16,
                                         1.64530828248243e-20,
                                         2.05896400113415e-27,
                                         2.47047918759352e-40,
                                         0.000110083021528042,
                                         0.0111850523055712,
                                         0.0856785429931705,
                                         0.207966242637218,
                                         0.235021661531145,
                                         0.143044447421535,
                                         0.0495428573650213,
                                         0.00983233361664657,
                                         0.00108718872070312,
                                         6.29139605495855e-05,
                                         1.71402564336388e-06,
                                         1.85454583585839e-08,
                                         6.03030673510241e-11,
                                         3.62137885999141e-14,
                                         1.5498753468838e-18,
                                         4.88565209961695e-25,
                                         3.13077345361789e-37,
                                         2.21508338094317e-06,
                                         0.00120199985436821,
                                         0.0231933118097438,
                                         0.108396719987009,
                                         0.206272784044215,
                                         0.195845992508844,
                                         0.101095008695632,
                                         0.0291298138187046,
                                         0.00462055206298828,
                                         0.000383569249599183,
                                         1.51721284881726e-05,
                                         2.4466494368537e-07,
                                         1.24103270165111e-09,
                                         1.25495607897605e-12,
                                         1.03415095212725e-16,
                                         8.21171754400991e-23,
                                         2.81034582853283e-34,
                                         3.35599159350053e-08,
                                         9.72594611277547e-05,
                                         0.00472731464135802,
                                         0.0425402937750748,
                                         0.136312918574985,
                                         0.201892145066644,
                                         0.155324307291139,
                                         0.0649800228948928,
                                         0.0147857666015625,
                                         0.00176076577654782,
                                         0.000101119929257028,
                                         2.43033963909104e-06,
                                         1.92303901707097e-08,
                                         3.27449355121542e-11,
                                         5.1955574371838e-15,
                                         1.03921765284558e-20,
                                         1.89945468277664e-31,
                                         3.9722967063782e-10,
                                         6.14821906519258e-06,
                                         0.00075275956375767,
                                         0.0130429208522072,
                                         0.070375603659556,
                                         0.162597620542365,
                                         0.18644003134674,
                                         0.113243176134869,
                                         0.0369644165039062,
                                         0.00631465203786083,
                                         0.000526522618684767,
                                         1.8860457541595e-05,
                                         2.3280001104733e-07,
                                         6.67497713950124e-10,
                                         2.03924964269221e-13,
                                         1.0274697826972e-18,
                                         1.00297045053906e-28,
                                         3.76142566127094e-12,
                                         3.10925824472292e-07,
                                         9.58932508314455e-05,
                                         0.00319918400670036,
                                         0.0290668009589858,
                                         0.104760831372857,
                                         0.179031271510748,
                                         0.157882578303872,
                                         0.0739288330078125,
                                         0.0181170401607596,
                                         0.00219324574313739,
                                         0.000117092064979476,
                                         2.25459159850783e-06,
                                         1.08854255758302e-08,
                                         6.40322299271802e-12,
                                         8.1268377338675e-17,
                                         4.23679378624193e-26,
                                         2.89392089551666e-14,
                                         1.2775785911364e-08,
                                         9.92528922669523e-06,
                                         0.000637568645096372,
                                         0.00975429104931156,
                                         0.0548412135403703,
                                         0.139682498480195,
                                         0.178846212844034,
                                         0.120134353637695,
                                         0.0422326603354595,
                                         0.00742302506911788,
                                         0.000590644379457549,
                                         1.77409201710079e-05,
                                         1.44232925575728e-07,
                                         1.63361693766334e-10,
                                         5.22272895905807e-15,
                                         1.45415226657087e-23,
                                         1.82686392326575e-16,
                                         4.30728717335528e-10,
                                         8.42914785556082e-07,
                                         0.000104255758643753,
                                         0.00268583639483858,
                                         0.0235559458776076,
                                         0.0894211939716547,
                                         0.166230479501154,
                                         0.160179138183594,
                                         0.0807783547222909,
                                         0.0206138888831444,
                                         0.00244461266494306,
                                         0.000114543320931431,
                                         1.5680820514452e-06,
                                         3.41969363552426e-09,
                                         2.75396717210931e-13,
                                         4.09513114033473e-21,
                                         9.51436244999927e-19,
                                         1.19804733822519e-11,
                                         5.90579164568229e-08,
                                         1.40645908758761e-05,
                                         0.000610122930542206,
                                         0.00834733777852596,
                                         0.047227274462529,
                                         0.127466339921364,
                                         0.176197052001953,
                                         0.127466339921364,
                                         0.047227274462529,
                                         0.00834733777852594,
                                         0.000610122930542204,
                                         1.40645908758761e-05,
                                         5.90579164568229e-08,
                                         1.19804733822516e-11,
                                         9.51436245000039e-19,
                                         4.0951311403342e-21,
                                         2.75396717210938e-13,
                                         3.41969363552426e-09,
                                         1.56808205144521e-06,
                                         0.000114543320931431,
                                         0.00244461266494307,
                                         0.0206138888831444,
                                         0.0807783547222908,
                                         0.160179138183594,
                                         0.166230479501154,
                                         0.0894211939716548,
                                         0.0235559458776076,
                                         0.00268583639483857,
                                         0.000104255758643752,
                                         8.42914785556082e-07,
                                         4.30728717335519e-10,
                                         1.82686392326595e-16,
                                         1.45415226657067e-23,
                                         5.22272895905821e-15,
                                         1.63361693766334e-10,
                                         1.44232925575729e-07,
                                         1.7740920171008e-05,
                                         0.000590644379457551,
                                         0.00742302506911788,
                                         0.0422326603354595,
                                         0.120134353637695,
                                         0.178846212844034,
                                         0.139682498480195,
                                         0.0548412135403702,
                                         0.00975429104931153,
                                         0.000637568645096369,
                                         9.92528922669523e-06,
                                         1.27757859113638e-08,
                                         2.89392089551693e-14,
                                         4.23679378624128e-26,
                                         8.12683773386774e-17,
                                         6.40322299271802e-12,
                                         1.08854255758302e-08,
                                         2.25459159850784e-06,
                                         0.000117092064979477,
                                         0.00219324574313739,
                                         0.0181170401607596,
                                         0.0739288330078125,
                                         0.157882578303872,
                                         0.179031271510748,
                                         0.104760831372857,
                                         0.0290668009589857,
                                         0.00319918400670035,
                                         9.58932508314455e-05,
                                         3.10925824472288e-07,
                                         3.76142566127125e-12,
                                         1.00297045053889e-28,
                                         1.02746978269723e-18,
                                         2.03924964269221e-13,
                                         6.67497713950128e-10,
                                         2.32800011047331e-07,
                                         1.8860457541595e-05,
                                         0.000526522618684767,
                                         0.00631465203786083,
                                         0.0369644165039062,
                                         0.113243176134869,
                                         0.18644003134674,
                                         0.162597620542365,
                                         0.0703756036595559,
                                         0.0130429208522072,
                                         0.00075275956375767,
                                         6.1482190651925e-06,
                                         3.97229670637848e-10,
                                         1.8994546827763e-31,
                                         1.03921765284561e-20,
                                         5.1955574371838e-15,
                                         3.27449355121544e-11,
                                         1.92303901707098e-08,
                                         2.43033963909105e-06,
                                         0.000101119929257028,
                                         0.00176076577654782,
                                         0.0147857666015625,
                                         0.0649800228948928,
                                         0.155324307291139,
                                         0.201892145066644,
                                         0.136312918574985,
                                         0.0425402937750747,
                                         0.00472731464135802,
                                         9.72594611277536e-05,
                                         3.35599159350073e-08,
                                         2.8103458285323e-34,
                                         8.2117175440102e-23,
                                         1.03415095212725e-16,
                                         1.25495607897606e-12,
                                         1.24103270165111e-09,
                                         2.44664943685371e-07,
                                         1.51721284881726e-05,
                                         0.000383569249599183,
                                         0.00462055206298828,
                                         0.0291298138187046,
                                         0.101095008695632,
                                         0.195845992508844,
                                         0.206272784044215,
                                         0.108396719987008,
                                         0.0231933118097438,
                                         0.0012019998543682,
                                         2.21508338094327e-06,
                                         3.13077345361727e-37,
                                         4.88565209961714e-25,
                                         1.5498753468838e-18,
                                         3.62137885999144e-14,
                                         6.03030673510244e-11,
                                         1.8545458358584e-08,
                                         1.71402564336388e-06,
                                         6.29139605495854e-05,
                                         0.00108718872070312,
                                         0.00983233361664657,
                                         0.0495428573650213,
                                         0.143044447421535,
                                         0.235021661531145,
                                         0.207966242637217,
                                         0.0856785429931705,
                                         0.0111850523055711,
                                         0.000110083021528046,
                                         2.47047918759299e-40,
                                         2.05896400113424e-27,
                                         1.64530828248243e-20,
                                         7.40213628933608e-16,
                                         2.07555029105901e-12,
                                         9.95728816815494e-10,
                                         1.37159469864942e-07,
                                         7.30950467385247e-06,
                                         0.000181198120117188,
                                         0.00235078647166859,
                                         0.0171976880578786,
                                         0.0740056707509903,
                                         0.189675629442497,
                                         0.282622822670524,
                                         0.224191456255516,
                                         0.0737240700918546,
                                         0.00387514791130427,
                                         1.23122776764958e-43,
                                         5.48027830518448e-30,
                                         1.10312681794826e-22,
                                         9.55581728994023e-18,
                                         4.51185115096835e-14,
                                         3.37654253887772e-11,
                                         6.9320538175149e-09,
                                         5.36360164968587e-07,
                                         1.9073486328125e-05,
                                         0.000354974703121008,
                                         0.00377039394002695,
                                         0.0241816894289737,
                                         0.0966813755401941,
                                         0.242576797833357,
                                         0.370504671866269,
                                         0.306908016487878,
                                         0.0861556714255935,
                                         2.91466880650579e-47,
                                         6.92867329672698e-33,
                                         3.51315419110571e-25,
                                         5.85965716409183e-20,
                                         4.65875504230791e-16,
                                         5.43872355509784e-13,
                                         1.66414691116217e-10,
                                         1.86947082895038e-08,
                                         9.5367431640625e-07,
                                         2.54609870581073e-05,
                                         0.000392642223024139,
                                         0.00375320156428647,
                                         0.0234081837071719,
                                         0.0988974053413581,
                                         0.290845218767757,
                                         0.606877129435534,
                                         0.909856848597257,

                                         0.918947134390828,
                                         0.639727888926431,
                                         0.331579842977657,
                                         0.126680588630639,
                                         0.0351399020278785,
                                         0.0069281291583213,
                                         0.000943111761335068,
                                         8.52505729337222e-05,
                                         4.85504820935553e-06,
                                         1.62165123150905e-07,
                                         2.8757631084904e-09,
                                         2.34758677307624e-11,
                                         7.1281426807014e-14,
                                         5.74428019627973e-17,
                                         6.87997168943035e-21,
                                         3.9745341755279e-26,
                                         7.64078162946959e-34,
                                         3.16904490450462e-48,
                                         0.0778399264212925,
                                         0.288989727199017,
                                         0.376316293836701,
                                         0.275729921558807,
                                         0.128086186451902,
                                         0.0391078545747665,
                                         0.0078595210453551,
                                         0.00101889655098937,
                                         8.19241110952509e-05,
                                         3.84414053566393e-06,
                                         9.62454735497589e-08,
                                         1.12680489493762e-09,
                                         5.05112782972312e-12,
                                         6.30367563933216e-15,
                                         1.26436602665668e-18,
                                         1.40081674847994e-23,
                                         6.76566762278333e-31,
                                         1.49649922225711e-44,
                                         0.00313190020545869,
                                         0.0620102004919454,
                                         0.202867059331026,
                                         0.285069879061235,
                                         0.221767374158252,
                                         0.104858977204828,
                                         0.0311116189592358,
                                         0.00578437562703464,
                                         0.000656634260348159,
                                         4.32847253535955e-05,
                                         1.53003416621753e-06,
                                         2.56903135919448e-08,
                                         1.70017624648671e-10,
                                         3.28583468665601e-13,
                                         1.10370394345414e-16,
                                         2.34514675475549e-21,
                                         2.84562153269511e-28,
                                         3.35674389321711e-41,
                                         7.95868029056999e-05,
                                         0.00840371878446485,
                                         0.0690713095435172,
                                         0.18614287218426,
                                         0.242504978068922,
                                         0.177572164332115,
                                         0.0777815845342773,
                                         0.0207400848106718,
                                         0.00332401507301518,
                                         0.000307820628888243,
                                         1.53620646749535e-05,
                                         3.69928448735715e-07,
                                         3.61432477101518e-09,
                                         1.08174577291133e-11,
                                         6.08499205774771e-15,
                                         2.47962707574545e-19,
                                         7.55912041623947e-26,
                                         4.75540559716458e-38,
                                         1.4325569047651e-06,
                                         0.000806710233704608,
                                         0.0166579489096636,
                                         0.086095245699764,
                                         0.18783708574398,
                                         0.21300109623581,
                                         0.137742715604671,
                                         0.0526747215020538,
                                         0.0119190079647073,
                                         0.0015505952730407,
                                         0.000109253593179412,
                                         3.77314709783855e-06,
                                         5.4424955651721e-08,
                                         2.52256510263365e-10,
                                         2.37632105950193e-13,
                                         1.85712182651136e-17,
                                         1.4223388367323e-23,
                                         4.77193590127464e-35,
                                         1.94152842770701e-08,
                                         5.83075334128532e-05,
                                         0.0030248678763157,
                                         0.0299828544846412,
                                         0.109547651979667,
                                         0.192375533550227,
                                         0.183662944990826,
                                         0.100729114591628,
                                         0.0321794253087502,
                                         0.00588112267564001,
                                         0.000585036439560421,
                                         2.8976824493472e-05,
                                         6.17063760787243e-07,
                                         4.42915223986454e-09,
                                         6.9873301849283e-12,
                                         1.04726229284939e-15,
                                         2.01509683861493e-21,
                                         3.60547664301883e-32,
                                         2.05572802152738e-10,
                                         3.29246982707475e-06,
                                         0.000429122537402523,
                                         0.00815749496876719,
                                         0.0499131335287435,
                                         0.135740006751843,
                                         0.191321808372408,
                                         0.150486594854638,
                                         0.0678744757079693,
                                         0.0174265754816481,
                                         0.00244748554753678,
                                         0.000173855275838254,
                                         5.46577852472111e-06,
                                         6.07559566284081e-08,
                                         1.60511977767811e-10,
                                         4.61381917693562e-14,
                                         2.23037970768713e-19,
                                         2.12824098219401e-29,
                                         1.74131581621348e-12,
                                         1.48733543371711e-07,
                                         4.87019359883104e-05,
                                         0.00177554072977407,
                                         0.0181935138080044,
                                         0.0766224232071789,
                                         0.15944004104134,
                                         0.179858348385213,
                                         0.114531428909739,
                                         0.0413098722494551,
                                         0.00819119644568089,
                                         0.000834478103523175,
                                         3.8731472213748e-05,
                                         6.66725561177984e-07,
                                         2.94980707366685e-09,
                                         1.62613148914511e-12,
                                         1.97492985751764e-17,
                                         1.00500657788152e-26,
                                         1.19843036203455e-14,
                                         5.45907945089408e-09,
                                         4.49091149811251e-06,
                                         0.000313998687993674,
                                         0.00538817513629531,
                                         0.0351420642402024,
                                         0.10795771589822,
                                         0.17465730572894,
                                         0.157024073599366,
                                         0.0795644420780797,
                                         0.0222739846878363,
                                         0.00325435844725145,
                                         0.000222997169224985,
                                         5.94468322712083e-06,
                                         4.40456627471071e-08,
                                         4.65665560102918e-11,
                                         1.42084896667859e-15,
                                         3.85602975369778e-24,
                                         6.76758054304189e-17,
                                         1.64405096664786e-10,
                                         3.39788079545034e-07,
                                         4.55627993448945e-05,
                                         0.0013093392702034,
                                         0.0132246450795666,
                                         0.0599784652534638,
                                         0.139164442175784,
                                         0.176641674060616,
                                         0.125738872842266,
                                         0.0496974222897943,
                                         0.0104136073415206,
                                         0.00105346489062418,
                                         4.34906301711137e-05,
                                         5.39632449210763e-07,
                                         1.0941527350113e-09,
                                         8.38744194585738e-14,
                                         1.21393999376205e-21,
                                         3.15288413765356e-19,
                                         4.08474629162393e-12,
                                         2.1209758801922e-08,
                                         5.45439911026819e-06,
                                         0.000262492338673737,
                                         0.00410577107637165,
                                         0.0274910266301006,
                                         0.0914794949764464,
                                         0.163935893696154,
                                         0.163935893696154,
                                         0.0914794949764464,
                                         0.0274910266301006,
                                         0.00410577107637165,
                                         0.000262492338673737,
                                         5.45439911026822e-06,
                                         2.1209758801922e-08,
                                         4.08474629162393e-12,
                                         3.15288413765356e-19,
                                         1.21393999376205e-21,
                                         8.38744194585738e-14,
                                         1.0941527350113e-09,
                                         5.39632449210759e-07,
                                         4.34906301711137e-05,
                                         0.00105346489062418,
                                         0.0104136073415206,
                                         0.0496974222897943,
                                         0.125738872842266,
                                         0.176641674060616,
                                         0.139164442175784,
                                         0.0599784652534638,
                                         0.0132246450795666,
                                         0.0013093392702034,
                                         4.55627993448948e-05,
                                         3.39788079545034e-07,
                                         1.64405096664786e-10,
                                         6.76758054304189e-17,
                                         3.85602975369778e-24,
                                         1.42084896667859e-15,
                                         4.65665560102918e-11,
                                         4.40456627471068e-08,
                                         5.94468322712084e-06,
                                         0.000222997169224985,
                                         0.00325435844725145,
                                         0.0222739846878363,
                                         0.0795644420780797,
                                         0.157024073599366,
                                         0.17465730572894,
                                         0.10795771589822,
                                         0.0351420642402024,
                                         0.00538817513629531,
                                         0.000313998687993676,
                                         4.49091149811251e-06,
                                         5.45907945089408e-09,
                                         1.19843036203455e-14,
                                         1.00500657788152e-26,
                                         1.97492985751764e-17,
                                         1.62613148914511e-12,
                                         2.94980707366682e-09,
                                         6.66725561177983e-07,
                                         3.8731472213748e-05,
                                         0.000834478103523175,
                                         0.00819119644568089,
                                         0.0413098722494551,
                                         0.114531428909739,
                                         0.179858348385213,
                                         0.15944004104134,
                                         0.0766224232071789,
                                         0.0181935138080044,
                                         0.00177554072977408,
                                         4.87019359883104e-05,
                                         1.48733543371711e-07,
                                         1.74131581621348e-12,
                                         2.12824098219401e-29,
                                         2.23037970768713e-19,
                                         4.61381917693562e-14,
                                         1.6051197776781e-10,
                                         6.07559566284081e-08,
                                         5.46577852472111e-06,
                                         0.000173855275838254,
                                         0.00244748554753678,
                                         0.0174265754816481,
                                         0.0678744757079693,
                                         0.150486594854638,
                                         0.191321808372408,
                                         0.135740006751843,
                                         0.0499131335287435,
                                         0.00815749496876721,
                                         0.000429122537402523,
                                         3.29246982707475e-06,
                                         2.05572802152738e-10,
                                         3.60547664301883e-32,
                                         2.01509683861493e-21,
                                         1.04726229284939e-15,
                                         6.98733018492824e-12,
                                         4.42915223986454e-09,
                                         6.17063760787244e-07,
                                         2.8976824493472e-05,
                                         0.000585036439560421,
                                         0.00588112267564,
                                         0.0321794253087502,
                                         0.100729114591628,
                                         0.183662944990826,
                                         0.192375533550227,
                                         0.109547651979668,
                                         0.0299828544846413,
                                         0.0030248678763157,
                                         5.83075334128532e-05,
                                         1.94152842770701e-08,
                                         4.77193590127464e-35,
                                         1.4223388367323e-23,
                                         1.85712182651136e-17,
                                         2.37632105950191e-13,
                                         2.52256510263365e-10,
                                         5.4424955651721e-08,
                                         3.77314709783855e-06,
                                         0.000109253593179412,
                                         0.0015505952730407,
                                         0.0119190079647073,
                                         0.0526747215020538,
                                         0.137742715604671,
                                         0.21300109623581,
                                         0.18783708574398,
                                         0.0860952456997642,
                                         0.0166579489096636,
                                         0.000806710233704608,
                                         1.4325569047651e-06,
                                         4.75540559716458e-38,
                                         7.55912041623947e-26,
                                         2.47962707574545e-19,
                                         6.08499205774766e-15,
                                         1.08174577291133e-11,
                                         3.61432477101518e-09,
                                         3.69928448735715e-07,
                                         1.53620646749535e-05,
                                         0.000307820628888243,
                                         0.00332401507301518,
                                         0.0207400848106718,
                                         0.0777815845342773,
                                         0.177572164332115,
                                         0.242504978068922,
                                         0.18614287218426,
                                         0.0690713095435172,
                                         0.00840371878446485,
                                         7.95868029056999e-05,
                                         3.35674389321711e-41,
                                         2.84562153269511e-28,
                                         2.34514675475549e-21,
                                         1.10370394345413e-16,
                                         3.28583468665601e-13,
                                         1.70017624648671e-10,
                                         2.56903135919448e-08,
                                         1.53003416621753e-06,
                                         4.32847253535955e-05,
                                         0.000656634260348159,
                                         0.00578437562703464,
                                         0.0311116189592358,
                                         0.104858977204828,
                                         0.221767374158252,
                                         0.285069879061235,
                                         0.202867059331026,
                                         0.0620102004919454,
                                         0.00313190020545869,
                                         1.49649922225711e-44,
                                         6.76566762278333e-31,
                                         1.40081674847994e-23,
                                         1.26436602665667e-18,
                                         6.30367563933216e-15,
                                         5.05112782972312e-12,
                                         1.12680489493762e-09,
                                         9.62454735497589e-08,
                                         3.84414053566393e-06,
                                         8.19241110952509e-05,
                                         0.00101889655098937,
                                         0.0078595210453551,
                                         0.0391078545747665,
                                         0.128086186451902,
                                         0.275729921558808,
                                         0.376316293836701,
                                         0.288989727199017,
                                         0.0778399264212925,
                                         3.16904490450462e-48,
                                         7.64078162946959e-34,
                                         3.9745341755279e-26,
                                         6.87997168943027e-21,
                                         5.74428019627973e-17,
                                         7.1281426807014e-14,
                                         2.34758677307624e-11,
                                         2.8757631084904e-09,
                                         1.62165123150905e-07,
                                         4.85504820935553e-06,
                                         8.52505729337222e-05,
                                         0.000943111761335068,
                                         0.0069281291583213,
                                         0.0351399020278785,
                                         0.126680588630639,
                                         0.331579842977657,
                                         0.639727888926431,
                                         0.918947134390828,

                                         0.926745708664536,
                                         0.669027403538992,
                                         0.370584641420133,
                                         0.156233370535048,
                                         0.0495275435527743,
                                         0.0116080566947649,
                                         0.00196610705305545,
                                         0.00023347493246659,
                                         1.86743259926473e-05,
                                         9.5367431640625e-07,
                                         2.89252083880886e-08,
                                         4.7136338820118e-10,
                                         3.57636699798774e-12,
                                         1.02035358421658e-14,
                                         7.80795070981088e-18,
                                         8.97132827312624e-22,
                                         5.02184278247327e-27,
                                         9.44674596607124e-35,
                                         3.87120985819086e-49,
                                         0.070637429347544,
                                         0.271622548148877,
                                         0.377151810468,
                                         0.303919065513933,
                                         0.160605826967796,
                                         0.0579414120546547,
                                         0.0143756967424319,
                                         0.00242396326332524,
                                         0.000270256724972666,
                                         1.9073486328125e-05,
                                         7.99475048621348e-07,
                                         1.81605945754348e-08,
                                         1.95650214526407e-10,
                                         8.17675775186161e-13,
                                         9.63124753663178e-16,
                                         1.84472909182808e-19,
                                         1.97375990798172e-24,
                                         9.30722720723318e-32,
                                         2.03157286812729e-45,
                                         0.00255742436100325,
                                         0.0523819113081067,
                                         0.182321794576379,
                                         0.280824954883867,
                                         0.247382752256815,
                                         0.137376434023675,
                                         0.0499280096889292,
                                         0.011953784391075,
                                         0.0018578116969422,
                                         0.000181198120117188,
                                         1.04960753878224e-05,
                                         3.32351688112994e-07,
                                         5.08407780052757e-09,
                                         3.11247002747411e-11,
                                         5.64315054816838e-14,
                                         1.80178121494513e-17,
                                         3.6848443150738e-22,
                                         4.35564027381869e-29,
                                         5.0642099578887e-42,
                                         5.8478786874247e-05,
                                         0.00638005625082914,
                                         0.0556658214291448,
                                         0.163885713216724,
                                         0.240660703037529,
                                         0.205713620625783,
                                         0.109518440279102,
                                         0.0372316641220009,
                                         0.00806593134920962,
                                         0.00108718872070312,
                                         8.70315290568645e-05,
                                         3.84143385890549e-06,
                                         8.3439499880899e-08,
                                         7.48267477615405e-10,
                                         2.08827828847316e-12,
                                         1.11147361565459e-15,
                                         4.34481820197388e-20,
                                         1.2873935789214e-26,
                                         7.97294276917977e-39,
                                         9.47177976360807e-07,
                                         0.000550434129440718,
                                         0.0120386097659269,
                                         0.0677460803430387,
                                         0.16583592779282,
                                         0.218198387457319,
                                         0.170164094104556,
                                         0.0821404650856281,
                                         0.0248053311599778,
                                         0.00462055206298828,
                                         0.00051116836171207,
                                         3.14504244226707e-05,
                                         9.69993617246946e-07,
                                         1.27422535035786e-08,
                                         5.47384870524625e-11,
                                         4.85661794858846e-14,
                                         3.62878931803022e-18,
                                         2.69530770148545e-24,
                                         8.89125918123304e-36,
                                         1.15511673606603e-08,
                                         3.57558617266615e-05,
                                         0.00196031155585409,
                                         0.0210857133628508,
                                         0.0860425190948785,
                                         0.174261349816149,
                                         0.199071757318292,
                                         0.136446661881187,
                                         0.0574376397952788,
                                         0.0147857666015625,
                                         0.00226054088378767,
                                         0.000193874508360145,
                                         8.49037962425629e-06,
                                         1.63379160672533e-07,
                                         1.08033452192887e-09,
                                         1.59782705880783e-12,
                                         2.2819858544159e-16,
                                         4.24880122932969e-22,
                                         7.46567262546589e-33,
                                         1.1005510474606e-10,
                                         1.81459276220975e-06,
                                         0.000249381305151528,
                                         0.00512722592409816,
                                         0.0348768809796222,
                                         0.108727810136508,
                                         0.181945536024054,
                                         0.177075592392926,
                                         0.103905412887081,
                                         0.0369644165039062,
                                         0.00780999573420928,
                                         0.000933694797096121,
                                         5.80597781152994e-05,
                                         1.63657951418317e-06,
                                         1.66576506271863e-08,
                                         4.10691422908128e-11,
                                         1.12112534808617e-14,
                                         5.23256348558681e-20,
                                         4.89738948023126e-30,
                                         8.38850357023926e-13,
                                         7.36717670033672e-08,
                                         2.53800617247225e-05,
                                         0.000997393646564869,
                                         0.0113097276989338,
                                         0.0542712963467934,
                                         0.133034152207339,
                                         0.183841890971926,
                                         0.150372959135724,
                                         0.0739288330078125,
                                         0.0215863499946666,
                                         0.00359732068541955,
                                         0.000317624227323576,
                                         1.31149774314849e-05,
                                         2.05475114446575e-07,
                                         8.44484108192026e-10,
                                         4.40641485551388e-13,
                                         5.15528388415871e-18,
                                         2.57010184339103e-27,
                                         5.19495949343769e-15,
                                         2.43022435128421e-09,
                                         2.09867323601886e-06,
                                         0.000157642789104547,
                                         0.00297981997925064,
                                         0.0220101559372357,
                                         0.0790329617737276,
                                         0.155079225435676,
                                         0.176817273483563,
                                         0.120134353637695,
                                         0.0484764715736788,
                                         0.0112609943582424,
                                         0.00141180670783524,
                                         8.53928196732274e-05,
                                         2.05934013003557e-06,
                                         1.41088202110587e-08,
                                         1.40714860104905e-11,
                                         4.12680581890158e-16,
                                         1.09587089275891e-24,
                                         2.63976464993009e-17,
                                         6.57774482158709e-11,
                                         1.42390936208624e-07,
                                         2.04440528019443e-05,
                                         0.000644188960529622,
                                         0.00732422057128487,
                                         0.0385246533871648,
                                         0.107336675330888,
                                         0.170594491671182,
                                         0.160179138183594,
                                         0.0893239994410732,
                                         0.0289240895913642,
                                         0.00514899217396131,
                                         0.000456205516085868,
                                         1.69348870927767e-05,
                                         1.93408380582662e-07,
                                         3.68705753860779e-10,
                                         2.71057141683838e-14,
                                         3.83401512576892e-22,
                                         1.10662939105762e-19,
                                         1.46879647983603e-12,
                                         7.97028434549663e-09,
                                         2.18732756701284e-06,
                                         0.000114892181843277,
                                         0.00201072954149388,
                                         0.0154925594380674,
                                         0.0612909854104894,
                                         0.135787335776246,
                                         0.176197052001953,
                                         0.135787335776246,
                                         0.0612909854104895,
                                         0.0154925594380674,
                                         0.00201072954149388,
                                         0.000114892181843277,
                                         2.18732756701284e-06,
                                         7.97028434549663e-09,
                                         1.46879647983603e-12,
                                         1.10662939105762e-19,
                                         3.83401512576892e-22,
                                         2.71057141683838e-14,
                                         3.68705753860779e-10,
                                         1.93408380582662e-07,
                                         1.69348870927767e-05,
                                         0.000456205516085869,
                                         0.00514899217396131,
                                         0.0289240895913641,
                                         0.0893239994410732,
                                         0.160179138183594,
                                         0.170594491671182,
                                         0.107336675330888,
                                         0.0385246533871649,
                                         0.00732422057128485,
                                         0.000644188960529624,
                                         2.04440528019443e-05,
                                         1.42390936208624e-07,
                                         6.57774482158709e-11,
                                         2.63976464993009e-17,
                                         1.09587089275891e-24,
                                         4.12680581890158e-16,
                                         1.40714860104905e-11,
                                         1.41088202110587e-08,
                                         2.05934013003556e-06,
                                         8.53928196732276e-05,
                                         0.00141180670783524,
                                         0.0112609943582424,
                                         0.0484764715736788,
                                         0.120134353637695,
                                         0.176817273483563,
                                         0.155079225435676,
                                         0.0790329617737276,
                                         0.0220101559372356,
                                         0.00297981997925065,
                                         0.000157642789104547,
                                         2.09867323601886e-06,
                                         2.43022435128421e-09,
                                         5.19495949343769e-15,
                                         2.57010184339103e-27,
                                         5.15528388415871e-18,
                                         4.40641485551388e-13,
                                         8.44484108192026e-10,
                                         2.05475114446574e-07,
                                         1.3114977431485e-05,
                                         0.000317624227323576,
                                         0.00359732068541954,
                                         0.0215863499946666,
                                         0.0739288330078125,
                                         0.150372959135724,
                                         0.183841890971926,
                                         0.133034152207339,
                                         0.0542712963467933,
                                         0.0113097276989338,
                                         0.000997393646564869,
                                         2.53800617247225e-05,
                                         7.36717670033672e-08,
                                         8.38850357023926e-13,
                                         4.89738948023126e-30,
                                         5.23256348558681e-20,
                                         1.12112534808617e-14,
                                         4.10691422908128e-11,
                                         1.66576506271862e-08,
                                         1.63657951418318e-06,
                                         5.80597781152994e-05,
                                         0.000933694797096119,
                                         0.00780999573420928,
                                         0.0369644165039062,
                                         0.103905412887081,
                                         0.177075592392926,
                                         0.181945536024054,
                                         0.108727810136508,
                                         0.0348768809796223,
                                         0.00512722592409816,
                                         0.000249381305151528,
                                         1.81459276220975e-06,
                                         1.1005510474606e-10,
                                         7.46567262546589e-33,
                                         4.24880122932969e-22,
                                         2.2819858544159e-16,
                                         1.59782705880783e-12,
                                         1.08033452192886e-09,
                                         1.63379160672534e-07,
                                         8.49037962425629e-06,
                                         0.000193874508360145,
                                         0.00226054088378767,
                                         0.0147857666015625,
                                         0.0574376397952788,
                                         0.136446661881188,
                                         0.199071757318292,
                                         0.174261349816148,
                                         0.0860425190948786,
                                         0.0210857133628508,
                                         0.00196031155585409,
                                         3.57558617266614e-05,
                                         1.15511673606603e-08,
                                         8.89125918123304e-36,
                                         2.69530770148545e-24,
                                         3.62878931803022e-18,
                                         4.85661794858846e-14,
                                         5.47384870524622e-11,
                                         1.27422535035786e-08,
                                         9.69993617246947e-07,
                                         3.14504244226706e-05,
                                         0.00051116836171207,
                                         0.00462055206298828,
                                         0.0248053311599778,
                                         0.0821404650856282,
                                         0.170164094104556,
                                         0.218198387457319,
                                         0.16583592779282,
                                         0.0677460803430387,
                                         0.0120386097659269,
                                         0.000550434129440718,
                                         9.47177976360807e-07,
                                         7.97294276917977e-39,
                                         1.2873935789214e-26,
                                         4.34481820197388e-20,
                                         1.11147361565459e-15,
                                         2.08827828847315e-12,
                                         7.48267477615409e-10,
                                         8.3439499880899e-08,
                                         3.84143385890548e-06,
                                         8.70315290568645e-05,
                                         0.00108718872070312,
                                         0.00806593134920962,
                                         0.0372316641220009,
                                         0.109518440279102,
                                         0.205713620625783,
                                         0.24066070303753,
                                         0.163885713216724,
                                         0.0556658214291448,
                                         0.00638005625082914,
                                         5.8478786874247e-05,
                                         5.0642099578887e-42,
                                         4.35564027381869e-29,
                                         3.6848443150738e-22,
                                         1.80178121494513e-17,
                                         5.64315054816834e-14,
                                         3.11247002747413e-11,
                                         5.08407780052757e-09,
                                         3.32351688112993e-07,
                                         1.04960753878224e-05,
                                         0.000181198120117188,
                                         0.0018578116969422,
                                         0.011953784391075,
                                         0.0499280096889292,
                                         0.137376434023675,
                                         0.247382752256815,
                                         0.280824954883867,
                                         0.182321794576379,
                                         0.0523819113081067,
                                         0.00255742436100325,
                                         2.03157286812729e-45,
                                         9.30722720723318e-32,
                                         1.97375990798172e-24,
                                         1.84472909182808e-19,
                                         9.6312475366317e-16,
                                         8.17675775186166e-13,
                                         1.95650214526407e-10,
                                         1.81605945754347e-08,
                                         7.99475048621348e-07,
                                         1.9073486328125e-05,
                                         0.000270256724972666,
                                         0.00242396326332524,
                                         0.0143756967424319,
                                         0.0579414120546547,
                                         0.160605826967796,
                                         0.303919065513933,
                                         0.377151810468,
                                         0.271622548148877,
                                         0.070637429347544,
                                         3.87120985819086e-49,
                                         9.44674596607124e-35,
                                         5.02184278247327e-27,
                                         8.97132827312624e-22,
                                         7.80795070981082e-18,
                                         1.02035358421658e-14,
                                         3.57636699798774e-12,
                                         4.71363388201179e-10,
                                         2.89252083880886e-08,
                                         9.5367431640625e-07,
                                         1.86743259926473e-05,
                                         0.00023347493246659,
                                         0.00196610705305545,
                                         0.0116080566947649,
                                         0.0495275435527743,
                                         0.156233370535048,
                                         0.370584641420133,
                                         0.669027403538992,
                                         0.926745708664536,

                                         0.933483193802115,
                                         0.695193338278448,
                                         0.40758927498791,
                                         0.186890458911011,
                                         0.0663645057482855,
                                         0.0180032566085457,
                                         0.00366374137310215,
                                         0.000546105499984444,
                                         5.77899838398234e-05,
                                         4.1676837508227e-06,
                                         1.94038825276865e-07,
                                         5.4219459107863e-09,
                                         8.2217426458051e-11,
                                         5.86056292344384e-13,
                                         1.58542870774191e-15,
                                         1.16069198547605e-18,
                                         1.28707758771345e-22,
                                         7.01267665201419e-28,
                                         1.29485369521262e-35,
                                         5.25177036219757e-50,
                                         0.0643645089555639,
                                         0.255059410977634,
                                         0.374141425647837,
                                         0.326977603141221,
                                         0.192799061731695,
                                         0.080097848313958,
                                         0.0237222310049574,
                                         0.00497955934488302,
                                         0.000726938547824314,
                                         7.15030300997175e-05,
                                         4.52396189648086e-06,
                                         1.72413014829123e-07,
                                         3.60669574744172e-09,
                                         3.62050042389396e-11,
                                         1.42540057995834e-13,
                                         1.59811462255578e-16,
                                         2.94261770483347e-20,
                                         3.05584102307825e-25,
                                         1.41170821261872e-32,
                                         3.0466724288716e-46,
                                         0.00210804572517597,
                                         0.0444498922593894,
                                         0.163133237583903,
                                         0.271733067327268,
                                         0.266052642871547,
                                         0.169271653785306,
                                         0.0729591662207414,
                                         0.0215674541882679,
                                         0.00434346608478659,
                                         0.000582703659653911,
                                         5.01005910828102e-05,
                                         2.60422510322025e-06,
                                         7.5153419635366e-08,
                                         1.06240836555407e-09,
                                         6.08724461482532e-12,
                                         1.04518333022269e-14,
                                         3.19563058493365e-18,
                                         6.32515698400518e-23,
                                         7.31076445430187e-30,
                                         8.39536159832605e-43,
                                         4.36054967644945e-05,
                                         0.00489246489290645,
                                         0.0449238283431274,
                                         0.14262465976049,
                                         0.231877112645552,
                                         0.225930711485484,
                                         0.141720226847131,
                                         0.0589976211839571,
                                         0.0163909005596041,
                                         0.00299915347275746,
                                         0.000350424454579749,
                                         2.48436064502222e-05,
                                         9.89044040959647e-07,
                                         1.96898317862103e-08,
                                         1.64184525522028e-10,
                                         4.3172248559863e-13,
                                         2.19183036681702e-16,
                                         8.26874322670273e-21,
                                         2.39115807264177e-27,
                                         1.46110253960102e-39,
                                         6.38910653346032e-07,
                                         0.000381436635443063,
                                         0.00876291789387843,
                                         0.0530254583603361,
                                         0.143148192936068,
                                         0.213601344659264,
                                         0.194994110830318,
                                         0.114316204705725,
                                         0.0438133918710428,
                                         0.0109342075706218,
                                         0.00173613559834493,
                                         0.000167875933480421,
                                         9.21977227548139e-06,
                                         2.58481943077202e-07,
                                         3.13676056897119e-09,
                                         1.26314888422482e-11,
                                         1.06486607124804e-14,
                                         7.65676613688974e-19,
                                         5.53976653135334e-25,
                                         1.8011905694311e-36,
                                         7.04855461645316e-09,
                                         2.23912395483699e-05,
                                         0.00128700956373217,
                                         0.0148434648883884,
                                         0.0665387908201202,
                                         0.152052562277462,
                                         0.202009688814095,
                                         0.166779298265311,
                                         0.0881803837849027,
                                         0.0300149059202257,
                                         0.00647640391170554,
                                         0.000854128647828291,
                                         6.4712146591601e-05,
                                         2.55493268262366e-06,
                                         4.51223350491249e-08,
                                         2.78269408608593e-10,
                                         3.89532849853781e-13,
                                         5.33841469699104e-17,
                                         9.66352746997274e-23,
                                         1.67185913124594e-33,
                                         6.0750525524546e-11,
                                         1.02688994122288e-06,
                                         0.000147674271180049,
                                         0.00324620676183578,
                                         0.0241631733225333,
                                         0.0845616666324948,
                                         0.163498184481222,
                                         0.1900931718399,
                                         0.138652314544807,
                                         0.0643689891606589,
                                         0.0188744472149904,
                                         0.00339506557348342,
                                         0.000354847253761398,
                                         1.97296210732941e-05,
                                         5.0709787701289e-07,
                                         4.78923797691416e-09,
                                         1.11322568229e-11,
                                         2.90783167376317e-15,
                                         1.31695197907928e-20,
                                         1.21235477963493e-30,
                                         4.18880358011968e-13,
                                         3.76755542847646e-08,
                                         1.35555731569355e-05,
                                         0.000567946010979245,
                                         0.00701977222992442,
                                         0.037622124121088,
                                         0.105862868204332,
                                         0.173332850568396,
                                         0.174410348455965,
                                         0.11043490928347,
                                         0.0440052550802246,
                                         0.0107960038830774,
                                         0.00155663602750398,
                                         0.000121884369116462,
                                         4.55913031257839e-06,
                                         6.59412775962856e-08,
                                         2.54514384637951e-10,
                                         1.26711550493913e-13,
                                         1.43580076371903e-18,
                                         7.03314811270399e-28,
                                         2.34667703136311e-15,
                                         1.12310088804773e-09,
                                         1.01100732878211e-06,
                                         8.07348975990421e-05,
                                         0.00165697303994743,
                                         0.0135999233139097,
                                         0.0556926326506672,
                                         0.128415844168612,
                                         0.17825459979209,
                                         0.153942820057062,
                                         0.0833600975303983,
                                         0.0278933879399612,
                                         0.00554825070042406,
                                         0.000611787575444583,
                                         3.3303937950259e-05,
                                         7.37686212502344e-07,
                                         4.72786229039249e-09,
                                         4.48628146306405e-12,
                                         1.27186726699151e-16,
                                         3.3150735436518e-25,
                                         1.07870333252089e-17,
                                         2.74702911359196e-11,
                                         6.18694269849913e-08,
                                         9.41674727916657e-06,
                                         0.000320917377760407,
                                         0.00403380572883658,
                                         0.024040152082655,
                                         0.0780623666891067,
                                         0.149483966258494,
                                         0.176075069579558,
                                         0.129567851305482,
                                         0.0591323138427692,
                                         0.0162259617123627,
                                         0.00251964098007225,
                                         0.000199615629671161,
                                         6.77128895411628e-06,
                                         7.20613966323469e-08,
                                         1.30329341336155e-10,
                                         9.24431577513989e-15,
                                         1.28210012606103e-22,
                                         4.09076627432411e-20,
                                         5.54321472876078e-13,
                                         3.12357423763821e-09,
                                         9.06138285479327e-07,
                                         5.12772795417984e-05,
                                         0.000987068848330629,
                                         0.00856112074141572,
                                         0.0391488280465632,
                                         0.103419498629531,
                                         0.166146136894611,
                                         0.166146136894611,
                                         0.103419498629531,
                                         0.0391488280465632,
                                         0.00856112074141572,
                                         0.000987068848330626,
                                         5.12772795417984e-05,
                                         9.06138285479333e-07,
                                         3.12357423763821e-09,
                                         5.54321472876095e-13,
                                         4.09076627432477e-20,
                                         1.2821001260608e-22,
                                         9.24431577513958e-15,
                                         1.30329341336155e-10,
                                         7.20613966323464e-08,
                                         6.77128895411628e-06,
                                         0.000199615629671162,
                                         0.00251964098007225,
                                         0.0162259617123627,
                                         0.0591323138427692,
                                         0.129567851305482,
                                         0.176075069579558,
                                         0.149483966258494,
                                         0.0780623666891067,
                                         0.024040152082655,
                                         0.00403380572883657,
                                         0.000320917377760407,
                                         9.41674727916663e-06,
                                         6.18694269849913e-08,
                                         2.74702911359204e-11,
                                         1.07870333252104e-17,
                                         3.31507354365116e-25,
                                         1.27186726699146e-16,
                                         4.48628146306405e-12,
                                         4.72786229039245e-09,
                                         7.37686212502344e-07,
                                         3.33039379502591e-05,
                                         0.000611787575444583,
                                         0.00554825070042406,
                                         0.0278933879399612,
                                         0.0833600975303983,
                                         0.153942820057062,
                                         0.17825459979209,
                                         0.128415844168612,
                                         0.0556926326506672,
                                         0.0135999233139097,
                                         0.00165697303994743,
                                         8.07348975990426e-05,
                                         1.01100732878211e-06,
                                         1.12310088804776e-09,
                                         2.34667703136342e-15,
                                         7.03314811270251e-28,
                                         1.43580076371897e-18,
                                         1.26711550493913e-13,
                                         2.54514384637949e-10,
                                         6.59412775962856e-08,
                                         4.55913031257841e-06,
                                         0.000121884369116462,
                                         0.00155663602750398,
                                         0.0107960038830774,
                                         0.0440052550802246,
                                         0.11043490928347,
                                         0.174410348455965,
                                         0.173332850568396,
                                         0.105862868204332,
                                         0.037622124121088,
                                         0.00701977222992442,
                                         0.000567946010979248,
                                         1.35555731569355e-05,
                                         3.76755542847654e-08,
                                         4.18880358012016e-13,
                                         1.21235477963466e-30,
                                         1.31695197907922e-20,
                                         2.90783167376317e-15,
                                         1.11322568228999e-11,
                                         4.78923797691416e-09,
                                         5.07097877012892e-07,
                                         1.97296210732941e-05,
                                         0.000354847253761398,
                                         0.00339506557348342,
                                         0.0188744472149904,
                                         0.0643689891606589,
                                         0.138652314544807,
                                         0.1900931718399,
                                         0.163498184481222,
                                         0.0845616666324946,
                                         0.0241631733225333,
                                         0.00324620676183579,
                                         0.000147674271180049,
                                         1.0268899412229e-06,
                                         6.07505255245519e-11,
                                         1.67185913124554e-33,
                                         9.66352746997229e-23,
                                         5.33841469699104e-17,
                                         3.89532849853777e-13,
                                         2.78269408608593e-10,
                                         4.51223350491251e-08,
                                         2.55493268262366e-06,
                                         6.4712146591601e-05,
                                         0.000854128647828291,
                                         0.00647640391170554,
                                         0.0300149059202257,
                                         0.0881803837849027,
                                         0.166779298265311,
                                         0.202009688814095,
                                         0.152052562277462,
                                         0.0665387908201202,
                                         0.0148434648883884,
                                         0.00128700956373217,
                                         2.23912395483703e-05,
                                         7.04855461645373e-09,
                                         1.80119056943063e-36,
                                         5.53976653135306e-25,
                                         7.65676613688974e-19,
                                         1.06486607124803e-14,
                                         1.26314888422482e-11,
                                         3.1367605689712e-09,
                                         2.58481943077202e-07,
                                         9.21977227548139e-06,
                                         0.000167875933480421,
                                         0.00173613559834493,
                                         0.0109342075706218,
                                         0.0438133918710428,
                                         0.114316204705725,
                                         0.194994110830318,
                                         0.213601344659263,
                                         0.143148192936068,
                                         0.0530254583603363,
                                         0.00876291789387843,
                                         0.000381436635443068,
                                         6.38910653346073e-07,
                                         1.46110253960062e-39,
                                         2.39115807264164e-27,
                                         8.26874322670273e-21,
                                         2.19183036681699e-16,
                                         4.3172248559863e-13,
                                         1.64184525522029e-10,
                                         1.96898317862103e-08,
                                         9.89044040959647e-07,
                                         2.48436064502222e-05,
                                         0.000350424454579749,
                                         0.00299915347275746,
                                         0.0163909005596041,
                                         0.0589976211839571,
                                         0.141720226847131,
                                         0.225930711485484,
                                         0.231877112645552,
                                         0.14262465976049,
                                         0.0449238283431274,
                                         0.00489246489290649,
                                         4.36054967644966e-05,
                                         8.39536159832361e-43,
                                         7.31076445430147e-30,
                                         6.32515698400518e-23,
                                         3.19563058493361e-18,
                                         1.04518333022269e-14,
                                         6.08724461482535e-12,
                                         1.06240836555407e-09,
                                         7.5153419635366e-08,
                                         2.60422510322025e-06,
                                         5.01005910828102e-05,
                                         0.000582703659653911,
                                         0.00434346608478659,
                                         0.0215674541882679,
                                         0.0729591662207414,
                                         0.169271653785306,
                                         0.266052642871547,
                                         0.271733067327268,
                                         0.163133237583903,
                                         0.0444498922593897,
                                         0.00210804572517604,
                                         3.04667242887067e-46,
                                         1.41170821261863e-32,
                                         3.05584102307825e-25,
                                         2.94261770483344e-20,
                                         1.59811462255578e-16,
                                         1.42540057995834e-13,
                                         3.62050042389396e-11,
                                         3.60669574744172e-09,
                                         1.72413014829123e-07,
                                         4.52396189648086e-06,
                                         7.15030300997175e-05,
                                         0.000726938547824314,
                                         0.00497955934488302,
                                         0.0237222310049574,
                                         0.0800978483139579,
                                         0.192799061731695,
                                         0.326977603141222,
                                         0.374141425647837,
                                         0.255059410977634,
                                         0.0643645089555649,
                                         5.25177036219587e-50,
                                         1.29485369521254e-35,
                                         7.01267665201419e-28,
                                         1.28707758771343e-22,
                                         1.16069198547605e-18,
                                         1.58542870774192e-15,
                                         5.86056292344384e-13,
                                         8.2217426458051e-11,
                                         5.4219459107863e-09,
                                         1.94038825276865e-07,
                                         4.1676837508227e-06,
                                         5.77899838398234e-05,
                                         0.000546105499984444,
                                         0.00366374137310215,
                                         0.0180032566085457,
                                         0.0663645057482855,
                                         0.186890458911011,
                                         0.40758927498791,
                                         0.695193338278448,
                                         0.933483193802115,

                                         0.939341589404771,
                                         0.718602795514609,
                                         0.442460012239669,
                                         0.218077855702791,
                                         0.0853505843162263,
                                         0.0262322348191943,
                                         0.00623879834589749,
                                         0.00112638389611864,
                                         0.000150612528717074,
                                         1.44467767337479e-05,
                                         9.5367431640625e-07,
                                         4.10216065630283e-08,
                                         1.06841482580087e-09,
                                         1.52296233509395e-11,
                                         1.02883869615308e-13,
                                         2.65866407791553e-16,
                                         1.87358014187711e-19,
                                         2.01488298778691e-23,
                                         1.07252656257161e-28,
                                         1.94879802740493e-36,
                                         7.8339952987001e-51,
                                         0.0588723720656557,
                                         0.239432331466845,
                                         0.368239798441617,
                                         0.345082930095919,
                                         0.223516600278519,
                                         0.104750642580056,
                                         0.0360573048006196,
                                         0.00910471938269587,
                                         0.00166507973851326,
                                         0.000215507824058558,
                                         1.9073486328125e-05,
                                         1.09996932847262e-06,
                                         3.86568054155441e-08,
                                         7.53648817295949e-10,
                                         7.12057342194348e-12,
                                         2.66318940598137e-14,
                                         2.86173214290533e-17,
                                         5.09328422992658e-21,
                                         5.15479443570297e-26,
                                         2.3395532287676e-33,
                                         4.99983087962815e-47,
                                         0.00175264164822703,
                                         0.0378939865138332,
                                         0.145573066644413,
                                         0.259375526338592,
                                         0.278039614200044,
                                         0.198688032810317,
                                         0.0989872327595537,
                                         0.0349574952008464,
                                         0.00874384764422475,
                                         0.00152703408975177,
                                         0.000181198120117188,
                                         1.40101277559109e-05,
                                         6.64363288680658e-07,
                                         1.77150544168815e-08,
                                         2.34086440102447e-10,
                                         1.26716815740174e-12,
                                         2.07624833893039e-15,
                                         6.11560750273312e-19,
                                         1.17681516111407e-23,
                                         1.33411307164789e-30,
                                         1.51572680848938e-43,
                                         3.29535620617742e-05,
                                         0.00378778607439852,
                                         0.0363461975569194,
                                         0.12312951216754,
                                         0.218439521439562,
                                         0.238020503246859,
                                         0.171629821566271,
                                         0.0847699044408535,
                                         0.0289999849409509,
                                         0.00683379690869503,
                                         0.00108718872070312,
                                         0.000112701905975868,
                                         7.211289591524e-06,
                                         2.62992646015619e-07,
                                         4.86032365441716e-09,
                                         3.80797503575756e-11,
                                         9.51387102483592e-14,
                                         4.63776717142825e-17,
                                         1.69680839320657e-21,
                                         4.80485176055601e-28,
                                         2.90211217132959e-40,
                                         4.38883530283689e-07,
                                         0.000268187338008199,
                                         0.00642798144374736,
                                         0.0414031135780117,
                                         0.121560768011497,
                                         0.201973647204082,
                                         0.210787085071693,
                                         0.145606499144682,
                                         0.0681287848206071,
                                         0.021662724835517,
                                         0.00462055206298828,
                                         0.000642181966330829,
                                         5.54444303917405e-05,
                                         2.76555562555541e-06,
                                         7.14811948421217e-08,
                                         8.10572030240169e-10,
                                         3.08796877975532e-12,
                                         2.49124224068641e-15,
                                         1.73298591234089e-19,
                                         1.22575882008646e-25,
                                         3.93590979843042e-37,
                                         4.40105959830397e-09,
                                         1.42972379200291e-05,
                                         0.00085595571152308,
                                         0.010482500538205,
                                         0.0509350459605652,
                                         0.129043488515189,
                                         0.194919951005297,
                                         0.188313248380382,
                                         0.120510409252787,
                                         0.0517041195114578,
                                         0.0147857666015625,
                                         0.00275515476041365,
                                         0.000320969646660992,
                                         2.18968800825916e-05,
                                         7.91552025505608e-07,
                                         1.29912273560361e-08,
                                         7.54656974915231e-11,
                                         1.00759040752848e-13,
                                         1.33265676764849e-17,
                                         2.35445887353849e-23,
                                         4.01917695363208e-34,
                                         3.44790464242433e-11,
                                         5.95464739021317e-07,
                                         8.90467358538028e-05,
                                         0.00207341776449425,
                                         0.0166736238529348,
                                         0.064412108022376,
                                         0.140818145086816,
                                         0.190270307536284,
                                         0.166536146126078,
                                         0.0964111449921949,
                                         0.0369644165039062,
                                         0.00923472404412645,
                                         0.00145164369630135,
                                         0.00013544784887686,
                                         6.84789600154937e-06,
                                         1.62666747525024e-07,
                                         1.44084183651394e-09,
                                         3.18377317938379e-12,
                                         8.0062955202073e-16,
                                         3.5331918586008e-21,
                                         3.20641069423317e-31,
                                         2.16094259261247e-13,
                                         1.98403779751608e-08,
                                         7.41096396411217e-06,
                                         0.000328094329054544,
                                         0.00436649818836951,
                                         0.0257210632330225,
                                         0.0813862301254226,
                                         0.153798164457657,
                                         0.184112148575314,
                                         0.143820012278073,
                                         0.0739288330078125,
                                         0.0247623485682854,
                                         0.00525225844358334,
                                         0.000670273379446569,
                                         4.73941604715948e-05,
                                         1.62943623571181e-06,
                                         2.2007616884024e-08,
                                         8.04804141208703e-11,
                                         3.84799864529253e-14,
                                         4.24163525640071e-19,
                                         2.0464029643287e-28,
                                         1.100410137588e-15,
                                         5.37114903832808e-10,
                                         5.01135037797758e-07,
                                         4.21826638670701e-05,
                                         0.000929094605361603,
                                         0.00834514123580532,
                                         0.0382178819397176,
                                         0.101007740007738,
                                         0.165378767815529,
                                         0.174315004721067,
                                         0.120134353637695,
                                         0.0539489644461439,
                                         0.0154402921310096,
                                         0.00269497785031325,
                                         0.000266511456601851,
                                         1.32617038847894e-05,
                                         2.73119742063236e-07,
                                         1.65295737552677e-09,
                                         1.50266293551755e-12,
                                         4.13735502466217e-17,
                                         1.06117383643505e-25,
                                         4.59781303384011e-18,
                                         1.19308074403082e-11,
                                         2.78048275860492e-08,
                                         4.44994508943885e-06,
                                         0.000162207885816865,
                                         0.00222158961521216,
                                         0.0147254192397554,
                                         0.0544306508298953,
                                         0.121888414735717,
                                         0.173354689240999,
                                         0.160179138183594,
                                         0.0964405690327133,
                                         0.0372434847123595,
                                         0.00889086058455529,
                                         0.00122968054604886,
                                         8.85618447365086e-05,
                                         2.7811123999258e-06,
                                         2.78559823226874e-08,
                                         4.81474853183461e-11,
                                         3.31129313911708e-15,
                                         4.51509905547328e-23,
                                         1.58490041821148e-20,
                                         2.18638388655044e-13,
                                         1.27273970585267e-09,
                                         3.87283728054109e-07,
                                         2.33635018513781e-05,
                                         0.000487919225108519,
                                         0.00468082786497906,
                                         0.0241983831473675,
                                         0.0741137953831561,
                                         0.142229722978772,
                                         0.176197052001953,
                                         0.142229722978772,
                                         0.0741137953831561,
                                         0.0241983831473676,
                                         0.00468082786497906,
                                         0.000487919225108519,
                                         2.33635018513781e-05,
                                         3.87283728054109e-07,
                                         1.27273970585266e-09,
                                         2.18638388655051e-13,
                                         1.5849004182112e-20,
                                         4.51509905547416e-23,
                                         3.31129313911695e-15,
                                         4.81474853183468e-11,
                                         2.78559823226874e-08,
                                         2.7811123999258e-06,
                                         8.85618447365086e-05,
                                         0.00122968054604886,
                                         0.00889086058455527,
                                         0.0372434847123595,
                                         0.0964405690327133,
                                         0.160179138183594,
                                         0.173354689240999,
                                         0.121888414735717,
                                         0.0544306508298954,
                                         0.0147254192397554,
                                         0.00222158961521216,
                                         0.000162207885816865,
                                         4.44994508943885e-06,
                                         2.78048275860489e-08,
                                         1.19308074403086e-11,
                                         4.59781303383938e-18,
                                         1.06117383643528e-25,
                                         4.137355024662e-17,
                                         1.50266293551757e-12,
                                         1.65295737552677e-09,
                                         2.73119742063236e-07,
                                         1.32617038847894e-05,
                                         0.000266511456601851,
                                         0.00269497785031324,
                                         0.0154402921310096,
                                         0.0539489644461439,
                                         0.120134353637695,
                                         0.174315004721067,
                                         0.165378767815529,
                                         0.101007740007739,
                                         0.0382178819397176,
                                         0.00834514123580532,
                                         0.000929094605361603,
                                         4.21826638670701e-05,
                                         5.01135037797753e-07,
                                         5.37114903832823e-10,
                                         1.10041013758784e-15,
                                         2.04640296432917e-28,
                                         4.24163525640052e-19,
                                         3.8479986452926e-14,
                                         8.04804141208703e-11,
                                         2.2007616884024e-08,
                                         1.62943623571181e-06,
                                         4.73941604715948e-05,
                                         0.000670273379446568,
                                         0.00525225844358334,
                                         0.0247623485682854,
                                         0.0739288330078125,
                                         0.143820012278073,
                                         0.184112148575314,
                                         0.153798164457657,
                                         0.0813862301254226,
                                         0.0257210632330225,
                                         0.00436649818836951,
                                         0.000328094329054544,
                                         7.4109639641121e-06,
                                         1.98403779751613e-08,
                                         2.1609425926122e-13,
                                         3.20641069423397e-31,
                                         3.53319185860063e-21,
                                         8.00629552020746e-16,
                                         3.18377317938379e-12,
                                         1.44084183651394e-09,
                                         1.62666747525024e-07,
                                         6.84789600154937e-06,
                                         0.00013544784887686,
                                         0.00145164369630135,
                                         0.00923472404412645,
                                         0.0369644165039062,
                                         0.0964111449921949,
                                         0.166536146126078,
                                         0.190270307536284,
                                         0.140818145086816,
                                         0.064412108022376,
                                         0.0166736238529348,
                                         0.00207341776449425,
                                         8.9046735853802e-05,
                                         5.95464739021329e-07,
                                         3.44790464242396e-11,
                                         4.01917695363315e-34,
                                         2.35445887353837e-23,
                                         1.33265676764852e-17,
                                         1.00759040752848e-13,
                                         7.54656974915231e-11,
                                         1.29912273560361e-08,
                                         7.91552025505608e-07,
                                         2.18968800825915e-05,
                                         0.000320969646660992,
                                         0.00275515476041365,
                                         0.0147857666015625,
                                         0.0517041195114577,
                                         0.120510409252787,
                                         0.188313248380382,
                                         0.194919951005297,
                                         0.129043488515189,
                                         0.0509350459605652,
                                         0.010482500538205,
                                         0.000855955711523074,
                                         1.42972379200294e-05,
                                         4.40105959830358e-09,
                                         3.93590979843154e-37,
                                         1.22575882008639e-25,
                                         1.73298591234093e-19,
                                         2.49124224068641e-15,
                                         3.08796877975532e-12,
                                         8.10572030240169e-10,
                                         7.14811948421217e-08,
                                         2.7655556255554e-06,
                                         5.54444303917405e-05,
                                         0.000642181966330829,
                                         0.00462055206298828,
                                         0.021662724835517,
                                         0.0681287848206071,
                                         0.145606499144682,
                                         0.210787085071693,
                                         0.201973647204082,
                                         0.121560768011497,
                                         0.0414031135780117,
                                         0.00642798144374732,
                                         0.000268187338008203,
                                         4.38883530283658e-07,
                                         2.90211217133047e-40,
                                         4.80485176055574e-28,
                                         1.69680839320661e-21,
                                         4.63776717142825e-17,
                                         9.51387102483592e-14,
                                         3.80797503575756e-11,
                                         4.86032365441716e-09,
                                         2.62992646015618e-07,
                                         7.211289591524e-06,
                                         0.000112701905975868,
                                         0.00108718872070312,
                                         0.00683379690869503,
                                         0.0289999849409509,
                                         0.0847699044408535,
                                         0.171629821566271,
                                         0.238020503246859,
                                         0.218439521439562,
                                         0.12312951216754,
                                         0.0363461975569192,
                                         0.00378778607439855,
                                         3.29535620617724e-05,
                                         1.51572680848987e-43,
                                         1.33411307164781e-30,
                                         1.1768151611141e-23,
                                         6.11560750273312e-19,
                                         2.07624833893039e-15,
                                         1.26716815740174e-12,
                                         2.34086440102447e-10,
                                         1.77150544168814e-08,
                                         6.64363288680658e-07,
                                         1.40101277559109e-05,
                                         0.000181198120117188,
                                         0.00152703408975177,
                                         0.00874384764422475,
                                         0.0349574952008464,
                                         0.0989872327595536,
                                         0.198688032810317,
                                         0.278039614200044,
                                         0.259375526338592,
                                         0.145573066644413,
                                         0.0378939865138334,
                                         0.00175264164822697,
                                         4.99983087962984e-47,
                                         2.33955322876745e-33,
                                         5.15479443570311e-26,
                                         5.09328422992658e-21,
                                         2.86173214290533e-17,
                                         2.66318940598137e-14,
                                         7.12057342194348e-12,
                                         7.53648817295946e-10,
                                         3.86568054155441e-08,
                                         1.09996932847262e-06,
                                         1.9073486328125e-05,
                                         0.000215507824058558,
                                         0.00166507973851326,
                                         0.00910471938269587,
                                         0.0360573048006196,
                                         0.104750642580056,
                                         0.223516600278519,
                                         0.345082930095919,
                                         0.368239798441616,
                                         0.239432331466846,
                                         0.0588723720656547,
                                         7.83399529870288e-51,
                                         1.9487980274048e-36,
                                         1.07252656257164e-28,
                                         2.01488298778691e-23,
                                         1.87358014187711e-19,
                                         2.65866407791553e-16,
                                         1.02883869615308e-13,
                                         1.52296233509394e-11,
                                         1.06841482580087e-09,
                                         4.10216065630283e-08,
                                         9.5367431640625e-07,
                                         1.44467767337479e-05,
                                         0.000150612528717074,
                                         0.00112638389611864,
                                         0.00623879834589749,
                                         0.0262322348191943,
                                         0.0853505843162263,
                                         0.218077855702791,
                                         0.442460012239669,
                                         0.718602795514609,
                                         0.939341589404771,

                                         0.944465915855956,
                                         0.739590536923105,
                                         0.475159623661507,
                                         0.249321892387955,
                                         0.106135432926607,
                                         0.0363217079357188,
                                         0.00987285911043225,
                                         0.0020986486839492,
                                         0.000342007007054795,
                                         4.16615227532259e-05,
                                         3.67247538491828e-06,
                                         2.24643479200538e-07,
                                         9.02473147617314e-09,
                                         2.21189807246095e-10,
                                         2.98860073570662e-12,
                                         1.92717967334472e-14,
                                         4.78615214627052e-17,
                                         3.26302002469368e-20,
                                         3.41693164982738e-24,
                                         1.78237556243129e-29,
                                         3.19368132437006e-37,
                                         1.27392583141398e-51,
                                         0.0540398553647022,
                                         0.224794847726095,
                                         0.360228035620852,
                                         0.358619750525144,
                                         0.251929113562757,
                                         0.130974656915469,
                                         0.0512857721435939,
                                         0.0151581481280977,
                                         0.00335429693198074,
                                         0.000546449575266662,
                                         6.38727530249035e-05,
                                         5.16650752426292e-06,
                                         2.75219579448197e-07,
                                         9.02108137726411e-09,
                                         1.65508951300305e-10,
                                         1.48398065195537e-12,
                                         5.30915596915663e-15,
                                         5.49872205036409e-18,
                                         9.50216337887425e-22,
                                         9.40418643439193e-27,
                                         4.2029726381099e-34,
                                         8.90586785607761e-48,
                                         0.00146870872885431,
                                         0.0324545035322562,
                                         0.129720645048656,
                                         0.245020038198786,
                                         0.284046819638379,
                                         0.224337505621432,
                                         0.126544847606815,
                                         0.0520051268249736,
                                         0.0156264963758303,
                                         0.00340454168074898,
                                         0.000527674353645469,
                                         5.64408993499318e-05,
                                         3.98674056153601e-06,
                                         1.74761474583782e-07,
                                         4.35380209901931e-09,
                                         5.42785054124732e-11,
                                         2.79742258825347e-13,
                                         4.40146654945809e-16,
                                         1.25516928966389e-19,
                                         2.35687663529106e-24,
                                         2.62733321556051e-31,
                                         2.95734478014267e-44,
                                         2.52106950439492e-05,
                                         0.00295931524891241,
                                         0.0295031464070498,
                                         0.10572961419182,
                                         0.202268916728217,
                                         0.242685678911683,
                                         0.197205798676431,
                                         0.112687000239132,
                                         0.0459779253961232,
                                         0.0133966084229041,
                                         0.00275323944768026,
                                         0.000389420158829463,
                                         3.64740733930494e-05,
                                         2.13825967406842e-06,
                                         7.23341740803702e-08,
                                         1.25387766843819e-09,
                                         9.3093276470029e-12,
                                         2.22515715376744e-14,
                                         1.04715208387043e-17,
                                         3.73061229322042e-22,
                                         1.03729287564809e-28,
                                         6.20233639212221e-41,
                                         3.06529051844167e-07,
                                         0.000191137179767164,
                                         0.00475297192446977,
                                         0.0323168759632544,
                                         0.102024868991429,
                                         0.185961949748533,
                                         0.217687040089122,
                                         0.172957403087496,
                                         0.0958241127204409,
                                         0.0373395217222435,
                                         0.0101755913998055,
                                         0.00190318327920575,
                                         0.000236367760401672,
                                         1.85315979868223e-05,
                                         8.51247891643124e-07,
                                         2.05172964899794e-08,
                                         2.19440221994239e-10,
                                         7.9682261541444e-13,
                                         6.18806543611494e-16,
                                         4.18274185873874e-20,
                                         2.90085027399996e-26,
                                         9.21396067638236e-38,
                                         2.8062057674957e-09,
                                         9.29523049860702e-06,
                                         0.000576531726839654,
                                         0.00743743432055174,
                                         0.0387475271511121,
                                         0.107291441738372,
                                         0.180928305557644,
                                         0.199878251631359,
                                         0.150370031339834,
                                         0.0783616706730204,
                                         0.0283162921241067,
                                         0.00700331798069434,
                                         0.00115332912957442,
                                         0.000120927853283126,
                                         7.54274837232881e-06,
                                         2.52782002157326e-07,
                                         3.89471066984478e-09,
                                         2.14844214375644e-11,
                                         2.75334784767378e-14,
                                         3.53104340751692e-18,
                                         6.10815824902269e-24,
                                         1.03061970097744e-34,
                                         2.00704640650829e-11,
                                         3.53154857349562e-07,
                                         5.46350323897929e-05,
                                         0.00133723257063259,
                                         0.0114966673932054,
                                         0.0483610923408359,
                                         0.117481771862711,
                                         0.1804604177551,
                                         0.184347734249873,
                                         0.128477966078251,
                                         0.0615606417966615,
                                         0.0201333993595264,
                                         0.00439651250074405,
                                         0.000616495408868007,
                                         5.22147521292709e-05,
                                         2.43311252676927e-06,
                                         5.40037846923048e-08,
                                         4.52559519264444e-10,
                                         9.57099957237577e-13,
                                         2.32881548732941e-16,
                                         1.00481307394341e-21,
                                         9.0061786177415e-32,
                                         1.14837916008496e-13,
                                         1.07339654063042e-08,
                                         4.1419913253987e-06,
                                         0.000192344926585164,
                                         0.00272891450614699,
                                         0.0174388205769624,
                                         0.0610273408683624,
                                         0.130343194862071,
                                         0.180802447510345,
                                         0.168516956066251,
                                         0.107068046954934,
                                         0.0463041970548925,
                                         0.0134076711832163,
                                         0.00251433613571548,
                                         0.000289165721070388,
                                         1.87356268006361e-05,
                                         5.99050149460071e-07,
                                         7.62636756394158e-09,
                                         2.66160435607322e-11,
                                         1.22873178222165e-14,
                                         1.32236169713404e-19,
                                         6.2961151018354e-29,
                                         5.33871282931389e-16,
                                         2.65080980463827e-10,
                                         2.55135290559333e-07,
                                         2.2479046967133e-05,
                                         0.000526297445887386,
                                         0.00510930097566656,
                                         0.0257573853625151,
                                         0.076492373146338,
                                         0.144076840387991,
                                         0.179589908260516,
                                         0.151300370734351,
                                         0.0865260696478261,
                                         0.0332216928310897,
                                         0.00833182610340176,
                                         0.00130113921451841,
                                         0.000117218896597451,
                                         5.39915175270403e-06,
                                         1.04419865879372e-07,
                                         6.0138566269301e-10,
                                         5.26747487379524e-13,
                                         1.41396485476399e-17,
                                         3.57625214696433e-26,
                                         2.03644736696391e-18,
                                         5.37133595447336e-12,
                                         1.28948785431337e-08,
                                         2.1555615348457e-06,
                                         8.32833078496591e-05,
                                         0.00122826262015746,
                                         0.00891999221934849,
                                         0.0368326765595983,
                                         0.0942040160413999,
                                         0.157038313541622,
                                         0.175430650984138,
                                         0.132665796034175,
                                         0.0675422521151607,
                                         0.0226538712718576,
                                         0.00480381302009164,
                                         0.000601744877934409,
                                         3.99276048625511e-05,
                                         1.17309693609346e-06,
                                         1.11493105632949e-08,
                                         1.85281997785302e-11,
                                         1.24054447772103e-15,
                                         1.66674428790605e-23,
                                         6.40860888959155e-21,
                                         8.97924896588276e-14,
                                         5.37672625096483e-10,
                                         1.70528473394468e-07,
                                         1.08727303770515e-05,
                                         0.00024359868087442,
                                         0.00254847938020722,
                                         0.0146319555213377,
                                         0.0508157814045951,
                                         0.11328783187797,
                                         0.167812731410764,
                                         0.167812731410764,
                                         0.11328783187797,
                                         0.0508157814045951,
                                         0.0146319555213377,
                                         0.00254847938020722,
                                         0.00024359868087442,
                                         1.08727303770514e-05,
                                         1.70528473394467e-07,
                                         5.37672625096491e-10,
                                         8.97924896588276e-14,
                                         6.40860888959155e-21,
                                         1.66674428790605e-23,
                                         1.24054447772103e-15,
                                         1.85281997785299e-11,
                                         1.1149310563295e-08,
                                         1.17309693609347e-06,
                                         3.99276048625511e-05,
                                         0.000601744877934409,
                                         0.00480381302009164,
                                         0.0226538712718576,
                                         0.0675422521151606,
                                         0.132665796034175,
                                         0.175430650984138,
                                         0.157038313541622,
                                         0.0942040160413999,
                                         0.0368326765595983,
                                         0.00891999221934849,
                                         0.00122826262015746,
                                         8.32833078496587e-05,
                                         2.15556153484569e-06,
                                         1.28948785431339e-08,
                                         5.37133595447336e-12,
                                         2.03644736696391e-18,
                                         3.57625214696433e-26,
                                         1.41396485476399e-17,
                                         5.26747487379515e-13,
                                         6.01385662693016e-10,
                                         1.04419865879373e-07,
                                         5.39915175270403e-06,
                                         0.000117218896597451,
                                         0.00130113921451841,
                                         0.00833182610340176,
                                         0.0332216928310897,
                                         0.0865260696478261,
                                         0.151300370734351,
                                         0.179589908260516,
                                         0.144076840387991,
                                         0.076492373146338,
                                         0.0257573853625151,
                                         0.00510930097566656,
                                         0.000526297445887384,
                                         2.24790469671329e-05,
                                         2.55135290559337e-07,
                                         2.65080980463827e-10,
                                         5.33871282931389e-16,
                                         6.2961151018354e-29,
                                         1.32236169713404e-19,
                                         1.22873178222162e-14,
                                         2.66160435607325e-11,
                                         7.62636756394163e-09,
                                         5.99050149460071e-07,
                                         1.87356268006361e-05,
                                         0.000289165721070388,
                                         0.00251433613571548,
                                         0.0134076711832163,
                                         0.0463041970548925,
                                         0.107068046954934,
                                         0.168516956066251,
                                         0.180802447510345,
                                         0.130343194862071,
                                         0.0610273408683624,
                                         0.0174388205769624,
                                         0.00272891450614698,
                                         0.000192344926585163,
                                         4.14199132539874e-06,
                                         1.07339654063042e-08,
                                         1.14837916008496e-13,
                                         9.0061786177415e-32,
                                         1.00481307394341e-21,
                                         2.32881548732936e-16,
                                         9.57099957237588e-13,
                                         4.52559519264447e-10,
                                         5.40037846923048e-08,
                                         2.43311252676927e-06,
                                         5.22147521292708e-05,
                                         0.000616495408868007,
                                         0.00439651250074404,
                                         0.0201333993595264,
                                         0.0615606417966615,
                                         0.128477966078251,
                                         0.184347734249873,
                                         0.1804604177551,
                                         0.117481771862711,
                                         0.0483610923408359,
                                         0.0114966673932054,
                                         0.00133723257063258,
                                         5.46350323897934e-05,
                                         3.53154857349562e-07,
                                         2.00704640650829e-11,
                                         1.03061970097744e-34,
                                         6.1081582490227e-24,
                                         3.53104340751684e-18,
                                         2.75334784767382e-14,
                                         2.14844214375645e-11,
                                         3.89471066984479e-09,
                                         2.52782002157326e-07,
                                         7.54274837232881e-06,
                                         0.000120927853283126,
                                         0.00115332912957442,
                                         0.00700331798069434,
                                         0.0283162921241067,
                                         0.0783616706730205,
                                         0.150370031339834,
                                         0.199878251631359,
                                         0.180928305557644,
                                         0.107291441738372,
                                         0.038747527151112,
                                         0.00743743432055171,
                                         0.000576531726839658,
                                         9.29523049860702e-06,
                                         2.8062057674957e-09,
                                         9.21396067638236e-38,
                                         2.90085027399996e-26,
                                         4.18274185873864e-20,
                                         6.18806543611503e-16,
                                         7.96822615414447e-13,
                                         2.19440221994239e-10,
                                         2.05172964899794e-08,
                                         8.51247891643124e-07,
                                         1.85315979868223e-05,
                                         0.000236367760401672,
                                         0.00190318327920575,
                                         0.0101755913998055,
                                         0.0373395217222435,
                                         0.0958241127204409,
                                         0.172957403087496,
                                         0.217687040089122,
                                         0.185961949748533,
                                         0.102024868991429,
                                         0.0323168759632542,
                                         0.0047529719244698,
                                         0.000191137179767164,
                                         3.06529051844167e-07,
                                         6.20233639212221e-41,
                                         1.03729287564809e-28,
                                         3.73061229322033e-22,
                                         1.04715208387044e-17,
                                         2.22515715376746e-14,
                                         9.3093276470029e-12,
                                         1.25387766843819e-09,
                                         7.23341740803702e-08,
                                         2.13825967406842e-06,
                                         3.64740733930494e-05,
                                         0.000389420158829463,
                                         0.00275323944768026,
                                         0.0133966084229041,
                                         0.0459779253961232,
                                         0.112687000239132,
                                         0.197205798676431,
                                         0.242685678911683,
                                         0.202268916728216,
                                         0.105729614191819,
                                         0.02950314640705,
                                         0.00295931524891241,
                                         2.52106950439492e-05,
                                         2.95734478014267e-44,
                                         2.62733321556051e-31,
                                         2.35687663529099e-24,
                                         1.25516928966391e-19,
                                         4.40146654945813e-16,
                                         2.79742258825347e-13,
                                         5.42785054124732e-11,
                                         4.35380209901931e-09,
                                         1.74761474583782e-07,
                                         3.986740561536e-06,
                                         5.64408993499318e-05,
                                         0.000527674353645469,
                                         0.00340454168074898,
                                         0.0156264963758303,
                                         0.0520051268249736,
                                         0.126544847606815,
                                         0.224337505621432,
                                         0.284046819638379,
                                         0.245020038198785,
                                         0.129720645048657,
                                         0.0324545035322562,
                                         0.00146870872885431,
                                         8.90586785607761e-48,
                                         4.2029726381099e-34,
                                         9.40418643439166e-27,
                                         9.5021633788744e-22,
                                         5.49872205036415e-18,
                                         5.30915596915663e-15,
                                         1.48398065195537e-12,
                                         1.65508951300305e-10,
                                         9.02108137726412e-09,
                                         2.75219579448196e-07,
                                         5.16650752426292e-06,
                                         6.38727530249035e-05,
                                         0.000546449575266662,
                                         0.00335429693198074,
                                         0.0151581481280977,
                                         0.0512857721435939,
                                         0.130974656915469,
                                         0.251929113562757,
                                         0.358619750525143,
                                         0.360228035620852,
                                         0.224794847726095,
                                         0.0540398553647022,
                                         1.27392583141398e-51,
                                         3.19368132437006e-37,
                                         1.78237556243123e-29,
                                         3.41693164982743e-24,
                                         3.26302002469372e-20,
                                         4.78615214627052e-17,
                                         1.92717967334472e-14,
                                         2.98860073570662e-12,
                                         2.21189807246095e-10,
                                         9.02473147617311e-09,
                                         2.24643479200538e-07,
                                         3.67247538491828e-06,
                                         4.16615227532259e-05,
                                         0.000342007007054795,
                                         0.0020986486839492,
                                         0.00987285911043225,
                                         0.0363217079357188,
                                         0.106135432926607,
                                         0.249321892387955,
                                         0.475159623661507,
                                         0.739590536923105,
                                         0.944465915855956,

                                         0.948972774732484,
                                         0.758450678249979,
                                         0.5057166421558,
                                         0.280246044765323,
                                         0.128353338526454,
                                         0.0482176857974275,
                                         0.0147098556873013,
                                         0.00359808101399596,
                                         0.000694347741245921,
                                         0.000103579516887804,
                                         1.16403743312854e-05,
                                         9.5367431640625e-07,
                                         5.4602019438389e-08,
                                         2.06706907451688e-09,
                                         4.80517489264574e-11,
                                         6.19660982042135e-13,
                                         3.83693772220904e-15,
                                         9.2042342854235e-18,
                                         6.09623670446247e-21,
                                         6.23695266420865e-25,
                                         3.19627133711316e-30,
                                         5.65755828772815e-38,
                                         2.24149728891612e-52,
                                         0.0497677460143553,
                                         0.211150650812895,
                                         0.350730925348299,
                                         0.368079080536688,
                                         0.277504415397571,
                                         0.157861208568272,
                                         0.0690966870000483,
                                         0.0233832619027803,
                                         0.00608994865865053,
                                         0.00120589066201435,
                                         0.000178056991106117,
                                         1.9073486328125e-05,
                                         1.4278303627588e-06,
                                         7.1020042813676e-08,
                                         2.19145514684159e-09,
                                         3.81399382835428e-11,
                                         3.26735203236694e-13,
                                         1.12454954780996e-15,
                                         1.12787010231062e-18,
                                         1.8994628143327e-22,
                                         1.84347314844677e-27,
                                         8.12875338824987e-35,
                                         1.70963732310044e-48,
                                         0.00123975480584531,
                                         0.027922295204154,
                                         0.11554056476936,
                                         0.229634104488569,
                                         0.284987778178485,
                                         0.245492423792007,
                                         0.154169919919031,
                                         0.072182656301065,
                                         0.0253713656996478,
                                         0.00666861420001175,
                                         0.00129373320051769,
                                         0.000181198120117188,
                                         1.77352833054143e-05,
                                         1.15904548528778e-06,
                                         4.74733176160787e-08,
                                         1.11506306489124e-09,
                                         1.3216020916288e-11,
                                         6.5262414229731e-14,
                                         9.91174127488555e-17,
                                         2.74778503094182e-20,
                                         5.05037471178519e-25,
                                         5.54769362260001e-32,
                                         6.19388834740417e-45,
                                         1.95052389089178e-05,
                                         0.00233204780757161,
                                         0.0240393408989326,
                                         0.0904813947802165,
                                         0.184846068719684,
                                         0.241117320791627,
                                         0.217254966899451,
                                         0.140730513061878,
                                         0.0667577520595599,
                                         0.0232911481941388,
                                         0.0059368771421502,
                                         0.00108718872070312,
                                         0.000139132084798658,
                                         1.19466922034101e-05,
                                         6.49522536962972e-07,
                                         2.05895341370637e-08,
                                         3.37623875550761e-10,
                                         2.39207791116064e-12,
                                         5.50133985328934e-15,
                                         2.51051231381258e-18,
                                         8.73852297456649e-23,
                                         2.39127010478176e-29,
                                         1.41726283759542e-41,
                                         2.17372413504089e-07,
                                         0.000137962629757597,
                                         0.00354281362284153,
                                         0.0252534003919911,
                                         0.0849242814861016,
                                         0.167747634409487,
                                         0.216859005133101,
                                         0.194348575467199,
                                         0.124422018568864,
                                         0.0576214103786761,
                                         0.0192978568139405,
                                         0.00462055206298828,
                                         0.000773132864772392,
                                         8.72233071325099e-05,
                                         6.29472187618116e-06,
                                         2.69296860068375e-07,
                                         6.10946616448523e-09,
                                         6.21048025946245e-11,
                                         2.16284139190251e-13,
                                         1.62472415836986e-16,
                                         1.07100165688381e-20,
                                         7.30100220855625e-27,
                                         2.29707463556961e-38,
                                         1.82397393962727e-09,
                                         6.1453486475119e-06,
                                         0.00039312902022883,
                                         0.00530690716694623,
                                         0.0293774835714175,
                                         0.0878709755208138,
                                         0.162984481907697,
                                         0.202085661680407,
                                         0.174603855694647,
                                         0.107334149137626,
                                         0.0472303480461352,
                                         0.0147857666015625,
                                         0.0032347597107153,
                                         0.000479488805344279,
                                         4.59324822481896e-05,
                                         2.65202190762079e-06,
                                         8.32404511490077e-08,
                                         1.21404867384074e-09,
                                         6.40238694191403e-12,
                                         7.91695121261751e-15,
                                         9.88333010896642e-19,
                                         1.67840735305628e-24,
                                         2.8032437427153e-35,
                                         1.19570173904307e-11,
                                         2.13856089065063e-07,
                                         3.4080968854998e-05,
                                         0.000871270775711957,
                                         0.00793939360862214,
                                         0.0359604006402999,
                                         0.0956984892615497,
                                         0.164164659421525,
                                         0.191425789560937,
                                         0.156200342559667,
                                         0.0903073799717409,
                                         0.0369644165039062,
                                         0.0105735286411882,
                                         0.00205927537110656,
                                         0.000261850444777937,
                                         2.04038891455454e-05,
                                         8.86044669368087e-07,
                                         1.85411660118893e-08,
                                         1.48063903299615e-10,
                                         3.01388258835718e-13,
                                         7.12535473035389e-17,
                                         3.0144098310957e-22,
                                         2.67261650950471e-32,
                                         6.27071524516536e-14,
                                         5.95369661882026e-09,
                                         2.36362594126327e-06,
                                         0.000114433924050951,
                                         0.00171652471783412,
                                         0.0117731745350165,
                                         0.0449525046298799,
                                         0.10668757072179,
                                         0.167894724950917,
                                         0.181851142152145,
                                         0.138138687767349,
                                         0.0739288330078125,
                                         0.0276495363920075,
                                         0.00707522679450461,
                                         0.00119419899947786,
                                         0.000125585295073886,
                                         7.54513119791001e-06,
                                         2.26531172588248e-07,
                                         2.73934326796733e-09,
                                         9.1787740128305e-12,
                                         4.10960107361514e-15,
                                         4.33109000065336e-20,
                                         2.03846105796233e-29,
                                         2.6719890150367e-16,
                                         1.34671330435026e-10,
                                         1.33189176233824e-07,
                                         1.22118033942272e-05,
                                         0.00030153391019501,
                                         0.00313174226046469,
                                         0.0171563965802897,
                                         0.0563340989566994,
                                         0.119645680808926,
                                         0.172017825666057,
                                         0.171684378004831,
                                         0.120134353637695,
                                         0.0587461122976837,
                                         0.0197510266732836,
                                         0.00442510377209868,
                                         0.000628040997912818,
                                         5.2203707097351e-05,
                                         2.24875567958115e-06,
                                         4.11781745258717e-08,
                                         2.27125760711652e-10,
                                         1.9258223020468e-13,
                                         5.05609807107404e-18,
                                         1.26325655754075e-26,
                                         9.34195192909027e-19,
                                         2.49947600484767e-12,
                                         6.15806812654789e-09,
                                         1.06927690839336e-06,
                                         4.34617919730301e-05,
                                         0.000683539811841771,
                                         0.0053725891436644,
                                         0.0244069914831018,
                                         0.0699588030677735,
                                         0.1335107531575,
                                         0.175078010938214,
                                         0.160179138183594,
                                         0.102413163097161,
                                         0.0452401833831157,
                                         0.013454129007896,
                                         0.00257704837474079,
                                         0.000296361144973041,
                                         1.83164769908339e-05,
                                         5.07894013602004e-07,
                                         4.61140731033107e-09,
                                         7.40488176807861e-12,
                                         4.8430521008467e-16,
                                         6.42341637768198e-24,
                                         2.69460143426127e-21,
                                         3.82716108162631e-14,
                                         2.34895119873105e-10,
                                         7.72421797729798e-08,
                                         5.1681254431231e-06,
                                         0.000123082289121797,
                                         0.00138801810896119,
                                         0.0087239083705611,
                                         0.0337475045250948,
                                         0.0854895412159899,
                                         0.14729444794567,
                                         0.176197052001953,
                                         0.14729444794567,
                                         0.0854895412159899,
                                         0.0337475045250948,
                                         0.00872390837056112,
                                         0.00138801810896119,
                                         0.000123082289121797,
                                         5.1681254431231e-06,
                                         7.72421797729798e-08,
                                         2.34895119873108e-10,
                                         3.82716108162631e-14,
                                         2.6946014342607e-21,
                                         6.42341637768348e-24,
                                         4.8430521008467e-16,
                                         7.40488176807847e-12,
                                         4.61140731033107e-09,
                                         5.07894013602005e-07,
                                         1.83164769908338e-05,
                                         0.000296361144973042,
                                         0.00257704837474079,
                                         0.013454129007896,
                                         0.0452401833831157,
                                         0.102413163097161,
                                         0.160179138183594,
                                         0.175078010938214,
                                         0.1335107531575,
                                         0.0699588030677735,
                                         0.0244069914831018,
                                         0.00537258914366439,
                                         0.000683539811841774,
                                         4.34617919730301e-05,
                                         1.06927690839336e-06,
                                         6.15806812654798e-09,
                                         2.49947600484767e-12,
                                         9.34195192908848e-19,
                                         1.26325655754107e-26,
                                         5.05609807107404e-18,
                                         1.92582230204677e-13,
                                         2.27125760711652e-10,
                                         4.11781745258717e-08,
                                         2.24875567958114e-06,
                                         5.22037070973512e-05,
                                         0.000628040997912816,
                                         0.00442510377209868,
                                         0.0197510266732836,
                                         0.0587461122976837,
                                         0.120134353637695,
                                         0.171684378004831,
                                         0.172017825666057,
                                         0.119645680808926,
                                         0.0563340989566995,
                                         0.0171563965802897,
                                         0.0031317422604647,
                                         0.00030153391019501,
                                         1.22118033942272e-05,
                                         1.33189176233826e-07,
                                         1.34671330435025e-10,
                                         2.67198901503625e-16,
                                         2.0384610579629e-29,
                                         4.33109000065336e-20,
                                         4.10960107361505e-15,
                                         9.1787740128305e-12,
                                         2.73934326796733e-09,
                                         2.26531172588247e-07,
                                         7.54513119791003e-06,
                                         0.000125585295073885,
                                         0.00119419899947786,
                                         0.00707522679450461,
                                         0.0276495363920075,
                                         0.0739288330078125,
                                         0.138138687767349,
                                         0.181851142152145,
                                         0.167894724950917,
                                         0.10668757072179,
                                         0.0449525046298798,
                                         0.0117731745350166,
                                         0.00171652471783412,
                                         0.000114433924050951,
                                         2.3636259412633e-06,
                                         5.95369661882026e-09,
                                         6.27071524516442e-14,
                                         2.67261650950551e-32,
                                         3.0144098310957e-22,
                                         7.12535473035373e-17,
                                         3.01388258835718e-13,
                                         1.48063903299615e-10,
                                         1.85411660118892e-08,
                                         8.86044669368091e-07,
                                         2.04038891455454e-05,
                                         0.000261850444777937,
                                         0.00205927537110656,
                                         0.0105735286411882,
                                         0.0369644165039062,
                                         0.0903073799717409,
                                         0.156200342559667,
                                         0.191425789560937,
                                         0.164164659421525,
                                         0.0956984892615495,
                                         0.0359604006403,
                                         0.00793939360862214,
                                         0.000871270775711957,
                                         3.40809688549984e-05,
                                         2.13856089065063e-07,
                                         1.19570173904292e-11,
                                         2.80324374271619e-35,
                                         1.67840735305628e-24,
                                         9.88333010896617e-19,
                                         7.91695121261751e-15,
                                         6.40238694191403e-12,
                                         1.21404867384073e-09,
                                         8.32404511490081e-08,
                                         2.65202190762078e-06,
                                         4.59324822481896e-05,
                                         0.000479488805344279,
                                         0.0032347597107153,
                                         0.0147857666015625,
                                         0.0472303480461352,
                                         0.107334149137626,
                                         0.174603855694647,
                                         0.202085661680407,
                                         0.162984481907697,
                                         0.087870975520814,
                                         0.0293774835714175,
                                         0.00530690716694623,
                                         0.000393129020228833,
                                         6.1453486475119e-06,
                                         1.82397393962707e-09,
                                         2.29707463557039e-38,
                                         7.30100220855625e-27,
                                         1.07100165688378e-20,
                                         1.62472415836986e-16,
                                         2.16284139190251e-13,
                                         6.21048025946242e-11,
                                         6.10946616448526e-09,
                                         2.69296860068374e-07,
                                         6.29472187618116e-06,
                                         8.72233071325098e-05,
                                         0.000773132864772392,
                                         0.00462055206298828,
                                         0.0192978568139405,
                                         0.0576214103786761,
                                         0.124422018568864,
                                         0.194348575467199,
                                         0.216859005133101,
                                         0.167747634409487,
                                         0.0849242814861016,
                                         0.0252534003919911,
                                         0.00354281362284156,
                                         0.000137962629757597,
                                         2.17372413504071e-07,
                                         1.41726283759593e-41,
                                         2.39127010478176e-29,
                                         8.73852297456624e-23,
                                         2.51051231381258e-18,
                                         5.50133985328934e-15,
                                         2.39207791116063e-12,
                                         3.37623875550763e-10,
                                         2.05895341370636e-08,
                                         6.49522536962972e-07,
                                         1.19466922034101e-05,
                                         0.000139132084798658,
                                         0.00108718872070312,
                                         0.0059368771421502,
                                         0.0232911481941388,
                                         0.0667577520595599,
                                         0.140730513061878,
                                         0.217254966899451,
                                         0.241117320791627,
                                         0.184846068719684,
                                         0.0904813947802165,
                                         0.0240393408989327,
                                         0.00233204780757161,
                                         1.95052389089165e-05,
                                         6.19388834740654e-45,
                                         5.54769362260001e-32,
                                         5.05037471178504e-25,
                                         2.74778503094182e-20,
                                         9.91174127488555e-17,
                                         6.52624142297305e-14,
                                         1.3216020916288e-11,
                                         1.11506306489123e-09,
                                         4.74733176160787e-08,
                                         1.15904548528778e-06,
                                         1.77352833054143e-05,
                                         0.000181198120117188,
                                         0.00129373320051769,
                                         0.00666861420001175,
                                         0.0253713656996478,
                                         0.072182656301065,
                                         0.154169919919031,
                                         0.245492423792007,
                                         0.284987778178485,
                                         0.229634104488569,
                                         0.11554056476936,
                                         0.027922295204154,
                                         0.00123975480584526,
                                         1.70963732310113e-48,
                                         8.12875338824988e-35,
                                         1.84347314844671e-27,
                                         1.8994628143327e-22,
                                         1.12787010231062e-18,
                                         1.12454954780995e-15,
                                         3.26735203236696e-13,
                                         3.81399382835427e-11,
                                         2.19145514684159e-09,
                                         7.1020042813676e-08,
                                         1.4278303627588e-06,
                                         1.9073486328125e-05,
                                         0.000178056991106117,
                                         0.00120589066201435,
                                         0.00608994865865053,
                                         0.0233832619027803,
                                         0.0690966870000482,
                                         0.157861208568272,
                                         0.277504415397571,
                                         0.368079080536688,
                                         0.3507309253483,
                                         0.211150650812895,
                                         0.0497677460143543,
                                         2.24149728891708e-52,
                                         5.65755828772815e-38,
                                         3.19627133711306e-30,
                                         6.23695266420865e-25,
                                         6.09623670446247e-21,
                                         9.20423428542343e-18,
                                         3.83693772220906e-15,
                                         6.19660982042132e-13,
                                         4.80517489264574e-11,
                                         2.06706907451688e-09,
                                         5.4602019438389e-08,
                                         9.5367431640625e-07,
                                         1.16403743312854e-05,
                                         0.000103579516887804,
                                         0.000694347741245921,
                                         0.00359808101399596,
                                         0.0147098556873013,
                                         0.0482176857974275,
                                         0.128353338526454,
                                         0.280246044765323,
                                         0.5057166421558,
                                         0.758450678249979,
                                         0.948972774732484,

                                         0.952956711057011,
                                         0.775440027493346,
                                         0.534202558718257,
                                         0.310561299445429,
                                         0.15164822256624,
                                         0.0618023488349812,
                                         0.0208467240890532,
                                         0.00575889186147732,
                                         0.00128567819140045,
                                         0.000228137694447208,
                                         3.15139009945227e-05,
                                         3.30139184668633e-06,
                                         2.53740090975848e-07,
                                         1.37117913354748e-08,
                                         4.9279956474654e-10,
                                         1.09369690750897e-11,
                                         1.35387465505119e-13,
                                         8.08988056970795e-16,
                                         1.88242270990682e-18,
                                         1.21549829839741e-21,
                                         1.2183706646699e-25,
                                         6.1473097996727e-31,
                                         1.07646833016865e-38,
                                         4.23955514460259e-53,
                                         0.0459743422242519,
                                         0.19847273887996,
                                         0.3402396448336,
                                         0.373989092279722,
                                         0.29995988029545,
                                         0.184595277090714,
                                         0.0890252144911087,
                                         0.0338790244252076,
                                         0.0101543121126254,
                                         0.00237657052902587,
                                         0.00042801773396203,
                                         5.80780024750922e-05,
                                         5.76945095785189e-06,
                                         4.03826290657424e-07,
                                         1.8922418695808e-08,
                                         5.53909411643183e-10,
                                         9.20548081264524e-12,
                                         7.57751650986195e-14,
                                         2.52093437721878e-16,
                                         2.45803747224728e-19,
                                         4.04694986711613e-23,
                                         3.86070074323647e-28,
                                         1.68231997251081e-35,
                                         3.5151019733909e-49,
                                         0.00105354110659328,
                                         0.0241294331913426,
                                         0.102933674994499,
                                         0.213926283352553,
                                         0.281827019964516,
                                         0.261896563142151,
                                         0.180585072797653,
                                         0.0946708939351318,
                                         0.0380945062348303,
                                         0.0117597469336752,
                                         0.00276130875683942,
                                         0.00048531071162278,
                                         6.2312258216056e-05,
                                         5.64922137394323e-06,
                                         3.4512513521541e-07,
                                         1.3325211605262e-08,
                                         2.97309033897739e-10,
                                         3.37136118168542e-12,
                                         1.60361288560689e-14,
                                         2.36111017670324e-17,
                                         6.385131190687e-21,
                                         1.15170376789366e-25,
                                         1.24884791779805e-32,
                                         1.38436043273475e-45,
                                         1.52480775316999e-05,
                                         0.00185276792142943,
                                         0.0196678861510376,
                                         0.0772853185525659,
                                         0.167235984191613,
                                         0.23467490906252,
                                         0.23135469301115,
                                         0.167082015349484,
                                         0.090261352025455,
                                         0.036751315549709,
                                         0.0112511216914466,
                                         0.00256127215608537,
                                         0.000425050116993248,
                                         4.99126056558343e-05,
                                         3.97561368445132e-06,
                                         2.02459019577078e-07,
                                         6.06453321973498e-09,
                                         9.47351872310194e-11,
                                         6.44265949906256e-13,
                                         1.43242420777378e-15,
                                         6.36267104247889e-19,
                                         2.1699167949525e-23,
                                         5.85515171006757e-30,
                                         3.44340473682859e-42,
                                         1.56320659056412e-07,
                                         0.00010077031240706,
                                         0.00266192351385671,
                                         0.0197773243608359,
                                         0.0702933938291002,
                                         0.14895023643542,
                                         0.20994834630527,
                                         0.2088721859568,
                                         0.151488267896049,
                                         0.0813552311056085,
                                         0.0324723974189983,
                                         0.00957479003547739,
                                         0.00205373530638374,
                                         0.000312370182349203,
                                         3.24391483261374e-05,
                                         2.17890263515818e-06,
                                         8.76242538149475e-08,
                                         1.88562372229728e-09,
                                         1.83344759222544e-11,
                                         6.15551976909114e-14,
                                         4.4910386787575e-17,
                                         2.89589662292535e-21,
                                         1.94448425227396e-27,
                                         6.06686941014698e-39,
                                         1.20664276020826e-09,
                                         4.12672221027745e-06,
                                         0.000271265465474937,
                                         0.00381065050855397,
                                         0.0222464307373762,
                                         0.0711830813833688,
                                         0.143452344700584,
                                         0.196604098426996,
                                         0.191433179955169,
                                         0.135599823673974,
                                         0.0705657453807947,
                                         0.0269503145438296,
                                         0.00747153519809703,
                                         0.00147193945956522,
                                         0.00019929470419181,
                                         1.76562946224969e-05,
                                         9.53262116792172e-07,
                                         2.82591956980859e-08,
                                         3.92855529397977e-10,
                                         1.99167676644667e-12,
                                         2.38679527158646e-15,
                                         2.90994027860686e-19,
                                         4.8621881047937e-25,
                                         8.04826503103747e-36,
                                         7.27664312509778e-12,
                                         1.3202856291771e-07,
                                         2.15965049498881e-05,
                                         0.000573615308489115,
                                         0.00550042442639709,
                                         0.0265767824359297,
                                         0.0765760348104769,
                                         0.144575449863445,
                                         0.188992886301098,
                                         0.176572390560848,
                                         0.11980185521335,
                                         0.0592636570978422,
                                         0.0212356351234756,
                                         0.00541876547605673,
                                         0.000956559752268203,
                                         0.000111776714588381,
                                         8.10196506450235e-06,
                                         3.30867866516286e-07,
                                         6.5763856075455e-09,
                                         5.03457731181616e-11,
                                         9.90999813036908e-14,
                                         2.28441565548506e-17,
                                         9.49837065517451e-23,
                                         8.34122665233803e-33,
                                         3.51053597078851e-14,
                                         3.37925172335623e-09,
                                         1.37550579904137e-06,
                                         6.90768195917125e-05,
                                         0.00108798284912015,
                                         0.00793812631788395,
                                         0.0327015309204436,
                                         0.0850523905471034,
                                         0.149266960224071,
                                         0.183940115930146,
                                         0.162713332766317,
                                         0.104256476766495,
                                         0.0482848236289911,
                                         0.0159588190091337,
                                         0.00367297892182328,
                                         0.000566099929404347,
                                         5.50881750151224e-05,
                                         3.09912698896675e-06,
                                         8.80707423930774e-08,
                                         1.01811575595674e-09,
                                         3.29171300489939e-12,
                                         1.43468370822335e-15,
                                         1.48441883627056e-20,
                                         6.91588180034927e-30,
                                         1.37606563282612e-16,
                                         7.02743053505046e-11,
                                         7.11811086406863e-08,
                                         6.75876433928835e-06,
                                         0.000174852266466536,
                                         0.00192644661905018,
                                         0.0113466240232964,
                                         0.0406538674442673,
                                         0.0957867171808153,
                                         0.155687406674987,
                                         0.179558557318482,
                                         0.149018782379172,
                                         0.0892029904310777,
                                         0.0381877889067996,
                                         0.011459037429849,
                                         0.00232947594802762,
                                         0.000304333509142336,
                                         2.35856322595816e-05,
                                         9.58295423115742e-07,
                                         1.67284103116626e-08,
                                         8.88369671801735e-11,
                                         7.32083426621907e-14,
                                         1.88489503220531e-18,
                                         4.65895560021547e-27,
                                         4.4257842699842e-19,
                                         1.1991070326773e-12,
                                         3.02240348441088e-09,
                                         5.42609815345471e-07,
                                         2.3057162174906e-05,
                                         0.000383602407942522,
                                         0.00323035770707131,
                                         0.0159441943722621,
                                         0.0504350274173611,
                                         0.108122451615833,
                                         0.162582742623762,
                                         0.174768978535314,
                                         0.135217701600761,
                                         0.0749779649030694,
                                         0.0293334544928101,
                                         0.00786518119081571,
                                         0.00137951533824419,
                                         0.000147279073984289,
                                         8.55564078124786e-06,
                                         2.25526488197795e-07,
                                         1.9672105977273e-09,
                                         3.06514027693955e-12,
                                         1.96382712486524e-16,
                                         2.57522365211789e-24,
                                         1.17434386108999e-21,
                                         1.68800328460282e-14,
                                         1.0587516087548e-10,
                                         3.59386645940823e-08,
                                         2.50838613133373e-06,
                                         6.301727485591e-05,
                                         0.000758732199791154,
                                         0.00515890157280632,
                                         0.0219085271319543,
                                         0.0619487051284381,
                                         0.121449779892966,
                                         0.169099231524795,
                                         0.169099231524795,
                                         0.121449779892966,
                                         0.0619487051284381,
                                         0.0219085271319543,
                                         0.00515890157280631,
                                         0.000758732199791154,
                                         6.30172748559097e-05,
                                         2.50838613133373e-06,
                                         3.59386645940823e-08,
                                         1.05875160875478e-10,
                                         1.68800328460282e-14,
                                         1.17434386108972e-21,
                                         2.57522365211854e-24,
                                         1.96382712486524e-16,
                                         3.06514027693961e-12,
                                         1.9672105977273e-09,
                                         2.25526488197794e-07,
                                         8.5556407812479e-06,
                                         0.000147279073984289,
                                         0.00137951533824419,
                                         0.00786518119081571,
                                         0.0293334544928101,
                                         0.0749779649030694,
                                         0.135217701600761,
                                         0.174768978535314,
                                         0.162582742623762,
                                         0.108122451615833,
                                         0.0504350274173611,
                                         0.015944194372262,
                                         0.00323035770707131,
                                         0.000383602407942521,
                                         2.3057162174906e-05,
                                         5.42609815345471e-07,
                                         3.02240348441083e-09,
                                         1.1991070326773e-12,
                                         4.42578426998328e-19,
                                         4.65895560021676e-27,
                                         1.88489503220531e-18,
                                         7.32083426621923e-14,
                                         8.88369671801735e-11,
                                         1.67284103116626e-08,
                                         9.58295423115747e-07,
                                         2.35856322595816e-05,
                                         0.000304333509142337,
                                         0.00232947594802762,
                                         0.011459037429849,
                                         0.0381877889067996,
                                         0.0892029904310776,
                                         0.149018782379172,
                                         0.179558557318482,
                                         0.155687406674987,
                                         0.0957867171808153,
                                         0.0406538674442672,
                                         0.0113466240232964,
                                         0.00192644661905017,
                                         0.000174852266466536,
                                         6.75876433928835e-06,
                                         7.11811086406853e-08,
                                         7.02743053505046e-11,
                                         1.37606563282587e-16,
                                         6.91588180035134e-30,
                                         1.48441883627056e-20,
                                         1.43468370822339e-15,
                                         3.29171300489939e-12,
                                         1.01811575595674e-09,
                                         8.80707423930779e-08,
                                         3.09912698896675e-06,
                                         5.50881750151226e-05,
                                         0.000566099929404347,
                                         0.00367297892182328,
                                         0.0159588190091337,
                                         0.048284823628991,
                                         0.104256476766495,
                                         0.162713332766317,
                                         0.183940115930146,
                                         0.149266960224071,
                                         0.0850523905471032,
                                         0.0327015309204436,
                                         0.00793812631788393,
                                         0.00108798284912015,
                                         6.90768195917125e-05,
                                         1.37550579904135e-06,
                                         3.37925172335623e-09,
                                         3.51053597078795e-14,
                                         8.34122665234073e-33,
                                         9.49837065517451e-23,
                                         2.28441565548512e-17,
                                         9.90999813036908e-14,
                                         5.03457731181617e-11,
                                         6.57638560754554e-09,
                                         3.30867866516286e-07,
                                         8.10196506450237e-06,
                                         0.000111776714588381,
                                         0.000956559752268203,
                                         0.00541876547605673,
                                         0.0212356351234756,
                                         0.0592636570978423,
                                         0.11980185521335,
                                         0.176572390560848,
                                         0.188992886301098,
                                         0.144575449863445,
                                         0.0765760348104769,
                                         0.0265767824359297,
                                         0.00550042442639709,
                                         0.000573615308489115,
                                         2.15965049498878e-05,
                                         1.3202856291771e-07,
                                         7.27664312509678e-12,
                                         8.04826503104025e-36,
                                         4.8621881047937e-25,
                                         2.90994027860694e-19,
                                         2.38679527158646e-15,
                                         1.99167676644667e-12,
                                         3.92855529397979e-10,
                                         2.82591956980859e-08,
                                         9.53262116792176e-07,
                                         1.76562946224969e-05,
                                         0.00019929470419181,
                                         0.00147193945956522,
                                         0.00747153519809701,
                                         0.0269503145438296,
                                         0.0705657453807947,
                                         0.135599823673974,
                                         0.191433179955169,
                                         0.196604098426996,
                                         0.143452344700584,
                                         0.0711830813833687,
                                         0.0222464307373762,
                                         0.00381065050855397,
                                         0.000271265465474935,
                                         4.12672221027745e-06,
                                         1.20664276020812e-09,
                                         6.06686941014922e-39,
                                         1.94448425227396e-27,
                                         2.89589662292544e-21,
                                         4.4910386787575e-17,
                                         6.15551976909114e-14,
                                         1.83344759222546e-11,
                                         1.88562372229728e-09,
                                         8.76242538149479e-08,
                                         2.17890263515818e-06,
                                         3.24391483261374e-05,
                                         0.000312370182349203,
                                         0.00205373530638374,
                                         0.0095747900354774,
                                         0.0324723974189983,
                                         0.0813552311056085,
                                         0.151488267896049,
                                         0.2088721859568,
                                         0.20994834630527,
                                         0.14895023643542,
                                         0.0702933938291002,
                                         0.0197773243608359,
                                         0.00266192351385669,
                                         0.00010077031240706,
                                         1.56320659056398e-07,
                                         3.44340473682994e-42,
                                         5.85515171006757e-30,
                                         2.16991679495256e-23,
                                         6.36267104247889e-19,
                                         1.43242420777378e-15,
                                         6.44265949906261e-13,
                                         9.47351872310194e-11,
                                         6.06453321973501e-09,
                                         2.02459019577078e-07,
                                         3.97561368445132e-06,
                                         4.99126056558343e-05,
                                         0.000425050116993247,
                                         0.00256127215608537,
                                         0.0112511216914466,
                                         0.036751315549709,
                                         0.090261352025455,
                                         0.167082015349484,
                                         0.23135469301115,
                                         0.23467490906252,
                                         0.167235984191613,
                                         0.0772853185525659,
                                         0.0196678861510375,
                                         0.00185276792142943,
                                         1.52480775316988e-05,
                                         1.38436043273532e-45,
                                         1.24884791779805e-32,
                                         1.1517037678937e-25,
                                         6.385131190687e-21,
                                         2.36111017670324e-17,
                                         1.60361288560691e-14,
                                         3.37136118168542e-12,
                                         2.9730903389774e-10,
                                         1.3325211605262e-08,
                                         3.4512513521541e-07,
                                         5.64922137394323e-06,
                                         6.23122582160559e-05,
                                         0.00048531071162278,
                                         0.00276130875683942,
                                         0.0117597469336752,
                                         0.0380945062348304,
                                         0.0946708939351318,
                                         0.180585072797653,
                                         0.261896563142151,
                                         0.281827019964516,
                                         0.213926283352553,
                                         0.102933674994498,
                                         0.0241294331913425,
                                         0.00105354110659323,
                                         3.51510197339244e-49,
                                         1.68231997251081e-35,
                                         3.8607007432366e-28,
                                         4.04694986711613e-23,
                                         2.45803747224728e-19,
                                         2.5209343772188e-16,
                                         7.57751650986195e-14,
                                         9.20548081264529e-12,
                                         5.53909411643183e-10,
                                         1.8922418695808e-08,
                                         4.03826290657424e-07,
                                         5.76945095785188e-06,
                                         5.80780024750922e-05,
                                         0.00042801773396203,
                                         0.00237657052902587,
                                         0.0101543121126254,
                                         0.0338790244252076,
                                         0.0890252144911087,
                                         0.184595277090714,
                                         0.29995988029545,
                                         0.373989092279722,
                                         0.340239644833599,
                                         0.19847273887996,
                                         0.0459743422242508,
                                         4.23955514460454e-53,
                                         1.07646833016865e-38,
                                         6.14730979967293e-31,
                                         1.2183706646699e-25,
                                         1.21549829839741e-21,
                                         1.88242270990684e-18,
                                         8.08988056970795e-16,
                                         1.3538746550512e-13,
                                         1.09369690750897e-11,
                                         4.9279956474654e-10,
                                         1.37117913354748e-08,
                                         2.53740090975847e-07,
                                         3.30139184668633e-06,
                                         3.15139009945227e-05,
                                         0.000228137694447208,
                                         0.00128567819140045,
                                         0.00575889186147732,
                                         0.0208467240890532,
                                         0.0618023488349812,
                                         0.15164822256624,
                                         0.310561299445429,
                                         0.534202558718257,
                                         0.775440027493346,
                                         0.952956711057011,

                                         4.76837158203125e-07,
                                         1.00135803222656e-05,
                                         0.000100135803222656,
                                         0.000634193420410156,
                                         0.0028538703918457,
                                         0.00970315933227539,
                                         0.0258750915527344,
                                         0.0554466247558594,
                                         0.0970315933227539,
                                         0.140156745910645,
                                         0.168188095092773,
                                         0.168188095092773,
                                         0.140156745910645,
                                         0.0970315933227539,
                                         0.0554466247558594,
                                         0.0258750915527344,
                                         0.00970315933227539,
                                         0.0028538703918457,
                                         0.000634193420410156,
                                         0.000100135803222656,
                                         1.00135803222656e-05,
                                         4.76837158203125e-07,

                                         0.00683692979029463,
                                         6.66749382077383e-15,
                                         0.0384709461363723,
                                         5.22551939663869e-13,
                                         0.103082589493022,
                                         1.95019038841923e-11,
                                         0.174932345186306,
                                         4.60953275554159e-10,
                                         0.210928412802389,
                                         7.74135469927539e-09,
                                         0.192161532762007,
                                         9.82298387941667e-08,
                                         0.137305406719751,
                                         9.77596664531015e-07,
                                         0.0788375846721085,
                                         7.81808661715633e-06,
                                         0.0369678175056955,
                                         5.1060618828103e-05,
                                         0.0143079398895221,
                                         0.000275254523169006,
                                         0.00460056112650059,
                                         0.00123271663857585,
                                         0.00123271663857585,
                                         0.00460056112650059,
                                         0.000275254523169006,
                                         0.0143079398895221,
                                         5.1060618828103e-05,
                                         0.0369678175056955,
                                         7.81808661715633e-06,
                                         0.0788375846721085,
                                         9.77596664531015e-07,
                                         0.137305406719751,
                                         9.82298387941667e-08,
                                         0.192161532762007,
                                         7.74135469927539e-09,
                                         0.210928412802389,
                                         4.60953275554159e-10,
                                         0.174932345186306,
                                         1.95019038841923e-11,
                                         0.103082589493022,
                                         5.22551939663869e-13,
                                         0.0384709461363723,
                                         6.66749382077383e-15,
                                         0.00683692979029463,

                                         0.0811826070999996,
                                         4.76837158203125e-07,
                                         1.23179094109191e-20,
                                         0.216542405099999,
                                         1.00135803222656e-05,
                                         2.03655260869741e-18,
                                         0.275044917,
                                         0.000100135803222656,
                                         1.60337447719499e-16,
                                         0.221256804999992,
                                         0.000634193420410156,
                                         7.99478235257045e-15,
                                         0.126464845499717,
                                         0.0028538703918457,
                                         2.8324254743253e-13,
                                         0.0546146810924184,
                                         0.00970315933227539,
                                         7.58187712017047e-12,
                                         0.0184985974408215,
                                         0.0258750915527344,
                                         1.59178646133581e-10,
                                         0.00503492131454827,
                                         0.0554466247558594,
                                         2.68545177874613e-09,
                                         0.00111915800059503,
                                         0.0970315933227539,
                                         3.69994049794448e-08,
                                         0.00020533023954557,
                                         0.140156745910645,
                                         4.20760454432864e-07,
                                         3.12964319394095e-05,
                                         0.168188095092773,
                                         3.97516806059112e-06,
                                         3.97516806059112e-06,
                                         0.168188095092773,
                                         3.12964319394095e-05,
                                         4.20760454432864e-07,
                                         0.140156745910645,
                                         0.00020533023954557,
                                         3.69994049794448e-08,
                                         0.0970315933227539,
                                         0.00111915800059503,
                                         2.68545177874613e-09,
                                         0.0554466247558594,
                                         0.00503492131454827,
                                         1.59178646133581e-10,
                                         0.0258750915527344,
                                         0.0184985974408215,
                                         7.58187712017047e-12,
                                         0.00970315933227539,
                                         0.0546146810924184,
                                         2.8324254743253e-13,
                                         0.0028538703918457,
                                         0.126464845499717,
                                         7.99478235257045e-15,
                                         0.000634193420410156,
                                         0.221256804999992,
                                         1.60337447719499e-16,
                                         0.000100135803222656,
                                         0.275044917,
                                         2.03655260869741e-18,
                                         1.00135803222656e-05,
                                         0.216542405099999,
                                         1.23179094109191e-20,
                                         4.76837158203125e-07,
                                         0.0811826070999996,

                                         0.220654060203124,
                                         0.000222557296135072,
                                         7.74559316667436e-11,
                                         4.70687557431301e-25,
                                         0.345733714335791,
                                         0.00230207191053836,
                                         3.30229770205042e-09,
                                         1.32477309271739e-22,
                                         0.25795992738348,
                                         0.0113390492146162,
                                         6.70437762129686e-08,
                                         1.77554214192505e-20,
                                         0.12189758631698,
                                         0.0353725802910442,
                                         8.6205119742304e-07,
                                         1.50713729484597e-18,
                                         0.0409277748880578,
                                         0.0784037814454899,
                                         7.87567559027761e-06,
                                         9.08981023180067e-17,
                                         0.0103826344845978,
                                         0.131302707356755,
                                         5.43636967377911e-05,
                                         4.14212517895413e-15,
                                         0.00206579304757055,
                                         0.172464680350483,
                                         0.000294320124716808,
                                         1.48040804930627e-13,
                                         0.000330286288330538,
                                         0.182033425379449,
                                         0.00128042876874798,
                                         4.25171497546801e-12,
                                         4.31259929206153e-05,
                                         0.156908671243465,
                                         0.0045492071961516,
                                         9.97221601406236e-11,
                                         4.64783245444942e-06,
                                         0.111636327763429,
                                         0.0133407057664591,
                                         1.93055423287934e-09,
                                         4.16142645993458e-07,
                                         0.0659848969367344,
                                         0.0325014171087312,
                                         3.10493661134192e-08,
                                         3.10493661134189e-08,
                                         0.0325014171087312,
                                         0.0659848969367344,
                                         4.16142645993461e-07,
                                         1.93055423287932e-09,
                                         0.0133407057664591,
                                         0.111636327763429,
                                         4.64783245444945e-06,
                                         9.97221601406226e-11,
                                         0.0045492071961516,
                                         0.156908671243465,
                                         4.31259929206155e-05,
                                         4.25171497546796e-12,
                                         0.00128042876874798,
                                         0.182033425379449,
                                         0.00033028628833054,
                                         1.48040804930625e-13,
                                         0.000294320124716808,
                                         0.172464680350483,
                                         0.00206579304757056,
                                         4.14212517895408e-15,
                                         5.43636967377911e-05,
                                         0.131302707356755,
                                         0.0103826344845979,
                                         9.08981023180054e-17,
                                         7.87567559027761e-06,
                                         0.0784037814454899,
                                         0.0409277748880579,
                                         1.50713729484594e-18,
                                         8.6205119742304e-07,
                                         0.0353725802910442,
                                         0.12189758631698,
                                         1.77554214192502e-20,
                                         6.70437762129686e-08,
                                         0.0113390492146162,
                                         0.25795992738348,
                                         1.32477309271736e-22,
                                         3.30229770205042e-09,
                                         0.00230207191053836,
                                         0.345733714335791,
                                         4.70687557431293e-25,
                                         7.74559316667436e-11,
                                         0.000222557296135072,
                                         0.220654060203124,

                                         0.364595608368846,
                                         0.00404795496970863,
                                         4.76837158203125e-07,
                                         4.23235783035854e-14,
                                         1.24907658750036e-28,
                                         0.376845207246724,
                                         0.0255015580673174,
                                         1.00135803222656e-05,
                                         2.96271533515019e-12,
                                         5.32937271953096e-26,
                                         0.185479221577613,
                                         0.0765029995130712,
                                         0.000100135803222656,
                                         9.87593396823027e-11,
                                         1.08278897760324e-23,
                                         0.0578175758935913,
                                         0.145352517235613,
                                         0.000634193420410156,
                                         2.08496503322023e-09,
                                         1.39329764825323e-21,
                                         0.0128057394490148,
                                         0.196221602879156,
                                         0.0028538703918457,
                                         3.12751601121971e-08,
                                         1.27386718278141e-19,
                                         0.00214296879269885,
                                         0.200141653735946,
                                         0.00970315933227539,
                                         3.54459573732211e-07,
                                         8.79975940494892e-18,
                                         0.000281265747591809,
                                         0.160109818104846,
                                         0.0258750915527344,
                                         3.15082073803886e-06,
                                         4.76768179705658e-16,
                                         2.96648621521325e-05,
                                         0.102925487119922,
                                         0.0554466247558594,
                                         2.25063550791358e-05,
                                         2.07571733790416e-14,
                                         2.5551279438242e-06,
                                         0.0540346978914259,
                                         0.0970315933227539,
                                         0.000131289945234542,
                                         7.38029961812398e-13,
                                         1.81654333521105e-07,
                                         0.0234145231973403,
                                         0.140156745910645,
                                         0.000632150611141289,
                                         2.16592077905154e-11,
                                         1.07290007875203e-08,
                                         0.00842904383506169,
                                         0.168188095092773,
                                         0.00252865779693599,
                                         5.28070060626248e-10,
                                         5.28070060626241e-10,
                                         0.00252865779693599,
                                         0.168188095092773,
                                         0.00842904383506169,
                                         1.07290007875204e-08,
                                         2.1659207790515e-11,
                                         0.000632150611141289,
                                         0.140156745910645,
                                         0.0234145231973403,
                                         1.81654333521106e-07,
                                         7.38029961812387e-13,
                                         0.000131289945234542,
                                         0.0970315933227539,
                                         0.0540346978914259,
                                         2.55512794382423e-06,
                                         2.07571733790412e-14,
                                         2.25063550791358e-05,
                                         0.0554466247558594,
                                         0.102925487119922,
                                         2.96648621521327e-05,
                                         4.76768179705649e-16,
                                         3.15082073803886e-06,
                                         0.0258750915527344,
                                         0.160109818104846,
                                         0.000281265747591811,
                                         8.79975940494876e-18,
                                         3.54459573732211e-07,
                                         0.00970315933227539,
                                         0.200141653735946,
                                         0.00214296879269886,
                                         1.27386718278138e-19,
                                         3.12751601121971e-08,
                                         0.0028538703918457,
                                         0.196221602879156,
                                         0.0128057394490149,
                                         1.3932976482532e-21,
                                         2.08496503322023e-09,
                                         0.000634193420410156,
                                         0.145352517235613,
                                         0.0578175758935915,
                                         1.08278897760321e-23,
                                         9.87593396823027e-11,
                                         0.000100135803222656,
                                         0.0765029995130712,
                                         0.185479221577614,
                                         5.32937271953083e-26,
                                         2.96271533515019e-12,
                                         1.00135803222656e-05,
                                         0.0255015580673174,
                                         0.376845207246725,
                                         1.24907658750033e-28,
                                         4.23235783035854e-14,
                                         4.76837158203125e-07,
                                         0.00404795496970863,
                                         0.364595608368846,

                                         0.486111360292378,
                                         0.0202899975649854,
                                         4.26642339365132e-05,
                                         1.55600359446431e-09,
                                         6.41104532609661e-17,
                                         1.25275874674036e-31,
                                         0.356732179981835,
                                         0.0868977001770381,
                                         0.000550740954135588,
                                         5.31576489455627e-08,
                                         6.60147752605754e-15,
                                         7.52834504614831e-29,
                                         0.12466068538978,
                                         0.177221037862509,
                                         0.00338541176035768,
                                         8.64772038742807e-07,
                                         3.23693632372165e-13,
                                         2.15433031799143e-26,
                                         0.0275898334683625,
                                         0.228904649852445,
                                         0.0131797742248275,
                                         8.90983905589156e-06,
                                         1.00521627028161e-11,
                                         3.90442798608796e-24,
                                         0.00433859204280081,
                                         0.210074878749274,
                                         0.0364573090961241,
                                         6.52256246679729e-05,
                                         2.21801781575736e-10,
                                         5.02785458155395e-22,
                                         0.000515483721208749,
                                         0.145666616125387,
                                         0.0761952070079273,
                                         0.000360772177927849,
                                         3.69774496404794e-09,
                                         4.89186431765915e-20,
                                         4.8036450573035e-05,
                                         0.0792201675888377,
                                         0.124899445834696,
                                         0.00156508397451435,
                                         4.83502466136902e-08,
                                         3.73298214377158e-18,
                                         3.59709039131165e-06,
                                         0.034620709896644,
                                         0.164519680110353,
                                         0.00545590422192173,
                                         5.08024807131604e-07,
                                         2.28908494483666e-16,
                                         2.19976669562663e-07,
                                         0.0123560885064969,
                                         0.176978304850708,
                                         0.0155324749003004,
                                         4.35929214916465e-06,
                                         1.14633812173291e-14,
                                         1.11036224923977e-08,
                                         0.00363989492108475,
                                         0.157139367144471,
                                         0.0364986984281677,
                                         3.08752032521701e-05,
                                         4.73834147513186e-13,
                                         4.6562164039661e-10,
                                         0.000890793594275904,
                                         0.115912526399499,
                                         0.0712515797307176,
                                         0.000181670360639262,
                                         1.62712298136687e-11,
                                         1.62712298136687e-11,
                                         0.000181670360639263,
                                         0.0712515797307176,
                                         0.115912526399499,
                                         0.000890793594275901,
                                         4.6562164039661e-10,
                                         4.73834147513186e-13,
                                         3.08752032521702e-05,
                                         0.0364986984281677,
                                         0.157139367144471,
                                         0.00363989492108474,
                                         1.11036224923977e-08,
                                         1.14633812173291e-14,
                                         4.35929214916467e-06,
                                         0.0155324749003004,
                                         0.176978304850708,
                                         0.0123560885064969,
                                         2.19976669562663e-07,
                                         2.28908494483666e-16,
                                         5.08024807131606e-07,
                                         0.00545590422192173,
                                         0.164519680110353,
                                         0.034620709896644,
                                         3.59709039131165e-06,
                                         3.73298214377158e-18,
                                         4.83502466136904e-08,
                                         0.00156508397451435,
                                         0.124899445834696,
                                         0.0792201675888375,
                                         4.8036450573035e-05,
                                         4.89186431765915e-20,
                                         3.69774496404796e-09,
                                         0.000360772177927849,
                                         0.0761952070079273,
                                         0.145666616125387,
                                         0.000515483721208749,
                                         5.02785458155395e-22,
                                         2.21801781575738e-10,
                                         6.52256246679729e-05,
                                         0.0364573090961241,
                                         0.210074878749274,
                                         0.00433859204280081,
                                         3.90442798608796e-24,
                                         1.00521627028162e-11,
                                         8.90983905589157e-06,
                                         0.0131797742248275,
                                         0.228904649852445,
                                         0.0275898334683625,
                                         2.15433031799143e-26,
                                         3.23693632372167e-13,
                                         8.64772038742807e-07,
                                         0.00338541176035768,
                                         0.177221037862509,
                                         0.12466068538978,
                                         7.52834504614831e-29,
                                         6.60147752605759e-15,
                                         5.31576489455627e-08,
                                         0.000550740954135588,
                                         0.0868977001770381,
                                         0.356732179981835,
                                         1.25275874674036e-31,
                                         6.41104532609665e-17,
                                         1.55600359446431e-09,
                                         4.26642339365132e-05,
                                         0.0202899975649854,
                                         0.486111360292378,

                                         0.58200091230393,
                                         0.0546923346313563,
                                         0.000609617360653281,
                                         4.76837158203125e-07,
                                         8.51652935351768e-12,
                                         2.18243468946804e-19,
                                         3.2962920912131e-34,
                                         0.319122438741356,
                                         0.170460065902381,
                                         0.00541051716536309,
                                         1.00135803222656e-05,
                                         4.23174786918272e-10,
                                         3.08804525227497e-17,
                                         2.6511274175388e-31,
                                         0.0833243096648099,
                                         0.252987781676007,
                                         0.0228665653838665,
                                         0.000100135803222656,
                                         1.00128480606257e-08,
                                         2.08068703446894e-15,
                                         1.01535104257378e-28,
                                         0.0137790238715807,
                                         0.237798323306605,
                                         0.061206203479882,
                                         0.000634193420410156,
                                         1.50047171862519e-07,
                                         8.87896230494442e-14,
                                         2.46282857170782e-26,
                                         0.00161899440671491,
                                         0.158817442029671,
                                         0.116404592194777,
                                         0.0028538703918457,
                                         1.59763722013586e-06,
                                         2.69214117902638e-12,
                                         4.24455646167503e-24,
                                         0.000143727094903581,
                                         0.0801408618896597,
                                         0.167267354782839,
                                         0.00970315933227539,
                                         1.28527165137215e-05,
                                         6.16736785929995e-11,
                                         5.52709399042327e-22,
                                         1.00074115838453e-05,
                                         0.0317175502093714,
                                         0.18851333617244,
                                         0.0258750915527344,
                                         8.10963863925678e-05,
                                         1.10813249533329e-09,
                                         5.64483432817117e-20,
                                         5.59924360768197e-07,
                                         0.0100871795210567,
                                         0.170725131742901,
                                         0.0554466247558594,
                                         0.000411180550473813,
                                         1.59995590864139e-08,
                                         4.63265583514257e-18,
                                         2.55847862866307e-08,
                                         0.00261989987188972,
                                         0.126269268160819,
                                         0.0970315933227539,
                                         0.00170258363284867,
                                         1.88655368526881e-07,
                                         3.10494351448884e-16,
                                         9.64932687404426e-10,
                                         0.000561645691214183,
                                         0.0770833648730049,
                                         0.140156745910645,
                                         0.00581898367587883,
                                         1.83608641055719e-06,
                                         1.71767171699872e-14,
                                         3.02337937882764e-11,
                                         0.000100027766683748,
                                         0.0390934562964364,
                                         0.168188095092773,
                                         0.0165221372959788,
                                         1.48455901770087e-05,
                                         7.89418007048447e-13,
                                         7.89418007048428e-13,
                                         1.48455901770087e-05,
                                         0.0165221372959788,
                                         0.168188095092773,
                                         0.0390934562964364,
                                         0.000100027766683748,
                                         3.02337937882771e-11,
                                         1.71767171699867e-14,
                                         1.83608641055719e-06,
                                         0.00581898367587883,
                                         0.140156745910645,
                                         0.0770833648730049,
                                         0.000561645691214183,
                                         9.64932687404445e-10,
                                         3.10494351448875e-16,
                                         1.88655368526881e-07,
                                         0.00170258363284867,
                                         0.0970315933227539,
                                         0.126269268160819,
                                         0.00261989987188972,
                                         2.55847862866312e-08,
                                         4.63265583514242e-18,
                                         1.59995590864139e-08,
                                         0.000411180550473813,
                                         0.0554466247558594,
                                         0.170725131742901,
                                         0.0100871795210567,
                                         5.59924360768205e-07,
                                         5.64483432817098e-20,
                                         1.10813249533329e-09,
                                         8.10963863925678e-05,
                                         0.0258750915527344,
                                         0.18851333617244,
                                         0.0317175502093714,
                                         1.00074115838454e-05,
                                         5.52709399042308e-22,
                                         6.16736785929995e-11,
                                         1.28527165137215e-05,
                                         0.00970315933227539,
                                         0.167267354782839,
                                         0.0801408618896597,
                                         0.000143727094903582,
                                         4.24455646167487e-24,
                                         2.69214117902638e-12,
                                         1.59763722013586e-06,
                                         0.0028538703918457,
                                         0.116404592194777,
                                         0.158817442029671,
                                         0.00161899440671493,
                                         2.46282857170773e-26,
                                         8.87896230494442e-14,
                                         1.50047171862519e-07,
                                         0.000634193420410156,
                                         0.061206203479882,
                                         0.237798323306605,
                                         0.0137790238715808,
                                         1.01535104257374e-28,
                                         2.08068703446894e-15,
                                         1.00128480606257e-08,
                                         0.000100135803222656,
                                         0.0228665653838666,
                                         0.252987781676007,
                                         0.0833243096648103,
                                         2.65112741753868e-31,
                                         3.08804525227497e-17,
                                         4.23174786918272e-10,
                                         1.00135803222656e-05,
                                         0.00541051716536309,
                                         0.170460065902381,
                                         0.319122438741357,
                                         3.29629209121294e-34,
                                         2.18243468946804e-19,
                                         8.51652935351768e-12,
                                         4.76837158203125e-07,
                                         0.000609617360653281,
                                         0.0546923346313563,
                                         0.58200091230393,

                                         0.656290680310972,
                                         0.105241456564439,
                                         0.00339016083885688,
                                         1.63832452475242e-05,
                                         6.76363349222813e-09,
                                         7.56323653954263e-14,
                                         1.41500757830306e-21,
                                         1.8001167686314e-36,
                                         0.279187966600967,
                                         0.250119565006652,
                                         0.0221424009662504,
                                         0.0002373919020764,
                                         2.05850860945792e-07,
                                         5.10671786287702e-12,
                                         2.62564822694259e-19,
                                         1.86611673862741e-33,
                                         0.0565558923908019,
                                         0.283066962399084,
                                         0.0688667874692967,
                                         0.00163799500561237,
                                         2.98336241908851e-06,
                                         1.6419379892194e-10,
                                         2.32003758692785e-17,
                                         9.2120788068786e-31,
                                         0.00725590135064093,
                                         0.20289112512977,
                                         0.135652338166083,
                                         0.00715799832596425,
                                         2.73836722327329e-05,
                                         3.34352045180585e-09,
                                         1.29833225742109e-15,
                                         2.88010835471572e-28,
                                         0.000661431763845462,
                                         0.103327776533635,
                                         0.189856255443849,
                                         0.0222254610730234,
                                         0.000178590160940734,
                                         4.83761591639335e-08,
                                         5.16246168463471e-14,
                                         6.39794020602247e-26,
                                         4.55559521393883e-05,
                                         0.0397591893444118,
                                         0.200765433726943,
                                         0.0521406414104704,
                                         0.000880014387704469,
                                         5.28841085731579e-07,
                                         1.55093729198719e-12,
                                         1.07383592753127e-23,
                                         2.46090599421687e-06,
                                         0.0119990751408547,
                                         0.166510945544142,
                                         0.0959382461072489,
                                         0.00340104072171106,
                                         4.53428492814634e-06,
                                         3.65444517708634e-11,
                                         1.41359662166355e-21,
                                         1.06824140448103e-07,
                                         0.00290992979837159,
                                         0.110973969117747,
                                         0.141850759918547,
                                         0.0105622970661419,
                                         3.1240423514021e-05,
                                         6.91947509234441e-10,
                                         1.49533174697774e-19,
                                         3.78694170260428e-09,
                                         0.000576317862377543,
                                         0.0604010212155103,
                                         0.171283839064869,
                                         0.0267885837192008,
                                         0.000175780157045199,
                                         1.06996526533186e-08,
                                         1.29179749681723e-17,
                                         1.10807874963358e-10,
                                         9.42116137659416e-05,
                                         0.0271350257238855,
                                         0.170711942581623,
                                         0.056079472957404,
                                         0.000816367257055305,
                                         1.36561727620731e-07,
                                         9.21115557834845e-16,
                                         2.69359958258004e-12,
                                         1.27946146063695e-05,
                                         0.0101273681332879,
                                         0.141348701569656,
                                         0.0975300611334146,
                                         0.00314979080146505,
                                         1.4480005558924e-06,
                                         5.45650050720267e-14,
                                         5.45650050720267e-14,
                                         1.4480005558924e-06,
                                         0.00314979080146505,
                                         0.0975300611334148,
                                         0.141348701569656,
                                         0.0101273681332879,
                                         1.27946146063695e-05,
                                         2.69359958258004e-12,
                                         9.21115557834845e-16,
                                         1.36561727620731e-07,
                                         0.000816367257055305,
                                         0.0560794729574041,
                                         0.170711942581623,
                                         0.0271350257238855,
                                         9.42116137659416e-05,
                                         1.10807874963358e-10,
                                         1.29179749681723e-17,
                                         1.06996526533186e-08,
                                         0.000175780157045199,
                                         0.0267885837192008,
                                         0.171283839064869,
                                         0.0604010212155103,
                                         0.000576317862377543,
                                         3.78694170260428e-09,
                                         1.49533174697774e-19,
                                         6.91947509234441e-10,
                                         3.1240423514021e-05,
                                         0.0105622970661419,
                                         0.141850759918546,
                                         0.110973969117747,
                                         0.00290992979837159,
                                         1.06824140448103e-07,
                                         1.41359662166355e-21,
                                         3.65444517708634e-11,
                                         4.53428492814634e-06,
                                         0.00340104072171107,
                                         0.0959382461072488,
                                         0.166510945544142,
                                         0.0119990751408547,
                                         2.46090599421687e-06,
                                         1.07383592753127e-23,
                                         1.55093729198719e-12,
                                         5.28841085731579e-07,
                                         0.000880014387704471,
                                         0.0521406414104704,
                                         0.200765433726943,
                                         0.0397591893444118,
                                         4.55559521393883e-05,
                                         6.39794020602247e-26,
                                         5.16246168463471e-14,
                                         4.83761591639335e-08,
                                         0.000178590160940735,
                                         0.0222254610730234,
                                         0.189856255443849,
                                         0.103327776533635,
                                         0.000661431763845462,
                                         2.88010835471572e-28,
                                         1.29833225742109e-15,
                                         3.34352045180585e-09,
                                         2.73836722327329e-05,
                                         0.00715799832596424,
                                         0.135652338166083,
                                         0.20289112512977,
                                         0.00725590135064093,
                                         9.2120788068786e-31,
                                         2.32003758692785e-17,
                                         1.6419379892194e-10,
                                         2.98336241908852e-06,
                                         0.00163799500561237,
                                         0.0688667874692967,
                                         0.283066962399084,
                                         0.0565558923908019,
                                         1.86611673862741e-33,
                                         2.62564822694259e-19,
                                         5.10671786287703e-12,
                                         2.05850860945792e-07,
                                         0.0002373919020764,
                                         0.0221424009662504,
                                         0.250119565006652,
                                         0.279187966600967,
                                         1.8001167686314e-36,
                                         1.41500757830306e-21,
                                         7.56323653954263e-14,
                                         6.76363349222815e-09,
                                         1.63832452475242e-05,
                                         0.00339016083885688,
                                         0.105241456564439,
                                         0.656290680310972,

                                         0.713903687398444,
                                         0.165901696490901,
                                         0.0109847887639536,
                                         0.000173685387516928,
                                         4.76837158203125e-07,
                                         1.27007985936994e-10,
                                         1.02693471151281e-15,
                                         1.54298189508293e-23,
                                         1.74074728746475e-38,
                                         0.242531558796548,
                                         0.311136921652128,
                                         0.0552803242775194,
                                         0.00186120978269483,
                                         1.00135803222656e-05,
                                         5.22682035845675e-09,
                                         8.99917197166826e-14,
                                         3.62826246708926e-21,
                                         2.25967345388255e-35,
                                         0.0392353558867547,
                                         0.27786444526739,
                                         0.132473850361218,
                                         0.00949747319323981,
                                         0.000100135803222656,
                                         1.02429446081223e-07,
                                         3.75528561649344e-12,
                                         4.06272351206999e-19,
                                         1.39680681557504e-32,
                                         0.00401993731245805,
                                         0.157161713149931,
                                         0.201058335504131,
                                         0.0306939776483877,
                                         0.000634193420410156,
                                         1.2712893843328e-06,
                                         9.92466240268019e-11,
                                         2.88116536238892e-17,
                                         5.46838840172634e-30,
                                         0.000292645016367006,
                                         0.0631597928153531,
                                         0.216817420841841,
                                         0.070482035221909,
                                         0.0028538703918457,
                                         1.12109956593319e-05,
                                         1.86366856184137e-09,
                                         1.45177485497579e-15,
                                         1.52111668147381e-27,
                                         1.60964234250192e-05,
                                         0.0191778996790921,
                                         0.176657739249837,
                                         0.122284131406717,
                                         0.00970315933227539,
                                         7.46982369512244e-05,
                                         2.64416175234489e-08,
                                         5.52709282460791e-14,
                                         3.19692249810577e-25,
                                         6.94396396321421e-07,
                                         0.00456721234062355,
                                         0.11289143094047,
                                         0.166399334567913,
                                         0.0258750915527344,
                                         0.000390360921015313,
                                         2.94236925427473e-07,
                                         1.65038041766897e-12,
                                         5.26976807384479e-23,
                                         2.40718803223629e-08,
                                         0.000874029017312447,
                                         0.0579713980836757,
                                         0.181952301837573,
                                         0.0554466247558594,
                                         0.00163925608014639,
                                         2.63106038464513e-06,
                                         3.96000607583782e-11,
                                         6.98032080956414e-21,
                                         6.81486728158638e-10,
                                         0.000136598310743286,
                                         0.0243114801275838,
                                         0.162483158488183,
                                         0.0970315933227539,
                                         0.00562175742285188,
                                         1.92136239722463e-05,
                                         7.75983473957933e-10,
                                         7.55099344715359e-19,
                                         1.59245652067831e-11,
                                         1.76208802681475e-05,
                                         0.00841534297006499,
                                         0.119762801197338,
                                         0.140156745910645,
                                         0.015913297975854,
                                         0.000115811197546346,
                                         1.25508137946032e-08,
                                         6.7421063385123e-17,
                                         3.09142110658233e-13,
                                         1.88838383879347e-06,
                                         0.00241998483169116,
                                         0.073335784358846,
                                         0.168188095092773,
                                         0.0374221462045451,
                                         0.000579925520806402,
                                         1.68644314224114e-07,
                                         5.00112265448871e-15,
                                         5.00112265448852e-15,
                                         1.68644314224114e-07,
                                         0.0005799255208064,
                                         0.0374221462045451,
                                         0.168188095092773,
                                         0.073335784358846,
                                         0.00241998483169117,
                                         1.88838383879347e-06,
                                         3.09142110658244e-13,
                                         6.74210633851202e-17,
                                         1.25508137946032e-08,
                                         0.000115811197546346,
                                         0.015913297975854,
                                         0.140156745910645,
                                         0.119762801197338,
                                         0.00841534297006501,
                                         1.76208802681475e-05,
                                         1.59245652067836e-11,
                                         7.55099344715325e-19,
                                         7.75983473957933e-10,
                                         1.92136239722463e-05,
                                         0.00562175742285188,
                                         0.0970315933227539,
                                         0.162483158488183,
                                         0.0243114801275838,
                                         0.000136598310743286,
                                         6.81486728158657e-10,
                                         6.9803208095638e-21,
                                         3.96000607583782e-11,
                                         2.63106038464512e-06,
                                         0.00163925608014639,
                                         0.0554466247558594,
                                         0.181952301837573,
                                         0.0579713980836758,
                                         0.000874029017312447,
                                         2.40718803223635e-08,
                                         5.26976807384452e-23,
                                         1.65038041766897e-12,
                                         2.94236925427472e-07,
                                         0.000390360921015313,
                                         0.0258750915527344,
                                         0.166399334567913,
                                         0.11289143094047,
                                         0.00456721234062355,
                                         6.94396396321436e-07,
                                         3.19692249810559e-25,
                                         5.52709282460791e-14,
                                         2.64416175234487e-08,
                                         7.46982369512243e-05,
                                         0.00970315933227539,
                                         0.122284131406717,
                                         0.176657739249837,
                                         0.0191778996790921,
                                         1.60964234250195e-05,
                                         1.52111668147372e-27,
                                         1.45177485497579e-15,
                                         1.86366856184136e-09,
                                         1.12109956593319e-05,
                                         0.0028538703918457,
                                         0.070482035221909,
                                         0.216817420841841,
                                         0.0631597928153531,
                                         0.00029264501636701,
                                         5.468388401726e-30,
                                         2.88116536238892e-17,
                                         9.92466240268014e-11,
                                         1.2712893843328e-06,
                                         0.000634193420410156,
                                         0.0306939776483877,
                                         0.201058335504131,
                                         0.157161713149931,
                                         0.00401993731245809,
                                         1.39680681557495e-32,
                                         4.06272351206999e-19,
                                         3.75528561649342e-12,
                                         1.02429446081223e-07,
                                         0.000100135803222656,
                                         0.00949747319323981,
                                         0.132473850361218,
                                         0.27786444526739,
                                         0.039235355886755,
                                         2.25967345388239e-35,
                                         3.62826246708926e-21,
                                         8.99917197166821e-14,
                                         5.22682035845675e-09,
                                         1.00135803222656e-05,
                                         0.00186120978269483,
                                         0.0552803242775195,
                                         0.311136921652128,
                                         0.242531558796549,
                                         1.74074728746462e-38,
                                         1.54298189508293e-23,
                                         1.0269347115128e-15,
                                         1.27007985936994e-10,
                                         4.76837158203125e-07,
                                         0.000173685387516928,
                                         0.0109847887639536,
                                         0.165901696490901,
                                         0.713903687398444,

                                         0.758976524526876,
                                         0.230640472604676,
                                         0.0255069880414945,
                                         0.000916339037074808,
                                         8.79211186246487e-06,
                                         1.6152415028833e-08,
                                         3.142332970911e-12,
                                         2.01062821973315e-17,
                                         2.57689881713344e-25,
                                         2.66403045136127e-40,
                                         0.210694362812033,
                                         0.35042178090148,
                                         0.10225214009816,
                                         0.00760658248416122,
                                         0.000136783481972224,
                                         4.57863058031977e-07,
                                         1.6693884372586e-10,
                                         2.21185745639749e-15,
                                         7.47963460899954e-23,
                                         4.23207349681298e-37,
                                         0.0278521159345797,
                                         0.253528840886729,
                                         0.19519394408131,
                                         0.0300679399983681,
                                         0.00101333912299929,
                                         6.18036962381331e-06,
                                         4.22321610552693e-09,
                                         1.1586791771291e-13,
                                         1.03381803467046e-20,
                                         3.20145884384191e-34,
                                         0.00233182427251764,
                                         0.116170536633904,
                                         0.235989466697843,
                                         0.0752748878353352,
                                         0.00475453787852841,
                                         5.28354779390134e-05,
                                         6.76646057207524e-08,
                                         3.844161416002e-12,
                                         9.04982480603821e-19,
                                         1.5338221007906e-31,
                                         0.000138711864900847,
                                         0.0378220259026038,
                                         0.202721152095004,
                                         0.133898812901632,
                                         0.0158504640222783,
                                         0.000320934938369737,
                                         7.70300134014795e-07,
                                         9.06191941986463e-11,
                                         5.62880763951356e-17,
                                         5.22134303650295e-29,
                                         6.23444511278511e-06,
                                         0.00930379179975146,
                                         0.13157455681582,
                                         0.17995741233217,
                                         0.0399247359926678,
                                         0.00147290508207357,
                                         6.62559361389266e-06,
                                         1.61400613051481e-09,
                                         2.64520323907564e-15,
                                         1.34293769949912e-26,
                                         2.19771724630306e-07,
                                         0.00179500223589553,
                                         0.0669783710762453,
                                         0.189693551605939,
                                         0.0788736502847688,
                                         0.0053017877541146,
                                         4.46971086886046e-05,
                                         2.25465438836144e-08,
                                         9.74970554714796e-14,
                                         2.70906434117324e-24,
                                         6.22544375096274e-09,
                                         0.000278288007517041,
                                         0.027398181186435,
                                         0.160679282139832,
                                         0.125212104438297,
                                         0.0153353750393702,
                                         0.000242302867673658,
                                         2.53092493808871e-07,
                                         2.88767584656012e-12,
                                         4.39144310708473e-22,
                                         1.440169603345e-10,
                                         3.52345572369088e-05,
                                         0.00915279457454503,
                                         0.111150655274392,
                                         0.16233251557435,
                                         0.0362252419584495,
                                         0.0010727104387979,
                                         2.32018909187698e-06,
                                         6.98473945306404e-11,
                                         5.81353151599011e-20,
                                         2.74991811146155e-12,
                                         3.68218784237828e-06,
                                         0.00252376254733676,
                                         0.0634639305669698,
                                         0.173711078570567,
                                         0.0706302888832437,
                                         0.0039198482448651,
                                         1.75561907330144e-05,
                                         1.39448797954391e-09,
                                         6.35236721820399e-18,
                                         4.36220715327008e-14,
                                         3.19685887060028e-07,
                                         0.000578127556136956,
                                         0.0301038684900345,
                                         0.154429376538056,
                                         0.114406598464995,
                                         0.0118997107224156,
                                         0.000110361502220033,
                                         2.31291537260239e-08,
                                         5.76649027254338e-16,
                                         5.76649027254338e-16,
                                         2.31291537260241e-08,
                                         0.000110361502220033,
                                         0.0118997107224156,
                                         0.114406598464995,
                                         0.154429376538056,
                                         0.0301038684900346,
                                         0.000578127556136956,
                                         3.19685887060025e-07,
                                         4.36220715327008e-14,
                                         6.35236721820399e-18,
                                         1.39448797954392e-09,
                                         1.75561907330144e-05,
                                         0.0039198482448651,
                                         0.0706302888832437,
                                         0.173711078570567,
                                         0.0634639305669698,
                                         0.00252376254733676,
                                         3.68218784237825e-06,
                                         2.74991811146155e-12,
                                         5.81353151599011e-20,
                                         6.98473945306411e-11,
                                         2.32018909187698e-06,
                                         0.0010727104387979,
                                         0.0362252419584495,
                                         0.16233251557435,
                                         0.111150655274392,
                                         0.00915279457454503,
                                         3.52345572369086e-05,
                                         1.440169603345e-10,
                                         4.39144310708473e-22,
                                         2.88767584656016e-12,
                                         2.53092493808871e-07,
                                         0.000242302867673658,
                                         0.0153353750393702,
                                         0.125212104438297,
                                         0.160679282139832,
                                         0.027398181186435,
                                         0.000278288007517039,
                                         6.22544375096274e-09,
                                         2.70906434117324e-24,
                                         9.74970554714808e-14,
                                         2.25465438836144e-08,
                                         4.46971086886046e-05,
                                         0.0053017877541146,
                                         0.0788736502847688,
                                         0.189693551605939,
                                         0.0669783710762453,
                                         0.00179500223589552,
                                         2.19771724630306e-07,
                                         1.34293769949912e-26,
                                         2.64520323907567e-15,
                                         1.61400613051481e-09,
                                         6.62559361389266e-06,
                                         0.00147290508207357,
                                         0.0399247359926678,
                                         0.17995741233217,
                                         0.13157455681582,
                                         0.00930379179975143,
                                         6.23444511278511e-06,
                                         5.22134303650295e-29,
                                         5.62880763951364e-17,
                                         9.06191941986462e-11,
                                         7.70300134014795e-07,
                                         0.000320934938369737,
                                         0.0158504640222783,
                                         0.133898812901632,
                                         0.202721152095004,
                                         0.0378220259026037,
                                         0.000138711864900847,
                                         1.5338221007906e-31,
                                         9.04982480603835e-19,
                                         3.844161416002e-12,
                                         6.76646057207524e-08,
                                         5.28354779390134e-05,
                                         0.00475453787852841,
                                         0.0752748878353352,
                                         0.235989466697843,
                                         0.116170536633904,
                                         0.00233182427251764,
                                         3.20145884384191e-34,
                                         1.03381803467047e-20,
                                         1.1586791771291e-13,
                                         4.22321610552693e-09,
                                         6.18036962381331e-06,
                                         0.00101333912299929,
                                         0.0300679399983681,
                                         0.19519394408131,
                                         0.253528840886729,
                                         0.0278521159345797,
                                         4.23207349681298e-37,
                                         7.47963460899967e-23,
                                         2.21185745639749e-15,
                                         1.6693884372586e-10,
                                         4.57863058031977e-07,
                                         0.000136783481972224,
                                         0.00760658248416122,
                                         0.10225214009816,
                                         0.35042178090148,
                                         0.210694362812033,
                                         2.66403045136127e-40,
                                         2.57689881713348e-25,
                                         2.01062821973315e-17,
                                         3.142332970911e-12,
                                         1.6152415028833e-08,
                                         8.79211186246487e-06,
                                         0.000916339037074808,
                                         0.0255069880414945,
                                         0.230640472604676,
                                         0.758976524526876,

                                         0.794650867977646,
                                         0.295041117672899,
                                         0.0476584114026797,
                                         0.00310213354867319,
                                         7.16088881835648e-05,
                                         4.76837158203125e-07,
                                         6.51412225720696e-10,
                                         1.00367664442081e-13,
                                         5.3935309892307e-19,
                                         6.13710568746168e-27,
                                         5.94236372976081e-42,
                                         0.183655680161483,
                                         0.370811605670814,
                                         0.156096724770569,
                                         0.0206230455429567,
                                         0.000865233487089213,
                                         1.00135803222656e-05,
                                         2.37754149772635e-08,
                                         6.65794337712514e-12,
                                         7.2620216443663e-17,
                                         2.15343326822559e-24,
                                         1.13388710916521e-38,
                                         0.0202121760785798,
                                         0.221924268571835,
                                         0.243460620685879,
                                         0.0652868652514475,
                                         0.00497829254074808,
                                         0.000100135803222656,
                                         4.13219694089725e-07,
                                         2.1031346651436e-10,
                                         4.65610327742136e-15,
                                         3.5981555917e-22,
                                         1.030293855795e-35,
                                         0.001408815519704,
                                         0.0841179388069506,
                                         0.240489436774024,
                                         0.130897447621829,
                                         0.0181409427948451,
                                         0.000634193420410156,
                                         4.54847973448063e-06,
                                         4.20752216385541e-09,
                                         1.89068864953426e-13,
                                         3.80768318834801e-20,
                                         5.92904502087455e-33,
                                         6.97710644977332e-05,
                                         0.0226544026430333,
                                         0.168788732974096,
                                         0.18647332194849,
                                         0.0469698811885923,
                                         0.0028538703918457,
                                         3.55739167040304e-05,
                                         5.98089131608592e-08,
                                         5.4550356938541e-12,
                                         2.86300325205319e-18,
                                         2.42431223784379e-30,
                                         2.61073609842382e-06,
                                         0.00460980984484901,
                                         0.0895070624750069,
                                         0.200709828721081,
                                         0.0918851804756143,
                                         0.00970315933227539,
                                         0.000210214905493927,
                                         6.42350123538348e-07,
                                         1.18916352464812e-10,
                                         1.62648206760193e-16,
                                         7.4896023970685e-28,
                                         7.66196942302012e-08,
                                         0.000735704177847156,
                                         0.0372272512750694,
                                         0.169437832901515,
                                         0.140981231199119,
                                         0.0258750915527344,
                                         0.000974283045735773,
                                         5.41087534273426e-06,
                                         2.03317788742732e-09,
                                         7.24713802552013e-15,
                                         1.81475828170776e-25,
                                         1.80693421535897e-09,
                                         9.43513069269135e-05,
                                         0.0124419692549772,
                                         0.114941437121504,
                                         0.173820747018038,
                                         0.0554466247558594,
                                         0.00362853461233627,
                                         3.66258516572627e-05,
                                         2.7934034336927e-08,
                                         2.59482619105194e-13,
                                         3.53348537386545e-23,
                                         3.480078964042e-11,
                                         9.88183158296174e-06,
                                         0.00339595640663908,
                                         0.0636777467114402,
                                         0.175019627318399,
                                         0.0970315933227539,
                                         0.0110362676609031,
                                         0.000202466308769661,
                                         3.13427268046174e-07,
                                         7.58743253692912e-12,
                                         5.61865852554029e-21,
                                         5.5322098967874e-13,
                                         8.54259406621519e-07,
                                         0.000765064205854176,
                                         0.0291180002624194,
                                         0.145457021924587,
                                         0.140156745910645,
                                         0.0277061367636339,
                                         0.000923805603975007,
                                         2.90270356273416e-06,
                                         1.83123553216135e-10,
                                         7.3743674429979e-19,
                                         7.3061503203741e-15,
                                         6.13511198956562e-08,
                                         0.000143190453249222,
                                         0.01106154542828,
                                         0.100429900328896,
                                         0.168188095092773,
                                         0.0577843358048235,
                                         0.00350177971699006,
                                         2.23330993495135e-05,
                                         3.67175735621214e-09,
                                         8.04076392312707e-17,
                                         8.04076392312662e-17,
                                         3.67175735621214e-09,
                                         2.23330993495136e-05,
                                         0.00350177971699007,
                                         0.0577843358048234,
                                         0.168188095092773,
                                         0.100429900328896,
                                         0.01106154542828,
                                         0.000143190453249222,
                                         6.13511198956562e-08,
                                         7.30615032037448e-15,
                                         7.37436744299745e-19,
                                         1.83123553216135e-10,
                                         2.90270356273417e-06,
                                         0.00092380560397501,
                                         0.0277061367636339,
                                         0.140156745910645,
                                         0.145457021924588,
                                         0.0291180002624193,
                                         0.000765064205854174,
                                         8.54259406621519e-07,
                                         5.53220989678766e-13,
                                         5.61865852553992e-21,
                                         7.58743253692912e-12,
                                         3.13427268046176e-07,
                                         0.000202466308769661,
                                         0.0110362676609031,
                                         0.0970315933227539,
                                         0.1750196273184,
                                         0.0636777467114401,
                                         0.00339595640663907,
                                         9.88183158296174e-06,
                                         3.48007896404215e-11,
                                         3.5334853738652e-23,
                                         2.59482619105194e-13,
                                         2.79340343369271e-08,
                                         3.66258516572628e-05,
                                         0.00362853461233626,
                                         0.0554466247558594,
                                         0.173820747018039,
                                         0.114941437121504,
                                         0.0124419692549771,
                                         9.43513069269135e-05,
                                         1.80693421535904e-09,
                                         1.81475828170763e-25,
                                         7.24713802552013e-15,
                                         2.03317788742733e-09,
                                         5.41087534273428e-06,
                                         0.000974283045735771,
                                         0.0258750915527344,
                                         0.140981231199119,
                                         0.169437832901515,
                                         0.0372272512750693,
                                         0.000735704177847155,
                                         7.66196942302035e-08,
                                         7.48960239706789e-28,
                                         1.62648206760193e-16,
                                         1.18916352464813e-10,
                                         6.4235012353835e-07,
                                         0.000210214905493926,
                                         0.00970315933227539,
                                         0.0918851804756144,
                                         0.200709828721081,
                                         0.0895070624750067,
                                         0.00460980984484901,
                                         2.61073609842389e-06,
                                         2.42431223784358e-30,
                                         2.86300325205319e-18,
                                         5.45503569385414e-12,
                                         5.98089131608594e-08,
                                         3.55739167040303e-05,
                                         0.0028538703918457,
                                         0.0469698811885923,
                                         0.18647332194849,
                                         0.168788732974096,
                                         0.0226544026430333,
                                         6.97710644977346e-05,
                                         5.929045020874e-33,
                                         3.80768318834801e-20,
                                         1.89068864953427e-13,
                                         4.20752216385543e-09,
                                         4.54847973448061e-06,
                                         0.000634193420410156,
                                         0.0181409427948451,
                                         0.130897447621828,
                                         0.240489436774023,
                                         0.0841179388069506,
                                         0.00140881551970402,
                                         1.0302938557949e-35,
                                         3.5981555917e-22,
                                         4.65610327742139e-15,
                                         2.10313466514361e-10,
                                         4.13219694089724e-07,
                                         0.000100135803222656,
                                         0.00497829254074808,
                                         0.0652868652514475,
                                         0.243460620685879,
                                         0.221924268571835,
                                         0.02021217607858,
                                         1.13388710916509e-38,
                                         2.15343326822559e-24,
                                         7.26202164436636e-17,
                                         6.65794337712517e-12,
                                         2.37754149772635e-08,
                                         1.00135803222656e-05,
                                         0.000865233487089213,
                                         0.0206230455429567,
                                         0.156096724770569,
                                         0.370811605670814,
                                         0.183655680161484,
                                         5.94236372976018e-42,
                                         6.13710568746168e-27,
                                         5.39353098923075e-19,
                                         1.00367664442081e-13,
                                         6.51412225720694e-10,
                                         4.76837158203125e-07,
                                         7.16088881835648e-05,
                                         0.00310213354867319,
                                         0.0476584114026797,
                                         0.295041117672899,
                                         0.794650867977646,

                                         0.823236800465254,
                                         0.356399887502512,
                                         0.0767904451901271,
                                         0.00780391610811998,
                                         0.000342810964697834,
                                         5.68152692797883e-06,
                                         2.87148166323468e-08,
                                         3.13227578772329e-11,
                                         4.03921676499772e-15,
                                         1.89894807191384e-20,
                                         1.97205760231078e-28,
                                         1.81566468500875e-43,
                                         0.160872823300365,
                                         0.376880456023381,
                                         0.209647006702292,
                                         0.0426072527957774,
                                         0.00332716432587902,
                                         9.27542748558034e-05,
                                         7.75667815250396e-07,
                                         1.42324293385955e-09,
                                         3.26260733990293e-13,
                                         3.06739847741498e-18,
                                         8.2241709142296e-26,
                                         4.09747515027561e-40,
                                         0.0149699826355605,
                                         0.189779972966315,
                                         0.272552927958739,
                                         0.11077332174379,
                                         0.0153771022674809,
                                         0.000721080091009176,
                                         9.97760257558134e-06,
                                         3.07948990273096e-08,
                                         1.25491168375472e-11,
                                         2.35943496908817e-16,
                                         1.6332225345607e-23,
                                         4.40329432488515e-37,
                                         0.000882251195844886,
                                         0.0605242602975728,
                                         0.224411642287047,
                                         0.182397621762039,
                                         0.0450098018283665,
                                         0.0035503030168347,
                                         8.12847318048598e-05,
                                         4.21998472345125e-07,
                                         3.05698890787023e-10,
                                         1.14941847163055e-14,
                                         2.05414428717072e-21,
                                         2.99689445987219e-34,
                                         3.69439546405146e-05,
                                         0.0137147775502191,
                                         0.131286503971406,
                                         0.213394553784465,
                                         0.0936094872641867,
                                         0.0124201690101199,
                                         0.000470513317762719,
                                         4.10887699940147e-06,
                                         5.29120673764278e-09,
                                         3.97858370936182e-13,
                                         1.83567881591415e-19,
                                         1.44925681000074e-31,
                                         1.16885573559212e-06,
                                         0.00234808843011421,
                                         0.0580311512736895,
                                         0.188631359325937,
                                         0.147095380955003,
                                         0.0328288744512364,
                                         0.00205779101776617,
                                         3.02274704919734e-05,
                                         6.91961611655428e-08,
                                         1.04050753892794e-11,
                                         1.2394495365861e-17,
                                         5.29524006352637e-29,
                                         2.90046919569826e-08,
                                         0.000315304337077429,
                                         0.0201183382057324,
                                         0.130777873947016,
                                         0.181287566586425,
                                         0.0680572342031745,
                                         0.0070586301868561,
                                         0.000174409559934043,
                                         7.09739682170457e-07,
                                         2.13428188177267e-10,
                                         6.56373166272854e-16,
                                         1.51745483355764e-26,
                                         5.78362450765294e-10,
                                         3.40227902188495e-05,
                                         0.0056046372593028,
                                         0.0728583174918522,
                                         0.179540111435738,
                                         0.113374943887299,
                                         0.0194564712997065,
                                         0.000808654957701047,
                                         5.84979518431873e-06,
                                         3.51789449791076e-09,
                                         2.79316938557727e-14,
                                         3.49438199927186e-24,
                                         9.41839175687845e-12,
                                         2.99815954458773e-06,
                                         0.00127511027159236,
                                         0.0331488779547952,
                                         0.145211091862212,
                                         0.154242730290476,
                                         0.0437978262196897,
                                         0.00306197083866151,
                                         3.93755889975902e-05,
                                         4.73542184845995e-08,
                                         9.70707887050899e-13,
                                         6.57158024359082e-22,
                                         1.26594899009669e-13,
                                         2.18073225046087e-07,
                                         0.000239447756862699,
                                         0.0124486231981636,
                                         0.0969395246256481,
                                         0.173202883303757,
                                         0.0813774034642483,
                                         0.00956977311191054,
                                         0.000218764203669935,
                                         5.26135166344368e-07,
                                         2.7844706047087e-11,
                                         1.02007486209701e-19,
                                         1.41363101688362e-15,
                                         1.31774190226588e-08,
                                         3.73554699888977e-05,
                                         0.00388377635912932,
                                         0.0537628624712698,
                                         0.161579378696951,
                                         0.125613265024317,
                                         0.0248474963494571,
                                         0.00100973139543369,
                                         4.85642169707049e-06,
                                         6.63555296093911e-10,
                                         1.31545100916919e-17,
                                         1.31545100916928e-17,
                                         6.63555296093902e-10,
                                         4.85642169707046e-06,
                                         0.00100973139543368,
                                         0.0248474963494571,
                                         0.125613265024317,
                                         0.161579378696951,
                                         0.0537628624712699,
                                         0.00388377635912934,
                                         3.73554699888979e-05,
                                         1.3177419022659e-08,
                                         1.41363101688354e-15,
                                         1.02007486209709e-19,
                                         2.78447060470866e-11,
                                         5.26135166344365e-07,
                                         0.000218764203669934,
                                         0.00956977311191052,
                                         0.0813774034642483,
                                         0.173202883303757,
                                         0.0969395246256483,
                                         0.0124486231981636,
                                         0.0002394477568627,
                                         2.18073225046089e-07,
                                         1.26594899009662e-13,
                                         6.57158024359134e-22,
                                         9.70707887050884e-13,
                                         4.73542184845992e-08,
                                         3.93755889975901e-05,
                                         0.0030619708386615,
                                         0.0437978262196897,
                                         0.154242730290476,
                                         0.145211091862213,
                                         0.0331488779547953,
                                         0.00127511027159236,
                                         2.99815954458776e-06,
                                         9.418391756878e-12,
                                         3.49438199927216e-24,
                                         2.79316938557722e-14,
                                         3.51789449791074e-09,
                                         5.84979518431871e-06,
                                         0.000808654957701045,
                                         0.0194564712997065,
                                         0.113374943887299,
                                         0.179540111435738,
                                         0.0728583174918523,
                                         0.00560463725930282,
                                         3.40227902188497e-05,
                                         5.7836245076527e-10,
                                         1.51745483355777e-26,
                                         6.56373166272842e-16,
                                         2.13428188177265e-10,
                                         7.09739682170454e-07,
                                         0.000174409559934043,
                                         0.0070586301868561,
                                         0.0680572342031745,
                                         0.181287566586425,
                                         0.130777873947016,
                                         0.0201183382057325,
                                         0.000315304337077431,
                                         2.90046919569816e-08,
                                         5.29524006352688e-29,
                                         1.23944953658608e-17,
                                         1.04050753892793e-11,
                                         6.91961611655425e-08,
                                         3.02274704919733e-05,
                                         0.00205779101776617,
                                         0.0328288744512364,
                                         0.147095380955003,
                                         0.188631359325937,
                                         0.0580311512736897,
                                         0.00234808843011422,
                                         1.16885573559209e-06,
                                         1.44925681000089e-31,
                                         1.83567881591411e-19,
                                         3.97858370936179e-13,
                                         5.29120673764275e-09,
                                         4.10887699940146e-06,
                                         0.000470513317762719,
                                         0.0124201690101199,
                                         0.0936094872641867,
                                         0.213394553784466,
                                         0.131286503971406,
                                         0.0137147775502192,
                                         3.69439546405137e-05,
                                         2.99689445987252e-34,
                                         2.05414428717068e-21,
                                         1.14941847163054e-14,
                                         3.05698890787022e-10,
                                         4.21998472345124e-07,
                                         8.12847318048598e-05,
                                         0.0035503030168347,
                                         0.0450098018283666,
                                         0.182397621762039,
                                         0.224411642287048,
                                         0.060524260297573,
                                         0.00088225119584487,
                                         4.40329432488565e-37,
                                         1.63322253456067e-23,
                                         2.35943496908815e-16,
                                         1.25491168375471e-11,
                                         3.07948990273095e-08,
                                         9.97760257558134e-06,
                                         0.000721080091009176,
                                         0.0153771022674809,
                                         0.11077332174379,
                                         0.272552927958739,
                                         0.189779972966316,
                                         0.0149699826355604,
                                         4.09747515027611e-40,
                                         8.2241709142294e-26,
                                         3.06739847741495e-18,
                                         3.26260733990291e-13,
                                         1.42324293385954e-09,
                                         7.75667815250396e-07,
                                         9.27542748558034e-05,
                                         0.00332716432587902,
                                         0.0426072527957774,
                                         0.209647006702292,
                                         0.376880456023381,
                                         0.160872823300364,
                                         1.81566468500898e-43,
                                         1.97205760231073e-28,
                                         1.89894807191382e-20,
                                         4.03921676499769e-15,
                                         3.13227578772328e-11,
                                         2.87148166323468e-08,
                                         5.68152692797883e-06,
                                         0.000342810964697834,
                                         0.00780391610811998,
                                         0.0767904451901271,
                                         0.356399887502512,
                                         0.823236800465254,

                                         0.846421243215255,
                                         0.413314764242921,
                                         0.111451268638169,
                                         0.0159641733199441,
                                         0.00114186021038895,
                                         3.71342889172689e-05,
                                         4.76837158203125e-07,
                                         1.9464588569037e-09,
                                         1.78215641845319e-12,
                                         1.99990677897182e-16,
                                         8.46744139503085e-22,
                                         8.18500053115329e-30,
                                         7.24344133339848e-45,
                                         0.141692455038148,
                                         0.372973695877138,
                                         0.257775028553971,
                                         0.0730061119867251,
                                         0.00912992339141563,
                                         0.000487707742742549,
                                         1.00135803222656e-05,
                                         6.53580790654648e-08,
                                         9.82947256306605e-11,
                                         1.92857066214235e-14,
                                         1.61448852156949e-19,
                                         4.00000023217591e-27,
                                         1.90819769059848e-41,
                                         0.0112950355200224,
                                         0.160271460387387,
                                         0.283907833859897,
                                         0.158983740851641,
                                         0.0347617820610062,
                                         0.00305017596130827,
                                         0.000100135803222656,
                                         1.04504269967871e-06,
                                         2.58163782631509e-09,
                                         8.85609087951106e-13,
                                         1.46588003257778e-17,
                                         9.30854979731292e-25,
                                         2.39377038699642e-38,
                                         0.000570244129541031,
                                         0.0436180785825159,
                                         0.198036961791112,
                                         0.219269663036696,
                                         0.083824173050814,
                                         0.0120815452241874,
                                         0.000634193420410156,
                                         1.05828133953669e-05,
                                         4.29430377694845e-08,
                                         2.57561484421086e-11,
                                         8.42935302890951e-16,
                                         1.37194399307694e-22,
                                         1.90183992612444e-35,
                                         2.0455693730709e-05,
                                         0.00843445517457221,
                                         0.0981511293889438,
                                         0.214874339302389,
                                         0.143620584702748,
                                         0.034001669473141,
                                         0.0028538703918457,
                                         7.6146230390518e-05,
                                         5.07540119646043e-07,
                                         5.3223072456805e-10,
                                         3.44405615611125e-14,
                                         1.43671592670891e-20,
                                         1.07360765253904e-32,
                                         5.54413766437812e-07,
                                         0.00123229288696976,
                                         0.0367545196107322,
                                         0.15909515816594,
                                         0.185921881849067,
                                         0.0723010162970984,
                                         0.00970315933227539,
                                         0.000413963530251641,
                                         4.53225561561851e-06,
                                         8.30970043139762e-09,
                                         1.06319422416218e-12,
                                         1.13676798262827e-18,
                                         4.57913702710848e-30,
                                         1.17853806363339e-08,
                                         0.000141208442885075,
                                         0.0107948351767873,
                                         0.0923887702304664,
                                         0.188770505078168,
                                         0.120580860249048,
                                         0.0258750915527344,
                                         0.00176508478179879,
                                         3.17430189958162e-05,
                                         1.01756136322874e-07,
                                         2.57421495105878e-11,
                                         7.05444050672304e-17,
                                         1.53183326409655e-27,
                                         2.01315729199561e-10,
                                         1.30026500204733e-05,
                                         0.00254768521753399,
                                         0.0431127669731262,
                                         0.154014728955758,
                                         0.161598319660499,
                                         0.0554466247558594,
                                         0.00604774604294643,
                                         0.000178651451471767,
                                         1.00129105439106e-06,
                                         5.00842869239558e-10,
                                         3.5178539430783e-15,
                                         4.11778663126916e-25,
                                         2.80838492424534e-12,
                                         9.77795224908887e-07,
                                         0.00049104393166701,
                                         0.0164299952076934,
                                         0.102620754546124,
                                         0.17686428098883,
                                         0.0970315933227539,
                                         0.0169225712392535,
                                         0.00082112580130958,
                                         8.04646068201976e-06,
                                         7.95798307278244e-09,
                                         1.43264260597951e-13,
                                         9.03984558639628e-23,
                                         3.23368976249916e-14,
                                         6.06913426111242e-08,
                                         7.81191886054805e-05,
                                         0.00516811026421579,
                                         0.056437912966611,
                                         0.159774040956115,
                                         0.140156745910645,
                                         0.0390842650928507,
                                         0.00311512617882376,
                                         5.33718485545181e-05,
                                         1.04367990434219e-07,
                                         4.8157143180785e-12,
                                         1.63802671009379e-20,
                                         3.09328883192778e-16,
                                         3.12957944885005e-09,
                                         1.03246539019498e-05,
                                         0.00135053706695931,
                                         0.0257861847870665,
                                         0.119909256201253,
                                         0.168188095092773,
                                         0.0749925208787355,
                                         0.00981797652974448,
                                         0.000294103369185085,
                                         1.13713502083543e-06,
                                         1.34481941812533e-10,
                                         2.4658198787581e-18,
                                         2.46581987875829e-18,
                                         1.34481941812535e-10,
                                         1.13713502083542e-06,
                                         0.000294103369185086,
                                         0.00981797652974448,
                                         0.0749925208787355,
                                         0.168188095092773,
                                         0.119909256201253,
                                         0.0257861847870665,
                                         0.00135053706695931,
                                         1.03246539019498e-05,
                                         3.12957944885001e-09,
                                         3.09328883192756e-16,
                                         1.63802671009393e-20,
                                         4.81571431807858e-12,
                                         1.04367990434218e-07,
                                         5.33718485545183e-05,
                                         0.00311512617882376,
                                         0.0390842650928507,
                                         0.140156745910645,
                                         0.159774040956115,
                                         0.056437912966611,
                                         0.00516811026421578,
                                         7.81191886054809e-05,
                                         6.06913426111235e-08,
                                         3.23368976249895e-14,
                                         9.03984558639711e-23,
                                         1.43264260597954e-13,
                                         7.95798307278239e-09,
                                         8.0464606820198e-06,
                                         0.00082112580130958,
                                         0.0169225712392535,
                                         0.0970315933227539,
                                         0.17686428098883,
                                         0.102620754546124,
                                         0.0164299952076934,
                                         0.000491043931667012,
                                         9.77795224908876e-07,
                                         2.80838492424519e-12,
                                         4.11778663126956e-25,
                                         3.51785394307837e-15,
                                         5.00842869239554e-10,
                                         1.00129105439107e-06,
                                         0.000178651451471767,
                                         0.00604774604294643,
                                         0.0554466247558594,
                                         0.161598319660499,
                                         0.154014728955758,
                                         0.0431127669731262,
                                         0.002547685217534,
                                         1.30026500204731e-05,
                                         2.01315729199551e-10,
                                         1.53183326409671e-27,
                                         7.05444050672318e-17,
                                         2.57421495105876e-11,
                                         1.01756136322874e-07,
                                         3.17430189958162e-05,
                                         0.00176508478179879,
                                         0.0258750915527344,
                                         0.120580860249048,
                                         0.188770505078168,
                                         0.0923887702304662,
                                         0.0107948351767874,
                                         0.000141208442885074,
                                         1.17853806363334e-08,
                                         4.579137027109e-30,
                                         1.1367679826283e-18,
                                         1.06319422416217e-12,
                                         8.30970043139766e-09,
                                         4.53225561561851e-06,
                                         0.000413963530251641,
                                         0.00970315933227539,
                                         0.0723010162970984,
                                         0.185921881849067,
                                         0.159095158165939,
                                         0.0367545196107323,
                                         0.00123229288696975,
                                         5.54413766437792e-07,
                                         1.07360765253917e-32,
                                         1.43671592670894e-20,
                                         3.44405615611122e-14,
                                         5.32230724568053e-10,
                                         5.07540119646043e-07,
                                         7.6146230390518e-05,
                                         0.0028538703918457,
                                         0.034001669473141,
                                         0.143620584702748,
                                         0.214874339302389,
                                         0.098151129388944,
                                         0.00843445517457216,
                                         2.04556937307084e-05,
                                         1.90183992612468e-35,
                                         1.37194399307698e-22,
                                         8.42935302890942e-16,
                                         2.57561484421087e-11,
                                         4.29430377694845e-08,
                                         1.05828133953669e-05,
                                         0.000634193420410156,
                                         0.0120815452241874,
                                         0.0838241730508139,
                                         0.219269663036696,
                                         0.198036961791112,
                                         0.0436180785825158,
                                         0.000570244129541019,
                                         2.39377038699674e-38,
                                         9.30854979731316e-25,
                                         1.46588003257776e-17,
                                         8.85609087951111e-13,
                                         2.58163782631509e-09,
                                         1.04504269967871e-06,
                                         0.000100135803222656,
                                         0.00305017596130827,
                                         0.0347617820610062,
                                         0.158983740851641,
                                         0.283907833859898,
                                         0.160271460387387,
                                         0.0112950355200223,
                                         1.90819769059875e-41,
                                         4.00000023217602e-27,
                                         1.61448852156947e-19,
                                         1.92857066214236e-14,
                                         9.82947256306605e-11,
                                         6.53580790654648e-08,
                                         1.00135803222656e-05,
                                         0.000487707742742549,
                                         0.00912992339141563,
                                         0.0730061119867251,
                                         0.257775028553971,
                                         0.372973695877137,
                                         0.141692455038147,
                                         7.24344133339954e-45,
                                         8.18500053115352e-30,
                                         8.46744139503075e-22,
                                         1.99990677897184e-16,
                                         1.78215641845319e-12,
                                         1.9464588569037e-09,
                                         4.76837158203125e-07,
                                         3.71342889172689e-05,
                                         0.00114186021038895,
                                         0.0159641733199441,
                                         0.111451268638169,
                                         0.413314764242921,
                                         0.846421243215255,

                                         0.865440952918766,
                                         0.46523742953571,
                                         0.149928595593012,
                                         0.0281429861465616,
                                         0.00294125060631578,
                                         0.000160061814502894,
                                         4.11298705468267e-06,
                                         4.31986521879083e-08,
                                         1.48920482299312e-10,
                                         1.18650594779358e-13,
                                         1.19196233566695e-17,
                                         4.64215015386872e-23,
                                         4.23739651203593e-31,
                                         3.63307445935053e-46,
                                         0.12550151494833,
                                         0.362568729733899,
                                         0.297754135082443,
                                         0.109530917766034,
                                         0.0197599782426905,
                                         0.00173500480903054,
                                         6.9526991895509e-05,
                                         1.12697086615761e-06,
                                         6.05870875851675e-09,
                                         7.78850452823503e-12,
                                         1.35062434070489e-15,
                                         1.0308238182242e-20,
                                         2.39784991667608e-28,
                                         1.10484501938328e-42,
                                         0.00866644927746057,
                                         0.134550943582098,
                                         0.281586503635048,
                                         0.20299436470249,
                                         0.0632152121364018,
                                         0.00895559273149459,
                                         0.000559667699014473,
                                         1.40002530815681e-05,
                                         1.17377924027008e-07,
                                         2.43455134958566e-10,
                                         7.2876468177511e-14,
                                         1.09000982098037e-18,
                                         6.46138462679244e-26,
                                         1.59996002141663e-39,
                                         0.000379023185253888,
                                         0.0316239050005111,
                                         0.168654609519996,
                                         0.238266827877396,
                                         0.128082289921282,
                                         0.0292765794143447,
                                         0.0028532471446615,
                                         0.000110151757318228,
                                         1.44020766559616e-06,
                                         4.81965717523683e-09,
                                         2.49041892763618e-12,
                                         7.29976238025762e-17,
                                         1.1027105701306e-23,
                                         1.46740247060919e-36,
                                         1.17779721232969e-05,
                                         0.00528110026838644,
                                         0.0717736054667527,
                                         0.198711660173286,
                                         0.184389732771001,
                                         0.0680027537668234,
                                         0.0103354337543828,
                                         0.00061578221475385,
                                         1.25557881575052e-05,
                                         6.77943535513024e-08,
                                         6.04696795522158e-11,
                                         3.47349920043076e-15,
                                         1.33714236418268e-21,
                                         9.56245773118427e-34,
                                         2.76529628615026e-07,
                                         0.000666345904752342,
                                         0.0230779758197589,
                                         0.125213029645983,
                                         0.200562987271999,
                                         0.119343582117599,
                                         0.0282868395680138,
                                         0.00260093354399287,
                                         8.27045197986978e-05,
                                         7.20505504922705e-07,
                                         1.10935189395379e-09,
                                         1.2487976891863e-13,
                                         1.22506790726561e-19,
                                         4.70821421530528e-31,
                                         5.09216695080012e-09,
                                         6.59424179996967e-05,
                                         0.00581996643663586,
                                         0.0618821666645857,
                                         0.171101831947274,
                                         0.164271209086366,
                                         0.0607197482371136,
                                         0.00861630741491232,
                                         0.000427271892567345,
                                         6.00580044785899e-06,
                                         1.5962128636537e-08,
                                         3.52133047132628e-12,
                                         8.80303629539975e-18,
                                         1.81816258017407e-28,
                                         7.53508375854595e-11,
                                         5.24390106894227e-06,
                                         0.00117941794953352,
                                         0.0245757049986378,
                                         0.117295949750729,
                                         0.181697237761924,
                                         0.104736964699806,
                                         0.0229370521444138,
                                         0.00177379689811252,
                                         4.02280498301917e-05,
                                         1.84559655312365e-07,
                                         7.97895416019637e-11,
                                         5.0831076677082e-16,
                                         5.64200807032257e-26,
                                         9.10580538713388e-13,
                                         3.40556317540565e-07,
                                         0.000195191011914179,
                                         0.00797060502994097,
                                         0.0656683648460387,
                                         0.164126997999474,
                                         0.147541717633841,
                                         0.0498653472833887,
                                         0.00601379040036064,
                                         0.000220055329155853,
                                         1.74271979434028e-06,
                                         1.47648821491246e-09,
                                         2.39701612040344e-14,
                                         1.42981391875338e-23,
                                         9.08262614427549e-15,
                                         1.82551828240876e-08,
                                         2.66633486949773e-05,
                                         0.00213372962381729,
                                         0.0303453522312744,
                                         0.122369876343241,
                                         0.171550694478063,
                                         0.0894793639413605,
                                         0.0168288902456375,
                                         0.000993568000162662,
                                         1.35825449797172e-05,
                                         2.25515706119134e-08,
                                         9.32986575630197e-13,
                                         2.99080523443674e-21,
                                         7.5263587520429e-17,
                                         8.12950000199107e-10,
                                         3.02586795597996e-06,
                                         0.00047453460440335,
                                         0.0116495343573374,
                                         0.0757965466905489,
                                         0.165710691330641,
                                         0.133391246752771,
                                         0.0391239839267974,
                                         0.00372686536718126,
                                         8.79458079187916e-05,
                                         2.86156757470324e-07,
                                         3.01689529850339e-11,
                                         5.19729235104495e-19,
                                         5.19729235104495e-19,
                                         3.01689529850344e-11,
                                         2.86156757470324e-07,
                                         8.79458079187913e-05,
                                         0.00372686536718125,
                                         0.0391239839267974,
                                         0.133391246752771,
                                         0.165710691330642,
                                         0.0757965466905489,
                                         0.0116495343573374,
                                         0.000474534604403352,
                                         3.02586795597996e-06,
                                         8.12950000199094e-10,
                                         7.5263587520429e-17,
                                         2.99080523443674e-21,
                                         9.32986575630214e-13,
                                         2.25515706119134e-08,
                                         1.35825449797171e-05,
                                         0.000993568000162659,
                                         0.0168288902456375,
                                         0.0894793639413604,
                                         0.171550694478064,
                                         0.122369876343241,
                                         0.0303453522312745,
                                         0.0021337296238173,
                                         2.66633486949773e-05,
                                         1.82551828240874e-08,
                                         9.08262614427549e-15,
                                         1.42981391875338e-23,
                                         2.39701612040348e-14,
                                         1.47648821491246e-09,
                                         1.74271979434027e-06,
                                         0.000220055329155853,
                                         0.00601379040036064,
                                         0.0498653472833886,
                                         0.147541717633841,
                                         0.164126997999474,
                                         0.0656683648460388,
                                         0.00797060502994099,
                                         0.000195191011914179,
                                         3.40556317540561e-07,
                                         9.10580538713388e-13,
                                         5.64200807032257e-26,
                                         5.08310766770832e-16,
                                         7.97895416019637e-11,
                                         1.84559655312364e-07,
                                         4.02280498301916e-05,
                                         0.00177379689811252,
                                         0.0229370521444138,
                                         0.104736964699806,
                                         0.181697237761924,
                                         0.117295949750729,
                                         0.0245757049986379,
                                         0.00117941794953352,
                                         5.24390106894222e-06,
                                         7.53508375854595e-11,
                                         1.81816258017407e-28,
                                         8.80303629539995e-18,
                                         3.52133047132628e-12,
                                         1.59621286365369e-08,
                                         6.00580044785897e-06,
                                         0.000427271892567345,
                                         0.00861630741491231,
                                         0.0607197482371137,
                                         0.164271209086366,
                                         0.171101831947274,
                                         0.0618821666645858,
                                         0.00581996643663586,
                                         6.5942417999696e-05,
                                         5.09216695080012e-09,
                                         4.70821421530528e-31,
                                         1.22506790726564e-19,
                                         1.2487976891863e-13,
                                         1.10935189395378e-09,
                                         7.20505504922702e-07,
                                         8.27045197986978e-05,
                                         0.00260093354399286,
                                         0.0282868395680138,
                                         0.119343582117599,
                                         0.200562987271999,
                                         0.125213029645983,
                                         0.0230779758197589,
                                         0.000666345904752337,
                                         2.76529628615026e-07,
                                         9.56245773118427e-34,
                                         1.33714236418272e-21,
                                         3.47349920043076e-15,
                                         6.04696795522154e-11,
                                         6.77943535513022e-08,
                                         1.25557881575052e-05,
                                         0.000615782214753849,
                                         0.0103354337543828,
                                         0.0680027537668234,
                                         0.184389732771001,
                                         0.198711660173286,
                                         0.0717736054667527,
                                         0.00528110026838641,
                                         1.17779721232969e-05,
                                         1.46740247060919e-36,
                                         1.10271057013063e-23,
                                         7.29976238025762e-17,
                                         2.49041892763617e-12,
                                         4.81965717523681e-09,
                                         1.44020766559616e-06,
                                         0.000110151757318228,
                                         0.0028532471446615,
                                         0.0292765794143447,
                                         0.128082289921282,
                                         0.238266827877396,
                                         0.168654609519996,
                                         0.0316239050005109,
                                         0.000379023185253888,
                                         1.59996002141663e-39,
                                         6.46138462679264e-26,
                                         1.09000982098037e-18,
                                         7.28764681775105e-14,
                                         2.43455134958565e-10,
                                         1.17377924027008e-07,
                                         1.40002530815681e-05,
                                         0.000559667699014473,
                                         0.00895559273149459,
                                         0.0632152121364018,
                                         0.202994364702491,
                                         0.281586503635048,
                                         0.134550943582097,
                                         0.00866644927746057,
                                         1.10484501938328e-42,
                                         2.39784991667616e-28,
                                         1.0308238182242e-20,
                                         1.35062434070488e-15,
                                         7.788504528235e-12,
                                         6.05870875851675e-09,
                                         1.12697086615761e-06,
                                         6.95269918955091e-05,
                                         0.00173500480903054,
                                         0.0197599782426905,
                                         0.109530917766034,
                                         0.297754135082443,
                                         0.362568729733898,
                                         0.12550151494833,
                                         3.63307445935053e-46,
                                         4.23739651203606e-31,
                                         4.64215015386872e-23,
                                         1.19196233566694e-17,
                                         1.18650594779357e-13,
                                         1.48920482299312e-10,
                                         4.31986521879082e-08,
                                         4.11298705468267e-06,
                                         0.000160061814502894,
                                         0.00294125060631578,
                                         0.0281429861465616,
                                         0.149928595593012,
                                         0.46523742953571,
                                         0.865440952918766,

                                         0.881210671461403,
                                         0.512128892618063,
                                         0.190603495857982,
                                         0.0444490177105533,
                                         0.00627925945333513,
                                         0.000511538667615648,
                                         2.23999497112793e-05,
                                         4.76837158203125e-07,
                                         4.26222823686167e-09,
                                         1.28238127959813e-11,
                                         9.13197670586721e-15,
                                         8.38777519292428e-19,
                                         3.05268330945685e-24,
                                         2.65985463184246e-32,
                                         2.22259792872893e-47,
                                         0.111772828317063,
                                         0.348224609125413,
                                         0.328740021980145,
                                         0.149173010257967,
                                         0.0360117886523647,
                                         0.0046682272195116,
                                         0.000312819931826054,
                                         1.00135803222656e-05,
                                         1.34594687251585e-07,
                                         6.19701061902022e-10,
                                         7.02210177913412e-13,
                                         1.102192079176e-16,
                                         7.8054621764044e-22,
                                         1.72510808197419e-29,
                                         7.72757096523639e-44,
                                         0.00675108286198655,
                                         0.112750985476341,
                                         0.269994553390335,
                                         0.23839604429431,
                                         0.0983471236180478,
                                         0.02028645726596,
                                         0.00208028336391825,
                                         0.000100135803222656,
                                         2.02395027622037e-06,
                                         1.42602788017869e-08,
                                         2.57128460764832e-11,
                                         6.89681369587626e-15,
                                         9.50377618813189e-20,
                                         5.32789212446047e-27,
                                         1.27939840239366e-40,
                                         0.00025825157730413,
                                         0.0231213708491658,
                                         0.140439661104212,
                                         0.241290477167187,
                                         0.17010261049878,
                                         0.0558332335594616,
                                         0.00876158989504829,
                                         0.000634193420410156,
                                         1.92754313663103e-05,
                                         2.07828995630588e-07,
                                         5.96300984558613e-10,
                                         2.73320401235493e-13,
                                         7.32868614338514e-18,
                                         1.04214195996086e-24,
                                         1.34153174000002e-37,
                                         7.01928206292634e-06,
                                         0.00336889115881186,
                                         0.0519044639206885,
                                         0.173524773900996,
                                         0.20904532626207,
                                         0.109184130080604,
                                         0.0262194461752687,
                                         0.0028538703918457,
                                         0.000130433317582103,
                                         2.15210981769345e-06,
                                         9.82564502916635e-09,
                                         7.69618984746895e-12,
                                         4.01546840898871e-16,
                                         1.4483666267513e-22,
                                         9.99484971004421e-35,
                                         1.44148062527113e-07,
                                         0.00037087433792998,
                                         0.0144939263557825,
                                         0.0942864306927792,
                                         0.194104833068021,
                                         0.161321651421071,
                                         0.0592830460201075,
                                         0.00970315933227539,
                                         0.000666867233346014,
                                         1.68379463675278e-05,
                                         1.22327195051296e-07,
                                         1.63736649052692e-10,
                                         1.66231263999559e-14,
                                         1.52088557022895e-20,
                                         5.62623654310681e-32,
                                         2.32174618027876e-09,
                                         3.20225861842563e-05,
                                         0.0031743675136606,
                                         0.0401815530730257,
                                         0.141358540092751,
                                         0.186945758135572,
                                         0.105130159005005,
                                         0.0258750915527344,
                                         0.0026741147043149,
                                         0.00010332457563265,
                                         1.19446871653851e-06,
                                         2.73215817832823e-09,
                                         5.39733054560287e-13,
                                         1.25257686155027e-18,
                                         2.48398816035788e-29,
                                         3.00500470787173e-11,
                                         2.22182837609414e-06,
                                         0.000558666780853448,
                                         0.0137603261900573,
                                         0.0827241360431704,
                                         0.174085684005263,
                                         0.14981269590433,
                                         0.0554466247558594,
                                         0.00861678331912344,
                                         0.000509498166425442,
                                         9.37240461338553e-06,
                                         3.6634499881004e-08,
                                         1.40821774252908e-11,
                                         8.28965997509883e-17,
                                         8.81263012166824e-27,
                                         3.17629187304981e-13,
                                         1.25895278994758e-07,
                                         8.02958907751709e-05,
                                         0.00384835890950605,
                                         0.039535502099878,
                                         0.132390045309875,
                                         0.174347107633132,
                                         0.0970315933227539,
                                         0.0226754096586118,
                                         0.00205175948704536,
                                         6.00581718049924e-05,
                                         4.0116175688915e-07,
                                         3.00058156797143e-10,
                                         4.48036998233592e-15,
                                         2.55332673892261e-24,
                                         2.7711398534556e-15,
                                         5.88804572998244e-09,
                                         9.52569312973196e-06,
                                         0.000888352242289984,
                                         0.0155957071440884,
                                         0.0831017936413835,
                                         0.167472563767164,
                                         0.140156745910645,
                                         0.0492524575926934,
                                         0.00681982246561459,
                                         0.000317655208682062,
                                         3.62586455047358e-06,
                                         5.27720368054446e-09,
                                         1.99872863689476e-13,
                                         6.10618612647733e-22,
                                         2.00852336524182e-17,
                                         2.2877749861022e-10,
                                         9.38815510251696e-07,
                                         0.000170362984673175,
                                         0.00511096947685707,
                                         0.0433357051272307,
                                         0.133645085701749,
                                         0.168188095092773,
                                         0.0888752985885097,
                                         0.0188321580937482,
                                         0.00139579056125424,
                                         2.72260119776512e-05,
                                         7.71050244748227e-08,
                                         7.40754321984879e-12,
                                         1.21314883707876e-19,
                                         1.21314883707864e-19,
                                         7.40754321984879e-12,
                                         7.71050244748221e-08,
                                         2.72260119776513e-05,
                                         0.00139579056125424,
                                         0.0188321580937482,
                                         0.0888752985885097,
                                         0.168188095092773,
                                         0.133645085701749,
                                         0.0433357051272306,
                                         0.00511096947685707,
                                         0.000170362984673175,
                                         9.38815510251703e-07,
                                         2.2877749861022e-10,
                                         2.008523365242e-17,
                                         6.10618612647665e-22,
                                         1.99872863689476e-13,
                                         5.27720368054441e-09,
                                         3.6258645504736e-06,
                                         0.000317655208682062,
                                         0.00681982246561459,
                                         0.0492524575926934,
                                         0.140156745910645,
                                         0.167472563767164,
                                         0.0831017936413835,
                                         0.0155957071440884,
                                         0.000888352242289981,
                                         9.52569312973202e-06,
                                         5.88804572998244e-09,
                                         2.77113985345583e-15,
                                         2.5533267389223e-24,
                                         4.48036998233592e-15,
                                         3.0005815679714e-10,
                                         4.01161756889152e-07,
                                         6.00581718049924e-05,
                                         0.00205175948704536,
                                         0.0226754096586118,
                                         0.0970315933227539,
                                         0.174347107633132,
                                         0.132390045309875,
                                         0.039535502099878,
                                         0.00384835890950604,
                                         8.02958907751713e-05,
                                         1.25895278994758e-07,
                                         3.17629187305005e-13,
                                         8.8126301216671e-27,
                                         8.28965997509883e-17,
                                         1.40821774252906e-11,
                                         3.66344998810042e-08,
                                         9.37240461338553e-06,
                                         0.000509498166425442,
                                         0.00861678331912344,
                                         0.0554466247558594,
                                         0.14981269590433,
                                         0.174085684005263,
                                         0.0827241360431704,
                                         0.0137603261900573,
                                         0.00055866678085345,
                                         2.22182837609414e-06,
                                         3.00500470787193e-11,
                                         2.48398816035754e-29,
                                         1.25257686155027e-18,
                                         5.39733054560281e-13,
                                         2.73215817832825e-09,
                                         1.19446871653851e-06,
                                         0.00010332457563265,
                                         0.0026741147043149,
                                         0.0258750915527344,
                                         0.105130159005005,
                                         0.186945758135572,
                                         0.141358540092751,
                                         0.0401815530730256,
                                         0.00317436751366062,
                                         3.20225861842563e-05,
                                         2.32174618027889e-09,
                                         5.62623654310598e-32,
                                         1.52088557022895e-20,
                                         1.66231263999557e-14,
                                         1.63736649052693e-10,
                                         1.22327195051296e-07,
                                         1.68379463675278e-05,
                                         0.000666867233346014,
                                         0.00970315933227539,
                                         0.0592830460201075,
                                         0.161321651421071,
                                         0.194104833068021,
                                         0.094286430692779,
                                         0.0144939263557826,
                                         0.00037087433792998,
                                         1.4414806252712e-07,
                                         9.99484971004264e-35,
                                         1.4483666267513e-22,
                                         4.01546840898866e-16,
                                         7.696189847469e-12,
                                         9.82564502916635e-09,
                                         2.15210981769345e-06,
                                         0.000130433317582103,
                                         0.0028538703918457,
                                         0.0262194461752687,
                                         0.109184130080604,
                                         0.20904532626207,
                                         0.173524773900996,
                                         0.0519044639206886,
                                         0.00336889115881186,
                                         7.0192820629266e-06,
                                         1.3415317399998e-37,
                                         1.04214195996086e-24,
                                         7.32868614338504e-18,
                                         2.73320401235495e-13,
                                         5.96300984558613e-10,
                                         2.07828995630588e-07,
                                         1.92754313663103e-05,
                                         0.000634193420410156,
                                         0.00876158989504829,
                                         0.0558332335594616,
                                         0.17010261049878,
                                         0.241290477167187,
                                         0.140439661104212,
                                         0.0231213708491658,
                                         0.000258251577304137,
                                         1.27939840239344e-40,
                                         5.32789212446047e-27,
                                         9.50377618813176e-20,
                                         6.89681369587631e-15,
                                         2.57128460764832e-11,
                                         1.42602788017869e-08,
                                         2.02395027622037e-06,
                                         0.000100135803222656,
                                         0.00208028336391825,
                                         0.02028645726596,
                                         0.0983471236180478,
                                         0.23839604429431,
                                         0.269994553390335,
                                         0.112750985476341,
                                         0.00675108286198668,
                                         7.72757096523496e-44,
                                         1.72510808197419e-29,
                                         7.80546217640429e-22,
                                         1.10219207917601e-16,
                                         7.02210177913412e-13,
                                         6.19701061902022e-10,
                                         1.34594687251585e-07,
                                         1.00135803222656e-05,
                                         0.000312819931826054,
                                         0.0046682272195116,
                                         0.0360117886523647,
                                         0.149173010257967,
                                         0.328740021980145,
                                         0.348224609125413,
                                         0.111772828317065,
                                         2.2225979287285e-47,
                                         2.65985463184246e-32,
                                         3.05268330945681e-24,
                                         8.38777519292435e-19,
                                         9.13197670586721e-15,
                                         1.28238127959813e-11,
                                         4.26222823686167e-09,
                                         4.76837158203125e-07,
                                         2.23999497112793e-05,
                                         0.000511538667615648,
                                         0.00627925945333513,
                                         0.0444490177105533,
                                         0.190603495857982,
                                         0.512128892618063,
                                         0.881210671461403,

                                         0.894414359708785,
                                         0.55422850439908,
                                         0.232119603813995,
                                         0.0646086950079665,
                                         0.0116471951916026,
                                         0.00131033711610896,
                                         8.73369120862596e-05,
                                         3.20753044143477e-06,
                                         5.85954892714038e-08,
                                         4.59216844674167e-10,
                                         1.23615459600275e-12,
                                         8.02887834107084e-16,
                                         6.85158777564381e-20,
                                         2.35850684313659e-25,
                                         1.97781666479102e-33,
                                         1.61803817639156e-48,
                                         0.100069860881458,
                                         0.331733224886584,
                                         0.35107810834337,
                                         0.189052186504174,
                                         0.0577696036195359,
                                         0.010228822885968,
                                         0.00102808163827339,
                                         5.56690200818989e-05,
                                         1.48888098881388e-06,
                                         1.72038889200518e-08,
                                         6.98342963314367e-11,
                                         7.13863920486008e-14,
                                         1.03261654635259e-17,
                                         6.87675894639769e-23,
                                         1.45721793848827e-30,
                                         6.37768141123997e-45,
                                         0.00533148920463115,
                                         0.0945517972668544,
                                         0.252857816837941,
                                         0.263422668814542,
                                         0.136445157476296,
                                         0.0380232276710724,
                                         0.00576286011376582,
                                         0.000460083937955977,
                                         1.8015092209928e-05,
                                         3.06913615052899e-07,
                                         1.87864811141195e-09,
                                         3.0224330776949e-12,
                                         7.4108434474098e-16,
                                         9.54797266157675e-21,
                                         5.11262207669059e-28,
                                         1.19706458565962e-41,
                                         0.000179897910281776,
                                         0.0170680125091005,
                                         0.115340376430498,
                                         0.232464656248595,
                                         0.204102917343583,
                                         0.0895168203100577,
                                         0.0204588349678002,
                                         0.00240820200260973,
                                         0.000138052838465941,
                                         3.46767211241009e-06,
                                         3.20076998105468e-08,
                                         8.10457602599173e-11,
                                         3.36843789408613e-14,
                                         8.39597001280896e-19,
                                         1.13604417560546e-25,
                                         1.42299931417003e-38,
                                         4.31304381928365e-06,
                                         0.00218915384968497,
                                         0.0373823425271251,
                                         0.145760848255086,
                                         0.21693042411675,
                                         0.14974091492669,
                                         0.0516064384669181,
                                         0.00895630597782857,
                                         0.000751682360827499,
                                         2.78381303757925e-05,
                                         3.87474974065072e-07,
                                         1.54413076237809e-09,
                                         1.08785150621557e-12,
                                         5.24578827737886e-17,
                                         1.79360545221173e-23,
                                         1.2019100493699e-35,
                                         7.81282372342808e-08,
                                         0.000212146572167215,
                                         0.00915415082946075,
                                         0.0690543823081395,
                                         0.174203999485294,
                                         0.189252977352122,
                                         0.0983542837710378,
                                         0.0251669933481312,
                                         0.00309235810596816,
                                         0.000168852893184904,
                                         3.54404521281729e-06,
                                         2.22281978107107e-08,
                                         2.65446555153836e-11,
                                         2.47637859250598e-15,
                                         2.13956268344016e-21,
                                         7.67018157919538e-33,
                                         1.10999748834762e-09,
                                         1.61244744693234e-05,
                                         0.00175816419099444,
                                         0.0256585091024387,
                                         0.109719947993474,
                                         0.187600838024341,
                                         0.147018669299203,
                                         0.0554655803848553,
                                         0.00997780513798542,
                                         0.000803279647609095,
                                         2.5424045950979e-05,
                                         2.50965628574881e-07,
                                         5.08012484298017e-10,
                                         9.16881350076381e-14,
                                         2.0017642515285e-19,
                                         3.83909747842502e-30,
                                         1.267244611664e-11,
                                         9.84826226973247e-07,
                                         0.000271347162037084,
                                         0.00766118725947368,
                                         0.055531263023184,
                                         0.149434651230263,
                                         0.176594096729423,
                                         0.0982291251050867,
                                         0.0258704952094881,
                                         0.00307078568553985,
                                         0.000146559698414739,
                                         2.27692500149488e-06,
                                         7.81261032698311e-09,
                                         2.7279331521921e-12,
                                         1.50496129366774e-17,
                                         1.54410603477478e-27,
                                         1.18152724382768e-13,
                                         4.91222750233896e-08,
                                         3.42007765203162e-05,
                                         0.0018681234293656,
                                         0.0229527295382271,
                                         0.0972103897074304,
                                         0.173230638243465,
                                         0.142069816585965,
                                         0.054779655350543,
                                         0.00958687535892353,
                                         0.000689968541855294,
                                         1.68705031509519e-05,
                                         9.81211820045342e-08,
                                         6.62825393253319e-11,
                                         9.24022531555378e-16,
                                         5.07189122536058e-25,
                                         9.09263632531348e-16,
                                         2.02236775856128e-09,
                                         3.5580286752412e-06,
                                         0.000375991347562456,
                                         0.00783057982855438,
                                         0.0521959550245549,
                                         0.140260706930681,
                                         0.169600121013666,
                                         0.095740709873705,
                                         0.024704009678566,
                                         0.00268106171965151,
                                         0.000103173969290801,
                                         1.01716679252284e-06,
                                         1.32931329013256e-09,
                                         4.68276740084115e-14,
                                         1.37507230636196e-22,
                                         5.81321392671526e-18,
                                         6.91707041542978e-11,
                                         3.0751288958924e-07,
                                         6.28681293081315e-05,
                                         0.00221939143956977,
                                         0.0232831328124407,
                                         0.0943469238326546,
                                         0.168201900674339,
                                         0.13901267297337,
                                         0.0528856954516605,
                                         0.00865495345547255,
                                         0.000524194724388301,
                                         8.75995708484441e-06,
                                         2.2148073623267e-08,
                                         1.97152829603931e-12,
                                         3.0971450366264e-20,
                                         3.09714503662676e-20,
                                         1.97152829603931e-12,
                                         2.21480736232668e-08,
                                         8.75995708484446e-06,
                                         0.000524194724388301,
                                         0.00865495345547255,
                                         0.0528856954516606,
                                         0.13901267297337,
                                         0.168201900674338,
                                         0.0943469238326545,
                                         0.0232831328124407,
                                         0.00221939143956977,
                                         6.28681293081312e-05,
                                         3.07512889589242e-07,
                                         6.91707041542978e-11,
                                         5.81321392671465e-18,
                                         1.37507230636213e-22,
                                         4.68276740084115e-14,
                                         1.32931329013255e-09,
                                         1.01716679252285e-06,
                                         0.000103173969290801,
                                         0.00268106171965151,
                                         0.024704009678566,
                                         0.0957407098737051,
                                         0.169600121013666,
                                         0.140260706930681,
                                         0.0521959550245549,
                                         0.00783057982855438,
                                         0.000375991347562455,
                                         3.55802867524123e-06,
                                         2.02236775856128e-09,
                                         9.09263632531262e-16,
                                         5.07189122536127e-25,
                                         9.24022531555378e-16,
                                         6.62825393253312e-11,
                                         9.81211820045348e-08,
                                         1.68705031509519e-05,
                                         0.000689968541855294,
                                         0.00958687535892355,
                                         0.054779655350543,
                                         0.142069816585964,
                                         0.173230638243465,
                                         0.0972103897074304,
                                         0.0229527295382271,
                                         0.00186812342936559,
                                         3.42007765203164e-05,
                                         4.91222750233896e-08,
                                         1.18152724382758e-13,
                                         1.54410603477501e-27,
                                         1.50496129366774e-17,
                                         2.72793315219207e-12,
                                         7.81261032698316e-09,
                                         2.27692500149488e-06,
                                         0.000146559698414739,
                                         0.00307078568553985,
                                         0.0258704952094881,
                                         0.0982291251050865,
                                         0.176594096729423,
                                         0.149434651230263,
                                         0.055531263023184,
                                         0.00766118725947365,
                                         0.000271347162037086,
                                         9.84826226973247e-07,
                                         1.26724461166391e-11,
                                         3.83909747842562e-30,
                                         2.0017642515285e-19,
                                         9.1688135007637e-14,
                                         5.0801248429802e-10,
                                         2.50965628574881e-07,
                                         2.5424045950979e-05,
                                         0.000803279647609097,
                                         0.00997780513798541,
                                         0.0554655803848551,
                                         0.147018669299203,
                                         0.187600838024341,
                                         0.109719947993474,
                                         0.0256585091024386,
                                         0.00175816419099445,
                                         1.61244744693234e-05,
                                         1.10999748834755e-09,
                                         7.67018157919666e-33,
                                         2.13956268344016e-21,
                                         2.47637859250595e-15,
                                         2.65446555153838e-11,
                                         2.22281978107107e-08,
                                         3.54404521281729e-06,
                                         0.000168852893184904,
                                         0.00309235810596816,
                                         0.0251669933481311,
                                         0.0983542837710377,
                                         0.189252977352122,
                                         0.174203999485294,
                                         0.0690543823081393,
                                         0.00915415082946079,
                                         0.000212146572167215,
                                         7.81282372342767e-08,
                                         1.20191004937011e-35,
                                         1.79360545221173e-23,
                                         5.24578827737878e-17,
                                         1.08785150621557e-12,
                                         1.54413076237809e-09,
                                         3.87474974065072e-07,
                                         2.78381303757926e-05,
                                         0.000751682360827498,
                                         0.00895630597782855,
                                         0.051606438466918,
                                         0.14974091492669,
                                         0.21693042411675,
                                         0.145760848255086,
                                         0.0373823425271252,
                                         0.00218915384968497,
                                         4.31304381928347e-06,
                                         1.4229993141703e-38,
                                         1.13604417560546e-25,
                                         8.39597001280884e-19,
                                         3.36843789408615e-14,
                                         8.10457602599173e-11,
                                         3.20076998105469e-08,
                                         3.4676721124101e-06,
                                         0.00013805283846594,
                                         0.00240820200260972,
                                         0.0204588349678002,
                                         0.0895168203100577,
                                         0.204102917343583,
                                         0.232464656248595,
                                         0.115340376430499,
                                         0.0170680125091005,
                                         0.000179897910281771,
                                         1.19706458565986e-41,
                                         5.11262207669059e-28,
                                         9.5479726615766e-21,
                                         7.41084344740986e-16,
                                         3.0224330776949e-12,
                                         1.87864811141195e-09,
                                         3.06913615052899e-07,
                                         1.8015092209928e-05,
                                         0.000460083937955975,
                                         0.00576286011376582,
                                         0.0380232276710724,
                                         0.136445157476296,
                                         0.263422668814542,
                                         0.252857816837942,
                                         0.0945517972668544,
                                         0.00533148920463104,
                                         6.3776814112413e-45,
                                         1.45721793848827e-30,
                                         6.87675894639758e-23,
                                         1.0326165463526e-17,
                                         7.13863920486008e-14,
                                         6.98342963314367e-11,
                                         1.72038889200519e-08,
                                         1.48888098881387e-06,
                                         5.56690200818987e-05,
                                         0.00102808163827339,
                                         0.010228822885968,
                                         0.0577696036195359,
                                         0.189052186504174,
                                         0.35107810834337,
                                         0.331733224886584,
                                         0.100069860881457,
                                         1.61803817639192e-48,
                                         1.97781666479102e-33,
                                         2.35850684313655e-25,
                                         6.85158777564387e-20,
                                         8.02887834107084e-16,
                                         1.23615459600275e-12,
                                         4.59216844674169e-10,
                                         5.85954892714036e-08,
                                         3.20753044143476e-06,
                                         8.73369120862596e-05,
                                         0.00131033711610896,
                                         0.0116471951916026,
                                         0.0646086950079665,
                                         0.232119603813995,
                                         0.55422850439908,
                                         0.894414359708785,

                                         0.905569364432281,
                                         0.591910180555597,
                                         0.273429284000713,
                                         0.0880935562823439,
                                         0.0194015355105124,
                                         0.00283871624503715,
                                         0.000265275321711679,
                                         1.5002685246625e-05,
                                         4.76837158203125e-07,
                                         7.67899929107403e-09,
                                         5.39822828454182e-11,
                                         1.32517072728492e-13,
                                         7.97413106563326e-17,
                                         6.40126516159918e-21,
                                         2.10369159507203e-26,
                                         1.70876597597856e-34,
                                         1.37346840586096e-49,
                                         0.0900371674644915,
                                         0.314305926478693,
                                         0.365734630107907,
                                         0.226880830239299,
                                         0.0841396121298484,
                                         0.0192041917042356,
                                         0.00267470492756167,
                                         0.000219624338041128,
                                         1.00135803222656e-05,
                                         2.31329888967025e-07,
                                         2.36108057368678e-09,
                                         8.63846093840708e-12,
                                         8.10881806506357e-15,
                                         1.09610143606569e-18,
                                         6.93584856635686e-24,
                                         1.41913730681705e-31,
                                         6.09196165714051e-46,
                                         0.00426287532380573,
                                         0.0794749083357967,
                                         0.232952753516751,
                                         0.278248074857263,
                                         0.173758321873555,
                                         0.0618658219618677,
                                         0.0128420870946662,
                                         0.00153099100235929,
                                         0.000100135803222656,
                                         3.31848284249991e-06,
                                         4.91757593470386e-08,
                                         2.68152357198088e-10,
                                         3.92656190201038e-13,
                                         8.93750671837145e-17,
                                         1.0889246731821e-21,
                                         5.61237848976175e-29,
                                         1.28669719437521e-42,
                                         0.0001278249950263,
                                         0.0127274045162723,
                                         0.0939727548491901,
                                         0.216121951264335,
                                         0.22726003423163,
                                         0.12622282283177,
                                         0.0390505981389309,
                                         0.00675922895340457,
                                         0.000634193420410156,
                                         3.01494747144679e-05,
                                         6.48669813190014e-07,
                                         5.27180345545387e-09,
                                         1.20420294994731e-11,
                                         4.61545331037169e-15,
                                         1.08275056843616e-19,
                                         1.40572973702542e-26,
                                         1.72118634908442e-39,
                                         2.7233854038683e-06,
                                         0.00144820428102352,
                                         0.0269349538927899,
                                         0.119273993155884,
                                         0.211193568996176,
                                         0.182980646220094,
                                         0.0843723095411727,
                                         0.0212032211970739,
                                         0.0028538703918457,
                                         0.000194625656670218,
                                         6.07962046830533e-06,
                                         7.36405408330232e-08,
                                         2.62401970845248e-10,
                                         1.69352850252876e-13,
                                         7.64960781534369e-18,
                                         2.50170951413276e-24,
                                         1.63591072457851e-36,
                                         4.38398268090793e-08,
                                         0.00012450483286653,
                                         0.00583306636543851,
                                         0.0497346472154304,
                                         0.148287558398704,
                                         0.200418971388827,
                                         0.137733184081669,
                                         0.050254265968508,
                                         0.00970315933227539,
                                         0.000949263615637826,
                                         4.30522300580866e-05,
                                         7.77214924646277e-07,
                                         4.3201706460608e-09,
                                         4.69501584083963e-12,
                                         4.08334733321277e-16,
                                         3.36386324500365e-22,
                                         1.17478315152022e-33,
                                         5.53500948307207e-10,
                                         8.39522630306026e-06,
                                         0.000990757603583731,
                                         0.0162653023049474,
                                         0.0816617259492375,
                                         0.172171917363049,
                                         0.176346584634536,
                                         0.0934187042161038,
                                         0.0258750915527344,
                                         0.00363131057621651,
                                         0.000239113805578035,
                                         6.43361560442856e-06,
                                         5.57859105413218e-08,
                                         1.02087232050301e-10,
                                         1.70955584079532e-14,
                                         3.54755876509277e-20,
                                         6.61676383901103e-31,
                                         5.61555125515819e-12,
                                         4.54886546162905e-07,
                                         0.000135226683593585,
                                         0.00427454333173453,
                                         0.0361373982303021,
                                         0.118853038706275,
                                         0.181434555537643,
                                         0.13954659079867,
                                         0.0554466247558594,
                                         0.0111625763074043,
                                         0.00106718112981566,
                                         4.2795053472438e-05,
                                         5.78858796267729e-07,
                                         1.78373478745687e-09,
                                         5.75141553362049e-13,
                                         3.00639023169676e-18,
                                         2.99473257196216e-28,
                                         4.65276643093936e-14,
                                         2.01288334546838e-08,
                                         1.50730871437707e-05,
                                         0.000917407187053024,
                                         0.013059905614309,
                                         0.0670043646177691,
                                         0.152446615524838,
                                         0.170235234149993,
                                         0.0970315933227539,
                                         0.0280227218840365,
                                         0.00388970309855826,
                                         0.000232475326994359,
                                         4.90530005261452e-06,
                                         2.54527062585047e-08,
                                         1.58019626375012e-11,
                                         2.08068307608552e-16,
                                         1.10691758688757e-25,
                                         3.1819467070908e-16,
                                         7.35185469750615e-10,
                                         1.38677121017569e-06,
                                         0.000162516457259389,
                                         0.00389570433883614,
                                         0.031178749368531,
                                         0.105725162917911,
                                         0.171412491752757,
                                         0.140156745910645,
                                         0.0580656091724638,
                                         0.0117019317966909,
                                         0.00104237252418687,
                                         3.43100469896864e-05,
                                         2.99778472858859e-07,
                                         3.58352228311722e-10,
                                         1.18858244601454e-14,
                                         3.37703314574366e-23,
                                         1.807819034912e-18,
                                         2.23077427038719e-11,
                                         1.05995597456944e-07,
                                         2.39173444406137e-05,
                                         0.000965411222557648,
                                         0.0120529871007388,
                                         0.0609143118389813,
                                         0.143389016849114,
                                         0.168188095092773,
                                         0.0999558139098542,
                                         0.0292468484985741,
                                         0.00388283956735618,
                                         0.000199368917568381,
                                         2.9332381406043e-06,
                                         6.75133396062689e-09,
                                         5.6407011679078e-13,
                                         8.55925099695401e-21,
                                         8.5592509969529e-21,
                                         5.64070116790794e-13,
                                         6.75133396062689e-09,
                                         2.93323814060432e-06,
                                         0.000199368917568382,
                                         0.00388283956735619,
                                         0.0292468484985741,
                                         0.0999558139098542,
                                         0.168188095092773,
                                         0.143389016849114,
                                         0.0609143118389813,
                                         0.0120529871007388,
                                         0.000965411222557645,
                                         2.39173444406136e-05,
                                         1.05995597456944e-07,
                                         2.23077427038714e-11,
                                         1.80781903491221e-18,
                                         3.37703314574318e-23,
                                         1.18858244601457e-14,
                                         3.58352228311722e-10,
                                         2.99778472858861e-07,
                                         3.43100469896865e-05,
                                         0.00104237252418688,
                                         0.0117019317966909,
                                         0.0580656091724638,
                                         0.140156745910645,
                                         0.171412491752757,
                                         0.105725162917911,
                                         0.0311787493685309,
                                         0.00389570433883613,
                                         0.000162516457259388,
                                         1.38677121017569e-06,
                                         7.351854697506e-10,
                                         3.18194670709114e-16,
                                         1.1069175868874e-25,
                                         2.08068307608559e-16,
                                         1.58019626375012e-11,
                                         2.54527062585049e-08,
                                         4.90530005261454e-06,
                                         0.00023247532699436,
                                         0.00388970309855826,
                                         0.0280227218840365,
                                         0.0970315933227539,
                                         0.170235234149993,
                                         0.152446615524838,
                                         0.067004364617769,
                                         0.013059905614309,
                                         0.00091740718705302,
                                         1.50730871437707e-05,
                                         2.01288334546834e-08,
                                         4.6527664309398e-14,
                                         2.99473257196166e-28,
                                         3.00639023169685e-18,
                                         5.75141553362049e-13,
                                         1.78373478745688e-09,
                                         5.78858796267732e-07,
                                         4.27950534724381e-05,
                                         0.00106718112981566,
                                         0.0111625763074043,
                                         0.0554466247558594,
                                         0.13954659079867,
                                         0.181434555537643,
                                         0.118853038706275,
                                         0.036137398230302,
                                         0.00427454333173451,
                                         0.000135226683593585,
                                         4.54886546162898e-07,
                                         5.61555125515865e-12,
                                         6.61676383900986e-31,
                                         3.54755876509288e-20,
                                         1.70955584079532e-14,
                                         1.02087232050301e-10,
                                         5.57859105413221e-08,
                                         6.43361560442858e-06,
                                         0.000239113805578035,
                                         0.00363131057621651,
                                         0.0258750915527344,
                                         0.0934187042161038,
                                         0.176346584634536,
                                         0.172171917363049,
                                         0.0816617259492373,
                                         0.0162653023049473,
                                         0.000990757603583731,
                                         8.39522630306014e-06,
                                         5.53500948307246e-10,
                                         1.17478315152e-33,
                                         3.36386324500377e-22,
                                         4.08334733321277e-16,
                                         4.69501584083967e-12,
                                         4.32017064606082e-09,
                                         7.7721492464628e-07,
                                         4.30522300580866e-05,
                                         0.000949263615637827,
                                         0.00970315933227539,
                                         0.0502542659685079,
                                         0.137733184081669,
                                         0.200418971388826,
                                         0.148287558398704,
                                         0.0497346472154302,
                                         0.00583306636543851,
                                         0.000124504832866529,
                                         4.38398268090819e-08,
                                         1.63591072457819e-36,
                                         2.50170951413285e-24,
                                         7.64960781534369e-18,
                                         1.69352850252878e-13,
                                         2.62401970845249e-10,
                                         7.36405408330235e-08,
                                         6.07962046830533e-06,
                                         0.000194625656670218,
                                         0.0028538703918457,
                                         0.0212032211970739,
                                         0.0843723095411726,
                                         0.182980646220094,
                                         0.211193568996176,
                                         0.119273993155884,
                                         0.0269349538927899,
                                         0.0014482042810235,
                                         2.72338540386843e-06,
                                         1.72118634908405e-39,
                                         1.40572973702547e-26,
                                         1.08275056843616e-19,
                                         4.61545331037173e-15,
                                         1.20420294994732e-11,
                                         5.27180345545389e-09,
                                         6.48669813190014e-07,
                                         3.01494747144679e-05,
                                         0.000634193420410156,
                                         0.00675922895340457,
                                         0.0390505981389309,
                                         0.12622282283177,
                                         0.22726003423163,
                                         0.216121951264335,
                                         0.0939727548491901,
                                         0.0127274045162722,
                                         0.000127824995026305,
                                         1.28669719437492e-42,
                                         5.61237848976199e-29,
                                         1.0889246731821e-21,
                                         8.93750671837154e-17,
                                         3.9265619020104e-13,
                                         2.68152357198089e-10,
                                         4.91757593470386e-08,
                                         3.31848284249991e-06,
                                         0.000100135803222656,
                                         0.00153099100235929,
                                         0.0128420870946662,
                                         0.0618658219618677,
                                         0.173758321873555,
                                         0.278248074857263,
                                         0.232952753516751,
                                         0.0794749083357963,
                                         0.00426287532380583,
                                         6.09196165713908e-46,
                                         1.41913730681712e-31,
                                         6.93584856635686e-24,
                                         1.0961014360657e-18,
                                         8.10881806506362e-15,
                                         8.63846093840712e-12,
                                         2.36108057368678e-09,
                                         2.31329888967025e-07,
                                         1.00135803222656e-05,
                                         0.000219624338041128,
                                         0.00267470492756167,
                                         0.0192041917042356,
                                         0.0841396121298483,
                                         0.226880830239299,
                                         0.365734630107907,
                                         0.314305926478693,
                                         0.0900371674644926,
                                         1.37346840586062e-49,
                                         1.70876597597864e-34,
                                         2.10369159507203e-26,
                                         6.40126516159925e-21,
                                         7.97413106563332e-17,
                                         1.32517072728493e-13,
                                         5.39822828454182e-11,
                                         7.67899929107403e-09,
                                         4.76837158203125e-07,
                                         1.5002685246625e-05,
                                         0.000265275321711679,
                                         0.00283871624503715,
                                         0.0194015355105124,
                                         0.0880935562823439,
                                         0.273429284000713,
                                         0.591910180555597,
                                         0.905569364432281,

                                         0.915071552236501,
                                         0.625597563185532,
                                         0.313774414330193,
                                         0.114247202959321,
                                         0.0297228584774861,
                                         0.00540314688753123,
                                         0.000665719562227025,
                                         5.3362008994018e-05,
                                         2.63331749231095e-06,
                                         7.42087863604363e-08,
                                         1.07569899655494e-09,
                                         6.90482596313644e-12,
                                         1.56900816415575e-14,
                                         8.85518006402429e-18,
                                         6.75252162522306e-22,
                                         2.13427583636433e-27,
                                         1.68769777288692e-35,
                                         1.33651800180016e-50,
                                         0.0813872252408745,
                                         0.296736840558869,
                                         0.373914001596744,
                                         0.261101099097681,
                                         0.11375791455824,
                                         0.0320246276865914,
                                         0.0058252361812689,
                                         0.000669659842733789,
                                         4.66563450579362e-05,
                                         1.84708307259985e-06,
                                         3.78013463506447e-08,
                                         3.47991877120144e-10,
                                         1.16741824847459e-12,
                                         1.02034005987423e-15,
                                         1.30299110065069e-19,
                                         7.89832384297209e-25,
                                         1.56912248895799e-32,
                                         6.62692742142191e-47,
                                         0.0034469750148263,
                                         0.0670237300009813,
                                         0.212181069317924,
                                         0.284153185390668,
                                         0.207325812162571,
                                         0.0903861961691344,
                                         0.0242726091635395,
                                         0.00400181535805044,
                                         0.000393639715920772,
                                         2.18926448984728e-05,
                                         6.32564008766022e-07,
                                         8.35153262564354e-09,
                                         4.13626597273469e-11,
                                         5.59851936142454e-14,
                                         1.19728521792445e-17,
                                         1.39187434747421e-22,
                                         6.9470387550267e-30,
                                         1.56469725594855e-43,
                                         9.24596776442346e-05,
                                         0.00958778010406964,
                                         0.0762559763036614,
                                         0.19585231262108,
                                         0.239308142078151,
                                         0.16156693136261,
                                         0.0640548080122336,
                                         0.0151457987882563,
                                         0.00210338828827223,
                                         0.000164339659784841,
                                         6.70400044133788e-06,
                                         1.26939155181205e-07,
                                         9.28159860934165e-10,
                                         1.94551138776899e-12,
                                         6.96764696616018e-16,
                                         1.55345068632214e-20,
                                         1.94793716077141e-27,
                                         2.3398109759643e-40,
                                         1.76216585586861e-06,
                                         0.00097451315012706,
                                         0.0194724817369886,
                                         0.0959146721725019,
                                         0.196264495510161,
                                         0.20520267161186,
                                         0.120106682749904,
                                         0.0407293507088736,
                                         0.00798583183610465,
                                         0.000876529832631491,
                                         5.04828375574855e-05,
                                         1.37089815754708e-06,
                                         1.47984856736259e-08,
                                         4.80368518328842e-11,
                                         2.88107671684034e-14,
                                         1.23189893388186e-18,
                                         3.88088104629101e-25,
                                         2.48605878911956e-37,
                                         2.53750900601647e-08,
                                         7.48382711273485e-05,
                                         0.00375694751482596,
                                         0.0354901465525024,
                                         0.12161647539017,
                                         0.19691552071008,
                                         0.170156684189943,
                                         0.0827540378984555,
                                         0.0229080052090149,
                                         0.00353229871582388,
                                         0.000287223443658079,
                                         1.11861640752618e-05,
                                         1.782699624469e-07,
                                         8.96152046874326e-10,
                                         9.00098236618238e-13,
                                         7.38106603382786e-17,
                                         5.84187315870175e-23,
                                         1.99576047865234e-34,
                                         2.86588142639457e-10,
                                         4.5076439387238e-06,
                                         0.000568510860902393,
                                         0.0102995998895711,
                                         0.0591061808883816,
                                         0.148206312198914,
                                         0.189069149628043,
                                         0.131874466674818,
                                         0.0515399746899193,
                                         0.011164466122543,
                                         0.00128169835537326,
                                         7.15891148597871e-05,
                                         1.68433659623029e-06,
                                         1.3112294047861e-08,
                                         2.20553936829337e-11,
                                         3.46858959740413e-15,
                                         6.8970556509202e-21,
                                         1.25659479711352e-31,
                                         2.60095794509343e-12,
                                         2.18172469673257e-07,
                                         6.91300531310118e-05,
                                         0.00240191371436339,
                                         0.0230832986825555,
                                         0.089635065543571,
                                         0.168817247342846,
                                         0.168871641689305,
                                         0.0931806242169379,
                                         0.0283558704680664,
                                         0.00459596016681049,
                                         0.000368160581386648,
                                         1.27880428679556e-05,
                                         1.54170096925522e-07,
                                         4.34274375411432e-10,
                                         1.30981883313631e-13,
                                         6.54334507339882e-19,
                                         6.35779591521531e-29,
                                         1.92776136468953e-14,
                                         8.62372942254255e-09,
                                         6.86498899004421e-06,
                                         0.000457445415521007,
                                         0.00736220105153943,
                                         0.0442724962175956,
                                         0.123099924883536,
                                         0.176602791554902,
                                         0.137578908508423,
                                         0.0588156413357036,
                                         0.013458960377994,
                                         0.0015462240043217,
                                         7.92910395421663e-05,
                                         1.48035692847254e-06,
                                         6.98326341140546e-09,
                                         4.03937686320707e-12,
                                         5.06968233719932e-17,
                                         2.62701612408736e-26,
                                         1.17933179735048e-16,
                                         2.81353997302515e-10,
                                         5.62698287754225e-07,
                                         7.19091162326742e-05,
                                         0.00193811824357654,
                                         0.0180489886906119,
                                         0.0740903344862943,
                                         0.152440792232668,
                                         0.167664414997466,
                                         0.100694438475058,
                                         0.0325319103922934,
                                         0.00536006837067758,
                                         0.000405795226630725,
                                         1.17326341888216e-05,
                                         9.26862770378865e-08,
                                         1.02820641888271e-10,
                                         3.24208551071271e-15,
                                         8.95945697016911e-24,
                                         5.99375721822597e-19,
                                         7.6259062330319e-12,
                                         3.83170217395023e-08,
                                         9.39093914506941e-06,
                                         0.00042387057513529,
                                         0.0061129682383556,
                                         0.0370463756487209,
                                         0.109316377889945,
                                         0.169750217530334,
                                         0.143218306798688,
                                         0.0653262943378158,
                                         0.0154364933723802,
                                         0.00172532199835391,
                                         7.72511623327529e-05,
                                         1.02200461089714e-06,
                                         2.1743359732578e-09,
                                         1.7224577825015e-13,
                                         2.53852263853611e-21,
                                         2.53852263853611e-21,
                                         1.7224577825015e-13,
                                         2.1743359732578e-09,
                                         1.02200461089713e-06,
                                         7.72511623327529e-05,
                                         0.00172532199835391,
                                         0.0154364933723802,
                                         0.0653262943378158,
                                         0.143218306798688,
                                         0.169750217530334,
                                         0.109316377889945,
                                         0.0370463756487209,
                                         0.0061129682383556,
                                         0.00042387057513529,
                                         9.39093914506946e-06,
                                         3.83170217395023e-08,
                                         7.6259062330319e-12,
                                         5.99375721822597e-19,
                                         8.95945697016911e-24,
                                         3.24208551071271e-15,
                                         1.02820641888271e-10,
                                         9.26862770378859e-08,
                                         1.17326341888216e-05,
                                         0.000405795226630725,
                                         0.00536006837067758,
                                         0.0325319103922934,
                                         0.100694438475058,
                                         0.167664414997466,
                                         0.152440792232668,
                                         0.0740903344862943,
                                         0.0180489886906119,
                                         0.00193811824357654,
                                         7.19091162326746e-05,
                                         5.62698287754225e-07,
                                         2.81353997302515e-10,
                                         1.17933179735048e-16,
                                         2.62701612408736e-26,
                                         5.06968233719932e-17,
                                         4.03937686320707e-12,
                                         6.98326341140541e-09,
                                         1.48035692847254e-06,
                                         7.92910395421663e-05,
                                         0.0015462240043217,
                                         0.013458960377994,
                                         0.0588156413357036,
                                         0.137578908508423,
                                         0.176602791554902,
                                         0.123099924883536,
                                         0.0442724962175956,
                                         0.00736220105153943,
                                         0.000457445415521009,
                                         6.86498899004421e-06,
                                         8.62372942254255e-09,
                                         1.92776136468953e-14,
                                         6.35779591521531e-29,
                                         6.54334507339882e-19,
                                         1.30981883313631e-13,
                                         4.34274375411428e-10,
                                         1.54170096925522e-07,
                                         1.27880428679556e-05,
                                         0.000368160581386648,
                                         0.00459596016681049,
                                         0.0283558704680664,
                                         0.0931806242169379,
                                         0.168871641689305,
                                         0.168817247342846,
                                         0.089635065543571,
                                         0.0230832986825555,
                                         0.0024019137143634,
                                         6.91300531310118e-05,
                                         2.18172469673257e-07,
                                         2.60095794509343e-12,
                                         1.25659479711352e-31,
                                         6.8970556509202e-21,
                                         3.46858959740413e-15,
                                         2.20553936829336e-11,
                                         1.3112294047861e-08,
                                         1.68433659623029e-06,
                                         7.15891148597871e-05,
                                         0.00128169835537326,
                                         0.011164466122543,
                                         0.0515399746899193,
                                         0.131874466674818,
                                         0.189069149628043,
                                         0.148206312198914,
                                         0.0591061808883817,
                                         0.0102995998895712,
                                         0.000568510860902393,
                                         4.5076439387238e-06,
                                         2.86588142639457e-10,
                                         1.99576047865234e-34,
                                         5.84187315870175e-23,
                                         7.38106603382786e-17,
                                         9.0009823661823e-13,
                                         8.96152046874326e-10,
                                         1.782699624469e-07,
                                         1.11861640752618e-05,
                                         0.000287223443658079,
                                         0.00353229871582388,
                                         0.0229080052090149,
                                         0.0827540378984555,
                                         0.170156684189943,
                                         0.19691552071008,
                                         0.12161647539017,
                                         0.0354901465525025,
                                         0.00375694751482596,
                                         7.48382711273485e-05,
                                         2.53750900601647e-08,
                                         2.48605878911956e-37,
                                         3.88088104629101e-25,
                                         1.23189893388186e-18,
                                         2.88107671684031e-14,
                                         4.80368518328842e-11,
                                         1.47984856736259e-08,
                                         1.37089815754708e-06,
                                         5.04828375574855e-05,
                                         0.000876529832631491,
                                         0.00798583183610465,
                                         0.0407293507088736,
                                         0.120106682749904,
                                         0.20520267161186,
                                         0.196264495510161,
                                         0.0959146721725021,
                                         0.0194724817369886,
                                         0.00097451315012706,
                                         1.76216585586861e-06,
                                         2.3398109759643e-40,
                                         1.94793716077141e-27,
                                         1.55345068632214e-20,
                                         6.96764696616011e-16,
                                         1.94551138776899e-12,
                                         9.28159860934165e-10,
                                         1.26939155181205e-07,
                                         6.70400044133789e-06,
                                         0.000164339659784841,
                                         0.00210338828827223,
                                         0.0151457987882563,
                                         0.0640548080122336,
                                         0.16156693136261,
                                         0.239308142078151,
                                         0.195852312621081,
                                         0.0762559763036614,
                                         0.00958778010406964,
                                         9.24596776442346e-05,
                                         1.56469725594855e-43,
                                         6.9470387550267e-30,
                                         1.39187434747421e-22,
                                         1.19728521792444e-17,
                                         5.59851936142454e-14,
                                         4.13626597273469e-11,
                                         8.35153262564354e-09,
                                         6.32564008766022e-07,
                                         2.18926448984728e-05,
                                         0.000393639715920772,
                                         0.00400181535805044,
                                         0.0242726091635395,
                                         0.0903861961691344,
                                         0.207325812162571,
                                         0.284153185390668,
                                         0.212181069317924,
                                         0.0670237300009813,
                                         0.0034469750148263,
                                         6.62692742142191e-47,
                                         1.56912248895799e-32,
                                         7.89832384297209e-25,
                                         1.30299110065067e-19,
                                         1.02034005987423e-15,
                                         1.16741824847459e-12,
                                         3.47991877120144e-10,
                                         3.78013463506447e-08,
                                         1.84708307259985e-06,
                                         4.66563450579362e-05,
                                         0.000669659842733789,
                                         0.0058252361812689,
                                         0.0320246276865914,
                                         0.11375791455824,
                                         0.261101099097681,
                                         0.373914001596744,
                                         0.296736840558869,
                                         0.0813872252408745,
                                         1.33651800180016e-50,
                                         1.68769777288692e-35,
                                         2.13427583636433e-27,
                                         6.75252162522298e-22,
                                         8.85518006402429e-18,
                                         1.56900816415575e-14,
                                         6.90482596313644e-12,
                                         1.07569899655494e-09,
                                         7.42087863604363e-08,
                                         2.63331749231095e-06,
                                         5.3362008994018e-05,
                                         0.000665719562227025,
                                         0.00540314688753123,
                                         0.0297228584774861,
                                         0.114247202959321,
                                         0.313774414330193,
                                         0.625597563185532,
                                         0.915071552236501,

                                         0.923227246222946,
                                         0.655716485209365,
                                         0.352640174053854,
                                         0.142384427157215,
                                         0.042617613650411,
                                         0.00928961031952631,
                                         0.00143975143215897,
                                         0.000153692357163473,
                                         1.08344578085361e-05,
                                         4.76837158203125e-07,
                                         1.21434005730567e-08,
                                         1.61073330729733e-10,
                                         9.57445765150028e-13,
                                         2.03792515060297e-15,
                                         1.08934116686832e-18,
                                         7.95242507109613e-23,
                                         2.43167913228536e-28,
                                         1.87951739151408e-36,
                                         1.46973504837501e-51,
                                         0.0738877112733712,
                                         0.279529284922159,
                                         0.376833814691875,
                                         0.290827810934492,
                                         0.145108527949629,
                                         0.0486873738800113,
                                         0.0110534680388262,
                                         0.00167543408136547,
                                         0.000164637231866334,
                                         1.00135803222656e-05,
                                         3.5241796411567e-07,
                                         6.51609120690039e-09,
                                         5.49973458895919e-11,
                                         1.71477824522819e-13,
                                         1.41090800401794e-16,
                                         1.71697801086349e-20,
                                         1.00352172254139e-25,
                                         1.94434678765317e-33,
                                         8.09867626618493e-48,
                                         0.00281589541549944,
                                         0.0567439063416222,
                                         0.191755862995246,
                                         0.282872078551377,
                                         0.235275903665569,
                                         0.121511087773762,
                                         0.0404101354072728,
                                         0.00869727345126036,
                                         0.00119132329354966,
                                         0.000100135803222656,
                                         4.87030836936746e-06,
                                         1.25525330973061e-07,
                                         1.50435379363136e-09,
                                         6.87081739422651e-12,
                                         8.70190190944543e-15,
                                         1.76526774533313e-18,
                                         1.97209573112667e-23,
                                         9.57815388882952e-31,
                                         2.12505283526747e-44,
                                         6.796622885962e-05,
                                         0.00729530565980654,
                                         0.0617987630647597,
                                         0.174251520028347,
                                         0.241598542582428,
                                         0.192064815598572,
                                         0.0935652102431102,
                                         0.0285937588795427,
                                         0.00545963435278094,
                                         0.000634193420410156,
                                         4.26272413754295e-05,
                                         1.53146805396157e-06,
                                         2.6060970577361e-08,
                                         1.74357725093087e-10,
                                         3.39908492945299e-13,
                                         1.14944655992383e-16,
                                         2.45449162424697e-21,
                                         2.98828655037717e-28,
                                         3.5314883025654e-41,
                                         1.16560122150656e-06,
                                         0.000666419847723577,
                                         0.0141511287115916,
                                         0.0762681542602408,
                                         0.176275249326102,
                                         0.215704838091789,
                                         0.153928365371289,
                                         0.0667943216825627,
                                         0.0177777849958362,
                                         0.0028538703918457,
                                         0.000265092941359105,
                                         1.32759215164268e-05,
                                         3.20783006776595e-07,
                                         3.14379449456199e-09,
                                         9.43387279623026e-12,
                                         5.31798553022089e-15,
                                         2.17057743294517e-19,
                                         6.62434339457039e-26,
                                         4.16990325645784e-38,
                                         1.51033475930939e-08,
                                         4.59958613908549e-05,
                                         0.00244832339748154,
                                         0.0252218129559436,
                                         0.097175049021098,
                                         0.18303677780866,
                                         0.19133275297675,
                                         0.117889259638925,
                                         0.0437379218860637,
                                         0.00970315933227539,
                                         0.00124559111856974,
                                         8.69536494193661e-05,
                                         2.98331096939675e-06,
                                         4.28285634335192e-08,
                                         1.9782647811316e-10,
                                         1.85896803037964e-13,
                                         1.45029188085256e-17,
                                         1.10950646708235e-23,
                                         3.72015118504592e-35,
                                         1.53492180727181e-10,
                                         2.48988566770189e-06,
                                         0.00033222805220521,
                                         0.00654182888746164,
                                         0.0420153527758134,
                                         0.121816650200094,
                                         0.186530476009356,
                                         0.163191957547023,
                                         0.0843972809206394,
                                         0.0258750915527344,
                                         0.00459031677707088,
                                         0.000446684380808865,
                                         2.17608327665057e-05,
                                         4.5761755986448e-07,
                                         3.2536335517826e-09,
                                         5.09666989772618e-12,
                                         7.60020598889497e-16,
                                         1.4574935911432e-21,
                                         2.60306501575294e-32,
                                         1.25349839422133e-12,
                                         1.08308998696634e-07,
                                         3.62266607291356e-05,
                                         0.00136347301344811,
                                         0.0145977441335523,
                                         0.0651477514093221,
                                         0.146128445194969,
                                         0.181529725292299,
                                         0.130864922402731,
                                         0.0554466247558594,
                                         0.0135935941089045,
                                         0.00184390357526937,
                                         0.000127548978417672,
                                         3.92912948569705e-06,
                                         4.30008799848818e-08,
                                         1.12285991377311e-10,
                                         3.20051448735363e-14,
                                         1.53853704015154e-19,
                                         1.46363884760705e-29,
                                         8.35999730048171e-15,
                                         3.84764066473006e-09,
                                         3.2260057514093e-06,
                                         0.000232080548698588,
                                         0.00414198297598462,
                                         0.028453587944019,
                                         0.0934898704530696,
                                         0.164907944539782,
                                         0.165715403526495,
                                         0.0970315933227539,
                                         0.032875379045417,
                                         0.00621613554250491,
                                         0.000610552884493463,
                                         2.75508391576781e-05,
                                         4.64120635448715e-07,
                                         2.02027029909377e-09,
                                         1.10067489604371e-12,
                                         1.32633762138901e-17,
                                         6.72090365906833e-27,
                                         4.60204951032494e-17,
                                         1.12820303941955e-10,
                                         2.37118132008348e-07,
                                         3.26057153459827e-05,
                                         0.00097004898627369,
                                         0.0102574034899669,
                                         0.0493693201509305,
                                         0.123651161681337,
                                         0.173206944145599,
                                         0.140156745910645,
                                         0.0656251083840926,
                                         0.0172967910522808,
                                         0.00241230592956945,
                                         0.000159454256009773,
                                         4.13473049665706e-06,
                                         3.00024122826682e-08,
                                         3.12436036190814e-11,
                                         9.43763147656971e-16,
                                         2.54732411115213e-24,
                                         2.10463524632612e-19,
                                         2.74827652298328e-12,
                                         1.44792076280925e-08,
                                         3.80565246890375e-06,
                                         0.000188738033583777,
                                         0.00307197901173791,
                                         0.0216585879319296,
                                         0.0770256241772611,
                                         0.150400099534764,
                                         0.168188095092773,
                                         0.108830268765342,
                                         0.0399844388791279,
                                         0.00791811644983541,
                                         0.000766686476568595,
                                         3.06015862988427e-05,
                                         3.70154704484389e-07,
                                         7.36789758764738e-10,
                                         5.57894839764176e-14,
                                         8.02086387478091e-22,
                                         8.02086387478091e-22,
                                         5.57894839764176e-14,
                                         7.36789758764738e-10,
                                         3.70154704484389e-07,
                                         3.06015862988426e-05,
                                         0.000766686476568597,
                                         0.00791811644983541,
                                         0.0399844388791278,
                                         0.108830268765342,
                                         0.168188095092773,
                                         0.150400099534764,
                                         0.0770256241772613,
                                         0.0216585879319296,
                                         0.0030719790117379,
                                         0.000188738033583778,
                                         3.80565246890375e-06,
                                         1.44792076280925e-08,
                                         2.74827652298328e-12,
                                         2.10463524632612e-19,
                                         2.54732411115213e-24,
                                         9.43763147656971e-16,
                                         3.12436036190814e-11,
                                         3.00024122826682e-08,
                                         4.13473049665704e-06,
                                         0.000159454256009774,
                                         0.00241230592956945,
                                         0.0172967910522807,
                                         0.0656251083840926,
                                         0.140156745910645,
                                         0.173206944145599,
                                         0.123651161681337,
                                         0.0493693201509305,
                                         0.0102574034899669,
                                         0.000970048986273694,
                                         3.26057153459827e-05,
                                         2.37118132008348e-07,
                                         1.12820303941955e-10,
                                         4.60204951032494e-17,
                                         6.72090365906833e-27,
                                         1.32633762138901e-17,
                                         1.10067489604371e-12,
                                         2.02027029909377e-09,
                                         4.64120635448713e-07,
                                         2.75508391576782e-05,
                                         0.000610552884493463,
                                         0.0062161355425049,
                                         0.032875379045417,
                                         0.0970315933227539,
                                         0.165715403526495,
                                         0.164907944539782,
                                         0.0934898704530696,
                                         0.0284535879440189,
                                         0.00414198297598464,
                                         0.000232080548698588,
                                         3.2260057514093e-06,
                                         3.84764066473006e-09,
                                         8.35999730048171e-15,
                                         1.46363884760705e-29,
                                         1.53853704015155e-19,
                                         3.20051448735363e-14,
                                         1.12285991377311e-10,
                                         4.30008799848815e-08,
                                         3.92912948569707e-06,
                                         0.000127548978417672,
                                         0.00184390357526936,
                                         0.0135935941089045,
                                         0.0554466247558594,
                                         0.130864922402731,
                                         0.181529725292299,
                                         0.146128445194969,
                                         0.065147751409322,
                                         0.0145977441335524,
                                         0.00136347301344811,
                                         3.62266607291356e-05,
                                         1.08308998696634e-07,
                                         1.25349839422133e-12,
                                         2.60306501575294e-32,
                                         1.4574935911432e-21,
                                         7.60020598889497e-16,
                                         5.09666989772618e-12,
                                         3.25363355178258e-09,
                                         4.57617559864482e-07,
                                         2.17608327665057e-05,
                                         0.000446684380808864,
                                         0.00459031677707088,
                                         0.0258750915527344,
                                         0.0843972809206394,
                                         0.163191957547023,
                                         0.186530476009356,
                                         0.121816650200094,
                                         0.0420153527758135,
                                         0.00654182888746164,
                                         0.00033222805220521,
                                         2.48988566770189e-06,
                                         1.53492180727181e-10,
                                         3.72015118504592e-35,
                                         1.10950646708235e-23,
                                         1.45029188085256e-17,
                                         1.85896803037964e-13,
                                         1.97826478113159e-10,
                                         4.28285634335194e-08,
                                         2.98331096939675e-06,
                                         8.69536494193658e-05,
                                         0.00124559111856974,
                                         0.00970315933227539,
                                         0.0437379218860637,
                                         0.117889259638925,
                                         0.19133275297675,
                                         0.183036777808659,
                                         0.0971750490210982,
                                         0.0252218129559436,
                                         0.00244832339748154,
                                         4.59958613908549e-05,
                                         1.51033475930939e-08,
                                         4.16990325645784e-38,
                                         6.62434339457039e-26,
                                         2.17057743294517e-19,
                                         5.31798553022088e-15,
                                         9.43387279623019e-12,
                                         3.143794494562e-09,
                                         3.20783006776595e-07,
                                         1.32759215164267e-05,
                                         0.000265092941359106,
                                         0.0028538703918457,
                                         0.0177777849958362,
                                         0.0667943216825628,
                                         0.153928365371289,
                                         0.215704838091789,
                                         0.176275249326102,
                                         0.0762681542602408,
                                         0.0141511287115916,
                                         0.000666419847723577,
                                         1.16560122150656e-06,
                                         3.5314883025654e-41,
                                         2.98828655037717e-28,
                                         2.45449162424697e-21,
                                         1.14944655992383e-16,
                                         3.39908492945296e-13,
                                         1.74357725093087e-10,
                                         2.6060970577361e-08,
                                         1.53146805396156e-06,
                                         4.26272413754295e-05,
                                         0.000634193420410156,
                                         0.00545963435278094,
                                         0.0285937588795427,
                                         0.0935652102431102,
                                         0.192064815598572,
                                         0.241598542582429,
                                         0.174251520028347,
                                         0.0617987630647597,
                                         0.00729530565980654,
                                         6.796622885962e-05,
                                         2.12505283526747e-44,
                                         9.57815388882952e-31,
                                         1.97209573112667e-23,
                                         1.76526774533313e-18,
                                         8.70190190944536e-15,
                                         6.87081739422654e-12,
                                         1.50435379363136e-09,
                                         1.25525330973061e-07,
                                         4.87030836936746e-06,
                                         0.000100135803222656,
                                         0.00119132329354966,
                                         0.00869727345126036,
                                         0.0404101354072728,
                                         0.121511087773761,
                                         0.23527590366557,
                                         0.282872078551377,
                                         0.191755862995246,
                                         0.0567439063416222,
                                         0.00281589541549944,
                                         8.09867626618493e-48,
                                         1.94434678765317e-33,
                                         1.00352172254139e-25,
                                         1.71697801086349e-20,
                                         1.41090800401793e-16,
                                         1.7147782452282e-13,
                                         5.49973458895919e-11,
                                         6.51609120690037e-09,
                                         3.5241796411567e-07,
                                         1.00135803222656e-05,
                                         0.000164637231866334,
                                         0.00167543408136547,
                                         0.0110534680388262,
                                         0.0486873738800113,
                                         0.145108527949629,
                                         0.290827810934492,
                                         0.376833814691875,
                                         0.279529284922159,
                                         0.0738877112733712,
                                         1.46973504837501e-51,
                                         1.87951739151408e-36,
                                         2.43167913228536e-28,
                                         7.95242507109613e-23,
                                         1.08934116686831e-18,
                                         2.03792515060298e-15,
                                         9.57445765150028e-13,
                                         1.61073330729733e-10,
                                         1.21434005730567e-08,
                                         4.76837158203125e-07,
                                         1.08344578085361e-05,
                                         0.000153692357163473,
                                         0.00143975143215897,
                                         0.00928961031952631,
                                         0.042617613650411,
                                         0.142384427157215,
                                         0.352640174053854,
                                         0.655716485209365,
                                         0.923227246222946,

                                         0.930276025212819,
                                         0.682670100204745,
                                         0.389703122108939,
                                         0.171856707427441,
                                         0.0579472269223445,
                                         0.0147271471197376,
                                         0.00276771297215102,
                                         0.000375094225204805,
                                         3.54768622481198e-05,
                                         2.24331104472888e-06,
                                         8.9594854506798e-08,
                                         2.09345167332694e-09,
                                         2.57461367960825e-11,
                                         1.43329735649996e-13,
                                         2.88505471326593e-16,
                                         1.47215261571345e-19,
                                         1.03534469905547e-23,
                                         3.07735787926301e-29,
                                         2.33255415477911e-37,
                                         1.80435095732495e-52,
                                         0.0673505403752158,
                                         0.262987999547759,
                                         0.375609210458385,
                                         0.315708781154955,
                                         0.17676285534476,
                                         0.0687982992649706,
                                         0.0188165964199736,
                                         0.00359123677037243,
                                         0.000468575553425777,
                                         4.04118268279701e-05,
                                         2.19332338617141e-06,
                                         6.98983789866465e-08,
                                         1.18589708290134e-09,
                                         9.29725769058215e-12,
                                         2.72353879647217e-14,
                                         2.12830112019987e-17,
                                         2.4854405473966e-21,
                                         1.40803758145846e-26,
                                         2.67020912269615e-34,
                                         1.09908263905111e-48,
                                         0.00232194028126317,
                                         0.0482438197875616,
                                         0.172392864718444,
                                         0.27617702143328,
                                         0.256761594735191,
                                         0.153044414646852,
                                         0.0609174613523168,
                                         0.0163730054175475,
                                         0.00294709921789753,
                                         0.000346663547767332,
                                         2.55683660513349e-05,
                                         1.11135286583933e-06,
                                         2.60113345191247e-08,
                                         2.87179967603044e-10,
                                         1.22431672930904e-12,
                                         1.46519024166358e-15,
                                         2.84120453533549e-19,
                                         3.06782931608632e-24,
                                         1.45559153202269e-31,
                                         3.18801522392467e-45,
                                         5.06982948984091e-05,
                                         0.00560505382783559,
                                         0.0501111865371722,
                                         0.153010335546767,
                                         0.236211733444623,
                                         0.215620283733742,
                                         0.124903574980517,
                                         0.0472764783400745,
                                         0.0117393008801551,
                                         0.00188338981505094,
                                         0.000188771152587884,
                                         1.1191007572226e-05,
                                         3.61335485493105e-07,
                                         5.61805209739255e-09,
                                         3.48567063514867e-11,
                                         6.38832949183855e-14,
                                         2.05699845554897e-17,
                                         4.23331403211816e-22,
                                         5.02534765431654e-29,
                                         5.85656235574338e-42,
                                         7.86530970754948e-07,
                                         0.000462698462498701,
                                         0.0103497593841439,
                                         0.0602329537821199,
                                         0.154402040888345,
                                         0.215844958496951,
                                         0.181965103535108,
                                         0.0969933586854395,
                                         0.0332253739772237,
                                         0.00727030156424745,
                                         0.000990258199898207,
                                         8.00693997886496e-05,
                                         3.56647153031917e-06,
                                         7.80903824393376e-08,
                                         7.05113580408959e-10,
                                         1.97906822226007e-12,
                                         1.05814601207923e-15,
                                         4.15059106257362e-20,
                                         1.2327439236925e-26,
                                         7.6444338023045e-39,
                                         9.21944348651217e-09,
                                         2.8859096365282e-05,
                                         0.00161507324820015,
                                         0.017914882254204,
                                         0.0762554713111102,
                                         0.163252788679273,
                                         0.200293913467969,
                                         0.15035064023355,
                                         0.0710499743358193,
                                         0.0212046464781702,
                                         0.00392489163339482,
                                         0.000432842961336359,
                                         2.65970403539367e-05,
                                         8.20116860630501e-07,
                                         1.07770082162885e-08,
                                         4.63234211817582e-11,
                                         4.11266785513482e-14,
                                         3.07472168136619e-18,
                                         2.28478864976286e-24,
                                         7.53900931668277e-36,
                                         8.47585268869477e-11,
                                         1.41174811187606e-06,
                                         0.000197671477862189,
                                         0.00417910776481536,
                                         0.0295378443741267,
                                         0.0968431981597232,
                                         0.172916808268083,
                                         0.182792503305789,
                                         0.119164745018308,
                                         0.0485064467790028,
                                         0.0122010360019165,
                                         0.00183520237050206,
                                         0.000155567072126773,
                                         6.75528609416813e-06,
                                         1.29189484095168e-07,
                                         8.5041380697872e-10,
                                         1.25369383168464e-12,
                                         1.7864525657771e-16,
                                         3.32130691716345e-22,
                                         5.83140293451631e-33,
                                         6.26161815892909e-13,
                                         5.54952982199346e-08,
                                         1.94410752640286e-05,
                                         0.000783389360902993,
                                         0.00919413916589995,
                                         0.0461638609837919,
                                         0.119958535726345,
                                         0.178581294150155,
                                         0.160603918595188,
                                         0.0891645815269708,
                                         0.0304782644980072,
                                         0.00625262021223156,
                                         0.000731183749583967,
                                         4.47132501609505e-05,
                                         1.24445902226331e-06,
                                         1.25453986300738e-08,
                                         3.07102836865186e-11,
                                         8.34068375719441e-15,
                                         3.8796850747272e-20,
                                         3.62456847547369e-30,
                                         3.77776196882587e-15,
                                         1.78155811262152e-09,
                                         1.56149782490549e-06,
                                         0.00011992689724028,
                                         0.00233715856322668,
                                         0.0179713190962203,
                                         0.0679625915152658,
                                         0.14248146797927,
                                         0.176770307155328,
                                         0.133853619196911,
                                         0.0621768317140771,
                                         0.0173974248216728,
                                         0.00280659801042599,
                                         0.000241698281149051,
                                         9.7899137815575e-06,
                                         1.5114140608762e-07,
                                         6.14357263223214e-10,
                                         3.1802162347143e-13,
                                         3.70108251595473e-18,
                                         1.83985838475272e-27,
                                         1.88124515432319e-17,
                                         4.72070205469602e-11,
                                         1.03520242294772e-07,
                                         1.51536872729165e-05,
                                         0.000490374724105458,
                                         0.00577458237124894,
                                         0.0317812884406175,
                                         0.0938304048678165,
                                         0.160592511401624,
                                         0.165855796848717,
                                         0.10469591509504,
                                         0.0399549582286043,
                                         0.00889194339704081,
                                         0.00107838460326651,
                                         6.35682019772434e-05,
                                         1.50295246482335e-06,
                                         1.01442737418156e-08,
                                         1.00086255132463e-11,
                                         2.91423020886084e-16,
                                         7.70859586962999e-25,
                                         7.782813106031e-20,
                                         1.03918672907887e-12,
                                         5.70150591475563e-09,
                                         1.59074455875006e-06,
                                         8.54768243703044e-05,
                                         0.00154149318505809,
                                         0.0123467732448344,
                                         0.0513344842057444,
                                         0.121205315460887,
                                         0.170730689898611,
                                         0.146457389627465,
                                         0.0762319091954902,
                                         0.0234041875195128,
                                         0.00399718453423192,
                                         0.000342910979936747,
                                         1.24161638458562e-05,
                                         1.3915580443775e-07,
                                         2.6168126619003e-10,
                                         1.90633555027305e-14,
                                         2.6831599496846e-22,
                                         2.68315994968413e-22,
                                         1.90633555027298e-14,
                                         2.6168126619003e-10,
                                         1.39155804437749e-07,
                                         1.24161638458562e-05,
                                         0.000342910979936748,
                                         0.00399718453423192,
                                         0.0234041875195128,
                                         0.0762319091954902,
                                         0.146457389627465,
                                         0.170730689898611,
                                         0.121205315460887,
                                         0.0513344842057443,
                                         0.0123467732448344,
                                         0.00154149318505808,
                                         8.54768243703044e-05,
                                         1.59074455875007e-06,
                                         5.70150591475563e-09,
                                         1.03918672907891e-12,
                                         7.78281310603226e-20,
                                         7.7085958696285e-25,
                                         2.91423020886073e-16,
                                         1.00086255132463e-11,
                                         1.01442737418155e-08,
                                         1.50295246482335e-06,
                                         6.35682019772437e-05,
                                         0.00107838460326651,
                                         0.00889194339704081,
                                         0.0399549582286043,
                                         0.10469591509504,
                                         0.165855796848717,
                                         0.160592511401624,
                                         0.0938304048678165,
                                         0.0317812884406175,
                                         0.00577458237124892,
                                         0.000490374724105458,
                                         1.51536872729165e-05,
                                         1.03520242294772e-07,
                                         4.72070205469615e-11,
                                         1.88124515432347e-17,
                                         1.83985838475233e-27,
                                         3.70108251595458e-18,
                                         3.1802162347143e-13,
                                         6.14357263223208e-10,
                                         1.5114140608762e-07,
                                         9.78991378155753e-06,
                                         0.000241698281149051,
                                         0.00280659801042599,
                                         0.0173974248216728,
                                         0.0621768317140771,
                                         0.133853619196911,
                                         0.176770307155328,
                                         0.14248146797927,
                                         0.0679625915152658,
                                         0.0179713190962202,
                                         0.00233715856322668,
                                         0.00011992689724028,
                                         1.56149782490549e-06,
                                         1.78155811262156e-09,
                                         3.77776196882636e-15,
                                         3.62456847547287e-30,
                                         3.87968507472703e-20,
                                         8.34068375719441e-15,
                                         3.07102836865183e-11,
                                         1.25453986300738e-08,
                                         1.24445902226331e-06,
                                         4.47132501609505e-05,
                                         0.000731183749583967,
                                         0.00625262021223156,
                                         0.0304782644980072,
                                         0.0891645815269708,
                                         0.160603918595188,
                                         0.178581294150155,
                                         0.119958535726345,
                                         0.0461638609837918,
                                         0.00919413916589995,
                                         0.000783389360902997,
                                         1.94410752640286e-05,
                                         5.54952982199358e-08,
                                         6.2616181589298e-13,
                                         5.8314029345149e-33,
                                         3.32130691716329e-22,
                                         1.7864525657771e-16,
                                         1.25369383168463e-12,
                                         8.5041380697872e-10,
                                         1.29189484095168e-07,
                                         6.75528609416813e-06,
                                         0.000155567072126773,
                                         0.00183520237050206,
                                         0.0122010360019165,
                                         0.0485064467790028,
                                         0.119164745018308,
                                         0.182792503305788,
                                         0.172916808268083,
                                         0.096843198159723,
                                         0.0295378443741267,
                                         0.00417910776481538,
                                         0.000197671477862189,
                                         1.41174811187608e-06,
                                         8.47585268869559e-11,
                                         7.53900931668082e-36,
                                         2.28478864976274e-24,
                                         3.07472168136619e-18,
                                         4.11266785513477e-14,
                                         4.63234211817582e-11,
                                         1.07770082162886e-08,
                                         8.20116860630501e-07,
                                         2.65970403539367e-05,
                                         0.000432842961336359,
                                         0.00392489163339482,
                                         0.0212046464781702,
                                         0.0710499743358193,
                                         0.15035064023355,
                                         0.200293913467969,
                                         0.163252788679273,
                                         0.0762554713111102,
                                         0.0179148822542041,
                                         0.00161507324820015,
                                         2.88590963652824e-05,
                                         9.21944348651291e-09,
                                         7.6444338023024e-39,
                                         1.23274392369244e-26,
                                         4.15059106257362e-20,
                                         1.05814601207922e-15,
                                         1.97906822226007e-12,
                                         7.05113580408962e-10,
                                         7.80903824393376e-08,
                                         3.56647153031917e-06,
                                         8.00693997886496e-05,
                                         0.000990258199898207,
                                         0.00727030156424745,
                                         0.0332253739772237,
                                         0.0969933586854395,
                                         0.181965103535108,
                                         0.215844958496951,
                                         0.154402040888345,
                                         0.0602329537821201,
                                         0.0103497593841439,
                                         0.000462698462498706,
                                         7.86530970754999e-07,
                                         5.85656235574168e-42,
                                         5.02534765431626e-29,
                                         4.23331403211816e-22,
                                         2.05699845554895e-17,
                                         6.38832949183855e-14,
                                         3.48567063514869e-11,
                                         5.61805209739255e-09,
                                         3.61335485493105e-07,
                                         1.1191007572226e-05,
                                         0.000188771152587884,
                                         0.00188338981505094,
                                         0.0117393008801551,
                                         0.0472764783400745,
                                         0.124903574980517,
                                         0.215620283733742,
                                         0.236211733444623,
                                         0.153010335546767,
                                         0.0501111865371722,
                                         0.00560505382783564,
                                         5.06982948984116e-05,
                                         3.1880152239237e-45,
                                         1.45559153202261e-31,
                                         3.06782931608632e-24,
                                         2.84120453533545e-19,
                                         1.46519024166358e-15,
                                         1.22431672930904e-12,
                                         2.87179967603044e-10,
                                         2.60113345191247e-08,
                                         1.11135286583933e-06,
                                         2.55683660513349e-05,
                                         0.000346663547767332,
                                         0.00294709921789753,
                                         0.0163730054175475,
                                         0.0609174613523168,
                                         0.153044414646852,
                                         0.256761594735191,
                                         0.276177021433281,
                                         0.172392864718444,
                                         0.0482438197875619,
                                         0.00232194028126324,
                                         1.09908263905075e-48,
                                         2.67020912269598e-34,
                                         1.40803758145846e-26,
                                         2.48544054739656e-21,
                                         2.12830112019987e-17,
                                         2.72353879647218e-14,
                                         9.29725769058215e-12,
                                         1.18589708290134e-09,
                                         6.98983789866465e-08,
                                         2.19332338617141e-06,
                                         4.04118268279701e-05,
                                         0.000468575553425777,
                                         0.00359123677037243,
                                         0.0188165964199736,
                                         0.0687982992649706,
                                         0.17676285534476,
                                         0.315708781154955,
                                         0.375609210458385,
                                         0.26298799954776,
                                         0.0673505403752169,
                                         1.80435095732433e-52,
                                         2.33255415477896e-37,
                                         3.07735787926301e-29,
                                         1.03534469905545e-23,
                                         1.47215261571345e-19,
                                         2.88505471326595e-16,
                                         1.43329735649996e-13,
                                         2.57461367960825e-11,
                                         2.09345167332694e-09,
                                         8.9594854506798e-08,
                                         2.24331104472888e-06,
                                         3.54768622481198e-05,
                                         0.000375094225204805,
                                         0.00276771297215102,
                                         0.0147271471197376,
                                         0.0579472269223445,
                                         0.171856707427441,
                                         0.389703122108939,
                                         0.682670100204745,
                                         0.930276025212819,

                                         0.936407166414884,
                                         0.706827352802557,
                                         0.424783587444784,
                                         0.20208875419322,
                                         0.0754686921054075,
                                         0.0218663882939486,
                                         0.00484011851339829,
                                         0.000802178013382569,
                                         9.69960737825523e-05,
                                         8.27483817088036e-06,
                                         4.76837158203125e-07,
                                         1.75252127255276e-08,
                                         3.80344290391248e-10,
                                         4.38352634412055e-12,
                                         2.30655946132697e-14,
                                         4.42483052106175e-17,
                                         2.16923143042154e-20,
                                         1.47727831043693e-24,
                                         4.28477932454267e-30,
                                         3.19341361769032e-38,
                                         2.4472732993476e-53,
                                         0.0616228827876229,
                                         0.247284296953093,
                                         0.371204920692599,
                                         0.335771131700992,
                                         0.207519736427195,
                                         0.0916827770301585,
                                         0.0293722764824832,
                                         0.00680832353745747,
                                         0.00112594555362495,
                                         0.000129610709820218,
                                         1.00135803222656e-05,
                                         4.93424270587516e-07,
                                         1.44494812436021e-08,
                                         2.27768037143198e-10,
                                         1.67618377504281e-12,
                                         4.65398015419965e-15,
                                         3.47897969755341e-18,
                                         3.92102582916075e-22,
                                         2.16232541558499e-27,
                                         4.02541417157886e-35,
                                         1.63999973879839e-49,
                                         0.00193107881315576,
                                         0.0411965108709455,
                                         0.154468676710988,
                                         0.265659448997229,
                                         0.271726938652502,
                                         0.183053976789007,
                                         0.0848789355816735,
                                         0.0275163181437319,
                                         0.0062238817181398,
                                         0.000966725054412675,
                                         0.000100135803222656,
                                         6.61543524308734e-06,
                                         2.61401644427192e-07,
                                         5.63563221017549e-09,
                                         5.80041831799785e-11,
                                         2.33095086086047e-13,
                                         2.65692078029525e-16,
                                         4.95584585850684e-20,
                                         5.19630161592962e-25,
                                         2.4162767484902e-32,
                                         5.23342242622157e-46,
                                         3.83257209360293e-05,
                                         0.00434667008084427,
                                         0.0407098473412982,
                                         0.133118840721029,
                                         0.225340021267797,
                                         0.23147437667517,
                                         0.15534403729961,
                                         0.0704324514956225,
                                         0.0217890159613545,
                                         0.00456664661698214,
                                         0.000634193420410156,
                                         5.6173137751823e-05,
                                         2.99499927272572e-06,
                                         8.83130435870589e-08,
                                         1.2712452539106e-09,
                                         7.39390822326721e-12,
                                         1.28510218783257e-14,
                                         3.96705501420775e-18,
                                         7.90860702545626e-23,
                                         9.18576064079752e-30,
                                         1.05769426507263e-42,
                                         5.40456612182515e-07,
                                         0.000325861526792972,
                                         0.00762322413798468,
                                         0.0473951556349573,
                                         0.132777391852614,
                                         0.207972947007744,
                                         0.202008395374677,
                                         0.128095966584179,
                                         0.0541993491138971,
                                         0.0153275239942293,
                                         0.0028538703918457,
                                         0.000338905886490103,
                                         2.43817736282353e-05,
                                         9.83302695440234e-07,
                                         1.97960955430924e-08,
                                         1.66646102372569e-10,
                                         4.4164863035142e-13,
                                         2.25631028936048e-16,
                                         8.55235674818897e-21,
                                         2.48121125140832e-27,
                                         1.51884926575521e-39,
                                         5.75834577094103e-09,
                                         1.84576285383323e-05,
                                         0.00107855999459099,
                                         0.0127495478687943,
                                         0.0591120933494004,
                                         0.141181298430818,
                                         0.198477213027209,
                                         0.176021010021454,
                                         0.1018631092593,
                                         0.038869862728792,
                                         0.00970315933227539,
                                         0.00154488424452313,
                                         0.00014996857730931,
                                         8.27210446283593e-06,
                                         2.32914293490397e-07,
                                         2.83780577894198e-09,
                                         1.14678635317775e-11,
                                         9.69607191250036e-15,
                                         6.98776070239332e-19,
                                         5.06382586181525e-25,
                                         1.64791802783721e-36,
                                         4.81198715544793e-11,
                                         8.19989951215831e-07,
                                         0.000119685004754807,
                                         0.00268995756693292,
                                         0.0206404119302439,
                                         0.0751687473209809,
                                         0.152947260445982,
                                         0.189707009274126,
                                         0.150151474359955,
                                         0.0773114510133235,
                                         0.0258750915527344,
                                         0.00552335034271944,
                                         0.000723477557155311,
                                         5.45801350548415e-05,
                                         2.14932730662857e-06,
                                         3.79018136689479e-08,
                                         2.33548971802257e-10,
                                         3.268004508683e-13,
                                         4.47795840137257e-17,
                                         8.10557070106964e-23,
                                         1.402317485697e-33,
                                         3.23128798845538e-13,
                                         2.92728930300271e-08,
                                         1.06723402296783e-05,
                                         0.000456058507197927,
                                         0.00579141742256759,
                                         0.0321604369507403,
                                         0.0947103057333433,
                                         0.164295902735723,
                                         0.177855279035471,
                                         0.123566039948034,
                                         0.0554466247558594,
                                         0.0158684213979805,
                                         0.00280462203785695,
                                         0.000289386114370205,
                                         1.59379866333012e-05,
                                         4.06782070427325e-07,
                                         3.82206342710841e-09,
                                         8.8510328577193e-12,
                                         2.30593240460421e-15,
                                         1.04258847750652e-20,
                                         9.58918242229172e-31,
                                         1.77203258108322e-15,
                                         8.53429382249657e-10,
                                         7.77185003857429e-07,
                                         6.31452261718063e-05,
                                         0.00132707725497668,
                                         0.0112370263228884,
                                         0.0478958190458837,
                                         0.116202351913833,
                                         0.172047651698124,
                                         0.161286962320882,
                                         0.0970315933227539,
                                         0.0372314275452832,
                                         0.0088790898481566,
                                         0.00125304192089939,
                                         9.65181946296592e-05,
                                         3.56540149549568e-06,
                                         5.10813833231233e-08,
                                         1.95771779566276e-10,
                                         9.69745827308715e-14,
                                         1.09518395644882e-18,
                                         5.35502671212382e-28,
                                         8.02103725178476e-18,
                                         2.05367790270483e-11,
                                         4.67145270673723e-08,
                                         7.21644455277673e-06,
                                         0.000250998044210764,
                                         0.00324073597270692,
                                         0.0199922081263979,
                                         0.0678368850314084,
                                         0.137370516894498,
                                         0.173764954330104,
                                         0.140156745910645,
                                         0.0721021883644823,
                                         0.0232019963027962,
                                         0.0044783322094318,
                                         0.000482444895383699,
                                         2.57939513487927e-05,
                                         5.63495177791929e-07,
                                         3.5741190835223e-09,
                                         3.36613911892968e-12,
                                         9.49563515272468e-17,
                                         2.46833724643094e-25,
                                         3.01626689225238e-20,
                                         4.10560792189316e-13,
                                         2.33270544985656e-09,
                                         6.85151224489519e-07,
                                         3.94389071625007e-05,
                                         0.000776455024697238,
                                         0.00693273065180875,
                                         0.0329001047050901,
                                         0.0911210506716075,
                                         0.155526902209973,
                                         0.168188095092773,
                                         0.116002568931319,
                                         0.0503689223325996,
                                         0.0132968085762479,
                                         0.00200339527224217,
                                         0.000155027132328117,
                                         5.16414182649386e-06,
                                         5.42086199774168e-08,
                                         9.70703522258111e-11,
                                         6.83976797031317e-15,
                                         9.45208796953671e-23,
                                         9.45208796953856e-23,
                                         6.83976797031291e-15,
                                         9.70703522258126e-11,
                                         5.42086199774168e-08,
                                         5.16414182649386e-06,
                                         0.000155027132328117,
                                         0.00200339527224217,
                                         0.0132968085762479,
                                         0.0503689223325996,
                                         0.116002568931319,
                                         0.168188095092773,
                                         0.155526902209973,
                                         0.0911210506716075,
                                         0.0329001047050901,
                                         0.00693273065180875,
                                         0.000776455024697238,
                                         3.94389071625007e-05,
                                         6.85151224489519e-07,
                                         2.33270544985653e-09,
                                         4.1056079218933e-13,
                                         3.01626689225185e-20,
                                         2.46833724643146e-25,
                                         9.4956351527243e-17,
                                         3.36613911892974e-12,
                                         3.5741190835223e-09,
                                         5.63495177791929e-07,
                                         2.57939513487927e-05,
                                         0.000482444895383699,
                                         0.00447833220943179,
                                         0.0232019963027962,
                                         0.0721021883644823,
                                         0.140156745910645,
                                         0.173764954330104,
                                         0.137370516894498,
                                         0.0678368850314085,
                                         0.0199922081263979,
                                         0.00324073597270692,
                                         0.000250998044210764,
                                         7.21644455277673e-06,
                                         4.67145270673718e-08,
                                         2.05367790270489e-11,
                                         8.02103725178347e-18,
                                         5.35502671212505e-28,
                                         1.09518395644877e-18,
                                         9.69745827308733e-14,
                                         1.95771779566276e-10,
                                         5.10813833231233e-08,
                                         3.56540149549568e-06,
                                         9.65181946296592e-05,
                                         0.00125304192089938,
                                         0.0088790898481566,
                                         0.0372314275452832,
                                         0.0970315933227539,
                                         0.161286962320882,
                                         0.172047651698124,
                                         0.116202351913833,
                                         0.0478958190458837,
                                         0.0112370263228884,
                                         0.00132707725497668,
                                         6.31452261718063e-05,
                                         7.77185003857421e-07,
                                         8.5342938224968e-10,
                                         1.77203258108297e-15,
                                         9.5891824222941e-31,
                                         1.04258847750647e-20,
                                         2.30593240460425e-15,
                                         8.8510328577193e-12,
                                         3.82206342710841e-09,
                                         4.06782070427325e-07,
                                         1.59379866333012e-05,
                                         0.000289386114370204,
                                         0.00280462203785695,
                                         0.0158684213979805,
                                         0.0554466247558594,
                                         0.123566039948034,
                                         0.177855279035471,
                                         0.164295902735724,
                                         0.0947103057333433,
                                         0.0321604369507403,
                                         0.00579141742256759,
                                         0.000456058507197927,
                                         1.06723402296782e-05,
                                         2.92728930300278e-08,
                                         3.23128798845497e-13,
                                         1.40231748569737e-33,
                                         8.10557070106922e-23,
                                         4.47795840137266e-17,
                                         3.268004508683e-13,
                                         2.33548971802257e-10,
                                         3.79018136689479e-08,
                                         2.14932730662857e-06,
                                         5.45801350548413e-05,
                                         0.000723477557155311,
                                         0.00552335034271944,
                                         0.0258750915527344,
                                         0.0773114510133235,
                                         0.150151474359955,
                                         0.189707009274126,
                                         0.152947260445982,
                                         0.0751687473209809,
                                         0.0206404119302439,
                                         0.00268995756693292,
                                         0.000119685004754806,
                                         8.19989951215848e-07,
                                         4.81198715544742e-11,
                                         1.64791802783768e-36,
                                         5.06382586181498e-25,
                                         6.98776070239348e-19,
                                         9.69607191250036e-15,
                                         1.14678635317775e-11,
                                         2.83780577894198e-09,
                                         2.32914293490397e-07,
                                         8.2721044628359e-06,
                                         0.00014996857730931,
                                         0.00154488424452313,
                                         0.00970315933227539,
                                         0.038869862728792,
                                         0.1018631092593,
                                         0.176021010021454,
                                         0.198477213027209,
                                         0.141181298430818,
                                         0.0591120933494004,
                                         0.0127495478687943,
                                         0.00107855999459098,
                                         1.84576285383326e-05,
                                         5.75834577094052e-09,
                                         1.51884926575567e-39,
                                         2.48121125140818e-27,
                                         8.55235674818917e-21,
                                         2.25631028936048e-16,
                                         4.4164863035142e-13,
                                         1.66646102372569e-10,
                                         1.97960955430924e-08,
                                         9.83302695440231e-07,
                                         2.43817736282353e-05,
                                         0.000338905886490103,
                                         0.0028538703918457,
                                         0.0153275239942293,
                                         0.0541993491138971,
                                         0.128095966584179,
                                         0.202008395374677,
                                         0.207972947007744,
                                         0.132777391852614,
                                         0.0473951556349573,
                                         0.00762322413798464,
                                         0.000325861526792976,
                                         5.40456612182477e-07,
                                         1.05769426507297e-42,
                                         9.18576064079696e-30,
                                         7.90860702545646e-23,
                                         3.96705501420775e-18,
                                         1.28510218783257e-14,
                                         7.39390822326721e-12,
                                         1.2712452539106e-09,
                                         8.83130435870586e-08,
                                         2.99499927272572e-06,
                                         5.6173137751823e-05,
                                         0.000634193420410156,
                                         0.00456664661698214,
                                         0.0217890159613545,
                                         0.0704324514956226,
                                         0.15534403729961,
                                         0.23147437667517,
                                         0.225340021267797,
                                         0.133118840721029,
                                         0.0407098473412981,
                                         0.00434667008084432,
                                         3.83257209360273e-05,
                                         5.23342242622334e-46,
                                         2.41627674849004e-32,
                                         5.19630161592976e-25,
                                         4.95584585850684e-20,
                                         2.65692078029525e-16,
                                         2.33095086086047e-13,
                                         5.80041831799785e-11,
                                         5.63563221017547e-09,
                                         2.61401644427192e-07,
                                         6.61543524308734e-06,
                                         0.000100135803222656,
                                         0.000966725054412675,
                                         0.0062238817181398,
                                         0.0275163181437319,
                                         0.0848789355816735,
                                         0.183053976789007,
                                         0.271726938652502,
                                         0.265659448997229,
                                         0.154468676710988,
                                         0.0411965108709457,
                                         0.0019310788131557,
                                         1.63999973879897e-49,
                                         4.02541417157858e-35,
                                         2.16232541558505e-27,
                                         3.92102582916075e-22,
                                         3.47897969755341e-18,
                                         4.65398015419965e-15,
                                         1.67618377504281e-12,
                                         2.27768037143197e-10,
                                         1.44494812436021e-08,
                                         4.93424270587516e-07,
                                         1.00135803222656e-05,
                                         0.000129610709820218,
                                         0.00112594555362495,
                                         0.00680832353745747,
                                         0.0293722764824832,
                                         0.0916827770301585,
                                         0.207519736427195,
                                         0.335771131700992,
                                         0.371204920692598,
                                         0.247284296953094,
                                         0.0616228827876218,
                                         2.44727329934851e-53,
                                         3.19341361769009e-38,
                                         4.28477932454279e-30,
                                         1.47727831043693e-24,
                                         2.16923143042154e-20,
                                         4.42483052106175e-17,
                                         2.30655946132697e-14,
                                         4.38352634412053e-12,
                                         3.80344290391248e-10,
                                         1.75252127255276e-08,
                                         4.76837158203125e-07,
                                         8.27483817088036e-06,
                                         9.69960737825523e-05,
                                         0.000802178013382569,
                                         0.00484011851339829,
                                         0.0218663882939486,
                                         0.0754686921054075,
                                         0.20208875419322,
                                         0.424783587444784,
                                         0.706827352802557,
                                         0.936407166414884,

                                         0.941771631082104,
                                         0.728519050682552,
                                         0.457806027342225,
                                         0.232593961359237,
                                         0.0948753553480337,
                                         0.0307733357371311,
                                         0.00783727501092678,
                                         0.00154183084782199,
                                         0.00022947572688146,
                                         2.51606523245963e-05,
                                         1.96429557477353e-06,
                                         1.04488503102539e-07,
                                         3.57442347039459e-09,
                                         7.277854446622e-11,
                                         7.929417664956e-13,
                                         3.97345516235055e-15,
                                         7.31115220506019e-18,
                                         3.46179005496654e-21,
                                         2.29254625098129e-25,
                                         6.50952321273414e-31,
                                         4.78086144619637e-39,
                                         3.6341374664481e-54,
                                         0.0565799802508907,
                                         0.232501211051608,
                                         0.364425522704924,
                                         0.351286551603083,
                                         0.236461629150044,
                                         0.116515816170342,
                                         0.042747266089615,
                                         0.0116931745586714,
                                         0.00236315688364002,
                                         0.000346518279001223,
                                         3.58717760130399e-05,
                                         2.52325449805797e-06,
                                         1.1445646812135e-07,
                                         3.11663651837739e-09,
                                         4.61088383534314e-11,
                                         3.2126517299303e-13,
                                         8.51557754410546e-16,
                                         6.12536614031376e-19,
                                         6.69412175193142e-23,
                                         3.60628869363829e-28,
                                         6.60633269080701e-36,
                                         2.66761255728981e-50,
                                         0.00161867882046585,
                                         0.0353337906079191,
                                         0.138139146969709,
                                         0.25264186448318,
                                         0.280639400908506,
                                         0.210075735909009,
                                         0.111027946611586,
                                         0.0422288097583119,
                                         0.0115885489493976,
                                         0.00227253775028772,
                                         0.000311946146631087,
                                         2.90157840241809e-05,
                                         1.74524090299257e-06,
                                         6.35549892774994e-08,
                                         1.27675560511888e-09,
                                         1.23691451156011e-11,
                                         4.72305601320392e-14,
                                         5.16112153885092e-17,
                                         9.30785937262482e-21,
                                         9.5137668867477e-26,
                                         4.34705794310733e-33,
                                         9.32448512330859e-47,
                                         2.93285723630642e-05,
                                         0.00340085087563921,
                                         0.0331632511991051,
                                         0.11507512566469,
                                         0.210944865048117,
                                         0.239882878592968,
                                         0.182636938040989,
                                         0.0965867593055069,
                                         0.0359913312846276,
                                         0.00943905268008732,
                                         0.00171806154685473,
                                         0.000211319663296377,
                                         1.68539915451324e-05,
                                         8.20815389995646e-07,
                                         2.23904958607156e-08,
                                         3.01611618821839e-10,
                                         1.65906893094118e-12,
                                         2.75415555382677e-15,
                                         8.196687267381e-19,
                                         1.58955978775434e-23,
                                         1.81160188391934e-30,
                                         2.0642358388551e-43,
                                         3.77573346944217e-07,
                                         0.00023257611641371,
                                         0.00565688824103892,
                                         0.0372424090159471,
                                         0.112659920106611,
                                         0.194626860617979,
                                         0.213464221866815,
                                         0.15696633438066,
                                         0.0794231175488226,
                                         0.0278564571598537,
                                         0.0067232301394751,
                                         0.00109351734902099,
                                         0.000115645923360414,
                                         7.53219403387878e-06,
                                         2.78997182548724e-07,
                                         5.22560547460222e-09,
                                         4.14081599575299e-11,
                                         1.0442705058057e-13,
                                         5.12869751328759e-17,
                                         1.88704126349178e-21,
                                         5.36426551237612e-28,
                                         3.24693754458931e-40,
                                         3.67263813516216e-09,
                                         1.20173592154749e-05,
                                         0.000729062063240724,
                                         0.00910668839144799,
                                         0.0454607214015241,
                                         0.119308862842711,
                                         0.188507214027143,
                                         0.192735556073237,
                                         0.132422673759855,
                                         0.0621140368899201,
                                         0.0198785014049678,
                                         0.00427541078599279,
                                         0.000599548454261616,
                                         5.22232518294658e-05,
                                         2.62665072423546e-06,
                                         6.84055866442657e-08,
                                         7.80861188520202e-10,
                                         2.99160301276671e-12,
                                         2.4246117677165e-15,
                                         1.69259217771155e-19,
                                         1.20011861233791e-25,
                                         3.85882389642899e-37,
                                         2.80184924691797e-11,
                                         4.87015503858154e-07,
                                         7.3695541963524e-05,
                                         0.00174651774473647,
                                         0.0143877546248281,
                                         0.0573630785037391,
                                         0.130563165379375,
                                         0.18561239784779,
                                         0.173167979663139,
                                         0.108628415649118,
                                         0.0460976853544595,
                                         0.0131105175031161,
                                         0.0024378560754795,
                                         0.000283985481612367,
                                         1.93952173718563e-05,
                                         7.02322109832598e-07,
                                         1.15491908417362e-08,
                                         6.72178669974307e-11,
                                         8.9901543288005e-14,
                                         1.19072940122528e-17,
                                         2.10585222416227e-23,
                                         3.59687875636389e-34,
                                         1.71765475577869e-13,
                                         1.58599209240689e-08,
                                         5.98607867611316e-06,
                                         0.000269159687462465,
                                         0.00365909941212752,
                                         0.0221623945144955,
                                         0.0726671040609005,
                                         0.143640400734323,
                                         0.18196896061432,
                                         0.152658721843787,
                                         0.0859011710589999,
                                         0.0323062320004733,
                                         0.0079655601976332,
                                         0.00124094590886324,
                                         0.000115083076305263,
                                         5.79436163066653e-06,
                                         1.3726308798748e-07,
                                         1.21364027137027e-09,
                                         2.67865370752415e-12,
                                         6.73129016172568e-16,
                                         2.96931381702677e-21,
                                         2.69414598798739e-31,
                                         8.59947261030262e-16,
                                         4.21797574427873e-10,
                                         3.97089545973548e-07,
                                         3.38759792267417e-05,
                                         0.000759976607412671,
                                         0.00699271361415923,
                                         0.0330293376728753,
                                         0.0907801852278729,
                                         0.156160743639597,
                                         0.175204246447283,
                                         0.130726573903452,
                                         0.0650126112682645,
                                         0.0212554065100847,
                                         0.00442847701574249,
                                         0.000557664634275557,
                                         3.90408874980032e-05,
                                         1.3322962383546e-06,
                                         1.78953443804487e-08,
                                         6.51794703587549e-11,
                                         3.10762335050163e-14,
                                         3.41923646317888e-19,
                                         1.64801545875281e-28,
                                         3.55361646286151e-18,
                                         9.25912468644724e-12,
                                         2.17418893433198e-08,
                                         3.51913959579454e-06,
                                         0.000130283385252266,
                                         0.00182111594499198,
                                         0.0123915225406043,
                                         0.0473552697900833,
                                         0.110613775648116,
                                         0.165970318850684,
                                         0.164206924964055,
                                         0.107987057346323,
                                         0.0468150294246425,
                                         0.013044238551865,
                                         0.00223047591770048,
                                         0.000217118365674713,
                                         1.06735928564639e-05,
                                         2.17797522946776e-07,
                                         1.30908508909882e-09,
                                         1.18418957771164e-12,
                                         3.24986235664783e-17,
                                         8.32078367517924e-26,
                                         1.21997151903965e-20,
                                         1.68855880657373e-13,
                                         9.8897772859708e-10,
                                         3.0371170822253e-07,
                                         1.85548841815655e-05,
                                         0.000394012368340292,
                                         0.00386215661190663,
                                         0.0205222970270565,
                                         0.0650919029092004,
                                         0.130616075816585,
                                         0.171356057109823,
                                         0.14901370285618,
                                         0.0856606941322665,
                                         0.031920043408663,
                                         0.00741143624095188,
                                         0.0010031222048526,
                                         7.10396587836008e-05,
                                         2.20214653826013e-06,
                                         2.18426500760006e-08,
                                         3.74881920416594e-11,
                                         2.56614505493438e-15,
                                         3.49017806419249e-23,
                                         3.49017806419249e-23,
                                         2.56614505493438e-15,
                                         3.74881920416588e-11,
                                         2.18426500760008e-08,
                                         2.20214653826015e-06,
                                         7.10396587836008e-05,
                                         0.0010031222048526,
                                         0.00741143624095188,
                                         0.031920043408663,
                                         0.0856606941322664,
                                         0.14901370285618,
                                         0.171356057109823,
                                         0.130616075816585,
                                         0.0650919029092004,
                                         0.0205222970270565,
                                         0.00386215661190663,
                                         0.000394012368340292,
                                         1.85548841815654e-05,
                                         3.03711708222527e-07,
                                         9.88977728597096e-10,
                                         1.68855880657373e-13,
                                         1.21997151903965e-20,
                                         8.32078367517924e-26,
                                         3.24986235664783e-17,
                                         1.18418957771162e-12,
                                         1.30908508909883e-09,
                                         2.17797522946777e-07,
                                         1.06735928564639e-05,
                                         0.000217118365674713,
                                         0.00223047591770048,
                                         0.013044238551865,
                                         0.0468150294246424,
                                         0.107987057346323,
                                         0.164206924964055,
                                         0.165970318850684,
                                         0.110613775648116,
                                         0.0473552697900833,
                                         0.0123915225406043,
                                         0.00182111594499198,
                                         0.000130283385252265,
                                         3.51913959579451e-06,
                                         2.17418893433201e-08,
                                         9.25912468644724e-12,
                                         3.55361646286151e-18,
                                         1.64801545875281e-28,
                                         3.41923646317888e-19,
                                         3.10762335050157e-14,
                                         6.51794703587555e-11,
                                         1.78953443804488e-08,
                                         1.3322962383546e-06,
                                         3.90408874980032e-05,
                                         0.000557664634275557,
                                         0.00442847701574249,
                                         0.0212554065100847,
                                         0.0650126112682645,
                                         0.130726573903452,
                                         0.175204246447283,
                                         0.156160743639597,
                                         0.0907801852278729,
                                         0.0330293376728753,
                                         0.00699271361415923,
                                         0.000759976607412667,
                                         3.38759792267415e-05,
                                         3.97089545973553e-07,
                                         4.21797574427873e-10,
                                         8.59947261030262e-16,
                                         2.69414598798739e-31,
                                         2.96931381702677e-21,
                                         6.73129016172553e-16,
                                         2.67865370752418e-12,
                                         1.21364027137028e-09,
                                         1.3726308798748e-07,
                                         5.79436163066653e-06,
                                         0.000115083076305263,
                                         0.00124094590886324,
                                         0.00796556019763319,
                                         0.0323062320004733,
                                         0.0859011710589999,
                                         0.152658721843787,
                                         0.18196896061432,
                                         0.143640400734323,
                                         0.0726671040609005,
                                         0.0221623945144955,
                                         0.00365909941212751,
                                         0.000269159687462464,
                                         5.98607867611322e-06,
                                         1.58599209240689e-08,
                                         1.71765475577869e-13,
                                         3.59687875636389e-34,
                                         2.10585222416227e-23,
                                         1.19072940122525e-17,
                                         8.99015432880061e-14,
                                         6.72178669974312e-11,
                                         1.15491908417362e-08,
                                         7.02322109832598e-07,
                                         1.93952173718563e-05,
                                         0.000283985481612367,
                                         0.0024378560754795,
                                         0.0131105175031161,
                                         0.0460976853544595,
                                         0.108628415649118,
                                         0.173167979663139,
                                         0.18561239784779,
                                         0.130563165379375,
                                         0.0573630785037391,
                                         0.014387754624828,
                                         0.00174651774473646,
                                         7.36955419635246e-05,
                                         4.87015503858154e-07,
                                         2.80184924691797e-11,
                                         3.85882389642899e-37,
                                         1.20011861233791e-25,
                                         1.69259217771151e-19,
                                         2.42461176771653e-15,
                                         2.99160301276674e-12,
                                         7.80861188520202e-10,
                                         6.84055866442657e-08,
                                         2.62665072423546e-06,
                                         5.22232518294658e-05,
                                         0.000599548454261615,
                                         0.00427541078599279,
                                         0.0198785014049678,
                                         0.0621140368899201,
                                         0.132422673759855,
                                         0.192735556073237,
                                         0.188507214027143,
                                         0.119308862842711,
                                         0.045460721401524,
                                         0.00910668839144795,
                                         0.00072906206324073,
                                         1.20173592154749e-05,
                                         3.67263813516216e-09,
                                         3.24693754458931e-40,
                                         5.36426551237612e-28,
                                         1.88704126349173e-21,
                                         5.12869751328766e-17,
                                         1.04427050580571e-13,
                                         4.14081599575299e-11,
                                         5.22560547460222e-09,
                                         2.78997182548724e-07,
                                         7.53219403387878e-06,
                                         0.000115645923360413,
                                         0.00109351734902099,
                                         0.0067232301394751,
                                         0.0278564571598537,
                                         0.0794231175488226,
                                         0.15696633438066,
                                         0.213464221866815,
                                         0.194626860617979,
                                         0.112659920106611,
                                         0.037242409015947,
                                         0.00565688824103895,
                                         0.00023257611641371,
                                         3.77573346944217e-07,
                                         2.0642358388551e-43,
                                         1.81160188391934e-30,
                                         1.58955978775429e-23,
                                         8.19668726738112e-19,
                                         2.75415555382679e-15,
                                         1.65906893094118e-12,
                                         3.01611618821839e-10,
                                         2.23904958607156e-08,
                                         8.20815389995646e-07,
                                         1.68539915451324e-05,
                                         0.000211319663296377,
                                         0.00171806154685473,
                                         0.00943905268008733,
                                         0.0359913312846276,
                                         0.0965867593055069,
                                         0.182636938040989,
                                         0.239882878592968,
                                         0.210944865048117,
                                         0.11507512566469,
                                         0.0331632511991052,
                                         0.00340085087563921,
                                         2.93285723630642e-05,
                                         9.3244851233086e-47,
                                         4.34705794310733e-33,
                                         9.51376688674743e-26,
                                         9.30785937262496e-21,
                                         5.16112153885097e-17,
                                         4.72305601320392e-14,
                                         1.23691451156011e-11,
                                         1.27675560511888e-09,
                                         6.35549892774994e-08,
                                         1.74524090299257e-06,
                                         2.90157840241809e-05,
                                         0.000311946146631087,
                                         0.00227253775028772,
                                         0.0115885489493976,
                                         0.0422288097583118,
                                         0.111027946611586,
                                         0.210075735909009,
                                         0.280639400908506,
                                         0.25264186448318,
                                         0.138139146969709,
                                         0.0353337906079191,
                                         0.00161867882046585,
                                         2.66761255728981e-50,
                                         6.60633269080701e-36,
                                         3.60628869363818e-28,
                                         6.69412175193153e-23,
                                         6.12536614031382e-19,
                                         8.51557754410546e-16,
                                         3.2126517299303e-13,
                                         4.61088383534314e-11,
                                         3.11663651837739e-09,
                                         1.14456468121349e-07,
                                         2.52325449805797e-06,
                                         3.58717760130399e-05,
                                         0.000346518279001223,
                                         0.00236315688364002,
                                         0.0116931745586714,
                                         0.042747266089615,
                                         0.116515816170342,
                                         0.236461629150044,
                                         0.351286551603083,
                                         0.364425522704924,
                                         0.232501211051608,
                                         0.0565799802508907,
                                         3.6341374664481e-54,
                                         4.78086144619637e-39,
                                         6.50952321273393e-31,
                                         2.29254625098133e-25,
                                         3.46179005496658e-21,
                                         7.31115220506019e-18,
                                         3.97345516235055e-15,
                                         7.929417664956e-13,
                                         7.277854446622e-11,
                                         3.57442347039458e-09,
                                         1.04488503102539e-07,
                                         1.96429557477353e-06,
                                         2.51606523245963e-05,
                                         0.00022947572688146,
                                         0.00154183084782199,
                                         0.00783727501092678,
                                         0.0307733357371311,
                                         0.0948753553480337,
                                         0.232593961359237,
                                         0.457806027342225,
                                         0.728519050682552,
                                         0.941771631082104,

                                         0.94649089539195,
                                         0.748038087586135,
                                         0.48876782370695,
                                         0.262976212704642,
                                         0.115831726007727,
                                         0.0414349367998374,
                                         0.0119121137711056,
                                         0.00271565430401828,
                                         0.000482676192704096,
                                         6.54692815577716e-05,
                                         6.59576682415894e-06,
                                         4.76837158203125e-07,
                                         2.36629638638731e-08,
                                         7.6054118845331e-10,
                                         1.46485507206451e-11,
                                         1.51971397964079e-13,
                                         7.29766608419124e-16,
                                         1.29475336363761e-18,
                                         5.94723243758784e-22,
                                         3.843448537213e-26,
                                         1.07121296967926e-31,
                                         7.76712847602439e-40,
                                         5.86226071110765e-55,
                                         0.0521194661512259,
                                         0.218664403940735,
                                         0.355925187425848,
                                         0.362666473274303,
                                         0.262953862893273,
                                         0.142437728949392,
                                         0.0587525802401095,
                                         0.0185309609095312,
                                         0.00444510251937833,
                                         0.000800314941930675,
                                         0.000105936757649656,
                                         1.00135803222656e-05,
                                         6.49720167064834e-07,
                                         2.74370856073349e-08,
                                         7.01467162322059e-10,
                                         9.82148126563916e-12,
                                         6.52505933895882e-14,
                                         1.66099099357504e-16,
                                         1.15531782674777e-19,
                                         1.22904764020234e-23,
                                         6.487215084305e-29,
                                         1.17177627062326e-36,
                                         4.69483355923053e-51,
                                         0.00136667163847204,
                                         0.0304377941280462,
                                         0.123422841898662,
                                         0.238165612892187,
                                         0.284257732741759,
                                         0.233165400472938,
                                         0.137989411099411,
                                         0.0602146408838808,
                                         0.0194934357220473,
                                         0.00465870253184389,
                                         0.000810230830117673,
                                         0.000100135803222656,
                                         8.49501713831906e-06,
                                         4.71339593470249e-07,
                                         1.59956074186161e-08,
                                         3.02254539320808e-10,
                                         2.77821370008941e-12,
                                         1.01467792584295e-14,
                                         1.06873182475137e-17,
                                         1.8715311910291e-21,
                                         1.87077465502606e-26,
                                         8.41801343060004e-34,
                                         1.79042435569623e-47,
                                         2.2696597260931e-05,
                                         0.00268337028478522,
                                         0.0271059546939947,
                                         0.0990565164361326,
                                         0.194615473218249,
                                         0.241732763548769,
                                         0.205256502470282,
                                         0.123919201842877,
                                         0.054141133657902,
                                         0.017175183365071,
                                         0.00392467047954525,
                                         0.000634193420410156,
                                         7.03452079285458e-05,
                                         5.12816763836955e-06,
                                         2.31007709661316e-07,
                                         5.89116270520687e-09,
                                         7.49167929801161e-11,
                                         3.9257396430473e-13,
                                         6.26135540341068e-16,
                                         1.8049192128941e-19,
                                         3.41677990343131e-24,
                                         3.83007145075954e-31,
                                         4.32438248893017e-44,
                                         2.67816597628814e-07,
                                         0.000168084708217464,
                                         0.00422974359642019,
                                         0.0292729986335399,
                                         0.0946722312962249,
                                         0.17806849818658,
                                         0.216934315105849,
                                         0.181198785281913,
                                         0.10684309685213,
                                         0.0449902028764091,
                                         0.0135075878383349,
                                         0.0028538703918457,
                                         0.0004138900095145,
                                         3.96433796952398e-05,
                                         2.37045862557968e-06,
                                         8.15848220461535e-08,
                                         1.43539977823097e-09,
                                         1.07918261942221e-11,
                                         2.60644242982342e-14,
                                         1.23679760017137e-17,
                                         4.433969465993e-22,
                                         1.23818148348201e-28,
                                         7.42117017617578e-41,
                                         2.38770477920832e-09,
                                         7.95503704252689e-06,
                                         0.000498688988105811,
                                         0.00653608629232692,
                                         0.0347963958344621,
                                         0.0991071706854705,
                                         0.17323115019914,
                                         0.200188147003729,
                                         0.159305948691986,
                                         0.0890432338106489,
                                         0.0351251999682114,
                                         0.00970315933227539,
                                         0.00183993199969475,
                                         0.000231550398992726,
                                         1.837827255299e-05,
                                         8.53658417330255e-07,
                                         2.07793986448527e-08,
                                         2.24147961837068e-10,
                                         8.19774341985056e-13,
                                         6.40333028109517e-16,
                                         4.34745199706825e-20,
                                         3.02432275715524e-26,
                                         9.62248149079337e-38,
                                         1.66960441399246e-11,
                                         2.95288152886608e-07,
                                         4.61142691854104e-05,
                                         0.00114461163810667,
                                         0.0100308036080622,
                                         0.0432626258282602,
                                         0.108495952812567,
                                         0.173464757204815,
                                         0.186297631752637,
                                         0.138220898486626,
                                         0.071639018246243,
                                         0.0258750915527344,
                                         0.00641517365498423,
                                         0.00106074308805771,
                                         0.00011175496106069,
                                         7.00565423045874e-06,
                                         2.3592984930192e-07,
                                         3.65144246021041e-09,
                                         2.0222289384739e-11,
                                         2.60017745025359e-14,
                                         3.34323348488309e-18,
                                         5.79377712883951e-24,
                                         9.78569314886142e-35,
                                         9.38147285966813e-14,
                                         8.80793958102798e-09,
                                         3.42661545340037e-06,
                                         0.000161073102621572,
                                         0.00232360166573315,
                                         0.0151755751460564,
                                         0.0546041403291484,
                                         0.120783784254257,
                                         0.175068157784303,
                                         0.172413388064802,
                                         0.117409957958988,
                                         0.0554466247558594,
                                         0.0179737852343128,
                                         0.00390480521033888,
                                         0.000546076417775188,
                                         4.61995512285104e-05,
                                         2.15257004531443e-06,
                                         4.77989784780741e-08,
                                         4.00858232645833e-10,
                                         8.48446688287434e-13,
                                         2.06596570157296e-16,
                                         8.91907712624054e-22,
                                         7.99688018605229e-32,
                                         4.30500141398293e-16,
                                         2.14559357604203e-10,
                                         2.07941052365434e-07,
                                         1.8511121045995e-05,
                                         0.000439574469281568,
                                         0.00434732664881968,
                                         0.0224430790774251,
                                         0.0686832506997492,
                                         0.134354376873628,
                                         0.175635819035976,
                                         0.157146628961063,
                                         0.0970315933227539,
                                         0.0411259088689723,
                                         0.0117390612174816,
                                         0.00217913864252281,
                                         0.000248812184919056,
                                         1.60389718152135e-05,
                                         5.10996115707523e-07,
                                         6.48927417128405e-09,
                                         2.26095000791771e-11,
                                         1.04261588204645e-14,
                                         1.12130274020058e-19,
                                         5.33696573682554e-29,
                                         1.63056593325242e-18,
                                         4.31403225343354e-12,
                                         1.04154466844067e-08,
                                         1.75592196453744e-06,
                                         6.86382237149813e-05,
                                         0.00102792678167807,
                                         0.00761381113106199,
                                         0.0322370909993274,
                                         0.0851058219589219,
                                         0.147678743502167,
                                         0.173607306845291,
                                         0.140156745910645,
                                         0.0776701714394132,
                                         0.029129307145744,
                                         0.00717759742903064,
                                         0.00110603361691349,
                                         9.86412461607328e-05,
                                         4.50899108522293e-06,
                                         8.6709011201846e-08,
                                         4.97301941546155e-10,
                                         4.34298529959139e-13,
                                         1.16355962144771e-17,
                                         2.939889683753e-26,
                                         5.13078520605458e-21,
                                         7.20609060598728e-14,
                                         4.33407044462549e-10,
                                         1.38375150181122e-07,
                                         8.90389468538575e-06,
                                         0.000201921466854038,
                                         0.00214586384442086,
                                         0.0125701729153208,
                                         0.044786500091618,
                                         0.103158089428149,
                                         0.159335054755901,
                                         0.168188095092773,
                                         0.121863436776742,
                                         0.0600492165296502,
                                         0.0196405540017448,
                                         0.00408456124665948,
                                         0.000503988909050507,
                                         3.30538123784836e-05,
                                         9.62526615121994e-07,
                                         9.08719302183986e-09,
                                         1.50290934770141e-11,
                                         1.00307822572043e-15,
                                         1.34538957169271e-23,
                                         1.34538957169303e-23,
                                         1.00307822572043e-15,
                                         1.50290934770138e-11,
                                         9.08719302183986e-09,
                                         9.62526615121994e-07,
                                         3.30538123784835e-05,
                                         0.000503988909050509,
                                         0.00408456124665946,
                                         0.0196405540017448,
                                         0.0600492165296502,
                                         0.121863436776742,
                                         0.168188095092773,
                                         0.159335054755901,
                                         0.103158089428149,
                                         0.044786500091618,
                                         0.0125701729153208,
                                         0.00214586384442085,
                                         0.000201921466854039,
                                         8.90389468538575e-06,
                                         1.38375150181122e-07,
                                         4.33407044462557e-10,
                                         7.20609060598728e-14,
                                         5.13078520605349e-21,
                                         2.93988968375375e-26,
                                         1.16355962144771e-17,
                                         4.3429852995913e-13,
                                         4.97301941546155e-10,
                                         8.6709011201846e-08,
                                         4.50899108522291e-06,
                                         9.86412461607332e-05,
                                         0.00110603361691349,
                                         0.00717759742903064,
                                         0.029129307145744,
                                         0.0776701714394132,
                                         0.140156745910645,
                                         0.173607306845291,
                                         0.147678743502167,
                                         0.0851058219589219,
                                         0.0322370909993275,
                                         0.00761381113106197,
                                         0.00102792678167807,
                                         6.86382237149813e-05,
                                         1.75592196453744e-06,
                                         1.04154466844068e-08,
                                         4.31403225343354e-12,
                                         1.63056593325211e-18,
                                         5.33696573682701e-29,
                                         1.12130274020058e-19,
                                         1.04261588204643e-14,
                                         2.26095000791771e-11,
                                         6.48927417128405e-09,
                                         5.1099611570752e-07,
                                         1.60389718152136e-05,
                                         0.000248812184919056,
                                         0.00217913864252281,
                                         0.0117390612174816,
                                         0.0411259088689723,
                                         0.0970315933227539,
                                         0.157146628961063,
                                         0.175635819035976,
                                         0.134354376873628,
                                         0.0686832506997493,
                                         0.022443079077425,
                                         0.0043473266488197,
                                         0.000439574469281568,
                                         1.8511121045995e-05,
                                         2.07941052365437e-07,
                                         2.14559357604203e-10,
                                         4.3050014139822e-16,
                                         7.99688018605466e-32,
                                         8.91907712624054e-22,
                                         2.06596570157291e-16,
                                         8.48446688287434e-13,
                                         4.00858232645833e-10,
                                         4.77989784780739e-08,
                                         2.15257004531444e-06,
                                         4.61995512285103e-05,
                                         0.000546076417775188,
                                         0.00390480521033888,
                                         0.0179737852343128,
                                         0.0554466247558594,
                                         0.117409957958988,
                                         0.172413388064802,
                                         0.175068157784303,
                                         0.120783784254257,
                                         0.0546041403291483,
                                         0.0151755751460564,
                                         0.00232360166573315,
                                         0.000161073102621572,
                                         3.42661545340041e-06,
                                         8.80793958102798e-09,
                                         9.38147285966674e-14,
                                         9.78569314886454e-35,
                                         5.79377712883951e-24,
                                         3.34323348488301e-18,
                                         2.60017745025359e-14,
                                         2.0222289384739e-11,
                                         3.65144246021039e-09,
                                         2.35929849301921e-07,
                                         7.00565423045872e-06,
                                         0.00011175496106069,
                                         0.00106074308805771,
                                         0.00641517365498423,
                                         0.0258750915527344,
                                         0.071639018246243,
                                         0.138220898486626,
                                         0.186297631752637,
                                         0.173464757204815,
                                         0.108495952812567,
                                         0.0432626258282603,
                                         0.0100308036080622,
                                         0.00114461163810667,
                                         4.61142691854108e-05,
                                         2.95288152886608e-07,
                                         1.66960441399224e-11,
                                         9.62248149079664e-38,
                                         3.02432275715524e-26,
                                         4.34745199706813e-20,
                                         6.40333028109517e-16,
                                         8.19774341985056e-13,
                                         2.24147961837067e-10,
                                         2.07793986448528e-08,
                                         8.53658417330252e-07,
                                         1.837827255299e-05,
                                         0.000231550398992726,
                                         0.00183993199969475,
                                         0.00970315933227539,
                                         0.0351251999682114,
                                         0.0890432338106489,
                                         0.159305948691986,
                                         0.200188147003729,
                                         0.17323115019914,
                                         0.0991071706854707,
                                         0.0347963958344621,
                                         0.00653608629232692,
                                         0.000498688988105815,
                                         7.95503704252689e-06,
                                         2.38770477920806e-09,
                                         7.42117017617846e-41,
                                         1.23818148348201e-28,
                                         4.43396946599287e-22,
                                         1.23679760017137e-17,
                                         2.60644242982343e-14,
                                         1.0791826194222e-11,
                                         1.43539977823098e-09,
                                         8.15848220461532e-08,
                                         2.37045862557968e-06,
                                         3.96433796952398e-05,
                                         0.0004138900095145,
                                         0.0028538703918457,
                                         0.0135075878383349,
                                         0.0449902028764091,
                                         0.10684309685213,
                                         0.181198785281914,
                                         0.216934315105848,
                                         0.178068498186581,
                                         0.0946722312962249,
                                         0.0292729986335399,
                                         0.00422974359642021,
                                         0.000168084708217464,
                                         2.67816597628791e-07,
                                         4.32438248893183e-44,
                                         3.83007145075954e-31,
                                         3.41677990343121e-24,
                                         1.8049192128941e-19,
                                         6.26135540341068e-16,
                                         3.92573964304727e-13,
                                         7.49167929801165e-11,
                                         5.89116270520685e-09,
                                         2.31007709661316e-07,
                                         5.12816763836956e-06,
                                         7.03452079285458e-05,
                                         0.000634193420410156,
                                         0.00392467047954525,
                                         0.017175183365071,
                                         0.054141133657902,
                                         0.123919201842877,
                                         0.205256502470282,
                                         0.241732763548769,
                                         0.194615473218249,
                                         0.0990565164361326,
                                         0.0271059546939948,
                                         0.00268337028478522,
                                         2.26965972609296e-05,
                                         1.79042435569695e-47,
                                         8.41801343060004e-34,
                                         1.870774655026e-26,
                                         1.8715311910291e-21,
                                         1.06873182475137e-17,
                                         1.01467792584294e-14,
                                         2.77821370008942e-12,
                                         3.02254539320807e-10,
                                         1.59956074186161e-08,
                                         4.71339593470249e-07,
                                         8.49501713831906e-06,
                                         0.000100135803222656,
                                         0.000810230830117673,
                                         0.00465870253184389,
                                         0.0194934357220473,
                                         0.0602146408838808,
                                         0.137989411099411,
                                         0.233165400472939,
                                         0.284257732741759,
                                         0.238165612892187,
                                         0.123422841898662,
                                         0.0304377941280462,
                                         0.00136667163847198,
                                         4.69483355923253e-51,
                                         1.17177627062326e-36,
                                         6.48721508430478e-29,
                                         1.22904764020234e-23,
                                         1.15531782674777e-19,
                                         1.66099099357502e-16,
                                         6.52505933895886e-14,
                                         9.82148126563912e-12,
                                         7.01467162322059e-10,
                                         2.74370856073349e-08,
                                         6.49720167064834e-07,
                                         1.00135803222656e-05,
                                         0.000105936757649656,
                                         0.000800314941930675,
                                         0.00444510251937833,
                                         0.0185309609095312,
                                         0.0587525802401095,
                                         0.142437728949392,
                                         0.262953862893273,
                                         0.362666473274303,
                                         0.355925187425849,
                                         0.218664403940735,
                                         0.0521194661512248,
                                         5.86226071111026e-55,
                                         7.76712847602439e-40,
                                         1.07121296967922e-31,
                                         3.843448537213e-26,
                                         5.94723243758784e-22,
                                         1.2947533636376e-18,
                                         7.29766608419129e-16,
                                         1.51971397964079e-13,
                                         1.46485507206451e-11,
                                         7.6054118845331e-10,
                                         2.36629638638731e-08,
                                         4.76837158203125e-07,
                                         6.59576682415894e-06,
                                         6.54692815577716e-05,
                                         0.000482676192704096,
                                         0.00271565430401828,
                                         0.0119121137711056,
                                         0.0414349367998374,
                                         0.115831726007727,
                                         0.262976212704642,
                                         0.48876782370695,
                                         0.748038087586135,
                                         0.94649089539195,

                                         0.950663525555671,
                                         0.765641782867143,
                                         0.517715612589172,
                                         0.29292382868026,
                                         0.138000033119984,
                                         0.0537718811520637,
                                         0.0171786768174251,
                                         0.00444995756706675,
                                         0.000921698521863178,
                                         0.00015000540252278,
                                         1.87684010311357e-05,
                                         1.7564343752535e-06,
                                         1.1874314457663e-07,
                                         5.54560465220217e-09,
                                         1.68773334657844e-10,
                                         3.09629144860447e-12,
                                         3.07721174308556e-14,
                                         1.42344016377597e-16,
                                         2.44598061763962e-19,
                                         1.09393639881817e-22,
                                         6.91939949299143e-27,
                                         1.89722725681393e-32,
                                         1.36019545772578e-40,
                                         1.02020231107349e-55,
                                         0.0481568955281383,
                                         0.205763137150249,
                                         0.346225868710793,
                                         0.370386886068543,
                                         0.286611978371362,
                                         0.168639821341267,
                                         0.0770289927041901,
                                         0.027487620182622,
                                         0.00764357306028264,
                                         0.00164077813041298,
                                         0.000267655499231125,
                                         3.24441069000895e-05,
                                         2.83493587438357e-06,
                                         1.71489920348726e-07,
                                         6.80455083186262e-09,
                                         1.64654230156189e-10,
                                         2.19692230955953e-12,
                                         1.39995248524572e-14,
                                         3.43943176110001e-17,
                                         2.32281978288275e-20,
                                         2.41327100645397e-24,
                                         1.25109328553818e-29,
                                         2.23201938874193e-37,
                                         8.88164155513289e-52,
                                         0.00116163807326172,
                                         0.026332386737086,
                                         0.110257583644871,
                                         0.223016608251647,
                                         0.283458959388606,
                                         0.251852196039834,
                                         0.164474825114741,
                                         0.0808535546384603,
                                         0.0301845465797398,
                                         0.00854620925064181,
                                         0.00181763121429006,
                                         0.000285377956987573,
                                         3.22298763136091e-05,
                                         2.5252768484157e-06,
                                         1.30639887987357e-07,
                                         4.16950652069153e-09,
                                         7.46883254371797e-11,
                                         6.55643985010934e-13,
                                         2.30303791996972e-15,
                                         2.34865736757137e-18,
                                         4.00797025982654e-22,
                                         3.92862645184448e-27,
                                         1.74411577724893e-34,
                                         3.68197543050531e-48,
                                         1.77466154954499e-05,
                                         0.00213424967118668,
                                         0.0222376952106418,
                                         0.0850454645407742,
                                         0.177549063623771,
                                         0.238212033742777,
                                         0.22242161719021,
                                         0.150623744835674,
                                         0.0754927486387937,
                                         0.028192236614995,
                                         0.00781749694070551,
                                         0.0015897812537715,
                                         0.000232063257526201,
                                         2.35511295776365e-05,
                                         1.58848998925461e-06,
                                         6.68696066057875e-08,
                                         1.60813717618203e-09,
                                         1.94471163667287e-11,
                                         9.76669951660077e-14,
                                         1.50402882374608e-16,
                                         4.21575371702522e-20,
                                         7.81311336663879e-25,
                                         8.63147475869389e-32,
                                         9.66720385141671e-45,
                                         1.92637311899652e-07,
                                         0.000122908067164465,
                                         0.0031867738450591,
                                         0.0230433319674867,
                                         0.0790181306989991,
                                         0.160089120735736,
                                         0.213714860952589,
                                         0.199373728824259,
                                         0.134154729259067,
                                         0.0660793418684947,
                                         0.0238896526469199,
                                         0.00629266317747647,
                                         0.00118722845534665,
                                         0.000156061096593766,
                                         1.37237668917237e-05,
                                         7.61996623053615e-07,
                                         2.46021821107895e-08,
                                         4.09847709312573e-10,
                                         2.94289475876081e-12,
                                         6.84341412012661e-15,
                                         3.15069338003528e-18,
                                         1.10404730720019e-22,
                                         3.03511301137135e-29,
                                         1.80343724510187e-41,
                                         1.57990757811432e-09,
                                         5.34788375047232e-06,
                                         0.000345047684997256,
                                         0.004717433626056,
                                         0.0265706097056239,
                                         0.0812879825272627,
                                         0.155152527243333,
                                         0.199392503016082,
                                         0.180124645682579,
                                         0.117022208290674,
                                         0.0551592501602649,
                                         0.018819063345585,
                                         0.00458911153863313,
                                         0.000781347037447851,
                                         8.95836155379167e-05,
                                         6.56060254928208e-06,
                                         2.84374446846095e-07,
                                         6.52613742198583e-09,
                                         6.69989566936819e-11,
                                         2.35264222293397e-13,
                                         1.77911267015695e-16,
                                         1.17873897318384e-20,
                                         8.063640017164e-27,
                                         2.54195346562838e-38,
                                         1.0162785757372e-11,
                                         1.82504401378219e-07,
                                         2.93019691695984e-05,
                                         0.0007574537771229,
                                         0.00700754836581945,
                                         0.032372831435757,
                                         0.0883431338031563,
                                         0.156401003118268,
                                         0.189683741356215,
                                         0.162540160750447,
                                         0.0998887750720749,
                                         0.0441419319818434,
                                         0.0139127424236512,
                                         0.00306819600861735,
                                         0.000458641823236892,
                                         4.43020910473201e-05,
                                         2.57808555051635e-06,
                                         8.15041518180052e-08,
                                         1.19633046837643e-09,
                                         6.34349742311429e-12,
                                         7.87935340517741e-15,
                                         9.87046058227497e-19,
                                         1.68026279663202e-24,
                                         2.81011421828811e-35,
                                         5.25313237819281e-14,
                                         5.00482865694511e-09,
                                         1.99958091481055e-06,
                                         9.77306887754162e-05,
                                         0.00148509820957817,
                                         0.0103599942311672,
                                         0.0404213891389529,
                                         0.0985813429083321,
                                         0.160513498854261,
                                         0.181416827217302,
                                         0.145358190485605,
                                         0.0832011170467193,
                                         0.0338938858911743,
                                         0.00968159069541874,
                                         0.00188687534986841,
                                         0.000240397091520884,
                                         1.87814261566863e-05,
                                         8.17951845653133e-07,
                                         1.71655915332584e-08,
                                         1.37444110590669e-10,
                                         2.80415615185692e-13,
                                         6.641737096835e-17,
                                         2.81350556519595e-22,
                                         2.49634632237651e-32,
                                         2.21752614824337e-16,
                                         1.12085624156172e-10,
                                         1.11436121565126e-07,
                                         1.0297946071267e-05,
                                         0.000257033112178608,
                                         0.00270759167990279,
                                         0.0151040876729969,
                                         0.0507451750965652,
                                         0.11092714759323,
                                         0.165363356686356,
                                         0.172745665161774,
                                         0.128071296680291,
                                         0.0674333617165467,
                                         0.0249491161819932,
                                         0.00633953780751639,
                                         0.00106531740452486,
                                         0.000111738963640495,
                                         6.70379261614473e-06,
                                         2.01145913598626e-07,
                                         2.43202666585277e-09,
                                         8.15001931128593e-12,
                                         3.64981433489169e-15,
                                         3.84736309781929e-20,
                                         1.81105036652752e-29,
                                         7.72648468719428e-19,
                                         2.07192203670715e-12,
                                         5.12596678975325e-09,
                                         8.95639133176102e-07,
                                         3.67185708305948e-05,
                                         0.000584076351257497,
                                         0.00465844052669617,
                                         0.0215604377860295,
                                         0.0632742380094597,
                                         0.124412433694679,
                                         0.169448450731674,
                                         0.162718619235425,
                                         0.11073656630417,
                                         0.0530672285196535,
                                         0.0175806438365685,
                                         0.0038966520721952,
                                         0.000548710796073624,
                                         4.53498859379258e-05,
                                         1.94547855651649e-06,
                                         3.55200299876477e-08,
                                         1.95514006637436e-10,
                                         1.6554751217249e-13,
                                         4.34251538595499e-18,
                                         1.08447367840974e-26,
                                         2.2365342333683e-21,
                                         3.18183245737498e-14,
                                         1.9588716955895e-10,
                                         6.47136524141674e-08,
                                         4.35775557058875e-06,
                                         0.0001046734351703,
                                         0.00119362255281434,
                                         0.00761028283851489,
                                         0.0299844719651067,
                                         0.0777622279596355,
                                         0.138085618631892,
                                         0.171752511841649,
                                         0.151073293796596,
                                         0.0937730520728616,
                                         0.040503481831019,
                                         0.0118408980140788,
                                         0.00223852925502445,
                                         0.000254866192241497,
                                         1.56322713728007e-05,
                                         4.30981589230191e-07,
                                         3.89652544726241e-09,
                                         6.23813756696606e-12,
                                         4.07192677758554e-16,
                                         5.39495598530664e-24,
                                         5.39495598530801e-24,
                                         4.07192677758554e-16,
                                         6.23813756696619e-12,
                                         3.89652544726241e-09,
                                         4.30981589230191e-07,
                                         1.56322713728008e-05,
                                         0.000254866192241497,
                                         0.00223852925502446,
                                         0.0118408980140788,
                                         0.040503481831019,
                                         0.0937730520728616,
                                         0.151073293796595,
                                         0.171752511841649,
                                         0.138085618631892,
                                         0.0777622279596355,
                                         0.0299844719651067,
                                         0.00761028283851487,
                                         0.00119362255281434,
                                         0.0001046734351703,
                                         4.35775557058875e-06,
                                         6.47136524141674e-08,
                                         1.95887169558947e-10,
                                         3.18183245737498e-14,
                                         2.23653423336779e-21,
                                         1.08447367841004e-26,
                                         4.34251538595499e-18,
                                         1.65547512172494e-13,
                                         1.95514006637436e-10,
                                         3.55200299876476e-08,
                                         1.9454785565165e-06,
                                         4.53498859379258e-05,
                                         0.000548710796073626,
                                         0.0038966520721952,
                                         0.0175806438365685,
                                         0.0530672285196535,
                                         0.110736566304169,
                                         0.162718619235426,
                                         0.169448450731674,
                                         0.124412433694679,
                                         0.0632742380094597,
                                         0.0215604377860295,
                                         0.00465844052669617,
                                         0.000584076351257495,
                                         3.67185708305948e-05,
                                         8.95639133176102e-07,
                                         5.12596678975316e-09,
                                         2.07192203670715e-12,
                                         7.72648468719268e-19,
                                         1.81105036652807e-29,
                                         3.84736309781929e-20,
                                         3.64981433489178e-15,
                                         8.15001931128593e-12,
                                         2.43202666585277e-09,
                                         2.01145913598627e-07,
                                         6.70379261614473e-06,
                                         0.000111738963640496,
                                         0.00106531740452486,
                                         0.00633953780751639,
                                         0.0249491161819932,
                                         0.0674333617165466,
                                         0.128071296680291,
                                         0.172745665161774,
                                         0.165363356686356,
                                         0.11092714759323,
                                         0.0507451750965651,
                                         0.0151040876729969,
                                         0.00270759167990278,
                                         0.000257033112178608,
                                         1.0297946071267e-05,
                                         1.11436121565124e-07,
                                         1.12085624156172e-10,
                                         2.21752614824296e-16,
                                         2.49634632237731e-32,
                                         2.81350556519595e-22,
                                         6.64173709683517e-17,
                                         2.80415615185692e-13,
                                         1.37444110590669e-10,
                                         1.71655915332585e-08,
                                         8.17951845653133e-07,
                                         1.87814261566864e-05,
                                         0.000240397091520884,
                                         0.00188687534986841,
                                         0.00968159069541874,
                                         0.0338938858911742,
                                         0.0832011170467194,
                                         0.145358190485605,
                                         0.181416827217302,
                                         0.160513498854261,
                                         0.0985813429083319,
                                         0.0404213891389529,
                                         0.0103599942311672,
                                         0.00148509820957817,
                                         9.77306887754162e-05,
                                         1.99958091481053e-06,
                                         5.00482865694511e-09,
                                         5.25313237819196e-14,
                                         2.81011421828908e-35,
                                         1.68026279663202e-24,
                                         9.87046058227524e-19,
                                         7.87935340517741e-15,
                                         6.34349742311429e-12,
                                         1.19633046837644e-09,
                                         8.15041518180052e-08,
                                         2.57808555051636e-06,
                                         4.43020910473201e-05,
                                         0.000458641823236892,
                                         0.00306819600861735,
                                         0.0139127424236512,
                                         0.0441419319818435,
                                         0.0998887750720749,
                                         0.162540160750447,
                                         0.189683741356215,
                                         0.156401003118268,
                                         0.0883431338031563,
                                         0.0323728314357569,
                                         0.00700754836581945,
                                         0.0007574537771229,
                                         2.93019691695981e-05,
                                         1.82504401378219e-07,
                                         1.01627857573706e-11,
                                         2.54195346562932e-38,
                                         8.063640017164e-27,
                                         1.17873897318388e-20,
                                         1.77911267015695e-16,
                                         2.35264222293397e-13,
                                         6.69989566936823e-11,
                                         6.52613742198583e-09,
                                         2.84374446846097e-07,
                                         6.56060254928208e-06,
                                         8.95836155379167e-05,
                                         0.000781347037447851,
                                         0.00458911153863312,
                                         0.018819063345585,
                                         0.0551592501602649,
                                         0.117022208290674,
                                         0.180124645682579,
                                         0.199392503016081,
                                         0.155152527243333,
                                         0.0812879825272625,
                                         0.0265706097056239,
                                         0.004717433626056,
                                         0.000345047684997252,
                                         5.34788375047232e-06,
                                         1.57990757811414e-09,
                                         1.80343724510258e-41,
                                         3.03511301137135e-29,
                                         1.10404730720022e-22,
                                         3.15069338003528e-18,
                                         6.84341412012661e-15,
                                         2.94289475876083e-12,
                                         4.09847709312573e-10,
                                         2.46021821107896e-08,
                                         7.61996623053615e-07,
                                         1.37237668917237e-05,
                                         0.000156061096593766,
                                         0.00118722845534664,
                                         0.00629266317747648,
                                         0.0238896526469199,
                                         0.0660793418684947,
                                         0.134154729259067,
                                         0.199373728824259,
                                         0.213714860952589,
                                         0.160089120735736,
                                         0.0790181306989991,
                                         0.0230433319674867,
                                         0.00318677384505907,
                                         0.000122908067164465,
                                         1.92637311899634e-07,
                                         9.66720385142072e-45,
                                         8.63147475869389e-32,
                                         7.81311336663904e-25,
                                         4.21575371702522e-20,
                                         1.50402882374608e-16,
                                         9.76669951660085e-14,
                                         1.94471163667287e-11,
                                         1.60813717618204e-09,
                                         6.68696066057875e-08,
                                         1.58848998925461e-06,
                                         2.35511295776365e-05,
                                         0.000232063257526201,
                                         0.0015897812537715,
                                         0.00781749694070551,
                                         0.028192236614995,
                                         0.0754927486387937,
                                         0.150623744835674,
                                         0.22242161719021,
                                         0.238212033742776,
                                         0.177549063623771,
                                         0.0850454645407742,
                                         0.0222376952106417,
                                         0.00213424967118668,
                                         1.77466154954487e-05,
                                         3.68197543050692e-48,
                                         1.74411577724893e-34,
                                         3.92862645184461e-27,
                                         4.00797025982654e-22,
                                         2.34865736757137e-18,
                                         2.30303791996974e-15,
                                         6.55643985010934e-13,
                                         7.46883254371801e-11,
                                         4.16950652069153e-09,
                                         1.30639887987357e-07,
                                         2.5252768484157e-06,
                                         3.2229876313609e-05,
                                         0.000285377956987573,
                                         0.00181763121429006,
                                         0.00854620925064181,
                                         0.0301845465797398,
                                         0.0808535546384603,
                                         0.164474825114741,
                                         0.251852196039834,
                                         0.283458959388606,
                                         0.223016608251647,
                                         0.110257583644871,
                                         0.026332386737086,
                                         0.00116163807326166,
                                         8.88164155513699e-52,
                                         2.23201938874193e-37,
                                         1.25109328553822e-29,
                                         2.41327100645397e-24,
                                         2.32281978288275e-20,
                                         3.43943176110004e-17,
                                         1.39995248524572e-14,
                                         2.19692230955954e-12,
                                         1.64654230156189e-10,
                                         6.80455083186262e-09,
                                         1.71489920348726e-07,
                                         2.83493587438357e-06,
                                         3.24441069000895e-05,
                                         0.000267655499231125,
                                         0.00164077813041298,
                                         0.00764357306028264,
                                         0.027487620182622,
                                         0.0770289927041901,
                                         0.168639821341267,
                                         0.286611978371362,
                                         0.370386886068543,
                                         0.346225868710792,
                                         0.205763137150249,
                                         0.0481568955281372,
                                         1.02020231107398e-55,
                                         1.36019545772578e-40,
                                         1.897227256814e-32,
                                         6.91939949299143e-27,
                                         1.09393639881817e-22,
                                         2.44598061763965e-19,
                                         1.42344016377597e-16,
                                         3.07721174308558e-14,
                                         3.09629144860447e-12,
                                         1.68773334657844e-10,
                                         5.54560465220217e-09,
                                         1.1874314457663e-07,
                                         1.7564343752535e-06,
                                         1.87684010311357e-05,
                                         0.00015000540252278,
                                         0.000921698521863178,
                                         0.00444995756706675,
                                         0.0171786768174251,
                                         0.0537718811520637,
                                         0.138000033119984,
                                         0.29292382868026,
                                         0.517715612589172,
                                         0.765641782867143,
                                         0.950663525555671,

                                         2.38418579101562e-07,
                                         5.24520874023438e-06,
                                         5.50746917724609e-05,
                                         0.000367164611816406,
                                         0.00174403190612793,
                                         0.00627851486206055,
                                         0.0177891254425049,
                                         0.0406608581542969,
                                         0.0762391090393066,
                                         0.118594169616699,
                                         0.154172420501709,
                                         0.168188095092773,
                                         0.154172420501709,
                                         0.118594169616699,
                                         0.0762391090393066,
                                         0.0406608581542969,
                                         0.0177891254425049,
                                         0.00627851486206055,
                                         0.00174403190612793,
                                         0.000367164611816406,
                                         5.50746917724609e-05,
                                         5.24520874023438e-06,
                                         2.38418579101562e-07,

                                         0.0053921165225759,
                                         1.40900723426495e-15,
                                         0.031785891889812,
                                         1.15686704903195e-13,
                                         0.0894285426570699,
                                         4.53336093481901e-12,
                                         0.159748705235027,
                                         1.12791555585283e-10,
                                         0.203321548658304,
                                         1.99948312651778e-09,
                                         0.196127441160616,
                                         2.68638214213477e-08,
                                         0.14889787017228,
                                         2.84061914884793e-07,
                                         0.0911932892968609,
                                         2.42316228307509e-06,
                                         0.0458159404166843,
                                         1.69563089164141e-05,
                                         0.019096535476858,
                                         9.84383654876076e-05,
                                         0.00665197163703329,
                                         0.000477590075837923,
                                         0.00194442592169215,
                                         0.00194442592169215,
                                         0.000477590075837923,
                                         0.00665197163703329,
                                         9.84383654876076e-05,
                                         0.019096535476858,
                                         1.69563089164141e-05,
                                         0.0458159404166843,
                                         2.42316228307509e-06,
                                         0.0911932892968609,
                                         2.84061914884793e-07,
                                         0.14889787017228,
                                         2.68638214213477e-08,
                                         0.196127441160616,
                                         1.99948312651778e-09,
                                         0.203321548658304,
                                         1.12791555585283e-10,
                                         0.159748705235027,
                                         4.53336093481901e-12,
                                         0.0894285426570699,
                                         1.15686704903195e-13,
                                         0.031785891889812,
                                         1.40900723426495e-15,
                                         0.0053921165225759,

                                         0.0720331920799996,
                                         2.38418579101562e-07,
                                         1.38824890460143e-21,
                                         0.201287130439999,
                                         5.24520874023438e-06,
                                         2.40452531138989e-19,
                                         0.26845158648,
                                         5.50746917724609e-05,
                                         1.98773271187121e-17,
                                         0.227318814799999,
                                         0.000367164611816406,
                                         1.04329243581825e-15,
                                         0.137148057199961,
                                         0.00174403190612793,
                                         3.90156638690008e-14,
                                         0.0627123142788945,
                                         0.00627851486206055,
                                         1.10581081877473e-12,
                                         0.0225689402153331,
                                         0.0177891254425049,
                                         2.46670854540967e-11,
                                         0.00655230003610623,
                                         0.0406608581542969,
                                         4.43893835381892e-10,
                                         0.00156047104730857,
                                         0.0762391090393066,
                                         6.55269145021081e-09,
                                         0.000308320150085644,
                                         0.118594169616699,
                                         8.02499143605372e-08,
                                         5.09103318889172e-05,
                                         0.154172420501709,
                                         8.21348111083603e-07,
                                         7.05432000000012e-06,
                                         0.168188095092773,
                                         7.05432000000012e-06,
                                         8.21348111083603e-07,
                                         0.154172420501709,
                                         5.09103318889172e-05,
                                         8.02499143605372e-08,
                                         0.118594169616699,
                                         0.000308320150085644,
                                         6.55269145021081e-09,
                                         0.0762391090393066,
                                         0.00156047104730857,
                                         4.43893835381892e-10,
                                         0.0406608581542969,
                                         0.00655230003610623,
                                         2.46670854540967e-11,
                                         0.0177891254425049,
                                         0.0225689402153331,
                                         1.10581081877473e-12,
                                         0.00627851486206055,
                                         0.0627123142788945,
                                         3.90156638690008e-14,
                                         0.00174403190612793,
                                         0.137148057199961,
                                         1.04329243581825e-15,
                                         0.000367164611816406,
                                         0.227318814799999,
                                         1.98773271187121e-17,
                                         5.50746917724609e-05,
                                         0.26845158648,
                                         2.40452531138989e-19,
                                         5.24520874023438e-06,
                                         0.201287130439999,
                                         1.38824890460143e-21,
                                         2.38418579101562e-07,
                                         0.0720331920799996,

                                         0.205333641872347,
                                         0.000149111278966249,
                                         2.55611915934234e-11,
                                         3.26807051558482e-26,
                                         0.33704920327709,
                                         0.00161581237771411,
                                         1.14168428161304e-09,
                                         9.63615075005995e-24,
                                         0.264054223284258,
                                         0.00835676104992379,
                                         2.43375897656177e-08,
                                         1.3560708191135e-21,
                                         0.131344645583801,
                                         0.0274412872410267,
                                         3.29403760457652e-07,
                                         1.21165951613747e-19,
                                         0.0465496582203349,
                                         0.0642030772058662,
                                         3.17661372365287e-06,
                                         7.71371685148752e-18,
                                         0.0125034399142488,
                                         0.113845560419037,
                                         2.32171631920711e-05,
                                         3.72182269533874e-16,
                                         0.00264324668648706,
                                         0.158880839120829,
                                         0.000133551592327699,
                                         1.41332758924858e-14,
                                         0.000450785723251298,
                                         0.178875648825338,
                                         0.000619745323789513,
                                         4.32966470599145e-13,
                                         6.30640618127969e-05,
                                         0.165200078245974,
                                         0.00235915663195162,
                                         1.08804040501636e-11,
                                         7.31944209715326e-06,
                                         0.126576630212509,
                                         0.00745048505200139,
                                         2.26840207376826e-10,
                                         7.09956673489483e-07,
                                         0.0810503018032808,
                                         0.0196639221186082,
                                         3.95232704274468e-09,
                                         5.77871027256619e-08,
                                         0.0435512828153443,
                                         0.0435512828153443,
                                         5.77871027256624e-08,
                                         3.95232704274464e-09,
                                         0.0196639221186082,
                                         0.0810503018032808,
                                         7.09956673489489e-07,
                                         2.26840207376823e-10,
                                         0.00745048505200139,
                                         0.126576630212509,
                                         7.31944209715331e-06,
                                         1.08804040501635e-11,
                                         0.00235915663195162,
                                         0.165200078245974,
                                         6.30640618127973e-05,
                                         4.3296647059914e-13,
                                         0.000619745323789513,
                                         0.178875648825338,
                                         0.000450785723251301,
                                         1.41332758924857e-14,
                                         0.000133551592327699,
                                         0.158880839120829,
                                         0.00264324668648707,
                                         3.72182269533869e-16,
                                         2.32171631920711e-05,
                                         0.113845560419037,
                                         0.0125034399142488,
                                         7.71371685148741e-18,
                                         3.17661372365287e-06,
                                         0.0642030772058663,
                                         0.046549658220335,
                                         1.21165951613745e-19,
                                         3.29403760457652e-07,
                                         0.0274412872410267,
                                         0.131344645583801,
                                         1.35607081911347e-21,
                                         2.43375897656177e-08,
                                         0.00835676104992379,
                                         0.264054223284258,
                                         9.63615075005979e-24,
                                         1.14168428161304e-09,
                                         0.00161581237771411,
                                         0.33704920327709,
                                         3.26807051558476e-26,
                                         2.55611915934234e-11,
                                         0.000149111278966249,
                                         0.205333641872347,

                                         0.34749240029522,
                                         0.00311382724479326,
                                         2.38418579101562e-07,
                                         9.76681514662496e-15,
                                         5.8594278936846e-30,
                                         0.376270577619768,
                                         0.0205508099481383,
                                         5.24520874023438e-06,
                                         7.1624878945313e-13,
                                         2.61906107883973e-27,
                                         0.194456214706401,
                                         0.0647336342850395,
                                         5.50746917724609e-05,
                                         2.50692563973942e-11,
                                         5.58730857820588e-25,
                                         0.0638061935264906,
                                         0.129464434528925,
                                         0.000367164611816406,
                                         5.57106781687592e-10,
                                         7.56796526387669e-23,
                                         0.0149172481639217,
                                         0.184482780783477,
                                         0.00174403190612793,
                                         8.82105046969518e-09,
                                         7.30365871535863e-21,
                                         0.00264316018554698,
                                         0.199237041847768,
                                         0.00627851486206055,
                                         1.05854922802765e-07,
                                         5.34208389048529e-19,
                                         0.000368598380846009,
                                         0.169347778463124,
                                         0.0177891254425049,
                                         9.99762822410273e-07,
                                         3.07521940491752e-17,
                                         4.1467479069092e-05,
                                         0.11612164898523,
                                         0.0406608581542969,
                                         7.61740729689866e-06,
                                         1.42812354981967e-15,
                                         3.82684766381688e-06,
                                         0.0653169977324786,
                                         0.0762391090393066,
                                         4.76098377859383e-05,
                                         5.44044951364524e-14,
                                         2.92994163410681e-07,
                                         0.0304805983729632,
                                         0.118594169616699,
                                         0.000246871229572976,
                                         1.71944402532916e-12,
                                         1.87471213125453e-08,
                                         0.0118871731492964,
                                         0.154172420501709,
                                         0.00106979874606597,
                                         4.54149799062059e-11,
                                         1.00659650680935e-09,
                                         0.00389026241634546,
                                         0.168188095092773,
                                         0.00389026241634546,
                                         1.00659650680936e-09,
                                         4.54149799062052e-11,
                                         0.00106979874606597,
                                         0.154172420501709,
                                         0.0118871731492964,
                                         1.87471213125455e-08,
                                         1.71944402532913e-12,
                                         0.000246871229572976,
                                         0.118594169616699,
                                         0.0304805983729632,
                                         2.92994163410684e-07,
                                         5.44044951364515e-14,
                                         4.76098377859383e-05,
                                         0.0762391090393066,
                                         0.0653169977324786,
                                         3.82684766381692e-06,
                                         1.42812354981964e-15,
                                         7.61740729689866e-06,
                                         0.0406608581542969,
                                         0.11612164898523,
                                         4.14674790690923e-05,
                                         3.07521940491746e-17,
                                         9.99762822410273e-07,
                                         0.0177891254425049,
                                         0.169347778463124,
                                         0.000368598380846012,
                                         5.34208389048518e-19,
                                         1.05854922802765e-07,
                                         0.00627851486206055,
                                         0.199237041847768,
                                         0.00264316018554699,
                                         7.30365871535847e-21,
                                         8.82105046969519e-09,
                                         0.00174403190612793,
                                         0.184482780783477,
                                         0.0149172481639218,
                                         7.56796526387652e-23,
                                         5.57106781687592e-10,
                                         0.000367164611816406,
                                         0.129464434528925,
                                         0.0638061935264908,
                                         5.58730857820575e-25,
                                         2.50692563973942e-11,
                                         5.50746917724609e-05,
                                         0.0647336342850395,
                                         0.194456214706401,
                                         2.61906107883967e-27,
                                         7.1624878945313e-13,
                                         5.24520874023438e-06,
                                         0.0205508099481383,
                                         0.376270577619769,
                                         5.85942789368445e-30,
                                         9.76681514662496e-15,
                                         2.38418579101562e-07,
                                         0.00311382724479326,
                                         0.34749240029522,

                                         0.469697692136423,
                                         0.0168529672031657,
                                         2.64223693566535e-05,
                                         5.92355641605354e-10,
                                         1.08600098971043e-17,
                                         4.22997033768134e-33,
                                         0.36110069943102,
                                         0.0756146679600332,
                                         0.000357321020756912,
                                         2.1200254962897e-08,
                                         1.17150975400496e-15,
                                         2.6630098953998e-30,
                                         0.132496635774481,
                                         0.161920688367071,
                                         0.00230627977754389,
                                         3.6213146129077e-07,
                                         6.03154003695809e-14,
                                         8.00156351174617e-28,
                                         0.0308674543618461,
                                         0.22014968854459,
                                         0.00945115439233036,
                                         3.92745187550878e-06,
                                         1.971650634932e-12,
                                         1.52649847644874e-25,
                                         0.0051236758572248,
                                         0.213264553601645,
                                         0.0275957748729818,
                                         3.03487183987369e-05,
                                         4.59215543495617e-11,
                                         2.07492671471706e-23,
                                         0.000644571902314291,
                                         0.156577073530881,
                                         0.0610673704802115,
                                         0.000177737362303628,
                                         8.10610243274818e-10,
                                         2.13755865415393e-21,
                                         6.38199212082552e-05,
                                         0.0904758841222553,
                                         0.106358209335134,
                                         0.000819242125875166,
                                         1.12616691788971e-08,
                                         1.73311942127399e-19,
                                         5.09759618209808e-06,
                                         0.0421756487140491,
                                         0.149436637000166,
                                         0.00304628191790185,
                                         1.26216959805547e-07,
                                         1.13360880127052e-17,
                                         3.34005734673118e-07,
                                         0.016127610876929,
                                         0.17223542592816,
                                         0.00929195801421879,
                                         1.16041141997651e-06,
                                         6.08243194806261e-16,
                                         1.81562716616592e-08,
                                         0.00511637720732504,
                                         0.164691860413481,
                                         0.023514115067505,
                                         8.85096304466124e-06,
                                         2.70754424504391e-14,
                                         8.24816323118068e-10,
                                         0.00135647845686389,
                                         0.131607189180312,
                                         0.0497287869542124,
                                         5.64191951967145e-05,
                                         1.00723704934344e-12,
                                         3.14436555735083e-11,
                                         0.000301792508336655,
                                         0.0882535736932035,
                                         0.0882535736932035,
                                         0.000301792508336653,
                                         3.14436555735083e-11,
                                         1.00723704934344e-12,
                                         5.64191951967148e-05,
                                         0.0497287869542124,
                                         0.131607189180312,
                                         0.00135647845686389,
                                         8.24816323118068e-10,
                                         2.70754424504391e-14,
                                         8.85096304466128e-06,
                                         0.023514115067505,
                                         0.164691860413481,
                                         0.00511637720732503,
                                         1.81562716616592e-08,
                                         6.0824319480626e-16,
                                         1.16041141997652e-06,
                                         0.00929195801421879,
                                         0.17223542592816,
                                         0.016127610876929,
                                         3.34005734673118e-07,
                                         1.13360880127052e-17,
                                         1.26216959805548e-07,
                                         0.00304628191790185,
                                         0.149436637000166,
                                         0.042175648714049,
                                         5.09759618209808e-06,
                                         1.73311942127399e-19,
                                         1.12616691788972e-08,
                                         0.000819242125875166,
                                         0.106358209335134,
                                         0.0904758841222551,
                                         6.38199212082552e-05,
                                         2.13755865415393e-21,
                                         8.10610243274822e-10,
                                         0.000177737362303628,
                                         0.0610673704802115,
                                         0.156577073530881,
                                         0.000644571902314291,
                                         2.07492671471706e-23,
                                         4.5921554349562e-11,
                                         3.03487183987369e-05,
                                         0.0275957748729818,
                                         0.213264553601645,
                                         0.0051236758572248,
                                         1.52649847644874e-25,
                                         1.97165063493201e-12,
                                         3.92745187550878e-06,
                                         0.00945115439233036,
                                         0.22014968854459,
                                         0.0308674543618461,
                                         8.00156351174616e-28,
                                         6.03154003695813e-14,
                                         3.6213146129077e-07,
                                         0.00230627977754389,
                                         0.161920688367071,
                                         0.132496635774481,
                                         2.6630098953998e-30,
                                         1.17150975400497e-15,
                                         2.1200254962897e-08,
                                         0.000357321020756912,
                                         0.0756146679600331,
                                         0.36110069943102,
                                         4.22997033768134e-33,
                                         1.08600098971043e-17,
                                         5.92355641605354e-10,
                                         2.64223693566535e-05,
                                         0.0168529672031657,
                                         0.469697692136423,

                                         0.567191291581146,
                                         0.0476242031868723,
                                         0.000428513805334951,
                                         2.38418579101562e-07,
                                         2.53006860441463e-12,
                                         2.8204565334678e-20,
                                         8.38775930249453e-36,
                                         0.32581165590123,
                                         0.155498891778646,
                                         0.00398427821700327,
                                         5.24520874023438e-06,
                                         1.31702136480267e-10,
                                         4.18085587946678e-18,
                                         7.06731189601393e-33,
                                         0.0893244391919985,
                                         0.242322361250407,
                                         0.0176807675404597,
                                         5.50746917724609e-05,
                                         3.27205022305819e-09,
                                         2.95785991015845e-16,
                                         2.84203338438453e-30,
                                         0.0155486762619492,
                                         0.239761323954668,
                                         0.0498163625452903,
                                         0.000367164611816406,
                                         5.16138842908669e-08,
                                         1.32864649793333e-14,
                                         7.25643875324875e-28,
                                         0.00192841904943521,
                                         0.169024489401653,
                                         0.100006397049222,
                                         0.00174403190612793,
                                         5.80093494862237e-07,
                                         4.2523221812011e-13,
                                         1.32008763036137e-25,
                                         0.000181266811578864,
                                         0.0903085830846012,
                                         0.15215717627982,
                                         0.00627851486206055,
                                         4.94126718709375e-06,
                                         1.03145852024837e-11,
                                         1.820081688444e-23,
                                         1.34100485063486e-05,
                                         0.0379755081890489,
                                         0.182201634744251,
                                         0.0177891254425049,
                                         3.31263701868799e-05,
                                         1.96912163433961e-10,
                                         1.97503214864929e-21,
                                         8.00325534716952e-07,
                                         0.0128825676544839,
                                         0.176009605697951,
                                         0.0406608581542969,
                                         0.000179156939663773,
                                         3.03261718319842e-09,
                                         1.72894719688467e-19,
                                         3.91816145182553e-08,
                                         0.0035849293307487,
                                         0.139476101609318,
                                         0.0762391090393066,
                                         0.000794827251933412,
                                         3.83126302691412e-08,
                                         1.24156259472262e-17,
                                         1.59141056114457e-09,
                                         0.000827642950121331,
                                         0.0916953863341735,
                                         0.118594169616699,
                                         0.002925473155067,
                                         4.01560142647046e-07,
                                         7.39672996512698e-16,
                                         5.40181828017965e-11,
                                         0.000159684685513473,
                                         0.0503794004861852,
                                         0.154172420501709,
                                         0.00899864898534633,
                                         3.51736191778466e-06,
                                         3.68272028785053e-14,
                                         1.53866088368394e-12,
                                         2.58540582618887e-05,
                                         0.0232275666079435,
                                         0.168188095092773,
                                         0.0232275666079435,
                                         2.58540582618887e-05,
                                         1.53866088368398e-12,
                                         3.68272028785044e-14,
                                         3.51736191778466e-06,
                                         0.00899864898534633,
                                         0.154172420501709,
                                         0.0503794004861852,
                                         0.000159684685513473,
                                         5.40181828017977e-11,
                                         7.39672996512677e-16,
                                         4.01560142647046e-07,
                                         0.002925473155067,
                                         0.118594169616699,
                                         0.0916953863341735,
                                         0.000827642950121331,
                                         1.5914105611446e-09,
                                         1.24156259472258e-17,
                                         3.83126302691412e-08,
                                         0.000794827251933412,
                                         0.0762391090393066,
                                         0.139476101609318,
                                         0.0035849293307487,
                                         3.9181614518256e-08,
                                         1.72894719688461e-19,
                                         3.03261718319842e-09,
                                         0.000179156939663773,
                                         0.0406608581542969,
                                         0.176009605697951,
                                         0.0128825676544839,
                                         8.00325534716964e-07,
                                         1.97503214864922e-21,
                                         1.96912163433961e-10,
                                         3.31263701868799e-05,
                                         0.0177891254425049,
                                         0.182201634744251,
                                         0.0379755081890489,
                                         1.34100485063487e-05,
                                         1.82008168844394e-23,
                                         1.03145852024837e-11,
                                         4.94126718709375e-06,
                                         0.00627851486206055,
                                         0.15215717627982,
                                         0.0903085830846012,
                                         0.000181266811578866,
                                         1.32008763036132e-25,
                                         4.2523221812011e-13,
                                         5.80093494862237e-07,
                                         0.00174403190612793,
                                         0.100006397049222,
                                         0.169024489401653,
                                         0.00192841904943523,
                                         7.25643875324845e-28,
                                         1.32864649793333e-14,
                                         5.16138842908669e-08,
                                         0.000367164611816406,
                                         0.0498163625452903,
                                         0.239761323954668,
                                         0.0155486762619493,
                                         2.84203338438441e-30,
                                         2.95785991015845e-16,
                                         3.27205022305819e-09,
                                         5.50746917724609e-05,
                                         0.0176807675404597,
                                         0.242322361250407,
                                         0.0893244391919989,
                                         7.06731189601361e-33,
                                         4.18085587946678e-18,
                                         1.31702136480267e-10,
                                         5.24520874023438e-06,
                                         0.00398427821700327,
                                         0.155498891778646,
                                         0.325811655901231,
                                         8.38775930249413e-36,
                                         2.8204565334678e-20,
                                         2.53006860441463e-12,
                                         2.38418579101562e-07,
                                         0.000428513805334951,
                                         0.0476242031868723,
                                         0.567191291581146,

                                         0.643259981763733,
                                         0.0945418985217546,
                                         0.00258590011725265,
                                         9.69424999121119e-06,
                                         2.76147440030483e-09,
                                         1.79425530707478e-14,
                                         1.43859237691387e-22,
                                         3.57414476017725e-38,
                                         0.286675368039263,
                                         0.235390276939049,
                                         0.017693735875293,
                                         0.000147157895638887,
                                         8.80475000223127e-08,
                                         1.26917587114293e-12,
                                         2.79652634934568e-20,
                                         3.88162570626519e-35,
                                         0.0609762681983704,
                                         0.279717307213156,
                                         0.0577821839381775,
                                         0.00106615301863209,
                                         1.33986072016943e-06,
                                         4.28475498446465e-11,
                                         2.59457778295556e-18,
                                         2.01197134257436e-32,
                                         0.00823475621007842,
                                         0.211042342838911,
                                         0.119808548520326,
                                         0.00490427658361008,
                                         1.29455862721862e-05,
                                         9.18437526463247e-10,
                                         1.52838904488339e-16,
                                         6.62139303553935e-30,
                                         0.000792365430652603,
                                         0.113450059728909,
                                         0.176997254441911,
                                         0.0160736770206555,
                                         8.91186624871463e-05,
                                         1.40267842342318e-08,
                                         6.4148426194964e-15,
                                         1.55260797817484e-27,
                                         5.77842105706606e-05,
                                         0.0462220017239237,
                                         0.198177407962056,
                                         0.0399267914469432,
                                         0.000464969523185505,
                                         1.62358677078073e-07,
                                         2.0405488069374e-13,
                                         2.75919980343559e-25,
                                         3.3165612275519e-06,
                                         0.0148213560450594,
                                         0.174637267772963,
                                         0.0780564427387193,
                                         0.00190930577255746,
                                         1.47906772929458e-06,
                                         5.10861457532077e-12,
                                         3.85922107318398e-23,
                                         1.53564604562955e-07,
                                         0.0038339937682649,
                                         0.124149216800529,
                                         0.123105475791424,
                                         0.0063248576452463,
                                         1.08698835357866e-05,
                                         1.03177157964838e-10,
                                         4.35452147212694e-21,
                                         5.83275267674211e-09,
                                         0.00081356863002518,
                                         0.0723986335728109,
                                         0.159266822667084,
                                         0.0171872088470534,
                                         6.55301330339579e-05,
                                         1.70939847921064e-09,
                                         4.03051502658642e-19,
                                         1.837977758783e-10,
                                         0.000143225794661491,
                                         0.0350268440802083,
                                         0.170945438009771,
                                         0.0387475464404076,
                                         0.00032774906583544,
                                         2.34956421848956e-08,
                                         3.09503031402856e-17,
                                         4.84021627759784e-12,
                                         2.10720172251333e-05,
                                         0.0141621592882774,
                                         0.153337204266867,
                                         0.0730030301337588,
                                         0.0013699341799356,
                                         2.69891465925243e-07,
                                         1.98621883315429e-15,
                                         1.06963225962431e-13,
                                         2.60157405804818e-06,
                                         0.00480510795209126,
                                         0.115420453030002,
                                         0.115420453030001,
                                         0.00480510795209126,
                                         2.60157405804818e-06,
                                         1.06963225962431e-13,
                                         1.98621883315429e-15,
                                         2.69891465925243e-07,
                                         0.0013699341799356,
                                         0.073003030133759,
                                         0.153337204266867,
                                         0.0141621592882774,
                                         2.10720172251333e-05,
                                         4.84021627759784e-12,
                                         3.09503031402856e-17,
                                         2.34956421848956e-08,
                                         0.00032774906583544,
                                         0.0387475464404077,
                                         0.170945438009771,
                                         0.0350268440802083,
                                         0.000143225794661491,
                                         1.837977758783e-10,
                                         4.03051502658642e-19,
                                         1.70939847921064e-09,
                                         6.55301330339579e-05,
                                         0.0171872088470535,
                                         0.159266822667083,
                                         0.0723986335728109,
                                         0.00081356863002518,
                                         5.83275267674211e-09,
                                         4.35452147212694e-21,
                                         1.03177157964838e-10,
                                         1.08698835357866e-05,
                                         0.00632485764524631,
                                         0.123105475791424,
                                         0.124149216800529,
                                         0.0038339937682649,
                                         1.53564604562955e-07,
                                         3.85922107318398e-23,
                                         5.10861457532077e-12,
                                         1.47906772929458e-06,
                                         0.00190930577255746,
                                         0.0780564427387193,
                                         0.174637267772963,
                                         0.0148213560450594,
                                         3.3165612275519e-06,
                                         2.75919980343559e-25,
                                         2.0405488069374e-13,
                                         1.62358677078073e-07,
                                         0.000464969523185506,
                                         0.0399267914469431,
                                         0.198177407962056,
                                         0.0462220017239237,
                                         5.77842105706606e-05,
                                         1.55260797817484e-27,
                                         6.4148426194964e-15,
                                         1.40267842342318e-08,
                                         8.91186624871465e-05,
                                         0.0160736770206555,
                                         0.176997254441911,
                                         0.113450059728909,
                                         0.000792365430652603,
                                         6.62139303553935e-30,
                                         1.52838904488339e-16,
                                         9.18437526463247e-10,
                                         1.29455862721862e-05,
                                         0.00490427658361008,
                                         0.119808548520326,
                                         0.211042342838911,
                                         0.00823475621007842,
                                         2.01197134257436e-32,
                                         2.59457778295556e-18,
                                         4.28475498446465e-11,
                                         1.33986072016943e-06,
                                         0.00106615301863209,
                                         0.0577821839381775,
                                         0.279717307213156,
                                         0.0609762681983704,
                                         3.88162570626519e-35,
                                         2.79652634934568e-20,
                                         1.26917587114293e-12,
                                         8.80475000223129e-08,
                                         0.000147157895638887,
                                         0.017693735875293,
                                         0.235390276939049,
                                         0.286675368039263,
                                         3.57414476017725e-38,
                                         1.43859237691387e-22,
                                         1.79425530707478e-14,
                                         2.76147440030484e-09,
                                         9.69424999121119e-06,
                                         0.00258590011725265,
                                         0.0945418985217546,
                                         0.643259981763733,

                                         0.702538426187749,
                                         0.152300337757778,
                                         0.00886127219300658,
                                         0.000115001734507255,
                                         2.38418579101562e-07,
                                         4.29126058486504e-11,
                                         1.98521148111115e-16,
                                         1.26500516375899e-24,
                                         2.77124883553138e-40,
                                         0.25003574663528,
                                         0.299229892128702,
                                         0.0467173645608344,
                                         0.00129104036621282,
                                         5.24520874023438e-06,
                                         1.85009836194356e-09,
                                         1.82250983948372e-14,
                                         3.11625903315548e-22,
                                         3.76867655804076e-37,
                                         0.0424718070915832,
                                         0.280592270822043,
                                         0.117551239163953,
                                         0.00691738376440855,
                                         5.50746917724609e-05,
                                         3.80689911426169e-08,
                                         7.98545383737718e-13,
                                         3.66388151531686e-20,
                                         2.44606969541138e-34,
                                         0.0045805625589802,
                                         0.167057459813906,
                                         0.187799974889245,
                                         0.0235322449877016,
                                         0.000367164611816406,
                                         4.97355996978191e-07,
                                         2.22151252960338e-11,
                                         2.7350718078302e-18,
                                         1.00801786678562e-31,
                                         0.000351983063363311,
                                         0.0708664882085683,
                                         0.213770964548807,
                                         0.0570387133745494,
                                         0.00174403190612793,
                                         4.62965062818397e-06,
                                         4.4033458699125e-10,
                                         1.45472503844197e-16,
                                         2.95973277227717e-29,
                                         2.04990439069071e-05,
                                         0.0227837308367076,
                                         0.184421179328395,
                                         0.104781586828022,
                                         0.00627851486206055,
                                         3.26616386395983e-05,
                                         6.61493715893354e-09,
                                         5.86410834805435e-15,
                                         6.58636301868278e-27,
                                         9.39594822166825e-07,
                                         0.00576506145266626,
                                         0.125218369152281,
                                         0.151493985811901,
                                         0.0177891254425049,
                                         0.000181352226008961,
                                         7.82102756356675e-08,
                                         1.86045096582803e-13,
                                         1.15354355408585e-24,
                                         3.47433663431512e-08,
                                         0.00117681260729397,
                                         0.0685882248934051,
                                         0.176697369643381,
                                         0.0406608581542969,
                                         0.000812329235170502,
                                         7.45978278461961e-07,
                                         4.76166394905608e-12,
                                         1.62984611340069e-22,
                                         1.05385899308955e-09,
                                         0.000197056158933027,
                                         0.0308184230549735,
                                         0.169061261971989,
                                         0.0762391090393066,
                                         0.00298483690445787,
                                         5.83670678565794e-06,
                                         9.99720471810598e-11,
                                         1.88902860800388e-20,
                                         2.65202351373699e-11,
                                         2.73751790322128e-05,
                                         0.0114882933374682,
                                         0.134196868792465,
                                         0.118594169616699,
                                         0.0090989940711479,
                                         3.78873147100231e-05,
                                         1.74133641850441e-09,
                                         1.81641350873526e-18,
                                         5.57737776342085e-13,
                                         3.17820384804789e-06,
                                         0.00357897319543433,
                                         0.0890222332039382,
                                         0.154172420501709,
                                         0.0231805632543865,
                                         0.000205531125478932,
                                         2.53480530040713e-08,
                                         1.45965001885915e-16,
                                         9.84301076146514e-15,
                                         3.09636206989242e-07,
                                         0.000935635268363058,
                                         0.0495564052224867,
                                         0.168188095092773,
                                         0.0495564052224867,
                                         0.000935635268363061,
                                         3.09636206989242e-07,
                                         9.84301076146552e-15,
                                         1.45965001885909e-16,
                                         2.53480530040713e-08,
                                         0.000205531125478931,
                                         0.0231805632543865,
                                         0.154172420501709,
                                         0.0890222332039382,
                                         0.00357897319543434,
                                         3.17820384804789e-06,
                                         5.57737776342104e-13,
                                         1.81641350873518e-18,
                                         1.74133641850441e-09,
                                         3.7887314710023e-05,
                                         0.0090989940711479,
                                         0.118594169616699,
                                         0.134196868792465,
                                         0.0114882933374682,
                                         2.73751790322128e-05,
                                         2.65202351373707e-11,
                                         1.88902860800378e-20,
                                         9.99720471810598e-11,
                                         5.83670678565792e-06,
                                         0.00298483690445787,
                                         0.0762391090393066,
                                         0.169061261971988,
                                         0.0308184230549736,
                                         0.000197056158933027,
                                         1.05385899308958e-09,
                                         1.6298461134006e-22,
                                         4.76166394905608e-12,
                                         7.45978278461958e-07,
                                         0.000812329235170502,
                                         0.0406608581542969,
                                         0.176697369643381,
                                         0.0685882248934053,
                                         0.00117681260729397,
                                         3.47433663431521e-08,
                                         1.15354355408578e-24,
                                         1.86045096582803e-13,
                                         7.82102756356671e-08,
                                         0.000181352226008961,
                                         0.0177891254425049,
                                         0.151493985811901,
                                         0.125218369152282,
                                         0.00576506145266626,
                                         9.39594822166845e-07,
                                         6.58636301868239e-27,
                                         5.86410834805435e-15,
                                         6.6149371589335e-09,
                                         3.26616386395983e-05,
                                         0.00627851486206055,
                                         0.104781586828022,
                                         0.184421179328396,
                                         0.0227837308367076,
                                         2.04990439069075e-05,
                                         2.95973277227699e-29,
                                         1.45472503844197e-16,
                                         4.40334586991248e-10,
                                         4.62965062818397e-06,
                                         0.00174403190612793,
                                         0.0570387133745494,
                                         0.213770964548807,
                                         0.0708664882085683,
                                         0.000351983063363315,
                                         1.00801786678555e-31,
                                         2.7350718078302e-18,
                                         2.22151252960337e-11,
                                         4.97355996978191e-07,
                                         0.000367164611816406,
                                         0.0235322449877016,
                                         0.187799974889245,
                                         0.167057459813906,
                                         0.00458056255898025,
                                         2.44606969541121e-34,
                                         3.66388151531686e-20,
                                         7.98545383737713e-13,
                                         3.80689911426169e-08,
                                         5.50746917724609e-05,
                                         0.00691738376440855,
                                         0.117551239163953,
                                         0.280592270822043,
                                         0.0424718070915835,
                                         3.76867655804048e-37,
                                         3.11625903315548e-22,
                                         1.82250983948371e-14,
                                         1.85009836194356e-09,
                                         5.24520874023438e-06,
                                         0.00129104036621282,
                                         0.0467173645608344,
                                         0.299229892128702,
                                         0.250035746635281,
                                         2.77124883553117e-40,
                                         1.26500516375899e-24,
                                         1.98521148111113e-16,
                                         4.29126058486504e-11,
                                         2.38418579101562e-07,
                                         0.000115001734507255,
                                         0.00886127219300658,
                                         0.152300337757778,
                                         0.702538426187749,

                                         0.749074358377438,
                                         0.215079548165408,
                                         0.0214183398876873,
                                         0.000656738077601929,
                                         5.05051585212094e-06,
                                         6.87386745922928e-09,
                                         8.90230167248851e-13,
                                         3.22294084477208e-18,
                                         1.73859025383561e-26,
                                         3.47569013059904e-42,
                                         0.217847655287658,
                                         0.342340337663901,
                                         0.089950259383758,
                                         0.00571122110840332,
                                         8.23151122275664e-05,
                                         2.04128046531283e-07,
                                         4.95462616805674e-11,
                                         3.71433509756307e-16,
                                         5.28668754184973e-24,
                                         5.78440181012162e-39,
                                         0.0302376104119562,
                                         0.260066044445327,
                                         0.180295817550297,
                                         0.0237045856875385,
                                         0.000640309623305019,
                                         2.89314914977327e-06,
                                         1.31609153333851e-09,
                                         2.04303801679312e-14,
                                         7.67249587800528e-22,
                                         4.59454462748801e-36,
                                         0.00266478077387638,
                                         0.125437870200497,
                                         0.229450139594292,
                                         0.062467655404443,
                                         0.003162422746628,
                                         2.60350495761425e-05,
                                         2.21963078849408e-08,
                                         7.13495528775131e-13,
                                         7.06983252904966e-20,
                                         2.31710618796748e-33,
                                         0.00016732482293424,
                                         0.0431080680341106,
                                         0.208053903765248,
                                         0.117290519923239,
                                         0.0111284502854233,
                                         0.000166928643177897,
                                         2.6672286901067e-07,
                                         1.77537840263291e-11,
                                         4.64158659819116e-18,
                                         8.32595901041987e-31,
                                         7.96284300046433e-06,
                                         0.0112278690486586,
                                         0.142979015663124,
                                         0.16690890504352,
                                         0.029679620670501,
                                         0.000811170613386411,
                                         2.42911826122669e-06,
                                         3.34810831979259e-10,
                                         2.30957824385108e-16,
                                         2.26741748362378e-28,
                                         2.98243578896478e-07,
                                         0.00230160762788933,
                                         0.0773328306124886,
                                         0.186935280927981,
                                         0.0622984400733627,
                                         0.00310233522967492,
                                         1.74113415107975e-05,
                                         4.96939178794639e-09,
                                         9.04469804085287e-15,
                                         4.85986140279408e-26,
                                         9.01152564615731e-09,
                                         0.000380618163353195,
                                         0.0337426962682254,
                                         0.168899091497567,
                                         0.105492180727302,
                                         0.00957170955938893,
                                         0.000100679275282363,
                                         5.95019566903391e-08,
                                         2.85745721674129e-13,
                                         8.40311966876632e-24,
                                         2.23359728602582e-10,
                                         5.16329643846224e-05,
                                         0.0120774427597735,
                                         0.125182229326598,
                                         0.146535448341627,
                                         0.0242253259344137,
                                         0.000477559244948128,
                                         5.84438189180008e-07,
                                         7.40533534990135e-12,
                                         1.19189100506936e-21,
                                         4.59299188031672e-12,
                                         5.81097309192001e-06,
                                         0.00358636466701791,
                                         0.0769736894960278,
                                         0.16886878509477,
                                         0.0508667511115662,
                                         0.00187931113603111,
                                         4.7624472636415e-06,
                                         1.59218664975052e-10,
                                         1.40254495494095e-19,
                                         7.89304008036878e-14,
                                         5.46548233736203e-07,
                                         0.000890003537017591,
                                         0.0395548509024974,
                                         0.162634952232046,
                                         0.0892598590981001,
                                         0.00618056166186277,
                                         3.24324381698978e-05,
                                         2.86088929052903e-09,
                                         1.37928770359065e-17,
                                         1.13825127956041e-15,
                                         4.31373373169256e-08,
                                         0.000185342162800177,
                                         0.017056990540981,
                                         0.131438805186608,
                                         0.131438805186608,
                                         0.017056990540981,
                                         0.000185342162800177,
                                         4.31373373169252e-08,
                                         1.13825127956041e-15,
                                         1.37928770359065e-17,
                                         2.86088929052906e-09,
                                         3.24324381698978e-05,
                                         0.00618056166186277,
                                         0.0892598590981001,
                                         0.162634952232046,
                                         0.0395548509024974,
                                         0.000890003537017591,
                                         5.46548233736199e-07,
                                         7.89304008036878e-14,
                                         1.40254495494095e-19,
                                         1.59218664975054e-10,
                                         4.7624472636415e-06,
                                         0.00187931113603111,
                                         0.0508667511115662,
                                         0.16886878509477,
                                         0.0769736894960278,
                                         0.00358636466701791,
                                         5.81097309191997e-06,
                                         4.59299188031672e-12,
                                         1.19189100506936e-21,
                                         7.40533534990144e-12,
                                         5.84438189180008e-07,
                                         0.000477559244948128,
                                         0.0242253259344137,
                                         0.146535448341627,
                                         0.125182229326598,
                                         0.0120774427597735,
                                         5.16329643846221e-05,
                                         2.23359728602582e-10,
                                         8.40311966876632e-24,
                                         2.85745721674133e-13,
                                         5.95019566903391e-08,
                                         0.000100679275282363,
                                         0.00957170955938893,
                                         0.105492180727302,
                                         0.168899091497567,
                                         0.0337426962682254,
                                         0.000380618163353193,
                                         9.01152564615731e-09,
                                         4.85986140279408e-26,
                                         9.04469804085299e-15,
                                         4.96939178794639e-09,
                                         1.74113415107975e-05,
                                         0.00310233522967492,
                                         0.0622984400733627,
                                         0.186935280927981,
                                         0.0773328306124886,
                                         0.00230160762788932,
                                         2.98243578896478e-07,
                                         2.26741748362378e-28,
                                         2.30957824385112e-16,
                                         3.34810831979259e-10,
                                         2.42911826122669e-06,
                                         0.000811170613386412,
                                         0.029679620670501,
                                         0.16690890504352,
                                         0.142979015663124,
                                         0.0112278690486586,
                                         7.96284300046433e-06,
                                         8.32595901041987e-31,
                                         4.64158659819123e-18,
                                         1.77537840263291e-11,
                                         2.6672286901067e-07,
                                         0.000166928643177897,
                                         0.0111284502854233,
                                         0.117290519923239,
                                         0.208053903765248,
                                         0.0431080680341105,
                                         0.00016732482293424,
                                         2.31710618796748e-33,
                                         7.06983252904977e-20,
                                         7.13495528775131e-13,
                                         2.21963078849408e-08,
                                         2.60350495761425e-05,
                                         0.003162422746628,
                                         0.062467655404443,
                                         0.229450139594292,
                                         0.125437870200497,
                                         0.00266478077387638,
                                         4.59454462748801e-36,
                                         7.67249587800541e-22,
                                         2.04303801679312e-14,
                                         1.31609153333851e-09,
                                         2.89314914977327e-06,
                                         0.000640309623305019,
                                         0.0237045856875385,
                                         0.180295817550297,
                                         0.260066044445327,
                                         0.0302376104119562,
                                         5.78440181012162e-39,
                                         5.28668754184982e-24,
                                         3.71433509756307e-16,
                                         4.95462616805674e-11,
                                         2.04128046531283e-07,
                                         8.23151122275664e-05,
                                         0.00571122110840333,
                                         0.089950259383758,
                                         0.3423403376639,
                                         0.217847655287658,
                                         3.47569013059904e-42,
                                         1.73859025383565e-26,
                                         3.22294084477208e-18,
                                         8.90230167248851e-13,
                                         6.87386745922928e-09,
                                         5.05051585212094e-06,
                                         0.000656738077601929,
                                         0.0214183398876873,
                                         0.215079548165408,
                                         0.749074358377438,

                                         0.786000560127009,
                                         0.278380529277153,
                                         0.0412281480354097,
                                         0.00235621953303602,
                                         4.54552869494007e-05,
                                         2.38418579101562e-07,
                                         2.37914259286999e-10,
                                         2.41335991663342e-14,
                                         7.2771676015906e-20,
                                         3.4655438064518e-28,
                                         6.46866160905476e-44,
                                         0.190306772714013,
                                         0.36653294470642,
                                         0.14146579407994,
                                         0.0164101083440178,
                                         0.00057537922715161,
                                         5.24520874023438e-06,
                                         9.09695526154133e-09,
                                         1.67714943606642e-12,
                                         1.02647913039576e-17,
                                         1.27392128749963e-25,
                                         1.29308896500746e-40,
                                         0.0219913682791774,
                                         0.23033174296154,
                                         0.231673134636887,
                                         0.0545473633603364,
                                         0.00347608647288943,
                                         5.50746917724609e-05,
                                         1.66011534503715e-07,
                                         5.56273080696791e-11,
                                         6.91042072188738e-16,
                                         2.23501485986069e-23,
                                         1.23369838594915e-37,
                                         0.00161350271506911,
                                         0.0918996831165239,
                                         0.240890320783866,
                                         0.115121256108373,
                                         0.0133335688128897,
                                         0.000367164611816406,
                                         1.92353419329988e-06,
                                         1.17145003397411e-09,
                                         2.95378102198317e-14,
                                         2.48964644325595e-21,
                                         7.47324171676669e-35,
                                         8.43474617936944e-05,
                                         0.0261251686347395,
                                         0.178462878533766,
                                         0.173109995405286,
                                         0.0364407335104218,
                                         0.00174403190612793,
                                         1.5879851121469e-05,
                                         1.75769842398277e-08,
                                         8.99574158699642e-13,
                                         1.97596754753675e-19,
                                         3.22547686332636e-32,
                                         3.34182133272604e-06,
                                         0.00562876454428415,
                                         0.100204062364466,
                                         0.197286633114325,
                                         0.0754808365922873,
                                         0.00627851486206055,
                                         9.93577694604452e-05,
                                         1.99882074644401e-07,
                                         2.07636900816393e-11,
                                         1.18858659919208e-17,
                                         1.05508566794329e-29,
                                         1.04205251496893e-07,
                                         0.000954469888974595,
                                         0.0442810523757047,
                                         0.176957244820044,
                                         0.123049958065772,
                                         0.0177891254425049,
                                         0.000489274306673137,
                                         1.78895124148147e-06,
                                         3.77196170472999e-10,
                                         5.62700137810267e-16,
                                         2.71629345166672e-27,
                                         2.62132130201904e-09,
                                         0.000130567669863414,
                                         0.0157860985771789,
                                         0.128045200958947,
                                         0.161826822475468,
                                         0.0406608581542969,
                                         0.00194369115705955,
                                         1.29165768109215e-05,
                                         5.52782497083329e-09,
                                         2.14905477652112e-14,
                                         5.64143932075773e-25,
                                         5.40916509514789e-11,
                                         1.46517130551112e-05,
                                         0.00461648061897679,
                                         0.076004200286112,
                                         0.174581762158103,
                                         0.0762391090393066,
                                         0.0063340492644381,
                                         7.65025105369946e-05,
                                         6.64539226062367e-08,
                                         6.73282425479512e-13,
                                         9.61130779086577e-23,
                                         9.26028752059576e-13,
                                         1.36403467262239e-06,
                                         0.00112003469782052,
                                         0.037427958182902,
                                         0.156254125643482,
                                         0.118594169616699,
                                         0.0171245776486373,
                                         0.000375913738379401,
                                         6.62782775614279e-07,
                                         1.74997290950808e-11,
                                         1.3584989385685e-20,
                                         1.32487996157798e-14,
                                         1.06125620158233e-07,
                                         0.000227096145712516,
                                         0.01540325493955,
                                         0.116875084897565,
                                         0.154172420501709,
                                         0.0386915499367662,
                                         0.00154368446885179,
                                         5.52433022971659e-06,
                                         3.80122169251891e-10,
                                         1.60470035125815e-18,
                                         1.5906469626116e-16,
                                         6.92883598233131e-09,
                                         3.86396566302455e-05,
                                         0.00531954001341454,
                                         0.0733597080422656,
                                         0.168188095092773,
                                         0.0733597080422658,
                                         0.00531954001341452,
                                         3.86396566302453e-05,
                                         6.92883598233131e-09,
                                         1.59064696261169e-16,
                                         1.60470035125805e-18,
                                         3.80122169251891e-10,
                                         5.52433022971662e-06,
                                         0.0015436844688518,
                                         0.0386915499367661,
                                         0.154172420501709,
                                         0.116875084897565,
                                         0.0154032549395499,
                                         0.000227096145712515,
                                         1.06125620158233e-07,
                                         1.32487996157805e-14,
                                         1.35849893856841e-20,
                                         1.74997290950808e-11,
                                         6.62782775614283e-07,
                                         0.000375913738379402,
                                         0.0171245776486372,
                                         0.118594169616699,
                                         0.156254125643483,
                                         0.0374279581829019,
                                         0.00112003469782051,
                                         1.36403467262239e-06,
                                         9.26028752059618e-13,
                                         9.61130779086509e-23,
                                         6.73282425479512e-13,
                                         6.64539226062371e-08,
                                         7.65025105369948e-05,
                                         0.00633404926443809,
                                         0.0762391090393066,
                                         0.174581762158103,
                                         0.0760042002861119,
                                         0.00461648061897677,
                                         1.46517130551112e-05,
                                         5.40916509514811e-11,
                                         5.6414393207573e-25,
                                         2.14905477652112e-14,
                                         5.52782497083332e-09,
                                         1.29165768109215e-05,
                                         0.00194369115705954,
                                         0.0406608581542969,
                                         0.161826822475468,
                                         0.128045200958946,
                                         0.0157860985771789,
                                         0.000130567669863414,
                                         2.62132130201913e-09,
                                         2.7162934516665e-27,
                                         5.62700137810267e-16,
                                         3.77196170473002e-10,
                                         1.78895124148148e-06,
                                         0.000489274306673136,
                                         0.0177891254425049,
                                         0.123049958065772,
                                         0.176957244820044,
                                         0.0442810523757045,
                                         0.000954469888974595,
                                         1.04205251496896e-07,
                                         1.0550856679432e-29,
                                         1.18858659919208e-17,
                                         2.07636900816395e-11,
                                         1.99882074644401e-07,
                                         9.93577694604449e-05,
                                         0.00627851486206055,
                                         0.0754808365922874,
                                         0.197286633114325,
                                         0.100204062364466,
                                         0.00562876454428415,
                                         3.34182133272613e-06,
                                         3.22547686332606e-32,
                                         1.97596754753675e-19,
                                         8.99574158699649e-13,
                                         1.75769842398278e-08,
                                         1.5879851121469e-05,
                                         0.00174403190612793,
                                         0.0364407335104219,
                                         0.173109995405286,
                                         0.178462878533766,
                                         0.0261251686347395,
                                         8.43474617936961e-05,
                                         7.47324171676596e-35,
                                         2.48964644325595e-21,
                                         2.95378102198319e-14,
                                         1.17145003397412e-09,
                                         1.92353419329988e-06,
                                         0.000367164611816406,
                                         0.0133335688128897,
                                         0.115121256108373,
                                         0.240890320783866,
                                         0.0918996831165239,
                                         0.00161350271506914,
                                         1.23369838594903e-37,
                                         2.23501485986069e-23,
                                         6.91042072188743e-16,
                                         5.56273080696794e-11,
                                         1.66011534503714e-07,
                                         5.50746917724609e-05,
                                         0.00347608647288944,
                                         0.0545473633603363,
                                         0.231673134636887,
                                         0.23033174296154,
                                         0.0219913682791776,
                                         1.29308896500732e-40,
                                         1.27392128749963e-25,
                                         1.02647913039577e-17,
                                         1.67714943606643e-12,
                                         9.0969552615413e-09,
                                         5.24520874023438e-06,
                                         0.00057537922715161,
                                         0.0164101083440178,
                                         0.14146579407994,
                                         0.36653294470642,
                                         0.190306772714014,
                                         6.46866160905404e-44,
                                         3.4655438064518e-28,
                                         7.27716760159067e-20,
                                         2.41335991663343e-14,
                                         2.37914259286998e-10,
                                         2.38418579101562e-07,
                                         4.54552869494007e-05,
                                         0.00235621953303602,
                                         0.0412281480354097,
                                         0.278380529277153,
                                         0.786000560127009,

                                         0.815646818232584,
                                         0.339313587981015,
                                         0.0679558071472886,
                                         0.00619364807608269,
                                         0.000234453817857825,
                                         3.19652195535966e-06,
                                         1.25593811353481e-08,
                                         9.90063044724663e-12,
                                         8.33456118829525e-16,
                                         2.18471436595627e-21,
                                         9.45431467525102e-30,
                                         1.67398526060959e-45,
                                         0.166979609118736,
                                         0.375898589472944,
                                         0.194362036942447,
                                         0.0354258967048203,
                                         0.0023838572304802,
                                         5.46701093976218e-05,
                                         3.55419580933971e-07,
                                         4.71286803459697e-10,
                                         7.05267342157002e-14,
                                         3.69704859770006e-19,
                                         4.13053180222819e-27,
                                         3.95763463128584e-42,
                                         0.0163151605572911,
                                         0.198749826791276,
                                         0.265315685829513,
                                         0.0967078653529377,
                                         0.0115683066646272,
                                         0.000446260874738808,
                                         4.80044036794766e-06,
                                         1.07071608361282e-08,
                                         2.84833736462837e-12,
                                         2.98594822239798e-17,
                                         8.61288216641859e-25,
                                         4.46566750167467e-39,
                                         0.00101213561217034,
                                         0.0667209564778071,
                                         0.229950232833999,
                                         0.167618590434879,
                                         0.0356433721973454,
                                         0.0023128481691419,
                                         4.11661497679454e-05,
                                         1.54448187292749e-07,
                                         7.303794104449e-11,
                                         1.53118909583813e-15,
                                         1.14027731090173e-22,
                                         3.19931138823794e-36,
                                         4.47374193377468e-05,
                                         0.0159588883670667,
                                         0.142000426617263,
                                         0.206998615125538,
                                         0.0782478921186255,
                                         0.00854063778916684,
                                         0.000251526813528988,
                                         1.58736270825763e-06,
                                         1.3344136793673e-09,
                                         5.59448677344491e-14,
                                         1.07561618567606e-20,
                                         1.63309522383558e-33,
                                         1.49869080237596e-06,
                                         0.00289302309952398,
                                         0.0664590816520383,
                                         0.193741022199709,
                                         0.13018933233537,
                                         0.023902447603527,
                                         0.0011647620694516,
                                         1.2364553047639e-05,
                                         1.84774203999059e-08,
                                         1.54917530827519e-12,
                                         7.68976496317888e-19,
                                         6.31793853594517e-31,
                                         3.95137571059063e-08,
                                         0.00041275879510084,
                                         0.0244801566560865,
                                         0.142715421295927,
                                         0.170479955218129,
                                         0.0526489381112072,
                                         0.00424507453502974,
                                         7.58011581689251e-05,
                                         2.01366566473924e-07,
                                         3.37626130539113e-11,
                                         4.32677162685898e-17,
                                         1.92368719744116e-28,
                                         8.40444194159506e-10,
                                         4.75078134414282e-05,
                                         0.00727441914696132,
                                         0.0848094980142166,
                                         0.180092454487326,
                                         0.0935537346700748,
                                         0.0124812387929083,
                                         0.000374884541120878,
                                         1.7703439920239e-06,
                                         5.93602618719612e-10,
                                         1.96398945681505e-15,
                                         4.72517376952535e-26,
                                         1.46638755554847e-11,
                                         4.4855228991582e-06,
                                         0.00177321656253024,
                                         0.0413425643259375,
                                         0.156061954284541,
                                         0.136367843183682,
                                         0.0301029733374896,
                                         0.00152089261907623,
                                         1.27675417718317e-05,
                                         8.56120495915532e-09,
                                         7.31296778718466e-14,
                                         9.52095348981902e-24,
                                         2.12262319393384e-13,
                                         3.51354376968367e-07,
                                         0.000358599344400946,
                                         0.0167199349380411,
                                         0.112197406776122,
                                         0.164910029090993,
                                         0.060234505567591,
                                         0.00511898464260955,
                                         7.63908192379267e-05,
                                         1.02437326359224e-07,
                                         2.25908422499044e-12,
                                         1.59157590967137e-21,
                                         2.56776260987285e-15,
                                         2.30004050425134e-08,
                                         6.06059173767072e-05,
                                         0.00565105561650656,
                                         0.0674103253674675,
                                         0.166663305449974,
                                         0.100725430383478,
                                         0.0143988208108108,
                                         0.000381973183064552,
                                         1.02432884169062e-06,
                                         5.83215438111039e-11,
                                         2.2234742097877e-19,
                                         2.60664593604996e-17,
                                         1.26348708984842e-09,
                                         8.59539374866025e-06,
                                         0.00160276477297876,
                                         0.0339871881780296,
                                         0.141344242357566,
                                         0.141344242357566,
                                         0.0339871881780297,
                                         0.00160276477297877,
                                         8.5953937486603e-06,
                                         1.26348708984844e-09,
                                         2.60664593604979e-17,
                                         2.22347420978786e-19,
                                         5.83215438111031e-11,
                                         1.02432884169061e-06,
                                         0.000381973183064551,
                                         0.0143988208108108,
                                         0.100725430383478,
                                         0.166663305449974,
                                         0.0674103253674676,
                                         0.00565105561650658,
                                         6.06059173767075e-05,
                                         2.30004050425137e-08,
                                         2.56776260987269e-15,
                                         1.5915759096715e-21,
                                         2.2590842249904e-12,
                                         1.02437326359223e-07,
                                         7.63908192379264e-05,
                                         0.00511898464260954,
                                         0.060234505567591,
                                         0.164910029090993,
                                         0.112197406776122,
                                         0.0167199349380411,
                                         0.000358599344400948,
                                         3.51354376968371e-07,
                                         2.12262319393372e-13,
                                         9.52095348981983e-24,
                                         7.31296778718454e-14,
                                         8.56120495915526e-09,
                                         1.27675417718317e-05,
                                         0.00152089261907623,
                                         0.0301029733374896,
                                         0.136367843183682,
                                         0.156061954284541,
                                         0.0413425643259376,
                                         0.00177321656253024,
                                         4.48552289915824e-06,
                                         1.4663875555484e-11,
                                         4.72517376952577e-26,
                                         1.96398945681502e-15,
                                         5.93602618719608e-10,
                                         1.77034399202389e-06,
                                         0.000374884541120877,
                                         0.0124812387929083,
                                         0.0935537346700748,
                                         0.180092454487327,
                                         0.0848094980142167,
                                         0.00727441914696134,
                                         4.75078134414286e-05,
                                         8.40444194159471e-10,
                                         1.92368719744134e-28,
                                         4.3267716268589e-17,
                                         3.3762613053911e-11,
                                         2.01366566473923e-07,
                                         7.58011581689249e-05,
                                         0.00424507453502974,
                                         0.0526489381112072,
                                         0.170479955218129,
                                         0.142715421295927,
                                         0.0244801566560865,
                                         0.000412758795100843,
                                         3.95137571059049e-08,
                                         6.31793853594582e-31,
                                         7.68976496317873e-19,
                                         1.54917530827517e-12,
                                         1.84774203999058e-08,
                                         1.2364553047639e-05,
                                         0.00116476206945161,
                                         0.023902447603527,
                                         0.13018933233537,
                                         0.193741022199709,
                                         0.0664590816520384,
                                         0.00289302309952399,
                                         1.49869080237591e-06,
                                         1.63309522383575e-33,
                                         1.07561618567604e-20,
                                         5.59448677344487e-14,
                                         1.33441367936729e-09,
                                         1.58736270825762e-06,
                                         0.000251526813528988,
                                         0.00854063778916684,
                                         0.0782478921186255,
                                         0.206998615125539,
                                         0.142000426617263,
                                         0.0159588883670668,
                                         4.47374193377457e-05,
                                         3.19931138823831e-36,
                                         1.1402773109017e-22,
                                         1.53118909583812e-15,
                                         7.30379410444896e-11,
                                         1.54448187292749e-07,
                                         4.11661497679454e-05,
                                         0.0023128481691419,
                                         0.0356433721973454,
                                         0.167618590434879,
                                         0.229950232834,
                                         0.0667209564778073,
                                         0.00101213561217032,
                                         4.46566750167521e-39,
                                         8.61288216641839e-25,
                                         2.98594822239795e-17,
                                         2.84833736462836e-12,
                                         1.07071608361282e-08,
                                         4.80044036794766e-06,
                                         0.000446260874738808,
                                         0.0115683066646272,
                                         0.0967078653529377,
                                         0.265315685829513,
                                         0.198749826791276,
                                         0.0163151605572909,
                                         3.95763463128634e-42,
                                         4.13053180222809e-27,
                                         3.69704859770003e-19,
                                         7.05267342156998e-14,
                                         4.71286803459696e-10,
                                         3.55419580933971e-07,
                                         5.46701093976218e-05,
                                         0.0023838572304802,
                                         0.0354258967048203,
                                         0.194362036942447,
                                         0.375898589472945,
                                         0.166979609118735,
                                         1.67398526060982e-45,
                                         9.45431467525078e-30,
                                         2.18471436595625e-21,
                                         8.3345611882952e-16,
                                         9.90063044724659e-12,
                                         1.25593811353481e-08,
                                         3.19652195535966e-06,
                                         0.000234453817857825,
                                         0.00619364807608269,
                                         0.0679558071472886,
                                         0.339313587981015,
                                         0.815646818232584,

                                         0.839727343970775,
                                         0.396285865143724,
                                         0.100394081881479,
                                         0.0131093747533607,
                                         0.000826988118722673,
                                         2.28460973026671e-05,
                                         2.38418579101562e-07,
                                         7.48940613332857e-10,
                                         4.91436091781927e-13,
                                         3.57633990278507e-17,
                                         8.40062944102914e-23,
                                         3.37228573063903e-31,
                                         5.72845576097404e-47,
                                         0.147265783378564,
                                         0.374635780182336,
                                         0.243258108647184,
                                         0.0628055684648362,
                                         0.00692718601665809,
                                         0.000314340215521241,
                                         5.24520874023438e-06,
                                         2.63454013585585e-08,
                                         2.83958471867677e-11,
                                         3.61300013512529e-15,
                                         1.67802325920415e-20,
                                         1.72650983077967e-28,
                                         1.58095449067352e-43,
                                         0.0123262799447002,
                                         0.169034962733991,
                                         0.281315173298249,
                                         0.143608762973196,
                                         0.0276937041306619,
                                         0.00206421290719502,
                                         5.50746917724609e-05,
                                         4.42312155455249e-07,
                                         7.83085586476205e-10,
                                         1.74206271416843e-13,
                                         1.59974493151e-18,
                                         4.21871672316164e-26,
                                         2.08241743750626e-40,
                                         0.000655060848829932,
                                         0.0484242912809032,
                                         0.206556292984254,
                                         0.20848901309073,
                                         0.0702950409096324,
                                         0.00860653766829389,
                                         0.000367164611816406,
                                         4.71489876127552e-06,
                                         1.37114401498026e-08,
                                         5.33309150219582e-12,
                                         9.68329028456369e-17,
                                         6.5450220698187e-24,
                                         1.741548834214e-37,
                                         2.48036805334914e-05,
                                         0.00988404861954744,
                                         0.108060898175909,
                                         0.215660334520863,
                                         0.127131507458723,
                                         0.0255674448086348,
                                         0.00174403190612793,
                                         3.5809704558459e-05,
                                         1.71057367020602e-07,
                                         1.16326631796167e-10,
                                         4.17618787738345e-15,
                                         7.23480341360679e-22,
                                         1.03773960240592e-34,
                                         7.11802494550281e-07,
                                         0.00152902773774693,
                                         0.0428457358780795,
                                         0.169069888655404,
                                         0.174257145840689,
                                         0.0575645443707352,
                                         0.00627851486206055,
                                         0.000206128477307827,
                                         1.61737000516842e-06,
                                         1.92303931363322e-09,
                                         1.36504194510315e-13,
                                         6.06109715462936e-20,
                                         4.68651504548518e-32,
                                         1.60767423795154e-08,
                                         0.000186161995272818,
                                         0.0133703202514594,
                                         0.104317562084802,
                                         0.18798498689796,
                                         0.102004184038945,
                                         0.0177891254425049,
                                         0.00093383559188384,
                                         1.2035722242624e-05,
                                         2.50202901931638e-08,
                                         3.5116169374821e-12,
                                         3.99641818358917e-18,
                                         1.66573845064424e-29,
                                         2.92927989585795e-10,
                                         1.82848313009385e-05,
                                         0.00336589283799587,
                                         0.0519245645304901,
                                         0.16359876019319,
                                         0.145815997265134,
                                         0.0406608581542969,
                                         0.00341292796134742,
                                         7.22535517179959e-05,
                                         2.62615765144658e-07,
                                         7.28773454618883e-11,
                                         2.12576317220234e-16,
                                         4.77625909400299e-27,
                                         4.37827482542588e-12,
                                         1.47322886704176e-06,
                                         0.000695085276976223,
                                         0.0212015506814283,
                                         0.116792829266104,
                                         0.170990384194246,
                                         0.0762391090393066,
                                         0.0102320616905763,
                                         0.000355816082076117,
                                         2.26114583992919e-06,
                                         1.24067286766775e-09,
                                         9.2755177486774e-15,
                                         1.12343583779776e-24,
                                         5.42911974928045e-14,
                                         9.84767566012028e-08,
                                         0.000119085846556343,
                                         0.00718202055880663,
                                         0.0691729989210293,
                                         0.166349867003868,
                                         0.118594169616699,
                                         0.0254497448439455,
                                         0.00145370471997168,
                                         1.61517881383807e-05,
                                         1.75229119393184e-08,
                                         3.35772942741494e-13,
                                         2.19226436364224e-22,
                                         5.62618034335624e-16,
                                         5.50117016290965e-09,
                                         1.70506144088105e-05,
                                         0.00203321585482613,
                                         0.0342385099292061,
                                         0.135248062998424,
                                         0.154172420501709,
                                         0.0529007149071425,
                                         0.0049634614574491,
                                         9.6420742240045e-05,
                                         2.06829793434167e-07,
                                         1.01580666742088e-11,
                                         3.57515932547899e-20,
                                         4.89263801942044e-18,
                                         2.57882356344114e-10,
                                         2.04863753974266e-06,
                                         0.000483020474108305,
                                         0.0142212678331808,
                                         0.0922751709496792,
                                         0.168188095092773,
                                         0.0922751709496792,
                                         0.0142212678331808,
                                         0.000483020474108303,
                                         2.04863753974267e-06,
                                         2.5788235634411e-10,
                                         4.89263801942006e-18,
                                         3.5751593254793e-20,
                                         1.01580666742089e-11,
                                         2.06829793434166e-07,
                                         9.64207422400453e-05,
                                         0.0049634614574491,
                                         0.0529007149071425,
                                         0.154172420501709,
                                         0.135248062998424,
                                         0.0342385099292061,
                                         0.00203321585482612,
                                         1.70506144088106e-05,
                                         5.50117016290958e-09,
                                         5.62618034335584e-16,
                                         2.19226436364244e-22,
                                         3.35772942741499e-13,
                                         1.75229119393182e-08,
                                         1.61517881383807e-05,
                                         0.00145370471997168,
                                         0.0254497448439455,
                                         0.118594169616699,
                                         0.166349867003868,
                                         0.0691729989210293,
                                         0.00718202055880661,
                                         0.000119085846556344,
                                         9.84767566012016e-08,
                                         5.4291197492801e-14,
                                         1.12343583779787e-24,
                                         9.27551774867757e-15,
                                         1.24067286766774e-09,
                                         2.2611458399292e-06,
                                         0.000355816082076117,
                                         0.0102320616905763,
                                         0.0762391090393066,
                                         0.170990384194246,
                                         0.116792829266104,
                                         0.0212015506814282,
                                         0.000695085276976226,
                                         1.47322886704175e-06,
                                         4.37827482542564e-12,
                                         4.7762590940035e-27,
                                         2.12576317220239e-16,
                                         7.28773454618877e-11,
                                         2.62615765144659e-07,
                                         7.22535517179959e-05,
                                         0.00341292796134742,
                                         0.0406608581542969,
                                         0.145815997265134,
                                         0.16359876019319,
                                         0.05192456453049,
                                         0.00336589283799588,
                                         1.82848313009383e-05,
                                         2.92927989585781e-10,
                                         1.66573845064442e-29,
                                         3.99641818358926e-18,
                                         3.51161693748207e-12,
                                         2.50202901931639e-08,
                                         1.2035722242624e-05,
                                         0.00093383559188384,
                                         0.0177891254425049,
                                         0.102004184038945,
                                         0.18798498689796,
                                         0.104317562084802,
                                         0.0133703202514594,
                                         0.000186161995272816,
                                         1.60767423795147e-08,
                                         4.68651504548574e-32,
                                         6.0610971546295e-20,
                                         1.36504194510313e-13,
                                         1.92303931363323e-09,
                                         1.61737000516842e-06,
                                         0.000206128477307827,
                                         0.00627851486206055,
                                         0.0575645443707352,
                                         0.174257145840689,
                                         0.169069888655403,
                                         0.0428457358780796,
                                         0.00152902773774692,
                                         7.11802494550256e-07,
                                         1.03773960240605e-34,
                                         7.23480341360697e-22,
                                         4.17618787738341e-15,
                                         1.16326631796168e-10,
                                         1.71057367020602e-07,
                                         3.5809704558459e-05,
                                         0.00174403190612793,
                                         0.0255674448086348,
                                         0.127131507458723,
                                         0.215660334520863,
                                         0.108060898175909,
                                         0.00988404861954739,
                                         2.48036805334907e-05,
                                         1.74154883421423e-37,
                                         6.54502206981887e-24,
                                         9.68329028456358e-17,
                                         5.33309150219585e-12,
                                         1.37114401498026e-08,
                                         4.71489876127552e-06,
                                         0.000367164611816406,
                                         0.00860653766829389,
                                         0.0702950409096324,
                                         0.208489013090729,
                                         0.206556292984254,
                                         0.048424291280903,
                                         0.000655060848829919,
                                         2.08241743750655e-40,
                                         4.21871672316175e-26,
                                         1.59974493150998e-18,
                                         1.74206271416844e-13,
                                         7.83085586476205e-10,
                                         4.42312155455249e-07,
                                         5.50746917724609e-05,
                                         0.00206421290719502,
                                         0.0276937041306619,
                                         0.143608762973196,
                                         0.281315173298249,
                                         0.16903496273399,
                                         0.0123262799447,
                                         1.58095449067376e-43,
                                         1.72650983077971e-28,
                                         1.67802325920413e-20,
                                         3.61300013512532e-15,
                                         2.83958471867677e-11,
                                         2.63454013585585e-08,
                                         5.24520874023438e-06,
                                         0.000314340215521241,
                                         0.00692718601665809,
                                         0.0628055684648362,
                                         0.243258108647184,
                                         0.374635780182335,
                                         0.147265783378563,
                                         5.72845576097492e-47,
                                         3.37228573063913e-31,
                                         8.40062944102904e-23,
                                         3.57633990278509e-17,
                                         4.91436091781927e-13,
                                         7.48940613332857e-10,
                                         2.38418579101562e-07,
                                         2.28460973026671e-05,
                                         0.000826988118722673,
                                         0.0131093747533607,
                                         0.100394081881479,
                                         0.396285865143724,
                                         0.839727343970775,

                                         0.859505676082842,
                                         0.448590044151318,
                                         0.136974863517057,
                                         0.0237427304226535,
                                         0.00222836298520379,
                                         0.000105569759651904,
                                         2.27870782995519e-06,
                                         1.92654120207587e-08,
                                         5.06990572054473e-11,
                                         2.87580188081049e-14,
                                         1.86367539780146e-18,
                                         4.01078720918601e-24,
                                         1.51624887173708e-32,
                                         2.49159721515891e-48,
                                         0.130576090390318,
                                         0.366242478456612,
                                         0.284982105671005,
                                         0.096805625925979,
                                         0.0156835276644638,
                                         0.00119882520672177,
                                         4.03541429440046e-05,
                                         5.2653128367729e-07,
                                         2.16087135206502e-09,
                                         1.97763667136756e-12,
                                         2.21230855095097e-16,
                                         9.33035715249025e-22,
                                         8.98869757469688e-30,
                                         7.93794867183767e-45,
                                         0.00946771533329166,
                                         0.142710003278462,
                                         0.282983376361321,
                                         0.188381023203597,
                                         0.0526827201927258,
                                         0.00649738822875737,
                                         0.000341078409938551,
                                         6.8681010491222e-06,
                                         4.39566471470015e-08,
                                         6.49083647612259e-11,
                                         1.25339437692552e-14,
                                         1.03593744994547e-19,
                                         2.54325358380937e-27,
                                         1.20699467521618e-41,
                                         0.000435859146099757,
                                         0.0353068977456386,
                                         0.178411850914877,
                                         0.232751853127615,
                                         0.112360087715441,
                                         0.0223584251725778,
                                         0.00183037372651579,
                                         5.6881182270685e-05,
                                         5.67727128551385e-07,
                                         1.35261522462131e-09,
                                         4.50867808173379e-13,
                                         7.30278038722574e-18,
                                         4.56879848739383e-25,
                                         1.16525738453742e-38,
                                         1.42965749225365e-05,
                                         0.00622371321102757,
                                         0.0801440605143133,
                                         0.204896250969505,
                                         0.170742177918707,
                                         0.0548186672091511,
                                         0.00699858409468825,
                                         0.000335649049464501,
                                         5.22443830015982e-06,
                                         2.00831921468513e-08,
                                         1.15556820131755e-11,
                                         3.66798724074847e-16,
                                         5.84789020756707e-23,
                                         8.01536386258499e-36,
                                         3.55407621374862e-07,
                                         0.000831473621201094,
                                         0.0272852462021841,
                                         0.136704801272239,
                                         0.196642983685056,
                                         0.101864914621079,
                                         0.0202810057784066,
                                         0.00150110516684474,
                                         3.64374900124476e-05,
                                         2.25995663897066e-07,
                                         2.24466134782318e-10,
                                         1.39629210752259e-14,
                                         5.6729023549314e-21,
                                         4.17862609181577e-33,
                                         6.95371102631926e-09,
                                         8.74263906888211e-05,
                                         0.00731104709959452,
                                         0.0717841717639266,
                                         0.178242499556336,
                                         0.148975876338138,
                                         0.0462555620438985,
                                         0.00528362502191374,
                                         0.00020001035089329,
                                         2.00153247034156e-06,
                                         3.43163622928067e-09,
                                         4.18330876321838e-13,
                                         4.33118342658418e-19,
                                         1.71450577168512e-30,
                                         1.09756642356344e-10,
                                         7.41584928174125e-06,
                                         0.00158035828749666,
                                         0.0304087026282942,
                                         0.130337187134093,
                                         0.175764654069977,
                                         0.0851064955020177,
                                         0.0150029661110645,
                                         0.000885688003169851,
                                         1.43004414753475e-05,
                                         4.23229500479032e-08,
                                         1.01108537640041e-11,
                                         2.66767007166085e-17,
                                         5.67503654862284e-28,
                                         1.42109894186922e-12,
                                         5.16010536326399e-07,
                                         0.000280227572160947,
                                         0.0105668713182024,
                                         0.0781816359380806,
                                         0.170108677464086,
                                         0.128451973858182,
                                         0.0349463319388922,
                                         0.00321727646386595,
                                         8.38138092667505e-05,
                                         4.28183520769184e-07,
                                         2.00463388597892e-10,
                                         1.34783579477612e-15,
                                         1.54091152581004e-25,
                                         1.52651850583868e-14,
                                         2.9787941924761e-08,
                                         4.12240279842991e-05,
                                         0.00304634580042982,
                                         0.038906791497747,
                                         0.136585830165692,
                                         0.160843350436661,
                                         0.0675321103433401,
                                         0.00969572425709009,
                                         0.000407535990188252,
                                         3.59391846496861e-06,
                                         3.29736147641151e-09,
                                         5.64970939290989e-14,
                                         3.47113095544011e-23,
                                         1.37036941503242e-16,
                                         1.43707771080341e-09,
                                         5.06813074936121e-06,
                                         0.000733955631839277,
                                         0.0161809459617327,
                                         0.0916521487397298,
                                         0.16831517228408,
                                         0.109062857224651,
                                         0.0244191170061854,
                                         0.00165605281311313,
                                         2.52095049509186e-05,
                                         4.53268854268745e-08,
                                         1.9791242442786e-12,
                                         6.5346468133401e-21,
                                         1.03232976459444e-18,
                                         5.81788613399466e-11,
                                         5.22866003565876e-07,
                                         0.000148390337709308,
                                         0.00564712766551182,
                                         0.0516089311195744,
                                         0.14780483107865,
                                         0.147804831078651,
                                         0.0516089311195744,
                                         0.00564712766551183,
                                         0.000148390337709308,
                                         5.22866003565876e-07,
                                         5.81788613399457e-11,
                                         1.03232976459444e-18,
                                         6.5346468133401e-21,
                                         1.97912424427864e-12,
                                         4.53268854268745e-08,
                                         2.52095049509185e-05,
                                         0.00165605281311312,
                                         0.0244191170061854,
                                         0.109062857224651,
                                         0.16831517228408,
                                         0.0916521487397298,
                                         0.0161809459617327,
                                         0.000733955631839279,
                                         5.06813074936121e-06,
                                         1.43707771080339e-09,
                                         1.37036941503242e-16,
                                         3.47113095544011e-23,
                                         5.64970939291e-14,
                                         3.29736147641151e-09,
                                         3.59391846496859e-06,
                                         0.000407535990188251,
                                         0.00969572425709009,
                                         0.06753211034334,
                                         0.160843350436661,
                                         0.136585830165692,
                                         0.0389067914977471,
                                         0.00304634580042983,
                                         4.12240279842991e-05,
                                         2.97879419247606e-08,
                                         1.52651850583868e-14,
                                         1.54091152581004e-25,
                                         1.34783579477614e-15,
                                         2.00463388597892e-10,
                                         4.28183520769182e-07,
                                         8.38138092667503e-05,
                                         0.00321727646386595,
                                         0.0349463319388921,
                                         0.128451973858182,
                                         0.170108677464086,
                                         0.0781816359380808,
                                         0.0105668713182024,
                                         0.000280227572160947,
                                         5.16010536326392e-07,
                                         1.42109894186922e-12,
                                         5.67503654862284e-28,
                                         2.66767007166092e-17,
                                         1.01108537640041e-11,
                                         4.2322950047903e-08,
                                         1.43004414753475e-05,
                                         0.000885688003169851,
                                         0.0150029661110644,
                                         0.0851064955020178,
                                         0.175764654069977,
                                         0.130337187134093,
                                         0.0304087026282943,
                                         0.00158035828749666,
                                         7.41584928174117e-06,
                                         1.09756642356344e-10,
                                         1.71450577168512e-30,
                                         4.33118342658429e-19,
                                         4.18330876321838e-13,
                                         3.43163622928065e-09,
                                         2.00153247034156e-06,
                                         0.00020001035089329,
                                         0.00528362502191373,
                                         0.0462555620438985,
                                         0.148975876338138,
                                         0.178242499556337,
                                         0.0717841717639268,
                                         0.00731104709959452,
                                         8.74263906888203e-05,
                                         6.95371102631926e-09,
                                         4.17862609181577e-33,
                                         5.67290235493155e-21,
                                         1.39629210752259e-14,
                                         2.24466134782317e-10,
                                         2.25995663897065e-07,
                                         3.64374900124476e-05,
                                         0.00150110516684474,
                                         0.0202810057784066,
                                         0.101864914621079,
                                         0.196642983685056,
                                         0.136704801272239,
                                         0.0272852462021841,
                                         0.000831473621201088,
                                         3.55407621374862e-07,
                                         8.01536386258499e-36,
                                         5.84789020756723e-23,
                                         3.66798724074847e-16,
                                         1.15556820131754e-11,
                                         2.00831921468513e-08,
                                         5.22443830015982e-06,
                                         0.0003356490494645,
                                         0.00699858409468826,
                                         0.0548186672091511,
                                         0.170742177918708,
                                         0.204896250969506,
                                         0.0801440605143133,
                                         0.00622371321102753,
                                         1.42965749225365e-05,
                                         1.16525738453742e-38,
                                         4.56879848739397e-25,
                                         7.30278038722574e-18,
                                         4.50867808173376e-13,
                                         1.3526152246213e-09,
                                         5.67727128551385e-07,
                                         5.68811822706849e-05,
                                         0.00183037372651579,
                                         0.0223584251725778,
                                         0.112360087715441,
                                         0.232751853127615,
                                         0.178411850914877,
                                         0.0353068977456385,
                                         0.000435859146099757,
                                         1.20699467521618e-41,
                                         2.54325358380945e-27,
                                         1.03593744994547e-19,
                                         1.25339437692551e-14,
                                         6.49083647612256e-11,
                                         4.39566471470015e-08,
                                         6.86810104912218e-06,
                                         0.000341078409938552,
                                         0.00649738822875737,
                                         0.0526827201927258,
                                         0.188381023203597,
                                         0.282983376361321,
                                         0.142710003278461,
                                         0.00946771533329166,
                                         7.93794867183767e-45,
                                         8.98869757469717e-30,
                                         9.33035715249025e-22,
                                         2.21230855095095e-16,
                                         1.97763667136756e-12,
                                         2.16087135206502e-09,
                                         5.26531283677289e-07,
                                         4.03541429440046e-05,
                                         0.00119882520672177,
                                         0.0156835276644638,
                                         0.096805625925979,
                                         0.284982105671005,
                                         0.366242478456611,
                                         0.130576090390318,
                                         2.49159721515891e-48,
                                         1.51624887173713e-32,
                                         4.01078720918601e-24,
                                         1.86367539780144e-18,
                                         2.87580188081048e-14,
                                         5.06990572054473e-11,
                                         1.92654120207587e-08,
                                         2.27870782995519e-06,
                                         0.000105569759651904,
                                         0.00222836298520379,
                                         0.0237427304226535,
                                         0.136974863517057,
                                         0.448590044151318,
                                         0.859505676082842,

                                         0.875920110832538,
                                         0.496066838574306,
                                         0.176137317932882,
                                         0.0383243371408101,
                                         0.0049322709328893,
                                         0.000356581161269309,
                                         1.34533436495141e-05,
                                         2.38418579101562e-07,
                                         1.702346544168e-09,
                                         3.88464485388739e-12,
                                         1.95893606295382e-15,
                                         1.15576105825348e-19,
                                         2.3168861465421e-25,
                                         8.34218288813733e-34,
                                         1.33439022884601e-49,
                                         0.11639233383502,
                                         0.353365188962657,
                                         0.318255914352196,
                                         0.134742972534349,
                                         0.0296337474498083,
                                         0.00340906513961946,
                                         0.000196825333358833,
                                         5.24520874023438e-06,
                                         5.63173972392607e-08,
                                         1.96661694726066e-10,
                                         1.57806894144095e-13,
                                         1.59104310962758e-17,
                                         6.20618832856581e-23,
                                         5.66815216651439e-31,
                                         4.86035885816904e-46,
                                         0.00738160621999107,
                                         0.12013621627165,
                                         0.274453141083536,
                                         0.226101901226978,
                                         0.0849753269530248,
                                         0.0155553154486233,
                                         0.00137435324981885,
                                         5.50746917724609e-05,
                                         8.89208888755195e-07,
                                         4.75176388629855e-09,
                                         6.06733956853454e-12,
                                         1.04535176058271e-15,
                                         7.93435861954543e-21,
                                         1.8381032924232e-28,
                                         8.44929429374926e-43,
                                         0.000297232854627558,
                                         0.0259324516821632,
                                         0.150272450972214,
                                         0.24089164997842,
                                         0.154710060178852,
                                         0.045065250292885,
                                         0.0060930563366082,
                                         0.000367164611816406,
                                         8.91424276724808e-06,
                                         7.28969519711139e-08,
                                         1.48111940770646e-10,
                                         4.36076220325412e-14,
                                         6.44047862999369e-19,
                                         3.78458067177613e-26,
                                         9.32592632226187e-40,
                                         8.52761569181681e-06,
                                         0.00398839418013679,
                                         0.0586239939551478,
                                         0.182862287022031,
                                         0.200691571893746,
                                         0.093022845685835,
                                         0.0192467268238767,
                                         0.00174403190612793,
                                         6.36722193702784e-05,
                                         7.96798954105444e-07,
                                         2.5761236964118e-09,
                                         1.29612600214064e-12,
                                         3.72485464393713e-17,
                                         5.55201319787536e-24,
                                         7.33412641996939e-37,
                                         1.85424586335385e-07,
                                         0.00046490205027975,
                                         0.0173332630124974,
                                         0.10520477188507,
                                         0.197309776735621,
                                         0.145527927885653,
                                         0.0460773466052261,
                                         0.00627851486206055,
                                         0.00034468660762825,
                                         6.60080696307157e-06,
                                         3.39587928212495e-08,
                                         2.91971817211571e-11,
                                         1.63271133277329e-15,
                                         6.17294068258223e-22,
                                         4.37133101730756e-34,
                                         3.17323464915656e-09,
                                         4.26500966173e-05,
                                         0.0040334847691268,
                                         0.0476367255324917,
                                         0.152673353831818,
                                         0.179183592867911,
                                         0.0868186866922535,
                                         0.0177891254425049,
                                         0.00146856780065534,
                                         4.30368502855659e-05,
                                         3.52316468861213e-07,
                                         5.1764236498326e-10,
                                         5.63254480236473e-14,
                                         5.40168043816632e-20,
                                         2.05056796125679e-31,
                                         4.38087970896791e-11,
                                         3.15647859669123e-06,
                                         0.000757189856357771,
                                         0.0174009370123854,
                                         0.0953020315330627,
                                         0.177981313299428,
                                         0.131966358719151,
                                         0.0406608581542969,
                                         0.00504763409777747,
                                         0.000226364437049891,
                                         2.94874975172397e-06,
                                         7.40360029764128e-09,
                                         1.56756000456399e-12,
                                         3.81320315485705e-18,
                                         7.75994980772462e-29,
                                         4.96134923324252e-13,
                                         1.91630665462821e-07,
                                         0.000116602666676159,
                                         0.0052141401244351,
                                         0.0488000649942259,
                                         0.145020668578046,
                                         0.164547991138499,
                                         0.0762391090393066,
                                         0.0142318401942567,
                                         0.00097668663820142,
                                         2.02452069023278e-05,
                                         8.68631241146836e-08,
                                         3.57868129109922e-11,
                                         2.20815893399861e-16,
                                         2.40892337756928e-26,
                                         4.66146601889533e-15,
                                         9.65186904502071e-09,
                                         1.48969181763922e-05,
                                         0.0012962149185602,
                                         0.0207311262531281,
                                         0.0980324040805124,
                                         0.170218276887769,
                                         0.118594169616699,
                                         0.0332903610855406,
                                         0.00349612175335311,
                                         0.000115316320341916,
                                         8.45497212661747e-07,
                                         6.77807118753696e-10,
                                         1.06085241226436e-14,
                                         6.20399322038197e-24,
                                         3.66018530383922e-17,
                                         4.06270847434438e-10,
                                         1.59053125610042e-06,
                                         0.000269295538909709,
                                         0.00736009158792791,
                                         0.0553818207063776,
                                         0.147155880333662,
                                         0.154172420501709,
                                         0.0650779372927227,
                                         0.0104586511449931,
                                         0.000548930242656045,
                                         6.87775563458161e-06,
                                         1.0728698842818e-08,
                                         4.25929218757411e-13,
                                         1.33529575663852e-21,
                                         2.41173081135759e-19,
                                         1.43504363828713e-11,
                                         1.42506013848388e-07,
                                         4.69490178084863e-05,
                                         0.00219274813052006,
                                         0.0262548785747766,
                                         0.106756483766776,
                                         0.168188095092773,
                                         0.106756483766776,
                                         0.0262548785747766,
                                         0.00219274813052006,
                                         4.69490178084861e-05,
                                         1.42506013848389e-07,
                                         1.43504363828713e-11,
                                         2.41173081135783e-19,
                                         1.33529575663837e-21,
                                         4.25929218757411e-13,
                                         1.07286988428179e-08,
                                         6.87775563458164e-06,
                                         0.000548930242656045,
                                         0.0104586511449931,
                                         0.0650779372927227,
                                         0.154172420501709,
                                         0.147155880333662,
                                         0.0553818207063776,
                                         0.00736009158792791,
                                         0.000269295538909708,
                                         1.59053125610043e-06,
                                         4.06270847434438e-10,
                                         3.66018530383956e-17,
                                         6.20399322038122e-24,
                                         1.06085241226436e-14,
                                         6.77807118753689e-10,
                                         8.45497212661752e-07,
                                         0.000115316320341916,
                                         0.00349612175335312,
                                         0.0332903610855406,
                                         0.118594169616699,
                                         0.170218276887769,
                                         0.0980324040805124,
                                         0.0207311262531281,
                                         0.00129621491856019,
                                         1.48969181763923e-05,
                                         9.65186904502071e-09,
                                         4.66146601889572e-15,
                                         2.40892337756897e-26,
                                         2.20815893399861e-16,
                                         3.57868129109918e-11,
                                         8.68631241146841e-08,
                                         2.02452069023278e-05,
                                         0.00097668663820142,
                                         0.0142318401942567,
                                         0.0762391090393066,
                                         0.164547991138499,
                                         0.145020668578046,
                                         0.0488000649942259,
                                         0.00521414012443508,
                                         0.00011660266667616,
                                         1.91630665462821e-07,
                                         4.96134923324289e-13,
                                         7.75994980772354e-29,
                                         3.81320315485705e-18,
                                         1.56756000456398e-12,
                                         7.40360029764133e-09,
                                         2.94874975172397e-06,
                                         0.000226364437049891,
                                         0.00504763409777747,
                                         0.0406608581542969,
                                         0.131966358719151,
                                         0.177981313299428,
                                         0.0953020315330627,
                                         0.0174009370123853,
                                         0.000757189856357775,
                                         3.15647859669123e-06,
                                         4.3808797089682e-11,
                                         2.05056796125649e-31,
                                         5.40168043816632e-20,
                                         5.63254480236467e-14,
                                         5.17642364983263e-10,
                                         3.52316468861213e-07,
                                         4.30368502855659e-05,
                                         0.00146856780065534,
                                         0.0177891254425049,
                                         0.0868186866922535,
                                         0.179183592867911,
                                         0.152673353831818,
                                         0.0476367255324916,
                                         0.00403348476912682,
                                         4.26500966173e-05,
                                         3.17323464915674e-09,
                                         4.37133101730688e-34,
                                         6.17294068258223e-22,
                                         1.63271133277327e-15,
                                         2.91971817211573e-11,
                                         3.39587928212495e-08,
                                         6.60080696307157e-06,
                                         0.00034468660762825,
                                         0.00627851486206055,
                                         0.0460773466052261,
                                         0.145527927885653,
                                         0.197309776735621,
                                         0.10520477188507,
                                         0.0173332630124974,
                                         0.00046490205027975,
                                         1.85424586335394e-07,
                                         7.33412641996817e-37,
                                         5.55201319787536e-24,
                                         3.72485464393708e-17,
                                         1.29612600214065e-12,
                                         2.5761236964118e-09,
                                         7.96798954105444e-07,
                                         6.36722193702784e-05,
                                         0.00174403190612793,
                                         0.0192467268238767,
                                         0.093022845685835,
                                         0.200691571893746,
                                         0.182862287022031,
                                         0.0586239939551479,
                                         0.00398839418013679,
                                         8.52761569181712e-06,
                                         9.32592632226023e-40,
                                         3.78458067177613e-26,
                                         6.4404786299936e-19,
                                         4.36076220325415e-14,
                                         1.48111940770646e-10,
                                         7.28969519711139e-08,
                                         8.91424276724808e-06,
                                         0.000367164611816406,
                                         0.0060930563366082,
                                         0.045065250292885,
                                         0.154710060178852,
                                         0.24089164997842,
                                         0.150272450972215,
                                         0.0259324516821632,
                                         0.000297232854627566,
                                         8.44929429374769e-43,
                                         1.8381032924232e-28,
                                         7.93435861954532e-21,
                                         1.04535176058271e-15,
                                         6.06733956853454e-12,
                                         4.75176388629855e-09,
                                         8.89208888755195e-07,
                                         5.50746917724609e-05,
                                         0.00137435324981885,
                                         0.0155553154486233,
                                         0.0849753269530248,
                                         0.226101901226978,
                                         0.274453141083537,
                                         0.12013621627165,
                                         0.00738160621999121,
                                         4.86035885816809e-46,
                                         5.66815216651439e-31,
                                         6.20618832856572e-23,
                                         1.59104310962759e-17,
                                         1.57806894144095e-13,
                                         1.96661694726066e-10,
                                         5.63173972392607e-08,
                                         5.24520874023438e-06,
                                         0.000196825333358833,
                                         0.00340906513961946,
                                         0.0296337474498083,
                                         0.134742972534349,
                                         0.318255914352196,
                                         0.353365188962657,
                                         0.116392333835021,
                                         1.33439022884574e-49,
                                         8.34218288813733e-34,
                                         2.31688614654207e-25,
                                         1.15576105825349e-19,
                                         1.95893606295382e-15,
                                         3.88464485388739e-12,
                                         1.702346544168e-09,
                                         2.38418579101562e-07,
                                         1.34533436495141e-05,
                                         0.000356581161269309,
                                         0.0049322709328893,
                                         0.0383243371408101,
                                         0.176137317932882,
                                         0.496066838574306,
                                         0.875920110832538,

                                         0.889674381737307,
                                         0.538869453364842,
                                         0.216524787780646,
                                         0.0567071940175241,
                                         0.00942186020720731,
                                         0.000955246749836926,
                                         5.59656483219231e-05,
                                         1.75614297952806e-06,
                                         2.65140923852817e-08,
                                         1.64949875318137e-10,
                                         3.349875256277e-13,
                                         1.53401257246213e-16,
                                         8.37934082845613e-21,
                                         1.58454864336167e-26,
                                         5.48102215057088e-35,
                                         8.574845908783e-51,
                                         0.104279515372506,
                                         0.337899122753231,
                                         0.343085952733693,
                                         0.173833021789732,
                                         0.0489573696566974,
                                         0.00781198805798478,
                                         0.000690167802815402,
                                         3.19305241619477e-05,
                                         7.05790731494686e-07,
                                         6.47387332583267e-09,
                                         1.98256755482511e-11,
                                         1.42887046909392e-14,
                                         1.3230038124156e-18,
                                         4.84011435336093e-24,
                                         4.23061417522769e-32,
                                         3.54081932706212e-47,
                                         0.00583355828472511,
                                         0.101124684843495,
                                         0.259456688073292,
                                         0.254327322753728,
                                         0.121413258419572,
                                         0.0304911771368073,
                                         0.00406213609144554,
                                         0.000277088717200437,
                                         8.96688819625845e-06,
                                         1.21267108199327e-07,
                                         5.60007666389167e-10,
                                         6.35218913279748e-13,
                                         9.96962800684207e-17,
                                         7.05622283393457e-22,
                                         1.5585182834972e-29,
                                         6.97827094942981e-44,
                                         0.000207198935794406,
                                         0.0192152810002991,
                                         0.124579402989624,
                                         0.236250752948456,
                                         0.191176098695687,
                                         0.075562488675816,
                                         0.0151800668913124,
                                         0.00152669076367425,
                                         7.23314215644157e-05,
                                         1.44225245572574e-06,
                                         1.00433683744265e-08,
                                         1.79297164812309e-11,
                                         4.76997666097772e-15,
                                         6.53143176289398e-20,
                                         3.64535497067328e-27,
                                         8.731951821841e-41,
                                         5.24356152634285e-06,
                                         0.00260148404863234,
                                         0.0426199061670275,
                                         0.156364532862109,
                                         0.214479576585194,
                                         0.133420690495191,
                                         0.0404182745891672,
                                         0.00599332988690083,
                                         0.000415716359131699,
                                         1.22214974535582e-05,
                                         1.28336349179482e-07,
                                         3.60585528143698e-10,
                                         1.62606695035093e-13,
                                         4.30754049830747e-18,
                                         6.07508860472302e-25,
                                         7.78501945678141e-38,
                                         1.00571310013789e-07,
                                         0.000266934363537451,
                                         0.0110506449180512,
                                         0.0784354140187845,
                                         0.182367390407004,
                                         0.17854553989475,
                                         0.0815625407334377,
                                         0.0178317587096027,
                                         0.00181082349476688,
                                         7.84903828206777e-05,
                                         1.24287902884018e-06,
                                         5.49606745314628e-09,
                                         4.20116252522216e-12,
                                         2.15307538410763e-16,
                                         7.67316079996158e-23,
                                         5.26037814718313e-35,
                                         1.51815815329554e-09,
                                         2.15567345903445e-05,
                                         0.00225505910687763,
                                         0.0309657287432885,
                                         0.122040391959567,
                                         0.188048553922656,
                                         0.129538508415732,
                                         0.0417556592659398,
                                         0.00620797982004375,
                                         0.000396737857019393,
                                         9.47334186528287e-06,
                                         6.59312008553582e-08,
                                         8.54271097349437e-11,
                                         8.47001681369145e-15,
                                         7.62765694994835e-21,
                                         2.79749550762129e-32,
                                         1.84877558455883e-11,
                                         1.40438355422906e-06,
                                         0.000371238070262233,
                                         0.0098622200515764,
                                         0.0658846549290486,
                                         0.159777367681858,
                                         0.165970655704395,
                                         0.0788788886016826,
                                         0.0171691479878542,
                                         0.00161776379072712,
                                         5.82507915824304e-05,
                                         6.38049230708807e-07,
                                         1.40134869982818e-09,
                                         2.68802671592711e-13,
                                         6.11691263166219e-19,
                                         1.2001792177733e-29,
                                         1.84684610281984e-13,
                                         7.50529599969424e-08,
                                         5.01333138602961e-05,
                                         0.00257660236684686,
                                         0.02917724532179,
                                         0.111362726479739,
                                         0.174438786560172,
                                         0.122232177910834,
                                         0.0389517093488656,
                                         0.00541135352762123,
                                         0.000293818936423475,
                                         5.06520144653189e-06,
                                         1.88571495870257e-08,
                                         6.99781115929193e-12,
                                         4.02395144574261e-17,
                                         4.22378823529739e-27,
                                         1.53059916367919e-15,
                                         3.32762339526426e-09,
                                         5.61674326697899e-06,
                                         0.000558475812243016,
                                         0.0107198461484372,
                                         0.0643944892052365,
                                         0.152103447723759,
                                         0.15714283045995,
                                         0.0733142763142031,
                                         0.0150169231677356,
                                         0.00122953809009865,
                                         3.33598054521662e-05,
                                         2.10518434431266e-07,
                                         1.51138500991913e-10,
                                         2.19612694353493e-15,
                                         1.23322529561101e-24,
                                         1.0601078786021e-17,
                                         1.23298654991266e-10,
                                         5.25896838457952e-07,
                                         0.000101162408721483,
                                         0.00329147562985131,
                                         0.0311182650872134,
                                         0.110839073206611,
                                         0.168834586632129,
                                         0.115321002513687,
                                         0.0348268211747889,
                                         0.00429993626610509,
                                         0.000183614985351947,
                                         1.96409297878961e-06,
                                         2.72800918700215e-09,
                                         1.0016591779191e-13,
                                         3.00912714515337e-22,
                                         6.16146323167003e-20,
                                         3.83378468176018e-12,
                                         4.13201372243495e-08,
                                         1.53772672837366e-05,
                                         0.000848082192029024,
                                         0.0126190673479214,
                                         0.0677784950762788,
                                         0.152220615931809,
                                         0.152220615931809,
                                         0.0677784950762787,
                                         0.0126190673479214,
                                         0.000848082192029024,
                                         1.53772672837365e-05,
                                         4.13201372243499e-08,
                                         3.83378468176018e-12,
                                         6.16146323166932e-20,
                                         3.00912714515375e-22,
                                         1.0016591779191e-13,
                                         2.72800918700213e-09,
                                         1.96409297878962e-06,
                                         0.000183614985351947,
                                         0.00429993626610509,
                                         0.0348268211747889,
                                         0.115321002513687,
                                         0.168834586632129,
                                         0.110839073206611,
                                         0.0311182650872134,
                                         0.00329147562985131,
                                         0.000101162408721482,
                                         5.25896838457957e-07,
                                         1.23298654991266e-10,
                                         1.06010787860199e-17,
                                         1.23322529561118e-24,
                                         2.19612694353493e-15,
                                         1.51138500991911e-10,
                                         2.10518434431267e-07,
                                         3.33598054521662e-05,
                                         0.00122953809009865,
                                         0.0150169231677356,
                                         0.0733142763142031,
                                         0.15714283045995,
                                         0.152103447723759,
                                         0.0643944892052365,
                                         0.0107198461484372,
                                         0.000558475812243014,
                                         5.61674326697903e-06,
                                         3.32762339526426e-09,
                                         1.53059916367904e-15,
                                         4.22378823529801e-27,
                                         4.02395144574261e-17,
                                         6.99781115929185e-12,
                                         1.88571495870258e-08,
                                         5.06520144653189e-06,
                                         0.000293818936423475,
                                         0.00541135352762125,
                                         0.0389517093488656,
                                         0.122232177910833,
                                         0.174438786560172,
                                         0.111362726479739,
                                         0.02917724532179,
                                         0.00257660236684685,
                                         5.01333138602964e-05,
                                         7.50529599969424e-08,
                                         1.84684610281968e-13,
                                         1.20017921777349e-29,
                                         6.11691263166219e-19,
                                         2.68802671592707e-13,
                                         1.40134869982819e-09,
                                         6.38049230708807e-07,
                                         5.82507915824304e-05,
                                         0.00161776379072712,
                                         0.0171691479878542,
                                         0.0788788886016824,
                                         0.165970655704395,
                                         0.159777367681858,
                                         0.0658846549290486,
                                         0.00986222005157637,
                                         0.000371238070262235,
                                         1.40438355422906e-06,
                                         1.84877558455869e-11,
                                         2.79749550762176e-32,
                                         7.62765694994835e-21,
                                         8.47001681369134e-15,
                                         8.54271097349443e-11,
                                         6.59312008553582e-08,
                                         9.47334186528287e-06,
                                         0.000396737857019394,
                                         0.00620797982004375,
                                         0.0417556592659398,
                                         0.129538508415731,
                                         0.188048553922656,
                                         0.122040391959567,
                                         0.0309657287432884,
                                         0.00225505910687764,
                                         2.15567345903445e-05,
                                         1.51815815329544e-09,
                                         5.26037814718407e-35,
                                         7.67316079996158e-23,
                                         2.1530753841076e-16,
                                         4.20116252522219e-12,
                                         5.49606745314628e-09,
                                         1.24287902884018e-06,
                                         7.84903828206779e-05,
                                         0.00181082349476688,
                                         0.0178317587096027,
                                         0.0815625407334377,
                                         0.17854553989475,
                                         0.182367390407004,
                                         0.0784354140187844,
                                         0.0110506449180513,
                                         0.000266934363537451,
                                         1.00571310013783e-07,
                                         7.78501945678288e-38,
                                         6.07508860472302e-25,
                                         4.3075404983074e-18,
                                         1.62606695035094e-13,
                                         3.60585528143698e-10,
                                         1.28336349179482e-07,
                                         1.22214974535582e-05,
                                         0.000415716359131698,
                                         0.00599332988690081,
                                         0.0404182745891671,
                                         0.133420690495191,
                                         0.214479576585194,
                                         0.156364532862109,
                                         0.0426199061670277,
                                         0.00260148404863234,
                                         5.24356152634263e-06,
                                         8.73195182184274e-41,
                                         3.64535497067328e-27,
                                         6.53143176289388e-20,
                                         4.76997666097776e-15,
                                         1.79297164812309e-11,
                                         1.00433683744265e-08,
                                         1.44225245572575e-06,
                                         7.23314215644156e-05,
                                         0.00152669076367425,
                                         0.0151800668913124,
                                         0.075562488675816,
                                         0.191176098695687,
                                         0.236250752948456,
                                         0.124579402989624,
                                         0.0192152810002991,
                                         0.000207198935794399,
                                         6.97827094943127e-44,
                                         1.5585182834972e-29,
                                         7.05622283393445e-22,
                                         9.96962800684216e-17,
                                         6.35218913279748e-13,
                                         5.60007666389167e-10,
                                         1.21267108199327e-07,
                                         8.96688819625844e-06,
                                         0.000277088717200436,
                                         0.00406213609144554,
                                         0.0304911771368073,
                                         0.121413258419572,
                                         0.254327322753728,
                                         0.259456688073292,
                                         0.101124684843495,
                                         0.00583355828472498,
                                         3.54081932706289e-47,
                                         4.23061417522769e-32,
                                         4.84011435336085e-24,
                                         1.32300381241562e-18,
                                         1.42887046909392e-14,
                                         1.98256755482511e-11,
                                         6.47387332583269e-09,
                                         7.05790731494685e-07,
                                         3.19305241619476e-05,
                                         0.000690167802815402,
                                         0.00781198805798478,
                                         0.0489573696566974,
                                         0.173833021789732,
                                         0.343085952733693,
                                         0.337899122753231,
                                         0.104279515372505,
                                         8.57484590878498e-51,
                                         5.48102215057088e-35,
                                         1.58454864336164e-26,
                                         8.37934082845621e-21,
                                         1.53401257246213e-16,
                                         3.349875256277e-13,
                                         1.64949875318138e-10,
                                         2.65140923852816e-08,
                                         1.75614297952805e-06,
                                         5.59656483219231e-05,
                                         0.000955246749836926,
                                         0.00942186020720731,
                                         0.0567071940175241,
                                         0.216524787780646,
                                         0.538869453364842,
                                         0.889674381737307,

                                         0.901302084017481,
                                         0.577312350147767,
                                         0.257056222776839,
                                         0.0784699520850333,
                                         0.0160806829301475,
                                         0.00214704960067068,
                                         0.000179224220378633,
                                         8.84021362155432e-06,
                                         2.38418579101562e-07,
                                         3.15420969394974e-09,
                                         1.7510995223178e-11,
                                         3.22884117690969e-14,
                                         1.3648875219762e-17,
                                         6.9929339757635e-22,
                                         1.25969942861626e-27,
                                         4.21419950922121e-36,
                                         6.47214344795254e-52,
                                         0.0938801691255969,
                                         0.32115226897225,
                                         0.360207346925247,
                                         0.211719292340832,
                                         0.0730587567680292,
                                         0.0152166661760624,
                                         0.00189312422932701,
                                         0.000135574375751554,
                                         5.24520874023438e-06,
                                         9.95453711367343e-08,
                                         8.02368327689283e-10,
                                         2.20503054110669e-12,
                                         1.45403357960455e-15,
                                         1.25443378808502e-19,
                                         4.35098763486288e-25,
                                         3.66657275794997e-33,
                                         3.00739177730862e-48,
                                         0.00466706629063988,
                                         0.0852663670570003,
                                         0.240903788471889,
                                         0.272636563053547,
                                         0.158418787364025,
                                         0.0514711138979367,
                                         0.00954394979524477,
                                         0.000992336773061096,
                                         5.50746917724609e-05,
                                         1.49940238170156e-06,
                                         1.75470188698172e-08,
                                         7.18702496408575e-11,
                                         7.39296461298514e-14,
                                         1.07399603192333e-17,
                                         7.17257972133195e-23,
                                         1.52255202354028e-30,
                                         6.66958020919283e-45,
                                         0.000147310436976155,
                                         0.0143735474158399,
                                         0.102294935976915,
                                         0.222908795262951,
                                         0.218102444645899,
                                         0.110541935067452,
                                         0.0305489733925873,
                                         0.00461168886356086,
                                         0.000367164611816406,
                                         1.43395249669889e-05,
                                         2.43642109412836e-07,
                                         1.48731562184693e-09,
                                         2.3866144206086e-12,
                                         5.83817423885685e-16,
                                         7.50727562191328e-21,
                                         4.01424075758927e-28,
                                         9.39131555735694e-42,
                                         3.31289700791469e-06,
                                         0.00172637461425813,
                                         0.0309492057802001,
                                         0.129853954454827,
                                         0.213943576205945,
                                         0.169151334004342,
                                         0.0696706661493303,
                                         0.0152702371418111,
                                         0.00174403190612793,
                                         9.77093673363759e-05,
                                         2.41038395283411e-06,
                                         2.19301698012234e-08,
                                         5.48947437492112e-11,
                                         2.26118604435873e-14,
                                         5.59853253435803e-19,
                                         7.54083711765431e-26,
                                         9.42191617106685e-39,
                                         5.64665485276489e-08,
                                         0.000157150225174187,
                                         0.00709665631955529,
                                         0.0573313338485129,
                                         0.159054829241772,
                                         0.196170040952783,
                                         0.120423763843571,
                                         0.0383213195566052,
                                         0.00627851486206055,
                                         0.000504599166938007,
                                         1.80729478303407e-05,
                                         2.45069768380898e-07,
                                         9.56947594221929e-10,
                                         6.63749843515741e-13,
                                         3.16428026751434e-17,
                                         1.07360517259486e-23,
                                         7.16408828992962e-36,
                                         7.57477471618725e-10,
                                         1.12587491837487e-05,
                                         0.00128071710120121,
                                         0.0199215938857916,
                                         0.0930656979435619,
                                         0.179054445726145,
                                         0.16382101074267,
                                         0.075688569807481,
                                         0.0177891254425049,
                                         0.00205093561768101,
                                         0.000106651491360352,
                                         2.15542371329047e-06,
                                         1.31292741852608e-08,
                                         1.53344335264507e-11,
                                         1.40757274793177e-15,
                                         1.20299770994448e-21,
                                         4.28723997208601e-33,
                                         8.19733097985192e-12,
                                         6.5071310390674e-07,
                                         0.00018645623708896,
                                         0.00558444979088242,
                                         0.0439295433980334,
                                         0.131844435766967,
                                         0.17978409858244,
                                         0.120599196547798,
                                         0.0406608581542969,
                                         0.006724837541981,
                                         0.000507725694421589,
                                         1.52932519835401e-05,
                                         1.45317377849272e-07,
                                         2.85795452669057e-10,
                                         5.05115887154375e-14,
                                         1.08744994994471e-19,
                                         2.06975494375298e-30,
                                         7.27703644626678e-14,
                                         3.08509321228522e-08,
                                         2.22679353405586e-05,
                                         0.00128415080436542,
                                         0.0170099512620182,
                                         0.0796375393791931,
                                         0.161849842886444,
                                         0.157629631169221,
                                         0.0762391090393066,
                                         0.0180880144541474,
                                         0.00198276242995257,
                                         8.90115495800667e-05,
                                         1.31939160626887e-06,
                                         4.36940419175191e-09,
                                         1.48693004290419e-12,
                                         8.06367627155145e-18,
                                         8.19670666770056e-28,
                                         5.3594581436782e-16,
                                         1.21347625367912e-09,
                                         2.20631359945928e-06,
                                         0.000244982983783406,
                                         0.00546428953850492,
                                         0.0399078300313573,
                                         0.120880860126246,
                                         0.170928923881194,
                                         0.118594169616699,
                                         0.0403630754545267,
                                         0.00642386601654953,
                                         0.000429810611083885,
                                         9.9383465188617e-06,
                                         5.54208754447307e-08,
                                         3.63140174915965e-11,
                                         4.96067922176271e-16,
                                         2.69304811313098e-25,
                                         3.29871688181054e-18,
                                         3.98889036684955e-11,
                                         1.82688983089462e-07,
                                         3.90583270522286e-05,
                                         0.00146697314538311,
                                         0.0167130695700036,
                                         0.0754502402552855,
                                         0.154899880810513,
                                         0.154172420501709,
                                         0.0752723420885357,
                                         0.0173932241312056,
                                         0.00173446575880207,
                                         6.25622529027898e-05,
                                         5.8746550221134e-07,
                                         7.41166679546714e-10,
                                         2.55039255134907e-14,
                                         7.39446329516534e-23,
                                         1.70378351215949e-20,
                                         1.10031776938505e-12,
                                         1.26941224526574e-08,
                                         5.2256048242508e-06,
                                         0.000330488104697357,
                                         0.00587353467056466,
                                         0.0395192706721966,
                                         0.117796345541306,
                                         0.168188095092773,
                                         0.117796345541306,
                                         0.0395192706721966,
                                         0.00587353467056465,
                                         0.000330488104697356,
                                         5.22560482425077e-06,
                                         1.26941224526574e-08,
                                         1.10031776938503e-12,
                                         1.70378351215971e-20,
                                         7.3944632951643e-23,
                                         2.55039255134913e-14,
                                         7.41166679546714e-10,
                                         5.87465502211343e-07,
                                         6.256225290279e-05,
                                         0.00173446575880208,
                                         0.0173932241312056,
                                         0.0752723420885357,
                                         0.154172420501709,
                                         0.154899880810513,
                                         0.0754502402552855,
                                         0.0167130695700035,
                                         0.00146697314538311,
                                         3.90583270522284e-05,
                                         1.82688983089462e-07,
                                         3.98889036684946e-11,
                                         3.29871688181092e-18,
                                         2.69304811313057e-25,
                                         4.96067922176285e-16,
                                         3.63140174915965e-11,
                                         5.54208754447311e-08,
                                         9.93834651886174e-06,
                                         0.000429810611083886,
                                         0.00642386601654953,
                                         0.0403630754545267,
                                         0.118594169616699,
                                         0.170928923881195,
                                         0.120880860126246,
                                         0.0399078300313573,
                                         0.0054642895385049,
                                         0.000244982983783404,
                                         2.20631359945928e-06,
                                         1.2134762536791e-09,
                                         5.35945814367876e-16,
                                         8.19670666769921e-28,
                                         8.06367627155171e-18,
                                         1.48693004290419e-12,
                                         4.36940419175194e-09,
                                         1.31939160626888e-06,
                                         8.9011549580067e-05,
                                         0.00198276242995257,
                                         0.0180880144541474,
                                         0.0762391090393066,
                                         0.157629631169221,
                                         0.161849842886444,
                                         0.079637539379193,
                                         0.0170099512620181,
                                         0.00128415080436541,
                                         2.22679353405586e-05,
                                         3.08509321228517e-08,
                                         7.27703644626747e-14,
                                         2.06975494375262e-30,
                                         1.08744994994475e-19,
                                         5.05115887154375e-14,
                                         2.8579545266906e-10,
                                         1.45317377849273e-07,
                                         1.52932519835402e-05,
                                         0.000507725694421589,
                                         0.006724837541981,
                                         0.0406608581542969,
                                         0.120599196547798,
                                         0.17978409858244,
                                         0.131844435766967,
                                         0.0439295433980333,
                                         0.0055844497908824,
                                         0.00018645623708896,
                                         6.5071310390673e-07,
                                         8.19733097985259e-12,
                                         4.2872399720852e-33,
                                         1.20299770994453e-21,
                                         1.40757274793177e-15,
                                         1.53344335264509e-11,
                                         1.31292741852609e-08,
                                         2.15542371329048e-06,
                                         0.000106651491360352,
                                         0.00205093561768101,
                                         0.0177891254425049,
                                         0.075688569807481,
                                         0.16382101074267,
                                         0.179054445726144,
                                         0.0930656979435617,
                                         0.0199215938857915,
                                         0.00128071710120121,
                                         1.12587491837485e-05,
                                         7.57477471618778e-10,
                                         7.16408828992819e-36,
                                         1.0736051725949e-23,
                                         3.16428026751434e-17,
                                         6.63749843515747e-13,
                                         9.56947594221934e-10,
                                         2.45069768380899e-07,
                                         1.80729478303407e-05,
                                         0.000504599166938007,
                                         0.00627851486206055,
                                         0.0383213195566052,
                                         0.120423763843571,
                                         0.196170040952783,
                                         0.159054829241771,
                                         0.0573313338485127,
                                         0.00709665631955529,
                                         0.000157150225174185,
                                         5.64665485276523e-08,
                                         9.42191617106486e-39,
                                         7.54083711765461e-26,
                                         5.59853253435803e-19,
                                         2.26118604435875e-14,
                                         5.48947437492115e-11,
                                         2.19301698012235e-08,
                                         2.41038395283411e-06,
                                         9.77093673363759e-05,
                                         0.00174403190612793,
                                         0.0152702371418111,
                                         0.0696706661493303,
                                         0.169151334004342,
                                         0.213943576205945,
                                         0.129853954454827,
                                         0.0309492057802001,
                                         0.00172637461425812,
                                         3.31289700791484e-06,
                                         9.39131555735484e-42,
                                         4.01424075758944e-28,
                                         7.50727562191328e-21,
                                         5.8381742388569e-16,
                                         2.38661442060862e-12,
                                         1.48731562184694e-09,
                                         2.43642109412836e-07,
                                         1.43395249669889e-05,
                                         0.000367164611816406,
                                         0.00461168886356086,
                                         0.0305489733925873,
                                         0.110541935067452,
                                         0.218102444645899,
                                         0.222908795262951,
                                         0.102294935976915,
                                         0.0143735474158398,
                                         0.00014731043697616,
                                         6.66958020919125e-45,
                                         1.52255202354035e-30,
                                         7.17257972133195e-23,
                                         1.07399603192334e-17,
                                         7.39296461298519e-14,
                                         7.18702496408579e-11,
                                         1.75470188698172e-08,
                                         1.49940238170156e-06,
                                         5.50746917724609e-05,
                                         0.000992336773061096,
                                         0.00954394979524477,
                                         0.0514711138979367,
                                         0.158418787364025,
                                         0.272636563053547,
                                         0.240903788471889,
                                         0.085266367057,
                                         0.00466706629063999,
                                         3.00739177730788e-48,
                                         3.66657275795014e-33,
                                         4.35098763486288e-25,
                                         1.25443378808504e-19,
                                         1.45403357960456e-15,
                                         2.20503054110671e-12,
                                         8.02368327689283e-10,
                                         9.95453711367343e-08,
                                         5.24520874023438e-06,
                                         0.000135574375751554,
                                         0.00189312422932701,
                                         0.0152166661760624,
                                         0.0730587567680292,
                                         0.211719292340832,
                                         0.360207346925247,
                                         0.32115226897225,
                                         0.093880169125598,
                                         6.47214344795086e-52,
                                         4.21419950922142e-36,
                                         1.25969942861626e-27,
                                         6.99293397576358e-22,
                                         1.36488752197621e-17,
                                         3.2288411769097e-14,
                                         1.7510995223178e-11,
                                         3.15420969394974e-09,
                                         2.38418579101562e-07,
                                         8.84021362155432e-06,
                                         0.000179224220378633,
                                         0.00214704960067068,
                                         0.0160806829301475,
                                         0.0784699520850333,
                                         0.257056222776839,
                                         0.577312350147767,
                                         0.901302084017481,

                                         0.911212315023544,
                                         0.611779348435885,
                                         0.296925115242574,
                                         0.103034123263234,
                                         0.0251408872845399,
                                         0.0042138354555896,
                                         0.000469915182591269,
                                         3.34015820174187e-05,
                                         1.42827850853232e-06,
                                         3.39588677644595e-08,
                                         4.02372618166288e-10,
                                         2.03087792655811e-12,
                                         3.45361580072238e-15,
                                         1.36508337488617e-18,
                                         6.62743254731043e-23,
                                         1.14607980319698e-28,
                                         3.72779109616459e-37,
                                         5.63665212378911e-53,
                                         0.0849032186850333,
                                         0.30400072449224,
                                         0.370684579927603,
                                         0.246687753313919,
                                         0.100803366244816,
                                         0.0261648515027158,
                                         0.00430769635198663,
                                         0.000439129393485185,
                                         2.65108576431299e-05,
                                         8.8549820911149e-07,
                                         1.48131803245504e-08,
                                         1.07226856804723e-10,
                                         2.69202248498373e-13,
                                         1.64782127161039e-16,
                                         1.33975124150824e-20,
                                         4.4432692832982e-26,
                                         3.6309236962356e-34,
                                         2.92793896928803e-49,
                                         0.00377568145676991,
                                         0.0720976389790394,
                                         0.220865928324356,
                                         0.281890680278342,
                                         0.192901714570062,
                                         0.077539963773989,
                                         0.0188467862980983,
                                         0.00275539963847724,
                                         0.000234855790384435,
                                         1.10201826029277e-05,
                                         2.60276416449313e-07,
                                         2.70202865187199e-09,
                                         1.00149771239875e-11,
                                         9.49352832342565e-15,
                                         1.29261633035739e-18,
                                         8.22161295252299e-24,
                                         1.68791003904331e-31,
                                         7.25887680438657e-46,
                                         0.000106607063593502,
                                         0.0108564268136,
                                         0.0835549861690725,
                                         0.204518824342615,
                                         0.234377858725105,
                                         0.145899013413726,
                                         0.0523538918786344,
                                         0.010977315035855,
                                         0.00132098598085609,
                                         8.70781785692826e-05,
                                         2.90362662128874e-06,
                                         4.32310482422394e-08,
                                         2.3655965717396e-10,
                                         3.47267897681628e-13,
                                         7.91834737787436e-17,
                                         9.65897101797604e-22,
                                         4.98196775108336e-29,
                                         1.14260540315934e-42,
                                         2.14467497415636e-06,
                                         0.00116476320778325,
                                         0.0225216853670436,
                                         0.105723303788523,
                                         0.202899952485582,
                                         0.195597808779159,
                                         0.103620457643772,
                                         0.0311596469150985,
                                         0.00529395217643082,
                                         0.000490246780612531,
                                         2.30797759762369e-05,
                                         4.92817874345989e-07,
                                         3.9812208635616e-09,
                                         9.0507901187417e-12,
                                         3.45608433093907e-15,
                                         8.08517765141793e-20,
                                         1.04770109160663e-26,
                                         1.28146866111536e-39,
                                         3.26998588590029e-08,
                                         9.47103125393673e-05,
                                         0.00460085232139271,
                                         0.0414206639203242,
                                         0.133123951296613,
                                         0.198739643487208,
                                         0.155435756582,
                                         0.0670344142246895,
                                         0.0160794322437095,
                                         0.00209184285337345,
                                         0.000139037291738483,
                                         4.25780754556161e-06,
                                         5.07809418551324e-08,
                                         1.7877930363722e-10,
                                         1.14325471949594e-13,
                                         5.12928891362916e-18,
                                         1.66987042107021e-24,
                                         1.08925258003259e-36,
                                         3.92396786762194e-10,
                                         6.06110860540374e-06,
                                         0.000739725977082481,
                                         0.0127719895849235,
                                         0.068742547756889,
                                         0.158927919389872,
                                         0.183506531714125,
                                         0.11350063199105,
                                         0.0384376277425442,
                                         0.00702487387346278,
                                         0.000659213633487255,
                                         2.89521468968689e-05,
                                         5.09777193715758e-07,
                                         2.77934947823374e-09,
                                         2.97643803040969e-12,
                                         2.56106102651739e-16,
                                         2.09470716292698e-22,
                                         7.286926265291e-34,
                                         3.79864998184888e-12,
                                         3.1291842363767e-07,
                                         9.5946186647564e-05,
                                         0.00317705203025566,
                                         0.0286364593477389,
                                         0.102527451162593,
                                         0.174773826341563,
                                         0.155032593569885,
                                         0.0741253427567883,
                                         0.0190314675315059,
                                         0.00252142081177367,
                                         0.000158818025223631,
                                         4.12842428823059e-06,
                                         3.48572682001718e-08,
                                         6.25136646482839e-11,
                                         1.03158962143519e-14,
                                         2.11976704085088e-20,
                                         3.93264210232182e-31,
                                         3.01656330402686e-14,
                                         1.32522472808253e-08,
                                         1.02085461460999e-05,
                                         0.000648290157769954,
                                         0.00978571010001716,
                                         0.0542574593149579,
                                         0.136546505802395,
                                         0.173710901702054,
                                         0.117261698927601,
                                         0.0422946421656089,
                                         0.00791122643665322,
                                         0.000714657801518974,
                                         2.74263223464455e-05,
                                         3.58610388669864e-07,
                                         1.0770414111659e-09,
                                         3.40857873710576e-13,
                                         1.75967426316872e-18,
                                         1.74102017274236e-28,
                                         1.98737518659685e-16,
                                         4.65620596042382e-10,
                                         9.0112352617489e-07,
                                         0.000109748548075867,
                                         0.00277427574818081,
                                         0.023821165153077,
                                         0.0885052518004739,
                                         0.161478754486566,
                                         0.153896911355432,
                                         0.0779799021185505,
                                         0.0205933286891914,
                                         0.00266796876375686,
                                         0.000151159373008602,
                                         3.0608118872242e-06,
                                         1.53948016993997e-08,
                                         9.34380897317861e-12,
                                         1.2118829716661e-16,
                                         6.39451243397088e-26,
                                         1.09422115951412e-18,
                                         1.36720192104371e-11,
                                         6.64756490319374e-08,
                                         1.55269432132566e-05,
                                         0.000657301663233334,
                                         0.00874026042034618,
                                         0.0479419085292313,
                                         0.125447362079334,
                                         0.168795728232363,
                                         0.120153891891013,
                                         0.0447988755670968,
                                         0.00832379102260676,
                                         0.000696242313676411,
                                         2.18327397620161e-05,
                                         1.83896567274131e-07,
                                         2.14058460489063e-10,
                                         6.97504333725137e-15,
                                         1.96276766727304e-23,
                                         5.05563326615651e-21,
                                         3.36882418314207e-13,
                                         4.11515362598208e-09,
                                         1.84339478476794e-06,
                                         0.00013068479037967,
                                         0.00269110692724264,
                                         0.0217924874473713,
                                         0.0817810880569807,
                                         0.155359640625362,
                                         0.155359640625362,
                                         0.0817810880569807,
                                         0.0217924874473713,
                                         0.00269110692724264,
                                         0.00013068479037967,
                                         1.84339478476795e-06,
                                         4.11515362598208e-09,
                                         3.36882418314207e-13,
                                         5.05563326615651e-21,
                                         1.96276766727304e-23,
                                         6.97504333725137e-15,
                                         2.14058460489063e-10,
                                         1.83896567274129e-07,
                                         2.18327397620161e-05,
                                         0.000696242313676411,
                                         0.00832379102260676,
                                         0.0447988755670968,
                                         0.120153891891013,
                                         0.168795728232363,
                                         0.125447362079334,
                                         0.0479419085292313,
                                         0.00874026042034618,
                                         0.000657301663233334,
                                         1.55269432132567e-05,
                                         6.64756490319374e-08,
                                         1.36720192104371e-11,
                                         1.09422115951412e-18,
                                         6.39451243397088e-26,
                                         1.2118829716661e-16,
                                         9.3438089731786e-12,
                                         1.53948016993996e-08,
                                         3.06081188722421e-06,
                                         0.000151159373008602,
                                         0.00266796876375686,
                                         0.0205933286891914,
                                         0.0779799021185505,
                                         0.153896911355432,
                                         0.161478754486566,
                                         0.088505251800474,
                                         0.023821165153077,
                                         0.00277427574818081,
                                         0.000109748548075867,
                                         9.0112352617489e-07,
                                         4.65620596042382e-10,
                                         1.98737518659685e-16,
                                         1.74102017274236e-28,
                                         1.75967426316872e-18,
                                         3.40857873710576e-13,
                                         1.0770414111659e-09,
                                         3.58610388669864e-07,
                                         2.74263223464455e-05,
                                         0.000714657801518974,
                                         0.00791122643665322,
                                         0.0422946421656089,
                                         0.117261698927601,
                                         0.173710901702054,
                                         0.136546505802395,
                                         0.0542574593149579,
                                         0.00978571010001716,
                                         0.000648290157769956,
                                         1.02085461460999e-05,
                                         1.32522472808253e-08,
                                         3.01656330402686e-14,
                                         3.93264210232182e-31,
                                         2.11976704085088e-20,
                                         1.03158962143519e-14,
                                         6.25136646482834e-11,
                                         3.48572682001718e-08,
                                         4.12842428823059e-06,
                                         0.000158818025223631,
                                         0.00252142081177367,
                                         0.0190314675315059,
                                         0.0741253427567883,
                                         0.155032593569885,
                                         0.174773826341563,
                                         0.102527451162593,
                                         0.0286364593477389,
                                         0.00317705203025567,
                                         9.5946186647564e-05,
                                         3.1291842363767e-07,
                                         3.79864998184888e-12,
                                         7.286926265291e-34,
                                         2.09470716292698e-22,
                                         2.56106102651739e-16,
                                         2.97643803040966e-12,
                                         2.77934947823374e-09,
                                         5.09777193715758e-07,
                                         2.89521468968689e-05,
                                         0.000659213633487255,
                                         0.00702487387346278,
                                         0.0384376277425442,
                                         0.11350063199105,
                                         0.183506531714125,
                                         0.158927919389872,
                                         0.068742547756889,
                                         0.0127719895849235,
                                         0.000739725977082481,
                                         6.06110860540374e-06,
                                         3.92396786762194e-10,
                                         1.08925258003259e-36,
                                         1.66987042107021e-24,
                                         5.12928891362916e-18,
                                         1.14325471949593e-13,
                                         1.7877930363722e-10,
                                         5.07809418551324e-08,
                                         4.25780754556161e-06,
                                         0.000139037291738483,
                                         0.00209184285337345,
                                         0.0160794322437095,
                                         0.0670344142246895,
                                         0.155435756582,
                                         0.198739643487208,
                                         0.133123951296613,
                                         0.0414206639203243,
                                         0.00460085232139271,
                                         9.47103125393673e-05,
                                         3.26998588590029e-08,
                                         1.28146866111536e-39,
                                         1.04770109160663e-26,
                                         8.08517765141793e-20,
                                         3.45608433093903e-15,
                                         9.0507901187417e-12,
                                         3.9812208635616e-09,
                                         4.92817874345989e-07,
                                         2.30797759762369e-05,
                                         0.000490246780612531,
                                         0.00529395217643082,
                                         0.0311596469150985,
                                         0.103620457643772,
                                         0.195597808779159,
                                         0.202899952485582,
                                         0.105723303788524,
                                         0.0225216853670436,
                                         0.00116476320778325,
                                         2.14467497415636e-06,
                                         1.14260540315934e-42,
                                         4.98196775108336e-29,
                                         9.65897101797604e-22,
                                         7.91834737787427e-17,
                                         3.47267897681628e-13,
                                         2.3655965717396e-10,
                                         4.32310482422394e-08,
                                         2.90362662128874e-06,
                                         8.70781785692826e-05,
                                         0.00132098598085609,
                                         0.010977315035855,
                                         0.0523538918786344,
                                         0.145899013413726,
                                         0.234377858725105,
                                         0.204518824342615,
                                         0.0835549861690725,
                                         0.0108564268136,
                                         0.000106607063593502,
                                         7.25887680438658e-46,
                                         1.68791003904331e-31,
                                         8.22161295252299e-24,
                                         1.29261633035738e-18,
                                         9.49352832342565e-15,
                                         1.00149771239875e-11,
                                         2.70202865187199e-09,
                                         2.60276416449313e-07,
                                         1.10201826029277e-05,
                                         0.000234855790384435,
                                         0.00275539963847724,
                                         0.0188467862980983,
                                         0.077539963773989,
                                         0.192901714570062,
                                         0.281890680278343,
                                         0.220865928324356,
                                         0.0720976389790394,
                                         0.00377568145676991,
                                         2.92793896928803e-49,
                                         3.6309236962356e-34,
                                         4.4432692832982e-26,
                                         1.33975124150823e-20,
                                         1.64782127161039e-16,
                                         2.69202248498373e-13,
                                         1.07226856804723e-10,
                                         1.48131803245504e-08,
                                         8.8549820911149e-07,
                                         2.65108576431299e-05,
                                         0.000439129393485185,
                                         0.00430769635198663,
                                         0.0261648515027158,
                                         0.100803366244816,
                                         0.246687753313919,
                                         0.370684579927603,
                                         0.30400072449224,
                                         0.0849032186850333,
                                         5.63665212378911e-53,
                                         3.72779109616459e-37,
                                         1.14607980319698e-28,
                                         6.62743254731035e-23,
                                         1.36508337488617e-18,
                                         3.45361580072238e-15,
                                         2.03087792655811e-12,
                                         4.02372618166288e-10,
                                         3.39588677644595e-08,
                                         1.42827850853232e-06,
                                         3.34015820174187e-05,
                                         0.000469915182591269,
                                         0.0042138354555896,
                                         0.0251408872845399,
                                         0.103034123263234,
                                         0.296925115242574,
                                         0.611779348435885,
                                         0.911212315023544,

                                         0.919722141898732,
                                         0.642670399898298,
                                         0.335564614551174,
                                         0.129763091121051,
                                         0.0366717358255488,
                                         0.00743422108952727,
                                         0.00105430891119708,
                                         0.000101172919940108,
                                         6.28592839447956e-06,
                                         2.38418579101562e-07,
                                         5.09805065184731e-09,
                                         5.50416483795656e-11,
                                         2.56322238103503e-13,
                                         4.07029385078204e-16,
                                         1.51981514988685e-19,
                                         7.04924205045956e-24,
                                         1.1774688413244e-29,
                                         3.73947350515346e-38,
                                         5.57996386543452e-54,
                                         0.0771122951327161,
                                         0.287013876843486,
                                         0.375662309058946,
                                         0.27766939279562,
                                         0.13080931214697,
                                         0.040818563059979,
                                         0.00847973546116151,
                                         0.00115542761891402,
                                         0.000100067647109244,
                                         5.24520874023438e-06,
                                         1.54997698266607e-07,
                                         2.33269701170369e-09,
                                         1.54247175950236e-11,
                                         3.58797068415449e-14,
                                         2.0621912341352e-17,
                                         1.59445019053104e-21,
                                         5.09065094593643e-27,
                                         4.05266984421759e-35,
                                         3.22114118592106e-50,
                                         0.00308572511356387,
                                         0.0611764272871527,
                                         0.200717716491687,
                                         0.283577295925397,
                                         0.222696029902735,
                                         0.106966200550345,
                                         0.0325509260848929,
                                         0.00629778489642289,
                                         0.000760299255882612,
                                         5.50746917724609e-05,
                                         2.24912177347299e-06,
                                         4.71836846530155e-08,
                                         4.43011270037764e-10,
                                         1.50951914791478e-12,
                                         1.33546872483554e-15,
                                         1.72125854194408e-19,
                                         1.0504220598632e-24,
                                         2.0962284330542e-32,
                                         8.87472191035125e-47,
                                         7.83989565700846e-05,
                                         0.00827913125754471,
                                         0.0680915520205631,
                                         0.183879936540788,
                                         0.240716427529278,
                                         0.177973306671951,
                                         0.0793348190873813,
                                         0.0217947726664234,
                                         0.00366770911348006,
                                         0.000367164611816406,
                                         2.07214495522081e-05,
                                         6.05961196115679e-07,
                                         8.07851935304486e-09,
                                         4.03225332382292e-11,
                                         5.49108225036962e-14,
                                         1.17977911044803e-17,
                                         1.37617539883534e-22,
                                         6.88423095627137e-30,
                                         1.55245559792258e-43,
                                         1.41921502000795e-06,
                                         0.000798307655598572,
                                         0.0164583247585037,
                                         0.0849536615871636,
                                         0.185388953439284,
                                         0.210983279100379,
                                         0.137768265672045,
                                         0.0537405036719738,
                                         0.0126063706512649,
                                         0.00174403190612793,
                                         0.000136022942191874,
                                         5.54475861523915e-06,
                                         1.04962371248523e-07,
                                         7.67435455130388e-10,
                                         1.60867030430659e-12,
                                         5.76156100211825e-16,
                                         1.28460206189115e-20,
                                         1.61085750566515e-27,
                                         1.93494440232084e-40,
                                         1.94713026002187e-08,
                                         5.83397698288819e-05,
                                         0.00301499720038989,
                                         0.0297466970312709,
                                         0.108210864653425,
                                         0.189561482842505,
                                         0.181319051214311,
                                         0.10042920218417,
                                         0.0328393196371258,
                                         0.00627851486206055,
                                         0.000676726350089318,
                                         3.84529236574168e-05,
                                         1.03358069332234e-06,
                                         1.10699281376034e-08,
                                         3.57178272079094e-11,
                                         2.13249743722093e-14,
                                         9.08808396267795e-19,
                                         2.85672022340702e-25,
                                         1.82779163299309e-37,
                                         2.10250474058152e-10,
                                         3.35547725130263e-06,
                                         0.000434693722994298,
                                         0.0081976725831924,
                                         0.049711063225988,
                                         0.13404398391132,
                                         0.187816115807536,
                                         0.147711212487575,
                                         0.0673276412770437,
                                         0.0177891254425049,
                                         0.00264977610950263,
                                         0.000209880097508328,
                                         8.01032825670814e-06,
                                         1.25673269533028e-07,
                                         6.24163242659176e-10,
                                         6.21200850417943e-13,
                                         5.0602411841835e-17,
                                         3.98724997297209e-23,
                                         1.35887669155669e-34,
                                         1.83148443822191e-12,
                                         1.55692666942781e-07,
                                         5.05596543722641e-05,
                                         0.00182249631329681,
                                         0.0184229642074407,
                                         0.0764660802601351,
                                         0.156944659897895,
                                         0.175263380890474,
                                         0.111356845688767,
                                         0.0406608581542969,
                                         0.00837007876335961,
                                         0.000924139259665968,
                                         5.00818669651136e-05,
                                         1.15097342921288e-06,
                                         8.79904660809578e-09,
                                         1.45982177347556e-11,
                                         2.27297351229994e-15,
                                         4.48955700135354e-21,
                                         8.15000143941651e-32,
                                         1.30872624425664e-14,
                                         5.92599589802734e-09,
                                         4.82396505712783e-06,
                                         0.000332370199550791,
                                         0.0056007384783176,
                                         0.0357824158878825,
                                         0.10758198697761,
                                         0.170587905384184,
                                         0.151084450941072,
                                         0.0762391090393066,
                                         0.0216884861181882,
                                         0.00333797372011707,
                                         0.00025685619008901,
                                         8.64703090589275e-06,
                                         1.01754207568245e-07,
                                         2.81414818034938e-10,
                                         8.37523230666625e-14,
                                         4.14679766664137e-19,
                                         4.00972557822048e-29,
                                         7.75851571852789e-17,
                                         1.87128005742064e-10,
                                         3.81846192357219e-07,
                                         5.02876975175391e-05,
                                         0.00141258741946837,
                                         0.0138916791486735,
                                         0.0611810369201099,
                                         0.13774934494407,
                                         0.170061840489766,
                                         0.118594169616699,
                                         0.0466243925938379,
                                         0.0100025944281632,
                                         0.00109290853306778,
                                         5.38955587540466e-05,
                                         9.762327859907e-07,
                                         4.50068212528599e-09,
                                         2.56025724333647e-12,
                                         3.17765288858093e-17,
                                         1.63665021020614e-26,
                                         3.84384886286004e-19,
                                         4.93826120761655e-12,
                                         2.52598403539798e-08,
                                         6.35856698836116e-06,
                                         0.000297744124493237,
                                         0.00450710478465164,
                                         0.0290771563359041,
                                         0.0929584072716499,
                                         0.159974884483621,
                                         0.154172420501709,
                                         0.0837635280730146,
                                         0.0250495675584055,
                                         0.00388629195206467,
                                         0.000280735136841241,
                                         7.82730447085763e-06,
                                         6.01544202589983e-08,
                                         6.54075935456417e-11,
                                         2.03496943729378e-15,
                                         5.582836591802e-24,
                                         1.59808240776513e-21,
                                         1.09359001293969e-13,
                                         1.40222577911605e-09,
                                         6.74686405049871e-07,
                                         5.26642949931132e-05,
                                         0.00122711644021948,
                                         0.0115966416792366,
                                         0.0526420767854496,
                                         0.126282143269214,
                                         0.168188095092773,
                                         0.126282143269214,
                                         0.0526420767854497,
                                         0.0115966416792366,
                                         0.00122711644021947,
                                         5.26642949931134e-05,
                                         6.74686405049871e-07,
                                         1.40222577911605e-09,
                                         1.09359001293969e-13,
                                         1.59808240776513e-21,
                                         5.582836591802e-24,
                                         2.03496943729378e-15,
                                         6.54075935456417e-11,
                                         6.01544202589983e-08,
                                         7.82730447085759e-06,
                                         0.000280735136841242,
                                         0.00388629195206467,
                                         0.0250495675584054,
                                         0.0837635280730146,
                                         0.154172420501709,
                                         0.159974884483621,
                                         0.09295840727165,
                                         0.0290771563359041,
                                         0.00450710478465163,
                                         0.000297744124493238,
                                         6.35856698836116e-06,
                                         2.52598403539798e-08,
                                         4.93826120761655e-12,
                                         3.84384886286004e-19,
                                         1.63665021020614e-26,
                                         3.17765288858094e-17,
                                         2.56025724333647e-12,
                                         4.50068212528599e-09,
                                         9.76232785990695e-07,
                                         5.38955587540467e-05,
                                         0.00109290853306778,
                                         0.0100025944281632,
                                         0.0466243925938379,
                                         0.118594169616699,
                                         0.170061840489766,
                                         0.13774934494407,
                                         0.0611810369201099,
                                         0.0138916791486735,
                                         0.00141258741946838,
                                         5.02876975175391e-05,
                                         3.81846192357219e-07,
                                         1.87128005742064e-10,
                                         7.75851571852789e-17,
                                         4.00972557822048e-29,
                                         4.14679766664137e-19,
                                         8.37523230666625e-14,
                                         2.81414818034938e-10,
                                         1.01754207568245e-07,
                                         8.64703090589279e-06,
                                         0.00025685619008901,
                                         0.00333797372011707,
                                         0.0216884861181882,
                                         0.0762391090393066,
                                         0.151084450941072,
                                         0.170587905384184,
                                         0.10758198697761,
                                         0.0357824158878825,
                                         0.00560073847831762,
                                         0.000332370199550791,
                                         4.82396505712783e-06,
                                         5.92599589802734e-09,
                                         1.30872624425664e-14,
                                         8.15000143941651e-32,
                                         4.48955700135354e-21,
                                         2.27297351229994e-15,
                                         1.45982177347556e-11,
                                         8.79904660809573e-09,
                                         1.15097342921288e-06,
                                         5.00818669651136e-05,
                                         0.000924139259665966,
                                         0.00837007876335961,
                                         0.0406608581542969,
                                         0.111356845688767,
                                         0.175263380890474,
                                         0.156944659897895,
                                         0.0764660802601349,
                                         0.0184229642074408,
                                         0.00182249631329681,
                                         5.05596543722641e-05,
                                         1.55692666942781e-07,
                                         1.83148443822191e-12,
                                         1.35887669155669e-34,
                                         3.98724997297209e-23,
                                         5.0602411841835e-17,
                                         6.21200850417943e-13,
                                         6.24163242659172e-10,
                                         1.25673269533028e-07,
                                         8.01032825670814e-06,
                                         0.000209880097508327,
                                         0.00264977610950263,
                                         0.0177891254425049,
                                         0.0673276412770437,
                                         0.147711212487575,
                                         0.187816115807536,
                                         0.13404398391132,
                                         0.0497110632259882,
                                         0.0081976725831924,
                                         0.000434693722994298,
                                         3.35547725130263e-06,
                                         2.10250474058152e-10,
                                         1.82779163299309e-37,
                                         2.85672022340702e-25,
                                         9.08808396267795e-19,
                                         2.13249743722093e-14,
                                         3.57178272079092e-11,
                                         1.10699281376034e-08,
                                         1.03358069332234e-06,
                                         3.84529236574167e-05,
                                         0.000676726350089318,
                                         0.00627851486206055,
                                         0.0328393196371258,
                                         0.10042920218417,
                                         0.181319051214311,
                                         0.189561482842505,
                                         0.108210864653425,
                                         0.0297466970312709,
                                         0.00301499720038989,
                                         5.83397698288819e-05,
                                         1.94713026002187e-08,
                                         1.93494440232084e-40,
                                         1.61085750566515e-27,
                                         1.28460206189115e-20,
                                         5.76156100211825e-16,
                                         1.60867030430657e-12,
                                         7.67435455130391e-10,
                                         1.04962371248523e-07,
                                         5.54475861523913e-06,
                                         0.000136022942191874,
                                         0.00174403190612793,
                                         0.0126063706512649,
                                         0.0537405036719738,
                                         0.137768265672045,
                                         0.210983279100379,
                                         0.185388953439284,
                                         0.0849536615871636,
                                         0.0164583247585037,
                                         0.000798307655598572,
                                         1.41921502000795e-06,
                                         1.55245559792258e-43,
                                         6.88423095627137e-30,
                                         1.37617539883534e-22,
                                         1.17977911044803e-17,
                                         5.49108225036958e-14,
                                         4.03225332382294e-11,
                                         8.07851935304486e-09,
                                         6.05961196115677e-07,
                                         2.07214495522081e-05,
                                         0.000367164611816406,
                                         0.00366770911348006,
                                         0.0217947726664234,
                                         0.0793348190873814,
                                         0.177973306671951,
                                         0.240716427529279,
                                         0.183879936540788,
                                         0.0680915520205631,
                                         0.00827913125754471,
                                         7.83989565700846e-05,
                                         8.87472191035125e-47,
                                         2.0962284330542e-32,
                                         1.0504220598632e-24,
                                         1.72125854194408e-19,
                                         1.33546872483552e-15,
                                         1.50951914791479e-12,
                                         4.43011270037764e-10,
                                         4.71836846530154e-08,
                                         2.24912177347299e-06,
                                         5.50746917724609e-05,
                                         0.000760299255882612,
                                         0.0062977848964229,
                                         0.0325509260848929,
                                         0.106966200550345,
                                         0.222696029902735,
                                         0.283577295925397,
                                         0.200717716491687,
                                         0.0611764272871527,
                                         0.00308572511356387,
                                         3.22114118592106e-50,
                                         4.05266984421759e-35,
                                         5.09065094593643e-27,
                                         1.59445019053104e-21,
                                         2.06219123413518e-17,
                                         3.58797068415451e-14,
                                         1.54247175950236e-11,
                                         2.33269701170368e-09,
                                         1.54997698266607e-07,
                                         5.24520874023438e-06,
                                         0.000100067647109244,
                                         0.00115542761891402,
                                         0.00847973546116151,
                                         0.040818563059979,
                                         0.13080931214697,
                                         0.27766939279562,
                                         0.375662309058946,
                                         0.287013876843486,
                                         0.0771122951327161,
                                         5.57996386543452e-54,
                                         3.73947350515346e-38,
                                         1.1774688413244e-29,
                                         7.04924205045956e-24,
                                         1.51981514988684e-19,
                                         4.07029385078206e-16,
                                         2.56322238103503e-13,
                                         5.50416483795654e-11,
                                         5.09805065184731e-09,
                                         2.38418579101562e-07,
                                         6.28592839447956e-06,
                                         0.000101172919940108,
                                         0.00105430891119708,
                                         0.00743422108952727,
                                         0.0366717358255488,
                                         0.129763091121051,
                                         0.335564614551174,
                                         0.642670399898298,
                                         0.919722141898732,

                                         0.927079875493952,
                                         0.67037245619706,
                                         0.372601863446872,
                                         0.158032293675646,
                                         0.0505975456326884,
                                         0.0120472016259238,
                                         0.00209082310024712,
                                         0.000257634610502917,
                                         2.17789947555713e-05,
                                         1.20749191740117e-06,
                                         4.13692359899655e-08,
                                         8.08296501047131e-10,
                                         8.06232435724319e-12,
                                         3.50536516540389e-14,
                                         5.25002521897877e-17,
                                         1.86719073713867e-20,
                                         8.32846951959286e-25,
                                         1.35043036874403e-30,
                                         4.2018715358293e-39,
                                         6.19920931926873e-55,
                                         0.0703152938150735,
                                         0.270548168169073,
                                         0.376227690565478,
                                         0.304137102539487,
                                         0.161692988372434,
                                         0.0589588008639036,
                                         0.014891577181886,
                                         0.00258411152344154,
                                         0.000301353084836066,
                                         2.27880208012097e-05,
                                         1.06096360737032e-06,
                                         2.82734137901558e-08,
                                         3.89043873654466e-10,
                                         2.38207384791105e-12,
                                         5.19211482100971e-15,
                                         2.82795379239909e-18,
                                         2.09453200849099e-22,
                                         6.47309265325494e-28,
                                         5.03917796672581e-36,
                                         3.95593384561249e-51,
                                         0.00254535906910202,
                                         0.0521122292500842,
                                         0.181310564104715,
                                         0.27935701603989,
                                         0.24661503088181,
                                         0.13771388284369,
                                         0.0506210002099066,
                                         0.0123704334779605,
                                         0.00199012369690485,
                                         0.000205255876694969,
                                         1.29864393704972e-05,
                                         4.72011324056476e-07,
                                         8.95990723854282e-09,
                                         7.72580591933377e-11,
                                         2.45072061991336e-13,
                                         2.04419608401796e-16,
                                         2.5140587412447e-20,
                                         1.48087386674513e-25,
                                         2.88431866631514e-33,
                                         1.20483717241833e-47,
                                         5.85016019163857e-05,
                                         0.00637315010822409,
                                         0.0554772472371555,
                                         0.162918050244789,
                                         0.238818155512666,
                                         0.204233155118985,
                                         0.109254715184279,
                                         0.0375991498756112,
                                         0.00834456961854949,
                                         0.00117382683899398,
                                         0.000100925088573142,
                                         5.00317885577857e-06,
                                         1.31017071549962e-07,
                                         1.5909327011334e-09,
                                         7.34450893499069e-12,
                                         9.38193104952098e-15,
                                         1.91594607649638e-18,
                                         2.15101657401362e-23,
                                         1.04820499904121e-30,
                                         2.32984558306197e-44,
                                         9.58012838417862e-07,
                                         0.000555333039031284,
                                         0.0120946015779583,
                                         0.0676961068444712,
                                         0.164778295260261,
                                         0.2158040737204,
                                         0.16800976526752,
                                         0.0814246689612564,
                                         0.0249294491527431,
                                         0.00478296649755878,
                                         0.000558847168510936,
                                         3.77854420822947e-05,
                                         1.36501408034976e-06,
                                         2.33423562052754e-08,
                                         1.56825467491971e-10,
                                         3.06793949565896e-13,
                                         1.04034171191836e-16,
                                         2.22614943039934e-21,
                                         2.71415147241964e-28,
                                         3.21004252913932e-41,
                                         1.18900530174659e-08,
                                         3.66742944816612e-05,
                                         0.00199837560958331,
                                         0.0213190120012315,
                                         0.0861671169717784,
                                         0.172823151993148,
                                         0.195811300591402,
                                         0.133641969955411,
                                         0.0564456285499092,
                                         0.0147706474914772,
                                         0.00234528627291274,
                                         0.000216277767573797,
                                         1.07784240441452e-05,
                                         2.59565200394094e-07,
                                         2.53792807082832e-09,
                                         7.6034419595071e-12,
                                         4.281319436858e-15,
                                         1.74611868803802e-19,
                                         5.3263638112399e-26,
                                         3.35199471990908e-38,
                                         1.16142567724682e-10,
                                         1.90618564132712e-06,
                                         0.000259871016247836,
                                         0.00528403426192534,
                                         0.0354632300540319,
                                         0.108927961176748,
                                         0.179612331040247,
                                         0.172633432649353,
                                         0.100587291673262,
                                         0.0359002025274386,
                                         0.00774629154919491,
                                         0.000974303850107558,
                                         6.69836131726896e-05,
                                         2.27166042119524e-06,
                                         3.23249005923777e-08,
                                         1.48309458781177e-10,
                                         1.38667416283846e-13,
                                         1.07792458700653e-17,
                                         8.22664474114534e-24,
                                         2.75480609774394e-35,
                                         9.15215416847474e-13,
                                         7.99268857199028e-08,
                                         2.72623218575406e-05,
                                         0.00105654609073321,
                                         0.0117744136237537,
                                         0.055386140097992,
                                         0.132910355036268,
                                         0.179900021476814,
                                         0.144603960518656,
                                         0.0703912269570063,
                                         0.0206403038935779,
                                         0.00354079864990349,
                                         0.000335819682289423,
                                         1.60385324760821e-05,
                                         3.32138605802235e-07,
                                         2.33373605900472e-09,
                                         3.62322651950293e-12,
                                         5.36818244398367e-16,
                                         1.02503555741447e-21,
                                         1.82642993691028e-32,
                                         5.91608711648478e-15,
                                         2.74915938000227e-09,
                                         2.34610349319016e-06,
                                         0.000173296822358454,
                                         0.00320685716168661,
                                         0.0231016050216929,
                                         0.0806790575544472,
                                         0.153786018643898,
                                         0.170528350164287,
                                         0.113219048654783,
                                         0.0451146575690611,
                                         0.0105557081150677,
                                         0.00138109340706324,
                                         9.28891895499598e-05,
                                         2.79950242534491e-06,
                                         3.01240911220691e-08,
                                         7.7659730413858e-11,
                                         2.19303461240377e-14,
                                         1.04769397884846e-19,
                                         9.93331774623558e-30,
                                         3.17270759313533e-17,
                                         7.84496841824059e-11,
                                         1.67500360362061e-07,
                                         2.35818029186444e-05,
                                         0.0007246097919305,
                                         0.00799406109034946,
                                         0.0406300230637317,
                                         0.109065337169929,
                                         0.1668388727908,
                                         0.151079215685009,
                                         0.0818094546380935,
                                         0.0261070480517616,
                                         0.00471220539227628,
                                         0.000446323725731075,
                                         1.95761188043818e-05,
                                         3.22597073135409e-07,
                                         1.38095817390185e-09,
                                         7.43272318959437e-13,
                                         8.88411597562403e-18,
                                         4.48197977956806e-27,
                                         1.42194684350977e-19,
                                         1.87085576618478e-12,
                                         9.99406457781811e-09,
                                         2.68176820617849e-06,
                                         0.000136831663522357,
                                         0.00231180179929337,
                                         0.0170998045865072,
                                         0.0646419523882885,
                                         0.136412990455534,
                                         0.168479772676667,
                                         0.123978722179567,
                                         0.0539617456356393,
                                         0.0134364084635306,
                                         0.00179222528373592,
                                         0.000114401089904239,
                                         2.88711922753534e-06,
                                         2.05221566059219e-08,
                                         2.10527221144945e-11,
                                         6.29581295181074e-16,
                                         1.69006451760516e-24,
                                         5.34788283190348e-22,
                                         3.74398950470803e-14,
                                         5.00395926436975e-10,
                                         2.55923801669037e-07,
                                         2.16827430798556e-05,
                                         0.000561020770887053,
                                         0.00603921421347919,
                                         0.0321504748969921,
                                         0.0935964595157376,
                                         0.157665264149948,
                                         0.157665264149948,
                                         0.0935964595157376,
                                         0.0321504748969921,
                                         0.00603921421347919,
                                         0.000561020770887051,
                                         2.16827430798556e-05,
                                         2.55923801669039e-07,
                                         5.00395926436975e-10,
                                         3.74398950470816e-14,
                                         5.34788283190443e-22,
                                         1.69006451760483e-24,
                                         6.29581295181051e-16,
                                         2.10527221144945e-11,
                                         2.05221566059218e-08,
                                         2.88711922753534e-06,
                                         0.00011440108990424,
                                         0.00179222528373592,
                                         0.0134364084635306,
                                         0.0539617456356393,
                                         0.123978722179567,
                                         0.168479772676667,
                                         0.136412990455534,
                                         0.0646419523882885,
                                         0.0170998045865072,
                                         0.00231180179929336,
                                         0.000136831663522357,
                                         2.68176820617851e-06,
                                         9.99406457781811e-09,
                                         1.87085576618484e-12,
                                         1.42194684351e-19,
                                         4.48197977956711e-27,
                                         8.88411597562367e-18,
                                         7.43272318959437e-13,
                                         1.38095817390184e-09,
                                         3.22597073135409e-07,
                                         1.95761188043819e-05,
                                         0.000446323725731075,
                                         0.00471220539227628,
                                         0.0261070480517616,
                                         0.0818094546380935,
                                         0.151079215685009,
                                         0.1668388727908,
                                         0.109065337169929,
                                         0.0406300230637317,
                                         0.00799406109034944,
                                         0.0007246097919305,
                                         2.35818029186445e-05,
                                         1.67500360362061e-07,
                                         7.84496841824081e-11,
                                         3.17270759313579e-17,
                                         9.93331774623334e-30,
                                         1.04769397884841e-19,
                                         2.19303461240377e-14,
                                         7.76597304138573e-11,
                                         3.01240911220691e-08,
                                         2.79950242534492e-06,
                                         9.28891895499598e-05,
                                         0.00138109340706324,
                                         0.0105557081150677,
                                         0.0451146575690611,
                                         0.113219048654783,
                                         0.170528350164287,
                                         0.153786018643898,
                                         0.0806790575544472,
                                         0.0231016050216928,
                                         0.00320685716168661,
                                         0.000173296822358455,
                                         2.34610349319016e-06,
                                         2.74915938000234e-09,
                                         5.91608711648555e-15,
                                         1.82642993690984e-32,
                                         1.02503555741442e-21,
                                         5.36818244398367e-16,
                                         3.6232265195029e-12,
                                         2.33373605900472e-09,
                                         3.32138605802236e-07,
                                         1.60385324760821e-05,
                                         0.000335819682289423,
                                         0.00354079864990349,
                                         0.0206403038935779,
                                         0.0703912269570063,
                                         0.144603960518656,
                                         0.179900021476814,
                                         0.132910355036268,
                                         0.0553861400979919,
                                         0.0117744136237537,
                                         0.00105654609073322,
                                         2.72623218575406e-05,
                                         7.99268857199045e-08,
                                         9.15215416847578e-13,
                                         2.75480609774323e-35,
                                         8.22664474114494e-24,
                                         1.07792458700653e-17,
                                         1.38667416283844e-13,
                                         1.48309458781177e-10,
                                         3.23249005923779e-08,
                                         2.27166042119524e-06,
                                         6.69836131726896e-05,
                                         0.000974303850107558,
                                         0.00774629154919491,
                                         0.0359002025274386,
                                         0.100587291673262,
                                         0.172633432649353,
                                         0.179612331040247,
                                         0.108927961176748,
                                         0.0354632300540319,
                                         0.00528403426192536,
                                         0.000259871016247836,
                                         1.90618564132716e-06,
                                         1.16142567724694e-10,
                                         3.35199471990816e-38,
                                         5.32636381123962e-26,
                                         1.74611868803802e-19,
                                         4.28131943685795e-15,
                                         7.6034419595071e-12,
                                         2.53792807082834e-09,
                                         2.59565200394094e-07,
                                         1.07784240441452e-05,
                                         0.000216277767573797,
                                         0.00234528627291274,
                                         0.0147706474914772,
                                         0.0564456285499092,
                                         0.133641969955411,
                                         0.195811300591402,
                                         0.172823151993148,
                                         0.0861671169717784,
                                         0.0213190120012316,
                                         0.00199837560958331,
                                         3.66742944816618e-05,
                                         1.18900530174668e-08,
                                         3.21004252913838e-41,
                                         2.71415147241949e-28,
                                         2.22614943039934e-21,
                                         1.04034171191835e-16,
                                         3.06793949565896e-13,
                                         1.56825467491972e-10,
                                         2.33423562052754e-08,
                                         1.36501408034976e-06,
                                         3.77854420822947e-05,
                                         0.000558847168510936,
                                         0.00478296649755878,
                                         0.0249294491527431,
                                         0.0814246689612564,
                                         0.16800976526752,
                                         0.215804073720399,
                                         0.164778295260261,
                                         0.0676961068444713,
                                         0.0120946015779583,
                                         0.000555333039031291,
                                         9.58012838417924e-07,
                                         2.32984558306126e-44,
                                         1.04820499904114e-30,
                                         2.15101657401362e-23,
                                         1.91594607649635e-18,
                                         9.38193104952098e-15,
                                         7.34450893499073e-12,
                                         1.5909327011334e-09,
                                         1.31017071549962e-07,
                                         5.00317885577857e-06,
                                         0.000100925088573142,
                                         0.00117382683899398,
                                         0.00834456961854949,
                                         0.0375991498756112,
                                         0.109254715184279,
                                         0.204233155118985,
                                         0.238818155512666,
                                         0.162918050244789,
                                         0.0554772472371556,
                                         0.00637315010822415,
                                         5.85016019163886e-05,
                                         1.20483717241794e-47,
                                         2.88431866631496e-33,
                                         1.48087386674513e-25,
                                         2.51405874124467e-20,
                                         2.04419608401796e-16,
                                         2.45072061991338e-13,
                                         7.72580591933377e-11,
                                         8.95990723854282e-09,
                                         4.72011324056476e-07,
                                         1.29864393704972e-05,
                                         0.000205255876694969,
                                         0.00199012369690485,
                                         0.0123704334779605,
                                         0.0506210002099066,
                                         0.13771388284369,
                                         0.24661503088181,
                                         0.27935701603989,
                                         0.181310564104715,
                                         0.0521122292500845,
                                         0.00254535906910211,
                                         3.95593384561115e-51,
                                         5.03917796672548e-36,
                                         6.47309265325494e-28,
                                         2.09453200849096e-22,
                                         2.82795379239909e-18,
                                         5.19211482100975e-15,
                                         2.38207384791105e-12,
                                         3.89043873654466e-10,
                                         2.82734137901558e-08,
                                         1.06096360737032e-06,
                                         2.27880208012097e-05,
                                         0.000301353084836066,
                                         0.00258411152344154,
                                         0.014891577181886,
                                         0.0589588008639035,
                                         0.161692988372434,
                                         0.304137102539487,
                                         0.376227690565478,
                                         0.270548168169074,
                                         0.0703152938150746,
                                         6.19920931926653e-55,
                                         4.20187153582901e-39,
                                         1.35043036874403e-30,
                                         8.32846951959273e-25,
                                         1.86719073713867e-20,
                                         5.25002521897881e-17,
                                         3.50536516540389e-14,
                                         8.06232435724319e-12,
                                         8.08296501047131e-10,
                                         4.13692359899655e-08,
                                         1.20749191740117e-06,
                                         2.17789947555713e-05,
                                         0.000257634610502917,
                                         0.00209082310024712,
                                         0.0120472016259238,
                                         0.0505975456326884,
                                         0.158032293675646,
                                         0.372601863446872,
                                         0.67037245619706,
                                         0.927079875493952,

                                         0.933481910312081,
                                         0.695244869332982,
                                         0.407813341705374,
                                         0.187271946707998,
                                         0.066730925555222,
                                         0.0182271522162449,
                                         0.00375500952665762,
                                         0.00057128796618256,
                                         6.24665053390329e-05,
                                         4.73966947895753e-06,
                                         2.38418579101562e-07,
                                         7.48710513331785e-09,
                                         1.35398513517243e-10,
                                         1.26170574063564e-12,
                                         5.17108908182566e-15,
                                         7.36427189232201e-18,
                                         2.5115365462906e-21,
                                         1.08311560508257e-25,
                                         1.71178360524777e-31,
                                         5.23291300085611e-40,
                                         7.64507300980059e-56,
                                         0.0643556342616571,
                                         0.254814636330637,
                                         0.373345406267015,
                                         0.325969764674871,
                                         0.192230864104081,
                                         0.080063193709482,
                                         0.0238723977082948,
                                         0.0050795810384002,
                                         0.000759650505757426,
                                         7.77737112223024e-05,
                                         5.24520874023438e-06,
                                         2.20838367028615e-07,
                                         5.38880709122811e-09,
                                         6.86800532766681e-11,
                                         3.93679121691768e-13,
                                         8.114487330025e-16,
                                         4.21977110674346e-19,
                                         3.01172684984308e-23,
                                         9.04992212083936e-29,
                                         6.91038587289987e-37,
                                         5.36718209794316e-52,
                                         0.00211755091645259,
                                         0.0445735850123369,
                                         0.163127361814924,
                                         0.27079991962477,
                                         0.264293027606294,
                                         0.167847013382183,
                                         0.0724348653702205,
                                         0.0215559580088301,
                                         0.00440907077942145,
                                         0.000609093840188227,
                                         5.50746917724609e-05,
                                         3.10886412266222e-06,
                                         1.02361819221728e-07,
                                         1.78430784917016e-09,
                                         1.43043907477073e-11,
                                         4.26735699996699e-14,
                                         3.38380170110069e-17,
                                         3.9968970928433e-21,
                                         2.28353377487468e-26,
                                         4.35539653707129e-34,
                                         1.79836417147539e-48,
                                         4.42385201249893e-05,
                                         0.00495051297135423,
                                         0.0452545504477528,
                                         0.14283649514788,
                                         0.230710699409724,
                                         0.223415740571496,
                                         0.139546425130803,
                                         0.0580799463285004,
                                         0.0162479940702156,
                                         0.00302869146443811,
                                         0.000367164611816406,
                                         2.77874309648924e-05,
                                         1.23453326432123e-06,
                                         2.94325838801525e-08,
                                         3.30001405001793e-10,
                                         1.42487349796655e-12,
                                         1.72282179214314e-15,
                                         3.36782715671546e-19,
                                         3.65838560002341e-24,
                                         1.74290030531234e-31,
                                         3.82585401808599e-45,
                                         6.58494554461964e-07,
                                         0.000391748830710922,
                                         0.00894504575031446,
                                         0.0536802720132277,
                                         0.143494294776697,
                                         0.211884303998829,
                                         0.19154668577654,
                                         0.111498738165547,
                                         0.0426616159539257,
                                         0.0107302719373208,
                                         0.00174403190612793,
                                         0.000176961960551788,
                                         1.06084629944656e-05,
                                         3.459169662981e-07,
                                         5.42434222274976e-09,
                                         3.38983461126285e-11,
                                         6.24972168181116e-14,
                                         2.02190846787028e-17,
                                         4.17596054799983e-22,
                                         4.9693805879363e-29,
                                         5.79914565131354e-42,
                                         7.42869753999768e-09,
                                         2.34949273297554e-05,
                                         0.00134002150600057,
                                         0.0152897055461924,
                                         0.06764106295539,
                                         0.152297472438289,
                                         0.199268870853035,
                                         0.16222679557442,
                                         0.0848953186670147,
                                         0.028812126600254,
                                         0.00627851486206055,
                                         0.000854122842570017,
                                         6.90893371841588e-05,
                                         3.08123078126387e-06,
                                         6.75751883877076e-08,
                                         6.11208804433843e-10,
                                         1.71826399300105e-12,
                                         9.19987822475283e-16,
                                         3.61270238947515e-20,
                                         1.07384318050309e-26,
                                         6.66205984497819e-39,
                                         6.59581301236808e-11,
                                         1.10901053957819e-06,
                                         0.000157992379832027,
                                         0.00342750980470055,
                                         0.0250946639075299,
                                         0.0861552556711808,
                                         0.163154647016167,
                                         0.185767782617808,
                                         0.13296133106089,
                                         0.0608884713048508,
                                         0.0177891254425049,
                                         0.00324456084263643,
                                         0.000354131661445687,
                                         2.16008958168174e-05,
                                         6.62556042366284e-07,
                                         8.67352957689135e-09,
                                         3.71804183030147e-11,
                                         3.2945631515488e-14,
                                         2.45981902317576e-18,
                                         1.82631059255134e-24,
                                         6.02349026584233e-36,
                                         4.72441745664716e-13,
                                         4.22300419281766e-08,
                                         1.50274324704767e-05,
                                         0.000619844228187943,
                                         0.00751063427784108,
                                         0.0393183357603839,
                                         0.107766482507561,
                                         0.171609954592263,
                                         0.167993019849253,
                                         0.103805197345072,
                                         0.0406608581542969,
                                         0.00994296200823499,
                                         0.00146434281061226,
                                         0.000122164091162491,
                                         5.24061486685257e-06,
                                         9.92947753552275e-08,
                                         6.49027240971633e-10,
                                         9.51782830693542e-13,
                                         1.35113391990165e-16,
                                         2.50572083536357e-22,
                                         4.39351554872578e-33,
                                         2.77592370388515e-15,
                                         1.31912721724346e-09,
                                         1.17249974947154e-06,
                                         9.19529669419063e-05,
                                         0.00184395864110885,
                                         0.0147193220638161,
                                         0.0583911860650164,
                                         0.130045067662746,
                                         0.174115105130198,
                                         0.145171864835084,
                                         0.0762391090393066,
                                         0.0249951050790059,
                                         0.00496706783420863,
                                         0.000566754143588395,
                                         3.40033103662297e-05,
                                         9.32472989884093e-07,
                                         9.29374834772632e-09,
                                         2.25557475511777e-11,
                                         6.08797650268001e-15,
                                         2.82013604747986e-20,
                                         2.62878732447636e-30,
                                         1.35316588265463e-17,
                                         3.41850408982204e-11,
                                         7.58970658068721e-08,
                                         1.13170487325612e-05,
                                         0.000375586514884773,
                                         0.00457156335667997,
                                         0.0262479348999125,
                                         0.0817578660917653,
                                         0.149715207281774,
                                         0.168434118152027,
                                         0.118594169616699,
                                         0.0521288816544609,
                                         0.0139778918867249,
                                         0.00218137380550544,
                                         0.00018303932630281,
                                         7.26491233805863e-06,
                                         1.10408661804505e-07,
                                         4.43466520526842e-10,
                                         2.27578794337962e-13,
                                         2.63324755502521e-18,
                                         1.3049173049033e-27,
                                         5.51254794163205e-20,
                                         7.4036069181971e-13,
                                         4.10577399928538e-09,
                                         1.16401466377925e-06,
                                         6.39332279134757e-05,
                                         0.00118658677627126,
                                         0.00986054250818922,
                                         0.0429559211498035,
                                         0.10758536770159,
                                         0.163318545928594,
                                         0.154172420501709,
                                         0.0908572682510619,
                                         0.0328731324134092,
                                         0.00701654491359289,
                                         0.000823427645650486,
                                         4.73023069278678e-05,
                                         1.09615813079639e-06,
                                         7.28655550706415e-09,
                                         7.10965362900595e-12,
                                         2.0548075153841e-16,
                                         5.41337801718432e-25,
                                         1.88451209061624e-22,
                                         1.34553751207658e-14,
                                         1.86384718674528e-10,
                                         1.00468270310764e-07,
                                         9.13247478302802e-06,
                                         0.000258451748007651,
                                         0.00310850584013836,
                                         0.0189392045194854,
                                         0.0648762456687528,
                                         0.132888117952389,
                                         0.168188095092773,
                                         0.132888117952389,
                                         0.0648762456687528,
                                         0.0189392045194854,
                                         0.00310850584013836,
                                         0.000258451748007651,
                                         9.13247478302802e-06,
                                         1.00468270310764e-07,
                                         1.86384718674525e-10,
                                         1.34553751207663e-14,
                                         1.88451209061587e-22,
                                         5.41337801718547e-25,
                                         2.05480751538402e-16,
                                         7.10965362900607e-12,
                                         7.28655550706415e-09,
                                         1.09615813079639e-06,
                                         4.73023069278678e-05,
                                         0.000823427645650486,
                                         0.00701654491359288,
                                         0.0328731324134092,
                                         0.0908572682510619,
                                         0.154172420501709,
                                         0.163318545928594,
                                         0.10758536770159,
                                         0.0429559211498036,
                                         0.00986054250818922,
                                         0.00118658677627126,
                                         6.39332279134757e-05,
                                         1.16401466377925e-06,
                                         4.10577399928532e-09,
                                         7.40360691819736e-13,
                                         5.51254794163107e-20,
                                         1.3049173049036e-27,
                                         2.6332475550251e-18,
                                         2.27578794337966e-13,
                                         4.43466520526842e-10,
                                         1.10408661804505e-07,
                                         7.26491233805863e-06,
                                         0.00018303932630281,
                                         0.00218137380550544,
                                         0.0139778918867249,
                                         0.0521288816544609,
                                         0.118594169616699,
                                         0.168434118152027,
                                         0.149715207281774,
                                         0.0817578660917655,
                                         0.0262479348999125,
                                         0.00457156335667997,
                                         0.000375586514884773,
                                         1.13170487325612e-05,
                                         7.58970658068712e-08,
                                         3.41850408982214e-11,
                                         1.35316588265441e-17,
                                         2.62878732447702e-30,
                                         2.82013604747972e-20,
                                         6.08797650268013e-15,
                                         2.25557475511777e-11,
                                         9.29374834772632e-09,
                                         9.32472989884093e-07,
                                         3.40033103662297e-05,
                                         0.000566754143588393,
                                         0.00496706783420863,
                                         0.0249951050790059,
                                         0.0762391090393066,
                                         0.145171864835084,
                                         0.174115105130198,
                                         0.130045067662746,
                                         0.0583911860650164,
                                         0.0147193220638161,
                                         0.00184395864110885,
                                         9.19529669419063e-05,
                                         1.17249974947153e-06,
                                         1.3191272172435e-09,
                                         2.77592370388475e-15,
                                         4.39351554872695e-33,
                                         2.50572083536344e-22,
                                         1.35113391990168e-16,
                                         9.51782830693542e-13,
                                         6.49027240971633e-10,
                                         9.92947753552275e-08,
                                         5.24061486685257e-06,
                                         0.00012216409116249,
                                         0.00146434281061226,
                                         0.00994296200823499,
                                         0.0406608581542969,
                                         0.103805197345072,
                                         0.167993019849253,
                                         0.171609954592264,
                                         0.107766482507561,
                                         0.0393183357603839,
                                         0.00751063427784108,
                                         0.000619844228187943,
                                         1.50274324704765e-05,
                                         4.22300419281776e-08,
                                         4.72441745664658e-13,
                                         6.02349026584404e-36,
                                         1.82631059255124e-24,
                                         2.45981902317581e-18,
                                         3.2945631515488e-14,
                                         3.71804183030147e-11,
                                         8.67352957689136e-09,
                                         6.62556042366284e-07,
                                         2.16008958168174e-05,
                                         0.000354131661445687,
                                         0.00324456084263643,
                                         0.0177891254425049,
                                         0.0608884713048508,
                                         0.13296133106089,
                                         0.185767782617809,
                                         0.163154647016167,
                                         0.0861552556711808,
                                         0.0250946639075299,
                                         0.00342750980470055,
                                         0.000157992379832025,
                                         1.10901053957821e-06,
                                         6.59581301236737e-11,
                                         6.6620598449802e-39,
                                         1.07384318050303e-26,
                                         3.61270238947524e-20,
                                         9.19987822475283e-16,
                                         1.71826399300105e-12,
                                         6.11208804433843e-10,
                                         6.75751883877076e-08,
                                         3.08123078126386e-06,
                                         6.90893371841588e-05,
                                         0.000854122842570017,
                                         0.00627851486206055,
                                         0.028812126600254,
                                         0.0848953186670147,
                                         0.16222679557442,
                                         0.199268870853035,
                                         0.152297472438289,
                                         0.06764106295539,
                                         0.0152897055461924,
                                         0.00134002150600056,
                                         2.34949273297558e-05,
                                         7.42869753999702e-09,
                                         5.7991456513154e-42,
                                         4.969380587936e-29,
                                         4.17596054799994e-22,
                                         2.02190846787028e-17,
                                         6.24972168181116e-14,
                                         3.38983461126285e-11,
                                         5.42434222274976e-09,
                                         3.45916966298098e-07,
                                         1.06084629944656e-05,
                                         0.000176961960551788,
                                         0.00174403190612793,
                                         0.0107302719373208,
                                         0.0426616159539257,
                                         0.111498738165547,
                                         0.19154668577654,
                                         0.211884303998829,
                                         0.143494294776697,
                                         0.0536802720132276,
                                         0.0089450457503144,
                                         0.000391748830710928,
                                         6.58494554461917e-07,
                                         3.82585401808728e-45,
                                         1.74290030531222e-31,
                                         3.65838560002351e-24,
                                         3.36782715671546e-19,
                                         1.72282179214314e-15,
                                         1.42487349796655e-12,
                                         3.30001405001793e-10,
                                         2.94325838801524e-08,
                                         1.23453326432123e-06,
                                         2.77874309648924e-05,
                                         0.000367164611816406,
                                         0.00302869146443811,
                                         0.0162479940702155,
                                         0.0580799463285005,
                                         0.139546425130803,
                                         0.223415740571496,
                                         0.230710699409724,
                                         0.14283649514788,
                                         0.0452545504477526,
                                         0.00495051297135428,
                                         4.4238520124987e-05,
                                         1.79836417147603e-48,
                                         4.355396537071e-34,
                                         2.28353377487474e-26,
                                         3.9968970928433e-21,
                                         3.38380170110069e-17,
                                         4.26735699996699e-14,
                                         1.43043907477073e-11,
                                         1.78430784917015e-09,
                                         1.02361819221728e-07,
                                         3.10886412266222e-06,
                                         5.50746917724609e-05,
                                         0.000609093840188227,
                                         0.00440907077942145,
                                         0.0215559580088301,
                                         0.0724348653702204,
                                         0.167847013382183,
                                         0.264293027606294,
                                         0.27079991962477,
                                         0.163127361814924,
                                         0.0445735850123372,
                                         0.00211755091645252,
                                         5.36718209794516e-52,
                                         6.91038587289938e-37,
                                         9.04992212083963e-29,
                                         3.01172684984308e-23,
                                         4.21977110674346e-19,
                                         8.114487330025e-16,
                                         3.93679121691768e-13,
                                         6.86800532766679e-11,
                                         5.38880709122811e-09,
                                         2.20838367028615e-07,
                                         5.24520874023438e-06,
                                         7.77737112223023e-05,
                                         0.000759650505757426,
                                         0.0050795810384002,
                                         0.0238723977082948,
                                         0.080063193709482,
                                         0.192230864104081,
                                         0.325969764674871,
                                         0.373345406267015,
                                         0.254814636330638,
                                         0.0643556342616559,
                                         7.64507300980358e-56,
                                         5.23291300085572e-40,
                                         1.71178360524782e-31,
                                         1.08311560508257e-25,
                                         2.5115365462906e-21,
                                         7.36427189232201e-18,
                                         5.17108908182566e-15,
                                         1.26170574063563e-12,
                                         1.35398513517243e-10,
                                         7.48710513331785e-09,
                                         2.38418579101562e-07,
                                         4.73966947895753e-06,
                                         6.24665053390329e-05,
                                         0.00057128796618256,
                                         0.00375500952665762,
                                         0.0182271522162449,
                                         0.066730925555222,
                                         0.187271946707998,
                                         0.407813341705374,
                                         0.695244869332982,
                                         0.933481910312081,

                                         0.939085032313984,
                                         0.717613301835131,
                                         0.441086212367604,
                                         0.216988369302927,
                                         0.084809877382232,
                                         0.0260725127261681,
                                         0.00622138723037121,
                                         0.00113274907871766,
                                         0.00015397084896374,
                                         1.51952781262704e-05,
                                         1.05064206037169e-06,
                                         4.86007754129512e-08,
                                         1.41572114133699e-09,
                                         2.3946476560416e-11,
                                         2.10384959603679e-13,
                                         8.19246183715126e-16,
                                         1.11682505971333e-18,
                                         3.67266835446107e-22,
                                         1.53815435938085e-26,
                                         2.3773829349031e-32,
                                         7.15683058084551e-41,
                                         1.03671303300153e-56,
                                         0.0591051728986465,
                                         0.239926474643256,
                                         0.367835929441673,
                                         0.343323025238822,
                                         0.221440515247638,
                                         0.103418106241186,
                                         0.0355495311722224,
                                         0.00899979892029537,
                                         0.00166110731418985,
                                         0.000219238232363169,
                                         2.01003773168404e-05,
                                         1.22953000917093e-06,
                                         4.74914512392672e-08,
                                         1.07430549392769e-09,
                                         1.28162497516223e-11,
                                         6.93925975299794e-14,
                                         1.36275197197631e-16,
                                         6.80795108294496e-20,
                                         4.70520779309505e-24,
                                         1.37979268223364e-29,
                                         1.03604449088534e-37,
                                         7.97229473945979e-53,
                                         0.00177546732400955,
                                         0.0382853378134974,
                                         0.146403490616592,
                                         0.259260302626284,
                                         0.27595251055029,
                                         0.195783862341312,
                                         0.0969498496774296,
                                         0.0341270314822842,
                                         0.00855309892104685,
                                         0.00150969962920019,
                                         0.000183535574316615,
                                         1.48457343823429e-05,
                                         7.6036091132254e-07,
                                         2.30027940159105e-08,
                                         3.72626599495712e-10,
                                         2.80529462885855e-12,
                                         7.93624572794088e-15,
                                         6.02306789063591e-18,
                                         6.86948710884958e-22,
                                         3.82203933136759e-27,
                                         7.15818128834475e-35,
                                         2.92600290354236e-49,
                                         3.38625233526034e-05,
                                         0.00387887903475706,
                                         0.0369971403339189,
                                         0.124304988701429,
                                         0.218338869660444,
                                         0.235329647723988,
                                         0.167872610635436,
                                         0.0821643950123922,
                                         0.0279620328219364,
                                         0.00660061264077535,
                                         0.00106403457985054,
                                         0.000113810853628374,
                                         7.72936054646193e-06,
                                         3.12717961262259e-07,
                                         6.87869710756707e-09,
                                         7.20050999886844e-11,
                                         2.93449136115348e-13,
                                         3.38328460244828e-16,
                                         6.36779773266823e-20,
                                         6.72195976152381e-25,
                                         3.14012128302307e-32,
                                         6.81844907106935e-46,
                                         4.60162071987028e-07,
                                         0.000280004400919604,
                                         0.00666146500896307,
                                         0.0424644948240077,
                                         0.123087126877535,
                                         0.201540005572385,
                                         0.207108258707117,
                                         0.140946299871425,
                                         0.0651326661612536,
                                         0.0205618796967974,
                                         0.00439517425341094,
                                         0.000621656593395294,
                                         5.59824909025341e-05,
                                         3.02907432898032e-06,
                                         9.0473915972992e-08,
                                         1.31683967857386e-09,
                                         7.73099572362857e-12,
                                         1.35407953598843e-14,
                                         4.20570760475781e-18,
                                         8.42328574397647e-23,
                                         9.81465460061279e-30,
                                         1.13209094806155e-42,
                                         4.7392674012524e-09,
                                         1.53190689097507e-05,
                                         0.000909034228304185,
                                         0.0109944183037397,
                                         0.0525899917099657,
                                         0.130814166658522,
                                         0.193652844868364,
                                         0.183245191737775,
                                         0.114984119034306,
                                         0.0485456445948855,
                                         0.0137595853014111,
                                         0.00257351259946928,
                                         0.000307305095536698,
                                         2.22369861647375e-05,
                                         9.01881505711613e-07,
                                         1.82520412453839e-08,
                                         1.54364319208069e-10,
                                         4.10732159258924e-13,
                                         2.10522143207526e-16,
                                         7.99974327258067e-21,
                                         2.32494406888329e-27,
                                         1.42457699220628e-39,
                                         3.84155253794569e-11,
                                         6.59621879114253e-07,
                                         9.76305850207989e-05,
                                         0.00224033890804694,
                                         0.0176843352940187,
                                         0.0668256915574625,
                                         0.142510057639159,
                                         0.187502329011505,
                                         0.159761466522266,
                                         0.0902054755775312,
                                         0.0339023467975504,
                                         0.00838488718347725,
                                         0.00132764656160528,
                                         0.000128480462574618,
                                         7.07572172268044e-06,
                                         1.99106367500387e-07,
                                         2.42579212015121e-09,
                                         9.80546992891099e-12,
                                         8.29376374253917e-15,
                                         5.97951192555256e-19,
                                         4.33456149662063e-25,
                                         1.41086579387938e-36,
                                         2.51204035806353e-13,
                                         2.29130027216213e-08,
                                         8.45893755210642e-06,
                                         0.000368281122207337,
                                         0.00479731957741683,
                                         0.0275395231661228,
                                         0.084604102302813,
                                         0.154776498352472,
                                         0.179073155461828,
                                         0.135219647862871,
                                         0.067387361291043,
                                         0.0220390271618454,
                                         0.0046272126678378,
                                         0.000598856170611169,
                                         4.47833192311359e-05,
                                         1.75219779090157e-06,
                                         3.07527892279682e-08,
                                         1.88843650725843e-10,
                                         2.63590533207926e-13,
                                         3.60561784555254e-17,
                                         6.51931701315865e-23,
                                         1.12722277304264e-33,
                                         1.34749070222884e-15,
                                         6.52902438149619e-10,
                                         6.01208449111646e-07,
                                         4.96620363830221e-05,
                                         0.00106754917569413,
                                         0.00930997897838245,
                                         0.041201844349702,
                                         0.104805167608503,
                                         0.164652475198452,
                                         0.166274645327531,
                                         0.109876917991544,
                                         0.0475189620728414,
                                         0.0132292667912954,
                                         0.00228974592947796,
                                         0.000232509736281094,
                                         1.26491236263925e-05,
                                         3.19812012163129e-07,
                                         2.98342890115728e-09,
                                         6.87206544592654e-12,
                                         1.78349945987045e-15,
                                         8.04337580282688e-21,
                                         7.38776603997076e-31,
                                         5.99665682910954e-18,
                                         1.54347225909483e-11,
                                         3.54501915394459e-08,
                                         5.55589262511206e-06,
                                         0.000197088544817885,
                                         0.00261111042379431,
                                         0.0166466231008254,
                                         0.0588768587215742,
                                         0.1256001896992,
                                         0.169627598583865,
                                         0.148634197110482,
                                         0.0850013309868932,
                                         0.0313788009048586,
                                         0.00726333903707148,
                                         0.00100149840512522,
                                         7.57568660207307e-05,
                                         2.75923878594637e-06,
                                         3.91032857504076e-08,
                                         1.48637714627737e-10,
                                         7.31897938524635e-14,
                                         8.23301439750439e-19,
                                         4.01699015200026e-28,
                                         2.23023404529155e-20,
                                         3.04934941951112e-13,
                                         1.7469075540239e-09,
                                         5.19446698102304e-07,
                                         3.04083392917338e-05,
                                         0.000612011528049758,
                                         0.00562073955825647,
                                         0.0276416772001368,
                                         0.0800700598168952,
                                         0.144618823312481,
                                         0.168030778677296,
                                         0.12706979587895,
                                         0.0622006235578974,
                                         0.01925498406591,
                                         0.00360509909227828,
                                         0.00037917647865742,
                                         1.98948938624902e-05,
                                         4.28320279007377e-07,
                                         2.68675816700135e-09,
                                         2.51007033895741e-12,
                                         7.04266799745768e-17,
                                         1.82535032134183e-25,
                                         6.96044321578853e-23,
                                         5.05546100444195e-15,
                                         7.22381255317278e-11,
                                         4.07542912425456e-08,
                                         3.93703459033043e-06,
                                         0.000120375796989939,
                                         0.00159259734207893,
                                         0.0108900371085093,
                                         0.0428346496545152,
                                         0.103466162565008,
                                         0.159405810208051,
                                         0.159405810208051,
                                         0.103466162565009,
                                         0.0428346496545152,
                                         0.0108900371085093,
                                         0.00159259734207893,
                                         0.000120375796989939,
                                         3.9370345903304e-06,
                                         4.07542912425453e-08,
                                         7.2238125531729e-11,
                                         5.05546100444195e-15,
                                         6.96044321578853e-23,
                                         1.82535032134183e-25,
                                         7.04266799745768e-17,
                                         2.51007033895737e-12,
                                         2.68675816700138e-09,
                                         4.28320279007379e-07,
                                         1.98948938624902e-05,
                                         0.00037917647865742,
                                         0.00360509909227828,
                                         0.01925498406591,
                                         0.0622006235578973,
                                         0.12706979587895,
                                         0.168030778677296,
                                         0.144618823312481,
                                         0.0800700598168952,
                                         0.0276416772001368,
                                         0.00562073955825647,
                                         0.000612011528049758,
                                         3.04083392917336e-05,
                                         5.194466981023e-07,
                                         1.74690755402392e-09,
                                         3.04934941951112e-13,
                                         2.23023404529155e-20,
                                         4.01699015200026e-28,
                                         8.23301439750439e-19,
                                         7.31897938524621e-14,
                                         1.48637714627739e-10,
                                         3.91032857504079e-08,
                                         2.75923878594637e-06,
                                         7.57568660207307e-05,
                                         0.00100149840512522,
                                         0.00726333903707148,
                                         0.0313788009048586,
                                         0.0850013309868932,
                                         0.148634197110482,
                                         0.169627598583865,
                                         0.1256001896992,
                                         0.0588768587215742,
                                         0.0166466231008254,
                                         0.00261111042379431,
                                         0.000197088544817884,
                                         5.55589262511202e-06,
                                         3.54501915394463e-08,
                                         1.54347225909483e-11,
                                         5.99665682910954e-18,
                                         7.38776603997076e-31,
                                         8.04337580282688e-21,
                                         1.78349945987041e-15,
                                         6.87206544592663e-12,
                                         2.9834289011573e-09,
                                         3.19812012163129e-07,
                                         1.26491236263925e-05,
                                         0.000232509736281094,
                                         0.00228974592947796,
                                         0.0132292667912954,
                                         0.0475189620728414,
                                         0.109876917991544,
                                         0.166274645327531,
                                         0.164652475198452,
                                         0.104805167608503,
                                         0.041201844349702,
                                         0.00930997897838245,
                                         0.00106754917569412,
                                         4.96620363830218e-05,
                                         6.01208449111653e-07,
                                         6.52902438149619e-10,
                                         1.34749070222884e-15,
                                         1.12722277304264e-33,
                                         6.51931701315865e-23,
                                         3.60561784555245e-17,
                                         2.63590533207929e-13,
                                         1.88843650725844e-10,
                                         3.07527892279682e-08,
                                         1.75219779090157e-06,
                                         4.47833192311359e-05,
                                         0.000598856170611169,
                                         0.00462721266783779,
                                         0.0220390271618454,
                                         0.067387361291043,
                                         0.135219647862871,
                                         0.179073155461828,
                                         0.154776498352472,
                                         0.084604102302813,
                                         0.0275395231661227,
                                         0.00479731957741682,
                                         0.000368281122207335,
                                         8.45893755210651e-06,
                                         2.29130027216213e-08,
                                         2.51204035806353e-13,
                                         1.41086579387938e-36,
                                         4.33456149662063e-25,
                                         5.97951192555242e-19,
                                         8.29376374253928e-15,
                                         9.80546992891107e-12,
                                         2.42579212015121e-09,
                                         1.99106367500387e-07,
                                         7.07572172268044e-06,
                                         0.000128480462574618,
                                         0.00132764656160528,
                                         0.00838488718347725,
                                         0.0339023467975504,
                                         0.0902054755775313,
                                         0.159761466522266,
                                         0.187502329011505,
                                         0.142510057639159,
                                         0.0668256915574625,
                                         0.0176843352940187,
                                         0.00224033890804692,
                                         9.76305850207998e-05,
                                         6.59621879114253e-07,
                                         3.84155253794569e-11,
                                         1.42457699220628e-39,
                                         2.32494406888329e-27,
                                         7.99974327258046e-21,
                                         2.10522143207529e-16,
                                         4.10732159258928e-13,
                                         1.54364319208069e-10,
                                         1.82520412453839e-08,
                                         9.01881505711613e-07,
                                         2.22369861647375e-05,
                                         0.000307305095536697,
                                         0.00257351259946928,
                                         0.0137595853014111,
                                         0.0485456445948856,
                                         0.114984119034306,
                                         0.183245191737775,
                                         0.193652844868364,
                                         0.130814166658522,
                                         0.0525899917099656,
                                         0.0109944183037396,
                                         0.000909034228304192,
                                         1.53190689097507e-05,
                                         4.7392674012524e-09,
                                         1.13209094806155e-42,
                                         9.81465460061279e-30,
                                         8.42328574397624e-23,
                                         4.20570760475787e-18,
                                         1.35407953598844e-14,
                                         7.73099572362857e-12,
                                         1.31683967857386e-09,
                                         9.0473915972992e-08,
                                         3.02907432898032e-06,
                                         5.59824909025339e-05,
                                         0.000621656593395294,
                                         0.00439517425341094,
                                         0.0205618796967974,
                                         0.0651326661612536,
                                         0.140946299871425,
                                         0.207108258707117,
                                         0.201540005572385,
                                         0.123087126877534,
                                         0.0424644948240076,
                                         0.00666146500896311,
                                         0.000280004400919604,
                                         4.60162071987028e-07,
                                         6.81844907106935e-46,
                                         3.14012128302307e-32,
                                         6.72195976152361e-25,
                                         6.36779773266833e-20,
                                         3.38328460244832e-16,
                                         2.93449136115348e-13,
                                         7.20050999886844e-11,
                                         6.87869710756707e-09,
                                         3.12717961262259e-07,
                                         7.72936054646191e-06,
                                         0.000113810853628374,
                                         0.00106403457985054,
                                         0.00660061264077535,
                                         0.0279620328219364,
                                         0.0821643950123922,
                                         0.167872610635436,
                                         0.235329647723988,
                                         0.218338869660444,
                                         0.124304988701429,
                                         0.0369971403339191,
                                         0.00387887903475706,
                                         3.38625233526034e-05,
                                         2.92600290354236e-49,
                                         7.15818128834475e-35,
                                         3.82203933136747e-27,
                                         6.86948710884969e-22,
                                         6.02306789063598e-18,
                                         7.93624572794088e-15,
                                         2.80529462885855e-12,
                                         3.72626599495712e-10,
                                         2.30027940159105e-08,
                                         7.60360911322538e-07,
                                         1.48457343823429e-05,
                                         0.000183535574316615,
                                         0.00150969962920019,
                                         0.00855309892104685,
                                         0.0341270314822842,
                                         0.0969498496774296,
                                         0.195783862341312,
                                         0.27595251055029,
                                         0.259260302626284,
                                         0.146403490616592,
                                         0.0382853378134974,
                                         0.00177546732400955,
                                         7.97229473945979e-53,
                                         1.03604449088534e-37,
                                         1.3797926822336e-29,
                                         4.70520779309513e-24,
                                         6.80795108294503e-20,
                                         1.36275197197631e-16,
                                         6.93925975299794e-14,
                                         1.28162497516223e-11,
                                         1.07430549392769e-09,
                                         4.7491451239267e-08,
                                         1.22953000917093e-06,
                                         2.01003773168404e-05,
                                         0.000219238232363169,
                                         0.00166110731418985,
                                         0.00899979892029537,
                                         0.0355495311722224,
                                         0.103418106241186,
                                         0.221440515247638,
                                         0.343323025238822,
                                         0.367835929441674,
                                         0.239926474643256,
                                         0.0591051728986465,
                                         1.03671303300153e-56,
                                         7.15683058084551e-41,
                                         2.37738293490302e-32,
                                         1.53815435938087e-26,
                                         3.67266835446111e-22,
                                         1.11682505971333e-18,
                                         8.19246183715126e-16,
                                         2.10384959603679e-13,
                                         2.3946476560416e-11,
                                         1.41572114133698e-09,
                                         4.86007754129512e-08,
                                         1.05064206037169e-06,
                                         1.51952781262704e-05,
                                         0.00015397084896374,
                                         0.00113274907871766,
                                         0.00622138723037121,
                                         0.0260725127261681,
                                         0.084809877382232,
                                         0.216988369302927,
                                         0.441086212367604,
                                         0.717613301835131,
                                         0.939085032313984,

                                         0.944015506991119,
                                         0.737768448926214,
                                         0.472387035698205,
                                         0.246770613681233,
                                         0.104531669405419,
                                         0.0356063124808474,
                                         0.0096464885524514,
                                         0.00204964209261724,
                                         0.000335532605874507,
                                         4.13810274123364e-05,
                                         3.73734888247977e-06,
                                         2.38418579101562e-07,
                                         1.0254856222942e-08,
                                         2.798275618676e-10,
                                         4.46560308436897e-12,
                                         3.72708730555285e-14,
                                         1.38798005420047e-16,
                                         1.82132073203064e-19,
                                         5.80187013421023e-23,
                                         2.36847984160211e-27,
                                         3.59011205677384e-33,
                                         1.06633076841519e-41,
                                         1.53317609684079e-57,
                                         0.0544585448182853,
                                         0.225932050518268,
                                         0.360377336192393,
                                         0.356523178514995,
                                         0.248601245250778,
                                         0.128229735017781,
                                         0.049843754810393,
                                         0.0146522686508231,
                                         0.00323715891025097,
                                         0.000529941591199573,
                                         6.28851947169417e-05,
                                         5.24520874023438e-06,
                                         2.94978368100484e-07,
                                         1.05756997848856e-08,
                                         2.24024847998075e-10,
                                         2.52341154798812e-12,
                                         1.30013092659797e-14,
                                         2.447766838956e-17,
                                         1.1807499933167e-20,
                                         7.93452121671614e-25,
                                         2.27768606804535e-30,
                                         1.68530898782023e-38,
                                         1.28632436903063e-53,
                                         0.00149940707148918,
                                         0.0330219129062705,
                                         0.131215031664206,
                                         0.245837831609885,
                                         0.282179416692838,
                                         0.220402800756615,
                                         0.122918957132079,
                                         0.0499917491712016,
                                         0.0149059591555265,
                                         0.00323907765364191,
                                         0.000505009789618325,
                                         5.50746917724609e-05,
                                         4.04964897265809e-06,
                                         1.90763093939385e-07,
                                         5.36387788156287e-09,
                                         8.15404726681556e-11,
                                         5.81242779992683e-13,
                                         1.57007457484216e-15,
                                         1.1468708601243e-18,
                                         1.26863993144706e-22,
                                         6.89677955559074e-28,
                                         1.27125815331347e-35,
                                         5.15081050927877e-50,
                                         2.62115388670442e-05,
                                         0.00306440423053571,
                                         0.0303339628288097,
                                         0.107628950476802,
                                         0.203360595487314,
                                         0.240527598424115,
                                         0.19246263384849,
                                         0.108295705340449,
                                         0.0435788009248367,
                                         0.0125699675425758,
                                         0.00257496082340743,
                                         0.000367164611816406,
                                         3.52991325299525e-05,
                                         2.18473639251926e-06,
                                         8.15419351927656e-08,
                                         1.67293013723155e-09,
                                         1.64986152673711e-11,
                                         6.39425507295354e-14,
                                         7.0727861414272e-17,
                                         1.28788021132487e-20,
                                         1.32592288331517e-25,
                                         6.08845930555247e-33,
                                         1.30954398717105e-46,
                                         3.26475316660372e-07,
                                         0.000202616471274099,
                                         0.00499642738012471,
                                         0.033573325633921,
                                         0.104422274135625,
                                         0.187024107003682,
                                         0.214713252806223,
                                         0.167151009768692,
                                         0.0907769307254869,
                                         0.0347561626806554,
                                         0.00935462372631356,
                                         0.00174403190612793,
                                         0.000219227764089728,
                                         1.78274241465661e-05,
                                         8.83218210971601e-07,
                                         2.44549767267879e-08,
                                         3.33673938870626e-10,
                                         1.85542968771072e-12,
                                         3.10778813015808e-15,
                                         9.31531257053823e-19,
                                         1.81624760992975e-23,
                                         2.07761911621637e-30,
                                         2.37219003497253e-43,
                                         3.08188958944179e-09,
                                         1.01534195700867e-05,
                                         0.000623733255799859,
                                         0.00793722170546002,
                                         0.0406376308151381,
                                         0.110214606807697,
                                         0.181543276363333,
                                         0.19553102007313,
                                         0.14331267553762,
                                         0.0728347070058405,
                                         0.0257567410739448,
                                         0.00627851486206055,
                                         0.00103189609114078,
                                         0.000110252143731418,
                                         7.25043239305283e-06,
                                         2.70935300786639e-07,
                                         5.11453284428203e-09,
                                         4.08044883788186e-11,
                                         1.03495429643662e-13,
                                         5.10655818821466e-17,
                                         1.88556165107945e-21,
                                         5.37320423913704e-28,
                                         3.25677499339144e-40,
                                         2.28970203454349e-11,
                                         4.00447040686263e-07,
                                         6.12820649550382e-05,
                                         0.00147685490639534,
                                         0.0124468307501367,
                                         0.0511182398915823,
                                         0.120808267700739,
                                         0.180018648806471,
                                         0.17806923118069,
                                         0.120126854122498,
                                         0.0558149668405981,
                                         0.0177891254425049,
                                         0.00382271174064852,
                                         0.000536637055734312,
                                         4.68441075806468e-05,
                                         2.36243084464879e-06,
                                         6.16999519723273e-08,
                                         7.0626314299593e-10,
                                         2.71260220328816e-12,
                                         2.20320195440215e-15,
                                         1.54064148547777e-19,
                                         1.0936942656127e-25,
                                         3.51901568414296e-37,
                                         1.37235078768904e-13,
                                         1.27409589321672e-08,
                                         4.85726897120237e-06,
                                         0.000221682505145913,
                                         0.00307548391074025,
                                         0.0191265614223439,
                                         0.0648540969520933,
                                         0.133703753943198,
                                         0.178491457095282,
                                         0.159832871535116,
                                         0.0975741331382538,
                                         0.0406608581542969,
                                         0.0114243475084681,
                                         0.00210716500650294,
                                         0.000244157631720691,
                                         1.66179285079588e-05,
                                         6.00465350440715e-07,
                                         9.86164626238487e-09,
                                         5.73555330302352e-11,
                                         7.66839725407651e-14,
                                         1.01551586129519e-17,
                                         1.79590265727841e-23,
                                         3.06746009664984e-34,
                                         6.7473094917912e-16,
                                         3.32535850013373e-10,
                                         3.15813175846582e-07,
                                         2.72963350607363e-05,
                                         0.00062337224812819,
                                         0.00587052898476023,
                                         0.0285599541199831,
                                         0.0814608680557094,
                                         0.146765951853178,
                                         0.174450183049864,
                                         0.13992588475299,
                                         0.0762391090393066,
                                         0.0280072578159825,
                                         0.0067872799412389,
                                         0.00104391458940547,
                                         9.58901499259809e-05,
                                         4.79369509253835e-06,
                                         1.12956604072732e-07,
                                         9.94818515344349e-10,
                                         2.18944594427651e-12,
                                         5.49099645533279e-16,
                                         2.41907303489218e-21,
                                         2.19339056348316e-31,
                                         2.75220247275227e-18,
                                         7.2004407894713e-12,
                                         1.70354100208228e-08,
                                         2.78844135128689e-06,
                                         0.000104825205599983,
                                         0.00149486449859887,
                                         0.0104342646692876,
                                         0.04117548473495,
                                         0.100119218363924,
                                         0.15796505067704,
                                         0.166473716733503,
                                         0.118594169616699,
                                         0.0569631539659596,
                                         0.0181374919563611,
                                         0.00370291620931391,
                                         0.000459045107695057,
                                         3.17495165154623e-05,
                                         1.07339134317192e-06,
                                         1.43151747281587e-08,
                                         5.18618620580027e-11,
                                         2.4632011001417e-14,
                                         2.70333222883644e-19,
                                         1.30117968368082e-28,
                                         9.38183857737204e-21,
                                         1.30297931241095e-13,
                                         7.6794967862509e-10,
                                         2.38054565309409e-07,
                                         1.4731324892981e-05,
                                         0.000318115071513216,
                                         0.00318584041826249,
                                         0.0173934700430852,
                                         0.0570778244365269,
                                         0.119538669824615,
                                         0.165520243306086,
                                         0.154172420501709,
                                         0.0968222770109614,
                                         0.0405057361773674,
                                         0.0109769232717593,
                                         0.00183651531720611,
                                         0.000175736370083511,
                                         8.52437164136696e-06,
                                         1.72150097497455e-07,
                                         1.02664385072614e-09,
                                         9.23435151608264e-13,
                                         2.52468784821009e-17,
                                         6.45084196836875e-26,
                                         2.68374186123874e-23,
                                         1.97861440218767e-15,
                                         2.90508031522732e-11,
                                         1.70544109337967e-08,
                                         1.73725312388313e-06,
                                         5.68083102391121e-05,
                                         0.00081626541437355,
                                         0.006165651238185,
                                         0.0273062826161158,
                                         0.075910357229445,
                                         0.138102571359707,
                                         0.168188095092773,
                                         0.138102571359707,
                                         0.075910357229445,
                                         0.0273062826161158,
                                         0.00616565123818501,
                                         0.000816265414373548,
                                         5.68083102391124e-05,
                                         1.73725312388313e-06,
                                         1.70544109337967e-08,
                                         2.90508031522738e-11,
                                         1.97861440218767e-15,
                                         2.68374186123811e-23,
                                         6.45084196837039e-26,
                                         2.52468784821009e-17,
                                         9.23435151608245e-13,
                                         1.02664385072614e-09,
                                         1.72150097497455e-07,
                                         8.52437164136692e-06,
                                         0.000175736370083512,
                                         0.00183651531720611,
                                         0.0109769232717593,
                                         0.0405057361773674,
                                         0.0968222770109614,
                                         0.154172420501709,
                                         0.165520243306086,
                                         0.119538669824615,
                                         0.0570778244365269,
                                         0.0173934700430853,
                                         0.00318584041826248,
                                         0.000318115071513218,
                                         1.4731324892981e-05,
                                         2.38054565309409e-07,
                                         7.67949678625103e-10,
                                         1.30297931241095e-13,
                                         9.38183857737005e-21,
                                         1.30117968368118e-28,
                                         2.70333222883644e-19,
                                         2.46320110014164e-14,
                                         5.18618620580027e-11,
                                         1.43151747281587e-08,
                                         1.07339134317191e-06,
                                         3.17495165154624e-05,
                                         0.000459045107695055,
                                         0.00370291620931391,
                                         0.0181374919563611,
                                         0.0569631539659596,
                                         0.118594169616699,
                                         0.166473716733503,
                                         0.15796505067704,
                                         0.100119218363924,
                                         0.0411754847349501,
                                         0.0104342646692876,
                                         0.00149486449859888,
                                         0.000104825205599983,
                                         2.78844135128689e-06,
                                         1.7035410020823e-08,
                                         7.20044078947129e-12,
                                         2.75220247275174e-18,
                                         2.19339056348381e-31,
                                         2.41907303489218e-21,
                                         5.49099645533266e-16,
                                         2.18944594427651e-12,
                                         9.94818515344349e-10,
                                         1.12956604072732e-07,
                                         4.79369509253837e-06,
                                         9.58901499259806e-05,
                                         0.00104391458940547,
                                         0.0067872799412389,
                                         0.0280072578159825,
                                         0.0762391090393066,
                                         0.13992588475299,
                                         0.174450183049864,
                                         0.146765951853178,
                                         0.0814608680557096,
                                         0.028559954119983,
                                         0.00587052898476024,
                                         0.00062337224812819,
                                         2.72963350607363e-05,
                                         3.15813175846586e-07,
                                         3.32535850013373e-10,
                                         6.74730949179005e-16,
                                         3.06746009665081e-34,
                                         1.79590265727841e-23,
                                         1.01551586129517e-17,
                                         7.66839725407651e-14,
                                         5.73555330302352e-11,
                                         9.86164626238481e-09,
                                         6.00465350440717e-07,
                                         1.66179285079588e-05,
                                         0.000244157631720691,
                                         0.00210716500650294,
                                         0.0114243475084681,
                                         0.0406608581542969,
                                         0.0975741331382538,
                                         0.159832871535116,
                                         0.178491457095282,
                                         0.133703753943199,
                                         0.0648540969520932,
                                         0.019126561422344,
                                         0.00307548391074025,
                                         0.000221682505145913,
                                         4.85726897120242e-06,
                                         1.27409589321672e-08,
                                         1.37235078768883e-13,
                                         3.51901568414416e-37,
                                         1.0936942656127e-25,
                                         1.54064148547773e-19,
                                         2.20320195440215e-15,
                                         2.71260220328816e-12,
                                         7.06263142995925e-10,
                                         6.16999519723276e-08,
                                         2.36243084464878e-06,
                                         4.68441075806468e-05,
                                         0.000536637055734312,
                                         0.00382271174064852,
                                         0.0177891254425049,
                                         0.0558149668405981,
                                         0.120126854122498,
                                         0.17806923118069,
                                         0.180018648806471,
                                         0.120808267700739,
                                         0.0511182398915824,
                                         0.0124468307501367,
                                         0.00147685490639534,
                                         6.12820649550388e-05,
                                         4.00447040686263e-07,
                                         2.2897020345432e-11,
                                         3.25677499339262e-40,
                                         5.37320423913704e-28,
                                         1.88556165107939e-21,
                                         5.10655818821466e-17,
                                         1.03495429643662e-13,
                                         4.08044883788184e-11,
                                         5.11453284428206e-09,
                                         2.70935300786638e-07,
                                         7.25043239305283e-06,
                                         0.000110252143731418,
                                         0.00103189609114078,
                                         0.00627851486206055,
                                         0.0257567410739448,
                                         0.0728347070058405,
                                         0.14331267553762,
                                         0.19553102007313,
                                         0.181543276363332,
                                         0.110214606807698,
                                         0.0406376308151381,
                                         0.00793722170546002,
                                         0.000623733255799864,
                                         1.01534195700867e-05,
                                         3.08188958944146e-09,
                                         2.37219003497344e-43,
                                         2.07761911621637e-30,
                                         1.8162476099297e-23,
                                         9.31531257053823e-19,
                                         3.10778813015808e-15,
                                         1.85542968771071e-12,
                                         3.33673938870627e-10,
                                         2.44549767267878e-08,
                                         8.83218210971601e-07,
                                         1.78274241465661e-05,
                                         0.000219227764089728,
                                         0.00174403190612793,
                                         0.00935462372631356,
                                         0.0347561626806554,
                                         0.0907769307254869,
                                         0.167151009768692,
                                         0.214713252806222,
                                         0.187024107003683,
                                         0.104422274135625,
                                         0.033573325633921,
                                         0.00499642738012474,
                                         0.000202616471274099,
                                         3.26475316660344e-07,
                                         1.30954398717158e-46,
                                         6.08845930555247e-33,
                                         1.32592288331513e-25,
                                         1.28788021132487e-20,
                                         7.0727861414272e-17,
                                         6.39425507295349e-14,
                                         1.64986152673712e-11,
                                         1.67293013723155e-09,
                                         8.15419351927656e-08,
                                         2.18473639251926e-06,
                                         3.52991325299525e-05,
                                         0.000367164611816406,
                                         0.00257496082340743,
                                         0.0125699675425758,
                                         0.0435788009248367,
                                         0.108295705340449,
                                         0.19246263384849,
                                         0.240527598424115,
                                         0.203360595487314,
                                         0.107628950476802,
                                         0.0303339628288098,
                                         0.00306440423053571,
                                         2.62115388670426e-05,
                                         5.15081050928095e-50,
                                         1.27125815331347e-35,
                                         6.89677955559051e-28,
                                         1.26863993144706e-22,
                                         1.1468708601243e-18,
                                         1.57007457484215e-15,
                                         5.81242779992686e-13,
                                         8.15404726681552e-11,
                                         5.36387788156287e-09,
                                         1.90763093939385e-07,
                                         4.04964897265809e-06,
                                         5.50746917724609e-05,
                                         0.000505009789618325,
                                         0.00323907765364191,
                                         0.0149059591555265,
                                         0.0499917491712016,
                                         0.122918957132079,
                                         0.220402800756615,
                                         0.282179416692838,
                                         0.245837831609885,
                                         0.131215031664207,
                                         0.0330219129062705,
                                         0.00149940707148912,
                                         1.28632436903121e-53,
                                         1.68530898782023e-38,
                                         2.27768606804527e-30,
                                         7.93452121671614e-25,
                                         1.1807499933167e-20,
                                         2.44776683895598e-17,
                                         1.30013092659798e-14,
                                         2.5234115479881e-12,
                                         2.24024847998075e-10,
                                         1.05756997848856e-08,
                                         2.94978368100484e-07,
                                         5.24520874023438e-06,
                                         6.28851947169417e-05,
                                         0.000529941591199573,
                                         0.00323715891025097,
                                         0.0146522686508231,
                                         0.0498437548103929,
                                         0.128229735017781,
                                         0.248601245250778,
                                         0.356523178514995,
                                         0.360377336192393,
                                         0.225932050518268,
                                         0.0544585448182841,
                                         1.5331760968415e-57,
                                         1.06633076841519e-41,
                                         3.59011205677371e-33,
                                         2.36847984160211e-27,
                                         5.80187013421023e-23,
                                         1.82132073203062e-19,
                                         1.38798005420047e-16,
                                         3.72708730555284e-14,
                                         4.46560308436897e-12,
                                         2.798275618676e-10,
                                         1.0254856222942e-08,
                                         2.38418579101562e-07,
                                         3.73734888247977e-06,
                                         4.13810274123364e-05,
                                         0.000335532605874507,
                                         0.00204964209261724,
                                         0.0096464885524514,
                                         0.0356063124808474,
                                         0.104531669405419,
                                         0.246770613681233,
                                         0.472387035698205,
                                         0.737768448926214,
                                         0.944015506991119,

                                         0.948375858352993,
                                         0.755967346136266,
                                         0.501737498528049,
                                         0.27628802932601,
                                         0.125580167171418,
                                         0.0467848756096965,
                                         0.0141560341057376,
                                         0.0034385299854571,
                                         0.000660762678317973,
                                         9.86317532512293e-05,
                                         1.11776982902485e-05,
                                         9.34473051924655e-07,
                                         5.5568413843158e-08,
                                         2.24286748580844e-09,
                                         5.78012655229896e-11,
                                         8.76570160240904e-13,
                                         6.99417193198408e-15,
                                         2.50458814860317e-17,
                                         3.17825595196142e-20,
                                         9.84531896290659e-24,
                                         3.92968172428723e-28,
                                         5.85536011897328e-34,
                                         1.71870516889983e-42,
                                         2.45500464086367e-58,
                                         0.0503286784589146,
                                         0.212837608079296,
                                         0.351518509344708,
                                         0.365987585793505,
                                         0.273237050868461,
                                         0.153714121932079,
                                         0.0664981396571251,
                                         0.0222514067954123,
                                         0.0057405885579945,
                                         0.00113022028397412,
                                         0.000166995460299519,
                                         1.80831491132346e-05,
                                         1.38984407613639e-06,
                                         7.26602176606621e-08,
                                         2.44138552096679e-09,
                                         4.88338683439984e-11,
                                         5.23114800975173e-13,
                                         2.58055896761444e-15,
                                         4.68194104937566e-18,
                                         2.19006306021604e-21,
                                         1.43581489052379e-25,
                                         4.04508204237324e-31,
                                         2.95461849328091e-39,
                                         2.23904407415177e-54,
                                         0.00127472699091856,
                                         0.0285996238201321,
                                         0.117540207699291,
                                         0.23138609592217,
                                         0.283742727966147,
                                         0.241039754467105,
                                         0.149088453346277,
                                         0.068724050657013,
                                         0.0238031238041668,
                                         0.00618124645281448,
                                         0.00119075815839743,
                                         0.000167012110212021,
                                         1.65909318187873e-05,
                                         1.12345683839903e-06,
                                         4.92155111803376e-08,
                                         1.2984409141061e-09,
                                         1.86734399949155e-11,
                                         1.26898904217077e-13,
                                         3.29177112702556e-16,
                                         2.32514513984834e-19,
                                         2.50383754359437e-23,
                                         1.33372925264707e-28,
                                         2.42419783343667e-36,
                                         9.74629574786759e-51,
                                         2.04992644622079e-05,
                                         0.00244001060441641,
                                         0.0249542287337779,
                                         0.092881154364274,
                                         0.187080849075462,
                                         0.23998440784475,
                                         0.212225695199587,
                                         0.134765729635289,
                                         0.06266584955698,
                                         0.0214638914859434,
                                         0.00539090784881093,
                                         0.000979357616495393,
                                         0.000125746214174552,
                                         1.10289846323882e-05,
                                         6.29922437371704e-07,
                                         2.19201083910306e-08,
                                         4.23224786573214e-10,
                                         3.96206319529967e-12,
                                         1.46944306617609e-14,
                                         1.56733906022912e-17,
                                         2.77225568763317e-21,
                                         2.79207744784281e-26,
                                         1.26285691775964e-33,
                                         2.69361778520531e-47,
                                         2.34879029486912e-07,
                                         0.000148322820548777,
                                         0.00377473717308475,
                                         0.0265645717439568,
                                         0.0878858168222979,
                                         0.170240248322707,
                                         0.215246842348117,
                                         0.188293380828584,
                                         0.11754733211771,
                                         0.0531038168242414,
                                         0.0173894208920284,
                                         0.00409184821739011,
                                         0.000679053399064987,
                                         7.71435356731567e-05,
                                         5.74456336338475e-06,
                                         2.63662321474436e-07,
                                         6.83443673277841e-09,
                                         8.81393398393343e-11,
                                         4.67369927769678e-13,
                                         7.52767247699463e-16,
                                         2.1869823992013e-19,
                                         4.1645886728788e-24,
                                         4.68732541368806e-31,
                                         5.30416743379946e-44,
                                         2.03966620558421e-09,
                                         6.833341548047e-06,
                                         0.000432751095154941,
                                         0.00575820237869717,
                                         0.0312908345153237,
                                         0.0915272372754912,
                                         0.165456755738169,
                                         0.199388235843834,
                                         0.167110413116139,
                                         0.0995753636541077,
                                         0.0425125564351456,
                                         0.0129570643982921,
                                         0.00277921296689129,
                                         0.000408952096589205,
                                         3.97041462153992e-05,
                                         2.40360078412794e-06,
                                         8.36456290494715e-08,
                                         1.48602829755372e-09,
                                         1.12662051985767e-11,
                                         2.7401060036839e-14,
                                         1.30757372084428e-17,
                                         4.70788295945718e-22,
                                         1.31857535351412e-28,
                                         7.91602887568656e-41,
                                         1.394020393059e-11,
                                         2.47772698932013e-07,
                                         3.90467420509383e-05,
                                         0.000982349889230029,
                                         0.00876820446053721,
                                         0.0387287636527381,
                                         0.100098458634075,
                                         0.166172509501436,
                                         0.186977530340396,
                                         0.146951233379165,
                                         0.0817983406880589,
                                         0.0322915498053176,
                                         0.00895230556879614,
                                         0.00170624153030604,
                                         0.00021597817602873,
                                         1.72453404590051e-05,
                                         8.05710356128847e-07,
                                         1.97187570375458e-08,
                                         2.13741926480866e-10,
                                         7.84999144971413e-13,
                                         6.15293390300294e-16,
                                         4.18865288448947e-20,
                                         2.91930837127723e-26,
                                         9.2980672921563e-38,
                                         7.68605389634342e-14,
                                         7.24766391551623e-09,
                                         2.8422069880216e-06,
                                         0.000135197838431905,
                                         0.00198211323991894,
                                         0.0132202961632634,
                                         0.0488533722177471,
                                         0.111723130939847,
                                         0.168771689198626,
                                         0.174951971777599,
                                         0.126968514368101,
                                         0.0649225295307821,
                                         0.023263349174227,
                                         0.00574292110066929,
                                         0.000947781327821705,
                                         9.9817222242423e-05,
                                         6.26093091618544e-06,
                                         2.11084461056483e-07,
                                         3.2713427829411e-09,
                                         1.8142422426996e-11,
                                         2.33572972384426e-14,
                                         3.00640411706952e-18,
                                         5.21409888378574e-24,
                                         8.81053481795197e-35,
                                         3.4762984386306e-16,
                                         1.73908965006118e-10,
                                         1.69709413188516e-07,
                                         1.52634470725723e-05,
                                         0.000367557751491951,
                                         0.00370192882959087,
                                         0.0195587472238445,
                                         0.0616178224857,
                                         0.124965204601795,
                                         0.170861327764583,
                                         0.161669059395226,
                                         0.107073329008262,
                                         0.0495894064990536,
                                         0.0158563973486466,
                                         0.00341181722247244,
                                         0.000473934709977888,
                                         3.99096764630396e-05,
                                         1.85358421106521e-06,
                                         4.10716089984461e-08,
                                         3.43954262073721e-10,
                                         7.27348009438574e-13,
                                         1.77010762443457e-16,
                                         7.63937595021788e-22,
                                         6.84843263375173e-32,
                                         1.30441245028708e-18,
                                         3.46202459445771e-12,
                                         8.40698775483757e-09,
                                         1.42961717242906e-06,
                                         5.65466607824512e-05,
                                         0.000860001482621011,
                                         0.00649638529690099,
                                         0.0281938152582928,
                                         0.0767649314028795,
                                         0.138437250932853,
                                         0.170781978002875,
                                         0.146504657872303,
                                         0.0876980296419197,
                                         0.0363212214580885,
                                         0.0101893767389718,
                                         0.00186687743998406,
                                         0.000211057969956482,
                                         1.35036954000301e-05,
                                         4.27800841465725e-07,
                                         5.40991410885876e-09,
                                         1.87908391906834e-11,
                                         8.64641829926765e-15,
                                         9.28583061322154e-20,
                                         4.416358889637e-29,
                                         4.09044580954375e-21,
                                         5.75965079607018e-14,
                                         3.48042856168295e-10,
                                         1.11903768829647e-07,
                                         7.270196810122e-06,
                                         0.00016696604535918,
                                         0.00180326827276028,
                                         0.0107810032934843,
                                         0.0394089127970681,
                                         0.0937389279155844,
                                         0.150770020205945,
                                         0.167524907083942,
                                         0.129613007334677,
                                         0.0695303148477226,
                                         0.0254312266797874,
                                         0.00614569388685016,
                                         0.000932788390418547,
                                         8.22149450433975e-05,
                                         3.72391173043084e-06,
                                         7.11111776313084e-08,
                                         4.05702723654472e-10,
                                         3.52964182990431e-13,
                                         9.4328180511291e-18,
                                         2.38010082594489e-26,
                                         1.0763947234333e-23,
                                         8.04095008552241e-16,
                                         1.20912232979428e-11,
                                         7.35046610508358e-09,
                                         7.84387348317137e-07,
                                         2.72020935851316e-05,
                                         0.000420043353526559,
                                         0.00345947117504686,
                                         0.0169774026970483,
                                         0.053263807284088,
                                         0.111694851584571,
                                         0.160750579591725,
                                         0.160750579591725,
                                         0.111694851584571,
                                         0.0532638072840881,
                                         0.0169774026970483,
                                         0.00345947117504685,
                                         0.000420043353526559,
                                         2.72020935851314e-05,
                                         7.84387348317137e-07,
                                         7.35046610508358e-09,
                                         1.20912232979426e-11,
                                         8.04095008552241e-16,
                                         1.07639472343303e-23,
                                         2.38010082594555e-26,
                                         9.4328180511291e-18,
                                         3.52964182990438e-13,
                                         4.05702723654472e-10,
                                         7.11111776313084e-08,
                                         3.72391173043086e-06,
                                         8.22149450433976e-05,
                                         0.000932788390418549,
                                         0.00614569388685016,
                                         0.0254312266797874,
                                         0.0695303148477226,
                                         0.129613007334677,
                                         0.167524907083942,
                                         0.150770020205945,
                                         0.0937389279155844,
                                         0.0394089127970681,
                                         0.0107810032934843,
                                         0.00180326827276028,
                                         0.000166966045359179,
                                         7.270196810122e-06,
                                         1.11903768829647e-07,
                                         3.48042856168288e-10,
                                         5.75965079607018e-14,
                                         4.0904458095428e-21,
                                         4.41635888963833e-29,
                                         9.28583061322154e-20,
                                         8.64641829926785e-15,
                                         1.87908391906834e-11,
                                         5.40991410885876e-09,
                                         4.27800841465727e-07,
                                         1.35036954000301e-05,
                                         0.000211057969956483,
                                         0.00186687743998406,
                                         0.0101893767389718,
                                         0.0363212214580885,
                                         0.0876980296419195,
                                         0.146504657872303,
                                         0.170781978002875,
                                         0.138437250932853,
                                         0.0767649314028795,
                                         0.0281938152582927,
                                         0.00649638529690099,
                                         0.000860001482621008,
                                         5.65466607824512e-05,
                                         1.42961717242906e-06,
                                         8.40698775483744e-09,
                                         3.46202459445771e-12,
                                         1.3044124502868e-18,
                                         6.84843263375395e-32,
                                         7.63937595021788e-22,
                                         1.77010762443462e-16,
                                         7.27348009438574e-13,
                                         3.43954262073721e-10,
                                         4.10716089984464e-08,
                                         1.85358421106521e-06,
                                         3.99096764630397e-05,
                                         0.000473934709977888,
                                         0.00341181722247244,
                                         0.0158563973486466,
                                         0.0495894064990535,
                                         0.107073329008262,
                                         0.161669059395226,
                                         0.170861327764583,
                                         0.124965204601795,
                                         0.0616178224856999,
                                         0.0195587472238445,
                                         0.00370192882959086,
                                         0.000367557751491951,
                                         1.52634470725723e-05,
                                         1.69709413188514e-07,
                                         1.73908965006118e-10,
                                         3.47629843862996e-16,
                                         8.81053481795502e-35,
                                         5.21409888378574e-24,
                                         3.0064041170696e-18,
                                         2.33572972384426e-14,
                                         1.8142422426996e-11,
                                         3.27134278294112e-09,
                                         2.11084461056483e-07,
                                         6.26093091618547e-06,
                                         9.9817222242423e-05,
                                         0.000947781327821705,
                                         0.00574292110066929,
                                         0.023263349174227,
                                         0.0649225295307822,
                                         0.126968514368101,
                                         0.174951971777599,
                                         0.168771689198626,
                                         0.111723130939847,
                                         0.0488533722177471,
                                         0.0132202961632634,
                                         0.00198211323991894,
                                         0.000135197838431905,
                                         2.84220698802157e-06,
                                         7.24766391551623e-09,
                                         7.68605389634218e-14,
                                         9.29806729215973e-38,
                                         2.91930837127723e-26,
                                         4.18865288448959e-20,
                                         6.15293390300294e-16,
                                         7.84999144971413e-13,
                                         2.13741926480868e-10,
                                         1.97187570375458e-08,
                                         8.0571035612885e-07,
                                         1.72453404590051e-05,
                                         0.00021597817602873,
                                         0.00170624153030604,
                                         0.00895230556879613,
                                         0.0322915498053176,
                                         0.0817983406880589,
                                         0.146951233379165,
                                         0.186977530340396,
                                         0.166172509501435,
                                         0.100098458634075,
                                         0.038728763652738,
                                         0.00876820446053721,
                                         0.000982349889230029,
                                         3.90467420509379e-05,
                                         2.47772698932013e-07,
                                         1.39402039305881e-11,
                                         7.91602887568967e-41,
                                         1.31857535351412e-28,
                                         4.70788295945733e-22,
                                         1.30757372084428e-17,
                                         2.7401060036839e-14,
                                         1.12662051985768e-11,
                                         1.48602829755372e-09,
                                         8.36456290494719e-08,
                                         2.40360078412794e-06,
                                         3.97041462153992e-05,
                                         0.000408952096589205,
                                         0.00277921296689129,
                                         0.0129570643982921,
                                         0.0425125564351456,
                                         0.0995753636541077,
                                         0.167110413116139,
                                         0.199388235843834,
                                         0.165456755738169,
                                         0.091527237275491,
                                         0.0312908345153237,
                                         0.00575820237869717,
                                         0.000432751095154937,
                                         6.833341548047e-06,
                                         2.03966620558398e-09,
                                         5.30416743380167e-44,
                                         4.68732541368806e-31,
                                         4.16458867287893e-24,
                                         2.1869823992013e-19,
                                         7.52767247699463e-16,
                                         4.67369927769682e-13,
                                         8.81393398393343e-11,
                                         6.83443673277844e-09,
                                         2.63662321474436e-07,
                                         5.74456336338475e-06,
                                         7.71435356731567e-05,
                                         0.000679053399064985,
                                         0.00409184821739011,
                                         0.0173894208920284,
                                         0.0531038168242414,
                                         0.11754733211771,
                                         0.188293380828584,
                                         0.215246842348117,
                                         0.170240248322707,
                                         0.0878858168222979,
                                         0.0265645717439568,
                                         0.00377473717308472,
                                         0.000148322820548777,
                                         2.34879029486891e-07,
                                         2.69361778520649e-47,
                                         1.26285691775964e-33,
                                         2.79207744784291e-26,
                                         2.77225568763317e-21,
                                         1.56733906022912e-17,
                                         1.4694430661761e-14,
                                         3.96206319529967e-12,
                                         4.23224786573216e-10,
                                         2.19201083910306e-08,
                                         6.29922437371704e-07,
                                         1.10289846323882e-05,
                                         0.000125746214174551,
                                         0.000979357616495394,
                                         0.00539090784881093,
                                         0.0214638914859434,
                                         0.06266584955698,
                                         0.134765729635289,
                                         0.212225695199587,
                                         0.23998440784475,
                                         0.187080849075462,
                                         0.092881154364274,
                                         0.0249542287337778,
                                         0.00244001060441641,
                                         2.04992644622065e-05,
                                         9.74629574787208e-51,
                                         2.42419783343667e-36,
                                         1.33372925264712e-28,
                                         2.50383754359437e-23,
                                         2.32514513984834e-19,
                                         3.29177112702559e-16,
                                         1.26898904217077e-13,
                                         1.86734399949156e-11,
                                         1.2984409141061e-09,
                                         4.92155111803376e-08,
                                         1.12345683839903e-06,
                                         1.65909318187872e-05,
                                         0.000167012110212022,
                                         0.00119075815839743,
                                         0.00618124645281448,
                                         0.0238031238041668,
                                         0.068724050657013,
                                         0.149088453346277,
                                         0.241039754467104,
                                         0.283742727966147,
                                         0.23138609592217,
                                         0.117540207699291,
                                         0.0285996238201321,
                                         0.0012747269909185,
                                         2.23904407415285e-54,
                                         2.95461849328091e-39,
                                         4.04508204237339e-31,
                                         1.43581489052379e-25,
                                         2.19006306021604e-21,
                                         4.6819410493757e-18,
                                         2.58055896761444e-15,
                                         5.23114800975176e-13,
                                         4.88338683439984e-11,
                                         2.44138552096679e-09,
                                         7.26602176606621e-08,
                                         1.38984407613638e-06,
                                         1.80831491132346e-05,
                                         0.000166995460299519,
                                         0.00113022028397412,
                                         0.0057405885579945,
                                         0.0222514067954123,
                                         0.0664981396571251,
                                         0.153714121932079,
                                         0.273237050868461,
                                         0.365987585793505,
                                         0.351518509344707,
                                         0.212837608079296,
                                         0.0503286784589134,
                                         2.45500464086492e-58,
                                         1.71870516889983e-42,
                                         5.85536011897351e-34,
                                         3.92968172428723e-28,
                                         9.84531896290659e-24,
                                         3.17825595196145e-20,
                                         2.50458814860317e-17,
                                         6.99417193198412e-15,
                                         8.76570160240904e-13,
                                         5.78012655229896e-11,
                                         2.24286748580844e-09,
                                         5.55684138431579e-08,
                                         9.34473051924655e-07,
                                         1.11776982902485e-05,
                                         9.86317532512293e-05,
                                         0.000660762678317973,
                                         0.0034385299854571,
                                         0.0141560341057376,
                                         0.0467848756096965,
                                         0.125580167171418,
                                         0.27628802932601,
                                         0.501737498528049,
                                         0.755967346136266,
                                         0.948375858352993,

                                         1.19209289550781e-07,
                                         2.74181365966797e-06,
                                         3.01599502563477e-05,
                                         0.000211119651794434,
                                         0.00105559825897217,
                                         0.00401127338409424,
                                         0.0120338201522827,
                                         0.0292249917984009,
                                         0.0584499835968018,
                                         0.0974166393280029,
                                         0.136383295059204,
                                         0.161180257797241,
                                         0.161180257797241,
                                         0.136383295059204,
                                         0.0974166393280029,
                                         0.0584499835968018,
                                         0.0292249917984009,
                                         0.0120338201522827,
                                         0.00401127338409424,
                                         0.00105559825897217,
                                         0.000211119651794434,
                                         3.01599502563477e-05,
                                         2.74181365966797e-06,
                                         1.19209289550781e-07,

                                         0.00425262822419346,
                                         2.97758264135974e-16,
                                         0.0262082308627961,
                                         2.55587263129663e-14,
                                         0.0772472172420809,
                                         1.04925111694412e-12,
                                         0.144888306342969,
                                         2.74110093483369e-11,
                                         0.194113523386559,
                                         5.11496397873694e-10,
                                         0.197647734959275,
                                         7.25393607006175e-09,
                                         0.158878652904535,
                                         8.1216173904981e-08,
                                         0.103387702083622,
                                         7.36107012280794e-07,
                                         0.0554053025612318,
                                         5.49437753927931e-06,
                                         0.0247430101294734,
                                         3.41754935542594e-05,
                                         0.0092818174155105,
                                         0.000178562549645534,
                                         0.00293924738637538,
                                         0.00078756896353457,
                                         0.00078756896353457,
                                         0.00293924738637538,
                                         0.000178562549645534,
                                         0.00928181741551049,
                                         3.41754935542594e-05,
                                         0.0247430101294734,
                                         5.49437753927931e-06,
                                         0.0554053025612318,
                                         7.36107012280794e-07,
                                         0.103387702083622,
                                         8.1216173904981e-08,
                                         0.158878652904535,
                                         7.25393607006175e-09,
                                         0.197647734959275,
                                         5.11496397873694e-10,
                                         0.194113523386559,
                                         2.74110093483369e-11,
                                         0.144888306342969,
                                         1.04925111694412e-12,
                                         0.0772472172420809,
                                         2.55587263129663e-14,
                                         0.0262082308627961,
                                         2.97758264135974e-16,
                                         0.00425262822419346,

                                         0.0639149313699997,
                                         1.19209289550781e-07,
                                         1.56457963509512e-22,
                                         0.186719996329999,
                                         2.74181365966797e-06,
                                         2.8331191645114e-20,
                                         0.26088204043,
                                         3.01599502563477e-05,
                                         2.45356100000212e-18,
                                         0.23195454667,
                                         0.000211119651794434,
                                         1.35217914243445e-16,
                                         0.147310451749995,
                                         0.00105559825897217,
                                         5.3228419347377e-15,
                                         0.0711013464698411,
                                         0.00401127338409424,
                                         1.592452544454e-13,
                                         0.027093165326239,
                                         0.0120338201522827,
                                         3.76120570863359e-12,
                                         0.00835740205808544,
                                         0.0292249917984009,
                                         7.19146383424932e-11,
                                         0.00212305848763452,
                                         0.0584499835968018,
                                         1.13236550003795e-09,
                                         0.0004494396415088,
                                         0.0974166393280029,
                                         1.4858491206031e-08,
                                         7.99208470846604e-05,
                                         0.136383295059204,
                                         1.63772915340792e-07,
                                         1.19969655767736e-05,
                                         0.161180257797241,
                                         1.5238144232266e-06,
                                         1.5238144232266e-06,
                                         0.161180257797241,
                                         1.19969655767736e-05,
                                         1.63772915340792e-07,
                                         0.136383295059204,
                                         7.99208470846604e-05,
                                         1.4858491206031e-08,
                                         0.0974166393280029,
                                         0.0004494396415088,
                                         1.13236550003795e-09,
                                         0.0584499835968018,
                                         0.00212305848763452,
                                         7.19146383424932e-11,
                                         0.0292249917984009,
                                         0.00835740205808544,
                                         3.76120570863359e-12,
                                         0.0120338201522827,
                                         0.027093165326239,
                                         1.592452544454e-13,
                                         0.00401127338409424,
                                         0.0711013464698411,
                                         5.3228419347377e-15,
                                         0.00105559825897217,
                                         0.147310451749995,
                                         1.35217914243445e-16,
                                         0.000211119651794434,
                                         0.23195454667,
                                         2.45356100000212e-18,
                                         3.01599502563477e-05,
                                         0.26088204043,
                                         2.8331191645114e-20,
                                         2.74181365966797e-06,
                                         0.186719996329999,
                                         1.56457963509512e-22,
                                         1.19209289550781e-07,
                                         0.0639149313699997,

                                         0.191076948440237,
                                         9.99031435997333e-05,
                                         8.43543550010943e-12,
                                         2.26908162882417e-27,
                                         0.327903948938529,
                                         0.00113178711342986,
                                         3.93892390146221e-10,
                                         6.99467341121553e-25,
                                         0.269122399362719,
                                         0.00613218409598385,
                                         8.79655294694157e-09,
                                         1.03121592873352e-22,
                                         0.140558916306942,
                                         0.0211432127108621,
                                         1.25012317574478e-07,
                                         9.67469179642333e-21,
                                         0.0524371305721481,
                                         0.0520713380815932,
                                         1.26901003475911e-06,
                                         6.48330762796929e-19,
                                         0.0148673316393774,
                                         0.0974630704369305,
                                         9.79018499671862e-06,
                                         3.30194406182142e-17,
                                         0.00332785808648822,
                                         0.144018770295518,
                                         5.96285705794501e-05,
                                         1.32764037802521e-15,
                                         0.000603012331271784,
                                         0.172277172107893,
                                         0.000294000131955205,
                                         4.32136370143923e-14,
                                         8.9984291803913e-05,
                                         0.169713146157061,
                                         0.00119376754198444,
                                         1.15835132894376e-12,
                                         1.11898938484038e-05,
                                         0.13932273414461,
                                         0.00403934326723563,
                                         2.58748914688452e-11,
                                         1.16886543568713e-06,
                                         0.0960744216863169,
                                         0.0114810350454733,
                                         4.85507628910315e-10,
                                         1.03068438756856e-07,
                                         0.055926314504849,
                                         0.0275469775578941,
                                         7.69017480066305e-09,
                                         7.69017480066298e-09,
                                         0.0275469775578941,
                                         0.055926314504849,
                                         1.03068438756857e-07,
                                         4.8550762891031e-10,
                                         0.0114810350454733,
                                         0.0960744216863169,
                                         1.16886543568714e-06,
                                         2.58748914688449e-11,
                                         0.00403934326723563,
                                         0.13932273414461,
                                         1.11898938484039e-05,
                                         1.15835132894374e-12,
                                         0.00119376754198444,
                                         0.169713146157061,
                                         8.99842918039135e-05,
                                         4.32136370143917e-14,
                                         0.000294000131955205,
                                         0.172277172107893,
                                         0.000603012331271788,
                                         1.32764037802519e-15,
                                         5.96285705794501e-05,
                                         0.144018770295518,
                                         0.00332785808648824,
                                         3.30194406182138e-17,
                                         9.79018499671862e-06,
                                         0.0974630704369305,
                                         0.0148673316393774,
                                         6.48330762796919e-19,
                                         1.26901003475911e-06,
                                         0.0520713380815932,
                                         0.0524371305721483,
                                         9.67469179642317e-21,
                                         1.25012317574478e-07,
                                         0.0211432127108621,
                                         0.140558916306942,
                                         1.03121592873351e-22,
                                         8.79655294694157e-09,
                                         0.00613218409598385,
                                         0.26912239936272,
                                         6.99467341121541e-25,
                                         3.93892390146221e-10,
                                         0.00113178711342986,
                                         0.327903948938529,
                                         2.26908162882413e-27,
                                         8.43543550010943e-12,
                                         9.99031435997333e-05,
                                         0.191076948440237,

                                         0.331191505029799,
                                         0.00239526382654268,
                                         1.19209289550781e-07,
                                         2.25384246634604e-15,
                                         2.7486621384839e-31,
                                         0.374920591104671,
                                         0.0165269586197633,
                                         2.74181365966797e-06,
                                         1.72798371646415e-13,
                                         1.28444918636233e-28,
                                         0.202985140475949,
                                         0.0545377695861948,
                                         3.01599502563477e-05,
                                         6.33607899060043e-12,
                                         2.87063083016584e-26,
                                         0.0699349960840986,
                                         0.114526809081939,
                                         0.000211119651794434,
                                         1.47845079445819e-10,
                                         4.08265908517957e-24,
                                         0.017210632356828,
                                         0.171786453131623,
                                         0.00105559825897217,
                                         2.46413859747456e-09,
                                         4.14744707247012e-22,
                                         0.00321893859809373,
                                         0.195832269703827,
                                         0.00401127338409424,
                                         3.12131054901945e-08,
                                         3.2020800203111e-20,
                                         0.000475298250315547,
                                         0.176245184638298,
                                         0.0120338201522827,
                                         3.12137887606681e-07,
                                         1.9517364240767e-18,
                                         5.68132148705591e-05,
                                         0.128404395114399,
                                         0.0292249917984009,
                                         2.52688583230324e-06,
                                         9.6302991988818e-17,
                                         5.59257258252132e-06,
                                         0.07704095060374,
                                         0.0584499835968018,
                                         1.68462743139772e-05,
                                         3.9132492217539e-15,
                                         4.58767503329831e-07,
                                         0.0385196320878787,
                                         0.0974166393280029,
                                         9.35924615963249e-05,
                                         1.325116277569e-13,
                                         3.16120711828017e-08,
                                         0.0161778913347851,
                                         0.136383295059204,
                                         0.000436774381782989,
                                         3.76920497939741e-12,
                                         1.8388058920109e-09,
                                         0.00573567228014653,
                                         0.161180257797241,
                                         0.00172066401784894,
                                         9.05040793736881e-11,
                                         9.05040793736868e-11,
                                         0.00172066401784894,
                                         0.161180257797241,
                                         0.00573567228014653,
                                         1.83880589201092e-09,
                                         3.76920497939735e-12,
                                         0.000436774381782989,
                                         0.136383295059204,
                                         0.0161778913347851,
                                         3.1612071182802e-08,
                                         1.32511627756897e-13,
                                         9.35924615963249e-05,
                                         0.0974166393280029,
                                         0.0385196320878787,
                                         4.58767503329836e-07,
                                         3.91324922175384e-15,
                                         1.68462743139772e-05,
                                         0.0584499835968018,
                                         0.07704095060374,
                                         5.59257258252137e-06,
                                         9.63029919888162e-17,
                                         2.52688583230324e-06,
                                         0.0292249917984009,
                                         0.128404395114399,
                                         5.68132148705595e-05,
                                         1.95173642407666e-18,
                                         3.12137887606681e-07,
                                         0.0120338201522827,
                                         0.176245184638298,
                                         0.00047529825031555,
                                         3.20208002031104e-20,
                                         3.12131054901945e-08,
                                         0.00401127338409424,
                                         0.195832269703827,
                                         0.00321893859809375,
                                         4.14744707247002e-22,
                                         2.46413859747456e-09,
                                         0.00105559825897217,
                                         0.171786453131623,
                                         0.017210632356828,
                                         4.08265908517947e-24,
                                         1.47845079445819e-10,
                                         0.000211119651794434,
                                         0.114526809081939,
                                         0.0699349960840988,
                                         2.87063083016577e-26,
                                         6.33607899060043e-12,
                                         3.01599502563477e-05,
                                         0.0545377695861948,
                                         0.202985140475949,
                                         1.28444918636229e-28,
                                         1.72798371646415e-13,
                                         2.74181365966797e-06,
                                         0.0165269586197633,
                                         0.374920591104672,
                                         2.74866213848382e-31,
                                         2.25384246634604e-15,
                                         1.19209289550781e-07,
                                         0.00239526382654268,
                                         0.331191505029799,

                                         0.453838235472607,
                                         0.0139981536538535,
                                         1.63636268134639e-05,
                                         2.25504110266847e-10,
                                         1.83963470801119e-18,
                                         1.42825975904939e-34,
                                         0.364767503267756,
                                         0.0656607116341807,
                                         0.000231351078493362,
                                         8.43758522078565e-09,
                                         2.07468629349141e-16,
                                         9.40043203208573e-32,
                                         0.14021550751141,
                                         0.147300853564393,
                                         0.00156432987527751,
                                         1.50989494644674e-07,
                                         1.11902072482165e-14,
                                         2.95905662735683e-29,
                                         0.0342989883578117,
                                         0.210285969196791,
                                         0.00673116916756055,
                                         1.71941474071652e-06,
                                         3.84086618762605e-13,
                                         5.92739806175708e-27,
                                         0.00599292079155656,
                                         0.214430863147437,
                                         0.0206882919180968,
                                         1.39857745805929e-05,
                                         9.41655805704599e-12,
                                         8.48099633649242e-25,
                                         0.000795809935319135,
                                         0.166179666607309,
                                         0.0483250551269482,
                                         8.64581612279368e-05,
                                         1.75456229391209e-10,
                                         9.22238502691176e-23,
                                         8.34291529140414e-05,
                                         0.101673115379784,
                                         0.0891164214599662,
                                         0.00042195207181343,
                                         2.58097057771317e-09,
                                         7.91730329011603e-21,
                                         7.08036975016648e-06,
                                         0.0503574819079341,
                                         0.133037092841237,
                                         0.00166705481061436,
                                         3.07345558990729e-08,
                                         5.50225787571173e-19,
                                         4.94849523199034e-07,
                                         0.020540026237023,
                                         0.163556145693003,
                                         0.00542395089273911,
                                         3.01404647642802e-07,
                                         3.14908014613851e-17,
                                         2.88210054995779e-08,
                                         0.00698162851266927,
                                         0.167563623439181,
                                         0.0147061967706606,
                                         2.46315477164641e-06,
                                         1.50191460651369e-15,
                                         1.4100171224071e-09,
                                         0.00199338765911062,
                                         0.144202206136152,
                                         0.0334937891763366,
                                         1.69087983224159e-05,
                                         6.01708371868908e-14,
                                         5.82320764021209e-11,
                                         0.00048045135813013,
                                         0.104757879216109,
                                         0.0643948036958646,
                                         9.79842828484571e-05,
                                         2.03493011376996e-12,
                                         2.03493011376996e-12,
                                         9.79842828484575e-05,
                                         0.0643948036958646,
                                         0.104757879216109,
                                         0.000480451358130129,
                                         5.82320764021209e-11,
                                         6.01708371868908e-14,
                                         1.69087983224159e-05,
                                         0.0334937891763366,
                                         0.144202206136152,
                                         0.00199338765911061,
                                         1.4100171224071e-09,
                                         1.50191460651369e-15,
                                         2.46315477164642e-06,
                                         0.0147061967706606,
                                         0.167563623439181,
                                         0.00698162851266925,
                                         2.88210054995779e-08,
                                         3.14908014613851e-17,
                                         3.01404647642804e-07,
                                         0.00542395089273911,
                                         0.163556145693003,
                                         0.0205400262370229,
                                         4.94849523199034e-07,
                                         5.50225787571173e-19,
                                         3.07345558990731e-08,
                                         0.00166705481061436,
                                         0.133037092841237,
                                         0.050357481907934,
                                         7.08036975016648e-06,
                                         7.91730329011603e-21,
                                         2.58097057771319e-09,
                                         0.00042195207181343,
                                         0.0891164214599662,
                                         0.101673115379784,
                                         8.34291529140414e-05,
                                         9.22238502691176e-23,
                                         1.7545622939121e-10,
                                         8.64581612279368e-05,
                                         0.0483250551269482,
                                         0.166179666607309,
                                         0.000795809935319135,
                                         8.48099633649242e-25,
                                         9.41655805704604e-12,
                                         1.39857745805929e-05,
                                         0.0206882919180968,
                                         0.214430863147437,
                                         0.00599292079155656,
                                         5.92739806175708e-27,
                                         3.84086618762607e-13,
                                         1.71941474071652e-06,
                                         0.00673116916756055,
                                         0.210285969196791,
                                         0.0342989883578117,
                                         2.95905662735683e-29,
                                         1.11902072482166e-14,
                                         1.50989494644674e-07,
                                         0.00156432987527752,
                                         0.147300853564393,
                                         0.14021550751141,
                                         9.40043203208573e-32,
                                         2.07468629349142e-16,
                                         8.43758522078565e-09,
                                         0.000231351078493362,
                                         0.0656607116341807,
                                         0.364767503267756,
                                         1.42825975904939e-34,
                                         1.83963470801121e-18,
                                         2.25504110266847e-10,
                                         1.63636268134639e-05,
                                         0.0139981536538535,
                                         0.453838235472607,

                                         0.55275851711636,
                                         0.0414695175196301,
                                         0.000301212027764209,
                                         1.19209289550781e-07,
                                         7.51626264330387e-13,
                                         3.64500028136933e-21,
                                         2.134352908352e-37,
                                         0.331953812690078,
                                         0.141557770346571,
                                         0.00292794088412706,
                                         2.74181365966797e-06,
                                         4.09041738219375e-11,
                                         5.648699962261e-19,
                                         1.88009452268165e-34,
                                         0.0953421032732902,
                                         0.231101781642151,
                                         0.01361184977014,
                                         3.01599502563477e-05,
                                         1.06462865748176e-09,
                                         4.18662726553809e-17,
                                         7.92059828292104e-32,
                                         0.0174259775589571,
                                         0.240092298091401,
                                         0.0402696027525441,
                                         0.000211119651794434,
                                         1.76333177744071e-08,
                                         1.97462868920048e-15,
                                         2.12344807338103e-29,
                                         0.00227500071142193,
                                         0.178166122282338,
                                         0.0850960708726988,
                                         0.00105559825897217,
                                         2.08613245800449e-07,
                                         6.6524030185164e-14,
                                         4.06627987944898e-27,
                                         0.000225724923998629,
                                         0.10048138657472,
                                         0.136664357110167,
                                         0.00401127338409424,
                                         1.87569974232458e-06,
                                         1.70327688864888e-12,
                                         5.91788446424325e-25,
                                         1.76813390564222e-05,
                                         0.0447387421001447,
                                         0.17327610440881,
                                         0.0120338201522827,
                                         1.3314424990219e-05,
                                         3.44294126102408e-11,
                                         6.79944327177472e-23,
                                         1.12119309811986e-06,
                                         0.0161254389493808,
                                         0.177849117738287,
                                         0.0292249917984009,
                                         7.65087556377879e-05,
                                         5.63382820658144e-10,
                                         6.3242620089617e-21,
                                         5.85497160715197e-08,
                                         0.00478650410787943,
                                         0.150329380905036,
                                         0.0584499835968018,
                                         0.000362058690257772,
                                         7.59200876037916e-09,
                                         4.84423795086419e-19,
                                         2.54793253856401e-09,
                                         0.00118397922100318,
                                         0.105889958159865,
                                         0.0974166393280029,
                                         0.00142779404895577,
                                         8.52567071982844e-08,
                                         3.09214488733229e-17,
                                         9.31387366428946e-11,
                                         0.000246007875874612,
                                         0.0626534471447878,
                                         0.136383295059204,
                                         0.00472967658811603,
                                         8.04228938010607e-07,
                                         1.65795786355655e-15,
                                         2.87405709851731e-12,
                                         4.31495800399918e-05,
                                         0.0312937634818199,
                                         0.161180257797241,
                                         0.0132257391834052,
                                         6.40403162862812e-06,
                                         7.50429285435807e-14,
                                         7.50429285435787e-14,
                                         6.40403162862812e-06,
                                         0.0132257391834052,
                                         0.161180257797241,
                                         0.0312937634818199,
                                         4.31495800399918e-05,
                                         2.87405709851738e-12,
                                         1.65795786355651e-15,
                                         8.04228938010607e-07,
                                         0.00472967658811603,
                                         0.136383295059204,
                                         0.0626534471447878,
                                         0.000246007875874612,
                                         9.31387366428967e-11,
                                         3.09214488733219e-17,
                                         8.52567071982844e-08,
                                         0.00142779404895577,
                                         0.0974166393280029,
                                         0.105889958159865,
                                         0.00118397922100318,
                                         2.54793253856406e-09,
                                         4.84423795086403e-19,
                                         7.59200876037916e-09,
                                         0.000362058690257772,
                                         0.0584499835968018,
                                         0.150329380905036,
                                         0.00478650410787943,
                                         5.85497160715208e-08,
                                         6.32426200896148e-21,
                                         5.63382820658144e-10,
                                         7.65087556377879e-05,
                                         0.0292249917984009,
                                         0.177849117738287,
                                         0.0161254389493808,
                                         1.12119309811988e-06,
                                         6.79944327177447e-23,
                                         3.44294126102408e-11,
                                         1.3314424990219e-05,
                                         0.0120338201522827,
                                         0.17327610440881,
                                         0.0447387421001447,
                                         1.76813390564225e-05,
                                         5.91788446424301e-25,
                                         1.70327688864888e-12,
                                         1.87569974232458e-06,
                                         0.00401127338409424,
                                         0.136664357110167,
                                         0.10048138657472,
                                         0.000225724923998632,
                                         4.06627987944881e-27,
                                         6.6524030185164e-14,
                                         2.08613245800449e-07,
                                         0.00105559825897217,
                                         0.0850960708726988,
                                         0.178166122282338,
                                         0.00227500071142195,
                                         2.12344807338093e-29,
                                         1.97462868920048e-15,
                                         1.76333177744071e-08,
                                         0.000211119651794434,
                                         0.0402696027525441,
                                         0.240092298091401,
                                         0.0174259775589573,
                                         7.92059828292068e-32,
                                         4.18662726553809e-17,
                                         1.06462865748176e-09,
                                         3.01599502563477e-05,
                                         0.01361184977014,
                                         0.23110178164215,
                                         0.0953421032732906,
                                         1.88009452268156e-34,
                                         5.648699962261e-19,
                                         4.09041738219375e-11,
                                         2.74181365966797e-06,
                                         0.00292794088412706,
                                         0.141557770346571,
                                         0.331953812690079,
                                         2.13435290835189e-37,
                                         3.64500028136933e-21,
                                         7.51626264330387e-13,
                                         1.19209289550781e-07,
                                         0.000301212027764209,
                                         0.0414695175196301,
                                         0.55275851711636,

                                         0.630488008671118,
                                         0.0849301298925387,
                                         0.00197243721883767,
                                         5.73625563630624e-06,
                                         1.12746216546201e-09,
                                         4.25657995771303e-15,
                                         1.462570277819e-23,
                                         7.09649006626092e-40,
                                         0.293755381130148,
                                         0.221070678471965,
                                         0.0141096466635444,
                                         9.10338701628139e-05,
                                         3.75822814013848e-08,
                                         3.14777381599799e-13,
                                         2.97237130300353e-21,
                                         8.05731367688367e-37,
                                         0.0654575400198998,
                                         0.275210721607443,
                                         0.0482718492668819,
                                         0.000690943228056243,
                                         5.99141154841363e-07,
                                         1.11329713205459e-11,
                                         2.88904445841714e-19,
                                         4.37523911175925e-34,
                                         0.00928194271487442,
                                         0.218024304048765,
                                         0.105093798657854,
                                         0.00333723721311902,
                                         6.0782774374094e-06,
                                         2.50567082898469e-10,
                                         1.78694318817673e-17,
                                         1.51188468885803e-31,
                                         0.00094013463357881,
                                         0.123371951079911,
                                         0.163430160702602,
                                         0.0115134042901629,
                                         4.40457338780237e-05,
                                         4.02818036267132e-09,
                                         7.89476541399111e-16,
                                         3.73170676099222e-29,
                                         7.23693734024952e-05,
                                         0.053056860649321,
                                         0.193152759762904,
                                         0.0301879779923963,
                                         0.000242572058704383,
                                         4.92161220993151e-08,
                                         2.65082651923668e-14,
                                         7.00019184268654e-27,
                                         4.39802031338027e-06,
                                         0.0180137579937444,
                                         0.180221784565839,
                                         0.0624887665694266,
                                         0.00105466699609795,
                                         4.74726562501334e-07,
                                         7.0268558041557e-13,
                                         1.03669268245558e-24,
                                         2.16366129461305e-07,
                                         0.00495104330610152,
                                         0.136126688736985,
                                         0.10471273590147,
                                         0.00371209911930804,
                                         3.70688660160754e-06,
                                         1.50789257664733e-11,
                                         1.24285295890081e-22,
                                         8.76597919588525e-09,
                                         0.00112064547155854,
                                         0.0846756208275509,
                                         0.144502771097403,
                                         0.010759767491467,
                                         2.38371419621714e-05,
                                         2.66476477615962e-10,
                                         1.22706786405848e-20,
                                         2.95958180754424e-10,
                                         0.000211377379689009,
                                         0.043892695529043,
                                         0.166177261653543,
                                         0.0259899212344693,
                                         0.000127737325594274,
                                         3.92433531751171e-09,
                                         1.00956937572667e-18,
                                         8.39343146389516e-12,
                                         3.34909961568165e-05,
                                         0.0191119676438188,
                                         0.160526341107515,
                                         0.0527334670846805,
                                         0.000574990595589529,
                                         4.85459075807436e-08,
                                         6.97723000966396e-17,
                                         2.0094230491939e-13,
                                         4.47940419449559e-06,
                                         0.00702491675097598,
                                         0.130901205794569,
                                         0.0903213291796006,
                                         0.00218487349053226,
                                         5.06946083430084e-07,
                                         4.07054484193566e-15,
                                         4.07054484193566e-15,
                                         5.06946083430084e-07,
                                         0.00218487349053226,
                                         0.0903213291796008,
                                         0.130901205794569,
                                         0.00702491675097598,
                                         4.47940419449559e-06,
                                         2.0094230491939e-13,
                                         6.97723000966396e-17,
                                         4.85459075807436e-08,
                                         0.000574990595589529,
                                         0.0527334670846806,
                                         0.160526341107514,
                                         0.0191119676438189,
                                         3.34909961568165e-05,
                                         8.39343146389516e-12,
                                         1.00956937572667e-18,
                                         3.92433531751171e-09,
                                         0.000127737325594274,
                                         0.0259899212344694,
                                         0.166177261653542,
                                         0.043892695529043,
                                         0.000211377379689009,
                                         2.95958180754425e-10,
                                         1.22706786405848e-20,
                                         2.66476477615962e-10,
                                         2.38371419621714e-05,
                                         0.010759767491467,
                                         0.144502771097402,
                                         0.0846756208275509,
                                         0.00112064547155854,
                                         8.76597919588525e-09,
                                         1.24285295890081e-22,
                                         1.50789257664733e-11,
                                         3.70688660160754e-06,
                                         0.00371209911930805,
                                         0.10471273590147,
                                         0.136126688736985,
                                         0.00495104330610152,
                                         2.16366129461305e-07,
                                         1.03669268245558e-24,
                                         7.0268558041557e-13,
                                         4.74726562501334e-07,
                                         0.00105466699609796,
                                         0.0624887665694266,
                                         0.180221784565839,
                                         0.0180137579937444,
                                         4.39802031338027e-06,
                                         7.00019184268654e-27,
                                         2.65082651923668e-14,
                                         4.92161220993151e-08,
                                         0.000242572058704383,
                                         0.0301879779923962,
                                         0.193152759762904,
                                         0.053056860649321,
                                         7.23693734024952e-05,
                                         3.73170676099222e-29,
                                         7.89476541399111e-16,
                                         4.02818036267132e-09,
                                         4.40457338780238e-05,
                                         0.0115134042901629,
                                         0.163430160702602,
                                         0.123371951079911,
                                         0.00094013463357881,
                                         1.51188468885803e-31,
                                         1.78694318817673e-17,
                                         2.50567082898469e-10,
                                         6.07827743740942e-06,
                                         0.00333723721311902,
                                         0.105093798657854,
                                         0.218024304048765,
                                         0.00928194271487442,
                                         4.37523911175925e-34,
                                         2.88904445841714e-19,
                                         1.11329713205459e-11,
                                         5.99141154841364e-07,
                                         0.000690943228056243,
                                         0.0482718492668819,
                                         0.275210721607443,
                                         0.0654575400198998,
                                         8.05731367688366e-37,
                                         2.97237130300353e-21,
                                         3.14777381599799e-13,
                                         3.75822814013849e-08,
                                         9.10338701628139e-05,
                                         0.0141096466635444,
                                         0.221070678471965,
                                         0.293755381130148,
                                         7.09649006626092e-40,
                                         1.462570277819e-23,
                                         4.25657995771303e-15,
                                         1.12746216546202e-09,
                                         5.73625563630624e-06,
                                         0.00197243721883767,
                                         0.0849301298925387,
                                         0.630488008671118,

                                         0.691354098574496,
                                         0.139814078889817,
                                         0.00714826170679046,
                                         7.6145720309304e-05,
                                         1.19209289550781e-07,
                                         1.44990232475236e-11,
                                         3.83769735364171e-17,
                                         1.03710747963824e-25,
                                         4.41179495940449e-42,
                                         0.257239535104834,
                                         0.287183953963106,
                                         0.0393992411829707,
                                         0.000893688326552867,
                                         2.74181365966797e-06,
                                         6.53512399825916e-10,
                                         3.68331601521805e-15,
                                         2.67097715632887e-23,
                                         6.27240103765587e-39,
                                         0.0457762001525456,
                                         0.282120265885902,
                                         0.103858039436918,
                                         0.00501639261936588,
                                         3.01599502563477e-05,
                                         1.40874947642658e-08,
                                         1.6907215537323e-13,
                                         3.28989040093262e-21,
                                         4.26498163033266e-36,
                                         0.00518378663431823,
                                         0.176365548434348,
                                         0.174219890865212,
                                         0.0179185271915711,
                                         0.000211119651794434,
                                         1.93249802076869e-07,
                                         4.93867618770989e-12,
                                         2.57868350034431e-19,
                                         1.84546522840306e-33,
                                         0.000419301542545013,
                                         0.0787526517582194,
                                         0.208750401287095,
                                         0.045717772721429,
                                         0.00105559825897217,
                                         1.89354797224025e-06,
                                         1.03043589513645e-10,
                                         1.44373211448515e-17,
                                         5.70382947259716e-31,
                                         2.57762298001778e-05,
                                         0.0267257690781806,
                                         0.190094912033552,
                                         0.0886505451814972,
                                         0.00401127338409424,
                                         1.41009105951333e-05,
                                         1.6339734600079e-09,
                                         6.14311697332871e-16,
                                         1.33980252325163e-28,
                                         1.25097890927728e-06,
                                         0.00716032763950393,
                                         0.136663117991526,
                                         0.135711113962925,
                                         0.0120338201522827,
                                         8.29002163330601e-05,
                                         2.04553387292215e-08,
                                         2.06361469088764e-14,
                                         2.48457841479752e-26,
                                         4.91484931604521e-08,
                                         0.00155297764853674,
                                         0.0795356406637892,
                                         0.168181819472,
                                         0.0292249917984009,
                                         0.000394542502934868,
                                         2.07299368746227e-07,
                                         5.61174674850509e-13,
                                         3.72987477335128e-24,
                                         1.59019191565557e-09,
                                         0.000277380948896693,
                                         0.0381198652409612,
                                         0.17164129878072,
                                         0.0584499835968018,
                                         0.00154636154524546,
                                         1.73008881308568e-06,
                                         1.25674345038352e-11,
                                         4.61121008055995e-22,
                                         4.28753451362351e-11,
                                         4.12863802232467e-05,
                                         0.0152250835166636,
                                         0.145976615960548,
                                         0.0974166393280029,
                                         0.0050506472915388,
                                         1.20325470970942e-05,
                                         2.34538396400761e-10,
                                         4.75066405244502e-20,
                                         9.71057625221594e-13,
                                         5.16197946154409e-06,
                                         0.00510795775284779,
                                         0.104285535877902,
                                         0.136383295059204,
                                         0.0138567801554859,
                                         7.02952578971214e-05,
                                         3.67672000759908e-09,
                                         4.11124177335687e-18,
                                         1.85654298170213e-14,
                                         5.44814136820757e-07,
                                         0.00144663024617894,
                                         0.0628908542979867,
                                         0.161180257797241,
                                         0.0320922557117794,
                                         0.000346670684850259,
                                         4.86552599086229e-08,
                                         3.00340809120218e-16,
                                         3.00340809120206e-16,
                                         4.86552599086229e-08,
                                         0.000346670684850258,
                                         0.0320922557117794,
                                         0.161180257797241,
                                         0.0628908542979867,
                                         0.00144663024617894,
                                         5.44814136820757e-07,
                                         1.8565429817022e-14,
                                         4.11124177335669e-18,
                                         3.67672000759908e-09,
                                         7.02952578971211e-05,
                                         0.0138567801554859,
                                         0.136383295059204,
                                         0.104285535877902,
                                         0.0051079577528478,
                                         5.16197946154409e-06,
                                         9.71057625221627e-13,
                                         4.75066405244478e-20,
                                         2.34538396400761e-10,
                                         1.20325470970941e-05,
                                         0.0050506472915388,
                                         0.0974166393280029,
                                         0.145976615960548,
                                         0.0152250835166636,
                                         4.12863802232467e-05,
                                         4.28753451362364e-11,
                                         4.61121008055971e-22,
                                         1.25674345038352e-11,
                                         1.73008881308568e-06,
                                         0.00154636154524546,
                                         0.0584499835968018,
                                         0.17164129878072,
                                         0.0381198652409613,
                                         0.000277380948896693,
                                         1.59019191565561e-09,
                                         3.72987477335107e-24,
                                         5.61174674850509e-13,
                                         2.07299368746226e-07,
                                         0.000394542502934868,
                                         0.0292249917984009,
                                         0.168181819471999,
                                         0.0795356406637894,
                                         0.00155297764853674,
                                         4.91484931604533e-08,
                                         2.48457841479737e-26,
                                         2.06361469088764e-14,
                                         2.04553387292214e-08,
                                         8.29002163330601e-05,
                                         0.0120338201522827,
                                         0.135711113962925,
                                         0.136663117991527,
                                         0.00716032763950393,
                                         1.25097890927731e-06,
                                         1.33980252325155e-28,
                                         6.14311697332871e-16,
                                         1.63397346000789e-09,
                                         1.41009105951333e-05,
                                         0.00401127338409424,
                                         0.0886505451814972,
                                         0.190094912033552,
                                         0.0267257690781806,
                                         2.57762298001783e-05,
                                         5.70382947259678e-31,
                                         1.44373211448515e-17,
                                         1.03043589513644e-10,
                                         1.89354797224025e-06,
                                         0.00105559825897217,
                                         0.045717772721429,
                                         0.208750401287095,
                                         0.0787526517582194,
                                         0.000419301542545019,
                                         1.84546522840293e-33,
                                         2.57868350034431e-19,
                                         4.93867618770987e-12,
                                         1.93249802076869e-07,
                                         0.000211119651794434,
                                         0.0179185271915711,
                                         0.174219890865213,
                                         0.176365548434348,
                                         0.00518378663431828,
                                         4.26498163033234e-36,
                                         3.28989040093262e-21,
                                         1.69072155373229e-13,
                                         1.40874947642658e-08,
                                         3.01599502563477e-05,
                                         0.00501639261936588,
                                         0.103858039436918,
                                         0.282120265885902,
                                         0.0457762001525459,
                                         6.27240103765538e-39,
                                         2.67097715632887e-23,
                                         3.68331601521802e-15,
                                         6.53512399825916e-10,
                                         2.74181365966797e-06,
                                         0.000893688326552867,
                                         0.0393992411829707,
                                         0.287183953963106,
                                         0.257239535104835,
                                         4.41179495940414e-42,
                                         1.03710747963824e-25,
                                         3.83769735364169e-17,
                                         1.44990232475236e-11,
                                         1.19209289550781e-07,
                                         7.6145720309304e-05,
                                         0.00714826170679046,
                                         0.139814078889817,
                                         0.691354098574496,

                                         0.739301383173018,
                                         0.200568493103661,
                                         0.0179850824722314,
                                         0.000470682667791951,
                                         2.90120403056085e-06,
                                         2.92526249249459e-09,
                                         2.52204256524145e-13,
                                         5.16621998386094e-19,
                                         1.1729975777996e-27,
                                         4.53464106529665e-44,
                                         0.224778429701653,
                                         0.333754266420187,
                                         0.0789649205554871,
                                         0.00427927442562951,
                                         4.94341718958821e-05,
                                         9.08179142348979e-08,
                                         1.46745959466682e-11,
                                         6.22453334668776e-17,
                                         3.728968140928e-25,
                                         7.88979055587598e-41,
                                         0.0326853090898843,
                                         0.265616952512806,
                                         0.165813856802859,
                                         0.0186070240647137,
                                         0.00040284789976231,
                                         1.34847547852587e-06,
                                         4.08361453913174e-10,
                                         3.58678669406188e-15,
                                         5.66950417762511e-23,
                                         6.56527438552523e-38,
                                         0.00302451619580744,
                                         0.1345210064912,
                                         0.221570936932266,
                                         0.0514859884847994,
                                         0.00208910514700231,
                                         1.27414817985806e-05,
                                         7.23150491153634e-09,
                                         1.31525407762373e-13,
                                         5.48538154737029e-21,
                                         3.47652729370879e-35,
                                         0.000199908470752004,
                                         0.0486627211968577,
                                         0.211483618005852,
                                         0.10175907615155,
                                         0.00773840505809944,
                                         8.59941260699159e-05,
                                         9.14712457807279e-08,
                                         3.44497225164514e-12,
                                         3.79088462683504e-19,
                                         1.31495342161276e-32,
                                         1.00419966398888e-05,
                                         0.0133787724088497,
                                         0.1534102088979,
                                         0.15285190227101,
                                         0.0217849320921691,
                                         0.000441094079552646,
                                         8.79334463482314e-07,
                                         6.85765119648625e-11,
                                         1.9910762193482e-17,
                                         3.77997291477185e-30,
                                         3.98241582113468e-07,
                                         0.00290384745997564,
                                         0.0878556000149413,
                                         0.181261762520464,
                                         0.048417082960413,
                                         0.00178620511265664,
                                         6.67361661092203e-06,
                                         1.07771198669257e-09,
                                         8.25606040229136e-16,
                                         8.57836783311469e-28,
                                         1.27850598128608e-08,
                                         0.000510224088838413,
                                         0.0407299862618909,
                                         0.174008785499382,
                                         0.0871105301943317,
                                         0.00585546048105147,
                                         4.10013389089524e-05,
                                         1.37107010498562e-08,
                                         2.77132503222458e-14,
                                         1.57597842475192e-25,
                                         3.3801660698062e-10,
                                         7.38290419636092e-05,
                                         0.0155502792473664,
                                         0.137567317056742,
                                         0.129068959202329,
                                         0.0158077440211402,
                                         0.000207450238618889,
                                         1.43646723385013e-07,
                                         7.6609244916863e-13,
                                         2.38437733627528e-23,
                                         7.44718368334361e-12,
                                         8.90250571024196e-06,
                                         0.00494744386269952,
                                         0.0906312798511801,
                                         0.159364547091387,
                                         0.0355629262416014,
                                         0.000874678783835957,
                                         1.25415305559611e-06,
                                         1.7647925145578e-11,
                                         3.00620405735045e-21,
                                         1.37824168047391e-13,
                                         9.01730117077273e-07,
                                         0.00132221732636186,
                                         0.0501556940492117,
                                         0.165287839790028,
                                         0.0672054308183604,
                                         0.00309786531550122,
                                         9.1978144285409e-06,
                                         3.41495834238811e-10,
                                         3.18376653956127e-19,
                                         2.15318489715827e-15,
                                         7.71016230843743e-08,
                                         0.000298296009881859,
                                         0.0234306861925171,
                                         0.144714725824245,
                                         0.10720965078342,
                                         0.0092618790110298,
                                         5.69431354851471e-05,
                                         5.57827343973301e-09,
                                         2.84633886658562e-17,
                                         2.84633886658563e-17,
                                         5.57827343973307e-09,
                                         5.69431354851471e-05,
                                         0.0092618790110298,
                                         0.10720965078342,
                                         0.144714725824245,
                                         0.0234306861925171,
                                         0.000298296009881859,
                                         7.71016230843736e-08,
                                         2.15318489715827e-15,
                                         3.18376653956127e-19,
                                         3.41495834238815e-10,
                                         9.1978144285409e-06,
                                         0.00309786531550122,
                                         0.0672054308183604,
                                         0.165287839790028,
                                         0.0501556940492117,
                                         0.00132221732636186,
                                         9.01730117077266e-07,
                                         1.37824168047391e-13,
                                         3.00620405735045e-21,
                                         1.76479251455782e-11,
                                         1.25415305559611e-06,
                                         0.000874678783835957,
                                         0.0355629262416014,
                                         0.159364547091387,
                                         0.0906312798511801,
                                         0.00494744386269952,
                                         8.9025057102419e-06,
                                         7.44718368334361e-12,
                                         2.38437733627528e-23,
                                         7.66092449168639e-13,
                                         1.43646723385013e-07,
                                         0.000207450238618889,
                                         0.0158077440211402,
                                         0.129068959202329,
                                         0.137567317056742,
                                         0.0155502792473664,
                                         7.38290419636087e-05,
                                         3.3801660698062e-10,
                                         1.57597842475192e-25,
                                         2.77132503222462e-14,
                                         1.37107010498562e-08,
                                         4.10013389089524e-05,
                                         0.00585546048105147,
                                         0.0871105301943317,
                                         0.174008785499382,
                                         0.0407299862618909,
                                         0.00051022408883841,
                                         1.27850598128608e-08,
                                         8.57836783311469e-28,
                                         8.25606040229148e-16,
                                         1.07771198669257e-09,
                                         6.67361661092203e-06,
                                         0.00178620511265664,
                                         0.048417082960413,
                                         0.181261762520464,
                                         0.0878556000149413,
                                         0.00290384745997562,
                                         3.98241582113468e-07,
                                         3.77997291477185e-30,
                                         1.99107621934823e-17,
                                         6.85765119648625e-11,
                                         8.79334463482314e-07,
                                         0.000441094079552646,
                                         0.0217849320921691,
                                         0.15285190227101,
                                         0.1534102088979,
                                         0.0133787724088496,
                                         1.00419966398888e-05,
                                         1.31495342161276e-32,
                                         3.7908846268351e-19,
                                         3.44497225164514e-12,
                                         9.14712457807279e-08,
                                         8.59941260699159e-05,
                                         0.00773840505809944,
                                         0.10175907615155,
                                         0.211483618005852,
                                         0.0486627211968575,
                                         0.000199908470752004,
                                         3.47652729370879e-35,
                                         5.48538154737038e-21,
                                         1.31525407762373e-13,
                                         7.23150491153634e-09,
                                         1.27414817985806e-05,
                                         0.00208910514700231,
                                         0.0514859884847994,
                                         0.221570936932266,
                                         0.1345210064912,
                                         0.00302451619580744,
                                         6.56527438552523e-38,
                                         5.66950417762521e-23,
                                         3.58678669406188e-15,
                                         4.08361453913174e-10,
                                         1.34847547852587e-06,
                                         0.00040284789976231,
                                         0.0186070240647137,
                                         0.165813856802859,
                                         0.265616952512806,
                                         0.0326853090898843,
                                         7.88979055587598e-41,
                                         3.72896814092807e-25,
                                         6.22453334668776e-17,
                                         1.46745959466682e-11,
                                         9.08179142348979e-08,
                                         4.94341718958821e-05,
                                         0.00427927442562951,
                                         0.0789649205554871,
                                         0.333754266420186,
                                         0.224778429701653,
                                         4.53464106529665e-44,
                                         1.17299757779962e-27,
                                         5.16621998386094e-19,
                                         2.52204256524145e-13,
                                         2.92526249249459e-09,
                                         2.90120403056085e-06,
                                         0.000470682667791951,
                                         0.0179850824722314,
                                         0.200568493103661,
                                         0.739301383173018,

                                         0.777444416681051,
                                         0.262660742651281,
                                         0.035665481504784,
                                         0.00178966198609825,
                                         2.88537242242308e-05,
                                         1.19209289550781e-07,
                                         8.68930494963585e-11,
                                         5.80297062763068e-15,
                                         9.81864541195365e-21,
                                         1.95694753945224e-29,
                                         7.04157216141045e-46,
                                         0.196791299257032,
                                         0.361555092395048,
                                         0.127941330204393,
                                         0.0130308235795687,
                                         0.000381835942678908,
                                         2.74181365966797e-06,
                                         3.47348782518474e-09,
                                         4.21604456390181e-13,
                                         1.4479197038909e-18,
                                         7.52065282076514e-27,
                                         1.47159655411135e-42,
                                         0.0238235943837951,
                                         0.238022847778304,
                                         0.219501999670992,
                                         0.0453771865809491,
                                         0.00241666574277553,
                                         3.01599502563477e-05,
                                         6.64066194306932e-08,
                                         1.46495694944719e-11,
                                         1.02117983252713e-16,
                                         1.38228229959616e-24,
                                         1.47086474766335e-39,
                                         0.00183532945379406,
                                         0.0997167615903457,
                                         0.23964670118586,
                                         0.100556146362602,
                                         0.00973333609272365,
                                         0.000211119651794434,
                                         8.07908761846961e-07,
                                         3.23929042072904e-10,
                                         4.58316333734467e-15,
                                         1.61675163158813e-22,
                                         9.3553937599404e-37,
                                         0.0001009933426771,
                                         0.0298393699682838,
                                         0.186885838577932,
                                         0.159166490810135,
                                         0.0280013402104975,
                                         0.00105559825897217,
                                         7.02077780223952e-06,
                                         5.11619248498664e-09,
                                         1.46926592077309e-13,
                                         1.35070912329277e-20,
                                         4.25033701834114e-34,
                                         4.22362207801271e-06,
                                         0.00678616984032346,
                                         0.110763054659184,
                                         0.191473313785804,
                                         0.06122228134805,
                                         0.00401127338409424,
                                         4.63683595102472e-05,
                                         6.14125960602583e-08,
                                         3.57972008012458e-12,
                                         8.57618125181778e-19,
                                         1.46756807646043e-31,
                                         1.39448874745025e-07,
                                         0.00121842123211887,
                                         0.0518264084195695,
                                         0.181845485580833,
                                         0.105676362892951,
                                         0.0120338201522827,
                                         0.000241766371067632,
                                         5.81976831289427e-07,
                                         6.88549850725771e-11,
                                         4.29896319268178e-17,
                                         4.00046801815547e-29,
                                         3.72713053758888e-09,
                                         0.000177092357199271,
                                         0.0196307516440751,
                                         0.139806196569551,
                                         0.147664409476083,
                                         0.0292249917984009,
                                         0.00102046867790463,
                                         4.46461034602194e-06,
                                         1.07213959637788e-09,
                                         1.74446848955432e-15,
                                         8.82780997503546e-27,
                                         8.20376681269642e-11,
                                         2.11973364587729e-05,
                                         0.00612353012123926,
                                         0.0885175596178705,
                                         0.169923295664805,
                                         0.0584499835968018,
                                         0.00354717472073693,
                                         2.82059376393553e-05,
                                         1.37482175983899e-08,
                                         5.82963878458737e-14,
                                         1.60425818476778e-24,
                                         1.50477221995015e-12,
                                         2.11437259844038e-06,
                                         0.00159178915754193,
                                         0.0467036902569464,
                                         0.162947899407144,
                                         0.0974166393280029,
                                         0.0102750569190025,
                                         0.000148496519751172,
                                         1.46912995107511e-07,
                                         1.6234499964823e-12,
                                         2.42948546569735e-22,
                                         2.31850218068197e-14,
                                         1.77158109214973e-07,
                                         0.000347574984428494,
                                         0.0206991374609496,
                                         0.131257429810008,
                                         0.136383295059204,
                                         0.0250014489052622,
                                         0.000656706470620982,
                                         1.31872219076233e-06,
                                         3.79765469236107e-11,
                                         3.09053476218779e-20,
                                         3.01555243116328e-16,
                                         1.25303494404271e-08,
                                         6.40669594379489e-05,
                                         0.00774418878339133,
                                         0.0892527604648988,
                                         0.161180257797241,
                                         0.0513533466161104,
                                         0.00245159624231986,
                                         9.9923824366883e-06,
                                         7.49919525707869e-10,
                                         3.31875805089572e-18,
                                         3.31875805089552e-18,
                                         7.49919525707869e-10,
                                         9.99238243668835e-06,
                                         0.00245159624231987,
                                         0.0513533466161103,
                                         0.161180257797241,
                                         0.089252760464899,
                                         0.00774418878339131,
                                         6.40669594379486e-05,
                                         1.25303494404271e-08,
                                         3.01555243116345e-16,
                                         3.09053476218759e-20,
                                         3.79765469236107e-11,
                                         1.31872219076233e-06,
                                         0.000656706470620984,
                                         0.0250014489052622,
                                         0.136383295059204,
                                         0.131257429810009,
                                         0.0206991374609496,
                                         0.000347574984428493,
                                         1.77158109214973e-07,
                                         2.31850218068208e-14,
                                         2.42948546569717e-22,
                                         1.6234499964823e-12,
                                         1.46912995107512e-07,
                                         0.000148496519751172,
                                         0.0102750569190025,
                                         0.0974166393280029,
                                         0.162947899407144,
                                         0.0467036902569463,
                                         0.00159178915754192,
                                         2.11437259844038e-06,
                                         1.50477221995022e-12,
                                         1.60425818476765e-24,
                                         5.82963878458737e-14,
                                         1.374821759839e-08,
                                         2.82059376393554e-05,
                                         0.00354717472073692,
                                         0.0584499835968018,
                                         0.169923295664805,
                                         0.0885175596178704,
                                         0.00612353012123923,
                                         2.11973364587729e-05,
                                         8.20376681269676e-11,
                                         8.82780997503474e-27,
                                         1.74446848955432e-15,
                                         1.07213959637789e-09,
                                         4.46461034602196e-06,
                                         0.00102046867790463,
                                         0.0292249917984009,
                                         0.147664409476083,
                                         0.13980619656955,
                                         0.019630751644075,
                                         0.000177092357199271,
                                         3.72713053758901e-09,
                                         4.00046801815512e-29,
                                         4.29896319268178e-17,
                                         6.88549850725776e-11,
                                         5.8197683128943e-07,
                                         0.000241766371067631,
                                         0.0120338201522827,
                                         0.105676362892951,
                                         0.181845485580832,
                                         0.0518264084195694,
                                         0.00121842123211887,
                                         1.39448874745029e-07,
                                         1.46756807646029e-31,
                                         8.57618125181779e-19,
                                         3.57972008012461e-12,
                                         6.14125960602586e-08,
                                         4.63683595102471e-05,
                                         0.00401127338409424,
                                         0.0612222813480501,
                                         0.191473313785804,
                                         0.110763054659183,
                                         0.00678616984032346,
                                         4.22362207801282e-06,
                                         4.25033701834073e-34,
                                         1.35070912329277e-20,
                                         1.4692659207731e-13,
                                         5.11619248498666e-09,
                                         7.0207778022395e-06,
                                         0.00105559825897217,
                                         0.0280013402104975,
                                         0.159166490810135,
                                         0.186885838577932,
                                         0.0298393699682838,
                                         0.000100993342677102,
                                         9.35539375993944e-37,
                                         1.61675163158813e-22,
                                         4.5831633373447e-15,
                                         3.23929042072905e-10,
                                         8.07908761846959e-07,
                                         0.000211119651794434,
                                         0.00973333609272366,
                                         0.100556146362602,
                                         0.239646701185859,
                                         0.0997167615903457,
                                         0.00183532945379409,
                                         1.47086474766319e-39,
                                         1.38228229959616e-24,
                                         1.02117983252714e-16,
                                         1.46495694944719e-11,
                                         6.6406619430693e-08,
                                         3.01599502563477e-05,
                                         0.00241666574277553,
                                         0.0453771865809491,
                                         0.219501999670991,
                                         0.238022847778304,
                                         0.0238235943837953,
                                         1.47159655411119e-42,
                                         7.52065282076514e-27,
                                         1.44791970389092e-18,
                                         4.21604456390183e-13,
                                         3.47348782518472e-09,
                                         2.74181365966797e-06,
                                         0.000381835942678908,
                                         0.0130308235795687,
                                         0.127941330204392,
                                         0.361555092395048,
                                         0.196791299257033,
                                         7.04157216140963e-46,
                                         1.95694753945224e-29,
                                         9.81864541195375e-21,
                                         5.80297062763071e-15,
                                         8.68930494963581e-11,
                                         1.19209289550781e-07,
                                         2.88537242242308e-05,
                                         0.00178966198609825,
                                         0.035665481504784,
                                         0.262660742651281,
                                         0.777444416681051,

                                         0.808126813229139,
                                         0.323046429097816,
                                         0.0601375823985092,
                                         0.00491564439684941,
                                         0.000160346658563158,
                                         1.79841664760555e-06,
                                         5.49326351348689e-09,
                                         3.12943335440444e-12,
                                         1.71976188065453e-16,
                                         2.51348466627912e-22,
                                         4.53252815099975e-31,
                                         1.54336132429907e-47,
                                         0.172960115079241,
                                         0.374144654313576,
                                         0.179819169221924,
                                         0.0293940846223655,
                                         0.00170446466377736,
                                         3.21564220783446e-05,
                                         1.62520705302807e-07,
                                         1.5573753313537e-10,
                                         1.52140384075737e-14,
                                         4.44674156845523e-20,
                                         2.07024422783474e-28,
                                         3.81466878894319e-44,
                                         0.0177042389938115,
                                         0.207242581489523,
                                         0.257152563396978,
                                         0.0840628812594134,
                                         0.00866524684897134,
                                         0.000274985615210861,
                                         2.29959742240979e-06,
                                         3.70668537529745e-09,
                                         6.43703020997242e-13,
                                         3.762464314825e-18,
                                         4.5223847075006e-26,
                                         4.50931846930034e-41,
                                         0.00115322464921767,
                                         0.0730506016397881,
                                         0.234018980914087,
                                         0.152986798889961,
                                         0.0280336231526758,
                                         0.0014964340665215,
                                         2.07061941973969e-05,
                                         5.61414354499007e-08,
                                         1.73313319818368e-11,
                                         2.02585446846736e-16,
                                         6.28664273139588e-24,
                                         3.39211318866548e-38,
                                         5.36565238910984e-05,
                                         0.01839249154845,
                                         0.152118934225063,
                                         0.198872900550746,
                                         0.0647812743713566,
                                         0.0058167066399584,
                                         0.000133174390178702,
                                         6.07369899683804e-07,
                                         3.33311501096633e-10,
                                         7.79141006683556e-15,
                                         6.24226240111513e-22,
                                         1.82264348229349e-35,
                                         1.89733816746122e-06,
                                         0.00351941860439683,
                                         0.0751500123841701,
                                         0.196476607484642,
                                         0.113771461134522,
                                         0.0171834485983255,
                                         0.00065096065955428,
                                         4.99386283918664e-06,
                                         4.87171922092238e-09,
                                         2.27739033324491e-13,
                                         4.71062848286751e-20,
                                         7.44297757731649e-33,
                                         5.29669067241695e-08,
                                         0.000531666068318073,
                                         0.0293097758469696,
                                         0.153244095978291,
                                         0.157744723882017,
                                         0.0400757033518767,
                                         0.00251203928756831,
                                         3.24158648317229e-05,
                                         5.62149538702056e-08,
                                         5.25528824841474e-12,
                                         2.80642438139921e-18,
                                         2.39954750604703e-30,
                                         1.1969998749949e-09,
                                         6.5018446558869e-05,
                                         0.00925391624164354,
                                         0.0967578654536258,
                                         0.177054094860382,
                                         0.0756626599394086,
                                         0.0078474352024318,
                                         0.00017033670510657,
                                         5.25110973300964e-07,
                                         9.81714571452728e-11,
                                         1.35349751384825e-16,
                                         6.26241178073123e-28,
                                         2.22773082187849e-11,
                                         6.5480705263679e-06,
                                         0.00240612256422672,
                                         0.050311575883621,
                                         0.163657616930301,
                                         0.117641667297648,
                                         0.0201886911247479,
                                         0.000737119645509384,
                                         4.03951703046678e-06,
                                         1.51026461452834e-09,
                                         5.37577018557362e-15,
                                         1.34596263517707e-25,
                                         3.45501610486006e-13,
                                         5.49552087235557e-07,
                                         0.000521349163866067,
                                         0.0218005934713609,
                                         0.126062299398882,
                                         0.152426153751108,
                                         0.0432820022101159,
                                         0.00265819283956806,
                                         2.58956339216808e-05,
                                         1.93615272047386e-08,
                                         1.7792733758543e-13,
                                         2.41069984792302e-23,
                                         4.5010799243745e-15,
                                         3.87421448974649e-08,
                                         9.48896627096832e-05,
                                         0.00793501949758927,
                                         0.0815668164266452,
                                         0.165896451657732,
                                         0.0779445597112969,
                                         0.00805219470260667,
                                         0.000139444996756878,
                                         2.08499712539581e-07,
                                         4.9467954448584e-12,
                                         3.62687479437323e-21,
                                         4.95000918370013e-17,
                                         2.30558475552412e-09,
                                         1.4579134948944e-05,
                                         0.00243809324645367,
                                         0.0445517154459422,
                                         0.152418377618081,
                                         0.118491420233921,
                                         0.0205903952286147,
                                         0.000633872568422298,
                                         1.89536973598815e-06,
                                         1.16098833352021e-10,
                                         4.60621937289533e-19,
                                         4.60621937289567e-19,
                                         1.16098833352019e-10,
                                         1.89536973598814e-06,
                                         0.000633872568422296,
                                         0.0205903952286146,
                                         0.118491420233921,
                                         0.152418377618081,
                                         0.0445517154459423,
                                         0.00243809324645367,
                                         1.45791349489441e-05,
                                         2.30558475552415e-09,
                                         4.95000918369981e-17,
                                         3.62687479437352e-21,
                                         4.94679544485832e-12,
                                         2.08499712539579e-07,
                                         0.000139444996756878,
                                         0.00805219470260666,
                                         0.0779445597112969,
                                         0.165896451657732,
                                         0.0815668164266454,
                                         0.00793501949758929,
                                         9.48896627096836e-05,
                                         3.87421448974653e-08,
                                         4.50107992437423e-15,
                                         2.41069984792322e-23,
                                         1.77927337585427e-13,
                                         1.93615272047385e-08,
                                         2.58956339216807e-05,
                                         0.00265819283956806,
                                         0.0432820022101159,
                                         0.152426153751108,
                                         0.126062299398882,
                                         0.0218005934713609,
                                         0.000521349163866069,
                                         5.49552087235563e-07,
                                         3.45501610485987e-13,
                                         1.3459626351772e-25,
                                         5.37577018557352e-15,
                                         1.51026461452833e-09,
                                         4.03951703046676e-06,
                                         0.000737119645509383,
                                         0.0201886911247479,
                                         0.117641667297648,
                                         0.163657616930301,
                                         0.0503115758836211,
                                         0.00240612256422673,
                                         6.54807052636796e-06,
                                         2.22773082187838e-11,
                                         6.26241178073184e-28,
                                         1.35349751384823e-16,
                                         9.8171457145272e-11,
                                         5.25110973300962e-07,
                                         0.000170336705106569,
                                         0.0078474352024318,
                                         0.0756626599394086,
                                         0.177054094860382,
                                         0.096757865453626,
                                         0.00925391624164357,
                                         6.50184465588696e-05,
                                         1.19699987499485e-09,
                                         2.39954750604728e-30,
                                         2.80642438139916e-18,
                                         5.2552882484147e-12,
                                         5.62149538702053e-08,
                                         3.24158648317228e-05,
                                         0.00251203928756831,
                                         0.0400757033518768,
                                         0.157744723882017,
                                         0.153244095978291,
                                         0.0293097758469697,
                                         0.000531666068318077,
                                         5.29669067241676e-08,
                                         7.4429775773173e-33,
                                         4.71062848286742e-20,
                                         2.27739033324489e-13,
                                         4.87171922092236e-09,
                                         4.99386283918662e-06,
                                         0.00065096065955428,
                                         0.0171834485983255,
                                         0.113771461134522,
                                         0.196476607484642,
                                         0.0751500123841703,
                                         0.00351941860439685,
                                         1.89733816746117e-06,
                                         1.8226434822937e-35,
                                         6.24226240111499e-22,
                                         7.79141006683549e-15,
                                         3.33311501096632e-10,
                                         6.07369899683802e-07,
                                         0.000133174390178702,
                                         0.0058167066399584,
                                         0.0647812743713566,
                                         0.198872900550746,
                                         0.152118934225063,
                                         0.0183924915484501,
                                         5.36565238910971e-05,
                                         3.39211318866589e-38,
                                         6.28664273139573e-24,
                                         2.02585446846734e-16,
                                         1.73313319818367e-11,
                                         5.61414354499005e-08,
                                         2.07061941973969e-05,
                                         0.0014964340665215,
                                         0.0280336231526759,
                                         0.152986798889962,
                                         0.234018980914087,
                                         0.0730506016397884,
                                         0.00115322464921765,
                                         4.50931846930091e-41,
                                         4.52238470750049e-26,
                                         3.76246431482496e-18,
                                         6.43703020997238e-13,
                                         3.70668537529743e-09,
                                         2.29959742240979e-06,
                                         0.000274985615210861,
                                         0.00866524684897134,
                                         0.0840628812594135,
                                         0.257152563396978,
                                         0.207242581489523,
                                         0.0177042389938113,
                                         3.81466878894369e-44,
                                         2.07024422783469e-28,
                                         4.44674156845519e-20,
                                         1.52140384075736e-14,
                                         1.5573753313537e-10,
                                         1.62520705302807e-07,
                                         3.21564220783446e-05,
                                         0.00170446466377736,
                                         0.0293940846223655,
                                         0.179819169221924,
                                         0.374144654313576,
                                         0.17296011507924,
                                         1.54336132429929e-47,
                                         4.53252815099963e-31,
                                         2.51348466627909e-22,
                                         1.71976188065451e-16,
                                         3.12943335440443e-12,
                                         5.49326351348689e-09,
                                         1.79841664760555e-06,
                                         0.000160346658563158,
                                         0.00491564439684941,
                                         0.0601375823985092,
                                         0.323046429097816,
                                         0.808126813229139,

                                         0.83308638324533,
                                         0.379958570317149,
                                         0.0904338891784794,
                                         0.0107650864833289,
                                         0.00059894314758153,
                                         1.40555852065934e-05,
                                         1.19209289550781e-07,
                                         2.8817051041688e-10,
                                         1.35515283510025e-13,
                                         6.39540164308475e-18,
                                         8.33434466365485e-24,
                                         1.38940871241082e-32,
                                         4.53033356591574e-49,
                                         0.152742096685248,
                                         0.375527781011224,
                                         0.229084432168994,
                                         0.0539186302107312,
                                         0.00524503433624629,
                                         0.000202181778209694,
                                         2.74181365966797e-06,
                                         1.05977123670675e-08,
                                         8.18617859025374e-12,
                                         6.75463939849617e-16,
                                         1.74045484417264e-21,
                                         7.43669317661527e-30,
                                         1.30712505782242e-45,
                                         0.0133934453157574,
                                         0.177505880973395,
                                         0.277539495583674,
                                         0.129159105027573,
                                         0.0219672614928589,
                                         0.00139091291818764,
                                         3.01599502563477e-05,
                                         1.8639727958568e-07,
                                         2.36504278155037e-10,
                                         3.41193982155951e-14,
                                         1.73827671522578e-19,
                                         1.90368268045385e-27,
                                         1.8037192886385e-42,
                                         0.000747362365733308,
                                         0.0533935474801661,
                                         0.213973192867525,
                                         0.196886780934824,
                                         0.0585475678850625,
                                         0.00608924186118167,
                                         0.000211119651794434,
                                         2.08627890139048e-06,
                                         4.34812621589898e-09,
                                         1.09674562668663e-12,
                                         1.10479174409186e-17,
                                         3.10109170012549e-25,
                                         1.583894000401e-39,
                                         2.97880521055727e-05,
                                         0.0114719374643627,
                                         0.117832720321833,
                                         0.214377920597577,
                                         0.111458645805074,
                                         0.0190413822867815,
                                         0.00105559825897217,
                                         1.66792733703818e-05,
                                         5.71001497818702e-08,
                                         2.51815480041928e-11,
                                         5.01549604157819e-16,
                                         3.60833310513948e-23,
                                         9.93471109671046e-37,
                                         9.02332452884472e-07,
                                         0.00187326128533983,
                                         0.0493157943862157,
                                         0.177401440525179,
                                         0.161262080250844,
                                         0.0452529934299502,
                                         0.00401127338409424,
                                         0.000101343402161461,
                                         5.69883319123664e-07,
                                         4.39412623846435e-10,
                                         1.73045757401642e-14,
                                         3.19089291226382e-21,
                                         4.73585026889975e-34,
                                         2.15788704559925e-08,
                                         0.000241489138677063,
                                         0.016294604373991,
                                         0.115896918270177,
                                         0.184199484970108,
                                         0.0849051064646342,
                                         0.0120338201522827,
                                         0.000486128956528954,
                                         4.49026839577463e-06,
                                         6.05341283072137e-09,
                                         4.71352351735714e-13,
                                         2.22769378857334e-19,
                                         1.78228988329595e-31,
                                         4.17753853854535e-10,
                                         2.52015048235332e-05,
                                         0.00435844163224559,
                                         0.0612937596224902,
                                         0.170323350594465,
                                         0.128958488999564,
                                         0.0292249917984009,
                                         0.00188771805033373,
                                         2.86410069788834e-05,
                                         6.75083794743578e-08,
                                         1.03934570832259e-11,
                                         1.25900769688538e-17,
                                         5.4298564406653e-29,
                                         6.66026235009178e-12,
                                         2.16588034313175e-06,
                                         0.000960058644746937,
                                         0.0266956037801785,
                                         0.129699734366492,
                                         0.161304014138133,
                                         0.0584499835968018,
                                         0.00603673178820637,
                                         0.000150446947231471,
                                         6.20003565842175e-07,
                                         1.88735454036477e-10,
                                         5.85976758070466e-16,
                                         1.36231477664453e-26,
                                         8.84873037131739e-14,
                                         1.55117643887149e-07,
                                         0.000176231299916563,
                                         0.00968906766335247,
                                         0.0823043397358909,
                                         0.16813540271063,
                                         0.0974166393280029,
                                         0.0160873824511287,
                                         0.000658562853253179,
                                         4.74514453674876e-06,
                                         2.85604934953455e-09,
                                         2.27274729831692e-14,
                                         2.84829745031687e-24,
                                         9.87529035006922e-16,
                                         9.33183874075852e-09,
                                         2.71736271964022e-05,
                                         0.00295395255656179,
                                         0.04387182188812,
                                         0.147215130314016,
                                         0.136383295059204,
                                         0.0360120777094944,
                                         0.00242153286138041,
                                         3.05059103668273e-05,
                                         3.63042283710839e-08,
                                         7.40459306128998e-13,
                                         5.00233187207271e-22,
                                         9.30339405721404e-18,
                                         4.73910010641935e-10,
                                         3.53699798631024e-06,
                                         0.000760234675063437,
                                         0.019741094893289,
                                         0.108809886807414,
                                         0.161180257797241,
                                         0.0680508575128046,
                                         0.00751633512030763,
                                         0.00016555456697748,
                                         3.89557298196524e-07,
                                         2.0364505684284e-11,
                                         7.4162146675134e-20,
                                         7.41621466751403e-20,
                                         2.03645056842844e-11,
                                         3.89557298196521e-07,
                                         0.000165554566977481,
                                         0.00751633512030763,
                                         0.0680508575128046,
                                         0.161180257797241,
                                         0.108809886807414,
                                         0.019741094893289,
                                         0.000760234675063434,
                                         3.53699798631026e-06,
                                         4.73910010641928e-10,
                                         9.30339405721332e-18,
                                         5.00233187207316e-22,
                                         7.40459306129011e-13,
                                         3.63042283710836e-08,
                                         3.05059103668274e-05,
                                         0.00242153286138041,
                                         0.0360120777094944,
                                         0.136383295059204,
                                         0.147215130314015,
                                         0.04387182188812,
                                         0.00295395255656178,
                                         2.71736271964024e-05,
                                         9.33183874075839e-09,
                                         9.87529035006853e-16,
                                         2.84829745031715e-24,
                                         2.27274729831697e-14,
                                         2.85604934953453e-09,
                                         4.74514453674878e-06,
                                         0.000658562853253179,
                                         0.0160873824511287,
                                         0.0974166393280029,
                                         0.16813540271063,
                                         0.0823043397358909,
                                         0.00968906766335244,
                                         0.000176231299916564,
                                         1.55117643887147e-07,
                                         8.84873037131683e-14,
                                         1.36231477664467e-26,
                                         5.85976758070478e-16,
                                         1.88735454036475e-10,
                                         6.20003565842178e-07,
                                         0.000150446947231471,
                                         0.00603673178820637,
                                         0.0584499835968018,
                                         0.161304014138133,
                                         0.129699734366492,
                                         0.0266956037801785,
                                         0.000960058644746942,
                                         2.16588034313173e-06,
                                         6.66026235009141e-12,
                                         5.42985644066591e-29,
                                         1.2590076968854e-17,
                                         1.03934570832258e-11,
                                         6.75083794743581e-08,
                                         2.86410069788834e-05,
                                         0.00188771805033373,
                                         0.0292249917984009,
                                         0.128958488999564,
                                         0.170323350594465,
                                         0.0612937596224901,
                                         0.0043584416322456,
                                         2.5201504823533e-05,
                                         4.17753853854515e-10,
                                         1.78228988329617e-31,
                                         2.22769378857339e-19,
                                         4.71352351735709e-13,
                                         6.0534128307214e-09,
                                         4.49026839577463e-06,
                                         0.000486128956528954,
                                         0.0120338201522827,
                                         0.0849051064646342,
                                         0.184199484970108,
                                         0.115896918270177,
                                         0.016294604373991,
                                         0.000241489138677061,
                                         2.15788704559916e-08,
                                         4.73585026890034e-34,
                                         3.1908929122639e-21,
                                         1.7304575740164e-14,
                                         4.39412623846438e-10,
                                         5.69883319123664e-07,
                                         0.000101343402161461,
                                         0.00401127338409424,
                                         0.0452529934299502,
                                         0.161262080250844,
                                         0.177401440525179,
                                         0.0493157943862159,
                                         0.00187326128533982,
                                         9.02332452884441e-07,
                                         9.93471109671179e-37,
                                         3.60833310513957e-23,
                                         5.01549604157813e-16,
                                         2.5181548004193e-11,
                                         5.71001497818702e-08,
                                         1.66792733703818e-05,
                                         0.00105559825897217,
                                         0.0190413822867815,
                                         0.111458645805074,
                                         0.214377920597576,
                                         0.117832720321834,
                                         0.0114719374643627,
                                         2.97880521055718e-05,
                                         1.58389400040122e-39,
                                         3.10109170012557e-25,
                                         1.10479174409185e-17,
                                         1.09674562668664e-12,
                                         4.34812621589898e-09,
                                         2.08627890139048e-06,
                                         0.000211119651794434,
                                         0.00608924186118167,
                                         0.0585475678850625,
                                         0.196886780934823,
                                         0.213973192867526,
                                         0.0533935474801659,
                                         0.000747362365733292,
                                         1.80371928863877e-42,
                                         1.9036826804539e-27,
                                         1.73827671522576e-19,
                                         3.41193982155953e-14,
                                         2.36504278155037e-10,
                                         1.8639727958568e-07,
                                         3.01599502563477e-05,
                                         0.00139091291818764,
                                         0.0219672614928589,
                                         0.129159105027573,
                                         0.277539495583674,
                                         0.177505880973394,
                                         0.0133934453157572,
                                         1.30712505782262e-45,
                                         7.43669317661549e-30,
                                         1.74045484417262e-21,
                                         6.75463939849621e-16,
                                         8.18617859025374e-12,
                                         1.05977123670675e-08,
                                         2.74181365966797e-06,
                                         0.000202181778209694,
                                         0.00524503433624629,
                                         0.0539186302107311,
                                         0.229084432168994,
                                         0.375527781011224,
                                         0.152742096685247,
                                         4.53033356591647e-49,
                                         1.38940871241087e-32,
                                         8.33434466365474e-24,
                                         6.3954016430848e-18,
                                         1.35515283510025e-13,
                                         2.8817051041688e-10,
                                         1.19209289550781e-07,
                                         1.40555852065934e-05,
                                         0.00059894314758153,
                                         0.0107650864833289,
                                         0.0904338891784794,
                                         0.379958570317149,
                                         0.83308638324533,

                                         0.853611103943178,
                                         0.432538344802793,
                                         0.125140325374934,
                                         0.0200304702914999,
                                         0.00168826198732051,
                                         6.96291878707858e-05,
                                         1.26246674382001e-06,
                                         8.59184445651491e-09,
                                         1.72601804791032e-11,
                                         6.97024441643336e-15,
                                         2.91392260010211e-19,
                                         3.46529377641175e-25,
                                         5.42552634504194e-34,
                                         1.70876120268051e-50,
                                         0.135575159212275,
                                         0.369189085016093,
                                         0.272194377268828,
                                         0.0853819830165328,
                                         0.0124223229513153,
                                         0.000826633150965728,
                                         2.33735449811091e-05,
                                         2.45492053977608e-07,
                                         7.69094164705914e-10,
                                         5.01118811008446e-13,
                                         3.61625121691987e-17,
                                         8.42779301255312e-23,
                                         3.36258529905932e-31,
                                         5.69137208720385e-47,
                                         0.0102982881536419,
                                         0.150708567074008,
                                         0.283156065259443,
                                         0.174062884966897,
                                         0.0437150156768647,
                                         0.00469352521667732,
                                         0.000206963649063853,
                                         3.35469716853515e-06,
                                         1.63899847369827e-08,
                                         1.72305147996341e-11,
                                         2.14636719973243e-15,
                                         9.80285349398295e-21,
                                         9.96711782800488e-29,
                                         9.06603587965408e-44,
                                         0.00049780047974296,
                                         0.0391500556168136,
                                         0.187446761954024,
                                         0.225814316459298,
                                         0.097895745072845,
                                         0.0169586332370653,
                                         0.00116618893274859,
                                         2.91725611968575e-05,
                                         2.22271068301466e-07,
                                         3.77017192905293e-10,
                                         8.10689984994964e-14,
                                         7.25598737166979e-19,
                                         1.88005792279115e-26,
                                         9.19016359216646e-41,
                                         1.71876913588018e-05,
                                         0.00726438395335416,
                                         0.088634332990422,
                                         0.209251573187297,
                                         0.15659177899956,
                                         0.0437677785569962,
                                         0.00469370426372284,
                                         0.000181203992072151,
                                         2.15307564766313e-06,
                                         5.89245157704606e-09,
                                         2.18714490449945e-12,
                                         3.83629935407131e-17,
                                         2.5330562341119e-24,
                                         6.65427914312933e-38,
                                         4.51017480221103e-07,
                                         0.001024421747981,
                                         0.0318522130022378,
                                         0.147366776347995,
                                         0.190365258227726,
                                         0.0858483106455092,
                                         0.0143574106334192,
                                         0.000855410457662529,
                                         1.58507287196153e-05,
                                         6.99913678827411e-08,
                                         4.48449866235092e-11,
                                         1.54149475528959e-15,
                                         2.5937733585846e-22,
                                         3.66178111604521e-35,
                                         9.34344127366153e-09,
                                         0.000114050303994524,
                                         0.0090368047683256,
                                         0.0819347424995972,
                                         0.18270232944287,
                                         0.13293724077761,
                                         0.0346716235836344,
                                         0.00318800509991725,
                                         9.212485888887e-05,
                                         6.56342607957195e-07,
                                         7.25918556795024e-10,
                                         4.89000465224972e-14,
                                         2.09679936863283e-20,
                                         1.59082132518124e-32,
                                         1.56693136156708e-10,
                                         1.02788311337103e-05,
                                         0.00207548603273412,
                                         0.0368779040110229,
                                         0.141948269895279,
                                         0.166644580365402,
                                         0.0677800465839828,
                                         0.00961818411506042,
                                         0.000433445067062127,
                                         4.98248892608338e-06,
                                         9.51243111541999e-09,
                                         1.25575848064569e-12,
                                         1.37218085549658e-18,
                                         5.59474187478243e-30,
                                         2.1640744610731e-12,
                                         7.62904417585496e-07,
                                         0.000392558011084655,
                                         0.0136692120343,
                                         0.0908228732199602,
                                         0.172034219720378,
                                         0.109121081400335,
                                         0.0238971593391895,
                                         0.00167946287498907,
                                         3.11487913894574e-05,
                                         1.02653619156882e-07,
                                         2.65571876102204e-11,
                                         7.39511528492564e-17,
                                         1.6203835239795e-27,
                                         2.49065274328352e-14,
                                         4.71862301916371e-08,
                                         6.18737770479941e-05,
                                         0.00422220664490594,
                                         0.0484260586973835,
                                         0.14799847620759,
                                         0.146397686414796,
                                         0.0494786938340754,
                                         0.00542282394934231,
                                         0.000162276193588156,
                                         9.23157410037557e-07,
                                         4.68033347066469e-10,
                                         3.32121733190131e-15,
                                         3.91087861833711e-25,
                                         2.40787228320311e-16,
                                         2.45154415865846e-09,
                                         8.19197649669621e-06,
                                         0.00109550603812026,
                                         0.0216891382684811,
                                         0.106949542690466,
                                         0.164982945023607,
                                         0.0860536824219767,
                                         0.014708212262228,
                                         0.000710146106409562,
                                         6.97359209537522e-06,
                                         6.92868470985342e-09,
                                         1.25293611772266e-13,
                                         7.92884889685554e-23,
                                         1.96506240095599e-18,
                                         1.07519389628955e-10,
                                         9.15573888932473e-07,
                                         0.000239945548794545,
                                         0.00820026905723606,
                                         0.0652413969125207,
                                         0.156951879747904,
                                         0.126340713152843,
                                         0.0336757210666636,
                                         0.0026233923016616,
                                         4.44692651482955e-05,
                                         8.65859512354563e-08,
                                         3.99009460594259e-12,
                                         1.3569647850021e-20,
                                         1.3569647850021e-20,
                                         3.99009460594266e-12,
                                         8.65859512354563e-08,
                                         4.44692651482953e-05,
                                         0.0026233923016616,
                                         0.0336757210666636,
                                         0.126340713152843,
                                         0.156951879747904,
                                         0.0652413969125207,
                                         0.00820026905723608,
                                         0.000239945548794546,
                                         9.15573888932473e-07,
                                         1.07519389628953e-10,
                                         1.96506240095599e-18,
                                         7.92884889685554e-23,
                                         1.25293611772268e-13,
                                         6.92868470985342e-09,
                                         6.97359209537519e-06,
                                         0.000710146106409559,
                                         0.014708212262228,
                                         0.0860536824219765,
                                         0.164982945023607,
                                         0.106949542690466,
                                         0.0216891382684812,
                                         0.00109550603812026,
                                         8.19197649669621e-06,
                                         2.45154415865842e-09,
                                         2.40787228320311e-16,
                                         3.91087861833711e-25,
                                         3.32121733190138e-15,
                                         4.68033347066469e-10,
                                         9.23157410037552e-07,
                                         0.000162276193588155,
                                         0.00542282394934231,
                                         0.0494786938340753,
                                         0.146397686414796,
                                         0.14799847620759,
                                         0.0484260586973836,
                                         0.00422220664490595,
                                         6.18737770479941e-05,
                                         4.71862301916364e-08,
                                         2.49065274328352e-14,
                                         1.6203835239795e-27,
                                         7.39511528492581e-17,
                                         2.65571876102204e-11,
                                         1.02653619156881e-07,
                                         3.11487913894573e-05,
                                         0.00167946287498907,
                                         0.0238971593391894,
                                         0.109121081400335,
                                         0.172034219720378,
                                         0.0908228732199604,
                                         0.0136692120343001,
                                         0.000392558011084655,
                                         7.62904417585486e-07,
                                         2.1640744610731e-12,
                                         5.59474187478243e-30,
                                         1.37218085549661e-18,
                                         1.25575848064569e-12,
                                         9.51243111541994e-09,
                                         4.98248892608336e-06,
                                         0.000433445067062127,
                                         0.0096181841150604,
                                         0.0677800465839829,
                                         0.166644580365402,
                                         0.141948269895279,
                                         0.036877904011023,
                                         0.00207548603273412,
                                         1.02788311337102e-05,
                                         1.56693136156708e-10,
                                         1.59082132518124e-32,
                                         2.09679936863289e-20,
                                         4.89000465224972e-14,
                                         7.2591855679502e-10,
                                         6.56342607957193e-07,
                                         9.212485888887e-05,
                                         0.00318800509991724,
                                         0.0346716235836344,
                                         0.13293724077761,
                                         0.18270232944287,
                                         0.0819347424995974,
                                         0.0090368047683256,
                                         0.000114050303994523,
                                         9.34344127366153e-09,
                                         3.66178111604521e-35,
                                         2.59377335858467e-22,
                                         1.54149475528959e-15,
                                         4.48449866235089e-11,
                                         6.99913678827408e-08,
                                         1.58507287196153e-05,
                                         0.000855410457662527,
                                         0.0143574106334192,
                                         0.0858483106455092,
                                         0.190365258227726,
                                         0.147366776347995,
                                         0.0318522130022378,
                                         0.00102442174798099,
                                         4.51017480221103e-07,
                                         6.65427914312933e-38,
                                         2.53305623411197e-24,
                                         3.83629935407131e-17,
                                         2.18714490449943e-12,
                                         5.89245157704603e-09,
                                         2.15307564766313e-06,
                                         0.000181203992072151,
                                         0.00469370426372284,
                                         0.0437677785569962,
                                         0.15659177899956,
                                         0.209251573187298,
                                         0.088634332990422,
                                         0.00726438395335412,
                                         1.71876913588018e-05,
                                         9.19016359216646e-41,
                                         1.88005792279121e-26,
                                         7.25598737166979e-19,
                                         8.10689984994959e-14,
                                         3.77017192905292e-10,
                                         2.22271068301466e-07,
                                         2.91725611968574e-05,
                                         0.00116618893274859,
                                         0.0169586332370653,
                                         0.0978957450728451,
                                         0.225814316459298,
                                         0.187446761954024,
                                         0.0391500556168134,
                                         0.00049780047974296,
                                         9.06603587965408e-44,
                                         9.96711782800521e-29,
                                         9.80285349398295e-21,
                                         2.14636719973242e-15,
                                         1.7230514799634e-11,
                                         1.63899847369827e-08,
                                         3.35469716853514e-06,
                                         0.000206963649063853,
                                         0.00469352521667732,
                                         0.0437150156768647,
                                         0.174062884966897,
                                         0.283156065259443,
                                         0.150708567074008,
                                         0.0102982881536419,
                                         5.69137208720385e-47,
                                         3.36258529905944e-31,
                                         8.42779301255312e-23,
                                         3.61625121691984e-17,
                                         5.01118811008443e-13,
                                         7.69094164705914e-10,
                                         2.45492053977608e-07,
                                         2.33735449811091e-05,
                                         0.000826633150965728,
                                         0.0124223229513153,
                                         0.0853819830165328,
                                         0.272194377268828,
                                         0.369189085016093,
                                         0.135575159212275,
                                         1.70876120268051e-50,
                                         5.42552634504214e-34,
                                         3.46529377641175e-25,
                                         2.91392260010208e-19,
                                         6.97024441643332e-15,
                                         1.72601804791032e-11,
                                         8.59184445651488e-09,
                                         1.26246674382001e-06,
                                         6.96291878707858e-05,
                                         0.00168826198732051,
                                         0.0200304702914999,
                                         0.125140325374934,
                                         0.432538344802793,
                                         0.853611103943178,

                                         0.87066131335938,
                                         0.48050854361117,
                                         0.162769075293904,
                                         0.0330435832541191,
                                         0.00387423019166753,
                                         0.000248564053163044,
                                         8.08003846815711e-06,
                                         1.19209289550781e-07,
                                         6.79922236772227e-10,
                                         1.17675342590487e-12,
                                         4.2021904154393e-16,
                                         1.59253627219532e-20,
                                         1.75844032016338e-26,
                                         2.61638416272868e-35,
                                         8.01133331325472e-52,
                                         0.120952341882639,
                                         0.357840784152138,
                                         0.307469580696494,
                                         0.121457339393895,
                                         0.0243349370481008,
                                         0.00248439348644408,
                                         0.000123586019171211,
                                         2.74181365966797e-06,
                                         2.35157590701028e-08,
                                         6.22815028435977e-11,
                                         3.53904914924275e-14,
                                         2.29196709137808e-18,
                                         4.92439686340926e-24,
                                         1.85852522852883e-32,
                                         3.05067145972533e-48,
                                         0.00803607839369671,
                                         0.127451047397038,
                                         0.27777762738881,
                                         0.213513450812168,
                                         0.0731037881436864,
                                         0.0118759207547389,
                                         0.000904045122743269,
                                         3.01599502563477e-05,
                                         3.88976718480367e-07,
                                         1.57651295807018e-09,
                                         1.42548387624039e-12,
                                         1.57758319602012e-16,
                                         6.59543292287567e-22,
                                         6.31393721635338e-30,
                                         5.55585530083742e-45,
                                         0.000339765597387937,
                                         0.0288869929700537,
                                         0.159697356585439,
                                         0.238853753721653,
                                         0.139750989634052,
                                         0.0361259731562726,
                                         0.00420839238940826,
                                         0.000211119651794434,
                                         4.09443111776059e-06,
                                         2.53945990884643e-08,
                                         3.65378828917488e-11,
                                         6.91005526058666e-15,
                                         5.62132797038354e-20,
                                         1.36501496367665e-27,
                                         6.43890130476857e-42,
                                         1.0260927168773e-05,
                                         0.00467663232216977,
                                         0.0655798101630393,
                                         0.190858218767654,
                                         0.190827897858138,
                                         0.078495323402726,
                                         0.0139931119884558,
                                         0.00105559825897217,
                                         3.07847403228735e-05,
                                         2.92184478391583e-07,
                                         6.68954244972472e-10,
                                         2.16193550384179e-13,
                                         3.4222088137272e-18,
                                         2.10788313808744e-25,
                                         5.33021312877673e-39,
                                         2.35508941019754e-07,
                                         0.000575410404384122,
                                         0.0204670935741302,
                                         0.115905288984258,
                                         0.198035218850307,
                                         0.129622861224482,
                                         0.0353611178337005,
                                         0.00401127338409424,
                                         0.000175910195876361,
                                         2.55497417099703e-06,
                                         9.30814287259889e-09,
                                         5.14064411838707e-12,
                                         1.58338920128944e-16,
                                         2.47382651177534e-23,
                                         3.35344334329657e-36,
                                         4.26742455971388e-09,
                                         5.58933129200099e-05,
                                         0.00504289415884925,
                                         0.0555690919399949,
                                         0.16224848760229,
                                         0.168988473221556,
                                         0.0705464751522653,
                                         0.0120338201522827,
                                         0.000793568074945875,
                                         1.763817084545e-05,
                                         1.0225094386366e-07,
                                         9.65005975759409e-11,
                                         5.78371001524413e-15,
                                         2.29207913296993e-21,
                                         1.66561655993801e-33,
                                         6.25970593521398e-11,
                                         4.3951289368188e-06,
                                         0.0010058498556399,
                                         0.0215671606096279,
                                         0.107608978413268,
                                         0.178345513027929,
                                         0.113934245190475,
                                         0.0292249917984009,
                                         0.00289805744871329,
                                         9.85712360279096e-05,
                                         9.09287534017956e-07,
                                         1.46646631940343e-09,
                                         1.71023176326391e-13,
                                         1.71917307931109e-19,
                                         6.69712975624525e-31,
                                         7.56172928547915e-13,
                                         2.84618091849693e-07,
                                         0.000165221125748802,
                                         0.00689337269135218,
                                         0.0587753838310197,
                                         0.155005249679998,
                                         0.151534790936609,
                                         0.0584499835968018,
                                         0.00871583313368366,
                                         0.000453655284462617,
                                         6.65908046817049e-06,
                                         1.83524182169118e-08,
                                         4.1646886604687e-12,
                                         1.06191244543518e-17,
                                         2.21759131020834e-28,
                                         7.61214535989732e-15,
                                         1.53593253221663e-08,
                                         2.2616049702182e-05,
                                         0.00183607027685829,
                                         0.0267523041557667,
                                         0.112266292455009,
                                         0.167953548015149,
                                         0.0974166393280029,
                                         0.0218438697180721,
                                         0.00173988482352149,
                                         4.06392835256646e-05,
                                         1.91396175709338e-07,
                                         8.45140407828655e-11,
                                         5.46608742375725e-16,
                                         6.11917766528469e-26,
                                         6.43683396030081e-17,
                                         6.96243352514805e-10,
                                         2.60044222264737e-06,
                                         0.000410795925086854,
                                         0.0102283645641213,
                                         0.0683017199481654,
                                         0.156367069620659,
                                         0.136383295059204,
                                         0.0459864128914425,
                                         0.00560524127978208,
                                         0.000208332539850476,
                                         1.67668894312894e-06,
                                         1.44063671603749e-09,
                                         2.36343532427542e-14,
                                         1.41835159195645e-23,
                                         4.59473185810593e-19,
                                         2.66423552999641e-11,
                                         2.52406363263078e-07,
                                         7.75863971631107e-05,
                                         0.00330121519897864,
                                         0.0350781379018668,
                                         0.12289212205506,
                                         0.161180257797241,
                                         0.081724471831261,
                                         0.0152437126997885,
                                         0.000901552051184805,
                                         1.23992203031546e-05,
                                         2.0730163279666e-08,
                                         8.62647767205876e-13,
                                         2.77521969961196e-21,
                                         2.77521969961165e-21,
                                         8.62647767205876e-13,
                                         2.07301632796659e-08,
                                         1.23992203031547e-05,
                                         0.000901552051184805,
                                         0.0152437126997885,
                                         0.0817244718312609,
                                         0.161180257797241,
                                         0.12289212205506,
                                         0.0350781379018668,
                                         0.00330121519897864,
                                         7.75863971631104e-05,
                                         2.5240636326308e-07,
                                         2.66423552999641e-11,
                                         4.59473185810639e-19,
                                         1.41835159195628e-23,
                                         2.36343532427542e-14,
                                         1.44063671603747e-09,
                                         1.67668894312895e-06,
                                         0.000208332539850476,
                                         0.00560524127978208,
                                         0.0459864128914425,
                                         0.136383295059204,
                                         0.156367069620659,
                                         0.0683017199481654,
                                         0.0102283645641213,
                                         0.000410795925086852,
                                         2.60044222264739e-06,
                                         6.96243352514805e-10,
                                         6.4368339603014e-17,
                                         6.1191776652839e-26,
                                         5.46608742375725e-16,
                                         8.45140407828646e-11,
                                         1.91396175709339e-07,
                                         4.06392835256646e-05,
                                         0.00173988482352149,
                                         0.0218438697180721,
                                         0.0974166393280029,
                                         0.167953548015149,
                                         0.112266292455009,
                                         0.0267523041557667,
                                         0.00183607027685828,
                                         2.26160497021821e-05,
                                         1.53593253221663e-08,
                                         7.61214535989796e-15,
                                         2.21759131020803e-28,
                                         1.06191244543518e-17,
                                         4.16468866046865e-12,
                                         1.83524182169119e-08,
                                         6.65908046817049e-06,
                                         0.000453655284462617,
                                         0.00871583313368365,
                                         0.0584499835968018,
                                         0.151534790936609,
                                         0.155005249679998,
                                         0.0587753838310197,
                                         0.00689337269135215,
                                         0.000165221125748803,
                                         2.84618091849693e-07,
                                         7.56172928547971e-13,
                                         6.69712975624426e-31,
                                         1.71917307931109e-19,
                                         1.71023176326389e-13,
                                         1.46646631940343e-09,
                                         9.09287534017956e-07,
                                         9.85712360279096e-05,
                                         0.00289805744871329,
                                         0.0292249917984009,
                                         0.113934245190475,
                                         0.178345513027929,
                                         0.107608978413268,
                                         0.0215671606096278,
                                         0.0010058498556399,
                                         4.3951289368188e-06,
                                         6.25970593521439e-11,
                                         1.66561655993775e-33,
                                         2.29207913296993e-21,
                                         5.78371001524406e-15,
                                         9.65005975759416e-11,
                                         1.0225094386366e-07,
                                         1.763817084545e-05,
                                         0.000793568074945875,
                                         0.0120338201522827,
                                         0.0705464751522653,
                                         0.168988473221556,
                                         0.16224848760229,
                                         0.0555690919399948,
                                         0.00504289415884927,
                                         5.58933129200099e-05,
                                         4.26742455971412e-09,
                                         3.35344334329601e-36,
                                         2.47382651177534e-23,
                                         1.58338920128942e-16,
                                         5.1406441183871e-12,
                                         9.30814287259889e-09,
                                         2.55497417099703e-06,
                                         0.000175910195876361,
                                         0.00401127338409424,
                                         0.0353611178337005,
                                         0.129622861224482,
                                         0.198035218850307,
                                         0.115905288984258,
                                         0.0204670935741303,
                                         0.000575410404384122,
                                         2.35508941019765e-07,
                                         5.3302131287758e-39,
                                         2.10788313808744e-25,
                                         3.42220881372715e-18,
                                         2.1619355038418e-13,
                                         6.68954244972472e-10,
                                         2.92184478391583e-07,
                                         3.07847403228735e-05,
                                         0.00105559825897217,
                                         0.0139931119884558,
                                         0.078495323402726,
                                         0.190827897858138,
                                         0.190858218767654,
                                         0.0655798101630395,
                                         0.00467663232216977,
                                         1.02609271687734e-05,
                                         6.43890130476738e-42,
                                         1.36501496367665e-27,
                                         5.62132797038345e-20,
                                         6.91005526058672e-15,
                                         3.65378828917488e-11,
                                         2.53945990884643e-08,
                                         4.09443111776059e-06,
                                         0.000211119651794434,
                                         0.00420839238940826,
                                         0.0361259731562726,
                                         0.139750989634052,
                                         0.238853753721652,
                                         0.159697356585439,
                                         0.0288869929700537,
                                         0.000339765597387946,
                                         5.55585530083634e-45,
                                         6.31393721635338e-30,
                                         6.59543292287556e-22,
                                         1.57758319602014e-16,
                                         1.42548387624039e-12,
                                         1.57651295807018e-09,
                                         3.88976718480367e-07,
                                         3.01599502563477e-05,
                                         0.000904045122743269,
                                         0.0118759207547389,
                                         0.0731037881436864,
                                         0.213513450812168,
                                         0.277777627388811,
                                         0.127451047397038,
                                         0.00803607839369685,
                                         3.05067145972471e-48,
                                         1.85852522852883e-32,
                                         4.92439686340918e-24,
                                         2.2919670913781e-18,
                                         3.53904914924275e-14,
                                         6.22815028435977e-11,
                                         2.35157590701028e-08,
                                         2.74181365966797e-06,
                                         0.000123586019171211,
                                         0.00248439348644408,
                                         0.0243349370481008,
                                         0.121457339393895,
                                         0.307469580696495,
                                         0.357840784152138,
                                         0.12095234188264,
                                         8.01133331325301e-52,
                                         2.61638416272868e-35,
                                         1.75844032016335e-26,
                                         1.59253627219534e-20,
                                         4.2021904154393e-16,
                                         1.17675342590487e-12,
                                         6.79922236772227e-10,
                                         1.19209289550781e-07,
                                         8.08003846815711e-06,
                                         0.000248564053163044,
                                         0.00387423019166753,
                                         0.0330435832541191,
                                         0.162769075293904,
                                         0.48050854361117,
                                         0.87066131335938,

                                         0.884959523433158,
                                         0.523936039855199,
                                         0.201977700086988,
                                         0.0497720291819022,
                                         0.00762170190366164,
                                         0.000696382894032387,
                                         3.586288680552e-05,
                                         9.61499265823382e-07,
                                         1.19974609608445e-08,
                                         5.92497023639768e-11,
                                         9.07788092921666e-14,
                                         2.93091322661395e-17,
                                         1.02477491376565e-21,
                                         1.0645694798325e-27,
                                         1.51892763115245e-36,
                                         4.54426746118868e-53,
                                         0.108441740995419,
                                         0.343468510721788,
                                         0.334583016954125,
                                         0.159508791219305,
                                         0.0414036409815504,
                                         0.0059538686835044,
                                         0.000462363514877273,
                                         1.82768054152075e-05,
                                         3.33882522762054e-07,
                                         2.43110397794569e-09,
                                         5.61680047571727e-12,
                                         2.85411887454169e-15,
                                         1.69155016037881e-19,
                                         3.39961089937037e-25,
                                         1.2256997591148e-33,
                                         1.96176274385936e-49,
                                         0.00635527583421642,
                                         0.107686293722493,
                                         0.265075269942096,
                                         0.244483047169559,
                                         0.107569700254965,
                                         0.0243453071482767,
                                         0.00285093106872713,
                                         0.000166156168295116,
                                         4.4438856618063e-06,
                                         4.77073994896731e-08,
                                         1.66210463571998e-10,
                                         1.32924796325842e-13,
                                         1.33538386663627e-17,
                                         5.19217430743433e-23,
                                         4.73037932800921e-31,
                                         4.05036283593898e-46,
                                         0.000237016010044233,
                                         0.0214851944093463,
                                         0.133641052300567,
                                         0.238461477591672,
                                         0.177847079431964,
                                         0.0633485413442525,
                                         0.0111865258866593,
                                         0.000961253653804207,
                                         3.76389432053374e-05,
                                         5.95762699767129e-07,
                                         3.12991886397963e-09,
                                         3.93953809419717e-12,
                                         6.70861276526686e-16,
                                         5.04631863782943e-21,
                                         1.16175136205179e-28,
                                         5.32165518804462e-43,
                                         6.31383059666927e-06,
                                         0.0030618937049884,
                                         0.0481263056875028,
                                         0.16613444149526,
                                         0.210027170340385,
                                         0.117741603164679,
                                         0.03135275519175,
                                         0.00397220362210591,
                                         0.000227710957968703,
                                         5.31413812158738e-06,
                                         4.20997738330543e-08,
                                         8.33981792960921e-11,
                                         2.40730594179884e-14,
                                         3.50325733177257e-19,
                                         2.03799154003455e-26,
                                         4.99426402161835e-40,
                                         1.27826753833878e-07,
                                         0.000331630544752839,
                                         0.0131716067558159,
                                         0.0879659734837127,
                                         0.18850280499843,
                                         0.166317084252097,
                                         0.0667835933815192,
                                         0.0124749437157414,
                                         0.00104699361172474,
                                         3.60251634243357e-05,
                                         4.30368065660009e-07,
                                         1.34178034813586e-09,
                                         6.56513171373071e-13,
                                         1.84834485061408e-17,
                                         2.71709707965128e-24,
                                         3.5621307468373e-37,
                                         2.0430935512212e-09,
                                         2.83567593017136e-05,
                                         0.00284598525174865,
                                         0.0367711666208694,
                                         0.133566581564893,
                                         0.18547331684025,
                                         0.112305626000287,
                                         0.0309302439062531,
                                         0.00380050922809882,
                                         0.000192804310539591,
                                         3.473265413574e-06,
                                         1.70429175259865e-08,
                                         1.41349168325657e-11,
                                         7.69895218389502e-16,
                                         2.8598653942624e-22,
                                         2.00579189751302e-34,
                                         2.64353078624233e-11,
                                         1.96285534982746e-06,
                                         0.000497801454065492,
                                         0.0124431326486937,
                                         0.0766138754952683,
                                         0.167438621990978,
                                         0.152884293079564,
                                         0.0620808595300448,
                                         0.0111678398547609,
                                         0.000835328204610427,
                                         2.2691621552964e-05,
                                         1.75241145961638e-07,
                                         2.46361419678584e-10,
                                         2.59603097146117e-14,
                                         2.43677626683666e-20,
                                         9.1430588646732e-32,
                                         2.8168233121972e-13,
                                         1.11892018753623e-07,
                                         7.17065438729342e-05,
                                         0.00346761735089469,
                                         0.0361906319304781,
                                         0.124482688103386,
                                         0.171397048991009,
                                         0.102615085669748,
                                         0.0270256207555589,
                                         0.00298041448911961,
                                         0.00012208778269356,
                                         1.48390924636454e-06,
                                         3.53615467264885e-09,
                                         7.2088706139982e-13,
                                         1.70987685626615e-18,
                                         3.43222913336889e-29,
                                         2.50122979887067e-15,
                                         5.31531095837056e-09,
                                         8.6075622991996e-06,
                                         0.000805288241561919,
                                         0.0142463514937777,
                                         0.0771224874981338,
                                         0.160126261779869,
                                         0.141345978544773,
                                         0.0545005559656141,
                                         0.00886165708872159,
                                         0.000547390977481836,
                                         1.04712216194184e-05,
                                         4.22969022679843e-08,
                                         1.66818167491908e-11,
                                         9.99845199640899e-17,
                                         1.0736921671315e-26,
                                         1.8656358043196e-17,
                                         2.12098467389015e-10,
                                         8.67922295172238e-07,
                                         0.000157090829972251,
                                         0.00471075405011666,
                                         0.0401358426746566,
                                         0.125661163272828,
                                         0.163544140095204,
                                         0.0923220571708074,
                                         0.0221326033615817,
                                         0.00206159023875232,
                                         6.20678422727965e-05,
                                         4.24976736016733e-07,
                                         3.24264008832532e-10,
                                         4.91111364218061e-15,
                                         2.82138648956549e-24,
                                         1.17468865175843e-19,
                                         7.14445352199195e-12,
                                         7.38761315721645e-08,
                                         2.58686009958942e-05,
                                         0.00131492153046032,
                                         0.0176321947486702,
                                         0.0832457782913903,
                                         0.159738333754666,
                                         0.132017846781301,
                                         0.0466630039381439,
                                         0.00655435100151244,
                                         0.000310569337595312,
                                         3.60449463126759e-06,
                                         5.32079810783878e-09,
                                         2.03633784715052e-13,
                                         6.25846764498945e-22,
                                         6.25846764499024e-22,
                                         2.03633784715052e-13,
                                         5.32079810783873e-09,
                                         3.60449463126761e-06,
                                         0.000310569337595312,
                                         0.00655435100151244,
                                         0.046663003938144,
                                         0.132017846781301,
                                         0.159738333754666,
                                         0.0832457782913902,
                                         0.0176321947486702,
                                         0.00131492153046032,
                                         2.58686009958941e-05,
                                         7.38761315721652e-08,
                                         7.14445352199195e-12,
                                         1.1746886517583e-19,
                                         2.82138648956587e-24,
                                         4.91111364218061e-15,
                                         3.24264008832529e-10,
                                         4.24976736016736e-07,
                                         6.20678422727965e-05,
                                         0.00206159023875232,
                                         0.0221326033615817,
                                         0.0923220571708074,
                                         0.163544140095204,
                                         0.125661163272828,
                                         0.0401358426746566,
                                         0.00471075405011667,
                                         0.00015709082997225,
                                         8.67922295172245e-07,
                                         2.12098467389015e-10,
                                         1.86563580431941e-17,
                                         1.07369216713165e-26,
                                         9.99845199640899e-17,
                                         1.66818167491906e-11,
                                         4.22969022679846e-08,
                                         1.04712216194184e-05,
                                         0.000547390977481836,
                                         0.00886165708872161,
                                         0.0545005559656141,
                                         0.141345978544772,
                                         0.160126261779869,
                                         0.0771224874981338,
                                         0.0142463514937777,
                                         0.000805288241561916,
                                         8.60756229919966e-06,
                                         5.31531095837056e-09,
                                         2.50122979887043e-15,
                                         3.43222913336943e-29,
                                         1.70987685626615e-18,
                                         7.20887061399811e-13,
                                         3.53615467264888e-09,
                                         1.48390924636454e-06,
                                         0.00012208778269356,
                                         0.00298041448911961,
                                         0.0270256207555589,
                                         0.102615085669748,
                                         0.171397048991008,
                                         0.124482688103386,
                                         0.0361906319304781,
                                         0.00346761735089468,
                                         7.17065438729347e-05,
                                         1.11892018753623e-07,
                                         2.81682331219696e-13,
                                         9.14305886467473e-32,
                                         2.43677626683666e-20,
                                         2.59603097146113e-14,
                                         2.46361419678586e-10,
                                         1.75241145961638e-07,
                                         2.2691621552964e-05,
                                         0.000835328204610429,
                                         0.0111678398547609,
                                         0.0620808595300447,
                                         0.152884293079563,
                                         0.167438621990978,
                                         0.0766138754952683,
                                         0.0124431326486937,
                                         0.000497801454065495,
                                         1.96285534982746e-06,
                                         2.64353078624213e-11,
                                         2.00579189751338e-34,
                                         2.8598653942624e-22,
                                         7.69895218389491e-16,
                                         1.41349168325658e-11,
                                         1.70429175259865e-08,
                                         3.473265413574e-06,
                                         0.000192804310539591,
                                         0.00380050922809881,
                                         0.030930243906253,
                                         0.112305626000287,
                                         0.18547331684025,
                                         0.133566581564893,
                                         0.0367711666208693,
                                         0.00284598525174867,
                                         2.83567593017136e-05,
                                         2.04309355122107e-09,
                                         3.56213074683797e-37,
                                         2.71709707965128e-24,
                                         1.84834485061405e-17,
                                         6.56513171373076e-13,
                                         1.34178034813586e-09,
                                         4.3036806566001e-07,
                                         3.60251634243358e-05,
                                         0.00104699361172474,
                                         0.0124749437157413,
                                         0.0667835933815191,
                                         0.166317084252097,
                                         0.18850280499843,
                                         0.0879659734837125,
                                         0.013171606755816,
                                         0.000331630544752839,
                                         1.27826753833872e-07,
                                         4.99426402161935e-40,
                                         2.03799154003455e-26,
                                         3.50325733177251e-19,
                                         2.40730594179886e-14,
                                         8.33981792960921e-11,
                                         4.20997738330543e-08,
                                         5.31413812158739e-06,
                                         0.000227710957968703,
                                         0.0039722036221059,
                                         0.0313527551917499,
                                         0.117741603164679,
                                         0.210027170340385,
                                         0.16613444149526,
                                         0.048126305687503,
                                         0.0030618937049884,
                                         6.31383059666901e-06,
                                         5.32165518804573e-43,
                                         1.16175136205179e-28,
                                         5.04631863782935e-21,
                                         6.70861276526692e-16,
                                         3.93953809419717e-12,
                                         3.12991886397963e-09,
                                         5.9576269976713e-07,
                                         3.76389432053373e-05,
                                         0.000961253653804204,
                                         0.0111865258866593,
                                         0.0633485413442525,
                                         0.177847079431964,
                                         0.238461477591672,
                                         0.133641052300567,
                                         0.0214851944093463,
                                         0.000237016010044225,
                                         4.05036283593987e-46,
                                         4.73037932800921e-31,
                                         5.19217430743424e-23,
                                         1.33538386663629e-17,
                                         1.32924796325842e-13,
                                         1.66210463571998e-10,
                                         4.77073994896733e-08,
                                         4.44388566180629e-06,
                                         0.000166156168295115,
                                         0.00285093106872712,
                                         0.0243453071482767,
                                         0.107569700254965,
                                         0.244483047169558,
                                         0.265075269942096,
                                         0.107686293722493,
                                         0.00635527583421629,
                                         1.96176274385981e-49,
                                         1.2256997591148e-33,
                                         3.39961089937031e-25,
                                         1.69155016037883e-19,
                                         2.85411887454169e-15,
                                         5.61680047571727e-12,
                                         2.4311039779457e-09,
                                         3.33882522762053e-07,
                                         1.82768054152074e-05,
                                         0.000462363514877273,
                                         0.00595386868350439,
                                         0.0414036409815504,
                                         0.159508791219305,
                                         0.334583016954125,
                                         0.343468510721788,
                                         0.108441740995417,
                                         4.54426746118978e-53,
                                         1.51892763115245e-36,
                                         1.06456947983248e-27,
                                         1.02477491376566e-21,
                                         2.93091322661395e-17,
                                         9.07788092921666e-14,
                                         5.9249702363977e-11,
                                         1.19974609608445e-08,
                                         9.61499265823377e-07,
                                         3.586288680552e-05,
                                         0.000696382894032387,
                                         0.00762170190366164,
                                         0.0497720291819022,
                                         0.201977700086988,
                                         0.523936039855199,
                                         0.884959523433158,

                                         0.897054912147486,
                                         0.563074534924025,
                                         0.241663587387089,
                                         0.0698976592622989,
                                         0.0133282421569068,
                                         0.00162391080679492,
                                         0.000121086729677933,
                                         5.20902595702295e-06,
                                         1.19209289550781e-07,
                                         1.29561657922943e-09,
                                         5.6802887455544e-12,
                                         7.86722429876463e-15,
                                         2.33620181598858e-18,
                                         7.63929072689296e-23,
                                         7.54313348103579e-29,
                                         1.03931595977325e-37,
                                         3.04984378469317e-54,
                                         0.0976849530098865,
                                         0.327469750146076,
                                         0.354030613964234,
                                         0.197162734922891,
                                         0.0633061377845353,
                                         0.0120321922591425,
                                         0.0013371622861161,
                                         8.35173162842217e-05,
                                         2.74181365966797e-06,
                                         4.27476416385673e-08,
                                         2.72106248985343e-10,
                                         5.61687311817641e-13,
                                         2.60191488286789e-16,
                                         1.43267112770707e-20,
                                         2.72381661575358e-26,
                                         9.45361620046094e-35,
                                         1.48157835232429e-50,
                                         0.00508746183561263,
                                         0.091083841574037,
                                         0.248047736033767,
                                         0.265981777767769,
                                         0.143808187202448,
                                         0.0426375461741502,
                                         0.00706214348998354,
                                         0.000640414842016432,
                                         3.01599502563477e-05,
                                         6.74547710048205e-07,
                                         6.23406702958798e-09,
                                         1.91792907927329e-11,
                                         1.38592797942977e-14,
                                         1.28500503225e-18,
                                         4.70401595235942e-24,
                                         4.11256889343739e-32,
                                         3.44220318202935e-47,
                                         0.000168608712283986,
                                         0.0161219230854105,
                                         0.11059489271478,
                                         0.228341205702809,
                                         0.207886726040393,
                                         0.0961490499984632,
                                         0.0237352773336585,
                                         0.00312501136602004,
                                         0.000211119651794434,
                                         6.77358428937456e-06,
                                         9.08886754614823e-08,
                                         4.16750211697444e-10,
                                         4.69778995102111e-13,
                                         7.33446605550387e-17,
                                         5.16969666968933e-22,
                                         1.13850190246015e-29,
                                         5.08924940477363e-44,
                                         3.99145119296164e-06,
                                         0.00203828221402726,
                                         0.0352214182933624,
                                         0.140019544247921,
                                         0.214655426511955,
                                         0.154870876645532,
                                         0.0569802103390846,
                                         0.0108921541353747,
                                         0.00105559825897217,
                                         4.85843471133135e-05,
                                         9.46498751816393e-07,
                                         6.46831376713263e-09,
                                         1.13741379429889e-11,
                                         2.99022688456749e-15,
                                         4.05819864911567e-20,
                                         2.25126334049082e-27,
                                         5.37456019845637e-41,
                                         7.18117031533376e-08,
                                         0.000195850812283832,
                                         0.00852495707414316,
                                         0.0652539223501053,
                                         0.168449829801919,
                                         0.189586805166952,
                                         0.103960264998398,
                                         0.0288529051379071,
                                         0.00401127338409424,
                                         0.000264842570716738,
                                         7.49107092613459e-06,
                                         7.62991887705235e-08,
                                         2.09294097063014e-10,
                                         9.2651695879145e-14,
                                         2.4211134171383e-18,
                                         3.38323706718233e-25,
                                         4.31365810993662e-38,
                                         1.01999322930795e-09,
                                         1.4856759649555e-05,
                                         0.00162897800253246,
                                         0.02400834603565,
                                         0.104360689354369,
                                         0.183224741484814,
                                         0.149743633071827,
                                         0.0603396762199321,
                                         0.0120338201522827,
                                         0.00113976909444548,
                                         4.68064205712355e-05,
                                         7.10536545815205e-07,
                                         3.04041681999502e-09,
                                         2.26641931250624e-12,
                                         1.14034070003301e-16,
                                         4.01398938293148e-23,
                                         2.73329287014321e-35,
                                         1.1728135570786e-11,
                                         9.12331026254883e-07,
                                         0.000251981040653721,
                                         0.00715068239530808,
                                         0.0523399048110936,
                                         0.143347378065641,
                                         0.17460592640862,
                                         0.102151801404745,
                                         0.0292249917984009,
                                         0.00397077780015573,
                                         0.000236753593082443,
                                         5.35651773240319e-06,
                                         3.57551743315833e-08,
                                         4.48804061136801e-11,
                                         4.34794200176781e-15,
                                         3.8552241619464e-21,
                                         1.40202656528648e-32,
                                         1.11055425502293e-13,
                                         4.61381212221115e-08,
                                         3.20996003233174e-05,
                                         0.00175392835817079,
                                         0.021617627647159,
                                         0.0923579966987472,
                                         0.167667430607273,
                                         0.14241908726543,
                                         0.0584499835968018,
                                         0.0113923523328839,
                                         0.000986204185297183,
                                         3.32550639878714e-05,
                                         3.46277112653492e-07,
                                         7.3190111419618e-10,
                                         1.36524933553347e-13,
                                         3.04931412285211e-19,
                                         5.9225049319841e-30,
                                         8.76333345217638e-16,
                                         1.94440227710318e-09,
                                         3.40761199812085e-06,
                                         0.000358504791982532,
                                         0.00744049603544818,
                                         0.0495881616933595,
                                         0.134170547475457,
                                         0.165466134212293,
                                         0.0974166393280029,
                                         0.0272376719391256,
                                         0.00342338590105016,
                                         0.000172048853391496,
                                         2.79465002826462e-06,
                                         9.94641996637236e-09,
                                         3.57239077594401e-12,
                                         2.00989536734895e-17,
                                         2.08484308463684e-27,
                                         5.80868974129104e-18,
                                         6.88321955175196e-11,
                                         3.03864481387138e-07,
                                         6.15541539262889e-05,
                                         0.00215117148893386,
                                         0.0223645827014186,
                                         0.0901872118247256,
                                         0.161483937029537,
                                         0.136383295059204,
                                         0.0547023328306355,
                                         0.0099821515765937,
                                         0.000747696010744841,
                                         1.89456869538114e-05,
                                         1.13543025569959e-07,
                                         7.85208931443503e-11,
                                         1.11281768586254e-15,
                                         6.16482285701633e-25,
                                         3.25019677144351e-20,
                                         2.05693114978552e-12,
                                         2.28734866386207e-08,
                                         8.92159283268209e-06,
                                         0.00052501390797012,
                                         0.00851463863560365,
                                         0.0511747065591031,
                                         0.133036916114346,
                                         0.161180257797241,
                                         0.0927394128398236,
                                         0.0245705622292737,
                                         0.00274296948297861,
                                         0.000108421626033147,
                                         1.09414974713194e-06,
                                         1.45691472897253e-09,
                                         5.20112415358247e-14,
                                         1.53882935288537e-22,
                                         1.53882935288515e-22,
                                         5.20112415358261e-14,
                                         1.45691472897253e-09,
                                         1.09414974713195e-06,
                                         0.000108421626033147,
                                         0.00274296948297862,
                                         0.0245705622292737,
                                         0.0927394128398236,
                                         0.161180257797241,
                                         0.133036916114346,
                                         0.0511747065591031,
                                         0.00851463863560362,
                                         0.000525013907970118,
                                         8.92159283268204e-06,
                                         2.28734866386207e-08,
                                         2.05693114978547e-12,
                                         3.25019677144393e-20,
                                         6.16482285701539e-25,
                                         1.11281768586257e-15,
                                         7.85208931443503e-11,
                                         1.1354302556996e-07,
                                         1.89456869538115e-05,
                                         0.000747696010744844,
                                         0.0099821515765937,
                                         0.0547023328306355,
                                         0.136383295059204,
                                         0.161483937029537,
                                         0.0901872118247256,
                                         0.0223645827014186,
                                         0.00215117148893386,
                                         6.15541539262886e-05,
                                         3.03864481387138e-07,
                                         6.88321955175181e-11,
                                         5.80868974129172e-18,
                                         2.08484308463649e-27,
                                         2.00989536734901e-17,
                                         3.57239077594401e-12,
                                         9.94641996637243e-09,
                                         2.79465002826464e-06,
                                         0.000172048853391496,
                                         0.00342338590105016,
                                         0.0272376719391256,
                                         0.0974166393280029,
                                         0.165466134212293,
                                         0.134170547475457,
                                         0.0495881616933594,
                                         0.00744049603544815,
                                         0.00035850479198253,
                                         3.40761199812085e-06,
                                         1.94440227710314e-09,
                                         8.76333345217731e-16,
                                         5.92250493198305e-30,
                                         3.04931412285221e-19,
                                         1.36524933553347e-13,
                                         7.31901114196185e-10,
                                         3.46277112653493e-07,
                                         3.32550639878715e-05,
                                         0.000986204185297183,
                                         0.0113923523328839,
                                         0.0584499835968018,
                                         0.14241908726543,
                                         0.167667430607273,
                                         0.092357996698747,
                                         0.0216176276471589,
                                         0.00175392835817078,
                                         3.20996003233174e-05,
                                         4.61381212221107e-08,
                                         1.11055425502304e-13,
                                         1.40202656528622e-32,
                                         3.85522416194653e-21,
                                         4.34794200176781e-15,
                                         4.48804061136805e-11,
                                         3.57551743315835e-08,
                                         5.35651773240321e-06,
                                         0.000236753593082443,
                                         0.00397077780015573,
                                         0.0292249917984009,
                                         0.102151801404745,
                                         0.17460592640862,
                                         0.143347378065641,
                                         0.0523399048110934,
                                         0.00715068239530805,
                                         0.000251981040653721,
                                         9.12331026254869e-07,
                                         1.1728135570787e-11,
                                         2.73329287014266e-35,
                                         4.01398938293164e-23,
                                         1.14034070003301e-16,
                                         2.26641931250626e-12,
                                         3.04041681999504e-09,
                                         7.10536545815208e-07,
                                         4.68064205712356e-05,
                                         0.00113976909444548,
                                         0.0120338201522827,
                                         0.0603396762199321,
                                         0.149743633071827,
                                         0.183224741484814,
                                         0.104360689354369,
                                         0.0240083460356499,
                                         0.00162897800253246,
                                         1.48567596495548e-05,
                                         1.01999322930802e-09,
                                         4.3136581099357e-38,
                                         3.38323706718247e-25,
                                         2.4211134171383e-18,
                                         9.26516958791458e-14,
                                         2.09294097063015e-10,
                                         7.62991887705238e-08,
                                         7.49107092613459e-06,
                                         0.000264842570716738,
                                         0.00401127338409424,
                                         0.0288529051379071,
                                         0.103960264998398,
                                         0.189586805166952,
                                         0.168449829801918,
                                         0.0652539223501052,
                                         0.00852495707414316,
                                         0.000195850812283829,
                                         7.18117031533419e-08,
                                         5.37456019845517e-41,
                                         2.25126334049092e-27,
                                         4.05819864911567e-20,
                                         2.99022688456752e-15,
                                         1.1374137942989e-11,
                                         6.46831376713266e-09,
                                         9.46498751816393e-07,
                                         4.85843471133135e-05,
                                         0.00105559825897217,
                                         0.0108921541353747,
                                         0.0569802103390846,
                                         0.154870876645531,
                                         0.214655426511955,
                                         0.140019544247921,
                                         0.0352214182933624,
                                         0.00203828221402724,
                                         3.99145119296183e-06,
                                         5.08924940477243e-44,
                                         1.13850190246021e-29,
                                         5.16969666968933e-22,
                                         7.33446605550394e-17,
                                         4.69778995102114e-13,
                                         4.16750211697446e-10,
                                         9.08886754614823e-08,
                                         6.77358428937456e-06,
                                         0.000211119651794434,
                                         0.00312501136602004,
                                         0.0237352773336585,
                                         0.0961490499984631,
                                         0.207886726040393,
                                         0.228341205702809,
                                         0.11059489271478,
                                         0.0161219230854103,
                                         0.000168608712283992,
                                         3.44220318202849e-47,
                                         4.11256889343759e-32,
                                         4.70401595235942e-24,
                                         1.28500503225001e-18,
                                         1.38592797942978e-14,
                                         1.9179290792733e-11,
                                         6.23406702958798e-09,
                                         6.74547710048205e-07,
                                         3.01599502563477e-05,
                                         0.000640414842016432,
                                         0.00706214348998354,
                                         0.0426375461741502,
                                         0.143808187202448,
                                         0.265981777767769,
                                         0.248047736033767,
                                         0.0910838415740366,
                                         0.00508746183561275,
                                         1.48157835232391e-50,
                                         9.45361620046141e-35,
                                         2.72381661575358e-26,
                                         1.43267112770708e-20,
                                         2.60191488286791e-16,
                                         5.61687311817644e-13,
                                         2.72106248985343e-10,
                                         4.27476416385673e-08,
                                         2.74181365966797e-06,
                                         8.35173162842217e-05,
                                         0.0013371622861161,
                                         0.0120321922591425,
                                         0.0633061377845353,
                                         0.197162734922891,
                                         0.354030613964234,
                                         0.327469750146076,
                                         0.0976849530098876,
                                         3.04984378469235e-54,
                                         1.0393159597733e-37,
                                         7.54313348103579e-29,
                                         7.63929072689305e-23,
                                         2.3362018159886e-18,
                                         7.86722429876467e-15,
                                         5.6802887455544e-12,
                                         1.29561657922943e-09,
                                         1.19209289550781e-07,
                                         5.20902595702295e-06,
                                         0.000121086729677933,
                                         0.00162391080679492,
                                         0.0133282421569068,
                                         0.0698976592622989,
                                         0.241663587387089,
                                         0.563074534924025,
                                         0.897054912147486,

                                         0.907369353818546,
                                         0.598266350762044,
                                         0.280980602736584,
                                         0.0929215795366407,
                                         0.0212652566351485,
                                         0.00328630881528693,
                                         0.000331701652406124,
                                         2.09074902219558e-05,
                                         7.7468041886019e-07,
                                         1.55400021534224e-08,
                                         1.50510249027386e-10,
                                         5.97330790754278e-13,
                                         7.60191206870948e-16,
                                         2.10436445890154e-19,
                                         6.50466071890569e-24,
                                         6.15430720301608e-30,
                                         8.23395437257302e-39,
                                         2.37721056669813e-55,
                                         0.0883881077149673,
                                         0.31079894649833,
                                         0.366723787637786,
                                         0.232588505711646,
                                         0.0891395049360031,
                                         0.0213331127269614,
                                         0.00317891119425833,
                                         0.000287364111295647,
                                         1.5032756062459e-05,
                                         4.23633909053852e-07,
                                         5.79283449019475e-09,
                                         3.29715841234881e-11,
                                         6.1948765658583e-14,
                                         2.65568793669084e-17,
                                         1.37470228934657e-21,
                                         2.49443448168367e-27,
                                         8.38453857060938e-36,
                                         1.29096240416809e-51,
                                         0.00411783001324243,
                                         0.0772199201791292,
                                         0.228911005151781,
                                         0.278435600281958,
                                         0.178704157519355,
                                         0.0662315522846564,
                                         0.0145704849110046,
                                         0.00188898280082751,
                                         0.000139514546208944,
                                         5.52325640518976e-06,
                                         1.06630394340187e-07,
                                         8.70421427895949e-10,
                                         2.41438943548687e-12,
                                         1.60286878931595e-15,
                                         1.38949667590636e-19,
                                         4.83537188280772e-25,
                                         4.08333232639424e-33,
                                         3.35292922823538e-48,
                                         0.000122081075872261,
                                         0.0122091242520645,
                                         0.0909284144242596,
                                         0.212112680160252,
                                         0.227984042401657,
                                         0.130852189607988,
                                         0.0424986339083883,
                                         0.0079018509558663,
                                         0.000823959236151392,
                                         4.58252717861172e-05,
                                         1.24903976573009e-06,
                                         1.46226030024136e-08,
                                         5.98807652354964e-11,
                                         6.15636356210516e-14,
                                         8.93741085960556e-18,
                                         5.96476056513775e-23,
                                         1.26548103698178e-30,
                                         5.54166837876539e-45,
                                         2.58523630133207e-06,
                                         0.00137883291787705,
                                         0.025799098350869,
                                         0.115419839168771,
                                         0.20775247566107,
                                         0.184658379088984,
                                         0.0885417087602064,
                                         0.0236103066768347,
                                         0.00347587320916557,
                                         0.000271573167842789,
                                         1.04506542437598e-05,
                                         1.75465512380808e-07,
                                         1.06081420257279e-09,
                                         1.68897223356411e-12,
                                         4.10617919929748e-16,
                                         5.25567030707934e-21,
                                         2.80135740502384e-28,
                                         6.5422727262724e-42,
                                         4.16069360573619e-08,
                                         0.00011834566787015,
                                         0.00556317895509872,
                                         0.0477318085858758,
                                         0.14388037392161,
                                         0.198048079845994,
                                         0.140195611872566,
                                         0.053615210437481,
                                         0.0111438618167526,
                                         0.00122315715301504,
                                         6.64544833644023e-05,
                                         1.60019327494448e-06,
                                         1.42825220026067e-08,
                                         3.52155400586682e-11,
                                         1.43376398265867e-14,
                                         3.51946624798323e-19,
                                         4.71297343999958e-26,
                                         5.86989520477082e-39,
                                         5.28650787425449e-10,
                                         8.01919445712484e-06,
                                         0.000947063700042595,
                                         0.0155837859369486,
                                         0.0786673582055174,
                                         0.167691060496314,
                                         0.17525023127997,
                                         0.0961196232155337,
                                         0.0282062381506285,
                                         0.00434925947888644,
                                         0.000333612834904312,
                                         1.15210157664861e-05,
                                         1.5181271110352e-07,
                                         5.79674043766673e-10,
                                         3.95234722993685e-13,
                                         1.86064342945168e-17,
                                         6.2597807442358e-24,
                                         4.15785853784396e-36,
                                         5.43752989968999e-12,
                                         4.39884593309116e-07,
                                         0.000130516367453279,
                                         0.00411877136073046,
                                         0.0348190727020929,
                                         0.114942016789961,
                                         0.177342328237912,
                                         0.139497277304296,
                                         0.0577941985025474,
                                         0.0125192411355107,
                                         0.00135578505383336,
                                         6.71488729425315e-05,
                                         1.30629419524323e-06,
                                         7.72436846504893e-09,
                                         8.8198692012186e-12,
                                         7.96304425426173e-16,
                                         6.73058600297151e-22,
                                         2.38417811643228e-33,
                                         4.60588984355477e-14,
                                         1.98712813400685e-08,
                                         1.48125517051882e-05,
                                         0.000896481865524104,
                                         0.0126916752205636,
                                         0.0648823885125338,
                                         0.147790094256172,
                                         0.166724151904827,
                                         0.0975219634206715,
                                         0.0296769868820583,
                                         0.00453751472618256,
                                         0.000322304076632875,
                                         9.2566314381765e-06,
                                         8.47659091453961e-08,
                                         1.62087047461379e-10,
                                         2.80655927654092e-14,
                                         5.95971852238684e-20,
                                         1.12586624818466e-30,
                                         3.25120376995849e-16,
                                         7.48051928661722e-10,
                                         1.40092064249729e-06,
                                         0.000162605071760819,
                                         0.00385513377688225,
                                         0.0305206373951138,
                                         0.102635357734723,
                                         0.166054273397204,
                                         0.137132180447195,
                                         0.0586246629531255,
                                         0.0126550541278095,
                                         0.00128917425393814,
                                         5.46617713772887e-05,
                                         7.75172255802881e-07,
                                         2.48229408276612e-09,
                                         8.24305244873567e-13,
                                         4.39761669716917e-18,
                                         4.43050933731628e-28,
                                         1.927765123087e-18,
                                         2.36546707710686e-11,
                                         1.11295210706039e-07,
                                         2.4774560109347e-05,
                                         0.000983646933180714,
                                         0.0120597874989177,
                                         0.0598725783124775,
                                         0.138925152563017,
                                         0.161977843491422,
                                         0.0972792467382905,
                                         0.0296475802062069,
                                         0.00433148420112739,
                                         0.000271140077991581,
                                         5.95462618249164e-06,
                                         3.19328321927468e-08,
                                         2.03367332954878e-11,
                                         2.72576420107166e-16,
                                         1.46453514674106e-25,
                                         9.64909715397132e-21,
                                         6.31429256014771e-13,
                                         7.46383532327712e-09,
                                         3.18640113485377e-06,
                                         0.000211866193001582,
                                         0.00402261383473019,
                                         0.0294836707372829,
                                         0.0981147585914066,
                                         0.161508162177806,
                                         0.136264482354138,
                                         0.0586323268511396,
                                         0.0122852635368454,
                                         0.00113534110915154,
                                         3.86129885594526e-05,
                                         3.46772202618118e-07,
                                         4.23542459855193e-10,
                                         1.42620457541262e-14,
                                         4.08666061620033e-23,
                                         4.08666061620033e-23,
                                         1.42620457541262e-14,
                                         4.23542459855193e-10,
                                         3.46772202618116e-07,
                                         3.86129885594526e-05,
                                         0.00113534110915154,
                                         0.0122852635368454,
                                         0.0586323268511397,
                                         0.136264482354138,
                                         0.161508162177806,
                                         0.0981147585914066,
                                         0.0294836707372829,
                                         0.00402261383473019,
                                         0.000211866193001582,
                                         3.18640113485379e-06,
                                         7.46383532327712e-09,
                                         6.31429256014771e-13,
                                         9.64909715397132e-21,
                                         1.46453514674106e-25,
                                         2.72576420107166e-16,
                                         2.03367332954878e-11,
                                         3.19328321927466e-08,
                                         5.95462618249164e-06,
                                         0.000271140077991581,
                                         0.00433148420112739,
                                         0.0296475802062069,
                                         0.0972792467382905,
                                         0.161977843491422,
                                         0.138925152563017,
                                         0.0598725783124775,
                                         0.0120597874989177,
                                         0.000983646933180714,
                                         2.47745601093472e-05,
                                         1.11295210706039e-07,
                                         2.36546707710686e-11,
                                         1.927765123087e-18,
                                         4.43050933731628e-28,
                                         4.39761669716917e-18,
                                         8.24305244873567e-13,
                                         2.4822940827661e-09,
                                         7.75172255802881e-07,
                                         5.46617713772887e-05,
                                         0.00128917425393814,
                                         0.0126550541278095,
                                         0.0586246629531255,
                                         0.137132180447195,
                                         0.166054273397204,
                                         0.102635357734723,
                                         0.0305206373951138,
                                         0.00385513377688225,
                                         0.00016260507176082,
                                         1.40092064249729e-06,
                                         7.48051928661722e-10,
                                         3.25120376995849e-16,
                                         1.12586624818466e-30,
                                         5.95971852238684e-20,
                                         2.80655927654092e-14,
                                         1.62087047461378e-10,
                                         8.47659091453961e-08,
                                         9.2566314381765e-06,
                                         0.000322304076632875,
                                         0.00453751472618256,
                                         0.0296769868820583,
                                         0.0975219634206715,
                                         0.166724151904827,
                                         0.147790094256172,
                                         0.0648823885125338,
                                         0.0126916752205636,
                                         0.000896481865524108,
                                         1.48125517051882e-05,
                                         1.98712813400685e-08,
                                         4.60588984355477e-14,
                                         2.38417811643228e-33,
                                         6.73058600297151e-22,
                                         7.96304425426173e-16,
                                         8.81986920121852e-12,
                                         7.72436846504893e-09,
                                         1.30629419524323e-06,
                                         6.71488729425315e-05,
                                         0.00135578505383336,
                                         0.0125192411355107,
                                         0.0577941985025474,
                                         0.139497277304296,
                                         0.177342328237912,
                                         0.114942016789961,
                                         0.0348190727020929,
                                         0.00411877136073048,
                                         0.000130516367453279,
                                         4.39884593309116e-07,
                                         5.43752989969e-12,
                                         4.15785853784396e-36,
                                         6.2597807442358e-24,
                                         1.86064342945168e-17,
                                         3.95234722993681e-13,
                                         5.79674043766673e-10,
                                         1.5181271110352e-07,
                                         1.15210157664861e-05,
                                         0.000333612834904313,
                                         0.00434925947888644,
                                         0.0282062381506286,
                                         0.0961196232155337,
                                         0.17525023127997,
                                         0.167691060496314,
                                         0.0786673582055174,
                                         0.0155837859369487,
                                         0.000947063700042595,
                                         8.01919445712484e-06,
                                         5.28650787425449e-10,
                                         5.86989520477082e-39,
                                         4.71297343999958e-26,
                                         3.51946624798323e-19,
                                         1.43376398265865e-14,
                                         3.52155400586682e-11,
                                         1.42825220026067e-08,
                                         1.60019327494448e-06,
                                         6.64544833644023e-05,
                                         0.00122315715301504,
                                         0.0111438618167526,
                                         0.053615210437481,
                                         0.140195611872566,
                                         0.198048079845994,
                                         0.14388037392161,
                                         0.0477318085858759,
                                         0.00556317895509871,
                                         0.00011834566787015,
                                         4.16069360573619e-08,
                                         6.5422727262724e-42,
                                         2.80135740502384e-28,
                                         5.25567030707934e-21,
                                         4.10617919929743e-16,
                                         1.68897223356411e-12,
                                         1.06081420257279e-09,
                                         1.75465512380808e-07,
                                         1.04506542437598e-05,
                                         0.000271573167842789,
                                         0.00347587320916557,
                                         0.0236103066768347,
                                         0.0885417087602064,
                                         0.184658379088984,
                                         0.20775247566107,
                                         0.115419839168772,
                                         0.025799098350869,
                                         0.00137883291787705,
                                         2.58523630133207e-06,
                                         5.54166837876539e-45,
                                         1.26548103698178e-30,
                                         5.96476056513775e-23,
                                         8.93741085960546e-18,
                                         6.15636356210516e-14,
                                         5.98807652354964e-11,
                                         1.46226030024136e-08,
                                         1.24903976573009e-06,
                                         4.58252717861172e-05,
                                         0.000823959236151392,
                                         0.0079018509558663,
                                         0.0424986339083883,
                                         0.130852189607988,
                                         0.227984042401657,
                                         0.212112680160253,
                                         0.0909284144242596,
                                         0.0122091242520645,
                                         0.000122081075872261,
                                         3.35292922823538e-48,
                                         4.08333232639424e-33,
                                         4.83537188280772e-25,
                                         1.38949667590634e-19,
                                         1.60286878931595e-15,
                                         2.41438943548687e-12,
                                         8.70421427895949e-10,
                                         1.06630394340187e-07,
                                         5.52325640518976e-06,
                                         0.000139514546208944,
                                         0.00188898280082751,
                                         0.0145704849110046,
                                         0.0662315522846564,
                                         0.178704157519355,
                                         0.278435600281958,
                                         0.228911005151781,
                                         0.0772199201791292,
                                         0.00411783001324243,
                                         1.29096240416809e-51,
                                         8.38453857060938e-36,
                                         2.49443448168367e-27,
                                         1.37470228934655e-21,
                                         2.65568793669084e-17,
                                         6.1948765658583e-14,
                                         3.29715841234881e-11,
                                         5.79283449019475e-09,
                                         4.23633909053852e-07,
                                         1.5032756062459e-05,
                                         0.000287364111295647,
                                         0.00317891119425833,
                                         0.0213331127269614,
                                         0.0891395049360031,
                                         0.232588505711646,
                                         0.366723787637786,
                                         0.31079894649833,
                                         0.0883881077149673,
                                         2.37721056669813e-55,
                                         8.23395437257302e-39,
                                         6.15430720301608e-30,
                                         6.5046607189056e-24,
                                         2.10436445890154e-19,
                                         7.60191206870948e-16,
                                         5.97330790754278e-13,
                                         1.50510249027386e-10,
                                         1.55400021534224e-08,
                                         7.7468041886019e-07,
                                         2.09074902219558e-05,
                                         0.000331701652406124,
                                         0.00328630881528693,
                                         0.0212652566351485,
                                         0.0929215795366407,
                                         0.280980602736584,
                                         0.598266350762044,
                                         0.907369353818546,

                                         0.916230344976757,
                                         0.629883878508197,
                                         0.319315888613649,
                                         0.118260544031952,
                                         0.031555408509971,
                                         0.00594940382932986,
                                         0.000772054991855594,
                                         6.66003171408205e-05,
                                         3.64696567920487e-06,
                                         1.19209289550781e-07,
                                         2.1402670769559e-09,
                                         1.88087192498869e-11,
                                         6.86212129583064e-14,
                                         8.12948995050786e-17,
                                         2.1203991550838e-20,
                                         6.24863649034268e-25,
                                         5.70154529798842e-31,
                                         7.44002804064502e-40,
                                         2.11847684887014e-56,
                                         0.0803113292054302,
                                         0.294089991972312,
                                         0.373720696563079,
                                         0.264558583049271,
                                         0.117675528258288,
                                         0.0341507969845405,
                                         0.00649184014485415,
                                         0.000795169864383618,
                                         6.06961424513179e-05,
                                         2.74181365966797e-06,
                                         6.80290222225023e-08,
                                         8.3335736998261e-10,
                                         4.31712357833951e-12,
                                         7.49189316818188e-15,
                                         3.00788303907048e-18,
                                         1.47760703232782e-22,
                                         2.57704279319238e-28,
                                         8.42966841691813e-37,
                                         1.27851919229754e-52,
                                         0.00336677276650351,
                                         0.0656696720046545,
                                         0.209188891984012,
                                         0.283053603607643,
                                         0.20987627884898,
                                         0.0937547083598132,
                                         0.0261067162099617,
                                         0.00454054910929147,
                                         0.00048312037479181,
                                         3.01599502563477e-05,
                                         1.03415428561846e-06,
                                         1.76590845647837e-08,
                                         1.29895892981036e-10,
                                         3.30205803827766e-13,
                                         2.04065278495773e-16,
                                         1.67108094555463e-20,
                                         5.57077388057573e-26,
                                         4.56784396826413e-34,
                                         3.69024865269395e-49,
                                         8.98165051317487e-05,
                                         0.00933157183558922,
                                         0.0745135825481823,
                                         0.192717376841208,
                                         0.238202277311441,
                                         0.16379124570062,
                                         0.066810086514447,
                                         0.0164991737742019,
                                         0.00244711833822402,
                                         0.000211119651794434,
                                         1.00041869306304e-05,
                                         2.381279903863e-07,
                                         2.48714848608894e-09,
                                         9.26153950721899e-12,
                                         8.81013660760203e-15,
                                         1.20265588263497e-18,
                                         7.66328162064425e-24,
                                         1.5751335575637e-31,
                                         6.77812172403377e-46,
                                         1.71147461924309e-06,
                                         0.000947145552935979,
                                         0.0189585113773261,
                                         0.0937227509034908,
                                         0.193108071736143,
                                         0.204390284860617,
                                         0.122124777180208,
                                         0.0428240739609247,
                                         0.00885373571139025,
                                         0.00105559825897217,
                                         6.91274002720445e-05,
                                         2.29363692573365e-06,
                                         3.40157438495633e-08,
                                         1.85546868583723e-10,
                                         2.71686546358243e-13,
                                         6.18240194375866e-17,
                                         7.529844462271e-22,
                                         3.87967612197785e-29,
                                         8.89272907943467e-43,
                                         2.47855389128383e-08,
                                         7.30621145967114e-05,
                                         0.00366595065527766,
                                         0.0346403898676875,
                                         0.118978513223365,
                                         0.1938400013914,
                                         0.169659868806616,
                                         0.0844748358395655,
                                         0.0243451092925356,
                                         0.00401127338409424,
                                         0.000363021413048852,
                                         1.67900693123122e-05,
                                         3.53567081114863e-07,
                                         2.82512499298164e-09,
                                         6.36747452364897e-12,
                                         2.41538678711156e-15,
                                         5.62303539513301e-20,
                                         7.26251683342454e-27,
                                         8.866951880174e-40,
                                         2.83376562323554e-10,
                                         4.44944055391315e-06,
                                         0.000559637302328257,
                                         0.010107835683476,
                                         0.0578727748347019,
                                         0.145132346722071,
                                         0.186076756568344,
                                         0.131554100853957,
                                         0.0528487307313756,
                                         0.0120338201522827,
                                         0.00150505343619583,
                                         9.70326660831612e-05,
                                         2.9013580810577e-06,
                                         3.39593495485346e-08,
                                         1.17815914059373e-10,
                                         7.44995747321344e-14,
                                         3.31507445717256e-18,
                                         1.07328853513909e-24,
                                         6.97993373749967e-37,
                                         2.6227634052952e-12,
                                         2.19355337633863e-07,
                                         6.91602127554962e-05,
                                         0.00238760897061915,
                                         0.0227881831439703,
                                         0.0879659622407367,
                                         0.165209400273067,
                                         0.165848310385279,
                                         0.0928724752769456,
                                         0.0292249917984009,
                                         0.00505127744331877,
                                         0.000453955274182543,
                                         1.92734946466152e-05,
                                         3.30453750990649e-07,
                                         1.76469772030739e-09,
                                         1.86016096988092e-12,
                                         1.58214172370039e-16,
                                         1.28403084098031e-22,
                                         4.44792929032377e-34,
                                         1.99909492976056e-14,
                                         8.90574219277024e-09,
                                         7.03858080926673e-06,
                                         0.000464458959490009,
                                         0.0073896558084515,
                                         0.0439080562664148,
                                         0.120797096660315,
                                         0.172185599289554,
                                         0.134405980801314,
                                         0.0584499835968018,
                                         0.0139614215580213,
                                         0.00174898982317457,
                                         0.000105438378231471,
                                         2.64814110700572e-06,
                                         2.17678635576606e-08,
                                         3.82495540476604e-11,
                                         6.21837050312225e-15,
                                         1.26506702106954e-20,
                                         2.3342295552516e-31,
                                         1.26977412771517e-16,
                                         3.01308084786127e-10,
                                         5.9694268610436e-07,
                                         7.52922441464527e-05,
                                         0.00199690531939247,
                                         0.0182638579361196,
                                         0.0736032500644792,
                                         0.148970870499214,
                                         0.162094739958327,
                                         0.0974166393280029,
                                         0.0321570952608898,
                                         0.00561539424611935,
                                         0.000480679633175017,
                                         1.76843993589386e-05,
                                         2.23758757856081e-07,
                                         6.55421944898741e-10,
                                         2.03669763665156e-13,
                                         1.03865273112386e-18,
                                         1.02081726517537e-28,
                                         6.77483646017087e-19,
                                         8.56309450617102e-12,
                                         4.25264818755009e-08,
                                         1.02525624853062e-05,
                                         0.000453283617627787,
                                         0.00638146093138173,
                                         0.0376718348259821,
                                         0.108264274672462,
                                         0.164209597184805,
                                         0.136383295059204,
                                         0.0622161696005814,
                                         0.0151444152402083,
                                         0.00184073813961088,
                                         9.9201626031793e-05,
                                         1.9320731467801e-06,
                                         9.43397816529953e-09,
                                         5.60345399054267e-12,
                                         7.16319026137881e-17,
                                         3.75000404176166e-26,
                                         3.05136239599655e-21,
                                         2.05434472268855e-13,
                                         2.55746034182036e-09,
                                         1.17852076575697e-06,
                                         8.68570758348381e-05,
                                         0.00188221981263867,
                                         0.0162764311807297,
                                         0.0664188905914496,
                                         0.140427052701893,
                                         0.161180257797241,
                                         0.101613721897434,
                                         0.0344784232473289,
                                         0.00595046537114054,
                                         0.000469753364448657,
                                         1.40828229019622e-05,
                                         1.14628177255279e-07,
                                         1.30139068152064e-10,
                                         4.17028021125253e-15,
                                         1.16288855532743e-23,
                                         1.16288855532743e-23,
                                         4.17028021125253e-15,
                                         1.30139068152064e-10,
                                         1.14628177255279e-07,
                                         1.40828229019621e-05,
                                         0.000469753364448659,
                                         0.00595046537114054,
                                         0.0344784232473288,
                                         0.101613721897434,
                                         0.161180257797241,
                                         0.140427052701893,
                                         0.0664188905914498,
                                         0.0162764311807297,
                                         0.00188221981263866,
                                         8.68570758348385e-05,
                                         1.17852076575697e-06,
                                         2.55746034182036e-09,
                                         2.05434472268855e-13,
                                         3.05136239599655e-21,
                                         3.75000404176166e-26,
                                         7.16319026137881e-17,
                                         5.60345399054267e-12,
                                         9.43397816529953e-09,
                                         1.93207314678009e-06,
                                         9.92016260317934e-05,
                                         0.00184073813961088,
                                         0.0151444152402083,
                                         0.0622161696005814,
                                         0.136383295059204,
                                         0.164209597184805,
                                         0.108264274672462,
                                         0.0376718348259821,
                                         0.00638146093138171,
                                         0.000453283617627789,
                                         1.02525624853062e-05,
                                         4.25264818755009e-08,
                                         8.56309450617102e-12,
                                         6.77483646017088e-19,
                                         1.02081726517537e-28,
                                         1.03865273112386e-18,
                                         2.03669763665156e-13,
                                         6.55421944898741e-10,
                                         2.2375875785608e-07,
                                         1.76843993589387e-05,
                                         0.000480679633175017,
                                         0.00561539424611934,
                                         0.0321570952608898,
                                         0.0974166393280029,
                                         0.162094739958327,
                                         0.148970870499214,
                                         0.0736032500644791,
                                         0.0182638579361195,
                                         0.00199690531939248,
                                         7.52922441464527e-05,
                                         5.9694268610436e-07,
                                         3.01308084786127e-10,
                                         1.26977412771517e-16,
                                         2.3342295552516e-31,
                                         1.26506702106954e-20,
                                         6.21837050312225e-15,
                                         3.82495540476604e-11,
                                         2.17678635576605e-08,
                                         2.64814110700573e-06,
                                         0.000105438378231471,
                                         0.00174898982317457,
                                         0.0139614215580213,
                                         0.0584499835968018,
                                         0.134405980801314,
                                         0.172185599289554,
                                         0.120797096660315,
                                         0.0439080562664147,
                                         0.00738965580845153,
                                         0.000464458959490009,
                                         7.03858080926673e-06,
                                         8.90574219277024e-09,
                                         1.99909492976056e-14,
                                         4.44792929032377e-34,
                                         1.28403084098031e-22,
                                         1.58214172370039e-16,
                                         1.86016096988092e-12,
                                         1.76469772030738e-09,
                                         3.30453750990651e-07,
                                         1.92734946466152e-05,
                                         0.000453955274182542,
                                         0.00505127744331877,
                                         0.0292249917984009,
                                         0.0928724752769456,
                                         0.16584831038528,
                                         0.165209400273067,
                                         0.0879659622407366,
                                         0.0227881831439704,
                                         0.00238760897061915,
                                         6.91602127554963e-05,
                                         2.19355337633863e-07,
                                         2.6227634052952e-12,
                                         6.97993373749967e-37,
                                         1.07328853513909e-24,
                                         3.31507445717256e-18,
                                         7.44995747321344e-14,
                                         1.17815914059372e-10,
                                         3.39593495485348e-08,
                                         2.9013580810577e-06,
                                         9.70326660831609e-05,
                                         0.00150505343619583,
                                         0.0120338201522827,
                                         0.0528487307313756,
                                         0.131554100853957,
                                         0.186076756568344,
                                         0.145132346722071,
                                         0.0578727748347021,
                                         0.010107835683476,
                                         0.000559637302328257,
                                         4.44944055391315e-06,
                                         2.83376562323554e-10,
                                         8.866951880174e-40,
                                         7.26251683342454e-27,
                                         5.62303539513301e-20,
                                         2.41538678711156e-15,
                                         6.36747452364892e-12,
                                         2.82512499298165e-09,
                                         3.53567081114863e-07,
                                         1.67900693123121e-05,
                                         0.000363021413048852,
                                         0.00401127338409424,
                                         0.0243451092925356,
                                         0.0844748358395655,
                                         0.169659868806616,
                                         0.193840001391399,
                                         0.118978513223365,
                                         0.0346403898676875,
                                         0.00366595065527766,
                                         7.30621145967114e-05,
                                         2.47855389128383e-08,
                                         8.89272907943467e-43,
                                         3.87967612197785e-29,
                                         7.529844462271e-22,
                                         6.18240194375866e-17,
                                         2.71686546358241e-13,
                                         1.85546868583724e-10,
                                         3.40157438495633e-08,
                                         2.29363692573365e-06,
                                         6.91274002720445e-05,
                                         0.00105559825897217,
                                         0.00885373571139025,
                                         0.0428240739609247,
                                         0.122124777180208,
                                         0.204390284860617,
                                         0.193108071736143,
                                         0.0937227509034908,
                                         0.0189585113773261,
                                         0.000947145552935979,
                                         1.71147461924309e-06,
                                         6.77812172403377e-46,
                                         1.5751335575637e-31,
                                         7.66328162064425e-24,
                                         1.20265588263497e-18,
                                         8.81013660760196e-15,
                                         9.26153950721904e-12,
                                         2.48714848608894e-09,
                                         2.38127990386299e-07,
                                         1.00041869306304e-05,
                                         0.000211119651794434,
                                         0.00244711833822402,
                                         0.0164991737742019,
                                         0.066810086514447,
                                         0.16379124570062,
                                         0.238202277311442,
                                         0.192717376841208,
                                         0.0745135825481823,
                                         0.00933157183558922,
                                         8.98165051317487e-05,
                                         3.69024865269395e-49,
                                         4.56784396826413e-34,
                                         5.57077388057573e-26,
                                         1.67108094555463e-20,
                                         2.04065278495771e-16,
                                         3.30205803827767e-13,
                                         1.29895892981036e-10,
                                         1.76590845647837e-08,
                                         1.03415428561846e-06,
                                         3.01599502563477e-05,
                                         0.00048312037479181,
                                         0.00454054910929147,
                                         0.0261067162099617,
                                         0.0937547083598131,
                                         0.20987627884898,
                                         0.283053603607643,
                                         0.209188891984012,
                                         0.0656696720046545,
                                         0.00336677276650351,
                                         1.27851919229754e-52,
                                         8.42966841691813e-37,
                                         2.57704279319238e-28,
                                         1.47760703232782e-22,
                                         3.00788303907046e-18,
                                         7.49189316818193e-15,
                                         4.31712357833951e-12,
                                         8.33357369982607e-10,
                                         6.80290222225023e-08,
                                         2.74181365966797e-06,
                                         6.06961424513179e-05,
                                         0.000795169864383619,
                                         0.00649184014485415,
                                         0.0341507969845404,
                                         0.117675528258288,
                                         0.264558583049271,
                                         0.373720696563079,
                                         0.294089991972312,
                                         0.0803113292054302,
                                         2.11847684887014e-56,
                                         7.44002804064502e-40,
                                         5.70154529798842e-31,
                                         6.24863649034268e-25,
                                         2.12039915508378e-20,
                                         8.12948995050791e-17,
                                         6.86212129583064e-14,
                                         1.88087192498868e-11,
                                         2.1402670769559e-09,
                                         1.19209289550781e-07,
                                         3.64696567920487e-06,
                                         6.66003171408205e-05,
                                         0.000772054991855594,
                                         0.00594940382932986,
                                         0.031555408509971,
                                         0.118260544031952,
                                         0.319315888613649,
                                         0.629883878508197,
                                         0.916230344976757,

                                         0.923894706787976,
                                         0.658296342395684,
                                         0.356251055656854,
                                         0.145319936697437,
                                         0.0441800541634687,
                                         0.00985493428127353,
                                         0.00157947781453995,
                                         0.000176957116555841,
                                         1.33699708064893e-05,
                                         6.49948536568347e-07,
                                         1.91016961388514e-08,
                                         3.12088997290648e-10,
                                         2.52469232787139e-12,
                                         8.57294886305569e-15,
                                         9.55363677270148e-18,
                                         2.36823357282616e-21,
                                         6.69954698199224e-26,
                                         5.92606467097962e-32,
                                         7.56926666308607e-41,
                                         2.12986259840953e-57,
                                         0.0732588802374385,
                                         0.277750617431637,
                                         0.376068579170415,
                                         0.292384210498811,
                                         0.147602303792055,
                                         0.0504221489269558,
                                         0.0117609415712649,
                                         0.00185558236078274,
                                         0.000193407550828887,
                                         1.28234977591549e-05,
                                         5.12153416575624e-07,
                                         1.14127725863991e-08,
                                         1.27365536675551e-10,
                                         6.09056164192613e-13,
                                         9.87772154592984e-16,
                                         3.74984497366892e-19,
                                         1.76145840892054e-23,
                                         2.96969036067873e-29,
                                         9.4902113991564e-38,
                                         1.42091945945547e-53,
                                         0.00277819626152179,
                                         0.0560471421963294,
                                         0.189864070628434,
                                         0.281350363717171,
                                         0.235844024570378,
                                         0.123382571738377,
                                         0.0418827803077752,
                                         0.0093058765509676,
                                         0.001338077416497,
                                         0.000121003763863208,
                                         6.56739390242677e-06,
                                         1.99603760136402e-07,
                                         3.07298364359529e-09,
                                         2.06942314448583e-11,
                                         4.88438267410889e-14,
                                         2.83966391415537e-17,
                                         2.21495138478338e-21,
                                         7.11739061156853e-27,
                                         5.69066233634396e-35,
                                         4.53369380840035e-50,
                                         6.70456991296717e-05,
                                         0.00719709554300618,
                                         0.0609991637371123,
                                         0.172284576952291,
                                         0.239807064767895,
                                         0.192128432966313,
                                         0.0949148727881911,
                                         0.0296988541409242,
                                         0.00589107309412486,
                                         0.000726602040952306,
                                         5.35909445234908e-05,
                                         2.22152716347817e-06,
                                         4.71817366569946e-08,
                                         4.47452167034915e-10,
                                         1.53697902141262e-12,
                                         1.36844052375622e-15,
                                         1.77239843801944e-19,
                                         1.08551489689028e-24,
                                         2.17147634115387e-32,
                                         9.20534913188169e-47,
                                         1.15571537085932e-06,
                                         0.00066013540725762,
                                         0.0139983529280831,
                                         0.0753559041460793,
                                         0.174169070358354,
                                         0.213698477102601,
                                         0.153640248368648,
                                         0.0677008410801435,
                                         0.0185259098360353,
                                         0.00311649411945383,
                                         0.000312364536678112,
                                         1.76606426033359e-05,
                                         5.17439478127311e-07,
                                         6.91060219634251e-09,
                                         3.45460312691333e-11,
                                         4.71039009159645e-14,
                                         1.01304907208445e-17,
                                         1.18255878521332e-22,
                                         5.91860492742924e-30,
                                         1.33505853569469e-43,
                                         1.51406474567661e-08,
                                         4.6017431964951e-05,
                                         0.00244142613189243,
                                         0.0250496557294658,
                                         0.0961376908354587,
                                         0.180644526123955,
                                         0.189011976429732,
                                         0.117290281117234,
                                         0.0442770087256842,
                                         0.0101589682348458,
                                         0.00138371173577348,
                                         0.000106702591685879,
                                         4.31279475272511e-06,
                                         8.11145501981651e-08,
                                         5.9012223164036e-10,
                                         1.23225734452246e-12,
                                         4.40061322743236e-16,
                                         9.7909150414559e-21,
                                         1.2260189785888e-27,
                                         1.47154634096845e-40,
                                         1.56594196654167e-10,
                                         2.5324996181816e-06,
                                         0.000336161441058727,
                                         0.00657391214965678,
                                         0.0418942092187744,
                                         0.120555170935201,
                                         0.183574056311179,
                                         0.160423374810705,
                                         0.0835438832642657,
                                         0.0261439106794583,
                                         0.00483912883884505,
                                         0.000508957000641918,
                                         2.8378907911048e-05,
                                         7.51656284249532e-07,
                                         7.95835757658749e-09,
                                         2.54497554778765e-11,
                                         1.5091540539726e-14,
                                         6.39972751956872e-19,
                                         2.00499222495096e-25,
                                         1.28051667027557e-37,
                                         1.31110207812118e-12,
                                         1.12825177348084e-07,
                                         3.7469839385979e-05,
                                         0.00139661164001678,
                                         0.0147789620747958,
                                         0.0651293144523983,
                                         0.144332093805091,
                                         0.17762451130759,
                                         0.127608813284643,
                                         0.0544654537971851,
                                         0.0136999307673024,
                                         0.00196524564879446,
                                         0.000151168809783435,
                                         5.63857612217834e-06,
                                         8.68829478318143e-08,
                                         4.25495958406166e-10,
                                         4.18970626479016e-13,
                                         3.38633026040334e-17,
                                         2.65434774662752e-23,
                                         9.02040748773768e-35,
                                         9.04016719413783e-15,
                                         4.13944174855186e-09,
                                         3.43949656847117e-06,
                                         0.000244346730824424,
                                         0.0042935150187004,
                                         0.0289765238769303,
                                         0.0934530831190879,
                                         0.16196353913066,
                                         0.16051875992185,
                                         0.093443869907023,
                                         0.0319410121594317,
                                         0.00624930482088361,
                                         0.000663143967015221,
                                         3.48336286243795e-05,
                                         7.81132596017797e-07,
                                         5.85849925282623e-09,
                                         9.5788349906129e-12,
                                         1.47562584743724e-15,
                                         2.89389027263404e-21,
                                         5.23294525364159e-32,
                                         5.19439741202799e-17,
                                         1.26559945752695e-10,
                                         2.63103535145116e-07,
                                         3.56251057642303e-05,
                                         0.00103944327092067,
                                         0.0107432285938869,
                                         0.0504246752184409,
                                         0.123069482427751,
                                         0.168263406105649,
                                         0.133597785606074,
                                         0.0620579935218809,
                                         0.0165601904814782,
                                         0.00242422098413625,
                                         0.000179327436698154,
                                         5.85239631585178e-06,
                                         6.72196230016883e-08,
                                         1.82499030517727e-10,
                                         5.35848414601454e-14,
                                         2.62920866362438e-19,
                                         2.52979295972636e-29,
                                         2.50710873720642e-19,
                                         3.25034956576066e-12,
                                         1.6905879629578e-08,
                                         4.36299864295961e-06,
                                         0.000211381942151214,
                                         0.00334582047636208,
                                         0.0228545077407657,
                                         0.0785530000919853,
                                         0.14816063887093,
                                         0.160445296227016,
                                         0.101280554736982,
                                         0.0368619438449821,
                                         0.00744416302444469,
                                         0.000775486157804058,
                                         3.68317184078856e-05,
                                         6.47865796009077e-07,
                                         2.92070515724945e-09,
                                         1.6345075555625e-12,
                                         2.00653775310278e-17,
                                         1.02731363915704e-26,
                                         1.02148924583012e-21,
                                         7.04671152146681e-14,
                                         9.17004555027439e-10,
                                         4.5106239851184e-07,
                                         3.62875466407661e-05,
                                         0.000879615926458215,
                                         0.0087442640781557,
                                         0.0423250821577116,
                                         0.110128225014109,
                                         0.162658720055648,
                                         0.139533036231752,
                                         0.0692649890577216,
                                         0.0192966613948567,
                                         0.00283089649767941,
                                         0.000195673884408635,
                                         5.27104426229044e-06,
                                         3.94582213538147e-08,
                                         4.20876373100965e-11,
                                         1.29268362556921e-15,
                                         3.52163028471166e-24,
                                         3.52163028471098e-24,
                                         1.29268362556916e-15,
                                         4.20876373100965e-11,
                                         3.94582213538144e-08,
                                         5.27104426229044e-06,
                                         0.000195673884408636,
                                         0.00283089649767941,
                                         0.0192966613948567,
                                         0.0692649890577216,
                                         0.139533036231752,
                                         0.162658720055648,
                                         0.110128225014109,
                                         0.0423250821577116,
                                         0.0087442640781557,
                                         0.000879615926458212,
                                         3.62875466407661e-05,
                                         4.51062398511844e-07,
                                         9.17004555027439e-10,
                                         7.04671152146705e-14,
                                         1.0214892458303e-21,
                                         1.02731363915682e-26,
                                         2.0065377531027e-17,
                                         1.6345075555625e-12,
                                         2.92070515724942e-09,
                                         6.47865796009077e-07,
                                         3.68317184078858e-05,
                                         0.000775486157804058,
                                         0.00744416302444469,
                                         0.0368619438449821,
                                         0.101280554736982,
                                         0.160445296227016,
                                         0.14816063887093,
                                         0.0785530000919853,
                                         0.0228545077407657,
                                         0.00334582047636207,
                                         0.000211381942151214,
                                         4.36299864295964e-06,
                                         1.6905879629578e-08,
                                         3.25034956576076e-12,
                                         2.50710873720682e-19,
                                         2.52979295972579e-29,
                                         2.62920866362427e-19,
                                         5.35848414601454e-14,
                                         1.82499030517725e-10,
                                         6.72196230016883e-08,
                                         5.8523963158518e-06,
                                         0.000179327436698154,
                                         0.00242422098413625,
                                         0.0165601904814782,
                                         0.0620579935218809,
                                         0.133597785606074,
                                         0.168263406105649,
                                         0.123069482427751,
                                         0.0504246752184409,
                                         0.0107432285938869,
                                         0.00103944327092067,
                                         3.56251057642305e-05,
                                         2.63103535145116e-07,
                                         1.26559945752699e-10,
                                         5.19439741202874e-17,
                                         5.23294525364032e-32,
                                         2.8938902726339e-21,
                                         1.47562584743724e-15,
                                         9.5788349906128e-12,
                                         5.85849925282623e-09,
                                         7.811325960178e-07,
                                         3.48336286243795e-05,
                                         0.000663143967015221,
                                         0.00624930482088361,
                                         0.0319410121594317,
                                         0.093443869907023,
                                         0.16051875992185,
                                         0.16196353913066,
                                         0.0934530831190879,
                                         0.0289765238769302,
                                         0.0042935150187004,
                                         0.000244346730824426,
                                         3.43949656847117e-06,
                                         4.13944174855196e-09,
                                         9.040167194139e-15,
                                         9.02040748773534e-35,
                                         2.65434774662739e-23,
                                         3.38633026040334e-17,
                                         4.18970626479011e-13,
                                         4.25495958406166e-10,
                                         8.68829478318147e-08,
                                         5.63857612217834e-06,
                                         0.000151168809783435,
                                         0.00196524564879446,
                                         0.0136999307673024,
                                         0.0544654537971851,
                                         0.127608813284643,
                                         0.17762451130759,
                                         0.144332093805091,
                                         0.0651293144523982,
                                         0.0147789620747958,
                                         0.00139661164001679,
                                         3.7469839385979e-05,
                                         1.12825177348087e-07,
                                         1.31110207812132e-12,
                                         1.28051667027522e-37,
                                         2.00499222495086e-25,
                                         6.39972751956872e-19,
                                         1.50915405397258e-14,
                                         2.54497554778765e-11,
                                         7.95835757658753e-09,
                                         7.51656284249532e-07,
                                         2.8378907911048e-05,
                                         0.000508957000641918,
                                         0.00483912883884505,
                                         0.0261439106794583,
                                         0.0835438832642657,
                                         0.160423374810705,
                                         0.183574056311179,
                                         0.120555170935201,
                                         0.0418942092187744,
                                         0.00657391214965681,
                                         0.000336161441058727,
                                         2.53249961818165e-06,
                                         1.56594196654183e-10,
                                         1.47154634096802e-40,
                                         1.22601897858874e-27,
                                         9.7909150414559e-21,
                                         4.4006132274323e-16,
                                         1.23225734452246e-12,
                                         5.90122231640363e-10,
                                         8.11145501981651e-08,
                                         4.31279475272511e-06,
                                         0.000106702591685879,
                                         0.00138371173577348,
                                         0.0101589682348458,
                                         0.0442770087256842,
                                         0.117290281117234,
                                         0.189011976429732,
                                         0.180644526123955,
                                         0.0961376908354587,
                                         0.0250496557294659,
                                         0.00244142613189243,
                                         4.60174319649517e-05,
                                         1.51406474567673e-08,
                                         1.33505853569428e-43,
                                         5.91860492742889e-30,
                                         1.18255878521332e-22,
                                         1.01304907208443e-17,
                                         4.71039009159645e-14,
                                         3.45460312691335e-11,
                                         6.91060219634251e-09,
                                         5.17439478127311e-07,
                                         1.76606426033359e-05,
                                         0.000312364536678112,
                                         0.00311649411945383,
                                         0.0185259098360353,
                                         0.0677008410801436,
                                         0.153640248368648,
                                         0.2136984771026,
                                         0.174169070358354,
                                         0.0753559041460796,
                                         0.0139983529280831,
                                         0.000660135407257628,
                                         1.15571537085939e-06,
                                         9.20534913187872e-47,
                                         2.17147634115373e-32,
                                         1.08551489689028e-24,
                                         1.77239843801941e-19,
                                         1.36844052375622e-15,
                                         1.53697902141263e-12,
                                         4.47452167034914e-10,
                                         4.71817366569946e-08,
                                         2.22152716347817e-06,
                                         5.35909445234908e-05,
                                         0.000726602040952306,
                                         0.00589107309412486,
                                         0.0296988541409242,
                                         0.0949148727881911,
                                         0.192128432966313,
                                         0.239807064767895,
                                         0.172284576952292,
                                         0.0609991637371123,
                                         0.00719709554300625,
                                         6.7045699129675e-05,
                                         4.53369380839882e-50,
                                         5.69066233634359e-35,
                                         7.11739061156853e-27,
                                         2.21495138478334e-21,
                                         2.83966391415537e-17,
                                         4.88438267410892e-14,
                                         2.06942314448583e-11,
                                         3.07298364359529e-09,
                                         1.99603760136402e-07,
                                         6.56739390242677e-06,
                                         0.000121003763863208,
                                         0.001338077416497,
                                         0.0093058765509676,
                                         0.0418827803077752,
                                         0.123382571738377,
                                         0.235844024570378,
                                         0.281350363717172,
                                         0.189864070628434,
                                         0.0560471421963297,
                                         0.00277819626152188,
                                         1.42091945945496e-53,
                                         9.49021139915576e-38,
                                         2.96969036067873e-29,
                                         1.76145840892051e-23,
                                         3.74984497366892e-19,
                                         9.8777215459299e-16,
                                         6.09056164192613e-13,
                                         1.27365536675551e-10,
                                         1.14127725863991e-08,
                                         5.12153416575624e-07,
                                         1.28234977591549e-05,
                                         0.000193407550828887,
                                         0.00185558236078274,
                                         0.0117609415712649,
                                         0.0504221489269558,
                                         0.147602303792055,
                                         0.292384210498812,
                                         0.376068579170415,
                                         0.277750617431638,
                                         0.0732588802374396,
                                         2.12986259840874e-57,
                                         7.56926666308554e-41,
                                         5.92606467097962e-32,
                                         6.69954698199214e-26,
                                         2.36823357282616e-21,
                                         9.55363677270154e-18,
                                         8.57294886305569e-15,
                                         2.52469232787139e-12,
                                         3.12088997290648e-10,
                                         1.91016961388514e-08,
                                         6.49948536568347e-07,
                                         1.33699708064893e-05,
                                         0.000176957116555841,
                                         0.00157947781453995,
                                         0.00985493428127353,
                                         0.0441800541634687,
                                         0.145319936697437,
                                         0.356251055656854,
                                         0.658296342395684,
                                         0.923894706787976,

                                         0.93056579245979,
                                         0.683852183163684,
                                         0.391521062933069,
                                         0.173541482621402,
                                         0.0590048177757875,
                                         0.0151935963748574,
                                         0.00291317175524896,
                                         0.000406854756500731,
                                         4.02290952314127e-05,
                                         2.71479227821219e-06,
                                         1.19209289550781e-07,
                                         3.19863411390727e-09,
                                         4.82004276804594e-11,
                                         3.63155425788207e-13,
                                         1.15930947111994e-15,
                                         1.22564017414685e-18,
                                         2.9078574719561e-22,
                                         7.94122140483062e-27,
                                         6.83863249248618e-33,
                                         8.5749551272767e-42,
                                         2.38825558799512e-58,
                                         0.067070710602707,
                                         0.262031781893869,
                                         0.374722411763008,
                                         0.315800673991722,
                                         0.177700478926994,
                                         0.0697717843519133,
                                         0.0193622687423992,
                                         0.00378196382268205,
                                         0.000511460432475265,
                                         4.65721756171427e-05,
                                         2.74181365966797e-06,
                                         9.86348334464432e-08,
                                         2.00555597424602e-09,
                                         2.06666572414909e-11,
                                         9.22709310462316e-14,
                                         1.41188529518029e-16,
                                         5.10772683791847e-20,
                                         2.3085177993788e-24,
                                         3.77981374474269e-30,
                                         1.18384759340417e-38,
                                         1.75287380440175e-54,
                                         0.00231197737928002,
                                         0.048018716969759,
                                         0.171525642677586,
                                         0.274844879852074,
                                         0.255949668999997,
                                         0.153237099787996,
                                         0.0615476174789987,
                                         0.0168135798920997,
                                         0.00310991605898248,
                                         0.000382103747267908,
                                         3.01599502563477e-05,
                                         1.45465805291819e-06,
                                         3.9910165832417e-08,
                                         5.62487383025284e-10,
                                         3.51232965794679e-12,
                                         7.77858660483043e-15,
                                         4.29088682058395e-18,
                                         3.20954891938787e-22,
                                         9.99163092704357e-28,
                                         7.8167205185604e-36,
                                         6.15297780078621e-51,
                                         5.0715371176433e-05,
                                         0.00559979963960323,
                                         0.0499636085046534,
                                         0.152218558158724,
                                         0.234598861981609,
                                         0.214167404080765,
                                         0.124500645485366,
                                         0.0475672854896656,
                                         0.0120334635626871,
                                         0.00199499321056772,
                                         0.000211119651794434,
                                         1.3652018569983e-05,
                                         5.05402786539659e-07,
                                         9.74228182912756e-09,
                                         8.50806881267953e-11,
                                         2.72713930430323e-13,
                                         2.29388589340299e-16,
                                         2.83961934682271e-20,
                                         1.68076781091461e-25,
                                         3.28442030145807e-33,
                                         1.37443878033725e-47,
                                         7.94634836523834e-07,
                                         0.000466451387270642,
                                         0.0103956225513117,
                                         0.0602170563066899,
                                         0.153592211697869,
                                         0.213803487882279,
                                         0.179888717075288,
                                         0.0961232639405496,
                                         0.0332586480903041,
                                         0.0074400098676805,
                                         0.00105559825897217,
                                         9.15176351982161e-05,
                                         4.57155233714875e-06,
                                         1.20525948165239e-07,
                                         1.47210463812633e-09,
                                         6.82945296115604e-12,
                                         8.75928235812155e-15,
                                         1.79451964777025e-18,
                                         2.01953332946773e-23,
                                         9.85745574047303e-31,
                                         2.19299386649776e-44,
                                         9.46257173446701e-09,
                                         2.95293496418048e-05,
                                         0.00164384475646208,
                                         0.0181044372954257,
                                         0.0764233515209049,
                                         0.162214544441953,
                                         0.197537629685989,
                                         0.147625792587426,
                                         0.0698605706449027,
                                         0.0210872134144896,
                                         0.00401127338409424,
                                         0.000466258004785002,
                                         3.14270308933767e-05,
                                         1.13321944194335e-06,
                                         1.93579765997688e-08,
                                         1.29980470865698e-10,
                                         2.54201924402451e-13,
                                         8.61886148605059e-17,
                                         1.84419958556015e-21,
                                         2.2484567386369e-28,
                                         2.65927362291154e-41,
                                         8.8958699923396e-11,
                                         1.47583917198105e-06,
                                         0.000205214836949269,
                                         0.00429722604079378,
                                         0.0300206867662803,
                                         0.0971633443542476,
                                         0.171251115878669,
                                         0.178992005272998,
                                         0.115850342955515,
                                         0.047184845057505,
                                         0.0120338201522827,
                                         0.00187536354883006,
                                         0.000170561366525812,
                                         8.41172633568146e-06,
                                         2.00964292353573e-07,
                                         1.95302567106597e-09,
                                         5.82407286662999e-12,
                                         3.26805414157373e-15,
                                         1.329543261732e-19,
                                         4.04894515643607e-26,
                                         2.5458117863729e-38,
                                         6.77013449560826e-13,
                                         5.97109266600768e-08,
                                         2.0738929714147e-05,
                                         0.000825697544945468,
                                         0.00954651354948883,
                                         0.0471134323449929,
                                         0.120183987347781,
                                         0.17568498722409,
                                         0.15552211202239,
                                         0.0854703534334263,
                                         0.0292249917984009,
                                         0.00610624557864669,
                                         0.000749354997473144,
                                         5.05458937257451e-05,
                                         1.68891371491626e-06,
                                         2.375567769434e-08,
                                         1.08020054605232e-10,
                                         1.00313229207067e-13,
                                         7.75937342687114e-18,
                                         5.90240327886952e-24,
                                         1.972964115867e-35,
                                         4.24311966440816e-15,
                                         1.98951722335421e-09,
                                         1.7260089243264e-06,
                                         0.0001306570661494,
                                         0.00250004644981545,
                                         0.018813350621118,
                                         0.0694606625136774,
                                         0.142008645004577,
                                         0.171935708021822,
                                         0.12749923550023,
                                         0.0584499835968018,
                                         0.0163735246163822,
                                         0.00271127558556397,
                                         0.00025012997464067,
                                         1.16889403123686e-05,
                                         2.37961123757599e-07,
                                         1.64991320858298e-09,
                                         2.5357491798298e-12,
                                         3.72932255117981e-16,
                                         7.08589933609287e-22,
                                         1.25918979202693e-32,
                                         2.21611359151067e-17,
                                         5.52408495873827e-11,
                                         1.19706708105504e-07,
                                         1.72291386025378e-05,
                                         0.000545594666474645,
                                         0.0062604609056667,
                                         0.0334541490878019,
                                         0.0956563201304986,
                                         0.158401310851912,
                                         0.158496039855942,
                                         0.0974166393280029,
                                         0.036587171952378,
                                         0.00817482515592654,
                                         0.00103148840921367,
                                         6.7415781526417e-05,
                                         1.98638465677446e-06,
                                         2.1000834874329e-08,
                                         5.34162173310711e-11,
                                         1.49366084149856e-14,
                                         7.08891602129142e-20,
                                         6.69702555490581e-30,
                                         9.72250199070734e-20,
                                         1.28840464357103e-12,
                                         6.9738600080999e-09,
                                         1.90841804133782e-06,
                                         0.000100016451170475,
                                         0.00174995045243056,
                                         0.0135344415468761,
                                         0.0541242438283622,
                                         0.122583141555403,
                                         0.165504015951342,
                                         0.136383295059204,
                                         0.0686743870719308,
                                         0.0207043961211702,
                                         0.00357307597976338,
                                         0.000326608356359479,
                                         1.39283598580506e-05,
                                         2.24538753326301e-07,
                                         9.45190292948238e-10,
                                         5.02519975196332e-13,
                                         5.95722455225991e-18,
                                         2.99193396550073e-27,
                                         3.60069798492539e-22,
                                         2.53668677663582e-14,
                                         3.42965625296822e-10,
                                         1.78445702684647e-07,
                                         1.54773069812514e-05,
                                         0.000412921815694697,
                                         0.00462224887081477,
                                         0.0258519106069792,
                                         0.0800802379014854,
                                         0.145888577180928,
                                         0.161180257797241,
                                         0.108813648894484,
                                         0.0442658996302908,
                                         0.0104482313887012,
                                         0.00133572065611709,
                                         8.24440346533001e-05,
                                         2.02660301955227e-06,
                                         1.41184820776499e-08,
                                         1.42717521626895e-11,
                                         4.2260121510946e-16,
                                         1.12835220890658e-24,
                                         1.12835220890682e-24,
                                         4.22601215109443e-16,
                                         1.42717521626897e-11,
                                         1.41184820776499e-08,
                                         2.02660301955227e-06,
                                         8.24440346533001e-05,
                                         0.00133572065611709,
                                         0.0104482313887011,
                                         0.0442658996302908,
                                         0.108813648894484,
                                         0.161180257797241,
                                         0.145888577180928,
                                         0.0800802379014854,
                                         0.0258519106069793,
                                         0.00462224887081477,
                                         0.000412921815694697,
                                         1.54773069812514e-05,
                                         1.78445702684647e-07,
                                         3.42965625296816e-10,
                                         2.53668677663592e-14,
                                         3.60069798492468e-22,
                                         2.99193396550142e-27,
                                         5.95722455225965e-18,
                                         5.02519975196341e-13,
                                         9.45190292948238e-10,
                                         2.24538753326301e-07,
                                         1.39283598580506e-05,
                                         0.000326608356359479,
                                         0.00357307597976337,
                                         0.0207043961211702,
                                         0.0686743870719308,
                                         0.136383295059204,
                                         0.165504015951342,
                                         0.122583141555403,
                                         0.0541242438283623,
                                         0.0135344415468761,
                                         0.00174995045243056,
                                         0.000100016451170475,
                                         1.90841804133782e-06,
                                         6.97386000809981e-09,
                                         1.28840464357107e-12,
                                         9.72250199070561e-20,
                                         6.69702555490747e-30,
                                         7.08891602129109e-20,
                                         1.49366084149859e-14,
                                         5.34162173310711e-11,
                                         2.1000834874329e-08,
                                         1.98638465677446e-06,
                                         6.7415781526417e-05,
                                         0.00103148840921367,
                                         0.00817482515592654,
                                         0.036587171952378,
                                         0.0974166393280029,
                                         0.158496039855942,
                                         0.158401310851912,
                                         0.0956563201304988,
                                         0.0334541490878019,
                                         0.0062604609056667,
                                         0.000545594666474645,
                                         1.72291386025378e-05,
                                         1.19706708105503e-07,
                                         5.52408495873844e-11,
                                         2.21611359151031e-17,
                                         1.25918979202726e-32,
                                         7.08589933609251e-22,
                                         3.72932255117989e-16,
                                         2.5357491798298e-12,
                                         1.64991320858298e-09,
                                         2.37961123757599e-07,
                                         1.16889403123686e-05,
                                         0.00025012997464067,
                                         0.00271127558556397,
                                         0.0163735246163822,
                                         0.0584499835968018,
                                         0.12749923550023,
                                         0.171935708021822,
                                         0.142008645004578,
                                         0.0694606625136774,
                                         0.018813350621118,
                                         0.00250004644981545,
                                         0.0001306570661494,
                                         1.72600892432638e-06,
                                         1.98951722335427e-09,
                                         4.24311966440756e-15,
                                         1.97296411586756e-35,
                                         5.9024032788692e-24,
                                         7.75937342687131e-18,
                                         1.00313229207067e-13,
                                         1.08020054605232e-10,
                                         2.375567769434e-08,
                                         1.68891371491626e-06,
                                         5.0545893725745e-05,
                                         0.000749354997473144,
                                         0.00610624557864669,
                                         0.0292249917984009,
                                         0.0854703534334263,
                                         0.15552211202239,
                                         0.17568498722409,
                                         0.120183987347781,
                                         0.0471134323449929,
                                         0.00954651354948883,
                                         0.000825697544945468,
                                         2.07389297141468e-05,
                                         5.97109266600782e-08,
                                         6.77013449560742e-13,
                                         2.54581178637367e-38,
                                         4.04894515643584e-26,
                                         1.32954326173204e-19,
                                         3.26805414157373e-15,
                                         5.82407286662999e-12,
                                         1.95302567106597e-09,
                                         2.00964292353573e-07,
                                         8.41172633568144e-06,
                                         0.000170561366525812,
                                         0.00187536354883006,
                                         0.0120338201522827,
                                         0.047184845057505,
                                         0.115850342955515,
                                         0.178992005272998,
                                         0.171251115878669,
                                         0.0971633443542476,
                                         0.0300206867662803,
                                         0.00429722604079378,
                                         0.000205214836949268,
                                         1.47583917198108e-06,
                                         8.89586999233865e-11,
                                         2.65927362291239e-41,
                                         2.24845673863676e-28,
                                         1.8441995855602e-21,
                                         8.61886148605059e-17,
                                         2.54201924402451e-13,
                                         1.29980470865698e-10,
                                         1.93579765997688e-08,
                                         1.13321944194334e-06,
                                         3.14270308933767e-05,
                                         0.000466258004785002,
                                         0.00401127338409424,
                                         0.0210872134144896,
                                         0.0698605706449027,
                                         0.147625792587427,
                                         0.197537629685989,
                                         0.162214544441953,
                                         0.0764233515209049,
                                         0.0181044372954257,
                                         0.00164384475646207,
                                         2.95293496418053e-05,
                                         9.46257173446617e-09,
                                         2.1929938664985e-44,
                                         9.85745574047239e-31,
                                         2.01953332946778e-23,
                                         1.79451964777025e-18,
                                         8.75928235812155e-15,
                                         6.82945296115604e-12,
                                         1.47210463812633e-09,
                                         1.20525948165239e-07,
                                         4.57155233714875e-06,
                                         9.15176351982161e-05,
                                         0.00105559825897217,
                                         0.0074400098676805,
                                         0.0332586480903041,
                                         0.0961232639405497,
                                         0.179888717075288,
                                         0.213803487882279,
                                         0.153592211697869,
                                         0.0602170563066899,
                                         0.0103956225513116,
                                         0.000466451387270648,
                                         7.94634836523777e-07,
                                         1.37443878033774e-47,
                                         3.28442030145785e-33,
                                         1.68076781091466e-25,
                                         2.83961934682271e-20,
                                         2.29388589340299e-16,
                                         2.72713930430323e-13,
                                         8.50806881267953e-11,
                                         9.74228182912752e-09,
                                         5.05402786539659e-07,
                                         1.3652018569983e-05,
                                         0.000211119651794434,
                                         0.00199499321056772,
                                         0.0120334635626871,
                                         0.0475672854896656,
                                         0.124500645485366,
                                         0.214167404080765,
                                         0.234598861981609,
                                         0.152218558158724,
                                         0.0499636085046532,
                                         0.00559979963960329,
                                         5.07153711764303e-05,
                                         6.15297780078851e-51,
                                         7.81672051855984e-36,
                                         9.99163092704386e-28,
                                         3.20954891938787e-22,
                                         4.29088682058395e-18,
                                         7.77858660483043e-15,
                                         3.51232965794678e-12,
                                         5.62487383025281e-10,
                                         3.9910165832417e-08,
                                         1.45465805291819e-06,
                                         3.01599502563477e-05,
                                         0.000382103747267908,
                                         0.00310991605898248,
                                         0.0168135798920998,
                                         0.0615476174789987,
                                         0.153237099787996,
                                         0.255949668999997,
                                         0.274844879852074,
                                         0.171525642677585,
                                         0.0480187169697593,
                                         0.00231197737927993,
                                         1.75287380440243e-54,
                                         1.18384759340408e-38,
                                         3.7798137447428e-30,
                                         2.3085177993788e-24,
                                         5.10772683791847e-20,
                                         1.41188529518029e-16,
                                         9.22709310462316e-14,
                                         2.06666572414908e-11,
                                         2.00555597424602e-09,
                                         9.86348334464432e-08,
                                         2.74181365966797e-06,
                                         4.65721756171427e-05,
                                         0.000511460432475265,
                                         0.00378196382268205,
                                         0.0193622687423992,
                                         0.0697717843519133,
                                         0.177700478926994,
                                         0.315800673991722,
                                         0.374722411763008,
                                         0.26203178189387,
                                         0.0670707106027058,
                                         2.38825558799609e-58,
                                         8.57495512727603e-42,
                                         6.8386324924864e-33,
                                         7.94122140483062e-27,
                                         2.9078574719561e-22,
                                         1.22564017414685e-18,
                                         1.15930947111994e-15,
                                         3.63155425788205e-13,
                                         4.82004276804594e-11,
                                         3.19863411390727e-09,
                                         1.19209289550781e-07,
                                         2.71479227821219e-06,
                                         4.02290952314127e-05,
                                         0.000406854756500731,
                                         0.00291317175524896,
                                         0.0151935963748574,
                                         0.0590048177757875,
                                         0.173541482621402,
                                         0.391521062933069,
                                         0.683852183163684,
                                         0.93056579245979,

                                         0.936406097625671,
                                         0.706870809333322,
                                         0.424977032020072,
                                         0.202429814332209,
                                         0.0758122620484952,
                                         0.0220897703668823,
                                         0.00493866286639965,
                                         0.000832205735893957,
                                         0.000103309498798804,
                                         9.17688756062158e-06,
                                         5.61956537090573e-07,
                                         2.26056963264385e-08,
                                         5.60724370413573e-10,
                                         7.87915922012856e-12,
                                         5.58197752945421e-14,
                                         1.68912012872636e-16,
                                         1.70602140267358e-19,
                                         3.89639251014318e-23,
                                         1.03200484276799e-27,
                                         8.68258616562261e-34,
                                         1.07135972333346e-42,
                                         2.95743879453654e-59,
                                         0.0616154978311989,
                                         0.247077327541605,
                                         0.370511147993229,
                                         0.334846764326507,
                                         0.206945152675948,
                                         0.091603074263574,
                                         0.0295026603713459,
                                         0.00691249688494516,
                                         0.00116521105379353,
                                         0.000138422983009923,
                                         1.12397670354657e-05,
                                         5.97886818989792e-07,
                                         1.96649257312385e-08,
                                         3.69548298826611e-10,
                                         3.55499923911015e-12,
                                         1.49576859293773e-14,
                                         2.17631271472573e-17,
                                         7.55096693792753e-21,
                                         3.30039391273931e-25,
                                         5.2682812684678e-31,
                                         1.6214297599578e-39,
                                         2.37763786667609e-55,
                                         0.00193901219124721,
                                         0.041303855439788,
                                         0.154490560653717,
                                         0.264900382654879,
                                         0.270169245912408,
                                         0.181674404874321,
                                         0.0842903443957525,
                                         0.027460221849,
                                         0.00628541252145435,
                                         0.000998586859067287,
                                         0.000107516901753543,
                                         7.562840096578e-06,
                                         3.29837506207948e-07,
                                         8.28948189307568e-09,
                                         1.08281880513444e-10,
                                         6.33480326371613e-13,
                                         1.32777036915293e-15,
                                         6.99853738221468e-19,
                                         5.04794563165798e-23,
                                         1.52881049061554e-28,
                                         1.17361543715861e-36,
                                         9.14198493384532e-52,
                                         3.88308120093993e-05,
                                         0.00439393515829772,
                                         0.0409928368178524,
                                         0.133359641550691,
                                         0.224451192832031,
                                         0.229288777163141,
                                         0.153249770090027,
                                         0.0694190217545124,
                                         0.0215758707445121,
                                         0.00458425581039708,
                                         0.000654487757485915,
                                         6.08774162552496e-05,
                                         3.52057111001717e-06,
                                         1.18328380870451e-07,
                                         2.09883076587301e-09,
                                         1.70728965366476e-11,
                                         5.15501579968096e-14,
                                         4.12778776606584e-17,
                                         4.91325058923529e-21,
                                         2.82321341970539e-26,
                                         5.40578590712987e-34,
                                         2.23686950326212e-48,
                                         5.55449230473319e-07,
                                         0.000333878658364502,
                                         0.0077693728307714,
                                         0.0479554772797621,
                                         0.133192536387397,
                                         0.206701588597224,
                                         0.199018660703626,
                                         0.125350162548693,
                                         0.0529023350035741,
                                         0.0150322436324728,
                                         0.00284576004671104,
                                         0.000350025327086903,
                                         2.68409675920703e-05,
                                         1.20648637290573e-06,
                                         2.90583545391456e-08,
                                         3.28664842251698e-10,
                                         1.4295817426792e-12,
                                         1.73899925810447e-15,
                                         3.41582116682247e-19,
                                         3.72396619189092e-24,
                                         1.77854080820262e-31,
                                         3.90942386834857e-45,
                                         6.03845534171752e-09,
                                         1.92813424450703e-05,
                                         0.00111912228429877,
                                         0.0131058625273395,
                                         0.0600691453645509,
                                         0.141617988963518,
                                         0.196427079378962,
                                         0.172022361723521,
                                         0.0985813913281852,
                                         0.0374621208018712,
                                         0.00940391338818836,
                                         0.00152952408668812,
                                         0.00015552378130179,
                                         9.3490936962677e-06,
                                         3.0575826622701e-07,
                                         4.80853612088331e-09,
                                         3.01301697065105e-11,
                                         5.56794614746708e-14,
                                         1.80482429384934e-17,
                                         3.73320072693732e-22,
                                         4.44715656557018e-29,
                                         5.1927625503834e-42,
                                         5.18256796483055e-11,
                                         8.7907015216683e-07,
                                         0.000127264355603052,
                                         0.00282768258231686,
                                         0.0213875321278827,
                                         0.0766003386337802,
                                         0.153054667191843,
                                         0.186372816490908,
                                         0.145028282313619,
                                         0.0737052752081143,
                                         0.0245333368241774,
                                         0.00527655937123456,
                                         0.000711431522318639,
                                         5.71944992093572e-05,
                                         2.53993763988015e-06,
                                         5.5540549744655e-08,
                                         5.01339381178066e-10,
                                         1.4074348927352e-12,
                                         7.52856820146702e-16,
                                         2.95457223268114e-20,
                                         8.77887090041883e-27,
                                         5.44530018247291e-39,
                                         3.60075672330579e-13,
                                         3.24443761131007e-08,
                                         1.17156300323568e-05,
                                         0.000493884426527567,
                                         0.0061645236554892,
                                         0.0335407355781963,
                                         0.0965431404913343,
                                         0.163459383845597,
                                         0.172718990097228,
                                         0.117390894249325,
                                         0.051812464333235,
                                         0.0147358422727127,
                                         0.0026345050053578,
                                         0.000283249164665307,
                                         1.70803799633839e-05,
                                         5.1932244383568e-07,
                                         6.75292132620725e-09,
                                         2.87999164554935e-11,
                                         2.54225714479867e-14,
                                         1.8929428784593e-18,
                                         1.40289294813147e-24,
                                         4.62247759373195e-36,
                                         2.06025828210619e-15,
                                         9.86130598459903e-10,
                                         8.88185397592406e-07,
                                         7.10393732909618e-05,
                                         0.001463246474095,
                                         0.0120946579462104,
                                         0.0501505131379189,
                                         0.118063665072163,
                                         0.1693973417583,
                                         0.153974699107104,
                                         0.0901137350452786,
                                         0.0338905185448801,
                                         0.00803422640931808,
                                         0.0011552131611765,
                                         9.45912828627478e-05,
                                         3.99892376117064e-06,
                                         7.4908426377994e-08,
                                         4.85325662925811e-10,
                                         7.06977640076814e-13,
                                         9.98756273027168e-17,
                                         1.84624578232048e-22,
                                         3.23152051731013e-33,
                                         9.82354663005899e-18,
                                         2.49774556158548e-11,
                                         5.61125961868627e-08,
                                         8.51513749389798e-06,
                                         0.000289437103282222,
                                         0.00363440525662676,
                                         0.0217094136638181,
                                         0.071062653212569,
                                         0.138449644798877,
                                         0.168299595102961,
                                         0.130607009791815,
                                         0.0649531499446834,
                                         0.0204177488955582,
                                         0.00392621766226062,
                                         0.000436539410169328,
                                         2.5660665221052e-05,
                                         6.92449987120228e-07,
                                         6.81544219808114e-09,
                                         1.63836489616843e-11,
                                         4.3913725741644e-15,
                                         2.02475860879486e-20,
                                         1.88259878713034e-30,
                                         3.93454248693535e-20,
                                         5.31424096984437e-13,
                                         2.97780587911767e-09,
                                         8.57360546300571e-07,
                                         4.80917084860242e-05,
                                         0.000917386615449457,
                                         0.00789405400255323,
                                         0.0359290605620241,
                                         0.0950509335897332,
                                         0.154524043598743,
                                         0.159008839645567,
                                         0.104568651347298,
                                         0.0435863936273933,
                                         0.0112089750580995,
                                         0.00169229115755094,
                                         0.000138315860538208,
                                         5.37681922570833e-06,
                                         8.0395938148624e-08,
                                         3.18929635097438e-10,
                                         1.62188604256836e-13,
                                         1.86524669090288e-18,
                                         9.21272096658077e-28,
                                         1.33027919587296e-22,
                                         9.54458218889857e-15,
                                         1.33399755819992e-10,
                                         7.28715413132338e-08,
                                         6.74541758105111e-06,
                                         0.00019547628584559,
                                         0.0024231188915188,
                                         0.0153346171178938,
                                         0.0550862944656418,
                                         0.119765488127582,
                                         0.163417544925948,
                                         0.142110257972817,
                                         0.0785446567886842,
                                         0.0270134507055123,
                                         0.00553795400674893,
                                         0.000629359347465814,
                                         3.52439917184606e-05,
                                         8.00565383748868e-07,
                                         5.24085023497858e-09,
                                         5.05665144915288e-12,
                                         1.45051402948503e-16,
                                         3.80575381984514e-25,
                                         3.80575381984514e-25,
                                         1.45051402948503e-16,
                                         5.05665144915279e-12,
                                         5.24085023497863e-09,
                                         8.00565383748873e-07,
                                         3.52439917184606e-05,
                                         0.000629359347465814,
                                         0.00553795400674893,
                                         0.0270134507055123,
                                         0.0785446567886841,
                                         0.142110257972817,
                                         0.163417544925948,
                                         0.119765488127582,
                                         0.0550862944656418,
                                         0.0153346171178938,
                                         0.0024231188915188,
                                         0.00019547628584559,
                                         6.74541758105107e-06,
                                         7.28715413132331e-08,
                                         1.33399755819994e-10,
                                         9.54458218889857e-15,
                                         1.33027919587296e-22,
                                         9.21272096658077e-28,
                                         1.86524669090288e-18,
                                         1.62188604256833e-13,
                                         3.18929635097441e-10,
                                         8.03959381486245e-08,
                                         5.37681922570833e-06,
                                         0.000138315860538208,
                                         0.00169229115755094,
                                         0.0112089750580995,
                                         0.0435863936273933,
                                         0.104568651347298,
                                         0.159008839645567,
                                         0.154524043598743,
                                         0.0950509335897332,
                                         0.0359290605620241,
                                         0.00789405400255323,
                                         0.000917386615449457,
                                         4.80917084860239e-05,
                                         8.57360546300564e-07,
                                         2.97780587911772e-09,
                                         5.31424096984437e-13,
                                         3.93454248693535e-20,
                                         1.88259878713034e-30,
                                         2.02475860879486e-20,
                                         4.39137257416431e-15,
                                         1.63836489616844e-11,
                                         6.81544219808119e-09,
                                         6.92449987120228e-07,
                                         2.5660665221052e-05,
                                         0.000436539410169328,
                                         0.00392621766226062,
                                         0.0204177488955581,
                                         0.0649531499446834,
                                         0.130607009791815,
                                         0.168299595102962,
                                         0.138449644798877,
                                         0.071062653212569,
                                         0.021709413663818,
                                         0.00363440525662676,
                                         0.000289437103282221,
                                         8.51513749389792e-06,
                                         5.61125961868635e-08,
                                         2.49774556158548e-11,
                                         9.82354663005899e-18,
                                         3.23152051731013e-33,
                                         1.84624578232048e-22,
                                         9.98756273027145e-17,
                                         7.06977640076823e-13,
                                         4.85325662925815e-10,
                                         7.4908426377994e-08,
                                         3.99892376117064e-06,
                                         9.45912828627478e-05,
                                         0.0011552131611765,
                                         0.00803422640931806,
                                         0.0338905185448801,
                                         0.0901137350452786,
                                         0.153974699107104,
                                         0.1693973417583,
                                         0.118063665072163,
                                         0.0501505131379189,
                                         0.0120946579462104,
                                         0.00146324647409499,
                                         7.10393732909614e-05,
                                         8.88185397592417e-07,
                                         9.86130598459903e-10,
                                         2.06025828210619e-15,
                                         4.62247759373195e-36,
                                         1.40289294813147e-24,
                                         1.89294287845925e-18,
                                         2.54225714479871e-14,
                                         2.87999164554937e-11,
                                         6.75292132620725e-09,
                                         5.1932244383568e-07,
                                         1.70803799633839e-05,
                                         0.000283249164665307,
                                         0.00263450500535779,
                                         0.0147358422727127,
                                         0.051812464333235,
                                         0.117390894249325,
                                         0.172718990097228,
                                         0.163459383845597,
                                         0.0965431404913343,
                                         0.0335407355781963,
                                         0.00616452365548918,
                                         0.000493884426527564,
                                         1.17156300323569e-05,
                                         3.24443761131007e-08,
                                         3.60075672330579e-13,
                                         5.44530018247291e-39,
                                         8.77887090041883e-27,
                                         2.95457223268106e-20,
                                         7.52856820146713e-16,
                                         1.40743489273521e-12,
                                         5.01339381178066e-10,
                                         5.5540549744655e-08,
                                         2.53993763988015e-06,
                                         5.71944992093572e-05,
                                         0.000711431522318637,
                                         0.00527655937123456,
                                         0.0245333368241774,
                                         0.0737052752081144,
                                         0.145028282313619,
                                         0.186372816490908,
                                         0.153054667191843,
                                         0.0766003386337802,
                                         0.0213875321278826,
                                         0.00282768258231684,
                                         0.000127264355603053,
                                         8.7907015216683e-07,
                                         5.18256796483055e-11,
                                         5.1927625503834e-42,
                                         4.44715656557018e-29,
                                         3.73320072693722e-22,
                                         1.80482429384937e-17,
                                         5.56794614746713e-14,
                                         3.01301697065105e-11,
                                         4.80853612088331e-09,
                                         3.0575826622701e-07,
                                         9.3490936962677e-06,
                                         0.000155523781301789,
                                         0.00152952408668812,
                                         0.00940391338818836,
                                         0.0374621208018712,
                                         0.0985813913281852,
                                         0.172022361723521,
                                         0.196427079378962,
                                         0.141617988963518,
                                         0.0600691453645507,
                                         0.0131058625273394,
                                         0.00111912228429878,
                                         1.92813424450703e-05,
                                         6.03845534171752e-09,
                                         3.90942386834857e-45,
                                         1.77854080820262e-31,
                                         3.72396619189081e-24,
                                         3.41582116682252e-19,
                                         1.73899925810449e-15,
                                         1.4295817426792e-12,
                                         3.28664842251698e-10,
                                         2.90583545391456e-08,
                                         1.20648637290573e-06,
                                         2.68409675920702e-05,
                                         0.000350025327086903,
                                         0.00284576004671104,
                                         0.0150322436324729,
                                         0.0529023350035741,
                                         0.125350162548693,
                                         0.199018660703626,
                                         0.206701588597224,
                                         0.133192536387397,
                                         0.047955477279762,
                                         0.00776937283077145,
                                         0.000333878658364502,
                                         5.55449230473319e-07,
                                         2.23686950326212e-48,
                                         5.40578590712987e-34,
                                         2.82321341970531e-26,
                                         4.91325058923537e-21,
                                         4.12778776606588e-17,
                                         5.15501579968096e-14,
                                         1.70728965366476e-11,
                                         2.09883076587301e-09,
                                         1.18328380870451e-07,
                                         3.52057111001716e-06,
                                         6.08774162552496e-05,
                                         0.000654487757485915,
                                         0.00458425581039708,
                                         0.0215758707445121,
                                         0.0694190217545124,
                                         0.153249770090027,
                                         0.229288777163141,
                                         0.224451192832031,
                                         0.133359641550691,
                                         0.0409928368178526,
                                         0.00439393515829772,
                                         3.88308120093993e-05,
                                         9.14198493384532e-52,
                                         1.17361543715861e-36,
                                         1.52881049061549e-28,
                                         5.04794563165807e-23,
                                         6.99853738221475e-19,
                                         1.32777036915293e-15,
                                         6.33480326371613e-13,
                                         1.08281880513444e-10,
                                         8.28948189307568e-09,
                                         3.29837506207947e-07,
                                         7.562840096578e-06,
                                         0.000107516901753543,
                                         0.000998586859067288,
                                         0.00628541252145435,
                                         0.027460221849,
                                         0.0842903443957525,
                                         0.181674404874321,
                                         0.270169245912408,
                                         0.264900382654879,
                                         0.154490560653717,
                                         0.041303855439788,
                                         0.00193901219124721,
                                         2.37763786667609e-55,
                                         1.6214297599578e-39,
                                         5.26828126846763e-31,
                                         3.30039391273937e-25,
                                         7.55096693792761e-21,
                                         2.17631271472573e-17,
                                         1.49576859293773e-14,
                                         3.55499923911015e-12,
                                         3.69548298826612e-10,
                                         1.96649257312385e-08,
                                         5.97886818989792e-07,
                                         1.12397670354657e-05,
                                         0.000138422983009923,
                                         0.00116521105379353,
                                         0.00691249688494516,
                                         0.0295026603713459,
                                         0.091603074263574,
                                         0.206945152675948,
                                         0.334846764326506,
                                         0.37051114799323,
                                         0.247077327541605,
                                         0.0616154978311989,
                                         2.95743879453654e-59,
                                         1.07135972333346e-42,
                                         8.6825861656223e-34,
                                         1.03200484276801e-27,
                                         3.89639251014322e-23,
                                         1.70602140267358e-19,
                                         1.68912012872636e-16,
                                         5.58197752945421e-14,
                                         7.87915922012856e-12,
                                         5.60724370413572e-10,
                                         2.26056963264385e-08,
                                         5.61956537090573e-07,
                                         9.17688756062158e-06,
                                         0.000103309498798804,
                                         0.000832205735893957,
                                         0.00493866286639965,
                                         0.0220897703668823,
                                         0.0758122620484952,
                                         0.202429814332209,
                                         0.424977032020072,
                                         0.706870809333322,
                                         0.936406097625671,

                                         0.941546592554025,
                                         0.727639799715835,
                                         0.456555240898039,
                                         0.231563665589049,
                                         0.0943339988558477,
                                         0.0305975967722176,
                                         0.00781177406299562,
                                         0.00154696888393055,
                                         0.000233245665120168,
                                         2.61556166335732e-05,
                                         2.11768806292722e-06,
                                         1.19209289550781e-07,
                                         4.44416332451769e-09,
                                         1.02957559129715e-10,
                                         1.36133678255425e-12,
                                         9.1406540765629e-15,
                                         2.63986952627448e-17,
                                         2.5620394602451e-20,
                                         5.66006077742836e-24,
                                         1.45954777480729e-28,
                                         1.20320654669184e-34,
                                         1.46394038823848e-43,
                                         4.00976527616735e-60,
                                         0.056785032053146,
                                         0.232958931838719,
                                         0.364131280403802,
                                         0.349759806120229,
                                         0.234546422640136,
                                         0.115200461298484,
                                         0.0421984332574829,
                                         0.0115614837997937,
                                         0.0023525996373498,
                                         0.000350184447911554,
                                         3.72521988497086e-05,
                                         2.74181365966797e-06,
                                         1.33645936663759e-07,
                                         4.06801006297135e-09,
                                         7.13981249417387e-11,
                                         6.4699503651621e-13,
                                         2.58518413361794e-15,
                                         3.5997686078141e-18,
                                         1.2042487329875e-21,
                                         5.11180764747918e-26,
                                         7.98052022484071e-32,
                                         2.41889013842545e-40,
                                         3.51708256259863e-56,
                                         0.00163791282567518,
                                         0.0356703307341699,
                                         0.138895281770691,
                                         0.252658685599922,
                                         0.27890367134245,
                                         0.207436878421159,
                                         0.109020414487208,
                                         0.0413247676867338,
                                         0.0113487314570383,
                                         0.002242299371768,
                                         0.000313405551898036,
                                         3.01599502563477e-05,
                                         1.92214592985421e-06,
                                         7.68724368334998e-08,
                                         1.79090637761873e-09,
                                         2.1902287400185e-11,
                                         1.21078031088969e-13,
                                         2.41895731793985e-16,
                                         1.22539513168558e-19,
                                         8.56240055800085e-24,
                                         2.5315532557789e-29,
                                         1.91149754447059e-37,
                                         1.47540423356637e-52,
                                         3.00644350241689e-05,
                                         0.00347568380888448,
                                         0.0337149370307429,
                                         0.116145909809668,
                                         0.211049828581278,
                                         0.237696656852605,
                                         0.179235769935486,
                                         0.0939967031720751,
                                         0.0348378999931016,
                                         0.00913683307554528,
                                         0.00167790285712091,
                                         0.000211119651794434,
                                         1.75922872813992e-05,
                                         9.24409995700783e-07,
                                         2.85867191153175e-08,
                                         4.71827611987898e-10,
                                         3.60864842898778e-12,
                                         1.03439682845653e-14,
                                         7.93490000210638e-18,
                                         9.12687143536739e-22,
                                         5.11032226471504e-27,
                                         9.61250768062369e-35,
                                         3.93862689414542e-49,
                                         3.94173364659939e-07,
                                         0.000241905281157885,
                                         0.00584560111194103,
                                         0.038136916193268,
                                         0.114074281145664,
                                         0.194550406675635,
                                         0.210481294951389,
                                         0.152716789847203,
                                         0.0763886053523029,
                                         0.0265931479920844,
                                         0.00641651044898819,
                                         0.00105559825897217,
                                         0.000115008575640231,
                                         7.94018427848184e-06,
                                         3.25932531781815e-07,
                                         7.26021022914195e-09,
                                         7.6823795642445e-11,
                                         3.15949825272002e-13,
                                         3.67010703121532e-16,
                                         6.94896764334961e-20,
                                         7.36854046582648e-25,
                                         3.45280156228955e-32,
                                         7.5101847917628e-46,
                                         3.9276709299426e-09,
                                         1.27956994608929e-05,
                                         0.000770281723197739,
                                         0.00951701638161835,
                                         0.0468601926703543,
                                         0.121019346849525,
                                         0.187852042093347,
                                         0.188570843516609,
                                         0.127297180998489,
                                         0.0588243859610941,
                                         0.0186485294348872,
                                         0.00401127338409424,
                                         0.000571415127379869,
                                         5.18334508159728e-05,
                                         2.82426014969847e-06,
                                         8.4904094072485e-08,
                                         1.24296969536359e-09,
                                         7.33436722743571e-12,
                                         1.29011847268654e-14,
                                         4.0209830120003e-18,
                                         8.07473446797545e-23,
                                         9.42584147522832e-30,
                                         1.0883535458665e-42,
                                         3.08973030323952e-11,
                                         5.34343302653736e-07,
                                         8.01323109729089e-05,
                                         0.00187496739274167,
                                         0.0151970067802997,
                                         0.0594312855475993,
                                         0.132359766446069,
                                         0.183823046397171,
                                         0.167473713232079,
                                         0.102726552305773,
                                         0.0427885858121267,
                                         0.0120338201522827,
                                         0.00224135630056672,
                                         0.000267132865189182,
                                         1.93205437242738e-05,
                                         7.83873037464662e-07,
                                         1.58768001503237e-08,
                                         1.34414103769831e-10,
                                         3.58028926120107e-13,
                                         1.83688448178894e-16,
                                         6.98574429509862e-21,
                                         2.03145076724352e-27,
                                         1.24516535349579e-39,
                                         1.96759484897694e-13,
                                         1.80636843815047e-08,
                                         6.74831534663251e-06,
                                         0.000299031024354922,
                                         0.00398971314114968,
                                         0.0236268090281722,
                                         0.0754963039334031,
                                         0.145062447685275,
                                         0.17836274174436,
                                         0.145223750802758,
                                         0.0794768969325145,
                                         0.0292249917984009,
                                         0.00711704470361168,
                                         0.00111448479623901,
                                         0.000106995033006032,
                                         5.85858111286042e-06,
                                         1.64170470401147e-07,
                                         1.99414464640275e-09,
                                         8.04333699022656e-12,
                                         6.79299161888688e-15,
                                         4.89245394797457e-19,
                                         3.54423163980867e-25,
                                         1.15322403750277e-36,
                                         1.0318816652105e-15,
                                         5.02888166971396e-10,
                                         4.68017598941777e-07,
                                         3.92751535846557e-05,
                                         0.000862589961078851,
                                         0.00773524603289445,
                                         0.0354629208704623,
                                         0.0942733972161459,
                                         0.156437455660216,
                                         0.169072004057942,
                                         0.121571838907451,
                                         0.0584499835968018,
                                         0.0186109096796225,
                                         0.00382912980121794,
                                         0.000487963125184923,
                                         3.60593822346608e-05,
                                         1.39799694171476e-06,
                                         2.4363943711551e-08,
                                         1.48810483481473e-10,
                                         2.06880437816926e-13,
                                         2.82175902226419e-17,
                                         5.09233550687927e-23,
                                         8.79588329711822e-34,
                                         4.50965032914519e-18,
                                         1.16668939707371e-11,
                                         2.70487844827486e-08,
                                         4.29871140301104e-06,
                                         0.000155412476751734,
                                         0.00211038623956316,
                                         0.0138816813002974,
                                         0.0510554452265698,
                                         0.114339450857762,
                                         0.164030461030861,
                                         0.154968640634113,
                                         0.0974166393280029,
                                         0.04055592050814,
                                         0.0109633879589139,
                                         0.00185450985317245,
                                         0.000184953634975294,
                                         9.92055922251784e-06,
                                         2.48060304222175e-07,
                                         2.29429651118866e-09,
                                         5.25045001678953e-12,
                                         1.35622533265842e-15,
                                         6.09720327516536e-21,
                                         5.59067163451733e-31,
                                         1.65552265269574e-20,
                                         2.27362256752042e-13,
                                         1.31314477204429e-09,
                                         3.95219143744385e-07,
                                         2.35205054275333e-05,
                                         0.000483647611388985,
                                         0.00456445491894511,
                                         0.0232259915731873,
                                         0.0701989710361586,
                                         0.133676971113987,
                                         0.1659334515993,
                                         0.136383295059204,
                                         0.0742369654103111,
                                         0.026367488357151,
                                         0.00592039348698056,
                                         0.000796868658733218,
                                         5.91351050740382e-05,
                                         2.12151566338436e-06,
                                         2.97128867591008e-08,
                                         1.11931652709901e-10,
                                         5.47549098375372e-14,
                                         6.1323032804715e-19,
                                         2.98488633217757e-28,
                                         5.13038571918795e-23,
                                         3.74028006987572e-15,
                                         5.38145974364841e-11,
                                         3.06732848581279e-08,
                                         3.00490018005722e-06,
                                         9.35661542497429e-05,
                                         0.00126694687943189,
                                         0.00891926550359318,
                                         0.036382121688187,
                                         0.0919626164985738,
                                         0.149984611386281,
                                         0.161180257797241,
                                         0.114711983719824,
                                         0.0535322348578624,
                                         0.0159549199927016,
                                         0.00289823270292807,
                                         0.000297561831450751,
                                         1.53164403752221e-05,
                                         3.24834974052105e-07,
                                         2.01433609831568e-09,
                                         1.86610860537292e-12,
                                         5.20642009839734e-17,
                                         1.34528481862998e-25,
                                         1.34528481863033e-25,
                                         5.20642009839734e-17,
                                         1.86610860537288e-12,
                                         2.01433609831568e-09,
                                         3.24834974052105e-07,
                                         1.53164403752221e-05,
                                         0.000297561831450752,
                                         0.00289823270292806,
                                         0.0159549199927016,
                                         0.0535322348578624,
                                         0.114711983719824,
                                         0.161180257797241,
                                         0.149984611386281,
                                         0.0919626164985738,
                                         0.036382121688187,
                                         0.0089192655035932,
                                         0.00126694687943188,
                                         9.35661542497433e-05,
                                         3.00490018005723e-06,
                                         3.06732848581279e-08,
                                         5.38145974364851e-11,
                                         3.74028006987572e-15,
                                         5.13038571918675e-23,
                                         2.98488633217839e-28,
                                         6.1323032804715e-19,
                                         5.4754909837536e-14,
                                         1.11931652709901e-10,
                                         2.97128867591008e-08,
                                         2.12151566338435e-06,
                                         5.91351050740384e-05,
                                         0.000796868658733216,
                                         0.00592039348698056,
                                         0.026367488357151,
                                         0.0742369654103111,
                                         0.136383295059204,
                                         0.1659334515993,
                                         0.133676971113987,
                                         0.0701989710361586,
                                         0.0232259915731874,
                                         0.0045644549189451,
                                         0.000483647611388987,
                                         2.35205054275333e-05,
                                         3.95219143744385e-07,
                                         1.31314477204431e-09,
                                         2.27362256752042e-13,
                                         1.65552265269539e-20,
                                         5.59067163451899e-31,
                                         6.09720327516536e-21,
                                         1.35622533265839e-15,
                                         5.25045001678954e-12,
                                         2.29429651118866e-09,
                                         2.48060304222174e-07,
                                         9.92055922251789e-06,
                                         0.000184953634975294,
                                         0.00185450985317245,
                                         0.0109633879589139,
                                         0.04055592050814,
                                         0.0974166393280029,
                                         0.154968640634113,
                                         0.164030461030861,
                                         0.114339450857762,
                                         0.0510554452265699,
                                         0.0138816813002974,
                                         0.00211038623956317,
                                         0.000155412476751734,
                                         4.29871140301104e-06,
                                         2.7048784482749e-08,
                                         1.16668939707371e-11,
                                         4.50965032914432e-18,
                                         8.79588329712102e-34,
                                         5.09233550687927e-23,
                                         2.82175902226412e-17,
                                         2.06880437816926e-13,
                                         1.48810483481473e-10,
                                         2.43639437115509e-08,
                                         1.39799694171477e-06,
                                         3.60593822346606e-05,
                                         0.000487963125184923,
                                         0.00382912980121794,
                                         0.0186109096796225,
                                         0.0584499835968018,
                                         0.121571838907451,
                                         0.169072004057942,
                                         0.156437455660216,
                                         0.0942733972161461,
                                         0.0354629208704622,
                                         0.00773524603289447,
                                         0.000862589961078851,
                                         3.92751535846557e-05,
                                         4.68017598941783e-07,
                                         5.02888166971396e-10,
                                         1.03188166521032e-15,
                                         1.15322403750316e-36,
                                         3.54423163980867e-25,
                                         4.89245394797444e-19,
                                         6.79299161888688e-15,
                                         8.04333699022656e-12,
                                         1.99414464640274e-09,
                                         1.64170470401147e-07,
                                         5.8585811128604e-06,
                                         0.000106995033006032,
                                         0.00111448479623901,
                                         0.00711704470361168,
                                         0.0292249917984009,
                                         0.0794768969325145,
                                         0.145223750802758,
                                         0.17836274174436,
                                         0.145062447685275,
                                         0.0754963039334029,
                                         0.0236268090281722,
                                         0.00398971314114968,
                                         0.000299031024354922,
                                         6.74831534663259e-06,
                                         1.80636843815047e-08,
                                         1.96759484897665e-13,
                                         1.24516535349624e-39,
                                         2.03145076724352e-27,
                                         6.98574429509843e-21,
                                         1.83688448178894e-16,
                                         3.58028926120107e-13,
                                         1.3441410376983e-10,
                                         1.58768001503238e-08,
                                         7.83873037464659e-07,
                                         1.93205437242738e-05,
                                         0.000267132865189182,
                                         0.00224135630056672,
                                         0.0120338201522827,
                                         0.0427885858121267,
                                         0.102726552305773,
                                         0.167473713232079,
                                         0.183823046397171,
                                         0.132359766446069,
                                         0.0594312855475994,
                                         0.0151970067802997,
                                         0.00187496739274166,
                                         8.01323109729097e-05,
                                         5.34343302653736e-07,
                                         3.08973030323913e-11,
                                         1.08835354586691e-42,
                                         9.42584147522832e-30,
                                         8.07473446797521e-23,
                                         4.0209830120003e-18,
                                         1.29011847268654e-14,
                                         7.33436722743566e-12,
                                         1.24296969536359e-09,
                                         8.49040940724846e-08,
                                         2.82426014969847e-06,
                                         5.18334508159728e-05,
                                         0.000571415127379869,
                                         0.00401127338409424,
                                         0.0186485294348872,
                                         0.0588243859610941,
                                         0.127297180998489,
                                         0.188570843516609,
                                         0.187852042093346,
                                         0.121019346849525,
                                         0.0468601926703543,
                                         0.00951701638161835,
                                         0.000770281723197746,
                                         1.27956994608929e-05,
                                         3.92767092994218e-09,
                                         7.51018479176582e-46,
                                         3.45280156228955e-32,
                                         7.36854046582624e-25,
                                         6.94896764334961e-20,
                                         3.67010703121532e-16,
                                         3.15949825271999e-13,
                                         7.68237956424454e-11,
                                         7.26021022914192e-09,
                                         3.25932531781815e-07,
                                         7.94018427848184e-06,
                                         0.000115008575640231,
                                         0.00105559825897217,
                                         0.00641651044898819,
                                         0.0265931479920844,
                                         0.0763886053523029,
                                         0.152716789847203,
                                         0.210481294951389,
                                         0.194550406675636,
                                         0.114074281145664,
                                         0.038136916193268,
                                         0.00584560111194107,
                                         0.000241905281157885,
                                         3.94173364659905e-07,
                                         3.93862689414709e-49,
                                         9.61250768062369e-35,
                                         5.11032226471487e-27,
                                         9.12687143536739e-22,
                                         7.93490000210638e-18,
                                         1.03439682845653e-14,
                                         3.60864842898781e-12,
                                         4.71827611987896e-10,
                                         2.85867191153175e-08,
                                         9.24409995700783e-07,
                                         1.75922872813992e-05,
                                         0.000211119651794434,
                                         0.00167790285712091,
                                         0.00913683307554528,
                                         0.0348378999931016,
                                         0.0939967031720752,
                                         0.179235769935486,
                                         0.237696656852605,
                                         0.211049828581278,
                                         0.116145909809668,
                                         0.0337149370307431,
                                         0.00347568380888449,
                                         3.0064435024167e-05,
                                         1.47540423356703e-52,
                                         1.91149754447059e-37,
                                         2.53155325577882e-29,
                                         8.56240055800085e-24,
                                         1.22539513168558e-19,
                                         2.41895731793983e-16,
                                         1.2107803108897e-13,
                                         2.19022874001849e-11,
                                         1.79090637761873e-09,
                                         7.68724368334998e-08,
                                         1.92214592985421e-06,
                                         3.01599502563477e-05,
                                         0.000313405551898036,
                                         0.002242299371768,
                                         0.0113487314570383,
                                         0.0413247676867339,
                                         0.109020414487207,
                                         0.207436878421159,
                                         0.27890367134245,
                                         0.252658685599922,
                                         0.138895281770692,
                                         0.0356703307341699,
                                         0.00163791282567511,
                                         3.51708256260027e-56,
                                         2.41889013842545e-40,
                                         7.98052022484042e-32,
                                         5.11180764747918e-26,
                                         1.2042487329875e-21,
                                         3.59976860781407e-18,
                                         2.58518413361796e-15,
                                         6.46995036516207e-13,
                                         7.13981249417387e-11,
                                         4.06801006297135e-09,
                                         1.33645936663759e-07,
                                         2.74181365966797e-06,
                                         3.72521988497086e-05,
                                         0.000350184447911554,
                                         0.0023525996373498,
                                         0.0115614837997938,
                                         0.0421984332574829,
                                         0.115200461298484,
                                         0.234546422640136,
                                         0.349759806120229,
                                         0.364131280403803,
                                         0.232958931838719,
                                         0.0567850320531448,
                                         4.00976527616931e-60,
                                         1.46394038823848e-43,
                                         1.20320654669179e-34,
                                         1.45954777480729e-28,
                                         5.66006077742836e-24,
                                         2.56203946024507e-20,
                                         2.6398695262745e-17,
                                         9.14065407656285e-15,
                                         1.36133678255425e-12,
                                         1.02957559129715e-10,
                                         4.44416332451769e-09,
                                         1.19209289550781e-07,
                                         2.11768806292722e-06,
                                         2.61556166335732e-05,
                                         0.000233245665120168,
                                         0.00154696888393055,
                                         0.00781177406299562,
                                         0.0305975967722176,
                                         0.0943339988558477,
                                         0.231563665589049,
                                         0.456555240898039,
                                         0.727639799715835,
                                         0.941546592554025,

                                         0.946093696169798,
                                         0.746415152898566,
                                         0.486252512591213,
                                         0.260597014223016,
                                         0.114278076825458,
                                         0.0407057469242131,
                                         0.0116652349731348,
                                         0.00265698903476988,
                                         0.000473698619126954,
                                         6.48524825493137e-05,
                                         6.6569836642213e-06,
                                         4.97166246047392e-07,
                                         2.6004436955531e-08,
                                         9.07106596013666e-10,
                                         1.97956999713988e-11,
                                         2.48159857874839e-13,
                                         1.5897001928474e-15,
                                         4.40690234388436e-18,
                                         4.12975917443123e-21,
                                         8.8606893038833e-25,
                                         2.23175413846801e-29,
                                         1.80712363264478e-35,
                                         2.1717080738837e-44,
                                         5.90769862138455e-61,
                                         0.0524897302134775,
                                         0.219700444467108,
                                         0.356154676547227,
                                         0.360893347368855,
                                         0.259948077957086,
                                         0.139819959766117,
                                         0.0572883800498661,
                                         0.0179754418658061,
                                         0.00430247336139345,
                                         0.00077692322614406,
                                         0.000103976436398626,
                                         1.0058056535177e-05,
                                         6.79971468415423e-07,
                                         3.07225004652797e-08,
                                         8.7412800768659e-10,
                                         1.44534369544195e-11,
                                         1.24302850000144e-13,
                                         4.74696520269389e-16,
                                         6.36014407939209e-19,
                                         2.0606275074792e-22,
                                         8.52496451401299e-27,
                                         1.30516898381303e-32,
                                         3.90307260277029e-41,
                                         5.63292296715726e-57,
                                         0.00139276992926469,
                                         0.0309276037737143,
                                         0.124761415444643,
                                         0.239030415567899,
                                         0.282797227459356,
                                         0.229692844791628,
                                         0.134556425508412,
                                         0.0581613176233746,
                                         0.0186895614416087,
                                         0.00445137777811775,
                                         0.000776706993059578,
                                         9.73175929152501e-05,
                                         8.5035207229988e-06,
                                         4.97644997979537e-07,
                                         1.84605254065656e-08,
                                         4.02601679457367e-10,
                                         4.64848886121291e-12,
                                         2.44547664046027e-14,
                                         4.68461635804888e-17,
                                         2.29190349342573e-20,
                                         1.55741251444822e-24,
                                         4.50827576050979e-30,
                                         3.35487746864258e-38,
                                         2.56870447001066e-53,
                                         2.35174255227434e-05,
                                         0.00277055620501299,
                                         0.0278116842487339,
                                         0.100747159761344,
                                         0.195780322191639,
                                         0.240121537373075,
                                         0.201116497095902,
                                         0.119755165006811,
                                         0.0516636857406844,
                                         0.01622991169142,
                                         0.00369219692962992,
                                         0.000599203027885414,
                                         6.76724988830441e-05,
                                         5.12965410853582e-06,
                                         2.48095241203296e-07,
                                         7.13650191861184e-09,
                                         1.10623617932529e-10,
                                         8.01708234165372e-13,
                                         2.19576805232284e-15,
                                         1.62217802934393e-18,
                                         1.81058990741098e-22,
                                         9.90967830039188e-28,
                                         1.83506753002095e-35,
                                         7.45417914207508e-50,
                                         2.83643043978542e-07,
                                         0.0001772799503294,
                                         0.00442839397555381,
                                         0.0303308387878557,
                                         0.0968132712257099,
                                         0.17930265824194,
                                         0.214715261918117,
                                         0.176127120368855,
                                         0.102010206249213,
                                         0.0422678175870744,
                                         0.0125367354825132,
                                         0.00263529115542144,
                                         0.000384678237088451,
                                         3.77683910935533e-05,
                                         2.38157780887082e-06,
                                         9.03581136553667e-08,
                                         1.88042443313334e-09,
                                         1.87733222021462e-11,
                                         7.35141360435109e-14,
                                         8.20111058164546e-17,
                                         1.50351752501852e-20,
                                         1.55589614100766e-25,
                                         7.16967390061686e-33,
                                         1.54510313692202e-46,
                                         2.59996852133758e-09,
                                         8.6211632726548e-06,
                                         0.000535893889085364,
                                         0.00693984262834956,
                                         0.0363843267248724,
                                         0.101755040965082,
                                         0.174217479512494,
                                         0.196866494409841,
                                         0.153078943994455,
                                         0.083659850560628,
                                         0.0323517412697803,
                                         0.00880839540939303,
                                         0.00166186833476283,
                                         0.000211340377941018,
                                         1.73749957978607e-05,
                                         8.69485846892012e-07,
                                         2.4292796124874e-08,
                                         3.34102338892782e-10,
                                         1.87054795077518e-12,
                                         3.151087137315e-15,
                                         9.48878237681892e-19,
                                         1.85658673616596e-23,
                                         2.12892492947416e-30,
                                         2.43404562762745e-43,
                                         1.88148907267266e-11,
                                         3.30986116215763e-07,
                                         5.11975308378477e-05,
                                         0.00125358123329045,
                                         0.0107952188007902,
                                         0.0455892866608038,
                                         0.111598458458835,
                                         0.173722087505175,
                                         0.181353084961832,
                                         0.130726008992196,
                                         0.065909575858717,
                                         0.0232435606319405,
                                         0.00566804470212814,
                                         0.000933628569768898,
                                         0.000100074239765448,
                                         6.60534546514772e-06,
                                         2.47763189648352e-07,
                                         4.6941347906109e-09,
                                         3.75754760755519e-11,
                                         9.55841353959379e-14,
                                         4.7277024585985e-17,
                                         1.74899086570694e-21,
                                         4.99067110725322e-28,
                                         3.02717559879697e-40,
                                         1.1022114988826e-13,
                                         1.02868713954754e-08,
                                         3.95957756116725e-06,
                                         0.000183309498050428,
                                         0.00259285471127455,
                                         0.0165348129684731,
                                         0.0578701078262192,
                                         0.124098890135007,
                                         0.173925821925425,
                                         0.165362316407639,
                                         0.108699863746738,
                                         0.0496521592541878,
                                         0.015649466878019,
                                         0.00333883850156681,
                                         0.000466605138949739,
                                         4.06217082356581e-05,
                                         2.04562342384878e-06,
                                         5.33901600604527e-08,
                                         6.1104017368222e-10,
                                         2.34715000465879e-12,
                                         1.90686265127786e-15,
                                         1.33379616959366e-19,
                                         9.47081120730619e-26,
                                         3.04772754096856e-37,
                                         5.31749743353271e-16,
                                         2.63290966158364e-10,
                                         2.52189968227607e-07,
                                         2.20747893908711e-05,
                                         0.000512866142217847,
                                         0.00493872553243609,
                                         0.0247132294735845,
                                         0.0730062211820472,
                                         0.137366962595198,
                                         0.172262286045319,
                                         0.147634751314814,
                                         0.087347953892623,
                                         0.0355831951198647,
                                         0.00983322116129058,
                                         0.00179166103958792,
                                         0.00020573109747565,
                                         1.39089295363356e-05,
                                         5.00087505416483e-07,
                                         8.18303015359121e-09,
                                         4.74651644682958e-11,
                                         6.33385042579668e-14,
                                         8.37665260265613e-18,
                                         1.48011180667379e-23,
                                         2.52693330507926e-34,
                                         2.13780650570031e-18,
                                         5.61574475169517e-12,
                                         1.33852199913074e-08,
                                         2.21527131178832e-06,
                                         8.45373501163513e-05,
                                         0.00122927556600542,
                                         0.00879474933829508,
                                         0.0357907332711546,
                                         0.0904105852125917,
                                         0.149541805322846,
                                         0.167096344040887,
                                         0.128051917644519,
                                         0.0674231580755846,
                                         0.0241332023999459,
                                         0.00573298672478304,
                                         0.000868281318595187,
                                         7.88098461783197e-05,
                                         3.90345825258362e-06,
                                         9.13223949622658e-08,
                                         7.9988561785235e-10,
                                         1.75321407257974e-12,
                                         4.38399749684408e-16,
                                         1.92761643494445e-21,
                                         1.74594345089476e-31,
                                         7.21952767983528e-21,
                                         1.00613914879508e-13,
                                         5.9676384829612e-10,
                                         1.86739660083185e-07,
                                         1.17050296367459e-05,
                                         0.000257017617620744,
                                         0.00262903710925916,
                                         0.0147387485144571,
                                         0.0499845229289944,
                                         0.109047149693576,
                                         0.158863667749371,
                                         0.15768802840397,
                                         0.107313046870597,
                                         0.0497523259936793,
                                         0.0154093850849389,
                                         0.00307822426593008,
                                         0.000375099546250262,
                                         2.55936578664522e-05,
                                         8.56090582423995e-07,
                                         1.13229625853819e-08,
                                         4.07644304369601e-11,
                                         1.92730024387574e-14,
                                         2.10875441095173e-19,
                                         1.01331822378526e-28,
                                         2.05812528770525e-23,
                                         1.52170815113952e-15,
                                         2.24596058201107e-11,
                                         1.3288282000043e-08,
                                         1.36810375955541e-06,
                                         4.53627285628599e-05,
                                         0.000663426168465215,
                                         0.00512357682383619,
                                         0.0233278360232398,
                                         0.0671256723674502,
                                         0.12749843490863,
                                         0.163920772152641,
                                         0.144184505398166,
                                         0.0865833639617988,
                                         0.0349632915937186,
                                         0.00921218581276932,
                                         0.00150707626167029,
                                         0.000141656925794024,
                                         6.77461747530886e-06,
                                         1.35305324719103e-07,
                                         8.00111368033851e-10,
                                         7.15238834279642e-13,
                                         1.9473948585612e-17,
                                         4.96459887524004e-26,
                                         4.96459887524142e-26,
                                         1.9473948585612e-17,
                                         7.15238834279657e-13,
                                         8.00111368033851e-10,
                                         1.35305324719103e-07,
                                         6.7746174753089e-06,
                                         0.000141656925794024,
                                         0.00150707626167029,
                                         0.00921218581276932,
                                         0.0349632915937186,
                                         0.0865833639617988,
                                         0.144184505398165,
                                         0.163920772152641,
                                         0.12749843490863,
                                         0.0671256723674502,
                                         0.0233278360232398,
                                         0.00512357682383618,
                                         0.000663426168465215,
                                         4.53627285628597e-05,
                                         1.36810375955541e-06,
                                         1.3288282000043e-08,
                                         2.24596058201103e-11,
                                         1.52170815113952e-15,
                                         2.05812528770472e-23,
                                         1.01331822378556e-28,
                                         2.10875441095173e-19,
                                         1.92730024387579e-14,
                                         4.07644304369601e-11,
                                         1.13229625853819e-08,
                                         8.56090582424e-07,
                                         2.55936578664522e-05,
                                         0.000375099546250263,
                                         0.00307822426593008,
                                         0.0154093850849389,
                                         0.0497523259936793,
                                         0.107313046870597,
                                         0.15768802840397,
                                         0.158863667749371,
                                         0.109047149693576,
                                         0.0499845229289944,
                                         0.014738748514457,
                                         0.00262903710925916,
                                         0.000257017617620743,
                                         1.17050296367459e-05,
                                         1.86739660083185e-07,
                                         5.96763848296109e-10,
                                         1.00613914879508e-13,
                                         7.21952767983362e-21,
                                         1.74594345089532e-31,
                                         1.92761643494445e-21,
                                         4.38399749684419e-16,
                                         1.75321407257974e-12,
                                         7.9988561785235e-10,
                                         9.13223949622663e-08,
                                         3.90345825258362e-06,
                                         7.88098461783199e-05,
                                         0.000868281318595187,
                                         0.00573298672478304,
                                         0.0241332023999459,
                                         0.0674231580755845,
                                         0.12805191764452,
                                         0.167096344040887,
                                         0.149541805322846,
                                         0.0904105852125917,
                                         0.0357907332711545,
                                         0.00879474933829508,
                                         0.00122927556600541,
                                         8.45373501163513e-05,
                                         2.21527131178832e-06,
                                         1.33852199913071e-08,
                                         5.61574475169517e-12,
                                         2.13780650569987e-18,
                                         2.52693330508013e-34,
                                         1.48011180667379e-23,
                                         8.37665260265636e-18,
                                         6.33385042579668e-14,
                                         4.74651644682958e-11,
                                         8.18303015359127e-09,
                                         5.00087505416484e-07,
                                         1.39089295363356e-05,
                                         0.00020573109747565,
                                         0.00179166103958792,
                                         0.00983322116129058,
                                         0.0355831951198646,
                                         0.087347953892623,
                                         0.147634751314814,
                                         0.172262286045319,
                                         0.137366962595198,
                                         0.0730062211820471,
                                         0.0247132294735845,
                                         0.00493872553243608,
                                         0.000512866142217847,
                                         2.20747893908711e-05,
                                         2.52189968227604e-07,
                                         2.63290966158364e-10,
                                         5.31749743353173e-16,
                                         3.04772754096969e-37,
                                         9.47081120730619e-26,
                                         1.3337961695937e-19,
                                         1.90686265127786e-15,
                                         2.34715000465879e-12,
                                         6.11040173682224e-10,
                                         5.33901600604527e-08,
                                         2.04562342384879e-06,
                                         4.06217082356581e-05,
                                         0.000466605138949739,
                                         0.00333883850156681,
                                         0.015649466878019,
                                         0.0496521592541878,
                                         0.108699863746738,
                                         0.165362316407639,
                                         0.173925821925425,
                                         0.124098890135006,
                                         0.0578701078262192,
                                         0.0165348129684731,
                                         0.00259285471127455,
                                         0.000183309498050428,
                                         3.9595775611672e-06,
                                         1.02868713954754e-08,
                                         1.10221149888242e-13,
                                         3.02717559879815e-40,
                                         4.99067110725322e-28,
                                         1.748990865707e-21,
                                         4.7277024585985e-17,
                                         9.55841353959379e-14,
                                         3.75754760755522e-11,
                                         4.6941347906109e-09,
                                         2.47763189648353e-07,
                                         6.60534546514772e-06,
                                         0.000100074239765448,
                                         0.000933628569768898,
                                         0.00566804470212812,
                                         0.0232435606319405,
                                         0.0659095758587171,
                                         0.130726008992196,
                                         0.181353084961832,
                                         0.173722087505175,
                                         0.111598458458835,
                                         0.0455892866608036,
                                         0.0107952188007902,
                                         0.00125358123329045,
                                         5.11975308378471e-05,
                                         3.30986116215763e-07,
                                         1.8814890726724e-11,
                                         2.43404562762846e-43,
                                         2.12892492947416e-30,
                                         1.85658673616602e-23,
                                         9.48878237681892e-19,
                                         3.151087137315e-15,
                                         1.87054795077519e-12,
                                         3.34102338892782e-10,
                                         2.42927961248741e-08,
                                         8.69485846892012e-07,
                                         1.73749957978607e-05,
                                         0.000211340377941018,
                                         0.00166186833476282,
                                         0.00880839540939304,
                                         0.0323517412697803,
                                         0.083659850560628,
                                         0.153078943994455,
                                         0.196866494409841,
                                         0.174217479512494,
                                         0.101755040965082,
                                         0.0363843267248724,
                                         0.00693984262834956,
                                         0.000535893889085359,
                                         8.6211632726548e-06,
                                         2.59996852133728e-09,
                                         1.5451031369227e-46,
                                         7.16967390061686e-33,
                                         1.55589614100771e-25,
                                         1.50351752501852e-20,
                                         8.20111058164546e-17,
                                         7.35141360435115e-14,
                                         1.87733222021462e-11,
                                         1.88042443313335e-09,
                                         9.03581136553667e-08,
                                         2.38157780887082e-06,
                                         3.77683910935533e-05,
                                         0.00038467823708845,
                                         0.00263529115542144,
                                         0.0125367354825132,
                                         0.0422678175870744,
                                         0.102010206249213,
                                         0.176127120368854,
                                         0.214715261918117,
                                         0.17930265824194,
                                         0.0968132712257099,
                                         0.0303308387878557,
                                         0.00442839397555378,
                                         0.0001772799503294,
                                         2.83643043978516e-07,
                                         7.45417914207851e-50,
                                         1.83506753002095e-35,
                                         9.90967830039224e-28,
                                         1.81058990741098e-22,
                                         1.62217802934393e-18,
                                         2.19576805232286e-15,
                                         8.01708234165372e-13,
                                         1.10623617932529e-10,
                                         7.13650191861184e-09,
                                         2.48095241203296e-07,
                                         5.12965410853582e-06,
                                         6.76724988830439e-05,
                                         0.000599203027885414,
                                         0.00369219692962992,
                                         0.01622991169142,
                                         0.0516636857406844,
                                         0.119755165006811,
                                         0.201116497095902,
                                         0.240121537373075,
                                         0.195780322191639,
                                         0.100747159761344,
                                         0.0278116842487337,
                                         0.00277055620501299,
                                         2.35174255227418e-05,
                                         2.5687044700119e-53,
                                         3.35487746864258e-38,
                                         4.50827576050996e-30,
                                         1.55741251444822e-24,
                                         2.29190349342573e-20,
                                         4.68461635804892e-17,
                                         2.44547664046027e-14,
                                         4.64848886121294e-12,
                                         4.02601679457367e-10,
                                         1.84605254065656e-08,
                                         4.97644997979537e-07,
                                         8.50352072299878e-06,
                                         9.73175929152501e-05,
                                         0.000776706993059578,
                                         0.00445137777811775,
                                         0.0186895614416087,
                                         0.0581613176233745,
                                         0.134556425508412,
                                         0.229692844791628,
                                         0.282797227459356,
                                         0.239030415567899,
                                         0.124761415444643,
                                         0.0309276037737143,
                                         0.00139276992926463,
                                         5.63292296716012e-57,
                                         3.90307260277029e-41,
                                         1.30516898381308e-32,
                                         8.52496451401299e-27,
                                         2.0606275074792e-22,
                                         6.36014407939215e-19,
                                         4.74696520269389e-16,
                                         1.24302850000144e-13,
                                         1.44534369544195e-11,
                                         8.7412800768659e-10,
                                         3.07225004652797e-08,
                                         6.79971468415421e-07,
                                         1.0058056535177e-05,
                                         0.000103976436398626,
                                         0.00077692322614406,
                                         0.00430247336139346,
                                         0.0179754418658061,
                                         0.0572883800498661,
                                         0.139819959766116,
                                         0.259948077957086,
                                         0.360893347368855,
                                         0.356154676547226,
                                         0.219700444467108,
                                         0.0524897302134763,
                                         5.90769862138768e-61,
                                         2.1717080738837e-44,
                                         1.80712363264485e-35,
                                         2.23175413846801e-29,
                                         8.8606893038833e-25,
                                         4.12975917443127e-21,
                                         4.40690234388436e-18,
                                         1.58970019284741e-15,
                                         2.48159857874839e-13,
                                         1.97956999713988e-11,
                                         9.07106596013666e-10,
                                         2.60044369555309e-08,
                                         4.97166246047392e-07,
                                         6.6569836642213e-06,
                                         6.48524825493137e-05,
                                         0.000473698619126954,
                                         0.00265698903476988,
                                         0.0116652349731348,
                                         0.0407057469242131,
                                         0.114278076825458,
                                         0.260597014223016,
                                         0.486252512591213,
                                         0.746415152898566,
                                         0.946093696169798,

                                         5.96046447753906e-08,
                                         1.43051147460938e-06,
                                         1.64508819580078e-05,
                                         0.000120639801025391,
                                         0.000633358955383301,
                                         0.0025334358215332,
                                         0.00802254676818848,
                                         0.0206294059753418,
                                         0.0438374876976013,
                                         0.0779333114624023,
                                         0.116899967193604,
                                         0.148781776428223,
                                         0.161180257797241,
                                         0.148781776428223,
                                         0.116899967193604,
                                         0.0779333114624023,
                                         0.0438374876976013,
                                         0.0206294059753418,
                                         0.00802254676818848,
                                         0.0025334358215332,
                                         0.000633358955383301,
                                         0.000120639801025391,
                                         1.64508819580078e-05,
                                         1.43051147460938e-06,
                                         5.96046447753906e-08,

                                         0.00335394213709748,
                                         6.29237250918169e-17,
                                         0.0215684660903036,
                                         5.63602893705978e-15,
                                         0.0664614103150613,
                                         2.41890382979408e-13,
                                         0.130594062292864,
                                         6.62014612703728e-12,
                                         0.183711011020281,
                                         1.29710288923076e-10,
                                         0.196900868174357,
                                         1.93634155410143e-09,
                                         0.167071523951636,
                                         2.28839960255925e-08,
                                         0.115114319797057,
                                         2.1961089217984e-07,
                                         0.0655451766821188,
                                         1.74164789096022e-06,
                                         0.0312227149506293,
                                         1.15554105208155e-05,
                                         0.0125491518847924,
                                         6.46880687489993e-05,
                                         0.00427959014410533,
                                         0.00030726074809153,
                                         0.00124227211663665,
                                         0.00124227211663665,
                                         0.00030726074809153,
                                         0.00427959014410533,
                                         6.46880687489993e-05,
                                         0.0125491518847924,
                                         1.15554105208155e-05,
                                         0.0312227149506293,
                                         1.74164789096022e-06,
                                         0.0655451766821188,
                                         2.1961089217984e-07,
                                         0.115114319797057,
                                         2.28839960255925e-08,
                                         0.167071523951636,
                                         1.93634155410143e-09,
                                         0.196900868174357,
                                         1.29710288923076e-10,
                                         0.183711011020281,
                                         6.62014612703728e-12,
                                         0.130594062292864,
                                         2.41890382979408e-13,
                                         0.0664614103150613,
                                         5.63602893705978e-15,
                                         0.0215684660903036,
                                         6.29237250918169e-17,
                                         0.00335394213709748,

                                         0.0567116121619997,
                                         5.96046447753906e-08,
                                         1.76330730493693e-23,
                                         0.172879660991999,
                                         1.43051147460938e-06,
                                         3.33179737104343e-21,
                                         0.252523854552,
                                         1.64508819580078e-05,
                                         3.01658629974369e-19,
                                         0.235214723392,
                                         0.000120639801025391,
                                         1.74163247135383e-17,
                                         0.156849982212,
                                         0.000633358955383301,
                                         7.19871780714592e-16,
                                         0.0796902395519777,
                                         0.0025334358215332,
                                         2.26701541638826e-14,
                                         0.032052960631435,
                                         0.00802254676818848,
                                         5.6519219626264e-13,
                                         0.0104689537805579,
                                         0.0206294059753418,
                                         1.14422110677827e-11,
                                         0.00282567939057081,
                                         0.0438374876976013,
                                         1.91429216508441e-10,
                                         0.000638059272677282,
                                         0.0779333114624023,
                                         2.67932272630843e-09,
                                         0.000121566230605197,
                                         0.116899967193604,
                                         3.16413948050119e-08,
                                         1.96521001417359e-05,
                                         0.148781776428223,
                                         3.1705185826444e-07,
                                         2.70415600000005e-06,
                                         0.161180257797241,
                                         2.70415600000005e-06,
                                         3.1705185826444e-07,
                                         0.148781776428223,
                                         1.96521001417359e-05,
                                         3.16413948050119e-08,
                                         0.116899967193604,
                                         0.000121566230605197,
                                         2.67932272630843e-09,
                                         0.0779333114624023,
                                         0.000638059272677282,
                                         1.91429216508441e-10,
                                         0.0438374876976013,
                                         0.00282567939057081,
                                         1.14422110677827e-11,
                                         0.0206294059753418,
                                         0.0104689537805579,
                                         5.6519219626264e-13,
                                         0.00802254676818848,
                                         0.032052960631435,
                                         2.26701541638826e-14,
                                         0.0025334358215332,
                                         0.0796902395519777,
                                         7.19871780714592e-16,
                                         0.000633358955383301,
                                         0.156849982212,
                                         1.74163247135383e-17,
                                         0.000120639801025391,
                                         0.235214723392,
                                         3.01658629974369e-19,
                                         1.64508819580078e-05,
                                         0.252523854552,
                                         3.33179737104343e-21,
                                         1.43051147460938e-06,
                                         0.172879660991999,
                                         1.76330730493693e-23,
                                         5.96046447753906e-08,
                                         0.0567116121619997,

                                         0.177810123525355,
                                         6.69341593090892e-05,
                                         2.78377366784474e-12,
                                         1.57546522136349e-28,
                                         0.318403797957171,
                                         0.000791255622975459,
                                         1.35639883974353e-10,
                                         5.06768425605076e-26,
                                         0.273203710754876,
                                         0.00448200569694049,
                                         3.1668500160496e-09,
                                         7.8108244040128e-24,
                                         0.149485316032662,
                                         0.0161894309903072,
                                         4.71488567911688e-08,
                                         7.67693364024058e-22,
                                         0.0585555886701741,
                                         0.0418647635660599,
                                         5.02542407293231e-07,
                                         5.40177606174137e-20,
                                         0.0174758720656146,
                                         0.0824833685274079,
                                         4.08107853767079e-06,
                                         2.89591661895561e-18,
                                         0.00412906501639676,
                                         0.128654948077597,
                                         2.62373246175836e-05,
                                         1.22907359846164e-16,
                                         0.000792203397225106,
                                         0.162951631670811,
                                         0.000136973407255757,
                                         4.23586237076772e-15,
                                         0.000125604774712003,
                                         0.170559299023206,
                                         0.000590931905447131,
                                         1.20639703505295e-13,
                                         1.66607342113178e-05,
                                         0.149351858155352,
                                         0.00213283450227472,
                                         2.87446629317394e-12,
                                         1.86464391919245e-06,
                                         0.110346774714036,
                                         0.00649517208795343,
                                         5.77880400854676e-11,
                                         1.77068689799715e-07,
                                         0.0691755704068269,
                                         0.0167829483509102,
                                         9.85740957513729e-10,
                                         1.43124635640194e-08,
                                         0.0369124277356356,
                                         0.0369124277356356,
                                         1.43124635640196e-08,
                                         9.85740957513719e-10,
                                         0.0167829483509102,
                                         0.0691755704068269,
                                         1.77068689799717e-07,
                                         5.7788040085467e-11,
                                         0.00649517208795343,
                                         0.110346774714036,
                                         1.86464391919246e-06,
                                         2.8744662931739e-12,
                                         0.00213283450227472,
                                         0.149351858155352,
                                         1.6660734211318e-05,
                                         1.20639703505294e-13,
                                         0.000590931905447131,
                                         0.170559299023206,
                                         0.000125604774712004,
                                         4.23586237076766e-15,
                                         0.000136973407255757,
                                         0.162951631670811,
                                         0.00079220339722511,
                                         1.22907359846162e-16,
                                         2.62373246175836e-05,
                                         0.128654948077597,
                                         0.00412906501639678,
                                         2.89591661895556e-18,
                                         4.08107853767079e-06,
                                         0.0824833685274079,
                                         0.0174758720656146,
                                         5.40177606174128e-20,
                                         5.02542407293231e-07,
                                         0.0418647635660599,
                                         0.0585555886701743,
                                         7.67693364024045e-22,
                                         4.71488567911688e-08,
                                         0.0161894309903072,
                                         0.149485316032662,
                                         7.81082440401266e-24,
                                         3.1668500160496e-09,
                                         0.00448200569694049,
                                         0.273203710754877,
                                         5.06768425605067e-26,
                                         1.35639883974353e-10,
                                         0.000791255622975459,
                                         0.318403797957171,
                                         1.57546522136346e-28,
                                         2.78377366784474e-12,
                                         6.69341593090892e-05,
                                         0.177810123525355,

                                         0.315655286016949,
                                         0.00184251994337111,
                                         5.96046447753906e-08,
                                         5.20108734202899e-16,
                                         1.2893995264756e-32,
                                         0.372869256308418,
                                         0.0132658531961177,
                                         1.43051147460938e-06,
                                         4.16096095714354e-14,
                                         6.28733324600721e-30,
                                         0.211050645709245,
                                         0.045766191681805,
                                         1.64508819580078e-05,
                                         1.59506994968547e-12,
                                         1.46903469130571e-27,
                                         0.0761763886064618,
                                         0.100683417689655,
                                         0.000120639801025391,
                                         3.89914522937766e-11,
                                         2.18877545343692e-25,
                                         0.0196839363206672,
                                         0.158572911620947,
                                         0.000633358955383301,
                                         6.82365352132588e-10,
                                         2.3346847398023e-23,
                                         0.00387529003010547,
                                         0.190283328548006,
                                         0.0025334358215332,
                                         9.09840385934752e-09,
                                         1.89738720519356e-21,
                                         0.000604002630374271,
                                         0.180765205079954,
                                         0.00802254676818848,
                                         9.6040809739511e-08,
                                         1.22074807995998e-19,
                                         7.64443801194653e-05,
                                         0.139444391411426,
                                         0.0206294059753418,
                                         8.23224961035594e-07,
                                         6.37776109055896e-18,
                                         7.99533685781346e-06,
                                         0.0888938535939164,
                                         0.0438374876976013,
                                         5.83130445470909e-06,
                                         2.75356233649016e-16,
                                         6.99594695055138e-07,
                                         0.0474090174430108,
                                         0.0779333114624023,
                                         3.45566346955674e-05,
                                         9.9458090648566e-15,
                                         5.16499654088871e-08,
                                         0.0213335908463927,
                                         0.116899967193604,
                                         0.000172786955787829,
                                         3.03109193019274e-13,
                                         3.23547206025641e-09,
                                         0.00814537456230399,
                                         0.148781776428223,
                                         0.000733051616523831,
                                         7.8379446184789e-12,
                                         1.72517052077355e-10,
                                         0.00264718878446374,
                                         0.161180257797241,
                                         0.00264718878446374,
                                         1.72517052077357e-10,
                                         7.83794461847878e-12,
                                         0.000733051616523831,
                                         0.148781776428223,
                                         0.00814537456230399,
                                         3.23547206025645e-09,
                                         3.03109193019269e-13,
                                         0.000172786955787829,
                                         0.116899967193604,
                                         0.0213335908463927,
                                         5.16499654088877e-08,
                                         9.94580906485643e-15,
                                         3.45566346955674e-05,
                                         0.0779333114624023,
                                         0.0474090174430108,
                                         6.99594695055146e-07,
                                         2.75356233649011e-16,
                                         5.83130445470909e-06,
                                         0.0438374876976013,
                                         0.0888938535939164,
                                         7.99533685781353e-06,
                                         6.37776109055883e-18,
                                         8.23224961035594e-07,
                                         0.0206294059753418,
                                         0.139444391411426,
                                         7.64443801194659e-05,
                                         1.22074807995995e-19,
                                         9.6040809739511e-08,
                                         0.00802254676818848,
                                         0.180765205079954,
                                         0.000604002630374275,
                                         1.89738720519352e-21,
                                         9.09840385934752e-09,
                                         0.0025334358215332,
                                         0.190283328548006,
                                         0.0038752900301055,
                                         2.33468473980225e-23,
                                         6.82365352132588e-10,
                                         0.000633358955383301,
                                         0.158572911620947,
                                         0.0196839363206673,
                                         2.18877545343686e-25,
                                         3.89914522937766e-11,
                                         0.000120639801025391,
                                         0.100683417689655,
                                         0.076176388606462,
                                         1.46903469130567e-27,
                                         1.59506994968547e-12,
                                         1.64508819580078e-05,
                                         0.045766191681805,
                                         0.211050645709245,
                                         6.28733324600704e-30,
                                         4.16096095714354e-14,
                                         1.43051147460938e-06,
                                         0.0132658531961177,
                                         0.372869256308419,
                                         1.28939952647557e-32,
                                         5.20108734202899e-16,
                                         5.96046447753906e-08,
                                         0.00184251994337111,
                                         0.315655286016949,

                                         0.438514277215283,
                                         0.0116269321214892,
                                         1.01341510625309e-05,
                                         8.58472515082783e-11,
                                         3.11625485702533e-19,
                                         4.82255376863472e-36,
                                         0.367774998175793,
                                         0.0569093167767421,
                                         0.000149507418022392,
                                         3.35176461020565e-09,
                                         3.66722213354079e-17,
                                         3.3120821312713e-33,
                                         0.147797560191449,
                                         0.133471396677634,
                                         0.00105687763466284,
                                         6.27057296320628e-08,
                                         2.0678930068325e-15,
                                         1.08996289934067e-30,
                                         0.0378752853539885,
                                         0.199616586212495,
                                         0.00476420301469266,
                                         7.48073939855597e-07,
                                         7.43571092689603e-14,
                                         2.28731468926715e-28,
                                         0.00694868203843037,
                                         0.213728737565151,
                                         0.0153749491782268,
                                         6.38910026005725e-06,
                                         1.91414488891359e-12,
                                         3.43635481586772e-26,
                                         0.000971291645750028,
                                         0.174353192847092,
                                         0.0378040044939576,
                                         4.15753169466169e-05,
                                         3.75428991181664e-11,
                                         3.93342404888165e-24,
                                         0.000107482863068119,
                                         0.112600222413444,
                                         0.0735875396102604,
                                         0.000214177474580794,
                                         5.8293907035731e-10,
                                         3.56439558255012e-22,
                                         9.65830495869382e-06,
                                         0.0590501093818343,
                                         0.116316914579215,
                                         0.000895950740152576,
                                         7.35005579981209e-09,
                                         2.62284809877421e-20,
                                         7.17211213275074e-07,
                                         0.0255909632874044,
                                         0.151937835042879,
                                         0.00309726910901885,
                                         7.65847991226181e-08,
                                         1.59494184061457e-18,
                                         4.45565715972925e-08,
                                         0.00927835745445337,
                                         0.166038015105112,
                                         0.00895761285349299,
                                         6.67594973051707e-07,
                                         8.1140018402601e-17,
                                         2.33555580304814e-09,
                                         0.00283837224872621,
                                         0.153095673596367,
                                         0.021858452969346,
                                         4.91017899237382e-06,
                                         3.48288502802896e-15,
                                         1.03875426826936e-10,
                                         0.00073673566695345,
                                         0.1197739560835,
                                         0.0452575089692032,
                                         3.0642604894977e-05,
                                         1.26849063826634e-13,
                                         3.93244040839439e-12,
                                         0.000162772410394023,
                                         0.079760639341759,
                                         0.079760639341759,
                                         0.000162772410394022,
                                         3.93244040839439e-12,
                                         1.26849063826634e-13,
                                         3.06426048949772e-05,
                                         0.0452575089692032,
                                         0.1197739560835,
                                         0.000736735666953447,
                                         1.03875426826936e-10,
                                         3.48288502802896e-15,
                                         4.91017899237385e-06,
                                         0.021858452969346,
                                         0.153095673596367,
                                         0.0028383722487262,
                                         2.33555580304814e-09,
                                         8.1140018402601e-17,
                                         6.6759497305171e-07,
                                         0.00895761285349299,
                                         0.166038015105112,
                                         0.00927835745445334,
                                         4.45565715972925e-08,
                                         1.59494184061457e-18,
                                         7.65847991226185e-08,
                                         0.00309726910901885,
                                         0.151937835042879,
                                         0.0255909632874044,
                                         7.17211213275074e-07,
                                         2.62284809877421e-20,
                                         7.35005579981213e-09,
                                         0.000895950740152576,
                                         0.116316914579215,
                                         0.0590501093818342,
                                         9.65830495869382e-06,
                                         3.56439558255012e-22,
                                         5.82939070357313e-10,
                                         0.000214177474580794,
                                         0.0735875396102604,
                                         0.112600222413443,
                                         0.000107482863068119,
                                         3.93342404888165e-24,
                                         3.75428991181666e-11,
                                         4.15753169466168e-05,
                                         0.0378040044939576,
                                         0.174353192847092,
                                         0.000971291645750028,
                                         3.43635481586772e-26,
                                         1.9141448889136e-12,
                                         6.38910026005725e-06,
                                         0.0153749491782268,
                                         0.213728737565151,
                                         0.00694868203843037,
                                         2.28731468926715e-28,
                                         7.43571092689608e-14,
                                         7.48073939855596e-07,
                                         0.00476420301469266,
                                         0.199616586212495,
                                         0.0378752853539885,
                                         1.08996289934067e-30,
                                         2.06789300683251e-15,
                                         6.27057296320628e-08,
                                         0.00105687763466284,
                                         0.133471396677634,
                                         0.147797560191449,
                                         3.3120821312713e-33,
                                         3.66722213354082e-17,
                                         3.35176461020565e-09,
                                         0.000149507418022392,
                                         0.0569093167767421,
                                         0.367774998175793,
                                         4.82255376863472e-36,
                                         3.11625485702535e-19,
                                         8.58472515082783e-11,
                                         1.01341510625309e-05,
                                         0.0116269321214892,
                                         0.438514277215283,

                                         0.538692999663174,
                                         0.0361102290060981,
                                         0.000211728734384434,
                                         5.96046447753906e-08,
                                         2.23291194651997e-13,
                                         4.71059450607703e-22,
                                         5.43108376516697e-39,
                                         0.337572418876475,
                                         0.128622924324767,
                                         0.00214759904111461,
                                         1.43051147460938e-06,
                                         1.26800416722814e-11,
                                         7.6174579938279e-20,
                                         4.99210096968078e-36,
                                         0.101362935201475,
                                         0.219529614424029,
                                         0.0104379016367066,
                                         1.64508819580078e-05,
                                         3.45029606632015e-10,
                                         5.90243228542299e-18,
                                         2.19870426606665e-33,
                                         0.0194086347088404,
                                         0.238930414027657,
                                         0.0323501861586046,
                                         0.000120639801025391,
                                         5.98681214455264e-09,
                                         2.91645677816612e-16,
                                         6.17524031349195e-31,
                                         0.00266053313233073,
                                         0.186169114903204,
                                         0.0717791391825906,
                                         0.000633358955383301,
                                         7.43691428875415e-08,
                                         1.03166323266657e-14,
                                         1.24164883238278e-28,
                                         0.000277870885502335,
                                         0.110520927342406,
                                         0.121344583458592,
                                         0.0025334358215332,
                                         7.0386700829199e-07,
                                         2.78048815582124e-13,
                                         1.9021483888402e-26,
                                         2.29752252371251e-05,
                                         0.0519426097145937,
                                         0.162399580821793,
                                         0.00802254676818848,
                                         5.2738867763737e-06,
                                         5.93261963858548e-12,
                                         2.30691908671736e-24,
                                         1.54258329798318e-06,
                                         0.0198232622201124,
                                         0.176490578717025,
                                         0.0206294059753418,
                                         3.20880339700755e-05,
                                         1.02788392735892e-10,
                                         2.27191691666431e-22,
                                         8.55897861453158e-08,
                                         0.00625188463040372,
                                         0.158504873441183,
                                         0.0438374876976013,
                                         0.000161339194726953,
                                         1.47172312741066e-09,
                                         1.84900036820939e-20,
                                         3.97295637682455e-09,
                                         0.00164954938918297,
                                         0.119091907406882,
                                         0.0779333114624023,
                                         0.000678664605617253,
                                         1.76289600233921e-08,
                                         1.25892566924006e-18,
                                         1.55603527316806e-10,
                                         0.000367226046633173,
                                         0.0754980384733542,
                                         0.116899967193604,
                                         0.00240870880906796,
                                         1.78172657240794e-07,
                                         7.23230887921148e-17,
                                         5.17093609037996e-12,
                                         6.93658516478427e-05,
                                         0.0406100175316316,
                                         0.148781776428223,
                                         0.00725366498071213,
                                         1.52791611917122e-06,
                                         3.52531504384252e-15,
                                         1.46266765789661e-13,
                                         1.11528207948207e-05,
                                         0.0185933413043722,
                                         0.161180257797241,
                                         0.0185933413043722,
                                         1.11528207948207e-05,
                                         1.46266765789665e-13,
                                         3.52531504384242e-15,
                                         1.52791611917122e-06,
                                         0.00725366498071213,
                                         0.148781776428223,
                                         0.0406100175316316,
                                         6.93658516478427e-05,
                                         5.17093609038009e-12,
                                         7.23230887921126e-17,
                                         1.78172657240794e-07,
                                         0.00240870880906796,
                                         0.116899967193604,
                                         0.0754980384733542,
                                         0.000367226046633173,
                                         1.5560352731681e-10,
                                         1.25892566924002e-18,
                                         1.76289600233921e-08,
                                         0.000678664605617253,
                                         0.0779333114624023,
                                         0.119091907406882,
                                         0.00164954938918297,
                                         3.97295637682463e-09,
                                         1.84900036820933e-20,
                                         1.47172312741066e-09,
                                         0.000161339194726953,
                                         0.0438374876976013,
                                         0.158504873441183,
                                         0.00625188463040373,
                                         8.55897861453173e-08,
                                         2.27191691666423e-22,
                                         1.02788392735892e-10,
                                         3.20880339700755e-05,
                                         0.0206294059753418,
                                         0.176490578717025,
                                         0.0198232622201124,
                                         1.5425832979832e-06,
                                         2.30691908671727e-24,
                                         5.93261963858548e-12,
                                         5.2738867763737e-06,
                                         0.00802254676818848,
                                         0.162399580821793,
                                         0.0519426097145937,
                                         2.29752252371254e-05,
                                         1.90214838884012e-26,
                                         2.78048815582124e-13,
                                         7.0386700829199e-07,
                                         0.0025334358215332,
                                         0.121344583458592,
                                         0.110520927342406,
                                         0.000277870885502338,
                                         1.24164883238273e-28,
                                         1.03166323266657e-14,
                                         7.43691428875415e-08,
                                         0.000633358955383301,
                                         0.0717791391825906,
                                         0.186169114903204,
                                         0.00266053313233076,
                                         6.17524031349166e-31,
                                         2.91645677816612e-16,
                                         5.98681214455264e-09,
                                         0.000120639801025391,
                                         0.0323501861586046,
                                         0.238930414027657,
                                         0.0194086347088405,
                                         2.19870426606654e-33,
                                         5.90243228542299e-18,
                                         3.45029606632015e-10,
                                         1.64508819580078e-05,
                                         0.0104379016367066,
                                         0.219529614424029,
                                         0.101362935201475,
                                         4.99210096968054e-36,
                                         7.6174579938279e-20,
                                         1.26800416722814e-11,
                                         1.43051147460938e-06,
                                         0.00214759904111461,
                                         0.128622924324767,
                                         0.337572418876475,
                                         5.43108376516668e-39,
                                         4.71059450607703e-22,
                                         2.23291194651997e-13,
                                         5.96046447753906e-08,
                                         0.000211728734384434,
                                         0.0361102290060981,
                                         0.538692999663174,

                                         0.617969624020661,
                                         0.0762955586501546,
                                         0.00150450845193105,
                                         3.39424181910787e-06,
                                         4.60323273106558e-10,
                                         1.00980461726728e-15,
                                         1.48694783309534e-24,
                                         1.40901319447516e-41,
                                         0.300441231610955,
                                         0.207229709817217,
                                         0.011230290405759,
                                         5.62083316127609e-05,
                                         1.60113334165309e-08,
                                         7.79226081706981e-14,
                                         3.15330118682272e-22,
                                         1.66934129923522e-38,
                                         0.0699904100357951,
                                         0.269706478765585,
                                         0.0401674202963041,
                                         0.000446010628407016,
                                         2.66857042526513e-07,
                                         2.88121858523456e-12,
                                         3.20421592711962e-20,
                                         9.47680216284835e-36,
                                         0.010397313230034,
                                         0.223838261911916,
                                         0.0916137119621587,
                                         0.00225680121613182,
                                         2.83617759353648e-06,
                                         6.79348342726469e-11,
                                         2.07625973207827e-18,
                                         3.43069474021318e-33,
                                         0.00110576183156794,
                                         0.132994949255034,
                                         0.149590804145792,
                                         0.00817521689402206,
                                         2.15797322583899e-05,
                                         1.14674461745942e-09,
                                         9.63162276971927e-17,
                                         8.89119665928699e-31,
                                         8.95989149065224e-05,
                                         0.0602055681634366,
                                         0.186101554789321,
                                         0.0225634730166935,
                                         0.000125100593580954,
                                         1.47482872709847e-08,
                                         3.40422248792696e-15,
                                         1.75565445863912e-28,
                                         5.74759640599298e-06,
                                         0.0215764767464015,
                                         0.18328944888543,
                                         0.049300914083389,
                                         0.00057413635514451,
                                         1.50161578705809e-07,
                                         9.5253022891032e-14,
                                         2.74448101255104e-26,
                                         2.99393173321856e-07,
                                         0.00627908720209111,
                                         0.146587535663198,
                                         0.0874734205950338,
                                         0.00213965050196425,
                                         1.24150415476107e-06,
                                         2.16427135970501e-12,
                                         3.48380254238212e-24,
                                         1.28878950749717e-08,
                                         0.00151006961386096,
                                         0.0968815529266612,
                                         0.128257188939964,
                                         0.00658954004125009,
                                         8.48246347595535e-06,
                                         4.06377006600467e-11,
                                         3.65452807267682e-22,
                                         4.6413105574429e-10,
                                         0.00030381972173662,
                                         0.0535677836705157,
                                         0.157327942588693,
                                         0.0169779754594675,
                                         4.84857768307586e-05,
                                         6.38359139135815e-10,
                                         3.20721158286391e-20,
                                         1.41030501941837e-11,
                                         5.15761286486921e-05,
                                         0.0249907937639296,
                                         0.162833514085463,
                                         0.0369088477735251,
                                         0.000233840916180119,
                                         8.46086605332437e-09,
                                         2.37485832800104e-18,
                                         3.63604764992044e-13,
                                         7.4289187892642e-06,
                                         0.0098923737051489,
                                         0.142996635398534,
                                         0.0680799401093618,
                                         0.000956909224329731,
                                         9.51499688355732e-08,
                                         1.49207925975212e-16,
                                         7.97944776406485e-15,
                                         9.10813033954963e-07,
                                         0.00333309532137399,
                                         0.106889389907393,
                                         0.106889389907393,
                                         0.00333309532137399,
                                         9.10813033954963e-07,
                                         7.97944776406485e-15,
                                         1.49207925975212e-16,
                                         9.51499688355732e-08,
                                         0.000956909224329731,
                                         0.068079940109362,
                                         0.142996635398534,
                                         0.0098923737051489,
                                         7.4289187892642e-06,
                                         3.63604764992044e-13,
                                         2.37485832800104e-18,
                                         8.46086605332437e-09,
                                         0.000233840916180119,
                                         0.0369088477735252,
                                         0.162833514085462,
                                         0.0249907937639296,
                                         5.15761286486921e-05,
                                         1.41030501941837e-11,
                                         3.20721158286391e-20,
                                         6.38359139135814e-10,
                                         4.84857768307586e-05,
                                         0.0169779754594675,
                                         0.157327942588693,
                                         0.0535677836705157,
                                         0.00030381972173662,
                                         4.6413105574429e-10,
                                         3.65452807267682e-22,
                                         4.06377006600467e-11,
                                         8.48246347595535e-06,
                                         0.0065895400412501,
                                         0.128257188939964,
                                         0.0968815529266612,
                                         0.00151006961386096,
                                         1.28878950749717e-08,
                                         3.48380254238212e-24,
                                         2.16427135970501e-12,
                                         1.24150415476107e-06,
                                         0.00213965050196425,
                                         0.0874734205950337,
                                         0.146587535663198,
                                         0.00627908720209111,
                                         2.99393173321856e-07,
                                         2.74448101255104e-26,
                                         9.5253022891032e-14,
                                         1.50161578705809e-07,
                                         0.000574136355144512,
                                         0.049300914083389,
                                         0.18328944888543,
                                         0.0215764767464015,
                                         5.74759640599298e-06,
                                         1.75565445863912e-28,
                                         3.40422248792696e-15,
                                         1.47482872709847e-08,
                                         0.000125100593580954,
                                         0.0225634730166935,
                                         0.186101554789321,
                                         0.0602055681634366,
                                         8.95989149065224e-05,
                                         8.89119665928699e-31,
                                         9.63162276971927e-17,
                                         1.14674461745942e-09,
                                         2.157973225839e-05,
                                         0.00817521689402206,
                                         0.149590804145792,
                                         0.132994949255034,
                                         0.00110576183156794,
                                         3.43069474021318e-33,
                                         2.07625973207827e-18,
                                         6.79348342726469e-11,
                                         2.83617759353648e-06,
                                         0.00225680121613182,
                                         0.0916137119621587,
                                         0.223838261911916,
                                         0.010397313230034,
                                         9.47680216284835e-36,
                                         3.20421592711962e-20,
                                         2.88121858523456e-12,
                                         2.66857042526513e-07,
                                         0.000446010628407016,
                                         0.0401674202963041,
                                         0.269706478765585,
                                         0.0699904100357951,
                                         1.66934129923522e-38,
                                         3.15330118682272e-22,
                                         7.79226081706981e-14,
                                         1.60113334165309e-08,
                                         5.62083316127609e-05,
                                         0.011230290405759,
                                         0.207229709817217,
                                         0.300441231610955,
                                         1.40901319447516e-41,
                                         1.48694783309534e-24,
                                         1.00980461726728e-15,
                                         4.6032327310656e-10,
                                         3.39424181910787e-06,
                                         0.00150450845193105,
                                         0.0762955586501546,
                                         0.617969624020661,

                                         0.680347824117479,
                                         0.128351499041962,
                                         0.00576640061560164,
                                         5.04181153985724e-05,
                                         5.96046447753906e-08,
                                         4.89883266175132e-12,
                                         7.41881714783617e-18,
                                         8.5026682509773e-27,
                                         7.02352474244512e-44,
                                         0.264150586968397,
                                         0.275101916348516,
                                         0.0331646661885316,
                                         0.000617462517857559,
                                         1.43051147460938e-06,
                                         2.30404574058534e-10,
                                         7.42995753325943e-16,
                                         2.28499391310833e-24,
                                         1.04197433087521e-40,
                                         0.0491426711214454,
                                         0.282535409549333,
                                         0.0913972330275352,
                                         0.00362344096327247,
                                         1.64508819580078e-05,
                                         5.19249619623785e-09,
                                         3.56553410193682e-14,
                                         2.94239828758719e-22,
                                         7.40705419713639e-38,
                                         0.00583001299643211,
                                         0.185035790392105,
                                         0.160617939960085,
                                         0.0135592405575956,
                                         0.000120639801025391,
                                         7.4621652675049e-08,
                                         1.09110474217714e-12,
                                         2.41613644632304e-20,
                                         3.35766690682046e-35,
                                         0.000495151574640786,
                                         0.0867553910475315,
                                         0.20207516040083,
                                         0.0363251502220497,
                                         0.000633358955383301,
                                         7.67735135917207e-07,
                                         2.39037572298294e-11,
                                         1.42036293677463e-18,
                                         1.08965138578103e-32,
                                         3.20411056529189e-05,
                                         0.0309911642493271,
                                         0.193701284574735,
                                         0.07414470817466,
                                         0.0025334358215332,
                                         6.01808972308439e-06,
                                         3.98994200746177e-10,
                                         6.36176897481888e-17,
                                         2.6942520914154e-30,
                                         1.64141796646811e-06,
                                         0.00876438952318661,
                                         0.146992246980879,
                                         0.119810604839565,
                                         0.00802254676818848,
                                         3.73463582574327e-05,
                                         5.27241220433537e-09,
                                         2.25579077179555e-15,
                                         5.2738921101117e-28,
                                         6.82814894612571e-08,
                                         0.00201269313296219,
                                         0.0905791980201155,
                                         0.157210835428289,
                                         0.0206294059753418,
                                         0.000188195820479951,
                                         5.65749585461829e-08,
                                         6.49518988458162e-14,
                                         8.38294019647432e-26,
                                         2.34731437618482e-09,
                                         0.000381960038065582,
                                         0.0461261261986223,
                                         0.170472433130883,
                                         0.0438374876976013,
                                         0.00078371139028471,
                                         5.01676319328041e-07,
                                         1.54550123950417e-12,
                                         1.10114868408788e-23,
                                         6.75084396418437e-11,
                                         6.0642462719037e-05,
                                         0.0196509718450126,
                                         0.154648026738127,
                                         0.0779333114624023,
                                         0.00273036609348174,
                                         3.72170115608975e-06,
                                         3.07656009177754e-11,
                                         1.21008004487665e-21,
                                         1.63816886419872e-12,
                                         8.12361845723669e-06,
                                         0.00706374267247382,
                                         0.118371838198125,
                                         0.116899967193604,
                                         0.00802600435947052,
                                         2.32955612988914e-05,
                                         5.16743749985163e-10,
                                         1.12200817191365e-19,
                                         3.37289915032899e-14,
                                         9.23349879613125e-07,
                                         0.00215441715033759,
                                         0.0768770114814452,
                                         0.148781776428223,
                                         0.0200180602453703,
                                         0.000123722575576949,
                                         7.36426069841687e-09,
                                         8.82718101089871e-18,
                                         5.91118838811938e-16,
                                         8.93325707272941e-08,
                                         0.000559308579492155,
                                         0.0424982794910743,
                                         0.161180257797241,
                                         0.0424982794910743,
                                         0.000559308579492157,
                                         8.93325707272941e-08,
                                         5.91118838811963e-16,
                                         8.82718101089831e-18,
                                         7.36426069841687e-09,
                                         0.000123722575576948,
                                         0.0200180602453703,
                                         0.148781776428223,
                                         0.0768770114814453,
                                         0.00215441715033759,
                                         9.23349879613125e-07,
                                         3.37289915032912e-14,
                                         1.1220081719136e-19,
                                         5.16743749985163e-10,
                                         2.32955612988913e-05,
                                         0.00802600435947052,
                                         0.116899967193604,
                                         0.118371838198125,
                                         0.00706374267247384,
                                         8.12361845723669e-06,
                                         1.63816886419878e-12,
                                         1.21008004487658e-21,
                                         3.07656009177754e-11,
                                         3.72170115608974e-06,
                                         0.00273036609348174,
                                         0.0779333114624023,
                                         0.154648026738127,
                                         0.0196509718450126,
                                         6.0642462719037e-05,
                                         6.75084396418458e-11,
                                         1.10114868408781e-23,
                                         1.54550123950417e-12,
                                         5.01676319328039e-07,
                                         0.00078371139028471,
                                         0.0438374876976013,
                                         0.170472433130883,
                                         0.0461261261986224,
                                         0.000381960038065582,
                                         2.34731437618488e-09,
                                         8.38294019647383e-26,
                                         6.49518988458162e-14,
                                         5.65749585461826e-08,
                                         0.000188195820479951,
                                         0.0206294059753418,
                                         0.157210835428289,
                                         0.0905791980201157,
                                         0.00201269313296219,
                                         6.82814894612588e-08,
                                         5.27389211011137e-28,
                                         2.25579077179555e-15,
                                         5.27241220433534e-09,
                                         3.73463582574326e-05,
                                         0.00802254676818848,
                                         0.119810604839565,
                                         0.146992246980879,
                                         0.00876438952318661,
                                         1.64141796646815e-06,
                                         2.69425209141522e-30,
                                         6.36176897481888e-17,
                                         3.98994200746175e-10,
                                         6.0180897230844e-06,
                                         0.0025334358215332,
                                         0.07414470817466,
                                         0.193701284574736,
                                         0.0309911642493271,
                                         3.20411056529194e-05,
                                         1.08965138578095e-32,
                                         1.42036293677463e-18,
                                         2.39037572298293e-11,
                                         7.67735135917207e-07,
                                         0.000633358955383301,
                                         0.0363251502220497,
                                         0.20207516040083,
                                         0.0867553910475315,
                                         0.000495151574640793,
                                         3.35766690682021e-35,
                                         2.41613644632304e-20,
                                         1.09110474217713e-12,
                                         7.4621652675049e-08,
                                         0.000120639801025391,
                                         0.0135592405575956,
                                         0.160617939960085,
                                         0.185035790392105,
                                         0.00583001299643217,
                                         7.40705419713582e-38,
                                         2.94239828758719e-22,
                                         3.5655341019368e-14,
                                         5.19249619623785e-09,
                                         1.64508819580078e-05,
                                         0.00362344096327247,
                                         0.0913972330275352,
                                         0.282535409549333,
                                         0.0491426711214457,
                                         1.04197433087513e-40,
                                         2.28499391310833e-24,
                                         7.42995753325938e-16,
                                         2.30404574058534e-10,
                                         1.43051147460938e-06,
                                         0.000617462517857559,
                                         0.0331646661885316,
                                         0.275101916348516,
                                         0.264150586968398,
                                         7.02352474244453e-44,
                                         8.5026682509773e-27,
                                         7.41881714783612e-18,
                                         4.89883266175132e-12,
                                         5.96046447753906e-08,
                                         5.04181153985724e-05,
                                         0.00576640061560164,
                                         0.128351499041962,
                                         0.680347824117479,

                                         0.729655913393498,
                                         0.187036474499825,
                                         0.0151021597952562,
                                         0.000337337184054724,
                                         1.66655943143072e-06,
                                         1.24488298628835e-09,
                                         7.14500466833947e-14,
                                         8.28120347444094e-20,
                                         7.91401720151268e-29,
                                         5.91622636610896e-46,
                                         0.231491274708485,
                                         0.324768446492059,
                                         0.0691901442474038,
                                         0.00320029160969343,
                                         2.96314703791231e-05,
                                         4.03291081489498e-08,
                                         4.338101036178e-12,
                                         1.04114391274004e-17,
                                         2.62525777388274e-26,
                                         1.07411491239253e-42,
                                         0.0351914972722497,
                                         0.270214062383565,
                                         0.151892387820701,
                                         0.0145479395960796,
                                         0.00025244815339067,
                                         6.26030227105853e-07,
                                         1.26206989443972e-10,
                                         6.27212451637426e-16,
                                         4.17285712511709e-24,
                                         9.34422545212603e-40,
                                         0.00341149272257727,
                                         0.143365829289639,
                                         0.21263334373773,
                                         0.0421713021464588,
                                         0.00137149673990023,
                                         6.19691549609741e-06,
                                         2.34137370871627e-09,
                                         2.40947355738206e-14,
                                         4.2295938195915e-22,
                                         5.18369518843793e-37,
                                         0.000236760381313959,
                                         0.0544554351765953,
                                         0.213100566970514,
                                         0.0875165946398876,
                                         0.00533427299753766,
                                         4.39150844369725e-05,
                                         3.10968174984577e-08,
                                         6.62655084811679e-13,
                                         3.06917525289362e-20,
                                         2.05870197648598e-34,
                                         1.25191343537836e-05,
                                         0.0157593210385357,
                                         0.162719098546033,
                                         0.138377186967891,
                                         0.0158072522887267,
                                         0.000237111467387706,
                                         3.14674709753663e-07,
                                         1.38852464686499e-11,
                                         1.69685761076507e-18,
                                         6.22942834468179e-32,
                                         5.24061105907188e-07,
                                         0.00361057301336894,
                                         0.0983636901958295,
                                         0.173213183685721,
                                         0.0370834294543755,
                                         0.00101352333814951,
                                         2.52086793970932e-06,
                                         2.30336100708725e-10,
                                         7.42696663398387e-17,
                                         1.49226264281725e-29,
                                         1.78140091991213e-08,
                                         0.000671717828396322,
                                         0.0482839966905228,
                                         0.176063570592592,
                                         0.0706440372673075,
                                         0.00351792894529544,
                                         1.63987393924801e-05,
                                         3.10271969540923e-09,
                                         2.63967013876888e-15,
                                         2.90278221768116e-27,
                                         5.00409890449702e-10,
                                         0.000103271881173053,
                                         0.0195864658183116,
                                         0.147891268988889,
                                         0.111213011389967,
                                         0.0100907824344016,
                                         8.81566955178371e-05,
                                         3.45388237968239e-08,
                                         7.75304519218536e-14,
                                         4.66625115213004e-25,
                                         1.17600355932948e-11,
                                         1.32829898175294e-05,
                                         0.00664702764931188,
                                         0.103928367282176,
                                         0.14647187095738,
                                         0.0242148152841044,
                                         0.000396477622063106,
                                         3.21655575610124e-07,
                                         1.90508128325527e-12,
                                         6.27538398736288e-23,
                                         2.3318745008245e-13,
                                         1.44152897828656e-06,
                                         0.00190332379651102,
                                         0.0616225207195678,
                                         0.16276710658326,
                                         0.0490288000536868,
                                         0.00150451264811164,
                                         2.52748397001548e-06,
                                         3.94973984245042e-11,
                                         7.12075897783064e-21,
                                         3.92324836209719e-15,
                                         1.32737919440241e-07,
                                         0.000462425698320936,
                                         0.0310019424642847,
                                         0.153469878435912,
                                         0.0842297398990031,
                                         0.00484414459076967,
                                         1.68511609731604e-05,
                                         6.94812403980764e-10,
                                         6.85577188114311e-19,
                                         5.61840687112553e-17,
                                         1.04038334418203e-08,
                                         9.56308465827038e-05,
                                         0.0132759347153925,
                                         0.123170416676253,
                                         0.123170416676253,
                                         0.0132759347153925,
                                         9.56308465827038e-05,
                                         1.04038334418202e-08,
                                         5.61840687112553e-17,
                                         6.85577188114312e-19,
                                         6.94812403980771e-10,
                                         1.68511609731604e-05,
                                         0.00484414459076967,
                                         0.0842297398990031,
                                         0.153469878435912,
                                         0.0310019424642847,
                                         0.000462425698320936,
                                         1.3273791944024e-07,
                                         3.92324836209719e-15,
                                         7.12075897783064e-21,
                                         3.94973984245046e-11,
                                         2.52748397001548e-06,
                                         0.00150451264811164,
                                         0.0490288000536868,
                                         0.16276710658326,
                                         0.0616225207195678,
                                         0.00190332379651102,
                                         1.44152897828655e-06,
                                         2.3318745008245e-13,
                                         6.27538398736288e-23,
                                         1.9050812832553e-12,
                                         3.21655575610124e-07,
                                         0.000396477622063106,
                                         0.0242148152841044,
                                         0.14647187095738,
                                         0.103928367282176,
                                         0.00664702764931188,
                                         1.32829898175293e-05,
                                         1.17600355932948e-11,
                                         4.66625115213004e-25,
                                         7.75304519218546e-14,
                                         3.45388237968239e-08,
                                         8.81566955178371e-05,
                                         0.0100907824344016,
                                         0.111213011389967,
                                         0.147891268988889,
                                         0.0195864658183116,
                                         0.000103271881173053,
                                         5.00409890449702e-10,
                                         2.90278221768116e-27,
                                         2.63967013876892e-15,
                                         3.10271969540923e-09,
                                         1.63987393924801e-05,
                                         0.00351792894529544,
                                         0.0706440372673075,
                                         0.176063570592592,
                                         0.0482839966905228,
                                         0.000671717828396319,
                                         1.78140091991213e-08,
                                         1.49226264281725e-29,
                                         7.42696663398397e-17,
                                         2.30336100708725e-10,
                                         2.52086793970932e-06,
                                         0.00101352333814951,
                                         0.0370834294543755,
                                         0.173213183685721,
                                         0.0983636901958295,
                                         0.00361057301336893,
                                         5.24061105907188e-07,
                                         6.22942834468179e-32,
                                         1.6968576107651e-18,
                                         1.38852464686499e-11,
                                         3.14674709753663e-07,
                                         0.000237111467387706,
                                         0.0158072522887267,
                                         0.138377186967891,
                                         0.162719098546033,
                                         0.0157593210385356,
                                         1.25191343537836e-05,
                                         2.05870197648598e-34,
                                         3.06917525289367e-20,
                                         6.62655084811679e-13,
                                         3.10968174984577e-08,
                                         4.39150844369725e-05,
                                         0.00533427299753766,
                                         0.0875165946398876,
                                         0.213100566970514,
                                         0.0544554351765951,
                                         0.000236760381313959,
                                         5.18369518843793e-37,
                                         4.22959381959157e-22,
                                         2.40947355738206e-14,
                                         2.34137370871627e-09,
                                         6.19691549609741e-06,
                                         0.00137149673990023,
                                         0.0421713021464588,
                                         0.21263334373773,
                                         0.143365829289639,
                                         0.00341149272257727,
                                         9.34422545212603e-40,
                                         4.17285712511716e-24,
                                         6.27212451637426e-16,
                                         1.26206989443972e-10,
                                         6.26030227105853e-07,
                                         0.00025244815339067,
                                         0.0145479395960796,
                                         0.151892387820701,
                                         0.270214062383565,
                                         0.0351914972722497,
                                         1.07411491239253e-42,
                                         2.62525777388279e-26,
                                         1.04114391274004e-17,
                                         4.338101036178e-12,
                                         4.03291081489498e-08,
                                         2.96314703791231e-05,
                                         0.00320029160969343,
                                         0.0691901442474038,
                                         0.324768446492058,
                                         0.231491274708486,
                                         5.91622636610896e-46,
                                         7.91401720151284e-29,
                                         8.28120347444094e-20,
                                         7.14500466833947e-14,
                                         1.24488298628835e-09,
                                         1.66655943143072e-06,
                                         0.000337337184054724,
                                         0.0151021597952562,
                                         0.187036474499825,
                                         0.729655913393498,

                                         0.76898141259705,
                                         0.247828631942272,
                                         0.0308533521776331,
                                         0.00135933429783521,
                                         1.83155240563043e-05,
                                         5.96046447753906e-08,
                                         3.17358113523933e-11,
                                         1.39533551846338e-15,
                                         1.32477088619763e-21,
                                         1.10506283747979e-30,
                                         7.66522373576381e-48,
                                         0.20311209801603,
                                         0.355970657016205,
                                         0.115491103851622,
                                         0.0103278645183129,
                                         0.000252916804030236,
                                         1.43051147460938e-06,
                                         1.32377371545516e-09,
                                         1.05783242620015e-13,
                                         2.03852988618145e-19,
                                         4.43145901369021e-28,
                                         1.67158078177268e-44,
                                         0.0257064639000387,
                                         0.244998553054218,
                                         0.207148268159062,
                                         0.0375994409942264,
                                         0.00167348806579918,
                                         1.64508819580078e-05,
                                         2.64584561744825e-08,
                                         3.84274618655199e-12,
                                         1.50307270775822e-17,
                                         8.51516559834379e-26,
                                         1.74669268594324e-41,
                                         0.00207468647007712,
                                         0.107526726495505,
                                         0.236928697534811,
                                         0.0872882586899325,
                                         0.00706108012634357,
                                         0.000120639801025391,
                                         3.37224276832674e-07,
                                         8.90164172543936e-11,
                                         7.06718534119432e-16,
                                         1.04338129195574e-23,
                                         1.16388271843376e-38,
                                         0.000119872754859483,
                                         0.033785255440675,
                                         0.194004545057398,
                                         0.145073520053465,
                                         0.0213293458930382,
                                         0.000633358955383301,
                                         3.07702511771023e-06,
                                         1.47623806185186e-09,
                                         2.3788707719941e-14,
                                         9.15273461125203e-22,
                                         5.55213241324647e-36,
                                         5.27702541214936e-06,
                                         0.00808795408506229,
                                         0.121033844944481,
                                         0.183705075674789,
                                         0.0490890494382352,
                                         0.0025334358215332,
                                         2.13916329799088e-05,
                                         1.86527716805284e-08,
                                         6.10092811091319e-13,
                                         6.11729440735519e-20,
                                         2.01795323915076e-33,
                                         1.83907840244552e-07,
                                         0.00153282475859659,
                                         0.0597783763125453,
                                         0.184160515506384,
                                         0.0894404688377884,
                                         0.00802254676818848,
                                         0.000117733266937944,
                                         1.8658327391936e-07,
                                         1.23869197520425e-11,
                                         3.23675817782753e-18,
                                         5.80637045326861e-31,
                                         5.20455278266563e-09,
                                         0.000235894845159188,
                                         0.0239747183491218,
                                         0.14991462497501,
                                         0.132329181478663,
                                         0.0206294059753418,
                                         0.000526170585820023,
                                         1.51556357434254e-06,
                                         2.04222155172155e-10,
                                         1.39069931291819e-16,
                                         1.35665836791633e-28,
                                         1.21716949602177e-10,
                                         3.00005256345383e-05,
                                         0.0079459784403414,
                                         0.100850011636755,
                                         0.16179371778609,
                                         0.0438374876976013,
                                         0.00194329353884634,
                                         1.01732584425879e-05,
                                         2.78244670939281e-09,
                                         4.93788186466785e-15,
                                         2.61951400219242e-26,
                                         2.38142682358995e-12,
                                         3.19196276199306e-06,
                                         0.00220322976269775,
                                         0.056757916071201,
                                         0.16549551237532,
                                         0.0779333114624023,
                                         0.00600438851957166,
                                         5.71300409181245e-05,
                                         3.1715341667897e-08,
                                         1.46678577607365e-13,
                                         4.23145268823065e-24,
                                         3.93130924954216e-14,
                                         2.86550093267321e-07,
                                         0.000515449334053995,
                                         0.02695198250987,
                                         0.142831690014164,
                                         0.116899967193604,
                                         0.0156535538262485,
                                         0.000270696586025626,
                                         3.0501817575618e-07,
                                         3.67626212514646e-12,
                                         5.76729332735017e-22,
                                         5.50657129797158e-16,
                                         2.18266650378971e-08,
                                         0.000102318995411629,
                                         0.0108592312658738,
                                         0.104594059567446,
                                         0.148781776428223,
                                         0.0346259109235286,
                                         0.00108829119004953,
                                         2.48900710160994e-06,
                                         7.81790414922369e-11,
                                         6.66958302060709e-20,
                                         6.56526228573447e-18,
                                         1.41514508979915e-09,
                                         1.72883405132993e-05,
                                         0.00372421036208608,
                                         0.0651952897317313,
                                         0.161180257797241,
                                         0.0651952897317315,
                                         0.00372421036208607,
                                         1.72883405132992e-05,
                                         1.41514508979915e-09,
                                         6.56526228573487e-18,
                                         6.66958302060665e-20,
                                         7.81790414922369e-11,
                                         2.48900710160995e-06,
                                         0.00108829119004953,
                                         0.0346259109235285,
                                         0.148781776428223,
                                         0.104594059567446,
                                         0.0108592312658737,
                                         0.000102318995411629,
                                         2.18266650378971e-08,
                                         5.50657129797189e-16,
                                         5.76729332734976e-22,
                                         3.67626212514646e-12,
                                         3.05018175756182e-07,
                                         0.000270696586025627,
                                         0.0156535538262484,
                                         0.116899967193604,
                                         0.142831690014164,
                                         0.0269519825098699,
                                         0.000515449334053993,
                                         2.86550093267321e-07,
                                         3.93130924954237e-14,
                                         4.23145268823033e-24,
                                         1.46678577607365e-13,
                                         3.17153416678972e-08,
                                         5.71300409181247e-05,
                                         0.00600438851957164,
                                         0.0779333114624023,
                                         0.16549551237532,
                                         0.0567579160712009,
                                         0.00220322976269774,
                                         3.19196276199306e-06,
                                         2.38142682359006e-12,
                                         2.6195140021922e-26,
                                         4.93788186466785e-15,
                                         2.78244670939283e-09,
                                         1.0173258442588e-05,
                                         0.00194329353884634,
                                         0.0438374876976013,
                                         0.16179371778609,
                                         0.100850011636755,
                                         0.00794597844034137,
                                         3.00005256345383e-05,
                                         1.21716949602182e-10,
                                         1.35665836791621e-28,
                                         1.39069931291819e-16,
                                         2.04222155172157e-10,
                                         1.51556357434255e-06,
                                         0.000526170585820022,
                                         0.0206294059753418,
                                         0.132329181478663,
                                         0.14991462497501,
                                         0.0239747183491217,
                                         0.000235894845159188,
                                         5.20455278266582e-09,
                                         5.80637045326807e-31,
                                         3.23675817782753e-18,
                                         1.23869197520426e-11,
                                         1.86583273919361e-07,
                                         0.000117733266937944,
                                         0.00802254676818848,
                                         0.0894404688377885,
                                         0.184160515506384,
                                         0.0597783763125452,
                                         0.00153282475859659,
                                         1.83907840244558e-07,
                                         2.01795323915057e-33,
                                         6.11729440735519e-20,
                                         6.10092811091324e-13,
                                         1.86527716805285e-08,
                                         2.13916329799087e-05,
                                         0.0025334358215332,
                                         0.0490890494382353,
                                         0.183705075674789,
                                         0.12103384494448,
                                         0.00808795408506229,
                                         5.27702541214949e-06,
                                         5.5521324132459e-36,
                                         9.15273461125203e-22,
                                         2.37887077199412e-14,
                                         1.47623806185186e-09,
                                         3.07702511771022e-06,
                                         0.000633358955383301,
                                         0.0213293458930382,
                                         0.145073520053464,
                                         0.194004545057398,
                                         0.033785255440675,
                                         0.000119872754859486,
                                         1.16388271843364e-38,
                                         1.04338129195574e-23,
                                         7.06718534119438e-16,
                                         8.9016417254394e-11,
                                         3.37224276832673e-07,
                                         0.000120639801025391,
                                         0.00706108012634357,
                                         0.0872882586899325,
                                         0.236928697534811,
                                         0.107526726495505,
                                         0.00207468647007715,
                                         1.74669268594304e-41,
                                         8.51516559834379e-26,
                                         1.50307270775823e-17,
                                         3.84274618655201e-12,
                                         2.64584561744824e-08,
                                         1.64508819580078e-05,
                                         0.00167348806579918,
                                         0.0375994409942264,
                                         0.207148268159062,
                                         0.244998553054218,
                                         0.0257064639000389,
                                         1.67158078177248e-44,
                                         4.43145901369021e-28,
                                         2.03852988618147e-19,
                                         1.05783242620016e-13,
                                         1.32377371545516e-09,
                                         1.43051147460938e-06,
                                         0.000252916804030236,
                                         0.0103278645183129,
                                         0.115491103851622,
                                         0.355970657016205,
                                         0.203112098016031,
                                         7.66522373576288e-48,
                                         1.10506283747979e-30,
                                         1.32477088619764e-21,
                                         1.39533551846339e-15,
                                         3.17358113523932e-11,
                                         5.96046447753906e-08,
                                         1.83155240563043e-05,
                                         0.00135933429783521,
                                         0.0308533521776331,
                                         0.247828631942272,
                                         0.76898141259705,

                                         0.800676140287056,
                                         0.307559140127006,
                                         0.0532188339533509,
                                         0.00390134530400374,
                                         0.000109663605170897,
                                         1.01181924715448e-06,
                                         2.40266169992062e-09,
                                         9.89164596319476e-13,
                                         3.54857425524202e-17,
                                         2.89173050082222e-23,
                                         2.17295617347957e-32,
                                         1.42293019741093e-49,
                                         0.178816150609986,
                                         0.371694935299433,
                                         0.166049962683799,
                                         0.0243431782282961,
                                         0.00121639328141427,
                                         1.88783376108256e-05,
                                         7.41744435255906e-08,
                                         5.13664501940391e-11,
                                         3.27577069231278e-15,
                                         5.33834787887255e-21,
                                         1.03565580807643e-29,
                                         3.66991685357992e-46,
                                         0.0191356489360541,
                                         0.215244095819435,
                                         0.248255459799399,
                                         0.0727824658429805,
                                         0.00646505322906421,
                                         0.00016877618241564,
                                         1.09724236308939e-06,
                                         1.27813622039299e-09,
                                         1.44897097929287e-13,
                                         4.72217987607594e-19,
                                         2.3651926554729e-27,
                                         4.53539850291565e-43,
                                         0.00130581975276156,
                                         0.0794839492403233,
                                         0.236680468646896,
                                         0.138764967226784,
                                         0.0219115844452998,
                                         0.000962192917305526,
                                         1.03503353832894e-05,
                                         2.02804840528309e-08,
                                         4.08704544982983e-12,
                                         2.66367826094776e-17,
                                         3.44446030459913e-25,
                                         3.57419518641889e-40,
                                         6.37941933078715e-05,
                                         0.0210128763270311,
                                         0.161541425088316,
                                         0.189404715399154,
                                         0.053165920578231,
                                         0.00392709158327501,
                                         6.98979044221119e-05,
                                         2.30376071422042e-07,
                                         8.25310032794143e-11,
                                         1.07566957238066e-15,
                                         3.59115147284607e-23,
                                         2.01650338847059e-37,
                                         2.3745414457863e-06,
                                         0.00423245412443577,
                                         0.0840051839270386,
                                         0.196971061046966,
                                         0.0982864346695874,
                                         0.0122118255387003,
                                         0.00035964545516932,
                                         1.99387123279409e-06,
                                         1.26977119214618e-09,
                                         3.30960900312881e-14,
                                         2.85263989362142e-21,
                                         8.66802857946993e-35,
                                         6.99714248549547e-08,
                                         0.000674903023540725,
                                         0.0345836337677248,
                                         0.162164736623175,
                                         0.143845468084395,
                                         0.0300630135207512,
                                         0.00146496536351427,
                                         1.36615257862319e-05,
                                         1.54659347752266e-08,
                                         8.06151848198885e-13,
                                         1.79391779651566e-19,
                                         2.94974227375828e-32,
                                         1.67430199869751e-09,
                                         8.73901737001017e-05,
                                         0.0115613160726035,
                                         0.108413292037404,
                                         0.170950706807041,
                                         0.0600975195816458,
                                         0.0048456523369118,
                                         7.60104702584536e-05,
                                         1.52967438132979e-07,
                                         1.59451692420534e-11,
                                         9.16073330283612e-18,
                                         8.15116950512175e-30,
                                         3.31078777524938e-11,
                                         9.35122056389102e-06,
                                         0.0031939520706481,
                                         0.0598953507813939,
                                         0.167892032616182,
                                         0.0992807507072284,
                                         0.0132452943913578,
                                         0.000349487866020495,
                                         1.25027711387031e-06,
                                         2.60630886796455e-10,
                                         3.8658269588595e-16,
                                         1.86140229902099e-27,
                                         5.4770591232633e-13,
                                         8.3712929006371e-07,
                                         0.000738189823452413,
                                         0.0276835787449558,
                                         0.137945587163144,
                                         0.137212000425322,
                                         0.0302893196369137,
                                         0.00134434062621081,
                                         8.5493305454753e-06,
                                         3.56402850665965e-09,
                                         1.36481290355102e-14,
                                         3.55614209737849e-25,
                                         7.64499667691889e-15,
                                         6.32310742697727e-08,
                                         0.00014395325809994,
                                         0.0107960562138324,
                                         0.0956311378126007,
                                         0.160004768364676,
                                         0.0584428258489076,
                                         0.00436315187564713,
                                         4.93255255938211e-05,
                                         4.11216225313831e-08,
                                         4.06553416651766e-13,
                                         5.73233750355457e-23,
                                         9.05422461930455e-17,
                                         4.05240343293985e-09,
                                         2.38187537848391e-05,
                                         0.00357233463168079,
                                         0.0562516058966432,
                                         0.158313462061829,
                                         0.0956790791987655,
                                         0.0120153224185,
                                         0.000241465687622871,
                                         4.02571853228234e-07,
                                         1.02755766221343e-11,
                                         7.84022434676908e-21,
                                         9.12750298203466e-19,
                                         2.2106579203006e-10,
                                         3.35461996431236e-06,
                                         0.00100615730858648,
                                         0.0281641911705209,
                                         0.13333050466918,
                                         0.13333050466918,
                                         0.028164191170521,
                                         0.00100615730858649,
                                         3.35461996431238e-06,
                                         2.21065792030063e-10,
                                         9.127502982034e-19,
                                         7.8402243467697e-21,
                                         1.02755766221341e-11,
                                         4.02571853228231e-07,
                                         0.00024146568762287,
                                         0.0120153224185,
                                         0.0956790791987655,
                                         0.158313462061829,
                                         0.0562516058966433,
                                         0.0035723346316808,
                                         2.38187537848392e-05,
                                         4.05240343293991e-09,
                                         9.05422461930395e-17,
                                         5.73233750355506e-23,
                                         4.06553416651759e-13,
                                         4.11216225313829e-08,
                                         4.93255255938209e-05,
                                         0.00436315187564712,
                                         0.0584428258489076,
                                         0.160004768364676,
                                         0.0956311378126009,
                                         0.0107960562138324,
                                         0.000143953258099941,
                                         6.32310742697735e-08,
                                         7.64499667691843e-15,
                                         3.55614209737881e-25,
                                         1.36481290355099e-14,
                                         3.56402850665963e-09,
                                         8.54933054547527e-06,
                                         0.00134434062621081,
                                         0.0302893196369137,
                                         0.137212000425322,
                                         0.137945587163144,
                                         0.0276835787449558,
                                         0.000738189823452416,
                                         8.37129290063718e-07,
                                         5.477059123263e-13,
                                         1.86140229902117e-27,
                                         3.86582695885943e-16,
                                         2.60630886796453e-10,
                                         1.2502771138703e-06,
                                         0.000349487866020494,
                                         0.0132452943913578,
                                         0.0992807507072284,
                                         0.167892032616182,
                                         0.059895350781394,
                                         0.00319395207064811,
                                         9.3512205638911e-06,
                                         3.31078777524922e-11,
                                         8.15116950512259e-30,
                                         9.16073330283595e-18,
                                         1.59451692420533e-11,
                                         1.52967438132979e-07,
                                         7.60104702584534e-05,
                                         0.0048456523369118,
                                         0.0600975195816458,
                                         0.170950706807041,
                                         0.108413292037404,
                                         0.0115613160726036,
                                         8.73901737001024e-05,
                                         1.67430199869744e-09,
                                         2.9497422737586e-32,
                                         1.79391779651562e-19,
                                         8.06151848198878e-13,
                                         1.54659347752265e-08,
                                         1.36615257862319e-05,
                                         0.00146496536351427,
                                         0.0300630135207512,
                                         0.143845468084396,
                                         0.162164736623175,
                                         0.0345836337677249,
                                         0.000674903023540729,
                                         6.99714248549522e-08,
                                         8.66802857947092e-35,
                                         2.85263989362136e-21,
                                         3.30960900312878e-14,
                                         1.26977119214618e-09,
                                         1.99387123279408e-06,
                                         0.00035964545516932,
                                         0.0122118255387003,
                                         0.0982864346695875,
                                         0.196971061046967,
                                         0.0840051839270388,
                                         0.0042324541244358,
                                         2.37454144578622e-06,
                                         2.01650338847083e-37,
                                         3.59115147284599e-23,
                                         1.07566957238065e-15,
                                         8.25310032794139e-11,
                                         2.30376071422041e-07,
                                         6.98979044221119e-05,
                                         0.00392709158327501,
                                         0.053165920578231,
                                         0.189404715399154,
                                         0.161541425088316,
                                         0.0210128763270312,
                                         6.379419330787e-05,
                                         3.57419518641935e-40,
                                         3.44446030459905e-25,
                                         2.66367826094774e-17,
                                         4.08704544982981e-12,
                                         2.02804840528309e-08,
                                         1.03503353832894e-05,
                                         0.000962192917305526,
                                         0.0219115844452998,
                                         0.138764967226784,
                                         0.236680468646896,
                                         0.0794839492403236,
                                         0.00130581975276153,
                                         4.53539850291625e-43,
                                         2.36519265547284e-27,
                                         4.72217987607589e-19,
                                         1.44897097929286e-13,
                                         1.27813622039299e-09,
                                         1.09724236308939e-06,
                                         0.00016877618241564,
                                         0.00646505322906421,
                                         0.0727824658429805,
                                         0.248255459799399,
                                         0.215244095819436,
                                         0.0191356489360539,
                                         3.66991685358042e-46,
                                         1.0356558080764e-29,
                                         5.33834787887249e-21,
                                         3.27577069231276e-15,
                                         5.13664501940389e-11,
                                         7.41744435255906e-08,
                                         1.88783376108256e-05,
                                         0.00121639328141427,
                                         0.0243431782282961,
                                         0.166049962683799,
                                         0.371694935299433,
                                         0.178816150609985,
                                         1.42293019741113e-49,
                                         2.17295617347951e-32,
                                         2.89173050082219e-23,
                                         3.548574255242e-17,
                                         9.89164596319472e-13,
                                         2.40266169992062e-09,
                                         1.01181924715448e-06,
                                         0.000109663605170897,
                                         0.00390134530400374,
                                         0.0532188339533509,
                                         0.307559140127006,
                                         0.800676140287056,

                                         0.826497942376089,
                                         0.364303973105608,
                                         0.0814618567018763,
                                         0.00884001633745668,
                                         0.0004337824037774,
                                         8.64740585153443e-06,
                                         5.96046447753906e-08,
                                         1.10879609939137e-10,
                                         3.73688306005651e-14,
                                         1.1436598110968e-18,
                                         8.26858290324523e-25,
                                         5.72447510180963e-34,
                                         3.58280190593164e-51,
                                         0.158122580861769,
                                         0.375710333076981,
                                         0.215328779438474,
                                         0.0462016835009334,
                                         0.00396385785129913,
                                         0.000129796304521416,
                                         1.43051147460938e-06,
                                         4.25498161146583e-09,
                                         2.35551486982705e-12,
                                         1.26041803967711e-16,
                                         1.80179672959928e-22,
                                         3.19719350734254e-31,
                                         1.07868133124554e-47,
                                         0.0144954803126323,
                                         0.185664541749407,
                                         0.272732222485484,
                                         0.11570420226804,
                                         0.0173560467450154,
                                         0.000933523836520038,
                                         1.64508819580078e-05,
                                         7.82402598729524e-08,
                                         7.11457220800339e-11,
                                         6.65608653256673e-15,
                                         1.88133918910325e-20,
                                         8.55635455859393e-29,
                                         1.55614523407758e-44,
                                         0.000847373566755671,
                                         0.058507074958173,
                                         0.220279666442507,
                                         0.184775356921624,
                                         0.0484604158127585,
                                         0.00428146187768749,
                                         0.000120639801025391,
                                         9.17416330950459e-07,
                                         1.37029892998671e-09,
                                         2.24143884485938e-13,
                                         1.25265649831305e-18,
                                         1.46019954426672e-26,
                                         1.43156369919423e-41,
                                         3.5462968932571e-05,
                                         0.0131991413505889,
                                         0.127370908381992,
                                         0.211250061770414,
                                         0.096868224293393,
                                         0.0140577763092307,
                                         0.000633358955383301,
                                         7.70123767085299e-06,
                                         1.88946879129636e-08,
                                         5.40371836656861e-12,
                                         5.97110580293682e-17,
                                         1.78399454400129e-24,
                                         9.42820690819828e-39,
                                         1.13077437646486e-06,
                                         0.00226873442658571,
                                         0.0561134240168305,
                                         0.184013771786711,
                                         0.147528602690782,
                                         0.0351675297396286,
                                         0.0025334358215332,
                                         4.92555614944209e-05,
                                         1.98501967301122e-07,
                                         9.92565569538511e-11,
                                         2.16859386784006e-15,
                                         1.6606401087069e-22,
                                         4.73094849878823e-36,
                                         2.85442860658319e-08,
                                         0.000308719457171248,
                                         0.0195706681582329,
                                         0.126895484776131,
                                         0.177874412482566,
                                         0.0696481295443101,
                                         0.00802254676818848,
                                         0.00024939766391351,
                                         1.65094371337443e-06,
                                         1.44333806503188e-09,
                                         6.23510890458299e-14,
                                         1.22377022812981e-20,
                                         1.87935877064707e-33,
                                         5.85105965549332e-10,
                                         3.41127284524359e-05,
                                         0.00554263973251311,
                                         0.0710581875019848,
                                         0.174149744942344,
                                         0.112008031907663,
                                         0.0206294059753418,
                                         0.00102541957232168,
                                         1.11499220939787e-05,
                                         1.70431175381056e-08,
                                         1.45573383411888e-12,
                                         7.32312350216093e-19,
                                         6.06238180291235e-31,
                                         9.91138477127423e-12,
                                         3.11496650917347e-06,
                                         0.0012972154651464,
                                         0.032882630425753,
                                         0.140901843780912,
                                         0.148858399194908,
                                         0.0438374876976013,
                                         0.00348413755981763,
                                         6.22294364869452e-05,
                                         1.66308513654599e-07,
                                         2.80869368521751e-11,
                                         3.62140671623521e-17,
                                         1.6160743708684e-28,
                                         1.40460006868333e-13,
                                         2.37962676487389e-07,
                                         0.000253995559064904,
                                         0.0127302671013598,
                                         0.0953737915890232,
                                         0.165506883577408,
                                         0.0779333114624023,
                                         0.00990392910665232,
                                         0.000290561749973799,
                                         1.3576832624154e-06,
                                         4.53362211915628e-10,
                                         1.49822412434373e-15,
                                         3.60410919334775e-26,
                                         1.67951860911714e-15,
                                         1.53383305980733e-08,
                                         4.1961781202394e-05,
                                         0.0041583617400063,
                                         0.0544697279826034,
                                         0.155264641139399,
                                         0.116899967193604,
                                         0.0237538242227304,
                                         0.00114470822284693,
                                         9.35182199457391e-06,
                                         6.17447512100949e-09,
                                         5.22985989730906e-14,
                                         6.78185224286028e-24,
                                         1.70396647751019e-17,
                                         8.38863764107125e-10,
                                         5.88201053455787e-06,
                                         0.00115252699976315,
                                         0.0263952303234939,
                                         0.123587104689526,
                                         0.148781776428223,
                                         0.0483396659917854,
                                         0.00382644305029753,
                                         5.46560309890746e-05,
                                         7.13508026556255e-08,
                                         1.54898572377024e-12,
                                         1.08278641468859e-21,
                                         1.47151274734368e-19,
                                         3.90509435011509e-11,
                                         7.01817893516115e-07,
                                         0.000271898433716797,
                                         0.0108873569361263,
                                         0.0837337435345084,
                                         0.161180257797241,
                                         0.0837337435345084,
                                         0.0108873569361263,
                                         0.000271898433716796,
                                         7.0181789351612e-07,
                                         3.90509435011503e-11,
                                         1.47151274734355e-19,
                                         1.08278641468869e-21,
                                         1.54898572377027e-12,
                                         7.13508026556249e-08,
                                         5.46560309890748e-05,
                                         0.00382644305029753,
                                         0.0483396659917854,
                                         0.148781776428223,
                                         0.123587104689526,
                                         0.0263952303234939,
                                         0.00115252699976314,
                                         5.88201053455791e-06,
                                         8.38863764107112e-10,
                                         1.70396647751006e-17,
                                         6.78185224286095e-24,
                                         5.22985989730916e-14,
                                         6.17447512100944e-09,
                                         9.35182199457395e-06,
                                         0.00114470822284693,
                                         0.0237538242227304,
                                         0.116899967193604,
                                         0.155264641139399,
                                         0.0544697279826034,
                                         0.00415836174000629,
                                         4.19617812023943e-05,
                                         1.53383305980731e-08,
                                         1.67951860911703e-15,
                                         3.60410919334813e-26,
                                         1.49822412434376e-15,
                                         4.53362211915624e-10,
                                         1.35768326241541e-06,
                                         0.000290561749973799,
                                         0.00990392910665232,
                                         0.0779333114624023,
                                         0.165506883577408,
                                         0.0953737915890232,
                                         0.0127302671013597,
                                         0.000253995559064906,
                                         2.37962676487386e-07,
                                         1.40460006868325e-13,
                                         1.61607437086858e-28,
                                         3.62140671623528e-17,
                                         2.80869368521748e-11,
                                         1.663085136546e-07,
                                         6.22294364869453e-05,
                                         0.00348413755981763,
                                         0.0438374876976013,
                                         0.148858399194908,
                                         0.140901843780912,
                                         0.0328826304257529,
                                         0.0012972154651464,
                                         3.11496650917343e-06,
                                         9.91138477127368e-12,
                                         6.06238180291307e-31,
                                         7.3231235021611e-19,
                                         1.45573383411887e-12,
                                         1.70431175381057e-08,
                                         1.11499220939787e-05,
                                         0.00102541957232168,
                                         0.0206294059753418,
                                         0.112008031907663,
                                         0.174149744942344,
                                         0.0710581875019846,
                                         0.00554263973251313,
                                         3.41127284524355e-05,
                                         5.85105965549304e-10,
                                         1.87935877064731e-33,
                                         1.22377022812984e-20,
                                         6.23510890458292e-14,
                                         1.44333806503189e-09,
                                         1.65094371337443e-06,
                                         0.00024939766391351,
                                         0.00802254676818848,
                                         0.0696481295443101,
                                         0.177874412482566,
                                         0.126895484776131,
                                         0.0195706681582329,
                                         0.000308719457171245,
                                         2.85442860658307e-08,
                                         4.73094849878886e-36,
                                         1.66064010870694e-22,
                                         2.16859386784003e-15,
                                         9.92565569538517e-11,
                                         1.98501967301122e-07,
                                         4.92555614944209e-05,
                                         0.0025334358215332,
                                         0.0351675297396286,
                                         0.147528602690782,
                                         0.184013771786711,
                                         0.0561134240168307,
                                         0.0022687344265857,
                                         1.13077437646482e-06,
                                         9.4282069081996e-39,
                                         1.78399454400134e-24,
                                         5.97110580293676e-17,
                                         5.40371836656864e-12,
                                         1.88946879129636e-08,
                                         7.70123767085299e-06,
                                         0.000633358955383301,
                                         0.0140577763092307,
                                         0.096868224293393,
                                         0.211250061770414,
                                         0.127370908381992,
                                         0.0131991413505888,
                                         3.546296893257e-05,
                                         1.43156369919444e-41,
                                         1.46019954426676e-26,
                                         1.25265649831304e-18,
                                         2.2414388448594e-13,
                                         1.37029892998671e-09,
                                         9.17416330950459e-07,
                                         0.000120639801025391,
                                         0.00428146187768749,
                                         0.0484604158127585,
                                         0.184775356921624,
                                         0.220279666442507,
                                         0.0585070749581727,
                                         0.000847373566755654,
                                         1.55614523407782e-44,
                                         8.55635455859419e-29,
                                         1.88133918910323e-20,
                                         6.65608653256677e-15,
                                         7.11457220800339e-11,
                                         7.82402598729524e-08,
                                         1.64508819580078e-05,
                                         0.000933523836520038,
                                         0.0173560467450154,
                                         0.11570420226804,
                                         0.272732222485485,
                                         0.185664541749406,
                                         0.0144954803126321,
                                         1.07868133124572e-47,
                                         3.19719350734264e-31,
                                         1.80179672959926e-22,
                                         1.26041803967712e-16,
                                         2.35551486982705e-12,
                                         4.25498161146583e-09,
                                         1.43051147460938e-06,
                                         0.000129796304521416,
                                         0.00396385785129913,
                                         0.0462016835009334,
                                         0.215328779438474,
                                         0.375710333076981,
                                         0.158122580861768,
                                         3.58280190593224e-51,
                                         5.72447510180981e-34,
                                         8.26858290324512e-25,
                                         1.14365981109681e-18,
                                         3.73688306005651e-14,
                                         1.10879609939137e-10,
                                         5.96046447753906e-08,
                                         8.64740585153443e-06,
                                         0.0004337824037774,
                                         0.00884001633745668,
                                         0.0814618567018763,
                                         0.364303973105608,
                                         0.826497942376089,

                                         0.847756957343073,
                                         0.417061016319904,
                                         0.114328283546669,
                                         0.0168986352014445,
                                         0.00127906833705136,
                                         4.59243614793787e-05,
                                         6.99441261534103e-07,
                                         3.83172657223236e-09,
                                         5.87612169914683e-12,
                                         1.68941774289152e-15,
                                         4.55602135940756e-20,
                                         2.99399103730439e-26,
                                         1.94139212034635e-35,
                                         1.17188477737157e-52,
                                         0.140499518402522,
                                         0.371455883589324,
                                         0.259489003878364,
                                         0.0751640421613292,
                                         0.00982064760645973,
                                         0.000568915833393771,
                                         1.35126115748617e-05,
                                         1.14242829222781e-07,
                                         2.73217410718953e-10,
                                         1.26739840165004e-13,
                                         5.89996911398724e-18,
                                         7.59814721443514e-24,
                                         1.25553291192175e-32,
                                         4.0729016517763e-49,
                                         0.0111574489182944,
                                         0.158526358915892,
                                         0.282208982624746,
                                         0.160197311343108,
                                         0.036130427941497,
                                         0.00337706572756037,
                                         0.000125087506662399,
                                         1.63211211166932e-06,
                                         6.08712975320301e-09,
                                         4.5559175702038e-12,
                                         3.66100501219531e-16,
                                         9.2395646854037e-22,
                                         3.89071607400019e-30,
                                         6.78280813564919e-46,
                                         0.000565013161642827,
                                         0.0431419045421899,
                                         0.195715982827406,
                                         0.217722796552385,
                                         0.0847636538439393,
                                         0.0127830530646425,
                                         0.000738400810319893,
                                         1.48687551960395e-05,
                                         8.64809263723729e-08,
                                         1.04434056215578e-10,
                                         1.44862005889162e-14,
                                         7.16471471825675e-20,
                                         7.68837508364389e-28,
                                         7.2030881107285e-43,
                                         2.04837798329197e-05,
                                         0.00840533485438444,
                                         0.0971716618802583,
                                         0.211841216855766,
                                         0.142365287756388,
                                         0.0346407708330184,
                                         0.00312052934231211,
                                         9.69744024019373e-05,
                                         8.7960154635384e-07,
                                         1.71382436229997e-09,
                                         4.10361437905168e-13,
                                         3.97744490029339e-18,
                                         1.08767078448556e-25,
                                         5.47628194271855e-40,
                                         5.65799190569945e-07,
                                         0.00124770355856222,
                                         0.0367581508329923,
                                         0.157042683875964,
                                         0.182179388172334,
                                         0.0715222537415085,
                                         0.0100476630966212,
                                         0.000481881552338576,
                                         6.81635692336768e-06,
                                         2.14284701206212e-08,
                                         8.85684978997668e-12,
                                         1.68232589394249e-16,
                                         1.17236016099429e-23,
                                         3.1721488609312e-37,
                                         1.23724840795889e-08,
                                         0.000146625723143651,
                                         0.0110080410378088,
                                         0.0921652731180952,
                                         0.184559637031844,
                                         0.116906105733927,
                                         0.0256120427277096,
                                         0.00189568358157796,
                                         4.18177846211301e-05,
                                         2.12108649482331e-07,
                                         1.51333255492444e-10,
                                         5.63324248807655e-15,
                                         1.00038460500235e-21,
                                         1.45466730835847e-34,
                                         2.19696733610819e-10,
                                         1.39920398975535e-05,
                                         0.00267693939417933,
                                         0.0439227005520885,
                                         0.151826062865096,
                                         0.155169125064565,
                                         0.0530145995583505,
                                         0.00605568827565867,
                                         0.000208325212878934,
                                         1.70489527147011e-06,
                                         2.09972096631666e-09,
                                         1.53171821679222e-13,
                                         6.9317846511691e-20,
                                         5.41683881269219e-32,
                                         3.22384954713283e-12,
                                         1.10340861882968e-06,
                                         0.00053796188557129,
                                         0.0172979733598807,
                                         0.103214426097506,
                                         0.170199350334007,
                                         0.0906841156904538,
                                         0.0159862147594066,
                                         0.000857644123818646,
                                         1.13245643263762e-05,
                                         2.40753862928371e-08,
                                         3.44178532086872e-12,
                                         3.96924214265239e-18,
                                         1.66691177995776e-29,
                                         3.95771457365629e-14,
                                         7.27964578641058e-08,
                                         9.04446774323427e-05,
                                         0.00569927945167883,
                                         0.0587020155243283,
                                         0.15618129643833,
                                         0.129773344728976,
                                         0.0353058208877824,
                                         0.00295386700207103,
                                         6.29308849027733e-05,
                                         2.3094229767553e-07,
                                         6.47004375012657e-11,
                                         1.90146643788857e-16,
                                         4.29138874341276e-27,
                                         4.09947233960242e-16,
                                         4.05226566377026e-09,
                                         1.28300487666719e-05,
                                         0.00158437677025601,
                                         0.0281695175872281,
                                         0.120924398240721,
                                         0.156694430302045,
                                         0.0657901338701073,
                                         0.008583976975315,
                                         0.000295066537257414,
                                         1.86916433757684e-06,
                                         1.02622937670763e-09,
                                         7.68570163087986e-15,
                                         9.32173785285787e-25,
                                         3.60292765855177e-18,
                                         1.91394592274485e-10,
                                         1.54425028975473e-06,
                                         0.000373715466482014,
                                         0.0114696429293048,
                                         0.0794406772000984,
                                         0.16053351421254,
                                         0.104020591267812,
                                         0.0211655833308238,
                                         0.00117387045747506,
                                         1.283617359663e-05,
                                         1.38110201602341e-08,
                                         2.63586078154732e-13,
                                         1.7180666384103e-22,
                                         2.69531718142143e-20,
                                         7.69463762721769e-12,
                                         1.58209963963992e-07,
                                         7.50326755669082e-05,
                                         0.00397509160772521,
                                         0.0444220601916015,
                                         0.139992452432223,
                                         0.139992452432223,
                                         0.0444220601916015,
                                         0.00397509160772523,
                                         7.50326755669085e-05,
                                         1.58209963963992e-07,
                                         7.69463762721755e-12,
                                         2.69531718142142e-20,
                                         1.7180666384103e-22,
                                         2.63586078154738e-13,
                                         1.38110201602341e-08,
                                         1.28361735966299e-05,
                                         0.00117387045747506,
                                         0.0211655833308238,
                                         0.104020591267812,
                                         0.16053351421254,
                                         0.0794406772000984,
                                         0.0114696429293049,
                                         0.000373715466482015,
                                         1.54425028975473e-06,
                                         1.91394592274482e-10,
                                         3.60292765855177e-18,
                                         9.32173785285787e-25,
                                         7.68570163088003e-15,
                                         1.02622937670763e-09,
                                         1.86916433757683e-06,
                                         0.000295066537257413,
                                         0.008583976975315,
                                         0.0657901338701072,
                                         0.156694430302045,
                                         0.120924398240721,
                                         0.0281695175872282,
                                         0.00158437677025601,
                                         1.28300487666719e-05,
                                         4.0522656637702e-09,
                                         4.09947233960242e-16,
                                         4.29138874341275e-27,
                                         1.90146643788862e-16,
                                         6.47004375012657e-11,
                                         2.30942297675529e-07,
                                         6.29308849027731e-05,
                                         0.00295386700207103,
                                         0.0353058208877823,
                                         0.129773344728976,
                                         0.15618129643833,
                                         0.0587020155243284,
                                         0.00569927945167885,
                                         9.04446774323427e-05,
                                         7.27964578641048e-08,
                                         3.95771457365629e-14,
                                         1.66691177995776e-29,
                                         3.96924214265248e-18,
                                         3.44178532086872e-12,
                                         2.40753862928369e-08,
                                         1.13245643263761e-05,
                                         0.000857644123818646,
                                         0.0159862147594066,
                                         0.0906841156904539,
                                         0.170199350334007,
                                         0.103214426097506,
                                         0.0172979733598808,
                                         0.00053796188557129,
                                         1.10340861882967e-06,
                                         3.22384954713283e-12,
                                         5.41683881269219e-32,
                                         6.93178465116929e-20,
                                         1.53171821679222e-13,
                                         2.09972096631664e-09,
                                         1.7048952714701e-06,
                                         0.000208325212878934,
                                         0.00605568827565865,
                                         0.0530145995583505,
                                         0.155169125064565,
                                         0.151826062865097,
                                         0.0439227005520886,
                                         0.00267693939417933,
                                         1.39920398975534e-05,
                                         2.19696733610819e-10,
                                         1.45466730835847e-34,
                                         1.00038460500238e-21,
                                         5.63324248807655e-15,
                                         1.51333255492443e-10,
                                         2.1210864948233e-07,
                                         4.18177846211301e-05,
                                         0.00189568358157795,
                                         0.0256120427277096,
                                         0.116906105733927,
                                         0.184559637031845,
                                         0.0921652731180954,
                                         0.0110080410378088,
                                         0.000146625723143649,
                                         1.23724840795889e-08,
                                         3.1721488609312e-37,
                                         1.17236016099432e-23,
                                         1.68232589394249e-16,
                                         8.85684978997662e-12,
                                         2.14284701206211e-08,
                                         6.81635692336768e-06,
                                         0.000481881552338575,
                                         0.0100476630966212,
                                         0.0715222537415085,
                                         0.182179388172335,
                                         0.157042683875964,
                                         0.0367581508329923,
                                         0.00124770355856221,
                                         5.65799190569945e-07,
                                         5.47628194271855e-40,
                                         1.0876707844856e-25,
                                         3.97744490029339e-18,
                                         4.10361437905165e-13,
                                         1.71382436229997e-09,
                                         8.7960154635384e-07,
                                         9.69744024019371e-05,
                                         0.00312052934231211,
                                         0.0346407708330184,
                                         0.142365287756389,
                                         0.211841216855766,
                                         0.0971716618802583,
                                         0.00840533485438439,
                                         2.04837798329197e-05,
                                         7.2030881107285e-43,
                                         7.68837508364414e-28,
                                         7.16471471825675e-20,
                                         1.44862005889161e-14,
                                         1.04434056215578e-10,
                                         8.64809263723729e-08,
                                         1.48687551960395e-05,
                                         0.000738400810319894,
                                         0.0127830530646425,
                                         0.0847636538439394,
                                         0.217722796552385,
                                         0.195715982827406,
                                         0.0431419045421897,
                                         0.000565013161642827,
                                         6.78280813564919e-46,
                                         3.89071607400032e-30,
                                         9.2395646854037e-22,
                                         3.66100501219528e-16,
                                         4.55591757020378e-12,
                                         6.08712975320301e-09,
                                         1.63211211166931e-06,
                                         0.000125087506662399,
                                         0.00337706572756037,
                                         0.036130427941497,
                                         0.160197311343108,
                                         0.282208982624746,
                                         0.158526358915892,
                                         0.0111574489182944,
                                         4.0729016517763e-49,
                                         1.2555329119218e-32,
                                         7.59814721443515e-24,
                                         5.8999691139872e-18,
                                         1.26739840165004e-13,
                                         2.73217410718953e-10,
                                         1.14242829222781e-07,
                                         1.35126115748617e-05,
                                         0.000568915833393771,
                                         0.00982064760645973,
                                         0.0751640421613292,
                                         0.259489003878364,
                                         0.371455883589323,
                                         0.140499518402522,
                                         1.17188477737157e-52,
                                         1.94139212034642e-35,
                                         2.99399103730439e-26,
                                         4.55602135940752e-20,
                                         1.68941774289151e-15,
                                         5.87612169914683e-12,
                                         3.83172657223234e-09,
                                         6.99441261534103e-07,
                                         4.59243614793787e-05,
                                         0.00127906833705136,
                                         0.0168986352014445,
                                         0.114328283546669,
                                         0.417061016319904,
                                         0.847756957343073,

                                         0.865434088344168,
                                         0.465438208179567,
                                         0.150415438266911,
                                         0.0284904704355395,
                                         0.00304315391069476,
                                         0.000173267954776159,
                                         4.8528472436112e-06,
                                         5.96046447753906e-08,
                                         2.71562949178065e-10,
                                         3.56467238953166e-13,
                                         9.01428312110572e-17,
                                         2.19437379391402e-21,
                                         1.33459832033231e-27,
                                         8.20584513522447e-37,
                                         4.80979701953974e-54,
                                         0.125453400365083,
                                         0.361688050358474,
                                         0.296487288647846,
                                         0.109274707645911,
                                         0.0199458307433463,
                                         0.00180710636128526,
                                         7.74525893891018e-05,
                                         1.43051147460938e-06,
                                         9.8006229022599e-09,
                                         1.9686868486841e-11,
                                         7.92182904798895e-15,
                                         3.29543734272941e-19,
                                         3.89995317151236e-25,
                                         6.08238170730345e-34,
                                         1.91117648233424e-50,
                                         0.00871399839321553,
                                         0.134676830703209,
                                         0.280031148907706,
                                         0.200828934798764,
                                         0.0626421910287281,
                                         0.00903099868254854,
                                         0.000592327452079855,
                                         1.64508819580078e-05,
                                         1.69481945465244e-07,
                                         5.20979046524501e-10,
                                         3.33584863857257e-13,
                                         2.37138521523981e-17,
                                         5.46078162136719e-23,
                                         2.1602828846006e-31,
                                         3.63882722212356e-47,
                                         0.000385972265993131,
                                         0.0319782873527722,
                                         0.168659260453971,
                                         0.235362084639745,
                                         0.125454237605486,
                                         0.0287800423658888,
                                         0.00288862633336055,
                                         0.000120639801025391,
                                         1.86894614776448e-06,
                                         8.79159065671511e-09,
                                         8.95758200830321e-12,
                                         1.08816497436518e-15,
                                         4.87588901940027e-21,
                                         4.89272902821199e-29,
                                         4.41799950770769e-44,
                                         1.22391878636844e-05,
                                         0.00543594921826837,
                                         0.0727230221292812,
                                         0.197471577971257,
                                         0.179871190375512,
                                         0.0656606165167196,
                                         0.0100850660863067,
                                         0.000633358955383301,
                                         1.47546194308e-05,
                                         1.06211743583031e-07,
                                         1.72199991806901e-10,
                                         3.57474654481028e-14,
                                         3.11681260871161e-19,
                                         7.93322150807877e-27,
                                         3.84014628544568e-41,
                                         2.95698955373179e-07,
                                         0.000704038273341404,
                                         0.023891000265464,
                                         0.126233138199713,
                                         0.196489148217016,
                                         0.114135086266206,
                                         0.0268266731993611,
                                         0.0025334358215332,
                                         8.87482758265929e-05,
                                         9.77638521947475e-07,
                                         2.52218040864026e-09,
                                         8.94739180051646e-13,
                                         1.51798772624059e-17,
                                         9.80051020249657e-25,
                                         2.54314171667105e-38,
                                         5.6557387306187e-09,
                                         7.21870852887078e-05,
                                         0.00621354012255144,
                                         0.0638828849712738,
                                         0.16992523938068,
                                         0.157063671721609,
                                         0.0564933395368252,
                                         0.00802254676818848,
                                         0.000422604576721234,
                                         7.12404136448777e-06,
                                         2.92456668630347e-08,
                                         1.77292357367181e-11,
                                         5.85286069184826e-16,
                                         9.58495655735565e-23,
                                         1.33332405521583e-35,
                                         8.7841754570921e-11,
                                         6.01028212621373e-06,
                                         0.00131224822949371,
                                         0.026252325296707,
                                         0.119329913371819,
                                         0.175511038046913,
                                         0.0966050417130732,
                                         0.0206294059753418,
                                         0.0016341073453884,
                                         4.21547651042884e-05,
                                         2.75371521313315e-07,
                                         2.85269728365951e-10,
                                         1.83248415886475e-14,
                                         7.6120867158506e-21,
                                         5.67639987265334e-33,
                                         1.12744959321217e-12,
                                         4.13537292252203e-07,
                                         0.000229022079245563,
                                         0.00891529057338125,
                                         0.0692508693246884,
                                         0.162075583234098,
                                         0.136517021931143,
                                         0.0438374876976013,
                                         0.00522169423718951,
                                         0.000206134832237116,
                                         2.14269811926307e-06,
                                         3.79320078543263e-09,
                                         4.74129240603298e-13,
                                         4.99576239522145e-19,
                                         1.99707657714419e-30,
                                         1.21063104172482e-14,
                                         2.38041698174873e-08,
                                         3.34393055602481e-05,
                                         0.00253292171315024,
                                         0.0336217003054954,
                                         0.125212962286044,
                                         0.16139584795337,
                                         0.0779333114624023,
                                         0.0139592097125973,
                                         0.000843285501256549,
                                         1.39483068142092e-05,
                                         4.21963138487736e-08,
                                         1.02629400001773e-11,
                                         2.74295296746766e-17,
                                         5.87807324362856e-28,
                                         1.09683237881299e-16,
                                         1.15612604696811e-09,
                                         4.11956094486455e-06,
                                         0.000607186094734521,
                                         0.013772979515597,
                                         0.0816196584629566,
                                         0.160994743306304,
                                         0.116899967193604,
                                         0.0314864727544772,
                                         0.00291079532456675,
                                         7.66118202402813e-05,
                                         3.96056350929251e-07,
                                         1.87439287878611e-10,
                                         1.27071668718972e-15,
                                         1.45978552980288e-25,
                                         8.43165466728456e-19,
                                         4.76432548001598e-11,
                                         4.30614555948466e-07,
                                         0.000123499715981927,
                                         0.00478718512005029,
                                         0.0451423691158708,
                                         0.136262114964324,
                                         0.148781776428223,
                                         0.0602602991665399,
                                         0.00852496874280322,
                                         0.000357037770277045,
                                         3.15415870200774e-06,
                                         2.90464828678174e-09,
                                         4.99485858368586e-14,
                                         3.07600620298023e-23,
                                         5.51711599869102e-21,
                                         1.67118456642182e-12,
                                         3.83136242486516e-08,
                                         2.13814353458009e-05,
                                         0.00141631318456948,
                                         0.0212520425948734,
                                         0.0981669528987704,
                                         0.161180257797241,
                                         0.0981669528987704,
                                         0.0212520425948735,
                                         0.00141631318456948,
                                         2.13814353458008e-05,
                                         3.83136242486519e-08,
                                         1.67118456642182e-12,
                                         5.51711599869163e-21,
                                         3.07600620297986e-23,
                                         4.99485858368586e-14,
                                         2.90464828678171e-09,
                                         3.15415870200776e-06,
                                         0.000357037770277045,
                                         0.00852496874280322,
                                         0.0602602991665398,
                                         0.148781776428223,
                                         0.136262114964324,
                                         0.0451423691158708,
                                         0.00478718512005029,
                                         0.000123499715981926,
                                         4.3061455594847e-07,
                                         4.76432548001598e-11,
                                         8.43165466728542e-19,
                                         1.45978552980269e-25,
                                         1.27071668718972e-15,
                                         1.87439287878609e-10,
                                         3.96056350929254e-07,
                                         7.66118202402813e-05,
                                         0.00291079532456675,
                                         0.0314864727544772,
                                         0.116899967193604,
                                         0.160994743306304,
                                         0.0816196584629566,
                                         0.013772979515597,
                                         0.000607186094734519,
                                         4.11956094486458e-06,
                                         1.15612604696811e-09,
                                         1.09683237881309e-16,
                                         5.87807324362774e-28,
                                         2.74295296746766e-17,
                                         1.02629400001772e-11,
                                         4.21963138487738e-08,
                                         1.39483068142092e-05,
                                         0.000843285501256549,
                                         0.0139592097125973,
                                         0.0779333114624023,
                                         0.16139584795337,
                                         0.125212962286044,
                                         0.0336217003054955,
                                         0.00253292171315024,
                                         3.34393055602483e-05,
                                         2.38041698174873e-08,
                                         1.21063104172492e-14,
                                         1.99707657714389e-30,
                                         4.99576239522145e-19,
                                         4.74129240603292e-13,
                                         3.79320078543265e-09,
                                         2.14269811926307e-06,
                                         0.000206134832237116,
                                         0.00522169423718951,
                                         0.0438374876976013,
                                         0.136517021931143,
                                         0.162075583234098,
                                         0.0692508693246884,
                                         0.00891529057338122,
                                         0.000229022079245564,
                                         4.13537292252203e-07,
                                         1.12744959321225e-12,
                                         5.67639987265245e-33,
                                         7.6120867158506e-21,
                                         1.83248415886472e-14,
                                         2.85269728365953e-10,
                                         2.75371521313315e-07,
                                         4.21547651042884e-05,
                                         0.0016341073453884,
                                         0.0206294059753418,
                                         0.0966050417130732,
                                         0.175511038046913,
                                         0.119329913371819,
                                         0.026252325296707,
                                         0.00131224822949371,
                                         6.01028212621374e-06,
                                         8.78417545709267e-11,
                                         1.33332405521561e-35,
                                         9.58495655735565e-23,
                                         5.85286069184818e-16,
                                         1.77292357367182e-11,
                                         2.92456668630347e-08,
                                         7.12404136448777e-06,
                                         0.000422604576721234,
                                         0.00802254676818848,
                                         0.0564933395368252,
                                         0.157063671721609,
                                         0.16992523938068,
                                         0.0638828849712737,
                                         0.00621354012255147,
                                         7.21870852887078e-05,
                                         5.65573873061902e-09,
                                         2.5431417166706e-38,
                                         9.80051020249657e-25,
                                         1.51798772624057e-17,
                                         8.94739180051653e-13,
                                         2.52218040864026e-09,
                                         9.77638521947475e-07,
                                         8.87482758265929e-05,
                                         0.0025334358215332,
                                         0.0268266731993611,
                                         0.114135086266206,
                                         0.196489148217016,
                                         0.126233138199713,
                                         0.0238910002654641,
                                         0.000704038273341404,
                                         2.95698955373192e-07,
                                         3.84014628544497e-41,
                                         7.93322150807877e-27,
                                         3.11681260871156e-19,
                                         3.57474654481031e-14,
                                         1.72199991806901e-10,
                                         1.06211743583031e-07,
                                         1.47546194308e-05,
                                         0.000633358955383301,
                                         0.0100850660863067,
                                         0.0656606165167196,
                                         0.179871190375512,
                                         0.197471577971256,
                                         0.0727230221292814,
                                         0.00543594921826837,
                                         1.22391878636848e-05,
                                         4.41799950770683e-44,
                                         4.89272902821199e-29,
                                         4.8758890194002e-21,
                                         1.08816497436519e-15,
                                         8.95758200830321e-12,
                                         8.79159065671511e-09,
                                         1.86894614776448e-06,
                                         0.000120639801025391,
                                         0.00288862633336055,
                                         0.0287800423658888,
                                         0.125454237605486,
                                         0.235362084639744,
                                         0.168659260453972,
                                         0.0319782873527721,
                                         0.000385972265993141,
                                         3.63882722212282e-47,
                                         2.1602828846006e-31,
                                         5.4607816213671e-23,
                                         2.37138521523983e-17,
                                         3.33584863857257e-13,
                                         5.20979046524501e-10,
                                         1.69481945465244e-07,
                                         1.64508819580078e-05,
                                         0.000592327452079855,
                                         0.00903099868254854,
                                         0.062642191028728,
                                         0.200828934798764,
                                         0.280031148907706,
                                         0.134676830703209,
                                         0.00871399839321569,
                                         1.91117648233383e-50,
                                         6.08238170730345e-34,
                                         3.8999531715123e-25,
                                         3.29543734272944e-19,
                                         7.92182904798895e-15,
                                         1.9686868486841e-11,
                                         9.8006229022599e-09,
                                         1.43051147460938e-06,
                                         7.74525893891018e-05,
                                         0.00180710636128526,
                                         0.0199458307433463,
                                         0.109274707645911,
                                         0.296487288647846,
                                         0.361688050358474,
                                         0.125453400365084,
                                         4.80979701953867e-54,
                                         8.20584513522447e-37,
                                         1.33459832033229e-27,
                                         2.19437379391405e-21,
                                         9.01428312110572e-17,
                                         3.56467238953166e-13,
                                         2.71562949178065e-10,
                                         5.96046447753906e-08,
                                         4.8528472436112e-06,
                                         0.000173267954776159,
                                         0.00304315391069476,
                                         0.0284904704355395,
                                         0.150415438266911,
                                         0.465438208179567,
                                         0.865434088344168,

                                         0.880269651673845,
                                         0.509416468395161,
                                         0.188407949734408,
                                         0.0436850197193425,
                                         0.00616548522592629,
                                         0.000507668971586358,
                                         2.29810015355745e-05,
                                         5.26426862138154e-07,
                                         5.42877604163781e-09,
                                         2.12823878978334e-11,
                                         2.46002957903043e-14,
                                         5.59985784741844e-18,
                                         1.2532771316773e-22,
                                         7.15224604898521e-29,
                                         4.20932644550266e-38,
                                         2.40824931182339e-55,
                                         0.112556922223517,
                                         0.348469715040913,
                                         0.325674008461916,
                                         0.146088227101433,
                                         0.0349492002656485,
                                         0.00452913413870471,
                                         0.000309165246478691,
                                         1.04417376884455e-05,
                                         1.57648438060961e-07,
                                         9.11215547187444e-10,
                                         1.5882843240447e-12,
                                         5.69022586049306e-16,
                                         2.15867328143501e-20,
                                         2.38331284642236e-26,
                                         3.54440248007381e-35,
                                         1.08484439233691e-51,
                                         0.00689628637457454,
                                         0.114220405690957,
                                         0.269745106137464,
                                         0.234090882965186,
                                         0.0949278887236479,
                                         0.0193613816069486,
                                         0.00199296184402232,
                                         9.92416815653671e-05,
                                         2.19363323544359e-06,
                                         1.86942689426927e-08,
                                         4.91363359820933e-11,
                                         2.77056667549332e-14,
                                         1.78161276508954e-18,
                                         3.80545210190587e-24,
                                         1.43007908241691e-32,
                                         2.34163958211935e-48,
                                         0.000269439926851421,
                                         0.0238740413795928,
                                         0.142471381195361,
                                         0.23919790227844,
                                         0.164419751399637,
                                         0.0527789920685901,
                                         0.00819239502698664,
                                         0.000601477014881568,
                                         1.9464441567864e-05,
                                         2.44567890337639e-07,
                                         9.69350578040635e-10,
                                         8.60223481070557e-13,
                                         9.37655490535786e-17,
                                         3.87467295847437e-22,
                                         3.67943099636346e-30,
                                         3.22311824514898e-45,
                                         7.53644429543564e-06,
                                         0.00357244921321536,
                                         0.0538715625277561,
                                         0.174979878588226,
                                         0.20387878174369,
                                         0.103001539705417,
                                         0.0241090814282758,
                                         0.00260976759469701,
                                         0.000123645341000738,
                                         2.29059477433017e-06,
                                         1.36904226491644e-08,
                                         1.91210552895626e-11,
                                         3.53289852662883e-15,
                                         2.82437085237775e-20,
                                         6.77733804500166e-28,
                                         3.17607174203974e-42,
                                         1.60609682269936e-07,
                                         0.000407292931082868,
                                         0.0155200171889889,
                                         0.0975258048243464,
                                         0.192615290659088,
                                         0.153153536368793,
                                         0.0540568992072968,
                                         0.00862750700732034,
                                         0.000598431234246822,
                                         1.63454838862987e-05,
                                         1.47317223802003e-07,
                                         3.23827039462992e-10,
                                         1.01419091099829e-13,
                                         1.56858868515572e-18,
                                         9.51126587036576e-26,
                                         2.38454244340865e-39,
                                         2.70968814738324e-09,
                                         3.67612305822745e-05,
                                         0.00353970592479858,
                                         0.0430321786577536,
                                         0.144062799573272,
                                         0.180282138507212,
                                         0.0959541927029703,
                                         0.022579336006451,
                                         0.00229294220511736,
                                         9.23399547240634e-05,
                                         1.25496772060036e-06,
                                         4.34166910091064e-09,
                                         2.30489223034282e-12,
                                         6.896659652157e-17,
                                         1.05671982327102e-23,
                                         1.41730124766413e-36,
                                         3.71226537729155e-11,
                                         2.69429610859776e-06,
                                         0.000655562770799022,
                                         0.0154183975801857,
                                         0.0874953664626468,
                                         0.172325873005167,
                                         0.138308507907632,
                                         0.0479854008048509,
                                         0.00713418025460844,
                                         0.000423597752559577,
                                         8.6812787078528e-06,
                                         4.72685681153264e-08,
                                         4.25357062622009e-11,
                                         2.46229807199425e-15,
                                         9.53352482577283e-22,
                                         6.84055590224756e-34,
                                         4.20284319824353e-13,
                                         1.63186818712152e-07,
                                         0.000100333474248554,
                                         0.00456530308818663,
                                         0.0439139727526802,
                                         0.136123385836956,
                                         0.164747299934972,
                                         0.0842736018798261,
                                         0.0183433865232398,
                                         0.00160583938964218,
                                         4.96271474047047e-05,
                                         4.25277730639846e-07,
                                         6.48695883228575e-10,
                                         7.26485457408473e-14,
                                         7.1077413979623e-20,
                                         2.72838147810968e-31,
                                         3.98075912040183e-15,
                                         8.26881674361188e-09,
                                         1.28468294415071e-05,
                                         0.00113088522338739,
                                         0.0184390669209545,
                                         0.089956704565551,
                                         0.164174708536072,
                                         0.123820491777415,
                                         0.0394578570846196,
                                         0.00509294638939952,
                                         0.000237341380685573,
                                         3.2010420247235e-06,
                                         8.27650866799059e-09,
                                         1.79321252686024e-12,
                                         4.43331176963484e-18,
                                         9.1041064627062e-29,
                                         3.18128366868628e-17,
                                         3.53521184671536e-10,
                                         1.38790535581833e-06,
                                         0.000236363944667516,
                                         0.00653264320363482,
                                         0.0501589131471945,
                                         0.138040965467579,
                                         0.153499610841332,
                                         0.0716145486905444,
                                         0.0136285574288325,
                                         0.000957726274928047,
                                         2.03293688495189e-05,
                                         8.90978024411765e-08,
                                         3.73465414077676e-11,
                                         2.33312880259363e-16,
                                         2.56320504142153e-26,
                                         2.15716310966059e-19,
                                         1.2824239266806e-11,
                                         1.27223645697913e-07,
                                         4.19167903626178e-05,
                                         0.00196373566835567,
                                         0.0237304945573669,
                                         0.0984813092728882,
                                         0.16146043731875,
                                         0.110284153675614,
                                         0.030943879697664,
                                         0.00327909071646027,
                                         0.000109547004604896,
                                         8.13824766384103e-07,
                                         6.59953148388366e-10,
                                         1.04182133717003e-14,
                                         6.12312973125206e-24,
                                         1.24506013845586e-21,
                                         3.95981171610763e-13,
                                         9.92664697159007e-09,
                                         6.3273457656191e-06,
                                         0.000502462753535319,
                                         0.00955635372685959,
                                         0.0598034715371519,
                                         0.144561193747353,
                                         0.144561193747353,
                                         0.0598034715371518,
                                         0.00955635372685959,
                                         0.000502462753535319,
                                         6.32734576561907e-06,
                                         9.92664697159017e-09,
                                         3.95981171610763e-13,
                                         1.2450601384557e-21,
                                         6.1231297312529e-24,
                                         1.04182133717003e-14,
                                         6.59953148388359e-10,
                                         8.13824766384107e-07,
                                         0.000109547004604896,
                                         0.00327909071646027,
                                         0.0309438796976641,
                                         0.110284153675614,
                                         0.16146043731875,
                                         0.098481309272888,
                                         0.0237304945573669,
                                         0.00196373566835567,
                                         4.19167903626176e-05,
                                         1.27223645697914e-07,
                                         1.2824239266806e-11,
                                         2.15716310966034e-19,
                                         2.56320504142191e-26,
                                         2.33312880259363e-16,
                                         3.73465414077671e-11,
                                         8.9097802441177e-08,
                                         2.03293688495189e-05,
                                         0.000957726274928047,
                                         0.0136285574288326,
                                         0.0716145486905444,
                                         0.153499610841332,
                                         0.138040965467579,
                                         0.0501589131471945,
                                         0.00653264320363482,
                                         0.000236363944667515,
                                         1.38790535581835e-06,
                                         3.53521184671536e-10,
                                         3.18128366868594e-17,
                                         9.10410646270763e-29,
                                         4.43331176963484e-18,
                                         1.79321252686021e-12,
                                         8.27650866799064e-09,
                                         3.2010420247235e-06,
                                         0.000237341380685573,
                                         0.00509294638939953,
                                         0.0394578570846196,
                                         0.123820491777414,
                                         0.164174708536072,
                                         0.089956704565551,
                                         0.0184390669209545,
                                         0.00113088522338739,
                                         1.28468294415072e-05,
                                         8.26881674361188e-09,
                                         3.98075912040145e-15,
                                         2.72838147811014e-31,
                                         7.1077413979623e-20,
                                         7.26485457408463e-14,
                                         6.4869588322858e-10,
                                         4.25277730639846e-07,
                                         4.96271474047047e-05,
                                         0.00160583938964218,
                                         0.0183433865232398,
                                         0.084273601879826,
                                         0.164747299934972,
                                         0.136123385836956,
                                         0.0439139727526802,
                                         0.00456530308818662,
                                         0.000100333474248554,
                                         1.63186818712152e-07,
                                         4.20284319824318e-13,
                                         6.84055590224878e-34,
                                         9.53352482577283e-22,
                                         2.46229807199422e-15,
                                         4.25357062622012e-11,
                                         4.72685681153264e-08,
                                         8.6812787078528e-06,
                                         0.000423597752559578,
                                         0.00713418025460843,
                                         0.0479854008048508,
                                         0.138308507907632,
                                         0.172325873005167,
                                         0.0874953664626469,
                                         0.0154183975801856,
                                         0.000655562770799026,
                                         2.69429610859776e-06,
                                         3.71226537729128e-11,
                                         1.41730124766439e-36,
                                         1.05671982327102e-23,
                                         6.89665965215689e-17,
                                         2.30489223034284e-12,
                                         4.34166910091064e-09,
                                         1.25496772060036e-06,
                                         9.23399547240637e-05,
                                         0.00229294220511736,
                                         0.0225793360064509,
                                         0.0959541927029702,
                                         0.180282138507212,
                                         0.144062799573272,
                                         0.0430321786577535,
                                         0.0035397059247986,
                                         3.67612305822745e-05,
                                         2.70968814738307e-09,
                                         2.38454244340912e-39,
                                         9.51126587036576e-26,
                                         1.5685886851557e-18,
                                         1.0141909109983e-13,
                                         3.23827039462992e-10,
                                         1.47317223802003e-07,
                                         1.63454838862988e-05,
                                         0.000598431234246821,
                                         0.00862750700732031,
                                         0.0540568992072967,
                                         0.153153536368793,
                                         0.192615290659088,
                                         0.0975258048243462,
                                         0.015520017188989,
                                         0.000407292931082868,
                                         1.60609682269927e-07,
                                         3.1760717420404e-42,
                                         6.77733804500166e-28,
                                         2.82437085237771e-20,
                                         3.53289852662886e-15,
                                         1.91210552895626e-11,
                                         1.36904226491644e-08,
                                         2.29059477433018e-06,
                                         0.000123645341000738,
                                         0.002609767594697,
                                         0.0241090814282757,
                                         0.103001539705417,
                                         0.20387878174369,
                                         0.174979878588226,
                                         0.0538715625277563,
                                         0.00357244921321536,
                                         7.53644429543533e-06,
                                         3.22311824514968e-45,
                                         3.67943099636346e-30,
                                         3.8746729584743e-22,
                                         9.37655490535795e-17,
                                         8.60223481070557e-13,
                                         9.69350578040635e-10,
                                         2.44567890337639e-07,
                                         1.9464441567864e-05,
                                         0.000601477014881566,
                                         0.00819239502698664,
                                         0.0527789920685901,
                                         0.164419751399637,
                                         0.239197902278439,
                                         0.142471381195361,
                                         0.0238740413795928,
                                         0.000269439926851413,
                                         2.34163958211989e-48,
                                         1.43007908241691e-32,
                                         3.8054521019058e-24,
                                         1.78161276508956e-18,
                                         2.77056667549332e-14,
                                         4.91363359820933e-11,
                                         1.86942689426927e-08,
                                         2.19363323544359e-06,
                                         9.92416815653667e-05,
                                         0.00199296184402232,
                                         0.0193613816069486,
                                         0.0949278887236479,
                                         0.234090882965186,
                                         0.269745106137465,
                                         0.114220405690957,
                                         0.00689628637457439,
                                         1.08484439233717e-51,
                                         3.54440248007381e-35,
                                         2.38331284642231e-26,
                                         2.15867328143504e-20,
                                         5.69022586049306e-16,
                                         1.5882843240447e-12,
                                         9.11215547187447e-10,
                                         1.57648438060961e-07,
                                         1.04417376884454e-05,
                                         0.000309165246478691,
                                         0.00452913413870471,
                                         0.0349492002656485,
                                         0.146088227101433,
                                         0.325674008461916,
                                         0.348469715040913,
                                         0.112556922223516,
                                         2.40824931182399e-55,
                                         4.20932644550266e-38,
                                         7.15224604898507e-29,
                                         1.25327713167732e-22,
                                         5.59985784741844e-18,
                                         2.46002957903043e-14,
                                         2.12823878978334e-11,
                                         5.4287760416378e-09,
                                         5.26426862138151e-07,
                                         2.29810015355745e-05,
                                         0.000507668971586358,
                                         0.00616548522592629,
                                         0.0436850197193425,
                                         0.188407949734408,
                                         0.509416468395161,
                                         0.880269651673845,

                                         0.892827754065557,
                                         0.549187856103814,
                                         0.227192669517664,
                                         0.0622618294076962,
                                         0.0110469213132802,
                                         0.00122823725525557,
                                         8.18081176367883e-05,
                                         3.06937734567641e-06,
                                         5.96046447753906e-08,
                                         5.32184757276606e-10,
                                         1.84259545626306e-12,
                                         1.9168864238256e-15,
                                         3.99874629751617e-19,
                                         8.34539021993569e-24,
                                         4.51686024619576e-30,
                                         2.56318587166037e-39,
                                         1.43716640180065e-56,
                                         0.101451793966289,
                                         0.333280291685054,
                                         0.347302028866203,
                                         0.183259916510464,
                                         0.0547517002470377,
                                         0.00949616523694436,
                                         0.00094268668898748,
                                         5.1351566672317e-05,
                                         1.43051147460938e-06,
                                         1.83223637268677e-08,
                                         9.21046389429921e-11,
                                         1.42808108998537e-13,
                                         4.64718524696872e-17,
                                         1.63314040917585e-21,
                                         1.70194738953989e-27,
                                         2.43284184253595e-36,
                                         7.2851330896204e-53,
                                         0.00552380550631012,
                                         0.0969136473748028,
                                         0.254394035609468,
                                         0.258463779204353,
                                         0.13002910057349,
                                         0.0351804068848499,
                                         0.00520505051003714,
                                         0.000411664778679015,
                                         1.64508819580078e-05,
                                         3.02264516803829e-07,
                                         2.20607163997971e-09,
                                         5.09795448832852e-12,
                                         2.58787155604007e-15,
                                         1.53139420619326e-19,
                                         3.07285598910721e-25,
                                         1.10645626286615e-33,
                                         1.76951611973608e-49,
                                         0.00019178763862677,
                                         0.0179706518437423,
                                         0.118825627134037,
                                         0.232453174643563,
                                         0.196918760080652,
                                         0.0831107189043027,
                                         0.0183267775129292,
                                         0.00210444369248535,
                                         0.000120639801025391,
                                         3.17977522382423e-06,
                                         3.36946775435194e-08,
                                         1.16049326760788e-10,
                                         9.18965136100876e-14,
                                         9.15701784012493e-18,
                                         3.53786998935301e-23,
                                         3.2089149888064e-31,
                                         2.74078609407875e-46,
                                         4.76717091337435e-06,
                                         0.00238561633281577,
                                         0.0397348138349856,
                                         0.149668067338151,
                                         0.213496865818935,
                                         0.14056302574319,
                                         0.0461960820590724,
                                         0.0077017388105722,
                                         0.000633358955383301,
                                         2.39476858111701e-05,
                                         3.68434995665417e-07,
                                         1.89124230469053e-09,
                                         2.3362172741597e-12,
                                         3.91993619669576e-16,
                                         2.91607982737257e-21,
                                         6.66254337784859e-29,
                                         3.03916051587026e-43,
                                         9.02820727184797e-08,
                                         0.000241289296067748,
                                         0.0101235524681974,
                                         0.073421543037415,
                                         0.176358583981642,
                                         0.181128104925793,
                                         0.0887206813913163,
                                         0.0214753846075098,
                                         0.0025334358215332,
                                         0.000137414122899224,
                                         3.06945402605702e-06,
                                         2.34829368634745e-08,
                                         4.52509930297079e-11,
                                         1.27851145672457e-14,
                                         1.83129215848062e-19,
                                         1.0539562299242e-26,
                                         2.56763225319558e-40,
                                         1.35358233816477e-09,
                                         1.93204782541259e-05,
                                         0.002041912147281,
                                         0.0285141364486071,
                                         0.115330469932473,
                                         0.184774888402796,
                                         0.134892235587757,
                                         0.0474062359611808,
                                         0.00802254676818848,
                                         0.000624225560352743,
                                         2.02443459553578e-05,
                                         2.30834121681091e-07,
                                         6.93881576991313e-10,
                                         3.30120587386969e-13,
                                         9.10454448503432e-18,
                                         1.31991954625867e-24,
                                         1.71733240850619e-37,
                                         1.64793452035548e-11,
                                         1.25623185929335e-06,
                                         0.000334436201388705,
                                         0.00899226411152447,
                                         0.0612440122457624,
                                         0.153063686340744,
                                         0.166540993306318,
                                         0.0849771402824453,
                                         0.0206294059753418,
                                         0.00230262831147745,
                                         0.000108422266644745,
                                         1.84255184418647e-06,
                                         8.64001932771955e-09,
                                         6.92169898959776e-12,
                                         3.67562268478374e-16,
                                         1.34228414295843e-22,
                                         9.32712986427228e-35,
                                         1.65798154804104e-13,
                                         6.75003777662798e-08,
                                         4.52661940101666e-05,
                                         0.00234348594893474,
                                         0.0268761884110356,
                                         0.104781800722844,
                                         0.169918168449934,
                                         0.125878981114039,
                                         0.0438374876976013,
                                         0.00701924823857761,
                                         0.000479863462627245,
                                         1.21541305283133e-05,
                                         8.89054819233458e-08,
                                         1.19932607988145e-10,
                                         1.22627561847869e-14,
                                         1.12804371054605e-20,
                                         4.18625954489787e-32,
                                         1.39552613215238e-15,
                                         3.03431834113331e-09,
                                         5.12570039966134e-06,
                                         0.000510945045904792,
                                         0.00986711655058286,
                                         0.0600092343560472,
                                         0.145036404375067,
                                         0.155999377393965,
                                         0.0779333114624023,
                                         0.0179009426857747,
                                         0.00177678722723294,
                                         6.70728274728778e-05,
                                         7.65351443656696e-07,
                                         1.73852277921089e-09,
                                         3.42266034035971e-13,
                                         7.93096322350856e-19,
                                         1.57189240933816e-29,
                                         9.91083029376376e-18,
                                         1.15087953347681e-10,
                                         4.89718195998036e-07,
                                         9.39939319008882e-05,
                                         0.00305651565920134,
                                         0.0289977365299919,
                                         0.104454707378496,
                                         0.163119656018556,
                                         0.116899967193604,
                                         0.0385189986252395,
                                         0.00555094532167097,
                                         0.000312308006930273,
                                         5.55913290235005e-06,
                                         2.12636237504773e-08,
                                         8.06033881121168e-12,
                                         4.70478443328485e-17,
                                         4.98004501698833e-27,
                                         5.97208798447527e-20,
                                         3.70375868663105e-12,
                                         3.969934630171e-08,
                                         1.46713316016817e-05,
                                         0.000803354227781274,
                                         0.011889243037651,
                                         0.0638297436813152,
                                         0.14472175494992,
                                         0.148781776428223,
                                         0.0703263642892635,
                                         0.0147144003031686,
                                         0.00123385383280476,
                                         3.42607842053249e-05,
                                         2.2066744374294e-07,
                                         1.61059699282495e-10,
                                         2.36808678545827e-15,
                                         1.33871402060012e-24,
                                         3.06315597054714e-22,
                                         1.01457055720739e-13,
                                         2.7393481170557e-09,
                                         1.94924309687571e-06,
                                         0.000179727402510526,
                                         0.00414926398041874,
                                         0.0332005241301147,
                                         0.109291931032945,
                                         0.161180257797241,
                                         0.109291931032945,
                                         0.0332005241301147,
                                         0.00414926398041873,
                                         0.000179727402510525,
                                         1.9492430968757e-06,
                                         2.7393481170557e-09,
                                         1.01457055720736e-13,
                                         3.06315597054757e-22,
                                         1.33871402059992e-24,
                                         2.36808678545834e-15,
                                         1.61059699282495e-10,
                                         2.20667443742942e-07,
                                         3.4260784205325e-05,
                                         0.00123385383280476,
                                         0.0147144003031686,
                                         0.0703263642892635,
                                         0.148781776428223,
                                         0.14472175494992,
                                         0.0638297436813152,
                                         0.0118892430376509,
                                         0.000803354227781271,
                                         1.46713316016816e-05,
                                         3.96993463017099e-08,
                                         3.70375868663096e-12,
                                         5.97208798447604e-20,
                                         4.98004501698751e-27,
                                         4.70478443328499e-17,
                                         8.06033881121168e-12,
                                         2.12636237504775e-08,
                                         5.55913290235008e-06,
                                         0.000312308006930274,
                                         0.00555094532167098,
                                         0.0385189986252395,
                                         0.116899967193604,
                                         0.163119656018556,
                                         0.104454707378496,
                                         0.0289977365299919,
                                         0.00305651565920133,
                                         9.39939319008877e-05,
                                         4.89718195998036e-07,
                                         1.15087953347678e-10,
                                         9.91083029376493e-18,
                                         1.57189240933789e-29,
                                         7.93096322350882e-19,
                                         3.42266034035971e-13,
                                         1.7385227792109e-09,
                                         7.653514436567e-07,
                                         6.70728274728781e-05,
                                         0.00177678722723294,
                                         0.0179009426857747,
                                         0.0779333114624023,
                                         0.155999377393965,
                                         0.145036404375067,
                                         0.0600092343560471,
                                         0.00986711655058283,
                                         0.000510945045904789,
                                         5.12570039966134e-06,
                                         3.03431834113325e-09,
                                         1.39552613215253e-15,
                                         4.18625954489709e-32,
                                         1.12804371054609e-20,
                                         1.22627561847869e-14,
                                         1.19932607988146e-10,
                                         8.89054819233463e-08,
                                         1.21541305283134e-05,
                                         0.000479863462627245,
                                         0.00701924823857761,
                                         0.0438374876976013,
                                         0.125878981114039,
                                         0.169918168449934,
                                         0.104781800722844,
                                         0.0268761884110355,
                                         0.00234348594893473,
                                         4.52661940101666e-05,
                                         6.75003777662785e-08,
                                         1.6579815480412e-13,
                                         9.32712986427041e-35,
                                         1.34228414295848e-22,
                                         3.67562268478373e-16,
                                         6.92169898959782e-12,
                                         8.6400193277196e-09,
                                         1.84255184418647e-06,
                                         0.000108422266644745,
                                         0.00230262831147745,
                                         0.0206294059753418,
                                         0.0849771402824453,
                                         0.166540993306318,
                                         0.153063686340743,
                                         0.0612440122457623,
                                         0.00899226411152444,
                                         0.000334436201388705,
                                         1.25623185929333e-06,
                                         1.64793452035561e-11,
                                         1.71733240850583e-37,
                                         1.31991954625872e-24,
                                         9.10454448503432e-18,
                                         3.30120587386972e-13,
                                         6.93881576991317e-10,
                                         2.30834121681092e-07,
                                         2.02443459553578e-05,
                                         0.000624225560352743,
                                         0.00802254676818848,
                                         0.0474062359611808,
                                         0.134892235587757,
                                         0.184774888402796,
                                         0.115330469932473,
                                         0.028514136448607,
                                         0.002041912147281,
                                         1.93204782541256e-05,
                                         1.35358233816487e-09,
                                         2.567632253195e-40,
                                         1.05395622992424e-26,
                                         1.83129215848062e-19,
                                         1.27851145672458e-14,
                                         4.52509930297082e-11,
                                         2.34829368634746e-08,
                                         3.06945402605702e-06,
                                         0.000137414122899224,
                                         0.0025334358215332,
                                         0.0214753846075098,
                                         0.0887206813913163,
                                         0.181128104925793,
                                         0.176358583981642,
                                         0.0734215430374149,
                                         0.0101235524681974,
                                         0.000241289296067745,
                                         9.0282072718485e-08,
                                         3.03916051586955e-43,
                                         6.66254337784889e-29,
                                         2.91607982737257e-21,
                                         3.9199361966958e-16,
                                         2.33621727415972e-12,
                                         1.89124230469054e-09,
                                         3.68434995665417e-07,
                                         2.39476858111701e-05,
                                         0.000633358955383301,
                                         0.00770173881057221,
                                         0.0461960820590724,
                                         0.140563025743189,
                                         0.213496865818935,
                                         0.149668067338151,
                                         0.0397348138349856,
                                         0.00238561633281575,
                                         4.76717091337457e-06,
                                         2.74078609407807e-46,
                                         3.20891498880655e-31,
                                         3.53786998935301e-23,
                                         9.15701784012503e-18,
                                         9.18965136100882e-14,
                                         1.16049326760788e-10,
                                         3.36946775435194e-08,
                                         3.17977522382423e-06,
                                         0.000120639801025391,
                                         0.00210444369248535,
                                         0.0183267775129292,
                                         0.0831107189043027,
                                         0.196918760080652,
                                         0.232453174643562,
                                         0.118825627134037,
                                         0.0179706518437422,
                                         0.000191787638626777,
                                         1.76951611973563e-49,
                                         1.1064562628662e-33,
                                         3.07285598910721e-25,
                                         1.53139420619327e-19,
                                         2.58787155604009e-15,
                                         5.09795448832855e-12,
                                         2.20607163997971e-09,
                                         3.02264516803829e-07,
                                         1.64508819580078e-05,
                                         0.000411664778679015,
                                         0.00520505051003714,
                                         0.0351804068848499,
                                         0.13002910057349,
                                         0.258463779204353,
                                         0.254394035609468,
                                         0.0969136473748023,
                                         0.00552380550631025,
                                         7.28513308961843e-53,
                                         2.43284184253607e-36,
                                         1.70194738953989e-27,
                                         1.63314040917587e-21,
                                         4.64718524696876e-17,
                                         1.42808108998537e-13,
                                         9.21046389429921e-11,
                                         1.83223637268677e-08,
                                         1.43051147460938e-06,
                                         5.1351566672317e-05,
                                         0.00094268668898748,
                                         0.00949616523694436,
                                         0.0547517002470376,
                                         0.183259916510464,
                                         0.347302028866203,
                                         0.333280291685053,
                                         0.101451793966291,
                                         1.43716640180024e-56,
                                         2.56318587166051e-39,
                                         4.51686024619576e-30,
                                         8.34539021993579e-24,
                                         3.9987462975162e-19,
                                         1.91688642382561e-15,
                                         1.84259545626306e-12,
                                         5.32184757276606e-10,
                                         5.96046447753906e-08,
                                         3.06937734567641e-06,
                                         8.18081176367883e-05,
                                         0.00122823725525557,
                                         0.0110469213132802,
                                         0.0622618294076962,
                                         0.227192669517664,
                                         0.549187856103814,
                                         0.892827754065557,

                                         0.903542599978812,
                                         0.585051828521545,
                                         0.265892290888614,
                                         0.0838015569028992,
                                         0.017987079558517,
                                         0.00256294431599287,
                                         0.000234140096521745,
                                         1.30868995113231e-05,
                                         4.2017698073612e-07,
                                         7.11129913409873e-09,
                                         5.62993952359946e-11,
                                         1.75689571941843e-13,
                                         1.67329171612845e-16,
                                         3.24401414401317e-20,
                                         6.38416321343705e-25,
                                         3.30478707010126e-31,
                                         1.81871791794797e-40,
                                         1.00256853790409e-57,
                                         0.0918420921536171,
                                         0.317148533771997,
                                         0.36211948435127,
                                         0.218880543209794,
                                         0.078676249839157,
                                         0.0173607479830575,
                                         0.0023414773412251,
                                         0.000187694177055183,
                                         8.50808251497768e-06,
                                         2.02288872463769e-07,
                                         2.2610604909934e-09,
                                         1.01193892514984e-11,
                                         1.42286888461945e-14,
                                         4.27191130680053e-18,
                                         1.40789865541488e-22,
                                         1.39771883904143e-28,
                                         1.93249981938677e-37,
                                         5.68124371516582e-54,
                                         0.00447323281301015,
                                         0.0823792196019996,
                                         0.236311381613834,
                                         0.273935821627123,
                                         0.164897186081732,
                                         0.0563487509183761,
                                         0.0112199449070114,
                                         0.00128988629941316,
                                         8.25501238272651e-05,
                                         2.75728487531288e-06,
                                         4.35118182359129e-08,
                                         2.79286033089625e-10,
                                         5.79755266171759e-13,
                                         2.69555572374694e-16,
                                         1.48773440184317e-20,
                                         2.83258371153064e-26,
                                         9.83920880550178e-35,
                                         1.54262145472927e-50,
                                         0.000138932810531741,
                                         0.0136450843517033,
                                         0.098337909379466,
                                         0.218622110323429,
                                         0.220387228888803,
                                         0.116628244875827,
                                         0.0342842833032865,
                                         0.00565269621092359,
                                         0.000510748794938277,
                                         2.39659621558903e-05,
                                         5.33956487658134e-07,
                                         4.91527384717701e-09,
                                         1.50635768653021e-11,
                                         1.08462094504465e-14,
                                         1.00249681792325e-18,
                                         3.66057470073393e-24,
                                         3.19451166320838e-32,
                                         2.67103082525362e-47,
                                         3.08919994192219e-06,
                                         0.00161805266594481,
                                         0.0292964623033606,
                                         0.124910001763515,
                                         0.210871302743726,
                                         0.172814852049836,
                                         0.0749993161080757,
                                         0.017734450627849,
                                         0.0022623242434824,
                                         0.000149130329398279,
                                         4.69096703417534e-06,
                                         6.19304303168025e-08,
                                         2.80200812870143e-10,
                                         3.12439214111466e-13,
                                         4.83613568635363e-17,
                                         3.38667616729412e-22,
                                         7.42517435957225e-30,
                                         3.31097811542665e-44,
                                         5.23344787051951e-08,
                                         0.000146187342030607,
                                         0.00664982287072871,
                                         0.0543752209560451,
                                         0.153726672198232,
                                         0.195100811427781,
                                         0.125002937616688,
                                         0.0423916695374109,
                                         0.00763489443006513,
                                         0.000707029888052273,
                                         3.13992722333457e-05,
                                         5.94512738160669e-07,
                                         3.97110492086881e-09,
                                         6.85730986466184e-12,
                                         1.77752058820864e-15,
                                         2.38725470070632e-20,
                                         1.31495085697316e-27,
                                         3.12704699614905e-41,
                                         7.01894996329881e-10,
                                         1.04560883836766e-05,
                                         0.00119494339642061,
                                         0.0187390346493603,
                                         0.0887203670587076,
                                         0.174373083196005,
                                         0.164939811704086,
                                         0.0802205548814561,
                                         0.0203982815718043,
                                         0.00265370063322797,
                                         0.000166386904718681,
                                         4.5181494289358e-06,
                                         4.45549224276758e-08,
                                         1.19147345663244e-10,
                                         5.17217441103526e-14,
                                         1.33219010033759e-18,
                                         1.84354926552901e-25,
                                         2.33805576642052e-38,
                                         7.64413775327291e-12,
                                         6.07296580688078e-07,
                                         0.000174363952207325,
                                         0.00524403411404014,
                                         0.0415785699822403,
                                         0.126552850626207,
                                         0.176726991435103,
                                         0.1232715670438,
                                         0.0442543781889441,
                                         0.00808794515645302,
                                         0.000715963393252463,
                                         2.78825269535459e-05,
                                         4.0593092325519e-07,
                                         1.68107497549454e-09,
                                         1.22209170826601e-12,
                                         6.03678573231895e-17,
                                         2.09880498833867e-23,
                                         1.41953935528999e-35,
                                         6.87969733650555e-14,
                                         2.91485459651829e-08,
                                         2.10257039192713e-05,
                                         0.00121274158985607,
                                         0.0161027568940182,
                                         0.0759012427891932,
                                         0.15648212791414,
                                         0.156539751944813,
                                         0.0793420418561359,
                                         0.0203708399490693,
                                         0.00254593295083861,
                                         0.00014219624905131,
                                         3.0562793738124e-06,
                                         1.96008210722209e-08,
                                         2.38626627235905e-11,
                                         2.26063157946674e-15,
                                         1.97457619488926e-21,
                                         7.12236913799692e-33,
                                         5.17998734695052e-16,
                                         1.17044630207974e-09,
                                         2.12110869068617e-06,
                                         0.000234633259431257,
                                         0.00521734388769282,
                                         0.0380841599637466,
                                         0.11591646839132,
                                         0.166304846066537,
                                         0.119006050266438,
                                         0.0429238051093656,
                                         0.00757393624610709,
                                         0.000606684206040894,
                                         1.92509649483597e-05,
                                         1.9119652025933e-07,
                                         3.89809615055072e-10,
                                         7.08226801220393e-14,
                                         1.55415469583846e-19,
                                         2.98964800558043e-30,
                                         3.29080274746008e-18,
                                         3.96551756685278e-11,
                                         1.80546505964242e-07,
                                         3.83022830790789e-05,
                                         0.00142630523297817,
                                         0.0161232898026532,
                                         0.0724501559763559,
                                         0.149072987053484,
                                         0.150608157673611,
                                         0.0763134834234529,
                                         0.0190112255375822,
                                         0.0021839919003902,
                                         0.000102311803882953,
                                         1.57361863353792e-06,
                                         5.37279137605609e-09,
                                         1.8720985675135e-12,
                                         1.03211568688302e-17,
                                         1.05883776894754e-27,
                                         1.77385899679004e-20,
                                         1.13996719511422e-12,
                                         1.30394521314146e-08,
                                         5.30522541065669e-06,
                                         0.000330841194058431,
                                         0.00579171297608,
                                         0.0384217905300435,
                                         0.113380167523965,
                                         0.16172309421487,
                                         0.11511937761733,
                                         0.0404895243111649,
                                         0.00667088492923587,
                                         0.000461363328857873,
                                         1.09891243191153e-05,
                                         6.28335357601034e-08,
                                         4.19883835587744e-11,
                                         5.81576171491669e-16,
                                         3.18187329401207e-25,
                                         8.1388509383822e-23,
                                         2.78940506558031e-14,
                                         8.0159750418838e-10,
                                         6.2547474149612e-07,
                                         6.53210924398636e-05,
                                         0.00177087194537371,
                                         0.0173437350670186,
                                         0.073401002365184,
                                         0.147816305622836,
                                         0.147816305622836,
                                         0.0734010023651839,
                                         0.0173437350670186,
                                         0.00177087194537371,
                                         6.53210924398636e-05,
                                         6.25474741496124e-07,
                                         8.0159750418838e-10,
                                         2.78940506558031e-14,
                                         8.1388509383822e-23,
                                         3.18187329401207e-25,
                                         5.81576171491669e-16,
                                         4.19883835587744e-11,
                                         6.2833535760103e-08,
                                         1.09891243191153e-05,
                                         0.000461363328857873,
                                         0.00667088492923587,
                                         0.0404895243111649,
                                         0.11511937761733,
                                         0.16172309421487,
                                         0.113380167523965,
                                         0.0384217905300435,
                                         0.00579171297608,
                                         0.000330841194058431,
                                         5.30522541065673e-06,
                                         1.30394521314146e-08,
                                         1.13996719511422e-12,
                                         1.77385899679004e-20,
                                         1.05883776894754e-27,
                                         1.03211568688302e-17,
                                         1.8720985675135e-12,
                                         5.37279137605605e-09,
                                         1.57361863353792e-06,
                                         0.000102311803882953,
                                         0.0021839919003902,
                                         0.0190112255375822,
                                         0.0763134834234529,
                                         0.150608157673611,
                                         0.149072987053484,
                                         0.0724501559763559,
                                         0.0161232898026532,
                                         0.00142630523297817,
                                         3.83022830790791e-05,
                                         1.80546505964242e-07,
                                         3.96551756685278e-11,
                                         3.29080274746008e-18,
                                         2.98964800558043e-30,
                                         1.55415469583846e-19,
                                         7.08226801220393e-14,
                                         3.89809615055069e-10,
                                         1.9119652025933e-07,
                                         1.92509649483597e-05,
                                         0.000606684206040894,
                                         0.00757393624610709,
                                         0.0429238051093656,
                                         0.119006050266438,
                                         0.166304846066537,
                                         0.11591646839132,
                                         0.0380841599637466,
                                         0.00521734388769282,
                                         0.000234633259431259,
                                         2.12110869068617e-06,
                                         1.17044630207974e-09,
                                         5.17998734695052e-16,
                                         7.12236913799692e-33,
                                         1.97457619488926e-21,
                                         2.26063157946674e-15,
                                         2.38626627235903e-11,
                                         1.96008210722209e-08,
                                         3.0562793738124e-06,
                                         0.00014219624905131,
                                         0.00254593295083861,
                                         0.0203708399490693,
                                         0.0793420418561359,
                                         0.156539751944813,
                                         0.15648212791414,
                                         0.0759012427891932,
                                         0.0161027568940182,
                                         0.00121274158985608,
                                         2.10257039192713e-05,
                                         2.91485459651829e-08,
                                         6.87969733650555e-14,
                                         1.41953935528999e-35,
                                         2.09880498833867e-23,
                                         6.03678573231895e-17,
                                         1.222091708266e-12,
                                         1.68107497549454e-09,
                                         4.0593092325519e-07,
                                         2.78825269535459e-05,
                                         0.000715963393252463,
                                         0.00808794515645302,
                                         0.0442543781889441,
                                         0.1232715670438,
                                         0.176726991435103,
                                         0.126552850626207,
                                         0.0415785699822403,
                                         0.00524403411404016,
                                         0.000174363952207325,
                                         6.07296580688078e-07,
                                         7.64413775327291e-12,
                                         2.33805576642052e-38,
                                         1.84354926552901e-25,
                                         1.33219010033759e-18,
                                         5.17217441103521e-14,
                                         1.19147345663244e-10,
                                         4.45549224276758e-08,
                                         4.5181494289358e-06,
                                         0.000166386904718681,
                                         0.00265370063322797,
                                         0.0203982815718043,
                                         0.0802205548814561,
                                         0.164939811704086,
                                         0.174373083196005,
                                         0.0887203670587076,
                                         0.0187390346493604,
                                         0.00119494339642061,
                                         1.04560883836766e-05,
                                         7.01894996329881e-10,
                                         3.12704699614905e-41,
                                         1.31495085697316e-27,
                                         2.38725470070632e-20,
                                         1.77752058820863e-15,
                                         6.85730986466184e-12,
                                         3.97110492086881e-09,
                                         5.94512738160669e-07,
                                         3.13992722333457e-05,
                                         0.000707029888052273,
                                         0.00763489443006513,
                                         0.0423916695374109,
                                         0.125002937616688,
                                         0.195100811427781,
                                         0.153726672198232,
                                         0.0543752209560453,
                                         0.00664982287072871,
                                         0.000146187342030607,
                                         5.23344787051951e-08,
                                         3.31097811542665e-44,
                                         7.42517435957225e-30,
                                         3.38667616729412e-22,
                                         4.83613568635357e-17,
                                         3.12439214111466e-13,
                                         2.80200812870143e-10,
                                         6.19304303168025e-08,
                                         4.69096703417534e-06,
                                         0.000149130329398279,
                                         0.0022623242434824,
                                         0.017734450627849,
                                         0.0749993161080757,
                                         0.172814852049836,
                                         0.210871302743726,
                                         0.124910001763515,
                                         0.0292964623033606,
                                         0.00161805266594481,
                                         3.08919994192219e-06,
                                         2.67103082525362e-47,
                                         3.19451166320838e-32,
                                         3.66057470073393e-24,
                                         1.00249681792324e-18,
                                         1.08462094504465e-14,
                                         1.50635768653021e-11,
                                         4.91527384717701e-09,
                                         5.33956487658134e-07,
                                         2.39659621558903e-05,
                                         0.000510748794938277,
                                         0.00565269621092359,
                                         0.0342842833032865,
                                         0.116628244875827,
                                         0.220387228888803,
                                         0.218622110323429,
                                         0.0983379093794661,
                                         0.0136450843517033,
                                         0.000138932810531741,
                                         1.54262145472927e-50,
                                         9.83920880550178e-35,
                                         2.83258371153064e-26,
                                         1.48773440184315e-20,
                                         2.69555572374694e-16,
                                         5.79755266171759e-13,
                                         2.79286033089625e-10,
                                         4.35118182359129e-08,
                                         2.75728487531288e-06,
                                         8.25501238272651e-05,
                                         0.00128988629941316,
                                         0.0112199449070114,
                                         0.0563487509183761,
                                         0.164897186081732,
                                         0.273935821627123,
                                         0.236311381613834,
                                         0.0823792196019996,
                                         0.00447323281301015,
                                         5.68124371516582e-54,
                                         1.93249981938677e-37,
                                         1.39771883904143e-28,
                                         1.40789865541486e-22,
                                         4.27191130680053e-18,
                                         1.42286888461945e-14,
                                         1.01193892514984e-11,
                                         2.2610604909934e-09,
                                         2.02288872463769e-07,
                                         8.50808251497768e-06,
                                         0.000187694177055183,
                                         0.0023414773412251,
                                         0.0173607479830575,
                                         0.078676249839157,
                                         0.218880543209794,
                                         0.36211948435127,
                                         0.317148533771997,
                                         0.0918420921536171,
                                         1.00256853790409e-57,
                                         1.81871791794797e-40,
                                         3.30478707010126e-31,
                                         6.38416321343697e-25,
                                         3.24401414401317e-20,
                                         1.67329171612845e-16,
                                         1.75689571941843e-13,
                                         5.62993952359946e-11,
                                         7.11129913409873e-09,
                                         4.2017698073612e-07,
                                         1.30868995113231e-05,
                                         0.000234140096521745,
                                         0.00256294431599287,
                                         0.017987079558517,
                                         0.0838015569028992,
                                         0.265892290888614,
                                         0.585051828521545,
                                         0.903542599978812,

                                         0.912751804934429,
                                         0.617351756775037,
                                         0.303853959266539,
                                         0.107777613448548,
                                         0.0271528953788282,
                                         0.00476114518228507,
                                         0.00056536457590248,
                                         4.38417932968983e-05,
                                         2.11589407810927e-06,
                                         5.96046447753906e-08,
                                         8.98528373593446e-10,
                                         6.42727698453893e-12,
                                         1.83709025900741e-14,
                                         1.62368146571802e-17,
                                         2.95831540909092e-21,
                                         5.53895833182472e-26,
                                         2.76080501191436e-32,
                                         1.48026231952973e-41,
                                         8.04296276361152e-59,
                                         0.0834849610158648,
                                         0.300770921595857,
                                         0.371086304330643,
                                         0.251590334001708,
                                         0.105660315147428,
                                         0.0285182075290751,
                                         0.00496056998287475,
                                         0.000546204572254132,
                                         3.67457184262943e-05,
                                         1.43051147460938e-06,
                                         2.9801728880699e-08,
                                         2.97154614368352e-10,
                                         1.20600744883757e-12,
                                         1.56139403634956e-15,
                                         4.3789622740193e-19,
                                         1.36673775771845e-23,
                                         1.30211155123128e-29,
                                         1.75008043408609e-38,
                                         5.06504132665567e-55,
                                         0.00365889878271698,
                                         0.0702143053153877,
                                         0.217155858954553,
                                         0.281414155571621,
                                         0.197012714904039,
                                         0.0818501772301223,
                                         0.0208555269351902,
                                         0.0032606857916809,
                                         0.00030577794751343,
                                         1.64508819580078e-05,
                                         4.73628384541989e-07,
                                         6.58301037455527e-09,
                                         3.7936397278442e-11,
                                         7.19466866001626e-14,
                                         3.10587898537236e-17,
                                         1.61595359665576e-21,
                                         2.94270852343926e-27,
                                         9.91434285038185e-36,
                                         1.52839823323139e-51,
                                         0.000102257725436919,
                                         0.0104524703910593,
                                         0.0810348368720362,
                                         0.200725021335921,
                                         0.234250321495553,
                                         0.149803033857609,
                                         0.0559131988216318,
                                         0.0124126970686718,
                                         0.00162259138323599,
                                         0.000120639801025391,
                                         4.79995946497307e-06,
                                         9.29972671048645e-08,
                                         7.60966897139715e-10,
                                         2.11403739555427e-12,
                                         1.40475776903888e-15,
                                         1.21836149268895e-19,
                                         4.24082047940837e-25,
                                         3.5815699937085e-33,
                                         2.94099066844479e-48,
                                         2.04597224666672e-06,
                                         0.00111396146047401,
                                         0.0216486017109042,
                                         0.102497899033663,
                                         0.199399476016996,
                                         0.196281546451275,
                                         0.107316225273115,
                                         0.0338283830346845,
                                         0.00616410526735517,
                                         0.000633358955383301,
                                         3.48253343926738e-05,
                                         9.4053229001726e-07,
                                         1.09278147065501e-08,
                                         4.4470540716654e-11,
                                         4.54858413581581e-14,
                                         6.57629551214813e-18,
                                         4.37532589721761e-23,
                                         9.26276892071252e-31,
                                         4.05143283341093e-45,
                                         3.11891856999729e-08,
                                         9.04528121966418e-05,
                                         0.00440644776754882,
                                         0.0398776082021019,
                                         0.129320840265501,
                                         0.195947181525864,
                                         0.156934029372539,
                                         0.0702420228737008,
                                         0.0178415103452525,
                                         0.0025334358215332,
                                         0.000192510183735118,
                                         7.2473280834525e-06,
                                         1.19564311651703e-07,
                                         7.12742806335253e-10,
                                         1.12215205708693e-12,
                                         2.70450111251823e-16,
                                         3.43931230600137e-21,
                                         1.82519346286652e-28,
                                         4.252304226795e-42,
                                         3.76400934772211e-10,
                                         5.81455309747991e-06,
                                         0.000710051357206062,
                                         0.0122824668307605,
                                         0.0663980587193729,
                                         0.154860597400362,
                                         0.181681715546756,
                                         0.115466270924876,
                                         0.0408823210768426,
                                         0.00802254676818848,
                                         0.000842470070367533,
                                         4.42104049849825e-05,
                                         1.03564800422906e-06,
                                         9.04348108519825e-09,
                                         2.19164165804872e-11,
                                         8.80512179614882e-15,
                                         2.14030260169649e-19,
                                         2.84720894037904e-26,
                                         3.53331512201808e-39,
                                         3.68866712364187e-12,
                                         3.03516791325308e-07,
                                         9.29101180241502e-05,
                                         0.00307195049281912,
                                         0.027683077012019,
                                         0.0993836525890291,
                                         0.170795896828379,
                                         0.154129363406742,
                                         0.0760696797385496,
                                         0.0206294059753418,
                                         0.00299383160029776,
                                         0.000218999528038026,
                                         7.28441855275168e-06,
                                         9.31773899473232e-08,
                                         3.47583776996596e-10,
                                         2.32785371605792e-13,
                                         1.08156060412982e-17,
                                         3.60663246200998e-24,
                                         2.3840344711147e-36,
                                         2.98725781466096e-14,
                                         1.30928313353095e-08,
                                         1.00466374651695e-05,
                                         0.000634932114616822,
                                         0.00953801078137039,
                                         0.0527076249705234,
                                         0.132686920058896,
                                         0.170020033916512,
                                         0.116969356386419,
                                         0.0438374876976013,
                                         0.00879194017932359,
                                         0.000896491825466824,
                                         4.23410945152482e-05,
                                         7.93359299333886e-07,
                                         4.5554776348044e-09,
                                         5.08581399498044e-12,
                                         4.51659354272358e-16,
                                         3.77545158312321e-22,
                                         1.32931271384601e-33,
                                         2.02392532975618e-16,
                                         4.72501251281518e-10,
                                         9.08859997743359e-07,
                                         0.00010978902154345,
                                         0.0027492852112122,
                                         0.0233857056517313,
                                         0.0862377332116916,
                                         0.156903760031677,
                                         0.150470426338759,
                                         0.0779333114624023,
                                         0.0216003416137038,
                                         0.0030702096165245,
                                         0.00020589595170126,
                                         5.65129308653279e-06,
                                         4.99490092474426e-08,
                                         9.29573636915736e-11,
                                         1.57793713785085e-14,
                                         3.30639291692991e-20,
                                         6.20097992131367e-31,
                                         1.15699118821419e-18,
                                         1.43875265644263e-11,
                                         6.93724500354241e-08,
                                         1.60178536363118e-05,
                                         0.000668644949723637,
                                         0.00875470056908998,
                                         0.0472912003372126,
                                         0.122174449150598,
                                         0.163321736391846,
                                         0.116899967193604,
                                         0.0447765162055797,
                                         0.00887163176589968,
                                         0.000844787192068152,
                                         3.39656188316535e-05,
                                         4.62097504983432e-07,
                                         1.43357662221962e-09,
                                         4.65138375728611e-13,
                                         2.44317066094331e-18,
                                         2.44065996653892e-28,
                                         5.61189721922814e-21,
                                         3.71717840557778e-13,
                                         4.49284222873495e-09,
                                         1.98286806718026e-06,
                                         0.000137979775175998,
                                         0.00278084130780924,
                                         0.0220042937365995,
                                         0.0807182094573366,
                                         0.15041145663177,
                                         0.148781776428223,
                                         0.0787560766850762,
                                         0.0217511928220367,
                                         0.00294097224197335,
                                         0.000173210941919989,
                                         3.62730822299584e-06,
                                         1.87586730232831e-08,
                                         1.16337235011658e-11,
                                         1.53178297588883e-16,
                                         8.15076405517858e-26,
                                         2.31694711626174e-23,
                                         8.17461726678376e-15,
                                         2.47674935844532e-10,
                                         2.08934458735334e-07,
                                         2.42360618956788e-05,
                                         0.000751861541817328,
                                         0.00871487748014329,
                                         0.0453930542707846,
                                         0.117908360719369,
                                         0.161180257797241,
                                         0.117908360719369,
                                         0.0453930542707847,
                                         0.00871487748014329,
                                         0.000751861541817326,
                                         2.42360618956789e-05,
                                         2.08934458735334e-07,
                                         2.47674935844532e-10,
                                         8.17461726678376e-15,
                                         2.31694711626174e-23,
                                         8.15076405517858e-26,
                                         1.53178297588883e-16,
                                         1.16337235011658e-11,
                                         1.87586730232831e-08,
                                         3.62730822299582e-06,
                                         0.00017321094191999,
                                         0.00294097224197335,
                                         0.0217511928220367,
                                         0.0787560766850762,
                                         0.148781776428223,
                                         0.15041145663177,
                                         0.0807182094573368,
                                         0.0220042937365995,
                                         0.00278084130780923,
                                         0.000137979775175998,
                                         1.98286806718026e-06,
                                         4.49284222873495e-09,
                                         3.71717840557778e-13,
                                         5.61189721922814e-21,
                                         2.44065996653892e-28,
                                         2.44317066094331e-18,
                                         4.65138375728611e-13,
                                         1.43357662221962e-09,
                                         4.62097504983429e-07,
                                         3.39656188316536e-05,
                                         0.000844787192068152,
                                         0.00887163176589966,
                                         0.0447765162055797,
                                         0.116899967193604,
                                         0.163321736391846,
                                         0.122174449150598,
                                         0.0472912003372126,
                                         0.00875470056908996,
                                         0.00066864494972364,
                                         1.60178536363118e-05,
                                         6.93724500354241e-08,
                                         1.43875265644263e-11,
                                         1.15699118821419e-18,
                                         6.20097992131367e-31,
                                         3.30639291692991e-20,
                                         1.57793713785085e-14,
                                         9.29573636915736e-11,
                                         4.99490092474423e-08,
                                         5.65129308653281e-06,
                                         0.00020589595170126,
                                         0.0030702096165245,
                                         0.0216003416137038,
                                         0.0779333114624023,
                                         0.150470426338759,
                                         0.156903760031677,
                                         0.0862377332116916,
                                         0.0233857056517313,
                                         0.00274928521121221,
                                         0.00010978902154345,
                                         9.08859997743359e-07,
                                         4.72501251281518e-10,
                                         2.02392532975618e-16,
                                         1.32931271384601e-33,
                                         3.77545158312321e-22,
                                         4.51659354272358e-16,
                                         5.08581399498044e-12,
                                         4.55547763480437e-09,
                                         7.9335929933389e-07,
                                         4.23410945152482e-05,
                                         0.000896491825466822,
                                         0.00879194017932359,
                                         0.0438374876976013,
                                         0.116969356386419,
                                         0.170020033916512,
                                         0.132686920058896,
                                         0.0527076249705232,
                                         0.00953801078137042,
                                         0.000634932114616822,
                                         1.00466374651695e-05,
                                         1.30928313353095e-08,
                                         2.98725781466096e-14,
                                         2.3840344711147e-36,
                                         3.60663246200998e-24,
                                         1.08156060412982e-17,
                                         2.32785371605792e-13,
                                         3.47583776996593e-10,
                                         9.31773899473236e-08,
                                         7.28441855275168e-06,
                                         0.000218999528038025,
                                         0.00299383160029776,
                                         0.0206294059753418,
                                         0.0760696797385496,
                                         0.154129363406742,
                                         0.170795896828379,
                                         0.0993836525890289,
                                         0.0276830770120191,
                                         0.00307195049281912,
                                         9.29101180241502e-05,
                                         3.03516791325308e-07,
                                         3.68866712364187e-12,
                                         3.53331512201808e-39,
                                         2.84720894037904e-26,
                                         2.14030260169649e-19,
                                         8.80512179614882e-15,
                                         2.19164165804871e-11,
                                         9.04348108519829e-09,
                                         1.03564800422906e-06,
                                         4.42104049849824e-05,
                                         0.000842470070367533,
                                         0.00802254676818848,
                                         0.0408823210768426,
                                         0.115466270924876,
                                         0.181681715546756,
                                         0.154860597400361,
                                         0.0663980587193731,
                                         0.0122824668307605,
                                         0.000710051357206062,
                                         5.81455309747991e-06,
                                         3.76400934772212e-10,
                                         4.252304226795e-42,
                                         1.82519346286652e-28,
                                         3.43931230600138e-21,
                                         2.70450111251823e-16,
                                         1.12215205708693e-12,
                                         7.12742806335256e-10,
                                         1.19564311651703e-07,
                                         7.24732808345248e-06,
                                         0.000192510183735118,
                                         0.0025334358215332,
                                         0.0178415103452525,
                                         0.0702420228737008,
                                         0.156934029372539,
                                         0.195947181525864,
                                         0.129320840265501,
                                         0.0398776082021019,
                                         0.00440644776754882,
                                         9.04528121966418e-05,
                                         3.11891856999729e-08,
                                         4.05143283341093e-45,
                                         9.26276892071252e-31,
                                         4.37532589721761e-23,
                                         6.57629551214813e-18,
                                         4.54858413581577e-14,
                                         4.44705407166543e-11,
                                         1.09278147065501e-08,
                                         9.40532290017256e-07,
                                         3.48253343926738e-05,
                                         0.000633358955383301,
                                         0.00616410526735517,
                                         0.0338283830346845,
                                         0.107316225273115,
                                         0.196281546451274,
                                         0.199399476016996,
                                         0.102497899033663,
                                         0.0216486017109042,
                                         0.00111396146047401,
                                         2.04597224666672e-06,
                                         2.94099066844479e-48,
                                         3.5815699937085e-33,
                                         4.24082047940837e-25,
                                         1.21836149268895e-19,
                                         1.40475776903886e-15,
                                         2.11403739555428e-12,
                                         7.60966897139715e-10,
                                         9.29972671048641e-08,
                                         4.79995946497307e-06,
                                         0.000120639801025391,
                                         0.00162259138323599,
                                         0.0124126970686718,
                                         0.0559131988216318,
                                         0.149803033857608,
                                         0.234250321495553,
                                         0.200725021335921,
                                         0.0810348368720362,
                                         0.0104524703910593,
                                         0.000102257725436919,
                                         1.52839823323139e-51,
                                         9.91434285038185e-36,
                                         2.94270852343926e-27,
                                         1.61595359665576e-21,
                                         3.10587898537233e-17,
                                         7.19466866001631e-14,
                                         3.7936397278442e-11,
                                         6.58301037455525e-09,
                                         4.73628384541989e-07,
                                         1.64508819580078e-05,
                                         0.00030577794751343,
                                         0.0032606857916809,
                                         0.0208555269351902,
                                         0.0818501772301223,
                                         0.197012714904039,
                                         0.281414155571621,
                                         0.217155858954553,
                                         0.0702143053153877,
                                         0.00365889878271698,
                                         5.06504132665567e-55,
                                         1.75008043408609e-38,
                                         1.30211155123128e-29,
                                         1.36673775771845e-23,
                                         4.37896227401926e-19,
                                         1.56139403634957e-15,
                                         1.20600744883757e-12,
                                         2.9715461436835e-10,
                                         2.9801728880699e-08,
                                         1.43051147460938e-06,
                                         3.67457184262943e-05,
                                         0.000546204572254132,
                                         0.00496056998287475,
                                         0.028518207529075,
                                         0.105660315147428,
                                         0.251590334001708,
                                         0.371086304330643,
                                         0.300770921595857,
                                         0.0834849610158648,
                                         8.04296276361152e-59,
                                         1.48026231952973e-41,
                                         2.76080501191436e-32,
                                         5.53895833182472e-26,
                                         2.95831540909089e-21,
                                         1.62368146571803e-17,
                                         1.83709025900741e-14,
                                         6.42727698453891e-12,
                                         8.98528373593446e-10,
                                         5.96046447753906e-08,
                                         2.11589407810927e-06,
                                         4.38417932968983e-05,
                                         0.00056536457590248,
                                         0.00476114518228507,
                                         0.0271528953788282,
                                         0.107777613448548,
                                         0.303853959266539,
                                         0.617351756775037,
                                         0.912751804934429,

                                         0.920720481367421,
                                         0.646437768147427,
                                         0.340617766863958,
                                         0.133630180962194,
                                         0.0385765191074015,
                                         0.00806160075209773,
                                         0.00119319045515091,
                                         0.000121543534227142,
                                         8.20773049319219e-06,
                                         3.49843418493865e-07,
                                         8.81995489279795e-09,
                                         1.20499769705428e-10,
                                         7.90599716406371e-13,
                                         2.09665608975434e-15,
                                         1.73850546955026e-18,
                                         3.00372647737938e-22,
                                         5.38921702941113e-27,
                                         2.60052227034072e-33,
                                         1.36353044895263e-42,
                                         7.31756979717358e-60,
                                         0.0761814100933118,
                                         0.284605781958179,
                                         0.37519893102951,
                                         0.280554137645846,
                                         0.134484841345612,
                                         0.043040004700219,
                                         0.00927089662533695,
                                         0.00132992597588879,
                                         0.00012389376751913,
                                         7.20252283378757e-06,
                                         2.46761789905282e-07,
                                         4.59814146204529e-09,
                                         4.16182226751604e-11,
                                         1.55431026559232e-13,
                                         1.87563151275629e-16,
                                         4.96286622021173e-20,
                                         1.47855006697227e-24,
                                         1.35984298654693e-30,
                                         1.78389926836579e-39,
                                         5.09410806866965e-56,
                                         0.00302034677617538,
                                         0.0600409166605878,
                                         0.198035243205618,
                                         0.282237943058511,
                                         0.224651970030122,
                                         0.11010573307095,
                                         0.0345159876638037,
                                         0.00697283960667177,
                                         0.000896112283476647,
                                         7.10529605213012e-05,
                                         3.30808041499675e-06,
                                         8.40746442232683e-08,
                                         1.04977687934227e-09,
                                         5.52121716488018e-12,
                                         9.69628961544607e-15,
                                         3.92908435307836e-18,
                                         1.94371683300283e-22,
                                         3.40724648936158e-28,
                                         1.11831052631256e-36,
                                         1.69924512706759e-52,
                                         7.63604002215474e-05,
                                         0.00807708205965779,
                                         0.0666541148529405,
                                         0.181057993974954,
                                         0.239304416342132,
                                         0.179618531386717,
                                         0.0819449995943077,
                                         0.0233128552921478,
                                         0.00413312925314728,
                                         0.000446975067082791,
                                         2.82798948427713e-05,
                                         9.80282690120578e-07,
                                         1.6885505366919e-08,
                                         1.25064925683078e-10,
                                         3.19644490082107e-13,
                                         1.98359827876522e-16,
                                         1.62942187340649e-20,
                                         5.44404774670164e-26,
                                         4.47052043047891e-34,
                                         3.61449391578845e-49,
                                         1.38209361490651e-06,
                                         0.000777892444833683,
                                         0.0160608794447359,
                                         0.0831529933452377,
                                         0.182494202811177,
                                         0.209773308017616,
                                         0.139277988859031,
                                         0.0558006345617689,
                                         0.013647509985726,
                                         0.00201299314352918,
                                         0.000173076219216396,
                                         8.18267885773599e-06,
                                         1.94441516765643e-07,
                                         2.02812214237333e-09,
                                         7.54374055554468e-12,
                                         7.1692540461856e-15,
                                         9.77894414457821e-19,
                                         6.22727687463986e-24,
                                         1.27941557243231e-31,
                                         5.50423338607113e-46,
                                         1.90593204986903e-08,
                                         5.70801755018433e-05,
                                         0.00294857627585505,
                                         0.02909636652023,
                                         0.106034726475387,
                                         0.186659458022017,
                                         0.180361236733383,
                                         0.101761498937613,
                                         0.0343343272700654,
                                         0.00690719919926166,
                                         0.000807044899189355,
                                         5.20403690650685e-05,
                                         1.70594342794852e-06,
                                         2.50584019729507e-08,
                                         1.35645987869661e-10,
                                         1.97421708211887e-13,
                                         4.47147778022221e-17,
                                         5.42719109403834e-22,
                                         2.78975374226874e-29,
                                         6.38626403779024e-43,
                                         2.08074914545224e-10,
                                         3.3158387706338e-06,
                                         0.000428546320695385,
                                         0.00806012893713472,
                                         0.0487741295031084,
                                         0.131489820759433,
                                         0.184903989396549,
                                         0.146916377833161,
                                         0.0683826652141964,
                                         0.0187630754750548,
                                         0.00297920476232764,
                                         0.000262015864704133,
                                         1.18490248223968e-05,
                                         2.45106594544983e-07,
                                         1.93094329830751e-09,
                                         4.30386063541884e-12,
                                         1.61864849459924e-15,
                                         3.74450496527115e-20,
                                         4.8157337125167e-27,
                                         5.86596219442078e-40,
                                         1.84460826942658e-12,
                                         1.56413280707146e-07,
                                         5.0577258984647e-05,
                                         0.00181308153190541,
                                         0.0182180985992332,
                                         0.0752153326821465,
                                         0.153929363190059,
                                         0.172238027780003,
                                         0.110595032069549,
                                         0.0413883568222877,
                                         0.00893048663005482,
                                         0.00107124035010452,
                                         6.68301918660015e-05,
                                         1.94683316002558e-06,
                                         2.23205146383664e-08,
                                         7.61892342902139e-11,
                                         4.75801857215196e-14,
                                         2.09790502188802e-18,
                                         6.75042590436717e-25,
                                         4.3752589667306e-37,
                                         1.35136618320382e-14,
                                         6.09731054156855e-09,
                                         4.93284281556215e-06,
                                         0.000337036664751356,
                                         0.00562342670101677,
                                         0.0355553614076336,
                                         0.105896384636398,
                                         0.166867724890264,
                                         0.147811996706138,
                                         0.0754461031441939,
                                         0.0221225082130408,
                                         0.00361935120241129,
                                         0.000311492271635052,
                                         1.27787079014807e-05,
                                         2.13217749888914e-07,
                                         1.11458575235179e-09,
                                         1.15580398477882e-12,
                                         9.71318596405882e-17,
                                         7.81959668941475e-23,
                                         2.696824821017e-34,
                                         8.28248162995682e-17,
                                         1.98848144460887e-10,
                                         4.024925104793e-07,
                                         5.2414989307166e-05,
                                         0.00145217035917125,
                                         0.014061198947132,
                                         0.060947982297304,
                                         0.135249037876846,
                                         0.165273143425134,
                                         0.115057247495717,
                                         0.045847128935301,
                                         0.0102304107180696,
                                         0.00121461987357828,
                                         7.0171973395713e-05,
                                         1.70396647846717e-06,
                                         1.36411788922445e-08,
                                         2.34887973353609e-11,
                                         3.76232339824937e-15,
                                         7.57802567476784e-21,
                                         1.39065771240039e-31,
                                         4.28313439319395e-19,
                                         5.47165311513762e-12,
                                         2.77097186293302e-08,
                                         6.87776987340372e-06,
                                         0.000316408311452725,
                                         0.00469195020463056,
                                         0.0295972470783022,
                                         0.0924932010113341,
                                         0.155922459515858,
                                         0.148048814211003,
                                         0.0801684910495627,
                                         0.0243988410784434,
                                         0.00399620055155958,
                                         0.000325127887982,
                                         1.14898014403435e-05,
                                         1.40865326865685e-07,
                                         4.02764477239503e-10,
                                         1.22960134406975e-13,
                                         6.196430407577e-19,
                                         6.05064323765725e-29,
                                         1.8793471658068e-21,
                                         1.27749633302646e-13,
                                         1.61864093629547e-09,
                                         7.65744473034427e-07,
                                         5.849547739009e-05,
                                         0.00132839896071473,
                                         0.0121951569711041,
                                         0.0536697443680881,
                                         0.124812809061846,
                                         0.161636700954032,
                                         0.118943130817608,
                                         0.0493729888346693,
                                         0.0111557368058944,
                                         0.00127817066868631,
                                         6.5736729238586e-05,
                                         1.23424222982712e-06,
                                         5.85983828114852e-09,
                                         3.40970358652749e-12,
                                         4.29903689249146e-17,
                                         2.23371075767651e-26,
                                         7.0190620362138e-24,
                                         2.53879435146967e-15,
                                         8.04814290681217e-11,
                                         7.25682845697179e-08,
                                         9.2049927756015e-06,
                                         0.000320132978808804,
                                         0.00427710810428199,
                                         0.0265079412499944,
                                         0.0850425735445514,
                                         0.150211014077762,
                                         0.150211014077762,
                                         0.0850425735445514,
                                         0.0265079412499944,
                                         0.00427710810428199,
                                         0.000320132978808803,
                                         9.2049927756015e-06,
                                         7.25682845697185e-08,
                                         8.04814290681217e-11,
                                         2.53879435146977e-15,
                                         7.01906203621516e-24,
                                         2.23371075767605e-26,
                                         4.29903689249129e-17,
                                         3.40970358652749e-12,
                                         5.85983828114847e-09,
                                         1.23424222982712e-06,
                                         6.57367292385863e-05,
                                         0.00127817066868631,
                                         0.0111557368058944,
                                         0.0493729888346693,
                                         0.118943130817608,
                                         0.161636700954032,
                                         0.124812809061846,
                                         0.0536697443680881,
                                         0.0121951569711041,
                                         0.00132839896071473,
                                         5.849547739009e-05,
                                         7.65744473034433e-07,
                                         1.61864093629547e-09,
                                         1.2774963330265e-13,
                                         1.87934716580713e-21,
                                         6.05064323765588e-29,
                                         6.19643040757673e-19,
                                         1.22960134406975e-13,
                                         4.02764477239499e-10,
                                         1.40865326865685e-07,
                                         1.14898014403436e-05,
                                         0.000325127887982,
                                         0.00399620055155958,
                                         0.0243988410784434,
                                         0.0801684910495627,
                                         0.148048814211003,
                                         0.155922459515858,
                                         0.0924932010113341,
                                         0.0295972470783022,
                                         0.00469195020463055,
                                         0.000316408311452725,
                                         6.87776987340376e-06,
                                         2.77097186293302e-08,
                                         5.47165311513779e-12,
                                         4.28313439319464e-19,
                                         1.39065771240006e-31,
                                         7.57802567476749e-21,
                                         3.76232339824937e-15,
                                         2.34887973353607e-11,
                                         1.36411788922445e-08,
                                         1.70396647846717e-06,
                                         7.0171973395713e-05,
                                         0.00121461987357828,
                                         0.0102304107180696,
                                         0.045847128935301,
                                         0.115057247495717,
                                         0.165273143425134,
                                         0.135249037876846,
                                         0.060947982297304,
                                         0.014061198947132,
                                         0.00145217035917125,
                                         5.24149893071664e-05,
                                         4.024925104793e-07,
                                         1.98848144460892e-10,
                                         8.28248162995803e-17,
                                         2.6968248210163e-34,
                                         7.81959668941436e-23,
                                         9.71318596405882e-17,
                                         1.15580398477881e-12,
                                         1.11458575235179e-09,
                                         2.13217749888915e-07,
                                         1.27787079014807e-05,
                                         0.000311492271635052,
                                         0.00361935120241129,
                                         0.0221225082130408,
                                         0.0754461031441939,
                                         0.147811996706138,
                                         0.166867724890264,
                                         0.105896384636398,
                                         0.0355553614076336,
                                         0.00562342670101677,
                                         0.000337036664751358,
                                         4.93284281556215e-06,
                                         6.09731054156869e-09,
                                         1.351366183204e-14,
                                         4.3752589667294e-37,
                                         6.75042590436681e-25,
                                         2.09790502188802e-18,
                                         4.7580185721519e-14,
                                         7.61892342902139e-11,
                                         2.23205146383665e-08,
                                         1.94683316002558e-06,
                                         6.68301918660015e-05,
                                         0.00107124035010452,
                                         0.00893048663005482,
                                         0.0413883568222877,
                                         0.110595032069549,
                                         0.172238027780003,
                                         0.153929363190059,
                                         0.0752153326821464,
                                         0.0182180985992332,
                                         0.00181308153190542,
                                         5.0577258984647e-05,
                                         1.56413280707149e-07,
                                         1.84460826942679e-12,
                                         5.86596219441907e-40,
                                         4.81573371251644e-27,
                                         3.74450496527115e-20,
                                         1.61864849459922e-15,
                                         4.30386063541885e-12,
                                         1.93094329830752e-09,
                                         2.45106594544983e-07,
                                         1.18490248223968e-05,
                                         0.000262015864704133,
                                         0.00297920476232764,
                                         0.0187630754750548,
                                         0.0683826652141964,
                                         0.146916377833161,
                                         0.184903989396549,
                                         0.131489820759433,
                                         0.0487741295031084,
                                         0.00806012893713475,
                                         0.000428546320695385,
                                         3.31583877063386e-06,
                                         2.08074914545244e-10,
                                         6.38626403778828e-43,
                                         2.78975374226858e-29,
                                         5.42719109403834e-22,
                                         4.47147778022215e-17,
                                         1.97421708211887e-13,
                                         1.35645987869662e-10,
                                         2.50584019729507e-08,
                                         1.70594342794852e-06,
                                         5.20403690650685e-05,
                                         0.000807044899189355,
                                         0.00690719919926166,
                                         0.0343343272700654,
                                         0.101761498937613,
                                         0.180361236733383,
                                         0.186659458022017,
                                         0.106034726475387,
                                         0.0290963665202301,
                                         0.00294857627585505,
                                         5.70801755018441e-05,
                                         1.90593204986918e-08,
                                         5.50423338606935e-46,
                                         1.27941557243223e-31,
                                         6.22727687463986e-24,
                                         9.77894414457807e-19,
                                         7.1692540461856e-15,
                                         7.54374055554472e-12,
                                         2.02812214237333e-09,
                                         1.94441516765643e-07,
                                         8.18267885773599e-06,
                                         0.000173076219216396,
                                         0.00201299314352918,
                                         0.013647509985726,
                                         0.0558006345617689,
                                         0.139277988859031,
                                         0.209773308017616,
                                         0.182494202811177,
                                         0.0831529933452379,
                                         0.0160608794447359,
                                         0.000777892444833692,
                                         1.3820936149066e-06,
                                         3.61449391578723e-49,
                                         4.47052043047862e-34,
                                         5.44404774670164e-26,
                                         1.62942187340647e-20,
                                         1.98359827876522e-16,
                                         3.19644490082109e-13,
                                         1.25064925683078e-10,
                                         1.6885505366919e-08,
                                         9.80282690120578e-07,
                                         2.82798948427713e-05,
                                         0.000446975067082791,
                                         0.00413312925314728,
                                         0.0233128552921478,
                                         0.0819449995943077,
                                         0.179618531386716,
                                         0.239304416342132,
                                         0.181057993974955,
                                         0.0666541148529405,
                                         0.00807708205965787,
                                         7.63604002215511e-05,
                                         1.69924512706699e-52,
                                         1.11831052631249e-36,
                                         3.40724648936158e-28,
                                         1.9437168330028e-22,
                                         3.92908435307836e-18,
                                         9.69628961544613e-15,
                                         5.52121716488018e-12,
                                         1.04977687934227e-09,
                                         8.40746442232683e-08,
                                         3.30808041499675e-06,
                                         7.10529605213012e-05,
                                         0.000896112283476647,
                                         0.00697283960667177,
                                         0.0345159876638037,
                                         0.11010573307095,
                                         0.224651970030122,
                                         0.282237943058512,
                                         0.198035243205618,
                                         0.0600409166605882,
                                         0.00302034677617547,
                                         5.09410806866776e-56,
                                         1.78389926836567e-39,
                                         1.35984298654693e-30,
                                         1.47855006697225e-24,
                                         4.96286622021173e-20,
                                         1.87563151275631e-16,
                                         1.55431026559232e-13,
                                         4.16182226751604e-11,
                                         4.59814146204529e-09,
                                         2.46761789905282e-07,
                                         7.20252283378757e-06,
                                         0.00012389376751913,
                                         0.00132992597588879,
                                         0.00927089662533695,
                                         0.043040004700219,
                                         0.134484841345612,
                                         0.280554137645846,
                                         0.37519893102951,
                                         0.28460578195818,
                                         0.0761814100933131,
                                         7.31756979717074e-60,
                                         1.36353044895253e-42,
                                         2.60052227034072e-33,
                                         5.38921702941104e-27,
                                         3.00372647737938e-22,
                                         1.73850546955027e-18,
                                         2.09665608975434e-15,
                                         7.90599716406371e-13,
                                         1.20499769705428e-10,
                                         8.81995489279795e-09,
                                         3.49843418493865e-07,
                                         8.20773049319219e-06,
                                         0.000121543534227142,
                                         0.00119319045515091,
                                         0.00806160075209773,
                                         0.0385765191074015,
                                         0.133630180962194,
                                         0.340617766863958,
                                         0.646437768147427,
                                         0.920720481367421,

                                         0.927658784310894,
                                         0.67264618416588,
                                         0.375879666121822,
                                         0.160817713062989,
                                         0.0521732388961514,
                                         0.0126649170459189,
                                         0.00226006608380946,
                                         0.000289750183245367,
                                         2.59079661068667e-05,
                                         1.55498123794522e-06,
                                         5.96046447753906e-08,
                                         1.36651750075232e-09,
                                         1.71588385147473e-11,
                                         1.04526641222202e-13,
                                         2.59906265113864e-16,
                                         2.03984026995106e-19,
                                         3.36671791206839e-23,
                                         5.82236993951653e-28,
                                         2.73205644825174e-34,
                                         1.40514194336461e-43,
                                         7.46070671434787e-61,
                                         0.0697681955734951,
                                         0.26894397594729,
                                         0.37539352346994,
                                         0.305370469401901,
                                         0.163957893111268,
                                         0.0606883038945227,
                                         0.015674536114548,
                                         0.00281050975812876,
                                         0.000343707098989104,
                                         2.78354649664074e-05,
                                         1.43051147460938e-06,
                                         4.39707987157189e-08,
                                         7.44998139977091e-10,
                                         6.20709082958412e-12,
                                         2.15856769441458e-14,
                                         2.45197475316417e-17,
                                         6.17084563379823e-21,
                                         1.76615625861095e-25,
                                         1.57570244343864e-31,
                                         2.02426582390566e-40,
                                         5.71390771507384e-57,
                                         0.00251427813728987,
                                         0.0515256593325977,
                                         0.179643421251792,
                                         0.277847689778802,
                                         0.246889976344345,
                                         0.139345917435949,
                                         0.0520900595914887,
                                         0.0130627036537039,
                                         0.00218489355132848,
                                         0.000238758260292027,
                                         1.64508819580078e-05,
                                         6.77953816126551e-07,
                                         1.54991930812157e-08,
                                         1.76618342357673e-10,
                                         8.59015887697103e-13,
                                         1.41228525760246e-15,
                                         5.41962495761537e-19,
                                         2.5671133895143e-23,
                                         4.35457071269578e-29,
                                         1.39733805511009e-37,
                                         2.09687757140976e-53,
                                         5.77793607810468e-05,
                                         0.00629490065235509,
                                         0.0548200522408779,
                                         0.161209313772042,
                                         0.237070858808109,
                                         0.204026737107012,
                                         0.110387169494406,
                                         0.0387154790096359,
                                         0.00885677576211766,
                                         0.00130593606933506,
                                         0.000120639801025391,
                                         6.66560310508417e-06,
                                         2.05620577397088e-07,
                                         3.20469788691267e-09,
                                         2.17991874204622e-11,
                                         5.18719342828921e-14,
                                         3.0352702929087e-17,
                                         2.37938415361258e-21,
                                         7.67396955603717e-27,
                                         6.15090495747358e-35,
                                         4.90700513843863e-50,
                                         9.50582958101983e-07,
                                         0.000550569412755175,
                                         0.0119763767633115,
                                         0.0669624516491219,
                                         0.162971163147077,
                                         0.213864054672779,
                                         0.167471233066562,
                                         0.082147448137405,
                                         0.0257027086250047,
                                         0.00511379489939726,
                                         0.000633358955383301,
                                         4.6917695118206e-05,
                                         1.95290868790324e-06,
                                         4.16290270684739e-08,
                                         3.96038394803345e-10,
                                         1.36395592759675e-12,
                                         1.21697984566409e-15,
                                         1.57885394002897e-19,
                                         9.68172346379574e-25,
                                         1.9383599298481e-32,
                                         8.22087090504669e-47,
                                         1.19153829064704e-08,
                                         3.6689007878379e-05,
                                         0.00199348119305019,
                                         0.0211920636756239,
                                         0.085357963561464,
                                         0.170800523143823,
                                         0.193580909695133,
                                         0.132801874365018,
                                         0.0568306763334409,
                                         0.0152568677672774,
                                         0.0025334358215332,
                                         0.000251613868478613,
                                         1.4131816466701e-05,
                                         4.12008442919253e-07,
                                         5.48194868379302e-09,
                                         2.7325550503162e-11,
                                         3.71766359363271e-14,
                                         7.98215273328563e-18,
                                         9.30649104289327e-23,
                                         4.65404435823313e-30,
                                         1.0493487075569e-43,
                                         1.18241067378559e-10,
                                         1.93554028568573e-06,
                                         0.000262688581189384,
                                         0.00530954754222707,
                                         0.0353931881389837,
                                         0.107989854479041,
                                         0.177144304709366,
                                         0.169963901527149,
                                         0.0994784741903813,
                                         0.03603543906158,
                                         0.00802254676818848,
                                         0.00106825476895773,
                                         8.09573714289537e-05,
                                         3.22818436519576e-06,
                                         6.00724022337306e-08,
                                         4.33390973536114e-10,
                                         8.99081683811436e-13,
                                         3.19477642453286e-16,
                                         7.08209279254897e-21,
                                         8.84644888320355e-28,
                                         1.06038651159069e-40,
                                         9.52797906777243e-13,
                                         8.29164264574358e-08,
                                         2.81088036247946e-05,
                                         0.00108022417413764,
                                         0.0119170136984316,
                                         0.0554432691256003,
                                         0.131632756617069,
                                         0.176636842723325,
                                         0.141399385072213,
                                         0.0691140540388115,
                                         0.0206294059753418,
                                         0.00368287704724032,
                                         0.000376605730128331,
                                         2.05391590689759e-05,
                                         5.3454853946837e-07,
                                         5.58165408313332e-09,
                                         1.76563254986449e-11,
                                         1.03832431190872e-14,
                                         4.37632308270133e-19,
                                         1.36546175650699e-25,
                                         8.70123047155191e-38,
                                         6.34479678083687e-15,
                                         2.93537375817924e-09,
                                         2.48558143975257e-06,
                                         0.000181616264793928,
                                         0.00331588653929935,
                                         0.023523350143078,
                                         0.0808323542320704,
                                         0.151701670885204,
                                         0.166092642788716,
                                         0.109543695467805,
                                         0.0438374876976013,
                                         0.0104926230105544,
                                         0.00144777781589673,
                                         0.000107991968155662,
                                         3.93082549837849e-06,
                                         5.94060181563616e-08,
                                         2.86540472131077e-10,
                                         2.78875295993141e-13,
                                         2.23481516255394e-17,
                                         1.74170492133508e-23,
                                         5.90040223284896e-35,
                                         3.53470502673259e-17,
                                         8.6937025514803e-11,
                                         1.83879016421372e-07,
                                         2.55454834314174e-05,
                                         0.000771881129642357,
                                         0.00834964584639826,
                                         0.0415264702905701,
                                         0.108997860660733,
                                         0.163219411989365,
                                         0.145253613835626,
                                         0.0779333114624023,
                                         0.0250091802915891,
                                         0.00465624099990975,
                                         0.000475027544542834,
                                         2.41823732803101e-05,
                                         5.28952297742288e-07,
                                         3.8903632724327e-09,
                                         6.26621950889166e-12,
                                         9.54755966313658e-16,
                                         1.85860951324548e-21,
                                         3.34734984143563e-32,
                                         1.66150795267147e-19,
                                         2.17250073751405e-12,
                                         1.14775748211521e-08,
                                         3.0317074989646e-06,
                                         0.000151605505075613,
                                         0.00250063740400268,
                                         0.0180002523748693,
                                         0.0660783773220976,
                                         0.135334028060542,
                                         0.162510074900823,
                                         0.116899967193604,
                                         0.0502954422483236,
                                         0.0126352188743591,
                                         0.00176303086529855,
                                         0.000125524315742935,
                                         3.97389472964527e-06,
                                         4.45664587897405e-08,
                                         1.18799592331233e-10,
                                         3.4415726246495e-14,
                                         1.67346070241126e-19,
                                         1.60226510841524e-29,
                                         6.62667639064011e-22,
                                         4.60637382280084e-14,
                                         6.07872063478988e-10,
                                         3.05284364236663e-07,
                                         2.52652506393472e-05,
                                         0.000635444291117798,
                                         0.00662027853425966,
                                         0.0339895063064843,
                                         0.0952108185892807,
                                         0.154268666747335,
                                         0.148781776428223,
                                         0.0858226452954371,
                                         0.0290920402424599,
                                         0.00555194467274012,
                                         0.000552841830202399,
                                         2.53314645798346e-05,
                                         4.33181786979532e-07,
                                         1.91103388528368e-09,
                                         1.05260538520555e-12,
                                         1.27845949337147e-17,
                                         6.50746345971267e-27,
                                         2.24965466573229e-24,
                                         8.31352452374028e-16,
                                         2.74031818247401e-11,
                                         2.61666774462424e-08,
                                         3.58392576987672e-06,
                                         0.000137445649345113,
                                         0.00207252932951491,
                                         0.0148818560486005,
                                         0.05701542233125,
                                         0.124652765385578,
                                         0.161180257797241,
                                         0.124652765385578,
                                         0.05701542233125,
                                         0.0148818560486005,
                                         0.00207252932951491,
                                         0.000137445649345113,
                                         3.58392576987672e-06,
                                         2.61666774462424e-08,
                                         2.74031818247397e-11,
                                         8.31352452374062e-16,
                                         2.24965466573181e-24,
                                         6.50746345971417e-27,
                                         1.27845949337142e-17,
                                         1.05260538520557e-12,
                                         1.91103388528368e-09,
                                         4.33181786979532e-07,
                                         2.53314645798346e-05,
                                         0.000552841830202399,
                                         0.0055519446727401,
                                         0.0290920402424598,
                                         0.0858226452954371,
                                         0.148781776428223,
                                         0.154268666747335,
                                         0.0952108185892807,
                                         0.0339895063064844,
                                         0.00662027853425966,
                                         0.000635444291117798,
                                         2.52652506393472e-05,
                                         3.05284364236663e-07,
                                         6.07872063478978e-10,
                                         4.60637382280101e-14,
                                         6.62667639063881e-22,
                                         1.60226510841564e-29,
                                         1.67346070241118e-19,
                                         3.44157262464957e-14,
                                         1.18799592331233e-10,
                                         4.45664587897405e-08,
                                         3.97389472964527e-06,
                                         0.000125524315742935,
                                         0.00176303086529855,
                                         0.0126352188743591,
                                         0.0502954422483236,
                                         0.116899967193604,
                                         0.162510074900823,
                                         0.135334028060542,
                                         0.0660783773220977,
                                         0.0180002523748693,
                                         0.00250063740400268,
                                         0.000151605505075613,
                                         3.0317074989646e-06,
                                         1.14775748211519e-08,
                                         2.17250073751412e-12,
                                         1.66150795267117e-19,
                                         3.34734984143652e-32,
                                         1.85860951324538e-21,
                                         9.54755966313678e-16,
                                         6.26621950889166e-12,
                                         3.8903632724327e-09,
                                         5.28952297742288e-07,
                                         2.41823732803101e-05,
                                         0.000475027544542832,
                                         0.00465624099990975,
                                         0.0250091802915891,
                                         0.0779333114624023,
                                         0.145253613835626,
                                         0.163219411989365,
                                         0.108997860660733,
                                         0.0415264702905701,
                                         0.00834964584639826,
                                         0.000771881129642357,
                                         2.55454834314174e-05,
                                         1.8387901642137e-07,
                                         8.69370255148057e-11,
                                         3.53470502673203e-17,
                                         5.90040223285064e-35,
                                         1.74170492133499e-23,
                                         2.23481516255399e-17,
                                         2.78875295993141e-13,
                                         2.86540472131077e-10,
                                         5.94060181563616e-08,
                                         3.93082549837849e-06,
                                         0.000107991968155661,
                                         0.00144777781589673,
                                         0.0104926230105544,
                                         0.0438374876976013,
                                         0.109543695467805,
                                         0.166092642788716,
                                         0.151701670885204,
                                         0.0808323542320704,
                                         0.023523350143078,
                                         0.00331588653929935,
                                         0.000181616264793928,
                                         2.48558143975254e-06,
                                         2.93537375817932e-09,
                                         6.34479678083597e-15,
                                         8.70123047155453e-38,
                                         1.36546175650691e-25,
                                         4.37632308270143e-19,
                                         1.03832431190872e-14,
                                         1.76563254986449e-11,
                                         5.58165408313332e-09,
                                         5.3454853946837e-07,
                                         2.05391590689759e-05,
                                         0.000376605730128331,
                                         0.00368287704724032,
                                         0.0206294059753418,
                                         0.0691140540388115,
                                         0.141399385072213,
                                         0.176636842723325,
                                         0.131632756617069,
                                         0.0554432691256003,
                                         0.0119170136984316,
                                         0.00108022417413764,
                                         2.81088036247943e-05,
                                         8.29164264574378e-08,
                                         9.52797906777124e-13,
                                         1.06038651159102e-40,
                                         8.84644888320301e-28,
                                         7.08209279254915e-21,
                                         3.19477642453286e-16,
                                         8.99081683811436e-13,
                                         4.33390973536114e-10,
                                         6.00724022337306e-08,
                                         3.22818436519575e-06,
                                         8.09573714289537e-05,
                                         0.00106825476895773,
                                         0.00802254676818848,
                                         0.03603543906158,
                                         0.0994784741903813,
                                         0.169963901527149,
                                         0.177144304709366,
                                         0.107989854479041,
                                         0.0353931881389837,
                                         0.00530954754222707,
                                         0.000262688581189381,
                                         1.93554028568577e-06,
                                         1.18241067378546e-10,
                                         1.04934870755726e-43,
                                         4.65404435823283e-30,
                                         9.30649104289352e-23,
                                         7.98215273328563e-18,
                                         3.71766359363271e-14,
                                         2.7325550503162e-11,
                                         5.48194868379302e-09,
                                         4.12008442919252e-07,
                                         1.4131816466701e-05,
                                         0.000251613868478613,
                                         0.0025334358215332,
                                         0.0152568677672774,
                                         0.0568306763334409,
                                         0.132801874365018,
                                         0.193580909695133,
                                         0.170800523143822,
                                         0.085357963561464,
                                         0.0211920636756239,
                                         0.00199348119305017,
                                         3.66890078783797e-05,
                                         1.19153829064693e-08,
                                         8.22087090504961e-47,
                                         1.93835992984797e-32,
                                         9.68172346379601e-25,
                                         1.57885394002897e-19,
                                         1.21697984566409e-15,
                                         1.36395592759675e-12,
                                         3.96038394803345e-10,
                                         4.16290270684737e-08,
                                         1.95290868790324e-06,
                                         4.69176951182061e-05,
                                         0.000633358955383301,
                                         0.00511379489939726,
                                         0.0257027086250047,
                                         0.0821474481374051,
                                         0.167471233066562,
                                         0.213864054672779,
                                         0.162971163147077,
                                         0.0669624516491219,
                                         0.0119763767633114,
                                         0.000550569412755183,
                                         9.50582958101915e-07,
                                         4.90700513844046e-50,
                                         6.15090495747314e-35,
                                         7.67396955603739e-27,
                                         2.37938415361258e-21,
                                         3.0352702929087e-17,
                                         5.18719342828921e-14,
                                         2.17991874204622e-11,
                                         3.20469788691265e-09,
                                         2.05620577397088e-07,
                                         6.66560310508417e-06,
                                         0.000120639801025391,
                                         0.00130593606933506,
                                         0.00885677576211766,
                                         0.0387154790096359,
                                         0.110387169494406,
                                         0.204026737107012,
                                         0.237070858808109,
                                         0.161209313772042,
                                         0.0548200522408777,
                                         0.00629490065235516,
                                         5.77793607810437e-05,
                                         2.09687757141058e-53,
                                         1.39733805510998e-37,
                                         4.35457071269591e-29,
                                         2.5671133895143e-23,
                                         5.41962495761537e-19,
                                         1.41228525760246e-15,
                                         8.59015887697103e-13,
                                         1.76618342357673e-10,
                                         1.54991930812157e-08,
                                         6.77953816126551e-07,
                                         1.64508819580078e-05,
                                         0.000238758260292027,
                                         0.00218489355132848,
                                         0.0130627036537039,
                                         0.0520900595914887,
                                         0.139345917435949,
                                         0.246889976344345,
                                         0.277847689778802,
                                         0.179643421251791,
                                         0.0515256593325981,
                                         0.00251427813728978,
                                         5.71390771507618e-57,
                                         2.0242658239055e-40,
                                         1.57570244343869e-31,
                                         1.76615625861095e-25,
                                         6.17084563379824e-21,
                                         2.45197475316417e-17,
                                         2.15856769441458e-14,
                                         6.20709082958409e-12,
                                         7.44998139977091e-10,
                                         4.39707987157189e-08,
                                         1.43051147460938e-06,
                                         2.78354649664074e-05,
                                         0.000343707098989104,
                                         0.00281050975812876,
                                         0.015674536114548,
                                         0.0606883038945227,
                                         0.163957893111268,
                                         0.305370469401901,
                                         0.375393523469939,
                                         0.268943975947291,
                                         0.0697681955734939,
                                         7.46070671435105e-61,
                                         1.4051419433645e-43,
                                         2.73205644825183e-34,
                                         5.82236993951653e-28,
                                         3.36671791206839e-23,
                                         2.03984026995106e-19,
                                         2.59906265113864e-16,
                                         1.04526641222201e-13,
                                         1.71588385147473e-11,
                                         1.36651750075232e-09,
                                         5.96046447753906e-08,
                                         1.55498123794522e-06,
                                         2.59079661068667e-05,
                                         0.000289750183245367,
                                         0.00226006608380946,
                                         0.0126649170459189,
                                         0.0521732388961514,
                                         0.160817713062989,
                                         0.375879666121822,
                                         0.67264618416588,
                                         0.927658784310894,

                                         0.933734805153789,
                                         0.696289129270269,
                                         0.409456184937541,
                                         0.188848046843311,
                                         0.0677692180947997,
                                         0.0187154172667003,
                                         0.00392041035299767,
                                         0.000611403178220925,
                                         6.93173585382612e-05,
                                         5.54219966232111e-06,
                                         3.00573488812269e-07,
                                         1.05145957458732e-08,
                                         2.22085981762462e-10,
                                         2.59249622212723e-12,
                                         1.48102189424699e-14,
                                         3.48262447354977e-17,
                                         2.60605633896449e-20,
                                         4.13374503980435e-24,
                                         6.92410348156021e-29,
                                         3.17102059650032e-35,
                                         1.60379883778881e-44,
                                         8.43670711663257e-62,
                                         0.064111019325163,
                                         0.253960321513284,
                                         0.372500329980729,
                                         0.325962419733569,
                                         0.193033054888692,
                                         0.0809844744043669,
                                         0.0244380603216475,
                                         0.00529926138415275,
                                         0.00081581136625302,
                                         8.72325095592114e-05,
                                         6.27319315867931e-06,
                                         2.90186413933567e-07,
                                         8.12732132762667e-09,
                                         1.26879911952032e-10,
                                         9.84229352449733e-13,
                                         3.21805843529131e-15,
                                         3.46899784506511e-18,
                                         8.35924321479058e-22,
                                         2.31063313908574e-26,
                                         2.00771618543342e-32,
                                         2.53277216389337e-41,
                                         7.07760500980779e-58,
                                         0.002109251735012,
                                         0.0443842330964898,
                                         0.162379981140367,
                                         0.269593344982033,
                                         0.263461700891418,
                                         0.167915435512668,
                                         0.0729942307572049,
                                         0.0220084567015853,
                                         0.00460070193361267,
                                         0.000657901936188144,
                                         6.27354831007759e-05,
                                         3.83749806764149e-06,
                                         1.42514913507156e-07,
                                         2.97546059847097e-09,
                                         3.13413533161499e-11,
                                         1.42484559146677e-13,
                                         2.21264050548839e-16,
                                         8.09984735581212e-20,
                                         3.69474988429231e-24,
                                         6.09105016083652e-30,
                                         1.91658883206458e-38,
                                         2.84502619425003e-54,
                                         4.42514732229885e-05,
                                         0.00494646747737863,
                                         0.0451379568670426,
                                         0.142185198037457,
                                         0.229301494095535,
                                         0.222015378568338,
                                         0.139031729613184,
                                         0.0582864256470414,
                                         0.0165448193251419,
                                         0.00316408067382524,
                                         0.00040007500462265,
                                         3.23610682765864e-05,
                                         1.59359068394444e-06,
                                         4.44958107558183e-08,
                                         6.36418453122455e-10,
                                         4.02295584389727e-12,
                                         8.99955991586525e-15,
                                         5.00484109967885e-18,
                                         3.76740818047828e-22,
                                         1.17838069131297e-27,
                                         9.24837364958412e-36,
                                         7.29272442165176e-51,
                                         6.64637635705984e-07,
                                         0.000394656693548518,
                                         0.00898274735514119,
                                         0.0536855596074961,
                                         0.142874312990631,
                                         0.21015192549507,
                                         0.189582040070946,
                                         0.110510395880107,
                                         0.0425949230100775,
                                         0.0108941113248,
                                         0.00182653277064658,
                                         0.000195368889079419,
                                         1.27570755693947e-05,
                                         4.76367278754659e-07,
                                         9.25178771634518e-09,
                                         8.13168610394247e-11,
                                         2.62053258422565e-13,
                                         2.21391850190636e-16,
                                         2.75016142406606e-20,
                                         1.63206306552931e-25,
                                         3.19491758261761e-33,
                                         1.3382930216359e-47,
                                         7.60576344799865e-09,
                                         2.39907859555394e-05,
                                         0.00136200016713796,
                                         0.0154440525128736,
                                         0.0678269226969803,
                                         0.151559923286396,
                                         0.19696141109362,
                                         0.1596391967133,
                                         0.0835515159768455,
                                         0.0285783241366698,
                                         0.00635351714162667,
                                         0.000898646013699461,
                                         7.78083421643584e-05,
                                         3.88566547492888e-06,
                                         1.02472950922518e-07,
                                         1.25232379865045e-09,
                                         5.81377933116992e-12,
                                         7.46162903813891e-15,
                                         1.52958770311214e-18,
                                         1.72222124948647e-23,
                                         8.40919917606786e-31,
                                         1.87117028472077e-44,
                                         6.89037815410407e-11,
                                         1.15454758774003e-06,
                                         0.000163488607924891,
                                         0.00351728381859161,
                                         0.0254913262510993,
                                         0.0865321987804849,
                                         0.161997182386049,
                                         0.182565323968634,
                                         0.129745764719397,
                                         0.0593504567746971,
                                         0.017496182604269,
                                         0.00327238397003753,
                                         0.000375702041686691,
                                         2.5091767447796e-05,
                                         8.98535386986732e-07,
                                         1.52684524544735e-08,
                                         1.0211037761067e-10,
                                         1.99089353944577e-13,
                                         6.73492773607851e-17,
                                         1.43874328454119e-21,
                                         1.75223349550386e-28,
                                         2.07117964758508e-41,
                                         5.06891974371394e-13,
                                         4.51181532404718e-08,
                                         1.59356559750278e-05,
                                         0.000650467605850808,
                                         0.00777955693562822,
                                         0.0401183641659995,
                                         0.108194675665049,
                                         0.169538823478056,
                                         0.163607858653959,
                                         0.100088340435742,
                                         0.039124113843345,
                                         0.00967635906413627,
                                         0.00147310282646956,
                                         0.000131573738137749,
                                         6.39783805590893e-06,
                                         1.51163007098742e-07,
                                         1.45630833589736e-09,
                                         4.31354700780605e-12,
                                         2.40803952728953e-15,
                                         9.75999938173902e-20,
                                         2.96484116168779e-26,
                                         1.86163417203978e-38,
                                         3.08157145252473e-15,
                                         1.4570527032995e-09,
                                         1.28362115013633e-06,
                                         9.94096171497331e-05,
                                         0.00196201247825764,
                                         0.0153706828818398,
                                         0.0597157356857732,
                                         0.130108151645922,
                                         0.170490270652021,
                                         0.139484959322023,
                                         0.0722986510825967,
                                         0.0236452638068487,
                                         0.00477317150982559,
                                         0.000570153300453204,
                                         3.76457340213453e-05,
                                         1.23674594142221e-06,
                                         1.71641087648399e-08,
                                         7.72334619748926e-11,
                                         7.11506303484699e-14,
                                         5.47142864851595e-18,
                                         4.14567596970856e-24,
                                         1.382787305504e-35,
                                         1.56728366836484e-17,
                                         3.93656789161866e-11,
                                         8.65012377818234e-08,
                                         1.27101205097061e-05,
                                         0.000413968414017518,
                                         0.00492676273329022,
                                         0.027573393815298,
                                         0.0835330594885733,
                                         0.148632430196318,
                                         0.162625936602014,
                                         0.111772358196127,
                                         0.0483386915741714,
                                         0.0129389655184916,
                                         0.00206696256233163,
                                         0.000185317671596047,
                                         8.46513724503778e-06,
                                         1.69241832537158e-07,
                                         1.15689783540235e-09,
                                         1.75878369736311e-12,
                                         2.5660802187655e-16,
                                         4.8496211800598e-22,
                                         8.59280516072917e-33,
                                         6.72568866689625e-20,
                                         8.97375103771632e-13,
                                         4.91837417573535e-09,
                                         1.37114922079607e-06,
                                         7.36964268510574e-05,
                                         0.0013324285159689,
                                         0.0107425020702163,
                                         0.0452507784773056,
                                         0.109330502222826,
                                         0.159980123344086,
                                         0.145798286206166,
                                         0.0833795225059831,
                                         0.0295941490716022,
                                         0.00632247854592804,
                                         0.000769718077012318,
                                         4.88878906629682e-05,
                                         1.40801722028281e-06,
                                         1.46217145222912e-08,
                                         3.66825819619975e-11,
                                         1.01543821451797e-14,
                                         4.78667634340677e-20,
                                         4.50534788137173e-30,
                                         2.4488940900086e-22,
                                         1.73569886203317e-14,
                                         2.37282058048107e-10,
                                         1.25505820006243e-07,
                                         1.11319116136161e-05,
                                         0.000305752686111131,
                                         0.00355111518893181,
                                         0.0207987777096636,
                                         0.0682359431849114,
                                         0.133532301777512,
                                         0.161366922237025,
                                         0.122030392477398,
                                         0.0574323054595463,
                                         0.0164091547046724,
                                         0.00271263042755,
                                         0.000239559107603221,
                                         9.93922003027642e-06,
                                         1.56799864750445e-07,
                                         6.4915955407914e-10,
                                         3.40942286557413e-13,
                                         4.00870780850838e-18,
                                         2.0043141317686e-27,
                                         7.58979423659613e-25,
                                         2.85760039104454e-16,
                                         9.74394875453505e-12,
                                         9.77844428637151e-09,
                                         1.43126424735143e-06,
                                         5.97204950707885e-05,
                                         0.000999196328361471,
                                         0.0081372250503187,
                                         0.0362503171476295,
                                         0.0948709826628598,
                                         0.15202092409512,
                                         0.15202092409512,
                                         0.09487098266286,
                                         0.0362503171476295,
                                         0.0081372250503187,
                                         0.000999196328361471,
                                         5.97204950707885e-05,
                                         1.43126424735142e-06,
                                         9.77844428637142e-09,
                                         9.74394875453522e-12,
                                         2.85760039104454e-16,
                                         7.58979423659613e-25,
                                         2.0043141317686e-27,
                                         4.00870780850838e-18,
                                         3.40942286557407e-13,
                                         6.49159554079147e-10,
                                         1.56799864750446e-07,
                                         9.93922003027642e-06,
                                         0.000239559107603221,
                                         0.00271263042755,
                                         0.0164091547046724,
                                         0.0574323054595462,
                                         0.122030392477398,
                                         0.161366922237025,
                                         0.133532301777512,
                                         0.0682359431849114,
                                         0.0207987777096636,
                                         0.00355111518893181,
                                         0.000305752686111131,
                                         1.11319116136161e-05,
                                         1.25505820006241e-07,
                                         2.37282058048111e-10,
                                         1.73569886203317e-14,
                                         2.4488940900086e-22,
                                         4.50534788137173e-30,
                                         4.78667634340677e-20,
                                         1.01543821451795e-14,
                                         3.6682581961998e-11,
                                         1.46217145222913e-08,
                                         1.40801722028281e-06,
                                         4.88878906629682e-05,
                                         0.000769718077012318,
                                         0.00632247854592804,
                                         0.0295941490716022,
                                         0.0833795225059831,
                                         0.145798286206166,
                                         0.159980123344086,
                                         0.109330502222826,
                                         0.0452507784773056,
                                         0.0107425020702163,
                                         0.0013324285159689,
                                         7.3696426851057e-05,
                                         1.37114922079606e-06,
                                         4.91837417573543e-09,
                                         8.97375103771632e-13,
                                         6.72568866689625e-20,
                                         8.59280516072917e-33,
                                         4.8496211800598e-22,
                                         2.56608021876544e-16,
                                         1.75878369736314e-12,
                                         1.15689783540236e-09,
                                         1.69241832537158e-07,
                                         8.46513724503778e-06,
                                         0.000185317671596047,
                                         0.00206696256233163,
                                         0.0129389655184916,
                                         0.0483386915741714,
                                         0.111772358196127,
                                         0.162625936602014,
                                         0.148632430196318,
                                         0.0835330594885733,
                                         0.027573393815298,
                                         0.00492676273329022,
                                         0.000413968414017516,
                                         1.2710120509706e-05,
                                         8.65012377818246e-08,
                                         3.93656789161866e-11,
                                         1.56728366836484e-17,
                                         1.382787305504e-35,
                                         4.14567596970856e-24,
                                         5.47142864851581e-18,
                                         7.11506303484708e-14,
                                         7.72334619748933e-11,
                                         1.71641087648399e-08,
                                         1.23674594142221e-06,
                                         3.76457340213453e-05,
                                         0.000570153300453204,
                                         0.00477317150982558,
                                         0.0236452638068487,
                                         0.0722986510825967,
                                         0.139484959322023,
                                         0.170490270652021,
                                         0.130108151645922,
                                         0.0597157356857732,
                                         0.0153706828818398,
                                         0.00196201247825763,
                                         9.94096171497324e-05,
                                         1.28362115013635e-06,
                                         1.4570527032995e-09,
                                         3.08157145252473e-15,
                                         1.86163417203978e-38,
                                         2.96484116168779e-26,
                                         9.75999938173877e-20,
                                         2.40803952728957e-15,
                                         4.31354700780609e-12,
                                         1.45630833589736e-09,
                                         1.51163007098742e-07,
                                         6.39783805590893e-06,
                                         0.000131573738137749,
                                         0.00147310282646955,
                                         0.00967635906413627,
                                         0.039124113843345,
                                         0.100088340435742,
                                         0.163607858653959,
                                         0.169538823478056,
                                         0.108194675665049,
                                         0.0401183641659995,
                                         0.00777955693562819,
                                         0.000650467605850804,
                                         1.59356559750279e-05,
                                         4.51181532404718e-08,
                                         5.06891974371394e-13,
                                         2.07117964758508e-41,
                                         1.75223349550386e-28,
                                         1.43874328454115e-21,
                                         6.73492773607861e-17,
                                         1.99089353944578e-13,
                                         1.0211037761067e-10,
                                         1.52684524544735e-08,
                                         8.98535386986732e-07,
                                         2.5091767447796e-05,
                                         0.00037570204168669,
                                         0.00327238397003753,
                                         0.017496182604269,
                                         0.0593504567746972,
                                         0.129745764719397,
                                         0.182565323968634,
                                         0.161997182386049,
                                         0.0865321987804849,
                                         0.0254913262510992,
                                         0.00351728381859159,
                                         0.000163488607924893,
                                         1.15454758774003e-06,
                                         6.89037815410407e-11,
                                         1.87117028472077e-44,
                                         8.40919917606786e-31,
                                         1.72222124948642e-23,
                                         1.52958770311217e-18,
                                         7.46162903813899e-15,
                                         5.81377933116992e-12,
                                         1.25232379865045e-09,
                                         1.02472950922518e-07,
                                         3.88566547492888e-06,
                                         7.78083421643582e-05,
                                         0.000898646013699461,
                                         0.00635351714162667,
                                         0.0285783241366698,
                                         0.0835515159768455,
                                         0.1596391967133,
                                         0.19696141109362,
                                         0.151559923286396,
                                         0.0678269226969801,
                                         0.0154440525128735,
                                         0.00136200016713797,
                                         2.39907859555393e-05,
                                         7.60576344799865e-09,
                                         1.3382930216359e-47,
                                         3.19491758261761e-33,
                                         1.63206306552926e-25,
                                         2.75016142406611e-20,
                                         2.21391850190639e-16,
                                         2.62053258422565e-13,
                                         8.13168610394247e-11,
                                         9.25178771634518e-09,
                                         4.76367278754659e-07,
                                         1.27570755693947e-05,
                                         0.000195368889079419,
                                         0.00182653277064658,
                                         0.0108941113248,
                                         0.0425949230100775,
                                         0.110510395880107,
                                         0.189582040070946,
                                         0.21015192549507,
                                         0.142874312990631,
                                         0.053685559607496,
                                         0.00898274735514125,
                                         0.000394656693548518,
                                         6.64637635705983e-07,
                                         7.29272442165176e-51,
                                         9.24837364958412e-36,
                                         1.17838069131293e-27,
                                         3.76740818047835e-22,
                                         5.00484109967891e-18,
                                         8.99955991586525e-15,
                                         4.02295584389727e-12,
                                         6.36418453122455e-10,
                                         4.44958107558183e-08,
                                         1.59359068394444e-06,
                                         3.23610682765864e-05,
                                         0.00040007500462265,
                                         0.00316408067382524,
                                         0.0165448193251419,
                                         0.0582864256470414,
                                         0.139031729613184,
                                         0.222015378568338,
                                         0.229301494095534,
                                         0.142185198037457,
                                         0.0451379568670428,
                                         0.00494646747737863,
                                         4.42514732229885e-05,
                                         2.84502619425003e-54,
                                         1.91658883206458e-38,
                                         6.09105016083632e-30,
                                         3.69474988429238e-24,
                                         8.09984735581221e-20,
                                         2.21264050548839e-16,
                                         1.42484559146677e-13,
                                         3.13413533161499e-11,
                                         2.97546059847097e-09,
                                         1.42514913507155e-07,
                                         3.83749806764149e-06,
                                         6.27354831007759e-05,
                                         0.000657901936188144,
                                         0.00460070193361267,
                                         0.0220084567015853,
                                         0.0729942307572049,
                                         0.167915435512668,
                                         0.263461700891418,
                                         0.269593344982032,
                                         0.162379981140367,
                                         0.0443842330964898,
                                         0.002109251735012,
                                         7.07760500980779e-58,
                                         2.53277216389337e-41,
                                         2.00771618543335e-32,
                                         2.31063313908579e-26,
                                         8.35924321479068e-22,
                                         3.46899784506511e-18,
                                         3.21805843529131e-15,
                                         9.84229352449733e-13,
                                         1.26879911952032e-10,
                                         8.12732132762665e-09,
                                         2.90186413933567e-07,
                                         6.27319315867931e-06,
                                         8.72325095592114e-05,
                                         0.00081581136625302,
                                         0.00529926138415275,
                                         0.0244380603216475,
                                         0.0809844744043669,
                                         0.193033054888692,
                                         0.325962419733569,
                                         0.37250032998073,
                                         0.253960321513284,
                                         0.064111019325163,
                                         8.43670711663257e-62,
                                         1.60379883778881e-44,
                                         3.17102059650021e-35,
                                         6.92410348156035e-29,
                                         4.1337450398044e-24,
                                         2.60605633896449e-20,
                                         3.48262447354977e-17,
                                         1.48102189424699e-14,
                                         2.59249622212723e-12,
                                         2.22085981762461e-10,
                                         1.05145957458732e-08,
                                         3.00573488812269e-07,
                                         5.54219966232111e-06,
                                         6.93173585382612e-05,
                                         0.000611403178220925,
                                         0.00392041035299767,
                                         0.0187154172667003,
                                         0.0677692180947997,
                                         0.188848046843311,
                                         0.409456184937541,
                                         0.696289129270269,
                                         0.933734805153789,

                                         0.939084135149102,
                                         0.717650204344063,
                                         0.441254040097398,
                                         0.217293827741998,
                                         0.0851311702066222,
                                         0.0262934536885507,
                                         0.00632601320983101,
                                         0.00116757590823748,
                                         0.000162140845166317,
                                         1.65321241221407e-05,
                                         1.19994222452384e-06,
                                         5.96046447753906e-08,
                                         1.92597411661438e-09,
                                         3.78813970689716e-11,
                                         4.15002811607254e-13,
                                         2.24173865803748e-15,
                                         5.02090148533657e-18,
                                         3.60400344673755e-21,
                                         5.52171752609279e-25,
                                         8.99429106183111e-30,
                                         4.03248135742881e-36,
                                         2.0098092672511e-45,
                                         1.04868694490396e-62,
                                         0.0590989777181576,
                                         0.239750288922511,
                                         0.367228819215384,
                                         0.342476108329228,
                                         0.220867887581412,
                                         0.103299434008008,
                                         0.0356582604759508,
                                         0.00910543141663374,
                                         0.00170651567889241,
                                         0.000230963820274381,
                                         2.20259001216812e-05,
                                         1.43051147460938e-06,
                                         6.04365409896796e-08,
                                         1.56182788945784e-09,
                                         2.27120153027278e-11,
                                         1.6557397004461e-13,
                                         5.13067050657798e-16,
                                         5.28393387737122e-19,
                                         1.22589336595658e-22,
                                         3.28705167405354e-27,
                                         2.79091615948211e-33,
                                         3.46522150935833e-42,
                                         9.59826817612394e-59,
                                         0.00178214087893966,
                                         0.0383788594557529,
                                         0.146443943868708,
                                         0.258642427656634,
                                         0.2745763644954,
                                         0.194462044489715,
                                         0.0963112036163604,
                                         0.0340253443062369,
                                         0.00860626534093492,
                                         0.00154612944178327,
                                         0.000193728534797169,
                                         1.64508819580078e-05,
                                         9.08731018583793e-07,
                                         3.0855100026891e-08,
                                         5.95589323319494e-10,
                                         5.8598397826815e-12,
                                         2.51219385208506e-14,
                                         3.71206993347923e-17,
                                         1.30412074249999e-20,
                                         5.75615823445886e-25,
                                         9.25566891146841e-31,
                                         2.86281811875292e-39,
                                         4.20946106671581e-55,
                                         3.42694931772791e-05,
                                         0.00391767653117131,
                                         0.0372399991283348,
                                         0.124558348650721,
                                         0.217669364439996,
                                         0.233440034444689,
                                         0.165881156044351,
                                         0.0810789499148,
                                         0.0276772391561169,
                                         0.00660011240106674,
                                         0.00108656849333838,
                                         0.000120639801025391,
                                         8.71313996921924e-06,
                                         3.88708761137465e-07,
                                         9.95959598327356e-09,
                                         1.32246140795149e-10,
                                         7.84396699558849e-13,
                                         1.6629473925634e-15,
                                         8.84680584231797e-19,
                                         6.4278021758737e-23,
                                         1.95736769927236e-28,
                                         1.50820403603895e-36,
                                         1.1772364487375e-51,
                                         4.71770964297872e-07,
                                         0.000286301064657532,
                                         0.00677962676070001,
                                         0.0429441284417267,
                                         0.123534808177689,
                                         0.200619760281014,
                                         0.204538550380074,
                                         0.138315731979751,
                                         0.0637218943889958,
                                         0.0201704083476713,
                                         0.00436293255269897,
                                         0.000633358955383301,
                                         5.98097388499943e-05,
                                         3.50573897823301e-06,
                                         1.19232435779719e-07,
                                         2.13667343275285e-09,
                                         1.75338079012427e-11,
                                         5.33333358964342e-14,
                                         4.29648269454213e-17,
                                         5.13866324698707e-21,
                                         2.96343155461723e-26,
                                         5.68832389648217e-34,
                                         2.35699564513138e-48,
                                         4.94829317621735e-09,
                                         1.59410909277183e-05,
                                         0.000940378294516895,
                                         0.0112806839607794,
                                         0.0534173167884295,
                                         0.131362910918995,
                                         0.19215601424637,
                                         0.179777663347572,
                                         0.111777728135071,
                                         0.04696547697132,
                                         0.0133475199443474,
                                         0.0025334358215332,
                                         0.000312802207673131,
                                         2.40899286237808e-05,
                                         1.08754640943383e-06,
                                         2.63023153688699e-08,
                                         2.98618987478765e-10,
                                         1.30322581771987e-12,
                                         1.58979206720167e-15,
                                         3.12995793892833e-19,
                                         3.41836216141202e-24,
                                         1.63459145431306e-31,
                                         3.59546071746562e-45,
                                         4.10886617487659e-11,
                                         7.02676572463633e-07,
                                         0.000103262293487456,
                                         0.00234589965067207,
                                         0.0182859341847239,
                                         0.0680948361546141,
                                         0.142914123259882,
                                         0.184987440132456,
                                         0.155225918232898,
                                         0.0865735334351962,
                                         0.0323269712491154,
                                         0.00802254676818848,
                                         0.00129512018522123,
                                         0.000131049029288585,
                                         7.85314363558673e-06,
                                         2.56325710955185e-07,
                                         4.02625198777159e-09,
                                         2.52105871536299e-11,
                                         4.65703973613228e-14,
                                         1.50927787006739e-17,
                                         3.1216456520788e-22,
                                         3.71857452737141e-29,
                                         4.34202855786067e-42,
                                         2.77051614242706e-13,
                                         2.51515656206099e-08,
                                         9.20774008222936e-06,
                                         0.000396146244814677,
                                         0.00508304962888047,
                                         0.028663400337047,
                                         0.0863114537182541,
                                         0.154568455878269,
                                         0.175043227053962,
                                         0.129587664786432,
                                         0.0635772252867091,
                                         0.0206294059753418,
                                         0.00435434112565989,
                                         0.000578900891049405,
                                         4.60480662774299e-05,
                                         2.02844144313694e-06,
                                         4.40815239754114e-08,
                                         3.96021131672943e-10,
                                         1.10777529633982e-12,
                                         5.90978962811619e-16,
                                         2.31484144155179e-20,
                                         6.86935357127393e-27,
                                         4.25797313855106e-39,
                                         1.54377442733108e-15,
                                         7.43976200718079e-10,
                                         6.78498365160141e-07,
                                         5.52823028335773e-05,
                                         0.00116765896207805,
                                         0.00997070136829154,
                                         0.0430770726489191,
                                         0.106729374314502,
                                         0.163121367743412,
                                         0.160297464737106,
                                         0.103329087063287,
                                         0.0438374876976013,
                                         0.0120981592188078,
                                         0.00211328999523705,
                                         0.000223132958178557,
                                         1.32653052719153e-05,
                                         3.98838172755691e-07,
                                         5.14088903440325e-09,
                                         2.17759884659576e-11,
                                         1.91231445606859e-14,
                                         1.4185458037594e-18,
                                         1.04867211560364e-24,
                                         3.45062391958889e-36,
                                         7.19656975051235e-18,
                                         1.8410755091582e-11,
                                         4.18276146711531e-08,
                                         6.45409341050001e-06,
                                         0.00022440174140485,
                                         0.00290163143297802,
                                         0.0179863265009322,
                                         0.0616546160172744,
                                         0.127173005772288,
                                         0.165885406844102,
                                         0.140495415640692,
                                         0.0779333114624023,
                                         0.0281212538677794,
                                         0.00645405281171531,
                                         0.00090455418595347,
                                         7.25755876979127e-05,
                                         3.01894620411814e-06,
                                         5.58311582807525e-08,
                                         3.5811508756667e-10,
                                         5.17684466070583e-13,
                                         7.27250480536951e-17,
                                         1.33931307533485e-22,
                                         2.33943443670781e-33,
                                         2.83061641799251e-20,
                                         3.84412892396057e-13,
                                         2.17566075186716e-09,
                                         6.35767251476485e-07,
                                         3.63873320968868e-05,
                                         0.000712479825484547,
                                         0.00633654536931553,
                                         0.0300511445178559,
                                         0.0836551734001968,
                                         0.144844996207914,
                                         0.161181614060833,
                                         0.116899967193604,
                                         0.0551523284178669,
                                         0.0166310518838205,
                                         0.00309399236868366,
                                         0.000335025342393838,
                                         1.92809228278656e-05,
                                         5.11597992712092e-07,
                                         4.96913899550277e-09,
                                         1.1824553341189e-11,
                                         3.14585322629966e-15,
                                         1.44323908990966e-20,
                                         1.3382520406291e-30,
                                         9.44671025478045e-23,
                                         6.81033350038202e-15,
                                         9.60203639020664e-11,
                                         5.31379935631316e-08,
                                         5.00631644585321e-06,
                                         0.000148438646959271,
                                         0.00189411662584229,
                                         0.0124279795005921,
                                         0.0466911706604592,
                                         0.10731066907499,
                                         0.156896385593775,
                                         0.148781776428223,
                                         0.0917776883633938,
                                         0.0363622721998306,
                                         0.00897941368417838,
                                         0.00131222663782577,
                                         0.000104482691107891,
                                         3.97763672938685e-06,
                                         5.8503757843762e-08,
                                         2.2916508347827e-10,
                                         1.15461444630245e-13,
                                         1.31958858186766e-18,
                                         6.49544697150669e-28,
                                         2.68353290304152e-25,
                                         1.02698847670924e-16,
                                         3.607133979063e-12,
                                         3.78041002286323e-09,
                                         5.86290877106801e-07,
                                         2.63237742936084e-05,
                                         0.000481934080867956,
                                         0.00437488654821154,
                                         0.0221821418275433,
                                         0.0676720081659083,
                                         0.129998138379305,
                                         0.161180257797241,
                                         0.129998138379305,
                                         0.0676720081659083,
                                         0.0221821418275433,
                                         0.00437488654821155,
                                         0.000481934080867954,
                                         2.63237742936085e-05,
                                         5.86290877106801e-07,
                                         3.78041002286323e-09,
                                         3.60713397906307e-12,
                                         1.02698847670924e-16,
                                         2.68353290304084e-25,
                                         6.49544697150848e-28,
                                         1.31958858186766e-18,
                                         1.15461444630243e-13,
                                         2.2916508347827e-10,
                                         5.8503757843762e-08,
                                         3.97763672938684e-06,
                                         0.000104482691107891,
                                         0.00131222663782577,
                                         0.00897941368417838,
                                         0.0363622721998306,
                                         0.0917776883633938,
                                         0.148781776428223,
                                         0.156896385593775,
                                         0.10731066907499,
                                         0.0466911706604591,
                                         0.0124279795005922,
                                         0.00189411662584229,
                                         0.000148438646959272,
                                         5.00631644585321e-06,
                                         5.31379935631316e-08,
                                         9.60203639020682e-11,
                                         6.81033350038202e-15,
                                         9.44671025477824e-23,
                                         1.33825204062949e-30,
                                         1.44323908990966e-20,
                                         3.14585322629959e-15,
                                         1.1824553341189e-11,
                                         4.96913899550277e-09,
                                         5.11597992712089e-07,
                                         1.92809228278657e-05,
                                         0.000335025342393837,
                                         0.00309399236868367,
                                         0.0166310518838205,
                                         0.0551523284178669,
                                         0.116899967193604,
                                         0.161181614060833,
                                         0.144844996207914,
                                         0.0836551734001968,
                                         0.030051144517856,
                                         0.00633654536931551,
                                         0.00071247982548455,
                                         3.63873320968868e-05,
                                         6.35767251476485e-07,
                                         2.17566075186719e-09,
                                         3.84412892396057e-13,
                                         2.83061641799191e-20,
                                         2.33943443670856e-33,
                                         1.33931307533485e-22,
                                         7.27250480536933e-17,
                                         5.17684466070583e-13,
                                         3.5811508756667e-10,
                                         5.58311582807521e-08,
                                         3.01894620411815e-06,
                                         7.25755876979124e-05,
                                         0.00090455418595347,
                                         0.00645405281171531,
                                         0.0281212538677794,
                                         0.0779333114624023,
                                         0.140495415640692,
                                         0.165885406844102,
                                         0.127173005772288,
                                         0.0616546160172746,
                                         0.0179863265009321,
                                         0.00290163143297803,
                                         0.00022440174140485,
                                         6.45409341050001e-06,
                                         4.18276146711537e-08,
                                         1.8410755091582e-11,
                                         7.19656975051097e-18,
                                         3.45062391959006e-36,
                                         1.04867211560364e-24,
                                         1.41854580375936e-18,
                                         1.91231445606859e-14,
                                         2.17759884659576e-11,
                                         5.14088903440321e-09,
                                         3.98838172755693e-07,
                                         1.32653052719152e-05,
                                         0.000223132958178557,
                                         0.00211328999523705,
                                         0.0120981592188078,
                                         0.0438374876976013,
                                         0.103329087063287,
                                         0.160297464737106,
                                         0.163121367743412,
                                         0.106729374314502,
                                         0.043077072648919,
                                         0.00997070136829157,
                                         0.00116765896207805,
                                         5.52823028335773e-05,
                                         6.7849836516015e-07,
                                         7.43976200718079e-10,
                                         1.54377442733082e-15,
                                         4.25797313855259e-39,
                                         6.86935357127393e-27,
                                         2.31484144155172e-20,
                                         5.90978962811619e-16,
                                         1.10777529633982e-12,
                                         3.96021131672941e-10,
                                         4.40815239754116e-08,
                                         2.02844144313693e-06,
                                         4.60480662774299e-05,
                                         0.000578900891049405,
                                         0.00435434112565989,
                                         0.0206294059753418,
                                         0.0635772252867091,
                                         0.129587664786432,
                                         0.175043227053962,
                                         0.154568455878269,
                                         0.086311453718254,
                                         0.0286634003370471,
                                         0.00508304962888047,
                                         0.000396146244814677,
                                         9.20774008222947e-06,
                                         2.51515656206099e-08,
                                         2.77051614242665e-13,
                                         4.34202855786233e-42,
                                         3.71857452737141e-29,
                                         3.12164565207871e-22,
                                         1.50927787006739e-17,
                                         4.65703973613228e-14,
                                         2.52105871536297e-11,
                                         4.02625198777161e-09,
                                         2.56325710955184e-07,
                                         7.85314363558673e-06,
                                         0.000131049029288585,
                                         0.00129512018522123,
                                         0.00802254676818848,
                                         0.0323269712491154,
                                         0.0865735334351962,
                                         0.155225918232898,
                                         0.184987440132457,
                                         0.142914123259882,
                                         0.0680948361546143,
                                         0.0182859341847239,
                                         0.00234589965067207,
                                         0.000103262293487457,
                                         7.02676572463633e-07,
                                         4.10886617487607e-11,
                                         3.59546071746707e-45,
                                         1.63459145431305e-31,
                                         3.41836216141191e-24,
                                         3.12995793892833e-19,
                                         1.58979206720167e-15,
                                         1.30322581771986e-12,
                                         2.98618987478767e-10,
                                         2.63023153688698e-08,
                                         1.08754640943383e-06,
                                         2.40899286237808e-05,
                                         0.000312802207673131,
                                         0.0025334358215332,
                                         0.0133475199443474,
                                         0.04696547697132,
                                         0.111777728135071,
                                         0.179777663347572,
                                         0.19215601424637,
                                         0.131362910918996,
                                         0.0534173167884295,
                                         0.0112806839607794,
                                         0.000940378294516903,
                                         1.59410909277183e-05,
                                         4.94829317621683e-09,
                                         2.35699564513238e-48,
                                         5.68832389648217e-34,
                                         2.96343155461713e-26,
                                         5.13866324698707e-21,
                                         4.29648269454213e-17,
                                         5.33333358964337e-14,
                                         1.75338079012428e-11,
                                         2.13667343275284e-09,
                                         1.19232435779719e-07,
                                         3.50573897823301e-06,
                                         5.98097388499943e-05,
                                         0.000633358955383301,
                                         0.00436293255269897,
                                         0.0201704083476713,
                                         0.0637218943889958,
                                         0.138315731979751,
                                         0.204538550380074,
                                         0.200619760281014,
                                         0.123534808177689,
                                         0.0429441284417267,
                                         0.00677962676070006,
                                         0.000286301064657532,
                                         4.71770964297832e-07,
                                         1.17723644873803e-51,
                                         1.50820403603895e-36,
                                         1.95736769927229e-28,
                                         6.4278021758737e-23,
                                         8.84680584231797e-19,
                                         1.66294739256339e-15,
                                         7.84396699558854e-13,
                                         1.32246140795149e-10,
                                         9.95959598327356e-09,
                                         3.88708761137465e-07,
                                         8.71313996921924e-06,
                                         0.000120639801025391,
                                         0.00108656849333838,
                                         0.00660011240106674,
                                         0.0276772391561169,
                                         0.0810789499148,
                                         0.165881156044351,
                                         0.233440034444689,
                                         0.217669364439996,
                                         0.124558348650721,
                                         0.037239999128335,
                                         0.00391767653117131,
                                         3.4269493177277e-05,
                                         4.20946106671778e-55,
                                         2.86281811875292e-39,
                                         9.25566891146807e-31,
                                         5.75615823445886e-25,
                                         1.30412074249999e-20,
                                         3.7120699334792e-17,
                                         2.51219385208508e-14,
                                         5.85983978268147e-12,
                                         5.95589323319494e-10,
                                         3.0855100026891e-08,
                                         9.08731018583793e-07,
                                         1.64508819580078e-05,
                                         0.000193728534797169,
                                         0.00154612944178327,
                                         0.00860626534093492,
                                         0.0340253443062369,
                                         0.0963112036163603,
                                         0.194462044489716,
                                         0.2745763644954,
                                         0.258642427656634,
                                         0.146443943868709,
                                         0.0383788594557529,
                                         0.00178214087893958,
                                         9.59826817612863e-59,
                                         3.46522150935833e-42,
                                         2.79091615948201e-33,
                                         3.28705167405354e-27,
                                         1.22589336595658e-22,
                                         5.28393387737117e-19,
                                         5.13067050657801e-16,
                                         1.65573970044609e-13,
                                         2.27120153027278e-11,
                                         1.56182788945784e-09,
                                         6.04365409896796e-08,
                                         1.43051147460938e-06,
                                         2.20259001216812e-05,
                                         0.000230963820274381,
                                         0.00170651567889241,
                                         0.00910543141663374,
                                         0.0356582604759508,
                                         0.103299434008008,
                                         0.220867887581412,
                                         0.342476108329228,
                                         0.367228819215385,
                                         0.239750288922511,
                                         0.0590989777181563,
                                         1.04868694490449e-62,
                                         2.0098092672511e-45,
                                         4.03248135742865e-36,
                                         8.99429106183111e-30,
                                         5.52171752609279e-25,
                                         3.60400344673752e-21,
                                         5.02090148533661e-18,
                                         2.24173865803747e-15,
                                         4.15002811607254e-13,
                                         3.78813970689716e-11,
                                         1.92597411661438e-09,
                                         5.96046447753906e-08,
                                         1.19994222452384e-06,
                                         1.65321241221407e-05,
                                         0.000162140845166317,
                                         0.00116757590823748,
                                         0.00632601320983101,
                                         0.0262934536885507,
                                         0.0851311702066222,
                                         0.217293827741998,
                                         0.441254040097398,
                                         0.717650204344063,
                                         0.939084135149102,

                                         0.943817025758863,
                                         0.736983658519245,
                                         0.471245435501468,
                                         0.245797126960642,
                                         0.103993163387806,
                                         0.0354165274795488,
                                         0.0096126998537883,
                                         0.00205308395178904,
                                         0.000339593002337824,
                                         4.26418912182991e-05,
                                         3.96462942145882e-06,
                                         2.64506585502677e-07,
                                         1.21693367617591e-08,
                                         3.66870705352844e-10,
                                         6.77960480297402e-12,
                                         7.02548613376658e-14,
                                         3.61321787298724e-16,
                                         7.75408455053364e-19,
                                         5.366122520206e-22,
                                         7.97453238800637e-26,
                                         1.26746308836818e-30,
                                         5.57727579057236e-37,
                                         2.74410995179041e-46,
                                         1.42162268943128e-63,
                                         0.0546400898624462,
                                         0.226355865103715,
                                         0.360169850153872,
                                         0.355197294296975,
                                         0.24683792250365,
                                         0.126941266671944,
                                         0.0492608428643151,
                                         0.0144937219915401,
                                         0.0032185348029391,
                                         0.000533054191944351,
                                         6.46165018262996e-05,
                                         5.58383185307317e-06,
                                         3.32042404650526e-07,
                                         1.29656613758597e-08,
                                         3.12386284042194e-10,
                                         4.26971991689215e-12,
                                         2.94810817331682e-14,
                                         8.7155853331944e-17,
                                         8.62355261378552e-20,
                                         1.93517665561984e-23,
                                         5.05201879111486e-28,
                                         4.20324209937373e-34,
                                         5.14624073847791e-43,
                                         1.41443577467766e-59,
                                         0.00151572914359913,
                                         0.0333128849125743,
                                         0.131902841797196,
                                         0.245951284011055,
                                         0.280740826693059,
                                         0.218014950466042,
                                         0.12096086765877,
                                         0.0490274994869614,
                                         0.0146165301029218,
                                         0.00319295550636868,
                                         0.00050462746578107,
                                         5.64826121117831e-05,
                                         4.34116996750403e-06,
                                         2.1956489976097e-07,
                                         6.89709382575385e-09,
                                         1.24339464408774e-10,
                                         1.15260176007029e-12,
                                         4.69406592991532e-15,
                                         6.64046434468517e-18,
                                         2.25020769357876e-21,
                                         9.64897525583738e-26,
                                         1.51786549643283e-31,
                                         4.62450535483185e-40,
                                         6.74324154917991e-56,
                                         2.68123810572625e-05,
                                         0.00312634083083581,
                                         0.0308038170443749,
                                         0.108600575128796,
                                         0.203611757259082,
                                         0.238766454915384,
                                         0.189405041236319,
                                         0.10575554474928,
                                         0.0423286041114431,
                                         0.0121960151782383,
                                         0.00251305452874877,
                                         0.000364334921168925,
                                         3.61929193556275e-05,
                                         2.37101738558918e-06,
                                         9.71055151969963e-08,
                                         2.30899069666126e-09,
                                         2.87354979825212e-11,
                                         1.6121498108411e-13,
                                         3.26072570116219e-16,
                                         1.66850756387814e-19,
                                         1.17517085968244e-23,
                                         3.49531047200276e-29,
                                         2.64998893564529e-37,
                                         2.0500185322058e-52,
                                         3.39552585832224e-07,
                                         0.000210047868189985,
                                         0.00515006600943489,
                                         0.034329939141887,
                                         0.105720207539653,
                                         0.187205335932685,
                                         0.21232251608474,
                                         0.163314380107149,
                                         0.0877569428590296,
                                         0.0333503904627689,
                                         0.00895964530184847,
                                         0.00168245983117563,
                                         0.00021602216668122,
                                         1.83300833768717e-05,
                                         9.78767492435544e-07,
                                         3.06968103541994e-08,
                                         5.12880343186936e-10,
                                         3.96387075430066e-12,
                                         1.14627273208269e-14,
                                         8.85710170502755e-18,
                                         1.02465747431326e-21,
                                         5.76230318045498e-27,
                                         1.08712807610043e-34,
                                         4.46174488795097e-49,
                                         3.27626776810566e-09,
                                         1.07522888211805e-05,
                                         0.000656027044918697,
                                         0.00826826961415936,
                                         0.0418228717247944,
                                         0.111831415830572,
                                         0.181343192868001,
                                         0.192152657341907,
                                         0.138621218560106,
                                         0.0694839625668813,
                                         0.0243377491086696,
                                         0.00591955822132041,
                                         0.000982366871299684,
                                         0.000107967943741569,
                                         7.51650351283775e-06,
                                         3.10931704128962e-07,
                                         6.97451590629228e-09,
                                         7.42564635530992e-11,
                                         3.07016943725544e-13,
                                         3.58224901098872e-16,
                                         6.80702113036361e-20,
                                         7.23780934961856e-25,
                                         3.39795834925208e-32,
                                         7.39864807767388e-46,
                                         2.50261530157369e-11,
                                         4.35738490214204e-07,
                                         6.61565807655836e-05,
                                         0.00157651673522696,
                                         0.0130982131043074,
                                         0.0528873470635161,
                                         0.122616473967973,
                                         0.178982562723325,
                                         0.173348583870819,
                                         0.114606854114054,
                                         0.052337426235001,
                                         0.0164883139367241,
                                         0.00353664491326897,
                                         0.000503463023249103,
                                         4.56977220674567e-05,
                                         2.493326324493e-06,
                                         7.50852174629038e-08,
                                         1.10126388765298e-09,
                                         6.50997148130316e-12,
                                         1.14699696957803e-14,
                                         3.57995728159939e-18,
                                         7.19714964859257e-23,
                                         8.40809770350367e-30,
                                         9.71275345826383e-43,
                                         1.55231879739129e-13,
                                         1.43391379032351e-08,
                                         5.41746947540579e-06,
                                         0.00024409262355509,
                                         0.00333105933449045,
                                         0.0203100903880001,
                                         0.067323781655506,
                                         0.135377710157763,
                                         0.176028504201317,
                                         0.153500120251388,
                                         0.0913937354827417,
                                         0.0372936863293626,
                                         0.0103390663446048,
                                         0.0019063930365671,
                                         0.000225603251022362,
                                         1.6235488188953e-05,
                                         6.56397519604026e-07,
                                         1.32623549995583e-08,
                                         1.12090277021398e-10,
                                         2.98222827854066e-13,
                                         1.52887051284979e-16,
                                         5.81147054860285e-21,
                                         1.68946641410638e-27,
                                         1.03539121155541e-39,
                                         7.95705219131357e-16,
                                         3.89946142051648e-10,
                                         3.66610048264462e-07,
                                         3.12316690967185e-05,
                                         0.000700063048031442,
                                         0.00644549683091904,
                                         0.0305472874607073,
                                         0.0846190363197739,
                                         0.147716894348478,
                                         0.169899193688717,
                                         0.131887903339387,
                                         0.0697073943126679,
                                         0.0249778846517717,
                                         0.00596543030199535,
                                         0.000920408508426698,
                                         8.7364712305449e-05,
                                         4.7420255423878e-06,
                                         1.31987975807297e-07,
                                         1.59492868237619e-09,
                                         6.40772650478649e-12,
                                         5.395702969853e-15,
                                         3.87789475962316e-19,
                                         2.8053422008921e-25,
                                         9.12118055966014e-37,
                                         3.4122593751998e-18,
                                         8.87165721937476e-12,
                                         2.07553850256874e-08,
                                         3.34313775926792e-06,
                                         0.000123086516080584,
                                         0.00171127372041795,
                                         0.0115956564438568,
                                         0.0442494141391946,
                                         0.103704088078788,
                                         0.157323085118686,
                                         0.15922528645837,
                                         0.109003620491141,
                                         0.0504833920498227,
                                         0.0156167136710054,
                                         0.00314148097947589,
                                         0.000393301215836491,
                                         2.86602111326499e-05,
                                         1.09892139078654e-06,
                                         1.89859849742411e-08,
                                         1.15182258129168e-10,
                                         1.59310317185951e-13,
                                         2.16483367609278e-17,
                                         3.89709206755868e-23,
                                         6.72227338143862e-34,
                                         1.23465508810475e-20,
                                         1.70301575006255e-13,
                                         9.91450440606505e-10,
                                         3.01944509390095e-07,
                                         1.82598661583665e-05,
                                         0.000383350777786065,
                                         0.00371391374612294,
                                         0.0195236386419791,
                                         0.0614292723920377,
                                         0.122915705096802,
                                         0.162193296010573,
                                         0.143819171610136,
                                         0.086090491306669,
                                         0.0344946152533622,
                                         0.00904694667026545,
                                         0.00149392334087371,
                                         0.000146153314128992,
                                         7.71991772002088e-06,
                                         1.90694770448076e-07,
                                         1.74695209565189e-09,
                                         3.96874829841246e-12,
                                         1.01968689410119e-15,
                                         4.5678230628533e-21,
                                         4.18018087207526e-31,
                                         3.79047255801434e-23,
                                         2.77380972914724e-15,
                                         4.01841991468921e-11,
                                         2.31389736850101e-08,
                                         2.29841682407004e-06,
                                         7.28647212629945e-05,
                                         0.00100928165240898,
                                         0.00730900212356925,
                                         0.0308744306188488,
                                         0.0814828837536908,
                                         0.140183807439716,
                                         0.16100402537758,
                                         0.124567840600087,
                                         0.0646482918455667,
                                         0.0221061807868017,
                                         0.00481476869182637,
                                         0.000632385701109126,
                                         4.60153582467783e-05,
                                         1.62513156290026e-06,
                                         2.24812520645489e-08,
                                         8.38894413008426e-11,
                                         4.0752403819342e-14,
                                         4.54278278491292e-19,
                                         2.20555589442156e-28,
                                         9.90530422829326e-26,
                                         3.84557623695251e-17,
                                         1.38632923108836e-12,
                                         1.50934250799179e-09,
                                         2.46255959701611e-07,
                                         1.17886757574758e-05,
                                         0.000233463880154038,
                                         0.0023290680138057,
                                         0.0132083722093934,
                                         0.045978220668402,
                                         0.103131078424234,
                                         0.153420516812903,
                                         0.153420516812904,
                                         0.103131078424234,
                                         0.045978220668402,
                                         0.0132083722093934,
                                         0.00232906801380569,
                                         0.000233463880154038,
                                         1.17886757574758e-05,
                                         2.46255959701611e-07,
                                         1.50934250799179e-09,
                                         1.38632923108833e-12,
                                         3.84557623695251e-17,
                                         9.90530422829052e-26,
                                         2.20555589442222e-28,
                                         4.54278278491292e-19,
                                         4.07524038193429e-14,
                                         8.38894413008426e-11,
                                         2.24812520645489e-08,
                                         1.62513156290027e-06,
                                         4.60153582467783e-05,
                                         0.000632385701109128,
                                         0.00481476869182637,
                                         0.0221061807868017,
                                         0.0646482918455667,
                                         0.124567840600087,
                                         0.16100402537758,
                                         0.140183807439716,
                                         0.0814828837536908,
                                         0.0308744306188488,
                                         0.00730900212356923,
                                         0.00100928165240898,
                                         7.28647212629941e-05,
                                         2.29841682407004e-06,
                                         2.31389736850101e-08,
                                         4.01841991468913e-11,
                                         2.77380972914724e-15,
                                         3.79047255801338e-23,
                                         4.18018087207661e-31,
                                         4.5678230628533e-21,
                                         1.01968689410121e-15,
                                         3.96874829841246e-12,
                                         1.74695209565189e-09,
                                         1.90694770448077e-07,
                                         7.71991772002087e-06,
                                         0.000146153314128993,
                                         0.00149392334087371,
                                         0.00904694667026545,
                                         0.0344946152533622,
                                         0.0860904913066689,
                                         0.143819171610136,
                                         0.162193296010573,
                                         0.122915705096802,
                                         0.0614292723920377,
                                         0.0195236386419791,
                                         0.00371391374612294,
                                         0.000383350777786064,
                                         1.82598661583665e-05,
                                         3.01944509390095e-07,
                                         9.91450440606487e-10,
                                         1.70301575006255e-13,
                                         1.23465508810447e-20,
                                         6.72227338144094e-34,
                                         3.89709206755868e-23,
                                         2.16483367609284e-17,
                                         1.59310317185951e-13,
                                         1.15182258129168e-10,
                                         1.89859849742412e-08,
                                         1.09892139078654e-06,
                                         2.866021113265e-05,
                                         0.000393301215836491,
                                         0.00314148097947589,
                                         0.0156167136710054,
                                         0.0504833920498226,
                                         0.109003620491141,
                                         0.15922528645837,
                                         0.157323085118686,
                                         0.103704088078788,
                                         0.0442494141391945,
                                         0.0115956564438568,
                                         0.00171127372041795,
                                         0.000123086516080584,
                                         3.34313775926792e-06,
                                         2.07553850256871e-08,
                                         8.87165721937476e-12,
                                         3.41225937519909e-18,
                                         9.1211805596635e-37,
                                         2.8053422008921e-25,
                                         3.87789475962327e-19,
                                         5.395702969853e-15,
                                         6.40772650478649e-12,
                                         1.5949286823762e-09,
                                         1.31987975807297e-07,
                                         4.74202554238782e-06,
                                         8.7364712305449e-05,
                                         0.000920408508426698,
                                         0.00596543030199535,
                                         0.0249778846517717,
                                         0.0697073943126679,
                                         0.131887903339387,
                                         0.169899193688717,
                                         0.147716894348478,
                                         0.0846190363197737,
                                         0.0305472874607074,
                                         0.00644549683091902,
                                         0.000700063048031442,
                                         3.12316690967185e-05,
                                         3.66610048264456e-07,
                                         3.89946142051648e-10,
                                         7.9570521913121e-16,
                                         1.03539121155581e-39,
                                         1.68946641410638e-27,
                                         5.81147054860302e-21,
                                         1.52887051284979e-16,
                                         2.98222827854066e-13,
                                         1.12090277021399e-10,
                                         1.32623549995583e-08,
                                         6.56397519604029e-07,
                                         1.6235488188953e-05,
                                         0.000225603251022362,
                                         0.0019063930365671,
                                         0.0103390663446048,
                                         0.0372936863293626,
                                         0.0913937354827417,
                                         0.153500120251388,
                                         0.176028504201317,
                                         0.135377710157762,
                                         0.067323781655506,
                                         0.0203100903880001,
                                         0.00333105933449045,
                                         0.00024409262355509,
                                         5.41746947540572e-06,
                                         1.43391379032351e-08,
                                         1.55231879739104e-13,
                                         9.71275345826787e-43,
                                         8.40809770350367e-30,
                                         7.1971496485928e-23,
                                         3.57995728159939e-18,
                                         1.14699696957803e-14,
                                         6.50997148130321e-12,
                                         1.10126388765298e-09,
                                         7.50852174629041e-08,
                                         2.493326324493e-06,
                                         4.56977220674567e-05,
                                         0.000503463023249103,
                                         0.00353664491326896,
                                         0.0164883139367242,
                                         0.0523374262350009,
                                         0.114606854114054,
                                         0.173348583870819,
                                         0.178982562723325,
                                         0.122616473967973,
                                         0.0528873470635159,
                                         0.0130982131043074,
                                         0.00157651673522696,
                                         6.61565807655829e-05,
                                         4.35738490214204e-07,
                                         2.50261530157334e-11,
                                         7.39864807767713e-46,
                                         3.39795834925208e-32,
                                         7.2378093496188e-25,
                                         6.80702113036361e-20,
                                         3.58224901098872e-16,
                                         3.07016943725547e-13,
                                         7.42564635530992e-11,
                                         6.97451590629231e-09,
                                         3.10931704128963e-07,
                                         7.51650351283775e-06,
                                         0.000107967943741569,
                                         0.000982366871299682,
                                         0.00591955822132042,
                                         0.0243377491086696,
                                         0.0694839625668813,
                                         0.138621218560106,
                                         0.192152657341907,
                                         0.181343192868001,
                                         0.111831415830572,
                                         0.0418228717247944,
                                         0.00826826961415936,
                                         0.000656027044918691,
                                         1.07522888211805e-05,
                                         3.27626776810528e-09,
                                         4.46174488795302e-49,
                                         1.08712807610043e-34,
                                         5.76230318045519e-27,
                                         1.02465747431326e-21,
                                         8.85710170502755e-18,
                                         1.1462727320827e-14,
                                         3.96387075430066e-12,
                                         5.12880343186939e-10,
                                         3.06968103541994e-08,
                                         9.78767492435544e-07,
                                         1.83300833768717e-05,
                                         0.00021602216668122,
                                         0.00168245983117563,
                                         0.00895964530184847,
                                         0.033350390462769,
                                         0.0877569428590296,
                                         0.163314380107149,
                                         0.21232251608474,
                                         0.187205335932685,
                                         0.105720207539653,
                                         0.034329939141887,
                                         0.00515006600943486,
                                         0.000210047868189985,
                                         3.39552585832192e-07,
                                         2.05001853220679e-52,
                                         2.64998893564529e-37,
                                         3.49531047200289e-29,
                                         1.17517085968244e-23,
                                         1.66850756387814e-19,
                                         3.26072570116222e-16,
                                         1.6121498108411e-13,
                                         2.87354979825213e-11,
                                         2.30899069666126e-09,
                                         9.71055151969963e-08,
                                         2.37101738558918e-06,
                                         3.61929193556274e-05,
                                         0.000364334921168925,
                                         0.00251305452874877,
                                         0.0121960151782383,
                                         0.0423286041114432,
                                         0.10575554474928,
                                         0.189405041236319,
                                         0.238766454915384,
                                         0.203611757259082,
                                         0.108600575128796,
                                         0.0308038170443748,
                                         0.00312634083083581,
                                         2.68123810572607e-05,
                                         6.74324154918334e-56,
                                         4.62450535483185e-40,
                                         1.51786549643289e-31,
                                         9.64897525583738e-26,
                                         2.25020769357876e-21,
                                         6.64046434468523e-18,
                                         4.69406592991532e-15,
                                         1.1526017600703e-12,
                                         1.24339464408774e-10,
                                         6.89709382575385e-09,
                                         2.1956489976097e-07,
                                         4.34116996750402e-06,
                                         5.64826121117831e-05,
                                         0.00050462746578107,
                                         0.00319295550636868,
                                         0.0146165301029218,
                                         0.0490274994869614,
                                         0.12096086765877,
                                         0.218014950466041,
                                         0.280740826693059,
                                         0.245951284011055,
                                         0.131902841797195,
                                         0.0333128849125743,
                                         0.00151572914359906,
                                         1.41443577467841e-59,
                                         5.14624073847791e-43,
                                         4.20324209937391e-34,
                                         5.05201879111486e-28,
                                         1.93517665561984e-23,
                                         8.62355261378561e-20,
                                         8.7155853331944e-17,
                                         2.94810817331684e-14,
                                         4.26971991689215e-12,
                                         3.12386284042194e-10,
                                         1.29656613758597e-08,
                                         3.32042404650525e-07,
                                         5.58383185307317e-06,
                                         6.46165018262996e-05,
                                         0.000533054191944351,
                                         0.0032185348029391,
                                         0.0144937219915401,
                                         0.0492608428643151,
                                         0.126941266671944,
                                         0.24683792250365,
                                         0.355197294296975,
                                         0.360169850153871,
                                         0.226355865103715,
                                         0.0546400898624449,
                                         1.42162268943207e-63,
                                         2.74410995179041e-46,
                                         5.57727579057261e-37,
                                         1.26746308836818e-30,
                                         7.97453238800637e-26,
                                         5.36612252020605e-22,
                                         7.75408455053364e-19,
                                         3.61321787298726e-16,
                                         7.02548613376658e-14,
                                         6.77960480297402e-12,
                                         3.66870705352844e-10,
                                         1.2169336761759e-08,
                                         2.64506585502677e-07,
                                         3.96462942145882e-06,
                                         4.26418912182991e-05,
                                         0.000339593002337824,
                                         0.00205308395178904,
                                         0.0096126998537883,
                                         0.0354165274795488,
                                         0.103993163387806,
                                         0.245797126960642,
                                         0.471245435501468,
                                         0.736983658519245,
                                         0.943817025758863,

                                         2.98023223876953e-08,
                                         7.45058059692383e-07,
                                         8.94069671630859e-06,
                                         6.85453414916992e-05,
                                         0.000376999378204346,
                                         0.00158339738845825,
                                         0.00527799129486084,
                                         0.0143259763717651,
                                         0.0322334468364716,
                                         0.0608853995800018,
                                         0.0974166393280029,
                                         0.132840871810913,
                                         0.154981017112732,
                                         0.154981017112732,
                                         0.132840871810913,
                                         0.0974166393280029,
                                         0.0608853995800018,
                                         0.0322334468364716,
                                         0.0143259763717651,
                                         0.00527799129486084,
                                         0.00158339738845825,
                                         0.000376999378204346,
                                         6.85453414916992e-05,
                                         8.94069671630859e-06,
                                         7.45058059692383e-07,
                                         2.98023223876953e-08,

                                         0.00264517076639857,
                                         1.32973477358212e-17,
                                         0.0177192842674728,
                                         1.24065943389989e-15,
                                         0.0569744149191218,
                                         5.55624485064485e-14,
                                         0.117041238245577,
                                         1.5897744196123e-12,
                                         0.172486078979725,
                                         3.26321539861153e-11,
                                         0.194113523386559,
                                         5.11496397873694e-10,
                                         0.173375206104627,
                                         6.36310181584364e-09,
                                         0.126094168971853,
                                         6.4457280876969e-08,
                                         0.0760203691791335,
                                         5.41255156088819e-07,
                                         0.038475904556411,
                                         3.81553995783285e-06,
                                         0.0164953400863156,
                                         2.27836623695063e-05,
                                         0.0060271541659159,
                                         0.000115949707562035,
                                         0.00188413293998422,
                                         0.000504851899701647,
                                         0.000504851899701647,
                                         0.00188413293998422,
                                         0.000115949707562035,
                                         0.0060271541659159,
                                         2.27836623695063e-05,
                                         0.0164953400863156,
                                         3.81553995783285e-06,
                                         0.038475904556411,
                                         5.41255156088819e-07,
                                         0.0760203691791335,
                                         6.4457280876969e-08,
                                         0.126094168971853,
                                         6.36310181584364e-09,
                                         0.173375206104627,
                                         5.11496397873694e-10,
                                         0.194113523386559,
                                         3.26321539861153e-11,
                                         0.172486078979725,
                                         1.5897744196123e-12,
                                         0.117041238245577,
                                         5.55624485064485e-14,
                                         0.0569744149191218,
                                         1.24065943389989e-15,
                                         0.0177192842674728,
                                         1.32973477358212e-17,
                                         0.00264517076639857,

                                         0.0503201190249997,
                                         2.98023223876953e-08,
                                         1.98727669841804e-24,
                                         0.159787328424999,
                                         7.45058059692383e-07,
                                         3.91144908773781e-22,
                                         0.2435478213,
                                         8.94069671630859e-06,
                                         3.69537282327575e-20,
                                         0.2371654913,
                                         6.85453414916992e-05,
                                         2.23051000000193e-18,
                                         0.16568181905,
                                         0.000376999378204346,
                                         9.65842244596035e-17,
                                         0.0883862710499972,
                                         0.00158339738845825,
                                         3.19370516084263e-15,
                                         0.0374217612999164,
                                         0.00527799129486084,
                                         8.38132918133688e-14,
                                         0.0129015072982091,
                                         0.0143259763717651,
                                         1.79105033744457e-12,
                                         0.00368708914327299,
                                         0.0322334468364716,
                                         3.17270463275706e-11,
                                         0.000884607703181053,
                                         0.0608853995800018,
                                         4.71818958349148e-10,
                                         0.00017977585660352,
                                         0.0974166393280029,
                                         5.94339648241243e-09,
                                         3.1137992370647e-05,
                                         0.132840871810913,
                                         6.38076293535555e-08,
                                         4.61421752952833e-06,
                                         0.154981017112732,
                                         5.8608247047177e-07,
                                         5.8608247047177e-07,
                                         0.154981017112732,
                                         4.61421752952833e-06,
                                         6.38076293535555e-08,
                                         0.132840871810913,
                                         3.1137992370647e-05,
                                         5.94339648241243e-09,
                                         0.0974166393280029,
                                         0.00017977585660352,
                                         4.71818958349148e-10,
                                         0.0608853995800018,
                                         0.000884607703181053,
                                         3.17270463275706e-11,
                                         0.0322334468364716,
                                         0.00368708914327299,
                                         1.79105033744457e-12,
                                         0.0143259763717651,
                                         0.0129015072982091,
                                         8.38132918133688e-14,
                                         0.00527799129486084,
                                         0.0374217612999164,
                                         3.19370516084263e-15,
                                         0.00158339738845825,
                                         0.0883862710499972,
                                         9.65842244596035e-17,
                                         0.000376999378204346,
                                         0.16568181905,
                                         2.23051000000193e-18,
                                         6.85453414916992e-05,
                                         0.2371654913,
                                         3.69537282327575e-20,
                                         8.94069671630859e-06,
                                         0.2435478213,
                                         3.91144908773781e-22,
                                         7.45058059692383e-07,
                                         0.159787328424999,
                                         1.98727669841804e-24,
                                         2.98023223876953e-08,
                                         0.0503201190249997,

                                         0.165464438731031,
                                         4.48452523212342e-05,
                                         9.18671695573422e-13,
                                         1.09387455796913e-29,
                                         0.308642119858097,
                                         0.000552222674696375,
                                         4.6627549306783e-11,
                                         3.66519441391644e-27,
                                         0.276342036167467,
                                         0.00326402319083673,
                                         1.13596795799801e-09,
                                         5.89478199039348e-25,
                                         0.158075312340057,
                                         0.0123258696780891,
                                         1.76813291224286e-08,
                                         6.05708705074717e-23,
                                         0.0648690073338252,
                                         0.0333916604599299,
                                         1.97433044771448e-07,
                                         4.46494373735927e-21,
                                         0.0203281125488046,
                                         0.069078843898594,
                                         1.68349324842607e-06,
                                         2.51336039390159e-19,
                                         0.0050557584440452,
                                         0.11341788920222,
                                         1.13928497455414e-05,
                                         1.12285324476812e-17,
                                         0.00102388785329431,
                                         0.151633401013965,
                                         6.27812814677577e-05,
                                         4.08477411378309e-16,
                                         0.00017188794641625,
                                         0.168048696689865,
                                         0.000286784014371785,
                                         1.23179957330479e-14,
                                         2.42249198582038e-05,
                                         0.156350514650271,
                                         0.0010997743768731,
                                         3.11842962241175e-13,
                                         2.89196375516843e-06,
                                         0.123218821947948,
                                         0.00357244725268983,
                                         6.68721699334896e-12,
                                         2.94240150207709e-07,
                                         0.0827624580574264,
                                         0.00989023576440802,
                                         1.22217522476041e-10,
                                         2.56129285070802e-08,
                                         0.0475595706138919,
                                         0.0234258673392535,
                                         1.91103988526489e-09,
                                         1.91103988526487e-09,
                                         0.0234258673392535,
                                         0.0475595706138919,
                                         2.56129285070804e-08,
                                         1.2221752247604e-10,
                                         0.00989023576440803,
                                         0.0827624580574264,
                                         2.94240150207712e-07,
                                         6.68721699334888e-12,
                                         0.00357244725268983,
                                         0.123218821947948,
                                         2.89196375516845e-06,
                                         3.11842962241171e-13,
                                         0.0010997743768731,
                                         0.156350514650271,
                                         2.4224919858204e-05,
                                         1.23179957330478e-14,
                                         0.000286784014371785,
                                         0.168048696689865,
                                         0.000171887946416251,
                                         4.08477411378303e-16,
                                         6.27812814677577e-05,
                                         0.151633401013965,
                                         0.00102388785329432,
                                         1.1228532447681e-17,
                                         1.13928497455414e-05,
                                         0.11341788920222,
                                         0.00505575844404522,
                                         2.51336039390155e-19,
                                         1.68349324842607e-06,
                                         0.0690788438985939,
                                         0.0203281125488047,
                                         4.4649437373592e-21,
                                         1.97433044771448e-07,
                                         0.0333916604599299,
                                         0.0648690073338254,
                                         6.05708705074706e-23,
                                         1.76813291224286e-08,
                                         0.0123258696780891,
                                         0.158075312340057,
                                         5.89478199039337e-25,
                                         1.13596795799801e-09,
                                         0.00326402319083673,
                                         0.276342036167467,
                                         3.66519441391637e-27,
                                         4.6627549306783e-11,
                                         0.000552222674696375,
                                         0.308642119858098,
                                         1.09387455796911e-29,
                                         9.18671695573422e-13,
                                         4.48452523212342e-05,
                                         0.165464438731031,

                                         0.300847872234756,
                                         0.00141733019306706,
                                         2.98023223876953e-08,
                                         1.20023071458362e-16,
                                         6.04858311102775e-34,
                                         0.370185344554815,
                                         0.0106297437576013,
                                         7.45058059692383e-07,
                                         1.00021415686134e-14,
                                         3.07228423841332e-31,
                                         0.218641569197448,
                                         0.038266239860256,
                                         8.94069671630859e-06,
                                         4.00094420819582e-13,
                                         7.49049244889941e-29,
                                         0.0825033503966074,
                                         0.0880104250864125,
                                         6.85453414916992e-05,
                                         1.02248590210955e-11,
                                         1.1667684673132e-26,
                                         0.0223340016090452,
                                         0.145214022585076,
                                         0.000376999378204346,
                                         1.87459852220079e-10,
                                         1.30381239269585e-24,
                                         0.00461687484534631,
                                         0.182965663247415,
                                         0.00158339738845825,
                                         2.62449538133861e-09,
                                         1.11258224940793e-22,
                                         0.000757458974285115,
                                         0.182961658125309,
                                         0.00527799129486084,
                                         2.91616981428193e-08,
                                         7.53491927183721e-21,
                                         0.000101192178277083,
                                         0.148979803231141,
                                         0.0143259763717651,
                                         2.63849711253458e-07,
                                         4.1552953339072e-19,
                                         1.12062867498931e-05,
                                         0.100559165890638,
                                         0.0322334468364716,
                                         1.97891615291595e-06,
                                         1.89955619578676e-17,
                                         1.04183852190602e-06,
                                         0.0569822799674509,
                                         0.0608853995800018,
                                         1.24601151964618e-05,
                                         7.28997920882406e-16,
                                         8.20451025882081e-08,
                                         0.0273508956596056,
                                         0.0974166393280029,
                                         6.64554024245796e-05,
                                         2.36981259887297e-14,
                                         5.50659967264144e-09,
                                         0.0111887578423395,
                                         0.132840871810913,
                                         0.000302076623484275,
                                         6.56568903240992e-13,
                                         3.16200507452505e-10,
                                         0.0039159795220706,
                                         0.154981017112732,
                                         0.00117476814035966,
                                         1.55630542347166e-11,
                                         1.55630542347163e-11,
                                         0.00117476814035966,
                                         0.154981017112732,
                                         0.0039159795220706,
                                         3.16200507452509e-10,
                                         6.56568903240981e-13,
                                         0.000302076623484275,
                                         0.132840871810913,
                                         0.0111887578423395,
                                         5.50659967264151e-09,
                                         2.36981259887292e-14,
                                         6.64554024245796e-05,
                                         0.0974166393280029,
                                         0.0273508956596056,
                                         8.20451025882091e-08,
                                         7.28997920882393e-16,
                                         1.24601151964618e-05,
                                         0.0608853995800018,
                                         0.0569822799674509,
                                         1.04183852190603e-06,
                                         1.89955619578673e-17,
                                         1.97891615291595e-06,
                                         0.0322334468364716,
                                         0.100559165890638,
                                         1.12062867498932e-05,
                                         4.15529533390711e-19,
                                         2.63849711253458e-07,
                                         0.0143259763717651,
                                         0.148979803231141,
                                         0.000101192178277083,
                                         7.53491927183704e-21,
                                         2.91616981428193e-08,
                                         0.00527799129486084,
                                         0.182961658125309,
                                         0.00075745897428512,
                                         1.1125822494079e-22,
                                         2.62449538133861e-09,
                                         0.00158339738845825,
                                         0.182965663247415,
                                         0.00461687484534634,
                                         1.30381239269582e-24,
                                         1.87459852220079e-10,
                                         0.000376999378204346,
                                         0.145214022585076,
                                         0.0223340016090453,
                                         1.16676846731317e-26,
                                         1.02248590210955e-11,
                                         6.85453414916992e-05,
                                         0.0880104250864125,
                                         0.0825033503966077,
                                         7.49049244889921e-29,
                                         4.00094420819582e-13,
                                         8.94069671630859e-06,
                                         0.038266239860256,
                                         0.218641569197448,
                                         3.07228423841323e-31,
                                         1.00021415686134e-14,
                                         7.45058059692383e-07,
                                         0.0106297437576013,
                                         0.370185344554816,
                                         6.04858311102758e-34,
                                         1.20023071458362e-16,
                                         2.98023223876953e-08,
                                         0.00141733019306706,
                                         0.300847872234756,

                                         0.423707736130682,
                                         0.009657384388012,
                                         6.27617697035811e-06,
                                         3.26812251129467e-11,
                                         5.27878947469546e-20,
                                         1.62834699388661e-37,
                                         0.370163527115021,
                                         0.0492386933369303,
                                         9.64493523043204e-05,
                                         1.32915065988329e-09,
                                         6.47093977388945e-18,
                                         1.16492976731151e-34,
                                         0.155225151817164,
                                         0.120502139666113,
                                         0.000711450497628052,
                                         2.59472497089712e-08,
                                         3.80751489405925e-16,
                                         4.00031109201175e-32,
                                         0.0415868376638227,
                                         0.188411901540085,
                                         0.0033528598070419,
                                         3.23618832498411e-07,
                                         1.43133469714921e-14,
                                         8.776333644118e-30,
                                         0.00799292631140339,
                                         0.211338205357628,
                                         0.0113355399030987,
                                         2.89555854535622e-06,
                                         3.86008524587796e-13,
                                         1.38130184574932e-27,
                                         0.00117311968425759,
                                         0.181023225323721,
                                         0.0292654782981195,
                                         1.97841554097901e-05,
                                         7.94948864129919e-12,
                                         1.66016273041239e-25,
                                         0.000136649576433165,
                                         0.123060885783817,
                                         0.0599650910644248,
                                         0.00010728330257827,
                                         1.29930450854696e-10,
                                         1.58358792935361e-23,
                                         1.29613749246895e-05,
                                         0.0681212472062242,
                                         0.100050251433205,
                                         0.000473722016504674,
                                         1.7292568846705e-09,
                                         1.23001532139973e-21,
                                         1.01910941536672e-06,
                                         0.0312587856042278,
                                         0.138377292335334,
                                         0.00173397152584128,
                                         1.90780963839041e-08,
                                         7.91964686135446e-20,
                                         6.72689189602951e-08,
                                         0.0120416363236931,
                                         0.160670211819034,
                                         0.00532824575401883,
                                         1.76699148837676e-07,
                                         4.28080067210377e-18,
                                         3.761158656759e-09,
                                         0.00392927551822445,
                                         0.158022698852326,
                                         0.0138688389273023,
                                         1.38626879448899e-06,
                                         1.96000764931136e-16,
                                         1.79228656801691e-10,
                                         0.00109274304043532,
                                         0.132459214284026,
                                         0.030766249120374,
                                         9.26913117200098e-06,
                                         7.64837402061426e-15,
                                         7.30703962081034e-12,
                                         0.000259999092311817,
                                         0.0949933251759278,
                                         0.0583925197120702,
                                         5.30247738305344e-05,
                                         2.55345779948104e-13,
                                         2.55345779948104e-13,
                                         5.30247738305346e-05,
                                         0.0583925197120702,
                                         0.0949933251759278,
                                         0.000259999092311817,
                                         7.30703962081034e-12,
                                         7.64837402061426e-15,
                                         9.26913117200102e-06,
                                         0.030766249120374,
                                         0.132459214284026,
                                         0.00109274304043531,
                                         1.79228656801691e-10,
                                         1.96000764931136e-16,
                                         1.38626879448899e-06,
                                         0.0138688389273023,
                                         0.158022698852326,
                                         0.00392927551822444,
                                         3.761158656759e-09,
                                         4.28080067210377e-18,
                                         1.76699148837677e-07,
                                         0.00532824575401883,
                                         0.160670211819034,
                                         0.0120416363236931,
                                         6.72689189602951e-08,
                                         7.91964686135446e-20,
                                         1.90780963839042e-08,
                                         0.00173397152584128,
                                         0.138377292335334,
                                         0.0312587856042277,
                                         1.01910941536672e-06,
                                         1.23001532139973e-21,
                                         1.72925688467051e-09,
                                         0.000473722016504674,
                                         0.100050251433205,
                                         0.068121247206224,
                                         1.29613749246895e-05,
                                         1.58358792935361e-23,
                                         1.29930450854697e-10,
                                         0.00010728330257827,
                                         0.0599650910644248,
                                         0.123060885783817,
                                         0.000136649576433165,
                                         1.66016273041239e-25,
                                         7.94948864129924e-12,
                                         1.97841554097901e-05,
                                         0.0292654782981195,
                                         0.18102322532372,
                                         0.00117311968425759,
                                         1.38130184574932e-27,
                                         3.86008524587798e-13,
                                         2.89555854535622e-06,
                                         0.0113355399030987,
                                         0.211338205357627,
                                         0.00799292631140339,
                                         8.776333644118e-30,
                                         1.43133469714922e-14,
                                         3.23618832498411e-07,
                                         0.0033528598070419,
                                         0.188411901540084,
                                         0.0415868376638227,
                                         4.00031109201175e-32,
                                         3.80751489405928e-16,
                                         2.59472497089712e-08,
                                         0.000711450497628052,
                                         0.120502139666113,
                                         0.155225151817164,
                                         1.16492976731151e-34,
                                         6.4709397738895e-18,
                                         1.32915065988329e-09,
                                         9.64493523043204e-05,
                                         0.0492386933369303,
                                         0.370163527115021,
                                         1.62834699388661e-37,
                                         5.2787894746955e-20,
                                         3.26812251129467e-11,
                                         6.27617697035811e-06,
                                         0.009657384388012,
                                         0.423707736130682,

                                         0.524985393983573,
                                         0.0314435449666279,
                                         0.000148828907320814,
                                         2.98023223876953e-08,
                                         6.63347729786086e-14,
                                         6.08770888553869e-23,
                                         1.38199595525351e-40,
                                         0.342690141990011,
                                         0.116667100986757,
                                         0.00157249567659048,
                                         7.45058059692383e-07,
                                         3.9239105418347e-12,
                                         1.02545590438079e-20,
                                         1.3232210424104e-37,
                                         0.107373532075805,
                                         0.207781342218509,
                                         0.00797503989484686,
                                         8.94069671630859e-06,
                                         1.11413594401501e-10,
                                         8.29127540702793e-19,
                                         6.08133968701173e-35,
                                         0.0214940474311199,
                                         0.236423163191671,
                                         0.0258405411120627,
                                         6.85453414916992e-05,
                                         2.02107583152195e-09,
                                         4.28302912331368e-17,
                                         1.78562995078845e-32,
                                         0.0030867060590928,
                                         0.19298769011867,
                                         0.0600656873749335,
                                         0.000376999378204346,
                                         2.63016588300833e-08,
                                         1.58721888457157e-15,
                                         3.7613155486391e-30,
                                         0.000338500213463931,
                                         0.120297276017603,
                                         0.106619808938232,
                                         0.00158339738845825,
                                         2.61378747351357e-07,
                                         4.49168423181277e-14,
                                         6.05026890887108e-28,
                                         2.94613113799589e-05,
                                         0.0595129438680151,
                                         0.150203067950026,
                                         0.00527799129486084,
                                         2.06151671004543e-06,
                                         1.00881392386509e-12,
                                         7.72394265653846e-26,
                                         2.08795924412973e-06,
                                         0.0239741870532222,
                                         0.172304461356333,
                                         0.0143259763717651,
                                         1.32397645597827e-05,
                                         1.84497180587429e-11,
                                         8.02934686617024e-24,
                                         1.22664506905559e-07,
                                         0.0080057735681012,
                                         0.163848020438952,
                                         0.0322334468364716,
                                         7.04856358969747e-05,
                                         2.7970186166749e-10,
                                         6.91908006008715e-22,
                                         6.04978180426667e-09,
                                         0.0022443294558192,
                                         0.130800609840821,
                                         0.0608853995800018,
                                         0.000315024895325029,
                                         3.55979405965769e-09,
                                         5.00541839944667e-20,
                                         2.52740055234692e-10,
                                         0.000532946343646707,
                                         0.088448792771891,
                                         0.0974166393280029,
                                         0.00119262168152309,
                                         3.8376729563028e-08,
                                         3.06722747870901e-18,
                                         8.99885039998064e-12,
                                         0.000107859637375339,
                                         0.0509744609323173,
                                         0.132840871810913,
                                         0.00384803590944115,
                                         3.52605953415858e-07,
                                         1.60188073420203e-16,
                                         2.74124721647514e-13,
                                         1.8675947328444e-05,
                                         0.0251339987698623,
                                         0.154981017112732,
                                         0.0106224268154689,
                                         2.77178496928821e-06,
                                         7.15752025568167e-15,
                                         7.15752025568147e-15,
                                         2.77178496928821e-06,
                                         0.0106224268154689,
                                         0.154981017112732,
                                         0.0251339987698623,
                                         1.8675947328444e-05,
                                         2.74124721647521e-13,
                                         1.60188073420198e-16,
                                         3.52605953415858e-07,
                                         0.00384803590944115,
                                         0.132840871810913,
                                         0.0509744609323173,
                                         0.000107859637375339,
                                         8.99885039998085e-12,
                                         3.06722747870891e-18,
                                         3.8376729563028e-08,
                                         0.00119262168152309,
                                         0.0974166393280029,
                                         0.088448792771891,
                                         0.000532946343646707,
                                         2.52740055234697e-10,
                                         5.00541839944649e-20,
                                         3.55979405965769e-09,
                                         0.000315024895325029,
                                         0.0608853995800018,
                                         0.130800609840821,
                                         0.0022443294558192,
                                         6.04978180426679e-09,
                                         6.9190800600869e-22,
                                         2.7970186166749e-10,
                                         7.04856358969747e-05,
                                         0.0322334468364716,
                                         0.163848020438952,
                                         0.0080057735681012,
                                         1.22664506905561e-07,
                                         8.02934686616992e-24,
                                         1.84497180587429e-11,
                                         1.32397645597827e-05,
                                         0.0143259763717651,
                                         0.172304461356333,
                                         0.0239741870532222,
                                         2.08795924412976e-06,
                                         7.72394265653814e-26,
                                         1.00881392386509e-12,
                                         2.06151671004543e-06,
                                         0.00527799129486084,
                                         0.150203067950026,
                                         0.0595129438680151,
                                         2.94613113799593e-05,
                                         6.05026890887081e-28,
                                         4.49168423181277e-14,
                                         2.61378747351357e-07,
                                         0.00158339738845825,
                                         0.106619808938232,
                                         0.120297276017603,
                                         0.000338500213463934,
                                         3.76131554863893e-30,
                                         1.58721888457157e-15,
                                         2.63016588300833e-08,
                                         0.000376999378204346,
                                         0.0600656873749335,
                                         0.19298769011867,
                                         0.00308670605909282,
                                         1.78562995078836e-32,
                                         4.28302912331368e-17,
                                         2.02107583152195e-09,
                                         6.85453414916992e-05,
                                         0.0258405411120627,
                                         0.236423163191671,
                                         0.0214940474311201,
                                         6.08133968701143e-35,
                                         8.29127540702793e-19,
                                         1.11413594401501e-10,
                                         8.94069671630859e-06,
                                         0.00797503989484686,
                                         0.207781342218509,
                                         0.107373532075805,
                                         1.32322104241034e-37,
                                         1.02545590438079e-20,
                                         3.9239105418347e-12,
                                         7.45058059692383e-07,
                                         0.00157249567659048,
                                         0.116667100986757,
                                         0.342690141990011,
                                         1.38199595525343e-40,
                                         6.08770888553869e-23,
                                         6.63347729786086e-14,
                                         2.98023223876953e-08,
                                         0.000148828907320814,
                                         0.0314435449666279,
                                         0.524985393983573,

                                         0.605699792795649,
                                         0.0685388363011391,
                                         0.00114758820220693,
                                         2.00843167686985e-06,
                                         1.87942019035915e-10,
                                         2.39559781605085e-16,
                                         1.51173170402725e-25,
                                         2.79760580747369e-43,
                                         0.306745780625303,
                                         0.19391805872539,
                                         0.00892300624310299,
                                         3.46452535559504e-05,
                                         6.80953135176609e-09,
                                         1.92561208915548e-14,
                                         3.33943665673154e-23,
                                         3.45259284100105e-40,
                                         0.0745660276333003,
                                         0.263354668010542,
                                         0.0333025551547515,
                                         0.000286861102488107,
                                         1.18427291485443e-07,
                                         7.42959151435068e-13,
                                         3.54089408472062e-21,
                                         2.04524550995201e-37,
                                         0.0115805384429905,
                                         0.228501534965722,
                                         0.0794089129494388,
                                         0.00151748678431631,
                                         1.31586611966587e-06,
                                         1.83141347159525e-11,
                                         2.39871139277111e-19,
                                         7.74053297994397e-35,
                                         0.00129024625126475,
                                         0.142230694638003,
                                         0.135836678541578,
                                         0.00575883028708417,
                                         1.04888463014407e-05,
                                         3.23864973266304e-10,
                                         1.16573320594651e-17,
                                         2.10161128124354e-32,
                                         0.000109774902527776,
                                         0.0676058287955548,
                                         0.177439970854331,
                                         0.0166889972643568,
                                         6.38455068258988e-05,
                                         4.37349019957861e-09,
                                         4.3262034383621e-16,
                                         4.35733065583127e-30,
                                         7.4124703512564e-06,
                                         0.0255037713624699,
                                         0.18395657544107,
                                         0.0383844800217002,
                                         0.000308434117167646,
                                         4.68728962971741e-08,
                                         1.2742192553575e-14,
                                         7.16998255580196e-28,
                                         4.07567639421835e-07,
                                         0.00783432325330141,
                                         0.155294469822204,
                                         0.0718882473817746,
                                         0.0012133086646325,
                                         4.09064919714131e-07,
                                         3.05603416251125e-13,
                                         9.6071040897391e-26,
                                         1.85764779316703e-08,
                                         0.00199492018660656,
                                         0.108673491847534,
                                         0.111605882750488,
                                         0.00395646332321515,
                                         2.95930441427154e-06,
                                         6.07574031251314e-12,
                                         1.0670723102925e-23,
                                         7.10805781766425e-10,
                                         0.000426455241579157,
                                         0.0638432735287175,
                                         0.145458857415645,
                                         0.0108309582818438,
                                         1.79726013173631e-05,
                                         1.01406103465383e-10,
                                         9.94990876549147e-22,
                                         2.30383885344449e-11,
                                         7.72209178148981e-05,
                                         0.0317702215303413,
                                         0.1605856846067,
                                         0.0251154053977185,
                                         9.24582799691156e-05,
                                         1.43364808229492e-09,
                                         7.85883041691192e-20,
                                         6.36402439810781e-13,
                                         1.19172226358029e-05,
                                         0.0134742291948292,
                                         0.151095689366915,
                                         0.0496354648520319,
                                         0.00040537715499693,
                                         1.72742663726078e-08,
                                         5.29023942158992e-18,
                                         1.5040413954181e-14,
                                         1.57348773586373e-06,
                                         0.00488917782509279,
                                         0.121631352818878,
                                         0.0839251662337999,
                                         0.00152062086985719,
                                         1.78075790972735e-07,
                                         3.04677899789837e-16,
                                         3.04677899789837e-16,
                                         1.78075790972735e-07,
                                         0.00152062086985719,
                                         0.0839251662338,
                                         0.121631352818878,
                                         0.00488917782509279,
                                         1.57348773586373e-06,
                                         1.5040413954181e-14,
                                         5.29023942158992e-18,
                                         1.72742663726078e-08,
                                         0.00040537715499693,
                                         0.049635464852032,
                                         0.151095689366915,
                                         0.0134742291948292,
                                         1.19172226358029e-05,
                                         6.36402439810781e-13,
                                         7.85883041691192e-20,
                                         1.43364808229492e-09,
                                         9.24582799691156e-05,
                                         0.0251154053977185,
                                         0.1605856846067,
                                         0.0317702215303413,
                                         7.72209178148981e-05,
                                         2.30383885344449e-11,
                                         9.94990876549147e-22,
                                         1.01406103465383e-10,
                                         1.79726013173631e-05,
                                         0.0108309582818439,
                                         0.145458857415645,
                                         0.0638432735287175,
                                         0.000426455241579157,
                                         7.10805781766425e-10,
                                         1.0670723102925e-23,
                                         6.07574031251314e-12,
                                         2.95930441427154e-06,
                                         0.00395646332321516,
                                         0.111605882750488,
                                         0.108673491847534,
                                         0.00199492018660656,
                                         1.85764779316703e-08,
                                         9.6071040897391e-26,
                                         3.05603416251125e-13,
                                         4.09064919714131e-07,
                                         0.0012133086646325,
                                         0.0718882473817746,
                                         0.155294469822204,
                                         0.00783432325330141,
                                         4.07567639421835e-07,
                                         7.16998255580196e-28,
                                         1.2742192553575e-14,
                                         4.68728962971741e-08,
                                         0.000308434117167646,
                                         0.0383844800217001,
                                         0.18395657544107,
                                         0.0255037713624699,
                                         7.4124703512564e-06,
                                         4.35733065583127e-30,
                                         4.3262034383621e-16,
                                         4.37349019957861e-09,
                                         6.3845506825899e-05,
                                         0.0166889972643568,
                                         0.177439970854331,
                                         0.0676058287955548,
                                         0.000109774902527776,
                                         2.10161128124354e-32,
                                         1.16573320594651e-17,
                                         3.23864973266304e-10,
                                         1.04888463014407e-05,
                                         0.00575883028708416,
                                         0.135836678541578,
                                         0.142230694638003,
                                         0.00129024625126475,
                                         7.74053297994397e-35,
                                         2.39871139277111e-19,
                                         1.83141347159525e-11,
                                         1.31586611966588e-06,
                                         0.00151748678431631,
                                         0.0794089129494388,
                                         0.228501534965722,
                                         0.0115805384429905,
                                         2.04524550995201e-37,
                                         3.54089408472062e-21,
                                         7.42959151435068e-13,
                                         1.18427291485443e-07,
                                         0.000286861102488107,
                                         0.0333025551547515,
                                         0.263354668010542,
                                         0.0745660276333003,
                                         3.45259284100105e-40,
                                         3.33943665673154e-23,
                                         1.92561208915548e-14,
                                         6.80953135176611e-09,
                                         3.46452535559504e-05,
                                         0.008923006243103,
                                         0.19391805872539,
                                         0.306745780625303,
                                         2.79760580747369e-43,
                                         1.51173170402725e-25,
                                         2.39559781605085e-16,
                                         1.87942019035915e-10,
                                         2.00843167686985e-06,
                                         0.00114758820220693,
                                         0.0685388363011391,
                                         0.605699792795649,

                                         0.669516768231775,
                                         0.117828672456524,
                                         0.00465167301135939,
                                         3.33831809590639e-05,
                                         2.98023223876953e-08,
                                         1.65518470024803e-12,
                                         1.4341633224621e-18,
                                         6.97086548940858e-28,
                                         1.11813672805858e-45,
                                         0.270776397142604,
                                         0.263070664635962,
                                         0.0278681901060562,
                                         0.000425873360987713,
                                         7.45058059692383e-07,
                                         8.10911990375823e-11,
                                         1.49616345634352e-16,
                                         1.95139542550911e-25,
                                         1.72792776740982e-42,
                                         0.0525655713937111,
                                         0.281925978724917,
                                         0.0801400460839696,
                                         0.00260780114136694,
                                         8.94069671630859e-06,
                                         1.90696278728069e-09,
                                         7.49205076896207e-15,
                                         2.62207494032431e-23,
                                         1.28173278038509e-39,
                                         0.00651954532692636,
                                         0.193029242686748,
                                         0.147236588585693,
                                         0.0102021992767907,
                                         6.85453414916992e-05,
                                         2.86507535994968e-08,
                                         2.39688785932692e-13,
                                         2.25097282756446e-21,
                                         6.0742789844508e-37,
                                         0.000580081929605707,
                                         0.0948128551735444,
                                         0.194060887529219,
                                         0.0286331574626232,
                                         0.000376999378204346,
                                         3.08806184421824e-07,
                                         5.5011163159773e-12,
                                         1.38628177343585e-19,
                                         2.06513328234831e-34,
                                         3.94137688599612e-05,
                                         0.0355629635087711,
                                         0.195320074381428,
                                         0.0613664897672312,
                                         0.00158339738845825,
                                         2.54168970501437e-06,
                                         9.64140976220424e-11,
                                         6.51957633903008e-18,
                                         5.36152133104652e-32,
                                         2.12537735395789e-06,
                                         0.0105866393429594,
                                         0.156021771123303,
                                         0.104381318170313,
                                         0.00527799129486084,
                                         1.66030748294704e-05,
                                         1.3410955110356e-09,
                                         2.4334178615402e-16,
                                         1.10473330031959e-29,
                                         9.33256337861148e-08,
                                         0.00256622722334811,
                                         0.101484646168461,
                                         0.14457429653617,
                                         0.0143259763717651,
                                         8.83143620965541e-05,
                                         1.51899271998154e-08,
                                         7.39589647970892e-15,
                                         1.85354727831693e-27,
                                         3.39697854767029e-09,
                                         0.000515654787973584,
                                         0.0547195399338231,
                                         0.165991693507022,
                                         0.0322334468364716,
                                         0.000389404624282599,
                                         1.42619409257237e-07,
                                         1.86333916813831e-13,
                                         2.5779639976361e-25,
                                         1.03802677136154e-10,
                                         8.69855062320685e-05,
                                         0.024768997343396,
                                         0.159994670961413,
                                         0.0608853995800018,
                                         0.00144143401603484,
                                         1.12415311398089e-06,
                                         3.94109493352989e-12,
                                         3.01005146917764e-23,
                                         2.68681568676274e-12,
                                         1.24293468262828e-05,
                                         0.00949703386309781,
                                         0.130628593307057,
                                         0.0974166393280029,
                                         0.00451962080804859,
                                         7.50560790785496e-06,
                                         7.06082504007907e-11,
                                         2.97703928868477e-21,
                                         5.92714821388086e-14,
                                         1.513659912425e-06,
                                         0.00310345989685257,
                                         0.0908970050315695,
                                         0.132840871810913,
                                         0.012077799715094,
                                         4.27095376231327e-05,
                                         1.07813363578338e-09,
                                         2.50942258223078e-19,
                                         1.11866980324382e-15,
                                         1.57709018031508e-07,
                                         0.000867665850208644,
                                         0.0541139347161799,
                                         0.154981017112732,
                                         0.0276135258435785,
                                         0.000207927571891655,
                                         1.40843872132879e-08,
                                         1.80971944714454e-17,
                                         1.80971944714446e-17,
                                         1.40843872132879e-08,
                                         0.000207927571891654,
                                         0.0276135258435785,
                                         0.154981017112732,
                                         0.0541139347161799,
                                         0.000867665850208647,
                                         1.57709018031508e-07,
                                         1.11866980324387e-15,
                                         2.50942258223066e-19,
                                         1.07813363578338e-09,
                                         4.27095376231325e-05,
                                         0.012077799715094,
                                         0.132840871810913,
                                         0.0908970050315695,
                                         0.00310345989685258,
                                         1.513659912425e-06,
                                         5.92714821388109e-14,
                                         2.97703928868462e-21,
                                         7.06082504007907e-11,
                                         7.50560790785492e-06,
                                         0.00451962080804859,
                                         0.0974166393280029,
                                         0.130628593307057,
                                         0.00949703386309784,
                                         1.24293468262828e-05,
                                         2.68681568676283e-12,
                                         3.01005146917747e-23,
                                         3.94109493352989e-12,
                                         1.12415311398088e-06,
                                         0.00144143401603484,
                                         0.0608853995800018,
                                         0.159994670961413,
                                         0.0247689973433961,
                                         8.69855062320685e-05,
                                         1.03802677136157e-10,
                                         2.57796399763594e-25,
                                         1.86333916813831e-13,
                                         1.42619409257236e-07,
                                         0.000389404624282599,
                                         0.0322334468364716,
                                         0.165991693507022,
                                         0.0547195399338233,
                                         0.000515654787973584,
                                         3.39697854767039e-09,
                                         1.85354727831682e-27,
                                         7.39589647970892e-15,
                                         1.51899271998153e-08,
                                         8.83143620965541e-05,
                                         0.0143259763717651,
                                         0.14457429653617,
                                         0.101484646168461,
                                         0.00256622722334811,
                                         9.3325633786117e-08,
                                         1.10473330031952e-29,
                                         2.4334178615402e-16,
                                         1.34109551103559e-09,
                                         1.66030748294704e-05,
                                         0.00527799129486084,
                                         0.104381318170313,
                                         0.156021771123303,
                                         0.0105866393429594,
                                         2.12537735395794e-06,
                                         5.36152133104615e-32,
                                         6.51957633903008e-18,
                                         9.64140976220418e-11,
                                         2.54168970501437e-06,
                                         0.00158339738845825,
                                         0.0613664897672312,
                                         0.195320074381429,
                                         0.0355629635087711,
                                         3.94137688599619e-05,
                                         2.06513328234816e-34,
                                         1.38628177343585e-19,
                                         5.50111631597726e-12,
                                         3.08806184421824e-07,
                                         0.000376999378204346,
                                         0.0286331574626232,
                                         0.19406088752922,
                                         0.0948128551735444,
                                         0.000580081929605715,
                                         6.07427898445033e-37,
                                         2.25097282756446e-21,
                                         2.39688785932691e-13,
                                         2.86507535994968e-08,
                                         6.85453414916992e-05,
                                         0.0102021992767907,
                                         0.147236588585693,
                                         0.193029242686748,
                                         0.00651954532692643,
                                         1.28173278038499e-39,
                                         2.62207494032431e-23,
                                         7.49205076896202e-15,
                                         1.90696278728069e-09,
                                         8.94069671630859e-06,
                                         0.00260780114136694,
                                         0.0801400460839697,
                                         0.281925978724917,
                                         0.0525655713937115,
                                         1.72792776740967e-42,
                                         1.95139542550911e-25,
                                         1.49616345634351e-16,
                                         8.10911990375823e-11,
                                         7.45058059692383e-07,
                                         0.000425873360987713,
                                         0.0278681901060562,
                                         0.263070664635962,
                                         0.270776397142605,
                                         1.11813672805848e-45,
                                         6.97086548940858e-28,
                                         1.43416332246209e-18,
                                         1.65518470024803e-12,
                                         2.98023223876953e-08,
                                         3.33831809590639e-05,
                                         0.00465167301135939,
                                         0.117828672456524,
                                         0.669516768231775,

                                         0.720136285509293,
                                         0.174417438412141,
                                         0.0126813558310671,
                                         0.000241768782946286,
                                         9.57333682579284e-07,
                                         5.29775927297597e-10,
                                         2.02419627702459e-14,
                                         1.32743729843732e-20,
                                         5.33945418568791e-30,
                                         7.71874419850097e-48,
                                         0.237990697105126,
                                         0.315475902192099,
                                         0.060520099104729,
                                         0.00238921002771096,
                                         1.77306437212858e-05,
                                         1.78776764747688e-08,
                                         1.28020209782872e-12,
                                         1.7384415440009e-18,
                                         1.84501794573597e-27,
                                         1.45975973103099e-44,
                                         0.0377525685945586,
                                         0.273894754845543,
                                         0.1386356138358,
                                         0.0113331247886364,
                                         0.000157625655083609,
                                         2.89581734164646e-07,
                                         3.88638377782804e-11,
                                         1.09281690564495e-16,
                                         3.06017006386511e-25,
                                         1.3251265237183e-41,
                                         0.00382611804379778,
                                         0.151924066047212,
                                         0.202896859098045,
                                         0.0343455399211175,
                                         0.00089527125594792,
                                         2.99679193061982e-06,
                                         7.53768822399615e-10,
                                         4.38894413598409e-15,
                                         3.24276789936791e-23,
                                         7.68526150995329e-39,
                                         0.000278180275220142,
                                         0.0604540698005801,
                                         0.213025673321565,
                                         0.074670168849221,
                                         0.00364786271666288,
                                         2.22483662321998e-05,
                                         1.04878571562788e-08,
                                         1.26452904588466e-13,
                                         2.46514390277945e-21,
                                         3.19754055446896e-36,
                                         1.54447506451982e-05,
                                         0.01837008272054,
                                         0.170795006902,
                                         0.12396826403271,
                                         0.0113503415777042,
                                         0.000126132284009623,
                                         1.11435087435917e-07,
                                         2.78217322478684e-12,
                                         1.43105158253007e-19,
                                         1.01592131791422e-33,
                                         6.80557656771136e-07,
                                         0.00443022859209887,
                                         0.108679554268472,
                                         0.163344065590511,
                                         0.0280290792773471,
                                         0.000567523500750033,
                                         9.39694332520539e-07,
                                         4.85812828700853e-11,
                                         6.59322285067779e-18,
                                         2.56173109968694e-31,
                                         2.44188812897295e-08,
                                         0.000869997440620735,
                                         0.0563115319706822,
                                         0.175256049417619,
                                         0.0563618900128275,
                                         0.00207930527706961,
                                         6.45250088212041e-06,
                                         6.90765449026644e-10,
                                         2.4735291776187e-16,
                                         5.25997673735582e-29,
                                         7.26295845362669e-10,
                                         0.000141623972341853,
                                         0.0241865427238488,
                                         0.155872546912208,
                                         0.093948363931474,
                                         0.00631509108064163,
                                         3.67279336167295e-05,
                                         8.14177678784391e-09,
                                         7.69242511868855e-15,
                                         8.95284495366313e-27,
                                         1.81353211196867e-11,
                                         1.9354388834982e-05,
                                         0.00872115768359574,
                                         0.11638315857941,
                                         0.131467010879346,
                                         0.0161014458321258,
                                         0.000175504724051281,
                                         8.05622655030279e-08,
                                         2.00832230114293e-13,
                                         1.27926994623476e-24,
                                         3.83575191738351e-13,
                                         2.24045240785239e-06,
                                         0.00266371682952653,
                                         0.0736078644783855,
                                         0.155832459986496,
                                         0.0347747248788597,
                                         0.000710386496675715,
                                         6.75239314220466e-07,
                                         4.44137163995532e-12,
                                         1.54837767770096e-22,
                                         6.91439781690887e-15,
                                         2.21039849034382e-07,
                                         0.000693394769988873,
                                         0.0396768228012193,
                                         0.157426433162187,
                                         0.0640090116508431,
                                         0.0024506380684232,
                                         4.82350086700727e-06,
                                         8.37103987284795e-11,
                                         1.59724007208377e-20,
                                         1.0663663464212e-16,
                                         1.86575082937239e-08,
                                         0.000154426306514931,
                                         0.018297753532504,
                                         0.136064741385598,
                                         0.100801444530273,
                                         0.00723289015094307,
                                         2.94791676825758e-05,
                                         1.3498637097767e-09,
                                         1.40965134106384e-18,
                                         1.40965134106384e-18,
                                         1.34986370977671e-09,
                                         2.94791676825758e-05,
                                         0.00723289015094307,
                                         0.100801444530273,
                                         0.136064741385598,
                                         0.018297753532504,
                                         0.000154426306514931,
                                         1.86575082937237e-08,
                                         1.06636634642119e-16,
                                         1.59724007208377e-20,
                                         8.37103987284805e-11,
                                         4.82350086700727e-06,
                                         0.0024506380684232,
                                         0.0640090116508431,
                                         0.157426433162187,
                                         0.0396768228012193,
                                         0.000693394769988873,
                                         2.2103984903438e-07,
                                         6.91439781690887e-15,
                                         1.54837767770096e-22,
                                         4.44137163995537e-12,
                                         6.75239314220466e-07,
                                         0.000710386496675715,
                                         0.0347747248788597,
                                         0.155832459986496,
                                         0.0736078644783855,
                                         0.00266371682952653,
                                         2.24045240785238e-06,
                                         3.83575191738351e-13,
                                         1.27926994623476e-24,
                                         2.00832230114295e-13,
                                         8.05622655030279e-08,
                                         0.000175504724051281,
                                         0.0161014458321258,
                                         0.131467010879346,
                                         0.11638315857941,
                                         0.00872115768359574,
                                         1.93543888349818e-05,
                                         1.81353211196867e-11,
                                         8.95284495366313e-27,
                                         7.69242511868866e-15,
                                         8.14177678784391e-09,
                                         3.67279336167295e-05,
                                         0.00631509108064163,
                                         0.093948363931474,
                                         0.155872546912208,
                                         0.0241865427238488,
                                         0.000141623972341852,
                                         7.26295845362669e-10,
                                         5.25997673735582e-29,
                                         2.47352917761873e-16,
                                         6.90765449026644e-10,
                                         6.45250088212041e-06,
                                         0.00207930527706961,
                                         0.0563618900128275,
                                         0.175256049417619,
                                         0.0563115319706822,
                                         0.00086999744062073,
                                         2.44188812897295e-08,
                                         2.56173109968694e-31,
                                         6.59322285067789e-18,
                                         4.85812828700853e-11,
                                         9.39694332520539e-07,
                                         0.000567523500750033,
                                         0.0280290792773471,
                                         0.163344065590511,
                                         0.108679554268472,
                                         0.00443022859209885,
                                         6.80557656771136e-07,
                                         1.01592131791422e-33,
                                         1.4310515825301e-19,
                                         2.78217322478684e-12,
                                         1.11435087435917e-07,
                                         0.000126132284009623,
                                         0.0113503415777042,
                                         0.12396826403271,
                                         0.170795006902,
                                         0.0183700827205399,
                                         1.54447506451982e-05,
                                         3.19754055446896e-36,
                                         2.46514390277949e-21,
                                         1.26452904588466e-13,
                                         1.04878571562788e-08,
                                         2.22483662321998e-05,
                                         0.00364786271666288,
                                         0.074670168849221,
                                         0.213025673321565,
                                         0.0604540698005799,
                                         0.000278180275220142,
                                         7.68526150995329e-39,
                                         3.24276789936797e-23,
                                         4.38894413598409e-15,
                                         7.53768822399615e-10,
                                         2.99679193061982e-06,
                                         0.00089527125594792,
                                         0.0343455399211175,
                                         0.202896859098045,
                                         0.151924066047212,
                                         0.00382611804379778,
                                         1.3251265237183e-41,
                                         3.06017006386517e-25,
                                         1.09281690564495e-16,
                                         3.88638377782804e-11,
                                         2.89581734164646e-07,
                                         0.000157625655083609,
                                         0.0113331247886364,
                                         0.1386356138358,
                                         0.273894754845543,
                                         0.0377525685945586,
                                         1.45975973103099e-44,
                                         1.84501794573601e-27,
                                         1.7384415440009e-18,
                                         1.28020209782872e-12,
                                         1.78776764747688e-08,
                                         1.77306437212858e-05,
                                         0.00238921002771096,
                                         0.060520099104729,
                                         0.315475902192099,
                                         0.237990697105126,
                                         7.71874419850097e-48,
                                         5.33945418568802e-30,
                                         1.32743729843732e-20,
                                         2.02419627702459e-14,
                                         5.29775927297597e-10,
                                         9.57333682579284e-07,
                                         0.000241768782946286,
                                         0.0126813558310671,
                                         0.174417438412141,
                                         0.720136285509293,

                                         0.760610533990561,
                                         0.233834071244977,
                                         0.0266904945744071,
                                         0.00103247973506977,
                                         1.16261741066808e-05,
                                         2.98023223876953e-08,
                                         1.15908203018805e-11,
                                         3.35511125942131e-16,
                                         1.7874338335716e-22,
                                         6.240146197893e-32,
                                         8.34411031691366e-50,
                                         0.20927196516223,
                                         0.349864017432374,
                                         0.104071440080648,
                                         0.00817136406913618,
                                         0.000167233748740587,
                                         7.45058059692383e-07,
                                         5.03624776262821e-10,
                                         2.64956098130313e-14,
                                         2.86506875710117e-20,
                                         2.60665343875216e-29,
                                         1.89544565814867e-46,
                                         0.0276376432536161,
                                         0.251265003514084,
                                         0.194781517177496,
                                         0.0310419376492768,
                                         0.00115465506549091,
                                         8.94069671630859e-06,
                                         1.05036741280357e-08,
                                         1.00434321499381e-12,
                                         2.20435410687467e-18,
                                         5.22652535446251e-27,
                                         2.06673062931806e-43,
                                         0.00233193422259911,
                                         0.115289581843841,
                                         0.232910602964717,
                                         0.0753404863074315,
                                         0.00509337837956284,
                                         6.85453414916992e-05,
                                         1.39958966472414e-07,
                                         2.43229202396474e-11,
                                         1.08356010827146e-16,
                                         6.6952710547777e-25,
                                         1.43973230346126e-40,
                                         0.000141152213686873,
                                         0.0379493404557794,
                                         0.19979604328663,
                                         0.131178942121205,
                                         0.0161181697020517,
                                         0.000376999378204346,
                                         1.33787741460594e-06,
                                         4.22576546521899e-10,
                                         3.82103277869715e-15,
                                         6.1528931667106e-23,
                                         7.19508171352066e-38,
                                         6.52447681254929e-06,
                                         0.00953904728910164,
                                         0.130879343483145,
                                         0.174416043358261,
                                         0.0389504167165736,
                                         0.00158339738845825,
                                         9.76604044720619e-06,
                                         5.6063688138673e-09,
                                         1.02895200929177e-13,
                                         4.31794579262417e-21,
                                         2.74584686342645e-35,
                                         2.39349842124684e-07,
                                         0.00190298439075406,
                                         0.0680432089915201,
                                         0.18405100411742,
                                         0.0747029836040679,
                                         0.00527799129486084,
                                         5.65783359255993e-05,
                                         5.90319859559775e-08,
                                         2.19906937644991e-12,
                                         2.40494114331052e-19,
                                         8.31661026768063e-33,
                                         7.14985796354372e-09,
                                         0.000309130791512493,
                                         0.0288054909963931,
                                         0.158149115632659,
                                         0.116665004638203,
                                         0.0143259763717651,
                                         0.000266906184408889,
                                         5.0613915926006e-07,
                                         3.82700965212162e-11,
                                         1.09070808961998e-17,
                                         2.05113007686937e-30,
                                         1.77047027856725e-10,
                                         4.16271102193539e-05,
                                         0.0101086400991212,
                                         0.112647692873424,
                                         0.151032431684865,
                                         0.0322334468364716,
                                         0.00104374416576757,
                                         3.5973230614853e-06,
                                         5.52086517740248e-10,
                                         4.10052603270484e-16,
                                         4.19340697300896e-28,
                                         3.68047405445576e-12,
                                         4.70580745938247e-06,
                                         0.00297806436927496,
                                         0.0673599457856301,
                                         0.164143511778839,
                                         0.0608853995800018,
                                         0.00342652085034554,
                                         2.14641076688276e-05,
                                         6.68618854813735e-09,
                                         1.29417947067887e-14,
                                         7.19721898548877e-26,
                                         6.48085718456582e-14,
                                         4.50614969970684e-07,
                                         0.00074317141590444,
                                         0.0341188769209943,
                                         0.151109162092158,
                                         0.0974166393280029,
                                         0.00952853793837629,
                                         0.000108482530025187,
                                         6.85904524927228e-08,
                                         3.4598957248755e-13,
                                         1.04634762168088e-23,
                                         9.72612245515349e-16,
                                         3.67752529202513e-08,
                                         0.000158060192071208,
                                         0.0147287644483486,
                                         0.118559528997431,
                                         0.132840871810913,
                                         0.0225827978709607,
                                         0.000467288790932985,
                                         5.99690691501242e-07,
                                         7.88333723103167e-12,
                                         1.29648010682939e-21,
                                         1.24880673095052e-17,
                                         2.56775708865907e-09,
                                         2.8761016357819e-05,
                                         0.00543983994749698,
                                         0.0795848402685093,
                                         0.154981017112732,
                                         0.0457907169078972,
                                         0.0017221030565147,
                                         4.48579232160259e-06,
                                         1.5367577633929e-10,
                                         1.37437086138013e-19,
                                         1.37437086138004e-19,
                                         1.5367577633929e-10,
                                         4.48579232160262e-06,
                                         0.00172210305651471,
                                         0.0457907169078971,
                                         0.154981017112732,
                                         0.0795848402685095,
                                         0.00543983994749697,
                                         2.87610163578189e-05,
                                         2.56775708865907e-09,
                                         1.2488067309506e-17,
                                         1.2964801068293e-21,
                                         7.88333723103167e-12,
                                         5.99690691501245e-07,
                                         0.000467288790932987,
                                         0.0225827978709606,
                                         0.132840871810913,
                                         0.118559528997431,
                                         0.0147287644483486,
                                         0.000158060192071207,
                                         3.67752529202513e-08,
                                         9.72612245515403e-16,
                                         1.0463476216808e-23,
                                         3.4598957248755e-13,
                                         6.85904524927233e-08,
                                         0.000108482530025187,
                                         0.00952853793837627,
                                         0.0974166393280029,
                                         0.151109162092158,
                                         0.0341188769209942,
                                         0.000743171415904437,
                                         4.50614969970684e-07,
                                         6.48085718456615e-14,
                                         7.19721898548818e-26,
                                         1.29417947067887e-14,
                                         6.68618854813739e-09,
                                         2.14641076688276e-05,
                                         0.00342652085034553,
                                         0.0608853995800018,
                                         0.16414351177884,
                                         0.06735994578563,
                                         0.00297806436927495,
                                         4.70580745938247e-06,
                                         3.68047405445593e-12,
                                         4.1934069730086e-28,
                                         4.10052603270484e-16,
                                         5.52086517740252e-10,
                                         3.59732306148532e-06,
                                         0.00104374416576757,
                                         0.0322334468364716,
                                         0.151032431684865,
                                         0.112647692873424,
                                         0.0101086400991212,
                                         4.16271102193539e-05,
                                         1.77047027856732e-10,
                                         2.05113007686918e-30,
                                         1.09070808961998e-17,
                                         3.82700965212165e-11,
                                         5.06139159260062e-07,
                                         0.000266906184408888,
                                         0.0143259763717651,
                                         0.116665004638203,
                                         0.158149115632659,
                                         0.028805490996393,
                                         0.000309130791512493,
                                         7.14985796354398e-09,
                                         8.31661026767983e-33,
                                         2.40494114331052e-19,
                                         2.19906937644992e-12,
                                         5.90319859559777e-08,
                                         5.65783359255992e-05,
                                         0.00527799129486084,
                                         0.074702983604068,
                                         0.18405100411742,
                                         0.0680432089915199,
                                         0.00190298439075406,
                                         2.39349842124691e-07,
                                         2.74584686342617e-35,
                                         4.31794579262417e-21,
                                         1.02895200929178e-13,
                                         5.60636881386733e-09,
                                         9.76604044720616e-06,
                                         0.00158339738845825,
                                         0.0389504167165737,
                                         0.17441604335826,
                                         0.130879343483145,
                                         0.00953904728910164,
                                         6.52447681254946e-06,
                                         7.19508171351989e-38,
                                         6.1528931667106e-23,
                                         3.82103277869718e-15,
                                         4.22576546521901e-10,
                                         1.33787741460593e-06,
                                         0.000376999378204346,
                                         0.0161181697020517,
                                         0.131178942121205,
                                         0.19979604328663,
                                         0.0379493404557794,
                                         0.000141152213686876,
                                         1.4397323034611e-40,
                                         6.6952710547777e-25,
                                         1.08356010827147e-16,
                                         2.43229202396475e-11,
                                         1.39958966472413e-07,
                                         6.85453414916992e-05,
                                         0.00509337837956284,
                                         0.0753404863074314,
                                         0.232910602964716,
                                         0.115289581843841,
                                         0.00233193422259915,
                                         2.06673062931782e-43,
                                         5.22652535446251e-27,
                                         2.20435410687469e-18,
                                         1.00434321499382e-12,
                                         1.05036741280356e-08,
                                         8.94069671630859e-06,
                                         0.00115465506549091,
                                         0.0310419376492768,
                                         0.194781517177496,
                                         0.251265003514083,
                                         0.0276376432536164,
                                         1.89544565814844e-46,
                                         2.60665343875216e-29,
                                         2.8650687571012e-20,
                                         2.64956098130315e-14,
                                         5.03624776262819e-10,
                                         7.45058059692383e-07,
                                         0.000167233748740587,
                                         0.00817136406913618,
                                         0.104071440080648,
                                         0.349864017432374,
                                         0.209271965162231,
                                         8.34411031691259e-50,
                                         6.240146197893e-32,
                                         1.78743383357162e-22,
                                         3.35511125942133e-16,
                                         1.15908203018804e-11,
                                         2.98023223876953e-08,
                                         1.16261741066808e-05,
                                         0.00103247973506977,
                                         0.0266904945744071,
                                         0.233834071244977,
                                         0.760610533990561,

                                         0.793294160186717,
                                         0.292814333035149,
                                         0.0470960782657692,
                                         0.00309633772345846,
                                         7.50006667232261e-05,
                                         5.69266410136572e-07,
                                         1.0508840928698e-09,
                                         3.12659350049677e-13,
                                         7.32216441509561e-18,
                                         3.32689727594979e-24,
                                         1.04174499849968e-33,
                                         1.31189651757241e-51,
                                         0.184549502508479,
                                         0.368620177296436,
                                         0.153068892189542,
                                         0.020125189513632,
                                         0.000866573461191765,
                                         1.10638209254477e-05,
                                         3.37944401762705e-08,
                                         1.6912631156745e-11,
                                         7.04089453433115e-16,
                                         6.39760193306811e-22,
                                         5.17195418407401e-31,
                                         3.52452808058801e-48,
                                         0.0206078525230706,
                                         0.222744563685677,
                                         0.238797827272985,
                                         0.0627874536901171,
                                         0.00480603448337713,
                                         0.000103213369029948,
                                         5.21647261954636e-07,
                                         4.39129053544549e-10,
                                         3.24980602127123e-14,
                                         5.90522261662251e-20,
                                         1.23250630988665e-28,
                                         4.54510173000784e-45,
                                         0.00147020512357618,
                                         0.0859924769050992,
                                         0.238012155902104,
                                         0.125150070400607,
                                         0.0170291792029771,
                                         0.000615165690900734,
                                         5.14439068409274e-06,
                                         7.28447909276673e-09,
                                         9.58324021113266e-13,
                                         3.48241616278889e-18,
                                         1.87650172913611e-26,
                                         3.74465297249965e-42,
                                         7.52452750907165e-05,
                                         0.0238160597739753,
                                         0.170186071581529,
                                         0.178955657964061,
                                         0.0432869171667499,
                                         0.0026302944332055,
                                         3.63954473830488e-05,
                                         8.66883903199764e-08,
                                         2.02732519453135e-11,
                                         1.47326602413896e-16,
                                         2.04958009527197e-24,
                                         2.21327640016306e-39,
                                         2.94081115834838e-06,
                                         0.00503693058445909,
                                         0.0929256247991563,
                                         0.195409813546709,
                                         0.0840245507908053,
                                         0.00858822129691196,
                                         0.000196628643101755,
                                         7.87789117766307e-07,
                                         3.27507358226755e-10,
                                         4.75957613176494e-15,
                                         1.70949337242161e-22,
                                         9.9895593335461e-37,
                                         9.12188296149544e-08,
                                         0.000845456903618752,
                                         0.0402695170321399,
                                         0.169346709206662,
                                         0.12944497515393,
                                         0.0222552020882113,
                                         0.000843093919532985,
                                         5.68183307742102e-06,
                                         4.19902210651991e-09,
                                         1.22035121357817e-13,
                                         1.13161684326154e-20,
                                         3.57838004366732e-34,
                                         2.30397981281915e-09,
                                         0.000115556345680262,
                                         0.0142100029403518,
                                         0.119504420093257,
                                         0.162383167740745,
                                         0.0469609283346806,
                                         0.00294362137381859,
                                         3.3369045169257e-05,
                                         4.38381356223981e-08,
                                         2.5478755570248e-12,
                                         6.09969613009923e-19,
                                         1.04376663765229e-31,
                                         4.82391670866136e-11,
                                         1.30925150322289e-05,
                                         0.00415660611109449,
                                         0.0699065924070599,
                                         0.168858831355326,
                                         0.0821426599396118,
                                         0.00851951546175754,
                                         0.000162452367926839,
                                         3.79387439015165e-07,
                                         4.40959338781111e-11,
                                         2.72548599420906e-17,
                                         2.52375572100337e-29,
                                         8.47900371101594e-13,
                                         1.24530650548709e-06,
                                         0.00102072198639957,
                                         0.034330188734981,
                                         0.147411186929334,
                                         0.120621505411923,
                                         0.0207000663260075,
                                         0.000663945155084013,
                                         2.75637126483333e-06,
                                         6.40682365998165e-10,
                                         1.02235941979258e-15,
                                         5.12289100032823e-27,
                                         1.26241870532745e-14,
                                         1.00332816379931e-07,
                                         0.000212319380391723,
                                         0.0142806448864198,
                                         0.109006068820926,
                                         0.150035592404227,
                                         0.0426031929706722,
                                         0.00229853931739261,
                                         1.69631323399549e-05,
                                         7.88497948105206e-09,
                                         3.24845475171073e-14,
                                         8.80838898744098e-25,
                                         1.60191920350369e-16,
                                         6.8895100950319e-09,
                                         3.76400678747878e-05,
                                         0.00506288473177391,
                                         0.0686988557273753,
                                         0.159053211186681,
                                         0.0747293410602369,
                                         0.00678188246638993,
                                         8.89719231587457e-05,
                                         8.27059882789816e-08,
                                         8.79687927594322e-13,
                                         1.29079253855226e-22,
                                         1.73910582674706e-18,
                                         4.04745374420811e-10,
                                         5.7089911978289e-06,
                                         0.00153566496226541,
                                         0.0370421806027355,
                                         0.144257632911015,
                                         0.112147183760485,
                                         0.0171197254944201,
                                         0.000399252938862445,
                                         7.42201041233342e-07,
                                         2.03811487139197e-11,
                                         1.61832082596045e-20,
                                         1.61832082596058e-20,
                                         2.03811487139194e-11,
                                         7.42201041233337e-07,
                                         0.000399252938862444,
                                         0.0171197254944201,
                                         0.112147183760485,
                                         0.144257632911015,
                                         0.0370421806027356,
                                         0.00153566496226541,
                                         5.70899119782893e-06,
                                         4.04745374420817e-10,
                                         1.73910582674693e-18,
                                         1.29079253855237e-22,
                                         8.79687927594308e-13,
                                         8.2705988278981e-08,
                                         8.89719231587454e-05,
                                         0.00678188246638992,
                                         0.0747293410602369,
                                         0.159053211186681,
                                         0.0686988557273754,
                                         0.00506288473177393,
                                         3.7640067874788e-05,
                                         6.88951009503199e-09,
                                         1.60191920350359e-16,
                                         8.80838898744177e-25,
                                         3.24845475171067e-14,
                                         7.88497948105201e-09,
                                         1.69631323399549e-05,
                                         0.0022985393173926,
                                         0.0426031929706722,
                                         0.150035592404227,
                                         0.109006068820926,
                                         0.0142806448864199,
                                         0.000212319380391724,
                                         1.00332816379932e-07,
                                         1.26241870532738e-14,
                                         5.12289100032873e-27,
                                         1.02235941979256e-15,
                                         6.4068236599816e-10,
                                         2.75637126483332e-06,
                                         0.000663945155084011,
                                         0.0207000663260075,
                                         0.120621505411923,
                                         0.147411186929334,
                                         0.034330188734981,
                                         0.00102072198639957,
                                         1.2453065054871e-06,
                                         8.47900371101548e-13,
                                         2.52375572100363e-29,
                                         2.725485994209e-17,
                                         4.40959338781107e-11,
                                         3.79387439015163e-07,
                                         0.000162452367926839,
                                         0.00851951546175754,
                                         0.0821426599396118,
                                         0.168858831355327,
                                         0.06990659240706,
                                         0.00415660611109451,
                                         1.3092515032229e-05,
                                         4.82391670866112e-11,
                                         1.0437666376524e-31,
                                         6.0996961300991e-19,
                                         2.54787555702478e-12,
                                         4.38381356223979e-08,
                                         3.33690451692569e-05,
                                         0.00294362137381859,
                                         0.0469609283346806,
                                         0.162383167740746,
                                         0.119504420093257,
                                         0.0142100029403518,
                                         0.000115556345680263,
                                         2.30397981281906e-09,
                                         3.57838004366773e-34,
                                         1.13161684326151e-20,
                                         1.22035121357816e-13,
                                         4.19902210651989e-09,
                                         5.681833077421e-06,
                                         0.000843093919532985,
                                         0.0222552020882113,
                                         0.12944497515393,
                                         0.169346709206662,
                                         0.04026951703214,
                                         0.000845456903618757,
                                         9.12188296149511e-08,
                                         9.9895593335473e-37,
                                         1.70949337242157e-22,
                                         4.75957613176489e-15,
                                         3.27507358226753e-10,
                                         7.87789117766304e-07,
                                         0.000196628643101755,
                                         0.00858822129691196,
                                         0.0840245507908053,
                                         0.19540981354671,
                                         0.0929256247991565,
                                         0.00503693058445912,
                                         2.94081115834829e-06,
                                         2.21327640016334e-39,
                                         2.04958009527192e-24,
                                         1.47326602413895e-16,
                                         2.02732519453134e-11,
                                         8.6688390319976e-08,
                                         3.63954473830488e-05,
                                         0.0026302944332055,
                                         0.04328691716675,
                                         0.178955657964062,
                                         0.17018607158153,
                                         0.0238160597739754,
                                         7.52452750907147e-05,
                                         3.74465297250015e-42,
                                         1.87650172913606e-26,
                                         3.48241616278885e-18,
                                         9.5832402111326e-13,
                                         7.28447909276671e-09,
                                         5.14439068409274e-06,
                                         0.000615165690900734,
                                         0.0170291792029771,
                                         0.125150070400607,
                                         0.238012155902104,
                                         0.0859924769050995,
                                         0.00147020512357616,
                                         4.54510173000847e-45,
                                         1.23250630988662e-28,
                                         5.90522261662245e-20,
                                         3.24980602127121e-14,
                                         4.39129053544548e-10,
                                         5.21647261954636e-07,
                                         0.000103213369029948,
                                         0.00480603448337713,
                                         0.0627874536901171,
                                         0.238797827272986,
                                         0.222744563685678,
                                         0.0206078525230704,
                                         3.52452808058852e-48,
                                         5.17195418407387e-31,
                                         6.39760193306804e-22,
                                         7.04089453433111e-16,
                                         1.69126311567449e-11,
                                         3.37944401762705e-08,
                                         1.10638209254477e-05,
                                         0.000866573461191765,
                                         0.020125189513632,
                                         0.153068892189542,
                                         0.368620177296436,
                                         0.184549502508478,
                                         1.31189651757261e-51,
                                         1.04174499849965e-33,
                                         3.32689727594975e-24,
                                         7.32216441509556e-18,
                                         3.12659350049675e-13,
                                         1.0508840928698e-09,
                                         5.69266410136572e-07,
                                         7.50006667232261e-05,
                                         0.00309633772345846,
                                         0.0470960782657692,
                                         0.292814333035149,
                                         0.793294160186717,

                                         0.819961606011208,
                                         0.349294357828943,
                                         0.0733799481322784,
                                         0.00725919749623189,
                                         0.000314165333699364,
                                         5.32013622072982e-06,
                                         2.98023223876953e-08,
                                         4.26632408794007e-11,
                                         1.03045904807513e-14,
                                         2.04515343447153e-19,
                                         8.20334003295915e-26,
                                         2.35852955998659e-35,
                                         2.83344908501294e-53,
                                         0.163408409122027,
                                         0.375240381916641,
                                         0.202047714239948,
                                         0.0395204710306196,
                                         0.00299042675195089,
                                         8.31817407701153e-05,
                                         7.45058059692383e-07,
                                         1.70540922649341e-09,
                                         6.76606002995343e-13,
                                         2.34786116912411e-17,
                                         1.86206222498733e-23,
                                         1.37215553645274e-32,
                                         8.88616853770378e-50,
                                         0.015631351307789,
                                         0.193494580462571,
                                         0.26703717210155,
                                         0.103275391394232,
                                         0.0136631021178285,
                                         0.000624272917276317,
                                         8.94069671630859e-06,
                                         3.2722359425402e-08,
                                         2.1324663836894e-11,
                                         1.29377920930149e-15,
                                         2.02879844500062e-21,
                                         3.83183321980385e-30,
                                         1.33768826181168e-46,
                                         0.000955309245552872,
                                         0.0637460643653473,
                                         0.225483534600489,
                                         0.172423684877888,
                                         0.0398832733051094,
                                         0.00299327293854855,
                                         6.85453414916992e-05,
                                         4.01130743346521e-07,
                                         4.29391927917428e-10,
                                         4.55484138334113e-14,
                                         1.41224144346933e-19,
                                         6.83652158530998e-28,
                                         1.28653208505743e-43,
                                         4.18839416821542e-05,
                                         0.0150658644791708,
                                         0.136588474962974,
                                         0.206515713931768,
                                         0.0835195956516384,
                                         0.0102961355735297,
                                         0.000376999378204346,
                                         3.5276329800345e-06,
                                         6.20271270887111e-09,
                                         1.15038300195335e-12,
                                         7.05237032054844e-18,
                                         8.75023846447497e-26,
                                         8.8765138552858e-41,
                                         1.40228959780722e-06,
                                         0.00271907594042676,
                                         0.0631829470654689,
                                         0.188884310338644,
                                         0.133558819730084,
                                         0.0270451121373285,
                                         0.00158339738845825,
                                         2.369012983812e-05,
                                         6.84220461875595e-08,
                                         2.2186983224639e-11,
                                         2.68935334800538e-16,
                                         8.5524627044985e-24,
                                         4.67682209590694e-38,
                                         3.72612425795155e-08,
                                         0.000389473642684587,
                                         0.0231961098518976,
                                         0.137109681315816,
                                         0.169506445444647,
                                         0.0563810001240243,
                                         0.00527799129486084,
                                         0.000126264406766354,
                                         5.99018043129478e-07,
                                         3.39612376558917e-10,
                                         8.13935666666511e-15,
                                         6.63425169612879e-22,
                                         1.95563913417541e-35,
                                         8.06220376428988e-10,
                                         4.54267454677216e-05,
                                         0.00693437382425247,
                                         0.0810433106289671,
                                         0.17517683551655,
                                         0.0957091766073274,
                                         0.0143259763717651,
                                         0.000547988267039575,
                                         4.27032143070011e-06,
                                         4.2329737815968e-09,
                                         2.00589921354158e-13,
                                         4.19053541156869e-20,
                                         6.65891397581191e-33,
                                         1.44602953764399e-11,
                                         4.3920991114884e-06,
                                         0.00171840805953542,
                                         0.0397093870285266,
                                         0.150070073032589,
                                         0.134679452344959,
                                         0.0322334468364716,
                                         0.00197146256266619,
                                         2.52352833246083e-05,
                                         4.37355512979872e-08,
                                         4.09784090857466e-12,
                                         2.19418904766499e-18,
                                         1.87951175696453e-30,
                                         2.17733098042034e-13,
                                         3.56497537114869e-07,
                                         0.000357494401839758,
                                         0.0163340201287635,
                                         0.107928316996532,
                                         0.159101032223156,
                                         0.0608853995800018,
                                         0.00595428615890171,
                                         0.000125192899517254,
                                         3.79356496588799e-07,
                                         7.02789028731786e-11,
                                         9.64500516942774e-17,
                                         4.45359358589179e-28,
                                         2.77706030357897e-15,
                                         2.45106318346818e-08,
                                         6.29978547186483e-05,
                                         0.00569123554737778,
                                         0.0657491717781068,
                                         0.15920555738647,
                                         0.0974166393280029,
                                         0.0152329649123881,
                                         0.000526095735706891,
                                         2.78723776149643e-06,
                                         1.02095928519198e-09,
                                         3.59124022814848e-15,
                                         8.93901548599511e-26,
                                         3.01873340221807e-17,
                                         1.4362534028732e-09,
                                         9.46151902546606e-06,
                                         0.00169004638995372,
                                         0.0341368748084985,
                                         0.13577569706254,
                                         0.132840871810913,
                                         0.03321373926204,
                                         0.00188420632141545,
                                         1.74533621310819e-05,
                                         1.26406807952143e-08,
                                         1.13963306445912e-13,
                                         1.52914048862371e-23,
                                         2.80745255584781e-19,
                                         7.20038718711096e-11,
                                         1.2157497506185e-06,
                                         0.000429377127893889,
                                         0.0151637092306974,
                                         0.0990681548635485,
                                         0.154981017112732,
                                         0.0619582750105062,
                                         0.00577351564646851,
                                         9.35044754076431e-05,
                                         1.33900044604803e-07,
                                         3.09409640033445e-12,
                                         2.23796505823395e-21,
                                         2.23796505823415e-21,
                                         3.0940964003345e-12,
                                         1.33900044604802e-07,
                                         9.35044754076435e-05,
                                         0.00577351564646851,
                                         0.0619582750105062,
                                         0.154981017112732,
                                         0.0990681548635485,
                                         0.0151637092306974,
                                         0.000429377127893887,
                                         1.2157497506185e-06,
                                         7.20038718711084e-11,
                                         2.80745255584757e-19,
                                         1.52914048862386e-23,
                                         1.13963306445915e-13,
                                         1.26406807952142e-08,
                                         1.7453362131082e-05,
                                         0.00188420632141545,
                                         0.03321373926204,
                                         0.132840871810913,
                                         0.13577569706254,
                                         0.0341368748084985,
                                         0.00169004638995371,
                                         9.46151902546611e-06,
                                         1.43625340287318e-09,
                                         3.01873340221784e-17,
                                         8.93901548599606e-26,
                                         3.59124022814855e-15,
                                         1.02095928519197e-09,
                                         2.78723776149644e-06,
                                         0.000526095735706891,
                                         0.0152329649123881,
                                         0.0974166393280029,
                                         0.15920555738647,
                                         0.0657491717781068,
                                         0.00569123554737776,
                                         6.29978547186486e-05,
                                         2.45106318346815e-08,
                                         2.77706030357878e-15,
                                         4.45359358589229e-28,
                                         9.64500516942795e-17,
                                         7.0278902873178e-11,
                                         3.79356496588801e-07,
                                         0.000125192899517254,
                                         0.00595428615890171,
                                         0.0608853995800018,
                                         0.159101032223156,
                                         0.107928316996532,
                                         0.0163340201287635,
                                         0.000357494401839759,
                                         3.56497537114865e-07,
                                         2.1773309804202e-13,
                                         1.87951175696476e-30,
                                         2.19418904766504e-18,
                                         4.09784090857462e-12,
                                         4.37355512979874e-08,
                                         2.52352833246083e-05,
                                         0.00197146256266619,
                                         0.0322334468364716,
                                         0.134679452344959,
                                         0.150070073032589,
                                         0.0397093870285265,
                                         0.00171840805953543,
                                         4.39209911148835e-06,
                                         1.4460295376439e-11,
                                         6.65891397581276e-33,
                                         4.19053541156879e-20,
                                         2.00589921354156e-13,
                                         4.23297378159682e-09,
                                         4.27032143070011e-06,
                                         0.000547988267039575,
                                         0.0143259763717651,
                                         0.0957091766073274,
                                         0.17517683551655,
                                         0.0810433106289669,
                                         0.0069343738242525,
                                         4.54267454677212e-05,
                                         8.06220376428949e-10,
                                         1.95563913417567e-35,
                                         6.63425169612896e-22,
                                         8.13935666666502e-15,
                                         3.39612376558919e-10,
                                         5.99018043129478e-07,
                                         0.000126264406766354,
                                         0.00527799129486084,
                                         0.0563810001240243,
                                         0.169506445444647,
                                         0.137109681315816,
                                         0.0231961098518977,
                                         0.000389473642684584,
                                         3.72612425795139e-08,
                                         4.6768220959076e-38,
                                         8.55246270449873e-24,
                                         2.68935334800535e-16,
                                         2.21869832246391e-11,
                                         6.84220461875595e-08,
                                         2.369012983812e-05,
                                         0.00158339738845825,
                                         0.0270451121373285,
                                         0.133558819730084,
                                         0.188884310338644,
                                         0.0631829470654691,
                                         0.00271907594042675,
                                         1.40228959780717e-06,
                                         8.87651385528711e-41,
                                         8.75023846447521e-26,
                                         7.05237032054835e-18,
                                         1.15038300195336e-12,
                                         6.20271270887111e-09,
                                         3.5276329800345e-06,
                                         0.000376999378204346,
                                         0.0102961355735297,
                                         0.0835195956516384,
                                         0.206515713931768,
                                         0.136588474962975,
                                         0.0150658644791708,
                                         4.18839416821531e-05,
                                         1.28653208505763e-43,
                                         6.83652158531019e-28,
                                         1.41224144346931e-19,
                                         4.55484138334116e-14,
                                         4.29391927917428e-10,
                                         4.01130743346521e-07,
                                         6.85453414916992e-05,
                                         0.00299327293854856,
                                         0.0398832733051094,
                                         0.172423684877888,
                                         0.22548353460049,
                                         0.063746064365347,
                                         0.000955309245552851,
                                         1.3376882618119e-46,
                                         3.83183321980397e-30,
                                         2.02879844500059e-21,
                                         1.2937792093015e-15,
                                         2.1324663836894e-11,
                                         3.2722359425402e-08,
                                         8.94069671630859e-06,
                                         0.000624272917276317,
                                         0.0136631021178285,
                                         0.103275391394232,
                                         0.26703717210155,
                                         0.19349458046257,
                                         0.0156313513077888,
                                         8.88616853770528e-50,
                                         1.37215553645279e-32,
                                         1.8620622249873e-23,
                                         2.34786116912413e-17,
                                         6.76606002995343e-13,
                                         1.70540922649341e-09,
                                         7.45058059692383e-07,
                                         8.31817407701153e-05,
                                         0.00299042675195089,
                                         0.0395204710306196,
                                         0.202047714239948,
                                         0.375240381916641,
                                         0.163408409122026,
                                         2.83344908501344e-53,
                                         2.35852955998667e-35,
                                         8.20334003295903e-26,
                                         2.04515343447154e-19,
                                         1.03045904807513e-14,
                                         4.26632408794007e-11,
                                         2.98023223876953e-08,
                                         5.32013622072982e-06,
                                         0.000314165333699364,
                                         0.00725919749623189,
                                         0.0733799481322784,
                                         0.349294357828943,
                                         0.819961606011208,

                                         0.84194295904031,
                                         0.402137506243743,
                                         0.104450395023071,
                                         0.0142564736381991,
                                         0.000969053276762979,
                                         3.0289696631282e-05,
                                         3.87509675586485e-07,
                                         1.70884477700461e-09,
                                         2.00048929181175e-12,
                                         4.09473777313696e-16,
                                         7.12350102458129e-21,
                                         2.58678856969554e-27,
                                         6.94679764736036e-37,
                                         8.03689789586121e-55,
                                         0.145349957569077,
                                         0.373087751904025,
                                         0.246947213089943,
                                         0.0660540390811339,
                                         0.00775037650720941,
                                         0.000390866621202418,
                                         7.79828964869045e-06,
                                         5.30720448806936e-08,
                                         9.68908101833769e-11,
                                         3.19985991394455e-14,
                                         9.60917814237358e-19,
                                         6.83828045083709e-25,
                                         4.67981035968187e-34,
                                         2.90961969868928e-51,
                                         0.0120444892025923,
                                         0.166145522018249,
                                         0.280245991400238,
                                         0.146902058043008,
                                         0.0297535769942336,
                                         0.00242104846299312,
                                         7.53281689014861e-05,
                                         7.91170826716442e-07,
                                         2.25252791178639e-09,
                                         1.20026481238921e-12,
                                         6.22186001539923e-17,
                                         8.67709036394348e-23,
                                         1.51325841558601e-31,
                                         5.05621162833611e-48,
                                         0.000637657099245308,
                                         0.0472706554925245,
                                         0.203188921137732,
                                         0.208728482862839,
                                         0.0729761425566839,
                                         0.00958084284672246,
                                         0.000464879927275267,
                                         7.53529125908491e-06,
                                         3.34567006196628e-08,
                                         2.87639495233811e-11,
                                         2.57382824231548e-15,
                                         7.03439364326742e-21,
                                         3.1262469164719e-29,
                                         5.61357582389042e-45,
                                         2.42182144184754e-05,
                                         0.00964829817980252,
                                         0.105685826413763,
                                         0.21276082270679,
                                         0.128404052462859,
                                         0.0271994459970423,
                                         0.00205816546448536,
                                         5.14856180502802e-05,
                                         3.56493936419185e-07,
                                         4.94511128968768e-10,
                                         7.63826983479913e-14,
                                         4.09105504853076e-19,
                                         4.63329084687148e-27,
                                         4.47105540217392e-42,
                                         7.02398607001777e-07,
                                         0.0015038219167516,
                                         0.0419778384634852,
                                         0.165610628910311,
                                         0.172529418437933,
                                         0.0589661809775143,
                                         0.00695835176072204,
                                         0.00026863241619851,
                                         2.90073319880862e-06,
                                         6.49217506983105e-09,
                                         1.73099985646428e-12,
                                         1.8168981381084e-17,
                                         5.2437557068592e-25,
                                         2.71936253867015e-39,
                                         1.61679373688464e-08,
                                         0.000186025104837245,
                                         0.0132328335925173,
                                         0.102309086448811,
                                         0.183982722591617,
                                         0.101455453997904,
                                         0.0186707570335148,
                                         0.00111239841408237,
                                         1.87323765180033e-05,
                                         6.76447086031514e-08,
                                         3.11335412700219e-11,
                                         6.40405851205759e-16,
                                         4.71004214724756e-23,
                                         1.3126641502591e-36,
                                         3.03041711663018e-10,
                                         1.87380123833713e-05,
                                         0.00339674109819898,
                                         0.0514655979178522,
                                         0.159759885222198,
                                         0.142142716769712,
                                         0.0407938120794225,
                                         0.00375093138169771,
                                         9.85042088123128e-05,
                                         5.7392382479977e-07,
                                         4.55970586168669e-10,
                                         1.83804787184292e-14,
                                         3.44495815958311e-21,
                                         5.15961093148749e-34,
                                         4.70844129201801e-12,
                                         1.56459681726927e-06,
                                         0.000722768135862699,
                                         0.021460843910109,
                                         0.114996704703481,
                                         0.165082403094912,
                                         0.0738845464411446,
                                         0.0104844302526135,
                                         0.000429381820418965,
                                         4.0364691175446e-06,
                                         5.53569420086004e-09,
                                         4.37305865631104e-13,
                                         2.08867114489972e-19,
                                         1.68115300437046e-31,
                                         6.14151904460694e-14,
                                         1.09674397462724e-07,
                                         0.000129109869957373,
                                         0.00751277639168494,
                                         0.069490741386495,
                                         0.160953656192964,
                                         0.112340622529099,
                                         0.0246022282989706,
                                         0.00157129023870486,
                                         2.38326814623496e-05,
                                         5.64197617673673e-08,
                                         8.73449258955435e-12,
                                         1.06311458466645e-17,
                                         4.59855538757789e-29,
                                         6.7855962769606e-16,
                                         6.51210871990558e-09,
                                         1.95359016490601e-05,
                                         0.00222775640250118,
                                         0.0355698525924287,
                                         0.132927391187081,
                                         0.144688365775883,
                                         0.048900986941103,
                                         0.0048706031232498,
                                         0.000119194921917174,
                                         4.87084125361038e-07,
                                         1.47775905609877e-10,
                                         4.5835677611748e-16,
                                         1.06548949723306e-26,
                                         6.38967577864938e-18,
                                         3.29546435970254e-10,
                                         2.5193358573542e-06,
                                         0.000563006656262045,
                                         0.0155172864358429,
                                         0.0935635534738015,
                                         0.158821388264808,
                                         0.0828398675123763,
                                         0.0128673070485571,
                                         0.000508067236607066,
                                         3.58389514172777e-06,
                                         2.13082689395841e-09,
                                         1.68424717391122e-14,
                                         2.10404738250543e-24,
                                         5.14775468919202e-20,
                                         1.42678919398816e-11,
                                         2.77962936742449e-07,
                                         0.000121732789531809,
                                         0.00579158859423497,
                                         0.05634393178077,
                                         0.149153201633848,
                                         0.120063053043504,
                                         0.0290831070492329,
                                         0.00185281834369814,
                                         2.256081732763e-05,
                                         2.62869939575363e-08,
                                         5.29488112460062e-13,
                                         3.55475827722556e-22,
                                         3.55475827722556e-22,
                                         5.29488112460072e-13,
                                         2.62869939575363e-08,
                                         2.25608173276299e-05,
                                         0.00185281834369813,
                                         0.0290831070492329,
                                         0.120063053043503,
                                         0.149153201633848,
                                         0.05634393178077,
                                         0.00579158859423499,
                                         0.000121732789531809,
                                         2.77962936742449e-07,
                                         1.42678919398814e-11,
                                         5.14775468919202e-20,
                                         2.10404738250543e-24,
                                         1.68424717391126e-14,
                                         2.13082689395841e-09,
                                         3.58389514172775e-06,
                                         0.000508067236607065,
                                         0.0128673070485571,
                                         0.0828398675123761,
                                         0.158821388264809,
                                         0.0935635534738015,
                                         0.015517286435843,
                                         0.000563006656262047,
                                         2.5193358573542e-06,
                                         3.29546435970248e-10,
                                         6.38967577864938e-18,
                                         1.06548949723306e-26,
                                         4.58356776117491e-16,
                                         1.47775905609877e-10,
                                         4.87084125361035e-07,
                                         0.000119194921917174,
                                         0.0048706031232498,
                                         0.0489009869411029,
                                         0.144688365775883,
                                         0.132927391187081,
                                         0.0355698525924288,
                                         0.00222775640250118,
                                         1.95359016490601e-05,
                                         6.51210871990548e-09,
                                         6.7855962769606e-16,
                                         4.59855538757789e-29,
                                         1.06311458466647e-17,
                                         8.73449258955435e-12,
                                         5.6419761767367e-08,
                                         2.38326814623495e-05,
                                         0.00157129023870486,
                                         0.0246022282989706,
                                         0.112340622529099,
                                         0.160953656192964,
                                         0.0694907413864951,
                                         0.00751277639168497,
                                         0.000129109869957373,
                                         1.09674397462723e-07,
                                         6.14151904460694e-14,
                                         1.68115300437046e-31,
                                         2.08867114489978e-19,
                                         4.37305865631104e-13,
                                         5.53569420086001e-09,
                                         4.03646911754458e-06,
                                         0.000429381820418965,
                                         0.0104844302526135,
                                         0.0738845464411446,
                                         0.165082403094912,
                                         0.114996704703482,
                                         0.0214608439101091,
                                         0.000722768135862699,
                                         1.56459681726925e-06,
                                         4.70844129201801e-12,
                                         5.15961093148749e-34,
                                         3.44495815958321e-21,
                                         1.83804787184292e-14,
                                         4.55970586168666e-10,
                                         5.73923824799768e-07,
                                         9.85042088123128e-05,
                                         0.0037509313816977,
                                         0.0407938120794225,
                                         0.142142716769712,
                                         0.159759885222198,
                                         0.0514655979178523,
                                         0.00339674109819898,
                                         1.87380123833711e-05,
                                         3.03041711663018e-10,
                                         1.3126641502591e-36,
                                         4.7100421472477e-23,
                                         6.40405851205759e-16,
                                         3.11335412700217e-11,
                                         6.76447086031511e-08,
                                         1.87323765180033e-05,
                                         0.00111239841408236,
                                         0.0186707570335148,
                                         0.101455453997904,
                                         0.183982722591617,
                                         0.102309086448811,
                                         0.0132328335925173,
                                         0.000186025104837244,
                                         1.61679373688464e-08,
                                         2.71936253867015e-39,
                                         5.24375570685937e-25,
                                         1.81689813810841e-17,
                                         1.73099985646427e-12,
                                         6.49217506983102e-09,
                                         2.90073319880862e-06,
                                         0.000268632416198509,
                                         0.00695835176072204,
                                         0.0589661809775143,
                                         0.172529418437933,
                                         0.165610628910311,
                                         0.0419778384634852,
                                         0.00150382191675159,
                                         7.02398607001777e-07,
                                         4.47105540217392e-42,
                                         4.63329084687163e-27,
                                         4.09105504853076e-19,
                                         7.63826983479907e-14,
                                         4.94511128968765e-10,
                                         3.56493936419185e-07,
                                         5.148561805028e-05,
                                         0.00205816546448536,
                                         0.0271994459970423,
                                         0.128404052462859,
                                         0.21276082270679,
                                         0.105685826413763,
                                         0.00964829817980247,
                                         2.42182144184754e-05,
                                         5.61357582389042e-45,
                                         3.126246916472e-29,
                                         7.03439364326742e-21,
                                         2.57382824231546e-15,
                                         2.8763949523381e-11,
                                         3.34567006196628e-08,
                                         7.53529125908489e-06,
                                         0.000464879927275267,
                                         0.00958084284672246,
                                         0.072976142556684,
                                         0.20872848286284,
                                         0.203188921137732,
                                         0.0472706554925242,
                                         0.000637657099245308,
                                         5.05621162833611e-48,
                                         1.51325841558606e-31,
                                         8.67709036394348e-23,
                                         6.22186001539917e-17,
                                         1.2002648123892e-12,
                                         2.25252791178639e-09,
                                         7.9117082671644e-07,
                                         7.53281689014861e-05,
                                         0.00242104846299312,
                                         0.0297535769942337,
                                         0.146902058043008,
                                         0.280245991400238,
                                         0.166145522018249,
                                         0.0120444892025923,
                                         2.90961969868928e-51,
                                         4.67981035968204e-34,
                                         6.83828045083709e-25,
                                         9.6091781423735e-19,
                                         3.19985991394454e-14,
                                         9.68908101833769e-11,
                                         5.30720448806935e-08,
                                         7.79828964869045e-06,
                                         0.000390866621202418,
                                         0.00775037650720941,
                                         0.0660540390811339,
                                         0.246947213089943,
                                         0.373087751904025,
                                         0.145349957569077,
                                         8.03689789586121e-55,
                                         6.94679764736063e-37,
                                         2.58678856969554e-27,
                                         7.12350102458122e-21,
                                         4.09473777313694e-16,
                                         2.00048929181175e-12,
                                         1.70884477700461e-09,
                                         3.87509675586485e-07,
                                         3.0289696631282e-05,
                                         0.000969053276762979,
                                         0.0142564736381991,
                                         0.104450395023071,
                                         0.402137506243743,
                                         0.84194295904031,

                                         0.860238246234043,
                                         0.450840528256468,
                                         0.138999401625734,
                                         0.0245647361969185,
                                         0.00239035505533316,
                                         0.000120780876278278,
                                         2.91460572404875e-06,
                                         2.98023223876953e-08,
                                         1.08463043827456e-10,
                                         1.07982598265378e-13,
                                         1.93368915146975e-17,
                                         3.02365254185309e-22,
                                         1.01291619408974e-28,
                                         2.57362413908901e-38,
                                         2.88767755186243e-56,
                                         0.129896052753132,
                                         0.364941998077478,
                                         0.285400916029404,
                                         0.0981433559655246,
                                         0.01631997138404,
                                         0.00131217696244702,
                                         4.84560379890612e-05,
                                         7.45058059692383e-07,
                                         4.07749763376521e-09,
                                         6.21211601719469e-12,
                                         1.77014849240899e-15,
                                         4.73002134932179e-20,
                                         3.08326675230835e-26,
                                         1.98712068032889e-35,
                                         1.19523006100528e-52,
                                         0.0094148715259538,
                                         0.141796652551185,
                                         0.281280115745222,
                                         0.188213573987589,
                                         0.0534832276833479,
                                         0.00684270596670149,
                                         0.000386684911495038,
                                         8.94069671630859e-06,
                                         7.35778146730662e-08,
                                         1.71540463879176e-10,
                                         7.77810811909539e-14,
                                         3.55169411649315e-18,
                                         4.50494945411345e-24,
                                         7.36452265248984e-33,
                                         2.37462784218573e-49,
                                         0.000435971577816907,
                                         0.0351992529676531,
                                         0.177112020184184,
                                         0.230603722751516,
                                         0.111980179667067,
                                         0.0227975766098597,
                                         0.0019714777792035,
                                         6.85453414916992e-05,
                                         8.48252966383529e-07,
                                         3.02634849796382e-09,
                                         2.18355224301316e-12,
                                         1.70385779710204e-16,
                                         4.20527189299063e-22,
                                         1.74377439683141e-30,
                                         3.01415053831287e-46,
                                         1.44829844640783e-05,
                                         0.00626840463273377,
                                         0.0800042668243093,
                                         0.202692553865064,
                                         0.168197996865418,
                                         0.0544885934325762,
                                         0.00721078679788419,
                                         0.000376999378204346,
                                         7.0155221084186e-06,
                                         3.83025248656684e-08,
                                         4.39753502153227e-11,
                                         5.86390930137625e-15,
                                         2.81614068301069e-20,
                                         2.96204805080677e-28,
                                         2.74467186435659e-43,
                                         3.6740456929272e-07,
                                         0.000852446633860023,
                                         0.0275971899843068,
                                         0.136049163623015,
                                         0.192924365042804,
                                         0.0994509901667782,
                                         0.0201400258804199,
                                         0.00158339738845825,
                                         4.43079042986419e-05,
                                         3.7018811347935e-07,
                                         6.76303488130149e-10,
                                         1.54108908174734e-13,
                                         1.44012839566935e-18,
                                         3.8422047359055e-26,
                                         1.90854552089254e-40,
                                         7.39708307917675e-09,
                                         9.20040260557724e-05,
                                         0.00755520115841079,
                                         0.0724741970887544,
                                         0.175623567428219,
                                         0.144059558886598,
                                         0.0446443853959383,
                                         0.00527799129486084,
                                         0.000222091468281997,
                                         2.83953346318331e-06,
                                         8.25474007556726e-09,
                                         3.21438355629941e-12,
                                         5.84490606077934e-17,
                                         3.95547242651005e-24,
                                         1.0532805635433e-37,
                                         1.21269965872766e-10,
                                         8.08580530828848e-06,
                                         0.00168424015056871,
                                         0.0314374827993588,
                                         0.130183314768691,
                                         0.169922882027837,
                                         0.0805843094139718,
                                         0.0143259763717651,
                                         0.000906482360096129,
                                         1.77356997588159e-05,
                                         8.20430871628701e-08,
                                         5.45939436926089e-11,
                                         1.93165993972465e-15,
                                         3.31583594747889e-22,
                                         4.73328258190324e-35,
                                         1.64805982040517e-12,
                                         5.89069700768843e-07,
                                         0.000311235378388231,
                                         0.0113041802536521,
                                         0.0799935210573792,
                                         0.16614550933397,
                                         0.120576059171917,
                                         0.0322334468364716,
                                         0.00306700014412245,
                                         9.18283164935656e-05,
                                         6.75939057987652e-07,
                                         7.68631527835227e-10,
                                         5.29188951001429e-14,
                                         2.30417078988504e-20,
                                         1.76322507439489e-32,
                                         1.88025426018147e-14,
                                         3.60274881371923e-08,
                                         4.82833942821277e-05,
                                         0.00341235555804958,
                                         0.0412646527935292,
                                         0.136379546908328,
                                         0.151459171372887,
                                         0.0608853995800018,
                                         0.00871148371996178,
                                         0.000399143269504142,
                                         4.67516544397631e-06,
                                         9.08480929584632e-09,
                                         1.21706775537556e-12,
                                         1.34418855041924e-18,
                                         5.51412957399528e-30,
                                         1.81707880216902e-16,
                                         1.86644352421061e-09,
                                         6.34483304921595e-06,
                                         0.000872535389996287,
                                         0.0180308062940919,
                                         0.0948251306617846,
                                         0.161154945686805,
                                         0.0974166393280029,
                                         0.0209596503295544,
                                         0.00146958452193474,
                                         2.73905023251609e-05,
                                         9.09550897485798e-08,
                                         2.37100415918425e-11,
                                         6.64231225060206e-17,
                                         1.46069570358875e-27,
                                         1.49661307081198e-18,
                                         8.24089373676043e-11,
                                         7.10593443943137e-07,
                                         0.000190147410765339,
                                         0.00671476304350417,
                                         0.0561922274224677,
                                         0.146140399759594,
                                         0.132840871810913,
                                         0.0429788239926013,
                                         0.00461146502592251,
                                         0.000136767088284511,
                                         7.76098402454781e-07,
                                         3.93666506644331e-10,
                                         2.79741548565026e-15,
                                         3.29777580813943e-25,
                                         1.05461397369701e-20,
                                         3.11302057146126e-12,
                                         6.80879736256454e-08,
                                         3.54524290694521e-05,
                                         0.00213941211601657,
                                         0.0284890036651852,
                                         0.113382273122815,
                                         0.154981017112732,
                                         0.0754003286055898,
                                         0.0123803090172638,
                                         0.000584267085078581,
                                         5.66571582631888e-06,
                                         5.59207300637691e-09,
                                         1.00795902426863e-13,
                                         6.36987220511385e-23,
                                         6.36987220511308e-23,
                                         1.00795902426863e-13,
                                         5.59207300637686e-09,
                                         5.66571582631891e-06,
                                         0.000584267085078581,
                                         0.0123803090172638,
                                         0.0754003286055898,
                                         0.154981017112732,
                                         0.113382273122815,
                                         0.0284890036651852,
                                         0.00213941211601657,
                                         3.54524290694519e-05,
                                         6.80879736256459e-08,
                                         3.11302057146126e-12,
                                         1.05461397369712e-20,
                                         3.29777580813901e-25,
                                         2.79741548565026e-15,
                                         3.93666506644327e-10,
                                         7.76098402454785e-07,
                                         0.000136767088284511,
                                         0.00461146502592251,
                                         0.0429788239926013,
                                         0.132840871810913,
                                         0.146140399759594,
                                         0.0561922274224678,
                                         0.00671476304350417,
                                         0.000190147410765338,
                                         7.10593443943142e-07,
                                         8.24089373676043e-11,
                                         1.49661307081214e-18,
                                         1.46069570358855e-27,
                                         6.64231225060206e-17,
                                         2.37100415918422e-11,
                                         9.09550897485803e-08,
                                         2.73905023251609e-05,
                                         0.00146958452193475,
                                         0.0209596503295544,
                                         0.0974166393280029,
                                         0.161154945686805,
                                         0.0948251306617846,
                                         0.0180308062940919,
                                         0.000872535389996283,
                                         6.344833049216e-06,
                                         1.86644352421061e-09,
                                         1.81707880216919e-16,
                                         5.51412957399447e-30,
                                         1.34418855041924e-18,
                                         1.21706775537554e-12,
                                         9.08480929584638e-09,
                                         4.67516544397631e-06,
                                         0.000399143269504142,
                                         0.00871148371996178,
                                         0.0608853995800018,
                                         0.151459171372887,
                                         0.136379546908328,
                                         0.0412646527935292,
                                         0.00341235555804957,
                                         4.8283394282128e-05,
                                         3.60274881371924e-08,
                                         1.88025426018163e-14,
                                         1.76322507439461e-32,
                                         2.30417078988504e-20,
                                         5.29188951001422e-14,
                                         7.68631527835232e-10,
                                         6.75939057987652e-07,
                                         9.18283164935656e-05,
                                         0.00306700014412245,
                                         0.0322334468364716,
                                         0.120576059171917,
                                         0.16614550933397,
                                         0.0799935210573792,
                                         0.011304180253652,
                                         0.000311235378388233,
                                         5.89069700768843e-07,
                                         1.64805982040529e-12,
                                         4.73328258190245e-35,
                                         3.31583594747889e-22,
                                         1.93165993972463e-15,
                                         5.45939436926093e-11,
                                         8.20430871628701e-08,
                                         1.77356997588159e-05,
                                         0.000906482360096129,
                                         0.0143259763717651,
                                         0.0805843094139718,
                                         0.169922882027837,
                                         0.130183314768691,
                                         0.0314374827993587,
                                         0.00168424015056872,
                                         8.08580530828848e-06,
                                         1.21269965872774e-10,
                                         1.05328056354311e-37,
                                         3.95547242651005e-24,
                                         5.84490606077926e-17,
                                         3.21438355629944e-12,
                                         8.25474007556726e-09,
                                         2.83953346318331e-06,
                                         0.000222091468281997,
                                         0.00527799129486084,
                                         0.0446443853959383,
                                         0.144059558886598,
                                         0.175623567428219,
                                         0.0724741970887542,
                                         0.00755520115841083,
                                         9.20040260557724e-05,
                                         7.39708307917716e-09,
                                         1.90854552089219e-40,
                                         3.8422047359055e-26,
                                         1.44012839566933e-18,
                                         1.54108908174735e-13,
                                         6.76303488130149e-10,
                                         3.7018811347935e-07,
                                         4.43079042986419e-05,
                                         0.00158339738845825,
                                         0.0201400258804199,
                                         0.0994509901667782,
                                         0.192924365042804,
                                         0.136049163623015,
                                         0.0275971899843069,
                                         0.000852446633860023,
                                         3.67404569292737e-07,
                                         2.74467186435605e-43,
                                         2.96204805080677e-28,
                                         2.81614068301064e-20,
                                         5.8639093013763e-15,
                                         4.39753502153227e-11,
                                         3.83025248656684e-08,
                                         7.0155221084186e-06,
                                         0.000376999378204346,
                                         0.00721078679788419,
                                         0.0544885934325762,
                                         0.168197996865418,
                                         0.202692553865064,
                                         0.0800042668243096,
                                         0.00626840463273377,
                                         1.44829844640789e-05,
                                         3.01415053831226e-46,
                                         1.74377439683141e-30,
                                         4.20527189299056e-22,
                                         1.70385779710205e-16,
                                         2.18355224301316e-12,
                                         3.02634849796382e-09,
                                         8.48252966383528e-07,
                                         6.85453414916992e-05,
                                         0.0019714777792035,
                                         0.0227975766098597,
                                         0.111980179667067,
                                         0.230603722751516,
                                         0.177112020184184,
                                         0.0351992529676531,
                                         0.000435971577816919,
                                         2.37462784218523e-49,
                                         7.36452265248984e-33,
                                         4.50494945411337e-24,
                                         3.55169411649318e-18,
                                         7.77810811909539e-14,
                                         1.71540463879176e-10,
                                         7.35778146730662e-08,
                                         8.94069671630859e-06,
                                         0.000386684911495038,
                                         0.00684270596670149,
                                         0.0534832276833479,
                                         0.188213573987589,
                                         0.281280115745222,
                                         0.141796652551185,
                                         0.00941487152595397,
                                         1.19523006100501e-52,
                                         1.98712068032889e-35,
                                         3.08326675230829e-26,
                                         4.73002134932184e-20,
                                         1.77014849240899e-15,
                                         6.21211601719469e-12,
                                         4.07749763376521e-09,
                                         7.45058059692383e-07,
                                         4.84560379890612e-05,
                                         0.00131217696244702,
                                         0.01631997138404,
                                         0.0981433559655246,
                                         0.285400916029405,
                                         0.364941998077478,
                                         0.129896052753133,
                                         2.88767755186177e-56,
                                         2.57362413908901e-38,
                                         1.01291619408972e-28,
                                         3.02365254185312e-22,
                                         1.93368915146975e-17,
                                         1.07982598265378e-13,
                                         1.08463043827456e-10,
                                         2.98023223876953e-08,
                                         2.91460572404875e-06,
                                         0.000120780876278278,
                                         0.00239035505533316,
                                         0.0245647361969185,
                                         0.138999401625734,
                                         0.450840528256468,
                                         0.860238246234043,

                                         0.875604634042361,
                                         0.495299270391703,
                                         0.175749874901215,
                                         0.0383424380972045,
                                         0.00498749604112081,
                                         0.00037009493903445,
                                         1.4726266584228e-05,
                                         2.88221999777926e-07,
                                         2.45648720228769e-09,
                                         7.64459594837097e-12,
                                         6.66647379150723e-15,
                                         1.06991935573343e-18,
                                         1.53273030758877e-23,
                                         4.80519350914262e-30,
                                         1.16650910559607e-39,
                                         1.27625955061649e-57,
                                         0.116625440787108,
                                         0.352929950086461,
                                         0.316451870829828,
                                         0.133564540553449,
                                         0.0294497296201371,
                                         0.0034393508137977,
                                         0.000206368373783664,
                                         5.9551215590057e-06,
                                         7.43072209837532e-08,
                                         3.40944798736559e-10,
                                         4.48345549969926e-13,
                                         1.13248462292125e-16,
                                         2.75001025229606e-21,
                                         1.66793167451774e-27,
                                         1.02316888373576e-36,
                                         5.98871679079305e-54,
                                         0.0074562383486678,
                                         0.120712036973873,
                                         0.273502655816017,
                                         0.223328352126523,
                                         0.0834682478789606,
                                         0.0153419669682365,
                                         0.00138814509557967,
                                         5.90602623975001e-05,
                                         1.07891882395698e-06,
                                         7.29885675500433e-09,
                                         1.44734074509196e-11,
                                         5.75380077811082e-15,
                                         2.36834037151824e-19,
                                         2.77898925708582e-25,
                                         4.30772283404397e-34,
                                         1.34886903027218e-50,
                                         0.000304559115001306,
                                         0.0263777639582801,
                                         0.151022189889407,
                                         0.238573325073206,
                                         0.151142505625034,
                                         0.0437230999680921,
                                         0.00596556963407524,
                                         0.000374218667997225,
                                         1.00085659783598e-05,
                                         9.98276727340722e-08,
                                         2.98506676060394e-10,
                                         1.86768083658927e-13,
                                         1.30310454242489e-17,
                                         2.95815424187831e-23,
                                         1.15870669361975e-31,
                                         1.94102498920071e-47,
                                         8.92441031419771e-06,
                                         0.00413505685191441,
                                         0.0598240880792672,
                                         0.182833601337616,
                                         0.19633966531004,
                                         0.0893916506041817,
                                         0.0183918359312527,
                                         0.00170102866902506,
                                         6.66056469181713e-05,
                                         9.79497114572844e-07,
                                         4.4166543944218e-09,
                                         4.34917229656688e-12,
                                         5.14363931751497e-16,
                                         2.25897211574317e-21,
                                         2.23591550457808e-29,
                                         2.00377326577751e-44,
                                         1.99698157547826e-07,
                                         0.00049500728803627,
                                         0.0180966427058583,
                                         0.106998267323142,
                                         0.194767314416281,
                                         0.139562765989523,
                                         0.0432996962301175,
                                         0.00590451756357978,
                                         0.000338482987947373,
                                         7.3390859904449e-06,
                                         4.99021647892506e-08,
                                         7.73387528022322e-11,
                                         1.55041641197878e-14,
                                         1.31730859732766e-19,
                                         3.29476057130855e-27,
                                         1.5796200233036e-41,
                                         3.54648429864475e-09,
                                         4.70295860577151e-05,
                                         0.00434459593459297,
                                         0.0496966290243032,
                                         0.153339329691932,
                                         0.172930514904895,
                                         0.0809047592633448,
                                         0.0162662206519021,
                                         0.00136518684953718,
                                         4.36425628914284e-05,
                                         4.47481216544177e-07,
                                         1.09148348842169e-09,
                                         3.70898999183329e-13,
                                         6.09668332237296e-18,
                                         3.85320209360878e-25,
                                         9.88293951342592e-39,
                                         5.12860014758399e-11,
                                         3.63837563718236e-06,
                                         0.000849332194671166,
                                         0.0187955021402973,
                                         0.098303246455014,
                                         0.174481954212473,
                                         0.12309491308153,
                                         0.0364893153964478,
                                         0.00448357214096109,
                                         0.000211327167594921,
                                         3.26743570009567e-06,
                                         1.25433630346791e-08,
                                         7.22503211058391e-12,
                                         2.2976113284488e-16,
                                         3.66941245485569e-23,
                                         5.03496504869258e-36,
                                         6.14789719721249e-13,
                                         2.3333015570767e-07,
                                         0.000137636220736822,
                                         0.00589261262241142,
                                         0.0522407156719899,
                                         0.145933956163081,
                                         0.155250532777909,
                                         0.0678534178731517,
                                         0.0122062759784889,
                                         0.000848256849660106,
                                         1.97772656368247e-05,
                                         1.19491708532367e-07,
                                         1.16667759820564e-10,
                                         7.17771892608106e-15,
                                         2.89666472781976e-21,
                                         2.12634504809281e-33,
                                         6.1869733719562e-15,
                                         1.25619800859355e-08,
                                         1.872456707643e-05,
                                         0.00155090695818575,
                                         0.0233063502659085,
                                         0.102467487905725,
                                         0.164380382349984,
                                         0.105925771461342,
                                         0.0278975523829649,
                                         0.00285840203298141,
                                         0.000100496129921289,
                                         9.55620468993992e-07,
                                         1.58156057375164e-09,
                                         1.88243602419756e-13,
                                         1.91965783235293e-19,
                                         7.5386731438407e-31,
                                         5.27404058746512e-17,
                                         5.72873721532941e-10,
                                         2.15776628147989e-06,
                                         0.00034576192537128,
                                         0.00880750687693264,
                                         0.0609437807647177,
                                         0.147428159580205,
                                         0.14006999510539,
                                         0.0540085588988051,
                                         0.00815892272072854,
                                         0.00043255964383987,
                                         6.47361231141837e-06,
                                         1.81607747519738e-08,
                                         4.18184155327898e-12,
                                         1.07761341709106e-17,
                                         2.26396473864641e-28,
                                         3.83166277436453e-19,
                                         2.22657994358446e-11,
                                         2.11921788478179e-07,
                                         6.56972487380642e-05,
                                         0.00283667972153464,
                                         0.0308923742917361,
                                         0.112691067544218,
                                         0.157858213132042,
                                         0.0891123030710485,
                                         0.0198481904463532,
                                         0.00158679656505483,
                                         3.73754578696996e-05,
                                         1.77730312704528e-07,
                                         7.91759919904549e-11,
                                         5.1556181762913e-16,
                                         5.79459375679467e-26,
                                         2.3816575034278e-21,
                                         7.40399130693813e-13,
                                         1.78071753127761e-08,
                                         1.0679856394379e-05,
                                         0.000781656300617244,
                                         0.0133974269874889,
                                         0.0736964821835963,
                                         0.152207995760013,
                                         0.125794299907973,
                                         0.0413101938493879,
                                         0.00498017633339493,
                                         0.000184618225412216,
                                         1.4881162318116e-06,
                                         1.28253040182023e-09,
                                         2.11031224038084e-14,
                                         1.26889166796125e-23,
                                         1.26889166796143e-23,
                                         2.11031224038083e-14,
                                         1.28253040182022e-09,
                                         1.48811623181161e-06,
                                         0.000184618225412216,
                                         0.00498017633339493,
                                         0.041310193849388,
                                         0.125794299907973,
                                         0.152207995760013,
                                         0.0736964821835962,
                                         0.0133974269874889,
                                         0.000781656300617244,
                                         1.06798563943789e-05,
                                         1.78071753127762e-08,
                                         7.40399130693813e-13,
                                         2.3816575034275e-21,
                                         5.79459375679552e-26,
                                         5.1556181762913e-16,
                                         7.9175991990454e-11,
                                         1.77730312704529e-07,
                                         3.73754578696996e-05,
                                         0.00158679656505483,
                                         0.0198481904463533,
                                         0.0891123030710486,
                                         0.157858213132041,
                                         0.112691067544218,
                                         0.0308923742917361,
                                         0.00283667972153464,
                                         6.56972487380638e-05,
                                         2.11921788478181e-07,
                                         2.22657994358446e-11,
                                         3.83166277436409e-19,
                                         2.26396473864676e-28,
                                         1.07761341709106e-17,
                                         4.18184155327893e-12,
                                         1.8160774751974e-08,
                                         6.47361231141837e-06,
                                         0.00043255964383987,
                                         0.00815892272072856,
                                         0.0540085588988051,
                                         0.14006999510539,
                                         0.147428159580205,
                                         0.0609437807647177,
                                         0.00880750687693264,
                                         0.000345761925371279,
                                         2.15776628147991e-06,
                                         5.72873721532941e-10,
                                         5.27404058746457e-17,
                                         7.53867314384196e-31,
                                         1.91965783235293e-19,
                                         1.88243602419753e-13,
                                         1.58156057375165e-09,
                                         9.55620468993992e-07,
                                         0.000100496129921289,
                                         0.00285840203298142,
                                         0.0278975523829649,
                                         0.105925771461341,
                                         0.164380382349984,
                                         0.102467487905725,
                                         0.0233063502659085,
                                         0.00155090695818575,
                                         1.87245670764301e-05,
                                         1.25619800859355e-08,
                                         6.18697337195562e-15,
                                         2.12634504809318e-33,
                                         2.89666472781976e-21,
                                         7.17771892608096e-15,
                                         1.16667759820565e-10,
                                         1.19491708532367e-07,
                                         1.97772656368247e-05,
                                         0.000848256849660108,
                                         0.0122062759784889,
                                         0.0678534178731515,
                                         0.155250532777909,
                                         0.145933956163081,
                                         0.0522407156719899,
                                         0.00589261262241139,
                                         0.000137636220736823,
                                         2.3333015570767e-07,
                                         6.14789719721197e-13,
                                         5.03496504869353e-36,
                                         3.66941245485569e-23,
                                         2.29761132844877e-16,
                                         7.22503211058397e-12,
                                         1.25433630346791e-08,
                                         3.26743570009567e-06,
                                         0.000211327167594921,
                                         0.00448357214096109,
                                         0.0364893153964477,
                                         0.12309491308153,
                                         0.174481954212473,
                                         0.098303246455014,
                                         0.0187955021402972,
                                         0.000849332194671171,
                                         3.63837563718236e-06,
                                         5.12860014758361e-11,
                                         9.88293951342789e-39,
                                         3.85320209360878e-25,
                                         6.09668332237286e-18,
                                         3.70898999183332e-13,
                                         1.09148348842169e-09,
                                         4.47481216544177e-07,
                                         4.36425628914285e-05,
                                         0.00136518684953718,
                                         0.0162662206519021,
                                         0.0809047592633447,
                                         0.172930514904895,
                                         0.153339329691932,
                                         0.049696629024303,
                                         0.00434459593459299,
                                         4.70295860577151e-05,
                                         3.54648429864453e-09,
                                         1.57962002330393e-41,
                                         3.29476057130855e-27,
                                         1.31730859732764e-19,
                                         1.5504164119788e-14,
                                         7.73387528022322e-11,
                                         4.99021647892506e-08,
                                         7.33908599044492e-06,
                                         0.000338482987947372,
                                         0.00590451756357976,
                                         0.0432996962301175,
                                         0.139562765989523,
                                         0.194767314416281,
                                         0.106998267323142,
                                         0.0180966427058584,
                                         0.00049500728803627,
                                         1.99698157547816e-07,
                                         2.00377326577795e-44,
                                         2.23591550457808e-29,
                                         2.25897211574313e-21,
                                         5.14363931751502e-16,
                                         4.34917229656688e-12,
                                         4.4166543944218e-09,
                                         9.79497114572847e-07,
                                         6.66056469181711e-05,
                                         0.00170102866902506,
                                         0.0183918359312527,
                                         0.0893916506041817,
                                         0.19633966531004,
                                         0.182833601337616,
                                         0.0598240880792674,
                                         0.00413505685191441,
                                         8.92441031419734e-06,
                                         1.94102498920115e-47,
                                         1.15870669361975e-31,
                                         2.95815424187826e-23,
                                         1.3031045424249e-17,
                                         1.86768083658927e-13,
                                         2.98506676060394e-10,
                                         9.98276727340726e-08,
                                         1.00085659783598e-05,
                                         0.000374218667997224,
                                         0.00596556963407524,
                                         0.0437230999680921,
                                         0.151142505625034,
                                         0.238573325073205,
                                         0.151022189889408,
                                         0.0263777639582801,
                                         0.000304559115001297,
                                         1.34886903027251e-50,
                                         4.30772283404397e-34,
                                         2.77898925708576e-25,
                                         2.36834037151826e-19,
                                         5.75380077811082e-15,
                                         1.44734074509196e-11,
                                         7.29885675500436e-09,
                                         1.07891882395698e-06,
                                         5.90602623974999e-05,
                                         0.00138814509557967,
                                         0.0153419669682365,
                                         0.0834682478789606,
                                         0.223328352126523,
                                         0.273502655816017,
                                         0.120712036973873,
                                         0.00745623834866765,
                                         5.98871679079456e-54,
                                         1.02316888373576e-36,
                                         1.66793167451771e-27,
                                         2.75001025229609e-21,
                                         1.13248462292125e-16,
                                         4.48345549969926e-13,
                                         3.40944798736561e-10,
                                         7.4307220983753e-08,
                                         5.95512155900567e-06,
                                         0.000206368373783664,
                                         0.0034393508137977,
                                         0.0294497296201371,
                                         0.133564540553449,
                                         0.316451870829828,
                                         0.352929950086461,
                                         0.116625440787107,
                                         1.27625955061683e-57,
                                         1.16650910559607e-39,
                                         4.80519350914252e-30,
                                         1.53273030758879e-23,
                                         1.06991935573343e-18,
                                         6.66647379150723e-15,
                                         7.644595948371e-12,
                                         2.45648720228768e-09,
                                         2.88221999777924e-07,
                                         1.4726266584228e-05,
                                         0.00037009493903445,
                                         0.00498749604112081,
                                         0.0383424380972045,
                                         0.175749874901215,
                                         0.495299270391703,
                                         0.875604634042361,

                                         0.888620515461475,
                                         0.535643653877189,
                                         0.213588276333434,
                                         0.0554601604990223,
                                         0.00915608142958041,
                                         0.000928971436661083,
                                         5.52708635295998e-05,
                                         1.80860632446068e-06,
                                         2.98023223876953e-08,
                                         2.18599098234762e-10,
                                         5.97708702413844e-13,
                                         4.67058446830339e-16,
                                         6.84443092307629e-20,
                                         9.11675447536271e-25,
                                         2.70471502790669e-31,
                                         6.32139028646566e-41,
                                         6.77230511553041e-59,
                                         0.105180965102056,
                                         0.338605055665619,
                                         0.340109829605758,
                                         0.170041722716849,
                                         0.0472709970924957,
                                         0.00748164546486213,
                                         0.000663431352679713,
                                         3.15192755303932e-05,
                                         7.45058059692383e-07,
                                         7.8396414760461e-09,
                                         3.11221688462304e-11,
                                         3.62456994248816e-14,
                                         8.28575801302134e-18,
                                         1.85842869309985e-22,
                                         1.06159718585127e-28,
                                         6.24992992198929e-38,
                                         3.57598524171279e-55,
                                         0.00597584335394982,
                                         0.102742978075738,
                                         0.259957405558445,
                                         0.250248283778612,
                                         0.117144287978023,
                                         0.0289223198834589,
                                         0.00382240738018695,
                                         0.000263663277039244,
                                         8.94069671630859e-06,
                                         1.34953848873293e-07,
                                         7.77841960632636e-10,
                                         1.35015296938313e-12,
                                         4.81469059714834e-16,
                                         1.81841406829783e-20,
                                         2.00004257462271e-26,
                                         2.96605314410606e-35,
                                         9.06350453912495e-52,
                                         0.000216913505635745,
                                         0.0199175628760323,
                                         0.126943520797488,
                                         0.235294651066918,
                                         0.185469630280914,
                                         0.0714322716005642,
                                         0.0140702976688763,
                                         0.00140912136502426,
                                         6.85453414916992e-05,
                                         1.48422479866999e-06,
                                         1.24204753016473e-08,
                                         3.21317813041337e-11,
                                         1.78743335091869e-14,
                                         1.13675009325821e-18,
                                         2.40737672686827e-24,
                                         8.99307144950311e-33,
                                         1.46764807963341e-48,
                                         5.64846042071143e-06,
                                         0.00276997820521161,
                                         0.0444708052015432,
                                         0.158711760654218,
                                         0.210659283959051,
                                         0.126564499348789,
                                         0.0371557222769883,
                                         0.00540260557039999,
                                         0.000376999378204346,
                                         1.17103587562165e-05,
                                         1.42279120487936e-07,
                                         5.48583495082187e-10,
                                         4.7604437576252e-13,
                                         5.09792359878606e-17,
                                         2.07876302336788e-22,
                                         1.95610997503173e-30,
                                         1.70491924436123e-45,
                                         1.12320799883724e-07,
                                         0.000294173202190108,
                                         0.0118966873284463,
                                         0.0817509419430416,
                                         0.182715336466665,
                                         0.171244231451034,
                                         0.0749263767320111,
                                         0.015817750657181,
                                         0.00158339738845825,
                                         7.05549222797414e-05,
                                         1.24460267227776e-06,
                                         7.15216084410746e-09,
                                         9.68169999259594e-12,
                                         1.74585530719887e-15,
                                         1.37073186670483e-20,
                                         3.24911506997296e-28,
                                         1.51241959710881e-42,
                                         1.77263671458523e-09,
                                         2.47947263152561e-05,
                                         0.00252584418933453,
                                         0.0334199561790907,
                                         0.125776311701295,
                                         0.183886332354026,
                                         0.119914916690447,
                                         0.0367549336740205,
                                         0.00527799129486084,
                                         0.000337375771147629,
                                         8.64071620097838e-06,
                                         7.40050341174523e-08,
                                         1.56273470981796e-10,
                                         4.7451793006194e-14,
                                         7.1734733714343e-19,
                                         4.28318045568507e-26,
                                         1.06480537350779e-39,
                                         2.27801252857197e-11,
                                         1.70173662332611e-06,
                                         0.000436680583524143,
                                         0.011124891973208,
                                         0.070501689426746,
                                         0.160790270763344,
                                         0.156274638939345,
                                         0.0695445941747329,
                                         0.0143259763717651,
                                         0.00131364276528766,
                                         4.88478630093365e-05,
                                         6.23536770542242e-07,
                                         2.05397763944696e-09,
                                         1.05020437393665e-12,
                                         3.05691446743047e-17,
                                         4.5977406242695e-24,
                                         6.1044281702412e-37,
                                         2.42671868239289e-13,
                                         9.68171578079608e-08,
                                         6.25818164103803e-05,
                                         0.00306981840879605,
                                         0.0327587370578291,
                                         0.1165459105973,
                                         0.168822666468719,
                                         0.109078226489492,
                                         0.0322334468364716,
                                         0.00424001725146979,
                                         0.00022891189149382,
                                         4.35501677936267e-06,
                                         2.23786107103679e-08,
                                         1.92673495011355e-11,
                                         1.07985151347773e-15,
                                         4.09118878269903e-22,
                                         2.90099311920204e-34,
                                         2.17023444829915e-15,
                                         4.62419571351778e-09,
                                         7.52933014196665e-06,
                                         0.000711137308203969,
                                         0.0127784644769773,
                                         0.0709182819907792,
                                         0.153107653475571,
                                         0.143627186392179,
                                         0.0608853995800018,
                                         0.0114889902988282,
                                         0.000900564934476245,
                                         2.55353308842965e-05,
                                         2.0468896288971e-07,
                                         2.96752250909369e-10,
                                         3.20234920989501e-14,
                                         3.05617674117694e-20,
                                         1.15737021991314e-31,
                                         1.64402131023078e-17,
                                         1.87082711204822e-10,
                                         7.6732277200671e-07,
                                         0.000139542921635629,
                                         0.00422224821329354,
                                         0.0365538347048714,
                                         0.117618765376753,
                                         0.160194945257426,
                                         0.0974166393280029,
                                         0.0263699722363116,
                                         0.00300106417292036,
                                         0.00012682553926732,
                                         1.5858762685182e-06,
                                         3.87150334657223e-09,
                                         8.04427497731608e-13,
                                         1.93384197801831e-18,
                                         3.9112130998268e-29,
                                         1.06141891770682e-19,
                                         6.45074232906218e-12,
                                         6.66466654409301e-08,
                                         2.33367702716157e-05,
                                         0.00118901529823912,
                                         0.0160578084251571,
                                         0.0770078367101004,
                                         0.152278838327499,
                                         0.132840871810913,
                                         0.0515841256442103,
                                         0.00852342458617899,
                                         0.000536847007658821,
                                         1.04718344119071e-05,
                                         4.30470948693954e-08,
                                         1.72220052558471e-11,
                                         1.0428986169554e-16,
                                         1.12649494054304e-26,
                                         5.86292610772506e-22,
                                         1.90297879908823e-13,
                                         4.95252844747732e-09,
                                         3.3390421866185e-06,
                                         0.000286470126598681,
                                         0.00603514649908962,
                                         0.0431361565076229,
                                         0.123845011430251,
                                         0.154981017112732,
                                         0.0863317790177204,
                                         0.0207110052810592,
                                         0.00194420730940796,
                                         5.91594936138686e-05,
                                         4.09502230450164e-07,
                                         3.15448700706709e-10,
                                         4.81184263105994e-15,
                                         2.77584510194392e-24,
                                         2.7758451019435e-24,
                                         4.81184263106008e-15,
                                         3.15448700706709e-10,
                                         4.09502230450167e-07,
                                         5.91594936138688e-05,
                                         0.00194420730940797,
                                         0.0207110052810592,
                                         0.0863317790177204,
                                         0.154981017112732,
                                         0.123845011430251,
                                         0.0431361565076229,
                                         0.0060351464990896,
                                         0.00028647012659868,
                                         3.33904218661848e-06,
                                         4.95252844747732e-09,
                                         1.90297879908818e-13,
                                         5.86292610772589e-22,
                                         1.12649494054285e-26,
                                         1.04289861695543e-16,
                                         1.72220052558471e-11,
                                         4.30470948693957e-08,
                                         1.04718344119072e-05,
                                         0.000536847007658823,
                                         0.00852342458617899,
                                         0.0515841256442103,
                                         0.132840871810913,
                                         0.152278838327499,
                                         0.0770078367101004,
                                         0.016057808425157,
                                         0.00118901529823912,
                                         2.33367702716156e-05,
                                         6.66466654409301e-08,
                                         6.45074232906202e-12,
                                         1.06141891770696e-19,
                                         3.91121309982611e-29,
                                         1.93384197801837e-18,
                                         8.04427497731608e-13,
                                         3.87150334657226e-09,
                                         1.58587626851821e-06,
                                         0.000126825539267321,
                                         0.00300106417292036,
                                         0.0263699722363116,
                                         0.0974166393280029,
                                         0.160194945257426,
                                         0.117618765376753,
                                         0.0365538347048713,
                                         0.00422224821329353,
                                         0.000139542921635628,
                                         7.6732277200671e-07,
                                         1.87082711204818e-10,
                                         1.64402131023097e-17,
                                         1.15737021991292e-31,
                                         3.05617674117705e-20,
                                         3.20234920989501e-14,
                                         2.96752250909372e-10,
                                         2.04688962889711e-07,
                                         2.55353308842966e-05,
                                         0.000900564934476245,
                                         0.0114889902988282,
                                         0.0608853995800018,
                                         0.143627186392179,
                                         0.153107653475571,
                                         0.070918281990779,
                                         0.0127784644769772,
                                         0.000711137308203966,
                                         7.52933014196665e-06,
                                         4.62419571351769e-09,
                                         2.17023444829938e-15,
                                         2.90099311920146e-34,
                                         4.09118878269918e-22,
                                         1.07985151347773e-15,
                                         1.92673495011357e-11,
                                         2.23786107103681e-08,
                                         4.35501677936268e-06,
                                         0.00022891189149382,
                                         0.00424001725146979,
                                         0.0322334468364716,
                                         0.109078226489492,
                                         0.168822666468719,
                                         0.116545910597299,
                                         0.032758737057829,
                                         0.00306981840879604,
                                         6.25818164103803e-05,
                                         9.68171578079591e-08,
                                         2.42671868239312e-13,
                                         6.1044281702399e-37,
                                         4.59774062426968e-24,
                                         3.05691446743047e-17,
                                         1.05020437393666e-12,
                                         2.05397763944697e-09,
                                         6.23536770542244e-07,
                                         4.88478630093365e-05,
                                         0.00131364276528766,
                                         0.0143259763717651,
                                         0.0695445941747329,
                                         0.156274638939345,
                                         0.160790270763344,
                                         0.0705016894267458,
                                         0.0111248919732079,
                                         0.000436680583524143,
                                         1.70173662332609e-06,
                                         2.27801252857216e-11,
                                         1.06480537350756e-39,
                                         4.28318045568525e-26,
                                         7.1734733714343e-19,
                                         4.74517930061945e-14,
                                         1.56273470981797e-10,
                                         7.40050341174526e-08,
                                         8.64071620097838e-06,
                                         0.000337375771147629,
                                         0.00527799129486084,
                                         0.0367549336740205,
                                         0.119914916690447,
                                         0.183886332354026,
                                         0.125776311701295,
                                         0.0334199561790906,
                                         0.00252584418933453,
                                         2.47947263152557e-05,
                                         1.77263671458535e-09,
                                         1.51241959710846e-42,
                                         3.24911506997311e-28,
                                         1.37073186670483e-20,
                                         1.74585530719888e-15,
                                         9.681699992596e-12,
                                         7.15216084410749e-09,
                                         1.24460267227776e-06,
                                         7.05549222797414e-05,
                                         0.00158339738845825,
                                         0.015817750657181,
                                         0.0749263767320111,
                                         0.171244231451033,
                                         0.182715336466664,
                                         0.0817509419430414,
                                         0.0118966873284463,
                                         0.000294173202190104,
                                         1.12320799883731e-07,
                                         1.70491924436081e-45,
                                         1.95610997503182e-30,
                                         2.07876302336788e-22,
                                         5.09792359878612e-17,
                                         4.76044375762523e-13,
                                         5.4858349508219e-10,
                                         1.42279120487936e-07,
                                         1.17103587562165e-05,
                                         0.000376999378204346,
                                         0.00540260557039999,
                                         0.0371557222769883,
                                         0.126564499348789,
                                         0.21065928395905,
                                         0.158711760654217,
                                         0.0444708052015432,
                                         0.00276997820521158,
                                         5.6484604207117e-06,
                                         1.46764807963303e-48,
                                         8.99307144950356e-33,
                                         2.40737672686827e-24,
                                         1.13675009325823e-18,
                                         1.7874333509187e-14,
                                         3.21317813041339e-11,
                                         1.24204753016473e-08,
                                         1.48422479866999e-06,
                                         6.85453414916992e-05,
                                         0.00140912136502426,
                                         0.0140702976688763,
                                         0.0714322716005641,
                                         0.185469630280914,
                                         0.235294651066918,
                                         0.126943520797488,
                                         0.0199175628760322,
                                         0.000216913505635753,
                                         9.06350453912249e-52,
                                         2.96605314410622e-35,
                                         2.00004257462271e-26,
                                         1.81841406829786e-20,
                                         4.81469059714838e-16,
                                         1.35015296938314e-12,
                                         7.77841960632636e-10,
                                         1.34953848873293e-07,
                                         8.94069671630859e-06,
                                         0.000263663277039244,
                                         0.00382240738018695,
                                         0.0289223198834589,
                                         0.117144287978023,
                                         0.250248283778612,
                                         0.259957405558445,
                                         0.102742978075737,
                                         0.00597584335394996,
                                         3.57598524171178e-55,
                                         6.24992992198963e-38,
                                         1.06159718585127e-28,
                                         1.85842869309988e-22,
                                         8.28575801302141e-18,
                                         3.62456994248818e-14,
                                         3.11221688462304e-11,
                                         7.8396414760461e-09,
                                         7.45058059692383e-07,
                                         3.15192755303932e-05,
                                         0.000663431352679713,
                                         0.00748164546486213,
                                         0.0472709970924957,
                                         0.170041722716849,
                                         0.340109829605758,
                                         0.338605055665619,
                                         0.105180965102057,
                                         6.77230511552841e-59,
                                         6.32139028646601e-41,
                                         2.70471502790669e-31,
                                         9.11675447536283e-25,
                                         6.84443092307635e-20,
                                         4.67058446830342e-16,
                                         5.97708702413844e-13,
                                         2.18599098234762e-10,
                                         2.98023223876953e-08,
                                         1.80860632446068e-06,
                                         5.52708635295998e-05,
                                         0.000928971436661083,
                                         0.00915608142958041,
                                         0.0554601604990223,
                                         0.213588276333434,
                                         0.535643653877189,
                                         0.888620515461475,

                                         0.899731985151144,
                                         0.572129188981488,
                                         0.251614202779238,
                                         0.0755766418776887,
                                         0.0152142547158194,
                                         0.00199880289287622,
                                         0.000165273776604798,
                                         8.19165461761717e-06,
                                         2.27898744878931e-07,
                                         3.25421932863091e-09,
                                         2.10591765306428e-11,
                                         5.16745933189397e-14,
                                         3.68315910781037e-17,
                                         5.00085796547369e-21,
                                         6.26589789953868e-26,
                                         1.77462990039822e-32,
                                         4.01718872293356e-42,
                                         4.22824838184724e-60,
                                         0.0952653706916923,
                                         0.323065988501419,
                                         0.356952202734394,
                                         0.205622875630263,
                                         0.069320621067438,
                                         0.0141035355779161,
                                         0.00172165799792368,
                                         0.000122381122342649,
                                         4.80695589642973e-06,
                                         9.64269951366953e-08,
                                         8.81005467633796e-10,
                                         3.10037446557258e-12,
                                         3.26243951336853e-15,
                                         6.8598208686645e-19,
                                         1.4393933558708e-23,
                                         7.81831020015359e-30,
                                         4.44636507679658e-39,
                                         2.49585072380562e-56,
                                         0.00484170361990684,
                                         0.0875648101329306,
                                         0.243067121578145,
                                         0.268532282559269,
                                         0.151605670180206,
                                         0.0477669228532252,
                                         0.00860857079022958,
                                         0.000877603745077999,
                                         4.86675606800643e-05,
                                         1.37148696415677e-06,
                                         1.76911905258119e-08,
                                         8.92878720568597e-11,
                                         1.38709336417009e-13,
                                         4.51671062926093e-17,
                                         1.5871461165641e-21,
                                         1.65332882639994e-27,
                                         2.36226839331191e-36,
                                         7.07160443527161e-53,
                                         0.000157212355798773,
                                         0.015163285664195,
                                         0.105746914682836,
                                         0.224051013938299,
                                         0.211833079299525,
                                         0.103359849111741,
                                         0.0275004981666681,
                                         0.0040207571161783,
                                         0.000314799733346716,
                                         1.24626405690721e-05,
                                         2.26966024601383e-07,
                                         1.64284325664428e-09,
                                         3.76785563890093e-12,
                                         1.90001528821245e-15,
                                         1.11809746593273e-19,
                                         2.23373121625154e-25,
                                         8.01823342776423e-34,
                                         1.28009631554068e-49,
                                         3.66210893013107e-06,
                                         0.00188370604507304,
                                         0.0330039028660672,
                                         0.134107612860784,
                                         0.21233824440763,
                                         0.160447360359341,
                                         0.0630240307288659,
                                         0.0132151871792918,
                                         0.00146078143495729,
                                         8.12427403444176e-05,
                                         2.08891491255573e-06,
                                         2.16848236333127e-08,
                                         7.34241493941828e-11,
                                         5.73387249801223e-14,
                                         5.65065150575733e-18,
                                         2.16500397106487e-23,
                                         1.95246950565253e-31,
                                         1.66235373604804e-46,
                                         6.51422030604584e-08,
                                         0.000178697940417279,
                                         0.00786591947932063,
                                         0.0612980348022783,
                                         0.162535887206584,
                                         0.190195346739944,
                                         0.110295651479142,
                                         0.0331684669862193,
                                         0.00517633919059136,
                                         0.000404432137544841,
                                         1.46813925381426e-05,
                                         2.18575892324099e-07,
                                         1.09262263689515e-09,
                                         1.32137342564081e-12,
                                         2.18074047993501e-16,
                                         1.60240791686233e-21,
                                         3.63058346054872e-29,
                                         1.64850717202193e-43,
                                         9.19651070118111e-10,
                                         1.34541237365987e-05,
                                         0.00148786369696751,
                                         0.0222366379759271,
                                         0.0987415101373514,
                                         0.178935558482605,
                                         0.153193401805728,
                                         0.0660703997851476,
                                         0.0145575961566335,
                                         0.001597850741735,
                                         8.18923258451318e-05,
                                         1.74855010125579e-06,
                                         1.29041950473029e-08,
                                         2.41675463506216e-11,
                                         6.67942229089102e-15,
                                         9.41275861398888e-20,
                                         5.35794245536986e-27,
                                         1.2974412244887e-40,
                                         1.05720822861318e-11,
                                         8.2483694236289e-07,
                                         0.000229167809733226,
                                         0.00656853495591339,
                                         0.0488457834082875,
                                         0.137078781247233,
                                         0.173260094041904,
                                         0.107168039445514,
                                         0.0333375303312956,
                                         0.0051404788196763,
                                         0.000371959775272789,
                                         1.13901833999336e-05,
                                         1.24099050684734e-07,
                                         3.59928608702754e-10,
                                         1.66590654175984e-13,
                                         4.50233262454028e-18,
                                         6.43867465310073e-25,
                                         8.31498290398003e-38,
                                         1.00745335181343e-13,
                                         4.19186943337407e-08,
                                         2.92597787481324e-05,
                                         0.00160840295557031,
                                         0.020030018525854,
                                         0.0870504004006216,
                                         0.162436636640415,
                                         0.144095558259468,
                                         0.0632854885950351,
                                         0.013708751269644,
                                         0.00140047610955017,
                                         6.15049840799803e-05,
                                         9.89311271131816e-07,
                                         4.44351992883924e-09,
                                         3.44420761643533e-12,
                                         1.78519305729752e-16,
                                         6.41389540886358e-23,
                                         4.41735177374727e-35,
                                         8.05959560394359e-16,
                                         1.78842727288658e-09,
                                         3.13626214039223e-06,
                                         0.000330632166856286,
                                         0.00689540082343756,
                                         0.0464082514354736,
                                         0.127847819440716,
                                         0.162652145356597,
                                         0.100855304476422,
                                         0.0306913585714205,
                                         0.00442669221206803,
                                         0.00027881349965812,
                                         6.62096585956325e-06,
                                         4.60534097794728e-08,
                                         5.97794487344847e-11,
                                         5.94232903214031e-15,
                                         5.3637825169694e-21,
                                         1.97009198498207e-32,
                                         5.4615401066545e-18,
                                         6.46321185808064e-11,
                                         2.84752302087866e-07,
                                         5.75716816080867e-05,
                                         0.00201071840173196,
                                         0.0209571976126088,
                                         0.0852346598731538,
                                         0.155518682595788,
                                         0.136146638503819,
                                         0.0582033390591412,
                                         0.0118521330759589,
                                         0.00107060891564924,
                                         3.75338669955981e-05,
                                         4.04305845001169e-07,
                                         8.78876919662505e-10,
                                         1.67548973853674e-13,
                                         3.79956621932464e-19,
                                         7.44259854219137e-30,
                                         3.15424624258674e-20,
                                         1.99069209100904e-12,
                                         2.2034374344368e-08,
                                         8.54380480506117e-06,
                                         0.000499714072588628,
                                         0.00806584371610899,
                                         0.0484303637555991,
                                         0.126731312490936,
                                         0.156636760389362,
                                         0.0940715454272004,
                                         0.0270453311181974,
                                         0.00350369670681967,
                                         0.000181344281103624,
                                         3.02507983303913e-06,
                                         1.10124209642241e-08,
                                         4.02629359818477e-12,
                                         2.29390520356154e-17,
                                         2.39630047686869e-27,
                                         1.55856269613962e-22,
                                         5.24575503107476e-14,
                                         1.45875520535117e-09,
                                         1.08478066629674e-06,
                                         0.000106252736268332,
                                         0.00265591769803951,
                                         0.0235433058893917,
                                         0.0883554844355033,
                                         0.154180225826724,
                                         0.130081900371037,
                                         0.0528002893436966,
                                         0.00981003077795179,
                                         0.000749605273833009,
                                         1.93647491929446e-05,
                                         1.18055375041954e-07,
                                         8.27784565495644e-11,
                                         1.18485479656609e-15,
                                         6.60094585696168e-25,
                                         6.60094585696168e-25,
                                         1.18485479656609e-15,
                                         8.27784565495644e-11,
                                         1.18055375041953e-07,
                                         1.93647491929446e-05,
                                         0.000749605273833009,
                                         0.00981003077795179,
                                         0.0528002893436966,
                                         0.130081900371037,
                                         0.154180225826724,
                                         0.0883554844355033,
                                         0.0235433058893917,
                                         0.00265591769803951,
                                         0.000106252736268332,
                                         1.08478066629675e-06,
                                         1.45875520535117e-09,
                                         5.24575503107476e-14,
                                         1.55856269613961e-22,
                                         2.39630047686869e-27,
                                         2.29390520356154e-17,
                                         4.02629359818477e-12,
                                         1.1012420964224e-08,
                                         3.02507983303913e-06,
                                         0.000181344281103624,
                                         0.00350369670681967,
                                         0.0270453311181974,
                                         0.0940715454272004,
                                         0.156636760389362,
                                         0.126731312490936,
                                         0.0484303637555991,
                                         0.00806584371610899,
                                         0.000499714072588628,
                                         8.54380480506123e-06,
                                         2.2034374344368e-08,
                                         1.99069209100904e-12,
                                         3.15424624258674e-20,
                                         7.44259854219136e-30,
                                         3.79956621932464e-19,
                                         1.67548973853674e-13,
                                         8.78876919662497e-10,
                                         4.04305845001169e-07,
                                         3.75338669955981e-05,
                                         0.00107060891564924,
                                         0.0118521330759589,
                                         0.0582033390591412,
                                         0.136146638503819,
                                         0.155518682595788,
                                         0.0852346598731538,
                                         0.0209571976126087,
                                         0.00201071840173196,
                                         5.7571681608087e-05,
                                         2.84752302087866e-07,
                                         6.46321185808064e-11,
                                         5.4615401066545e-18,
                                         1.97009198498207e-32,
                                         5.3637825169694e-21,
                                         5.94232903214031e-15,
                                         5.97794487344842e-11,
                                         4.60534097794728e-08,
                                         6.62096585956325e-06,
                                         0.00027881349965812,
                                         0.00442669221206803,
                                         0.0306913585714205,
                                         0.100855304476422,
                                         0.162652145356597,
                                         0.127847819440716,
                                         0.0464082514354736,
                                         0.00689540082343756,
                                         0.000330632166856287,
                                         3.13626214039223e-06,
                                         1.78842727288658e-09,
                                         8.05959560394359e-16,
                                         4.41735177374727e-35,
                                         6.41389540886358e-23,
                                         1.78519305729752e-16,
                                         3.4442076164353e-12,
                                         4.44351992883924e-09,
                                         9.89311271131816e-07,
                                         6.15049840799803e-05,
                                         0.00140047610955017,
                                         0.013708751269644,
                                         0.0632854885950351,
                                         0.144095558259468,
                                         0.162436636640415,
                                         0.0870504004006216,
                                         0.020030018525854,
                                         0.00160840295557032,
                                         2.92597787481324e-05,
                                         4.19186943337407e-08,
                                         1.00745335181343e-13,
                                         8.31498290398003e-38,
                                         6.43867465310073e-25,
                                         4.50233262454028e-18,
                                         1.66590654175982e-13,
                                         3.59928608702754e-10,
                                         1.24099050684734e-07,
                                         1.13901833999336e-05,
                                         0.000371959775272789,
                                         0.0051404788196763,
                                         0.0333375303312956,
                                         0.107168039445514,
                                         0.173260094041904,
                                         0.137078781247233,
                                         0.0488457834082875,
                                         0.00656853495591342,
                                         0.000229167809733226,
                                         8.2483694236289e-07,
                                         1.05720822861318e-11,
                                         1.2974412244887e-40,
                                         5.35794245536986e-27,
                                         9.41275861398888e-20,
                                         6.67942229089094e-15,
                                         2.41675463506216e-11,
                                         1.29041950473029e-08,
                                         1.74855010125579e-06,
                                         8.18923258451318e-05,
                                         0.001597850741735,
                                         0.0145575961566335,
                                         0.0660703997851476,
                                         0.153193401805728,
                                         0.178935558482605,
                                         0.0987415101373514,
                                         0.0222366379759272,
                                         0.00148786369696751,
                                         1.34541237365987e-05,
                                         9.19651070118111e-10,
                                         1.64850717202193e-43,
                                         3.63058346054872e-29,
                                         1.60240791686233e-21,
                                         2.18074047993498e-16,
                                         1.32137342564081e-12,
                                         1.09262263689515e-09,
                                         2.18575892324099e-07,
                                         1.46813925381426e-05,
                                         0.000404432137544841,
                                         0.00517633919059136,
                                         0.0331684669862193,
                                         0.110295651479142,
                                         0.190195346739944,
                                         0.162535887206584,
                                         0.0612980348022784,
                                         0.00786591947932063,
                                         0.000178697940417279,
                                         6.51422030604584e-08,
                                         1.66235373604804e-46,
                                         1.95246950565253e-31,
                                         2.16500397106487e-23,
                                         5.65065150575726e-18,
                                         5.73387249801223e-14,
                                         7.34241493941828e-11,
                                         2.16848236333127e-08,
                                         2.08891491255573e-06,
                                         8.12427403444176e-05,
                                         0.00146078143495729,
                                         0.0132151871792918,
                                         0.0630240307288659,
                                         0.160447360359341,
                                         0.21233824440763,
                                         0.134107612860785,
                                         0.0330039028660672,
                                         0.00188370604507304,
                                         3.66210893013107e-06,
                                         1.28009631554068e-49,
                                         8.01823342776424e-34,
                                         2.23373121625154e-25,
                                         1.11809746593271e-19,
                                         1.90001528821245e-15,
                                         3.76785563890093e-12,
                                         1.64284325664428e-09,
                                         2.26966024601383e-07,
                                         1.24626405690721e-05,
                                         0.000314799733346716,
                                         0.0040207571161783,
                                         0.0275004981666681,
                                         0.103359849111741,
                                         0.211833079299525,
                                         0.224051013938299,
                                         0.105746914682836,
                                         0.015163285664195,
                                         0.000157212355798773,
                                         7.07160443527161e-53,
                                         2.36226839331191e-36,
                                         1.65332882639994e-27,
                                         1.58714611656408e-21,
                                         4.51671062926093e-17,
                                         1.38709336417009e-13,
                                         8.92878720568597e-11,
                                         1.76911905258119e-08,
                                         1.37148696415677e-06,
                                         4.86675606800643e-05,
                                         0.000877603745077999,
                                         0.00860857079022958,
                                         0.0477669228532252,
                                         0.151605670180206,
                                         0.268532282559269,
                                         0.243067121578145,
                                         0.0875648101329306,
                                         0.00484170361990684,
                                         2.49585072380562e-56,
                                         4.44636507679658e-39,
                                         7.81831020015359e-30,
                                         1.43939335587078e-23,
                                         6.8598208686645e-19,
                                         3.26243951336853e-15,
                                         3.10037446557258e-12,
                                         8.81005467633796e-10,
                                         9.64269951366954e-08,
                                         4.80695589642973e-06,
                                         0.000122381122342649,
                                         0.00172165799792368,
                                         0.0141035355779161,
                                         0.069320621067438,
                                         0.205622875630263,
                                         0.356952202734394,
                                         0.323065988501419,
                                         0.0952653706916923,
                                         4.22824838184724e-60,
                                         4.01718872293356e-42,
                                         1.77462990039822e-32,
                                         6.26589789953859e-26,
                                         5.00085796547369e-21,
                                         3.68315910781037e-17,
                                         5.16745933189397e-14,
                                         2.10591765306428e-11,
                                         3.25421932863091e-09,
                                         2.27898744878931e-07,
                                         8.19165461761717e-06,
                                         0.000165273776604798,
                                         0.00199880289287622,
                                         0.0152142547158194,
                                         0.0755766418776887,
                                         0.251614202779238,
                                         0.572129188981488,
                                         0.899731985151144,

                                         0.90928647144097,
                                         0.605068973182593,
                                         0.289140728207425,
                                         0.0982239178396313,
                                         0.0233646072818427,
                                         0.00381021428315947,
                                         0.0004140082080386,
                                         2.88602655663612e-05,
                                         1.22759799339652e-06,
                                         2.98023223876953e-08,
                                         3.77220790267343e-10,
                                         2.19631591535571e-12,
                                         4.91815937702892e-15,
                                         3.24293592607434e-18,
                                         4.12735028623367e-22,
                                         4.90988065141999e-27,
                                         1.33683833337264e-33,
                                         2.94511865096353e-43,
                                         3.05357361121706e-61,
                                         0.0866333373364707,
                                         0.307069589811096,
                                         0.367830776477848,
                                         0.23884239022291,
                                         0.0947072024246378,
                                         0.0237732724781398,
                                         0.00378390919659699,
                                         0.000374538193263428,
                                         2.22074021178187e-05,
                                         7.45058059692383e-07,
                                         1.30326895831526e-08,
                                         1.05774026729581e-10,
                                         3.36318580326131e-13,
                                         3.24846968277647e-16,
                                         6.36395095116887e-20,
                                         1.26199256667068e-24,
                                         6.56780294644273e-31,
                                         3.62702783255023e-40,
                                         2.00310675687484e-57,
                                         0.00396196466066101,
                                         0.0748014422150581,
                                         0.224609486398861,
                                         0.278770492346423,
                                         0.184267510247195,
                                         0.0711983243757602,
                                         0.0166002144267706,
                                         0.00233309883401552,
                                         0.000192832654914854,
                                         8.94069671630859e-06,
                                         2.16129336010907e-07,
                                         2.44514435884943e-09,
                                         1.10392701465561e-11,
                                         1.56192618350378e-14,
                                         4.71002872838386e-18,
                                         1.55698308914758e-22,
                                         1.54882580368178e-28,
                                         2.14407620861701e-37,
                                         6.30726437723709e-54,
                                         0.000115760790907043,
                                         0.0116414873127854,
                                         0.0876260955633405,
                                         0.20787752177427,
                                         0.229055045638494,
                                         0.136230990036858,
                                         0.0465277471880107,
                                         0.00928529053655516,
                                         0.00106976587493137,
                                         6.85453414916992e-05,
                                         2.28991162843296e-06,
                                         3.61123130367817e-08,
                                         2.31502239530087e-10,
                                         4.79808047599399e-13,
                                         2.22713028530087e-16,
                                         1.22725929371182e-20,
                                         2.33351379125044e-26,
                                         8.09757319932424e-35,
                                         1.26882962500361e-50,
                                         2.42643399200762e-06,
                                         0.0012997597238007,
                                         0.024524204851853,
                                         0.111205013591025,
                                         0.204261758335486,
                                         0.186998516407334,
                                         0.09355488310114,
                                         0.0265102587281454,
                                         0.0042574838331024,
                                         0.000376999378204346,
                                         1.74052326997004e-05,
                                         3.82615197703104e-07,
                                         3.48278078970774e-09,
                                         1.05737894604175e-11,
                                         7.55481439957749e-15,
                                         6.93976671776445e-19,
                                         2.52216954111146e-24,
                                         2.19394459347153e-32,
                                         1.83114060484048e-47,
                                         3.8838466901584e-08,
                                         0.000110816462407111,
                                         0.00524134817673805,
                                         0.0454284380860106,
                                         0.139097995075937,
                                         0.19601396334557,
                                         0.143650617340785,
                                         0.0577988285152116,
                                         0.0129390942377458,
                                         0.00158339738845825,
                                         0.000101024694624627,
                                         3.09567761973326e-06,
                                         4.00113942159781e-08,
                                         1.77942787849517e-10,
                                         1.95698986312565e-13,
                                         2.99667755392796e-17,
                                         2.08173182788212e-22,
                                         4.53923878743046e-30,
                                         2.01802562616513e-44,
                                         4.93384077940314e-10,
                                         7.49850946230326e-06,
                                         0.0008890384423266,
                                         0.0147285738887227,
                                         0.0751768508531311,
                                         0.163066711872534,
                                         0.175056397916296,
                                         0.100012333589715,
                                         0.0312093123127328,
                                         0.00527799129486084,
                                         0.000465376783480901,
                                         1.98782749486079e-05,
                                         3.64813317828836e-07,
                                         2.3766190668985e-09,
                                         4.02330361682035e-12,
                                         1.02698621175167e-15,
                                         1.36365573323784e-20,
                                         7.45366480236281e-28,
                                         1.76506667574403e-41,
                                         5.10369834847562e-12,
                                         4.13163950462274e-07,
                                         0.000122793360849452,
                                         0.00388839526904022,
                                         0.0330844716821189,
                                         0.110463915632985,
                                         0.173710189751327,
                                         0.140917490702475,
                                         0.0612972989970202,
                                         0.0143259763717651,
                                         0.0017456561247216,
                                         0.000103938985800145,
                                         2.70853529528266e-06,
                                         2.58473235512693e-08,
                                         6.73525208275135e-11,
                                         2.86593295543484e-14,
                                         7.27380273560864e-19,
                                         9.96628959957531e-26,
                                         1.25710736260087e-38,
                                         4.37634773107039e-14,
                                         1.88710843514706e-08,
                                         1.40590569142022e-05,
                                         0.000850955934719267,
                                         0.0120695543777921,
                                         0.0620301041664999,
                                         0.142889250648198,
                                         0.164589906565499,
                                         0.099798826439672,
                                         0.0322334468364716,
                                         0.0054279974703069,
                                         0.000450510807068505,
                                         1.6669603562963e-05,
                                         2.33022865595029e-07,
                                         9.34656131252457e-10,
                                         6.62970794770817e-13,
                                         3.21621632635451e-17,
                                         1.10464849277907e-23,
                                         7.42182280657491e-36,
                                         3.15037709252703e-16,
                                         7.23594378637453e-10,
                                         1.35132989864424e-06,
                                         0.00015633910835478,
                                         0.00369642723464386,
                                         0.0292420948258428,
                                         0.0986728600503355,
                                         0.16138582625548,
                                         0.136405984465117,
                                         0.0608853995800018,
                                         0.0141691719430969,
                                         0.00163929021294511,
                                         8.61271224789815e-05,
                                         1.76362152980352e-06,
                                         1.08886429598687e-08,
                                         1.28749829292674e-11,
                                         1.19385856459208e-15,
                                         1.02787096822618e-21,
                                         3.67851631760428e-33,
                                         1.92099763472055e-18,
                                         2.35021223838708e-11,
                                         1.10022156527618e-07,
                                         2.43299804909769e-05,
                                         0.000958929452600328,
                                         0.0116769124079799,
                                         0.0577177569486921,
                                         0.134042078070425,
                                         0.15792649070271,
                                         0.0974166393280029,
                                         0.0313301789253044,
                                         0.00505265970059908,
                                         0.000376936483286778,
                                         1.13064382686463e-05,
                                         1.07450694382817e-07,
                                         2.11793436542106e-10,
                                         3.75382547429239e-14,
                                         8.10152293740858e-20,
                                         1.54435935422025e-30,
                                         9.98319859512225e-21,
                                         6.50575304781528e-13,
                                         7.63444572466685e-09,
                                         3.22696668574012e-06,
                                         0.000212016541280546,
                                         0.00397398437341369,
                                         0.0287739685636303,
                                         0.0948845507007805,
                                         0.155831459023227,
                                         0.132840871810913,
                                         0.0590418382963569,
                                         0.0132728089671369,
                                         0.00140596659567292,
                                         6.1776717887422e-05,
                                         9.03697928076686e-07,
                                         2.96931945521444e-09,
                                         1.00594414291558e-12,
                                         5.44218528027013e-18,
                                         5.52589502366726e-28,
                                         4.43875124159957e-23,
                                         1.54076455835882e-14,
                                         4.53234631086482e-10,
                                         3.66180673262066e-07,
                                         4.01052334560253e-05,
                                         0.00115710428895329,
                                         0.0122726486270136,
                                         0.0574642938852074,
                                         0.131553832455756,
                                         0.154981017112732,
                                         0.0951930150814924,
                                         0.0298300412509168,
                                         0.00448673114249274,
                                         0.000288783291464652,
                                         6.50257788181854e-06,
                                         3.56163627674229e-08,
                                         2.30633224606938e-11,
                                         3.12772237149857e-16,
                                         1.69162896730126e-25,
                                         1.69162896730126e-25,
                                         3.12772237149857e-16,
                                         2.30633224606938e-11,
                                         3.56163627674229e-08,
                                         6.5025778818185e-06,
                                         0.000288783291464653,
                                         0.00448673114249275,
                                         0.0298300412509168,
                                         0.0951930150814924,
                                         0.154981017112732,
                                         0.131553832455756,
                                         0.0574642938852075,
                                         0.0122726486270136,
                                         0.00115710428895328,
                                         4.01052334560255e-05,
                                         3.66180673262066e-07,
                                         4.53234631086482e-10,
                                         1.54076455835882e-14,
                                         4.43875124159957e-23,
                                         5.52589502366726e-28,
                                         5.44218528027013e-18,
                                         1.00594414291558e-12,
                                         2.96931945521444e-09,
                                         9.03697928076681e-07,
                                         6.17767178874222e-05,
                                         0.00140596659567292,
                                         0.0132728089671369,
                                         0.0590418382963569,
                                         0.132840871810913,
                                         0.155831459023227,
                                         0.0948845507007807,
                                         0.0287739685636303,
                                         0.00397398437341368,
                                         0.000212016541280546,
                                         3.22696668574012e-06,
                                         7.63444572466685e-09,
                                         6.50575304781528e-13,
                                         9.98319859512225e-21,
                                         1.54435935422025e-30,
                                         8.10152293740858e-20,
                                         3.75382547429239e-14,
                                         2.11793436542106e-10,
                                         1.07450694382816e-07,
                                         1.13064382686464e-05,
                                         0.000376936483286778,
                                         0.00505265970059907,
                                         0.0313301789253044,
                                         0.0974166393280029,
                                         0.15792649070271,
                                         0.134042078070425,
                                         0.0577177569486921,
                                         0.0116769124079799,
                                         0.000958929452600332,
                                         2.43299804909769e-05,
                                         1.10022156527618e-07,
                                         2.35021223838708e-11,
                                         1.92099763472055e-18,
                                         3.67851631760428e-33,
                                         1.02787096822618e-21,
                                         1.19385856459208e-15,
                                         1.28749829292674e-11,
                                         1.08886429598686e-08,
                                         1.76362152980353e-06,
                                         8.61271224789815e-05,
                                         0.00163929021294511,
                                         0.0141691719430969,
                                         0.0608853995800018,
                                         0.136405984465117,
                                         0.16138582625548,
                                         0.0986728600503355,
                                         0.0292420948258427,
                                         0.00369642723464387,
                                         0.00015633910835478,
                                         1.35132989864424e-06,
                                         7.23594378637453e-10,
                                         3.15037709252703e-16,
                                         7.42182280657491e-36,
                                         1.10464849277907e-23,
                                         3.21621632635451e-17,
                                         6.62970794770817e-13,
                                         9.3465613125245e-10,
                                         2.3302286559503e-07,
                                         1.6669603562963e-05,
                                         0.000450510807068504,
                                         0.0054279974703069,
                                         0.0322334468364716,
                                         0.099798826439672,
                                         0.164589906565499,
                                         0.142889250648198,
                                         0.0620301041664998,
                                         0.0120695543777921,
                                         0.000850955934719267,
                                         1.40590569142022e-05,
                                         1.88710843514706e-08,
                                         4.37634773107039e-14,
                                         1.25710736260087e-38,
                                         9.96628959957531e-26,
                                         7.27380273560864e-19,
                                         2.86593295543484e-14,
                                         6.7352520827513e-11,
                                         2.58473235512694e-08,
                                         2.70853529528266e-06,
                                         0.000103938985800145,
                                         0.0017456561247216,
                                         0.0143259763717651,
                                         0.0612972989970202,
                                         0.140917490702475,
                                         0.173710189751327,
                                         0.110463915632985,
                                         0.0330844716821189,
                                         0.00388839526904022,
                                         0.000122793360849452,
                                         4.13163950462274e-07,
                                         5.10369834847562e-12,
                                         1.76506667574403e-41,
                                         7.45366480236281e-28,
                                         1.36365573323784e-20,
                                         1.02698621175167e-15,
                                         4.02330361682032e-12,
                                         2.37661906689851e-09,
                                         3.64813317828836e-07,
                                         1.98782749486078e-05,
                                         0.000465376783480901,
                                         0.00527799129486084,
                                         0.0312093123127328,
                                         0.100012333589715,
                                         0.175056397916296,
                                         0.163066711872534,
                                         0.0751768508531313,
                                         0.0147285738887227,
                                         0.0008890384423266,
                                         7.49850946230326e-06,
                                         4.93384077940314e-10,
                                         2.01802562616513e-44,
                                         4.53923878743045e-30,
                                         2.08173182788212e-22,
                                         2.99667755392796e-17,
                                         1.95698986312563e-13,
                                         1.77942787849518e-10,
                                         4.00113942159781e-08,
                                         3.09567761973325e-06,
                                         0.000101024694624627,
                                         0.00158339738845825,
                                         0.0129390942377458,
                                         0.0577988285152117,
                                         0.143650617340785,
                                         0.19601396334557,
                                         0.139097995075937,
                                         0.0454284380860106,
                                         0.00524134817673805,
                                         0.000110816462407111,
                                         3.8838466901584e-08,
                                         1.83114060484048e-47,
                                         2.19394459347153e-32,
                                         2.52216954111146e-24,
                                         6.93976671776445e-19,
                                         7.55481439957743e-15,
                                         1.05737894604175e-11,
                                         3.48278078970774e-09,
                                         3.82615197703102e-07,
                                         1.74052326997004e-05,
                                         0.000376999378204346,
                                         0.0042574838331024,
                                         0.0265102587281454,
                                         0.0935548831011401,
                                         0.186998516407334,
                                         0.204261758335487,
                                         0.111205013591025,
                                         0.024524204851853,
                                         0.0012997597238007,
                                         2.42643399200762e-06,
                                         1.26882962500361e-50,
                                         8.09757319932424e-35,
                                         2.33351379125044e-26,
                                         1.22725929371182e-20,
                                         2.22713028530085e-16,
                                         4.79808047599402e-13,
                                         2.31502239530087e-10,
                                         3.61123130367816e-08,
                                         2.28991162843296e-06,
                                         6.85453414916992e-05,
                                         0.00106976587493137,
                                         0.00928529053655517,
                                         0.0465277471880107,
                                         0.136230990036858,
                                         0.229055045638495,
                                         0.20787752177427,
                                         0.0876260955633405,
                                         0.0116414873127854,
                                         0.000115760790907043,
                                         6.30726437723709e-54,
                                         2.14407620861701e-37,
                                         1.54882580368178e-28,
                                         1.55698308914758e-22,
                                         4.71002872838381e-18,
                                         1.56192618350379e-14,
                                         1.10392701465561e-11,
                                         2.44514435884942e-09,
                                         2.16129336010907e-07,
                                         8.94069671630859e-06,
                                         0.000192832654914854,
                                         0.00233309883401552,
                                         0.0166002144267706,
                                         0.0711983243757602,
                                         0.184267510247195,
                                         0.278770492346423,
                                         0.224609486398861,
                                         0.0748014422150581,
                                         0.00396196466066101,
                                         2.00310675687484e-57,
                                         3.62702783255023e-40,
                                         6.56780294644274e-31,
                                         1.26199256667068e-24,
                                         6.36395095116881e-20,
                                         3.24846968277649e-16,
                                         3.36318580326131e-13,
                                         1.0577402672958e-10,
                                         1.30326895831526e-08,
                                         7.45058059692383e-07,
                                         2.22074021178187e-05,
                                         0.000374538193263428,
                                         0.00378390919659699,
                                         0.0237732724781398,
                                         0.0947072024246378,
                                         0.23884239022291,
                                         0.367830776477848,
                                         0.307069589811096,
                                         0.0866333373364707,
                                         3.05357361121706e-61,
                                         2.94511865096353e-43,
                                         1.33683833337264e-33,
                                         4.90988065141999e-27,
                                         4.12735028623363e-22,
                                         3.24293592607436e-18,
                                         4.91815937702892e-15,
                                         2.1963159153557e-12,
                                         3.77220790267343e-10,
                                         2.98023223876953e-08,
                                         1.22759799339652e-06,
                                         2.88602655663612e-05,
                                         0.0004140082080386,
                                         0.00381021428315947,
                                         0.0233646072818427,
                                         0.0982239178396313,
                                         0.289140728207425,
                                         0.605068973182593,
                                         0.90928647144097,

                                         0.917557161634437,
                                         0.634792814686867,
                                         0.325670510335672,
                                         0.122880766877622,
                                         0.0336837030787124,
                                         0.00659460579150857,
                                         0.000901376042865096,
                                         8.34825465059078e-05,
                                         5.0386676847626e-06,
                                         1.88307859126326e-07,
                                         4.07249721414888e-09,
                                         4.65258135503523e-11,
                                         2.47573894324313e-13,
                                         5.12771839529798e-16,
                                         3.16361333339819e-19,
                                         3.80974784515997e-23,
                                         4.33516777599464e-28,
                                         1.14118161950807e-34,
                                         2.45626870868221e-44,
                                         2.51409775337118e-62,
                                         0.0790829933245978,
                                         0.29112383651398,
                                         0.373681413207147,
                                         0.268735352114282,
                                         0.122320400717226,
                                         0.0366748740147291,
                                         0.00729536030714525,
                                         0.000951524693030847,
                                         7.92265702107396e-05,
                                         4.03838898418849e-06,
                                         1.18686441966227e-07,
                                         1.84934890387688e-09,
                                         1.35756455520515e-11,
                                         3.95971062556135e-14,
                                         3.5553603405261e-17,
                                         6.55687923215846e-21,
                                         1.23892506295292e-25,
                                         6.21601027097478e-32,
                                         3.34741940466453e-41,
                                         1.82310720490997e-58,
                                         0.00327170627122372,
                                         0.0640862363094713,
                                         0.205809679383102,
                                         0.282102495201685,
                                         0.213215708213441,
                                         0.0979015705759888,
                                         0.0283418841309689,
                                         0.0052057783822397,
                                         0.000597953251460249,
                                         4.15708676120827e-05,
                                         1.6602850702213e-06,
                                         3.52845814290436e-08,
                                         3.57320036814887e-10,
                                         1.46772255692304e-12,
                                         1.91789615008223e-15,
                                         5.41675726740564e-19,
                                         1.69951657616099e-23,
                                         1.62521160993197e-29,
                                         2.18970505260127e-38,
                                         6.34575779937814e-55,
                                         8.64750554129466e-05,
                                         0.00901316046561817,
                                         0.0724194847763698,
                                         0.189197062274672,
                                         0.237445987281301,
                                         0.166969067842005,
                                         0.070345452194269,
                                         0.0181960291250937,
                                         0.00288329410111015,
                                         0.00027339801931821,
                                         1.48384845866096e-05,
                                         4.30106910904569e-07,
                                         6.0086870456048e-09,
                                         3.47576034375915e-11,
                                         6.60985429780868e-14,
                                         2.8589522370642e-17,
                                         1.48946775666335e-21,
                                         2.71477251770653e-27,
                                         9.15137699857192e-36,
                                         1.41117252491013e-51,
                                         1.63969661346504e-06,
                                         0.000909380311961271,
                                         0.0182810515608441,
                                         0.0910286198327659,
                                         0.189699672091172,
                                         0.204285948035951,
                                         0.125256260395944,
                                         0.0456271853879084,
                                         0.00997394027606467,
                                         0.00128990506301729,
                                         9.5137677540968e-05,
                                         3.76117880327849e-06,
                                         7.24866297924172e-08,
                                         5.90488966612486e-10,
                                         1.63423607663369e-12,
                                         1.08250655118973e-15,
                                         9.36467944262574e-20,
                                         3.25321735321466e-25,
                                         2.74374269555717e-33,
                                         2.25129724848078e-48,
                                         2.37422979793589e-08,
                                         7.00649139310729e-05,
                                         0.00352398066813444,
                                         0.0334447634285718,
                                         0.115732391272965,
                                         0.190865558337086,
                                         0.170313650632193,
                                         0.0873689941796294,
                                         0.0263470007691582,
                                         0.00464736445297327,
                                         0.000465802850409912,
                                         2.51164433149103e-05,
                                         6.67763738700063e-07,
                                         7.66055705209419e-09,
                                         3.08549112558619e-11,
                                         3.12997427159311e-14,
                                         4.49615553569882e-18,
                                         2.97700330848492e-23,
                                         6.28184066894813e-31,
                                         2.74266124459194e-45,
                                         2.72842146679658e-10,
                                         4.28435148743716e-06,
                                         0.000539132255614572,
                                         0.00975231573905252,
                                         0.0560367227375642,
                                         0.141529213968118,
                                         0.183792984281784,
                                         0.132776264974624,
                                         0.055236361061412,
                                         0.013288781820346,
                                         0.0018100109119226,
                                         0.000133113393388084,
                                         4.88221848745195e-06,
                                         7.88748180469807e-08,
                                         4.62341911937383e-10,
                                         7.18257975163093e-13,
                                         1.71324389056929e-16,
                                         2.16209617889981e-21,
                                         1.14145792371548e-28,
                                         2.65180114493063e-42,
                                         2.55315381672739e-12,
                                         2.1332728636269e-07,
                                         6.71635943868234e-05,
                                         0.00231560348376713,
                                         0.0220936436519986,
                                         0.0854557790845105,
                                         0.161504719079976,
                                         0.16430863018731,
                                         0.0942965385982974,
                                         0.0309414331842564,
                                         0.00572713024738022,
                                         0.000574463163318533,
                                         2.90662098997618e-05,
                                         6.61291902961707e-07,
                                         5.64129801869679e-09,
                                         1.34213734796246e-11,
                                         5.31586413898562e-15,
                                         1.27863773416956e-19,
                                         1.6889224679694e-26,
                                         2.08778875204261e-39,
                                         1.98047543214349e-14,
                                         8.8051078937767e-09,
                                         6.93584695666922e-06,
                                         0.000455771948728363,
                                         0.00722085990560701,
                                         0.0427724116915592,
                                         0.11764364203899,
                                         0.168549418891063,
                                         0.13344226337117,
                                         0.0597203904050721,
                                         0.0150217276623158,
                                         0.00205508397660992,
                                         0.000143445377306791,
                                         4.5959468434161e-06,
                                         5.70586877028271e-08,
                                         2.07893266827763e-10,
                                         1.36727386067031e-13,
                                         6.2682597032119e-18,
                                         2.07150733958543e-24,
                                         1.36257090241122e-36,
                                         1.2896914850697e-16,
                                         3.05103260556629e-10,
                                         6.01296902852988e-07,
                                         7.53103878224134e-05,
                                         0.00198122768112223,
                                         0.017972559603815,
                                         0.0719408556941248,
                                         0.145150333456504,
                                         0.158531271149284,
                                         0.0967673268576246,
                                         0.0330770372296278,
                                         0.00617192805087927,
                                         0.000594303930740094,
                                         2.68151779109937e-05,
                                         4.84494006252755e-07,
                                         2.70338425252476e-09,
                                         2.95230378403677e-12,
                                         2.57970675117789e-16,
                                         2.13298171953415e-22,
                                         7.46544260800168e-34,
                                         7.11403137769013e-19,
                                         8.95514426161086e-12,
                                         4.41562316334686e-08,
                                         1.05408527520536e-05,
                                         0.000460461608132562,
                                         0.00639690200172613,
                                         0.0372645866326603,
                                         0.105882061161707,
                                         0.159532824723981,
                                         0.132815395767092,
                                         0.0616945627708481,
                                         0.015700941913767,
                                         0.00208566339476154,
                                         0.000132525648831693,
                                         3.48472578616351e-06,
                                         2.97775324265717e-08,
                                         5.39983072839435e-11,
                                         8.99305541543496e-15,
                                         1.86037871117941e-20,
                                         3.46469957282818e-31,
                                         3.34444694997052e-21,
                                         2.24014904934325e-13,
                                         2.7635901119296e-09,
                                         1.25740505038989e-06,
                                         9.1207605848154e-05,
                                         0.00194047500817019,
                                         0.0164511252243319,
                                         0.065827191623431,
                                         0.136824465185159,
                                         0.155362674433517,
                                         0.0980721667887587,
                                         0.0340415362049618,
                                         0.00623818753341514,
                                         0.000558210224279509,
                                         2.13612862923759e-05,
                                         2.7954274411305e-07,
                                         8.41739756520766e-10,
                                         2.67191593540259e-13,
                                         1.38291083747869e-18,
                                         1.37042160005006e-28,
                                         1.34518204652232e-23,
                                         4.79434695713286e-15,
                                         1.47980153364355e-10,
                                         1.28328426700185e-07,
                                         1.54567044065078e-05,
                                         0.000503610325290468,
                                         0.00621359759474626,
                                         0.0350135772061807,
                                         0.100398142829493,
                                         0.155486673481796,
                                         0.133380661283131,
                                         0.0631452678331056,
                                         0.0159632328899623,
                                         0.00201161029810371,
                                         0.000112030018572616,
                                         2.24520477734126e-06,
                                         1.12259667031185e-08,
                                         6.79182561280197e-12,
                                         8.79498726165136e-17,
                                         4.63757582649215e-26,
                                         4.63757582649118e-26,
                                         8.79498726165101e-17,
                                         6.79182561280197e-12,
                                         1.12259667031184e-08,
                                         2.24520477734126e-06,
                                         0.000112030018572616,
                                         0.00201161029810371,
                                         0.0159632328899623,
                                         0.0631452678331056,
                                         0.133380661283131,
                                         0.155486673481796,
                                         0.100398142829493,
                                         0.0350135772061807,
                                         0.00621359759474626,
                                         0.000503610325290466,
                                         1.54567044065078e-05,
                                         1.28328426700186e-07,
                                         1.47980153364355e-10,
                                         4.79434695713304e-15,
                                         1.34518204652258e-23,
                                         1.37042160004975e-28,
                                         1.38291083747863e-18,
                                         2.67191593540259e-13,
                                         8.41739756520757e-10,
                                         2.7954274411305e-07,
                                         2.1361286292376e-05,
                                         0.000558210224279509,
                                         0.00623818753341514,
                                         0.0340415362049618,
                                         0.0980721667887587,
                                         0.155362674433517,
                                         0.136824465185159,
                                         0.065827191623431,
                                         0.0164511252243319,
                                         0.00194047500817018,
                                         9.1207605848154e-05,
                                         1.2574050503899e-06,
                                         2.7635901119296e-09,
                                         2.24014904934333e-13,
                                         3.34444694997111e-21,
                                         3.46469957282734e-31,
                                         1.86037871117933e-20,
                                         8.99305541543496e-15,
                                         5.3998307283943e-11,
                                         2.97775324265717e-08,
                                         3.48472578616352e-06,
                                         0.000132525648831693,
                                         0.00208566339476154,
                                         0.015700941913767,
                                         0.0616945627708481,
                                         0.132815395767092,
                                         0.159532824723981,
                                         0.105882061161707,
                                         0.0372645866326603,
                                         0.00639690200172611,
                                         0.000460461608132562,
                                         1.05408527520537e-05,
                                         4.41562316334686e-08,
                                         8.95514426161114e-12,
                                         7.11403137769128e-19,
                                         7.46544260799975e-34,
                                         2.13298171953405e-22,
                                         2.57970675117789e-16,
                                         2.95230378403674e-12,
                                         2.70338425252476e-09,
                                         4.84494006252758e-07,
                                         2.68151779109937e-05,
                                         0.000594303930740094,
                                         0.00617192805087927,
                                         0.0330770372296278,
                                         0.0967673268576246,
                                         0.158531271149284,
                                         0.145150333456504,
                                         0.0719408556941248,
                                         0.0179725596038149,
                                         0.00198122768112223,
                                         7.53103878224139e-05,
                                         6.01296902852988e-07,
                                         3.05103260556637e-10,
                                         1.28969148506988e-16,
                                         1.36257090241084e-36,
                                         2.07150733958532e-24,
                                         6.2682597032119e-18,
                                         1.36727386067029e-13,
                                         2.07893266827763e-10,
                                         5.70586877028274e-08,
                                         4.5959468434161e-06,
                                         0.000143445377306791,
                                         0.00205508397660992,
                                         0.0150217276623158,
                                         0.0597203904050721,
                                         0.13344226337117,
                                         0.168549418891063,
                                         0.11764364203899,
                                         0.0427724116915591,
                                         0.00722085990560701,
                                         0.000455771948728365,
                                         6.93584695666922e-06,
                                         8.80510789377692e-09,
                                         1.98047543214375e-14,
                                         2.087788752042e-39,
                                         1.68892246796931e-26,
                                         1.27863773416956e-19,
                                         5.31586413898556e-15,
                                         1.34213734796246e-11,
                                         5.64129801869682e-09,
                                         6.61291902961707e-07,
                                         2.90662098997618e-05,
                                         0.000574463163318533,
                                         0.00572713024738022,
                                         0.0309414331842564,
                                         0.0942965385982974,
                                         0.16430863018731,
                                         0.161504719079976,
                                         0.0854557790845103,
                                         0.0220936436519986,
                                         0.00231560348376714,
                                         6.71635943868234e-05,
                                         2.13327286362695e-07,
                                         2.55315381672768e-12,
                                         2.65180114492981e-42,
                                         1.14145792371542e-28,
                                         2.16209617889981e-21,
                                         1.71324389056927e-16,
                                         7.18257975163093e-13,
                                         4.62341911937385e-10,
                                         7.88748180469807e-08,
                                         4.88221848745195e-06,
                                         0.000133113393388084,
                                         0.0018100109119226,
                                         0.013288781820346,
                                         0.055236361061412,
                                         0.132776264974624,
                                         0.183792984281784,
                                         0.141529213968118,
                                         0.0560367227375642,
                                         0.00975231573905256,
                                         0.000539132255614572,
                                         4.28435148743724e-06,
                                         2.72842146679685e-10,
                                         2.74266124459106e-45,
                                         6.28184066894774e-31,
                                         2.97700330848492e-23,
                                         4.49615553569876e-18,
                                         3.12997427159311e-14,
                                         3.08549112558621e-11,
                                         7.66055705209419e-09,
                                         6.67763738700063e-07,
                                         2.51164433149103e-05,
                                         0.000465802850409912,
                                         0.00464736445297327,
                                         0.0263470007691582,
                                         0.0873689941796294,
                                         0.170313650632193,
                                         0.190865558337086,
                                         0.115732391272965,
                                         0.0334447634285719,
                                         0.00352398066813444,
                                         7.0064913931074e-05,
                                         2.37422979793608e-08,
                                         2.25129724848001e-48,
                                         2.743742695557e-33,
                                         3.25321735321466e-25,
                                         9.3646794426256e-20,
                                         1.08250655118973e-15,
                                         1.6342360766337e-12,
                                         5.90488966612486e-10,
                                         7.24866297924172e-08,
                                         3.76117880327849e-06,
                                         9.5137677540968e-05,
                                         0.00128990506301729,
                                         0.00997394027606467,
                                         0.0456271853879084,
                                         0.125256260395944,
                                         0.204285948035951,
                                         0.189699672091172,
                                         0.0910286198327661,
                                         0.0182810515608441,
                                         0.000909380311961283,
                                         1.63969661346515e-06,
                                         1.41117252490963e-51,
                                         9.1513769985713e-36,
                                         2.71477251770653e-27,
                                         1.48946775666333e-21,
                                         2.8589522370642e-17,
                                         6.60985429780872e-14,
                                         3.47576034375915e-11,
                                         6.0086870456048e-09,
                                         4.30106910904569e-07,
                                         1.48384845866096e-05,
                                         0.00027339801931821,
                                         0.00288329410111015,
                                         0.0181960291250937,
                                         0.070345452194269,
                                         0.166969067842005,
                                         0.237445987281301,
                                         0.189197062274673,
                                         0.0724194847763698,
                                         0.00901316046561825,
                                         8.64750554129508e-05,
                                         6.34575779937578e-55,
                                         2.18970505260111e-38,
                                         1.62521160993197e-29,
                                         1.69951657616096e-23,
                                         5.41675726740564e-19,
                                         1.91789615008225e-15,
                                         1.46772255692304e-12,
                                         3.57320036814887e-10,
                                         3.52845814290436e-08,
                                         1.6602850702213e-06,
                                         4.15708676120827e-05,
                                         0.000597953251460249,
                                         0.0052057783822397,
                                         0.0283418841309689,
                                         0.0979015705759888,
                                         0.213215708213441,
                                         0.282102495201686,
                                         0.205809679383101,
                                         0.0640862363094717,
                                         0.00327170627122383,
                                         1.82310720490926e-58,
                                         3.34741940466428e-41,
                                         6.21601027097478e-32,
                                         1.2389250629529e-25,
                                         6.55687923215846e-21,
                                         3.55536034052612e-17,
                                         3.95971062556135e-14,
                                         1.35756455520515e-11,
                                         1.84934890387688e-09,
                                         1.18686441966227e-07,
                                         4.03838898418849e-06,
                                         7.92265702107396e-05,
                                         0.000951524693030847,
                                         0.00729536030714525,
                                         0.0366748740147291,
                                         0.122320400717226,
                                         0.268735352114282,
                                         0.373681413207147,
                                         0.291123836513981,
                                         0.0790829933245991,
                                         2.51409775337016e-62,
                                         2.45626870868202e-44,
                                         1.14118161950807e-34,
                                         4.33516777599457e-28,
                                         3.80974784515997e-23,
                                         3.16361333339821e-19,
                                         5.12771839529798e-16,
                                         2.47573894324313e-13,
                                         4.65258135503523e-11,
                                         4.07249721414888e-09,
                                         1.88307859126326e-07,
                                         5.0386676847626e-06,
                                         8.34825465059078e-05,
                                         0.000901376042865096,
                                         0.00659460579150857,
                                         0.0336837030787124,
                                         0.122880766877622,
                                         0.325670510335672,
                                         0.634792814686867,
                                         0.917557161634437,

                                         0.924760857407458,
                                         0.661623813175167,
                                         0.360863148320593,
                                         0.149026828883508,
                                         0.0461326203439589,
                                         0.0105570873295963,
                                         0.00175338055299431,
                                         0.0002063516951671,
                                         1.66850063103192e-05,
                                         8.90663595062965e-07,
                                         2.98023223876953e-08,
                                         5.8380233979975e-10,
                                         6.10836362546501e-12,
                                         3.00857923339053e-14,
                                         5.82685368559797e-17,
                                         3.39491835751091e-20,
                                         3.89798661342823e-24,
                                         4.2688636904097e-29,
                                         1.09146564677002e-35,
                                         2.30254718735716e-45,
                                         2.33066113012814e-63,
                                         0.0724481725858978,
                                         0.275559274767833,
                                         0.375412945030731,
                                         0.294772104487028,
                                         0.151015463804812,
                                         0.0526957429080644,
                                         0.0126671382703787,
                                         0.00208496220195667,
                                         0.000230573994913687,
                                         1.66079410720564e-05,
                                         7.45058059692383e-07,
                                         1.95678790238142e-08,
                                         2.76261872232057e-10,
                                         1.86102122220742e-12,
                                         5.04094320644712e-15,
                                         4.25087108549991e-18,
                                         7.44229812681392e-22,
                                         1.34887089261889e-26,
                                         6.55727470893684e-33,
                                         3.4552911787276e-42,
                                         1.85935002576165e-59,
                                         0.0027243736379158,
                                         0.0550884021271254,
                                         0.18746370300547,
                                         0.279865613679425,
                                         0.2372880982331,
                                         0.12625488378476,
                                         0.0439260421873052,
                                         0.0101118255531294,
                                         0.00152945079839956,
                                         0.000148648019215416,
                                         8.94069671630859e-06,
                                         3.14820435660715e-07,
                                         5.99733428292895e-09,
                                         5.52563807033124e-11,
                                         2.09329643324457e-13,
                                         2.55486391119523e-16,
                                         6.82048126703012e-20,
                                         2.04583081614942e-24,
                                         1.89094075779106e-30,
                                         2.48886878573734e-39,
                                         7.12007244353316e-56,
                                         6.5453253394472e-05,
                                         0.00703607813035311,
                                         0.059806787389659,
                                         0.169761043281091,
                                         0.238207716415773,
                                         0.193261869616411,
                                         0.0973175064930657,
                                         0.0313318678735406,
                                         0.00648165680667408,
                                         0.000850017355115376,
                                         6.85453414916992e-05,
                                         3.23599179432244e-06,
                                         8.31803795076753e-08,
                                         1.04818726758855e-09,
                                         5.55360513198836e-12,
                                         9.81031481477086e-15,
                                         3.9934505675405e-18,
                                         1.98241412869046e-22,
                                         3.4838368024825e-28,
                                         1.14536705190109e-36,
                                         1.74193925396809e-52,
                                         1.12811121194677e-06,
                                         0.000644699360277185,
                                         0.0136879958623623,
                                         0.0738724730292644,
                                         0.171550427263932,
                                         0.212226824028563,
                                         0.154673523628177,
                                         0.0696464705057512,
                                         0.0197057360765279,
                                         0.00348700498020955,
                                         0.000376999378204346,
                                         2.38620645380026e-05,
                                         8.27636521439585e-07,
                                         1.42643318214671e-08,
                                         1.05700093151953e-10,
                                         2.70242857786836e-13,
                                         1.67740415185321e-16,
                                         1.37808231892989e-20,
                                         4.60461994838669e-26,
                                         3.78132041056643e-34,
                                         3.05729754562732e-49,
                                         1.48477003334677e-08,
                                         4.51097506116987e-05,
                                         0.00239230119463563,
                                         0.0245478715226989,
                                         0.094344021226871,
                                         0.177967612443928,
                                         0.18772736609447,
                                         0.11822206456287,
                                         0.045749451603606,
                                         0.0109235535801062,
                                         0.00158339738845825,
                                         0.000134367804531419,
                                         6.28847004946996e-06,
                                         1.48234941692207e-07,
                                         1.53625158277852e-09,
                                         5.68475963527906e-12,
                                         5.38038948454208e-15,
                                         7.31547512619428e-19,
                                         4.64746769406563e-24,
                                         9.53298419199673e-32,
                                         4.09759480283171e-46,
                                         1.55094332067417e-10,
                                         2.50503078758372e-06,
                                         0.000331834322257006,
                                         0.00647402690610332,
                                         0.0411781107498632,
                                         0.118443471619501,
                                         0.180829236748156,
                                         0.159267594644676,
                                         0.0842963127106504,
                                         0.0271584370966352,
                                         0.00527799129486084,
                                         0.000600498436889491,
                                         3.79210017796877e-05,
                                         1.22258537318953e-06,
                                         1.77206142398759e-08,
                                         9.4907261645578e-11,
                                         1.36968018119556e-13,
                                         3.0820478013292e-17,
                                         3.72278901140334e-22,
                                         1.90740853529048e-29,
                                         4.35862763214433e-43,
                                         1.31919645472201e-12,
                                         1.13274596864637e-07,
                                         3.74803438359253e-05,
                                         0.00139031104853053,
                                         0.0146350856038844,
                                         0.0641886287436448,
                                         0.141836017074169,
                                         0.174716177132839,
                                         0.126475987608168,
                                         0.0549822388147758,
                                         0.0143259763717651,
                                         0.00218527127472043,
                                         0.000186205035987111,
                                         8.21078386275613e-06,
                                         1.66445483612232e-07,
                                         1.29021948101955e-09,
                                         2.83923567871633e-12,
                                         1.05733599701137e-15,
                                         2.42827172417226e-20,
                                         3.1076735123291e-27,
                                         3.77526412353663e-40,
                                         9.30143555436674e-15,
                                         4.245989734054e-09,
                                         3.50923769665111e-06,
                                         0.000247500684986419,
                                         0.00431172519885885,
                                         0.0288358013443,
                                         0.0922213260114621,
                                         0.158878734961502,
                                         0.157302106232288,
                                         0.0922713815380403,
                                         0.0322334468364716,
                                         0.00659213040450503,
                                         0.000757931575680035,
                                         4.57106083993485e-05,
                                         1.29596184771114e-06,
                                         1.45396751774976e-08,
                                         4.87877369061534e-11,
                                         3.0068628753872e-14,
                                         1.31296484955029e-18,
                                         4.19714533505693e-25,
                                         2.71064017808201e-37,
                                         5.50572329652414e-17,
                                         1.33613163951441e-10,
                                         2.75832794527271e-07,
                                         3.69883305643431e-05,
                                         0.00106642612243148,
                                         0.0108750145248721,
                                         0.050338479289656,
                                         0.121289253087174,
                                         0.164242251529887,
                                         0.129997655616493,
                                         0.0608853995800018,
                                         0.0166943731541416,
                                         0.00258995651231751,
                                         0.000213635429011402,
                                         8.47103178315254e-06,
                                         1.37552886210176e-07,
                                         7.03791141763591e-10,
                                         7.17857301223635e-13,
                                         5.95981542440144e-17,
                                         4.75878981404638e-23,
                                         1.63388052212167e-34,
                                         2.76052923928482e-19,
                                         3.56150146470227e-12,
                                         1.83650698097962e-08,
                                         4.68237967559459e-06,
                                         0.000223421028215524,
                                         0.0034740913762003,
                                         0.0232746088629758,
                                         0.078431846712353,
                                         0.145260927525593,
                                         0.155137785602677,
                                         0.0974166393280029,
                                         0.0358119536823462,
                                         0.00749667208006635,
                                         0.000845752174938842,
                                         4.69022823477313e-05,
                                         1.10229612641944e-06,
                                         8.59984235426e-09,
                                         1.45169770902713e-11,
                                         2.29153286899372e-15,
                                         4.57038314608895e-21,
                                         8.34223251523513e-32,
                                         1.17963843194979e-21,
                                         8.0908769756099e-14,
                                         1.04212030744164e-09,
                                         5.05181121836018e-07,
                                         3.98929276052241e-05,
                                         0.000945869496096606,
                                         0.00917156149337243,
                                         0.0432256120333767,
                                         0.109494253511786,
                                         0.15778955349822,
                                         0.132840871810913,
                                         0.0654733409975362,
                                         0.0184936718779983,
                                         0.00285358990985284,
                                         0.000221324878032493,
                                         7.52844784953093e-06,
                                         8.95603485845102e-08,
                                         2.50203195629705e-10,
                                         7.50927421024972e-14,
                                         3.74099637166983e-19,
                                         3.63013583842295e-29,
                                         4.31274410860581e-24,
                                         1.57255659290202e-15,
                                         5.05931068993133e-11,
                                         4.66311166843598e-08,
                                         6.09418995921186e-06,
                                         0.000220327861049373,
                                         0.00309209187077312,
                                         0.0203815907662361,
                                         0.0706125762972516,
                                         0.137305243309024,
                                         0.154981017112732,
                                         0.102411613201702,
                                         0.03903246664746,
                                         0.00823736317338026,
                                         0.000893541455217091,
                                         4.3990695383536e-05,
                                         7.97974982858762e-07,
                                         3.68941686610634e-09,
                                         2.10531968672531e-12,
                                         2.61981232018775e-17,
                                         1.35148639562865e-26,
                                         1.35148639562896e-26,
                                         2.61981232018764e-17,
                                         2.10531968672535e-12,
                                         3.68941686610634e-09,
                                         7.97974982858762e-07,
                                         4.3990695383536e-05,
                                         0.000893541455217091,
                                         0.00823736317338024,
                                         0.03903246664746,
                                         0.102411613201702,
                                         0.154981017112732,
                                         0.137305243309024,
                                         0.0706125762972516,
                                         0.0203815907662361,
                                         0.00309209187077312,
                                         0.000220327861049373,
                                         6.09418995921186e-06,
                                         4.66311166843598e-08,
                                         5.05931068993125e-11,
                                         1.57255659290209e-15,
                                         4.31274410860489e-24,
                                         3.63013583842386e-29,
                                         3.74099637166965e-19,
                                         7.50927421024986e-14,
                                         2.50203195629705e-10,
                                         8.95603485845102e-08,
                                         7.52844784953093e-06,
                                         0.000221324878032493,
                                         0.00285358990985283,
                                         0.0184936718779983,
                                         0.0654733409975362,
                                         0.132840871810913,
                                         0.15778955349822,
                                         0.109494253511786,
                                         0.0432256120333768,
                                         0.00917156149337243,
                                         0.000945869496096606,
                                         3.98929276052241e-05,
                                         5.05181121836018e-07,
                                         1.04212030744162e-09,
                                         8.0908769756102e-14,
                                         1.17963843194956e-21,
                                         8.34223251523735e-32,
                                         4.57038314608871e-21,
                                         2.29153286899377e-15,
                                         1.45169770902713e-11,
                                         8.59984235426e-09,
                                         1.10229612641944e-06,
                                         4.69022823477313e-05,
                                         0.000845752174938839,
                                         0.00749667208006635,
                                         0.0358119536823462,
                                         0.0974166393280029,
                                         0.155137785602677,
                                         0.145260927525593,
                                         0.0784318467123531,
                                         0.0232746088629758,
                                         0.0034740913762003,
                                         0.000223421028215524,
                                         4.68237967559459e-06,
                                         1.8365069809796e-08,
                                         3.56150146470239e-12,
                                         2.76052923928433e-19,
                                         1.63388052212213e-34,
                                         4.75878981404612e-23,
                                         5.95981542440157e-17,
                                         7.17857301223635e-13,
                                         7.03791141763591e-10,
                                         1.37552886210176e-07,
                                         8.47103178315254e-06,
                                         0.000213635429011401,
                                         0.00258995651231751,
                                         0.0166943731541416,
                                         0.0608853995800018,
                                         0.129997655616493,
                                         0.164242251529887,
                                         0.121289253087174,
                                         0.050338479289656,
                                         0.0108750145248721,
                                         0.00106642612243148,
                                         3.69883305643431e-05,
                                         2.75832794527268e-07,
                                         1.33613163951445e-10,
                                         5.50572329652326e-17,
                                         2.71064017808283e-37,
                                         4.19714533505669e-25,
                                         1.31296484955032e-18,
                                         3.00686287538719e-14,
                                         4.87877369061534e-11,
                                         1.45396751774976e-08,
                                         1.29596184771114e-06,
                                         4.57106083993483e-05,
                                         0.000757931575680035,
                                         0.00659213040450503,
                                         0.0322334468364716,
                                         0.0922713815380403,
                                         0.157302106232288,
                                         0.158878734961502,
                                         0.0922213260114621,
                                         0.0288358013443,
                                         0.00431172519885885,
                                         0.000247500684986419,
                                         3.50923769665107e-06,
                                         4.24598973405412e-09,
                                         9.30143555436542e-15,
                                         3.77526412353784e-40,
                                         3.10767351232891e-27,
                                         2.42827172417232e-20,
                                         1.05733599701137e-15,
                                         2.83923567871633e-12,
                                         1.29021948101955e-09,
                                         1.66445483612232e-07,
                                         8.2107838627561e-06,
                                         0.000186205035987111,
                                         0.00218527127472043,
                                         0.0143259763717651,
                                         0.0549822388147758,
                                         0.126475987608168,
                                         0.174716177132839,
                                         0.141836017074169,
                                         0.0641886287436448,
                                         0.0146350856038844,
                                         0.00139031104853053,
                                         3.7480343835925e-05,
                                         1.1327459686464e-07,
                                         1.31919645472184e-12,
                                         4.3586276321458e-43,
                                         1.90740853529036e-29,
                                         3.72278901140344e-22,
                                         3.0820478013292e-17,
                                         1.36968018119556e-13,
                                         9.4907261645578e-11,
                                         1.77206142398759e-08,
                                         1.22258537318953e-06,
                                         3.79210017796877e-05,
                                         0.000600498436889491,
                                         0.00527799129486084,
                                         0.0271584370966352,
                                         0.0842963127106504,
                                         0.159267594644676,
                                         0.180829236748156,
                                         0.118443471619501,
                                         0.0411781107498632,
                                         0.00647402690610332,
                                         0.000331834322257003,
                                         2.50503078758378e-06,
                                         1.550943320674e-10,
                                         4.09759480283316e-46,
                                         9.53298419199609e-32,
                                         4.64746769406576e-24,
                                         7.31547512619428e-19,
                                         5.38038948454208e-15,
                                         5.68475963527906e-12,
                                         1.53625158277852e-09,
                                         1.48234941692207e-07,
                                         6.28847004946996e-06,
                                         0.000134367804531419,
                                         0.00158339738845825,
                                         0.0109235535801062,
                                         0.045749451603606,
                                         0.11822206456287,
                                         0.18772736609447,
                                         0.177967612443928,
                                         0.094344021226871,
                                         0.0245478715226989,
                                         0.00239230119463561,
                                         4.51097506116994e-05,
                                         1.48477003334664e-08,
                                         3.05729754562846e-49,
                                         3.78132041056616e-34,
                                         4.60461994838683e-26,
                                         1.37808231892989e-20,
                                         1.67740415185321e-16,
                                         2.70242857786836e-13,
                                         1.05700093151953e-10,
                                         1.42643318214671e-08,
                                         8.27636521439585e-07,
                                         2.38620645380026e-05,
                                         0.000376999378204346,
                                         0.00348700498020955,
                                         0.0197057360765279,
                                         0.0696464705057513,
                                         0.154673523628177,
                                         0.212226824028563,
                                         0.171550427263932,
                                         0.0738724730292644,
                                         0.0136879958623623,
                                         0.000644699360277194,
                                         1.12811121194669e-06,
                                         1.74193925396877e-52,
                                         1.145367051901e-36,
                                         3.48383680248261e-28,
                                         1.98241412869046e-22,
                                         3.9934505675405e-18,
                                         9.81031481477086e-15,
                                         5.55360513198836e-12,
                                         1.04818726758854e-09,
                                         8.31803795076753e-08,
                                         3.23599179432244e-06,
                                         6.85453414916992e-05,
                                         0.000850017355115376,
                                         0.00648165680667408,
                                         0.0313318678735406,
                                         0.0973175064930657,
                                         0.193261869616411,
                                         0.238207716415773,
                                         0.169761043281091,
                                         0.0598067873896588,
                                         0.00703607813035319,
                                         6.54532533944685e-05,
                                         7.12007244353608e-56,
                                         2.48886878573715e-39,
                                         1.89094075779112e-30,
                                         2.04583081614942e-24,
                                         6.82048126703012e-20,
                                         2.55486391119523e-16,
                                         2.09329643324457e-13,
                                         5.52563807033122e-11,
                                         5.99733428292895e-09,
                                         3.14820435660715e-07,
                                         8.94069671630859e-06,
                                         0.000148648019215415,
                                         0.00152945079839956,
                                         0.0101118255531294,
                                         0.0439260421873052,
                                         0.12625488378476,
                                         0.2372880982331,
                                         0.279865613679425,
                                         0.18746370300547,
                                         0.0550884021271257,
                                         0.0027243736379157,
                                         1.85935002576244e-59,
                                         3.45529117872732e-42,
                                         6.55727470893706e-33,
                                         1.34887089261889e-26,
                                         7.44229812681392e-22,
                                         4.25087108549991e-18,
                                         5.04094320644712e-15,
                                         1.86102122220741e-12,
                                         2.76261872232057e-10,
                                         1.95678790238142e-08,
                                         7.45058059692383e-07,
                                         1.66079410720564e-05,
                                         0.000230573994913687,
                                         0.00208496220195667,
                                         0.0126671382703787,
                                         0.0526957429080644,
                                         0.151015463804812,
                                         0.294772104487028,
                                         0.375412945030731,
                                         0.275559274767834,
                                         0.0724481725858965,
                                         2.33066113012918e-63,
                                         2.30254718735696e-45,
                                         1.09146564677006e-35,
                                         4.2688636904097e-29,
                                         3.89798661342823e-24,
                                         3.39491835751091e-20,
                                         5.82685368559797e-17,
                                         3.00857923339051e-14,
                                         6.10836362546501e-12,
                                         5.8380233979975e-10,
                                         2.98023223876953e-08,
                                         8.90663595062965e-07,
                                         1.66850063103192e-05,
                                         0.0002063516951671,
                                         0.00175338055299431,
                                         0.0105570873295963,
                                         0.0461326203439589,
                                         0.149026828883508,
                                         0.360863148320593,
                                         0.661623813175167,
                                         0.924760857407458,

                                         0.931071133097332,
                                         0.685865854323791,
                                         0.39450218423966,
                                         0.176177530539081,
                                         0.0605794735189766,
                                         0.0158565180918239,
                                         0.00311210093737295,
                                         0.000449184414641287,
                                         4.65097232160563e-05,
                                         3.34710182446124e-06,
                                         1.60767632750605e-07,
                                         4.89065773965267e-09,
                                         8.79615474159221e-11,
                                         8.53014449127266e-13,
                                         3.9294781099798e-15,
                                         7.18046811324967e-18,
                                         3.98091702203368e-21,
                                         4.38555612906646e-25,
                                         4.6456379889423e-30,
                                         1.15810789914668e-36,
                                         2.40084693877573e-46,
                                         2.40674556319915e-64,
                                         0.0665918014114219,
                                         0.260581873661931,
                                         0.373850017447043,
                                         0.316762907605749,
                                         0.179743614395578,
                                         0.0714724793719103,
                                         0.0202077353906181,
                                         0.00405546908949095,
                                         0.000570190883055125,
                                         5.48774459464966e-05,
                                         3.49514640154159e-06,
                                         1.40598450155513e-07,
                                         3.3531108586635e-09,
                                         4.34870443249992e-11,
                                         2.72018520812253e-13,
                                         6.911444155562e-16,
                                         5.51991159190281e-19,
                                         9.23797356724426e-23,
                                         1.6148849206665e-27,
                                         7.63802451646414e-34,
                                         3.94947592100264e-43,
                                         2.10315991525015e-60,
                                         0.00228612462747489,
                                         0.0475215349728883,
                                         0.170053915394595,
                                         0.273375355400633,
                                         0.255989813361709,
                                         0.154636177591664,
                                         0.0629829293331769,
                                         0.017575138228018,
                                         0.00335535148150126,
                                         0.000431877018132184,
                                         3.64731576649923e-05,
                                         1.94014877230344e-06,
                                         6.13541862913714e-08,
                                         1.06415436750041e-09,
                                         9.03864465587463e-12,
                                         3.1931997454467e-14,
                                         3.67385791530305e-17,
                                         9.34049719041892e-21,
                                         2.6945052333772e-25,
                                         2.4179889375942e-31,
                                         3.11857149381468e-40,
                                         8.82176834327674e-57,
                                         5.01423144592189e-05,
                                         0.00553684101193824,
                                         0.0494198248111592,
                                         0.15073348344542,
                                         0.232925604988715,
                                         0.213751267736147,
                                         0.125416131422352,
                                         0.0486610794317394,
                                         0.0126148214219293,
                                         0.00217145899588779,
                                         0.000243168150408192,
                                         1.7104676642686e-05,
                                         7.17242184325783e-07,
                                         1.6636988169755e-08,
                                         1.9188166860621e-10,
                                         9.42559345738062e-13,
                                         1.56220464773376e-15,
                                         6.03376801191131e-19,
                                         2.87237950235134e-23,
                                         4.89049598214821e-29,
                                         1.57324831193457e-37,
                                         2.36409180614518e-53,
                                         7.88978117973968e-07,
                                         0.000462796167956117,
                                         0.0103031939576403,
                                         0.0596233287842971,
                                         0.152043510659157,
                                         0.211964143503305,
                                         0.179159587259464,
                                         0.0966542234194421,
                                         0.0340236029615259,
                                         0.00783247973402492,
                                         0.00116304395164651,
                                         0.000108180955193892,
                                         6.01510976086097e-06,
                                         1.86595382290212e-07,
                                         2.92226615468119e-09,
                                         1.99593976227986e-11,
                                         4.76551239116222e-14,
                                         2.79616844664416e-17,
                                         2.1966492401696e-21,
                                         7.09590204168789e-27,
                                         5.69370487383367e-35,
                                         4.54495025859855e-50,
                                         9.48008303925228e-09,
                                         2.95395623268957e-05,
                                         0.0016403221536166,
                                         0.0180098171434327,
                                         0.0757888201846962,
                                         0.160510224761468,
                                         0.195439933864979,
                                         0.146604241038878,
                                         0.0700754826119789,
                                         0.0215741417410952,
                                         0.00424787925631757,
                                         0.000522484433582747,
                                         3.85219168513574e-05,
                                         1.5981357881544e-06,
                                         3.39854207320649e-08,
                                         3.22754835181369e-10,
                                         1.11011477168401e-12,
                                         9.89520176194127e-16,
                                         1.28282144394591e-19,
                                         7.86228744189564e-25,
                                         1.5735452308387e-32,
                                         6.6723763170934e-47,
                                         9.0404235820105e-11,
                                         1.49640039176173e-06,
                                         0.000207260184352844,
                                         0.00431749499219757,
                                         0.0299827772884305,
                                         0.0964655978217565,
                                         0.169206100006822,
                                         0.176482516175826,
                                         0.11454637453026,
                                         0.0471625447658069,
                                         0.0123133905690526,
                                         0.00200274361180526,
                                         0.000195795036180302,
                                         1.08631535183557e-05,
                                         3.13685893070276e-07,
                                         4.14216637710564e-09,
                                         2.05236979742613e-11,
                                         2.77917204049317e-14,
                                         5.94567494831862e-18,
                                         6.9138456247971e-23,
                                         3.45138148233365e-30,
                                         7.77430159861289e-44,
                                         7.02008277717607e-13,
                                         6.1726035718291e-08,
                                         2.13245279168922e-05,
                                         0.000842812944719478,
                                         0.00965862682818627,
                                         0.0472083729855356,
                                         0.119287665645943,
                                         0.17299504169645,
                                         0.152466143129998,
                                         0.0839532955452996,
                                         0.0290643063278181,
                                         0.00625106723237687,
                                         0.000810349336391646,
                                         6.01277527637346e-05,
                                         2.35762181519044e-06,
                                         4.32871643138328e-08,
                                         3.08972739822257e-10,
                                         6.35598737274387e-13,
                                         2.24394872857368e-16,
                                         4.95070734925976e-21,
                                         6.16429641513615e-28,
                                         7.37596849417905e-41,
                                         4.51879504599059e-15,
                                         2.11064851031961e-09,
                                         1.81873710895434e-06,
                                         0.00013638214266495,
                                         0.00257920506041906,
                                         0.0191510488012935,
                                         0.0697111137499066,
                                         0.140569979551913,
                                         0.168225736251127,
                                         0.12388114888477,
                                         0.0568681665228626,
                                         0.0161737208025779,
                                         0.00278016032583616,
                                         0.000275880487962064,
                                         1.46885948405369e-05,
                                         3.74988277477446e-07,
                                         3.85577488507917e-09,
                                         1.20497372405265e-11,
                                         7.02023505885071e-15,
                                         2.9386088914351e-19,
                                         9.12643196093377e-26,
                                         5.8010108585124e-38,
                                         2.44189479198046e-17,
                                         6.05881007837908e-11,
                                         1.30221989020514e-07,
                                         1.85271114932424e-05,
                                         0.000578202881035216,
                                         0.00652213804711178,
                                         0.0342004953773241,
                                         0.0958904598628361,
                                         0.155824361114595,
                                         0.153460494667719,
                                         0.0934119384640281,
                                         0.035130926836377,
                                         0.00800739580071388,
                                         0.00106265157955278,
                                         7.6826359804945e-05,
                                         2.72709420204874e-06,
                                         4.03949495638501e-08,
                                         1.9177677958704e-10,
                                         1.84380195856809e-13,
                                         1.46433423441621e-17,
                                         1.13433784232617e-23,
                                         3.83011838366725e-35,
                                         1.11775037433596e-19,
                                         1.47323603640122e-12,
                                         7.89791202173645e-09,
                                         2.13192288507725e-06,
                                         0.000109796425387448,
                                         0.00188148595784669,
                                         0.0142126919345266,
                                         0.0554079129408954,
                                         0.122262097707444,
                                         0.161028050036685,
                                         0.129971793947871,
                                         0.0646372459972254,
                                         0.0195355805150868,
                                         0.00346716387854462,
                                         0.000340372003302205,
                                         1.67994923893165e-05,
                                         3.58472662040734e-07,
                                         2.58540174116662e-09,
                                         4.10195093003689e-12,
                                         6.18090706940715e-16,
                                         1.19425588953773e-21,
                                         2.14207310076843e-32,
                                         4.36055020011071e-22,
                                         3.05306407520487e-14,
                                         4.08243097030634e-10,
                                         2.09080658522096e-07,
                                         1.77694809513375e-05,
                                         0.000462583957411095,
                                         0.00503383388522788,
                                         0.0272864334381099,
                                         0.0817573718144551,
                                         0.144007484822898,
                                         0.154125476903281,
                                         0.1013572156992,
                                         0.0406200017330685,
                                         0.00964131867909379,
                                         0.00128521562507044,
                                         8.82004437031325e-05,
                                         2.71121277967812e-06,
                                         2.97056215399801e-08,
                                         7.7775935009035e-11,
                                         2.22352902750348e-14,
                                         1.0715956795533e-19,
                                         1.02102169153798e-29,
                                         1.45541207220857e-24,
                                         5.41312081634224e-16,
                                         1.80540077311501e-11,
                                         1.75430234038934e-08,
                                         2.46042141847314e-06,
                                         9.73034790852444e-05,
                                         0.00152535044417541,
                                         0.011496614550671,
                                         0.0467746145183678,
                                         0.110183563076435,
                                         0.15636803160664,
                                         0.135979899345513,
                                         0.0722606343521414,
                                         0.0229375338424581,
                                         0.00415189516148032,
                                         0.000396180956519722,
                                         1.75436268201181e-05,
                                         2.92009826433451e-07,
                                         1.261677146821e-09,
                                         6.84355258340404e-13,
                                         8.22645510511125e-18,
                                         4.16374252144998e-27,
                                         4.16374252144998e-27,
                                         8.22645510511125e-18,
                                         6.8435525834039e-13,
                                         1.26167714682101e-09,
                                         2.92009826433453e-07,
                                         1.75436268201181e-05,
                                         0.000396180956519722,
                                         0.00415189516148032,
                                         0.0229375338424581,
                                         0.0722606343521413,
                                         0.135979899345513,
                                         0.15636803160664,
                                         0.110183563076435,
                                         0.0467746145183678,
                                         0.011496614550671,
                                         0.00152535044417541,
                                         9.73034790852444e-05,
                                         2.46042141847313e-06,
                                         1.75430234038933e-08,
                                         1.80540077311504e-11,
                                         5.41312081634224e-16,
                                         1.45541207220857e-24,
                                         1.02102169153798e-29,
                                         1.0715956795533e-19,
                                         2.22352902750343e-14,
                                         7.77759350090359e-11,
                                         2.97056215399803e-08,
                                         2.71121277967812e-06,
                                         8.82004437031325e-05,
                                         0.00128521562507044,
                                         0.00964131867909379,
                                         0.0406200017330684,
                                         0.1013572156992,
                                         0.154125476903281,
                                         0.144007484822898,
                                         0.0817573718144551,
                                         0.0272864334381099,
                                         0.00503383388522788,
                                         0.000462583957411095,
                                         1.77694809513374e-05,
                                         2.09080658522094e-07,
                                         4.0824309703064e-10,
                                         3.05306407520487e-14,
                                         4.36055020011071e-22,
                                         2.14207310076843e-32,
                                         1.19425588953773e-21,
                                         6.18090706940701e-16,
                                         4.10195093003694e-12,
                                         2.58540174116664e-09,
                                         3.58472662040734e-07,
                                         1.67994923893165e-05,
                                         0.000340372003302205,
                                         0.00346716387854462,
                                         0.0195355805150868,
                                         0.0646372459972254,
                                         0.129971793947871,
                                         0.161028050036685,
                                         0.122262097707444,
                                         0.0554079129408954,
                                         0.0142126919345266,
                                         0.00188148595784669,
                                         0.000109796425387448,
                                         2.13192288507724e-06,
                                         7.89791202173657e-09,
                                         1.47323603640122e-12,
                                         1.11775037433596e-19,
                                         3.83011838366725e-35,
                                         1.13433784232617e-23,
                                         1.46433423441618e-17,
                                         1.84380195856812e-13,
                                         1.91776779587042e-10,
                                         4.03949495638501e-08,
                                         2.72709420204874e-06,
                                         7.6826359804945e-05,
                                         0.00106265157955278,
                                         0.00800739580071386,
                                         0.035130926836377,
                                         0.0934119384640281,
                                         0.153460494667719,
                                         0.155824361114595,
                                         0.0958904598628361,
                                         0.0342004953773241,
                                         0.00652213804711178,
                                         0.000578202881035214,
                                         1.85271114932423e-05,
                                         1.30221989020516e-07,
                                         6.05881007837908e-11,
                                         2.44189479198046e-17,
                                         5.8010108585124e-38,
                                         9.12643196093377e-26,
                                         2.93860889143502e-19,
                                         7.02023505885081e-15,
                                         1.20497372405266e-11,
                                         3.85577488507917e-09,
                                         3.74988277477446e-07,
                                         1.46885948405369e-05,
                                         0.000275880487962064,
                                         0.00278016032583616,
                                         0.0161737208025779,
                                         0.0568681665228626,
                                         0.12388114888477,
                                         0.168225736251127,
                                         0.140569979551913,
                                         0.0697111137499066,
                                         0.0191510488012935,
                                         0.00257920506041905,
                                         0.000136382142664949,
                                         1.81873710895436e-06,
                                         2.11064851031961e-09,
                                         4.51879504599059e-15,
                                         7.37596849417905e-41,
                                         6.16429641513615e-28,
                                         4.95070734925962e-21,
                                         2.24394872857371e-16,
                                         6.35598737274393e-13,
                                         3.08972739822257e-10,
                                         4.32871643138328e-08,
                                         2.35762181519044e-06,
                                         6.01277527637346e-05,
                                         0.000810349336391644,
                                         0.00625106723237687,
                                         0.0290643063278181,
                                         0.0839532955452997,
                                         0.152466143129998,
                                         0.17299504169645,
                                         0.119287665645943,
                                         0.0472083729855356,
                                         0.00965862682818624,
                                         0.000842812944719473,
                                         2.13245279168924e-05,
                                         6.1726035718291e-08,
                                         7.02008277717607e-13,
                                         7.77430159861289e-44,
                                         3.45138148233365e-30,
                                         6.9138456247969e-23,
                                         5.94567494831871e-18,
                                         2.7791720404932e-14,
                                         2.05236979742613e-11,
                                         4.14216637710564e-09,
                                         3.13685893070276e-07,
                                         1.08631535183557e-05,
                                         0.000195795036180302,
                                         0.00200274361180526,
                                         0.0123133905690526,
                                         0.0471625447658069,
                                         0.11454637453026,
                                         0.176482516175826,
                                         0.169206100006822,
                                         0.0964655978217566,
                                         0.0299827772884304,
                                         0.00431749499219754,
                                         0.000207260184352846,
                                         1.49640039176173e-06,
                                         9.0404235820105e-11,
                                         6.6723763170934e-47,
                                         1.5735452308387e-32,
                                         7.8622874418954e-25,
                                         1.28282144394593e-19,
                                         9.89520176194138e-16,
                                         1.11011477168401e-12,
                                         3.22754835181369e-10,
                                         3.39854207320649e-08,
                                         1.5981357881544e-06,
                                         3.85219168513573e-05,
                                         0.000522484433582747,
                                         0.00424787925631757,
                                         0.0215741417410952,
                                         0.0700754826119789,
                                         0.146604241038878,
                                         0.195439933864979,
                                         0.160510224761468,
                                         0.075788820184696,
                                         0.0180098171434327,
                                         0.00164032215361661,
                                         2.95395623268957e-05,
                                         9.48008303925228e-09,
                                         4.54495025859855e-50,
                                         5.69370487383367e-35,
                                         7.09590204168766e-27,
                                         2.19664924016964e-21,
                                         2.79616844664419e-17,
                                         4.76551239116222e-14,
                                         1.99593976227986e-11,
                                         2.92226615468119e-09,
                                         1.86595382290212e-07,
                                         6.01510976086095e-06,
                                         0.000108180955193892,
                                         0.00116304395164651,
                                         0.00783247973402492,
                                         0.0340236029615259,
                                         0.0966542234194421,
                                         0.179159587259464,
                                         0.211964143503305,
                                         0.152043510659157,
                                         0.0596233287842969,
                                         0.0103031939576404,
                                         0.000462796167956117,
                                         7.88978117973968e-07,
                                         2.36409180614518e-53,
                                         1.57324831193457e-37,
                                         4.89049598214805e-29,
                                         2.87237950235139e-23,
                                         6.03376801191138e-19,
                                         1.56220464773376e-15,
                                         9.42559345738062e-13,
                                         1.9188166860621e-10,
                                         1.6636988169755e-08,
                                         7.17242184325781e-07,
                                         1.7104676642686e-05,
                                         0.000243168150408192,
                                         0.00217145899588779,
                                         0.0126148214219293,
                                         0.0486610794317394,
                                         0.125416131422352,
                                         0.213751267736147,
                                         0.232925604988715,
                                         0.15073348344542,
                                         0.0494198248111594,
                                         0.00553684101193824,
                                         5.01423144592189e-05,
                                         8.82176834327674e-57,
                                         3.11857149381468e-40,
                                         2.41798893759412e-31,
                                         2.69450523337725e-25,
                                         9.34049719041903e-21,
                                         3.67385791530304e-17,
                                         3.1931997454467e-14,
                                         9.03864465587463e-12,
                                         1.06415436750041e-09,
                                         6.13541862913712e-08,
                                         1.94014877230344e-06,
                                         3.64731576649923e-05,
                                         0.000431877018132184,
                                         0.00335535148150126,
                                         0.017575138228018,
                                         0.0629829293331769,
                                         0.154636177591664,
                                         0.255989813361709,
                                         0.273375355400633,
                                         0.170053915394596,
                                         0.0475215349728883,
                                         0.00228612462747489,
                                         2.10315991525015e-60,
                                         3.94947592100264e-43,
                                         7.63802451646386e-34,
                                         1.61488492066653e-27,
                                         9.23797356724437e-23,
                                         5.51991159190281e-19,
                                         6.911444155562e-16,
                                         2.72018520812253e-13,
                                         4.34870443249992e-11,
                                         3.35311085866349e-09,
                                         1.40598450155513e-07,
                                         3.49514640154159e-06,
                                         5.48774459464966e-05,
                                         0.000570190883055125,
                                         0.00405546908949095,
                                         0.0202077353906181,
                                         0.0714724793719103,
                                         0.179743614395578,
                                         0.316762907605748,
                                         0.373850017447044,
                                         0.260581873661931,
                                         0.0665918014114219,
                                         2.40674556319915e-64,
                                         2.40084693877573e-46,
                                         1.15810789914664e-36,
                                         4.6456379889424e-30,
                                         4.38555612906652e-25,
                                         3.98091702203368e-21,
                                         7.18046811324967e-18,
                                         3.9294781099798e-15,
                                         8.53014449127266e-13,
                                         8.79615474159218e-11,
                                         4.89065773965267e-09,
                                         1.60767632750605e-07,
                                         3.34710182446124e-06,
                                         4.65097232160563e-05,
                                         0.000449184414641287,
                                         0.00311210093737295,
                                         0.0158565180918239,
                                         0.0605794735189766,
                                         0.176177530539081,
                                         0.39450218423966,
                                         0.685865854323791,
                                         0.931071133097332,

                                         0.936628117889063,
                                         0.707797753773512,
                                         0.426465650726719,
                                         0.203903351826203,
                                         0.0768261308610859,
                                         0.0225947714789056,
                                         0.00512283673442679,
                                         0.000881228779490933,
                                         0.00011271229267092,
                                         1.04494239925143e-05,
                                         6.7992135735276e-07,
                                         2.98023223876953e-08,
                                         8.3466246107665e-10,
                                         1.39377842290254e-11,
                                         1.26513391725727e-13,
                                         5.49784749411429e-16,
                                         9.54950669892837e-19,
                                         5.06972708486449e-22,
                                         5.38675566162615e-26,
                                         5.54262581199963e-31,
                                         1.35146421391403e-37,
                                         2.75921979008232e-47,
                                         2.74266505061654e-65,
                                         0.0614004315009814,
                                         0.246311264263785,
                                         0.369709734266977,
                                         0.334761897894876,
                                         0.207625983638408,
                                         0.0924670552411256,
                                         0.0300794118851054,
                                         0.0071586782186636,
                                         0.00123571381238494,
                                         0.00015206750324066,
                                         1.30005216792769e-05,
                                         7.45058059692383e-07,
                                         2.72827913884431e-08,
                                         5.98590320998655e-10,
                                         7.21223549703818e-12,
                                         4.22988477156513e-14,
                                         1.01648770386093e-16,
                                         7.74257684562776e-20,
                                         1.24576048998254e-23,
                                         2.11000712015779e-28,
                                         9.74333734009353e-35,
                                         4.9555426733757e-44,
                                         2.61486069963335e-61,
                                         0.00193204346519261,
                                         0.0411434403659693,
                                         0.153843428988579,
                                         0.263808579376837,
                                         0.269336791106747,
                                         0.181638262206587,
                                         0.0847753133281199,
                                         0.0279137540839586,
                                         0.00650288023753584,
                                         0.00106223771454184,
                                         0.000119317491369693,
                                         8.94069671630859e-06,
                                         4.28063265709677e-07,
                                         1.23397647662392e-08,
                                         1.9735336531964e-10,
                                         1.56208845296981e-12,
                                         5.19355288858935e-15,
                                         5.67580812523869e-18,
                                         1.38287544864782e-21,
                                         3.85561373814799e-26,
                                         3.37172515127152e-32,
                                         4.27206037461741e-41,
                                         1.19664568917593e-57,
                                         3.88407580204229e-05,
                                         0.00439078599217606,
                                         0.0408999099934643,
                                         0.1328211219162,
                                         0.22322097230994,
                                         0.227957027163794,
                                         0.15264929462075,
                                         0.0695390879082915,
                                         0.0218634626866829,
                                         0.00474058953670643,
                                         0.000699637022808763,
                                         6.85453414916992e-05,
                                         4.29094011775741e-06,
                                         1.62520970349592e-07,
                                         3.45020189354521e-09,
                                         3.6855986716244e-11,
                                         1.69532248861237e-13,
                                         2.65824632163106e-16,
                                         9.80746834353661e-20,
                                         4.5012014754577e-24,
                                         7.45455849795953e-30,
                                         2.35292930275537e-38,
                                         3.49870993864616e-54,
                                         5.60163245668822e-07,
                                         0.000336155362852349,
                                         0.00780048958803872,
                                         0.0479735085279044,
                                         0.132718180045309,
                                         0.205236565878436,
                                         0.197186104863064,
                                         0.124278453471897,
                                         0.0527336999489746,
                                         0.0151774600547817,
                                         0.00294304945791669,
                                         0.000376999378204346,
                                         3.08569541599545e-05,
                                         1.5355644201864e-06,
                                         4.32713644809611e-08,
                                         6.2383045303034e-10,
                                         3.97005200350601e-12,
                                         8.93138572662419e-15,
                                         4.98984289255422e-18,
                                         3.76981027877089e-22,
                                         1.18235474030644e-27,
                                         9.29686411359187e-36,
                                         7.33848489994684e-51,
                                         6.16918968031001e-09,
                                         1.96527993077963e-05,
                                         0.00113607753373745,
                                         0.0132319063914351,
                                         0.0602576846981473,
                                         0.141105224715638,
                                         0.1945111114755,
                                         0.169609155316789,
                                         0.0971279321592854,
                                         0.0371067095082729,
                                         0.00945385504024476,
                                         0.00158339738845825,
                                         0.000169449486778163,
                                         1.10793243263822e-05,
                                         4.14422448078558e-07,
                                         8.06327926103684e-09,
                                         7.09948210914885e-11,
                                         2.29154859430349e-13,
                                         1.93866794578379e-16,
                                         2.41099959178516e-20,
                                         1.43205687821574e-25,
                                         2.805115118964e-33,
                                         1.17541565890771e-47,
                                         5.39226332056086e-11,
                                         9.1188117283848e-07,
                                         0.000131317781362233,
                                         0.00289649519846356,
                                         0.0217132042912984,
                                         0.0769947131103525,
                                         0.152279687774873,
                                         0.183709746225587,
                                         0.141980760031844,
                                         0.072000455686257,
                                         0.0241018163006318,
                                         0.00527799129486084,
                                         0.000738510909377504,
                                         6.34436215111462e-05,
                                         3.15003521237912e-06,
                                         8.27153831668353e-08,
                                         1.00759637752323e-09,
                                         4.6662580423983e-12,
                                         5.97791096474569e-15,
                                         1.22378248816063e-18,
                                         1.37658233798115e-23,
                                         6.7172938890056e-31,
                                         1.49419058008098e-44,
                                         3.83787544654807e-13,
                                         3.44531468085295e-08,
                                         1.23599273291377e-05,
                                         0.000516297499427726,
                                         0.00637106758334674,
                                         0.0342101935383792,
                                         0.0970770019677787,
                                         0.162028689289321,
                                         0.169001930745344,
                                         0.113761382548717,
                                         0.0500342530736973,
                                         0.0143259763717651,
                                         0.00262089962176545,
                                         0.00029582813192898,
                                         1.94968459792093e-05,
                                         6.90935784815679e-07,
                                         1.16445669316212e-08,
                                         7.73722537193115e-11,
                                         1.50097089386129e-13,
                                         5.05810857488279e-17,
                                         1.0775090694258e-21,
                                         1.30982967636249e-28,
                                         1.54666882480431e-41,
                                         2.26431903514224e-15,
                                         1.07906224521463e-09,
                                         9.64351266406819e-07,
                                         7.6287641333483e-05,
                                         0.0015496280277213,
                                         0.0126001905919188,
                                         0.0513000384420422,
                                         0.118461873718618,
                                         0.166755740366607,
                                         0.148998341722986,
                                         0.086101759605147,
                                         0.0322334468364716,
                                         0.00771029186871489,
                                         0.00114345198768919,
                                         0.000100032303663748,
                                         4.78427399396765e-06,
                                         1.11554486827013e-07,
                                         1.0634784656095e-09,
                                         3.12407934994315e-12,
                                         1.73300181585145e-15,
                                         6.99143996422853e-20,
                                         2.11720182330495e-26,
                                         1.3271366009414e-38,
                                         1.12152317621039e-17,
                                         2.83718721447974e-11,
                                         6.31652888986238e-08,
                                         9.46307424113564e-06,
                                         0.000316421953409909,
                                         0.00389603268274103,
                                         0.02275846252314,
                                         0.0727091671828937,
                                         0.138131845261441,
                                         0.163829423237431,
                                         0.124388584810519,
                                         0.0608853995800018,
                                         0.0190421131891157,
                                         0.00371039623224556,
                                         0.000430863865797802,
                                         2.78111082111591e-05,
                                         8.97169782537005e-07,
                                         1.22714546605244e-08,
                                         5.45878180777673e-11,
                                         4.98463981275193e-14,
                                         3.80834447778512e-18,
                                         2.87298650890325e-24,
                                         9.5599983075069e-36,
                                         4.70535569146007e-20,
                                         6.31892297279186e-13,
                                         3.50457444008447e-09,
                                         9.94314740433009e-07,
                                         5.47292280562699e-05,
                                         0.0010204262900594,
                                         0.00855227621530653,
                                         0.0378018725505562,
                                         0.0969215620124147,
                                         0.152586439930365,
                                         0.152216803404901,
                                         0.0974166393280029,
                                         0.0398357535668632,
                                         0.0101984980576954,
                                         0.00157200327960719,
                                         0.000136941196106927,
                                         6.11189385823613e-06,
                                         1.19943568245042e-07,
                                         8.07947209992248e-10,
                                         1.21445692817243e-12,
                                         1.75719268969782e-16,
                                         3.30231490419468e-22,
                                         5.83329009447752e-33,
                                         1.68250070828955e-22,
                                         1.19943500648777e-14,
                                         1.65718381401078e-10,
                                         8.90418345833633e-08,
                                         8.06771650710189e-06,
                                         0.000227781935111146,
                                         0.00273904463666276,
                                         0.016750047698914,
                                         0.0579596277107909,
                                         0.121120755112943,
                                         0.158753481858847,
                                         0.132840871810913,
                                         0.0710247188130657,
                                         0.0238908023845529,
                                         0.00488816000208943,
                                         0.000574683238022003,
                                         3.54858784384635e-05,
                                         9.9916329946606e-07,
                                         1.01917515216078e-08,
                                         2.52179072370126e-11,
                                         6.91004923844954e-15,
                                         3.2350572738284e-20,
                                         3.0335274240282e-30,
                                         5.1471434081194e-25,
                                         1.94786383438516e-16,
                                         6.7043131613812e-12,
                                         6.82201290926709e-09,
                                         1.01749000390865e-06,
                                         4.3501590202145e-05,
                                         0.000750524227731555,
                                         0.00634990164416724,
                                         0.0296537065467005,
                                         0.0822563462744628,
                                         0.141655074485043,
                                         0.154981017112732,
                                         0.10834134547516,
                                         0.0478821309676686,
                                         0.0130042585062673,
                                         0.00206334171777802,
                                         0.000176272081629898,
                                         7.12105267017888e-06,
                                         1.09992452065962e-07,
                                         4.48006365469882e-10,
                                         2.32482952201487e-13,
                                         2.71140054402962e-18,
                                         1.34967900373586e-27,
                                         1.34967900373623e-27,
                                         2.71140054402962e-18,
                                         2.32482952201482e-13,
                                         4.48006365469882e-10,
                                         1.09992452065962e-07,
                                         7.12105267017884e-06,
                                         0.000176272081629899,
                                         0.00206334171777802,
                                         0.0130042585062673,
                                         0.0478821309676686,
                                         0.10834134547516,
                                         0.154981017112732,
                                         0.141655074485043,
                                         0.0822563462744628,
                                         0.0296537065467005,
                                         0.00634990164416726,
                                         0.000750524227731552,
                                         4.35015902021452e-05,
                                         1.01749000390865e-06,
                                         6.82201290926709e-09,
                                         6.70431316138133e-12,
                                         1.94786383438516e-16,
                                         5.14714340811809e-25,
                                         3.0335274240291e-30,
                                         3.2350572738284e-20,
                                         6.91004923844938e-15,
                                         2.52179072370126e-11,
                                         1.01917515216078e-08,
                                         9.99163299466055e-07,
                                         3.54858784384637e-05,
                                         0.000574683238022001,
                                         0.00488816000208943,
                                         0.0238908023845529,
                                         0.0710247188130657,
                                         0.132840871810913,
                                         0.158753481858847,
                                         0.121120755112943,
                                         0.0579596277107909,
                                         0.0167500476989141,
                                         0.00273904463666275,
                                         0.000227781935111147,
                                         8.06771650710189e-06,
                                         8.90418345833633e-08,
                                         1.65718381401081e-10,
                                         1.19943500648777e-14,
                                         1.68250070828916e-22,
                                         5.83329009447938e-33,
                                         3.30231490419468e-22,
                                         1.75719268969777e-16,
                                         1.21445692817243e-12,
                                         8.07947209992248e-10,
                                         1.19943568245041e-07,
                                         6.11189385823616e-06,
                                         0.000136941196106927,
                                         0.00157200327960719,
                                         0.0101984980576954,
                                         0.0398357535668632,
                                         0.0974166393280029,
                                         0.152216803404901,
                                         0.152586439930365,
                                         0.0969215620124147,
                                         0.0378018725505563,
                                         0.0085522762153065,
                                         0.00102042629005941,
                                         5.47292280562699e-05,
                                         9.94314740433009e-07,
                                         3.50457444008453e-09,
                                         6.31892297279186e-13,
                                         4.70535569145907e-20,
                                         9.55999830751015e-36,
                                         2.87298650890325e-24,
                                         3.80834447778501e-18,
                                         4.98463981275193e-14,
                                         5.45878180777673e-11,
                                         1.22714546605243e-08,
                                         8.97169782537009e-07,
                                         2.7811108211159e-05,
                                         0.000430863865797802,
                                         0.00371039623224556,
                                         0.0190421131891157,
                                         0.0608853995800018,
                                         0.124388584810519,
                                         0.163829423237431,
                                         0.138131845261441,
                                         0.0727091671828938,
                                         0.0227584625231399,
                                         0.00389603268274105,
                                         0.000316421953409909,
                                         9.46307424113564e-06,
                                         6.31652888986248e-08,
                                         2.83718721447974e-11,
                                         1.12152317621018e-17,
                                         1.32713660094187e-38,
                                         2.11720182330495e-26,
                                         6.99143996422833e-20,
                                         1.73300181585145e-15,
                                         3.12407934994315e-12,
                                         1.06347846560949e-09,
                                         1.11554486827013e-07,
                                         4.78427399396763e-06,
                                         0.000100032303663748,
                                         0.00114345198768919,
                                         0.00771029186871489,
                                         0.0322334468364716,
                                         0.0861017596051471,
                                         0.148998341722986,
                                         0.166755740366607,
                                         0.118461873718618,
                                         0.0513000384420421,
                                         0.0126001905919188,
                                         0.0015496280277213,
                                         7.6287641333483e-05,
                                         9.64351266406831e-07,
                                         1.07906224521463e-09,
                                         2.26431903514186e-15,
                                         1.5466688248049e-41,
                                         1.30982967636249e-28,
                                         1.07750906942577e-21,
                                         5.05810857488279e-17,
                                         1.50097089386129e-13,
                                         7.73722537193109e-11,
                                         1.16445669316213e-08,
                                         6.90935784815677e-07,
                                         1.94968459792093e-05,
                                         0.00029582813192898,
                                         0.00262089962176545,
                                         0.0143259763717651,
                                         0.0500342530736973,
                                         0.113761382548717,
                                         0.169001930745344,
                                         0.162028689289322,
                                         0.0970770019677785,
                                         0.0342101935383793,
                                         0.00637106758334674,
                                         0.000516297499427726,
                                         1.23599273291379e-05,
                                         3.44531468085295e-08,
                                         3.8378754465475e-13,
                                         1.49419058008158e-44,
                                         6.7172938890056e-31,
                                         1.37658233798111e-23,
                                         1.22378248816063e-18,
                                         5.97791096474569e-15,
                                         4.66625804239827e-12,
                                         1.00759637752323e-09,
                                         8.27153831668349e-08,
                                         3.15003521237912e-06,
                                         6.34436215111462e-05,
                                         0.000738510909377504,
                                         0.00527799129486084,
                                         0.0241018163006318,
                                         0.072000455686257,
                                         0.141980760031844,
                                         0.183709746225588,
                                         0.152279687774873,
                                         0.0769947131103527,
                                         0.0217132042912984,
                                         0.00289649519846356,
                                         0.000131317781362235,
                                         9.1188117283848e-07,
                                         5.39226332056017e-11,
                                         1.17541565890821e-47,
                                         2.805115118964e-33,
                                         1.4320568782157e-25,
                                         2.41099959178516e-20,
                                         1.93866794578379e-16,
                                         2.29154859430347e-13,
                                         7.09948210914889e-11,
                                         8.0632792610368e-09,
                                         4.14422448078558e-07,
                                         1.10793243263822e-05,
                                         0.000169449486778163,
                                         0.00158339738845825,
                                         0.00945385504024476,
                                         0.0371067095082729,
                                         0.0971279321592854,
                                         0.169609155316789,
                                         0.1945111114755,
                                         0.141105224715639,
                                         0.0602576846981473,
                                         0.0132319063914351,
                                         0.00113607753373746,
                                         1.96527993077963e-05,
                                         6.16918968030935e-09,
                                         7.33848489995011e-51,
                                         9.29686411359187e-36,
                                         1.1823547403064e-27,
                                         3.76981027877089e-22,
                                         4.98984289255422e-18,
                                         8.93138572662411e-15,
                                         3.97005200350603e-12,
                                         6.23830453030337e-10,
                                         4.32713644809611e-08,
                                         1.5355644201864e-06,
                                         3.08569541599545e-05,
                                         0.000376999378204346,
                                         0.00294304945791669,
                                         0.0151774600547817,
                                         0.0527336999489746,
                                         0.124278453471897,
                                         0.197186104863064,
                                         0.205236565878436,
                                         0.132718180045309,
                                         0.0479735085279044,
                                         0.00780048958803877,
                                         0.000336155362852349,
                                         5.60163245668774e-07,
                                         3.49870993864779e-54,
                                         2.35292930275537e-38,
                                         7.45455849795925e-30,
                                         4.5012014754577e-24,
                                         9.80746834353661e-20,
                                         2.65824632163103e-16,
                                         1.69532248861238e-13,
                                         3.68559867162438e-11,
                                         3.45020189354521e-09,
                                         1.62520970349592e-07,
                                         4.29094011775741e-06,
                                         6.85453414916992e-05,
                                         0.000699637022808763,
                                         0.00474058953670643,
                                         0.0218634626866829,
                                         0.0695390879082915,
                                         0.15264929462075,
                                         0.227957027163795,
                                         0.22322097230994,
                                         0.1328211219162,
                                         0.0408999099934645,
                                         0.00439078599217606,
                                         3.88407580204204e-05,
                                         1.19664568917652e-57,
                                         4.27206037461741e-41,
                                         3.37172515127139e-32,
                                         3.85561373814799e-26,
                                         1.38287544864782e-21,
                                         5.67580812523864e-18,
                                         5.19355288858939e-15,
                                         1.5620884529698e-12,
                                         1.9735336531964e-10,
                                         1.23397647662392e-08,
                                         4.28063265709677e-07,
                                         8.94069671630859e-06,
                                         0.000119317491369693,
                                         0.00106223771454184,
                                         0.00650288023753584,
                                         0.0279137540839586,
                                         0.0847753133281199,
                                         0.181638262206587,
                                         0.269336791106747,
                                         0.263808579376837,
                                         0.153843428988579,
                                         0.0411434403659693,
                                         0.00193204346519253,
                                         2.61486069963468e-61,
                                         4.9555426733757e-44,
                                         9.74333734009314e-35,
                                         2.11000712015779e-28,
                                         1.24576048998255e-23,
                                         7.74257684562769e-20,
                                         1.01648770386094e-16,
                                         4.22988477156511e-14,
                                         7.21223549703818e-12,
                                         5.98590320998655e-10,
                                         2.72827913884431e-08,
                                         7.45058059692383e-07,
                                         1.30005216792769e-05,
                                         0.00015206750324066,
                                         0.00123571381238494,
                                         0.0071586782186636,
                                         0.0300794118851054,
                                         0.0924670552411256,
                                         0.207625983638408,
                                         0.334761897894876,
                                         0.369709734266977,
                                         0.246311264263785,
                                         0.0614004315009801,
                                         2.742665050618e-65,
                                         2.75921979008232e-47,
                                         1.35146421391397e-37,
                                         5.54262581199963e-31,
                                         5.38675566162615e-26,
                                         5.06972708486444e-22,
                                         9.54950669892844e-19,
                                         5.49784749411426e-16,
                                         1.26513391725727e-13,
                                         1.39377842290254e-11,
                                         8.3466246107665e-10,
                                         2.98023223876953e-08,
                                         6.7992135735276e-07,
                                         1.04494239925143e-05,
                                         0.00011271229267092,
                                         0.000881228779490933,
                                         0.00512283673442679,
                                         0.0225947714789056,
                                         0.0768261308610859,
                                         0.203903351826203,
                                         0.426465650726719,
                                         0.707797753773512,
                                         0.936628117889063,

                                         0.941545833904841,
                                         0.72767133788108,
                                         0.456701517689971,
                                         0.231837758395814,
                                         0.0946338819467574,
                                         0.0308145781244347,
                                         0.007921314803502,
                                         0.00158644001082932,
                                         0.000243453120993604,
                                         2.80379534475121e-05,
                                         2.3611724532203e-06,
                                         1.40725027755838e-07,
                                         5.69490343030105e-09,
                                         1.48377395818281e-10,
                                         2.32186996928205e-12,
                                         1.98893793051092e-14,
                                         8.21245632127042e-17,
                                         1.3643557883751e-19,
                                         6.9726271401353e-23,
                                         7.17699996313968e-27,
                                         7.19820634668371e-32,
                                         1.72129923388695e-38,
                                         3.46738105276226e-48,
                                         3.42097862573799e-66,
                                         0.0567797963505424,
                                         0.232808015954112,
                                         0.363597945287441,
                                         0.348984214120711,
                                         0.233982036026205,
                                         0.115048733877851,
                                         0.0422846262571576,
                                         0.0116660985239931,
                                         0.0024034970336055,
                                         0.000365098444269673,
                                         4.00864242059629e-05,
                                         3.09453894367098e-06,
                                         1.61860833286451e-07,
                                         5.46233273836407e-09,
                                         1.11443370842299e-10,
                                         1.25913705081392e-12,
                                         6.97993060215049e-15,
                                         1.59743219053963e-17,
                                         1.16721495154812e-20,
                                         1.8142080979231e-24,
                                         2.98870256225336e-29,
                                         1.35128646679592e-35,
                                         6.77359035315697e-45,
                                         3.54550427701386e-62,
                                         0.00164356707406849,
                                         0.0357521223470904,
                                         0.138947783474102,
                                         0.252155609263654,
                                         0.277689598981154,
                                         0.206181026865086,
                                         0.108345020718047,
                                         0.0411783426063345,
                                         0.0113897206334728,
                                         0.00228199606806831,
                                         0.00032666918235719,
                                         3.26634308393629e-05,
                                         2.20820005869416e-06,
                                         9.65227743378779e-08,
                                         2.56750810041983e-09,
                                         3.82618543513849e-11,
                                         2.84754354438797e-13,
                                         8.97756303784544e-16,
                                         9.37878282537416e-19,
                                         2.20126584777403e-22,
                                         5.95637918142318e-27,
                                         5.09189824820165e-33,
                                         6.3515178388595e-42,
                                         1.76379011321465e-58,
                                         3.03952954676934e-05,
                                         0.00350776961042644,
                                         0.0339240083418522,
                                         0.116401029070777,
                                         0.21055329691998,
                                         0.236070047918017,
                                         0.177362071651385,
                                         0.0928618690761138,
                                         0.034483226001057,
                                         0.00911265936454865,
                                         0.0017007651501038,
                                         0.000220268797829744,
                                         1.9246882612545e-05,
                                         1.08969956141768e-06,
                                         3.77915531113967e-08,
                                         7.42821320045834e-10,
                                         7.42189794988831e-12,
                                         3.22344177536128e-14,
                                         4.81468027062562e-17,
                                         1.70640936298629e-20,
                                         7.58415697595537e-25,
                                         1.22585002712448e-30,
                                         3.80505949226195e-39,
                                         5.60584556678195e-55,
                                         4.0325653557702e-07,
                                         0.00024689733537837,
                                         0.00594181064715591,
                                         0.0385479346656975,
                                         0.11453034980937,
                                         0.193905079672057,
                                         0.208290113644373,
                                         0.150231874764373,
                                         0.0748960326907062,
                                         0.026105468358972,
                                         0.00635238247910896,
                                         0.00106561486924219,
                                         0.000120347891603675,
                                         8.82551107213513e-06,
                                         3.99055927868545e-07,
                                         1.03456745938808e-08,
                                         1.38776423666372e-10,
                                         8.30304334130815e-13,
                                         1.77314614834196e-15,
                                         9.48964712459593e-19,
                                         6.92768923933768e-23,
                                         2.11714729350988e-28,
                                         1.63531525757087e-36,
                                         1.27817836756689e-51,
                                         4.08547973763286e-09,
                                         1.32705323607727e-05,
                                         0.000794725329119665,
                                         0.00974837011350556,
                                         0.047573568498911,
                                         0.121625345040787,
                                         0.186794103117334,
                                         0.185598026525379,
                                         0.124221376994182,
                                         0.0571089852061622,
                                         0.0181182200969847,
                                         0.00393671670506093,
                                         0.000574649688670667,
                                         5.45832703813909e-05,
                                         3.21780256512983e-06,
                                         1.10032218476698e-07,
                                         1.98154073653592e-09,
                                         1.63320755681539e-11,
                                         4.98664227810984e-14,
                                         4.02998567328074e-17,
                                         4.83232442351411e-21,
                                         2.79223140390008e-26,
                                         5.36695713968419e-34,
                                         2.2255040948317e-48,
                                         3.28499083307159e-11,
                                         5.66095552343298e-07,
                                         8.4361590095687e-05,
                                         0.00195655634731212,
                                         0.01568340385694,
                                         0.0605464158247601,
                                         0.132949626558889,
                                         0.181975983840018,
                                         0.163517154019835,
                                         0.0991532266747982,
                                         0.041013220962841,
                                         0.0115424369052986,
                                         0.00217769633484646,
                                         0.000267922197651902,
                                         2.05927560863912e-05,
                                         9.28774705615018e-07,
                                         2.24553471544314e-08,
                                         2.549616795774e-10,
                                         1.11301585625277e-12,
                                         1.35827089880261e-15,
                                         2.67518132353437e-19,
                                         2.9226795155444e-24,
                                         1.39792612172275e-31,
                                         3.07535168538135e-45,
                                         2.15081015688628e-13,
                                         1.96638229760617e-08,
                                         7.29204390307682e-06,
                                         0.000319763968820529,
                                         0.00421009347511772,
                                         0.0245431108453515,
                                         0.0770527063686323,
                                         0.145288625017543,
                                         0.17526981005623,
                                         0.140180006575742,
                                         0.0755977796544351,
                                         0.0275573638882043,
                                         0.00671998463852021,
                                         0.00107086483226306,
                                         0.000107311526577855,
                                         6.38377695794853e-06,
                                         2.07211262948342e-07,
                                         3.24104646847913e-09,
                                         2.02288551091109e-11,
                                         3.72774421881796e-14,
                                         1.20594410750385e-17,
                                         2.49108071621828e-22,
                                         2.96494832794741e-29,
                                         3.46049313766248e-42,
                                         1.16733888536457e-15,
                                         5.66204251470761e-10,
                                         5.2249332872022e-07,
                                         4.33205187634657e-05,
                                         0.000936850101267771,
                                         0.00824703306045965,
                                         0.0370182283440337,
                                         0.0961559379535373,
                                         0.155732003002596,
                                         0.164282860990168,
                                         0.115510419161089,
                                         0.0545387010307984,
                                         0.0171896168902196,
                                         0.0035480323666803,
                                         0.000463559224644708,
                                         3.63724024350941e-05,
                                         1.58501690712881e-06,
                                         3.41525048195416e-08,
                                         3.0476719169637e-10,
                                         8.48072092120553e-13,
                                         4.50638292846722e-16,
                                         1.76003523032348e-20,
                                         5.21287120670363e-27,
                                         3.2278114265509e-39,
                                         5.31882523180755e-18,
                                         1.36868084764753e-11,
                                         3.14294020408661e-08,
                                         4.92698982656071e-06,
                                         0.000175013831025994,
                                         0.00232642874946244,
                                         0.0149302560743457,
                                         0.0534249958649127,
                                         0.116164256407536,
                                         0.161630133931676,
                                         0.148168939749575,
                                         0.0906141044770137,
                                         0.0369137365733955,
                                         0.00986880081292429,
                                         0.00168107843241194,
                                         0.000173976329582181,
                                         1.01783723487228e-05,
                                         3.02122979250024e-07,
                                         3.85468608895182e-09,
                                         1.61973263392903e-11,
                                         1.41368581408812e-14,
                                         1.0439478788781e-18,
                                         7.69415187968475e-25,
                                         2.52756428943322e-36,
                                         2.05280671074241e-20,
                                         2.80249486076452e-13,
                                         1.6014192988329e-09,
                                         4.74660675672668e-07,
                                         2.76941605598706e-05,
                                         0.000555898301904989,
                                         0.00510073139068891,
                                         0.0251435419641262,
                                         0.073397409944913,
                                         0.134699498506035,
                                         0.160992912546605,
                                         0.127526484064629,
                                         0.0671465016071239,
                                         0.0232517028768346,
                                         0.00516397695683062,
                                         0.000704890912259737,
                                         5.53651320736683e-05,
                                         2.26390671016631e-06,
                                         4.12974646932799e-08,
                                         2.62039923180057e-10,
                                         3.75656819939467e-13,
                                         5.24505252961145e-17,
                                         9.61962373882175e-23,
                                         1.67652424249656e-33,
                                         6.75241286205324e-23,
                                         4.8906440008733e-15,
                                         6.95428666062776e-11,
                                         3.89729636056689e-08,
                                         3.7349314146458e-06,
                                         0.000113208628734254,
                                         0.00148517025388544,
                                         0.0100852578715987,
                                         0.0395246055115681,
                                         0.0956727408935933,
                                         0.149085337460476,
                                         0.152962002662178,
                                         0.104096796232715,
                                         0.0466899852892305,
                                         0.013519455673107,
                                         0.00243406543981334,
                                         0.000256668715647253,
                                         1.44581220316388e-05,
                                         3.77082480982064e-07,
                                         3.6130185039633e-09,
                                         8.5076231964745e-12,
                                         2.24594677028072e-15,
                                         1.02502393645918e-20,
                                         9.47754937868245e-31,
                                         1.90028234677627e-25,
                                         7.30189347045655e-17,
                                         2.5837371820347e-12,
                                         2.73773763715719e-09,
                                         4.30948399725809e-07,
                                         1.97247691079417e-05,
                                         0.000369971479652351,
                                         0.00346095357390412,
                                         0.018209690692439,
                                         0.058137810110997,
                                         0.118116705128852,
                                         0.156969383097416,
                                         0.138070072312013,
                                         0.0802122918408284,
                                         0.0302818450205453,
                                         0.00719102509485603,
                                         0.00101802337572222,
                                         7.89975206438761e-05,
                                         2.94576119489646e-06,
                                         4.26207535465197e-08,
                                         1.64844108980868e-10,
                                         8.22805700583016e-14,
                                         9.34454467598167e-19,
                                         4.58385097243645e-28,
                                         4.58385097243782e-28,
                                         9.34454467598167e-19,
                                         8.22805700583035e-14,
                                         1.64844108980868e-10,
                                         4.26207535465197e-08,
                                         2.94576119489648e-06,
                                         7.89975206438761e-05,
                                         0.00101802337572222,
                                         0.00719102509485603,
                                         0.0302818450205453,
                                         0.0802122918408284,
                                         0.138070072312013,
                                         0.156969383097417,
                                         0.118116705128852,
                                         0.058137810110997,
                                         0.018209690692439,
                                         0.00346095357390411,
                                         0.000369971479652351,
                                         1.97247691079416e-05,
                                         4.30948399725809e-07,
                                         2.73773763715719e-09,
                                         2.58373718203464e-12,
                                         7.30189347045655e-17,
                                         1.90028234677574e-25,
                                         9.4775493786855e-31,
                                         1.02502393645918e-20,
                                         2.24594677028078e-15,
                                         8.5076231964745e-12,
                                         3.6130185039633e-09,
                                         3.77082480982067e-07,
                                         1.44581220316388e-05,
                                         0.000256668715647254,
                                         0.00243406543981334,
                                         0.013519455673107,
                                         0.0466899852892305,
                                         0.104096796232715,
                                         0.152962002662178,
                                         0.149085337460476,
                                         0.0956727408935933,
                                         0.0395246055115681,
                                         0.0100852578715987,
                                         0.00148517025388544,
                                         0.000113208628734254,
                                         3.7349314146458e-06,
                                         3.89729636056689e-08,
                                         6.95428666062762e-11,
                                         4.8906440008733e-15,
                                         6.75241286205152e-23,
                                         1.67652424249714e-33,
                                         9.61962373882175e-23,
                                         5.24505252961159e-17,
                                         3.75656819939467e-13,
                                         2.62039923180057e-10,
                                         4.12974646932801e-08,
                                         2.26390671016631e-06,
                                         5.53651320736685e-05,
                                         0.000704890912259738,
                                         0.00516397695683062,
                                         0.0232517028768346,
                                         0.0671465016071238,
                                         0.127526484064629,
                                         0.160992912546605,
                                         0.134699498506035,
                                         0.073397409944913,
                                         0.0251435419641261,
                                         0.00510073139068891,
                                         0.000555898301904987,
                                         2.76941605598706e-05,
                                         4.74660675672668e-07,
                                         1.60141929883288e-09,
                                         2.80249486076452e-13,
                                         2.05280671074194e-20,
                                         2.52756428943415e-36,
                                         7.69415187968475e-25,
                                         1.04394787887813e-18,
                                         1.41368581408812e-14,
                                         1.61973263392903e-11,
                                         3.85468608895185e-09,
                                         3.02122979250024e-07,
                                         1.01783723487228e-05,
                                         0.000173976329582181,
                                         0.00168107843241194,
                                         0.00986880081292429,
                                         0.0369137365733955,
                                         0.0906141044770139,
                                         0.148168939749575,
                                         0.161630133931675,
                                         0.116164256407536,
                                         0.0534249958649126,
                                         0.0149302560743457,
                                         0.00232642874946243,
                                         0.000175013831025994,
                                         4.92698982656071e-06,
                                         3.14294020408655e-08,
                                         1.36868084764753e-11,
                                         5.31882523180645e-18,
                                         3.22781142655217e-39,
                                         5.21287120670363e-27,
                                         1.76003523032353e-20,
                                         4.50638292846722e-16,
                                         8.48072092120553e-13,
                                         3.04767191696372e-10,
                                         3.41525048195416e-08,
                                         1.58501690712882e-06,
                                         3.63724024350941e-05,
                                         0.000463559224644708,
                                         0.0035480323666803,
                                         0.0171896168902196,
                                         0.0545387010307985,
                                         0.115510419161089,
                                         0.164282860990168,
                                         0.155732003002596,
                                         0.0961559379535371,
                                         0.0370182283440337,
                                         0.00824703306045962,
                                         0.000936850101267771,
                                         4.33205187634657e-05,
                                         5.22493328720213e-07,
                                         5.66204251470761e-10,
                                         1.16733888536435e-15,
                                         3.46049313766391e-42,
                                         2.96494832794741e-29,
                                         2.49108071621837e-22,
                                         1.20594410750385e-17,
                                         3.72774421881796e-14,
                                         2.02288551091111e-11,
                                         3.24104646847913e-09,
                                         2.07211262948343e-07,
                                         6.38377695794853e-06,
                                         0.000107311526577855,
                                         0.00107086483226306,
                                         0.0067199846385202,
                                         0.0275573638882043,
                                         0.0755977796544351,
                                         0.140180006575742,
                                         0.17526981005623,
                                         0.145288625017542,
                                         0.0770527063686323,
                                         0.0245431108453514,
                                         0.00421009347511772,
                                         0.000319763968820529,
                                         7.29204390307673e-06,
                                         1.96638229760617e-08,
                                         2.15081015688593e-13,
                                         3.07535168538269e-45,
                                         1.39792612172275e-31,
                                         2.9226795155445e-24,
                                         2.67518132353437e-19,
                                         1.35827089880261e-15,
                                         1.11301585625278e-12,
                                         2.549616795774e-10,
                                         2.24553471544315e-08,
                                         9.28774705615018e-07,
                                         2.05927560863912e-05,
                                         0.000267922197651902,
                                         0.00217769633484646,
                                         0.0115424369052986,
                                         0.041013220962841,
                                         0.0991532266747982,
                                         0.163517154019835,
                                         0.181975983840017,
                                         0.132949626558889,
                                         0.0605464158247599,
                                         0.01568340385694,
                                         0.00195655634731212,
                                         8.4361590095686e-05,
                                         5.66095552343298e-07,
                                         3.28499083307114e-11,
                                         2.22550409483273e-48,
                                         5.36695713968419e-34,
                                         2.79223140390018e-26,
                                         4.83232442351411e-21,
                                         4.02998567328075e-17,
                                         4.98664227810988e-14,
                                         1.63320755681539e-11,
                                         1.98154073653593e-09,
                                         1.10032218476698e-07,
                                         3.21780256512983e-06,
                                         5.45832703813909e-05,
                                         0.000574649688670666,
                                         0.00393671670506093,
                                         0.0181182200969847,
                                         0.0571089852061622,
                                         0.124221376994182,
                                         0.185598026525378,
                                         0.186794103117334,
                                         0.121625345040787,
                                         0.047573568498911,
                                         0.00974837011350557,
                                         0.000794725329119657,
                                         1.32705323607727e-05,
                                         4.08547973763239e-09,
                                         1.27817836756751e-51,
                                         1.63531525757087e-36,
                                         2.11714729350996e-28,
                                         6.92768923933768e-23,
                                         9.48964712459593e-19,
                                         1.77314614834197e-15,
                                         8.30304334130815e-13,
                                         1.38776423666372e-10,
                                         1.03456745938808e-08,
                                         3.99055927868545e-07,
                                         8.82551107213513e-06,
                                         0.000120347891603674,
                                         0.00106561486924219,
                                         0.00635238247910896,
                                         0.026105468358972,
                                         0.0748960326907062,
                                         0.150231874764373,
                                         0.208290113644373,
                                         0.193905079672056,
                                         0.11453034980937,
                                         0.0385479346656975,
                                         0.00594181064715586,
                                         0.00024689733537837,
                                         4.03256535576983e-07,
                                         5.6058455667848e-55,
                                         3.80505949226195e-39,
                                         1.22585002712453e-30,
                                         7.58415697595537e-25,
                                         1.70640936298629e-20,
                                         4.81468027062567e-17,
                                         3.22344177536128e-14,
                                         7.42189794988835e-12,
                                         7.42821320045834e-10,
                                         3.77915531113967e-08,
                                         1.08969956141768e-06,
                                         1.92468826125449e-05,
                                         0.000220268797829744,
                                         0.0017007651501038,
                                         0.00911265936454865,
                                         0.034483226001057,
                                         0.0928618690761137,
                                         0.177362071651385,
                                         0.236070047918017,
                                         0.21055329691998,
                                         0.116401029070777,
                                         0.0339240083418521,
                                         0.00350776961042644,
                                         3.03952954676913e-05,
                                         1.76379011321559e-58,
                                         6.3515178388595e-42,
                                         5.09189824820187e-33,
                                         5.95637918142318e-27,
                                         2.20126584777403e-22,
                                         9.37878282537425e-19,
                                         8.97756303784544e-16,
                                         2.84754354438798e-13,
                                         3.82618543513849e-11,
                                         2.56750810041983e-09,
                                         9.65227743378779e-08,
                                         2.20820005869416e-06,
                                         3.26634308393629e-05,
                                         0.00032666918235719,
                                         0.00228199606806831,
                                         0.0113897206334728,
                                         0.0411783426063345,
                                         0.108345020718047,
                                         0.206181026865086,
                                         0.277689598981154,
                                         0.252155609263654,
                                         0.138947783474101,
                                         0.0357521223470904,
                                         0.00164356707406841,
                                         3.54550427701582e-62,
                                         6.77359035315697e-45,
                                         1.35128646679598e-35,
                                         2.98870256225336e-29,
                                         1.8142080979231e-24,
                                         1.16721495154813e-20,
                                         1.59743219053963e-17,
                                         6.97993060215053e-15,
                                         1.25913705081392e-12,
                                         1.11443370842299e-10,
                                         5.46233273836407e-09,
                                         1.6186083328645e-07,
                                         3.09453894367098e-06,
                                         4.00864242059629e-05,
                                         0.000365098444269673,
                                         0.0024034970336055,
                                         0.0116660985239931,
                                         0.0422846262571576,
                                         0.115048733877851,
                                         0.233982036026205,
                                         0.348984214120711,
                                         0.363597945287441,
                                         0.232808015954112,
                                         0.0567797963505411,
                                         3.42097862573996e-66,
                                         3.46738105276226e-48,
                                         1.72129923388703e-38,
                                         7.19820634668371e-32,
                                         7.17699996313968e-27,
                                         6.97262714013538e-23,
                                         1.3643557883751e-19,
                                         8.21245632127047e-17,
                                         1.98893793051092e-14,
                                         2.32186996928205e-12,
                                         1.48377395818281e-10,
                                         5.69490343030104e-09,
                                         1.40725027755838e-07,
                                         2.3611724532203e-06,
                                         2.80379534475121e-05,
                                         0.000243453120993604,
                                         0.00158644001082932,
                                         0.007921314803502,
                                         0.0308145781244347,
                                         0.0946338819467574,
                                         0.231837758395814,
                                         0.456701517689971,
                                         0.72767133788108,
                                         0.941545833904841

};

void VBernstein(const Standard_Integer classe, const Standard_Integer nbpoints, math_Matrix& M)
{
  if (classe > 26)
    throw Standard_DimensionError("VBernstein: classe > 26");
  if (nbpoints > 24)
    throw Standard_DimensionError("VBernstein: nbpoints > 24");
  //  math_Matrix VB(classe, nbpoints);

  Standard_Integer i, j, Som;
  // 300 = 1+2+  ... +24 points.
  Som = (Standard_Integer)(300 * ((classe - 1) * classe / 2. - 1)
                           + classe * (nbpoints - 1) * nbpoints / 2.);

  const Standard_Real* tmpVB;
  tmpVB = VBMatrix + Som;
  for (i = 1; i <= classe; i++)
  {
    for (j = 1; j <= nbpoints; j++)
    {
      M(i, j) = *tmpVB;
      tmpVB++;
    }
  }
}
